# Cloudera Data Visualization

This is the Cloudera Data Visualization repository.

The bits here make it into CML, CDSW, and CDW as Cloudera Data Visualization

## Local Dev Environment

### Without docker

- Start with a Python 3.9.18 [venv](https://realpython.com/intro-to-pyenv/)
  - Install pyenv: `$ curl https://pyenv.run | bash`
  - As instructed in the console, add pyenv to the `~/.bashrc` and reload bashrc afterwards (`$ . ~/.bashrc`)
  - You might want to install the python version generally being used (3.9.18): `$ pyenv install -v 3.9.18`
    - Check it’s installed correctly: `$ pyenv versions`
  - Now create a pyenv for running the application
    - Create pyenv: `$ pyenv virtualenv 3.9.18 <environment_name>`
    - Activate it for the folder: `$ pyenv local <environment_name>`
    - Optionally, you can manually activate/deactivate with:
      - `$ pyenv activate <environment_name>`
      - `$ pyenv deactivate`
- Install [Node.js version v16](https://nodejs.org/en/download/) or higher
  - (Note that the version used to build CDV in production comes from packaging/build/Dockerfile_build
       - RUN curl -sL https://deb.nodesource.com/setup_16.x | bash - && \
    apt-get install -y nodejs)
- Within that environment:
  - `./dev/initialbuild` (this has to only be run once per environment)
  - `scons runserver`
    - Note that this uses Django's standard manage.py runserver which will detect changes in any `.py` files and restart automatically.
    - To run closer to a prod environment using cherrypy, use: `scons runcpserver`

You should now have VizApps running on `localhost:8090`, login as vizapps_admin/vizapps_admin

To run tests and some of the other environments, be sure to download [Docker](https://docs.docker.com/engine/install/)

#### Troubleshooting Development Environment

If you have any issues with running `scons runserver`, please take a look at the troubleshooting tips within the [Viz Developer Team Onboarding](https://docs.google.com/document/d/16uPOHsmmbPYdkGMlZxw2LOzZ70s6-ltLUVLszD6T2uo/edit#bookmark=id.cndmnlnjei93) document.

If you run into problems setting up the development environment and just need to run the latest release, see the [Viz Bugblitz Primer](https://docs.google.com/document/d/1oGpaXpjm2kXMlkqk6wAszQdTNowWcHZmneJ4GKM9e_A/edit#heading=h.f00rv2qs6i4w) for instructions on running the latest docker images

### With docker

- The only dependency needed is docker
- Allocate 6GB of memory to it
- use the `./dev/dockerrun` command line tool to build and run the image:

```sh
./dev/dockerrun build
./dev/dockerrun run ./dev/startup
```

Detailed documentation about the `./dev/dockerrun` tool [here](./dev/dockerrun.md)

### Within cdsw/cml

You can run CDV from source within a CDSW/CML project. This is useful for debugging environment specific issues. You will need a version 7.0.1 or older (or create a container with linux-libc-dev)

You will need at least 2vCPU/4GiB of resources for the session to set this up

- git clone the sources within, for example, /home/cdsw/code/viz, in a CDSW/CML project
- cd /home/cdsw/code/viz
- Run `./dev/installnode_cdsw`
- Run `./dev/initialbuild`
- Run `/home/cdsw/.local/bin/scons pyrequirements`
  - First run to install python requirements and fix any errors individually
  - You might need to downgrade setuptools: [pip install --upgrade setuptools==59.8.0](https://stackoverflow.com/a/71027007)
- Run `/home/cdsw/.local/bin/scons runserver`
  - Navigate to the running application from the top-right workbench menu
- If needed, stop the session and run CDV as an application in the project by using the `./dev/startup.py` script

## Checking in code

- Create a fork of the Sense/viz repo
- Create a branch with your fixes
- For anything non-trivial, run `scons tests`.
  - This will take about 30 minutes, and runs all the unit tests + linting.
- File a PR from your repo+branch to the `viz` branch on `sense/viz`
- Follow the PR workflow outlined [here](https://docs.google.com/document/d/1yHw4Ch5klGVUUKETRVhal377cYe_LyN6K-Ecj-WWEd0/edit#heading=h.cvpb6xpzrrgb)

## CML/CDSW Environment

(Note: you need docker to run this)

Build:
- `scons cmldataviz`
- `scons cdswdataviz`
- `scons runtimedataviz`

Run:
- `scons run_cmldataviz`
- `scons run_cdswdataviz`
- `scons run_runtimedataviz`

Smoke tests:
- `scons smoke_cmldataviz`
- `scons smoke_cdswdataviz`
- `scons smoke_runtimedataviz`


## CDW Environment

(Note: you need docker to run this)

Build:
- `scons cdwdataviz`

Run:
- `scons run_cdwdataviz`

Test:
- `scons smoke_cdwdataviz`

## Tests

Make sure you have docker downloaded and 6GB of memory allocated to it

Run all tests:
- `scons tests`

Individual tests:
- `scons pytests`
- `scons jstests`
- `scons smoke_cmldataviz`
- `scons smoke_cdswdataviz`
- `scons smoke_runtimedataviz`
- `scons smoke_cdwdataviz`

To test email sending locally visit [this](https://docs.google.com/document/d/152yagEVu877SeQ3Y-931VpCoJZ-_9skRs-sTQjHCc9M/edit?usp=sharing) document

## Environments for local development with attached databases

### Solr

- `scons run_cdwdataviz_solr`

## Packaging

`scons packaging` - generates artifacts in packaging/artifacts


## Publishing

The following `scons` targets push cdsw/cml engines to `docker-registry.infra.cloudera.com`:
- `scons docker_push_cmldataviz_registry docker_push_cdswdataviz_registry docker_push_cdwdataviz_registry`

The following `scons` targets push cdsw/cml engines to the public `docker.repository.cloudera.com`:
- `scons docker_push_cmldataviz_prod docker_push_cdswdataviz_prod docker_push_cdwdataviz_prod`

A shortcut to this is:
- `scons docker_prod`

Auth config files can be passed in as necessary via the `DOCKER_CONFIG_DIR` flag:
- `DOCKER_CONFIG_DIR=/path/to/docker/config/dir scons docker_push_cmldataviz_prod`

## Release Engineering

Related repositories:
- [Sense/viztest](https://github.infra.cloudera.com/Sense/viztest) is the related automated tests repository
- [Sense/RE-VIZ](https://github.infra.cloudera.com/Sense/RE-VIZ) is the related RELENG repo for this repository

`scons artifacts`
- This is the nightly run, which runs a max of once every 8 hours, and is triggered on every commit into `Sense/viz:viz`
- Runs are tracked [here](http://release.infra.cloudera.com/buildinfo?stack=VIZ&stack_version=1.0.0.0&branch=viz&irn=VIZ-1.0.0.0)

`scons tests`
- We run this against every PR filed on GitHub
  Runs can be tracked within within GitHub PRs for Sense/viz

<!--
NOT YET
`scons tests_nightly`
 - We run this every night against the `Sense/viz:viz`
-->

## Releasing new versions

For new releases, RELENG creates new branches of `viz/` & `viztest/` and runs the equivalent of:

```shell
PKG_VERSION=6.2.0-b3 WHEEL_VERSION=6.2.0.6.2.0-b3 scons docker_push_cmldataviz_registry
PKG_VERSION=6.2.0-b3 WHEEL_VERSION=6.2.0.6.2.0-b3 scons docker_push_cdswdataviz_registry
PKG_VERSION=6.2.0-b3 WHEEL_VERSION=6.2.0.6.2.0-b3 scons docker_push_cdwdataviz_registry
```

The equivalent local commands are to go to a branch and run:
- sanity test via: `scons artifacts tests docker_dev`
- release via: `DOCKER_CONFIG_DIR=/path/to/docker/config/dir scons docker_prod`

## Making changes to django models

- Make changes to the `models.py` file as needed
- Run `python manage.py makemigrations`
- Validate the new migrations file that gets created
- The next call to `scons runserver` should now run the new migrations

## Using Typescript

However it's in an early stage, technically, it's possible to create new
modules using Typescript. If you would like to do so please,
follow [this guideline](frontend/README.md).

## Using React
It's also in an early stage but it is possible to use React components.
[Guidelines](frontend/README.md).

## The new frontend code

1. Go to the `frontend` folder
2. run `npm install`

### Troubleshooting

If you get the `gyp: No Xcode or CLT version detected!` error during `npm install`, run the following commands:

```bash
$ sudo rm -rf $(xcode-select -print-path)
$ xcode-select --install
```

### Testing the new frontend code

```bash
$ npm test
```

### Building the new frontend code

```bash
$ npm run build
```

### Running the development server

We use webpack and ts-loader to perform typechecking during compilation. If
you want to re-compile typescript modules on every file change, you can start
webpack in `--watch` mode by running `npm start`

## Useful Environment Variables

`IS_EMBED='true'` Sets the necessary headers and cookie properties for
embedding the site in an iframe
