# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
urls for the admin api
"""

from django.urls import re_path
from django.conf import settings

from adminapi import views_adminapi, views_cprofile

urlpatterns = []

apis = getattr(settings, 'ADMIN_API_URL_LIST', [])
for t in ['visuals', 'datasets', 'connections', 'users', 'groups', 'roles',
          'segments', 'filterassociations', 'workspaces']:
  if '*' in apis or t in apis:
    handler = getattr(views_adminapi, 'aa_' + t)
    prefix = r'^' + t
    v1prefix = r'^' + 'v1/' + t
    urlpatterns += [
      re_path(prefix + r'$', handler, name='admin-api-' + t),
      re_path(prefix + r'/(?P<id>\d+)$', handler),
      re_path(v1prefix + r'$', handler, name='admin-api-' + t),
      re_path(v1prefix + r'/(?P<id>\d+)$', handler),]

urlpatterns += [
      re_path(r'^edituserprofile/(?P<username>.+)$', views_adminapi.edit_userprofile, name="edit_userprofile"),
      re_path(r'^loglevel$', views_adminapi.admin_loglevel),
      re_path(r'^loglevel/(?P<logger_name>.+)$', views_adminapi.admin_loglevel),
      re_path(r'^gcstats$', views_adminapi.admin_gcstats),
      re_path(r'^gcmonitor$', views_adminapi.admin_gcmonitor),
      re_path(r'^cprofile$', views_cprofile.toggle_cprofile),
]

if settings.CDV_ADDITIONAL_API_ENDPOINTS:
  urlpatterns += [
      re_path(r'^datasets/sql', views_adminapi.create_dataset_from_sql),
      re_path(r'^v1/datasets/sql', views_adminapi.create_dataset_from_sql),
      re_path(r'^visuals/smart', views_adminapi.generate_visual),
      re_path(r'^v1/visuals/smart', views_adminapi.generate_visual),
      re_path(r'^render_skeleton', views_adminapi.render_skeleton, name="render_skeleton"),
  ]

