# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
Provide a standard web interface for data access to Arcadia engine.

Format: .../api/data?dsreq="..."

Returned data content is always a JSON formatted list of objects.

"""
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.template import loader
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.models import User
from django.utils import timezone, dateparse
from django.http import HttpResponse, HttpResponseRedirect, QueryDict

import json

from apps.models import APIKey
from apps.models import lookup_apikey
from vizlib.misc import check_login_or_use_apikey
from sqlrun import views
from apps import views_dataapi

from django.conf import settings

from django.contrib.auth.models import User

import re
import json
import logging

LOGGER = logging.getLogger("arcweb")

# Enhanced Data API which exports the entire dsreq
# The hope is to make this API the new default Data API and in time retire
# the old one that is used from the end point /arcweb/apps/dataapi
@csrf_exempt
def data(request):
  if getattr(settings, 'DATA_API_ENABLED', False) == False:
    LOGGER.warning("Data Request API called but not enabled")
    return HttpResponse('Data Request API is not enabled', status=500)

  err = check_login_or_use_apikey(request)
  if (err != None):
    return err

  if request.method == 'GET':
    requestParams = request.GET
  elif request.method == 'POST':
    requestParams = request.POST

  version = requestParams.get("version", None)
  # Version 0 is the old data api available at the end point
  # /arc/apps/dataapi
  if version == '0':
    return views_dataapi.dataapi(request)

  if version != '1':
    return HttpResponse('Data Request API: unsupported version', status=500)

  # This is version 1
  dsreq = requestParams.get("dsreq", None)
  if dsreq is None:
    LOGGER.warning('Data Request API: dsreq not specified')
    return HttpResponse('Data Request not specified', status=500)

  params = {
    'dsreq': dsreq,
  }

  data = views.jsonselect_work(request, params)
  if 'data' in data:
    return HttpResponse(json.dumps(data['data']), content_type="text/json")

  if 'error' in data:
    LOGGER.warning('Data Request API: Error in query response: %s' % data['error'])
    return HttpResponse(data['error'], status=500)

  LOGGER.error('Data Request API: Unexpected response')
  return HttpResponse('Unexpected response', status=500)
