# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.core.management.base import BaseCommand, CommandError
from instant.instant import createSamples
from datasets.models import DataConnection


class Command(BaseCommand):
  args = ''
  help = 'Install Sample Apps'

  def add_arguments(self, parser):
    parser.add_argument('--force',
                action='store_true',
                dest='force',
                default=False,
                help='Delete the current sample data connection if there is any, '
                     'and create new sample apps again.')
    parser.add_argument('--skip_thumbnails',
                action='store_true',
                dest='skip_thumbnails',
                default=False,
                help='Do not import the artifacts thumbnails.')
    parser.add_argument('--keep_older',
                action='store_true',
                dest='keep_older_dc',
                default=False,
                help='Rename the current sample data connection if there is any,'
                     'and create new sample apps again.')
    parser.add_argument('--upgrade',
                action='store_true',
                dest='upgrade_ds',
                default=False,
                help='Upgrades sample datasets to newer version if it exists')

  def handle(self, *args, **options):
    if options['force'] and options['keep_older_dc']:
      print("\nOptions --force and --keep_older are mutually exclusive. Choose one.")
    elif options['upgrade_ds'] and options['keep_older_dc']:
      print("\nOptions --upgrade and --keep_older are mutually exclusive. Choose one.")
    elif DataConnection.objects.filter(dataconnection_name="samples").exists() and \
        not options['force'] and not options['keep_older_dc'] and not options['upgrade_ds']:
      print("\nWarning: samples data connection already exists aborting. Please " \
            "use --force or --keep_older if you would like to " \
            "recreate sample apps.")
    else:
      createSamples(options['skip_thumbnails'], options['force'], options['upgrade_ds'])
