# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import sys
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
  args = 'command args'
  help = 'run Arcadia utility programs'

  def run_from_argv(self, argv):

    commands = [
      'app',
      'av',
      'cleanup_currency_format',
      'cleanup_reports',
      'dataconnection',
      'dataset',
      'foreign_key_cleanup',
      'log_bundling',
      'logout_all',
      'migrateperms',
      'recommendation',
      'role',
      'user',
    ]
    cmdname = argv[1]
    usage_message = f'Usage: {cmdname} [{"|".join(commands)}]'

    if len(argv) < 3:
      print(usage_message)
      exit(1)
    cmd = argv[2]
    if cmd in ["help", "-h", "--help"]:
      print("%s - Run Arcadia Utility programs" % cmdname)
      print(usage_message)
      print("Notice: command syntax is subject to change")
      exit(0)
    if cmd == "dataset":
      from util.ds_util import main as ds_util
      sys.argv = argv[2:]
      ds_util()
      exit(0)
    if cmd =="app":
      if len(argv) < 4:
        print("Usage: app [show|export|import]")
        exit(1)
      cmd = argv[3]
      if cmd =="show":
        from util.appshow import main as appshow
        sys.argv = argv[3:]
        appshow()
        exit(0)
      if cmd =="export":
        from util.appexport import main as appexport
        sys.argv = argv[3:]
        appexport()
        exit(0)
      if cmd =="import":
        from util.appimport import main as appimport
        sys.argv = argv[3:]
        appimport()
        exit(0)
      print("Usage: app [show|export|import]")
      exit(1)
    if cmd =="dataconnection":
      from util.dctool import main as dctool
      sys.argv = argv[2:]
      dctool()
      exit(0)
    if cmd =="user":
      from util.usermanage import main as usermanage
      sys.argv = argv[2:]
      usermanage()
      exit(0)
    if cmd == "role":
      from util.role_tool import main as roletool
      sys.argv = argv[2:]
      roletool()
      exit(0)
    if cmd =="av":
      from util.avtool import main as avtool
      sys.argv = argv[2:]
      avtool()
      exit(0)
    if cmd =="logout_all":
      from util.flush_sessions import main as flush_sessions
      sys.argv = argv[2:]
      flush_sessions()
      exit(0)
    if cmd =="recommendation":
      from util.recommendation import main as recommendation
      sys.argv = argv[2:]
      recommendation()
      exit(0)
    if cmd =="cleanup_reports":
      from util.cleanup_reports import main as cleanup_reports
      sys.argv = argv[2:]
      cleanup_reports()
      exit(0)
    if cmd == "log_bundling":
      from util.log_bundling import main as log_bundling
      sys.argv = argv[2:]
      log_bundling()
      exit(0)
    if cmd == "foreign_key_cleanup":
      from util.foreign_key_cleanup import main as foreign_key_cleanup
      sys.argv = argv[2:]
      foreign_key_cleanup()
      exit(0)
    if cmd == "cleanup_currency_format":
      from util.cleanup_currency_format import main as cleanup_currency_format
      sys.argv = argv[2:]
      cleanup_currency_format(None, None)
      exit(0)

    if cmd == 'cml_sync_connections':
      from dataconn.cml_connection_importer import CmlConnectionImporter
      CmlConnectionImporter().connect()
      exit(0)

    if cmd == 'remove_columns_parenthesis':
      from util.remove_columns_parenthesis import main as remove_columns_parenthesis
      sys.argv = argv[2:]
      remove_columns_parenthesis()
      exit(0)

    if cmd == 'remove_redundant_dataset_privileges':
      from util.remove_redundant_dataset_privileges import main as remove_redundant_dataset_privileges
      sys.argv = argv[2:]
      remove_redundant_dataset_privileges()
      exit(0)

    if cmd == 'migrateperms':
      from rbac.migrateperms import main
      sys.argv = argv[2:]
      main()
      exit(0)

    print(usage_message)
    exit(1)


  def handle(self, *args, **options):
    pass
