# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.contrib.auth.models import User
from django.db import migrations

def generate_should_change_password_value(apps, schema_editor):
    UserProfile = apps.get_model('apps', 'UserProfile')

    for user_profile in UserProfile.objects.all():
        user = User.objects.filter(id=user_profile.user.id).first()

        restricted_passwords_by_username = {
            "admin": "admin",
            "vizapps_admin": "vizapps_admin",
        }

        for username, password in restricted_passwords_by_username.items():
            if user.username == username and user.check_password(password):
                user_profile.should_change_password = True
                user_profile.save(update_fields=['should_change_password'])
                break;

class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0005_add_should_change_password_field'),
    ]

    operations = [
        migrations.RunPython(generate_should_change_password_value, reverse_code=migrations.RunPython.noop)
    ]
