# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations, models
from django.db import transaction

import logging

LOGGER = logging.getLogger("arcweb")

def copy_dashboard_id_to_report_id_or_app_id_depends_on_type_and_update_last_viewed_field(apps, schema):
  ShortUrl = apps.get_model('apps', 'shorturl')
  Report = apps.get_model('reports', 'Report')
  AppGroup = apps.get_model('reports', 'AppGroup')

  with transaction.atomic():
    for short_url in ShortUrl.objects.all():
      dashboard_id = short_url.url_data['dashboardId']
      dashboard_type = short_url.url_data['type']

      if dashboard_type == 'dashboard' and Report.objects.filter(id=dashboard_id).exists():
        short_url.report_id = dashboard_id

      elif dashboard_type == 'apps' and AppGroup.objects.filter(id=dashboard_id).exists():
          short_url.app_id = dashboard_id

      short_url.last_viewed = short_url.create_time

      short_url.save()

class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0010_auto_20220825_0700'),
    ]

    operations = [
        migrations.AddField(
            model_name='shorturl',
            name='report',
            field=models.ForeignKey(to='reports.Report', on_delete=models.SET_NULL, null=True),
        ),
        migrations.AddField(
            model_name='shorturl',
            name='app',
            field=models.ForeignKey(to='reports.AppGroup', on_delete=models.SET_NULL, null=True),
        ),
        migrations.AddField(
            model_name='shorturl',
            name='last_viewed',
            field=models.DateTimeField(null=True),
        ),
        migrations.RunPython(
            copy_dashboard_id_to_report_id_or_app_id_depends_on_type_and_update_last_viewed_field,
            migrations.RunPython.noop,
        ),
    ]