# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.test import TestCase
from django.conf import settings

import apps.views
import json

from reports.models import AppGroup, Report

class Test_VisualTracker(TestCase):
  def __init__(self, *args, **kwargs):
    super(Test_VisualTracker, self).__init__(*args, **kwargs)

  def test_visuals(self):
    r = Report(
      report_type="dashboard",
      report_data=json.dumps({
        "dashboard_sheets": [
          {
            "visual_widgets": [
              {
                "id": "abc-23",
              },
              {
                "id": "abc-24",
              },
              {
                "id": "abc-25",
              }
            ],
            "control_widgets": [31, 32]
          },
          {
            "visual_widgets": [
              {
                "id": "abc-53",
              },
              {
                "id": "abc-54",
              },
            ],
            "control_widgets": [31]
          },
        ]
      }))
    r.save()
    id1 = r.id

    r = Report(
      report_type="dashboard",
      report_data=json.dumps({
        "dashboard_sheets": [
          {
            "visual_widgets": [
              {
                "id": "abc-23",
              },
            ],
            "control_widgets": [41, 42]
          },
          {
            "visual_widgets": [
              {
                "id": "abc-53",
              },
              {
                "id": "abc-54",
              },
            ],
            "control_widgets": [31]
          },
        ]
      }))
    r.save()
    id2 = r.id

    v = apps.views._VisualTracker()
    v.trackVisuals(set([23, 31, 41]))
    self.assertEquals(v.getDashboards([23]), set([tuple([id1, 23]), tuple([id2, 23])]))
    self.assertEquals(v.getDashboards([31]), set([tuple([id1, 31]), tuple([id2, 31])]))
    self.assertEquals(v.getDashboards([41]), set([tuple([id2, 41])]))
    self.assertEquals(v.getDashboards([53]), set([tuple([53, None])]))

  def test_appgroups(self):
    ag = AppGroup(
      group_name='group name',
      group_type='group type',
      group_username='group user1',
      group_options=json.dumps({
        "user_settings": {
          "css_styles": json.dumps([
            {
              "cssid": 7,
            },
            {
              "cssid": 13,
            },
          ]),
        }
      }),
      create_user='user1',
      update_user='user2')
    ag.save()
    id1 = ag.id

    ag = AppGroup(
      group_name='group name',
      group_type='group type',
      group_username='group user1',
      group_options=json.dumps({
        "user_settings": {
          "css_styles": json.dumps([
            {
              "cssid": 9,
            }
          ]),
        }
      }),
      create_user='user1',
      update_user='user2')
    ag.save()
    id2 = ag.id

    v = apps.views._VisualTracker()
    css = v.trackAppgroups()

    self.assertEquals(css, {7: [id1], 9: [id2], 13: [id1]})

