# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from typing import List
from django.contrib.auth.models import User, Group

from rbac.models import Role

from django.test.client import Client
from django.test import TestCase

import json


def create_client(user: User):
    username = user.username
    client = Client()
    client.login(username=username, password=username)
    return client


def make_user(username, is_superuser=False) -> User:
    u = User()
    u.username = username
    u.set_password(username)
    u.is_superuser = is_superuser
    u.save()
    return u


def make_group(groupname):
    group = Group()
    group.name = groupname
    group.save()
    return group


def make_role(rolename, permissions: List[str]):
    role = Role()
    role.name = rolename
    role.privileges = json.dumps([{"ptype": "system", "perms": permissions}])
    role.save()
    return role


def new_user_payload(username, is_admin=False):
    return {
        "username": username,
        "is_superuser": "true" if is_admin else "false",
        "is_active": "true",
        "password": username,
        "new_password": username,
    }


def update_user_payload(user: User):
    return {
        "username": user.username,
        "first_name": user.first_name,
        "last_name": user.last_name,
        "is_superuser": "true" if user.is_superuser else "false",
        "is_active": "true" if user.is_active else "false",
        "groups": [],
        "roles": [],
    }


def new_group_payload(name):
    return {"name": name, "users": [], "roles": []}


def role_payload(name, id=0):
    data = {
        "name": name,
        "desc": "",
        "privs": [{"ptype": "system", "perms": []}],
        "users": [],
        "groups": [],
        "sentry_groups": [],
    }
    if id != 0:
        data["id"] = id
    return {"data": json.dumps([data])}


class TestBase(TestCase):
    def setUp(self):

        user_admin = make_user("user_admin", is_superuser=True)
        user_manager = make_user("user_manager")
        user_normal = make_user("user_normal")

        role_manager = make_role("role_manager", ["sys_editperm"])
        role_manager.add_users([user_manager.username])
        role_manager.save()

        client_admin = create_client(user_admin)
        client_manager = create_client(user_manager)
        client_normal = create_client(user_normal)

        self.clients = {
            "admin": client_admin,
            "manager": client_manager,
            "normal": client_normal,
        }

        self.users = {
            "admin": user_admin,
            "manager": user_manager,
            "normal": user_normal,
        }
