# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
permission_matrix = """
# Resource | Action | UserType | Details      | Normal user | Manager user | AdminUser

 Users     | Create | Normal   |              | N           | Y            | Y
 Users     | Create | Admin    |              | N           | N            | Y
 Users     | Delete | Normal   |              | N           | N            | Y
 Users     | Delete | Admin    |              | N           | N            | Y
 Users     | Update | Normal   | username     | N           | N            | Y
 Users     | Update | Normal   | first_name   | N           | Y            | Y
 Users     | Update | Normal   | last_name    | N           | Y            | Y
 Users     | Update | Normal   | is_superuser | N           | N            | Y
 Users     | Update | Normal   | groups       | N           | Y            | Y
 Users     | Update | Normal   | roles        | N           | Y            | Y
 Users     | Update | Normal   | is_active    | N           | Y            | Y
 Users     | Update | Normal   | password     | N           | N            | Y
 Users     | Update | Admin    | username     | N           | N            | Y
 Users     | Update | Admin    | first_name   | N           | Y            | Y
 Users     | Update | Admin    | last_name    | N           | Y            | Y
 Users     | Update | Admin    | is_superuser | N           | N            | Y
 Users     | Update | Admin    | groups       | N           | Y            | Y
 Users     | Update | Admin    | roles        | N           | Y            | Y
 Users     | Update | Admin    | is_active    | N           | N            | Y
 Users     | Update | Admin    | password     | N           | N            | Y

 Groups    | Create |          |              | N           | Y            | Y
 Groups    | Update |          |              | N           | Y            | Y
 Groups    | Delete |          |              | N           | Y            | Y

 Roles     | Create |          |              | N           | Y            | Y
 Roles     | Update |          |              | N           | Y            | Y
 Roles     | Delete |          |              | N           | Y            | Y
"""

acting_user_type = ["normal", "manager", "admin"]


def create_test_parameters():
    lines = [line.strip() for line in permission_matrix.split("\n")]
    lines = [line for line in lines if len(line) > 0]
    lines = [line for line in lines if not line[0] == "#"]
    lines = [[p.lower().strip() for p in line.split("|")] for line in lines]

    params = []
    for param_line in lines:
        resource = param_line[0]
        action = param_line[1]
        target_user_type = param_line[2]
        detail = param_line[3]
        expected_results = param_line[4:]
        expected_results = [200 if r == "y" else 500 for r in expected_results]

        test_method_name = f"_test_{resource}_{action}"

        if detail:
            test_method_name = f"{test_method_name}_{detail}"

        for result_tuple in zip(acting_user_type, expected_results):
            action_params = [*result_tuple]
            if target_user_type:
                action_params.insert(0, target_user_type == "admin")
            params.append([test_method_name, action_params])

    return params


if __name__ == "__main__":
    print(create_test_parameters())
