# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
urls for the app dashboard
"""

from django.urls import include, re_path
from django.conf import settings

from apps import views, views_querylogs, views_snapshot, \
                 views_dataapi, views_remotecalls, views_cdw, \
                 views_visuals_page, views_home_page

urlpatterns = [
  re_path(r'^login$', views.LogIn.as_view(), name='login'),
  re_path(r'^logout$', views.dologout, name='dologout'),

  re_path(r'^$', views.home),
  re_path(r'^home$', views.home, name='home'),
  re_path(r'^apps$', views_visuals_page.page, name='apps'),
  re_path(r'^get_report_count_per_group$', views_visuals_page.get_report_count_per_group, name='get_report_count_per_group'),
  re_path(r'^data$', views.data, name='apps-data'),
  re_path(r'^datasql$', views.datasql, name='apps-datasql'),
  re_path(r'^cdw_warehouse$', views_cdw.cdw_warehouse, name='cdw-warehouse'),

  re_path(r'^restart$', views.admin_restart),
  re_path(r'^stop_work$', views.admin_stop_work, name='apps-stop-work'),

  re_path(r'^users$', views.admin_users, name="apps-users"),
  re_path(r'^roles$', views.admin_roles, name="apps-roles"),
  re_path(r'^aliases', views.admin_aliases, name="apps-aliases"),
  re_path(r'^role_detail$', views.admin_role_detail, name="role-details-page"),
  re_path(r'^role_detail/(?P<role_id>\d+)$', views.admin_role_detail),
  re_path(r'^app_accel$', views.admin_accel, name='apps-admin-accel'),
  re_path(r'^query_accel$', views.admin_accel_query, name='apps-query-accel'),
  re_path(r'^query_accel/(?P<dc_id>\d+)$', views.admin_accel_query),
  re_path(r'^activity$', views.admin_activity, name='apps-activity'),
  re_path(r'^settings$', views.admin_settings, name='apps-adminsettings'),
  re_path(r'^staticassets$', views.admin_staticassets, name='apps-staticassets'),
  re_path(r'^staticassets/(?P<assetid>\d+)$', views.admin_staticassets),
  re_path(r'^customstyles$', views.admin_customstyles, name='apps-customextedit'),
  re_path(r'^customstyles/(?P<styleid>\d+)$', views.admin_customstyles),
  re_path(r'^customcolors$', views.admin_customcolors, name='apps-customcolors'),
  re_path(r'^customcolors/(?P<colorid>\d+)$', views.admin_customcolors),
  re_path(r'^customdates$', views.admin_customdates, name='apps-customdates'),
  re_path(r'^customdates/(?P<daterange_id>\d+)$', views.admin_customdates),

  re_path(r'^arcvizsettings$', views.admin_arcvizsettings,
      name='apps-arcvizsettings'),

  re_path(r'^activeusers$', views_querylogs.activeusers,
      name='apps-activeusers'),

  re_path(r'^querylogs$', views_querylogs.querylogs,
      name='apps-querylogs'),
  re_path(r'^querylogs/(?P<querycount>\d+)$', views_querylogs.querylogs),

  re_path(r'^serverlogs$', views_querylogs.serverlogs,
      name='apps-serverlogs'),
  re_path(r'^bundledlogs$', views_querylogs.bundledlogs,
      name='apps-bundledlogs'),

  re_path(r'^users_api$', views.users, name='api-user'),
  re_path(r'^users_api/(?P<username>.+)$', views.users, name='api-user-single'),
  re_path(r'^users_basic_api$', views.users_basic, name='api-user-basic'),

  re_path(r'^groups_api$', views.groups, name='api-groups'),
  re_path(r'^groups_basic_api$', views.groups_basic, name='api-groups-basic'),
  re_path(r'^groups_api/(?P<group_id>.+)$', views.groups),

  # Apps (formerly appgroups)
  re_path(r'^appedit$', views.appedit, name="appedit"),
  re_path(r'^appedit/(?P<app_id>\d+)$', views.appedit),

  re_path(r'^appgroup$', views.appgroup, name='apps-view-appgroup'),
  re_path(r'^appgroup/(?P<appgroup_id>\d+)$', views.appgroup),
  re_path(r'^appgroup/(?P<appgroup_name>.+)$', views.appgroup),

  re_path(r'^homepage$', views.homepage, name='assign-homepage'),
  re_path(r'^homepage_statistics$', views_home_page.get_statistics, name='homepage-statistics'),
  re_path(r'^homepage_number_of_recently_created_resources$', views_home_page.get_number_of_recently_created_resources, name='homepage-number-of-recently-created-resources'),
  re_path(r'^homepage_recently_created_dashboards_by_user$', views_home_page.get_recently_created_dashboards_by_user, name='homepage-recently-created-dashboards-by-user'),
  re_path(r'^homepage_last_viewed_reports_by_user$', views_home_page.get_last_viewed_reports_by_user, name='homepage-last-viewed-reports-by-user'),
  re_path(r'^homepage_most_popular_reports$', views_home_page.get_most_popular_reports, name='homepage-most-popular-reports'),
  re_path(r'^homepage_samples$', views_home_page.get_samples, name='homepage-samples'),
  re_path(r'^homepage_favorite_dashboards$', views_home_page.get_favorite_dashboards, name='homepage-favorite-dashboards'),
  re_path(r'^homepage_datasets$', views_home_page.get_datasets, name='homepage-datasets'),

  re_path(r'^cssusage$', views.cssusage, name='apps-cssusage'),
  re_path(r'^colorpalette_usageusage$', views.colorpaletteUsage, name='apps-colorpalette-usage'),

  re_path(r'^app$', views.app, name='apps-app'),
  re_path(r'^app/(?P<report_id>\d+)$', views.app,  name='apps-app-id'),
  re_path(r'^app/new$', views.appnew),
  re_path(r'^app/(?P<dbname>.+)$', views.app),

  re_path(r'^builder$', views.builder, name='apps-builder'),
  re_path(r'^builder/(?P<report_id>\d+)$', views.builder),

  re_path(r'^dataset$', views.dataset, name='apps-dataset'),
  re_path(r'^dataset/(?P<dataset_id>\d+)$', views.dataset, name='apps-dataset-by-id'),

  re_path(r'^snapshot$', views_snapshot.snapshot, name='apps-snapshot'),
  re_path(r'^snapshot/(?P<app_id>\d+)$', views_snapshot.snapshot),
  re_path(r'^snapshot_ag/(?P<appgroup_id>\d+)$', views_snapshot.snapshot_ag),
  re_path(r'^template_snapshot/(?P<app_id>\d+)$', views_snapshot.template_snapshot),
  re_path(r'^snapshot_ready$', views_snapshot.snapshotReady,
      name='apps-snapshotready'),
  re_path(r'^snapshot_ready/(?P<snapshotId>\d+)$', views_snapshot.snapshotReady),
  re_path(r'^snapshot_fetch$', views_snapshot.snapshotFetch,
      name='apps-snapshotfetch'),
  re_path(r'^snapshot_fetch/(?P<snapshotId>\d+)$', views_snapshot.snapshotFetch),
  re_path(r'^groupsnapshot$', views_snapshot.groupsnapshot),

  re_path(r'^register$', views.register, name='app-register'),
  re_path(r'^unregister$', views.unregister, name='app-unregister'),
  re_path(r'^shutdown/(?P<shutdown_key>.+)$', views.shutdown, name='shutdown'),
  re_path(r'^bulk_users_groups', views.bulk_user_group_associate, name='bulk_users_groups'),

  re_path(r'^aviews$', views.aview_list, name='aview_list'),
  re_path(r'^aviews/(?P<dc_id>\d+)$', views.aview_list),

  re_path(r'^remotecalls$', views_remotecalls.fetchdata, name='remotecalls'),
  re_path(r'^demoapikeys$', views_dataapi.demo_apikeys, name="demo_apikeys"),
 ]

if getattr(settings, 'DATA_API_ENABLED', False):
  urlpatterns += [
    re_path(r'^dataapi$', views_dataapi.dataapi, name='api-dataapi'),
  ]

if getattr(settings, 'DATA_API_ENABLED', False) or \
  getattr(settings, 'ENABLE_API_KEYS', False):
  urlpatterns += [
    re_path(r'^apikeys$', views_dataapi.admin_apikeys, name="apps-apikeys"),
    re_path(r'^apikey_api', views_dataapi.apikeys, name='api-apikey'),
  ]

if len(getattr(settings, 'ADMIN_API_DEMO_LIST', [])) > 0:
  urlpatterns += [
    re_path(r'^apidemo$', views.admin_apidemo),
    ]

urlpatterns += [
  re_path(r'^apidemo-v2/', include('swagger_docs.urls')),
  ]
