# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
views for cdw
"""

import json, urllib.request, urllib.error, logging, ssl

from django.http import HttpResponse
from django.contrib.auth.decorators import login_required

from django.conf import settings

LOGGER = logging.getLogger('arcweb')

@login_required
def cdw_warehouse(request):
  PATHS = {'hive': 'computes', 'impala': 'impalas'}
  warehouse_type = request.GET.get('type')
  path = PATHS[warehouse_type]
  try:
    # Disable certificate checking to avoid issues with self-signed certificates
    ssl_context = ssl.create_default_context()
    ssl_context.check_hostname = False
    ssl_context.verify_mode = ssl.CERT_NONE
    service_discovery_endpoint = f'https://{settings.SERVICE_DISCOVERY_ENDPOINT}/{path}'
    service_response = json.loads(urllib.request.urlopen(service_discovery_endpoint, context=ssl_context).read())
  except urllib.error.URLError:
    # Discovering endpoint isn't available though https, switching to http
    service_discovery_endpoint = f'http://{settings.SERVICE_DISCOVERY_ENDPOINT}/{path}'
    service_response = json.loads(urllib.request.urlopen(service_discovery_endpoint).read())

    # hiveWarehouses = [
    # {
    #   "namespace": "anurag-hive",
    #   "hostName": "hiveserver2-service.compute-1731309045-cqjg.svc.cluster.local",
    #   "port": 80,
    #   "connectionMode": "http",
    #   "httpPath": "cliservice",
    #   "socketType": "normal",
    #   "authenticationMode": "ldap",
    #   "username": "hive"
    # },
    # {
    #   "namespace": "tab",
    #   "hostName": "hiveserver2-service.compute-1731350134-qmcc.svc.cluster.local",
    #   "port": 80,
    #   "connectionMode": "http",
    #   "httpPath": "cliservice",
    #   "socketType": "normal",
    #   "authenticationMode": "ldap",
    #   "username": "hive"
    # },
    # {
    #   "namespace": "sree-test",
    #   "hostName": "hiveserver2-service.compute-1731358741-vrnf.svc.cluster.local",
    #   "port": 80,
    #   "connectionMode": "http",
    #   "httpPath": "cliservice",
    #   "socketType": "normal",
    #   "authenticationMode": "ldap",
    #   "username": "hive"
    # }]

    # impalaWarehouses = [
    # {
    #   "namespace": "dbt-impala1",
    #   "hostName": "coordinator.impala-1731454728-tgfg.svc.cluster.local",
    #   "port": 28000,
    #   "connectionMode": "http",
    #   "httpPath": "cliservice",
    #   "socketType": "normal",
    #   "authenticationMode": "ldap",
    #   "username": "srv_env-2w6lxd"
    # },
    # {
    #   "namespace": "bnz-imp-xsmall",
    #   "hostName": "coordinator.impala-1731664397-zksm.svc.cluster.local",
    #   "port": 28000,
    #   "connectionMode": "http",
    #   "httpPath": "cliservice",
    #   "socketType": "normal",
    #   "authenticationMode": "ldap",
    #   "username": "srv_env-2w6lxd"
    # },
    # {
    #   "namespace": "niru-dbt-impala-test",
    #   "hostName": "coordinator.impala-1732006360-ghcz.svc.cluster.local",
    #   "port": 28000,
    #   "connectionMode": "http",
    #   "httpPath": "cliservice",
    #   "socketType": "normal",
    #   "authenticationMode": "ldap",
    #   "username": "srv_env-2w6lxd"
    # }]

    # if warehouse_type == 'impala':
    #   service_response = impalaWarehouses
    # elif warehouse_type == 'hive':
    #   service_response = hiveWarehouses


  # Loop through Virtual Warehouses and attach the proper proxy username for impersonation
  for service in service_response:
    if warehouse_type == 'impala':
      # CDW with SAML authentication should allow us to retrieve a proxy Impala username using CDW_IMPALA_USERNAME,
      #  but in cases with other authentication schemes are used (i.e. ldap) we should default to a blank username
      #  and let users configure the connection with their workload credentials.
      service['username'] = getattr(settings, 'CDW_IMPALA_USERNAME', '')
    else:
      service['username'] = 'hive'
  return HttpResponse(json.dumps(service_response), content_type="application/json")