# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import os
import json

from datasets.models import DataSet, DataConnection


# Store the static dataset for its separate loading
static_datasets = {}


def get_static_dataset_model(id):
  dataset = static_datasets.get(id)
  if dataset:
    return DataSet(
      id=id,
      dataset_info=dataset['info'],
      dataconnection_id=dataset['dc_id'],
      dataconnection=None,
      dataset_name=dataset['name'],
      dataset_type=dataset['type'],
      dataset_detail=dataset['detail'],
    )
  return None


def convert_to_static_dataset(data, name):
  connection = DataConnection.objects.get(dataconnection_name=name)
  if not connection:
    raise Exception(f"Data connection not found: {name}")

  dataset = data['fields']
  id = -data['pk']
  dataset.update({
    'id' : id,
    'info' : dataset.pop('dataset_info'),
    'dc_id' : connection.id,
    'dc_name': connection.dataconnection_name,
    'dc_type': connection.dataconnection_type,
    'url': f'/arc/datasets/dataset/{id}?detail=true',
    'name': dataset.pop('dataset_name'),
    'type': dataset.pop('dataset_type'),
    'detail': dataset.pop('dataset_detail'),
    'settings': dataset.pop('dataset_settings'),
    'events': [],
    'segments': [],
  })
  add_static_perms(dataset)
  return dataset


def convert_to_static_visual(data):
    visual = data['fields']
    id = data['pk']
    visual.update({
      'id' : id,
      'data' : visual.pop('report_data'),
      'dataset_id' : -visual.pop('dataset'),
      'type' : visual['report_type'],
      'url': f'/arc/reports/report/{id}',
    })
    add_static_perms(visual)
    return visual


def convert_to_static_dashboard(data, name):
  dashboard = data['dashboards'][0]
  dashboard = convert_to_static_visual(dashboard)
  dashboard['url'] = f'/arc/apps/app/{dashboard["id"]}'
  dashboard['visuals'] = []

  assets = {}
  for visual in data['visuals']:
    visual = convert_to_static_visual(visual)
    assets[visual['url']] = [visual]
    dashboard['visuals'].append(visual['id'])

  for dataset in data['datasets']:
    dataset = convert_to_static_dataset(dataset, name)
    assets[dataset['url']] = [dataset]
    static_datasets[dataset['id']] = dataset

  dashboard['assets'] = assets
  return [dashboard]


def add_static_perms(data):
  data.update({
    'workspace_perm': [],
    'dataset_perm': [],
    'perm': [],
    'workspace_id': 2,
    'workspace_name': 'Private',
  })


def get_static_dashboard_path(filename):
  return os.path.normpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), '../samples/dashboards/static', f'{filename}.json'))


def load_static_dashboard(filename):
  with open(get_static_dashboard_path(filename), 'r') as f:
    data = json.loads(f.read())

  assets = convert_to_static_dashboard(data, convert_filename_to_static_dashboard_name(filename))
  return assets


def convert_filename_to_static_dashboard_name(text):
    return ' '.join([word.capitalize() for word in text.split('_')])

