# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import asyncio
import concurrent.futures

# parts of Django 4 (e.g. the ORM) can not be used from async code
# this executor is used for such code parts
# (with Django 5 this might change)
DJANGO_EXECUTOR = concurrent.futures.ThreadPoolExecutor(
  max_workers=1,
  thread_name_prefix="ProxyDjangoExecutor")


def django_submit(function: callable, *args):
  """
  Utility to run a function in the dedicated thread from a synchronous context.
  (when the coroutine provided below can not be used.)
  The returned future can be avaited by the caller, if needed.
  """
  return DJANGO_EXECUTOR.submit(function, *args)

async def django_run_async(function: callable, *args):
  """
  Utility to run the provided function in the Django executor from async context.
  Useful for invoking functions that need metadata access (Django models)
  """
  return await asyncio.get_event_loop().run_in_executor(
    DJANGO_EXECUTOR,
    function,
    *args
  )
