# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.contrib.auth.models import User
from apps.models import UserProfile


AUTH_TYPE_TO_DB_VALUE = dict(
  (auth_type, db_value)
  for db_value, auth_type in UserProfile.AUTHENTICATION_SOURCES
)


def extend_user_profile_with_authentication_source(user, auth_type):
  if user:
    user = User.objects.get(username=user.username)
    is_default_user = user.userprofile.is_password_editable and \
            user.userprofile.authentication_source == \
            AUTH_TYPE_TO_DB_VALUE['Default']
    if is_default_user:
      user.userprofile.authentication_source = AUTH_TYPE_TO_DB_VALUE[auth_type]
      user.userprofile.is_password_editable = False
      user.userprofile.save(update_fields=['authentication_source', 'is_password_editable'])

  return user