# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.


# Default routine to update user's group membership upon authentication from
# Also now supports setting up user profile entries from saml attributes
# this SAML backend--called with same parameters as plugin would be
def saml_post_authenticate(user, attributes, attribute_mapping, settings, logger, update):

  updated = False

  # Setup group membership
  groupattr = getattr(settings, 'SAML_GROUP_ATTRIBUTE_NAME', 'groups')
  if groupattr in attributes or getattr(settings, 'SAML_GROUP_ATTRIBUTE_MISSING_IS_EMTPY', True):
    raw_groups = attributes.get(groupattr, [])
    groups = []
    # Check if group should be ignored based on a set of conditions
    #   NOTE: This is mostly applicable for CDW installs
    for group in raw_groups:
      for prefix in getattr(settings, 'CDW_SAML_IGNORE_GROUP_PREFIXES', []):
        if group.startswith(prefix):
          logger.debug("Group %s matches an ignore prefix condition: %s. Skipping group." % (group, prefix))
        else:
          groups.append(group)

    logger.debug("Set SAML user %s groups to: %s" % (user.username, groups))
    updated = update(user, groups, logger, create_if_needed=getattr(settings, 'SAML_CREATE_UNKNOWN_GROUP', False))

  # Setup profile attributes
  profmap = getattr(settings, 'SAML_PROFILE_ATTRIBUTE_MAP', {})
  for (saml_attr, prof_attr) in list(profmap.items()):
    if saml_attr in attributes:
      prof_val = attributes[saml_attr][0]
      if user.userprofile.getkey(prof_attr) != prof_val:
        logger.debug('SAML user %s profile entry %s set to %s' % (user.username, prof_attr, prof_val))
        user.userprofile.setkey(prof_attr, prof_val)
      else:
        logger.debug('SAML user %s profile entry %s already set to %s' % (user.username, prof_attr, prof_val))
    else:
      logger.debug('SAML user %s SAML attribute %s not found' % (user.username, saml_attr))

  return updated