# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.contrib.auth.models import User, Group


# Helper function to set the groups for a user to given set of group names
# Returns True if any updates made
def update_user_groups(user, names, logger, create_if_needed=False):

  cur_groups = set(user.groups.values_list('name', flat=True))
  new_groups = set(names)
  logger.debug("SAML user's current groups: %s." % cur_groups)
  logger.debug("SAML user's incoming groups from assertion: %s" % new_groups)

  # Create missing groups if needed, remove non-existent groups from new_groups
  if create_if_needed:
    logger.debug("SAML automatic group creation enabled. Checking if new groups should be created.")
    for name in new_groups.difference(cur_groups):
      logger.debug("SAML user's group does not exist. Creating new Group object: %s." % str(name))
      Group.objects.get_or_create(name=name)
  else:
    new_groups = set(Group.objects.filter(name__in=new_groups).values_list('name', flat=True))

  if cur_groups == new_groups:
    logger.debug("SAML user's current groups match incoming group list. No changes made.")
    return False

  logger.debug("SAML user's current groups do not match incoming group list. Removing user from existing groups "
               "not in incoming group list (if Group exists), and adding groups not in the existing groups list "
               "(if Group exists).")
  # Remove existing groups not in new_groups
  user.groups.remove(*list(Group.objects.filter(name__in=cur_groups.difference(new_groups))))
  # Add groups not in existing groups
  user.groups.add(*list(Group.objects.filter(name__in=new_groups.difference(cur_groups))))

  return True
