# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import os

from django.contrib.auth.models import Group
from django_auth_ldap.backend import LDAPBackend
from django.contrib.auth.backends import RemoteUserBackend
from util.ff_flipper import FeatureFlipper

from arcwebbase.auth.common.extend_user_profile_with_authentication_source import extend_user_profile_with_authentication_source
from arcwebbase.auth.saml.backend import VizSaml2Backend
from dataconn.cml_dependencies import VIZCML_RW, VIZCML_RO

class VizBaseLDAPBackend(LDAPBackend):
  def authenticate(self, request, username=None, password=None):
    user = super().authenticate(request, username, password)
    user = extend_user_profile_with_authentication_source(user, 'LDAP')
    return user


class VizBaseSaml2Backend(VizSaml2Backend):
  def __init__(self):
    # Abstraction super class for pointing settings to our custom SAML authentication backend.
    super().__init__()


class VizBaseRemoteBackend(RemoteUserBackend):
  def authenticate(self, request, remote_user):
    user = super().authenticate(request, remote_user)
    user = extend_user_profile_with_authentication_source(user, 'Remote')
    # Perform group sync if enabled with RemoteUser Auth
    if FeatureFlipper().get_flipper_value('enable_cml_sync') and user:
      add_user_to_group(request, user)

    return user

def add_user_to_group(request, user):
  if not user:
    return

  permission = request.headers.get('Remote-User-Perm')

  group = None
  if permission == 'RW':
    group = Group.objects.get(name=VIZCML_RW)
  else: # permission == 'RO' or in dev mode without headers
    group = Group.objects.get(name=VIZCML_RO)

  if group:
    group.user_set.add(user.id)
    group.save()
