# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
Django settings for arcweb project.
"""

import datetime
import glob
import os
import posixpath
import sys
import tempfile
from apps.config import product_name

START_TIME = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

THISDIR = os.path.dirname(os.path.realpath(__file__))

BASEDIR = os.environ.get("BASEDIR", os.path.expanduser("~/.arc"))

sys.path.append(THISDIR)
sys.path.append('%s/../../acli' % THISDIR)

for egg in glob.glob(os.path.join("%s/.." % THISDIR, "ext-py","*.egg")):
  sys.path.append(egg)

# Recheck tables used to cache query results via explain every so often
CACHE_RECHECK_SECONDS = (5 * 60)

# PAM Authentication settings
PAM_AUTH_SERVICE='login'
PAM_AUTH_CREATE_USER=True     # Create a Django user if one does not exist

# When the groups for a user can come from a remote source like LDAP or PAM,
# then we can cache the results to avoid too many slow operations.
REMOTE_GROUP_CACHE_TIMEOUT = 60

# Set to True to force use of pure-sasl python library (no impact
# on Windows platforms as we always use this mode.)
USE_PURE_PYTHON_SASL = False

# Default site settings
SITE_SETTING_DEFAULTS = {
  'download_maxrows' : '1000000',
  'visual_maxrows' : '100000',
  'maximum_canvas_width': '8000',
  'bulk_max_entries': '10000',
  'bulk_max_interval': '7',
  'bulk_max_retention': '365',
  'unnamed_versions_deletion_interval': '7',
  'unnamed_versions_max_retention': '30',
  'notification_max_retention': '1'
}

# Default IE optimizations to False
ENABLE_IE_OPTIMISATIONS = False

# Set to True enable django cache for Hive connections.  Note there is no
# data coherence checking so updates to hive tables should clear
# the django cache (via 'rm -rf <cache directory>').  See CACHES/LOCATION
# setup below for location of the cache directory.
IMPALA_CACHE_ENABLE = False
SPOT_CACHE_ENABLE = False
HIVE_CACHE_ENABLE = False
ASTER_CACHE_ENABLE = True
HIDE_PROFILE_QUERY = False
ACCESS_LOG = True
USE_DATA_API = True
DERIVED_DATA = False
NO_CACHE = False
JOIN_POLICY = 'MinimumSet'
#TEMPLATE_DEBUG = DEBUG
SNAPSHOT_DEBUG = False
RUN_FROM_SOURCE_DEBUG = False
EVERYONE_GROUP = 'Everyone'
ARCENGINE_LOG_UI_ENABLE = True
ARCVIZ_LOG_UI_ENABLE = True
ETAGS_FOR_AJAX_ENABLE = True
ETAGS_AJAX_WITH_ARGS_ENABLE = True
REFRESH_RECREATE_INVALID_AVIEWS = True
# To enable admin api demo tabs and/or urls:
# ADMIN_API_DEMO_LIST = ['visuals', 'datasets', 'connections', 'users', 'groups', 'roles', 'segments', 'filterassociations']
# ADMIN_API_URL_LIST  = ['visuals', 'datasets', 'connections', 'users', 'groups', 'roles', 'segments', 'filterassociations']

ADMINS = (
  # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

# Do not send back any settings that may have secrets associated with them
HIDE_SETTINGS_FROM_DISPLAY = ['DATABASES', 'SECRET_KEY', 'SENTRY']

DATABASES = {
  'default': {
    # Add 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
    'ENGINE': 'django.db.backends.sqlite3',
    # Or path to database file if using sqlite3.
    'NAME': '/tmp/arcweb.django.sqlite3',
    # The following settings are not used with sqlite3:
    'USER': '',
    'PASSWORD': '',
    # Empty for localhost through domain sockets or '127.0.0.1' for
    # localhost through TCP.
    'HOST': '',
    # Set to empty string for default.
    'PORT': '',
  },
  'usage': {
    # Add 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
    'ENGINE': 'django.db.backends.sqlite3',
    # Or path to database file if using sqlite3.
    'NAME': '/tmp/cdvusage.django.sqlite3',
    # The following settings are not used with sqlite3:
    'USER': '',
    'PASSWORD': '',
    # Empty for localhost through domain sockets or '127.0.0.1' for
    # localhost through TCP.
    'HOST': '',
    # Set to empty string for default.
    'PORT': '',
  }
}

DATABASE_ROUTERS = ['arcwebbase.db_router.UsageRouter']

# Kerberos settings
#KERBEROS_PRINCIPAL = '<principal name>'
#KERBEROS_KEYTAB = '<path to keytab file>'
#KERBEROS_RENEW_SECONDS = '43200'

# Sentry settings -- If this is present, will connect to sentry server
# and periodically sync roles and group membership.
#
# Required settings
#
#   HOST - Host where sentry server is running
#   AUTH - Auth type: 'kerberos', 'nosasl', 'plain', 'none'
#
# Optional settings (with - default - ):
#
#   PORT - 8038 - Port number on server
#   USERNAME - '' - Username to connect with (if required by AUTH type)
#   PASSWORD - '' - Ditto for password
#   SENTRY_CLIENTNAME - 'arcadia' - Client name used in server requests
#   POLL_FREQUENCY - 300 - Seconds between poll attempts
#   SOCKTYPE - 'NORMAL' - Server socket type: 'normal', 'ssl'
#   SSLCERT - None - SSL certificate file (if SOCKTYPE='ssl')
#   SSLCERT_NAME_MISMATCH - False - Allow local host name mismatch in cert check
#   GROUP_NAME_IGNORE_CASE - False - Ignore case on matching Sentry group names
#
# Can also pull some settings from the sentry-site.xml file.  These will override
# any locally set values:
#
#   SENTRY-SITE - None - Full path to sentry-site.xml file.
#
# The xml parameter names used and corresponding setting name:
#
#  sentry.service.security.mode             : AUTH
#  sentry.service.client.server.rpc-address : HOST
#  sentry.service.client.server.rpc-port    : PORT
#
# Like so:
#
"""
SENTRY = {
  'SENTRY-SITE' : '/Users/doug/test/sentry-site.xml'
}
"""

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ["*"]

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = os.environ.get("CDV_TIME_ZONE", "UTC")

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"

MEDIA_ROOT = os.path.expanduser(f'{BASEDIR}/media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = ''

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = os.path.join(THISDIR,"../../static_runtime")

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/arc/static/'
if os.getenv('CDV_SUB_MODE') == 'C1C':
  USE_X_FORWARDED_HOST = True
  FORCE_SCRIPT_NAME = '/vw/default/viz'
  STATIC_URL = posixpath.join(FORCE_SCRIPT_NAME, STATIC_URL.lstrip('/'))
  ARC_BASEURL = posixpath.dirname(STATIC_URL.rstrip('/'))
  LOGOUT_REDIRECT_URL = posixpath.join(ARC_BASEURL, 'apps/login') # SAML login redirect default
else:
  ARC_BASEURL = posixpath.dirname(STATIC_URL.rstrip("/"))
if ARC_BASEURL == "/":
  ARC_BASEURL = ""
LOGIN_URL = posixpath.join(ARC_BASEURL, 'apps/login')
HOME_URL = posixpath.join(ARC_BASEURL, 'apps/home')
USE_LOGOUT_PAGE = False       # For use with SAML or other federated auth
#LOGOUT_REDIRECT_URL = ''     # For use with SAML or other federated auth
#PRE_LOGOUT_REDIRECT_URL = '' # OpenID, redirect URL before clearing session
LOGIN_REDIRECT_URL = posixpath.join(ARC_BASEURL, 'apps/') # SAML login redirect default

# Additional locations of static files
STATICFILES_DIRS = (
  # Put strings here, like "/home/html/static" or "C:/www/django/static".
  # Always use forward slashes, even on Windows.
  # Don't forget to use absolute paths, not relative paths.
  "%s/../static" % THISDIR,
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
  'django.contrib.staticfiles.finders.FileSystemFinder',
  'django.contrib.staticfiles.finders.AppDirectoriesFinder',
  #  'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
#
#   SECRET_KEY Customization:
#   - Setting the SECRET_KEY as an environment variable will allow customers to use their own secret
#     for encrypting certain passwords and secrets in CDV.
#   - If a customer chooses to set SECRET_KEY during startup, they will have to re-enter
#     and save passwords for Connections and Email configurations in the Site Settings.
#     They may also have to re-generate API keys. All of these updates should be a one-time operation.
#   - If no SECRET_KEY is provided, we will continue to use our default key for now.
#
SECRET_KEY = os.environ.get('SECRET_KEY', 'u+tz8_f8#e755ge5f%un=i04tao)+qyd3hfl-atjab^n$mxv$r')

# List of callables that know how to import templates from various sources.
#TEMPLATE_LOADERS = (
#  'django.template.loaders.filesystem.Loader',
#  'django.template.loaders.app_directories.Loader',
#  #     'django.template.loaders.eggs.Loader',
#)

MIDDLEWARE = [
  'arcwebbase.middleware.RequestDataTooBigMiddleware',
  'arcwebbase.middleware.StoreRequestInfoMiddleware',
  'arcwebbase.middleware.iframe_embed_middleware',
  'csp.middleware.CSPMiddleware',
  'django.middleware.security.SecurityMiddleware',
  'django.middleware.clickjacking.XFrameOptionsMiddleware',
  'django.middleware.common.CommonMiddleware',
  'arcwebbase.middleware.RemoveVaryMiddleware',
  'django.contrib.sessions.middleware.SessionMiddleware',
  'arcwebbase.middleware.FlipperMiddleware',
  'django.middleware.csrf.CsrfViewMiddleware',
  'django.contrib.auth.middleware.AuthenticationMiddleware',
  'arcwebbase.middleware.ProfileMiddleware',
  'arcwebbase.middleware.ArcvizSecureMiddleware',
  'django.contrib.messages.middleware.MessageMiddleware',
  'arcwebbase.middleware.AccessLogMiddleware',
  'arcwebbase.middleware.StoreAccessInfoMiddleware',
  'arcwebbase.middleware.ExceptionHandler',
  'arcwebbase.middleware.DynamicCSRFMiddleware',
]

CSP_DEFAULT_SRC = ("'self'",)
CSP_IMG_SRC = ("'self'", "data:", "https:", "http:", "blob:")
CSP_WORKER_SRC = ("'self'", "blob:")
CSP_STYLE_SRC = ("'self'", "'unsafe-inline'")
CSP_SCRIPT_SRC = ("'self'", "'unsafe-inline'", "'unsafe-eval'")
CSP_CONNECT_SRC = ("'self'", "https:")
CSP_FRAME_SRC = ("'self'", "https:")
CSP_FRAME_ANCESTORS = ("'self'",)

SECURE_BROWSER_XSS_FILTER = True
SECURE_CONTENT_TYPE_NOSNIFF = True
SECURE_HSTS_SECONDS = 86400
SECURE_HSTS_INCLUDE_SUBDOMAINS = True
SECURE_HSTS_PRELOAD = True
X_FRAME_OPTIONS = 'DENY'

ROOT_URLCONF = 'arcwebbase.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'arcwebbase.wsgi.application'

TEMPLATES = [
  {
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'DIRS': [
      # insert your TEMPLATE_DIRS here
    ],
    'APP_DIRS': True,
    'OPTIONS': {
      'context_processors': [
        # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
        # list if you haven't customized them:
        'django.contrib.auth.context_processors.auth',
        'django.template.context_processors.debug',
        'django.template.context_processors.i18n',
        'django.template.context_processors.media',
        'django.template.context_processors.static',
        'django.template.context_processors.tz',
        'django.template.context_processors.request',
        'django.contrib.messages.context_processors.messages',

        'arcwebbase.processors.static_url',
        'arcwebbase.processors.arc_css_js_version',
        'arcwebbase.processors.auth_backend',
        'arcwebbase.processors.user_authentication',
        'arcwebbase.processors.app_settings',
        'arcwebbase.processors.userprofile',
        'arcwebbase.processors.arcviz_settings',
        'arcwebbase.processors.feature_flipper',
        'arcwebbase.processors.javascript_directory'
      ],
      # PY3 Fixme: https://docs.djangoproject.com/en/1.11/ref/templates/api/#invalid-template-variables
      'string_if_invalid': ''
    },
  },
]

#TEMPLATE_DIRS = (
#  # Put strings here, like "/home/html/django_templates"
#  # Always use forward slashes, even on Windows.
#  # Don't forget to use absolute paths, not relative paths.
#  "%s/templates" % THISDIR,
#)

INSTALLED_APPS = (
  # The 5 apps below are necessary for admin
  'django.contrib.admin',
  'django.contrib.auth',
  'django.contrib.contenttypes',
  'django.contrib.sessions',
  'django.contrib.messages',

  # 'django.contrib.sites',
  'django.contrib.staticfiles',

  'axes',

  'apps',
  'datasources',
  'datasets',
  'workspace',
  'reports',
  'jobs',
  'rbac',
  'searches',
  'sqlrun',
  'sql',
  'apps.ArcwebConfig',
  'usage',
  'swagger_docs',

  # Uncomment the next line to enable admin documentation:
  # 'django.contrib.admindocs',
)

# For audit logging.  Put in a fully qualified filename in AUDIT_LOGFILE
AUDIT_LOGFILE = None
AUDIT_FILE_MAXBYTES = 100*1024*1024
AUDIT_FILE_MAXCOUNT = 10

# Do not modify this variable directly. Instead, modify AUDIT_LOGFILE
# and this variable will be initialized in the top-level urls.py
AUDIT_LOGGER = None

# Top level arcviz log
ARCVIZ_LOGFILE = os.environ.get('ARCVIZ_LOGFILE', os.path.join(tempfile.gettempdir(), 'arcviz.log'))
ARCVIZ_LOGLEVEL = os.environ.get('ARCVIZ_LOGLEVEL', 'DEBUG')
ARCVIZ_FILE_MAXBYTES = 100*1024*1024
ARCVIZ_FILE_MAXCOUNT = 10

# Used in multi-process mode
try:
  PERSISTENT_PROCESS_ID = int(os.environ.get('PERSISTENT_PROCESS_ID', 1))
except:
  PERSISTENT_PROCESS_ID = 1
try:
  ARCVIZ_LOGGING_PORT = int(os.environ.get('ARCVIZ_LOGGING_PORT', None))
except:
  ARCVIZ_LOGGING_PORT = None

IS_MASTER = PERSISTENT_PROCESS_ID == 1

ARCFMT_PID = '%(asctime)s %(levelname)s P%(persistent_id)s %(process)d %(threadName)s %(message)s'

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
  'version': 1,
  'disable_existing_loggers': False,
  'formatters': {
    'verbose': {
      'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
    },
    'simple': {
      'format': '%(levelname)s %(message)s'
    },
    'arcfmt': {
      'format': '%(asctime)s %(levelname)s %(process)d %(threadName)s %(message)s'
    },
  },
  'handlers': {
    'console':{
      'level': ARCVIZ_LOGLEVEL,
      'class': 'logging.StreamHandler',
      'formatter': 'arcfmt'
    },
    'migrate':{
      'level': ARCVIZ_LOGLEVEL,
      'class': 'logging.handlers.RotatingFileHandler',
      'formatter': 'arcfmt',
      'filename': os.path.join(os.environ.get('LOG_DIR', tempfile.gettempdir()), 'arcviz_migration.log'),
      'maxBytes': 1024 * 1024 * 10,
      'backupCount': 5,
    },
    # This pre_init logger handler is active before the programatic
    # logging configuration in arcviz_initialize() has run and here so
    # that we can capture logs from stuff that get run upon various module
    # imports.
    'pre_init_buffer': {
      'capacity': 10*1024,
      'class': 'logging.handlers.BufferingHandler',
    }
  },
  'loggers': {
    'arcweb': {
      'handlers': ['pre_init_buffer'],
      'level': ARCVIZ_LOGLEVEL,
    },
  }
}

# These loggers will be set up with the same handling and level as the
# main 'arcweb' logger.  One can set a custom level by using a tuple
# rather than number the logger name: example: ('saml2', 'DEBUG')
ADDITIONAL_LOGGERS = ['saml2', 'djangosaml2', 'django_auth_ldap']

# Set up root logger during migrations to support 'logging.debug(msg..)' calls
if 'migrate' in sys.argv:
  LOGGING['loggers'][''] = {'handlers': ['migrate'], 'level': ARCVIZ_LOGLEVEL}

#from django.conf import global_settings
#TEMPLATE_CONTEXT_PROCESSORS = global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
#  "arcwebbase.processors.static_url",
#  "arcwebbase.processors.arc_css_js_version",
#  "arcwebbase.processors.auth_backend",
#  "arcwebbase.processors.app_settings",
#  "arcwebbase.processors.userprofile",
#  "arcwebbase.processors.arcviz_settings",
#)

# CACHES = {
#   'default': {
#     'BACKEND': 'diskcache.DjangoCache',
#     'LOCATION': os.path.expanduser("~/.arc/cache"),
#     'SHARDS': 8,
#     'DATABASE_TIMEOUT': 10.0,
#     'OPTIONS': {
#         'size_limit': 10 * (2 ** 30)  # 10 gigabytes
#     },
#   },
# }

CACHES = {
  'default': {
    # To disable caching
    # 'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    'BACKEND': 'vizlib.arcfilecache.FileBasedCache',
    'LOCATION': f'{BASEDIR}/cache',
    'TIMEOUT': 31536000,
    'OPTIONS': {
      'MAX_ENTRIES': 10000
    }
  }
}

USE_X_FORWARDED_HOST = False

#SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')

# Which OpenSSL Ciphers to use for HTTPS with CherryPy.  Use None for the default OpenSSL ciphers.
HTTPS_CIPHERS = 'ECDH+AESGCM:DH+AESGCM:ECDH+AES256:DH+AES256:ECDH+AES128:DH+AES:RSA+AESGCM:RSA+AES:!aNULL:!MD5:!DSS'

SESSION_COOKIE_NAME = 'arcsessionid'

# Whether to use a secure cookie for the session cookie.
# If this is set to True, the cookie will be marked as “secure”,
# which means browsers may ensure that the cookie is only sent under an HTTPS connection
SESSION_COOKIE_SECURE = False

# The value of the SameSite flag on the session cookie.
# This flag prevents the cookie from being sent in cross-site requests
# thus preventing CSRF attacks and making some methods of stealing session cookie impossible.
# Possible values for the setting are: Strict, Lax, None
SESSION_COOKIE_SAMESITE = 'Lax'

CSRF_COOKIE_NAME = 'arccsrftoken'

CSRF_TRUSTED_ORIGINS = []

SKIP_DYNAMIC_CSRF_MIDDLEWARE = os.getenv('SKIP_DYNAMIC_CSRF_MIDDLEWARE', '').lower() == 'true'

# Whether to use HttpOnly flag on the CSRF cookie.
# If this is set to True, client-side JavaScript will
# not be able to access the CSRF cookie.
CSRF_COOKIE_HTTPONLY = True

# Whether to use a secure cookie for the CSRF cookie.
# If this is set to True, the cookie will be marked as “secure”,
# which means browsers may ensure that the cookie is only sent with an HTTPS connection.
CSRF_COOKIE_SECURE = False

# The value of the SameSite flag on the CSRF cookie.
# This flag prevents the cookie from being sent in cross-site requests.
CSRF_COOKIE_SAMESITE = 'Strict'

# Keep this variable around by default for backward compatibility
ARCSQL_SERVER = "localhost:31050"

# Used to enforce per-user whitelists. Users in this hash will only be able
# to access datasets that are listed as being accessible to them
# Named users are not allowed to create new datasets, or modify existing
# datasets
# Example usage: USER_DATASET_WHITELIST = {'user1': [32, 33], 'user2': [33,35]}
USER_DATASET_WHITELIST = {}

# The list of users in RAWSQL_WHITELIST is permitted to run raw sql commands
# against data
RAWSQL_WHITELIST = set([])

# The path where we keep the file locks
FILELOCK_PATH = os.environ.get('FILELOCK_PATH', tempfile.gettempdir())

# Show all analytical views used to have permissioning issues. Enumerate all
# by running a series of 'show analytical view in `<db`' queries if
# this setting is turned on
SHOW_ALL_AVIEWS_ENUMERATES_DBS = False

# Set to True to enable <script> parameters within shelf elements
# Only to be used for flexibility in dev and testing
ENABLE_SCRIPT_PARAMETERS = False

# If ARCENGINE_VALIDATE_ACCESS is true, we will validate that any ds query run
# is validated against the set of tables in the dataset.
ARCENGINE_VALIDATE_ACCESS = False

# We permit read-only & invalidate/compute queries. Additionally, for different
# bits of functionality from the UI, we permit the following additional commands.
# For read-only systems, reduce this set as needed.
IMPYLA_PERMIT_ADDITIONAL_CMDS = ["drop analytical view", "drop view", "drop table", \
    "alter table", "alter view", "insert", "create analytical view"]

# Enable tooltip highlights
ENABLE_TOOLTIP_HIGHLIGHTS = True

# Example trustedauth settings
# INSTALLED_APPS = INSTALLED_APPS + ('trustedauth',)
#
# AUTHENTICATION_BACKENDS = (
#   'django.contrib.auth.backends.ModelBackend',
#   'trustedauth.backends.TrustedAuthBackend',
# )
#
# TRUSTED_AUTH = {
#   'trusted_ips': ['127.0.0.1'],
#   'trusted_users': ['tuser1, tuser2'],
#   'timeout': 120,
#   'single_use': True,
#   'session_expiry': 30,
#   'allow_superuser': False,
# }

# Arcviz Secure settings
# See arcwebbase/middleware.py
ARCVIZ_SECURE = {
  "enabled": False,
  "default_user": None,
  # "enabled": True,
  # "default_user": "duser",
  "force_default_user": False,
  "session_expiry": 3600,
  "force_embed": True,
  "message": "Access denied. Please contact: <a href='mailto:admin@example.com'>admin@example.com</a>",
  "include_base_urls": True,
  # Access whitelists below
  # format: (regex to match url path, http method(s))
  # List of URLs accessible with no authentication
  "all_list": [
    (r"/arc/static/.+$", ["GET"]),
    (r"/arc/apps/login(\?.+)?$", ["GET","POST"]),
    (r"/arc/trustedauth/getticket$", ["POST"]),
    (r"/arc/trustedauth/trusted/\S+/(app|appgroup)/\d+$", ["GET"]),
    (r"/arc/trustedauth/trusted/\S+/redirect$", ["GET"]),
  ],
  "authenticated_list": [
    # regex to match url path, http method(s)
    (r"/arc/.*$", ["GET", "POST", "PUT", "DELETE"]),
  ],
  # URLs for regular trustedauth users
  "trustedauth_list": [
    (r"/arc/reports/report/\d+$", ["GET"]),
    (r"/arc/sqlrun/csvdownloadStore$", ["POST"]),
    (r"/arc/jobs/adminlogs", ["GET"]),
    (r"/arc/jobs/rerun", ["POST"]),
    (r"/arc/jobs/schedule(s|group)", ["GET"]),
    (r"/arc/jobs/schedulelog", ["DELETE"]),
  ],
  # URLs for default user
  "default_user_list": [
    (r"/arc/apps/app/\d+(\?.+)?$", ["GET"]),
    (r"/arc/reports/report/\d+$", ["GET"]),
  ],
}


# File format for Download Screenshot [PDF or PNG]
#SNAPSHOT_FORMAT = "PDF"
SNAPSHOT_DEFAULT_WIDTH = 1280
SNAPSHOT_DEFAULT_HEIGHT = 1600
SNAPSHOT_TIMEOUT = 180
SNAPSHOT_TMP_DIR_PREFIX = 'snapshot_files'
BULK_REMOVE_OLD_SNAPSHOT_DIRS_INTERVAL = 1*60*60

# size of thread pool performing thumbnail creation
SNAPSHOT_WORKERS=1
# size of thread pool responsible for PDF/PNG creation when the user clicks "Download as PDF/PNG" on a dashboard
ADHOC_SNAPSHOT_WORKERS=1
# size of thread pool responsible for CSV/XLSX creation when the user clicks "Download as CSV/XLSX" on a dashboard
CSV_XLSX_WORKERS=10

PDF_OPTIONS = {
  'ENABLE_CHROMIUM_PDF': True,
  'CHROMIUM_PATH': os.environ.get('CHROMIUM_PATH', '/usr/lib64/chromium-browser/headless_shell'),
  # 'CHROMIUM_PATH': "*"  # For development systems,
  'DPI': 100,
  'HEADER_TEMPLATE': """<span style="font-size: 10px; margin:0 auto"><span class='title'></span></span>""",
  'FOOTER_TEMPLATE': """<span style="font-size: 10px; margin:0 auto"></span><span style="font-size: 10px; right: 0; padding-right: 20px"><span class='pageNumber'></span>/<span class='totalPages'></span></span>"""
}

# Data API settings
DATA_API_ENABLED = False
DATA_API_DEFAULT_ROW_LIMIT = 1000
DATA_API_MAX_ROW_LIMIT = 10000
APIKEYSIZE = 48
APIKEYDEFAULTDAYS = 365

# AdminAPI
ADMINAPI_REQUIRE_APIKEY = False  # If True disable session access to adminapi
ADMINAPI_RETURN_EXISTING_DATASET = False

# DataImport
# If true, will enable data import to overwrite and append existing tables
ENABLE_DATAIMPORT_RW = False

# Enable custom email templates by default
CUSTOM_EMAIL_TEMPLATE = True

# enable dsreq validation
DSREQ_VALIDATION = True

# lockout parameters
# The number of seconds the IP or IP plus user-agent should be locked out
LOCKOUT_TIME = 600
# The maximum number of login attempts before the IP or IP plus user-agent is locked out
LOCKOUT_MAX_ATTEMPTS = 5
# Whether to track failed login attempts by IP plus user-agent, instead of by IP only
LOCKOUT_USE_USER_AGENT = True

# Enable SQL scheduling
ENABLE_SQL_SCHEDULER = True

# Impyla based connections
ENABLE_DATACONN_IMPYLA = True
ENABLE_DATACONN_HYVE = True

# Check whether Legacy thriftpy based connections are enabled
if os.environ.get("ENABLE_LEGACY_DATACONN") == "true":
  ENABLE_DATACONN_ARCENGINE = True
  ENABLE_DATACONN_HIVE = True
  ENABLE_DATACONN_IMPALA = True
else:
  # Disable Legacy thriftpy based connections
  ENABLE_DATACONN_ARCENGINE = False
  ENABLE_DATACONN_IMPALA = False
  ENABLE_DATACONN_HIVE = False

# Disable Arcengine Impyla connector by default since it does not support Smart Acceleration
ENABLE_DATACONN_ARCENGINE_V2 = False

# Other connection types that can be enabled/disabled
ENABLE_DATACONN_KSQL = False
ENABLE_DATACONN_TERADATA = False
ENABLE_DATACONN_BIGQUERY = False
ENABLE_DATACONN_SNOWFLAKE = True
ENABLE_DATACONN_VERTICA = False
ENABLE_DATACONN_ATHENA = False
ENABLE_DATACONN_ELASTICSEARCH = False
ENABLE_DATACONN_CASSANDRA = False
ENABLE_DATACONN_DRILL = False
ENABLE_DATACONN_DENODO = False
ENABLE_DATACONN_ORACLE = False
ENABLE_DATACONN_MSSQL = False
ENABLE_DATACONN_REDSHIFT = False
ENABLE_DATACONN_ASTER = False
ENABLE_DATACONN_DUCKDB = False
ENABLE_DATACONN_DRUID = True
ENABLE_DATACONN_ODBC = False

ENABLE_VISUAL_ACTION = False
ENABLE_VISUAL_TIMELINE = True
ENABLE_WEBHOOK = False
ENABLE_JIRA = False

# Enable lockout feature
LOCKOUT_ENABLED = True

# Enable raw sql queries
ENABLE_QUERIES = True

# Enable timewarp
ENABLE_TIMEWARP = True

# Controls whether particular Arcadia help links are displayed
# in top menu and homepage side panel
HELP_SHOW_WHATSNEW = True
HELP_SHOW_DOCUMENTATION = True
HELP_SHOW_SUPPORT_WEB_PAGE = True
HELP_SHOW_SUPPORT_EMAIL = True
HELP_SHOW_VERSION = True
# Only on homepage
HELP_SHOW_GETSTARTED = True

# Controls links shown in the Help menu within Data Viz
HELP_MENU_LINKS = [
#  {
#    "type": "link",
#    "link": "https://www.arcadiadata.com/resources/knowledge-base/",
#    "label": "Knowledge Base",
#  },
#  {
#    "type": "link",
#    "link": "https://alexandria.arcadiadata.com/",
#    "label": "User Community",
#  },
#  Note that dividers may be specified
#  {
#    "type": "divider",
#  },
#  {
#    "type": "link",
#    "link": "https://www.youtube.com/channel/UCmubDZSISQMKg1Q0HXo03HQ",
#    "label": "YouTube Channel",
#  },
]

# Disable details of dataset request to be printed in the Performance Profile
# through shortcut of control-shift-g
COMPRESS_DSREQ = True
ENABLE_DSREQ_PERF_DISPLAY = False
ENABLE_API_KEYS = True

# AXES SETTINGS
# For additional settings, visit:
# https://django-axes.readthedocs.io/en/latest/4_configuration.html


# If you are implementing custom authentication,
# request middleware, or signal handlers the Axes checks system might
# generate false positives in the Django checks framework.
# https://django-axes.readthedocs.io/en/latest/2_installation.html#disabling-axes-system-checks
# You can silence the unnecessary warnings by using the following Django settings:
SILENCED_SYSTEM_CHECKS = [
  'axes.W002',
  'axes.W003'
]

# lock after 5 failed attempts. I am not sure why we need to set to 10 instead of 5
AXES_FAILURE_LIMIT = 10
AXES_LOCKOUT_TEMPLATE = 'login.html'
AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP = True
# 20 min. cooloff time for locked out users
AXES_COOLOFF_TIME = datetime.timedelta(minutes=20)
# If True, disable writing login and logout access logs to database, so the
# admin interface will not have user login trail for successful user authentication.
AXES_DISABLE_ACCESS_LOG = True

# Turn axes off by default
AXES_ENABLED = False

# Apps were removed from the sidebar as of Banff
# This setting can force it to be drawn
SHOW_APPS_SIDEBAR = False

# Load concatenated 3rd party scripts (vendor.js & vendor.css)
LOAD_VENDOR_SCRIPTS = False

ENABLE_APP_ACCELERATION = True

SOLR_IMPERSONATION_KEY = "doAs"

# Log all django ORM queries if this is set to True and DEBUG is set to True
LOG_ORM_QUERIES = False

# Enable or disable calls to remote REST apis
ENABLE_REMOTECALLS = False

# Enable automatic login as a particular user when an unauthenticated
# user comes into the system. Used for anonymous access to a running
# dataviz instance
AUTOLOGIN_ENABLED = False
AUTOLOGIN_USERNAME = None
AUTOLOGIN_PASSWORD = None

AUTOREGISTER = True

# In some cases, external applications want to embed dataviz thumbnails without
# requiring authentication to fetch the thumbnails. Permit this via a setting
ALLOW_THUMBNAIL_GET_WITHOUT_AUTH = False

HIVE_FIXUP_COLNAMES = True

# whether health check requests should be logged
# a health check request is one made by a health check agent (see HEALTH_CHECK_AGENTS)
SKIP_CML_LOGGING = True

# Identify an HTTP request as being a health check if the User-Agent of the request starts with one of the listed values
# Important in CML/CDSW where a health check is performed every second.
HEALTH_CHECK_AGENTS = ['Go-http-client']

DIRECTSQL_QUERIES_PERMITTED = set(['with', 'select', 'explain', 'describe',
    'show', 'invalidate', '.schema', '.rows', 'create analytical view'])

JSON_ENSURE_ASCII = True

# Note that changing CLEARTEXT_DSREQ to False is surprisingly memory intensive
# We can no longer use streaming http responses, and the response data itself
# is stored multiple times, which has a substantial impact for large resultsets
CLEARTEXT_DSREQ = True

# Allow for slowdown of the query path for testing
#
# Example:
# WORKQ_DELAY_SECONDS = {
#   'sql_1': 10
# }
#
# will delay sending queries to dataconnection 1 by 10 seconds
#
# The lines below will add in random sleeps into sql queries to dataconnection 1
#
# import random
# random.seed()
# def sleepRandom():
#   return random.randint(1, 30)
#
# WORKQ_DELAY_SECONDS = {
#   'sql_1': sleepRandom
# }
#
#
WORKQ_DELAY_SECONDS = {}

# Number of threads used to parallelize batch backend queries
JSONSELECT_THREADCOUNT = 30

# Number of threads used to run async http threads for all potentially
# long-running dataconnection interactions
HTTP_DATACONNECTION_WORKER_THREADCOUNT = 30

# Max values for data connection concurrency settings
DATACONNECTION_CONCURRENCY_DEFAULT = 100
DATACONNECTION_CONCURRENCY_MAX = 100
DATACONNECTION_CONCURRENCY_USER_DEFAULT = 5
DATACONNECTION_CONCURRENCY_USER_MAX = 100
DATACONNECTION_QUERY_TIMEOUT_MINUTES_DEFAULT = 10
DATACONNECTION_QUERY_TIMEOUT_MINUTES_MAX = 60

# Max row upload for csv/url import and data extracts
UPLOAD_MAXROWS_MAX = 1000000000
UPLOAD_MAXROWS_DEFAULT = 1000000

# If set, the http header specified here is used to authenticate users
# Setup for jwt-based authentication
HTTP_HEADERNAME_FOR_TRUSTED_AUTH = None

DEFAULT_EMAIL_PREFIX = f'[{product_name}]'

# initialize list for settings-based FEATURE_FLIPPERS
FEATURE_FLIPPERS = []

#### START CDSW / CML VIZ METASTORE CONFIGURATION ####
metadata_store = os.getenv('METADATA_STORE')
# If the environment variable is set, let's check it for 'mysql', 'postgresql', or 'oracle' values.
#   Else, use existing SQLite database configured above.
if metadata_store in ('mysql', 'postgresql', 'oracle'):
  EXTERNAL_DB_CONFIG = {}
  EXTERNAL_DB_CONFIG['CONN_MAX_AGE'] = int(os.getenv('CONN_MAX_AGE', 60))
  if metadata_store == 'mysql':
    print('# Using MySQL or MariaDB metastore')
    import pymysql
    # Added after 6.3.2 to get around the mysqlclient version error:
    #  django.core.exceptions.ImproperlyConfigured: mysqlclient 1.4.0 or newer is required; you have 0.10.1.
    pymysql.version_info = (2, 0, 3, "final", 0)
    pymysql.install_as_MySQLdb()
    EXTERNAL_DB_CONFIG['ENGINE'] = 'django.db.backends.mysql'
    SOUTH_DATABASE_ADAPTERS = { 'default' : 'south.db.mysql' }
    EXTERNAL_DB_CONFIG['NAME'] = os.getenv('METASTORE_DB_NAME', '')
    EXTERNAL_DB_CONFIG['USER'] = os.getenv('METASTORE_DB_USER', '')
    EXTERNAL_DB_CONFIG['PASSWORD'] = os.getenv('METASTORE_DB_PASSWORD', '')
    EXTERNAL_DB_CONFIG['HOST'] = os.getenv('METASTORE_DB_HOST', '')
    EXTERNAL_DB_CONFIG['PORT'] = os.getenv('METASTORE_DB_PORT', 3306)
  elif metadata_store == 'postgresql':
    print('# Using Postgres metastore')
    EXTERNAL_DB_CONFIG['ENGINE'] = 'django.db.backends.postgresql'
    SOUTH_DATABASE_ADAPTERS = { 'default' : 'south.db.postgresql_psycopg2' }
    EXTERNAL_DB_CONFIG['NAME'] = os.getenv('METASTORE_DB_NAME', '')
    EXTERNAL_DB_CONFIG['USER'] = os.getenv('METASTORE_DB_USER', '')
    EXTERNAL_DB_CONFIG['PASSWORD'] = os.getenv('METASTORE_DB_PASSWORD', '')
    EXTERNAL_DB_CONFIG['HOST'] = os.getenv('METASTORE_DB_HOST', '')
    EXTERNAL_DB_CONFIG['PORT'] = os.getenv('METASTORE_DB_PORT', 5432)
  elif metadata_store == 'oracle':
    print('# Using Oracle metastore')
    try:
      import cx_Oracle
    except ImportError as e:
      print('# Oracle metastore specified, but could not import cx_Oracle')
      print('# Please install cx_Oracle and Oracle instantclient')
      raise
    # Default behavior will be to allow Oracle metadata connections using a service name id (SID)
    #  NOTE: If we use an SID, we don't need to specify a host and port as separate configuration.
    print('# Connecting with Oracle SID')
    use_oracle_sid = os.getenv('METASTORE_DB_ORACLE_USE_SID', 'true')
    EXTERNAL_DB_CONFIG['ENGINE'] = 'django.db.backends.oracle'
    SOUTH_DATABASE_ADAPTERS = {'default': 'south.db.oracle'}
    EXTERNAL_DB_CONFIG['NAME'] = os.getenv('METASTORE_DB_NAME', '')
    EXTERNAL_DB_CONFIG['USER'] = os.getenv('METASTORE_DB_USER', '')
    EXTERNAL_DB_CONFIG['PASSWORD'] = os.getenv('METASTORE_DB_PASSWORD', '')
    # Hostname should be blank when connecting with a Service Name or SID
    EXTERNAL_DB_CONFIG['HOST'] = ''
    EXTERNAL_DB_CONFIG['OPTIONS'] = { 'threaded': True }
    # If we're not using an SID while connecting with the NAME option,
    # then we need to specify the hostname and port.
    if use_oracle_sid.lower() == 'false':
      print('# Connecting with Oracle Hostname/Port')
      EXTERNAL_DB_CONFIG['HOST'] = os.getenv('METASTORE_DB_HOST', '')
      EXTERNAL_DB_CONFIG['PORT'] = os.getenv('METASTORE_DB_PORT', 1521)

  # Update and replace default Databases settings (SQLite) with External Database settings
  DATABASES['default'].update(EXTERNAL_DB_CONFIG)
#### END CDSW / CML VIZ METASTORE CONFIGURATION ####

# This is the prefix that we insist on the data extract tablenames. If it is set
# to an empty string, no prefix is checked for.
DATA_EXTRACTS_TARGET_TABLENAME_PREFIX = "cdv_"

# Number of rows inserted at a time via data import and data extract inserts
IMPORT_BATCH_INSERT = 1000


if os.getenv('CDV_SUB_MODE') == 'C1C':
  # This has to be the same as the default data connection name defined in
  # the packaging/assets/startup/initviz.py file
  DATASET_AUTOCREATE_DATACONNECTION = 'Default Impala VW'
else:
  DATASET_AUTOCREATE_DATACONNECTION = None

# Use the name or id of a different workspace as default
# If users do not have permission to write to this workspace for some reason
# the user's private workspace will be used instead
DEFAULT_WORKSPACE = None
EMBED_URL_PREFIX = ''

# SETUP CDSW/CML EXPLORE
if os.getenv('CDV_MODE') ==  'runtime':
  FEATURE_FLIPPERS.extend([{'id': 'enable_cml_sync', 'value': True}])

if os.getenv('CDV_SUB_MODE') == 'MLExplore':
  # TODO: Update once DSE-21100 fixed to include project url instead of api url
  # Currently, url in format "https://ml-afa4e058-1a6.eng-ml-d.xcu2-8y8x.dev.cldr.work/api/v1/projects/csso_szhou/cml-explore"
  # We need the url to point to the data page where it's embedded "https://ml-afa4e058-1a6.eng-ml-d.xcu2-8y8x.dev.cldr.work/csso_szhou/cml-explore/data"
  cml_url = os.getenv('CDSW_PROJECT_URL')
  if cml_url:
    EMBED_URL_PREFIX = f'{cml_url.replace("api/v1/projects/", "")}/data#'

  os.environ['IS_EMBED'] = 'true'
  DEFAULT_WORKSPACE = 'Public'
  DEFAULT_EMAIL_PREFIX = '[Data Discovery and Visualization]'
  FEATURE_FLIPPERS.extend([
      {'id': 'disable_visuals_for_workflows', 'value': True},
      {'id': 'enable_explore_style', 'value': True},
      {'id': 'enable_jobs', 'value': False},
      {'id': 'hide_workspaces', 'value': True},
    ])

if os.getenv('IS_EMBED') == 'true':
  SESSION_COOKIE_SAMESITE = 'None'
  CSRF_COOKIE_SAMESITE = 'None'
  CSRF_COOKIE_SECURE = True
  SESSION_COOKIE_SECURE = True
  # Set referrer policy to avoid CORS related issues when iframing
  SECURE_REFERRER_POLICY = 'strict-origin-when-cross-origin'

SERVICE_DISCOVERY_ENDPOINT = os.getenv('SERVICE_DISCOVERY_ENDPOINT', '')

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

# When set to True, snapshot creation will open headed Chrome instances (useful for debugging).
# PDF generation will be switched off in this case
DEBUG_SNAPSHOT_HEADED=False

# maximum number of logs shown by the Job status page
JOBS_MAX_LOGS=100

# The interval (in seconds) for removing read notifications
BULK_REMOVE_READ_NOTIFICATION_INTERVAL = 24*60*60

# Permission to delete and create via the CLI tool
ENABLE_DELETE_VIA_CLI_TOOL = False
ENABLE_CREATE_VIA_CLI_TOOL = False

# The maximum size in bytes that a request body may be before a SuspiciousOperation (RequestDataTooBig) is raised.
# Default: 2621440 (i.e. 2.5 MB). More info: https://docs.djangoproject.com/en/4.2/ref/settings/#data-upload-max-memory-size
# Avoid setting it too small(1000) otherwise sending queries, even to reset this setting will be impossible
DATA_UPLOAD_MAX_MEMORY_SIZE = 100000000

# enable removal of timed-out query trackers:
CLEANUP_QUERY_TRACKERS=True

# extra time we should keep the trackers of expired queries (after expiration):
CLEANUP_QUERY_TRACKERS_GRACE_PERIOD_SECONDS=10
# hostname poll attempts in second
HOSTNAME_POLL_INTERVAL = 0.5

# Remove daily the orphaned reports generated during editing a new dashboard without saving it.
BULK_REMOVE_ORPHAN_REPORTS = 24*60*60

# Daily remove old sessions. 0 or negative value turns this feature off
SESSION_CLEANUP_INTERVAL = 24*60*60

# Password default placeholder
DEFAULT_PASSWORD_PLACEHOLDER = "******************************"

# object urls used to temporarily store downloaded CSV or Excel content
# will be revoked after this time:
OBJECT_URL_CLEANUP_SECONDS=int(os.getenv('OBJECT_URL_CLEANUP_SECONDS', '180'))

HTTP_PROXY_SSL_VERIFY=True

from dataconn.ext.phoenix.phoenix_ext import PhoenixExtension
from dataconn.ext.trino.trino_ext import TrinoExtension
DATACONN_EXTENSIONS = [PhoenixExtension(), TrinoExtension()]

# eventstracker intervals in seconds
EVENTSTRACKER_DATA_TO_CML = float(os.getenv('EVENTSTRACKER_DATA_TO_CML', '86400')) # 24 hours
EVENTSTRACKER_BACKEND_DATA_TO_USAGE_DB = float(os.getenv('EVENTSTRACKER_BACKEND_DATA_TO_USAGE_DB', '3600')) # 1 hour
EVENTSTRACKER_FRONTEND_DATA_TO_USAGE_DB = float(os.getenv('EVENTSTRACKER_FRONTEND_DATA_TO_USAGE_DB', '1800')) # half an hour

CDV_ADDITIONAL_API_ENDPOINTS = os.getenv('CDV_ADDITIONAL_API_ENDPOINTS', False)