# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from apps.tests.userrole import base
from arcwebbase import access_info
from django.http import HttpRequest
from util.request import create_request

import django.test
import threading


class TestAccessInfo(django.test.TestCase):

  def tearDown(self):
    # make sure nothing remains in the main thread
    access_info.clear()

  def test_use(self):

    # it is OK to access it without first setting it
    ai = access_info.get()
    self.assertIs(ai, access_info.EMPTY)

    # if we set it, we should get it back
    my_ai = access_info.AccessInfo(username='a')
    access_info.store(my_ai)
    ai = access_info.get()
    self.assertEquals(ai.username, 'a')

    # clearing works, too:
    access_info.clear()
    ai = access_info.get()
    self.assertIs(ai, access_info.EMPTY)

  def test_create(self):
    ai = access_info.create_from_request(HttpRequest())
    self.assertIsNotNone(ai)

    user1 = base.make_user('user1')
    self.assertGreater(user1.id, 0)

    req = create_request(user1, 'a.b.c.d')
    ai = access_info.create_from_request(req)
    self.assertEqual(ai.username, 'user1')
    self.assertEqual(ai.user_id, user1.id)
    self.assertEqual(ai.client_ip, 'a.b.c.d')

  def test_decorator(self):
    ai1 = access_info.AccessInfo(username='user1')
    access_info.store(ai1)

    callback_ai = None

    @access_info.store_access_info
    def callback():
      nonlocal callback_ai
      callback_ai = access_info.get()

    access_info.clear()
    self.assertIs(access_info.get(), access_info.EMPTY)

    t = threading.Thread(target=callback)
    t.start()
    t.join()

    self.assertIs(callback_ai, ai1,
                    'callback has the access_info from callback definition time')
