# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from arcwebbase import ai_support
from util.ff_flipper import FeatureFlipper
from vizlib import querylog

import json
import django.test


class TestAiSupport(django.test.TestCase):

  def setUp(self):
    FeatureFlipper().set_flipper('redact_ai_assistant_logs', True, 'DB')

    self.old_redactors = querylog.REDACTORS.copy()
    querylog.REDACTORS.clear()

  def tearDown(self):
    querylog.REDACTORS.extend(self.old_redactors)

  def test_initialize(self):
    ai_support.initialize()
    self.assertEquals(querylog.REDACTORS, [ai_support.redact_query])

  def test_redacting(self):

    acted, result = ai_support.redact_query('a')
    self.assertFalse(acted)
    self.assertIsNone(result)

    acted, result = ai_support.redact_query('{"a}')
    self.assertFalse(acted)  # invalid json
    self.assertIsNone(result)

    acted, result = ai_support.redact_query('{"a":"b"}')
    self.assertFalse(acted)
    self.assertIsNone(result)

    input = json.dumps({'aiQuery': True})
    acted, result = ai_support.redact_query(input)
    self.assertFalse(acted)  # no body
    self.assertIsNone(result)

    valid_input = json.dumps(
      {'aiQuery': True, 'body': {'upstreamReqBody': 'a'}})
    expected_result = json.dumps(
      {'aiQuery': True, 'body': 'REDACTED'})

    input = valid_input
    acted, result = ai_support.redact_query(input)
    self.assertTrue(acted)
    self.assertEquals(result, expected_result)

    # redacting can be switched on/off during runtime:
    FeatureFlipper().set_flipper('redact_ai_assistant_logs', False, 'DB')

    input = valid_input
    acted, result = ai_support.redact_query(input)
    self.assertFalse(acted)  # redacting is switched off
    self.assertIsNone(result)
