# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.


from arcwebbase import props
import unittest
import os
import tempfile


class TestProps(unittest.TestCase):
  def setUp(self):
    self.environ_ori = os.environ
    os.environ = {}

    tempdir = tempfile.gettempdir()
    file_name = 'dataviz-test.properties'

    os.environ['CONF_DIR'] = tempdir
    os.environ['DATAVIZ_PROPERTIES'] = file_name
    self.properties_file = f'{tempdir}/{file_name}'

  def tearDown(self):
    os.environ = self.environ_ori
    if os.path.isfile(self.properties_file):
      os.remove(self.properties_file)

  def test_property_file_reading(self):
    print(os.environ)

    with open(self.properties_file, mode='wt') as f:
      # we preserve casing
      f.write('prop_A=aAa\n')
      f.write('Prop_b=\n')
      f.write('prop_c=cCc\n')
      f.write('prop_d=dDd\n')

    # already existing environment values have priority over values read from file
    os.environ['prop_c'] = 'c'
    os.environ['prop_d'] = None
    props.update_env_with_properties()

    self.assertEqual(os.environ['prop_A'], 'aAa')
    self.assertEqual(os.environ['Prop_b'], '')
    self.assertEqual(os.environ['prop_c'], 'c')
    self.assertEqual(os.environ['prop_d'], None)
