# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from arcwebbase import util
from django.test import TestCase
from django.contrib.sessions.backends.db import SessionStore
from django.contrib.sessions.models import Session
from vizlib.tests.test_report_count import run_with_db_query_check2

import django
import django.db


class TestUtil(TestCase):

  def setUp(self):
    s = SessionStore()
    # count = 10_000
    count = 300
    for _ in range(count):
      s.create()

    self.assertEquals(Session.objects.count(), count)

  def test_delete_all_sessions(self):

    run_with_db_query_check2(lambda: Session.objects.all().delete())

    self.assertEquals(Session.objects.count(), 0)
    # multiple queries are issued: 1 select + 3 deletes in batches of 100:
    self.assertEquals(len(django.db.connection.queries), 1 + 3)

  def test_delete_all_sessions_raw(self):
    all_sessions = Session.objects.all()

    run_with_db_query_check2(lambda: all_sessions._raw_delete(all_sessions.db))

    self.assertEquals(Session.objects.count(), 0)
    # a single query is issued:
    self.assertEquals(len(django.db.connection.queries), 1)

  def test_util_delete_all_sessions(self):

    run_with_db_query_check2(util.delete_all_sessions)

    self.assertEquals(Session.objects.count(), 0)
    # a single query is issued:
    self.assertEquals(len(django.db.connection.queries), 1)
