# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import os
from django.conf.urls import include
from django.urls import re_path
from django.conf import settings
from django.contrib.staticfiles import views as staticviews
from django.conf.urls.static import static
from django.views import static as djangoviewsstatic
from arcwebbase import views, ai_profile_tester

from apps import views_shortener

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = [
  re_path(r'^$', views.home, name='root-url'),

  re_path(r'^stats_data$', views.stats_data, name='stats-data-url'),
  re_path(r'^stats_data/(?P<all>.*)$', views.stats_data, name='stats-data-url-all'),

  re_path(r'^actions/', include('actions.urls')),
  re_path(r'^api/', include('api.urls')),
  re_path(r'^searches/', include('searches.urls')),
  re_path(r'^apps/', include('apps.urls')),
  re_path(r'^reports/', include('reports.urls')),
  re_path(r'^datasets/', include('datasets.urls')),
  re_path(r'^datasources/', include('datasources.urls')),
  re_path(r'^roles/', include('rbac.urls')),
  re_path(r'^sqlrun/', include('sqlrun.urls')),
  re_path(r'^trustedauth/', include('trustedauth.urls')),
  re_path(r'^jobs/', include('jobs.urls')),
  re_path(r'^migration/', include('migration.urls')),
  re_path(r'^workspace/', include('workspace.urls')),
  re_path(r'^sql/', include('sql.urls')),
  re_path(r'^short/', include([
    re_path(r'^$', views_shortener.urlShortener, name='urlshortener'),
    re_path(r'^(?P<slug>.+)$', views_shortener.redirect_url_view)
  ])),
  re_path(r'^usage/', include('usage.urls')),
  re_path(r'^validate_ai_profile/$', ai_profile_tester.validate_ai_service_connection, name='validate_ai_profile'),
]

ENABLE_QUNIT = os.getenv('ENABLE_QUNIT', '').lower() == 'true'
if ENABLE_QUNIT:
  urlpatterns += [re_path(r'^qunit/', include('qunit.urls'))]

if len(getattr(settings, 'ADMIN_API_URL_LIST', [])) > 0:
  urlpatterns += [re_path(r'^adminapi/', include('adminapi.urls'))]

if 'djangosaml2' in settings.INSTALLED_APPS:
  urlpatterns += [
    re_path(r'^saml2/', include('djangosaml2.urls')),
  ]

if 'djangooidc' in settings.INSTALLED_APPS:
  urlpatterns += [
    re_path(r'^openid/', include('djangooidc.urls')),
  ]

if settings.DEBUG:
  try:
    import debug_toolbar
    urlpatterns += [
      re_path(r'^__debug__/', include(debug_toolbar.urls)),
    ]
  except:
    pass

urlpatterns += [
  # Uncomment the admin/doc line below to enable admin documentation:
  # re_path(r'^admin/doc/', include('django.contrib.admindocs.urls')),

  # Uncomment the next line to enable the admin:
  re_path(r'^admin/', admin.site.urls),
]

if getattr(settings, 'RUN_FROM_SOURCE_DEBUG', False):

  # Add arc prefix when running builtin django webserver
  urlpatterns = [re_path(r'arc/', include(urlpatterns)),]

  # Workaround for serving static files in non-DEBUG mode
  if settings.DEBUG is False:   #if DEBUG is True it will be served automatically
    urlpatterns += [
      re_path(r'arc/static/(?P<path>.*)$', djangoviewsstatic.serve,
          {'document_root': settings.STATICFILES_DIRS[0]}),
    ]
  else:
    # PY3 FIX?
    urlpatterns += [re_path(r'^arc/static/(?P<path>.*)$', staticviews.serve), ]
    #urlpatterns += static(settings.STATIC_URL,
    #                      document_root=settings.STATICFILES_DIRS[0])

  # Make the root url re-direct to /arc
  urlpatterns += [
    re_path(r'^$', views.home, name='root-url'),
  ]

