# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from arcwebbase import ai_support
import logging
from util.perfcounters import APPROX_COUNTERS

proxy_request_counter = APPROX_COUNTERS.get('proxy.http_requests.current')

LOG = logging.getLogger('arcweb.proxy')

ai_support.initialize()


async def proxy_comm(req: dict, upstream_comm):
  try:
    proxy_request_counter.value += 1
    async for chunk in _proxy_comm0(req, upstream_comm):
      yield chunk
  finally:
    proxy_request_counter.value -= 1


async def _proxy_comm0(req: dict, upstream_comm):
  body = req.get('body')
  model_type = req.get('modelType', '')
  settings = req.get('settings', {})
  streaming_enabled = settings.get(f"{model_type}_stream", False)
  
  prof_data = {
    'settings': settings,
    'input': body.get('input'),
    'model_type': model_type
  }

  async for chunk in upstream_comm(prof_data):
    if streaming_enabled:
      yield chunk
    else:
      req.pop('settings')
      status_code = chunk['status_code']
      req_info = chunk.get('req_info')
      key = 'body' if status_code == 200 else 'error'
      proxy_response = {
        'request': req,
        'response': {
          'status': status_code,
          key: chunk['response_body']
        }
      }
      
      if req_info:
        proxy_response['clear_text_request'] = req_info

      yield {
        'status_code': status_code,
        'response_body': proxy_response
      }
