# Cloudera Data Visualization CLI tool

## Getting started
To see all the available commands, execute the following
from the `viz` directory:
`dev/vizcli manage.py cli`
or from the `dev` directory:
`./vizcli manage.py cli`

## Retrieving metadata of the CDV components

### Show
Currently this functionality is implemented for dataconnections, datasets, visuals and users.
```
cli *object_type* show
```
where `*object_type*` can be connection, dataset, visual or user. This command will list out all the choosen type to the stdout.

### Read
Currently this functionality is implemented for dataconnections, datasets, visuals and users. To retrieve metadata in JSON format, use the following command:
```
cli *object_type* read
```
where `*object_type*` can be connection, dataset, visual or user. This command will print the metadata so it can be piped into a user-specified file location.

### Export_assets
For dataset and visual **only**, you can use the `export_assets` command.
```
cli *object_type* export_assets
```
Its implementation contains the Django `export_objects` function. So with export_assets you can export alongside all the relations / dependencies as well. This command will automatically save the result in the file of the following format: `*object_type*_read_*current_datetime*.json` in the arcweb directory of the project root folder. You can use this json file to use the `import_assets` functionality of the CLI tool.

## Object creation based on the CDV component metadata

### Create
Currently this functionality is implemented for dataconnections, datasets and users. To create and persist an object to the local database based on the JSON file, use the following command:
```
cli *object_type* create <source_filepath>
```
When using `create` command, the Django `get_or_create` method is called, consequently auditing is added in order for changes to be tracked. `create` command also checks the JSON for validity, more on this in the next section. `create` returns the IDs of the created objects as well, so they can be referenced by the caller.

To use this function, you have to enable the `ENABLE_CREATE_VIA_CLI_TOOL` flag via the advanced settings section under site settings.

## Object deletion
Currently this functionality is implemented for dataconnections, users, datasets and visuals. To delete an object from the local database, use the following command:
```
cli *object_type* delete <integer_id>
```
where `*object_type*` can be connection, user, dataset, visual.

When using this command, the Django `delete` method is called, consequently auditing is added in order for changes to be tracked.

To use this function, you have to enable the `ENABLE_DELETE_VIA_CLI_TOOL` flag via the advanced settings section under site settings.

### Import_assets

For dataset and visual **only**, you can also use `import_assets` command.
```
cli *object_type* import_assets <dataconnection_name> <source_filepath> <workspace_id>
```
Its implementation contains the Django `import_objects` function, with the [settings](../../../arcweb/migration/migrate.py) `dry_run = False` and `skip_permission_check = True`. This command takes care of all the dependencies between visuals (e.g. dashboards -> other visuals). Because of this, for the command to work properly, you should use the source file created as a result of the `export_assets` command since in this way the JSON file is certainly valid for `import_assets`, unlike if you create a file with the `create` command. Also you can use any other JSON files that have been created in the app via the built-in exporting functionality ( that is to say using the UI ).

**Important!** Please use the absolute path of the file for the `<source_filepath>`.

## Data validation in `create` command
### Data Connection

Down find the list of **compulsory** fields in `"PARAM"` of the json file for the tested Data Connections ("dataconnection_name"):
```
- SQLite ("sqlite"): "FILE"
- Postgres ("postgres"): "HOST", "PORT", "DATABASE"
- MySQL ("mysql"): "HOST", "PORT", "DATABASE", "USERNAME"
- Hive ("hyve"): "HOST", "PORT", "USERNAME"
- Impala ("impyla"): "HOST", "PORT"
```

To find the **non-compulsory** fields in `"PARAMS"` for a Data Connection, check all the fields which are not in `Basic`(e.g. `Cache`, `Advanced`, `Parameters`). Use for reference the [CDV Dataconnection docs](https://docs.google.com/document/d/10uz5SaZOc6gjWPc8OkWjaR-rmZ8KvCTMs0-HDyQoCSg). To find the exact json name of a field, take a look into an HTTP request.

To retrieve the fields of an existing Data Connection, in `.arc/db.local` run:
```sql
select dataconnection_info from datasets_dataconnection;
```

This is an example of `JSON` file.

```json
{
  "dataconnection_type": "sqlite",
  "dataconnection_name": "cli_test_sqlite_conn",
  "dataconnection_info": {"PARAMS": {"FILE": "../arcweb/samples/samples.sqlite"}}
}
```

Find more examples of json files to test [here](https://drive.google.com/file/d/185_26itv4shBRRgwapYBpL7cTktodxF7/view).

### Dataset

The necessary fields in json file are: `dataset_info` as a `list of dict`,
`dataconnection_id` as an `int`. `create_user` should be a `vizapps_admin`.

This is an example of `JSON` file.

```json
{
    "dataset_name":"JSON PARSER",
    "dataset_type":"singletable",
    "dataset_detail":"main.details",
    "create_user":"vizapps_admin",
    "update_user":"vizapps_admin",
    "dataset_info":[
       {
          "tablename":"main.retail_food_store_inspections_current_critical_violations",
          "columns":[
             {
                "name":"county",
                "type":"STRING",
                "isdim":true,
                "alias":"county"
             }
          ]
       }
    ],
    "dataset_everyone_perm":"readwrite",
    "dataset_create_user_perm":"setupperm",
    "dataconnection_id":"1"
 }
 ```
datasets are stored in datasets_dataset
you can query them from db.local:
```sql
select * from datasets_dataset;
```

### User

This command creates a User from the parameters given in the `<filepath>` json file. The absolute path is preferred. The **compulsory** fields of the json are `"username"` and `"password"`. More on user fields in [Django docs](https://django.readthedocs.io/en/3.2.x/ref/contrib/auth.html#django.contrib.auth.models.User).

Here is an example of the JSON file:
 ```json
[
	{
	"username": "Test1",
	"password": "12ddd45"
	},

	{
   "username": "Test2",
   "password": "12345"
   },

	{
	"username": "Test3",
	"password": "abcdefgh"
	},

	{
	"username": "Test4",
	"password": "00000000",
	"first_name": "Admin"
	}
]
 ```

To retrieve the existing Users, in `.arc/db.local` run:
```sql
select * from auth_user;
```

And for the corresponding User Profiles:
```sql
select * from apps_userprofile;
```

**Important!** This command executes **cascade** deletion, which means that if you delete an object that is referenced by other objects (e.g. datasets refer to dataconnections), then the referred object will be deleted as well. For this purpose always make sure that you **definitely** want to delete something. The CLI interface will ask you for a confirmation with "Y" or "y".

## CLI usage examples

```
./vizcli manage.py cli connection read
```

```
./vizcli manage.py cli connection delete 12
```

```
./vizcli manage.py cli dataset export_assets
```

```
./vizcli manage.py cli dataset import_assets samples /Users/admin/dataset_export.json 1
```

## Adding new commands to the CLI
Every CDV component as an according file in `plugins`. To register a new command, you should add a tuple to the `_COMMANDS` array in the according .py file:

```python
_COMMANDS = [
  ('name to be used in the CLI',
  corresponding_python_function_implementation,
  boolean_local_datatbase_remains_unchanged,
  'regex for acceptable command arguments',
  'description of the command'),
   ...
   #example
  ('create', create, False, '^.*\.json$',
   "creates a new user from a json file"),
   ...
]
```

