# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
module tracker
"""

import os
import imp

from cli.core import misc
from cli import plugins

PLUGINS_DIR = "plugins"

PLUGIN_COMMANDS_ORDERED = []
PLUGIN_COMMANDS = {}
PLUGIN_HELP = {}

def _load_plugin(plugin):
  """Load a plugin identified by its filename"""
  plugin_dir = os.path.join(misc.acli_path(), PLUGINS_DIR)
  plugin_name = plugin[:-3]

  fullpath = os.path.join(plugin_dir, plugin)
  if not os.path.isfile(fullpath):
    raise Exception(f"Invalid plugin {plugin}")

  # Load module only if it has not been already explicitly imported by some
  # other module
  module = plugins.__dict__.get(plugin_name, None)
  if not module:
    module = imp.load_source(f'plugins.{plugin_name}', fullpath)
  plugin_defn = module.plugin_defn()
  if not plugin_defn:
    return

  commands = plugin_defn['commands']

  for cmd in commands:
    (name, fn_or_dict, readonly, pattern, help_text) = cmd
    cmd_name = ' '.join([plugin_name, name])
    PLUGIN_COMMANDS_ORDERED.append(cmd_name)
    PLUGIN_COMMANDS[cmd_name] = (fn_or_dict, readonly, pattern, help_text)
    PLUGIN_HELP[cmd_name] = help_text

def load_plugins(pluginsl=None):
  """Load the specified plugins or all plugins (if not specified)"""
  plugin_dir = os.path.join(misc.acli_path(), PLUGINS_DIR)

  possible_plugin = []
  loaded_plugin = None

  dirlist = sorted(os.listdir(plugin_dir))

  for plugin in dirlist:
    if plugin.startswith('_'):
      continue
    if not plugin.endswith('.py'):
      continue
    plugin_name = plugin[:-3]
    if pluginsl and not plugin_name in pluginsl:
      if len(pluginsl) == 1 and plugin_name.startswith(pluginsl[0]):
        possible_plugin.append(plugin)
      continue

    loaded_plugin = plugin
    _load_plugin(plugin)

  if pluginsl and not PLUGIN_COMMANDS:
    if len(possible_plugin) == 1:
      loaded_plugin = possible_plugin[0]
      _load_plugin(possible_plugin[0])

  if pluginsl:
    if PLUGIN_COMMANDS:
      return loaded_plugin[:-3]
    else:
      if len(possible_plugin) == 0:
        raise Exception(f"Invalid command section '{','.join(pluginsl)}'")
      if len(possible_plugin) > 1:
        raise Exception(f"Invalid command section '{','.join(pluginsl)}'. Multiple options {possible_plugin}")
  else:
    return None

