# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
from django.test import TestCase
from unittest.mock import patch
from cli.plugins.connection import *
from cli.utils import validate_params_of_a_dataconnection, create_dataconnection

class ConnectionTest(TestCase):
    def test_impyla_host_missing(self):
        json_to_test = {'dataconnection_type':'impyla', 'dataconnection_info': {'PARAMS': {'PORT': 80}}}
        self.assertRaises(ValueError, validate_params_of_a_dataconnection, json_to_test)

    def test_successful_check_impyla(self):
        json_to_test = {'dataconnection_type': 'impyla', 'dataconnection_info': {'PARAMS': {'HOST': 'localhost', 'PORT': 80}}}
        result = validate_params_of_a_dataconnection(json_to_test)
        self.assertEqual(json.dumps(json_to_test['dataconnection_info']), result)

    def test_json_missing(self):
        self.assertRaises(KeyError, validate_params_of_a_dataconnection, {})

    def test_json_params_missing(self):
        json_to_test = {'dataconnection_type':'mysql', 'dataconnection_info': {'PARAMS': {'PORT': 80}}}
        self.assertRaises(ValueError, validate_params_of_a_dataconnection, json_to_test)

    def test_create_connection_with_failed_info_validation(self):
        dict_to_test = {'dc_type': 'any',
                        'dc_name': 'any',
                        'dc_description': 'any',
                        'dc_password': 'any'}
        self.assertRaises(Exception, create_dataconnection, dict_to_test)

    def test_create_empty_connection(self):
        self.assertRaises(Exception, create_dataconnection, {})

    @patch('cli.plugins.connection.settings')
    def test_create_connection_with_disabled_env_flag(self, settings):
        settings.ENABLE_CREATE_VIA_CLI_TOOL=False
        result = create({})
        message = 'You have no permission to create via the CLI Tool. You can enable it via the Advanced Settings, please check the documentation'
        self.assertEqual(message, result)

    @patch('cli.plugins.connection.settings')
    def test_delete_connection_with_disabled_env_flag(self, settings):
        settings.ENABLE_DELETE_VIA_CLI_TOOL=False
        result = delete([1])
        message = 'You have no permission to delete via the CLI Tool. You can enable it via the Advanced Settings, please check the documentation'
        self.assertEqual(message, result)
