# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from dataconn.connection_updater import ConnectionUpdater
from datasets.models import DataConnection

import json


class CmlConnectionUpdater(ConnectionUpdater):
    def __init__(self):
        self.unavailable_connections = []

    def sync_out_connections(self, new_connections):
        synced_out_connections = []
        new_connections_cml_ids = [c['cml_id'] for c in new_connections]
        old_connections = DataConnection.objects.filter(cml_id__isnull=False).exclude(cml_id__in=new_connections_cml_ids)

        unavailable_connection_ids = [c['id'] for c in self.unavailable_connections]
        for connection in old_connections:
            if connection.cml_id in unavailable_connection_ids:
                cml_status = json.loads(connection.cml_status)
                cml_status['isAvailable'] = False
                connection.cml_status = json.dumps(cml_status)
            else:
                synced_out_connections.append(connection.dataconnection_name)
                connection.cml_id = None
                connection.cml_status = '{}'

            connection.save()

        return synced_out_connections

    def set_unavailabe_connections(self, unavailable_connections):
        self.unavailable_connections = unavailable_connections
