# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json

from jobs.views_encryption import decrypt_secret


class ConnectionUpdater:
    def update(self, parsed_dataconnection: dict, dataconnection_in_db) -> bool:
        should_update: bool = False

        if self.__should_update_password(dataconnection_in_db.dataconnection_password, parsed_dataconnection["dataconnection_password"]):
           dataconnection_in_db.dataconnection_password  = parsed_dataconnection["dataconnection_password"]
           should_update = True

        existing_info = json.loads(dataconnection_in_db.dataconnection_info)
        new_info = json.loads(parsed_dataconnection["dataconnection_info"])
        if self.__should_update_info(existing_info, new_info):
            existing_info["PARAMS"]["USERNAME"] = new_info["PARAMS"]["USERNAME"]
            existing_info['PARAMS']['CONNECTIONNAME_EXTENDED'] = new_info['PARAMS']['CONNECTIONNAME_EXTENDED']
            dataconnection_in_db.dataconnection_info = json.dumps(existing_info)
            should_update = True

        old_conn_name: str = dataconnection_in_db.dataconnection_name
        new_conn_name: str = parsed_dataconnection['dataconnection_name']
        if new_conn_name != old_conn_name:
            dataconnection_in_db.dataconnection_name = new_conn_name
            should_update = True

        old_cml_status: dict = dataconnection_in_db.cml_status
        new_cml_status: dict = parsed_dataconnection['cml_status']
        if new_cml_status != old_cml_status:
            dataconnection_in_db.cml_status = new_cml_status
            should_update = True

        if should_update:
            dataconnection_in_db.save()
            return True
        
        return False

    def __should_update_password(self, old_password: str, new_password: str) -> bool:
        if old_password is None and new_password is None:
            return False
        
        if old_password is None or new_password is None:
            return True
    
        if decrypt_secret(**json.loads(old_password)) == decrypt_secret(**json.loads(new_password)):
            return False
        
        return True

    def __should_update_info(self, old_info: dict, new_info: dict) -> bool:
        is_same_username: bool = old_info["PARAMS"]["USERNAME"] == new_info["PARAMS"]["USERNAME"]
        is_same_extended_connectionname = old_info['PARAMS']['CONNECTIONNAME_EXTENDED'] == new_info['PARAMS']['CONNECTIONNAME_EXTENDED']
        if is_same_username and is_same_extended_connectionname:
            return False

        return True
