# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from . import dataconn_odbc
from .connections_types import DATACONN_ASTER
from .data_api_cmds import SPECIAL_CMDS, VALIDATE_CMDS
from .dataconn_dataengine import BuildRequestExecutor
from django.conf import settings

import json

class AsterConnProperties(dataconn_odbc.OdbcConnProperties):
  def __init__(self):
    super(AsterConnProperties, self).__init__()
    self.quote = '"'
    self.quote_end = '"'
    self.quote_alias = '"'
    self.quote_escape_character = '"'
    self.quote_string_constant = "'"
    self.supports_partitions = False
    self.supports_async = False
    self.supports_backticks = False
    self.supports_reuse = False
    self.supports_import_data_perm = False
    self.supports_import_data_dbname = 'public'
    self.alias_supports_backticks = False
    self.string_type = "varchar"
    self.date_type = "date"
    self.double_type = "double precision"


class AsterConn(dataconn_odbc.OdbcConn):
  def __init__(self, params, connection_id, conn_type=DATACONN_ASTER):
    super(AsterConn, self).__init__(params, connection_id, conn_type=conn_type)

  def properties(self):
    return AsterConnProperties()

  def execute(self, build_request, async_flag=False):

    query = None
    if build_request.query == SPECIAL_CMDS['SHOW_DATABASES']:
      query = "select schemaname from nc_system.nc_user_schemas where schemaname not like 'nc_%'"
    elif build_request.query.startswith(SPECIAL_CMDS['SHOW_TABLES']):
      schema = build_request.query.split()[-1]
      query = "select tablename from nc_system.nc_user_tables as a " \
              "join " \
              "(select schemaid from nc_system.nc_user_schemas where schemaname = '%s') as b " \
              "on a.schemaid = b.schemaid " \
              "union all " \
              "select viewname from nc_system.nc_user_views as a " \
              "join " \
              "(select schemaid from nc_system.nc_user_schemas where schemaname = '%s') as b " \
              "on a.schemaid = b.schemaid;" % (schema, schema)
    elif build_request.query.startswith(SPECIAL_CMDS['VALIDATE']):
      for exprType, cmd in VALIDATE_CMDS.items():
        if build_request.query.startswith(cmd):
          expression = build_request.query[len(cmd):].strip()
          return self.validate_expression(expression, build_request, exprType)

    executor = BuildRequestExecutor(build_request)

    return executor.execute(query=query,
                            async_flag=async_flag)

