# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
from . import arctypes

from .connections_types import DATACONN_DENODO
from .data_api_cmds import SPECIAL_CMDS, VALIDATE_CMDS
from .dataconn_dataengine import BuildRequestExecutor, ConnProperties
from . import dataconn_pg

from vizlib.querylog import QueryLog

class DenodoConnProperties(dataconn_pg.PgConnProperties):
  def __init__(self):
    super(DenodoConnProperties, self).__init__()
    self.date_type = "timestamp"
    self.double_type = "double"
    self.supports_backticks = True
    self.quote_alias = '`'
    self.alias_supports_backticks = True
    self.string_type = "TEXT"
    self.supports_dataextract_source = False
    self.supports_dataextract_target = False

class DenodoConn(dataconn_pg.PgConn):
  def __init__(self, params, connection_id):
    dataconn_pg.PgConn.__init__(self, params, connection_id,
        conn_type=DATACONN_DENODO)

  def execute(self, build_request, async_flag=False):

    query = None
    if build_request.query == SPECIAL_CMDS['SHOW_DATABASES']:
      query = "LIST DATABASES"
    elif build_request.query.startswith(SPECIAL_CMDS['SHOW_TABLES']):
      query = "LIST VIEWS ALL  ON `%s`" % build_request.query.split()[-1]
    elif build_request.query.startswith(SPECIAL_CMDS['VALIDATE']):
      for exprType, cmd in VALIDATE_CMDS.items():
        if build_request.query.startswith(cmd):
          expression = build_request.query[len(cmd):].strip()
          try:
            data = self.validate_expression(expression, build_request, exprType)
          except Exception as e:
            error_message = 'Error validating expression'
            LOGGER.exception(error_message)
            data = {'is_valid': 'unknown', 'error_msg': '%s: %s' % (error_message, str(e))}
          return data

    executor = BuildRequestExecutor(build_request)

    return executor.execute(query=query,
                            async_flag=async_flag)

  def properties(self):
    return DenodoConnProperties()
