# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
from . import dataconn_odbc
from .connections_types import DATACONN_DRILL
from .dataconn_odbc import OdbcConnProperties
from django.conf import settings
from . import arctypes


class DrillConnProperties(OdbcConnProperties):

  TYPEMAP = {
    "<type 'float'>" : arctypes.ARC_DATATYPE_REAL,
    "<type 'int'>"  : arctypes.ARC_DATATYPE_INT,
    "<type 'long'>" : arctypes.ARC_DATATYPE_BIGINT,
    "<type 'bool'>" : arctypes.ARC_DATATYPE_BOOLEAN,
    "<type 'str'>" : arctypes.ARC_DATATYPE_STRING,
    "<type 'datetime.datetime'>" : arctypes.ARC_DATATYPE_TIMESTAMP,
    "<type 'datetime.time'>": arctypes.ARC_DATATYPE_STRING,
    "<type 'datetime.date'>": arctypes.ARC_DATATYPE_STRING
  }

  def __init__(self):
    super(DrillConnProperties, self).__init__()
    self.quote = "`"
    self.quote_end = "`"
    self.quote_alias = "`"
    self.quote_escape_character = "\\"
    self.quote_string_constant = "'"
    self.supports_partitions = True
    self.supports_async = False
    self.supports_backticks = True
    self.supports_order_outside_union = True
    self.supports_limit_outside_union = True
    self.supports_import_data_perm = True
    self.supports_import_data_dbname = 'default'
    self.import_data_insertrows = 500
    self.escape_backslash = True
    self.alias_supports_backticks = True
    self.string_type = "string"
    self.timestamp_type = "timestamp"
    self.date_type = "string"
    self.double_type = "double"
    self.booleantruevalue = 'TRUE'
    self.booleanfalsevalue = 'FALSE'
    self.supports_deriveddata = False
    self.supports_tableproperties = False
    self.supports_nulls_order = True
    self.display_to_table_type = None
    self.supports_limit_outside_union = False
    self.data_import_hdfs_location = '/user/hive/data'
    self.supports_deriveddata = True

  @staticmethod
  def cursor_coltype_to_arctype(native_coltype):
    return DrillConnProperties.TYPEMAP.get(str(native_coltype),
        arctypes.ARC_DATATYPE_STRING)

class DrillConn(dataconn_odbc.OdbcConn):
  def __init__(self, params, connection_id, conn_type=DATACONN_DRILL):
    super(DrillConn, self).__init__(params, connection_id, conn_type=conn_type)

  def properties(self):
    return DrillConnProperties()

  def expression_isAggregate(self, expression, **kwargs):
    # Now see if expression has aggregation
    builder = kwargs.get('builder')
    dataset = kwargs.get('dataset')
    username = kwargs.get('username')
    if dataset is None or builder is None or username is None:
      raise ValueError('builder, dataset and username are required')

    hasaggr_query = builder.expr_hasAggregate(expression, dataset)
    sum1_query = builder.expr_hasAggregate("sum(1)", dataset)
    hasaggr = "false"
    res1 = self.log_sync_runquery(hasaggr_query, username=username).get("rows")
    res2 = self.log_sync_runquery(sum1_query, username=username).get("rows")
    if res1 and res2:
      if len(res1) == 1 and res2[0] > 1: # expression is an aggregate
        hasaggr = "true"
    return hasaggr
