# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import json


class DataconnExtension:
  def external_name(self) -> str:
    raise NotImplementedError()

  def internal_name(self) -> str:
    raise NotImplementedError()

  def ui_config(self) -> dict:
    raise NotImplementedError()

  def create_properties(self):
    '''
    Return a subclass of dataconn.dataconn_dataengine.ConnProperties
    '''
    raise NotImplementedError()

  def create_connection(self, params: dict, connection_id: int):
    '''
    Return a subclass of -> dataconn.dataconn_dbapi2.DbApi2Conn
    '''
    raise NotImplementedError()


def connection_names():
  from django.conf import settings
  DATACONN_EXTENSIONS: list[DataconnExtension] = settings.DATACONN_EXTENSIONS

  return {ec.internal_name(): ec.external_name() for ec in DATACONN_EXTENSIONS}


def connection_types():
  from django.conf import settings
  DATACONN_EXTENSIONS: list[DataconnExtension] = settings.DATACONN_EXTENSIONS

  return [ec.internal_name() for ec in DATACONN_EXTENSIONS]


def create_properties(connection_type: str):
  from django.conf import settings
  DATACONN_EXTENSIONS: list[DataconnExtension] = settings.DATACONN_EXTENSIONS

  for ec in DATACONN_EXTENSIONS:
    if ec.internal_name() == connection_type:
      return ec.create_properties()

  return None


def create_connection(connection_type: str, params: dict, connection_id: int):
  from django.conf import settings
  DATACONN_EXTENSIONS: list[DataconnExtension] = settings.DATACONN_EXTENSIONS

  for ec in DATACONN_EXTENSIONS:
    if ec.internal_name() == connection_type:
      return ec.create_connection(params, connection_id)

  return None

def ui_configs():
  from django.conf import settings
  DATACONN_EXTENSIONS: list[DataconnExtension] = settings.DATACONN_EXTENSIONS

  configs = []
  for ec in DATACONN_EXTENSIONS:
    config = {
      'name': ec.internal_name(),
      'conf': ec.ui_config(),
    }
    configs.append(json.dumps(config))

  return configs
