# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from dataconn.ext.main import DataconnExtension

TIMESTAMP_RANGE_TEMPLATE=r'%HTML_colname% between TO_TIMESTAMP(%HTML_quotedStart%) and TO_TIMESTAMP(%HTML_quotedEnd%)'

class PhoenixExtension(DataconnExtension):

  def external_name(self) -> str:
    return 'Phoenix (Tech Preview)'

  def internal_name(self) -> str:
    return 'phoenix'

  def create_properties(self) -> dict:
    from dataconn.ext.phoenix.phoenix import PhoenixConnProperties
    return PhoenixConnProperties()

  def create_connection(self, params: dict, connection_id: int) -> dict:
    from dataconn.ext.phoenix.phoenix import PhoenixDataConn
    return PhoenixDataConn(params, connection_id)

  def ui_config(self) -> dict:
    props = self.create_properties()
    config = {
      'base': 'postgres',
      'overrides': {
        'default_port': 8765,
        'cast_string': 'varchar',
        'timestampRangeExprTemplate': TIMESTAMP_RANGE_TEMPLATE
      },
      'has_advanced_settings': True,
      'has_http_connection_mode': True,
      'has_concurrency_settings': True,
      'has_impersonation': True,
      'is_direct_accessible': True,
      'is_extract_source': props.supports_dataextract_source,
      'is_extract_target': props.supports_dataextract_target,
    }
    return config
