# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from vizlib.tests.test_build_sql import TestBuild_sql

class Test_BuildSql_Phoenix(TestBuild_sql):

  def __init__(self, *args, **kwargs):
    self.dataconnection_type = 'phoenix'
    super(Test_BuildSql_Phoenix, self).__init__(*args, **kwargs)

    self.ext_accepted_errors={}

    self.ext_outputs['expr_hasaggr'] = "select 'a'"


    self.ext_outputs['select_from_dataset_cast1'] = """
      SELECT (CAST(TA_0."year" as VARCHAR)) as "year", TA_0."state" as "state", TA_0."population" as "population"
        FROM "default"."census_pop"  TA_0
        LIMIT 100
    """
    # INTEGER can not be cast to VARCHAR in Phoenix:
    self.ext_accepted_errors['select_from_dataset_cast1'] = 'Type mismatch. BIGINT and VARCHAR for "year"'

    self.ext_outputs['select_from_dataset_cast2'] = """
      SELECT (CAST(TA_0."year" as VARCHAR)) as "year", TA_0."state" as "state", sum((TA_0."population")) as "sum(population)"
        FROM "default"."census_pop"  TA_0
        GROUP BY (CAST(TA_0."year" as VARCHAR)), TA_0."state"
        LIMIT 100
    """
    self.ext_outputs['no_dims_no_aggrs1'] = """
      SELECT TA_0."year" as "year", TA_0."state" as "state", TA_0."population" as "population"
        FROM "default"."census_pop"  TA_0
        LIMIT 100
    """
    self.ext_outputs['no_dims_no_aggrs2'] = """
      SELECT TA_0."year" as "year", TA_0."state" as "state", TA_0."population" as "population"
        FROM "default"."census_pop"  TA_0
        WHERE (((TA_0."state") in ("CA")))
        LIMIT 100
    """
    self.ext_outputs['segmentShelfNoDataWglobalSegs'] = """
      SELECT * from (SELECT 'all' as Segment, TA_0."year" as "year", TA_0."state" as "state", avg((TA_0."population")) as "avg(population)"
        FROM "default"."census_pop"  TA_0
        WHERE ((((TA_0."state") in ("CA")) OR ((TA_0."state") in ("DC", "DE", "FL", "GA"))))
        GROUP BY TA_0."year", TA_0."state") TA
        LIMIT 100000
    """

  # TODO: this function has to be deleted as soon as all test cases are validated
  # it is the super() implementation that should perform the testing
  def test_build_sql(self):
    if not hasattr(self, 'dataconnection_type'):
      return

    # TODO: check aggregates validation
    # self.expr_hasaggr()
    self.select_from_dataset_cast1()
    self.select_from_dataset_cast2()
    self.select_from_lv()
    self.no_dims_no_aggrs1()
    self.no_dims_no_aggrs2()
    self.segmentShelfNoDataWglobalSegs()
    # TODO move this line down as test cases are validated
    """
    self.segmentsUnionAll()
    self.simple_dims_and_aggrs()
    self.nulls_order_first()
    self.nulls_order_last()
    self.entity_year_pop_forCA_popL10M()
    self.ca_global_seg()
    self.ca_global_seg_neg()
    self.ca_global_entityseg_neg_withfilter()
    self.global_segs_samegroup()
    self.global_segsdifgroup()
    self.global_segs_samegroup_error()
    self.multitable_nodimsnoaggrs()
    self.multitable_nofactselection1()
    self.multitable_nofactselection2()
    self.multitable_nofactselection3()
    self.multitable_nofactselection4()
    self.inapp_purchase_flow1()
    self.inapp_purchase_flow2()
    self.inapp_purchase_flow3()
    self.inapp_purchase_funnel1()
    self.SlideUpFunnel()
    """

"""
Code below is to meant to be used as a script (not from unit tests):
cd arcweb

DJANGO_SETTINGS_MODULE=arcwebbase.settings_tests python manage.py shell -c 'from dataconn.ext.tests.test_phoenix_build_sql import main; main()'

"""
import phoenixdb
import phoenixdb.cursor

import logging
LOG = logging.getLogger(__name__)

def main():
  """
  Execute queries.
  """
  logging.basicConfig(level=logging.INFO)
  import urllib3
  urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

  url = 'https://viz-team-1.cc-os.cloudera.com:8443/gateway/cdp-proxy-api/avatica'
  verify = False
  user = 'knoxui'
  password = 'knoxui'
  authentication = 'BASIC'
  conn = phoenixdb.connect(
    url=url,
    verify=verify,
    autocommit=True,
    authentication=authentication,
    user=user,
    password=password,
  )

  with conn:
    create_tables(conn)
    execute_test_queries(conn)

def create_tables(conn: phoenixdb.Connection):

  queries = list[str]()
  # create tables:
  create_census_pop = """
    create table if not exists "census_pop" ("pk" VARCHAR PRIMARY KEY, "year" INTEGER, "state" VARCHAR, "population" INTEGER)
  """
  # although "year" is meant to be an INTEGER, the casts to VARCHAR will not work:
  create_census_pop = """
    create table if not exists "census_pop" ("pk" VARCHAR PRIMARY KEY, "year" VARCHAR, "state" VARCHAR, "population" INTEGER)
  """
  queries.append(create_census_pop)

  for query in queries:
    execute(conn, query)

def execute(conn: phoenixdb.Connection, sql: str):
  cursor = conn.cursor()
  logging.info('executing %s', sql)
  response = cursor.execute(sql)
  logging.info('response = %s', str(response))

def fix_namespace(sql: str):
  """
  The "default" namespace in HBase is the schema with no name in Phoenix.
  Have no idea how to create a schema called "default" in Phoenix.
  """
  return sql.replace('"default"."', '""."')

def execute_test_queries(conn: phoenixdb.Connection):
  test = Test_BuildSql_Phoenix()
  failures = 0
  accepted_errors = 0

  for test_function, sql in test.ext_outputs.items():
    print()
    LOG.info('test_function: %s', test_function)
    accepted_error = test.ext_accepted_errors.get(test_function)

    try:
      execute(conn, fix_namespace(sql))
    except Exception as e:
      msg = str(e)
      if accepted_error and accepted_error in msg:
        accepted_errors += 1
        LOG.warn('test_function [%s], accepted error: %s', test_function, accepted_error)
      else:
        LOG.error(e)
        failures += 1

  LOG.info('failures = %d', failures)

if __name__ == '__main__':
  main()
