# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import django.test
from unittest import mock
from dataconn.ext import main
from dataconn.dataconn_dbapi2 import DbApi2Conn

from dataconn.ext.phoenix.phoenix_ext import PhoenixExtension
from dataconn.ext.phoenix.phoenix import PhoenixConnProperties, PhoenixDataConn


class TestPhoenixExt(django.test.TestCase):

  @django.test.override_settings(DATACONN_EXTENSIONS=[PhoenixExtension()])
  def test_extension(self):
    self.assertEqual(main.connection_types(), ['phoenix'])
    self.assertEqual(main.connection_names(), {'phoenix': 'Phoenix (Tech Preview)'})

    self.assertIsInstance(main.create_properties(
      'phoenix'), PhoenixConnProperties)

  @django.test.override_settings(DATACONN_EXTENSIONS=[PhoenixExtension()])
  @mock.patch.object(PhoenixDataConn, 'dbapi2_module')
  def test_connection_creation(self, dbapi2_module: mock.Mock):
    conn: DbApi2Conn = main.create_connection('phoenix', {}, 10)
    self.assertIsNotNone(conn.get_connection())
