# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import django.test
import json
from unittest import mock

from dataconn.ext import base
from dataconn.dataconn_dataengine import BuildRequest

from vizlib.dataset_request import DatasetRequest

connection_mock = mock.MagicMock()
show_databases = mock.MagicMock()
show_tables = mock.MagicMock()


class MyProps(base.ExtensionConnProperties):
  def get_typemap(self) -> dict:
    return {'a': 'b'}


class MyConn(base.ExtensionDataConn):
  def properties(self):
    return MyProps()

  def create_connection(self, params: dict, username: str = None):
    return connection_mock()

  def show_databases(self, build_request):
    return show_databases()

  def show_tables(self, build_request, dbname):
    return show_tables()


class TestBase(django.test.TestCase):

  def setUp(self):
    connection_mock.reset_mock()
    show_databases.reset_mock()
    show_tables.reset_mock()

  @mock.patch.object(base, 'BuildRequestExecutor')
  def test_base(self, executor_mock: mock.Mock):
    conn = MyConn(dict(), 10, 'my-conn')
    # the underlying connection is created lazily.
    dbapi2_conn = conn.get_connection()
    connection_mock.assert_called_once()
    self.assertIsNotNone(dbapi2_conn)

  @mock.patch.object(base, 'BuildRequestExecutor')
  def test_show_databases(self, executor_mock: mock.Mock):
    conn = MyConn(dict(), 10, 'my-conn')
    self.assertIsNotNone(conn.execute(BuildRequest(query='.databases')))
    show_databases.assert_called_once()

  @mock.patch.object(base, 'BuildRequestExecutor')
  def test_show_tables(self, executor_mock: mock.Mock):
    conn = MyConn(dict(), 10, 'my-conn')

    ds_req = {
      'type': '.tables',
      'dbname': 'SYSTEM'
    }
    ds_req_obj = DatasetRequest(json.dumps(ds_req))
    br = BuildRequest(connection=conn, dataset_request=ds_req_obj)
    self.assertIsNotNone(conn.execute(br))
    show_tables.assert_called_once()
