# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import django.test
import json
from dataconn.ext import main


class MyExtension(main.DataconnExtension):
  def internal_name(self) -> str:
    return 'my-conn'

  def external_name(self) -> str:
    return 'My Conn'

  def create_properties(self):
    return 'my-props'

  def create_connection(self, params: dict, connection_id: int):
    return 'my-connection'

  def ui_config(self) -> dict:
    return {'base': 'postgres'}


class TestMain(django.test.TestCase):

  @django.test.override_settings(DATACONN_EXTENSIONS=[MyExtension()])
  def test_extensions(self):
    self.assertEqual(main.connection_types(), ['my-conn'])
    self.assertEqual(main.connection_names(), {'my-conn': 'My Conn'})
    self.assertEqual(main.create_properties('my-conn'), 'my-props')
    self.assertIsNone(main.create_properties('a'))
    self.assertEqual(main.create_connection(
      'my-conn', {}, 10), 'my-connection')
    self.assertIsNone(main.create_connection('b', {}, 10))

    configs = [json.loads(c) for c in main.ui_configs()]
    self.assertDictEqual(configs[0], {
      'name': 'my-conn',
      'conf': {
        'base': 'postgres'
      }
    })
