# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from django.test import TestCase, override_settings
from unittest.mock import patch, Mock
from dataconn.ext import main
from dataconn.dataconn_dbapi2 import DbApi2Conn
from dataconn.dataconn_dataengine import BuildRequest
from vizlib.dataset_request import DatasetRequest
from json import dumps

from dataconn.ext.trino.trino_ext import TrinoExtension


class TestTrino(TestCase):

    @override_settings(DATACONN_EXTENSIONS=[TrinoExtension()])
    @patch('dataconn.ext.base.BuildRequestExecutor')
    def test_when_show_database_should_return_all_catalogs_with_schemas(self, executor_mock: Mock):
        # Given
        conn: DbApi2Conn = main.create_connection('trino', {}, 10)
        executor_mock.return_value.execute.side_effect = [
            {'rows': dumps([['hive'], ['system']])},
            {'rows': dumps([['db1'], ['db2']])},
        ]
        # When
        res = conn.execute(BuildRequest(query='.databases'))
        # Then
        self.assertEqual(res['rows'], '[["hive.db1"], ["hive.db2"]]')

    @override_settings(DATACONN_EXTENSIONS=[TrinoExtension()])
    @patch('dataconn.ext.base.BuildRequestExecutor')
    def test_when_show_tables_should_return(self, executor_mock: Mock):
        # Given
        conn: DbApi2Conn = main.create_connection('trino', {}, 10)
        executor_mock.return_value.execute.return_value = [
            {'rows': dumps([['hive'], ['system']])},
        ]
        ds_req = {
            'type': '.tables',
            'dbname': 'hive.db1'
        }
        # When
        res = conn.execute(BuildRequest(connection=conn, dataset_request=DatasetRequest(dumps(ds_req))))
        # Then
        self.assertIsNotNone(res)
        executor_mock.return_value.execute.assert_called_once()
        args = executor_mock.return_value.execute.call_args_list[0].kwargs
        expected_query = "SHOW TABLES FROM hive.db1"
        self.assertEqual(args['query'], expected_query)
