# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# Simple tests for hs2conn module

import sys
import time
import json
import random
import threading
import signal
import traceback
import itertools

import logging

logging.basicConfig(filename='hs2test.log', filemode='w',
                    format='%(asctime)s %(thread)d %(levelname)s %(message)s',
                    level=logging.DEBUG)

from .dataconn_consts import DCS
from .connections_types import DATACONN_ARCENGINE

sys.path.append('../../arcweb')
from . import hs2conn

TEST_QUERY = """
select _T0.year as 'year', sum((_T0.top_10pct_income_share)) as 'sum(top_10pct_income_share)', sum((_T0.top_5pct_income_share)) as 'sum(top_5pct_income_share)', sum((_T0.top_0_5pct_income_share)) as 'sum(top_0_5pct_income_share)', sum((_T0.top_1pct_income_share)) as 'sum(top_1pct_income_share)' FROM default.world_top_income _T0 WHERE _T0.country in ("United States") GROUP BY 1 ORDER BY 1 ASC LIMIT 100
"""

TEST_QUERY = """
select TA_0.status as `status`, sum(1) as `sum(1)` FROM xademo.customer_details TA_0 GROUP BY `status` LIMIT 100;
"""

SLOW_QUERY = "select p1,p2,count(distinct s1),max(s1) from testasync_csv group by p1, p2"
SLOW_QUERY = "select p1,count(distinct s1),max(s1) from testasync_csv group by p1"

# This one works well when testasync_csv is 1000 rows
SLOW_QUERY = "select t1.p1, count(distinct t1.s1),max(t1.s1) from testasync_csv t1 join testasync_csv t2 group by 1"
FAIL_QUERY = "select p1,p2,cast (rownum / 10 as int),count(distinct s1),max(s1) from testasync_csv group by p1,p2,cast (rownum / 10 as int)"
NDV_QUERY = "select count(distinct value) from grouptest"

# These two are only valid for arcengine/impala connections
FAIL_QUERY = None
#NDV_QUERY = None

# Example kerberos ticket renewal
if False:
  from .thriftconn import kt_renewer
  principal = 'shant@EC2.INTERNAL'
  keytab = '/home/ec2-user/doug/test.keytab'
  frequency = 5
  kt_renewer(principal, keytab, frequency)

TIMEOUT=10
hs2 = hs2conn.HS2Pool('localhost', 21050, max_connections=2, max_reqperconn=3, max_requests=6,
                      username='hs2test', conntype=DATACONN_ARCENGINE, timeout=TIMEOUT)
#hs2 = hs2conn.HS2Pool('curly', 21050, max_connections=1, max_reqperconn=2, max_requests=6,
#                      ssl=True, sslcert='/Users/doug/arcdev/certlist.txt')
#hs2 = hs2conn.HS2Pool('10.0.0.202', 21050, max_connections=4, max_reqperconn=10, max_requests=100)
#hs2 = hs2conn.HS2Pool('localhost', 10000, auth='plain', timeout=TIMEOUT,
#                     settings={"hive.server2.long.polling.timeout":"0ms"},
#                     max_connections=2, max_requests=10, max_reqperconn=1, conn_timeout=15)
print("Got pool")

def handle_signal(unused_signal, unused_frame):
  global hs2
  if hs2:
    hs2.close()
    hs2 = None
  exit(-1)

signal.signal(signal.SIGINT, handle_signal)

DCS = hs2conn.DCS

try:

  if False:
    # Test some unicode values in queries
    stmt = """
              insert into utftest values
              ('s1', 1),
              ('aa\xc2\xa3bb', 100),
              ('s2', 2)
            """
    print("stmt: ", stmt)
    print("Type stmt: ", type(stmt))
    hs2.run("drop table if exists utftest")
    hs2.run("create table utftest (s1 string, c1 int) stored as parquet")
    hs2.run(stmt)

    res = hs2.run('select s1 from utftest where c1=100')
    ans = res[2][0][0]
    print("ans: ", ans)
    print("Type ans: ", type(ans))
    print("len ans: ", len(ans))

    res = hs2.run('select * from utftest where s1 in ("aa\xc2\xa3bb","s1")')
    print("res = %s" % str(res))
    print("Done")

    exit(0);

# query = NDV_QUERY
# query = open('/Users/doug/test/temp.sql','r').read()
# query = query.strip(';\n')
  query = None
  if query:
    print("Query: \n%s" % query)
    print("Issue test query")
    res = hs2.run(query, doasuser='doug')
    print(",".join(["%s %s" % (n, t) for (n, t) in zip(res[0], res[1])]))
    for r in res[2]:
      print(",".join([str(c) for c in r]))

    colnames, coltypes, rows, info = res
    schema = ["%s %s" % (colname, coltype) \
        for (colname, coltype) in zip(colnames, coltypes)]
    data = {'colnames':colnames, 'coltypes': coltypes, 'info':info, 'rows':rows}
    print("JSON:\n%s" % json.dumps(data))
    exit(0)

  # This expects 2 connections with at least 3 commands per connection so that
  # we can be sure of request emit order
  query = "select a,b,sum(1) from grouptest group by 1,2"
  query = None
  names = ['doug','doug','veronica','sean','alan','sean','doug','veronica']
  order = [1, 2, 3, 6, 8, 7, 4, 5]
  if query:
    results = []
    handles = []

    # Emit all the requests
    for i,n in enumerate(names):
      hnd = hs2.async_issue(query + (" limit %d" % (i + 4)), doasuser=n)
      handles.append(hnd)
      results.append(None)

    # Poll handles until all are complete
    while sum(h != None for h in handles):
      time.sleep(0.5)
      for i,h in enumerate(handles):
        if h is not None:
          s = hs2.async_status(h)
          if s.status == DCS.FINISHED:
            results[i] = hs2.async_results(h)
            handles[i] = None

    # Now ensure commands got emitted in the expected order
    emit = []
    key = "Sequence:"
    for i,r in enumerate(results):
      for t in r[3]:
        if t.startswith(key):
          seq = int(t[len(key):])
      emit.append(seq)

    actual = []
    for i in range(0, len(emit)):
      actual.append(emit[i] - emit[0] + 1)

    if actual != order:
      raise Exception

    print("Actual == Expected order: (%s) == (%s)" % (str(actual), str(order)))

  print("Verify simple queries")
  res = hs2.run("select * from grouptest limit 5")
  res = hs2.run("show tables")
  res = hs2.run("describe grouptest")

  if False:
    print("Check progress")
    hnd = hs2.async_issue(SLOW_QUERY)
    status = hs2.async_status(hnd)
    while status.status != DCS.FINISHED:
      print("Message: %s" % status.message)
      time.sleep(.25)
      status = hs2.async_status(hnd)
    res = hs2.async_results(hnd)
    print("Done")
    exit(0)

  print("Run some synchronous queries")
  for i in range(0, 10):
    res = hs2.run("select * from grouptest limit %d" % i)
    print("Got result: %d rows of %d columns" % (len(res[2]), len(res[0])))

  print("Verify excepion on sync issue invalid query")
  query = "select a from missing_table"
  exc = None
  try:
    res = hs2.run(query)
  except Exception as e:
    exc = e
  if not exc:
    raise Exception("Did not get expected exception")
  print("Got exception: %s" % exc)

  print("Verify excepion on async issue invalid query")
  exc = None
  try:
    res = hs2.async_issue(query)
  except Exception as e:
    exc = e
  if not exc:
    raise Exception("Did not get expected exception")
  print("Got exception: %s" % exc)

  print("Run multi-threaded async simple queries")
  rnd = random.Random(1)
  thlist = []

  def onequery(cnt, exclist):
    try:
      query = "select * from grouptest limit %d" % cnt
      print("Issue: %s" % query)
      hnd = hs2.async_issue(query)
      done = False
      while not done:
        status = hs2.async_status(hnd)
        if DCS.active(status.status):
          time.sleep(.5 + .001 * cnt)
        elif status.status == DCS.FINISHED:
          done = True
        else:
          raise Exception("Got unexpected status: %s" % status.status)
      res = hs2.async_results(hnd)
      if len(res[2]) != cnt:
        raise Exception("Expected %d rows, got %d" % (cnt, len(res[2])))
      print("Verified %d rows recieved" % cnt)
    except Exception as e:
      exclist.append(e)

  exclist = []
  for i in range(0, 10):
    cnt = rnd.randint(10, 500)
    th = threading.Thread(target=onequery, kwargs={"cnt":cnt, "exclist":exclist})
    thlist.append(th)

  for th in thlist:
    th.start()
  for th in thlist:
    th.join(15)
    if th.isAlive():
      raise Exception("Timeout waiting for thread completion")
  print("All threads finished")
  if len(exclist):
    raise Exception("Exception from one or more threads:\n  %s" % "\n  ".join(str(e) for e in exclist))

  print("Verify fast request is cleaned up on timeout")
  hnd = hs2.async_issue("show tables")
  status = hs2.async_status(hnd)
  time.sleep(TIMEOUT + 5)
  exc = None
  try:
    status = hs2.async_status(hnd)
  except Exception as e:
    exc = e
  if not exc:
    raise Exception("Expected handle to have gone away upon timeout")

  print("Verify slow request is cleaned up on timeout")
  hnd = hs2.async_issue(SLOW_QUERY)
  status = hs2.async_status(hnd)
  time.sleep(TIMEOUT + 5)
  exc = None
  try:
    status = hs2.async_status(hnd)
  except Exception as e:
    exc = e
  if not exc:
    raise Exception("Expected handle to have gone away upon timeout")

  print("Verify agg works after abandoned query")
  res = hs2.run("select a,max(value) from grouptest group by a")

  query = FAIL_QUERY
  if query:
    print("Verify excepion on sync issue out of memory query")
    exc = None
    try:
      res = hs2.run(query, settings={'MEM_LIMIT':'600000000'})
    except Exception as e:
      exc = e
    if not exc:
      raise Exception("Did not get expected exception")
    print("Got exception: %s" % exc)

    print("Verify eventual error on async out of memory query")
    hnd = hs2.async_issue(query, settings={'MEM_LIMIT':'600000000'})
    res = None
    done = False
    while not done:
      res = hs2.async_status(hnd)
      if DCS.active(res.status):
        time.sleep(1)
      else:
        done = True
    if res.status == DCS.ERROR:
      print("Got error: %s" % res.message)
    else:
      exc = None
      try:
        res = hs2.async_results(hnd)
      except Exception as e:
        exc = e
      if not exc:
        raise Exception("Did not get expected exception")
      print("Got exception: %s" % exc)

  query = NDV_QUERY
  if query:
    print("Verify input configuration settings are picked up")
    res1 = hs2.run(query, settings={"APPX_COUNT_DISTINCT":"True"})
    res2 = hs2.run(query, settings={"APPX_COUNT_DISTINCT":"False"})
    res3 = hs2.run(query, settings=None)
    if res1[2][0][0] == res2[2][0][0]:
      raise Exception("Expected different results from query %s" % query)
    if res3[2][0][0] != res1[2][0][0] and res3[2][0][0] != res2[2][0][0]:
      raise Exception("Expected one result set to match for query %s" % query)

  print("Run some synchronous queries")
  for i in range(0, 10):
    res = hs2.run("select * from grouptest limit %d" % i)
    print("Got result: %d rows of %d columns" % (len(res[2]), len(res[0])))

  query = SLOW_QUERY
  if query:
    print("Ensure we can cancel queries")
    for i in range(0, 4):
      hnd = hs2.async_issue(query)
      print("Got handle...")
      for i in range(0,2):
        time.sleep(.25)
        print("Check status...")
        status = hs2.async_status(hnd)
        print("Got status: %s" % status.status)
        if not DCS.active(status.status):
          raise Exception("Got unexpected status: %s" % status.status)
      print("Doing cancel...")
      hs2.async_cancel(hnd)
      exc = None
      try:
        status = hs2.async_status(hnd)
      except Exception as e:
        exc = e
      if not exc:
        raise Exception("Expected invalid handle on status check, not: %s" % status.status)
      print("Worked!")
    print("Now ensure query runs to completion")
    hs2.run(query)

  print("Run multi-threaded async queries with agg")
  rnd = random.Random(1)
  thlist = []

  def oneagg(query, exclist):
    try:
      print("Issue: %s" % query)
      hnd = hs2.async_issue(query)
      done = False
      while not done:
        status = hs2.async_status(hnd)
        if DCS.active(status.status):
          time.sleep(.001 * rnd.randint(750, 1000))
        elif status.status == DCS.FINISHED:
          res = hs2.async_results(hnd)
          done = True
        else:
          raise Exception("Got unexpected status: %s" % status.status)
    except Exception as e:
      exclist.append(e)

  exclist = []
  for i in range(0, 5):
    query = ("select a,sum(value) from grouptest where value > %d group by a" %
             (i*1000))
    th = threading.Thread(target=oneagg, kwargs={"query":query, "exclist":exclist})
    thlist.append(th)

  for th in thlist:
    th.start()
  for th in thlist:
    th.join(600)
    if th.isAlive():
      raise Exception("Timeout waiting for thread completion")

  print("All threads finished")
  if len(exclist):
    raise Exception("Exception from one or more threads:\n  %s" % "\n  ".join(str(e) for e in exclist))

  print("Ensure queries work after restart")
  input("Now reset engine and hit enter when ready...")
  for i in range(0, 10):
    res = hs2.run("select * from grouptest limit %d" % i)
    print("Got result: %d rows of %d columns" % (len(res[2]), len(res[0])))

except Exception as e:
  print("Exception: %s" % str(e))
  print("%s" % traceback.format_exc())
finally:
  if hs2:
    hs2.close()
    hs2 = None

