# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
  Query control class for arcviz request execution

  The goal is a single place for linking together all of the aspects
  that go into a query/data-api request

  Most requests are initiated from the arcviz browser client and correspond
  to a visualization data request.

  Others are indirectly initiated by the front end:
    Create/remove/update a logical view for a dataset
    Dataset expression validation
    Dashboard acceleration, other analytical view functions

  Other requests can have no front end components:
    Requests from back end utility programs
    Periodic requests from scheduled jobs

  Rather than passing an ever growing list of parameters to the various drivers,
  especially in the arcengine case where we have lots of extended functionality,
  we can pass a single object that links together all elements of a request:
    User
    Request
    Cache control
    Settings/flags
    Dataset
    data-api request (dsreq)
"""

class QueryControl:

  def __init__(self):
    self.fa_expr = None
    self.fa_engine = False
    self.fa_tables = None
    self.ds_tables = []
    self.fallback_dcid = None
    self.system_functions_allowed = True
    self.tables_text = None
    self.exec_method = None
