# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import sys
import io
import math
from util.json_utils import json_dump

from util.perfcounters import APPROX_COUNTERS
rowdata_count = APPROX_COUNTERS.get('rowdata.count')

class RowsData(object):
  def __init__(self, maxSize):
    self._str = io.StringIO()
    self._str.write("[")
    self._rowCount = 0
    self._pos = None
    self._maxSize = maxSize
    self._query_warnings = {'inf':[], 'nan':[]}

  def rowCount(self):
    return self._rowCount

  def addRow(self, row):
    for i, value in enumerate(row):
      if isinstance(value,float) and not math.isfinite(value):
        self._query_warnings[str(value)].append({'row_idx':self._rowCount, 'column_idx': i })
        row[i] = None

    global rowdata_count

    if self._rowCount > 0:
      self._str.write(",")

    self._rowCount += 1
    p1 = self._str.tell()
    json_dump(row, self._str)
    p2 = self._str.tell()
    rowdata_count += (p2 - p1)

    if self._str.tell() > self._maxSize:
      raise Exception("The resultset being returned is above the max permitted " +
          "on this system. You can change the allowed limit in the advanced settings")

  def __len__(self):
    # Return lengths on closed StringIO objects also
    if self._pos is not None:
      return self._pos

    return self._str.tell()

  def fetchAndClose(self):
    self._str.write("]")
    self._pos = self._str.tell()
    v = self._str.getvalue()
    self._str.close()
    return v

  def getQueryWarnings(self):
   return self._query_warnings

