#!python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import snowflake.connector as dbapi2

import os
import sys

def runcmd(conn, query):
  cursor = conn.cursor()
  cursor.execute(query)
  if cursor.description:
    # name, type_code, display_size, internal_size, precision, scale, null_ok
    # pyodbc only provides values for name, type_code, internal_size, and null_ok
    colnames = [x[0] for x in cursor.description]
    coltypes = [x[1] or 'STRING' for x in cursor.description]

    print("description", cursor.description)
    print("colnames", colnames)
    print("coltypes", coltypes)

  print("rows", cursor.fetchall())
  
def main():
  conn_params = {}

  conn_params["account"] = os.getenv('SNOWFLAKE_ACCOUNT')
  conn_params["user"] = os.getenv('SNOWFLAKE_USERNAME')
  conn_params["password"] = os.getenv('SNOWFLAKE_PASSWORD')
  conn_params["database"] = os.getenv('SNOWFLAKE_DATABASE')

  conn = dbapi2.connect(**conn_params)
  for query in sys.argv[1:]:
    runcmd(conn, query)

if __name__ == "__main__":
  main()

