/*
 * Re-implements the StandardLuceneGrammar for Solr as an Antlr4 grammar by
 * referencing
 * https://github.com/romanchyla/montysolr/blob/master/contrib/antlrqueryparser/grammars/StandardLuceneGrammar.g
 * which in turn reimplements from
 * http://lucene.apache.org/core/4_3_0/queryparser/index.html
 */

grammar SolrGrammar;

tokens {
  OPERATOR
}

mainQ:
  (clauseOr)+
  ;

clauseOr:
  clauseAnd (orclause clauseAnd)*
  ;

clauseAnd:
  clauseNot (andclause clauseNot)*
  ;

clauseNot:
  clauseBasic (notclause clauseBasic)*
  ;

clauseBasic
  :
  modifier? LPAREN clauseOr+ RPAREN term_modifier?
  | LPAREN clauseOr+ RPAREN
  | atom
  ;

/*
 * TODO: This part of the grammar has not been specified (not copied from the
 * Antlr3 version).
 * Need to make sure its needed and not already handled by recursion
 * The example given for multi-clause seems to work already:
 * (a b AND c OR d OR e)
atom
  :
  modifier? field multi_value term_modifier?
  | modifier? field? value term_modifier?
  ;
*/

atom
  :
  modifier? field? value term_modifier?
  ;

field
  :
  TERM_NORMAL COLON
  ;

COMMENT
  : '/*' .*? '*/' -> skip // match anything between /* and */
  ;

value
  :
  range_term_in
  | range_term_ex
  | normal
  | truncated
  | quoted
  | quoted_truncated
  | QMARK
  | STAR COLON b=STAR
  | STAR
  ;

range_term_in
  :
   LBRACK
   (range_value)
   (TO? range_value)?
   RBRACK
  ;

range_term_ex
  :
   LCURLY
   (range_value)
   (TO? range_value)?
   RCURLY
  ;

range_value
  :
  truncated
  | quoted
  | quoted_truncated
  | date
  | normal
  | STAR
  ;

normal
  :
  TERM_NORMAL
  | NUMBER
  ;

truncated
  :
  TERM_TRUNCATED
  ;

quoted_truncated
  :
  PHRASE_ANYTHING
  ;

quoted
  :
  PHRASE
  ;

modifier
  :
  PLUS
  | MINUS
  ;

term_modifier
  :
  TILDE CARAT?
  | CARAT TILDE?
  ;

boost
  :
  (CARAT) // set the default value
  (NUMBER)? //replace the default with user input
  ;

fuzzy
  :
  (TILDE) // set the default value
  (NUMBER)? //replace the default with user input
  ;

notclause
  :
  AND NOT
  | NOT
  ;

andclause
  :
  AND
  ;

orclause
  :
  OR
  ;

date  :
  DATE_TOKEN
  ;

LPAREN  : '(';
RPAREN  : ')';
LBRACK  : '[';
RBRACK  : ']';
COLON   : ':' ;  //this must NOT be fragment
PLUS  : '+' ;
MINUS : ('-'|'!');
STAR  : '*' ;
QMARK  : '?'+ ;
fragment VBAR  : '|' ;
fragment AMPER : '&' ;
LCURLY  : '{' ;
RCURLY  : '}' ;
CARAT : '^' (INT+ ('.' INT+)?)?;
TILDE : '~' (INT+ ('.' INT+)?)?;
DQUOTE
  : '"';
SQUOTE
  : '\'';

TO: 'TO';

/* We want to be case insensitive */
AND   : (('a' | 'A') ('n' | 'N') ('d' | 'D') | (AMPER AMPER?)) ;
OR  : (('o' | 'O') ('r' | 'R') | (VBAR VBAR?));
NOT   : ('n' | 'N') ('o' | 'O') ('t' | 'T');
WS  :   ( ' '
        | '\t'
        | '\r'
        | '\n'
        | '\u3000'
        )
        -> skip
    ;
fragment INT: '0' .. '9';
fragment ESC_CHAR:  '\\' .;
fragment TERM_START_CHAR
  :
  (~(' ' | '\t' | '\n' | '\r' | '\u3000'
        | '\'' | '"'
        | '(' | ')' | '[' | ']' | '{' | '}'
        | '+' | '-' | '!' | ':' | '~' | '^'
        | '?' | '*' | '\\'
        )
   | ESC_CHAR );
fragment TERM_CHAR
  :
  (TERM_START_CHAR | '-' | '+')
  ;
NUMBER
  :
  INT+ ('.' INT+)?
  ;
DATE_TOKEN
  :
  INT INT? ('/'|'-'|'.') INT INT? ('/'|'-'|'.') INT INT (INT INT)?
  ;
TERM_NORMAL
  :
  TERM_START_CHAR ( TERM_CHAR )*
  ;
TERM_TRUNCATED
  :
  (STAR|QMARK) (TERM_CHAR+ (QMARK|STAR))+ (TERM_CHAR)*
  | TERM_START_CHAR (TERM_CHAR* (QMARK|STAR))+ (TERM_CHAR)*
  | (STAR|QMARK) TERM_CHAR+
  ;
PHRASE
  :
  DQUOTE (ESC_CHAR|~('"'|'\\'|'?'|'*'))+ DQUOTE
  ;
PHRASE_ANYTHING
  :
  DQUOTE (ESC_CHAR|~('"'|'\\'))+ DQUOTE
  ;
