# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import sys
import pdb

import logging
LOGGER = logging.getLogger("arcweb")

try:
  from antlr4 import *
  from .SolrGrammarLexer import SolrGrammarLexer
  from .SolrGrammarListener import SolrGrammarListener
  from .SolrGrammarParser import SolrGrammarParser
except Exception as e:
  LOGGER.info("Libraries missing for ANTLR4 (SOLR connection)")

# This file is based on the example given at
# https://dzone.com/articles/using-antlr-4-with-python-2

# 1. Grammar has been specified in SolrGrammar.g4 manually
# 2. antlr4 was installed by running these steps from www.antlr4.org:
  # $ cd /usr/local/lib
  # $ sudo curl -O http://www.antlr.org/download/antlr-4.7-complete.jar
  # $ export CLASSPATH=".:/usr/local/lib/antlr-4.7-complete.jar:$CLASSPATH"
  # $ alias antlr4='java -jar /usr/local/lib/antlr-4.7-complete.jar'
# 3. pip install antlr4-python2-runtime==4.7
# 4. antlr4 -Dlanguage=python SolrGrammar.g4
      # This automatically generates these files:
      # SolrGrammarLexer.py
      # SolrGrammarListener.py
      # SolrGrammar.tokens
      # SolrGrammarLexer.tokens
      # SolrGrammarParser.py
  # DO NOT EDIT these files, instead edit SolrGrammar.g4 if you need to make any
  # changes
# 5. To test, run "python solrlistener.py <string_to_parse>".

global SOLRSEGMENTS

class SolrGrammarQueryListener(SolrGrammarListener):

    def enterClauseAnd(self, ctx):
        global SOLRSEGMENTS
        t = self.inputstring[ctx.start.column : ctx.stop.column] + ctx.stop.text
        LOGGER.info("SolrGrammarClauseand: %s" % t)
        SOLRSEGMENTS.append(t)
        return

    def setSource(self, inputstring):
       self.inputstring = inputstring

def getSolrSegments(inputstring):
    global SOLRSEGMENTS
    SOLRSEGMENTS = []
    inputstream = InputStream(inputstring)
    lexer = SolrGrammarLexer(inputstream)
    stream = CommonTokenStream(lexer)
    parser = SolrGrammarParser(stream)
    tree = parser.mainQ()
    printer = SolrGrammarQueryListener()
    printer.setSource(inputstring)
    walker = ParseTreeWalker()
    walker.walk(printer, tree)
    LOGGER.info("Segments are %s" % SOLRSEGMENTS)
    return SOLRSEGMENTS

def main():
    if len(sys.argv) != 2:
      print("Usage: python solrlistener <string_to_parse>")
      exit(0)
    s = sys.argv[1]

    segments = getSolrSegments(s)
    print("segments are %s" % segments)

if __name__ == '__main__':
    main()
