# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.test import TestCase
from unittest.mock import patch, call, Mock

from dataconn.cml_connection_importer import CmlConnectionImporter
from datasets.models import DataConnection
import json

SPARK = {
    "id": 13,
    "workspaceConnectionId": "1",
    "projectId": "4",
    "name": "eng-ml-int-env-aws-dl",
    "type": "SPARK",
    "connectionInfo": {
        "dataLakeExternalDir": "s3a://eng-ml-weekly/eng-ml-int-env-aws-dl/warehouse/tablespace/external/hive"
    },
    "availability": True,
    "crn": "crn:cdp:datalake:us-west-1:c8dbde4b-ccce-4f8d-a581-830970ba4908:datalake:9109d8c9-0533-404d-b3d9-d89dcb8be44b",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:12:48.810Z",
}

DATAHUB = {
    'id': 49,
    'workspaceConnectionId': '4',
    'projectId': '11',
    'name': 'DataHub Test',
    'type': 'HIVE',
    'connectionInfo': {'jdbc_url': 'jdbc:hive2://charu-test-master0.eng-ml-i.svbr-nqvp.int.cldr.work/;ssl=true;transportMode=http;httpPath=charu-test/cdp-proxy-api/hive'},
    'availability': True,
    'crn': None,
    'createdAt': '2022-04-20T09:21:46.216Z',
    'updatedAt': '2022-04-20T09:21:46.216Z',
    'lastWorkspaceSyncAt': '2022-04-20T10:28:33.466Z',
    'workspaceAutoDiscovered': False,
    'workspaceLastAutoDiscoveredAt': None,
    'workspaceUpdatedAt': '2022-04-19T15:51:54.019Z'
}

HIVE = {
    "id": 15,
    "workspaceConnectionId": "3",
    "projectId": "4",
    "name": "eng-ml-hive-no-sso-aws",
    "type": "HIVE",
    "connectionInfo": {
        "jdbc_url": "jdbc:hive2://hs2-eng-ml-hive-no-sso-aws.dw-eng-ml-int-env-aws.svbr-nqvp.int.cldr.work/default;transportMode=http;httpPath=cliservice;socketTimeout=60;ssl=true;retries=3;",
        "virtualWarehouseName": "eng-ml-hive-no-sso-aws",
    },
    "availability": True,
    "crn": "compute-1634603993-prxd",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:00.189Z",
}

HIVE_KERBEROS = {
    "id": 15,
    "workspaceConnectionId": "3",
    "projectId": "4",
    "name": "eng-ml-hive-no-sso-aws",
    "type": "HIVE",
    "connectionInfo": {
        "jdbc_url": "jdbc:hive2://hs2-cdw-hive.apps.shared-os-qe-01.kcloud.cloudera.com/default;transportMode=http;httpPath=cliservice;socketTimeout=60;ssl=true;retries=3;kerberosEnableCanonicalHostnameCheck=false;principal=hive/dwx-env-rhcxab-env.cdp.local@QE-AD-1.CLOUDERA.COM",
        "virtualWarehouseName": "eng-ml-hive-no-sso-aws",
        "isPrivateCloud": True,
    },
    "availability": True,
    "crn": "compute-1634603993-prxd",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:00.189Z",
}
HIVE_UPDATED_AVAILABLE = {
    "id": 15,
    "workspaceConnectionId": "3",
    "projectId": "4",
    "name": "new-hive",
    "type": "HIVE",
    "connectionInfo": {
        "jdbc_url": "jdbc:hive2://hs2-eng-ml-hive-no-sso-aws.dw-eng-ml-int-env-aws.svbr-nqvp.int.cldr.work/default;transportMode=http;httpPath=cliservice;socketTimeout=60;ssl=true;retries=3;",
        "virtualWarehouseName": "eng-ml-hive-no-sso-aws",
    },
    "availability": True,
    "crn": "compute-1634603993-prxd",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:00.189Z",
}


HIVE_UPDATED = {
    "id": 15,
    "workspaceConnectionId": "3",
    "projectId": "4",
    "name": "new-hive",
    "type": "HIVE",
    "connectionInfo": {
        "jdbc_url": "jdbc:hive2://hs2-eng-ml-hive-no-sso-aws.dw-eng-ml-int-env-aws.svbr-nqvp.int.cldr.work/default;transportMode=http;httpPath=cliservice;socketTimeout=60;ssl=true;retries=3;",
        "virtualWarehouseName": "eng-ml-hive-no-sso-aws",
    },
    "availability": False,
    "crn": "compute-1634603993-prxd",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:00.189Z",
}

IMPALA = {
    "id": 16,
    "workspaceConnectionId": "4",
    "projectId": "4",
    "name": "eng-ml-impala-aws",
    "type": "IMPALA",
    "connectionInfo": {
        "jdbc_url": "jdbc:impala://coordinator-eng-ml-impala-aws.dw-eng-ml-int-env-aws.svbr-nqvp.int.cldr.work:443/default;AuthMech=3;transportMode=http;httpPath=cliservice;ssl=1;UID=srv_cml_machine_user_2bd324aa-;PWD=PASSWORD",
        "virtualWarehouseName": "eng-ml-impala-aws",
    },
    "availability": True,
    "crn": "impala-1634604048-qf5l",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:01.501Z",
}

IMPALA_UNAVAILABLE = {
    "id": 177,
    "workspaceConnectionId": "5",
    "projectId": "4",
    "name": "impala unavailable",
    "type": "IMPALA",
    "connectionInfo": {
        "jdbc_url": "jdbc:impala://coordinator-eng-ml-impala-aws.dw-eng-ml-int-env-aws.svbr-nqvp.int.cldr.work:443/default;AuthMech=3;transportMode=http;httpPath=cliservice;ssl=1;UID=srv_cml_machine_user_2bd324aa-;PWD=PASSWORD",
        "virtualWarehouseName": "eng-ml-impala-aws",
    },
    "availability": False,
    "crn": "impala-1634604048-qf5l",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:01.501Z",
}

IMPALA_KERBEROS = {
    "id": 100,
    "workspaceConnectionId": "4",
    "projectId": "4",
    "name": "eng-ml-impala-aws",
    "type": "IMPALA",
    "connectionInfo": {
        "jdbc_url": "jdbc:impala://coordinator-cdw-impala.apps.shared-os-qe-01.kcloud.cloudera.com:443/default;AuthMech=1;transportMode=http;httpPath=cliservice;ssl=1;KrbHostFQDN=dwx-env-rhcxab-env.cdp.local.;KrbServiceName=hive",
        "virtualWarehouseName": "eng-ml-impala-aws",
        "isPrivateCloud": True,
    },
    "availability": True,
    "crn": "impala-1634604048-qf5l",
    "createdAt": "2021-10-26T17:01:24.300Z",
    "updatedAt": "2021-10-26T17:01:24.300Z",
    "lastWorkspaceSyncAt": None,
    "workspaceAutoDiscovered": True,
    "workspaceLastAutoDiscoveredAt": None,
    "workspaceUpdatedAt": "2021-10-26T16:13:01.501Z",
}


class CmlConnectionImporterTest(TestCase):
    @patch("dataconn.cml_connection_importer.LOGGER")
    def test_when_no_api_found_should_log_warning(self, LOGGER):
        # Given & When
        CmlConnectionImporter()
        # Then
        LOGGER.warning.assert_called_once_with(
            "MLExplore: Error while fetching credentials (Exception: 'NoneType' object has no attribute 'get_hadoop_username')"
        )

    @patch("dataconn.cml_connection_importer.LOGGER")
    def test_when_no_api_found_should_stop_connecting(self, LOGGER):
        # Given & When
        CmlConnectionImporter().connect()
        # Then
        warnings = [
            call("MLExplore: Error while fetching credentials (Exception: 'NoneType' object has no attribute 'get_hadoop_username')"),
            call("MLExplore: Synced stopped because the required cml library was not imported"),
        ]

        LOGGER.warning.assert_has_calls(warnings)

    @patch('dataconn.cml_connection_importer.get_cmldata')
    @patch("dataconn.cml_connection_importer.LOGGER")
    def test_when_no_workload_password_should_log_error(self, LOGGER, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = None
        cmldata.get_project_dataconnections.return_value = []

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        LOGGER.warning.assert_called_once_with('MLExplore: Cloudera AI workload password is not set')

    @patch('dataconn.cml_connection_importer.get_cmldata')
    @patch("dataconn.cml_connection_importer.LOGGER")
    def test_updating_dataconnection_after_adding_workload_password(self, LOGGER, get_cmldata):
        # Given & When
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = None
        cmldata.get_project_dataconnections.return_value = [HIVE]

        get_cmldata.return_value = cmldata
        CmlConnectionImporter().connect()

        hive = DataConnection.objects.first()
        self.assertEqual(hive.dataconnection_password, None)

        cmldata.get_workload_password.return_value = ''
        cmldata.get_project_dataconnections.return_value = [HIVE]

        get_cmldata.return_value = cmldata
        CmlConnectionImporter().connect()

        # Then
        hive = DataConnection.objects.first()
        self.assertTrue(hive.dataconnection_password is not None)
        connection_cnt = len(DataConnection.objects.all())
        self.assertEqual(connection_cnt, 1)

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_parsing_datahub_connection(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [DATAHUB]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        datahub_connection = DataConnection.objects.first()
        connection_info = json.loads(datahub_connection.dataconnection_info)['PARAMS']
        self.assertEqual(connection_info['HOST'], 'charu-test-master0.eng-ml-i.svbr-nqvp.int.cldr.work')
        self.assertEqual(connection_info['PORT'], 443)
        self.assertEqual(connection_info['HS2_HTTP_SQLPATH'], 'charu-test/cdp-proxy-api/hive')

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_invalid_dataconnection_should_filter_it_out(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [SPARK]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        connection_cnt = len(DataConnection.objects.all())
        self.assertEqual(connection_cnt, 0)

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_valid_dataconnection_should_add_to_db(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [HIVE]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        connection_cnt = len(DataConnection.objects.all())
        self.assertEqual(connection_cnt, 1)

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_valid_dataconnection_should_contain_correct_attributes(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [HIVE]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        connection = DataConnection.objects.first()
        params = json.loads(connection.dataconnection_info)['PARAMS']
        self.assertEqual(params['HOST'], 'hs2-eng-ml-hive-no-sso-aws.dw-eng-ml-int-env-aws.svbr-nqvp.int.cldr.work')
        self.assertEqual(params['PORT'], 443)
        self.assertEqual(params['AUTH'], 'LDAP')
        self.assertEqual(params['HS2_HTTP_SQLPATH'], 'cliservice')

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_impala_with_kerberos_auth_should_contain_correct_parameters(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [IMPALA_KERBEROS]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        connection = DataConnection.objects.first()
        params = json.loads(connection.dataconnection_info)['PARAMS']
        self.assertEqual(params['HOST'], 'coordinator-cdw-impala.apps.shared-os-qe-01.kcloud.cloudera.com')
        self.assertEqual(params['PORT'], 443)
        self.assertEqual(params['AUTH'], 'KERBEROS')
        self.assertEqual(params['MODE'], 'http')
        self.assertEqual(params['HS2_HTTP_SQLPATH'], 'cliservice')
        self.assertEqual(params['SOCK'], 'ssl')
        self.assertEqual(params['KERBSERV'], 'hive')
        self.assertEqual(params['KRB_HOST'], 'dwx-env-rhcxab-env.cdp.local')

    # This test case will valuable once the DW bug is fixed (https://jira.cloudera.com/browse/DWX-19109)
    # @patch('dataconn.cml_connection_importer.get_cmldata')
    # def test_when_hive_with_kerberos_auth_should_contain_correct_parameters(self, get_cmldata):
    #     # Given
    #     cmldata = Mock()
    #
    #     cmldata.get_hadoop_username.return_value = ''
    #     cmldata.get_workload_password.return_value = ''
    #
    #     cmldata.get_project_dataconnections.return_value = [HIVE_KERBEROS]
    #
    #     get_cmldata.return_value = cmldata
    #     # When
    #     CmlConnectionImporter().connect()
    #     # Then
    #     connection = DataConnection.objects.first()
    #     params = json.loads(connection.dataconnection_info)['PARAMS']
    #     self.assertEqual(params['HOST'], 'hs2-cdw-hive.apps.shared-os-qe-01.kcloud.cloudera.com')
    #     self.assertEqual(params['PORT'], 443)
    #     self.assertEqual(params['SOCK'], 'ssl')
    #     self.assertEqual(params['MODE'], 'http')
    #     self.assertEqual(params['AUTH'], 'KERBEROS')
    #     self.assertEqual(params['HS2_HTTP_SQLPATH'], 'cliservice')
    #     self.assertEqual(params['KERBSERV'], 'hive')
    #     self.assertEqual(params['KERBHOST'], 'dwx-env-rhcxab-env.cdp.local')

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_updating_dataconnection(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [HIVE]
        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()

        connection = DataConnection.objects.first()
        self.assertEqual(connection.dataconnection_name, 'eng-ml-hive-no-sso-aws')

        cmldata.get_project_dataconnections.return_value = [HIVE_UPDATED_AVAILABLE]
        get_cmldata.return_value = cmldata

        CmlConnectionImporter().connect()
        # Then
        connection = DataConnection.objects.first()
        self.assertEqual(connection.dataconnection_name, 'new-hive')
        self.assertTrue(json.loads(connection.cml_status)['isAvailable'])

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_updating_removed_dataconnection_should_remove_cml_id(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [HIVE]
        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()

        connection = DataConnection.objects.first()
        self.assertEqual(connection.dataconnection_name, 'eng-ml-hive-no-sso-aws')

        cmldata.get_project_dataconnections.return_value = []
        get_cmldata.return_value = cmldata

        CmlConnectionImporter().connect()
        # Then
        connection = DataConnection.objects.first()
        self.assertEqual(connection.cml_id, None)

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_updating_unavailable_dataconnection_should_mark_it_unavailable(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [HIVE]
        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()

        connection = DataConnection.objects.first()
        self.assertEqual(connection.dataconnection_name, 'eng-ml-hive-no-sso-aws')

        cmldata.get_project_dataconnections.return_value = [HIVE_UPDATED]
        get_cmldata.return_value = cmldata

        CmlConnectionImporter().connect()
        # Then
        connection = DataConnection.objects.first()
        self.assertEqual(connection.dataconnection_name, 'eng-ml-hive-no-sso-aws')
        self.assertEqual(connection.cml_id, 15)
        self.assertFalse(json.loads(connection.cml_status)['isAvailable'])

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_multiple_dataconnections_should_filter_n_add_to_db(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = ''

        cmldata.get_project_dataconnections.return_value = [SPARK, HIVE, IMPALA, IMPALA_UNAVAILABLE]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        connection_cnt = len(DataConnection.objects.all())
        self.assertEqual(connection_cnt, 2)

    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_multiple_dataconnections_when_no_password_should_filter_n_add_to_db(self, get_cmldata):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = None

        cmldata.get_project_dataconnections.return_value = [SPARK, HIVE, IMPALA]

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        connection_cnt = len(DataConnection.objects.all())
        self.assertEqual(connection_cnt, 2)

    @patch('dataconn.cml_dependencies.FeatureFlipper')
    @patch("dataconn.cml_dependencies.LOGGER")
    @patch('dataconn.cml_connection_importer.get_cmldata')
    def test_when_fail_api_call_should_log_warning_and_disable_syncing(self, get_cmldata, LOGGER, featureflipper):
        # Given
        cmldata = Mock()

        cmldata.get_hadoop_username.return_value = ''
        cmldata.get_workload_password.return_value = None

        cmldata.get_project_dataconnections.side_effect = Exception('Examplery issue')

        get_cmldata.return_value = cmldata
        # When
        CmlConnectionImporter().connect()
        # Then
        print('===================')
        print(LOGGER.method_calls)
        last_warning = LOGGER.method_calls[-1][1][0]
        self.assertEqual(last_warning, 'MLExplore: Sync fails, cannot fetch project data connections due to: Examplery issue')
        featureflipper.return_value.set_flipper.assert_called_with('enable_cml_sync', False, 'SETTINGS')
