# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from dataconn.dataconn_dataengine import DataEngine
from django.core.exceptions import FieldError

import unittest

class TestCacheValidation(unittest.TestCase):

  def setUp(self):
    super().setUp()
    self.data_engine = DataEngine()
    self.data_engine.connection_type = 'test_dataconnection'

  def is_valid(self, expr: str):
    self.data_engine.validate_dsreq({
        'dimensions': [{
            'expr': expr,
          }
        ],
      }, validation_type='cache')

  def is_invalid(self, expr: str):
    with self.assertRaises(FieldError):
      self.data_engine.validate_dsreq({
          'dimensions': [{
              'expr': expr,
            }
          ],
        }, validation_type='cache')

  def test_valid_dsreq(self):
    # this could be a user-defined function:
    self.is_valid('snow()')

  def test_invalid_valid_dsreq(self):
    # "now" must be not cached:
    self.is_invalid('now()')

  def test_invalid_valid_dsreq_with_spaces(self):
    # some databases allow spaces around and inside the parantheses
    self.is_invalid(' now ( ) ')
