# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import unittest
import datetime
import decimal

from dataconn.dataconn_dbapi2 import type_convert, arctypes

class TestDataconnDbapi2(unittest.TestCase):

  def test_type_convert_for_frontend(self):

    self.assertEqual('2024-11-12 14:20:30.123456', type_convert(
      datetime.datetime.fromisoformat('2024-11-12T14:20:30.123456')))

    self.assertIsNone(type_convert(None))

    # only milliseconds are preserved for time
    self.assertEqual('10:30:10.012', type_convert(
      datetime.time(10, 30, 10, 12000)))

    # only milliseconds are preserved for time
    self.assertEqual('10:30:10.012', type_convert(
      datetime.time(10, 30, 10, 12), milliToMicro=True))

    # datetime has microseconds:
    # self.assertEqual('2024-11-12 14:20:30.123456', type_convert(
    #   datetime.datetime.fromisoformat('2024-11-12T14:20:30.123456')))

    self.assertEqual(
      '2024-11-12', type_convert(datetime.date.fromisoformat('2024-11-12')))

    self.assertEqual('1.1', type_convert(decimal.Decimal('1.1')))

    # when timedelta exceeds one day it will be probably unusable
    self.assertEqual('64 days, 8:05:56.000010', type_convert(
      datetime.timedelta(days=64, seconds=29156, microseconds=10)))

    """
    Can not test Oracle types conversion:
    >>> import cx_Oracle
    >>> cx_Oracle.LOB
    <class 'cx_Oracle.LOB'>
    >>> cx_Oracle.LOB()
    Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
    TypeError: cannot create 'cx_Oracle.LOB' instances
    """

    self.assertIsNone(type_convert(None, arctypes.ARC_DATATYPE_BINARY))

    snowman = 'A\u2603B'
    snowman_bytes = snowman.encode('utf-8')
    snowman_bytes_broken = bytearray(snowman_bytes)
    snowman_bytes_broken[2] = 0  # results in invalid continuation byte
    snowman_broken = 'A\ufffd\x00\ufffdB'  # contains question marks

    self.assertEqual(type_convert(snowman_bytes), snowman)
    self.assertEqual(type_convert(snowman_bytes_broken), snowman_broken)

    self.assertEqual(type_convert(123), 123)
    self.assertEqual(type_convert(123.4), 123.4)
