# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from dataconn.thriftconn import kt_control

import unittest
import logging

LOG = logging.getLogger(__name__)


class TestKinitErrorHandling(unittest.TestCase):
  def setUp(self) -> None:
    super().setUp()
    self.kt_control = kt_control(
      principal='testuser',
      keytab='/etc/testuser.keytab',
      frequency=12 * 3600,
      logger=LOG)

  def test_success(self):
    self.kt_control.get_kinit_command = lambda: ['echo', 'abc']
    success = self.kt_control.run_kinit()
    self.assertTrue(success)

  def test_failure(self):

    failing_command = ['python', '-c',
                       "import sys; sys.stderr.write('TEST ERROR');sys.exit(13)"]
    self.kt_control.get_kinit_command = lambda: failing_command

    with self.assertLogs(LOG, level=logging.ERROR) as logs:
      success = self.kt_control.run_kinit()
      self.assertFalse(success)
      self.assertIn("Couldn't reinit from keytab!", logs.output[-1])
