# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.


from django.db.migrations.migration import Migration
from django.db.migrations.state import ProjectState
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.backends.base.introspection import BaseDatabaseIntrospection

import logging

LOG = logging.getLogger('arcweb.migration_utils')

def table_column_exists(connection: BaseDatabaseWrapper, table_name, column_name):
  """
  Does not consult the Django models but rather checks the database tables.
  column_name is a column name, not a Django model field. So it is 'dataset_id' for
  ForeignKey('dataset').
  """
  with connection.cursor() as cursor:
    if table_name not in connection.introspection.table_names():
      # change in Django 4.2: get_table_description throws error for unexisting tables
      # so return early if the table is missing
      return False

    columns = [col.name for col in connection.introspection.get_table_description(
      cursor, table_name)]
  return column_name in columns


def apply_conditionally_on_segment_dataset_existence(migration_name: str, migration: Migration, project_state: ProjectState, schema_editor: BaseDatabaseSchemaEditor, collect_sql=False):

  segment_table = project_state.apps.get_model(
    'datasets', 'segment')._meta.db_table
  field_exists = table_column_exists(
    schema_editor.connection, segment_table, 'dataset_id')

  if field_exists:
    LOG.debug('datasets_segment.dataset exists. Applying %s', migration_name)
    return Migration.apply(migration, project_state, schema_editor, collect_sql)
  else:
    LOG.debug('datasets_segment.dataset does not exist. Skipping %s', migration_name)
    return project_state
