# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations, models
import uuid
import arcwebbase.validator


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DataConnection',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('dataconnection_type', models.CharField(max_length=100, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataconnection_name', models.CharField(max_length=1000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataconnection_description', models.TextField(max_length=4000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataconnection_password', models.TextField(max_length=4000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataconnection_info', models.TextField(max_length=4000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataconnection_settings', models.TextField(max_length=4000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('cache_sequence', models.IntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name='DataSet',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('dataset_name', models.CharField(max_length=1000, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataset_type', models.CharField(max_length=20, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataset_detail', models.TextField(max_length=20000, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataset_description', models.TextField(max_length=20000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataset_info', models.TextField(max_length=20000, blank=True)),
                ('dataset_lvinfo', models.TextField(max_length=20000, blank=True, null=True)),
                ('dataset_everyone_perm', models.CharField(max_length=20, default='readwrite', validators=[arcwebbase.validator.validateStringNoHtml])),
                ('dataset_create_user_perm', models.CharField(max_length=20, default='setupperm', validators=[arcwebbase.validator.validateStringNoHtml])),
                ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
                ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
                ('cache_sequence', models.IntegerField(default=0)),
                ('dataset_settings', models.TextField(max_length=20000, default='{}')),
                ('dataconnection', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='datasets.DataConnection')),
            ],
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('event_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('event_data', models.TextField(max_length=32768, blank=True, null=True)),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
                ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
                ('dataset', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='datasets.DataSet')),
            ],
        ),
        migrations.CreateModel(
            name='FilterAssociation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('filterassociation_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('filterassociation_data', models.TextField(max_length=32768, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
                ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
                ('dataset', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='datasets.DataSet')),
            ],
        ),
        migrations.CreateModel(
            name='Segment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('segment_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('segment_data', models.TextField(max_length=32768, blank=True, null=True)),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
                ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
                ('dataset', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='datasets.DataSet')),
            ],
        ),
    ]
