# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# Generated by Django 3.2.6 on 2022-04-22 15:05

from django.db import migrations, models
import json


# Fixes original 0011 datasets migrations where JSONField was used originally and then later migrated to a TextField
def add_json_as_text(apps, schema_editor):
    DataConnection = apps.get_model('datasets', 'dataconnection')
    for connection in DataConnection.objects.all():
        cml_status_json = connection.cml_status
        # If original 0011 migration ran with JSONField type,
        # we need to dump the dict into a json string so that it stores properly as a TextField.
        if isinstance(cml_status_json, dict):
            connection.cml_status_text = json.dumps(cml_status_json)
        # If fixed 0011 migration ran with TextField type, we don't need to transform the output to a string
        # since it will be stored that way already
        else:
            connection.cml_status_text = cml_status_json
        # Update Samples data connections with Python 3.9 version in file path
        if connection.dataconnection_type == "sqlite":
            try:
                connection_info = json.loads(connection.dataconnection_info)
                connection_info_file = connection_info["PARAMS"]["FILE"]
                connection_info_file_updated = connection_info_file.replace("python3.6", "python3.9")
                connection_info["PARAMS"]["FILE"] = connection_info_file_updated
                connection.dataconnection_info = json.dumps(connection_info)
            except Exception as e:
                print("sqlite connections could not be updated: %s" % str(e))
        connection.save(update_fields=['cml_status_text', 'dataconnection_info'])


class Migration(migrations.Migration):

    dependencies = [
        ('datasets', '0011_auto_20220412_0915'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataconnection',
            name='cml_status_text',
            field=models.TextField(default='{}'),
        ),
        migrations.RunPython(
            add_json_as_text,
            migrations.RunPython.noop,
        ),
        migrations.RemoveField(
            model_name='dataconnection',
            name='cml_status',
        ),
        migrations.RenameField(
            model_name='dataconnection',
            old_name='cml_status_text',
            new_name='cml_status',
        ),
    ]
