# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations
from dataconn.cml_dependencies import get_cmldata, get_project_dataconnections


def add_cml_id_to_in_sync_connections(apps, schema_editor):
  DataConnection = apps.get_model('datasets', 'dataconnection')
  cml_connections = DataConnection.objects.filter(is_in_sync=True)
  for connection in cml_connections:
    cmldata = get_cmldata()
    if cmldata:
        project_connections = get_project_dataconnections(cmldata)

        for p_connection in project_connections:
            if p_connection['name'] == connection.dataconnection_name:
                connection.cml_id = int(p_connection['id'])
                connection.save()


class Migration(migrations.Migration):
  dependencies = [
    ('datasets', '0012_alter_dataconnection_cml_status'),
  ]
  operations = [
    migrations.RunPython(
      add_cml_id_to_in_sync_connections,
      migrations.RunPython.noop,
    ),
    migrations.RemoveField(
            model_name='dataconnection',
            name='is_in_sync',
    ),
  ]
