from django.db import migrations, models, connection
import django.db.models.deletion
import logging

LOG = logging.getLogger('arcweb.migrations')

def populate_version_id(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'DataSet')
    for dataset in DataSet.objects.filter(version_id__isnull=True):
        dataset.version_id = dataset.id
        dataset.save()

def populate_version_group_id(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'DataSet')
    for dataset in DataSet.objects.filter(version_group_id__isnull=True):
        dataset.version_group_id = dataset.id
        dataset.save()

def populate_version_name(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'DataSet')
    for dataset in DataSet.objects.filter(version_name__isnull=True):
        dataset.version_name = dataset.create_time
        dataset.save()

def create_dataset_segment_entries(apps, schema_editor):
    DatasetSegment = apps.get_model('datasets', 'DatasetSegment')
    Segment = apps.get_model('datasets', 'Segment')
    for segment in Segment.objects.all():
        DatasetSegment.objects.create(
            dataset_id=segment.dataset_id,
            segment_id=segment.id,
            dataset_version_id=segment.dataset.version_id)


class Migration(migrations.Migration):

    dependencies = [
        ('datasets', '0017_update_dataset_type_singletable'),
    ]

    operations = []

    def apply(self, project_state, schema_editor, collect_sql=False):
        LOG.debug('0018_modify_relation_between_datsets_and_segments.py is skipped for all database types starting with 7.2.8')
        # This migration fails during CDW update because the strict nature of postgresql
        # it needs to be split in three parts
        return project_state

    def mutate_state(self, project_state, preserve=True):
        return project_state

    operations += [
        migrations.AddField(
            model_name='dataset',
            name='is_active_version',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='dataset',
            name='version_group_id',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='dataset',
            name='version_name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='dataset',
            name='is_named_version',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='dataset',
            name='version_id',
            field=models.IntegerField(editable=False, null=True),
        ),
        migrations.RunPython(populate_version_id),
        migrations.RunPython(populate_version_group_id),
        migrations.RunPython(populate_version_name),
        migrations.CreateModel(
            name='DatasetSegment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dataset_version_id', models.IntegerField(null=True)),
                ('dataset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='datasets.dataset')),
                ('segment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='datasets.segment')),
            ],
            options={
                'unique_together': {('dataset', 'segment', 'dataset_version_id')},
            },
        ),
        migrations.RunPython(create_dataset_segment_entries),
        migrations.AddField(
            model_name='dataset',
            name='segments',
            field=models.ManyToManyField(related_name='dataset', through='datasets.DatasetSegment', to='datasets.Segment'),
        ),
        migrations.RemoveField(
            model_name='segment',
            name='dataset',
        ),
        migrations.AlterField(
            model_name='filterassociation',
            name='dataset',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='filterassociations', to='datasets.dataset'),
        ),
    ]
