# Generated by Django 3.2.24 on 2024-09-20 13:24

from django.db import migrations, models

from django.db.migrations.state import ProjectState
from django.db.backends.base.schema import BaseDatabaseSchemaEditor

from datasets.migration_utils import apply_conditionally_on_segment_dataset_existence

def populate_version_id(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'DataSet')
    for dataset in DataSet.objects.filter(version_id__isnull=True):
        dataset.version_id = dataset.id
        dataset.save()

def populate_version_group_id(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'DataSet')
    for dataset in DataSet.objects.filter(version_group_id__isnull=True):
        dataset.version_group_id = dataset.id
        dataset.save()

def populate_version_name(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'DataSet')
    for dataset in DataSet.objects.filter(version_name__isnull=True):
        dataset.version_name = dataset.create_time
        dataset.save()

class Migration(migrations.Migration):

    dependencies = [
        ('datasets', '0018_modify_relation_between_datsets_and_segments'),
    ]

    operations = []

    def apply(self, project_state: ProjectState, schema_editor: BaseDatabaseSchemaEditor, collect_sql=False):
        return apply_conditionally_on_segment_dataset_existence(
            '0019_postgres_friendly_version_of_0018_1.py',
            self,
            project_state,
            schema_editor,
            collect_sql
        )

    operations += [
        migrations.AddField(
            model_name='dataset',
            name='is_active_version',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='dataset',
            name='version_group_id',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='dataset',
            name='version_name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='dataset',
            name='is_named_version',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='dataset',
            name='version_id',
            field=models.IntegerField(editable=False, null=True),
        ),
        migrations.RunPython(populate_version_id),
        migrations.RunPython(populate_version_group_id),
        migrations.RunPython(populate_version_name),
    ]
