# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# Generated by Django 3.2.15 on 2022-09-19 21:59

from django.db import migrations, models
import json
from util.string_operations import extract_tablenames_from_sqlquery

def get_tablenames(ds):
    if ds.dataset_type == "singletable":
        if ds.dataset_detail == "Created from SQL":
            tl = json.loads(ds.dataset_info)
            return extract_tablenames_from_sqlquery(tl[0].get("select"))

        return [ds.dataset_detail]

    tl = json.loads(ds.dataset_info)
    tablenames = []
    for table in tl:
        if table.get("tablename") == "text.sql":
            tablenames.extend(extract_tablenames_from_sqlquery(table.get("select")))
        else:
            tablenames.append(table.get("tablename"))

    return tablenames

def copy_tablenames_from_dataset_info_to_dataset_tablenames(apps, _):
    Dataset = apps.get_model('datasets', 'dataset')
    for ds in Dataset.objects.all():
        ds.dataset_tablenames = json.dumps(get_tablenames(ds))
        ds.save(update_fields=['dataset_tablenames'])

class Migration(migrations.Migration):

    dependencies = [
        ('datasets', '0021_postgres_friendly_version_of_0018_3'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataset',
            name='dataset_tablenames',
            field=models.TextField(max_length=10000, blank=True),
        ),
        migrations.RunPython(
            copy_tablenames_from_dataset_info_to_dataset_tablenames,
            migrations.RunPython.noop,
        ),
    ]
