# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from uuid import UUID
from ..models import DataConnection
from jobs.views_encryption import encrypt_secret
import json


class Migration(SchemaMigration):

    def forwards(self, orm):
        def encrypt_passwords():
            for id, password in DataConnection.objects.all().values_list('id',
                                                         'dataconnection_password'):
                DataConnection.objects.filter(id=id) \
                    .update(dataconnection_password=json.dumps(encrypt_secret(str(password))))

        # Changing field 'DataConnection.dataconnection_password'
        db.alter_column('datasets_dataconnection', 'dataconnection_password', self.gf('django.db.models.fields.TextField')(max_length=4000))

        if not db.dry_run:
          from datasets.models import enque_migration
          enque_migration(encrypt_passwords)

    def backwards(self, orm):
        pass


    models = {
        'datasets.dataconnection': {
            'Meta': {'object_name': 'DataConnection'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_description': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_info': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'dataconnection_password': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_settings': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        'datasets.dataset': {
            'Meta': {'object_name': 'DataSet'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataConnection']", 'null': 'True', 'blank': 'True'}),
            'dataset_create_user_perm': ('django.db.models.fields.CharField', [], {'default': "'setupperm'", 'max_length': '20'}),
            'dataset_description': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_detail': ('django.db.models.fields.TextField', [], {'max_length': '20000'}),
            'dataset_everyone_perm': ('django.db.models.fields.CharField', [], {'default': "'readwrite'", 'max_length': '20'}),
            'dataset_info': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_lvinfo': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'null': 'True', 'blank': 'True'}),
            'dataset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'dataset_settings': ('django.db.models.fields.TextField', [], {'default': "'{}'", 'max_length': '20000'}),
            'dataset_type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('8338dbce-ecc1-44ef-ab9c-9b09c112abfc')", 'unique': 'True', 'max_length': '36'})
        },
        'datasets.event': {
            'Meta': {'object_name': 'Event'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            'event_data': ('django.db.models.fields.TextField', [], {'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'event_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('c69e3d15-bea6-470c-b237-8a37aeee045b')", 'unique': 'True', 'max_length': '36'})
        },
        'datasets.filterassociation': {
            'Meta': {'object_name': 'FilterAssociation'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            'filterassociation_data': ('django.db.models.fields.TextField', [], {'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'filterassociation_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('7b8c5829-dda6-4b78-b173-9cd1705cfdce')", 'unique': 'True', 'max_length': '36'})
        },
        'datasets.segment': {
            'Meta': {'object_name': 'Segment'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'segment_data': ('django.db.models.fields.TextField', [], {'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'segment_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('a928f03d-9d1a-4b13-992c-6ed5168c86c3')", 'unique': 'True', 'max_length': '36'})
        }
    }

    complete_apps = ['datasets']