# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.urls import re_path

from datasets import views
from datasets import views_connections
import extracts.views

urlpatterns = [
  re_path(r'^dataset$', views.dataset, name='datasets-dataset'),
  re_path(r'^dataset/(?P<dataset_id>\d+)$', views.dataset),
  re_path(r'^dataset/cache_reset/(?P<id>\d+)$', views.dataset_cache_reset),
  re_path(r'^dataset/associated_dashboards/(?P<dataset_id>\d+)$', views.get_associated_dashboards),
  re_path(r'^dataset/version_control/$', views.swap_records, name='swap_records'),
  re_path(r'^dataset/report_creation_permissions$', views.get_report_creation_permissions, name='get_report_creation_permissions'),

  re_path(r'^dataconnection$', views_connections.dataconnection, name='datasets-dataconnection'),
  re_path(r'^dataconnection/(?P<dataconnection_id>\d+)$', views_connections.dataconnection, name='datasets-dataconnection-id'),
  re_path(r'^dataconnection/clone$', views_connections.clone_dataconnection, name='clone-dataconnection'),
  re_path(r'^dataconnection/clone/(?P<dataconnection_id>\d+)$', views_connections.clone_dataconnection, name='clone-dataconnection-id'),
  re_path(r'^dataconnection/cache_reset/(?P<id>\d+)$', views.dataconnection_cache_reset),
  re_path(r'^dataconnection/sync_cml_connections/', views.sync_cml_connections, name='dataconnection-sync-cml'),

  re_path(r'^segment$', views.segment, name='datasets-segment'),
  re_path(r'^segment/(?P<segment_id>\d+)$', views.segment),

  re_path(r'^event$', views.event, name='datasets-event'),
  re_path(r'^event/(?P<event_id>\d+)$', views.event),

  re_path(r'^filterassociation$', views.filterassociation, name='datasets-filterassociation'),
  re_path(r'^filterassociation/(?P<filterassociation_id>\d+)$', views.filterassociation),

  re_path(r'^userperms$', views.userperms, name='datasets-userperms'),
  re_path(r'^userperms/(?P<dataset_id>\d+)$', views.userperms),
  re_path(r'^userperms/(?P<dataset_id>\d+)/(?P<username_in>[\w\.\@]+)$', views.userperms),

  re_path(r'^groupperms$', views.groupperms, name='datasets-groupperms'),
  re_path(r'^groupperms/(?P<dataset_id>\d+)$', views.groupperms),
  re_path(r'^groupperms/(?P<dataset_id>\d+)/(?P<groupname_in>[\w\.]+)$', views.groupperms),

  re_path(r'^extract$', extracts.views.extract, name='datasets-extract'),
  re_path(r'^extract/(?P<extract_id>\d+)$', extracts.views.extract),

  re_path(
    r'^possible_deleted_reports',
    views.possible_deleted_reports,
    name='possible-deleted-reports'
  )
]
