# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
import csv
import io

import logging

# create logger
LOGGER = logging.getLogger("arcweb")

class OpenDataUrlToData(object):
  def __init__(self, url_request):
    self.url_request = url_request
    url = url_request.url()
    self.name = url
    self._handled = False
    self._error = None

    self.response = url_request.response()
    headers = self.response.headers
    headerFields = headers.get('X-Soda2-Fields', None)
    if headerFields is None:
      return

    d = json.loads(self.response.text)
    if hasattr(d, 'has_key') and 'error' in d and d['error'] == True:
      self._handled = True
      self._error = "Error: %s" % d.get('message', 'unknown')
      return

    self._handled = True
    columns = json.loads(headerFields)
    columnPos = {}
    columnActive = {}
    for idx, elem in enumerate(columns):
      columnPos[elem] = idx
      columnActive[elem] = 0

    rows = []
    for elem in d:
      row = [None] * len(columns)
      for k, v in list(elem.items()):
        if hasattr(v, 'has_key'):
          for k1, v1 in list(v.items()):
            c = 'nested_' +  k + '_' + k1
            pos = columnPos.get(c)
            if pos is None:
              columns.append(c)
              pos = len(columns) - 1
              columnPos[c] = pos
              columnActive[c] = 1
              row.append(v1)
            else:
              columnActive[c] += 1
              row[pos] = v1
        else:
          pos = columnPos.get(k)
          if pos is None:
            raise Exception('Invalid column %s' % k)

          columnActive[k] += 1
          row[pos] = v

      rows.append(row)

    activeColumns = []
    for idx, elem in enumerate(columns):
      if columnActive[elem] > 0:
        activeColumns.append(idx)

    if len(activeColumns) < len(columns):
      newColumns = []
      newColumnPositions = []
      for idx, col in enumerate(columns):
        if columnActive[col] > 0:
          newColumns.append(col)
          newColumnPositions.append(idx)

      newRows = []
      for row in rows:
        newRow = [row[x] for x in newColumnPositions]
        newRows.append(newRow)

      columns = newColumns
      rows = newRows

    out = io.StringIO()
    w = csv.writer(out)
    w.writerow(columns)
    for row in rows:
      w.writerow(row)
    self.data = out.getvalue().encode('utf-8')

  def handled(self):
    return self._handled

  def chunks(self):
    return [self.data]

  def error(self):
    return self._error

