# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion


def create_default_schedule_groups(apps, schema_editor):
    ScheduleGroup = apps.get_model('jobs', 'ScheduleGroup')

    # hourly at 15 past
    ScheduleGroup.objects.create(id=1, \
                                 schedule_name="Hourly", \
                                 run_interval="15 * * * *")

    # daily at 3 am
    ScheduleGroup.objects.create(id=2, \
                                 schedule_name="Daily", \
                                 run_interval="0 3 * * *")

    # daily at 2 am weekdays only
    ScheduleGroup.objects.create(id=3, \
                                 schedule_name="Every Weekday", \
                                 run_interval="0 2 * * 1-5")

    # weekly Sunday 6 am
    ScheduleGroup.objects.create(id=4, \
                                 schedule_name="Weekly", \
                                 run_interval="0 6 * * 7")

    # monthly 1:30 am 1st
    ScheduleGroup.objects.create(id=5, \
                                 schedule_name="Monthly", \
                                 run_interval="30 1 1 * *")

    # Monthly 1st at 1 am quarterly
    ScheduleGroup.objects.create(id=6, \
                                 schedule_name="Quarterly", \
                                 run_interval="0 1 1 1,4,7,10 *")

class Migration(migrations.Migration):

    atomic = False

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='JobContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('field', models.CharField(max_length=200, blank=True, default=None)),
                ('value', models.TextField(blank=True, default='')),
                ('creator', models.ManyToManyField(default=1, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='JobLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('start_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('end_time', models.DateTimeField(null=True, auto_now=True)),
                ('contents_archive', models.TextField(blank=True, default='')),
                ('error_message', models.TextField(blank=True, default='')),
                ('last_task', models.TextField(blank=True, default='')),
                ('last_completed_target', models.IntegerField(blank=True, null=True, default=None)),
                ('status', models.CharField(max_length=1, default='R', choices=[('R', 'Still Running'), ('C', 'Completed'), ('F', 'Error'), ('P', 'Paused'), ('X', 'Canceled'), ('Q', 'Queued')])),
                ('image', models.FileField(blank=True, null=True, default='', upload_to='job/email')),
            ],
            options={
                'ordering': ['-end_time'],
            },
        ),
        migrations.CreateModel(
            name='JobSchedule',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('job_name', models.CharField(max_length=200, null=True, default=None)),
                ('run_counter', models.IntegerField(default=0)),
                ('success_counter', models.IntegerField(default=0)),
                ('max_success_run', models.IntegerField(blank=True, null=True)),
                ('last_finished_time', models.DateTimeField(blank=True, null=True, default=None)),
                ('last_status', models.CharField(max_length=1, blank=True, null=True, default=None, choices=[('R', 'Still Running'), ('C', 'Completed'), ('F', 'Error'), ('P', 'Paused'), ('X', 'Canceled'), ('Q', 'Queued')])),
                ('finished', models.BooleanField(default=False)),
                ('last_started_time', models.DateTimeField(blank=True, null=True, default=None)),
                ('consecutive_fails', models.IntegerField(default=0)),
                ('last_finished_job', models.IntegerField(blank=True, null=True, default=None)),
                ('creator', models.ForeignKey(on_delete=models.CASCADE, default=1, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='JobTarget',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('update_time', models.DateTimeField(auto_now=True)),
                ('target_order', models.IntegerField(default=0)),
                ('run_counter', models.IntegerField(default=0)),
                ('success_counter', models.IntegerField(default=0)),
                ('minimum_time_interval', models.IntegerField(blank=True, null=True, default=None)),
                ('last_finished_time', models.DateTimeField(blank=True, null=True, default=None)),
                ('target_type', models.CharField(max_length=4, default='E', choices=[('E', 'Email'), ('S', 'Snapshot'), ('T', 'Trigger Assessment'), ('R', 'Refresh Analytical Views'), ('C', 'Script')])),
                ('target_data', models.TextField(blank=True, default='')),
                ('schedule', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='jobs.JobSchedule')),
            ],
            options={
                'ordering': ['target_order'],
            },
        ),
        migrations.CreateModel(
            name='ScheduleGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
                ('schedule_name', models.CharField(max_length=200, unique=True, null=True, default=None)),
                ('update_time', models.DateTimeField(null=True, auto_now=True)),
                ('run_interval', models.CharField(max_length=200, unique=True, blank=True, null=True, default=None)),
                ('next_run_time', models.DateTimeField(blank=True, null=True)),
                ('creator', models.ForeignKey(blank=True, null=True, default=None, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='jobschedule',
            name='schedule_group',
            field=models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='jobs.ScheduleGroup'),
        ),
        migrations.AddField(
            model_name='joblog',
            name='schedule',
            field=models.ForeignKey(on_delete=models.CASCADE, to='jobs.JobSchedule'),
        ),
        migrations.CreateModel(
            name='CustomEmailTemplate',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('jobs.jobcontent',),
        ),
        migrations.CreateModel(
            name='EmailJobContent',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('jobs.jobcontent',),
        ),
        migrations.CreateModel(
            name='EmailJobTarget',
            fields=[
            ],
            options={
                'ordering': ['target_order'],
                'proxy': True,
            },
            bases=('jobs.jobtarget',),
        ),
        migrations.CreateModel(
            name='JobScheduleNow',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('jobs.jobschedule',),
        ),
        migrations.CreateModel(
            name='JobScheduleRepeat',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('jobs.jobschedule',),
        ),
        migrations.CreateModel(
            name='RefreshJobTarget',
            fields=[
            ],
            options={
                'ordering': ['target_order'],
                'proxy': True,
            },
            bases=('jobs.jobtarget',),
        ),
        migrations.CreateModel(
            name='ScriptTarget',
            fields=[
            ],
            options={
                'ordering': ['target_order'],
                'proxy': True,
            },
            bases=('jobs.jobtarget',),
        ),
        migrations.CreateModel(
            name='SnapshotJobTarget',
            fields=[
            ],
            options={
                'ordering': ['target_order'],
                'proxy': True,
            },
            bases=('jobs.jobtarget',),
        ),
        migrations.CreateModel(
            name='TriggerJobTarget',
            fields=[
            ],
            options={
                'ordering': ['target_order'],
                'proxy': True,
            },
            bases=('jobs.jobtarget',),
        ),
        migrations.RunPython(create_default_schedule_groups)
    ]
