# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.db import migrations
from django.db.models import Q


def copy_key(apps, schema_editor):
    JobSchedules = apps.get_model('jobs', 'jobschedule')
    Extracts = apps.get_model('datasets', 'extract')
    extract_jobs = JobSchedules.objects.filter(Q(job_name__istartswith="Data Extract: ") |
                                               Q(job_name__istartswith="Adhoc Data Extract: "))
    for schedule in extract_jobs:
        (_, _, extract_id) = schedule.job_name.rpartition(" ")
        try:
            extract_id = int(extract_id)
        except:
            continue

        extract = Extracts.objects.filter(id=extract_id).first()
        if extract:
            schedule.extract = extract
            schedule.save()


class Migration(migrations.Migration):
    dependencies = [
        ('jobs', '0010_jobschedule_extract')
    ]

    operations = [
        migrations.RunPython(copy_key,
                             migrations.RunPython.noop,
                             ),
    ]
