# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
from django.db import migrations, models


def copy_last_task_to_task_list(job_logs, _):
    logs = job_logs.get_model('jobs', 'JobLog')

    for log in logs.objects.all():
        tasks = [log.last_task]
        log.task_list = json.dumps({'tasks': tasks})
        
        log.save(update_fields=['task_list'])

def copy_task_list_to_last_task(job_logs, _):
    logs = job_logs.get_model('jobs', 'JobLog')

    for log in logs.objects.all():
        log.last_task = ''
        task_list = json.loads(log.task_list)
        if task_list and len(task_list['tasks']):
            log.last_task = task_list['tasks'][-1]
        
        log.save(update_fields=['last_task'])

class Migration(migrations.Migration):

    dependencies = [
        ('jobs', '0012_joblog_task_list'),
    ]

    operations = [
        migrations.RunPython(copy_last_task_to_task_list,
                             copy_task_list_to_last_task
                             ),
    ]
