# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
from django.db import migrations, models


def convert_task_to_object(job_logs, _):
    logs = job_logs.get_model('jobs', 'JobLog')

    for log in logs.objects.only('task_list'):
        task_list = json.loads(log.task_list)
        task_list['tasks'] = [{'message': task} if isinstance(task, str) else task for task in task_list['tasks']]
        log.task_list = json.dumps(task_list)
        
        log.save(update_fields=['task_list'])

def convert_task_to_string(job_logs, _):
    logs = job_logs.get_model('jobs', 'JobLog')

    for log in logs.objects.only('task_list'):
        task_list = json.loads(log.task_list)
        task_list['tasks'] = [task.get('message') if isinstance(task, dict) else task for task in task_list['tasks']]
        log.task_list = json.dumps(task_list)
        
        log.save(update_fields=['task_list'])

class Migration(migrations.Migration):

    dependencies = [
        ('jobs', '0014_auto_20240514_0506'),
    ]

    operations = [
        migrations.RunPython(convert_task_to_object,
                             convert_task_to_string
                             ),
    ]
