# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):
    no_dry_run = True

    def forwards(self, orm):
        # Deleting field 'JobLog.target_type'
        db.delete_column('jobs_joblog', 'target_type')

        # Deleting field 'JobSchedule.minimum_time_interval'
        db.delete_column('jobs_jobschedule', 'minimum_time_interval')

        # Deleting field 'JobSchedule.run_interval'
        db.delete_column('jobs_jobschedule', 'run_interval')

        # Adding field 'JobTarget.minimum_time_interval'
        db.add_column('jobs_jobtarget', 'minimum_time_interval',
                      self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobSchedule.last_finished_time'
        db.add_column('jobs_jobschedule', 'last_finished_time',
                      self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobSchedule.last_status'
        db.add_column('jobs_jobschedule', 'last_status',
                      self.gf('django.db.models.fields.CharField')(default=None, max_length=1, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobSchedule.last_finished_job'
        db.add_column('jobs_jobschedule', 'last_finished_job',
                      self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobSchedule.finished'
        db.add_column('jobs_jobschedule', 'finished',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

        # Adding field 'JobSchedule.last_started_time'
        db.add_column('jobs_jobschedule', 'last_started_time',
                      self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobSchedule.consecutive_fails'
        db.add_column('jobs_jobschedule', 'consecutive_fails',
                      self.gf('django.db.models.fields.IntegerField')(default=0),
                      keep_default=False)

        # Adding model 'ScheduleGroup'
        db.create_table('jobs_schedulegroup', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('create_time', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, null=True, blank=True)),
            ('creator', self.gf('django.db.models.fields.related.ForeignKey')(default=None, to=orm['auth.User'], null=True, on_delete=models.SET_NULL)),
            ('schedule_name', self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True)),
            ('run_interval', self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True, blank=True)),
            ('next_run_time', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('update_time', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, null=True, blank=True)),
        ))
        db.send_create_signal('jobs', ['ScheduleGroup'])

        # Adding unique constraint on 'ScheduleGroup', fields ['run_interval']
        db.create_unique('jobs_schedulegroup', ['run_interval'])

        # Adding field 'JobSchedule.schedule_group'
        db.add_column('jobs_jobschedule', 'schedule_group',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['jobs.ScheduleGroup'], null=True, blank=True),
                      keep_default=False)

        # Deleting field 'JobTarget.max_success_run'
        db.delete_column('jobs_jobtarget', 'max_success_run')

        # Fixes the run_counter and success_counter not updating for JobSchedule or JobTarget
        # Used notes from http://south.readthedocs.io/en/latest/commands.html and
        # user password example from http://south.readthedocs.io/en/latest/tutorial/part3.html

        # Update JobSchedule
        # Also update last_finished_job, last_finished_time, last_status, and finished fields
        for schedule in orm.JobSchedule.objects.all():
          last_job = schedule.jobtarget_set.last()
          if last_job:
            schedule.run_counter = last_job.joblog_set.count()
            last_log = last_job.joblog_set.first()
            schedule.last_status = last_log.status

            # Mark all jobs, regardless of status, as as finished
            schedule.finished = True
            schedule.last_finished_job = last_job.target_order
            schedule.last_finished_time = last_log.end_time
            schedule.success_counter = 1

          # Now SAVE
          schedule.save()

        # Update JobTarget
        for target in orm.JobTarget.objects.all():
          target.success_counter = 1
          target.run_counter = 1
          target.save()

        # Create default schedule
        ScheduleGroup = orm.ScheduleGroup;

        # hourly at 15 past
        ScheduleGroup.objects.create(id=1,\
                                     schedule_name="Hourly",\
                                     run_interval="15 * * * *")

        # daily at 3 am
        ScheduleGroup.objects.create(id=2,\
                                     schedule_name="Daily",\
                                     run_interval="0 3 * * *")

        # daily at 2 am weekdays only
        sched1 = ScheduleGroup(id=3,\
                               schedule_name="Every Weekday",\
                               run_interval="0 2 * * 1-5")
        sched1.save()

        # weekly Sunday 6 am
        sched2 = ScheduleGroup(id=4,\
                               schedule_name="Weekly",\
                               run_interval="0 6 * * 7")
        sched2.save()


        # monthly 1:30 am 1st
        ScheduleGroup.objects.create(id=5,\
                                     schedule_name="Monthly",\
                                     run_interval="30 1 1 * *")

        # Monthly 1st at 1 am quarterly
        sched3 = ScheduleGroup(id=6,\
                               schedule_name="Quarterly",\
                               run_interval="0 1 1 1,4,7,10 *")
        sched3.save()

    def backwards(self, orm):
        # Adding field 'JobLog.target_type'
        db.add_column('jobs_joblog', 'target_type',
                      self.gf('django.db.models.fields.CharField')(default='E', max_length=4),
                      keep_default=False)

        # Adding field 'JobSchedule.minimum_time_interval'
        db.add_column('jobs_jobschedule', 'minimum_time_interval',
                      self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobSchedule.run_interval'
        db.add_column('jobs_jobschedule', 'run_interval',
                      self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True, blank=True),
                      keep_default=False)

        # Deleting field 'JobTarget.minimum_time_interval'
        db.delete_column('jobs_jobtarget', 'minimum_time_interval')

        # Deleting field 'JobSchedule.last_finished_time'
        db.delete_column('jobs_jobschedule', 'last_finished_time')

        # Deleting field 'JobSchedule.last_status'
        db.delete_column('jobs_jobschedule', 'last_status')

        # Deleting field 'JobSchedule.last_finished_job'
        db.delete_column('jobs_jobschedule', 'last_finished_job')

        # Deleting field 'JobSchedule.finished'
        db.delete_column('jobs_jobschedule', 'finished')

        # Deleting field 'JobSchedule.last_started_time'
        db.delete_column('jobs_jobschedule', 'last_started_time')

        # Deleting field 'JobSchedule.consecutive_fails'
        db.delete_column('jobs_jobschedule', 'consecutive_fails')

        # Deleting model 'ScheduleGroup'
        db.delete_table('jobs_schedulegroup')

        # Deleting field 'JobSchedule.schedule_group'
        db.delete_column('jobs_jobschedule', 'schedule_group_id')

        # Adding field 'JobTarget.max_success_run'
        db.add_column('jobs_jobtarget', 'max_success_run',
                      self.gf('django.db.models.fields.IntegerField')(null=True, blank=True),
                      keep_default=False)

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'jobs.jobcontent': {
            'Meta': {'object_name': 'JobContent'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ManyToManyField', [], {'default': '1', 'to': "orm['auth.User']", 'symmetrical': 'False'}),
            'field': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'})
        },
        'jobs.joblog': {
            'Meta': {'ordering': "['-end_time']", 'object_name': 'JobLog'},
            'contents_archive': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'end_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'error_message': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.FileField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'last_task': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'start_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'R'", 'max_length': '1'}),
            'target': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.JobTarget']"})
        },
        'jobs.jobschedule': {
            'Meta': {'object_name': 'JobSchedule'},
            'consecutive_fails': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': "orm['auth.User']"}),
            'finished': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'job_name': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True'}),
            'last_finished_job': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_finished_time': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_started_time': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_status': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '1', 'null': 'True', 'blank': 'True'}),
            'max_success_run': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'run_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'schedule_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.ScheduleGroup']", 'null': 'True', 'blank': 'True'}),
            'success_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'})
        },
        'jobs.jobtarget': {
            'Meta': {'ordering': "['target_order']", 'object_name': 'JobTarget'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'minimum_time_interval': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'run_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'schedule': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.JobSchedule']", 'null': 'True', 'blank': 'True'}),
            'success_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'target_data': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'target_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'target_type': ('django.db.models.fields.CharField', [], {'default': "'E'", 'max_length': '4'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'jobs.schedulegroup': {
            'Meta': {'object_name': 'ScheduleGroup'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['auth.User']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'next_run_time': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'run_interval': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'schedule_name': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['jobs']