# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
import json


class Migration(SchemaMigration):
    no_dry_run = True

    def forwards(self, orm):
        # Adding field 'JobLog.schedule'
        first_schedule = orm.JobSchedule.objects.first()
        if first_schedule:
            first_schedule_id = orm.JobSchedule.objects.first().id
        else:
            first_schedule_id = 1

        db.add_column('jobs_joblog', 'schedule',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=first_schedule_id, to=orm['jobs.JobSchedule']),
                      keep_default=False)

        # Adding field 'JobLog.last_completed_target'
        db.add_column('jobs_joblog', 'last_completed_target',
                      self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True),
                      keep_default=False)

        # Adding field 'JobTarget.last_finished_time'
        db.add_column('jobs_jobtarget', 'last_finished_time',
                      self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True),
                      keep_default=False)

        # Add last_finished_time to schedule targets
        for schedule in orm.JobSchedule.objects.all():
          targets = schedule.jobtarget_set
          for targ in targets.all():
              # Update last_finished_time for each target
              last_finished_log = targ.joblog_set.filter(status='C').first()
              if last_finished_log:
                  targ.last_finished_time = last_finished_log.end_time
              targ.save()

        # Add last_completed_target to logs
        # Combine contents_archive, last_task, last_completed_target, image, end_time, error_message, status
        contents_archive = {}
        other_properties = {
            'last_task': None,
            'last_completed_target': None,
            'image': None,
            'end_time': None,
            'error_message': '',
            'status': None,
        }
        needs_latest = True
        for log in orm.JobLog.objects.order_by('-start_time').all():
          current_contents = log.contents_archive
          if current_contents:
              current_contents = json.loads(current_contents)
          else:
              current_contents = {}
          target_order = log.target.target_order
          if needs_latest:
              needs_latest = False
              # Add last_completed_target info
              contents_archive['run_parameters'] = current_contents.get('run_parameters', {})
              other_properties['last_task'] = log.last_task
              other_properties['last_completed_target'] = target_order
              other_properties['end_time'] = log.end_time
              other_properties['status'] = log.status

          if log.image:
              other_properties['image'] = log.image

          if log.error_message:
              other_properties['error_message'] += log.error_message

          if current_contents.get('run_parameters'):
            del current_contents['run_parameters']

          contents_archive['run_parameters'].update(current_contents)

          contents_archive[str(target_order)] = current_contents

          # Update info on first target's job log
          if target_order == 0:
              # Add schedule info
              log.schedule = log.target.schedule
              # Add compiled info from other job logs
              log.last_task = other_properties.get('last_task')
              log.last_completed_target = other_properties.get('last_completed_target')
              log.image = other_properties.get('image')
              log.end_time = other_properties.get('end_time')
              log.error_message = other_properties.get('error_message')
              log.status = other_properties.get('status')
              log.contents_archive = json.dumps(contents_archive)
              log.save()
              # Reset for next round
              other_properties = { 'error_message': '' }
              contents_archive = {}
              needs_latest = True
          # Delete all others
          else:
              log.delete()




    def backwards(self, orm):
        # Deleting field 'JobLog.schedule'
        db.delete_column('jobs_joblog', 'schedule_id')

        # Deleting field 'JobLog.last_completed_target'
        db.delete_column('jobs_joblog', 'last_completed_target')

        # Deleting field 'JobTarget.last_finished_time'
        db.delete_column('jobs_jobtarget', 'last_finished_time')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'jobs.jobcontent': {
            'Meta': {'object_name': 'JobContent'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ManyToManyField', [], {'default': '1', 'to': "orm['auth.User']", 'symmetrical': 'False'}),
            'field': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'})
        },
        'jobs.joblog': {
            'Meta': {'ordering': "['-end_time']", 'object_name': 'JobLog'},
            'contents_archive': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'end_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'error_message': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.FileField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'last_completed_target': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_task': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'schedule': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.JobSchedule']"}),
            'start_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'R'", 'max_length': '1'}),
            'target': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.JobTarget']"})
        },
        'jobs.jobschedule': {
            'Meta': {'object_name': 'JobSchedule'},
            'consecutive_fails': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': "orm['auth.User']"}),
            'finished': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'job_name': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True'}),
            'last_finished_job': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_finished_time': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_started_time': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'last_status': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '1', 'null': 'True', 'blank': 'True'}),
            'max_success_run': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'run_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'schedule_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.ScheduleGroup']", 'null': 'True', 'blank': 'True'}),
            'success_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'})
        },
        'jobs.jobtarget': {
            'Meta': {'ordering': "['target_order']", 'object_name': 'JobTarget'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_finished_time': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'minimum_time_interval': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'run_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'schedule': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['jobs.JobSchedule']", 'null': 'True', 'blank': 'True'}),
            'success_counter': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'target_data': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'target_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'target_type': ('django.db.models.fields.CharField', [], {'default': "'E'", 'max_length': '4'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'jobs.schedulegroup': {
            'Meta': {'object_name': 'ScheduleGroup'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['auth.User']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'next_run_time': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'run_interval': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'schedule_name': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['jobs']
