# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
These tests verify the arcengine_v2 DataEngine interface.

We mock out the cursor into the backend, and verify that the
appropriate SQL command is sent it by the backend for SHOW_DATABASES,
SHOW_TABLES, table_columns(), and simple SQL execution. Note that
the actual SQL-generation tests are elsewhere (in test_build_sql.py)
"""
from arcwebbase import access_info
from cli.plugins.tests import test_extract
from django.conf import settings
from jobs.datacopy import DataCopyJobRunner
from unittest import mock
from vizlib.tests.test_querylog import get_logged_field

import django.test
import json


class TestDataCopyJobRunner(django.test.TestCase):

  def setUp(self):
    super().setUp()
    self.helper = test_extract.TestExtract()
    self.helper.setUp()
    self.extract = self.helper.make_extract()

  def tearDown(self):
    super().tearDown()
    access_info.clear()

  @mock.patch.object(settings, 'AUDIT_LOGGER')
  def test_audit_logging(self, logger: mock.Mock):

    target_data = {'extractId': self.extract.id, 'username': 'test_user_1'}
    runner = DataCopyJobRunner(json.dumps(target_data), None, None, None)

    logger.info.reset_mock()
    with mock.patch.object(runner, '_run_work'):
      runner.run()

    self.assertTrue(get_logged_field(
      logger.info, 'query').startswith('Data Extract: '))
    self.assertEquals(get_logged_field(logger.info, 'username'), 'test_user_1')

    # access info has priority:
    access_info.store(access_info.AccessInfo(username='test_user_2'))
    logger.info.reset_mock()
    with mock.patch.object(runner, '_run_work'):
      runner.run()

    self.assertEquals(get_logged_field(logger.info, 'username'), 'test_user_2')
