# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from django.contrib.auth.models import User
from unittest import mock

import django.test
import json

from jobs import views_helpers
from jobs.views_encryption import encrypt_secret
from reports.views_settings import setting_update_single


def save_setting(key: str, value: str):
  setting_update_single(key.lower(), value, False)


class TestSetupEmail(django.test.TestCase):

  @mock.patch.object(views_helpers, 'settings')
  def test_setup_with_no_arguments(self, settings):

    # if settings are stored then setup can be invoked without arguments
    save_setting('EMAIL_HOST', 'test-host')
    save_setting('EMAIL_PORT', '1234')
    save_setting('EMAIL_HOST_USER', 'test-user')
    save_setting('EMAIL_HOST_PASSWORD', json.dumps(
      encrypt_secret('test-password')))
    views_helpers.setup_email()

    self.assertEqual(settings.EMAIL_HOST, 'test-host')
    self.assertEqual(settings.EMAIL_PORT, 1234)
    self.assertEqual(settings.EMAIL_HOST_USER, 'test-user')
    self.assertEqual(settings.EMAIL_HOST_PASSWORD, 'test-password')

    # arguments have priority
    values = {
      'EMAIL_HOST': 'test-host-2',
      'EMAIL_PORT': '12345',
      'EMAIL_HOST_USER': 'test-user-2',
      'EMAIL_HOST_PASSWORD': 'test-password-2'
    }
    views_helpers.setup_email(values)

    self.assertEqual(settings.EMAIL_HOST, 'test-host-2')
    self.assertEqual(settings.EMAIL_PORT, 12345)
    self.assertEqual(settings.EMAIL_HOST_USER, 'test-user-2')
    self.assertEqual(settings.EMAIL_HOST_PASSWORD, 'test-password-2')

    # empty password can be used:
    values = {
      'EMAIL_HOST_PASSWORD': ''
    }
    views_helpers.setup_email(values)
    self.assertEqual(settings.EMAIL_HOST_PASSWORD, '')

    # when using the masked star values, the previous password saved in the DB is used:
    values = {
      'EMAIL_HOST_PASSWORD': settings.DEFAULT_PASSWORD_PLACEHOLDER
    }
    views_helpers.setup_email(values)
    self.assertEqual(settings.EMAIL_HOST_PASSWORD, 'test-password')
