# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from apps.tests.userrole import base
from jobs import models
from jobs.tests import test_jobs_views
from jobs.views_schedulegroups import schedule_group_get
from vizlib.tests.test_report_count import run_with_db_query_check2

import django.test
import django.db


class TestSchedulegroupsView(django.test.TestCase):

  def test_schedulegroup_get(self):
    admin1 = base.make_user('admin1', is_superuser=True)

    group_1 = models.ScheduleGroup(
      schedule_name='Test Schedule', creator=admin1)
    group_1.save()

    # there are a number of predefined schedule groups (in addition to the above created one):
    groups = models.ScheduleGroup.objects.all()
    self.assertGreater(len(groups), 1)

    util = test_jobs_views.TestJobsPage()
    util.create_job_schedule_and_log(groups[1], admin1)

    result = run_with_db_query_check2(
      lambda: schedule_group_get(id=group_1.id))
    self.assertEquals(len(result), 1)
    self.assertIsNotNone(result[0].get('jobschedule_set'))
    self.assertEquals(len(django.db.connection.queries),
                      2, 'details require 2 queries')

    result = run_with_db_query_check2(
      lambda: schedule_group_get(id=group_1.id, omit_details=True))
    self.assertIsNone(result[0].get('jobschedule_set'))
    self.assertEquals(len(django.db.connection.queries),
                      1, '1 query if no details')

    result = run_with_db_query_check2(lambda: schedule_group_get())
    self.assertEquals(len(result), len(groups))
    self.assertTrue(any([group.get('jobschedule_set') for group in result]))
    self.assertEquals(len(django.db.connection.queries),
                      2, 'details require 2 queries')

    result = run_with_db_query_check2(
      lambda: schedule_group_get(omit_details=True))
    self.assertEquals(len(result), len(groups))
    self.assertFalse(any([group.get('jobschedule_set') for group in result]))
    self.assertEquals(len(django.db.connection.queries),
                      1, '1 query if no details')
