# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.urls import re_path

from jobs import views, views_schedulegroups

urlpatterns = [
  re_path(r'^email$', views.email_snapshot, name='jobs-emailsnapshot'),
  re_path(r'^email/(?P<app_id>\d+)$', views.email_snapshot),
  re_path(r'^schedulescript$', views.scriptSchedule, name='jobs-schedulescript'),

  re_path(r'^scheduledatacopy$', views.dataCopySchedule, name='jobs-datacopy'),
  re_path(r'^datacopylogs$', views.dataCopyLogs, name='jobs-datacopy-logs'),
  re_path(r'^datacopyjobs$', views.dataCopyJobs, name='jobs-datacopy-jobs'),

  re_path(r'^viewrefresh$', views.viewrefresh, name='jobs-viewrefresh'),
  re_path(r'^schedulelog$', views.job_logs, name='jobs-joblogs'),
  re_path(r'^schedulelog/(?P<schedule_id>\d+)$', views.job_logs),
  re_path(r'^schedulegroup$', views_schedulegroups.schedule_group, name='jobs-schedulegroup'),
  re_path(r'^schedulegroup/(?P<schedule_group_id>\d+)$', views_schedulegroups.schedule_group),

  #Maybe need a URL for fetching schedules within a date range?
  re_path(r'^adminlogs$', views.render_job_overview_page, name='jobs-render-job-overview-page'),
  re_path(r'^customemails$', views.custom_email, name='jobs-customemail'),
  re_path(r'^customemails/(?P<template_id>\d+)$', views.custom_email),
  re_path(r'^previewcustomemails$', views.preview_custom_email, name='jobs-previewcustomemail'),
  re_path(r'^previewcustomemails/(?P<template_id>\d+)$', views.preview_custom_email),
  re_path(r'^job$', views.render_job_page, name='jobs-render-job-page'),
  re_path(r'^job/(?P<schedule_id>\d+)$', views.render_job_page),
  re_path(r'^rerun$', views.rerun_schedules_list, name='jobs-rerun'),
  re_path(r'^pauseorresume$', views.pause_or_resume, name='jobs-pauseorresume'),
  re_path(r'^editjob$', views.edit_job, name='jobs-editjob'),
  re_path(r'^editjob/(?P<schedule_id>\d+)$', views.edit_job),
  re_path(r'^editjobowner$', views.edit_job_owner, name='jobs-editjobowner'),
  re_path(r'^editjobowner/(?P<schedule_id>\d+)$', views.edit_job_owner),
  re_path(r'^canceljob$', views.cancel_job, name='jobs-canceljob'),
  re_path(r'^canceljob/(?P<schedule_id>\d+)$', views.cancel_job),
  re_path(r'^canceljob/(?P<schedule_id>\d+)/(?P<job_log_id>\d+)$', views.cancel_job),
  re_path(r'^testemail$', views.test_email, name='jobs-testemail'),
  re_path(r'^emailtemplates$', views.get_email_templates, name='jobs-email-templates'),
  re_path(r'^api/run/', views.api_rerun_schedules_list),
  re_path(r'^api/extracts/create$', views.api_extracts_create),
  re_path(r'^api/extracts/run$', views.api_extracts_run),
  re_path(r'^api/logs/latest$', views.get_latest_logs,  name='jobs-latest-logs-api'),
  re_path(r'^api/jobs$', views.jobs, name='jobs-api'),
  re_path(r'^api/jobs/(?P<schedule_id>\w+)$', views.get_job,  name='jobs-get-job'),
  re_path(r'^api/jobs/(?P<schedule_id>(\w+))/logs$', views.get_job_logs_for_schedule, name='jobs-get-logs'),
  re_path(r'^api/jobs/(?P<schedule_id>(\w+))/status$', views.get_job_status, name='jobs-get-status'),
]
