# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals
import json

from django.db import migrations

def add_sys_misc_priv_to_sys_admins(apps, schema):
  Role = apps.get_model('rbac', 'role')

  for role in Role.objects.filter(name='System Admin'):
    add_sys_misc_priv(role)

def add_sys_misc_priv(role):
  role_privileges = json.loads(role.privileges)
  role_system_perms = role_privileges[0]['perms']

  if 'sys_misc' not in role_system_perms:
    role_privileges[0]['perms'].append('sys_misc')
    role.privileges = json.dumps(role_privileges)
    role.save()

class Migration(migrations.Migration):
  dependencies = [
    ('rbac', '0005_adding_view_perm'),
  ]
  operations = [
    migrations.RunPython(
      add_sys_misc_priv_to_sys_admins,
      migrations.RunPython.noop,
    ),
  ]