# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

# Permission definitions RBAC

from apps.templatetags.site_setting_name import site_setting_name

PERMDEFS = {

  'sys_editperm' : 'Manage roles and users',
  'sys_settings' : f'Manage {site_setting_name(False)}',
  'sys_styles'   : 'Manage custom styles',
  'sys_viewlogs' : 'View activity logs',
  'sys_schedules': 'Manage jobs, email templates',
  'sys_editconn' : 'Manage data connections',
  'sys_workspace': 'Create workspaces',
  'sys_viewroles': 'View roles and users',
  'sys_misc'     : 'Additional system privilege',

  # Note that these role type permissions provide ability to 'grant' of the
  # the corresponding dataset type permission.  A couple places in code expect
  # 'rg' + <dataset permission name> = <grant permission name>
  'rgds_manage'  : 'Grant manage dataset',
  'rgds_appedit' : 'Grant manage dashboards',
  'rgds_appview' : 'Grant view dashboards',

  'dc_aviews'    : 'Manage AVs/Extracts',
  'dc_upload'    : 'Import data',
  'dc_explore'   : 'Create datasets, explore tables',

  'ds_manage'    : 'Manage dataset',
  'ds_appedit'   : 'Manage dashboards',
  'ds_appview'   : 'View dashboards',

  # Note that workspace permissions are currently managed via ACLs
  # within the workspace objects themselves, not within roles.
  'ws_manage'    : 'Manage workspace',
  'ws_appedit'   : 'Manage workspace dashboards',
  'ws_appview'   : 'View workspace dashboards',
}

# Supported permission types
PERMTYPES = ['system', 'role', 'dataconn', 'dataset']

# Note that order here dictates the order these are displayed in the privilege table
PERMLISTS = {

  'system'   : ['sys_workspace', 'sys_viewroles', 'sys_editperm', 'sys_settings', 'sys_styles', 'sys_schedules',
                'sys_viewlogs', 'sys_editconn', 'sys_misc'],
  'role'     : ['rgds_manage', 'rgds_appedit', 'rgds_appview'],
  'dataconn' : ['dc_aviews', 'dc_upload', 'dc_explore'],
  'dataset'  : ['ds_manage', 'ds_appedit', 'ds_appview'],
  'workspace': ['ws_manage', 'ws_appedit', 'ws_appview'],

  # These are the dependenty lists--some permissions force others as well
  'sys_editperm': ['sys_viewroles'],

  'ds_manage'  : ['ds_appedit', 'ds_appview'],
  'ds_appedit' : ['ds_appview'],

  'dc_upload'  : ['dc_explore'],
  'dc_aviews'  : ['dc_explore'],

  'rgds_manage' : ['rgds_appedit', 'rgds_appview'],
  'rgds_appedit': ['rgds_appview'],
}

# Permission Exception
class PermissionException(Exception):
  def __init__(self, user, perm, obj=None):
    self.user = user
    self.perm = perm
    self.obj = obj

  def __str__(self):
    return "You do not have the necessary permission on this %s" % \
           ('system' if self.obj is None else self.obj.perm_type())

