# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import logging

from arcwebbase.async_util import django_submit

def store_record(record: logging.LogRecord):
  from reports.notification_center import NotificationCenter
  stacktrace = record.exc_text if record.exc_text else ''
  NotificationCenter().add_errors([{'title': record.message, 'body': stacktrace}], False)

class NotificationCenterHandler(logging.Handler):
  level = logging.ERROR

  # Docs to refer to later in case of changes
  # https://docs.python.org/3/library/logging.html#logging.Handler
  def handle(record):

    # with Django 3 we get SynchronousOnlyOperation exception if trying to access
    # models from an async function
    from django.core.exceptions import SynchronousOnlyOperation
    try:
      store_record(record)
    except SynchronousOnlyOperation:
      # we submit execution but do not wait for it
      django_submit(store_record, record)

  def get_name():
    return 'NotificationCenterHandler'
