# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion
import uuid
import arcwebbase.validator


class Migration(migrations.Migration):

    dependencies = [
        ('datasets', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('workspace', '__first__'),
    ]
    operations = [
        migrations.CreateModel(
        name='AppGroup',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('group_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('group_type', models.CharField(max_length=100, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('group_username', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('group_subgroup_order', models.FloatField(blank=True, null=True)),
            ('group_visible', models.BooleanField(default=True)),
            ('group_options', models.TextField(max_length=4096, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
            ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('update_time', models.DateTimeField(null=True, auto_now=True)),
            ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
            ('group_parent', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.AppGroup')),
        ],
    ),
    migrations.CreateModel(
        name='AppGroupAppMembership',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('appgroup_order', models.FloatField(blank=True, null=True)),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
        ],
    ),
    migrations.CreateModel(
        name='AppSetting',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('setting_key', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('setting_value', models.TextField(max_length=16000, blank=True, null=True)),
            ('setting_read_admin_only', models.BooleanField(default=True)),
        ],
    ),
    migrations.CreateModel(
        name='ColorPalette',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('colorpalette_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('colorpalette_value', models.TextField(max_length=16000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('colorpalette_type', models.IntegerField(blank=True, null=True)),
            ('colorpalette_description', models.TextField(max_length=16000, blank=True, null=True, default='', validators=[arcwebbase.validator.validateStringNoHtml])),
        ],
    ),
    migrations.CreateModel(
        name='CustomCSS',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('customcss_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('customcss_description', models.TextField(max_length=16000, blank=True, null=True, default='', validators=[arcwebbase.validator.validateStringNoHtml])),
            ('customcss_value', models.TextField(max_length=16000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('customcss_js', models.TextField(max_length=16000, blank=True, null=True)),
            ('customcss_type', models.IntegerField(blank=True, null=True)),
            ('customcss_default', models.BooleanField(default=False)),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
        ],
    ),
    migrations.CreateModel(
        name='DateRanges',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('dateranges_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('dateranges_value', models.TextField(max_length=16000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('dateranges_type', models.IntegerField(blank=True, null=True)),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
        ],
    ),
    migrations.CreateModel(
        name='ParameterSet',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('parameterset_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('parameterset_value', models.TextField(max_length=16000, blank=True, null=True)),
            ('parameterset_type', models.IntegerField(blank=True, null=True)),
        ],
    ),
    migrations.CreateModel(
        name='Report',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('report_name', models.CharField(max_length=1000, blank=True)),
            ('report_description', models.TextField(max_length=2000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('report_type', models.CharField(max_length=100, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('report_mode', models.CharField(max_length=100, blank=True, null=True, default='', validators=[arcwebbase.validator.validateStringNoHtml])),
            ('report_data', models.TextField(max_length=32768)),
            ('shared_visual_dashboards', models.TextField(max_length=32768, blank=True, null=True)),
            ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
            ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('update_time', models.DateTimeField(null=True, auto_now=True)),
            ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
            ('dataset', models.ForeignKey(blank=True, null=True, on_delete=models.CASCADE, to='datasets.DataSet')),
            ('workspace', models.ForeignKey(blank=True, null=True, default=1, on_delete=django.db.models.deletion.SET_DEFAULT, to='workspace.Workspace')),
        ],
    ),
    migrations.CreateModel(
        name='ReportImage',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('img', models.BinaryField(blank=True, null=True)),
            ('update_time', models.DateTimeField(null=True, auto_now=True)),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
            ('report', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.Report')),
        ],
    ),
    migrations.CreateModel(
        name='ReportUsage',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('timewindow_start', models.DateTimeField(blank=True, null=True)),
            ('count', models.IntegerField(blank=True, null=True)),
            ('report', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.Report')),
        ],
    ),
    migrations.CreateModel(
        name='ReportUser',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('order', models.IntegerField(blank=True, null=True)),
            ('report', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.Report')),
            ('user', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to=settings.AUTH_USER_MODEL)),
        ],
    ),
    migrations.CreateModel(
        name='StaticAsset',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('staticasset_mimetype', models.CharField(max_length=100, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('staticasset_name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('staticasset_description', models.TextField(max_length=16000, blank=True, null=True, default='', validators=[arcwebbase.validator.validateStringNoHtml])),
            ('staticasset_size', models.IntegerField(blank=True, null=True)),
            ('staticasset_apps', models.TextField(max_length=16000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('staticasset_contents', models.TextField(max_length=104857600, blank=True, null=True)),
            ('create_time', models.DateTimeField(auto_now_add=True)),
            ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('update_time', models.DateTimeField(auto_now=True)),
            ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('uuid', models.CharField(max_length=36, unique=True, default=uuid.uuid4)),
            ('imported_uuid', models.CharField(max_length=100, blank=True, null=True)),
        ],
    ),
    migrations.CreateModel(
        name='UserAppGroupMembership',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('usergroup_username', models.CharField(max_length=100, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('usergroup_order', models.FloatField(blank=True, null=True)),
            ('usergroup_group', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.AppGroup')),
        ],
    ),
    migrations.AddField(
        model_name='appgroupappmembership',
        name='appgroup_app',
        field=models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.Report'),
    ),
    migrations.AddField(
        model_name='appgroupappmembership',
        name='appgroup_group',
        field=models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.AppGroup'),
    ),
    ]
