# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion
from django.db import connection
import uuid
import arcwebbase.validator


CREATE_TEMPLATE_MODEL = [
    migrations.CreateModel(
        name='ReportTemplate',
        fields=[
            ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
            ('name', models.CharField(max_length=1000, blank=True)),
            ('description', models.TextField(max_length=2000, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('data', models.TextField(max_length=32768)),
            ('params', models.TextField(max_length=32768)),
            ('img', models.BinaryField(blank=True, null=True)),
            ('create_time', models.DateTimeField(null=True, auto_now_add=True)),
            ('create_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('update_time', models.DateTimeField(null=True, auto_now=True)),
            ('update_user', models.CharField(max_length=200, blank=True, validators=[arcwebbase.validator.validateStringNoHtml])),
            ('report', models.ForeignKey(on_delete=models.CASCADE, blank=True, null=True, to='reports.Report')),
        ],
    ),
]

def get_operation_list():
    if 'reports_reporttemplate' in connection.introspection.table_names():
        return []
    else:
        return CREATE_TEMPLATE_MODEL

class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0001_initial'),
    ]

    state_operations = CREATE_TEMPLATE_MODEL

    operations = [
        migrations.SeparateDatabaseAndState(
            database_operations=get_operation_list(),
            state_operations=state_operations)]
