# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# Generated by Django 1.11.12 on 2018-05-18 18:48
from __future__ import unicode_literals

from django.db import migrations, models
from uuid import UUID, uuid4
from migration.migrate import addDashboardSheetData
from arcwebbase.migration_helper import south_migration_applied
from django.conf import settings
import django.db.models.deletion
import json, os, logging, tempfile

def add_fields_if_necessary(apps, schema_editor):
    Report = apps.get_model('reports', 'report')
    try:
        r = Report.objects.all().values_list('report_mode')
        len(r)
    except:
        field = Report._meta.get_field('report_mode')
        schema_editor.add_field(Report, field)

    try:
        r = Report.objects.all().values_list('shared_visual_dashboards')
        len(r)
    except:
        field = Report._meta.get_field('shared_visual_dashboards')
        schema_editor.add_field(Report, field)

    ReportTemplate = apps.get_model('reports', 'reporttemplate')
    try:
        r = ReportTemplate.objects.all().values_list('report')
        len(r)
    except:
        field = Report._meta.get_field('reporttemplate')
        schema_editor.add_field(Report, field)

def make_dashboard_centric(apps, schema_editor):
    Report = apps.get_model('reports', 'report')

    # In the case of 4.2.0 and 4.2.1 don't apply make_dashboard_centric
    if south_migration_applied(
        '0034_auto__add_field_report_report_mode__add_field_reporttemplate_report'):
        logging.info('make dashboard centric function is skipped')

        if not south_migration_applied('0036_update_report_data_if_missing_sheet_id_dashboard_id'):
            logging.info('Running cleanup report')

            from util.cleanup_reports import update_reports_data
            update_reports_data(Report, logging)
            return

    from util.cleanup_reports import update_reports_data
    logging.info('running dashboard centric function')
    ReportImage = apps.get_model('reports', 'reportimage')
    AppGroupAppMembership = apps.get_model('reports', 'appgroupappmembership')

    # Create new dashboard with single visual
    #   add it to dashboard, make full page, duplicate thumbnail,
    #   same update_user and create_user, click_behavior and csv/email/png settings
    #   take on visual's title, but hide it
    def create_single_visual_dashboard(viz_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard):
        new_dashboard = Report.objects.filter(id=viz_id)
        if not new_dashboard.exists():
            return None

        other_dashboards = visual_id_to_dashboards.get(viz_id)
        number_of_dashboards = len(other_dashboards)
        new_dashboard = new_dashboard.first()
        new_dashboard.pk = None
        new_dashboard.uuid = uuid4()
        new_dashboard.imported_uuid = None

        if number_of_dashboards > 1:
            new_dashboard.shared_visual_dashboards = '[%s]' % viz_id

            if number_of_dashboards == 1:
                for other_dash_id in other_dashboards:
                    try:
                        other_dashboard = Report.objects.get(id=other_dash_id)
                        new_linked_visuals = set(json.loads(other_dashboard.shared_visual_dashboards or '[]'))
                        new_linked_visuals.add(viz_id)
                        other_dashboard.shared_visual_dashboards = json.dumps(list(new_linked_visuals))
                        other_dashboard.save()
                        logging.info(
                            'Added visual %s to dashboard %s shared_visual_dashboards' % (
                            viz_id, other_dash_id))

                    except:
                        logging.info(
                            'Dashboard %s no longer exists' % other_dash_id)

        else:
            new_dashboard.shared_visual_dashboards = '[]'

        # Update report_data
        viz_json = json.loads(new_dashboard.report_data).get(
            'report_data', {})
        viz_settings = viz_json.get('user_settings', {})
        dashboard_json = {
            'report_title': viz_json.get('report_title', ''),
            'numColumns': 64,
            'dashboard_widgets': [{
                'col': 1,
                'row': 1,
                'size_x': 64,
                'size_y': 30,
                # approximate height visual should be to be full screen
                'id': 'uri-1-widget-{0}'.format(viz_id)
            }],
        }

        new_user_setting = {}
        if new_dashboard.report_type in ['extension', 'html', 'link']:
            new_user_setting = {
                # Settings needed to make visual full screen
                'dashboard_width': '',
                'single_visual_dashboard': 'true',
                'horizontal_margin_between_visuals': '0',
                'vertical_margin_between_visuals': '0',
            }

        # Hide title and border around visuals
        new_user_setting['borders_around_visuals'] = 'false'
        new_user_setting['dashboard_hide_title'] = 'true'

        # Settings to move over from visual
        new_user_setting['click_behavior'] = viz_json.get(
            'click_behavior', {})

        new_dashboard.report_type = 'dashboard'

        csv_setting = viz_settings.get('permit_csv_download_visual')
        if csv_setting:
            new_user_setting[
                'permit_csv_download_dashboard'] = csv_setting

        email_setting = viz_settings.get('permit_email_report')
        if email_setting:
            new_user_setting['permit_email_report'] = email_setting

        png_setting = viz_settings.get('permit_png_download')
        if png_setting:
            new_user_setting['permit_png_download'] = png_setting

        dashboard_json['user_settings'] = new_user_setting
        new_dashboard.report_data = json.dumps(
            addDashboardSheetData(dashboard_json))

        new_dashboard.save()

        # This also adds to visual_id_to_dashboards.get(viz_id)
        other_dashboards.add(new_dashboard.id)

        current_viz = Report.objects.filter(id=viz_id)
        if current_viz.exists():
            # Add dashboard to single_visual_dashboard if visual is linked
            if number_of_dashboards >= 1:
                current_viz = current_viz.first()
                current_viz_json = json.loads(current_viz.report_data)
                current_viz_json['report_data'].pop('dashboard_id', None)
                current_viz.report_data = json.dumps(current_viz_json)
                current_viz.shared_visual_dashboards = json.dumps(list(other_dashboards))
            elif number_of_dashboards == 0:
                current_viz = current_viz.first()
                current_viz_json = json.loads(current_viz.report_data)
                current_viz_json['report_data'][
                    'dashboard_id'] = new_dashboard.id
                current_viz.report_data = json.dumps(current_viz_json)

            current_viz.save()

        new_thumbnail = ReportImage.objects.filter(report_id=viz_id)
        if new_thumbnail.exists():
            new_thumbnail = new_thumbnail.first()
            new_thumbnail.pk = None
            new_thumbnail.uuid = uuid4()
            new_thumbnail.imported_uuid = None
            new_thumbnail.report = new_dashboard
            new_thumbnail.save()

        dashboard_id_to_visuals[new_dashboard.id] = set([viz_id])
        visual_id_to_full_screen_dashboard[viz_id] = new_dashboard.id
        return new_dashboard

    def handle_unused_visual(viz_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard):
        # Remove picklist if it's not used in any visuals
        # Shouldn't need
        if picklist_ids.get(viz_id):
            logging.info('deleting picklist ID %s' % viz_id)
            Report.objects.filter(id=viz_id).delete()

        # Create single visual dashboard
        else:
            logging.info('converting visual ID %s to single visual dashboard' % viz_id)
            create_single_visual_dashboard(viz_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard)

    def handle_visuals_in_apps(visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard):
        # - If single visual is used in appgroups, change single visual to full page visual
        all_appgroup_memberships = AppGroupAppMembership.objects.values(
            'id', 'appgroup_app')
        for agm in all_appgroup_memberships:

            # If it's a visual, assign a single visual dashboard if it's already created
            # If not created already, create one and assign (if the visual still exists)
            appgroup_report_id = agm['appgroup_app']
            try:
                list_of_dashboards = visual_id_to_dashboards.get(appgroup_report_id)
                if list_of_dashboards or list_of_dashboards == set([]):
                    logging.info('App group report  %s in membership %s is visual' %
                                 (appgroup_report_id, agm['id']))
                    appgroup_instance = AppGroupAppMembership.objects.filter(id=agm['id'])
                    full_screen_dashboard_id = visual_id_to_full_screen_dashboard.get(
                        appgroup_report_id)
                    if full_screen_dashboard_id or full_screen_dashboard_id == 0:
                        appgroup_instance.update(
                            appgroup_app=full_screen_dashboard_id)
                    else:
                        logging.info('converting visual ID %s to single'
                                     ' visual dashboard through appgroup' % appgroup_report_id)
                        new_report = create_single_visual_dashboard(appgroup_report_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard)
                        if new_report:
                            appgroup_instance.update(
                                appgroup_app=new_report)
                        else:
                            # Visual no longer exists. Delete the visual membership
                            logging.info('deleting appgroup_membership ID: %s' % appgroup_instance.id)
                            appgroup_instance.delete()

            except Exception as e:
                msg = 'ERROR: Cannot update appgroup report ID: %s' % appgroup_report_id
                print(msg)
                print(str(e))
                logging.exception(msg)

    # Run main update function
    # create shared visuals or make single visual dashboards
    # - If visual used in just 1 dashboard, add dashboard_id to report_data
    # - If single visual is not used in any dashboard, create dashboard
    # - If visual used in 2 or more dashboards, make shared visual
    update_reports_data(Report, logging, handle_unused_visual, handle_visuals_in_apps)

def update_shared_visual(apps, schema_editor):
    if south_migration_applied(
      '0035_delete_empty_string_shared_visual_dashboard'):
        logging.info('update shared visual function is skipped')
        return
    logging.info('running update shared visual function')
    Report = apps.get_model('reports', 'report')
    Report.objects.filter(shared_visual_dashboards='') \
        .update(shared_visual_dashboards=None)


def get_operation_list():
    if not south_migration_applied('0031_auto__add_field_report_workspace'):
        return [migrations.AddField(
            model_name = 'report',
            name = 'workspace',
            field = models.ForeignKey(blank=True, default=1, null=True,
                                      on_delete = django.db.models.deletion.SET_DEFAULT, to = 'workspace.Workspace'),
        )]
    else:
        return []

class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ('reports', '0002_initial'),
    ]

    operations = get_operation_list()
    operations += [
        migrations.RunPython(add_fields_if_necessary),
        migrations.RunPython(make_dashboard_centric),
        migrations.RunPython(update_shared_visual)
    ]
