# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# Generated by Django 1.11.4 on 2018-07-19 20:49
from __future__ import unicode_literals
from django.db import migrations, models
import django.db.models.deletion
from uuid import UUID, uuid4
from django.db.models import F


# If appgroup is parent appgroup, move all children reports to subgroup
# And make that subgroup first
def update_parentgroup(parent_group):
    if not parent_group.group_parent and parent_group.group_type == 'default':
        # Return if the parent appgroup does not have any dashboard
        if not parent_group.appgroupappmembership_set.count():
            return

        parent_group.subgroups.update(group_subgroup_order=F('group_subgroup_order') + 1)
        new_group = parent_group.subgroups.create(
            group_name=parent_group.group_name,
            group_username=parent_group.group_username,
            create_time=parent_group.create_time,
            create_user=parent_group.create_user,
            update_time=parent_group.update_time,
            update_user=parent_group.update_user,
            workspace_id=parent_group.workspace_id,
            group_type='default',
            group_options='{}',
            group_subgroup_order=1,
            imported_uuid='upgrade-auto'
        )

        parent_group.appgroupappmembership_set.update(appgroup_group_id=new_group.id)


def cleanup_appgroups(apps, schema_editor):
    import logging, os, tempfile
    from workspace.models import get_default_workspace
    from django.db.models import Q

    AppGroup = apps.get_model('reports', 'appgroup')
    User = apps.get_model('auth', 'user')
    for appgroup in AppGroup.objects.all():
      if not appgroup.group_visible:
          try:
              private_ws = get_default_workspace(User.objects.get(username=appgroup.create_user))
              appgroup_id = appgroup.id
              AppGroup.objects.filter(Q(id=appgroup_id) | Q(group_parent__id=appgroup_id))\
                              .update(workspace=private_ws)
          except Exception as e:
              logging.info("Not able to move app %s to %s's private workspace" % (appgroup.id, appgroup.create_user))
              logging.info(str(e))

      update_parentgroup(appgroup)

class Migration(migrations.Migration):

    atomic = False

    dependencies = [
        ('workspace', '0001_initial'),
        ('reports', '0004_auto_20180713_1819'),
    ]

    operations = [
        migrations.AddField(
            model_name='appgroup',
            name='workspace',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.SET_DEFAULT, to='workspace.Workspace'),
        ),
        migrations.AlterField(
            model_name='appgroup',
            name='group_parent',
            field=models.ForeignKey(blank=True, null=True,
                                    on_delete=django.db.models.deletion.CASCADE,
                                    related_name='subgroups', to='reports.AppGroup'),
        ),
        migrations.RunPython(cleanup_appgroups),
        migrations.RemoveField(
            model_name='appgroup',
            name='group_visible',
        ),

    ]
