# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations

FEATURES = [
  'site_enable_search',
  'site_enable_search_menu',
  'site_enable_search_visual',
]

def to_boolean(val):
  return val.lower() == 'true'

def move_to_feature_flipper(apps, schema_editor):
  AppSetting = apps.get_model('reports', 'appsetting')
  AppFeatureFlipper = apps.get_model('reports', 'appfeatureflipper')

  for feature in FEATURES:
      setting = AppSetting.objects.filter(setting_key=feature).first()
      if setting:
        flipper = AppFeatureFlipper(id=feature,
                                    value=to_boolean(setting.setting_value))
        flipper.save()
        setting.delete()


class Migration(migrations.Migration):
  dependencies = [
    ('reports', '0017_appfeatureflipper'),
  ]
  operations = [
    migrations.RunPython(
      move_to_feature_flipper,
      migrations.RunPython.noop,
    ),
  ]
