# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations
import json
from reports.models import get_visuals_picklist

def associate_dashboards_to_datasets(apps, schema_editor):
    DataSet = apps.get_model('datasets', 'dataset')
    Report = apps.get_model('reports', 'report')

    datasets = DataSet.objects.all()
    datasets_dashboards_dict = {dataset.id: [] for dataset in datasets}
    dashboards = Report.objects.filter(report_type="dashboard")

    for dashboard in dashboards:
        dashboard_data = json.loads(dashboard.report_data)
        (visuals, _) = get_visuals_picklist(dashboard_data)
        reports_in_dashboard = Report.objects.filter(id__in=visuals)
        dashboard_datasets = list(set([report.dataset_id for report in reports_in_dashboard]))
        for dataset in dashboard_datasets:
            datasets_dashboards_dict[dataset].append(dashboard.id)

    for dataset_id, dashboards_ids in datasets_dashboards_dict.items():
        ds = DataSet.objects.get(id=dataset_id)
        ds.dashboards = json.dumps(dashboards_ids)
        ds.save()

class Migration(migrations.Migration):
    dependencies = [
        ("reports", "0023_auto_20210909_0725"),
        ("datasets", "0008_dataset_dashboards"),
    ]
    operations = [
        migrations.RunPython(
            associate_dashboards_to_datasets,
            migrations.RunPython.noop,
        ),
    ]
