# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals

from django.db import migrations
import json, os, tempfile, math, copy

# Make sure site_thumbnails_enable is not admin only
def update_email_protocol(apps, schema_editor):
  AppSetting = apps.get_model('reports', 'appsetting')

  ssl_key = 'email_use_ssl'
  tls_key = 'email_use_tls'

  ssl = AppSetting.objects\
  .filter(setting_key=ssl_key)\
  .first()

  tls = AppSetting.objects\
  .filter(setting_key=tls_key)\
  .first()

  protocol = 'None'

  if ssl is not None:
    if ssl.setting_value == 'true':
      protocol = ssl_key.upper()
  if tls is not None:
    if tls.setting_value == 'true':
      protocol = tls_key.upper()

  if ssl is not None or tls is not None:
    email_protocol = AppSetting(setting_key='email_protocol', setting_value=protocol)
    email_protocol.save()

def migrate_data_backward(apps, schema_editor):
  pass

class Migration(migrations.Migration):
  dependencies = [
    ('reports', '0025_set_extracts_setting_from_advanced_settings'),
  ]

  operations = [
    migrations.RunPython(
      update_email_protocol,
      migrate_data_backward,
    ),
  ]
