# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.db import migrations


def group_duplicated_notifications(apps, schema_editor):
    notifications = apps.get_model('reports', 'appnotificationcenter')
    for notification in notifications.objects.all():
        notification.timestamps = {'timestamps': [int(notification.created_time.timestamp() * 1000)]}
        notification.save()

    for notification in notifications.objects.all().order_by('title', 'body'):
        for cmp_notification in notifications.objects.all().order_by('title', 'body', '-created_time'):
            is_same_notification = notification.id == cmp_notification.id
            if notification.title == cmp_notification.title and notification.body == cmp_notification.body and not is_same_notification:
                notification.timestamps['timestamps'].insert(0, int(cmp_notification.created_time.timestamp() * 1000))
                cmp_notification.delete()
        notification.save()


class Migration(migrations.Migration):
    dependencies = [
        ('reports', '0035_appnotificationcenter_timestamps'),
    ]
    operations = [
        migrations.RunPython(
            group_duplicated_notifications,
            migrations.RunPython.noop,
        ),
    ]
