# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.db import migrations
from django.db.models import Q
from reports.models import get_visuals_picklist
import json
def removeVisualsOrFiltersWithoutDashboard(apps, schema_editor):
  Report = apps.get_model('reports', 'Report')
  dashboards = Report.objects.filter(report_type='dashboard')
  visual_and_report_ids_present_in_dashboards = set()
  for dashboard in dashboards:
    (visualIds, filterIds) = get_visuals_picklist(json.loads(dashboard.report_data))
    visual_and_report_ids_present_in_dashboards.update(visualIds)
    visual_and_report_ids_present_in_dashboards.update(filterIds)
  
  reportsNotInDashboard = Report.objects.exclude(report_type='dashboard') \
                                        .exclude(id__in=list(visual_and_report_ids_present_in_dashboards)) \
                                        .filter(Q(shared_visual_dashboards__isnull=True)) # Delete only non shared visuals
  if reportsNotInDashboard.count():
    try:
      reportsNotInDashboardIds = list(reportsNotInDashboard.values_list('id', flat=True))
      reportsNotInDashboard.delete()
      print('Orphan visuals and filters(ids: %s) were deleted, because no dashboard points to them' % reportsNotInDashboardIds)
    except Exception as e:
      print('Failed to delete some of the orphan visuals and filters, error: %s' % e)
    
class Migration(migrations.Migration):
    
    dependencies = [
        ("reports", '0037_remove_appnotificationcenter_created_time'),
    ]

    operations = [
        migrations.RunPython(removeVisualsOrFiltersWithoutDashboard, migrations.RunPython.noop),
    ]
