# Generated by Django 3.2.24 on 2024-07-25 11:02
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from django.db import migrations
import json
from apps.config import default_embeddings_engine_settings, default_completion_engine_settings
from reports.constants import FINAL_SETTINGS

def createDefaultAiProfilesFromOldData(apps, _):
        AppSetting = apps.get_model('reports', 'appsetting')
        settings = AppSetting.objects.all()
        settings_list = list(settings.values())
        
        settings_dict = {
            'ai_term_agreed': None,
            'ai_engine': FINAL_SETTINGS['ai_default_engine'],
            'ai_embeddings_settings': None,
            'ai_completion_settings': None,
            'embeddings_api_key': '',
            'completion_api_key': '',
            'embedding_service_url': '',
            'completion_service_url': '',
            'embedding_input_param':  FINAL_SETTINGS['embedding_input_param'],
            'embedding_response_param': FINAL_SETTINGS['embedding_response_param'],
            'completion_input_param': FINAL_SETTINGS['completion_input_param'],
            'completion_response_param': FINAL_SETTINGS['completion_response_param'],
            'completion_extra_arg': FINAL_SETTINGS['completion_extra_arg'],
            'ai_engine_settings': ""
        }
        
        for setting in settings_list:
            key = setting['setting_key']
            if key in settings_dict:
                settings_dict[key] = setting['setting_value']

        
        if settings_dict['ai_term_agreed'] and not settings_dict['ai_embeddings_settings'] and not settings_dict['ai_completion_settings']:
            def getSettings(engine: str):
                embeddings_settings = {
                    "embeddings_engine": settings_dict['ai_engine'],
                    "embeddings_api_key": settings_dict['embeddings_api_key'],
                }
                completion_settings = {
                    "completion_engine": settings_dict['ai_engine'],
                    "completion_api_key": settings_dict['completion_api_key'],
                }

                if engine == 'open_ai':
                    embeddings_settings['embeddings_model'] = "text-embedding-ada-002"

                    completion_settings['completion_model'] = "gpt-3.5-turbo"
                    completion_settings['completion_temperature'] = "1"

                elif engine == 'llama':
                    embeddings_settings['embeddings_service_url'] = settings_dict['embedding_service_url']
                    embeddings_settings['embeddings_input_param'] = settings_dict['embedding_input_param']
                    embeddings_settings['embeddings_response_param'] = settings_dict['embedding_response_param']
                    
                    completion_settings['completion_service_url'] = settings_dict['completion_service_url']
                    completion_settings['completion_input_param'] = settings_dict['completion_input_param']
                    completion_settings['completion_response_param'] = settings_dict['completion_response_param']
                    completion_settings['completion_extra_arg'] = settings_dict['completion_extra_arg']

                elif engine == 'other':
                    embeddings_settings['embeddings_service_url'] = settings_dict['embedding_service_url']
                    embeddings_settings['embeddings_custom_engine_settings'] = settings_dict['ai_engine_settings'] if settings_dict['ai_engine_settings'] else default_embeddings_engine_settings
                    
                    completion_settings['completion_service_url'] = settings_dict['completion_service_url']
                    completion_settings['completion_custom_engine_settings'] = settings_dict['ai_engine_settings'] if settings_dict['ai_engine_settings'] else default_completion_engine_settings
                    
                return embeddings_settings, completion_settings
                
            embeddings_settings, completion_settings = getSettings(settings_dict['ai_engine'])
                
            ai_embeddings_settings_value = json.dumps({
                "defaultProf": { "value": "default", "label": "Default" },
                "profileSettings": {
                    "default": {
                        "label": "Default",
                        "values": embeddings_settings
                    }
                }
            })
            
            ai_completion_settings_value = json.dumps({
                "defaultProf": { "value": "default", "label": "Default" },
                "profileSettings": {
                    "default": {
                        "label": "Default",
                        "values": completion_settings
                    }
                }
            })
            
            # Save new settings to the database
            AppSetting.objects.create(setting_key='ai_embeddings_settings', setting_value=ai_embeddings_settings_value, setting_read_admin_only=False)
            AppSetting.objects.create(setting_key='ai_completion_settings', setting_value=ai_completion_settings_value, setting_read_admin_only=False)


class Migration(migrations.Migration):
    dependencies = [
        ('reports', '0039_associate_dashboard_to_dataset_again'),
    ]

    operations = [
        migrations.RunPython(createDefaultAiProfilesFromOldData, migrations.RunPython.noop),
    ]
