# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.db import migrations
import json, os
from dataconn.cml_dependencies import get_cmldata

def set_caii_profile_cml_auth_mode_to_api_key(apps, _):
    if get_cmldata():
        AppSetting = apps.get_model('reports', 'appsetting')
        for tab in ['embeddings', 'completion']:
            ai_setting_obj = AppSetting.objects.filter(setting_key=f'ai_{tab}_settings').first()
            if ai_setting_obj:
                setting_value = json.loads(ai_setting_obj.setting_value)

                for profile_setting in setting_value['profileSettings'].values():
                    profileValues = profile_setting['values']
                    if profileValues[f'{tab}_engine'] == 'caii':
                        profileValues[f'{tab}_cml_auth_mode'] = 'api_key'
                        print(f'Set Cloudera AI authentication mode to default api_key value for profile {profile_setting["label"]}')

                ai_setting_obj.setting_value = json.dumps(setting_value)
                ai_setting_obj.save(update_fields=['setting_value'])

class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0043_update_sample_app_group_type'),
    ]

    operations = [
        migrations.RunPython(set_caii_profile_cml_auth_mode_to_api_key, migrations.RunPython.noop),
    ]
