# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from uuid import UUID
import json

def move_aggregates_to_color_shelf(report_obj):
    # This function moves the proper aggregates from the aggregate shelf
    # to the new color shelf. Also moves values from the dimensions
    # shelf into the tooltip shelf

    settings = report_obj.get('user_settings', {})
    circle_color_style = settings.get('circle-color-style', '')
    circle_marker_enabled = settings.get('circle_enabled', '')

    dimensions_shelf = report_obj.get('dimensions_shelf', None)
    tooltip_shelf = report_obj.get('tooltip_shelf', [])

    if dimensions_shelf:
        for dimension in dimensions_shelf:
            tooltip_shelf.append(dimension)
        del report_obj['dimensions_shelf']

    report_obj['tooltip_shelf'] = tooltip_shelf

    color_shelf = []
    aggregate_shelf = report_obj.get('aggregates_shelf', [])

    aggregate_to_move = None
    if len(aggregate_shelf) == 1 and circle_marker_enabled == 'true' and circle_color_style == 'color_palette':
        aggregate_to_move = aggregate_shelf[0]
        color_shelf.append(aggregate_to_move)
    elif len(aggregate_shelf) == 2:
        aggregate_to_move = aggregate_shelf[1]
        aggregate_shelf.remove(aggregate_to_move)
        color_shelf.append(aggregate_to_move)

    # These lines are probably not necessary, but include just in case the report_obj.get
    # returns new arrays
    report_obj['color_shelf'] = color_shelf
    report_obj['aggregates_shelf'] = aggregate_shelf

    core = report_obj.get('core', None)
    if core == None:
        return report_obj

    shelves = core.get('shelves', [])
    for shelf in shelves:
        if shelf['name'] == 'dimensions_shelf':
            shelf['name'] = 'color_shelf'
            shelf['column_type'] = 3

    saved_shelf_sources = core.get('saved_shelf_sources', {})
    for shelf_elem in list(saved_shelf_sources.keys()):
        shelf_name = saved_shelf_sources[shelf_elem]
        if shelf_name == 'dimensions_shelf':
            saved_shelf_sources[shelf_elem] = 'tooltip_shelf'
        elif shelf_name == 'aggregates_shelf' and aggregate_to_move:
            dataset_colname = aggregate_to_move.get('dataset_colname', '')
            if dataset_colname in shelf_elem:
                saved_shelf_sources[shelf_elem] = 'color_shelf'

    core['shelves'] = shelves
    core['saved_shelf_sources'] = saved_shelf_sources

    return report_obj

def move_colors_to_aggregate_shelf(report_obj):
    # This function moves the proper aggregates from the color shelf
    # to the aggregate shelf. Also moves dimensions from the tooltips
    # shelf into the dimensions shelf

    dimensions_shelf = []
    tooltip_shelf = report_obj.get('tooltip_shelf', [])

    for tooltip in tooltip_shelf:
        if tooltip.get('fn_name', None) == None:
            dimensions_shelf.append(tooltip)
            tooltip_shelf.remove(tooltip)

    report_obj['dimensions_shelf'] = dimensions_shelf
    report_obj['tooltip_shelf'] = tooltip_shelf

    color_shelf = report_obj.get('color_shelf', [])
    aggregate_shelf = report_obj.get('aggregates_shelf', [])

    aggregate_shelf[:0] = color_shelf
    if report_obj.get('color_shelf', None) != None:
        del report_obj['color_shelf']

    report_obj['aggregates_shelf'] = aggregate_shelf

    core = report_obj.get('core', None)
    if core == None:
        return report_obj

    shelves = core.get('shelves', [])
    for shelf in shelves:
        if shelf['name'] == 'color_shelf':
            shelf['name'] = 'dimensions_shelf'
            shelf['column_type'] = 1

    saved_shelf_sources = core.get('saved_shelf_sources', {})
    for shelf_elem in list(saved_shelf_sources.keys()):
        shelf_name = saved_shelf_sources[shelf_elem]
        if shelf_name == 'tooltip_shelf':
            for dim in dimensions_shelf:
                dataset_colname = dim.get('dataset_colname', None)
                if dataset_colname and dataset_colname in shelf_elem:
                    saved_shelf_sources[shelf_elem] = 'dimensions_shelf'
        elif shelf_name == 'color_shelf':
            saved_shelf_sources[shelf_elem] = 'aggregates_shelf'

    core['shelves'] = shelves
    core['saved_shelf_sources'] = saved_shelf_sources

    return report_obj

def modify_obj(data, isForward):
    if isForward:
        data = move_aggregates_to_color_shelf(data)
    else:
        data = move_colors_to_aggregate_shelf(data)

    return data

def modify_report_data(orm, isForward):
    all_reports = orm.Report.objects.all()

    for report in all_reports:
        report_modified = False
        try:
            report_data = json.loads(report.report_data)
        except:
            continue

        report_data_internal = report_data.get('report_data', {})
        if report_data.get('report_type', '') == 'leaflet':
            report_modified = True

            report_data_internal = modify_obj(report_data_internal, isForward)

            last_shelf_history = report_data_internal.get('lastShelfHistory', {})
            last_shelf_history = modify_obj(last_shelf_history, isForward)
            report_data_internal['lastShelfHistory'] = last_shelf_history

        shelf_history = report_data_internal.get('shelfHistory', {})
        leaflet_shelf_history = shelf_history.get('leaflet', None)
        if leaflet_shelf_history:
            report_modified = True
            leaflet_shelf_history = modify_obj(leaflet_shelf_history, isForward)
            shelf_history['leaflet'] = leaflet_shelf_history
            report_data_internal['shelfHistory'] = shelf_history

        if report_modified:
            report_data['report_data'] = report_data_internal
            report_data = json.dumps(report_data)
            orm.Report.objects.filter(id=report.id).update(report_data=report_data)

class Migration(SchemaMigration):
    no_dry_run = True

    def forwards(self, orm):
        "Write your forwards methods here."
        # Note: Don't use "from appname.models import ModelName". 
        # Use orm.ModelName to refer to models in this application,
        # and orm['appname.ModelName'] for models in other applications.
        modify_report_data(orm, True)

    def backwards(self, orm):
        "Write your backwards methods here."
        modify_report_data(orm, False)

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'datasets.dataconnection': {
            'Meta': {'object_name': 'DataConnection'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_description': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_info': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'dataconnection_password': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_settings': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        'datasets.dataset': {
            'Meta': {'object_name': 'DataSet'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataConnection']", 'null': 'True', 'blank': 'True'}),
            'dataset_create_user_perm': ('django.db.models.fields.CharField', [], {'default': "'setupperm'", 'max_length': '20'}),
            'dataset_description': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_detail': ('django.db.models.fields.TextField', [], {'max_length': '20000'}),
            'dataset_everyone_perm': ('django.db.models.fields.CharField', [], {'default': "'readwrite'", 'max_length': '20'}),
            'dataset_info': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_lvinfo': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'null': 'True', 'blank': 'True'}),
            'dataset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'dataset_settings': ('django.db.models.fields.TextField', [], {'default': "'{}'", 'max_length': '20000'}),
            'dataset_type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('37bcc01c-28ba-49a4-a39e-df02dba53966')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appgroup': {
            'Meta': {'object_name': 'AppGroup'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'group_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_options': ('django.db.models.fields.TextField', [], {'max_length': '4096', 'blank': 'True'}),
            'group_parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'group_subgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'group_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'group_username': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('6580af43-2003-40a9-9dab-85c63631c37c')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appgroupappmembership': {
            'Meta': {'object_name': 'AppGroupAppMembership'},
            'appgroup_app': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'appgroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'appgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('67bb676d-0f39-4efb-9b9f-eea9a568d273')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appsetting': {
            'Meta': {'object_name': 'AppSetting'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'setting_key': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'setting_read_admin_only': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'setting_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        'reports.colorpalette': {
            'Meta': {'object_name': 'ColorPalette'},
            'colorpalette_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'colorpalette_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'reports.customcss': {
            'Meta': {'object_name': 'CustomCSS'},
            'customcss_default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'customcss_js': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'customcss_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'customcss_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('fefdb63e-ad1c-430c-ab93-2622c086085b')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.dateranges': {
            'Meta': {'object_name': 'DateRanges'},
            'dateranges_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'dateranges_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'dateranges_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('93047f72-22ea-4cd0-bac8-e5c54b440343')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.parameterset': {
            'Meta': {'object_name': 'ParameterSet'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameterset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'parameterset_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'parameterset_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        'reports.report': {
            'Meta': {'object_name': 'Report'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report_data': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'report_description': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'report_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'report_type': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('df3b1994-e270-4728-a87f-1b8e68cbd362')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.reportimage': {
            'Meta': {'object_name': 'ReportImage'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'img': ('django.db.models.fields.BinaryField', [], {'null': 'True', 'blank': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('cf206246-c78f-42c1-9c35-75c4d803f24b')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.reportusage': {
            'Meta': {'object_name': 'ReportUsage'},
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'timewindow_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'reports.reportuser': {
            'Meta': {'object_name': 'ReportUser'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'reports.staticasset': {
            'Meta': {'object_name': 'StaticAsset'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now_add': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_apps': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_contents': ('django.db.models.fields.TextField', [], {'max_length': '104857600', 'null': 'True', 'blank': 'True'}),
            'staticasset_description': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_mimetype': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'staticasset_size': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('dc10f5db-457a-4fee-9432-47de5c53f959')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.userappgroupmembership': {
            'Meta': {'object_name': 'UserAppGroupMembership'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'usergroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'usergroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'usergroup_username': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['reports']
    symmetrical = True
