# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models
from uuid import UUID
import json

def update_gauge_setting(orm, isForward):
    gauge_reports = orm.Report.objects.filter(report_type='gauge')

    for report in gauge_reports:
        report_data = json.loads(report.report_data)
        report_data_internal = report_data.get('report_data', {})
        user_settings = report_data_internal.get('user_settings', {})
        key_measure_line = user_settings.get('gauge_key_measure_mark')
        has_gauge_line = user_settings.get('include_gauge_line')
        if isForward:
            if not key_measure_line:
                if has_gauge_line == 'true':
                    user_settings['gauge_key_measure_mark'] = 'NEEDLE'
                elif has_gauge_line == 'false':
                    user_settings['gauge_key_measure_mark'] = 'NONE'
        else:
            if key_measure_line == 'NONE':
                user_settings['include_gauge_line'] = 'false'
            else:
                user_settings['include_gauge_line'] = 'true'

        report_data_internal['user_settings'] = user_settings
        report_data['report_data'] = report_data_internal
        report_data = json.dumps(report_data)
        report.report_data = report_data
        report.save(update_fields=['report_data'])

class Migration(DataMigration):

    def forwards(self, orm):
        "Write your forwards methods here."
        # Note: Don't use "from appname.models import ModelName". 
        # Use orm.ModelName to refer to models in this application,
        # and orm['appname.ModelName'] for models in other applications.
        update_gauge_setting(orm, True)


    def backwards(self, orm):
        "Write your backwards methods here."
        update_gauge_setting(orm, False)

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'datasets.dataconnection': {
            'Meta': {'object_name': 'DataConnection'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_description': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_info': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'dataconnection_password': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_settings': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        'datasets.dataset': {
            'Meta': {'object_name': 'DataSet'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataConnection']", 'null': 'True', 'blank': 'True'}),
            'dataset_create_user_perm': ('django.db.models.fields.CharField', [], {'default': "'setupperm'", 'max_length': '20'}),
            'dataset_description': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_detail': ('django.db.models.fields.TextField', [], {'max_length': '20000'}),
            'dataset_everyone_perm': ('django.db.models.fields.CharField', [], {'default': "'readwrite'", 'max_length': '20'}),
            'dataset_info': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_lvinfo': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'null': 'True', 'blank': 'True'}),
            'dataset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'dataset_settings': ('django.db.models.fields.TextField', [], {'default': "'{}'", 'max_length': '20000'}),
            'dataset_type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('af317435-85a2-4e8b-8cc5-f4a170e04239')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appgroup': {
            'Meta': {'object_name': 'AppGroup'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'group_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_options': ('django.db.models.fields.TextField', [], {'max_length': '4096', 'blank': 'True'}),
            'group_parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'group_subgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'group_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'group_username': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('3eb2cc72-8a9b-49cf-a078-3c3d1938f0ec')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appgroupappmembership': {
            'Meta': {'object_name': 'AppGroupAppMembership'},
            'appgroup_app': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'appgroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'appgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('7910761c-07c8-4c32-8f56-dd5cd14edff5')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appsetting': {
            'Meta': {'object_name': 'AppSetting'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'setting_key': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'setting_read_admin_only': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'setting_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        'reports.colorpalette': {
            'Meta': {'object_name': 'ColorPalette'},
            'colorpalette_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'colorpalette_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'reports.customcss': {
            'Meta': {'object_name': 'CustomCSS'},
            'customcss_default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'customcss_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_js': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'customcss_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'customcss_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('de12d03b-9ed5-4ad6-940b-bbb2f3d7c354')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.dateranges': {
            'Meta': {'object_name': 'DateRanges'},
            'dateranges_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'dateranges_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'dateranges_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('cba97ced-2b54-4abf-a307-c82ddecab1b8')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.parameterset': {
            'Meta': {'object_name': 'ParameterSet'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameterset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'parameterset_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'parameterset_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        'reports.report': {
            'Meta': {'object_name': 'Report'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report_data': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'report_description': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'report_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'report_type': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('2ffe2a92-297f-4265-be5e-da42175f9105')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.reportimage': {
            'Meta': {'object_name': 'ReportImage'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'img': ('django.db.models.fields.BinaryField', [], {'null': 'True', 'blank': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('b7be9857-5d0e-4e61-99a3-c3177f309f87')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.reportusage': {
            'Meta': {'object_name': 'ReportUsage'},
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'timewindow_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'reports.reportuser': {
            'Meta': {'object_name': 'ReportUser'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'reports.staticasset': {
            'Meta': {'object_name': 'StaticAsset'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now_add': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_apps': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_contents': ('django.db.models.fields.TextField', [], {'max_length': '104857600', 'null': 'True', 'blank': 'True'}),
            'staticasset_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_mimetype': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'staticasset_size': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('c1274ae6-9647-46e1-9435-9a7a3cd04953')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.userappgroupmembership': {
            'Meta': {'object_name': 'UserAppGroupMembership'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'usergroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'usergroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'usergroup_username': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['reports']
    symmetrical = True
