# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from reports.models import Report
from uuid import UUID, uuid4
from django.db import connection
import json, os, logging, tempfile
from migration.migrate import addDashboardSheetData
from util.cleanup_reports import update_reports_data


class Migration(SchemaMigration):

    def forwards(self, orm):
        try:
            r = Report.objects.all().values_list('report_mode')
            len(r)
        except:
            # if the migration has not been applied already
            connection.close()

            # Adding field 'Report.report_mode'
            db.add_column('reports_report', 'report_mode',
                          self.gf('django.db.models.fields.CharField')(
                              default='', max_length=100, null=True,
                              blank=True),
                          keep_default=False)

            # Adding field 'ReportTemplate.report'
            db.add_column('reports_reporttemplate', 'report',
                          self.gf('django.db.models.fields.related.ForeignKey')(to=orm['reports.Report'], null=True, blank=True),
                          keep_default=False)
        finally:
            if db.dry_run:
                return

            # Create new dashboard with single visual
            #   add it to dashboard, make full page, duplicate thumbnail,
            #   same update_user and create_user, click_behavior and csv/email/png settings
            #   take on visual's title, but hide it
            def create_single_visual_dashboard(viz_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard):
                new_dashboard = orm.Report.objects.filter(id=viz_id)
                if not new_dashboard.exists():
                    return None

                other_dashboards = visual_id_to_dashboards.get(viz_id)
                number_of_dashboards = len(other_dashboards)
                new_dashboard = new_dashboard.first()
                new_dashboard.pk = None
                new_dashboard.uuid = uuid4()
                new_dashboard.imported_uuid = None

                if number_of_dashboards >= 1:
                    new_dashboard.shared_visual_dashboards = '[%s]' % viz_id

                    if number_of_dashboards == 1:
                        for other_dash_id in other_dashboards:
                            try:
                                other_dashboard = orm.Report.objects.get(id=other_dash_id)
                                new_linked_visuals = set(json.loads(other_dashboard.shared_visual_dashboards or '[]'))
                                new_linked_visuals.add(viz_id)
                                other_dashboard.shared_visual_dashboards = json.dumps(list(new_linked_visuals))
                                other_dashboard.save()
                                logging.info(
                                    'Added visual %s to dashboard %s shared_visual_dashboards' % (
                                    viz_id, other_dash_id))

                            except:
                                logging.info(
                                    'Dashboard %s no longer exists' % other_dash_id)

                else:
                    new_dashboard.shared_visual_dashboards = '[]'

                # Update report_data
                viz_json = json.loads(new_dashboard.report_data).get(
                    'report_data', {})
                viz_settings = viz_json.get('user_settings', {})
                dashboard_json = {
                    'report_title': viz_json.get('report_title', ''),
                    'numColumns': 64,
                    'dashboard_widgets': [{
                        'col': 1,
                        'row': 1,
                        'size_x': 64,
                        'size_y': 30,
                    # approximate height visual should be to be full screen
                        'id': 'uri-1-widget-{0}'.format(viz_id)
                    }],
                }

                new_user_setting = {}
                if new_dashboard.report_type in ['extension', 'html', 'link']:
                    new_user_setting = {
                        # Settings needed to make visual full screen
                        'dashboard_width': '',
                        'single_visual_dashboard': 'true',
                        'horizontal_margin_between_visuals': '0',
                        'vertical_margin_between_visuals': '0',
                    }

                # Hide title and border around visuals
                new_user_setting['borders_around_visuals'] = 'false'
                new_user_setting['dashboard_hide_title'] = 'true'

                # Settings to move over from visual
                new_user_setting['click_behavior'] = viz_json.get(
                    'click_behavior', {})

                new_dashboard.report_type = 'dashboard'

                csv_setting = viz_settings.get('permit_csv_download_visual')
                if csv_setting:
                    new_user_setting[
                        'permit_csv_download_dashboard'] = csv_setting

                email_setting = viz_settings.get('permit_email_report')
                if email_setting:
                    new_user_setting['permit_email_report'] = email_setting

                png_setting = viz_settings.get('permit_png_download')
                if png_setting:
                    new_user_setting['permit_png_download'] = png_setting

                dashboard_json['user_settings'] = new_user_setting
                new_dashboard.report_data = json.dumps(
                    addDashboardSheetData(dashboard_json))

                new_dashboard.save()

                other_dashboards.add(new_dashboard.id)

                current_viz = orm.Report.objects.filter(id=viz_id)
                if current_viz.exists():
                    # Add dashboard to single_visual_dashboard if visual is linked
                    if number_of_dashboards >= 1:
                        current_viz = current_viz.first()
                        current_viz_json = json.loads(current_viz.report_data)
                        current_viz_json['report_data'].pop('dashboard_id', None)
                        current_viz.report_data = json.dumps(current_viz_json)
                        current_viz.shared_visual_dashboards = json.dumps(list(other_dashboards))
                    elif number_of_dashboards == 0:
                        current_viz = current_viz.first()
                        current_viz_json = json.loads(current_viz.report_data)
                        current_viz_json['report_data'][
                            'dashboard_id'] = new_dashboard.id
                        current_viz.report_data = json.dumps(current_viz_json)

                    current_viz.save()

                new_thumbnail = orm.ReportImage.objects.filter(report_id=viz_id)
                if new_thumbnail.exists():
                    new_thumbnail = new_thumbnail.first()
                    new_thumbnail.pk = None
                    new_thumbnail.uuid = uuid4()
                    new_thumbnail.imported_uuid = None
                    new_thumbnail.report = new_dashboard
                    new_thumbnail.save()

                dashboard_id_to_visuals[new_dashboard.id] = set([viz_id])
                visual_id_to_full_screen_dashboard[viz_id] = new_dashboard.id
                return new_dashboard

            def handle_unused_visual(viz_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard):
                # Remove picklist if it's not used in any visuals
                # Shouldn't need
                if picklist_ids.get(viz_id):
                    logging.info('deleting picklist ID %s' % viz_id)
                    orm.Report.objects.filter(id=viz_id).delete()

                # Create single visual dashboard
                else:
                    logging.info('converting visual ID %s to single visual dashboard' % viz_id)
                    create_single_visual_dashboard(viz_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard)

            def handle_visuals_in_apps(visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard):
                # - If single visual is used in appgroups, change single visual to full page visual
                all_appgroup_memberships = orm.AppGroupAppMembership.objects.values(
                    'id', 'appgroup_app')
                for agm in all_appgroup_memberships:

                    # If it's a visual, assign a single visual dashboard if it's already created
                    # If not created already, create one and assign (if the visual still exists)
                    appgroup_report_id = agm['appgroup_app']
                    try:
                        list_of_dashboards = visual_id_to_dashboards.get(appgroup_report_id)
                        if list_of_dashboards or list_of_dashboards == set([]):
                            logging.info('App group report  %s in membership %s is visual' %
                                         (appgroup_report_id, agm['id']))
                            appgroup_instance = orm.AppGroupAppMembership.objects.filter(id=agm['id'])
                            full_screen_dashboard_id = visual_id_to_full_screen_dashboard.get(
                                appgroup_report_id)
                            if full_screen_dashboard_id or full_screen_dashboard_id == 0:
                                appgroup_instance.update(
                                    appgroup_app=full_screen_dashboard_id)
                            else:
                                logging.info('converting visual ID %s to single'
                                             ' visual dashboard through appgroup' % appgroup_report_id)
                                new_report = create_single_visual_dashboard(appgroup_report_id, visual_id_to_dashboards, dashboard_id_to_visuals, picklist_ids, visual_id_to_full_screen_dashboard)
                                if new_report:
                                    appgroup_instance.update(
                                        appgroup_app=new_report)
                                else:
                                    # Visual no longer exists. Delete the visual membership
                                    logging.info('deleting appgroup_membership ID: %s' % appgroup_instance.id)
                                    appgroup_instance.delete()

                    except Exception as e:
                        msg = 'ERROR: Cannot update appgroup report ID: %s' % appgroup_report_id
                        print(msg)
                        print(str(e))
                        logging.exception(msg)

            # Run main update function
            # create shared visuals or make single visual dashboards
            # - If visual used in just 1 dashboard, add dashboard_id to report_data
            # - If single visual is not used in any dashboard, create dashboard
            # - If visual used in 2 or more dashboards, make shared visual
            update_reports_data(orm.Report, logging, handle_unused_visual, handle_visuals_in_apps)


    def backwards(self, orm):
        # Deleting field 'Report.report_mode'
        db.delete_column('reports_report', 'report_mode')

        # Deleting field 'ReportTemplate.report'
        db.delete_column('reports_reporttemplate', 'report_id')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'datasets.dataconnection': {
            'Meta': {'object_name': 'DataConnection'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_description': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_info': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'dataconnection_password': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_settings': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        'datasets.dataset': {
            'Meta': {'object_name': 'DataSet'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataConnection']", 'null': 'True', 'blank': 'True'}),
            'dataset_create_user_perm': ('django.db.models.fields.CharField', [], {'default': "'setupperm'", 'max_length': '20'}),
            'dataset_description': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_detail': ('django.db.models.fields.TextField', [], {'max_length': '20000'}),
            'dataset_everyone_perm': ('django.db.models.fields.CharField', [], {'default': "'readwrite'", 'max_length': '20'}),
            'dataset_info': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_lvinfo': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'null': 'True', 'blank': 'True'}),
            'dataset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'dataset_settings': ('django.db.models.fields.TextField', [], {'default': "'{}'", 'max_length': '20000'}),
            'dataset_type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('2bc4dc83-ccef-421d-aa7d-1ef78d57ffdd')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appgroup': {
            'Meta': {'object_name': 'AppGroup'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'group_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_options': ('django.db.models.fields.TextField', [], {'max_length': '4096', 'blank': 'True'}),
            'group_parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'group_subgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'group_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'group_username': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('45d63939-d069-4742-b515-ebcd83375fce')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appgroupappmembership': {
            'Meta': {'object_name': 'AppGroupAppMembership'},
            'appgroup_app': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'appgroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'appgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('ac8f6497-b17c-4de9-932a-4b8dae058933')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.appsetting': {
            'Meta': {'object_name': 'AppSetting'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'setting_key': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'setting_read_admin_only': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'setting_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        'reports.colorpalette': {
            'Meta': {'object_name': 'ColorPalette'},
            'colorpalette_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'colorpalette_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'reports.customcss': {
            'Meta': {'object_name': 'CustomCSS'},
            'customcss_default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'customcss_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_js': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'customcss_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'customcss_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('340a7663-baae-4b35-826c-2dc22fede1b9')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.dateranges': {
            'Meta': {'object_name': 'DateRanges'},
            'dateranges_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'dateranges_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'dateranges_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('03d7ed69-d50d-4a61-9209-80e8580b5718')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.parameterset': {
            'Meta': {'object_name': 'ParameterSet'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameterset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'parameterset_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'parameterset_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        'reports.report': {
            'Meta': {'object_name': 'Report'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report_data': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'report_description': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'report_mode': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'report_type': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'shared_visual_dashboards': ('django.db.models.fields.TextField', [], {'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('dcea58a8-454b-4c25-86d2-81a5f9ab1fb0')", 'unique': 'True', 'max_length': '36'}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': "orm['workspace.Workspace']", 'null': 'True', 'on_delete': 'models.SET_DEFAULT', 'blank': 'True'})
        },
        'reports.reportimage': {
            'Meta': {'object_name': 'ReportImage'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'img': ('django.db.models.fields.BinaryField', [], {'null': 'True', 'blank': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('6826fbfc-dcff-4366-ba14-0c7a6e616bba')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.reporttemplate': {
            'Meta': {'object_name': 'ReportTemplate'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'data': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'img': ('django.db.models.fields.BinaryField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'params': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        'reports.reportusage': {
            'Meta': {'object_name': 'ReportUsage'},
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'timewindow_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'reports.reportuser': {
            'Meta': {'object_name': 'ReportUser'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'reports.staticasset': {
            'Meta': {'object_name': 'StaticAsset'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now_add': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_apps': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_contents': ('django.db.models.fields.TextField', [], {'max_length': '104857600', 'null': 'True', 'blank': 'True'}),
            'staticasset_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_mimetype': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'staticasset_size': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('4ef85295-2dce-4d07-95e1-03f387796769')", 'unique': 'True', 'max_length': '36'})
        },
        'reports.userappgroupmembership': {
            'Meta': {'object_name': 'UserAppGroupMembership'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'usergroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'usergroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'usergroup_username': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'workspace.workspace': {
            'Meta': {'object_name': 'Workspace'},
            'acl': ('django.db.models.fields.TextField', [], {'default': "'[]'", 'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'editable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'private_user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'})
        }
    }

    complete_apps = ['reports']
