# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
import json, os, logging, tempfile
from util.cleanup_reports import update_reports_data
from uuid import UUID, uuid4

logfile = os.path.join(os.environ.get('LOG_DIR', tempfile.gettempdir()),
                       'arcviz_migration.log')
print('logging the migration to %s file' % logfile)
logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)s %(message)s',
                    filename=logfile)

# This migration is specifically for Citi, who was migrating from 4.1.3 to 4.2.0,
# before migrating to 4.2.1
# This is specifically just to add dashboard_id and dashboard_sheets to report_data
class Migration(SchemaMigration):
    def forwards(self, orm):
        if db.dry_run:
            return

        if not db.dry_run:
            update_reports_data(orm.Report, logging)

    def backwards(self, orm):
        # No way to go back
        pass

    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'datasets.dataconnection': {
            'Meta': {'object_name': 'DataConnection'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection_description': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_info': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'dataconnection_password': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_settings': ('django.db.models.fields.TextField', [], {'max_length': '4000', 'blank': 'True'}),
            'dataconnection_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        u'datasets.dataset': {
            'Meta': {'object_name': 'DataSet'},
            'cache_sequence': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataconnection': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['datasets.DataConnection']", 'null': 'True', 'blank': 'True'}),
            'dataset_create_user_perm': ('django.db.models.fields.CharField', [], {'default': "'setupperm'", 'max_length': '20'}),
            'dataset_description': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_detail': ('django.db.models.fields.TextField', [], {'max_length': '20000'}),
            'dataset_everyone_perm': ('django.db.models.fields.CharField', [], {'default': "'readwrite'", 'max_length': '20'}),
            'dataset_info': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'blank': 'True'}),
            'dataset_lvinfo': ('django.db.models.fields.TextField', [], {'max_length': '20000', 'null': 'True', 'blank': 'True'}),
            'dataset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'dataset_settings': ('django.db.models.fields.TextField', [], {'default': "'{}'", 'max_length': '20000'}),
            'dataset_type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('c7b6234f-018e-4eb9-80c7-e805d83c3a73')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.appgroup': {
            'Meta': {'object_name': 'AppGroup'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'group_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_options': ('django.db.models.fields.TextField', [], {'max_length': '4096', 'blank': 'True'}),
            'group_parent': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'group_subgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'group_type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'group_username': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'group_visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('98c8335d-24fb-4889-95cc-975898a2a6b7')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.appgroupappmembership': {
            'Meta': {'object_name': 'AppGroupAppMembership'},
            'appgroup_app': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'appgroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'appgroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('161e722d-ed30-4441-9568-b3d1c3e84499')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.appsetting': {
            'Meta': {'object_name': 'AppSetting'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'setting_key': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'setting_read_admin_only': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'setting_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        u'reports.colorpalette': {
            'Meta': {'object_name': 'ColorPalette'},
            'colorpalette_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'colorpalette_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'colorpalette_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'reports.customcss': {
            'Meta': {'object_name': 'CustomCSS'},
            'customcss_default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'customcss_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_js': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'customcss_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'customcss_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'customcss_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('8341ce1f-2340-46cb-abe0-6d5555e4f890')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.dateranges': {
            'Meta': {'object_name': 'DateRanges'},
            'dateranges_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'dateranges_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'dateranges_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('4a9ca4a9-4711-4ad4-af80-0353444c8e93')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.parameterset': {
            'Meta': {'object_name': 'ParameterSet'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameterset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'parameterset_type': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'parameterset_value': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'})
        },
        u'reports.report': {
            'Meta': {'object_name': 'Report'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'dataset': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['datasets.DataSet']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report_data': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'report_description': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'report_mode': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'report_type': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'shared_visual_dashboards': ('django.db.models.fields.TextField', [], {'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('62d8521a-e8c9-41e1-9670-9dcb921fae90')", 'unique': 'True', 'max_length': '36'}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['workspace.Workspace']", 'null': 'True', 'on_delete': 'models.SET_DEFAULT', 'blank': 'True'})
        },
        u'reports.reportimage': {
            'Meta': {'object_name': 'ReportImage'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'img': ('django.db.models.fields.BinaryField', [], {'null': 'True', 'blank': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('e9e0c2c8-6aeb-47a3-a33b-ee15a28d11fa')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.reporttemplate': {
            'Meta': {'object_name': 'ReportTemplate'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'null': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'data': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'img': ('django.db.models.fields.BinaryField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'params': ('django.db.models.fields.TextField', [], {'max_length': '32768'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'})
        },
        u'reports.reportusage': {
            'Meta': {'object_name': 'ReportUsage'},
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'timewindow_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        u'reports.reportuser': {
            'Meta': {'object_name': 'ReportUser'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'report': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.Report']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        u'reports.staticasset': {
            'Meta': {'object_name': 'StaticAsset'},
            'create_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now_add': 'True', 'blank': 'True'}),
            'create_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'imported_uuid': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_apps': ('django.db.models.fields.TextField', [], {'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_contents': ('django.db.models.fields.TextField', [], {'max_length': '104857600', 'null': 'True', 'blank': 'True'}),
            'staticasset_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '16000', 'null': 'True', 'blank': 'True'}),
            'staticasset_mimetype': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'staticasset_name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'staticasset_size': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'update_time': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'update_user': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "UUID('82b5d004-03ca-410f-86a7-df3c967f4922')", 'unique': 'True', 'max_length': '36'})
        },
        u'reports.userappgroupmembership': {
            'Meta': {'object_name': 'UserAppGroupMembership'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'usergroup_group': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['reports.AppGroup']", 'null': 'True', 'blank': 'True'}),
            'usergroup_order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'usergroup_username': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        u'workspace.workspace': {
            'Meta': {'object_name': 'Workspace'},
            'acl': ('django.db.models.fields.TextField', [], {'default': "'[]'", 'max_length': '32768', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'editable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'null': 'True', 'blank': 'True'}),
            'private_user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'})
        }
    }

    complete_apps = ['reports']
