# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import asyncio
import django.test
import logging
import time

from reports.handlers import NotificationCenterHandler
from reports.notification_center import NotificationCenter
from unittest import mock


def get_logger():
  logger = logging.getLogger(__name__)
  if NotificationCenterHandler not in logger.handlers:

    # we must add a stream handler before the NotificationCenterHandler
    # so that the log records are formatted and their message field set
    # (LogRecord has getMessage() method but no message field)
    nullStreamHandler = logging.StreamHandler(stream=mock.Mock())
    logger.addHandler(nullStreamHandler)
    logger.addHandler(NotificationCenterHandler)

  return logger


class TestNotificationCenterHandler(django.test.TestCase):

  def test_sync_access(self):
    notification_center = NotificationCenter()

    start_count = len(notification_center.get_all_notifications())

    get_logger().error('error1')
    self.assertEquals(
      len(notification_center.get_all_notifications()), start_count + 1)

    # duplicate must be detected
    get_logger().error('error1')
    self.assertEquals(
      len(notification_center.get_all_notifications()), start_count + 1)

  async def log_error(self, msg):
    get_logger().error(msg)

  def wait_for_notification_count(self, expected_count):
    notification_center = NotificationCenter()

    start_time = time.time()
    while len(notification_center.get_all_notifications()) < expected_count:
      time.sleep(0.5)
      if time.time() > start_time + 2:
        break

  def test_async_access(self):
    notification_center = NotificationCenter()

    start_count = len(notification_center.get_all_notifications())

    event_loop = asyncio.new_event_loop()
    asyncio.set_event_loop(event_loop)
    event_loop.run_until_complete(self.log_error('error2'))
    self.wait_for_notification_count(start_count + 1)

    # duplicate must be detected
    event_loop.run_until_complete(self.log_error('error2'))

    # different messages must be stored
    event_loop.run_until_complete(self.log_error('error3'))
    event_loop.run_until_complete(self.log_error('error4'))
    self.wait_for_notification_count(start_count + 1 + 2)
