# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import django.test

from reports import migration_utils
from reports.models import Report
from vizlib.tests import test_report_operations as test_utils
from vizlib.tests.util import run_with_db_query_check

helper = test_utils.TestDashboardOperations()


class TestMigrationUtils(django.test.TestCase):
  def test_update_shared_visual_dashboards(self):

    admin = test_utils.make_user('admin1', is_superuser=True)
    admin_client = test_utils.create_client(admin)
    dc_id = helper.create_dataconnection(admin_client, connection_name='DC1')
    ds_id = helper.create_dataset(admin_client, dc_id=dc_id, ds_name='ds1')

    v1_id = helper.create_visual(admin_client, ds_id)
    v2_id = helper.create_visual(admin_client, ds_id)

    v1 = Report.objects.get(id=v1_id)
    v2 = Report.objects.get(id=v2_id)

    self.assertEqual(2, Report.objects.count())

    self.assertIsNone(v1.shared_visual_dashboards)
    self.assertIsNone(v2.shared_visual_dashboards)

    # insert empty strings:
    Report.objects.update(shared_visual_dashboards='')
    v1.refresh_from_db()
    v2.refresh_from_db()

    self.assertEqual(v1.shared_visual_dashboards, '')
    self.assertEqual(v2.shared_visual_dashboards, '')

    # execute migration script:
    run_with_db_query_check(
      lambda: migration_utils.update_shared_visual_dashboards(Report),
      expected_query_count=1,  # single UPDATE query is issued
      asserter=self
    )
    v1.refresh_from_db()
    v2.refresh_from_db()

    # empty strings were turned back to NULLs:
    self.assertIsNone(v1.shared_visual_dashboards)
    self.assertIsNone(v2.shared_visual_dashboards)
