# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import unittest
import base64

from reports.views_sitedata import _remove_scripts_from_contents

class TestRemoveScriptsFromContents(unittest.TestCase):
  def test_no_script_tags(self):
    content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />
      </svg>
      '''.replace(b' ' , b'')
    base64_content = base64.b64encode(content).decode('ascii')

    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id', True), content)
    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id'), base64_content)

  def test_single_script_tag(self):
    content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />
        <script type="text/javascript">alert("1");</script>
      </svg>
      '''.replace(b' ' , b'')
    base64_content = base64.b64encode(content).decode('ascii')

    expected_content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />

      </svg>
      '''.replace(b' ' , b'')

    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id', True), expected_content)

  def test_multiple_script_tags(self):
    content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />
        <script type="text/javascript">alert("1");</script>
        <script>console.log('Second script');</script>
      </svg>
      '''.replace(b' ' , b'')
    base64_content = base64.b64encode(content).decode('ascii')

    expected_content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />


      </svg>
      '''.replace(b' ' , b'')

    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id', True), expected_content)

  def test_script_with_newlines(self):
    content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />
        <script>
            var x = 10;
            var y = 20;
            console.log(x + y);
        </script>
      </svg>
      '''.replace(b' ' , b'')
    base64_content = base64.b64encode(content).decode('ascii')

    expected_content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />

      </svg>
      '''.replace(b' ' , b'')

    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id', True), expected_content)

  def test_script_with_nested_tags(self):
    content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />
          <script>
            var x = 10;
            <script>console.log('nested script');</script>
            var y = 20;
          </script>
      </svg>
      '''.replace(b' ' , b'')
    base64_content = base64.b64encode(content).decode('ascii')

    expected_content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />

            var y = 20;
          </script>
      </svg>
      '''.replace(b' ' , b'')

    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id', True), expected_content)

  def test_script_with_special_characters(self):
    content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />
          <script>
            var message = "Hello, <world> & 'everyone'";
            console.log(message);
          </script>
      </svg>
      '''.replace(b' ' , b'')
    base64_content = base64.b64encode(content).decode('ascii')

    expected_content = b'''
      <svg width="200" height="200" xmlns="http://www.w3.org/2000/svg">
        <circle cx="100" cy="100" r="50" fill="green" />

      </svg>
      '''.replace(b' ' , b'')

    self.assertEqual(_remove_scripts_from_contents(base64_content, 'id', True), expected_content)