# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import django.test
import json

from vizlib.tests import test_report_operations as util


class TestSettingsView(django.test.TestCase):
  def test_retrieval(self):

    admin1 = util.make_user('admin1', is_superuser=True)
    client = util.create_client(admin1)

    # get all settings
    response = client.get('/reports/setting')
    self.assertEqual(response.status_code, 200)

    payload = {
      'settings': [
        {
          'setting_key': 'email_host_password',
          'setting_value': 'a'
        }
      ],
      'flippers': []
    }
    response = client.post(
      '/reports/setting', content_type="application/json", data=json.dumps(payload))
    self.assertEqual(response.status_code, 200)

    response = client.get('/reports/setting')
    self.assertEqual(response.status_code, 200)

    settings = response.json()
    setting = [s for s in settings if s['setting_key']
               == 'email_host_password']
    self.assertEquals(len(setting), 1)
    setting_value = setting[0]['setting_value']
    self.assertTrue('*' in setting_value)
    self.assertTrue('a' not in setting_value)
