# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.test import TestCase

from django.test import TestCase

from django.core.exceptions import ValidationError

from reports.models import CustomCSS, ColorPalette, ParameterSet, \
    DateRanges, AppSetting, AppGroup, UserAppGroupMembership, Report

INVALID_FIELD = 'a<b'
INVALID_MSG = 'Invalid character in field a<b'

class ModelValidation(TestCase):
  def test_customcss(self):
    cc = CustomCSS(
        customcss_name='c name',
        customcss_value='c value',
        customcss_js=INVALID_FIELD,
        customcss_type=1,
        customcss_default=True)
    cc.save()

    cc = CustomCSS(
        customcss_name=INVALID_FIELD,
        customcss_value=INVALID_FIELD,
        customcss_js=INVALID_FIELD,
        customcss_type=1,
        customcss_default=True)

    try:
      cc.save()
    except:
      self.fail('customcss failed to save, should allow for html inputs')
      raise

  def test_custompalette(self):
    cp = ColorPalette(
        colorpalette_name='cp name',
        colorpalette_value='cp value',
        colorpalette_type=1)
    cp.save()

    cp = ColorPalette(
        colorpalette_name=INVALID_FIELD,
        colorpalette_value=INVALID_FIELD,
        colorpalette_type=1)
    with self.assertRaises(ValidationError):
      try:
        cp.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG] * 2)
        raise

  def test_parameterset(self):
    ps = ParameterSet(
        parameterset_name='param name',
        parameterset_value=INVALID_FIELD,
        parameterset_type=1)
    ps.save()

    ps = ParameterSet(
        parameterset_name=INVALID_FIELD,
        parameterset_value=INVALID_FIELD,
        parameterset_type=1)
    with self.assertRaises(ValidationError):
      try:
        ps.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG])
        raise

  def test_dateranges(self):
    dr = DateRanges(
        dateranges_name='dr name',
        dateranges_value='dr value',
        dateranges_type=1)
    dr.save()

    dr = DateRanges(
        dateranges_name=INVALID_FIELD,
        dateranges_value=INVALID_FIELD,
        dateranges_type=1)
    with self.assertRaises(ValidationError):
      try:
        dr.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG] * 2)
        raise

  def test_appsetting(self):
    apps = AppSetting(
        setting_key='key',
        setting_value=INVALID_FIELD,
        setting_read_admin_only=True)
    apps.save()

    apps = AppSetting(
        setting_key=INVALID_FIELD,
        setting_value=INVALID_FIELD,
        setting_read_admin_only=True)
    with self.assertRaises(ValidationError):
      try:
        apps.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG])
        raise

  def test_appgroup(self):
    ag = AppGroup(
      group_name='group name',
      group_type='group type',
      group_username='group user1',
      group_options='group options',
      create_user='user1',
      update_user='user2')
    ag.save()

    ag = AppGroup(
      group_name=INVALID_FIELD,
      group_type=INVALID_FIELD,
      group_username=INVALID_FIELD,
      group_options=INVALID_FIELD,
      create_user=INVALID_FIELD,
      update_user=INVALID_FIELD)
    with self.assertRaises(ValidationError):
      try:
        ag.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG] * 6)
        raise

  def test_userappgroup(self):
    ua = UserAppGroupMembership(
        usergroup_username='user1')
    ua.save()

    ua = UserAppGroupMembership(
        usergroup_username=INVALID_FIELD)
    with self.assertRaises(ValidationError):
      try:
        ua.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG])
        raise

  def test_report(self):
    r = Report(
      report_name=INVALID_FIELD,
      report_description='report desc',
      report_type='report type',
      report_data=INVALID_FIELD,
      create_user='user1',
      update_user='user2')
    r.save()

    r = Report(
      report_name=INVALID_FIELD,
      report_description=INVALID_FIELD,
      report_type=INVALID_FIELD,
      report_data=INVALID_FIELD,
      create_user=INVALID_FIELD,
      update_user=INVALID_FIELD)
    with self.assertRaises(ValidationError):
      try:
        r.save()
      except ValidationError as v:
        self.assertEqual(v.messages, [INVALID_MSG] * 4)
        raise

