# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""urls for reports"""

from django.urls import re_path

from reports import views, views_appgroups, views_settings, views_sitedata, views_notification_center

# pylint: disable=C0103
urlpatterns = [
   re_path(r'^columns$', views.columns, name='reports-columns'),
   re_path(r'^columns/(?P<no_cache>\d)/(?P<table_name>.+)$', views.columns),
   re_path(r'^columns/(?P<table_name>.+)$', views.columns),

   re_path(r'^myapps$', views_appgroups.myapps, name='reports-myapps'),
   re_path(r'^myapps/(?P<app_id>\d+)$', views_appgroups.myapps),

   re_path(r'^add_search_report$', views.add_search_report, name='reports-add-search'),
   re_path(r'^report$', views.report, name='reports-report'),
   re_path(r'^report/(?P<report_id>\d+)$', views.report),
   re_path(r'^reportcloneinfo$', views.get_clone_info, name='reports-get_clone_info'),
   re_path(r'^reportcloneinfo/(?P<report_id>\d+)$', views.get_clone_info),
   re_path(r'^report_usage$', views.report_usage, name='reports-reportusage'),

   re_path(r'^template/$', views.report, name='reports-template'),
   re_path(r'^template/save_template/(?P<report_id>\d+)$', views.save_template),
   re_path(r'^template/publishtemplate$', views.convert_to_new_dashboard),
   re_path(r'^template/showall$', views.get_matched_templates, {'showall': True}),
   re_path(r'^template/filtertemplate$', views.get_matched_templates),

   re_path(r'^reports_accel$', views.reports_accel, name='reports-accel'),

   re_path(r'^clone$', views.clone, name='reports-clone'),
   re_path(r'^create$', views.create, name='reports-create'),
   re_path(r'^create/(?P<dataset_id>\d+)$', views.create),

   re_path(r'^clonewidgets$', views.clone_widgets, name='reports-clonewidgets'),
   re_path(r'^savedashboard$', views.savedashboard, name='reports-savedashboard'),

   re_path(r'^appgroup$', views_appgroups.appgroup, name='apps-appgroup'),
   re_path(r'^appgroup/(?P<appgroup_id>-?\d+)$', views_appgroups.appgroup),

   re_path(r'^appgroup/(?P<appgroup_id>\d+)/app/(?P<app_id>\d+)$',
      views_appgroups.appgroup_app),
   re_path(r'^userappgroup$', views_appgroups.user_appgroup,
      name="apps-userappgroup"),

   re_path(r'^setting$', views_settings.setting, name='apps-setting'),
   re_path(r'^setting/(?P<setting_key>.+)$', views_settings.setting),

   re_path(r'^colorpalette$', views_sitedata.colorpalette, name='apps-colorpalette'),
   re_path(r'^colorpalette/(?P<cp_id>\d+)$', views_sitedata.colorpalette),

   re_path(r'^staticasset$', views_sitedata.staticasset, name='data-staticassets'),
   re_path(r'^staticasset/(?P<asset_id>\d+)$', views_sitedata.staticasset),
   re_path(r'^staticasset/raw$', views_sitedata.staticasset_raw, name='raw-staticassets'),
   re_path(r'^staticasset/raw/(?P<asset_id>\d+)(\.[a-z]+)?$', views_sitedata.staticasset_raw),

   re_path(r'^dateranges$', views_sitedata.dateranges, name='apps-dateranges'),
   re_path(r'^dateranges/(?P<daterange_id>\d+)$', views_sitedata.dateranges),
   re_path(r'^customcss$', views_sitedata.customcss, name='apps-customextension'),
   re_path(r'^customcss/(?P<css_id>\d+)$', views_sitedata.customcss),

   re_path(r'^customcssraw$', views_sitedata.customcss_raw, name='apps-customcssraw'),
   re_path(r'^customcssraw/(?P<css_id>\d+)$', views_sitedata.customcss_raw),

   re_path(r'^customjsraw$', views_sitedata.customjs_raw, name='apps-customjsraw'),
   re_path(r'^customjsraw/(?P<js_id>\d+)$', views_sitedata.customjs_raw),

   re_path(r'^move$', views.move_workspace, name='report-move-workspace'),

   re_path(r'^userdata$', views_settings.user_data, name='apps-user-data'),
   re_path(r'^userdata/(?P<setting_key>.+)$', views_settings.user_data),
   re_path(r'^homepage/defaultappid$', views_settings.assign_default_homepage, name='assign-default-app'),
   re_path(r'^homepage/disableoverride$', views_settings.set_homepage_override, name='disable-homepage-override'),

   re_path(r'^notification_center', views_notification_center.notification_center, name='notification-center')
   ]
