# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
from typing import Dict
from urllib.request import Request
from django.http import HttpResponse, QueryDict
from django.contrib.auth.decorators import login_required
from typing import Dict
from rbac.permcache import RBAC
from reports.notification_center import NotificationCenter

@login_required
def notification_center(request:Request):
    try:
        RBAC.checkPerm(request.user, 'sys_viewlogs')
    except:
        return HttpResponse(status=403)

    if request.method == 'GET':
        notifications: Dict[str, str] = NotificationCenter().get_all_notifications()
        return HttpResponse(json.dumps(notifications), content_type="application/json")

    if request.method == 'PUT':
        data = QueryDict(request.body)
        if data.get('action') == 'read':
            notification_ids = json.loads(data.get('ids'))

            NotificationCenter().read_notifications(notification_ids)

        notification_id = data.get('id')

        if data.get('action') == 'pin':
            NotificationCenter().pin_notification(notification_id)

        if data.get('action') == 'unpin':
            NotificationCenter().unpin_notification(notification_id)

        return HttpResponse(status=204)
