# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# Generated by Django 1.11.4 on 2018-10-15 19:06
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
from django.contrib.auth.models import User

def _update_update_user_time(item, username_to_user):
    create_user = username_to_user.get(item.create_user)
    if not create_user:
        try:
            create_user = User.objects.get(username=item.create_user)
            username_to_user[item.create_user] = create_user
        except Exception:
            create_user = User.objects.first()
    item.update_user_id = create_user.id
    item.update_time = item.create_time

# Populate update time/user and make sure bookmark dataset, bookmark search are not blank
def populate_update_values(apps, schema_editor):
    Search = apps.get_model('searches', 'Search')
    Bookmark = apps.get_model('searches', 'Bookmark')
    username_to_user = {}

    for search in Search.objects.all():
        _update_update_user_time(search, username_to_user)
        search.save(update_fields=['update_user', 'update_time'])

    for bookmark in Bookmark.objects.all():
        if not bookmark.dataset:
            bookmark.delete()
        else:
            _update_update_user_time(bookmark, username_to_user)
            bookmark.save(update_fields=['update_user', 'update_time'])


def _update_create_user(item):
    item.create_user = item.update_user
    item.save(update_fields=['create_user'])

def populate_create_users(apps, schema_editor):
    Search = apps.get_model('searches', 'Search')
    Bookmark = apps.get_model('searches', 'Bookmark')

    for search in Search.objects.all():
        _update_create_user(search)

    for bookmark in Bookmark.objects.all():
        _update_create_user(bookmark)

class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('searches', '0002_auto_20180915_1218'),
    ]

    operations = [
        migrations.AddField(
            model_name='bookmark',
            name='bookmark_data',
            field=models.TextField(default='{}'),
        ),
        migrations.AddField(
            model_name='search',
            name='search_data',
            field=models.TextField(default='{}'),
        ),
        migrations.AddField(
            model_name='bookmark',
            name='update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='updated_bookmark', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='search',
            name='update_time',
            field=models.DateTimeField(blank=True, null=True, auto_now=True),
        ),
        migrations.AddField(
            model_name='search',
            name='update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='updated_search', to=settings.AUTH_USER_MODEL),
        ),

        # Populate update time/user and make sure create time, bookmark dataset, bookmark search are not blank
        migrations.RunPython(populate_update_values, reverse_code=migrations.RunPython.noop),

        # Remove create user for bookmark and search
        migrations.RemoveField(
            model_name='bookmark',
            name='create_user',
        ),
        migrations.RemoveField(
            model_name='search',
            name='create_user',
        ),

        # Add back create user and set as update_user
        migrations.AddField(
            model_name='bookmark',
            name='create_user',
            field=models.ForeignKey(blank=True, null=True,
                                    on_delete=django.db.models.deletion.CASCADE,
                                    related_name='created_bookmark',
                                    to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='search',
            name='create_user',
            field=models.ForeignKey(blank=True, null=True,
                                    on_delete=django.db.models.deletion.CASCADE,
                                    related_name='created_search',
                                    to=settings.AUTH_USER_MODEL),
        ),

        # Populate create users
        migrations.RunPython(populate_create_users, reverse_code=migrations.RunPython.noop),

        # Make update/create user/date and dataset non-null
        migrations.AlterField(
            model_name='bookmark',
            name='update_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='updated_bookmark',
                                    to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='search',
            name='update_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='updated_search',
                                    to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='create_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='created_bookmark',
                                    to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='search',
            name='create_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='created_search',
                                    to=settings.AUTH_USER_MODEL),
        ),

        migrations.AlterField(
            model_name='bookmark',
            name='create_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='search',
            name='create_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='dataset',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    to='datasets.DataSet'),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='update_time',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='search',
            name='update_time',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
