# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from re import S
from datasets.models import DataConnection
from django.db import models
from arcwebbase.validator import validateNoHtml

class CacheStatisticModel(models.Model):
  """CacheStatistic"""
  dataconnection = models.ForeignKey(DataConnection, on_delete=models.CASCADE, blank=True, null=True)
  total_number_of_queries_returned = models.IntegerField(default=0)
  number_of_queries_returned_from_cache = models.IntegerField(default=0)
  number_of_queries_returned_from_backend = models.IntegerField(default=0)
  create_time = models.DateTimeField(auto_now_add=True)
  update_time = models.DateTimeField(auto_now=True)

  def save(self, *args, **kwargs):
      validateNoHtml(self)
      super(CacheStatisticModel, self).save(*args, **kwargs)

  def increment_total(self, number):
      self.total_number_of_queries_returned += number

  def increment_from_cache(self, number):
      self.number_of_queries_returned_from_cache += number
      self.increment_total(number)

  def increment_from_backend(self, number):
      self.number_of_queries_returned_from_backend += number
      self.increment_total(number)

