# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.test import TestCase
from dataconn import connections_types

from vizlib.dsreq import DsreqCreator

from unittest import mock

from extracts.tests import tests_rewrites

class TestKnnSearch(TestCase):
  def create_helper(self):
    helper = tests_rewrites.Test_Rewrites()
    return helper

  def test_sqlite(self):
    columns = [
      {
        "name": "data",
        "alias": "My data"
      },
      {
        "name": "vector",
        "alias": "Embedding Vector"
      },
    ]
    helper = self.create_helper()

    dc = helper.make_dataconnection()
    dc.dataconnection_type = connections_types.DATACONN_SQLITE
    ds = helper.make_dataset(dc, 'main.embeddings', columns=columns)

    # test KNN search with extra fields selected. Use field aliases.
    dsreq = DsreqCreator() \
      .datasetId(4001) \
      .addDimension("[My Data] as 'A'") \
      .addFilter("[Embedding Vector] knn (0,0,0,2)") \
      .setLimit(6) \
      .getdsreq()

    build_request = helper.create_build_request(dsreq, ds=ds)

    expected_query = 'SELECT TA_0."data" as `A` FROM "main"."embeddings"  TA_0  ORDER BY vector_distance(TA_0."vector","[0,0,0,2]") ASC LIMIT 6'
    self.assertEquals(build_request.query, expected_query)