# Table of Contents

1. [Linting](#linting)
    - [Setup hooks](#setup-hooks)
    - [Lint commands](#lint-commands)
    - [Configurations](#configurations)
2. [Testing](#testing)
      - [Structure](#structure)
      - [How to run tests](#how-to-run-tests)
      - [Debugging breaking tests](#debugging-breaking-tests)
3. [Folder structure](#folder-structure)
      - [Visuals & Dashboards](#visuals--dashboards)
      - [Admin and Page support](#admin-and-page-support)
      - [Data](#data)
      - [Other](#other)

## Linting

Linting help developers find common mistakes and write consistent code.

The eslint linter we currently use was set up after years of development, thus most of the files are not styled accordingly, it would be overwhelming to fix everything at once.

We plan to fix these lint errors gradually, by only checking lines of code that the developer change.

To achive this we set up pre-commit/push/merge git hooks.

### Lint commands

All of these commands check only the files that changed.
The difference is in how they decide if somethings is changed or not.

1. ```npm run lint```

    It compares HEAD with your "viz" branch to decide which lines changed.

    You can check all the lines that changed in your feature branch compared to the viz branch.

2. ```npm run lint:staged```

    It checks the staged lines (the lines you ```git add```-ed).
    Same as ```git diff HEAD --staged```.

    It is used for the pre-commit git hook, to check all the lines you are about to commit.

3. ```npm run lint:staged:ci```

    If ZUUL_BRANCH and ZUUL_COMMIT environment variables are defined, it will compare origin/ZUUL_BRANCH with ZUUL_COMMIT.
    Otherwise it just runs ```npm run lint:staged```.

    It is used in our CI to check each PR build.

    You can run this locally to check if it will pass the CI lint check or not.

    **!!! IMPORTANT !!!**

    To make sure it works the same as in the CI make sure
      1. origin is the Sense repository
      2. ZUUL_BRANCH is the branch you want your PR to merge into
      3. ZUUL_COMMIT is the hash (first seven digits are enough) of the last commit of your PR.

#### Configurations

1. You can specify which file(s) or folder(s) to check, by setting the  ```ESLINT_SOURCE_FILES``` environment variable.

    Examples:

    ```bash
    ESLINT_SOURCE_FILES=arc/visuals npm run lint
    ```

    ```bash
    ESLINT_SOURCE_FILES=arc/visuals/arc_visual_pie.js npm run lint
    ```

2. You can fix all the automatically fixable issues by adding ```:fix``` at the end of any of these commands.
    Examples:

     ```bash
    npm run lint:fix
    ```

    ```bash
    npm run lint:staged:fix
    ```

    ```bash
    npm run lint:staged:ci:fix
    ```

## Testing

### Structure
The test environment is configured using HTML files in `arcweb/qunit/templates/tests`, where the product's JavaScript, CSS, and HTML components are imported.

Subsequently, the QUnit tests are imported. These tests are located in `arcweb/static/js/test` and are structured as follows:

1. QUnit.modules
   - Modules can include setup and teardown functions, which run before and after each test.

2. QUnit.tests
   - Tests consist of a group of assertions designed to compare expected and actual values.
   - Each assertion should include a clear description.

Currently we have the following folders:

  1. builder - tests the creation and display of shelves
  2. framework - tests framework within arcviz (including data connections, sql queries, etc)
  3. visuals - tests rendering of visuals

### How to run tests

```scons arc:test```

1. Bundles the arc package for production if needed
2. Bundles the frontend package for production if needed
3. Starts the django webserver
5. Runs ```<root-folder>/arcweb/static/js/test/build-tools/run.js``` with node

    1. It finds all the HTML files inside `arcweb/qunit/templates/tests`.
    2. Runs the tests by "visiting" the served up HTML files with puppeteer (headless chrome browser).
    3. It combines the results
    4. Prints a success message if there were no test failures or a detailed error report.

### Debugging breaking tests
1. Start your development environment by running the following commands:
   - `scons arc:start`
   - `scons frontend:start`
   - `scons django:start`
2. Add `console.log` or `debugger` statements to your test or production code as needed.
3. Open a browser and launch the developer tools.
4. Navigate to `<base_url>/arc/qunit`.
5. Select the test module you want to examine.
6. Click the "Toggle QUnit" button to view the test results.

## Folder structure

```<root-folder>/arcweb/static/js/arc```

### Visuals & Dashboards

- visuals
  - All supported visual types
  - /common
    - Common code used by visuals
- filters
  - All supported filters
- dashboards
  - Support for building/displaying dashboards and apps
- search
  - Search framework, translates natural language queries into visuals
- showme
  - ShowMe framework, help suggest and select different visual types for displayed data
- actions
  - Action framework, used for sending JIRA, calling webhooks, and other actions
- extensions
  - Extension framework, used for extending visual functionality
- settings
  - Settings used within visuals and dashboards
- reports
  - Higher level support for integrating visuals within dashboards, including switching visuals and loading visual metadata from the backend
- params
  - Javascript module to help support page parameters

### Admin and Page support

- admin
  - Support for administrative tasks
- pages
  - Support common dom elements and initialization for application pages

### Data

- data
  - All interfaces with the backend data via datasets, dataconnectionsm and data requests
- acceleration
  - Javascript module to help support dashboard acceleration

### Other

- auth
  - Users, roles, and workspaces
- misc
  - Miscellaneous javascript functions used by multiple modules
- helpers
  - Javascript module to provide helper functions
