window.arc = window.arc || {};

(function() {
'use strict';

arc.action = arc.action || {};
arc.action.plugins = arc.action.plugins || {};
arc.action.extensions = arc.action.extensions || [];

let jira_newissue_defaults;

arc.action.jiraFetch = function(cb) {
  if (!arc.globals.jira_enabled) {
    cb();
    return;
  }

  const assets = [
    {
      asset: arc.frontend.globals.urls.actionsJiraDefaultsUrl,
      postprocess: d3.json,
    },
  ];

  arc.staticfetcher.fetch(assets, function(error, data) {
    if (!error) {
      if (data.jira_newissue_defaults) {
        jira_newissue_defaults = data.jira_newissue_defaults;
      }
    }

    cb();
  });
};

arc.action.plugins.createJira = function() {
  let c = function() {
    return 'createJira';
  };

  let viz,
      lib;

  c.viz = function(_) {
    if (!arguments.length) return viz;
    viz = _;
    lib = viz.viz_lib();
    return c;
  };

  c.settings = function() {
    return [
      {
        id: 'summary',
        displayName: 'Default Summary',
        type: 'string',
        defaultValue: '',
      },
      {
        id: 'description',
        displayName: 'Default Description',
        type: 'string',
        subtype: 'multiline',
        defaultValue: '',
      },
      {
        id: 'includeProject',
        displayName: 'Include project selector',
        type: 'boolean',
        defaultValue: 'true',
      },
      {
        id: 'includePriority',
        displayName: 'Include priority selector',
        type: 'boolean',
        defaultValue: 'true',
      },
      {
        id: 'properties',
        displayName: 'Other Properties in JSON',
        type: 'string',
        subtype: 'multiline',
        defaultValue: jira_newissue_defaults || '',
      },
      {
        id: 'includelink',
        displayName: 'Include link to page',
        type: 'boolean',
        defaultValue: 'true',
      },
      {
        id: 'includeData',
        displayName: 'Include Data From Visuals (Comma-Separated List of Visual IDs)',
        type: 'string',
        defaultValue: '',
      },
      {
        id: 'includeDataAsCSV',
        displayName: 'Display Data from Visuals as CSV',
        type: 'boolean',
        defaultValue: 'false',
      },
      {
        id: 'includeDataAsJiraTable',
        displayName: 'Display Data from Visuals as JIRA Tables',
        type: 'boolean',
        defaultValue: 'true',
      },
    ];
  };

  c.run = function(params) {
    if (!arc.globals.jira_enabled) {
      arc.report.alertMessageSmall('This action is disabled on the system. ' +
          'Please contact your system administrator.', 'alert-danger');
      return;
    }

    const $modal = $('#createjira-modal');
    if (arc.misc.modalShown($modal)) {
      console.log('modal already open');
      return;
    }

    let dashboard = arc.globals.dashboard;

    let other = {};

    let properties = params.properties;
    if (properties) {
      let j = JSON.parse(params.properties);
      $.each(j, (k, v) => other[k] = v);
    }

    const includeProject = (params.includeProject === 'true'),
          includePriority = (params.includePriority === 'true'),
          includeCSV = (params.includeDataAsCSV === 'true');

    let includeJiraTable = (params.includeDataAsJiraTable === 'true');

    if (!includeCSV) {
      includeJiraTable = true;
    }

    let summary = params.summary || '';

    let description = [params.description || ''];
    description.push(`Reporter: ${arc.globals.username}\n`);
    let url = '';
    if (dashboard && (params.includelink === 'true')) {
      const data = [];
      arc.misc.getURLParams(data, dashboard.current_params()[0]);
      url = `Link: [${arc.misc.getBlankTargetUrl(arc.misc.dedupURLParams(data))}]`;
      description.push(url);
    }

    populateProperties({
      includeProject,
      includePriority,
    });

    $('#jira-summary').val(summary);

    const visualData = [];
    const visualIdStore = params.includeData;
    if (visualIdStore) {
      const visualIds = new Set(visualIdStore.split(',')
          .filter(function(v) {
            return (!isNaN(v));
          })
          .map(function(v) {
            return Number(v);
          }));

      if (arc.globals.dashboard) {
        arc.globals.dashboard.eachViz(function(viz) {
          if (visualIds.has(viz.report_id())) {
            const dataResult = viz.dataResult(),
                  columns = dataResult.columns(),
                  rows = dataResult.rows();

            if (includeCSV) {
              visualData.push(columns.map(function(c) {
                return c.colname();
              }).join(','));

              rows.forEach(function(row) {
                visualData.push(row.map(function(c) {
                  return c + '';
                }).join(','));
              });

              visualData.push('');
            }

            if (includeJiraTable) {
              visualData.push('||' + columns.map(function(c) {
                return c.colname();
              }).join('||') + '||');

              rows.forEach(function(row) {
                visualData.push('|' + row.map(function(c) {
                  return c + '';
                }).join('|') + '|');
              });

              visualData.push('');
            }
          }
        });
      }
    }

    $('#jira-description').val(description.join('\n') + '\n\n' + visualData.join('\n'));

    $modal.modal('show');

    $('#createjira-modal .createjira-modal-create')
      .offOn('click.create', function handleJiraCreate() {
        let data = {};
        data.summary = $('#jira-summary').val();
        data.description = $('#jira-description').val();

        if (includeProject) {
          data.project = $('#jira-project').val();
        }

        if (includePriority) {
          data.priority = $('#jira-priority').val();
        }

        data.other = JSON.stringify(other);

        let $this = $(this);
        $this.find('.fa-refresh')
             .removeClass('arc-invisible');
        $this.attr('disabled', true);
        doCreate(data, function resetModal() {
          $('#createjira-modal .createjira-modal-create')
              .attr('disabled', false)
              .find('.fa-refresh')
              .addClass('arc-invisible');
        });
      });
  };

  function populateProperties(config) {
    const {
      includeProject,
      includePriority,
    } = config;

    const $project = $('#jira-project');
    $project.empty();

    const $priority = $('#jira-priority');
    $priority.empty();

    if (!includeProject) {
      $project.closest('.form-group').hide();
    } else {
      $project.closest('.form-group').show();
    }

    if (!includePriority) {
      $project.closest('.form-group').hide();
    } else {
      $project.closest('.form-group').show();
    }

    if (includeProject || includePriority) {
      const assets = [
        {
          asset: arc.frontend.globals.urls.actionsJiraPropertiesUrl,
          postprocess: d3.json,
        },
      ];

      arc.staticfetcher.fetch(assets,
        function(error, properties) {
          if (includeProject) {
            $project.empty();
            properties.projects.forEach(function addPropertyOption(project) {
              $project.append(arc.misc.safe_substitute(
                  $('#simple-select-option select').html(),
                  {
                    VALUE: project.key,
                    LABEL: `${project.name} (${project.key})`,
                  }));
            });
          }

          if (includePriority) {
            $priority.empty();
            properties.priorities.forEach(function addPriorityOption(priority) {
              $priority.append(arc.misc.safe_substitute(
                  $('#simple-select-option select').html(),
                  {
                    VALUE: priority.id,
                    LABEL: `${priority.name}`,
                  }));
            });
          }
        });
    }
  }

  function doCreate(data, cb) {
    const url = arc.frontend.globals.urls.actionsJiraCreateUrl;

    $.ajax({
      url,
      data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      let err = '';
      if (req.responseText) {
        err = ': ' + req.responseText.substring(0, 200);
      }

      arc.report.alertMessageSmall(`Failed to create JIRA ticket${err}`,
          'alert-danger');
      cb();
    })
    .done((_data) => {
      arc.report.alertMessageSmall(`Created JIRA ticket ${_data.id}`,
          'alert-success');
      $('#createjira-modal').modal('hide');
      cb();
    });
  }

  return c;
};

function createJiraExtension() {
  let ext = function() {
    return 'createJiraExtention';
  };

  ext.enabled = function() {
    return arc.globals.jira_enabled;
  };

  ext.action_name = function() {
    return 'createjira';
  };

  ext.action_label = function() {
    return 'Create JIRA Ticket';
  };

  ext.action = arc.action.plugins.createJira;

  return ext;
}

arc.action.extensions.push(createJiraExtension());

}());
