window.arc = window.arc || {};

(function() {
'use strict';

arc.action = arc.action || {};
arc.action.plugins = arc.action.plugins || {};
arc.action.extensions = arc.action.extensions || [];

let webhook_whitelist = [];

arc.action.webhookFetch = function(cb) {
  const assets = [
    {
      asset: arc.frontend.globals.urls.actionsWebhookWhitelistUrl,
      postprocess: d3.json,
    },
  ];

  arc.staticfetcher.fetch(assets, function afterFetch(error, data) {
    if (!error) {
      if (data.webhook_whitelist) {
        webhook_whitelist = data.webhook_whitelist.split('\n');
      }
    }

    if (cb) {
      cb();
    }
  });
};

function validateWebhook(s) {
  let valid = false;

  webhook_whitelist.some(function(w) {
    if (w === s) {
      valid = true;
      return true;
    }
  });

  return valid;
}

arc.action.plugins.webhook = function() {
  let c = function() {
    return 'webhook';
  };

  let viz,
      lib;

  c.settings = function() {
    return [
      {
        id: 'url',
        displayName: 'URL',
        type: 'string',
        defaultValue: '',
      },
      {
        id: 'call_source',
        displayName: 'Call Source',
        type: 'dropdown',
        defaultValue: 'Browser',
        names: ['Browser', 'Server'],
      },
      {
        id: 'request_type',
        displayName: 'Request Type',
        type: 'dropdown',
        defaultValue: 'GET',
        names: ['GET', 'POST', 'PUT'],
      },
      {
        id: 'headers',
        displayName: 'Request Headers (in JSON)',
        type: 'string',
        subtype: 'multiline',
        defaultValue: '',
      },
      {
        id: 'parameters',
        displayName: 'Parameters (in JSON)',
        type: 'string',
        subtype: 'multiline',
        defaultValue: '',
      },
      {
        id: 'success_msg',
        displayName: 'Success message',
        type: 'string',
        defaultValue: 'Call made successfully',
      },
    ];
  };

  c.viz = function(_) {
    if (!arguments.length) return viz;
    viz = _;
    lib = viz.viz_lib();
    return c;
  };

  c.run = function(params) {
    if (!arc.globals.webhook_enabled) {
      arc.report.alertMessageSmall('This action is disabled on the system. ' +
          'Please contact your system administrator.', 'alert-danger');
      return;
    }

    let url = params.url;
    if (url) {
      let valid = validateWebhook(url);
      if (!valid) {
        arc.report.alertMessageSmall(
            `URL specified is not in webhook settings whitelist.
             Please contact your system administrator.`,
            'alert-danger');
        return;
      }

      let headers = params.headers.trim();
      if (headers) {
        headers = JSON.parse(headers);
      } else {
        headers = undefined;
      }

      let parameters = params.parameters.trim();
      if (parameters) {
        parameters = JSON.parse(parameters);
      } else {
        parameters = undefined;
      }

      const config = {
        url,
        request_type: params.request_type,
        headers,
        parameters,
        success_msg: params.success_msg,
      };

      if (params.call_source === 'Server') {
        invokeServer(config);
      } else {
        invokeBrowser(config);
      }
    }
  };

  function alertFailure(req, textstatus, errorthrown) {
    let err = '';
    if (req.responseText) {
      err = `: ${req.responseText.substring(0, 200)}`;
    }

    arc.report.alertMessageSmall(`Failed${err}`, 'alert-danger');
  }

  function invokeServer(config) {
    let {
      url,
      request_type,
      headers,
      parameters,
      success_msg,
    } = config;

    let data = {
      url,
      request_type,
      headers,
      parameters,
    };

    $.ajax({
      url: arc.frontend.globals.urls.actionsWebhookInvokeUrl,
      data,
      type: 'POST',
      beforeSend(xhr) {
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(alertFailure)
    .done(() => {
      if (!success_msg || success_msg.trim() === '') {
        return;
      }

      arc.report.alertMessageSmall(success_msg, 'alert-success');
    });
  }

  function invokeBrowser(config) {
    let {
      url,
      request_type,
      headers,
      parameters,
      success_msg,
    } = config;

    $.ajax({
      url,
      headers,
      data: parameters,
      type: request_type,
    })
    .fail(alertFailure)
    .done(() => {
      if (!success_msg || success_msg.trim() === '') {
        return;
      }

      arc.report.alertMessageSmall(success_msg, 'alert-success');
    });
  }

  return c;
};

function webhookExtension() {
  let ext = function() {
    return 'webhookExtension';
  };

  ext.enabled = function() {
    return arc.globals.webhook_enabled;
  };

  ext.action_name = function() {
    return 'webhook';
  };

  ext.action_label = function() {
    return 'Webhook';
  };

  ext.action = arc.action.plugins.webhook;

  return ext;
}

arc.action.extensions.push(webhookExtension());

}());
