window.arc = window.arc || {};

(function () {
  'use strict';

  arc.aliasesadmin = arc.aliasesadmin || {};

  const $table = $('#aliases-table');
  let $aliasesDataTable;
  let aliases;

  arc.aliasesadmin.init = function () {
    _drawPaginatedAliases();
  };

  arc.aliasesadmin.updateAliasRow = function (data) {
    return arc.frontend.vizHttp.patch({
      url: arc.frontend.globals.urls.urlShortenerUrl,
      params: data,
      headers: {
        'X-CSRFTOKEN': arc.globals.csrf_token,
      },
    })
      .then(() => {
        $table.DataTable().ajax.reload(null, false);
        arc.frontend.legacyBridge.events.next({ type: 'close-set-url-alias-modal' });
      })
      .catch((error) => {
        arc.report.alertMessage(error);
      });
  };

  function getParameter(param) {
    const parameter = Object.keys(param)
      .filter((key) => key !== 'scope')
      .reduce((cur, key) => {
        const value = param[key];
        cur.push({ key, value, value_list: [value] });
        return cur;
      }, []);

    return parameter;
  }

  function parameterTooltip(row) {
    const parameter = getParameter(row[1]);
    const alias = row[0];

    const btn = d3.select(`#btn-${alias}`);
    const dataTableFromWindowLeft = 25;
    const tooltip = arc.tooltip()
      .dom_id(`#filter-params-managing.freezableTooltip`)
      .addSelection(btn)
      .text_fn(() => `<h5>Current Parameters</h5><hr>
        <div class="fil-summary-body">
          ${arc.report.params_history().tooltip_text([parameter])}
        </div>`)
      .getXPos(() => ($(`#btn-${alias} .fa-filter`).offset().left + $(`#btn-${alias} .fa-filter`).width() - dataTableFromWindowLeft))
      .getYPos(() => $(`#btn-${alias}`).offset().top - $('#url-managing-aliases').offset().top + $(`#btn-${alias}`).height())
      .activate();

    btn.on('click', () => {
      const evt = d4.event || d3.event;
      if (!tooltip.isFrozen()) {
        evt.stopPropagation();
        tooltip.freeze();
      }
    });
  }

  function _drawPaginatedAliases() {
    $aliasesDataTable = $table.DataTable({
      paging: true,
      deferRender: true,
      destroy: true,
      searching: true,
      order: [[7, 'desc']],
      processing: true,
      search: {
        return: true,
      },
      serverSide: true,
      ajax(params, callback) {
        arc.frontend.vizHttp.get({
          url: arc.frontend.globals.urls.urlShortenerUrl,
          params
        })
          .then(({ data }) => {
            callback(
              {
                recordsTotal: data.numberOfAliases,
                recordsFiltered: data.numberOfFiltered,
                data: data.aliases.map((alias) => [
                  alias.slug,
                  alias.parameters,
                  alias.times_followed,
                  alias.last_viewed,
                  alias.dashboardId,
                  alias.alias_count,
                  alias.create_time,
                  alias.update_time,
                  alias.creator_name,
                  alias.visual_type,
                  alias.dashboard_name,
                  alias.is_created_in_email,
                ]),
              },
            );
            aliases = data.aliases;
          });
      },
      columns: [
        {
          className: 'aliases-table-slug',
          aliases: 'slug',
          name: 'slug',
          render(data, _, row) {
            const path = `${window.location.origin}${arc.frontend.globals.urls.urlShortenerUrl}`;
            const aliasCount = `${row[5]}`;
            let slugCellHtml;

            if (aliasCount <= 0) {
              slugCellHtml = `<span class="disabled-alias">${path}${data}</span>`;
            } else {
              slugCellHtml = `
              <div class="aliases-url-wrapper">
                <span>${path}</span>
                <span class="font-weight-bold current-alias-text">${data}</span>
                <a class="alias-link" href="${path}${data}" target="_blank">
                  <i class="fa fa-external-link" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="" data-original-title="Open in new tab"></i>
                </a>
              </div>`;
            }

            return slugCellHtml;
          }
        },
        {
          className: 'aliases-table-parameters',
          aliases: 'parameters',
          orderable: false,
          render(data, _, row) {
            const aliasCount = `${row[5]}`;
            let parametersCellHtml;

            const parameter = getParameter(row[1]);

            if (aliasCount <= 0) {
              parametersCellHtml = '';
            } else {
              parametersCellHtml = `
                <div class="d-flex align-items-center">
                  <div id="btn-${row[0]}" class="p-2 viz-params-display" style="display:${parameter.length ? 'block' : 'none'}">
                    <i class="fa fa-filter" aria-hidden="true"></i>
                  </div>
                  <div>
                    <i class="fa fa-envelope-o" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="" data-original-title="The alias has been created/used in an email" style="display:${row[11] ? 'block' : 'none'}"></i>
                  </div>
                </div>`;
            }

            return parametersCellHtml;
          }
        },
        {
          className: 'aliases-table-times-followed',
          aliases: 'times-followed',
          name: 'times_followed',
          render(data, _, row) {
            const aliasCount = `${row[5]}`;
            let timesFollowedCellHtml;

            if (aliasCount <= 0) {
              timesFollowedCellHtml = `<span class="disabled-alias">${data}</span>`;
            } else {
              timesFollowedCellHtml = `${data}`;
            }

            return timesFollowedCellHtml;
          }
        },
        {
          className: 'aliases-table-last_viewed',
          aliases: 'last_viewed',
          name: 'last_viewed',
          render(data, _, row) {
            const formattedJoinedDate = arc.misc.fromNow(data, 'Never');
            const aliasCount = `${row[5]}`;
            let lastViewedCellHtml;

            if (aliasCount <= 0) {
              lastViewedCellHtml = `<span class="cell-wrapper disabled-alias">
                    <span class="orig-date">${data}</span>
                    <span class="formatted-date">${formattedJoinedDate}</span>
                  </span>`;
            } else {
              lastViewedCellHtml = `<span class="cell-wrapper">
                    <span class="orig-date">${data}</span>
                    <span class="formatted-date">${formattedJoinedDate}</span>
                  </span>`;
            }

            return lastViewedCellHtml;
          }
        },
        {
          className: 'aliases-table-dashboard-id-and-name',
          aliases: 'dashboard-name',
          name: 'url_data__dashboardId',
          render(data, _, row) {
            const path = `${window.location.origin}${arc.frontend.globals.urls.urlShortenerUrl}`;
            const visualType = `${row[9]}`;
            const dashboardName = `${row[10]}`;
            const aliasCount = `${row[5]}`;
            let dashboardCellHtml;

            if (dashboardName !== '' && aliasCount > 0) {
              dashboardCellHtml = `<div class="current_dashboard">
                                    <a href="${path}${row[0]}" target="_blank"><span class="object-type ${visualType === 'dashboard' ? 'dashboard' : 'apps'}"></span><span>${data} - ${dashboardName}</span></a>
                                   </div>`;
            } else if (dashboardName !== '' && aliasCount >= 0) {
              dashboardCellHtml = `<span class="disabled-alias">${data} - ${dashboardName}</span>`;
            } else if (dashboardName === '') {
              dashboardCellHtml = `<div class="current_dashboard"><a href="${path}${row[0]}" target="_blank"><span class="object-type ${visualType === 'dashboard' ? 'dashboard' : 'apps'}"></span><span>${data} - Untitled ${visualType === 'dashboard' ? 'dashboard' : 'apps'}</span></a></div>`;
            }

            return dashboardCellHtml;
          }
        },
        {
          className: 'aliases-table-aliases-count',
          aliases: 'alias_count',
          name: 'alias_count',
          render(data) {
            return `<span class="badge badge-secondary badge-pill pull-right existingUrlsCount">${data}</span>`;
          }
        },
        {
          className: 'aliases-table-created',
          aliases: 'create-time',
          name: 'create_time',
          render(data, _, row) {
            const date = new Date(data);
            const day = date.getDay();
            const month = date.toLocaleString('default', { month: 'short' });
            const year = date.getFullYear();
            const aliasCount = `${row[5]}`;
            let createTimeCellHtml;

            if (aliasCount <= 0) {
              createTimeCellHtml = `<span class="disabled-alias">${month} ${day}, ${year}</span>`;
            } else {
              createTimeCellHtml = `${month} ${day}, ${year}`;
            }

            return createTimeCellHtml;
          }
        },
        {
          className: 'aliases-table-update-time',
          aliases: 'update-time',
          name: 'update_time',
          render(data, _, row) {
            const formattedJoinedDate = arc.misc.fromNow(data, 'Never');
            const aliasCount = `${row[5]}`;
            let updateTimeCellHtml;

            if (aliasCount <= 0) {
              updateTimeCellHtml = `<span class="cell-wrapper disabled-alias">
                    <span class="orig-date">${data}</span>
                    <span class="formatted-date">${formattedJoinedDate}</span>
                  </span>`;
            } else {
              updateTimeCellHtml = `<span class="cell-wrapper">
                    <span class="orig-date">${data}</span>
                    <span class="formatted-date">${formattedJoinedDate}</span>
                  </span>`;
            }

            return updateTimeCellHtml;
          }
        },
        {
          className: 'aliases-table-creator-id',
          aliases: 'creator-name',
          name: 'creator',
          render(data, _, row) {
            const aliasCount = `${row[5]}`;
            let creatorNameCellHtml;

            if (aliasCount <= 0) {
              creatorNameCellHtml = `<span class="disabled-alias">${data}</span>`;
            } else {
              creatorNameCellHtml = `${data}`;
            }

            return creatorNameCellHtml;
          }
        },
        {
          className: 'action-button-wrapper',
          users: '',
          orderable: false,
          render(data, _, row, meta) {
            const aliasCount = `${row[5]}`;
            const creator = `${row[8]}`;
            const isSuperUser = arc.globals.is_superuser;
            const creatorOrSuperuser = isSuperUser || creator === arc.globals.username;
            const rowIndex = meta.row;
            const visualType = `${row[9]}`;
            let actionButtonCellHtml;

            if (aliasCount <= 0) {
              actionButtonCellHtml = isSuperUser ? `
              <a class="edit-alias disabled" data-index="${rowIndex}" data-type="${visualType}">
                <span class="fa fa-pencil "></span>
               </a>

               <a class="copy-to-clipboard-link disabled">
                <i class="fa fa-files-o existing url-copy-btn disabled" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="" data-original-title="Copy to clipboard" href></i>
               </a>

              <a data-id="${row[0]}" class="alias-delete-link">
                <span class="fa fa-trash"></span>
              </a>` : '';
            } else {
              actionButtonCellHtml =
                `<a class="edit-alias ${!creatorOrSuperuser ? 'disabled' : ''}" data-index="${rowIndex}" data-type="${visualType}">
                <span class="fa fa-pencil "></span>
               </a>

               <a class="copy-to-clipboard-link">
                <i class="fa fa-files-o existing url-copy-btn" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="" data-original-title="Copy to clipboard"></i>
               </a>

              <a data-id="${row[0]}" class="alias-delete-link ${!creatorOrSuperuser ? 'disabled' : ''}">
                <span class="fa fa-trash  ${!creatorOrSuperuser ? 'disabled' : ''}"></span>
              </a>`;
            }

            return actionButtonCellHtml;
          }
        }
      ],
      drawCallback() {
        const $deleteModal = $('#delete-alias-modal');
        const $deleteConfirmButton = $deleteModal.find('.btn-primary');

        const rows = $table.DataTable().rows({ page: 'current' }).nodes();
        rows.each((row) => {
          const rowData = $table.DataTable().row(row).data();
          return parameterTooltip(rowData);
        });

        $deleteConfirmButton.offOn('click.deleteModal', () => {
          const deleteLinkAlias = $deleteModal.attr('data-alias');
          arc.frontend.vizHttp.delete({
            url: arc.frontend.globals.urls.urlShortenerUrl,
            data: { slug: deleteLinkAlias },
            headers: {
              'X-CSRFTOKEN': arc.globals.csrf_token,
            },
          })
            .then(() => {
              $table.DataTable().ajax.reload(null, false);
              $deleteModal.modal('hide');
            })
            .catch((error) => {
              arc.report.alertMessage(error);
            });
        });

        $table.offOn('click.editAlias', '.edit-alias:not(.disabled)', (evt) => {
          const rowIndex = $(evt.currentTarget).data('index');
          const urlOrigin = `${window.location.origin}${arc.frontend.globals.urls.urlShortenerUrl}`;
          const urlAlias = aliases[rowIndex].slug;
          const { dashboardId } = aliases[rowIndex];
          const { parameters } = aliases[rowIndex];
          const reportName = aliases[rowIndex].dashboard_name;
          const reportType = $(evt.currentTarget).data('type');
          const isCreatedOrUsedInEmail = aliases[rowIndex].is_created_in_email;

          arc.frontend.legacyBridge.events.next({
            type: 'open-set-url-alias-modal',
            payload: {
              urlOrigin,
              urlAlias,
              dashboardId,
              reportName,
              parameters,
              reportType,
              isCreatedOrUsedInEmail
            }
          });
        });

        $table.offOn('click.copyAlias', '.copy-to-clipboard-link .url-copy-btn:not(.disabled)', (evt) => {
          const $this = $(evt.target);
          const aliasCell = $this.closest('tr').find('td:first');
          const aliasLink = aliasCell.find('.alias-link').attr('href');

          arc.misc.copyToClipboard(aliasLink).then(() => {
            $this.attr('data-original-title', 'Copied');
            $this.tooltip('update');
            $this.tooltip('show');

            setTimeout(() => {
              $this.attr('data-original-title', 'Copy to clipboard');
              $this.tooltip('update');
            }, 300);
          });
        });

        $table.offOn('click.deleteAlias', '.alias-delete-link .fa-trash:not(.disabled)', (evt) => {
          if (arc.misc.modalShown($deleteModal)) {
            return;
          }

          const $this = $(evt.target);
          const deleteLinkAlias = $this.closest('.alias-delete-link').attr('data-id');
          $deleteModal.attr('data-alias', deleteLinkAlias);

          $deleteModal.modal('show');
        });

        $('[data-toggle="tooltip"]').tooltip();
      }
    });
    $aliasesDataTable.columns.adjust();
  }
}());
