window.arc = window.arc || {};

(function() {
'use strict';

arc.admin = arc.admin || {};

const colorHtml = `
      <li>
        <i class="fa fa-ellipsis-v color-move-handle"></i>
        <label>Color <span class="custom-color-number">%INDEX%</span>:</label>
        <input class="spectrum-color-picker">

        <a style="margin-left:10px;" class="btn btn-secondary viz-btn-xs remove-color-btn">
          <i class="fa fa-remove"></i>
        </a>
      </li>
    `;

arc.admin.CustomColor = function() {
  var CustomColor = function() { return 'arc.admin.CustomColor'; };

  var styleid,
      name,
      type,
      value,
      description;

  CustomColor.styleid = function(_) {
    if (!arguments.length) return styleid;
    styleid = _;
    return CustomColor;
  };

  CustomColor.stylename = function(_) {
    if (!arguments.length) return name;
    name = _;
    return CustomColor;
  };

  CustomColor.styledescription = function(_) {
    if (!arguments.length) return description;
    description = _;
    return CustomColor;
  };

  // Types:
  //  1: distinct values
  //  100: Gradients with values
  //  101: From JSON - (may be used in custom styles for POC)
  //  102: Gradient with percentages
  CustomColor.styletype = function(_) {
    if (!arguments.length) return type;
    type = _;
    return CustomColor;
  };

  CustomColor.stylevalue = function(_) {
    if (!arguments.length) return value;
    value = _;
    return CustomColor;
  };

  CustomColor.delete = function(done_cb) {
    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };

    $.ajax({
      url: `${arc.frontend.globals.urls.colorpaletteUrl}/${styleid}`,
      data: post_data,
      type: 'DELETE',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      arc.report.alertMessage('Failed to delete custom style',
          req.responseText ? req.responseText : req, { is_html: true });
    })
    .always(function() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomColor;
  };

  CustomColor.save = function(done_cb) {

    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      name: name,
      description: description,
      type: type,
      value: type === 1 ? value : JSON.stringify(value),
    };

    let url = arc.frontend.globals.urls.colorpaletteUrl;
    if (styleid) {
      url += '/' + styleid;
    }

    // save event
    $.ajax({
      url: url,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(function(data) {
      styleid = data[0];
    })
    .fail(function(req) {
      arc.report.alertMessage('Failed to ' +
          (styleid ? 'update' : 'create') + ' custom style: ',
          req.responseText ? req.responseText : req);
    })
    .always(function() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomColor;
  };

  return CustomColor;
};

arc.admin.customcolor = function(_styleid) {
  var cs = function() { return 'arc.admin.customcolor'; };

  var styleid = _styleid,
      styletype;

  cs.styletype = function(_) {
    if (!arguments.length) return styletype;
    styletype = _;
    return cs;
  };

  cs.initialize = function() {

    $('#customColorType').removeClass('arc-invisible');
    if (_styleid === 0) {
      setupEdit();
      $('#style-name')[0].focus();
      $('#typeOfColorScale input[value=1]').attr('checked', true).trigger('change');
      return cs;
    }

    fetchAndPaint(function(CustomColor) {
      setupEdit(CustomColor);
    });

    return cs;
  };

  function setupNeedsSave() {
    $('#style-name, [id=customColorContentForType_1] input')
        .offOn('input.requireSave', function() {
          arc.report.pagenav.needs_save(true);
        });

    $('.input-short')
        .offOn('change', function() {
          arc.report.pagenav.needs_save(true);
        });

    $('#style-description')
        .offOn('input.requireSave', function() {
          arc.report.pagenav.needs_save(true);
        });
  }

  function setupEdit(CustomColor) {
    if (!CustomColor) {
      CustomColor = {
        id: null,
        name: null,
        description: null,
        type: null,
        value: [],
      };
    } else {
      var $colorRadio = $('#customColorType');
        $colorRadio.find('input').attr('checked', false);
        $colorRadio.find('input[value="' + CustomColor.type + '"]').prop('checked', true);
    }

    setupNeedsSave();

    $('#custom-style-form').on('submit', function() {
      $('.save-btn').prop('disabled', true);
      var style = arc.admin.CustomColor();
      var data = [], tempdata;
      var chosenType = Number($('#customColorType input[name=color_type]:checked').val());
      style.stylename($('#style-name').val());
      style.styledescription($('#style-description').val());
      var startAxis = 26;
      var $valDiv;

      if (isNaN(chosenType)) {
        chosenType = CustomColor.type;
      }

      var gradx_id = 'gradX_' + chosenType;
      var lengthAxis = $('#gradx_panel_' + gradx_id).width() + 2;

      switch (chosenType) {
        case 100:
          lengthAxis = $('#gradx_panel_' + gradx_id).width() + 2;
          $.each($('#gradx_start_sliders_' + gradx_id + ' > div'), function (idx, div) {
            tempdata = {};
            if (idx === 0) {
              return;
            }

            tempdata.color = div.style.backgroundColor;
            $valDiv = $('#c' + (idx - 1));
            tempdata.value = $valDiv.val();
            tempdata.position = ((div.style.left.split('px')[0] - startAxis) / lengthAxis) * 100;

            data.push(tempdata);
          });

          let prevVal;
          let invalidData = false;
          let alertMessage;

          data.forEach(function(colorData) {
            if (!$.isNumeric(colorData.value) && colorData.value !== '') {
              alertMessage = 'No non-numeric value fields allowed';
              invalidData = true;
              return;
            } else if (prevVal !== undefined && prevVal >= Number(colorData.value) &&
              colorData.value !== '') {
              invalidData = true;
              alertMessage = 'Values are not in ascending order';
              return;
            }

            if (colorData.value !== '') {
              prevVal = Number(colorData.value);
            }
          });

          if (invalidData) {
            arc.report.alertMessageSmall(alertMessage, 'alert-danger', true);
            $('.save-btn').prop('disabled', false);
            return;
          }

          data = { colors: data };
          break;
        case 1:
          const colorSelector1 = 'div [id=customColorContentForType_1]',
                colorSelector2 = 'ul.custom-color-list',
                colorSelector3 = 'input.spectrum-color-picker';

          $.each($(`${colorSelector1} ${colorSelector2} ${colorSelector3}`),
            function (i, ip) {
              let spectrumColor = $(ip).spectrum('get');
              if (spectrumColor === null) {
                data.push('');
              } else {
                data.push(spectrumColor.toRgbString());
              }
            });

          data = data.join('\r\n');

          break;

        case 101:
          data = $('div [id=customColorContentForType_' + chosenType + '] textarea').val();
          break;
        case 102:
          lengthAxis = $('#gradx_panel_' + gradx_id).width() + 2;
          $.each($('#gradx_start_sliders_' + gradx_id + ' > div'), function (idx, div) {
            tempdata = {};
            if (idx === 0) {
              return;
            }

            tempdata.color = div.style.backgroundColor;
            tempdata.position = ((div.style.left.split('px')[0] - startAxis) / lengthAxis) * 100;

            data.push(tempdata);
          });

          data.sort((colorData1, colorData2) => colorData1.position > colorData2.position);

          data = { colors: data };
          break;
      }
      if (chosenType === 1) {
        CustomColor.value = data.split('\r\n');
      } else {
        CustomColor.value = data.colors;
      }

      CustomColor.type = chosenType;
      style.stylevalue(data);
      style.styletype(chosenType);

      if (styleid) {
        style.styleid(styleid);
      }

      function saveCallBack() {
        $('.save-btn').prop('disabled', false);
        arc.report.pagenav.needs_save(false);
        if (!styleid) {
          styleid = style.styleid();
          const newPath = location.pathname.replace(/\d+$/, styleid);
          arc.misc.replaceState(newPath, { html: $('html').html() });
          $('.breadcrumb .active').text(`ID: ${styleid}`);
        } else {
          // Update name if it's changed
          var $title = $('#title-style-name');
          if ($title.text() !== style.stylename()) {
            $title.text(style.stylename());
          }
        }

        if (styleid) {
          arc.report.alertMessageSmall('Save complete', 'alert-success');
        }
      }

      style.save(saveCallBack);
    });

    $('#customColorType').on('change', function (e) {
      customColorTypeEventHandler(CustomColor, e);
    });

  }

  function paintCommon(c) {
    $('.breadcrumb .active').text('ID: ' + styleid);
    $('#title-style-name').text(c.name);
    $('#style-name').val(c.name);
  }

  function addColor($list, color, index) {
    let $colorOption = $(arc.misc.safe_substitute(
        colorHtml,
        {
          INDEX: index,
        }));

    $colorOption.find('.spectrum-color-picker').spectrum({
      color,
      allowEmpty: false,
      showInput: true,
      showPalette: true,
      showAlpha: true,
      preferredFormat: 'hex',
      cancelText: 'Cancel',
      chooseText: 'OK',
      palette: [
        ['#0097a7', '#7dcad2', '#546e7a', '#b0bec5', '#ed8111', '#f6c088', '#e54954', '#f2a4a9'],
        ['#000', '#444', '#666', '#999', '#ccc', '#eee', '#f3f3f3', '#fff'],
        ['#f00', '#f90', '#ff0', '#0f0', '#0ff', '#00f', '#90f', '#f0f'],
        ['#f4cccc', '#fce5cd', '#fff2cc', '#d9ead3', '#d0e0e3', '#cfe2f3', '#d9d2e9', '#ead1dc'],
        ['#ea9999', '#f9cb9c', '#ffe599', '#b6d7a8', '#a2c4c9', '#9fc5e8', '#b4a7d6', '#d5a6bd'],
        ['#e06666', '#f6b26b', '#ffd966', '#93c47d', '#76a5af', '#6fa8dc', '#8e7cc3', '#c27ba0'],
        ['#c00', '#e69138', '#f1c232', '#6aa84f', '#45818e', '#3d85c6', '#674ea7', '#a64d79'],
        ['#900', '#b45f06', '#bf9000', '#38761d', '#134f5c', '#0b5394', '#351c75', '#741b47'],
        ['#600', '#783f04', '#7f6000', '#274e13', '#0c343d', '#073763', '#20124d', '#4c1130'],
      ],
    });

    $list.append($colorOption);
  }

  function transformCustomColor(currentColor, toType) {
      var fromType = currentColor.type;
      var currentValue = currentColor.value;
      var transformedValue = [];
      if (!fromType) {
        if (toType === 1) {
          transformedValue.push('#000000');
        }

        return transformedValue;
      }

      var samples = [0, Math.floor(currentValue.length / 2), currentValue.length - 1];
      switch (toType) {
        case 1:
          currentValue.forEach(function (v) {
            transformedValue.push(v.color);
          });

          break;
        case 100:
          samples.forEach(function (i, i2) {
            transformedValue.push({
              color: currentValue[i].color ? currentValue[i].color : currentValue[i],
              position: currentValue[i].position ? currentValue[i].position : i2 * 50,
              value: currentValue[i].value ? currentValue[i].value : null,
            });
          });

          break;
        case 102:
          samples.forEach(function(i, i2) {
            transformedValue.push({
              color: currentValue[i].color ? currentValue[i].color : currentValue[i],
              position: currentValue[i].position ? currentValue[i].position : i2 * 50,
            });
          });

          break;
      }
    return transformedValue;
  }

      // Expects either click event or the type value
  function customColorTypeEventHandler(CustomColor, e) {
    var typeValue = e && e.target ?
      Number(e.target.value) :
      Number(CustomColor.type);
    var value = CustomColor.value;
    if (typeValue !== CustomColor.type) {
      // TODO: Pass in currentValue instead of CustomColor.value
      value = transformCustomColor(CustomColor, typeValue);
      if (CustomColor.type) {
        arc.report.pagenav.needs_save(true);
      }

    }

    $('div [id^=customColorContentForType_]').addClass('arc-invisible');
    $('div [id=customColorContentForType_' + typeValue + ']')
      .removeClass('arc-invisible');

    switch (typeValue) {
      case 100:
      case 102:
        try {
          if (!value || value.length === 0) {
            value = [
              { color: 'red', position: 0, isdraggable: isTypeDraggable(typeValue) },
              { color: 'yellow', position: 50, isdraggable: isTypeDraggable(typeValue) },
              { color: 'green', position: 100, isdraggable: isTypeDraggable(typeValue) },
            ];
          }
        } catch (f) {
          value = CustomColor.value;
        }

        if (typeValue === 100) {
          // Even if the stored value has some other position values,
          // we show sliders at hardcoded positions for type 100
          value[0].position = 0;
          value[1].position = 50;
          value[2].position = 100;
        }

        var sliders = [];
        var len = value.length;

        value.forEach(function (c, i) {
          if (typeValue === 100) {
            $('#c' + i).val(c.value);
            c.position = c.position || i * 50;
          }

            c.color = c.color.trim();
            sliders.push({
              color: c.color.trim(),
              position: c.position || 0,
              isdraggable: isTypeDraggable(typeValue, c.isdraggable),
            });
        });

        $('[id^=gradX]').empty();

        gradX('#gradX_' + typeValue, {
          sliders,
          change() {
            arc.report.pagenav.needs_save(true);
          },
        });

        break;

      case 1:

        var $contentDiv = $(`div [id=customColorContentForType_${typeValue}]`);
        var $list = $contentDiv.find('.custom-color-list');
        $list.empty();
        value.forEach(function (v, i) {
          if (v.length > 0) {
            addColor($list, v, i + 1);
          }
        });

        const $customColorUl = $('.custom-color-list');
        function updateDistinctUl() {
          $customColorUl.find('.custom-color-number').each(function(i) {
            this.textContent = i + 1;
          });
        }

        $customColorUl.sortable({
          handle: '.color-move-handle',
          update() {
            arc.report.pagenav.needs_save(true);
            updateDistinctUl();
          },
        });

        $customColorUl.off('changeColor.arcSave').on('changeColor.arcSave', function () {
          arc.report.pagenav.needs_save(true);
        });

        $contentDiv.find('.add-more-btn').offOn('click', () => {
          addColor($list, '#00000', $list.find('li').length + 1);
          setupNeedsSave();
          arc.report.pagenav.needs_save(true);
        });

        $contentDiv.find('.paste-colors-btn').offOn('click', () => {
          const $modal = $('#paste-colors-modal');
          if (arc.misc.modalShown($modal)) {
            console.log('modal already open');
            return;
          }

          $modal.modal('show')
                .find('textarea').val('');

          $modal.find('.save-btn')
                .offOn('click.save', function() {
                  const v = $modal.find('textarea').val(),
                        values = v.split(/\s+/);

                  values.forEach(function(color) {
                    if (color.trim()) {
                      addColor($list, color, $list.find('li').length + 1);
                    }
                  });

                  $modal.modal('hide');
                });
        });

        $list.off('click').on('click', '.remove-color-btn', function (event) {
          arc.report.pagenav.needs_save(true);
          $(event.currentTarget).closest('li').remove();
          updateDistinctUl();
        });

        break;
      case 101:
        try {
          value = JSON.stringify(value);
        } catch (f) {
          value = CustomColor.value;
        }

        break;
    }
    setCursorType(typeValue);
  }

  function isTypeDraggable(type, isDraggable) {
    if (isDraggable !== undefined) {
      return isDraggable;
    }

    return type !== 100;
  }

  // set cursor type to pointer if not draggable
  function setCursorType(type) {
    if (!isTypeDraggable(type)) {
      $('.gradx_slider').addClass('non_sliding');
    }
  }

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.colorpaletteUrl}/${styleid}`)
      .await(function(error, CustomColor) {
        if (error) {
          arc.report.alertMessage('Could not load custom styles',
              error);
          return;
        }

        CustomColor = CustomColor[0];
        $('.breadcrumb .active').text('ID: ' + styleid);
        $('#title-style-name').text(CustomColor.name);
        $('#style-name').val(CustomColor.name);
        $('#style-description').val(CustomColor.description);
        CustomColor.value = CustomColor.value || (CustomColor.type === 100 ? '{"colors":[]}' : '');
        if (!CustomColor.type || CustomColor.type < 100) {
          CustomColor.type = 1;
          CustomColor.value = CustomColor.value.split('\r\n');

        } else if (CustomColor.type === 100 || CustomColor.type === 102) {
          try {
            CustomColor.value = JSON.parse(CustomColor.value).colors;
            if (!CustomColor.value) {
              throw new Error('Invalid data');
            }
          } catch (e) {
            CustomColor.value = [];
          }
        } else if (CustomColor.type === 101) {
          CustomColor.value = JSON.parse(CustomColor.value);
        }

        customColorTypeEventHandler(CustomColor);

        // Based on type update section accordingly
        var $colorRadio = $('#customColorType');
        $colorRadio.find('input').attr('checked', false);
        $colorRadio.find('input[value="' + CustomColor.type + '"]').attr('checked', true);
        $('div [id^=customColorContentForType_]').addClass('arc-invisible');
        $('div [id=customColorContentForType_' + CustomColor.type + ']')
          .removeClass('arc-invisible');

        if (cb) {
          cb(CustomColor);
        }
      });
  }

  return cs;
};

arc.admin.customcolors = function() {
  var cs = function() { return 'arc.admin.customcolors'; };

  var styles = {};

  cs.initialize = function() {
    if (!arc.globals.enable_custom_styles) {
      arc.report.alertMessageSmall('Custom styles are not enabled. You can ' +
          'still create custom styles here, but you need to enable them for ' +
          'use using the "Site Settings" page',
          'alert-warning');
    }

    fetchAndPaint();
    return cs;
  };

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.colorpaletteUrl)
      .defer(d3.json, arc.frontend.globals.urls.colorPaletteUsageUrl)
      .await(function(error, CustomColors, cpUsage) {
        if (error) {
          arc.report.alertMessage('Could not load custom styles',
              error);
          return;
        }

        // ARC-10089: Reverse array so newest is on top
        paintTable(CustomColors.reverse(), cpUsage);
        if (cb) {
          cb();
        }
      });
  }

  function getTypeIcon(type) {
    var ret = '';
     switch (type) {
       case 100:
         ret = '<div class="gradient-icon"><span>#</span></div>';
         break;
       case 101:

         // Dim-color mapping
         ret = '<div>Dim-color mapping</div>';
         break;
       case 1:

         // Discrete
         ret = '<div class="discrete-icon-1"></div>' +
                '<div class="discrete-icon-2"></div>' +
                '<div class="discrete-icon-3"></div>';
         break;
       case 102:
         ret = '<div class="gradient-icon"><span>%</span></div>';
         break;
     }
    return ret;
  }

  function paintTable(customColors, cpUsage) {
    var $tbody = $('#table-customcolors tbody');
    $tbody.empty();

    var templateIn = $('#table-customcolors-template').html();
    customColors.forEach(function(c) {
      var apps = cpUsage[c.id];
      styles[c.id] = c.name;
      $tbody.append(arc.misc.safe_substitute(
          templateIn,
          {
            ID: c.id,
            NAME: c.name,
            DESCRIPTION: c.description,
            HTML_TYPE: getTypeIcon(c.type),
            VALUE: c.value,
            CSS_COUNT: (apps ? apps.length : 0),
            HTML_APPS: arc.misc.appsToLinks(apps, 10),
          }));
    });

    $tbody.find('a.delete')
          .off('click')
          .on('click', function() {
            var styleid = Number($(this).closest('tr').attr('data-styleid'));
            var c = arc.misc.deleteConfirmation();
            c.objectType('Custom Color')
             .objectName(styles[styleid])
             .askUser(function() {
               arc.admin.CustomColor()
                        .styleid(styleid)
                        .delete(function() {
                          fetchAndPaint(function() {
                            c.hideModal();
                          });
                        });
             });
          });
  }

  return cs;
};

}());
