window.arc = window.arc || {};

(function() {
'use strict';

arc.admin = arc.admin || {};

arc.admin.CustomDate = function() {
  const CustomDate = function() { return 'arc.admin.CustomDate'; };

  var styleid,
      stylename,
      ranges;

  CustomDate.styleid = function(_) {
    if (!arguments.length) return styleid;
    styleid = _;
    return CustomDate;
  };

  CustomDate.stylename = function(_) {
    if (!arguments.length) return stylename;
    stylename = _;
    return CustomDate;
  };

  CustomDate.ranges = function(_) {
    if (!arguments.length) return ranges;
    ranges = _;
    return CustomDate;
  };

  CustomDate.save = function(done_cb) {
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      value: ranges,
      name: stylename,
    };

    let url = arc.frontend.globals.urls.daterangesUrl;
    if (styleid) {
      url += `/${styleid}`;
    }

    // save event
    $.ajax({
      url: url,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(function(data) {
      styleid = data[0];
    })
    .fail(function(req) {
      arc.report.alertMessage('Failed to ' +
          (styleid ? 'update' : 'create') + ' custom date: ',
          req.responseText ? req.responseText : req);
    })
    .always(function() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomDate;
  };

  CustomDate.delete = function(done_cb) {
    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };

    $.ajax({
      url: `${arc.frontend.globals.urls.daterangesUrl}/${styleid}`,
      data: post_data,
      type: 'DELETE',
      beforeSend: function(xhr) {
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      arc.report.alertMessage('Failed to delete custom date',
          req.responseText ? req.responseText : req, { is_html: true });
    })
    .always(function() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomDate;
  };

  return CustomDate;
};

arc.admin.customdate = function(_styleid) {
  const cd = function() { return 'arc.admin.customdate'; };

  let styleid = _styleid;
  const rulesMath = { SUBTRACT: true, ADD: true };
  const rulesStartEnd = { STARTOF: true, ENDOF: true };

  let unique_daterange_id = 0;

  cd.initialize = function() {
    if (_styleid === 0) {
      setupEdit();
      $('#style-name').focus();
      return cd;
    }

    fetchAndPaint(setupEdit);

    return cd;
  };

  function setupEdit(daterange) {
    const trs = [];

    if (!daterange) {
      $('.clone-btn').prop('disabled', true);
      trs.push(getNewRow());
    } else {
      const ranges = daterange.value;
      if (!ranges) {
        return;
      }

      ranges.split('\n').forEach((range) => {
        const values = range.split(',').map((s) => s.toUpperCase().trim());

        if (values.length === 3) {
          const regexForFixedDate = /^\d{4}-\d{2}-\d{2}$/;
          const regexForFixedTime = /^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$/;
          let [rangeName, startTime, endTime] = values;

          const $tr = getNewRow();

          const startAnchorIndex = startTime.search(/\s\D/) > -1 ?
            startTime.search(/\s\D/) : startTime.length;
          const endAnchorIndex = endTime.search(/\s\D/) > -1 ?
            endTime.search(/\s\D/) : endTime.length;

          const anchorStart = startTime.substring(0, startAnchorIndex).trim();
          startTime = startTime.substring(startAnchorIndex, startTime.length).trim();
          const anchorEnd = endTime.substring(0, endAnchorIndex).trim();
          endTime = endTime.substring(endAnchorIndex, endTime.length).trim();

          parseRules(startTime).forEach((rule) => {
            addRule($tr, 'start', rule);
          });

          parseRules(endTime).forEach((rule) => {
            addRule($tr, 'end', rule);
          });

          const inputValues = [anchorStart, anchorEnd].map((value) => {
            if (regexForFixedDate.test(value)) {
              return 'fixed-date';
            }

            if (regexForFixedTime.test(value)) {
              return 'fixed-time';
            }

            return value;
          });

          $tr.find('.daterange-name-input').val(rangeName);

          $tr.find('.daterange-start-anchor-input').val(anchorStart);
          $tr.find(`.daterange-start-anchor-dropdown input[type=radio][value=${inputValues[0]}]`).prop('checked', true);

          $tr.find('.daterange-end-anchor-input').val(anchorEnd);
          $tr.find(`.daterange-end-anchor-dropdown input[type=radio][value=${inputValues[1]}]`).prop('checked', true);

          trs.push($tr);
        }
      });
    }

    const $tbody = $('.custom-daterange-table-body');
    trs.forEach(function($tr) {
      $tbody.append($tr);
    });

    $tbody.on('input.requireSave', 'select, input[type=text]', function() {
      arc.report.pagenav.needs_save(true);
    });

    $tbody.sortable({
      handle: '.daterange-move-handle',
      update() {
        arc.report.pagenav.needs_save(true);
      },
    });

    $tbody.on('click', 'a', function(e) {
      arc.report.pagenav.needs_save(true);

      const $this = $(this);
      const $tr = $this.closest('tr');
      if ($this.hasClass('add-more-btn')) {
        const $ul = $this.closest('ul');
        if ($ul.hasClass('daterange-start-rule-list')) {
          addRule($tr, 'start');
        } else if ($ul.hasClass('daterange-end-rule-list')) {
          addRule($tr, 'end');
        }
      } else if ($this.hasClass('remove-rule-btn')) {
        $this.closest('.daterange-rule').remove();
      } else if ($this.hasClass('remove-range-btn')) {
         if ($('.custom-daterange-table-body').find('tr').length === 1) {
          // Require a minimum of at least one range
          return;
         }

         $tr.remove();
      }
    });

    setupDatePickerHandlers();

    $('#style-name').on('input.requireSave', () => arc.report.pagenav.needs_save(true));

    $('#add-range-btn')
      .off('click')
      .on('click', function() {
        arc.report.pagenav.needs_save(true);
        $tbody.append(getNewRow());
        setupDatePickerHandlers();
      });

    $('#custom-style-form').on('submit', function() {
      let style = arc.admin.CustomDate()
                           .stylename($('#style-name').val())
                           .styleid(styleid)
                           .ranges(stripAndProcessRows());

      const $saveBtn = $('.save-btn');

      if (!checkSaveAbility()) {
        const message = 'Custom date ranges require a name. ' +
          'Please fill out the all name fields and then save';
        arc.report.alertMessageSmall(message, 'alert-danger');
        return;
      }

      $saveBtn.prop('disabled', true);

      function saveCallBack() {
        $('.save-btn').prop('disabled', false);
        arc.report.pagenav.needs_save(false);
        if (!styleid) {
          styleid = style.styleid();
          const newPath = location.pathname.replace(/\d+$/, styleid);
          arc.misc.replaceState(newPath, { html: $('html').html() });
          $('.breadcrumb .active').text(`ID: ${styleid}`);
        } else {
          // Update name if it's changed
          var $title = $('#title-style-name');
          if ($title.text() !== style.stylename()) {
            $title.text(style.stylename());
          }
        }

        arc.report.alertMessageSmall('Save complete', 'alert-success');
      }

      style.save(saveCallBack);
    });

    $('#clone-link').off('click').on('click', function() {
      if (!checkSaveAbility()) {
        return;
      }
      $('.clone-btn').prop('disabled', true);
      let style =
        arc.admin.CustomDate()
           .stylename(`Clone of ${$('#style-name').val()}`)
           .ranges(stripAndProcessRows());

      style.save(function() {
        $('.clone-btn').prop('disabled', false);
        arc.report.pagenav.needs_save(false);
        arc.report.alertMessageSmall('Clone complete', 'alert-success');
        window.open(style.styleid(), '_self');
      });
    });
  }

  function setupDatePickerHandlers() {
    // Need this to prevent dropdowns from closing when selecting a date in
    // the datepickers
    $('.daterangepicker').off('click').on('click', (e) => e.stopPropagation());
    $('.daterangepicker .applyBtn').off('click').on('click', (e) => e.preventDefault());
  }

  function checkSaveAbility() {
    if ($('#style-name').val() === '') {
      return false;
    }

    let hasEmptyName = false;
    $('.custom-daterange-wrapper .daterange-name-input').each(function (index, item) {
      if ($(item).val() === '') {
        hasEmptyName = true;
      }
    });

    return !hasEmptyName;
  }

  function getNewRow() {
    const text = arc.misc.safe_substitute($('.daterange-tr:last')[0].outerHTML, {
      ID: unique_daterange_id
    });
    const $tr = $(text);

    unique_daterange_id += 1;

    setupAnchorPoint('daterange-start-anchor', $tr);
    setupAnchorPoint('daterange-end-anchor', $tr);

    return $tr;
  }

  function addRule($tr, selectorString, config) {
    const $rule = $($('#daterange-rule-li-template').html());
    const $ruleDropdown = $rule.find('.daterange-rule-dropdown');
    const $unitDropdown = $rule.find('.daterange-unit-dropdown');
    const $numberDropdown = $rule.find('.daterange-number-dropdown');
    $tr.find(`.daterange-${selectorString}-rule-list .add-more-btn-container`)
       .before($rule);

    if (config) {
      const { rule, number, unit } = config;
      $ruleDropdown.find('option').prop('selected', false);
      $ruleDropdown.find(`option[value=${rule}]`).prop('selected', true);
      $unitDropdown.find('option').prop('selected', false);
      $unitDropdown.find(`option[value=${unit}]`).prop('selected', true);
      $unitDropdown.find(`option[value=${unit.substring(0, unit.length - 1)}]`)
                   .prop('selected', true);
      generateNumberDropdown($numberDropdown, $unitDropdown.val());
      toggleNumberDropdown($rule, rule, number);
    } else {
      generateNumberDropdown($numberDropdown, $unitDropdown.val());
    }

    $ruleDropdown.off('change').on('change', function() {
      const ruleName = $(this).find(':selected').text();
      toggleNumberDropdown($rule, ruleName);
    });

    $unitDropdown.off('change').on('change', function() {
      generateNumberDropdown($numberDropdown, $unitDropdown.val());
    });
  }

  function toggleNumberDropdown($rule, ruleName, num) {
    const $dropdown = $rule.find('.daterange-number-dropdown');
    $dropdown.toggle(Boolean(rulesMath[ruleName]));

    if (num) {
      $dropdown.find('option').prop('selected', false);
      $dropdown.find(`option[value=${num}]`).prop('selected', true);
    }
  }

  const numbersToShow = {
    SECOND: 60,
    MINUTE: 120,
    HOUR: 48,
    DAY: 60,
    WEEK: 52,
    MONTH: 24,
    QUARTER: 40,
    YEAR: 10,
  };

  function generateNumberDropdown($dropdown, unit) {
    const currVal = $dropdown.val() || 1;
    let optionHTML = '';
    for (var i = 1; i < numbersToShow[unit] + 1; i++) {
      optionHTML += `<option value="${i}"${i === Number(currVal) ? ' selected' : ''}>${i}</option>`;
    }

    $dropdown.html(optionHTML);
  }

  function setupAnchorPoint(selectorString, $tr) {
    // This function is used to setup an Anchor Dropdown for both start and End
    const $dropdown = $tr.find(`.${selectorString}-dropdown`);
    const $input = $tr.find(`.${selectorString}-input`);
    $dropdown.on('click', function(e) {
      e.stopPropagation();
    });

    $dropdown.on('submit', function(e) {
      e.stopPropagation();
    });

    const $fixedTimeInput = $dropdown.find('div.fixed-time .form-control-sm');
    const $fixedDateInput = $dropdown.find('div.fixed-date .form-control-sm');

    setupRadioButtons($dropdown);
    setupDatePicker($fixedTimeInput, true);
    setupDatePicker($fixedDateInput);

    $dropdown
      .find('.applyBtn')
      .off('click')
      .on('click', function(e) {
        let anchorVal = $dropdown.find('input[type=radio]:checked').val();

        if (anchorVal === 'fixed-date') {
          anchorVal = $fixedDateInput.val();
        } else if (anchorVal === 'fixed-time') {
          anchorVal = $fixedTimeInput.val();
        }

        $input.val(anchorVal);
        $input.trigger('change');
        $input.dropdown('toggle');
      });

    $dropdown
      .find('.cancelBtn')
      .off('click')
      .on('click', function(e) {
        e.stopPropagation();
        $input.dropdown('toggle');
      });
  }

  function parseRules(ruleString) {
    const tokens = ruleString.trim().split(/\s+/);
    const rules = [];
    while (tokens.length) {
      const rule = tokens.shift();
      if (rulesMath[rule]) {
        const number = tokens.shift();
        const unit = tokens.shift();
        rules.push({ rule, number, unit });
      } else if (rulesStartEnd[rule]) {
        const unit = tokens.shift();
        rules.push({ rule, unit });
      }
    }

    return rules;
  }

  function setupDatePicker($sel, isTime) {
    const timeformat = isTime ? 'YYYY-MM-DD HH:mm:ss' : 'YYYY-MM-DD';
    $sel.val(moment.utc().format(timeformat));
    const opts = {
      singleDatePicker: true,
      showDropdowns: true,
      locale: { format: timeformat },
      format: timeformat,
    };

    if (isTime) {
      opts.timePicker = true;
    }

    $sel.daterangepicker(opts);
  }

  function setupRadioButtons($div) {
    $div.find('input[type=radio]')
        .off('click')
        .on('click', function() {
          const name = $(this).attr('name');
          const val = $(this).val();
          chooseRadio($div, name, val);
        });
  }

  function chooseRadio($sel, name, value) {
    const $a = $sel.find(`input[type=radio][name=${name}][value=${value}]`);
    if ($a.length === 0) {
      return;
    }

    $sel.find('input[type=radio]:checked').prop('checked', false);
    $a.prop('checked', true);
  }

  function stripAndProcessRows() {
    let range = '';
    $('.custom-daterange-table-body tr').each(function(i, tr) {
      const $tr = $(tr);
      const name = $tr.find('.daterange-name-input').val().trim();
      const startDate = stripAnchorAndRules('start', $tr);
      const endDate = stripAnchorAndRules('end', $tr);

      range += `${name}, ${startDate}, ${endDate}\n`;
    });

    return range;
  }

  function stripAnchorAndRules(selector, $tr) {
    const anchor = $tr.find(`.daterange-${selector}-anchor-input`).val().trim();
    const rules = stripRule($tr.find(`.daterange-${selector}-rule-input`));
    return `${anchor} ${rules}`.trim();
  }

  function stripRule($div) {
    let overallRule = '';
    $div.find('.daterange-rule').each(function(i, li) {
      const $li = $(li);
      const rule = $li.find('.daterange-rule-dropdown :selected').val();
      const num = $li.find('.daterange-number-dropdown :selected').val();
      const unit = $li.find('.daterange-unit-dropdown :selected').val();

      if (rulesMath[rule]) {
        overallRule = `${overallRule} ${rule} ${num} ${unit}`.trim();
      } else {
        overallRule = `${overallRule} ${rule} ${unit}`.trim();
      }
    });

    return overallRule;
  }

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.daterangesUrl}/${styleid}`)
      .await(function(error, dateranges) {
        if (error) {
          arc.report.alertMessage('Could not load custom date', error);
          return;
        }

        const [daterange] = dateranges;

        $('.breadcrumb .active').text('ID: ' + styleid);
        $('#title-style-name').text(daterange.name);

        // PUT THIS BACK IN AFTER MIGRATION FOR DESCRIPTIONS

        // $('#style-description').text(daterange.description);
        $('#style-name').val(daterange.name);

        if (cb) {
          cb(daterange);
        }
      });
  }

  return cd;
};

arc.admin.customdates = function() {
  const cd = function() { return 'arc.admin.customdates'; };

  const allDateranges = {};

  cd.initialize = function() {
    if (!arc.globals.enable_custom_styles) {
      arc.report.alertMessageSmall("Custom styles are not enabled. " +
          "You can still create custom styles here, but you need to enable them for " +
          "use using the 'Site Settings' page",
          "alert-warning");
    }

    fetchAndPaint();
    return cd;
  };

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.daterangesUrl)
      .await(function(error, dateranges) {
        if (error) {
          arc.report.alertMessage('Could not load custom dateranges', error);
          return cd;
        }

        // ARC-10089: Reverse order to display newest first
        paintTable(dateranges.reverse());
        if (cb) {
          cb();
        }
      });
  }

  function paintTable(dateranges) {
    const $tbody = $('#table-customdates tbody');
    $tbody.empty();

    const templateIn = $('#table-customdates-template').html();
    dateranges.forEach(function(dr) {
      allDateranges[dr.id] = dr.name;
      $tbody.append(arc.misc.safe_substitute(
          templateIn,
          {
            ID: dr.id,
            NAME: dr.name,
          }));
      const range = arc.misc.rangesFromText(dr.value);

      if (range) {
        $tbody.find(`tr[data-styleid=${dr.id}] .date-range-input`)
        .html(arc.dateranges.getQuickFilterHTML(range, true));
      }
    });

    $tbody.find('a.delete')
          .off('click.del_date')
          .on('click.del_date', function deleteDateRange() {
            const styleid = Number($(this).closest('tr').attr('data-styleid'));
            const c = arc.misc.deleteConfirmation();
            c.objectType('Custom Date')
             .objectName(allDateranges[styleid])
             .askUser(function() {
               arc.admin.CustomDate()
                        .styleid(styleid)
                        .delete(function() {
                          fetchAndPaint(function() {
                            c.hideModal();
                          });
                        });
             });
          });
  }

  return cd;
};

}());

