window.arc = window.arc || {};

(function() {
'use strict';

function hasScript(str) {
  const sel = new DOMParser().parseFromString(str, 'text/html').querySelector('script');
  return (sel !== null);
}

arc.admin = arc.admin || {};

arc.admin.CustomEmail = function(templateInfo) {
  const CustomEmail = function() { return 'arc.admin.CustomEmail'; };
  const $editorContainer = $('.arc-htmleditor');
  const $editor = $editorContainer.find('.arc-htmleditor-predecessor');
  const $name = $('#style-name');
  const $description = $('#style-description');
  const $title = $('#title-style-name');
  const $cloneLink = $('#clone-link');
  const $saveButton = $('.save-btn');
  const templates = {
    'arc-email-template-1': $('#arc-email-template-1').html(),
    'arc-email-template-2': $('#arc-email-template-2').html(),
  };

  let customEmailId = templateInfo && templateInfo.id,
      name = templateInfo && templateInfo.template_name,
      template = templateInfo && templateInfo.template_value,
      description = templateInfo && templateInfo.template_description;

  function handleChange() {
    $editor.summernote('editor.saveRange');
    const tempContent = $editor.summernote().data('summernote').code();
    if (tempContent !== template) {
      template = tempContent;
      arc.report.pagenav.needs_save(true);
    }
  }

  // Insert text before select -- prefer over SummerNote's insertText or insertNode
  // because don't add unnecessary div's
  // https://stackoverflow.com/questions/8284588/add-element-before-after-text-selection
  function insertTextOrHTML(html) {
    let sel, range, node, selection;
    if (window.getSelection) {
      // IE9 and non-IE
      sel = window.getSelection();
      if (sel.getRangeAt && sel.rangeCount) {
        range = sel.getRangeAt(0);

        if (!sel.isCollapsed) {
          sel.deleteFromDocument();
        }

        const nodeToInsert = $(html)[0];
        range.insertNode(nodeToInsert);

        // https://stackoverflow.com/questions/26456991/... (copy from next line)
        // br-inserted-in-contenteditable-span-fails-to-move-cursor
        // <br> will not show up if it's the last child within a parent
        // In that case, detect and add another br
        if (nodeToInsert.tagName === 'BR' && !nodeToInsert.nextElementSibling) {
          range.insertNode($(html)[0]);
        }

        range.collapse(false);
      }
    } else if (document.selection && document.selection.createRange) {
      // IE < 9
      selection = document.selection;
      range = range.createRange();
      selection.deleteFromDocument();
      range.pasteHTML(html);
    }
  }

  CustomEmail.initialize = function(showCodeView) {
    function AddParams() {
      const { ui } = $.summernote;
      const dropdownText = $('#arc-button-container').text();
      const buttonText = $('#arc-button-text').text();

      // create button
      return ui.buttonGroup([
        ui.button({
            className: 'dropdown-toggle',
            contents: dropdownText,
            tooltip: 'Job Parameters',
            data: {
                toggle: 'dropdown',
            },
        }),
        ui.dropdown({
            className: 'dropdown-style',
            contents: buttonText,
            click(e) {
              const $insertText = $(e.target).closest('a');
              let insertText = $insertText.data('arcInsert') ||
                               $insertText.attr('title');
              const $insertNode = $('<span>').html(insertText);
              $editor.summernote('editor.restoreRange');
              $editor.summernote('editor.focus');

              // Remember messagebody previous location so can be replaced later
              let match;
              if ($insertText.data('isBody')) {
                const code = $editor.summernote('code');
                const myRegexp = /(<messagebody>[\s\S]*<\/messagebody>)/;
                match = myRegexp.exec(code);
                if (match) {
                  insertText = match[1];
                }
              }

              insertTextOrHTML($insertNode.html());

              // Don't allow duplicates of messagebody
              if (match) {
                let code = $editor.summernote('code');
                code = code.replace(insertText, '');
                code = code.replace($insertNode.html(), insertText);
                $editor.summernote('code', code);
              }
            },
        }),
      ]).render();
    }

    const customIndentation = arc.misc.customSummerNoteTabs();

    const options = {
      height: 480,                 // set editor height
      minHeight: null,             // set minimum height of editor
      maxHeight: null,             // set maximum height of editor
      focus: false,                 // set focus to editable area after initializing summernote
      lang: 'en-US',               // default: 'en-US'
      dialogsFade: true,           // Add fade effect on dialogs
      airMode: false,
      disableDragAndDrop: false,
      shortcuts: true,
      tabDisable: true,
      tabSize: 4,
      toolbar: [
        ['controls', ['undo', 'redo', 'clear']],
        ['arcadiaButtons', ['arcadiaJobParameters']],
        ['color', ['color']],
        ['font', ['style', 'fontname', 'fontsize', 'height']],
        ['style', ['bold', 'italic', 'underline']],
        ['style2', ['strikethrough', 'superscript', 'subscript']],
        ['para', ['ul', 'ol', 'paragraph']],
        ...customIndentation.toolbar,
        ['insert', ['hr', 'link', 'table', 'picture']],
      ],
      onCreateLink(link) {
        if (hasScript(link)) {
          console.log('specified link in not permitted');
          return '';
        }

        return link;
      },
      buttons: {
        arcadiaJobParameters: AddParams,
        ...customIndentation.buttons
      },
      popover: {
        image: [
          ['imagesize', ['imageSize100', 'imageSize50', 'imageSize25']],
          ['remove', ['removeMedia']],
        ],
        link: [
          ['link', ['linkDialogShow', 'unlink']],
        ],
      },
      fontNames: ['Andale Mono', 'Arial', 'Arial Black', 'Book Antiqua', 'Comic Sans MS',
        'Courier New', 'Georgia', 'Helvetica', 'Impact', 'Lato', 'Roboto', 'Symbol', 'Tahoma',
        'Terminal', 'Times New Roman', 'Trebuchet MS', 'Verdana', 'Webdings', 'Wingings'],
      fontNamesIgnoreCheck: ['Andale Mono', 'Book Antiqua', 'Georgia', 'Lato', 'Roboto',
        'Symbol', 'Terminal', 'Times New Roman', 'Trebuchet MS', 'Webdings', 'Wingings'],
      fontSizes: ['6', '8', '10', '11', '12', '14', '16', '18', '20', '22', '24', '26',
        '28', '30', '32', '34', '36', '38', '40', '48', '64', '82', '150'],
      callbacks: {
        onInit() {
          // summernote onChange isn't called when changes are made in codeEditor section
          $editorContainer.on('keyup', '.note-editing-area', handleChange)
                          .on('paste', '.note-editing-area', handleChange)
                          .on('click', '.note-editing-area', handleChange)
                          .on('change', '.note-editing-area', handleChange);
        },

        onChange: handleChange,

        onEnter(e) {
          e.preventDefault();
          insertTextOrHTML('<br>');
        },
      },
    };

    if (showCodeView) {
      options.toolbar.push(['misc', ['codeview']]);
    }

    // Add code before initializing summernote
    if (customEmailId) {
      $editor.html(template);
      $('.breadcrumb .active').text(`ID: ${customEmailId}`);
    } else {
      $editor.html(templates['arc-email-template-1']);
      $('.breadcrumb .active').text('');
    }

    $editor.summernote(options);
    $editor.summernote('enable');
    $name.val(name);
    $description.val(description);
    $title.text(name);
    $name.on('input.nameOnChange', () => arc.report.pagenav.needs_save(true));

    $description.on('input.descriptionOnChange', () => arc.report.pagenav.needs_save(true));

    $('#custom-style-form').on('submit', function customStyleOnSubmit() {
      $saveButton.prop('disabled', true);
      CustomEmail.emailname($name.val())
                 .emaildescription($description.val())
                 .template($editor.summernote().data('summernote').code());

      CustomEmail.save(function enabledSaveButton() {
        $saveButton.prop('disabled', false);
      });
    });

    $('.pre-made-templates').on('click.pasteTemplates', function useTemplate(e) {
      $editor.summernote('code', templates[e.target.name]);
    });

    $cloneLink.on('click', function cloneTemplate() {
      $cloneLink.prop('disabled', true);
      CustomEmail.emailname(
        $name.val())
                 .customEmailId(null)
                 .emailname(`Clone of ${$name.val()}`)
                 .emaildescription($description.val())
                 .template($editor.summernote().data('summernote').code());

      CustomEmail.save(function enableCloneButton() {
        $cloneLink.prop('disabled', false);
      });
    });
  };

  CustomEmail.customEmailId = function(_) {
    if (!arguments.length) return customEmailId;
    customEmailId = _;
    return CustomEmail;
  };

  CustomEmail.emailname = function(_) {
    if (!arguments.length) return name;
    name = _;
    return CustomEmail;
  };

  CustomEmail.emaildescription = function(_) {
    if (!arguments.length) return description;
    description = _;
    return CustomEmail;
  };

  CustomEmail.template = function(_) {
    if (!arguments.length) return template;
    template = _;
    return CustomEmail;
  };

  CustomEmail.delete = function(done_cb, success_cb) {
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };
    $.ajax({
      url: `${arc.frontend.globals.urls.customEmailUrl}/${customEmailId}`,
      data: post_data,
      type: 'DELETE',
      beforeSend(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(function deleteEmailTemplateDone() {
      arc.report.alertMessageSmall('Deleted email template', 'alert-success');
      success_cb();
    })
    .fail(function deleteEmailTemplateFail(req) {
      arc.report.alertMessage('Failed to delete email template',
          req.responseText ? req.responseText : req, { is_html: true });
    })
    .always(function deleteEmailTemplateAlways() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomEmail;
  };

  CustomEmail.save = function(done_cb) {
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      name,
      description,
      template,
    };

    if (hasScript(template)) {
      arc.report.alertMessage('Failed to ' +
          (customEmailId ? 'update' : 'create') + ' email template: ' +
          'template contains executable code');
      return CustomEmail;
    }

    let url = arc.frontend.globals.urls.customEmailUrl;
    if (customEmailId) {
      url = `${url}/${customEmailId}`;
    }

    // save event
    $.ajax({
      url,
      data: post_data,
      type: 'POST',
      beforeSend(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(function saveEmailDone(data) {
      $saveButton.prop('disabled', false);
      arc.report.pagenav.needs_save(false);
      if (!customEmailId) {
        customEmailId = data[0];
        window.open(customEmailId, '_self');
      } else {
        arc.report.alertMessageSmall('Save complete', 'alert-success');

        // Update name if it's changed
        if ($title.text() !== name) {
          $title.text(name);
        }
      }
    })
    .fail(function saveEmailFail(req) {
      arc.report.alertMessage('Failed to ' +
          (customEmailId ? 'update' : 'create') + ' email template: ',
          req.responseText ? req.responseText : req);
    })
    .always(function saveEmailAlways() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomEmail;
  };

  return CustomEmail;
};

arc.admin.CustomEmailList = function(emailTemplates) {
  const customEmailsList = function() { return 'arc.admin.customemails'; };

  const templateInfo = emailTemplates.reverse();
  const customEmailIdToName = {};
  customEmailsList.initialize = function() {
    paintTable();
    return customEmailsList;
  };

  function paintTable() {
    const $tbody = $('#table-customemails tbody');
    $tbody.empty();

    const templateIn = $('#table-customemails-template').html();
    templateInfo.forEach(function paintTableRow(c) {
      const schedules = c.schedule_ids;
      customEmailIdToName[c.id] = c.template_name;
      $tbody.append(arc.misc.safe_substitute(
          templateIn,
          {
            ID: c.id,
            NAME: c.template_name,
            DESCRIPTION: c.template_description,
            CSS_COUNT: (schedules ? schedules.length : 0),
            HTML_APPS: ` ${schedules.join(', ')}`,
          }));
    });

    $tbody.find('a.delete')
          .off('click')
          .on('click', function clickDeleteRow() {
            const $row = $(this).closest('tr');
            const customEmailId = Number($row.data('styleid'));
            const c = arc.misc.deleteConfirmation();
            c.objectType('Email Template')
             .objectName(customEmailIdToName[customEmailId])
             .askUser(function successfullyDeleteRow() {
               arc.admin.CustomEmail()
                        .customEmailId(customEmailId)
                        .delete(function() {
                          c.hideModal();
                        }, function () {
                          $row.remove();
                        });
             });
          });
  }

  return customEmailsList;
};
}());
