window.arc = window.arc || {};

(function() {
"use strict";

arc.admin = arc.admin || {};

arc.admin.CustomStyle = function() {
  var CustomStyle = function() { return "arc.admin.CustomStyle"; };

  var styleid,
      name,
      css,
      js,
      description;

  CustomStyle.styleid = function(_) {
    if (!arguments.length) return styleid;
    styleid = _;
    return CustomStyle;
  };

  CustomStyle.stylename = function(_) {
    if (!arguments.length) return name;
    name = _;
    return CustomStyle;
  };

  CustomStyle.styledescription = function(_) {
    if (!arguments.length) return description;
    description = _;
    return CustomStyle;
  };

  CustomStyle.css = function(_) {
    if (!arguments.length) return css;
    css = _;
    return CustomStyle;
  };

  CustomStyle.js = function(_) {
    if (!arguments.length) return js;
    js = _;
    return CustomStyle;
  };

  CustomStyle.delete = function(done_cb) {
    var post_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
    };
    $.ajax({
      url: `${arc.frontend.globals.urls.customextensionUrl}/${styleid}`,
      data: post_data,
      type: 'DELETE',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      arc.report.alertMessage("Failed to delete custom style",
          req.responseText ? req.responseText : req, { is_html: true });
    })
    .always(function() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomStyle;
  };

  CustomStyle.save = function(done_cb) {
    var post_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
      'name': name,
      'description': description,
      'css': css,
      'js': js
    };
    let url = arc.frontend.globals.urls.customextensionUrl;
    if (styleid) {
      url += "/" + styleid;
    }

    // save event
    $.ajax({
      url: url,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      }
    })
    .done(function(data) {
      styleid = data[0];
    })
    .fail(function(req) {
      arc.report.alertMessage("Failed to " +
          (styleid ? "update" : "create") + " custom style: ",
          req.responseText ? req.responseText : req);
    })
    .always(function() {
      if (done_cb) {
        done_cb();
      }
    });

    return CustomStyle;
  };

  return CustomStyle;
};

arc.admin.customstyle = function(_styleid) {
  var cs = function() { return "arc.admin.customstyle"; };

  var styleid = _styleid;

  var jsEditor,
      cssEditor;

  cs.setupEditorDivs = function() {
    const $autocompleteOnOffButton = $(".autocomplete-wrapper");
    const editorSettings = {
          autocompleteType: 'customstyle',
          autocompleteOnOffButton: $autocompleteOnOffButton,
          excludeAutocomplete: ['keyWordCompleter'],
          aceOptions: {
            theme: "ace/theme/chrome",
            showGutter: true,
            wrap: false,
            tabSize: 2,
          }
        };

    cssEditor = arc.utils.autocompleteEditor("style-css", 'css', editorSettings);
    jsEditor = arc.utils.autocompleteEditor("style-js", 'javascript', editorSettings);
  };

  cs.initialize = function() {
    cs.setupEditorDivs();
    if (_styleid === 0) {
      setupEdit();
      $("#style-name")[0].focus();
      return cs;
    }
    fetchAndPaint(function() {
      setupEdit();
      cssEditor.focusEditor();
      jsEditor.focusEditor();
    });
    return cs;
  };

  function setupEdit() {
    jsEditor.getSession()
            .on('change', function(e) {
              arc.report.pagenav.needs_save(true);
            });
    cssEditor.getSession()
             .on('change', function(e) {
               arc.report.pagenav.needs_save(true);
             });

    $('#style-name').on('input.onNameChange', () => arc.report.pagenav.needs_save(true));
    $('#style-description').on('input.onDescripChange', () => arc.report.pagenav.needs_save(true));

    $("#custom-style-form").on('submit', function() {
      $(".save-btn").prop("disabled", true);
      var style = arc.admin.CustomStyle();
      style.stylename($('#style-name').val())
           .styledescription($('#style-description').val())
           .css(cssEditor.value())
           .js(jsEditor.value());
      if (styleid) {
        style.styleid(styleid);
      }

      function saveCallBack() {
        $('.save-btn').prop('disabled', false);
        arc.report.pagenav.needs_save(false);
        if (!styleid) {
          styleid = style.styleid();
          const newPath = location.pathname.replace(/\d+$/, styleid);
          arc.misc.replaceState(newPath, { html: $('html').html() });
          $('.breadcrumb .active').text(`ID: ${styleid}`);

        } else {
          // Update name if it's changed
          var $title = $("#title-style-name");
          if ($title.text() !== style.stylename()) {
            $title.text(style.stylename());
          }
        }

        arc.report.alertMessageSmall('Save complete', 'alert-success');
      }

      style.save(saveCallBack);
    });

    $("#clone-link").off('click').on('click', function() {
      $(".clone-btn").prop("disabled", true);
      var style = arc.admin.CustomStyle();
      style.stylename("Clone of " + $('#style-name').val())
           .styledescription($('#style-description').val())
           .css(cssEditor.value())
           .js(jsEditor.value());
      style.save(function() {
        $(".clone-btn").prop("disabled", false);
        arc.report.pagenav.needs_save(false);
        window.open(style.styleid(), "_self");
        arc.report.alertMessageSmall("Clone complete", "alert-success");
      });
    });
  }

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.customextensionUrl}/${styleid}`)
      .await(function(error, customStyle) {
        if (error) {
          arc.report.alertMessage("Could not load custom styles",
              error);
          return;
        }

        $(".breadcrumb .active").text('ID: ' + styleid);
        $("#title-style-name").text(customStyle.name);
        $("#style-description").text(customStyle.description);
        $("#style-name").val(customStyle.name);
        cssEditor.value(customStyle.css);
        jsEditor.value(customStyle.js);
        if (!customStyle.css) {
          $('.nav-tabs[role=tablist] [href=\\#custom-js]').tab('show');
        }
        if (cb) {
          cb();
        }
      });
  }

  return cs;
};

arc.admin.customstyles = function() {
  var cs = function() { return "arc.admin.customstyles"; };

  let customStylesMap = {};
  let displayedStyles = [];

  cs.initialize = function() {
    if (!arc.globals.enable_custom_styles) {
      arc.report.alertMessageSmall("Custom styles are not enabled. " +
          "You can still create custom styles here, but you need to enable them for " +
          "use using the 'Site Settings' page",
          "alert-warning");
    }

    fetchAndPaint();
    return cs;
  };

  function fetchAndPaint(cb) {
    const $spinner = $('#custom-styles-spinner');
    const $searchBar = $('.search-textbox');
    const $tableCustom = $('table.arc-custom-table');

    $tableCustom.hide();
    $searchBar.addClass('arc-invisible');
    $spinner.removeClass('arc-invisible');
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.customextensionUrl)
      .defer(d3.json, arc.frontend.globals.urls.cssUsageUrl)
      .await(function handleCssStylesAndUsage(error, customStyles, cssUsage) {
        $spinner.addClass('arc-invisible');
        $tableCustom.show();
        $searchBar.removeClass('arc-invisible');
        if (error) {
          arc.report.alertMessage("Could not load custom styles",
              error);
          return;
        }

        // Aggregate the custom styles and css usage information into 1 dictionary
        customStyles.forEach(cstyle => {
          customStylesMap[cstyle.id] = cstyle;
          displayedStyles.push(cstyle.id);
        });

        Object.keys(cssUsage.stylesToDashboards).forEach((id) => {
          if (customStylesMap[id]) {
            customStylesMap[id].appAndVisualIds = cssUsage.stylesToDashboards[id];
          }
        });

        Object.keys(cssUsage.stylesForAppgroups).forEach((id) => {
          if (customStylesMap[id]) {
            customStylesMap[id].appgroupIds = cssUsage.stylesForAppgroups[id];
          }
        });

        paintTable(displayedStyles.reverse());
        initializeSearchBar();
        if (cb) {
          cb();
        }
      });
  }

  function paintTable(displayedIds) {
    let $cssTable = $("#table-customstyles").DataTable({
      paging: false,
      destroy: true,
      searching: false,
      columnDefs: [{ targets: [8], orderable: false }],
      order: [5, 'desc'],
    });
    $($cssTable.table().container())
    .addClass('d-flex flex-column justify-content-start align-items-stretch col-sm-12 col-md-12');
    $cssTable.clear();

    const templateIn = $("#table-customstyles-template").html();
    displayedIds.forEach(styleId => {
      const {
        id,
        name,
        description,
        author,
        last_editor,
        last_edited,
        appAndVisualIds,
        appgroupIds,
      } = customStylesMap[styleId];

      let count = 0;
      if (appAndVisualIds) {
        count += appAndVisualIds.length;
      }

      if (appgroupIds) {
        count += appgroupIds.length;
      }

      const styleRow = arc.misc.safe_substitute(
          templateIn,
          {
            ID: id,
            NAME: name,
            DESCRIPTION: description,
            CSS_COUNT: count,
            AUTHOR: author,
            EDITOR: last_editor,
            EDITDATE: last_edited,
            FORMATTEDDATE: arc.misc.fromNow(last_edited),
            HTML_APPS: arc.misc.appsAndAppgroupsToLinks({
              appAndVisualIds,
              appgroupIds,
              newTabs: true,
              joinStr: '<br>',
            }),
          });

      $cssTable.rows.add($(styleRow));
    });
    $cssTable.draw();

    $('#table-customstyles')
      .find('a.delete')
      .offOn('click.deleteStyle', function handleDeleteStyle() {
        const styleId = Number($(this).closest('tr').attr('data-styleid'));
        const c = arc.misc.deleteConfirmation();
        const appAndVisuals = customStylesMap[styleId].appAndVisualIds || [];
        const appgroupIds = customStylesMap[styleId].appgroupIds || [];
        const extraContent = [];

        if (appAndVisuals.length || appgroupIds.length) {
          let msg;

          const dashboardName = appAndVisuals.length > 1 ? 'dashboards' : 'dashboard';
          const appName = appgroupIds.length > 1 ? 'apps' : 'app';

          if (appAndVisuals.length && appgroupIds.length) {
            msg = `${appAndVisuals.length} ${dashboardName} and ${appgroupIds.length} ${appName}`;
          } else if (appAndVisuals.length) {
            msg = `${appAndVisuals.length} ${dashboardName}`;
          } else {
            msg = `${appgroupIds.length} ${appName}`;
          }

          extraContent.push('<div class="delete-customstyle-app-list">');

          extraContent.push(`<p style="color: red;">The custom style you\'re about to delete
              is applied to ${msg}. Please detach the custom style from these objects before
              deleting it otherwise you'll get unexpected errors.</p>`);

          extraContent.push('</div>');
        }

        c.objectType('Custom Style')
          .objectName(customStylesMap[styleId].name)
          .askUser(() => {
            arc.admin.CustomStyle()
                    .styleid(styleId)
                    .delete(() => {
                      customStylesMap = {};
                      displayedStyles = [];
                      fetchAndPaint(() => c.hideModal());
                    });
          }, extraContent.join(''));
      });
  }

  // Attach listener to search bar
  function initializeSearchBar() {
    $('.custom-css-search').val('');
    $('.custom-css-search')
      .offOn('input.customStyles', function redrawSearchResults() {
        if (this.value === '') {
          paintTable(Object.keys(customStylesMap).sort((a, b) => b - a));
        } else {
          displayedStyles = arc.css.searchStyles(this.value, customStylesMap);
          paintTable(displayedStyles);
        }
      });
  }

  return cs;
};

}());
