window.arc = window.arc || {};

(function() {
'use strict';

arc.admin = arc.admin || {};
arc.dataconnection = arc.dataconnection || {};

const $spinnerLoading = $('#addtable-dataset-loading-spinner');
const $selectDbName = $('#addtable-dataset-dbname');
const $selectTableName = $('#addtable-dataset-tablename');
const $commitButton = $('#addtable-dataset-create-commit');
const $labelDbName = $('#addtable-dataset-dbname-label');
const $labelTableName = $('#addtable-dataset-tablename-label');

function removeDbNames() {
  d3.select('#addtable-dataset-dbname')
    .selectAll('option')
    .remove();
  $selectTableName.trigger('chosen:updated');
}

function removeTableNames() {
  d3.select('#addtable-dataset-tablename')
    .selectAll('option')
    .remove();
  $selectTableName.trigger('chosen:updated');
}

arc.admin.dataset = function() {
  const ds = function() { return 'arc.admin.dataset'; };

  let dataset_id;
  let dataconnection_type;
  let popover_column;
  let builder;
  let loader;
  let table_editor;
  let js_editor;
  let join_elim;
  let incremental_results;
  let ksql_stream_position;
  const joinEditors = {};
  let p_fk;
  let child_fk;
  const searchSettingsHandler = arc.search.searchSettingsHelper();
  const timeSettingsHandler = arc.admin.timeSettingsHandler();
  let isDisplayFormatUsed = arc.frontend.vizCookie.getDisplayFormat();
  let sampleData;
  let sampleDataTableName;
  let sampleDataColNames;

  searchSettingsHandler.datasetAdmin(ds);
  timeSettingsHandler.datasetAdmin(ds);

  const aviewManager = arc.admin.aviewManager();
  const dataExtractManager = arc.admin.dataExtractManager();
  const roleManager = arc.roles.roleDatasetConfig();

  ds.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return ds;
  };

  function destroyEditors(displayOptions) {
    if (table_editor) {
      table_editor.destroy();
      table_editor = null;
    }

    if (js_editor) {
      displayOptions.destroyEditor();
      js_editor = null;
    }
  }

  function load_tables(dbname, curtab) {
    $spinnerLoading.show();
    $commitButton.prop('disabled', true);
    removeTableNames();
    const rb = arc.requestBuilder();
    rb.requestType('.tables');
    rb.dbname(dbname);
    arc.report.sql_runner.run(activeDataconnectionId(),
      `.tables ${dbname}`, (errorMsg, tables) => {
      if (errorMsg) {
        return;
      }

      $spinnerLoading.hide();

      tables.rows.sort((a, b) => {
        if (a[0].toLowerCase() < b[0].toLowerCase()) {
          return -1;
        }

        if (a[0].toLowerCase() > b[0].toLowerCase()) {
          return 1;
        }

        return 0;
      });

      d3.select('#addtable-dataset-tablename')
        .selectAll('option')
        .data(tables.rows)
          .enter()
            .append('option')
            .text((d) => d[0]);

      $selectTableName.trigger('chosen:updated');

      // Initialize the picker to the current table if possible.
      if (curtab) {
        tables.rows.forEach((row) => {
          if (row[0] === curtab) {
            $selectTableName.val(curtab)
                            .trigger('chosen:updated');
          }
        });
      }

      $commitButton.prop('disabled', false);
    }, false, rb.dsreq());
  }

  ds.builderReload = function(cb) {
    arc.globals.singleVisualBuilder =
        arc.reportbuilder()
           .loader(loader)
           .dataset_id(dataset_id)
           .dataset_cb(cb)
           .segments_dom_id('filterassociation-modal-datasets-segments-container');

    builder = arc.globals.singleVisualBuilder;
    timeSettingsHandler.builder(builder);
    searchSettingsHandler.builder(builder);
    builder.removeCache();
    aviewManager.builder(builder);
    dataExtractManager.builder(builder);
    builder.showDatasetDetail();
    arc.frontend.legacyBridge.events.next({ type: 'new-dataset-version-created' });

    return builder;
  };

  function redrawJoins(dataset) {
    const tables = [];
    const thash = {};
    const config = {};
    config.x = 0;
    config.y = 0;
    config.max_x = 0;
    config.max_y = 0;

    const startx = 10;
    const starty = 10;
    const width = 200;
    const total_width = 350;
    const height = 30;
    const total_height = 50;
    const start_width = 10;

    dataconnection_type = dataset.dataconnection_type();

    // We're in edit mode if the save button is visible
    const editmode = $('#save-join-btn').is(':visible');

    // Start with adding the first table--add_tables() will add the children
    addTable(dataset, 0, tables, config, thash, 0);

    tables.forEach((t) => {
      t.x_pos = startx + t.x * total_width;
      t.y_pos = starty + t.y * total_height;
      t.x_conn_src = t.x_pos + width;
      t.x_conn_tgt = t.x_pos;
      t.y_conn_src = t.y_pos + height / 2;
      t.y_conn_tgt = t.y_conn_src;
    });

    // Remove any existing popovers
    d3.selectAll('[arcid=join-content-popover-active]')
      .each(function() {
        $(this).parent().parent().remove();
      });

    d3.select('#viz-join')
      .selectAll('svg')
      .remove();

    d3.select('#viz-join')
      .selectAll('div')
      .remove();

    const svg = d3.select('#viz-join')
                  .append('svg')
                  .attr('width', startx + (1 + config.max_x) * total_width)
                  .attr('height', starty + (1 + config.max_y) * total_height);

    // create table name boxes
    const tbls = d3.select('#viz-join')
        .selectAll('div')
        .data(tables)
        .enter()
        .append('div')
        .classed('join-tables-tabname', true);

    tbls.html((d) => {
      const tableonly = d.name.substr(d.name.indexOf('.') + 1);
      let dom = `<div class="join-table-box" data-test-id="join-table-box" style="top:${d.y_pos}px; left:${d.x_pos}px;"><div>${tableonly}</div>`;

      if (editmode) {
        dom = dom.replace('join-table-box', 'join-table-box edit-mode');
        dom += '<a href="#">' +
               '  <i arcid="dataset-add-table" data-test-id="dataset-add-table"' +
               '     class="fa fa-plus-circle pull-right"></i>' +
               '</a>';
      }

      return `${dom}</div>`;
    });

    // Put the fully qualified name <dbname>.<tablename> into the tooltip
    tbls.each(function(d) {
      $(this).find('.join-table-box').tooltip({
        container: 'body',
        placement: 'top',
        delay: {
          show: 1000,
          hide: 100
        }
      }).attr('data-original-title', d.table_select || d.name);
    });

    svg.selectAll('path')
       .data(tables)
       .enter()
       .append('path')
         .filter((d) => d.src_table)
         .attr('stroke', '#999')
         .attr('stroke-width', '2')
         .attr('fill', 'none')
         .attr('d', (d) => {
           if (d.src_table) {
             const paths = [];
             paths.push(['M', d.src_table.x_conn_src, d.src_table.y_conn_src]);
             paths.push(['L', d.src_table.x_conn_src + start_width, d.src_table.y_conn_src]);
             paths.push(['L', d.src_table.x_conn_src + start_width, d.y_conn_tgt]);
             paths.push(['L', d.x_conn_tgt, d.y_conn_tgt]);
             let pathstr = '';
             paths.forEach((p) => {
               pathstr += `${p.join(' ')} `;
             });
             return pathstr;
           }
         });

      svg.selectAll('image')
         .data(tables)
         .enter()
         .append('image')
           .filter((d) => d.src_table)
           .attr('x', (d) => d.x_conn_tgt - (total_width - width) / 2 - 15)
           .attr('y', (d) => d.y_conn_tgt - 15)
          .attr('width', `${30}px`)
          .attr('height', `${30}px`)
          .classed('join-image', true)
          .attr('data-toggle', 'popover')
          .attr('xlink:href', () => `${arc.frontend.globals.urls.staticUrl}/img/table_link.png`)
          .each(function(d) {
            const pop = arc.clone_template('join-content-popover');

            if (dataconnection_type) {
              if (dataconnection_type === 'mysql') {
                pop.find('.image-wrapper.join-type-image[typetext="full outer join"]').remove();
              } else if (dataconnection_type === 'sqlite') {
                pop.find('.image-wrapper.join-type-image[typetext="full outer join"]').remove();
                pop.find('.image-wrapper.join-type-image[typetext="right outer join"]').remove();
              } else if (dataconnection_type === 'ksql') {
                pop.find('.image-wrapper.join-type-image[typetext="inner join"]').remove();
                pop.find('.image-wrapper.join-type-image[typetext="full outer join"]').remove();
                pop.find('.image-wrapper.join-type-image[typetext="right outer join"]').remove();
              }
            }

            const jclist = pop.find('.join-tables-list').empty();
            pop.find('[arcid=edit-join-button]').data('tabinfo', d);

            if (!editmode) {
              pop.find('.popover-btns').remove();
            }

            drawJoinType(pop, d);

            if (!d.join_pairs.length) {
              pop.find('.popover-source-lbl').remove();
              pop.find('.popover-target-lbl').remove();
            }

            d.join_pairs.forEach((p) => {
              const joincol = arc.clone_template('join-list');
              joincol.find('.column-name-src').text(p[0]);
              joincol.find('.column-name-src').attr('title', p[0]);
              joincol.find('.column-name-tgt').text(p[1]);
              joincol.find('.column-name-tgt').attr('title', p[1]);

              jclist.append(joincol);
            });
            if (d.join_exprs.length > 0) {
              const $temp = $('<div>Join Expressions</div>');
              jclist.append($temp);
            }

            d.join_exprs.forEach((e) => {
              const joinexpr = arc.clone_template('join-expr');
              joinexpr.text(e).attr('title', e);
              jclist.append(joinexpr);
            });
            $(this).popover({
              container: 'body', // makes popover be on top
              animation: false,
              html: true,
              trigger: 'click focus',
              placement: 'bottom',
              title: 'Join Details',
              sanitize: false,
              content() {
                return pop;
              }
            });
          });

    function show_table_picker(t, mode) {
      const $modal = $('#add-table-modal');
      if (arc.misc.modalShown($modal)) {
        return;
      }

      $modal.find('.database_and_table').show();
      $modal.find('.sql_query').hide();
      $commitButton.show();

      let displayMode = mode;
      if ((mode === 'change') && (t.table_select)) {
        // dataset from sql is now in table edit mode
        // Display the SQL but don't permit editing here for now
        displayMode = 'change-sql';
      }

      // Hide the tooltip with the full table name
      $('.join-table-box').tooltip('hide');

      // First, clean up the list of databases and tables
      removeDbNames();
      removeTableNames();

      // Select the correct help text
      $modal.find('.popup-instructions').hide();
      $modal.find(`.popup-instructions.mode-${displayMode}`).show();
      $modal.attr('arc-mode', mode);

      if (displayMode === 'change-sql') {
        $spinnerLoading.hide();
        $modal.find('.database_and_table').hide();

        $modal.find('.sql_query').show();
        $modal.find('.sql_query .sql').text(t.table_select);

        $commitButton.hide();
        $modal.modal('show');
        return;
      }

      // Select the correct Database and Table labels
      const database_label = arc.dataconnection.database_label(dataconnection_type);
      const table_label = arc.dataconnection.table_label(dataconnection_type);
      $labelDbName.text(database_label);
      $labelTableName.text(table_label);

      // Show that we are loading information
      $spinnerLoading.show();
      $commitButton.prop('disabled', true);
      $modal.modal('show');
      d3.selectAll('#addtable-dataset-create-commit').datum(t);
      const rb = arc.requestBuilder();
      rb.requestType('.databases');

      // Get and display the list of databases
      arc.report.sql_runner.run(activeDataconnectionId(),
          '.databases', (errorMsg, data) => {
        if (errorMsg) {
          arc.report.alertMessage('Could not read a list of databases', errorMsg);
          return;
        }

        const dbnames = {};

        data.rows.sort((a, b) => {
          if (a[0].toLowerCase() < b[0].toLowerCase()) {
            return -1;
          }

          if (a[0].toLowerCase() > b[0].toLowerCase()) {
            return 1;
          }

          return 0;
        });

        data.rows.forEach((d) => {
          dbnames[d[0]] = 1;
        });

        d3.select('#addtable-dataset-dbname')
          .selectAll('option')
          .data(data.rows)
            .enter()
              .append('option')
              .text((d) => d[0]);

        const default_tab = arc.datasets.drop_dbname(t.dstab.tablename_noalias());
        let default_db = arc.datasets.just_dbname(t.dstab.tablename_noalias());
        if (!dbnames[default_db]) {
          default_db = data.rows[0][0];
        }

        $selectDbName.val(default_db)
                     .trigger('chosen:updated');

        $selectDbName.offOn('change.handleRename', () => {
          const dbname = $selectDbName.val();
          removeTableNames();
          $spinnerLoading.show();
          $commitButton.prop('disabled', true);
          load_tables(dbname);
        });

        load_tables(default_db, default_tab);
      }, false, rb.dsreq());
    }

    // Popup the modal on a 'Add Table' button click
    d3.selectAll('[arcid=dataset-add-table]')
      .on('click.fa-plus-circle', function() {
        if (arc.misc.modalShown($('#add-table-modal'))) {
          // eslint-disable-next-line no-console
          console.log('modal already open');
          return;
        }

        // DJC -- this seem hokey
        const t = d3.selectAll($(this).closest('.join-tables-tabname')).datum();
        show_table_picker(t, 'join');
      });

    // Popup table picker modal in edit mode click on table name
    d3.selectAll('.join-table-box.edit-mode div')
      .on('click', function() {
        // DJC -- this seem hokey
        const t = d3.selectAll($(this).closest('.join-tables-tabname')).datum();
        show_table_picker(t, 'change');
      });

    let created = false;

    $commitButton.offOn('click.commit', async () => {
        // Keep track of whether we were called earlier
        if (created) {
          // eslint-disable-next-line no-console
          console.log('create already done, ignoring');
          return;
        }

        created = true;

        arc.report.pagenav.needs_save(true);

        let tablename = `${$selectDbName.val()}.${$selectTableName.val()}`;

        const mode = $('#add-table-modal').attr('arc-mode');
        if (mode === 'change') {
          const t = d3.selectAll('#addtable-dataset-create-commit').datum();
          t.dstab.dataset().update_tablename(t.dstab, tablename);

          // refresh to validate the new table columns + set up new join pairs + redraw the display
          try {
            const tableFieldsChanged = await refreshFields();
            if (tableFieldsChanged) {
              // Updating join pairs for join after making sure updating table fields has finished
              setUpJoinPairsAndEditJoinBtnData(t.dstab, t.src_table, dataset, true);
              const $msg = $('#reload-tables-message');
              $msg.text('Table columns and join pairs updated');
            }
          } catch (_e) {
              // refreshFields handles the error
          } finally {
            arcl.attributes_set_editmode(true);
          }

          return;
        }

        // DJC -- perhaps parent should be supplied to this call rather than filled in here
        dataset.fetch_and_add_table(tablename, (table) => {
          const t = d3.selectAll('#addtable-dataset-create-commit').datum();
          table.parent_tablename(t.name);

          setUpJoinPairsAndEditJoinBtnData(table, t, dataset);

          /*
          * NOTE: bootstrap modal chaos
          *
          * We want to open the edit-join-modal modal here.
          *
          * However, we are making this call within the add-table-modal modal.
          *
          * Normally, when a modal is open, the <body> has a 'modal-open' class set on it.
          * Because the add-table-modal is not closed yet, the <body> still has 'modal-open'
          * set on it. If we trigger a click on the button that opens up the modal here,
          * then closing the add-table-modal later will lead to the modal-open class no
          * longer being set on the body. This causes the x axis of the modal to not be
          * scrollable among other things.
          *
          * Hence, what we do here is that we trigger the opening of the next modal after
          * the closing of this modal.
          */
          const $modal = $('#add-table-modal');
          const modalCheckInterval = setInterval(() => {
            if (arc.misc.modalShown($modal)) {
              return;
            }

            // invoke the join editor by posting an event to edit join button
            // because it contains data about join
            const ej = $('#join-content-popover-template')
                          .find('[arcid=edit-join-button]');
            ej.trigger('click');
            clearInterval(modalCheckInterval);
          },
          20);
        });
      });

    return tables;
  }

  function setUpJoinPairsAndEditJoinBtnData(table, parent_table, dataset, resetJoinPairs) {
    // Set up columns with same name and type as join columns
    const p = dataset.get_table(parent_table.name);
    let currentMatchColIdx = 0;

    function columnSortCompare(a, b) {
      const nameCompare = a.attr('name').localeCompare(b.attr('name'));
      if (!nameCompare) {
        return a.attr('type').localeCompare(b.attr('type'));
      }

      return nameCompare;
    }

    const sortedTargetColumns = table.base_columns().slice().sort(columnSortCompare);
    const sortedParentColumns = p.base_columns().slice().sort(columnSortCompare);
    const matches = [];

    // Because both parent and target arrays are sorted using the same ordering scheme,
    // once a column match in the target array is found, any subsequent matches must appear
    // later in the array -- hence the use of currentMatchColIdx
    function findMatch(parentCol) {
      const foundIdx = arc.misc.binarySearch({
        target: parentCol,
        array: sortedTargetColumns,
        startIdx: currentMatchColIdx,
        endIdx: sortedTargetColumns.length - 1,
        comparator: columnSortCompare
      });
      if (foundIdx !== -1) {
        currentMatchColIdx = foundIdx;

        matches.push(
          [parentCol.attr('name'), sortedTargetColumns[foundIdx].attr('name')]
        );
      }
    }

    sortedParentColumns.forEach(findMatch);
    // reset join pairs upon changing join table, but concat upon creating a new join
    table.join_pairs(resetJoinPairs ? matches : table.join_pairs().concat(matches));

    const joinTables = redrawJoins(dataset);

    // DJC -- more hokeyness...here we find the table struct within redrawJoins that
    //         we just created and associate it with the edit-join-button within the
    //         popover template

    // Tablename may now have alias
    const tablename = table.tablename();
    joinTables.forEach((tbl) => {
      if (tbl.name === tablename) {
        const ej = $('#join-content-popover-template')
            .find('[arcid=edit-join-button]');
        ej.data('tabinfo', tbl);
      }
    });
  }

  function viewsEnabled(enabled) {
    if (enabled) {
      $('.views-none').addClass('arc-invisible');
      $('.nav-views').show();
      $('.views-enabled').removeClass('arc-invisible');
    } else {
      $('.nav-views').hide();
      $('.views-enabled').addClass('arc-invisible');
    }
  }

  function extractsEnabled(enabled) {
    if (enabled) {
      $('.views-none').addClass('arc-invisible');
      $('.nav-extracts').show();
      $('.views-enabled').removeClass('arc-invisible');
    } else {
      $('.nav-extracts').hide();
      $('.views-enabled').addClass('arc-invisible');
    }
  }

  // Search in all hierarchies for children with old_alias and update them with
  // updated parent li info.
  function updateAllHierarchies(old_alias, new_li) {
    const new_alias = new_li.find('.table-list-value').first().text().trim();
    const new_coltype_class = new_li.data('_arc_data').coltype_class();
    const allHierarchyChildLis = $('.table-list-wrapper ' +
        'li[arcid=dsattr-hierarchy-column-active] ul.child-elements-list > li');
    $.each(allHierarchyChildLis, (i, childLi) => {
      childLi = $(childLi);
      if (old_alias === childLi.find('.table-list-value').text().trim()) {
        childLi.find('.table-list-value').text(new_alias);

        childLi.find('.dim-icon-type')

          // remove classes that begin with 'type-'
          .removeClass((index, css) => (css.match(/(^|\s)type-\S+/g) || []).join(' ')).addClass(new_coltype_class);
        childLi.effect('highlight', { color: '#E2F8D3' }, 1500).fadeIn('fast');
      }
    });
  }

  ds.datasetRefreshed = function() {
    const dataset = builder.dataset();
    const perms = dataset.dataset_perms();

    arc.misc.updateDivForPerms(perms, 'dataset');
    arc.misc.updateDivForPerms(perms, 'dataconn');
    ds.init_lv();
    redrawJoins(builder.dataset());
    datasetSourcesSetEditMode(false);
    arc.report.pagenav.needs_save(false);
    roleManager.initialize(undefined, builder.dataset());

    dataconnection_type = builder.dataset().dataconnection_type();

    if (arc.dataconnection.supportsDataExtractsSource(dataconnection_type)) {
      dataExtractManager.initialize();
      extractsEnabled(true);
    } else {
      extractsEnabled(false);
    }

    if (arc.misc.directAccessTypes.includes(dataconnection_type)) {
      $('.nav-data-profile').show();
    } else {
      $('.nav-data-profile').hide();
    }

    if (arc.dataconnection.isArcEngine(dataconnection_type) &&
        arc.misc.hasPerm('dc_aviews', perms)) {
      builder.dataset().update_lv();
      aviewManager.tables(builder.dataset().tables());
      aviewManager.lvtable(builder.dataset().dataset_lvtable());
      aviewManager.dataconnection_id(builder.dataset().dataconnection_id());
      aviewManager.dataconnection_type(dataconnection_type);
      aviewManager.aviewDisplayConfig({
        style: 'flat-table',
        tableFilter: null,
        statusFilter: null,
        order: [{ field: 'name', direction: 'asc' }],
      });
      aviewManager.loadViews();
      viewsEnabled(true);
    } else {
      viewsEnabled(false);
    }

    ds.set_join_elim_value();
    setIncrementalResultsValue();
    setKSQLStreamPositionValue();
    setupSQLDisplay(dataset);
    ds.set_cache_options();

    timeSettingsHandler.setupTimeSettings();
    searchSettingsHandler.setupSearch();
    if (perms.includes('ds_appedit') && arc.frontend.featureFlipper.getValueById('site_enable_data_profiling')) {
      arc.frontend.legacyBridge.events.next({ type: 'profile-dataset-event', payload: { dataset, title: 'Dataset Profiling' } });
    }
  };

  /*
    1, A dataset field, can be dragged and dropped amongst to a hierarchy field's subfields.
    2, A hierarchy field's sub-fields can be dragged out from the hierarchy field, or moved to
    another hiearchy field.
  */
  const dimensionFieldTableSel = '#viz-attributes-dimensions';
  const aggregatesFieldTableSel = '#viz-attributes-aggregates';
  const hierarchyFieldsChildElementListSel = '.child-elements-list';
  const hierarchyChildElementListSel = `li[arcid=dsattr-hierarchy-column-active] ul${hierarchyFieldsChildElementListSel}`;
  function setDraggablityOfDatasetFields(disable) {
    function getHierarchyChildList() {
      // Their number can vary over time, as user can add Hierarchy Fields to the dataset.
      return $(`${dimensionFieldTableSel} ${hierarchyChildElementListSel}, ${aggregatesFieldTableSel} ${hierarchyChildElementListSel}`);
    }

    function highLightDropAreas(highLight) {
      if (highLight) {
        getHierarchyChildList().addClass('highlight');
      } else {
        getHierarchyChildList().removeClass('highlight');
      }
    }

    let sortableIn;
    const $regularDimensionField = $(`${dimensionFieldTableSel} .table-list > li:not([arcid=dsattr-hierarchy-column-active]):not(.hierarchy-field)`);
    const isAnyReceiverColumn = builder
                                  .dataset()
                                  .tables()
                                  .flatMap((table) => table.columns())
                                  .some((column) => column.isHierarchy());
    if (!disable && isAnyReceiverColumn) {
      $regularDimensionField.draggable({
        connectToSortable: hierarchyFieldsChildElementListSel,
        cancel: 'a, button',
        placeholder: 'drag-highlight',
        helper: 'clone',
        revert: 'invalid',
        dropOnEmpty: true,
        disabled: false,
        // eslint-disable-next-line no-unused-vars
        drag(e, ui) {
          // Removing style attribute all together causes dragged pill to look weird.
          ui.helper.css('width', '');
          ui.helper.css('height', '');
          ui.helper.css('opacity', '');
          highLightDropAreas(true);
        },
        // eslint-disable-next-line no-unused-vars
        stop(e, ui) {
          highLightDropAreas(false);
          ui.helper.removeAttr('style');
          ui.helper.removeClass('ui-draggable-dragging');
        }
      });
      getHierarchyChildList().sortable({
        connectWith: hierarchyFieldsChildElementListSel,
        placeholder: 'drag-highlight',
        helper: 'clone',
        dropOnEmpty: true,
        opacity: 0.5,
        // eslint-disable-next-line no-unused-vars
        start(e, ui) {
          // from the one at setup time!
          highLightDropAreas(true);
        },
        // eslint-disable-next-line no-unused-vars
        stop(e, ui) {
          highLightDropAreas(false);
        },

        // add the ability to drag an element out of the hierarchy to remove it
        out() {
          sortableIn = 0;
        },
        receive() {
          const $currentItem = $.ui.ddmanager.current.currentItem;
          sortableIn = 1;

          // TODO: Create some utility function for highlighting such that color is defined
          // at one place
          $currentItem.effect('highlight', { color: '#E2F8D3' }, 1500)
                      .css('display', '');
        },
        over() {
          sortableIn = 1;
        },
        beforeStop(_, ui) {
          if (sortableIn === 0) {
            ui.item.remove();
          }
        }
      });
    } else {
      $regularDimensionField.draggable({ disabled: true });
    }
  }

  function activeDataconnectionId() {
    return builder.dataset() ? builder.dataset().dataconnection_id() : undefined;
  }

  function activeDataconnectionType() {
    return builder.dataset() ? builder.dataset().dataconnection_type() : undefined;
  }

  function drawJoinType(pop, t) {
    pop.find('.join-type-image').each((i, elem) => {
      const $e = $(elem);
      if (t.join_type === $e.attr('typetext')) {
        $e.addClass('join-selected');
      } else {
        $e.removeClass('join-selected');
      }
    });
  }

  // Set the edit mode state for dataset sources page
  function datasetSourcesSetEditMode(enabled) {
    if (enabled) {
      $('#edit-join-btn').hide();
      $('#save-join-btn').show();
      $('#cancel-join-btn').show();
    } else {
      if (arc.misc.hasPerm('dc_explore', builder.dataset().dataset_perms())) {
        $('#edit-join-btn').show();
      }

      $('#save-join-btn').hide();
      $('#cancel-join-btn').hide();
    }
  }

  function addTable(dataset, index, jointabs, config, thash, cnum) {
    const dstab = dataset.tables()[index];
    const src_table = thash[dstab.parent_tablename()];
    if (src_table) {
      config.x = src_table.x + 1;
      config.y = config.max_y + (cnum > 0);
    }

    const t = {
      name: dstab.tablename(),
      table_select: dstab.table_select(),
      x: config.x,
      y: config.y,
      dstab,
      src_table,
      join_pairs: dstab.join_pairs(),
      join_exprs: dstab.join_exprs(),
      join_type: dstab.join_type(),
    };
    jointabs.push(t);
    thash[t.name] = t;
    config.max_x = Math.max(config.x, config.max_x);
    config.max_y = Math.max(config.y, config.max_y);

    // Now add all child tables of this table
    let child = 0;
    for (let i = 0; i < dataset.tables().length; i++) {
      if (dataset.tables()[i].parent_tablename() === dstab.tablename()) {
        addTable(dataset, i, jointabs, config, thash, child);
        child += 1;
      }
    }
  }

  function buildEditJoinPair(pcols, ccols, p, t) {
    // For each join pair, need to add one of these
    const temp = arc.clone_template('join-tables-list');
    const slist = temp.find('[arcid=modal-source-input]');
    d3.select(slist.get(0))
      .selectAll('option')
      .data(pcols)
      .enter()
      .append('option')
      .text((d) => d);
      slist.val(p[0]);

    const tlist = temp.find('[arcid=modal-target-input]');
    d3.select(tlist.get(0))
      .selectAll('option')
      .data(ccols)
      .enter()
      .append('option')
      .text((d) => d);
      tlist.val(p[1]);

    const fill_in_sample = function(elem, colname, dstab) {
      const sample = $(elem).parent().find('.sample-list');
      d3.select(sample.get(0))
        .selectAll('li')
        .remove();

      // Look for this column in the
      const sampdata = [];
      const sampresp = dstab.sample_data();
      let colindex;

      if (sampresp) {
        sampresp.colnames.forEach((hdrcol, idx) => {
          hdrcol = hdrcol.substr(hdrcol.indexOf('.') + 1);
          if (hdrcol.toLowerCase() === String(colname).toLowerCase()) {
            colindex = idx;
          }
        });
        if (colindex !== undefined) {
          sampresp.rows.forEach((row) => {
            sampdata.push(row[colindex]);
          });
        }
      }

      if (sampdata.length > 0) {
        d3.select(sample.get(0))
          .selectAll('li')
          .data(sampdata)
          .enter()
          .append('li')
          .text((d) => d);
      }
    };

    // check the checkbox if the foreign-key primary-key relation is already set
    function check_fk_parent(p_colname, c_colname) {
      if (p_colname in p_fk && p_fk[p_colname] === c_colname) {
        temp.find("[name='foreign-key-parent']").prop('checked', true);
      }
    }

    function check_fk_child(c_colname, p_colname) {
      if (c_colname in child_fk && child_fk[c_colname] === p_colname) {
        temp.find("[name='foreign-key-child']").prop('checked', true);
      }
    }

    // On source or target column change, need to adjust the sample data
    slist.on('change', function() {
      temp.find("[name^='foreign-key-']").attr('checked', null);
      check_fk_parent($(this).val(), tlist.val());
      check_fk_child(tlist.val(), $(this).val());
      fill_in_sample(slist, $(this).val(), t.src_table.dstab);
    });
    tlist.on('change', function() {
      temp.find("[name^='foreign-key-']").attr('checked', null);
      check_fk_parent(slist.val(), $(this).val());
      check_fk_child($(this).val(), slist.val());
      fill_in_sample(tlist, $(this).val(), t.dstab);
    });

    check_fk_parent(p[0], p[1]);
    check_fk_child(p[1], p[0]);

    fill_in_sample(slist, slist.val(), t.src_table.dstab);
    fill_in_sample(tlist, tlist.val(), t.dstab);
    return temp;
  }

  function toggleLoadingSpinner(showSpinner = true) {
    $('#join-sample-column-loading-spinner').toggle(showSpinner);
    $('#edit-join-modal .modal-body').toggle(!showSpinner);
  }

  // Ensure we have sample data for parent and child tables then to the callback
  // DJC -- need some error handling here
  function getSampleData(dstab, pk_fk, cb) {
    toggleLoadingSpinner(true);

    const fullname = dstab.tablename_noalias();
    arc.apps_data.fetch_sample_data(
      {
        limit: 10,
        cache: false,
        pk_fk,
        dcid: activeDataconnectionId(),
        dctype: activeDataconnectionType(),
        fullname,
        hideAlertsOnSuccess: true,
        cb(_, response) {
          dstab.sample_data(response);
          cb();
        }
      }
    );
  }

  function deleteColumn(col_li) {
    arc.report.pagenav.needs_save(true);
    const col = col_li.data('_arc_data');
    const col_alias = col.attr('alias');
    if (col.isBaseCol()) {
      // eslint-disable-next-line no-console
      console.log(`Error: Cannot delete field ${col.attr('name')}`);
      return;
    }

    // Remove the column from other hierarchies as well
    const $allHierarchyChildLis = $('.table-list-wrapper li[arcid=dsattr-hierarchy-column-active] ' +
        'ul.child-elements-list > li');
    $.each($allHierarchyChildLis, (_, childLi) => {
      const $childLi = $(childLi);

      // TODO: Create some utility function for highlighting such that color is defined at
      // one place
      if (col_alias === $childLi.find('.table-list-value').text().trim()) {
        $childLi.effect('highlight', { color: '#E2F8D3' }, 500).fadeOut('fast', () => {
          $childLi.remove();
        });
      }
    });

    // Remove the column
    col.table().drop_column(col);
    col_li.remove();
    setDraggablityOfDatasetFields();
  }

  function initializeAllPopovers() {
    $('[data-toggle="popover"]').popover();
  }

  function hideAllPopovers() {
    $('[data-toggle="popover"]').popover('hide');
  }

  ds.init_lv = function() {
    let db_names;

    // build dataset Assoc. LV editor
    const $detailTab = $('#detail');
    const $editLVname = $detailTab.find('#edit-lvname');
    $editLVname.hide();

    const $editLVnameForm = $detailTab.find('#edit-lvname-form');
    const $editLVnameIcon = $detailTab.find('#edit-lvname-icon');
    const $deleteLVbutton = $editLVnameForm.find('#del-lvname-btn');
    const $editLVnameInput = $editLVnameForm.find('#edit-lvname-input');
    const $datasetLVname = $detailTab.find('#dataset-lvname');

    const $deleteModal = $('#delete-associated-logical-view-modal');

    // load DBs only after dataset detail has been fetched, i.e. when
    // activeDataconnectionId is available as well as dataset-table name is known
    function load_dbs(default_db) {
      arc.apps_data.load_databases_cb(activeDataconnectionId(), (error, data) => {
        if (!default_db) {
          default_db = $('#dataset-table').text().split('.')[0];
        }

        if (error) {
          arc.report.alertMessage('Could not read a list of databases', error);
          return;
        }

        // chack DB names, filter names starting with an underscore
        function checkDBname(dbs) {
          return dbs.filter((name) => name[0] && name[0][0] !== '_');
        }

        db_names = checkDBname(data.rows);

        d3.select('#lv-dbname')
          .selectAll('option')
          .data(db_names)
          .enter()
          .append('option')
          .text((d) => d[0]);

        function checkDefaultDb(dbs, defaultDb) {
          return dbs.some((db) => defaultDb === db[0]);
        }

        // if db names don't contain default DB that is parsed from table name
        // set it to first element in array
        if (!checkDefaultDb(data.rows, default_db)) {
          default_db = data.rows[0][0];
        }

        $editLVnameForm.find('#lv-dbname').val(default_db);
      });
    }

    // show Associate Logical View option for ArcEngine
    if (arc.dataconnection.isArcEngine(activeDataconnectionType())) {
      $editLVname.show();

      const hideLVnameForm = function() {
        $editLVnameForm.hide();
        $datasetLVname.show();
        $editLVnameIcon.show();
        $deleteModal.modal('hide');
        $deleteModal.find('span.delete-spinner').remove();
        $deleteModal.find('button').attr('disabled', false);
      };

      // reveal Dataset->LV form when clicking on pensil icon
      $editLVnameIcon.offOn('click.editLVnameIcon', () => {
        $editLVnameIcon.hide();
        let lv_name;
        if (builder && builder.dataset()) {
          lv_name = builder.dataset().lvname();
        } else {
          lv_name = $datasetLVname.text();
        }

        let lv_dbname; let lv_tablename;
        const splits = lv_name.split('.');
        if (splits.length > 1) {
          lv_dbname = splits[0];
          lv_tablename = splits.splice(1).join('.');
        } else {
          lv_tablename = '';
        }

        // load DB names only once per dataset initialization
        if (!db_names) {
          load_dbs(lv_dbname);
        }

        $datasetLVname.hide();
        $editLVnameInput.val(lv_tablename);

        // show delete button only when there is an existing LV
        if (lv_name) {
          $deleteLVbutton.show();
        } else {
          $deleteLVbutton.hide();
        }

        $editLVnameForm.show();
      });

      // SAVE LV
      $editLVnameForm.find('#save-lvname-btn').off().on('click', () => {
        $('#edit-lvname-form').validator('validate');

        // saving multiple times using same LV name shouldn't send a backend call
        if (`${$('#lv-dbname').val()}.${$editLVnameInput.val().trim()}` ===
            builder.dataset().lvname()) {
          return;
        }

        // use of 'DELETE' value is reserved for deleting an associated LV
        // pass a valid LV name (non-empty string)
        if (!$editLVnameInput.val().trim() || $editLVnameInput.val().trim() === 'DELETE') {
          arc.report.alertMessage('Invalid logical view name');
          return;
        }

        builder.dataset()
          .lvname(`${$('#lv-dbname').val()}.${$editLVnameInput.val().trim()}`)
          .save(() => {
            ds.builderReload(() => {
              hideLVnameForm();
              ds.datasetRefreshed();
            });
          });
      });

      // DELETE LV
      $deleteLVbutton.off().on('click', () => {
        if (arc.misc.modalShown($deleteModal)) {
          return;
        }

        // pop up delete confirmation modal
        $deleteModal.find('.arc-delete-type').text(`"${builder.dataset().lvname()}"`);
        $deleteModal.modal('show');
        $deleteModal.find('#delete-associated-logical-view-yes').off().on('click', () => {
          // disable buttons during view deletion and start spinner
          $deleteModal.find('button').attr('disabled', true);
          $deleteModal.find('.modal-body').append('<span class="delete-spinner">' +
              '<i class="fa fa-spinner fa-spin"></i></span>');
          builder.dataset()
            .lvname('DELETE')
            .save(() => {
              ds.builderReload(() => {
                hideLVnameForm();
                ds.datasetRefreshed();
              });
            });
        });
      });

      $editLVnameForm.find('#canc-lvname-btn').off().on('click', hideLVnameForm);
    }
  };

  let $joinElimValue;
  let $joinElimEditIcon;
  let $joinElimOptions;
  let $cacheText;
  let $cachePencil;
  let $cacheReset;
  let $cacheOptions;
  let $cacheRadio;
  let $cacheRetention;

  let $ksqlStreamPositionValue;
  let $ksqlStreamPositionEditIcon;
  let $ksqlStreamPositionOptions;

  let $incrementalResultsValue;
  let $incrementalResultsEditIcon;
  let $incrementalResultsOptions;

  ds.set_join_elim_value = function() {
    // read  join_elim flag from Dataset.info if needed
    if (join_elim === undefined && builder.dataset()) {
      join_elim = builder.dataset().enable_join_elim();
    }

    // by default set to on (when flag not present in Dataset.info)
    if (join_elim === undefined) {
      join_elim = true;
    }

    $joinElimValue.text(join_elim ? 'Enabled' : 'Disabled');
    if (join_elim) {
      $joinElimOptions.find('#join_elim_enabled').prop('checked', true);
    } else {
      $joinElimOptions.find('#join_elim_disabled').prop('checked', true);
    }
  };

  ds.set_cache_options = function() {
    const cache = builder.dataset().get_cache_settings();
    const cacheMode = cache.ENABLED ? 'Enabled' : 'Disabled';
    const mode = cache.OVERRIDE ? cacheMode : 'From Connection';
    $cacheRadio.filter(`[value="${mode}"]`).prop('checked', true);
    $cacheText.text(mode);

    $cacheRetention.val(cache.RETENTION || 0);
    $cacheRetention.prop('disabled', mode !== 'Enabled');
  };

  function baseSetup() {
    // enable join elimination button (always visible)
    $joinElimValue = $('#enable-join_elim'); // value in view mode
    $joinElimEditIcon = $('#edit-enable-join_elim-icon'); // icon to edit
    $joinElimOptions = $('#enable-join_elim-options'); // .form-check.form-check-inline .form-check-label options

    // cache control
    $cacheText = $('#ds-cache-text');
    $cachePencil = $('#ds-cache-pencil');
    $cacheReset = $('#ds-cache-reset');
    $cacheOptions = $('#ds-cache-options');
    $cacheRadio = $('#ds-cache-options').find('[name=ds-cache-radio]');
    $cacheRetention = $cacheOptions.find('#ds-cache-retention');

    $incrementalResultsValue = $('#enable-incremental-results'); // value in view mode
    $incrementalResultsEditIcon = $('#edit-enable-incremental-results-icon'); // icon to edit
    $incrementalResultsOptions = $('#enable-incremental-results-options'); // .form-check.form-check-inline .form-check-label options

    $ksqlStreamPositionValue = $('#ksql-stream-position-results');
    $ksqlStreamPositionEditIcon = $('#edit-ksql-stream-position-icon');
    $ksqlStreamPositionOptions = $('#ksql-stream-position-options');
  }

  let exprEditor;

  function enableOrDisableSQL(aceEditor, enableOpacity) {
    aceEditor.setOptions({
     readOnly: !enableOpacity,
     highlightActiveLine: !!enableOpacity,
   });

   // Prevent allowing tabbing into the input field
   aceEditor.renderer.$cursorLayer.element.style.opacity = enableOpacity;

   // Prevent adding cursor to end of input
   aceEditor.renderer.setStyle('disabled', !enableOpacity);

   if (enableOpacity) {
     aceEditor.focus();
   }
  }

  const $sampleDataTable = $('#datasets-data-table');
  function renderSampleData(errorMsg) {
    const $thead = $sampleDataTable.find('thead');
    const $tbody = $sampleDataTable.find('tbody');
    $thead.empty();
    $tbody.empty();

    if (errorMsg) {
      return;
    }

    d3.select($thead.get(0))
      .append('tr')
      .selectAll('th')
      .data(sampleDataTableName)
      .enter()
      .append('th')
      .attr('colspan', (d) => d[1])
      .classed('table-header-dataset', true)
      .text((d) => d[0]);

    const colNames = sampleDataColNames || sampleData.colnames;
    d3.select($thead.get(0))
      .append('tr')
      .classed('table-subheader-dataset', true)
      .selectAll('th')
      .data(colNames)
      .enter()
      .append('th')
      .text((d) => d);

    // Get column formatters for display in arc.report.sql_runner.run cb
    const colFormatters = [];
    if (isDisplayFormatUsed) {
      colNames.forEach(addDisplayFormatter);
    }

    sampleData.rows.forEach((row) => {
      d3.select($tbody.get(0))
        .append('tr')
        .selectAll('td')
        .data(row)
        .enter()
        .append('td')
        .text((d, idx) => {
          const currFormatter = colFormatters[idx];
          return currFormatter ? currFormatter(d) : d;
        });
    });

    function addDisplayFormatter(colName) {
      const dataset = builder.dataset();
      const col = dataset.get_column(colName);
      const displayFormat = col.attr('displayformat');
      if (!displayFormat) {
        colFormatters.push(null);
      } else {
        const formatter = arc.utils.translateDisplayFormat(displayFormat, dataset);
        colFormatters.push(formatter);
      }
    }
  }

  function setSampleData(data, tableName, tableData) {
    sampleData = data;
    sampleDataTableName = tableName;
    sampleDataColNames = null;
    if (dataconnection_type === 'ksql') {
      sampleDataColNames = [];
      tableData.forEach((table) => {
        table.columns.forEach((column) => {
          if (column.hidden) {
            return;
          }

          sampleDataColNames.push(column.name || column.alias);
        });
      });
    }
  }

  // Function for showing data
  const $showDataBtn = $('#show-data-btn');
  const $dataBtn = $showDataBtn.find('.data-btn');
  const dataBtnTitles = { hide: 'Hide Data', show: 'Show Data', reload: 'Reload Data' };
  const $refreshInfo = $showDataBtn.find('.viz-notification-msg');
  function setDataButton(text) {
    switch (text) {
      case dataBtnTitles.reload:
        if (isSampleDataTableDisplayed()) {
          $dataBtn.text(text);
          $refreshInfo.show();
        }

        break;
      default:
        $refreshInfo.hide();
        $dataBtn.text(text);
    }
  }

  function isSampleDataTableDisplayed() { return $sampleDataTable.css('display') === 'table'; }

  const $loadingSpinner = $('#sample-table-loading-spinner');

  function loadSampleDataTable() {
    $loadingSpinner.show();
    const dataset = builder.dataset();
    const tableName = dataset.all_columns_select_text()[2];
    const rb = arc.requestBuilder();
    rb.requestType('getrows');
    rb.datasetId(dataset.dataset_id());
    rb.limit(100);
    const dsreq = rb.dsreq();
    dsreq.dataset_object = dataset.buildInfo();

    // DJC -- need to handle errors here

    arc.report.sql_runner.run(
      activeDataconnectionId(),
      null,
      (errorMsg, resp) => {
        setSampleData(resp, tableName, JSON.parse(dsreq.dataset_object));
        renderSampleData(errorMsg);
        $loadingSpinner.hide();

        if (!isSampleDataTableDisplayed()) {
          $sampleDataTable.slideToggle();
        }
      },
      false,
      dsreq,
    );
  }

  function setupSQLDisplay(dataset) {
    const tables = dataset.tables();
    const table = tables[0];
    const tableSelect = table.table_select();
    if (!tableSelect) {
      $('#dataset-sql-section').hide();
      return;
    }

    $('#dataset-sql-section').show();
    const $sel = $('#dataset-sql-display');
    // Ace editor is resiable. When we destroy it, it will loose its set dimensions.
    // For seamless UX we set it back, but we have to store the dimension before destory.
    let previoslySetDimensions;

    if (exprEditor) {
      previoslySetDimensions = {
        width: exprEditor.aceEditor().container.offsetWidth,
        height: exprEditor.aceEditor().container.offsetHeight
      };
      exprEditor.destroy();
      exprEditor = undefined;
    }

    $sel.find('.rawsql-show-container')
        .remove();

    $sel.append($('#rawsql-show-template').html());
    const exprId = `expr-editor-${arc.misc.uniqueId()}`;
    const maxWidth = window.innerWidth - $('.side-menu-wrapper').width() - 300;
    const aceEditorExpressionField = $sel.find('.expression-field');
    aceEditorExpressionField.attr('id', exprId);
    aceEditorExpressionField.css('max-width', `${maxWidth}px`);

    if (previoslySetDimensions) {
      aceEditorExpressionField.css({
        width: `${previoslySetDimensions.width}px`,
        height: `${previoslySetDimensions.height}px`,
      });
    }

    exprEditor = arc.misc.expressionEditor(
      dataset,
      undefined,
      exprId,
      $sel.find('.rawsql-show-container .edit-expression-textbox'),
      {
        dataconnection_type,
        datasetSql: true,
      },
      true
    );

    exprEditor.getSession().setValue(tableSelect);
    const aceEditor = exprEditor.aceEditor();
    enableOrDisableSQL(aceEditor, 0);

    const $sqlEditBtn = $('#edit-dataset-sql-icon');

    $sqlEditBtn.offOn('click.edit', function setupSQLEdit() {
      arc.frontend.legacyBridge.events.next({
        type: 'open-edit-dataset-sql-modal',
        payload: {
          dataset: builder.dataset(),
          onSuccess: async () => {
            const updatedQuery = builder.dataset().tables()[0].table_select();
            exprEditor.getSession().setValue(updatedQuery);
            try {
              await refreshFields();
              $('#save-attr-btn').trigger('click');
              arcl.attributes_set_editmode(false);
            } catch (_e) {
              // refreshFields handle the error
            } finally {
              setDataButton(dataBtnTitles.reload);
            }
          }
        }
      });
    });

  }

  function setKSQLStreamPositionValue() {
    if (dataconnection_type !== 'ksql') {
      $('#ksql-stream-position-section').hide();
      return;
    }

    $('#ksql-stream-position-section').show();
    if ((ksql_stream_position === undefined) && builder.dataset()) {
      ksql_stream_position = builder.dataset().ksql_stream_position();
    }

    if (ksql_stream_position === undefined) {
      ksql_stream_position = 'latest';
    }

    $ksqlStreamPositionValue.text(ksql_stream_position === 'earliest' ? 'Earliest' : 'Latest');
    if (ksql_stream_position === 'earliest') {
      $ksqlStreamPositionOptions.find('#ksql-stream-position-earliest').prop('checked', true);
    } else {
      $ksqlStreamPositionOptions.find('#ksql-stream-position-latest').prop('checked', true);
    }
  }

  // enable incremental results button
  function setIncrementalResultsValue() {
    if (!arc.dataconnection.supportsIncrementalResults(dataconnection_type)) {
      $('#incremental-results-section').hide();
      return;
    }

    $('#incremental-results-section').show();

    // read  incremental_results flag from Dataset.info if needed
    if (!incremental_results && builder.dataset()) {
      incremental_results = builder.dataset().enable_incremental_results();
    }

    // by default set to on (when flag not present in Dataset.info)
    if (incremental_results === undefined) {
      incremental_results = false;
    }

    $incrementalResultsValue.text(incremental_results ? 'Enabled' : 'Disabled');
    if (incremental_results) {
      $incrementalResultsOptions.find('#incremental-results_enabled').prop('checked', true);
    } else {
      $incrementalResultsOptions.find('#incremental-results_disabled').prop('checked', true);
    }
  }

  function showIncrementalResultsOptions(_) {
    if (_) {
      $incrementalResultsValue.hide();
      $incrementalResultsEditIcon.hide();
      $incrementalResultsOptions.show();
    } else {
      $incrementalResultsValue.show();
      $incrementalResultsEditIcon.show();
      $incrementalResultsOptions.hide();
    }
  }

  function showKSQLStreamPositionOptions(_) {
    if (_) {
      $ksqlStreamPositionValue.hide();
      $ksqlStreamPositionEditIcon.hide();
      $ksqlStreamPositionOptions.show();
    } else {
      $ksqlStreamPositionValue.show();
      $ksqlStreamPositionEditIcon.show();
      $ksqlStreamPositionOptions.hide();
    }
  }

  function setupKSQLStreamPositionEdit() {
    $ksqlStreamPositionEditIcon
      .offOn('click.showIncrementally', () => {
        showKSQLStreamPositionOptions(true);
      });

    $ksqlStreamPositionOptions
        .find('#canc-ksql-stream-position')
        .offOn('click.hide', function hide() {
          setKSQLStreamPositionValue();
          showKSQLStreamPositionOptions(false);
        });

    $ksqlStreamPositionOptions
        .find('#save-ksql-stream-position')
        .offOn('click.save', function triggerSave() {
          let ksql_stream_position_value;
          if ($ksqlStreamPositionOptions.find('#ksql-stream-position-earliest').prop('checked')) {
            ksql_stream_position_value = 'earliest';
          } else {
            ksql_stream_position_value = 'latest';
          }

          if (ksql_stream_position_value !== ksql_stream_position && builder.dataset()) {
            builder.dataset().ksql_stream_position(ksql_stream_position_value);
            builder.dataset()
              .save(function afterSave() {
                ds.builderReload(function resetOptions() {
                  ksql_stream_position = ksql_stream_position_value;
                  setKSQLStreamPositionValue();
                  showKSQLStreamPositionOptions(false);
                });
            });
          }
        });
  }

  function setupIncrementalResultsEdit() {
    $incrementalResultsEditIcon
      .offOn('click.showIncrementally', () => showIncrementalResultsOptions(true));

    $incrementalResultsOptions
        .find('#canc-incremental-results')
        .offOn('click.hideIncremental', function hideIncrementalResults() {
          setIncrementalResultsValue();
          showIncrementalResultsOptions(false);
        });

    $incrementalResultsOptions
        .find('#save-incremental-results')
        .offOn('click.saveIncremental', function triggerSave() {
          const incremental_results_enabled = $incrementalResultsOptions
              .find('#incremental-results_enabled').prop('checked');
          if (incremental_results !== incremental_results_enabled && builder.dataset()) {
            builder.dataset().enable_incremental_results(incremental_results_enabled);
            builder.dataset()
              .save(function afterSave() {
                ds.builderReload(function resetOptions() {
                  incremental_results = incremental_results_enabled;
                  setIncrementalResultsValue();
                  showIncrementalResultsOptions(false);
                });
            });
          }
        });
  }

  // Set up edit/save of DataConnection switch of a Dataset
  ds.setupSwitchDataConnection = function() {
    arc.dataconnection.dataconnections()
                      .dataconnections_fetch(setupSwitchDataConnection);

    function setupSwitchDataConnection(dataconnections) {
      // Sort and fill up single select list of data connections
      dataconnections.sort((a, b) => {
        if (a.name.toLowerCase() < b.name.toLowerCase()) {
          return -1;
        }

        if (a.name.toLowerCase() > b.name.toLowerCase()) {
          return 1;
        }

        return 0;
      });

      const $dataconnswitchlist = $('#dataconnection-name');
      const $extractDataConnectionList = $('#extract-dc-name');

      dataconnections.forEach((dc) => {
        const option = arc.misc.safe_substitute(
          '<option value="%VALUE%" data-perms="%PERMS%">%NAME%</option>',
          {
            NAME: dc.name,
            VALUE: dc.id,
            PERMS: dc.perms.join(' '),
          }
        );
        // Append all connection types to connection switch list
        $dataconnswitchlist.append(option);

        // Check if connection type support Data Extracts
        if (!arc.dataconnection.supportsDataExtractsTarget(dc.type)) {
          return;
        }

        // And check if user has connection level permissions for creating Data Extracts
        if (!dc.perms.includes('dc_aviews')) {
          return;
        }

        // If both conditions are met, add connection option to extract connection list
        $extractDataConnectionList.append(option);
      });

      // On edit click, show form and hide edit pencil
      $('#edit-dataconnection-name-icon').offOn('click.editDataConnectionName',
        function showSwitchDataConnection() {
          const dcid = builder.dataset().dataconnection_id();
          $('#dataconnection-name').val(dcid);
          $('.edit-dataconnection-name').addClass('edc-switchmode');
        });

      // On save, get selected value, save to backend and hide form
      $('#save-switch-dc-btn').offOn('click.saveSwitchDc', () => {
        const dcid = Number($('#dataconnection-name option:selected').attr('value'));
        const existing_dcid = builder.dataset().dataconnection_id();
        if (dcid !== existing_dcid) {
          switchDataConnection(dcid);
        }
      });

      $('#cancel-switch-dc-btn').offOn('click.cancelSwitch', hideSwitchDataConnection);
    }

    function hideSwitchDataConnection() {
      $('.edit-dataconnection-name').removeClass('edc-switchmode');
    }

    function switchDataConnection(newdcid) {
      builder.dataset()
             .dataconnection_id(newdcid)
             .save(() => {
                ds.builderReload(hideSwitchDataConnection);
              });
    }

    return ds;
  };

  const $reloadTableModal = $('#reload-tables-modal');
  arc.misc.handleAlertContainerActivationOnModal($reloadTableModal);
  function refreshFields() {
    if (arc.misc.modalShown($reloadTableModal)) {
      return;
    }

    $('#reload-tables-spinner').show();
    const $msg = $('#reload-tables-message');
    $msg.text('Scanning dataset table column information...');
    $reloadTableModal.find('.alert').remove();
    $reloadTableModal.modal('show');
    return new Promise((resolve) => {
      builder.dataset().update_tables((changed, errorText) => {
        arcl.attributes_set_editmode(true);

        $('#reload-tables-spinner').hide();
        $('#reload-tables-close').show();

        if (errorText) {
          $msg.text('');
          arc.misc.addErrors([errorText], $reloadTableModal.find('.modal-body'), 'field-refresh-failed');
        } else if (!changed) {
          $msg.text('No table changes detected');
          resolve();
        } else {
          $msg.text('Table columns updated');
          resolve(changed);
        }

        builder.paintAttributes(builder.dataset());
      });
    });
  }

  ds.initialize = function() {
    baseSetup();
    arc.misc.updateDivSysPerms();

    $('.chosen-select').chosen({
      search_contains: true,
    });

    arcl.first_sort = true;

    loader = arc.loadbar();

    $('#viz-segment-create-title').text('Edit Segment');
    $('#viz-segment-save').text('Save');

    // remove commas to resolve sorting issue
    ds.builderReload(ds.datasetRefreshed);

    d3.selectAll('.dataset-dashboard-create')
      .on('click.create', () => {
        builder.newDashboard(dataset_id);
      });

    // build the dataset renaming editor
    const $datasetName = $('#app-dataset-name');
    const $datasetNameIcon = $('#edit-dsname-icon');
    const $editDatasetNameForm = $('#edit-dsname-form');
    const $editDatasetNameInput = $editDatasetNameForm.find('#edit-dsname-input');
    const hideDatasetNameForm = function() {
      $datasetName.show();
      $datasetNameIcon.show();
      $editDatasetNameForm.hide();
    };

    $datasetNameIcon.offOn('click.datasetNameIcon', () => {
      $datasetNameIcon.hide();
      $datasetName.hide();
      $editDatasetNameForm.show();
      $editDatasetNameInput.val($datasetName.text());
    });
    $editDatasetNameForm.find('#canc-dsname-btn').offOn('click.cancelDatasetName', hideDatasetNameForm);
    $editDatasetNameForm.find('#save-dsname-btn').offOn('click.saveDatasetName', () => {
      if ($editDatasetNameInput.val() === $datasetName.text()) return;
      builder.dataset()
        .dataset_name($editDatasetNameInput.val())
        .save(() => {
          ds.builderReload(hideDatasetNameForm);
        });
    });

    // build the dataset description editor
    const $detailTab = $('#detail');
    const $editDescForm = $detailTab.find('#edit-desc-form');
    const $datasetDescription = $detailTab.find('#dataset-description');
    const $editDescInput = $editDescForm.find('#edit-desc-input');
    const $editDescIcon = $detailTab.find('#edit-desc-icon');

    const hideDescriptionForm = function() {
      $editDescForm.hide();
      $editDescIcon.show();
      $datasetDescription.show();
    };

    $editDescIcon.offOn('click.editDescIcon', () => {
      $datasetDescription.hide();
      $editDescIcon.hide();
      $editDescForm.show();
      $editDescInput.css('height', $datasetDescription.height());
      $editDescInput.val($datasetDescription.text());
    });

    $editDescForm.find('#save-desc-btn').offOn('click.saveDescIcon', () => {
      builder.dataset()
        .description($editDescInput.val())
        .save(() => {
          ds.builderReload(hideDescriptionForm);
        });
    });

    $editDescForm.find('#canc-desc-btn').offOn('click.cancelDescBtn', hideDescriptionForm);

    ds.setupSwitchDataConnection();

    const showJoinElimOptions = function(_) {
      if (_) {
        $joinElimValue.hide();
        $joinElimEditIcon.hide();
        $joinElimOptions.show();
      } else {
        $joinElimValue.show();
        $joinElimEditIcon.show();
        $joinElimOptions.hide();
      }
    };

    $joinElimEditIcon.off().on('click', () => {
      showJoinElimOptions(true);
    });
    $joinElimOptions.find('#canc-join_elim').off().on('click', () => {
      ds.set_join_elim_value();
      showJoinElimOptions(false);
    });
    $joinElimOptions.find('#save-join_elim').off().on('click', () => {
      const join_elim_enabled = $joinElimOptions.find('#join_elim_enabled').prop('checked');
      if (join_elim !== join_elim_enabled && builder.dataset()) {
        builder.dataset().enable_join_elim(join_elim_enabled);
        builder.dataset()
        .save(() => {
          ds.builderReload(() => {
            join_elim = join_elim_enabled;
            ds.set_join_elim_value();
            $joinElimValue.show();
            $joinElimEditIcon.show();
            $joinElimOptions.hide();
          });
        });
      }
    });

    const showCacheOptions = function(editMode) {
      if (editMode) {
        $cacheOptions.show();
        $cacheText.hide();
        $cachePencil.hide();
        $cacheReset.hide();
        const mode = $cacheRadio.filter(':checked').val();
        $cacheRetention.prop('disabled', (mode !== 'Enabled'));
      } else {
        $cacheOptions.hide();
        $cacheText.show();
        $cachePencil.show();
        $cacheReset.show();
      }
    };

    $cachePencil.off().off().on('click', () => {
      showCacheOptions(true);
    });

    $cacheOptions.find('#canc-ds-cache').off().on('click', () => {
      ds.set_cache_options();
      showCacheOptions(false);
    });

    $cacheRadio.off().on('click', () => {
      showCacheOptions(true);
    });

    $cacheReset.off().on('click', () => {
      builder.dataset().reset_cache((err) => {
        if (err) {
          arc.report.alertMessageSmall(`Error on cache reset: ${err}`, 'alert-danger');
        } else {
          arc.report.alertMessageSmall('Dataset result cache cleared!', 'alert-success');
          ds.builderReload();
        }
      });
    });

    d3.selectAll('.dataset-set-version').on('click.setversion', () => {
      builder.dataset().set_version((err) => {
        if (err) {
          arc.report.alertMessageSmall(`Error on version reset: ${err}`, 'alert-danger');
        } else {
          arc.report.alertMessageSmall('Version successfully set!', 'alert-success');
          ds.builderReload();
        }
      });
    });

    $cacheOptions.find('#save-ds-cache').off().on('click', () => {
      const mode = $cacheRadio.filter(':checked').val();
      const cache = {
        OVERRIDE: (mode !== 'From Connection') ? 1 : 0,
        ENABLED: (mode === 'Enabled') ? 1 : 0,
        RETENTION: parseInt($cacheRetention.val() || 0, 10),
      };
      builder.dataset().put_cache_settings(cache);
      builder.dataset().save(() => {
        $cacheText.text(mode);
        showCacheOptions(false);
        arc.report.alertMessageSmall('Dataset result cache settings updated!', 'alert-success');
      });
    });

    setupIncrementalResultsEdit();
    setupKSQLStreamPositionEdit();

    arc.report.fetchDatasetApps({
      datasetId: dataset_id,
      successCb: arc.report.drawPermissionedApps,
    });

    // ---- Code to support dataset attribute edit page ----
    const $attributes = $('#attributes');

    // Dynamic popover for column information in display mode
    $('body').on('show.bs.popover', '.table-list-value', function() {
      if (!$(this).attr('data-content')) {
        // If popover is being created for first time
        const li = $(this).closest('li[arcid$=-column-active]');
        const col = li.data('_arc_data');
        const name = col.attr('name') || col.attr('basecol');
        const display = col.attr('alias') || '';
        const comment = col.attr('comment') || col.attr('hivecomment') || '-';
        const cmmntlabel = 'Comment:';
        const dflagg = col.attr('dflagg') || arc.misc.defaultAgg(col.attr('type'));
        const expr = col.attr('expr') || '';
        const html = arc.clone_template('column-content-popover',
          { basecol: name, display, comment, cmmntlabel, dflagg, expr });

        // If we have column name, it's not a cloned column so expression not allowed
        if (col.attr('name')) {
          html.find('[arcid=column-content-expr]').empty();
        }

        $(this).attr('data-content', html.html());
      }

      hideAllPopovers();
    });

    // Make the column type dropdown show/hide on click
    $attributes.on('click', '.dsattr-coltype-dropdown', function() {
      // Add in the list items if not already there
      const list = $(this).parent().find('[arcid=dsattr-coltype-placeholder]');
      if (list.length) {
        list.replaceWith(arc.clone_template('dsattr-coltype-list').html());
        const col = $(this).closest('[arcid$=-column-active]').data('_arc_data');
        if (!col.attr('cast')) {
          $(this).parent().find('.type-removecast').closest('a')
            .remove();
        }

        $(this).dropdown('toggle');
      }

      $(this).dropdown('toggle');
    });

    // Set the visibility attributes for dataset attributes column entry
    arcl.attributes_set_column_hidden = function(li, col) {
      if (col.attr('hidden')) {
        li.find('.fa-eye').hide();
        li.find('.fa-eye-slash').show();
        li.addClass('hidden-columns');
      } else {
        li.find('.fa-eye').show();
        li.find('.fa-eye-slash').hide();
        li.removeClass('hidden-columns');
      }
    };

    // Enable click handlers for various items in column rows shown
    $('body').offOn('click.setUpClickHandlers', '.table-list-wrapper', (e) => {
      const $target = $(e.target);
      const $targetParent = $target.parent();
      const li = $target.closest('li');
      const col = li.data('_arc_data');
      popover_column = li;

      if ($target.hasClass('column-action-visibility-menu')) {
        // Click handler for single column visibility icon
        const $targetChild = $target.children();
        if ($targetChild.hasClass('fa-eye-slash')) {
          $target.html('<i class="fa fa-eye"></i> Show');
        } else {
          $target.html('<i class="fa fa-eye-slash"></i> Hide');
        }

        col.attr('hidden', !col.attr('hidden'));
        arc.report.pagenav.needs_save(true);
        arcl.attributes_set_column_hidden(li, col);
      }

      const $modal = $('#edit-column-modal');
      arc.misc.handleAlertContainerActivationOnModal($modal);
      function createHierarchy() {
        const action = $target.data('action');
        const $hierarchyModal = $('#hierarchy-modal');
        if (arc.misc.modalShown($hierarchyModal)) {
          return;
        }

        $hierarchyModal.find('textarea.hierarchy-comment').val(
          col.attr('comment') || col.attr('hivecomment') || ''
        );

        switch (action) {
          case 'create':
            $hierarchyModal.find('.modal-title').text('Create Hierarchy');
            $hierarchyModal.addClass('create').removeClass('edit');
            $hierarchyModal.find('input.hierarchy-name').val('');
            $hierarchyModal.find('button[type=submit]').attr('action', 'create').text('Create');
            break;
          case 'edit':
            $hierarchyModal.find('.modal-title').text('Edit Hierarchy');
            $hierarchyModal.find('input.hierarchy-name').val(col.attr('alias'));
            $hierarchyModal.addClass('edit').removeClass('create');
            $hierarchyModal.find('button[type=submit]').attr('action', 'edit').text('Update');
            break;
          default: break;
        }

        $hierarchyModal.modal('show');
      }

      if ($targetParent.hasClass('column-edit-attr')) {
        const $editColumnModal = $('#edit-column-modal');
        if (arc.misc.modalShown($editColumnModal)) {
          return;
        }

        $editColumnModal.modal('show');
        arc.admin.columnColorHandler(col, builder).draw();

        const displayOptions = arc.utils.displayOptions($editColumnModal);
        js_editor = displayOptions.setUpEditor();
        const dataset = builder.dataset();
        displayOptions.resetHandlers(dataset, col.attr('displayformat'));
        table_editor = arc.datasets.populateEditColumnModal(col, table_editor, displayOptions);

        const handleButtonClick = function(event) {
          const $clickTarget = $(event.target);
          const action = $clickTarget.attr('action');

          function acceptColumnChanges(params) {
            arc.report.pagenav.needs_save(true);
            const old_alias = col.display_name();
            arc.datasets.applyColumnEditChanges(col, params);
            const newli = builder.paintOneColumn(col);
            li.replaceWith(newli);
            setDraggablityOfDatasetFields(/* disable */ false);
            updateAllHierarchies(old_alias, newli);
            $editColumnModal.modal('hide');
            destroyEditors(displayOptions);
          }

          function showValidationPassed(params) {
            params.loc.after(arc.clone_template('col-val-passed').html());
          }

          const mustValidate = $('#override-validation').prop('checked');
          switch (action) {
            case 'validate':
              arc.datasets.validateColumnEditChanges(col, table_editor, displayOptions, true,
                showValidationPassed);
              break;

            case 'cancel':
              $editColumnModal.modal('hide');
              break;

            case 'remove':
              deleteColumn(li);
              $editColumnModal.modal('hide');
              break;

            case 'apply':
              arc.datasets.validateColumnEditChanges(col, table_editor, displayOptions,
                  mustValidate, acceptColumnChanges);
              break;
            default: break;
          }

          if (action !== 'validate' && action !== 'apply' && action !== 'test') {
            destroyEditors(displayOptions);
          }

          event.stopPropagation();
          event.preventDefault();
        };

        $editColumnModal.find('button:not(.update-example-btn)').off()
          .on('click', handleButtonClick);
      }

      if ($target.hasClass('column-action-clone')) {
        arc.report.pagenav.needs_save(true);
        const newcol = col.table().clone_column(col);
        const newli = builder.paintOneColumn(newcol);
        li.after(newli);
        setDraggablityOfDatasetFields(/* disable */ false);
      }

      if ($target.hasClass('dim-agg-type')) {
        const type = li.find('[arcid=dsattr-dimagg]');
        if (type.text() === 'Dim') {
          type.text('Mes');
        } else {
          type.text('Dim');
        }

        col.attr('isdim', !col.attr('isdim'));
        col.attr('useroverrodeisdim', true);

        arc.report.pagenav.needs_save(true);
      }

      if ($target.hasClass('expand-icon')) {
        $target.toggleClass('fa-angle-down fa-angle-right');
        li.find('> ul').slideToggle();
      }

      if ($targetParent.hasClass('hierarchy-cross')) {
        li.remove();
      }

      if ($target.hasClass('hierarchy-modal-trigger')) {
        createHierarchy();
      }

      if ($target.attr('arcid') === 'column-edit-remove') {
        deleteColumn(li);
      }
    });

    const $hierarchyModal = $('#hierarchy-modal');
    const $hierarchyForm = $hierarchyModal.find('form');
    $hierarchyForm.find('.modal-footer').on('click', (e) => {
      const $target = $(e.target);
      const action = $target.attr('action');
      if (!action) {
        // User Clicked on Cancel.
        // Modal should disappear automatically
        return;
      }

      const li = popover_column;
      const col = li.data('_arc_data');
      const enteredName = $hierarchyForm.find('.modal-body input[type=text]').val();
      const enteredComment = $hierarchyModal.find('textarea.hierarchy-comment').val();
      let erroneousName = '';

      switch (action) {
        case 'create': {
          const newcol = col.table().clone_column(col);
          newcol.attr('dimh', col.attr('alias') || col.attr('name'));
          newcol.attr('type', 'HIERARCHY');
          newcol.attr('comment', enteredComment);
          newcol.attr('isdim', true);
          $hierarchyForm.find('.form-text.with-errors').empty();
          erroneousName = newcol.table().dataset().check_alias(newcol, enteredName);
          if (!enteredName.trim()) {
            erroneousName = 'Hierarchy cannot have a blank name';
          }

          if (erroneousName) {
            $hierarchyForm.find('.form-text.with-errors').append(
                arc.clone_template('col-val-failed', { errtext: erroneousName }).html()
            );
            return;
          }

          newcol.attr('alias', enteredName);
          const newli = builder.paintOneColumn(newcol);
          li.after(newli);

          setDraggablityOfDatasetFields(false /* disable */);

          break;
        }

        case 'edit':
          $hierarchyForm.find('.form-text').empty();
          erroneousName = col.table().dataset().check_alias(col, enteredName);
          if (!enteredName.trim()) {
            erroneousName = 'Hierarchy cannot have a blank name';
          }

          if (erroneousName) {
            $hierarchyForm.find('.form-text').append(
                arc.clone_template('col-val-failed', { errtext: erroneousName }).html()
                );
            return;
          }

          col.attr('alias', enteredName);
          col.attr('comment', enteredComment);
          li.find('.table-list-value-edit').text(enteredName);

          break;
        case 'delete':
          deleteColumn(li);
          break;
        default: break;
      }

      arc.report.pagenav.needs_save(true);
      $hierarchyModal.modal('hide');
      e.preventDefault();
    });

    // Click handler for attribute column data type menu
    $('body').on('click', '.dropdown-menu.dsattr-coltype-menu a', function() {
      // DJC -- bad to be dependent on specific text of the dropdown display
      const li = $(this).closest('[arcid=dsattr-column-active]');
      const col = li.data('_arc_data');
      const coltype = col.attr('type');
      let newtype = coltype;
      const colval = $(this).attr('value');
      switch (colval) {
        case 'real': newtype = 'DOUBLE'; break;
        case 'integer': newtype = 'INT'; break;
        case 'boolean': newtype = 'BOOLEAN'; break;
        case 'timestamp': newtype = 'TIMESTAMP'; break;
        case 'string': newtype = 'STRING'; break;
        case 'remove': newtype = undefined; break;
        default: newtype = 'STRING';
      }

      col.attr('cast', newtype);
      newtype = col.attr('cast') || col.attr('type');

      // Repaint the row
      const newli = builder.paintOneColumn(col);
      li.replaceWith(newli);
      arc.report.pagenav.needs_save(true);
    });

    // Click handler for attribute edit expand/collapse table
    $('body').on('click', '.table-list-header', function() {
      $(this).parent().children('.table-list-wrapper').slideToggle();
      $(this).find('.fa-caret-down').toggle();
      $(this).find('.fa-caret-right').toggle();
    });

    // Click handler for attribute page expand/collapse all button
    $('body').on('click', '.collapse-table-link', function() {
            if ($(this).find('a').text() === 'Collapse All') {
                    $('.table-list-header').find('.fa-caret-down').hide();
              $('.table-list-header').find('.fa-caret-right').show();
                    $('.table-list-wrapper').slideUp();
            } else {
                    $('.table-list-header').find('.fa-caret-down').show();
              $('.table-list-header').find('.fa-caret-right').hide();
                    $('.table-list-wrapper').slideDown();
            }

            $(this).find('a').text((i, text) => (text === 'Collapse All' ? 'Expand All' : 'Collapse All'));
    });

    // make attributes panel resizable
    $attributes.find('.attribute-wrapper').resizable({
        handles: 'e'
    });

    $attributes.find('#display-comments-btn').offOn('click.displayCommentsBtn', function() {
      $('.attr-comment').toggle();
      $(this).find('span').text((i, text) => (text === 'Show Comments' ? 'Hide Comments' : 'Show Comments'));
    });

    // Set the edit mode state for dataset sources page
    arcl.attributes_set_editmode = function(enabled) {
      const $btn = $('#toggle-names-btn');
      if (builder.dataset().has_title_cased_names()) {
        $btn.empty().append('<span class="fa fa-remove"></span> title_case');
      } else {
        $btn.empty().append('<span class="titlecase-icon">T</span> Title Case');
      }

      // hide and show the Edit/Save buttons
      // Update CSS, draggability, sortability, etc for different modes
      if (enabled) {
        $('#attributes').addClass('editmode')
                        .removeClass('viewmode');
        setDraggablityOfDatasetFields(/* disable */ false);
      } else if (arc.misc.hasPerm('ds_manage', builder.dataset().dataset_perms())) {
          $attributes.addClass('viewmode')
            .removeClass('editmode');
          setDraggablityOfDatasetFields(/* disable */ true);
        }

      initializeAllPopovers();
      hideAllPopovers();
    };

    $('[data-toggle="tooltip"]').tooltip({
      delay: {
        show: 500,
        hide: 100
      }
    });

    $('.table-list-header').tooltip({
      delay: {
        show: 1000,
        hide: 100
      }
    });

    $('.column-actions a, .column-actions span, .attribute-header span').tooltip({
        placement: 'top'
    });

    // need to organize the tooltip like this so that they work on dynamically added content
    $('body').tooltip({
      selector: '.derived-column-indicator, .column-actions a, .column-actions span',
      placement: 'top'
    });

    // ---- End of dataset attribute edit stuff

    // Set up the edit-join-modal with table names and join columns
    $('body').on('click', '.image-wrapper.join-type-image', function() {
      // Don't do anything if not in edit mode
      if (!$('#save-join-btn').is(':visible')) {
        return;
      }

      const typetext = $(this).attr('typetext');
      if (!typetext) {
        return;
      }

      arc.report.pagenav.needs_save(true);

      const pop = $(this).closest('[arcid=join-content-popover-active]');
      const t = pop.find('[arcid=edit-join-button]').data('tabinfo');

      t.join_type = typetext;
      t.dstab.join_type(t.join_type);
      drawJoinType(pop, t);
    });

    $('body').on('click', '[arcid=edit-join-button]', function setupEditJoinModal() {
      const $modal = $('#edit-join-modal');
      if (arc.misc.modalShown($modal)) {
        return;
      }

      function filter_pk_fk_columns(table_params, child_tname) {
        const pk_fk = {};
        let child_tablename;
        for (let i = 0; i < table_params.length; i++) {
          if (table_params[i][0].startsWith('fkey_')) {
            child_tablename = (table_params[i][1] !== '') ?
               table_params[i][1].split('pkey_')[1].split('.').slice(0, 2).join('.') : '';
            if (child_tablename === child_tname) {
              pk_fk[table_params[i][0].split('.').pop()] = table_params[i][1].split('.').pop();
            }
          }
        }

        return pk_fk;
      }

      function setup_checked_click() {
        const selector_map = {
          "[name='foreign-key-parent']": "[name='foreign-key-child']",
          "[name='foreign-key-child']": "[name='foreign-key-parent']"
        };
        $.each(selector_map, (selector, counterpart_selector) => {
          function apply_changes(elmt) {
            const counterpart_elmnt = $(elmt).closest('li').find(counterpart_selector);
            if (elmt.checked) {
              counterpart_elmnt.closest('label').find('span').html('Primary Key');
              counterpart_elmnt.attr('disabled', 'disabled');
              counterpart_elmnt.hide();
            } else {
              counterpart_elmnt.closest('label').find('span').html('Foreign Key');
              counterpart_elmnt.removeAttr('disabled');
              counterpart_elmnt.show();
            }
          }

          // if pk and fk is already set on a join pair, make necessary changes
          $(selector).each(function () {
            apply_changes(this);
          });

          // setup click function on all join pairs
          $(selector).offOn('click.applyChanges', function () {
            apply_changes(this);
          });
        });
      }

      const t = $(this).data('tabinfo');

      // When 'Edit' button is clicked, hide the popover showing join details
      $('.join-image').popover('hide');

      const p = t.src_table.dstab;
      const child = t.dstab;
      const p_table_params = [];
      const child_table_params = [];

      $modal.find('.join-tables-list-wrapper').remove();
      $modal.find('#join-tables-expr').empty();
      getSampleData(p, p_table_params, function gotFirstTableCb() {
        getSampleData(child, child_table_params, function gotSecondTableCb() {
          toggleLoadingSpinner(false);
          p_fk = filter_pk_fk_columns(p_table_params, child.tablename());
          child_fk = filter_pk_fk_columns(child_table_params, p.tablename());
          const parent_colnames = p.get_colnames().sort();
          const child_colnames = child.get_colnames().sort();
          const $form = $modal.find('form');
          const $editTargetLblIcon = $modal.find('#edit-target-lbl-icon');
          const $targetLbl = $modal.find('.modal-target-lbl');
          const $editTargetLblForm = $modal.find('#edit-target-lbl-form');
          const $editTargetLblInput = $editTargetLblForm.find('#edit-target-lbl-input');
          const dataset = builder.dataset();

          $editTargetLblIcon.find('span').tooltip({ trigger: 'hover' });
          removeErrAlerts();

          const hideTargetLblForm = function() {
            $targetLbl.show();
            $editTargetLblIcon.show();
            $editTargetLblForm.hide();
          };

          $editTargetLblIcon.offOn('click.editTargetLblIcon', () => {
            $editTargetLblIcon.hide();
            $targetLbl.hide();
            $editTargetLblForm.show();
            $editTargetLblInput.val($targetLbl.text());
          });

          $editTargetLblForm.find('#canc-target-lbl-btn').tooltip({ trigger: 'hover' });
          $editTargetLblForm.find('#canc-target-lbl-btn').offOn('click.cancelTargetLbl', () => {
            hideTargetLblForm();
            removeErrAlerts();
          });

          $editTargetLblForm.find('#save-target-lbl-btn').tooltip({ trigger: 'hover' });
          $editTargetLblForm.find('#save-target-lbl-btn').offOn('click.saveTargetLbl', () => {
            if ($editTargetLblInput.val() === $targetLbl.text()) return;

            const table = dataset.get_table(t.name);
            try {
              const alias = $editTargetLblInput.val();
              dataset.validate_user_input_table_alias(table, alias);
              $targetLbl.text(alias)
                        .attr('title', alias);
              hideTargetLblForm();
              removeErrAlerts();
            } catch (error) {
              removeErrAlerts();
              $modal.find('#edit-join-clear').before(
                arc.clone_template('col-val-failed', { errtext: error.message }).html()
              );
            }
          });

          function removeErrAlerts() {
            $modal.find('.alert-danger').remove();
          }

          // Set up add a new join pair on '+' click
          const add_new_pair = function() {
            const pair = ['select column...', 'select column...'];
            const temp = buildEditJoinPair(
                         pair.slice(0, 1).concat(parent_colnames),
                         pair.slice(1, 2).concat(child_colnames),
                         pair, t
            );
            $form.find('.join-tables-list-wrapper').append(temp);
            setup_checked_click();
          };

          const $exprList = $('#join-tables-expr');
          const $addExprBtn = $modal.find('[arcid="add-expr-link"]');

          $modal.find('.edit-join-clear')
                .off('click.clearJoin')
                .on('click.clearJoin', function resetJoins() {
                  $modal.find('.join-tables-list li').remove();
                  $exprList.find('li').remove();
                  $addExprBtn.trigger('click');
                  add_new_pair();
                });
          $modal.find('.modal-source-lbl')
                .text(t.src_table.name)
                .attr('title', t.src_table.name);
          $targetLbl.text(t.name)
                    .attr('title', t.name);
          hideTargetLblForm();

          // allow table name alias edit only for duplicated tables
          const table = dataset.get_table(t.name);
          if (table.is_duplicate()) {
            $editTargetLblIcon.show();
          } else {
            $editTargetLblIcon.hide();
          }

          const joinTables = builder.dataset().tables().filter(
            (table) => [t.name, t.src_table.name].includes(table.tablename())
          );
          arc.report.addableExprList(
            'join-tables-expr',
            t.join_exprs, $addExprBtn,
            builder.dataset(), {
              exprType: 'join',
              title: 'Join Expression',
              tables: joinTables,
              excludeRecordCountAutocomplete: true,
            }
          );

          $form.data('tabinfo', t);
          const $pairsList = $('<div class="join-tables-list-wrapper"></div>');

          t.join_pairs.forEach(function createAndDisplayJoinPair(pair) {
            const temp = buildEditJoinPair(parent_colnames, child_colnames, pair, t);
            $pairsList.append(temp);
          });

          $modal.find('.modal-target-lbl-wrapper').after($pairsList);

          if (!arc.dataconnection.isArcEngine(builder.dataset().dataconnection_type())) {
            // hide foreign key for non arcengine connections
            $("label[arcid='foreign-key']").each(function () {
              $(this).hide();
            });
          } else {
            setup_checked_click();
          }

          $form.find('[arcid=add-column-link]').off('click.addPair')
                                               .on('click.addPair', add_new_pair);

          // And add one now if no existing pairs
          if (t.join_pairs.length === 0) {
            add_new_pair();
          }
        });
      });

      $modal.modal('show');
    });

    // Update join column on 'Apply' button hit
    // DJC -- Need to add validation here: need at least one column pair
    $('body').on('click', '#edit-join-apply', function applyJoin() {
      arc.report.pagenav.needs_save(true);

      const $form = $('#join-tables-form');
      const t = $form.data('tabinfo');
      const joinedTable = t.dstab;
      const newTableAlias = $form.find('.modal-target-lbl').text();
      if (joinedTable.is_duplicate() && joinedTable.table_alias() !== newTableAlias) {
        builder.dataset().update_tablealias(t.dstab, $form.find('.modal-target-lbl').text());
        builder.paintAttributes(builder.dataset());
      }

      arc.admin.dataset().readJoinFromForm({ update: true, p_fk, child_fk });

      redrawJoins(builder.dataset());

      if (builder.dataset().has_title_cased_names()) {
        builder.dataset().toggle_colnames();
      }

      setDataButton(dataBtnTitles.reload);
    });

    // Drop this table and all children on the delete-join-link-button press
    $('body').offOn('click.deleteJoin', '[arcid=delete-join-link-button]', function() {
      arc.report.pagenav.needs_save(true);
      const t = $(this).parent().find('[arcid=edit-join-button]').data('tabinfo');
      builder.dataset().drop_table(t.name);
      redrawJoins(builder.dataset());
      setDataButton(dataBtnTitles.reload);
    });

    // remove row of linked columns
    $('body').on('click', '.join-modal-remove', function() {
      const joinList = $(this).parents('.join-tables-list');
      const pairsList = $(this).parents('.join-tables-list-wrapper');
      const joinId = joinList.find('.form-control')[0].id;
      if (joinEditors[joinId]) {
        joinEditors[joinId].destroy();
      }

      joinList.remove();
      if (!pairsList.find('.join-tables-list > li').length) {
        $('[arcid=add-column-link]').trigger('click');
      }
    });

    $('#viz-join image').on('click', function () {
      $('#viz-join image').not(this).popover('hide'); // close all other popovers
    });

    // close popovers when user clicks anywhere other than the open popover
    $('body').on('click', (e) => {
        if ($(e.target).data('toggle') !== 'popover' &&
            $(e.target).parents('.popover.in').length === 0) {
            hideAllPopovers();
        }
    });

    // Edit/Save/Cancel button handling for join edit page
    $('#edit-join-btn').offOn('click.editJoin', () => {
      datasetSourcesSetEditMode(true);
      redrawJoins(builder.dataset());
    });

    $('#save-join-btn').offOn('click.saveJoin', () => {
      builder.dataset().save(() => {
        arc.report.pagenav.needs_save(false);
        datasetSourcesSetEditMode(false);
        ds.builderReload(ds.datasetRefreshed);
        setDataButton(dataBtnTitles.reload);
      });
    });

    $('#cancel-join-btn').offOn('click.cancelJoin', () => {
      datasetSourcesSetEditMode(false);

      // DJC -- might be better here to reset just dataset, not other bits...not sure
      ds.builderReload(() => {
        ds.datasetRefreshed();
        setDataButton(dataBtnTitles.reload);
      });
    });

    // Edit/Save/Undo button handling for attribute edit page
    $('#edit-attr-btn').offOn('click.editAttr', () => {
      arcl.attributes_set_editmode(true);
      const $display_comments = $attributes.find('#display-comments-btn');
      if ($display_comments.find('span').text().trim() === 'Hide Comments') { $display_comments.trigger('click'); }
    });

    $('#toggle-names-btn').offOn('click.toggleNames', () => {
      builder.dataset().toggle_title_cased_names();
      builder.dataset().toggle_colnames();
      builder.paintAttributes(builder.dataset());
      arcl.attributes_set_editmode(true);
      arc.report.pagenav.needs_save(true);
    });

    $('#refresh-attr-btn').offOn('click.refreshAttr', async () => {
      try {
        await refreshFields();
      } catch (_e) {
        // refreshFields handles the error
      } finally {
        arcl.attributes_set_editmode(true);
      }
    });

    $('#save-attr-btn').offOn('click.saveAttr', () => {
      // Update latest hierarchy information in columns before saving to dataset
      const $allHierarchyLis = $('.table-list-wrapper li[arcid=dsattr-hierarchy-column-active]');
      let col;
      let $childElementLis;
      let childElementNames;

      // Update hierarchy columns with latest child elements info
      $.each($allHierarchyLis, (_, li) => {
        const $li = $(li);
        col = $li.data('_arc_data');
        $childElementLis = $li.find('ul.child-elements-list > li');
        childElementNames = [];
        $.each($childElementLis, (index, childLi) => {
          childElementNames.push($(childLi).find('.table-list-value').text().trim());
        });
        if (childElementNames.length > 0) {
          col.attr('dimh', childElementNames.join(','));
        } else {
          col.attr('dimh', []);
        }
      });

      builder.dataset().save((err_msg) => {
        if (!err_msg) {
          builder.removeCache();
          ds.builderReload(() => {
            arcl.attributes_set_editmode(false);
            arc.report.pagenav.needs_save(false);
            ds.datasetRefreshed();
            setDataButton(dataBtnTitles.reload);
          });
        }

        // we are currently not using the err_msg here, it gets displayed however in an
        // arc.report.alertMessage call in Dataset.save
      });
    });
    $('#undo-attr-btn').offOn('click.undoAttr', () => {
      ds.builderReload(() => {
        ds.datasetRefreshed();
        arcl.attributes_set_editmode(false);
      });
    });
    $('#reload-tables-close').offOn('click.reloadTables', () => {
      $('#reload-tables-modal').modal('hide');
    });

    // toggle the sample data list in the join popover
    // need to handle the jquery differently for popovers that don't exist on page load
    $('body').on('click', '.sample-data-link', function() {
      $(this).parent().children('.sample-list').slideToggle();
      $(this).children('.fa-caret-down').toggle();
      $(this).children('.fa-caret-right').toggle();
    });

    // Handle "Display format setting"
    const $applyDisplayFormat = $('#apply-display-format');
    $applyDisplayFormat.attr('checked', isDisplayFormatUsed);
    $applyDisplayFormat.offOn('change.updateDisplayFormat', (event) => {
      const val = event.currentTarget.checked;
      arc.frontend.vizCookie.setDisplayFormat(val);
      isDisplayFormatUsed = val;

      if (isSampleDataTableDisplayed()) {
        renderSampleData();
      }
    });

    // toggle display data table
    $showDataBtn.offOn('click.showData', () => {
      const btnText = $dataBtn.text();
      switch (btnText) {
        case dataBtnTitles.hide:
          setDataButton(dataBtnTitles.show);
          $sampleDataTable.slideToggle();
          break;
        default: // dataBtnTitles.show, dataBtnTitles.reload
          loadSampleDataTable(btnText);
          setDataButton(dataBtnTitles.hide);
      }
    });

    /* add hash and selected tab id to the url so that when the page is
       refreshed or user uses the back button, the selected tab is maintained */

    // TODO: There is a problem here in that we haven't loaded the dataset yet
    //       so we don't know if view management for this dataset is enabled.
    if (window.location.hash !== '') {
      $(`.nav-sidebar > li > a[href="${window.location.hash.replace('tab_', '')}"]`).tab('show');
    }

    $('.nav-sidebar a[data-toggle="tab"]').on('shown.bs.tab', (e) => {
      window.location.hash = `tab_${e.target.hash.substr(1)}`;
      return false;
    });
  };

  /* Read Join data from DOM form
   * @param  config   object - with the following possible keys:
   *                      update    boolean - whether or not to update the table data
   *                                  If true, will update existing table data object and return
   *                                  that, otherwise will make a clone and update and return that
   *                      p_fk      object - existing primary and foreign key info on parent table
   *                      child_fk  object - existing primary and foreign key info on child table
   *                      skipElem jQuery element - input element whose expression is to be ignored
   * @return    tableinfo object on table that is target of join
   */
  ds.readJoinFromForm = function(config = {}) {
    const $form = $('#join-tables-form');
    const t = $form.data('tabinfo');

    const p = t.src_table.dstab;
    const child = t.dstab;

    const parent_colnames = p.get_colnames();
    const child_colnames = child.get_colnames();
    const pairs = [];
    const exprs = [];
    const curr_p_checked = {};
    const curr_child_checked = {};
    let add_fks;
    let tForUpdate;
    let rm_fks;

    function get_fk_changes(old_fk, new_fk) {
      add_fks = [];
      rm_fks = [];
      Object.keys(new_fk).forEach((key) => {
        if (!(key in old_fk)) {
          add_fks.push([key, new_fk[key]]);
        } else if (new_fk[key] !== old_fk[key]) {
          add_fks.push([key, new_fk[key]]);
          delete old_fk[key];
        } else {
          delete old_fk[key];
        }
      });
      Object.keys(old_fk).forEach((key) => {
        if (old_fk[key] !== '') {
          rm_fks.push([key, new_fk[key]]);
        }
      });
    }

    $form.find('ul:not(.sample-list)').each(function getJoinData() {
      const $ul = $(this);
      const $exprInputs = $ul.find('.modal-custom-expr.form-control');
      if ($exprInputs.length) {
        $exprInputs.each(function getExprVal() {
          const $input = $(this);
          if (config.skipElem && $input.is(config.skipElem)) {
            return;
          }

          const expr = $input.val();

          if (expr && expr.trim()) {
            exprs.push(expr.trim());
          }
        });
      }

      const pcol = $ul.find('[arcid=modal-source-input]').val();
      const tcol = $ul.find('[arcid=modal-target-input]').val();

      if ($ul.find('[name="foreign-key-parent"]').is(':checked')) {
        curr_p_checked[pcol] = tcol;
      } else if ($ul.find('[name="foreign-key-child"]').is(':checked')) {
        curr_child_checked[tcol] = pcol;
      }

      if (parent_colnames.includes(pcol) && child_colnames.includes(tcol)) {
        pairs.push([pcol, tcol]);
      }
    });

    if (config.update) {
      tForUpdate = t;
    } else {
      tForUpdate = arc.misc.OBJ.sliceRecursive(t);
      tForUpdate.dstab = t.dstab.clone();
      tForUpdate.src_table.dstab = t.src_table.dstab.clone();
    }

    tForUpdate.join_pairs = pairs;
    tForUpdate.join_exprs = exprs;

    tForUpdate.dstab.join_pairs(pairs);
    tForUpdate.dstab.join_exprs(exprs);

    // populate the foreign/primary keys that need to be either set or removed
    if (config.update) {
      get_fk_changes(config.p_fk, curr_p_checked);
      t.src_table.dstab.foreign_keys(t.name, { add: add_fks, rm: rm_fks });
      get_fk_changes(config.child_fk, curr_child_checked);
      t.dstab.foreign_keys(t.src_table.name, { add: add_fks, rm: rm_fks });
    }

    return tForUpdate;
  };

  ds.setDataButton = setDataButton;

  return ds;
};

const colorHtml = `
  <div id="custom-colors-enabled">
  </div>
  <div id="custom-colors-data" style="display:none;height:400px;overflow:scroll;">
    <div id="column-color-setting">
    </div>
    <div id="column-opacity-setting" style="width: 200px;">
    </div>
    <div id="column-measure-values">
      <div id="measure-ranges-setting">
      </div>
    </div>
    <div id="column-dimension-values" style="display:none;">
      <div id="fixed-colors-setting">
      </div>
      <div class='modal-body-loading arc-invisible'>
        <i class="fa fa-spinner fa-spin"></i>
        <span>Loading</span>
      </div>
      <div id="dim-colors-mapping">
      </div>
    </div>
  </div>
`;

arc.admin.columnColorHandler = function(col, builder) {
  const h = function() { return 'columnColorHandler'; };

  let colColor;
  let dimToColorSetting;
  let dimColorScale;
  let drawId;

  const fixedColors = arc.report.user_settings.fixed_color_legend();
  const rangeColors = arc.colormap.measureRangesSetting();

  function drawMeasureColor() {
    $('#column-dimension-values').hide();
    $('#column-measure-values').show();
    const $m = $('#column-measure-values');

    $m.attr('data-ranges', JSON.stringify({
      ranges: colColor.colorSettings.measureRanges || [],
    }));

    rangeColors.value(JSON.stringify({
                 ranges: colColor.colorSettings.measureRanges || [],
               }));

    rangeColors.paintControl(d3.select('#column-measure-values'));
    rangeColors.addChangeCb('ranges.change', () => {
      $m.attr('data-ranges', rangeColors.value());
    });
  }

  function drawDimColorMappingsWithData(data) {
    // Make sure while we were fetching data, the modal did not get rendered
    // for a different column
    const $color = $('#column-color');
    if ($color.attr('data-version') !== drawId) {
      return;
    }

    const domain = data.rows.map((d) => d[0]);

    if (colColor.dimensionMappings === undefined) {
      colColor.dimensionMappings = {};
    }

    if (!colColor.colors) {
      colColor.colors = arc.report.base_colors();
    }

    const dimToColorSettingValue = {
      isChecked: Object.keys(colColor.dimensionMappings).length > 0,
      mappings: colColor.dimensionMappings,
    };

    dimToColorSetting = arc.colormap.dimToColorMappingSetting()
        .value(JSON.stringify(dimToColorSettingValue));

    dimColorScale = arc.colorpalette.dimScale(domain, colColor.colors, {
      fixedDimensionColors: colColor.fixedDimensionColors,
      dimensionMappings: colColor.dimensionMappings,
    });

    domain.forEach((d) => {
      dimColorScale.setColor(d, dimColorScale(d));
    });

    dimToColorSetting.colorScale(dimColorScale);
    dimToColorSetting.paintControl(d3.select('#dim-colors-mapping'));
    dimToColorSetting.changeCb('column-color', () => {
      const v = JSON.parse(dimToColorSetting.fetch_value());
      dimColorScale.range(colColor.colors);
      domain.forEach((d) => {
        dimColorScale.setColor(d, dimColorScale(d));
      });
      dimToColorSetting.colorScale(dimColorScale);
      if (v.isChecked) {
        colColor.dimensionMappings = v.mappings;
      } else {
        colColor.dimensionMappings = {};
      }

      $('#dim-colors-mapping').attr('data-mappings', JSON.stringify({
        isChecked: (Object.keys(colColor.dimensionMappings).length > 0),
        mappings: colColor.dimensionMappings,
      }));
    });
  }

  function drawDimColorMappings() {
    if (colColor.dimensionMappings) {
      $('#dim-colors-mapping').attr('data-mappings', JSON.stringify({
        isChecked: (Object.keys(colColor.dimensionMappings).length > 0),
        mappings: colColor.dimensionMappings,
      }));
    } else {
      $('#dim-colors-mapping').attr('data-mappings', JSON.stringify({
        isChecked: false,
        mappings: {},
      }));
    }

    const expr = col.expr();
    if (!expr) {
      return;
    }

    if (arc.cviz.isCvizExpr(expr)) {
      return;
    }

    const columnAlias = getServerAliasForSimpleColumn(col);
    const isExpression = !columnAlias;

    const rb = arc.data_requests.distinctColumnValues(
        undefined, col.table().dataset(), isExpression ? expr : columnAlias, 500, col.attr('type'), undefined, isExpression
    );

    // After refreshing the dataset tables, base column information can be diverged
    // from the saved one in the metadata DB(might somebody added a new column).
    // Hence it can not properly decompose the column expression, causing bugs.
    // Adding the buildInfo to the request helps to overcome this issue
    const dsreq = rb.dsreq();
    dsreq.dataset_object = col.table().dataset().buildInfo();

    function successCb(data) {
      drawDimColorMappingsWithData(data);
      $('#edit-column-modal .modal-body-loading').addClass('arc-invisible');
    }

    function errorReportingCb() {
      arc.report.alertMessage(
        "Fixed color mapping currently unavailable",
        `Error fetching values for ${col.display_name()}`,
        { alertContainerSelector: '#edit-column-modal' }
      );
      $('#edit-column-modal .modal-body-loading').addClass('arc-invisible');
    }

    // here we fetch the data
    $('#edit-column-modal .modal-body-loading').removeClass('arc-invisible');

    arc.data.runQueryParams({
      rb,
      successCb,
      errorReportingCb,
    });
  }

  /**
   * If the argument is a simple column, then:
   *  1. if server info is available, return the alias known to the server.
   *     This can differ from the current alias, if the field was edited, but not saved yet.
   *  2. if no server info is available, then return the current alias
   *  3. Return null if the argument is a computed column
   *
   * @returns: null if the argument is not a simple column.
   */
  function getServerAliasForSimpleColumn(column) {
    const columnName = column.attr('name');

    const isComputedColumn = !columnName;
    if (isComputedColumn) {
      return null;
    }

    const tablename = column.table().tablename();
    const columnfqn = `${tablename}.${columnName}`;
    const aliasMap = builder?.get_dataset_server_info()?.column_aliases_by_fqn;
    if (aliasMap) {
      const serverAlias = aliasMap[columnfqn];
      if (serverAlias) {
        return serverAlias;
      }
    }

   // This happens, when the column is cloned from a column that is only available on client side,
   // it is not yet saved to the database.
    return column.display_name();
  }

  function drawDimensionColor() {
    $('#column-measure-values').hide();
    $('#column-dimension-values').show();

    fixedColors.value((colColor.fixedDimensionColors === true) ? 'true' : 'false');
    fixedColors.paintControl(d3.select('#fixed-colors-setting'));

    drawDimColorMappings();
  }

  h.fetch = function(newAttrs) {
    newAttrs.colors = {};
    newAttrs.colors.customColorsEnabled = $('#custom-colors-enabled input').prop('checked');
    if (!newAttrs.colors.customColorsEnabled) {
      return;
    }

    newAttrs.colors.fixedDimensionColors = $('#fixed-colors-setting input').prop('checked');
    const data = $('#dim-colors-mapping').attr('data-mappings');
    let m;
    if (data) {
      m = JSON.parse(data);
    } else {
      m = {};
    }

    if (m.isChecked) {
      newAttrs.colors.dimensionMappings = m.mappings;
    } else {
      newAttrs.colors.dimensionMappings = {};
    }

    newAttrs.colors.opacity = Number($('#column-opacity-setting input').val());

    const colorValue = JSON.parse($('#column-color-setting').attr('data-colors'));
    newAttrs.colors.colors = colorValue.colors;
    newAttrs.colors.isReversed = colorValue.isReversed;
    newAttrs.colors.colorSettings = colorValue.colorSettings || {};
    newAttrs.colors.firstColor = colorValue?.firstColor;

    if (!newAttrs.isdim) {
      const $m = $('#column-measure-values');
      const { ranges } = JSON.parse($m.attr('data-ranges'));
      newAttrs.colors.colorSettings.measureRanges = ranges || [];
    }

    return h;
  };

  h.draw = function() {
    const $color = $('#column-color');
    drawId = `draw-column-${arc.misc.uniqueId()}`;
    $color.attr('data-version', drawId);
    dimToColorSetting = undefined;

    const colorSettingId = 'column-color-setting';
    const opacitySettingId = 'column-opacity-setting';

    colColor = $.extend({}, col.attr('colors'));
    if (colColor === undefined) {
      colColor = {
        customColorsEnabled: false,
        colors: arc.report.graded_colors(),
        isReversed: false,
        colorSettings: {},
        opacity: 80,
        fixedDimensionColors: false,
        dimensionMappings: {},
      };
    }

    if (!colColor.colorSettings) {
      colColor.colorSettings = {};
    }

    $color.html(colorHtml);

    const customColorsEnabled = arc.settings.columns.customColors();
    if (colColor.customColorsEnabled) {
      customColorsEnabled.value('true');
      $('#custom-colors-data').show();
    } else {
      customColorsEnabled.value('false');
      $('#custom-colors-data').hide();
    }

    customColorsEnabled.paintControl(d3.select('#custom-colors-enabled'));
    customColorsEnabled.changeCb('column-color', () => {
      const v = customColorsEnabled.fetch_value();
      if (v === 'true') {
        $('#custom-colors-data').show();
      } else {
        $('#custom-colors-data').hide();
      }
    });

    if (col.isDimension()) {
      drawDimensionColor();
    } else {
      drawMeasureColor();
    }

    const colorStr = JSON.stringify({
      colorSettings: colColor.colorSettings,
      isReversed: colColor.isReversed,
      colors: colColor.colors,
      firstColor: colColor.firstColor,
    });

    function setColors() {
      if (dimColorScale) {
        dimColorScale.range(colColor.colors);
        dimColorScale.domain().forEach((d) => {
          dimColorScale.setColor(d, dimColorScale(d));
        });
        dimToColorSetting.colorScale(dimColorScale);
      }

      if (rangeColors) {
        rangeColors.setColors(colColor.colors);
      }
    }

    $(`#${colorSettingId}`).attr('data-colors', colorStr);
    const colorPalettes = arc.colorpalette.setting();
    colorPalettes.value(colorStr);
    colorPalettes.selectedColors(colColor.colors);
    colorPalettes.setOpacity(colColor.opacity);
    colorPalettes.paintControl(d3.select(`#${colorSettingId}`));
    colorPalettes.changeCb('column-color', () => {
      const v = colorPalettes.fetch_fn()();
      $(`#${colorSettingId}`).attr('data-colors', v);

      const colorValue = JSON.parse(v);
      colColor.colors = colorValue.colors;
      colColor.isReversed = colorValue.isReversed;
      colColor.firstColor = colorValue.firstColor;

      setColors();
    });

    setColors();

    function setOpacity(o) {
      colorPalettes.setOpacity(o);
      if (rangeColors) {
        rangeColors.setOpacity(o);
      }

      $color.find('.single-color')
            .css('opacity', o / 100);
    }

    const colorOpacity = arc.report.user_settings.color_opacity();
    colorOpacity.value(colColor.opacity || 80);
    colorOpacity.paintControl(d3.select(`#${opacitySettingId}`));
    colorOpacity.changeCb('column-color', () => {
      const o = colorOpacity.fetch_value();
      setOpacity(Number(o));
    });

    setOpacity(Number(colColor.opacity));

    arc.visual.settings()
       .handlePopovers($color);
  };

  return h;
};
}());
