window.arc = window.arc || {};

(function() {
'use strict';

arc.admin = arc.admin || {};

const fileTypes = {
  'image/jpeg': 'Image (jpg)',
  'image/png': 'Image (png)',
  'image/gif': 'Image (gif)',
  'image/svg+xml': 'Image (svg)',
  'image/x-icon': 'Image (ico)',
  'text/javascript': 'JS',
  'application/javascript': 'JS',
  'application/x-javascript': 'JS',
  'application/vnd.geo+json': 'GEOJSON',
  'text/x-json': 'JSON',
  'application/json': 'JSON',
  'text/css': 'CSS',
  'application/pdf': 'PDF',
  'application/msword': 'Document (doc)',
  'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 'Document (docx)',
  'application/vnd.ms-excel': 'Spreadsheet (xls)',
  'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': 'Spreadsheet (xlsx)',
  'font/otf': 'Font (otf)',
  'font/sfnt': 'Font (sfnt)',
  'font/ttf': 'Font (ttf)',
  'font/woff': 'Font (woff)',
  'font/woff2': 'Font (woff2)',
  'video/avi': 'Video (avi)',
  'video/quicktime': 'Video (mov)',
  'video/mpeg': 'Video (mp3)',
};

const fileExtensions = {
  'image/jpeg': 'jpg',
  'image/png': 'png',
  'image/gif': 'gif',
  'image/svg+xml': 'svg',
  'image/x-icon': 'ico',
  'text/javascript': 'js',
  'application/javascript': 'js',
  'application/x-javascript': 'js',
  'application/vnd.geo+json': 'geojson',
  'text/x-json': 'json',
  'application/json': 'json',
  'text/css': 'css',
  'application/pdf': 'pdf',
  'application/msword': 'doc',
  'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 'docx',
  'application/vnd.ms-excel': 'xls',
  'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': 'xlsx',
  'font/otf': 'otf',
  'font/sfnt': 'sfnt',
  'font/ttf': 'ttf',
  'font/woff': 'woff',
  'font/woff2': 'woff2',
  'video/avi': 'avi',
  'video/quicktime': 'mov',
  'video/mpeg': 'mp3',
};

function assetUrl(assetId, assetMimetype) {
  return `${arc.frontend.globals.urls.staticassetsRawUrl}/${assetId}.${fileExtensions[assetMimetype]}`;
}

arc.admin.StaticAsset = function() {
  const StaticAsset = function() { return 'arc.admin.StaticAsset'; };

  var assetId,
      name,
      description,
      mimetype,
      fileSize,
      contents;

  StaticAsset.url = function() {
    return assetUrl(assetId, mimetype);
  };

  StaticAsset.fileSize = function(_) {
    if (!arguments.length) return fileSize;
    fileSize = _;
    return StaticAsset;
  };

  StaticAsset.assetId = function(_) {
    if (!arguments.length) return assetId;
    assetId = _;
    return StaticAsset;
  };

  StaticAsset.assetName = function(_) {
    if (!arguments.length) return name;
    name = _;
    return StaticAsset;
  };

  StaticAsset.assetDescription = function(_) {
    if (!arguments.length) return description;
    description = _;
    return StaticAsset;
  };

  StaticAsset.assetMimetype = function(_) {
    if (!arguments.length) return mimetype;
    mimetype = _;
    return StaticAsset;
  };

  StaticAsset.assetContents = function(_) {
    if (!arguments.length) return contents;
    contents = _;
    return StaticAsset;
  };

  StaticAsset.delete = function(done_cb) {
    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };

    $.ajax({
      url: `${arc.frontend.globals.urls.staticassetsDataUrl}/${assetId}`,
      data: post_data,
      type: 'DELETE',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail((req) => {
      arc.report.alertMessage('Failed to delete static asset',
          req.responseText ? req.responseText : req, { is_html: true });
    })
    .always((req, textstatus, errorthrown) => {
      if (done_cb) {
        done_cb(req, textstatus, errorthrown);
      }
    });

    return StaticAsset;
  };

  StaticAsset.save = function(done_cb) {
    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      name: name,
      description: description,
      mimetype: mimetype,
    };

    if (contents) {
      if (fileSize > 104857600) {
        arc.report.alertMessage('Max upload size 100MB');
        return StaticAsset;
      }

      post_data.contents = contents;
    }

    let url = arc.frontend.globals.urls.staticassetsDataUrl;
    if (assetId) {
      url += '/' + assetId;
    }

    // save event
    $.ajax({
      url: url,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(function(data) {
      assetId = data[0];
    })
    .fail((req) => {
      arc.report.alertMessage('Failed to ' +
          (assetId ? 'update' : 'create') + ' static asset: ',
          req.responseText ? req.responseText : req);
    })
    .always((req, textstatus, errorthrown) => {
      if (done_cb) {
        done_cb(req, textstatus, errorthrown);
      }
    });

    return StaticAsset;
  };

  return StaticAsset;
};

arc.admin.staticasset = function(_assetId) {
  var sa = function() { return 'arc.admin.staticasset'; };

  var assetId = _assetId,
      files,
      fileContents;
  var $mimetypeField = $('#asset-mimetype');

  function setMimeType(newType) {
    $mimetypeField.attr('data-mimetype', '');
    $mimetypeField.text('');
    if (newType in fileTypes) {
      $mimetypeField.attr('data-mimetype', newType);
      $mimetypeField.text(fileTypes[newType]);
    }
  }

  // TODO: Fix file upload async issue (ARC-7711)
  // This code will cause an issue for large files because it loads the file contents
  // into staticasset.fileContents on upload and assumes that this is complete on save.
  // If the user saves before read is complete the file contents will be undefined and the
  // upload will fail.  Should read file contents on save instead and send api call on
  // complete
  function handleFiles() {
    const mimeTypes = arc.misc.OBJ.invert(fileExtensions);
    $('#asset-file').on('change', function(event) {
      arc.report.pagenav.needs_save(true);
      fileContents = undefined;
      files = event.target.files;
      if (files.length === 1) {
        var f = files[0];
        var extension = f.name.split('.').pop();
        var mimeType = f.type || mimeTypes[extension];
        $('#asset-size').text(f.size);
        setMimeType(mimeType);
        var fileData = new FileReader();
        fileData.readAsDataURL(f);
        fileData.onloadend = () => {
          fileContents = fileData.result.replace(/^data:.*base64,/, '');
        };
      }
    });
  }

  sa.initialize = function() {
    var validTypes = [];
    var $mimetype = $mimetypeField;
    $.each(fileTypes, function(k, v) {
      validTypes.push(k);
    });

    $('#asset-file').attr('accept', validTypes.join(','));

    $('.mimetypes a').offOn('click.mimeTypes', function() {
      var $this = $(this);
      $mimetype.attr('data-mimetype', $this.attr('data-mimetype').split(' ')[0]);
      $mimetype.text($this.text());
    });

    handleFiles();
    if (_assetId === 0) {
      setupEdit();
      $('#style-name')[0].focus();

      return sa;
    }

    fetchAndPaint(function() {
      setupEdit();
    });

    return sa;
  };

  function setupEdit() {
    var assetId = _assetId;
    $('#style-name').on('input.nameOnChange', function() {
      if (assetId !== 0) {
        arc.report.pagenav.needs_save(true);
      }
    });

    $('#style-description').on('input.descriptionOnChange', function() {
      if (assetId !== 0) {
        arc.report.pagenav.needs_save(true);
      }
    });

    $('#custom-style-form').on('submit', function() {
      $('.save-btn').prop('disabled', true);
      const asset = arc.admin.StaticAsset()
                             .assetName($('#style-name').val())
                             .assetDescription($('#style-description').val());

      if (files) {
        asset.fileSize(files[0].size);
      }

      if (fileContents) {
        asset.assetMimetype($('#asset-mimetype').attr('data-mimetype'))
             .assetContents(fileContents);
      }

      if (assetId) {
        asset.assetId(assetId);
      }

      asset.save((_req, textstatus) => {
        $('.save-btn').prop('disabled', false);
        arc.report.pagenav.needs_save(false);
        if (!assetId) {
          assetId = asset.assetId();
          window.open(asset.assetId(), '_self');
        } else if (textstatus === 'success') {
          arc.report.alertMessageSmall('Save complete', 'alert-success');

          // Update name if it's changed
          var $title = $('#title-style-name');
          if ($title.text() !== asset.assetName()) {
            $title.text(asset.assetName());
          }
        }
      });
    });
  }

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.staticassetsDataUrl}/${assetId}`)
      .await(function(error, staticAssets) {
        if (error) {
          arc.report.alertMessage('Could not load static assets',
              error);
          return;
        }

        var staticAsset = staticAssets[0];
        const url = assetUrl(assetId, staticAsset.mimetype);
        $('.breadcrumb .active').text(assetId);
        $('#style-name').val(staticAsset.name);
        $('#title-style-name').text(staticAsset.name);
        $('#style-description').val(staticAsset.description);
        setMimeType(staticAsset.mimetype);
        $('#asset-size').text(staticAsset.size);

        const linkHtml = `${location.protocol}//${location.host + url}`;
        $('#asset-url').html(`<a href="${url}" target="_blank">${linkHtml}</a>`);
        if (cb) {
          cb();
        }
      });
  }

  return sa;
};

arc.admin.staticassets = function() {
  var sa = function() { return 'arc.admin.staticassets'; };

  var assetNames = {};

  sa.initialize = function() {
    fetchAndPaint();
    return sa;
  };

  function fetchAndPaint(cb) {
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.staticassetsDataUrl)
      .await(function(error, assets) {
        if (error) {
          arc.report.alertMessage('Could not load static assets',
              error);
          return;
        }

        paintTable(assets.reverse());
        if (cb) {
          cb();
        }
      });
  }

  function paintTable(assets) {
    var $tbody = $('#table-staticassets tbody');
    $tbody.empty();

    assetNames = {};

    var templateIn = $('#table-staticassets-template').html();
    assets.forEach(function(c) {
      assetNames[c.id] = c.name;
      $tbody.append(arc.misc.safe_substitute(
          templateIn,
          {
            ID: c.id,
            TYPE: fileExtensions[c.mimetype],
            NAME: c.name,
            DESCRIPTION: c.description,
            SIZE: c.size,
            HTML_URL: `<a href="${assetUrl(c.id, c.mimetype)}" target="_blank">${c.id}</a>`,
            OWNER: c.update_user,
          }));
    });

    $tbody.find('a.delete')
          .off('click')
          .on('click', function() {
            var assetId = Number($(this).closest('tr').attr('data-styleid'));
            var c = arc.misc.deleteConfirmation();
            c.objectType('Static Asset')
             .objectName(assetNames[assetId])
             .askUser(function() {
               arc.admin.StaticAsset()
                        .assetId(assetId)
                        .delete(function() {
                          fetchAndPaint(function() {
                            c.hideModal();
                          });
                        });
             });
          });
  }

  return sa;
};

}());
