window.arc = window.arc || {};

(function() {
'use strict';

arc.admin = arc.admin || {};

arc.admin.timeSettingsHandler = function() {
  let t = function() { return 'arc.admin.timeSettingsHandler'; };

  let datasetAdmin,
      builder,
      $edit,
      $undo,
      $save,
      $timecol,
      $timeInterval,
      $timeColDateSection,
      $timecol_format,
      $timecol_agg,
      $timeselection_pos,
      $timeselection_range,
      $allElements,
      refreshFrequenciesTable,
      refreshIncrementsTable;

  t.datasetAdmin = function(_) {
    if (!arguments.length) return datasetAdmin;
    datasetAdmin = _;
    return t;
  };

  t.builder = function(_) {
    if (!arguments.length) return builder;
    builder = _;
    return t;
  };

  function addDefaultDateTime(nameUnquoted, coltypeClass) {
    let $appendArea = $timecol;
    if (coltypeClass === 'type-timestamp') {
      $appendArea = $timeColDateSection;
      $timeColDateSection.removeClass('arc-invisible');
    }

    $appendArea.append(arc.misc.safe_substitute('<option>%DIM%</option>',
        {
          DIM: nameUnquoted,
        }));
  }

  function addDefaultDateTimeFormat(format) {
    $timecol_format.append(arc.misc.safe_substitute('<option>%FORMAT%</option>',
        {
          FORMAT: format,
        }));
  }

  function addDefaultAgg(nameUnquoted) {
    $timecol_agg.append(arc.misc.safe_substitute('<option>%AGG%</option>',
        {
          AGG: nameUnquoted,
        }));
  }

  function addIntervalOptions(interval) {
    $timeInterval.append(arc.misc.safe_substitute('<option>%INTERVAL%</option>',
        {
          INTERVAL: interval,
        }));
  }

  function addSearchColumn(column) {
    let attr = column.attr(),
        nameUnquoted = column.display_name();

    if (column.isDimension()) {
      addDefaultDateTime(nameUnquoted, column.coltype_class());
    } else {
      addDefaultAgg(nameUnquoted);
    }
  }

  function makeEditableUneditable(isEditable) {
    $edit.toggleClass('disabled', isEditable);
    $save.toggleClass('btn-primary', isEditable);

    const isDisabled = !isEditable;
    $undo.toggleClass('disabled', isDisabled);
    $save.toggleClass('disabled', isDisabled);
    $timecol.prop('disabled', isDisabled);
    $timecol_format.prop('disabled', isDisabled);
    $timeInterval.prop('disabled', isDisabled);
    $timecol_agg.prop('disabled', isDisabled);
    $timeselection_pos.prop('disabled', isDisabled);
    $timeselection_range.prop('disabled', isDisabled);

    refreshIncrementsTable.makeEditable(isEditable);
    refreshFrequenciesTable.makeEditable(isEditable);
  }

  function makeEditable() {
    makeEditableUneditable(true);
  }

  function makeUneditable() {
    makeEditableUneditable(false);
  }

  function undoChanges() {
    makeUneditable();
    arc.report.pagenav.needs_save(false);
    t.setupTimeSettings();
  }

  function saveChanges() {
    let timeSettings = {
      defaultDateTime: $timecol.val(),
      defaultDateTimeFormat: $timecol_format.val(),
      defaultTimeInterval: $timeInterval.val(),
      defaultAgg: $timecol_agg.val(),
      frequencies: refreshFrequenciesTable.getRows(),
      increments: refreshIncrementsTable.getRows(),
      selectionPos: $timeselection_pos.val(),
      selectionRange: $timeselection_range.val(),
    };

    let dataset = builder.dataset();
    dataset.timeSettings(timeSettings);

    dataset.save(function() {
             datasetAdmin.builderReload(function() {
               makeUneditable();
               datasetAdmin.datasetRefreshed();
             });
           });
  }

  t.setupTimeSettings = function() {
    const $timeModel = $('#timemodeling');

    $edit = $timeModel.find('.edit-time');
    $undo = $timeModel.find('.undo-time');
    $save = $timeModel.find('.save-time');

    $edit.offOn('click.allowEdit', makeEditable);

    $undo.offOn('click.undo', undoChanges);

    $save.offOn('click.save', saveChanges);

    $timecol = $timeModel.find('.default-timecol');
    $timecol_format = $timeModel.find('.default-timecol-format');
    $timeInterval = $timeModel.find('.default-time-interval');
    $timecol_agg = $timeModel.find('.default-timeagg');
    $timeselection_pos = $timeModel.find('.timeselection-pos');
    $timeselection_range = $timeModel.find('.timeselection-range');

    $allElements = $timecol
                      .add($timecol_format)
                      .add($timeInterval)
                      .add($timecol_agg)
                      .add($timeselection_pos)
                      .add($timeselection_range);

    $allElements.offOn('change.markNeedsSave', () => arc.report.pagenav.needs_save(true));
    $timecol.empty();
    $timecol_agg.empty();
    addDefaultDateTime('');
    $timeColDateSection = $('<optgroup label="Date Fields" class="arc-invisible">');
    $timecol.append($timeColDateSection);

    $timecol_format.empty();
    addDefaultDateTimeFormat('YYYY-MM-DD');
    addDefaultDateTimeFormat('YYYY-MM-DD HH:mm:ss');
    addDefaultDateTimeFormat('YYYY-MM-DDTHH:mm:ss[Z]');

    $timeInterval.empty();
    let dataset = builder.dataset();
    const conn = arc.dataconnection.types[dataset.dataconnection_type()]();
    const supportWeekYear = conn.expr_yearweek;
    const supportsHour = conn.expr_fullhour;
    const intervals = [
      '',
    ];

    if (supportsHour) {
      intervals.push('Hour');
    }

    intervals.push('Date');

    if (supportWeekYear) {
      intervals.push('Week');
    }

    [
      ...intervals,
      'Month',
      'Quarter',
      'Year',
    ].forEach(addIntervalOptions);

    dataset.columnIterator(function(column) {
      addSearchColumn(column);
    });

    let timeSettings = dataset.timeSettings();
    if (timeSettings.defaultDateTime) {
      $timecol.val(timeSettings.defaultDateTime);
    }

    if (timeSettings.defaultTimeInterval) {
      $timeInterval.val(timeSettings.defaultTimeInterval);
    }

    if (timeSettings.defaultDateTimeFormat) {
      $timecol_format.val(timeSettings.defaultDateTimeFormat);
    }

    if (timeSettings.defaultAgg) {
      $timecol_agg.val(timeSettings.defaultAgg);
    }

    if (timeSettings.selectionPos) {
      $timeselection_pos.val(timeSettings.selectionPos);
    }

    if (timeSettings.selectionRange) {
      $timeselection_range.val(timeSettings.selectionRange);
    }

    setupFrequencies(timeSettings.frequencies);
    setupIncrements(timeSettings.increments);
  };

  function setupFrequencies(frequencies) {
    if ((frequencies === undefined) || (frequencies.length === 0)) {
      frequencies = [
        [
          '1 sec',
          '1 second',
        ],
        [
          '5 sec',
          '5 seconds',
        ],
        [
          '10 sec',
          '10 seconds',
        ],
      ];
    }

    const $table = $('#time-frequencies');
    refreshFrequenciesTable = arc.utils.modifiableRowsTable({
      $table,
      initialData: frequencies,
      needsSaveCb: () => arc.report.pagenav.needs_save(true),
    });

    refreshFrequenciesTable.setup();
    refreshFrequenciesTable.makeEditable(false);
  }

  function setupIncrements(increments) {
    if ((increments === undefined) || (increments.length === 0)) {
      increments = [
        [
          '1 month',
          '1 month',
        ],
        [
          '1 day',
          '1 day',
        ],
        [
          '1 hour',
          '1 hour',
        ],
        [
          '1 minute',
          '1 minute',
        ],
        [
          '1 second',
          '1 second',
        ],
      ];
    }

    const $table = $('#time-increments');
    refreshIncrementsTable = arc.utils.modifiableRowsTable({
      $table,
      initialData: increments,
      needsSaveCb: () => arc.report.pagenav.needs_save(true),
    });

    refreshIncrementsTable.setup();
    refreshIncrementsTable.makeEditable(false);
  }

  return t;
};

}());
