window.arc = window.arc || {};

(function() {
arc.useradmin = {};

let group_userSelectionWidgets;
let user_groupSelectionWidgets;
let $usersDataTable;
let $groupsDataTable;
let group_roleSelectionWidgets;
let user_roleSelectionWidgets;
let bulkUsers;
let bulkGroups;
let globalusers = Object.create(null);
let globalgroups = Object.create(null);
let globalroles = Object.create(null);
let globalFetchDetail = false;
let paginatedUsers;
let summarizedUsers;
let totalUsers;
let paginatedGroups;
let summarizedGroups;
let totalGroups;

// Only allow alphanumeric characters, underscores, periods, and dashes
// in usernames
const validUsernameCharSet = /^[0-9a-zA-Z_.-]+$/i;

// Custom validators and error messages for validator
// Cannot have duplicate properties with custom group validator
// TODO: change for i18n
const userValidatorChecks = {
  custom: {
    noduplicateusers($el) {
      if (globalusers[$el.val()] && ($el.data('old-username') !== $el.val())) {
        return 'Already exists';
      }
    },

    validchars($el) {
      // If the username is being changed, force the username to only have chars
      // from the valid character set
      const newUsername = $el.val();
      return $el.data('old-username') !== newUsername &&
             validUsernameCharSet.exec(newUsername) === null &&
             'Only alphanumeric characters, periods, dashes, and underscores allowed';
    },

    nospecial($el) {
      return $el.val().match(/[@'";><(){}]/g) && 'No special characters';
    },

    // difference between this and nospecial (above): allows apostrophes
    nospecial_usernames($el) {
      return $el.val().match(/[@";><(){}]/g) && 'No special characters';
    },
  },
};

// Entrypoint from arc_admin_users.js
const $refreshButton = $('.arc-refresh-button');
const $refreshIcon = $('#refresh-icon');
const $refreshingIcon = $('#refresh-spin');
const $createUserForms = $('#new-user-form');
const $editUserForms = $('#edit-user-form');

arc.useradmin.init = function (detail) {
  // make sure icons are disabled immediately on page load
  dataDownloadStart();
  globalFetchDetail = Boolean(detail);
  userValidatorChecks.custom = $.extend(
    userValidatorChecks.custom,
    arc.authentication.passwordValidators.custom
  );

  $('#users-table [data-toggle="tooltip"]').tooltip({
    placement: 'top',
    delay: {
      show: 500,
      hide: 100,
    },
  });

  // if user checks the pass credentials box in the new connection modal,
  // then disable credentials fields
  const $showPasswordButton = $('#change-password-btn');

  $showPasswordButton.offOn('click.showPassword', () => {
    const isHidden = $('#password-edit').data('skip');
    _togglePwSection(isHidden);
  });

  $('#new-user-modal')
    .on('show.bs.modal', (event) => {
      _resetModal(event);
      arc.frontend.legacyBridge.events.next({ type: 'password-novel-reset-visibility' });
    });

  $('#new-group-modal')
    .on('show.bs.modal', _resetModal);

  $('#new-user-modal')
    .on('hide.bs.modal', _resetModal);

  $('#new-group-modal')
    .on('hide.bs.modal', _resetModal);

  $('#edit-user-modal')
    .on('hide.bs.modal', _resetModal);

  $('#edit-group-modal')
    .on('hide.bs.modal', _resetModal);

  $refreshButton.on('click.refreshTabl', _redrawPaginatedUsersAndGroups);

  $('#populate-users').offOn('click.populateUser', () => {
    let index = -1;
    let $tr;
    let data;
    let user;
    let username;

    _updatePopulateButton(false, true);

    const nextUser = function() {
      do {
        index += 1;
        data = $usersDataTable.row(index);
        if (data.length === 0) {
          const notdone = Object.values(globalusers).some((u) => u.groups === undefined);
          _updatePopulateButton(!notdone, false);
          return;
        }

        $tr = $(data.node());
        username = $tr.data().username;
        user = globalusers[username];
      } while (user.groups !== undefined);

      _updateUserRow(user, $tr, true);
      d4.queue()
        .defer(d3.json, `${arc.frontend.globals.urls.usersDetailedApiUrl}/${username}`)
        .await((error, users) => {
          if (error) {
            _showFetchError(user, error);
            _updatePopulateButton(false, false);
            _updateUserRow(user, $tr);
          } else {
            user = users[0];
            _updateGlobalUser(user);
            _updateUserRow(user, $tr);
            nextUser();
          }
        });
    };

    nextUser();
  });

  const rolesPromise = arc.frontend.vizHttp.get({ url: arc.frontend.globals.urls.rbacRolesUrl });

  const usersPromise = _drawPaginatedUsers(rolesPromise);
  const groupsPromise = _drawPaginatedGroups(rolesPromise);
  Promise.allSettled([usersPromise, groupsPromise]).then(dataDownloadEnd);

  _setupValidators();
};

// Get groups/roles table text for a user.  Data / Button (to fetch) / Spinner (when fetching)
function _userGroupsRolesText(user, fetching) {
  const rv = { groupsText: '', rolesText: '' };
  if (fetching) {
    rv.groupsText = '<i class="fa fa-spinner fa-spin pull-right"></i>';
    rv.rolesText = '';
  } else if (user.groups === undefined) {
    rv.groupsText = '<button class="btn btn-secondary viz-btn-xs pull-right rg_populate">Fetch</button>';
    rv.rolesText = '';
  } else {
    rv.groupsText = user.groups.map((group) => group.name).join(', ');
    rv.rolesText = user.roles.map((role) => role.name).join(', ');
  }

  return rv;
}

function _updateGlobalUser(user) {
  globalusers[user.username] = user;
  globalusers[user.username].lastlogin_formatted = arc.misc.fromNow(user.last_login, 'Never');
  globalusers[user.username].datejoined_formatted = arc.misc.fromNow(user.date_joined);
}

function _updateGlobalGroup(group) {
  globalgroups[group.name] = group;
}

// Update the table text for a user.
function _updateUserRow(user, $tr, fetching) {
  const rv = _userGroupsRolesText(user, fetching);
  $usersDataTable.cell($tr, 5).data(rv.groupsText);
  $usersDataTable.cell($tr, 6).data(rv.rolesText);
}

function _resetModal(evt) {
  const name = evt.currentTarget.id;
  const $modal = $(evt.currentTarget);

  switch (name) {
    case 'new-user-modal':
      // refresh works only for not dataTables fetched list (roles)
      $('#new-user-modal .modal-body .tab-content .first-tab-selection .check-all-wrapper').show();
      user_groupSelectionWidgets.selectionField('selectionList', summarizedGroups);

      user_roleSelectionWidgets.selectionField('refresh');
      $modal.find('input:radio[name=user-perm]').val(['normal']);
      $modal.find('.user-profile-section')
        .toggleClass('arc-invisible', !arc.globals.arcviz_settings.PROFILE_PROXY_USERNAME);
      break;
    case 'new-group-modal':
      // refresh works only for not dataTables fetched list (roles)
      $('#new-group-modal .modal-body .tab-content .first-tab-selection .check-all-wrapper').show();
      group_userSelectionWidgets.selectionField('selectionList', summarizedUsers);
      group_roleSelectionWidgets.selectionField('refresh');
      break;
    case 'edit-user-modal':
      $('#edit-user-modal .modal-body .tab-content .first-tab-selection .check-all-wrapper').show();
      user_groupSelectionWidgets.selectionField('selectionList', summarizedGroups);

      user_roleSelectionWidgets.selectionField('refresh');
      $modal.find('input:radio[name=user-perm]').val(['normal']);
      $modal.find('.user-profile-section')
        .toggleClass('arc-invisible', !arc.globals.arcviz_settings.PROFILE_PROXY_USERNAME);
      break;
    case 'edit-group-modal':
      // refresh works only for not dataTables fetched list (roles)
      $('#edit-group-modal .modal-body .tab-content .first-tab-selection .check-all-wrapper').show();
      group_userSelectionWidgets.selectionField('selectionList', summarizedUsers);
      group_roleSelectionWidgets.selectionField('refresh');
      break;
    default:
      break;
  }

  _clearFormErrors($modal);
  $('#new-user-modal .nav-link:first').click(); // set the first link element to be active
  $modal.find('[data-action=save]').addClass('disabled');
  $modal.find('.form-control').each((idx, ele) => {
    if ($(ele).is('input:text') || $(ele).is('input:password')) {
      $(ele).val('');
    } else if ($(ele).is('input:checkbox')) {
      $(ele).prop('checked', false);
    }
  });
}

function _clearFormErrors($modal) {
  $modal.find('form').data('bs.validator').reset();
}

function _togglePwSection(isHidden) {
  const $passwordGroup = $('#password-edit-field');
  const $passwordInput = $('#password-edit');

  if (isHidden) {
    arc.frontend.legacyBridge.events.next({ type: 'password-edit-change-validate', payload: { validate: true } });
    $passwordInput.data('skip', false);
    $passwordGroup.show();
  } else {
    arc.frontend.legacyBridge.events.next({ type: 'password-edit-change-validate', payload: { validate: false } });
    $passwordInput.data('skip', true);
    $passwordGroup.hide();
    $passwordInput.val('');
  }

  // we may want to save the password instead of just hiding it
  $('#change-password-btn').html(isHidden ? 'Hide password' : 'Change password');
}

function _saveUser(modal_div, is_new, old_username) {
  const $modal_div = $(modal_div);
  const username = $modal_div.find('.user-username').val();
  const first_name = $modal_div.find('.user-fname').val();
  const last_name = $modal_div.find('.user-lname').val();

  // Password value in #password-novel and #password-edit
  const password = $modal_div.find(`#password-${is_new ? 'novel' : 'edit'}`).val();
  const usertype = $modal_div.find('input:radio[name=user-perm]:checked').val();
  const is_superuser = (usertype === 'admin');
  const profile = {};

  // In the 'new-user-modal', there is no radio button indicating active
  // In that case, we want to make the user active. So, default to active
  // in the code below
  const useractive = $modal_div.find('input:radio[name=user-active]:checked').val();
  const is_active = useractive !== 'inactive';

  const u = arc.auth.User()
                  .username(username)
                  .first_name(first_name)
                  .last_name(last_name)
                  .is_superuser(is_superuser)
                  .is_active(is_active);

  if (password) {
    u.password(password);
  }

  if (!u.is_valid()) {
    arc.report.alertMessage('Invalid password specified');
    $('#edit-user-modal').modal('hide');
    return;
  }

  profile.proxy_username = $modal_div.find('.user-proxy').val();
  const $firstTab = $modal_div.find('.first-tab-selection');
  const $secondTab = $modal_div.find('.second-tab-selection');

  const userData = {
    is_new,
    save_password: password,
    url_user: is_new ? username : old_username,
    profile,
    groups: user_groupSelectionWidgets.selectionField('selected', $firstTab),
    roles: user_roleSelectionWidgets.selectionField('selected', $secondTab),
  };

  const _closeModal = () => $modal_div.modal('hide');

  u.save(userData, _closeModal, _redrawPaginatedUsersAndGroups);
}

function _saveGroup(modal_div) {
  const $modal_div = $(modal_div);
  const $groupName = $modal_div.find('.group-name');
  const groupName = $groupName.val();
  const oldGroupName = $groupName.data('old-groupname');
  const groupId = oldGroupName ? globalgroups[oldGroupName].id : undefined;

  const $userSel = $modal_div.find('.first-tab-selection');
  const users = group_userSelectionWidgets.selectionField('selected', $userSel);
  const $roleSel = $modal_div.find('.second-tab-selection');
  const roles = group_roleSelectionWidgets.selectionField('selected', $roleSel);

  const $saveButton = $modal_div.find('#save-new-group-btn');

  $saveButton.prop('disabled', true);

  const _closeModal = () => $modal_div.modal('hide');

  arc.auth.Group()
    .users(users)
    .roles(roles)
    .groupId(groupId)
    .groupName(groupName)
    .save(
      _closeModal,
      function () {
        $saveButton.prop('disabled', false);
        _redrawPaginatedUsersAndGroups();
      }
    );
}

// Custom validators and error messages for validator
// Cannot have duplicate properties with custom user validator
const groupValidatorChecks = {
  custom: {
    noduplicategroup($el) {
      const groupname = $el.val();
      if (globalgroups[groupname] && ($el.data('old-groupname') !== groupname)) {
        return 'Already exists';
      }
    },
  },
};

function _setupValidators() {
  // validate new group form in modal
  $('#new-group-form, #edit-group-form').validator(groupValidatorChecks).on('submit', function (e) {
    if (!e.isDefaultPrevented()) {
      e.preventDefault();
      e.stopPropagation();
      const modal = this.parentNode.parentNode;
      _saveGroup(modal);
    }
  });

  // validate new user/edit user form in modal
  const onSubmit = function (e) {
    if (!e.isDefaultPrevented()) {
      e.preventDefault();
      e.stopPropagation();
      const isNew = this.id === 'new-user-form';
      const oldUsername = $(this).find('.user-username').data('old-username');
      _saveUser(`#${isNew ? 'new' : 'edit'}-user-modal`, isNew, oldUsername);
    }
  };

  $createUserForms.validator(userValidatorChecks).on('submit', onSubmit);
  $editUserForms.validator(userValidatorChecks).on('submit', onSubmit);

  // HACK: HTML runs the form validation any button click on any form if any validation set.
  // To avoid unnecessary validation - we have to overwrite the dismiss behaviour :(
  const avoidValidationSelector = '.avoid-validation-on-dismiss';
  $(`${avoidValidationSelector}[data-dismiss="modal"]`).on(
    'mousedown',
    (e) => {
      e.preventDefault();
      const $form = $(e.target).closest('form');
      const $modal = $form.closest(`modal${avoidValidationSelector}`);
      _resetModal($modal);
      $modal.modal('hide');
    }
  );
}

const _displayname =
  ({ username, full_name }) => (full_name ? `${username} (${full_name})` : username);

// Update the populate users button
function _updatePopulateButton(alldone, updating) {
  $('#populate-users-spinner').toggleClass('arc-invisible', !updating);
  $('#populate-users').toggleClass('arc-invisible', alldone);
  $('#populate-users').attr('disabled', updating);
}

// Display error from fetching user data
function _showFetchError(user, error) {
  const msg = error.responseText || error.toString();
  arc.report.alertMessageSmall(`Error getting data for ${user.username}: ${msg}`, 'alert-danger');
}

function dataDownloadStart() {
  $refreshButton.prop('disabled', true);
  $refreshIcon.hide();
  $refreshingIcon.show();
  $('[href="#new-user-modal"]').addClass('disabled');
  $('[href="#new-group-modal"]').addClass('disabled');
}

function dataDownloadEnd() {
  $refreshButton.prop('disabled', false);
  $refreshIcon.show();
  $refreshingIcon.hide();
  $('[href="#new-user-modal"]').removeClass('disabled');
  $('[href="#new-group-modal"]').removeClass('disabled');
}

function promiseWithResolver() {
  let resolve;
  let reject;
  const promise = new Promise((res, rej) => {
    resolve = res;
    reject = rej;
  });
  return {
    promise, resolve, reject
  };
}

let $table = $('#users-table');
function _drawPaginatedUsers(rolesPromise) {
  if (!$table.length) {
    $table = $('#users-table');
  }

  const usersPromise = promiseWithResolver();

  $usersDataTable = $table.DataTable({
    paging: true,
    deferRender: true,
    destroy: true,
    searching: true,
    order: [[1, 'asc']],
    processing: true,
    serverSide: true,
    ajax(params, callback) {
      rolesPromise.then(() => {
        arc.frontend.vizHttp.get({
          url: arc.frontend.globals.urls.usersDetailedApiUrl,
          params
        })
        .then(({ data }) => {
          callback(
              {
                // recordsTotal and recordsFiltered
                // needs for the info text
                recordsTotal: data.numberOfUsers,
                recordsFiltered: data.numberOfFiltered,
                // order is important
                // needs to mirror how the columns
                // displayed in UI
                data: data.users.map((user) => [
                  [],
                  user.username,
                  user.is_superuser,
                  user.date_joined,
                  user.last_login,
                  user.groups,
                  user.roles,
                  user.is_active,
                  user.id,
                  user.first_name,
                  user.full_name,
                  user.is_password_editable,
                  user.last_name,
                  user.profile,
                ]),
              },
            );
            data.users.forEach((user) => { user.displayname = _displayname(user); });
            paginatedUsers = data.users;
            totalUsers = data.numberOfUsers;
        });
      }).then(() => {
        arc.frontend.vizHttp.get({
          url: arc.frontend.globals.urls.usersBasicApiUrl
        }).then((response) => {
            summarizedUsers = response.data;
            summarizedUsers.forEach((summarizedUser) => {
              summarizedUser.displayname = _displayname(summarizedUser);
            });
        });
      });
    },
    columns: [
      // order needs to mirror the UI
      // className - the td class
      // users the json obj we receive from the backend
      {
        className: 'user-table-ckb',
        users: '',
        orderable: false,
        render(data, _, row) {
          return `<input type="checkbox" value="${row[1]}">`;
        }
      },
      {
        className: 'user-table-name',
        users: 'username',
        name: 'username',
        render(data, _, row) {
          const fullname = row[10] ? ` (${row[10]})` : '';
          return `<span class="cell-wrapper">${data}${fullname}</span>`;
        }
      },
      {
        className: 'user-table-type',
        users: 'is_superuser',
        name: 'is_superuser',
        render(data) {
          const usertype = data ? 'Admin' : 'Normal';

          return `<span class="cell-wrapper"> ${usertype}</span>`;
        }
      },
      {
        className: 'user-table-datejoined',
        users: 'date_joined',
        name: 'date_joined',
        render(data) {
          const formattedJoinedDate = arc.misc.fromNow(data, 'Never');

          return `<span class="cell-wrapper">
                    <span class="orig-date">${data}</span>
                    <span class="formatted-date">${formattedJoinedDate}</span>
                  </span>`;
        }
      },
      {
        className: 'user-table-lastlogin',
        users: 'last_login',
        name: 'last_login',
        render(data) {
          const formattedLoginDate = arc.misc.fromNow(data, 'Never');

          return `<span class="cell-wrapper">
                    <span class="orig-date">${data}</span>
                    <span class="formatted-date">${formattedLoginDate}</span>
                  </span>`;
        }
      },
      {
        className: 'user-table-groups',
        users: 'groups',
        name: 'groups',
        render(data) {
          const groups = data.map((group) => arc.misc.escapeHTML(group.name)).join(', ');

          return groups;
        }
      },
      {
        className: 'user-table-roles',
        users: 'roles',
        name: 'roles',
        render(data) {
          const roles = data.map((role) => role.name).join(', ');

          return roles;
        }
      },
      {
        className: 'user-table-status',
        users: 'is_active',
        name: 'is_active',
        render(data) {
          const act = '<span class="badge badge-success badge-pill" data-toggle="tooltip" title="active">A</span>';

          const inact = '<span class="badge badge-warning badge-pill" data-toggle="tooltip" title="inactive">I</span>';

          return data ? act : inact;
        }
      },
      {
        className: 'action-button-wrapper',
        users: '',
        orderable: false,
        render(data) {
          return data ? `<a class="link"><span class="fa fa-pencil"></span></a>
                          <a data-id="${data}" class="group-delete-link">
                            <span class="fa fa-trash"></span>
                          </a>` : '';
        }
      }
    ],
    async drawCallback() {
      try {
        const { data: roles } = await rolesPromise;
        usersPromise.resolve();

        if (bulkUsers) {
          bulkUsers.userSelect.checkBoxes('clear');
          bulkUsers.rolesSelect.checkBoxes('clear');
        }

        _updatePopulateButton(globalFetchDetail, false);

        globalusers = Object.create(null);
        globalroles = Object.create(null);

        $('div.user-grp-dropdown :not(.persistent-li)').remove();

        _addUserAndSetupEvents(paginatedUsers, roles);
        _addUserCounts();

        $('ul.nav.nav-tabs[role=tablist]').on('shown.bs.tab', () => {
          if ($usersDataTable) {
            $usersDataTable.columns.adjust();
          }
        });
      } catch (error) {
        arc.report.alertMessage(`Failed to load groups and roles - ${error}`);
      }
    }
  });

  return usersPromise.promise;
}

let $groupTable = $('#groups-table');
function _drawPaginatedGroups(rolesPromise) {
  if (!$groupTable.length) {
    $groupTable = $('#groups-table');
  }

  const groupsPromise = promiseWithResolver();

  $groupsDataTable = $groupTable.DataTable({
    paging: true,
    deferRender: true,
    destroy: true,
    searching: true,
    order: [[1, 'asc']],
    processing: true,
    serverSide: true,
    ajax(params, callback) {
      rolesPromise.then(() => {
        arc.frontend.vizHttp.get({
          url: arc.frontend.globals.urls.groupsUrl,
          params
        })
        .then(({ data }) => {
          paginatedGroups = data.groups;
          totalGroups = data.numberOfGroups;
          callback(
              {
                recordsTotal: data.numberOfGroups,
                recordsFiltered: data.numberOfFiltered,
                data: data.groups.map((group) => [
                  [],
                  group.id,
                  arc.misc.escapeHTML(group.name),
                  group.users,
                  group.roles
                ]),
              },
          );
        }).then(() => {
          arc.frontend.vizHttp.get({
            url: arc.frontend.globals.urls.groupsBasicUrl
          }).then((response) => {
            summarizedGroups = response.data;
          });
        });
      });
    },
    columns: [
      {
        className: 'group-table-ckb',
        groups: '',
        orderable: false,
        render(data, _, row) {
          return `<input type="checkbox" value="${row[1]}">`;
        }
      },
      {
        className: 'group-table-name',
        groups: 'name',
        name: 'name',
        render(data, _, row) {
          return `<span data-id="${row[1]}">${row[2]}</span>`;
        }
      },
      {
        className: 'group-table-users-count',
        groups: 'members_count',
        name: 'members_count',
        render(data, _, row) {
          const membersCount = row[3].length;
          return `<span class="badge badge-secondary badge-pill">${membersCount}</span>`;
        }
      },
      {
        className: 'group-table-users',
        groups: 'members',
        name: 'members',
        render(data, _, row) {
          const usernamesAndFullnames = row[3].map((user) => {
            if (user.full_name) {
              return `${user.username} (${user.full_name})`;
            }

            return `${user.username}`;
          });

          // Create the HTML for li elements
          const liElements = usernamesAndFullnames.map((user) => `<li>${user}</li>`).join('');

          return `<div class="gp-users-list-wrapper">
              <ul class="gp-users-list">
                  ${liElements}
              </ul>
            </div>`;
        }
      },
      {
        className: 'group-table-roles',
        groups: 'roles',
        name: 'roles',
        render(data, _, row) {
          const roles = [];
          row[4].forEach((role) => {
            roles.push(`${role.name}`);
          });
          return roles;
        }
      },
      {
        className: 'action-button-wrapper',
        orderable: false,
        render(data) {
          return `<a class="link"><span class="fa fa-pencil"></span></a>
                          <a data-id="${data}" class="group-delete-link">
                            <span class="fa fa-trash"></span>
                          </a>`;
        }
      }
    ],
    async drawCallback() {
      try {
        const { data: roles } = await rolesPromise;
        groupsPromise.resolve();

        _updatePopulateButton(globalFetchDetail, false);

        globalgroups = Object.create(null);

        _addGroupAndSetupEvents(paginatedGroups, roles);
        _addGroupCounts();

        $('ul.nav.nav-tabs[role=tablist]').on('shown.bs.tab', () => {
          if ($groupsDataTable) {
            $groupsDataTable.columns.adjust();
          }
        });
      } catch (error) {
        arc.report.alertMessage(`Failed to load groups and roles - ${error}`);
      }
    }
  });

  return groupsPromise.promise;
}

function _addGroupCounts() {
  const $tabs = $('ul.nav.nav-tabs[role=tablist]');
  $tabs.find('[href=\\#groups-tab] .badge').text(totalGroups);
}

function _addUserCounts() {
  const $tabs = $('ul.nav.nav-tabs[role=tablist]');
  $tabs.find('[href=\\#users-tab] .badge').text(totalUsers);
}

function _redrawPaginatedUsersAndGroups() {
  // stay on the current paginated page on reload
  $table.DataTable().ajax.reload(null, false);
  $groupTable.DataTable().ajax.reload(null, false);
}

function _addGroupAndSetupEvents(groups, roles) {
  const $deleteModal = $('#delete-group-modal');
  const $modifier = $deleteModal.find('.arc-modifier');
  const $names = $deleteModal.find('.arc-delete-name');

  if (!$groupTable.length) {
    $groupTable = $('#groups-table');
  }

  function _setupDeleteConfirmation(groupId, groupName, $editModal) {
    if (arc.misc.modalShown($deleteModal)) {
      return;
    }

    const g = arc.auth.Group();
    let gList;
    let groupNames;

    $modifier.toggleClass('arc-invisible', groupName || groupId.length === 1);

    if (groupName) {
      $names.text(`: ${groupName}`);
      g.groupId(groupId);
    } else {
      groupNames = groupId.map((gId) => $groupTable.find(`.group-table-name span[data-id=${gId}]`).text());

      $names.text(`: ${groupNames.join(', ')}`);
      gList = groupId;
    }

    const _closeModal = () => {
      $deleteModal.find('#delete-group-yes').off('click.deletegroup');
      $deleteModal.modal('hide');
    };

    const _redrawAndCloseModal = function () {
      _redrawPaginatedUsersAndGroups();
      if ($editModal) {
        $editModal.modal('hide');
      }
    };

    $deleteModal.modal('show');
    $deleteModal.find('#delete-group-yes')
                .offOn('click.deletegroup', () => {
                  g.delete(_closeModal, _redrawAndCloseModal, gList);
                });
  }

  if (groups && groups.length) {
    $('#bulk-assign-groups').show();
    $('#group-information').show();

    // Set up checkboxes and bulk delete group action, but only once
    if (!bulkGroups) {
      const $actionButtons = $('#bulk-delete-groups');
      bulkGroups = $groupTable.checkBoxes({ objectType: 'group', $actionButtons });
      $actionButtons.on('click.bulkdelete', () => {
        _setupDeleteConfirmation(bulkGroups.checkBoxes('selected'));
      });
    } else {
      bulkGroups.checkBoxes('reset', $groupTable);
    }

    groups.forEach((group) => {
      _updateGlobalGroup(group);
    });

    // open edit group modal when user clicks on table row
    // but don't trigger if delete icon is clicked
    $groupTable.find('tbody').offOn('click.open_modal', (e) => {
      const tr = $(e.target).closest('tr');
      const groupName = tr.find('.group-table-name').text();
      const groupId = tr.find('.group-table-name span').attr('data-id');

      if ($(e.target).closest('a').hasClass('group-delete-link')) {
        // Setup deletion
        _setupDeleteConfirmation(groupId, groupName);
      } else {
        // Setup editing
        const group = globalgroups[groupName];
        const $editModal = $('#edit-group-modal');
        if (arc.misc.modalShown($editModal)) {
          return;
        }

        _clearFormErrors($editModal);
        $editModal.find('.group-name').val(group.name).data('old-groupname', group.name);
        group_userSelectionWidgets.selectionField('setPreviousSelections', group.users);
        group_roleSelectionWidgets.selectionField('setPreviousSelections', group.roles);

        // Turn on submit button
        $editModal.find('#save-edit-group-btn').removeClass('disabled');

        $editModal.find('#edit-delete-group-btn').offOn('click.editDeleteGroup',
          () => _setupDeleteConfirmation(groupId, groupName, $editModal));

        $editModal.modal();
      }
    });
  }

  if (user_groupSelectionWidgets) {
    user_groupSelectionWidgets.selectionField('selectionList', groups);
  } else {
    user_groupSelectionWidgets =
      $('#groups-tab-edit, #groups-tab-new')
        .find('.first-tab-selection')
        .selectionField({
          selectionList: groups,
          modalName: 'user',
          easyAdd: {
            placeholder: 'New group name',
            addCb(val, e, easyAdd) {
              e.preventDefault();
              e.stopPropagation();
              arc.auth.Group().groupName(easyAdd.value()).save(undefined, (group) => {
                user_groupSelectionWidgets.selectionField('addSelectionToUnselected', group);
              });
            },
          },
        });
  }

  if (user_roleSelectionWidgets) {
    user_roleSelectionWidgets.selectionField('selectionList', roles);
  } else {
    user_roleSelectionWidgets =
      $('#user-roles-tab-edit, #user-roles-tab-new')
        .find('.second-tab-selection')
        .selectionField({
          selectionList: roles,
          modalName: 'user',
          searchProperty: 'name',
        });
  }
}

function _addUserAndSetupEvents(users, roles) {
  const $deleteModal = $('#delete-user-modal');
  const $modifier = $deleteModal.find('.arc-modifier');
  const $names = $deleteModal.find('.arc-delete-name');

  function _setupDeleteConfirmation(username, $editModal) {
    if (arc.misc.modalShown($deleteModal)) {
      return;
    }

    const singleUser = typeof username === 'string';
    let uList;
    const u = arc.auth.User();

    $modifier.toggleClass('arc-invisible', singleUser || username.length === 1);

    if (singleUser) {
      $names.text(`: ${username}`);
      u.username(username);
    } else {
      $names.text(`: ${username.join(', ')}`);
      uList = username;
    }

    const _afterDelete = () => {
      $deleteModal.find('#delete-user-yes').off('click.deleteuser');
      $deleteModal.modal('hide');
      _redrawPaginatedUsersAndGroups();
      if ($editModal) {
        $editModal.modal('hide');
      }
    };

    $deleteModal.modal('show');
    $deleteModal.find('#delete-user-yes').offOn('click.deleteuser', () => {
      u.delete(_afterDelete, uList);
    });
  }

  const $userTable = $('#users-table');

  if (group_userSelectionWidgets) {
    group_userSelectionWidgets.selectionField('selectionList', users);
  } else {
    group_userSelectionWidgets =
      $('#users-tab-edit, #users-tab-new')
        .find('.first-tab-selection')
        .selectionField({
          selectionList: users,
          modalName: 'group',
          searchProperty: 'displayname',
        });
  }

  if (group_roleSelectionWidgets) {
    group_roleSelectionWidgets.selectionField('selectionList', roles);
  } else {
    group_roleSelectionWidgets =
      $('#group-roles-tab-edit, #group-roles-tab-new')
        .find('.second-tab-selection')
        .selectionField({
          selectionList: roles,
          modalName: 'group',
          searchProperty: 'name',
        });
  }

  users.forEach((user) => {
    _updateGlobalUser(user);
  });

  globalroles = Object.assign(Object.create(null), globalroles, ...roles.map((r) => (
    { [r.id]: r })));

  // Set up checkboxes and bulk user actions, but only once
  const $groupDropdown = $('.user-grp-dropdown');
  const $rolesDropdown = $('.user-role-dropdown');
  if (!bulkUsers) {
    if (roles.length) {
      $('#bulk-assign-roles').show();
      const $userRoleDivider = $('#user-role-divider');
      roles.sort().forEach((r) => {
        const htmlString = `
          <div class="dropdown-item form-check form-check-inline">
            <input id=${r.id} class="form-check-input" type="checkbox" value="${r.id}">
            <label for=${r.id} class="form-check-label">${r.name}</label>
          </div>
        `;

        $(htmlString).insertBefore($userRoleDivider);
      });
    } else {
      $('#bulk-assign-roles').hide();
    }

    const $bulkActionButtons = $('#bulk-assign-groups, #bulk-assign-roles, #bulk-delete-users');

    bulkUsers = {
      userSelect: $userTable.checkBoxes({ $actionButtons: $bulkActionButtons }),
      groupSelect: $groupDropdown.checkBoxes({ hasAllCheckbox: false }),
      rolesSelect: $rolesDropdown.checkBoxes({ hasAllCheckbox: false }),
    };

    // Set up bulk group adding
    $groupDropdown.offOn('click.groupdropdown', (e) => {
      e.stopPropagation();
      if (e.target.id === 'app-group-save-lnk') {
        const data = {
          user_ids: bulkUsers.userSelect.checkBoxes('selected').map((name) => globalusers[name].id),
          group_ids: bulkUsers.groupSelect.checkBoxes('selected').map(Number),
        };

        const _onSave = () => $(e.target).closest('.show').removeClass('show');

        const _onSuccess = () => {
          arc.report.alertMessageSmall('Users saved', 'alert-success');
          _redrawPaginatedUsersAndGroups();
        };

        $.ajax({
          url: arc.frontend.globals.urls.bulkUsersGroupsUrl,
          method: 'POST',
          data: JSON.stringify(data),
          headers: {
            'X-CSRFToken': arc.globals.csrf_token,
            contentType: 'application/json',
          },
        }).done(_onSuccess).always(_onSave);
      }
    });

    // Set up bulk role adding
    $rolesDropdown.offOn('click.rolesDropdown', (evt) => {
      evt.stopPropagation();
      if (evt.target.id === 'app-role-save-lnk') {
        const selectedUsers = bulkUsers.userSelect.checkBoxes('selected');
        const selectedRoles = bulkUsers.rolesSelect.checkBoxes('selected').map(Number);

        const numberRoles = selectedRoles.length;
        let originalUsers;

        const _onSave = () => $(evt.target).closest('.show').removeClass('show');

        const _onSuccess = () => {
          arc.report.alertMessageSmall('Users saved', 'alert-success');
          _redrawPaginatedUsersAndGroups();
        };

        selectedRoles.forEach((r, i) => {
          const originalRole = globalroles[r];
          originalUsers = arc.misc.arrayUnique(originalRole.users.concat(selectedUsers));
          originalRole.users = originalUsers;

          // Only run success and done callbacks after saving the last user
          if (i + 1 === numberRoles) {
            arc.roles.saveRole(originalRole, _onSave, _onSuccess);
          } else {
            arc.roles.saveRole(originalRole);
          }
        });
      }
    });

    // Set up bulk deletion and bulk setting groups
    $bulkActionButtons.filter('#bulk-delete-users').on('click.bulkdelete', () => {
      _setupDeleteConfirmation(bulkUsers.userSelect.checkBoxes('selected'));
    });
  } else {
    bulkUsers.userSelect.checkBoxes('reset', $userTable);
    bulkUsers.groupSelect.checkBoxes('reset', $groupDropdown);
  }

  $userTable.find('.rg_populate').offOn('click.rgPopulate', (e) => {
    const $tr = $(e.target).closest('tr');
    const username = $tr.data('username');

    e.stopImmediatePropagation();
    let user = globalusers[username];
    _updateUserRow(user, $tr, true);

    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.usersDetailedApiUrl}/${username}`)
      .await((error, [firstUser]) => {
        if (error) {
          _showFetchError(user, error);
          _updateUserRow(user, $tr);
        } else {
          user = firstUser;
          _updateGlobalUser(user);
          _updateUserRow(user, $tr);
        }
      });
  });

  // open edit user modal when user clicks on table row
  // but don't trigger if delete icon is clicked
  $userTable.find('tbody').offOn('click.openusermodal', (e) => {
    const $tr = $(e.target).closest('tr');
    // const username = $tr.data('username');
    const usernameArray = $tr.find('.user-table-name').text().split(' ');
    const username = usernameArray[0];

    function after_details(event, name) {
      if (
        $(event.target).closest('a').hasClass('user-delete-link') &&
        !$(event.target).closest('a').hasClass('disabled')
      ) {
        // Setup deletion
        _setupDeleteConfirmation(name);
      } else {
        _openEditModal(name);
      }
    }

    let user = globalusers[username];
    if (user?.groups !== undefined) {
      after_details(e, username);
    } else {
      _updateUserRow(user, $tr, true);
      d4.queue()
        .defer(d3.json, `${arc.frontend.globals.urls.usersDetailedApiUrl}/${username}`)
        .await((error, [firstUser]) => {
          if (error) {
            _showFetchError(user, error);
            _updateUserRow(user, $tr);
          } else {
            user = firstUser;
            _updateGlobalUser(user);
            _updateUserRow(user, $tr);
            after_details(e, username);
          }
        });
    }
  });

  function _openEditModal(username) {
    const user = globalusers[username];
    const $showPasswordFormGroup =
      $('#change-password-btn')
        .parents('.form-group');

    if (!user.is_password_editable) {
      $showPasswordFormGroup.hide();
    } else if ($showPasswordFormGroup.is(':hidden')) {
      $showPasswordFormGroup.show();
    }

    const $editModal = $('#edit-user-modal');
    if (arc.misc.modalShown($editModal)) {
      return;
    }

    const profile = JSON.parse(user.profile);
    const proxy_username = profile.proxy_username || '';
    $editModal.find('.user-profile-section')
      .toggleClass('arc-invisible', !arc.globals.arcviz_settings.PROFILE_PROXY_USERNAME);
    $editModal.find('.user-username').val(user.username).data('old-username', user.username);
    $editModal.find('.user-fname').val(user.first_name).data('old-fname', user.first_name);
    $editModal.find('.user-lname').val(user.last_name).data('old-lname', user.last_name);
    $editModal.find('.user-proxy').val(proxy_username).data('old-proxy', proxy_username);
    $editModal.find('.user-datejoined').text(user.datejoined_formatted);
    $editModal.find('.user-lastlogin').text(user.lastlogin_formatted);
    _clearFormErrors($editModal);
    user_groupSelectionWidgets.selectionField('setPreviousSelections', user.groups);
    user_roleSelectionWidgets.selectionField('setPreviousSelections', user.roles);

    const permLevel = user.is_superuser ? 'admin' : 'normal';
    $editModal.find('input:radio[name=user-perm]').val([permLevel]);

    // Disable set permissions in cdw form factor
    const cdv_mode = arc.globals.arcviz_settings.CDV_MODE;
    const saml_required_groups_check = arc.globals.arcviz_settings.SAML_REQUIRED_GROUPS_CHECK;
    if (cdv_mode === 'cdw' || saml_required_groups_check === 'True') {
      $editModal.find('.user-permission').attr('disabled', 'disabled');
    }

    const isActive = user.is_active ? 'active' : 'inactive';
    $editModal.find('input:radio[name=user-perm]').val([permLevel]);
    $editModal.find('input:radio[name=user-active]').val([isActive]);

    // Turn on submit button
    const saveBtn = $('#edit-user-modal').find('#save-edit-user-btn');
    saveBtn.removeClass('disabled');
    saveBtn.prop('disabled', false);

    $editModal.find('#edit-delete-user-btn').offOn('click.editDeleteUser', () => _setupDeleteConfirmation(username, $editModal));
    arc.frontend.legacyBridge.events.next({ type: 'password-edit-reset-visibility' });
    _togglePwSection(false);
    $editModal.modal();
  }
}
}());
