window.arc = window.arc || {};

(function() {
'use strict';

arc.querylogs = arc.querylogs || {};

function colnamesToStringDimensions(colnames) {
  const cols = [];

  colnames.forEach((colname, coli) => {
    cols.push(arc.data.column()
              .colname(colname)
              .coltype('STRING')
              .colproperty(arc.report.columnTypeEnum.DIMENSION)
              .colsource('dimensions_shelf')
              .colindex(coli));
  });

  return cols;
}

// Usage Analytics
function setupImportUsageDashboard() {
  $('#usage-dashboard-import').offOn('click', async () => {
    const $status = $('#usage-dashboard-import-status');
    $status.addClass('arc-invisible');

    try {
      const { data } = await arc.frontend.vizHttp.post({
        url: arc.frontend.globals.urls.importUsageDashboardUrl,
        headers: {
          'X-CSRFToken': arc.globals.csrf_token,
        }
      });

      $status.removeClass('arc-invisible');
      $status.removeClass('alert-danger');
      $status.addClass('alert-success');
      $status.text(`A customizable copy was successfully imported. `);
      if (data.dashboards.at(0)?.id) {
        $status.append($(`<a href="/arc/apps/app/${data.dashboards.at(0).id}">Go to the dashboard.</a>`));
      }
    } catch (e) {
      $status.removeClass('arc-invisible');
      $status.removeClass('alert-success');
      $status.addClass('alert-danger');
      $status.text('Could not import the Usage Analytics Dashboard');
    }
  });
}

// Current Load
function currentLoadStatsRefresher() {
  const r = function() { return 'currentLoadStatsRefresher'; };

  const currentLoadStats = arc.currentLoadStats();

  r.initialize = function() {
    $('#tab-server-current-load').offOn('shown.bs.tab', r.on);
    $('#tab-server-current-load').offOn('hidden.bs.tab', r.off);

    $('#current-load-refresh-now').offOn('click.stats', r.refreshServerStats);
    return r;
  };

  r.on = function() {
    arc.shortcut.remove('Ctrl+R');
    arc.shortcut.add('Ctrl+R', r.refreshServerStats);

    r.refreshServerStats();
    return r;
  };

  r.off = function() {
    arc.shortcut.remove('Ctrl+R');
    return r;
  };

  r.refreshServerStats = function() {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.statsDataUrl}/all`)
      .await((error, statsData) => {
        if (error) {
          arc.report.alertMessage('Failed to load server stats',
              error.response ? error.response : error);
          return;
        }

        currentLoadStats.load(statsData)
                  .paint($('#server-current-load-data'));
      });

    return r;
  };

  return r;
}

// WebServer Stats
function serverStatsRefresher() {
  const r = function() { return 'serverStatsRefresher'; };

  const serverStats = arc.serverStats();

  let statsRefreshTimer;

  function handleAutoRefresh() {
    if ($('#server-stats-autorefresh input').prop('checked')) {
      setupNextServerStatsRefresh();
    } else {
      r.stopServerStatsRefresh();
    }
  }

  function handleStatsDiff() {
    serverStats.showDiffs($('#server-stats-showdiff input').prop('checked'));
    r.refreshServerStats();
  }

  r.initialize = function() {
    $('#server-stats-showdiff input').prop('checked', false);
    $('#server-stats-showdiff input').offOn('change.refresh', handleStatsDiff);
    $('#server-stats-autorefresh input').offOn('change.refresh', handleAutoRefresh);

    $('#stats-refresh-now').offOn('click.stats', r.refreshServerStats);

    $('#tab-server-stats').offOn('shown.bs.tab', r.startServerStatsRefresh);
    $('#tab-server-stats').offOn('hidden.bs.tab', r.stopServerStatsRefresh);

    return r;
  };

  r.refreshServerStats = function() {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.statsDataUrl}/all`)
      .await((error, statsData) => {
        if (error) {
          arc.report.alertMessage('Failed to load server stats',
              error.response ? error.response : error);
          return;
        }

        serverStats.load(statsData)
                   .paint($('#server-stats-data'));
      });
  };

  r.startServerStatsRefresh = function() {
    r.refreshServerStats();
    handleAutoRefresh();

    arc.shortcut.remove('Ctrl+R');
    arc.shortcut.add('Ctrl+R', r.refreshServerStats);
  };

  function setupNextServerStatsRefresh() {
    const statsRefreshTimeSeconds = 5;

    if (statsRefreshTimer) {
      return;
    }

    statsRefreshTimer = setTimeout(() => {
      statsRefreshTimer = undefined;
      r.refreshServerStats();
      setupNextServerStatsRefresh();
    }, statsRefreshTimeSeconds * 1000);
  }

  r.stopServerStatsRefresh = function() {
    arc.shortcut.remove('Ctrl+R');

    if (statsRefreshTimer) {
      clearTimeout(statsRefreshTimer);
      statsRefreshTimer = undefined;
    }
  };

  return r;
}

arc.querylogs.querylogs = function() {
  const querylogs = function() { return 'arc.querylogs.querylogs'; };

  setupImportUsageDashboard();

  const statsRefresher = serverStatsRefresher();
  const currentLoad = currentLoadStatsRefresher();

  const dimColumnIdx = 0;
  const metricColumnIdx = 1;

  function plotOneStat(config) {
    const {
      dimensionName,
      chartDiv,
      aggDict,
      sortAgg,
      colorName,
      tooltipName,
    } = config;

    // Create the bar
    const bar = arc.visual.plugins.bars();
    const driver = arc.report.driver(bar);
    driver.chart_div(chartDiv)
      .params(arc.params.tracker());
    bar.viz_setSettings({
      settings: {
        colors: [
          '#3182bd', '#6baed6', '#9ecae1', '#c6dbef', '#e6550d',
          '#fd8d3c', '#fdae6b', '#fdd0a2', '#31a354', '#74c476', '#a1d99b',
          '#c7e9c0', '#756bb1', '#9e9ac8', '#bcbddc', '#dadaeb', '#636363',
          '#969696', '#bdbdbd', '#d9d9d9'
        ]
      },
      user_settings: {
        bar_mark_size: '18-18',
        fixed_color_legend: 'true',
        color_opacity: '100'
      }
    });

    // Setup columns
    const colDim =
      arc.data.column()
        .colname(dimensionName)
        .coltype('STRING')
        .colproperty(arc.report.columnTypeEnum.DIMENSION)
        .colsource('y_shelf')
        .colindex(dimColumnIdx);
    const colQueryCount =
      arc.data.column()
        .colname('Count')
        .coltype('INT')
        .colproperty(arc.report.columnTypeEnum.AGGREGATE)
        .colsource('x_shelf')
        .colindex(metricColumnIdx);

    let colColor;
    if (colorName) {
      colColor =
        arc.data.column()
          .colname(colorName)
          .coltype('STRING')
          .colproperty(arc.report.columnTypeEnum.DIMENSION)
          .colsource('color_shelf')
          .colindex(2);
    }

    let colTooltip;
    if (tooltipName) {
      colTooltip =
        arc.data.column()
          .colname(tooltipName)
          .coltype('STRING')
          .colproperty(arc.report.columnTypeEnum.DIMENSION)
          .colsource('tooltip_shelf')
          .colindex(colorName ? 3 : 2);
    }

    // Setup results
    const result = arc.data.result();
    const rows = aggDictToRows({
      aggDict,
      sortAgg,
      colColor,
      colTooltip
    });
    result.rows(rows);

    const cols = [colDim, colQueryCount];
    if (colColor) {
      cols.push(colColor);
    }

    if (colTooltip) {
      cols.push(colTooltip);
    }

    result.columns(cols);

    // create the visual by setting the data and then drawing it
    bar.viz_setDataResult(result);
    bar.viz_drawVisual(chartDiv);
  }

  function aggDictToRows(config) {
    const { aggDict, sortAgg, colColor, colTooltip } = config;

    let rows = [];
    $.each(aggDict, (k, v) => {
      if (colColor && colTooltip) {
        rows.push([v[dimColumnIdx], v[metricColumnIdx], v[2], v[3]]);
      } else if (colColor || colTooltip) {
        rows.push([v[dimColumnIdx], v[metricColumnIdx], v[2]]);
      } else {
        rows.push([k, v]);
      }
    });

    // First, understand the counts by the dimension
    const totals_by_dim = {};
    rows.forEach((row) => {
      const dim = row[dimColumnIdx];
      if (!totals_by_dim[dim]) {
        totals_by_dim[dim] = row[metricColumnIdx];
      } else {
        totals_by_dim[dim] += row[metricColumnIdx];
      }
    });

    if (sortAgg) {
      rows.sort((a, b) => {
        const a_dim = a[dimColumnIdx];
        const b_dim = b[dimColumnIdx];

        return totals_by_dim[b_dim] - totals_by_dim[a_dim];
      });
    }

    const specificCount = 15;

    // We want to draw 10 bars. How many of the rows fall below that line?
    const uniqueDims = {};
    let endIdx;
    rows.some((row, idx) => {
      const dim = row[dimColumnIdx];
      if (uniqueDims[dim]) {
        return false;
      }

      if (Object.keys(uniqueDims).length === specificCount) {
        endIdx = idx;
        return true;
      }

      uniqueDims[dim] = true;
      return false;
    });

    if (endIdx) {
      let i = endIdx;
      let others_total = 0;

      while (i < rows.length) {
        others_total += rows[i][metricColumnIdx];
        i += 1;
      }

      if (colColor && colTooltip) {
        rows[specificCount] = ['Others', others_total, 'Mixed', 'Mixed'];
      } else if (colColor || colTooltip) {
        rows[specificCount] = ['Others', others_total, 'Mixed'];
      } else {
        rows[specificCount] = ['Others', others_total];
      }

      rows = rows.splice(0, specificCount + 1);
    }

    return rows;
  }

  function plotStats(statistics, allDataConnection) {
    plotOneStat({
      dimensionName: 'Action',
      chartDiv: 'bar-by-action',
      aggDict: statistics.aggAction,
      sortAgg: true,
      colorName: 'Component'
    });

    plotOneStat({
      dimensionName: 'Queries returned from backend VS cache',
      chartDiv: 'bar-by-cache-statistic',
      aggDict: statistics.aggCacheStat,
      sortAgg: true,
      colorName: 'Component'
    });

    plotOneStat({
      dimensionName: 'User',
      chartDiv: 'bar-by-user',
      aggDict: statistics.aggUsername,
      sortAgg: true
    });
    plotOneStat({
      dimensionName: 'IP',
      chartDiv: 'bar-by-ip',
      aggDict: statistics.aggIp,
      sortAgg: true
    });
    plotOneStat({
      dimensionName: 'Data Connection',
      chartDiv: 'bar-by-connection',
      aggDict: statistics.aggConnection,
      sortAgg: true
    });
    plotOneStat({
      dimensionName: 'Dashboard',
      chartDiv: 'bar-by-app',
      aggDict: statistics.aggApp,
      sortAgg: true,
      colorName: 'User',
      tooltipName: 'Dataset Name'
    });
    plotOneStat({
      dimensionName: 'Dataset',
      chartDiv: 'bar-by-dataset',
      aggDict: statistics.aggDataset,
      sortAgg: true,
      tooltipName: 'Dataset Name'
    });
    plotOneStat({
      dimensionName: 'Runtime Bucket',
      chartDiv: 'bar-by-runtime',
      aggDict: statistics.aggRuntime,
    });

    let $sel = $('#bar-by-app .y text, #bar-by-visual .y text');
    $sel.offOn(
      'click.barByVisual',
      function() {
          const appid = $(this).text();
          arc.misc.openInNewWindow(linkApp(appid));
      }
    );
    $sel = $('#bar-by-connection .y text');
    $sel.offOn(
      'click.barByConnection',
      function() {
        const dcname = $(this).text().replace('...', '').trim();
        const dc = allDataConnection.find((dataconn) => dataconn.name.includes(dcname));
        const dcid = dc ? dc.id : undefined;
        arc.misc.openInNewWindow(linkDataConnection(dcid));
      }
    );
  }

  function plotTable(data, chartId, downloadId, colnames, csvName) {
    const $container = $(`#${chartId}`);
    $container.empty();

    const table = arc.visual.plugins.table();
    const driver = arc.report.driver(table)
                     .chart_div(chartId)
                     .params(arc.params.tracker());

    const settings = {
      table_template: 'table-template-3',
      download_rows: '',
      page_size: '10',
    };

    $.each(settings, (k, v) => {
      driver.user_settings().find_by_id(k)
                            .value(v);
    });

    const cols = colnamesToStringDimensions(colnames);

    const result = arc.data.result();
    result.columns(cols);
    result.rows(data);
    result.finalize();

    // create the visual by setting the data and then drawing it
    table.viz_setDataResult(result);

    table.viz_drawVisual(chartId);

    driver.title(csvName);
    driver.sb_colnames(colnames);
    driver.sb_rows(data);
    $(`#${downloadId}`).offOn('click.download', () => {
      driver.exportCSV({});
    });
  }

  function linkDataConnection(dcid) {
    return `${arc.frontend.globals.urls.dataUrl}?dataconnection_id=${dcid}`;
  }

  function linkApp(appid) {
    return `${arc.frontend.globals.urls.viewDashboardUrl}/${appid}`;
  }

  function adjustContainer($container) {
    const { top } = $container.offset();
    const documentHeight = $(document).height();

    // Allow some space at the bottom of the page
    let scrollHeight = documentHeight - top - 40;
    if (scrollHeight < 100) {
      scrollHeight = 100;
    }

    $container.css('width', '300px')
              .css('max-height', `${scrollHeight}px`);
  }

  function showArcvizSettings() {
    $('#setting-name').empty();
    $('#setting-value').empty();

    $('#setting-name').on('change.name', function() {
      const name = $(this).val().trim();
      if (name) {
        fetchArcvizSettings(name);
      } else {
        $('#setting-value').empty();
      }
    });

    function fetchArcvizSettings(name) {
      $('#setting-value').empty();

      d4.queue()
        .defer(d3.json, `${arc.frontend.globals.urls.arcvizSettingsUrl}?name=${name}`)
        .await((error, arcvizSettings) => {
          if (error) {
            arc.report.alertMessage('Failed to fetch data');
            return;
          }

          const keys = Object.keys(arcvizSettings);
          if (keys.length === 0) {
            $('#setting-value').text('no value set');
          } else {
            let v = arcvizSettings[keys[0]];
            try {
              v = JSON.stringify(v, null, 4);
            } catch (err) {
              // Ignore any errors - setting value may not be json
            }

            $('#setting-value').text(v);
          }
        });
    }
  }

  let inputDays = 30;
  let $spinner;

  function showUsersAndApps() {
    $('#viz-activities-download').hide();
    $('#viz-users-download').hide();
    $('#viz-apps-download').hide();

    const $tab = $('#pane-users-and-apps');
    if ($spinner === undefined) {
      $spinner = $tab.find('.wan-spinner');
      $spinner.WanSpinner({
        start: inputDays,
        step: 10,
        valueChanged() {
          const v = Number($spinner.find('input').val());
          if (v === 11 && inputDays === 1) {
            inputDays = 10;
            $spinner.find('input').val(10);
          } else if (v < 1) {
            inputDays = 1;
            $spinner.find('input').val(1);
          } else {
            inputDays = v;
          }

          fetchUsersAndApps(inputDays);
        },
      });
    }

    fetchUsersAndApps(inputDays);

    function fetchUsersAndApps(maxDays) {
      $tab.find('.refreshing').show();
      const asset = {
        asset: `${arc.frontend.globals.urls.activeusersUrl}?days=${maxDays}`,
        postprocess: d3.csv,
      };
      arc.staticfetcher.fetch(
        [asset],
        (error, usersAndApps) => {
          $tab.find('.refreshing').hide();
          if (error) {
            arc.report.alertMessage('Failed to fetch data');
            return;
          }

          const days = usersAndApps.map((item) => item.day).sort();
          const today = new Date();
          const firstDay = days.length ? new Date(days[0]) : today; // 2024-9-12
          const millisecondsInADay = (24 * 60 * 60 * 1000);
          const actualNumberOfDayReturned =
            Math.floor((today - firstDay) / millisecondsInADay) + 1;

          $('#day-count').text(actualNumberOfDayReturned);
          const users = [];
          const apps = [];
          const activities = [];

          usersAndApps.forEach((d) => {
            if (d.type === 'Users') {
              users.push([d.day, d.id, d.count]);
            } else if (d.type === 'Dashboards') {
              apps.push([d.day, d.id, d.count]);
            } else if (d.type === 'Activities') {
              activities.push([d.day, d.count]);
            }
          });

          const fmt = d3.format(',');
          let activitiesCount = 0;
          activities.forEach((r) => {
            activitiesCount += Number(r[1]);
          });
          $('#viz-activities-count').text(`(${fmt(activitiesCount)})`);

          let usersCount = 0;
          const usersH = {};
          users.forEach((r) => {
            if (usersH[r[1]] === undefined) {
              usersH[r[1]] = true;
              usersCount += 1;
            }
          });
          $('#viz-users-count').text(`(${fmt(usersCount)})`);

          let appsCount = 0;
          const appsH = {};
          apps.forEach((r) => {
            if (appsH[r[1]] === undefined) {
              appsH[r[1]] = true;
              appsCount += 1;
            }
          });
          $('#viz-apps-count').text(`(${fmt(appsCount)})`);

          adjustContainer($('#viz-activities-table'));
          adjustContainer($('#viz-users-table'));
          adjustContainer($('#viz-apps-table'));

          plotTable(activities, 'viz-activities-table', 'viz-activities-download',
              ['Date', 'Activity Count'], 'activities');
          plotTable(users, 'viz-users-table', 'viz-users-download',
              ['Date', 'User', 'Activity Count'], 'users');
          plotTable(apps, 'viz-apps-table', 'viz-apps-download',
              ['Date', 'App', 'Activity Count'], 'apps');

          $('#viz-activities-download').show();
          $('#viz-users-download').show();
          $('#viz-apps-download').show();
        }
      );
    }
  }

  querylogs.arcengineLogsEnabled = function(_) {
    if (_.toLowerCase() === 'true') {
      const tab_template = $('#server-log-tab-template').html();
      const pane_template = $('#server-log-pane-template').html();
      const parms = { TABNAME: 'arcengine', TABTITLE: 'WebServer Log' };
      const $main = $('.settings-tab-bottom');

      d4.queue()
        .defer(d3.json, arc.frontend.globals.urls.dataconnectionUrl)
        .await((error, dataConnections) => {
          if (error) {
            /* eslint-disable no-console */
            console.log('Could not load data');
            console.log(error);
            /* eslint-enable no-console */
            return;
          }

          const dclist = dataConnections.filter((dc) => (dc.type === 'arcengine' && arc.misc.hasPerm('dc_explore', dc.perms)));

          if (dclist.length > 0) {
            parms.TABTITLE = `ArcEngine Log${dclist.length > 1 ? 's' : ''}`;

            const tab = arc.misc.safe_substitute(tab_template, parms);
            const pane = arc.misc.safe_substitute(pane_template, parms);
            $('#data-tabs').append(tab);
            $('#activity-log-tab-content').append(pane);

            const viewer = arc.serverlogs.serverlogs();
            viewer.initialize($(`#tab-server-log-${parms.TABNAME}`), $main, dclist);
          }
        });
    }

    return querylogs;
  };

  querylogs.webserverLogsEnabled = function(_) {
    if (_.toLowerCase() === 'true') {
      const tab_template = $('#server-log-tab-template').html();
      const pane_template = $('#server-log-pane-template').html();
      const parms = { TABNAME: 'webserver', TABTITLE: 'WebServer Log' };
      const $main = $('.settings-tab-bottom');

      const tab = arc.misc.safe_substitute(tab_template, parms);
      const pane = arc.misc.safe_substitute(pane_template, parms);
      $('#data-tabs').append(tab);
      $('#activity-log-tab-content').append(pane);

      const sl = arc.serverlogs.serverlogs();
      sl.initialize($(`#tab-server-log-${parms.TABNAME}`), $main);
    }

    return querylogs;
  };

  querylogs.initialize = function() {
    statsRefresher.initialize();
    currentLoad.initialize();

    $('#tab-users-and-apps').offOn('shown.bs.tab', () => {
      showUsersAndApps();
    });

    $('#tab-arcviz-settings').offOn('shown.bs.tab', () => {
      showArcvizSettings();
    });

    const { events, ofType } = arc.frontend.legacyBridge;

    events
      .pipe(ofType('send-activity-logs-stats'))
      .subscribe(({ payload }) => {
        const { statistics, allDataConnection } = payload;
        plotStats(statistics, allDataConnection);
      });

    function changeToTab() {
      const hash = window.location.hash || '#tab-activity';
      $('#data-tabs > li > a').blur();
      $(hash).tab('show');
    }

    window.onpopstate = changeToTab;

    $('#data-tabs > li > a').on('click.tab', function() {
      const hash = $(this).attr('id');
      arc.misc.pushState(`${window.location.pathname}#${hash}`);
    });

    changeToTab();

    return querylogs;
  };

  return querylogs;
};
}());
