
(function() {
'use strict';

const valueTemplate = `
  <div class="setting-label">%LABEL%</div>
  <div class="setting-content assign_app">%VALUE%</div>
`;

const valueTemplateHtml = `
  <div class="setting-label">%LABEL%</div>
  <div class="setting-content assign_app">%HTML_VALUE%</div>
`;

const valueTemplateStats = `
  <div class="setting-label">%LABEL%</div>
  <div class="setting-content assign_app">%HTML_VALUE_CONCURRENCY%</div>
  <div class="setting-content assign_app">%HTML_VALUE_COUNT%</div>
  <div class="setting-content assign_app">%HTML_VALUE_LATENCY%</div>
`;

const valueTemplateHistogram = `
  <div class="setting-label">
  %LABEL%
    <select id="histogram-picker" class="setting-content">
    </select>
  </div>
  <div>
    <div class="setting-content assign_app">%HTML_VALUE_BUCKET%</div>
    <div class="setting-content assign_app">%HTML_VALUE_HISTOGRAM%</div>
  </div>
`;

const numDiv = `
  <div>%VALUE%</div>
`;

const statsDiv = `
  <div class="stat" style="width:%WIDTH%px; background-color:lightblue;">%VALUE%</div>
`;

const optionElement = '<option value="%VALUE%">%LABEL%</option>';

const cancelButtonHtml = `
  <form class="form-inline">
    <div class="form-group">
      <button type="button" id="cancel-all-queries" class="btn btn-secondary viz-btn-xs">
          <i class="fa fa-remove"></i><span> Cancel All Queries </span>
      </button>
    </div>
  </form>
`;

const maxWidth = 250;

arc.currentLoadStats = function() {
  const stats = function() { return 'currentLoadStats'; };

  let statsData;

  stats.load = function(d) {
    statsData = d;

    return stats;
  };

  stats.paint = function($d) {
    $d.empty();

    $d.append(arc.misc.safe_substitute(valueTemplate, {
      LABEL: 'Start Time',
      VALUE: statsData.start_time,
    }));

    $d.append(arc.misc.safe_substitute(valueTemplate, {
      LABEL: 'Current Time',
      VALUE: statsData.current_time,
    }));

    $d.append(arc.misc.safe_substitute(valueTemplate, {
      LABEL: 'Uptime',
      VALUE: statsData.uptime,
    }));

    $d.append(arc.misc.safe_substitute(valueTemplateHtml, {
      LABEL: 'Active Queries Summary',
      HTML_VALUE: `
        <div id="current-load-active-counters"></div>
      `,
    }));

    arc.perf.drawQueues(statsData.requests.queues, 'current-load-active-counters', true);

    $d.append(arc.misc.safe_substitute(valueTemplateHtml, {
      LABEL: 'Cancel All',
      HTML_VALUE: cancelButtonHtml,
    }));

    $('#cancel-all-queries')
        .offOn('click.download', () => {
          arc.perf.cancelQuery('*', '*', true);
        });

    $d.append(arc.misc.safe_substitute(valueTemplateHtml, {
      LABEL: 'Active Queries Detail',
      HTML_VALUE: `
        <div id="current-load-active-detail"></div>
      `,
    }));

    arc.perf.drawQueries(statsData.requests.queues, 'current-load-active-detail');

    return stats;
  };

  return stats;
};

arc.serverStats = function() {
  const stats = function() { return 'stats'; };

  let data;
  let diffData;
  let showDiffs;

  stats.showDiffs = function(_) {
    if (!arguments.length) return showDiffs;
    showDiffs = _;
    if (showDiffs) {
      diffData = data;
    }

    return stats;
  };

  stats.load = function(d) {
    if (showDiffs && !diffData) {
      diffData = d;
    }

    data = d;

    if (showDiffs) {
      data.concurrentRequests.forEach((_, i) => {
        data.concurrentRequests[i] -= diffData.concurrentRequests[i];
      });

      data.concurrentRequestsTimes.forEach((_, i) => {
        data.concurrentRequestsTimes[i] -= diffData.concurrentRequestsTimes[i];
      });

      Object.keys(data.histograms).forEach((k) => {
        data.histograms[k] -= diffData.histograms[k];
      });

      let max_nonzero = 0;
      data.concurrentRequests.forEach((_d, i) => {
        if (_d > 0) {
          max_nonzero = i;
        }
      });

      data.concurrentRequests = data.concurrentRequests.slice(0, max_nonzero + 1);
      data.concurrentRequestsTimes = data.concurrentRequestsTimes.slice(0, max_nonzero + 1);
      Object.keys(data.counters).forEach((_d) => {
        if (diffData.counters[_d]) {
          data.counters[_d] -= diffData.counters[_d];
        }
      });
    }

    return stats;
  };

  stats.paint = function($d) {
    const currentHistogram = $('#histogram-picker').val();
    const $search = $('#stats-counters-search');
    const currentSearch = $search.val();
    const currentScrollPos = { x: window.scrollX, y: window.scrollY };

    $d.empty();

    $d.append(arc.misc.safe_substitute(valueTemplate, {
      LABEL: 'Start Time',
      VALUE: data.start_time,
    }));

    $d.append(arc.misc.safe_substitute(valueTemplate, {
      LABEL: 'Current Time',
      VALUE: data.current_time,
    }));

    $d.append(arc.misc.safe_substitute(valueTemplate, {
      LABEL: 'Uptime',
      VALUE: data.uptime,
    }));

    paintHistograms($d, currentHistogram);
    paintConcurrency($d);
    paintCounters($d, currentSearch);
    drawCurrentLoad($d);
    window.scrollTo(currentScrollPos.x, currentScrollPos.y);
  };

  function paintCounters($d, currentSearch) {
    $d.append(arc.misc.safe_substitute(valueTemplateHtml, {
      LABEL: 'Counters',
      HTML_VALUE: `
        <div class="selected-control-wrapper" style="margin-top:10px;">
          <div class="pick-textbox">
            <div class="widget-content">
              <div class="input-group">
                <input placeholder="Search" type="text" id="stats-counters-search"
                  class="picklist-textbox form-control" style="width:400px;">
                <div class="input-group-append picklist-textbox-search-button">
                  <div class="input-group-text">
                    <span id="queries-search-icon" class="fa fa-search"></span>
                  </div>
                </div>
              </div>
            </div>
          </div>
        </div>
        <div id="stats-perf-counters"></div>
      `,
    }));

    const colnames = ['Name', 'Value'];
    const keys = Object.keys(data.counters);
    keys.sort();
    const rows = keys.map((k) => [k, data.counters[k]]);

    arc.misc.drawTable('stats-perf-counters', rows, colnames);

    {
      const sel = d3.selectAll('#stats-perf-counters tbody td:nth-child(2)');
      arc.marks()
         .addSelection(sel)
         .params_fn((d, i, elem) => [{
                                      key: 'Data',
                                      value: $(elem).text()
                                    }])
         .marker_name('msg')
         .activate();
    }

    $('#stats-perf-counters .dataTables_scrollBody').css('max-height', '');
    $('#stats-perf-download').offOn('click.download', () => {
      arc.misc.downloadData([JSON.stringify(data, null, 2)], 'perf.json', 'text/plain');
    });

    const $search = $('#stats-counters-search');
    function searchCounters() {
      const searchStr = $search.val();
      $('#stats-perf-counters tr').each((_, row) => {
        const $row = $(row);
        let found = false;
        $row.find('td').each((_, col) => {
          const $col = $(col);
          const v = $col.text();
          if (v.search(searchStr) >= 0) {
            found = true;
          }
        });

        if (found) {
          $row.show();
        } else {
          $row.hide();
        }
      });
    }

    $search
        .offOn('keyup.search', () => {
          setTimeout(searchCounters, 0);
        });

    $search.val(currentSearch);
    setTimeout(searchCounters, 0);
  }

  function paintConcurrency($d) {
    const maxCount = Math.max(...data.concurrentRequests);
    const vals = [];
    const counts = [];
    const latency = [];

    vals.push('<div class="concurrency-nums">');
    vals.push('<h4>&nbsp;</h4>');

    counts.push('<div class="concurrency-stats">');
    counts.push('<h4>Count</h4>');
    data.concurrentRequests.forEach(function(t, i) {
      const w = ((maxWidth * t) / maxCount);
      counts.push(arc.misc.safe_substitute(statsDiv, {
        WIDTH: w,
        VALUE: t,
      }));

      vals.push(arc.misc.safe_substitute(numDiv, {
        VALUE: i + 1,
      }));
    });

    counts.push('</div>');
    vals.push('</div>');

    latency.push('<div class="concurrency-stats">');
    latency.push('<h4>Average Latency</h4>');
    const avgs = data.concurrentRequestsTimes.map(function(d, i) {
      return d / data.concurrentRequests[i];
    });

    const maxAvg = Math.max(...avgs);
    avgs.forEach(function(t) {
      const w = ((maxWidth * t) / maxAvg);
      latency.push(arc.misc.safe_substitute(statsDiv, {
        WIDTH: w,
        VALUE: Number.parseFloat(t).toFixed(2),
      }));
    });

    latency.push('</div>');

    $d.append(arc.misc.safe_substitute(valueTemplateStats, {
      LABEL: 'Webserver Requests',
      HTML_VALUE_CONCURRENCY: vals.join(''),
      HTML_VALUE_COUNT: counts.join(''),
      HTML_VALUE_LATENCY: latency.join(''),
    }));
  }

  function drawCurrentLoad($d) {
    $d.append(arc.misc.safe_substitute(valueTemplateHtml, {
      LABEL: 'Active Queries Summary',
      HTML_VALUE: `
        <div id="ws-current-load-active-counters"></div>
      `,
    }));

    arc.perf.drawQueues(data.requests.queues, 'ws-current-load-active-counters');

    $d.append(arc.misc.safe_substitute(valueTemplateHtml, {
      LABEL: 'Active Queries Detail',
      HTML_VALUE: `
        <div id="ws-current-load-active-detail"></div>
      `,
    }));

    arc.perf.drawQueries(data.requests.queues, 'ws-current-load-active-detail');
  }

  function paintHistograms($d, currentHistogram) {
    const options = [];
    Object.keys(data.histograms).forEach((k) => {
      options.push(arc.misc.safe_substitute(optionElement, {
        VALUE: k,
        LABEL: k,
      }));
    });

    const buckets = [];
    const bucketCounts = [];

    buckets.push('<div id="hist_buckets"></div>');
    bucketCounts.push('<div id="hist_counts"></div>');

    $d.append(arc.misc.safe_substitute(valueTemplateHistogram, {
      LABEL: 'Histograms',
      HTML_VALUE_BUCKET: buckets.join(''),
      HTML_VALUE_HISTOGRAM: bucketCounts.join(''),
    }));

    $('#histogram-picker').html(options.join(''));

    $('#histogram-picker')
        .offOn('change.histogram', () => {
          paintHistogram($('#histogram-picker').val());
        });

    if (currentHistogram) {
      $('#histogram-picker').val(currentHistogram);
    }

    $('#histogram-picker').chosen();

    paintHistogram($('#histogram-picker').val());
  }

  function paintHistogram(histogramName) {
    const hist = data.histograms[histogramName];

    const buckets = [];
    const bucketCounts = [];

    buckets.push('<h4>Seconds</h4>');
    bucketCounts.push('<h4>Count</h4>');
    const sortedBuckets = Object.keys(hist);
    sortedBuckets.sort(function(a, b) {
      return a - b;
    });

    const maxBucketCount = Math.max(...Object.values(hist));
    sortedBuckets.forEach(function(b, i) {
      buckets.push(arc.misc.safe_substitute(numDiv, {
        VALUE: b,
      }));

      const t = hist[b];
      const w = ((maxWidth * t) / maxBucketCount);
      bucketCounts.push(arc.misc.safe_substitute(statsDiv, {
        WIDTH: w,
        VALUE: t,
      }));
    });

    $('#hist_buckets').html(buckets.join(''));
    $('#hist_counts').html(bucketCounts.join(''));
  }

  return stats;
};

}());
