window.arc = window.arc || {};

(function() {
"use strict";

arc.auth = arc.auth || {};

arc.auth.User = function() {
  var username,
      first_name,
      last_name,
      password,
      old_password,
      is_superuser,
      is_active;

  var User = function() { return "arc.auth.User"; };

  User.username = function(_) {
    if (!arguments.length) return username;
    username = _;
    return User;
  };

  User.first_name = function(_) {
    if (!arguments.length) return first_name;
    first_name = _;
    return User;
  };

  User.last_name = function(_) {
    if (!arguments.length) return last_name;
    last_name = _;
    return User;
  };

  User.old_password = function(_) {
    if (!arguments.length) return old_password;
    old_password = _;
    return User;
  };

  User.password = function(_) {
    if (!arguments.length) return password;
    password = _;
    return User;
  };

  User.is_superuser = function(_) {
    if (!arguments.length) return is_superuser;
    is_superuser = _;
    return User;
  };

  User.is_active = function(_) {
    if (!arguments.length) return is_active;
    is_active = _;
    return User;
  };

  User.is_valid = function() {
    if (username.length < 1) {
      return false;
    }

    if ((password !== undefined) && (password.length < 5)) {
      return false;
    }

    if (old_password !== undefined && password === undefined) {
      return false;
    }

    return true;
  };

  User.delete = function(done_cb, user_list) {
    User.makeRESTCall({
      type: 'DELETE',
      data: user_list ? { users: JSON.stringify(user_list) } : {},
      idString: user_list ? '' : username,
      done_cb: done_cb
    });
  };

  /*
   * config is:
   *   config.is_new: save as a new user
   *   config.save_password: if config.is_new is not set, config.save_password
   *                         indicates whether the password should be saved
   */
  User.save = function(config, done_cb, success_cb) {
    var post_data = {
      username,
      first_name,
      last_name,
      is_superuser,
      is_active,
      profile: JSON.stringify(config.profile),
      groups: JSON.stringify(config.groups),
      roles: JSON.stringify(config.roles),
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };

    if (config.is_new) {
      if (password === undefined) {
        arc.report.alertMessage("Cannot create a user without a password");
        if (done_cb) {
          done_cb();
        }
        return;
      }
      post_data.password = password;
    }

    if (config.save_password) {
      post_data.new_password = password;
      post_data.old_password = old_password;
    }

    User.makeRESTCall({
      type: config.is_new ? 'POST' : 'PUT',
      data: post_data,
      idString: config.url_user,
      done_cb: done_cb,
      success_cb: success_cb
    });
  };

  // Config takes:
  //  type: PUT, POST, or DELETE
  //  data
  //  idString: url to be appended to groups_url
  //  done_cb: always run after get response
  //  success_cb: only run after success response
  User.makeRESTCall = function ({ idString, data, type, success_cb, done_cb }) {
    $.ajax({
      url: `${arc.frontend.globals.urls.usersDetailedApiUrl}${idString ? `/${idString}` : ''}`,
      data,
      type,
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      var responseText = req.responseText || "";
      arc.report.alertMessage(
        `Failed to create user: ${textstatus}\n${errorthrown}\n${responseText}`
      );
    })
    .done(success_cb || (() => {}))
    .always(done_cb || (() => {}));
  };

  return User;
};

arc.auth.Group = function() {
  var users = [], roles = [], id, name;

  var Group = function() { return "arc.auth.Group"; };

  Group.users = function(_) {
    if (!arguments.length) return users;
    users = _;
    return Group;
  };

  Group.roles = function(_) {
    if (!arguments.length) return roles;
    roles = _;
    return Group;
  };

  Group.groupId = function(_) {
    if (!arguments.length) return id;
    id = _;
    return Group;
  };

  Group.groupName = function(_) {
    if (!arguments.length) return name;
    name = _;
    return Group;
  };

  Group.delete = function(done_cb, success_cb, group_list) {
    let id_string = '';
    const data = {};
    if (group_list) {
      data.groups = JSON.stringify(group_list);
    } else {
      id_string = `/${id}`;
    }

    Group.makeRESTCall({
      type: 'DELETE',
      data,
      idString: id_string,
      done_cb,
      success_cb
    });
  };

  Group.save = function(done_cb, success_cb) {
    var post_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
      "id": id,
      "name": name,
      "users": JSON.stringify(users),
      "roles": JSON.stringify(roles)
    };
    Group.makeRESTCall({
      type: id ? 'PUT' : 'POST',
      data: post_data,
      idString: id ? "/" + id : "",
      done_cb: done_cb,
      success_cb: success_cb
    });
  };

  // Config takes:
  //  type: PUT, POST, or DELETE
  //  data
  //  idString: url to be appended to groups_url
  //  done_cb: always run after get response
  //  success_cb: only run after success response
  Group.makeRESTCall = function (config) {
    $.ajax({
      url: arc.frontend.globals.urls.groupsUrl + config.idString,
      data: config.data,
      type: config.type,
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      const reqTypeToErrVerb = {
        PUT: 'edit',
        POST: 'create',
        DELETE: 'delete'
      };
      arc.report.alertMessage(`Failed to ${reqTypeToErrVerb[config.type]} group: `,
        req.responseText ? req.responseText : `${textstatus}. ${errorthrown}`);
    })
    .done((g) => {
      if (config.success_cb) {
        config.success_cb(g);
      }
    })
    .always((g) => {
      if (config.done_cb) {
        config.done_cb(g);
      }
    });
  };

  return Group;
};

}());
