window.arc = window.arc || {};

(function() {
  'use strict';

  arc.roles = {};

  // Set up for new role, overwritten when editing existing role
  arc.roles.current_role = {
    name: '',
    desc: '',
    // Need one privilege to keep the table colapsing and looking weird
    privs: [{
      ptype: 'system',
      perms: [],
    }],
    users: [],
    groups: [],
    sentry_groups: [],
  };

  arc.roles.saved_roledata = JSON.stringify(arc.roles.current_role);

  // Callback for manually added user name
  arc.roles.easyAddUser = function(val) {
    val = val.trim();
    const { roles } = arc;
    const user = {
      username: val,
      first_name: '',
      last_name: '',
      modalId: roles.usersList.length + 1
    };
    roles.usersList.push(user);
    roles.usersByName[val] = user;
    arc.roles.userSelectionWidget.selectionField('addSelectionToUnselected', [{ name: user.username, id: user.modalId }]);
  };

  // Callback for manually added group name
  arc.roles.easyAddGroup = function(val) {
    val = val.trim();
    const { roles } = arc;
    const group = {
      name: val,
      modalId: roles.groupsList.length + 1
    };
    roles.groupsList.push(group);
    roles.groupsByName[val] = group;
    arc.roles.groupSelectionWidget.selectionField('addSelectionToUnselected', [{ name: group.name, id: group.modalId }]);
  };

  arc.roles.clickUsers = function() {
    const role = arc.roles.current_role;
    const users = [];
    role.users.forEach((name) => {
      const user = arc.roles.usersByName[name];
      users.push({ name: user.username, id: user.modalId });
    });

    arc.roles.userSelectionWidget.selectionField('setPreviousSelections', users);
    $('#role-user-modal').modal('toggle');
  };

  arc.roles.clickGroups = function() {
    const role = arc.roles.current_role;
    const groups = [];
    role.groups.forEach((name) => {
      const group = arc.roles.groupsByName[name];
      groups.push({ name: group.name, id: group.modalId });
    });

    arc.roles.groupSelectionWidget.selectionField('setPreviousSelections', groups);
    $('#role-group-modal').modal('toggle');
  };

  arc.roles.clickRequireAllGroups = function() {
    const role = arc.roles.current_role;
    role.require_all_groups = $(this).is(':checked');
    arc.report.pagenav.needs_save(true);
  };

  arc.roles.clickSaveMembers = function() {
    const modal = $(this).closest('.modal');
    const role = arc.roles.current_role;

    if (modal[0].id === 'role-user-modal') {
      role.users = [];
      arc.roles.userSelectionWidget
              .selectionField('selected', $('#role-user-selection'))
              .forEach((mi) => {
        if (!role.users.includes(arc.roles.usersList[mi - 1].username)) {
          role.users.push(arc.roles.usersList[mi - 1].username);
        }
      });
    } else {
      role.groups = [];
      arc.roles.groupSelectionWidget
          .selectionField('selected', $('#role-group-selection'))
          .forEach((mi) => {
        if (!role.groups.includes(arc.roles.groupsList[mi - 1].name)) {
          role.groups.push(arc.roles.groupsList[mi - 1].name);
        }
      });
    }

    modal.modal('toggle');
    arc.report.pagenav.needs_save(true);
    arc.roles.loadUsersAndGroups(true);
    arc.roles.enableUndo(true);
  };

  function updateRoleName() {
    const role = arc.roles.current_role;

    role.name = $('#role-name-input').val();
    arc.report.pagenav.needs_save(true);
    $('#role-header-name').text(role.name);
  }

  function updateRoleDesc() {
    const role = arc.roles.current_role;

    role.desc = $('#role-desc-input').val();
    arc.report.pagenav.needs_save(true);
  }

  arc.roles.clickSaveRole = function() {
    const role = Object.assign({}, arc.roles.current_role);
    const roleName = role.name;
    const successCb = (function() {
      return function(response) {
      // If this was a new role creation, then pick up the ID allocated
        if (!role.id) {
          arc.roles.current_role.id = response.data[0].id;
        }

        arc.report.pagenav.needs_save(false);
        arc.roles.saved_roledata = JSON.stringify(arc.roles.current_role);
        arc.roles.drawRoleDetail(false);
        arc.report.alertMessageSmall(`Role '${roleName}' updated.`, 'alert-success');
      };
    }());

    arc.roles.saveRole(role, () => {}, successCb);
    arc.roles.enableUndo(false);
  };

  arc.roles.saveRole = function(role, doneCb, successCb) {
    const url = arc.frontend.globals.urls.rbacRolesUrl;

    $.ajax({
      url,
      data: { data: JSON.stringify([role]) },
      type: 'POST',
      beforeSend(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail((res, textstatus) => {
      arc.report.alertMessage('Error saving role: ', res.responseText || textstatus);
    })
    .done(successCb)
    .always(doneCb);
  };

  arc.roles.enableUndo = function(needsEnable) {
    const undoBtn = $('#undo-role-btn');
    if (needsEnable) {
      undoBtn.removeClass('disabled');
    } else {
      undoBtn.addClass('disabled');
    }
  };

  arc.roles.clickUndoRole = function() {
    arc.roles.current_role = JSON.parse(arc.roles.saved_roledata);
    arc.roles.drawRoleDetail();
    arc.report.pagenav.needs_save(false);
    arc.roles.loadUsersAndGroups(true);
    arc.roles.enableUndo(false);
  };

  // Get the permission updates from within the dataset permissions pane
  arc.roles.getDatasetRoleUpdates = function() {
    // Locate role entries that have changed
    const data = [];
    arc.roles.roleinfo.forEach((ri) => {
      if (ri.origperms && ri.origperms.length !== ri.editpriv.perms.length) {
        data.push({ id: ri.id,
                  dsid: ri.editpriv.dslist[0],
                  perms: ri.editpriv.perms });
      }
    });

    return data;
  };

  // Reset dataset role permissions pane
  arc.roles.clickUndoDatasetRole = function() {
    arc.roles.roleinfo = JSON.parse(arc.roles.saved_roleinfo);
    arc.report.pagenav.needs_save(false);
    arc.roles.drawRoleDataset(false, false);
  };

  // Do the publish function in the dataset role permissions pane
  arc.roles.clickPublishDatasetRole = function() {
    arc.report.pagenav.needs_save(true);
    arc.roles.roleinfo = JSON.parse(arc.roles.saved_roleinfo);
    arc.roles.drawRoleDataset(true, true);
  };

  // Do the edit function in the dataset role permissions pane
  arc.roles.clickEditDatasetRole = function() {
    arc.roles.drawRoleDataset(true, false);
  };

  // Get the permission updates from within the dataset permissions pane
  arc.roles.clickSaveDatasetRole = function() {
    const data = arc.roles.getDatasetRoleUpdates();

    // If nothing to update, treat like 'Undo'
    if (!data.length) {
      arc.roles.clickUndoDatasetRole();
      return;
    }

    const url = `${arc.frontend.globals.urls.rbacRolesUrl}_dspublish`;
    $.ajax({
      url,
      data: { data: JSON.stringify(data) },
      type: 'POST',
      beforeSend(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail((rsp, textstatus) => {
      const err = rsp.responseText || textstatus;
      arc.report.alertMessage('Error updating roles: ', err, { is_html: true });
    })
    .done(() => {
      // Remove where we save the original state, then update saved_roleinfo with current data
      arc.roles.roleinfo.forEach((ri) => {
        delete ri.origperms;
        delete ri.priv;
      });
      arc.roles.saved_roleinfo = JSON.stringify(arc.roles.roleinfo);

      // Draw current data
      arc.report.pagenav.needs_save(false);
      arc.roles.drawRoleDataset(false, false);
      arc.report.alertMessageSmall('Roles updated.', 'alert-success');
    });
  };

  const softForceMap = {
    sys_workspace: ['sys_viewroles'],
  };

  // Build one row of the privilege table
  arc.roles.onePrivRow = function(priv) {
    const { ptype } = priv;
    const permlist = arc.roles.permlists[ptype];
    const startcol = arc.roles.startcols[ptype];
    const endcol = startcol + permlist.length;
    let $tr;

    if (ptype === 'system') {
      $tr = $('#priv-system-tds-template tr').html();
    } else if (ptype === 'role') {
      const rolenames = [];
      priv.rolelist.forEach((rid) => {
        if (arc.roles.rolesById[rid]) {
          rolenames.push(arc.roles.rolesById[rid].name);
        }
      });
      $tr = arc.misc.safe_substitute($('#priv-role-tds-template tr').html(),
      {
        ROLENAMES: rolenames.join(', '),
      });
    } else if (ptype === 'dataconn') {
      // Filter out dataconns that are either deleted or that we don't have access to
      // And skip this privilege if that filtering results in empty set
      const dclist = priv.dclist.filter((dcid) => (
        parseInt(dcid, 10) === -1 || arc.roles.dataconnectionsById[dcid] !== undefined));
      if (dclist.length === 0) {
        return undefined;
      }

      const dcnames = [];
      dclist.forEach((id) => {
        dcnames.push(parseInt(id, 10) === -1 ?
          'All connections' :
          arc.roles.dataconnectionsById[id].name);
      });
      $tr = arc.misc.safe_substitute($('#priv-dataconn-tds-template tr').html(),
      {
        DCNAMES: dcnames.join(', '),
      });
    } else if (ptype === 'dataset') {
      // Skip this privilege if dataconn deleted or unaccessible
      if (parseInt(priv.dcid, 10) !== -1 &&
        arc.roles.dataconnectionsById[priv.dcid] === undefined) {
        return undefined;
      }

      // Now filter datasets like dataconns above
      const dslist = priv.dslist.filter((dsid) => (
        parseInt(dsid, 10) === -1 || arc.roles.datasetsById[dsid] !== undefined));
      if (dslist.length === 0) {
        return undefined;
      }

      const dsnames = [];
      dslist.forEach((id) => {
        dsnames.push(parseInt(id, 10) === -1 ?
          'All datasets' :
          arc.roles.datasetsById[id].name);
      });
      $tr = arc.misc.safe_substitute($('#priv-dataset-tds-template tr').html(),
      {
        DCNAME: parseInt(priv.dcid, 10) === -1 ?
          'All connections' :
          arc.roles.dataconnectionsById[priv.dcid].name,
        DSNAMES: dsnames.join(', '),
      });
    }

    // Get permissions that are forced by other permissions and add these
    // to the full set of working permissions.
    const { perms } = priv;
    const force = [];

    perms.forEach((p) => {
      const dependentPermissions = arc.roles.permlists[p] || [];
      dependentPermissions.forEach((permission) => {
        if (!force.includes(permission)) {
          force.push(permission);
        }

        if (!perms.includes(permission)) {
          perms.push(permission);
        }
      });
    });

    $tr = $(`<tr>${$tr}</tr>`);
    $tr.data('priv', priv);

    function disabledAtt() {
      if (arc.globals.is_superuser || arc.globals.perms.sysperms.includes('sys_editperm')) {
        return '';
      }

      return 'disabled';
    }

    const unused = '<td class="exclude-cell">&nbsp;</td>';
    const checked = `<td class="%CLASS%"><input ${disabledAtt()} class="role-perms-one" perm="%PERM%" type="checkbox" checked></td>`;
    const unchecked = `<td class="%CLASS%"><input ${disabledAtt()} class="role-perms-one" perm="%PERM%" type="checkbox"></td>`;
    const allChecked = `<td><input ${disabledAtt()} class="role-perms-all" type="checkbox" checked></td>`;
    const allUnchecked = `<td><input ${disabledAtt()} class="role-perms-all" type="checkbox"></td>`;
    $tr.append(perms.length === permlist.length ? allChecked : allUnchecked);

    for (let i = arc.roles.startcols.permstart; i < arc.roles.startcols.permend; i++) {
      let td = unused;
      if (i >= startcol && i < endcol) {
        const perm = permlist[i - startcol];
        if (perms.indexOf(perm) === -1) {
          td = unchecked;
        } else {
          td = checked;
        }

        td = arc.misc.safe_substitute(td, {
          PERM: perm,
          CLASS: `priv-${ptype}`,
        });
      }

      $tr.append(td);
    }

    if (ptype !== 'system') {
      $tr.append($('#priv-edit-tail-td-template tr').html());
    } else {
      $tr.append($('#priv-tail-td-template tr').html());
    }

    return $tr;
  };

  function getRow(obj, type) {
    const fields = (type === 'user') ?
        {
          USER_NAME: obj.username,
          FIRST_NAME: obj.first_name,
          LAST_NAME: obj.last_name,
        } :
        {
          GROUP_NAME: obj.name,
        };

    const row = arc.misc.safe_substitute($(`#role-${(type === 'sentry_group') ? 'group' : type}-list-template tbody`).html(), fields);
    const $row = $(row);
    $row.data(`${(type === 'user') ? 'username' : 'name'}`, `type.${(type === 'users' ? 'username' : 'name')}`);

    return $row;
  }

  arc.roles.drawUsersAndGroups = function() {
    const role = arc.roles.current_role;
    const dataTablesOptions = {
      paging: true,
      pagingType: 'full_numbers',
      destroy: true,
      searching: true,
      ordering: false
    };

    const $members = $('#role-users');
    const $membersDataTable = $members.DataTable(dataTablesOptions);
    $membersDataTable.clear();

    role.users.forEach((user) => {
      $membersDataTable.rows.add(getRow(arc.roles.usersByName[user], 'user'));
    });

    $membersDataTable.draw();

    const $groups = $('#role-groups');
    const $groupsDataTable = $groups.DataTable(dataTablesOptions);
    $groupsDataTable.clear();

    role.groups.forEach((group) => {
      $groupsDataTable.rows.add(getRow(arc.roles.groupsByName[group], 'group'));
    });

    $groupsDataTable.draw();

    if (role.sentry_groups.length === 0) {
      $('#role-sentry-groups-wrapper').hide();
    } else {
      /*
        If we have a role from sentry (i.e. where the sentry_groups is not empty),
        then we should not have any users or groups.  There's a funky situation
        where a group could have been created with a name exactly matching the
        sentry name and there could be user/groups already present.

        This should not happen but here we check for users/groups empty before
        hidding those elements.
      */
      if (role.users.length === 0 && role.groups.length === 0) {
        $('#role-users-wrapper').hide();
        $('#role-groups-wrapper').hide();
      }

      const $sentry_groups = $('#role-sentry-groups');
      const $sentry_groupsDataTable = $sentry_groups.DataTable(dataTablesOptions);
      $sentry_groupsDataTable.clear();

      role.sentry_groups.forEach((sentry_group) => {
        const $sentry_group = arc.roles.groupsByName[sentry_group];
        $sentry_groupsDataTable.rows.add(getRow($sentry_group, 'sentry_group'));
      });

      $sentry_groupsDataTable.draw();
      $('#role-sentry-groups-wrapper').show();
    }

    // Set the inital state of the 'Require all groups' checkbox
    $('#role-require-all-groups').prop('checked', role.require_all_groups);
  };

  // Draw details for the active role
  arc.roles.drawRoleDetail = function() {
    const role = arc.roles.current_role;

    $('#role-header-name').text(role.name);
    $('#role-name-input').val(role.name);
    $('#role-desc-input').val(role.desc);

    arc.roles.drawPriviliges();
  };

  arc.roles.drawPriviliges = function() {
    const { privs } = arc.roles.current_role;
    const tbody = $('#role-priv-table tbody');
    tbody.empty();

    arc.roles.current_role_has_system_priv = false;
    ['system', 'role', 'dataconn', 'dataset'].forEach((ptype) => {
      privs.filter((p) => p.ptype === ptype).forEach((p) => {
        if (!arc.roles.current_role_has_system_priv) {
          arc.roles.current_role_has_system_priv = (ptype === 'system');
        }

        const tr = arc.roles.onePrivRow(p);
        if (tr) {
          tbody.append(tr);
        }
      });
    });
  };

  // Build one row of the dataset privilege table
  arc.roles.oneDatasetPrivRow = function(ds, ri, priv, info) {
    const ptype = 'dataset';
    const permlist = arc.roles.permlists[ptype];
    const startcol = arc.roles.startcols[ptype];
    const endcol = startcol + permlist.length;
    let tr;
    let all;
    let can_edit = false;

    if (priv.ptype !== ptype) {
      return undefined;
    }

    if (priv.dcid === '-1') {
      all = 'All connections / All datasets';
    } else if (priv.dcid !== ds.dataconnection_id().toString()) {
      return undefined;
    } else if (priv.dslist.indexOf('-1') !== -1) {
      all = `${ds.dataconnection_name()} / All datasets`;
    } else if (priv.dslist.indexOf(ds.dataset_id().toString()) !== -1) {
      // Don't show empty privilege rows when not in edit mode
      if (!priv.perms.length && !info.editmode) {
        return undefined;
      }

      info.found_ds = true;
      if (priv.dslist.length > 1) {
        all = `${ds.dataconnection_name()} / Multiple datasets`;
      } else {
        info.got_single = true;

        all = `${ds.dataconnection_name()} / ${ds.dataset_name()}`;
        can_edit = info.can_edit && info.editmode;

        // Here we track the original state of the permission entry so that on save, we only
        // update those that have changed
        if (ri.origperms === undefined) {
          ri.origperms = priv.perms.slice();
          ri.editpriv = priv;
        }

        // If we are on the 'publishing' path, then we set up the initial permissions
        // to the full extend that we are allowed.
        if (info.publishing) {
          priv.perms = arc.globals.perms.permlists.dataset
                          .filter((perm) => arc.misc.hasPerm(`rg${perm}`, ri.perms));
        }
      }
    } else {
      return undefined;
    }

    let roletext = ri.name;
    if (arc.misc.hasPerm('sys_editperm', arc.globals.perms.sysperms)) {
      roletext = arc.misc.appsToLinks([ri.id], 2).replace('/app/', '/role_detail/')
                                                .replace('href=', 'target="_blank" href=')
                                                .replace(`${ri.id}</a>`, `${ri.name}</a>`);
    }

    tr = arc.misc.safe_substitute($('#priv-dataset-tds-template tr').html(),
    {
      HTML_HREF: roletext,
      PRIV_TYPE: all,
    });

    // Get permissions that are forced by other permissions and add these
    // to the full set of working permissions.
    const { perms } = priv;
    const force = [];

    perms.forEach((p) => {
      const flist = arc.roles.permlists[p] || [];
      flist.forEach((fperm) => {
        if (!force.includes(fperm)) {
          force.push(fperm);
        }

        if (!perms.includes(fperm)) {
          perms.push(fperm);
        }
      });
    });

    tr = $(`<tr>${tr}</tr>`);
    tr.data('priv', priv);
    tr.data('role', ri);

    const unused = '<td class="exclude-cell">&nbsp;</td>';
    const checked = '<td class="%CLASS%"><input class="role-perms-one" perm="%PERM%" type="checkbox" checked></input></td>';
    const unchecked = '<td class="%CLASS%"><input class="role-perms-one" perm="%PERM%" type="checkbox"></input></td>';
    const all_checked = '<td><input class="role-perms-all" type="checkbox" checked></input></td>';
    const all_unchecked = '<td><input class="role-perms-all" type="checkbox"></input></td>';
    tr.append(perms.length === permlist.length ? all_checked : all_unchecked);

    for (let i = arc.roles.startcols.permstart; i < arc.roles.startcols.permend; i++) {
      let td = unused;
      if (i >= startcol && i < endcol) {
        const perm = permlist[i - startcol];
        if (perms.indexOf(perm) === -1) {
          td = unchecked;
        } else {
          td = checked;
        }

        td = arc.misc.safe_substitute(td, {
          PERM: perm,
          CLASS: `priv-${ptype}`,
        });
      }

      tr.append(td);
    }

    tr.append(unused);

    // Turn off inputs which can not be changed
    if (!can_edit) {
      tr.find('input').attr('disabled', true);
      if (info.editmode) {
        tr.attr('datatoggle', 'tooltip');
        tr.attr('title',
                info.can_edit ? 'This privilege row must be edited in the detail page for the role' :
                                'You do not have permission to edit this privilege row');
      }
    } else if (!arc.misc.hasPerm('sys_editperm', arc.globals.perms.sysperms)) {
      // For each dataset permission, check that we have the corresponding grant permission
      arc.globals.perms.permlists.dataset.forEach((perm) => {
        if (!arc.misc.hasPerm(`rg${perm}`, ri.perms)) {
          tr.find('input.role-perms-all').attr('disabled', true);
          tr.find(`input[perm=${perm}]`).attr('disabled', true);
        }
      });
    }

    return tr;
  };

  // Draw dataset role info
  arc.roles.drawRoleDataset = function(editmode, publishing) {
    const tbody = $('#role-dataset-priv-table tbody');
    tbody.empty();

    const ds = arc.roles.dataset;
    const permlist = arc.roles.permlists.role;

    // If we don't have editrole permission but we do have dataset manage permission,
    // then we then order the roles so that we show the ones we can edit first
    if (!arc.misc.hasPerm('sys_editperm', arc.globals.perms.sysperms) &&
        arc.misc.hasPerm('ds_manage', ds.dataset_perms())) {
      arc.roles.roleinfo = arc.roles.roleinfo.sort((a, b) => {
        const a_edit = permlist.filter((p) => arc.misc.hasPerm(p, a.perms)).length ? 1 : 0;
        const b_edit = permlist.filter((p) => arc.misc.hasPerm(p, b.perms)).length ? 1 : 0;
        return (b_edit - a_edit) || (b.id - a.id);
      });
    }

    let did_double = false;
    let edit_count = 0;
    let can_publish = !publishing;

    arc.roles.roleinfo.forEach((ri) => {
      const info = { editmode, publishing };
      info.can_edit = arc.misc.hasPerm('ds_manage', ds.dataset_perms()) &&
                      (arc.misc.hasPerm('sys_editperm', arc.globals.perms.sysperms) ||
                      arc.misc.hasPerm('rgds_appview', ri.perms));
      if (info.can_edit) {
        edit_count += 1;
      }

      // Paint the privilege rows for this role that name the dataset
      ri.privs.forEach((p) => {
        const tr = arc.roles.oneDatasetPrivRow(ds, ri, p, info);
        if (tr) {
          // If we have some roles that can be edited, then for the first line
          // on a role that can not be edited, we add a style
          if (edit_count > 0 && !info.can_edit && editmode && !did_double) {
            tr.attr('style', 'border-top: solid 3px #999;');
            did_double = true;
          }

          tbody.append(tr);
        }
      });

      // If we found the dataset already named in any role, then kill
      // off the publish button capability
      if (info.found_ds) {
        can_publish = false;
      }

      // If we can edit this role but the dataset is not named in an individual row
      // then force the individual row here.
      if (info.can_edit && !info.got_single && editmode) {
        const p = { ptype: 'dataset',
                dcid: ds.dataconnection_id().toString(),
                dslist: [ds.dataset_id().toString()],
                perms: [] };

        ri.privs.push(p);
        tbody.append(arc.roles.oneDatasetPrivRow(ds, ri, p, info));
      }
    });

    // The permissions page has a bunch of combinations depending on if the dataset
    // is named in any role and what permissions the user has.
    // Start out hiding all text and buttons
    $('#edit-dsperm-btn').addClass('arc-invisible');
    $('#save-dsperm-btn').addClass('arc-invisible');
    $('#undo-dsperm-btn').addClass('arc-invisible');
    $('#publish-dsperm-btn').addClass('arc-invisible');
    $('#dataset-permissions-roles').addClass('arc-invisible');
    $('#dataset-permissions-pubtext').addClass('arc-invisible');
    $('#dataset-permissions-edittext').addClass('arc-invisible');
    $('#dataset-permissions-noedittext').addClass('arc-invisible');

    // Only show the permissions table if there are some rows
    const rowcount = tbody.children().length;
    if (rowcount) {
      $('#dataset-permissions-roles').removeClass('arc-invisible');
    }

    // Here if the dataset is not named in any role but this user has permission to
    // edit some role entries (i.e. those created above in the !info.got_single case),
    // we can show the 'Publish' option

    // But...we only do this for users who do not have the sys_editperm permission
    // since publish in that case would put the dataset into every role with full
    // access and that seems excessive.
    if (edit_count && can_publish && !editmode &&
        !arc.misc.hasPerm('sys_editperm', arc.globals.perms.sysperms)) {
      $('#dataset-permissions-pubtext').removeClass('arc-invisible');
      $('#publish-dsperm-btn').removeClass('arc-invisible');
      $('#edit-dsperm-btn').removeClass('arc-invisible');
      return;
    }

    // If there is something we can edit, show buttons according to edit/view
    if (edit_count) {
      if (!editmode) {
        if (!rowcount) {
          $('#dataset-permissions-edittext').removeClass('arc-invisible');
        }

        $('#edit-dsperm-btn').removeClass('arc-invisible');
      } else {
        $('#save-dsperm-btn').removeClass('arc-invisible');
        $('#undo-dsperm-btn').removeClass('arc-invisible');
      }
    } else if (!rowcount) {
      $('#dataset-permissions-noedittext').removeClass('arc-invisible');
    }
  };

  function sortRoles(roles) {
    roles.sort((a, b) => {
      if (a.name.toLowerCase() < b.name.toLowerCase()) {
        return -1;
      }

      if (a.name.toLowerCase() > b.name.toLowerCase()) {
        return 1;
      }

      return 0;
    });
  }

  // Draw roles summary
  arc.roles.drawSummary = function() {
    const tbody = $('#role-summary-table tbody');
    tbody.empty();

    sortRoles(arc.roles.roleinfo);
    arc.roles.roleinfo.forEach((ri) => {
      const tr = arc.misc.safe_substitute($('#role-summary-template').html(),
        {
          ROLE_NAME: ri.role,
          ROLE_DESC: ri.desc,
          PERM_COUNT: ri.perms.length,
          USER_COUNT: ri.users.length,
          GROUP_COUNT: ri.groups.length,
        });

      tbody.append(tr);
    });
  };

  // Update privilege role after click
  arc.roles.updatePrivRow = function(tr) {
    if (!arc.roles.dataset) {
      arc.report.pagenav.needs_save(true);
      tr.replaceWith(arc.roles.onePrivRow(tr.data('priv')));
    } else {
      tr.replaceWith(arc.roles.oneDatasetPrivRow(arc.roles.dataset, tr.data('role'), tr.data('priv'), { can_edit: true, editmode: true }));
      arc.report.pagenav.needs_save(arc.roles.getDatasetRoleUpdates().length);
    }
  };

  // Handle click on 'all' checkbox one privilege row
  arc.roles.clickPermsAll = function() {
    const checked = $(this).is(':checked');
    const tr = $(this).closest('tr');

    // Update the permissions for this privilege
    const priv = tr.data('priv');
    if (checked) {
      priv.perms = arc.roles.permlists[priv.ptype].slice();
    } else {
      priv.perms = [];
    }

    arc.roles.updatePrivRow(tr);
    arc.roles.enableUndo(true);
  };

  // Handle click on a single permission checkbox on a privilege row
  arc.roles.clickPermsOne = function() {
    const isChecked = $(this).is(':checked');
    const perm = $(this).attr('perm');

    const $td = $(this).closest('td');
    const $tr = $td.closest('tr');
    const priv = $tr.data('priv');
    const permissionIndex = priv.perms.indexOf(perm);
    const softForcePerms = softForceMap[perm];

    // Update the permissions for this privilege
    if (isChecked) {
      if (permissionIndex === -1) {
        priv.perms.push(perm);
        if (softForcePerms !== undefined) {
          priv.perms.push(...softForcePerms);
        }
      }
    } else if (permissionIndex !== -1) {
        priv.perms.splice(permissionIndex, 1);
    }

    arc.roles.updatePrivRow($tr);
    arc.roles.enableUndo(true);
  };

  // Handle click on a single permission checkbox on a privilege row
  arc.roles.clickPermsTrash = function() {
    const tr = $(this).closest('tr');
    const m = $('#delete-privilege-modal');
    if (arc.misc.modalShown(m)) {
      return;
    }

    m.find('#delete-privilege-yes').off().on('click', () => {
      const r = arc.roles.current_role;
      const index = r.privs.indexOf(tr.data('priv'));
      if (index !== -1) {
        r.privs.splice(index, 1);
      }

      arc.report.pagenav.needs_save(true);
      arc.roles.drawRoleDetail();
      m.modal('toggle');
    });
    m.modal('show');
    arc.roles.enableUndo(true);
  };

  // Handle change of component select in the new privilege modal
  arc.roles.selectComponent = function() {
    const pm = $('#privilege-modal');
    const ptype = pm.find('.component-select').val();

    if (ptype === 'system') {
      pm.find('.connection-multi-select').hide();
      pm.find('.connection-single-select').hide();
      pm.find('.dataset-select').hide();
      pm.find('.role-multi-select').hide();
    }

    if (ptype === 'dataconn') {
      pm.find('.connection-multi-select').fadeIn();
      pm.find('.connection-single-select').hide();
      pm.find('.dataset-select').hide();
      pm.find('.role-multi-select').hide();

      // Reset the selections
      const csm = pm.find('.connection-multi-select select');
      csm.find('option').removeAttr('selected').end().trigger('chosen:updated');
    }

    if (ptype === 'dataset') {
      pm.find('.connection-multi-select').hide();
      pm.find('.connection-single-select').show();
      pm.find('.dataset-select').fadeIn();
      pm.find('.role-multi-select').hide();

      // Reset the selection
      const css = pm.find('.connection-single-select select');
      css.val('');
      css.trigger('chosen:updated');

      const dsm = pm.find('.dataset-select select');
      dsm.find('option').removeAttr('selected').end().trigger('chosen:updated');
    }

    if (ptype === 'role') {
      pm.find('.connection-multi-select').hide();
      pm.find('.connection-single-select').hide();
      pm.find('.dataset-single-select').hide();
      pm.find('.dataset-select').hide();
      pm.find('.role-multi-select').fadeIn();

      // Reset the selection
      const csr = pm.find('.role-mult-select select');
      csr.val('');
      csr.trigger('chosen:updated');

      const rsm = pm.find('.role-multi-select select');
      rsm.find('option').removeAttr('selected').end().trigger('chosen:updated');
    }
  };

  // Initialize the new privilege modal
  arc.roles.addPrivilege = function() {
    const pm = $('#privilege-modal');
    pm.find('.role-multi-select').hide();
    pm.find('.connection-multi-select').hide();
    pm.find('.connection-single-select').hide();
    pm.find('.dataset-select').hide();
    pm.find('#priv-error').empty();

    // Initialize the connection single and multi-select lists
    const csr = pm.find('.role-multi-select select');
    const csm = pm.find('.connection-multi-select select');
    const css = pm.find('.connection-single-select select');
    csr.empty();
    csm.empty();
    css.empty();
    csm.append('<option value="-1">All connections</option>');
    css.append('<option value="-1">All connections</option>');

    arc.roles.roleinfo.forEach((role) => {
      csr.append(`<option value="${role.id}">${role.name}</option>`);
    });

    arc.roles.dataconnections
      .sort((a, b) => (a.name.toUpperCase() > b.name.toUpperCase() ? 1 : -1))
      .forEach((d) => {
        csm.append(`<option value="${d.id}">${d.name}</option>`);
        css.append(`<option value="${d.id}">${d.name}</option>`);
      });

    const cs = pm.find('.component-select');
    if (arc.roles.current_role_has_system_priv) {
      cs.find('[value=system]').hide();
      cs.val('dataconn');
    } else {
      cs.find('[value=system]').show();
      cs.val('system');
    }

    cs.trigger('chosen:updated');

    arc.roles.selectComponent();
    pm.modal('toggle');
  };

  // Update the list of datasets according to data connection selection
  arc.roles.updateDatasets = function() {
    const pm = $('#privilege-modal');

    const dcid = pm.find('.connection-single-select select').val();

    const dsm = pm.find('.dataset-select select');
    dsm.empty();
    dsm.append('<option value="-1">All datasets</option>');
    if (parseInt(dcid, 10) === -1) {
      dsm.find('option').attr('selected', true).end().trigger('chosen:updated');
    } else {
      arc.roles.datasets
        .sort((a, b) => (a.name.toUpperCase() > b.name.toUpperCase() ? 1 : -1))
        .forEach((d) => {
          if (d.dc_id === parseInt(dcid, 10)) dsm.append(`<option value="${d.id}">${d.name} (ID: ${d.id})</option>`);
        });
      dsm.find('option').removeAttr('selected').end().trigger('chosen:updated');
    }
  };

  // Handle click of the Add Privilege modal 'Create' button
  arc.roles.clickSavePrivilege = function() {
    const pm = $('#privilege-modal');
    const ptype = pm.find('.component-select').val();

    const priv = { ptype,
                perms: arc.roles.permlists[ptype].slice() };

    let error;
    if (ptype === 'system') {
      // No check
    }

    if (ptype === 'role') {
      const csr = pm.find('.role-multi-select select');
      if (!csr.val()) {
        error = "For privilege type 'Role', you must select at least one role";
      } else {
        priv.rolelist = csr.val();
      }
    }

    if (ptype === 'dataconn') {
      const csm = pm.find('.connection-multi-select select');
      if (!csm.val()) {
        error = "For privilege type 'Connection', you must select at least one data connection";
      } else {
        priv.dclist = csm.val();
      }
    }

    if (ptype === 'dataset') {
      const css = pm.find('.connection-single-select select');
      const dsm = pm.find('.dataset-select select');
      if (!css.val()) {
        error = "For privilege type 'Dataset', you must first specify a data connection then select at least one dataset for the selected connection";
      } else if (!dsm.val()) {
        error = "For privilege type 'Dataset', you must select least one dataset for the specified connection";
      } else {
        priv.dcid = css.val();
        priv.dslist = dsm.val();
      }
    }

    if (error) {
      pm.find('#priv-error').empty().append(arc.misc.safe_substitute(
        $('#priv-error-template').html(), { ERROR: error }
      ));
    } else {
      pm.modal('toggle');
      arc.roles.current_role.privs.push(priv);
      arc.roles.drawRoleDetail();
      arc.report.pagenav.needs_save(true);
      arc.roles.enableUndo(true);
    }
  };

  // Load the detail data for role edit / creation
  arc.roles.loadRolesDatasetsAndDataconnections = function(role_id, cb) {
    const q = d4.queue().defer(d3.json, arc.frontend.globals.urls.datasetUrl)
                  .defer(d3.json, arc.frontend.globals.urls.dataconnectionUrl)
                  .defer(d3.json, arc.frontend.globals.urls.rbacRolesUrl);

    q.await((error, datasets, dataconnections, roles) => {
      if (error) {
        arc.report.alertMessage(`Error loading data: ${error}`);
        return;
      }

      const r = arc.roles;
      console.log('r: ', r);
      r.roleinfo = roles;
      r.rolesById = {};
      r.roleinfo.forEach((role) => { r.rolesById[role.id] = role; });
      if (role_id !== undefined) {
        r.current_role = r.rolesById[role_id];
        r.saved_roledata = JSON.stringify(r.current_role);
        if (!r.current_role) {
          arc.report.alertMessage(`Role ${role_id} not found`);
          return;
        }
      }

      r.datasets = datasets;
      r.dataconnections = dataconnections;
      r.usersByName = {};
      r.groupsByName = {};

      r.dataconnectionsById = {};
      r.dataconnections.forEach((d) => {
        r.dataconnectionsById[d.id] = d;
      });

      r.datasetsById = {};
      r.datasets.forEach((d) => {
        r.datasetsById[d.id] = d;
      });

      // Parse response into role info...
      const tr = $('#role-priv-table thead tr');
      const template = $('#role-priv-th-template tr').html();
      const sys_misc_template = `<th class="rotate">
        <div class="%CLASS%"><span>%DESC%</span><a class="sys_misc_link">*</a></div>
      </th>`;

      r.startcols = {};
      r.startcols.permstart = tr.children().length;
      r.startcols.system = tr.children().length;
      r.permlists.system.forEach((p) => {
        console.log('p: ', p);
        if (p === 'sys_misc') {
          tr.append(arc.misc.safe_substitute(sys_misc_template, {
            CLASS: 'priv-system',
            DESC: r.permdefs[p],
          }));
        } else {
          tr.append(arc.misc.safe_substitute(template, {
            CLASS: 'priv-system',
            DESC: r.permdefs[p],
          }));
        }
      });

      r.startcols.role = tr.children().length;
      r.permlists.role.forEach((p) => {
        tr.append(arc.misc.safe_substitute(template, {
          CLASS: 'priv-role',
          DESC: r.permdefs[p],
        }));
      });

      r.startcols.dataconn = tr.children().length;
      r.permlists.dataconn.forEach((p) => {
        tr.append(arc.misc.safe_substitute(template, {
          CLASS: 'priv-dataconn',
          DESC: r.permdefs[p],
        }));
      });

      r.startcols.dataset = tr.children().length;
      r.permlists.dataset.forEach((p) => {
        tr.append(arc.misc.safe_substitute(template, {
          CLASS: 'priv-dataset',
          DESC: r.permdefs[p],
        }));
      });

      r.startcols.permend = tr.children().length;
      tr.append('<th>&nbsp;</th>');

      cb();
    });
  };

  let hasUserAndGroupLoaded = false;
  arc.roles.loadUsersAndGroups = function(needsUpdate) {
    if (!hasUserAndGroupLoaded || needsUpdate) {
      const q = d4.queue().defer(d3.json, `${arc.frontend.globals.urls.usersDetailedApiUrl}?summary=true`)
                      .defer(d3.json, arc.frontend.globals.urls.groupsUrl);

      q.await((error, users, groups) => {
        if (error) {
          arc.report.alertMessage(`Error loading data: ${error}`);
          return;
        }

        const r = arc.roles;

        r.usersList = users;
        r.groupsList = groups;
        r.usersByName = {};
        r.groupsByName = {};

        // We assume role user names not present in the system user list were
        // manually added to the role so we construct objects for them here.
        users.forEach((u, idx) => {
          u.modalId = idx + 1;
          r.usersByName[u.username] = u;
        });
        r.current_role.users.forEach((name) => {
          if (r.usersByName[name] === undefined) {
            const u = { username: name,
                    first_name: '',
                    last_name: '',
                    modalId: users.length + 1 };
            users.push(u);
            r.usersByName[name] = u;
          }
        });

        const usel = [];
        users.forEach((u) => {
          usel.push({ name: u.username, id: u.modalId });
        });

        r.userSelectionWidget =
          $('#role-user-selection.form-group').selectionField({
            selectionList: usel,
            easyAdd: {
              addCb: arc.roles.easyAddUser,
              buttonLabel: 'Add',
              placeholder: 'Enter new user name',
            },
          });

        // Add in the manufactured 'Everyone' group if defined.
        if (arc.globals.perms.everyone_group) {
          groups.splice(0, 0, { name: arc.globals.perms.everyone_group, id: -1 });
        }

        // Like users, need to set up for the manually added group names
        groups.forEach((g, idx) => {
          g.modalId = idx + 1;
          r.groupsByName[g.name] = g;
        });
        r.current_role.groups.forEach((name) => {
          if (r.groupsByName[name] === undefined) {
            const g = { name,
                    modalId: groups.length + 1 };
            groups.push(g);
            r.groupsByName[name] = g;
          }
        });

        const gsel = [];
        groups.forEach((g) => {
          gsel.push({ name: g.name, id: g.modalId });
        });
        r.groupSelectionWidget =
          $('#role-group-selection.form-group').selectionField({
            selectionList: gsel,
            easyAdd: {
              addCb: arc.roles.easyAddGroup,
              buttonLabel: 'Add',
              placeholder: 'Enter new group name',
            },
          });

        arc.roles.drawUsersAndGroups();

        hasUserAndGroupLoaded = true;
      });
    }
  };

  arc.roles.roleDetailConfig = function() {
    const config = function() { return 'arc.roles.config'; };

    config.initialize = function($div, role_id) {
      // Load the permission definitions and lists for each type that were passed to
      // us by the server
      const r = arc.roles;
      r.permdefs = arc.globals.perms.permdefs;
      r.permlists = arc.globals.perms.permlists;

      // Set up click handlers
      const privtab = $('#role-priv-table');
      privtab.on('click', '.role-perms-trash', arc.roles.clickPermsTrash);
      privtab.on('click', '.role-perms-one', arc.roles.clickPermsOne);
      privtab.on('click', '.role-perms-all', arc.roles.clickPermsAll);

      $('#save-privilege-btn').on('click', arc.roles.clickSavePrivilege);

      $('#role-name-input').on('input', arc.Stream.debounce(updateRoleName));
      $('#role-desc-input').on('input', arc.Stream.debounce(updateRoleDesc));

      const privmod = $('#privilege-modal');
      privmod.on('change', '.component-select', arc.roles.selectComponent);
      privmod.on('change', '.connection-single-select select', arc.roles.updateDatasets);

      $('#role-add-privilege').on('click', arc.roles.addPrivilege);

      $('#role-edit-users').on('click', arc.roles.clickUsers);
      $('#role-edit-groups').on('click', arc.roles.clickGroups);
      $('#role-require-all-groups').on('click', arc.roles.clickRequireAllGroups);
      $('[arcid=role-user-save], [arcid=role-group-save]').on('click', arc.roles.clickSaveMembers);
      $('#save-role-btn').on('click', arc.roles.clickSaveRole);
      $('#undo-role-btn').on('click', arc.roles.clickUndoRole);
      $('a[href$="members"]').on('click', () => {
        arc.roles.loadUsersAndGroups();
      });

      // Set up chosen support
      $('.chosen-select').chosen({
        search_contains: true,
      });
      $('.chosen-select-nosearch').chosen({
        disable_search: true
      });

      arc.roles.loadRolesDatasetsAndDataconnections(role_id, arc.roles.drawRoleDetail);

      return config;
    };

    return config;
  };

  // -- For role update in dataset page -- //

  // Load the detail data for roles update for a particular dataset
  arc.roles.loadDataset = function(cb) {
    const q = d4.queue().defer(d3.json, arc.frontend.globals.urls.rbacRolesUrl);

    q.await((error, roles) => {
      let msg;
      if (error) {
        msg = `Error loading data: ${error.responseText || error.statusText}`;
      } else if (roles.length === 0) {
        msg = 'There are no roles defined on this system';
      }

      if (msg) {
        $('#dataset-permissions-no-roles').append(`<h5>${msg}</h5>`);
        $('#dataset-permissions-no-roles').removeClass('arc-invisible');
        return;
      }

      $('#dataset-permissions-roles').removeClass('arc-invisible');

      const r = arc.roles;
      r.roleinfo = roles;
      r.saved_roleinfo = JSON.stringify(roles);

      // Paint the table header...
      const tr = $('#role-dataset-priv-table thead tr');
      tr.find('th.rotate').remove();
      const template = $('#role-dataset-priv-th-template tr').html();

      r.startcols = {};
      r.startcols.permstart = tr.children().length;

      r.startcols.dataset = tr.children().length;
      r.permlists.dataset.forEach((p) => {
        tr.append(arc.misc.safe_substitute(template, {
          CLASS: 'priv-dataset',
          DESC: r.permdefs[p],
        }));
      });

      r.startcols.permend = tr.children().length;

      cb();
    });
  };

  arc.roles.roleDatasetConfig = function() {
    const config = function() { return 'arc.roles.config'; };

    config.initialize = function($div, dataset) {
      // Load the permission definitions and lists for each type that were passed to
      // us by the server
      const r = arc.roles;
      r.permdefs = arc.globals.perms.permdefs;
      r.permlists = arc.globals.perms.permlists;
      r.dataset = dataset;

      // Set up click handlers
      const privtab = $('#role-dataset-priv-table');
      privtab.off();
      privtab.on('click', '.role-perms-one', arc.roles.clickPermsOne);
      privtab.on('click', '.role-perms-all', arc.roles.clickPermsAll);
      $('#save-dsperm-btn').off().on('click', arc.roles.clickSaveDatasetRole);
      $('#undo-dsperm-btn').off().on('click', arc.roles.clickUndoDatasetRole);
      $('#publish-dsperm-btn').off().on('click', arc.roles.clickPublishDatasetRole);
      $('#edit-dsperm-btn').off().on('click', arc.roles.clickEditDatasetRole);
      arc.roles.loadDataset(() => { arc.roles.drawRoleDataset(false, false); });

      return config;
    };

    return config;
  };

  // --- Role summary code

  arc.roles.clickSummaryRow = function() {
    const index = $(this).closest('tr').attr('role_index');
    window.location.href = `role_detail/${arc.roles.roleinfo[index].id}`;
  };

  arc.roles.clickSummaryTrash = function() {
    const index = $(this).closest('tr').attr('role_index');
    const m = $('#delete-role-modal');
    if (arc.misc.modalShown(m)) {
      return;
    }

    m.find('#delete-role-yes').off().on('click', () => {
      m.modal('toggle');

      const url = arc.frontend.globals.urls.rbacRolesUrl;
      const role = arc.roles.roleinfo[index];
      const { name } = role;

      $.ajax({
        url: `${url}/${role.id}`,
        data: { data: JSON.stringify([role]) },
        type: 'DELETE',
        beforeSend(xhr) {
          // Because of a jquery bug when handling PUT requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
      .fail((res, textstatus) => {
        arc.report.alertMessage(`Failed to delete role "${name}": `, res.responseText || textstatus);
      })
      .done(() => {
        arc.roles.roleinfo.splice(index, 1);
        arc.roles.drawSummary();
        arc.report.alertMessageSmall(`Role '${name}' deleted.`, 'alert-success');
      });
    });
    m.modal('show');
    return false;
  };

  arc.roles.drawSummary = function() {
    const table = $('#role-summary-table');
    const tbody = table.find('tbody');
    // TODO: MOVE TO DATATABLE

    const template = $('#role-summary-template tbody').html();
    tbody.empty();

    sortRoles(arc.roles.roleinfo);
    arc.roles.roleinfo.forEach((role, index) => {
      const { privs } = role;
      const hasEmptySystemPriv = privs.some(
        (priv) => priv.ptype === 'system' && priv.perms.length === 0
      );
      tbody.append(arc.misc.safe_substitute(template,
        { ROLE_ID: role.id,
        ROLE_NAME: role.name,
        ROLE_DESC: role.desc,
        ROLE_INDEX: index,
        PRIV_COUNT: hasEmptySystemPriv ? privs.length - 1 : privs.length,
        USER_COUNT: role.users.length,
        GROUP_COUNT: role.groups.length + role.sentry_groups.length, }));
    });
  };

  // Load the role summary data
  arc.roles.loadSummary = function(cb) {
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.rbacRolesUrl)
      .await((error, roles) => {
        if (error) {
          // eslint-disable-next-line no-console
          console.log('Could not load data :', error);
          return;
        }

        arc.roles.roleinfo = roles;
        cb();
      });
  };

  arc.roles.roleSummaryConfig = function() {
    const config = function() { return 'arc.roles.config'; };

    config.initialize = function() {
      // Set up click handlers

      // click on table row to open detail page but still
      // allow clicking on a link like trash can
      const table = $('#role-summary-table');
      table.on('click', 'tr td', arc.roles.clickSummaryRow);
      table.on('click', '.fa-trash', arc.roles.clickSummaryTrash);

      arc.roles.loadSummary(() => {
        arc.roles.drawSummary();
      });

      return config;
    };

    return config;
  };
}());
