window.arc = window.arc || {};

(function() {
'use strict';

arc.workspace = arc.workspace || {};

// This handles numerical values so long as it appears at beginning of string
//  in numeric order. If the number is the same, then sort in alphabetical order
// Numerical workspaces always appear before non-numeric
function _sortWorkspaces(wksp1, wksp2) {
  const wksp1Name = wksp1.name.toLowerCase();
  const wksp2Name = wksp2.name.toLowerCase();
  const matchRegex = /^(\d+)(.*)/;
  const wksp1Number = wksp1Name.match(matchRegex);
  const wksp2Number = wksp2Name.match(matchRegex);
  if (wksp1Number && wksp2Number) {
    const wkspDifference = Number(wksp1Number[1]) - Number(wksp2Number[1]);
    if (wkspDifference === 0) {
      return _sortWorkspaces({ name: wksp1Number[2] }, { name: wksp2Number[2] });
    }

    return wkspDifference;
  } else if (wksp1Number || wksp1Name < wksp2Name) {
    return -1;
  } else if (wksp2Number || wksp1Name > wksp2Name) {
    return 1;
  }

  return 0;
}

arc.workspace.sortWorkspaces = function (allWorkspaces) {
  return allWorkspaces.sort(_sortWorkspaces);
};

arc.workspace.workspaces = function () {
  let currentWorkspace;
  let originalWorkspace;
  let privateWorkspace;
  let $workspaceBar;
  let $workspaceMoveButton;
  let $workspaceName;
  let $workspaceMoveSpan;
  let $workspaceMoveLink;
  let popoverVisible;
  let dashboard;
  let msdb;
  let appgroup_id;
  let submitMoveCb;
  let hasEditPermissions;

  // Create/Edit
  let usersAndGroups; // Object with USER and GROUP
  let $workspaceEditButton;
  let $workspaceSidebar;
  let tokenfieldAlreadySetup;

  const $body = $('body');
  const $workspacePopoverTemplate = $('#move-viz-popover');
  const $appsPageOnlyText = $workspacePopoverTemplate.find('.apps-page-only');
  const $workspaceList = $workspacePopoverTemplate.find('.ws-selection-list');
  const $workspaceLiTemplate = $workspacePopoverTemplate.find('.workspace-template');

  // Move confirm modal
  const $moveModal = $('.clone-move-with-linked-modal');
  const $sharedTable = $moveModal.find('.linked-visual-table');
  const $actionSpan = $moveModal.find('.action-type');
  const $cloneModalCheckbox = $moveModal.find('.clone-only');
  const $moveModalCheckbox = $moveModal.find('.move-only');
  const $headerTemplate = $moveModal.find('#dashboard-row-modal-template');
  const $rowTemplate = $($moveModal.find('#visual-row-modal-template').html());
  const $cloneRowTemplateCheckbox = $rowTemplate.find('.clone-only');
  const $moveRowTemplateCheckbox = $rowTemplate.find('.move-only');
  const $multipleDashboardText = $rowTemplate.find('.multiple-dashboards-only');
  const $warningText = $moveModal.find('.moving-warning');

  // Create/Edit/Delete button
  const $createEditModal = $('#workspace-edit-modal');
  const $modalType = $createEditModal.find('.ws-edit-type');
  const $manageSection = $createEditModal.find('.ws-manage-users-section');
  const $addAcl = $createEditModal.find('.add-more-acl');
  const $saveChanges = $createEditModal.find('.ws-save-button');
  const $deleteButton = $createEditModal.find('.ws-delete-button');
  const $wsUserInput = $createEditModal.find('input[name=USER]');
  const $wsGroupInput = $createEditModal.find('input[name=GROUP]');
  const $wsNameInput = $createEditModal.find('input[name=name]');
  const $wsNameFormGroup = $wsNameInput.closest('.form-group');
  const $wsNameFormError = $wsNameFormGroup.find('.form-text');
  const $wsError = $createEditModal.find('.alert-danger');
  const $wsDescInput = $createEditModal.find('textarea[name=description]');
  const $currentUsers = $manageSection.find('tbody');
  const $wsUserTemplate = $manageSection.find('#ws-user-template');
  const currentUsername = $createEditModal.find('#workspace-current-username').text();
  const deleteConfirmation = arc.misc.deleteConfirmation();

  const $submitButton = $moveModal.find('.btn-submit-clone-move');

  // Update from WSACL_Member and WSACL_Level class in workspace/model.py
  const workspaceInfo = {
    WSACL_Member: {
      USER: 1,
      GROUP: 2,
      1: 'USER',
      2: 'GROUP',
    },
    WSACL_Level: {
      VIEW: 1,
      EDIT: 2,
      MANAGE: 3,
      1: 'VIEW',
      2: 'EDIT',
      3: 'MANAGE',
    },
    isSetup: false,
  };

  const specialButtons = {
    // For workspace groups
    Private: '<i class="fa fa-lock"></i> ',
    Public: '<i class="fa fa-users"></i> ',
    Other: '<i class="fa sym-workspace"></i> ',
    Empty: '<i class="fa fa-user-circle-o shared-workspace-icon"></i> ',

    // For user/usergroup in edit form
    USER: '<i class="fa fa-user"></i>',
    GROUP: '<i class="fa fa-users"></i>',
  };

  const workspaces = function () { return 'arc.workspace.workspaces'; };

  deleteConfirmation.objectType('Workspace');

  // ================================ Setters/Getters ==========================
  /** Object with workspace_id and workspace_name properties */
  workspaces.currentWorkspace = function (_) {
    if (!arguments.length) return currentWorkspace;
    currentWorkspace = _;

    if (workspaceIsEditable() !== hasEditPermissions) {
      hasEditPermissions = !hasEditPermissions;
      hideLinkIfAvailable(!hasEditPermissions);
    }

    // Update private workspace id if nonexistent
    if (_ &&
        _.workspace_name === 'Private' &&
        _.workspace_id !== undefined &&
        privateWorkspace &&
        !privateWorkspace.id) {
      privateWorkspace.id = _.workspace_id;

      if (arc.globals.workspaces_by_id) {
        arc.globals.workspaces_by_id[privateWorkspace.id] = _;
      }
    }

    return currentWorkspace;
  };

  /** Processes workspace */
  workspaces.processWorkspace = function () {
    if (arc.globals.workspaces) {
      arc.globals.workspaces_by_id = {};

      // Get list of workspaces that user can edit
      let userCanEdit;
      arc.globals.editWorkspaces = arc.globals.workspaces.filter(ws => {
        arc.globals.workspaces_by_id[ws.id] = ws;
        const hasPerm = arc.misc.hasPerm('ws_appedit', ws.perms);

        if (!userCanEdit && (msdb || appgroup_id) && hasPerm && currentWorkspace) {
          userCanEdit = currentWorkspace.workspace_id === ws.id;
        }

        if (ws.name === 'Private') {
          privateWorkspace = ws;
        }

        return hasPerm;
      });
    }
  };

  /** Object with workspace_id and workspace_name properties */
  workspaces.originalWorkspace = function (_) {
    if (!arguments.length) return originalWorkspace;
    originalWorkspace = _;
    return originalWorkspace;
  };

  /** Multisheet dashboard instance (needed for dashboard_id) */
  workspaces.appgroup_id = function (_) {
    if (!arguments.length) return appgroup_id;
    appgroup_id = _;
    return appgroup_id;
  };

  /** Multisheet dashboard instance (needed for dashboard_id) */
  workspaces.msdb = function (_) {
    if (!arguments.length) return msdb;
    msdb = _;
    return msdb;
  };

  /** Dashboard instance */
  workspaces.dashboard = function (_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    return dashboard;
  };

  /** Returns back workspace symbol based on name */
  workspaces.getSpecialIcon = function (workspaceName) {
    return specialButtons[workspaceName] || '';
  };

  // ================================ MAIN FUNCTIONS ===========================
  /**
  * MAIN FUNCTION: sets up workspace button and popover
  * Called on dashboard and apps page
  */
  workspaces.setupWorkspaceButton = function($navbar, cb) {
    if ((arc.testutil && arc.testutil.disableWorkspaces) || arc.globals.embedded) {
      return;
    }

    $workspaceBar = $navbar;
    submitMoveCb = cb;
    $workspaceMoveButton = $workspaceBar.find('.move-viz-btn');
    $workspaceName = $workspaceMoveButton.find('.viz-workspace-wrapper');

    // Assign span and link containers
    $workspaceName.each((i, el) => {
      if (el.tagName === 'A') {
        $workspaceMoveLink = $(el);
      } else {
        $workspaceMoveSpan = $(el);
      }
    });

    // For dashboards/visuals, add workspace name to button
    setNameButton();

    // Don't try to set up workspace on visuals page
    if (arc.globals.isInModal()) {
      return;
    }

    // Select different workspaces
    $body.off('click.selectNewWorkspace')
      .on('click.selectNewWorkspace', '.workspace-option', (e) => {
        e.stopPropagation();

        const $li = $(e.currentTarget);

        if ($li.hasClass('active')) {
          return;
        }

        changeWorkspace({
          workspace_id: $li.data('workspaceId'),
          workspace_name: $li.data('workspaceName'),
          workspace_perm: $li.data('workspacePerm'),
        });

        $li.closest('.popover-body').find('li.active').removeClass('active');
        $li.addClass('active');
      });

    // Open/close popover
    setupMovePopover();
  };

  /**
  * Returns true if user has changed workspaces
  * Always returns true if not on dashboard page
  */
  workspaces.hasChanged = function () {
    const originalWorkspaceId = originalWorkspace && originalWorkspace.workspace_id;
    return !msdb ||
           (currentWorkspace && currentWorkspace.workspace_id !== originalWorkspaceId);
  };

  /** Used on apps page to save workspace changes */
  workspaces.moveToNewWorkspace = function(dashboardsInfo, moveSuccessCb) {
    $.ajax({
      type: 'POST',
      data: {
        dashboards_info: JSON.stringify(dashboardsInfo),
        current_workspace_id: currentWorkspace.workspace_id,
      },
      url: arc.frontend.globals.urls.moveReportsToWorkspaceUrl,
      beforeSend(xhr) {
        // Because of a jquery bug when handling POST requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done((resp) => {
      // If move failed, give user status
      if (resp.some(moveResult => moveResult.error)) {
        let movedReports = [];
        let clonedReports = [];
        const failedReports = resp.map(function parseMoveResults(moveResult) {
          movedReports = movedReports.concat(Object.keys(moveResult.reports_to_move));
          clonedReports = clonedReports.concat(moveResult.reports_to_clone);
          return moveResult.reports_failed;
        });

        let moveErrors = '';
        if (failedReports.length) {
          moveErrors += `Failed to move the following reports: ${failedReports.join(', ')}.`;
        }

        if (movedReports.length) {
          moveErrors += ` Successfully moved #${movedReports.join(', ')}`;
        }

        if (clonedReports.length) {
          moveErrors += ` Successfully cloned #${clonedReports.join(', ')}`;
        }

        arc.report.alertMessage('Move failed', moveErrors);
        return;
      }

      // If success, set current workspace as original, rename button, and update shared visuals
      originalWorkspace = currentWorkspace;
      setNameButton();

      // Remove "shared_visual_dashboards" and update linked icon from all cloned visuals
      // Update workspaces for all cloned or moved visuals
      if (msdb && arc.globals.reports) {
        if (resp[0].reports_to_clone) {
          resp[0].reports_to_clone.forEach(function updateReportSharedVisualDashboards(newOldIds) {
            const oldId = newOldIds.old_id;
            const newId = newOldIds.new_id;
            const oldReport = arc.globals.reports.getReport(oldId);
            oldReport.workspace(currentWorkspace);
            arc.misc.updateReports({ oldReport, oldId, newId, removeSharedIcon: true });

            // Update so next fetch will get correct workspace information
            updateReportWorkspace(newId);

            // Forget old report
            arc.staticfetcher.forget([{ asset: `${arc.frontend.globals.urls.reportUrl}/${oldId}` }]);
          });
        }

        if (resp[0].reports_to_move) {
          const listOfMoved = Object.keys(resp[0].reports_to_move);
          listOfMoved.forEach(function updatedMovedVisuals(visual_id) {
            const visualReport = arc.globals.reports.getReport(visual_id);
            if (!visualReport) {
              // @assert - visualReport is filter on another sheet (it is unlinked by default)
              return;
            }

            visualReport.workspace(currentWorkspace);

            // Update so next fetch will get correct workspace information
            updateReportWorkspace(visual_id);
          });
        }
      }

      if (moveSuccessCb) {
        moveSuccessCb(resp);
      } else {
        let failedString = '';
        const reportsIds = resp.reduce((string, reportObj) => {
          if (string) {
            string += ', ';
          }

          if (reportObj.reports_failed) {
            if (failedString) {
              failedString += `, ${reportObj.reports_failed.join(', ')}`;
            } else {
              failedString = reportObj.reports_failed.join(', ');
            }
          }

          if (reportObj.report_id) {
            string += reportObj.report_id;
          }

          return string;
        }, '');

        let message;
        if (!reportsIds) {
          message = `Move successful. No reports to move.`;
        } else {
          message = `${reportsIds} successfully moved.`;
        }

        let alertStatus = 'alert-success';
        if (failedString) {
          message += `. However failed to move ${failedString}`;
          alertStatus = 'alert-info';
        }

        arc.report.alertMessageSmall(message, alertStatus);
        console.log(resp);
      }
    })
    .fail((resp) => {
      arc.report.alertMessage('Move failed',
        `Failed to move dashboards: ${resp.responseText ? resp.responseText : ''}`);
    });

    function updateReportWorkspace(reportId) {
      arc.staticfetcher.update([
        {
          asset: `${arc.frontend.globals.urls.reportUrl}/${reportId}`,
          cb: function updateWorkspace(data) {
            data[0].workspace_id = currentWorkspace.workspace_id;
            data[0].workspace_name = currentWorkspace.workspace_name;
            data[0].workspace_perm = currentWorkspace.workspace_perm;
          },
        },
      ]);
    }
  };

  /** MAIN FUNCTION: Setup edit workspace button. Used on apps page */
  workspaces.setupEditButton = function($sidebar, successCb) {
    $workspaceSidebar = $sidebar;
    $workspaceEditButton = $sidebar.find('.ws-list-edit');
    let editWorkspace;
    let canManage;

    if (!workspaceInfo.isSetup) {
      setupEventHandlers();
      workspaceInfo.isSetup = true;
    }

    // ================================ EDIT HELPER FUNCTIONS =========================
    function setupEventHandlers() {
      // only bother setting up edit button if user has system permissions
      if (arc.misc.hasPerm('sys_workspace', arc.globals.perms.sysperms)) {
        $workspaceEditButton.off('click.openCreateModal')
                            .on('click.openCreateModal', function openCreateModal() {
                              _resetModal();
                              editWorkspace = undefined;
                              $modalType.text('Create ');
                              setupManage(['ws_manage']);
                              $createEditModal.modal('show');
                            });
      }

      $workspaceSidebar.off('click.openEditModal')
                       .on('click.openEditModal', '.edit-workspace', (e) => {
                         const workspaceId = $(e.currentTarget).closest('a').data('appgroup');
                         editWorkspace = arc.globals.workspaces_by_id[parseInt(workspaceId, 10)];
                         const cannotEditAcl = editWorkspace.editable ||
                                               editWorkspace.name === 'Public';
                        _resetModal();
                         if (editWorkspace &&
                             cannotEditAcl &&
                             arc.misc.hasPerm('ws_manage', editWorkspace.perms)) {
                           $wsNameInput.val(editWorkspace.name);
                           $wsDescInput.val(editWorkspace.desc);
                           $modalType.text('Edit ');
                           setupManage(editWorkspace.perms);
                           $createEditModal.modal('show');
                         } else {
                           const message = `No edit permission for workspace ${workspaceId}`;
                           arc.report.alertMessageSmall(message, 'alert-info');
                         }
                       });

      $saveChanges.off('click.submitNewEditWS')
                     .on('click.submitNewEditWS', saveWS);
    }

    function _resetModal() {
      $createEditModal.find('.modal-body .tokenfield div.token').remove();
      $wsNameInput.val('');
      $wsDescInput.val('');
      arc.report.alertMessageAllHide();
      $wsError.addClass('arc-invisible');
      $wsNameFormGroup.removeClass('has-error');
      $wsNameFormError.addClass('arc-invisible').find('li').html('');
    }

    function setupManage(perms) {
      if (!usersAndGroups) {
        d4.queue()
          .defer(d3.json, `${arc.frontend.globals.urls.usersDetailedApiUrl}?summary=true`)
          .defer(d3.json, `${arc.frontend.globals.urls.groupsUrl}?summary=true`)
          .await((error, users = [], groups = []) => {
            groups.push({
              name: arc.globals.perms.everyone_group,
            });

            usersAndGroups = {
              USER: users,
              GROUP: groups,
            };

            setupManageEventHandlers(perms);
          });
      } else {
        setupManageEventHandlers(perms);
      }
    }

    function setupManageEventHandlers(perms) {
      // Setup manage section if user has manage access
      if (!arc.misc.hasPerm('ws_manage', perms)) {
        $manageSection.addClass('arc-invisible');
        $deleteButton.addClass('arc-invisible');
      } else {
        canManage = true;
        $currentUsers.empty();

        // Enable anything that was disabled with public workspace modal
        $deleteButton.removeClass('arc-invisible');
        $wsNameInput.prop('disabled', false);
        $wsDescInput.prop('disabled', false);
        if (editWorkspace) {
          const acl = editWorkspace.acl;
          acl.forEach(addUserOrGroupRow);
          $deleteButton.removeClass('arc-invisible');

          if (editWorkspace.name !== 'Public') {
            $deleteButton.off('click.askDeleteConfirm')
                         .on('click.askDeleteConfirm', askDeleteConfirm);
          } else {
            // If Public workspace, disable name, description, and delete button
            $deleteButton.addClass('arc-invisible');
            $wsNameInput.prop('disabled', true);
            $wsDescInput.prop('disabled', true);
          }
        }

        if (!tokenfieldAlreadySetup) {
          setupTypeahead($wsUserInput, 'USER', 'username');
          setupTypeahead($wsGroupInput, 'GROUP', 'name');
          tokenfieldAlreadySetup = true;
        }

        // Add event handler: Add users/groups
        $addAcl.off('click.addAcl').on('click.addAcl', function addAcl(_e, defaultSelection) {
          addAclValues($wsGroupInput, defaultSelection);
          addAclValues($wsUserInput, defaultSelection);
        });

        // Add event handler: Delete user groups
        $currentUsers.off('click.addAcl')
                     .on('click.addAcl', '.ws-remove-item', function addAcl(e) {
                       $(e.currentTarget).closest('tr').remove();
                     });

        if (!editWorkspace) {
          // Now add current user if creating new workspace
          $wsUserInput.tokenfield('setTokens', [currentUsername]);
          $addAcl.trigger('click.addAcl', ['MANAGE']);
        }

        $manageSection.removeClass('arc-invisible');
      }
    }

    function askDeleteConfirm() {
      deleteConfirmation.objectName(editWorkspace.name);
      deleteConfirmation.askUser(deleteWS);
    }

    function deleteWS() {
      $.ajax({
        type: 'DELETE',
        url: `${arc.frontend.globals.urls.workspacesUrl}/${editWorkspace.id}`,
        beforeSend(xhr) {
          // Because of a jquery bug when handling POST requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
      .done((resp) => {
        deleteConfirmation.hideModal();
        if (successCb) {
          successCb(resp);
        }
      })
      .fail((resp) => {
        deleteConfirmation.hideModal();
        arc.report.alertMessage('Delete failed',
          `Failed to delete workspace: ${resp.responseText ? resp.responseText : ''}`);
      });
    }

    function setupTypeahead($el, type, value) {
      const availableValues = {};
      const engine = new Bloodhound({
        local: usersAndGroups[type].map(function getValue(usrOrGp) {
          availableValues[usrOrGp[value]] = true;
          return { value: usrOrGp[value] };
        }),

        datumTokenizer(d) {
          return Bloodhound.tokenizers.whitespace(d.value);
        },

        queryTokenizer: Bloodhound.tokenizers.whitespace,
      });
      engine.initialize();

      $el.tokenfield({
        createTokensOnBlur: true,
        typeahead: [null, { source: engine.ttAdapter() }],
      });

      $el.tokenfield('setTokens', []);
    }

    function addAclValues($userOrGroupInput, defaultSelection) {
      const itemsToAdd = $userOrGroupInput.tokenfield('getTokens');
      const type = $userOrGroupInput.attr('name');
      const member = workspaceInfo.WSACL_Member[type];

      // for updating the .token-input's width
      if (itemsToAdd.length === 0) {
        $wsGroupInput.tokenfield('update');
      }

      itemsToAdd.forEach(function addItem(itm) {
        const permissionType = defaultSelection || 'VIEW';
        const itemAcl = [member, workspaceInfo.WSACL_Level[permissionType], itm.value];
        addUserOrGroupRow(itemAcl);
      });

      $userOrGroupInput.tokenfield('setTokens', []);
    }

    function addUserOrGroupRow(userOrGroupAcl) {
      const username = userOrGroupAcl[2];
      const usertype = userOrGroupAcl[0];
      const exists = $currentUsers.find(`tr[data-username="${username}"]` +
                                        `[data-user-type="${usertype}"]`);

      if (exists.length) {
        return;
      }

      const permission = workspaceInfo.WSACL_Level[userOrGroupAcl[1]] || 'VIEW';
      const type = workspaceInfo.WSACL_Member[usertype];
      const userObj = {
        USER_TYPE: usertype,
        HTML_USER_ICON: workspaces.getSpecialIcon(type),
        USERNAME: username,
        VIEW_CHECKED: permission === 'VIEW' ? 'checked' : '',
        EDIT_CHECKED: permission === 'EDIT' ? 'checked' : '',
        MANAGE_CHECKED: permission === 'MANAGE' ? 'checked' : '',
      };

      const newRow = arc.misc.safe_substitute($wsUserTemplate.html(), userObj);

      $currentUsers.append(newRow);
    }

    function saveWS() {
      const wsname = $wsNameInput.val().trim(),
            duplicateName = arc.globals.workspaces.some((ws) => {
                const sameId = editWorkspace && ws.id === editWorkspace.id;
                const sameName = ws.name === wsname;
                return !sameId && sameName;
            }),
            hasErr = !wsname || duplicateName;

      let errTxt = '';

      if (!wsname) {
        errTxt = 'No workspace name specified';
      }

      if (duplicateName) {
        errTxt = 'Duplicate workspace name not allowed';
      }

      $wsNameFormGroup.toggleClass('has-error', hasErr);
      $wsNameFormError
        .toggleClass('arc-invisible', !hasErr)
        .find('li')
        .html(errTxt);
      if (hasErr) {
        return;
      }

      const newWorkspaceValues = editWorkspace || {};
      newWorkspaceValues.name = wsname;
      newWorkspaceValues.desc = $wsDescInput.val();

      if (canManage) {
        const newAcl = [];
        $currentUsers.find('tr').each((i, el) => {
          const checkedPermission = $(el).find('input:checked').val();
          const permission = workspaceInfo.WSACL_Level[checkedPermission];
          newAcl.push([parseInt(el.dataset.userType), permission, el.dataset.username]);
        });
        newWorkspaceValues.acl = newAcl;
      }

      $.ajax({
        type: 'POST',
        data: {
          data: JSON.stringify([newWorkspaceValues]),
        },
        url: arc.frontend.globals.urls.workspacesUrl,
        beforeSend(xhr) {
          // Because of a jquery bug when handling POST requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
      .done((resp) => {
        $createEditModal.modal('hide');
        if (successCb) {
          successCb(resp);
        }
      })
      .fail((resp) => {
        $wsError.removeClass('arc-invisible');
        $wsError.text(`Failed to edit workspace: ${resp.responseText ? resp.responseText : ''}`);
      });
    }
  };

  return workspaces;

  // ================================ MOVE HELPER FUNCTIONS =========================
  function setNameButton() {
    // Only show workspace name if place for it exists
    if ($workspaceName.length) {
      if (currentWorkspace && currentWorkspace.workspace_id) {
        const currentWorkspaceName = currentWorkspace.workspace_name;
        $workspaceName.empty();

        $workspaceName.append(workspaces.getSpecialIcon(currentWorkspaceName) ||
                              specialButtons.Other);
        $workspaceName.append(currentWorkspaceName);
      } else {
        $workspaceName.html(`${workspaces.getSpecialIcon('Private')}Private`);
      }
    }
  }

  function changeWorkspace(newWorkspace) {
    if (!originalWorkspace) {
      originalWorkspace = currentWorkspace;
    }

    currentWorkspace = newWorkspace;
  }

  function revertWorkspace() {
    if (originalWorkspace) {
      currentWorkspace = originalWorkspace;
    }
  }

  function workspaceIsEditable() {
    const workspacePerms = ((currentWorkspace && currentWorkspace.workspace_perm) || []);
    return workspacePerms.includes('ws_appedit');
  }

  function setupMovePopover() {
    hasEditPermissions = workspaceIsEditable();
    hideLinkIfAvailable(!hasEditPermissions);

    $workspaceMoveButton.off('click.moveWorkspace')
                        .one('click.moveWorkspace', function () {
      if (arc.globals.workspaces) {
        setupMoveEventListeners();
      } else {
        $.ajax({
          url: arc.frontend.globals.urls.workspacesUrl,
          type: 'GET',
          beforeSend(xhr) {
            // Because of a jquery bug when handling GET requests,
            // we have to explicitly set X-CSRFToken here
            xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
          },
        })
        .done((resp) => {
          arc.globals.workspaces = arc.workspace.sortWorkspaces(JSON.parse(resp));
          setupMoveEventListeners();
        })
        .fail((resp) => {
          arc.report.alertMessage('Failed to get workspaces',
            `${resp.responseText ? resp.responseText : ''}`);
        });
      }
    });
  }

  function hideLinkIfAvailable(hideLink) {
    if ($workspaceMoveLink && $workspaceMoveSpan) {
      if (hideLink) {
        $workspaceMoveLink.addClass('arc-invisible');
        $workspaceMoveSpan.removeClass('arc-invisible');
      } else {
        $workspaceMoveLink.removeClass('arc-invisible');
        $workspaceMoveSpan.addClass('arc-invisible');
      }
    } else if (hideLink && $workspaceMoveButton) {
      $workspaceMoveButton.addClass('arc-invisible');
    } else if ($workspaceMoveButton) {
      $workspaceMoveButton.removeClass('arc-invisible');
    }
  }

  function setupMoveEventListeners() {
    // If user doesn't have any workspaces he can move it to, don't bother setting up move button
    if (!arc.globals.workspaces.length) {
      hideLinkIfAvailable(true);
      return;
    }

    workspaces.processWorkspace();

    // Close popovers
    let canClosePopover;
    let forceShowPopoverFirstTime = false;

    // Don't close if click on popover
    $body.off('click.openPopover').on('click.openPopover', function openPopover(e) {
      if (popoverVisible) {
        const $cancel = $(e.target).closest('.cancel-workspace-move');
        if ($cancel.length) {
          canClosePopover = true;
          return;
        }

        const $popover = $(e.target).closest('.workspace-move-popover');
        if ($popover.length) {
          canClosePopover = false;
          e.stopImmediatePropagation();
          e.stopPropagation();
        } else {
          canClosePopover = true;
        }
      }
    });

    // Cancel
    $body.off('click.closeWorkspaces').on('click.closeWorkspaces', function closeWorkspaces(e) {
      if (popoverVisible && canClosePopover && !forceShowPopoverFirstTime) {
        e.stopPropagation();
        popoverVisible = false;
        $workspaceMoveButton.popover('hide');
        revertWorkspace();
      }

      forceShowPopoverFirstTime = false;
    });

    // Save
    $body.offOn('click.saveWorkspace', '.save-workspace-move', () => {
      openMoveForm();
      popoverVisible = false;
      $workspaceMoveButton.popover('hide');
    });

    $workspaceMoveButton.popover({
      animation: true,
      html: true,
      trigger: 'click',
      placement: 'bottom',
      template: '<div class="popover workspace-move-popover" role="tooltip">' +
      '<div class="arrow"></div><h3 class="popover-header"></h3>' +
      '<div class="popover-body"></div></div>',
      sanitize: false,
      content() {
        if (!hasEditPermissions) {
          return '';
        }

        $workspaceList.empty();
        arc.globals.workspaces.forEach(function addWorkspace(wrkspc) {
          if (!arc.misc.hasPerm('ws_appedit', wrkspc.perms)) {
            return;
          }

          const currentName = wrkspc.name;
          const currentID = wrkspc.id;
          const workspaceData = {
            WORKSPACE_ID: currentID,
            WORKSPACE_NAME: currentName,
            WORKSPACE_PERM: JSON.stringify(wrkspc.perms),
            HTML_WORKSPACE_ICON: workspaces.getSpecialIcon(currentName) || specialButtons.Empty,
            ACTIVE: currentWorkspace && (currentID === currentWorkspace.workspace_id) ?
                    'active' : '',
          };

          const rowHTML = arc.misc.safe_substitute($workspaceLiTemplate.html(), workspaceData);

          const isReservedWorkspace = wrkspc.name === 'Public' || wrkspc.name === 'Private';
          if (isReservedWorkspace) {
            $workspaceList.prepend(rowHTML);
          } else if (wrkspc.editable) {
            $workspaceList.append(rowHTML);
          }
        });

        // Hide some text in dashboard
        if (msdb) {
          $appsPageOnlyText.addClass('arc-invisible');
        }

        return $workspacePopoverTemplate.html();
      },
    }).off('click.openPopover').on('click.openPopover', function openPopover(e) {
      popoverVisible = true;
      e.stopPropagation();
    });

    popoverVisible = true;
    forceShowPopoverFirstTime = true;
    $workspaceMoveButton.popover('show');
  }

  /**
  * Opens modal to confirm move. When saving new dashboard, opens upon clicking "save" button
  */
  function openMoveForm() {
    const saveFormCb = submitMoveCb || workspaces.moveToNewWorkspace;
    const dashboardsInfo = getDashboardInfo();
    const hasSharedVisual = dashboardsInfo.filter(dashInfo => dashInfo.sharedVisuals &&
                                                              dashInfo.sharedVisuals.length);

    if (!workspaces.hasChanged()) {
      arc.report.alertMessageSmall('No workspace changes to save', 'alert-success');
      return;
    }

    if (arc.report.pagenav.needs_save() &&
        arc.globals.dashboard &&
        arc.globals.dashboard.reports_to_update().length) {
      const error = 'Dashboard cannot be moved with unsaved changes to linked visuals.';
      const solution = 'Please save and try again.';
      arc.report.alertMessageSmallBase({
        alertMessage: `${error} ${solution}`,
        type: 'alert-warning',
        alertHeader: 'Alert',
      });
      return;
    }

    if (hasSharedVisual.length) {
      if (arc.misc.modalShown($moveModal)) {
        return;
      }

      $cloneModalCheckbox.addClass('arc-invisible');
      $moveModalCheckbox.removeClass('arc-invisible');
      $cloneRowTemplateCheckbox.addClass('arc-invisible');
      $moveRowTemplateCheckbox.removeClass('arc-invisible');
      $actionSpan.text('Move');

      if (hasSharedVisual.length > 1) {
        $multipleDashboardText.removeClass('arc-invisible');
      } else {
        $multipleDashboardText.addClass('arc-invisible');
      }

      // Add shared visuals
      $sharedTable.empty();
      hasSharedVisual.forEach(addDashboardInfo);

      $sharedTable.find('[data-toggle="tooltip"]').tooltip();

      // Only show warning if user decides to move
      $warningText.addClass('arc-invisible');
      $sharedTable.off('change.showWarning')
                  .on('change.showWarning', '.move-only input', function showAttention() {
        $warningText.removeClass('arc-invisible');
      });

      // Add event handler
      $submitButton.offOn('click.submitSave', function cloneDashboard() {
        const $cloneVisuals = $sharedTable.find('.clone-option input:checked');
        const $moveVisuals = $sharedTable.find('.move-option input:checked');

        getUserSelections($cloneVisuals, 'reports_to_clone');
        getUserSelections($moveVisuals, 'reports_to_move');
        saveFormCb(dashboardsInfo);
        $moveModal.modal('hide');
      });

      $moveModal.modal('show');
    } else {
      saveFormCb(dashboardsInfo);
    }

    // ================================ opeMoveForm HELPER ================================
    function getUserSelections($lis, type) {
      $lis.each(function addLinkedVisualToInfo(i, el) {
        const dashboardIdx = parseInt(el.dataset.dashboardIdx);
        const visualId = parseInt(el.name);
        if (dashboardsInfo[dashboardIdx][type]) {
          dashboardsInfo[dashboardIdx][type].push(visualId);
        } else {
          dashboardsInfo[dashboardIdx][type] = [visualId];
        }
      });
    }

    function addDashboardInfo(info, idx) {
      if (info.sharedVisuals.length) {
        const dashboardRow = arc.misc.safe_substitute($headerTemplate.html(), {
          DASHBOARD_ID: info.report_id,
          DASHBOARD_NAME: info.dashboardName || 'Untitled',
        });

        $sharedTable.append(dashboardRow);
        addVisualInfo(info.sharedVisuals, idx);
      }
    }

    function addVisualInfo(sharedVisuals, dashboardIdx) {
      sharedVisuals.forEach(sharedId => {
        const sharedVisualData = getSharedVisualObject(sharedId, dashboardIdx);
        const tableRow = arc.misc.safe_substitute($rowTemplate[0].outerHTML, sharedVisualData);
        $sharedTable.append(tableRow);
      });
    }

    /**
    * Returns an array of object with dashboardId, dashboardName, sharedVisuals
    * Works for apps and dashboard page
    */
    function getDashboardInfo() {
      const returnObj = [];
      if (msdb && dashboard && arc.globals.reports) {
        const sharedUnshared = arc.globals.reports.getSharedUnsharedVisuals();
        returnObj.push({
          report_id: msdb.dashboard_id(),
          dashboardName: dashboard.dashboard_name(),
          sharedVisuals: sharedUnshared.sharedVisuals,
          unsharedVisuals: sharedUnshared.unsharedVisuals,
        });
      } else if (appgroup_id) {
        returnObj.push({
          report_id: appgroup_id,
          type: 'apps',
        });
      } else if (arcl.apps_by_typeid) {
        const selectedReports = arc.screen.apps.selectedAppIdsAndTypes();
        selectedReports.dashboard.forEach(function addDashboardToObject(report) {
          const reportInfo = arcl.apps_by_typeid[`dashboard${report}`];
          returnObj.push({
            report_id: report,
            type: 'dashboard',
            dashboardName: reportInfo.title,
            sharedVisuals: reportInfo.shared_visual_dashboards &&
                           JSON.parse(reportInfo.shared_visual_dashboards),
          });
        });

        selectedReports.apps.forEach(function addAppToObject(report) {
          returnObj.push({
            report_id: report,
            type: 'apps',
          });
        });

        selectedReports.visual.forEach(function addVisualToObject(report) {
          // If single visual, only push in report_id
          returnObj.push({
            report_id: report,
            type: 'visual',
          });
        });
      }

      return returnObj;
    }

    /**
    * Given dashboard id, returns object for adding to template
    * Works for apps and dashboard page
    * */
    function getSharedVisualObject(reportId, dashboardIdx) {
      if (msdb) {
        const report = arc.globals.reports.getReport(reportId);
        const workspace = report.workspace() || {};
        return {
          DASHBOARD_IDX: dashboardIdx,
          WORKSPACE_ID: workspace.workspace_id,
          WORKSPACE_NAME: workspace.workspace_name,
          VISUAL_NAME: report.viz() ? report.viz().title() || 'Untitled' : '',
          VISUAL_ID: report.report_id(),
          SHARED_VISUAL_DASHBOARDS: report.shared_visual_dashboards().replace(/\[|\]/g, '') ||
                                    'only this dashboard',
        };
      } else if (arcl.apps_by_typeid) {
        const reportInfo = arcl.apps_by_typeid[`visual${reportId}`];
        return {
          DASHBOARD_IDX: dashboardIdx,
          WORKSPACE_ID: reportInfo.workspace_id,
          WORKSPACE_NAME: reportInfo.workspace_name,
          VISUAL_NAME: reportInfo.title || 'Untitled',
          VISUAL_ID: reportInfo.id,
          SHARED_VISUAL_DASHBOARDS: reportInfo.shared_visual_dashboards.replace(/\[|\]/g, '') ||
                                    'No dashboard',
        };
      }
    }
  }
};
}());
