window.arc = window.arc || {};

(function() {
'use strict';

arc.model = arc.model || {};

arc.model.AppGroupSaver = function() {
  var ags = function() { return 'arc.model.AppGroupSaver'; };

  var appgroups = [],
      updatedAppsInSubgroups,
      deletedGroups,
      workspaces;

  ags.updatedAppsInSubgroups = function(_) {
    if (!arguments.length) return updatedAppsInSubgroups;
    updatedAppsInSubgroups = _;
    return ags;
  };

  ags.workspaces = function(_) {
    if (!arguments.length) return workspaces;
    workspaces = _;
    return ags;
  };

  ags.deletedGroups = function(_) {
    if (!arguments.length) return deletedGroups;
    deletedGroups = _;
    return ags;
  };

  ags.appgroups = function(_) {
    if (!arguments.length) return appgroups;
    appgroups = _;
    return ags;
  };

  ags.addAppGroup = function(_) {
    appgroups.push(_);
    return ags;
  };

  ags.saveAppgroups = function(done_cb) {
    var datal = appgroups.map(function(ag) {
      return ag.collectData();
    });

    ags.sendPostData(datal, deletedGroups, updatedAppsInSubgroups, done_cb);
    return ags;
  };

  ags.sendPostData = function(datal, deleted_groups, updatedAppsInSubgroups, done_cb) {
    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      data: JSON.stringify(datal),
    };

    if (deleted_groups) {
      post_data.deleted_groups = JSON.stringify(deleted_groups);
    }

    if (updatedAppsInSubgroups) {
      post_data.updated_apps_in_subgroups =
                                    JSON.stringify(updatedAppsInSubgroups);
    }

    if (workspaces && workspaces.currentWorkspace()) {
      post_data.workspace_id = workspaces.currentWorkspace().workspace_id;
    }

    $.ajax({
      url: arc.frontend.globals.urls.appgroupUrl,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      const additionalInfo = req.responseText ? req.responseText : `${textstatus}. ${errorthrown}`;
      arc.report.alertMessage('Failed to update apps: ', additionalInfo);
    })
    .done((returned_data) => {
      done_cb(returned_data);
    });

    return ags;
  };

  return ags;
};

arc.model.appGroupCreate = function(names, doneCb) {
  var ags = arc.model.AppGroupSaver();
  names.forEach(function(name) {
    // Empty appgroup names are ignored here
    name = name.trim();
    if (name.length === 0) {
      return;
    }

    var ag = arc.model.AppGroup()
                      .group_name(name)
                      .in_userlist('true')
                      .group_options({});
    ags.addAppGroup(ag);
  });

  if (ags.appgroups().length === 0) {
    // Nothing to do
    doneCb('error: nothing to save');
    return;
  }

  ags.saveAppgroups(doneCb);
};

arc.model.AppGroup = function() {
  var ag = function() { return 'arc.model.AppGroup'; };

  var group_id,
      group_name,
      group_update_user,
      group_updated,
      in_userlist,
      group_options,
      group_options_json,
      group_subgroup_order,
      apps_in_subgroup,
      subgroups = [];

  // This is for the JS/frontend to send information about
  // which dashboards are in the newly created subgroup. For existing,
  // subgroups, this method is not used and instead the updatedAppsInSubgroups
  // field is used
  // This is also used for the backend to return information about which
  // dashboards are in a subgroup
  ag.apps_in_subgroup = function(_) {
    if (!arguments.length) return apps_in_subgroup;
    apps_in_subgroup = _;
    return ag;
  };

  ag.group_id = function(_) {
    if (!arguments.length) return group_id;
    group_id = _;
    return ag;
  };

  ag.group_subgroup_order = function(_) {
    if (!arguments.length) return group_subgroup_order;
    group_subgroup_order = _;
    return ag;
  };

  ag.group_name = function(_) {
    if (!arguments.length) return group_name;
    group_name = _;
    return ag;
  };

  ag.group_update_user = function(_) {
    if (!arguments.length) return group_update_user;
    group_update_user = _;
    return ag;
  };

  ag.group_updated = function(_) {
    if (!arguments.length) return group_updated;
    group_updated = _;
    return ag;
  };

  ag.in_userlist = function(_) {
    if (!arguments.length) return in_userlist;
    in_userlist = _;
    return ag;
  };

  ag.group_options = function(_) {
    if (!arguments.length) return group_options;
    group_options = _;
    group_options_json = JSON.stringify(group_options);
    return ag;
  };

  ag.group_options_json = function(_) {
    if (!arguments.length) return group_options_json;
    group_options_json = _;
    group_options = JSON.parse(group_options_json);
    return ag;
  };

  ag.subgroups = function(_) {
    if (!arguments.length) return subgroups;
    subgroups = _;
    return ag;
  };

  ag.addSubGroup = function(_) {
    subgroups.push(_);
    return ag;
  };

  ag.loadFromModel = function(in_data) {
    group_id = in_data.group_id;
    group_name = in_data.name;
    group_update_user = in_data.update_user;
    group_updated = in_data.updated;
    ag.group_options(in_data.group_options);
    in_userlist = in_data.in_userlist;
    group_subgroup_order = in_data.subgroup_order;
    apps_in_subgroup = in_data.apps;
    return ag;
  };

  ag.deleteAppgroup = function(done_cb) {
    const url = `${arc.frontend.globals.urls.appgroupUrl}/${group_id}`;
    $.ajax({
      url: url,
      type: 'DELETE',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      const failedMessage = req.responseText ? req.responseText : `${textstatus}. ${errorthrown}`;
      arc.report.alertMessage('Failed to delete app ', failedMessage);
    })
    .done(() => {
      done_cb();
    });
  };

  ag.collectData = function() {
    var data = {};
    if (group_options_json) {
      data.group_options = group_options_json;
    }

    if (group_name) {
      data.group_name = group_name;
    }

    if (in_userlist) {
      data.in_userlist = in_userlist;
    }

    if (group_id) {
      data.group_id = group_id;
    }

    if (group_subgroup_order) {
      data.group_subgroup_order = group_subgroup_order;
    }

    if (apps_in_subgroup) {
      data.apps_in_subgroup = apps_in_subgroup;
    }

    if (subgroups.length > 0) {
      data.subgroups = [];
      subgroups.forEach(function(sg) {
        data.subgroups.push(sg.collectData());
      });
    }

    return data;
  };

  /*
   * Update an appgroup's options. Does not update any other properties.
   */
  ag.saveGroupOptions = function(done_cb) {
    var datal = [{
      group_id: group_id,
      group_options: group_options_json,
    },
    ];

    arc.model.AppGroupSaver()
       .sendPostData(datal, undefined, done_cb);
    return ag;
  };

  return ag;
};

arc.model.UserAppGroupMembership = function() {
  var userappgroups = [];

  var UserAppGroupMembership = function() {
    return 'arc.model.UserAppGroupMembership';
  };

  UserAppGroupMembership.add_membership = function(groupid, in_userlist, order) {
    var d = {
      groupid: Number(groupid),
      in_userlist: in_userlist,
    };

    if (order) {
      d.order = Number(order);
    }

    userappgroups.push(d);
    return UserAppGroupMembership;
  };

  UserAppGroupMembership.save = function(done_cb) {
    if (userappgroups.length === 0) {
      done_cb();
      return UserAppGroupMembership;
    }

    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      data: JSON.stringify(userappgroups),
    };
    const url = arc.frontend.globals.urls.userAppGroupUrl;
    $.ajax({
      url: url,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail(function(req, textstatus, errorthrown) {
      arc.report.alertMessage('Failed to save user apps: ',
          textstatus + '. ' + errorthrown);
    })
    .always(function() {
      done_cb();
    });

    return UserAppGroupMembership;
  };

  return UserAppGroupMembership;
};

}());
