window.arc = window.arc || {};

(function() {
"use strict";

//
// This code helps handle the 'Controls' tab within the dashboard edit screen.
//

arc.report = arc.report || {};

function timeControlTracker() {
  let t = function() {
    return 'timeControlTracker';
  };

  let filter,
      settings,
      dataset,
      dashboard;

  t.filter = function(_) {
    if (!arguments.length) return filter;
    filter = _;
    return t;
  };

  t.remove = function() {
    const $create = $('.create-timecontrol');
    $create.prop('disabled', false);
    t.settings(undefined);
  };

  t.settings = function(_) {
    if (!arguments.length) {
      const s = settings;
      if (s && filter) {
        s.user_settings = filter.viz_getSettings();
      }

      return s;
    }

    settings = _;
    if (settings && !settings.dataset_id) {
      settings = undefined;
    }

    return t;
  };

  t.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return t;
  };

  t.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    return t;
  };

  t.setupTimeControlActions = function() {
    $('.create-timecontrol')
        .offOn('click.create', function newTimeControl() {
          if (settings) {
            return;
          }

          t.settings({
            dataset_id: t.dataset().dataset_id(),
          });

          drawTimeControlFromDataset(t.dataset());
          arc.report.pagenav.needs_save(true);
        });

    return t;
  };

  t.draw = function() {
    if (!settings) {
      return;
    }

    const {
      dataset_id,
    } = settings;

    /*
     * We are about to fetch a dataset from the backend. We don't know
     * how long it will take. Set up a marker on the dom and look for
     * it after the fetch completes.
     */
    const uniqueId = arc.misc.uniqueId() + '';
    const $timeControlContainer = $('.timecontrol-container');
    $timeControlContainer.attr('data-drawid', uniqueId);

    dashboard.fetchDataset(
        {
          uniqueId,
          dataset_id,
        }, drawTimeControl);
  };

  t.setupTimeControls = function(ds) {
    let timeSettings = ds.timeSettings();
    if (timeSettings.defaultDateTime) {
      $('.create-timecontrol').show();
      t.dataset(ds);
    } else {
      $('.create-timecontrol').hide();
      t.dataset(undefined);
    }
  };

  function drawTimeControl(config, datasetData, columninfo) {
    let {
      uniqueId,
      dataset_id,
    } = config;

    const $timeControlContainer = $('.timecontrol-container');
    let drawId = $timeControlContainer.attr('data-drawid');
    if (drawId !== uniqueId) {
      // Attempt to draw is now stale - some other draw has happened in the meantime
      return;
    }

    const dataset = arc.datasets.Dataset()
                       .dataset_id(dataset_id)
                       .init_from_fetch(datasetData, columninfo);

    drawTimeControlFromDataset(dataset);
  }

  function drawTimeControlFromDataset(dataset) {
    t.filter(arc.visual.plugins.createTimeControl({
      dataset,
      dashboard,
      settings,
    }));

    const $create = $('.create-timecontrol');
    $create.prop('disabled', true);
  }

  return t;
}

arc.report.dashboard_filter_controls = function() {
  var dom_id,
      dashboard,
      data_connections,
      dataset_hash = {},
      data_connections_hash = {},
      build_mode,
      isDataConnectionLoaded = false,
      timeControl = timeControlTracker();

  var dashboard_filter_controls = function() {
    return "arc.report.dashboard_filter_controls";
  };

  dashboard_filter_controls.data_connection = function() {
    return data_connections;
  };

  dashboard_filter_controls.build_mode = function(_) {
    if (!arguments.length) return build_mode;
    build_mode = _;
    return dashboard_filter_controls;
  };

  dashboard_filter_controls.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return dashboard_filter_controls;
  };

  dashboard_filter_controls.timeControl = function() {
    return timeControl;
  };

  dashboard_filter_controls.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    timeControl.dashboard(dashboard);
    return dashboard_filter_controls;
  };

  dashboard_filter_controls.active_dataconnection = function() {
    var dc_id = $(dom_id + " .dataconnections-sel").val();
    if (!dc_id) {
      return undefined;
    }

    return data_connections_hash[dc_id];
  };

  dashboard_filter_controls.active_dataset = function() {
    var ds_id = $(dom_id + " .datasets-sel").val();
    if (!ds_id) {
      return undefined;
    }

    return dataset_hash[ds_id];
  };

  dashboard_filter_controls.filterContainerVisibility = function() {
    // Show filter toggle if there are any dashboard filter controls
    if ($('.control-list').has('li').length) {
      $('#toggle-filter-container').removeClass('arc-invisible');
    } else {
      $('#toggle-filter-container').addClass('arc-invisible');
      $(".control-instructions").show();
    }
  };

  dashboard_filter_controls.load = function() {
    dashboard_filter_controls.handle_orientation();
    dashboard_filter_controls.filterContainerVisibility();
    return dashboard_filter_controls;
  };

  dashboard_filter_controls.handle_orientation = function() {
    var $control = $('.control-container');
    if (dashboard.controls_orientation() === "left") {
      // Only set classes and call resizeCallback (which leads to a full dashboard
      // redraw) if necessary
      if (!$control.hasClass("control-container-vert")) {
        $('.control-container').addClass('control-container-vert');
        $('.gridster-container').addClass('gridster-container-vert');
        dashboard.resizeCallback(true);
      }
    } else {
      if ($control.hasClass("control-container-vert")) {
        $('.control-container').removeClass('control-container-vert');
        $('.gridster-container').removeClass('gridster-container-vert');
        dashboard.resizeCallback(true);
      }
    }
  };

  function createFilter(config) {
    const dataset_id = dashboard.getDatasetId();
    if (dataset_id === undefined) {
      arc.report.alertMessage("Please add in a visual to this app " +
          "before adding in a parameter", '',
          { alert_report_id: dashboard.dashboard_id() });
      return;
    }

    const viz = dashboard_filter_controls.createControl({
      dataset_id,
      visual_type: config.type,
      single_select: true,
      picklistConfig: config.picklistConfig,
      userSettings: config.userSettings,
    });

    arc.visual.plugins.picklist.showSettings = true;
  }

  dashboard_filter_controls.getTimeControlSettings = function() {
    return timeControl.settings();
  };

  dashboard_filter_controls.setTimeControlSettings = function(_) {
    timeControl.settings(_);
  };

  dashboard_filter_controls.activate = function(do_not_fetch_apps) {
    timeControl.draw();
    timeControl.setupTimeControlActions();

    // Toggle filter section in dashboards
    $('#toggle-filter-container').off('click')
                                 .on('click',
        function() {
          $('.control-list').fadeToggle();
          $(this).toggleClass('closed');
        });

    if (!build_mode) {
      return dashboard_filter_controls;
    }

    if (dashboard && dashboard.reports() && dashboard.reports().numberReports()) {
      $('.create-newparam').prop('disabled', false);
    } else {
      $('.create-newparam').prop('disabled', true);
    }

    $('.create-newparam')
        .offOn('click.create', function handleCreate() {
          createFilter({
            type: 'picklist',
            picklistConfig: {
              picklistSettings: {
                title: 'New Filter',
              },
            },
          });
        });

    $(`${dom_id} .dataconnections-sel`)
        .offOn('change.updateDc', function handleDcSelection(e, cb) {
          dashboard_filter_controls.paint_datasets(cb);
        });

    $(`${dom_id} .datasets-sel`)
        .offOn('change.updateDs', function handleDsSelection(e, cb) {
          paintColumns(cb);
        });

    arc.misc.setupDimAggSearch(dom_id, '');

    if (!do_not_fetch_apps) {
      dashboard_filter_controls.fetch_and_paint();
    } else {
      isDataConnectionLoaded = true;
    }

    return dashboard_filter_controls;
  };

  function reqFail(config) {
    const {
      dataset,
      dataset_id,
      colname,
    } = config;

    dashboard_filter_controls.createControl({
      dataset,
      dataset_id,
      colname,
      visual_type: 'picklist',
    });
  }

  function reqSuccess(data, config) {
    let {
      dataset,
      dataset_id,
      colname,
    } = config;

    var dateConfig = arc.misc.columnIsDate(data.rows, 0);
    if (dateConfig && dateConfig.arcengineAutoDetect) {
      dashboard_filter_controls.createControl({
        dataset,
        dataset_id,
        colname,
        visual_type: 'daterange',
      });
    } else {
      dashboard_filter_controls.createControl({
        dataset,
        dataset_id,
        colname,
        visual_type: 'picklist',
      });
    }
  }

  dashboard_filter_controls.createControlFromColumn = function(config) {
    /*
     * Callers must specify 'dataset'
     * Callers either specify 'column', or they specify 'colname' and 'coltype'
     */
    let {
      dataset,
      column,
      coltype,
      colname,
      isDatasetSegment,
    } = config;

    if (isDatasetSegment) {
      const control = dashboard_filter_controls.createControl({
        dataset,
        dataset_id: dataset.dataset_id(),
        colname,
        visual_type: 'picklist',
        isDatasetSegment,
      });

      return dashboard_filter_controls;
    }

    if (coltype === undefined) {
      coltype = column.attr('type').toLowerCase();
      config.coltype = coltype;
    }

    if (colname === undefined) {
      colname = column.display_name();
      config.colname = colname;
    }

    let dataset_id = dataset.dataset_id();
    config.dataset_id = dataset_id;

    dashboard.hideInstructions();

    let isAggregate;

    if (!column) {
      column = dataset.get_column(colname);
    }

    if (column) {
      isAggregate = column.isAggregate();
    }

    let useRange = arc.report.coltypeIsReal(coltype);
    if (!useRange) {
      useRange = isAggregate && arc.report.coltypeIsInteger(coltype);
    }

    if (useRange) {
      dashboard_filter_controls.createControl({
        dataset,
        dataset_id,
        colname,
        visual_type: 'range',
      });

      return;
    }

    if (coltype === 'timestamp') {
      dashboard_filter_controls.createControl({
        dataset,
        dataset_id,
        colname,
        visual_type: 'daterange',
      });

      return;
    }

    var dc = arc.dataconnection.types[dataset.dataconnection_type()]();
    if (!dc.autoDateDetection) {
      dashboard_filter_controls.createControl({
        dataset,
        dataset_id,
        colname,
        visual_type: 'picklist',
      });

      return;
    }

    let rb = arc.data_requests.columnValues(null, dataset, colname, 100);

    arc.data.runQueryParams({
      rb,
      successCb: function(receivedData) {
        reqSuccess(receivedData, config);
      },
      failCb: function() {
        reqFail(config);
      },
    });
  };

  function callbackOnLiClick(evt) {
    var data = arc.report.shelf.getDatum($(evt.currentTarget));
    evt.stopPropagation();

    let coltype = data.getComputedType(),
        colname = data.dataset_colname(),
        dataset = data.dataset(),
        isDatasetSegment = data.isDatasetSegment();

    return dashboard_filter_controls.createControlFromColumn({
      dataset,
      coltype,
      colname,
      isDatasetSegment,
    });
  }

  function paintColumnsInternal(dataset, alwaysIncludeSegment) {
    arc.report.datasetSetupCols({
      dataset,
      includeAggregateColumns: false,
      $dimShelf: $('#dash-filters-tab .dim-app-params'),
      $aggShelf: $('#dash-filters-tab .agg-app-params'),
      config: {
        callback: callbackOnLiClick
      },
      includeSegment: true,
      alwaysIncludeSegment,
    });

    timeControl.setupTimeControls(dataset);

    arc.report.reportRefresh();
    return dashboard_filter_controls;
  }

  dashboard_filter_controls.createControl = function(config) {
    let {
      dataset,
      dataset_id,
      colname,
      visual_type,
      single_select,
      picklistConfig,
      userSettings,
      isDatasetSegment,
      filterSettings,
    } = config;

    var viz;
    if (visual_type === "daterange") {
      let dataRangeType = 'fixed';
      if (dataset && (dataset.dataconnection_type() === 'solr' ||
                      dataset.dataconnection_type() === 'elasticsearch')) {
        // For solr, create quick filters by default
        dataRangeType = 'quick_filters';
      }

      viz = arc.visual.plugins.daterange(dataRangeType);
    }

    if (visual_type === "picklist") {
      viz = arc.visual.plugins.picklist();
      if (single_select) {
        viz.picklist_single_select.value('true');
      }

      if (isDatasetSegment) {
        viz.isDatasetSegment(true);

        // For dataset segments, default filters shoulb be of scope DATASET, not GLOBAL.
        // It's hard/impossible to match up dataset segment ids across datasets.
        viz.filter_dataset_scope().value('true');
      }
    }

    if (visual_type === "range") {
      viz = arc.visual.plugins.range();
    }

    if (!viz) {
      return undefined;
    }

    if (picklistConfig) {
      viz.updateWithUrlParameters(picklistConfig.picklistSettings, picklistConfig);
    }

    if (colname) {
      viz.base_field(colname);
    }

    if (filterSettings) {
      viz.filterSettings(filterSettings);
    }

    var driver = arc.report.driver(viz);
    if (userSettings) {
      driver.restoreUserSettings(userSettings);
    }

    if (isDatasetSegment) {
      const settings = driver.user_settings();
      settings.find_by_id('picklist_base_field')
              .hidden(true);
      settings.find_by_id('picklist_title')
              .value(arc.report.segment_display_name);
      settings.find_by_id('picklist_output_parameter')
              .value(arc.report.segment_display_name)
              .hidden(true);
    }

    var viz_data = driver.save();
    var report_data = JSON.stringify({
      "report_type": visual_type,
      "report_data": viz_data,
    });

    var create_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
      "visual_type": visual_type,
      "dashboard_only": "true",
    };

    $.post(`${arc.frontend.globals.urls.reportCreateUrl}/${dataset_id}`, create_data, (data) => {
      var r = data[0];
      driver.report_id(r.id);

      var post_data = {
        "dataset_id": dataset_id,
        "report_type": visual_type,
        "report_data": report_data,
        "report_id": r.id,
        "csrfmiddlewaretoken": arc.globals.csrf_token,
      };

      $.post(`${arc.frontend.globals.urls.reportUrl}/${post_data.report_id}`, post_data, () => {
        arc.report.pagenav.needs_save(true);
        dashboard.addFilter(driver.report_id(), viz.viz_lib());
      })
      .fail(function(req, textStatus, errorThrown) {
        arc.report.alertMessage("Failed to save report",
            req.responseText ? req.responseText : req,
            { alert_report_id: dashboard.dashboard_id() });
      });
    })
    .fail(function(req, textStatus, errorThrown) {
      arc.report.alertMessage("Failed to create report",
          req.responseText ? req.responseText : req,
          { alert_report_id: dashboard.dashboard_id() });
    });

    dashboard.hideInstructions();
    return viz;
  };

  // cb is used to run the column update only after the dataset update
  function paintColumns(cb) {
    if (!isDataConnectionLoaded) return;

    var ds = dashboard_filter_controls.active_dataset();
    if (!ds) {
      return dashboard_filter_controls;
    }

    if (ds.dataset_type() === "singletable" && ds.needs_column_info()) {
      const assets = [
        {
          asset: `${arc.frontend.globals.urls.datasetUrl}/${ds.dataset_id()}?detail=true`,
          postprocess: d3.json,
        },
        {
          asset: arc.datasets.dataset_columns_url(ds.dataconnection_id(),
              ds.dataset_detail()),
          postprocess: d3.json,
        },
      ];

      if (arc.testutil && arc.testutil.testMode) {
        return;
      }

      arc.staticfetcher.fetch(assets,
        function(error, in_ds, columninfo) {
          if (error) {
            arc.report.alertMessage("Failed to load dataset table column detail",
                error.response ? error.response : error,
                { alert_report_id: dashboard.dashboard_id() });
            return;
          }

          var ds = arc.datasets.Dataset()
                      .init_from_fetch(in_ds[0], columninfo);
          dataset_hash[in_ds.id] = ds;
          paintColumnsInternal(ds);
          setup_draggable_filter_fields();

          // cb is an object and not a function when called through chosen
          if (cb && typeof (cb) === 'function') {
            cb();
          }
        });
    } else {
      const assets = [
        {
          asset: `${arc.frontend.globals.urls.datasetUrl}/${ds.dataset_id()}?detail=true`,
          postprocess: d3.json,
        },
      ];

      /*
       * Fetch dataset detail to see if segments exist in the dataset.
       *
       * When we do the initial fetch of datasets, we only fetch the dataset
       * names, and do not fetch dataset details. So we fetch those details
       * now.
       */
      arc.staticfetcher.fetch(assets,
        function(error, in_ds) {
          if (error) {
            arc.report.alertMessage("Failed to load dataset detail",
                error.response ? error.response : error,
                { alert_report_id: dashboard.dashboard_id() });
            return;
          }

          let alwaysIncludeSegment = false;
          if (in_ds[0].segments && in_ds[0].segments.length > 0) {
            alwaysIncludeSegment = true;
          }

          paintColumnsInternal(ds, alwaysIncludeSegment);
          setup_draggable_filter_fields();

          // cb is an object and not a function when called through chosen
          if (cb && typeof (cb) === 'function') {
            cb();
          }
        });
    }

    return dashboard_filter_controls;
  }

  function setup_draggable_filter_fields() {
    $('#dash-filters-tab .selection_element')
      .draggable({
        appendTo: 'body',
        helper: 'clone',
        cursor: 'pointer',
        opacity: 0.5,
        zIndex: 1040,
        revert: 'invalid',
        start(_, ui) {
          ui.helper.addClass('field-drag-helper');
        },
        stop(_, ui) {
          ui.helper.removeClass('field-drag-helper');
        },
    });
  }

  dashboard_filter_controls.paint_datasets = function(cb) {
    if (!isDataConnectionLoaded) return;

    var dc = dashboard_filter_controls.active_dataconnection();
    if (!dc) {
      return dashboard_filter_controls;
    }

    let $solr_controls = $('#dash-filters-tab .solr-controls');
    if (dc.dataconnection_type() === 'solr' ||
        dc.dataconnection_type() === 'elasticsearch') {
      $solr_controls.removeClass('arc-invisible');
      $solr_controls.find('.create-search')
                    .off('click')
                    .on('click', function() {
                      createFilter({
                        type: 'picklist',
                        picklistConfig: {
                          picklistSettings: {
                            title: 'Search',
                          },
                        },
                        userSettings: {
                          picklist_textbox: 'true',
                          picklist_output_parameter: 'search',
                          picklist_width: '400',
                        },
                      });
                    });
    } else {
      $solr_controls.addClass('arc-invisible');
    }

    // Remove all current datasets
    $(dom_id + " .datasets-sel option").remove();

    // Repaint with datasets in the active dat connection
    arc.dataconnection.sortDataSets(dc.datasets());

    dc.datasets().filter(function (d) {
      return arc.misc.hasPerm('ds_appedit', d.dataset_perms());
     }).forEach(function(ds) {
      var m = {
        'DATASET_ID': ds.dataset_id(),
        'DATASET_NAME': ds.dataset_name(),
      };
      $(dom_id + " .datasets-sel").append(
          arc.misc.safe_substitute($("#dataset-sidebar-item").html(),
              m));
    });

    $('.datasets-sel').trigger('chosen:updated');

    if (cb && typeof (cb) === 'function') {
      cb();
    }

    return dashboard_filter_controls;
  };

  dashboard_filter_controls.paint = function() {
    $(`${dom_id} .dataconnections-sel option`).remove();

    data_connections.forEach(function(data_connection) {
      if (data_connection.datasets().length) {
        var m = {
          'DATACONNECTION_ID': data_connection.dataconnection_id(),
          'DATACONNECTION_NAME': data_connection.dataconnection_name(),
        };
        $(dom_id + " .dataconnections-sel").append(
          arc.misc.safe_substitute($("#dataconnection-sidebar-item").html(),
          m));
      }
    });

    dashboard_filter_controls.paint_datasets();
    return dashboard_filter_controls;
  };

  dashboard_filter_controls.fetch_and_paint = function() {
    dashboard_filter_controls.fetch(function dashboardFilterFetchAndPaint() {
      dashboard_filter_controls.paint();
    });
  };

  dashboard_filter_controls.fetch = function(cb) {
    if (arc.testutil && arc.testutil.testMode) {
      return;
    }

    isDataConnectionLoaded = false;

    var dataconnections_asset = {
      asset: arc.frontend.globals.urls.dataconnectionUrl,
      postprocess: d3.json
    };
    var datasets_asset = {
      asset: arc.frontend.globals.urls.datasetUrl,
      postprocess: d3.json
    };
    arc.staticfetcher.fetch([datasets_asset, dataconnections_asset],
      function(error, in_datasets, in_dataconnections) {
        if (error) {
          arc.report.alertMessage("Failed to load data connections",
              error, { alert_report_id: dashboard.dashboard_id() });
          return;
        }

        data_connections = [];
        data_connections_hash = {};
        dataset_hash = {};

        in_dataconnections.forEach(function(dc) {
          var dataconn = arc.dataconnection.DataConnection()
                            .load(dc);
          data_connections_hash[dc.id] = dataconn;
          data_connections.push(dataconn);
        });

        arc.dataconnection.sortDataConnections(data_connections);

        in_datasets.forEach(function(in_ds) {
          var ds = arc.datasets.Dataset()
                      .init_from_fetch(in_ds);
          dataset_hash[in_ds.id] = ds;
          if (in_ds.dc_id in data_connections_hash) {
            var dc = data_connections_hash[in_ds.dc_id];
            dc.add_dataset(ds);
          }
        });

        isDataConnectionLoaded = true;

        cb();
      });
  };

  return dashboard_filter_controls;
};
}());
