window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

arc.visual.dashboardResizer = function() {
  const dashboardResizer = function() { return 'dashboardResizer'; };

  const scaleFactor = 8,
        datasetColor = d4.scaleOrdinal(d4.schemeCategory10);

  let dashboard,
      modalDivId,
      forResize = true,
      $modal,
      numColumns,
      widgetInfo,
      clearCanvasFn,
      context,
      moveStartX,
      moveStartY,
      moveWidget,
      moveWidgetResize,
      moveWidgetColor = 'steelblue',
      yMax;

  dashboardResizer.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    return dashboardResizer;
  };

  dashboardResizer.forResize = function(_) {
    if (!arguments.length) return forResize;
    forResize = _;
    return dashboardResizer;
  };

  dashboardResizer.colors = function(_) {
    return datasetColor;
  };

  dashboardResizer.modalDivId = function(_) {
    if (!arguments.length) return modalDivId;
    modalDivId = _;
    $modal = $(`#${modalDivId}`);
    return dashboardResizer;
  };

  dashboardResizer.modal = function() {
    let $m = $('#dashboard-resizer-modal');
    if (arc.misc.modalShown($m)) {
      return;
    }

    $('#dashboard-resizer-modal').remove();
    $('body').append(modalHTML);
    $m = $('#dashboard-resizer-modal');

    $m.modal('show', {
      keyboard: true,
    });

    $m.off('shown.bs.modal')
          .on('shown.bs.modal', () => {
            dashboardResizer.modalDivId('dashboard-resizer-modal');
            dashboardResizer.paintLayout();
            setupActions();
          });

    return dashboardResizer;
  };

  const defaultFontSizeInPx = 12;
  const defaultFontType = 'Roboto';
  const defaultFont = `${defaultFontSizeInPx}px ${defaultFontType}`;
  const ellipsis = '...';

  function truncateTextToFitInSpace(text, maxWidth, ctx) {
    const widthOfTitle = ctx.measureText(text).width;
    let visibleTitle = `${text}`;
    if (maxWidth < widthOfTitle) {
      visibleTitle = '';
      for (let i = text.length - 2; i > 0; i--) {
        const substr = `${text.substring(0, i)}${ellipsis}`;
        if (ctx.measureText(substr).width < maxWidth) {
          visibleTitle = substr;
          break;
        }
      }
    }

    return visibleTitle;
  }

  const margin = 3;
  const rectangleOffset = 21; // the blue rectangle size
  const bottomNumberAndIdSpace = 30; // the height of the ID: 123 and the w.size_x

  function drawImgOntoCanvas(img, widget, ctx) {
    const r = img.width / img.height;
    const maxImgHeight = (widget.endY -
      widget.startY -
      2 * rectangleOffset -
      bottomNumberAndIdSpace
    );
    const maxImgWidth = (widget.endX - widget.startX - 2 * rectangleOffset);
    const R = maxImgWidth / maxImgHeight;

    let displayHeight = maxImgHeight;
    let displayWidth = maxImgWidth;
    const cropInfo = widget.imgData?.cropInfo || {
      x: 0,
      y: 0,
      height: img.height,
      width: img.width,
    };
    // fit image size to the widget size
    if (R > r) {
      displayWidth = r * maxImgHeight;
    } else {
      displayHeight = maxImgWidth / r;
    }

    const { imageSizeLimit } = widget.imgData;
    if (imageSizeLimit) {
      displayHeight = displayHeight > imageSizeLimit ? imageSizeLimit : displayHeight;
      displayWidth = displayWidth > imageSizeLimit ? imageSizeLimit : displayWidth;
    }

    const startX = widget.startX + (maxImgWidth - displayWidth) / 2 + rectangleOffset;
    const startY = widget.startY + (maxImgHeight - displayHeight) / 2 + rectangleOffset;

    ctx.drawImage(
      img,
      cropInfo.x,
      cropInfo.y,
      cropInfo.width,
      cropInfo.height,
      startX,
      startY,
      displayWidth,
      displayHeight
    );
  }

  const chartIconUrl = `${arc.frontend.globals.urls.staticUrl}/img/chart-icons@2x.png`;
  const filterIconUrl = `${arc.frontend.globals.urls.staticUrl}/img/filter-chart.png`;

  function paintWidget(w, fillStyle) {
    try {
      const report = dashboard.reports()?.getReport(w.visualId);
      if (report && report.viz()) {
        const visualType = report.viz().viz_type();
        const imageSizeLimit = 24;
        if (arc.misc.isFilterVisual(report.viz().viz_type())) {
          w.title = report.viz().subtitle() || 'Filter';
          w.imgData = { url: filterIconUrl, imageSizeLimit };
        } else {
          w.title = report.viz().title() || 'Untitled';

          // report snapshot from the served - doesn't contain the new changes!
          const serverSideReport =
            arc.globals.dashboard.reports_hash()[w.visualId];
          // if the visual type changed - the snapshot image shows the previous type!
          // when snapshot is not enabled hard to guess the img reliability
          const isSnapshotConsistentWithTheVisual =
            arc.globals.enable_thumbnails &&
            serverSideReport?.type === visualType;

          const imgUrl = serverSideReport?.img;
          if (isSnapshotConsistentWithTheVisual && imgUrl) {
            w.imgData = { url: imgUrl };
          } else {
            let iconPos = $(`.chart-icon-${visualType}`).css(
              "background-position"
            );
            let cropInfo;
            if (iconPos) {
              iconPos = iconPos
                .split("px")
                .map((x) => Math.abs(parseInt(x.trim(), 10)))
                .slice(0, 2);

              cropInfo = {
                x: iconPos[0] * 2,
                y: iconPos[1] * 2,
                height: 48,
                width: 48,
              };
            }

            w.imgData = { cropInfo, url: chartIconUrl, imageSizeLimit };
          }
        }

        w.datasetId = report.dataset_id();
      }
    } catch (e) {
      console.log(e);
      return;
    }

    context.beginPath();
    context.rect(w.startX, w.startY, 15, 15);
    context.fillStyle = datasetColor(w.datasetId);
    context.fill();

    context.fillStyle = fillStyle || 'grey';

    context.beginPath();
    context.rect(w.startX, w.startY, w.endX - w.startX, w.endY - w.startY);
    context.strokeStyle = '#aaa';
    context.stroke();

    if (forResize) {
      context.beginPath();
      context.moveTo(w.endX - 5, w.endY - 10);
      context.lineTo(w.endX - 5, w.endY - 5);
      context.lineTo(w.endX - 10, w.endY - 5);
      context.stroke();
    }

    if (w.size_x >= 10 && w.size_y >= 10) {
      // Display visual IDs if there is enough space
      context.textBaseline = 'middle';
      context.textAlign = 'center';
      context.fillText(`Id:  ${w.visualId}`, (w.startX + w.endX) / 2, (w.endY - 35) / 1);
    }

    // Display visual icons if there is enough space
    if (w.size_x >= 10 && w.size_y >= 10) {
      const img = new Image();
      img.onload = function () {
          drawImgOntoCanvas(img, w, context);
      };

      img.src = w.imgData.url;
    }

    if (forResize) {
      context.textBaseline = 'bottom';
      context.textAlign = 'center';
      context.fillText(`${w.size_x}`, (w.startX + w.endX) / 2, w.endY - margin);

      context.textBaseline = 'top';
      context.textAlign = 'left';
      const gridSizeYStartCoordinate = { x: w.startX + rectangleOffset, y: w.startY + margin };
      context.fillText(`${w.col}`, gridSizeYStartCoordinate.x, gridSizeYStartCoordinate.y);

      context.textBaseline = 'middle';
      context.textAlign = 'right';
      context.fillText(`${w.size_y}`, w.endX - margin, (w.startY + w.endY) / 2);

      context.textBaseline = 'top';
      context.textAlign = 'left';
      context.fillText(`${w.row}`, w.startX + margin, w.startY + rectangleOffset);

      // Display Title, next to the grid size number
      context.textBaseline = 'top';
      context.textAlign = 'left';
      context.font = `${defaultFontSizeInPx + 1}px ${defaultFontType}`;
      // full widget width minus the blue rectangle and the number width from the left corner
      const maxTitleWidth = (w.endX - w.startX) - rectangleOffset - context.measureText(`${w.col}`).width - margin;

      const visibleTitle = truncateTextToFitInSpace(w.title, maxTitleWidth, context);
      context.fillText(visibleTitle, gridSizeYStartCoordinate.x + context.measureText(`${w.col}`).width + margin, gridSizeYStartCoordinate.y + margin);
      context.font = defaultFont;
    }
  }

  function createCanvas() {
    yMax = 0;
    widgetInfo.forEach(function(w) {
      const y = w.row + w.size_y;
      yMax = Math.max(yMax, y);
    });
    // eslint-disable-next-line no-undef
    let canvas = d4.selectAll(`#${modalDivId} .rearrange-canvas canvas`);
    arc.canvas.adjustPixelRatio(canvas, (1 + numColumns) * scaleFactor + 2,
        (1 + yMax) * scaleFactor + 2);
    context = canvas.node().getContext('2d');

    clearCanvasFn = function() {
      context.clearRect(
          0, 0,
          (1 + numColumns) * scaleFactor + 2,
          (1 + yMax) * scaleFactor + 2
      );

      if (forResize) {
        context.strokeStyle = '#eee';
        for (let x = 1; x <= numColumns; ++x) {
          context.beginPath();
          context.moveTo(x * scaleFactor, 0);
          context.lineTo(x * scaleFactor, yMax * scaleFactor);
          context.stroke();
        }

        for (let y = 1; y <= yMax; ++y) {
          context.beginPath();
          context.moveTo(0, y * scaleFactor);
          context.lineTo(numColumns * scaleFactor, y * scaleFactor);
          context.stroke();
        }
      }
    };

    clearCanvasFn();
  }

  function paintAllWidgets() {
    clearCanvasFn();

    context.lineWidth = 2;
    context.font = defaultFont;

    widgetInfo.forEach(function(w) {
      paintWidget(w);
    });
  }

  function computeAllWidgetXY() {
    widgetInfo.forEach(function(w) {
      const x = w.row + w.size_y;

      w.visualId = w.id.split("-");
      w.visualId = w.visualId[w.visualId.length - 1];

      w.startX = w.col * scaleFactor;
      w.startY = w.row * scaleFactor;
      w.endX = w.startX + (w.size_x - 1) * scaleFactor;
      w.endY = w.startY + (w.size_y - 1) * scaleFactor;
    });
  }

  dashboardResizer.paintLayout = function() {
    numColumns = dashboard.numColumns();
    widgetInfo = dashboard.widgetInfo();

    computeAllWidgetXY();
    createCanvas();
    paintAllWidgets();

    return dashboardResizer;
  };

  function setupActions() {
    $modal.find('button.rearrange')
          .on('click.rearrange', function() {
            const newWidgets = widgetInfo.map(function(w) {
              return {
                col: w.col,
                row: w.row,
                size_x: w.size_x,
                size_y: w.size_y,
                id: w.id,
              };
            });

            dashboard.resetupGridster(newWidgets);
            arc.report.pagenav.needs_save(true);
          });

    function unsetMoveWidget() {
      if (moveWidget !== undefined) {
        paintWidget(moveWidget);
        moveWidget = undefined;
      }
    }

    function setMoveWidget(w, e, resize) {
      unsetMoveWidget();
      moveWidget = w;
      moveWidgetResize = resize;
      paintWidget(moveWidget, moveWidgetColor);

      moveStartX = e.offsetX;
      moveStartY = e.offsetY;
    }

    function mousemove() {
      if (moveWidget === undefined) {
        return;
      }

      if (moveWidgetResize) {
        mousemoveResize();
      } else {
        mousemoveRearrange();
      }

      computeAllWidgetXY();
      createCanvas();
      paintAllWidgets();
      paintWidget(moveWidget, moveWidgetColor);
    }

    function mousemoveResize() {
      const e = d4.event;
      const x = e.offsetX;
      const y = e.offsetY;

      let colOffset = Math.floor((x - moveStartX) / scaleFactor);
      let rowOffset = Math.floor((y - moveStartY) / scaleFactor);

      if (moveWidget.col + moveWidget.size_x + colOffset > (numColumns + 1)) {
        colOffset = (numColumns + 1) - (moveWidget.col + moveWidget.size_x);
      }

      if (moveWidget.row + moveWidget.size_y + rowOffset > (yMax + 1)) {
        rowOffset = (yMax + 1) - (moveWidget.row + moveWidget.size_y);
      }

      if (moveWidget.size_x + colOffset < 5) {
        colOffset = moveWidget.size_x - 5;
      }

      if (moveWidget.size_y + rowOffset < 5) {
        rowOffset = moveWidget.size_y - 5;
      }

      moveStartX += colOffset * scaleFactor;
      moveStartY += rowOffset * scaleFactor;

      moveWidget.size_x += colOffset;
      moveWidget.size_y += rowOffset;
    }

    function mousemoveRearrange() {
      const e = d4.event;
      const x = e.offsetX;
      const y = e.offsetY;

      let colOffset = Math.floor((x - moveStartX) / scaleFactor);
      let rowOffset = Math.floor((y - moveStartY) / scaleFactor);

      if (moveWidget.col + moveWidget.size_x + colOffset > (numColumns + 1)) {
        colOffset = (numColumns + 1) - (moveWidget.col + moveWidget.size_x);
      }

      if (moveWidget.row + moveWidget.size_y + rowOffset > (yMax + 1)) {
        rowOffset = (yMax + 1) - (moveWidget.row + moveWidget.size_y);
      }

      if (moveWidget.col + colOffset < 1) {
        colOffset = moveWidget.col - 1;
      }

      if (moveWidget.row + rowOffset < 1) {
        rowOffset = moveWidget.row - 1;
      }

      moveStartX += colOffset * scaleFactor;
      moveStartY += rowOffset * scaleFactor;

      moveWidget.col += colOffset;
      moveWidget.row += rowOffset;
    }

    function mousedown() {
      const e = d4.event;
      const x = e.offsetX;
      const y = e.offsetY;

      unsetMoveWidget();

      widgetInfo.some(function(w) {
        if ((x >= w.startX) && (x <= w.endX) &&
            (y >= w.startY) && (y <= w.endY)) {

          setMoveWidget(w, e, ((w.endX - x) < 10 && (w.endY - y) < 10));
          return true;
        }
      });
    }

    function mouseup() {
      // moveWidget is in its final place.
      // Look for overlaps and fix

      let actions = handleOverlaps();
      let cnt = 0;
      while (actions > 0 && cnt < 5) {
        ++cnt;
        actions = handleOverlaps();
      }

      createCanvas();
      paintAllWidgets();
      unsetMoveWidget();
    }

    function handleOverlaps() {
      if (moveWidget === undefined) {
        return;
      }

      let actions = 0;
      let current = [moveWidget];

      // Sort widgets by row, and then by column
      function sortWidgets() {
        widgetInfo.sort(function(a, b) {
          if (a.row < b.row) {
            return -1;
          }

          if (a.row > b.row) {
            return 1;
          }

          if (a.col < b.col) {
            return -1;
          }

          if (a.col > b.col) {
            return 1;
          }

          return 0;
        });
      }

      sortWidgets();

      // First, start with moveWidget as the list of current widgets.
      // Then, move one widget at a time, and change their position so
      //   they do not overlap with the current set of widgets.
      // Finally, add in that non-overlapping widget to the list of current
      //   widgets.
      widgetInfo.forEach((w) => {
        if (w === moveWidget) {
          return;
        }

        current.forEach((ww) => {
          if ((w.endX >= ww.startX) &&
              (w.startX <= ww.endX) &&
              (w.endY >= ww.startY) &&
              (w.startY <= ww.endY)) {

            // overlap - move it down
            ++actions;
            w.row = ww.row + ww.size_y;
            w.startY = w.row * scaleFactor;
            w.endY = w.startY + (w.size_y - 1) * scaleFactor;
          }
        });

        current.push(w);
      });

      sortWidgets();

      // Now, squish up any empty rows
      let currentY = 1;
      let compactY = 0;
      widgetInfo.forEach((w) => {
        if (w.row > currentY) {
          compactY += (w.row - currentY);
          currentY += (w.row - currentY) + w.size_y;
        }

        ++actions;
        w.row -= compactY;
        w.startY = w.row * scaleFactor;
        w.endY = w.startY + (w.size_y - 1) * scaleFactor;

        if (w.row + w.size_y > currentY) {
          currentY = w.row + w.size_y;
        }
      });

      return actions;
    }

    const canvas = d4.selectAll(`#${modalDivId} .rearrange-canvas canvas`);
    canvas.on('mousedown.rearrange', mousedown);
    canvas.on('mouseup.rearrange', mouseup);
    canvas.on('mouseout.rearrange', mouseup);
    canvas.on('mousemove.rearrange', mousemove);
  }

  dashboardResizer.setup = function() {
    $('.js-open-rearrange-dashboard').offOn('click.rearrange', dashboardResizer.modal);
    return dashboardResizer;
  };

  return dashboardResizer;
};

const modalHTML = `
<div id="dashboard-resizer-modal" class="modal fade fouc" tabindex="-1" style="display:none;">
  <div class="modal-dialog modal-lg">
    <div class="modal-content">
      <form onsubmit="return false;">
        <div class="modal-header">
          <h4 class="modal-title">Layout of Dashboard Visuals</h4>
          <button type="button" class="close" data-dismiss="modal" aria-hidden="true">
            &times;
          </button>
        </div>

        <div class="modal-body">
          <div class="rearrange-contents arc-no-dragging">

            <div style="padding-bottom: 15px;">
              <div>
                Click & drag the widgets below to resize and rearrange them.
              </div>
              <div>
                Top left color squares represent datasets.
              </div>
            </div>
            <div class="rearrange-canvas" style="width: 100%; text-align:center;">
              <canvas style="display: inline;">
              </canvas>
            </div>

          </div>
        </div>

        <div class="modal-footer">
          <button type="button" class="btn btn-secondary" data-dismiss="modal">Cancel</button>
          <button type="button" class="btn btn-primary rearrange" data-dismiss="modal">
            Apply Layout
          </button>
        </div>
      </form>
    </div>
  </div>
</div>
`;

})();
