window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.user_settings = arc.visual.user_settings || {};

arc.report = arc.report || {};

const dashboardPopoverContentTemplate = `
  <div class="%POPOVER_ID%">
    <div class="popover-header">%TITLE%</div>
    <table class="popover-table">
      <tr>
        <td class="header-label">ID:</td>
        <td>%ID%</td>
      </tr>
      <tr>
        <td class="header-label">Filters:</td>
        <td>%HTML_FILTERS%</td>
      </tr>
      <tr>
        <td class="header-label">Visuals:</td>
        <td>%HTML_VISUALS%</td>
      </tr>
      <tr>
        <td class="header-label">Datasets:</td>
        <td>%HTML_DATASETS%</td>
      </tr>
      <tr>
        <td class="header-label">Dataconnections:</td>
        <td>%HTML_DATACONNECTIONS%</td>
      </tr>
      <tr>
        <td class="header-label">Workspace:</td>
        <td>%WORKSPACE_NAME%</td>
      </tr>
    </table>
  </div>
`;

const visualPopoverContentTemplate = `
  <div class="%POPOVER_ID%">
    <div class="popover-header %PICKLIST_HEADER_CLASS%">%TITLE%</div>
    <table class="popover-table">
      <tr>
        <td class="header-label">ID:</td>
        <td>%ID%</td>
      </tr>
      <tr>
        <td class="header-label">Last updated:</td>
        <td>%LAST_UPDATED_DATETIME%</td>
      </tr>
      <tr>
        <td class="header-label">Modified by:</td>
        <td>%LAST_UPDATED_USER%</td>
      </tr>
      <tr>
        <td class="header-label">Dataset:</td>
        <td><a href="%DATASET_URL%" target="_blank">%DATASET_NAME%</a></td>
      </tr>
      <tr class="%TAGS_VISIBILITY%">
        <td class="header-label">Dataset Tags:</td>
        <td>%HTML_DATASET_TAGS%</td>
      </tr>
      <tr>
        <td class="header-label">Dataconnection:</td>
        <td><a href="%DATACONNECTION_URL%" target="_blank">%DATACONNECTION_NAME%</a></td>
      </tr>
      <tr>
        <td class="header-label">Workspace:</td>
        <td>%WORKSPACE_NAME%</td>
      </tr>
      <tr class="shared-dashboards %DASHBOARD_USAGE_CLASS%">
        <td class="header-label">Used in Dashboards:</td>
        <td>%HTML_DASHBOARD_USAGE%</td>
      </tr>
    </table>
  </div>
`;

/*
 * There are 3 visual side tabs in the dashboard editor - Shelves, Settings & Style.
 * Track the side tab visual elements here, they are used in the code below.
 */
const visualSideTabs = {
  shelves: {
    $tab: $('#dash-visual-data-tab'),
    $display: $('#visual-data-tab'),
  },
  settings: {
    $tab: $('#dash-visual-settings-tab'),
    $display: $('#visual-settings-tab'),
  },
  style: {
    $tab: $('#dash-visual-style-tab'),
    $display: $('#visual-style-tab'),
  },
};

/*
 * We try and remember the last open visual side tab so we can open up the appropriate
 * one as the user is clicking between visuals.
 */
let lastSelectedVisualSideTab = 'settings';

function setupTabMemory() {
  $.each(visualSideTabs, (tabName, tabData) => {
    tabData.$display.offOn('show.bs.tab.tracker', () => {
      lastSelectedVisualSideTab = tabName;
    });
  });
}

function widgetId(report) {
  return report.reportWidgetId();
}

function hideTooltip() {
  $('#tooltip').fadeOut();
}

function dashboardSearchHandler() {
  const s = function() {
    return 'dashboardSearchHandler';
  };

  let lastActivated;
  let currentSearchWidget;
  let dashboard;

  s.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    return s;
  };

  function searchEnabled(report) {
    if (!arc.frontend.featureFlipper.getValueById('site_enable_search')) {
      return false;
    }

    const dataset = report.dataset();
    if (!dataset) {
      return false;
    }

    if (report.viz() && arc.misc.isFilterVisual(report.viz().viz_type())) {
      return false;
    }

    const searchSettings = dataset.searchSettings();
    return (searchSettings.enabled === true);
  }

  s.setupSearch = function(report) {
    const $sel = $(`#${widgetId(report)} .widget-btns-wrapper .search-app`);

    if (!searchEnabled(report)) {
      $sel.hide()
          .off('click.search-app');
      return;
    }

    // Activate search on click
    $sel.show()
        .offOn('click.search-app', function launchSearch() {
          hideTooltip();

          if (currentSearchWidget && currentSearchWidget.active()) {
            currentSearchWidget.deactivate();

            if (currentSearchWidget.report() === report) {
              return;
            }
          }

          s.activateSearch(report);
        });
  };

  // If report data is available after activateSearch is called,
  // this next call does the actual activation.
  s.activateSearchIfLastActivated = function(report) {
    if (report === lastActivated) {
      s.activateSearch(report);
    }
  };

  s.activateSearch = function(report) {
    lastActivated = undefined;

    if (!searchEnabled(report)) {
      // When loading reports on page load, we end up in this code before we
      // have a report dataset available. In that case, remember that we
      // tried to activate this report so we can activate it when the
      // dataset becomes available
      if (report.dataset() === undefined) {
        lastActivated = report;
      }

      return;
    }

    if (dashboard) {
      dashboard.activateReport(report);
    }

    const wid = `#${widgetId(report)}`;
    currentSearchWidget = arc.search.dashboardWidget();
    currentSearchWidget.gridster(dashboard.gridster())
                       .wid(wid)
                       .report(report)
                       .activate();

    return currentSearchWidget;
  };

  return s;
}

arc.visual.createTitleSubtitleArea = function(config) {
  const {
    d3Container,
    titleId,
    subtitleId,
    filterId,
    chartId,
  } = config;

  if (titleId) {
    d3Container.append('div')
       .attr('class', 'viz-permanent')
       .html(
         arc.misc.safe_substitute(
             $('#title-template').html(),
             {
               TITLE: titleId,
             }
          )
        );
  }

  if (filterId) {
    d3Container.append('div')
       .attr('class', 'viz-permanent')
       .html(
         arc.misc.safe_substitute(
             $('#filter-template').html(),
             {
               FILTER: filterId,
             }
          )
        );
  }

  if (subtitleId) {
    d3Container.append('div')
       .attr('class', 'viz-permanent')
       .html(
         arc.misc.safe_substitute(
             $('#subtitle-template').html(),
             {
               SUBTITLE: subtitleId,
             }
          )
        );
  }

  if (arc.globals.is_edit) {
    if (titleId) {
      arc.editTitleHelper('does-not-exist', titleId);
    }

    if (subtitleId) {
      arc.editTitleHelper('does-not-exist', subtitleId);
    }
  }

  d3Container.append('div')
     .attr('class', 'dash-viz-wrapper')
     .append('div')
     .attr('id', chartId)
     .attr('class', 'viz-permanent viz-chart');
};

arc.dashboard = function() {
  let displaydb;
  let gridster; // true for view only mode of dashboard/false in edit mode
  let widget_base_dim_px;
  let reports; // these are all reports, not those of just the active dashboard
  let dashboard_id;
  let metadataVersion = 2;
  let sheet_id;
  let currentSheet;
  let currentDataSetSelection;
  let onSaveDashboardCb;
  const params_history = arc.report.params_history();
  let click_behavior = {};
  let last_dimensions;

  let dashboard_filter_controls;

  // Tracks when resizing is ongoing, don't permit dashboard saving concurrently
  let resizing;

  // For showing/hiding single visual dashboard
  let showSingleVisual;
  const $appContainer = $('.arc-app-container');
  const $gridsterUl = $appContainer.find('.gridster-container').find('ul');
  const $dashSheetContainer = $('.dash-sheets-container');
  const $buildContainer = $('.builder-container, .builder-container-app');

  let data_connections = [];
  let data_connections_hash = {};
  let dataset_hash = {};
  let reports_all_data = [];
  const reports_by_dataset = {};
  let reports_hash = {};

  // these are the connections that the dashboard actually uses
  let active_connection_ids = [];

  // Tracks controls and unlinked visuals to delete when this dashboard is saved
  let reports_to_delete = [];
  let reports_to_update = [];
  let originalVisuals;

  // Workspace
  let workspaces;
  let order;

  const user_settings = arc.visual.dashboardSettings();
  const dashboardWidthSetting = user_settings.find_by_id('dashboard_width');
  const singleVisualDashboard = user_settings.find_by_id('single_visual_dashboard');
  const manualVisualRefresh = user_settings.find_by_id('manual_visual_refresh');
  const autoRefresh = user_settings.find_by_id('auto_refresh_seconds');
  const enableAnimations = user_settings.find_by_id('dashboard_enable_animations');
  const dashboardHideTitle = user_settings.find_by_id('dashboard_hide_title');
  const dashboardBgColor = user_settings.find_by_id('dashboard_bg_color');
  const dashboardFgColor = user_settings.find_by_id('dashboard_fg_color');
  const pageBgColor = user_settings.find_by_id('page_bg_color');
  const appTitleBgColor = user_settings.find_by_id('app_title_bg');
  const appTitleFgColor = user_settings.find_by_id('app_title_fg');
  const dashboardTitleFontWeight =
                  user_settings.find_by_id('dashboard_title_font_weight');
  const dashboardSubtitleBgColor = user_settings.find_by_id('dashboard_subtitle_bg_color');
  const dashboardSubtitleFgColor = user_settings.find_by_id('dashboard_subtitle_fg_color');
  const visualBgColor = user_settings.find_by_id('visual_bg_color');
  const visualBorderColor = user_settings.find_by_id('visual_border_color');
  const visualBorderRadius = user_settings.find_by_id('visual_border_radius');
  const visualTitleBgColor = user_settings.find_by_id('visual_title_bg');
  const visualTitleFgColor = user_settings.find_by_id('visual_title_fg');
  const visualSubtitleBgColor = user_settings.find_by_id('visual_subtitle_bg');
  const visualSubtitleFgColor = user_settings.find_by_id('visual_subtitle_fg');
  const filterTitleColor = user_settings.find_by_id('filter_title_color');
  const filterSelectionColor = user_settings.find_by_id('filter_selection_color');
  const filterBgColor = user_settings.find_by_id('filter_bg_color');
  const filterContainerBgColor = user_settings.find_by_id('filter_container_bg_color');
  const filterInlineLabels = user_settings.find_by_id('filter_inline_labels');
  const filterLabelLine = user_settings.find_by_id('filter_label_line');
  const bordersAroundFilters = user_settings.find_by_id('borders_around_filters');
  const bordersAroundVisuals = user_settings.find_by_id('borders_around_visuals');
  const filtersOnLeft = user_settings.find_by_id('filters_on_left');
  const horizontalMarginBetweenVisuals =
                  user_settings.find_by_id('horizontal_margin_between_visuals');
  const verticalMarginBetweenVisuals = user_settings.find_by_id('vertical_margin_between_visuals');
  const pngDownload = arc.report.user_settings.permit_png_download();
  const pdfScale = arc.report.user_settings.print_pdf_scale();
  const pdfSize = arc.report.user_settings.pdf_size();
  const pdfOrientation = arc.report.user_settings.pdf_orientation();
  const pdfHeaderFooter = arc.report.user_settings.pdf_enable_header_footer();
  const permitEmail = arc.report.user_settings.permit_email();
  const csvDownload = arc.report.user_settings.permit_csv_download_dashboard();
  const caseSensitive = arc.settings.dashboards.caseSensitiveParameters();
  const visualsAfterFilters = arc.settings.dashboards.visualsAfterFilters();

  // Used for making app title/subtitle accept parameters,
  let dashboard_data;
  let dashboard_name;
  let dashboard_mode;

  // Track appgroup-provided global filters
  let globalFilters;

  // For keeping track of how many filters have been loaded
  let filtersToBeLoaded = 0;
  let filtersLoaded = 0;
  let initialParamCount = 0;
  let allFilterCount = 0;
  const initialParams = [];

  let dashboard_title;
  let dashboard_subtitle;

  let autoRefreshSeconds;
  let numColumns = 64;
  const widgetXColumns = numColumns / 2;
  const widgetYColumns = 20;
  const filterXColumns = numColumns / 4;
  const filterYColumns = numColumns / 4;

  const searchHandler = dashboardSearchHandler();

  let highlightedVisual;

  const linkedVisualTemplate = '<span class="linked-visual-icon" title="%TITLE_AND_ID%"></span>';
  const unlinkedVisualTemplate = '<span class="dashboard-link-btn"' +
                               'title="Containing Dashboard: %TITLE_AND_ID%"></span>';

  user_settings.addSetting([
    pngDownload,
    pdfScale,
    pdfSize,
    pdfOrientation,
    pdfHeaderFooter,
    csvDownload,
    permitEmail,
    caseSensitive,
    visualsAfterFilters,
  ]);

  if (arc.globals.arcviz_settings.ENABLE_URL_NAMES) {
    user_settings.addSetting(arc.settings.dashboards.dashboardURLName());
  }

  const dashboard = function() { };

  document.getElementById('dashboard-title').addEventListener('input', () => {
    const dashboardTitle = $('#dashboard-title').text();
    if (window.top) {
      window.top.postMessage({ topic: 'viz.document.name.changed', value: { name: dashboardTitle } }, '*');
    }
  }, false);

  searchHandler.dashboard(dashboard);

  dashboard.numColumns = function() {
    return numColumns;
  };

  dashboard.visualsAfterFiltersEnabled = function() {
    return visualsAfterFilters.isChecked;
  };

  dashboard.enableAnimations = function() {
    return enableAnimations.isChecked;
  };

  dashboard.manualVisualRefresh = function() {
    if (arc.globals.gflags && arc.globals.gflags.is_snapshot) {
      return false;
    }

    return manualVisualRefresh.isChecked;
  };

  let manualRefreshes = [];
  dashboard.addManualRefresh = function(refreshConfig) {
    const {
      driver,
      force_redraw,
      filter_widget,
    } = refreshConfig;

    const done = manualRefreshes.some((m) => {
      if (m.driver === driver) {
        m.filter_widget = filter_widget;
        if (!m.force_redraw) {
          m.force_redraw = force_redraw;
        }

        return true;
      }

      return false;
    });

    if (done) {
      return;
    }

    manualRefreshes.push(refreshConfig);
    saveRefreshState();
    return dashboard;
  };

  function saveRefreshState() {
    if (manualRefreshes.length === 0) {
      $('.page-refresh-wrapper button')
          .addClass('btn-secondary')
          .removeClass('btn-primary')
          .prop('disabled', true);
    } else {
      $('.page-refresh-wrapper button')
          .removeClass('btn-secondary')
          .addClass('btn-primary')
          .prop('disabled', false);
    }
  }

  function runManualRefreshes() {
    const toRun = manualRefreshes;
    manualRefreshes = [];

    toRun.forEach((refreshConfig) => {
      const {
        driver,
        force_redraw,
        filter_widget,
      } = refreshConfig;

      driver.refresh(force_redraw, filter_widget, true);
    });

    saveRefreshState();
  }

  dashboard.dashboardWidth = function() {
    if (!dashboardWidthSetting) {
      return undefined;
    }

    let v = dashboardWidthSetting.value();
    if (Number.isNaN(v)) {
      return undefined;
    }

    v = Math.floor(Number(v));
    if (v <= 0) {
      return undefined;
    }

    return v;
  };

  dashboard.displayFilters = function() {
    return user_settings.checkboxSet('display_filters');
  };

  dashboard.displayContextMenu = function() {
    return user_settings.checkboxSet('display_context_menu');
  };

  dashboard.destroy = function() {
    if (singleVisualDashboard.isChecked) {
      clearSingleVisualCss(true);
    }

    gridster = gridster.remove_all_widgets();
    const picklistWidgets = getPicklistWidgets();
    picklistWidgets.forEach((report_id) => {
      const report = reports.getReport(report_id);
      if (!report) {
        return;
      }

      const wnames = widgetNames(report);
      const $widget_li = $(`#${wnames.app}`);
      $widget_li.remove();
      destroyReport(report);
      reports.removeReport(report_id, dashboard.sheet_id());
    });
  };

  function destroyReport(report) {
    if (report && report.viz()) {
      report.viz().destroy();
    }
  }

  dashboard.highlightMarks = function() {
    if (!gridster) {
      return;
    }

    // Collect all the widgets in existing dashboard, and redraw each one
    // Pass in params
    const jdd = gridster.serialize();
    $.each(jdd, (idx, element) => {
      const report_id = dashboard.gridsterElementToReportId(element.id);
      const report = reports.getReport(report_id);
      if (report) {
        const viz = report.viz();
        if (viz) {
          viz.highlightMarks();
        }
      }
    });
  };

  dashboard.pngDownloadPermitted = function() {
    return !pngDownload.hidden() && pngDownload.isChecked;
  };

  dashboard.csvDownloadPermitted = function() {
    return csvDownload.isChecked;
  };

  dashboard.emailPermitted = function() {
    return !permitEmail.hidden() && permitEmail.isChecked;
  };

  dashboard.resizing = function(_) {
    if (!arguments.length) return resizing;
    resizing = _;
    return dashboard;
  };

  dashboard.globalFilters = function(_) {
    if (!arguments.length) return globalFilters;
    globalFilters = _;
    return dashboard;
  };

  dashboard.dashboard_name = function(_) {
    if (!arguments.length) return dashboard_name;
    dashboard_name = _;
    return dashboard;
  };

  dashboard.dashboard_mode = function(_) {
    if (!arguments.length) return dashboard_mode;
    dashboard_mode = _;
    return dashboard;
  };

  dashboard.dashboard_data = function(_) {
    if (!arguments.length) return dashboard_data;
    dashboard_data = _;
    return dashboard;
  };

  dashboard.dashboard_id = function(_) {
    if (!arguments.length) return dashboard_id;
    dashboard_id = _;
    return dashboard;
  };

  dashboard.sheet_id = function(_) {
    if (!arguments.length) return sheet_id;
    sheet_id = _;
    return dashboard;
  };

  dashboard.originalVisuals = function(_) {
    if (!arguments.length) return originalVisuals;
    originalVisuals = _;
    return dashboard;
  };

  dashboard.reports_to_delete = function (_) {
    if (!arguments.length) return reports_to_delete;
    reports_to_delete = _;
    return dashboard;
  };

  dashboard.reports_to_update = function (_) {
    if (!arguments.length) return reports_to_update;
    reports_to_update = _;
    return dashboard;
  };

  // Used externally to add reports to reports_to_update
  // Used when unlinking with cloneInstead === true
  // Only add if there's report_id and dataset_id
  dashboard.addReportToUpdate = function (newReport) {
    if (newReport && newReport.report_id() && newReport.dataset_id()) {
      reports_to_update.push(newReport);
    }
  };

  dashboard.onSaveDashboardCb = function (_) {
    if (!arguments.length) return onSaveDashboardCb;
    onSaveDashboardCb = _;
    return dashboard;
  };

  /** Object with workspace_id and workspace_name properties */
  dashboard.workspaces = function(_) {
    if (!arguments.length) return workspaces;
    workspaces = _;
    return dashboard;
  };

  dashboard.displaydb = function(_) {
    if (!arguments.length) return displaydb;
    displaydb = _;
    return dashboard;
  };

  dashboard.dashboard_filter_controls = function(_) {
    if (!arguments.length) return dashboard_filter_controls;
    dashboard_filter_controls = _;
    return dashboard;
  };

  dashboard.currentDataSetSelection = function(_) {
    if (!arguments.length) return currentDataSetSelection;
    currentDataSetSelection = _;
    return dashboard;
  };

  dashboard.data_connections = function (_) {
    if (!arguments.length) return data_connections;
    data_connections = _;
    return dashboard;
  };

  dashboard.data_connections_hash = function (_) {
    if (!arguments.length) return data_connections_hash;
    data_connections_hash = _;
    return dashboard;
  };

  dashboard.dataset_hash = function (_) {
    if (!arguments.length) return dataset_hash;
    dataset_hash = _;
    return dashboard;
  };

  dashboard.active_connection_ids = function (_) {
    if (!arguments.length) return active_connection_ids;
    active_connection_ids = _;
    return dashboard;
  };

  dashboard.reports_all_data = function (_) {
    if (!arguments.length) return reports_all_data;
    reports_all_data = _;
    return dashboard;
  };

  dashboard.reports_hash = function (_) {
    if (!arguments.length) return reports_hash;
    reports_hash = _;
    return dashboard;
  };

  dashboard.widget_base_dim_px = function (_) {
    if (!arguments.length) return widget_base_dim_px;
    widget_base_dim_px = _;
    return dashboard;
  };

  dashboard.gridster = function (_) {
    if (!arguments.length) return gridster;
    gridster = _;
    return dashboard;
  };

  dashboard.controls_orientation = function() {
    if (filtersOnLeft.isChecked) {
      $('.filter_inline_labels input').prop('checked', '');
      $('.control-container').removeClass('inline-filters');
      return 'left';
    }

    return 'top';
  };

  dashboard.user_settings = function() {
    return user_settings;
  };

  dashboard.params_history = function() {
    return params_history;
  };

  dashboard.current_params = function() {
    return params_history.current_params();
  };

  dashboard.setPageLoadParams = function(params) {
    if (params && params.length) {
      params_history.add(params);
    }

    return dashboard;
  };

  dashboard.dashboard_title = function() {
    return dashboard_title;
  };

  // TODO: rename this n similar functions to match varaiable name like sheet_id
  dashboard.order = function (_) {
    if (!arguments.length) return order;
    order = _;
    return dashboard;
  };

  dashboard.sheet_handle_title = function (_) {
    if (!arguments.length) return currentSheet.sheet_handle_title;
    currentSheet.sheet_handle_title = _;
    return dashboard;
  };

  dashboard.deactivate = function (ignoreWarnings) {
    // TODO: If sheet has unsaved changes then ask user to save those first
    // before switching sheets or lose them
    // dashboard.destroy();
    let discardChanges = true;
    if (!ignoreWarnings && arc.report.pagenav.needs_save()) {
      discardChanges = confirm('You have unsaved changes in current Sheet. ' +
        'If you continue, you may lose these changes.');
    }

    if (!discardChanges) {
      return false;
    }

    dashboard.destroy();

    // $('.arc-app-container ul.control-list').empty();
    // $('#app-selected-list ul').empty();
    return dashboard_data;
  };

  dashboard.paintBorders = function() {
    if (bordersAroundVisuals.isChecked) {
      $('.arc-viz-container').addClass('gridster-border');
    } else {
      $('.gridster .arc-viz-container').removeClass('gridster-border');
    }

    return dashboard;
  };

  dashboard.refresh_sortable_controls = function() {
    if (!displaydb) {
      $('ul.control-list').sortable('refresh');
    }
  };

  dashboard.setup_sortable_controls = function() {
    if (!displaydb) {
      $('ul.control-list').sortable({
                            update() {
                              arc.report.pagenav.needs_save(true);
                            }
                          })
                          .disableSelection();
    }

    return dashboard;
  };

  dashboard.setup_droppable_controls = function() {
    if (!displaydb) {
      $('div.control-container').droppable({
        accept: '#dash-filters-tab .selection_element',
        classes: {
          'ui-droppable-active': 'shelf-highlight'
        },
        drop: (_, ui) => {
          const filter = arc.report.shelf.getDatum($(ui.draggable));

          dashboard_filter_controls.createControlFromColumn({
            dataset: filter.dataset(),
            coltype: filter.getComputedType(),
            colname: filter.dataset_colname(),
            isDatasetSegment: filter.isDatasetSegment(),
          });
        }
      });
    }

    return dashboard;
   };

  dashboard.updateAllDropdowns = function() {
    updateDownloadDropDown(csvDownload);
    updateReportCsv();
  };

  function updateDownloadDropDown(csvSetting) {
    let showDownload;
    if (!arc.globals.ms_dashboard.dashboard_id() ||
        !dashboard.sheet_id() || dashboard.sheet_id() < 1) {
      showDownload = false;
    } else {
      const csvAllowed = csvSetting.isChecked && dashboard.csvDownloadPermitted();
      showDownload = (csvAllowed || dashboard.pngDownloadPermitted());
    }

    if (showDownload) {
      $('.viz-params-more .image-dropdown-submenu, .report-menu-list .image-dropdown-submenu')
        .toggleClass('arc-invisible', !dashboard.pngDownloadPermitted());

        $('.viz-params-more .png-download, .report-menu-list .png-download')
        .toggleClass('arc-invisible', !dashboard.pngDownloadPermitted());
      } else {
        $('.viz-params-more .image-dropdown-submenu, .report-menu-list .image-dropdown-submenu')
          .addClass('arc-invisible');
    }
  }

  function allReportsIter(cb) {
    getGridsterWidgets().forEach((report_id) => {
      const r = reports.getReport(report_id);
      cb(r);
    });
  }

  function updateReportCsv() {
    allReportsIter((report) => {
      const driver = report.viz();
      if (driver) {
        driver.updateCsvSettings();
      }
    });
  }

  function handleCsvDownloads() {
    csvDownload.add_postpaint_fn(() => {
      $(`#${csvDownload.div_id()} input`)
        .offOn('click.dashboard', function handleCsvSetting() {
          arc.report.pagenav.needs_save(true);
          csvDownload.process_value();
          updateReportCsv();
        });
    });
  }

  function handlePngDownloads() {
    pngDownload.add_postpaint_fn(() => {
      $(`#${pngDownload.div_id()} input`)
        .offOn('click.dashboard', function handlePngSetting() {
          arc.report.pagenav.needs_save(true);
          pngDownload.process_value();
          updateDownloadDropDown(csvDownload, pngDownload);
          updateReportCsv();
        });
    });
  }

  function handlePermitEmail() {
    permitEmail.add_postpaint_fn(() => {
      $(`#${permitEmail.div_id()} input`)
        .offOn('click.updateEmail', function handleEmailSetting() {
          arc.report.pagenav.needs_save(true);
          permitEmail.process_value();
        });
    });
  }

  function manualRefreshVisibility() {
    const s = user_settings.find_by_id('manual_visual_refresh');
    if (!s) {
      return;
    }

    if (s.isChecked) {
      $('.page-refresh-wrapper').removeClass('arc-invisible');
      $('.page-refresh-wrapper')
          .offOn('click.refresh', () => {
            runManualRefreshes();
          });
    } else {
      $('.page-refresh-wrapper').addClass('arc-invisible');
    }
  }

  function handleManualVisualRefresh() {
    const s = user_settings.find_by_id('manual_visual_refresh');
    if (!s) {
      return;
    }

    s.add_postpaint_fn(() => {
      $(`#${s.div_id()} input`)
          .off('click.manual')
          .on('click.manual', () => {
            arc.report.pagenav.needs_save(true);
            s.process_value();
            manualRefreshVisibility();
          });
    });
  }

  function handleEnableAnimations() {
    const s = user_settings.find_by_id('dashboard_enable_animations');
    if (!s) {
      return;
    }

    s.add_postpaint_fn(() => {
      $(`#${s.div_id()} input`)
          .off('click.animate')
          .on('click.animate', () => {
            arc.report.pagenav.needs_save(true);
            s.process_value();
          });
    });
  }

  function changeCbOnSettingChange(settingName, changeCb) {
    const s = user_settings.find_by_id(settingName);
    if (!s) {
      return;
    }

    s.addChangeCb('click.dashboard_change', () => {
      arc.report.pagenav.needs_save(true);
      s.process_value();
      changeCb();
    });
  }

  function redrawAllOnSettingChange(settingName, changeCb) {
    return changeCbOnSettingChange(settingName, () => {
      redrawAll();
      if (changeCb) {
        changeCb();
      }
    });
  }

  function drawViewData(report) {
    const wid = `#${widgetId(report)}`;

    if (report.viz().enableViewData()) {
      $(`${wid} .widget-btns-wrapper .view-data-li`)
         .show();
    } else {
      $(`${wid} .widget-btns-wrapper .view-data-li`)
         .hide();
    }
  }

  function handleViewDataChanges() {
    return changeCbOnSettingChange('dashboard_display_view_data', () => {
      allReportsIter((report) => {
        drawViewData(report);
      });
      redrawAll({ autoRefresh: true });
      drawActiveVisualSettings();
    });
  }

  function handleVisualStyleOverrides() {
    return redrawAllOnSettingChange('override_visual_font_settings');
  }

  function handleDashboardWidth() {
    if (dashboardWidthSetting) {
      dashboardWidthSetting.changeCb('dashboard', () => {
        arc.report.pagenav.needs_save(true);
        setWidth();
        dashboard.resizeCallback(true);
      });
    }

    if (singleVisualDashboard) {
      singleVisualDashboard.changeCb('dashboard', () => {
        adjustForSingleVisualDashboard(true);
        arc.report.pagenav.needs_save(true);
      });
    }
  }

  function handleDisplayFilters() {
    return redrawAllOnSettingChange('display_filters');
  }

  function handleContextMenus() {
    return redrawAllOnSettingChange('display_context_menu');
  }

  function handleBorders() {
    bordersAroundVisuals.add_postpaint_fn(() => {
      $(`#${bordersAroundVisuals.div_id()} input`)
          .off('change.dashboard')
          .on('change.dashboard', () => {
            arc.report.pagenav.needs_save(true);
            bordersAroundVisuals.process_value();
            dashboard.paintBorders();
          });
    });
  }

  function handleBorderRadius() {
      if (!visualBorderRadius) {
        return;
      }

      visualBorderRadius.changeCb('dashboard', () => {
        arc.report.pagenav.needs_save(true);
        paintInlineCSS();
      });
  }

  function handleFilterOrientation() {
    // Move controls left or top
    filtersOnLeft.add_postpaint_fn(() => {
      $(`#${filtersOnLeft.div_id()} input`)
          .off('change.dashboard')
          .on('change.dashboard', () => {
            arc.report.pagenav.needs_save(true);
            filtersOnLeft.process_value();
            dashboard_filter_controls.handle_orientation();
          });
    });
  }

  function paintCSSClass() {
    const s = user_settings.find_by_id('container_css_class');
    const v = s.value();
    if (v) {
      $('#main').addClass(v);
    }
  }

  function paintInlineCSS() {
    arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
  }

  function handleCSSIds() {
    const cssStyles = user_settings.find_by_id('css_styles');
    cssStyles.changeCb('dashboard', () => {
      arc.report.pagenav.needs_save(true);

      arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
    });
  }

  function handleCSSChanges() {
    const containerCss = user_settings.find_by_id('container_css_class');
    containerCss.add_postpaint_fn(() => {
      $(`#${containerCss.div_id()} input`)
          .off('change.dashboard')
          .on('change.dashboard', () => {
            arc.report.pagenav.needs_save(true);
            containerCss.process_value();
            paintCSSClass();
          })
          .off('input.classUpdate')
          .on('input.classUpdate', () => arc.report.pagenav.needs_save(true));
    });

    const cssInline = user_settings.find_by_id('inline_css');
    cssInline.changeCb('dashboard', () => {
      cssInline.process_value();
      paintInlineCSS();
    });
  }

  function handleSearch() {
    arc.shortcut.remove('Ctrl+F');
    arc.shortcut.add('Ctrl+F', () => {
      $('.arc-viz-container.active .search-app').trigger('click');
    });
  }

  dashboard.activate = function() {
    if (arc.globals.is_edit) {
      dashboard.setup_sortable_controls();
      dashboard.setup_droppable_controls();

      $('.dash-main-panel').offOn('click.deactivatereport', (e) => {
        // during resizing, we sometimes get spurious events outside of .gs-w
        // ignore these deactivation events during resizing
        if (dashboard.resizing()) {
          return;
        }

        /*
         * Make sure event target is still in dom (some visual repainting happens on
         * click and makes the propogated event happen on a dom element that is not
         * in the dom anymore.
         */
        const $body = $(e.target).closest('body');
        if ($body.length === 0) {
          return;
        }

        const $viz = $(e.target).closest('.gs-w');
        if ($viz.length > 0) {
          return;
        }

        deactivateReport();
      });
    }

    arc.colorpalette.fetch();
    arc.dateranges.fetch();

    handleSearch();
    handleBorders();
    handleDisplayFilters();
    handleContextMenus();
    handlePngDownloads();
    handlePermitEmail();
    handleCsvDownloads();
    handleFilterOrientation();
    handleCSSChanges();
    handleCSSIds();

    arc.css.CUSTOMCSS_MANAGER.appSettings(user_settings);

    handleDashboardWidth();
    handleEnableAnimations();
    handleManualVisualRefresh();
    handleVisualStyleOverrides();
    handleViewDataChanges();
    handleDashboardColors();
    redrawAllOnSettingChange('filter_inline_labels');
    handleDashboardMargins();
    handleBorderRadius();

    setupTabMemory();

    dashboard.reports(arc.globals.reports)
             .setupEditTitle();

    if (sheet_id) {
      dashboard.load();
    } else {
      throw new Error('Failed to load dashboard');
    }

    updateDownloadDropDown(csvDownload);
    handleFontChanges();
    handleAutoRefresh();
    updateDatasetsShortcuts();

    if (arc.globals.is_edit && Number(autoRefresh.value()) > 0) {
      pauseAutoRefresh();
    }

    return dashboard;
  };

  function paintUserSettings() {
    // Don't draw dashboard settings unless you are in dashboard edit mode
    if (!arc.globals.is_edit) {
      return;
    }

    const $stylingSettings = $('#dashboard-styling-settings-sidebar');
    $stylingSettings.empty();

    const $cssSettings = $('#dashboard-css-settings-sidebar');
    $cssSettings.empty();

    const $defaultSettings = $('#dashboard-settings-sidebar');
    $defaultSettings.empty();

    user_settings.paint({
      default: $defaultSettings,
      Style: $stylingSettings,
      'Custom Style': $cssSettings,
    });
  }

  dashboard.setupParamsControl = function() {
    params_history.dashboard(this);
    params_history.setupControls(false);
    return dashboard;
  };

  dashboard.setSize = function() {
    last_dimensions = arc.globals.ms_dashboard.weight_base_dimensions();
    return dashboard;
  };

  dashboard.autoRefreshSeconds = function(_) {
    if (!arguments.length) return autoRefreshSeconds;

    if (arc.globals.auto_refresh) {
      // Global URL parameters override local values
      if (!Number.isNaN(arc.globals.auto_refresh)) {
        const ar = Number(arc.globals.auto_refresh);
        if (ar >= 1) {
          _ = ar;
        }
      }
    }

    if (autoRefreshSeconds) {
      arc.loader.enableLoadbars = false;
    }

    autoRefreshSeconds = _;
    return dashboard;
  };

  const playPauseControls = arc.misc.playpause($.now());
  playPauseControls.setPlayCb(() => {
                                redrawAll({ autoRefresh: true });
                                markLoading();
                                setupAutoRefresh();
                              });

  dashboard.isPlaying = function() {
    return playPauseControls.isPlaying();
  };

  function handleAutoRefresh() {
    setupAutoRefresh();

    autoRefresh.addChangeCb('dashboard', () => {
      arc.report.pagenav.needs_save(true);
      setupAutoRefresh();
    });
  }

  function getBrowserEvent() {
    let hidden;
    let visibilityChangeEvent;

    if (document.hidden !== undefined) {
      hidden = 'hidden';
      visibilityChangeEvent = 'visibilitychange';
    } else if (document.webkitHidden !== undefined) {
      hidden = 'webkitHidden';
      visibilityChangeEvent = 'webkitvisibilitychange';
    } else if (document.msHidden !== undefined) {
      hidden = 'msHidden';
      visibilityChangeEvent = 'msvisibilitychange';
    }

    return { hidden, visibilityChangeEvent };
  }

  function setupAutoRefresh() {
    clearAutoRefresh();

    const autoRefreshVal = autoRefresh.value();

    // Pause the autorefresh when the browser tab in question isn't active, then
    // restart if the user comes back
    function updateAutoRefreshByBrowserActivity() {
      const { hidden, visibilityChangeEvent } = getBrowserEvent();

      $(document).off(`${visibilityChangeEvent}.toggleRefresh`)
                 .on(`${visibilityChangeEvent}.toggleRefresh`, onVisibilityChange);

      function onVisibilityChange(evt) {
        if (evt.target[hidden]) {
          pauseAutoRefresh();
        } else {
          setupAutoRefresh();
        }
      }
    }

    if (Number(autoRefreshVal) > 0) {
      updateAutoRefreshByBrowserActivity();
      dashboard.autoRefreshSeconds(Number(autoRefreshVal));
      dashboard.autoRefresh();
      setupPause();
    } else if (containsKSQLVisual()) {
        playPauseControls.show();
      } else {
        playPauseControls.hide();
      }
  }

  function setupPause() {
    playPauseControls.showAsPlaying();
    const { visibilityChangeEvent } = getBrowserEvent();
    playPauseControls.setPauseCb(() => {
      $(document).off(`${visibilityChangeEvent}.toggleRefresh`);
      pauseAutoRefresh();
    });
  }

  function pauseAutoRefresh() {
    clearAutoRefresh();
  }

  function clearAutoRefresh() {
    if (arc.report.visualRefresh) {
      clearTimeout(arc.report.visualRefresh);
      arc.report.visualRefresh = undefined;
    }
  }

  function checkDoneLoading() {
    if (arc.counters.data.dataapi_start <= arc.counters.data.dataapi_end) {
      const $loading = $('.viz-params-loading');
      $loading.addClass('arc-invisible');
      return;
    }

    setTimeout(checkDoneLoading, 500);
  }

  function markLoading() {
    const $loading = $('.viz-params-loading');

    if (!$loading.hasClass('arc-invisible')) {
      return;
    }

    if (arc.counters.data.dataapi_start > arc.counters.data.dataapi_end) {
      $loading.removeClass('arc-invisible');

      checkDoneLoading();
    }
  }

  // Check every second for ongoing data requests to be complete, and then
  // run the cb function
  function afterDataRequests(cb) {
    if (arc.counters.data.dataapi_start > arc.counters.data.dataapi_end) {
      setTimeout(() => {
        afterDataRequests(cb);
      }, 1000);

      return;
    }

    cb();
  }

  dashboard.autoRefresh = function() {
    if (autoRefreshSeconds < 1) {
      autoRefreshSeconds = 1;
    }

    clearAutoRefresh();

    const v = setTimeout(() => {
      afterDataRequests(() => {
        // Ensure that this refresh cycle is still active, and it
        // is not the case that the arc.report.visualRefresh timeout
        // has been canceled.
        if (v === arc.report.visualRefresh) {
          // Ensure loading bars are disabled
          arc.loader.enableLoadbars = false;

          redrawAll({
            autoRefresh: true,
          });
          markLoading();
          dashboard.autoRefresh();
        }
      });
    }, autoRefreshSeconds * 1000);

    arc.report.visualRefresh = v;
  };

  // This is common code for updating dataconnection and dataset dropdowns
  // on both the filters sidebar and the visuals sidebar in dashboard edit
  function updateDropdowns(dcid, dsid) {
    dashboard.currentDataSetSelection(dsid);

    updateItem(
      [dcid, '.dataconnections-sidebar-sel, .dataconnections-sel'],
      () => updateItem([dsid, '.datasets-sidebar-sel, .datasets-sel'])
    );

    function updateItem([objectId, objectSelector], cb) {
      const $inputs = $(objectSelector);
      $inputs.val(objectId);
      $inputs.trigger('chosen:updated');
      $inputs.trigger('change', cb);
    }
  }

  function drawDatasetsShortcuts(datasetIdsUsedInDashboard, datasetsById) {
    $('.app-ds-list').empty();

    const adl = d3.select('.app-ds-list');

    // Redraw datasets shortcuts
    // If nothing in dashboard (new dashboard probably), then don't put
    // anything in the datasets shortcuts
    if (datasetIdsUsedInDashboard.size === 0) {
      return false;
    }

    adl.selectAll('nosel')
       .data([...datasetIdsUsedInDashboard])
       .enter()
         .append('li')
         .html((id) => `<a class="link"><span>${datasetsById[id].dataset_name()}</span></a>`);

    adl.on('click.adl', () => {
      // Update connection/dataset in dropdown to match clicked dataset
      const datasetId = d3.select($(d3.event.target).closest('li')[0]).datum();
      const dataconnectionId = datasetsById[datasetId].dataconnection_id();
      updateDropdowns(dataconnectionId, datasetId);
    });

    return true;
  }

  const datasetIdsUsedInDashboard = new Set();
  function updateDatasetsShortcuts(dataset_id) {
    // Temporary variables
    let report_id;
    let rd;
    let dsid;
    let jdd;
    let dcid;

    // If this is called before the ajax calls to retrieve all
    // dataconn/dataset/viz info (because of vizual being added to the db),
    // simply return. It'll get called again at the end of the ajax call
    if (Object.keys(dataset_hash).length === 0) {
      return;
    }

    // dataset_id if specified, then it is being added, so add a single
    // element to the shortcuts list. For removal, datasetIdsUsedInDashboard will
    // recalculated and shortcuts redrawn overall (instead of trying to figure
    // if the dataset of the removed visual still has other visuals in the db)
    if (dataset_id) {
      datasetIdsUsedInDashboard.add(dataset_id);
      drawDatasetsShortcuts(datasetIdsUsedInDashboard, dataset_hash);
      dcid = dataset_hash[dataset_id].dataconnection_id();
      if (dcid >= 0) {
        updateDropdowns(dcid, dataset_id);
      }

      return;
    }

    datasetIdsUsedInDashboard.clear();
    if (currentSheet) {
      jdd = currentSheet.visual_widgets.concat(currentSheet.control_widgets);
    } else {
      throw new Error('This scenario should never occur');
    }

    jdd.forEach((widget) => {
      report_id = widget.id ? dashboard.gridsterElementToReportId(widget.id) : widget;
      rd = reports_hash[report_id];

      // Filter visuals and viz created using 'New Visual' from the
      // dashboard edit side navbar won't have an associated entry in
      // reports_hash (former are filtered at the backend, latter are newly
      // created so not part of the reports_hash retrieved from backend)
      if (rd) {
        dsid = rd.dataset_id;
        if (!datasetIdsUsedInDashboard.has(dsid)) {
          datasetIdsUsedInDashboard.add(dsid);
        }
      }
    });

    if (drawDatasetsShortcuts(datasetIdsUsedInDashboard, dataset_hash) && dsid) {
      // If there are existing visuals in the dashboard, update selected values
      // in the dataconnection and dataset dropdown with information from the
      // last added visual in the dashboard
      dcid = dataset_hash[dsid].dataconnection_id();

      if (dcid >= 0) {
        updateDropdowns(dcid, dsid);
      }
    }
  }

  dashboard.scrollToBottom = function scrollToBottom() {
    const $main = $('#main');
    const $bottomAnchor = $main.find('#bottom-anchor');

    // scroll to bottom of the main div
    $main.scrollTo($bottomAnchor, 1000);
  };

  // Return true if user has access to report and
  // if it's either linked visual or unlinked visual that exists in dashboard
  function shouldDrawReportId(reportId, datasetId) {
    if (!reports_hash[reportId]) {
      return false;
    }

    // Optional filter by datasetId
    const inDataset = !datasetId || (datasetId === reports_hash[reportId].dataset_id);
    const hasPermission = arc.misc.hasPerm('ds_appedit', reports_hash[reportId].perm);

    const isLinked = reports_hash[reportId].shared_visual_dashboards;
    const reportHash = reports_hash[reportId];
    let inDashboard;
    if (!isLinked && hasPermission && reportHash.dashboard_id) {
      // If unlinked visual, make sure it still exists in dashboard
      if (reports_hash[reportHash.dashboard_id]?.visuals) {
        inDashboard = reports_hash[reportHash.dashboard_id].visuals.some(
          (vizId) => vizId === reportId
        );
      }
    }

    // Don't show visual if is unlinked visual that's not in any dashboard
    const isLinkedOrInDashboard = isLinked || inDashboard;
    return hasPermission && isLinkedOrInDashboard && inDataset;
  }

  function getLinkedUnlinkedIcon(reportData) {
    let template = unlinkedVisualTemplate;
    let containingDashboards;
    if (reportData.shared_visual_dashboards) {
      template = linkedVisualTemplate;

      containingDashboards = {
        TITLE_AND_ID: JSON.parse(reportData.shared_visual_dashboards).map(parseTitleId).join(', '),
      };
    } else {
      containingDashboards = {
        TITLE_AND_ID: parseTitleId(reportData.dashboard_id),
      };
    }

    return arc.misc.safe_substitute(template, containingDashboards);

    function parseTitleId(reportId) {
      const report = reports_hash[Number(reportId)];

      if (report) {
        return `${report.title ? report.title : 'Untitled'} (${reportId})`;
      }

      return reportId;
    }
  }

  dashboard.paintVisualSidebar = function() {
    // Temporary variables used below
    let dsid;
    let dcid;

    // Paint most recent visuals
    const recently_viewed_visuals =
        arc.frontend.vizCookie.getLatestVisuals('visual')
            .filter((d) => shouldDrawReportId(d))
            .reverse()
            .slice(0, 5);

    if (recently_viewed_visuals.length > 0) {
      $('#recent-app-list').removeClass('arc-invisible');
      d3.selectAll('.recent-app-list li')
        .remove();
      const ral = d3.select('.recent-app-list');
      ral.selectAll('nosel')
         .data(recently_viewed_visuals)
         .enter()
           .append('li')
           .classed('app_selection_element', true)
           .html((d) => `${'<div class="app-img-wrapper"><img src="' +
                    `${reports_hash[d].img}"></div>` +
                    `<div class="app-title">${arc.misc.escapeHTML(reports_hash[d].title)}</div>`}${getLinkedUnlinkedIcon(reports_hash[d])}`);

      ral.on('click.ral', () => {
        // TODO: find a less hacky way to do this
        const id = d3.select($(d3.event.target).closest('li')[0]).datum();

        // Open new tab if clicked on dashboard icon
        if (d3.event.target.className.includes('dashboard-link-btn')) {
          arc.misc.openInNewWindow(`${arc.frontend.globals.urls.viewDashboardUrl}/${reports_hash[id].dashboard_id}`);
        } else {
          dashboard.addAppFromSidebar(reports_hash[id]);
        }
      });
    }

    arc.dataconnection.sortDataConnections(data_connections);

    // Paint Connections and Datasets drop downs..
    const $dataconnectionsSel = $('.dataconnections-sidebar-sel');
    d3.select('.dataconnections-sidebar-sel')
      .html('')
      .selectAll('nosel')
      .data(data_connections)
      .enter()
        .append('option')
        .attr('value', (d) => d.dataconnection_id())
        .text((d) => d.dataconnection_name());

    $dataconnectionsSel.trigger('chosen:updated');

    $dataconnectionsSel.off('change.das');
    $dataconnectionsSel.on('change.das', function() {
      dashboard.paintVisualSidebarDatasetsOfConn($(this).val());
    });

    // .. and set the values of the dataconn/dataset dropdowns to be:
    // a) For a new dashboard sheet (with no dataset info), use selections from previous sheet
    // b) For a new dashboard (with no dataset info), that of the most recently viewed visual
    // c) For a new dashboard (with dataset info), same as passed in info from URL
    // d) For an app with existing visuals (not filters), that of the last
    // visual in the dashboard
    // If there are existing visuals in the dashboord, this will get
    // overwritten by the following call to updateDatasetsShortcuts
    dsid = dashboard.getDatasetId() || dashboard.currentDataSetSelection();
    if (!dsid && recently_viewed_visuals.length > 0) {
      dsid = reports_hash[recently_viewed_visuals[0]].dataset_id;
    }

    if (dsid >= 0) {
      dcid = dataset_hash[dsid].dataconnection_id();
      if (dcid >= 0) {
        updateDatasetsShortcuts();
        updateDropdowns(dcid, dsid);
      }
    } else {
      // This is to paint the datasets the first time a new app is loaded,
      // and there are no cookies
      updateDatasetsShortcuts();
      dashboard.paintVisualSidebarDatasetsOfConn($dataconnectionsSel.val());
      $('.dataconnections-sel').trigger('chosen:updated');
    }

    // Add popovers
    const popoverText = $('#chart-content-popover').html();
    $('#dash-viz-tab').popover({
      selector: '.app_selection_element',
      container: 'body',  // makes popover be on top
      html: true,
      trigger: 'hover',
      placement: 'left',
      delay: { show: '500' },
      title: '',
      sanitize: false,
      content: function showDashPopover() {
        const idOrData = d3.select(this).datum();

        // In "Recently Viewed" section, data consists of only id
        // The visuals by dataset include full visualInfo
        const data = $.isNumeric(idOrData) ? reports_hash[idOrData] : idOrData;

        const dataset = dataset_hash[data.dataset_id];
        const tags = dataset ? dataset.tags() : [];
        data.HTML_DATASET_TAGS = tags.join('');
        if (tags.length === 0) {
          data.TAGS_VISIBILITY = 'arc-invisible';
        }

        return arc.misc.safe_substitute(popoverText, arc.report.mapDashboardOrVisualInfo(data));
      },
      boundary: 'window',
    });

    return dashboard;
  };

  dashboard.paintVisualSidebarDatasetsOfConn = function(dataconnection_id) {
    const dataconn = data_connections_hash[dataconnection_id];

    // Dashboards can contain reports from multiple datasets and it's not
    // clear that we should require 'ds_appedit' on the datasets presented
    // here.
    // However, dashboards are saved as reports based on a dataset and
    // that will fail if we don't have the permission on that dataset.
    // So for now, we restrict to datasets that we have edit permission on.
    const $sel = $('.datasets-sidebar-sel');
    $sel.offOn('change.dasd', function updateSidebar() {
          paintVisualSidebarVisualsOfDataset($(this).val());
        });

    const datasetsofconn = dataconn.datasets().filter((ds) => arc.misc.hasPerm('ds_appedit', ds.dataset_perms()));

    arc.dataconnection.sortDataSets(datasetsofconn);

    d3.selectAll('.datasets-sidebar-sel option')
      .remove();

    d3.select('.datasets-sidebar-sel')
      .selectAll('nosel')
      .data(datasetsofconn)
      .enter()
        .append('option')
        .attr('value', (d) => d.dataset_id())
        .text((d) => d.dataset_name());

    $sel.trigger('chosen:updated');
    $sel.trigger('change');

    return dashboard;
  };

  function loadDatasetVisuals(dataset_id, success_cb, error_cb) {
    if (Number.isNaN(dataset_id)) return;

    const reports_asset = {
      asset: `${arc.frontend.globals.urls.reportUrl}?all=true&exclude_dashboards=true&dataset_id=${dataset_id}`,
      postprocess: d3.json,
    };

    arc.staticfetcher.fetch(
      [reports_asset],
      (error, reports_data) => {
        if (error) {
          arc.report.alertMessage('Could not retrieve data reports for dataset ',
            error.response ? error.response : error,
            { alert_report_id: dashboard_id });
          error_cb();
          return;
        }

        const no_dashboard_reports = reports_data.filter((d) => (d.report_type !== 'dashboard'));
        success_cb(no_dashboard_reports);
      }
    );
  }

  function paintVisualSidebarVisualsOfDataset(dataset_id) {
    dataset_id = parseInt(dataset_id, 10);
    dashboard.currentDataSetSelection(dataset_id);

    // Only allow creating dashboards from datasets with RW access
    if (!Number.isNaN(dataset_id) &&
        !arc.misc.hasPerm('ds_appedit', dataset_hash[dataset_id].dataset_perms())) {
      return false;
    }

    if (!$('.dash-settings-panel').is(':visible')) {
      return;
    }

    const dataset_reports = reports_by_dataset[dataset_id];
    if (dataset_reports === undefined) {
      const success_cb = (downloaded_reports) => {
        downloaded_reports.forEach((rd) => {
          reports_hash[rd.id] = rd;
        });
        const ds_reports = downloaded_reports;

        // sort by updated time. The updated property is a string:
        ds_reports.sort((a, b) => {
          if (a.updated < b.updated) {
            return 1;
          }

          if (a.updated > b.updated) {
            return -1;
          }

          return 0;
        });

        reports_by_dataset[dataset_id] = ds_reports;
        drawVisualsList(ds_reports);
        $('.dash-settings-panel #visuals-loading').addClass('arc-invisible');
      };

      const error_cb = () => {
        // hide loading indicator
        $('.dash-settings-panel #visuals-loading').addClass('arc-invisible');
      };

      // show loading indicator
      $('.dash-settings-panel #visuals-loading').removeClass('arc-invisible');

      loadDatasetVisuals(dataset_id, success_cb, error_cb);
    } else {
      drawVisualsList(dataset_reports);
    }
  }

  function drawVisualsList(dataset_reports) {
    d3.selectAll('.dataset-app-list li')
      .remove();

    d3.select('.dataset-app-list')
      .selectAll('nosel')
      .data(dataset_reports)
      .enter()
        .append('li')
        .classed('app_selection_element', true);

    d3.selectAll('.dataset-app-list li')
      .on('click', (d) => {
        // Open new tab if clicked on dashboard icon
        if (d3.event.target.className.includes('dashboard-link-btn')) {
          arc.misc.openInNewWindow(`${arc.frontend.globals.urls.viewDashboardUrl}/${d.dashboard_id}`);
        } else {
          dashboard.addAppFromSidebar(d);
        }
       })
      .html((d) => {
        const title = arc.misc.escapeHTML(d.title);
        const { img } = d;
        return `${`<div class="app-img-wrapper"><img src="${img}"></div>` +
               `<div class="app-title">${title}</div>`}${getLinkedUnlinkedIcon(d)}`;
      });

    return dashboard;
  }

  function timeControlFilter(cb) {
    const timeControl = dashboard_filter_controls.timeControl();
    if (timeControl) {
      const filter = timeControl.filter();
      if (filter) {
        cb(filter);
      }
    }
  }

  function eachFilter(cb) {
    const control_widgets = getPicklistWidgets();
    control_widgets.forEach((d) => {
      const report = reports.getReport(d);
      if (report && report.viz() && report.viz().viz()) {
        cb(report.viz().viz());
      }
    });
  }

  dashboard.widgetInfo = function() {
    if (!gridster) {
      return;
    }

    return gridster.serialize();
  };

  dashboard.getVisuals = function () {
    if (!gridster) {
      return [];
    }

    const visuals = [];
    gridster.serialize().forEach(({ id }) => {
      const reportId = dashboard.gridsterElementToReportId(id);
      const report = reports.getReport(reportId);
      if (report) {
        visuals.push(report);
      }
    });

    return visuals;
  };

  dashboard.eachViz = function(cb) {
    if (!gridster) {
      return;
    }

    const dashboard_widgets = gridster.serialize();
    const jdd = JSON.parse(JSON.stringify(dashboard_widgets));

    $.each(jdd, (idx, element) => {
      const report_id = dashboard.gridsterElementToReportId(element.id);
      const report = reports.getReport(report_id);
      if (report && report.viz()) {
        cb(report.viz());
      }
    });
  };

  function containsKSQLVisual() {
    let ksqlCount = 0;

    dashboard.eachViz((viz) => {
      if (viz.isKSQL()) {
        ksqlCount += 1;
      }
    });

    return ksqlCount > 0;
  }

  const redrawDelayedTimeout = 2000;
  let redrawDelayedIncarnation = 1;

  // When settings are being modified, we don't want to redraw all visuals
  // on each keystroke, but we do want to redraw visuals when all the typing
  // is done. Do this by using the redrawDelayed function that will redraw
  // after 2 seconds as long as no additional changes are made
  function redrawDelayed() {
    redrawDelayedIncarnation += 1;
    const curr = redrawDelayedIncarnation;

    setTimeout(() => {
      if (curr === redrawDelayedIncarnation) {
        redrawAll();
      }
    }, redrawDelayedTimeout);
  }

  function redrawAll(config = {}) {
    arc.css.CUSTOMCSS_MANAGER.applyCustomCss();

    eachFilter((viz) => {
      if (viz.viz_processRefresh) {
        viz.viz_processRefresh();
      }
    });

    eachFilter((viz) => {
      viz.viz_lib().refresh(/* force_redraw */ true);
    });

    timeControlFilter((viz) => {
      viz.viz_lib().refresh();
    });

    dashboard.eachViz((viz) => {
      if (config.autoRefresh && viz.isKSQL()) {
        return;
      }

      if (!config.noReloadData) {
        viz.forceReloadData();
      }

      viz.refresh(/* force_redraw */ true);
    });
  }

  dashboard.resizeCallback = function (force) {
    // Don't redraw if the dashboard is fixed width or the visual modal is open
    if (!force && (dashboard.dashboardWidth() || arc.misc.isInSuperModal())) return;

    setWidth();

    timeControlFilter((filter) => {
      const driver = filter.viz_lib();
      if (driver) {
        filter.viz_drawVisual(driver.chart_div());
      }
    });

    const gridsterWidth = arc.globals.ms_dashboard.weight_base_dimensions();
    if (!force) {
      if (last_dimensions === gridsterWidth) {
        return dashboard;
      }
    }

    last_dimensions = gridsterWidth;
    return dashboard.resetupGridster();
  };

  // Re set up gridster based on gridster width
  dashboard.resetupGridster = function(_jdd) {
    // Get existing information of report ids and widget sizes
    const jdd = _jdd || gridster.serialize();

    // Re set it up such that new base widget dimensions are calculated
    // TODO: This seems to stuff new css into the head of the document without
    // deleting previous css, making the document larger and larger

    const curr_params = params_history.current_params();

    // This is start of async resizing
    dashboard.resizing(true);

    // Each don't proceed until all widgets removed
    let lengthGridster = gridster.$widgets.length;
    if (lengthGridster === 0) {
      widget_base_dim_px = arc.globals.ms_dashboard.weight_base_dimensions();
      gridster.resize_widget_dimensions({
        widget_base_dimensions: [widget_base_dim_px, widget_base_dim_px]
      });
    }

    gridster = gridster.remove_all_widgets(() => {
      lengthGridster -= 1;
      if (lengthGridster === 0) {
        widget_base_dim_px = arc.globals.ms_dashboard.weight_base_dimensions();
        gridster.resize_widget_dimensions({
          widget_base_dimensions: [widget_base_dim_px, widget_base_dim_px]
        });
        $.each(jdd, (_, element) => {
          const report_id = dashboard.gridsterElementToReportId(element.id);

          const config = {
            report_id,
            force_redraw: true,
            force_reloadreport: false,
            widget_sizex: element.size_x,
            widget_sizey: element.size_y,
            widget_col: element.col,
            widget_row: element.row
          };

          if (curr_params) {
            config.params_data = curr_params[0];
            config.click_report_id = curr_params[1];
          }

          dashboard.drawInWidget(config);
        });

        // Resizing is done
        dashboard.resizing(false);
      }
    });

    return dashboard;
  };

  dashboard.widgetNames = widgetNames;

  function widgetNames(report) {
    const wname = widgetId(report);

    return {
      id: report.report_id(),
      name: wname,
      app: `${wname}-a`,
      title: `${wname}-title`,
      subtitle: `${wname}-subtitle`,
      filter: `${wname}-filter`,
      refresh: `chart-refresh-indicator-${report.unique_report_id()}`
    };
  }

  function hideFilterSwitchButton(report) {
    const wnames = widgetNames(report);
    d3.selectAll(`#${wnames.app} .change-filter-type`).style('display', 'none');
  }

  function getRangeType(report) {
    const colType = getColType(report);
    if (colType === 'TIMESTAMP') {
      return 'daterange';
    }

    return arc.report.coltypeIsNumber(colType) ? 'range' : null;
  }

  function getOriginalFilterType(report) {
    return getRangeType(report) ? getRangeType(report) : 'picklist';
  }

  function getColType(report) {
    const viz = report.viz();
    if (!viz) {
      return;
    }

    const colName = viz.getBaseField();
    if (!colName) {
      return;
    }

    const dataset = viz.dataset();
    if (!dataset) {
      return;
    }

    const col = dataset.get_column(colName);
    if (!col) {
      return;
    }

    return col.coltype();
  }

  function getNewFilterType(report) {
    const newFilterTypeMap = {
      daterange: 'picklist',
      range: 'picklist',
      picklist: getOriginalFilterType(report),
    };
    return newFilterTypeMap[report.report_type()];
  }

  function setUpTemplate(wnames, report) {
    const refreshHtml = arc.misc.safe_substitute(
      $('#widget-refresh-template').html(),
      {
        ATTR_ID: wnames.refresh,
      }
    );

    const filterHTML = '<li id="%APP%" class="selected-control dropdown arc-viz-container">' +
      '%HTML_CONFIGURE_APP% %HTML_REFRESH% </li>';

    const substitutedFilterHTML = arc.misc.safe_substitute(
      filterHTML,
      {
        APP: wnames.app,
        HTML_CONFIGURE_APP: !displaydb ? $('#widget-configure-app').html() : '',
        HTML_REFRESH: refreshHtml,
      },
    );

    if (!report.isFromGrid && report.viz() && report.viz().viz() &&
      report.viz().viz().filterSettings() &&
      report.viz().viz().filterSettings().nextFilterId) {
      const { nextFilterId } = report.viz().viz().filterSettings();
      $(substitutedFilterHTML).insertBefore(`#${nextFilterId}`);
    } else {
      $('ul.control-list').append(substitutedFilterHTML);
    }

    $(`#${wnames.app}`)
      .append(arc.misc.safe_substitute(
        $('#viz-error-template').html(),
        {}
      ));
  }

  function eraseWidget(report, context) {
    const $widget_li = $(context).closest('li');
    $widget_li.find('a').tooltip('dispose');
    removeFilter(report, $widget_li);
  }

  function setSwitchTooltip(report) {
    const wnames = widgetNames(report);
    const $widget_li = $(`#${wnames.app}`).find('a.change-filter-type');
    $widget_li.attr('data-tooltip-contents', `Change Filter to ${getNewFilterType(report)}`);
  }

  function getNewFilter(report, context) {
    return dashboard.dashboard_filter_controls()
      .createControl({
        dataset_id: report.dataset_id(),
        visual_type: getNewFilterType(report),
        single_select: false,
        dataset: report.viz().dataset(),
        colname: report.viz().getBaseField(),
        filterSettings: {
          nextFilterId: $(context).parents('li').next('li').attr('id')
        },
      });
  }

  function addFilterElement(report, viz) {
    const wnames = widgetNames(report);
    setUpTemplate(wnames, report);

    if (!displaydb) {
      dashboard.refresh_sortable_controls();
    }

    d3.selectAll(`#${wnames.app} .widget-btns-wrapper .close-app`)
      .on('click.close-app', function () {
        eraseWidget(report, this);
      });

    d3.selectAll(`#${wnames.app} .configure-app`)
      .on('click.configure-app', () => {
        arc.globals.reports.setReport(report, -1);
        report.viz().viz().viz_configure();
      });

    d3.selectAll(`#${wnames.app} .widget-btns-wrapper .change-filter-type`)
      .on('click.change-filter-type', function () {
        viz = getNewFilter(report, this);
        eraseWidget(report, this);
      });

    return dashboard;
  }

  // Move a visual from the filters area to the grid area. This is triggered
  // when the move-app icon is clicked.
  function moveReportToGrid(report) {
    const wnames = widgetNames(report);
    const $widget_li = $(`#${wnames.app}`);
    const picklist = report.viz().viz();

    $widget_li.remove();
    if (dashboard._get_control_widgets().length === 0) {
      $('.control-instructions').show();
    }

    picklist.viz_lib().subtitle(picklist.picklist_title.value());

    picklist.resetSearch(() => {
      dashboard.refresh_sortable_controls();
      arc.report.pagenav.needs_save(true);

      picklist.expanded(true);
      report.viz().needs_save(true);
      dashboard.drawInWidget({
        report_id: report.report_id(),
        force_redraw: true,
        force_reloadreport: false,
        widget_sizex: filterXColumns,
        widget_sizey: filterYColumns,
      });

      configurePicklistMoveAndIcons(report);
    });
  }

  // Move a visual from the grid area to the filters area. This is triggered
  // when the move-app icon is clicked.
  function moveReportFromGrid(report) {
    const wnames = widgetNames(report);
    const $widget_li = $(`#${wnames.name}`);
    const picklist = report.viz().viz();

    gridster.remove_widget($widget_li, () => {
      dashboard.activateReport();
    });

    picklist.picklist_title.value(picklist.viz_lib().subtitle());

    arc.report.pagenav.needs_save(true);

    picklist.expanded(false);
    report.viz().needs_save(true);

    $('.control-instructions').hide();
    addFilterElement(report);
    configurePicklistMoveAndIcons(report);

    report.viz().refresh(true);
  }

  function removeOrRemoveAtSave(report) {
    // If the visual was only in dashboard, delete immediately if newly added
    // If it was in original dashboard, wait until save before deleting
    if (originalVisuals.has(report.report_id())) {
      reports_to_delete.push(report);
    } else {
      report.delete(undefined, undefined, dashboard_id);
    }
  }

  // Remove a visual from the filters area. Triggered when the close-app
  // icon is clicked.
  function removeFilter(report, $widget_li) {
    hideTooltip();

    if (arc.report.isAppScopeFilter(report && report.viz())) {
      const confirmDelete = arc.misc.simpleUserConfirmation({
        confirmTitle: 'Confirmation',
        confirmQus: 'Deleting this app scope filter will affect all sheets and ' +
                    'dashboards where this filter is currently visible.',
        confirmText: 'Delete Filter',
      });

      confirmDelete.askUser(() => {
        removeFilterHelper();
      });

      return;
    }

    removeFilterHelper();

    function removeFilterHelper() {
      removeOrRemoveAtSave(report);
      if (report && report.viz() && report.viz().viz()) {
        report.viz().viz().removeAllParams();
        report.viz().destroy();
      }

      $widget_li.remove();
      dashboard.refresh_sortable_controls();
      removeGridCommon(report, $widget_li);
      dashboard_filter_controls.filterContainerVisibility();
    }
  }

  function removeGridCommon(report) {
    reports.removeReport(report.report_id(), dashboard.sheet_id());
    updateDatasetsShortcuts();

    arc.report.pagenav.needs_save(true);

    // Adjust initial visual if showSingleVisual is checked
    // And there's only 1 visual on dashboard
    adjustForSingleVisualDashboard();

    // On error, report.viz() or report.viz().viz() may not be set yet.
    if (report.viz() && report.viz().viz()) {
      const viz_type = report.viz().viz_type();
      const visual = report.viz().viz();
      let del_param_name = '';
      if (arc.misc.isFilterVisual(viz_type)) {
        del_param_name = visual.viz_getOutputParameter();
      }

      if (!del_param_name) {
        return;
      }

      if (!del_param_name) {
        return;
      }

      const paramConfig = {
        outputParameter: del_param_name,
        isIgnore: true,
      };

      const paramObject = new arc.params.Param(report.viz(), paramConfig);
      report.viz().dashboard_click([paramObject]);
    }
  }

  function isReportPresentInOtherSheets(report_id) {
    const currentSheetId = dashboard.sheet_id();
    const isPresent = dashboard_data.dashboard_sheets.some(function doesSheetHaveReport(sheet) {
      return (
        sheet.sheet_id !== currentSheetId &&
        sheet.visual_widgets.some(
            (visualWidget) => dashboard.gridsterElementToReportId(visualWidget.id) === report_id
        )
      );
    });

    return isPresent;
  }

  function removeVisualDB(report) {
    // Remove dashboard from linked visual upon save
    // add to delete list if unlinked visual
    let dashboardsWithSharedViz = report.shared_visual_dashboards();
    if (dashboardsWithSharedViz) {
      if (!isReportPresentInOtherSheets(report.report_id())) {
        dashboardsWithSharedViz = JSON.parse(dashboardsWithSharedViz)
          .filter((dasId) => dasId !== dashboard.dashboard_id());
        report.shared_visual_dashboards(JSON.stringify(dashboardsWithSharedViz));
        reports_to_update.push(report);
      }
    } else {
      removeOrRemoveAtSave(report);
    }
  }

  function removeVisualFromSheet(report) {
    dashboard_data.dashboard_sheets.some((ds) => {
      if (ds.sheet_id === currentSheet.sheet_id) {
        currentSheet = ds;
        return true;
      }

      return false;
    });

    if (currentSheet) {
      let reportIdx = -1;
      currentSheet.visual_widgets.some((visualWidget, i) => {
        if (dashboard.gridsterElementToReportId(visualWidget.id) === Number(report.report_id())) {
          reportIdx = i;
          return true;
        }

        return false;
      });

      if (reportIdx >= 0) {
        currentSheet.visual_widgets.splice(reportIdx, 1);
      } else {
        reportIdx = -1;
        currentSheet.control_widgets.some((cw, i) => {
          if (cw.id === Number(report.report_id())) {
            reportIdx = i;
            return true;
          }

          return false;
        });
        if (reportIdx >= 0) {
          currentSheet.control_widgets.splice(reportIdx, 1);
        }
      }
    }
  }

  // Remove a visual from the grid area. Triggered when the close-app
  // icon is clicked.
  dashboard.removeFromGrid = function(report, $widget_li) {
    $widget_li.find('a').tooltip('dispose');
    gridster.remove_widget($widget_li, () => {
      if (dashboard.getDatasetId() === undefined) {
        $('.create-newparam').prop('disabled', true);
      }

      removeVisualDB(report);
      removeGridCommon(report, $widget_li);
      arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
      dashboard.activateReport();
    });
    removeVisualFromSheet(report);
  };

  // Only 1 report and 'single_visual_dashboard' is true, make first widget full width
  function shouldShowSingleVisual() {
    showSingleVisual = reports.numberReports(sheet_id) === 1 && singleVisualDashboard.isChecked;
    return showSingleVisual;
  }

  function clearSingleVisualCss(fullClear) {
    // Don't need to remove class/fullClear
    //  when running this before adding single visual dashboard css
    if (fullClear) {
      $buildContainer.removeClass('single-visual-dashboard');
    }

    $gridsterUl.css({
      width: '',
      height: '',
    });

    setWidth();
  }

  function adjustForSingleVisualDashboard(forceClear) {
    shouldShowSingleVisual();
    if (showSingleVisual) {
      $buildContainer.addClass('single-visual-dashboard');
      clearSingleVisualCss();
      const containerDimensions = $appContainer[0].getBoundingClientRect();
      const liDimensions = $gridsterUl[0].getBoundingClientRect();
      const tabHeight = $buildContainer.hasClass('single-sheet') ?
                        0 : $dashSheetContainer.height();
      const appHeight = containerDimensions.bottom - liDimensions.top - tabHeight;
      $gridsterUl.css({
        width: '100%',
        height: `${appHeight}px`,
      });

      // If a visual doesn't start in data-col="1", dashboard looks bad
      $gridsterUl.find('li').attr('data-col', 1);
    } else if (singleVisualDashboard.isChecked || forceClear) {
      clearSingleVisualCss(true);
    }
  }

  function highlightVisual($wid) {
    arc.misc.highlightDiv({
      $div: $wid,
      container: '#main',
    });
  }

  dashboard.addGridsterElement = function(report, widget_sizex, widget_sizey,
      widget_col, widget_row, isNew) {
    // Add widget with default sizes of 3 by 2, row/col not specified
    const wnames = widgetNames(report);
    const refreshHtml = arc.misc.safe_substitute(
        $('#widget-refresh-template').html(),
        {
          ATTR_ID: wnames.refresh,
        }
    );

    const widgetHtml = arc.misc.safe_substitute($('#widget-template').html(),
        {
          LIST_STYLE: (displaydb ? 'cursor:default;' : ''),
          VIZ_ID: report.report_id(),
          VIZ_ATTR_ID: wnames.name,
          HTML_WIDGET_BUTTONS: $('#dashboard-app-buttons').html(),
          HTML_REFRESH_BUTTONS: refreshHtml,
        });

    // ARC-5201: Adjust widget_sizex so is below numColumns
    //           Subtract extra 1 because column count starts at 1
    //           Skip if new visual without widget_col defined
    const maxWidth = numColumns - (widget_col - 1);
    if ($.isNumeric(maxWidth)) {
      widget_sizex = Math.min(widget_sizex, maxWidth);
    }

    // Adjust before so the .single-visual-dashboard style is set when visual calculates
    // its height
    adjustForSingleVisualDashboard();

    // Tell the report its x,y position so we can prioritize queries accordingly
    report.dashboardXY([widget_row, widget_col]);

    gridster.add_widget(widgetHtml,
        widget_sizex, widget_sizey, widget_col, widget_row);

    // And adjust after for calculations
    adjustForSingleVisualDashboard();

    activateReportOnClick(report);

    configurePicklistMoveAndIcons(report);
    $('.create-newparam').prop('disabled', false);

    dashboard.paintBorders();

    // Create placeholder for report title and subtitle
    const sel = d3.selectAll(`#${wnames.name}`)
                .selectAll('.arc-viz-container');

    arc.visual.createTitleSubtitleArea({
      d3Container: sel,
      titleId: wnames.title,
      subtitleId: wnames.subtitle,
      filterId: wnames.filter,
      chartId: wnames.app,
    });

    const wid = `#${wnames.name}`;
    const $wid = $(wid);
    report.updateSharedIconInDashboard();

    if (highlightedVisual && highlightedVisual === wnames.id) {
      highlightVisual($wid);
    }

    function drawInModal(dataset_id) {
      $wid.find('a').tooltip('dispose');
      if (report.report_type() === 'picklist') {
        if (displaydb) {
          return;
        }

        arc.globals.reports.setReport(report, -1);
        report.viz().viz().viz_configure();
      } else {
        const app_id = report.report_id();
        if (app_id) {
          arc.globals.use_derived_data = true;
          arc.globals.singleVisualBuilder
             .setupVisualInModal({
               app_id: Number(app_id),
               dataset_id: dataset_id || report.dataset_id(),
               dashboard
             });
        }
      }

      $('.super-modal .view-data')
          .offOn('click.view', () => {
            if (report && report.viz()) {
              report.viz().showData();
            }
          });
    }

    const builder = arc.globals.singleVisualBuilder;
    builder.setupVisualDropdownButtons($(`${wid}`), report);

    d3.selectAll(`${wid} .widget-btns-wrapper .save-csv`)
      .on('click.save-csv', function clickOnSaveCSV() {
        hideTooltip();
        $wid.find('a').tooltip('dispose');
        report.viz().exportCSV({});
      });

    d3.selectAll(`${wid} .widget-btns-wrapper .save-xlsx`)
      .on('click.save-excel', () => {
        hideTooltip();
        $wid.find('a').tooltip('dispose');
        report.viz().exportXlsx();
      });

    d3.selectAll(`${wid} .widget-btns-wrapper .viz-show-settings`)
      .on('click.viz-show-settings', function() {
        const parent = this.closest('.arc-viz-container');
        if (d3.select(parent).classed('arc-viz-type-picklist')) {
          hideTooltip();
          drawInModal();
        } else {
          dashboard.activateReport(report);
          showVisualShelves(report);
        }
      });

    if (report.viz() && arc.misc.isFilterVisual(report.viz().viz_type())) {
      $(`${wid} .widget-btns-wrapper .viz-modal-link`).hide();
    } else {
      $(`${wid} .widget-btns-wrapper .viz-modal-link`)
        .offOn('click.viz-modal-link', () => {
          if (Number(autoRefresh.value()) > 0) {
            pauseAutoRefresh();
          }

          hideTooltip();
          drawInModal();
        });
    }

    // This is edit mode
    if (!displaydb) {
      if (isNew) {
        report.showEmptyTitleSubtitle(true);
      }

      // Hide/show visual menus when title/subtitle are being edited
      d3.selectAll(`${wid} .arc-viz-title, ${wid} .arc-viz-subtitle`)
        .on('focus.title', () => {
          $(`${wid} .widget-btns-wrapper`).hide();
        })
        .on('blur.title', () => {
          $(`${wid} .widget-btns-wrapper`).show();
        });

      // Blur title and subtitle when clicking on other parts of visual
      $(wid).on('click.blurtitle', (e) => {
        // no click within title
        if ($(e.target).closest('.arc-viz-title').length === 0) {
          $(`${wid} .arc-viz-title`).blur();
        }

        // no click within title
        if ($(e.target).closest('.arc-viz-subtitle').length === 0) {
          $(`${wid} .arc-viz-subtitle`).blur();
        }
      });

      setupVisualInfoPopover(report, wid);

      // remove gridster widget when close button is clicked
      d3.selectAll(`${wid} .widget-btns-wrapper .close-app`)
         .on('click.close-app', function() {
            hideTooltip();
            $wid.find('a').tooltip('dispose');
            dashboard.removeFromGrid(report, $(this).closest('li'));
         });

      searchHandler.setupSearch(report);

      const $w = $(wid);
      $w.find('.download-container')
        .off('click.download')
        .on('click.download', hideTooltip);

      $w.find('.explore-dropdown').off('click.explore')
        .on('click.explore', '.explore', function() {
          const exploreType = $(this).attr('data-explore');

          const exploreData = arc.showme.setupSettings({
            vizDriver: report.viz(),
          });

          const selectedExplore = exploreData.filter((explore) => explore.id === exploreType);
          selectedExplore.forEach((explore) => explore.fn());
        });
    }
  };

  function appLink(appid) {
    const url = arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.viewDashboardUrl}/${appid}`);
    return `<a target="_blank" href="${url}">${appid}</a>`;
  }

  /*
   * Displays a popover with information about the visual in dashboard edit mode.
   * Uses manual triggers to ensure that popover stays open if the user wants to
   * interact with elements within it.
   */
  function setupVisualInfoPopover(report, wid) {
    const $popoverSelector = $(`${wid} .widget-btns-wrapper .viz-info`);
    const popoverId = `popover-visual-${arc.misc.uniqueId()}`;

    function content() {
      const viz = report.viz();
      if (!viz) {
        return '';
      }

      let dashboardUsage;
      const multipleDashboardsStr = report.shared_visual_dashboards();
      let dashboardUsageClass;
      if (multipleDashboardsStr) {
        const multipleDashboards = JSON.parse(multipleDashboardsStr).map(appLink);
        dashboardUsageClass = '';
        dashboardUsage = `<span class="badge badge-secondary badge-pill">${multipleDashboards.length}</span> ${multipleDashboards.join(', ')}`;
      } else {
        dashboardUsageClass = 'arc-invisible';
        dashboardUsage = '';
      }

      const dataset = report.dataset();

      let datasetId;
      let datasetName;
      let dataconnectionName;

      if (dataset) {
        datasetId = dataset.dataset_id();
        datasetName = dataset.dataset_name();
        dataconnectionName = dataset.dataconnection_name();
      } else {
        datasetName = 'Not set';
        dataconnectionName = 'Not set';
      }

      const tags = dataset.tags();
      const isFilter = arc.misc.isFilterVisual(viz.viz_type());
      let paramOutput = isFilter ? viz.viz()?.viz_getOutputParameter() : '';
      if (paramOutput) {
        paramOutput = `Output: ${paramOutput}`;
      }

      return arc.misc.safe_substitute(visualPopoverContentTemplate,
          {
            POPOVER_ID: popoverId,
            PICKLIST_HEADER_CLASS: (!paramOutput && isFilter) ? 'arc-invisible' : '',
            TITLE: paramOutput || viz.title() || viz.subtitle() || '(no title)',
            ID: viz.report_id(),
            LAST_UPDATED_DATETIME: moment.utc(report.lastUpdatedDate()).fromNow(),
            LAST_UPDATED_USER: report.lastUpdatedUser(),
            DATASET_URL: arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.datasetEditUrl}/${datasetId}`),
            DATASET_NAME: datasetName,
            DATACONNECTION_NAME: dataconnectionName,
            DATACONNECTION_URL: arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.dataUrl}?dataconnection_name=${dataconnectionName}`),
            WORKSPACE_NAME: report.workspace() ? report.workspace().workspace_name : 'Not set',
            DASHBOARD_USAGE_CLASS: dashboardUsageClass,
            HTML_DASHBOARD_USAGE: dashboardUsage,
            HTML_DATASET_TAGS: tags.join(''),
            TAGS_VISIBILITY: tags.length === 0 ? 'arc-invisible' : '',
          });
    }

    arc.misc.setupMarkablePopover({
      $popoverSelector,
      content,
      popoverId,
    });
  }

  function setupDashboardInfoHandler() {
    const $popoverSelector = $('.viz-params-btn.viz-dashboard-info');
    const popoverId = `popover-dashboard-${arc.misc.uniqueId()}`;

    const filterIds = dashboard._get_control_widgets();
    const visualIds = getGridsterWidgets();

    let workspaceName;
    if (workspaces.currentWorkspace()) {
      workspaceName = workspaces.currentWorkspace().workspace_name;
    } else {
      workspaceName = 'Not set';
    }

    const filtersDisplay = `<span class="badge badge-secondary badge-pill">${filterIds.length}</span> ${filterIds.map((i) => (`${i}`)).join(', ')}`;
    const visualsDisplay = `<span class="badge badge-secondary badge-pill">${visualIds.length}</span> ${visualIds.map((i) => (`${i}`)).join(', ')}`;

    const filtersAndVisuals = filterIds.map((id) => reports.getReport(id));

    visualIds.map((id) => reports.getReport(id))
      .forEach((r) => { filtersAndVisuals.push(r); });

    const datasets = filtersAndVisuals
      .filter((report) => report && report.viz() && report.viz().dataset())
      .map((r) => r.viz().dataset());

    const uniqueDatasets = [];
    const datasetsH = {};

    datasets.forEach((d) => {
      const id = d.dataset_id();
      if (datasetsH[id]) {
        return;
      }

      datasetsH[id] = d;
      uniqueDatasets.push(d);
    });

    const datasetsDisplay = `<span class="badge badge-secondary badge-pill">${uniqueDatasets.length}</span> ${uniqueDatasets.map((d) => `
              <a href="${arc.misc.getBlankTargetUrl(arc.frontend.globals.urls.datasetEditUrl)}/${d.dataset_id()}" target="_blank">
                ${arc.misc.escapeHTML(d.dataset_name())}
              </a>
          `)
          .join(', ')}`;

    const dataconnectionsH = {};
    const uniqueDataconnections = [];
    const uniqueIds = [];
    uniqueDatasets.forEach((d) => {
      const id = d.dataconnection_id();
      if (dataconnectionsH[id]) {
        return;
      }

      dataconnectionsH[id] = d;
      uniqueIds.push(id);
      uniqueDataconnections.push(arc.misc.safe_substitute(
          '<a href="%DATACONNECTION_URL%" target="_blank">%DATACONNECTION_NAME%</a> ',
          {
            DATACONNECTION_NAME: d.dataconnection_name(),
            DATACONNECTION_URL: arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.dataUrl}?dataconnection_name=${d.dataconnection_name()}`),
          },
      ));
    });

    dashboard.active_connection_ids(uniqueIds);

    function content() {
      return arc.misc.safe_substitute(dashboardPopoverContentTemplate,
          {
            POPOVER_ID: popoverId,
            TITLE: $('#dashboard-title').text() || '(no title)',
            ID: dashboard_id,
            HTML_FILTERS: filtersDisplay,
            HTML_VISUALS: visualsDisplay,
            HTML_DATASETS: datasetsDisplay,
            HTML_DATACONNECTIONS: uniqueDataconnections.join(', '),
            WORKSPACE_NAME: workspaceName,
          });
    }

    arc.misc.setupMarkablePopover({
      $popoverSelector,
      content,
      popoverId,
    });
  }

  function activateReportOnClick(report) {
    const wid = `#${widgetId(report)}`;
    const $vizContainer = $(`${wid} .arc-viz-container`);

    if (!arc.globals.is_edit) {
      return;
    }

    $vizContainer.offOn('click.activate', () => {
      const activeReport = dashboard.getActiveReport();
      if (activeReport !== report) {
        dashboard.activateReport(report);
      }
    });

    $vizContainer.offOn('dblclick.activate', () => {
      const activeReport = dashboard.getActiveReport();
      if (activeReport !== report) {
        dashboard.activateReport(report);
      }

      showVisualShelves(report);
    });
  }

  dashboard.reports = function(_) {
    if (!arguments.length) return reports;
    reports = _;

    return dashboard;
  };

  // TODO - this is duplicate code from builder
  // move to arc.js or arc.report.js
  dashboard.setupEditTitle = function() {
    arc.editTitleHelper('does-not-exist', 'dashboard-title');
    arc.editTitleHelper('does-not-exist', 'dashboard-subtitle');
    return dashboard;
  };

  function setParams(report, params) {
    if (!report) {
      return;
    }

    const driver = report.viz();
    if (!driver) {
      return;
    }

    const visual = driver.viz();
    driver.params(params);
    if (visual && visual.updateToSaveSelectionsFromParams) {
      visual.updateToSaveSelectionsFromParams();
    }
  }

  function drawWithSegmentsandEvents(config, datasetData, columninfo) {
    const {
      report,
      dataset_id,
      params,
      isNew,
    } = config;

    const dataset_segments = datasetData.segments;
    const dataset_events = datasetData.events;

    const dataset = arc.datasets.Dataset()
                     .dataset_id(dataset_id)
                     .init_from_fetch(datasetData, columninfo);

    const ds_events = arc.report.events.dataset_events()
                         .dataset(dataset)
                         .events(dataset_events);
    report.dataset_events(ds_events);

    const ds_segments =
        arc.report.dataset_segments()
           .dataset(dataset)
           .parse_segments(dataset_segments);
    report.dataset_segments(ds_segments);
    report.dataset(dataset);

    searchHandler.setupSearch(report);
    searchHandler.activateSearchIfLastActivated(report);

    const report_type = report.report_type();
    try {
      const viz = report.viz();
      if (viz) {
        report.populateViz(report.viz(), report_type, isNew);
      } else {
        report.createViz(report_type, isNew);
      }
    } catch (err) {
      console.log(err);
      return;
    }

    setParams(report, params);

    if (!isNew) {
      const report_parsed_data = JSON.parse(report.saved_data());
      report.viz().restore(report_parsed_data.report_data);
    }

    report.viz().dashboard_click_behavior(
                    dashboard.get_click_behavior(report.report_id())
)
                .click_behavior_change_cb(
                    (marker_name, filtering_enabled) => {
                      const report_id = report.report_id();
                      dashboard.save_click_behavior(report_id, marker_name,
                          filtering_enabled);
                    }
);

    if (arc.globals.parameters && report_type === 'picklist') {
      arc.globals.parameters.addPicklist(report);
      filtersLoaded += 1;
      if (filtersLoaded >= filtersToBeLoaded && arc.globals.parameters) {
        arc.globals.parameters.finalize(dashboard);
      }
    }

    // Call setupVisualInModal if redrawing segments and events after _cloneVisualCb
    if (arc.globals.fillModalConfig) {
      arc.globals.singleVisualBuilder.setupVisualInModal(arc.globals.fillModalConfig);
      arc.globals.fillModalConfig = undefined;
    }

    if (!getRangeType(report)) {
      hideFilterSwitchButton(report);
    }

    if ([
      'daterange',
      'picklist',
      'range',
    ].includes(report.report_type())) {
      setSwitchTooltip(report);
    }

    drawViewData(report);
    dashboard.drawReport(report, arc.misc.isFilterVisual(report_type));
  }

  dashboard.save_click_behavior = function(report_id, marker_name,
      filtering_enabled) {
    click_behavior.reports = click_behavior.reports || {};
    click_behavior.reports[report_id] = click_behavior.reports[report_id] || {};
    click_behavior.reports[report_id][marker_name] = filtering_enabled;
    arc.report.pagenav.needs_save(true);
    return dashboard;
  };

  dashboard.get_click_behavior = function(report_id) {
    if (click_behavior.reports && click_behavior.reports[report_id]) {
      return click_behavior.reports[report_id];
    }

    return {};
  };

  /*
   * Fetch dataset information from the backend and call success_cb.
   *
   * Expected parameters: config must contain dataset_id
   *
   * Note that this function may modify config.dataset_id if dataset
   *     switching is in place
   *
   */
  dashboard.fetchDataset = function(config, success_cb, error_cb) {
    const {
      dataset_id,
    } = config;

    let actual_dsid = dataset_id;
    if (arc.globals.dssw && arc.globals.dssw[actual_dsid]) {
      actual_dsid = arc.globals.dssw[actual_dsid];
      config.dataset_id = actual_dsid;
    }

    const assets = [
      {
        asset: `${arc.frontend.globals.urls.datasetUrl}/${actual_dsid}?detail=true`,
        postprocess: d3.json,
      },
    ];

    arc.staticfetcher.fetch(assets,
      (error, datasets) => {
        if (error) {
          arc.report.alertMessage('Failed to load segments and events',
              error, { alert_report_id: sheet_id });
          if (error_cb) {
            error_cb();
          }

          return;
        }

        const datasetData = datasets[0];

        // For single table datasets right now, need to fetch the table column info
        if (datasetData.type === 'singletable' && datasetData.info.length === 0) {
          arc.staticfetcher.fetch([{
              asset: arc.datasets.dataset_columns_url(datasetData.dc_id, datasetData.detail),
              postprocess: d3.json,
            }], (err, columninfo) => {
              if (err) {
                arc.report.alertMessage('Failed to load dataset table column detail',
                    err.response ? err.response : err,
                    { alert_report_id: sheet_id });
                if (error_cb) {
                  error_cb();
                }

                return;
              }

              success_cb(config, datasetData, columninfo);
            });
        } else {
          success_cb(config, datasetData);
        }
      });

    return dashboard;
  };

  dashboard.drawReport = function(report, filter_widget, force_redraw) {
    arc.css.CUSTOMCSS_MANAGER.applyCustomCss();

    setTimeout(() => {
      report.refresh(force_redraw, filter_widget);
    }, 0);

    return dashboard;
  };

  dashboard.reloadParams = function(report, params, force_redraw) {
    if (report.viz()) {
      report.viz().params(params);
    }

    setTimeout(() => {
      report.refresh(force_redraw);
    }, 0);
  };

  dashboard.addFilter = function(report_id, viz) {
    $('.control-instructions').hide();

    let report = reports.getReport(report_id);
    filtersToBeLoaded += 1;

    const params = dashboard.getDashboardParamsTracker();
    if (globalFilters) {
      report = globalFilters.getFilterReport(report_id);
      if (report) {
        report = reports.addReport(report_id, dashboard, report);
        setParams(report, params);

        if (viz) {
          report.viz(viz);
        }

        addFilterElement(report);
        dashboard.drawReport(report, /* filter_widget */ true, /* force_redraw */ true);
        dashboard_filter_controls.filterContainerVisibility();
        return;
      }
    }

    report = reports.addReport(report_id, dashboard);
    setParams(report, params);

    if (singleVisualDashboard.isChecked) {
      shouldShowSingleVisual();
      clearSingleVisualCss(true);
    }

    if (viz) {
      report.viz(viz);
    }

    addFilterElement(report);

    dashboard.fetchReportAndDraw(report, params);

    if (globalFilters) {
      globalFilters.addFilterToReportMapping(report_id, report);
    }

    dashboard_filter_controls.filterContainerVisibility();

    return dashboard;
  };

  dashboard.hideInstructions = function () {
    $('#dashboard-instruction').hide();
  };

  dashboard.showInstructions = function () {
    $('#dashboard-instruction').show();
  };

  dashboard.showLayoutRearrangeButton = function () {
    $('#open-rearrange-modal').show();
  };

  dashboard.hideLayoutRearrangeButton = function () {
    $('#open-rearrange-modal').hide();
  };

  dashboard.drawClonedWidgetFromModal = function(oldId, newId) {
    // Add new report to grid
    const currWidget = gridster.serialize().find((widg) => widg.id.endsWith(`-${oldId}`));

    dashboard.drawInWidget({
      report_id: newId,
      force_redraw: false,
      force_reloadreport: false,
      widget_sizex: currWidget.size_x,
      widget_sizey: currWidget.size_y,
      shouldUpdateDatasetsSelector: true,
    });
    dashboard.scrollToBottom();
  };

  dashboard.drawInWidget = function({
    report_id,
    force_redraw,
    force_reloadreport,
    widget_sizex,
    widget_sizey,
    widget_col,
    widget_row,
    params_data,
    click_report_id,
    unpivotExistingParams,
    isNew,
    vizDriver,
    cb,
    shouldUpdateDatasetsSelector,
  }) {
    dashboard.hideInstructions();
    dashboard.showLayoutRearrangeButton();
    if (arc.frontend.globals.isMobileView) {
      widget_sizex = 1;
      widget_sizey = 1;
    }

    if (!widget_sizex) {
      widget_sizex = widgetXColumns;
    }

    if (!widget_sizey) {
      widget_sizey = widgetYColumns;
    }

    const params = arc.misc.paramsFromData(params_data);
    let report = reports.getReport(report_id);
    if (report !== null) {
      if (vizDriver && report.viz() !== vizDriver) {
        report.viz(vizDriver);
      }

      // In case report refers to linked visual that is already loaded and present in another sheet
      reports.addReport(report_id, dashboard, report);
      if (force_redraw) {
        dashboard.addGridsterElement(
          report,
          widget_sizex,
          widget_sizey,
          widget_col,
          widget_row,
          isNew
        );
      }

      // force_reloadreport and force_redraw are true when loading sheet
      // force_reloadreport only is true when user clicks "Refresh" icon on visual
      // force_redraw only is true when adding new linked visual, resizing, moving filter to grid
      if (force_reloadreport && !report.chart_div()) {
        // Only refetch if report wasn't initialized already
        dashboard.fetchReportAndDraw(
          report,
          params,
          isNew,
          undefined,
          shouldUpdateDatasetsSelector
        );
      } else {
        setTimeout(() => {
          // TODO: Hack to make dashboard.reloadParams execute after dashboard.activate
          dashboard.reloadParams(report, params, force_redraw);
        }, 1000);
        const lib = report.viz();
        if (lib) {
          if (lib.report_id() !== click_report_id) {
            lib.unselectAll();

            // TODO: Call a function that highlights correct marks
            // report.viz().contextmenu()._activateClick();
          }

          if (unpivotExistingParams) {
            lib.unpivotParamsExcept(params_data);
          } else {
            lib.pivotParams(params_data, 'refreshNow');
          }
        }
      }

      return dashboard;
    }

    if (report === null) {
      // Add a new report to gridster
      report = reports.addReport(report_id, dashboard);
      if (report && vizDriver && report.viz() !== vizDriver) {
        report.viz(vizDriver);
      }

      setParams(report, params);
      dashboard.addGridsterElement(
        report,
        widget_sizex,
        widget_sizey,
        widget_col,
        widget_row,
        isNew
      );
    }

    dashboard.fetchReportAndDraw(report, params, isNew, cb, shouldUpdateDatasetsSelector);
  };

  function drawShelvesAndSegments(report) {
    const uri = report.unique_report_id();
    d3.selectAll(`#app-shelves-${uri}`)
      .remove();
    d3.select('body')
      .append('div')
      .style('display', 'none')
      .attr('id', `app-shelves-${uri}`);
    report.shelves_dom_id(`app-shelves-${uri}`);
  }

  function initializeReport(report, report_data, shouldUpdateDatasetsSelector) {
    const wnames = widgetNames(report);

    const loader = arc.loadbar()
                    .dom_id(`#${wnames.refresh}`)
                    .refresh_icon(true);

    const workspace = {
      workspace_id: report_data[0].workspace_id,
      workspace_name: report_data[0].workspace_name,
      workspace_perm: report_data[0].workspace_perm,
    };

    // DJC -- More tablename/dataset issues
    report.dataset_id(report_data[0].dataset_id)
          .dashboard(dashboard)
          .in_dashboard(true)
          .chart_div(wnames.app)
          .title_elem(wnames.title)
          .subtitle_elem(wnames.subtitle)
          .loader(loader)
          .lastUpdatedDate(report_data[0].updated)
          .lastUpdatedUser(report_data[0].updated_by)
          .workspace(workspace)
          .shared_visual_dashboards(report_data[0].shared_visual_dashboards)
          .report_description(report_data[0].description)
          .reportSearchText(report_data[0].report_search_text)
          .report_type(report_data[0].type)
          .saved_data(report_data[0].data);

    report.updateSharedIconInDashboard();

    // Remove any sidebar popover before updating dataset shortcuts
    $('.popover').remove();

    if (shouldUpdateDatasetsSelector) {
      updateDatasetsShortcuts(report_data[0].dataset_id);
    }
  }

  dashboard.fetchReport = function(report, cb, shouldUpdateDatasetsSelector) {
    const report_id = report.report_id();

    // Get details of the report including shelf data
    arc.staticfetcher.fetch([{
      asset: `${arc.frontend.globals.urls.reportUrl}/${report_id}`,
      postprocess: d3.json,
    }], (error, report_data) => {
      if (error || report_data.length === 0) {
        const errorMessage = error && error.responseText ? error.responseText : (error || '');
        arc.report.alertMessage(`Error fetching data for visual ${report_id}`,
          errorMessage,
          { alert_report_id: sheet_id });
        return;
      }

      initializeReport(report, report_data, shouldUpdateDatasetsSelector);
      if (cb) {
        cb(report_data);
      }
    });
  };

  dashboard.fetchReportAndDraw = function(report, params, isNew, cb, shouldUpdateDatasetsSelector) {
    function drawAfterFetch() {
      drawShelvesAndSegments(report);
      configurePicklistMoveAndIcons(report);
      dashboard.configureMapHandlers(report);
      dashboard.fetchDataset(
        {
          report,
          dataset_id: report.dataset_id(),
          params,
          isNew,
        },
        (config, datasetData, columninfo) => {
          drawWithSegmentsandEvents(config, datasetData, columninfo);
          if (cb) {
            cb();
          }
        }
      );
    }

    dashboard.fetchReport(report, drawAfterFetch, shouldUpdateDatasetsSelector);
  };

  function liIsGridster($li) {
    return $li.hasClass('gs-w');
  }

  dashboard.configureMapHandlers = function(report) {
    // This function prevents mouse events that do panning/zooming on maps from
    // affecting gridster
    if (displaydb) {
      return;
    }

    const reportType = report.report_type();
    if (reportType !== 'leaflet' && reportType !== 'map') {
      return;
    }

    try {
      if (reportType === 'map') {
        const saved_data = JSON.parse(report.saved_data());
        if (saved_data.report_data.user_settings.zoom_enabled === 'false') {
          return;
        }
      }
    } catch (e) {
      return;
    }
  };

  function configurePicklistMoveAndIcons(report) {
    // Don't render icons in view mode or not picklist.
    if (displaydb || report.report_type() !== 'picklist') {
      return;
    }

    const showMoveAppIcon = !arc.report.isAppScopeFilter(report && report.viz());
    const wnames = widgetNames(report);
    const $li = $(`#${wnames.app},#${wnames.name}`).first();
    const $btn_wrapper = $li.find('.widget-btns-wrapper');

    if ($btn_wrapper.find('.move-app').length > 0) {
      // move-app already exists
      $btn_wrapper.find('.move-app').toggleClass('arc-invisible', !showMoveAppIcon);
      return;
    }

    if (liIsGridster($li)) {
      $btn_wrapper.prepend($('#widget-popout-btn-template-grid').html());
      $btn_wrapper.find('.edit-app').attr('title', 'configure filter')
                                    .toggleClass('fa-pencil fa-cog');
    } else {
      $btn_wrapper.prepend($('#widget-popout-btn-template-filter').html());
    }

    const $move = $btn_wrapper.find('.move-app');
    $move.toggleClass('arc-invisible', !showMoveAppIcon);
    $move.offOn('click.moveApp', () => {
      $move.tooltip('hide');
      if (liIsGridster($li)) {
        /* When we move the visual    from grid to filter area it hits
           the logic that otherwise keeps the right order between
           switched filters during switching, however in this case
           we can't guarantee the correct order so we rather skip
           this logic via the isFromGrid flag and draw the visual
           as the last element in the widget container. */
        report.isFromGrid = true;
        moveReportFromGrid(report);
        setSwitchTooltip(report);
      } else {
        moveReportToGrid(report);
      }
    });
  }

  dashboard.loadById = (sheetId) => {
    arc.staticfetcher.fetch([{
      asset: `${arc.frontend.globals.urls.reportUrl}/${sheetId}`,
      postprocess: d3.json,
    }], (error, report_data) => {
      if (error || report_data.length === 0) {
        const errorMessage = error && error.responseText ? error.responseText : (error || '');
        arc.report.alertMessage('Error loading app', errorMessage,
          { alert_report_id: sheetId });
        return;
      }

      const currentData = report_data[0];

      arc.misc.updateCloneButton(currentData.dataset_perm);
      arc.misc.updateAccelerateButton(currentData.dataset_perm);

      dashboard.load(sheetId, currentData.name,
        JSON.parse(currentData.data));
    });
  };

  dashboard.redrawExistingWidgetsWithParams = function(datum, index,
                             in_click_behavior, params_fn, viz, elem,
                             in_params_data, do_not_refresh) {
    // Later, when we want to add a widget for a non-existing report,
    // we can use in_click_behavior.reportid and add a widget for it first, and
    // then call the rest of the code

    const lastStore = params_history.current_params()[0];
    const hasStore = lastStore.length > 0;
    const paramArrToUse = hasStore ? lastStore : initialParams;
    let params_data = in_params_data;

    if (params_data === undefined) {
      if (params_fn) {
        params_data = params_fn(datum, index, elem);
      }
    }

    const updatedParams = params_history.update(params_data, paramArrToUse);

    if (hasStore || allFilterCount === 0) {
      params_history.add(updatedParams, viz, do_not_refresh);
    } else if (initialParamCount < allFilterCount &&
      !params_history.is_identical(paramArrToUse, updatedParams)) {
        initialParams.push(...params_data);
        initialParamCount += 1;

        if (initialParamCount === allFilterCount) {
          params_history.add(updatedParams, viz, do_not_refresh);
        }
    }

    return dashboard;
  };

  dashboard.resetControls = function() {
    const control_widgets = getPicklistWidgets();
    control_widgets.forEach((report_id) => {
      const r = reports.getReport(report_id);
      r.viz().viz().viz_resetControl(true);
    });

    return dashboard;
  };

  function refreshControls(params_data, updateFromParamHistory, picklistWidgets) {
    const params = arc.misc.paramsFromData(params_data);
    const listOfNewParamsSourceVisualIds = {};
    params_data.forEach(function populateAboveLsits(param) {
      listOfNewParamsSourceVisualIds[param.source_visual_id] = true;
    });

    picklistWidgets.filter((reportId) => !listOfNewParamsSourceVisualIds[reportId] ||
                                       updateFromParamHistory)
                   .forEach((report_id) => {
      const r = reports.getReport(report_id);
      if (!r) {
        return;
      }

      let force_redraw = false;
      setParams(r, params);

      const visual = r.viz() ? r.viz().viz() : undefined;

      let shouldUpdate;
      if (visual) {
        shouldUpdate = updateFromParamHistory;
        if (!shouldUpdate) {
          if (visual.viz_conflictingParams) {
            shouldUpdate = visual.viz_conflictingParams();
          }
        }
      }

      if (shouldUpdate) {
        visual.updateToSaveSelectionsFromParams();
        force_redraw = true;
      }

      dashboard.drawReport(r, /* filter_widget */ true, force_redraw);
    });

    const control_widgets = dashboard._get_control_widgets();
    control_widgets.forEach((report_id) => {
      const r = reports.getReport(report_id);
      setupVisualInfoPopover(r, `#${r.chart_div()}`);
      const driver = r.viz();
      if (driver) {
        driver.refresh(/* force_redraw */ false, /* filter_widget */ true);
      }
    });

    timeControlFilter((viz) => {
      viz.viz_lib().refresh();
    });

    return dashboard;
  }

  dashboard.redrawExistingWidgetsWithParamsData = function(params_data, viz,
    updateFromParamHistory, prev_params_data, forward) {
    if (arc.globals.send_message_params) {
      /*
        Send information about current parameters to any containing window.

        The containing window needs to contain code similar to this:

        <head></head>
        <body>
        <script>
        window.addEventListener("message", receiveMessage, false);

        function receiveMessage(event) {
          // This is where you receive the posted messages
          console.log(event.data);
        }
        </script>

        <iframe src="http://127.0.0.1:8000/arc/apps/app/106?embed=true&send_message_params=true">
        </iframe>
        </body>
       */
      if (window.parent !== window) {
        window.parent.postMessage({
          type: 'parameters',
          data: params_data
        }, '*');
      }
    }

    // Clear previous errors when parameter navigation causes visuals refresh
    arc.report.alertMessageAllHide();

    const picklistWidgets = getPicklistWidgets();
    const picklistWidgetHash = {};
    picklistWidgets.forEach((report_id) => {
      picklistWidgetHash[report_id] = 1;
    });

    refreshControls(params_data, updateFromParamHistory, picklistWidgets);

    if (gridster) {
      // Collect all the widgets in existing dashboard, and redraw each one
      // Pass in params
      const jdd = gridster.serialize();
      $.each(jdd, (_, element) => {
        const report_id = dashboard.gridsterElementToReportId(element.id);
        if (picklistWidgetHash[report_id]) {
          return;
        }

        dashboard.drawInWidget({
          report_id,
          force_redraw: false,
          force_reloadreport: false,
          widget_sizex: element.size_x,
          widget_sizey: element.size_y,
          widget_col: element.col,
          widget_row: element.row,
          params_data,
          click_report_id: viz ? viz.report_id() : undefined,
          unpivotExistingParams: (updateFromParamHistory && !forward)
        });
      });
    }

    return dashboard;
  };

  dashboard.getDatasetId = function() {
    // TODO: Use dashboard.getPrimaryReport() to get report and return its dataset_id
    const dashboard_widgets = gridster.serialize();
    if (dashboard_widgets.length === 0) {
      return undefined;
    }

    const topleft_report_idx = dashboard.getDrivingReport(dashboard_widgets);
    const reportid = dashboard.gridsterElementToReportId(
        dashboard_widgets[topleft_report_idx].id
    );
    const r = reports.getReport(reportid);
    if (r) {
      return r.dataset_id();
    }

    // TODO: Fix else case here for null case
  };

  dashboard.getPrimaryReport = function getPrimaryReportInDashboard() {
    if (!gridster) {
      return reports.getPrimaryReport();
    }

    const dashboard_widgets = gridster.serialize();
    if (dashboard_widgets.length === 0) {
      return undefined;
    }

    const topleft_report_idx = dashboard.getDrivingReport(dashboard_widgets);
    const reportid = dashboard.gridsterElementToReportId(
        dashboard_widgets[topleft_report_idx].id
);
    return reports.getReport(reportid);
  };

  dashboard.getDrivingReport = function(jdd) {
    // Simplifying assumption:
    // Save the top left report's dataset to be the dashboard's dataset and
    // use this to figure out permissions
    let topleft_report_idx = 0;
    let topleft_col = jdd[0].col;
    let topleft_row = jdd[0].row;
    $.each(jdd, (idx, element) => {
      if (element.row < topleft_row) {
        topleft_report_idx = idx;
        topleft_row = element.row;
        topleft_col = element.col;
      } else if (element.row === topleft_row) {
        if (element.col < topleft_col) {
          topleft_report_idx = idx;
          topleft_row = element.row;
          topleft_col = element.col;
        }
      }
    });

    return topleft_report_idx;
  };

  function getGridsterWidgets() {
    const dashboard_widgets = gridster.serialize();
    return dashboard_widgets.map((w) => dashboard.gridsterElementToReportId(w.id));
  }

  const picklistSelector = 'ul.control-list li.selected-control:not(.ui-sortable-placeholder)';
  function getPicklistWidgets() {
    // Future TODO: Don't strip this from the information from the DOM
    const wl = [];
    const $lis = $(`${picklistSelector}, .arc-viz-type-picklist`);

    $lis.each(function() {
      const $this = $(this);

      // If the picklist is in the grid, get containing li and use report id from that
      // If picklist is in the filter area, use the picklist id
      const gwid = this.id;
      let report_id = dashboard.gridsterElementToReportId(gwid);
      if (!report_id) {
        report_id = $this.closest('li').attr('data-arc-reportid');
      }

      wl.push(report_id);
    });
    return wl;
  }

  dashboard._get_control_widgets = function() {
    const $lis = $(picklistSelector);
    const control_widgets = [];
    $lis.each(function() {
      const gwid = $(this).attr('id');
      const report_id = dashboard.gridsterElementToReportId(gwid);
      control_widgets.push(report_id);
    });
    return control_widgets;
  };

  function getModifiedReports() {
    return reports
            .getAllReports()
             // Reports on other sheets might have undefined 'viz'
            .filter((report) => report?.viz?.()?.needs_save());
    }

  dashboard.save = function(ms_dashboard_id, _, dashboardDatasetId, save_cb, error_cb) {
    // First, save away any widgets that need saving. This may require saving
    // away individual visuals as well. Then, save away the dashboard.

    const modifiedReports = getModifiedReports();
    let modifiedReportsCount = modifiedReports.length;
    let has_errors = false;

    function save_after_all_controls() {
      // Don't try saving while resizing -- gridster will be empty
      if (resizing) {
        setTimeout(save_after_all_controls, 500);
      } else if ((modifiedReportsCount === 0) && !has_errors) {
        dashboard.save_after_controls(ms_dashboard_id, dashboardDatasetId, save_cb, error_cb);
      }
    }

    save_after_all_controls();

    modifiedReports.forEach((r) => {
      // r.viz() is not present if user adds duplicate viz
      // Or otherwise bad viz to the dashboard
      dashboard.save_report(r, (error) => {
        modifiedReportsCount -= 1;
        if (error) {
          has_errors = true;
        }

        save_after_all_controls();
      });
    });

    return dashboard;
  };

  function get_post_data_for_save_or_clone(ms_dashboard_id, for_clone, shared_visuals_to_clone,
                                  dashboardDatasetId) {
    const sheetVisualWidgets = gridster.serialize();
    const jdd = JSON.parse(JSON.stringify(sheetVisualWidgets));

    if (!dashboardDatasetId) {
      if (jdd.length) {
        const topleft_report_idx = dashboard.getDrivingReport(jdd);
        dashboardDatasetId =
          reports.getReport(dashboard.gridsterElementToReportId(
            jdd[topleft_report_idx].id
          )).dataset_id();
      }

      if (!dashboardDatasetId) {
        arc.report.alertMessage('Cannot save empty dashboard, please add a visual ' +
          'to at least one sheet before saving it', '', { alert_report_id: dashboard_id });
        return undefined;
      }
    }

    const dd = $.extend({}, dashboard.dashboard_data());
    dd.time_control_settings = dashboard_filter_controls.getTimeControlSettings();
    const sheetData = dd.dashboard_sheets.filter((ds) => ds.sheet_id === sheet_id)[0];
    const dashboardName = d3.select('#dashboard-title').text();
    let dashboard_name_r = dashboardName.replace(/^\s+/g, '');
    if (for_clone) {
      dashboard_name_r = `Clone of ${dashboard_name_r}`;
    } else {
      // for each sheet, identify order from handles and update it
      dd.dashboard_sheets.forEach((sheet) => {
        const $currentSheetHandle =
          $(`.dash-sheets-container ul.dashboard-sheet-handles li.sheet-handle[arc-sheet-id=${
            sheet.sheet_id}]`);
        sheet.order = $currentSheetHandle.index() + 1;
      });
    }

    sheetData.behaviors = click_behavior;
    dd.numColumns = numColumns;
    dd.metadataVersion = metadataVersion;

    if (!displaydb) {
      processUserSettings();
    }

    saveUserSettings(dd);

    updateControlWidgets(sheetData, dd);

    sheetData.visual_widgets = sheetVisualWidgets;
    dd.dashboard_widgets = [];
    dd.dashboard_sheets.forEach((sheet) => {
      dd.dashboard_widgets = dd.dashboard_widgets.concat(sheet.visual_widgets.slice(0));
    });

    if (for_clone && arc.globals.arcviz_settings.ENABLE_URL_NAMES) {
      dd.user_settings.dashboard_url_name = '';
    }

    dd.report_title = dashboard_name_r;
    dd.report_subtitle = $('#dashboard-subtitle').text().trim();
    sheetData.sheet_handle_title =
      $(`ul.dashboard-sheet-handles [arc-sheet-id=${sheet_id}] .sheet-title`).text().trim();
    const toStoreDashboardData = JSON.stringify(dd);
    const sharedUnshared = reports.getSharedUnsharedVisuals();

    const post_data = {
      dashboard_name: dashboard_name_r,
      dashboard_description: 'placeholder desc',
      dashboards_shared_visuals: JSON.stringify(sharedUnshared.sharedVisuals),
      unshared_visuals: JSON.stringify(sharedUnshared.unsharedVisuals),
      dashboard_data: toStoreDashboardData,
      csrfmiddlewaretoken: arc.globals.csrf_token,
      dashboard_dataset_id: dashboardDatasetId,
      dashboard_workspace_id: workspaces.currentWorkspace() &&
                              workspaces.currentWorkspace().workspace_id,
    };

    if (for_clone) {
      post_data.clone_data = JSON.stringify({
        parent_dashboard_id: dashboard_id,
        shared_visuals_to_clone: shared_visuals_to_clone ?? [],
        not_saved_report_data_of_original_dashboard: getNotYetSavedReportOfCurrentSheet(),
      });
    }

    // If sheet_id is not specified, a new dashboard will be created
    // And the sheet_id will be added to report_data.report_data for all unshared visuals
    if (sheet_id) {
      post_data.dashboard_id = ms_dashboard_id;
    }

    return post_data;
  }

  function updateControlWidgets(sheetData, dd) {
    const listOfCurrentFilters = dashboard._get_control_widgets();

    const globalControlWidgets = [];
    const sheetControlWidgets = [];
    listOfCurrentFilters.forEach(function populateGlobalAndSheetControlWidgets(cw) {
      const report = reports.getReport(cw);

      const isAppScope = arc.report.isAppScopeFilter(report && report.viz());
      if (report && !isAppScope) {
        sheetControlWidgets.push(cw);
      } else {
        globalControlWidgets.push(cw);
      }
    });

    dd.global_control_widgets = globalControlWidgets;
    sheetData.control_widgets = sheetControlWidgets;

    dd.time_control_settings = dashboard_filter_controls.getTimeControlSettings();
  }

  dashboard.save_after_controls = function(dashboardId, dashboardDatasetId, save_cb, error_cb) {
    const post_data = get_post_data_for_save_or_clone(dashboardId, null, null, dashboardDatasetId);
    if (post_data === undefined) {
      return dashboard;
    }

    jQuery.post(arc.frontend.globals.urls.dashboardSaveUrl, post_data, (data) => {
      reports_to_delete.forEach((report) => {
        report.delete(function deleteSuccess() {
          const index = reports_to_delete.indexOf(report);
          if (index >= 0) {
            reports_to_delete.splice(index, 1);
          }
        }, null, dashboardId);
      });

      reports_to_update.forEach((report) => {
        report.updateSharedVisualDashboards(function updateSuccess() {
          const index = reports_to_update.indexOf(report);
          if (index >= 0) {
            reports_to_update.splice(index, 1);
          }
        });
      });

      if (!dashboardId) {
        dashboard.dashboard_id(data.id);
        const url = `${arc.frontend.globals.urls.staticUrl}/../apps/builder/${data.id}`;

        // Modify the URL to reflect the new report id
        if (window.history.replaceState) {
          arc.misc.replaceState(url);
          const l = $('.subnav-report .view-link-container');
          l.show();
          l.find('a').attr('href', `${arc.frontend.globals.urls.staticUrl}/../apps/app/${data.id}`);
          const currentWorkspace = {
            workspace_id: data.workspace_id,
            workspace_name: data.workspace_name,
            workspace_perm: data.workspace_perm,
          };

          // Add new private workspace and set as current workspace
          if (arc.globals.editWorkspace) {
            arc.globals.editWorkspace.push(currentWorkspace);
          }

          if (arc.globals.workspaces) {
            arc.globals.workspaces.push(currentWorkspace);
          }

          workspaces.originalWorkspace(currentWorkspace);
          workspaces.currentWorkspace(currentWorkspace);
          workspaces.setupWorkspaceButton($('.subnav-report'));
        } else {
          window.open(url, '_self');
        }
      }

      if (save_cb) {
        dashboard_data = JSON.parse(post_data.dashboard_data);
        if (window.top) {
          window.top.postMessage({
            topic: 'viz.document.saved',
            value: {
              name: dashboard_data.report_title,
              id: String(dashboardId ?? data.id),
            },
          }, '*');
        }

        save_cb(data.id, dashboard_data);
      }

      const dsid = post_data.dashboard_dataset_id;

      if (dataset_hash[dsid]) {
        const dp = dataset_hash[dsid].dataset_perms();
        arc.misc.updateAccelerateButton(dp);
      } else {
        console.log(`dataset_hash is empty or dataset_hash is missing dsid: ${dsid}`);
      }
    })
    .fail((req) => {
      const response = req.responseText;
      arc.report.alertMessage(
        'Failed to save the dashboard',
        response?.length ? response : 'Check notification center for error details',
        { alert_report_id: sheet_id, alertContainerSelector: '.dash-main-panel' },
        );
      if (error_cb) {
        error_cb(req);
      }
    });
  };

  function addReportToGrid(report_id, isNew, vizDriver, cb) {
    arc.report.pagenav.needs_save(true);
    dashboard.drawInWidget({
      report_id,
      force_redraw: false,
      force_reloadreport: false,
      widget_sizex: widgetXColumns,
      widget_sizey: widgetYColumns,
      isNew,
      vizDriver,
      cb,
      shouldUpdateDatasetsSelector: true
    });
  }

  const $addLinkedModal = $('#add-linked-modal');
  const $linkLabel = $addLinkedModal.find('.link-option');
  const $disabledWarning = $linkLabel.find('.disabled-warning');
  const $disabledId = $linkLabel.find('.disabled-report-id');
  const $linkInput = $addLinkedModal.find('#maintainLink');
  const $cloneInput = $addLinkedModal.find('#cloneVisual');
  const $submitButton = $addLinkedModal.find('.add-visual-btn');
  const $warningText = $addLinkedModal.find('.alert-warning');
  const $containingDashboards = $addLinkedModal.find('.containing-dashboard-list');

  const displayError = (error, alertMessage) => {
    arc.report.alertMessage(
      alertMessage,
      error.responseText ?? error,
      { alert_report_id: sheet_id }
    );
  };

  dashboard.addAppFromSidebar = function(reportData) {
    const { dataset_id } = reportData;
    const report_id = reportData.id;
    if (!reportData) {
      arc.report.alertMessageSmall(`Visual ${report_id} does not exist`,
        'alert-info');
      return;
    }

    if (reportData.shared_visual_dashboards) {
      if (arc.misc.modalShown($addLinkedModal)) {
        return;
      }

      $cloneInput.prop('checked', true);
      const isOnReport = !!(dashboard.reports().getAllSheetReports(sheet_id)[report_id]);
      $linkLabel.toggleClass('disabled', isOnReport);
      $linkInput.prop('disabled', isOnReport);
      $disabledWarning.toggleClass('arc-invisible', !isOnReport);

      if (isOnReport) {
        $disabledId.text(report_id);
      }

      // Only show warning if user decides to maintain
      $warningText.addClass('arc-invisible');
      $addLinkedModal.off('change.showWarning')
                     .on('change.showWarning', '#maintainLink', function showAttention() {
        $warningText.removeClass('arc-invisible');
        const sharedDashboards = JSON.parse(reportData.shared_visual_dashboards);
        if (sharedDashboards.length) {
          $containingDashboards.text(` (${sharedDashboards.join(', ')})`);
        } else {
          $containingDashboards.text('');
        }
      });

      $submitButton.off('click.addLinkedVisual').on('click.addLinkedVisual', () => {
        if ($linkInput.prop('checked')) {
          dashboard.drawInWidget({
            report_id,
            force_redraw: true,
            force_reloadreport: false,
            widget_sizex: widgetXColumns,
            widget_sizey: widgetYColumns,
            shouldUpdateDatasetsSelector: true
          });
          successCb();
        } else {
          addCloneApp(workspaces);
        }

        $addLinkedModal.modal('hide');
      });

      $addLinkedModal.modal('show');
    } else {
      addCloneApp(workspaces);
    }

    function successCb() {
      dashboard.hideInstructions();
      dashboard.showLayoutRearrangeButton();
      arc.report.pagenav.needs_save(true);

      // scroll to bottom of the main div
      dashboard.scrollToBottom();
    }

    function addCloneApp(workspacesToAdd) {
      const postData = {
        add_clone_of: 'true',
        dataset_id,
        report_id,
        sheet_id,
        csrfmiddlewaretoken: arc.globals.csrf_token,
      };

      if (workspacesToAdd && workspacesToAdd.currentWorkspace()) {
        postData.workspace_id = workspacesToAdd.currentWorkspace().workspace_id;
      }

      $.ajax({
        url: arc.frontend.globals.urls.reportCloneUrl,
        data: postData,
        type: 'POST',
        beforeSend(xhr) {
          // Because of a jquery bug when handling POST requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
      .done((data) => {
        const newId = data[0].id;
        arc.frontend.vizCookie.setLatestVisuals(newId, 'visual');
        addReportToGrid(newId);
        let alertMessage = `Visual ${report_id}`;

        if (reportData && reportData.sheet_id) {
          const oldDashboardId = reportData.sheet_id;
          const url = `${arc.frontend.globals.urls.viewDashboardUrl}/${oldDashboardId}`;
          alertMessage += ` from dashboard <a href=${url}>${oldDashboardId}</a>`;
        }

        alertMessage += ' has been cloned';

        arc.report.alertMessageSmallBase({
          type: 'alert-success',
          fadeOut: true,
          alertHeader: 'Success',
          useHTML: true,
          alertMessage,
        });

        successCb();
      })
      .fail((req) => {
        displayError(req, 'Failed to create visual');
      });
    }
  };

  dashboard.createNewApp = function(dataset_id) {
    dashboard.hideInstructions();
    dashboard.showLayoutRearrangeButton();
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      create_limit: 100
    };

    $.post(`${arc.frontend.globals.urls.reportCreateUrl}/${dataset_id}`, post_data, (data) => {
      const report_id = data[0].id;

      const tableVisual = arc.visual.plugins.table();
      const vizDriver = arc.report.driver(tableVisual);
      vizDriver.restoreLimit({
        limit: 100,
      });

      // Ensure default shelves are empty so the vizDriver.save() call
      // below does not pick up any shelf elements
      $('#app-shelves').empty();

      const report_data = vizDriver.save();

      const postData = {
        dataset_id,
        report_description: '',
        report_type: 'table',
        report_data: JSON.stringify({
          report_data,
          report_type: 'table',
        }),
        report_id,
        csrfmiddlewaretoken: arc.globals.csrf_token,
      };

      if (workspaces && workspaces.currentWorkspace()) {
        postData.workspace_id = workspaces.currentWorkspace().workspace_id;
      }

      $.post(
        `${arc.frontend.globals.urls.reportUrl}/${postData.report_id}`,
        postData,
        () => {
          arc.frontend.vizCookie.setLatestVisuals(report_id, 'visual');
          addReportToGrid(report_id, true, vizDriver, () => {
            const report = reports.getReport(report_id);
            if (report) {
              if (report.viz()) {
                report.viz().showEmptyTitleSubtitle(true);
              }

              dashboard.activateReport(report);
              showVisualShelves(report);
              searchHandler.activateSearch(report);
            }
          });
        }
      )
      .fail((req) => {
        displayError(req, 'Failed to create visual');
      });
    })
    .fail((req) => {
      displayError(req, 'Failed to create visual');
    });
  };

  dashboard.cloneDB = function(ms_dashboard_id, save_cb, shared_visuals_to_clone) {
    dashboard.cloneDashboardWithUnsavedChanged(ms_dashboard_id, shared_visuals_to_clone);
  };

  // To delete current sheet
  dashboard.delete = function deleteSheet(isSheetActive = true) {
    if (isSheetActive) {
      currentSheet.visual_widgets.forEach((visualWidget) => {
        const reportId = dashboard.gridsterElementToReportId(visualWidget.id);

        dashboard.removeFromGrid(
          dashboard.reports().getReport(reportId),
          $(`li[data-arc-reportid=${reportId}]`)
        );
      });
      currentSheet.control_widgets.forEach((cw) => {
        const filterReport = dashboard.reports().getReport(cw);
        removeFilter(filterReport, $(`#${filterReport.chart_div()}`));
      });
      // Do not ask user to save unsaved changes in sheet to be deleted
      dashboard.deactivate(true);
    } else {
      currentSheet.visual_widgets.forEach((visualWidget) => {
        const reportId = dashboard.gridsterElementToReportId(visualWidget.id);
        const report = dashboard.reports().getReport(reportId);
        // report can be null in case of filters moved to grid (visuals) area
        if (report) {
          removeVisualDB(report);
        } else {
          reports.addReport(reportId, dashboard);
          const newReport = dashboard.reports().getReport(reportId);
          removeVisualDB(newReport);
        }
      });

      currentSheet.control_widgets.forEach((cw) => {
        // Filters are not in the current sheet until load
        // that is the reason we use globalFilters
        const filterReport = globalFilters.getFilterReport(cw);
        removeOrRemoveAtSave(filterReport);
      });
    }

    dashboard_data.dashboard_sheets.splice(
      dashboard_data.dashboard_sheets.indexOf(currentSheet), 1
    );

    return {
      dashboard_data,
      reports_to_delete,
      reports_to_update,
    };
  };

  // To delete dashboard - should move to ms_dashboard
  dashboard.deleteDB = function(report_id, success_cb, failure_cb) {
    if (!report_id || report_id <= 0) {
      success_cb();
      return dashboard;
    }

    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };
    $.ajax({
      url: `${arc.frontend.globals.urls.reportUrl}/${report_id}`,
      data: post_data,
      type: 'DELETE',
      beforeSend(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(() => {
      if (success_cb) {
        success_cb();
      }
    })
    .fail((req) => {
      // Notify of failure
      /* eslint-disable no-console */
      console.log(`Failed to delete app ${report_id}. ${req.responseText}`);
      console.log(req);
      /* eslint-enable no-console */
      if (failure_cb) {
        failure_cb(req);
      }
    });
  };

  dashboard.gridsterElementToReportId = function(element_id) {
    const m = element_id.match(/\D(\d+)(-a)?$/);
    if (!m) {
      return null;
    }

    const report_id = m[1];
    return Number(report_id);
  };

  function processUserSettings() {
    let needs_save = false;
    user_settings.settings().forEach((user_setting) => {
      const setting_needs_save = user_setting.process_value();
      if (setting_needs_save) {
        needs_save = true;
      }
    });
    return needs_save;
  }

  function saveUserSettings(dd) {
    dd.user_settings = {};
    user_settings.settings().forEach((user_setting) => {
      dd.user_settings[user_setting.id()] = user_setting.value();
    });
  }

  function loadUserSettings(dd) {
    if (dd.user_settings) {
      user_settings.settings().forEach((user_setting) => {
        let val = dd.user_settings[user_setting.id()];
        if (val !== undefined) {
          user_setting.value(val);
        } else {
          val = user_setting.valueBeforeRestore();
          if (val !== undefined) {
            user_setting.value(val);
          }
        }
      });
    } else {
      // HACK_HERE: For dashboards already created very early without user_settings
      // in the report_data and were designed for a max dashboard size, set the
      // dashboard width to max. Don't expect to enter this portion of the code very often.
      dashboardWidthSetting.value('');
    }
  }

  function setWidth() {
    let w = dashboard.dashboardWidth();
    if (w === undefined) {
      const windowWidth = $(window).width();
      if (arc.globals.is_edit) {
        w = windowWidth - 85;
      } else {
        w = windowWidth - 30;
      }
    }

    if (showSingleVisual) {
      $appContainer.css({
        width: `${w}px`,
        height: '100%',
        padding: '0',
      });
    } else {
      $appContainer.css({
        width: `${w}px`,
        height: '',
        padding: '',
      });

      $gridsterUl.css({
        width: '',
        height: '',
      });
    }

    $('.dash-sheets-wrapper').css('width', `${w}px`);
  }

  dashboard.loadData = function () {
    dashboard_title = dashboard_name = dashboard_data.report_title;
    dashboard_subtitle = dashboard_data.report_subtitle;

    // currentSheet should never point to anything other then corresponding obj in dashboard_sheets
    // In many places, we modify dashboard_data.dashboard_sheets via currentSheet handle
    currentSheet = dashboard_data.dashboard_sheets.filter((s) => s.sheet_id === sheet_id)[0];
    if (!currentSheet) {
      throw new Error('Could not find data for sheet to be loaded');
    }

    dashboard.setupParamsControl();
    return dashboard;
  };

  function getDashboardParamsData() {
    return params_history.current_params_optimized()[0];
  }

  dashboard.getDashboardParamsTracker = function() {
    return arc.misc.paramsFromData(getDashboardParamsData());
  };

  dashboard.reactivateReport = function(report) {
    deactivateReport();

    // Deactivation is a fadeout action and takes a little bit of time
    // Re-activate after the deactivation completes
    setTimeout(() => {
      dashboard.activateReport(report, true);
      showVisualShelves(report);
    }, 500);
  };

  dashboard.activateReport = function(report, force_redraw) {
    if (!report) {
      drawActiveVisualSettings();
      return;
    }

    const activeReport = dashboard.getActiveReport();
    if (activeReport && activeReport !== report) {
      activeReport?.viz?.().cleanupDashboardAndVisualDesignerTab?.();
    }

    // Set the specified visual to being active
    const wid = `#${widgetId(report)}`;
    const $vizContainer = $(`${wid} .arc-viz-container`);
    if (!$vizContainer.length) {
      return;
    }

    if (!force_redraw && $vizContainer.hasClass('active')) {
      return;
    }

    // Refresh currently active visual upon focus to another visual or focus loss
    arc.globals.singleVisualBuilder.refresh();

    // Remove any currently active visual
    $('.arc-viz-container.active').removeClass('active');

    $vizContainer.addClass('active');

    drawActiveVisualSettings();
    return dashboard;
  };

  let lastDrawnVisualSettings;
  function showVisualTabs() {
    $('.visual-tabs').fadeIn('slow');
  }

  function hideVisualTabs() {
    $.each(visualSideTabs, (tabName, tabData) => {
      if (tabData.$tab.hasClass('active')) {
        $('#close-dash-settings').trigger('click');

        // early exit from .each
        return false;
      }
    });

    $('.visual-tabs').hide();
  }

  function deactivateReport() {
    const report = dashboard.getActiveReport();
    if (!report) {
      return;
    }

    const wid = `#${widgetId(report)}`;
    const $vizContainer = $(`${wid} .arc-viz-container`);
    if (!$vizContainer.hasClass('active')) {
      return;
    }

    if (report.viz()) {
      report.viz().saveAwayShelfInfo();
    }

    // Refresh currently active visual upon focus to another visual or focus loss
    arc.globals.singleVisualBuilder.refresh();

    $vizContainer.removeClass('active');
    hideVisualTabs();
  }

  dashboard.getActiveReport = function() {
    const $activeVisual = $('.arc-viz-container.active');
    if ($activeVisual.length === 0) {
      return undefined;
    }

    const reportId = $activeVisual.closest('li').attr('data-arc-reportid');
    return reports.getReport(reportId);
  };

  function activateSideTab(tabName) {
    if ($('.dash-settings-panel').css('display') === 'none') {
      visualSideTabs[tabName].$display.find('> a').trigger('click');
      return;
    }

    if (!visualSideTabs[tabName].$tab.hasClass('active')) {
      visualSideTabs[tabName].$display.find('> a').trigger('click');
    }
  }

  function showVisualShelves(report) {
    const activeReport = dashboard.getActiveReport();
    if (report !== activeReport) {
      return;
    }

    const driver = report.viz();
    if (driver.isEmptyTable()) {
      activateSideTab('shelves');
    } else {
      activateSideTab(lastSelectedVisualSideTab);
    }
  }

  function setupDatasetAndDerivedData(report) {
    const driver = report.viz();
    const dataset = report.dataset();

    /*
     * Setup dataset level information in visual tabs
     */
    report.shelves_dom_id('app-shelves');
    arc.globals.reports.setReport(report, -1);
    arc.report.dataset_setup(dataset);
    const builder = arc.globals.singleVisualBuilder;
    arc.misc.displayVisualType(driver.viz_type());
    builder.setupChartSwitch();

    $('#visual-data-panel .derived-list-wrapper li.selection_element').remove();
    const derivedData = driver.derivedData();
    if (derivedData && derivedData.enabled() &&
      arc.dataconnection.supports_derived_data(dataset.dataconnection_type())) {
      const derivedDataList = derivedData.derivedDataList();
      derivedDataList.forEach((ddItem) => {
        derivedData.addDerivedDataField(ddItem, dataset);
      });
    }

    $('#app-dataset-edit').offOn(
      'click.editdataset',
      () => {
        arc.misc.openInNewWindow(`${arc.frontend.globals.urls.datasetEditUrl}/${dataset.dataset_id()}`);
      }
    );

    // Setup data sampling value + modal
    driver.setSamplingPct();
    arc.globals.singleVisualBuilder.setupSample();
  }

  function drawActiveVisualSettings() {
    if (!arc.globals.is_edit) {
      return;
    }

    const $defaultSidebar = $('#dashboard-visual-settings-sidebar');
    const $styleSidebar = $('#dashboard-visual-style-sidebar');

    const $sidebars = $('#dashboard-visual-settings-sidebar, #dashboard-visual-style-sidebar');

    /*
     * First, find and store open tabs
     */
    const tabsToOpen = new Set();
    $sidebars.find('.settings-tab-holder.open')
             .each(function() {
               tabsToOpen.add($(this).attr('data-tabname'));
             });

    if (lastDrawnVisualSettings) {
      lastDrawnVisualSettings.cleanupSettings();
      lastDrawnVisualSettings = undefined;
    }

    $sidebars.empty();
    visualSideTabs.settings.$tab.find('.sections-expand-all')
        .removeClass('open')
        .addClass('closed');

    const report = dashboard.getActiveReport();
    const reportId = report?.report_id();
    const driver = report?.viz();
    const activePickList = $('.arc-viz-container.arc-viz-type-picklist').hasClass('active');

    if (!report || !driver || driver.inModal() || activePickList) {
      hideVisualTabs();
      return;
    }

    showVisualTabs();

    setupDatasetAndDerivedData(report);

    const visualSettings = driver.user_settings();
    lastDrawnVisualSettings = visualSettings;

    visualSettings.removeChangeCbs();

    visualSettings.paint({
      default: $defaultSidebar,
      Style: $styleSidebar,
    }, {
      tabsToOpen,
    });

    /*
     * If either of the apply-to-all checkboxes are modified, make the other one match
     */
    const $applyToAllCheckboxes = $('.apply_to_all_visuals input');
    $applyToAllCheckboxes.offOn('change', function() {
      const $that = $(this);
      const checked = $that.prop('checked');

      $applyToAllCheckboxes.each(function() {
        const $this = $(this);
        if ($this[0] !== $that[0]) {
          $this.prop('checked', checked);
        }
      });
    });

    visualSettings.addChangeCb('visualSettings', (setting) => {
      // Re-ferch driver from the report object - we may have switched the visual
      // that the report uses.
      const reportDriver = report.viz();
      if (setting.id() === 'color_palette') {
        setting.saveIntoVisual();
      }

      if (setting.id() !== 'inline_css') {
        // Inline CSS marks the page as needing to be saved as soon as characters are typed in.
        // However, the change cb is made 2 seconds later and the page may no longer be dirty
        //  - so do not mark the page as dirty here.
        reportDriver.page_needs_save();
      }

      reportDriver.refresh(true);

      const applyToAll = $('.apply_to_all_visuals input').prop('checked');
      if (applyToAll) {
        dashboard.eachViz((viz) => {
          if (viz.report_id() === Number(reportId)) {
            // don't process this report
            return;
          }

          const s = viz.user_settings().find_by_id(setting.id());
          if (!s) {
            return;
          }

          s.value(setting.value());
          if (setting.id() === 'color_palette') {
            s.saveIntoVisual();
          }

          viz.page_needs_save();
          viz.refresh(true);
        });
      }
    });
  }

  function setupVisualSettings() {
    if (!arc.globals.is_edit) {
      return;
    }

    hideVisualTabs();

    $('[data-target="#dash-viz-tab"]').offOn('show.bs.tab.load-data', () => {
      paintVisualSidebarVisualsOfDataset(dashboard.currentDataSetSelection());
    });

    visualSideTabs.shelves.$display.find('> a')
        .offOn('show.bs.tab.setup', () => {
          $('.dash-settings-panel').addClass('expanded-data');
          $('.dash-main-panel').removeClass('narrow');
          $('.dash-main-panel').addClass('more-narrow');
        })
        .offOn('hide.bs.tab.setup', () => {
          $('.dash-settings-panel').removeClass('expanded-data');
          $('.dash-main-panel').addClass('narrow');
          $('.dash-main-panel').removeClass('more-narrow');
        });
  }

  const v2HiddenSettings = new Set([
    'override_visual_font_settings',
    'visual_font',
    'visual_font_size',
    'visual_bg_color',
    'title_font',
    'title_font_size',
    'visual_title_font_weight',
    'visual_title_fg',
    'visual_title_bg',
    'title_alignment',
    'subtitle_font',
    'subtitle_font_size',
    'visual_subtitle_fg',
    'visual_subtitle_bg',
    'subtitle_alignment',
    'axis_label_font_size',
    'axis_ticks_font_size',
    'label_font_size',
    'label_color',
    'trellis_font_size',
    'legend_font_size',
    'tooltip_font',
    'tooltip_font_size',
  ]);

  function handleMetadataVersion(dd) {
    metadataVersion = dd.metadataVersion;

    if (!dashboard_id && !metadataVersion) {
      metadataVersion = 2;
    }

    metadataVersion = metadataVersion || 1;

    if (metadataVersion > 1) {
      user_settings.settings().forEach((user_setting) => {
        if (v2HiddenSettings.has(user_setting.id())) {
          user_setting.hide();
        }
      });
    }
  }

  function setupHighlighter() {
    highlightedVisual = arc.globals.urlparams.highlighted_visual;

    if (highlightedVisual) {
      try {
        highlightedVisual = Number(highlightedVisual);
      } catch (err) {
        highlightedVisual = undefined;
      }
    }

    // Highlighting is a one-time function - forget the global setting
    // so that we don't use it again after first use
    arc.globals.urlparams.highlight = undefined;
  }

  function setupResizer() {
    arc.visual.dashboardResizer()
       .dashboard(dashboard)
       .setup();
  }

  let checkQueriesDoneTimeout;
  let checkStartTime;
  let isQueryWarningActive = false;
  let maxQueryLoading;
  let preventMoreCheck = false;

  const alertHeader = 'Data Load';
  const alertId = 'data-load-warning-alert';

  function checkQueriesComplete() {
    checkQueriesDoneTimeout = undefined;

    if (!data_connections.length) {
      checkQueriesDoneTimeout = setTimeout(checkQueriesComplete, 1000);

      return;
    }

    if (!maxQueryLoading) {
      const defaultTime = arc.globals.dashboard_loading_warning_seconds_default;
      const activeConnectionIDs = dashboard.active_connection_ids();
      const activeConnections = data_connections
        .filter((c) => activeConnectionIDs.includes(c.dataconnection_id()));

      activeConnections.forEach((conn) => {
        const { PARAMS: info } = JSON.parse(conn.dataconnection_info());
        const warningTime = info.QUERY_LOADING_WARNING_SECONDS ?? defaultTime;

        maxQueryLoading = Math.max((maxQueryLoading ?? 0), warningTime);
      });
    }

    if (arc.counters.data.dataapi_query_start > arc.counters.data.dataapi_query_end &&
      !preventMoreCheck) {
      if (!isQueryWarningActive &&
          ((new Date() - checkStartTime) > (maxQueryLoading * 1000))) {
            isQueryWarningActive = true;

            arc.report.alertMessageSmallBase({
              id: alertId,
              stackMessages: false,
              useHTML: true,
              alertHeader,
              onClose: () => {
                isQueryWarningActive = false;
                preventMoreCheck = true;
              },
              type: 'alert-warning',
              alertMessage: 'This dashboard is still loading data. ' +
                  '<a class="show-current-load">Click here</a> for details.',
            });

            $('body').offOn('click.showload', '.show-current-load', () => {
              arc.report.alertMessageSmallHide(alertId);
              arc.perf.PAGEPERF.modal('nav-tab-current-load');
              isQueryWarningActive = false;
              preventMoreCheck = true;
            });

            arc.misc.afterCountersEqual({
              counter1: 'dataapi_query_start',
              counter2: 'dataapi_query_end',
              cb: () => {
                arc.report.alertMessageSmallHide(alertId);
                isQueryWarningActive = false;
              },
            });
      }

      checkQueriesDoneTimeout = setTimeout(checkQueriesComplete, 1000);
    }
  }

  dashboard.queryStarted = function() {
    const shouldRunQueryCheck = checkQueriesDoneTimeout === undefined &&
      (arc.globals.dashboard_loading_message_in_viewmode ||
        arc.globals.is_edit);

    if (shouldRunQueryCheck) {
      checkStartTime = new Date();
      checkQueriesDoneTimeout = setTimeout(checkQueriesComplete, 1000);
    }
  };

  dashboard.load = function() {
    arc.errors.clearAllErrors();
    hideTooltip();
    setupVisualSettings();

    setupResizer();
    setupHighlighter();

    $('.control-instructions').show();
    dashboard.loadData();

    const dd = dashboard_data;
    const jdd = currentSheet.visual_widgets;

    if (dashboard_mode === 'template') {
      $('ul#report-menu-list').hide();
      $('#template-btns').show();
      arc.report.loadTemplate();
      arc.report.loadTemplate.setupDrawBtns(dashboard_id);
    }

    if (dashboard_id || dd.user_settings) {
      loadUserSettings(dd);
    }

    handleMetadataVersion(dd);

    paintUserSettings();

    if (jdd.length === 0) {
      dashboard.showInstructions();
      dashboard.hideLayoutRearrangeButton();
    }

    if (!dd.numColumns) {
      // Upgrade from older versions
      const oldNumColumns = 16;
      const scaleFactor = numColumns / oldNumColumns;
      const scale = function(x) {
        // gridster column indexes are not 0-based. Do math instead.
        return ((x - 1) * scaleFactor) + 1;
      };

      $.each(jdd, (idx, element) => {
        element.size_x *= scaleFactor;
        element.size_y *= scaleFactor;
        element.col = scale(element.col);
        element.row = scale(element.row);
      });
    } else {
      numColumns = dd.numColumns;
    }

    handleAutoRefresh();
    handleDashboardWidth();
    handleDashboardColors();
    handleDashboardMargins();
    handleCaseSensitive();

    const filtersH = {};
    if (globalFilters) {
      globalFilters.filters().forEach((filterId) => {
        filtersH[filterId] = true;
        dashboard.addFilter(filterId);
      });
    }

    const dashboardParams = dashboard.getDashboardParamsTracker();
    const control_widgets = dd.global_control_widgets.concat(currentSheet.control_widgets) || [];

    allFilterCount = control_widgets.length;

    control_widgets.forEach((report_id) => {
      report_id = Number(report_id);
      if (report_id in filtersH) {
        return;
      }

      const report = reports.getReport(report_id);
      setParams(report, dashboardParams);

      dashboard.addFilter(report_id);
    });

    setWidth();
    dashboard.setSize();

    if ('show_borders' in dd) {
      // This is for backward compatibility with the previous implementation
      // of settings in dashboards. Added in Nov 2015 v2.4
      if (dd.show_borders === true) {
        bordersAroundVisuals.value('true');
      } else {
        bordersAroundVisuals.value('false');
      }
    }

    if ('controls_orientation' in dd) {
      // This is for backward compatibility with the previous implementation
      // of settings in dashboards. Added in Nov 2015 v2.4
      if (dd.controls_orientation === 'top') {
        filtersOnLeft.value('false');
      } else {
        filtersOnLeft.value('true');
      }
    }

    if (arc.frontend.globals.isMobileView) {
      jdd.sort((a, b) => {
        if (a.row < b.row) {
          return -1;
        }

        if (a.row > b.row) {
          return 1;
        }

        if (a.col < b.col) {
          return -1;
        }

        if (a.col > b.col) {
          return 1;
        }

        return 0;
      });

      $.each(jdd, (idx, element) => {
        element.size_x = 1;
        element.size_y = 1;
        element.col = 1;
        element.row = idx + 1;
      });
    }

    dashboard.hideInstructions();
    dashboard.showLayoutRearrangeButton();
    dashboard.loadTitleSubtitle();
    manualRefreshVisibility();

    click_behavior = currentSheet.behaviors || {};

    paintCSSClass();
    paintInlineCSS();

    widget_base_dim_px = arc.globals.ms_dashboard.weight_base_dimensions();
    gridster.resize_widget_dimensions({
      widget_base_dimensions: [widget_base_dim_px, widget_base_dim_px]
    });

    if (!jdd.length) {
      setWidth();
      if (arc.globals.is_edit) {
        $('.control-instructions').show();
        dashboard.showInstructions();
        dashboard.hideLayoutRearrangeButton();
      }

      arc.misc.updateDivForPerms(
        ['ds_appedit'],
        'dataset',
        $('.subnav-report'),
        'forceVisible'
      );
    }

    updateDatasetsShortcuts();

    $.each(jdd, (_, element) => {
      const report_id = Number(dashboard.gridsterElementToReportId(element.id));
      dashboard.drawInWidget({
        report_id,
        force_redraw: true,
        force_reloadreport: true,
        widget_sizex: element.size_x,
        widget_sizey: element.size_y,
        widget_col: element.col,
        widget_row: element.row,
        params_data: dashboardParams.getkvsList()
      });
    });

    dashboard_filter_controls.load();
    dashboard_filter_controls.setTimeControlSettings(dd.time_control_settings);

    setupDashboardInfoHandler();

    return dashboard;
  };

  dashboard.getCss = function() {
    const cssl = [];

    const dashboardFg = dashboardFgColor.value().trim();
    if (dashboardFg !== '') {
      cssl.push(`${'.arc-app-container, .arc-app-container h1, ' +
          '.arc-app-container h2, .arc-app-container h3, ' +
          '.arc-app-container .table { color: '}${dashboardFg};}`);
      cssl.push(`.arc-app-container text { fill: ${dashboardFg};}`);
    }

    if (!arc.globals.is_edit) {
      if (dashboardHideTitle.isChecked) {
        cssl.push('.arc-app-container .arc-dashboard-title-container {display: none;}');
      }
    }

    const dashboardBg = dashboardBgColor.value().trim();
    if (dashboardBg !== '') {
      cssl.push(`.arc-app-container { background-color: ${dashboardBg};}`);
    }

    const pageBg = pageBgColor.value().trim();
    if (pageBg !== '') {
      cssl.push(`${'.dash-main-panel, .builder-container, .builder-container-app ' +
          '{ background-color: '}${pageBg};}`);
    }

    const dashboardTitleFontWeightValue = dashboardTitleFontWeight.value();
    if (dashboardTitleFontWeightValue !== 'default') {
      cssl.push(`.arc-dashboard-title-container { font-weight: ${
          dashboardTitleFontWeightValue};}`);
    }

    const visualBg = visualBgColor.value().trim();
    if (visualBg !== '') {
      cssl.push(`.gridster .arc-viz-container { background-color: ${visualBg};}`);
    }

    const visualBorderClr = visualBorderColor.value().trim();
    if (visualBorderClr !== '') {
      cssl.push(`.gridster .arc-viz-container { border-color: ${visualBorderClr};}`);
    }

    const visualBorderRds = visualBorderRadius.value().trim();
    if (visualBorderRds !== '') {
      cssl.push(`.gridster .arc-viz-container { border-radius: ${
           visualBorderRds}px !important;}`);
    }

    const appTitleBg = appTitleBgColor.value().trim();
    if (appTitleBg !== '') {
      cssl.push(`.arc-app-container .arc-dashboard-title-container { background-color: ${appTitleBg};}`);
    }

    let appTitleFg = appTitleFgColor.value().trim();
    if (appTitleFg === '') {
      appTitleFg = dashboardFg;
    }

    if (appTitleFg !== '') {
      cssl.push(`.arc-dashboard-title { color: ${appTitleFg};}`);
    }

    const dashboardSubtitleBg = dashboardSubtitleBgColor.value().trim();
    if (dashboardSubtitleBg !== '') {
      cssl.push(`.arc-dashboard-subtitle-container { background-color: ${dashboardSubtitleBg};}`);
    }

    let dashboardSubtitleFg = dashboardSubtitleFgColor.value().trim();
    if (dashboardSubtitleFg === '') {
      dashboardSubtitleFg = dashboardFg;
    }

    if (dashboardSubtitleFg !== '') {
      cssl.push(`.arc-dashboard-subtitle { color: ${dashboardSubtitleFg};}`);
    }

    const visualTitleBg = visualTitleBgColor.value().trim();
    if (visualTitleBg !== '') {
      cssl.push(`.arc-app-container .arc-viz-title-container { background-color: ${
          visualTitleBg};}`);
    }

    let visualTitleFg = visualTitleFgColor.value().trim();
    if (visualTitleFg === '') {
      visualTitleFg = dashboardFg;
    }

    if (visualTitleFg !== '') {
      cssl.push(`.arc-app-container .arc-viz-title-container { color: ${visualTitleFg};}`);
    }

    const visualSubtitleBg = visualSubtitleBgColor.value().trim();
    if (visualSubtitleBg !== '') {
      cssl.push(`.arc-app-container .arc-viz-subtitle-container { background-color: ${
          visualSubtitleBg};}`);
    }

    let visualSubtitleFg = visualSubtitleFgColor.value().trim();
    if (visualSubtitleFg === '') {
      visualSubtitleFg = dashboardFg;
    }

    if (visualSubtitleFg !== '') {
      cssl.push(`.arc-app-container .arc-viz-subtitle-container { color: ${
          visualSubtitleFg};}`);
    }

    let filterTitle = filterTitleColor.value().trim();
    if (filterTitle === '') {
      filterTitle = dashboardFg;
    }

    if (filterTitle !== '') {
      cssl.push(`.arc-app-container .selected-control-title { color: ${filterTitle};}`);
    }

    filtersOnLeft.addDependents([filterInlineLabels], ['true'], ['false']);
    filterInlineLabels.addDependents([filterLabelLine], ['true'], ['false']);

    if (filterInlineLabels.isChecked) {
      $('.control-container').addClass('inline-filters');
      $('.filter_label_line input').prop('checked', '');
      filterLabelLine.disable();
    } else {
      $('.control-container').removeClass('inline-filters');
      filterLabelLine.enable();
    }

    if (filterLabelLine.isChecked) {
      $('.control-container').removeClass('noline-filters');
    } else {
      $('.control-container').addClass('noline-filters');
      $('.filter_label_line input').prop('checked', '');
      filterLabelLine.enable();
    }

    if (filtersOnLeft.isChecked) {
      $('.filter_inline_labels input').prop('checked', '');
      $('.control-container').removeClass('inline-filters');
      filterInlineLabels.disable();
      filterLabelLine.enable();
    } else {
      filterInlineLabels.enable();
    }

    if (bordersAroundFilters.isChecked !== true) {
      cssl.push('.arc-app-container .selected-control { border: none;}');
    }

    let filterSelection = filterSelectionColor.value().trim();
    if (filterSelection === '') {
      filterSelection = dashboardFg;
    }

    if (filterSelection !== '') {
      cssl.push(`.arc-app-container .selected-control a { color: ${filterSelection};}`);
      cssl.push(`.inline-filters .chosen-container-single .chosen-single span { color: ${
      filterSelection};}`);
    }

    let filterBg = filterBgColor.value().trim();
    if (filterBg === '') {
      filterBg = dashboardBg;
    }

    if (filterBg !== '') {
      cssl.push(`.selected-control { background-color: ${filterBg};}`);
    }

    let filterContainerBg = filterContainerBgColor.value().trim();
    if (filterContainerBg === '') {
      filterContainerBg = dashboardBg;
    }

    if (filterContainerBg !== '') {
      cssl.push(`.control-container { background-color: ${filterContainerBg};}`);
    }

    const hm = horizontalMarginBetweenVisuals.value().trim();
    const vm = verticalMarginBetweenVisuals.value().trim();
    cssl.push(`.gridster .arc-viz-container { margin: ${vm}px ${hm}px;}`);

    return cssl.join('\n');
  };

  function setupColorStyles() {
    paintInlineCSS();
  }

  function setupDashboardMargins() {
    redrawAll();
  }

  function setupFontStyles() {
    const $title = $('.arc-dashboard-title-container');
    const $subtitle = $('.arc-dashboard-subtitle-container');
    const $filter = $('.control-container');

    const titleFont = arc.settings.fonts.fontSetting(user_settings,
        'dashboard_title_font');
    arc.misc.setFont($title, 'data-fontclass', titleFont);

    const $titleContainer = $title.closest('.arc-dashboard-title-container');
    let sz = arc.settings.fonts.fontSizeSetting(user_settings, 'dashboard_title_font_size');
    if (sz === undefined) {
      $title.css('font-size', '');
      $titleContainer.css('padding-bottom', '');
    } else {
      $title.css('font-size', `${sz}px`);
      if (sz > 30) {
        const padding = 10 + (sz - 30);
        $titleContainer.css('padding-bottom', `${padding}px`);
      } else {
        $titleContainer.css('padding-bottom', '');
      }
    }

    let sa = arc.settings.fonts.fontAlignmentSetting(user_settings, 'dashboard_title_alignment');
    if (sa === undefined) {
      sa = null;
    }

    $titleContainer.css('text-align', sa);

    const subtitleFont = arc.settings.fonts.fontSetting(user_settings,
        'dashboard_subtitle_font');
    arc.misc.setFont($subtitle, 'data-fontclass', subtitleFont);

    sz = arc.settings.fonts.fontSizeSetting(user_settings, 'dashboard_subtitle_font_size');
    if (sz === undefined) {
      $subtitle.css('font-size', '');
    } else {
      $subtitle.css('font-size', `${sz}px`);
    }

    const filterFont = arc.settings.fonts.fontSetting(user_settings,
        'filter_font');
    arc.misc.setFont($filter.find('.selected-control'), 'data-fontclass', filterFont);

    const fsz = arc.settings.fonts.fontSizeSetting(user_settings, 'filter_font_size');
    if (fsz === undefined) {
      $filter.css('font-size', '');
    } else {
      $filter.css('font-size', `${fsz}px`);
    }

    const $subtitleContainer = $subtitle.closest('.arc-dashboard-subtitle-container');
    sa = arc.settings.fonts.fontAlignmentSetting(user_settings, 'dashboard_subtitle_alignment');
    if (sa === undefined) {
      sa = null;
    }

    $subtitleContainer.css('text-align', sa);

    dashboard.eachViz((viz) => {
      viz.setFontStyles();
    });
  }

  function handleDashboardColors() {
    [
      'dashboard_hide_title',
      'dashboard_bg_color', 'filter_bg_color', 'filter_container_bg_color',
      'dashboard_fg_color', 'filter_title_color', 'filter_selection_color',
      'page_bg_color', 'borders_around_filters',
      'filter_inline_labels', 'filter_label_line',
      'visual_bg_color', 'visual_border_color', 'visual_title_bg', 'visual_title_fg',
      'visual_subtitle_fg', 'visual_subtitle_bg',
      'app_title_bg', 'app_title_fg',
      'dashboard_subtitle_fg_color', 'dashboard_subtitle_bg_color',
      'dashboard_title_font_weight',
    ].forEach((settingId) => {
      const setting = user_settings.find_by_id(settingId);
      if (!setting) {
        return;
      }

      setting.changeCb('dashboard', () => {
        arc.report.pagenav.needs_save(true);
        setupColorStyles();

        const hasVisualBgColor = Boolean(user_settings.find_by_id('visual_bg_color').value());

        if (settingId === 'visual_bg_color' || (settingId === 'dashboard_bg_color' && !hasVisualBgColor)) {
          redrawDelayed({
            noReloadData: true,
          });
        }
      });
    });
  }

  function handleCaseSensitive() {
    caseSensitive.changeCb('dashboard', () => {
      arc.globals.parameters_case_sensitive = caseSensitive.isChecked;
    });

    arc.globals.parameters_case_sensitive = caseSensitive.isChecked;
  }

  function handleDashboardMargins() {
    [
      'horizontal_margin_between_visuals', 'vertical_margin_between_visuals',
    ].forEach((setting) => {
      const s = user_settings.find_by_id(setting);
      if (!s) {
        return;
      }

      s.changeCb('dashboard', () => {
        arc.report.pagenav.needs_save(true);
        setupDashboardMargins();
      });
    });
  }

  function handleFontChanges() {
    [
      'title_font', 'subtitle_font', 'visual_font',
      'filter_font', 'filter_font_size',
      'title_font_size', 'subtitle_font_size',
      'visual_title_font_weight',
      'title_alignment', 'subtitle_alignment',
      'tooltip_font', 'tooltip_font_size',
      'dashboard_title_font', 'dashboard_subtitle_font',
      'dashboard_title_font_size', 'dashboard_subtitle_font_size',
      'dashboard_title_alignment', 'dashboard_subtitle_alignment',
      'visual_font_size', 'axis_label_font_size', 'axis_ticks_font_size',
      'label_font_size', 'trellis_font_size', 'legend_font_size', 'label_color',
    ].forEach((setting) => {
      const s = user_settings.find_by_id(setting);
      if (!s) {
        return;
      }

      s.addChangeCb('dashboard', () => {
        arc.report.pagenav.needs_save(true);

        if ((setting === 'visual_font_size') ||
            (setting === 'axis_label_font_size') ||
            (setting === 'axis_ticks_font_size') ||
            (setting === 'label_font_size') ||
            (setting === 'trellis_font_size') ||
            (setting === 'legend_font_size') ||
            (setting === 'label_color')) {
          redrawDelayed({
            noReloadData: true,
          });

          return;
        }

        setupFontStyles();
      });
    });
  }

  dashboard.loadTitleSubtitle = function loadTitleSubtitle() {
    let title = dashboard_title;
        let subtitle = dashboard_subtitle;

    if (title === undefined) {
      if (dashboard_data) {
        title = dashboard_data.report_title;
      }
    }

    if (subtitle === undefined) {
      if (dashboard_data) {
        subtitle = dashboard_data.report_subtitle;
      }
    }

    setupFontStyles();
    setupColorStyles();

    const d3Title = d3.select('#dashboard-title');
    const d3Subtitle = d3.select('#dashboard-subtitle');
    const argsl = [title, subtitle];
    const d3AppContainer = d3.select('.arc-app-container');

    if (dashboard.displaydb()) {
      title = arc.globals.params.substitute(title, argsl);
      subtitle = arc.globals.params.substitute(subtitle, argsl);

      const paramsData = getDashboardParamsData();
      const params = arc.misc.paramsFromData(paramsData);
      title = params.substitute(title, argsl);
      subtitle = params.substitute(subtitle, argsl);
    }

    d3Title.text(title);
    d3Subtitle.text(subtitle);

    if (d3AppContainer.classed('view-mode')) {
      d3Title.attr('contenteditable', 'false');
    }

    // If we are not in an appgroup, set the title of the page
    if (!arc.globals.appgroup && title) {
      document.title = title;
    }
  };

  $('.settings-reset').on('click', () => {
    user_settings.resetDashboardStyleValues();
  });

  function getNotYetSavedReportOfCurrentSheet() {
    const modifiedReports = getModifiedReports();
    const modifiedReportsData = modifiedReports.map((report) => {
      const driver = report.viz();
      const viz_data = driver.save();
      const report_data = JSON.stringify({
        report_type: driver.viz_type(),
        report_data: viz_data
      });

      return {
        report_data,
        dataset_id: driver.dataset_id(),
        report_type: driver.viz_type(),
        report_id: driver.report_id(),
        csrfmiddlewaretoken: arc.globals.csrf_token,
        report_description: report.report_description(),
        shared_visual_dashboards: report.shared_visual_dashboards(),
        report_search_text: report.reportSearchText(),
        workspace_id: report.workspace() && report.workspace().workspace_id,
      };
    });

    return modifiedReportsData;
  }

  dashboard.cloneDashboardWithUnsavedChanged = function (dashboardId, shared_visuals_to_clone) {
    const post_data = get_post_data_for_save_or_clone(dashboardId, true, shared_visuals_to_clone);
    post_data.csrfmiddlewaretoken = arc.globals.csrf_token;

    jQuery.post(arc.frontend.globals.urls.dashboardSaveUrl, post_data)
    .then((data) => {
      arc.report.alertMessageSmall('Successfully cloned dashboard', 'alert-success');
      const url = `${arc.frontend.globals.urls.staticUrl}/../apps/builder/${data.id}`;
      arc.misc.openInNewWindow(url);
    })
    .fail((req) => {
      displayError(req, 'Failed to clone dashboard / visual');
    });
  };

  // This is to save default filter values, pivot values, and exploration visuals
  dashboard.save_report = function(report, cb) {
    const driver = report.viz();
    const viz_data = driver.save();
    const report_data = JSON.stringify({
      report_type: driver.viz_type(),
      report_data: viz_data
    });

    const post_data = {
      report_data,
      dataset_id: driver.dataset_id(),
      report_type: driver.viz_type(),
      report_id: driver.report_id(),
      csrfmiddlewaretoken: arc.globals.csrf_token,
      report_description: report.report_description(),
      shared_visual_dashboards: report.shared_visual_dashboards(),
      report_search_text: report.reportSearchText(),
      workspace_id: report.workspace() && report.workspace().workspace_id,
    };

    $.post(`${arc.frontend.globals.urls.reportUrl}/${post_data.report_id}`, post_data, () => {
      report.viz().needs_save(false);
      arc.frontend.vizCookie.setLatestVisuals(post_data.report_id, 'visual');
      cb();
    })
    .fail((req) => {
      displayError(req, 'Failed to save report');
      cb('Failed to save report');
    });

    return dashboard;
  };

  dashboard.patchDatasetRefOfReports = function (patchData) {
    // It should be on the report object but similar methods are created here,
    // so I will continue this logic not to cause confusion
    // Used only by ML Explore style workflow
    const data = JSON.stringify(patchData);
    return arc.frontend.vizHttp.patch({
      url: `${arc.frontend.globals.urls.reportUrl}`,
      data,
      headers: {
        'X-CSRFTOKEN': arc.globals.csrf_token,
      },
      isOldEndpoint: false,
    }).catch((error) => {
      const alertMessage = 'Failed to update reports';
      displayError(error, alertMessage);
      throw new Error(
        `${alertMessage} (error: ${error.responseText ?? error
        })`
      );
    });
  };

  return dashboard;
};

arc.setup = arc.setup || {};

arc.setup.dashboardEdit = function(app_id) {
  // class dashboard settings panel
  $('#close-dash-settings').offOn('click.closeDashSettings', () => {
    $('.dash-settings-panel').fadeOut();
    $('.builder-container .side-tabs li a').removeClass('active');
    $('.builder-container .dash-main-panel').removeClass('narrow more-narrow');
    $('#dash-visual-data-tab').removeClass('show active in');
    $('.dash-settings-panel').removeClass('expanded-data');
  });

  // reopen dash settings panel
  $('.side-tabs li a').offOn('click.reOpenDashSettings', function(e) {
    const $dash = $('.dash-settings-panel');
    $('.builder-container .dash-main-panel').addClass('narrow');
    if ($(this).closest('li a').hasClass('active')) {
      $('.builder-container .side-tabs li a').removeClass('active');
      $('.builder-container .dash-main-panel').removeClass('narrow more-narrow');
      $('.dash-settings-panel').removeClass('expanded-data');
      e.stopPropagation();
      $dash.hide();
    } else {
      // don't allow style resetting on other tabs yet
      const resetStyleIcon = $('#reset-style-settings');
      if (this.dataset.target === '#dash-basic-styling-tab') {
        resetStyleIcon.show();
      } else {
        resetStyleIcon.hide();
      }

      $dash.show();
    }
  });

  if (!app_id) {
    // New dashboard
    $('.side-tabs-panel .side-tabs li a:first').trigger('click');
  }

  // expand/collapse dashboard settings panel
  $('#expand-dash-settings').offOn('click.expandDashSettings', function() {
    $('.dash-settings-panel').toggleClass('expanded');
    $(this).toggleClass('fa-expand fa-compress');
    $(this).attr('data-original-title', function(i, attr) {
      return attr === 'click to expand' ? 'click to collapse' : 'click to expand';
    });
  });

  // add toggle General section in Settings tab on click
  $('.settings-tab-holder[data-tabname="General"]').on('click', function() {
    $(this).toggleClass('open closed');
  });
};
}());
