window.arc = window.arc || {};

(function() {
  'use strict';

  arc.visual = arc.visual || {};
  arc.visual.user_settings = arc.visual.user_settings || {};

  arc.report = arc.report || {};

  arc.ms_dashboard = function () {
    let globalFilters;
    let dashboard_id;
    let dashboard_mode;
    let dashboard_data = {};
    let originalVisuals = new Set();
    let reports_to_delete = [];
    let reports_to_update = [];
    let autoRefreshSeconds;
    let dashboard_filter_controls;
    let oldSheetTitle; // Used to know when renaming a sheet if title has been changed
    let gridster;
    let dashboard_perm;

    // Workspace
    const workspaces = arc.workspace.workspaces();

    // for hiding/showing tabs
    const $builderContainer = $('.builder-container.view-mode, .builder-container-app');

    // Items needed to make a few functions from arc_dashboards.js work
    let data_connections = [];
    let data_connections_hash = {};
    let dataset_hash = {};
    const params_history = arc.report.params_history();
    let reports_all_data = [];
    const reports_hash = {};
    let widget_base_dim_px;

    // new items defined for multi sheet management
    const allSheets = [];
    const cbOnSheetActivation = [];
    let currentlyActiveSheet;
    let initialSheetId;

    // for toggle between edit and view modes
    const $toggleViewButton = $('#report-menu-list a.editapp-link, #report-menu-list a.view-link');

    var msdb = function () { return 'arc.report.ms_dashboard'; };

    msdb.globalFilters = function (_) {
      if (!arguments.length) return globalFilters;
      globalFilters = _;
      return msdb;
    };

    msdb.loadTitleSubtitle = function () {
      if (currentlyActiveSheet) {
        return currentlyActiveSheet.loadTitleSubtitle();
      }
    };

    msdb.dashboard_id = function (_) {
      if (!arguments.length) return dashboard_id;
      dashboard_id = _;
      return msdb;
    };

    msdb.initialSheet = function (_) {
      if (!arguments.length) return initialSheetId;
      initialSheetId = _;
      return msdb;
    };

    msdb.dashboard_perm = function (_) {
      if (!arguments.length) return dashboard_perm || [];
      dashboard_perm = _;
      return msdb;
    };

    msdb.autoRefreshSeconds = function (_) {
      if (!arguments.length) return autoRefreshSeconds;
      autoRefreshSeconds = _;
      return msdb;
    };

    msdb.dashboard_filter_controls = function (_) {
      if (!arguments.length) return dashboard_filter_controls;
      dashboard_filter_controls = _;
      return msdb;
    };

    msdb.params_history = function () {
      if (currentlyActiveSheet) {
        return currentlyActiveSheet.params_history();
      }

      return params_history;
    };

    msdb.registerParamDistributionCb = function(config={}) {
      if (!msdb.beforeParamDistributionCbs) {
        msdb.beforeParamDistributionCbs = {};
      }

      msdb.beforeParamDistributionCbs[config.callbackName] = config.callbackFn;
    };

    msdb.getCurrentlyActiveSheet = function() {
      return currentlyActiveSheet;
    };

    msdb.createNewApp = function (datasetId) {
      if (currentlyActiveSheet) {
        return currentlyActiveSheet.createNewApp(datasetId);
      }

      let sheet = createNewSheet();
      activate(sheet);
      return sheet.createNewApp(datasetId);
    };

    msdb.createNewVisual = function() {
      var dataset_id = $('.datasets-sidebar-sel option:selected').val();
      if (dataset_id) {
        msdb.createNewApp(dataset_id);
        currentlyActiveSheet.scrollToBottom();
      }

      return msdb;
    };

    function createNewSheetObject(sheetData) {
      if (!sheetData) {
        return;
      }

      let sheet = arc.dashboard().sheet_id(sheetData.sheet_id)
                                 .dashboard_id(dashboard_id)
                                 .globalFilters(globalFilters)
                                 .dashboard_mode(dashboard_mode)
                                 .order(sheetData.order)
                                 .dashboard_data(dashboard_data)
                                 .originalVisuals(originalVisuals)
                                 .reports_to_update(reports_to_update)
                                 .reports_to_delete(reports_to_delete)
                                 .loadData()
                                 .displaydb(!arc.globals.is_edit)
                                 .workspaces(workspaces)
                                 .data_connections(data_connections)
                                 .data_connections_hash(data_connections_hash)
                                 .dataset_hash(dataset_hash)
                                 .reports_all_data(reports_all_data)
                                 .reports_hash(reports_hash)
                                 .reports(arc.globals.reports)
                                 .widget_base_dim_px(widget_base_dim_px)
                                 .onSaveDashboardCb(function () {});

      if (gridster) {
        sheet.gridster(gridster);
      } else {
        msdb.setupGridster();
        sheet.gridster(gridster);
      }

      allSheets.push(sheet);
      createNewSheetHandle(sheet);
      return sheet;
    }

    // we expect config to have sheet_id, order
    // This is supposed to update allSheets after creating object
    // createNewSheet and deleteExistingSheet are the only places where allSheets is modified
    function createNewSheet(sheetData) {
      if (!sheetData) {
        // We are creating a new sheet by hitting +
        sheetData = {
          order: (getDashboardMaxOrderId() + 1),
          sheet_id: (getDashboardMaxSheetId() + 1),
          sheet_handle_title: 'Sheet ' + (getDashboardMaxSheetId() + 1),
          sheet_title: dashboard_data.report_title,
          behaviors: {},
          visual_widgets: [],
          control_widgets: [],
        };
      }

      dashboard_data.dashboard_sheets.push(sheetData);
      return createNewSheetObject(sheetData);
    }

    msdb.onSheetActivation = function (cbName, cb) {
      if (cb) {
        cbOnSheetActivation[cbName] = cb;
      } else {
        delete cbOnSheetActivation[cbName];
      }

      return msdb;
    };

    msdb.loadById = function(dashboardId) {
      let suffix = '';

      if (arc.globals.appgroup) {
        suffix = `&appgroup=${arc.globals.appgroup.appgroup_id()}`;
      }

      arc.staticfetcher.fetch([
        {
          asset: `${arc.frontend.globals.urls.reportUrl}/${dashboardId}?assets=true${suffix}`,
          postprocess: d3.json,
        },
      ], (error, reportData) => {
        if (error || reportData.length === 0) {
          const errorMessage = error && error.responseText ? error.responseText : error;
          arc.report.alertMessage('Error loading app', errorMessage, { alert_report_id: dashboardId });
          return;
        }

        const assets = reportData[0].assets || {};
        arc.staticfetcher.populate(assets);

        const reportsInDashboard = Object.values(assets).flat().filter((object) => 'report_type' in object);
        for (const report of reportsInDashboard) {
          reports_hash[report.id] = report;
        }

        if (arc.misc.hasPerm('ds_appedit', reportData[0].perm)) {
          arc.misc.updateDivForPerms(['ds_appedit'], 'dataset', $('.subnav-report'));
        }

        msdb.load(dashboardId, reportData[0]);
      });
    };

    // Find dataset of first top left visual of first non-empty sheet
    // TODO: Currently not allowing empty sheets
    msdb.getPostDataForSave = function () {
      let primarySheet = getPrimarySheet();
      let primarySheetData;
      let datasetId;
      if (primarySheet) {
        primarySheetData = dashboard_data.dashboard_sheets.map(
            ds =>
              ds.sheet_id === primarySheet.sheet_id()
        )[0];
      }

      let primaryVisualData = getPrimaryVisualInSheetData(primarySheetData);

      // TODO: check for all reports instead of currentlyActiveSheet.reports()
      let report = currentlyActiveSheet.reports().filter(
          r => r.report_id() === primaryVisualData.id)[0];
      if (report) {
        // TODO: If report is not loaded, load it to identify dataset_id()
        datasetId = report.dataset_id();
      }

      return {
        dashboardDatasetId: datasetId,
      };
    };

    function getPrimaryVisualInSheetData(sheetData) {
      if (!sheetData || !sheetData.visual_widgets) {
        return;
      }

      let jdd = sheetData.visual_widgets;
      let topleft_report_idx = 0;
      let topleft_col = jdd[0].col;
      let topleft_row = jdd[0].row;

      $.each(jdd, function (idx, element) {
        if (element.row < topleft_row) {
          topleft_report_idx = idx;
          topleft_row = element.row;
          topleft_col = element.col;
        } else if (element.row === topleft_row) {
          if (element.col < topleft_col) {
            topleft_report_idx = idx;
            topleft_row = element.row;
            topleft_col = element.col;
          }
        }
      });

      return sheetData.visual_widgets[topleft_report_idx];
    }

    function getDashboardDatasetId(cb) {
      // fetch dataset id from first non empty sheet
      // if not found, let currentlyActiveSheet identify dataset
      // or else ask to add visual before saving
      let dashboardDatasetId;
      if (currentlyActiveSheet.order() !== 1) {
        allSheets.sort(sortAllSheets);

        // To maintain allSheets and dashboard_data.dashboard_sheets in same order
        dashboard_data.dashboard_sheets.sort(sortDashboardSheets);

        let firstSheetWithVisuals;
        allSheets.some((sheet) => {
          const sheetId = sheet.sheet_id();
          if (arc.globals.reports.numberReports(sheetId)) {
            firstSheetWithVisuals = sheet;
            return true;
          }

          return false;
        });

        if (firstSheetWithVisuals) {
          // TODO: Fix identifying primary report
          let allSheetReports = arc.globals.reports
                                   .getAllSheetReports(firstSheetWithVisuals.sheet_id());

          // Object.keys convert all numeric keys to strings as well
          let ds = allSheetReports[
            Number(Object.keys(allSheetReports)[0])
          ].dataset();
          if (ds) {
            dashboardDatasetId = ds.dataset_id();
          } else {
            // We need to first load the driving visual of first non empty sheet to
            // identify its dataset id and set that as dashboard's dataset id.
            let report = arc.globals.reports.getReport(Number(Object.keys(allSheetReports)[0]));
            firstSheetWithVisuals.fetchReport(report, function () {
              dashboardDatasetId = report.dataset_id();
              if (cb) {
                cb(dashboardDatasetId);
              }

            });

            return;
          }
        }
      }

      if (cb) {
        cb(dashboardDatasetId);
      }
    }

    msdb.save = function (dashboard_id, save_cb, error_cb) {
      // TODO: When saving this sheet, delete dashboard_data.click_behaviors;
      getDashboardDatasetId(function (dashboardDatasetId) {
        currentlyActiveSheet.save(
          dashboard_id,
          currentlyActiveSheet.sheet_id(),
          dashboardDatasetId,
          function (updatedDBId, db_data) {
            dashboard_data = db_data;
            updateOriginalVisuals();
            save_cb(updatedDBId);
          },

          error_cb
        );
      });
    };

    msdb.clone = function (dashboard_id, clone_cb, shared_visuals_to_clone) {
      currentlyActiveSheet.cloneDB(dashboard_id, clone_cb, shared_visuals_to_clone);
    };

    /* @param sheetCloneConfig = {
      report_id: id of the dashboard being imported
      sheetOrderIds: array of order ids of the selected sheets
    } */
    msdb.cloneExternalSheets = function (sheetCloneConfig) {
      const externalDashboardId = sheetCloneConfig.report_id;
      let sheetOrderIds = sheetCloneConfig.sheetOrderIds;
      sheetOrderIds = (sheetOrderIds || [1]).slice(0);
      const beforeCloneNeededSave = arc.report.pagenav.needs_save();
      const assets = [
        {
          asset: `${arc.frontend.globals.urls.reportUrl}/${externalDashboardId}`,
          postprocess: d3.json,
        },
      ];

      arc.staticfetcher.fetch(assets, function (error, report_data) {
        if (error || report_data.length === 0) {
          arc.report.alertMessage(`Error fetching App.`, error,
            { alert_report_id: sheetCloneConfig.report_id });
          return;
        }

        let externalDashboardData;
        try {
          report_data = report_data[0];
          externalDashboardData = JSON.parse(report_data.data);
          externalDashboardData = externalDashboardData.report_data || externalDashboardData;
        } catch (e) {
          arc.report.alertMessage('Invalid report data. Cannot clone');
          return;
        }

        if (externalDashboardData.global_control_widgets &&
            externalDashboardData.global_control_widgets.length) {
          cloneQuery(externalDashboardData.global_control_widgets, null,
            function updateGlobalControlWidgets(error, newWidgetsMap) {
              if (!error) {
                const newControlWidgets = externalDashboardData.global_control_widgets
                                                             .map(cw => newWidgetsMap[cw]);
                dashboard_data.global_control_widgets.concat(newControlWidgets);
              }
            });
        }

        const sheets = [];
        if (!arc.misc.isDataInSheetedFormat(externalDashboardData)) {
          externalDashboardData = getSheetedDashboardData(externalDashboardData);
        }

        const sheetsToBeCloned = externalDashboardData.dashboard_sheets
                                 .filter(sheet => sheetOrderIds.indexOf(sheet.order) >= 0)
                                 .sort(sortDashboardSheets);
        const countOfSheetsToBeCloned = sheetsToBeCloned.length;
        sheetsToBeCloned.forEach(function populateSheetObjectFromSheetData(sheetData) {
          cloneSheet(sheetData, function updateSheetsArrayCb(sheet) {
            sheets.push(sheet);
            if (sheets.length === countOfSheetsToBeCloned) {
              activate(sheet, !beforeCloneNeededSave);
              arc.report.pagenav.needs_save(true);
            }
          });
        });
      });
    };

    // @param originalWidgets: array of ids of visuals/filters that need to be cloned
    // @param cb: function to be executed with map of original and new ids
    function cloneQuery(originalWidgets, workspace_id, cb) {
      const workspaceId = workspaces.currentWorkspace() &&
                          workspaces.currentWorkspace().workspace_id;
      const post_data = {
        workspace_id: workspace_id || workspaceId,
        original_widgets: originalWidgets || [],
        dashboard_id,
      };

      $.ajax({
        url: arc.frontend.globals.urls.cloneWidgetsUrl,
        type: 'POST',
        data: JSON.stringify(post_data),
        headers: {
          'X-CSRFToken': arc.globals.csrf_token,
          contentType: 'application/json',
        },
      }).done(function cloneWidgetsSuccess(data) {
        // data is map of original visual ids and corresponding cloned new visual ids
        if (cb) {
          cb(null, data);
        }
      }).fail(function cloneWidgetsFail(req) {
        arc.report.alertMessage('Error Cloning Widgets: ', req.responseText,
          { alert_report_id: JSON.stringify(post_data.original_widgets) });
        if (cb) {
          cb(req);
        }
      });
    }

    function cloneSheet(sheetData, cloneSheetCB) {
      const reportIds = sheetData.visual_widgets.map(
          visualWidget => gridsterElementToReportId(visualWidget.id)
      );

      if (!reportIds || !reportIds.length) {
        cloneSheetHelper(sheetData, [], [], cloneSheetCB);
        return;
      }

      const assets = reportIds.map(function(report_id) {
        return {
          asset: `${arc.frontend.globals.urls.reportUrl}/${report_id}`,
          postprocess: d3.json,
        };
      });

      arc.staticfetcher.fetch(assets, function fetchAllReportsDetails(error) {
        let reportDataArr = Array.prototype.slice.call(arguments, 1);

        if (error || reportDataArr.length === 0) {
          const errorMessage = error && error.responseText ? error.responseText : (error || '');
          arc.report.alertMessage(`Error fetching data for visual(s) ${reportIds.join(', ')}`,
            errorMessage,
            { alert_report_id: dashboard_id });
          return;
        }

        let allLinkedVisualsData = [];
        reportDataArr.forEach(function (reportData) {
          if (reportData && reportData[0] && reportData[0].shared_visual_dashboards) {
            allLinkedVisualsData.push(reportData[0]);
          }
        });

        if (allLinkedVisualsData.length) {
          showCloneOptions(sheetData, allLinkedVisualsData, cloneSheetCB);
        } else {
          cloneSheetHelper(sheetData, [], [], cloneSheetCB);
        }
      });
    }

    function showCloneOptions(sheetData, allLinkedVisualsData, cloneSheetCB) {
      const DB_IDX = 0;
      const $cloneModal = $('.clone-move-with-linked-modal');
      if (arc.misc.modalShown($cloneModal)) {
        return;
      }

      const $rowTemplate = $($cloneModal.find('#visual-row-modal-template').html());
      const $submitButton = $cloneModal.find('.btn-submit-clone-move');
      const $sharedTable = $cloneModal.find('.linked-visual-table');
      const $actionSpan = $cloneModal.find('.action-type');
      const $actionSpanSmall = $cloneModal.find('.action-type.small-case');
      const $modalCloneWarningText = $cloneModal.find('.clone-only');
      const $modalMoveWarningText = $cloneModal.find('.move-only');
      const $cloneRowTemplateCheckbox = $rowTemplate.find('.clone-only');
      const $rowTemplateMoveWarningText = $rowTemplate.find('.move-only');

      // Hide all clone/move inputs and warnings
      $modalCloneWarningText.addClass('arc-invisible');
      $modalMoveWarningText.addClass('arc-invisible');
      $cloneRowTemplateCheckbox.removeClass('arc-invisible');
      $rowTemplateMoveWarningText.addClass('arc-invisible');

      // Add event handler
      $submitButton.offOn('click.submitSave', function submitSave() {
        const $cloneVisuals = $sharedTable.find('.clone-option input:checked');
        const reportsToCloneSet = new Set();
        $cloneVisuals.each(function addLinkedVisualToInfo(i, el) {
          const visualId = parseInt(el.name);
          reportsToCloneSet.add(visualId);
        });

        $cloneModal.modal('hide');

        const allLinkedVisualIds = allLinkedVisualsData.map(lv => lv.id);
        const reportsToClone = Array.from(reportsToCloneSet);
        cloneSheetHelper(sheetData, allLinkedVisualIds, reportsToClone, cloneSheetCB);
      });

      let allSharedVisualsHTML = '';
      allLinkedVisualsData.forEach(function populateAllSharedVisualsHTML(reportData) {
        const sharedVisualData = {
          VISUAL_NAME: reportData.title || 'Untitled',
          VISUAL_ID: reportData.id,
          SHARED_VISUAL_DASHBOARDS: reportData.shared_visual_dashboards.replace(/\[|\]/g, '') ||
          dashboard_id,
          DASHBOARD_IDX: DB_IDX,
          WORKSPACE_ID: reportData.workspace_id,
          WORKSPACE_NAME: reportData.workspace_name,
        };

        allSharedVisualsHTML += arc.misc.safe_substitute(
          $rowTemplate[0].outerHTML, sharedVisualData
        );
      });

      $actionSpan.text('Clone');
      $actionSpanSmall.text('clone');

      // Add shared visuals
      $sharedTable.empty();
      $sharedTable.append(allSharedVisualsHTML);
      $sharedTable.find('[data-toggle="tooltip"]').tooltip();

      // Only show warning if user decides to maintain
      $sharedTable.off('change.showWarning')
        .on('change.showWarning', '.clone-only input', function showAttention() {
          $modalCloneWarningText.removeClass('arc-invisible');
        });

      $cloneModal.modal('show');
    }

    function cloneSheetHelper(sheetData, allLinkedReportIds, cloneLinkedVisualIds, cloneSheetCB) {
      const cloneStatus = [];
      arc.report.alertMessageSmall('Cloning in process', 'alert-info');
      let newSheetData = $.extend({}, sheetData);
      newSheetData.order = getDashboardMaxOrderId() + 1;
      newSheetData.sheet_id = getDashboardMaxSheetId() + 1;
      newSheetData.sheet_handle_title = `Clone of ${sheetData.sheet_handle_title}`;
      const countCloneQueries = ['control_widgets', 'visual_widgets'].length;

      cloneQuery(sheetData.control_widgets, sheetData.workspace_id,
        function cloneFilters(error, newWidgetsMap) {
          if (!error) {
            const newControlWidgets = sheetData.control_widgets.map(cw =>
              newWidgetsMap[cw] ? String(newWidgetsMap[cw]) : cw
            );
            newSheetData.control_widgets = newControlWidgets;
            cloneComplete({
              type: 'filters',
              status: 'success',
            });
          } else {
            cloneComplete({
              type: 'filters',
              status: 'fail',
              error: error,
            });
          }
        });

      let visualWidgetIds = [];
      let filteredVisualWidgetIds = [];
      sheetData.visual_widgets.forEach(function (visualWidget) {
        const vizId = currentlyActiveSheet.gridsterElementToReportId(visualWidget.id);
        visualWidgetIds.push(vizId);
        if (!allLinkedReportIds.includes(vizId) || cloneLinkedVisualIds.includes(vizId)) {
          filteredVisualWidgetIds.push(vizId);
        }
      });

      cloneQuery(filteredVisualWidgetIds, sheetData.workspace_id,
        function cloneVisualWidgets(error, newWidgetsMap) {
          if (!error) {
            const newWidgets = sheetData.visual_widgets.map(
              function cloneVisualWidget(visualWidget) {
                const originalId = currentlyActiveSheet.gridsterElementToReportId(visualWidget.id);
                const newVisualWidget = $.extend({}, visualWidget);
                if (newWidgetsMap[originalId]) {
                  newVisualWidget.id =
                    visualWidget.id.substring(0, visualWidget.id.indexOf(originalId)) +
                    newWidgetsMap[originalId];
                }

                return newVisualWidget;
              });

            newSheetData.visual_widgets = newWidgets;
            cloneComplete({
              type: 'visuals',
              status: 'success',
            });
          } else {
            cloneComplete({
              type: 'visuals',
              status: 'fail',
              error: error,
            });
          }
        });

      // TODO: Fix clone of behaviors as well
      newSheetData.behaviors = $.extend({}, sheetData.behaviors);

      function cloneComplete(cloneStatusConfig) {
        cloneStatus.push(cloneStatusConfig);

        if (cloneStatus.length < countCloneQueries) {
          return;
        }

        arc.report.alertMessageSmallHide();
        if (cloneStatus.some(config => config.status === 'fail')) {
          arc.report.alertMessageSmall('Cloning could not complete', 'alert-danger', true);
        } else {
          arc.report.alertMessageSmall('Cloning was successful', 'alert-success', true);
          const clonedSheet = createNewSheet(newSheetData);
          if (cloneSheetCB) {
            cloneSheetCB(clonedSheet);
          }
        }
      }
    }

    // This is needed only for the dashboard_edit pages. The overall
    // report framework is used for the apps_app page.
    // TODO: This is same code as report.delete().
    msdb.delete = function (report_id, success_cb, failure_cb) {
      if (!report_id || report_id <= 0) {
        success_cb();
        return dashboard;
      }

      var post_data = {
        csrfmiddlewaretoken: arc.globals.csrf_token,
      };
      $.ajax({
        url: `${arc.frontend.globals.urls.reportUrl}/${report_id}`,
        data: post_data,
        type: 'DELETE',
        beforeSend: function (xhr) {
          // Because of a jquery bug when handling DELETE requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
        .done(() => {
          if (success_cb) {
            success_cb();
          }
        })
        .fail((req) => {
          // Notify of failure
          console.log('Failed to delete app ' + report_id + '. ' + req.responseText);
          console.log(req);
          if (failure_cb) {
            failure_cb(req);
          }
        });
    };

    function updateOriginalVisuals() {
      originalVisuals  = new Set();

      (dashboard_data.global_control_widgets || []).forEach(
          filterId => originalVisuals.add(Number(filterId))
      );

      (dashboard_data.dashboard_sheets || []).forEach(sheet => {
          (sheet.visual_widgets || []).forEach(widget =>
            originalVisuals.add(Number(gridsterElementToReportId(widget.id)))
          );

          (sheet.control_widgets || []).forEach(
              filterId => originalVisuals.add(Number(filterId))
          );
        }
      );

      (allSheets || []).forEach(sheet => sheet.originalVisuals(originalVisuals));
    }

    function gridsterElementToReportId(element_id) {
      var m = element_id.match(/\D(\d+)(-a)?$/);
      if (!m) {
        return null;
      }

      var report_id = m[1];
      return report_id;
    }

    msdb.setupMyFavoriteApps = function(add, dashboard_id, $navbar) {
      var $my_dashlink = $navbar.find('.myapp-addremove-link');
      var link_html = add ? '<i class="fa fa-star-o"></i> Add to Favorites' :
        '<i class="fa fa-star"></i> Remove from Favorites';

      $my_dashlink
        .html(link_html)
        .off('click')
        .on('click', function () {
          var post_data = {
            csrfmiddlewaretoken: arc.globals.csrf_token,
          };
          $.ajax({
            url: `${arc.frontend.globals.urls.reportMyappUrl}/${dashboard_id}`,
            data: post_data,
            type: add ? 'POST' : 'DELETE',
            beforeSend: function (xhr) {
              // Because of a jquery bug when handling DELETE requests,
              // we have to explicitly set X-CSRFToken here
              xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
            },
          })
            .done(() => {
              var message = 'Dashboard successfully ' + (add ? 'added to' :
                  'removed from') + ' Favorites.';

              arc.report.alertMessageSmall(message, 'alert-success');

              msdb.setupMyFavoriteApps(!add, dashboard_id, $navbar);
            })
            .fail((req) => {
              arc.report.alertMessage('Failed to ' + (add ? 'add' : 'remove') +
                ' app ' + dashboard_id,
                req.responseText ? req.responseText : req,
                { alert_report_id: dashboard_id });
            });
        });
    };

    function sortAllSheets(a, b) {
      if (a.order() && b.order()) {
        return a.order() - b.order();
      } else {
        return a.sheet_id() - b.sheet_id();
      }
    }

    function sortDashboardSheets(a, b) {
      if (a.order && b.order) {
        return a.order - b.order;
      } else {
        return a.sheet_id - b.sheet_id;
      }
    }

    function getSheetedDashboardData(dbData) {
      if (!dbData) {
        return;
      }

      const newDbData = $.extend({}, dbData);
      if (!arc.misc.isDataInSheetedFormat(dbData.dashboard_sheets)) {
        newDbData.dashboard_sheets = [populateSheetDataForOldCode(dbData)];
        newDbData.global_control_widgets = [];
        delete newDbData.click_behaviors;
      }

      return newDbData;

      function populateSheetDataForOldCode(dbData) {
        const newSheetId = 1;
        const sheetData = {
          order: 1,
          sheet_id: newSheetId,
          sheet_handle_title: `Sheet ${newSheetId}`,
          sheet_title: dbData.report_title || '',
          behaviors: $.extend({}, dbData.click_behaviors || {}),
          visual_widgets: (dbData.dashboard_widgets || []).slice(0),
          control_widgets: (dbData.control_widgets || []).slice(0),
        };
        return sheetData;
      }
    }

    function setDataconnections() {
      if (arc.testutil && arc.testutil.testMode) {
        return;
      }

      const filteredDataConnections = [];

      const latestVisualIds = arc.frontend.vizCookie.getLatestVisuals('visual');
      const latestVisualsUrl = `${arc.frontend.globals.urls.reportUrl}?all=true&exclude_dashboards=true&report_ids=[${latestVisualIds.join(',')}]`;

      const assets = [
        arc.frontend.globals.urls.dataconnectionUrl,
        arc.frontend.globals.urls.datasetUrl,
        latestVisualsUrl
      ]
        .map((asset) => ({ asset, postprocess: d3.json }));

      arc.staticfetcher.fetch(
        assets,
        (error, dataconnections_data, datasets_data, latestVisuals) => {
          if (error) {
            arc.report.alertMessage(
              'Could not retrieve data connections and datasets',
              error.response ? error.response : error,
              { alert_report_id: dashboard_id }
            );
            return;
          }

          data_connections = [];
          data_connections_hash = {};
          dataset_hash = {};

          // Set up hashes for quicker indexing
          dataconnections_data.forEach((dc) => {
            const dataconn = arc.dataconnection.DataConnection().load(dc);
            data_connections_hash[dc.id] = dataconn;
            data_connections.push(dataconn);
          });

          datasets_data.forEach((in_ds) => {
            const ds = arc.datasets.Dataset()
              .init_from_fetch(in_ds);
            dataset_hash[in_ds.id] = ds;
            if (in_ds.dc_id in data_connections_hash) {
              const dc = data_connections_hash[in_ds.dc_id];
              dc.add_dataset(ds);
            }
          });

          // Remove all dataconnections that don't have datasets
          data_connections.forEach((dc) => {
            if (!dc.datasets().length) {
              delete data_connections_hash[dc.dataconnection_id()];
            } else {
              filteredDataConnections.push(dc);
            }
          });

          data_connections = filteredDataConnections;

          for (const visual of latestVisuals) {
            reports_hash[visual.id] = visual;
          }

          if (currentlyActiveSheet) {
            currentlyActiveSheet
              .data_connections(data_connections)
              .data_connections_hash(data_connections_hash)
              .dataset_hash(dataset_hash)
              .reports_all_data(reports_all_data)
              .reports_hash(reports_hash)
              .widget_base_dim_px(widget_base_dim_px)
              .workspaces(workspaces);
          }

          if (arc.globals.is_edit) {
            setupVisualsSidebar();
          }
        }
      );
    }

    // This is called after data has been fetched from BE
    // Its called with data from BE.
    // Its supposed to parse fetched data and create basic sheets accordingly
    // i.e. with just sheet_id, order (via createSheetObjects)
    // Also it will create sheet handles
    msdb.load = function (dashboard_id, report_data) {
      dashboard_mode = report_data.mode;
      dashboard_data = JSON.parse(report_data.data);
      dashboard_perm = report_data.perm;
      arc.misc.updateCloneButton(report_data.dataset_perm);
      arc.misc.updateAccelerateButton(report_data.dataset_perm);

      msdb.setupGridster();
      if (!arc.misc.isDataInSheetedFormat(dashboard_data)) {
        dashboard_data = getSheetedDashboardData(dashboard_data);
      }

      dashboard_data.dashboard_sheets.sort(sortDashboardSheets)
                                     .forEach(createNewSheetObject);
      updateOriginalVisuals();

      let $subnav_report = $('.subnav-report');
      let isAddedToFavorites = !!report_data.in_userlist;
      msdb.setupMyFavoriteApps(!isAddedToFavorites, dashboard_id, $subnav_report);

      if (arc.misc.hasPerm('ds_appedit', report_data.perm)) {
        arc.misc.updateDivForPerms(['ds_appedit'], 'dataset', $subnav_report);
      }

      workspaces.currentWorkspace({
        workspace_id: report_data.workspace_id,
        workspace_name: report_data.workspace_name,
        workspace_perm: report_data.workspace_perm,
      });

      workspaces.setupWorkspaceButton($subnav_report);
      let firstSheet = getPrimarySheet(initialSheetId);
      if (!firstSheet) {
        console.log('NOTE: This scenario should not occur');
        firstSheet = allSheets[0];
        firstSheet.order(1);
      }

      // shared_visual_dashboards => all linked visuals present in current dashboard
      let sharedVisualData;
      if (report_data.shared_visual_dashboards) {
        sharedVisualData = JSON.parse(report_data.shared_visual_dashboards);
      } else {
        sharedVisualData = [];
      }

      // Adding all visuals in all sheets to arc.globals.reports.
      // We are not loading all visuals here
      dashboard_data.dashboard_sheets.forEach(function updateReportTracker(sheetInfo, sheetIdx) {
        const visualWidgets = sheetInfo.visual_widgets;
        for (let i = 0; i < visualWidgets.length; i++) {
          const reportId = parseInt(gridsterElementToReportId(visualWidgets[i].id));
          let currentSheet = allSheets[sheetIdx];
          const report = arc.globals.reports.addReport(reportId, currentSheet);

          // Give temporary, incorrect value (gets updated correctly when report is loaded)
          if (sharedVisualData.includes(reportId)) {
            report.shared_visual_dashboards('[]');
          }
        }
      });

      if (firstSheet) {
        activate(firstSheet);
      }

      if (arc.globals.is_edit) {
        setDataconnections();
      } else {
        showHideSheetHandles();
      }
    };

    function showHideSheetHandles(numberSheets) {
      if (numberSheets === undefined) {
        numberSheets = allSheets.length;
      }

      const singleSheet = numberSheets <= 1;
      $builderContainer.toggleClass('single-sheet', singleSheet && !arc.globals.is_edit);
    }

    function getPrimarySheet(sheetId) {
      if (allSheets && allSheets.length) {
        let primarySheet;

        if (sheetId) {
          primarySheet = allSheets.find((sheet) => sheet.sheet_id() === Number(sheetId));
        }

        if (!primarySheet) {
          primarySheet = allSheets.find((sheet) => sheet.order() === 1);
        }

        return primarySheet;
      }
    }

    function deactivate(sheet, ignoreDeactivateWarning) {
      arc.globals_init().click_behavior_cb(null);
      if (sheet) {
        let dd = sheet.deactivate(ignoreDeactivateWarning);
        if (dd) {
          dashboard_data = dd;
        }

        return !!dd;
      }

      return true;
    }

    function populateSheetLoadParams() {
      let currParams;
      let paramsFromURL;

      // true - if we are navigating between tabs in app
      // false - first load of apps or dashboards
      if (arc.globals.dashboard) {
        currParams = arc.globals.dashboard.current_params()[0];
        paramsFromURL = (!arc.globals.appgroup ||
          (arc.globals.appgroup.initialapp() === dashboard_id)) ?
        arc.globals.params.getCopy() : [];
      } else {
        currParams = msdb.params_history().current_params()[0];
        paramsFromURL = arc.globals.params.getCopy();
      }

      const newParams = paramsFromURL.concat(
        currParams.filter((param) => param.scope && param.scope.appId)
      );

      return newParams;
    }

    function activate(sheet, ignoreDeactivateWarning) {
      if (!sheet || sheet === currentlyActiveSheet) return;
      const isDeactivationSuccess = deactivate(currentlyActiveSheet, ignoreDeactivateWarning);
      if (!isDeactivationSuccess) {
        // User most likely cancelled the tab switch
        return;
      }

      const newParams = populateSheetLoadParams();
      dashboard_filter_controls.dashboard(sheet);

      // We might be safe with changing below to arc.globals.click_behavior_cb(sheet.red...);
      arc.globals_init().click_behavior_cb(sheet.redrawExistingWidgetsWithParams);
      arc.globals.dashboard = sheet;
      const prevActiveSheet = currentlyActiveSheet;
      currentlyActiveSheet = sheet;
      arc.globals.reports.switchSheets(sheet.sheet_id());
      workspaces.dashboard(currentlyActiveSheet);
      activateSheetHandle(
        $(
          `${'.dash-sheets-container ul.dashboard-sheet-handles li.sheet-handle' +
          '[arc-sheet-id='}${sheet.sheet_id()}]`
        )
      );
      currentlyActiveSheet.dashboard_filter_controls(dashboard_filter_controls)
                          .dashboard_data(dashboard_data)
                          .reports_to_update(reports_to_update)
                          .reports_to_delete(reports_to_delete)
                          .originalVisuals(originalVisuals)
                          .setPageLoadParams(newParams);

      if (prevActiveSheet) {
        currentlyActiveSheet.currentDataSetSelection(prevActiveSheet.currentDataSetSelection());
      }

      if (data_connections.length) {
        // Make sidebar paint again to make adding of filters work
        currentlyActiveSheet.data_connections(data_connections)
          .data_connections_hash(data_connections_hash)
          .dataset_hash(dataset_hash)
          .reports_all_data(reports_all_data)
          .reports_hash(reports_hash)
          .widget_base_dim_px(widget_base_dim_px);
      }

      sheet.activate();
      dashboard_filter_controls.activate();
      Object.keys(cbOnSheetActivation).forEach((k) => cbOnSheetActivation[k](sheet));

      if (!arc.globals.is_edit && arc.globals.dashboard_loading_message_in_viewmode) {
        const dataconnectionAssets = sheet.active_connection_ids().map(
          (dataconnectionId) => ({
            asset: `${arc.frontend.globals.urls.dataconnectionUrl}/${dataconnectionId}`,
            postprocess: d3.json,
          })
        );

        arc.staticfetcher.fetch(
          dataconnectionAssets,
          (_, ...connections) => {
            sheet.data_connections(
              connections.flat().map((dc) => arc.dataconnection.DataConnection().load(dc))
            );
          }
        );
      }

      setSheetId();
    }

    function getSheetURL(currentHref) {
      const regexp = /\bsheet=\d+/g;
      const hrefHasParams = currentHref.includes('?');
      const hrefHasSheetId = regexp.test(currentHref);

      if (hrefHasSheetId) {
        return currentHref.replace(regexp, `sheet=${currentlyActiveSheet.sheet_id()}`);
      }

      const paramConnector = hrefHasParams ? '&' : '?';
      return `${currentHref}${paramConnector}sheet=${currentlyActiveSheet.sheet_id()}`;
    }

    function setSheetId() {
      if (allSheets.length <= 1) return;

      if ($toggleViewButton.length !== 0) {
        // The location + the href link are slightly different. Make least changes necessary
        const newLinkHref = getSheetURL($toggleViewButton.attr('href'));
        $toggleViewButton.attr('href', newLinkHref);
      }

      const newLocationHref = getSheetURL(window.location.href);
      window.history.replaceState(null, '', newLocationHref);
    }

    function setupMultiSheetDashboard() {
      workspaces.msdb(msdb);

      if (arc.globals.is_edit) {
        arc.setup.dashboardEdit(dashboard_id);
      }

      dashboard_filter_controls =
        arc.report.dashboard_filter_controls()
           .build_mode(arc.globals.is_edit)
           .dom_id('#dash-filters-tab');

      loadSheetHandles();
      $('#add-dash-sheet .import-sheet').off('click.importExternalSheet')
                                        .on('click.importExternalSheet',
                                            function importExternalSheet() {

        arc.report.reportPicker({},
          showExternalApps,
          undefined,
          { displaySheets: true }
        );

        function showExternalApps(d, cb, passParams) {
          let cl = {};
          if (d) {
            cl.report_id = d.id;
            cl.sheetOrderIds = d.sheetOrderIds;
          } else {
            cl.report_id = 0;
            cl.sheetOrderIds = [];
          }

          msdb.cloneExternalSheets(cl);
          cb();
        }
      });
    }

    function gridsterDimensions() {
      const verticalScrollbar = arc.misc.estimateScrollbarWidth('vertical', '.gridster-sizer'),
            $gridsterSizer = $('.gridster-sizer'),
            width = $gridsterSizer.width() - verticalScrollbar;

      return width;
    }

    // ARC-5201: Don't use parseInt like gridster does so that calculations of
    //          column doesn't change upon resize
    msdb.weight_base_dimensions = function (serializedGridster) {
      var gridsterWidth = gridsterDimensions();

      if (arc.frontend.globals.isMobileView) {
        return gridsterWidth;
      }

      // Assume 64 columns and margins of 0px on each size of a col
      // So, if m is margin and c is column, then layout is
      //    m, c, m, m, c, m, m, c ...
      // Then, base widget calculation is below:
      // Set a minimum setting of gridster base pixels

      // Not using parseInt here might cause bug where repositioning would not work
      // However, have not been able to reproduce
      const width = Math.max(5, gridsterWidth / (dashboard_data.numColumns || 64));

      return width;
    };

    msdb.setupGridster = function () {
      // TODO: To be fixed here after merge with workspaces branch
      widget_base_dim_px = msdb.weight_base_dimensions();

      let resize,
          draggable;

      if (arc.globals.is_edit) {
        draggable = {
          stop: function () {
            arc.report.pagenav.needs_save(true);
          },
        };

        resize = {
          handle_append_to: '.arc-viz-container',
          enabled: true,
          start: function (e, ui, $widget) {
            $widget.css('overflow', 'hidden');
            if (arc.globals.dashboard) {
              arc.globals.dashboard.resizing(true);
            }
          },

          stop: function (e, ui, $widget) {
            $widget.css('overflow', '');
            arc.report.pagenav.needs_save(true);

            if (arc.globals.dashboard) {
              arc.globals.dashboard.resizing(false);
            }

            const resizerep = arc.globals.reports.getReport($widget.attr('data-arc-reportid'));
            resizerep.viz().redraw(true);
          },
        };
      }

      gridster = $('#app-selected-list ul').gridster({
        // The default widget_margins for gridster are 10px x 10 px. We
        // are going to set it to be minimum (0)
        widget_margins: [0, 0],
        widget_base_dimensions: [widget_base_dim_px, widget_base_dim_px],
        avoid_overlapped_widgets: true,

        serialize_params: function ($w, wgd) {
          return {
            col: wgd.col,
            row: wgd.row,
            size_x: wgd.size_x,
            size_y: wgd.size_y,
            id: $($w).attr('id'),
          };
        },

        draggable,
        resize,
      }).data('gridster');

      // disable dragging
      gridster.disable();

      gridster = $('#app-selected-list ul').gridster().data('gridster');

      return msdb;
    };

    function setupVisualsSidebar() {
      $('.btn-new-visual')
        .off('click')
        .on('click', msdb.createNewVisual);

      arc.shortcut.remove('Ctrl+N');
      arc.shortcut.add('Ctrl+N', msdb.createNewVisual, {
        propagate: false,
      });

      if (currentlyActiveSheet) {
        currentlyActiveSheet.paintVisualSidebar();
      }
    }

    function setCurrentWorkspaceToPrivate() {
      if (arc.testutil.testMode) {
        return;
      }

      const assets = [
        {
          asset: arc.frontend.globals.urls.workspacesUrl,
          postprocess: d3.json,
        },
      ];

      arc.staticfetcher.fetch(assets, function(error, allWorkspaces) {
        if (error) {
          arc.report.alertMessage('Unable to fetch workspaces');
          return;
        }

        allWorkspaces.some(function(ws) {
          if (ws.name === 'Private') {
            workspaces.currentWorkspace({
              workspace_id: ws.id,
              workspace_name: ws.name,
              workspace_perm: ws.perms,
            });

            return true;
          }
        });
      });
    }

    // This is practically the first function called from displayapp
    msdb.activate = function() {
      setupMultiSheetDashboard();
      if (dashboard_id || dashboard_id === 0) {
        msdb.loadById(dashboard_id);
      } else {
        // In CML workflow mode we don't want to assign the new reports to
        // Private workspace
        if (!arc.frontend.featureFlipper.getValueById('hide_workspaces')) {
          setCurrentWorkspaceToPrivate();
        }

        dashboard_data.dashboard_sheets = [];
        dashboard_data.global_control_widgets = [];
        dashboard_data.dashboard_widgets = [];
        const sheet = createNewSheet();
        currentlyActiveSheet = sheet; // Can't wait for timeout to complete to set this
        setTimeout(() => {
          currentlyActiveSheet = null;

          activate(sheet);
          workspaces.setupWorkspaceButton($('.subnav-report'));

          if (arc.globals.is_edit) {
            setDataconnections();
          }

          $('.subnav-report .view-link-container').hide();
        }, 0);
      }

      return msdb;
    };

    function getDashboardMaxOrderId() {
      let max = Math.max.apply(Math, allSheets.map(s => s.order()));
      return max > 0 ? max : 0;
    }

    function getDashboardMaxSheetId() {
      let max = Math.max.apply(Math, allSheets.map(s => s.sheet_id()));
      return max > 0 ? max : 0;
    }

    function findSheetFromHandle($handle) {
      if ($handle && $handle.length) {
        const sheetId = Number.parseInt($handle.attr('arc-sheet-id'), 10);
        return allSheets.find((sheet) => sheet.sheet_id() === sheetId);
      }
    }

    function activateSheetFromHandle($handle) {
      const sheet = arguments.length ? findSheetFromHandle($handle) : allSheets[0];
      if (!sheet || sheet === currentlyActiveSheet) {
        return;
      }

      setTimeout(() => {
        activate(sheet);
      }, 0);
    }

    function activateSheetHandle($handle) {
      const $sheetHandleUL = $('.dash-sheets-container ul.dashboard-sheet-handles');
      $sheetHandleUL.find('li.sheet-handle').removeClass('active');
      if ($handle) {
        $handle.addClass('active');
      }
    }

    function createNewSheetHandle(sheet) {
      const $sheetHandleUL = $('.dash-sheets-container ul.dashboard-sheet-handles');
      const sheetHandleTemplate = $('#dash-sheet-handle-template').html();
      const countOfAllSheetHandles = $sheetHandleUL.find('li.sheet-handle').length;
      const sheetOrder = sheet.order() ? (sheet.order() - 1)
                                     : countOfAllSheetHandles;
      const $newSheetHandle = arc.misc.safe_substitute(sheetHandleTemplate, {
        SHEET_TITLE: sheet.sheet_handle_title() || 'Untitled Sheet',
        SHEET_ID: sheet.sheet_id(),
      });
      const $precedingHandle = $sheetHandleUL.find(`li.sheet-handle:nth-child(${sheetOrder})`);
      if (!$precedingHandle.length || !sheetOrder) {
        $sheetHandleUL.prepend($newSheetHandle);
      } else {
        $precedingHandle.after($newSheetHandle);
      }

      showHideSheetHandles($sheetHandleUL.find('li.sheet-handle').length);
    }

    function makeSheetTitleUneditable(evt) {
      const $titleContainer = $(evt.currentTarget);
      const newSheetTitle = $titleContainer.text().trim();
      $titleContainer.attr('contenteditable', 'false');
      if (oldSheetTitle !== newSheetTitle) {
        arc.report.pagenav.needs_save(true);
      }

      const clone = $titleContainer.clone();
      $titleContainer.after(clone).remove();

      if (document.selection) {
        document.selection.empty();
      } else if (window.getSelection) {
        window.getSelection().removeAllRanges();
      }
    }

    function loadSheetHandles() {
      const $sheetHandlesContainer = $('.dash-sheets-container');
      const $sheetHandleUL = $sheetHandlesContainer.find('ul.dashboard-sheet-handles');
      $sheetHandleUL.empty().off();
      if ((allSheets.length >= 2) || arc.globals.is_edit) {
        activateSheetFromHandle($sheetHandleUL.find('li.sheet-handle:first-of-type'));
      } else {
        activateSheetFromHandle();
      }

      showHideSheetHandles();

      $sheetHandleUL.on('click', '.sheet-handle', function singleClickOnSheetHandle(evt) {
        if ($(evt.target).closest('ul').hasClass('dropdown-menu')) {
          // User clicked inside dropdown menu. Ignore.
          return;
        }

        let $currentLi = $(evt.currentTarget).closest('li.sheet-handle');
        activateSheetFromHandle($currentLi);
      });

      if (arc.globals.is_edit) {
        // make sheet title editable
        $sheetHandleUL.on('click.renameSheet', '.sheet-rename', function renameSheet(evt) {
          let $titleContainer = $(evt.currentTarget).closest('li.sheet-handle')
                                                    .find('.sheet-title');
          oldSheetTitle = $titleContainer.text().trim();
          makeSheetTitleEditable($titleContainer);
        });

        $sheetHandleUL.on('click.cloneSheet', '.sheet-clone', function cloneInternalSheet(evt) {
          const $currentLi = $(evt.currentTarget).closest('li.sheet-handle');
          const currentSheet = findSheetFromHandle($currentLi);
          const sheetData = dashboard_data.dashboard_sheets.find(
              (ds) => ds.sheet_id === currentSheet.sheet_id()
          );
          cloneSheet(sheetData, (sheet) => {
            activate(sheet);
            arc.report.pagenav.needs_save(true);
          });
        });

        // make sheet title editable if user double-clicks on sheet handle
        $sheetHandleUL.on('dblclick', '.sheet-title', function doubleClickOnSheetTitle(evt) {
          let $titleContainer = $(evt.currentTarget);
          oldSheetTitle = $titleContainer.text().trim();
          makeSheetTitleEditable($titleContainer);
        });

        // TODO: Add confirmation to tell the user that all the visuals will also get deleted
        $sheetHandleUL.on('click', 'a.sheet-delete', function deleteSheet(evt) {
          let $currentLi = $(evt.currentTarget).closest('li.sheet-handle');
          if (!$currentLi.length || $sheetHandleUL.find('li.sheet-handle').length < 2) {
            arc.report.alertMessageSmall(`Cannot delete last sheet in an dashboard`,
                                         'alert-danger');
            return;
          }

          let currentSheet = findSheetFromHandle($currentLi);
          const isCurrentActiveSheet = currentSheet === currentlyActiveSheet;
          if (isCurrentActiveSheet) {
            currentlyActiveSheet = null;
          }

          const data = currentSheet.delete(isCurrentActiveSheet);
          if (!data || !data.dashboard_data) {
            arc.report.alertMessageSmall('Could not delete this sheet', 'alert-danger');
            return;
          } else {
            dashboard_data = data.dashboard_data;
            reports_to_delete = data.reports_to_delete || [];
            reports_to_update = data.reports_to_update || [];
            allSheets.sort(sortAllSheets);
            dashboard_data.dashboard_sheets.sort(sortDashboardSheets);
            let currentSheetOrder = currentSheet.order();
            allSheets.splice(allSheets.indexOf(currentSheet), 1);
            allSheets.forEach(sheet => {
              // @assert sheet.order() !== currentSheetOrder // For any of the sheets
              if (sheet.order() > currentSheetOrder) {
                sheet.order(sheet.order() - 1);
              }
            });
            dashboard_data.dashboard_sheets.forEach(sheet => {
              if (sheet.order > currentSheetOrder) {
                sheet.order--;
              }
            });
          }

          if ($sheetHandleUL.find('li.sheet-handle').length > 1) {
            $(evt.currentTarget).closest('li.sheet-handle').remove();
            arc.report.pagenav.needs_save(true);
            if (isCurrentActiveSheet) {
              activateSheetFromHandle($sheetHandleUL.find('li.sheet-handle:first-of-type'));
            }
          }
        });

        // make sheet tab title uneditable when user clicks away
        $sheetHandleUL.on('blur', '.sheet-title', makeSheetTitleUneditable);

        $sheetHandleUL.on('click', 'a.sheet-move-right', function moveSheetRight(evt) {
          let $currentLi = $(evt.currentTarget).closest('li.sheet-handle');
          let $nextLi = $currentLi.next();
          let currentSheet = findSheetFromHandle($currentLi);
          if ($nextLi.hasClass('sheet-handle') && currentSheet.order() < allSheets.length) {
            $currentLi.remove();
            $nextLi.after($currentLi);
            let nextSheet = findSheetFromHandle($nextLi);
            currentSheet.order(currentSheet.order() + 1);
            nextSheet.order(nextSheet.order() - 1);
          }

          arc.report.pagenav.needs_save(true);
        });

        $sheetHandleUL.on('click', 'a.sheet-move-left', function moveSheetLeft(evt) {
          let $currentLi = $(evt.currentTarget).closest('li.sheet-handle');
          let $prevLi = $currentLi.prev();
          let currentSheet = findSheetFromHandle($currentLi);
          if ($prevLi.hasClass('sheet-handle') && currentSheet.order() > 1) {
            $currentLi.remove();
            $prevLi.before($currentLi);
            let prevSheet = findSheetFromHandle($prevLi);
            currentSheet.order(currentSheet.order() - 1);
            prevSheet.order(prevSheet.order() + 1);
          }

          arc.report.pagenav.needs_save(true);
        });

        // create new sheet and bring it to focus
        $('#add-dash-sheet .more-btn, #add-dash-sheet .add-new-sheet').on('click.addNewSheet',
          addNewSheetHandle
        );

        $('#add-dash-sheet .more-btn, #add-dash-sheet .reset-sheet-id').on('click', () => {
          const sheetHandleUL = $('.dash-sheets-container ul.dashboard-sheet-handles');

          dashboard_data.dashboard_sheets.forEach((sheet) => {
            const $handle = sheetHandleUL.find(`li.sheet-handle:nth-child(${sheet.order})`);
            $handle.attr('arc-sheet-id', sheet.order);

            sheet.sheet_id = sheet.order;
          });

          allSheets.forEach((sheet) => {
            sheet.sheet_id(sheet.order());
          });

          setSheetId();
          arc.report.pagenav.needs_save(true);
        });
      } else {
        $sheetHandlesContainer.find('#add-dash-sheet').addClass('arc-invisible');
      }

      function addNewSheetHandle() {
        cbOnSheetActivation.ClickOnNewSheetHandle = function clickOnNewSheetHandle() {
          makeSheetTitleEditable($sheetHandleUL.find('li.sheet-handle:last-of-type .sheet-title'));
          delete cbOnSheetActivation.ClickOnNewSheetHandle;
        };

        const newSheet = createNewSheet();
        activate(newSheet);
      }

      function makeSheetTitleEditable($titleContainer) {
        if ($titleContainer && $titleContainer.length) {
          $titleContainer.attr('contenteditable', 'true').focus();
          selectTitleText($titleContainer[0]);
          $titleContainer.on('keydown', function(e) {
            let disable = false;
            if (e.keyCode === 13) {
              makeSheetTitleUneditable(e);
            } else if (e.ctrlKey || e.metaKey) {
              // This will let ctrl+c and ctrl+v work. metaKey => cmd key in mac
              switch (e.keyCode) {
                case 66: //ctrl+B or ctrl+b
                case 98:
                  disable = true;
                  break;
                case 73: //ctrl+I or ctrl+i
                case 105:
                  disable = true;
                  break;
                case 85: //ctrl+U or ctrl+u
                case 117:
                  disable = true;
                  break;
              }
            }

            if (disable) {
              e.stopPropagation();
              e.preventDefault();
            }
          });
        }
      }

      function selectTitleText(div) {
        let range, sel;
        if (!div) {
          return;
        }

        if (window.getSelection && document.createRange) {
            range = document.createRange();
            range.selectNodeContents(div);
            sel = window.getSelection();
            sel.removeAllRanges();
            sel.addRange(range);
        } else if (document.body.createTextRange) {
            range = document.body.createTextRange();
            range.moveToElementText(div);
            range.select();
        }
      }
    }

    return msdb;
  };
}());
