window.arc = window.arc || {};

(function() {
"use strict";

arc.params = arc.params || {};
arc.globals = arc.globals || {};

arc.params.convertKey = function(key) {
  return typeof key === 'string' && !arc.globals.parameters_case_sensitive ?
           key.toLowerCase() :
           key;
};

// These two functions are used to format a list to a list param
// so it can be used by other apps.
arc.params.format_list = function(in_list, coltype, reverse, conn_type, isMultiSelectStringParam) {
  let quote_params = false;
  if (coltype) {
    coltype = coltype.toLowerCase();
    if ((coltype === 'string') || (coltype === 'timestamp')) {
      quote_params = true;
    }
  } else if (in_list.length) {
      if (typeof (in_list[0]) === 'number') {
        quote_params = false;
      } else {
        quote_params = true;
        coltype = 'string';
      }
  }

  let ret_list = [];
  if (in_list.length) {
    if (!quote_params) {
      ret_list = in_list.slice(0);
    } else {
      ret_list = in_list.map((d) => arc.report.shelf.parse_filter_value(
          d, coltype, reverse, conn_type, isMultiSelectStringParam
        ));
    }
  }

  return ret_list;
};

arc.params.splitParams = function (key) {
  if (key.length === 0) {
    return ['', ''];
  }

  const regexp = /\.(data|alias|exclude|ignore|start|end|config|pivot|ignore|include_nulls)$/;
  let pos = key.search(regexp);
  pos = pos < 0 ? key.length : pos;
  return [key.substring(0, pos), key.substring(pos + 1, key.length)];
};

// parse a string to a list
arc.params.parse_list = function(in_str) {
  if (!in_str) {
    return [];
  }

  if (in_str.length === 0) {
    return [];
  }

  let list;
  if (in_str[0] === "'") {
    // string
    list = in_str.split(/'\s*,\s*'/);
    list[0] = list[0].substr(1);
    list[list.length - 1] = list[list.length - 1].substr(0, list[list.length - 1].length - 1);
  } else if (/^[\d]+$/.test(in_str[0]) || (in_str[0] === '-' && /^[\d]+$/.test(in_str[1]))) {
    // Assuming input list to be numbers
    list = in_str.split(/, /).map(Number);
    // eslint-disable-next-line no-restricted-globals
    if (list.some((value) => isNaN(value))) {
      // not numbers
      list = null;
    }
  }

  if (!list) {
    if (in_str) {
      list = [in_str];
    } else {
      list = [];
    }
  }

  return list;
};

// Initialize parameters passed in
arc.params.tracker = function() {
  var kvs = {},
      kvlist = [];

  var params = function() { return "arc.params.tracker"; };

  params.getSegmentData = function(dataset_id) {
    let param;
    let savedParamName;

    kvlist.forEach(function(kv) {
      const k_arr = arc.params.splitParams(kv.key);
      const [key, ext] = k_arr;
      if (ext === 'ignore') {
        if (key === savedParamName) {
          savedParamName = undefined;
          param = undefined;
        }

        return;
      }

      if (kv.isDatasetSegment && kv.scope && (ext === 'data' || ext === '')) {
        const scopeType = kv.scope.scopeType;
        if ((scopeType === arc.params.ParamEnums.ScopeTypeEnum.GLOBAL) ||
            ((scopeType === arc.params.ParamEnums.ScopeTypeEnum.DATASET) &&
             (kv.source_dataset_id === dataset_id))) {

          // Note that even when we find the param, we want to continue iterating
          // through kvlist so that we can get the most recent setting
          savedParamName = key;
          param = kv;
        }
      }
    });

    return param;
  };

  params.get = function(k, returnCompleteParamObject) {
    let param;

    kvlist.forEach(function(kv) {
      const k_arr = arc.params.splitParams(kv.key);
      if (k_arr[1] === 'ignore') {
        if (k_arr[0] === k) {
          param = undefined;
        }

        return;
      }

      if (kv.key === k) {
        // Note that even when we find the param, we want to continue iterating
        // through kvlist so that we can get the most recent setting
        param = kv;
        return;
      }
    });

    if (returnCompleteParamObject) {
      return param;
    }

    return param && param.value;
  };

  params.exists = function(k) {
    return (params.get(k) !== undefined);
  };

  // Introducing new add function specifically for adding params from URL
  // Eg. URL:
  // ?param.alpha=1&param.beta.data=2, 3, 4, 5&param.beta.exclude='not in'&
  // param.beta.alpha='Apple', 'Google', 'Microsoft'&
  // param.gamma.data='apple', 'orange', 'banana'&param.gamma.exclude='in'&param.delta=microsoft&
  // param.theta.start=2010-01-02 00:00:00&param.theta.end=2016-12-31 00:00:00&
  // scope=[{"filterId":148,"datasetId":8,"filterName":"start_date"}]
  params.addFromUrl = function (urlParams) {
    let dateParams = [];
    let otherParams = [];
    urlParams.forEach(function separateDateAndOtherParams(param) {
      if (param.key && ['start', 'end', 'config'].includes(arc.params.splitParams(param.key)[1])) {
        dateParams.push(param);
      } else {
        otherParams.push(param);
      }
    });

    dateParams.sort(function forSpecialDateParamsMakeConfigSortAfterStartEnd(param1, param2) {
      let param1KeySplitArr = arc.params.splitParams(param1.key);
      let param2KeySplitArr = arc.params.splitParams(param2.key);
      if (param1KeySplitArr[0] === param2KeySplitArr[0]) {
        // config should be sorted after start/end
        if (param1KeySplitArr[1] === 'config' && param2KeySplitArr[1] !== 'config') {
          return 1;
        } else if (param1KeySplitArr[1] !== 'config' && param2KeySplitArr[1] === 'config') {
          return -1;
        }

        return param1KeySplitArr[1].localeCompare(param2KeySplitArr[1]);
      }

      return param1KeySplitArr[0].localeCompare(param2KeySplitArr[0]);
    });

    // This is necessary to make sure include_nulls come last
    // when they are converted to paramObject to avoid conflicts
    urlParams = otherParams.concat(dateParams).sort((a, b) => {
      if (a.key.includes('include_nulls')) return 1;
      if (b.key.includes('include_nulls')) return -1;

      return a.key.localeCompare(b.key);
    });

    addAllUrlParams(urlParams);
    return params;
  };

  function addAllUrlParams(urlParams) {
    const scopeConfig = {
      dataset: {},
      global: {},
    };

    arc.globals.filter_scope.forEach((scope) => {
      if (scope.scopeType === arc.params.ParamEnums.ScopeTypeEnum.DATASET && scope.datasetId) {
        if (!scopeConfig.dataset[scope.filterName]) {
          scopeConfig.dataset[scope.filterName] = {};
        }

        scopeConfig.dataset[scope.filterName].datasetId = scope.datasetId;
        scopeConfig.dataset[scope.filterName].filterId = scope.filterId;
      } else {
        if (!scopeConfig.global[scope.filterName]) {
          scopeConfig.global[scope.filterName] = {};
        }

        scopeConfig.global[scope.filterName].filterId = scope.filterId;
      }
    });

    const paramObjects = arc.params.convertOldParamArrToNewParamObjects(
      urlParams, null, scopeConfig, null, true
    );
    const oldFormatParamsArr = arc.params.convertToOldFormatArr(paramObjects);
    oldFormatParamsArr.forEach(a => kvlist.push(a));
  }

  params.add = function(p) {
    var newk = p.key;

    kvs[newk] = p.value;
    kvlist.push({
      key: newk,
      value: p.value,
      value_list: p.value_list,
      source_visual_id: p.source_visual_id,
      source_dataset_id: p.source_dataset_id,
      scope: p.scope,
      isDatasetSegment: p.isDatasetSegment,
    });

    var k_arr = p.key.split(".");
    if (k_arr[1] === 'ignore') {
      params.delete(k_arr[0]);
    }

    return params;
  };

  params.delete = function(key) {
    delete kvs[arc.params.convertKey(key)];

    // TODO: Confirm if this deletes param and all possible suffixes while keeping param.ignore
    const casedKey = arc.params.convertKey(key);
    kvlist = kvlist.filter(pair => (arc.params.convertKey(pair.key) !== casedKey));
  };

  params.getCopy = function() {
    return kvlist.slice(0);
  };

  params.getkvsList = function(source_visual_id) {
    var l = [];
    params.getkvs(source_visual_id, l);
    return l;
  };

  params.getkvs = function(source_visual_id, returnList) {
    var ignoreKeys = {};
    kvlist.forEach(function(kv) {
      if (source_visual_id && kv.source_visual_id !== source_visual_id) {
        return;
      }

      var k_arr = arc.params.splitParams(kv.key);
      if (k_arr[1] === 'ignore') {
        ignoreKeys[k_arr[0]] = true;
        return;
      }

      ignoreKeys[kv.key] = false;
    });

    var kvhash = {};
    kvlist.forEach(function(kv) {
      if (source_visual_id && kv.source_visual_id !== source_visual_id) {
        return;
      }

      var k_arr = arc.params.splitParams(kv.key);

      kvhash[kv.key] = kv.value;
      if (returnList) {
        returnList.push(kv);
      }
    });

    return kvhash;
  };

  params.escape = function(s, paramName) {
    // The 'xxxx.data' parameter usage has already done the escaping via the
    // format_list() function above so don't try to do it again
    if (paramName && paramName.substr(-5) === '.data') {
      return s;
    }
    if (typeof (s) === 'string') {
      return arc.misc.escapeString(s);
    }
    return s;
  };

  params.substitute = function(in_str, default_setters, leave_code_alone,
      parametersSubstituted) {
    if (!in_str) {
      return in_str;
    }

    var vt = arc.misc.viztemplates();

    $.each(kvs, function(k, v) {
      vt.addKV(k, v);
    });

    if (arc.globals.ms_dashboard && arc.globals.ms_dashboard.params_history()) {
      arc.globals.ms_dashboard.params_history().current_params()[0]?.forEach((kv) => {
        vt.addKV(kv.key, kv.value);
      });
    }

    vt.addTemplate(in_str);
    if (default_setters) {
      default_setters.forEach(function(s) {
        vt.addTemplate(s);
      });
    }

    return vt.transformInput(in_str, params.escape, leave_code_alone,
        parametersSubstituted);
  };

  return params;
};

// Common Function to be called from date filters or other filter widgets
arc.params.sendParameters = function(config) {
  const {paramObjects, do_not_refresh, lib} = config;
  lib.dashboard_click(paramObjects, do_not_refresh);

  arc.perf.PAGEPERF.addProfile({
    id: lib.report_id(),
    type: 'Parameters',
    time: 0,
    profile: paramObjects.map(function (p) {
      return [p.outputParameter, JSON.stringify(p)];
    }),
  });
};

arc.params.isMultiSelectStringParam = function(paramName, allParamsHash={}, valuetype=null) {
  // Either data param or alias param for multi-select value - and string valuetype
  if (valuetype && !arc.misc.isString(valuetype)) {
    // Do not do special quoting for non string values
    return false;
  }

  const keyArr = arc.params.splitParams(String(paramName));
  return (keyArr[1] === 'data' ||
    (keyArr[1] === 'alias' && (allParamsHash[`${keyArr[0]}.data`]) !== undefined));
};

arc.params.setting = function(vizApi, settingName, vizType, value) {
  try {
    let setting = vizApi[`${vizType}_${settingName}`];
    if (typeof setting === 'undefined') {
      setting = vizApi[settingName];
    }

    if (arguments.length === 3) return setting.value();
    if (settingName === 'values') {
      // Append settings to currently existing setting
      const oldValues = JSON.parse(setting.value()),
        newValues = JSON.parse(value);
      value = JSON.stringify(oldValues.concat(newValues));
    }

    setting.value(value);
  } catch (e) {
    arc.report.alertMessage(`Ignoring url parameter property: ${settingName}\n${e}`);
  }

  return vizApi;
};

}());
