window.arc = window.arc || {};

(function () {
  "use strict";

arc.report = arc.report || {};

arc.report.savetemplate = function () {
  const savetemplate = function() { };

  savetemplate.setup = function () {
    // event handler for save button in modal
    $('#create-newtemplate-form')
      .offOn('submit.createNewTemplate', (e) => {
        e.preventDefault();
        const dsreqs = [];
        let report_type;
        let report_id;
        const name = $('input#new-template-name').val();
        const description = $('textarea#new-template-description').val();
        if (arc.globals.is_dashboard) {
          report_type = 'dashboards';
          report_id = arc.globals.dashboard.dashboard_id();
          arc.globals.reports.vizCallback((viz) => { dsreqs.push(viz.dsreq()); });
        } else {
          report_type = 'visuals';
          report_id = arc.report.getPrimaryReport().report_id();
          dsreqs.push(arc.report.getPrimaryReport().viz().dsreq());
        }

        $.ajax({
          url: `${arc.frontend.globals.urls.templateUrl}save_template/${report_id}`,
          data: {
            name,
            description,
            report_id,
            report_type,
            dsreqs: JSON.stringify(dsreqs)
          },
          type: 'POST',
          beforeSend(xhr) {
            // Because of a jquery bug when handling PUT requests,
            // we have to explicitly set X-CSRFToken here
            xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
          },
        })
          .done((res) => {
            arc.report.alertMessageSmall(res, 'alert-success');
            $('input#new-template-name').val('');
            $('textarea#new-template-description').val('');
          })
          .fail((req) => {
            arc.report.alertMessage('Cannot save as template', req.responseText);
          });
        $('#create-newtemplate').modal('hide');
        e.stopImmediatePropagation();
        return false;
      });
  };

  return savetemplate;
};

arc.report.loadTemplate = function() {
  var $modal = $('#showme-template-modal');
  var dataset;

  function paintModal(data) {
    var $ul = $("#app-grid-list-template");
    var div = $ul.find("li").find('div');
    var span = $ul.find("li").find('span')[0];
    var params;
    $ul.find('li').filter(function () {
      return $(this).attr('template') !== "true";
    }).remove();
    d3.select('#app-grid-list-template')
      .selectAll('li[template="false"]')
      .data(data)
      .enter()
      .append('li')
      .attr('class', 'appgrid-li')
      .attr('template', false)
      .each(function (d) {
        d3.select(this)
          .append(function () {
            var new_div = div[0].cloneNode(true);
            if (d.img) {
              $(new_div).find('img').attr('src', "/arc/apps/template_snapshot/" + d.id);
            }
            return new_div;
          });
        d3.select(this)
          .append(function () {
            var new_div = div[1].cloneNode(true);
            params = JSON.parse(d.params);
            $(new_div).text(d.name)
              .attr('title', `Requires: ${params.no_dims} dimension
                              and ${params.no_aggs} measures`)
              .attr('data-content', d.desc);
            return new_div;
          });
        d3.select(this)
          .append(function () {
            return span.cloneNode();
          });
        d3.select(this)
          .append("input")
          .attr("style", "display: none")
          .attr("params", d.params)
          .attr("report-id", d.report_id);
      });
  }

  function paintDimsAggs(dataset_id) {
    var data;
    $.ajax({
      url: `${arc.frontend.globals.urls.datasetUrl}/${dataset_id}`,
      type: 'GET',
      async: false,
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .done(function(res) {
      data = res[0];
    })
    .fail(function(req, textstatus, errorthrown) {
      arc.report.alertMessage("Cannot load dataset details", req.responseText);
    });

    dataset = arc.datasets.Dataset()
                              .dataset_id(dataset_id)
                              .dataconnection_id(data.dc_id)
                              .init_from_fetch(data);
    arc.report.datasetSetupCols({
      dataset,
      includeAggregateColumns: true,
      $dimShelf: $('.dim-app-params'),
      $aggShelf: $('.agg-app-params'),
      includeSegment: true,
    });

    $modal.find('.selection_element')
          .off("click")
          .on("click", function(e, i) {
            var $li = $(this)[0].cloneNode(true);
            if ($(this).parents('ul').prev().text().startsWith("Dimen")) {
              $("#dim-shelf-template").append($li);
            } else {
              $("#msr-shelf-template").append($li);
            }
          });
  }

  arc.report.loadTemplate.setup = function () {
    var setup = function() { return "arc.report.loadTemplate.setup";};
    setup.dimPicked = [];
    setup.aggPicked = [];
    setup.$active_li = undefined;
    setup.workflow = undefined;
    setup.active_report_id = undefined;

    $modal.on('show.bs.modal', function(e) {
      // if the modal comes from data page
      if ($(e.relatedTarget).data('value') !== undefined) {
        setup.dataset_id = $(e.relatedTarget).data('value');
        setup.show_workflow_selector();
      } else {
        $modal.find('#template-workflow-selector').hide();
        $modal.find('ul#dim-shelf-template').empty();
        $modal.find('ul#msr-shelf-template').empty();
        setup.pick_columns_first_workflow();
      }
    });

    setup.show_workflow_selector = function () {
      setup.dimPicked = [];
      setup.aggPicked = [];
      setup.$active_li = undefined;
      setup.active_report_id = undefined;

      $modal.find('.modal-title').text('Step 1 - Select how to build your ' +
        'dashboard from templates');
      $modal.find('#template-next-button').hide();
      $modal.find('#template-go-button').hide();
      $modal.find('#template-dashboard-selector').hide();
      $modal.find('#template-dataset-column-selector').hide();
      $modal.find('#template-workflow-selector').show();
      $modal.find('#template-next-button').hide();
      $modal.find('#template-go-button').hide();
      $modal.find('#template-back-button').hide();
      $modal.find('ul#dim-shelf-template').empty();
      $modal.find('ul#msr-shelf-template').empty();

      $modal.find('#template-workflow-columns-first').off('click').on('click', function () {
        $modal.find('#template-workflow-selector').hide();
        $modal.find('#template-back-button').show();
        $modal.find('#template-next-button').show();
        setup.workflow = 1;
        setup.pick_columns_first_workflow();
      });

      $modal.find('#template-workflow-dashboard-first').off('click').on('click', function () {
        $modal.find('#template-workflow-selector').hide();
        $modal.find('#template-back-button').show();
        $modal.find('#template-next-button').show();
        setup.workflow = 2;
        setup.pick_dashboard_first_workflow();
      });
    };

    setup.pick_dashboard_first_workflow = function () {

      show_all_templates();

      //setup next button
      $modal.find('#template-next-button').off('click').on('click', function () {
        setup.$active_li = $('#app-grid-list-template > li.active');
        setup.active_report_id = setup.$active_li.find('input').attr('report-id');
        var params = JSON.parse(setup.$active_li.find('input').attr('params')),
            geo_text = '';
        const num_geo = params.columns[Object.keys(params.columns)[0]].geo.length;
        if (num_geo > 0) {
          geo_text = `(${num_geo} of them being Geo point type) `;
        }
        $modal.find('.modal-title').text('Step 3 - Select Dimension and Measure Fields');
        $modal.find('#template-dataset-column-selector').show();
        $modal.find('#template-dashboard-selector').hide();
        $modal.find('#template-next-button').hide();
        $modal.find('#template-go-button').show();
        $modal.find('#order-detail-text').hide();
        $modal.find('#dim-msr-select-text').text(
          `Select ${params.no_dims} dimension ${geo_text}and ${params.no_aggs}
          measure fields, then click GO.`);

        paintDimsAggs(parseInt(setup.dataset_id));

        //setup back button
        $("#template-back-button").off('click').on('click', function () {
          $modal.find("#template-next-button").show();
          $modal.find("#template-go-button").hide();
          show_all_templates();
        });

      });

      function show_all_templates() {
        $modal.find('.modal-title').text('Step 2 - Select a Template');
        $modal.find('#template-next-button').show();
        $modal.find('#template-go-button').hide();
        $modal.find('#template-dataset-column-selector').hide();
        $modal.find('#template-dashboard-selector').show();

        //setup back button
        $modal.find('#template-back-button').off('click').on('click', function () {
          setup.show_workflow_selector();
        });

        // get all templates and paint modal
        $.ajax({
          url: `${arc.frontend.globals.urls.templateUrl}showall`,
          data: {},
          type: 'GET',
          beforeSend: function(xhr) {
            // Because of a jquery bug when handling PUT requests,
            // we have to explicitly set X-CSRFToken here
            xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
          },
        })
          .done(function(res) {
            $modal.find('#select-template-text').text(
              `Choose one of these Dashboard Templates, then click NEXT.`);
            $("#template-dataset-columns").hide();
            $("#app-grid-list-template").show();
            paintModal(res.templates);
            $('#app-grid-list-template > li').removeClass('active');
            $("#app-grid-list-template div[data-toggle='popover']").popover({
              container: '.showme-modal',
              placement: 'bottom',
              trigger: 'hover',
              title: 'Template Description'
            });
            if (setup.active_report_id !== undefined) {
              $("#app-grid-list-template input[report-id=" +
                setup.active_report_id + "]").parent('li').addClass('active');
            }

            $('#app-grid-list-template > li').offOn('click.allTemplatesActiveAppGridList', function() {
              if ($(this).attr("class") !== 'active') {
                $('#app-grid-list-template > li').removeClass('active');
                $(this).addClass('active');
              }
            });
            $('.showme-modal').find('.modal-body')
              .height(($(window).height() - 220) + 'px');
          })
          .fail(function(req, textstatus, errorthrown) {
            arc.report.alertMessage("Cannot load templates", req.responseText);
          });
      }
    };

    // workflow for the case where dims and aggs are selected first
    setup.pick_columns_first_workflow = function () {
      function show_dim_agg_selector() {
        $modal.find('.modal-title').text('Step 2 - Select Dimension and Measure Fields');
        $modal.find("#template-next-button").show();
        $modal.find("#template-go-button").hide();
        $modal.find('#template-dataset-column-selector').show();
        $modal.find('#template-dashboard-selector').hide();

        $modal.find('#order-detail-text').show();
        $modal.find('#dim-msr-select-text').text(`Select the dimension and
                  measure fields that you are interested in, then click NEXT.`);

        paintDimsAggs(parseInt(setup.dataset_id));

        // handle change selection button on dashboard template: populate dimension and
        // measure shelves with columns already picked
        var found = false;
        setup.dimPicked.forEach(function(key) {
          found = false;
          $modal.find('ul.dim-app-params li').each(function () {
            if ($(this).text() === key) {
              $("#dim-shelf-template").append($(this)[0].cloneNode(true));
              found = true;
              return false;
            }
          });
          if (!found) {
            $modal.find('ul.agg-app-params li').each(function () {
              if ($(this).text() === key) {
                $("#dim-shelf-template").append($(this)[0].cloneNode(true));
                return false;
              }
            });
          }
        });
        setup.aggPicked.forEach(function(key) {
          found = false;
          $modal.find('ul.agg-app-params li').each(function () {
            if ($(this).text() === key) {
              $("#msr-shelf-template").append($(this)[0].cloneNode(true));
              found = true;
              return false;
            }
          });
          if (!found) {
            $modal.find('ul.dim-app-params li').each(function () {
              if ($(this).text() === key) {
                $("#msr-shelf-template").append($(this)[0].cloneNode(true));
                return false;
              }
            });
          }
        });

        // setup back button
        $modal.find('#template-back-button').off('click').on('click', function () {
          setup.show_workflow_selector();
        });
      }

      // draw dims and aggs and populated columns already picked if applicable
      show_dim_agg_selector();

      // setup next button
      $modal.find('#template-next-button').off('click').on('click', function () {
        $modal.find('.modal-title').text('Step 3 - Select a Template');
        $modal.find('#template-next-button').hide();
        $modal.find('#template-go-button').show();
        $modal.find('#select-template-text').text(`Select the template you
                  would like to use for your dashboard, then click GO.`);
        $modal.find('#template-dataset-column-selector').hide();
        $modal.find('#template-dashboard-selector').show();

        // populate/repopulate picked dimension and meatures
        setup.dimPicked = [];
        setup.aggPicked = [];
        $("#dim-shelf-template li").each(function () {
          setup.dimPicked.push($(this).text());
        });
        $("#msr-shelf-template li").each(function () {
          setup.aggPicked.push($(this).text());
        });

        $("#template-back-button").off('click').on('click', function () {
          setup.$active_li = $('#app-grid-list-template > li.active');
          setup.active_report_id = setup.$active_li.find('input').attr('report-id');
          setup.dimPicked = [];
          setup.aggPicked = [];
          show_dim_agg_selector();
        });

        // show the templates that have the same num. dim and aggs requirement
        $.ajax({
          url: `${arc.frontend.globals.urls.templateUrl}filtertemplate`,
          data: {
            'nDims': setup.dimPicked.length,
            'nAggs': setup.aggPicked.length
          },
          type: 'GET',
          beforeSend: function (xhr) {
            // Because of a jquery bug when handling PUT requests,
            // we have to explicitly set X-CSRFToken here
            xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
          },
        })
          .done(function (res) {
            paintModal(res.templates);
            $('#app-grid-list-template > li').removeClass('active');
            if (setup.active_report_id !== undefined) {
              $("#app-grid-list-template input[report-id=" +
                setup.active_report_id + "]").parent('li').addClass('active');
            }
            $("#app-grid-list-template div[data-toggle='popover']").popover({
              container: '.showme-modal',
              placement: 'bottom',
              trigger: 'hover',
              title: 'Template Description'
            });
            $('#app-grid-list-template > li').offOn('click.sameNumTemplatesActiveAppGridList', function () {
              if ($(this).attr("class") !== 'active') {
                $('#app-grid-list-template > li').removeClass('active');
                $(this).addClass('active');
              }
            });
            $('.showme-modal').find('.modal-body')
              .height(($(window).height() - 220) + 'px');
          })
          .fail(function (req, textstatus, errorthrown) {
            arc.report.alertMessage("Cannot load templates", req.responseText);
          });
      });
    };

    function create_params(param_name, dimPicked, aggPicked) {
      function build_param(picked_column, param_name) {
        var param_mapping = [];
        for (var i = 0; i < param_name.length; i++) {
          var value = picked_column[i];
          if (picked_column[i] === 'Record Count') {
            value = '_arc_record_count_';
          }
          param_mapping.push(`param.${param_name[i].name}=${value}`);
        }
        return param_mapping;
      }

      function get_prime_fields(fields) {
        /* creating a dictionary of columns used in heuristic functions.
           At this moment these columns are GEO and day
           we probably need to enhance this to be smarter
        */
        var prime_fields = {'geo':[], 'date':[]};
        for (let f of fields) {
          if (arc.geo.geoTypes.indexOf(f) !== -1) {
            prime_fields.geo.push(f);
          } else if (dataset.get_column(f).coltype_class() === 'type-timestamp')  {
            prime_fields.date.push(f);
          }
        }
        return prime_fields;
      }

      var url_params = [];
      var params = JSON.parse(setup.$active_li.find('input').attr('params'));

      // first we will assign geo and date fields
      // if these fields exist in the picked columns
      // assigning  fields
      var prime_fields = get_prime_fields(dimPicked);
      var key = Object.keys(params.columns)[0];
      Object.keys(prime_fields).forEach(function (p_key) {
        params.columns[key][p_key].forEach(function (geo) {
          var geo_dim = prime_fields[p_key].pop();
          if (geo_dim !== undefined) {
            url_params.push(`${url_params}param.${geo}=${geo_dim}`);

            /* removing the assigned field from the list of
            *  picked columns and param_name
            */
            dimPicked = dimPicked.filter(d => d !== geo_dim);
            param_name[key].dimensions =
              param_name[key].dimensions.filter(d => d.name !== geo);
          }

        });
      });

      Object.keys(param_name).forEach(function(key) {
        url_params = url_params.concat(build_param(aggPicked,
                                       param_name[key].aggregates));
        url_params = url_params.concat(build_param(dimPicked,
                                       param_name[key].dimensions));
      });
      return '?' + url_params.join('&');
    }

    $modal.find('#template-go-button').off('click').on('click', () => {
      if (setup.workflow === 1 || setup.workflow === undefined) {
        setup.$active_li = $('#app-grid-list-template > li.active');
        setup.active_report_id = setup.$active_li.find('input').attr('report-id');
      } else {
        setup.dimPicked = [];
        setup.aggPicked = [];
        $modal.find('#dim-shelf-template li').each(function () {
          setup.dimPicked.push($(this).text());
        });
        $modal.find('#msr-shelf-template li').each(function () {
          setup.aggPicked.push($(this).text());
        });
      }
      var params = JSON.parse(setup.$active_li.find('input').attr('params'));
      var dashboard_id = setup.$active_li.find('input').attr('report-id');
      var subs_params = create_params(params.columns,
                                      setup.dimPicked,
                                      setup.aggPicked);

      const ls = arc.misc.localStorage();
      ls.setItem('dash_template_columns',
      JSON.stringify({
        'dims': setup.dimPicked,
        'aggs': setup.aggPicked,
        'active_report_id': setup.active_report_id
      }));

      window.location.href = `${arc.frontend.globals.urls.viewDashboardUrl}/${
        dashboard_id
        }${subs_params
        }&arcparam.dssw.${
        params.report_dataset_id
        }=${setup.dataset_id}`;
    });

    return setup;
  };

  arc.report.loadTemplate.setupDrawBtns = function (dashboard_id) {
    $('a#publish-template-btn').offOn('click.publishTemplate', () => {
      $.ajax({
        url: `${arc.frontend.globals.urls.templateUrl}publishtemplate`,
        data: {"report_id": dashboard_id,
               "url_params":window.location.search},
        type: 'POST',
        beforeSend: function(xhr) {
          // Because of a jquery bug when handling PUT requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
        },
      })
      .done((id) => {
        window.location.href = `${arc.frontend.globals.urls.viewDashboardUrl}/${id}`;
      })
      .fail(function(req, textstatus, errorthrown) {
        arc.report.alertMessage("Cannot load templates", req.responseText);
      });
    });

    $('a#update-template-btn').offOn('click.updateTemplate', () => {
      const $modal = $('#showme-template-modal');
      if (arc.misc.modalShown($modal)) {
        return;
      }

      var s = arc.report.loadTemplate.setup();
      let selected_items = {};
      const ls = arc.misc.localStorage();
      selected_items = JSON.parse(ls.getItem('dash_template_columns'));

      s.dataset_id  = arc.globals.dssw[Object.keys(arc.globals.dssw)[0]];
      s.dimPicked = selected_items.dims;
      s.aggPicked = selected_items.aggs;
      s.active_report_id = selected_items.active_report_id;
      $modal.modal('show');
    });
  };

  $('#showme-template-modal .modal-content').draggable('disable');
  var removeItem = false;
  arc.misc.setupDimAggSearch('#showme-template-modal', '');
  $('#dim-shelf-template, #msr-shelf-template').sortable({
    connectWith: ".selected-list",
    over: function () {
      removeItem = false;
    },
    out: function () {
      removeItem = true;
    },
    beforeStop: function (event, ui) {
      if (removeItem) {
        ui.item.remove();
      }
    }
  }).disableSelection();
};

}());
