window.arc = window.arc || {};

(function() {
  "use strict";

  arc.report = arc.report || {};

  function historyCallouts() {
    const callouts = function() { return 'callouts'; };

    let dashboard;

    callouts.dashboard = function (_) {
      if (!arguments.length) return dashboard;
      dashboard = _;
      return callouts;
    };

    callouts.getPrimaryReport = function() {
      if (dashboard) {
        return dashboard.getPrimaryReport();
      }

      return undefined;
    };

    callouts.highlightMarks = function() {
      if (dashboard) {
        dashboard.highlightMarks();
      }
    };

    callouts.resetControls = function() {
      if (dashboard) {
        dashboard.resetControls();
      }
    };

    callouts.redraw = function(params_data, viz, updateFromParamHistory,
        prev_params_data, forward) {
      if (dashboard) {
        dashboard.redrawExistingWidgetsWithParamsData(params_data, viz, updateFromParamHistory,
            prev_params_data, forward);
      }
    };

    callouts.loadTitleSubtitle = function () {
      if (dashboard) {
        dashboard.loadTitleSubtitle();
      }
    };
    return callouts;
  }

  arc.report.params_history = function() {
    let stored_history = [],
        stored_history_pos = 0,
        tooltip,
        baseParamsl = [],
        $vizParamsControl,
        callouts = historyCallouts(),
        checkpoint;

    const params_history = function () {
      return "arc.report.params_history";
    };

    function state(pos = stored_history_pos) {
      if (pos === 0) {
        const param = stored_history.length ? stored_history[0][0] : baseParamsl;

        return [
          param
        ];
      }

      if (
        (pos > 0 && pos <= stored_history.length - 1)
      ) {
        return stored_history[pos] ?? [];
      }

      throw new Error(`Invalid stored history position ${pos}`);
    }

    function movePosBack() {
      if (stored_history_pos > 0) {
        stored_history_pos -= 1;
        return true;
      }

      return false;
    }

    function movePosForward() {
      if (stored_history_pos < stored_history.length - 1) {
        stored_history_pos += 1;
        return true;
      }

      return false;
    }

    function getPos() {
      return stored_history_pos;
    }

    function setPosToLast() {
      stored_history_pos = stored_history.length - 1;
    }

    function setPos(_) {
      stored_history_pos = _;
    }

    function resetPos() {
      stored_history_pos = 0;
    }

    params_history.util = {};

    params_history.getCheckPointState = function () {
      setPos(typeof checkpoint !== 'undefined' ? checkpoint : 0);
      return params_history.current_params();
    };

    params_history.setCheckPointState = function (isSave) {
      checkpoint = isSave ? 0 : getPos();
      return params_history;
    };

    params_history.util.getInfoFromParam = function (param) {
      if (arc.params.splitParams(param.key)[1] !== 'pivot') {
        return param;
      }

      try {
        const value = JSON.parse(param.value);
        return {
          filters: value.filters,
          columns: [
            {
              name: value.columnName,
              alias: value.columnAlias,
            }
          ]
        };
      } catch (e) {}
    };

    params_history.util.sort = function (listOfKeyNames) {
      listOfKeyNames = listOfKeyNames.slice(0);
      listOfKeyNames.sort((a, b) => {
        const a_arr = arc.params.splitParams(a);
        const b_arr = arc.params.splitParams(b);
        if (a_arr[1] === 'pivot' && b_arr[1] === 'pivot') {
          const vizId = getPivotKeyName(a_arr[0]);
          if (vizId === getPivotKeyName(b_arr[0])) {
            return a.substr(vizId.length + 1, a.length) - b.substr(vizId.length + 1, b.length);
          } else {
            return a - b;
          }
        } else {
          return a - b;
        }
      });
      return listOfKeyNames;
    };

    params_history.util.isParamPivotType = function (param) {
      try {
        const k_arr = arc.params.splitParams(param.key);
        return (k_arr[1] === 'pivot');
      } catch (e) {
        return false;
      }
    };

    params_history.util.filterRelevantParams = function (currentParams, report_id) {
      let relevantParams = [];
      try {
        if (!report_id) {
          report_id = arc.report.getPrimaryReport().report_id();
        }
      } catch (e) {}

      if (!currentParams || !report_id) {
        return relevantParams;
      }

      relevantParams = currentParams.filter((cp) => {
        const k_arr = arc.params.splitParams(cp.key);
        if (k_arr[1] === 'pivot') {
          return (Number(getPivotKeyName(k_arr[0])) === Number(report_id));
        }

        return true;
      });

      return relevantParams;
    };

    params_history.currentParamObjects = function() {
      return arc.params.convertOldParamArrToNewParamObjects(
        params_history.current_params_optimized()[0] || []
      );
    };

    params_history.current_params = function() {
      return state();
    };

    params_history.current_params_optimized = function() {
      const s = state();
      let params = s[0];
      if (params) {
        params = makeSmaller(params);
      }

      return [params, s[1]];
    };

    params_history.getSerializedParams = function() {
      return arc.params.stringifyParamObjects(params_history.currentParamObjects());
    };

    params_history.setSerializedParams = function(serializedParams='[]') {
      try {
        const params = arc.params.convertToOldFormatArr(
          arc.params.parseParamObjects(serializedParams)
        );
        params_history.add(params, null, false, true);
      } catch (e) {
        console.error('Invalid parameters passed.', e.message);
      }
    };

    function formatTitle(origTitle) {
      let newTitle = '[';
      if (origTitle.length > 15) {
        newTitle += `${origTitle.substr(0, 12)}...`;
      } else {
        newTitle += origTitle;
      }
      newTitle += ']';
      return newTitle;
    }

    params_history.tooltip_text = function (params) {
      function getAllParamsToBeShown() {
        const paramsData = params[0];
        const segmentDebug = false;
        const pivotDebug = false;
        const dateRangeDebug = false;

        /**
         * To see .ignore in param history, set params_to_be_hidden_from_user = ""
         * To not see .exclude, add "exclude" to array below.
         * Known key extensions are:
         * ignore
         * exclude
         * pivot
         * config
         * include_nulls
         */
        const params_to_be_hidden_from_user = new Set(['ignore']);
        const paramKeysToHide = new Set();
        if (!pivotDebug) {
          params_to_be_hidden_from_user.add('pivot');
        }

        if (paramsData.length === 1 && paramsData[0].key.split('.')[1] === 'include_nulls' && paramsData[0].value === 'true') {
          params_to_be_hidden_from_user.add('include_nulls');
        }

        if (!segmentDebug) {
          paramKeysToHide.add(`${arc.report.segment_display_name}.data`);
          paramKeysToHide.add(`${arc.report.segment_display_name}`);
        }

        if (!dateRangeDebug) {
          params_to_be_hidden_from_user.add('config');
        }

        return paramsData.filter((param) => {
          if (paramKeysToHide.has(param.key)) {
            return false;
          }

          const keyArr = arc.params.splitParams(param.key);
          return (!params_to_be_hidden_from_user.has(keyArr[1]));
        }).sort((a, b) => a.key.localeCompare(b.key));
      }

      const paramsToShow = getAllParamsToBeShown();
      const textl = [];

      const templateParam =
        `<div class="fil-cont">
          <div class="fil-header">
              %PARAM_NAME% %HTML_PARAM_COUNT%
             <hr>
          </div>
          <div class="fil-body">
            %HTML_PARAM_VALUES%
          </div>
        </div>`;

      const templateParamValue = `<span class="fil-cat">%PARAM_VALUE%</span>`;

      paramsToShow.forEach((param) => {
        let valuesHTML;
        const keyArr = arc.params.splitParams(param.key);

        const paramTooltipObj = {
          PARAM_NAME: param.key,
          HTML_PARAM_VALUES: '',
          HTML_PARAM_COUNT: '',
        };

        if (['data', 'alias'].includes(keyArr[1])) {
          let excludeParam = paramsToShow.filter(param => param.key === `${keyArr[0]}.exclude`);
          excludeParam = excludeParam && excludeParam[0] || {};
          valuesHTML = param.value_list.map(
            value => getHtmlParamValue(value, excludeParam.value === 'not in')).join('');
          paramTooltipObj.HTML_PARAM_VALUES = valuesHTML;
          paramTooltipObj.HTML_PARAM_COUNT = getHtmlParamCount(param, paramsToShow);
        } else if (['start'].includes(keyArr[1])) {
          const endDateParam = paramsToShow.filter(p => p.key === `${keyArr[0]}.end`)[0] || param;
          valuesHTML = getHtmlParamValue(param.value) + getHtmlParamValue('-') +
                       getHtmlParamValue(endDateParam.value);
          paramTooltipObj.HTML_PARAM_VALUES = valuesHTML;
          paramTooltipObj.PARAM_NAME = `${param.key} - ${keyArr[0]}.end`;
        } else if (!['end'].includes(keyArr[1])) {
          valuesHTML = getHtmlParamValue(param.value);
          paramTooltipObj.HTML_PARAM_VALUES = valuesHTML;
        } else {
          return;
        }

        textl.push(
          arc.misc.safe_substitute(templateParam, paramTooltipObj)
        );
      });

      return textl.join('') || 'No parameters';
    };

    function getHtmlParamValue(value, isExcluded) {
      let excludeClass = '';
      if (isExcluded) {
        excludeClass = ' excluded-param';
      }

      const templateParamValue = `<span class="fil-cat${excludeClass}">%PARAM_VALUE%</span>`;
      return arc.misc.safe_substitute(templateParamValue,
              {PARAM_VALUE: arc.misc.d_string(value)});
    }

    function getHtmlParamCount(param, paramsToShow) {
      const keyArr = arc.params.splitParams(param.key);
      const excludeParam = paramsToShow.filter(p => p.key === `${keyArr[0]}.exclude`)[0];
      const excludeValue = excludeParam && excludeParam.value === 'not in' ? 'excluded' : '';
      let htmlCountValue = '';
      if (['data', 'alias'].includes(keyArr[1])) {
        if (param.value_list && param.value_list.length > 1) {
          htmlCountValue = `(${param.value_list.length}${excludeValue ? ' ' + excludeValue : ''})`;
        } else if (excludeValue) {
          htmlCountValue = `(${excludeValue})`;
        }
      }

      return htmlCountValue;
    }

    function tooltipTextFn(d, i, elem) {
      let textToShow = '';
      const $elem = $(elem).closest('span');
      const isTooltipRedraw = arguments.length === 0;

      if ($elem.hasClass('viz-params-link')) {
        textToShow = 'Link to this page';
      } else if ($elem.hasClass('viz-params-clear')) {
        textToShow = 'Clear Time Selection';
      } else if ($elem.hasClass('viz-params-refresh')) {
        textToShow = 'Reset (Ctrl + R)';
      } else if ($elem.hasClass('viz-params-backward')) {
        textToShow = 'Back (Ctrl + B)';
      } else if ($elem.hasClass('viz-params-forward')) {
        textToShow = 'Forward (Ctrl + F)';
      } else if ($elem.hasClass('viz-params-display') || isTooltipRedraw) {
        textToShow = `<h5>Current Parameters</h5><hr>
         <div class="fil-summary-body">
           ${params_history.tooltip_text(params_history.current_params())}
         </div>`;
      }

      return textToShow;
    }

    params_history.setupControls = function (inModal) {
      const selectorSpecificity = inModal ? '.inmodal' : ':not(.inmodal)';
      const controlSelector = `.viz-params-control${selectorSpecificity}`;
      $vizParamsControl = $(controlSelector);
      $vizParamsControl.removeClass('arc-invisible');

      const btns = d3.select($vizParamsControl[0]).selectAll('.viz-params-btn');
      if (!arc.globals.embedded && arc.globals.is_dashboard && !inModal) {
        $vizParamsControl.find('.viz-params-link').removeClass('arc-invisible');
      }

      tooltip =
        arc.tooltip()
          .dom_id(`${controlSelector} .freezableTooltip`)
          .addSelection(btns)
          .text_fn(tooltipTextFn)
          .getXPos(() => {
            // position of your click, shifted over by the dashboard position,
            // tooltip, and width of the icon with 10px padding
            let marginForCurrentParams = 80;
            if ($(d3.event.currentTarget).hasClass('viz-params-display')) {
              const tooltipText = tooltipTextFn(undefined, undefined, d3.event.currentTarget) || '';
              marginForCurrentParams = (tooltipText.length - 200) / 4;
              marginForCurrentParams = Math.min(marginForCurrentParams, 200);
              marginForCurrentParams = Math.max(marginForCurrentParams, 110);
            }

            const iconWidth = $vizParamsControl.find('.viz-params-display').width() + 10;
            return d3.event.pageX - $vizParamsControl.offset().left -
              iconWidth - marginForCurrentParams;
          })
          .getYPos(() => {
            const iconHeight = 10;
            return d3.event.pageY - $vizParamsControl.offset().top + iconHeight;
          })
          .activate();

      arc.report.shorturl.setup();
      // showLink function and all the url shortener function
      // moved to arc_report_shorturl.js
      $vizParamsControl.find('.viz-params-link')
                       .off('click.showLink')
                       .on('click.showLink', () => {
                          arc.report.shorturl.openModal();
                       });

        btns.on('click.param', function handleClickParam() {
          const evt = d4.event || d3.event;
          const $target = $(evt.target).closest('span');

          if ($target.hasClass('disabled')) {
            return;
          }

          if ($target.hasClass('viz-params-refresh')) {
            params_history.clear();
          } else if ($target.hasClass('viz-params-backward')) {
            params_history.back();
          } else if ($target.hasClass('viz-params-forward')) {
            params_history.forward();
          } else if ($target.hasClass('viz-params-display')) {
            if (!tooltip.isFrozen()) {
              evt.stopPropagation();
              tooltip.freeze();
            }
          }
        });

      arc.shortcut.remove('Ctrl+R');
      arc.shortcut.add('Ctrl+R', params_history.clear);

      arc.shortcut.remove('Ctrl+B');
      arc.shortcut.add('Ctrl+B', params_history.back);

      arc.shortcut.remove('Ctrl+F');
      arc.shortcut.add('Ctrl+F', params_history.forward);

      return params_history;
    };

    params_history.destroy = function () {
      if ($vizParamsControl) {
        $vizParamsControl.addClass('arc-invisible');
      }
    };

    params_history.hideControls = function () {
      if ($vizParamsControl) {
        $vizParamsControl.addClass('arc-invisible');
      }
    };

    params_history.setVisibility = function () {
      if (!$vizParamsControl) {
        return;
      }

      const $vizParamsRefresh = $vizParamsControl.find('.viz-params-refresh');
      const $vizParamsBackward = $vizParamsControl.find('.viz-params-backward');
      const $vizParamsForward = $vizParamsControl.find('.viz-params-forward');

      if (arc.globals.is_edit) {
        $vizParamsBackward.addClass('arc-invisible');
        $vizParamsForward.addClass('arc-invisible');
      }

      tooltip.text_fn(tooltipTextFn);

      if (tooltip.isFrozen()) {
        tooltip.redraw();
      }

      const pos = getPos();

      if (stored_history.length === 0) {
        $vizParamsRefresh.addClass('disabled');
        $vizParamsBackward.addClass('disabled');
        $vizParamsForward.addClass('disabled');
        return params_history;
      }

      if (stored_history.length === 1) {
        $vizParamsRefresh.addClass('disabled');
      } else {
        $vizParamsRefresh.removeClass('disabled');
      }

      if (pos > 0) {
        $vizParamsBackward.removeClass('disabled');
      } else {
        $vizParamsBackward.addClass('disabled');
      }

      if (pos < stored_history.length - 1) {
        $vizParamsForward.removeClass('disabled');
      } else {
        $vizParamsForward.addClass('disabled');
      }

      return params_history;
    };

    params_history.baseParamsl = function (_) {
      if (!arguments.length) return baseParamsl;
      baseParamsl = _;
      params_history.setVisibility();
      return params_history;
    };

    params_history.dashboard = callouts.dashboard;

    params_history.clear = function () {
      params_history.paramsHistoryReset();

      return params_history;
    };

    params_history.paramsHistoryReset = function () {
      const prev_params_data = state()[0];
      const defState = state(0);

      resetPos();

      if (arc.globals.is_dashboard) {
        callouts.resetControls();
      }

      callouts.redraw(defState[0], null, true, prev_params_data, false);
      stored_history = [defState];

      params_history.setVisibility();
      callouts.highlightMarks();

      return params_history;
    };

    function paramKvs(pl) {
      const ignoreKeys = {};

      pl?.forEach((p) => {
        const k_arr = arc.params.splitParams(p.key);
        const k = k_arr[0];

        if (k_arr[1] === 'ignore') {
          ignoreKeys[k] = true;
          return;
        }

        ignoreKeys[k] = false;
      });

      const kvs = {};
      pl?.forEach((p) => {
        const k_arr = arc.params.splitParams(p.key);
        if (k_arr[1] === 'ignore') {
          return;
        }

        const k = k_arr[0];
        if (ignoreKeys[k]) {
          return;
        }

        kvs[p.key] = p;
      });

      return kvs;
    }

    params_history.is_identical = function (pl1, pl2) {
      const kvs1 = paramKvs(pl1);
      const kvs2 = paramKvs(pl2);
      return arc.misc.isIdentical(kvs1, kvs2);
    };

    function getPivotKeyName(keyname) {
      let dashIndex = keyname.indexOf('-');
      let prefix = '';
      if (dashIndex === 0) {
        // If it is a new report id
        prefix = keyname[0];
        keyname = keyname.substring(1, keyname.length);
        dashIndex = keyname.indexOf('-');
      }

      if (dashIndex > 0) {
        return prefix + keyname.substring(0, dashIndex);
      }
      return prefix + keyname;
    }

    /**
     *
     * @desc  function is supposed to return a union of current and new params.
     *        All current_params should be included other then:
     *          - those whose source visual has emitted updated params
     *          - those whose output paramname is same, and value maybe updated by same/diff source.
     * @param current_params - params that existed before user emitted new params.
     * @param new_params - just the params being emitted out due to current action.
     * @return pl - combined params overriding current params with same op name with new params.
     *
     */
    params_history.update = function (new_params, current_params) {
      let paramsList = [];
      const listOfNewParamsOutputParameterRoots = {};
      const listOfNewParamsSourceVisualIds = {};
      new_params.forEach(function populateAboveLsits(param) {
        const keyArr = arc.params.splitParams(param.key);
        const paramExtension = keyArr[1];
        const outputParamRoot = keyArr[0];
        if (outputParamRoot) {
          listOfNewParamsOutputParameterRoots[outputParamRoot] = true;
        }

        if (typeof param.source_visual_id !== 'undefined') {
          listOfNewParamsSourceVisualIds[param.source_visual_id] = true;
        }
      });

      const filteredCurrentParams = current_params?.filter(function paramsNotInAboveLists(param) {
        const keyArr = arc.params.splitParams(param.key);
        const outputParamRoot = keyArr[0];
        return !listOfNewParamsOutputParameterRoots[outputParamRoot] &&
               !listOfNewParamsSourceVisualIds[param.source_visual_id];
      }) ?? [];

      paramsList = filteredCurrentParams.concat(new_params);
      return makeSmaller(paramsList);
    };

    function copyParams(params_data) {
      const paramsDataCopy = [];
      params_data?.forEach((param_data) => {
        paramsDataCopy.push({
          key: param_data.key,
          value: param_data.value,
          value_list: param_data.value_list,
          keytype: param_data.keytype,
          valuetype: param_data.valuetype,
          source_visual_id: param_data.source_visual_id,
          source_dataset_id: param_data.source_dataset_id,
          isDatasetSegment: param_data.isDatasetSegment,
        });
      });

      return paramsDataCopy;
    }

    function makeSmaller(params) {
      const lastPositions = {};

      params.forEach((p, i) => {
        lastPositions[p.key] = i;
      });

      const allLastPositions = {};
      $.each(lastPositions, (k, v) => {
        allLastPositions[v] = true;
      });

      const outParams = [];
      params.forEach((p, i) => {
        if (i in allLastPositions) {
          outParams.push(p);
        }
      });

      return outParams;
    }

    params_history.add = function (params_data, viz, dontRefresh, isForceRefreshSourceVisual) {
      if (!params_data) {
        return params_history;
      }

      const last_store = params_history.current_params() || [[]];
      let new_params = false;
      let prev_params_data;

      prev_params_data = last_store || [[]];

      params_data = params_history.update(params_data, last_store[0]);

      if (!params_history.is_identical(last_store[0], params_data)) {
        new_params = true;
      }

      if (new_params) {
        if (arc.globals.ms_dashboard) {
          if (arc.globals.ms_dashboard.beforeParamDistributionCbs) {
            let paramObjects = arc.params.convertOldParamArrToNewParamObjects(params_data);
            $.each(arc.globals.ms_dashboard.beforeParamDistributionCbs, (hookName, hook) => {
              try {
                const config = hook(paramObjects.slice(0), dontRefresh);
                if (config.paramObjects) {
                  paramObjects = config.paramObjects.slice(0);
                }

                if (typeof config.doNotRefresh !== 'undefined') {
                  dontRefresh = newDoNotRefresh;
                }
              } catch (e) {
                console.err(e);
              }
            });

            params_data = arc.params.convertToOldFormatArr(paramObjects);
          }
        }

        // deleting any history after current state
        stored_history.splice(getPos() + 1);

        stored_history.push([params_data]);
        if (stored_history.length > 40) {
          // Remove the first few elements of history once we get to tracking more than
          // the max number of elements
          stored_history.splice(0, stored_history.length - 25);
        }

        setPosToLast();

        if (!dontRefresh) {
          callouts.redraw(
            copyParams(params_data), viz, isForceRefreshSourceVisual,
                       copyParams(prev_params_data[0]));
        }
        params_history.setVisibility();
        if (arc.globals.parameters) {
          arc.globals.parameters.checkParamsAndHide(params_data);
        }

        // Only update title if in view mode
        if (!arc.globals.is_edit) {
          callouts.loadTitleSubtitle();
        }
      }

      return params_history;
    };

    params_history.back = function () {
      let prev_history_item,
          new_history_item;

      prev_history_item = state()[0];
      if (movePosBack()) {
        new_history_item = state()[0];
        callouts.redraw(
          new_history_item, undefined, true, prev_history_item, false);
      }

      params_history.setVisibility();
      callouts.highlightMarks();

      return params_history;
    };

    params_history.forward = function () {
      let prev_history_item,
          new_history_item;

      prev_history_item = state()[0];
      if (movePosForward()) {
        new_history_item = state()[0];
        callouts.redraw(
          new_history_item, undefined, true, prev_history_item, true);
      }

      params_history.setVisibility();
      callouts.highlightMarks();

      return params_history;
    };

    return params_history;
  };
}());
