window.arc = window.arc || {};

(function() {
  'use strict';

  arc.report = arc.report || {};

  const shorturl = function () {
      return 'arc.report.shorturl';
  };

  arc.report.shorturl = shorturl;

  const createListOfExistingUrlsHTML = (originUrl, rowData) => {
    $('.existingUrlsCount').text(rowData.length);
    if (rowData.length === 0) {
      $('#existing-short-urls').hide();
      $('.existing-urls-message').show();
    } else {
      const existingUrls = rowData.map((row) => {
        const slug = arc.misc.escapeHTML(row.slug);
        const isCreatedFromEmail = row.is_created_in_email;

        return `
        <div class='existing-url-wrapper d-flex align-items-center'>
          <div class="flex-grow-1">
            ${originUrl}<span class='font-weight-bold existing-slug'>${slug}</span>
            <a class="existing-url" href='${originUrl}${slug}' target='_blank'>
              <i class="fa fa-external-link" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="Open in new tab"></i>
            </a>
          </div>
          <div class="existing-urls-control-${slug} d-flex align-items-center">
            <div class="p-2" style="display:${isCreatedFromEmail ? 'block' : 'none'}">
              <i class="fa fa-envelope-o" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="The alias has been created/used in an email"></i>
            </div>
            <div id="btn-${slug}" class="p-2 viz-params-display" style="display:${row.parametersLength ? 'block' : 'none'}"><i class="fa fa-filter" aria-hidden="true"></i></div>
            <div class="p-2">
              <a>
                <i class="fa fa-trash delete-alias" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="Delete URL alias"></i>
              </a>
            </div>
            <div class="p-2">
              <a>
                <i class="fa fa-files-o existing url-copy-btn" aria-hidden="true" data-toggle="tooltip" data-placement="top" title="Copy to clipboard"></i>
              </a>
            </div>
          </div>
        </div>`;
      });

      $('#existing-short-urls').css('overflow-y', (rowData.length >= 3) ? 'scroll' : '');

      $('#existing-short-urls').show();
      $('.existing-urls-message').hide();
      $('#existing-short-urls').html(existingUrls);

      rowData.forEach((row) => {
        const parameters = Object.keys(row.parameters)
          .filter((key) => key !== 'scope')
          .reduce((cur, key) => {
            const value = row.parameters[key];
            cur.push({ key, value, value_list: [value] });
            return cur;
        }, []);
        const btn = d3.select(`#btn-${row.slug}`);
        const tooltip = arc.tooltip()
          .dom_id(`#filter-params.freezableTooltip`)
          .addSelection(btn)
          .text_fn(() => `<h5>Current Parameters</h5><hr>
            <div class="fil-summary-body">
              ${arc.report.params_history().tooltip_text([parameters])}
            </div>`)
          .getXPos(() => $(`#btn-${row.slug}`).offset().left - $('#url-modal-aliases').offset().left - $('#filter-params').width() - $(`#btn-${row.slug}`).width())
          .getYPos(() => $(`#btn-${row.slug}`).offset().top - $('#url-modal-aliases').offset().top + $(`#btn-${row.slug}`).height())
          .activate();

          btn.on('click', () => {
            const evt = d4.event || d3.event;
            if (!tooltip.isFrozen()) {
              evt.stopPropagation();
              tooltip.freeze();
            }
          });
        });

      $('existing-urls-message').hide();
    }
  };

  const deleteAlias = (slug) => {
    arc.frontend.vizHttp.delete({
      url: arc.frontend.globals.urls.urlShortenerUrl,
      data: { slug },
      headers: {
        'X-CSRFTOKEN': arc.globals.csrf_token,
      },
    })
    .then(shorturl.loadExistingURLAliases)
    .then((aliases) => {
      createListOfExistingUrlsHTML(shorturl.getOrigin(), aliases);
    })
    .catch((error) => {
      arc.report.alertMessage(error);
    });
  };

  const updateAliasCopyUrlTooltipText = (text) => {
    const aliasCopyBtn = '.short.url-copy-btn[data-toggle="tooltip"]';

    $(aliasCopyBtn).attr('data-original-title', text);
    $(aliasCopyBtn).tooltip('update');
  };

  const closeURLShortenerSection = () => {
    const saveBtn = $('#save-url-alias');
    const urlShortenerContainer = $('#url-shortener');

    saveBtn.hide();
    saveBtn.prop('disabled', false);
    $('#url-shortener-btn').slideDown();
    urlShortenerContainer.slideUp();
    urlShortenerContainer.find('.text-danger').hide();
    urlShortenerContainer.find('.text-info').show();
    updateAliasCopyUrlTooltipText('The alias is not active yet - please save to copy');
    $('.short.url-copy-btn[data-toggle="tooltip"]').tooltip('enable');
  };

  shorturl.typeAndAssetIdFinder = function() {
    const type = arc.globals.appgroup ? 'apps' : 'dashboard';
    const id = type === 'apps' ? arc.globals.appgroup.appgroup_id() : arc.globals.dashboard.dashboard_id();
    const shortUrlAssetId = parseInt(id, 10);

    return { type, shortUrlAssetId };
  };

  shorturl.displayURLAlias = function({ path, slug }) {
    $('.short-url-origin').text(path || '');
    $('.short-url-slug').val(slug || '');
    $('#URLAlias').attr('data-url', slug);
  };

  shorturl.getOrigin = () => `${window.location.origin}${arc.frontend.globals.urls.urlShortenerUrl}`;
  shorturl.getSlug = () => {
    const { randomUUID } = new window.ShortUniqueId({ length: 10 });
    const shortenedURLSlug = arc.misc.safe_substitute(randomUUID());

    return shortenedURLSlug;
  };

  shorturl.saveAlias = (parameters, slug, isCreatedInEmail) => {
    const { shortUrlAssetId, type } = shorturl.typeAndAssetIdFinder();
    const $urlShortener = $('#url-shortener');

    return arc.frontend.vizHttp.post({
      url: arc.frontend.globals.urls.urlShortenerUrl,
      data: { dashboardId: shortUrlAssetId, type, parameters, slug, isCreatedInEmail },
      headers: {
        'X-CSRFTOKEN': arc.globals.csrf_token,
      },
    })
      .then(() => {
        $urlShortener.find('.short.url-copy-btn').removeClass('disabled');
        $('#save-url-alias').prop('disabled', true);
        updateAliasCopyUrlTooltipText('Copy to clipboard');
        shorturl.loadExistingURLAliases().then((aliases) => {
          createListOfExistingUrlsHTML(shorturl.getOrigin(), aliases);
        });
      })
      .catch(() => {
        $urlShortener.find('.text-danger').html('This alias is already being used. Please change in order to save.');
        $urlShortener.find('.text-danger').show();
        $urlShortener.find('.text-info').hide();
      });
  };

  // Load existing URLs from DB
  shorturl.loadExistingURLAliases = () => {
    const id = shorturl.typeAndAssetIdFinder().shortUrlAssetId;

    return arc.frontend.vizHttp.get({
      url: arc.frontend.globals.urls.urlShortenerUrl,
      params: { dashboardId: id },
      headers: {
        'X-CSRFTOKEN': arc.globals.csrf_token,
      },
    })
    .then((result) => result.data)
    .catch((error) => {
      arc.report.alertMessage(error);
    });
  };

  shorturl.fillDropdownOfExistingUrlsHTML = (rowData) => {
    const dropDown = $('#existing-alias-container');
    const existingAliases = rowData.map((row) => `
      <option>${row.slug}</option>
    `);

    dropDown.children('option:not(:first)').remove();
    dropDown.append(existingAliases);
  };

  shorturl.urlGenerator = () => {
    const { type, shortUrlAssetId } = shorturl.typeAndAssetIdFinder();
    const data = [];
    arc.misc.getURLParams(data, arc.globals.dashboard.current_params()[0]);
    const appOrDashboardUrl = type === 'dashboard' ? arc.frontend.globals.urls.viewDashboardUrl : arc.frontend.globals.urls.viewAppUrl;
    const urlPath = `${window.location.origin}${appOrDashboardUrl}/${shortUrlAssetId}`;
    const url = arc.misc.getBlankTargetUrl(arc.misc.dedupURLParams(data, urlPath));

    return url;
  };

  shorturl.openModal = function() {
    const url = shorturl.urlGenerator();

    const $initial_url = $('#link-modal-link');
    const $modal = $('#link-modal');

    $modal.on('show.bs.modal', function () {
      $(this).find('.nav a:first').tab('show');
    });

    if (arc.misc.modalShown($modal)) {
      return;
    }

    $modal.modal('show').offOn('shown.bs.modal', () => {
      $initial_url.val(url);
      shorturl.loadExistingURLAliases().then((aliases) => {
        createListOfExistingUrlsHTML(shorturl.getOrigin(), aliases);
      });
    });

    $modal.offOn('hide.bs.modal', () => {
      closeURLShortenerSection();
    });

    shorturl.setup(url);
  };

  shorturl.setup = function(url) {
    const origin = shorturl.getOrigin();
    const $modal = $('#link-modal');

    let $urlToCopy;

    const $urlShortenerSection = $('#url-shortener');
    const includeParamsBtn = $('#includePageParametersShortUrl');

    // copy to clipboard
    $modal.offOn('click.copyToClipboard', '.url-copy-btn:not(".disabled")', (evt) => {
      const $target = $(evt.target);
      const defaultUrl = $target.hasClass('origin');
      const existingUrl = $target.hasClass('existing');

      if (defaultUrl) {
        $urlToCopy = $('#link-modal-link').val();
      } else if (existingUrl) {
        const $existingUrlLinkWrapper = $target.closest('.existing-url-wrapper');
        const $existingUrlLink = $existingUrlLinkWrapper.find('.existing-url');
        $urlToCopy = $existingUrlLink.attr('href');
      } else {
        const $shortSlug = $('#url-shortener .short-url-slug').val();
        $urlToCopy = origin + $shortSlug;
      }

      arc.misc.copyToClipboard($urlToCopy).then(() => {
        $target.attr('data-original-title', 'Copied');
        $target.tooltip('update');
        $target.tooltip('show');

        setTimeout(() => {
          $target.attr('data-original-title', 'Copy to clipboard');
          $target.tooltip('update');
        }, 300);
      });
    });

    $urlShortenerSection.find('.short-url-slug').on('change keyup paste', (evt) => {
      const noSpecialCharOrSpaceAllowed = /^[a-zA-Z0-9_-]*$/;
      const $this = $(evt.target);

      if (!noSpecialCharOrSpaceAllowed.test($this.val())) {
        $urlShortenerSection.find('.text-danger').html('Alias can contain any of the followings: A-Z a-z 0-9 _ - only!');
        $urlShortenerSection.find('.text-danger').show();
        $urlShortenerSection.find('.text-info').hide();
        $('#save-url-alias').prop('disabled', true);
        return;
      }

      $('#save-url-alias').prop('disabled', false);
      $urlShortenerSection.find('.short.url-copy-btn').addClass('disabled');
      updateAliasCopyUrlTooltipText('The alias is not active yet - please save to copy');
      $urlShortenerSection.find('.text-danger').hide();
      $urlShortenerSection.find('.text-info').show();
    });

    // save short URL in DB
    $modal.offOn('click.saveURLShortener', '#save-url-alias', () => {
      const isChecked = includeParamsBtn.prop('checked');
      const parameters = (isChecked) ? url.split('?')[1] : '';
      const slug = $modal.find('.short-url-slug').val();

      if (slug === '') {
        $urlShortenerSection.find('.text-danger').html('Please enter an alias');
        $urlShortenerSection.find('.text-danger').show();
        $urlShortenerSection.find('.text-info').hide();
        return;
      }

      shorturl.saveAlias(parameters, slug, false);
    });

    // generate short URL
    $modal.offOn('click.openURLShortener', '#generate-url-alias', () => {
      shorturl.displayURLAlias({ path: origin, slug: shorturl.getSlug() });

      $urlShortenerSection.find('.short.url-copy-btn').addClass('disabled');
      $('#url-shortener-btn').slideUp();
      $urlShortenerSection.slideDown();
      $('#save-url-alias').show();
      updateAliasCopyUrlTooltipText('The alias is not active yet - please save to copy');
      $('.short.url-copy-btn[data-toggle="tooltip"]').tooltip('enable');
    });

    // tab controller
    $('.url-modal ul.nav-tabs a').offOn('shown.bs.tab', function (event) {
      event.preventDefault();
      $(this).tab('show');

      // close URL Shortener section on tab switch
      if ($(this).hasClass('nav-tab-aliases')) {
        closeURLShortenerSection();
      }
    });

    // hide delete icon tooltip on click
    // solution for the tooltip lingering
    $modal.offOn('click.hideDeleteTooltip', '[data-toggle="tooltip"]', (evt) => {
      $(evt.target).tooltip('hide');
    });

    // delete exisiting URL from DB
    $modal.offOn('click.deleteAlias', '.delete-alias', (evt) => {
      const slug = $(evt.target).closest('.existing-url-wrapper').find('.existing-slug').html();
      deleteAlias(slug);
    });
  };
}());
