window.arc = window.arc || {};

(function() {
'use strict';

arc.testutil = arc.testutil || {};
arc.report = arc.report || {};

arc.report.getPrimaryReport = function() {
  const report = arc.globals.reports.getPrimaryReport();
  return report;
};

arc.report.getPrimaryViz = function() {
  return arc.report.getPrimaryReport().viz();
};

arc.reportbuilder = function() {
  var loader,
      dataset,
      dataset_server_info,
      dataset_cb,
      dataset_id,
      dataset_events = arc.report.events.dataset_events(),
      dataset_segments = arc.report.dataset_segments(),
      dataset_filterassociations = arc.report.filterassociations.dataset_filterassociations(),
      visual_type,
      dashboard,
      deriveddata,
      displayapp,
      isFirstSave,
      unique_chartid = 0,
      $hideIfNewNavbarItems;

  const visualMenu = arc.menu()
                        .containerId('#visual-submenu');

  var builder = function() { };

  function hideContextMenus() {
    $('.contextmenu').hide();
    arc.menu_hideAll();
  }

  function hideTooltipsMenu() {
    $('#tooltip').fadeOut();
    visualMenu.hide();
  }

  builder.visual_type = function(_) {
    if (!arguments.length) return visual_type;
    visual_type = _;
    return builder;
  };

  builder.segments_dom_id = function(_) {
    if (!arguments.length) return dataset_segments.segments_dom_id();
    dataset_segments.segments_dom_id(_);
    return builder;
  };

  builder.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    builder.store_dataset_server_info(dataset);
    dataset_segments.dataset(dataset);
    dataset_events.dataset(dataset);
    return builder;
  };

  /**
   * Invoke it with the dataset loaded from server.
   * Will store column alias and name info as known by the server.
   * Used by column editor to figure out whether an edited column is known
   * by the server.
   */
  builder.store_dataset_server_info = function(server_dataset) {
    const column_aliases_by_fqn = {};

    if (!server_dataset) {
      dataset_server_info = undefined;
      return;
    }

    dataset_server_info = {
      column_aliases_by_fqn
    };

    for (const table of server_dataset.tables()) {
      const tablename = table.tablename();
      for (const column of table.columns()) {
        const alias = column.display_name();
        const column_name = column.attr('name');
        if (column_name) {
          const column_fqn = `${tablename}.${column_name}`;
          column_aliases_by_fqn[column_fqn] = alias;
        }
      }
    }

    return builder;
  };

  builder.get_dataset_server_info = function() {
    return dataset_server_info;
  };

  builder.dataset_cb = function(_) {
    if (!arguments.length) return dataset_cb;
    dataset_cb = _;
    return builder;
  };

  builder.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return builder;
  };

  builder.loader = function(_) {
    if (!arguments.length) return loader;
    loader = _;
    return builder;
  };

  builder.displayapp = function(_) {
    if (!arguments.length) return displayapp;
    displayapp = _;
    return builder;
  };

  builder.redraw = function(isWindowResize) {
    var report = arc.report.getPrimaryReport();
    report.redraw(isWindowResize);
    return builder;
  };

  builder.setupEditDataset = function() {
    d3.select('#app-dataset-edit')
      .on('click.editdataset', function() {
        arc.misc.openInNewWindow(`${arc.frontend.globals.urls.datasetEditUrl}/${dataset_id}`);
      });
  };

  builder.setupSample = function() {
    $('#dataset-sample-apply').offOn('click.applySample', function () {
      const driver = arc.report.getPrimaryViz();
      const newSamplePercentage = $('#dataset-sample-selection').val().trim();
      if (driver) {
        driver.setSamplingPct(newSamplePercentage);
      } else {
        arc.report.setSamplingPct(newSamplePercentage);
      }

      arc.report.pagenav.needs_save(true);
    });

    $('#dataset-sample-display').offOn('click.dataset-sample-set', function() {
      $('#dataset-sample-selection').val($('#dataset-sample-display').text());
    });
  };

  function activateBuilder(cb) {
    // In build mode, setup menus
    if (arc.globals.is_edit) {
      if (!(arc.testutil && arc.testutil.derived_data)) {
        d4.queue()
          .defer(arc.colorpalette.fetch)
          .defer(arc.dateranges.fetch)
          .awaitAll(function() {
            if (cb) {
              cb();
            }
          });
      }

      builder.setupMenu();
      builder.setupEditDataset();
      builder.setupSample();
      builder.setupVisualDropdownButtons($('.modal-button-bar'));
    } else {
      builder.setupMenu();
      builder.setupVisualDropdownButtons($('.modal-button-bar'));
    }

    return builder;
  }

  builder.setupAndDrawReport = function(report) {
    report = report || arc.report.getPrimaryReport();
    report.dataset_segments(dataset_segments)
          .dataset_events(dataset_events)
          .save_segment_cb((segment_data, done_cb) => {
            dataset_segments.saveSegment(segment_data, () => {
              dataset_segments.reloadSegments(done_cb);
            });
          });

    if (report.dataset_id()) {
      loadDataset(report.dataset_id());
    } else if (report.report_id()) {
      builder.loadReport(report.report_id());
    }

    builder.setupVisualDropdownButtons($('#subnav-report'));

    return builder;
  };

  function clearDatasetInfo() {
    dataset = undefined;
    dataset_server_info = undefined;
    dataset_cb = undefined;
    dataset_id = undefined;
  }

  builder.firstVisual = function(visual_id, visual_dataset_id, chart_id) {
    // Setup the report to display/edit
    var report = arc.report.getPrimaryReport();
    report.report_id(visual_id)
          .dataset_id(visual_dataset_id)
          .chart_div(chart_id)
          .shelves_dom_id('app-shelves');

    if (visual_id && (visual_id > 0)) {
      arc.globals.reports.setReport(report, visual_id);
    }

    return builder;
  };

  builder.setupChartContainer = function($chartContainer) {
    // Create a div to display the chart in
    var unique_chartdiv_id = 'chart-' + (++unique_chartid);

    $chartContainer.empty();
    $chartContainer.html(
        arc.misc.safe_substitute($('#chart-div-template').html(),
        {
          CHARTID: unique_chartdiv_id,
        }));

    return unique_chartdiv_id;
  };

  builder.setupVisualInModal = function(config) {
    arc.errors.dismissPopovers();

    let visual_id = config.app_id,
        visual_dataset_id = config.dataset_id,
        $chartContainer = config.$chartContainer || $('#chart-container'),
        title_elem = config.title_elem || 'viz-title-modal',
        subtitle_elem = config.subtitle_elem || 'viz-subtitle-modal';
    dashboard = config.dashboard;

    let $superModal = $('.super-modal');

    // Create a div to display the chart in
    let unique_chartdiv_id = builder.setupChartContainer($chartContainer);

    // Determine whether to show alert, keeping in mind is_edit might be undefined.
    const showOverrideAlert = !!arc.globals.is_edit && $('#style').hasClass('active') &&
      arc.misc.dashboardOverride();
    $('#override-warning').toggle(showOverrideAlert);

    $('.navbar').css('z-index', '100');
    $('.subnav-report').css('z-index', '99');

    $superModal.fadeIn(null, function () {
      arc.report.superModal = arc.report.superModal || {};
      arc.report.superModal.isOpen = true;
      arc.report.superModal.appId = visual_id;
      arc.report.activateModalAlertContainer();
    });

    // Setup the report to display/edit
    let report = arc.globals.reports.getReport(visual_id);

    // Set the builder's dataset if we've already loaded it
    if (report && report.viz() && report.viz().dataset()) {
      dataset = report.viz().dataset();
      builder.store_dataset_server_info(dataset);
    }

    if (visual_id === arc.report.REPORTID_NEW) {
      if (!report) {
        report = createNewReport();
      }

      // So we can properly restore new visual if changes discarded
      if (report.viz() && report.viz().viz()) {
        const newSettings = {
          report_type: report.viz().viz_type(),
          report_data: report.viz().viz().viz_getSettings(),
        };
        report.saved_data(JSON.stringify(newSettings));
      }
    }

    let params_data;
    if (config.dashboard) {
      // modal was opened in dashboard
      let dph = dashboard.params_history();
      params_data = dph.util.filterRelevantParams(
        dph.current_params()[0], report.report_id()
      );
    }

    report.modalConfig({
      chart_div: unique_chartdiv_id,
      shelves_dom_id: 'app-shelves',
      loader_dom_id: loader.dom_id(),
      params_data,
      title_elem,
      subtitle_elem,
    });

    // Make this the primary report
    arc.globals.reports.setReport(report, -1);

    // Modify builder parameters
    if (arc.globals.is_dashboard) {
      clearDatasetInfo();
    }

    // Need this here for dataset switching
    builder.dataset_id(visual_dataset_id)
           .dataset_cb(arc.report.dataset_setup);

    // Load the dataset, load the report, activate the builder
    loadDataset(visual_dataset_id, {
      manualRefresh: true,
    });

    // When this page is resized, redraw charts
    builder.displayapp().setupResizeVisual();

    return builder;
  };

  function createNewReport() {
    var report = arc.report.getPrimaryReport();
      report.dataset_id(dataset_id)
            .report_id(arc.report.REPORTID_NEW);
      arc.globals.reports.addReport(arc.report.REPORTID_NEW, undefined, report);
    return report;
  }

  builder.loadReport = function(report_id, config={}) {
    let {
      manualRefresh,
    } = config;

    if (!report_id) {
      return;
    }

    if (report_id === arc.report.REPORTID_NEW) {
      var report = createNewReport();
      if (!report.viz()) {
        builder.createReport(report.report_type());
        loadDataset(dataset_id);
      }

      setupBuilderAfterLoadingReport(report);

      arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
      report.refresh();
      return;
    }

    arc.staticfetcher.fetch([
        {
          asset: `${arc.frontend.globals.urls.reportUrl}/${report_id}`,
          postprocess: d3.json,
        },
      ], function(error, report_data) {

      if (error || report_data.length === 0) {
        arc.report.alertMessage(`Visual ${report_id} does not exist.`,
          error && error.responseText ? error.responseText : error,
          { alert_report_id: report_id });
        return;
      }

      report_data = report_data[0];

      var $subnav_report = $('.subnav-report');
      if (arc.misc.hasPerm('ds_appedit', report_data.perm)) {
        arc.misc.updateDivForPerms(['ds_appedit'], 'dataset', $subnav_report);
      }

      arc.misc.updateCloneButton(report_data.dataset_perm);

      var report = arc.report.getPrimaryReport();
      if (!report.dataset_id()) {
        // We did not have a dataset before, because only a
        // report id was given at the URL
        // Reload the dataset information - that call will eventually
        // call loadReport again. We will re-fetch this information
        // and then display the report information.
        report.dataset_id(report_data.dataset_id);
        loadDataset(report.dataset_id());
        return;
      }

      var parsed_deriveddata, report_parsed_data;
      if (report.saved_data()) {
        report_parsed_data = JSON.parse(report.saved_data());

        // when switching datasets derived data may have already been defined
        parsed_deriveddata = report_parsed_data.report_data.report_derived_data;

        // since we feasibly could make changes in the dashboard through the explore option,
        // we want to make sure to use the correct things that have been changed so instead of using
        // the fetched visual type and data, we used the local_saved_data
        if (arc.globals.is_edit) {
          report_data.type = report_parsed_data.report_type;
          report_data.report_type = report_parsed_data.report_type;
          report_data.data = report.saved_data();
        }
      }

      // If the page hasn't been saved yet, the report_search_text has been updated but not saved
      let searchText = report_data.report_search_text;
      const oldReport = arc.globals.reports.getReport(report_id);
      if (oldReport) {
        searchText = oldReport.reportSearchText();
      }

      report.report_description(report_data.description)
            .report_type(report_data.type)
            .workspace({
              workspace_id: report_data.workspace_id,
              workspace_name: report_data.workspace_name,
              workspace_perm: report_data.workspace_perm,
            })
            .reportSearchText(searchText)
            .shared_visual_dashboards(report_data.shared_visual_dashboards)
            .saved_data(report_data.data)
            .in_userlist(report_data.in_userlist);

      let reportLoaded = (report.viz() !== undefined);
      if (!reportLoaded) {
        builder.createReport(report.report_type());
      }

      setupBuilderAfterLoadingReport(report);

      // On a single visual load (as opposed to dashboard), the report.saved_data() is
      // not set up until after the builder.createReport() call above, so here we
      // need to set up the report_parsed_data, then load parsed_deriveddata as well.
      // TODO: This entire load logic needs to be made much more clear
      if (!report_parsed_data) {
        report_parsed_data = JSON.parse(report.saved_data());

        if (!parsed_deriveddata) {
          // when loading a new report derived data don't exist in the builder and
          // need to be fetched from saved report
          parsed_deriveddata = report_parsed_data.report_data.report_derived_data;
        }
      }

      deriveddata = report.viz().derivedData();
      if (parsed_deriveddata && parsed_deriveddata.length && deriveddata) {
        var hidden = [];
        var message;
        deriveddata.clearList();
        parsed_deriveddata.forEach(function(data) {
          deriveddata.createDerivedData(data, dataset, hidden);
        });

        if (hidden.length) {
          if (hidden.length > 1) {
            message = 'Derived Data ' + hidden.join(', ') + ' have been modified';
          } else {
            message = 'Derived Data ' + hidden.join(', ') + ' has been modified';
          }

          arc.report.alertMessageSmall(message, 'alert-warning');
        }
      }

      if (deriveddata) {
        deriveddata.derived_popover_add();
        deriveddata.derived_popover_delete();
      }

      // Let's redraw some of the settings if necessary
      // Sampling and segments need to be loaded
      if (!reportLoaded) {
        report.viz().restore(report_parsed_data.report_data);
      }

      $('#open-form').dialog('close');

      report.refresh(true, // force_redraw
          undefined, //filter_widget
          manualRefresh);

      arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
    });

    return builder;
  };

  function setupBuilderAfterLoadingReport(report) {
    let viz = report.viz();
    visual_type = viz.viz_type();

    if (report.viz().inModal()) {
      activateBuilder();
    }

    let openVizTypes = viz.isEmptyTable();
    if (openVizTypes) {
      $('.viz-icons-section').show();
      $('.selected-viz-section').removeClass('is-closed');
      $('.selected-viz-section').addClass('is-open');
    } else {
      $('.viz-icons-section').hide();
      $('.selected-viz-section').addClass('is-closed');
      $('.selected-viz-section').removeClass('is-open');
    }

    arc.misc.displayVisualType(visual_type);
  }

  function disableChartTypesForConnections() {
    // Disable chart types depending on connection
    let dataconnectionType = arc.report.getPrimaryReport().dataset().dataconnection_type();

    let $rawtable = $('#chart-rawtable');
    let $colstats = $('#chart-column-stats');

    if (dataconnectionType === 'ksql') {
      $rawtable.addClass('chart-icon-disabled');
      $colstats.addClass('chart-icon-disabled');
    } else {
      $rawtable.removeClass('chart-icon-disabled');
      $colstats.removeClass('chart-icon-disabled');
    }
  }

  builder.refresh = function() {
    const report = arc.report.getPrimaryReport();
    if (report) {
      const reportViz = report.viz();
      if (reportViz) {
        const isVisible = $(`#${reportViz.chart_div()}`).length > 0;
        if (isVisible) {
          report.viz().saveAwayShelfInfo();
        }
      }

      // TODO: Make changes for refresh button
      arc.css.CUSTOMCSS_MANAGER.applyCustomCss();

      // Force-redraw when the user clicks on the refresh button
      report.refresh(true, // force_redraw
          undefined, // filter_widget
          true); // manualRefresh
    }
  };

  function setupRefresh() {
    d3.select('#viz-refresh')
      .on('click.report', builder.refresh);
  }

  builder.setupMenu = function() {
    const save_fn = function() {
      // Clear errors if user saves visual
      arc.report.alertMessageAllHide();
      const report = arc.report.getPrimaryReport();
      const viz = report.viz();

      arc.staticfetcher.forget([
        {
          asset: `${arc.frontend.globals.urls.reportUrl}/${report.report_id()}`,
        },
      ]);

      if (viz) {
        // Indicate that we are saving
        $('#save-spin').css('display', 'inline-block');
        $('#save-icon').css('display', 'none');

        // Disable the save button till we are done saving
        $('#save-link').off('click.report');

        // We are going to save the settings values into the base
        // Before doing this, set the color setting based on the current
        // visual state
        viz.loadColorIntoSetting();
        viz.saveVisualSettings();
        const report_data = viz.save();
        report.saved_data(JSON.stringify({
          report_type: report.report_type(),
          report_data,
        }));

        builder.saveApp(null, true, function() {
          $('#save-spin').css('display', 'none');
          $('#save-icon').css('display', 'inline-block');
          $('#save-link')
            .offOn('click.report', save_fn);
          visual_type = report.report_type();
          if (isFirstSave) {
            $hideIfNewNavbarItems.removeClass('arc-invisible');
            isFirstSave = false;
          }

          if (displayapp) {
            if (displayapp.app_id() === arc.report.REPORTID_NEW) {
              arc.globals.reports.addReport(report.report_id(), undefined, report);
              arc.globals.reports.removeReport(arc.report.REPORTID_NEW);
              displayapp.app_id(report.report_id());
            }
          }

          if (report.visual_params_history()) {
            report.visual_params_history().setCheckPointState(true);
          }
        });
      }
    };

    $('#save-link')
      .offOn('click.report', save_fn);

    builder.setupChartSwitch();
    return builder;
  };

  builder.setupChartSwitch = function() {
    disableChartTypesForConnections();
    setupRefresh();

    d3.select('#chart-type-menu')
      .selectAll('li')
      .on('click.report', function() {
        /*
         * The icon representing a differnt chart type was clicked
         */
        let viz_type = $(this).attr('id').slice(6);
        arc.misc.displayVisualType(viz_type);

        var report = arc.report.getPrimaryReport();
        report.switchViz(viz_type);
      });

    return builder;
  };

  function setupMenuButtons(report, $gridsterElement) {
    arc.globals.reports.setReport(report, -1);
    const app_id = report.report_id();
    const dataset_id = report.dataset_id();
    const $container = visualMenu.container();
    const report_type = report.report_type();
    const hasStylePerm = arc.misc.hasPerm('sys_styles', arc.globals.perms.sysperms);
    const snapshotAdmin = arc.snapshot()
                             .is_dashboard(false);

    const $visualId = $container.find('.visual-id');
    $visualId.find('span').text(app_id);
    $visualId.offOn('click.allowCopy', function allowCopy(e) {
               // Otherwise, the dropdown closes before user can copy id
               e.preventDefault();
               e.stopPropagation();
             });

    // If single visual, need to update both edit and view mode dropdowns
    if (arc.globals.is_dashboard) {
      $hideIfNewNavbarItems = $container.find('.arc-hide-if-new');
    } else {
      $hideIfNewNavbarItems = $('.arc-hide-if-new');
    }

    if (!app_id || app_id < 0) {
      $hideIfNewNavbarItems.addClass('arc-invisible');
      isFirstSave = true;
    }

    const $viewData = $container.find('.view-data');

    if (report.viz() && report.viz().enableViewData()) {
      $container.find('.view-data-li')
         .show();
    } else {
      $container.find('.view-data-li')
         .hide();
    }

    $viewData.offOn('click.view-data', function handleShow() {
               hideTooltipsMenu();
               report.viz().showData();
             });

    function cloneVisual() {
      arc.globals.singleVisualBuilder.cloneApp(undefined);
      arc.menu_hideAll();
      $hideIfNewNavbarItems.removeClass('arc-invisible');
    }

    // For now, hide "clone" link in dashboard view mode for visuals in modals
    if (arc.globals.is_edit || !arc.globals.is_dashboard) {
      $container.find('.clone-link')
             .offOn('click', cloneVisual);
      if (report_type === 'html' && !hasStylePerm) {
        $container.find('.clone-link')
             .hide();
      }
    } else {
      $container.find('.clone-link')
             .hide();
    }

    $container.find('.email-visual').attr('data-appid', app_id)
                                    .attr('data-datasetid', dataset_id)
                                    .attr('data-report-type', report_type)
                                    .on('click.send-email', () => { hideTooltipsMenu(); });
    $container.find('.download-image-link')
           .offOn('click', function() {
              const base = report.viz();
              const title = base?.title() ?? '';
              const imageFormat = $(this).attr('data-arc-imageformat');
              snapshotAdmin.downloadImage(imageFormat,
                {
                  title,
                  isVisual: true,
                  appId: report.report_id(),
                });
              visualMenu.hide();
           });

    $container.find('.export-my-viz')
      .offOn('click', function (evt) {
        var id_object = {};
        id_object.dashboards = [];
        id_object.visuals = [report.report_id()];
        arc.migration.export();
        arc.migration.export.setup();
        arc.migration.export.setup.exportArtifacts(evt, id_object);
      });

    setupLinkWorkspaceButtons(report, $container, $gridsterElement);

    $container.find('.export-csv-link')
              .offOn('click.export', function() {
                report.viz().exportCSV({});
                visualMenu.hide();
              });

    $container.find('.export-xlsx-link')
              .offOn('click.export', function() {
                report.viz().exportXlsx();
                visualMenu.hide();
              });
  }

  builder.setupVisualDropdownButtons = function($navbar, report) {
    report = report || arc.report.getPrimaryReport();
    const app_id = report.report_id();

    const $gridsterElement = $navbar.closest('.gs-w');

    $navbar.find('.visual-more')
           .offOn('click.more', function(e) {
             const $copyFrom = $(this).closest('.more-container')
                                      .find('> ul.more-options');

             if (visualMenu.visualId() === app_id && visualMenu.isVisible()) {
               visualMenu.hide();
               return;
             }

             hideContextMenus();
             visualMenu.build($copyFrom, report)
                       .visualId(app_id)
                       .show(e.pageX, e.pageY);
             setupMenuButtons(report, $gridsterElement);
           });

    if (arc.globals.is_edit) {
      if (!dashboard) {
        builder.setupDelete();
      }
    }
  };

  function setupLinkWorkspaceButtons(report, $navbar, $gridsterElement) {
    const $linkLi = $navbar.find('.link-visual-toggle').closest('li');
    const $linkText = $navbar.find('.linked-icon-text');
    const $dropdownButton = $navbar.find('.link-visual-toggle');
    const $sharedText = $navbar.find('.linked-text-wrapper');
    const $workspaceButton = $navbar.find('.workspace-visual-wrapper');
    const $includedText = $navbar.find('.included-viz-list');
    const $modal = $('#confirm-link-unlink-modal');
    const $modalType = $modal.find('.confirm-type');
    const $modalConfirm = $modal.find('.confirm-button');
    const $deleteOrRemove = $navbar.find('.remove-from-dashboard-link > span');
    let cloneInstead;

    setupButtons();

    $deleteOrRemove.offOn('click.remove', function() {
      arc.globals.dashboard.removeFromGrid(report, $gridsterElement);
      hideTooltipsMenu();
    });

    if (!arc.globals.is_edit) {
      return;
    }

    if (!arc.globals.ms_dashboard) {
      $dropdownButton.closest('li').addClass('arc-invisible');
      return;
    }

    $dropdownButton.on('click.link', () => { hideTooltipsMenu(); });

    $modalConfirm.offOn('click.link', function() {
      // Link or Unlink or Clone
      if (cloneInstead) {
        // Add report with same id and updated shared_visual_dashboards to reports_to_update
        // So shared_visual_dashboards can be updated if visual saved
        // cloneInstead is only true if shared_visual_dashboards exists with > 1 dashboard id
        if (arc.globals.dashboard) {
          const oldReport = arc.reporttracker.report()
                                             .report_id(report.report_id())
                                             .dataset_id(report.dataset_id());

          const oldSharedDashboardIds = JSON.parse(report.shared_visual_dashboards());
          const dashId = arc.globals.ms_dashboard.dashboard_id();
          const newSharedDashboardIds = oldSharedDashboardIds.filter(id => id !== dashId);

          oldReport.shared_visual_dashboards(JSON.stringify(newSharedDashboardIds));
          arc.globals.dashboard.addReportToUpdate(oldReport);

          builder.cloneApp('replaceReport');
        }

        return;
      }

      const old_shared_visual_dashboards = report.shared_visual_dashboards();
      if (old_shared_visual_dashboards) {
        report.shared_visual_dashboards(undefined);
      } else {
        const dashboard_id = arc.misc.dashboard_id();
        report.shared_visual_dashboards(`[${dashboard_id}]`);
      }

      report.updateSharedVisualDashboards(
        function success() {
          $modal.offOn('hidden.bs.modal', function() {
            setupButtons();
          });

          $modal.modal('hide');
        },

        function failShareCb() {
          // If fail, reset value
          report.shared_visual_dashboards(old_shared_visual_dashboards);
        });
    });

    // Update buttons
    function setupButtons() {
      cloneInstead = false;
      report.updateSharedIconInDashboard();

      $linkLi.removeClass('arc-invisible');

      if (report.shared_visual_dashboards()) {
        const reportUrl = arc.frontend.globals.urls.viewDashboardUrl;
        const dashboardList = JSON.parse(report.shared_visual_dashboards());

        // Add current dashboard to shared visuals
        const currentDashboardId = arc.misc.dashboard_id();
        if (currentDashboardId && !dashboardList.includes(currentDashboardId)) {
          dashboardList.push(currentDashboardId);
        }

        const formatedList = dashboardList.map((id) => {
          const url = arc.misc.getBlankTargetUrl(`${reportUrl}/${id}`);
          return `<a href="${url}" target="_blank">${id}</a>`;
        });

        const formatted = formatedList.join(', ') || 'None';
        $includedText.html(`<span>Used in dashboards:</span> <a class="link">${formatted}</a>`);
        $linkText.text('Unlink');
        $workspaceButton.removeClass('arc-invisible');
        $sharedText.removeClass('arc-invisible');
        $includedText.removeClass('arc-invisible');
        $deleteOrRemove.text('Remove from Dashboard');

        // Cannot unlink if visual is in multiple dashboards
        if (dashboardList.length > 1) {
          const warningText = 'This linked visual is used in multiple dashboards ' +
                              `(${formatted}) and cannot be unlinked. ` +
                              'Would you like to clone instead?';
          cloneInstead = true;
          $modalConfirm.text('Clone');
          $modalType.html(warningText);
        } else {
          $modalConfirm.text('Unlink');
          $modalType.text('Are you sure you want to unlink this visual? ' +
              'This change will apply immediately if you confirm.');
        }
      } else {
        $linkText.text('Make Linked');
        $workspaceButton.addClass('arc-invisible');
        $sharedText.addClass('arc-invisible');
        $includedText.addClass('arc-invisible');
        $deleteOrRemove.text('Delete Visual');

        $modalConfirm.text('Link');
        $modalType.text('Are you sure you want to make this a linked visual? ' +
            'This change will apply immediately if you confirm.');
      }
    }
  }

  builder.setupDelete = function() {
    d3.select('.delete-visual-link')
      .on('click.delete', function() {
        const $modal = $('#delete-visual-modal');
        if (arc.misc.modalShown($modal)) {
          return;
        }

        $modal.modal('show');
      });

    // TODO: There seem to be two ways of deleting a visual/app.
    // One here below (which goes to report.delete in arc_reporttracker.js
    // and another in apps_groups.html. This should be consolidated.
    d3.select('#delete-visual-yes')
      .on('click.deletevisual', function () {
        let report = arc.report.getPrimaryReport();
        report.delete(
            function() {
              // Success callback
              arc.report.pagenav.needs_save(false);
              window.open(`${arc.frontend.globals.urls.staticUrl}/../apps/apps`, '_self');
            },

            function(req) {
              // Failure callback
              $('#delete-visual-modal').modal('hide');
              arc.report.alertMessage(`Failed to delete visual ${report.report_id()}`,
                  req.responseText ? req.responseText : req,
                  { is_html: true, alert_report_id: report.report_id() });
            }
            );
      });
  };

  builder.createReport = function(report_type) {
    arc.misc.displayVisualType(report_type);
    var report = arc.report.getPrimaryReport();
    try {
      report.createViz(report_type);
    } catch (e) {
      console.log(e);
      return;
    }

    return builder;
  };

  function getAssets() {
    var actual_dsid = dataset_id;
    if (arc.globals.dssw && arc.globals.dssw[actual_dsid]) {
      actual_dsid = arc.globals.dssw[actual_dsid];
    }

    var assets = [
      {
        asset: `${arc.frontend.globals.urls.datasetUrl}/${actual_dsid}?detail=true`,
        postprocess: d3.json,
      },
    ];

    return assets;
  }

  builder.removeCache = function() {
    var assets = getAssets();
    arc.staticfetcher.forget(assets);
  };

  function datasetDetailFetch(cb, error_cb, report_id) {
    var assets = getAssets();
    var actual_dsid = dataset_id;
    if (arc.globals.dssw && arc.globals.dssw[actual_dsid]) {
      actual_dsid = arc.globals.dssw[actual_dsid];
    }

    var t1 = new Date();
    arc.staticfetcher.fetch(assets,
      function(error, datasets) {
        var t2 = new Date();
        if (error) {
          arc.perf.PAGEPERF.addProfile({
            type: 'datasetdetail',
            time: t2 - t1,
            error: error,
          });

          console.log(error);
          let alertConfig = {};
          if (report_id) {
            alertConfig = { alert_report_id: report_id };
          }

          arc.report.alertMessage('Failed to load dataset',
              error && error.response ? error.response : error,
              alertConfig);
          if (error_cb) {
            error_cb();
          }

          return;
        }

        var segments = datasets[0].segments,
            events = datasets[0].events,
            filterassociations = datasets[0].filterassociations;

        arc.perf.PAGEPERF.addProfile({
          id: 0,
          type: 'datasetdetail',
          time: t2 - t1,
          profile: [
            [
              'dataset',
              actual_dsid,
            ],
          ],
        });

        // For single table datasets right now, need to fetch the table column info
        if (datasets[0].type === 'singletable' && datasets[0].info.length === 0) {
          arc.staticfetcher.fetch([
              {
                asset: arc.datasets.dataset_columns_url(datasets[0].dc_id, datasets[0].detail),
                postprocess: d3.json,
              },
            ], function(error, columninfo) {
              if (error) {
                console.log(error);
                arc.report.alertMessage(
                    'Failed to load dataset table column detail',
                    error.response ? error.response : error);
                if (error_cb) {
                  error_cb();
                }

                return;
              }

              cb(datasets, segments, events, filterassociations, columninfo);
            });
        } else {
          cb(datasets, segments, events, filterassociations, undefined);
        }
      });

    return builder;
  }

  builder.showDatasetDetail = function() {
    datasetDetailFetch((datasets, segments, events, filterassociations, columninfo,
                                actual_dsid) => {
      const $detail = $('#detail');
      const firstDataset = datasets[0];
      if (firstDataset) {
        const conn_type = firstDataset.dc_type;
        if (arc.dataconnection.isArcEngine(conn_type)) {
          $('#dataset-events-tab').show();
        } else {
          $('#dataset-events-tab').hide();
        }

        if (firstDataset.detail_display === arc.datasets.CREATED_FROM_SQL_DETAIL) {
          $('#dataset-table-section').hide();
        } else {
          $('#dataset-table-section').show();
        }

        if (firstDataset.perms.includes('dc_aviews')) {
          $('#edit-enable-extract-icon').attr('data-allow-extracts', 'true');
        } else {
          $('#edit-enable-extract-icon').hide()
                                        .attr('data-allow-extracts', 'false');
        }

        $detail.find('#dataset-dataconnection').text(firstDataset.dc_name);
        $detail.find('#dataset-dataconnection-type').text(firstDataset.dc_type_display);
        $detail.find('#dataset-description').text(firstDataset.description);
        $detail.find('#dataset-lvname').text(firstDataset.lvname);
        $detail.find('#dataset-id').text(firstDataset.id);
        $detail.find('#dataset-created-by').text(firstDataset.created_by);
        $detail.find('#dataset-updated-by').text(firstDataset.updated_by);
        const in_tf = d3.time.format.iso;
        const out_tf = d3.time.format('%b %d, %Y %I:%M %p');
        $detail.find('#dataset-created').text(out_tf(in_tf.parse(firstDataset.created)));
        $detail.find('#dataset-updated').text(out_tf(in_tf.parse(firstDataset.updated)));

        $detail.find('#dataset-tags').empty();
        if (arc.frontend.featureFlipper.getValueById('site_enable_extracts')) {
          if (firstDataset.extractSource) {
            $detail.find('#dataset-tags').append(arc.extracts.extractSourceBadge);
          }

          if (firstDataset.extractTarget) {
            $detail.find('#dataset-tags').append(arc.extracts.extractTargetBadge);
          }
        }
      }

      const $sidebar = $('.sidebar');
      $sidebar.find('#dataset-segments-count').text($('#segments-table tbody tr').length);
      $sidebar.find('#dataset-events-count').text(events.length);
      if (datasets.length === 0) {
        arc.report.alertMessage(`No information available about dataset ${dataset_id}`);
        return;
      }

      // DJC -- there's too much code dup between this and loadDataset()
      //        loadDataset() should do all data setup, then each call location
      //        can do the specific part.
      dataset = arc.datasets.Dataset()
                    .dataset_id(actual_dsid)
                    .init_from_fetch(firstDataset, columninfo);
      builder.store_dataset_server_info(dataset);

      arc.frontend.renderReactComponent(
        'actions-dropdown-root',
        arc.frontend.DatasetActionsDropdown,
        { id: dataset.dataset_id(), name: dataset.dataset_name(), hasEditPerm: dataset.dataset_perms().includes('ds_manage') }
      );

      {
        const allTableNames = dataset.tables().map((t, i) => {
          if (i === 0) {
            return t.tablename();
          }

          return t.tablename_noalias();
        });

        const tableDisplay = arc.misc.displayTableNames(allTableNames);
        $detail.find('#dataset-table').text(tableDisplay);
      }

      // DJC -- We should have a tooltip with dataset.detail_display()
      $('#app-dataset-name').text(dataset.dataset_name());
      $('.dataset-title-name').text(dataset.dataset_name());

      dataset_segments.dataset(dataset)
                      .parse_segments(segments)
                      .setupDatasetDetail();
      dataset_events.dataset(dataset)
                    .events(events)
                    .setupDatasetDetail();

      // FA setup can be enabled with ds_manage or sys_editperm based on setting
      const dofa = arc.globals.arcviz_settings.MANAGE_DS_FA ?
        arc.misc.hasPerm('ds_manage', dataset.dataset_perms()) :
        arc.misc.hasPerm('sys_editperm', arc.globals.perms.sysperms);

      if (dofa) {
        // Use arc.misc.updateDivSysPerms($sidebar) here
        dataset_filterassociations.dataset_id(dataset_id)
                      .dataset_perms(firstDataset.perms)
                      .fa_dataset_segments(dataset_segments)
                      .dom_id('#filterassociations')
                      .setupDatasetDetail();
      }

      builder.paintAttributes(dataset);

      if (dataset_cb) {
        dataset_cb(dataset);
      }

      // DJC -- redrawJoins should likely be part of painAttributes
      //        need to fix up edit button state, perhaps done automatically
      //        as part of redrawJoins
      // arcl.redrawJoins(dataset);
    });
  };

  // Construct one html element for the provided column data
  builder.paintOneColumn = function(col) {
    function isColHierarchy() {
      return col.isHierarchy();
    }

    var isCdt = col.isComplexType();
    var colname = col.display_name();
    var col_isdim = col.isDimension() ? 'Dim' : 'Mes';

    // Build a shelf element so we can get the datatype html
    var elem = arc.report.shelf.element()
                               .dataset_colname(col.attr('name'))
                               .dataset_coltype(col.attr('type'))
                               .fn_cast(col.attr('cast'));

    var templateName,
      templateParams,
      li;
    templateParams = {
      colname: arc.misc.escapeHTML(colname),
      col_isdim: col_isdim,
      CDT_field_wrapper: isCdt ? 'cdt-field-wrapper' : '',
      HTML_CDT_expand: isCdt ? '<a class="link"><i class="fa fa-angle-down expand-icon"></i></a>' : '',
      HTML_DIMAGG: isCdt ? '' : '<a class="dim-agg-type" arcid="dsattr-dimagg">' + col_isdim +
                      '</a>',
      HTML_coltype_class: col.coltype_class(),
      comment: col.attr('comment') || col.attr('hivecomment') || '',
      clone_class: builder.dataset().dataconnection_type() === 'solr' ? 'arc-invisible' : '',
      isSensitive: col.isSensitive()
    };

    if (isColHierarchy()) {
      templateName = 'dsattr-hierarchy-column';
      elem.dataset_dimhlist(col.attr('dimh'));
      var childElements = elem.dataset_dimhlist();
      templateParams.children = [];
      var Dataset = col.table().dataset();
      var column;
      li = arc.clone_template(templateName, templateParams);
      var ul = li.find('ul');
      var childTemplate = $('#hierarchy-child-element-template').html();

      try {
        childElements.forEach(function (c) {
          column = Dataset.get_column(c);
          if (!column) {
            throw new Error('Invalid hierarchy ' + colname + ' with children ' +
                JSON.stringify(childElements));
          }

          ul.append(arc.misc.safe_substitute(childTemplate, {
            child_class: !column.isBaseCol() ? 'deletable-column' : '',
            child_colname: column.attr('alias') || column.attr('name'),
            child_coltype_class: column.coltype_class(),
          }));
        });
      } catch (e) {
        // Generally occurs when stored hierarchy has dimh as ''
        console.log(e);
      }
    } else {
      templateName = 'dsattr-column';
      li = arc.clone_template(templateName, templateParams);
      if (col.attr('isdim')) {
        li.addClass('dimension-column');
      } else {
        li.addClass('measure-column');
      }

      if (col.attr('ispart')) {
        const partitionKeyIcon = $('#dsattr-partition-key-template').html();
        li.find('.table-list-value').append(partitionKeyIcon);
        li.find('.table-list-value-edit').append(partitionKeyIcon);
      }

      if (col.isSensitive()) {
        const sensitiveIcon = $('#dsattr-sensitive-field-template').html();
        li.find('.table-list-value').append(sensitiveIcon);
        li.find('.table-list-value-edit').append(sensitiveIcon);
      }

      if (col.hasComplexParent()) {
        li.find('.dim-agg-type').remove();
      }
    }

    li.css('display', ''); // jQuery sets display to list-item for lis by default

    li.data('_arc_data', col);
    li.attr('id', '');

    // Modify the menu to not show Remove option when not feasible
    if (!col.isBaseCol()) {
      li.addClass('deletable-column');
    }

    arcl.attributes_set_column_hidden(li, col);
    return li;
  };

  // Fill in the attributes page for the dataset builder
  builder.paintAttributes = function(dataset, config) {
    if (!config) {
      config = {};
    }

    if (!config.$dimDiv) {
      config.$dimDiv = $('#viz-attributes-dimensions');
    }

    if (!config.$aggDiv) {
      config.$aggDiv = $('#viz-attributes-aggregates');
    }

    config.$dimDiv.find('.table-list-container').remove();
    config.$aggDiv.find('.table-list-container').remove();
    if (config.$partDiv) {
      config.$partDiv.find('.table-list-container').remove();
    }

    var dims = $('<div class="table-list-container"></div>')
                  .appendTo(config.$dimDiv);

    var aggs = $('<div class="table-list-container"></div>')
                  .appendTo(config.$aggDiv);

    var parts;
    if (config.$partDiv) {
      parts = $('<div class="table-list-container"></div>')
          .appendTo(config.$partDiv);
    }

    const paint_hdr = function(container, hdr) {
      const div = arc.clone_template('dsattr-colhdr', {
                                                        coltype: hdr,
                                                      });
      if (config.selectAllOption) {
        div.prepend(
          $(`<input data-atleastonedimagg="custom-validator" class="select-all-fields form-check-input position-relative"
                    type="checkbox"></input>`)
        );
      }

      container.append(div);
      container.find('.select-all-fields')
        .offOn('click.toggleAll', function toggleAll(evt) {
          const $target = $(evt.target);
          const isChecked = $target.prop('checked');
          $target.closest('.attribute-wrapper').find('.aview-column-checkbox')
            .prop('checked', isChecked);
        });
    };

    var paint_title = function(container, fqname, cnt) {
      var div = arc.clone_template('dsattr-tabhdr', {
                                                      fqname: fqname,
                                                      tabonly: arc.datasets.drop_dbname(fqname),
                                                      colcnt: cnt,
                                                    });
      container.append(div);
      return div.find('.table-list');
    };

    var paint_columns = function(container, columns) {
      var cdt = {};
      columns.forEach(col => {
        var li = builder.paintOneColumn(col);
        if (col.isComplexType()) {
          cdt[col.attr('name')] = {
            col,
            li,
          };
        }

        const par = col.getComplexParent();
        if (par) {
          const parent_name = par.display_name();
          if (cdt[parent_name]) {
            cdt[parent_name].li.find('.child-table-list').first().append(li);
          }
        } else {
          container.append(li);
        }
      });
    };

    paint_hdr(dims, 'Dimensions');
    paint_hdr(aggs, 'Measures');
    if (parts) {
      paint_hdr(parts, 'Partitions');
    }

    if (config.headersOnly) {
      return builder;
    }

    dataset.tables().forEach(function(table) {

      if (table.dimensionsAndHierarchies().length > 0) {
        var dcols = paint_title(dims, table.tablename(), table.dimensionsAndHierarchies().length);
        paint_columns(dcols, table.dimensionsAndHierarchies());
      }

      if (table.measuresOnly().length > 0) {
        var acols = paint_title(aggs, table.tablename(), table.measuresOnly().length);
        paint_columns(acols, table.measuresOnly());
      }
    });

    // Setup tooltips we just created
    $('[data-toggle="tooltip"]').tooltip({
      delay: {
       show: 1500,
       hide: 100,
      },
    });

    arcl.attributes_set_editmode(false);
    return builder;
  };

  function loadDataset(in_dataset_id, config={}) {
    const {
      manualRefresh,
      storeIntoReport,
      loadCb = () => {},
    } = config;

    dataset_id = in_dataset_id;
    if (dataset && dataset.dataset_id() === dataset_id) {
      drawDatasetElements();
      loadCb(dataset);
    } else {
      let report = arc.report.getPrimaryReport();
      datasetDetailFetch(function(datasets, segments, events,
        filterassociations, columninfo, actual_dsid) {
          dataset = arc.datasets.Dataset()
                        .dataset_id(actual_dsid)
                        .init_from_fetch(datasets[0], columninfo);
          builder.store_dataset_server_info(dataset);
          dataset_segments.dataset(dataset)
                          .parse_segments(segments);
          dataset_events.dataset(dataset)
                        .events(events);
          dataset_filterassociations.dataset_id(dataset_id)
                        .dataset_perms(datasets[0].perms)
                        .filterassociations(filterassociations)
                        .fa_dataset_segments(dataset_segments);
          drawDatasetElements({
            manualRefresh,
          });

          // This function is called when switching datasets. In that case,
          // now load the dataset into the report
          if (storeIntoReport) {
            report.dataset(dataset);
          }

          loadCb(dataset);
        }, undefined, report.report_id());
    }

    return builder;

    function drawDatasetElements(config={}) {
      let {
        manualRefresh,
      } = config;

      $('#visual-data-panel .derived-list-wrapper li.selection_element').remove();

      // DJC -- another case where we are assuming a dataset is a single table
      //        perhaps we can use 'dataset.getPrimaryTablename()' or something

      dataset_events.paintBuilder();

      if (dataset_cb) {
        dataset_cb(dataset);
      }

      const report = arc.report.getPrimaryReport();
      report.dataset(dataset);

      // Now that we have the dataset, can update the display permission elements
      arc.misc.updateDivForPerms(dataset.dataset_perms(), 'dataset');

      var report_id = report.report_id();
      if (!report_id || report_id <= 0) {
        builder.createReport(visual_type);
      }

      builder.loadReport(report.report_id(), {
        manualRefresh,
      });

      if (report.viz()) {
        deriveddata = report.viz().derivedData();

        if (deriveddata && deriveddata.enabled() &&
          arc.dataconnection.supports_derived_data(dataset.dataconnection_type())) {
          deriveddata.setupDerivedData();
          $('.derived-list-wrapper').show();
        } else {
          $('.derived-list-wrapper').hide();
        }
      }

      arc.report.dropdownContent = $('#dropdown-base').html();

      return builder;
    }
  }

  // Saves either new, cloned, or existing visual
  builder.saveApp = function(overrides, resetNeedsSave, done_cb, success_cb) {
    var report = arc.report.getPrimaryReport();
    overrides = overrides || {};

    // Redraw visual in case display formatting or other value has changed
    // Display formatting especially will not refresh after save, so need to do it now
    arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
    report.refresh(true);
    if (!overrides.report_id && (report.report_id() === arc.report.REPORTID_NEW)) {
      var create_data = {
        csrfmiddlewaretoken: arc.globals.csrf_token,
        create_limit: 100,
      };
      $.post(`${arc.frontend.globals.urls.reportCreateUrl}/${report.dataset_id()}`, create_data, (data) => {
        const r = data[0];
        report.report_id(r.id);
        builder.setupVisualDropdownButtons($('.modal-button-bar'));
        builder.saveApp({ report_id: r.id }, resetNeedsSave, done_cb, undefined);
      })
      .done(() => {
        if (success_cb) {
          success_cb();
        }
      })
      .fail((req) => {
        arc.report.alertMessage('Failed to create report',
            req.responseText ? req.responseText : req,
            { alert_report_id: report.report_id() });
      });

      return;
    }

    // add dashboard_id if not linked visual
    let report_data = overrides.report_data || report.saved_data();
    const dashboardId = arc.misc.dashboard_id();
    if (dashboardId && !report.shared_visual_dashboards()) {
      report_data = JSON.parse(report_data);
      report_data.report_data.dashboard_id = dashboardId;
      report_data = JSON.stringify(report_data);
    }

    var post_data = {
      report_data,
      dataset_id: report.dataset_id(),
      report_description: report.report_description(),
      shared_visual_dashboards: report.shared_visual_dashboards(),
      workspace_id: report.workspace() && report.workspace().workspace_id,
      report_search_text: report.reportSearchText(),
      report_type: report.report_type(),
      report_id: report.report_id(),
      csrfmiddlewaretoken: arc.globals.csrf_token,
    };

    if (overrides.report_id) {
      post_data.report_id = overrides.report_id;
    }

    $.post(
      `${arc.frontend.globals.urls.reportUrl}/${post_data.report_id}`,
      post_data,
      () => {
        if (overrides.report_id) {
          arc.frontend.vizCookie.setLatestVisuals(overrides.report_id, 'visual');

          // Modify the URL to reflect the new report id
          // If in dashboard, mark page as needing to be saved and don't change url
          if (arc.globals.is_dashboard) {
            arc.report.pagenav.needs_save(true);
          } else {
            const url = `${arc.frontend.globals.urls.reportBuildUrl}/${overrides.report_id}`;
            arc.misc.replaceState(url);
          }
        } else {
          arc.frontend.vizCookie.setLatestVisuals(post_data.report_id, 'visual');
        }
      }
    )
    .fail((req) => {
      arc.report.alertMessage(
        'Failed to save report',
        req.responseText ? req.responseText : req,
        { alert_report_id: report.report_id() }
      );
    })
    .done(() => {
      if (resetNeedsSave) {
        arc.report.pagenav.needs_save(false);
      }

      if (success_cb) {
        success_cb();
      }

      visual_type = report.report_type();
    })
    .always(() => {
      if (done_cb) {
        done_cb();
      }
    });

    return builder;
  };

  builder.newApp = function(dataset_id) {
    window.open(`${arc.frontend.globals.urls.staticUrl}/../apps/app/new?dataset_id=${dataset_id}`, '_self');
  };

  builder.newDashboard = function(dataset_id) {
    window.open(`${arc.frontend.globals.urls.staticUrl}/../apps/builder/0?report_type=dashboard&dataset_id=${dataset_id}`, '_self');
  };

  function updateDatasetSetting(newDataset, newDatasetId, report) {
    report.dataset_id(newDatasetId);

    if (report.viz()) {
      report.viz().needs_save(true);
    }

    if (!newDataset) return;
    const newDatasetName = newDataset.dataset_name();
    report.dataset(newDataset);
    report.viz()?.dataset(newDataset);

    // clear shelf history now
    report.viz()?.shelfTrackerInstance()?.clearHistory();

    // For the picklist modal, redraw the dataset
    if (arc.misc.isFilterVisual(report.viz().viz_type())) {
      const currSettings = report.viz().user_settings();
      ['picklist_dataset', 'daterange_dataset'].forEach((settingId) => {
        const datasetSetting = currSettings.find_by_id(settingId);

        if (datasetSetting) {
          datasetSetting.value(newDatasetName);
        }

        report.viz()?.updateSettingsModal(settingId, newDatasetName);
      });
    }
  }

  builder.switchDataset = function ({ newDatasetId, applyAllVisuals }) {
    const report = arc.report.getPrimaryReport();
    const mainReportOriginalDatasetId = report.dataset_id();
    report.dataset_id(newDatasetId);
    $('#datasource-modal').modal('hide');
    arc.report.pagenav.needs_save(true);

    loadDataset(report.dataset_id(), {
      storeIntoReport: true,
      loadCb(newDataset) {
        updateDatasetSetting(newDataset, newDatasetId, report);

        // Handle other reports on the page if necessary
        const reportIds = new Set();
        reportIds.add(report.report_id());
        if (applyAllVisuals && newDatasetId !== mainReportOriginalDatasetId) {
          const allVisuals = arc.globals.reports.getAllReports();
          allVisuals.forEach((currReport) => {
            const currId = currReport.report_id();
            if (!reportIds.has(currId) && mainReportOriginalDatasetId === currReport.dataset_id()) {
              updateDatasetSetting(newDataset, newDatasetId, currReport);
              currReport.refresh(true);
              reportIds.add(currId);
            }
          });
        }

        if (arc.globals.ms_dashboard) {
          const currDashboard = arc.globals.ms_dashboard.getCurrentlyActiveSheet();
          if (currDashboard) {
            currDashboard.reactivateReport(report);
          }
        }
      },
    });
  };

  builder.cloneApp = function(replaceReport) {
    arc.report.alertMessageSmall('Cloning Visual...', 'alert-info');

    let report = arc.report.getPrimaryReport();
    var dataset_id = report.dataset_id();

    var post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      create_limit: 100,
    };

    const requestURL = `${arc.frontend.globals.urls.reportCreateUrl}/${dataset_id}`;

    // Create new visual now so any changes user made can be saved using saveApp
    $.post(requestURL, post_data, _cloneVisualCb)
     .fail(_alertFailCb);

    function _alertFailCb(req) {
      const text = req.responseText || req;
      const ctx = { alert_report_id: report.report_id() };
      arc.report.alertMessage('Failed to clone visual.', text, ctx);
    }

    function _cloneVisualCb(data) {
      report = arc.report.getPrimaryReport();
      const oldId = report.report_id();
      const newId = data[0].id;
      const appendToDashboard = !replaceReport && arc.globals.dashboard;
      let oldDatasetId;

      if (appendToDashboard) {
        const workspace = arc.globals.dashboard.workspaces().currentWorkspace();
        if (workspace) {
          report.workspace(workspace);
        }

        oldDatasetId = report.dataset_id();
        report.shared_visual_dashboards(undefined);
      } else {
        // Update divs on page with new id
        arc.misc.updateReports({ oldReport: report, oldId, newId });
      }

      const overrides = { report_id: newId };
      var viz = report.viz();
      if (viz) {
        const report_data = viz.save();
        const report_title_clone = report_data.report_title ?
            `Clone of ${report_data.report_title}` : '';
        report_data.report_title = report_title_clone;

        // Modify the current report's saved data to reflect what is on screen
        overrides.report_data = JSON.stringify({
          report_type: report.report_type(),
          report_data,
        });
      }

      builder.saveApp(overrides, true, null, function successSaveCb() {
        const successMessage = arc.globals.dashboard ?
                               'Successfully cloned and added visual to dashboard' :
                               'Successfully cloned visual';
        arc.report.alertMessageSmall(successMessage, 'alert-success');

        // If cloning within dashboard and not replacing current report
        // Create a new report, add it to the grid, and give it the same modalConfig
        // as current report
        if (appendToDashboard && oldDatasetId) {
          arc.report.pagenav.needs_save(true);
          arc.globals.dashboard.drawClonedWidgetFromModal(oldId, newId);
        } else if (replaceReport) {
          // Need to update saved_data or else title change is undone when user
          // closes modal and report.discardChanges is called
          report.saved_data(overrides.report_data);

          if (arc.globals.dashboard) {
            arc.globals.dashboard.activateReport(report);
          }
        }

        // Update the dropdowns
        if (arc.globals.isInModal()) {
          builder.setupVisualDropdownButtons($('.modal-button-bar'));
        } else {
          builder.setupVisualDropdownButtons($('#subnav-report'));
        }
      });

      // Always open new visual in edit mode when cloning
      if (!arc.globals.is_edit) {
        displayapp.enterVisualEditMode();
      }
    }
  };

  const $datasetsSel = $('.switch-dataset-dataset-sel');
  const $dataconnectionsSel = $('.switch-dataset-dataconn-sel');
  const $allVisualsCheckbox = $('#apply_all_dashboard_visuals');
  builder.setupSwitchDataset = function() {
    function handleSwitching(newDatasetId, applyAllVisuals) {
      newDatasetId = parseInt(newDatasetId, 10);
      builder.switchDataset({
        newDatasetId,
        applyAllVisuals,
      });
    }

    $('.switch-dataset-apply').offOn('click.switchdsa', function switchDSA() {
      // Show confirmation for filter switching and apply all switching
      let confirmationMessage = '';

      const applyAllVisuals = $allVisualsCheckbox.prop('checked');
      if (applyAllVisuals) {
        confirmationMessage = 'All visuals on the page that use this dataset will be switched.';
      }

      if (arc.misc.isFilterVisual(arc.report.getPrimaryReport().viz().viz_type())) {
        confirmationMessage += ' The change will happen immediately.';
      }

      if (confirmationMessage) {
        confirmationMessage += ' If a column is missing from the new dataset, this could result in errors that need to be manually fixed.';
        const confirmModal = arc.misc.simpleUserConfirmation({
          confirmTitle: 'Please confirm dataset switch',
          confirmQus: confirmationMessage,
          confirmText: 'Confirm',
        });

        confirmModal.askUser(() => {
          handleSwitching($datasetsSel.val(), applyAllVisuals);
        });
      } else {
        handleSwitching($datasetsSel.val(), applyAllVisuals);
      }
    });

    $('#visual-data-panel .app-dataset-switch').offOn('click.switchds', function switchDS() {
      $allVisualsCheckbox.prop('checked', false);
      $('#datasource-modal').modal('show');
      if ($dataconnectionsSel.find('option').length > 0) {
        setCurrentDataset();
      } else {
        loadAllDatasets(drawAllDatasets);
      }
    });

    function setCurrentDataset() {
      let r = arc.report.getPrimaryReport();
      $dataconnectionsSel.val(r.dataset().dataconnection_id())
                         .trigger('change');
      $dataconnectionsSel.trigger('chosen:updated');

      $datasetsSel.val(r.dataset_id());
      $datasetsSel.trigger('chosen:updated');
    }

    function loadAllDatasets(cb) {
      var assets = [
        {
          asset: arc.frontend.globals.urls.datasetUrl,
          postprocess: d3.json,
        },
      ];

      arc.staticfetcher.fetch(assets,
        function (error, alldatasets) {
          if (error) {
            console.log('Could not load data');
            console.log(error);
            return;
          }

          alldatasets = alldatasets.filter(function (d) {
            return arc.misc.hasPerm('ds_appedit', d.perms);
          });

          if (cb) {
            cb(alldatasets);
          }
        });
    }

    function drawAllDatasets(datasets) {
      var datasets_of_dataconn_hash = {};
      var dataconnections_hash = {};
      var dataconnections_array = [];
      let current_dataconnection_id;

      var ds = datasets.slice();
      ds.sort(function(a, b) {
        if (a.name < b.name) {
          return -1;
        }

        if (a.name > b.name) {
          return 1;
        }

        return 0;
      });

      for (var i = 0; i < ds.length; i++) {
        let eachds = ds[i];
        let dataconn_id = eachds.dc_id;
        if (!(dataconn_id in datasets_of_dataconn_hash)) {
          datasets_of_dataconn_hash[dataconn_id] = [];
          dataconnections_hash[dataconn_id] = eachds.dc_name;
          dataconnections_array.push(dataconn_id);
        }

        datasets_of_dataconn_hash[dataconn_id].push(eachds);
      }

      d3.select('.switch-dataset-dataconn-sel')
        .html('')
        .selectAll('nosel')
        .data(dataconnections_array)
        .enter()
          .append('option')
          .attr('value', function(d) {
              return d;
            })
          .text(function(d, idx) {
              return dataconnections_hash[d];
           });

      $dataconnectionsSel.trigger('chosen:updated');

      $dataconnectionsSel.offOn('change.dc', function() {
        updateDatasetsofDataConnection($(this).val(), false);
        $dataconnectionsSel.trigger('chosen:updated');
      });

      setCurrentDataset();

      function updateDatasetsofDataConnection(cdcid) {
        let datasetsofconn = datasets_of_dataconn_hash[cdcid];

        $('.switch-dataset-dataset-sel').find('option').remove();

        d3.select('.switch-dataset-dataset-sel')
          .selectAll('nosel')
          .data(datasetsofconn)
          .enter()
            .append('option')
            .attr('value', function(d) {
                return d.id;
              })
            .text(function(d) {
                return d.name;
             });

        $datasetsSel.trigger('chosen:updated');
      }
    }
  };

  return builder;
};

}());
