window.arc = window.arc || {};

(function() {
'use strict';

// Contains code specific to the arcweb/apps/templates/apps_data.html file

arc.apps_data = arc.apps_data || {};

function extractsEnabled() {
  return arc.frontend.featureFlipper.getValueById('site_enable_extracts');
}

let isExtractsFetched = false;

let initialDataConnectionName;
let dcTableToDataset = {};
let datasetSearchFn;
const encompassingSolrCollections = {};
let dashboard_template_enabled;
let sqlOnly;
let expressionEditorOnDirectAccessPage;

let initialQueries;

let extractTracker;
// Two type of Dataset type exist, one is BackendDataset[], the other arc.datasets
// We store the BackendFormat here
let allDatasets = []; // BackendFormat[]
let datasetIdToDataset = {}; // BackendFormat
let allSchedules = {};

const $data_tabs = $('#data-tabs');
const $rawAccessMenu = $('#raw-access');
const $databases = $('#databases-list');
const $bookmarksHolder = $('#sql-bookmarks');
const $historyHolder = $('#sql-history');
const searchModalResultsTemplate = $('#search-modal-results-template').html();
const $sqlDatabaseList = $('.sql-database-list');
const sqlTemplate = $('#sql-bookmark-template').html();
const extractTableId = 'dataset-extracts-table';
const $extractTable = $(`#${extractTableId}`);

const extractTemplate = `
    <tr data-state="%STATE%">
      <td>
        %EXTRACT_ID%
      </td>
      <td>
        <a href="dataset/%DATASET_ID%">%DATASET_NAME%</a>
      </td>
      <td>
        <a href="%JOB_PAGE_LINK%" style="%NEVER_RUN%">
          <span class="badge badge-pill %STATE_LABEL%" data-msg="%STATUS_MSG%">
            %HTML_STATE%
          </span>
        </a>
      </td>
      <td>
        %HTML_TARGET_DATASETS%
      </td>
      <td>
        %HTML_TARGET_DATACONNECTION%
      </td>
      <td>
        %TARGET_TABLE%
      </td>
      <td>
        %TARGET_SCHEDULE%
      </td>
      <td>
        %LAST_SUCCESSFUL_RUN%
      </td>
    </tr>
`;

let isSqlPage = false;

function defaultDbCookie(dcid) {
  return `sql-database-default.${dcid}`;
}

arc.apps_data.setSqlOnly = function(_) {
  sqlOnly = _;
};

arc.apps_data.dashboard_template_enabled = function(_) {
  dashboard_template_enabled = _;
};

arc.apps_data.initialDataconnectionName = function(_) {
  initialDataConnectionName = _;
};

arc.apps_data.initialQueries = function(_) {
  initialQueries = _;
};

arc.apps_data.cookie_get_defaultdb = function() {
  const cname = arc.apps_data.cookie_dc_default_database(arc.apps_data.active_dataconnection_id());
  const dbname = arc.frontend.vizCookie.get(cname);
  return dbname;
};

arc.apps_data.cookie_set_defaultdb = function(dbname) {
  const cname = arc.apps_data.cookie_dc_default_database(
      arc.apps_data.active_dataconnection_id()
  );
  arc.frontend.vizCookie.set(cname, dbname, { expires: 365 });
};

arc.apps_data.cookie_dc_default_database = function(dc_id) {
  return `dbactive.dc.${dc_id}.${arc.globals.username}`;
};

arc.apps_data.cookie_set_default_datatable = function(tablename) {
  const cookieName = arc.apps_data.cookie_dc_default_datatable(
      arc.apps_data.active_dataconnection_id()
    );
  arc.frontend.vizCookie.set(cookieName, tablename ?? '', { expires: 0 });
};

arc.apps_data.cookie_get_default_datatable = function() {
  const cookieName =
    arc.apps_data.cookie_dc_default_datatable(arc.apps_data.active_dataconnection_id());
  const tableName = arc.frontend.vizCookie.get(cookieName);
  return tableName;
};

arc.apps_data.cookie_dc_default_datatable = function(dc_id) {
  return `table.active.dc.${dc_id}.${arc.globals.username}`;
};

arc.apps_data.cookie_active_data_connection_id = function() {
  return `${arc.globals.username}_dataconnection_active`;
};

arc.apps_data.initialize_data = function() {
  const loader = arc.loadbar();

  arcl.fetch_and_draw = arc.apps_data.fetch_and_draw;
  arc.globals.singleVisualBuilder =
      arc.reportbuilder()
         .loader(loader);
};

function pickDataType(dataType) {
  $('input.data-type-selected').attr('value', dataType);
  $('.optionally-visible').each(function() {
    const $this = $(this);
    if ($this.attr('data-type') === dataType) {
      $this.show();
    } else {
      $this.hide();
    }
  });

  if (dataType === 'csv') {
    $('#id_datasource_file').prop('required', true);
    $('#datasource-url').prop('required', false);
  } else {
    $('#id_datasource_file').prop('required', false);
    $('#datasource-url').prop('required', true);
  }
}

function initializeDirectAccess() {
  $rawAccessMenu.offOn('click.direct', function redirectToDirectAccess() {
    const dc = arc.apps_data.get_active_dataconnection();
    window.open(`datasql?dataconnection_id=${dc.id}&dataconnection_name=${dc.name}`, '_self');
  });
}

const helpPrefix = '<span style="font-size:14px">';
const helpSuffix = '</span>';

const helpBasics = `
    The <strong>Datasets</strong> tab displays datasets that can be used to create visuals.<br>
    The <strong>Connection Explorer</strong> tab displays databases and tables in this
    data connection.<br>
`;

const helpExtracts = `
    The <strong>Data Extracts</strong> tab display sources and targets of data extracts.<br>
`;

const helpArcengine = `
    For Arcadia connections, the <strong>Analytical Views</strong> tab displays analytical views
    in this data connection.
`;

const $displayMoreOptionButton = $('.cloudera-cmdline-actions #more-option');
const $displayMoreOptionButtonIconUp = $displayMoreOptionButton.find('.fa-angle-up');
const $displayMoreOptionButtonIconDown = $displayMoreOptionButton.find('.fa-angle-down');
const $moreOptionSection = $('.cloudera-cmdline-secondary-actions');

function setupMoreOptionButton() {
  $displayMoreOptionButton.offOn(
    'click.toogleEmailAndCsvButtons',
    function toogleEmailAndCsvButtons() {
      $moreOptionSection.toggle();
      $displayMoreOptionButtonIconUp.toggle();
      $displayMoreOptionButtonIconDown.toggle();
      resetDownloadCSVSection();
    }
  );
}

arc.apps_data.initialize = function() {
  arc.apps_data.initialize_data();
  initializeDirectAccess();

  arc.misc.updateDivSysPerms();

  // set up chosen
  $('.chosen-select').chosen({
    search_contains: true,
  });

  $sqlDatabaseList.chosen({ disable_search_threshold: 10 });

  // remove row of linked columns
  $('body').on('click', '#conparm-table tbody .fa-trash-o, ' +
                        '#conlocs-table tbody .fa-trash-o', function() {
    $(this).closest('tr').remove();
    $('#conparm-addrow .add-new-row').removeClass('disabled').tooltip('hide');
  });

  // Edit parameter name/value on click
  $('body').on('click', '.conparm-edit', function() {
    $(this).focus();
  });

  $('.data-type-selector').offOn('click.selectDataTypes', function() {
    pickDataType($(this).attr('data-type'));
  });

  pickDataType('csv');

  const setupSearch = () => {
    const searchCallback = (dataset, term) => {
      const datasetSearchKeys = [
        'display_title',
        'created',
        'updated',
        'created_by',
        'report_count'
      ];

      for (const key of datasetSearchKeys) {
        const value = dataset[key];

        const result = value?.toString().toLowerCase().search(term);

        if (result !== -1 && result !== undefined) {
          return true;
        }
      }

      return false;
    };

    datasetSearchFn = arc.report.searchboxSetup({
      searchboxId: '#searchbox',
      search_cb: searchCallback,
      sel_cb: () => d3.selectAll('#datasets-table, #tabledatasets-table').select('tbody').selectAll('tr'),
    });
  };

  setupSearch();

  // search for terms entered from global search
  const queryParam = arc.misc.getQueryParameters();
  if (queryParam.searchbox) {
    const searchTerm = queryParam.searchbox;
    $('#searchbox').val(searchTerm).trigger('input');
  }

  function clickableRowListener(that, e) {
    const c = $(that).html();
    let url = $(that).parent().data('href') || $(that).parent().find('a').attr('href');

    if (url && c.indexOf('<a') === -1 &&
        c.indexOf('<button') === -1 &&
        c.indexOf('<input') === -1) {
      const target = (e.ctrlKey || e.which === 2 ? '_blank' : '');
      url = arc.misc.getBlankTargetUrl(url, target);
      $(that).html(`<a href='${url}' target='${target}'>${c}</a>`);
      const a = $(that).find('a');
      if (e.which === 3) {
        setTimeout(() => {
          a.parent().html(c);
        }, 300);
      } else {
        a[0].click();
        a.parent().html(c);
      }
    }
  }

  // allow selecting on table rows and still be able to use right click to open
  // item in a new tab
  $('body').on('mousedown', 'table.table-hover tr td', function(e) {
    clickableRowListener(this, e);
  });

  arc.apps_data.fetch_and_draw();
  arc.apps_data.setup_dataset_add_button();
  arc.apps_data.setup_tabs();
  arc.apps_data.setup_refresh_metadata_button();

  arc.dataconnection.newconnections_modal_setup();

  // toggle selected file type to import
  $('.image-wrapper').offOn('click.toggleFileType', function() {
    if ($(this).attr('class') !== 'type-selected') {
      $(this).parent().children().removeClass('type-selected');
      $(this).addClass('type-selected');
    }
  });

  // made table with table sample data have a sticky column header
  // we should probably also make the column stats table have a sticky header too

  const popoverContent = helpPrefix +
      helpBasics +
      (extractsEnabled() ? helpExtracts : '') +
      (arc.globals.arcviz_settings.ENABLE_DATACONN_ARCENGINE ? helpArcengine : '') +
      helpSuffix;

  $('#info-datasource').popover({
      html: true,
      trigger: 'hover',
      title: '',
      delay: { show: 500 },
      content: popoverContent,
      sanitize: false,
  });
};

arc.apps_data.update_refresh_button = function() {
  const dataConnectionType = arc.apps_data.active_dataconnection_type();
  const $tabDataSource = $('#tab-datasource');
  const isConnectionExplorerActive = $tabDataSource.hasClass('active');

  const shouldRefreshButtonHide = arc.globals.site_hide_refresh_button_from_connection_explorer ||
    !arc.dataconnection.isArcEngineOrImpala(dataConnectionType);

  const $tablesListTemplate = $('#tables-list-template');
  const $refreshMetaButton = $('#refresh-metadata');
  const $refreshMetaSpinner = $('#refresh-metadata-spinner');
  const $refreshTableButton = $tablesListTemplate.find('.table-refresh-link');
  const $statisticsLink = $tablesListTemplate.find('.table-statistics-link');
  const hideRefreshButtons = () => {
    $refreshMetaSpinner.hide();
    $refreshMetaButton.hide();
    $refreshTableButton.hide();
  };

  const showRefreshButtons = () => {
    $refreshMetaButton.show();
    $refreshTableButton.show();
  };

  const isStatisticsSupportedByType = (type) => [
    'arcengine',
    'arcengine_v2',
    'impala',
    'impyla',
    'sparksql'
  ].includes(type);

  if (isConnectionExplorerActive) {
    if (shouldRefreshButtonHide) {
      hideRefreshButtons();
    } else {
      showRefreshButtons();
    }

    if (isStatisticsSupportedByType(dataConnectionType)) {
      $statisticsLink.show();
    } else {
      $statisticsLink.hide();
    }
  } else {
    hideRefreshButtons();
    $statisticsLink.hide();
  }
};

const createdDataTables = {};

arc.apps_data.setup_tabs = function() {
  $('#tab-datasource').on('show.bs.tab', () => {
    drawDataTables();
  });

  $('#tab-dataextracts').on('shown.bs.tab', () => {
    if (extractsEnabled() && !isExtractsFetched) {
      fetchExtracts(drawExtracts);
      isExtractsFetched = true;
    }
  });

  $('#tab-dataset').on('shown.bs.tab', () => {
    const $datasetsDataTable = createdDataTables['#datasets-table'];
    if ($datasetsDataTable) {
      $datasetsDataTable.columns.adjust();
    }
  });

  // When a user refreshes the Data page, load with the last tab open by default.

  $('#data-tabs a').offOn('click.tabtoggle', function showTab(e) {
    e.preventDefault();
    const href = $(e.target).attr('href');
    if (href !== undefined) {
      window.location.hash = href.substr(1);
      $(this).tab('show');
    }
  });

  // on load of the page: switch to the currently selected tab, except
  // because showing that tab triggers a listener that needs async info
  const targetTab = window.location.hash;
  if (targetTab !== '#tables') {
    $(`#data-tabs a[href="${targetTab}"]`).tab('show');
  }
};

function drawExtracts() {
  if (!extractsEnabled() || !extractTracker) {
    $('#extracts-count').empty();
    return;
  }

  let activeDatasets = allDatasets;
  let activeDataConnectionId;
  const dc = arc.apps_data.get_active_dataconnection();
  if (dc) {
    activeDataConnectionId = dc.id;
  }

  activeDatasets = activeDatasets.filter((d) => {
    if (d.extractSource || d.extractTarget) {
      const tableNames = arc.datasets.datasetObjectToTableNames(d);
      d.extractSources = [];
      tableNames.forEach((tablename) => {
        const extract = extractTracker.isExtractTarget(Number(d.dc_id), tablename);
        if (extract) {
          d.extractSources.push(extract);
        }
      });

      return true;
    }
  });

  $extractTable.find('tbody').empty();

  const countByState = {};

  let extractsCount = 0;

  const displayExtracts = [];

  function addToDisplayExtracts(e, sourceDataset) {
    extractsCount += 1;
    const extractMap = getDatasetPropertyMap(sourceDataset);

    const {
      state,
      htmlState,
      stateLabel,
    } = arc.extracts.stateStringsFromLog(e.logs);

    if (state) {
      countByState[state] = (countByState[state] || 0) + 1;
    }

    extractMap.STATE = state;
    extractMap.HTML_STATE = htmlState;
    extractMap.STATE_LABEL = stateLabel;
    extractMap.JOB_PAGE_LINK = e.scheduleId && htmlState !== 'Never run' ? `/arc/jobs/job/${e.scheduleId}` : '#extracts';
    extractMap.NEVER_RUN = htmlState === 'Never run' ? 'pointer-events: none;' : '';

    if (e.logs) {
      const taskList = JSON.parse(e.logs.task_list).tasks || [];
      extractMap.LAST_SUCCESSFUL_RUN = e.logs.lastSuccessfulRun || '';
      extractMap.STATUS_MSG = e.logs.error_message || taskList[taskList.length - 1]?.message || '';
    } else {
      extractMap.LAST_SUCCESSFUL_RUN = '';
      extractMap.STATUS_MSG = '';
    }

    const fqTablename = `${e.tgtDbname}.${e.tgtTablename}`;
    extractMap.EXTRACT_ID = e.id;
    extractMap.TARGET_TABLE = fqTablename;
    extractMap.TARGET_SCHEDULE = e.scheduleGroupName || 'Manual';

    extractMap.HTML_TARGET_DATACONNECTION = 'Unknown';
    if (arcl.dataconnections) {
      arcl.dataconnections.some((_dc) => {
        if (_dc.id === e.tgtDataconnectionId) {
          extractMap.HTML_TARGET_DATACONNECTION = _dc.name;
          return true;
        }

        return false;
      });
    }

    const datasets = getActiveDatasets(fqTablename, e.tgtDataconnectionId);
    const datasetsHtml = [];
    if (datasets.length) {
      datasetsHtml.push(...datasets.map(
          (d) => `<div><a href="dataset/${d.id}">${d.name}</a></div>`
      ));
    }

    extractMap.HTML_TARGET_DATASETS = datasetsHtml.join('');
    displayExtracts.push(extractMap);
  }

  activeDatasets.forEach((d) => {
    if (!d.extractSource) {
      return;
    }

    const extracts = extractTracker.getDatasetExtracts(d.id);
    extracts.forEach((e) => {
      if ((activeDataConnectionId === undefined) ||
          (d.dc_id === activeDataConnectionId) ||
          (e.tgtDataconnectionId === activeDataConnectionId)) {
        addToDisplayExtracts(e, d);
      }
    });
  });

  // Sort by extract ID
  displayExtracts.sort((a, b) => (a.EXTRACT_ID - b.EXTRACT_ID));

  displayExtracts.forEach((e) => {
    $extractTable.find('tbody').append($(arc.misc.safe_substitute(extractTemplate, e)));
  });

  $('#extracts-count').text(extractsCount);
  $('#extracts .total .av-status-number').text(extractsCount);
  $('#extracts .have-error .av-status-number').text(countByState.Error || 0);
  $('#extracts .up-to-date .av-status-number').text(countByState.Completed || 0);
  $('#extracts .need-refresh .av-status-number').text(countByState['Never run'] || 0);

  $('#extracts .av-status-box').offOn('click.errors', function() {
    const $a = $(this).find('a');
    const state = $a.attr('data-state');

    if (state) {
      $extractTable.find('tbody tr')
          .hide();

      $extractTable.find(`tbody tr[data-state="${state}"]`)
          .show();
    } else {
      // Show all
      $extractTable.find('tbody tr')
          .show();
    }
  });

  setupMessageTooltips();
}

function setupMessageTooltips() {
  const sel = d3.select(`#${extractTableId}`)
                .selectAll('.badge');

  function msgParams(d, i, elem) {
    const msg = $(elem).attr('data-msg');
    if (msg) {
      return [
        {
          key: 'Last Message',
          value: msg,
        },
      ];
    }

    return [];
  }

  arc.marks()
     .addSelection(sel)
     .params_fn(msgParams)
     .text_fn((d, i, elem) => arc.misc.tooltipFromKVs(undefined, msgParams(d, i, elem)))
     .marker_name('msg')
     .activate();
}

function drawDataTables() {
  const start_dcid = arc.apps_data.active_dataconnection_id();
  const dcType = arc.apps_data.active_dataconnection_type();
  const isKsql = (dcType === 'ksql');
  const isSolr = isSolrConnection(dcType);

  if (dcType !== -1) {
    const tableLabel = `${arc.dataconnection.table_label(dcType)} Name`;
    $('#table-table .tablename-hdr').text(tableLabel);
  }

  $('#databases-list li').remove();
  $('#table-table tbody tr').remove();

  arc.apps_data.update_refresh_button();
  $('.sample-table-wrapper').hide();
  if (arc.apps_data.active_dataconnection_id() <= 0 ||
      !arc.misc.hasPerm('dc_explore', arc.apps_data.get_active_dataconnection().perms)) {
    return;
  }

  function toggleNoTableSelected(showNoTableSelected) {
    const $sampleTableWrapper = $('.sample-table-wrapper');
    const $sampleNoneSelected = $('.sample-none-selected');
    if (showNoTableSelected) {
      $sampleTableWrapper.hide();
      $sampleNoneSelected.show();
    } else {
      $sampleTableWrapper.show();
      $sampleNoneSelected.hide();
    }
  }

  const dc_id = arc.apps_data.active_dataconnection_id();

  function addSqlDatabase(dbName) {
    $sqlDatabaseList.append(
        arc.misc.safe_substitute('<option value="%DATABASE_NAME%">%DATABASE_DISPLAY_NAME%</option>',
        {
          DATABASE_NAME: dbName,
          DATABASE_DISPLAY_NAME: dbName === '' ? '<<DEFAULT>>' : dbName,
        })
    );
  }

  function displayLoadingSpinnerInSection($container) {
    $container.find('.loading-spinner').show();
  }

  function hideLoadingSpinnerInSection($container) {
    $container.find('.loading-spinner').hide();
  }

  const $sqlTabTableHolder = $('.sql-tab-tables-holder');
  const $datatableList = $('.sql-table-list');
  const $connectionExploreDatabaseSideNav = $('.db-side-nav');
  const $connectionExploreTableList = $('.table-list-main');

  function loadSqlTables(dbName) {
    if (dbName === null || dbName === undefined) {
      return;
    }

    displayLoadingSpinnerInSection($sqlTabTableHolder);

    $datatableList.empty();
    const dc = arc.apps_data.get_active_dataconnection();
    arc.apps_data.load_tables_cb(dc.id, dbName, (error, tables) => {
          if (error) {
            arc.report.alertMessage('Could not retrieve dataset tables', error);
            return;
          }

          let infoLink = '';
          if (arc.dataconnection.isArcEngineOrImpala(dc.type)) {
            infoLink = '<a class="info"><i class="fa fa-info-circle"></i></a>';
          }

          const tableLink = (arc.frontend.featureFlipper.getValueById('enable_cloudera_style') ? '<i class="fa fa-table"></i>' : '');

          tables.rows.forEach((tableRow) => {
            const tableName = tableRow[0];
            $datatableList.append(
                arc.misc.safe_substitute(
                    `<tr>
                      <td data-table="%TABLE%">
                        ${infoLink}
                        <a class="tablename" title="%TABLE%"> ${tableLink} %TABLE%</a>
                      </td>
                    </tr>`,
                {
                  TABLE: tableName,
                }
)
);
          });

          hideLoadingSpinnerInSection($sqlTabTableHolder);

          $('.sql-table-list tr td a.info').offOn('click.table', function() {
            const tableName = $(this).closest('td').attr('data-table');
            arc.apps_data.runQuery(`describe formatted ${dbName}.${tableName}`, arc.apps_data.active_dataconnection_id(), true);
          });

          $('.sql-table-list tr td a.tablename').offOn('click.table', function() {
            const tableName = $(this).closest('td').attr('data-table');
            const text = `${dbName}.${tableName}`;
            const editor = expressionEditorOnDirectAccessPage.aceEditor();
            editor.getSession().insert(editor.getCursorPosition(), text);
          });
        });
  }

  $sqlDatabaseList.empty();
  $datatableList.empty();
  addSqlDatabase('Loading...');
  displayLoadingSpinnerInSection($sqlTabTableHolder);
  displayLoadingSpinnerInSection($connectionExploreDatabaseSideNav);

  $sqlDatabaseList.trigger('chosen:updated');

  arc.apps_data.load_databases_cb(
    dc_id,
    (error, data) => {
      const curr_dcid = arc.apps_data.active_dataconnection_id();
      if (start_dcid !== curr_dcid) {
        // This is no longer the active dataset
        return;
      }

      $sqlDatabaseList.empty();

      if (error) {
        arc.report.alertMessage('Could not read a list of databases', error);
        $sqlDatabaseList.trigger('chosen:updated');
        return;
      }

      let tableToDataset = dcTableToDataset[dc_id];
      if (!tableToDataset) {
        tableToDataset = {};
      }

      const dbCounts = {};
      let totalCount = 0;

      $.each(tableToDataset, (k, v) => {
        const db = k.split('.')[0];
        if (dbCounts[db]) {
          dbCounts[db] += v.length;
        } else {
          dbCounts[db] = v.length;
        }

        totalCount += v.length;
      });

      let primaryDb;
      let primaryTable;

      let databaseLogo = 'fa fa-database';

      const $ul = $('#databases-list');
      hideLoadingSpinnerInSection($connectionExploreDatabaseSideNav);
      $ul.find('li').remove();
      if (isKsql) {
        databaseLogo = 'kafka-logo';
        const li = arc.misc.safe_substitute($('#databases-list-template').html(),
          {
            DATABASE_LOGO: databaseLogo,
            DATABASE_NAME: 'ALL',
            DATABASE_TABLECOUNT: 0,
            DATABASE_DATASETCOUNT: totalCount,
          });
        $ul.append(li);
      }

      const dbTemplate = $('#databases-list-template').html();
      data.rows.forEach((d) => {
        const db = d[0];
        const datasetCount = dbCounts[db] || 0;

        if (!sqlOnly) {
          const li = arc.misc.safe_substitute(dbTemplate,
            {
              DATABASE_LOGO: databaseLogo,
              DATABASE_NAME: db,
              DATABASE_TABLECOUNT: 0,
              DATABASE_DATASETCOUNT: datasetCount,
            });
          $ul.append(li);
        }

        addSqlDatabase(db);
      });

      hideLoadingSpinnerInSection($connectionExploreDatabaseSideNav);

      const cookieName = defaultDbCookie(arc.apps_data.active_dataconnection_id());

      {
        const defaultDb = arc.frontend.vizCookie.get(cookieName);
        if (defaultDb) {
          $sqlDatabaseList.val(defaultDb);
        }
      }

      $('.database-and-tables').removeClass('arc-invisible');
      $('.cmdline-actions').removeClass('arc-invisible');
      $sqlDatabaseList.trigger('chosen:updated');

      const dbName = $sqlDatabaseList.val();
      loadSqlTables(dbName);

      $sqlDatabaseList.offOn('change.database', function() {
        const $this = $(this);
        const databaseName = $this.val();
        loadSqlTables(databaseName);
        arc.frontend.vizCookie.set(cookieName, databaseName, { expires: 365 });
      });

      function activateDb($elem) {
        displayLoadingSpinnerInSection($connectionExploreTableList);

        if (!$elem.hasClass('active')) {
          $elem.parent().children().removeClass('active');
          $elem.addClass('active');
          toggleNoTableSelected(true);
        } else {
          return;
        }

        const dbname = $elem.data('databasename');
        primaryDb = dbname;

        const $tbody = $('#table-table tbody');
        $tbody.empty();

        const currentLoadId = `${arc.misc.uniqueId()}`;
        $databases.attr('data-tableloadid', currentLoadId);

        function loadTablesCB(err, tables) {
          hideLoadingSpinnerInSection($connectionExploreTableList);

          if (err) {
            arc.report.alertMessage('Could not read a list of tables', err);
            return;
          }

          if ($databases.attr('data-tableloadid') !== currentLoadId) {
            // Stale load - user clicked on something else already
            return;
          }

          if (!$elem.hasClass('active')) {
            $elem.parent().children().removeClass('active');
            $elem.addClass('active');
          }

          arc.apps_data.cookie_set_defaultdb(dbname);

          $('.table-prefix-icon-th').hide();
          $('.table-prefix-icon-td').hide();
          $tbody.empty();

          // enable extra column in table to indicate Solr collection aliases
          // or KSQL tables and streams
          if (isKsql || isSolr) {
            $('.table-prefix-icon-th').show();
            $('.table-prefix-icon-td').show();
          }

          tables.rows.forEach((d) => {
            const tableName = d[0];
            let tableIcon = '';
            const fullTableName = `${dbname}.${tableName}`;
            const datasets = tableToDataset[fullTableName];
            let datasetCount = 0;

            if (isKsql) {
              if (d[1] === 'stream') {
                tableIcon = '<a class="link"><i class="fa fa-rss" aria-hidden="true" ' +
                        'data-toggle="tooltip" title="Stream"></i></a>';
              } else {
                tableIcon = '<a class="link"><i class="fa fa-table" aria-hidden="true" ' +
                        'data-toggle="tooltip" title="Table"></i></a>';
              }
            } else if (isSolr) {
              if (d[1]) {
              tableIcon = '<a class="link"><i class="fa fa-object-group" aria-hidden="true" ' +
                        'data-toggle="tooltip" title="Collection Alias"></i></a>';
              }
            }

            encompassingSolrCollections[fullTableName] = d[1];

            if (datasets) {
              datasetCount = datasets.length;
            }

            const tr = arc.misc.safe_substitute(
              $('#tables-list-template tbody').html(),
              {
                HTML_ICON: tableIcon,
                TABLE_NAME: tableName,
                TABLE_DATASETCOUNT: datasetCount,
              }
            );

            $tbody.append(tr.trim());
          });

          $('.table-prefix-icon-td').offOn('click.tablePrefix', function() {
            const $tr = $(this).closest('tr');
            $tr.find('td.clickable').trigger('click');
            $('li.collections-tab > a').trigger('click');
          });

          $('.table-datasets').offOn('click.clickRow', function() {
            const $tr = $(this).closest('tr');
            $tr.find('td.clickable').trigger('click');
            $('li.datasets-tab > a').trigger('click');
          });

          $('.dataset-new').offOn('click.openCreateNewDatasetModal', function () {
            const selectedTable = $(this).closest('tr').data('tablename');
            const selectedDatabase = $connectionExploreDatabaseSideNav.find('ul li.database-name.active').attr('data-databasename');
            popupCreateDatasetModal({ initialData: { database: selectedDatabase, table: selectedTable, sourceType: 'table' } });
          });

          $('.table-refresh-link').offOn('click.refreshTable', function () {
            const spinner = $(this).find('.fa-spinner');
            const tablename = $(this).closest('tr').data('tablename');
            const databaseName = arc.apps_data.cookie_get_defaultdb();
            const rb = arc.requestBuilder();
            rb.requestType('.invalidate_metadata');
            rb.tableName(`${databaseName}.${tablename}`);

            function fetchCb(errorMsg) {
              spinner.hide();
              if (errorMsg) {
                arc.report.alertMessageAllHide();
                arc.report.alertMessage('Error on refresh operation', errorMsg);
              } else {
                const msg = `Table refresh for ${tablename} is complete`;
                arc.report.alertMessageSmall(msg, 'alert-success');
              }
            }

            spinner.show();
            arc.report.sql_runner.run(
              arc.apps_data.active_dataconnection_id(),
              '',
              fetchCb,
              false,
              rb.dsreq()
            );
          });

          const confirmComputeStats = arc.misc.simpleUserConfirmation({
            confirmTitle: 'Confirmation',
            confirmQus: 'Computing stats on hive connections can be resource intensive. Please confirm you understand and would like to continue',
            confirmText: 'Confirm',
          });

          $('.table-statistics-link').offOn('click.fetchTableStatistics', function () {
            const spinner = $(this).find('.fa-spinner');
            const tablename = $(this).closest('tr').data('tablename');
            const databaseName = arc.apps_data.cookie_get_defaultdb();
            const rb = arc.requestBuilder();
            rb.requestType('.compute_stats');
            rb.tableName(`${databaseName}.${tablename}`);

            function fetchCb(errorMsg) {
              spinner.hide();
              if (errorMsg) {
                arc.report.alertMessageAllHide();
                arc.report.alertMessage('Error computing stats', errorMsg);
              } else {
                const msg = `Statistics computation for table ${tablename} is complete`;
                arc.report.alertMessageSmall(msg, 'alert-success');
                if ((databaseName === primaryDb) && (tablename === primaryTable)) {
                  setupAndDrawConnectionExplorerCurrentDatasetTable(databaseName, tablename);
                }
              }
            }

            function runSql() {
              spinner.show();
              arc.report.sql_runner.run(arc.apps_data.active_dataconnection_id(), '', fetchCb, false, rb.dsreq());
            }

            if (['hive', 'hyve'].includes(arc.apps_data.active_dataconnection_type())) {
              confirmComputeStats.askUser(runSql);
            } else {
              runSql();
            }
          });

          const $drivingTable = $('.driving-table');

          // highlight selected table row
          $drivingTable.offOn('click', 'td.clickable', (e) => {
            const $currentTarget = $(e.currentTarget);
            const selected = $currentTarget.hasClass('table-row-active');
            const $td = $currentTarget.parent().find('td');
            const $tr = $currentTarget.parent().closest('tr');

            if (!selected) {
              $drivingTable.find('td').removeClass('table-row-active');
              $td.addClass('table-row-active');
              primaryTable = $tr.data('tablename');
              setupAndDrawConnectionExplorerCurrentDatasetTable(dbname, primaryTable);
              toggleNoTableSelected(false);
            }
          });

          const defaultTableName = arc.apps_data.cookie_get_default_datatable();
          if (defaultTableName && defaultTableName !== '') {
            let $tableElem = $drivingTable.find(`tr[data-tablename=${defaultTableName}] td.clickable`);
            // some dataconnection don't allow uppercase for example impala
            if (!$tableElem.length) {
              const tableNameRowSelector = `tr[data-tablename=${defaultTableName.toLowerCase()}] td.clickable`;
              $tableElem = $drivingTable.find(tableNameRowSelector);
            }

            if ($tableElem.length) {
              $tableElem.trigger('click');
              $tableElem[0].scrollIntoView({ block: 'end' });
            }

            arc.apps_data.cookie_set_default_datatable(''); // we need only on the first load
          }
        }

        arc.apps_data.load_tables_cb(
          arc.apps_data.active_dataconnection_id(),
          (isKsql && (dbname === 'ALL')) ? '' : dbname,
          loadTablesCB
        );
      }

      $('#databases-list').offOn('click', (e) => {
        activateDb($(e.target).closest('li.database-name'));
      });

      const dbname = arc.apps_data.cookie_get_defaultdb();
      if (dbname) {
        const $sel = $('#databases-list').find(`[data-databasename='${dbname}']`);
        if ($sel.length > 0) {
          const $db = $sel.first();
          activateDb($db);
          $db[0].scrollIntoView();
        }
      }
    }
  );
}

function isSolrConnection(dctype) {
  return (dctype === 'solr');
}

arc.apps_data.fetchTableStats = function(params) {
  if (!arc.dataconnection.isArcEngineOrImpala(params.dctype)) {
    params.cb('Table stats does not work with this engine');
    return;
  }

  arc.report.sql_runner.run(params.dcid,
      `show table stats ${params.fullname}`,
      params.cb,
      false,
      undefined,
      undefined,
      false);
};

function fetchCollectionsfromAlias(params) {
  const response = {};
  let rows = [];
  const eSC = encompassingSolrCollections[params.fullname];
  if (eSC) {
    rows = eSC.split(',');
  }

  response.rows = [];
  response.colnames = ['Collection Name'];
  response.coltypes = ['STRING'];
  rows.forEach((d, i) => {
    response.rows.push([rows[i]]);
  });

  params.cb(null, response);
}

arc.apps_data.fetchColumnStats = function(params) {
  if (!arc.dataconnection.isArcEngineOrImpala(params.dctype)) {
    params.cb('Table stats does not work with this engine');
    return;
  }

  arc.report.sql_runner.run(params.dcid,
      `show column stats ${params.fullname}`,
      params.cb,
      false,
      undefined,
      undefined,
      false);
};

// Count occurences of char in string
function ccount(text, ch) {
  return (text.match(new RegExp(`[${ch}]`, 'g')) || []).length;
}

// Split but handle case where comma can be embedded in <>, ()
function split_comma(text) {
  const input = text.split(',');
  const olist = [];
  let item = '';
  while (input.length) {
    item += input.splice(0, 1)[0];
    if (ccount(item, '<') !== ccount(item, '>') ||
        ccount(item, '(') !== ccount(item, ')')) {
      item += ',';
    } else {
      olist.push(item);
      item = '';
    }
  }

  return olist;
}

// Populate cols list with tuple for each element within a possibly
// complex type defintion.  Tuple format: (name, type, parent)
function get_type(name, text, parent, cols) {
  let index = text.indexOf('<');
  if (index === -1) {
    cols.push([name, text, parent]);
    return;
  }

  const type = text.substr(0, index);
  text = text.substr(index + 1, text.length - index - 1);
  cols.push([name, type, parent]);
  parent = name;

  if (type === 'map') {
    index = text.indexOf(',');
    const ktype = text.substr(0, index);
    const vtype = text.substr(index + 1);
    cols.push([`${name}.key`, ktype, parent]);
    get_type(`${name}.value`, vtype, parent, cols);
  } else if (type === 'array') {
    cols.push([`${name}.pos`, 'int', parent]);
    get_type(`${name}.item`, text, parent, cols);
  } else if (type === 'struct') {
    split_comma(text).forEach((pair) => {
      index = pair.indexOf(':');
      const sname = pair.substr(0, index);
      const stype = pair.substr(index + 1);

      get_type(`${name}.${sname}`, stype, parent, cols);
    });
  } else {
    // eslint-disable-next-line no-console
    console.log(`Unknown complex type: ${type}`);
  }
}

/*
 * params can contain:
 *  dcid: data connection id
 *  dctype: data connection type
 *  fullname: fully qualified table name
 *  displayErrorMessage: true/false for whether to alert via standard alerts on error
 *  limit: how many rows to fetch (defaults to 100 if nothing is set)
 *  cache: whether it is ok to use the query cache for results
 *  cb: callback function of type function(error, data)
 *  pk_fk: hadoop engine specific information about primary/foreign keys for arc_admin_dataset.js
 */
arc.apps_data.fetch_sample_data = function(params) {
  function sample(sql) {
    arc.report.sql_runner.runParams({
      dataconnection_id: params.dcid,
      query: sql,
      cb: params.cb,
      cache: params.cache,
      displayErrorMessage: params.displayErrorMessage,
      hideAlertsOnSuccess: params.hideAlertsOnSuccess,
    });
  }

  if (params.limit === undefined) {
    params.limit = 100;
  }

  if (params.dctype === 'solr') {
    const rb = arc.requestBuilder();
    rb.requestType('getrows');
    rb.tableName(params.fullname);

    arc.report.sql_runner.run(params.dcid,
        null,
        params.cb,
        params.cache,
        rb.dsreq());
    return;
  }

  // If table is a select statement or on  Non-hadoop connection, use normal sample path
  if (params.fullname.startsWith('(') || !arc.dataconnection.isArcEngineOrImpala(params.dctype)) {
    sample(arc.dataconnection.sample_sql(params.fullname, params.dctype, params.limit));
    return;
  }

  arc.report.sql_runner.run(params.dcid,
      `describe formatted ${params.fullname}`,
      (errorMsg, result) => {
        if (errorMsg) {
          params.cb(errorMsg, undefined);
          return;
        }

        const cols = [];
        let start_column = false;
        let start_table_param = false;

        const storedRows = [];
        let keepTrackOfOldRows = true;
        result.rows.forEach((r) => {
          const name = r[0];
          let type = '';
          if (r[1]) {
            type = r[1].replace(/\s/g, '').toLowerCase();
          }

          if (name.startsWith('# Detailed Table Information')) {
            /*
             * newer hive describe formatted output does not
             * have a "# col_name" header - instead, you grab all the
             * column names from prior to the 'Detailed table information'
             * header
             */
            keepTrackOfOldRows = false;
            if (cols.length === 0) {
              storedRows.forEach((storedRow) => {
                get_type(storedRow[0], storedRow[1], '', cols);
              });
            }
          }

          if (keepTrackOfOldRows) {
            if (name !== '') {
              storedRows.push([name, type]);
            }
          }

          if (start_table_param === false && name === '') {
            return;
          }

          if (name.startsWith('# col_name')) {
            start_column = true;
            return;
          }

          if (name.startsWith('Table Parameters:')) {
            start_table_param = true;
            return;
          }

          if (start_column && name.startsWith('#')) {
            start_column = false;
          }

          if (start_table_param && name.startsWith('#')) {
            start_table_param = false;
          }

          if (start_column) {
            get_type(name, type, '', cols);
          } else if (start_table_param && 'pk_fk' in params) {
            if ((typeof (r[1]) === typeof ('')) &&
                (typeof (r[1]) === typeof (''))) {
              params.pk_fk.push([r[1].replace(/\s/g, '').toLowerCase(),
                                 r[2].replace(/\s/g, '').toLowerCase()]);
            }
          }
        });

        const select = [];
        const tables = [arc.dataconnection._enquote_tablename(params.fullname,
                                                            params.dctype)];
        cols.forEach((cl) => {
          const cname = cl[0];
          const ctype = cl[1];
          const quoted_col = [];

          if (!arc.datasets.is_complex_type(ctype)) {
            cname.split('.').forEach((c) => { quoted_col.push(`\`${c}\``); });
            select.push(`${quoted_col.join('.')} as \`${cname}\``);
          } else if (ctype !== 'struct') {
              tables.push(
                `${arc.dataconnection._enquote_tablename(
                  params.fullname,
                  params.dctype
                  )
                }.\`${cname}\``
              );
            }
        });

        if (select.length > 0) {
          const cmd = `select ${select.join(', ')} from ${tables.join(', ')} limit ${params.limit}`;
          sample(cmd);
        } else {
          const table = arc.dataconnection._enquote_tablename(params.fullname,
                                                              params.dctype);
          sample(`select * from ${table} limit ${params.limit}`);
        }
      },
      false);
};

// When the user clicks on multiple table rows, we end up sending out multiple
// requests which come back in to us later on. We have to ensure that we only
// act on the currently active request. Start each request with a unique setup
// id and then use it to ensure we are the current request before acting.
let currentSetupId = 1;

function setupAndDrawConnectionExplorerCurrentDatasetTable(dbname, tablename) {
  currentSetupId += 1;
  const thisSetupId = currentSetupId;

  setupSampleData(dbname, tablename, thisSetupId);
  setupDatasets(dbname, tablename);

  const dctype = arc.apps_data.active_dataconnection_type();
  if (arc.dataconnection.isArcEngineOrImpala(dctype)) {
    $('.tablestats-tab').show();
    $('.columnstats-tab').show();
    setupTableStats(dbname, tablename, thisSetupId);
    setupColumnStats(dbname, tablename, thisSetupId);
  } else {
    $('.tablestats-tab').hide();
    $('.columnstats-tab').hide();
  }

  if (isSolrConnection(dctype)) {
    $('.collections-tab').show();
    setupCollections(dbname, tablename, thisSetupId);
  } else {
    $('.collections-tab').hide();
  }
}

function setupDatasets(dbname, tableName) {
  const dc_id = arc.apps_data.active_dataconnection_id();
  if (dc_id <= 0) {
    return;
  }

  let tableToDataset = dcTableToDataset[dc_id];
  if (!tableToDataset) {
    tableToDataset = {};
  }

  const fullTableName = `${dbname}.${tableName}`;
      const datasets = tableToDataset[fullTableName];

  const datasets_divid = '#tabledatasets-table';
  drawDatasets({
    datasets_divid,
    datasets_count_divid: '#does_not_exist',
    active_datasets: datasets,
  });

  $('.datasets-tab > a').off('shown.bs.tab')
                        .on('shown.bs.tab', () => {
                          const $t = createdDataTables[datasets_divid];
                          if ($t) {
                            $t.columns.adjust();
                          }
                        });
}

function setupTableStats(dbname, tablename, thisSetupId) {
  const $tab = $('.tablestats-tab a');

  const $spinner = $('#tablestats-loading-spinner');
  $spinner.show();

  const $container = $('#tablestats-data-table-container');
  $container.empty();

  arc.apps_data.fetchTableStats({
    cache: false,
    dcid: arc.apps_data.active_dataconnection_id(),
    dctype: arc.apps_data.active_dataconnection_type(),
    fullname: `${dbname}.${tablename}`,
    cb(errorMsg, response) {
      if (thisSetupId !== currentSetupId) {
        return;
      }

      $spinner.hide();
      if (errorMsg) {
        $container.append(arc.clone_template('table-stat-failed',
                                            { errtext: arc.misc.escapeHTML(errorMsg) }).html());
      } else {
        const table = arc.misc.plotTable(response.rows, response.colnames, $container);
        $tab.off('shown.bs.tab')
            .on('shown.bs.tab', () => {
              table.adjustColumnWidths();
            });
      }
    },
  });
}

function setupColumnStats(dbname, tablename, thisSetupId) {
  const $tab = $('.columnstats-tab a');

  const $spinner = $('#columnstats-loading-spinner');
  $spinner.show();

  const $container = $('#columnstats-data-table-container');
  $container.empty();

  arc.apps_data.fetchColumnStats({
    cache: false,
    dcid: arc.apps_data.active_dataconnection_id(),
    dctype: arc.apps_data.active_dataconnection_type(),
    fullname: `${dbname}.${tablename}`,
    cb(errorMsg, response) {
      if (thisSetupId !== currentSetupId) {
        return;
      }

      $spinner.hide();

      if (errorMsg) {
        $container.append(arc.clone_template('table-stat-failed',
                                              { errtext: arc.misc.escapeHTML(errorMsg) }).html());
      } else {
        // #Nulls are inaccurate and not helpful. Removing. Only applies to Impala/arcengine
        const nullsIdx = response.colnames.indexOf('#Nulls');
        if (nullsIdx > -1) {
          response.colnames.splice(nullsIdx, 1);
          response.rows.forEach((r) => r.splice(nullsIdx, 1));
        }

        const table = arc.misc.plotTable(response.rows, response.colnames, $container);
        $tab.off('shown.bs.tab')
            .on('shown.bs.tab', () => {
              table.adjustColumnWidths();
            });
      }
    },
  });
}

function setupCollections(dbname, tablename, thisSetupId) {
  const $tab = $('.collections-tab a');
  const $spinner = $('#collections-loading-spinner');
  const $container = $('#collections-data-table-container');

  $spinner.show();
  $container.empty();

  fetchCollectionsfromAlias({
    cache: false,
    dcid: arc.apps_data.active_dataconnection_id(),
    dctype: arc.apps_data.active_dataconnection_type(),
    fullname: `${dbname}.${tablename}`,
    cb(error, response) {
      if (thisSetupId !== currentSetupId) {
        return;
      }

      $spinner.hide();

      if (error) {
        $container.append($('<div>Could not fetch collection aliases</div>'));
      } else {
        const table = arc.misc.plotTable(response.rows, response.colnames, $container);
        $tab.off('shown.bs.tab')
            .on('shown.bs.tab', () => {
              table.adjustColumnWidths();
            });
      }
    },
  });
}

function setupSampleData(dbname, tablename, thisSetupId) {
  const $tab = $('.sampledata-tab a');
  const $spinner = $('#sample-loading-spinner');
  const $container = $('#sample-data-table-container');

  $spinner.show();

  $container.empty();

  arc.apps_data.fetch_sample_data({
    limit: 100,
    cache: false,
    hideAlertsOnSuccess: true,
    dcid: arc.apps_data.active_dataconnection_id(),
    dctype: arc.apps_data.active_dataconnection_type(),
    fullname: `${dbname}.${tablename}`,
    cb(errorMsg, response) {
      if (thisSetupId !== currentSetupId) {
        return;
      }

      $spinner.hide();

      if (errorMsg) {
        arc.report.alertMessage('Could not fetch sample data', errorMsg);
      } else {
        const table = arc.misc.plotTable(response.rows, response.colnames, $container);
        $tab.off('shown.bs.tab')
            .on('shown.bs.tab', () => {
              table.adjustColumnWidths();
            });
      }
    } });
}

function populateTables(datasets) {
  dcTableToDataset = {};
  datasets.forEach((ds) => {
    if (!ds.info) {
      return;
    }

    let tableToDataset = dcTableToDataset[ds.dc_id];
    if (tableToDataset === undefined) {
      tableToDataset = {};
      dcTableToDataset[ds.dc_id] = tableToDataset;
    }

    let tables = JSON.parse(ds.info).map((d) => d.tablename);

    tables = arc.misc.arrayUnique(tables);
    tables.forEach((t) => {
      if (t in tableToDataset) {
        tableToDataset[t].push(ds);
      } else {
        tableToDataset[t] = [ds];
      }
    });
  });
}

const extractLoadingSpinner = $('.tab-pane#extracts .loading-spinner');

function fetchExtracts(cb) {
  extractLoadingSpinner.show();
  const assets = [
    {
      asset: `${arc.frontend.globals.urls.extractUrl}?details=true`,
      postprocess: d3.json,
    },
    {
      asset: arc.frontend.globals.urls.dataCopyLogsUrl,
      postprocess: d3.json,
    },
  ];

  arc.staticfetcher.fetch(assets,
    (error, extracts, extractLogs) => {
      if (error) {
        isExtractsFetched = false;
        // eslint-disable-next-line no-console
        console.error('Could not load extracts');
        // eslint-disable-next-line no-console
        console.error(error);
        return;
      }

      extractTracker = arc.extracts.extractTracker();
      extractTracker.trackExtracts(extracts, extractLogs);

      cb();
      extractLoadingSpinner.hide();
    });
}

const datasetsLoadingSpinner = $('.tab-pane#datasets .loading-spinner');

/* DATA Page onLoad some global state are initialized (Connection lists,
 all datasets(or partial one on case SQL page), tables and their relationship)
 To avoid retreieiving all information from the backend when we modify something,
 worth just update the Frontend if a dataset was deleted or added for example */
const updateDatasetRelatedGlobalInformations = (datasets, overwrite) => {
  allDatasets = overwrite ? datasets : Array.from(new Set([...allDatasets, ...datasets]));
  datasetIdToDataset = overwrite ? {} : datasetIdToDataset;

  datasets.forEach((d) => {
    datasetIdToDataset[d.id] = d;
  });
  populateTables(allDatasets);
};

function fetchData(cb) {
  const get_associate_dashboard = arc.frontend.featureFlipper.getValueById('enable_counting_dashboards_for_datasets');

  const assets = [
    {
      asset: `${arc.frontend.globals.urls.datasetUrl}?get_associate_dashboard=${get_associate_dashboard}&data_page=true`,
      postprocess: d3.json,
    },
    {
      asset: `${arc.frontend.globals.urls.dataconnectionUrl}?import_data=true`,
      postprocess: d3.json,
    },
    {
      asset: arc.frontend.globals.urls.scheduleGroupUrl,
      postprocess: d3.json,
    },
  ];

  arc.staticfetcher.forget(assets);

  datasetsLoadingSpinner.show();
  arc.staticfetcher.fetch(assets,
    (error, datasets, dataconnections, schedules) => {
      if (error) {
        datasetsLoadingSpinner.hide();
        // eslint-disable-next-line no-console
        console.log('Could not load data');
        // eslint-disable-next-line no-console
        console.log(error);
        return;
      }

      allSchedules = {};
      if (schedules) {
        schedules.schedule_groups.forEach((g) => {
          allSchedules[g.id] = g.schedule_name;
        });
      }

      arcl.dataconnections = dataconnections;
      const dc = arc.apps_data.get_active_dataconnection();
      if (dc) {
        $('#cmdline .dcname').text(dc.name);
      }

      updateDatasetRelatedGlobalInformations(datasets, true);

      switch (window.location.hash) {
        case '#tables':
          $(`#data-tabs a[href="#tables"]`).tab('show');
          break;
        case '#dataset-modal':
          if (window.location.search.includes('&sqlquery')) {
            popupCreateDatasetModal({ dataSource: 'sql', param: arc.misc.getQueryParameters().sqlquery });
          } else {
            popupCreateDatasetModal({ dataSource: 'table' });
          }

          break;
        case '#connection-modal':
          arc.frontend.legacyBridge.events.next({
            type: 'open-data-connection-modal',
          });
          break;
        default:
      }

      cb();
      datasetsLoadingSpinner.hide();
    });
}

arc.apps_data.set_active_dataconnection = function(dc) {
  if (!dc) {
    arc.apps_data.set_active_dataconnection_id_cookie(0);
    $('#dataset-buttons').addClass('arc-invisible');
  } else {
    arc.apps_data.set_active_dataconnection_id_cookie(dc.id);
    $('#cmdline .dcname').text(dc.name);
    $('#dataset-buttons').removeClass('arc-invisible');
  }
};

arc.apps_data.set_active_dataconnection_id_cookie = function(dc_id) {
  arc.frontend.vizCookie.set(
    arc.apps_data.cookie_active_data_connection_id(),
    dc_id,
    { expires: 365 }
  );
};

arc.apps_data.get_active_dataconnection = function() {
  let activeDataConnection;

  // initialDataConnectionName is passed from context on the backend side
  if (initialDataConnectionName) {
    activeDataConnection = arcl.dataconnections?.find(
      ({ name }) => initialDataConnectionName === name
    );

    if (activeDataConnection) {
      arc.apps_data.set_active_dataconnection(activeDataConnection);
    }

    initialDataConnectionName = undefined;
  }

  // if initialDataConnectionName was not set or the data connection is not available
  // set the data connection based on the cookies
  if (!activeDataConnection) {
    const activeDataConnectionIdForCurrentUser = Number(arc.frontend.vizCookie.get(
      arc.apps_data.cookie_active_data_connection_id()
    ));
    const activeDataConnectionIdCookieNotSet = Number.isNaN(activeDataConnectionIdForCurrentUser);

    // 0 -> All data connections -> Only on Data page
    if (!isSqlPage && (
      activeDataConnectionIdCookieNotSet ||
      activeDataConnectionIdForCurrentUser === 0
    )) {
      return undefined;
    }

    activeDataConnection = arcl.dataconnections?.find(
      ({ id }) => id === activeDataConnectionIdForCurrentUser
    );
  }

  return activeDataConnection ?? arcl.dataconnections[0];
};

arc.apps_data.active_dataconnection_id = function() {
  const dc = arc.apps_data.get_active_dataconnection();
  if (dc) {
    return dc.id;
  }

  return 0;
};

arc.apps_data.active_dataconnection_name = function () {
  const dc = arc.apps_data.get_active_dataconnection();
  if (dc) {
    return dc.name;
  }

  return '';
};

arc.apps_data.active_dataconnection_type = function() {
  const dc = arc.apps_data.get_active_dataconnection();
  if (dc) {
    return dc.type;
  }

  return -1;
};

arc.apps_data.highlight_active_connection = function() {
  const dc = arc.apps_data.get_active_dataconnection();
  if (dc) {
    d3.select('#data-list')
      .select('.data-menu .active')
      .text(dc.name);
  }
};

arc.apps_data.load_data_import_data_connection = function() {
  // Fill in the active data conn id within the dataimport modal also
  $('#id_datasource_active_dataconnid').val(arc.apps_data.active_dataconnection_id());
  $('#viz_dash_import_active_dataconnid').val(arc.apps_data.active_dataconnection_name());
};

function setupDataImportGet() {
  const $addDataButton = $('#import-data-templ');
  const dctype = arc.apps_data.active_dataconnection_type();
  const dcid = arc.apps_data.active_dataconnection_id();
  const activeDc = arc.apps_data.get_active_dataconnection();

  if (dcid !== 0) {
    if (!activeDc.supportsImportData) {
      $addDataButton.addClass('arc-invisible');
    } else {
      $addDataButton.removeClass('arc-invisible');
    }
  }

  $addDataButton.on('click', () => {
    const activeConnection = arc.apps_data.get_active_dataconnection();
    const activeConnectionInfo = JSON.parse(activeConnection.info);
    const connectionMaxRowUpload = activeConnectionInfo.PARAMS.UPLOAD_MAXROWS;

    arc.frontend.legacyBridge.events.next({
      type: 'open-import-data-modal',
      payload: {
        connectionId: activeConnection.id,
        connectionMaxRowUpload,
        connectionType: activeConnection.type,
      }
    });
  });

  // Disable importing data into an existing sqlite dc in CDW. In CDW we lack
  // a persistent filesystem for a sqlite database to reside in
  if (dctype === 'sqlite' && arc.globals.arcviz_settings.CDV_MODE === 'cdw') {
    $addDataButton.addClass('disabled');
    $addDataButton.parent().tooltip('enable');
  } else {
    $addDataButton.removeClass('disabled');
    $addDataButton.parent().tooltip('disable');
  }

  if (dcid !== 0) {
    if (arc.misc.hasPerm('dc_upload', arc.apps_data.get_active_dataconnection().perms)) {
      d3.select('#datasource-get-import').on('click', () => {
        const filename = $('#id_datasource_file').val();
        if (filename) {
          $('#datasource-get-import-spinner').show();
        }
      });
    }
  }
}

function setupCacheReset() {
  $('#cache-reset-btn').offOn('click', function resetCache() {
    arc.dataconnection.reset_cache(arc.apps_data.active_dataconnection_id(), (err) => {
      if (err) {
        arc.report.alertMessageSmall(`Error on cache reset: ${err}`, 'alert-danger');
      } else {
        arc.report.alertMessageSmall('Connection result cache cleared!', 'alert-success');
      }
    });
  });
}

// makes sure that the user never sees the ellipsis dropdown as standalone. If don't have privileges
// to add see existing buttons, move dropdown items out of dropdown list. If do have privileges and
// dropdown items already moved, move relocated dropdown items back.
function rearrangeButtons() {
  const $btnContainer = $('#dataset-buttons');
  const $visibleButtons = $btnContainer.find('> a:visible');
  const $movedButtons = $visibleButtons.filter('.btn-moved');
  const $originalButtons = $visibleButtons.filter(':not(.btn-moved)');
  if (!$visibleButtons.length && $btnContainer.is(':visible')) {
    $btnContainer.prepend($btnContainer.find('.dropdown-menu a').addClass('btn btn-sm btn-moved').removeClass('dropdown-item'));
    $btnContainer.find('.more-btn').addClass('arc-invisible');
  } else if ($originalButtons.length && $movedButtons.length) {
    const $dropdownList = $btnContainer.find('div.dropdown-menu');
    $dropdownList.append($movedButtons);
    $movedButtons.removeClass('btn btn-sm btn-moved').addClass('dropdown-item');
    $btnContainer.find('.more-btn').removeClass('arc-invisible');
  } else if ($originalButtons.length) {
    $btnContainer.find('.more-btn').removeClass('arc-invisible');
  }
}

  arc.apps_data.draw = function () {
    drawDatasetsByConnectionOnDatasetsTab();
    drawDataTables();
    drawViews();
    drawExtracts();
    drawCommandLineAccess();
    setupDataImportGet();
    setupCacheReset();
    rearrangeButtons();
    showBookmarks();
    showHistory();
  };

  async function getDataconnections() {
    try {
      const response = await arc.frontend.vizHttp.get({
        url: arc.frontend.globals.urls.dataconnectionUrl,
        headers: {
          'X-CSRFToken': arc.globals.csrf_token,
        },
      });

      return response.data;
    } catch (error) {
      // eslint-disable-next-line no-console
      console.error('Could not load dataconnections', error);
      throw error;
    }
  }

  function is_direct_accessible(dataconnection_type) {
    return arc.misc.directAccessTypes.includes(dataconnection_type) ||
      arc.dataconnection.conns_direct_accessible().includes(dataconnection_type);
  }

  function is_connection_direct_accessible(dataconnection) {
    if (dataconnection) {
      return (
        arc.misc.hasPerm('dc_explore', dataconnection?.perms) &&
        is_direct_accessible(dataconnection?.type)
      );
    }

    return false;
  }

  const $connectionOptions = $('#data-connection-selector');

  const directAccessErrors = {
    noDataconnection: 'There are no available data connections.',
    noDirectAccessSupport: 'There are not data connections that support sql querying',
    failedToLoad: 'Could not load connections.',
  };

  const loadDatasetsOfConnection = (dataconnectionId) =>  new Promise((resolve, reject) => {
    const get_associate_dashboard = arc.frontend.featureFlipper.getValueById('enable_counting_dashboards_for_datasets');
    const assets = [
      {
        asset: `${arc.frontend.globals.urls.datasetUrl}?get_associate_dashboard=${get_associate_dashboard}&dataconnection_id=${dataconnectionId}`,
        postprocess: d3.json,
      },
    ];

    arc.staticfetcher.forget(assets);
    arc.staticfetcher.fetch(assets,
      (error, datasets) => {
        datasetsLoadingSpinner.hide();
        if (error) {
          reject(error);
        }

       resolve(datasets);
      });
    });

  function setupDataConnectionSelectionList() {
    // [CA] pulled code from apps_data.html -- setup connection explorer
    $connectionOptions.empty();

    if (!arcl.dataconnections?.length) {
      return { notificationMessage: null, error: new Error(directAccessErrors.noDataconnection) };
    }

    arcl.dataconnections.forEach((connection) => {
      $connectionOptions.append(
        `<option value="${connection.id}" title="${connection.description}">
          ${connection.name}
        </option>`
      );
    });

    // It is either given in the url params,
    // or was previously set as the active dataconnection
    const selectedDataconnecionId =
      parseInt(arc.misc.getQueryParameters().dataconnection_id, 10) ||
      parseInt(arc.apps_data.active_dataconnection_id(), 10);

    const selectedConnection =
      arcl.dataconnections.find((c) => parseInt(c.id, 10) === selectedDataconnecionId);
    const handleConnectionChangeOfDirectAccess = async (connection_id) => {
      arc.apps_data.setSqlQuery();
      arc.apps_data.set_active_dataconnection_id_cookie(connection_id);
      hideQueryActions();
      try {
        const datasets = await loadDatasetsOfConnection(connection_id);
        updateDatasetRelatedGlobalInformations(datasets, false);
      } catch (error) {
        arc.report.alertMessage('Could not fetch dataset related to the selected connection', error);
      }

      arc.apps_data.draw();
    };

    $connectionOptions.offOn('change.conn', async (event) => {
      const connection_id = event.target.value;
      handleConnectionChangeOfDirectAccess(connection_id);
      $connectionOptions.trigger('chosen:updated');
    });

    $connectionOptions.val(selectedConnection?.id);
    $connectionOptions.trigger('chosen:updated');
    return { selectedConnection };
  }

  const $databaseAndTables = $('.database-and-tables');
  arc.apps_data.setDefaultDatabaseAndTable = (databaseName, dataTable) => {
      if (databaseName) {
        arc.apps_data.cookie_set_defaultdb(databaseName);
      }

      if (dataTable) {
        arc.apps_data.cookie_set_default_datatable(dataTable);
      }
  };

  arc.apps_data.initialDataSqlHtml = async function (
      dataconnectionId,
      dashboardTemplateEnabled,
      databaseName,
      dataTable
    ) {
        isSqlPage = true;

        if (!arc.frontend.featureFlipper.getValueById('enable_direct_access')) {
          // The page will be empty no need to initialize it!
          return;
        }

        const tabLink = window.location.hash;
        $(`#datatable-selector-and-sql-editor-container .nav-tabs a[href="${tabLink}"]`).tab('show');
        arc.apps_data.setSqlOnly(tabLink === '#sql-tab');
        arc.apps_data.dashboard_template_enabled(dashboardTemplateEnabled);

        try {
          const dataconnections = await getDataconnections();

          arcl.dataconnections = dataconnections || [];
          const connection = arcl.dataconnections.find(
            (dataconnection) => dataconnection.id === parseInt(dataconnectionId, 10)
          );

          const directAccess = arc.apps_data.directAccess();
          if (connection?.name) {
            directAccess.initialDataconnectionName(connection.name);
          }

          if (connection?.id) {
            arc.apps_data.set_active_dataconnection_id_cookie(connection.id);
          }

          await directAccess.initialize();
          arc.apps_data.setDefaultDatabaseAndTable(databaseName, dataTable);
          // Handle tab changes
          $('#main-nav a[data-toggle=tab]').offOn('click.tabChanges', (e) => {
            const directAccessPageTablink = e.target.getAttribute('href');
            document.location.hash = directAccessPageTablink;
            // Handle direct access
            if (directAccessPageTablink === '#sql-tab') {
              arc.apps_data.setSqlOnly(true);
              arc.apps_data.draw();
            } else {
              arc.apps_data.setSqlOnly(false);
              arc.apps_data.draw();
            }
          });

          // js can be moved to another file
          $('.close-section-btn').offOn('click.closeSection', () => {
            $databaseAndTables.toggleClass('database-and-tables-closed');
            $('.rawcommand-holder-holder').css({
              maxWidth: `calc(100vw - ${$databaseAndTables.outerWidth(true)}px)`
            });
          });

          showActiveDirectAcessMenu(arc.misc.getQueryParameters().active_data_menu);
        } catch (error) {
          arc.report.alertMessageSmall('Could not load connections', 'alert-danger');
          enableDirectAccessComponents({
                enableDirectSqlAccessSection: false,
                enableConnectionSelector: false,
                explanation: `${directAccessErrors.failedToLoad} (${error})`,
          });
        }
      };

function addMissingPasswordPopover($popoverSelector) {
  const popoverId = `popover-visual-${arc.misc.uniqueId()}`;
  const content =
    arc.misc.safe_substitute(arc.misc.howToSetWorkloadPasswordText,
    {
      POPOVER_ID: popoverId,
      TITLE: 'Setting Workload password steps',
      PRODUCT_NAME: arc.frontend.globals.product.name,
    });
  const setting = {
    container: 'body',
    boundary: 'window',
    title: 'Setting Workload password steps',
  };

  arc.misc.setupMarkablePopover({ $popoverSelector, content, popoverId, setting });
}

const connectionAvailableIconTemplate = `
<svg width="16px"
  height="16px"
  viewBox="0 0 24 24"
  version="1.1"
  data-toggle="tooltip"
  title="Project synced connection"
>
    <g id="Symbols" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
        <g class='cml-status-icon' id="system-icons/data-flow" fill="#000000">
            <path d="M19,20 C18.448,20 18,19.552 18,19 C18,18.448 18.448,18 19,18 C19.552,18 20,18.448 20,19 C20,19.552 19.552,20 19,20 M12,10 C13.103,10 14,10.897 14,12 C14,13.103 13.103,14 12,14 C10.897,14 10,13.103 10,12 C10,10.897 10.897,10 12,10 M5,6 C4.448,6 4,5.552 4,5 C4,4.448 4.448,4 5,4 C5.552,4 6,4.448 6,5 C6,5.552 5.552,6 5,6 M19,16 C17.695,16 16.597,16.838 16.184,18 L6.5,18 C6.332,18 6.163,17.983 5.999,17.949 C4.841,17.715 4,16.685 4,15.5 C4,14.121 5.121,13 6.5,13 L8.142,13 C8.588,14.723 10.139,16 12,16 C13.861,16 15.412,14.723 15.858,13 L17.5,13 C17.801,13 18.104,12.97 18.399,12.91 C20.485,12.486 22,10.632 22,8.5 C22,6.019 19.981,4 17.5,4 L7.816,4 C7.403,2.838 6.305,2 5,2 C3.343,2 2,3.343 2,5 C2,6.657 3.343,8 5,8 C6.305,8 7.403,7.162 7.816,6 L17.5,6 C18.879,6 20,7.121 20,8.5 C20,9.685 19.159,10.715 18.002,10.949 C17.837,10.983 17.668,11 17.5,11 L15.858,11 C15.412,9.277 13.861,8 12,8 C10.139,8 8.588,9.277 8.142,11 L6.5,11 C4.019,11 2,13.019 2,15.5 C2,17.632 3.515,19.486 5.602,19.91 C5.896,19.97 6.199,20 6.5,20 L16.184,20 C16.597,21.162 17.695,22 19,22 C20.657,22 22,20.657 22,19 C22,17.343 20.657,16 19,16" id="Fill-1"></path>
        </g>
    </g>
</svg>
`;

const connectionUnAvailableIconTemplate = `
<svg
  width="16px"
  height="16px"
  viewBox="0 0 24 24"
  version="1.1"
  data-toggle="tooltip"
  title="Project synced connection now unavailable for further update"
>
  <g id="Symbols" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
    <g class='cml-status-icon' id="status-icons/inactive" fill="#838B92">
      <path d="M8.9580665,5.56712036 L5.21670623,5.56712036 L5.21670623,4.0004 L11.8223556,4.0004 L7.88879082,11.4336796 L12.0002,11.4336796 L12.0002,13.0004 L5.0002,13.0004 L8.9580665,5.56712036 Z M15.7622,2.0004 C17.7292,5.6984 16.7092,10.5934 13.6872,13.6754 C11.7272,15.6734 8.9102,16.7424 6.1892,16.7424 C4.7142,16.7424 3.2672,16.4284 2.0002,15.7784 C4.3722,20.2914 8.1382,21.9554 11.2232,22.0004 C14.1512,22.0004 16.7612,20.8904 18.8572,18.8244 C22.9492,14.7514 24.1002,6.2424 15.7622,2.0004 L15.7622,2.0004 Z" id="moonman"></path>
    </g>
  </g>
</svg>
`;

const warningIconTemplate = '<span><i class="fa fa-exclamation-circle" aria-hidden="true"></i></span>';

function createEditConnectionLink(dataconnection) {
  const $editConnectionLink = $($.parseHTML(`<a><i class="fa fa-pencil" data-toggle="tooltip" title="Edit Connection"></i></a>`));
  $editConnectionLink.offOn('click.openEditConnectionsModal', () => {
    const active_datasets = allDatasets.filter(
      (dataset) => parseInt(dataconnection.id, 10) === dataset.dc_id
    );

    arc.frontend.legacyBridge.events.next({
        type: 'open-data-connection-modal',
        payload: {
          dataconnectionId: dataconnection.id,
          associatedDatasetLength: active_datasets.length,
        }
      });
  });
  return $editConnectionLink;
}

const dataconnectionListId = '#dataconnections-list';

function updateVisibilityOfConnectionExplorerTab() {
  // Connection List has a special list item: "All Connection".
  // It summarizes  the dataconnections' available informations like datasets.
  // When it is selected Connection Explorer tab should not be visible
  // When a real connection is selected we should display it (if possible).
  const activeConnection = arc.apps_data.get_active_dataconnection();
  if (activeConnection) {
    arc.misc.updateDivForPerms(activeConnection.perms, 'dataconn', $data_tabs);
  } else {
    $data_tabs.find('#tab-datasource').closest('li.nav-item').addClass('arc-invisible');
    // fallback to dataset tab
    if (window.location.hash === '#tables') {
      window.location.hash = '#datasets';
      $('#tab-dataset').tab('show');
    }
  }
}

function onConnectionNameClick(dataconnection) {
  // Don't let old count linger as switching between connections
  $('#extracts-count').empty();
  $('#aviews-count').empty();

  arc.report.alertMessageAllHide();
  arc.apps_data.set_active_dataconnection(dataconnection);
  arc.apps_data.load_data_import_data_connection();

  $(dataconnectionListId).find('li').removeClass('active');

  let dc_perms = [];

  if (dataconnection) {
    // eslint-disable-next-line no-restricted-globals
    if (!is_direct_accessible(dataconnection.type) && location.hash === '#cmdline') {
      window.location.hash = '#datasets';
      $('#tab-dataset').tab('show');
    }

    dc_perms = dataconnection.perms;
  }

  // First update buttons so that rearrangeButtons properly redraws the connection buttons
  arc.misc.updateDivForPerms(dc_perms, 'dataconn', $('#dataset-buttons'));
  updateVisibilityOfConnectionExplorerTab();
  arc.apps_data.draw();
}

function createOpenConnectionDetailsLink(dataconnection) {
  const $connectionLink = $($.parseHTML(`<a title="${dataconnection.name}"><i class='fa fa-link'></i><span>${dataconnection.name}</span></a>`));
  $connectionLink.offOn('click', () => {
    // Don't let old count linger as switching between connections
    onConnectionNameClick(dataconnection);
    $connectionLink.closest('li').addClass('active');
  });
  return $connectionLink;
}

function getCmlStatusIcon(dataconnection) {
  let $cmlStatusIcon = $();
  if (arc.frontend.featureFlipper.getValueById('enable_cml_sync')) {
    if (dataconnection.cmlId !== null) {
      if (!dataconnection.isPasswordSet) {
        $cmlStatusIcon = $($.parseHTML(warningIconTemplate));
        addMissingPasswordPopover($cmlStatusIcon);
      } else {
        const cmlStatus = dataconnection.cmlStatus ? JSON.parse(dataconnection.cmlStatus) : null;
        $cmlStatusIcon = cmlStatus?.isAvailable ?
          $($.parseHTML(connectionAvailableIconTemplate)) :
          $($.parseHTML(connectionUnAvailableIconTemplate));
      }
    }
  }

  return $cmlStatusIcon;
}

function createDataconnectionListItem(dataconnection) {
  const $dataconnectionListItem = $($.parseHTML(`<li class=${dataconnection?.id === arc.apps_data.active_dataconnection_id() ? 'active' : ''}></li>`));
  const $connectionLink = createOpenConnectionDetailsLink(dataconnection);

  const $iconContainer = $($.parseHTML('<div class="icon-container-horiontal"></div>'));
  $dataconnectionListItem.append($connectionLink);

  if (arc.misc.sysPerm('sys_editconn')) {
    $iconContainer.append(createEditConnectionLink(dataconnection));
  }

  const $cmlStatusIcon = getCmlStatusIcon(dataconnection);
  $iconContainer.append($cmlStatusIcon);
  $dataconnectionListItem.append($iconContainer);

  return $dataconnectionListItem;
}

function createAllConnectionListElement() {
  const $allConnectionLink = $($.parseHTML(`<a title="All Connections"><i class="fa fa-globe"></i><span>All Connections</span></a>`));
  $allConnectionLink.offOn('click', () => {
    onConnectionNameClick(undefined);
    $allConnectionLink.closest('li').addClass('active');
  });
  const $allConnectionListElement = $($.parseHTML(`<li class='d-flex justify-content-between ${arc.apps_data.active_dataconnection_id() === 0 ? 'active' : ''}' data-test-id="all-connections-list"></li>`));
  $allConnectionListElement.append($allConnectionLink);

  if (arc.frontend.featureFlipper.getValueById('enable_cml_sync') && arc.misc.sysPerm('sys_editconn')) {
    const $syncButton = arc.dataconnection.createSyncButton();
    $allConnectionListElement.append($syncButton);
  }

  return $allConnectionListElement;
}

arc.apps_data.draw_dataconnections = function () {
  const $dcList = $(dataconnectionListId);
  $dcList.empty();
  arcl.dataconnections.sort((a, b) => {
    if (a.name.toLowerCase() < b.name.toLowerCase()) {
      return -1;
    }

    if (a.name.toLowerCase() > b.name.toLowerCase()) {
      return 1;
    }

    return 0;
  });

  arc.apps_data.load_data_import_data_connection();

  $dcList.append(createAllConnectionListElement());
  $dcList.append('<div class="line-separator"></div>');

  arcl.dataconnections.forEach((dc) => {
    const $connectionLi = createDataconnectionListItem(dc, $dcList);
    $dcList.append($connectionLi);
  });
};

function addLimitToSQL(cmd) {
  const addLimit = $('#limit-100').prop('checked');
  if (!addLimit) {
    return cmd;
  }

  return arc.misc.addLimitToSQLQuery(cmd, 100);
}

  arc.apps_data.setSqlQuery = function (value) {
    if (value) {
      arc.apps_data.initialQueries(value);
      expressionEditorOnDirectAccessPage?.aceEditor().getSession().setValue(value);
    } else {
      arc.apps_data.initialQueries(null);
      expressionEditorOnDirectAccessPage?.aceEditor().getSession().setValue('');
    }
  };

const $requestContainer = $('#request-actions');
const $runSqlButton = $requestContainer.find('#rawcommand-run');
const $cancelRequestButton = $requestContainer.find('#rawcommand-cancel');

function disableElement($elem) {
  $elem.prop('disabled', true);
}

function enableElement($elem) {
  $elem.prop('disabled', false);
}

function setButtonToImplyTaskIsRunning($button) {
  $button.find('.default-icon').hide();
  $button.find('.loading-icon').show();
  disableElement($button);
}

function setButtonToImplyDefaultTask($button) {
  $button.find('.default-icon').show();
  $button.find('.loading-icon').hide();
  enableElement($button);
}

const $numberRowsInput = $('#rawcommand-number-rows-csv-input');
const $downloadCSV = $('#rawcommand-download-csv');

const $dataProfilingNavItem = $('#menu-data-profiling');
arc.apps_data.runQuery = function(
  cmd,
  dataconnectionId,
  simpleTable,
  isBookMarkedQuery,
) {
  arc.misc.exChangeElementsVisibility(
    { elementToShow: $cancelRequestButton, elementToHide: $runSqlButton }
  );
  const $tableContainer = $('#rawcommand-output');

  // Add in a limit clause to the SQL if necessary
  cmd = addLimitToSQL(cmd);

  // clear previous results
  $tableContainer.empty();

  const progress = arc.misc.safe_substitute($('#rawcommand-progress-template').html(),
                                          { ATTR_ID: 'rawcommand-progress' });
  $tableContainer.append($(progress));

  setActiveQuery(cmd);

  const loader = arc.loadbar()
                    .dom_id('#rawcommand-progress')
                    .refresh_icon(true);

  const queryCmd = arc.data.queryCmd().query(cmd)
                                      .dataconnection_id(dataconnectionId)
                                      .cache_read(false);

  let error_shown;
  function show_error(error) {
    arc.misc.exChangeElementsVisibility(
      { elementToShow: $runSqlButton, elementToHide: $cancelRequestButton }
    );
    if (!error_shown) {
      error_shown = error;
      // eslint-disable-next-line no-console
      console.log(error);
      $tableContainer.html(`<h5>Error<h5><pre>${error}</pre>`);
    }
  }

  $downloadCSV.offOn('click.download', function downloadAsCsvResults() {
    setButtonToImplyTaskIsRunning($downloadCSV);

    const numberOfRowsToDownload = $numberRowsInput.val();
    const command = arc.misc.addLimitToSQLQuery(getCmdOrError(true), numberOfRowsToDownload);
    const queryCommand = arc.data
      .queryCmd()
      .query(command)
      .dataconnection_id(dataconnectionId)
      .cache_read(false);

    // Re-run the query to avoid the original #limit-100 modifier!

    arc.data.runQuery(
      undefined,
      function successcb(response) {
        if (response.colnames.length === 0) {
          if (response.rows && response.rows.length > 0) {
            response.colnames = [];
            response.rows[0].forEach(() => {
              response.colnames.push('');
            });
          }
        }

        const languageOverride = arc.visual
          .defaultSettings({ noTrellis: true })
          .find_by_id('default_language')
          .getValue().value;

        const fileName = arc.misc.getTimestampFilename('query-results', 'csv');
        const shouldDownload = true;
        arc.misc.downloadCsv(
          fileName,
          response.colnames,
          response.rows,
          response.coltypes,
          languageOverride,
          numberOfRowsToDownload,
          'false',
          undefined,
          shouldDownload
        );

        setButtonToImplyDefaultTask($downloadCSV);
      },

      show_error,
      /* lib */ undefined,
      function errorReportingCb(qstate, msg, error) {
        arc.report.alertMessage(msg, error);
        show_error(`${msg}: ${error}`);
        setButtonToImplyDefaultTask($downloadCSV);
      },

      undefined,
      undefined,
      queryCommand,
      loader
    );
  });

  function createDatasetObjectFromResponseObject(response, name) {
    const create = arc.datasets.createFromSql();
    const dc = arc.apps_data.get_active_dataconnection();
    // Response example:
    // { "info":["Query:select fat_grams, fat_grams*2 as 'alma' from main.cereals limit 100"] }
    const sql = response.info[0].split('Query:')[1];
    create.dataconnection_id(dc.id)
          .dataconnection_type(dc.type)
          .tgt_dataset_name(name)
          .sql(sql);

    return create.initializeDatasetObjectFromQueryResult(response);
  }

  function setupDataProfileTab(response) {
    const connectionType = arc.apps_data.get_active_dataconnection().type;
    if (arc.misc.directAccessTypes.includes(connectionType) && response.rows.length > 0) {
      $dataProfilingNavItem.show();
      const tempDatasetObject = createDatasetObjectFromResponseObject(response, 'temporary dataset for data profiling');
      arc.frontend.legacyBridge.events.next({ type: 'profile-dataset-event', payload: { dataset: tempDatasetObject, title: 'Data Profiling' } });
    } else {
      $dataProfilingNavItem.hide();
    }
  }

  const baseQuery = arc.data.runQuery(
    undefined,
    function successcb(response) {
      arc.sql.displayedQuery()
      .dataconnection_id(dataconnectionId)
      .query(cmd)
      .store();

      arc.misc.exChangeElementsVisibility(
        { elementToShow: $runSqlButton, elementToHide: $cancelRequestButton }
      );
      if (response.colnames.length === 0) {
        if (response.rows && response.rows.length > 0) {
          response.colnames = [];
          response.rows[0].forEach(() => {
            response.colnames.push('');
          });
        }
      }

      if (response.rows.length > 0) {
        arc.misc.plotTable(response.rows, response.colnames, $tableContainer, simpleTable);
      } else {
        $tableContainer.html(`<div style='text-align:center'>No data availale</div>`);
      }

      setupDataProfileTab(response);
      showQueryActions(isBookMarkedQuery);
    },

    show_error,
    /* lib */ undefined,
    function errorReportingCb(qstate, msg, error) {
      arc.report.alertMessage(msg, error);
      show_error(error);
      hideQueryActions();
    },

    undefined,
    undefined,
    queryCmd,
    loader
    );

  $cancelRequestButton.offOn('click.cancel', () => {
    baseQuery.qstate.cancel();
    show_error('Cancelled');
  });
};

  const $cmdTitleHolder = $('#rawcommand-output-title');
  const $title = $cmdTitleHolder.find('.title');
  const $clouderaCmdLineActions = $('.cloudera-cmdline-actions');
  const $bookmarkLink = $('.bookmark-query');
  const $bookmarkIcon = $bookmarkLink.find('i');
  const $numberRowsCSV = $('#rawcommand-number-rows-csv');
  const $numberRowsInputWarning = $('#allowOnlyNumbers-feedback');
  const $emailWarning = $('#mail-feedback');

  function disableWarning($input, $warning) {
    $warning.hide();
    $input.css('border', '1px solid #767676');
  }

  function enableWarning($input, $warning) {
    $input.css('border', '1px solid red');
    $warning.show();
  }

  function resetDownloadCSVSection() {
    disableWarning($numberRowsInput, $numberRowsInputWarning);
    disableElement($downloadCSV);
    $numberRowsInput.val('');
  }

  function displayTitle(title) {
    $cmdTitleHolder.removeClass('arc-invisible');
    $title.text(title);
  }

  function showQueryActions(isBookMarked) {
    // Hack - todo
    $clouderaCmdLineActions.show();
    $moreOptionSection.hide();
    resetDownloadCSVSection();
    $displayMoreOptionButtonIconDown.show();
    $displayMoreOptionButtonIconUp.hide();
    if (!arc.frontend.featureFlipper.getValueById('enable_cloudera_style')) {
      $numberRowsCSV.show();
    }

    if (isBookMarked) {
      $bookmarkIcon.addClass('fa-bookmark');
      $bookmarkIcon.removeClass('fa-bookmark-o');
    } else {
      $bookmarkIcon.addClass('fa-bookmark-o');
      $bookmarkIcon.removeClass('fa-bookmark');
    }
  }

function hideQueryActions() {
  $clouderaCmdLineActions.hide();
  $moreOptionSection.hide();
  resetDownloadCSVSection();
  $displayMoreOptionButtonIconDown.show();
  $displayMoreOptionButtonIconUp.hide();

  if (!arc.frontend.featureFlipper.getValueById('enable_cloudera_style')) {
    $numberRowsCSV.hide();
  }
}

function setActiveQuery(query) {
  $cmdTitleHolder.attr('data-cmd', '');
  if (query === undefined) {
    $title.empty();
    return;
  }

  query = query.trim();
  if (query.length === 0) {
    return;
  }

  $cmdTitleHolder.attr('data-cmd', query);
  displayTitle(query);
}

function getCmdOrError(showActive) {
  let cmd = expressionEditorOnDirectAccessPage.value();
  const selection = expressionEditorOnDirectAccessPage.getSession().getSelection();
  if (!selection.isEmpty()) {
    const range = selection.getAllRanges()[0];
    cmd = expressionEditorOnDirectAccessPage.getSession().getTextRange(range);
  }

  cmd = cmd.trim();

  while (cmd.length > 0) {
    if (cmd[cmd.length - 1] === ';') {
      cmd = cmd.substring(0, cmd.length - 1);
    } else {
      break;
    }
  }

  cmd = cmd.trim();
  if (showActive) {
    setActiveQuery(cmd);
  }

  if (cmd.length === 0) {
    return undefined;
  }

  return cmd;
}

  function activateHistory() {
    const $links = $historyHolder.find('.search-history .search-history-section div a');

    $links.offOn('click.history', function () {
      const $this = $(this);
      activateQuery($this.text());
    });
  }

  const rowTemplate = $('#search-history-template').html();
  function showHistory() {
    const maxHistoryDisplay = 1000;
    $historyHolder.html('<div class="search-history"></div>');
    const dataconnectionId = arc.apps_data.active_dataconnection_id();

    const showEmptyMessage = ($container) => {
      $container.html('<div class="placeholder-text">No queries have been created yet.</div>');
    };

    const $historyDiv = $historyHolder.find('.search-history');

    if (dataconnectionId) {
      const q = arc.sql.displayedQuery()
        .dataconnection_id(dataconnectionId);

      const nowDate = moment.utc(moment().format('YYYY-MM-DD'));

      const msToDaysFactor = 1000 * 60 * 60 * 24;

      q.fetch((displayedQueries) => {
        let last;
        let processed;
        const today = [];
        const last7 = [];
        const last30 = [];
        const others = [];

        if (!displayedQueries || displayedQueries.length === 0) {
          showEmptyMessage($historyDiv);
        } else {
          displayedQueries.forEach((displayQuery) => {
            const curr = displayQuery.query();
            if (curr === last) {
              // Only show unique queries
              return;
            }

            last = curr;

            const currDate = moment.utc(displayQuery.query_time().local().format('YYYY-MM-DD'));
            const dayDiff = (nowDate - currDate) / msToDaysFactor;

            if (dayDiff === 0) {
              ++processed; /* eslint-disable-line no-plusplus */
              today.push(displayQuery);
              return;
            }

            if (dayDiff <= 7) {
              ++processed; /* eslint-disable-line no-plusplus */
              last7.push(displayQuery);
              return;
            }

            if (dayDiff <= 30) {
              ++processed; /* eslint-disable-line no-plusplus */
              last30.push(displayQuery);
              return;
            }

            if (processed > maxHistoryDisplay) {
              return;
            }

            ++processed; /* eslint-disable-line no-plusplus */
            others.push(displayQuery);
          });

          [[today, 'Today'],
          [last7, 'Last 7 Days'],
          [last30, 'Last 30 Days'],
          [others, 'Older Searches']].forEach((d) => {
            const queryList = d[0];
            const text = d[1];

            if (queryList.length === 0) {
              return;
            }

            const data = [
              `<div class="search-history-section">
              <div class="search-history-header">${text}</div>
            `,
            ];

            queryList.forEach((queryItem) => {
              const t = queryItem.query();
              data.push(arc.misc.safe_substitute(rowTemplate, {
                SEARCH_TEXT: t,
                HTML_SEARCH_DATE: arc.misc.friendlyDate(queryItem.query_time()),
              }));
            });

            data.push('</div>');

            $historyDiv.append(data.join(''));
          });
        }

        activateHistory();
      });
    } else {
      showEmptyMessage($historyDiv);
    }
  }

  let bookmarkQueries;
  function activateBookmarks() {
    $bookmarksHolder.find('a.remove-link')
      .offOn('click.removeBookmark', function () {
        const $this = $(this);
        const $result = $this.closest('.search-result');
        const idx = $result.attr('data-idx');
        const q = bookmarkQueries[idx];
        if (q) {
          q.favorite(false);
          q.store();
        }

        $result.remove();
      });

    $bookmarksHolder.find('.search-text')
      .offOn('click.sql', function () {
        const $this = $(this);
        const idx = $this.closest('.search-result').attr('data-idx');
        const q = bookmarkQueries[idx];
        if (q) {
          activateQuery(q.query(), true);
        }
      });
  }

  function showBookmarks() {
    $bookmarksHolder.html(searchModalResultsTemplate);
    const dataconnectionId = arc.apps_data.active_dataconnection_id();

    const showEmptyMessage = ($container) => {
      $container.html('<div class="placeholder-text">No bookmarks have been created yet.</div>');
    };

    if (dataconnectionId) {
      const q = arc.sql.displayedQuery()
        .dataconnection_id(dataconnectionId)
        .favorite(true);

      $bookmarksHolder.empty();
      $bookmarksHolder.html(searchModalResultsTemplate);

      const $searchResults = $bookmarksHolder.find('.search-results');

      q.fetch((displayedQueries) => {
        bookmarkQueries = displayedQueries;
        if (bookmarkQueries && bookmarkQueries.length > 0) {
          displayedQueries.forEach((displayQuery, i) => {
            $searchResults.append(arc.misc.safe_substitute(sqlTemplate, {
              SEARCH_IDX: i,
              SEARCH_TEXT: displayQuery.query(),
              DATACONNECTION_NAME: displayQuery.dataconnection_name(),
            }));
          });
        } else {
          showEmptyMessage($bookmarksHolder);
        }

        activateBookmarks();
      });
    } else {
      showEmptyMessage($bookmarksHolder);
    }
  }

  const directAccessMenuItems = {
    result: { menuId: 'menu-sqlquery', tableId: 'cmdline', display: null },
    history: { menuId: 'menu-history', tableId: 'sql-history', display: showHistory },
    bookmarks: { menuId: 'menu-bookmarks', tableId: 'sql-bookmarks', display: showBookmarks },
    dataProfiling: { menuId: 'menu-data-profiling', tableId: 'react-root-sql-data-profiling', display: null },
  };

  function showActiveDirectAcessMenu(menuId) {
    const isClouderaStyleEnabled = arc.frontend.featureFlipper.getValueById('enable_cloudera_style');

    menuId = menuId ?? directAccessMenuItems.result.menuId;

    Object.keys(directAccessMenuItems).forEach((key) => {
      const menuItem = directAccessMenuItems[key];

      // When Cloudera style is enabled it is not really a tab but a button
      // And needs different styling
      const $tab = $(`#${menuItem.menuId} ${isClouderaStyleEnabled ? 'a' : ''}`);
      const $tabContent = $(`#${menuItem.tableId}`);

      if (menuItem.menuId !== menuId) {
        $tab.removeClass('active');
        $tabContent.hide();
      } else {
        $tab.addClass('active');
        $tabContent.show();
        menuItem.display?.();
      }
    });
  }

  function activateQuery(query, isBookMarkedQuery) {
    if (query) {
      arc.apps_data.setSqlQuery(query);
      arc.apps_data.runQuery(
        query,
        arc.apps_data.active_dataconnection_id(),
        undefined,
        isBookMarkedQuery
      );
    } else {
      arc.apps_data.setSqlQuery();
    }

    showActiveDirectAcessMenu(directAccessMenuItems.result.menuId);
  }

  const $directAccessErrorExplanation = $('#direct-access-error-explanation');
  const $directSqlAccessTabContent =
    $('#sql-tab #sql-tab-command-line-interface');
  const $dataConnectionSelectorContainer = $('#dataconnection-selector-container');

  function hideElement($element) {
    $element.addClass('arc-invisible');
  }

  function showElement($element) {
    $element.removeClass('arc-invisible');
  }

  function enableDirectAccessComponents(params) {
    const defaultParams = {
      enableDirectSqlAccessSection: true,
      enableConnectionSelector: true,
      explanation: '',
    };
    const { enableDirectSqlAccessSection, enableConnectionSelector, explanation } = {
      ...defaultParams,
      ...params,
    };

    if (enableConnectionSelector) {
      showElement($dataConnectionSelectorContainer);
    } else {
      hideElement($dataConnectionSelectorContainer);
    }

    if (enableDirectSqlAccessSection) {
      showElement($directSqlAccessTabContent);
    } else {
      hideElement($directSqlAccessTabContent);
    }

    if (explanation) {
      showElement($directAccessErrorExplanation);
      $directAccessErrorExplanation.text(explanation);
    } else {
      hideElement($directAccessErrorExplanation);
      $directAccessErrorExplanation.text('');
    }
  }

  function runDirectQuery() {
    hideQueryActions();
    const cmd = getCmdOrError(true);
    if (!cmd) {
      hideQueryActions();
      displayTitle('No command specified');
      $('#cmdline-content #rawcommand-output').empty();
      return;
    }

    arc.apps_data.runQuery(cmd, arc.apps_data.active_dataconnection_id());
  }

function drawCommandLineAccess() {
  const dcType = arc.apps_data.active_dataconnection_type();

  const dcId = arc.apps_data.active_dataconnection_id();
  const selectedConnection  = arcl.dataconnections.find((connection) => connection.id === dcId);

  const notificationMessage = is_connection_direct_accessible(selectedConnection) ? null : `The selected data connection cannot be queried, please use the Connection Explorer Tab instead!`;
  const $tabLabel = $('#data-tabs .cmdline');
  const $tabContent = $('#cmdline-content');
  let $rawCommandHolder = $tabContent.find('.rawcommand-holder');

  const userHasPermissionToExplore =
    arc.misc.hasPerm('dc_explore', arc.apps_data.get_active_dataconnection()?.perms);
  const $accelBtn = $('#rawcommand-accel');
  const $newDashboardBtn = $('.new-dashboard-from-query');
  const $saveAsDatasetBtn = $('#save-as-dataset');

  if (
    !is_direct_accessible(dcType) ||
    !arc.globals.enable_direct_access ||
    !userHasPermissionToExplore ||
    notificationMessage
  ) {
    $rawAccessMenu.hide();
    $tabLabel.addClass('arc-invisible');
    $tabContent.addClass('arc-invisible');
    $runSqlButton.off('click.runSql');
    if (window.location.hash === '#cmdline') {
      window.location.hash = '#datasets';
      $('#tab-dataset').tab('show');
    }

    enableDirectAccessComponents({
      enableDirectSqlAccessSection: false,
      explanation: notificationMessage,
      enableConnectionSelector: true,
    });
    return;
  }

  enableDirectAccessComponents();
  $rawAccessMenu.show();
  hideQueryActions();
  $tabLabel.removeClass('arc-invisible');
  $tabContent.removeClass('arc-invisible');

  if ($rawCommandHolder.length === 0) {
    $rawCommandHolder = $('.rawcommand-holder');

    if ($rawCommandHolder.length === 0) {
      return;
    }
  }

  if ($rawCommandHolder.length === 0) {
    return;
  }

  if (expressionEditorOnDirectAccessPage) {
    expressionEditorOnDirectAccessPage.destroy();
  }

  expressionEditorOnDirectAccessPage = addExpressionEditor($rawCommandHolder, dcType);

  arc.misc.setCommandEnterShortcutOnExpressionEditor(
    expressionEditorOnDirectAccessPage,
    runDirectQuery,
  );

  if (initialQueries) {
    expressionEditorOnDirectAccessPage.getSession()
                     .setValue(JSON.parse(initialQueries).map((val) => val.Query).join(';'));
  }

  const $tableContainer = $('#rawcommand-output');
  $title.empty();
  $tableContainer.empty();

  setActiveQuery(undefined);
  $tableContainer.empty();

  $newDashboardBtn.offOn('click.createDashboard', () => {
    arc.misc.createNewDashboardWithSql(expressionEditorOnDirectAccessPage, dcId);
  });

  $saveAsDatasetBtn.offOn('click.saveAsDataSet', () => {
    arc.frontend.legacyBridge.events.next({
      type: 'open-create-dataset-modal',
      payload: {
        connection: { id: dcId, type: dcType },
        initialData: { query: expressionEditorOnDirectAccessPage.value(), sourceType: 'sql' }
      }
    });
  });

  $accelBtn.offOn('click.accelerate', function handleAcceleration() {
    const cmd = getCmdOrError();
    if (!cmd) {
      arc.report.alertMessageSmall('Must enter SQL to get recommendations', 'alert-danger');
      return;
    }

    const queries = JSON.stringify(
      cmd.split(';').map((val, i) => ({ Query: val, 'Query Details': i + 1 }))
    );

    const data = {
      queries,
      dcId,
      fromDirect: true,
    };

    arc.admin.QueryTableManager.submitQueriesForAcceleration(data);
  });

  $runSqlButton.offOn('click.runSql', runDirectQuery);
}

function drawViews() {
  if (sqlOnly) {
    return;
  }

  $('.aviews-tables-container').empty();

  // Disable any active aview management
  if (arc.apps_data.aviewManager) {
    arc.apps_data.aviewManager.disable();
    arc.apps_data.aviewManager = undefined;
  }

  const dcType = arc.apps_data.active_dataconnection_type();
  const dcId = arc.apps_data.active_dataconnection_id();
  const isArcengine = arc.dataconnection.isArcEngine(dcType);

  const $queryPicker = $('#query-picker');

  if ((dcId === 0) || (arc.dataconnection.supportsDataExtractsSource(dcType))) {
    $('.extracts-related').removeClass('arc-invisible');
  } else {
    $('.extracts-related').addClass('arc-invisible');

    // If we are switching from a connection that supports extracts, and the extracts
    // tab was the active tab, then make the first tab (Datasets) active instead
    if ($('#extracts').hasClass('active')) {
      $('#data-tabs a:first').trigger('click');
    }
  }

  if (isArcengine &&
      arc.misc.hasPerm('dc_aviews', arc.apps_data.get_active_dataconnection().perms)) {
    // This is an arcengine connection.
    // Show the analytical views tab. Show create analytical views button.
    // Setup an aview manager object to load views

    $('.av-related').show();  // don't use arc-invisible as it will get undone by updateDivForPerms
    if (arc.globals.site_enable_app_acceleration) {
      $queryPicker.attr('href', `${arc.frontend.globals.urls.queryAccelUrl}/${dcId}`)
                  .attr('style', 'display:inline-block'); // ensure it doesn't show small inline
    } else {
      $queryPicker.hide();
    }

    $('.views-loading').removeClass('arc-invisible');
    $('.views-none').addClass('arc-invisible');
    $('#aview-tree-table').addClass('arc-invisible');
    $('#aview-summary').addClass('arc-invisible');

    arc.apps_data.aviewManager =
        arc.admin.aviewManager()
           .dataconnection_id(dcId)
           .allDatasets(allDatasets)
           .loadViews();
  } else {
    // This is not an arcengine connection.
    // Hide the analytical views tab, hide create analytical views button
    $('.av-related').hide();

    // If we are switching from an arcengine connection, and the analytical
    // views tab was the active tab, then make the first tab (Datasets) active
    // instead
    if ($('#tab-views').hasClass('active')) {
      $('#data-tabs a:first').trigger('click');
    }
  }
}

function getActiveDatasets(fulltablename, dc_id) {
  if (!dc_id) {
    const dc = arc.apps_data.get_active_dataconnection();
    if (!dc) {
      return allDatasets;
    }

    dc_id = dc.id;
  }

  return allDatasets.filter((d) => {
    if (d.dc_id !== dc_id) {
      return false;
    }

    if (fulltablename === undefined) {
      return true;
    }

    const tableNames = arc.datasets.datasetObjectToTableNames(d);
    return tableNames.find((d) => d === fulltablename) !== undefined;
  });
}

function drawDatasetsByConnectionOnDatasetsTab() {
  // draw Dataset Table on the DATA page's Dataset TAB,
  // where we display all Datasets related to a dataconnection
  drawDatasets({
    datasets_divid: '#datasets-table',
    datasets_count_divid: '#datasets-count',
    active_datasets: getActiveDatasets(),
  });
}

let DELETE_TEMPLATE;
let NEWAPP_TEMPLATE;
let SEARCH_TEMPLATE;

function initTemplates() {
  if (DELETE_TEMPLATE) {
    return;
  }

  DELETE_TEMPLATE = '<a class="link">' +
    '<span class="fa fa-trash-o" data-toggle="tooltip"' +
    'title="Delete Dataset">' +
    '</span></a>';

  let dashboard_template_icon = '';
  if (dashboard_template_enabled && arc.globals.enable_explore_interfaces) {
    dashboard_template_icon =
    '<a href="#showme-template-modal" data-toggle="modal" ' +
    'data-title="Explore Templates" data-value="%DATASET_ID%"' +
    ' data-backdrop="true"><span class="explore-icon ' +
    'data-viz-icon" data-toggle="tooltip" title="Explore Dashboard Templates"></span></a>';
  }

  const builderUrl = `${arc.frontend.globals.urls.reportBuildUrl}/0?` +
      'report_type=dashboard&dataset_id=';
  const newAppUrl = `
    <a href="${builderUrl}%DATASET_ID%" data-toggle="tooltip" data-title="New Dashboard">
      <span class="fa fa-plus add-icon"></span>
      <i class="fa sym-dashboard data-viz-icon"></i>
    </a>
  `;

  NEWAPP_TEMPLATE = `${dashboard_template_icon}${newAppUrl}`;
  SEARCH_TEMPLATE =
      '<a class="search-link" data-id="%DATASET_ID%" ' +
      'data-toggle="tooltip" data-title="Search">' +
      '<i class="fa fa-search"></i></a>';
}

function getDatasetPropertyMap(d) {
  initTemplates();

  const num_commas = d3.format(',');
  let tableDisplay = '';

  const tableNames = arc.datasets.datasetObjectToTableNames(d);
  if (tableNames.length === 1) {
    tableDisplay = tableNames[0];
  } else {
    tableDisplay = arc.misc.displayTableNames(tableNames);
  }

  d.display_dsname = `<span class='dataset-title' data-test-id='dataset-title'>${arc.misc.escapeHTML(d.name)}</span>`;
  d.display_title = `
    ${d.display_dsname}
    <span class='ds-tablename-wrapper' title='${arc.misc.escapeHTML(d.detail_display)}'>
      ${arc.misc.escapeHTML(tableDisplay)}
    </span>`;

  const dc_id = arc.apps_data.active_dataconnection_id();
  if (dc_id === 0) {
    d.display_title += `<div class='ds-connection-wrapper'>` +
        `<span class="header-label">Data Connection: </span>${d.dc_name}</div>`;
  }

  d.display_title += `<div class='ds-desc-wrapper'>` +
      `${arc.misc.escapeHTML(d.description)}</div>`;

  if ('rows' in d) {
    d.display_rowcount = num_commas(d.rows);
  } else {
    d.display_rowcount = '--';
  }

  let settings;
  try {
    settings = JSON.parse(d.settings);
  } catch (err) {
    // Ignore errors
  }

  const html_tags = [];
  if (extractsEnabled()) {
    if (d.extractSource) {
      html_tags.push(arc.extracts.extractSourceBadge);
    }

    if (d.extractTarget) {
      html_tags.push(arc.extracts.extractTargetBadge);
    }
  }

  const enable_dashboard_count = arc.frontend.featureFlipper.getValueById('enable_counting_dashboards_for_datasets');

  const templateObj = {
    DATA_HREF: `${arc.frontend.globals.urls.datasetEditUrl}/${d.id}`,
    DATASET_NAME: d.name,
    HTML_DATASET_NAME: d.display_title,
    DATASET_ID: d.id,
    NEW_SEARCH_URL: `${arc.frontend.globals.urls.staticUrl}/../apps/search?dataset_id=${d.id}`,
    NEW_DASHBOARD_URL: `${arc.frontend.globals.urls.staticUrl}/../apps/builder/0?report_type=dashboard&dataset_id=${d.id}`,
    HTML_CREATE_DATE: arc.misc.friendlyDate(d.created),
    HTML_UPDATE_DATE: arc.misc.friendlyDateSince(d.updated),
    MODIFIED_BY: d.updated_by,
    HTML_SEARCH: '',
    HTML_NEW_APP: '',
    HTML_DELETE_DATASET: '',
    HTML_APP_COUNT: enable_dashboard_count ?
    `<a href="${arc.frontend.globals.urls.staticUrl}/../apps/dataset/${d.id}#tab_apps">${d.report_count}</a>` :
    '',
    HTML_TAGS: html_tags.join(''),
  };

  if (arc.misc.hasPerm('ds_appview', d.perms)) {
    try {
      if (settings) {
        if (arc.frontend.featureFlipper.getValueById('site_enable_search') && settings.SEARCH &&
            settings.SEARCH.enabled) {
          templateObj.HTML_SEARCH = arc.misc.safe_substitute(SEARCH_TEMPLATE, templateObj);
        }
      }
    } catch (err) {
      // Ignore errors
    }
  }

  if (arc.misc.hasPerm('ds_appedit', d.perms)) {
    templateObj.HTML_NEW_APP = arc.misc.safe_substitute(NEWAPP_TEMPLATE, templateObj);
  }

  if (arc.misc.hasPerm('ds_manage', d.perms)) {
    templateObj.HTML_DELETE_DATASET = DELETE_TEMPLATE;
  }

  return templateObj;
}

const redrawEveryDatasetTables = () => {
  // There are 3 dataset table:
  // - in DATA page on the Dataset TAB (all related to a connection)
  // - in DATA page on the Connection Explorer TAB, when a user selects a datatable,
  // sample data and datasets tabs are displayed
  // - in SQL page/Direct Access menu point also in the ConnectionExplorer TAB
  const activeDbName = $('.database-name.active:visible').attr('data-databasename');
  const activeTableName = $('.table-row-active:visible').parent().attr('data-tablename');
  if (activeDbName && activeTableName) {
    // Redraws the dataset table without reloading all the table and dataset informations
    setupAndDrawConnectionExplorerCurrentDatasetTable(activeDbName, activeTableName);
  }

  drawDatasetsByConnectionOnDatasetsTab();
};

const handleDatasetCountChange = (dataset, increase) => {
  const databaseAndTableInfoOfDataset = dataset.buildInfoObj?.()?.[0]?.tablename ?? dataset.detail;
  const [database, dataTable] = databaseAndTableInfoOfDataset.split('.');
  const $databaseDatasetCountBadge = $(`.database-name[data-databasename="${database}"]`).find('.badge-pill');
  const databaseDatasetCount = $databaseDatasetCountBadge.text();
  $databaseDatasetCountBadge.text(parseInt(databaseDatasetCount, 10) + (increase ? 1 : (-1)));
  const $dataTableDatasetCountBadge = $(`.table-name[data-tablename="${dataTable}"]`).find('.table-datasets a.link');
  const datasetCount = $dataTableDatasetCountBadge.text();
  $dataTableDatasetCountBadge.text(parseInt(datasetCount, 10) + (increase ? 1 : (-1)));
  // Add to AllDatasetCount
};

function drawDatasets(config) {
  if (sqlOnly) {
    return;
  }

  if (config === undefined) {
    config = {};
  }

  const { datasets_divid, datasets_count_divid } = config;
  const $datasets_sel = $(datasets_divid);
  const $datasets_count_sel = $(datasets_count_divid);
  const active_datasets = config.active_datasets || [];
  let $datasetsDataTable = createdDataTables[datasets_divid];
  const isExtractsEnabled = arc.frontend.featureFlipper.getValueById('site_enable_extracts');
  const trashColumnIdx = isExtractsEnabled ? 8 : 7;
  const lastUpdatedIdx = isExtractsEnabled ? 5 : 4;
  if ($datasetsDataTable === undefined) {
    $datasetsDataTable =
      arc.misc.DataTable($datasets_sel, {
        paging: false,
        ordering: true,
        info: false,

        // disable sort on New Visual, Delete columns
        columnDefs: [{ targets: [1, trashColumnIdx], orderable: false }],
        order: [[lastUpdatedIdx, 'desc']],
      });
      $datasetsDataTable.columns.adjust();

    createdDataTables[datasets_divid] = $datasetsDataTable;
  }

  const $tableBody = $($datasetsDataTable.table().body());
  $($datasetsDataTable.table().container())
    .addClass('d-flex flex-column justify-content-start align-items-stretch');
  $datasetsDataTable.clear();
  $tableBody.hide();

  if ($datasets_count_sel.length) {
    $datasets_count_sel.text(active_datasets.length);
  }

  const templateIn = $('.dataset-row-template tbody').html();

  active_datasets.forEach((d) => {
    const templateObj = getDatasetPropertyMap(d);
    const $row = $(arc.misc.safe_substitute(templateIn, templateObj));

    // should be a better way to associating data
    d3.select($row.find('.fa-trash-o')[0]).datum(d);
    d3.select($row[0]).datum(d);
    $datasetsDataTable.rows.add($row);
  });

  $datasetsDataTable.draw();

  $datasets_sel.find('.search-link')
    .offOn('click.search', function openSearchModal() {
      const $this = $(this);
      const dataset_id = Number($this.attr('data-id'));

      arc.topnav.runModal([dataset_id]);
    });

  const rows = $datasets_sel.find('tbody tr .fa-trash-o');

  const { events, ofType } = arc.frontend.legacyBridge;

  let subscription;

  rows.on('click.datadelete', (evt) => {
    evt.preventDefault();
    evt.stopPropagation();
    const dsObject = evt.target.__data__;
    const { id, name } = dsObject;

    arc.frontend.legacyBridge.events.next({
      type: 'dataset-delete-modal',
      payload: {
        id,
        name
      }
    });

    if (subscription) {
      subscription.unsubscribe();
    }

    subscription = events
      .pipe(ofType('successfully-deleted-dataset'))
      .subscribe(
        () => {
          const remainingDatasets =
            allDatasets.filter((d) => parseInt(d.id, 10) !== parseInt(dsObject.id, 10));
          updateDatasetRelatedGlobalInformations(remainingDatasets, true);
          handleDatasetCountChange(dsObject /* Backend */, false /* decrease */);
          redrawEveryDatasetTables();
        }
      );
  });

   // Enable tooltips
  $("[data-toggle='tooltip']").tooltip({
    container: 'body',
    placement: 'top',
    delay: {
      show: 500,
      hide: 100
    }
  });

  if ($('#searchbox').val()) {
    datasetSearchFn();
  }

  $tableBody.show();
  $datasets_sel.trigger('update');
  arc.report.loadTemplate();
  arc.report.loadTemplate.setup();
  $datasetsDataTable.columns.adjust();
}

arc.apps_data.fetch_and_draw = function(callback) {
  fetchData(() => {
    /* Improvement opportunity: rethink draw logic.
     - Move 'dc' check within arc.apps_data.draw?
     - Implement logic/workflow etc to call draw after we have all the data for sure
     (for example: dataconnection initalization already finished)
     */
    arc.apps_data.draw_dataconnections();
    drawDataTables();
    drawDatasetsByConnectionOnDatasetsTab();
    drawViews();
    const dc = arc.apps_data.get_active_dataconnection();
    if (dc) {
      arc.misc.updateDivForPerms(dc.perms, 'dataconn');
      setupDataImportGet();
      setupCacheReset();
      $('#dataset-buttons .more-btn').show();
      rearrangeButtons();
      drawCommandLineAccess();
    }

    if (callback) {
      callback();
    }
  });
};

arc.apps_data.setup_refresh_metadata_button = function() {
  $('#data-tabs li').on('shown.bs.tab', () => {
    arc.apps_data.update_refresh_button();
  });

  $('#refresh-metadata').off().on('click', () => {
    // eslint-disable-next-line no-unused-vars
    function fetchCb(errorMsg, response) {
      $('#refresh-metadata-spinner').hide();
      if (errorMsg) {
        arc.report.alertMessageAllHide();
        arc.report.alertMessage('Error on refresh operation', errorMsg);
      }

      drawDataTables();
    }

    $('#databases-list li').remove();
    $('#table-table tbody tr').remove();
    $('.sample-table-wrapper').hide();
    $('#refresh-metadata').hide();
    $('#refresh-metadata-spinner').show();

    // Also reset connection cache when doing invalidate metadata
    arc.dataconnection.reset_cache(arc.apps_data.active_dataconnection_id());

    const rb = arc.requestBuilder();
    rb.requestType('.invalidate_metadata');
    arc.report.sql_runner.run(arc.apps_data.active_dataconnection_id(),
        '',
        fetchCb,
        false,
        rb.dsreq());
  });
};

function addExpressionEditor($sel, dataconnection_type,
    changeCb) {
  $sel.find('.rawsql-expr-editor-container')
      .remove();

  $sel.append($('#rawsql-textbox-template').html());
  const exprId = `expr-editor-${arc.misc.uniqueId()}`;

  $sel.find('.expression-field')
      .attr('id', exprId);

  const exprEditor = arc.misc.expressionEditor(
    undefined,
    undefined,
    exprId,
    $sel.find('.rawsql-expr-editor-container .edit-expression-textbox'),
    { dataconnection_type },
    true
  );

  if (changeCb) {
    exprEditor.getSession().on('change', () => {
      const t = exprEditor.getSession().getValue();
      changeCb(t);
    });
  }

  return exprEditor;
}

 async function handleNewlyCreatedDataset(dataset /* arc_datasets.js */) {
  const $datasetsTables = $('#tabledatasets-table, #datasets-table');
  // Currently spinner is displayed under the tables. (above would look ugly)
  // To make it clear we are refreshing I hide the table to display the spinner in a better position
  // until the table is refreshed.
  $datasetsTables.hide();
  datasetsLoadingSpinner.show();
  // The result dataset is an arc_datasets object (and not perfectly filled with data),
  // and BackendDataset type that we require
  // on the centralised dataset information, so we have to retrieve it.
  try {
    const datasetInfo = await getDataset(dataset.dataset_id());
    updateDatasetRelatedGlobalInformations(datasetInfo);
    redrawEveryDatasetTables();
    handleDatasetCountChange(dataset, true /* increase  */);
  } catch (error) {
    arc.report.alertMessage('Could not refresh Dataset table', error);
  } finally {
    datasetsLoadingSpinner.hide();
    $datasetsTables.show();
  }
}

  // On Data page load, we fetch all connection's dataset,
  // schedule groups etc all in fetch_and_draw.
  // Instead retreiving every data and redraw again, this updates the necessary lists only.

  async function getDataset(datasetId /* arc_datasets.js */) {
    const associated = arc.frontend.featureFlipper.getValueById('enable_counting_dashboards_for_datasets');
    const { data: datasets } = await arc.frontend.vizHttp.get({
      url: `${arc.frontend.globals.urls.datasetUrl}/${datasetId}?get_associate_dashboard=${associated}`,
      headers: {
        'X-CSRFTOKEN': arc.globals.csrf_token,
      },
      isOldEndpoint: true,
    });

    return datasets;
    // updateDatasetRelatedGlobalInformations(datasets);
}

function popupCreateDatasetModal(config) {
  arc.frontend.legacyBridge.events.next({
    type: 'open-create-dataset-modal',
    payload: {
      ...config,
      connection: {
        id: arc.apps_data.get_active_dataconnection().id,
        type: arc.apps_data.active_dataconnection_type()
      },
      onSuccess: (dataset /* arc_datasets.js */) => {
        handleNewlyCreatedDataset(dataset);
      }
    }
  });
}

arc.apps_data.setup_dataset_add_button = function() {
  // Popup the modal on a 'New Dataset' button click
  d3.select('#dataset-add')
    .on('click.dataset', () => {
      popupCreateDatasetModal();
    });
};

arc.apps_data.load_databases_cb = function(dc_id, cb) {
  const rb = arc.requestBuilder();
  rb.requestType('.databases');

  return arc.report.sql_runner.runParams({
    dataconnection_id: dc_id,
    query: '',
    cb,
    cache: false,
    dsreq: rb.dsreq(),
    displayErrorMessage: false,
    hideAlertsOnSuccess: true,
  });
};

arc.apps_data.load_tables_cb = function(dc_id, dbname, cb) {
  const rb = arc.requestBuilder();
  rb.requestType('.tables');
  rb.dbname(dbname);

  return arc.report.sql_runner.runParams({
    dataconnection_id: dc_id,
    query: '',
    cb,
    cache: false,
    dsreq: rb.dsreq(),
    displayErrorMessage: false,
    hideAlertsOnSuccess: true,
  });
};

arc.apps_data.directAccess = function() {
  const directAccess = function() { return 'arc.apps_data.directAccess'; };

  let dcName;

  directAccess.initialDataconnectionName = function(_) {
    dcName = _;
    $('#cmdline .dcname').text(dcName);
  };

  function handleSQLQuery() {
    $('#menu-sqlquery').offOn('click.query', () => {
      showActiveDirectAcessMenu(directAccessMenuItems.result.menuId);
    });
  }

  function handleBookmarks() {
    $('#menu-bookmarks').offOn('click.bookmarks', () => {
      showActiveDirectAcessMenu(directAccessMenuItems.bookmarks.menuId);
    });
  }

  function handleHistory() {
    $('#menu-history').offOn('click.history', () => {
      showActiveDirectAcessMenu(directAccessMenuItems.history.menuId);
    });
  }

  function handleDataProfiling() {
    $('#menu-data-profiling').offOn('click.profiling', () => {
      showActiveDirectAcessMenu(directAccessMenuItems.dataProfiling.menuId);
    });
  }

  const useConnectionExplorerTabWhenDirectAccessNotAnOption = (selectedConnection) => {
    if (!is_connection_direct_accessible(selectedConnection)) {
      const tabLink = '#explorer-tab';
      $(`#datatable-selector-and-sql-editor-container .nav-tabs a[href=${tabLink}]`).tab('show');
      arc.apps_data.setSqlOnly(tabLink !== '#explorer-tab');
    }
  };

  directAccess.initialize = async function() {
  if (arc.frontend.featureFlipper.getValueById('enable_direct_access')) {
    if (arc.frontend.featureFlipper.getValueById('enable_cloudera_style')) {
      const queryparams = arc.misc.getQueryParameters();
      if (queryparams.query_id !== undefined) {
        const query = await arc.sql.getQueryById(queryparams.query_id, queryparams.favorite);
        arc.apps_data.initialQueries(JSON.stringify([query]));
      }
    }

    const { selectedConnection } = setupDataConnectionSelectionList();
    const tabLink = is_connection_direct_accessible(selectedConnection) ? window.location.hash : '#explorer-tab';
    document.location.hash = tabLink;
    $(`#datatable-selector-and-sql-editor-container .nav-tabs a[href="${tabLink}"]`).tab('show');
    arc.apps_data.setSqlOnly(tabLink !== '#explorer-tab');

    $('#limit-100').prop('checked', true);

    setupMoreOptionButton();
    activateRunEmail();
    setupDownloadCsvInputCheck();
    setupEmailInputCheck();
    activateBookmark();

    handleSQLQuery();
    handleBookmarks();
    handleHistory();
    handleDataProfiling();

    useConnectionExplorerTabWhenDirectAccessNotAnOption(selectedConnection);
    showActiveDirectAcessMenu(arc.misc.getQueryParameters().active_data_menu);
    try {
      const datasets = await loadDatasetsOfConnection(arc.apps_data.active_dataconnection_id());
      updateDatasetRelatedGlobalInformations(datasets, false);
    } catch (err) {
      arc.report.alertMessage('Could not fetch dataset related to the selected connection', err);
    }

    drawDataTables();
    drawCommandLineAccess();
    }
  };

  function setupDownloadCsvInputCheck() {
    $numberRowsInput.offOn('input.downloadCsv', () => {
      const containsOnlyNumbers = /^\d+$/.test($numberRowsInput.val());
      const { valid } = $numberRowsInput[0].validity;
      const isEmpty = valid && !containsOnlyNumbers;
      const enableDownload = valid && containsOnlyNumbers;

      if (isEmpty) {
        disableWarning($numberRowsInput, $numberRowsInputWarning);
        disableElement($downloadCSV);
      } else if (enableDownload) {
        disableWarning($numberRowsInput, $numberRowsInputWarning);
        enableElement($downloadCSV);
      } else {
        // disableDownloadWitWarning
        enableWarning($numberRowsInput, $numberRowsInputWarning);
        disableElement($downloadCSV);
      }
    });
  }

  const $emailAddressInput = $('#run-email-address');
  const $sendEmailButton = $('#rawcommand-email-result');

  function setupEmailInputCheck() {
    $emailAddressInput.offOn('keyup.sendEmail', () => {
      if (isEmailValid()) {
        disableWarning($emailAddressInput, $emailWarning);
        enableElement($sendEmailButton);
      } else {
        enableWarning($emailAddressInput, $emailWarning);
        disableElement($sendEmailButton);
      }
    });
  }

  function isEmailValid() {
    return $emailAddressInput[0].validity.valid;
  }

  function activateRunEmail() {
    const cookieName = 'sql-email-address';
    disableWarning($emailAddressInput, $emailWarning);

    const cookieStoredEmailAddress = arc.frontend.vizCookie.get(cookieName);
    $emailAddressInput.val(cookieStoredEmailAddress);

    $sendEmailButton.offOn('click.email', () => {
      if (!isEmailValid()) {
        enableWarning($emailAddressInput, $emailWarning);
        disableElement($sendEmailButton);
        return;
      }

      const emailAddress = $('#run-email-address').val();
      arc.frontend.vizCookie.set(cookieName, emailAddress, { expires: 365 });

      let sql = getCmdOrError(false);
      if (!sql) {
        // eslint-disable-next-line no-console
        console.log('no sql');
        return;
      }

      sql = addLimitToSQL(sql);
      const dc = arc.apps_data.get_active_dataconnection();
      const params = {
        dataconnection_id: dc.id,
        sql,
        email_address: emailAddress,
      };

      $.ajax({
        url: arc.frontend.globals.urls.scriptScheduleUrl,
        data: params,
        type: 'POST',
        beforeSend(xhr) {
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      });

      arc.report.alertMessageSmallBase({
        type: 'alert-success',
        alertMessage: 'SQL command running. Results will be emailed out.',
        fadeOut: true,
      });
    });
  }

  function activateBookmark() {
    const $bookMarkLinks = $('.bookmark-query');
    $bookMarkLinks.offOn('click.bookmark', () => {
      const dc = arc.apps_data.get_active_dataconnection();
      const cmd = $cmdTitleHolder.attr('data-cmd');

      if ($bookmarkIcon.hasClass('fa fa-bookmark-o')) {
        // Create bookmark
        $bookmarkIcon.removeClass('fa fa-bookmark-o');
        $bookmarkIcon.addClass('fa fa-bookmark');
        arc.sql.displayedQuery()
               .dataconnection_id(dc.id)
               .query(cmd)
               .favorite(true)
               .store();
      } else {
        // Delete bookmark
        $bookmarkIcon.removeClass('fa fa-bookmark');
        $bookmarkIcon.addClass('fa fa-bookmark-o');
        arc.sql.displayedQuery()
               .dataconnection_id(dc.id)
               .query(cmd)
               .delete();
      }

      showBookmarks();
    });
  }

  return directAccess;
};
}());
