window.arc = window.arc || {};

(function() {
'use strict';

arc.data = arc.data || {};

arc.data.batch = function() {
  var batch = function() {
    return 'arc.data.batch';
  };

  let calls = [],
      type,
      url,
      asyncCaller,
      timeoutHandler;

  asyncCaller = function(paramsToSend, cb) {
    function doneCb(response) {
      cb(undefined, response);
    }

    function failCb(errorMessage) {
      cb(errorMessage);
    }

    arc.sqlrun.httpAsyncCall({
      url,
      paramsToSend,
      doneCb,
      failCb,
    });
  };

  batch.asyncCaller = function(_) {
    if (!arguments.length) return asyncCaller;
    asyncCaller = _;
    return batch;
  };

  batch.type = function(_) {
    if (!arguments.length) return type;
    type = _;
    return batch;
  };

  batch.url = function(_) {
    if (!arguments.length) return url;
    url = _;
    return batch;
  };

  function makeCall() {
    var currCalls = calls;
    calls = [];

    if (currCalls.length === 0) {
      timeoutHandler = undefined;
      return;
    }

    var valueGetter = function(call) {
      return call.value;
    };

    const paramsToSend = {
      qhandles: currCalls.map(valueGetter).join(','),
    };

    var t1 = new Date();
    arc.counters.increment('asynccaller_' + type);
    asyncCaller(paramsToSend, function(error, data) {
      var t2 = new Date();
      try {
        error = !error && !Array.isArray(data) ?
          `The result is not in the expected (array) format: ${typeof data === 'object' ? JSON.stringify(data) : data}` :
          error;
        if (error) {
          arc.perf.PAGEPERF.addProfile({
            type: type,
            time: t2 - t1,
            error: error,
          });

          currCalls.forEach(function(call) {
            call.cb(error, undefined);
            return;
          });

          return;
        }

        if (!arc.globals.arcviz_settings.ctds) {
          data = arc.misc.unobfuscate(data);
        }

        if (data) {
          data.forEach(function(d, i) {
            currCalls[i].cb(d.error, d);

            arc.perf.PAGEPERF.addProfile({
              id: currCalls[i].profileId,
              type: type,
              time: t2 - t1,
              error: d.error,
              profile: d.profile,
            });
          });
        }
      } finally {
        if (calls.length > 0) {
          makeCall();
        } else {
          timeoutHandler = undefined;
        }
      }
    });
  }

  function scheduleCall() {
    if (timeoutHandler) {
      clearTimeout(timeoutHandler);
    }

    timeoutHandler = setTimeout(makeCall, 0);
  }

  batch.cancel = function(callId) {
    calls = calls.filter(function(call) {
      if (call.callId === callId) {
        return false;
      }

      return true;
    });

    return batch;
  };

  batch.add = function(callId, value, cb, profileId) {
    calls.push({
      callId: callId,
      value: value,
      cb: cb,
      profileId: profileId,
    });

    if (!timeoutHandler) {
      scheduleCall();
    }
  };

  return batch;
};

}());

