window.arc = window.arc || {};

(function() {
'use strict';

arc.data = arc.data || {};

arc.data.column_meta = function() {
  let columns = [];
  let dim_start_idx;
  let dim_end_idx;
  let agg_start_idx;
  let agg_end_idx;

  const meta = function() { return 'arc.data.column_meta'; };

  meta.columns = function(_) {
    if (!arguments.length) return columns;
    columns = _;
    return meta;
  };

  meta.dim_cnt = function() {
    if (dim_start_idx < 0) {
      return 0;
    }

    return 1 + dim_end_idx - dim_start_idx;
  };

  meta.dim_start_idx = function() {
    return dim_start_idx;
  };

  meta.dim_end_idx = function() {
    return dim_end_idx;
  };

  meta.agg_cnt = function() {
    if (agg_start_idx < 0) {
      return 0;
    }

    return 1 + agg_end_idx - agg_start_idx;
  };

  meta.agg_start_idx = function() {
    return agg_start_idx;
  };

  meta.agg_end_idx = function() {
    return agg_end_idx;
  };

  // Compute dim/agg start/end indices based on the data in 'columns'
  meta.finalize = function() {
    dim_start_idx = dim_end_idx = agg_start_idx = agg_end_idx = -1;
    columns.forEach((column, colindex) => {
      if (column.colproperty() === arc.report.columnTypeEnum.DIMENSION) {
        if (dim_start_idx === -1) {
          dim_start_idx = dim_end_idx = colindex;
          return;
        }

        dim_end_idx = colindex;
        return;
      }

      if (column.colproperty() === arc.report.columnTypeEnum.AGGREGATE) {
        if (agg_start_idx === -1) {
          agg_start_idx = agg_end_idx = colindex;
          return;
        }

        agg_end_idx = colindex;
      }
    });

    return meta;
  };

  return meta;
};

arc.data.request = function() {
  let rb;
  let dsreq;
  let sql;
  let request_type = 'sql';
  let columns = [];
  const properties = {};
  let column_meta;

  const data_request = function() { return 'arc.data.request'; };

  data_request.clone = function() {
    const d = arc.data.request();
    d.properties($.extend({}, properties));
    d.sql(sql);
    d.rb(data_request.rb().cloneRequestBuilder());
    d.dsreq(d.rb().dsreq());
    d.request_type(request_type);
    d.columns = columns.slice(0);
    d.finalize();

    return d;
  };

  function addProperty(k, v) {
    properties[k] = v;
    return data_request;
  }

  function getProperty(k) {
    return properties[k];
  }

  data_request.sql = function(_) {
    if (!arguments.length) return sql;
    sql = _;
    return data_request;
  };

  data_request.isPrereqDataRequest = function() {
    const prop = 'isPrereqDataRequest';
    if (!arguments.length) return getProperty(prop);
    addProperty(prop, true);
    return data_request;
  };

  // isDefault means that it is default request of the entire table
  data_request.isDefault = function() {
    const prop = 'isDefaultDataRequest';
    if (!arguments.length) return getProperty(prop);
    addProperty(prop, true);
    return data_request;
  };

  data_request.dsreq = function(_) {
    if (!arguments.length) return dsreq;
    dsreq = JSON.stringify(_);
    return data_request;
  };

  data_request.dsreqNoDerivedData = function(nolimit) {
    const tmp = JSON.parse(dsreq);
    tmp.derived_data = [];
    let useLimit;
    if (nolimit) {
      tmp.aggregates.forEach((agg) => {
        if (agg.order) {
          useLimit = tmp.limit;
        }
      });

      tmp.dimensions.forEach((dim) => {
        if (dim.order) {
          useLimit = tmp.limit;
        }
      });

      tmp.limit = useLimit;
    }

    return tmp;
  };

  data_request.rb = function(_) {
    if (!arguments.length) return rb;
    rb = _;
    return data_request;
  };

  data_request.request_type = function(_) {
    if (!arguments.length) return request_type;
    request_type = _;
    return data_request;
  };

  data_request.columns = function(_) {
    if (!arguments.length) return columns;
    columns = _;
    return data_request;
  };

  data_request.add_column = function(_) {
    columns.push(_);
    _.colindex(columns.length - 1);
    return data_request;
  };

  // Compute dim/agg start/end indices based on the data in 'columns'
  data_request.finalize = function() {
    column_meta = arc.data.column_meta()
                     .columns(columns)
                     .finalize();
    return data_request;
  };

  data_request.dim_cnt = function() {
    return column_meta.dim_cnt();
  };

  data_request.dim_start_idx = function() {
    return column_meta.dim_start_idx();
  };

  data_request.dim_end_idx = function() {
    return column_meta.dim_end_idx();
  };

  data_request.agg_cnt = function() {
    return column_meta.agg_cnt();
  };

  data_request.agg_start_idx = function() {
    return column_meta.agg_start_idx();
  };

  data_request.agg_end_idx = function() {
    return column_meta.agg_end_idx();
  };

  return data_request;
};

// Represent the ordering parameters associated with a arc.data.column
// instance. Stores priority, ascending/descending order, and limit.
arc.data.order = function() {
  let priority;
  let ascending;
  let limit;
  let granular;

  const order = function() { return 'arc.data.order'; };

  order.cloneOrder = function() {
    const o = arc.data.order()
               .priority(priority)
               .ascending(ascending)
               .limit(limit)
               .granular(granular);

    return o;
  };

  order.granular = function(_) {
    if (!arguments.length) return granular;
    granular = _;
    return order;
  };

  order.limit = function(_) {
    if (!arguments.length) return limit;
    limit = _;
    return order;
  };

  order.ascending = function(_) {
    if (!arguments.length) return ascending;
    ascending = _;
    return order;
  };

  order.priority = function(_) {
    if (!arguments.length) return priority;
    priority = _;
    return order;
  };

  return order;
};

arc.data.sortColumnsByPriority = function(c1, c2) {
  if (c1.order()?.priority() < c2.order()?.priority()) { return -1; }

  if (c1.order()?.priority() > c2.order()?.priority()) { return 1; }

  return 0;
};

arc.data.column = function() {
  let colname;
  let coltype;
  let colfiltername;
  let coldesc;
  let colproperty;
  let colsource;
  let colindex;
  let colorder;
  let shelfIndex;
  let dimhlist;
  let dimhlevel;
  let coldisplayformat;
  let colvalues;
  let colgeotype;
  let secondaryaxis;
  let custom_expr;
  let dataTransformation;
  let expr;

  // For table visuals, track the column position to display this column in
  // if it is not the usual position
  let coldisplaybefore;

  let fn_name;
  let fn_cast;

  const data_column = function() { return 'arc.data.column'; };

  data_column.cloneDataColumn = function() {
    const c = arc.data.column()
               .colname(colname)
               .coltype(coltype)
               .colfiltername(colfiltername)
               .colproperty(colproperty)
               .colsource(colsource)
               .colindex(colindex)
               .shelfIndex(shelfIndex)
               .order(colorder ? colorder.cloneOrder() : colorder)
               .dimhlist(dimhlist ? dimhlist.slice(0) : dimhlist)
               .setDisplayFormat(coldisplayformat)
               .colvalues(colvalues)
               .colgeotype(colgeotype)
               .secondaryaxis(secondaryaxis)
               .expr(expr)
               .custom_expr(custom_expr)
               .dataTransformation(dataTransformation)
               .coldisplaybefore(coldisplaybefore)
               .fn_name(fn_name)
               .fn_cast(fn_cast);

    return c;
  };

  // Fixed values for this column. Used when measure values are
  // used to create a new column within a trellis
  data_column.colvalues = function(_) {
    if (!arguments.length) return colvalues;
    colvalues = _;
    return data_column;
  };

  // The name of the column
  data_column.colname = function(_) {
    if (!arguments.length) return colname;
    colname = _;
    return data_column;
  };

  // In table visuals, we permit measures to store column positions for
  // display that are not identical to the usual display positions.
  data_column.coldisplaybefore = function(_) {
    if (!arguments.length) return coldisplaybefore;
    coldisplaybefore = _;
    return data_column;
  };

  // At times, aliases are used in shelf expressions, say something like:
  //   stname as 'State'
  // In this scenario, when the 'Include' / 'Exclude' button is used to
  // add values to the filter shelves, it is important that we filter on
  // the original column name 'stname', and not the computed alias name
  // 'State', since only the original column name exists in the table and
  // is valid to place in the where clause.
  data_column.colfiltername = function(_) {
    if (!arguments.length) return colfiltername;
    colfiltername = _;
    return data_column;
  };

  data_column.coldesc = function(_) {
    if (!arguments.length) return coldesc;
    coldesc = _;
    return data_column;
  };

  // List of dim hierarchy associated with the column
  data_column.dimhlist = function(_) {
    if (!arguments.length) return dimhlist;
    dimhlist = _;
    return data_column;
  };

  data_column.dimhlevel = function(_) {
    if (!arguments.length) return dimhlevel;
    dimhlevel = _;
    return data_column;
  };

  const _fallbackToDefault = (func) => (...args) => {
    try {
      return func(...args);
    } catch (_) {
      return data_column._defaultFormat()(...args);
    }
  };

  // Display formatting associated with the column
  data_column.coldisplayformat = function(ignoreDefault = false) {
    if (ignoreDefault) {
      return coldisplayformat;
    }

    if (coldisplayformat) {
      return _fallbackToDefault(coldisplayformat);
    }

    return data_column._defaultFormat();
  };

  data_column.setDisplayFormat = function(format) {
    coldisplayformat = format;
    return data_column;
  };

  data_column._defaultFormat = function() {
    return arc.defaultFormat[data_column.isNumeric() ? 'number' : 'string'];
  };

  // Format a particular value in this column for display
  data_column.formatForDisplay = function(value) {
    return data_column.coldisplayformat()(value);
  };

  // Indicate whether a secondary axis is associated with this column
  data_column.secondaryaxis = function(_) {
    if (!arguments.length) return secondaryaxis;
    secondaryaxis = _;
    return data_column;
  };

  // The column index
  data_column.colindex = function(_) {
    if (!arguments.length) return colindex;
    colindex = _;
    return data_column;
  };

  // The index of the column within the shelf
  data_column.shelfIndex = function(_) {
    if (!arguments.length) return shelfIndex;
    shelfIndex = _;
    return data_column;
  };

  // The type of the column - 'string', 'integer', 'hierarchy', etc.
  data_column.coltype = function(_) {
    if (!arguments.length) return coltype;
    coltype = _;
    return data_column;
  };

  // colproperty is one of
  //   arc.report.columnTypeEnum.DIMENSION
  //   arc.report.columnTypeEnum.AGGREGATE
  //   arc.report.columnTypeEnum.UNKNOWN
  //
  data_column.colproperty = function(_) {
    if (!arguments.length) return colproperty;
    colproperty = _;
    return data_column;
  };

  // colsource indicates the shelf that this column came from. This
  // information is required for visuals to draw the data correctly.
  data_column.colsource = function(_) {
    if (!arguments.length) return colsource;
    colsource = _;
    return data_column;
  };

  // order indicates the ordering parameters associated with this column
  data_column.order = function(_) {
    if (!arguments.length) return colorder;
    colorder = _;
    return data_column;
  };

  // indicates whether this column has a geo type associated with it
  data_column.colgeotype = function(_) {
    if (!arguments.length) return colgeotype;
    colgeotype = _;
    return data_column;
  };

  // expression from the shelf element. Only included when doing shelvesToDataRequest
  data_column.expr = function(_) {
    if (!arguments.length) return expr;
    expr = _;
    return data_column;
  };

  // expression from the shelf element. Only included when doing shelvesToDataRequest
  data_column.custom_expr = function(_) {
    if (!arguments.length) return custom_expr;
    custom_expr = _;
    return data_column;
  };

  // dataTransformation from the shelf element. Only included when doing shelvesToDataRequest
  data_column.dataTransformation = function(_) {
    if (!arguments.length) return dataTransformation;
    dataTransformation = _;
    return data_column;
  };

  // fn_name from the shelf element. Only included when doing shelvesToDataRequest
  data_column.fn_name = function(_) {
    if (!arguments.length) return fn_name;
    fn_name = _;
    return data_column;
  };

  // fn_cast from the shelf element. Only included when doing shelvesToDataRequest
  data_column.fn_cast = function(_) {
    if (!arguments.length) return fn_cast;
    fn_cast = _;
    return data_column;
  };

  data_column.isNumeric = function() {
    return arc.report.coltypeIsNumber(coltype);
  };

  /* Hierarchy Related Functions */
  data_column.isHierarchy = function() {
    return (dimhlist && dimhlist.length > 0);
  };

  data_column.nextDrillDownColumnName = function () {
    if (!data_column.isHierarchy()) {
      return;
    }

    const currDimhLevel = dimhlevel || 0;
    return dimhlist[currDimhLevel + 1];
  };

  data_column.currentDrillDownColumnName = function() {
    if (!data_column.isHierarchy()) {
      return;
    }

    const currDimhLevel = dimhlevel || 0;
    return dimhlist[currDimhLevel];
  };

  data_column.prevDrillUpColumnName = function() {
    if (!data_column.isHierarchy()) {
      return;
    }

    const currDimhLevel = dimhlevel || 0;
    return dimhlist[currDimhLevel - 1];
  };

  return data_column;
};

arc.data.result = function() {
  let rows = [];
  let columns = [];
  let column_meta;

  const data_result = function() { return 'arc.data.result'; };

  data_result.rows = function(_) {
    if (!arguments.length) return rows;
    rows = _;
    return data_result;
  };

  data_result.columns = function(_) {
    if (!arguments.length) return columns;
    columns = _;
    return data_result;
  };

  data_result.add_column = function(_) {
    columns.push(_);
    _.colindex(columns.length - 1);
    return data_result;
  };

  // Compute dim/agg start/end indices based on the data in 'columns'
  data_result.finalize = function() {
    column_meta = arc.data.column_meta()
                     .columns(columns)
                     .finalize();
    return data_result;
  };

  // NOT USED
  data_result.columnNameToIndex = function(name) {
    let colindex;
    columns.some((col) => {
      if (col.colname() === name) {
        colindex = col.colindex();
        return true;
      }

      return false;
    });

    return colindex;
  };

  data_result.dim_cnt = function() {
    return column_meta.dim_cnt();
  };

  data_result.dim_start_idx = function() {
    return column_meta.dim_start_idx();
  };

  data_result.dim_end_idx = function() {
    return column_meta.dim_end_idx();
  };

  data_result.agg_cnt = function() {
    return column_meta.agg_cnt();
  };

  data_result.agg_start_idx = function() {
    return column_meta.agg_start_idx();
  };

  data_result.agg_end_idx = function() {
    return column_meta.agg_end_idx();
  };

  return data_result;
};
}());
