window.arc = window.arc || {};

(function() {
'use strict';

arc.dataconnection = arc.dataconnection || {};

arc.dataconnection.sortDataConnections = function(dataConnections) {
  dataConnections.sort((a, b) => {
    if (a.dataconnection_name().toLowerCase() < b.dataconnection_name().toLowerCase()) {
      return -1;
    }

    if (a.dataconnection_name().toLowerCase() > b.dataconnection_name().toLowerCase()) {
      return 1;
    }

    return 0;
  });
};

arc.dataconnection.sortDataSets = function(dataSets) {
  dataSets.sort((a, b) => {
    if (a.dataset_name().toLowerCase() < b.dataset_name().toLowerCase()) {
      return -1;
    }

    if (a.dataset_name().toLowerCase() > b.dataset_name().toLowerCase()) {
      return 1;
    }

    return 0;
  });
};

arc.dataconnection.DataConnection = function() {
  let dataconnection_id;
  let dataconnection_type;
  let dataconnection_name;
  let dataconnection_password;
  let dataconnection_description;
  let dataconnection_info;
  let do_validate = false;
  let dataconnection_datasets = [];

  const DataConnection = function() {
    return 'arc.dataconnection.DataConnection';
  };

  DataConnection.load = function(data) {
    dataconnection_id = data.id;
    dataconnection_type = data.type;
    dataconnection_name = data.name;
    dataconnection_description = data.description;
    dataconnection_info = data.info;

    return DataConnection;
  };

  DataConnection.datasets = function(_) {
    if (!arguments.length) return dataconnection_datasets;
    dataconnection_datasets = _;
    return DataConnection;
  };

  DataConnection.add_dataset = function(_) {
    dataconnection_datasets.push(_);
    return DataConnection;
  };

  DataConnection.validate = function(_) {
    if (!arguments.length) return do_validate;
    do_validate = _;
    return DataConnection;
  };

  DataConnection.dataconnection_id = function(_) {
    if (!arguments.length) return dataconnection_id;
    dataconnection_id = _;
    return DataConnection;
  };

  DataConnection.dataconnection_type = function(_) {
    if (!arguments.length) return dataconnection_type;
    dataconnection_type = _;
    return DataConnection;
  };

  DataConnection.dataconnection_name = function(_) {
    if (!arguments.length) return dataconnection_name;
    dataconnection_name = _;
    return DataConnection;
  };

  DataConnection.dataconnection_password = function(_) {
    if (!arguments.length) return dataconnection_password;
    dataconnection_password = _;
    return DataConnection;
  };

  DataConnection.dataconnection_description = function(_) {
    if (!arguments.length) return dataconnection_description;
    dataconnection_description = _;
    return DataConnection;
  };

  DataConnection.dataconnection_info = function(_) {
    if (!arguments.length) return dataconnection_info;
    dataconnection_info = _;
    return DataConnection;
  };

  DataConnection.save = function(done_cb) {
    let error;
    let response;
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      dataconnection_type,
      dataconnection_name,
      dataconnection_info,
      do_validate,
    };

    if (dataconnection_password) {
      post_data.dataconnection_password = dataconnection_password;
    }

    let url = arc.frontend.globals.urls.dataconnectionUrl;
    if (dataconnection_id) {
      url += `/${dataconnection_id}`;
    }

    $.ajax({
      url,
      data: post_data,

      // 'PUT' updates a event, 'POST' creates a new one
      type: dataconnection_id ? 'PUT' : 'POST',
      beforeSend: (xhr) => {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      }
    })
    .fail((rsp, textstatus, errorthrown) => {
      response = rsp;
      if (rsp.responseText) {
        textstatus = rsp.responseText;
      }

      if (do_validate) {
        error = arc.misc.escapeHTML(textstatus);
      } else {
        const action = dataconnection_id ? 'update' : 'create';
        arc.report.alertMessage(`Failed to ${action} data connection`,
            `${textstatus}. ${errorthrown}`);
      }
    })
    .always(() => {
      done_cb(error, response);
    });
  };

  return DataConnection;
};

arc.dataconnection.reset_cache = function(id, cb) {
  $.ajax({
    url: `${arc.frontend.globals.urls.dataconnectionUrl}/cache_reset/${id}`,
    type: 'POST',
    beforeSend: (xhr) => {
      // Because of a jquery bug when handling DELETE requests,
      // we have to explicitly set X-CSRFToken here
      xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
    }
  })
  .done(() => {
    if (cb) {
      cb();
    }
  })
  .fail((rsp) => {
    if (cb) {
      cb(rsp.responseText || rsp);
    }
  });
};

// Setup the 'New Connection' modal in the main 'Data' screen
arc.dataconnection.newconnections_modal_setup = function () {
  d3.select('#newconnection-button')
    .on('click.newconnection', () => {
      arc.frontend.legacyBridge.events.next({
        type: 'open-data-connection-modal',
        payload: {
          modalType: 'new'
        }
      });
    });
};

arc.dataconnection.dataconnections = function() {
  const dcs = function() { return 'arc.dataconnection.dataconnections'; };

  // eslint-disable-next-line no-unused-vars
  let dataconnections = {};

  dcs.dataconnections_fetch = function(cb) {
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.dataconnectionUrl)
      .await((error, dataconnections_data) => {
        if (error) {
          arc.report.alertMessage('Could not fetch dataconnections', error);
          return;
        }

        dataconnections = dataconnections_data;
        if (cb) {
          cb(dataconnections_data);
        }
      });
    return dcs;
  };

  return dcs;
};

arc.dataconnection.createSyncButton = function () {
  const $syncButton = $($.parseHTML(`
  <a
    id="syncconnection-button"
    data-toggle="tooltip"
    title="Detects connections in Cloudera AI Project. Local changes might be overwritten"
    class="do-not-detect-as-connectionclick btn btn-sm"
    role="button"
    data-toggle="modal"
    data-test-id="sync-connection-button"
  >
    <i class="fa fa-refresh fa-fw do-not-detect-as-connectionclick"></i>
    <i class="fa fa-spinner fa-spin do-not-detect-as-connectionclick" style="display: none"></i>
    Sync
    <sup><i class="fa fa-exclamation-circle" style="display:none; color: orange"></i><sup>
  </a>
`));
  const $syncIcon = $syncButton.find('.fa-refresh');
  const $loadingSpinnerIcon = $syncButton.find('.fa-spinner');

  const toggleIcons = () => {
    $syncIcon.toggle();
    $loadingSpinnerIcon.toggle();
  };

  const highlightConnectionOnSync = (connection, connectionClass, badgeClass, badgeText) => {
    const $connectionItem = $(`#dataconnections-list a[title=${connection}]`).closest('li');
    $connectionItem.addClass(connectionClass);
    $connectionItem.prepend(`<div class="connection-badge ${badgeClass}">${badgeText}</div>`);
  };

  $syncButton.find('[data-toggle="tooltip"]').tooltip({
    trigger: 'hover'
  });

  // set click for syncing cml connections
  $syncButton.offOn('click.syncCML', () => {
    const confirmSync = arc.misc.simpleUserConfirmation({
      confirmTitle: 'Confirmation',
      confirmQus: 'Any local modifications to previously synchronized connections will be overwritten. Are you sure you want to continue?',
      confirmText: 'Sync with Project',
    });

    confirmSync.askUser(() => {
      toggleIcons();

      $.ajax({
        type: 'GET',
        url: arc.frontend.globals.urls.syncCmlConnectionsUrl,
        beforeSend(xhr) {
          // Because of a jquery bug when handling GET requests,
          // we have to explicitly set X-CSRFToken here
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
        .done((connections) => {
          arc.apps_data.fetch_and_draw(() => {
            connections.new_connections.forEach((newConnection) => {
              highlightConnectionOnSync(newConnection, 'connection-new', 'connection-badge-new', 'New');
            });

            connections.updated_connections.forEach((updatedConnection) => {
              highlightConnectionOnSync(updatedConnection, 'connection-updated', 'connection-badge-updated', 'Updated');
            });
          });

          const syncedOutConnections = connections.synced_out_connection;
          if (syncedOutConnections?.length > 0) {
            toggleIcons();

            const outConnectionPopover = syncedOutConnections.join(', ');
            arc.report.alertMessageSmall(
              `Successful sync! These connections are out of sync: ${outConnectionPopover}`,
              'alert-success'
              );
            } else {
              arc.report.alertMessageSmall(
                `Successful sync! No Data Connections are out of sync with the project`,
                'alert-success'
                );
              }
        })
        .fail((rsp) => {
          toggleIcons();
          arc.report.alertMessageSmall(rsp.responseText, 'alert-danger');
        }).complete(() => $syncButton.tooltip('hide'));
    });
  });

  return $syncButton;
};
}());
