window.arc = window.arc || {};

(function() {
'use strict';

function castExpr(castType, expr) {
  return `cast((${expr}) as ${castType})`;
}

function scalarExpr(fn, colname) {
  return `${fn}(${colname})`;
}

const FIVE_MINUTES = 60 * 5,
      ONE_DAY = 60 * 60 * 24;

var DayNumberToShortDayNameSunIs0 = {
  0: 'Sun',
  1: 'Mon',
  2: 'Tue',
  3: 'Wed',
  4: 'Thu',
  5: 'Fri',
  6: 'Sat',
  7: 'Sun',
};

var DayNumberToLongDayNameSunIs0 = {
  0: 'Sunday',
  1: 'Monday',
  2: 'Tuesday',
  3: 'Wednesday',
  4: 'Thursday',
  5: 'Friday',
  6: 'Saturday',
  7: 'Sunday',
};

arc.dataconnection = arc.dataconnection || {};
arc.dataconnection.types = arc.dataconnection.types || {};

// enumerating connection types from apps_data.html
arc.dataconnection.connectionTypeEnum = {
  "arcengine": 1,
  "impala": 2,
  "hive": 3,
  "sqlite": 4,
  "aster": 5,
  "odbc": 6,
  "sparksql": 7,
  "mysql": 8,
  "mariadb": 9,
  "postgres": 10,
  "redshift": 11,
  "mongodb": 12,
  "oracle": 13,
  "drill": 14,
  'solr': 15,
  'mssqlserver': 16,
  'ksql': 17,
  'teradata': 18,
  'bigquery': 19,
  'snowflake': 20,
  'vertica': 21,
  'athena': 22,
  'denodo': 23,
  'elasticsearch': 24,
  'cassandra': 25,
  'impyla': 26,
  'duckdb': 27,
  'druid': 28,
  'hyve': 29,
  'arcengine_v2': 30,
};

arc.dataconnection.types.stripBrackets = function(colname) {
  if ((colname[0] === '[') && (colname[colname.length - 1] === ']')) {
    return colname.substr(1, colname.length - 2);
  }

  return colname;
};

arc.dataconnection.types.brackets = function(colname) {
  if (colname &&
      (colname.indexOf('(') < 0) &&
      (colname[0] !== '[') &&
      (colname[colname.length - 1] !== ']')) {
    colname = '[' + colname + ']';
  }

  return colname;
};

arc.dataconnection.sample_sql = function(fq_tablename, dc_type, rowcount) {
  var s = "select * from " +
      arc.dataconnection._enquote_tablename(fq_tablename, dc_type);

  if ((dc_type === "mssqlserver") || (dc_type === "ksql") ||
      (dc_type === 'bigquery') || (dc_type === "elasticsearch")) {
    return '.rows ' + fq_tablename;
  } else if (dc_type === "oracle") {
    return s + " where rownum <= " + rowcount;
  } else if (dc_type === "teradata") {
    return s + " sample " + rowcount;
  } else {
    return s + " limit " + rowcount;
  }
};

function getDC(dc_type) {
  if (dc_type) {
    return arc.dataconnection.types[dc_type]();
  }

  return arc.dataconnection.types.base();
}

arc.dataconnection.database_label = function(dc_type) {
  return getDC(dc_type).database_label;
};

arc.dataconnection.table_label = function(dc_type) {
  return getDC(dc_type).table_label;
};

// This corresponds to the supports_partitions property on the backend dataconnection code
const _supportsPartitions = new Set(['hyve', 'impyla', 'arcengine_v2']);
arc.dataconnection.supportsPartitions =
    (dc_type) => _supportsPartitions.has(dc_type);

// Note that this corresponds to the supports_dataextract_source property on the
// backend dataconnection code
const _supportsDataExtractsSource = new Set(['hyve', 'impyla', 'arcengine_v2',
    'druid', 'duckdb', 'mysql', 'mariadb', 'postgres', 'sqlite', 'snowflake']);
arc.dataconnection.supportsDataExtractsSource =
    (dc_type) => (_supportsDataExtractsSource.has(dc_type) ||
      arc.dataconnection.conns_extract_sources().includes(dc_type));

// Note that for data connections to be on this list, they need to be able to support
// the ability to create & delete data sources, and they need to support insert/select
// statements of the sort generated by datacopy.py in the backend. Data connections
// that support data extract target also support data import
const _supportsDataExtractsTarget = new Set(['hyve', 'impyla', 'arcengine_v2',
    'duckdb', 'sqlite', 'mysql', 'mariadb', 'postgres', 'snowflake']);
arc.dataconnection.supportsDataExtractsTarget =
    (dc_type) => (_supportsDataExtractsTarget.has(dc_type) ||
      arc.dataconnection.conns_extract_targets().includes(dc_type));

arc.dataconnection.defaultDatabaseNameForExtracts = function(dc_type) {
  if (dc_type === 'sqlite' || dc_type === 'duckdb') {
    return 'main';
  }

  return 'default';
};

const _isHiveEngine = new Set(['hive', 'hyve']);
arc.dataconnection.isHiveEngine = function(dc_type) {
  return _isHiveEngine.has(dc_type);
};

const _isFallbackTarget = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla', 'hive', 'hyve',
                                   'sparksql', 'athena']);
arc.dataconnection.isFallbackTarget = function(dc_type) {
  return _isFallbackTarget.has(dc_type);
};

const _isArcEngineCompatible = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla',
    'hive', 'hyve']);
arc.dataconnection.isArcEngineCompatible = function(dc_type) {
  return _isArcEngineCompatible.has(dc_type);
};

const _supportsIncrementalResults = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla',
    'cassandra']);
arc.dataconnection.supportsIncrementalResults = function(dc_type) {
  return _supportsIncrementalResults.has(dc_type);
};

const _supportsExportTable = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla',
    'hive', 'hyve', 'ksql', 'sqlite', 'postgres']);
arc.dataconnection.supportsExportTable = function(dc_type) {
  return _supportsExportTable.has(dc_type);
};

const hadoopOrSparkTypes = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla', 'hive',
    'hyve', 'sparksql']);
arc.dataconnection.isHadoopOrSparkEngine = function(dc_type) {
  return hadoopOrSparkTypes.has(dc_type);
};

const arcengineOrImpalaTypes = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla']);
arc.dataconnection.isArcEngineOrImpala = function(dc_type) {
  return arcengineOrImpalaTypes.has(dc_type);
};

arc.dataconnection.isSamplingSupported = arc.dataconnection.isHadoopOrSparkEngine;

arc.dataconnection.hasDescribeFormatted = arc.dataconnection.isArcEngineOrImpala;

const complexTypesConnections = new Set(['arcengine', 'arcengine_v2', 'impala', 'impyla',
    'hive', 'hyve', 'spark']);
arc.dataconnection.connectionSupportsComplexTypes = function(dc_type) {
  return complexTypesConnections.has(dc_type);
};

arc.dataconnection.isArcEngine = function(dc_type) {
  return (dc_type === 'arcengine' || dc_type === 'arcengine_v2');
};

arc.dataconnection.isVectorDB = function(dc_type) {
  return ['solr', 'sqlite'].includes(dc_type);
};

arc.dataconnection.supportsVectorDataType = function(dc_type) {
  return dc_type === 'solr';
};

arc.dataconnection.supportsNdv = function(dc_type) {
  return getDC(dc_type).supports_ndv;
};

arc.dataconnection.ndvFunction = function(dc_type, expr) {
  let dc = getDC(dc_type);
  if (dc.supports_ndv) {
    return `${dc.ndv}(${expr})`;
  }

  return undefined;
};

arc.dataconnection.exactCount = function(dc_type, expr) {
  let r = `count(distinct ${expr})`;
  return r;
};

arc.dataconnection.approximateOrExactCount = function(dc_type, expr) {
  let r = arc.dataconnection.ndvFunction(dc_type, expr);

  if (!r) {
    r = `count(distinct ${expr})`;
  }

  return r;
};

arc.dataconnection.supports_derived_data = function(dc_type) {
  return getDC(dc_type).supports_derived_data;
};

arc.dataconnection.supports_data_import = function(dc_type) {
  return arc.dataconnection.supportsDataExtractsTarget(dc_type);
};

function castAsColumnType(colname, coltype) {
  return "cast(" + arc.dataconnection.types.brackets(colname) + " as " + coltype + ")";
}

arc.dataconnection.castAsString = function(colname, dc_type) {
  const dc = getDC(dc_type);
  return castAsColumnType(colname, dc.cast_string);
};

arc.dataconnection.defaultPort = function(dc_type) {
  return getDC(dc_type).default_port;
};

arc.dataconnection.defaultCacheRetention = function(dc_type) {
  return getDC(dc_type).default_cache_retention;
};

arc.dataconnection.defaultCacheEnabled = function(dc_type) {
  return getDC(dc_type).default_cache_enabled;
};

arc.dataconnection._enquote_tablename = function(fq_tablename, dc_type) {
  // If table name is a select statement, don't do any quoting
  if (fq_tablename.indexOf('(') === 0) {
    return fq_tablename;
  }

  const dc = getDC(dc_type);
  return dc.quoteTablename(fq_tablename);
};

const SECOND = 1,
      MINUTE = 60,
      HOUR = 3600,
      DAY = 24 * HOUR,
      MONTH = 30 * DAY,
      YEAR = 365 * DAY;

const periodToSeconds = {
  SECOND: 1,
  MINUTE: 60,
  HOUR:   3600,
  DATE:   24 * 3600,
  MONTH:  30 * 24 * 3600,
  YEAR:   365 * 30 * 24 * 3600,
};

arc.dataconnection.dateConvert = function(config) {
  let {
    columnName,
    connection,
    unit,
    unitCount,
  } = config;

  columnName = arc.misc.quoteColumn(columnName);

  let expr;

  if (unitCount && unitCount !== 1 && periodToSeconds[unit]) {
    columnName = `from_unixtime((cast((unix_timestamp(${columnName})/` +
        `(${unitCount} * ${periodToSeconds[unit]})) as integer)) * ` +
        `${unitCount} * ${periodToSeconds[unit]}, 'yyyy-MM-dd HH:mm:ss')`;
  }

  if (unit === 'QUARTER') {
    if (!connection.expr_quarter) {
      return undefined;
    }

    return {
      expr: connection.expr_quarter(columnName),
      alias: 'quarter',
    };
  }

  if (unit === 'YEAR') {
    if (!connection.expr_year) {
      return undefined;
    }

    return {
      expr: connection.expr_year(columnName),
      alias: 'year',
    };
  }

  if (unit === 'YEARQUARTER') {
    if (!connection.expr_yearquarter) {
      return undefined;
    }

    return {
      expr: connection.expr_yearquarter(columnName),
      alias: 'year and quarter',
    };
  }

  if (unit === 'YEARMONTH') {
    if (!connection.expr_yearmonth) {
      return undefined;
    }

    return {
      expr: connection.expr_yearmonth(columnName),
      alias: 'year and month',
    };
  }

  if (unit === 'YEARWEEK') {
    if (!connection.expr_yearweek) {
      return undefined;
    }

    return {
      expr: connection.expr_yearweek(columnName),
      alias: 'year and week',
    };
  }

  if (unit === 'MONTH') {
    if (!connection.expr_month) {
      return undefined;
    }

    return {
      expr: connection.expr_month(columnName),
      alias: 'month',
    };
  }

  if (unit === 'DATE') {
    if (!connection.to_date) {
      return undefined;
    }

    return {
      expr: connection.to_date(columnName),
      alias: 'date',
    };
  }

  if (unit === 'HOUR') {
    if (!connection.expr_fullhour) {
      return undefined;
    }

    return {
      expr: connection.expr_fullhour(columnName),
      alias: 'hour',
    };
  }

  if (unit === 'MINUTE') {
    if (!connection.expr_fullminute) {
      return undefined;
    }

    return {
      expr: connection.expr_fullminute(columnName),
      alias: 'minute',
    };
  }

  if (unit === 'SECOND') {
    if (!connection.expr_fullsecond) {
      return undefined;
    }

    return {
      expr: connection.expr_fullsecond(columnName),
      alias: 'second',
    };
  }

  if (unit === 'DAYOFMONTH') {
    if (!connection.expr_dayofmonth) {
      return undefined;
    }

    return {
      expr: connection.expr_dayofmonth(columnName),
      alias: 'day of month',
    };
  }

  if (unit === 'DAYOFYEAR') {
    if (!connection.expr_dayofyear) {
      return undefined;
    }

    return {
      expr: connection.expr_dayofyear(columnName),
      alias: 'day of year',
    };
  }

  if (unit === 'DAYOFWEEK_STARTMONDAY_ENDAT6') {
    return {
      expr: connection.expr_dayofweek_mon(columnName, {
        endAt7: false,
      }),
      alias: 'day of week',
    };
  }

  if (unit === 'DAYOFWEEK_STARTMONDAY_ENDAT7') {
    return {
      expr: connection.expr_dayofweek_mon(columnName, {
        endAt7: true,
      }),
      alias: 'day of week',
    };
  }

  if (unit === 'DAYOFWEEK') {
    if (!connection.expr_dayofweek) {
      return undefined;
    }

    return {
      expr: connection.expr_dayofweek(columnName),
      alias: 'day of week',
    };
  }

  return undefined;
};

arc.dataconnection.types.quoteString = function(dc_type, value) {
  const dc = getDC(dc_type);
  return dc.quoteString(value);
};

arc.dataconnection.types.base = function() {
  var base = function() { return "arc.dataconnection.types.base"; };

  base.connection_type = "unknown";
  base.default_port = 0;
  base.default_cache_retention = ONE_DAY;
  base.default_cache_enabled = 0;

  base.database_label = 'Database';
  base.table_label = 'Table';

  base.quote_start = '`';
  base.quote_end = "`";

  base.str_quote_start = "'";
  base.str_quote_end = "'";

  base.quoteString = function(d) {
    return `${base.str_quote_start}${d}${base.str_quote_end}`;
  };

  base.quoteTableElement = function(e) {
    return `${base.quote_start}${e}${base.quote_end}`;
  };

  base.quoteTablename = function(fq_tablename) {
    var f = fq_tablename.split(".");
    if (f.length === 1) {
      return base.quoteTableElement(fq_tablename);
    }

    var name_list = [];
    f.forEach(function(s) {
      name_list.push(base.quoteTableElement(s));
    });

    return name_list.join(".");
  };

  base.cast_fn = 'cast';

  // Date functions
  base.date = "date";
  base.year = "year";
  base.month = "month";
  base.dayofyear = "dayofyear";
  base.dayofmonth = "dayofmonth";
  base.dayofweek = "dayofweek";

  base.upper = "upper";
  base.lower = "lower";

  base.supports_derived_data = false;
  base.supports_ndv = false;

  base.sum = "sum";
  base.count = "count";
  base.min = "min";
  base.max = "max";
  base.avg = "avg";
  base.ndv = "ndv";

  // TODO: handle this for connections that do not support it, such as teradata/snowflake
  base.group_concat = "group_concat";

  base.cast_timestamp = "timestamp";
  base.cast_string = "string";
  base.cast_bigint = "bigint";
  base.cast_real = "real";
  base.cast_boolean = "boolean";

  base.supportsLimit = true;
  base.supportsTopk = false;

  base.supportsWindowFunctions = true;

  base.ifCmd = "if";

  base.expr_if = function(column, ifClause) {
    return `${base.ifCmd}(${ifClause}, ${column}, NULL)`;
  };

  base.expr_if_else = function(ifClause, thenClause, elseClause) {
    return `${base.ifCmd}(${ifClause}, ${thenClause}, ${elseClause})`;
  };

  base.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    let yearExpr = `${base.year}(${colname})`;

    if (base.expr_year) {
      yearExpr = `${base.expr_year(colname)}`;
    }

    return `concat(cast(${yearExpr} as string), '-Q', ` +
           `cast(cast(((month(${colname}) + 2) / 3) as int) as string))`;
  };

  base.expr_quarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat('Q', cast(cast(((month(${colname}) + 2) / 3) as int) as string))`;
  };

  base.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(substring(to_date(${colname}), 1, 4), '-M', ` +
        `substring(to_date(${colname}), 6, 7))`;
  };

  base.is_string = function(coltype) {
    return coltype.toLowerCase() === "string";
  };

  base.dowToShortName = function(dow) {
    return DayNumberToShortDayNameSunIs0[dow];
  };

  base.dowToLongName = function(dow) {
    return DayNumberToLongDayNameSunIs0[dow];
  };

  base.dowToShortNameAlt = function(dow) {
    return DayNumberToShortDayNameSunIs0[dow - 1];
  };

  base.dowToLongNameAlt = function(dow) {
    return DayNumberToLongDayNameSunIs0[dow - 1];
  };

  base.dow_low = 0;

  base.expr_dayofweek_mon = function(colname, config) {
    config = config || {};

    // dow normally turns Sun into the lowest value. This transformation turns Sun into
    // a larger number than Saturday, so that the week starts at Monday
    var f;

    if (base.expr_dayofweek) {
      f = base.expr_dayofweek(colname);
    } else {
      colname = arc.dataconnection.types.brackets(colname);
      f = base.dayofweek + '(' + colname + ')';
    }

    let e = base.expr_if_else(`${f} = ${base.dow_low}`, `${base.dow_low + 7}`, f);
    if (config.endAt7) {
      e = e + ' - 1';
    }

    return e;
  };

  base.extract = function(colname, expr) {
    colname = arc.dataconnection.types.brackets(colname);
    return `extract(${expr} from ${colname})`;
  };

  base.extract_base = base.extract;

  base.timestampRangeExprTemplate = '%HTML_colname% between %HTML_quotedStart% and %HTML_quotedEnd%';
  base.timestampRangeExpr = function(colname, coltype, quotedStart, quotedEnd) {
    return arc.misc.safe_substitute(
      this.timestampRangeExprTemplate,
      { HTML_colname: colname,
        HTML_quotedStart: quotedStart,
        HTML_quotedEnd: quotedEnd }
    );
  };

  base.newConnectionDefaults = function() {
    return {};
  };

  return base;
};

arc.dataconnection.types.oracle = function() {
  var oracle = arc.dataconnection.types.base();

  oracle.database_label = 'User';

  oracle.quote_start = '"';
  oracle.quote_end = '"';

  oracle.cast_string = "varchar(30)";
  oracle.connection_type = "oracle";
  oracle.default_port = 1521;
  oracle.ndv = "APPROX_COUNT_DISTINCT";
  oracle.date = "";

  oracle.supportsLimit = false;

  oracle.timestampRangeExpr = function(colname, coltype, quotedStart, quotedEnd, timeformat) {
    return `${colname} between to_date(${quotedStart}, '${timeformat}') ` +
        `and to_date(${quotedEnd}, '${timeformat}')`;
  };

  oracle.expr_groupconcat = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "LISTAGG(" + colname + ", ',') WITHIN GROUP (ORDER BY " + colname + ")";
  };

  oracle.expr_year = function(colname) {
    return oracle.extract(colname, 'year');
  };

  oracle.expr_month = function(colname) {
    return oracle.extract(colname, 'month');
  };

  oracle.expr_dayofmonth = function(colname) {
    return oracle.extract(colname, 'day');
  };

  oracle.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "to_char(" + colname + ", 'YYYY-MM-DD')";
  };

  oracle.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(concat(to_char(${colname}, 'YYYY'), '-M'), to_char(${colname}, 'MM'))`;
  };

  oracle.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(concat(to_char(${colname}, 'YYYY'), '-W'), to_char(${colname}, 'WW'))`;
  };

  oracle.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_char(${colname}, 'WW')`;
  };

  oracle.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_char(${colname}, 'YYYY-MM-DD HH24')`;
  };

  oracle.expr_dayofyear = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "to_char(" + colname + ", 'DDD')";
  };

  oracle.expr_dayofweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "to_char(" + colname + ", 'd')";
  };

  return oracle;
};

arc.dataconnection.types.snowflake = function() {
  var snowflake = arc.dataconnection.types.base();

  snowflake.database_label = 'Schema';

  snowflake.supports_derived_data = true;
  snowflake.supportsWindowFunctions = false;
  snowflake.supports_ndv = true;
  snowflake.ndv = 'hll';

  snowflake.ifCmd = "iff";

  snowflake.quote_start = '"';
  snowflake.quote_end = '"';

  snowflake.connection_type = 'snowflake';
  snowflake.default_port = 0;

  snowflake.cast_bigint = 'integer';
  snowflake.cast_real = 'float';

  snowflake.autoDateDetection = true;
  snowflake.date = 'to_date';
  snowflake.dowToShortName = snowflake.dowToShortName;
  snowflake.dowToLongName = snowflake.dowToLongName;
  snowflake.dow_low = 0;

  snowflake.expr_appxmedian = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `approx_percentile(${colname},0.5)`;
  };

  snowflake.stddev = 'stddev';
  snowflake.stddev_pop = 'stddev_pop';
  snowflake.variance = 'variance';
  snowflake.variance_pop = 'variance_pop';

  snowflake.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(concat(YEAR(${colname})::string,'-M'),lpad(month(${colname})::string,2,'0'))`;
  };

  snowflake.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(concat(YEAR(${colname})::string,'-W'), ` +
        `lpad(WEEKOFYEAR(${colname})::string,2,'0'))`;
  };

  snowflake.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `lpad(WEEKOFYEAR(${colname})::string,2,'0')`;
  };

  snowflake.quoteTableElement = function(e) {
    return `"${e}"`;
  };

  snowflake.is_string = function(coltype) {
    coltype = coltype.toLowerCase();
    return ((coltype === "string") || (coltype === "timestamp"));
  };

  return snowflake;
};

arc.dataconnection.types.arcengine = function() {
  var arcengine = arc.dataconnection.types.base();

  arcengine.connection_type = "arcengine";
  arcengine.default_port = 31050;
  arcengine.default_cache_retention = FIVE_MINUTES;
  arcengine.default_cache_enabled = 1;

  arcengine.supports_derived_data = true;
  arcengine.supports_ndv = true;

  arcengine.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `from_timestamp(${colname}, 'yyyy-MM-dd')`;
  };

  arcengine.expr_year = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `from_timestamp(${colname}, 'yyyy')`;
  };

  arcengine.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(from_timestamp(${colname}, 'yyyy'), '-M', from_timestamp(${colname}, 'MM'))`;
  };

  arcengine.expr_yearweek = function(colname) {
    // Convert from ISO weekofyear to something that is expected by users
    // In Jan, if weekofyear returns a number > 50, then return '00' instead
    // In Dec, if weekofyear returns 01, then return '53' instead
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(from_timestamp(${colname}, 'yyyy'), '-W', ` +
        `if(month(${colname}) = 1 AND weekofyear(${colname}) > 50, '00', ` +
        `if(month(${colname}) = 12 AND weekofyear(${colname}) = 1, '53', ` +
        `concat(if(weekofyear(${colname}) < 10, '0', ''), ` +
        `cast(weekofyear(${colname}) as STRING)))))`;
  };

  arcengine.expr_week = function(colname) {
    // Convert from ISO weekofyear to something that is expected by users
    // In Jan, if weekofyear returns a number > 50, then return '00' instead
    // In Dec, if weekofyear returns 01, then return '52' instead
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(if(weekofyear(${colname}) < 10, '0', ''), ` +
           `cast(weekofyear(${colname}) as STRING))`;
  };

  arcengine.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `from_timestamp(${colname}, 'yyyy-MM-dd HH')`;
  };

  arcengine.expr_fullminute = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `from_timestamp(${colname}, 'yyyy-MM-dd HH:mm')`;
  };

  arcengine.expr_fullsecond = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `from_timestamp(${colname}, 'yyyy-MM-dd HH:mm:ss')`;
  };

  arcengine.autoDateDetection = true;
  arcengine.date = "to_date";
  arcengine.dowToShortName = arcengine.dowToShortNameAlt;
  arcengine.dowToLongName = arcengine.dowToLongNameAlt;
  arcengine.dow_low = 1;
  arcengine.appx_median = 'appx_median';
  arcengine.stddev = 'stddev';
  arcengine.stddev_pop = 'stddev_pop';
  arcengine.variance = 'variance';
  arcengine.variance_pop = 'variance_pop';

  return arcengine;
};

arc.dataconnection.types.arcengine_v2 = function() {
  var arcengine_v2 = arc.dataconnection.types.arcengine();
  arcengine_v2.connection_type = "arcengine_v2";
  return arcengine_v2;
};

arc.dataconnection.types.impala = function() {
  var impala = arc.dataconnection.types.arcengine();

  impala.connection_type = 'impala';
  impala.default_port = 21050;
  impala.default_cache_retention = ONE_DAY;
  impala.default_cache_enabled = 0;

  return impala;
};

arc.dataconnection.types.impyla = function() {
  var impyla = arc.dataconnection.types.impala();

  impyla.connection_type = 'impyla';

  impyla.newConnectionDefaults = function(dcTypeExtended) {
    if (dcTypeExtended === 'cdw-impyla') {
      return {
        PORT: '443',
        MODE: 'http',
        AUTH: 'ldap',
        SOCK: 'ssl',
      };
    }

    return {
      MODE: 'binary',
    };
  };

  return impyla;
};

arc.dataconnection.types.hive = function() {
  var hive = arc.dataconnection.types.arcengine();

  hive.connection_type = "hive";
  hive.default_port = 10000;
  hive.default_cache_retention = ONE_DAY;
  hive.default_cache_enabled = 0;
  hive.supports_ndv = false;

  hive.supports_derived_data = true;

  hive.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_date(${colname})`;
  };

  hive.expr_year = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `year(${colname})`;
  };

  hive.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, "yyyy'-M'MM")`;
  };

  hive.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${hive.expr_year(colname)} || '-W' || weekofyear(${colname})`;
  };

  hive.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${hive.expr_year(colname)} || '-Q' || quarter(${colname})`;
  };

  hive.expr_dayofyear = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, 'D')`;
  };

  hive.expr_dayofweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, 'EEEE')`;
  };

  hive.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, 'yyyy-MM-dd HH')`;
  };

  hive.expr_fullminute = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, 'yyyy-MM-dd HH:mm')`;
  };

  hive.expr_fullsecond = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, 'yyyy-MM-dd HH:mm:ss')`;
  };

  return hive;
};

arc.dataconnection.types.hyve = function() {
  var hyve = arc.dataconnection.types.hive();

  hyve.connection_type = 'hyve';
  hyve.supports_ndv = false;

  hyve.newConnectionDefaults = function(dcTypeExtended) {
    if (dcTypeExtended === 'cdw-hyve') {
      return {
        PORT: '443',
        MODE: 'http',
        AUTH: 'ldap',
        SOCK: 'ssl',
        SETTINGS: {
          'hive.server2.async.exec.async.compile': 'false',
        },
      };
    }

    return {
      SETTINGS: {
        'hive.server2.async.exec.async.compile': 'false'
      },
    };
  };

  return hyve;
};

arc.dataconnection.types.drill = function() {
  var drill = arc.dataconnection.types.base();

  drill.connection_type = "drill";
  drill.default_port = 31010;
  drill.date = "to_date";

  drill.expr_year = function(colname) {
    return drill.extract(colname, 'year');
  };

  drill.expr_month = function(colname) {
    return drill.extract(colname, 'month');
  };

  drill.expr_dayofmonth = function(colname) {
    return drill.extract(colname, 'day');
  };

  drill.expr_dayofyear = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "cast((unix_timestamp(" + colname + ", 'yyyy-MM-dd HH:mm:ss.SSS')" +
      "-unix_timestamp(" + drill.expr_year(colname) + ", 'yyyy'))/86400 as int)";
  };

  drill.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `REGEXP_REPLACE(` +
             `SUBSTR(to_date(${colname}),1, 7), '-', '-M'` +
           `)`;
  };

  drill.supportsWindowFunctions = false;

  return drill;
};

arc.dataconnection.types.sparksql = function() {
  var sparksql = arc.dataconnection.types.hive();

  sparksql.connection_type = "sparksql";

  return sparksql;
};

arc.dataconnection.types.mysql = function() {
  var mysql = arc.dataconnection.types.base();

  mysql.cast_string = "char";
  mysql.cast_bigint = "signed";

  mysql.connection_type = "mysql";
  mysql.default_port = 3306;

  mysql.supportsWindowFunctions = false;

  mysql.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
             `cast(year(${colname}) as char), ` +
             `'-M', ` +
             `${mysql.ifCmd}(month(${colname}) < 10, '0', ''), ` +
             `cast(month(${colname}) as char)` +
           `)`;
  };

  mysql.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
             `cast(year(${colname}) as char), ` +
             `'-W', ` +
             `${mysql.ifCmd}(weekofyear(${colname}) < 10, '0', ''), ` +
             `cast(weekofyear(${colname}) as char)` +
           `)`;
  };

  mysql.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
           `${mysql.ifCmd}(weekofyear(${colname}) < 10, '0', ''), ` +
           `cast(weekofyear(${colname}) as char)` +
           `)`;
  };

  mysql.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
           `date(${colname}), ' '` +
           `cast(hour(${colname}) as char)` +
           `)`;
  };

  mysql.is_string = function(coltype) {
    return String(coltype).toLowerCase() === 'string';
  };

  return mysql;
};

arc.dataconnection.types.mariadb = function() {
  var mariadb = arc.dataconnection.types.mysql();

  mariadb.connection_type = "mariadb";

  mariadb.supportsWindowFunctions = false;

  return mariadb;
};

arc.dataconnection.types.mssqlserver = function() {
  var mssqlserver = arc.dataconnection.types.mysql();

  mssqlserver.quote_start = '[';
  mssqlserver.quote_end = ']';

  mssqlserver.connection_type = "mssqlserver";
  mssqlserver.default_port = 1433;

  mssqlserver.supportsWindowFunctions = false;

  mssqlserver.timestampRangeExpr = function(colname, coltype, quotedStart, quotedEnd) {
    return `${colname} between convert(DATETIME, ${quotedStart}) ` +
        `and convert(DATETIME, ${quotedEnd})`;
  };

  return mssqlserver;
};

arc.dataconnection.types.postgres = function() {
  var postgres = arc.dataconnection.types.base();

  postgres.database_label = 'Schema';

  postgres.quote_start = '"';
  postgres.quote_end = '"';

  postgres.connection_type = "postgres";
  postgres.default_port = 5432;
  postgres.cast_string = "text";
  postgres.stddev = 'stddev_samp';
  postgres.stddev_pop = 'stddev_pop';
  postgres.variance = 'var_samp';
  postgres.variance_pop = 'var_pop';

  postgres.extract = function(colname, expr) {
    colname = arc.dataconnection.types.brackets(colname);
    return "extract(" + expr + " from " + colname + "::timestamp)";
  };

  postgres.expr_year = function(colname) {
    return postgres.extract(colname, 'year');
  };

  postgres.expr_month = function(colname) {
    return postgres.extract(colname, 'month');
  };

  postgres.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(TO_CHAR(${colname} :: DATE 'YYYY'), '-M', TO_CHAR(${colname} :: DATE 'MM'))`;
  };

  postgres.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(TO_CHAR(${colname} :: DATE 'YYYY'), '-W', TO_CHAR(${colname} :: DATE 'WW'))`;
  };

  postgres.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `TO_CHAR(${colname} :: DATE 'WW')`;
  };

  postgres.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `TO_CHAR(${colname} :: DATE 'YYYY-MM-DD HH24')`;
  };

  postgres.expr_dayofmonth = function(colname) {
    return postgres.extract(colname, 'day');
  };

  postgres.expr_dayofyear = function(colname) {
    return postgres.extract(colname, 'doy');
  };

  postgres.expr_dayofweek = function(colname) {
    return postgres.extract(colname, 'dow');
  };

  return postgres;
};

arc.dataconnection.types.redshift = function() {
  var redshift = arc.dataconnection.types.postgres();

  redshift.connection_type = "redshift";
  redshift.default_port = 5439;

  return redshift;
};

arc.dataconnection.types.denodo = function() {
  var denodo = arc.dataconnection.types.postgres();

  denodo.connection_type = "denodo";
  denodo.default_port = 9996;
  denodo.month = "getmonth";

  denodo.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "formatdate('YYYY-MM-DD', " + colname + ")";
  };

  denodo.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    let yearExpr = `${denodo.year}(${colname})`;

    if (denodo.expr_year) {
      yearExpr = `${denodo.expr_year(colname)}`;
    }

    return `concat(cast(${yearExpr} as text), '-Q', ` +
           `cast(cast(((getmonth(${colname}) + 2) / 3) as integer) as text))`;
  };

  denodo.expr_quarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat('Q', cast(cast(((getmonth(${colname}) + 2) / 3) as integer) as text))`;
  };

  denodo.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(formatdate('YYYY', ${colname}), '-M', formatdate('MM', ${colname}))`;
  };

  denodo.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(formatdate('YYYY', ${colname}), '-W', formatdate('WW', ${colname}))`;
  };

  denodo.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `formatdate('WW', ${colname})`;
  };

  denodo.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `formatdate('YYYY-MM-DD k', ${colname})`;
  };

  return denodo;
};

arc.dataconnection.types.sqlite = function() {
  var sqlite = arc.dataconnection.types.base();

  sqlite.connection_type = "sqlite";

  sqlite.cast_string = "text";

  sqlite.supports_derived_data = true;
  sqlite.supportsWindowFunctions = false;

  sqlite.is_string = function(coltype) {
    coltype = coltype.toLowerCase();
    return ((coltype === "string") || (coltype === "timestamp"));
  };

  sqlite.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date(${colname})`;
  };

  sqlite.expr_year = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "strftime('%Y', " + colname + ")";
  };

  sqlite.expr_month = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "strftime('%m', " + colname + ")";
  };

  sqlite.expr_dayofmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "strftime('%d', " + colname + ")";
  };

  sqlite.expr_dayofweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "strftime('%w', " + colname + ")";
  };

  sqlite.expr_dayofyear = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "strftime('%j', " + colname + ")";
  };

  sqlite.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `strftime('%Y-M%m', ${colname})`;
  };

  sqlite.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);

    // ssqlite week numbering is within 0-53 (POSIX) Impala or Hive from 1-53 (ISO)
    // this code snippet manage to shift the numbering if needed
    return `${sqlite.expr_year(colname)}
    || "-W"
    || printf('%02d', strftime('%W', ${colname})
    + (1 - strftime('%W', strftime('%Y', ${colname}) || '-01-04')))`;
    };

  sqlite.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `strftime('%W', ${colname})
     + (1 - strftime('%W', strftime('%Y', ${colname}) || '-01-04') )`;
  };

  sqlite.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `strftime('%Y-%m-%d %H', ${colname})`;
  };

  sqlite.expr_dayofweek_mon = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    const dayOfWeek = sqlite.expr_dayofweek(colname);
    return `case when ${dayOfWeek} = '0' then '7' else ${dayOfWeek} end`;
  };

  sqlite.expr_if = function(column, ifClause) {
    return `CASE WHEN ${ifClause} THEN ${column} ELSE NULL END`;
  };

  sqlite.expr_if_else = function(ifClause, thenClause, elseClause) {
    return `CASE WHEN ${ifClause} THEN ${thenClause} ELSE ${elseClause} END`;
  };

  sqlite.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${sqlite.expr_year(colname)} || "-Q" || ((strftime('%m', ${colname}) + 2) / 3)`;
  };

  sqlite.expr_quarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `"Q" || ((strftime('%m', ${colname}) + 2) / 3)`;
  };

  return sqlite;
};

arc.dataconnection.types.duckdb = function() {
  var duckdb = arc.dataconnection.types.sqlite();

  duckdb.quote_start = '"';
  duckdb.quote_end = '"';

  duckdb.connection_type = "duckdb";
  duckdb.cast_string = "varchar";

  duckdb.extract = function(colname, expr) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_part('${expr}', ${colname})`;
  };

  duckdb.expr_year = function(colname) {
    return duckdb.extract(duckdb.to_date(colname), 'year');
  };

  duckdb.expr_quarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `'Q' || cast((cast(${duckdb.expr_month(colname)} as INTEGER) + 2)/3 as varchar)`;
  };

  duckdb.expr_month = function(colname) {
    return duckdb.extract(duckdb.to_date(colname), 'month');
  };

  duckdb.expr_week = function(colname) {
    return duckdb.extract(duckdb.to_date(colname), 'week');
  };

  duckdb.expr_dayofmonth = function(colname) {
    return duckdb.extract(duckdb.to_date(colname), 'day');
  };

  duckdb.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `cast(${colname} as DATE)`;
  };

  duckdb.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${duckdb.expr_year(colname)} || '-M' || ${duckdb.expr_month(colname)}`;
  };

  duckdb.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${duckdb.expr_year(colname)} || '-' || ${duckdb.expr_quarter(colname)}`;
  };

  duckdb.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${duckdb.expr_year(colname)} || '-W' || ${duckdb.expr_week(colname)}`;
  };

  duckdb.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_char(${colname}, 'YYYY-MM-DD HH24')`;
  };

  duckdb.expr_dayofyear = function(colname) {
    return duckdb.extract(duckdb.to_date(colname), 'doy');
  };

  duckdb.expr_dayofweek = function(colname) {
    return duckdb.extract(duckdb.to_date(colname), 'dow');
  };

  return duckdb;
};

arc.dataconnection.types.druid = function() {
  var druid = arc.dataconnection.types.base();

  druid.quote_start = '"';
  druid.quote_end = '"';

  druid.connection_type = "druid";
  druid.cast_string = "string";

  druid.stddev = 'stddev';
  druid.stddev_pop = 'stddev_pop';
  druid.variance = 'variance';
  druid.variance_pop = 'var_pop';

  druid.extract = function(colname, expr) {
    colname = arc.dataconnection.types.brackets(colname);
    return `EXTRACT(${expr} FROM ${colname})`;
  };

  druid.timestampRangeExpr = function(colname, coltype, quotedStart, quotedEnd) {
    return `${colname} >= ${quotedStart} and ${colname} <= ${quotedEnd}`;
  };

  druid.expr_year = function(colname) {
    return druid.extract(colname, 'year');
  };

  druid.expr_quarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `'Q' || cast((cast(${druid.expr_month(colname)} as INTEGER) + 2)/3 as varchar)`;
  };

  druid.expr_month = function(colname) {
    return druid.extract(colname, 'month');
  };

  druid.expr_week = function(colname) {
    return druid.extract(colname, 'week');
  };

  druid.expr_dayofmonth = function(colname) {
    return druid.extract(colname, 'day');
  };

  druid.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `cast(${colname} as DATE)`;
  };

  druid.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${druid.expr_year(colname)} || '-M' || ${druid.expr_month(colname)}`;
  };

  druid.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${druid.expr_year(colname)} || '-' || ${druid.expr_quarter(colname)}`;
  };

  druid.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${druid.expr_year(colname)} || '-W' || ${druid.expr_week(colname)}`;
  };

  druid.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_char(${colname}, 'YYYY-MM-DD HH24')`;
  };

  druid.expr_dayofyear = function(colname) {
    return druid.extract(colname, 'doy');
  };

  druid.expr_dayofweek = function(colname) {
    return druid.extract(colname, 'dow');
  };

  druid.expr_if = function(column, ifClause) {
    return `CASE WHEN ${ifClause} THEN ${column} ELSE NULL END`;
  };

  druid.expr_if_else = function(ifClause, thenClause, elseClause) {
    return `CASE WHEN ${ifClause} THEN ${thenClause} ELSE ${elseClause} END`;
  };

  return druid;
};

arc.dataconnection.types.mongodb = function() {
  var mongodb = arc.dataconnection.types.base();

  mongodb.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
             `cast(year(${colname}) as string), ` +
             `concat(` +
               `'-M', ` +
               `concat(` +
                 `case when month(${colname}) < 10 then '0' else '' end,` +
                 `cast(month(${colname}) as string)
           )))`;
  };

  mongodb.connection_type = "mongodb";

  return mongodb;
};

arc.dataconnection.types.bigquery = function() {
  var bigquery = arc.dataconnection.types.base();

  bigquery.supports_ndv = true;
  bigquery.supports_derived_data = true;

  bigquery.cast_fn = 'safe_cast';

  bigquery.cast_bigint = "int64";
  bigquery.cast_real = "float64";
  bigquery.cast_boolean = "bool";

  bigquery.connection_type = "bigquery";
  bigquery.ndv = "approx_count_distinct";
  bigquery.stddev = 'stddev';
  bigquery.stddev_pop = 'stddev_pop';
  bigquery.variance = 'variance';
  bigquery.variance_pop = 'var_pop';
  bigquery.group_concat = 'string_agg';

  bigquery.date = undefined;
  bigquery.year = undefined;
  bigquery.month = undefined;
  bigquery.dayofyear = undefined;
  bigquery.dayofmonth = undefined;
  bigquery.dayofweek = undefined;

  bigquery.expr_year = function(colname) {
    return bigquery.extract(colname, 'year');
  };

  bigquery.expr_month = function(colname) {
    return bigquery.extract(colname, 'month');
  };

  bigquery.expr_dayofmonth = function(colname) {
    return bigquery.extract(colname, 'day');
  };

  bigquery.to_date = function(colname) {
    return bigquery.extract(colname, 'date');
  };

  bigquery.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "format_timestamp('%Y-M%m', " + colname + ")";
  };

  bigquery.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "format_timestamp('%Y-W%W', " + colname + ")";
  };

  bigquery.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "format_timestamp('%W', " + colname + ")";
  };

  bigquery.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "format_timestamp('%Y-%m-%d %k', " + colname + ")";
  };

  bigquery.expr_dayofyear = function(colname) {
    return bigquery.extract(colname, 'dayofyear');
  };

  bigquery.expr_dayofweek = function(colname) {
    return bigquery.extract(colname, 'dayofweek');
  };

  bigquery.dowToShortName = bigquery.dowToShortNameAlt;
  bigquery.dowToLongName = bigquery.dowToLongNameAlt;
  bigquery.dow_low = 1;

  return bigquery;
};

arc.dataconnection.types.odbc = function() {
  var odbc = arc.dataconnection.types.base();

  odbc.connection_type = "odbc";

  return odbc;
};

arc.dataconnection.types.aster = function() {
  var aster = arc.dataconnection.types.postgres();
  aster.default_port = 2406;

  aster.quote_start = '"';
  aster.quote_end = '"';

  aster.connection_type = "aster";

  return aster;
};

arc.dataconnection.types.teradata = function() {
  var teradata = arc.dataconnection.types.postgres();
  teradata.extract = teradata.extract_base;
  teradata.default_port = 1025;

  teradata.database_label = 'Database';

  teradata.quote_start = '"';
  teradata.quote_end = '"';

  teradata.supportsWindowFunctions = false;

  teradata.cast_string = "varchar(4096)";

  teradata.connection_type = "teradata";

  teradata.supportsLimit = false;
  teradata.supportsTopk = true;

  teradata.date = undefined;
  teradata.year = undefined;
  teradata.month = undefined;
  teradata.dayofyear = undefined;
  teradata.dayofmonth = undefined;
  teradata.dayofweek = undefined;

  teradata.topk = function(count) {
    return 'TOP ' + count;
  };

  teradata.timestampRangeExpr = function(colname, coltype, quotedStart, quotedEnd) {
    return `${colname} between DATE ${quotedStart} and DATE ${quotedEnd}`;
  };

  teradata.expr_year = function(colname) {
    return teradata.extract(colname, 'year');
  };

  teradata.expr_month = function(colname) {
    return teradata.extract(colname, 'month');
  };

  teradata.expr_dayofmonth = function(colname) {
    return teradata.extract(colname, 'day');
  };

  teradata.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "to_char(" + colname + ", 'YYYY-MM-DD')";
  };

  teradata.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(to_char(${colname}, 'YYYY'), '-M', to_char(${colname}, 'MM'))`;
  };

  teradata.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(to_char(${colname}, 'YYYY'), '-W', to_char(${colname}, 'WW'))`;
  };

  teradata.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_char(${colname}, 'WW')`;
  };

  teradata.expr_fullhour = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `to_char(${colname}, 'YYYY-MM-DD HH24')`;
  };

  teradata.expr_dayofyear = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "to_char(" + colname + ", 'DDD')";
  };

  teradata.expr_dayofweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "to_char(" + colname + ", 'd') - 1";
  };

  teradata.expr_dayofweek_mon = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return "COALESCE(NULLIF((to_char(cast(" + colname + " as date), 'd') - 1), 0), 7)";
  };

  return teradata;
};

arc.dataconnection.types.solr = function() {
  var solr = arc.dataconnection.types.base();
  solr.default_port = 443;

  solr.connection_type = "solr";
  solr.table_label = "Collection";

  solr.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
             `cast(year(${colname}) as string), ` +
             `concat(` +
               `'-M', ` +
               `concat(` +
                 `case when month(${colname}) < 10 then '0' else '' end,` +
                 `cast(month(${colname}) as string)
           )))`;
  };

  solr.newConnectionDefaults = function() {
    return {
      SOCK: 'ssl',
      SSLCERT_NAME_MISMATCH: false,
    };
  };

  solr.stddev = 'stddev';
  solr.missing = 'missing';
  solr.sumofsquares = 'sumOfSquares';

  solr.supportsWindowFunctions = false;

  return solr;
};

arc.dataconnection.types.elasticsearch = function() {
  var elasticsearch = arc.dataconnection.types.base();
  elasticsearch.default_port = 9200;

  elasticsearch.quote_start = '"';
  elasticsearch.quote_end = '"';

  elasticsearch.connection_type = "elasticsearch";
  elasticsearch.table_label = "Index";

  elasticsearch.date = undefined;

  elasticsearch.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
             `cast(year(${colname}) as string), ` +
             `concat(` +
               `'-M', ` +
               `cast(month(${colname}) as string)
           ))`;
  };

  elasticsearch.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `concat(` +
             `cast(year(${colname}) as string), ` +
             `concat(` +
               `'-W', ` +
               `cast(ISO_WEEK_OF_YEAR(${colname}) as string)
           ))`;
  };

  elasticsearch.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `cast(ISO_WEEK_OF_YEAR(${colname}) as string)`;
  };

  elasticsearch.expr_appxmedian = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `percentile(${colname},50)`;
  };

  elasticsearch.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `substring(` + `cast(${colname} as VARCHAR)` + `, 0, 10)`;
  };

  elasticsearch.stddev = 'stddev';
  elasticsearch.stddev_pop = 'stddev_pop';
  elasticsearch.variance = 'variance';
  elasticsearch.variance_pop = 'variance_pop';
  elasticsearch.upper = 'ucase';
  elasticsearch.lower = 'lcase';
  elasticsearch.dayofyear = "day_of_year";
  elasticsearch.dayofmonth = "day_of_month";
  elasticsearch.dayofweek = "day_of_week";
  elasticsearch.cast_string = "varchar";
  elasticsearch.supportsWindowFunctions = false;

  return elasticsearch;
};

arc.dataconnection.types.ksql = function() {
  var ksql = arc.dataconnection.types.base();
  ksql.default_port = 9098;

  ksql.database_label = 'Topic';
  ksql.table_label = "Table or Stream";

  ksql.upper = 'ucase';
  ksql.lower = 'lcase';

  ksql.connection_type = 'ksql';
  ksql.supportsWindowFunctions = false;

  return ksql;
};

arc.dataconnection.types.athena = function() {
  var athena = arc.dataconnection.types.base();

  athena.connection_type = "athena";

  athena.supportsWindowFunctions = true;
  athena.supports_ndv = true;
  athena.default_cache_retention = 6 * FIVE_MINUTES;
  athena.default_cache_enabled = 1;

  athena.quote_start = '"';
  athena.quote_end = '"';

  athena.cast_string = "varchar";
  athena.stddev = 'stddev';
  athena.stddev_pop = 'stddev_pop';
  athena.variance = 'variance';
  athena.variance_pop = 'var_pop';
  athena.supports_ndv = true;
  athena.ndv = "approx_distinct";
  athena.autoDateDetection = true;
  athena.date = 'date';
  athena.dowToShortName = athena.dowToShortName;
  athena.dowToLongName = athena.dowToLongName;
  athena.dow_low = 0;

  athena.timestampRangeExpr = function(colname, coltype, quotedStart, quotedEnd) {
    return `${colname} between timestamp ${quotedStart} and timestamp ${quotedEnd}`;
  };

  athena.expr_appxmedian = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `approx_percentile(${colname},0.5)`;
  };

  athena.extract = function(colname, expr) {
    colname = arc.dataconnection.types.brackets(colname);
    return "extract(" + expr + " from " + colname + ")";
  };

  athena.expr_year = function(colname) {
    return athena.extract(colname, 'year');
  };

  athena.expr_month = function(colname) {
    return athena.extract(colname, 'month');
  };

  athena.expr_quarter = function(colname) {
    return `'Q' || cast(cast(((${athena.expr_month(colname)} + 2) / 3) as INTEGER) as varchar)`;
  };

  athena.expr_yearquarter = function(colname) {
    return `cast(${athena.expr_year(colname)} as varchar) ||
            ${athena.expr_quarter(colname)}`;
  };

  athena.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, '%Y-%m')`;
  };

  athena.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `date_format(${colname}, '%w')`;
  };

  athena.expr_dayofmonth = function(colname) {
    return athena.extract(colname, 'day');
  };

  athena.expr_dayofyear = function(colname) {
    return athena.extract(colname, 'doy');
  };

  athena.expr_dayofweek = function(colname) {
    return athena.extract(colname, 'dow');
  };

  return athena;
};

arc.dataconnection.types.vertica = function() {
  var vertica = arc.dataconnection.types.base();

  vertica.database_label = 'Schema';

  vertica.connection_type = "vertica";
  vertica.default_port = 5433;
  vertica.supportsWindowFunctions = true;
  vertica.supports_ndv = true;
  vertica.default_cache_retention = 6 * FIVE_MINUTES;
  vertica.default_cache_enabled = 1;

  vertica.quote_start = '"';
  vertica.quote_end = '"';

  vertica.cast_string = "varchar";
  vertica.stddev = 'stddev';
  vertica.stddev_pop = 'stddev_pop';
  vertica.variance = 'variance';
  vertica.variance_pop = 'var_pop';
  vertica.ndv = "approximate_count_distinct";
  vertica.appx_median = 'approximate_median';

  vertica.extract = function(colname, expr) {
    colname = arc.dataconnection.types.brackets(colname);
    return "extract(" + expr + " from " + colname + "::timestamp)";
  };

  vertica.expr_year = function(colname) {
    return vertica.extract(colname, 'year');
  };

  vertica.expr_month = function(colname) {
    return vertica.extract(colname, 'month');
  };

  vertica.expr_yearmonth = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `TO_CHAR(${colname} :: DATE, 'YYYY') || '-M' || TO_CHAR(${colname} :: DATE, 'MM')`;
  };

  vertica.expr_yearweek = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `TO_CHAR(${colname} :: DATE, 'YYYY') || '-W' || TO_CHAR(${colname} :: DATE, 'WW')`;
  };

  vertica.expr_week = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `TO_CHAR(${colname} :: DATE, 'WW')`;
  };

  vertica.expr_dayofmonth = function(colname) {
    return vertica.extract(colname, 'day');
  };

  vertica.expr_dayofyear = function(colname) {
    return vertica.extract(colname, 'doy');
  };

  vertica.expr_dayofweek = function(colname) {
    return vertica.extract(colname, 'dow');
  };

  vertica.expr_dayofweek_mon = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `TO_CHAR(${colname} :: DATE, 'D')`;
  };

  vertica.expr_quarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `'Q' || ${vertica.extract(colname, 'quarter')}`;
  };

  vertica.expr_yearquarter = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `${vertica.expr_year(colname)} || 'Q' || ${vertica.extract(colname, 'quarter')}`;
  };

  return vertica;
};

arc.dataconnection.types.cassandra = function() {
  var cassandra = arc.dataconnection.types.base();
  cassandra.default_port = 9042;

  cassandra.quote_start = '"';
  cassandra.quote_end = '"';

  cassandra.connection_type = "cassandra";
  cassandra.database_label = 'Keyspace';

  cassandra.date = undefined;
  cassandra.year = undefined;
  cassandra.month = undefined;
  cassandra.dayofyear = undefined;
  cassandra.dayofmonth = undefined;
  cassandra.dayofweek = undefined;

  cassandra.to_date = function(colname) {
    colname = arc.dataconnection.types.brackets(colname);
    return `toDate(${colname})`;
  };

  cassandra.aggregate = 'agg';

  cassandra.dayofyear = "day_of_year";
  cassandra.dayofmonth = "day_of_month";
  cassandra.dayofweek = "day_of_week";
  cassandra.cast_string = "varchar";
  cassandra.supportsWindowFunctions = false;
  cassandra.supportsTopk = false;
  cassandra.supportsLimit = true;

  return cassandra;
};

const extensions = [];
arc.dataconnection.add_extension = function(ext_json) {
  const ext = JSON.parse(ext_json);
  arc.dataconnection.types[ext.name] = function() {
    const base = arc.dataconnection.types[ext.conf.base]();
    base.connection_type = ext.name;

    const { overrides } = ext.conf;
    if (overrides) {
      for (const [key, value] of Object.entries(overrides)) {
        base[key] = value;
      }
    }

    return base;
  };

  extensions.push(ext);
};

arc.dataconnection.conns_with_advanced_settings = function() {
  return extensions.filter((e) => e.conf.has_advanced_settings).map((e) => e.name);
};

arc.dataconnection.conns_with_concurrency_settings = function() {
  return extensions.filter((e) => e.conf.has_concurrency_settings).map((e) => e.name);
};

arc.dataconnection.conns_with_http_connection_mode = function() {
  return extensions.filter((e) => e.conf.has_http_connection_mode).map((e) => e.name);
};

arc.dataconnection.conns_with_impersonation = function() {
  return extensions.filter((e) => e.conf.has_impersonation).map((e) => e.name);
};

arc.dataconnection.conns_extract_sources = function() {
  return extensions.filter((e) => e.conf.is_extract_source).map((e) => e.name);
};

arc.dataconnection.conns_extract_targets = function() {
  return extensions.filter((e) => e.conf.is_extract_target).map((e) => e.name);
};

arc.dataconnection.conns_direct_accessible = function() {
  return extensions.filter((e) => e.conf.is_direct_accessible).map((e) => e.name);
};
}());
