window.arc = window.arc || {};

(function() {
'use strict';

arc.datasets = arc.datasets || {};

arc.datasets.CREATED_FROM_SQL_DETAIL = 'Created from SQL';

arc.datasets.createFromSql = function() {
  let c = function() { return 'arc.datasets.createFromSql'; };

  let dataconnection_id;
  let dataconnection_type;
  let tgt_dataset_name;
  let sql;
  let showSuccessMsg;
  let successCb;
  let errorCb;

  c.dataconnection_id = function(_) {
    if (!arguments.length) return dataconnection_id;
    dataconnection_id = _;
    return c;
  };

  c.dataconnection_type = function(_) {
    if (!arguments.length) return dataconnection_type;
    dataconnection_type = _;
    return c;
  };

  c.tgt_dataset_name = function(_) {
    if (!arguments.length) return tgt_dataset_name;
    tgt_dataset_name = _;
    return c;
  };

  c.sql = function(_) {
    if (!arguments.length) return sql;
    sql = _;
    return c;
  };

  c.showSuccessMsg = function(_) {
    if (!arguments.length) return showSuccessMsg;
    showSuccessMsg = _;
    return c;
  };

  c.successCb = function(_) {
    if (!arguments.length) return successCb;
    successCb = _;
    return c;
  };

  c.errorCb = function(_) {
    if (!arguments.length) return errorCb;
    errorCb = _;
    return c;
  };

  function successMsg(dataset) {
    const saved_dataset_id = dataset.dataset_id();

    const dslink = arc.misc.getBlankTargetUrl(`/arc/apps/dataset/${saved_dataset_id}`);
    const dblink = arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.reportBuildUrl}/0?report_type=dashboard&dataset_id=${saved_dataset_id}`);

    const msg =
        `Dataset <a target="_blank" href="${dslink}">${tgt_dataset_name}</a> created\n` +
        `<div><a target="_blank" href="${dblink}">Create new dashboard</a></div>`;

    arc.report.alertMessageSmallBase({
      type: "alert-success",
      alertHeader: "Success",
      alertMessage: msg,
      fadeOut: false,
      useHTML: true
    });
  }

  function createDataset(dataset) {
    dataset.save(function(errorMsg) {
      if (!dataset.dataset_id()) {
        if (errorCb) {
          if (!errorMsg) {
            errorMsg = 'Internal Error: Failed to create dataset';
          }

          errorCb(errorMsg);
        }

        return;
      }

      if (showSuccessMsg) {
        successMsg(dataset);
      }

      if (successCb) {
        successCb(dataset);
      }
    });
  }

  c.fetchColumns = function(onSuccess) {
    if (sql) {
      const params = {
        dataconnection_id,
        cache_read: true,
      };

      // This is the same logis ac in views_adminapi.py
      let query = sql;
      if (dataconnection_type === 'oracle') {
        query = `select * from (${query}) t1 where rownum <= 1`;
      } else if (dataconnection_type === 'mssqlserver') {
        query = `select top(1) * from (${query}) t1`;
      } else if (dataconnection_type === 'sqlite') {
        query = `select * from (${query}) t1 limit 1`;
      } else {
        query = `select * from (${query}) t1 limit 0`;
      }

      params.query = query;

      arc.sqlrun.runSQLWithParams(params,
          (r) => {
            onSuccess(r.response);
          },
          (msg) => {
            errorCb(`Failed to fetch columns: ${msg}`);
          });

      return c;
    }

    const msg = 'Internal error. Cannot create dataset.';
    if (errorCb) {
      errorCb(msg);
    }

    return c;
  };

  c.initializeDatasetObjectFromQueryResult = function(queryResult /* {colnames, coltypes} */) {
    const query = sql;
    const dataset = arc.datasets.Dataset()
                     .dataconnection_id(dataconnection_id)
                     .dataconnection_type(dataconnection_type)
                     .dataset_name(tgt_dataset_name)
                     .dataset_type('singletable')
                     .dataset_info('')
                     .dataset_detail(arc.datasets.CREATED_FROM_SQL_DETAIL);

    // Build and add new table structure
    const table = arc.datasets.Table()
                     .dataset(dataset)
                     .tablename('text.sql')
                     .init_from_query(queryResult.colnames, queryResult.coltypes, query);

    dataset.add_table(table);
    return dataset;
  };

  function columnsSuccess(r) {
    try {
      createDataset(c.initializeDatasetObjectFromQueryResult(r));
    } catch (e) {
      console.log(e);
      if (errorCb) {
        errorCb(e.message);
      }
    }
  }

  c.create = function() {
    c.fetchColumns(columnsSuccess);
  };

  return c;
};
}());
