window.arc = window.arc || {};

(function() {
'use strict';

arc.extracts = arc.extracts || {};

arc.extracts.extractSourceBadge = '<span class="badge badge-extract-source">Extract Source</span>';
arc.extracts.extractTargetBadge = '<span class="badge badge-extract-target">Extract Target</span>';

arc.extracts.stateStringsFromLog = function(extractLog) {
  const state = extractLog ? arc.misc.escapeHTML(extractLog.status) : 'Never run';

  let stateLabel;
  let htmlState = state;
  if (state === 'Never run') {
    stateLabel = 'badge-warning';
  }

  if (state === 'Completed') {
    htmlState = 'Success';
    stateLabel = 'badge-success';
  }

  if (state === 'Error') {
    htmlState = 'Error';
    stateLabel = 'badge-danger';
  }

  if (state === 'Still Running') {
    htmlState = 'Running';
    stateLabel = 'badge-warning';
  }

  return {
    state,
    htmlState,
    stateLabel,
  };
};

arc.extracts.extractTracker = function() {
  const et = function() { return 'arc.extracts.extractTracker'; };

  const extractSourceDatasets = {};
  const extractTablesMap = {};

  et.trackExtracts = function(extracts, extractLogs) {
    const extractIdToLog = Object.fromEntries(extractLogs.map((el) => [el.extractId, el]));

    extracts.forEach((e) => {
      const log = extractIdToLog[e.id];
      if (log) {
        e.logs = log;
      }

      extractTablesMap[[e.tgtDataconnectionId, `${e.tgtDbname}.${e.tgtTablename}`]] = e;

      if (extractSourceDatasets[e.srcDatasetId]) {
        extractSourceDatasets[e.srcDatasetId].push(e);
        return;
      }

      extractSourceDatasets[e.srcDatasetId] = [e];
    });
  };

  et.getDatasetExtracts = function(dsId) {
    return extractSourceDatasets[dsId] || [];
  };

  et.isExtractTarget = function(dcId, tableName) {
    return extractTablesMap[[dcId, tableName]];
  };

  return et;
};

})();

