window.arc = window.arc || {};

(function() {
'use strict';

arc.solr = arc.solr || {};

arc.solr.solrUrl = function() {
  var solrUrl = function() {
    return 'solrUrl';
  };

  var protocol,
      ip,
      port,
      collection,
      query,
      viz_lib,
      facets = [],
      rows = 10;

  function newkv(key, value) {
    return {
      key: key,
      value: value
    };
  }

  function dimShelf() {
    var vizType = viz_lib.viz_type();
    if (vizType === 'map') {
      return 'geo_shelf';
    }

    return 'x_shelf';
  }

  function aggShelf() {
    var vizType = viz_lib.viz_type();
    if (vizType === 'map') {
      return 'aggregates_shelf';
    }

    return 'y_shelf';
  }

  solrUrl.columns = function(facetName) {
    if (!facetName && facets.length === 0) {
      return undefined;
    }

    if (!facetName) {
      facetName = facets[0];
    }

    console.log('columns', facetName, viz_lib.viz_type());

    var columns = [];
    columns.push(
        arc.data.column()
                .colname(facetName)
                .coltype('string')
                .colproperty(arc.report.columnTypeEnum.DIMENSION)
                .colsource(dimShelf())
                .colindex(0));
    columns.push(
        arc.data.column()
                .colname('Record Count')
                .coltype('bigint')
                .colproperty(arc.report.columnTypeEnum.AGGREGATE)
                .colsource(aggShelf())
                .colindex(1));

    return columns;
  };

  // http://localhost:8983/solr/airbnb/select?indent=on&q=*:*&wt=json
  solrUrl.url = function() {
    var kvs = [];
    kvs.push(newkv('indent', 'on'));
    kvs.push(newkv('wt', 'json'));
    kvs.push(newkv('q', query));

    if (facets.length > 0) {
      kvs.push(newkv('rows', 0));
      kvs.push(newkv('facet.mincount', 1));
      kvs.push(newkv('facet', 'on'));

      facets.forEach(function(facet) {
        kvs.push(newkv('facet.field', facet));
      });
    } else {
      kvs.push(newkv('rows', rows));
    }

    var params = kvs.map(function(kv) {
      return kv.key + '=' + kv.value;
    }).join('&');

    return arc.misc.safe_substitute(
        '%PROTOCOL%://%IP%:%PORT%/solr/%COLLECTION%/select?%PARAMS%',
        {
          PROTOCOL: protocol,
          IP: ip,
          PORT: port,
          COLLECTION: collection,
          PARAMS: params
        });
  };

  solrUrl.addFacet = function(_) {
    facets.push(_);
    return solrUrl;
  };

  solrUrl.rows = function(_) {
    if (!arguments.length) return rows;
    rows = _;
    return solrUrl;
  };

  solrUrl.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return solrUrl;
  };

  solrUrl.query = function(_) {
    if (!arguments.length) return query;
    query = _;
    return solrUrl;
  };

  solrUrl.protocol = function(_) {
    if (!arguments.length) return protocol;
    protocol = _;
    return solrUrl;
  };

  solrUrl.ip = function(_) {
    if (!arguments.length) return ip;
    ip = _;
    return solrUrl;
  };

  solrUrl.port = function(_) {
    if (!arguments.length) return port;
    port = _;
    return solrUrl;
  };

  solrUrl.collection = function(_) {
    if (!arguments.length) return collection;
    collection = _;
    return solrUrl;
  };

  return solrUrl;
};

arc.solr.solrData = function() {
  var solrData = function() {
    return 'solrData';
  };

  var dataset,
      viz_lib;

  solrData.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return solrData;
  };

  solrData.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return solrData;
  };

  solrData.solrUrl = function() {
    var collection = dataset.dataset_name();
    var u = arc.solr.solrUrl()
                    .viz_lib(viz_lib);
    u.protocol('http')
     .ip('localhost')
     .port(8983)
     .collection(collection);
    return u;
  };

  return solrData;
};

arc.solr.requestHandler = function() {
  var requestHandler = function() {
    return 'arc.solr.requestHandler';
  };

  var dataRequest,
      driver,
      viz_lib;

  requestHandler.dataRequest = function(_) {
    if (!arguments.length) return dataRequest;
    dataRequest = _;
    return requestHandler;
  };

  requestHandler.driver = function(_) {
    if (!arguments.length) return driver;
    driver = _;
    return requestHandler;
  };

  requestHandler.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return requestHandler;
  };

  function handleSimpleDocuments(data) {
    var docs = data.response.docs;

    if (docs.length === 0) {
      return false;
    }

    var colnames = Object.keys(docs[0]);

    var rows = docs.map(function(doc) {
      return colnames.map(function(colname) {
        return doc[colname];
      });
    });

    var sUrl = arc.solr.solrUrl()
                       .viz_lib(viz_lib);
    var columns = [];
    colnames.forEach(function(colname, i) {
      columns.push(
          arc.data.column()
                  .colname(colname)
                  .coltype('string')
                  .colproperty(arc.report.columnTypeEnum.DIMENSION)
                  .colsource('dimensions_shelf')
                  .colindex(i));
    });

    var result = arc.data.result()
                         .rows(rows)
                         .columns(columns)
                         .finalize();
    driver.executeRedraw(result);
    return true;
  }

  function handleFacetedResults(data) {
    var facetCounts = data.facet_counts;
    if (!facetCounts) {
      return false;
    }

    var facetFields = facetCounts.facet_fields;

    var fieldNames = Object.keys(facetFields);
    if (fieldNames.length === 0) {
      return false;
    }

    var fieldName = fieldNames[0];
    var facetData = facetFields[fieldName];

    var rows = [];
    facetData.forEach(function(d, i) {
      if ((i % 2) === 1) {
        rows.push([facetData[i - 1], facetData[i]]);
      }
    });

    var sUrl = arc.solr.solrUrl()
                       .viz_lib(viz_lib);
    var columns = sUrl.columns(fieldName);
    var result = arc.data.result()
                         .rows(rows)
                         .columns(columns)
                         .finalize();
    driver.executeRedraw(result);
    return true;
  }

  requestHandler.send = function() {
    $.ajax({
      url: `${arc.frontend.globals.urls.sqlRunUrl}/../runsolr?query=${encodeURIComponent(dataRequest.url())}`,
    })
    .done((data) => {
      if (handleFacetedResults(data)) {
        return;
      }

      handleSimpleDocuments(data);
    });
  };

  return requestHandler;
};

arc.solr.processSolrRequest = function(data_request, driver) {
  arc.solr.requestHandler()
          .dataRequest(data_request)
          .driver(driver)
          .viz_lib(driver)
          .send();
};

}());
