window.arc = window.arc || {};

(function() {
"use strict";

arc._tableid = 1;
arc.data_requests = arc.data_requests || {};

arc.tableid = function() {
  return arc._tableid++;
};

arc.mrBuilder = function() {
  var select_builder = null,
    magnitude = null,
    magnitude_count_distinct = [],
    magnitude_alias = null,
    projections = [],
    partition_bys = [],
    orderby = [],
    session_start = null,
    path = null,
    path_length = 10,
    path_display = null,
    goal = null,
    goal_display = null,
    outerFilters = [],
    topk = null,
    funnel_prefix = "ARC:FUNNEL:",
    paths = [],
    funnel_paths = [],
    show_exit_events = true,
    show_dropoff = true,
    is_funnel = false;

  var _internal_tableid = 1;
  var _external_tableid = 2;

  var mr_builder = function() {};

  mr_builder.select_builder = function(_) {
    if (!arguments.length) return select_builder;
    select_builder = _;
    return mr_builder;
  };

  mr_builder.show_exit_events = function(_) {
    if (!arguments.length) return show_exit_events;
    show_exit_events = _;
    return mr_builder;
  };

  mr_builder.show_dropoff = function(_) {
    if (!arguments.length) return show_dropoff;
    show_dropoff = _;
    return mr_builder;
  };

  mr_builder.funnel_prefix = function(_) {
    if (!arguments.length) return funnel_prefix;
    funnel_prefix = _;
    return mr_builder;
  };

  mr_builder.sessionStart = function(_) {
    if (!arguments.length) return session_start;
    session_start = _;
    return mr_builder;
  };

  mr_builder.path = function(_) {
    if (!arguments.length) return path;
    path = _;
    return mr_builder;
  };

  mr_builder.pathLength = function(_) {
    if (!arguments.length) return path_length;
    path_length = _;
    return mr_builder;
  };

  mr_builder.pathDisplay = function(_) {
    if (!arguments.length) return path_display;
    path_display = _;
    return mr_builder;
  };

  mr_builder.goal = function(_) {
    if (!arguments.length) return goal;
    goal = _;
    return mr_builder;
  };

  mr_builder.goalDisplay = function(_) {
    if (!arguments.length) return goal_display;
    goal_display = _;
    return mr_builder;
  };

  mr_builder.magnitude = function(_) {
    if (!arguments.length) return magnitude;
    magnitude = _;
    return mr_builder;
  };

  var magnitude_measures = [];
  mr_builder.addMagnitudeMeasure = function(_) {
    magnitude_measures.push(_);
    return mr_builder;
  };

  mr_builder.magnitudeCountDistinct = function(_) {
    if (!arguments.length) return magnitude_count_distinct;
    magnitude_count_distinct = _;
    return mr_builder;
  };

  mr_builder.magnitudeAlias = function(_) {
    if (!arguments.length) return magnitude_alias;
    magnitude_alias = _;
    return mr_builder;
  };

  mr_builder.topk = function(_) {
    if (!arguments.length) return topk;
    topk = _;
    return mr_builder;
  };

  mr_builder.addProjection = function(_) {
    projections.push(_);
    return mr_builder;
  };

  mr_builder.projections = function(_) {
    if (!arguments.length) return projections;
    projections = _;
    return mr_builder;
  };

  mr_builder.is_funnel = function(_) {
    if (!arguments.length) return is_funnel;
    is_funnel = _;
    return mr_builder;
  };

  mr_builder.addPath = function(_) {
    paths.push(_);
    return mr_builder;
  };

  mr_builder.paths = function(_) {
    if (!arguments.length) return paths;
    paths = _;
    return mr_builder;
  };

  mr_builder.addFunnelPath = function(_) {
    funnel_paths.push(_);
    return mr_builder;
  };

  mr_builder.funnelPaths = function(_) {
    if (!arguments.length) return funnel_paths;
    funnel_paths = _;
    return mr_builder;
  };

  mr_builder.addPartitionBy = function(_) {
    partition_bys.push(_);
    return mr_builder;
  };

  mr_builder.partitionBys = function(_) {
    if (!arguments.length) return partition_bys;
    partition_bys = _;
    return mr_builder;
  };

  mr_builder.addOrderBy = function(_) {
    orderby.push(_);
    return mr_builder;
  };

  mr_builder.orderBys = function(_) {
    if (!arguments.length) return orderby;
    orderby = _;
    return mr_builder;
  };

  mr_builder.addOuterFilter = function(_) {
    outerFilters.push(_);
    return mr_builder;
  };

  mr_builder.outerFilters = function(_) {
    if (!arguments.length) return outerFilters;
    outerFilters = _;
    return mr_builder;
  };

  mr_builder.sqlPrefix = function(sql) {
    sql.push("SELECT");
    projections.forEach(function(d) {
      sql.push(d);
      sql.push(",");
    });
    if (magnitude_count_distinct.length > 0) {
      sql.push("count(distinct");
      magnitude_count_distinct.forEach(function(d, i) {
        sql.push("mr_tmp_" + _internal_tableid + "." + d);
        if (i < magnitude_count_distinct.length - 1) {
          sql.push(",");
        }
      });
      sql.push(")");
    } else {
      sql.push(magnitude);
    }
    if (magnitude_alias) {
      sql.push('AS');
      sql.push(magnitude_alias);
    }
    sql.push("FROM MATCH_RECOGNIZE");
    if (select_builder.isSimpleTable()) {
      sql.push(select_builder.getTableSql());
    } else {
      sql.push("(");
      sql.push(select_builder.getSql());
      sql.push(") INPUT");
    }
    sql.push("(");
    sql.push("PARTITION BY");
    partition_bys.forEach(function(d, i) {
      sql.push(d);
      if (i < partition_bys.length - 1) {
        sql.push(",");
      }
    });
    sql.push("ORDER BY");
    orderby.forEach(function(d, i) {
      sql.push(d);
      if (i < orderby.length - 1) {
        sql.push(",");
      }
    });
    sql.push("MEASURES");
    magnitude_measures.forEach(function(col) {
      if (col.search(/^\d+$/) < 0) {
        sql.push("A@" + col + " as " + col);
        sql.push(",");
      }
    });
  };

  mr_builder.sqlSuffix = function(sql) {
    sql.push(") mr_tmp_" + _internal_tableid);
    sql.push("GROUP BY");
    projections.forEach(function(d, i) {
      sql.push(i + 1);
      if (i < projections.length - 1) {
        sql.push(',');
      }
    });
  };

  mr_builder.getFullFunnelSql = function(sql) {
    this.sqlPrefix(sql);

    magnitude_count_distinct.forEach(function(d) {
      sql.push("A@" + d + " as " + d + ",");
    });

    var first_partition_col = partition_bys[0];
    sql.push('case');
    var curr_id = 'A';
    var path_event;
    paths.forEach(function(path, i) {
      path_event = path.event_element();
      if (i > 0) {
        sql.push("when (" + curr_id + "@" + first_partition_col + " is null)");
        sql.push("then concat_ws(',',");
        var t_curr_id = 'A';
        paths.forEach(function(t_d, t_i) {
          if (t_i < i) {
            sql.push("concat_ws('', '" + funnel_prefix + "', " +
            t_d.event_element().getComputedDisplay(t_curr_id) + ")");
            if (t_i < i - 1) {
              sql.push(",");
            }
            t_curr_id = String.fromCharCode(1 + t_curr_id.charCodeAt(0));
          }
        });
        if (show_exit_events) {
          sql.push(", isnull(LAST(Z" + curr_id + "@pagename), 'NoEvent')");
        }
        if (show_dropoff) {
          sql.push(", 'Drop Off'");
        }
        sql.push(")");
      }
      curr_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
    });
    sql.push("else concat_ws(',',");
    curr_id = 'A';
    paths.forEach(function(path, i) {
      path_event = path.event_element();
      sql.push("concat_ws('', '" + funnel_prefix + "', " +
      path_event.getComputedDisplay(curr_id) + ")");
      if (i < paths.length - 1) {
        sql.push(",");
      }
      curr_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
    });
    sql.push(') end as path,');

    sql.push("case");
    var last_id = String.fromCharCode(paths.length - 1 +
    "A".charCodeAt(0));
    sql.push("when (" + last_id + "@" + first_partition_col + " is null)");
    sql.push("then 2");
    sql.push("else 1 end as arc_path_priority");

    sql.push('ONE_ROW_PER_MATCH');
    sql.push("PATTERN ");
    curr_id = 'A';
    paths.forEach(function(path, i) {
      path_event = path.event_element();
      var next_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
      if (i === 0) {
        sql.push("('^ZA*? A+");
      } else if (i === paths.length - 1) {
        sql.push("Z" + curr_id + "* " + curr_id + "*')");
      } else {
        sql.push("Z" + curr_id + "* " + curr_id + "*");
      }
      curr_id = next_id;
    });
    sql.push("DEFINE");
    curr_id = 'A';
    paths.forEach(function(path, i) {
      path_event = path.event_element();
      sql.push(curr_id + " as " + path_event.filterExpr() + ",");
      sql.push("Z" + curr_id + " as (!(" + path_event.filterExpr() + "))");
      sql.push(",");
      curr_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
    });

    // Remove the trailing comma
    sql.pop();

    this.sqlSuffix(sql);
  };

  mr_builder.getFunnelSql = function(sql) {
    this.sqlPrefix(sql);

    magnitude_count_distinct.forEach(function(d) {
      sql.push("A@" + d + " as " + d + ",");
    });

    sql.push('case');
    var curr_id = 'A';
    funnel_paths.forEach(function(d, i) {
      if (i > 0) {
        sql.push("when (" + curr_id + "@" + path_display + " is null)");
        sql.push("then concat_ws(',',");
        var t_curr_id = 'A';
        funnel_paths.forEach(function(t_d, t_i) {
          if (t_i < i) {
            sql.push("concat_ws('', '" + funnel_prefix + "', " + t_curr_id + "@" +
                path_display + ")");
            if (t_i < i - 1) {
              sql.push(",");
            }
            t_curr_id = String.fromCharCode(1 + t_curr_id.charCodeAt(0));
          }
        });
        if (show_exit_events) {
          sql.push(", isnull(LAST(Z" + curr_id + "@pagename), 'NoEvent')");
        }
        if (show_dropoff) {
          sql.push(", 'Drop Off'");
        }
        sql.push(")");
      }
      curr_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
    });
    sql.push("else concat_ws(',',");
    curr_id = 'A';
    funnel_paths.forEach(function(d, i) {
      sql.push("concat_ws('', '" + funnel_prefix + "', " + curr_id + "@" + path_display + ")");
      if (i < funnel_paths.length - 1) {
        sql.push(",");
      }
      curr_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
    });
    sql.push(') end as path,');

    sql.push("case");
    var last_id = String.fromCharCode(funnel_paths.length - 1 + "A".charCodeAt(0));
    sql.push("when (" + last_id + "@" + path_display + " is null)");
    sql.push("then 2");
    sql.push("else 1 end as arc_path_priority");

    sql.push('ONE_ROW_PER_MATCH');
    sql.push("PATTERN ");
    curr_id = 'A';
    funnel_paths.forEach(function(d, i) {
      var next_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
      if (i === 0) {
        sql.push("('^ZA*? A+");
      } else if (i === funnel_paths.length - 1) {
        sql.push("Z" + curr_id + "* " + curr_id + "*')");
      } else {
        sql.push("Z" + curr_id + "* " + curr_id + "*");
      }
      curr_id = next_id;
    });
    sql.push("DEFINE");
    curr_id = 'A';
    funnel_paths.forEach(function(d, i) {
      sql.push(curr_id + " as " + d + ",");
      sql.push("Z" + curr_id + " as (!(" + d + "))");
      sql.push(",");
      curr_id = String.fromCharCode(1 + curr_id.charCodeAt(0));
    });

    // Remove the trailing comma
    sql.pop();

    this.sqlSuffix(sql);
  };

  mr_builder.getFullFlowSql = function(sql) {
    this.sqlPrefix(sql);
    var defines = ["DEFINE"];
    var patterns = ["PATTERN ('"];
    var measures = ["CONCAT_WS(','"];
    var a_charcode = "A".charCodeAt(0);
    var last_id = String.fromCharCode(paths.length - 1 + "A".charCodeAt(0));
    var path_event;
    paths.forEach(function(path, idx) {
      path_event = path.event_element();
      if (!path_event) {
        return;
      }

      var this_char = String.fromCharCode(idx + a_charcode);
      defines.push(this_char + " as " + path_event.filterExpr());
      var min_count = path.event_min_count();
      if (min_count === undefined) {
        min_count = 1;
      }
      var max_count = path.event_max_count();
      if (max_count === undefined) {
        max_count = 1;
      }
      if ((min_count === 1) && (max_count === 1)) {
        patterns.push(this_char);
      } else {
        patterns.push(this_char + "{" + min_count + "," + max_count + "}");
      }
      if (idx !== paths.length - 1) {
        defines.push(",");
        patterns.push(" ");
      }
      measures.push(",");
      if (path_event.isStringLiteral()) {
        measures.push(path_event.getComputedDisplay(this_char));
      } else {
        measures.push("GROUP_CONCAT(" + path_event.getComputedDisplay(this_char) + ", ',')");
      }
    });
    patterns.push("')");
    measures.push(") as path");
    sql.push(measures.join(" "));
    sql.push('ONE_ROW_PER_MATCH');
    sql.push(patterns.join(""));
    sql.push(defines.join(" "));
    this.sqlSuffix(sql);
  };

  mr_builder.getFlowSql = function(sql) {
    this.sqlPrefix(sql);

    if (goal_display) {
      if (session_start) {
        sql.push("CONCAT_WS(',',S@" + path_display + ",GROUP_CONCAT(A@" + path_display +
        ", ','),B@" + goal_display + ") as path");
      } else {
        sql.push("CONCAT_WS(',',GROUP_CONCAT(A@" + path_display + ", ','),B@" +
            goal_display + ") as path");
      }
    } else {
      if (session_start) {
        sql.push("CONCAT_WS(',',S@" + path_display + ",GROUP_CONCAT(A@" + path_display +
            ", ',')) as path");
      } else {
        sql.push("GROUP_CONCAT(A@" + path_display + ", ',') as path");
      }
    }
    sql.push('ONE_ROW_PER_MATCH');
    if (session_start) {
      if (goal_display) {
        sql.push("PATTERN ('S A{1," + path_length + "} B')");
      } else {
        sql.push("PATTERN ('S A{1," + path_length + "}')");
      }
      sql.push("DEFINE S as " + session_start + ",");
    } else {
      if (goal_display) {
        sql.push("PATTERN ('A{1," + path_length + "} B')");
      } else {
        sql.push("PATTERN ('^A{1," + path_length + "}')");
      }
      sql.push("DEFINE");
    }
    sql.push("A as " + path);
    if (goal_display) {
      sql.push(",");
      sql.push("B as " + goal);
    }

    this.sqlSuffix(sql);
  };

  mr_builder.getSql = function() {
    var sql = [];
    sql.push("SELECT * FROM (");

    if (paths.length > 0) {
      if (is_funnel) {
        this.getFullFunnelSql(sql);
      } else {
        this.getFullFlowSql(sql);
      }
    } else {
      if (funnel_paths.length > 0) {
        this.getFunnelSql(sql);
      } else {
        this.getFlowSql(sql);
      }
    }

    sql.push(") mr_tmp_" + _external_tableid);
    if (outerFilters.length > 0) {
      sql.push("WHERE");
      outerFilters.forEach(function(d, i) {
        sql.push(d);
        if (i < outerFilters.length - 1) {
          sql.push('AND');
        }
      });
    }
    if (topk) {
      sql.push("ORDER BY");
      if (funnel_paths.length > 0) {
        sql.push("arc_path_priority, ");
      }
      sql.push(magnitude_alias);
      sql.push("DESC");
      sql.push("LIMIT");
      sql.push(topk);
    }
    var text = sql.join(' ');
    if (select_builder.dataset()) {
      var info = select_builder.dataset().colnames_from_expr(text);
      text = info.sqltext;
    }
    return text;
  };

  return mr_builder;
};

arc.unionBuilder = function() {
  var sbs = [];

  var union_builder = function() {
  };

  union_builder.dsreq = function() {
    if (sbs.length > 0) {
      return sbs[0].dsreq();
    }
    return undefined;
  };

  union_builder.addSb = function(sb) {
    sbs.push(sb);
    return union_builder;
  };

  union_builder.getSql = function() {
    var sql = [];

    sbs.forEach(function(sb) {
      sql.push(sb.getSql());
    });

    return sql.join(' UNION ALL ');
  };

  return union_builder;
};

arc.selectBuilder = function() {
  var dbname,
      tablename,

  // DJC -- rather than dbname/tablename, should be dataset.  However, we may also need to
  //        issue queries directly against a table to might need to keep these.  Better would
  //        be against dataset only I think.
    dataset,
    distinct,
    projections = [],
    projectionswithalias = [],
    tablenames = [],    // DJC -- Tablename list built up as projections are added
    groups = [],
    filters = [],
    orderby = [],
    having_filters = [],
    limit = null,
    table_alias,
    default_table_alias = 'T',
    with_sbs = [],
    with_tablenames = [],
    join_tables = [],
    join_conditions = [],
    join_types = [];

  var select_builder = function() {
  };

  select_builder.clone = function() {
    var sb = arc.selectBuilder();
    sb.dbname(dbname);
    sb.tablename(tablename);
    sb.dataset(dataset);
    sb.distinct(distinct);
    sb.projections(projections);
    sb.projectionswithalias(projectionswithalias);
    sb.tablenames(tablenames);
    sb.groups(groups);
    sb.filters(filters);
    sb.orderBys(orderby);
    sb.havingFilters(having_filters);
    sb.limit(limit);
    sb.tableAlias(table_alias);
    sb.with_sbs(with_sbs);
    sb.with_tablenames(with_tablenames);
    sb.join_tables(join_tables);
    sb.join_conditions(join_conditions);
    sb.join_types(join_types);
    return sb;
  };

  // DJC -- This seems odd: isSimpleTable() and getTableSql are only used with the mr
  //        builder above and in the same place with the same sort of logic.
  //        Perhaps the mr builder should either have just a table or should use this
  //        this select builder and use the INPUT keyword in that case.
  //        Likewise 'default_table_alias' / 'table_alias' seems odd and perhaps are
  //        relevent to the user of this object rather than internal to this object.
  select_builder.isSimpleTable = function() {
    return (!(projections.length || projectionswithalias.length ||
    groups.length || filters.length || orderby.length ||
    having_filters.length || limit));
  };

  select_builder.getTableSql = function() {
    var sql = [];
    if (!this.isSimpleTable()) {
      sql.push('(');
      sql.push(this.getSql());
      sql.push(')');
      if (table_alias) {
        sql.push(table_alias);
      } else {
        sql.push(default_table_alias);
      }
    } else {
      sql.push(dbname + "." + tablename);
      if (table_alias) {
        sql.push(table_alias);
      }
    }
    return sql.join(' ');
  };

  select_builder.dbname = function(_) {
    if (!arguments.length) return dbname;
    dbname = _;
    return select_builder;
  };

  select_builder.tablename = function(_) {
    if (!arguments.length) return tablename;
    tablename = _;
    return select_builder;
  };

  select_builder.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return select_builder;
  };

  select_builder.tablenames = function(_) {
    if (!arguments.length) return tablenames;
    tablenames = _;
    return select_builder;
  };

  select_builder.distinct = function(_) {
    if (!arguments.length) return distinct;
    distinct = _;
    return select_builder;
  };

  select_builder.baseTable = function(in_dbname, in_tablename) {
    if (!arguments.length) return [dbname, tablename];
    dbname = in_dbname;
    tablename = in_tablename;
    return select_builder;
  };

  select_builder.tableAlias = function(_) {
    if (!arguments.length) return table_alias;
    table_alias = _;
    return select_builder;
  };

  // Add tablenames from expression text
  select_builder.addTablenamesFromExpression = function(exprtext) {
    if (dataset) {
      var info = dataset.colnames_from_expr(exprtext);
      info.columns.forEach(function(column) {
        if (column) {
          var tn = column.table().tablename_noalias();
          if (tn && (tablenames.indexOf(tn) === -1)) {
            tablenames.push(tn);
          }
        }
      });

      return info.sqltext;
    }

    return exprtext;
  };

  select_builder.expression = function(exprtext) {
    if (dataset) {
      var info = dataset.colnames_from_expr(exprtext);
      return info.sqltext;
    }
    return exprtext;
  };

  select_builder.addProjection = function(_) {
    projections.push(_);
    return select_builder;
  };

  select_builder.projections = function(_) {
    if (!arguments.length) return projections;
    projections = _.slice(0);
    return select_builder;
  };

  select_builder.setProjectionAtIndex = function(p, index) {
    projections[index] = p;
    return select_builder;
  };

  select_builder.addProjectionWithAlias = function(projectionname, projectionalias) {
    projectionswithalias.push({
      pn: projectionname,
      pa: projectionalias
    });
    return select_builder;
  };

  select_builder.projectionswithalias = function(_) {
    if (!arguments.length) return projectionswithalias;
    projectionswithalias = _.slice(0);
    return select_builder;
  };

  select_builder.addGroup = function(_) {
    projections.push(_);
    groups.push(_);
    return select_builder;
  };

  select_builder.addGroupWithoutProjection = function(_) {
    groups.push(_);
    return select_builder;
  };

  select_builder.addGroupWithProjectionAlias = function(projectionname, projectionalias) {
    projectionswithalias.push({
      pn: projectionname,
      pa: projectionalias
    });
    groups.push(projectionname);
    return select_builder;
  };

  select_builder.groups = function(_) {
    if (!arguments.length) return groups;
    groups = _.slice(0);
    return select_builder;
  };

  select_builder.filters = function(_) {
    if (!arguments.length) return filters;
    filters = _.slice(0);
    return select_builder;
  };

  select_builder.addFilter = function(_) {
    filters.push(_);
    return select_builder;
  };

  select_builder.addOrderBy = function(_) {
    orderby.push(_);
    return select_builder;
  };

  select_builder.orderBys = function(_) {
    if (!arguments.length) return orderby;
    orderby = _.slice(0);
    return select_builder;
  };

  select_builder.havingFilters = function(_) {
    if (!arguments.length) return having_filters;
    having_filters = _.slice(0);
    return select_builder;
  };

  select_builder.addHavingFilter = function(_) {
    having_filters.push(_);
    return select_builder;
  };

  select_builder.limit = function(_) {
    if (!arguments.length) return limit;
    if (dataset && dataset.dataconnection_type() === 'oracle') {
      filters.push("ROWNUM <= " + limit);
    } else {
      limit = _;
    }
    return select_builder;
  };

  select_builder.addWithClause = function(with_sb, with_tablename) {
    with_sbs.push(with_sb);
    with_tablenames.push(with_tablename);
    return select_builder;
  };

  select_builder.addJoin = function(join_table, join_condition, join_type) {
    join_tables.push(join_table);
    join_conditions.push(join_condition);
    if (join_type) {
      join_types.push(join_type);
    } else {
      join_types.push("join");
    }
    return select_builder;
  };

  select_builder.with_sbs = function(_) {
    if (!arguments.length) return with_sbs;
    with_sbs = _.slice(0);
    return select_builder;
  };

  select_builder.with_tablenames = function(_) {
    if (!arguments.length) return with_tablenames;
    with_tablenames = _.slice(0);
    return select_builder;
  };

  select_builder.join_tables = function(_) {
    if (!arguments.length) return join_tables;
    join_tables = _.slice(0);
    return select_builder;
  };

  select_builder.join_conditions = function(_) {
    if (!arguments.length) return join_conditions;
    join_conditions = _.slice(0);
    return select_builder;
  };

  select_builder.join_types = function(_) {
    if (!arguments.length) return join_types;
    join_types = _.slice(0);
    return select_builder;
  };

  select_builder.getSql = function() {
    var sql = [],
        dataconnection;

    if (limit !== null) {
      if (dataset) {
        let dc_type = dataset.dataconnection_type();
        if (dc_type) {
          dataconnection = arc.dataconnection.types[dc_type]();
        }
      }
    }

    if (with_sbs.length > 0) {
      sql.push('with');
      with_sbs.forEach(function(with_sb, idx) {
        if (idx > 0) {
          sql.push(',');
        }
        sql.push(with_tablenames[idx]);
        sql.push('as');
        sql.push('(');
        sql.push(with_sb.getSql());
        sql.push(')');
      });
    }

    var selinfo;
    sql.push("select");

    if (limit !== null && dataconnection && dataconnection.supportsTopk) {
      sql.push(dataconnection.topk(limit));
    }

    if (distinct) {
      sql.push("distinct");
    }
    if (projections.length === 0) {
      if (projectionswithalias.length === 0) {
        if (dataset) {
          selinfo = dataset.all_columns_select_text();
          sql.push(selinfo[0]);
        } else {
          sql.push("*");
        }
      }
    } else {
      sql.push(projections.join(', '));
    }
    if ((projectionswithalias.length > 0) && (projections.length > 0)) {
      sql.push(",");
    }
    projectionswithalias.forEach(function(d) {
      sql.push(d.pn);
      sql.push("AS");
      sql.push(d.pa);
      sql.push(",");
    });
    if (projectionswithalias.length > 0) {
      // Remove the extra ","
      sql.pop();
    }

    // DJC -- we need to figure out which tables are used here and join as needed.
    //        or could have a preprocessing stage that populates the join section below
    //  -->   Need to understand who uses the join_conditions below
    //  -->   Also need to know if there are any cases where dataset is valid but the
    //        select is for some other purpose (i.e. not against the dataset).
    //  -->   Not sure how the various apps/viz's handle NULL so might need to hold
    //        off on full outer join.
    sql.push("from");

    var dc_type;
    if (dataset) {

      // If tablenames list is empty, we assume all tables in set and do full join.
      // Otherwise we do join of each table in the list (which was built up as we
      // added the projections)

      // DJC -- Note that we could have two tables named which are not directly connected
      // in the schema.  In this, we need to include tables such that there is a fully
      // connected set.  Not sure of best way to do this.

      // Argh! For now, either do one or all

      dc_type = dataset.dataconnection_type();
      if ((projections.length === 0) && (projectionswithalias.length === 0)) {
        sql.push(arc.dataconnection._enquote_tablename(selinfo[1], dc_type));
      } else if (tablenames.length === 1) {
        sql.push(arc.dataconnection._enquote_tablename(
          dataset.get_table(tablenames[0]).tablename_withalias(), dc_type)
        );
      } else {
        sql.push(arc.dataconnection._enquote_tablename(
          dataset.tables()[0].tablename_withalias(), dc_type)
        );
        dataset.tables().slice(1).forEach(function(t) {
          sql.push(t.join_type());
          sql.push(arc.dataconnection._enquote_tablename(
            t.tablename_withalias(), dc_type)
          );
          sql.push("on (");
          t.join_pairs().forEach(function(p, idx) {
            if (idx > 0) {
              sql.push("and");
            }
            sql.push(arc.dataconnection._enquote_tablename(
              t.parent_tablename() + "." + p[0], dc_type) +
              "=" + arc.dataconnection._enquote_tablename(t.tablename() + "." + p[1], dc_type)
            );
          });
          sql.push(")");
        });
      }
    } else {
      if (dbname) {
        sql.push(arc.dataconnection._enquote_tablename(
          dbname + "." + tablename, dc_type)
        );
      } else if (tablename) {
        sql.push(arc.dataconnection._enquote_tablename(tablename, dc_type));
      }
    }

    if (join_tables.length > 0) {
      join_tables.forEach(function(join_table, idx) {
        sql.push(join_types[idx] + " " + join_table);
        sql.push("on");
        sql.push("(" + join_conditions[idx] + ")");
      });
    }
    if (filters.length > 0) {
      sql.push('where');
      var tmp_filters = [];
      tmp_filters.push('(');
      tmp_filters.push(filters.join(') and ('));
      tmp_filters.push(')');
      sql.push(tmp_filters.join(''));
    }
    if (groups.length > 0) {
      sql.push('group by');
      sql.push(groups.join(', '));
    }
    if (having_filters.length > 0) {
      sql.push('having');
      var tmp_having_filters = [];
      tmp_having_filters.push('(');
      tmp_having_filters.push(having_filters.join(') and ('));
      tmp_having_filters.push(')');
      sql.push(tmp_having_filters.join(''));
    }
    if (orderby.length > 0) {
      sql.push('order by');
      sql.push(orderby.join(', '));
    }

    if (limit !== null) {
      if (!dataconnection || dataconnection.supportsLimit) {
        sql.push('limit');
        sql.push(limit);
      }
    }

    return sql.join(' ');
  };

  return select_builder;
};

arc.data_requests.columnValues = function(
  rb,
  dataset,
  columnOrExpression,
  count,
  forDistinct,
  order,
  isExpression = false
  ) {
  if (!rb) {
    rb = arc.requestBuilder();
  }

  rb.datasetId(dataset.dataset_id());
  rb.limit(count);

  // This function is used to grab a few column values so we can find the format
  // of the data. Thus, just add an aggregate instead of a dimension for performance.
  // If we need unique column values, use this function to get the request builder,
  // and add distinct to the query.

  // For solr, to just quickly grab column values, add a single dimension to add
  // the field to the fl parameter (i.e. rows=count&fl=filed_name). If an aggregate,
  // is used instead, it expects to do faceting which is slower.

  const columnString = isExpression ?
    columnOrExpression :
    `${arc.dataconnection.types.brackets(columnOrExpression)} as '${columnOrExpression}'`;

  let dctype = dataset.dataconnection_type();

  if (forDistinct && dctype === 'teradata') {
    rb.addDimension(columnString, order);
  } else if (dctype === 'ksql') {
    rb.addDimension(columnString, order);
  } else if (forDistinct && dctype === 'elasticsearch') {
    rb.addDimension(columnString, order);
  } else if (dctype === 'solr') {
    rb.addDimension(columnString, order);
    rb.requestType('SOLR');
  } else {
    rb.addDimension(columnString, order);
  }

  return rb;
};

arc.data_requests.distinctColumnValues = function(
  rb,
  dataset,
  colname,
  count,
  coltype,
  alphabeticalOrder,
  isExpression = false,
  improveHive = false,
  ) {
  if (!rb) {
    rb = arc.requestBuilder();
  }

  rb = arc.data_requests.columnValues(
    rb, dataset, colname, count, true, alphabeticalOrder, isExpression
  );
  let dctype = dataset.dataconnection_type();

    switch (dctype) {
      case 'ksql':
        rb.addAggregate('count(*)');
        break;
      case 'elasticsearch':
        rb.addAggregate(`count(*) as 'Record Count'`);
        break;
      case 'solr':
         // If solr and not a timestamp, add an aggregate so we do faceting and
        // get unique values. Tim TODO: Figure out how to get unique time values
        // without overloading solr. Right now, there will be duplicate timestamps.
        if (coltype.toLowerCase() !== 'timestamp') {
          rb.addAggregate(`sum(1) as 'Record Count'`);
        }

        break;
      case 'hive':
      case 'hyve':
        if (improveHive) {
          rb.addAggregate(`sum(1) as 'Record Count'`);
        } else {
          rb.distinctON();
        }

        break;
      case 'teradata':
        rb.addAggregate(`sum(1) as 'Record Count'`);
        break;
      default:
        rb.distinctON();
    }

  return rb;
};

arc.requestBuilder = function() {
  var dataset,
    query,
    report,
    dsreq = {
      version: 1,
      highlighting: false,
      type: 'SQL',
      limit: 5000, // default limit
      time: [],
      session: [],
      dimensions: [],
      aggregates: [],
      partitions: [],

      // filters typically go in the where clause of generated
      // SQL statements
      filters: [],

      // outerFilters model the base SQL statement as a subselect
      // and then run as where clauses on the subselect that results.
      // It is used for cases where the having clause does not work
      // because of limitations with having syntax in many cases like
      // window functions
      outerFilters: [],

      // having predicates typically go in the having clause of generated
      // SQL statements
      having: [],
      global_segments: [],
      events: [],
      distinct: false,
      derived_data: [],
      solr_query: [],
      autotime_override: '',
      ksqlStreamPosition: '',
    };

  var request_builder = function() {
  };

  request_builder.defaultLimit = function() {
    return 5000;
  };

  request_builder.cloneRequestBuilder = function() {
    var new_rb = arc.requestBuilder();
    var dsreq_copy = jQuery.extend(true, {}, dsreq);
    new_rb.dsreq(dsreq_copy);
    return new_rb;
  };

  // Add tablenames from expression text
  request_builder.expression = function(exprtext) {
    if (dataset) {
      var info = dataset.colnames_from_expr(exprtext);
      return info.sqltext;
    }
    return exprtext;
  };

  request_builder.dsreq = function(_) {
    var deriveddata, list;
    if (!arguments.length) {
      // Don't add derived data more than once
      if (report && dsreq.derived_data.length === 0) {
        deriveddata = report.derivedData();
        if (deriveddata) {
          list = deriveddata.derivedDataList();
          list.forEach(function(ddata) {
            request_builder.deriveddata(ddata);
          });
        }
      }

      return dsreq;
    }

    dsreq = _;
    return request_builder;
  };

  request_builder.deriveddata = function(_) {
    dsreq.derived_data.push(_);
    return request_builder;
  };

  request_builder.deriveddataList = function(_) {
    dsreq.derived_data = _;
    return request_builder;
  };

  request_builder.report = function(_) {
    if (!arguments.length) return report;
    report = _;
    return request_builder;
  };

  request_builder.datasetId = function(_) {
    if (!arguments.length) return dsreq.dataset_id;
    dsreq.dataset_id = _;
    return request_builder;
  };

  request_builder.time = function(_) {
     if (!arguments.length) return dsreq.time;
    dsreq.time.push(_);
    return request_builder;
  };

  request_builder.session = function(_) {
     if (!arguments.length) return dsreq.session;
    dsreq.session.push(_);
    return request_builder;
  };

  request_builder.addDimension = function(expr, order) {
    var d = {type: 'SIMPLE', expr: expr};
    if (order) {
      d.order = {asc: order.ascending(), pri: order.priority()};
    }
    dsreq.dimensions.push(d);
    return request_builder;
  };

  request_builder.dimensions = function(_) {
    if (!arguments.length) return dsreq.dimensions;
    dsreq.dimensions = _.slice(0);
    return request_builder;
  };

  request_builder.addPartition = function(expr, order) {
    var d = {type: 'SIMPLE', expr: expr};
    if (order) {
      d.order = {asc: order.ascending(), pri: order.priority()};
    }
    dsreq.partitions.push(d);
    return request_builder;
  };

  request_builder.partitions = function(_) {
     if (!arguments.length) return dsreq.partitions;
    dsreq.partitions = _.slice(0);
    return request_builder;
  };

  request_builder.segments = function(segments) {
    var d = {type: 'SEGMENT', segments: []};

    segments.forEach(function(seg) {
      d.segments.push({seg_id: seg.id, negate: seg.negate});
    });
    dsreq.dimensions.push(d);
    return request_builder;
  };

  request_builder.globalSegments = function(segments) {
    if (!arguments.length) return dsreq.global_segments;

    if (segments.length === 0) {
      return request_builder;
    }

    dsreq.global_segments.push({
      type: 'SEGMENT',
      segments: segments.map(function(seg) {
        return {
          seg_id: seg.id,
          negate: seg.negate,
        };
      }),
    });
    return request_builder;
  };

  request_builder.addAggregate = function(expr, order) {
    var a = {expr: expr};
    if (order) {
      a.order = {asc: order.ascending(), pri: order.priority()};
    }
    dsreq.aggregates.push(a);
    return request_builder;
  };

  request_builder.aggregates = function(_) {
    if (!arguments.length) return dsreq.aggregates;
    dsreq.aggregates = _.slice(0);
    return request_builder;
  };

  request_builder.replaceFilters = function(filters) {
    dsreq.filters = filters;
    return request_builder;
  };

  request_builder.addFilter = function(expr) {
    dsreq.filters.push(expr);
    return request_builder;
  };

  request_builder.filters = function(expr) {
    if (!arguments.length) return dsreq.filters;
    dsreq.filters.push(expr);
    return request_builder;
  };

  request_builder.addOuterFilter = function(expr) {
    dsreq.outerFilters.push(expr);
    return request_builder;
  };

  request_builder.resetHaving = function() {
    dsreq.having = [];
    return request_builder;
  };

  request_builder.having = function(expr) {
    if (!arguments.length) return dsreq.having;
    dsreq.having.push(expr);
    return request_builder;
  };

  request_builder.limit = function(limit) {
    if (!arguments.length) return dsreq.limit;
    dsreq.limit = limit;
    return request_builder;
  };

  request_builder.nulls_order = function(_) {
    if (!arguments.length) return dsreq.nulls_order;

    // don't set nulls_order if undefined
    if (!_) return request_builder;

    // set to First/Last or Default
    if (["First", "Last"].indexOf(_) < 0) {
      dsreq.nulls_order = "Default";
    } else {
      dsreq.nulls_order = _;
    }
    return request_builder;
  };

  request_builder.events = function(type, events) {
    if (!arguments.length) return dsreq.events;
    var d = {type: type, events: []};
    events.forEach(function(eventId) {
      d.events.push(eventId);
    });
    dsreq.events.push(d);
    return request_builder;
  };

  request_builder.requestType = function(_) {
    if (!arguments.length) return dsreq.type;
    dsreq.type = _;
    return request_builder;
  };

  request_builder.highlighting = function(_) {
    if (!arguments.length) return dsreq.highlighting;
    dsreq.highlighting = _;
    return request_builder;
  };

  request_builder.connectionId = function(_) {
    if (!arguments.length) return dsreq.connection_id;
    dsreq.connection_id = _;
    return request_builder;
  };

  request_builder.dbname = function(_) {
    if (!arguments.length) return dsreq.dbname;
    dsreq.dbname = _;
    return request_builder;
  };

  request_builder.tableName = function(_) {
    if (!arguments.length) return dsreq.table_name;
    dsreq.table_name = _;
    return request_builder;
  };

  request_builder.viewLocation = function(_) {
    if (!arguments.length) return dsreq.view_location;
    dsreq.view_location = _;
    return request_builder;
  };

  request_builder.useAssocLV = function(_) {
    if (!arguments.length) return dsreq.use_assoc_lv;
    dsreq.use_assoc_lv = _;
    return request_builder;
  };

  request_builder.tableFormat = function(_) {
    if (!arguments.length) return dsreq.table_format;
    dsreq.table_format = _;
    return request_builder;
  };

  request_builder.validate = function(_) {
    if (!arguments.length) return dsreq.validate;
    dsreq.validate = _;
    return request_builder;
  };

  request_builder.phase = function(_) {
    if (!arguments.length) return dsreq.phase;
    dsreq.phase = _;
    return request_builder;
  };

  request_builder.distinctON = function() {
    dsreq.distinct = true;
    return request_builder;
  };

  request_builder.distinctOFF = function() {
    dsreq.distinct = false;
    return request_builder;
  };

  request_builder.query = function(_) {
    if (!arguments.length) return query;
    query = _;
    dsreq.query = _;
    return request_builder;
  };

  request_builder.addSolrQuery = function(_) {
    if (!arguments.length) return dsreq.solr_query;

    let fullSearch = '*:*';

    if (_ === fullSearch) {
      // Doesn't make sense to add fullSearch to an existing search query
      if (dsreq.solr_query.length > 0) {
        return request_builder;
      }
    } else {
      // Doesn't make sense to add a normal query to an existing fullSearch query
      if ((dsreq.solr_query.length === 1) && (dsreq.solr_query[0] === fullSearch)) {
        dsreq.solr_query = [];
      }
    }

    dsreq.solr_query.push(_);
    return request_builder;
  };

  request_builder.autotime_override = function(_) {
    if (!arguments.length) return dsreq.autotime_override;
    dsreq.autotime_override = _;
    return request_builder;
  };

  request_builder.ksqlStreamPosition = function(_) {
    if (!arguments.length) return dsreq.ksqlStreamPosition;
    dsreq.ksqlStreamPosition = _;
    return request_builder;
  };

  request_builder.materialized = function(_) {
    if (!arguments.length) return dsreq.materialized;
    dsreq.materialized = _;
    return request_builder;
  };

  request_builder.materializedType = function(_) {
    if (!arguments.length) return dsreq.materializedType;
    dsreq.materializedType = _;
    return request_builder;
  };

  return request_builder;
};

}());
