window.arc = window.arc || {};

(function() {
'use strict';

arc.ext = arc.ext || {};

/*
 * Incremental queries are enabled in the following cases
 *  dataset partitions are included in the set of group-by dimensions
 *  OR
 *  all aggregates used are sum or count, AND there are not having clauses
 *
 * If partial results are used, then column sorts and row limits
 *  are implemented in js after the rows are concatenated.
 */

arc.ext.extIncarnation = 1;

arc.ext.incrementalQueries = function() {
  var f = function() { return 'extension'; };

  var chart_div;
  f.incarnation = arc.ext.extIncarnation++;

  f.version = '1';

  f.settings = function() {
    return [
      {
        id: 'partitionsAtATime',
        defaultValue: '1',
      },
      {
        id: 'initialPartitionsAtATime',
        defaultValue: '1',
      },
      {
        id: 'parallelism',
        defaultValue: '5',
      },
    ];
  };

  function validVersion(myVersion) {
    var $c = $('#' + arcapi.chartId());
    var storedVersion = $c.attr('data-microquery-version');
    return ((Number(storedVersion)) === myVersion);
  }

  function getDsPartitions() {
    var dsPartitions = [];
    var lib = arcapi.unsupportedDriver;
    lib.dataset().tables().forEach(function(t) {
      t.partitions().forEach(function(p) {
        dsPartitions.push(p);
      });
    });

    return dsPartitions;
  }

  var storedRequest,
      config = {};

  f.afterDataRequest = function(data_request) {
    if (!data_request) {
      return data_request;
    }

    var rb = data_request.rb();
    if (!rb) {
      return data_request;
    }

    storedRequest = undefined;
    config = {
      queryPartitionCols: [],
      queryLimit: undefined,
      orderCols: [],
      sortColumnCount: 0,
      sortIndices: [],
      sortAsc: [],
      sortRange: [],
      addMeasures: false,
      dimColumns: [],
      aggColumns: [],
    };

    var partitions = getDsPartitions();
    if (partitions.length === 0) {
      return data_request;
    }

    var dataRequestDimensions = {};
    config.queryLimit = rb.limit();

    var dims = rb.dimensions();
    dims.forEach(function(col, i) {
      if (col.order) {
        config.orderCols.push({
          order: col.order,
          colIndex: i,
        });
      }
    });

    var aggs = rb.aggregates();
    var agg_re = /^\s*(sum|count)\s*\(/i;
    var aggMatches = 0;
    var containsHavingClauses = (rb.having().length > 0);
    aggs.forEach(function(col, i) {
      if (!containsHavingClauses && agg_re.exec(col.expr)) {
        ++aggMatches;
      }

      if (col.order) {
        config.orderCols.push({
          order: col.order,
          colIndex: dims.length + i,
        });
      }
    });

    // If there are having clauses in play, do not run partial queries based
    // on aggregates
    if (containsHavingClauses) {
      aggMatches = -1;
    }

    config.orderCols.sort(function(a, b) {
      return (a.order.pri - b.order.pri);
    });

    config.orderCols.forEach(function(o) {
      config.sortColumnCount++;
      config.sortIndices[o.colIndex] = true;
      config.sortAsc[o.colIndex] = o.order.asc;
      config.sortRange.push(o.colIndex);
    });

    if ((config.queryLimit !== rb.defaultLimit()) && config.sortColumnCount) {
      return data_request;
    }

    var dim_re = /^\[([^\[\]]+)\]/;
    dims.forEach(function(d) {
      var m = dim_re.exec(d.expr);
      if (m) {
        dataRequestDimensions[m[1]] = true;
      }
    });

    partitions.forEach(function(p) {
      if (p in dataRequestDimensions) {
        config.queryPartitionCols.push(p);
      }
    });

    if (config.queryPartitionCols.length === 0) {
      if ((aggMatches > 0) && (aggMatches === aggs.length)) {
        config.queryPartitionCols = partitions.slice(0);
        config.addMeasures = true;
        config.dimColumns = dims;
        config.aggColumns = aggs;
      } else {
        return data_request;
      }
    }

    storedRequest = data_request;

    var ignore = arc.data.request()
                    .request_type('ignore');
    return ignore;
  };

  function getDistinctValues(lib, base_fields, cb) {
    var data_request = arc.data.request();

    var sb = lib.getSelectBuilder(),
        rb = lib.getRequestBuilder()
                .addAggregate(arc.dataconnection.types.brackets(base_fields[0]))
                .distinctON();

    sb.distinct(true);
    sb.addProjection('[' + base_fields[0] + ']');
    sb = lib.finalizeSql(sb);

    var col = arc.data.column()
                      .colname(base_fields[0])
                      .colsource('base_shelf')
                      .coltype('string')
                      .colproperty(arc.report.columnTypeEnum.DIMENSION);
    data_request.add_column(col);

    data_request.sql(sb.getSql());
    data_request.dsreq(rb.dsreq());
    data_request.finalize();

    var dsreq = data_request.dsreq();
    lib.runSqlCmd('', function(o) {
      cb(o.rows);
    }, dsreq);

    return data_request;
  }

  function getData(lib, partitionCols, partitionVals, cb) {
    var rb = storedRequest.rb().cloneRequestBuilder();
    rb.filters('[' + partitionCols[0] + '] in ("' + partitionVals.join('", "') + '")');

    storedRequest.dsreq(rb.dsreq());
    lib.runSqlCmd('', function(o) {
      cb(o);
    }, storedRequest.dsreq());
  }

  let progressArc = d3.svg.arc()
                      .innerRadius(7)
                      .outerRadius(10)
                      .startAngle(0);

  function findProgressContainer() {
    let $c = $('#' + chart_div);
    let $p = $c.closest('.arc-viz-container')
               .find('.progress-container');
    if ($p.length > 0) {
      return $p;
    }

    let $sm = $c.closest('.super-modal');
    if ($sm.length > 0) {
      return $sm.find('.progress-container');
    }

    return $('body .subnav-report .progress-container');
  }

  function drawBackgroundPath() {
    let $progressContainer = findProgressContainer();

    d3.selectAll($progressContainer.first())
      .selectAll('path.all')
      .attr('d', progressArc({
        endAngle: 2 * Math.PI,
      }));
  }

  function updateProgress() {
    let $progressContainer = findProgressContainer();
    let path = d3.selectAll($progressContainer.first())
                 .selectAll('path.progress');

    if ($progressContainer.length === 0) {
      return;
    }

    if (completedPartitions === totalPartitions) {
      $progressContainer.addClass('arc-invisible');

      // Cancel ongoing transition
      path.transition();
      return;
    }

    drawBackgroundPath();

    function arcTransition() {
      let last = lastCompletedPartitions,
          curr = completedPartitions,
          total = totalPartitions;

      return function(t) {
        let diff = (curr - last) * t;
        lastCompletedPartitions += diff;
        lastCompletedPartitions = Math.min(lastCompletedPartitions, completedPartitions);
        let angle = (lastCompletedPartitions / total) * (2 * Math.PI);
        if (angle < 0.5) {
          angle = 0.5;
        }

        if (lastCompletedPartitions === totalPartitions) {
          $progressContainer.addClass('arc-invisible');
        }

        return progressArc({
          endAngle: angle,
        });
      };
    }

    path.transition()
        .duration(500)
        .attrTween('d', arcTransition);

    $progressContainer.removeClass('arc-invisible');
  }

  var currPartition,
      currValues,
      lastCompletedPartitions,
      completedPartitions,
      totalPartitions;

  function dataFetcher(myVersion, lib, partitionCols, partitionsAtATime) {
    if (!validVersion(myVersion)) {
      console.log('newer refresh, exiting');
      return;
    }

    if (currPartition >= currValues.length) {
      return;
    }

    let vals = [];
    if (partitionsAtATime === undefined) {
      partitionsAtATime = Number(arcapi.getSetting('partitionsAtATime'));
    }

    while (currPartition < currValues.length) {
      vals.push(currValues[currPartition]);
      ++currPartition;
      if (vals.length >= partitionsAtATime) {
        break;
      }
    }

    getData(lib, partitionCols, vals, function(v) {
      completedPartitions += vals.length;
      updateProgress();
      drawIt(myVersion, lib, v);
      dataFetcher(myVersion, lib, partitionCols);
    });
  }

  function processResults() {
    if (config.addMeasures) {
      var l = config.dimColumns.length;
      var rowsH = {};
      var newRows = [];
      result.rows.forEach(function(r) {
        var d = r.slice(0, l);
        if (d in rowsH) {
          var currRow = rowsH[d];
          config.aggColumns.forEach(function(c, i) {
            currRow[l + i] = currRow[l + i] + (Number(r[l + i]));
          });
        } else {
          rowsH[d] = r;
          config.aggColumns.forEach(function(c, i) {
            r[l + i] = Number(r[l + i]);
          });

          newRows.push(r);
        }
      });

      result.rows = newRows;
    }

    if (config.orderCols.length > 0) {
      arc.misc.sort_arr(result.rows, config.sortAsc, config.sortIndices, config.sortRange);
    }

    if (config.queryLimit) {
      if (result.rows.length > config.queryLimit) {
        result.rows = result.rows.slice(0, config.queryLimit);
      }
    }
  }

  var result;
  function drawIt(myVersion, lib, _result) {
    if (!validVersion(myVersion)) {
      console.log('newer refresh, exiting');
      return;
    }

    if (result === undefined) {
      result = _result;
    } else {
      _result.rows.forEach(function(row) {
        result.rows.push(row);
      });
    }

    processResults();
    lib.populateFromSqlOutput(result);
    lib.dataResult(lib.resultFromRows(result.rows, storedRequest.columns()));
    lib.viz().viz_drawVisual(lib.chart_div());
  }

  function startFetching(myVersion) {
    var partitions = config.queryPartitionCols.slice(0);
    if (partitions.length === 0) {
      return;
    }

    var lib = arcapi.unsupportedDriver;
    chart_div = lib.chart_div();
    var parallelism = Number(arcapi.getSetting('parallelism'));

    var initialPartitionsAtATime = Number(arcapi.getSetting('initialPartitionsAtATime'));

    var p = 0;
    while (p < parallelism) {
      ++p;
      dataFetcher(myVersion, lib, partitions, initialPartitionsAtATime);
    }
  }

  f.afterDraw = function() {
    if (!storedRequest) {
      return;
    }

    result = undefined;

    var myVersion = arc.ext.extIncarnation++;
    $('#' + arcapi.chartId()).attr('data-microquery-version', myVersion);
    var partitions = config.queryPartitionCols.slice(0);
    if (partitions.length === 0) {
      return;
    }

    var lib = arcapi.unsupportedDriver;
    getDistinctValues(lib, partitions, function(values) {
      if (!validVersion(myVersion)) {
        console.log('newer refresh, exiting');
        return;
      }

      if (values && values.length > 0) {
        currPartition = 0;
        currValues = values;
        totalPartitions = currValues.length;
        lastCompletedPartitions = 0;
        completedPartitions = 0;
        startFetching(myVersion);
      }
    });
  };

  return f;
};

}());
