window.arc = window.arc || {};

(function() {
'use strict';

arc.ext = arc.ext || {};

/*
 * Incremental queries are enabled in the following cases
 *  dataset partitions are included in the set of group-by dimensions
 *  OR
 *  all aggregates used are sum or count, AND there are not having clauses
 *
 * If partial results are used, then column sorts and row limits
 *  are implemented in js after the rows are concatenated.
 */

arc.ext.extIncarnation = 1;

arc.ext.STREAMING_POLL_TIME = 5;

arc.ext.streamingQueryRunner = function(_config) {
  var f = function() { return 'arc.ext.streamingQueryRunner'; };

  let {
    lib,
  } = _config;

  let drawCount = 0,
      config = {},
      columnDeduper = arc.misc.columnDeduper(),
      inRequestBuilder,
      columns = [],
      colnames,
      coltypes,
      result,
      dataCb;

  f.columns = function(_) {
    if (!arguments.length) return columns;
    columns = _;

    colnames = [];
    coltypes = [];
    if (columns.length === 0) {
      lib.dataset().tables().forEach(function(t) {
        t.columns().forEach(function(col) {
          colnames.push(col.display_name());
          coltypes.push(col.attr('type'));
        });
      });
    } else {
      columns.forEach(function(col) {
        colnames.push(col.colname());
        coltypes.push(col.coltype());
      });
    }

    return f;
  };

  f.colnames = function(_) {
    if (!arguments.length) return colnames;
    colnames = _;
    return f;
  };

  f.coltypes = function(_) {
    if (!arguments.length) return coltypes;
    coltypes = _;
    return f;
  };

  f.dataCb = function(_) {
    if (!arguments.length) return dataCb;
    dataCb = _;
    return f;
  };

  function validVersion(myVersion) {
    // Note that lib.chart_div() is always run live so we can handle
    // the case where a visual starts out in a dashboard and then move
    // into a modal, and there are potentially 2 extensions competing
    // to draw. If we cache the lib.chart_div() string, they are both
    // checking in different locations and both keep running. If we
    // do not cache, one of them quits.
    var $c = $('#' + lib.chart_div());
    var storedVersion = $c.attr('data-microquery-version');
    return ((Number(storedVersion)) === myVersion);
  }

  f.setRequestBuilder = function(rb) {
    inRequestBuilder = rb;
    config = {
      queryLimit: undefined,
      orderCols: [],
      sortColumnCount: 0,
      sortIndices: [],
      sortAsc: [],
      sortRange: [],
    };

    // For a table visual, removing the limit causes rb.limit to be
    // undefined. This is to allow no limit tables to be drawn as well
    if (rb.limit) {
      config.queryLimit = rb.limit();
    }

    config.dimensionCount = rb.dimensions().length;
    config.aggregateCount = rb.aggregates().length;

    columnDeduper.initializeWithRB(rb);

    rb.dimensions(columnDeduper.dedupedDims());
    rb.aggregates(columnDeduper.dedupedAggs());
    config.orderCols = columnDeduper.orderCols();

    config.orderCols.sort(function(a, b) {
      return (a.order.pri - b.order.pri);
    });

    config.orderCols.forEach(function(o) {
      config.sortColumnCount++;
      config.sortIndices[o.colIndex] = true;
      config.sortAsc[o.colIndex] = o.order.asc;
      config.sortRange.push(o.colIndex);
    });

    let dsreq = rb.dsreq();

    if (config.aggregateCount) {
      dsreq.limit = undefined;
    }

    dsreq.dimensions.forEach(function(d) {
      if (d.order) {
        d.order = undefined;
      }
    });

    dsreq.aggregates.forEach(function(a) {
      if (a.order) {
        a.order = undefined;
      }
    });

    let remainingAggregates = [];
    dsreq.aggregates.forEach(function(a) {
      if (arc.report.isSupportedAggregate(a.expr, 'ksql')) {
        remainingAggregates.push(a);
      } else {
        dsreq.dimensions.push(a);
      }
    });

    dsreq.aggregates = remainingAggregates;

  };

  function getData(lib, qstate, cb) {
    var rb = inRequestBuilder.cloneRequestBuilder();
    lib.fetchData({
      qstate: qstate,
      dsreq: JSON.stringify(rb.dsreq()),
      successCb: function(o, qstate) {
        cb(o, qstate);
      },

      failCb: function() {
        console.log('error streaming queries');
      },
    });
  }

  function processResults() {
    if (result.rows.length === 0) {
      return;
    }

    if (config.aggregateCount && config.dimensionCount) {
      let dimsSeen = {};
      let newRows = [];
      for (let i = result.rows.length - 1; i >= 0; --i) {
        let row = result.rows[i];
        let dims = row.slice(0, config.dimensionCount);

        if (dims in dimsSeen) {
          continue;
        }

        newRows.push(row);
        dimsSeen[dims] = true;
      }

      result.rows = newRows;
    }

    if (config.orderCols.length > 0) {
      arc.misc.sort_arr(result.rows, config.sortAsc, config.sortIndices, config.sortRange);
    }

    if (config.queryLimit) {
      if (result.rows.length > config.queryLimit) {
        result.rows = result.rows.slice(0, config.queryLimit);
      }
    }
  }

  function drawIt(myVersion, lib, _result) {
    if (!validVersion(myVersion)) {
      console.log('newer refresh, exiting');
      return;
    }

    if (result === undefined) {
      result = _result;
      result.colnames = colnames;
      result.coltypes = coltypes;
    } else {
      _result.rows.forEach(function(row) {
        result.rows.push(row);
      });
    }

    processResults();

    if (drawCount > 0) {
      // If dashboard has been paused, do not redraw
      if (arc.globals.dashboard && !arc.globals.dashboard.isPlaying()) {
        return;
      }

      // Don't redraw automatically in modal mode. Let user look at existing visuals.
      if (lib.inModal()) {
        return;
      }
    }

    if (result.rows.length > 0) {
      ++drawCount;

      // Once we have drawn streaming results, don't show the loadbar again
      lib.enableLoader(false);
    }

    if (dataCb) {
      dataCb({
        result,
      });
    }
  }

  function startFetching(myVersion) {
    let firstTime = true,
        qstate;

    // Enable loadbars for the initial fetch
    lib.enableLoader(true);

    // Note that we are passing in an undefined qstate to the call below
    dataFetcher(myVersion, lib, qstate, firstTime);
  }

  function dataFetcher(myVersion, lib, qstate, firstTime) {
    if (drawCount === 0) {
      if (validVersion(myVersion)) {
        let loader = lib.loader();
        if (loader) {
          loader.reset();
          loader.showProgress();
        }
      }
    }

    setTimeout(function() {
      if (!validVersion(myVersion)) {
        console.log('newer refresh, exiting');
        drawCount = 0;
        return;
      }

      getData(lib, qstate, function(v, qstate) {
        if (qstate.streamConsumed()) {
          return;
        }

        v.rows = columnDeduper.transformRows(v.rows);
        drawIt(myVersion, lib, v);
        dataFetcher(myVersion, lib, qstate);
      });
    }, firstTime ? 0 : arc.ext.STREAMING_POLL_TIME * 1000);
  }

  f.startQuerying = function() {
    if (!inRequestBuilder) {
      return;
    }

    result = undefined;

    var myVersion = arc.ext.extIncarnation++;
    $('#' + lib.chart_div()).attr('data-microquery-version', myVersion);

    startFetching(myVersion);
  };

  return f;
};

arc.ext.streamingQueries = function() {
  var f = function() { return 'arc.ext.streamingQueries'; };

  let lib = arcapi.unsupportedDriver,
      storedRequest;

  let runner = arc.ext.streamingQueryRunner({
    lib,
  });

  runner.dataCb(function(config) {
    let {
      result,
    } = config;

    let columns = storedRequest.columns();

    lib.populateFromSqlOutput(result);
    lib.dataResult(lib.resultFromRows(result.rows, columns));
    lib.drawVisual();
  });

  f.version = '1';

  f.settings = function() {
    return [];
  };

  f.afterDataRequest = function(data_request) {
    if (!data_request) {
      return data_request;
    }

    var rb = data_request.rb();
    if (!rb) {
      return data_request;
    }

    storedRequest = data_request;
    runner.setRequestBuilder(rb);
    runner.columns(storedRequest.columns());

    runner.startQuerying();
    var ignore = arc.data.request()
                    .request_type('ignore');
    return ignore;
  };

  return f;
};

}());
