window.arc = window.arc || {};

(function() {
'use strict';

arc.filter = arc.filter || {};

arc.filter.daterange = function() {
  var setting = arc.visual.setting()
                   .id('daterange_picker')
                   .tab('Date Range')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('DATERANGE_PICKER')
                   .value('');

  var datetimeFormat,
      filterInput,
      data;

  setting.datetimeFormat = function(_) {
    if (!arguments.length) return datetimeFormat;
    datetimeFormat = _;
    return setting;
  };

  setting.filterInput = function(_) {
    if (!arguments.length) return filterInput;
    filterInput = _;
    return setting;
  };

  function valueCb(value) {
    data = {};
    $.extend(data, value);

    var div_id = setting.div_id();
    var $sel = $(`#${div_id}`);
    var $d = $sel.find('.filter-date-range');
    var dp = $d.data('daterangepicker');

    const { min, max } = data;

    try {
      if (dp) {
        if (min && max) {
          dp.element.attr('disabled', true);
        }

        if (min) {
          dp.setStartDate(moment.utc(min));
        }

        dp.element.attr('disabled', false);

        if (max) {
          dp.setEndDate(moment.utc(max));
        }
      }

      // $sel.find('.filter-date-range').val(data.min + ' - ' + data.max);
    } catch (e) {
      // Best-effort at setting daterange values
    }
  }

  function fetch_fn() {
    var data = {};

    var div_id = setting.div_id();
    var $sel = $(`#${div_id}`);

    var $rs = $sel.find('.filter-date-range');
    var v = $rs.val();

    var d = arc.misc.daterangeToMinMax(v, datetimeFormat.fmtString);

    data.min = d.min;
    data.max = d.max;
    data.expr = filterInput.dateRangeFilter(data.min, data.max, datetimeFormat.fmtString);

    return data;
  }

  function paint_fn(d3_sel) {
    var div_id = setting.div_id();
    d3_sel.append('div')
          .attr('id', div_id);

    var $sel = $(`#${div_id}`);

    $sel.append($('#filter-daterange-template').html());
  }

  setting.inputValues = function(rows) {
    if (rows.length === 1) {
      var div_id = setting.div_id();
      var $sel = $(`#${div_id}`);

      var min = rows[0][0],
          max = rows[0][1];

     try {
        var minDate = arc.misc.dateToMoment(min);
        var maxDate = arc.misc.dateToMoment(max);
        var $d = $sel.find('.filter-date-range');
        var opts = {
          timePickerIncrement: 1,
          applyClass: 'btn-primary',
          ranges: arc.misc.defaultRanges(),
          format: datetimeFormat.fmtString,
          timePicker: datetimeFormat.is_datetime,
          startDate: minDate,
          endDate: maxDate,
        };

        if (data) {
          try {
            let startDate, endDate;
            if (data.min) {
              opts.startDate = arc.misc.dateToMoment(data.min);
              startDate = opts.startDate.format(datetimeFormat.fmtString);
            }

            if (data.max) {
              opts.endDate = arc.misc.dateToMoment(data.max);
              endDate = opts.endDate.format(datetimeFormat.fmtString);
            }

            $sel.find('.filter-date-range').val(`${startDate} - ${endDate}`);
          } catch (e) {
            // Best-effort at setting dates
          }
        }

        $sel.find('.filter-date-range').daterangepicker(opts);
      } catch (e) {
        // Invalid dates
        console.log(e);
      }
    }
  };

  setting.loading = function(l) {
    var div_id = setting.div_id();
    $(`#${div_id}`).find('.arc-spinner').toggleClass('arc-invisible', !l);
    return setting;
  };

  setting.valueCb(valueCb)
         .paint_fn(paint_fn)
         .fetch_fn(fetch_fn);

  return setting;
};

}());
