window.arc = window.arc || {};

(function() {
'use strict';

arc.filter = arc.filter || {};

arc.filter.input = function() {
  var input = function() {
    return 'arc.filter.input';
  };

  var shelf_element,
      d3_elem,
      coltype,
      lib;

  var filterModal;

  input.coltype = function() {
    return coltype;
  };

  input.lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    return input;
  };

  input.shelf_element = function(_) {
    if (!arguments.length) return shelf_element;
    shelf_element = _;
    return input;
  };

  input.isDatasetSegment = function() {
    return shelf_element.isDatasetSegment();
  };

  input.d3_elem = function(_) {
    if (!arguments.length) return d3_elem;
    d3_elem = _;
    return input;
  };

  function getColname() {
    var colname = shelf_element.filter_column();
    if (colname) {
      return colname;
    }

    return shelf_element.dataset_colname();
  }

  var valuePicker = arc.filter.valuePicker();
  var patternPicker = arc.filter.patternPicker();
  var uploadPicker = arc.filter.uploadPicker();
  var rangePicker = arc.filter.range();
  var dateRangePicker = arc.filter.daterange();
  var relativeDateRangePicker = arc.filter.relativeDateRange();
  const expressionEditor = arc.filter.expression();

  function loadFilter(filterSetting, rb, cb) {
    if (filterSetting) {
      filterSetting.loading(true);
    }

    function fail() {
      if (filterSetting) {
        filterSetting.loading(false);
      }

      filterModal.hide();
    }

    function success(data) {
      if (filterSetting) {
        filterSetting.loading(false);
      }

      cb(data);
    }

    function errorReportingCb(_qstate, msg, error) {
      arc.report.alertMessage(
        "Couldn't load filter data",
        `${msg}: ${error}`,
        { alertContainerSelector: '#filter-modal' }
      );
    }

    arc.data.runQueryParams({
      rb,
      successCb: success,
      failCb: fail,
      lib,
      errorReportingCb

      // Note that we are not using 'loader' here. We do not want to
      // use the loader progress reporting from the 'lib' object
    });
  }

  function minmax() {
    var colname = getColname();
    var rb = input.lib().viz().getRequestBuilder();
    rb.datasetId(shelf_element.dataset().dataset_id());

    if (shelf_element.viz_lib().isSolr()) {
      rb.requestType('SOLR');
      rb.addSolrQuery(`columnstats ${colname}`);
      return rb;
    }

    rb.addAggregate(`min([${colname}]) as 'minVal'`);
    rb.addAggregate(`max([${colname}]) as 'maxVal'`);
    return rb;
  }

  function segmentsToDataRows() {
    return lib.dataset_segments().segmentDataRows();
  }

  input.activate = function() {
    if (arc.misc.modalShown($('#filter-modal'))) {
      return;
    }

    var colname = getColname();

    /* No automatic modal for aggregates */
    if (shelf_element.expressionIsAggregate()) {
      return;
    }

    filterModal = arc.report.settingsModal()
                     .large(true)
                     .settings_modal_id('filter-modal')
                     .settings_modal_title(`Filter for ${colname}`);

    filterModal.drawModal()
               .showEmpty();
    showFilters();

    function showFilters() {
      var allSettings = [
        valuePicker,
        patternPicker,
        rangePicker,
        dateRangePicker,
        relativeDateRangePicker,
        uploadPicker,
        expressionEditor,
      ];

      function withData(data) {
        // From the data and the column type, figure out which of the
        // filter types apply
        coltype = data.coltypes[0];

        if (input.isDatasetSegment()) {
          valuePicker.colname(colname);
          rangePicker.hidden(true);
          dateRangePicker.hidden(true);
          relativeDateRangePicker.hidden(true);
          patternPicker.hidden(true);
          uploadPicker.hidden(true);
          expressionEditor.hidden(true);

          input.drawFilter(allSettings, data);
          return;
        }

        if (arc.report.coltypeIsBoolean(coltype)) {
          patternPicker.hidden(true);
          uploadPicker.hidden(true);
        }

        if (arc.report.coltypeIsInteger(coltype)) {
          patternPicker.hidden(true);
          dateRangePicker.hidden(true);
          relativeDateRangePicker.hidden(true);
        } else if (arc.report.coltypeIsNumber(coltype)) {
          valuePicker.hidden(true);
          patternPicker.hidden(true);
          dateRangePicker.hidden(true);
          relativeDateRangePicker.hidden(true);
          uploadPicker.hidden(true);
        } else {
          rangePicker.hidden(true);
          var c = arc.misc.columnIsDate(data.rows, 0);

          if (c) {
            dateRangePicker.datetimeFormat(c);
            relativeDateRangePicker.datetimeFormat(c);
            patternPicker.hidden(true);
            if (data.rows.length > 200) {
              valuePicker.hidden(true);
            }
          } else {
            dateRangePicker.hidden(true);
            relativeDateRangePicker.hidden(true);
          }
        }

        // Then draw the filters
        input.drawFilter(allSettings, data);
      }

      let rb = input.lib().viz().getRequestBuilder();
      rb = arc.data_requests.columnValues(rb, shelf_element.dataset(),
        colname, 100);

      if (input.isDatasetSegment()) {
        const data = segmentsToDataRows();
        withData(data);
        return;
      }

      loadFilter(undefined, rb, function(data) {
        withData(data);
      });
    }

    return input;
  };

  input.drawFilter = function(allSettings, columnData) {
    var colname = getColname();

    var filterSettings = arc.visual.settings();
    filterSettings.addSetting(allSettings.filter(function(s) {
      return s.hidden() ? false : true;
    }));

    filterModal.user_settings(filterSettings);

    var data = shelf_element.filter_input();

    filterSettings.settings().forEach(function(s) {
      if (s.colname) {
        s.colname(getColname());
      }

      if (s.coltype) {
        s.coltype(coltype);
      }

      if (s.filterInput) {
        s.filterInput(input);
      }

      if (s.id() in data) {
        s.value(data[s.id()]);
      }
    });

    filterModal.setup(() => {
      // This function is called on 'Apply' in the modal
      const filterInput = {};

      // If there are multiple tabs, then the presumption here is that there
      // is one setting per tab, and that the setting is the one that is
      // permitted to set the filter expression
      const activeTab = $('#settings-modal-holder .nav-tabs > li a.active[data-toggle="tab"]').text();

      filterSettings.settings().forEach((setting) => {
        const value = setting.fetch_value();
        filterInput[setting.id()] = value;

        if (setting.tab() === activeTab) {
          filterInput.activeSetting = setting.id();
          if (value.expr.trim() === '' && (!value.values || value.values.length)) {
            filterInput.expr = arc.report.shelf.EXPR_TRUE;
          } else {
            filterInput.expr = value.expr;
          }
        }
      });

      shelf_element.filter_input(filterInput);
      shelf_element.custom_expr(filterInput.expr);

      d3_elem.call(arc.report.shelf.settext);
      if (lib && lib.viz() && lib.viz().page_needs_save) {
        lib.viz().page_needs_save();
      }
    });

    if (!valuePicker.hidden()) {
      if (input.isDatasetSegment()) {
        const data = segmentsToDataRows();
        valuePicker.inputValues(data.rows);
        return;
      }

      var rb = input.lib().viz().getRequestBuilder();
      var vrb = arc.data_requests.distinctColumnValues(rb, shelf_element.dataset(),
        colname, 1000, shelf_element.dataset_coltype());
      loadFilter(valuePicker, vrb, function(data) {
        valuePicker.inputValues(data.rows);
      });
    }

    if (!rangePicker.hidden()) {
      var rrb = minmax();

      loadFilter(rangePicker, rrb, function(data) {
        rangePicker.inputValues(data.rows);
      });
    }

    if (!dateRangePicker.hidden()) {
      var drb = minmax();

      loadFilter(dateRangePicker, drb, function(data) {
        dateRangePicker.inputValues(data.rows);
        relativeDateRangePicker.inputValues(data.rows);
      });
    }

    return input;
  };

  input.valuesFilter = function(filter_data, filter_exclude, filter_includeNulls) {
    return arc.misc.whereClauseFromValues(
      getColname(), filter_data, filter_exclude, true, undefined, undefined, filter_includeNulls
    );
  };

  input.patternFilter = function(pattern_type, pattern_text, pattern_exclude,
      pattern_ignorecase) {

    var colname = getColname();

    var clause;
    if (pattern_ignorecase) {
      clause = ' ILIKE ';
    } else {
      clause = ' LIKE ';
    }

    var f = [];
    f.push('[' + colname + ']');
    if (pattern_exclude) {
      f.push(' NOT ');
    }

    f.push(clause);

    f.push("'");
    if (pattern_type === 'starts') {
      f.push(pattern_text + '%');
    }

    if (pattern_type === 'ends') {
      f.push('%' + pattern_text);
    }

    if (pattern_type === 'contains') {
      f.push('%' + pattern_text + '%');
    }

    f.push("'");

    return f.join('');
  };

  input.dateRangeFilter = function(min, max, timeformat) {
    if ((min.length === 0) || (max.length === 0)) {
      return arc.report.shelf.EXPR_TRUE;
    }

    return arc.misc.timestampRange(shelf_element.dataset().dataconnection_type(),
        getColname(), shelf_element.dataset_coltype(), min, max, timeformat);
  };

  return input;
};

}());
