window.arc = window.arc || {};

(function() {
'use strict';

arc.filter = arc.filter || {};

const inlineFilterHtml = `
  <div class="selected-control-wrapper">
    <div class="selected-control-title">
      <i class="fa fa-list"></i> <span class="base_field"></span>
    </div>

    <div class="inline-filter-range-widget">
    </div>
  </div>
`;

arc.filter.rangeDisplayTypeEnum = {
  minimal: 'minimal',
  filtershelf: 'filtershelf',
};

arc.filter.range = function() {
  var setting = arc.visual.setting()
                   .id('range_picker')
                   .tab('Range')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('RANGE_PICKER')
                   .value('');

  var colname,
      coltype,
      $sel = $(),
      rangeDisplayType = arc.filter.rangeDisplayTypeEnum.filtershelf,
      isReal,
      digitsSpinner,
      slider,
      valuesSet,
      digits = 2,
      defaultValues,
      data = {
        lowerBound: true,
        upperBound: true,
      };
  let displayFormatter;

  /* data is
   * {
   *   bounds: {
   *     min: <number>,
   *     max: <number>,
   *   },
   *   values: {
   *     min: <number>,
   *     max: <number>,
   *   },
   *   minOperator: '>=',
   *   maxOperator: '<=',
   *   lowerBound: true | false,
   *   upperBound: true | false,
   * }
   */

  function rangeFilterExpr(min, max, minOperator, maxOperator) {
    min = min + '';
    max = max + '';

    if ((min.length === 0) && (max.length === 0)) {
      return arc.report.shelf.EXPR_TRUE;
    }

    if ((min.length > 0) && (max.length > 0)) {
      return arc.misc.whereClauseFromMinMax(colname, min,
          max, minOperator, maxOperator);
    }

    if (min.length === 0) {
      return arc.misc.whereClauseFromMax(colname, max, maxOperator);
    }

    // max.length === 0
    return arc.misc.whereClauseFromMin(colname, min, minOperator);
  }

  function calculateValues(v) {
    const values = {
      min: v.min,
      max: v.max,
    };

    if (!data.lowerBound) {
      values.min = data.bounds.min;
    } else if (!data.upperBound) {
      values.max = data.bounds.max;
    }

    return values;
  }

  function setValues(v) {
    if (!initialized()) {
      return data;
    }

    const values = calculateValues(v);
    const $rs = $sel.find('.filter-range-widget, .inline-filter-range-widget');
    $rs.rangeSlider('values', values.min, values.max);
  }

  setting.defaultValues = function(_) {
    if (!arguments.length) return defaultValues;
    defaultValues = _;
    setValues(defaultValues);
    return setting;
  };

  setting.digits = function(_) {
    if (!arguments.length) return digits;
    digits = _;
    return setting;
  };

  setting.displayFormatter = function(_) {
    if (!arguments.length) return displayFormatter;
    displayFormatter = _;
    return setting;
  };

  setting.rangeDisplayType = function(_) {
    if (!arguments.length) return rangeDisplayType;
    rangeDisplayType = _;
    return setting;
  };

  setting.colname = function(_) {
    if (!arguments.length) return colname;
    colname = _;
    return setting;
  };

  setting.coltype = function(_) {
    if (!arguments.length) return coltype;
    coltype = _;
    return setting;
  };

  function valueCb(value) {
    data = $.extend({}, value);

    // Upgrade from a previous version of range filters to this one by using the right
    // defaults for lowerBound & upperBound below.
    if (data.lowerBound === undefined) {
      data.lowerBound = true;
    }

    if (data.upperBound === undefined) {
      data.upperBound = true;
    }

    setDropdownValues();
  }

  function getDecimalPointPrecision() {
    if (!isReal) {
      return 0;
    }

    if (digitsSpinner) {
      return Number($sel.find('.digits').val());
    }

    return digits || 0;
  }

  function rangeFormatter(val, config) {
    let {
      forMaximum,
      noDisplayFormat,
    } = config || {};

    if (!noDisplayFormat && setting.viz_lib()) {
      const displayFormatSetting = setting.viz_lib().user_settings().find_by_id('display_format');
      if (displayFormatSetting && displayFormatSetting.value().trim()) {
        return arc.utils.displayFormat(displayFormatSetting.value().trim())(val);
      }

      if (displayFormatter) {
        return displayFormatter(val);
      }
    }

    const fn = forMaximum ? Math.ceil : Math.floor;
    const d = getDecimalPointPrecision();
    return Number(fn(val + 'e+' + d)  + 'e-' + d);
  }

  function setDropdownValues() {
    if (data.values) {
      $sel.find('.lower-bound')
          .val(data.values.min);

      $sel.find('.upper-bound')
          .val(data.values.max);
    }
  }

  function addDropdown() {
    $sel.find('.inline-filter-range-widget').parent()
        .append($('#range-filter-dropdown').html());

    setDropdownValues();
  }

  function setupHandlers() {
    $sel.find('.range-filter-select')
        .offOn('change', function() {
          const val = $(this).val();
          if (val === 'gte') {
            data.lowerBound = true;
            data.upperBound = false;
          } else if (val === 'lte') {
            data.lowerBound = false;
            data.upperBound = true;
          } else {
            data.lowerBound = true;
            data.upperBound = true;
          }

          drawRangeSlider();
        });

    $sel.find('.range-cancel')
        .offOn('click.apply', () => {
          setDropdownValues();
          $sel.find('.dropdown-toggle')
              .trigger('click');
        });

    $sel.find('.range-apply')
        .offOn('click.apply', () => {
          let changeMade = false;
          const min = $sel.find('.lower-bound').val().trim();
          if (min !== '') {
            data.values.min = Number(min);
            changeMade = true;
          }

          const max = $sel.find('.upper-bound').val().trim();
          if (max !== '') {
            data.values.max = Number(max);
            changeMade = true;
          }

          if (changeMade) {
            drawRangeSlider();
            setting.value(data.values);
            setting.makeChangeCbs();
          }

          $sel.find('.dropdown-toggle')
              .trigger('click');
        });

    $sel.find('.range-filter-dropdown a')
        .offOn('click.rangeFilterDropdown', () => {
          const $filter_widget_open = $('.picklist-list-wrapper.dropdown-menu.open');

          $filter_widget_open.hide().removeClass('open');
        });
  }

  function initialized() {
    if (!slider) {
      return false;
    }

    const $rs = $sel.find('.filter-range-widget, .inline-filter-range-widget');

    try {
      $rs.rangeSlider('values');
    } catch (err) {
      return false;
    }

    return true;
  }

  function fetch_fn() {
    if (data) {
      const lowerBound = data.lowerBound,
            upperBound = data.upperBound;

      data = {
        lowerBound,
        upperBound,
      };
    } else {
      data = {
        lowerBound: true,
        upperBound: true,
      };
    }

    const rangeSliderDropdown = '';

    const isMinimal = (rangeDisplayType === arc.filter.rangeDisplayTypeEnum.minimal);

    if (!initialized()) {
      return data;
    }

    const $rs = $sel.find('.filter-range-widget, .inline-filter-range-widget');
    let r = $rs.rangeSlider('values');

    data.values = {
      min: rangeFormatter(r.min, {
        noDisplayFormat: true,
      }),
      max: rangeFormatter(r.max, {
        forMaximum: true,
        noDisplayFormat: true,
      }),
    };

    const b = $rs.rangeSlider('bounds');
    data.bounds = {
      min: b.min,
      max: b.max,
    };

    if (!data.lowerBound) {
      data.values.min = data.bounds.min;
    }

    if (!data.upperBound) {
      data.values.max = data.bounds.max;
    }

    data.minOperator = isMinimal ? '>=' : $sel.find('.minval-operator').val();
    data.maxOperator = isMinimal ? '<=' : $sel.find('.maxval-operator').val();

    data.digits = getDecimalPointPrecision();

    data.expr = rangeFilterExpr(data.values.min, data.values.max,
        data.minOperator, data.maxOperator);

    setDropdownValues();
    return data;
  }

  function shown_fn() {
    drawRangeSlider();
  }

  function paint_fn(d3_sel) {
    isReal = arc.report.coltypeIsReal(coltype);

    d3_sel.selectAll('.range-filter-setting')
          .remove();
    slider = undefined;

    $(`${setting.div_id()}>div:not(.viz-permanent)`)
      .remove();

    d3_sel.append('div')
          .classed('range-filter-setting', true)
          .attr('id', setting.div_id());
    $sel = $(`#${setting.div_id()}`);

    if (rangeDisplayType === arc.filter.rangeDisplayTypeEnum.minimal) {
      paint_fn_minimal(d3_sel);
      addDropdown();
      setupHandlers();
    } else {
      paint_fn_filtershelf(d3_sel);
    }
  }

  function paint_fn_minimal(d3_sel) {
    $sel.find('.selected-control-wrapper').remove();
    $sel.append(inlineFilterHtml);
    $sel.find('.base_field').text(colname);
  }

  function paint_fn_filtershelf(d3_sel) {
    $sel.append($('#filter-range-template').html());

    $sel.find('.arc-spinner').removeClass('arc-invisible');
    if (isReal) {
      $sel.find('.range-digits').removeClass('arc-invisible');
    }

    var $digits = $sel.find('.digits');
    $digits.val(data.digits);
    digitsSpinner = $digits.spinner({
      min: 0,
      max: 3,
      change: drawRangeSlider,
    });
  }

  function drawRangeSlider(bounds) {
    if (!valuesSet) {
      return;
    }

    if (!bounds && data && data.bounds) {
      bounds = data.bounds;
    }

    $sel.find('.arc-spinner')
        .addClass('arc-invisible');
    const $rs = $sel.find('.filter-range-widget, .inline-filter-range-widget');

    var minmax;
    if (slider && $rs.length === 1) {
      bounds = $rs.rangeSlider('bounds');
      minmax = $rs.rangeSlider('values');
      $rs.rangeSlider('destroy');
      slider = undefined;
    } else {
      minmax = $.extend({}, bounds);
      if (data && data.bounds !== undefined) {
        minmax.min = data.bounds.min;
        minmax.max = data.bounds.max;
      }
    }

    if (data.values === undefined) {
      data.values = {
        min: minmax.min,
        max: minmax.max,
      };
    }

    data.values.min = Math.max(data.values.min, data.bounds.min);
    data.values.max = Math.min(data.values.max, data.bounds.max);

    const precision = getDecimalPointPrecision();

    // If range is being drawn within an invisible tab, rangeSlider throws a JS
    // error. Track that condition and do not draw it.
    const $tab = $sel.closest('.tab-pane');
    if ($tab.length === 1) {
      if (!$tab.hasClass('active')) {
        return;
      }
    }

    slider = $rs.rangeSlider({
      formatter: rangeFormatter,
      arrows: false,
      step: Math.pow(10, -precision),
      bounds: bounds,
      defaultValues: calculateValues(data.values),
    });

    if (setting.viz_lib()) {
      const lookAndFeelSettings = setting.viz_lib().user_settings();
      const barColor = lookAndFeelSettings.find_by_id('range_sliderbar_color');
      if (barColor) {
        const barOpacity = lookAndFeelSettings.find_by_id('range_sliderbar_opacity');
        $rs.find('.ui-rangeSlider-bar')
           .css('background-color', barColor.value())
           .css('opacity', (Number(barOpacity.value())) / 100);
      }

      const innerBarColor = lookAndFeelSettings.find_by_id('range_innerbar_color');
      if (innerBarColor) {
        $rs.find('.ui-rangeSlider-innerBar')
           .css('background', innerBarColor.value());
      }

      const labelBgColor = lookAndFeelSettings.find_by_id('range_labelbg_color');
      if (labelBgColor) {
        const labelFgColor = lookAndFeelSettings.find_by_id('range_labelfg_color');
        $rs.find('.ui-rangeSlider-label')
           .css('background-color', labelBgColor.value())
           .css('color', labelFgColor.value());

        $rs.find('.ui-rangeSlider-handle')
           .css('background-color', labelBgColor.value());

        $rs.find('.ui-rangeSlider-label-inner')
           .css('border-top-color', labelBgColor.value());
      }
    }

    $rs.on('valuesChanged', (e, d) => {
      if (d.values) {
        data.values = {
          min: d.values.min,
          max: d.values.max,
        };
      }

      setting.value(data.values);
      setting.makeChangeCbs();
    });

    if (data && data.minOperator !== undefined) {
      $sel.find('.minval-operator').val(data.minOperator);
    }

    if (data && data.maxOperator !== undefined) {
      $sel.find('.maxval-operator').val(data.maxOperator);
    }

    $sel.find('.maxval-operator')
        .off('change.maxval')
        .on('change.maxval', function() {
          var op = $(this).val();
          if (op === '') {
            var $min = $sel.find('.minval-operator');
            if ($min.val() === '') {
              $min.val('>=');
            }

            return;
          }
        });

    $sel.find('.minval-operator')
        .off('change.minval')
        .on('change.minval', function() {
          var op = $(this).val();
          if (op === '') {
            var $max = $sel.find('.maxval-operator');
            if ($max.val() === '') {
              $max.val('<=');
            }

            return;
          }

          if ((op === '=') || (op === '!=')) {
            $sel.find('.maxval-operator')
                .val('');
          }
        });
  }

  setting.inputValues = function(rows) {
    valuesSet = true;
    if (rows.length === 1) {
      var min = Number(rows[0][0]),
          max = Number(rows[0][1]);

      data.bounds = {
        min,
        max,
      };

      drawRangeSlider({
        min: min,
        max: max,
      });
    }

    return setting;
  };

  setting.loading = function(l) {
    if (l) {
      $sel.find('.arc-spinner').removeClass('arc-invisible');
    } else {
      $sel.find('.arc-spinner').addClass('arc-invisible');
    }

    return setting;
  };

  setting.valueCb(valueCb)
         .paint_fn(paint_fn)
         .fetch_fn(fetch_fn)
         .shown_fn(shown_fn);

  return setting;
};

}());
