window.arc = window.arc || {};

(function() {
'use strict';

arc.filter = arc.filter || {};

arc.filter.helpers = arc.filter.helpers || {};

const defaultTimeZone = 'UTC';

function units($elem) {
  return $elem.closest('.filter-boundary').find('.filter-relative-units');
}

function relative($elem) {
  return $elem.closest('.filter-boundary').find('.relative-detail');
}

function visibility($elem) {
  var v = $elem.val();

  if ((v === undefined) || (v === '')) {
    return;
  }

  switch (v) {
    case 'Today':
      relative($elem).hide();
      break;
    default:
      relative($elem).show();
      var $u = units($elem);
      if ($u.val() === '') {
        $u.val('1');
      }

      break;
  }
}

function isTime(timeformat) {
  return (timeformat.indexOf('HH') >= 0 ||
    timeformat.indexOf('mm') >= 0 || timeformat.indexOf('ss') >= 0);
}

function setupTimePicker($sel, chart_div, timeformat, timezone) {
  const $fixedDate = $sel.find('.fixed-date');
  $fixedDate.text(moment.utc().tz(timezone).format(timeformat));

  const opts = {
    singleDatePicker: true,
    showDropdowns: true,
    locale: { format: timeformat },
    parentEl: `#${chart_div}`,
  };

  if (isTime(timeformat)) {
    opts.timePicker = true;
    opts.timePickerIncrement = 1;
  }

  const picker = $fixedDate.daterangepicker(opts);
  $(`#${chart_div} .daterangepicker`).on('click', function(e) {
    e.stopPropagation();
  });

  $(`#${chart_div} .daterangepicker .applyBtn`).on('click', e => e.preventDefault());

  return picker;
}

function displayRelativeElements($sel) {
  $sel.find('.relative-daterange-prevnext')
      .each(function() {
        visibility($(this));
      })
      .offOn('change.relative', function() {
        visibility($(this));
      });
}

const allTimeUnits = {
  Second: 0,
  Minute: 1,
  Hour: 2,
  Day: 3,
  Week: 4,
  Month: 5,
  Quarter: 6,
  Year: 7,
};

function changeUnits($sel, unit) {
  $sel.find('span.unit').text(unit);
  $sel.find('span.units').text(`${unit}s`);

  // Week, Month, Quarter, and Year need the "Week to date,"
  // "Month to date," etc option
  $sel.find('.unit-to-date').toggle(allTimeUnits[unit] > 3);
}

arc.filter.helpers.drawRelative = function(config) {
  let {
    $container,
    currentSelectionsRelative,
    timeformat,
    showtime,
    container_div_id,
    applyCb,
    isDropdown,
    timezone,
    $applyBtn,
  } = config;

  timezone = timezone || defaultTimeZone;

  if (isDropdown) {
    $container.html(`
      <input type="text" class="form-control form-control-sm relative-date-range"
          data-toggle="dropdown"
          placeholder="click to select relative range">
      <div class="dropdown-menu relative-date-menu">
        <div class="utc-info">Note that all times below are in the ${timezone} timezone</div>
        <div class="relative-date-holder"></div>
      </div>`);
  } else {
    $container.html(`
      <div class="relative-date-menu">
        <div class="utc-info">Note that all times below are in the ${timezone} timezone</div>
        <div class="relative-date-holder"></div>
      </div>`);
  }

  const $relativeDateRange = $container.find('.relative-date-range');
  const $relativeDateHolder = $container.find('.relative-date-holder');
  $relativeDateHolder
      .html($('#filter-relative-daterange-template').html());

  if (isDropdown) {
    $relativeDateHolder.append($('#filter-relative-btns-template').html());
  }

  $relativeDateHolder
      .on('click', function(event) {
        event.stopPropagation();
      });

  setupRelativeElements($container, container_div_id, timeformat, showtime, timezone);

  if (!$applyBtn && isDropdown) {
    $applyBtn = $relativeDateHolder.find('.applyBtn');
  }

  $applyBtn
      .on('click', function() {
        let data = arc.filter.helpers.saveUISelections($relativeDateHolder);
        arc.filter.helpers.fetchRelative(data, timeformat, timezone);
        $relativeDateRange.val(`${data.startDate} - ${data.endDate}`);
        if (isDropdown) {
          $relativeDateRange.dropdown('toggle');
        }

        if (applyCb) {
          applyCb(data);
        }
      });

  $relativeDateHolder.find('.cancelBtn')
      .off('click.cancelRelative')
      .on('click.cancelRelative', function toggleRelativeDate() {
        if (isDropdown) {
          $relativeDateRange.dropdown('toggle');
        }
      });

  if (currentSelectionsRelative) {
    arc.filter.helpers.restoreUISelections($container, currentSelectionsRelative, timeformat);
  }
};

function setupRelativeElements($sel, chart_div, timeformat, showtime, timezone) {
  timezone = timezone || defaultTimeZone;

  let unitOptions;
  if (showtime) {
    $sel.find('.anchor-label-current').text('Anchor to Now');
    unitOptions = Object.keys(allTimeUnits);
  } else {
    // If it's not a time filter, hide hour, minute, and second options.
    unitOptions = Object.keys(allTimeUnits).filter(unit => allTimeUnits[unit] >= 3);
  }

  const allOptions = unitOptions.map(unit =>
    `<option value="${unit}"${unit === 'Quarter' ? ' selected' : ''}>${unit}s</option>`
  ).join('');

  $sel.find('.unit-selector').html(allOptions);
  $sel.find('.relative-daterange-unit').each(function() {
    $(this).html(allOptions);
  });

  const picker = setupTimePicker($sel, chart_div, timeformat, timezone);
  displayRelativeElements($sel);

  $sel.find('.unit-selector')
      .offOn('change.unit', function() {
        changeUnits($sel, $(this).val());
      });

  $sel.find('input[type=radio][name=datetype]').offOn('change.input', function() {
    var v = $(this).val();
    $sel.find('.relative-custom').toggle(v === 'custom');
  });

  var $fixedDate = $sel.find('.fixed-date');
  $fixedDate.offOn('click.fixedDate', () => {
    $sel.find('input[name=anchor][value="specific"]').prop('checked', true);
  });

  picker.on('apply.daterangepicker', function(ev, picker) {
          $fixedDate.text(picker.startDate.format(timeformat));
        });
}

function chooseRadio($sel, name, value) {
  let $a = $sel.find('input[type=radio][name="' + name + '"][value="' +
      value + '"]');
  if ($a.length === 0) {
    return;
  }

  $sel.find('input[type=radio][name="' + name + '"]:checked').prop('checked', false);
  $a.prop('checked', true);
}

arc.filter.helpers.restoreUISelections = function($sel, data, timeformat) {
  if (!data) {
    //Most likely, the date is of Fixed type and NOT relative type.
    return;
  }

  if (data.anchorType) {
    chooseRadio($sel, 'anchor', data.anchorType);

    if (data.anchorType === 'specific') {
      $sel.find('.fixed-date').text(moment.utc(data.fixedDate).format(timeformat));
    }

    if (data.anchorType === 'basefield') {
      $sel.find('.basefield-selector').val(data.baseField);
    }
  }

  if (data.unit) {
    $sel.find('.unit-selector').val(data.unit);
    changeUnits($sel, data.unit);
  }

  if (data.relativeType) {
    chooseRadio($sel, 'datetype', data.relativeType);
  }

  $sel.find('.last-count').val(data.lastCount);
  $sel.find('.next-count').val(data.nextCount);

  if (data.custom) {
    restore($sel.find('.filter-startdate'), data.custom.start);
    restore($sel.find('.filter-enddate'), data.custom.end);
    $sel.find('.relative-custom').show();
  } else {
    $sel.find('.relative-custom').hide();
  }

  $sel.find('.relative-daterange-prevnext')
      .each(function() {
        visibility($(this));
      });

  function restore($elem, d) {
    $elem.find('.relative-daterange-prevnext').val(d.type);
    $elem.find('.filter-relative-units').val(d.relative);
    $elem.find('.relative-daterange-unit').val(d.relativeType);
  }
};

// Imp function to understand different values of relative dates.
// TODO: We might need to document this and publish for custom JS uses
arc.filter.helpers.saveUISelections = function($sel) {
  let data = {};

  // Possible values for data.anchorType are:
  // ["today", "specific"]
  data.anchorType = $sel.find('input[type=radio][name=anchor]:checked').val();

  if (data.anchorType === 'specific') {
    let $fixedDate = $sel.find('.fixed-date');
    data.fixedDate = $fixedDate.text().trim();
  }

  if (data.anchorType === 'basefield') {
    const $baseField = $sel.find('.basefield-selector');
    data.baseField = $baseField.val();
  }

  data.unit = $sel.find('.unit-selector').val();

  // Possible values for data.relativetype are:
  // ["previous-one", "this", "next-one", "previous", "next", "to-date", "custom"]
  data.relativeType = $sel.find('input[type=radio][name=datetype]:checked').val();

  if (data.relativeType === 'previous') {
    data.lastCount = $sel.find('.last-count').val();
  }

  if (data.relativeType === 'next') {
    data.nextCount = $sel.find('.next-count').val();
  }

  if (data.relativeType === 'custom') {
    data.custom = {};
    data.custom.start = store($sel.find('.filter-startdate'));
    data.custom.end = store($sel.find('.filter-enddate'));
  }

  function store($elem) {
    return {

      // ["Today", "Previous", "Next"]
      type: $elem.find('.relative-daterange-prevnext').val(),

      // Any number
      relative: $elem.find('.filter-relative-units').val(),

      // ["Day", "Week", "Month", "Quarter", "Year"]
      relativeType: $elem.find('.relative-daterange-unit').val(),
    };
  }

  return data;
};

arc.filter.helpers.fetchRelative = function(data, timeformat, timezone) {
  timezone = timezone || defaultTimeZone;

  // backwards compatibility for relative date range filters created before 4.2
  if (!timeformat) {
    timeformat = /^\d{4}-\d{2}-\d{2}$/.test(data.startDate) ? 'YYYY-MM-DD' : 'YYYY-MM-DD HH:mm:ss';
  }

  data.datetimeFormat = timeformat;
  const fixed = data.anchorType === 'specific' && data.fixedDate;
  const baseFieldDate = data.anchorType === 'basefield' && data.baseFieldDate;

  function anchor() {
    if (fixed || baseFieldDate) {
      return moment.utc(fixed || baseFieldDate);
    }

    return moment.utc().tz(timezone);
  }

  const roundToDays = isTime(timeformat) ? false : true;

  switch (data.relativeType) {
  case 'previous':
    data.startDate = anchor().subtract(Number(data.lastCount), data.unit);
    data.endDate = anchor();

    if (roundToDays) {
      data.startDate = data.startDate.startOf(data.unit);
      data.endDate = data.endDate.subtract(1, data.unit).endOf(data.unit);
    }

    break;
  case 'next':
    data.startDate = anchor();
    data.endDate = anchor().add(Number(data.nextCount), data.unit);

    if (roundToDays) {
      data.startDate = data.startDate.startOf(data.unit);
      data.endDate = data.endDate.subtract(1, data.unit).endOf(data.unit);
    }

    break;
  case 'to-date':
    data.startDate = anchor().startOf(data.unit);
    data.endDate = anchor();
    break;
  case 'previous-one':
    data.startDate = anchor().subtract(1, data.unit).startOf(data.unit);
    data.endDate = moment.utc(data.startDate).endOf(data.unit);
    break;
  case 'next-one':
    data.startDate = anchor().add(1, data.unit).startOf(data.unit);
    data.endDate = moment.utc(data.startDate).endOf(data.unit);
    break;
  case 'this':
    data.startDate = anchor().startOf(data.unit);
    data.endDate = anchor().endOf(data.unit);
    break;
  case 'custom':
    data.startDate = to_moment(data, true);
    data.endDate = to_moment(data, false);
    break;
  default:
    data.startDate = anchor();
    data.endDate = anchor();
    break;
  }

  data.startDate = data.startDate.format(timeformat);
  data.endDate = data.endDate.format(timeformat);
  return;

  function to_moment(ds, start) {
    var d,
        other,
        initial;

    // ds.fixedDate should never be '' unless user sets it from URL/custom JS
    if (!ds.fixedDate) {
      initial = moment.utc();
    } else {
      initial = moment.utc(ds.fixedDate);
    }

    if (start) {
      d = ds.custom.start;
      other = ds.custom.end;
    } else {
      d = ds.custom.end;
      other = ds.custom.start;
    }

    var s,
        r = Number(d.relative);

    switch (d.type) {
      case 'Today':
        s = initial;
        return s;

      case 'Previous':
        s = initial.subtract(r, d.relativeType);
        if (roundToDays) {
          s = s.startOf(d.relativeType);
        }

        return s;

      case 'Next':
        s = initial.add(r, d.relativeType);
        if (roundToDays) {
          s = s.startOf(d.relativeType);
        }

        return s;

      case 'Duration':
        s = moment.utc(ds.startDate).add(r, d.relativeType);

        if (roundToDays) {
          if (r > 0) {
            s = s.subtract(1, 'day');
          }

          s.endOf(d.relativeType);
        }

        return s;
    }

    return undefined;
  }
};

arc.filter.helpers.fetchRelativeExpression = function(data, colname, coltype, dataconn_type) {
  const startend = {};

  arc.filter.helpers.fetchRelative(data, data.datetimeFormat);
  startend.start = data.startDate;
  startend.end = data.endDate;

  if (!startend.start && !startend.end) {
    return undefined;
  }

  coltype = coltype.toLowerCase();

  return arc.filter.helpers.rangeExpression(colname, coltype, dataconn_type, startend);
};

arc.filter.helpers.rangeExpression = function(colname, coltype, conn_type, startend) {
  if ((coltype === 'timestamp') || (coltype === 'string')) {
    return arc.misc.timestampRange(
      conn_type, colname, coltype, startend.start, startend.end, undefined, startend.include_nulls
    );
  }

  return arc.misc.rangeExpression(
    conn_type, colname, coltype, startend.start, startend.end, startend.include_nulls
  );
};

arc.filter.relativeDateRange = function() {
  var setting = arc.visual.setting()
                   .id('relative_daterange_picker')
                   .tab('Relative Date Range')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('RELATIVE_DATERANGE_PICKER')
                   .value('');

  var datetimeFormat,
      filterInput,
      data;

  setting.datetimeFormat = function(_) {
    if (!arguments.length) return datetimeFormat;
    datetimeFormat = _;
    return setting;
  };

  setting.filterInput = function(_) {
    if (!arguments.length) return filterInput;
    filterInput = _;
    return setting;
  };

  function getTimeFormat() {
    return datetimeFormat.fmtString;
  }

  function showTime() {
    return datetimeFormat && datetimeFormat.is_datetime;
  }

  function valueCb(value) {
    data = {};
    $.extend(data, value);
  }

  function fetch_fn() {
    let div_id = setting.div_id(),
        $sel = $(`#${div_id}`),
        data = arc.filter.helpers.saveUISelections($sel);

    arc.filter.helpers.fetchRelative(data, getTimeFormat());
    if (data.startDate && data.endDate) {
      data.expr = filterInput.dateRangeFilter(data.startDate, data.endDate, 'YYYY-MM-DD');
    } else {
      data.expr = '';
    }

    return data;
  }

  function paint_fn(d3_sel) {
    var div_id = setting.div_id();
    d3_sel.append('div')
          .attr('id', div_id);

    var $sel = $(`#${div_id}`);

    $sel.append($('#filter-relative-daterange-template').html());

    setupRelativeElements($sel, div_id, getTimeFormat(), showTime());

    if (data) {
      arc.filter.helpers.restoreUISelections($sel, data, getTimeFormat());
    }
  }

  setting.inputValues = function(rows) {
    return setting;
  };

  setting.valueCb(valueCb)
         .paint_fn(paint_fn)
         .fetch_fn(fetch_fn);

  return setting;
};

}());
