window.arc = window.arc || {};

(function() {
'use strict';

arc.filter = arc.filter || {};

const rowHeaderTemplate = `
  <li class="segment-group">%VALUE%</li>
`;

const rowTemplateSegment = `
  <li class="segment" data-segmentid="%ID%">
    <label class="picker-input-label">
      <input data-value="%VALUE%" type="checkbox">
      <a class="segment-isnot" data-segment-include="is">is</a>
      <span class="filter-value-checkbox">%VALUE%</span>
    </label>
  </li>
`;

const rowTemplate = `
  <li>
    <label class="picker-input-label">
      <input data-value="%VALUE%" type="checkbox" data-test-id="filter-input-checkbox-%VALUE%">
      <span class="filter-value-checkbox">%VALUE%</span>
    </label>
  </li>
`;

arc.filter.valuePicker = function() {
  const setting = arc.visual.setting()
                     .id('value_picker')
                     .tab('Values')
                     .type(arc.visual.settingTypeEnum.CUSTOM)
                     .add_name('VALUE_PICKER')
                     .value('');

  let data = {};

  let filterInput;
  let colname;

  setting.colname = function(_) {
    if (!arguments.length) return colname;
    colname = _;
    return setting;
  };

  setting.isDatasetSegment = function() {
    return filterInput.isDatasetSegment();
  };

  setting.filterInput = function(_) {
    if (!arguments.length) return filterInput;
    filterInput = _;
    return setting;
  };

  function paintSelections() {
    if (!data) {
      return;
    }

    if (setting.isDatasetSegment()) {
      paintSelectionsSegment();
      return;
    }

    const h = {};
    if (data.values) {
      data.values.forEach((d) => {
        h[d] = true;
      });
    }

    const div_id = setting.div_id();
    const $sel = $(`#${div_id}`);
    $sel.find('.filter-pick-exclude')
        .prop('checked', (!!data.exclude));
    $sel.find('.filter-pick-select-all')
    .prop('checked', (!!data.selectAll));
    $sel.find('.filter-pick-include-null')
    .prop('checked', (!!data.includeNulls));

    if (data.values && (data.values.length > 0)) {
      const $inputs = $sel.find('.filter-pick-values input');
      $inputs.prop('checked', false);

      $inputs.each(function() {
        const v = $(this).attr('data-value');
        if (h[v]) {
          $(this).prop('checked', true);
        }
      });
    }
  }

  function valueCb(value) {
    data = {};
    $.extend(data, value);

    paintSelections();
  }

  function paintSelectionsSegment() {
    if (!data.expr) {
      return;
    }

    const div_id = setting.div_id();
    const $sel = $(`#${div_id}`);

    const segments = JSON.parse(data.expr);
    const segmentsById = {};

    segments.forEach((v) => {
      segmentsById[v[0]] = v[1];
    });

    $sel.find('li.segment')
        .each(function() {
          const $li = $(this);
          const segmentId = $li.attr('data-segmentid');

          if (segmentsById[segmentId] === undefined) {
            return;
          }

          $li.find('input').prop('checked', true);

          const $a = $li.find('a');
          const newVal = segmentsById[segmentId] ? 'is' : 'not';
          $a.attr('data-segment-include', newVal);
          $a.text(newVal);
        });
  }

  function fetch_fn_segment() {
    const div_id = setting.div_id();
    const $sel = $(`#${div_id}`);

    data.exclude = false;
    data.values = [];

    $sel.find('.picker-input-label input:checked')
        .each(function() {
          const $this = $(this);
          const $li = $this.closest('li');
          const $a = $li.find('a');

          const segmentId = $li.attr('data-segmentid');
          const segmentInclude = $a.attr('data-segment-include') === 'is';

          data.values.push([
              segmentId,
              segmentInclude,
          ]);
        });

    data.expr = JSON.stringify(data.values);
    return data;
  }

  function fetch_fn() {
    if (setting.isDatasetSegment()) {
      return fetch_fn_segment();
    }

    const div_id = setting.div_id();
    const $sel = $(`#${div_id}`);

    data.exclude = $sel.find('.filter-pick-exclude').prop('checked');
    data.selectAll = $sel.find('.filter-pick-select-all').prop('checked');
    data.includeNulls = $sel.find('.filter-pick-include-null').prop('checked');
    data.values = [];

    $sel.find('li input:checked').each(function() {
      data.values.push($(this).attr('data-value'));
    });

    const coltype = filterInput.coltype();
    const e = arc.params.format_list(data.values, coltype);

    data.expr = filterInput.valuesFilter(e, data.exclude, data.includeNulls);
    return data;
  }

  function paint_fn(d3_sel) {
    const div_id = setting.div_id();
    d3_sel.append('div')
          .attr('id', div_id);

    const $sel = $(`#${div_id}`);

    if (setting.isDatasetSegment()) {
      $sel.append($('#segment-pick-template').html());
    } else {
      $sel.append($('#filter-pick-template').html());
    }

    paintSelections();
  }

  setting.inputValues = function(rows) {
    const div_id = setting.div_id();

    const $ul = $(`#${div_id} .filter-pick-values`);
    const $selectAll = $(`#${div_id} .filter-pick-select-all`);

    $ul.empty();

    let lastGroup;
    let hasNullOption = false;

    // Handle impala unsorted results
    rows.sort((rowA, rowB) => {
      if (rowA[0] < rowB[0]) return -1;
      if (rowA[0] > rowB[0]) return 1;
      return 0;
    });

    rows.forEach((row) => {
      if (row[0] === null) {
        hasNullOption = true;
        return;
      }

      if (setting.isDatasetSegment()) {
        if (lastGroup !== row[1]) {
          $ul.append(arc.misc.safe_substitute(rowHeaderTemplate, {
            VALUE: row[1],
          }));

          lastGroup = row[1];
        }
      }

      if (setting.isDatasetSegment()) {
        $ul.append(arc.misc.safe_substitute(rowTemplateSegment, {
          VALUE: row[0],
          ID: row[2],
        }));
      } else {
        $ul.append(arc.misc.safe_substitute(rowTemplate, {
          VALUE: row[0],
        }));
      }
    });

      const $sel = $(`#${div_id} .include-null-div`);
      $sel.toggleClass('arc-invisible', !hasNullOption);

    $selectAll.offOn('click.triggerSelectAll', (e) => {
      $ul.find('input').prop('checked', e.target.checked);
    });

    $ul.offOn('click.changeSelectAll', 'input', (e) => {
      if (!e.target.checked) {
        $selectAll.prop('checked', false);
      } else {
        const isEverythingSelected = $ul.find('input').toArray().every((input) => input.checked);

        $selectAll.prop('checked', isEverythingSelected);
      }
    });

    $ul.find('.segment-isnot')
       .on('click.isnot', function() {
         const val = $(this).attr('data-segment-include');
         const newVal = (val === 'is') ? 'not' : 'is';
         $(this).attr('data-segment-include', newVal);
         $(this).text(newVal);
       });

    paintSelections();

    return setting;
  };

  setting.loading = function(l) {
    const div_id = setting.div_id();

    if (l) {
      $(`#${div_id} .arc-spinner`).removeClass('arc-invisible');
    } else {
      $(`#${div_id} .arc-spinner`).addClass('arc-invisible');
    }

    return setting;
  };

  setting.valueCb(valueCb)
         .paint_fn(paint_fn)
         .fetch_fn(fetch_fn);

  return setting;
};
}());
