window.arc = window.arc || {};

(function() {
'use strict';

arc.grammar = arc.grammar || {};

// Clone a template element, update its id and make it visible
//
// We will do html text replacement of the template for the items in the parms object.
//
// The text replacement doesn't work for the attributes of the top level element so
// these can be updated using the attrs object as needed.
//
// It's possible attrs support is not needed and we can just move all items inside the
// containing div element, as is done with dsattr-tabhdr-template
//
arc.clone_template = function(id, parms, attrs) {
  // Grab the template and bump the sequence number to ensure unique id
  const $template = $(`#${id}-template`);
  let seqnum = Number($template.attr('seqnum')) || 0;
  $template.attr('seqnum', seqnum += 1);

  const div = $template.clone(false)
                    .show()
                    .attr('id', `${id}-${seqnum}`)
                    .attr('arcid', `${id}-active`);

  // Set up attributes of the div
  if (attrs) {
    Object.keys(attrs).forEach((key) => {
      div.attr(key, attrs[key]);
    });
  }

  // Replace parameters in template
  if (parms) {
    let html = div.html();
    Object.keys(parms).forEach((key) => {
      html = html.replace(new RegExp(`\\$${key}`, 'gm'), parms[key]);
    });
    div.html(html);
  }

  return div;
};

// @Deprecated - Use arc.misc.isIdentical instead
// Compare arrays, and arrays of arrays, to check for equality
arc.arraysIdentical = function(a, b) {
  if ((!a) || (!b)) {
    return false;
  }

  let i = a.length;
  if (i !== b.length) return false;
  // eslint-disable-next-line no-plusplus
  while (i--) {
    // Permit comparison of arrays of arrays
    if (Array.isArray(a[i]) && Array.isArray(b[i])) {
      const r = arc.arraysIdentical(a[i], b[i]);
      if (r === false) {
        return false;
      }
    } else if (a[i] !== b[i]) return false;
  }

  return true;
};

// @Deprecated - Use arc.misc.isIdentical instead
// Compare two simpley key/value objects where values are simple types
arc.simpleDictionariesEqual = function(d1, d2) {
  const keys = Object.keys(d1);
  if (keys.length !== Object.keys(d2).length) {
    return false;
  }

  for (const key of keys) {
    if (d1[key] !== d2[key]) {
      return false;
    }
  }

  return true;
};

arc.endsWith = function(str, suffix) {
  return str.indexOf(suffix, str.length - suffix.length) !== -1;
};

arc.editTitleHelper = function(edit_btn, title_id, $edit_showhide) {
  // In reports and dashboards, we have a 'Edit' button that
  // controls the editability of a title or subtitle. This
  // code provides the functionality to make the interactions
  // there work smoothly.

  if ($edit_showhide === undefined) {
    $edit_showhide = $();
  }

  const $e = $(`#${title_id}`);
  $e.prop('contenteditable', true);

  const eventName = 'titleeditor';
  $(`#${edit_btn},.edit-mode #${title_id}`)
      .offOn(`click.${eventName}`, () => {
    $e.focus();
    let in_data = $e.text();
    $edit_showhide.addClass('arc-invisible');

    function commitChange($title) {
      $title.text($title.text());
      $edit_showhide.removeClass('arc-invisible');
      if (in_data !== $title.text()) {
        in_data = $title.text();
        arc.report.pagenav.needs_save(true);
      }
    }

    $e.offOn(`keypress.${eventName}`, (event) => {
      // If enter or ctrl-enter is pressed...
      if (event.which === arc.misc.ENTER_KEY) {
        commitChange($e);
      }

      // Do not propagate newline onwards
      return event.which !== 13;
    });

    $e.offOn(`focusout.${eventName}`, () => {
      commitChange($e);
    });
  });
};

// remove and add listener
$.fn.extend({
  offOn(eventName, handlerOrSelector, _handler) {
    this.off(eventName).on(eventName, handlerOrSelector, _handler);
    return this;
  },
});
}());
