window.arc = window.arc || {};

(function() {
'use strict';

arc.globals = arc.globals || {};
arc.globals.urlparams = arc.globals.urlparams || {};
arc.globals.query = arc.globals.query || {};
arc.globals.query.cache_read = arc.globals.query.cache_read || true;
arc.globals.perms = {};
arc.globals.perms.permlists = {};

arc.globals.app_settings = arc.globals.app_settings || {};
arc.globals.arcviz_settings = arc.globals.arcviz_settings || {};
arc.globals.dashboard_width_min = 500;
arc.globals.dashboard_width_default = 1024;
arc.globals.dashboard_loading_warning_seconds_default = 20;

// Control whether page parameters are case sensitive
arc.globals.parameters_case_sensitive = true;

arc.globals.isInModal = function isInModal() {
  return ($('.super-modal').css('display') === 'block');
};

arc.globals_init = function() {
  const g = function() { return 'arc.globals_init'; };

  g.dashboard_width = function(_) {
    if (!arguments.length) {
      if (arc.globals.dashboard_width) {
        if (arc.globals.dashboard_width < arc.globals.dashboard_width_min) {
          return arc.globals.dashboard_width_min;
        }

        return arc.globals.dashboard_width;
      }

      return arc.globals.dashboard_width_default;
    }

    if (!$.isNumeric(_)) {
      return g;
    }

    arc.globals.dashboard_width = Number(_);
    return g;
  };

  g.dashboard_loading_message_in_viewmode = function(_) {
    if (!arguments.length) return arc.globals.dashboard_loading_message_in_viewmode;

    arc.globals.dashboard_loading_message_in_viewmode = _ === 'true';
    return g;
  };

  g.saveAwayGlobalQueryParameters = function() {
    const params = arc.misc.getQueryParameters();

    Object.keys(params).forEach((k) => {
      g.setUrlParam(k, params[k]);
    });

    return g;
  };

  g.setUrlParam = function(k, v) {
    arc.globals.urlparams[k] = v;
    return g;
  };

  g.getUrlParam = function(k) {
    return arc.globals.urlparams[k];
  };

  g.enable_IE_optimisations = function(_) {
    if (!arguments.length) return arc.globals.enableIEOptimisations;
    arc.globals.enableIEOptimisations = _;
    return g;
  };

  g.cdvSubMode = function(_) {
    if (!arguments.length) return arc.globals.cdvSubMode;
    arc.globals.cdvSubMode = _;
    return g;
  };

  g.enable_advanced_settings = function (_) {
    if (!arguments.length) return arc.globals.enable_advanced_settings;
    arc.globals.enable_advanced_settings = _;
    return g;
  };

  g.auto_refresh = function(_) {
    if (!arguments.length) return arc.globals.auto_refresh;
    arc.globals.auto_refresh = _;
    return g;
  };

  g.apply_custom_styles = function(_) {
    if (!arguments.length) return arc.globals.apply_custom_styles;
    arc.globals.apply_custom_styles = _;
    return g;
  };

  g.use_derived_data = function(_) {
    if (!arguments.length) return arc.globals.use_derived_data;
    arc.globals.use_derived_data = _;
    return g;
  };

  g.click_behavior_cb = function(_) {
    if (!arguments.length) return arc.globals.click_behavior_cb;
    arc.globals.click_behavior_cb = _;
    return g;
  };

  g.enable_direct_access = function(_) {
    if (!arguments.length) return arc.globals.enable_direct_access;
    arc.globals.enable_direct_access = _;
    return g;
  };

  g.enable_download_from_url = function(_) {
    if (!arguments.length) return arc.globals.enable_download_from_url;
    arc.globals.enable_download_from_url = _;
    return g;
  };

  g.enable_user_create = function(_) {
    if (!arguments.length) return arc.globals.enable_user_create;
    arc.globals.enable_user_create = _;
    return g;
  };

  g.data_extracts_target_tablename_prefix = function(_) {
    if (!arguments.length) return arc.globals.data_extracts_target_tablename_prefix;
    arc.globals.data_extracts_target_tablename_prefix = _;
    return g;
  };

  g.enable_visual_transitions = function(_) {
    if (!arguments.length) return arc.globals.enable_visual_transitions;
    arc.globals.enable_visual_transitions = _;
    return g;
  };

  g.max_trellis_displayed = function(_) {
    if (!arguments.length) return arc.globals.max_trellis_displayed;
    let value = 16;
    if (arc.misc.isNumeric(_)) {
      value = parseInt(_, 10);
      if (value < 0 || value > 999) {
        value = 16;
      }
    }

    arc.globals.max_trellis_displayed = value;
    return g;
  };

  g.enable_explore_interfaces = function(_) {
    if (!arguments.length) return arc.globals.enable_explore_interfaces;
    arc.globals.enable_explore_interfaces = _;

    if (!arc.globals.enable_explore_interfaces) {
      $('.explore').addClass('arc-invisible');
      $('.explore-icon').addClass('arc-invisible');
    }

    return g;
  };

  g.enable_custom_styles = function(_) {
    if (!arguments.length) return arc.globals.enable_custom_styles;
    arc.globals.enable_custom_styles = _;
    if (arc.globals.enable_custom_styles === false) {
      // If custom styles are disabled site-wide, do not apply them
      arc.globals.apply_custom_styles = false;
    }

    return g;
  };

  g.enable_thumbnails = function(_) {
    if (!arguments.length) return arc.globals.enable_thumbnails;
    arc.globals.enable_thumbnails = _;
    return g;
  };

  g.chromium_pdf_available = function(_) {
    if (!arguments.length) return arc.globals.chromium_pdf_available;
    arc.globals.chromium_pdf_available = _;
    return g;
  };

  g.email_enabled = function(_) {
    if (!arguments.length) return arc.globals.email_enabled;
    arc.globals.email_enabled = _;
    return g;
  };

  g.jira_enabled = function(_) {
    if (!arguments.length) return arc.globals.jira_enabled;
    arc.globals.jira_enabled = _;
    return g;
  };

  g.webhook_enabled = function(_) {
    if (!arguments.length) return arc.globals.webhook_enabled;
    arc.globals.webhook_enabled = _;
    return g;
  };

  g.enable_tooltip_highlights = function(_) {
    if (!arguments.length) return arc.globals.enable_tooltip_highlights;
    arc.globals.enable_tooltip_highlights = _;
    return g;
  };

  g.enable_script_parameters = function(_) {
    if (!arguments.length) return arc.globals.enable_script_parameters;
    arc.globals.enable_script_parameters = _;
    return g;
  };

  g.allow_png_download_default = function(_) {
    if (!arguments.length) return arc.globals.allow_png_download_default;
    arc.globals.allow_png_download_default = _;
    return g;
  };

  g.allow_csv_download_default = function(_) {
    if (!arguments.length) return arc.globals.allow_csv_download_default;
    arc.globals.allow_csv_download_default = _;
    return g;
  };

  g.is_edit = function(_) {
    if (!arguments.length) return arc.globals.is_edit;
    arc.globals.is_edit = _;
    return g;
  };

  g.is_dashboard = function(_) {
    if (!arguments.length) return arc.globals.is_dashboard;
    arc.globals.is_dashboard = _;
    return g;
  };

  g.server_platform = function(_) {
    if (!arguments.length) return arc.globals.server_platform;
    arc.globals.server_platform = _;
    return g;
  };

  g.username = function(_) {
    if (!arguments.length) return arc.globals.username;
    arc.globals.username = _;
    return g;
  };

  g.is_superuser = function(_) {
    if (!arguments.length) return arc.globals.is_superuser;
    arc.globals.is_superuser = _;
    return g;
  };

  g.send_message_params = function(_) {
    if (!arguments.length) return arc.globals.send_message_params;
    arc.globals.send_message_params = _;
    return g;
  };

  g.can_user_explore_any_dataconnections = function(_) {
    if (!arguments.length) return arc.globals.can_user_explore_any_dataconnections;
    arc.globals.can_user_explore_any_dataconnections = _;
    return g;
  };

  g.embedded = function(_) {
    if (!arguments.length) return arc.globals.embedded;
    arc.globals.embedded = _;
    return g;
  };

  g.email_addresses = function(_) {
    if (!arguments.length) return arc.globals.email_addresses;
    arc.globals.email_addresses = JSON.parse(_);
    return g;
  };

  g.email_prefix = function(_) {
    if (!arguments.length) return arc.globals.email_prefix;
    arc.globals.email_prefix = _;
    return g;
  };

  g.csrf_token = function(_) {
    if (!arguments.length) return arc.globals.csrf_token;
    arc.globals.csrf_token = _;
    return g;
  };

  g.everyone_group = function(_) {
    if (!arguments.length) return arc.globals.perms.everyone_group;
    arc.globals.perms.everyone_group = _;
    return g;
  };

  g.permdefs = function(_) {
    if (!arguments.length) return arc.globals.perms.permdefs;
    arc.globals.perms.permdefs = JSON.parse(_ || '{}');
    return g;
  };

  g.permlists = function(_) {
    if (!arguments.length) return arc.globals.perms.permlists;
    arc.globals.perms.permlists = JSON.parse(_ || '{}');
    return g;
  };

  g.sysperms = function(_) {
    if (!arguments.length) return arc.globals.perms.sysperms;
    arc.globals.perms.sysperms = JSON.parse(_ || '[]');
    return g;
  };

  g.arcviz_settings = function(_) {
    if (!arguments.length) return arc.globals.arcviz_settings;
    arc.globals.arcviz_settings = JSON.parse(_ || '{}');
    return g;
  };

  g.site_hide_refresh_button_from_connection_explorer = function(_) {
    if (!arguments.length) return arc.globals.site_hide_refresh_button_from_connection_explorer;
    arc.globals.site_hide_refresh_button_from_connection_explorer = _;
    return g;
  };

  g.site_enable_column_rename = function(_) {
    if (!arguments.length) return arc.globals.site_enable_column_rename;
    arc.globals.site_enable_column_rename = _;
    return g;
  };

  g.site_enable_app_acceleration = function(_) {
    if (!arguments.length) return arc.globals.site_enable_app_acceleration;
    arc.globals.site_enable_app_acceleration = _;
    return g;
  };

  g.google_api_id = function(_) {
    if (!arguments.length) return arc.globals.app_settings.google_api_id;
    arc.globals.app_settings.google_api_id = _;
    return g;
  };

  g.google_client_id = function(_) {
    if (!arguments.length) return arc.globals.app_settings.google_client_id;
    arc.globals.app_settings.google_client_id = _;
    return g;
  };

  g.mapbox_access_token = function(_) {
    if (!arguments.length) return arc.globals.app_settings.mapbox_access_token;
    arc.globals.app_settings.mapbox_access_token = _;
    return g;
  };

  g.local_mapbox_server = function(_) {
    if (!arguments.length) return arc.globals.app_settings.local_mapbox_server;
    arc.globals.app_settings.local_mapbox_server = _;
    return g;
  };

  g.maximumCanvasWidth = function(_) {
    if (!arguments.length) return arc.globals.app_settings.maximumCanvasWidth;
    arc.globals.app_settings.maximumCanvasWidth = _;
    return g;
  };

  g.jserrors_enabled = function(_) {
    if (!arguments.length) return arc.globals.jserrors_enabled;
    arc.globals.jserrors_enabled = _;
    $.migrateMute = !arc.globals.jserrors_enabled;
    return g;
  };

  g.page_profile_enabled = function(_) {
    if (!arguments.length) return arc.globals.page_profile_enabled;
    arc.globals.page_profile_enabled = _;
    return g;
  };

  g.page_profile_appview_enabled = function(_) {
    if (!arguments.length) return arc.globals.page_profile_appview_enabled;
    arc.globals.page_profile_appview_enabled = _;
    return g;
  };

  g.visual_max_rows = function(_) {
    if (!arguments.length) return arc.globals.visual_max_rows;
    arc.globals.visual_max_rows = Number(_);
    return g;
  };

  g.download_maxrows = function(_) {
    if (!arguments.length) return arc.globals.download_maxrows;
    arc.globals.download_maxrows = Number(_);
    return g;
  };

  g.embed_url_prefix = function(_) {
    if (!arguments.length) return arc.globals.embed_url_prefix;
    arc.globals.embed_url_prefix = _;
    return g;
  };

  g.bulkMaxEntries = function(_) {
    if (!arguments.length) return arc.globals.app_settings.bulkMaxEntries;
    arc.globals.app_settings.bulkMaxEntries = _;
    return g;
  };

  g.bulkMaxInterval = function(_) {
    if (!arguments.length) return arc.globals.app_settings.bulkMaxInterval;
    arc.globals.app_settings.bulkMaxInterval = _;
    return g;
  };

  g.bulkMaxRetention = function(_) {
    if (!arguments.length) return arc.globals.app_settings.bulkMaxRetention;
    arc.globals.app_settings.bulkMaxRetention = _;
    return g;
  };

  g.http_dataconnection_id = function(_) {
    if (!arguments.length) return arc.globals.http_dataconnection_id;
    arc.globals.http_dataconnection_id = _;
    return g;
  };

  g.unnamedVersionDeletionInterval = function(_) {
    if (!arguments.length) return arc.globals.app_settings.unnamedVersionDeletionInterval;
    arc.globals.app_settings.unnamedVersionDeletionInterval = _;
    return g;
  };

  g.unnamedVersionMaxRetention = function(_) {
    if (!arguments.length) return arc.globals.app_settings.unnamedVersionMaxRetention;
    arc.globals.app_settings.unnamedVersionMaxRetention = _;
    return g;
  };

  return g;
};
}());
