import arc from 'arc';

const getProfileID = (settingID) => {
  let profileID = '';

  if (settingID === 'embedding-profile') {
    profileID = 'embeddings';
  } else if (settingID === 'completion-profile') {
      profileID = 'completion';
  }

  return profileID;
};

export const paintProfileSetting = (tab_d3_sel, setting, setProfileChangeSub, lib, profiles) => {
  const settingValue = setting.value();
  const settingID = setting.id();
  const div_id = setting.div_id();
  const domID = `${div_id}-${settingID}`;
  const profileID = getProfileID(settingID);

  const settingDiv = tab_d3_sel.append('div')
      .attr('id', div_id)
      .attr('class', `${settingID} mb-2`);

  const labelWrapper = settingDiv.append('div')
      .attr('class', 'label-wrapper')
      .style('display', 'flex')
      .style('justify-content', 'space-between');

  labelWrapper.append('label').text('Selected Profile:');

  const $changeLink = $('<a>')
      .attr('id', domID)
      .attr('href', '#')
      .addClass('change-link')
      .text('Change')
      .appendTo(labelWrapper);

  const $inputField = $('<input>')
    .attr('type', 'text')
    .addClass('form-control form-control-sm')
    .attr('disabled', true)
    .attr('value', profiles.profileSettings[settingValue].label)
    .appendTo(settingDiv);

  const { profileSettings } = profiles;

  const modalClass = `change-ai-profile-modal-${settingID}`;

  // remove old modals first
  $(`div.${modalClass}`).remove();

  const modalID = `ai-${domID}`;
  const $modalDiv = $(`<div id="${modalID}" class="${modalClass}">`);

  $('body').append($modalDiv);

  const { events, ofType } = arc.frontend.legacyBridge;

  const subscription = events.pipe(ofType(modalID)).subscribe(({ payload }) => {
    setting.value(payload.value);
    $inputField.val(payload.label);

    lib.refresh(true);
    lib.page_needs_save();
  });

  setProfileChangeSub(subscription);

  $changeLink.offOn('click.openChangeAiModal', () => {
    arc.frontend.renderReactComponent(
      modalID,
      arc.frontend.ChangeAiProfileModal,
      {
        modalID,
        profileID,
        profiles: profileSettings,
        currentProfile: setting.value(),
      },
    );
  });

  return setting;
};

export const checkIfActiveProfileExists = (setting, { defaultProf, profileSettings }, lib) => {
  const activeProfile = setting.value();
  const settingID = setting.id();
  const profileID = getProfileID(settingID);

  // in case the active profile has been deleted
  if (
    !Object.keys(profileSettings).some((profileKey) => profileKey === activeProfile) ||
    !activeProfile) {
      let settingVal;

      if (defaultProf) {
        settingVal = defaultProf.value;
      } else {
        const [key] = Object.entries(profileSettings)[0];
        settingVal = key;
      }

      setting.value(settingVal);

      lib.refresh(true);
      lib.page_needs_save();
      lib.alertMessage(`The previously active ${profileID} profile has been deleted. The default profile has been restored.`);
  }
};
