import { saveAs } from 'file-saver';
import FontFaceObserver from 'fontfaceobserver';
import XLSX from 'xlsx/dist/xlsx.full.min';
import { formatDialect, sql } from 'sql-formatter';

window.arc = window.arc || {};

(function() {
'use strict';

arc.misc = arc.misc || {};

function postMessage(url) {
  window.top.postMessage({
    urlUpdate: url,
  }, '*');
}

arc.misc.isNumeric = function(obj) {
  const type = typeof (obj);

  // eslint-disable-next-line no-restricted-globals
  return (type === 'number' || type === 'string') && !isNaN(obj - parseFloat(obj));
};

// Don't allow back navigation
arc.misc.replaceState = function(url, stateObj = null) {
  postMessage(url);

  if (window.history.replaceState) {
    window.history.replaceState(stateObj, null, url);
  } else {
    window.open(url, '_self');
  }
};

// Allow user to navigate back using back button
arc.misc.pushState = function(url, stateObj = null) {
  postMessage(url);

  if (window.history.pushState) {
    window.history.pushState(stateObj, null, url);
  } else {
    window.open(url, '_self');
  }
};

arc.misc.totalColumnName = 'Total';
arc.misc.relativeFilterSuffix = '.rel';

arc.misc.createNewDashboardWithSql = function(autocompleteEditor, dataConnectionId) {
  const sqlText = autocompleteEditor.getSession().getValue();
  const encodedSqlText = encodeURIComponent(sqlText);
  window.location.href = `${
  arc.frontend.globals.urls.reportBuildUrl
    }?report_type=dashboard&dataconnection_id=${dataConnectionId}&query=${encodedSqlText}`;
};

arc.misc.saveSQLAsDataset = function(autocompleteEditor, dataConnectionId, dataconnectionName) {
  const sqlText = encodeURIComponent(autocompleteEditor.getSession().getValue());
  window.location.href = `${
    arc.frontend.globals.urls.dataUrl
    }?&dataconnection_id=${dataConnectionId}&dataconnection_name=${dataconnectionName}&sqlquery=${sqlText}#dataset-modal`;
};

arc.misc.isInSuperModal = function() {
  const supermodalDisplay = $('.super-modal').css('display');
  return (supermodalDisplay && supermodalDisplay !== 'none');
};

// Used to determine if we can add visual to current dashboard for search
arc.misc.isEditDashboard = function() {
  if (arc.globals.ms_dashboard) {
    const dashboardPerm = arc.globals.ms_dashboard.dashboard_perm();
    const dashboardId = arc.globals.ms_dashboard.dashboard_id();
    return arc.globals.is_edit &&
           (arc.misc.hasPerm('ds_appedit', dashboardPerm) || dashboardId === undefined);
  } else {
    return false;
  }
};

arc.misc.isDataInSheetedFormat = function(dashboard_data) {
  return dashboard_data &&
         dashboard_data.dashboard_sheets &&
         dashboard_data.dashboard_sheets.length;
};

arc.misc.hasScreenshot = function(imageUrl) {
  return !!imageUrl && imageUrl.startsWith(arc.frontend.globals.urls.snapshotUrl);
};

arc.misc.extend = function (baseObject, addOnObject) {
  let newObject = {};
  if (Array.isArray(addOnObject)) {
    newObject = [];
  }

  if (typeof addOnObject !== 'object' || addOnObject === null) {
    // typeof null === 'object'
    return addOnObject;
  }

  Object.keys(baseObject).map(key => newObject[key] = duplicate(baseObject[key]));
  Object.keys(addOnObject).map(key => newObject[key] = duplicate(addOnObject[key]));
  return newObject;

  function duplicate(value) {
    if (typeof value !== 'object') {
      return value;
    }

    return arc.misc.extend({}, value);
  }
};

arc.misc.isMac = function() {
  return (navigator.platform.indexOf('Mac') > -1);
};

arc.misc.copyToClipboard = (str) => navigator.clipboard.writeText(str)
  .catch(() => {
    const el = document.createElement('textarea');
    el.value = str;
    el.setAttribute('readonly', '');
    el.style.position = 'absolute';
    el.style.left = '-9999px';
    document.body.appendChild(el);
    el.select();
    document.execCommand('copy');
    document.body.removeChild(el);
  });


arc.misc.dashboard_id = function () {
  return (arc.globals.ms_dashboard && arc.globals.ms_dashboard.dashboard_id()) || '';
};

// Used in dashboard and workspaces to update old report items
arc.misc.updateReports = function({ oldReport, oldId, newId, removeSharedIcon }) {
  oldReport.report_id(newId);
  oldReport.shared_visual_dashboards(undefined);

  // now if in dashboard, need to update all_reports within reports so can getReport
  if (arc.globals.reports) {
    arc.globals.reports.removeReport(oldId);
    arc.globals.reports.setReport(oldReport, newId);

    // change id on gridster element if it exists on same sheet
    // because it's used when saving
    if (arc.globals.dashboard) {
      const $widgetEl = $(`[data-arc-reportid=${oldId}]`);
      if ($widgetEl.length) {
        const oldWIdgetId = $widgetEl.attr('id'),
              newWidgetId = oldWIdgetId.replace(oldId, newId);
        $widgetEl.attr('id', newWidgetId);

        // this probably isn't needed, but for consistency
        $widgetEl.attr('data-arc-reportid', newId);

        if (removeSharedIcon) {
          $widgetEl.find('.linked-visual-icon')
                   .addClass('arc-invisible');
        }
      }
    } else if (arc.globals.singleVisualBuilder) {
      // if not in dashboard, update displayapp app_id
      arc.globals.singleVisualBuilder.displayapp().app_id(newId);
    }
  }
};

arc.misc.dedupURLParams = function (data, baseURL) {
  // eslint-disable-next-line no-restricted-globals
  baseURL = baseURL || location.href.split('?')[0];
  let query = location.search.replace(/^\?/, '');
  let queryStart = '';

  // If url already has query, figure out how much of it to keep and append to baseURL
  let paramConnector = '?';
  if (query) {
    // Filter out param and scope that already exists on page
    // Also remove sheet parameter if it exists.
    /*
      Test Cases:
        http://localhost:8000?sheet=1&alpha=beta
        http://localhost:8000?alpha=beta&sheet=1
        http://localhost:8000?sheet=1&alpha=beta&sheet=2
        http://localhost:8000?alpha=beta&alpha_sheet=1&sheet=2
    */
    const paramRegex = data.reduce((finalString, param) => {
      const paramBeginning = param.split('=')[0];
      return `${finalString}|^${paramBeginning}=`;
    }, '^sheet=');

    const alreadyExistingQueries = new RegExp(paramRegex);

    query = query.split('&')
                 .filter(kv => !kv.match(alreadyExistingQueries));

    // Append filtered queries to end of baseURL
    if (query.length) {
      paramConnector = '&';
      queryStart = `?${query.join('&')}`;
    } else if (data.length) {
      paramConnector = '?';
    } else {
      paramConnector = '';
    }
  }

  return `${baseURL}${queryStart}${paramConnector}${data.join('&')}`;
};

arc.misc.getURLParams = function (resultArray, params, dashboard, format) {
  if (!dashboard) {
    dashboard = arc.globals.dashboard;
  }

  if (arc.globals.appgroup) {
    const currentID = (dashboard && dashboard.dashboard_id()) ||
                      arc.globals.reports.getPrimaryReport().report_id();
    resultArray.push(`initialapp=${currentID}`);
  }

  if (dashboard && dashboard.sheet_id()) {
    resultArray.push(`sheet=${dashboard.sheet_id()}`);
  } else if (format !== 'pdf') {
    resultArray.push('sheet=1');
  }

  const allScopes = [];
  const filterNames = new Set();
  let allParamsHash = {};
  params.forEach(param => allParamsHash[param.key] = true);
  params.forEach(function(param) {
    let unencodedValue;
    if (param.value_list) {
      unencodedValue = arc.params.format_list(
          param.value_list || [], param.valuetype, false, 'url',
          arc.params.isMultiSelectStringParam(param.key, allParamsHash, param.valuetype)
        ).join(', ');
    } else {
      // TODO: Remove all scenarios for which value_list doesn't exist
      unencodedValue = param.value;
    }

    if (param.key && unencodedValue !== '') {
      resultArray.push(
        `param.${encodeURIComponent(param.key)}=${encodeURIComponent(unencodedValue)}`
      );
    }

    if (param.scope && !filterNames.has(param.scope.filterName)) {
      if (!param.scope.filterId && param.source_visual_id) {
        param.scope.filterId = param.source_visual_id;
      }

      allScopes.push(param.scope);
      filterNames.add(param.scope.filterName);
    }
  });

  if (allScopes.length) {
    resultArray.push(`scope=${encodeURIComponent(JSON.stringify(allScopes))}`);
  }
  return resultArray;
};

/*
 * If we open a modal on click, then on multiple clicks, we would open the modal
 * multiple times. This could be a problem in some cases.
 *
 * The function below, when passed in a modal element, returns true if that modal
 * is currently active, or in the process of being shown.
 */
arc.misc.modalShown = function($modal) {
  const modalData = $modal.data('bs.modal') || {};
  return modalData._isShown;
};

/*
 * Display informational message within a modal. Caller specifies a title for the
 * modal and an HTML message to the user.
 */
arc.misc.infoModal = function(config) {
  let $m = $('#info-modal');
  if (arc.misc.modalShown($m)) {
    return;
  }

  $m.find('.modal-title')
    .text(config.title);
  $m.find('.modal-body')
    .html(config.contentsHtml);

  $m.modal('show');
};

arc.misc.getTimestampFilename = (name, extension) => {
  const timestamp = moment().format('YYYYMMDDHHmmss');
  const title = name.replace(/[\\/?*[\].]/g, '_'); // Replace invalid filename chars
  return extension ? `${title}_${timestamp}.${extension}` : `${title}_${timestamp}`;
};

arc.misc.daterangeToMinMax = function(v, timeformat) {
  const tsre1 = /^\s*(\d+-\d+-\d+(\s+\d+:\d+:\d+)?)\s*-\s*(\d+-\d+-\d+(\s+\d+:\d+:\d+)?)\s*$/;
  const m1 = tsre1.exec(v);

  let min = '';
  let max = '';
  if (m1) {
    min = moment(m1[1]).format(timeformat);
    max = moment(m1[3]).format(timeformat);
    return { min, max };
  }

  const tsre2 = /^\s*(\d+-\d+-\d+T\d+:\d+:\d+.?\d*Z)\s*-\s*(\d+-\d+-\d+T\d+:\d+:\d+.?\d*Z)\s*$/;
  const m2 = tsre2.exec(v);
  if (m2) {
    min = moment.utc(m2[1]).format(timeformat);
    max = moment.utc(m2[2]).format(timeformat);
  }

  return { min, max };
};

arc.misc.whereClauseFromMin = function(filter_column, min, minOperator, noBrackets) {
  var f = [];

  if (minOperator === undefined) {
    minOperator = '>=';
  }

  if (noBrackets) {
    f.push(`(${filter_column}`);
  } else {
    f.push('([' + filter_column + ']');
  }

  f.push(' ' + minOperator + ' ');
  f.push(min);
  f.push(')');

  return f.join('');
};

arc.misc.whereClauseFromMax = function(filter_column, max, maxOperator, noBrackets) {
  var f = [];

  if (maxOperator === undefined) {
    maxOperator = '<=';
  }

  if (noBrackets) {
    f.push(`(${filter_column}`);
  } else {
    f.push('([' + filter_column + ']');
  }

  f.push(' ' + maxOperator + ' ');
  f.push(max);
  f.push(')');

  return f.join('');
};

arc.misc.whereClauseFromMinMax = function(filter_column, min, max, minOperator,
    maxOperator, rawColName = false) {
  var f = [];

  if (minOperator) {
    f.push(arc.misc.whereClauseFromMin(filter_column, min, minOperator, rawColName));
  }

  if (maxOperator) {
    f.push(arc.misc.whereClauseFromMax(filter_column, max, maxOperator, rawColName));
  }

  if (f.length === 1) {
    return f[0];
  }

  return '(' + f[0] + ' AND ' + f[1] + ')';
};

arc.misc.escapeString = function (s) {
  if (typeof (s) === 'string') {
    // If s appears to already be quoted, then don't escape that part
    var beg = s[0];
    var end = s[s.length - 1];
    var mid;
    if (s.length >= 2 && beg === end) {
      mid = s.slice(1, s.length - 1);
      if (beg === '"') {
        // Also don't escape quote chars that appear to be already escaped
        // (i.e. are preceded by \)
        mid = mid.replace(/(^|[^\\])(["])/g, "$1\\$2");
      } else if (beg === "'") {
        mid = mid.replace(/(^|[^\\])(['])/g, '$1\\$2');
      }
    } else {
      mid = s;
      beg = '';
      end = '';
    }

    return beg + mid + end;
  }

  return s;
};

arc.misc.unquoteColumn = function(colname) {
  return arc.dataconnection.types.stripBrackets(colname);
};

arc.misc.quoteColumn = function(colname) {
  return arc.dataconnection.types.brackets(colname);
};

arc.misc.removeLimitFromSQL = function(query) {
  return query.replace(/\s+limit\s+\d+\s*$/i, '');
};

arc.misc.quoteOrCastColumnName = function(column, needToLower) {
  let columnExp = column.display_name();

  // If column is not string and needs to be made "upper", cast as string
  if (needToLower && column.attr('type') !== 'STRING') {
    const connectionType = column.table().dataset().dataconnection_type();
    columnExp = arc.dataconnection.castAsString(columnExp, connectionType);
  } else {
    columnExp = arc.misc.quoteColumn(columnExp);
  }

  return needToLower ? `lower(${columnExp})` : columnExp;
};

arc.misc.approximateClauseFromValues = function(column, filter_data, filter_exclude) {
  const colType = column.attr('type'),
        notNumeric = !arc.report.measure_types[colType];

  let dataconnection_type = column.table().dataset().dataconnection_type();
  if (!dataconnection_type || !arc.dataconnection.types[dataconnection_type]) {
    dataconnection_type = 'impala';
  }

  const conn = arc.dataconnection.types[dataconnection_type](),
        quoteStart = conn.str_quote_start,
        quoteEnd = conn.str_quote_end;

  let colname;
  if (notNumeric) {
    colname = arc.misc.quoteOrCastColumnName(column, true);
  } else {
    colname = arc.misc.quoteColumn(column.display_name());
    filter_data = filter_data.filter(function(d) {
      if (isNaN(d)) {
        return false;
      }

      return true;
    });

    filter_data = filter_data.map(function(d) {
      return Number(d);
    });

    if (filter_data.length === 0) {
      return undefined;
    }
  }

  var f = [];
  f.push('(');

  filter_data.forEach(function(d, i) {
    if (notNumeric) {
      let orig = d.toLowerCase();
      d = orig.replace(/\*/g, "%");

      // We did not find any wildcards in the phrase being searched for
      // In that case, add in wildcards in the beginning and the end.
      if (d === orig) {
        d = '%' + d + '%';
      }

      f.push(colname);
      if (filter_exclude) {
        f.push(` not like ${quoteStart}`);
      } else {
        f.push(` like ${quoteStart}`);
      }

      f.push(d);
      f.push(quoteEnd);

      if (i !== filter_data.length - 1) {
        f.push(' OR ');
      }

      return;
    }

    f.push('(');
    f.push(colname);
    f.push(' = ');
    f.push(d);
    f.push(')');

    if (i !== filter_data.length - 1) {
      f.push(' OR ');
    }
  });

  f.push(')');

  return f.join('');
};

function whereClauseFromValuesKSQL(colname, filter_data, filter_exclude) {
  let f = [];

  filter_data.forEach(function(d, i) {
    f.push('(');
    f.push(colname);

    if (filter_exclude) {
      f.push(' != ');
    } else {
      f.push(' = ');
    }

    f.push(d);
    f.push(')');

    if (i < filter_data.length - 1) {
      if (filter_exclude) {
        f.push(' AND ');
      } else {
        f.push(' OR ');
      }
    }
  });

  return f.join('');
}

arc.misc.whereClauseFromValues = function(filter_column, filter_data,
    filter_exclude, bareColumnName, conn_type, coltype, filter_includeNulls) {
    const colname = bareColumnName ? arc.misc.quoteColumn(filter_column) : filter_column;

  filter_data = filter_data || [];
  filter_data = Array.isArray(filter_data) ? filter_data : [filter_data];

  if (conn_type === 'ksql') {
    return whereClauseFromValuesKSQL(colname, filter_data, filter_exclude);
  }

  filter_data = handleTimestampInFilterValues(filter_data, coltype, conn_type);

  const f = [colname];

  if (filter_exclude) {
    f.push(' not in (');
  } else {
    f.push(' in (');
  }

  let nullText = '';
  let nullJoin = '';
  const filterData = [];

  filter_data.forEach((d) => {
    if ((d !== 'NULL') && (d !== "'NULL'")) {
      filterData.push(d);
    } else {
      nullJoin = filter_exclude ? ' AND ' : ' OR ';
      nullText = filter_exclude ? `${colname} is not ${d}` : `${colname} is ${d}`;
    }
  });

  f.push(filterData.map((item) => (item === null ? 'NULL' : item)).join(', '));
  f.push(')');

  if (filterData.length !== 0 && nullText) {
    return `(${f.join('')}${nullJoin}${nullText})`;
  }

  if (nullText) {
    return nullText;
  }

  if (filterData.length === 0) {
    return filter_includeNulls?.toString() !== 'true' ? `(${arc.misc.quoteColumn(colname)} is not NULL)` : '';
  }

  let expr = f.join('');

  if (filter_includeNulls?.toString() === 'true') {
    expr = `((${expr}) or (${arc.misc.quoteColumn(colname)} is NULL))`;
  }

  return expr;
};

arc.misc.defaultRanges = function(isTime, dateRangeType) {
  var rawRanges;
  if (isTime && dateRangeType === 'quick_filters') {
    rawRanges =
      ['1M, NOW SUBTRACT 1 MINUTE, NOW',
       '5M, NOW SUBTRACT 5 MINUTES, NOW',
       '15M, NOW SUBTRACT 15 MINUTES, NOW',
       '1H, NOW SUBTRACT 1 HOUR, NOW',
       '6H, NOW SUBTRACT 6 HOURS, NOW',
       '12H, NOW SUBTRACT 12 HOURS, NOW',
       '24H, NOW SUBTRACT 24 HOURS, NOW',
      ];
  } else if (isTime) {
    rawRanges =
      ['Today, nTODAY STARTOF DAY, nTODAY ENDOF DAY',
       'Yesterday, nTODAY SUBTRACT 1 DAY STARTOF DAY, nTODAY SUBTRACT 1 DAY ENDOF DAY',
       'Last 7 Days, nTODAY SUBTRACT 6 DAYS STARTOF DAY, nTODAY ENDOF DAY',
       'Last Week, nTODAY SUBTRACT 1 WEEK STARTOF WEEK, nTODAY SUBTRACT 1 WEEK ENDOF WEEK',
       'Last 30 Days, nTODAY SUBTRACT 29 DAY STARTOF DAY, nTODAY ENDOF DAY',
       'This Month, nTODAY STARTOF MONTH, nTODAY ENDOF MONTH',
       'Last Month, nTODAY SUBTRACT 1 MONTH STARTOF MONTH, nTODAY SUBTRACT 1 MONTH ENDOF MONTH',
      ];
  } else {
    rawRanges =
      ['Today, TODAY STARTOF DAY, TODAY ENDOF DAY',
       'Yesterday, TODAY SUBTRACT 1 DAY STARTOF DAY, TODAY SUBTRACT 1 DAY ENDOF DAY',
       'Last 7 Days, TODAY SUBTRACT 6 DAYS STARTOF DAY, TODAY ENDOF DAY',
       'Last Week, TODAY SUBTRACT 1 WEEK STARTOF WEEK, TODAY SUBTRACT 1 WEEK ENDOF WEEK',
       'Last 30 Days, TODAY SUBTRACT 29 DAY STARTOF DAY, TODAY ENDOF DAY',
       'This Month, TODAY STARTOF MONTH, TODAY ENDOF MONTH',
       'Last Month, TODAY SUBTRACT 1 MONTH STARTOF MONTH, TODAY SUBTRACT 1 MONTH ENDOF MONTH',
      ];
  }

  var text = rawRanges.join('\n');
  return arc.misc.rangesFromText(text);
};

arc.misc.formatAsFileSize = function(n) {
  n = Number(n);
  if (n === 0) {
    return '0B';
  }

  const symbols = [' bytes', 'KB', 'MB', 'GB', 'TB', 'PB'];
  const log1024 = Math.floor(Math.log2(n) / 10);
  return `${Math.round(n / (Math.pow(1024, log1024)))} ${symbols[log1024]}`;
};

arc.misc.sizeDisplay = function(size, placeholder='TBD', estimated=false) {
  if (size === -1 || size === -2) {
    return placeholder;
  }

  const formattedSize = arc.misc.formatAsFileSize(size);
  return estimated ? `${formattedSize} (estimated)` : formattedSize;
};

arc.misc.formatReadableNumber = function(n) {
  n = Number(n);
  if (n === 0) {
    return '0';
  }

  const symbols = ['', 'K', 'M', 'B', 'T', 'P'];
  const log1000 = Math.floor(Math.log10(n) / 3);
  return `${Math.round(n / (Math.pow(1000, log1000)))}${symbols[log1000]}`;
};

// for taking lists of strings and making them more display-friendly
arc.misc.makeEnglishList = function(list) {
  let length = list.length;
  if (length <= 1) {
    return list[0];
  } else if (length === 2) {
    return list.join(' and ');
  } else {
    return `${list.slice(0, -1).join(', ')} and ${list[length - 1]}`;
  }
};

// new filters should be added to arc.report.noResultExcludeTypes too
arc.misc.isFilterVisual = function(vizType) {
  return (['picklist', 'daterange', 'range'].indexOf(vizType) >= 0);
};

function strictEval(s) {
  return eval(s);
}

arc.misc.rangesFromText = function(text) {
  var dateranges = arc.grammar._dateranges;
  var e;

  try {
    e = dateranges.parse(text);
  } catch (err) {
    console.log('Error parsing date ranges', text);
    return undefined;
  }

  var ranges = {};
  e.forEach(function(r) {
    ranges[r[0]] = [strictEval(r[1]), strictEval(r[2])];
  });

  return ranges;
};

/*
 * Any times we use to send to the backend must reflect the time of the data in
 * the backend, not the source browser. So always use UTC if a timezone is not
 * specified. If a module needs to use a different timezone from UTC, then it
 * needs to have a setting that the user can tweak to store the timezone of the
 * data in the backend, and then call this function with that specified timezone.
 */
arc.misc.dateToMoment = function(date, timezone) {
  if (!timezone) {
    timezone = 'UTC';
  }

  return moment.utc(date).tz(timezone);
};

arc.misc.columnIsDate = function(rows, colIdx) {
  var formatSearch = arc.dates.formatSearch();

  var dateConfig;

  rows.some(function(row, i) {
    // Only read through the first 100 rows to auto-detect dates
    // We want to read more than 1 row to handle NULLs and some
    // bad data, but we shouldn't be reading every row
    if (i > 100) {
      return true;
    }

    var possibleDate = row[colIdx];
    var c = formatSearch.identify(possibleDate);
    if (!c) {
      return false;
    }

    dateConfig = c;
    return true;
  });

  return dateConfig;
};

// Should remove the necessity for arc.arraysIdentical and arc.simpleDictionariesEqual
arc.misc.isIdentical = function(a, b) {
  if (typeof a !== 'object' || typeof b !== 'object' || !a || !b) {
    // NOTE: typeof null === 'object'
    // NOTE: typeof undefined === 'undefined'
    return a === b;
  }

  if (a instanceof Date) {
    if (!(b instanceof Date)) {
      return false;
    }

    return a.valueOf() === b.valueOf();
  }

  // Create arrays of property names
  var aProps = Object.getOwnPropertyNames(a);
  var bProps = Object.getOwnPropertyNames(b);

  // If number of properties is different, objects are not equivalent
  if (aProps.length !== bProps.length) {
    return false;
  }

  for (var i = 0; i < aProps.length; i++) {
    var propName = aProps[i];

    // If values of same property are not equal, objects are not equivalent
    if (!arc.misc.isIdentical(a[propName], b[propName])) {
      return false;
    }
  }

  // If we made it this far, objects
  // are considered equivalent
  return true;
};

/*
 * The functions below are a workaround gor jquery < 3.0 not being able
 * to support SVG objects for .addClass / .removeClass / .hasClass
 */
arc.misc.removeClass = function($sel, classname) {
  $sel.each(function() {
    var $this = $(this);
    var classes = $this.attr('class') || '';
    classes = classes.replace(classname, '');
    $this.attr('class', classes);
  });
};

arc.misc.addClass = function($sel, classname) {
  $sel.each(function() {
    var $this = $(this);
    var classes = $this.attr('class') || '';
    if (classes.indexOf(classname) < 0) {
      classes = classes + ' ' + classname;
      $this.attr('class', classes.trim());
    }
  });
};

arc.misc.hasClass = function($elem, classname) {
  var classes = $elem.attr('class') || '';
  return (classes.indexOf(classname) >= 0);
};

// Within $container, set the class to fontClass. Track and set class in
// dataAttributeName, and remove previously set font classes
arc.misc.setFont = function($container, dataAttributeName, fontClass) {
  var c = $container.attr(dataAttributeName);
  if (c === fontClass) {
    return;
  }

  if (c) {
    $container.removeClass(c);
    $container.attr(dataAttributeName, null);
  }

  if (fontClass) {
    $container.addClass(fontClass);
    $container.attr(dataAttributeName, fontClass);
  }
};

arc.misc.defaultAgg = function (columnType) {
  var lowercaseColumnType = columnType.toLowerCase(),
      result;
  if (lowercaseColumnType === 'boolean') {
    return 'count';
  } else if (lowercaseColumnType === 'string') {
    return 'max';
  } else if (lowercaseColumnType === 'timestamp') {
    return 'avg';
  }
  return 'sum';
};

arc.misc.isWindowFunction = function(expression) {
  const ret = (expression.match(/ over *\(.*\)/i) !== null);
  return ret;
};

arc.misc.paramsFromData = function(params_data) {
  var params = arc.params.tracker();

  if (params_data !== undefined) {
    params_data.forEach(function(param_data, i) {
      params.add(param_data);
    });
  }

  return params;
};

// when x_header, show
// when y_header, if no measure, show
// when y_header, but measure also is present, set to false
// when measure, but measure is not present, set to false
// when measure, and exact same filter is present, set to true
arc.misc.getHeaderSorter = function(config, data) {
  var pos = config.pos;
  var colname = config.colname;
  var sort_orders_asc = config.sort_orders_asc;
  var headerType = config.headerType;
  var columns = config.columns;
  var trellis = config.trellis;
  var col_idx = config.col_idx;

  var isMeasure = (headerType === 'measure');
  var col_sort_i = function (sort_order_asc, pos, isMeasure) {

    if (sort_order_asc === 'none') {
      return '';
    }

    var showFixed = false;
    var isYHeader = (headerType === 'y_header');
    var isXHeader = (headerType === 'x_header');
    var clickData = {};
    if (isMeasure) {
      var x_columns = trellis.x_trellis();
      var measureCount = trellis.data_columns().length;
      var x_data = measureCount > 0 ?
        trellis.unique_x_pos_to_values[Math.floor(col_idx / measureCount)] :
        trellis.unique_x_pos_to_values[col_idx];
      if (x_data) {
        x_data.forEach(function (d, i) {
          // x_data containe all the dimensional data, but we only want x columns
          if (i < x_columns.length) {
            clickData[x_columns[i].colname()] = d;
          }
        });
      }

      if (typeof sort_order_asc !== 'undefined') {
        var filters = $.extend({}, sort_orders_asc['arc-selected-params'] || {});
        var measureColName = filters['arc-measure-colname'];
        delete filters['arc-measure-colname'];
        if (measureColName === colname) {
          if (clickData && !Object.keys(clickData).some(function (key) {
              return filters[key] !== clickData[key];
            })) {
            showFixed = true;
          }
        }
      }

      clickData['arc-measure-colname'] = colname;
    } else if (!isMeasure) {
      if (isXHeader) {
        showFixed = true;
      } else if (isYHeader) {
        // if not sorting by agg then showFixed = true;
        // else if sorting by agg as well then showFixed = false;
        showFixed = trellis.data_columns().filter(function (col) {
            return typeof sort_orders_asc[col.colname()] !== 'undefined';
          }).length === 0;
      }
    } else {
      showFixed = false;
    }

    const classes = ['viz-col-sort'];
    const iconClasses = ['viz-sort-icon', 'fa', 'ml-1'];

    if (sort_order_asc == null ||
        sort_order_asc === arc.consts.SORT_OPTION_UNSORTED) {
      classes.push(arc.consts.SORT_OPTION_UNSORTED);
      iconClasses.push('fa-sort');
    } else if (sort_order_asc === true) {
      classes.push('asc');
      iconClasses.push('fa-sort-amount-asc');
    } else {
      classes.push('desc');
      iconClasses.push('fa-sort-amount-desc');
    }

    if (showFixed) {
      iconClasses.push('show-fixed');
    }

    if (data != null) {
      data = arc.misc.escapeHTML(data);
    }

    const sorter = [`<span
      class="${classes.join(' ')}"
      data-pos="${pos}"
      data-is-measure="${isMeasure}"
      data-click-info="${JSON.stringify(clickData)}"
    >`];
    sorter.push(data);
    sorter.push(`<i class="${iconClasses.join(' ')}"></i>`);
    sorter.push('</span>');

    return sorter.join('');
  };

  return col_sort_i(sort_orders_asc[colname], pos, isMeasure);
};

arc.misc.paramsToUrl = function(params, urlComponents) {
  params.forEach(function(m) {
    urlComponents.push('param.' + encodeURIComponent(m.key) + '=' + encodeURIComponent(m.value));
  });
};

arc.misc.navigateToApp = function(appId, params, lib) {
  if (arc.globals.appgroup) {
    var cb = arc.globals.click_behavior_cb;
    if (cb) {
      cb(undefined, undefined, undefined,
          undefined, lib, undefined, params);
    }

    arc.globals.appgroup.navigateToApp(appId);
  } else {
    // Navigate to the page for the specified app

    var l = [];
    arc.misc.paramsToUrl(params, l);
    var paramsUrl = '';
    if (l.length > 0) {
      paramsUrl = '?' + l.join('&');
    }

    const url = `${arc.frontend.globals.urls.viewDashboardUrl}/${appId}${paramsUrl}`;
    window.open(url, '_self');
  }
};

arc.misc.columnIsString = function(column) {
  let coltype = column.attr('type');
  let dataconnection_type = column.table().dataset().dataconnection_type();

  return arc.misc.isString(coltype, dataconnection_type);
};

function isTimestampHandlingNeeded(coltype, dataconnection_type) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toLowerCase();

  if (dataconnection_type === 'athena') {
    if (coltype === 'timestamp') {
      return true;
    }
  }

  return false;
}

function handleTimestampInFilterValues(filter_data, coltype, dataconnection_type) {
  if (!coltype) {
    return filter_data;
  }

  coltype = coltype.toLowerCase();

  if (dataconnection_type === 'athena') {
    if (coltype === 'timestamp') {
      return filter_data.map(function(d) {
        if (d.substring(0, 9) === 'timestamp') {
          // permit multiple calls to this transformation function
          return d;
        }

        return `timestamp ${d}`;
      });
    }
  }

  return filter_data;
}

const stringTypes = new Set(['string', 'date', 'timestamp', '253', 'unknown']);

arc.misc.isString = function(coltype, dataconnection_type) {
  coltype = coltype.toLowerCase();

  if (!dataconnection_type) {
    return stringTypes.has(coltype);
  }

  if (!dataconnection_type || !arc.dataconnection.types[dataconnection_type]) {
    dataconnection_type = 'impala';
  }

  const conn = arc.dataconnection.types[dataconnection_type]();
  return conn.is_string(coltype);
};

arc.misc.quoteValue = function(value, coltype, dataconnection_type) {
  if (arc.misc.isString(coltype, dataconnection_type)) {
    dataconnection_type = dataconnection_type || 'base';
    value = arc.report.shelf.parse_filter_value(value, coltype, false, dataconnection_type);
    return value;
  }

  return value;
};

arc.misc.inClause = function(colname, coltype, out_data, dataconnection_type) {
  colname = arc.misc.quoteColumn(colname);
  out_data = arc.misc.quoteValue(out_data, coltype, dataconnection_type);
  return colname + ' in (' + out_data + ')';
};

// Expect "1800" to return false, while 1800 to return true
arc.misc.isNumberType = function (val) {
  return typeof val === 'number';
};

arc.misc.filterExpression = function(colname, coltype, params, conn_type, visual_id) {
  const startend = {};
  const multiselect_params = {};
  let inOrEqual = ' in (';
  let predicateClose = ')';
  let expr;
  let includeNulls;

  coltype = coltype.toLowerCase();

  if (conn_type === 'ksql') {
    inOrEqual = ' = ';
    predicateClose = '';
  }

  params.forEach((param) => {
    if ((param.key === `${colname}.config`) && (param.value !== undefined)) {
      const relVals = JSON.parse(param.value)[arc.misc.relativeFilterSuffix];
      if (relVals && relVals.anchorType === 'current' && relVals.datetimeFormat) {
        arc.filter.helpers.fetchRelative(relVals, relVals.datetimeFormat);
        startend.start = relVals.startDate;
        startend.end = relVals.endDate;
      }
    }

    if (!param.source_pivot && param.source_visual_id && param.source_visual_id === visual_id) {
      return;
    }

    if (colname === param.key) {
      expr = arc.misc.quoteColumn(colname);

      if (isTimestampHandlingNeeded(coltype, conn_type)) {
        let filter_data;
        if (param.value !== undefined) {
          filter_data = [arc.dataconnection.types.quoteString(conn_type, param.value)];
        } else {
          filter_data = param.filter_data;
        }

        expr = arc.misc.whereClauseFromValues(expr, filter_data, false, false, conn_type, coltype);

        return;
      }

      if (param.value !== undefined) {
          expr = expr + inOrEqual + arc.report.shelf.parse_filter_value(
                     param.value, param.valuetype || coltype || ''
               ) + predicateClose;
      } else if (param.filter_data) {
        if (param.filter_data.length) {
          const combinedValue = param.filter_data.map((d) => arc.report.shelf.parse_filter_value(d, param.valuetype || coltype || '')).join(', ');

          expr = `${expr} in (${combinedValue})`;
        }
      }

      return;
    }

    if (param.key === `${colname}.start`) {
      startend.start = param.value;
      return;
    }

    if (param.key === `${colname}.end`) {
      startend.end = param.value;
      return;
    }

    if (param.key === `${colname}.exclude`) {
      multiselect_params.exclude = param.value;
      return;
    }

    if (param.key === `${colname}.include_nulls`) {
      includeNulls = param.value;
      return;
    }

    if (param.key === `${colname}.data`) {
      // value is ', '.join(value_list)
      if (param.value_list) {
        // TODO: Move below string conversion to arc.misc.whereClauseFromValues
        multiselect_params.data = arc.params.format_list(param.value_list, coltype);
      } else if (param.value) {
        // If parameter is from url,
        // there's no param.value_list, it needs to be parsed now
        param.value_list = param.value.split(',');
        multiselect_params.data = param.value_list;
      }
    }
  });

  if (expr) {
    if (includeNulls === 'true') {
      expr = `${expr} or (${arc.misc.quoteColumn(colname)} is NULL)`;
    }

    return expr;
  }

  if (startend.start !== undefined && startend.end !== undefined) {
    startend.include_nulls = includeNulls;

    const rangeExpr =
      arc.filter.helpers.rangeExpression(colname, coltype, conn_type, startend);
    if (rangeExpr) {
      return rangeExpr;
    }
  }

  if ('exclude' in multiselect_params && 'data' in multiselect_params) {
    return arc.misc.whereClauseFromValues(
      colname,
      multiselect_params.data,
      (String(multiselect_params.exclude).toLowerCase() === 'not in'),
      true,
      conn_type,
      undefined,
      includeNulls
    );
  }

  if (includeNulls === 'false') return `(${arc.misc.quoteColumn(colname)} is not NULL)`;
};

const timeformatDate = 'YYYY-MM-DD';
const timeformatDateTime = 'YYYY-MM-DD HH:mm:ss';

function defaultTimestamp(start) {
  if ((typeof (start) === typeof ('')) && (start.length === 10)) {
    return timeformatDate;
  }

  return timeformatDateTime;
}

arc.misc.timestampEqual = function(connectionType, colname, coltype, start, timeformat) {
  colname = arc.misc.quoteColumn(colname);
  timeformat = timeformat || defaultTimestamp(start);

  if (connectionType === 'teradata') {
    return colname + " = DATE '" + start + "'";
  }

  if (connectionType === 'oracle') {
    return colname + " = to_date('" + start + "', '" + timeformat + "')";
  }

  if (connectionType === 'mssqlserver') {
    return colname + " = convert(DATETIME, '" + start + "')";
  }

  const quotedStart = arc.misc.quoteValue(start, 'string', connectionType);

  return colname + ' = ' + quotedStart;
};

arc.misc.rangeExpression = function(connectionType, colname, coltype, start, end, include_nulls) {
  colname = arc.misc.quoteColumn(colname);
  let expr = `(${colname} >= ${start} AND ${colname} <= ${end})`;

  if (include_nulls === 'true') {
    expr = `((${expr}) or (${colname} is NULL))`;
  }

  return expr;
};

arc.misc.timestampRange = function(connectionType, colname, coltype, start, end,
                                   timeformat, include_nulls) {
  if (!connectionType || !arc.dataconnection.types[connectionType]) {
    connectionType = 'impala';
  }

  const conn = arc.dataconnection.types[connectionType]();
  colname = arc.misc.quoteColumn(colname);

  const quotedStart = arc.misc.quoteValue(start, 'string', connectionType);
  const quotedEnd = arc.misc.quoteValue(end, 'string', connectionType);

  function base() {
    timeformat = timeformat || defaultTimestamp(start);

    if (timeformat === timeformatDate) {
      if (conn.to_date) {
        colname = conn.to_date(colname);
      }
    }

    return conn.timestampRangeExpr(colname, coltype, quotedStart, quotedEnd, timeformat);
  }

  // Here we expect the base expression excludes NULLs, so we only update
  // the expression when we need to include NULLs.  Also, this assumes the
  // syntax to exclude NULLs is the same for all data connections.
  let expr = base();
  if (include_nulls === 'true') {
    expr = `((${expr}) or (${colname} is NULL))`;
  }
  return expr;
};

arc.misc.getDisplayFormattedValue = function (c={}) {
  if (!c.args) {
    c.args = [];
  }

  try {
    const formatter = c.formatter ? c.formatter : (c.config && c.config[c.formatterName]);
    if (!formatter) {
      return arc.misc.default_formatter(c.value);
    }

    const  formattedValue = formatter(c.value, ...c.args);
    if (arc.misc.formattingIsNaN(formattedValue)) {
      throw c.formatterName + ' display format making ' + c.value + ' NaN';
    } else {
      return formattedValue;
    }
  } catch (e) {
    console.log(e);
    if (c.config) {
      c.config[c.formatterName] = undefined;
    } else {
      c.formatter = undefined;
    }

    return arc.misc.default_formatter(c.value);
  }
};

arc.misc.formattedDisplay = function(displayFormat, value, ...args) {
  return arc.misc.getDisplayFormattedValue({formatter: displayFormat, value: value, args: args});
};

arc.misc.addTooltip = function(textl, key, value, coltype, formatter) {
  textl.push(arc.misc.escapeHTML(key));
  let formattedValue = '';
  textl.push(': <b>');
  if (formatter) {
    formattedValue = arc.misc.formattedDisplay(formatter, value);
  } else {
    if (isNaN(value)) {
      formattedValue = arc.misc.d_string(value);
    } else {
      formattedValue = value;
    }
  }
  textl.push(arc.misc.escapeHTML(formattedValue));
  textl.push('</b>');
};

arc.misc.text_from_column = function(row, col) {
  return {
    key: col.colname(),
    value: arc.misc.d_string(row[col.colindex()]),
    formatter: col.coldisplayformat(),
  };
};

class OBJ {

  // Creates copy of nested object or array
  static sliceRecursive(obj) {
    if (!obj || typeof obj !== 'object') {
      return obj;
    } else if (Array.isArray(obj)) {
      return obj.map(OBJ.sliceRecursive);
    } else {
      return Object.keys(obj)
                    .map(key => ({ [key]: OBJ.sliceRecursive(obj[key])}))
                    .reduce((a, b) => Object.assign(a, b), {});
    }
  }

  // overwrites collisions: { 1:2, 2:3, 4:3 } => { 2:1, 3:4 }
  static invert(obj) {
    return Object.keys(obj).map(k => ({ [obj[k]]: k })).reduce((a, b) => Object.assign(a, b));
  }

}

arc.misc.OBJ = OBJ;

// Add or remove character in this object to include/exclude from escaping
// escaping spaces omitted to avoid problems with escaping input into html attributes
const esc = {
  // ' ': '&#32;',
  '&': '&amp;',
  '<': '&lt;',
  '>': '&gt;',
  '"': '&quot;',
  "'": '&apos;',
  ';': '&#59;',
  '(': '&#40;',
  ')': '&#41;',
  '[': '&#91;',
  ']': '&#93;',
};

const unesc = arc.misc.OBJ.invert(esc);

const escTags = {
  '<': '&lt;',
  '>': '&gt;',
};

const unescTags = arc.misc.OBJ.invert(escTags);

arc.misc.removeHTML = function (text) {
  return text.replace(/(<([^>]+)>)/ig, '');
};

arc.misc.escapeHTMLTags = function(text) {
  return arc.misc.escapeHTML(text, escTags);
};

arc.misc.escapeHTML = function(text, tags=esc) {
  if (typeof text !== 'string') {
    return text;
  }

  let escaped = [];
  for (var i = 0; i < text.length; i++) {
    const char = text[i];
    escaped.push(tags[char] || char);
  }
  return escaped.join('');
};

arc.misc.unescapeHTMLTags = function(text) {
  return arc.misc.unescapeHTML(text, unescTags);
};

arc.misc.unescapeHTML = function(text, inverseTags=unesc) {
  if (typeof text !== 'string') {
    return text;
  }

  let inEscSeq = false;
  let escSeq = '';
  let output = '';
  for (var i = 0; i < text.length; i++) {
    const char = text[i];
    if (char === '&') {
      inEscSeq = true;
    }
    if (inEscSeq) {
      escSeq += char;
    } else {
      output += char;
    }
    if (char === ';') {
      inEscSeq = false;
      output += inverseTags[escSeq];
      escSeq = '';
    }
  }
  return output;
};

class Stream {

  static debounce(func, delay=300) {

    let timeout = null;
    function _wrapper(...args) {
      if (timeout) {
        clearTimeout(timeout);
      }

      timeout = setTimeout(() => {
        func(...args);
        timeout = null;
      }, delay);
    }

    return _wrapper;
  }

  static throttle(func, delay) {

    let allow = true;
    function _wrapper(...args) {
      if (allow) {
        func(...args);
        allow = false;
        setTimeout(() => {
          allow = true;
        }, delay);
      }
    }

    return _wrapper;
  }

  static waitForCondition(func, delayConditionFunction) {
    delayConditionFunction = delayConditionFunction ||
      function () {
        return true;
      };

    if (!func) {
      return;
    }

    let timeout = null;
    let waitCounter = 0;
    function _wrapper() {
      if (delayConditionFunction()) {
        clearTimeout(timeout);
        func(...arguments);
        return;
      }

      timeout = setTimeout(function () {
        _wrapper();
      }, waitCounter++);
    }
    _wrapper();
  }
}

arc.Stream = Stream;

arc.misc.detectFontsLoaded = function(successCb, errorCb) {
  // This is an async function that runs a cb once all the fonts are loaded.
  const lato0 = new FontFaceObserver('Lato', { weight: 300 });
  const lato1 = new FontFaceObserver('Lato', { weight: 400 });
  const lato2 = new FontFaceObserver('Lato', { weight: 700 });
  const roboto0 = new FontFaceObserver('Roboto', { weight: 100 });
  const roboto1 = new FontFaceObserver('Roboto', { weight: 300 });
  const roboto2 = new FontFaceObserver('Roboto', { weight: 400 });
  const roboto3 = new FontFaceObserver('Roboto', { weight: 500 });
  const roboto4 = new FontFaceObserver('Roboto', { weight: 700 });
  const roboto5 = new FontFaceObserver('Roboto', { weight: 100, style: 'italic'});
  const roboto6 = new FontFaceObserver('Roboto', { weight: 300, style: 'italic'});
  const roboto7 = new FontFaceObserver('Roboto', { weight: 400, style: 'italic'});
  const roboto8 = new FontFaceObserver('Roboto', { weight: 500, style: 'italic'});
  const roboto9 = new FontFaceObserver('Roboto', { weight: 700, style: 'italic'});

  Promise.all([
    lato0.load(),
    lato1.load(),
    lato2.load(),
    roboto0.load(),
    roboto1.load(),
    roboto2.load(),
    roboto3.load(),
    roboto4.load(),
    roboto5.load(),
    roboto6.load(),
    roboto7.load(),
    roboto8.load(),
    roboto9.load(),
  ]).then(() => {
    if (successCb) {
      successCb();
    }
  }).catch(() => {
    if (errorCb) {
      errorCb();
    }
  });
};

// Note: Issues with sScrollY making table too short and
//       also messing with $.checkBoxes widget's all button
arc.misc.DataTable = function ($table, params) {
  $.fn.DataTable.ext.pager.numbers_length = 5;
  $.fn.dataTable.ext.errMode = 'throw';
  $.extend(true, $.fn.dataTable.defaults, {
    language: {
      paginate: {
        next: '&gt;',
        previous: '&lt;',
      },
    },
  });

  const dtScrollBarHeight = arc.misc.estimateScrollbarWidth('horizontal', $table.selector);
  params.iDisplayLength = params.iDisplayLength !== undefined ? params.iDisplayLength : 10;
  var cb = params.fnDrawCallback;
  var obj = {
    deferRender: true,
    paging: params.paging !== undefined ? params.paging : false,
    order: params.order !== undefined ? params.order : false,
    info: params.info !== undefined ? params.info : true,
    searching: params.searching !== undefined ? params.searching : false,
    scrollX: params.scrollX !== undefined ? params.scrollX     : true,
    bLengthChange: params.bLengthChange !== undefined ? params.bLengthChange   : false,
    scrollCollapse: params.scrollCollapse !== undefined ? params.scrollCollapse : true,
    sScrollY: params.sScrollY !== undefined ? params.sScrollY : getTableHeight(true),
    oLanguage: params.oLanguage !== undefined ? params.oLanguage : {
                    sEmptyTable: 'No data',
                    sZeroRecords: 'No matching records found'
                  },
    iDisplayLength: params.iDisplayLength,
    lengthMenu: params.lengthMenu !== undefined ? params.lengthMenu : [
                    [10, 50, 100, -1], [10, 50, 100, 'All']
                  ],
    fnDrawCallback(settings) {
      var $datatablesWrapper = $table.closest('.dataTables_wrapper');
      var $infoContainer = $datatablesWrapper.find('.dataTables_info').parent();
      var $paginationContainer = $datatablesWrapper.find('.dataTables_paginate').parent();

      if ((params.iDisplayLength === -1) ||
          (settings && settings.fnRecordsTotal &&
           settings.fnRecordsTotal() <= params.iDisplayLength)) {
        // Hide pagination if it is enabled and only single page exists
        $paginationContainer.css('display', 'none');
      } else {
        // Update pagination/info styling based on table container width
        // if container width returns 0 (occurs when table is rendered on a
        // currently unselected tab) assume width 200 and, therefore, don't wrap.
        // This is causing text to wrap when it shouldn't in connection explorer.
        const width = $(params.$parent).width() || 200;
        if (width < 200) {
          $paginationContainer.css('display', 'inline-block')
                              .css('float', 'none');
          $infoContainer.css('float', 'none');
          $infoContainer.find('.dataTables_info')
                        .css('white-space', 'normal')
                        .width($infoContainer.closest('.row').width() + 'px');
        } else {
          $paginationContainer.css('display', 'inline-block')
                              .css('float', 'right');
          $infoContainer.css('float', 'left');
        }
      }
      adjustTotalAndHeaderWidth(settings);

      // Adjust the table height here. Can't know for sure the exact height until
      // after the draw has occured due to styling, unexpected wrapping headers, etc.
      // We supply an initial guess at the start, and if it's not accurate, adjust
      // the height.
      if (params.adjustTableHeight) {
        let finalHeight = getTableHeight(false).split('px')[0];
        let $scrollBody = $table.closest('.dataTables_scrollBody');
        let currentHeight = $scrollBody.css('max-height').split('px')[0];
        const negativeHeightTolerance = 15;
        if (params.sScrollY === undefined && !isNaN(finalHeight) &&
          !isNaN(currentHeight) && (currentHeight > finalHeight || finalHeight -
          currentHeight > negativeHeightTolerance)) {
            $scrollBody.css('max-height', finalHeight + 'px');
            adjustTotalAndHeaderWidth(settings);
        }
      }

      setupResizeHandling(settings);

      if (cb) {
        cb();
      }
    }
  };

  Object.keys(params).forEach(function (p) {
    if (obj[p] === undefined) {
      obj[p] = params[p];
    }
  });

  var $tableDataTable = $table.DataTable(obj);

  function getTableHeight(firstGuess) {
    // This function supplies the initial guess for the table heights, and also
    // computes the final height after draw to get rid of scrollbars.
    const { $parent } = params;
    const $scrollFooter = $parent?.find('.row:nth-child(2) .dataTables_scrollFoot') ?? [];
    const $scrollHeader = $parent?.find('.row:nth-child(2) .dataTables_scrollHead') ?? [];

    // Pagination and/or Row Info
    const tableFooter = $parent?.find('.row:nth-child(3)');
    const tableFooterHeight = firstGuess ? 32 : tableFooter.innerHeight();
    const scrollHeaderHeight = $scrollHeader.length > 0 ? $scrollHeader.height() : 45;
    const scrollFooterHeight = $scrollFooter.length > 0 ? $scrollFooter.innerHeight() : 46;

    const enablePagination = !((params.iDisplayLength === -1) || (params.data &&
      params.data.length <= params.iDisplayLength));

    let sumOfNonScrollBodyHeights = scrollHeaderHeight + dtScrollBarHeight;
    if (params.info || enablePagination) {
      sumOfNonScrollBodyHeights += tableFooterHeight;
    }

    if (params.showTotals) {
      sumOfNonScrollBodyHeights += scrollFooterHeight;
    }

    const isParentExist = $parent && $parent.length > 0;
    const tableBodyHeight = isParentExist ?
      `${$parent.height() - sumOfNonScrollBodyHeights}px` :
      '70vh';

    return tableBodyHeight;
  }

  return $tableDataTable;
};

function adjustTotalAndHeaderWidth(settings) {
  if (!settings ||
      !settings.nTBody ||
      !settings.nTBody.parentElement) {
    return;
  }

  const scrollBarBorder = 2;
  const bodyWidth = settings.nTBody.parentElement.clientWidth + scrollBarBorder;

  if (settings.nTFoot &&
      settings.nTFoot.parentElement &&
      settings.nTFoot.parentElement.parentElement) {
    settings.nTFoot.parentElement.parentElement.style.boxSizing = 'content-box';
    settings.nTFoot.parentElement.style.width = `${bodyWidth}px`;
  }

  if (settings.nTHead &&
      settings.nTHead.parentElement &&
      settings.nTHead.parentElement.parentElement) {
    settings.nTHead.parentElement.parentElement.style.boxSizing = 'content-box';
    settings.nTHead.parentElement.style.width = `${bodyWidth}px`;
  }
}

// TODO(4.2): See if we actually need this
let _debouncedResize = null;
function setupResizeHandling(settings) {
  if (!settings) {
    return;
  }

  function onResize() {
    adjustTotalAndHeaderWidth(settings);
  }

  removePassiveListener(window, 'resize', _debouncedResize);
  _debouncedResize = arc.Stream.debounce(onResize);
  addPassiveListener(window, 'resize', _debouncedResize);
}

function browserSupportsPassiveEventListeners() {
  let passiveSupported = false;

  try {
    const options = Object.defineProperty({}, 'passive', {
      get() { passiveSupported = true; }
    });
    window.addEventListener('test', null, options);
  } catch (err) {}

  return passiveSupported;
}

// not using jQuery for event handling here as it does not support passive listeners
function addPassiveListener(target, eventType, listener) {
  if (browserSupportsPassiveEventListeners()) {
    target.addEventListener(eventType, listener, { passive: true });
  } else {
    target.addEventListener(eventType, listener);
  }
}

function removePassiveListener(target, eventType, listener) {
  if (browserSupportsPassiveEventListeners()) {
    target.removeEventListener(eventType, listener, { passive: true });
  } else {
    target.removeEventListener(eventType, listener);
  }
}

// Used to setup dim and agg searchbox on dashboard and visual sidebar
arc.misc.setupDimAggSearch = function(dom_id, initialValue) {
  var sel_cb = function() {
    return d3.select(dom_id)
             .selectAll(".selection_element");
  };

  var search_selection = function(d, term) {
    if (d === undefined) {
      return false;
    }
    var term_found = false;
    var name = d.col_alias() || d.dataset_colname();
    if (d.event_element()) {
      name = d.event_element().getComputedDisplayName();
    }

    if (name && name.toLowerCase().indexOf(term) >= 0) {
      return true;
    }
    return false;
  };

  arc.report.searchboxSetup({
    searchboxId: `${dom_id} .dim-agg-searchbox`,
    search_cb: search_selection,
    sel_cb: sel_cb,
    initialValue: initialValue,
  });
};

// For getting range of more than 1 value
// Used for color range in CHM and Correlation visual
arc.misc.getRange = function (sb_rows, agg_getter) {
  return d3.extent(sb_rows, agg_getter);
};

// Reflects TaskType in the python code
arc.misc.taskTypes = {
  REFRESH_AVIEW: 1
};

arc.misc.simpleUserConfirmation = function(config = {}) {
  const discardChanges = function() { return 'arc.misc.simpleUserConfirmation'; };

  discardChanges.hideModal = function() {
    $("#discard-changes-modal").modal("hide");
  };

  discardChanges.askUser = function(confirmedCb) {
    const $modal = $("#discard-changes-modal");
    if (arc.misc.modalShown($modal)) {
      return;
    }

    updateModal();
    $modal.modal('show');
    var $yes = $modal.find(".discard-changes-yes");

    $yes.off("click")
        .on("click", function() {
          discardChanges.hideModal();
          confirmedCb();
        });

    function updateModal() {
      const text = arc.misc.safe_substitute($('#discard-changes-modal-template').html(), {
        CONFIRM_TITLE: config.confirmTitle || 'Confirmation',
        CONFIRM_QUESTION: config.confirmQus || 'Changes will be discarded. Please confirm.',
        CANCEL_TEXT: config.cancelText || 'Cancel',
        CONFIRM_TEXT: config.confirmText || 'Discard Changes',
      });

      $modal.find('.modal-content').html(text);
    }
  };

  return discardChanges;
};

/*
 * This helper object pops up a delete confirmation modal, makes the user
 * explicitly confirm the action, and then make a callback after confirmation.
 *
 * Note that if the user cancels the action, no callback is invoked.
 *
 * Note also that the modal has to be explicitly dismissed by the confirmation
 * callback by invoking the 'hideModal' method below. The expectation is that
 * the confirmation code will perform the actual deletion, and then hide the
 * modal after the delete action completes, which in most cases is the duration
 * of a db update call to the backend.
 */
arc.misc.deleteConfirmation = function() {
  var deleteConfirmation = function() { return "arc.misc.deleteConfirmation"; };

  var objectType,
      objectName;

  deleteConfirmation.objectType = function(_) {
    if (!arguments.length) return objectType;
    objectType = _;
    return deleteConfirmation;
  };

  deleteConfirmation.objectName = function(_) {
    if (!arguments.length) return objectName;
    objectName = _;
    return deleteConfirmation;
  };

  deleteConfirmation.hideModal = function() {
    $("#delete-confirmation-modal").modal("hide");
  };

  deleteConfirmation.askUser = function(confirmedCb, content) {
    const $modal = $("#delete-confirmation-modal");
    if (arc.misc.modalShown($modal)) {
      return;
    }

    $("#delete-confirmation-type").text(objectType);
    $("#delete-confirmation-name").text(objectName);
    var $confirmText = $("#delete-confirmation-text");
    $confirmText.val("");
    $("#delete-confirmation-yes").prop("disabled", true);
    $('#delete-spin').hide();
    let $contentContainer;
    if (content) {
      $contentContainer = $modal.find('.delete-confirmation-extra-content');
      $contentContainer.html(content);
      $contentContainer.show();
    }

    $modal.modal("show")
          .on('shown.bs.modal', function() {
            $confirmText.focus();
          })
          .on('hidden.bs.modal', function() {
            if ($contentContainer) {
              $contentContainer.html('');
              $contentContainer.hide();
            }
          });

    var $yes = $("#delete-confirmation-yes");
    $confirmText.off('keyup.deleteConfirm').on('keyup.deleteConfirm', function(e) {
      const inputTxt = $confirmText.val();
      $yes.prop("disabled", true);
      if (inputTxt !== 'DELETE') {
        return;
      }

      $yes.prop("disabled", false);
      if (e.keyCode === 13) {
        $yes.trigger("click");
      }
    });

    $yes.off("click")
        .on("click", function() {
            $('#delete-spin').show();
            confirmedCb();
        });
  };

  return deleteConfirmation;
};

arc.misc.nullSet = () => new Set([null, undefined, NaN, Infinity, 'undefined', 'null', 'nan', 'inf', 'infinity']);

arc.misc.isValidDate = function(d) {
  if (arc.misc.nullSet().has(String(d).toLowerCase())) {
    return false;
  }

  return moment(d).isValid();
};

arc.misc.getQueryParameters = function() {
  const params = {};
  if (!window.location.search) return params;

  const query = window.location.search.replace(/^[?|&]*/, '');
  const queryl = query.split('&');

  queryl.forEach((q) => {
    const pair = q.split('=');
    const key = decodeURIComponent(pair[0]);
    const val = decodeURIComponent(pair[1]);
    params[key] = val;
  });

  return params;
};

const DATE_REGEX = /(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3})\d{0,3}([\+-]\d{2}:\d{2})/;

arc.misc.fromNow = function (dateString, customTxt = 'Invalid', alwaysInPast = true) {
  if (dateString) {
    const truncateMicrosecs = dateString.replace(DATE_REGEX, '$1$2');
    const m = moment(truncateMicrosecs, 'YYYY-MM-DDTHH:mm:ss.SSSZZ');
    const isValidDate = m.isValid();

    // If the date is always supposed to be in past, and the saved date is somehow
    // in the future, just assume it is a couple seconds off
    const currentDate = new Date();
    if (alwaysInPast && isValidDate && m.toDate() > currentDate) {
      const timeDiff = (m.toDate() - currentDate) / 1000;
      console.log('Timezone or computer/server issue. ' +
                  `Off by ${timeDiff} seconds (~${Math.ceil(timeDiff / 3600)} hour(s))`);
      return 'a few seconds ago';
    }
    return isValidDate ? m.fromNow() : customTxt;
  } else {
    return customTxt;
  }
};

arc.misc.splitExprAndAlias = function(expr) {

  // Handle no expression case
  if (!expr) {
    return {
      expr: expr,
    };
  }

  // Look for simple ... as <alias>
  var m = expr.match(/(\s+as\s+)([a-zA-Z0-9_-]+)+$/i);
  if (!m) {
    // Or perhaps ... as 'Complex alias'
    m = expr.match(/\s+as\s+(['"`])([^\1]+?)(\1)$/i);
  }

  if (!m) {
    return {
      expr: expr,
    };
  }

  return {
    expr: expr.substr(0, expr.length - m[0].length),
    alias: m[2],
  };
};

/*
 * Call this routine like this:
 * var replacements = {
 *       "NAME": "Mike",
 *       "AGE" :"26",
 *       "EVENT": "20"
 *     },
 *
 *     str = 'My Name is %NAME% and my age is %AGE%.';
 *
 * str_substitute(str, replacements);
 *
 * For any replacement keys that do not start with HTML_, we will
 *     call arc.misc.escapeHTML on the values.
 *
 * Note: replacements are case-insensitive
 *
 */
function str_substitute(str, replacements) {
  return str.replace(/%\w+%/g, function(match) {
    var s = match.substr(1, match.length - 2);
    if (s in replacements) {
      return replacements[s];
    } else {
      s = s.toUpperCase();
      if (s in replacements) {
        return replacements[s];
      }
    }
    return match;
  });
}

/*
 * Escape replacement values & then call str_substitute
 *
 * For any replacement keys that do not start with HTML_, we will
 *     call arc.misc.escapeHTML on the values.
 */
arc.misc.safe_substitute = function(str, replacements) {
  var escaped_replacements = {};

  $.each(replacements, function(k, v) {
    if (k.substring(0, 5) === 'HTML_') {
      escaped_replacements[k] = v;
      return;
    }

    v = arc.misc.escapeHTML(v);
    escaped_replacements[k] = v;
  });

  return str_substitute(str, escaped_replacements);
};

arc.misc.logtrace = function(errorMessage) {
  try {
    throw new Error(errorMessage);
  } catch (error) {
    const stackTrace = error.stack;
    // eslint-disable-next-line no-console
    console.log(stackTrace);
  }
};

arc.misc.arrayAdd = function(target, source) {
  if (target) {
    target.push(...source);
  }
  return target;
};

// input.which === 13 when user presses "Enter" key
arc.misc.ENTER_KEY = 13;

arc.misc.arrayToMap = function(al, accessor) {
  var h = {};
  al.forEach(function(d, i) {
    if (accessor) {
      d = accessor(d);
    }
    h[d] = i;
  });
  return h;
};

// Take an array and a group id accessor. Return a second array which
// contains groups of array rows, grouped by the group id returned by
// the accessor
arc.misc.array_to_groups = function(rows, group_getter) {
  var groups = {},
      groupsl = [];

  rows.forEach(function(d) {
    var g = group_getter(d);
    if (!groups[g]) {
      groups[g] = [d];
      var pos = groupsl.length;
      groupsl.push({
        group: g,
        pos: pos,
        data: groups[g]
      });
    } else {
      groups[g].push(d);
    }
  });

  return groupsl;
};

arc.misc.friendlyDateSince = function(datestr) {
  // moment parsing:
  //   Good Example: datestr for AV's: "Jun 29 22:51:53 UTC 2018"
  //        Returns:
  //            title: "Jun 29 2018 3:51 PM"
  //            fromNow: "2 years ago"
  //   Good Example: datestr for Viz objects: "2018-09-05T04:58:42+00:00"
  //        Returns:
  //            title: "Sep 4 2018 9:58 PM"
  //            fromNow: "2 years ago"
  //   Bad Example: datestr for that could also come in as unix time ms: "1602080556000"
  //        NOTE: Unclear where this would happen in the code currently.
  //
  //        Returns:
  //            title: "N/A"
  //            fromNow: "N/A"
  //
  const dateMoment = moment(new Date(datestr));
  if (dateMoment.isValid()) {
    const title = dateMoment.format('MMM D YYYY h:mm A');
    return `<span title="${title}"><span class="orig-date" >${datestr}</span>` +
    `${dateMoment.fromNow()}</span>`;
  }

  // Return N/A for the friendly date if the datestr value isn't parseable
  return `<span title="N/A"><span class="orig-date">N/A</span>N/A</span>`;
};

arc.misc.friendlyDate = function(datestr, dateFormat = 'MMM DD, YYYY') {
  return `<span class="orig-date">${datestr}</span>${moment(datestr).format(dateFormat)}`;
};

arc.misc.arrayUnique = function(in_a, element_extractor, returnOriginalData) {
  var u = {},
      a = [];

  for (let i = 0; i < in_a.length; i++) {
    var elem = in_a[i];
    if (element_extractor) {
      elem = element_extractor(elem);
    }

    let elemCompare = elem;
    if (elem instanceof Date) {
      elemCompare = elem.valueOf();
    }

    if (u.hasOwnProperty(elemCompare)) {
      continue;
    }

    if (returnOriginalData) {
      a.push(in_a[i]);
    } else {
      a.push(elem);
    }

    u[elemCompare] = 1;
  }

  return a;
};

// Hash function from internet to compute hash for a dimension (non-aggregate)
// Copied from http://www.simplycalc.com/crc32-source.php
arc.misc.crc32_compute_string = function (polynomial, str) {
    function crc32_generate(polynomial) {
        var table = [],
        i, j, n;

        for (i = 0; i < 256; i++) {
            n = i;
            for (j = 8; j > 0; j--) {
                if ((n & 1) === 1) {
                    n = (n >>> 1) ^ polynomial;
                } else {
                    n = n >>> 1;
                }
            }
            table[i] = n;
        }

        return table;
    }

    function crc32_initial() {
        return 0xFFFFFFFF;
    }

    function crc32_final(crc) {
        crc = ~crc;
        return crc < 0 ? 0xFFFFFFFF + crc + 1 : crc;
    }

    function crc32_compute_buffer(polynomial, data) {
        var crc = 0;
        var dataView = new DataView(data);
        var table = crc32_generate(polynomial);
        var i;

        crc = crc32_initial();

        for (i = 0; i < dataView.byteLength; i++) {
            crc = (crc >>> 8) ^ table[dataView.getUint8(i) ^ (crc & 0x000000FF)];
        }

        crc = crc32_final(crc);
        return crc;
    }

    var crc = 0;
    var table = crc32_generate(polynomial);
    var i;

    crc = crc32_initial();

    for (i = 0; i < str.length; i++) {
        crc = (crc >>> 8) ^ table[str.charCodeAt(i) ^ (crc & 0x000000FF)];
    }

    crc = crc32_final(crc);
    return crc;

};

arc.misc.dimensionColorScale = function() {
  var domain,
      range,
      trackInputs = true,
      inputs = {},
      ord = d3.scale.ordinal(),
      fixedColors,
      fixedMappings = {},
      CRC_PRIME = 0xEDB88320; // Constant for crc32 hash

  function fixedScale(input) {
    var sum = arc.misc.crc32_compute_string(CRC_PRIME, ('' + input));

    return range[sum % range.length];
  }

  var scale = function(input) {
    var c;

    c = fixedMappings[input];
    if (c) {
      inputs[input] = true;
      return c;
    }

    if (fixedColors) {
      c = fixedScale(input);
    } else {
      c = ord(input);
    }

    if (input === undefined) {
      return c;
    }

    if (!(input in inputs)) {
      inputs[input] = true;
    }

    return c;
  };

  scale.init = function(_fixedColors, _fixedMappings) {
    fixedColors = _fixedColors;
    if (_fixedMappings) {
      fixedMappings = _fixedMappings;
    }

    inputs = {};
    trackInputs = true;

    return scale;
  };

  // In case, the fixed dimension colors are unset
  // the "inputs" information shouldn't been lost
  // but their coloring should!
  scale.clearFixedColors = function() {
    fixedColors = false;
    fixedMappings = {};
    Object.keys(inputs).forEach((key) => {
      inputs[key] = true;
    });
  };

  scale.trackInputs = function(_) {
    if (!arguments.length) return trackInputs;
    trackInputs = _;

    if (!trackInputs) {
      inputs = {};
    }

    return scale;
  };

  scale.setColor = function(input, color) {
    inputs[input] = color;
    return scale;
  };

  scale.inputs = function() {
    return inputs;
  };

  scale.domain = function(x) {
    if (!arguments.length) return domain;
    domain = x;
    if (!fixedColors) {
      ord.domain(x);
    }

    return scale;
  };

  scale.range = function(x) {
    if (!arguments.length) return range;
    range = x;
    if (!fixedColors) {
      ord.range(x);
    }

    return scale;
  };

  return scale;
};

const crosstabExpandSuffix = '-square-o"></i></span>',
      crosstabExpandSuffixLen = crosstabExpandSuffix.length;

arc.misc.isCrosstabExpandPrefix = function(s) {
  if (typeof (s) === typeof ('')) {
    const e = s.substring(s.length - crosstabExpandSuffixLen, s.length);
    if (e === crosstabExpandSuffix) {
      return true;
    }
  }

  return false;
};

arc.misc.sortNumeric = function(a, b, sort_orders_ascl, sortIndices, sortRange,
    crosstabExpansionsSortAbove) {
  var i, ae, be, range;
  if (sortRange === undefined) {
    sortRange = [];
    for (i = 0; i < a.length; ++i) {
      sortRange.push(i);
    }
  }

  var ret = 0;
  sortRange.some(function(i) {
    if (sortIndices && !(sortIndices[i])) {
      return false;
    }

    if (i >= b.length) {
      ret = -1;
      return true;
    }

    var sort_order_asc = true;
    if (sort_orders_ascl) {
      sort_order_asc = sort_orders_ascl[i];
    }

    ae = a[i];
    be = b[i];
    if (!isNaN(ae) && !isNaN(be)) {
      ae = Number(ae);
      be = Number(be);
    } else {
      if (crosstabExpansionsSortAbove !== undefined) {
        if (arc.misc.isCrosstabExpandPrefix(ae)) {
          ret = crosstabExpansionsSortAbove ? -1 : 1;
          return true;
        }

        if (arc.misc.isCrosstabExpandPrefix(be)) {
          ret = crosstabExpansionsSortAbove ? 1 : -1;
          return true;
        }
      }
    }

    if (ae > be) {
      ret = (sort_order_asc ? 1 : -1);
      return true;
    }

    if (ae < be) {
      ret = (sort_order_asc ? -1 : 1);
      return true;
    }

    return false;
  });

  return ret;
};

arc.misc.sort_arr = function(dims_l, sort_orders_ascl,
    sortIndices, sortRange, crosstabExpansionsSortAbove) {
  if (dims_l.length <= 1) {
    return dims_l;
  }

  dims_l.sort(function(a, b) {
    return arc.misc.sortNumeric(a, b, sort_orders_ascl,
        sortIndices, sortRange, crosstabExpansionsSortAbove);
  });

  return dims_l;
};

arc.misc.formattingIsNaN = function (value) {
  if (!isNaN(value)) {
    return false;
  }

  if (value === "Invalid date") {
    return true;
  }

  value = String(value);
  if (value.indexOf("NaN") === 0 ||
      value.indexOf("NaN") === 1 ||
      value.indexOf("NaN") === 2) {
    return true;
  }

  return false;
};

// Used in arc.report.base to filter out null values
arc.misc.check_if_value_null = function (value) {
  return value === undefined ||
         value === null ||
         arc.misc.nullSet().has(String(value).toLowerCase());
};

arc.misc.sortNums = function(a, b) {
  if (!isNaN(a) && !isNaN(b)) {
    a = Number(a);
    b = Number(b);
  }
  if (a < b) {
    return -1;
  }
  if (a > b) {
    return 1;
  }
  return 0;
};

arc.misc.uniqueIdStart = arc.misc.uniqueIdStart || 123;
arc.misc.uniqueId = function() {
  ++arc.misc.uniqueIdStart;
  return arc.misc.uniqueIdStart;
};

arc.misc.platformChecker = (function() {

  var pc = function() {return 'arc.misc.platformChecker';};

  pc.serverIsWindows = function() {
    return ['win32'].indexOf(arc.globals.server_platform) !== -1;
  };

  pc.serverIsMac = function() {
    return ['darwin'].indexOf(arc.globals.server_platform) !== -1;
  };

  pc.serverIsLinux = function() {
    return !(pc.serverIsWindows() || pc.serverIsMac());
  };

  return pc;
}());

/*
  Estimates the width of scrollbar by creating 2 fake elements, parent and child.
  Sets excessive width of child to know horizontal scrollbar width.
  Sets excessive height of child to know vertical scrollbar width.
*/
arc.misc.estimateScrollbarWidth = function(orientation, chart_div) {
  function returnScrollbarWidth() {
    switch (orientation) {
      case 'horizontal':
        return arc.misc.scrollbarDims.horWidth;
      case 'vertical':
        return arc.misc.scrollbarDims.verWidth;
      default:
        return arc.misc.scrollbarDims.verWidth;
    }
  }

  if (arc.misc.scrollbarDims && arc.misc.isIEOptimisationEnabled()) {
    return returnScrollbarWidth();
  }

  chart_div   = chart_div || 'body';
  var html    = '<div style="width:50px;height:50px;overflow:auto;' +
                'visibility:hidden;"><div style="height:1px;"></div></div>';
  var parent  = $(html).appendTo(chart_div);
  var child   = parent.children();
  var width   = 0;

  if (parent.length === 0) {
    // console.log('invalid chart div', chart_div);
    return 0;
  }

  const verWidth = child.innerWidth() - child.height(99).innerWidth();
  child.height(1).width(100);
  const horWidth = (parent[0]).offsetHeight - (parent[0]).clientHeight;
  parent.remove();

  arc.misc.scrollbarDims = {
    horWidth,
    verWidth,
  };

  return returnScrollbarWidth();
};

arc.misc.addScripts = function(filepaths, cb) {
  arc.frontend.loadResources('script', filepaths, undefined, cb);
};

arc.misc.addStyles = function(filepaths, cb) {
  arc.frontend.loadResources('link', filepaths, undefined, cb);
};

arc.misc.notify = function(message) {
  arc.report.alertMessageSmall(message, "alert-success");
};

arc.misc.appsAndAppgroupsToLinks = function(config) {
  const {
    appNames,
    newTabs,
  } = config;

  let {
    appIds,
    appAndVisualIds,
    appgroupIds,
    maxCount,
    joinStr,
  } = config;

  appIds = appIds || [];
  appAndVisualIds = appAndVisualIds || [];
  appgroupIds = appgroupIds || [];
  maxCount = maxCount || Infinity;
  joinStr = joinStr || ', ';

  if (appAndVisualIds.length > 0) {
    appIds = appAndVisualIds.map((x) => x[0]);
  }

  const overflow = appIds.length > maxCount || appAndVisualIds.length > maxCount ?
                   `<span title="Additional: ${appIds.slice(maxCount)}">...</span>` :
                   '';

  const target = newTabs ? 'target="_blank"' : '';

  function _link(_config) {
    const {
      id,
    } = _config;

    let {
      tooltip,
      baseUrl,
      fullUrl,
      linkName,
    } = _config;

    linkName = linkName || id;
    if (!baseUrl) {
      baseUrl = arc.frontend.globals.urls.viewDashboardUrl;
    }

    if (!fullUrl) {
      fullUrl = arc.misc.getBlankTargetUrl(`${baseUrl}/${id}`, target);
    }

    if (tooltip) {
      tooltip = arc.misc.escapeHTML(tooltip);

      return `<a data-toggle="tooltip" data-placement="top" title="${tooltip}" ` +
          `${target} href="${fullUrl}">${linkName}</a>`;
    }

    return `<a ${target} href="${fullUrl}">${linkName}</a>`;
  }

  let appLinks;

  if (appAndVisualIds.length > 0) {
    appLinks = appAndVisualIds.slice(0, maxCount).map((appAndVisualId) => {
      const appId = appAndVisualId[0];
      const visualId = appAndVisualId[1];

      const appIdElement = `<i class="fa sym-dashboard"></i> ${appId} `;
      const visualIdElement = `<span style="margin-left:10px;"></span> (Visual  ${visualId})`;
      const linkElementContainsWhereCustomStyleIsInUse =
        visualId == null ? appIdElement : appIdElement + visualIdElement;

      const appUrl = `${arc.frontend.globals.urls.viewDashboardUrl}/${appId}`;
      const visualToHighlightUrlPart = `?highlighted_visual=${visualId}`;
      const urlToWhereCustomStyleIsInUse =
        visualId == null ? appUrl : appUrl + visualToHighlightUrlPart;

      return _link({
        id: appId,
        fullUrl: urlToWhereCustomStyleIsInUse,
        linkName: linkElementContainsWhereCustomStyleIsInUse,
      });
    });
  } else {
    appLinks = appIds.slice(0, maxCount).map((appId, idx) => _link({
        id: appId,
        tooltip: appNames ? appNames[idx] : undefined,
        baseUrl: arc.frontend.globals.urls.viewDashboardUrl,
      }));
  }

  const appgroupLinks =
      appgroupIds.slice(0, maxCount === Infinity ? maxCount : maxCount - appIds.length)
                 .map((appgroupId) => _link({
                     id: appgroupId,
                     baseUrl: arc.frontend.globals.urls.editAppUrl,
                     linkName: `<i class="fa sym-app"></i> ${appgroupId}`,
                   }));

  appLinks.push(...appgroupLinks);
  return appLinks.join(joinStr) + overflow;
};

arc.misc.appsToLinks = function(appIds = [], maxCount = Infinity, appNames = undefined) {
  return arc.misc.appsAndAppgroupsToLinks({
    appIds,
    appNames,
    maxCount,
  });
};

arc.misc.customJsUnload = function(jsIds) {
  const assets = [];
  jsIds.forEach(function(jsId) {
    assets.push({
      asset: `${arc.frontend.globals.urls.customJsRawUrl}/${jsId}`,
    });
  });

  arc.staticfetcher.forget(assets);
};

arc.misc.customJsFetcher = function(jsIds, cb) {
  var assets = [];
  jsIds.forEach(function(jsId) {
    assets.push({
      asset: `${arc.frontend.globals.urls.customJsRawUrl}/${jsId}`,
      postprocess: d3.xhr,
    });
  });

  var jsAssets = [];
  var jsAssetIds = [];
  arc.staticfetcher.fetch(assets, function() {
    var error = arguments[0];
    if (error) {
      cb(error, undefined);
      return;
    }

    for (var i = 1; i < arguments.length; ++i) {
      if (arguments[i]) {
        var j = arguments[i].responseText.trim();
        if (j.length > 0) {
          jsAssets.push(j);
          jsAssetIds.push(jsIds[i - 1]);
        }
      }
    }

    cb(undefined, jsAssets, jsAssetIds);
  });
};

// True if specified permission is present in input list of permissions
arc.misc.hasPerm = function(perm, plist) {
  return plist && plist.indexOf(perm) !== -1;
};

// True if current have the specified permission
arc.misc.sysPerm = function(perm) {
  return arc.misc.hasPerm(perm, arc.globals.perms.sysperms);
};

// Update Accelerate link
arc.misc.updateAccelerateButton = function(dataconnPerm) {
  if (!dataconnPerm) return;

  const hasDataconnAVPerm = arc.misc.hasPerm('dc_aviews', dataconnPerm);
  if (hasDataconnAVPerm) {
    $('.accelerate-dashboard').show();
  } else {
    $('.accelerate-dashboard').hide();
  }
};

// Update clone icon
arc.misc.updateCloneButton = function(datasetPerm) {
  const hasDatasetEditPerm = arc.misc.hasPerm('ds_appedit', datasetPerm);
  $('.clone-link').toggleClass('arc-invisible', !hasDatasetEditPerm);
};

// Hide/show create App/Dashboard buttons
arc.misc.updateCreateAppDashboardButton = function(datasets, $subnavReport) {
  let hasViewPerm;
  const hasEditPerm = datasets.some((d) => {
    hasViewPerm = hasViewPerm || arc.misc.hasPerm('ds_appview', d.perms);
    return arc.misc.hasPerm('ds_appedit', d.perms);
  });

  if (hasViewPerm) {
    if (hasEditPerm) {
      arc.misc.updateDivForPerms(['ds_appedit', 'ds_appview'], 'dataset', $subnavReport);
    } else {
      arc.misc.updateDivForPerms(['ds_appview'], 'dataset', $subnavReport);
    }
  }
};

// Update a div (or entire body) according to object permissions
arc.misc.updateDivForPerms = function(plist, ptype, $div, forceVisible) {
  if (!$div) {
    $div = $('body');
  }
  var perms = arc.globals.perms.permlists[ptype];
  if (!perms) {
    return;
  }

  // Enable/disable div elements based on permission present or not
  perms.forEach(function (perm) {
    var jq = $div.find("[arcperm=" + perm + "]");
    if (arc.misc.hasPerm(perm, plist) || forceVisible) {
      jq.removeClass('arc-invisible');
    } else {
      jq.addClass('arc-invisible');
    }
  });
};

// Update a div (or entire body) according to system permissions
arc.misc.updateDivSysPerms = function(div) {
  arc.misc.updateDivForPerms(arc.globals.perms.sysperms, 'system', div);
};

var chartNames = {
  action: 'Action',
  table: 'Table',
  crosstab: 'Cross Tabulation',
  'trellis-bars': 'Bars',
  'trellis-lines': 'Lines',
  combo: 'Combined Bar/Line',
  'trellis-areas': 'Areas',
  'trellis-groupedbars': 'Grouped Bars',
  kpi: 'KPI',
  'packed-bubbles': 'Packed Bubbles',
  scatter: 'Scatter',
  flow: 'Flow',
  funnel: 'Funnel',
  pie: 'Pie',
  radial: 'Radial',
  chord: 'Chord',
  corelation: 'Correlation Heatmap',
  'corelation-flow': 'Correlation Flow',
  'calendar-heatmap': 'Calendar Heatmap',
  map: 'Map',
  leaflet: 'Interactive Map',
  sparklines: 'Sparklines',
  timecontrol: 'Time Control',
  link: 'External Link',
  list: 'Row Listing',
  histogram: 'Histogram',
  extension: 'Extension',
  'column-stats': 'Field Statistics',
  rawtable: 'Queries',
  html: 'Rich Text',
  network: 'Network',
  dendrogram: 'Dendrogram',
  treemap: 'Treemap',
  bullet: 'Bullet',
  gauge: 'Gauge',
  wordcloud: 'Word Cloud',
  chatbot: 'AI',
  boxplot: 'Box Plot',
  timeline: 'Timeline',
  search: 'Search',
};

arc.misc.chartTypeToName = function(chartType) {
  var name = chartNames[chartType];
  if (!name) {
    name = chartType;
  }

  return name;
};

// Used in map visual and interactive map visual.
// Returns the ordering of the geo casts in the geo shelf
arc.misc.geoCastLocations = function(columns) {
  var geo_element_locations = {};

  columns.forEach(function(col) {
            var cast = col.colgeotype();
            if (geo_element_locations[cast] === undefined)
              geo_element_locations[cast] = col.colindex();
          });
  return geo_element_locations;
};

/* Set up Ace SQL expression editor with autocompletes
 * @param  dataset         object - dataset object for which the editor should be set up
 * @param  editor          existing editor object, if any
 * @param  data_id         string - the ID for the ace editor div
 * @param  $editDiv        jQuery object - the div that houses both ace editor and autocomplete
 *                            checkbox
 * @param  config          object with following properties
 *           includeTables   boolean - whether or not table names should be included in
 *                              autocompletes (default false)
 *           excludeRecordCountAutocomplete boolean - whether or not the record count measure should
 *                            be included (default false)
 *           datasetSql   boolean - whether this editor is being used in dataset SQL editor
 *                            (default false)
 *           tables        array of table objects to use for autocompletes (otherwise uses all
 *                            tables in dataset)
 *           includeCalculated  boolean - whether calculated columns should be included in
 *                              autocompletes (default false)
 *           useRawNames        boolean if the raw tablename.columnname should be used as
 *                              autocomplete and snippet in place of [column alias] (default
 *                              behavior is false)
 *           dataconnection_type When we create a new dataset from SQL, we do not have a dataset
                                 available, but do have a data connection type
 *
 * @return returns editor object
 */
arc.misc.expressionEditor = (
  dataset, editor, data_id, $editDiv, config = {}, resizable = false
) => {
  let { dataconnection_type } = config;
  const { isSensitiveShelf } = config;
  const columns = dataset ? JSON.parse(dataset?.dataset_info())[0].columns : [];
  const sensitiveColumns = {};
  columns.forEach((col) => {
    if (col.isSensitive) {
      sensitiveColumns[col.alias] = true;
    }
  });

  // Helper Functions
  const sidebarClickCb = function(el) {
    editor.aceEditor().insertSnippet(el.__data__);
    editor.focusEditor(false);
  };

  const getFieldHtml = (fieldItem) => {
    const elem = arc.report.shelf.element().dataset_coltype(fieldItem.coltype);

    const typeIcon = `<span class="dim-icon-type ${elem.dataset_coltype_class()}"></span>`;
    const fieldSpan = `<span class="arc-field">${fieldItem.snippetName}</span>`;
    const $sensitiveWarningSpan = $('<svg class="pl-1" style="font-size: 13px; padding-bottom: 3px;" width="1.3em" height="1.3em" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M12 4.43C13.31 5.29 15.56 6.62 18 7.48V13C18 14.6 15.44 17.73 12 19.73C8.56 17.73 6 14.61 6 13V7.48C8.44 6.62 10.69 5.29 12 4.43ZM12 2C12 2 8 5 4 6V13C4 16 8 20 12 22C16 20 20 16 20 13V6C16 5 12 2 12 2ZM16.54 10.44L15.13 9.03L11.01 13.15L8.89 11.03L7.48 12.44L11.02 15.98L16.56 10.44H16.54Z" fill="#2e89c5"/></svg>');

    $sensitiveWarningSpan.attr('data-toggle', 'tooltip')
                            .attr('title', 'Sensitive field')
                            .tooltip({
                                container: 'body',
                                delay: {
                                  show: 500,
                                  hide: 100
                                }
                              });

    const sensitiveWarningHtml = fieldItem.snippetName in sensitiveColumns ? $sensitiveWarningSpan.prop('outerHTML') : '';

    return `${typeIcon}${fieldSpan}${sensitiveWarningHtml}`;
  };

  const tables = config.tables || dataset ? dataset.tables() : [];

  const getTableHtml = function(tableItem) {
    const typeIcon = '<span class="dim-icon-type fa fa-table"></span>';
    const tableSpan = `<span class="arc-field">${tableItem.snippetName}</span>`;
    return `${typeIcon}${tableSpan}`;
  };

  if (!dataconnection_type && dataset) {
    dataconnection_type = dataset.dataconnection_type();
  }

  const $functionsList = $editDiv.find('.edit-expression-functions-lists'),
      $fieldList = $editDiv.find('.edit-expression-fields-lists');

  if ($functionsList.find('li').length) {
    $functionsList.clickableList('destroy');
  }

  if ($fieldList.find('li').length) {
    $fieldList.clickableList('destroy');
  }

  if (!editor) {
    const $autocompleteButton = $editDiv.find('.edit-expression-autocomplete');
    editor = arc.utils.autocompleteEditor(
      data_id,
      dataconnection_type,
      {
        extraAutocompletes: config.includeTables ? 'tablesAndColumns' : 'columns',
        tables,
        datasetSql: config.datasetSql,
        autocompleteOnOffButton: $autocompleteButton,
        excludeRecordCount: config.excludeRecordCountAutocomplete,
        includeCalculated: config.includeCalculated,
        useRawNames: config.useRawNames,
        isSensitiveShelf
      }
    );

    if (resizable) {
      editor.enableResize();
    }
  } else {
    editor.mode(dataConnectionType);
    editor.tables(tables);
  }

  // Add snippetSidebars
  const fieldLists = {},
      snippetItems = arc.utils.snippetsGenerator(editor.mode(), 'hash');

  editor.extraAutocompletes().forEach(function(item) {
    //TOFIX: If an hierarchy, skip the item from being pushed in fieldLists
    fieldLists[item.type] = fieldLists[item.type] || [];
    fieldLists[item.type].push(item);
  });

  editor.snippetSidebars([
    $functionsList.clickableList({
      items: snippetItems,
      skipTypes: ['duplicate'],
      clickCb: sidebarClickCb,
      showAll: 'Functions',
      itemParser: function (item) {
        var snippetCaption = arc.utils.snippetsGenerator.parseText(item);
        return {
          text: snippetCaption.caption,
          data: snippetCaption.snippet,
          tooltip: arc.utils.snippetsGenerator.convertSnippetToDisplay(item)
        };
      }
    }),
    $fieldList.clickableList({
      items: fieldLists,
      skipTypes: ['duplicate'],
      clickCb: sidebarClickCb,
      showAll: config.includeTables ? 'Fields & Tables' : 'Fields',
      itemParser: (item) => {
        const html = item.type === 'Tables' ? getTableHtml(item) : getFieldHtml(item);
        let itemData = item.expression || item.value;
        const itemParts = itemData.split('.');
        const tooltip = `${itemData} (${item.coltype})`;

        // highlight the table name portion of field name so users update to alias if necessary
        if (item.type !== 'Tables' && itemParts.length > 1) {
          itemData = `\${1:${itemParts[0]}}.${itemParts.slice(1)}`;
        }

        return {
          text: html,
          data: itemData,
          tooltip: tooltip === '[Record Count] (BIGINT)' ? 'sum(1)' : tooltip
        };
      }
    })
  ]);

  // Expand arc.report.supported_aggregates based on snippets
  arc.report.addAndCheckAggregates(undefined, dataconnection_type, snippetItems);

  // Used in cypress testing - do not delete
  arc.testutil = arc.testutil || {};
  arc.testutil.lastEditor = editor;

  return editor;
};

arc.misc.shelfExpressionEditor = function(dataset, isSensitiveShelf = false) {
  // Set up Ace expression editor for this particular Dataset if in build mode within visuals
  if (!arc.globals.is_edit) {
    return;
  }

return arc.misc.expressionEditor(dataset,
                              undefined,
                              'expression-data',
                              $('#edit-expression-form'),
                              { includeCalculated: true, isSensitiveShelf });
};

/* Queue listeners for expression validation buttons on Ace expression editor
 * @param  $exprValBtn  jQuery object - the button that the user clicks to validate expression
 * @param  $exprSubmitBtn  jQuery object - the button that the user clicks to validate and submit
 *                                       expression
 * @param  exprEditor   object - an editor object as created by arc.misc.expressionEditor or
 *                               arc.utils.autocompleteEditor
 * @param  dataset      object - dataset the expression applies to
 * @param  type         string - type of expression being evaluated, column, filter, join or query
 * @param  viz          object  (optional) a visual with which the expression is associated
 * @param  config    object - with the following possible keys:
 *                        submitCb  function -- a function to be invoked if the expression
 *                               is valid and ready for submission
 *                        exprModifierCb function - a function to be invoked to modify
 *                                 the expression before submitting to the back end for validation,
 *                                 takes expression and returns updated
 *                                 expression
 *                        noAggs boolean  - set to true if aggregate expressions should
 *                                not be permitted
 *                        srcElem jQuery element - input element that SQL expression is linked to
 * @return    returns object validator with the "validate" function
 */

arc.misc.addExpressionValidation = function(
  $exprValBtn,
  $exprSubmitBtn,
  exprEditor,
  dataset,
  exprType,
  viz,
  config = {}
) {
  let validator = {};

  validator.validate = function(e, successCb) {

    // Remove any existing alert text
    const $parent = $(e.target).closest('.edit-expression-holder').find('.edit-expression-textbox');
    $parent.find('.alert').remove();
    $parent.find('[arcid=validate-spinner]').show();

    let expr = exprEditor.value();
    const validateConfig = {};
    let s_expr;

    // Do parameter substitution & split out the "as 'alias'" part if present
    const expr_beforesubstitution = expr;
    if (expr_beforesubstitution.trim() === '') {
      validateExprCb({});
      return;
    }

    if (exprType === 'join') {
      const tableInfo = arc.admin.dataset().readJoinFromForm({ skipElem: config.srcElem });
      tableInfo.join_exprs.push(expr);

      const leftTable = tableInfo.src_table.dstab;
      const rightTable = tableInfo.dstab;

      // table hierarchy must be preserved.
      // remove only the join conditions and preserve parent table
      leftTable.join_pairs([]);
      leftTable.join_exprs([]);

      // VIZ-2562: the above does not hold if, for example, there are computed fields
      // referencing multiple tables. It is safer to always include all tables.
      const allTablesClones = dataset.tables().map(
        (table) => {
          if (table.tablename() === leftTable.tablename()) {
            return leftTable;
          }

          if (table.tablename() === rightTable.tablename()) {
            return rightTable;
          }

          const clone = table.clone();
          // do not use the joins of the unedited tables:
          // their hierarchy (parent_table) must be preserved
          clone.join_pairs([]);
          clone.join_exprs([]);
          return clone;
        }
      );

      const testDataset = arc.datasets.Dataset();
      testDataset.tables(allTablesClones);

      // figure out how to do the substitution on correct join expression in tableInfo
      s_expr = 'SELECT *';

      validateConfig.dsObj = testDataset.buildInfo();
    } else {
      if (viz) {
        expr = viz.substituteParams(expr);
      }

      const esplit = arc.misc.splitExprAndAlias(expr);
      s_expr = esplit.expr;

      // When the expression is evaluated to be an empty string after
      // substitution, don't do full validation and instead immediately call
      // validation success callback
      if (viz && s_expr !== expr_beforesubstitution && viz.isShelfExpressionEmpty(s_expr)) {
        validateExprCb({});
        return;
      } else if (viz && arc.misc.isWindowFunction(s_expr)) {
        const info = 'Analytic functions cannot be properly validated. Please refresh visual to ' +
          'see any error messages.';
        validateExprCb({ info });
        return;
      }
    }

    if (config.exprModifierCb) {
      s_expr = config.exprModifierCb(s_expr, exprEditor);
    }

    if (s_expr.trim() === '') {
      validateExprCb({});
      return;
    }

    arc.validate_expression(s_expr, dataset, validateExprCb, exprType, validateConfig);

    function validateExprCb(results) {
      $parent.find('.alert').remove();
      let loc = $parent.find('[arcid=validate-spinner]');
      loc.hide();
      if (results.err) {
        loc.after(arc.clone_template('col-val-failed', {errtext: results.err}).html());
      } else if (config.noAggs && results.hasagg) {
        loc.after(arc.clone_template('col-val-failed',
          {errtext: 'Aggregate expressions not permitted'}).html());
      } else {
        if (successCb) {
          successCb();
        }

        if (results.info) {
          loc.after(arc.clone_template('col-val-info', {info: results.info}).html());
        } else {
          loc.after(arc.clone_template('col-val-passed').html());
        }
      }
    }
  };

  $exprValBtn.off('click').on('click', validator.validate);

  if ($exprSubmitBtn) {
    $exprSubmitBtn.off('click')
                  .on('click', function validateWithSubmit(e) {
                    validator.validate(e, config.submitCb);
                  });
  }

  return validator;
};

// Default to English if we can't get language for some reason
arc.misc.getLanguage = function (languageOverride) {
  return languageOverride ||
         (navigator.language || navigator.browserLanguage) ||
         'en-US';
};

// Returns back true if language code is for country that uses comma decimal point
// ISO language codes from http://www.lingoes.net/en/translator/langcode.htm
// Comma/Period decimal mark based on https://en.wikipedia.org/wiki/Decimal_mark
arc.misc.isCommaDecimal = function (languageFull) {
  const languageAbbreviated = languageFull.split('-')[0];

  const usesCommaAsDecimal = {
    'af': true, 'af-ZA': true,
    'ar': false, 'ar-AE': false, 'ar-BH': false, 'ar-DZ': true, 'ar-EG': false,
    'ar-IQ': false, 'ar-JO': false, 'ar-KW': false, 'ar-LB': true, 'ar-LY': false,
    'ar-MA': true, 'ar-OM': false, 'ar-QA': false, 'ar-SA': false, 'ar-SY': false,
    'ar-TN': true, 'ar-YE': false,
    'az': true, 'az-AZ': true,
    'be': true, 'be-BY': true,
    'bg': true, 'bg-BG': true,
    'bs': true, 'bs-BA': true,
    'ca': true, 'ca-ES': true,
    'cs': true, 'cs-CZ': true,
    'cy': false, 'cy-GB': false,
    'da': true, 'da-DK': true,
    'de': true, 'de-AT': true, 'de-DE': true,
    'el': true, 'el-GR': true,
    'en': false, 'en-AU': false, 'en-BZ': false, 'en-CA': false, 'en-CB': false,
    'en-GB': false, 'en-IE': false, 'en-JM': false, 'en-NZ': false, 'en-PH': false,
    'en-TT': false, 'en-US': false, 'en-ZA': true, 'en-ZW': false,
    'es': true, 'es-AR': true, 'es-BO': true, 'es-CL': true, 'es-CO': true,
    'es-CR': true, 'es-DO': false, 'es-EC': true, 'es-ES': true, 'es-GT': false,
    'es-HN': false, 'es-MX': false, 'es-NI': false, 'es-PA': false, 'es-PE': true,
    'es-PR': false, 'es-PY': true, 'es-SV': false, 'es-UY': true, 'es-VE': true,
    'et': true, 'et-EE': true,
    'eu': true, 'eu-ES': true,
    'fi': true, 'fi-FI': true,
    'fo': true, 'fo-FO': true,
    'fr': true, 'fr-BE': true, 'fr-CA': true, 'fr-FR': true,
    'gl': true, 'gl-ES': true,
    'gu': false, 'gu-IN': false,
    'he': false, 'he-IL': false,
    'hi': false, 'hi-IN': false,
    'hr': true, 'hr-BA': true, 'hr-HR': true,
    'hu': true, 'hu-HU': true,
    'hy': true, 'hy-AM': true,
    'id': true, 'id-ID': true,
    'is': true, 'is-IS': true,
    'it': true, 'it-IT': true,
    'ja': false, 'ja-JP': false,
    'ka': true, 'ka-GE': true,
    'kk': true, 'kk-KZ': true,
    'kn': false, 'kn-IN': false,
    'ko': false, 'ko-KR': false,
    'kok': false, 'kok-IN': false,
    'ky': true, 'ky-KG': true,
    'lt': true, 'lt-LT': true,
    'lv': true, 'lv-LV': true,
    'mi': false, 'mi-NZ': false,
    'mk': true, 'mk-MK': true,
    'mn': true, 'mn-MN': true,
    'mr': false, 'mr-IN': false,
    'ms': false, 'ms-BN': false, 'ms-MY': false,
    'mt': false, 'mt-MT': false,
    'nb': true, 'nb-NO': true,
    'nl': true, 'nl-BE': true, 'nl-NL': true, 'nn-NO': true,
    'ns': true, 'ns-ZA': true,
    'pa': false, 'pa-IN': false,
    'pl': true, 'pl-PL': true,
    'pt': true, 'pt-BR': true, 'pt-PT': true,
    'qu': true, 'qu-BO': true, 'qu-EC': true, 'qu-PE': true,
    'ro': true, 'ro-RO': true,
    'ru': true, 'ru-RU': true,
    'sa': false, 'sa-IN': false,
    'se': true, 'se-FI': true, 'se-NO': true, 'se-SE': true,
    'sk': true, 'sk-SK': true,
    'sl': true, 'sl-SI': true,
    'sq': true, 'sq-AL': true,
    'sr': true, 'sr-BA': true, 'sr-SP': true,
    'sv': true, 'sv-FI': true, 'sv-SE': true,
    'sw': false, 'sw-KE': false,
    'ta': false, 'ta-IN': false,
    'te': false, 'te-IN': false,
    'th': false, 'th-TH': false,
    'tl': false, 'tl-PH': false,
    'tn': true, 'tn-ZA': true,
    'tr': true, 'tr-TR': true,
    'tt': true, 'tt-RU': true,
    'uk': true, 'uk-UA': true,
    'ur': false, 'ur-PK': false,
    'uz': true, 'uz-UZ': true,
    'vi': true, 'vi-VN': true,
    'xh': true, 'xh-ZA': true,
    'zh': false, 'zh-CN': false, 'zh-HK': false, 'zh-MO': false, 'zh-SG': false,
    'zh-TW': false,
    'zu': true, 'zu-ZA': true,
  };

  return usesCommaAsDecimal[languageFull] ||
         usesCommaAsDecimal[languageAbbreviated];
};

const defaultCsvConfig = {
  columnSeparator: ',',
  quotationSetting: 'false',
};

/**
 * Handles conversion of data in single cell to its string representation.
 * Quote value of cell if numeric string or string contains separators.
 * This string is used in the download of the file, i.e. to CSV or Excel.
 *
 * If `type` is not recognized, guess the type and do conversion.
 * Optional `quotationSetting` allows single-quotes around numeric string.
 * @param {any} colValue: data in single cell of table column
 * @param {string} type: Can be one of the following values (STRING, BIGINT, DOUBLE, REAL)
 * @param {object} [config] (Optional) Configuration of column separators and quotations, etc.
 * User object is merged with `defaultCsvConfig` defined above.
 * @property {string} config.columnSeparator: Columns can be separated by either
 * comma or semi-colon, decimals are replaced by commas if separator is semi-colon.
 * @property {string} config.quotationSetting: Whether to wrap string entries, like '00123'.
 * A string of boolean value is expected, i.e. 'true' or 'false'.
 */
arc.misc.convertToStrRep = function(colValue, type, config = defaultCsvConfig) {
  const isNumber = $.isNumeric(colValue);
  let string = `${colValue ?? ''}`;

  // Create new object with merged configs
  const userConfig = Object.assign({}, defaultCsvConfig, config);
  const quotationMark = userConfig.quotationSetting === 'true' ? "'" : '';
  const columnSeparator = userConfig.columnSeparator;
  const isCommaDecimal = columnSeparator !== ",";

  if (type === 'STRING' && isNumber) {
    string = `${quotationMark}${string}${quotationMark}`;
  } else if (type === 'STRING') {
    if (string.search(columnSeparator) > -1 ||
                   string.search('\n') > -1 ||
                   string.search('\r') > -1) {
      string = `"${string.replace(/"/g, '""')}"`;
    }
  } else if ((type === 'DOUBLE' || type === 'REAL') && isNumber) {
    if (isCommaDecimal && string.includes('.')) {
      string = string.replace('.', ',');
    }
  } else if (isNumber) {
    if (isCommaDecimal && string.includes('.')) {
      string = string.replace('.', ',');
    }

    // $.isNumeric() is true for strings that can be coerced to a number
    // Add another check for numeric strings
    if (typeof colValue === 'string') {
      string = `${quotationMark}${string}${quotationMark}`;
    }
  } else {
    string = sanitizeString(string, true);
    string = `"${string.replace(/"/g, '""')}"`;
  }

  return string;

  // Adds quotes around string in the case of string having quotes inside,
  // or containing column separator or newlines.
  function sanitizeString(s, isHeader) {
    let result = s;
    if (s.search(quotationMark) > -1) {
      // Construct new Regex object to use variable in regex
      const re = new RegExp(quotationMark, 'g');

      // If there exists quote in string, double the quote so Excel escapes it
      const replacedQuotesStr = s.replace(re, `${quotationMark}${quotationMark}`);
      result = `${quotationMark}${replacedQuotesStr}${quotationMark}`;
    } else if (isHeader) {
      if (s.search(userConfig.columnSeparator) > -1 ||
          s.search('\n') > -1 ||
          s.search('\r') > -1) {
        // This block needs to be else-if because we don't want to double wrap string in quotes
        result = `${quotationMark}${s}${quotationMark}`;
      }
    } else {
      result = `${quotationMark}${s}${quotationMark}`;
    }

    return result;
  }
}

const formatEnums = {
  NONE: 0,
  PERCENT: 1,
  DOLLAR: 2,
  EURO: 3,
  POUND: 4,
  YEN: 5,
  RUPEE: 6,
};

function processExcelCell(data, type, colFormatter) {

  if (!colFormatter) {
    colFormatter = (val) => val;
  }

  if (type === 'TIMESTAMP') {
    const sortValue = data.sort;
    if (arc.misc.isValidDate(sortValue)) {
      const dateObj = new Date(sortValue);

      // Check if Date object formed is valid as well
      // Possible that it might be invalid date on IE 11
      if (arc.misc.isValidDate(dateObj)) {
        return colFormatter(sortValue);
      }
    }

    return colFormatter(data.display);
  }

  if (/<[a-z][\s\S]*>/i.test(data.display)) {
    // if the string contains html element then use the raw data
    // This is applicable for linked columns
    return colFormatter(data.sort);
  }

  if (type in arc.report.measure_types) {
    return colFormatter(data.sort);
  }

  return colFormatter(arc.misc.unescapeHTML(data.display));
}

arc.misc.downloadXlsx = function(base) {
  let title = base.title() || 'Untitled';
  title = title.replace(/[\\\/\?\*\[\]]/g, '_'); // Replace invalid filename chars
  const xlsxName = title.length > 31 ? title.substring(0, 31) : title;
  const xlsxFileName = arc.misc.getTimestampFilename(title, 'xlsx');
  const globalLimit = arc.globals.download_maxrows;
  const isChatbot = base.viz_type() === 'chatbot';
  const colnames = isChatbot ? base.viz().getDownloadInfo().colNames : base.sb_colnames();
  let sb_rows = isChatbot ? base.viz().getDownloadInfo().rows : base.sb_rows();
  let params = (arc.globals.ms_dashboard &&
          arc.globals.ms_dashboard.params_history().current_params()[0]) || [];
  let data = [];
  let max_number_rows = 0;
  let wb;
  let ws;

  if (!Number.isNaN(globalLimit) && sb_rows.length > globalLimit) {
    sb_rows = sb_rows.slice(0, globalLimit);
  }

  const alertMessageGroupHandler = arc.report.createAlertMessageGroup();
  alertMessageGroupHandler.addDelayedMessage('Preparing data columns for download', 'alert-info');

  if (base.viz_type() === 'crosstab') {
    wb = XLSX.utils.table_to_book($(base.viz().slickRowsToTable())[0],
      { sheet: xlsxName, display: true, raw: true });
  } else {
    // getting the formatted rows
    const columns = base.dataResult().columns();
    const rows = [colnames].concat(sb_rows.map((row) => {
      const dupRow = [];
      columns.forEach((col, colIdx) => {
        if (col.coldisplayformat() !== col._defaultFormat()) {
          dupRow.push(col.coldisplayformat()(row[colIdx]));
        } else {
          dupRow.push(row[colIdx]);
        }
      });

      return dupRow;
    }));

    wb = XLSX.utils.book_new();
    ws = XLSX.utils.aoa_to_sheet(rows);
    XLSX.utils.book_append_sheet(wb, ws, xlsxName);
  }

  alertMessageGroupHandler.addDelayedMessage('Preparing data rows for download', 'alert-info');

  for (let i = 0; i < params.length; i++) {
    let filter = [`${params[i].key}`];
    if (params[i].value_list) {
      for (let j in params[i].value_list) {
        filter.push(params[i].value_list[j]);
      }
    } else {
      filter.push(params[i].value);
    }

    data.push(filter);
    max_number_rows = Math.max(max_number_rows, filter.length);
  }

  // transpose data matrix
  var column_length = data.length;
  var new_data = [];
  var row;
  for (let i = 0; i < max_number_rows; ++i) {
    row = [];
    for (var j = 0; j < column_length; ++j) {
      if (i < data[j].length) {
        row.push(data[j][i]);
      } else {
        row.push('');
      }
    }
    new_data.push(row);
  }

  alertMessageGroupHandler.addDelayedMessage('Appending sheets to the excel file', 'alert-info');

  ws = XLSX.utils.aoa_to_sheet(new_data, { cellDates: true });
  XLSX.utils.book_append_sheet(wb, ws, 'Params');
  var wbout = XLSX.write(wb, {bookType:'xlsx', bookSST:true, type: 'binary'});
  function s2ab(s) {
    var buf = new ArrayBuffer(s.length);
    var view = new Uint8Array(buf);
    for (var i = 0; i !== s.length; ++i) view[i] = s.charCodeAt(i) & 0xFF;
    return buf;
  }

  alertMessageGroupHandler.addDelayedMessage('Excel file download in progress...', 'alert-info');

  saveAs(new Blob([s2ab(wbout)], {type:"application/octet-stream"}), xlsxFileName);

  // display the success notification and remove the previous ones
  alertMessageGroupHandler.addFinalMessage('Download Finished!', 'alert-success');
};

arc.misc.getCsvDelimiter = function(languageCode) {
  const isCommaDecimal = arc.misc.isCommaDecimal(languageCode);
  return isCommaDecimal ? ';' : ',';
};

arc.misc.downloadCsv = function(
  csvName,
  colnames,
  rows,
  coltypes,
  languageOverride,
  numberCSVRows,
  quotationSetting,
  columns,
  shouldDownload
) {
  const csvContent = [];
  const languageCode = arc.misc.getLanguage(languageOverride);
  const columnSeparator = arc.misc.getCsvDelimiter(languageCode);

  const alertMessageGroupHandler = arc.report.createAlertMessageGroup();
  alertMessageGroupHandler.addDelayedMessage('Preparing data columns for download', 'alert-info');

  // Adding column headers to content
  colnames.forEach(function(col, i) {
    csvContent.push(arc.misc.convertToStrRep(
      col,
      null,
      {
        columnSeparator,
        quotationSetting,
      }
    ));
    if (i < colnames.length - 1) {
      csvContent.push(columnSeparator);
    }
  });

  csvContent.push('\n');

  let rowLength;
  function addCell(col, idx, formatFn) {
    const typeExists = coltypes && coltypes[idx];

    if (typeof formatFn === 'function') {
      col = formatFn(col);
    }

    // Convert 'col' to string and sanitize it
    csvContent.push(arc.misc.convertToStrRep(
      col,
      typeExists ? coltypes[idx] : null,
      {
        columnSeparator,
        quotationSetting,
      }
    ));
    if (idx < rowLength - 1) {
      csvContent.push(columnSeparator);
    }
  }

  alertMessageGroupHandler.addDelayedMessage('Preparing data rows for download', 'alert-info');

  const rowsLength = rows.length;
  let numberRowsExport = numberCSVRows !== undefined && numberCSVRows >= 0 ?
                          Math.min(numberCSVRows, rowsLength) :
                          rowsLength;

  const globalLimit = arc.globals.download_maxrows;

  if (!Number.isNaN(globalLimit)) {
    numberRowsExport = Math.min(numberRowsExport, globalLimit);
  }

  let allColdisplayformats;

  if (!columns) {
    allColdisplayformats = [];
  } else {
    allColdisplayformats = columns.map((col) => col.coldisplayformat(true));
  }

  for (let i = 0; i < numberRowsExport; i++) {
    const row = rows[i];
    rowLength = row.length;
    for (let j = 0; j < rowLength; j++) {
      addCell(row[j], j, allColdisplayformats[j]);
    }
    csvContent.push('\n');
  }

  if (numberCSVRows !== undefined && !shouldDownload) {
    return csvContent;
  }

  alertMessageGroupHandler.addDelayedMessage('CSV file download in progress...', 'alert-info');

  arc.misc.downloadData(csvContent, csvName, 'text/csv');

  alertMessageGroupHandler.addFinalMessage('Download Finished!', 'alert-success');
};

arc.misc.downloadData = function(content, filename, type) {
  type = type || 'text/plain';

  // Use a Blob for very large datasets
  var dataBlob = new Blob(content, {
    type,
  });

  var csvUrl;
  if (window.navigator.msSaveOrOpenBlob) {
    // On IE
    window.navigator.msSaveOrOpenBlob(dataBlob, filename);
  } else {
    // On Chrome, Firefox, Safari
    const csvUrl = URL.createObjectURL(dataBlob);
    let $a = $('<a>').prop('download', filename)
                     .attr('href', csvUrl)
                     .addClass('arc-invisible')
                     .appendTo($('body'));
    $a[0].click();
    $a.remove();
  }
};

arc.misc.handleLegacyLegendSetting = function(saved_settings, legend_style_setting) {
  // This function uses the default value of deprecated setting legend_visibility
  // to set the value of legend_style.
  // Tim TODO: Write a migration to transfer this data instead of having this
  // function in the front-end

  let savedUserSettings = saved_settings.user_settings;
  if ('legend_visibility' in savedUserSettings) {
    let showLegend = savedUserSettings.legend_visibility;
    if (showLegend === 'true') {
      legend_style_setting.value('Right');
    } else if (showLegend === 'false') {
      legend_style_setting.value('None');
    }
  }
};

// Do not call this function.  This is a quick fix to add IE compatibility.
// If you see this method called in your code do the following and remove the call:
// 1. Set with .attr instead of .style if possible
// 2. make sure that your function returns a string in the format '100px'
//    rather than the number 100.  The latter will break in IE
arc.misc.parseD3Style = function(input) {
  // console.log('If you see this message fix and remove this call. See comment in code.');
  if (typeof input === 'number') {
    return `${input}px`;
  } else if (typeof input === 'string') {
    if (input.endsWith('px') || input.endsWith('%')) {
      return input;
    } else {
      return `${input}px`;
    }
  }
};

arc.misc.setWidth = function($sel, w) {
  if (w !== undefined) {
    w = w.trim();
    if (w !== '') {
      const parentWidth = $sel.parent().width();
      const minWidth = 50; // px
      let widthInPixel = w;
      let normalizedWidth;

      if (w[w.length - 1] === '%') {
        widthInPixel = (parentWidth / 100) * parseInt(w.substring(0, w.length - 1), 10);
      }

      if (widthInPixel < minWidth) {
        normalizedWidth = minWidth;
      } else if (widthInPixel > parentWidth) {
        normalizedWidth = parentWidth;
      } else {
        normalizedWidth = widthInPixel;
      }

      $sel.css('width', `${normalizedWidth}px`)
      .css('min-width', `${normalizedWidth}px`);
    } else {
      $sel.css('width', '')
      .css('min-width', '');
    }
  } else {
    $sel.css('width', '')
    .css('min-width', '');
  }
};

// Check every delay milliseconds whether counter1 and counter2 are equal.
// If they are, call the cb function.
// If finalTimeout is specified, invoke cb after finalTimeout milliseconds
//    even if counter1 and counter2 are not equal.
arc.misc.afterCountersEqual = function(config) {
  let {
    counter1,
    counter2,
    delay,
    cb,
    finalTimeout,
    startTime,
  } = config;

  if (arc.counters.data[counter1] === arc.counters.data[counter2]) {
    cb();
    return;
  }

  arc.counters.increment(`${counter1}_delay`);

  if (finalTimeout) {
    if (!startTime) {
      startTime = new Date();
      config.startTime = startTime;
    } else {
      let currTime = new Date();
      if ((currTime - startTime) >= finalTimeout) {
        cb();
        return;
      }
    }
  }

  delay = delay || 500;
  setTimeout(function() {
    arc.misc.afterCountersEqual(config);
  }, delay);
};

/**
    Method for determining if an app is loaded or not. return key
    inputs: (opt.) getDetailedStatus (boolean) if not defined or false returns a binary
      if getDetaled status is defined the return value is one of the following strings:
    noVisual: You are on the home or dataset page, no visuals loaded or to load
    initializing: app is building the queries, no queries have been sent
    query: the app is in the process of sending queries, not all query data has not been received.
    render: data has been received, visuals are being rendered
    finishedSuccess: visuals have been rendered
    finishedError: rendering is done, there are errors with at least one of the visuals.
    undetermined: status unclear
 */
arc.misc.allVisualsLoaded = function(getDetailedStatus) {
  if (!arc.globals.reports) {
    return getDetailedStatus ? 'noVisual' : false;
  }

  const numberOfVisuals = Math.max(1, arc.globals.reports.numberReports()),
    counters = arc.counters.data,
    dataApiStart = counters.dataapi_start ? counters.dataapi_start : 0,
    dataApiEnd = counters.dataapi_end ? counters.dataapi_end : 0,
    vizDrawVisualStart = counters.viz_drawVisual_start ? counters.viz_drawVisual_start : 0,
    vizDrawVisualEnd = counters.viz_drawVisual_end ? counters.viz_drawVisual_end : 0,
    vizErrorEnd = counters.viz_error_end ? counters.viz_error_end : 0;
  const hasErrors = arc.report.logAlertMessages('alert-danger').length > 0;

  // Log counters so that they get captured and shown in server log during snapshotting
  console.log(JSON.stringify(counters));

  if (getDetailedStatus) {
    if (dataApiStart === 0) {
      return 'initializing';
    } else if (dataApiEnd < dataApiStart) {
      return 'query';
    } else if (dataApiStart === dataApiEnd && vizDrawVisualEnd < vizDrawVisualStart) {
      return 'render';
    } else if (vizDrawVisualEnd === vizDrawVisualStart &&
      ((vizDrawVisualEnd + vizErrorEnd) < numberOfVisuals)) {
      return 'render';
    } else if (dataApiStart === dataApiEnd && vizDrawVisualStart === vizDrawVisualEnd &&
      vizDrawVisualEnd >= numberOfVisuals) {
      if (hasErrors) {
        return 'finishedError';
      } else {
        return 'finishedSuccess';
      }
    } else if (vizDrawVisualEnd + vizErrorEnd >= numberOfVisuals) {
      return 'finishedError';
    } else if (dataApiStart > 0 && dataApiStart === dataApiEnd && hasErrors) {
      return 'finishedError';
    }

    return 'undetermined';
  }

  return ((!numberOfVisuals || dataApiStart > 0) &&
          dataApiStart === dataApiEnd  &&
          (!numberOfVisuals || vizDrawVisualStart > 0) &&
          (vizDrawVisualEnd + vizErrorEnd) >= numberOfVisuals ||
          hasErrors);
};

/**
    Method for quickly resetting all the counters used for calculating if visuals are loaded
 */
arc.misc.resetVisualLoadedCounters = function() {
  arc.counters.initialize('dataapi_start');
  arc.counters.initialize('dataapi_end');
  arc.counters.initialize('viz_drawVisual_start');
  arc.counters.initialize('viz_drawVisual_end');
  arc.counters.initialize('viz_error_end');
};

  /**
   * This function gives a limited ability to test the status of custom JS running.
   * This function ONLY tracks the beforeDraw and afterDraw customJS functions and only those
   * functionality that happens inline. No async functions are tracked. If a user would like to
   * track custom JS with greater detail they would need to use the functionality of arc.counters
   * to keep track
   * If no arguments are passed in the function will return true if the number of before/afterDraw
   * customJS functions started is equal to the number finished false otherwise.
   * If an argument is passed in, then the readout will be as follows:
   *
   * none: no beforeDraw nor afterDraw customJS functions have been called
   * inBeforeDraw: started beforeDraw customJS without finishing it
   * inAfterDraw: started afterDraw customJS function without finishing it
   * completed: all before/afterDraw customJS functions have run to completion.
   * errors: Error in before draw or after draw or both
   * undertermined: unable to determine
   */
arc.misc.statusCustomJS = function(getDetailedStatus) {
  const counters = arc.counters.data,
    beforeDrawStart = counters.beforeDrawCustomJS_Start ? counters.beforeDrawCustomJS_Start : 0,
    beforeDrawFinish = counters.beforeDrawCustomJS_Finish ? counters.beforeDrawCustomJS_Finish : 0,
    afterDrawStart = counters.afterDrawCustomJS_Start ? counters.afterDrawCustomJS_Start : 0,
    afterDrawFinish = counters.afterDrawCustomJS_Finish ? counters.afterDrawCustomJS_Finish : 0,
    beforeDrawError = counters.beforeDrawCustomJS_Error ? counters.beforeDrawCustomJS_Error : 0,
    afterDrawError = counters.afterDrawCustomJS_Error ? counters.afterDrawCustomJS_Error : 0;

  if (getDetailedStatus) {
    if (beforeDrawStart === 0 && afterDrawStart === 0) {
      return 'none';
    } else if (beforeDrawStart > beforeDrawFinish) {
      return 'inBeforeDraw';
    } else if (afterDrawStart > afterDrawFinish) {
      return 'inAfterDraw';
    } else if (beforeDrawStart === beforeDrawFinish && afterDrawStart === afterDrawFinish) {
      return 'completed';
    } else if (beforeDrawError > 0 || afterDrawError > 0) {
      return 'errors';
    } else {
      return 'undetermined';
    }
  }

  return beforeDrawStart === beforeDrawFinish && afterDrawStart === afterDrawFinish;
};

arc.misc.addErrors = function(errs, $errLoc, templateId) {
  if (errs.length === 1) {
    $errLoc.append(arc.clone_template(templateId ?? 'col-val-failed',
      { errtext: errs[0] }).html());
  } else if (errs.length > 1) {
    const $err = arc.clone_template(templateId ?? 'col-val-multiple-errors');
    const $ul = $err.find('.error-list');
    errs.forEach(function appendError(err) {
      $ul.append(`<li>${err}</li>`);
    });
    $errLoc.append($err);
  }
};

arc.misc.idxIsNumeric = function(rows, idx) {
  let is_not_numeric = false;

  is_not_numeric = rows.some(function(row) {
    // Use row if idx is undefined, null, empty string, false, NaN
    // Use row[idx] if idx is truthy value or 0
    return isNaN((idx || idx === 0) ? row[idx] : row);
  });

  return !is_not_numeric;
};

arc.misc.makeSparseDsreq = function(_) {
    let dsreq_obj = _;
    let dsreq_nonempty_fields_obj = {};

    for (var property in dsreq_obj) {
      if (dsreq_obj.hasOwnProperty(property)) {
        if ((Array.isArray(dsreq_obj[property]) &&
            dsreq_obj[property].length === 0)) {
          continue;
        } else if (property === 'distinct' &&
            dsreq_obj[property] === false) {
          continue;
        } else if (property === 'global_segments' &&
            dsreq_obj[property][0].segments &&
            dsreq_obj[property][0].segments.length === 0) {
          continue;
        } else if (property === 'autotime_override' &&
            dsreq_obj[property] === '') {
          continue;
        } else if (property === 'nulls_order' &&
            dsreq_obj[property] === 'Default') {
          continue;
        }

        dsreq_nonempty_fields_obj[property] = dsreq_obj[property];
      }
    }

    return dsreq_nonempty_fields_obj;
};

arc.misc.getUniqueId = function() {
  const max = 0x7FFFFFFFFFFFFFFFF - 1;

  return (Math.random() * max) + 1;
};

const formatters = {};

function formatter(str) {
  formatters[str] = formatters[str] || d3.format(str);
  return formatters[str];
}

arc.defaultFormat = {

  number(d) {
    if (!arc.misc.isNumeric(d)) {
      return d;
    }

    const dNum = Number(d);
    const absVal = Math.abs(dNum);
    if (absVal >= 100000) {
      return formatter('.3s')(dNum).replace('G', 'B');
    } else if (Number.isInteger(dNum)) {
      return formatter(',')(dNum);
    } else if (absVal < 0.01) {
      return formatter('.3e')(dNum);
    } else if (absVal < 1) {
      return formatter('.3f')(dNum);
    } else {
      return formatter('.3s')(dNum);
    }
  },

  string(s) {
    return s instanceof Date ? d3.time.format('%Y-%m-%d')(s) : s;
  }

};

arc.misc.d_string = arc.defaultFormat.string;
arc.misc.tickFormatDefaultNumeric = arc.defaultFormat.number;
arc.misc.default_formatter = function(value) {
  if ((value === '') || typeof value === 'string') {
    return arc.defaultFormat.string(value);
  }

  return arc.defaultFormat.number(value);
};
arc.format = arc.format || {};
arc.format.defaultFormatter = arc.misc.default_formatter;

arc.misc.dashboardOverride = function() {
  const dashboard = arc.globals.dashboard;
  if (dashboard) {
    const dashboard_settings = dashboard.user_settings();
    if (dashboard_settings) {
      const override = dashboard_settings.find_by_id('override_visual_font_settings');
      if (override && !override.hidden()) {
        return override.isChecked;
      }
    }
  }

  return undefined;
};

arc.misc.handleDashboardFade = function($changePageLink) {
   const $dashboardFading = $('.dashboard-fade-loading');
   $dashboardFading
    .delay(500)
    .queue(function (next) {
      $dashboardFading.css('opacity', '1');
    });

    if ($changePageLink) {
      $changePageLink.on('click', function (e) {
          e.preventDefault();
          var goTo = this.getAttribute('href');
          $dashboardFading.css('opacity', '0');
          setTimeout(function() {
            window.location = goTo;
          }, 100);
        });
    }
};

// END || Dashboard fade transitions

// function to format array of table names to a single string for display
arc.misc.displayTableNames = function(tableNames) {
  if (tableNames.length <= 2) {
    return tableNames.join(', ');
  } else {
    return `${tableNames.slice(0, 3).join(', ')}, ...`;
  }
};

// Scramble a string with both character_code and string_position transforms
arc.misc.scramble = function(s, posenc=0x0D, chrenc=0x5A) {
  let ss = [];
  for (let i = 0; i < s.length; i++) {
    ss[i ^ posenc] = String.fromCharCode(s[i].charCodeAt(0) ^ chrenc);
  }
  return ss.join('');
};

arc.misc.unobfuscate = function(s) {
  return arc.misc.scramble(s);
};

arc.misc.obfuscate = function(s) {
  return arc.misc.scramble(s + '                '.slice(s.length % 16));
};

arc.misc.localStorage = function() {
  let ls;
  try {
    ls = localStorage;
    const x = '__storage_test__';
    ls.setItem(x, x);
    ls.setItem(x);
    ls.removeItem(x);
  } catch (e) {
    ls = {
      getItem(name) {
        return arc.frontend.vizCookie.get(name);
      },
      setItem(name, value) {
        arc.frontend.vizCookie.set(name, JSON.stringify(value));
      }
    };
  }

  return ls;
};

arc.misc.createFragmentElement = function (config) {
  let { tagName, attributes = {}, classes = [], style, text } = config;
  const elem = document.createElement(tagName);
  Object.keys(attributes).forEach(key => {
    elem.setAttribute(key, attributes[key]);
  });

  if (classes.length) {
    elem.setAttribute('class', classes.join(' '));
  }

  if (typeof style !== 'undefined') {
    elem.setAttribute('style', style);
  }

  if (typeof text !== 'undefined') {
    elem.textContent = text;
  }

  return elem;
};

/* function to generate fragments of the following structure
  <li>
    <div class="form-check">
      <input id="%LABEL%-%VALUE%" data-value="%VALUE%" data-alias="%LABEL%" type="checkbox">
      <label for="%LABEL%-%VALUE%" title="%LABEL%" class="form-check-label">%LABEL%</label>
    </div>
  </li>
*/
arc.misc.createSelectableOption = function(options) {
  const liElem = arc.misc.createFragmentElement({
    tagName: 'li',
  });

  const divElem = arc.misc.createFragmentElement({
    tagName: 'div',
    classes: ['form-check']
  });

  const inputAttributes = {
    'data-value': options.VALUE,
    'data-alias': options.LABEL,
    id: `${options.LABEL}-${options.VALUE}`,
    type: 'checkbox',
  };

  const labelAttributes = {
    title: options.LABEL,
    for: `${options.LABEL}-${options.VALUE}`
  };

  if (options.CHECKED) {
    inputAttributes.checked = true;
  }

  const inputElem = arc.misc.createFragmentElement({
    tagName: 'input',
    attributes: inputAttributes,
    classes: ['form-check-input']
  });

  const labelElem = arc.misc.createFragmentElement({
    tagName: 'label',
    attributes: labelAttributes,

    text: options.LABEL,
    classes: ['form-check-label']
  });

  liElem.appendChild(divElem);

  divElem.appendChild(inputElem);
  divElem.appendChild(labelElem);

  return liElem;
};

arc.misc.updatePicklistSummary = function($picklist, data = {}) {
  const count = data.selectedCount;
  const excluded = data.excluded;

  if (count && (data.individualSelections)) {
    if (!data.isSegmentFilter && count < 3) {
      $picklist.find('.selected-count-wrapper').addClass('arc-invisible');
      $picklist.find('.selected-names-wrapper').removeClass('arc-invisible');

      if (excluded) {
        $picklist.find('.fa-ban').removeClass('arc-invisible');
      } else {
        $picklist.find('.fa-ban').addClass('arc-invisible');
      }

      $picklist.find('.selected-names-wrapper .selected-names').text(` ${data.selectionString} `);
    } else {
        $picklist.find('.selected-count-wrapper').removeClass('arc-invisible');
        $picklist.find('.selected-names-wrapper').addClass('arc-invisible');
    }

    $picklist.find('.selected-count').text(count);

    if (excluded) {
      $picklist.find('.selected-count-suffix').text('Excluded');
    } else {
      $picklist.find('.selected-count-suffix').text('Selected');
    }
  } else {
    $picklist.find('.selected-count-wrapper').removeClass('arc-invisible');
    $picklist.find('.selected-names-wrapper').addClass('arc-invisible');
    $picklist.find('.selected-count').text('');
    $picklist.find('.selected-count-suffix').text('(All)');
  }
};

arc.misc.isIE = function () {
  const isIE11 = !!window.MSInputMethodContext && !!document.documentMode;
  return isIE11 ||
    (window && window.navigator && window.navigator.appName === "Microsoft Internet Explorer");
};

arc.misc.isIEOptimisationEnabled = function () {
  return arc.globals.enableIEOptimisations && arc.misc.isIE();
};

// takes an array sorted based on a comparator function, a target value to look for within that
// array, and start and end indices to look within
// returns found index within range or -1 if can't be found
arc.misc.binarySearch = function(params) {
  const target = params.target;
  if (target === undefined) {
    return 'Must specify target';
  }

  const array = params.array || [];
  const startIdx = params.startIdx || 0;
  const endIdx = params.hasOwnProperty('endIdx') ? params.endIdx : array.length - 1;

  function defaultComparator(a, b) {
    if (a === b) {
      return 0;
    } else if (a > b) {
      return 1;
    } else {
      return -1;
    }
  }

  const comparator = params.comparator || defaultComparator;
  const arrLength = endIdx - startIdx + 1;
  if (arrLength === 1) {
    if (comparator(target, array[startIdx]) === 0) {
      return startIdx;
    } else {
      return -1;
    }
  } else if (arrLength === 0) {
    return -1;
  } else {
    const midIdx = Math.floor((startIdx + endIdx) / 2);
    const midPoint = array[midIdx];
    const compareValue = comparator(target, midPoint);
    if (compareValue === 0) {
      return midIdx;
    } else if (compareValue < 0) {
      return arc.misc.binarySearch({
        target,
        array,
        startIdx,
        endIdx: midIdx,
        comparator
      });
    } else {
      return arc.misc.binarySearch({
        target,
        array,
        startIdx: midIdx + 1,
        endIdx,
        comparator
      });
    }
  }
};

arc.misc.displayVisualType = function(report_type) {
  if (!report_type) {
    return;
  }

  $('#chart-type-menu .active').removeClass('active');
  $(`#chart-${report_type}`).addClass('active');

  const $chartText = $('.selected-viz .text');
  const $chartIcon = $('.selected-viz .chart-icon');

  let c = $chartIcon.attr('data-trackclass');
  if (c) {
    $chartIcon.removeClass(c);
  }
  c = 'chart-icon-' + report_type;

  $chartIcon.attr('data-trackclass', c);
  $chartIcon.addClass(c);
  $chartText.text(arc.misc.chartTypeToName(report_type));
};

arc.misc.displayBaseTables = function(val) {
  function generateSpan(val) {
    return `<span title="${val}" class="truncated-needs-title">${val}</span>`;
  }

  if (val && Array.isArray(val)) {
    return val.map(generateSpan).join('<br>');
  } else {
    return generateSpan(val);
  }
};

arc.misc.hasScriptTag = function(str) {
  const sel = new DOMParser().parseFromString(str, 'text/html').querySelector('script');
  return (sel !== null);
};

arc.misc.arrayMultiSort = function(array, sortObject, sortTotalsAbove, columnsPriority) {
  const sortObjectCopy = Object.assign({}, sortObject);
  const sortKeys = Object.keys(sortObjectCopy || {});

  if (!sortKeys.length) {
      return array;
  }

  Object.keys(sortObjectCopy).forEach((key) => {
    if (sortObjectCopy[key] === arc.consts.SORT_OPTION_UNSORTED) {
      sortObjectCopy[key] = 0;
    } else if (sortObjectCopy[key] === true) {
      sortObjectCopy[key] = 1;
    } else {
      sortObjectCopy[key] = -1;
    }
  });

  const processValue = (value) => {
    if (arc.misc.isNumeric(value)) {
      return Number(value);
    } else if (typeof value === 'string') {
      return value.toLowerCase();
    }
    return value;
  };

  const keySort = (a, b, direction) => {
      direction = direction !== null ? direction : 1;

      const processedA = processValue(a);
      const processedB = processValue(b);

      if (processedA === processedB) {
          return 0;
      }

      if (arc.misc.isCrosstabExpandPrefix(processedA)) {
        if (direction === 0) {
          return 0;
        }
        return sortTotalsAbove ? -1 : 1;
      }

      if (arc.misc.isCrosstabExpandPrefix(processedB)) {
        if (direction === 0) {
          return 0;
        }
        return sortTotalsAbove ? 1 : -1;
      }

      return processedA > processedB ? direction : -1 * direction;
  };

  if (columnsPriority && columnsPriority.length) {
      // Prepare sorting orders based on priority and direction if columnsPriority is defined
      const sortInstructions = columnsPriority
          .map((priority, index) => ({
              index,
              priority,
              direction: sortObjectCopy[index] || 0
          }))
          .filter((instruction) => instruction.direction !== 0) // Exclude unsorted columns
          .sort((a, b) => a.priority - b.priority);

      return array.sort((a, b) => {
          for (const { index, direction } of sortInstructions) {
              const result = keySort(a[index], b[index], direction);
              if (result !== 0) return result;
          }

          return 0;
      });
  }

  // Original logic if columnsPriority is undefined
  return array.sort((a, b) => {
      let sorted = 0;
      let index = 0;

      while (sorted === 0 && index < sortKeys.length) {
          const key = sortKeys[index];
          if (key) {
              const direction = sortObjectCopy[key];
              sorted = keySort(a[key], b[key], direction);
              index += 1;
          }
      }

      return sorted;
  });
};

arc.misc.highlightDiv = function(config) {
  const {
    $div,
  } = config;

  let {
    container,
    fadeOutSeconds,
  } = config;

  fadeOutSeconds = fadeOutSeconds || 10;
  container = container || 'body';
  const $container = $(container);
  const containerOffset = $container.offset();

  $div.each(function() {
    const el = $(this);
    $('<div/>')
      .addClass('highlight')
      .width(el.outerWidth())
      .height(el.outerHeight())
      .css({
        position: 'absolute',
        left: el.offset().left - containerOffset.left + $container.scrollLeft(),
        top: el.offset().top - containerOffset.top + $container.scrollTop(),
        'background-color': '#ffff99',
        opacity: '.7',
        'z-index': '9999999',
      })
      .appendTo(container)
        .fadeOut(fadeOutSeconds * 1000)
        .queue(function() {
          $(this).remove();
        });
  });
};

/**
 * If not all the columns are being displayed in the data detail modal,
 * we warn the user and encourage them to download the csv which includes all
 * of them. This happens when you open click on the View Data option on a
 * visual or open the context menu on a visual and click Show Detailed Data.
 */
arc.misc.toggleColWarningByTotalCount = function(result, viz, excludeAggregates = true) {
  const $dataDetailModal = $('#datadetail-modal');
  const totalColCount = viz.dataset()
                           .tables()
                           .reduce((acc, table) => {
                            if (excludeAggregates) {
                              return acc + table
                                  .columns()
                                  .filter((column) => !column.attr('exprhasagg')).length;
                            }

                            return acc + table.columns().length;
                           }, 0);
  const $columnCountWarning = $dataDetailModal.find('.column-count-warning');
  if (result.columns().length < totalColCount) {
    $columnCountWarning.removeClass('arc-invisible');
  } else {
    $columnCountWarning.addClass('arc-invisible');
  }
};

arc.misc.duplicatedScheduleGroupWarning = function(existingScheduleGroupNames) {
  return existingScheduleGroupNames.length ? `This interval is a duplicate of the following shedule group(s): ${existingScheduleGroupNames.join(', ')}` : '';
};

function setupContextMenu(tagetAnchor, horizontalPosition, verticalPozition) {
  const $contextmenu = $('#mlexplore-contextmenu');
  const url = arc.misc.getBlankTargetUrl(tagetAnchor.href);
  $contextmenu.find('.copy-to-clipboard').offOn('click.copyToClipboard', () => {
    arc.misc.copyToClipboard(url);
  });

  $contextmenu.find('#open-in-new-tab').attr('href', url);

  $contextmenu.css('left', horizontalPosition);
  $contextmenu.css('top', verticalPozition);
  $contextmenu.show();
}

arc.misc.overrideOpenInTabOrWindowEvents = function () {
  const $contextMenu = $('#mlexplore-contextmenu');
  function getAnchor(e) {
    const target = e.target instanceof HTMLAnchorElement ? e.target : e.target.closest('a');
    if (target && target.href && !target.classList.contains('mlexplore-contextmenu-link')) {
      return target;
    }

    return null;
  }

  document.addEventListener('mousedown', (e) => {
    const isMiddleButtonClick = e.button === 1 || e.buttons === 4;
    const target = getAnchor(e);
    if (isMiddleButtonClick && target) {
      e.preventDefault();
      arc.misc.openInNewWindow(target.href);
      $contextMenu.hide();
    }
  });

  document.addEventListener('click', (e) => {
    const isCmdPlusLeftClick = e.button === 0 && e.metaKey;
    const target = getAnchor(e);
    if (isCmdPlusLeftClick && target) {
      e.preventDefault();
      arc.misc.openInNewWindow(target.href);
    }

    $contextMenu.hide();
  });

  document.addEventListener('contextmenu', (e) => {
    const target = getAnchor(e);
    if (target) {
      e.preventDefault();
      setupContextMenu(target, e.pageX, e.pageY);
    }
  });
};

arc.misc.drawTable = function(containerDivId, rows, colnames, settings, tableConfiguration) {
  $(containerDivId).empty();

  const table = arc.visual.plugins.table();
  const driver = arc.report.driver(table)
                    .params(arc.params.tracker());

  driver.sb_rows(rows);
  driver.sb_colnames(colnames);

  const tableSettings = driver.user_settings();
  tableSettings.find_by_id('table_template')
              .value('table-template-6');
  tableSettings.find_by_id('enable_pagination')
              .value('false');
  tableSettings.find_by_id('download_rows')
              .value('');
  tableSettings.find_by_id('enable_pagination')
               .value(tableConfiguration?.enablePagination ? 'true' : 'false');
  tableSettings.find_by_id('enable_info')
               .value(tableConfiguration?.enable_info ? 'true' : 'false');

  if (tableConfiguration?.pageSize) {
    tableSettings.find_by_id('page_size')
    .value(`${tableConfiguration.pageSize}`);
  }

  if (settings) {
    settings.forEach((setting) => {
      tableSettings.find_by_id(setting.key)
                   .value(setting.value);
    });
  }

  const cols = colnames.map(
      (colname, i) => arc.data.column()
                         .colname(colname)
                         .coltype('string')
                         .colproperty(arc.report.columnTypeEnum.DIMENSION)
                         .colsource('dimensions_shelf')
                         .colindex(i)
                         .order(
                          tableConfiguration?.orderBy?.colname === colname ?
                            tableConfiguration.orderBy.order :
                            undefined
                          )
  );

  const result = arc.data.result();
  result.columns(cols);
  result.rows(rows);
  result.finalize();

  table.viz_setDataResult(result);
  table.setConfiguration({
    cellsHtml: tableConfiguration?.cellsHtml ?? false,
    showTooltip: tableConfiguration?.showTooltip ?? false,
    disableProfile: tableConfiguration?.disableProfile ?? true,
    enablePagination: tableConfiguration?.enablePagination ?? false,
    pageSize: tableConfiguration?.pageSize,
    enableInfo: tableConfiguration?.enableInfo,
  });
  table.viz_drawVisual(`${containerDivId}`);
  table.adjustColumnWidths();

  return table;
};

arc.misc.customSummerNoteTabs = () => {
  const customIndent = (context) => {
    const { ui } = $.summernote;

    const button = ui.button({
      contents: '<i class="note-icon-align-indent"></i>',
      tooltip: 'Indent',
      click() {
        context.invoke('editor.indent', 'render');
      },
    });

    return button.render();
  };

  const customOutdent = (context) => {
    const { ui } = $.summernote;

    const button = ui.button({
      contents: '<i class="note-icon-align-outdent"></i>',
      tooltip: 'Outdent',
      click() {
        context.invoke('editor.outdent', 'render');
      },
    });

    return button.render();
  };

  return {
    buttons: { customIndent, customOutdent },
    toolbar: [
      ['indent', ['customIndent']],
      ['outdent', ['customOutdent']],
    ],
  };
};

/*
  This popover allows the user to click/mark/copy text on/from it.
  It appears on hovering on $popoverSelector DOM element(for example an info cion),
  and disappears when the mouse leaves the popover itself!
*/
arc.misc.setupMarkablePopover = (config) => {
  const {
    $popoverSelector,
    content,
    popoverId = `popover-visual-${arc.misc.uniqueId()}`,
    setting: customSettings
  } = config;

  let popoverTimeout;
  const setting = {
    container: 'body',
    html: true,
    animation: false,
    trigger: 'manual',
    placement: 'auto',
    title: '',
    content,
    sanitize: false,
    ...customSettings
  };

  if ($popoverSelector.data('popoverActivated')) return;

  $popoverSelector.data('popoverActivated', true);
  $popoverSelector.popover(setting)
  .on('mouseenter', function () {
    const $this = $(this);

    if (popoverTimeout) {
      return;
    }

    popoverTimeout = setTimeout(() => {
      popoverTimeout = undefined;
      $this.popover('show');
      $('.popover').on('mouseleave', () => {
        $this.popover('hide');
      });
    }, 300);
  }).on('mouseleave', function () {
    const $this = $(this);

    if (popoverTimeout) {
      clearTimeout(popoverTimeout);
      popoverTimeout = undefined;
      return;
    }

    setTimeout(() => {
      if (popoverTimeout) {
        clearTimeout(popoverTimeout);
        popoverTimeout = undefined;
        return;
      }

      if (!$(`.${popoverId}:hover`).length) {
        $this.popover('hide');
      }
    }, 300);
  });
};

arc.misc.setupQueryPopover = (element, queryTxtEl) => {
  const $queryInfo = $(element);
  function formatContent(content) {
    let formattedContent;

    try {
      const parsedJson = JSON.parse(content);
      formattedContent = JSON.stringify(parsedJson, null, 2);
    } catch (error) {
      try {
        formattedContent = formatDialect(content, { dialect: sql });
      } catch {
        formattedContent = content;
      }
    }

    return formattedContent;
  }

  $queryInfo.popover({
    trigger: 'click',
    html: true,
    placement: 'right',
    title: 'Full query <button type="button" class="query-close close" data-dismiss="popover" aria-hidden="true">×</button>',
    boundary: 'window',
    sanitize: false,
    content() {
      let query = $(this).siblings(queryTxtEl).text();
      query = formatContent(query);

      return `
          <pre class="sql-copy">${query}</pre>
          <div class="text-right"><button type="button" class="btn btn-secondary btn-sm queries-copy">Copy to Clipboard</button></div>
        `;
    }
  });

  $(document).offOn('inserted.bs.popover', element, () => {
    $(document).offOn('click.copy', 'button.queries-copy', () => {
      const text = $('.sql-copy').text();

      arc.misc.copyToClipboard(text)
          .then(() => {
            $('button.queries-copy').html('Copied to Clipboard').css('border', 'solid 1px #28A745');
          });
    });
  });

  $(document).offOn('show.bs.popover', element, (event) => {
      $('.popover').not(this).popover('hide');
      if (element === '.query-sql-link') {
        const textEl = event.currentTarget;
        textEl.textContent = 'Hide SQL';
      }
    });

  $(document).offOn('click.dismiss', '.query-close', () => {
    $queryInfo.popover('hide');
  });

  $(document).offOn('click.dismissOutsidePopover', (e) => {
    if ($(e.target).data('toggle') !== 'popover' &&
        $(e.target).parents('.popover.in').length === 0) {
      $('[data-toggle="popover"]').popover('hide');
    }
  });

  $(document).offOn('hide.bs.popover', element, (e) => {
    if (element === '.query-sql-link') {
      const textEl = e.currentTarget;
      textEl.textContent = 'Show SQL';
    }
  });
};

arc.misc.exChangeElementsVisibility = ({ elementToShow, elementToHide }) => {
  elementToHide.hide();
  elementToShow.show();
};

arc.misc.plotTable = (rows, colnames, $container, simpleTable) => {
  $container.empty();
  $container.addClass('arc-invisible');

  const table = arc.visual.plugins.table();
  const driver = arc.report.driver(table)
                 .chart_div($container.attr('id'))
                 .params(arc.params.tracker());

  // No column limits
  const settings = driver.user_settings();
  settings.find_by_id('max_columns')
          .value('-1');

  settings.find_by_id('table_template')
          .value(simpleTable ? 'table-template-4' : 'table-template-3');
  if (simpleTable) {
    settings.find_by_id('enable_pagination')
            .value('false');
  } else {
    settings.find_by_id('enable_pagination')
            .value('true');
    settings.find_by_id('page_size')
            .value('10');
  }

  const cols = [];
  colnames.forEach(function(colname, coli) {
    cols.push(arc.data.column()
              .colname(colname)
              .coltype('STRING')
              .colproperty(arc.report.columnTypeEnum.DIMENSION)
              .colsource('dimensions_shelf')
              .colindex(coli));
  });

  const result = arc.data.result();
  result.columns(cols);
  result.rows(rows);
  result.finalize();

  // create the visual by setting the data and then drawing it
  table.viz_setDataResult(result);

  $container.removeClass('arc-invisible');
  table.setConfiguration({
    cellsHtml: false,
    showTooltip: false,
    enableInfo: true,
  });

  table.viz_drawVisual(`${$container.attr('id')}`);
  table.adjustColumnWidths();
  return table;
};

arc.misc.createCSVorXLSXfile = (numberCSVRows, include_xlsx, filetype) => {
  const returnData = {};
  const delimiters = {};
  arc.globals.reports.vizCallback(function getCSVData(reportBase) {
    const isCrosstab = reportBase.viz_type() === 'crosstab';
    const isEmptyCrosstab = reportBase.sb_rows().length === 0;
    const needIncludeXlsx = Boolean(include_xlsx);
    const isXlsxFileType = filetype === 'xlsx';

    const needFormattingOnCrosstabWithData = (
      isCrosstab &&
      needIncludeXlsx &&
      isXlsxFileType &&
      !isEmptyCrosstab
    );

    if (arc.report.noResultExcludeTypes.has(reportBase.viz_type()) || reportBase.viz_type() === 'chatbot') {
      return;
    }

    let separator = '';
    if (reportBase.title()) {
      separator = '_';
    }

    const visualTitle = reportBase.report_id() + separator + reportBase.title();

    returnData[visualTitle] = needFormattingOnCrosstabWithData ?
    reportBase.viz().exportCSVForCrosstab() :
    reportBase.exportCSV({ numberCSVRows });

    delimiters[visualTitle] = reportBase.getCsvDelimiter();
  });

  return {
    returnData,
    delimiters,
  };
};

arc.misc.vizHTMLInfo = () => `
  <div class="viz-html-info">
    Click <a class="viz-html-view" data-test-id="viz-html-view">here</a> to open the visual in full-screen mode and
    edit the visual
  </div>
`;

arc.misc.vizHTMLInfoNoPermission = () => `
  <div class="viz-html-info">
    You don't have Manage custom styles permission to edit this type of visual. Please contact the application admin.
  </div>
`;
}());
