window.arc = window.arc || {};

(function() {
'use strict';

arc.misc = arc.misc || {};

arc.misc.columnDeduper = function() {
  let c = function() { return 'columnDeduper'; };

  // KSQL cannot handle duplicate expressions for dimensions or measures
  // Handle them in the ui instead by de-duplicating requests on the way in
  // and creating new columns on the data on the way out
  let columnsMap = {},
      totalDupCount = 0,
      dedupedDims = [],
      dedupedAggs = [],
      orderCols = [];

  c.transformRows = function(rows) {
    if (totalDupCount === 0) {
      return rows;
    }

    let newRows = [];
    rows.forEach(function(row) {
      let newRow = [];
      Object.keys(columnsMap).forEach(function(k) {
        newRow[k] = row[columnsMap[k]];
      });

      newRows.push(newRow);
    });

    return newRows;
  };

  c.dedupedDims = function() {
    return dedupedDims;
  };

  c.dedupedAggs = function() {
    return dedupedAggs;
  };

  c.orderCols = function() {
    return orderCols;
  };

  c.initializeWithRB = function(rb) {
    columnsMap = {};
    dedupedDims = [];
    dedupedAggs = [];
    orderCols = [];
    totalDupCount = 0;

    let dupCount = 0,
        dupTracker = {},
        dims = rb.dimensions(),
        aggs = rb.aggregates(),
        dimCount = 0,
        totalDimCount = dims.length;

    dims.forEach(function(col, i) {
      let dup = dupTracker[col.expr];
      if (dup !== undefined) {
        columnsMap[i] = dup;
        ++dupCount;
        ++totalDupCount;
      } else {
        ++dimCount;
        columnsMap[i] = i - dupCount;
        dedupedDims.push(col);
        dupTracker[col.expr] = i - dupCount;
      }

      if (col.order) {
        orderCols.push({
          order: col.order,
          colIndex: dup ? dup : i,
        });
      }
    });

    dupTracker = {};
    dupCount = 0;
    aggs.forEach(function(col, i) {
      let dup = dupTracker[col.expr];
      if (dup !== undefined) {
        columnsMap[totalDimCount + i] = dup;
        ++dupCount;
        ++totalDupCount;
      } else {
        columnsMap[totalDimCount + i] = dimCount + i - dupCount;
        dedupedAggs.push(col);
        dupTracker[col.expr] = dimCount + i - dupCount;
      }

      if (col.order) {
        orderCols.push({
          order: col.order,
          colIndex: dimCount + (dup ? dup : i),
        });
      }
    });

    return c;
  };

  return c;
};

}());
