window.arc = window.arc || {};

(function() {
'use strict';

arc.ping = arc.ping || {};

/*
 * Helper function to ping the backend server and verify ping times.
 * In order to use this, use the developer console on a connected
 * arcviz session, and then type in:
 *
 *   arc.ping.tester().ping(10);
 *
 * Note that 10 is the number of pings to do, with a gap of 1 second
 *   between calls to the server.
 *
 * This will be impacted by server response time, by network latency,
 *   and by any queueing the browser is doing if there are a lot of
 *   ongoing requests.
 */
arc.ping.tester = function() {
  var tester = function() {
    return 'arc.ping.tester';
  };

  const url = arc.frontend.globals.urls.siteSettingsUrl;

  tester.ping = function(count) {
    if (!count) {
      count = 1;
    }

    pingInternal(count);
    return tester;
  };

  const stats = [];

  tester.stats = function() {
    return stats;
  };

  tester.printSummary = function() {
    var timeGetter = function(d) {
      return d[0];
    };

    var min = d3.min(stats, timeGetter),
        max = d3.max(stats, timeGetter),
        median = d3.median(stats, timeGetter);

    var failures = 0;
    stats.forEach(function(s) {
      if (s[1]) {
        ++failures;
      }
    });

    console.log('SUMMARY: min', min, 'max', max, 'median', median,
        'failures', failures);
  };

  tester.printStats = function() {
    stats.forEach(function(stat) {
      var diffMs = stat[0],
          failed = stat[1];

      if (failed) {
        console.log(diffMs, 'FAILED');
      } else {
        console.log(diffMs);
      }
    });

    return tester;
  };

  function pingInternal(count) {
    if (!count) {
      count = 1;
    }

    var startTime = new Date();

    function nextIteration(failed) {
      var endTime = new Date();
      var diffMs = endTime - startTime;

      stats.push([diffMs, failed]);
      if (failed) {
        console.log('Ping Failure: ', diffMs);
      } else {
        console.log('Ping: ', diffMs);
      }

      --count;
      if (count > 0) {
        setTimeout(function() {
          pingInternal(count);
        }, 1000);
      } else {
        tester.printSummary();
      }
    }

    $.ajax({
      url: url,
      type: 'GET',
    })
    .fail(function(req, textstatus, errorthrown) {
      nextIteration(/* failed */ true);
    })
    .done(function() {
      nextIteration();
    });
  }

  return tester;
};

}());
