window.arc = window.arc || {};

(function() {
'use strict';

arc.sql = arc.sql || {};

arc.sql.displayedQuery = function() {
  const q = function() { return 'arc.sql.displayedQuery'; };

  let query,
      bookmark_id,
      username,
      dataconnection_id,
      dataconnection_name,
      favorite,
      query_time;

  q.query = function(_) {
    if (!arguments.length) return query;
    query = _;
    return q;
  };

  q.bookmark_id = function(_) {
    if (!arguments.length) return bookmark_id;
    bookmark_id = _;
    return q;
  };

  q.username = function(_) {
    if (!arguments.length) return username;
    username = _;
    return q;
  };

  q.dataconnection_id = function(_) {
    if (!arguments.length) return dataconnection_id;
    dataconnection_id = _;
    return q;
  };

  q.dataconnection_name = function(_) {
    if (!arguments.length) return dataconnection_name;
    dataconnection_name = _;
    return q;
  };

  q.favorite = function(_) {
    if (!arguments.length) return favorite;
    favorite = _;
    return q;
  };

  q.query_time = function(_) {
    if (!arguments.length) return query_time;
    query_time = _;
    return q;
  };

  q.fromParams = function(d) {
    if (d.bookmark_id) {
      bookmark_id = d.bookmark_id;
    }

    if (d.query) {
      query = d.query;
    }

    if (d.dataconnection_id) {
      dataconnection_id = d.dataconnection_id;
    }

    if (d.dataconnection_id) {
      dataconnection_name = d.dataconnection_name;
    }

    if (d.favorite) {
      favorite = d.favorite;
    }

    if (d.query_time) {
      query_time = moment.utc(d.query_time * 1000);
    }

    return q;
  };

  q.toParams = function() {
    const params = {
      query,
    };

    if (bookmark_id) {
      params.bookmark_id = bookmark_id;
    }

    if (dataconnection_id) {
      params.dataconnection_id = dataconnection_id;
    }

    if (dataconnection_name) {
      params.dataconnection_name = dataconnection_name;
    }

    if (favorite) {
      params.favorite = true;
    }

    return params;
  };

  q.delete = function() {
    const params = q.toParams();
    $.ajax({
      url: arc.frontend.globals.urls.savedSqlUrl,
      data: params,
      type: 'DELETE',
      beforeSend: function (xhr) {
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    });

    return q;
  };

  q.store = function() {
    const params = q.toParams();
    $.ajax({
      url: arc.frontend.globals.urls.savedSqlUrl,
      data: params,
      type: 'POST',
      beforeSend: function (xhr) {
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    });

    return q;
  };

  q.fetch = function(cb) {
    const params = q.toParams();

    $.ajax({
      url: arc.frontend.globals.urls.savedSqlUrl,
      data: params,
      type: 'GET',
    })
    .done(function(data) {
      const displayedQueries = data.map(function(d) {
        return arc.sql.displayedQuery()
                      .fromParams(d);
      });

      cb(displayedQueries);
    })
    .fail(function(req, textstatus, errorthrown) {
      arc.report.alertMessage('Error loading saved queries', req.responseText);
    });
  };

  return q;
};

arc.sql.getQueryById = async function(id, favorite) {
  try {
    const response = await arc.frontend.vizHttp.get({
      url: arc.frontend.globals.urls.savedSqlUrl,
      params: { get_type: 'get_by_id', query_id: id, favorite },
    });

    return response.data;
  } catch (error) {
    arc.report.alertMessage('Error loading query', error.responseText);
  }
};
}());
