window.arc = window.arc || {};

(function() {
"use strict";

arc.apidemo = arc.apidemo || {};

arc.apidemo.apidemo = function() {
  var apidemo = function() { return "arc.apidemo.apidemo"; };

  const baseurl = arc.frontend.globals.urls.reportUrl.replace('reports/report', 'adminapi/v1');

  var responses = {};

  // A table to control the order of object key display
  var tabkey_order = {
    visuals:      ['id', 'title', 'type', 'description',
                   'created', 'created_by', 'updated', 'updated_by',
                   'dataset_id', 'dataset_name', 'dataconn_name',
                   'workspace_id', 'workspace_name', 'perm', 'shared_visual_dashboards',
                   'url', 'img', 'baseImg', 'data'],
    datasets:     ['id', 'name', 'description', 'created', 'created_by',
                   'type', 'detail', 'dc_id', 'dc_name', 'lvname',
                   'perms', 'settings', 'info', 'lvcolumns'],
    connections:  ['id', 'name', 'type', 'info',
                   'created_by', 'created', 'updated_by', 'updated',
                   'password'],
    users:        ['id', 'username', 'is_superuser',
                   'is_active', 'date_joined', 'last_login', 'first_name', 'last_name',
                   'groups', 'roles', 'profile'],
    groups:       ['id', 'name', 'users', 'roles'],
    roles:        ['id', 'name', 'desc', 'require_all_groups', 'users', 'groups', 'privs'],
    segments:     ['id', 'name', 'dataset_id',
                   'created_by', 'created', 'updated_by', 'updated', 'data'],
    filterassociations:
                  ['id', 'name', 'dataset_id',
                   'created_by', 'created', 'updated_by', 'updated', 'data', 'users', 'groups'],
    workspaces:   ['id', 'name', 'desc', 'editable', 'private_user_id', 'perms', 'acl'],
  };

  apidemo.initialize = function(types) {

    var text = $("#tab-pane-template").html();
    $(".tab-pane.apidemo").each(function (idx, div) {
      var $div = $(div);
      var tab = $div.attr('id');
      if (types.indexOf('*') !== -1 || types.indexOf(tab) !== -1) {
        $div.html(text);
        $('#tab-' + tab).removeClass('arc-invisible');
      }
    });

    var $tabs = $("#demotabs");
    $tabs.on('click', '.fetch', function(e) {
      var $div = $(this).closest(".tab-pane");
      fetch($div);
    });

    $tabs.on('click', '.update,.create', function(e) {
      var $div = $(this).closest(".tab-pane");
      update($div);
    });

    $tabs.on('click', '.clone', function(e) {
      var $div = $(this).closest(".tab-pane");
      clone($div);
    });

    $tabs.on('click', '.delete', function(e) {
      const $div = $(this).closest(".tab-pane");
      const tabName = `${$div.attr('id').charAt(0).toUpperCase()}${$div.attr('id').slice(1)}`;
      const id = $div.find('.fetch-id').val();
      const delete_dialog = arc.misc.deleteConfirmation();

      delete_dialog.objectType(tabName)
       .objectName(id)
       .askUser(function() {
         do_delete($div);
         delete_dialog.hideModal();
       });

    });

    // Do fetch on CR entered in the ID box
    $tabs.on('keyup', '.fetch-id', function(e) {
      if (e.which === 13) {
        var $div = $(this).closest(".tab-pane");
        $div.find('.fetch').trigger('click');
      }
    });
  };

  // Report an error from an ajax call
  function reportError(rsp, errorthrown, textstatus, msg) {
    var err;
    if (textstatus === "NOT FOUND") {
      err = "URL not found";
    } else {
      err = rsp.responseText || textstatus;
    }
    arc.report.alertMessageSmall("Error " + msg + ": " + err, "alert-danger");
  }

  // Update the URL with the given text
  function setURL($div, text) {
    $div.find('.fetch-url').html(text);
  }

  // Build object from input in the order of the given keys
  function orderObject(o, tab) {

    var okeys = Object.keys(o);
    var keys = tabkey_order[tab].slice();

    // Remove any keys not in the object
    keys = tabkey_order[tab].filter((k) => (okeys.indexOf(k) !== -1));

    // Add in the object keys we don't know about
    keys = keys.concat(okeys.filter((k) => (keys.indexOf(k) === -1)));

    // Now build an object with keys in this order
    return keys.reduce((r, k) => (r[k] = o[k], r), {});
  }

  function setResponse($container, tab, rsp) {
    responses[tab] = rsp;

    const items = [];

    if (!rsp) {
      $container.find('.fetched-text').text('');
    } else {
      rsp.forEach((i) => {
        items.push(orderObject(i, tab));
      });
      let text = JSON.stringify(items, undefined, 2);
      text = text.replace(/\\n/g, '&#13;&#10;');
      text = text.replace(/</g, '&lt');
      $container.find('.fetched-text').text(text);
    }

    if (items.length === 1 &&
        items[0].id !== undefined &&
        $container.find('input.fetch-detail').is(':checked')) {
      $container.find('.clone').removeClass('arc-invisible');
    } else {
      $container.find('.clone').addClass('arc-invisible');
    }

    if (items.length === 1) {
      if (items[0].id !== undefined) {
        $container.find('.fetch-id').val(items[0].id);
        $container.find('.delete').removeClass('arc-invisible');
        $container.find('.update').removeClass('arc-invisible');
        $container.find('.create').addClass('arc-invisible');
      } else {
        $container.find('.delete').addClass('arc-invisible');
        $container.find('.update').addClass('arc-invisible');
        $container.find('.create').removeClass('arc-invisible');
      }
    } else {
      $container.find('.delete').addClass('arc-invisible');
      $container.find('.update').addClass('arc-invisible');
      $container.find('.create').addClass('arc-invisible');
    }
  }

  function clone($div) {

    var tab = $div.attr('id');
    var single = tab.slice(0, -1);
    var rsp = responses[tab];
    if (rsp && rsp.length === 1 && rsp[0].id) {
      $div.find('.fetch-id').val('');
      if (rsp[0].name) {
        rsp[0].name = 'Clone of ' + single + ' ' + rsp[0].name;
      }
      delete rsp[0].id;
      setResponse($div, tab, rsp);
    }
  }

  function fetch($div) {

    arc.report.alertMessageSmallHide();

    var tab = $div.attr('id');
    var url = baseurl + '/' + tab;

    var options = [];

    var $id_input = $div.find('.fetch-id');
    var idtxt = $id_input.val();
    var id;
    if (idtxt) {
      id = parseInt(idtxt);
      if (isNaN(idtxt)) {
        options.push('name=' + encodeURIComponent(idtxt));
      } else {
        url += '/' + id;
      }
      $div.find('.update').removeClass('arc-invisible');
    } else {
      $div.find('.update').addClass('arc-invisible');
    }

    var $filter_input = $div.find('.fetch-filter');
    var filter_text = $filter_input.val();
    if (filter_text) {
      if (id) {
        arc.report.alertMessageSmall("Only one of ID and Filter fields can be set",
                                     "alert-danger");
        return;
      }

      try {
        JSON.parse(filter_text);
      } catch (err) {
        arc.report.alertMessageSmall("Filter must be in JSON format:" + err.toString(),
                                     "alert-danger");
        return;
      }
      options.push('filter=' + btoa(filter_text));
    }

    if ($div.find('input.fetch-detail').is(':checked')) {
      options.push('detail=1');
    }

    if (options.length) {
      url += '?' + options.join('&');
    }

    setURL($div, 'GET ' + url);
    setResponse($div, tab);

    $.ajax({
      url: url,
      type: 'GET',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .fail((rsp, textstatus, errorthrown) => {
      reportError(rsp, textstatus, errorthrown, "fetching data");
    })
    .done((rsp) => {
      setResponse($div, tab, rsp);
    });
  }

  function do_delete($div) {

    var tab = $div.attr('id');
    var single = tab.slice(0, -1);
    var url = baseurl + '/' + tab;

    var $id_input = $div.find('.fetch-id');
    var id = $id_input.val();
    if (id) {
      id = parseInt(id);
      if (isNaN(id)) {
        arc.report.alertMessageSmall("ID must be integer", "alert-danger");
        return;
      }
      url += '/' + id;
    } else {
      arc.report.alertMessageSmall("Must specify " + single +
                                   " ID to delete", "alert-danger");
      return;
    }

    setURL($div, 'DELETE ' + url);
    $.ajax({
      url: url,
      type: 'DELETE',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .fail((rsp, textstatus, errorthrown) => {
      reportError(rsp, textstatus, errorthrown, "deleting " + single + " ID " + id);
    })
    .done(() => {
      arc.report.alertMessageSmall("Deleted " + single + " ID " + id + "!", "alert-success");
    });
  }

  function update($div) {

    var tab = $div.attr('id');
    var single = tab.slice(0, -1);
    var url = baseurl + '/' + tab;

    var $id_input = $div.find('.fetch-id');
    var id = $id_input.val();
    if (id) {
      id = parseInt(id);
      if (isNaN(id)) {
        arc.report.alertMessageSmall("ID must be integer", "alert-danger");
        return;
      }
      url += '/' + id;
    }

    var data = $div.find(".fetched-text").text();
    data = data.replace(/\015\012/g, '\\n');
    data = data.replace(/&lt/g, '<');
    try {
      data = JSON.parse(data);
    } catch (err) {
      arc.report.alertMessageSmall("Error on text to JSON:" + err.toString(), "alert-danger");
      return;
    }

    setURL($div, 'POST ' + url);

    $.ajax({
      url: url,
      type: 'POST',
      data: {'data':JSON.stringify(data)},
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .fail((rsp, textstatus, errorthrown) => {
      var action = id ? 'updating' : 'creating';
      reportError(rsp, textstatus, errorthrown, action + single);
    })
    .done((rsp) => {
      arc.report.alertMessageSmall("Update Success!", "alert-success");
      setResponse($div, tab, rsp);
    });
  }

  return apidemo;

};

}());
