window.arc = window.arc || {};

(function() {
'use strict';

arc.screen = arc.screen || {};
arc.report = arc.report || {};
arc.report.superModal = arc.report.superModal || {};

arc.screen.displayapp = function() {
  var displayapp = function() { return 'arc.screen.displayapp'; };

  var is_dashboard,
      dashboard,
      msdb,
      app_id,
      initialSheetId,
      current_dbid,
      autoRefreshSeconds,
      visual_type,
      dataset_id,
      globalFilters,
      snapshotAdmin,
      emailApp,

      // For dashboard dropdown
      isFirstSave,
      $hideIfNewNavbarItems,
      $buttonWrapper;

  displayapp.globalFilters = function(_) {
    if (!arguments.length) return globalFilters;
    globalFilters = _;
    return displayapp;
  };

  displayapp.msdb = function () {
    return msdb;
  };

  displayapp.dashboard = function () {
    return dashboard;
  };

  displayapp.visual_type = function(_) {
    if (!arguments.length) return visual_type;
    visual_type = _;
    return displayapp;
  };

  displayapp.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    if (isNaN(_)) {
      dataset_id = _;
    } else {
      dataset_id = parseInt(_);
    }

    return displayapp;
  };

  displayapp.is_dashboard = function(_) {
    if (!arguments.length) return is_dashboard;
    is_dashboard = _;
    return displayapp;
  };

  displayapp.app_id = function(_) {
    if (!arguments.length) return app_id;
    app_id = _;
    if (app_id === '') {
      app_id = undefined;
    } else {
      app_id = Number(app_id);
    }

    return displayapp;
  };

  displayapp.initialSheet = function (_) {
    if (!arguments.length) return initialSheetId;
    initialSheetId = ((!_) ? undefined : Number(_));

    return displayapp;
  };

  displayapp.autoRefreshSeconds = function(_) {
    if (!arguments.length) return autoRefreshSeconds;
    autoRefreshSeconds = _;
    return displayapp;
  };

  displayapp.preload_assets = function(assets) {
    try {
      assets = JSON.parse(assets);
      if (assets) {
        arc.staticfetcher.populate(assets);
      }
    } catch (e) {
      console.log('Error parsing preload assests', e);
    }

    return displayapp;
  };

  function setupResizeDashboard() {
    // When this page is resized, re-set up gridster with new base widget dims and
    // redraw all the widgets in the dashboard
    // Do this once in 500 milli seconds
    $(window).off('resize')
             .on('resize', function() {
               if (arc.screen.resizeEnd) {
                 clearTimeout(arc.screen.resizeEnd);
               }

               arc.screen.resizeEnd = setTimeout(function() {
                 if (dashboard) {
                   dashboard.resizeCallback();
                 }
               }, 200);
             });
  }

  displayapp.setupResizeVisual = function() {
    $(window).offOn('resize', () => {
              const currReport = arc.globals.reports.getPrimaryReport();
              // TODO: Figure out why the currReport doesn't have the correct report type
              const visualType = currReport.viz().viz_type();
              if (['rawtable', 'html', 'extension', 'column-stats'].includes(visualType)) return;

               if (arc.screen.resizeEnd) {
                 clearTimeout(arc.screen.resizeEnd);
               }

               arc.screen.resizeEnd = setTimeout(function() {
                 currReport.refresh(true /* force_redraw */);
               }, 500);
             });
  };

  function setupTooltips() {
    // bind tooltip to generated content
    $('body').tooltip({
      selector: '[data-toggle=tooltip]',
      container: 'body',
      delay: {
        show: 500,
        hide: 100,
      },
    });
  }

  const popoverText = $('#chart-content-popover').html();

  function setupPopovers() {
    $('.app-avail-list li').popover({
      container: 'body',  // makes popover be on top
      html: true,
      trigger: 'hover',
      title: '',
      sanitize: false,
      content: function() {
        return popoverText;
      },
    });
  }

  function setupSave() {
    function stopSaveButtonSpinning() {
      $('#app-save-spin').css('display', 'none');
      $('#app-save-icon').css('display', 'inline-block');
    }

    function dashboard_save_cb(data) {
      if (current_dbid !== data) {
        current_dbid = data;
        msdb.dashboard_id(data);
        let $dashboardDropdown = $('.subnav-report');
        setupDashboardDropdown($dashboardDropdown);
        msdb.setupMyFavoriteApps(true, current_dbid, $dashboardDropdown);
      }

      stopSaveButtonSpinning();
      arc.report.pagenav.needs_save(false);
      if (isFirstSave) {
        $hideIfNewNavbarItems.removeClass('arc-invisible');
        $buttonWrapper.removeClass('arc-invisible');
        dashboard.updateAllDropdowns();
        isFirstSave = false;
      }
    }

    function dashboard_error_cb() {
      stopSaveButtonSpinning();
    }

    function dashboard_save() {
      $('#app-save-spin').css('display', 'inline-block');
      $('#app-save-icon').css('display', 'none');
      msdb.save(current_dbid, dashboard_save_cb, dashboard_error_cb);
    }

    $('#app-save-link')
      .offOn('click.dashboard', dashboard_save);

    function saveShortcut() {
      const e = new MouseEvent('click');

      // See if modal save button is available
      let $saveBtn = $('#save-link');
      if (!$saveBtn.is(':visible')) {

        // Otherwise, pick the dashboard save button
        $saveBtn = $('#app-save-link');
      }

      $saveBtn.each(function(i, tgt) {
        tgt.dispatchEvent(e);
      });
    }

    let saveKey;
    if (arc.misc.isMac()) {
      saveKey = 'Meta+S';
    } else {
      saveKey = 'Ctrl+S';
    }

    arc.shortcut.remove(saveKey);
    arc.shortcut.add(saveKey, saveShortcut, {
      propagate: false,
    });
  }

  displayapp.baseSetup = function() {
    arc.globals_init()
       .is_dashboard(is_dashboard);

    if (arc.frontend.globals.isMobileView) {
      $('#report-menu-list').find('li[arcperm="ds_appedit"]').hide();
    }

    if (!arc.globals.is_edit) {
      setupPopovers();
    }

    // setup Chosen support for selects
    if (arc.globals.is_edit && is_dashboard) {
      $('.chosen-select').chosen({
        search_contains: true,
      });
    }

    setupTooltips();
    emailApp = arc.emailApp();

    return displayapp;
  };

  function solveTunnelingIssues() {
    // needed to solve the dropdown tunneling problem in sidenav
    $('body').on('click', '.selection_element .dropdown-toggle', function() {
      var $target = $(this).closest('.selection_element');
      var $dropdown = $('.selection_element .dropdown-menu');

      var dropDownTop = $target.offset().top + $target.outerHeight();
      var targetWidth = parseInt($target.width());
      var targetLeftOffset = $target.offset().left;
      var dropdownWidth = parseInt($dropdown.css('width').split('px'));
      var dropdownLeft = targetWidth + targetLeftOffset - dropdownWidth;

      $dropdown.css('top', dropDownTop + 'px');
      $dropdown.css('left', dropdownLeft);
      $dropdown.css('float', 'none');
    });

     // needed to solve the dropdown tunneling problem in shelves
    $('body').on('click', '.selected-list > li .dropdown-toggle', function() {
      var $target = $(this).closest('.selected-list > li');

      var $dropdown = $target.find('.dropdown-menu');

      var dropDownTop = $target.offset().top + $target.outerHeight();

      var targetWidth = parseInt($target.width());
      var targetLeftOffset = $target.offset().left;
      var dropdownLeft = targetWidth + targetLeftOffset - $(this).width();

      $dropdown.css('top', dropDownTop - 160 + 'px');
      $dropdown.css('left', dropdownLeft + 20);
      $dropdown.css('float', 'none');
    });

    // needed to solve the dropdown tunneling problem in sidenav
    $('body').on('mouseover', '.selected-list li .dropdown-submenu', function() {
      var $target = $(this).closest('.selected-list > li');

      var $dropdown = $(this).find('.dropdown-menu');

      var dropDownTop = $(this).offset().top + $(this).outerHeight();

      var targetWidth = parseInt($target.width());
      var targetLeftOffset = $target.offset().left;
      var dropdownWidth = parseInt($dropdown.css('width').split('px'));
      var dropdownLeft = targetWidth + targetLeftOffset - dropdownWidth;

      // var dropdownLeft = 40;
      $dropdown.css('top', dropDownTop + 'px');
      $dropdown.css('left', dropdownLeft + 25);
      $dropdown.css('float', 'none');
    });
  }

  // expand/collapse viz settings panels
  $('#expand-visuals-panel').on('click', function() {
    $(this).toggleClass('fa-expand fa-compress');
    $(this).attr('data-original-title', function(i, attr) {
      return attr === 'click to expand' ? 'click to collapse' : 'click to expand';
    });

    if ($('.main-panel').hasClass('reduced-v') &&
        $('.main-panel').hasClass('reduced-s')) {
      $('.main-panel').removeClass('reduced-v');
      $('.visuals-panel').removeClass('expanded');
    } else if ($('.main-panel').hasClass('reduced-s') &&
               !$('.main-panel').hasClass('reduced-v')) {
      $('.main-panel').addClass('reduced-v');
      $('.visuals-panel').addClass('expanded');
    } else {
      $('.main-panel').toggleClass('reduced-v');
      $('.visuals-panel').toggleClass('expanded');
    }
  });

  // expand/collapse viz settings panel
  $('#expand-viz-settings').on('click', function() {
    $(this).toggleClass('fa-expand fa-compress');
    $(this).attr('data-original-title', function(i, attr) {
      return attr === 'click to expand' ? 'click to collapse' : 'click to expand';
    });

    $('.properties-panel').toggleClass('properties-panel-moved');

    if ($('.main-panel').hasClass('reduced-v') &&
        $('.main-panel').hasClass('reduced-s')) {
      $('.main-panel').removeClass('reduced-s');
      $('.visuals-panel').removeClass('moved');
      $('.settings-panel').removeClass('expanded');

    } else if ($('.main-panel').hasClass('reduced-v') &&
               !$('.settings-panel').hasClass('expanded')) {
      $('.main-panel').addClass('reduced-s');
      $('.visuals-panel').addClass('moved');
      $('.settings-panel').addClass('expanded');
    } else {
      $('.main-panel').toggleClass('reduced-s');
      $('.visuals-panel').toggleClass('moved');
      $('.settings-panel').toggleClass('expanded');
    }
  });

  // toggle dashboard/viz settings sections
  $('body').on('click', '.settings-container h5', function() {
    $(this).next('.settings-tab-settings-holder').slideToggle();

      // for all tabs other than General, the default state is closed
      // but no class has been set yet - so need to add the 'open' class
      if (!($(this).parent().hasClass('open') || $(this).parent().hasClass('closed'))) {
        $(this).parent().addClass('open');
    } else {
      $(this).parent().toggleClass('open closed');
      return false;
    }
  });

  // expand/collapse all dashboard and viz modal settings sections
  $('body').on('click', '.sections-expand-all.closed', function () {
    $(this).toggleClass('open closed');
    $(this).closest('.tab-panel-content').find('.settings-tab-holder')
                                         .removeClass('closed')
                                         .addClass('open');
    $(this).closest('.tab-panel-content').find('.settings-tab-settings-holder')
                                         .css('display', 'block');
  });

  $('body').on('click', '.sections-expand-all.open', function () {
    $(this).toggleClass('open closed');
    $(this).closest('.tab-panel-content').find('.settings-tab-holder')
                                         .removeClass('open')
                                         .addClass('closed');
    $(this).closest('.tab-panel-content').find('.settings-tab-settings-holder')
                                         .css('display', 'none');
  });

  function toggleSelectedVizSection() {
    $('.selected-viz-section').on('click', () => {
      $('.viz-icons-section').slideToggle();
      $('.selected-viz-section').toggleClass('is-closed is-open');
    });
  }

  displayapp.initialize = function() {
    if (!is_dashboard || arc.globals.is_edit) {
      arc.report.shelf.editModeInitialize();
    }

    toggleSelectedVizSection();
    solveTunnelingIssues();
    displayapp.baseSetup();
    return displayapp.setup();
  };

  function setupDownloads() {
    // Setup click behavior so that if we are in dashboard edit mode and we
    // change the visibility of the download link by modifying a setting, we
    // do not have to setup this behavior at that point
    $('.subnav-report')
        .find('.download-image-link')
        .off('click')
        .on('click', function() {
          let imageFormat = $(this).attr('data-arc-imageformat');
          snapshotAdmin.downloadImage(imageFormat);
        });

    $('.viz-params-more')
        .find('.download-image-link')
        .off('click')
        .on('click', function() {
          let imageFormat = $(this).attr('data-arc-imageformat');
          snapshotAdmin.downloadImage(imageFormat);
        });

    $('body').off('click.img')
             .on('click.img', '.save-screenshot', function() {
                const $viz = $(this).closest('.gs-w');
                const appId = $viz.attr('data-arc-reportid');
                const title = $viz.find('.arc-viz-title-container').text().trim();
                const imageFormat = $(this).attr('data-arc-imageformat');
                snapshotAdmin.downloadImage(imageFormat,
                  {
                    appId,
                    title,
                    isVisual: true,
                  });
              });

  }

  let firstSheetInitiatedOnce = false;

  function setupMSDashboard(cb) {
    msdb = arc.globals.ms_dashboard =
        arc.ms_dashboard()
           .globalFilters(globalFilters)
           .dashboard_id(app_id)
           .initialSheet(initialSheetId)
           .onSheetActivation('displayappCb', function (newlyActiveSheet) {
             dashboard = newlyActiveSheet;
             if (snapshotAdmin) {
               snapshotAdmin.dashboard(dashboard);
             }

             const { temp_dashboard } = arc.misc.getQueryParameters();
             if (dataset_id && !app_id && !firstSheetInitiatedOnce && !temp_dashboard) {
               firstSheetInitiatedOnce = true;
               newlyActiveSheet.createNewApp(dataset_id);
             }
           });

    if (autoRefreshSeconds) {
      msdb.autoRefreshSeconds(autoRefreshSeconds);
    }

    setupDownloads();

    current_dbid = app_id;

    if (arc.globals.is_edit) {
      setupSave();
    }

    // Track all reports associated with this page
    var loader = arc.loadbar();
    createGlobalReports(loader);

    if (cb) {
      cb();
    }

    msdb.activate();
    setupDashboardDropdown($('.subnav-report'));

    setupResizeDashboard();

    return displayapp;
  }

  function setupDashboardDropdown($navbar) {
    $hideIfNewNavbarItems = $navbar.find('.arc-hide-if-new');
    $buttonWrapper = $('.viz-params-control:not(.inmodal)').find('.viz-params-more');
    if (!current_dbid) {
      $hideIfNewNavbarItems.addClass('arc-invisible');
      $buttonWrapper.addClass('arc-invisible');
      isFirstSave = true;
    }

    arc.report.savetemplate().setup();

    const $cloneModal = $('.clone-move-with-linked-modal');
    const $submitButton = $cloneModal.find('.btn-submit-clone-move');
    const $sharedTable = $cloneModal.find('.linked-visual-table');
    const $actionSpan = $cloneModal.find('.action-type');
    const $cloneModalCheckbox = $cloneModal.find('.clone-only');
    const $moveModalCheckbox = $cloneModal.find('.move-only');
    const $rowTemplate = $($cloneModal.find('#visual-row-modal-template').html());
    const $cloneRowTemplateCheckbox = $rowTemplate.find('.clone-only');
    const $moveRowTemplateCheckbox = $rowTemplate.find('.move-only');
    const $warningText = $cloneModal.find('.cloning-warning');
    function dashboard_clone() {
      const sharedVisuals = arc.globals.reports.getSharedUnsharedVisuals().sharedVisuals;
      if (sharedVisuals.length) {
        if (arc.misc.modalShown($cloneModal)) {
          return;
        }

        $cloneModalCheckbox.removeClass('arc-invisible');
        $moveModalCheckbox.addClass('arc-invisible');
        $cloneRowTemplateCheckbox.removeClass('arc-invisible');
        $moveRowTemplateCheckbox.addClass('arc-invisible');
        $actionSpan.text('Clone');

        // Add shared visuals
        $sharedTable.empty();
        sharedVisuals.forEach(sharedId => {
          const report = arc.globals.reports.getReport(sharedId);
          const sharedVisualData = {
            VISUAL_NAME: report.viz() ? report.viz().title() || 'Untitled' : '',
            VISUAL_ID: report.report_id(),
            SHARED_VISUAL_DASHBOARDS:
              report.shared_visual_dashboards().replace(/\[|\]/g, '') || 'only this dashboard',
          };

          const workspace = report.workspace();
          if (workspace) {
            sharedVisualData.WORKSPACE_NAME = workspace.workspace_name;
            sharedVisualData.WORKSPACE_ID = workspace.workspace_id;
          }

          $sharedTable.append(arc.misc.safe_substitute($rowTemplate[0].outerHTML,
                              sharedVisualData));
        });

        $sharedTable.find('[data-toggle="tooltip"]').tooltip();

        // Add event handler
        $submitButton.offOn('click.submitSave', function cloneDashboard() {
          const $cloneVisuals = $sharedTable.find('.clone-option input:checked');
          const shared_visual_clone = [];
          $cloneVisuals.each(function getSharedVisualToClone(i, el) {
            shared_visual_clone.push(parseInt(el.name));
          });

          msdb.clone(current_dbid, dashboard_clone_cb, shared_visual_clone);
          $cloneModal.modal('hide');
        });

        // Only show warning if user decides to maintain
        $warningText.addClass('arc-invisible');
        $sharedTable.off('change.showWarning')
                    .on('change.showWarning', '.clone-only input', function showAttention() {
          $warningText.removeClass('arc-invisible');
        });

        $cloneModal.modal('show');
      } else {
        msdb.clone(current_dbid, dashboard_clone_cb);
      }
    }

    function dashboard_clone_cb(data) {
      current_dbid = data;
      arc.report.alertMessageSmall('Successfully cloned dashboard', 'alert-success');
      window.open(`${arc.frontend.globals.urls.staticUrl}/../apps/builder/${data}`,
          '_self');
    }

    $navbar.find('.email-visual').attr('data-appid', current_dbid);

    $buttonWrapper.find('.email-visual').attr('data-appid', current_dbid);

    $navbar.find('.clone-link')
           .off('click.dashboard')
           .on('click.dashboard', dashboard_clone);

    $navbar.find('.show-audit-logs')
           .offOn('click.auditlogs', function() {
             arc.perf.PAGEPERF.modal();
           });

    $navbar.find('.accelerate-dashboard')
           .offOn('click.accelerate', function() {
              window.location = `../${arc.frontend.globals.urls.accelerateUrl}?appids=${current_dbid}`;
            });

    $navbar.find('.set-homepage-link')
           .offOn('click.dashboard', async () => {
              try {
                await arc.frontend.submitUserDefinedHomePage(current_dbid);
                arc.report.alertMessageSmall('Homepage has been set', 'alert-success');
              } catch (error) {
                console.error(error);
                arc.report.alertMessageSmall('Failed to set homepage', 'alert-danger');
              }
           });

    $navbar.find('.export-my-viz')
      .off('click')
      .on('click', function (evt) {
        var id_object = {};
        app_id = app_id || current_dbid;
        id_object.dashboard = [app_id];
        id_object.visual = [];
        id_object.apps = [];
        arc.migration.export();
        arc.migration.export.setup();
        arc.migration.export.setup.exportArtifacts(evt, id_object);
      });

    const deleteDashboardModalSelector = '#delete-dashboard-modal';
    function updateModalToReflectDeleteRequestLoadingState(loading) {
      const $loadingIndicator = $(`${deleteDashboardModalSelector} .fa-spinner`);
      const $deleteButton = $(`${deleteDashboardModalSelector} .btn-primary`);
      if (loading) {
        $loadingIndicator.removeClass('arc-invisible');
        $deleteButton.attr('disabled', true);
      } else {
        $loadingIndicator.addClass('arc-invisible');
        $deleteButton.attr('disabled', false);
      }
    }

    function dashboard_delete_success_cb() {
      updateModalToReflectDeleteRequestLoadingState(false);
      arc.report.pagenav.needs_save(false);
      window.open(arc.frontend.globals.urls.appsUrl, '_self');
    }

    function dashboard_delete_failure_cb(req) {
      $(deleteDashboardModalSelector).modal('hide');
      updateModalToReflectDeleteRequestLoadingState(false);
      arc.report.alertMessage('Failed to delete dashboard',
          req.responseText ? req.responseText : req);
    }

    function dashboard_delete() {
      updateModalToReflectDeleteRequestLoadingState(true);
      msdb.delete(current_dbid, dashboard_delete_success_cb,
          dashboard_delete_failure_cb);
    }

    $navbar.find('.delete-db-link')
           .off('click.dashboard')
           .on('click.dashboard', function() {
              const $modal = $('#delete-dashboard-modal');
              if (arc.misc.modalShown($modal)) {
                return;
              }

              $modal.modal('show');
           });

    $('#delete-dashboard-yes').off('click.deleteapp')
                              .on('click.deleteapp', dashboard_delete);

    $navbar.find('.newvisual-link')
           .off('click.dashboard')
           .on('click.dashboard', msdb.createNewVisual);
  }

  function createGlobalReports(loader) {
    // Track all reports associated with this page
    arc.globals.reports =
        arc.reporttracker.reports()
           .build_mode(arc.globals.is_edit ? 'build' : 'display')
           .loader(loader);
  }

  function setupSuperModal() {
    var $superModal = $('.super-modal');
    if (is_dashboard) {
      $superModal.addClass('in-dashboard');
    }

    function closeModal() {
      hideModal(closeModalCb);
      arc.report.superModal.isOpen = false;
      arc.report.superModal.appId = undefined;
    }

    function hideModal(cb) {
      $superModal.fadeOut('fast', function() {
        $('.navbar').css('z-index', '');
        $('.subnav-report').css('z-index', '99');
        if (cb) {
          cb();
        }
      });

      arc.report.activateMainAlertContainer();
      $(window).off('resize');
      if (dashboard) {
        setupResizeDashboard();
      }
    }

    function closeModalCb() {
      var report = arc.report.getPrimaryReport();

      if (report) {
        report.viz().cleanup();
        if (is_dashboard) {
          report.unmodal();
          dashboard.activateReport();
          report.refresh(true);
          dashboard.configureMapHandlers(report);
          const styleHolder =
            d3.select('#dashboard-css-settings-sidebar .settings-tab-settings-holder');
          dashboard.user_settings().find_by_id('inline_css').paintControl(styleHolder, true);
        } else {
          // If in single_visual mode, then we need to resetup the dropdown on
          // modal close in case some of the data has changed. If in dashboard, we
          // may need add/remove Map Handlers to the report's chart_div.
          report.discardChanges();
          report.refresh(true);
          arc.globals.singleVisualBuilder.setupVisualDropdownButtons($('#subnav-report'));
        }
      }
    }

    $superModal.find('.view-btn')
               .off('click.superModalView')
               .on('click.superModalView', closeModal);

    $superModal.find('.modal-close-btn')
               .off('click.superModalClose')
               .on('click.superModalClose', function closeSuperModal() {
                 arc.globals.is_edit = arc.globals.is_dashboard && arc.globals.is_edit;
                 closeModal();
               });
  }

  function onetimeSetupVisual() {
    var loader = arc.loadbar()
                    .dom_id('.super-modal .viz-refresh-indicator')
                    .refresh_icon(true);

    // Setup shelf handling
    arc.report.shelf.setup();
    setupUserSettingsTabs();

    // Use a builder to handle interactions with reports
    // Not sure if we need visual_type here, maybe if we want to create
    // default new visuals besides table
    arc.globals.singleVisualBuilder =
        arc.reportbuilder()
           .loader(loader)
           .displayapp(displayapp)
           .visual_type(visual_type);
  }

  function setupUserSettingsTabs() {
    const $sidebarTabs = $('li.user-control');
    const $shelves = $('.shelves-wrapper');
    $sidebarTabs.on('click', _sidebarClickHandler);

    function _sidebarClickHandler(event) {
      $sidebarTabs.each(function () {
        if ($(this).hasClass('active')) {
          $(this).removeClass('active');
          const id = $(this).children().first().attr('href');
          $(id).removeClass('in').removeClass('active');
        }
      });

      const $li = $(event.currentTarget);

      // Determine whether to show alert, keeping in mind is_edit might be undefined.
      const showOverrideAlert =  !!arc.globals.is_edit && $li.attr('id') === 'style' &&
        arc.misc.dashboardOverride();
      $('#override-warning').toggle(showOverrideAlert);
      $li.addClass('active');
      const id = $li.children().first().attr('href');
      $(id).addClass('active').addClass('in');

      $shelves.toggleClass('active', $li.attr('id') === 'data');
    }
  }

  displayapp.enterVisualEditMode = function() {
    arc.globals.is_edit = true;
    arc.globals.singleVisualBuilder.setupVisualInModal({ app_id, dataset_id });
  };

  displayapp.setup = function(cb) {
    // Turn off any earlier resize handling
    $(window).off('resize');

    setupSuperModal();
    onetimeSetupVisual();

    //Load datasets only when not in app group and in edit dashboard mode or single visual mode
    if (!arc.globals.appgroup && ((arc.globals.is_edit && arc.globals.is_dashboard) ||
      !arc.globals.is_dashboard)) {
        // Fetch datasets to populate the 'switch dataset' icon
        arc.globals.singleVisualBuilder.setupSwitchDataset();
        if (arc.globals.is_edit) {
          $('a.editapp-link').trigger('click');
        }
    }

    snapshotAdmin = arc.snapshot()
                       .is_dashboard(is_dashboard);
    emailApp.snapshotInstance(snapshotAdmin);
    emailApp.initialize();
    if (is_dashboard) {
      setupMSDashboard(cb);
    } else {
      setupVisual();
    }

    return displayapp;
  };

  function setupVisual() {
    const $emailBtn = $('.main-app-view .viz-params-more .email-visual');
    $emailBtn.attr('data-appid', app_id);
    $emailBtn.attr('data-datasetid', dataset_id);

    var loader = arc.loadbar()
      .dom_id('.main-app-view .viz-refresh-indicator')
      .refresh_icon(true);
    createGlobalReports(loader);

    var builder = arc.globals.singleVisualBuilder;
    var $chartContainer = $('#view-chart-container');
    var unique_chartdiv_id = builder.setupChartContainer($chartContainer);

    builder.firstVisual(app_id, dataset_id, unique_chartdiv_id)
      .dataset_id(dataset_id)
      .setupAndDrawReport();

    $('a.editapp-link').on('click.edit', displayapp.enterVisualEditMode);
    displayapp.setupResizeVisual();
    setupDownloads();
  }

  return displayapp;
};

}());
