window.arc = window.arc || {};
arc.screen = arc.screen || {};

arc.screen.displayappgroup = function() {
  const ag = function() { return 'arc.screen.displayappgroup'; };

  var appgroups,
      appgroup_id,
      appgroup_membership,
      apps_in_appgroups,
      apptitles_by_appid,
      apptypes_by_appid,
      apptitles,
      displayapp,
      visualSkeleton,
      appSkeleton,
      firstAppId,
      mainApp,
      sidenav,
      appgroupSettings,
      globalFilters,
      initialapp,
      initialSheetId,
      navigateToAppsPage,
      appMenuBarBgColor,
      appMenuBarBottomBarColor,
      appMenuBarBarColorSelected,
      appMenuTextFont,
      appMenuTextFontSize,
      appMenuTextFontWeight,
      appMenuTextColor,
      appMenuTextColorSelected,
      appMenuTextUppercase,
      appLogoImg,
      appLogoWidth,
      appLogoHeight,
      appLogoMarginTop,
      appLogoMarginLeft,
      appBgBgColor,
      appHideDropShadow,
      appShowTitle,

      $appModeHeader = $('.app-mode'),
      $visualModeHeader = $('.visual-mode'),
      $paramControls = $('.viz-params-control');

  ag.appgroup_id = function(_) {
    if (!arguments.length) return appgroup_id;
    appgroup_id = _;
    return ag;
  };

  ag.appgroup_name = function() {
    return mainApp?.name;
  };

  ag.globalFilters = function(_) {
    if (!arguments.length) return globalFilters;
    globalFilters = _;
    return ag;
  };

  ag.initialapp = function(_) {
    if (!arguments.length) return initialapp;

    if (_ && !isNaN(_)) {
      initialapp = Number(_);
    }

    return ag;
  };

  ag.initialSheet = function (_) {
    if (!arguments.length) return initialSheetId;
    initialSheetId = ((!_) ? undefined : Number(_));

    return ag;
  };

  ag.sidenav = function(_) {
    if (!arguments.length) return sidenav;
    if (_ === 'true') {
      sidenav = true;
    } else {
      sidenav = false;
    }
    return ag;
  };

  ag.appgroups = function(_) {
    if (!arguments.length) return appgroups;
    appgroups = _;

    appgroups.sort(function(a, b) {
      if (a.subgroup_order === b.subgroup_order) {
        return 0;
      }
      if (a.subgroup_order === 0) {
        return -1;
      }
      if (b.subgroup_order === 0) {
        return 1;
      }
      if (a.subgroup_order < b.subgroup_order) {
        return -1;
      }
      if (a.subgroup_order > b.subgroup_order) {
        return 1;
      }
      return 0;
    });

    // Store away the main app
    mainApp = undefined;
    appgroupSettings = {};
    appgroups.some(function(a) {
      if (a.subgroup_order === 0) {
        mainApp = a;
        if (mainApp.options) {
          var o = JSON.parse(mainApp.options);
          if (o.user_settings) {
            appgroupSettings = o.user_settings;
          }
        }
        return true;
      }
      return false;
    });

    // We've already added the appgroup inline css, so remove it from here to
    // prevent the code from appending it twice. Don't do this for the custom styles
    // because we still need to append the custom js.
    appgroupSettings.inline_css = undefined;

    var user_settings = arc.visual.appgroupSettings();
    user_settings.loadFromValues(appgroupSettings);
    arc.css.CUSTOMCSS_MANAGER.appgroupSettings(user_settings);
    ag.sidenav(user_settings.find_by_id('menu_on_left').value());
    navigateToAppsPage = user_settings.find_by_id('navigate_to_apps_page').isChecked;
    appMenuBarBgColor = user_settings.find_by_id('app_menubar_bg_color');
    appMenuBarBottomBarColor =
                    user_settings.find_by_id('app_menubar_bottombar_color');
    appMenuBarBarColorSelected =
                    user_settings.find_by_id('app_menubar_barcolor_selected');
    appMenuTextFont = user_settings.find_by_id('app_menutext_font');
    appMenuTextFontSize = user_settings.find_by_id('app_menutext_font_size');
    appMenuTextFontWeight = user_settings.find_by_id('app_menutext_font_weight');
    appMenuTextColor = user_settings.find_by_id('app_menutext_color');
    appMenuTextColorSelected =
                    user_settings.find_by_id('app_menutext_color_selected');
    appMenuTextUppercase = user_settings.find_by_id('app_menutext_uppercase');
    appLogoImg = user_settings.find_by_id('app_logo_image');
    appLogoWidth = user_settings.find_by_id('app_logo_width');
    appLogoHeight = user_settings.find_by_id('app_logo_height');
    appLogoMarginTop = user_settings.find_by_id('app_logo_margintop');
    appLogoMarginLeft = user_settings.find_by_id('app_logo_marginleft');
    appBgBgColor = user_settings.find_by_id('app_bg_bg_color');
    appHideDropShadow = user_settings.find_by_id('app_hide_dropshadow');
    appShowTitle = user_settings.find_by_id('app_show_title');

    return ag;
  };

  ag.getCss = function(_) {
    var cssl = [];

    let appMenuBarBg = appMenuBarBgColor.value().trim();
    if (appMenuBarBg !== '') {
      let cssStr = `
        .arcapp .navbar,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link:hover,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link:focus,
        .arcapp .navbar-default .navbar-nav >.open > .nav-link,
        .arcapp .navbar-default .navbar-nav >.open > .nav-link:hover,
        .arcapp.sidemenu-wrapper .app-side-nav,
        .arcapp.sidemenu-wrapper .logo-container,
        .arcapp.sidemenu-wrapper .collapse-sidemenu {
          background-color: %BGCOLOR%;
        }`;
      cssl.push(cssStr.replace('%BGCOLOR%', appMenuBarBg));
    }

    let appMenuBarBottomBar = appMenuBarBottomBarColor.value().trim();
    if (appMenuBarBottomBar !== '') {
      let cssStr = `
        .arcapp .navbar,
        .arcapp .navbar-default .navbar-nav > li > .nav-link {
          border-color: %BORDER_COLOR%;
        }`;
      cssl.push(cssStr.replace('%BORDER_COLOR%', appMenuBarBottomBar));
    }

    let appMenuBarBarSelected =
                              appMenuBarBarColorSelected.value().trim();
    if (appMenuBarBarSelected !== '') {
      let cssStr = `
        .arcapp .navbar-default .navbar-nav > .active > .nav-link,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link:hover,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link:focus,
        .arcapp .navbar-default .navbar-nav > li > .nav-link:hover,
        .arcapp .navbar-default .navbar-nav > li > .nav-link:focus,
        .arcapp.sidemenu-wrapper .app-menu > li.active,
        .arcapp.sidemenu-wrapper .app-menu > li > ul li.active {
           border-color: %BORDER_COLOR%;
        }`;
      cssl.push(cssStr.replace('%BORDER_COLOR%', appMenuBarBarSelected));
    }

    let appMenuFont = appMenuTextFont.value().trim();
    if (appMenuFont !== '') {
      arc.misc.setFont($('.arcapp .navbar'), 'data-fontclass', appMenuFont);
      arc.misc.setFont($('.arcapp.sidemenu-wrapper .app-menu'),
                       'data-fontclass', appMenuFont);
    }

    let appMenuFontSize = appMenuTextFontSize.value().trim();
    if (appMenuFontSize !== '') {
      let cssStr = `
      .arcapp .navbar,
      .arcapp .navbar .dropdown-menu,
      .arcapp.sidemenu-wrapper .app-menu > li > div,
      .arcapp.sidemenu-wrapper .app-menu > li > ul li {
        font-size: %FONT_SIZE%px;
      }`;
      cssl.push(cssStr.replace('%FONT_SIZE%', appMenuFontSize));
    }

    let appMenuFontWeight = appMenuTextFontWeight.value().trim();
    if (appMenuFontWeight !== '') {
      let cssStr = `
        .arcapp .navbar,
        .arcapp.sidemenu-wrapper .app-menu > li > div {
          font-weight: %FONT_WEIGHT%;
        }`;
      cssl.push(cssStr.replace('%FONT_WEIGHT%', appMenuFontWeight));
    }

    let appMenuText = appMenuTextColor.value().trim();
    if (appMenuText !== '') {
      let cssStr = `
        .arcapp .navbar-default .navbar-nav > li > .nav-link,
        .arcapp.sidemenu-wrapper .app-menu li .nav-link {
          color: %COLOR%;
        }`;
      cssl.push(cssStr.replace('%COLOR%', appMenuText));
    }

    let appMenuTextSelected = appMenuTextColorSelected.value().trim();
    if (appMenuTextSelected !== '') {
      let cssStr = `
        .arcapp .navbar-default .navbar-nav > .active > .nav-link,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link:hover,
        .arcapp .navbar-default .navbar-nav > .active > .nav-link:focus,
        .arcapp .navbar-default .navbar-nav > li > .nav-link:hover,
        .arcapp .navbar-default .navbar-nav > li > .nav-link:focus,
        .arcapp.sidemenu-wrapper .app-menu li .nav-link:hover,
        .arcapp.sidemenu-wrapper .app-menu li .nav-link:focus,
        .arcapp.sidemenu-wrapper .app-menu li.active .nav-link {
          color: %COLOR%;
        }`;
      cssl.push(cssStr.replace('%COLOR%', appMenuTextSelected));
    }

    let appMenuTextUpper = appMenuTextUppercase.isChecked;
    if (appMenuTextUpper !== true) {
      let cssStr = `
        .arcapp .navbar,
        .arcapp.sidemenu-wrapper .group-title {
          text-transform: none;
        }`;
      cssl.push(cssStr);
    }

    let logoImg = appLogoImg.value().trim();
    let logoWidth = appLogoWidth.value();
    let logoHeight = appLogoHeight.value();
    let logoMarginTop = appLogoMarginTop.value();
    let logoMarginLeft = appLogoMarginLeft.value();
    if (logoImg !== '') {
      cssl.push('.arcapp .arc-appgroup-logo { width: ' + logoWidth + 'px;}');
      cssl.push('.arcapp .arc-appgroup-logo { height: ' + logoHeight + 'px;}');
      cssl.push('.arcapp .arc-appgroup-logo { margin-top: ' + logoMarginTop + 'px;}');
      cssl.push('.arcapp .arc-appgroup-logo { margin-left: ' + logoMarginLeft + 'px;}');
      cssl.push('.arcapp .arc-appgroup-logo { background-image: url("' +
                logoImg + '");}');
    }

    let appBgBg = appBgBgColor.value().trim();
    if (appBgBg !== '') {
      cssl.push('.arcapp .builder-container-app { background-color: ' + appBgBg + ';}');
    }

    let appHideDropShadowSetting = appHideDropShadow.isChecked;
    if (appHideDropShadowSetting === true) {
        cssl.push('.arcapp .arc-app-container, .main-app-modal { box-shadow: none;}');
    }

    return cssl.join('\n');
  };

  ag.appgroup_membership = function(_) {
    if (!arguments.length) return appgroup_membership;
    appgroup_membership = _;
    appgroup_membership.sort(function(a, b) {
      if (a[2] < b[2]) {
        return -1;
      }
      if (a[2] > b[2]) {
        return 1;
      }
      return 0;
    });
    apps_in_appgroups = {};
    appgroup_membership.forEach(function(m) {
      if (m[1] in apps_in_appgroups) {
        apps_in_appgroups[m[1]].push(m[0]);
      } else {
        apps_in_appgroups[m[1]] = [m[0]];
      }
    });
    return ag;
  };

  ag.apptitles = function(_) {
    if (!arguments.length) return apptitles;
    apptitles = _;
    apptitles_by_appid = {};
    apptypes_by_appid = {};
    apptitles.forEach(function(a) {
      apptitles_by_appid[a[0]] = a[1];
      apptypes_by_appid[a[0]] = a[2];
    });
    return ag;
  };

  // params are passed when navigated within same app group or through custom JS
  function activateApp(target, params) {
    // Re-draw the visual and dashboard containers
    $visualModeHeader.html(visualSkeleton);
    $appModeHeader.html(appSkeleton);
    const { appId, sheetId } = target;

    const apptype = apptypes_by_appid[appId];

    // Setting arc.globals.is_dashboard just in case since we use that frequently in code
    const is_dashboard = apptype === 'app';
    arc.globals.is_dashboard = is_dashboard;
    if (is_dashboard) {
      $visualModeHeader.addClass('arc-invisible');
      $paramControls.removeClass('arc-invisible');
      $appModeHeader.removeClass('arc-invisible');
    } else {
      $appModeHeader.addClass('arc-invisible');
      $paramControls.addClass('arc-invisible');
      $visualModeHeader.removeClass('arc-invisible');
      arc.globals.dashboard = null;
    }

    let currentParams = arc.globals.ms_dashboard ?
      arc.globals.ms_dashboard.params_history().current_params()[0] :
      [];

    // TODO: In future we may also want to check if param.scope.appId === current app's id
    const paramsToPass = currentParams.filter(param => param.scope && param.scope.appId);

    // Passed params should override dashboard's params
    if (params) {
      params.forEach(p => paramsToPass.push(p));
    }

    displayapp.app_id(appId)
              .initialSheet(sheetId)
              .is_dashboard(is_dashboard)
              .setup(function() {
                if (paramsToPass) {
                  if (is_dashboard) {
                    const msdb = arc.globals.ms_dashboard;
                    msdb.onSheetActivation('setParamsOnDrillThrough', function () {
                      msdb.params_history().add(paramsToPass, null, false);
                      msdb.onSheetActivation('setParamsOnDrillThrough', null);
                    });
                  } else {
                    const report = arc.globals.reports.getPrimaryReport();
                    report.visual_params_history().add(paramsToPass, report.viz(), false);
                  }
                }
              });
  }

  function setupOneTopMenu(appgroup) {
    const $nav = $('.navbar-nav');
    const noSubmenuTemplate = $('.menu-noitems-template ul').html();
    const templateIn = $('.menu-template ul').html();
    const templateSubMenu = $('.submenu-template ul').html();

    var apps = apps_in_appgroups[appgroup.group_id];
    if (!apps) {
      return;
    }
    var menu_name;
    if (appgroup.subgroup_order === 0 && apps.length === 1) {
      menu_name = apptitles_by_appid[apps[0]];
    } else {
      menu_name = appgroup.name;
    }

    if (!appShowTitle.isChecked && apps.length === 1) {
      setFirstApp(apps[0]);
      $nav.append(arc.misc.safe_substitute(
          noSubmenuTemplate,
          {
            'APPID': apps[0],
            'MENUNAME': menu_name,
          }));
      return;
    }

    $nav.append(arc.misc.safe_substitute(
        templateIn,
        {
          'APPGROUP_ID': appgroup.group_id,
          'MENUNAME': menu_name,
        }));

    const $ul = $nav.find('ul').last();
    apps.forEach(function(app) {
      setFirstApp(app);
      $ul.append(arc.misc.safe_substitute(
          templateSubMenu,
          {
            'APPID': app,
            'LABEL': apptitles_by_appid[app] || 'Untitled',
          }));
    });
  }

  function setupOneSideMenu(appgroup) {
    const $nav = $('.sidemenu-holder .app-menu');
    const noSubMenuTemplate = $('.sidemenu-without-items-template ul').html();
    const templateIn = $('.sidemenu-with-items-template ul').html();
    const templateSubMenu = $('.sidemenu-item-template ul').html();

    const apps = apps_in_appgroups[appgroup.group_id];
    if (!apps) {
      return;
    }

    const isMenuTitleHidden = !appShowTitle.isChecked && apps.length === 1;
    var menu_name;
    if (appgroup.subgroup_order === 0 && isMenuTitleHidden) {
      menu_name = apptitles_by_appid[apps[0]];
    } else {
      menu_name = appgroup.name;
    }

    if (isMenuTitleHidden) {
      $nav.append(arc.misc.safe_substitute(
          noSubMenuTemplate,
          {
            'APPID': apps[0],
            'MENUNAME': menu_name,
          }));

      setFirstApp(apps[0]);
      return;
    }

    $nav.append(arc.misc.safe_substitute(
        templateIn,
        {
          'APPGROUP_ID': appgroup.group_id,
          'MENUNAME': menu_name,
        }));

    const $ul = $nav.find('.submenu-items').last();
    apps.forEach(function(app) {
      setFirstApp(app);

      $ul.append(arc.misc.safe_substitute(
          templateSubMenu,
          {
            'APPID': app,
            'LABEL': apptitles_by_appid[app] || 'Untitled',
          }));
    });
  }

  function setFirstApp(appid) {
    if (!firstAppId) {
      firstAppId = appid;
    }

    if (appid === initialapp) {
      firstAppId = initialapp;
    }
  }

  ag.navigateToApp = function(appId, el, params) {
    let $link;
    if (el) {
      $link = $(el);
    } else {
      $link = $('.arc-link[data-arc-app="' + appId + '"]').first();
    }
    if ($link.length === 0) {
      return;
    }

    // hide previous alerts
    arc.report.alertMessageAllHide();

    activateApp({
      appId,
      sheetId: initialSheetId
    }, params);

    var $nav;

    if (sidenav) {
      $nav = $('.app-menu');
      $nav.find('li').removeClass('active');
      $link.closest('li').addClass('active');
    } else {
      $nav = $('.navbar-nav');
      $nav.find('li.dropdown').removeClass('active');
      $link.closest('li.dropdown').addClass('active');
    }

    return ag;
  };

  function setupMenuActions() {
    var $nav;

    if (sidenav) {
      $nav = $('.app-menu');
    } else {
      $nav = $('.navbar-nav');
    }
    if (sidenav) {
      $nav.find('li:first').addClass('active');
    } else {
      $nav.find('li.dropdown:first').addClass('active');
    }

    $('.arc-link')
        .off('click')
        .on('click', function() {
          destroyApp();
          const appId = $(this).attr('data-arc-app');
          ag.navigateToApp(Number(appId), this);
        });

    // Clicking on the main logo brings us to the first page
    $('.navbar-brand')
        .off('click')
        .on('click', () => {
          if (navigateToAppsPage) {
            window.open(`${arc.frontend.globals.urls.staticUrl}/../apps/apps`, '_self');
          } else {
            destroyApp();
            ag.navigateToApp(firstAppId);
          }
        });

    function destroyApp() {
      // First pass at cleanup: ARC-3366
      // Remove extra context menus, we need to figure out still how to do this
      // properly
      // $('[id^="contextmenu-"]').remove();

      // Remove leftover dataTables
      $('.dataTables_scrollBody > table').each(function () {
        const dt = $(this).data('data-table');
        if (dt) {
          dt.destroy(true);
        }
      });

      // Leftover td elements from the dataTables footer
      $('html > td').remove();

      // Remove all old shelves
      $('[id^="app-shelves-"]').remove();

      // Remove elements generated by gridster
      $('[generated-from="gridster"]').remove();

      if (displayapp.is_dashboard()) {
        let dashboard = arc.globals.dashboard;

        if (dashboard) {
          dashboard.destroy();
        }

        arc.globals.reports = undefined;
        dashboard = undefined;
      }

      arc.globals.click_behavior_cb = undefined;
      arc.globals.parameters = undefined;

      // Empty the containers of previous html
      $visualModeHeader.empty();
      $appModeHeader.empty();
    }

    // Expand/collapse side menu groups
    $('.group-title').offOn('click.expandGroupsMenu', function () {
      $(this).find('i').toggleClass('fa-angle-down fa-angle-right');
      $(this).parent().next().slideToggle();
    });
  }

  function setupTopMenus() {
    const $nav = $('.navbar-nav');
    $nav.empty();
    firstAppId = undefined;

    appgroups.forEach((a) => {
      setupOneTopMenu(a);
    });

    setupMenuActions();
  }

  function setupSideMenus() {
    const $sidemenuHolder = $('.sidemenu-holder');
    const $appMenu = $sidemenuHolder.find('.app-menu');
    $appMenu.empty();

    firstAppId = undefined;

    appgroups.forEach((a) => {
      setupOneSideMenu(a);
    });

    setupMenuActions();
  }

  function setupMenus() {
    if (sidenav) {
      setupSideMenus();
    } else {
      setupTopMenus();
    }
  }

  function initializeSidenav() {
    $('.main-app-view').css('top', '0px');
    $('.navbar-fixed-top').addClass('arc-invisible');
    $('body').addClass('sidemenu-wrapper');
    $('.sidemenu-holder').append($('.sidemenu-template').html());

    $('.collapse-sidemenu').offOn('click.collapseSidemenu', () => {
      $('.app-side-nav').hide();
      $('.sidemenu-collapsed').show();
      $('.arc-appgroup-container').addClass('main-app-full');
    });
    $('.expand-sidemenu').offOn('click.expandSidemenu', () => {
      $('.app-side-nav').show();
      $('.sidemenu-collapsed').hide();
      $('.arc-appgroup-container').removeClass('main-app-full');
    });
  }

  ag.initialize = function() {
    if (mainApp && mainApp.name) {
      document.title = mainApp.name;
    }

    if (sidenav) {
      initializeSidenav();
    } else {
      $('.main-app-view').css('top', '50px');
    }

    visualSkeleton = $visualModeHeader.html();
    appSkeleton = $appModeHeader.html();

    displayapp = arc.screen.displayapp()
                    .globalFilters(globalFilters)
                    .initialSheet(initialSheetId);
    displayapp.baseSetup();

    setupMenus();

    if (firstAppId) {
      ag.navigateToApp(firstAppId);
    } else if (arc.css.CUSTOMCSS_MANAGER) {
      // This is only if there's no dashboard within the app
      // Use setTimeout so this runs after arc.globals.appgroup is set
      setTimeout(arc.css.CUSTOMCSS_MANAGER.applyCustomCss, 0);
    }

    return ag;
  };

  return ag;
};
