/*
 This file is js code for appgroup_edit.html which aims to create and modify
 an app (formerly appgroup)

initialize_app()
  setupPage()
  fetchAndPaint()

setupPage()
  setupAppOrg()
  setupAppSidebar()

fetchAndPaint()
  fetches the data for an appgroup
  paintAppGroupAndSubgroups()
    loadPageFromParent()

*/

window.arc = window.arc || {};
arc.screen = arc.screen || {};

arc.screen.appgroup = function() {
  var appgroup = function() { return 'arc.screen.appgroup'; };

  var appgroup_id,
      parentGroup,
      subGroups,
      deletedGroups,
      canEdit,
      updatedAppsInSubgroups,
      data_connections = [],
      data_connections_hash = {},
      dataset_hash = {},
      recentlyViewedDashboards = [],
      userSettings,
      sidenavMenuOnLeft,
      appMenuBarBgColor,
      appMenuBarBottomBarColor,
      appMenuBarBarColorSelected,
      appMenuTextFont,
      appMenuTextFontSize,
      appMenuTextFontWeight,
      appMenuTextColor,
      appMenuTextColorSelected,
      appMenuTextUppercase,
      appLogoImg,
      appLogoWidth,
      appLogoHeight,
      appLogoMarginTop,
      appLogoMarginLeft,
      appBgBgColor,
      appHideDropShadow,
      appShowTitle,
      workspaces = arc.workspace.workspaces(),
      isTabChanged = false;

  // Cached DOM elements
  const $subnavBar = $('#subnav-report'),
        $saveBtn = $subnavBar.find('.save-btn'),
        $undoBtn = $subnavBar.find('.undo-btn'),
        $cloneBtn = $subnavBar.find('.clone-appgroup'),
        $appNameInput = $('.app-name-input'),
        $appMenuSectionParent = $('.app-menu-section-parent'),
        appMenuSectionTemplate = $('#app-menu-section-template').html(),
        dashSidebarTemplate = $('#dashboard-item-template').html();

  appgroup.appgroup_id = function(_) {
    if (!arguments.length) return appgroup_id;
    appgroup_id = _;
    workspaces.appgroup_id(_);
    return appgroup;
  };

  // For modifying any style, 2 steps are needed:
  // 1. Styling has to be modified below for the preview-style option in
  // app builder
  // 2. Styling has to be modified in arc_displayappgroup.js for the actual
  // App loading
  appgroup.getCss = function() {
    var cssl = [];

    let sidenav = sidenavMenuOnLeft.isChecked;
    if (sidenav) {
      $('#app-top-menu-preview').addClass('arc-invisible');
      $('#app-side-menu-preview').removeClass('arc-invisible');
    } else {
      $('#app-top-menu-preview').removeClass('arc-invisible');
      $('#app-side-menu-preview').addClass('arc-invisible');
    }

    let appMenuBarBg = appMenuBarBgColor.value().trim();
    if (appMenuBarBg !== '') {
      let cssStr = `
        .app-preview .navbar,
        .app-preview .navbar-default .navbar-nav > .active > .nav-link,
        .app-preview .navbar-default .navbar-nav > .active > .nav-link:hover,
        .app-preview .navbar-default .navbar-nav > .active > .nav-link:focus,
        .app-preview .navbar-default .navbar-nav >.open > .nav-link,
        .app-preview .navbar-default .navbar-nav >.open > .nav-link:hover,
        .app-preview.sidemenu-wrapper .app-side-nav,
        .app-preview.sidemenu-wrapper .logo-container,
        .app-preview.sidemenu-wrapper .collapse-sidemenu {
          background-color: %BGCOLOR%;
        }`;
      cssl.push(cssStr.replace('%BGCOLOR%', appMenuBarBg));
    }

    let appMenuBarBottomBar = appMenuBarBottomBarColor.value().trim();
    if (appMenuBarBottomBar !== '') {
     let cssStr = `
        .arcapp .navbar,
        .arcapp .navbar-default .navbar-nav > li > .nav-link {
          border-color: %BORDER_COLOR%;
        }`;
      cssl.push(cssStr.replace('%BORDER_COLOR%', appMenuBarBottomBar));
    }

    let appMenuBarBottomBarSelected =
                              appMenuBarBarColorSelected.value().trim();
    if (appMenuBarBottomBarSelected !== '') {
      let cssStr = `
        .app-preview .navbar-default .navbar-nav > .active > .nav-link,
        .app-preview .navbar-default .navbar-nav > .active > .nav-link:hover,
        .app-preview .navbar-default .navbar-nav > .active > .nav-link:focus,
        .app-preview .navbar-default .navbar-nav > li > .nav-link:hover,
        .app-preview .navbar-default .navbar-nav > li > .nav-link:focus,
        .app-preview.sidemenu-wrapper .app-menu > li.active,
        .app-preview.sidemenu-wrapper .app-menu > li > ul li.active {
           border-color: %BORDER_COLOR%;
        }`;
      cssl.push(cssStr.replace('%BORDER_COLOR%', appMenuBarBottomBarSelected));
    }

    let appMenuFont = appMenuTextFont.value().trim();
    if (appMenuFont !== '') {
      arc.misc.setFont($('.app-preview .navbar'), 'data-fontclass', appMenuFont);
      arc.misc.setFont($('.app-preview.sidemenu-wrapper .app-menu'), 'data-fontclass',
                       appMenuFont);
    }

    let appMenuFontSize = appMenuTextFontSize.value().trim();
    if (appMenuFontSize !== '') {
      let cssStr = `
      .app-preview .navbar,
      .app-preview .navbar .dropdown-menu,
      .app-preview.sidemenu-wrapper .app-menu > li > div,
      .app-preview.sidemenu-wrapper .app-menu > li > ul li {
        font-size: %FONT_SIZE%px;
      }`;
      cssl.push(cssStr.replace('%FONT_SIZE%', appMenuFontSize));
    }

    let appMenuFontWeight = appMenuTextFontWeight.value().trim();
    if (appMenuFontWeight !== '') {
      let cssStr = `
        .app-preview .navbar,
        .app-preview.sidemenu-wrapper .app-menu > li > div {
          font-weight: %FONT_WEIGHT%;
        }`;
      cssl.push(cssStr.replace('%FONT_WEIGHT%', appMenuFontWeight));
    }

    let appMenuText = appMenuTextColor.value().trim();
    if (appMenuText !== '') {
      let cssStr = `
        .app-preview .navbar-default .navbar-nav > li > .nav-link,
        .app-preview.sidemenu-wrapper .app-menu li .nav-link {
          color: %COLOR%;
        }`;
      cssl.push(cssStr.replace('%COLOR%', appMenuText));
    }

    let appMenuTextSelected = appMenuTextColorSelected.value().trim();
    if (appMenuTextSelected !== '') {
      let cssStr = `
        .app-preview .navbar-default .navbar-nav > .active > .nav-link,
        .app-preview .navbar-default .navbar-nav > .active > .nav-link:hover,
        .app-preview .navbar-default .navbar-nav > li > .nav-link:hover,
        .app-preview .navbar-default .navbar-nav > li > .nav-link:focus,
        .app-preview.sidemenu-wrapper .app-menu li.active .nav-link {
          color: %COLOR%;
        }`;
      cssl.push(cssStr.replace('%COLOR%', appMenuTextSelected));
    }

    let appMenuTextUpper = appMenuTextUppercase.isChecked;
    if (appMenuTextUpper !== true) {
      let cssStr = `
        .app-preview .navbar,
        .app-preview.sidemenu-wrapper .group-title {
          text-transform: none;
        }`;
      cssl.push(cssStr);
    }

    let logoImg = appLogoImg.value().trim();
    let logoWidth = appLogoWidth.value();
    let logoHeight = appLogoHeight.value();
    let logoMarginTop = appLogoMarginTop.value();
    let logoMarginLeft = appLogoMarginLeft.value();
    if (logoImg !== '') {
      cssl.push('.app-preview .arc-appgroup-logo { width: ' + logoWidth + 'px;}');
      cssl.push('.app-preview .arc-appgroup-logo { height: ' + logoHeight + 'px;}');
      cssl.push('.app-preview .arc-appgroup-logo { margin-top: ' + logoMarginTop + 'px;}');
      cssl.push('.app-preview .arc-appgroup-logo { margin-left: ' + logoMarginLeft + 'px;}');
      cssl.push('.app-preview .arc-appgroup-logo { background-image: url("' +
                logoImg + '");}');
    }

    let appBgBg = appBgBgColor.value().trim();
    if (appBgBg !== '') {
      cssl.push('.app-preview .app-preview-body { background-color: ' + appBgBg + ';}');
      cssl.push('.app-preview .app-sidemenu-main { background-color: ' + appBgBg + ';}');
    }

    let appHideDropShadowSetting = appHideDropShadow.isChecked;
    if (appHideDropShadowSetting === true) {
        cssl.push('.app-preview .arc-app-container, .main-app-modal { box-shadow: none;}');
    }

    return cssl.join('\n');
  };

  function needsSave() {
    if (canEdit) {
      $saveBtn.addClass('app-btn-highlight');
    }

    $undoBtn.addClass('app-btn-highlight');
    arc.report.pagenav.needs_save(true);
  }

  function removeNeedsSave() {
    if (canEdit) {
      $saveBtn.removeClass('app-btn-highlight');
    }

    $undoBtn.removeClass('app-btn-highlight');
    arc.report.pagenav.needs_save(false);
  }

  function validateAppSaveInputs() {
    if ($appNameInput.val().trim().length < 1) {
      let message = 'Empty app name. An app name is required.';
      arc.report.alertMessageSmall(message, 'alert-danger', true);
      return false;
    }

    var hasError = false;
    $('.app-menu-section').each(function(idx) {
      let groupname;
      let groupid = Number($(this).attr('data-groupid'));
      groupname = $(this).find('.app-menu-title').text().trim();
      if (groupname.length < 1) {
        arc.report.alertMessageSmall('Empty menu tab title. Tab titles are required',
                                     'alert-danger', null, true);
        hasError = true;
        return false;
      }
    });

    if (hasError) {
      return false;
    }

    return true;
  }

  function saveAndRedraw(cloneApp) {
    $saveBtn.removeClass('app-btn-highlight');
    $undoBtn.removeClass('app-btn-highlight');
    if (!parentGroup) {
      parentGroup = arc.model.AppGroup()
                             .group_options({});
    }

    if (cloneApp) {
      deletedGroups = [];
      updatedAppsInSubgroups.forEach(updated => updated.group_id = -1);
      appgroup_id = undefined;
      parentGroup.group_id(-1);
      $appNameInput.val(`Clone of ${parentGroup.group_name()}`);
      workspaces.currentWorkspace(undefined);
    }

    loadParentFromPage(cloneApp);

    var ags = arc.model.AppGroupSaver()
                       .workspaces(workspaces);

    ags.deletedGroups(deletedGroups);

    ags.updatedAppsInSubgroups(updatedAppsInSubgroups);
    ags.addAppGroup(parentGroup);
    ags.saveAppgroups((returned_data) => {
      arc.misc.notify("App '" + parentGroup.group_name() + "' saved");

      // Ideally, here I wanted to not re-fetch and draw so that saving is
      // smooth and does not have a flash. However, it is too hard since saving
      // different subgroups in the backend gives them an ID and that ID is
      // painted on the page and then further used to figure out which subgroups
      // are new vs old to re-save. It was too hard to match the returning IDs
      // with the subgroups already painted, so I just re-fetchAndPaint. I do not
      // however, refetch the list of connections and dashboards etc, so that's
      // some saving.
      if (!appgroup_id || cloneApp) {
        appgroup.appgroup_id(returned_data.saved_app_id);

        parentGroup.group_id(appgroup_id);
        updatedAppsInSubgroups = [];
        deletedGroups = [];
        const url = `${arc.frontend.globals.urls.editAppUrl}/${appgroup_id}`;

        // Upon saving, replace the url to also show the newly generated appid
        arc.misc.replaceState(url);
      }

      fetchAndPaint();
    });
  }

  function undoChanges() {
    removeNeedsSave();
    fetchAndPaint();
    paintUserSettings();
  }

  function drawAppPreview() {
    const $singleMenuTemplate = $('#app-menu-single-preview-template').html();
    const $multipleMenuTemplate = $('#app-menu-multiple-preview-template').html();
    const $subMenuTemplate = $('#app-submenu-preview-template').html();

    const $singleMenuSideTemplate = $('#app-menu-single-preview-side-template').html();
    const $multipleMenuSideTemplate =
                                $('#app-menu-multiple-preview-side-template').html();
    const $subMenuSideTemplate = $('#app-submenu-preview-side-template').html();

    // If only an empty tab (as is done when new app is created), then do not
    // update styling, use default hard coded menu
    const $menuTabs = $('.app-menu-section');
    if ($menuTabs.length <= 1 &&
        $menuTabs.find('.app-menu-title').text() === '') {
        return;
    }

    // 1. Clear out existing preview menu bar
    $('.arc-navbar-menu').empty();

    // 2. Read from dom the existing sub-menus and containing dashboards
    // and plot using template based on number of items in each menu tab
    let $menuTemplate, $menuSideTemplate;
    let firstMenuDbs;
    $('.app-menu-section').each(function() {
      let $eachMenu = $(this);
      let menuName = $eachMenu.find('.app-menu-title').text();
      let $dashboardsInMenu = $($eachMenu).find('.menu-viz-list li');
      if (!firstMenuDbs) {
        firstMenuDbs = $dashboardsInMenu.length;
      }

      if (!appShowTitle.isChecked && $dashboardsInMenu.length < 2) {
        $menuTemplate = $singleMenuTemplate;
        $menuSideTemplate = $singleMenuSideTemplate;
      } else {
        $menuTemplate = $multipleMenuTemplate;
        $menuSideTemplate = $multipleMenuSideTemplate;
      }

      let $menu = $(arc.misc.safe_substitute(
                        $menuTemplate,
                        {
                          NAME: menuName,
                        }
                        ));
      let $menuSide = $(arc.misc.safe_substitute(
                        $menuSideTemplate,
                        {
                          NAME: menuName,
                        }
                        ));

      $('#app-top-menu-preview ul.arc-navbar-menu').append($menu);
      $('#app-side-menu-preview ul.arc-navbar-menu').append($menuSide);

      $dashboardsInMenu.each(function() {
        let $eachDashboard = $(this);
        let subMenuName = $eachDashboard.find('.app-submenu-item-name').text();
        let $subMenu = $(arc.misc.safe_substitute(
                        $subMenuTemplate,
                        {
                          NAME: subMenuName || 'Untitled',
                        }
                        ));
        let $subMenuSide = $(arc.misc.safe_substitute(
                        $subMenuSideTemplate,
                        {
                          NAME: subMenuName || 'Untitled',
                        }
                        ));
        $menu.find('ul').append($subMenu);
        $menuSide.find('ul').append($subMenuSide);
      });
    });

    // Set up active menu items on both side and top preview navbar
    $('.app-preview .navbar-nav > li:first-of-type').addClass('active');

    if (firstMenuDbs < 2) {
      $('.app-preview .app-menu > li:first-of-type').addClass('active');
    } else {
      $('.app-preview .app-menu > li:first-of-type li:first-of-type').addClass('active');
    }
  }

  function setupStylingPreview() {
    $('#app-tab-style').on('shown.bs.tab', drawAppPreview);
  }

  function setupInfoPopover() {
    $('.app-builder-info').popover({
      animation: false,
      html: true,
      trigger: 'hover',
      placement: 'right',
      title: 'How to create a new App',
      content: $('#app-builder-info-popover').html(),
      container: 'body',
      sanitize: false,
    });
  }

  function setupSettings() {
    userSettings = arc.visual.appgroupSettings();
    setupIndividualUserSettings();
    userSettings.addChangeCb('triggerNeedsSave', needsSave);
    paintUserSettings();
  }

  function setupNewApp() {
    canEdit = true;
    setupSettings();

    workspaces.appgroup_id(-1);
    workspaces.setupWorkspaceButton($('#subnav-report'));

    $appMenuSectionParent.empty();

    $appNameInput.val('');
    $appNameInput.focus();
    $saveBtn.removeClass('save-disabled');

    // Initialize appgroup variables here
    deletedGroups = [];
    updatedAppsInSubgroups = [];

    // For a new app, put a new subgroup/menu placeholder
    addNewSubGroupToAppGroup();
    removeNeedsSave();
  }

  function setupPageOnceOnly() {

    setupAppOrg();

    // Fetch Data Connections, Datasets, Related Dashboards, Recent Dashboards
    setupAppSidebar();

    setupStylingPreview();

    setupInfoPopover();

    arc.globals.screen_appgroup = true;

    // Display either the org or style depending on which side tab is clicked
    $('#style-side-tab, #custom-side-tab, #settings-side-tab').off('click')
      .on('click', function() {
        $('#app-tab-style').tab('show');
    });

    $('#dashboards-side-tab').off('click')
                             .on('click', function() {
      $('#app-tab-org').tab('show');
    });

    $('#app-tab-style').off('click.follow')
                       .on('click.follow', function() {
      $('#style-side-tab').tab('show');
    });

    $('#app-tab-org').off('click.follow')
                       .on('click.follow', function() {
      $('#dashboards-side-tab').tab('show');
    });

    // The following is code that also exists for dashboards and is the same,
    // so we should consider moving it to a common place

    // Set up chosen support
    $('.chosen-select').chosen({
      search_contains: true,
    });

    // expand/collapse settings panel
    $('#expand-dash-settings').offOn('click.expandDash', function() {
      $('.dash-main-panel').toggleClass('reduced');
      $('.dash-settings-panel').toggleClass('expanded');
      $(this).toggleClass('fa-expand fa-compress');
    });

    // close settings panel
    $('#close-dash-settings').offOn('click.closeDash', () => {
    //  $('.dash-settings-panel').toggle('slide', {direction: 'right'}, 500);
      $('.dash-settings-panel').fadeOut();
      $('.side-tabs li a').removeClass('active');
    });

    // reopen settings panel
    $('.side-tabs li a').offOn('click.reOpenSetting', () => {
      $('.dash-settings-panel').fadeIn();
    });

    // More code that is common in dashboards
    $('body').on('click', '.settings-container h5', function() {
      $(this).next('.settings-tab-settings-holder').slideToggle();

        // for all tabs other than General, the default state is closed
        // but no class has been set yet - so need to add the 'open' class
        if (!($(this).parent().hasClass('open') ||
              $(this).parent().hasClass('closed'))) {
          $(this).parent().addClass('open');
      } else {
        $(this).parent().toggleClass('open closed');
        return false;
      }
    });

    // expand/collapse all dashboard and viz modal settings sections
    $('body').on('click', '.sections-expand-all.closed', function () {
      $(this).toggleClass('open closed');
      $(this).closest('.tab-panel-content')
             .find('.settings-tab-holder')
             .removeClass('closed')
             .addClass('open');
      $(this).closest('.tab-panel-content')
             .find('.settings-tab-settings-holder')
             .css('display', 'block');
    });

    $('body').on('click', '.sections-expand-all.open', function () {
      $(this).toggleClass('open closed');
      $(this).closest('.tab-panel-content')
             .find('.settings-tab-holder')
             .removeClass('open')
             .addClass('closed');
      $(this).closest('.tab-panel-content')
             .find('.settings-tab-settings-holder')
             .css('display', 'none');
    });

    // add hash and selected tab id to the url so that when the page is
    // refreshed or user uses the back button, the selected tab is maintained
    if (location.hash !== '') {
      $('.save-tabs > li > a[href="' + location.hash.replace('tab-', '') + '"]').tab('show');
    } else {
      $('.save-tabs a:first').tab('show');
    }

    $('.save-tabs a[data-toggle="tab"]').on('shown.bs.tab', function(e) {
      isTabChanged = true;
      window.location.hash = 'tab-' + e.target.hash.substr(1);
      return false;
    });

    if (appgroup_id) {
      setupSettings();
    }

    // Allow for sorting of subgroups within the appgroup
    $appMenuSectionParent.sortable({
      placeholder: 'app-menu-section-placeholder',
      'ui-floating': 'auto',
      connectToSortable: '.app-menu-section-parent',
      tolerance: 'pointer',
      connectWith: '.app-menu-section-parent',
      change: function() {
        needsSave();
      },
    });

    $saveBtn.off('click')
            .on('click', function() {
              if (validateAppSaveInputs()) {
                saveAndRedraw();
              }
            });

    $undoBtn.off('click')
            .on('click', function() {
              undoChanges();
            });

    $cloneBtn.offOn('click.cloneApp', function cloneAppFunction() {
              if (validateAppSaveInputs()) {
                saveAndRedraw(true);
              }
            });

    // Launch app
    $('#view-appgroup-btn')
        .off('click')
        .on('click', function() {
          arc.misc.openInNewWindow(`${arc.frontend.globals.urls.viewAppUrl}/${appgroup_id}`);
        });

    $('#delete-appgroup')
        .off('click')
        .on('click', function() {
          if (!parentGroup) {
            return;
          }

          var groupid = parentGroup.group_id();
          var groupname = parentGroup.group_name();
          var c = arc.misc.deleteConfirmation();
          c.objectType('App')
           .objectName(groupname)
           .askUser(function() {
             arc.model.AppGroup()
                .group_id(groupid)
                .deleteAppgroup(function() {
                  c.hideModal();
                  // eslint-disable-next-line no-restricted-globals
                  location.href = arc.frontend.globals.urls.appsUrl;
                });
           });
        });

    $appNameInput
        .off('keyup')
        .on('keyup', function() {
          needsSave();
        });
  }

  // Gets called if an existing appgroup is being loaded and also gets
  // called for every save
  function fetchAndPaint() {
    if (!appgroup_id) {
      setupNewApp();
    } else {
      d4.queue()
        .defer(d3.json, `${arc.frontend.globals.urls.appgroupUrl}/${appgroup_id}`)
        .await(function(error, appgroups) {
          // Handle error scenarios
          if (error) {
            const errorMessage = error.responseText || error.response || error;
            arc.report.alertMessage('Failed to fetch app data: ', errorMessage);
            return;
          }

          if (!appgroups.length) {
            const errorMessage = 'You do not have permission to view this app';
            arc.report.alertMessage(errorMessage);
            return;
          }

          // If no error, render workspace button and app builder
          if (!workspaces.currentWorkspace()) {
            workspaces.currentWorkspace({
              workspace_name: appgroups[0].workspace_name,
              workspace_id: appgroups[0].workspace_id,
              workspace_perm: appgroups[0].workspace_perm,
            });

            workspaces.setupWorkspaceButton($subnavBar);
          }

          // Note: .../<appgroupid> returns a list of appgroup as well as
          // all subgroups that belong to that appgroup
          let raw_appgroups = appgroups;
          raw_appgroups.forEach(function(d) {
            if (d.group_options) {
              d.group_options = JSON.parse(d.group_options);
            }
          });

          var al = raw_appgroups.filter(function(appgroup) {
            return (appgroup.group_type !== 'starred');
          });

          paintAppGroupAndSubgroups(al);
        });
    }
  }

  function setupAppOrg() {

    // Clicking on any subgroup should make that one active
    // (so that new dashboards get added to that subgroup)
    $('.app-menu-section').off('click.aps')
                          .on('click.aps', function() {
      $('.app-menu-section').removeClass('active');
      $(this).addClass('active');
    });

    // Set up the 'Add Menu Tab' button to add a subgroup
    $('#ab-subgroup-add').off('click.absd')
                         .on('click.absd', function() {
      addNewSubGroupToAppGroup();
      $('.app-menu-section .app-menu-title')
                            .prop('contenteditable', true).focus();
    });
  }

  function setupAppSidebar() {
    const dataconnections_asset = {
      asset: arc.frontend.globals.urls.dataconnectionUrl,
      postprocess: d3.json,
    };
    const datasets_asset = {
      asset: arc.frontend.globals.urls.datasetUrl,
      postprocess: d3.json,
    };

    const recently_viewed_dashboards =
    arc.frontend.vizCookie.getLatestVisuals('dashboard')
      .filter((id) => id.length)
      .reverse();

    const recently_viewed_dash_asset = {
      asset: `${arc.frontend.globals.urls.reportUrl}?report_ids=${JSON.stringify(recently_viewed_dashboards)}`,
      postprocess: d3.json,
    };

    const sortByUpdate = function(a, b) {
      if (a.updated > b.updated) {
        return -1;
      }

      if (a.updated < b.updated) {
        return 1;
      }

      return 0;
    };

    var filteredDataConnections = [];

    arc.staticfetcher.fetch(
      [dataconnections_asset, datasets_asset, recently_viewed_dash_asset],
      (error, dataconnections_data, datasets_data, recently_viewed_dash_data) => {
        if (error) {
          arc.report.alertMessage('Could not retrieve data connections ' +
            'and datasets', error.responseText ? error.responseText : error);
          return;
        }

        data_connections = [];
        data_connections_hash = {};
        dataset_hash = {};

        // Set up hashes for quicker indexing
        dataconnections_data.forEach(function(dc) {
          var dataconn = arc.dataconnection.DataConnection()
                            .load(dc);
          data_connections_hash[dc.id] = dataconn;
          data_connections.push(dataconn);
        });

        datasets_data.forEach(function(in_ds) {
          var ds = arc.datasets.Dataset()
                      .init_from_fetch(in_ds);
          dataset_hash[in_ds.id] = ds;
          if (in_ds.dc_id in data_connections_hash) {
            var dc = data_connections_hash[in_ds.dc_id];
            dc.add_dataset(ds);
          }
        });

        // Remove all dataconnections that don't have datasets
        data_connections.forEach(function (dc) {
          if (!dc.datasets().length) {
            delete data_connections_hash[dc.dataconnection_id()];
          } else {
            filteredDataConnections.push(dc);
          }
        });

        data_connections = filteredDataConnections;
        recentlyViewedDashboards = recently_viewed_dash_data
                                    .filter((dashboard) => arc.misc.hasPerm('ds_appview', dashboard.perm))
                                    .sort(sortByUpdate);
        paintAppSidebar();
      }
    );
  }

  function paintRecentlyViewedDashboards() {
    if (recentlyViewedDashboards.length > 0) {
      const ral = d3.select('.recent-app-list');

      $('#recent-app-list').removeClass('arc-invisible');
      $('.recent-app-list').empty();

      ral.selectAll('nosel')
         .data(recentlyViewedDashboards)
         .enter()
           .append('li')
           .classed('app_selection_element', true)
           .html(function(d) {
             return arc.misc.safe_substitute(dashSidebarTemplate,
                      arc.report.mapDashboardOrVisualInfo(d));
           });

      ral.on('click.ral', () => {
        const event = d4.event || d3.event;
        const dashboard = d3.select($(event.target).closest('li')[0]).datum();
        addDashboardToActiveSubgroup(dashboard);
      });
    }
  }

  function paintAppSidebar() {
    paintRecentlyViewedDashboards();

    // Paint Connections and Datasets drop downs..
    const $dataConnections = $('.appsidebar-dataconnections-sel');

    data_connections.sort(function(a, b) {
      if (a.dataconnection_name().toLowerCase() < b.dataconnection_name().toLowerCase()) {
        return -1;
      }

      if (a.dataconnection_name().toLowerCase() > b.dataconnection_name().toLowerCase()) {
        return 1;
      }

      return 0;
    });

    d3.select('.appsidebar-dataconnections-sel')
      .selectAll('nosel')
      .data(data_connections)
      .enter()
        .append('option')
        .attr('value', function(d) {
            return d.dataconnection_id();
          })
        .text(function(d) {
            return d.dataconnection_name();
         });

    $dataConnections.trigger('chosen:updated');

    $dataConnections.off('change.das');
    $dataConnections.on('change.das', function() {
      paintAppSidebarDatasetsOfConn($(this).val());
    });

    // If there's a cookie from last used appbuilder, use that to choose
    // data connection and dataset
    const cookieData = arc.frontend.vizCookie.getAppBuilderConnectionDataset();
    if (cookieData && cookieData.appbuilder_dcid) {
      $dataConnections.val(cookieData.appbuilder_dcid);
      $dataConnections.trigger('chosen:updated');
      paintAppSidebarDatasetsOfConn(cookieData.appbuilder_dcid, true);
    } else {
      // This is to paint the datasets the first time a new app is loaded,
      // and there are no cookies
      paintAppSidebarDatasetsOfConn(
                    $('.appsidebar-dataconnections-sel option:selected').val());
    }

    // Add popovers
    const popoverText = $('#chart-content-popover').html();
    $('#dash-viz-tab').popover({
      selector: '.app_selection_element',
      container: 'body', // makes popover be on top
      html: true,
      trigger: 'hover',
      placement: 'left',
      delay: { show: '500' },
      title: '',
      sanitize: false,
      content: function showDashPopover() {
        const data = d3.select(this).datum();
        const dataset = dataset_hash[data.dataset_id];
        const tags = dataset ? dataset.tags() : [];
        data.HTML_DATASET_TAGS = tags.join('');
        if (tags.length === 0) {
          data.TAGS_VISIBILITY = 'arc-invisible';
        }

        return arc.misc.safe_substitute(popoverText, arc.report.mapDashboardOrVisualInfo(data));
      },
      boundary: 'window',
    });

    $('.app-menu-section-parent').popover({
      selector: '.chart-small-label',
      container: 'body', // makes popover be on top
      html: true,
      sanitize: false,
      trigger: 'hover',
      placement: 'top',
      delay: { show: '500' },
      title: '',
      content: function showDashPopover() {
        const data = $(this).closest('li').data('dashboard');
        if (data) {
          const dataset = dataset_hash[data.dataset_id];
          const tags = dataset ? dataset.tags() : [];
          data.HTML_DATASET_TAGS = tags.join('');
          if (tags.length === 0) {
            data.TAGS_VISIBILITY = 'arc-invisible';
          }
        }

        const defaultData = {
          ID: id,
          HIDE_INFO: 'arc-invisible',
          TAGS_VISIBILITY: 'arc-invisible',
        };
        const dashboardData = data ? arc.report.mapDashboardOrVisualInfo(data) : defaultData;

        return arc.misc.safe_substitute(popoverText, dashboardData);
      },
    });
  }

  function paintAppSidebarDatasetsOfConn(dataconnection_id, useCookieData) {
    var dataconn = data_connections_hash[dataconnection_id];

    // Dashboards can contain reports from multiple datasets and it's not
    // clear that we should require 'ds_appview' on the datasets presented
    // here.
    var datasetsofconn = dataconn.datasets().filter(function (ds) {
      return arc.misc.hasPerm('ds_appview', ds.dataset_perms());
    });

    arc.dataconnection.sortDataSets(datasetsofconn);

    d3.selectAll('.appsidebar-datasets-sel option')
      .remove();

    let $datasetsSelect = $('.appsidebar-datasets-sel');

    d3.select('.appsidebar-datasets-sel')
      .selectAll('nosel')
      .data(datasetsofconn)
      .enter()
        .append('option')
        .attr('value', function(d) {
            return d.dataset_id();
          })
        .text(function(d) {
            return d.dataset_name();
         });

    $datasetsSelect.trigger('chosen:updated');

    $datasetsSelect.off('change.dasd');
    $datasetsSelect.on('change.dasd', function() {
      const changedDataset = $(this).val();
      arc.frontend.vizCookie.setAppBuilderConnectionDataset('appbuilder_dcid', dataconnection_id);
      arc.frontend.vizCookie.setAppBuilderConnectionDataset('appbuilder_dsid', changedDataset);
      paintAppSidebarDashboardsOfDataset(changedDataset);
    });

    // If cookies exist for previously chosen dataset, and if the chosen
    // connection is also set to the cookie data (as indicated by
    // useCookieData), then select that dataset
    const cookieData = arc.frontend.vizCookie.getAppBuilderConnectionDataset();
    if (useCookieData && cookieData && cookieData.appbuilder_dsid) {
      $datasetsSelect.val(cookieData.appbuilder_dsid);
      $datasetsSelect.trigger('chosen:updated');
    }

    // Also paint visuals of the dataset selected (by default)
    const selectedDataset = $('.appsidebar-datasets-sel option:selected').val();
    arc.frontend.vizCookie.setAppBuilderConnectionDataset('appbuilder_dcid', dataconnection_id);
    arc.frontend.vizCookie.setAppBuilderConnectionDataset('appbuilder_dsid', selectedDataset);
    paintAppSidebarDashboardsOfDataset(selectedDataset);
  }

  function appMenuHideInstructions($inputSubgroup) {
    $inputSubgroup.parent().find('.app-section-instruction').hide();
  }

  function addDashboardToActiveSubgroup(dashboard) {
    let $activemenusection = $('.app-menu-section.active');
    if (d3.event.target.className.includes('dashboard-link-btn')) {
      return;
    }

    if (!$activemenusection.length) {
      let message = 'No Menu Tabs exist yet. Please add a Menu Tab before adding ' +
                    'dashboards to your app.';
      arc.report.alertMessageSmall(message, 'alert-danger', true);
      return;
    }

    addDashboardToSubgroup(dashboard.id,
                           dashboard.name,
                           dashboard.img,
                           -1, /* indicate newly added dashboard */
                           $activemenusection.attr('data-groupid'),
                           $activemenusection.find('ul'),
                           true /*needsSave*/);
  }

  // This is not called when dashboards are re-arranged in an app, this is
  // only when a dashboard is added to a subgroup by clicking on the list
  // of dashboards on the right side of the page
  const $appSubmenuItemTemplate = $('#app-submenu-item-template');
  function addDashboardToSubgroup(dashboardid, dashboardname, dashboardimg,
                                  order, groupid,
                                  $inputSubgroup, saveNeeded) {

    let $existingDashboards = $inputSubgroup.find('li');

    // Do not allow same dashboard to be added to same subgroup multiple times
    // The backend cannot handle 2 dashboards with same ID so preventing it here
    let noDuplicate = true;
    $existingDashboards.each(function() {
      if (Number($(this).attr('data-dashboard')) === dashboardid) {
        arc.report.alertMessageSmall(
          `Dashbord #${dashboardid} is already in the active menu tab`,
          'alert-danger');
        noDuplicate = false;
        return false;
      }
    });

    if (!noDuplicate) {
      return;
    }

    appMenuHideInstructions($inputSubgroup);

    // For a dashboard being newly added to app (not retrieved from backend),
    // calculate order by simply adding 1 to the previous app's order in that
    // subgroup or start from 1 if no previous dashboards
    if (order === -1) {
      if ($existingDashboards.length === 0) {
        order = 1;
      } else {
        order = Math.ceil(Number($existingDashboards.last().attr('data-order')) + 1);
      }
    }

    const dashboardData = {
      DASHBOARD_ID: dashboardid,
      ORDER: order,
      GROUPID: groupid,
    };

    if (dashboardimg) {
      // Only show image and name if user has permission to see that dashboard
      dashboardData.DASHBOARD_NAME = dashboardname;
      dashboardData.DASHBOARD_IMG = dashboardimg;
      dashboardData.HAS_SCREENSHOT = arc.misc.hasScreenshot(dashboardimg);
      dashboardData.NO_PERMISSION_DISPLAY = 'arc-invisible';
      dashboardData.NO_PERMISSION_CLASS = '';
    } else {
      // Otherwise, only show ID and indicate user doesn't have correct permissions
      dashboardData.HAS_SCREENSHOT = 'no-permission';
      dashboardData.DASHBOARD_NAME = `(ID: ${dashboardid})`;
      dashboardData.DASHBOARD_IMG = '';
      dashboardData.NO_PERMISSION_DISPLAY = '';
      dashboardData.NO_PERMISSION_CLASS = 'no-permissions';
    }

    let litxt = arc.misc.safe_substitute($appSubmenuItemTemplate.html(), dashboardData);
    let $li = $(litxt);

    $inputSubgroup.append($li);

    // Update title if needed
    const $titleInput = $inputSubgroup.siblings('.app-menu-title');
    if (!$titleInput.text() && dashboardname) {
      $titleInput.text(dashboardname);
    }

    $li.on('click.dashboard', () => {
      const url = `${arc.frontend.globals.urls.staticUrl}/../apps/app/${dashboardid}`;
      arc.misc.openInNewWindow(url);
    });

    $li.find('.app-section-item-delete').off('click.d')
                                        .on('click.d', function(event) {
      // Stop propogation so that it does not try to open the dashboard too
      event.stopPropagation();

      $li.remove();

      updatedAppsInSubgroups.push({
        group_id: groupid,
        app_id: dashboardid,
        app_present: false,
        order: 0,
      });
      needsSave();
    });

    if (saveNeeded) {
      scrollToBottom();
      updatedAppsInSubgroups.push({
         group_id: groupid,
         app_id: dashboardid,
         app_present: true,
         order: order,
        });
      needsSave();
    }
  }

  function paintAppSidebarDashboardsOfDataset(dataset_id) {
    dataset_id = parseInt(dataset_id, 10);

    // Only allow adding dashboards from datasets with R access
    if (!isNaN(dataset_id) &&
        !arc.misc.hasPerm('ds_appview',
                           dataset_hash[dataset_id].dataset_perms())) {
        return false;
    }

    const appDashboardListSelector = '.app-dashboard-list';
    const $loadingSpinner = $(appDashboardListSelector).find('.arc-spinner');

    $loadingSpinner.show();
    const dashboard_asset = [
      {
      asset: `${arc.frontend.globals.urls.reportUrl}?report_type=dashboard&dataset_id=${dataset_id}`,
      postprocess: d3.json,
      }
    ];

    arc.staticfetcher.fetch(dashboard_asset, (error, dashboardsRelatedToDataset) => {
      $loadingSpinner.hide();

      if (error) {
        arc.report.alertMessage('Could not retrieve data connections ' +
          'and datasets', error.responseText ? error.responseText : error);
        return;
      }

      d3.selectAll(`${appDashboardListSelector} li`)
      .remove();

      d3.select(appDashboardListSelector)
        .selectAll('nosel')
        .data(dashboardsRelatedToDataset)
        .enter()
          .append('li')
          .classed('app_selection_element', true);

      d3.selectAll(`${appDashboardListSelector} li`)
        .on('click', (d) => {
          addDashboardToActiveSubgroup(d);
        })
        .html((d) => arc.misc.safe_substitute(
          dashSidebarTemplate,
          arc.report.mapDashboardOrVisualInfo(d)
        ));
    });
  }

  function scrollToBottom() {
    var $main = $('#main'),
        $bottomAnchor = $('#bottom-anchor');

    // scroll to bottom of the main div
    $main.scrollTo($bottomAnchor, 1000);
  }

  function setupIndividualUserSettings() {

    sidenavMenuOnLeft = userSettings.find_by_id('menu_on_left');
    appMenuBarBgColor = userSettings.find_by_id('app_menubar_bg_color');
    appMenuBarBottomBarColor =
                    userSettings.find_by_id('app_menubar_bottombar_color');
    appMenuBarBarColorSelected =
                    userSettings.find_by_id('app_menubar_barcolor_selected');

    appMenuTextFont = userSettings.find_by_id('app_menutext_font');
    appMenuTextFontSize = userSettings.find_by_id('app_menutext_font_size');
    appMenuTextFontWeight = userSettings.find_by_id('app_menutext_font_weight');

    appMenuTextColor = userSettings.find_by_id('app_menutext_color');
    appMenuTextColorSelected =
                    userSettings.find_by_id('app_menutext_color_selected');
    appMenuTextUppercase = userSettings.find_by_id('app_menutext_uppercase');

    appLogoImg = userSettings.find_by_id('app_logo_image');
    appLogoWidth = userSettings.find_by_id('app_logo_width');
    appLogoHeight = userSettings.find_by_id('app_logo_height');
    appLogoMarginTop = userSettings.find_by_id('app_logo_margintop');
    appLogoMarginLeft = userSettings.find_by_id('app_logo_marginleft');

    appBgBgColor = userSettings.find_by_id('app_bg_bg_color');
    appHideDropShadow = userSettings.find_by_id('app_hide_dropshadow');
    appShowTitle = userSettings.find_by_id('app_show_title');

    appShowTitle.addChangeCb('click.app_menu_change', drawAppPreview);
  }

  function paintUserSettings() {

    var $defaultSettings = $('#app-settings-sidebar');
    $defaultSettings.empty();

    var $stylingSettings = $('#app-styling-settings-sidebar');
    $stylingSettings.empty();

    var $cssSettings = $('#app-css-settings-sidebar');
    $cssSettings.empty();

    if (!userSettings) {
      return;
    }

    // Disable highlighting of the Save button on modifications
    // as we bulk paint all the existing settings
    userSettings.settings().forEach(function(s) {
      s.changeCb('appgroup', undefined);
    });

    userSettings.paint({
      default: $defaultSettings,
      Style: $stylingSettings,
      'Custom Style': $cssSettings,
    });

   // Enable highlighting of the Save button
   userSettings.settings().forEach(function(s) {
      s.changeCb('appgroup', paintInlineCss);
    });

    paintInlineCss(true);
  }

  function paintInlineCss(saveNotNeeded) {
    arc.css.CUSTOMCSS_MANAGER.applyCustomCss();
    if (!saveNotNeeded) {
      needsSave();
    }
  }

  /*
   * Read in the new data on the app (formerly appgroup) builder page to
   * create a new AppGroup object, to then later save
   * This is called in the 'New App' flow, or when an existing app is modified.
   * The opposite function is loadPageFromParent() -- the one is called when
   * data for an appgroup is read from the backend and the page is rendered
   * based on that data.
   */
  function loadParentFromPage(cloneApp) {
    parentGroup.group_name($appNameInput.val());

    // For now, if new app, put it in the side menu and make it public by default
    // (TODO: This will be enhanced when apps are made part of RBAC)
    if (!appgroup_id) {
      parentGroup.in_userlist('true');
    }

    var group_options = parentGroup.group_options();
    group_options.user_settings = group_options.user_settings || {};
    if (userSettings) {
      userSettings.fillValues(group_options.user_settings);
    }

    if (cloneApp && arc.globals.arcviz_settings.ENABLE_URL_NAMES) {
      group_options.user_settings.app_url_name = '';
    }

    // Re-set group options so the json gets calculated for saving
    parentGroup.group_options(group_options);

    parentGroup.subgroups([]);
    $('.app-menu-section-parent .app-menu-section').each(function(idx) {
      const groupid = Number($(this).attr('data-groupid'));
      const groupname = $(this).find('.app-menu-title').text().trim();
      var g = arc.model.AppGroup()
                 .group_subgroup_order(idx + 1)
                 .group_options({})
                 .group_name(groupname);
      if (!cloneApp && groupid > 0) {
        g.group_id(groupid);
      } else {
        if (groupname === '') {

          // Don't permit empty subgroup names
          return;
        }

        // For new subgroups, also add included apps information
        // For existing subgroups, this data is not filled in, and instead
        // the updatedAppsInSubgroup field is used
        let apps_in_subgroup = [];
        $(this).find('li').each(function(idx, lielm) {
          apps_in_subgroup.push({
            group_id: cloneApp ? -1 : groupid,
            app_id: $(lielm).attr('data-dashboard'),
            app_present: true,
            order: $(lielm).attr('data-order'),
          });
        });

        g.apps_in_subgroup(apps_in_subgroup);
      }

      parentGroup.addSubGroup(g);
    });

  }

  function setupEditableTitle($editableTitle) {
    $editableTitle.off('click.et')
                  .on('click.et', function() {
                        $(this).prop('contenteditable', true);
                        $(this).focus();
                        $appMenuSectionParent.sortable('disable');
                   })
                   .off('blur.et')
                   .on('blur.et', function() {
                        $(this).prop('contenteditable', false);
                        needsSave();
                        $appMenuSectionParent.sortable('enable');
                   })
                  .off('keypress.et')
                  .on('keypress.et', function(event) {
                    if (event.which === arc.misc.ENTER_KEY) {
                      $editableTitle.prop('contenteditable', false);
                    }

                    return event.which !== 13;
                  });
  }

  function saveDashboardOrderChanges($sel) {
    $sel.off('sortupdate')
        .on('sortupdate', function(event, ui) {
      var $curr = $(ui.item);
      var prev_groupid = Number($curr.attr('data-groupid'));
      let $curr_group = $curr.parents('.app-menu-section');

      $('.app-menu-section').removeClass('active');
      $curr_group.addClass('active');
      appMenuHideInstructions($curr_group);

      let curr_groupid = Number($curr_group.attr('data-groupid'));
      $curr.attr('data-groupid', curr_groupid);
      var $prev = $curr.prev();
      var $next = $curr.next();
      var order = 1;
      if (($next.length === 0) && ($prev.length === 0)) {
        order = 1;
      } else if ($next.length === 0) {
        order = Math.ceil(Number($prev.attr('data-order')) + 1);
      } else if ($prev.length === 0) {
        order = Number($next.attr('data-order')) / 2;
        if (order > 1) {
          order = Math.floor(order);
        }
      } else {
        order = (Number($prev.attr('data-order')) + (Number($next.attr('data-order')))) / 2;
      }

      $curr.attr('data-order', order);

      // If added to new subgroup, or moved from a new subgroup then
      // because newly added Subgroups and dashboards are handled
      // elsewhere (in addDashboardToSubgroup), no need to do individual app
      // membership so return here
      if (prev_groupid < 0 && curr_groupid < 0) {
        return;
      }

      // Target
      if (curr_groupid > 0) {
        updatedAppsInSubgroups.push({
          group_id: curr_groupid,
          app_id: $curr.attr('data-dashboard'),
          app_present: true,
          order: order,
        });
      }

      // Source
      if (prev_groupid > 0 && prev_groupid !== curr_groupid) {
        updatedAppsInSubgroups.push({
          group_id: prev_groupid,
          app_id: $curr.attr('data-dashboard'),
          app_present: false,
          order: 0,
        });
      }

    });
  }

  function addNewSubGroupToAppGroup() {
      addSubGroupToAppGroup(-1,
                            '',
                            appMenuSectionTemplate,
                            true, /* saveNeeded */
                            []); /* no appsInSubgroup */

      $appMenuSectionParent.scrollTo('.right-anchor:last-of-type', 1000);
  }

  function addSubGroupToAppGroup(groupid, groupname, templateIn,
                                 saveNeeded,
                                 appsInSubgroup) {

    let $new_subgroup = $(arc.misc.safe_substitute(
                              templateIn,
                              {
                              GROUPID: groupid,
                              NAME: groupname,
                              }));
    $appMenuSectionParent.append($new_subgroup);

    $('.app-menu-section').removeClass('active');
    $new_subgroup.addClass('active');

    $new_subgroup.off('click')
                 .on('click', function() {
      $('.app-menu-section').removeClass('active');
      $new_subgroup.addClass('active');
    });

    $new_subgroup.find('.app-section-delete').off('click.asd')
                                             .on('click.asd', function() {
      $new_subgroup.remove();

      // If this is an existing subgroup, only then it needs to be removed
      // from backend. If this was added in this edit iteration, it does not
      // need to be removed from backend.
      if (groupid > 0) {
        deletedGroups.push(groupid);
        needsSave();
      }
    });

    $new_subgroup.find('.menu-viz-list').sortable({
      placeholder: 'app-menu-dashboard-placeholder',
      change: function() {
        needsSave();
      },

      connectWith: '.menu-viz-list',
    });

    saveDashboardOrderChanges($new_subgroup.find('.menu-viz-list'));

    appsInSubgroup.forEach(function(db) {
      addDashboardToSubgroup(db[0], db[2], db[3], db[1], groupid,
                             $new_subgroup.find('ul'),
                             false /* does not need saving in backend */);
    });

    // saveNeeded indicates if this only displaying existing subgroups or are
    // these new subgroups that need to be saved in the backend
    if (saveNeeded) {
      setupEditableTitle($new_subgroup.find('.editable-title'));
      needsSave();
    }
  }

  function loadPageFromParent() {
    $appNameInput.val(parentGroup.group_name());
    $('#app-metadata-id').text(`ID: ${appgroup_id}`);
    $('#app-metadata-updated')
      .text(`Last updated: ${arc.misc.fromNow(parentGroup.group_updated())}`);
    $('#app-metadata-update-user').text(`Modified by: ${parentGroup.group_update_user()}`);

    var parentSettings = parentGroup.group_options().user_settings;
    if (parentSettings) {
      userSettings.loadFromValues(parentSettings);
      arc.css.CUSTOMCSS_MANAGER.appgroupSettings(userSettings);
    }

    paintUserSettings();
  }

  function paintAppGroupAndSubgroups(al) {
    canEdit = true;
    parentGroup = undefined;

    deletedGroups = [];
    updatedAppsInSubgroups = [];
    subGroups = [];

    // Enable Delete App and show 'Launch Button'
    $('#view-appgroup-btn').removeClass('arc-invisible');

    $appMenuSectionParent.empty();

    al.forEach(function(ag) {
      canEdit = canEdit && ag.rw;

      if (ag.group_parent_id) {
        subGroups.push(arc.model.AppGroup()
                          .loadFromModel(ag));
      } else {
        parentGroup = arc.model.AppGroup()
                         .loadFromModel(ag);
      }
    });

    if (canEdit) {
      $('#delete-appgroup').removeClass('disabled');
      $saveBtn.removeClass('save-disabled');
    } else {
      arc.report.alertMessage('You do not have the necessary workspace permissions to edit');
      return;
    }

    loadPageFromParent();

    subGroups.sort(function(a, b) {
      var a1 = a.group_subgroup_order(),
          b1 = b.group_subgroup_order();
      if (a1 < b1) return -1;
      if (a1 > b1) return 1;
      return 0;
    });

    subGroups.forEach(function(ag) {
      addSubGroupToAppGroup(ag.group_id(),
                            ag.group_name(),
                            appMenuSectionTemplate,
                            false,
                            ag.apps_in_subgroup());

    });

    arc.report.pagenav.needs_save(false);

    // Set up editable titles for all titles in page that already exist.
    // New ones shall be handled as a subgroup is added.
    // Disable drag of menu section when user clicks on editable div
    setupEditableTitle($('.editable-title'));

    const isPreviewFirstScreenShown = !isTabChanged && location.hash === '#tab-app-content-style';
    if (isPreviewFirstScreenShown) {
      drawAppPreview();
    }
  }

  appgroup.initialize_app = function() {
    // setupPage sets up the template and the click handlers for
    // dragging, sorting, enablingSave on edits etc. This should be called only
    // once
    setupPageOnceOnly();

    // fetchAndPaint is called to read the appgroup details and draw that
    // including subgroup information and settings. This will be called
    // every time there is a save. The same appgroup can be saved multiple
    // times.
    fetchAndPaint();

    return appgroup;
  };

  return appgroup;
};
