/*
 * This file is js code that was moved from app_groups.html
 *
 * TODO: The code here needs to be cleaned up, it is currently one
 *       large initialize() function
 */

window.arcl = window.arcl || {};
window.arc = window.arc || {};
arc.screen = arc.screen || {};

arc.screen.isAppsScreen = false;

arc.screen.apps = function () {
  const apps = function () { return 'arc.screen.apps'; };

  const APPGROUP_ALL = -1;
  const APPGROUP_STARRED = -2;
  const APPGROUP_SAMPLES = -4;

  const $sidebarContainer = $('.app-side-nav');
  const $sidebar = $sidebarContainer.find('.side-menu-wrapper');
  const $appgroupSidebar = $sidebar.find('.app-menu-sortable');
  const $workspaceSidebar = $sidebar.find('.ws-menu-list');
  const $privateWorkspaceSidebar = $sidebar.find('.ws-private-menu-list');
  const $privateWorkspaceToggle = $sidebar.find('a.toggle-private-workspace');
  const $privateWorkspaceLabel = $privateWorkspaceToggle.find('span');
  const $appgroupSidebarTemplate = $('#add-app-group-sidebar');
  const $appsubgroupSidebarTemplate = $('#add-app-subgroup-sidebar');
  const $workspaceSidebarTemplate = $('#add-workspace-sidebar');

  let filteredReportsByGroup = [];
  let filteredSubgroups = [];
  let starredAppgroupId;
  let gridMode = true;
  let show_apps_sidebar;

  let groupActiveId;
  let groupActiveName;
  let groupActive;
  let selectionType;

  let listTable;
  let initialSelections = '';
  let datasetsHash = {};

  const workspaces = arc.workspace.workspaces();

  const VisualPageDisplayFilters = {
    all: 'all',
    dashboards: 'dashboards',
    apps: 'apps'
  };

  const $topMenuButtons = $('.viz-display-radio-btns');
  let displayMode = $topMenuButtons.find('label.active input').attr('value') || VisualPageDisplayFilters.all;

  const ascending = function (a, b) {
    if (a > b) {
      return 1;
    }

    if (a < b) {
      return -1;
    }

    return 0;
  };

  const descending = (a, b) => ascending(b, a);
  const ascendingOnKey = (key) => (a, b) => ascending(a[key], b[key]);
  const descendingOnKey = (key) => (a, b) => descending(a[key], b[key]);

  arc.screen.apps.groupActiveId = function (_) {
    if (!arguments.length) return groupActiveId;
    groupActiveId = _;
    return groupActiveId;
  };

  function setDisplayMode(evt) {
    const mode = $(evt.currentTarget).val();
    if (mode === displayMode) {
      return;
    }

    arc.frontend.vizCookie.setVisualTabSettingByKey('displaymode', mode);
    if (window.location.search !== '') {
      const { pathname, hash } = window.location;

      const url = `${pathname}?displaymode=${mode}${hash}`;
      arc.misc.pushState(url);
    }

    displayMode = mode;

    // If "All" tab, refilter. Otherwise, use stored filteredApps
    if (!isWorkspace()) {
      filteredReportsByGroup = [...arcl.apps];
    }

    drawDasboardVisualAndAppViewer();
  }

  function initializeTypeSelector() {
    $('input[name=viz-display-radio]').offOn('change.viztype', setDisplayMode);
    $('input[name=viz-display-radio-list]').offOn('change.viztype', setDisplayMode);

    // Set initial display mode to be based on previous set up stored in cookie..
    displayMode = arc.frontend.vizCookie.getVisualTabSettingByKey('displaymode') || VisualPageDisplayFilters.all;

    // .. Or if query parameter contains displaymode, use that. This  means the page
    // has been opened from the home page by clicking on visuals or dashboards
    if (arcl.displaymode !== '') {
      displayMode = arcl.displaymode;
    }

    // Or override display mode if workspaces are hidden
    if (arc.frontend.featureFlipper.getValueById('hide_workspaces')) {
      displayMode = 'all';
    }
  }

  function maybeFilterVisuals(applications, filter) {
    if (filter === VisualPageDisplayFilters.all) {
      return applications;
    }

    return applications.filter((a) => {
      switch (a.report_type) {
        case 'apps':
          return filter === VisualPageDisplayFilters.apps;
        default:
          return filter === VisualPageDisplayFilters.dashboards;
      }
    });
  }

  function processAppgroups() {
    arcl.appgroup_membership_by_id = {};

    function addAppgroups(appgroup_array, appgroup_id) {
      if (!appgroup_array) {
        return;
      }

      appgroup_array.forEach((app) => {
        const app_id = app[0];
        if (arcl.appgroup_membership_by_id[app_id]) {
          arcl.appgroup_membership_by_id[app_id].push(appgroup_id);
          return;
        }

        arcl.appgroup_membership_by_id[app_id] = [appgroup_id];
      });
    }

    arc.globals.appgroups.forEach((d) => {
      if (d.group_options) {
        try {
          d.group_options = JSON.parse(d.group_options);
        } catch (err) {
          d.group_options = {};
        }
      }

      addAppgroups(d.apps, d.group_id);

      const { subgroups } = d;
      if (subgroups) {
        subgroups.forEach((subgroup) => {
          addAppgroups(subgroup.apps, subgroup.group_id);
        });
      }
    });
  }

  function postAppToAppgroupMapping(appgroups, cb) {
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      data: JSON.stringify(appgroups),
    };

    $.ajax({
      url: `${arc.frontend.globals.urls.appgroupUrl}/0/app/0`,
      data: post_data,
      type: 'POST',
      beforeSend(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
      .fail((req, textstatus, errorthrown) => {
        arc.report.alertMessage('Failed to update apps: ', `${textstatus}. ${errorthrown}`);
      })
      .done((results) => {
        if (cb) {
          cb(results);
        }
      });
  }

  /* ### SEARCHBOX ###
     To handle async rendering if grid search must filter existing DOM elements and
     those queued to be added.  On input in the searchbox:
        1) update global '_searchFunc' with the new query
        2) iterate over the grid and filter by toggling style.display based on _searchFunc
        3) add selected apps to list

     To handle the unrendered grid elements, their display style is set based on the
     global '_searchFunc' from step 1 above as they are appended. see _addChunk
   */

  const searchKeys = [
    'created_by',
    'dataset_detail',
    'dataset_name',
    'id',
    'report_type',
    'report_tags',
    'title',
    'type',
    'updated_by',
    'shared_visual_dashboards',
  ];

  let activeGridItems = [];
  const _setDisplay = (applications) => {
    // iterate over grid and hide/show items
    const appIds = new Set(applications.map((a) => a.id));
    activeGridItems.forEach((li) => {
      li.style.display = appIds.has(li.__data__.id) ? null : 'none';
    });

    addAppsToList(applications);
  };

  const _searchFactory = arc.report.buildSearch({ searchKeys });

  let searchableApps = [];
  let _searchFunc = (obj) => ({ obj, priority: 1 });
  const _executeSearch = (evt) => {
    _searchFunc = _searchFactory(evt.currentTarget.value);
    const applications = searchableApps.map(_searchFunc)
      .filter((d) => d)
      .sort((a, b) => a.priority - b.priority)
      .map((d) => d.obj);
    _setDisplay(applications);
  };

  const _setupSearch = (selector) => {
    const $searchBox = $(selector);

    const _debouncedSearch = arc.Stream.debounce(_executeSearch, 300);

    const _blurAndSearchOnEnter = (evt) => {
      if (evt.key === 'Enter') {
        evt.preventDefault();
        $(evt.currentTarget).blur();
      }
    };

    const _clearSearch = () => $searchBox.val('').trigger('input');

    $searchBox.on('input', _debouncedSearch).on('keypress', _blurAndSearchOnEnter);
    $searchBox.siblings('.attr-search-clear').on('click', _clearSearch);
  };

  function setupTooltips() {
    $('[data-toggle="tooltip"]').tooltip({
      delay: { show: 500, hide: 100 },
    });

    $('.collapse').collapse({ toggle: false });
  }

  function displayTopMenuButtons() {
    if (
      (groupActiveId === APPGROUP_ALL || isWorkspace()) &&
      !arc.frontend.featureFlipper.getValueById('hide_workspaces')
    ) {
      $topMenuButtons.removeClass('arc-invisible');
    } else {
      $topMenuButtons.addClass('arc-invisible');
    }

    const selectedIdsAndTypes = arc.screen.apps.selectedAppIdsAndTypes();
    const selectedApps = selectedIdsAndTypes.apps.map((d) => `appgroups${d}`);
    const selectedIds = selectedIdsAndTypes.visual.map((v) => `visual${v}`)
      .concat(selectedIdsAndTypes.dashboard.map((d) => `dashboard${d}`))
      .concat(selectedApps);

    const accelSelector = '#app-accel-btn';
    const accelDisabledSelector = '.arc-accel';

    if (!arc.globals.site_enable_app_acceleration) {
      $(accelSelector).fadeOut(300);
    }

    if (selectedIds.length > 0) {
      $('.visuals-report-menu-list').removeClass('no-items-selected');

      // Only apps have been selected
      if (selectedIds.length === selectedApps.length) {
        $(accelDisabledSelector).addClass('disabled-accel-btn');
      } else {
        $(accelDisabledSelector).removeClass('disabled-accel-btn');
      }

      $('.app-grid-list a[data-href]')
        .removeAttr('href')
        .off('click')
        .on('click', function () {
          $(this).closest('li').find('.btn-sel').trigger('click');
        });
    } else {
      $('.visuals-report-menu-list').addClass('no-items-selected');
      $(accelDisabledSelector).removeClass('disabled-accel-btn');

      if (groupActiveId > 0 && isAppGroup()) {
        const appUrl = `${arc.frontend.globals.urls.viewAppUrl}/${groupActiveId}`;
        $('#view-appgroup-btn').offOn('click.viewAppGroup', () => arc.misc.openInNewWindow(appUrl));
      }

      $('.app-grid-list a[data-href]').attr('href', function () {
        return $(this).attr('data-href');
      })
        .off('click');
    }

    if ($('#report-table tbody input:not(:checked)').length > 0) {
      $('#all-apps-checkbox').prop('checked', false);
    }
  }

  function setupAllSelect() {
    const _selectAll = (evt) => {
      $('.app-sel').prop('checked', evt.currentTarget.checked);
      displayTopMenuButtons();
    };

    $('body').off('click.selectAll').on('click.selectAll', '#select-app-all', _selectAll);
  }

  function activateTopMenuTab() {
    $(`input[name=viz-display-radio][value="${displayMode}"]`).trigger('click');
    $(`input[name=viz-display-radio-list][value="${displayMode}"]`).trigger('click');
  }

  function setGridMode(mode) {
    if (gridMode !== mode) {
      gridMode = mode;
      arc.frontend.vizCookie.setVisualTabSettingByKey('displaystyle', mode);
    }

    activateTopMenuTab();
  }

  function setupGridAndList() {
    const displayStyle = arc.frontend.vizCookie.getVisualTabSettingByKey('displaystyle');
    gridMode = typeof displayStyle === 'boolean' ? displayStyle : true;

    function selectList(e) {
      e.stopPropagation();

      // Mark current selections on list
      markSelectionsOnList();
      setGridMode(false);

      $('#app-list').fadeIn();
      listTable.columns.adjust();
      $('#app-grid').fadeOut('fast');
      window.location.hash = 'arc-list';
    }

    function selectGrid(e) {
      e.stopPropagation();

      // Mark current selections on grid
      markSelectionsOnGrid();
      setGridMode(true);

      $('#app-grid').fadeIn();
      $('#app-list').fadeOut('fast');
      window.location.hash = 'arc-grid';
    }

    $('#app-grid-btn1').off('click.setLayout').on('click.setLayout', selectGrid);
    $('#app-grid-btn2').off('click.setLayout').on('click.setLayout', selectGrid);
    $('#app-list-btn1').off('click.setLayout').on('click.setLayout', selectList);
    $('#app-list-btn2').off('click.setLayout').on('click.setLayout', selectList);
  }

  function displayGridOrList(useInitial = false) {
    // Remember the user's current display-style
    if (window.location.hash === '#arc-list') {
      setGridMode(false);
      $('#app-list').fadeIn();
      listTable.columns.adjust();
    } else if (window.location.hash === '#arc-grid') {
      setGridMode(true);
      $('#app-grid').fadeIn();
    } else if (gridMode) {
      setGridMode(true);
      $('#app-grid').fadeIn();
    } else {
      setGridMode(false);
      $('#app-list').fadeIn();
      listTable.columns.adjust();
    }

    if (useInitial) {
      $('.edit-mode, .edit-accel-mode').show();
      if (gridMode === 'grid') {
        markSelectionsOnGrid(true);
      } else {
        markSelectionsOnList(true);
      }
    }
  }

  function setupSidebarOrdering() {
    // make side appgroup list sortable
    $appgroupSidebar.sortable({
      placeholder: 'sortable-placeholder',
      opacity: 0.7,
      cancel: 'ul[data-appgroup-parent]',
      start(event, ui) {
        const start_pos = ui.item.index();
        ui.item.data('start_pos', start_pos);
      },

      update() {
        const um = arc.model.UserAppGroupMembership();
        $appgroupSidebar.find('>li').each(function (i) {
          um.add_membership($(this).data('appgroup'), true, i + 1);
        });

        um.save(() => {
          arc.misc.notify('App order modification has been saved');
        });
      },
    });
  }

  function markSelectionsOnGrid(useInitial = false) {
    const appIdsHash = useInitial ? _getInitialAppsHash() : _getAppsHash();

    // Remove current selections
    $('.app-grid-list .btn-on').toggleClass('btn-on btn-off');

    $('.app-grid-list li').each(function () {
      const type = _getSimplifiedType($(this).attr('data-type'));
      const appId = `${type}${$(this).attr('data-app')}`;
      if (appIdsHash[appId]) {
        $(this).find('.btn-off').toggleClass('btn-on btn-off');
      }
    });
  }

  function markSelectionsOnList(useInitial = false) {
    const appIdsHash = useInitial ? _getInitialAppsHash() : _getAppsHash();
    $('#report-table tbody tr').each(function () {
      const $this = $(this);
      const type = _getSimplifiedType($this.attr('data-type'));
      const appId = `${type}${$this.attr('data-app')}`;
      $this.find('input').prop('checked', appIdsHash[appId]);
    });
  }

  function _getSimplifiedType(type) {
    return type !== 'dashboard' && type !== 'apps' ? 'visual' : type;
  }

  function _getAppsHash() {
    const appIds = arc.screen.apps.selectedAppIdsAndTypes();
    const appIdsHash = {};
    appIds.visual.forEach((appId) => { appIdsHash[`visual${appId}`] = true; });
    appIds.dashboard.forEach((appId) => { appIdsHash[`dashboard${appId}`] = true; });
    appIds.apps.forEach((appId) => { appIdsHash[`apps${appId}`] = true; });
    return appIdsHash;
  }

  function _getInitialAppsHash() {
    const ids = initialSelections.split('_');
    const appsIdsHash = {};

    // Don't know whether dashboards or visuals, but mark both because can't overlap
    ids.forEach(function markSelected(id) {
      appsIdsHash[`visual${id}`] = true;
      appsIdsHash[`dashboard${id}`] = true;
    });

    return appsIdsHash;
  }

  arc.screen.apps.selectedAppIdsAndTypes = function () {
    const appIds = {};
    let id;
    let type;
    let typeid;
    let $sel;

    appIds.visual = [];
    appIds.dashboard = [];
    appIds.apps = [];

    if (gridMode) {
      $sel = $('.app-grid-list .btn-on').closest('li');
    } else {
      $sel = $('#report-table tbody input:checked').closest('tr');
    }

    $sel.each(function () {
      id = $(this).attr('data-app');
      type = $(this).attr('data-type');
      typeid = `${type}${id}`;
      if (arcl.apps_by_typeid[typeid].report_type === 'dashboard') {
        appIds.dashboard.push(id);
      } else if (arcl.apps_by_typeid[typeid].report_type === 'apps') {
        appIds.apps.push(id);
      } else {
        appIds.visual.push(id);
      }
    });

    return appIds;
  };

  function checkLengthOfSelectedAppsVisuals(onlyAcceleratable) {
    const idAndTypes = arc.screen.apps.selectedAppIdsAndTypes();
    let ids = [];

    if (idAndTypes.visual.length) {
      ids = idAndTypes.visual;
    }

    if (idAndTypes.dashboard.length) {
      ids = ids.concat(idAndTypes.dashboard);
    }

    if (!onlyAcceleratable) {
      if (idAndTypes.apps.length) {
        ids = ids.concat(idAndTypes.apps);
      }
    }

    return ids;
  }

  function setupAppAccel() {
    $('#app-accel-btn').offOn('click.appAccel', (e) => {
      const ids = checkLengthOfSelectedAppsVisuals(true);

      if (!ids.length) {
        arc.report.alertMessageSmall('Please select dashboard(s) below', 'alert-info');
        e.stopPropagation();
        return;
      }

      window.location = `app_accel?appids=${ids.join('_')}`;
    });
  }

  function deleteApp(appId, type, cb) {
    const urlBase = type === 'apps' ? arc.frontend.globals.urls.appgroupUrl :
      arc.frontend.globals.urls.reportUrl;
    $.ajax({
      url: `${urlBase}/${appId}`,
      data: { csrfmiddlewaretoken: arc.globals.csrf_token },
      type: 'DELETE',
      beforeSend(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
      .done(() => {
        const $toDelete = $(`[data-app="${appId}"][data-type="${type}"]`);
        const row = $toDelete.filter((f, g) => {
          if (g.tagName === 'TR') {
            return true;
          }

          $(g).remove();
          return false;
        });
        listTable.row(row).remove();
        listTable.draw();
        listTable.columns.adjust();
      })
      .fail((req) => {
        arc.report.alertMessage('Failed to delete app',
          req.responseText ? req.responseText : req, { is_html: true });
        cb('error');
      })
      .always(cb);
  }

  function deleteApps(appIds, clearSelection) {
    const appDisplayName = [];
    if (appIds.visual && appIds.visual.length) {
      appDisplayName.push(`Visual(s) (${appIds.visual.join(', ')})`);
    }

    if (appIds.apps && appIds.apps.length) {
      appDisplayName.push(`App(s) (${appIds.apps.join(', ')})`);
    }

    if (appIds.dashboard && appIds.dashboard.length) {
      appDisplayName.push(`Dashboard(s) (${appIds.dashboard.join(', ')})`);
    }

    const c = arc.misc.deleteConfirmation();
    c.objectType('')
      .objectName(appDisplayName.join('; '))
      .askUser(() => {
        // We need to delete multiple apps. Do it one
        // at a time, and error out on the first one.
        let appIdIdx = 0;
        let typeIdx = 0;
        const types = Object.keys(appIds);
        function deleteNextApp() {
          const appId = appIds[types[typeIdx]][appIdIdx];
          if (!appId) {
            appIdIdx = 0;
            typeIdx += 1;
            if (!types[typeIdx]) {
              c.hideModal();
              if (clearSelection) {
                $('#clear-selection').trigger('click');
              }

              fetchAndDraw(true);
            } else {
              deleteNextApp();
            }

            return;
          }

          deleteApp(appId, types[typeIdx], (err) => {
            if (err && err.length) {
              c.hideModal();
              $('#clear-selection').trigger('click');
              return;
            }

            appIdIdx += 1;
            deleteNextApp();
          });
        }

        deleteNextApp();
      });
  }

  function setupAppDelete() {
    $('#delete-apps-btn').offOn('click.deleteApp', () => {
      const appIds = arc.screen.apps.selectedAppIdsAndTypes();
      deleteApps(appIds, true /* clearSelection */);
    });

    $(document)
      .off('click.deleteApp')

      // Set event on document and filter for .delete-app because
      // buttons aren't on page at this point
      .on('click.deleteApp', '.delete-app', (e) => {
        const appIds = {};
        appIds[e.target.dataset.type] = [e.target.dataset.appid];
        deleteApps(appIds);
      });
  }

  function saveOrderChange($sel) {
    $sel.offOn('sortupdate', (event, ui) => {
      const $curr = $(ui.item);
      const groupid = Number($curr.attr('data-group'));
      if (groupid < 0) {
        return;
      }

      const $prev = $curr.prev();
      const $next = $curr.next();
      let order = 1;
      if ($next.length === 0 && $prev.length === 0) {
        order = 1;
      } else if ($next.length === 0) {
        order = Math.ceil($prev.data('order') + 1);
      } else if ($prev.length === 0) {
        order = $next.data('order') / 2;
        if (order > 1) {
          order = Math.floor(order);
        }
      } else {
        order = ($prev.data('order') + $next.data('order')) / 2;
      }

      $curr.data('order', order);

      const appgroups = {
        group_id: $curr.attr('data-group'),
        app_id: $curr.attr('data-app'),
        app_present: true,
        order,
      };

      postAppToAppgroupMapping([appgroups], () => {
        arc.report.alertMessageSmall(
          'Visual ordering updates have been saved', 'alert-success'
        );

        fetchAndDraw();
      });
    });
  }

  function isWorkspace(forceSelectionType) {
    if (forceSelectionType) {
      return forceSelectionType === 'workspaces';
    }

    return selectionType === 'workspaces';
  }

  function isAppGroup(forceSelectionType) {
    if (forceSelectionType) {
      return forceSelectionType === 'appgroups';
    }

    return selectionType === 'appgroups';
  }

  function idProperty(forceSelectionType) {
    return isAppGroup(forceSelectionType) ? 'group_id' : 'id';
  }

  function selectCookieOrUrlGroup() {
    let groupid = groupActiveId;
    if (!groupActiveId) {
      groupid = arc.frontend.vizCookie.getVisualTabSettingByKey('appgroupid');
      selectionType = arc.frontend.vizCookie.getVisualTabSettingByKey('selectiontype') || 'appgroups';
    }

    if (groupid) {
      groupid = Number(groupid);
    } else {
      groupid = APPGROUP_ALL;
    }

    selectGroup(groupid);
  }

  function selectGroup(groupid, cb) {
    if (!arcl.apps) {
      return;
    }

    selectionType = selectionType || 'appgroups';

    if (arcl.override_group_name) {
      const overrideType = arcl.override_group_type || 'appgroups';
      arc.globals[overrideType].some((d) => {
        if (d.name === arcl.override_group_name) {
          if (d.group_type === 'samples') {
            groupid = APPGROUP_SAMPLES;
          } else {
            groupid = d[idProperty(overrideType)];
          }

          selectionType = overrideType;
          return true;
        }

        return false;
      });

      arcl.override_group_name = undefined;
    } else if (arc.frontend.featureFlipper.getValueById('hide_workspaces')) {
      if (![APPGROUP_ALL, APPGROUP_STARRED].includes(groupid)) {
        // If workspaces are hidden, then default to showing "All" unless
        // the page is forcing a specific tab via url
        selectionType = 'appgroups';
        groupid = APPGROUP_ALL;
        displayMode = 'all';
      }

      // Handle favorite icon
      const isFavorite = groupid === APPGROUP_STARRED;
      const starTooltip = isFavorite ? 'Show all' : 'Show favorite';
      const $favoriteIcon = $('.show-favorites-icon');
      $favoriteIcon.toggleClass('fa-star-o', !isFavorite);
      $favoriteIcon.toggleClass('fa-star', isFavorite);
      $favoriteIcon.attr('title', starTooltip);

      $favoriteIcon.offOn('click.toggleScreen', () => {
        selectGroup(isFavorite ? APPGROUP_ALL : APPGROUP_STARRED);
      });
    }

    const $e = $(`.app-menu[data-selectiontype=${selectionType}]`)
      .find(`a[data-appgroup="${groupid}"]`);
    $('.app-menu li, .app-menu li li').removeClass('active');
    $e.parent().parent().addClass('active');

    groupActive = undefined;
    groupActiveId = groupid;
    arc.frontend.vizCookie.setVisualTabSettingByKey('appgroupid', groupActiveId);
    arc.frontend.vizCookie.setVisualTabSettingByKey('selectiontype', selectionType);
    workspaces.currentWorkspace({ workspace_perm: ['ws_appedit'] });
    workspaces.originalWorkspace({ workspace_perm: ['ws_appedit'] });
    if (groupActiveId === APPGROUP_ALL) {
      groupActiveName = 'All';
    } else if (groupActiveId === APPGROUP_STARRED) {
      groupActiveName = 'My Favorites';
    } else if (groupActiveId === APPGROUP_SAMPLES) {
      groupActiveName = 'Samples';
    } else {
      const found = arc.globals[selectionType].some((d) => {
        if (d[idProperty(selectionType)] === groupActiveId) {
          groupActiveName = d.name;
          groupActive = d;
          return true;
        }

        if (isAppGroup() && d.subgroups) {
          return d.subgroups.some((sd) => {
            if (sd.group_id === groupActiveId) {
              groupActiveName = sd.name;

              // Expand the subgroups within the parent, so that the selected
              // group is visible
              $e.closest('ul').show();
              const expand_sel = $e.closest('ul').closest('li').find('.app-group-expand');
              expand_sel.addClass('fa-angle-down');
              expand_sel.removeClass('fa-angle-right');

              return true;
            }

            return false;
          });
        }

        return false;
      });

      if (!found) {
        selectGroup(APPGROUP_ALL, cb);
        return;
      }

      if (isWorkspace()) {
        // Set original and currentWorkspace selections
        const currWorkspace = {
          workspace_id: groupActive.id,
          workspace_name: groupActive.name,
          workspace_perm: groupActive.perms,
        };
        workspaces.currentWorkspace(currWorkspace);
        workspaces.originalWorkspace(currWorkspace);
      }
    }

    arcl.filterAppsOnGroupSelection(groupActiveId, cb);
  }

  function sortSubgroups(subgroups) {
    subgroups.sort(ascendingOnKey('subgroup_order'));
  }

  function drawSidebar() {
    drawWorkspaceSidebar();
    if (setupEditSettings) {
      drawAppGroupsSidebar();
    }
  }

  function drawAppGroupsSidebar() {
    let sidebar_groups = [];
    let sample_group;
    arc.globals.appgroups.forEach((d) => {
      if (d.group_type === 'samples') {
        sample_group = d;
        arcl.samples_appgroup_id = d.group_id;
      }

      if (d.in_userlist === 'true') {
        sidebar_groups.push(d);
      }
    });

    const sample_sel = $appgroupSidebar.find(`[data-appgroup="${APPGROUP_SAMPLES}"]`);
    if (sample_group) {
      sample_sel.find('.badge').text(sample_group.num_apps);
    } else {
      sample_sel.find('.badge').text('-1');
    }

    // Lower order ids come first
    sidebar_groups = sidebar_groups.sort(ascendingOnKey('userlist_order'));

    $appgroupSidebar.find('li').remove();
    sidebar_groups.forEach((d) => {
      const li = arc.misc.safe_substitute($appgroupSidebarTemplate.html(),
        arcl.appgroupToSidebarMap(d));
      $appgroupSidebar.append(li);

      if (d.subgroups && d.subgroups.length > 0) {
        sortSubgroups(d.subgroups);

        const { subgroups } = d;
        const subgroups_sel = $appgroupSidebar.find(`ul[data-appgroup-parent="${d.group_id}"]`);
        subgroups.forEach((sd) => {
          const sli = arc.misc.safe_substitute(
            $appsubgroupSidebarTemplate.html(),
            arcl.appgroupToSidebarMap(sd)
          );
          subgroups_sel.append(sli);
        });
      }
    });

    setupSidebarOrdering();
    setupGroupClicks();
    setupExpandCollapse();
    setupEditSettings();
    setupSubgroupNav();
  }

  function setupGroupClicks() {
    $('.app-menu > li').off('click.appgroup').on('click.appgroup', (evt) => {
      // Don't switch tabs if user is clicking on "edit" gear
      if ($(evt.target).hasClass('edit-workspace')) {
        return;
      }

      const d3_el = $(evt.currentTarget).find('a[data-appgroup]');
      selectionType = d3_el.closest('.app-menu').data('selectiontype');
      const selectionId = parseInt(d3_el.data('appgroup'), 10);
      selectGroup(selectionId);
    });
  }

  function isOtherUserPrivate(ws) {
    return !ws.editable && ws.name !== 'Private' && ws.name !== 'Public';
  }

  function drawWorkspaceSidebar() {
    $workspaceSidebar.find('li').remove();
    $privateWorkspaceSidebar.find('li').remove();

    let activatePrivate;
    arc.globals.workspaces.forEach((ws) => {
      if (!ws.id) {
        return;
      }

      let $ul = $workspaceSidebar;
      const editable = arc.misc.hasPerm('ws_manage', ws.perms);

      // Other user's private workspaces are hidden in expandable ul
      if (isOtherUserPrivate(ws)) {
        $ul = $privateWorkspaceSidebar;
        activatePrivate = true;
      }

      // Account for workspaces that have no apps
      ws.apps = arcl.apps_by_workspace_id[ws.id] || [];

      let workspaceDescription = '';
      if (ws.desc !== '') {
          workspaceDescription = `Description: ${ws.desc}`;
      }

      const workspaceInfo = {
        HTML_SPECIAL_ICON: workspaces.getSpecialIcon(ws.name) || workspaces.getSpecialIcon('Empty'),
        WORKSPACE_NAME: ws.name,
        WORKSPACE_ID: ws.id,
        WORKSPACE_DESC: workspaceDescription,
        NUMBER_REPORTS: ws.apps.length,
        EDITABLE: editable ? '' : 'arc-invisible',
      };

      const newRow = arc.misc.safe_substitute($workspaceSidebarTemplate.html(),
        workspaceInfo);

      if (['Public', 'Private'].includes(ws.name)) {
        $ul.prepend(newRow);
      } else {
        $ul.append(newRow);
      }
    });

    $privateWorkspaceSidebar.addClass('arc-invisible');
    if (activatePrivate) {
      $privateWorkspaceToggle.removeClass('arc-invisible');
      $privateWorkspaceToggle.off('click.togglePrivateWS')
        .on('click.togglePrivateWS', function togglePrivateWS() {
          $privateWorkspaceLabel.toggleClass('arc-invisible');
          $privateWorkspaceSidebar.toggleClass('arc-invisible');
        });
    } else {
      $privateWorkspaceToggle.addClass('arc-invisible');
    }

    workspaces.setupEditButton($sidebar, fetchAndDraw);
  }

  function scrollToSubgroup(subgroupId) {
    $(`.subgroup-header[data-appgroup="${subgroupId}"]`)
      .each(function () {
        this.scrollIntoView();
      });
  }

  function setupSubgroupNav() {
    $('.app-menu li ul li a')
      .off('click')
      .on('click', function (e) {
        e.stopPropagation();
        e.preventDefault();

        const $this = $(this);
        const groupId = parseInt($this.closest('ul').attr('data-appgroup-parent'), 10);
        const subgroupId = $this.attr('data-appgroup');
        selectionType = $this.closest('.app-menu').data('selectiontype');

        if (groupActiveId !== groupId) {
          selectGroup(groupId, function selectSubgroup() {
            scrollToSubgroup(subgroupId);
          });
        } else {
          scrollToSubgroup(subgroupId);
        }
      });
  }

  function setupEditSettings() {
    $('.edit-appgroup').removeClass('arc-invisible');
    $('.edit-appgroup').offOn('click.editAppGroup', function (e) {
      e.stopPropagation();
      e.preventDefault();
      const appgroup = $(this).closest('a').attr('data-appgroup');
      window.location.href = `${arc.frontend.globals.urls.editAppUrl}/${appgroup}`;
    });
  }

  function setupExpandCollapse() {
    // toggle groups with subgroups when user clicks on arrow
    $('.fa-angle-right, .fa-angle-down').offOn('click.expand', (evt) => {
      evt.stopPropagation();
      $(evt.currentTarget)
        .toggleClass('fa-angle-down fa-angle-right')
        .parent()
        .next()
        .slideToggle();
    });
  }

  function processReports(reports, reportusage, appgroups) {
    const isVisualEnabled = !arc.frontend.featureFlipper.getValueById('disable_visuals_for_workflows');
    arcl.apps = reports.filter((r) => r.report_type === 'dashboard' ||
     (isVisualEnabled && r.shared_visual_dashboards));
    arcl.apps_by_typeid = {};
    arcl.apps_by_workspace_id = {};

    arcl.apps = arcl.apps.concat(appgroups.filter((ag) => ag.group_type === 'default'));

    arcl.apps.sort(descendingOnKey('updated'));

    arcl.apps.forEach((app) => {
      if (!app.type) {
        app.type = 'apps';
        app.id = app.group_id;
      }

      app.typeId = `${_getSimplifiedType(app.type)}${app.id}`;
      app.views_count = reportusage[app.id];
      arcl.apps_by_typeid[app.typeId] = app;

      if (arcl.apps_by_workspace_id[app.workspace_id]) {
        arcl.apps_by_workspace_id[app.workspace_id].push(app.typeId);
      } else {
        arcl.apps_by_workspace_id[app.workspace_id] = [app.typeId];
      }
    });

    appgroups.forEach(function addAppGroup(currApp) {
      if (currApp.group_type !== 'default') return;

      const subgroups = currApp.subgroups || [];
      sortSubgroups(subgroups);
      const subgroupApps = subgroups.reduce((total, current) => total.concat(current.apps),
        []);
      const allDashboardIds = currApp.apps.concat(subgroupApps).map((app) => app[0]);
      const allDashboardsDeduped = Array.from(new Set(allDashboardIds));

      const defaultBase = { baseImg: currApp.baseImg };
      let currentImageDash = allDashboardsDeduped[0] ?
        arcl.apps_by_typeid[`dashboard${allDashboardsDeduped[0]}`] :
        defaultBase;
      if (!currentImageDash || !arc.misc.hasScreenshot(currentImageDash.img)) {
        currentImageDash = defaultBase;
      }

      currApp.url = `${arc.frontend.globals.urls.viewAppUrl}/${currApp.id}`;
      currApp.edit_url = `${arc.frontend.globals.urls.editAppUrl}/${currApp.id}`;
      currApp.img = currentImageDash.img || defaultBase.baseImg;
      currApp.updated_by = currApp.update_user;
      currApp.views = 0;
      currApp.title = currApp.name || '';
      currApp.dataset_name = '';
      currApp.dataset_id = 1;
      currApp.dataconn_name = '';
      currApp.shared_visual_dashboards = JSON.stringify(allDashboardsDeduped);
      currApp.type = 'apps';
      currApp.report_type = 'apps';

      // Add for better searching
      currApp.created_by = currApp.create_user;
      currApp.dataset_detail = '';
      currApp.dataset_name = '';
      currApp.report_tags = '';
    });
  }

  function filterAppsWithNoPermission(starredGroup) {
    return starredGroup.apps.filter((appRank) => {
      const appInfo = arcl.apps_by_typeid[`dashboard${appRank[0]}`];
      const isWorkspaceDisabledAndApp = !appInfo &&
        arc.frontend.featureFlipper.getValueById('enable_clon e_info_badge');
      return isWorkspaceDisabledAndApp ||
        (appInfo && arc.misc.hasPerm('ds_appview', appInfo.perm));
    });
  }

  function fetchAndDraw(refetching) {
    let urlExtensionForClones = '';
    if (refetching) {
      arc.report.alertMessageSmallBase({
        alertHeader: 'Saving',
        alertMessage: 'Changes are being saved<span class="arc-loading"></span>',
        type: 'alert-info',
        fadeOut: true,
        fadeOutDelay: 3000,
        useHTML: true,
      });
    }

    if (arc.frontend.featureFlipper.getValueById('enable_clone_info_badge')) {
      urlExtensionForClones += '?getcloneinfo=true';
    }

    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.reportUrl}${urlExtensionForClones}`)
      .defer(d3.json, arc.frontend.globals.urls.reportUsageUrl)
      .defer(d3.json, arc.frontend.globals.urls.appgroupUrl)
      .defer(d3.json, arc.frontend.globals.urls.workspacesUrl)
      .defer(d3.json, arc.frontend.globals.urls.datasetUrl)
      .await((error, reports, reportusage, appgroups, allWorkspaces, datasets) => {
        if (error) {
          console.log('Could not load data');
          console.log(error);
          return;
        }

        datasetsHash = Object.fromEntries(datasets.map((d) => {
          const dataset = arc.datasets.Dataset()
            .dataset_id(d.id)
            .init_from_fetch(d);
          return [d.id, dataset];
        }));

        arc.globals.workspaces = arc.workspace.sortWorkspaces(allWorkspaces);
        workspaces.processWorkspace();
        arc.globals.appgroups = appgroups;
        processAppgroups();

        arcl.starred_appgroup = arc.globals.appgroups.filter((d) => d.group_type === 'starred');

        arc.misc.updateCreateAppDashboardButton(datasets, $('.subnav-report'));

        processReports(reports, reportusage, appgroups);

        let favoriteCount = 0;
        if (arcl.starred_appgroup.length === 1) {
          starredAppgroupId = arcl.starred_appgroup[0].group_id;

          // Filter out apps user doesn't have permission to view
          // When workspaces is disabled, include apps as well
          const starredGroup = arcl.starred_appgroup[0];
          starredGroup.apps = filterAppsWithNoPermission(starredGroup);

          starredGroup.num_apps = starredGroup.apps.length;
          favoriteCount = starredGroup.num_apps;
        }

        d3.selectAll('.app-menu-perm .permanent a .badge')
          .text((d, i) => {
            // All apps count
            if (i === 0) {
              return arcl.apps.length;
            }

            // "My Favorites" app count
            if (i === 1) {
              return favoriteCount;
            }

            return '';
          });

        drawSidebar();

        selectCookieOrUrlGroup();
        displayGridOrList(!refetching);
      });
  }

  function tableSettings(reports) {
    const dtCols = [
      {
        title: '<input type="checkbox" class="app-sel" id="select-app-all">',
        data: 'col0',
        orderable: false,
      },
      {
        title: 'Title',
        className: 'table-app-title',
        data: {
          _: 'col2.display',
          sort: 'col2.sort',
        },
      },
      {
        title: 'ID',
        data: 'col3',
      },
      {
        title: `<span>Related Dashboards /<br>Linked Visuals<span>`,
        className: 'dashboard-badge-wrapper',
        type: 'number',
        data: {
          _: 'col4.display',
          sort: 'col4.sort',
        },
      },
      {
        title: 'Created',
        data: {
          _: 'col5.display',
          sort: 'col5.sort',
        },
      },
      {
        title: 'Last Updated',
        data: {
          sort: 'col6.sort',
          _: 'col6.display',
        },
      },
      {
        title: 'Modified By',
        data: {
          _: 'col7.display',
          sort: 'col7.sort',
        },
      },
      {
        title: 'Total Views',
        data: 'col8',
      },
      {
        title: 'Dataset',
        data: {
          _: 'col9.display',
          sort: 'col9.sort',
        },
      },
      {
        className: 'action-button-wrapper',
        data: 'col11',
        orderable: false,
      },
    ];

    if (!arc.frontend.featureFlipper.getValueById('hide_workspaces')) {
      dtCols.splice(9, 0, {
        title: 'Workspace',
        data: 'col10.display',
      });
    }

    if (arc.frontend.featureFlipper.getValueById('enable_clone_info_badge')) {
      dtCols.splice(4, 0, {
        title: `<span>Descendant <br> Dashboards<span>`,
        className: 'dashboard-badge-wrapper',
        type: 'number',
        data: {
          _: 'col1.display',
          sort: 'col1.sort',
        },
      });
    }

    const _escape = arc.misc.escapeHTML;

    const _col0Template = ({ APP_ID, IMAGE, LINK, VISUAL_TYPE, EDIT_VISIBLE, EDIT_LINK }) => {
      const inline = 'margin-right:10px;';
      const target = VISUAL_TYPE === 'apps' ? '_blank' : '_self';
      LINK = arc.misc.getBlankTargetUrl(LINK, target);
      const href = `${LINK}" target="${target}`;
      return `<div style="width:110px; display:flex">
          <input style="${inline}" type="checkbox" class="app-sel" id="select-app-${APP_ID}">
          <a href="${href}">
            <div class="thumbnail-wrapper">
              <img src="${IMAGE}" class="report-thumbnail-img">
              <div class="object-type ${VISUAL_TYPE}"></div>
              <a style="display:${EDIT_VISIBLE};" class="edit-viz-square" href=${EDIT_LINK}></a>
            </div>
          </a>
        </div>`;
    };

    const _col2Template = ({ TITLE }) => `<span class="cell-wrapper" title="${_escape(TITLE)}">` +
      `${_escape(TITLE)}</span>`;

    const _col4Template = ({ CSS_COUNT, HTML_APPS }) => `<span class="cell-wrapper">` +
      `<span class="badge badge-secondary badge-pill">${CSS_COUNT}</span>` +
      `<span class="applist">${HTML_APPS}</span>` +
      `</span>`;

    const _col9Template = ({ DATASET_ID, DATASET, DATACONN_NAME }) => '<a href=' +
      `${arc.frontend.globals.urls.datasetEditUrl}/${DATASET_ID}>${DATASET}</a><br>` +
      `<span class="dataconnection-name">${DATACONN_NAME}</span>`;

    const _col10Template = ({ WORKSPACE_URL, WORKSPACE_NAME }) => `<a href="${WORKSPACE_URL}">` +
      `${WORKSPACE_NAME}</a>`;

    const _col11Template = ({ VISUAL_TYPE, EDIT_VISIBLE, EDIT_LINK,
      DELETE_VISIBLE, APP_ID, }) => '' +
      `<a style="display:${EDIT_VISIBLE};" href=${EDIT_LINK}>` +
      `<span class="fa fa-pencil"></span>` +
      `</a>` +
      `<a style="display:${DELETE_VISIBLE};">` +
      `<span data-type="${VISUAL_TYPE}" data-appid="${APP_ID}"` +
      'class="fa fa-trash delete-app"></span>' +
      `</a>`;

    const _col1Template = ({ DESCENDANTS_COUNT, IS_CLONE }) => {
      let template = `<span class="badge badge-secondary badge-pill">${DESCENDANTS_COUNT}</span>`;
      if (IS_CLONE) {
        const style = 'margin-left: 0.5rem; background-color: #BBBBBB;';
        template += `<span class="badge badge-info badge-pill" style="${style}">Clone</span>`;
      }

      return template;
    };

    const toRow = report => ({
      type: report.VISUAL_TYPE,
      url: report.LINK,
      col0: _col0Template(report),
      col2: {
        sort: report.TITLE.toLowerCase(),
        display: _col2Template(report),
      },
      col3: report.APP_ID,
      col4: {
        sort: Number(report.CSS_COUNT) || 0,
        display: _col4Template(report),
      },
      col1: {
        sort: Number(report.DESCENDANTS_COUNT) || 0,
        display: _col1Template(report),
      },
      col5: {
        display: report.HTML_CREATED_DATE,
        sort: report.CREATED_DATE_RAW,
      },
      col6: {
        display: report.HTML_MODIFIED_DATETIME_FORMATTED,
        sort: report.MODIFIED_DATETIME,
      },
      col7: {
        display: report.AUTHOR,
        sort: report.AUTHOR.toLowerCase(),
      },
      col8: report.VIEWS,
      col9: {
        sort: report.DATASET.toLowerCase(),
        display: _col9Template(report),
      },
      col10: {
        sort: report.WORKSPACE_NAME.toLowerCase(),
        display: _col10Template(report),
      },
      col11: _col11Template(report),
    });

    return {
      paging: true,
      lengthChange: false,
      pageLength: 50,
      destroy: true,
      deferRender: true,
      searching: false,
      info: true,
      order: [[6, 'desc']],
      language: {
        emptyTable: 'No data',
        paginate: { previous: '&lt;', next: '&gt;' },
      },
      columns: dtCols,
      data: reports.map(toRow),
      createdRow(row, data) {
        row.dataset.app = data.col3;
        row.dataset.type = data.type;
        row.dataset.href = data.url;
      },

      drawCallback({ oInit, nTableWrapper }) {
        const multiPage = oInit.pageLength < oInit.data.length;
        $(nTableWrapper).find('.dataTables_paginate').toggle(multiPage);
        $('[data-toggle="tooltip"]').tooltip();
      },
    };
  }

  function initializeDataTable() {
    $.fn.DataTable.ext.errMode = 'throw';
    $.fn.DataTable.ext.pager.numbers_length = 5;
  }

  function addAppsToList(apps_to_add) {
    const uniqueApps = new Set(apps_to_add.map((app) => app.typeId));
    const reports = apps_to_add.filter((app) => uniqueApps.delete(app.typeId)).map(app_to_map);
    const $table = $('#report-table');
    const settings = tableSettings(reports);

    // When workspaces are disabled, need to empty the table element
    // or else the table will not be drawn full width
    $table.empty();

    if (listTable) {
      listTable.destroy();
    }

    listTable = $table.DataTable(settings);
    $(listTable.table().container())
    .addClass('d-flex flex-column justify-content-start align-items-stretch');
  }

  function addAppsToSubgroupGrid({ name, display_apps, group_id }) {
    if (display_apps.length === 0) {
      return;
    }

    // build headers and containers for subgroups in grid mode
    const sel = d3.select('#app-grid')
      .selectAll('.nosel')
      .data([name])
      .enter()
        .append('h4')
        .attr('class', 'subgroup-header')
        .attr('data-appgroup', group_id)
        .text((d) => d)
        .append('ul')
        .attr('class', 'app-grid-list subgroup');

    arcl.addAppsToGrid(sel, display_apps, group_id);
  }

  function _clearGridAndList() {
    $('.app-grid-list').empty();
    $('.app-grid-list.subgroup').remove();
    $('.subgroup-header').remove();
    activeGridItems = [];
  }

  function _drawGridDisplay(applications, subGroups = []) {
    // add apps to grid view
    arcl.addAppsToGrid(d3.select('.app-grid-list'), applications, groupActiveId);

    // add subgroups of selected appgroup to grid view and set up drag reordering
    subGroups.forEach(addAppsToSubgroupGrid);

    // set up drag reordering on subgroups
    if (filteredSubgroups.length > 0) {
      const $grid = $('.app-grid-list.subgroup');
      $grid.sortable();
      saveOrderChange($grid);
    }

    // set grid to draggable if an appgroup is selected
    const isAppgroup = isWorkspace() || groupActiveId < 0;
    $('.app-grid-list').sortable(isAppgroup ? 'disable' : 'enable');
  }

  function drawDasboardVisualAndAppViewer() {
    const filteredAppByDisplayMode = maybeFilterVisuals(filteredReportsByGroup, displayMode);

    // draw grid display header
    $('.app-list-header span').text(groupActiveName);

    // all reports from subgroups of selected appgroup
    const subApps = filteredSubgroups.map((s) => s.display_apps).reduce((a, b) => [...a, ...b], []);
    searchableApps = [...filteredAppByDisplayMode, ...subApps];
    const listApps = searchableApps.filter(_searchFunc);

    _clearGridAndList();
    _drawGridDisplay(filteredAppByDisplayMode, filteredSubgroups);
    addAppsToList(listApps);
    setupTopMenuButtons();

    // When we have a large number of uncategorized apps in an appgroup, and
    // then use the left hand nav to click on a subgroup and then on the main
    // appgroup, scrolling back up to the top of the page does not work.
    // Scroll explicitly to work around this.
    document.getElementById('app-grid').scrollTop = 0;
  }

  function setupTopMenuButtons() {
    // Display buttons on top menu only if one or more apps selected
    displayTopMenuButtons();

    $('.report-menu-list.no-items-selected > li > a').on('mouseenter mouseleave', () => {
      $('.btn-sel').toggleClass('btn-sel-show');
    });

    $('.report-menu-list.no-items-selected > li > a').offOn('click.noItemSelectedActionBtns', () => {
      $('.btn-sel').addClass('btn-sel-none');
    });

    // For apps from All workspaces, enable move workspace button
    workspaces.currentWorkspace({
      workspace_perm: ['ws_appedit'],
    });
    workspaces.setupWorkspaceButton($('#subnav-report'), submitMoveCb);

    // toggle selection button on grid items between on/off
    // listener on the container as this can have thousands of elements
    $('.app-grid-list').off('click.selectApp').on('click.selectApp', (e) => {
      if (e.target.classList.contains('btn-sel')) {
        $(e.target).toggleClass('btn-on btn-off');
        displayTopMenuButtons();
      }
    });

    $('#report-table tbody').off('click').on('click', 'input:checkbox', displayTopMenuButtons);

    $('#clear-selection')
      .offOn('click.clearSelection', () => {
        $('.app-grid-list .btn-on').toggleClass('btn-on btn-off');
        $('#report-table tbody input:checked').removeAttr('checked');
        displayTopMenuButtons();
      });
  }

  function submitMoveCb(dashboardInfo) {
    workspaces.moveToNewWorkspace(dashboardInfo, () => fetchAndDraw(true));
  }

  const popoverText = $('#chart-content-popover').html();

  function addScreenAppsPopovers() {
    // popover for app info
    $('.app-grid-list .chart-small-label').popover({
      container: 'body', // makes popover be on top
      html: true,
      sanitize: false,
      trigger: 'hover',
      placement: 'top',
      delay: { show: '500' },
      title: '',
      content() {
        const d = d3.selectAll($(this).closest('li').toArray()).datum();
        const p = app_to_map(d);
        p.CONTAINING_CLASS = d.type === 'dashboard' || d.type === 'apps' ? 'arc-invisible' : '';
        p.HIDE_IN_APPGROUP = d.type === 'apps' ? 'arc-invisible' : '';
        return arc.misc.safe_substitute(popoverText, p);
      },
    });
  }

  function lazyLoadOnload() {
    const $div = $(this).parent();

    // If we have a screenshot in the data, then style differently
    if ($div.attr('data-hasscreenshot') === 'true') {
      $div.removeClass('placeholder-img-wrapper')
        .addClass('chart-small-img-wrapper');
    } else {
      $div.addClass('placeholder-img-wrapper-final');
    }
  }

  function setupLazyLoad(container) {
    const lazyLoadOptions = {
      threshold: 200,
      load: lazyLoadOnload,
      skip_invisible: true,
      container: $('#app-grid'),
    };

    // remove listeners added by previous calls
    $(container).off('scroll');
    $(container).find('img.lazy').lazyload(lazyLoadOptions);
  }

  let asyncDrawProcess = null;
  arcl.addAppsToGrid = function (d3object, apps_to_add, group_id, lazyload = true) {
    const gridTemplate = $('#appgrid-item').html().replace('lazy-template', 'lazy');
    const gridItem = gridTemplate.replace('data-src', 'src');
    const gridItemVisible = gridTemplate.replace('data-original', 'src');

    const renderSyncSize = 100;
    const container = d3object[0][0];

    const _gridItem = (obj, visible) => {
      const appData = app_to_map(obj);
      if (visible) {
        const gridHTML = arc.misc.safe_substitute(gridItemVisible, appData);

        if (appData.HAS_SCREENSHOT === 'true') {
          return gridHTML.replace('placeholder-img-wrapper', 'chart-small-img-wrapper');
        }

        return gridHTML.replace('placeholder-img-wrapper',
          'placeholder-img-wrapper placeholder-img-wrapper-final');
      }

      return arc.misc.safe_substitute(gridItem, appData);
    };

    const _addChunk = (chunk, loadThumbnail = false) => {
      const frag = document.createDocumentFragment();
      d3.select(frag)
        .selectAll('.nosel')
        .data(chunk)
        .enter()
        .append('li')
        .attr('class', 'appgrid-li')
        .attr('data-app', (d) => d.id)
        .attr('data-type', (d) => _getSimplifiedType(d.type))
        .attr('data-group', group_id)
        .attr('data-edit-link', (d) => d.edit_url)
        .attr('data-order', (d) => d.order || 1)
        .style({ display: (d) => (_searchFunc(d) ? null : 'none') })
        .html((d) => _gridItem(d, loadThumbnail));

      // track appended grid items for search
      activeGridItems = [...activeGridItems, ...frag.childNodes];
      container.appendChild(frag);
    };

    // if an existing draw is still in progress kill it
    clearInterval(asyncDrawProcess);
    asyncDrawProcess = null;

    // add first chunk synchronously
    const chunkSize = 100;
    _addChunk(apps_to_add.slice(0, renderSyncSize), true);

    const stagger = 100;
    let idx = renderSyncSize;
    asyncDrawProcess = setInterval(() => {
      _addChunk(apps_to_add.slice(idx, idx + chunkSize));
      idx += chunkSize;
      if (idx >= apps_to_add.length) {
        clearInterval(asyncDrawProcess);
        asyncDrawProcess = null;

        if (lazyload) {
          setupLazyLoad(container);
        }

        setupTooltips();
        addScreenAppsPopovers();
      }
    }, stagger);
  };

  // Time display methods done in native JS instead of using moment because the
  // moment code to generate was causing major delay generating JSON for table
  const _toMMMDDYYY = (time) => {
    const stdTime = time;
    const months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
    const arr = stdTime.split('T')[0].split('-');
    const [y, m, d] = arr;
    return `${months[Number(m) - 1]} ${d}, ${y}`;
  };

  const _timeAgo = (time) => {
    const stdTime = time;
    let timeDiff = Math.floor((Date.now() - Date.parse(stdTime)) / 1000);

    if (timeDiff < 0) {
      console.log('Timezone or computer/server issue. ' +
        `Off by ${-timeDiff} seconds (~${Math.ceil(-timeDiff / 3600)} hour(s))`);
      timeDiff = 0;
    }

    const mult = [60, 60, 24, 30, 12];
    const interval = ['second', 'minute', 'hour', 'day', 'month', 'year'];

    let seconds = 1;
    let i;
    for (i = 0; i < 5; i++) {
      if (timeDiff / (seconds * mult[i]) < 1) {
        break;
      }

      seconds *= mult[i];
    }

    const num = Math.round(timeDiff / seconds);
    const s = num > 1 ? 's' : '';

    return `${num} ${interval[i]}${s} ago`;
  };

  // TODO: combine with mapping in arc_reports
  function app_to_map(app) {
    const shared = JSON.parse(app.shared_visual_dashboards || '[]');

    const simplifiedType = _getSimplifiedType(app.type);
    const relatedType = simplifiedType === 'dashboard' ? 'visual' : 'dashboard';
    const related = shared.filter((id) => (arcl.apps_by_typeid || {})[`${relatedType}${id}`]);
    const deleteable = shared.length === 0 || app.report_type === 'dashboard';
    const deletePerm = (arc.misc.hasPerm('ds_appedit', app.perm) && deleteable) ||
      (app.report_type === 'apps' && app.rw);
    const relatedNames = related.map((id) => {
      if (arcl.apps_by_typeid && arcl.apps_by_typeid[`${relatedType}${id}`]) {
        return arcl.apps_by_typeid[`${relatedType}${id}`].title;
      }

      return '';
    });

    const dataset = datasetsHash[app.dataset_id];
    const tags = dataset ? dataset.tags() : [];

    const isDashboardWithEditPerm = app.report_type === 'dashboard' &&
      arc.misc.hasPerm('ds_appedit', app.perm);
    const isAppWithEditPerm = app.report_type === 'apps' && app.rw;
    const target = simplifiedType === 'apps' ? '_blank' : '_self';
    return {
      SRC: 'src', // to substitute within template. See #app-grid-mobile-template
      VISUAL_TYPE: simplifiedType,
      APP_ID: app.id,
      APP_ORDER: app.order || 1,
      LINK: arc.misc.getBlankTargetUrl(app.url, target),
      EDIT_LINK: app.edit_url,
      BASEIMAGE: app.baseImg,
      IMAGE: app.img || app.baseImg,
      HAS_SCREENSHOT: String(arc.misc.hasScreenshot(app.img)),
      TITLE: app.title,
      CREATED_DATE_RAW: app.created,
      HTML_CREATED_DATE: _toMMMDDYYY(app.created),
      MODIFIED_DATETIME: app.updated,
      WORKSPACE_NAME: app.workspace_name,
      WORKSPACE_URL:
        `${arc.frontend.globals.urls.appsUrl}?group=${app.workspace_name}&grouptype=workspaces`,
      HTML_MODIFIED_DATETIME_FORMATTED: _timeAgo(app.updated),
      AUTHOR: app.updated_by || ' ',
      VIEWS: app.report_type === 'apps' ? '-' : (app.views_count || 0),
      DATASET: app.dataset_name,
      DATACONN_NAME: app.dataconn_name,
      DATASET_ID: app.dataset_id,
      DATASET_SHORTNAME: app.dataset_name,
      DELETE_VISIBLE: deletePerm ? 'inline' : 'none',
      EDIT_VISIBLE: isDashboardWithEditPerm || isAppWithEditPerm ?
        'inline' : 'none',
      CSS_COUNT: related.length,
      HTML_APPS: arc.misc.appsToLinks(related, 5, relatedNames),
      HIDE_LAUNCH_BANNER: simplifiedType === 'apps' ? '' : 'arc-invisible',
      VIEW_TARGET: target,
      HTML_DATASET_TAGS: tags.join(''),
      TAGS_VISIBILITY: tags.length === 0 ? 'arc-invisible' : '',
      DESCENDANTS_COUNT: app.descendants_count || 0,
      IS_CLONE: app.is_clone,
    };
  }

  function setupExpansion() {
    $('.group-selection-list li .fa-angle-right, ' +
      '.group-selection-list li .fa-angle-down')
      .off('click')
      .on('click', function () {
        $(this).toggleClass('fa-angle-down fa-angle-right');
        $(this).parent().next().slideToggle();
      });
  }

  apps.show_apps_sidebar = function (_) {
    if (!arguments.length) return show_apps_sidebar;
    show_apps_sidebar = _;
    return apps;
  };

  apps.initialize = function (initiallySelectedIds = '') {
    initialSelections = initiallySelectedIds;
    arc.screen.isAppsScreen = true;

    $('.arc-not-accel').on('click', (e) => {
      const ids = checkLengthOfSelectedAppsVisuals();
      if (!ids.length) {
        arc.report.alertMessageSmall('Please select item(s) below', 'alert-info');
        e.stopPropagation();
      }
    });

    if (arc.frontend.globals.isMobileView) {
      initializeMobile();
      return;
    }

    // make Visuals left side panel resizable
    $sidebarContainer.resizable({
      handles: 'e',
      resize: (event, ui) => arc.frontend.vizCookie.setVisualTabSettingByKey('sideBarStyle', ui.size),
    });

    initializeDataTable();
    initializeTypeSelector();

    const loader = arc.loadbar();
    arc.globals.singleVisualBuilder =
      arc.reportbuilder()
        .loader(loader);

    const queryParam = arc.misc.getQueryParameters();
    // Set group
    if (queryParam.setappgroup && arc.misc.isNumeric(queryParam.setappgroup)) {
      groupActiveId = Number(queryParam.setappgroup);
      arc.screen.apps.groupActiveId(groupActiveId);
    }

    // Apply bar width
    const sidebarStyle = arc.frontend.vizCookie.getVisualTabSettingByKey('sideBarStyle');
    $sidebarContainer.css('width', sidebarStyle?.width || '')
      .removeClass('arc-invisible');

    // Hide workspace create/edit button if user doesn't have access
    arc.misc.updateDivSysPerms($sidebarContainer);

    setupGridAndList();
    setupSidebarOrdering();
    setupAppAccel();
    arc.migration.export();
    arc.migration.export.setup();
    setupAppDelete();
    setupAllSelect();

    // setup searchbox functionality
    _setupSearch('#searchbox');

    // search for terms entered from global search
    if (queryParam.searchbox) {
      const searchterm = queryParam.searchbox;
      $('#searchbox').val(searchterm).trigger('input');
    }

    function markModifiedGroups() {
      $('input.appgroup-data').offOn('change', function () {
        $(this).closest('li').addClass('unsaved');
      });

      $('.appgroup-moveup').offOn('click.moveUp', function () {
        const li = $(this).closest('li');
        li.addClass('unsaved');
        if (li.not(':first-child')) {
          const other = li.prev();
          other.addClass('unsaved');
          other.before(li);
        }
      });

      $('.appgroup-movedown').offOn('click.moveDown', function () {
        const li = $(this).closest('li');
        li.addClass('unsaved');
        if (li.not(':last-child')) {
          const other = li.next();
          other.addClass('unsaved');
          other.after(li);
        }
      });
    }

    markModifiedGroups();

    // make grid app list sortable
    const $grid = $('.app-grid-list');
    $grid.sortable();
    saveOrderChange($grid);

    arcl.appgroupToSidebarMap = function (d) {
      let subgroup_counts = 0;
      (d.subgroups || []).forEach((sd) => {
        subgroup_counts += sd.apps.filter((a) => arcl.apps_by_typeid[`dashboard${a[0]}`]).length;
      });

      const numFiltApps = d.apps.filter((a) => arcl.apps_by_typeid[`dashboard${a[0]}`]).length;

      return {
        APPGROUP_ID: d.group_id,
        TITLE: d.name,
        APPCOUNT: numFiltApps + subgroup_counts,
        HTML_EDIT_ICON: d.rw ? '<span class="edit-appgroup edit-icon"></span>' : '',
        HTML_HAS_SUBGROUPS: (d.subgroups || []).length > 0 ?
          '<a class="fa fa-angle-right app-group-expand"></a>' : '<a class="fa"></a>',
      };
    };

    starredAppgroupId = -10;
    arcl.samples_appgroup_id = -10;

    function convertGroupId(groupid) {
      if (groupid === APPGROUP_STARRED) {
        return starredAppgroupId;
      }

      if (groupid === APPGROUP_SAMPLES) {
        groupid = arcl.samples_appgroup_id;
      }

      return groupid;
    }

    arcl.filterAppsOnGroupSelection = function (groupid, cb) {
      filteredSubgroups = [];
      groupid = convertGroupId(groupid);

      if (groupid === APPGROUP_ALL) {
        filteredReportsByGroup = [...arcl.apps];
      } else if (groupid > 0) {
        const appgroups = arc.globals[selectionType].filter((a) => a[idProperty()] === groupid);
        if (appgroups) {
          if (!appgroups[0].apps) {
            appgroups[0].apps = [];
          }

          // if selected workspace is another user's private workspace, show it
          if (isWorkspace() &&
            isOtherUserPrivate(appgroups[0]) &&
            $privateWorkspaceSidebar.hasClass('arc-invisible')) {
            $privateWorkspaceToggle.trigger('click');
          }

          filteredReportsByGroup = appsToFilteredApps(appgroups[0].apps.slice());
          filteredSubgroups = appgroups[0].subgroups || [];
          sortSubgroups(filteredSubgroups);
          filteredSubgroups.forEach((subgroup) => {
            if (!subgroup.apps) {
              subgroup.apps = [];
            }

            subgroup.display_apps = appsToFilteredApps(subgroup.apps.slice());
          });
        }
      }

      drawDasboardVisualAndAppViewer();
      if (cb) {
        cb();
      }
    };

    function appsToFilteredApps(display_apps) {
      const filtered_apps = [];

      display_apps.forEach((d) => {
        // for appgroups, need to use d[0] because report_id and position is stored,
        // for workspaces, only report_id is stored
        const app_id = $.isArray(d) ? `dashboard${d[0]}` : d;
        if (arcl.apps_by_typeid[app_id]) {
          arcl.apps_by_typeid[app_id].order = d[1];
          filtered_apps.push(arcl.apps_by_typeid[app_id]);
        }
      });

      return filtered_apps;
    }

    setupGroupClicks();
    fetchAndDraw();

    arcl.setup_edit_and_popover = function (display_edit) {
      $('#edit-app-menu-modal .group-selection-list li').remove();
      d4.queue()
        .defer(d3.json, arc.frontend.globals.urls.appgroupUrl)
        .await((error, appgroups) => {
          if (error) {
            return;
          }

          if (display_edit) {
            const $modal = $('#edit-app-menu-modal');
            if (arc.misc.modalShown($modal)) {
              return;
            }

            $modal.modal('show');
          }

          arc.globals.appgroups = appgroups;
          processAppgroups();

          $('#edit-app-menu-modal .group-selection-list li').remove();

          $('#app-group-popover-clone').html($('#app-group-popover').html());
          const $popover_group_selection =
            $('#app-group-popover-clone .group-selection-list');
          const $edit_app_menu_group_selection =
            $('#edit-app-menu-modal .group-selection-list');

          appgroups.forEach((d) => {
            if (d.group_type === 'samples' && !arc.globals.is_superuser) {
              // Only an admin user is permitted to add to samples
              return;
            }

            if (d.group_type === 'starred') {
              return;
            }

            const m = {
              APPGROUP_ID: d.group_id,
              TITLE: d.name,
              MYNAV_CHECKED: (d.in_userlist === 'true') ? 'checked' : '',
              UNSAVED: '',
            };

            if (d.subgroups) {
              m.HAS_SUBGROUPS_HIDDEN = '';
            } else {
              m.HAS_SUBGROUPS_HIDDEN = 'visibility: hidden;';
            }

            if (d.group_type !== 'samples') {
              $edit_app_menu_group_selection.append(
                arc.misc.safe_substitute($('#edit-app-menu-group').html(), m)
              );
            }

            if (d.rw === false) {
              // We are not permitted to update some appgroups
              return;
            }

            $popover_group_selection.append(
              arc.misc.safe_substitute($('#app-group-popover-group').html(), m)
            );
            if (d.subgroups) {
              const subgroup_popover_placeholder =
                $popover_group_selection.find('ul').last();
              const subgroup_editappmenu_placeholder =
                $edit_app_menu_group_selection.find('ul').last();
              let substituted_html;
              d.subgroups.forEach((sd) => {
                const a = {
                  APPGROUP_ID: sd.group_id,
                  TITLE: sd.name,
                  MYNAV_CHECKED: (sd.in_userlist === 'true') ? 'checked' : '',
                  UNSAVED: '',
                };

                substituted_html =
                  arc.misc.safe_substitute($('#app-group-popover-subgroup').html(), a);

                subgroup_popover_placeholder.append(substituted_html);
                subgroup_editappmenu_placeholder.append(substituted_html);
              });
            }
          });

          $('#edit-app-menu-modal input.edit-user-appgroup').on('change', function () {
            $(this).addClass('unsaved');
          });

          setupExpansion();
          markModifiedGroups();
        });
    };

    arcl.setup_edit_menu = function () {
      $('.app-menu-edit').offOn('click.appEdit', () => {
        arcl.setup_edit_and_popover(true);
      });
    };

    $('#edit-app-menu-save').offOn('click.appEditSave', () => {
      // 1. Collect up any new groups to be created
      const newGroups = [];
      $('#edit-app-menu-modal .group-selection-list li input:text').each(function () {
        newGroups.push($(this).val());
      });

      // 2. Create them
      arc.model.appGroupCreate(newGroups, () => {
        // 3. New groups have been collected. Now, collect up any new appgroup
        //    memberships to be saved away
        const um = arc.model.UserAppGroupMembership();
        $('#edit-app-menu-modal li input.unsaved').each(function () {
          um.add_membership($(this).data('appgroup'), $(this).prop('checked'));
        });

        // 4. Save new appgroups in the left side nav
        um.save(() => {
          $('#edit-app-menu-modal').modal('hide');
          fetchAndDraw(true);
          arcl.setup_edit_and_popover();
        });
      });
    });

    arcl.setup_edit_and_popover();
    arcl.setup_edit_menu();

    // allow selecting on table rows and still be able to use right click to
    // open item in a new tab
    $('body').on('mousedown', 'table.table-hover tr td', function (e) {
      clickableRowListener(this, e);
    });

    function clickableRowListener(that, e) {
      const $that = $(that);
      const c = $that.html();
      const $tr = $that.closest('tr');
      const type = $tr.data('type');
      let url = $tr.data('href');
      if (url && !c.match(/<a|<button|<input/)) {
        const target = e.ctrlKey || e.which === 2 || type === 'apps' ? '_blank' : '_self';
        url = target === '_blank' ? arc.misc.getBlankTargetUrl(url) : url;
        $that.html(`<a href="${url}" target="${target}">${c}</a>`);
        const a = $(that).find('a');
        if (e.which === 3) {
          setTimeout(() => {
            a.parent().html(c);
          }, 300);
        } else {
          a[0].click();
          a.parent().html(c);
        }
      }
    }

    return apps;
  };

  const subgroupTemplate = $('#app-grid-subgroup-mobile-template').html();
  const appTemplate = $('#app-grid-mobile-template').html();

  function paintSubgroup(ag, showTitle) {
    const $container = $('#app-container');

    $container.append(arc.misc.safe_substitute(subgroupTemplate, {
      SUBGROUP_TITLE: showTitle ? ag.name : '',
    }));
  }

  function paintApps(applications) {
    const $appList = $('#app-container .app-grid-list').last();
    applications.forEach((a) => {
      // If in mobile, "All" is considered appgroup
      // and can include apps + visuals in addition to dashboards
      const appType = a[2] ? a[2] : 'dashboard';
      const app = arcl.apps_by_typeid[`${appType}${a[0]}`];
      $appList.append(arc.misc.safe_substitute(appTemplate, app_to_map(app)));
    });
  }

  function paintAppgroupMobile(ag, showTitle) {
    if (ag.apps.length === 0) {
      return;
    }

    paintSubgroup(ag, showTitle);
    paintApps(ag.apps);
  }

  function selectMobile(ag) {
    $('#selected-appgroup .title').text(ag.name);

    const $container = $('#app-container');
    $container.empty();

    paintAppgroupMobile(ag);
    if (ag.subgroups) {
      ag.subgroups.forEach((g) => {
        paintAppgroupMobile(g, true);
      });
    }
  }

  function paintAppgroupsMobile() {
    const $appgroupTemplate = $('#app-list-mobile-template').html();

    const $container = $('#app-grid .app-list-header .dropdown-menu');
    $container.empty();

    const appgroupsById = {};

    arc.globals.appgroups.forEach((ag) => {
      appgroupsById[ag.group_id] = ag;

      $container.append(arc.misc.safe_substitute($appgroupTemplate, {
        APPGROUP_ID: ag.group_id,
        TITLE: ag.name,
      }));
    });

    $container.find('a')
      .off('click.appgroup')
      .on('click.appgroup', function () {
        const id = Number($(this).attr('data-appgroupid'));
        arc.frontend.vizCookie.setVisualTabSettingByKey('appgroupid', id);
        selectMobile(appgroupsById[id]);
      });
  }

  function processAppgroupsMobile(appgroups) {
    let starred;
    let samples;

    arc.globals.appgroups = appgroups.filter((ag) => {
      if (ag.group_type === 'starred') {
        starred = ag;
        starred.name = 'My Favorites';
        return false;
      }

      if (ag.group_type === 'samples') {
        samples = ag;
        return false;
      }

      return (ag.in_userlist === 'true');
    });

    if (!show_apps_sidebar) {
      arc.globals.appgroups = [];
    }

    arc.globals.appgroups.unshift({
      group_id: APPGROUP_ALL,
      name: 'All',
      apps: arcl.apps.map((a) => [a.id, 0, _getSimplifiedType(a.report_type)]),
    });

    if (starred) {
      arc.globals.appgroups.unshift(starred);
    }

    if (samples) {
      arc.globals.appgroups.push(samples);
    }
  }

  function initializeMobile() {
    d4.queue()
      .defer(d3.json, arc.frontend.globals.urls.reportUrl)
      .defer(d3.json, arc.frontend.globals.urls.reportUsageUrl)
      .defer(d3.json, arc.frontend.globals.urls.appgroupUrl)
      .defer(d3.json, arc.frontend.globals.urls.workspacesUrl)
      .await((error, reports, reportusage, appgroups, allWorkspaces) => {
        arc.globals.workspaces = arc.workspace.sortWorkspaces(allWorkspaces);

        processReports(reports, reportusage, appgroups);

        processAppgroupsMobile(appgroups);

        paintAppgroupsMobile();

        if (arc.globals.appgroups.length > 0) {
          let groupid = arc.frontend.vizCookie.getVisualTabSettingByKey('appgroupid');
          let pickedAppgroup = arc.globals.appgroups[0];
          if (groupid) {
            if (!Number.isNaN(groupid)) {
              groupid = Number(groupid);
              arc.globals.appgroups.some((ag) => {
                if (ag.group_id === groupid) {
                  pickedAppgroup = ag;
                  return true;
                }

                return false;
              });
            }
          }

          selectMobile(pickedAppgroup);
        }
      });
  }

  return apps;
};
