window.arc = window.arc || {};

(function() {
"use strict";

arc.serverlogs = arc.serverlogs || {};

arc.serverlogs.serverlogs = function() {
  const nologsMsg = 'Click \'Display Log\' to populate';
  var serverlogs = function() { return "arc.serverlogs.serverlogs"; };

  var logresp;
  var $tab;
  var $div;
  var $main;
  var dclist;
  var dcid;

  var ctoken;
  var max_fetch;
  var max_buffer;
  var data_buffer;

  var fetch_seconds;
  var fetch_active;
  var active;

  serverlogs.initialize = function($tabp, $mainp, dclistp) {

    $tab = $tabp;
    $main = $mainp;
    dclist = dclistp || [];

    dcid = undefined;

    max_fetch = 1000000;
    max_buffer = 1000000;
    data_buffer = '';
    ctoken = undefined;
    fetch_seconds = 1;
    fetch_active = false;
    active = false;

    $div = $($tab.attr('href'));

    function setupOptionsModal() {
      const $form = $('#download-bundled-log-form');
      const $downloadBundledModal = $('#download-bundled-log');
      const $dateField = $form.find('.fixed-date');

      $form.find('.has-error').removeClass('has-error');
      $form.find('.form-text').not('.permanent').text('');

      // set up datepicker and populate initial value
      const startMoment = moment(new Date()).subtract(1, 'hour');
      const dateFormat = 'MM/DD/YYYY HH:mm A';
      const defaultStartTime = startMoment.format(dateFormat);
      const opts = {
        singleDatePicker: true,
        timePickerIncrement: 1,
        showDropdowns: true,
        format: dateFormat,
        timePicker: true,
        startDate: defaultStartTime,
      };

      $dateField.daterangepicker(opts);
      $dateField.val(defaultStartTime);

      // populate all default the form values
      const $durationField = $('#log-duration');
      const $nodeField = $('#node-list');
      const $coordinatorOnlyField = $('#coordinator-only');
      const $maxBytesField = $('#max-bytes');
      const defaultMinutes = 60;
      const defaultMaxMegaBytes = 2000;
      $durationField.val(defaultMinutes);
      $maxBytesField.val(defaultMaxMegaBytes);
      $nodeField.val('');
      $coordinatorOnlyField.prop('checked', false);
      $form.find('input[name="log-levels"]').prop('checked', true);
      $form.find('input[name="processes"]').prop('checked', true);

      function showHideCaret(expanded) {
        $form.find('.fa-angle-right').toggleClass('arc-invisible', expanded);
        $form.find('.fa-angle-down').toggleClass('arc-invisible', !expanded);
      }

      const $advancedSettings = $('#advancedSettings');
      $advancedSettings.offOn('hide.bs.collapse', () => showHideCaret(false));
      $advancedSettings.offOn('show.bs.collapse', () => showHideCaret(true));
      if ($advancedSettings.hasClass('in')) {
        $advancedSettings.collapse('toggle');
      }

      function validateAndConfirm(evt) {
        let checkDownloadTimer;
        const $confirmModal = $('#confirm-bundled-log-download');
        evt.preventDefault();
        if (!$form.find('.has-error').length) {
          evt.stopPropagation();

          $('#confirm-bundled-log-download-btn').offOn('click.confirmRequest', requestBundledLogs);
          $confirmModal.modal();
        }

        function requestBundledLogs(e) {
          arc.report.alertMessageSmallHide();
          const $link = $(e.currentTarget);
          $link.prop('disabled', true);

          const logLevels = $form.find('input[name="log-levels"]:checked')
                                 .map((i, el) => $(el).val())
                                 .toArray()
                                 .join(',');
          const processes = $form.find('input[name="processes"]:checked')
                                 .map((i, el) => $(el).val())
                                 .toArray()
                                 .join(',');
          const startTime = moment($dateField.val(), dateFormat).toISOString();
          const downloadCookieName = arc.frontend.vizCookie.getArcCookieKey('downloadCookieToken', true);
          const options = {
            dcName: $div.find('.server-log-conn-options option:selected').text(),
            duration: Number($durationField.val()) * 60,
            nodeList: $nodeField.val(),
            coordinatorOnly: $coordinatorOnlyField.prop('checked'),
            maxBytes: Number($maxBytesField.val()),
            logLevels,
            processes,
            startTime,
            cookie: JSON.stringify({
              name: downloadCookieName,
              value: JSON.stringify(startTime),
            }),
          };
          const url = `${arc.frontend.globals.urls.bundledLogsUrl}?${$.param(options)}`;
          $link.attr('href', url);
          $('#download-spin').removeClass('arc-invisible');

          function stopProgressIndicator() {
            window.clearInterval(checkDownloadTimer);
            $('#download-spin').addClass('arc-invisible');
            $link.prop('disabled', false);
            $confirmModal.modal('hide');
          }

          // Poll to see if download has been returned
          const cookiePattern = new RegExp(`^${startTime} (SUCCESS|ERROR|WARNING):?(.*)`);
          checkDownloadTimer = window.setInterval(function checkDownloaded() {
            const match = cookiePattern.exec(arc.frontend.vizCookie.getJson(downloadCookieName));
            if (match) {
              stopProgressIndicator();
              switch (match[1]) {
                case 'SUCCESS':
                  $downloadBundledModal.modal('hide');
                  break;
                case 'ERROR':
                  arc.report.alertMessageSmall(match[2], 'alert-danger');
                  break;
                case 'WARNING':
                  arc.report.alertMessageSmall(match[2], 'alert-info');
              }
            }
          }, 1000);

        }
      }

      const logRequestValidatorChecks = {
        custom: {
          atleastonechecked($el) {
            const $formGroup = $el.closest('.form-group');
            const checkboxes = $formGroup.find('input[type=checkbox]');
            const atleastOneIsChecked = Array.from(checkboxes)
              .some((checkbox) => checkbox.checked);

            if (!atleastOneIsChecked) {
              return 'You must choose at least one option';
            }
          }
        }
      };

      $form.validator(logRequestValidatorChecks).offOn('submit', validateAndConfirm);

      const $downloadBtn = $('#download-bundled-log-btn');
      $form.offOn('invalid.bs.validator', () => $downloadBtn.prop('disabled', true));
      $form.offOn('valid.bs.validator', () => $downloadBtn.prop('disabled', false));

      $downloadBundledModal.modal();
    }

    if (dclist.length) {
      dcid = dclist[0].id;
      if (dclist.length >= 1) {
        var $form = $div.find('.server-log-conn-select');
        var $connlist = $form.find('.server-log-conn-options');
        dclist.forEach(function (dc) {
          $connlist.append($(`<option value=${dc.id}>${dc.name}</option>`));
        });
        $form.removeClass('arc-invisible');
        $connlist.on('change', function () {
          data_buffer = '';
          ctoken = undefined;
          $div.find(".server-log-container").text(data_buffer);
          updateActive(false);
          dcid = $connlist.val();
          let download_url = arc.frontend.globals.urls.serverLogsUrl;
          if (dcid) {
            download_url += '?dcid=' + dcid;
          }
          $div.find('.server-log-download').attr('href', download_url);
        });
      }

      $div.find('.bundled-log-download').offOn('click.openOptionsModal', setupOptionsModal);
    }

    $tab.off().on('hide.bs.tab', function tabhide(e) {
      updateActive(false);
    });

    $tab.on('shown.bs.tab', function tabshow(e) {
      updateActive(dclist.length < 1);
    });

    $div.find('.server-log-pause').off().on('click', function () {
      updateActive(false);
    });

    $div.find('.server-log-top').off().on('click', function () {
      scrollToTop(0);
      updateActive(false);
    });

    $div.find('.server-log-clear').off().on('click', function () {
      data_buffer = '(Cleared)';
      $div.find(".server-log-container").text(data_buffer);
    });

    $div.find('.server-log-getlog').off().on('click', function () {
      ctoken = undefined;
      updateActive(true);
    });

    $div.find('.server-log-continue').off().on('click', function () {
      updateActive(true);
    });

    let download_url = arc.frontend.globals.urls.serverLogsUrl;
    if (dcid) {
      download_url += '?dcid=' + dcid;
    }
    $div.find('.server-log-download').attr('href', download_url);

    // Here the goal is to stop the continuous display mode if we detect
    // the user is looking at specific data
    $main.bind('mousewheel DOMMouseScroll mousedown', function(e) {
      updateActive(false);
    });

    $div.find('.server-log-container').text(data_buffer || nologsMsg);
    updateActive($tab.closest('li').hasClass('active'));
    return serverlogs;
  };

  function updateActive(state) {
    active = state;
    if (!active) {
      $div.find('.server-log-pause').addClass('arc-invisible');
      $div.find('.server-log-continue').removeClass('arc-invisible');
    } else {
      $div.find('.server-log-pause').removeClass('arc-invisible');
      $div.find('.server-log-continue').addClass('arc-invisible');
      fetchlog();
    }
  }

  // Use a really big pixel count to cause scroll to bottom of window
  function scrollToBottom() {
    $main.find('.server-log-container')
         .scrollTop(1000000);
  }

  function scrollToTop() {
    $main.find('.server-log-container')
         .scrollTop(0);
  }

  function addData(rsp) {

    ctoken = rsp.ctoken;
    var input = rsp.data.replace(/</g, '&lt');

    var trim = (data_buffer.length + input.length) - max_buffer;
    if (trim > 0) {
      var extra = data_buffer.indexOf('\n', trim);
      if (extra !== -1) {
        trim = extra + 1;
      }
      data_buffer = data_buffer.substr(trim);
    }

    data_buffer = data_buffer + input;
    $div.find(".server-log-container").text(data_buffer || nologsMsg);
    $div.find(".view-spinner-container").addClass('arc-invisible');
    scrollToBottom();

    setTimeout(function () {
      if (active) {
        scrollToBottom();
        fetchlog();
      }
    }, 1000 * fetch_seconds);
  }

  function fetchlog() {

    if (fetch_active) {
      return;
    }
    fetch_active = true;

    // On initial fetch, show the spinner
    if (ctoken === undefined) {
      $div.find(".view-spinner-container").removeClass('arc-invisible');

      const $logContainer = $div.find('.server-log-container');
      const top = $logContainer.offset().top;
      const documentHeight = $(document).height();

      // Allow 80 px for buttons at the bottom of the page
      let scrollHeight = documentHeight - top - 80;
      if (scrollHeight < 100) {
        scrollHeight = 100;
      }

      $logContainer.css('overflow', 'scroll')
                   .css('height', scrollHeight + 'px');
    }

    var data = {
      dcid: dcid,
      max_fetch: max_fetch,
      stream: false,
    };
    if (ctoken) {
      data.ctoken = ctoken;
    } else {
      data.startp = max_buffer.toString();
    }

    const url = arc.frontend.globals.urls.serverLogsUrl;
    $.ajax({
      url: url,
      data: data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .always(function() {
      fetch_active = false;
    })
    .fail((rsp, textstatus) => {
      $div.find('.view-spinner-container').addClass('arc-invisible');
      var err = rsp.responseText || textstatus;
      arc.report.alertMessage("Error fetching log: ", err, true);
    })
    .done((rsp) => {
      // Give an indication that we are fetching
      if (rsp.data.length === 0) {
        rsp.data = '.';
      }
      addData(rsp);
    });
  }

  return serverlogs;
};

}());
