import arc from 'arc';
import { ParamEnums, Param } from './paramClass';

/**
 * @property {object} _source = {       - Stores the properties of emitting entity and column
   pivot: {
     sourceShelf,
     customExpr,
     expr,
     fnName,
     fnCast,
   }
 }
 */
export class ColParam extends Param {
  /**
   *
   * @param lib                         - base object corresponding to source visual/filter entity
   * @param paramConfig = {             - object used to initialise Param object
      pivot: {
        sourceShelf,
        customExpr,
        expr,
        fnName,
        fnCast,
      }
   }
   */
  constructor(lib, paramConfig = {}) {
    super(lib, paramConfig);

    this.name = 'ColParam';

    this._source.pivot = {};
    this._meta.paramType = ParamEnums.ParamTypeEnum.PIVOT;
    if (paramConfig.pivot) {
      this._setPivot(paramConfig.pivot);
    }
  }

  get pivot() {
    return arc.misc.extend({}, this._source.pivot);
  }

  _setPivot(pivotObj = {}) {
    this._source.pivot.sourceShelf = pivotObj.sourceShelf;
    this._source.pivot.customExpr = pivotObj.customExpr;
    this._source.pivot.expr = pivotObj.expr;
    this._source.pivot.fnName = pivotObj.fnName;
    this._source.pivot.fnCast = pivotObj.fnCast;
  }

  toOldParamFormat() {
    let oldFormatParam = {};
    if (this.isIgnore) {
      // @assert - 'This scenario should never occur'
      oldFormatParam[`${this._outputParameter}.ignore`] = [true];
      return oldFormatParam;
    }

    oldFormatParam[`${this._outputParameter}.include_nulls`] = [String(this.includeNulls)];

    if (!this.value.length) {
      return oldFormatParam;
    }

    const sourceObj = this.source;
    const pivotObj = this.pivot;

    oldFormatParam = {
      ...oldFormatParam,
      key: this.outputParameter,
      value: this.value,
      filtername: sourceObj.baseField,
      keytype: this.meta.keyType,
      valuetype: this.colDataType,
      isDatasetSegment: sourceObj.isDatasetSegment,
      source_visual_id: sourceObj.entityId,
      source_dataset_id: sourceObj.datasetId,
      source_shelf: pivotObj.sourceShelf,
      custom_expr: pivotObj.customExpr,
      expr: pivotObj.expr,
      fn_name: pivotObj.fnName,
      fn_cast: pivotObj.fnCast,
    };

    return oldFormatParam;
  }
}

/**
 * @property {object} _source = {       - Stores the properties of emitting entity and column
   pivot: {
     sourceShelf,
     customExpr,
     expr,
     fnName,
     fnCast,
   }
 }
 * @property {array} _value = []        - Array of length 1
 */
export class PivotParam extends ColParam {
  constructor(lib, paramConfig = {}) {
    super(lib, paramConfig);

    this.name = 'PivotParam';

    if (paramConfig.colDataType === 'undefined') {
      this._setDataType(ParamEnums.DataTypeEnum.STRING);
    }

    if (!this.outputParameter && lib) {
      this._outputParameter = `${lib.report_id()}-${arc.contextmenu.id += 1}`;
    }
  }

  toOldParamFormat() {
    const oldFormatParam = super.toOldParamFormat();
    if (!this.isIgnore) {
      oldFormatParam.key = `${this.outputParameter}.pivot`;
    }

    return oldFormatParam;
  }
}
