window.arc = window.arc || {};

(function() {
'use strict';

arc.search = arc.search || {};

arc.search.AUTOSUGGEST = {
  REPLACE_WORD: 1,
  REPLACE_ALL: 2,
};

arc.search.visualNames = [
  'area',
  'bar',
  'box plot',
  'packed bubbles',
  'bullet',
  'calendar heatmap',
  'chord',
  'combined visual',
  'correlation heatmap',
  'crosstab',
  'dendrogram',
  'gauge',
  'grouped bars',
  'histogram',
  'kpi',
  'interactive map',
  'map',
  'network',
  'pie',
  'radial',
  'scatter',
  'sparkline',
  'table',
  'treemap',
];

arc.search.autoSuggest = function() {
  const autoSuggest = function() { return 'arc.search.autoSuggest'; };

  let container;
  let $container;
  let $searchBox;
  let $examples;
  let alertContainerId;
  let datasets;
  const datasetIds = new Set();
  let activeDataset;
  let selectedCb;
  let suggestions = [];
  let input;

  const typeToIcon = {
    history: 'fa fa-history',
    expansion: 'fa fa-text-width',
    datasetSuggest: 'fa fa-lightbulb-o',
    favorite: 'fa fa-bookmark',
    column: 'dim-icon-type',
    visualType: 'fa fa-bar-chart',
  };

  autoSuggest.input = function(_) {
    if (!arguments.length) return input;
    input = _;
    return autoSuggest;
  };

  autoSuggest.selectedCb = function(_) {
    if (!arguments.length) return selectedCb;
    selectedCb = _;
    return autoSuggest;
  };

  autoSuggest.container = function(_) {
    if (!arguments.length) return container;
    container = _;

    if (container === '#test-search-modal') {
      alertContainerId = container;
    } else {
      alertContainerId = null;
    }

    $container = $(container);
    $searchBox = $container.find('.pop-search-box');
    $examples = $container.find('.pop-search-examples');

    return autoSuggest;
  };

  autoSuggest.datasets = function(_) {
    if (!arguments.length) return datasets;
    datasets = _;

    datasetIds.clear();
    datasets.forEach(function addToDatasetId(ds) {
      datasetIds.add(ds.dataset_id());
    });

    activeDataset = undefined;
    getSuggestions();
    populateSearchList(false);
    return autoSuggest;
  };

  autoSuggest.activeDataset = function(_) {
    if (!arguments.length) return activeDataset;
    activeDataset = _;
    return autoSuggest;
  };

  autoSuggest.resetActive = function() {
    const $active = $examples.find('li.active');
    $active.removeClass('active');
  };

  autoSuggest.hide = function() {
    $examples.hide();
  };

  function datasetToPhrases(dataset, includeJustOne) {
    const searchTokenizer = arc.grammar._searchtokens;
    const phrases = [];

    const settings = dataset.searchSettings();
    const { expansions } = settings;

    expansions.forEach((e) => {
      phrases.push({
        phrase: e.key,
        iconType: typeToIcon.expansion,
      });
    });

    dataset.columnIterator((c) => {
      if (c.attr('hidden') || c.isComplexType()) {
        return;
      }

      const terms = c.terms();
      if (!terms) {
        return;
      }

      let { tokens } = searchTokenizer.parse(terms, {
        startRule: 'strings',
      });

      if (includeJustOne && tokens.length) {
        tokens = [tokens[0]];
      }

      tokens.forEach((t) => {
        if (t.phrase) {
          const typeClass = c.isGeo() ? 'type-geo' : c.coltype_class();
          phrases.push({
            phrase: t.phrase,
            iconType: `${typeToIcon.column} ${typeClass}`,
          });
        }
      });
    });

    return phrases;
  }

  function getLastWord(enteredText) {
    const words = enteredText.split(/[\s,]+/);
    if (words.length >= 1) {
      return words[words.length - 1].toLowerCase();
    }

    return undefined;
  }

  function getSuggestions(enteredText) {
    suggestions = [];
    let lastWord;

    if (enteredText) {
      enteredText = enteredText.replace(/\s+/g, ' ').toLowerCase();
      lastWord = getLastWord(enteredText);
    }

    function checkSuggestion(allTerms, suggestion, termType, dataset_id = 'all') {
      allTerms.all = allTerms.all || new Set();
      const allTermsSet = allTerms[dataset_id] || new Set();
      let returnValue = false;
      if (allTermsSet.has(suggestion.toLowerCase())) {
        returnValue = false;
      } else if (!enteredText) {
        allTermsSet.add(suggestion.toLowerCase());
        returnValue = true;
      } else if (termType === arc.search.AUTOSUGGEST.REPLACE_ALL) {
        if (suggestion.toLowerCase().indexOf(enteredText) >= 0) {
          allTermsSet.add(suggestion.toLowerCase());
          returnValue = true;
        }
      } else if (!lastWord || suggestion.toLowerCase().indexOf(lastWord) >= 0) {
        allTermsSet.add(suggestion.toLowerCase());
        returnValue = true;
      }

      allTerms[dataset_id] = allTermsSet;
      allTerms.all = new Set([...allTerms.all, ...allTermsSet]);
      return returnValue;
    }

    let autoDatasets;
    if (activeDataset) {
      autoDatasets = [activeDataset];
    } else {
      autoDatasets = datasets;
    }

    const allTerms = {};
    autoDatasets.forEach((dataset) => {
      dataset.searchSettings().suggestions.forEach((suggestion) => {
        const satisfiesInput = checkSuggestion(
          allTerms,
          suggestion.s,
          arc.search.AUTOSUGGEST.REPLACE_ALL,
          dataset.dataset_id()
        );
        if (satisfiesInput) {
          suggestions.push({
            term: suggestion.s,
            termType: arc.search.AUTOSUGGEST.REPLACE_ALL,
            dataset,
            iconType: typeToIcon.datasetSuggest,
          });
        }
      });

      const includeJustOne = !lastWord;
      datasetToPhrases(dataset, includeJustOne).forEach((suggestion) => {
        const satisfiesInput = checkSuggestion(
          allTerms,
          suggestion.phrase,
          arc.search.AUTOSUGGEST.REPLACE_WORD,
          dataset.dataset_id()
        );
        if (satisfiesInput) {
          suggestions.push({
            term: suggestion.phrase,
            termType: arc.search.AUTOSUGGEST.REPLACE_WORD,
            dataset,
            iconType: suggestion.iconType,
          });
        }
      });
    });

    // Only show visual types after at least 1 full word has been typed
    if (enteredText && enteredText.match(/\s+/)) {
      arc.search.visualNames.forEach(function addVisuals(visualName) {
        const addSuggestion = checkSuggestion(allTerms,
                                              visualName,
                                              arc.search.AUTOSUGGEST.REPLACE_WORD);
        if (addSuggestion) {
          suggestions.push({
            term: visualName,
            termType: arc.search.AUTOSUGGEST.REPLACE_WORD,
            iconType: typeToIcon.visualType,
            type: 'Visual Types',
          });
        }
      });
    }

    if (arc.globals.favorites) {
      arc.globals.favorites.forEach(function checkFavorite(favoriteItem) {
        const addSuggestion = checkSuggestion(allTerms,
                                              favoriteItem.search_text,
                                              arc.search.AUTOSUGGEST.REPLACE_ALL);
        if (addSuggestion && datasetIds.has(favoriteItem.dataset_id)) {
          suggestions.push({
            term: favoriteItem.search_text,
            termType: arc.search.AUTOSUGGEST.REPLACE_ALL,
            iconType: typeToIcon.favorite,
            type: 'Bookmarked Queries',
          });
        }
      });
    }

    if (arc.globals.search_history) {
      arc.globals.search_history.forEach(function checkHistory(historyItem) {
        const addSuggestion = checkSuggestion(allTerms,
                                              historyItem.search_text,
                                              arc.search.AUTOSUGGEST.REPLACE_ALL);
        const historicSearchInDataset = historyItem.dataset_ids &&
          historyItem.dataset_ids.some((dsId) => datasetIds.has(dsId));
        const newSearchInDataset = datasetIds.has(historyItem.dataset_id);
        if (addSuggestion && (historicSearchInDataset || newSearchInDataset)) {
          suggestions.push({
            term: historyItem.search_text,
            termType: arc.search.AUTOSUGGEST.REPLACE_ALL,
            iconType: typeToIcon.history,
            type: 'Search History',
          });
        }
      });
    }

    return suggestions;
  }

  autoSuggest.activate = function() {
    autoSuggest.hide();
    getSuggestions();
    populateSearchList(false);
    setupAutosuggestToggle();
    return autoSuggest;
  };

  autoSuggest.enteredText = function(enteredText, show = true) {
    if ((enteredText === '') || (!enteredText)) {
      getSuggestions();
      populateSearchList(show);
      return autoSuggest;
    }

    getSuggestions(enteredText);
    populateSearchList(show, enteredText);
    return autoSuggest;
  };

  function setupAutosuggestToggle() {
    // Clear out entries if 'x' pressed
    $container.offOn('click.removeText', '.pop-search-remove', function removeText() {
      if ($searchBox.text() === '') {
        input.close();
        return;
      }

      $searchBox.empty();
      autoSuggest.enteredText('');
      $searchBox.focus();
    });

    // Close if click elsewhere on page
    $('body').offOn('click.closeSearchToggle', function closeToggle(e) {
      const $el = $(e.target);
      if (!$el.closest('.search-popover').length) {
        $examples.hide();
      }
    });

    $container
        .offOn('click.suggestOption', '.pop-search-wrapper .pop-search-view',
          function triggerSearch() {
            const $suggestions = $examples.find('li');
            if ($suggestions.length === 0) {
              // If empty, try refetching suggestions to at least get history
              const newSuggestions = getSuggestions($searchBox.text());
              if (newSuggestions.length) {
                populateSearchList(true);
              } else {
                arc.report.alertMessageSmall('No suggestions found', 'alert-info', true);
                $examples.hide();
              }
            } else {
              if ($examples.is(':hidden')) {
                $searchBox.focus();
              }

              $examples.toggle();
            }
          });
  }

  autoSuggest.activeSuggestion = function() {
    const $suggestions = $examples.find('li');
    const $active = $suggestions.filter('.active');

    if ($active.length > 0) {
      if ($active.hasClass('trigger-show-all')) {
        $active.trigger('click');
        return 'ShowAll';
      }

      return {
        term: $active.find('.term').text(),
        termType: Number($active.attr('data-type')),
        datasetId: Number($active.attr('data-dataset-id')),
      };
    }

    return undefined;
  };

  function updown(up) {
    let $suggestions;
    if ($examples.hasClass('show-all')) {
      $suggestions = $examples.find('li');
    } else {
      $suggestions = $examples.find('li:not(.extra-items)');
    }

    let idx = 0;
    let $active = $suggestions.filter((currIdx, elem) => {
      if ($(elem).hasClass('active')) {
        idx = currIdx;
        return true;
      }

      return false;
    });

    if ($active.length > 0) {
      $active.removeClass('active');

      if (up) {
        idx -= 1;
        if (idx <= -1) {
          $searchBox.text($searchBox.text());
          arc.misc.moveCursorToEnd($searchBox);
          return;
        }
      } else {
        idx += 1;
        if (idx >= $suggestions.length) {
          $searchBox.text($searchBox.text());
          arc.misc.moveCursorToEnd($searchBox);
          return;
        }
      }
    } else if (up) {
      idx = $suggestions.length - 1;
    } else {
      idx = 0;
    }

    $active = $suggestions.eq(idx);
    $active.addClass('active');

    const searchSuggestion = suggestions[idx];
    if (!searchSuggestion || $active.hasClass('trigger-show-all')) {
      return autoSuggest;
    }

    // Highlight terms that will be replaced in search box
    const searchText = $searchBox.text();
    const $toReplace = $('<span class="highlighted-search-term">');
    if (searchSuggestion.termType === arc.search.AUTOSUGGEST.REPLACE_ALL) {
      $searchBox.empty();
      $toReplace.addClass('arc-replace-all');
      $toReplace.text(searchText);
    } else {
      const highlightedTerm = $active.find('.highlighted-term')[0];
      const replaceTerm = highlightedTerm ? highlightedTerm.textContent : '';
      $toReplace.text(replaceTerm);
      $searchBox.text(searchText.replace(RegExp(`${replaceTerm}$`), ''));
    }

    $searchBox.append($toReplace);
    arc.misc.moveCursorToEnd($searchBox);

    return autoSuggest;
  }

  autoSuggest.up = function() {
    return updown(true);
  };

  autoSuggest.down = function() {
    if (($examples.length > 0) && ($examples.css('display') === 'none')) {
      $examples.show();
      return;
    }

    return updown(false);
  };

  function escapeRegExp(text) {
    return text.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'); // $& means the whole matched string
  }

  function populateSearchList(show, enteredText) {
    const $suggestionsHolder = $examples;
    const $suggestionList = $suggestionsHolder.find('.pop-search-list');
    $suggestionList.empty();

    let highlight_regex_all;
    let highlight_regex_word;
    if (enteredText) {
      const splitWords = enteredText.split(/\s+/);
      const lastIdx = splitWords.length - 1;
      const regexp = splitWords.map((wrd, idx) => {
        let extraSpace = '';
        if (idx !== lastIdx) {
          extraSpace = '\\s+';
        }

        return `(${escapeRegExp(wrd)}${extraSpace})`;
      });

      highlight_regex_all = new RegExp(escapeRegExp(enteredText), 'ig');
      highlight_regex_word = new RegExp(regexp.join('?'), 'ig');
    }

    let lastName;
    let hasMore;
    let numberItem = 0;
    let addSubShowAll;
    let $subList;
    const numberItemsToShow = enteredText ? 5 : 3;
    const maximumToShow = 30;
    const liItemTemplate = $('#search-suggestion-template ul').html();
    const subGroupTemplate = $('#search-suggestion-grouping').html();
    suggestions.forEach((suggestion, i) => {
      const currDataset = suggestion.dataset;
      const datasetName = currDataset ? currDataset.dataset_name() : suggestion.type;
      const isSameDataset = datasetName === lastName;

      const datasetInfo = {
        TERM_TYPE: suggestion.termType,
        DATASET_NAME: datasetName,
        DATASET_ID: currDataset ? currDataset.dataset_id() : '',
      };

      if (isSameDataset) {
        numberItem += 1;
      } else {
        if (addSubShowAll) {
          $subList.append(`<li class="trigger-show-all sub-menu">` +
                          '<a class="link">Show More</a></li>');
        }

        $subList = $(arc.misc.safe_substitute(subGroupTemplate, datasetInfo));
        $suggestionList.append($subList);
        numberItem = 0;
        addSubShowAll = false;
      }

      if (numberItem > maximumToShow) {
        return;
      }

      let term = arc.misc.escapeHTMLTags(suggestion.term);
      if ((suggestion.termType === arc.search.AUTOSUGGEST.REPLACE_ALL) && highlight_regex_all) {
        term = term.replace(
          highlight_regex_all, x => `<span class="highlighted-term">${x}</span>`
        );
      }

      if ((suggestion.termType === arc.search.AUTOSUGGEST.REPLACE_WORD) && highlight_regex_word) {
        term = term.replace(
          highlight_regex_word, x => `<span class="highlighted-term">${x}</span>`
        );
      }

      const isExtra = numberItem >= numberItemsToShow;
      addSubShowAll = isExtra;

      hasMore = hasMore || isExtra;
      const suggestItem = arc.misc.safe_substitute(
                            liItemTemplate,
                            Object.assign({
                              HTML_SUGGESTION: term,
                              INDEX: i,
                              HIDE_EXTRA: isExtra ? 'extra-items' : '',
                              ICON_TYPE: suggestion.iconType,
                            }, datasetInfo)
                          );

      $subList.append(suggestItem);
      lastName = datasetName;
    });

    if (addSubShowAll) {
      $subList.append(`<li class="trigger-show-all sub-menu">` +
                      '<a class="link">Show More</a></li>');
    }

    if (hasMore) {
      const viewMore = `<li class="trigger-show-all">` +
                       '<a class="link">Show All</a></li>';
      $suggestionList.append(viewMore);
    }

    $suggestionsHolder.removeClass('show-all');

    const $suggestions = $suggestionList.find('li');
    if (show && $suggestions.length > 0) {
      $suggestionsHolder.show();
    }

    if (show && $suggestions.length === 0) {
      $suggestionsHolder.hide();
    }

    if (!show) {
      $suggestionsHolder.hide();
    }

    function getButtonText($this, needShow) {
      if ($this.hasClass('sub-menu')) {
        return needShow ? 'Hide' : 'Show More';
      }

      return needShow ? 'Hide All' : 'Show All';
    }

    const $submenuLis = $suggestionsHolder.find('.trigger-show-all.sub-menu');
    $suggestions.off('click.suggest')
                .on('click.suggest', function() {
                  let $this = $(this);
                  const dataIdx = Number($(this).attr('data-idx'));

                  if ($this.hasClass('trigger-show-all')) {
                    const needShow = !$this.hasClass('trigger-hide-all');

                    if (!$this.hasClass('sub-menu')) {
                      $this.find('a').text(getButtonText($this, needShow));
                      $this.toggleClass('trigger-hide-all', needShow);
                      $this = $submenuLis;
                    }

                    $this.closest('div').toggleClass('show-all', needShow);
                    $this.toggleClass('trigger-hide-all', needShow);
                    $this.find('a').text(getButtonText($this, needShow));
                    return;
                  }

                  $suggestionsHolder.hide();
                  selectedCb(suggestions[dataIdx]);
                })
                .off('mouseenter')
                .on('mouseenter', function() {
                  $suggestions.removeClass('active');
                  $(this).addClass('active');
                })
                .off('mouseleave')
                .on('mouseleave', function() {
                  $(this).removeClass('active');
                });
  }

  return autoSuggest;
};
}());
