window.arc = window.arc || {};

(function() {
'use strict';

arc.search = arc.search || {};

arc.search.dashboardWidget = function() {
  const w = function() { return 'arc.search.dashboardWidget'; };

  let report;
  let wid;
  let $wid;
  let gridster;
  let searchCompiler;
  let input;
  let $search;

  const searchHighlight = arc.search.highlight()
                                    .enable(false);

  function searchEnabled(inputReport) {
    const dataset = inputReport.dataset();
    if (!dataset) {
      return false;
    }

    const searchSettings = dataset.searchSettings();
    return (searchSettings.enabled === true);
  }

  w.report = function(_) {
    if (!arguments.length) return report;
    report = _;
    return w;
  };

  w.gridster = function(_) {
    if (!arguments.length) return gridster;
    gridster = _;
    return w;
  };

  w.wid = function(_) {
    if (!arguments.length) return wid;
    wid = _;
    return w;
  };

  function searchCb(v) {
    try {
      report.reportSearchText(v);
      createVisual(v);
    } catch (error) {
      console.log(error);
    }
  }

  let active;

  w.active = function() {
    return active;
  };

  w.deactivate = function() {
    active = false;
    input.deactivate();
  };

  w.activate = function() {
    active = true;
    const dataset = report.dataset();

    if (!searchEnabled(report)) {
      return;
    }

    searchCompiler = arc.search.searchCompiler()
                       .dataset(dataset)
                       .initialize();

    searchHighlight.container(wid)
                   .searchCompiler(searchCompiler);

    $(`${wid} .widget-btns-wrapper .search-app`).show();

    let title = '';
    const viz = report.viz();
    if (report.reportSearchText()) {
      title = report.reportSearchText();
    } else if (viz && viz.title && viz.title()) {
      title = viz.title();
    }

    $('.arc-viz-container').removeClass('active');

    $wid = $(wid);
    $wid.find('.search-popover').remove();
    $wid.prepend($('#search-popover-template').html());
    $wid.find('.arc-viz-container').addClass('active');

    // clear search entries
    $wid.find('.pop-search-clear').offOn('click.clearPopSearch', function() {
      $(this).parent().find('.pop-search-box').empty();
    });

    $search = $wid.find('.pop-search-box');

    $wid.find('.search-popover').dblclick((evt) => {
      evt.preventDefault();
      evt.stopPropagation();
    });

    $search.text(report.reportSearchText() || title);
    searchHighlight.showHighlights(title, dataset);

    $search.off('click')
           .off('blur')
           .on('click', function() {
             gridster.disable().disable_resize();
             $(this).attr('contenteditable', 'true');
             $(this).focus();
           });

    $search.focus();

    // Indicate that the search is active
    $wid.find('.fa-search.search-app')
        .css('opacity', '1');

    input = arc.search.input()
               .autoSuggestContainer(wid)
               .datasets([dataset])
               .$elem($search)
               .searchHighlight(searchHighlight)
               .searchCb(searchCb)
               .deactivateCb(() => {
                 active = false;
                 $wid.find('.search-popover').remove();
                 $wid.find('.fa-search.search-app')
                     .css('opacity', '.4');
               });

    input.setup();
  };

  function createFilters(elements) {
    const filterControls = elements.filter(e => e.type === 'control');

    filterControls.forEach((e) => {
      arc.globals.dashboard.dashboard_filter_controls().createControlFromColumn({
        dataset: report.dataset(),
        column: e.column,
      });
    });
  }

  function createVisual(text) {
    const dataset = report.dataset();

    arc.visual.create()
       .makeNewVisual(text, {
         datasets: [dataset],
         report,
         $searchButtonContainer: $wid,

         // Add in controls as necessary once elements are parsed
         beforedraw_cb(config) {
           createFilters(config.elements);
         },

         // Input element for highlighting
         input,
       });
  }

  return w;
};
}());
