window.arc = window.arc || {};

/*
accept parenthesis
show all
list just relevant visual types in dropdown
toggle order working
is not
year - then autocomplete with date/time
measure after aggregates

fix view after save

as and list visual types
*/

(function() {
'use strict';

arc.search = arc.search || {};

function hideTooltip() {
  $('#tooltip').fadeOut();
}

function makeListOfObj(listOfStrings, isAggregateList = false) {
  const result = listOfStrings.map(function makeObj(value) {
    let newValue = value;
    if (value.toLowerCase) {
      newValue = value.toLowerCase();
    }

    // Now skip multi-word aggregates and add custom value for "count"
    if (isAggregateList && newValue.includes && newValue.includes(' ')) {
      return null;
    }

    if (isAggregateList && newValue === 'count') {
      newValue = 'count distinct';
    }

    return { name: newValue, originalValue: value };
  });

  return result.filter((val) => !!val);
}

function handleDropdown(selector, $menuItems, currHighlight, originalValue) {
  const $lis = $menuItems.find(`${selector} > li`);
  if ($lis.length > 1) {
    $lis.removeClass('arc-invisible');

    if (originalValue) {
      $lis.attr('data-originalvalue', originalValue);
    }

    if (currHighlight && currHighlight.dropdownIdx !== undefined) {
      $lis.eq(currHighlight.dropdownIdx).addClass('arc-invisible');
    }

    $lis.closest('.dropdown-on-hover').removeClass('arc-invisible');
  } else {
    $lis.closest('.dropdown-on-hover').addClass('arc-invisible');
  }
}

function handleInput(selector, $menuItems, value) {
  const $item = $menuItems.filter(selector);
  $item.removeClass('arc-invisible');
  $item.find('input').val(value);
  $item.find('input').attr('data-originalvalue', value);
}

const allVisuals = makeListOfObj(arc.search.visualNames);
const equalitySymbols = [
  '>',
  '>=',
  '<',
  '<=',
];

// Stores search element type's underline class
// and dropdown selector (in search_menus objects)
//  + any additional processsing needed for dropdown (used in cm_show_cb)
const typesData = {
  aggregate: {
    class: 'search-type-meas',
    dropdownSelector: '.search-measures,.search-measures-hover',
    processDropdown($menuItems, highlightElem) {
      if (!highlightElem.element.order) {
        $menuItems.filter('.search-agg-order-desc').removeClass('arc-invisible');
        $menuItems.filter('.search-agg-order-asc').removeClass('arc-invisible');
      }

      handleDropdown('.search-switch-aggs', $menuItems, highlightElem);

      if (!highlightElem.element.column.attr('exprhasagg') &&
        !highlightElem.element.column.attr('recordcount')) {
          handleDropdown('.search-change-aggregates', $menuItems, highlightElem);
      }
    },
  },
  dimension: {
    class: 'search-type-dim',
    dropdownSelector: '.search-dimensions-hover',
    processDropdown($menuItems, highlightElem) {
      if (!highlightElem.element.order) {
        $menuItems.filter('.search-order-hover').removeClass('arc-invisible');
      }

      handleDropdown('.search-switch-dims', $menuItems, highlightElem);
    },
  },
  negation: {
    class: 'search-type-filter',
  },
  filter: {
    class: 'search-type-filter',
    dropdownSelector: '.search-filter',
    processDropdown($menuItems, highlightElem) {
      if (new Set(equalitySymbols).has(highlightElem.element.token.subtype)) {
        $menuItems.filter('.search-order-equality-hover').removeClass('arc-invisible');
      } else if (highlightElem.element.token.subtype !== 'between' &&
        highlightElem.element.expr &&
        !highlightElem.element.expr.match(/\(?not\s+/)) {
          $menuItems.filter('.search-exclude').removeClass('arc-invisible');
      }

      let filterValues = highlightElem.element.values && highlightElem.element.values.join(', ');
      if (!filterValues && highlightElem.element.token && highlightElem.element.token.phrase) {
        filterValues = highlightElem.element.token.phrase;
      }

      if (['string', 'sstring'].includes(highlightElem.element.token.rule)) {
        filterValues = `"${filterValues}"`;
      }

      filterValues = filterValues.replaceAll("''", "'");

      handleInput('.search-filter', $menuItems, filterValues);
    },
  },
  control: {
    class: 'search-type-control',
  },
  top: {
    class: 'search-type-top',
    dropdownSelector: '.search-order-toggle',
    processDropdown($menuItems, highlightElem) {
      handleInput('.search-limit', $menuItems, highlightElem.element.k);
    },
  },
  bottom: {
    class: 'search-type-bottom',
    dropdownSelector: '.search-order-toggle',
    processDropdown($menuItems, highlightElem) {
      handleInput('.search-limit', $menuItems, highlightElem.element.k);
    },
  },
  distinct: {
    class: 'search-type-expansion',
  },
  order: {
    class: 'search-type-expansion',
    dropdownSelector: '.search-order-toggle',
  },
  expansion: {
    class: 'search-type-expansion',
  },
  trend: {
    class: 'search-type-expansion',
  },
  each: {
    class: 'search-type-expansion',
  },
  'visual type': {
    class: 'search-type-expansion',
    dropdownSelector: '.search-visual-type-hover',
    processDropdown($menuItems, highlightElem) {
      handleDropdown('.search-visual-type', $menuItems, highlightElem.element.phrase);
    },
  },
  unknown: {
    class: 'search-type-unknown',
    processDropdown($menuItems, highlightElem) {
      handleInput('.arc-unknown', $menuItems, highlightElem.element.phrase);
    },
  },
};

// Uses typesData to return back class, dropdownSelector, or processDropdown
// for specific search element type
function getTypeData(t, dataType) {
  let result;
  if (typesData[t]) {
    result = typesData[t][dataType];
  } else if (t.slice(0, 3) === 'top') {
    result = typesData.top[dataType];
  } else if (t.slice(0, 6) === 'bottom') {
    result = typesData.bottom[dataType];
  } else {
    result = typesData.unknown[dataType];
  }

  return result || '';
}

function elementsToHighlights(elements, addedHighlights) {
  addedHighlights = addedHighlights || [];

  const elementsWithTokens = elements.filter(element => (element.token !== undefined));

  let highlights = [];

  addedHighlights.forEach((highlight) => {
    highlights.push({
      type: highlight.type,
      startIndex: highlight.startIndex,
      endIndex: highlight.endIndex,
      tooltip: highlight.highlightNote,
    });
  });

  elementsWithTokens.forEach((element) => {
    let { token, type } = element;

    if ((type === 'top') || (type === 'bottom')) {
      type = `${type} ${element.k}`;
    }

    highlights.push({
      type,
      startIndex: token.startIndex,
      endIndex: token.endIndex,
      element,
    });

    if (element.additionalElements) {
      element.additionalElements.forEach((e) => {
        if (e.token === undefined) {
          return;
        }

        let elemType = e.type;
        if ((elemType === 'top') || (elemType === 'bottom')) {
          elemType = `${elemType} ${e.k}`;
        }

        highlights.push({
          type: elemType,
          startIndex: e.token.startIndex,
          endIndex: e.token.endIndex,
          element: e,
        });
      });
    }
  });

  highlights = highlights.filter(h => (h.startIndex !== undefined));

  highlights.sort((e1, e2) => {
    const s1 = e1.startIndex;
    const s2 = e2.startIndex;

    if (s1 < s2) {
      return -1;
    }

    if (s1 > s2) {
      return 1;
    }

    return 0;
  });

  // Remove overlaps
  let s = -1;
  highlights = highlights.filter((h) => {
    if (h.startIndex <= s) {
      return false;
    }

    s = h.endIndex;
    return true;
  });

  return highlights;
}

// Support for testing
arc.testutil = arc.testutil || {};
arc.testutil.elementsToHighlights = elementsToHighlights;

arc.search.stylizeSearchElement = function($search, text, config) {
  config = config || {};

  let {
    elements,
    highlights,
    showUnknowns,

    // used to redraw the visual when title updated
    // Added in places that call:
    //    arc.search.stylizeSearchElement
    //    searchHighlight.displaySearchElements
    redrawTextCb,
  } = config;

  highlights = elementsToHighlights(elements, highlights);

  if (!showUnknowns) {
    highlights = highlights.filter(h => (h.type !== 'unknown'));
  }

  const outText = [];

  const dropdownCols = {
    aggregate: [],
    dimension: [],
  };

  let connectionType;
  let sortItems = {
    order: [],
    limit: undefined,
  };
  let aggregateFunctions = makeListOfObj([
    'avg',
    'max',
    'min',
    'sum',
  ]);
  let s = 0;
  highlights.forEach((h, i) => {
    outText.push(text.slice(s, h.startIndex));

    const t = text.slice(h.startIndex, h.endIndex);
    const c = getTypeData(h.type, 'class');
    if (c) {
      outText.push(
        `<span class="search-type ${c}" data-search="${i}">
          ${arc.misc.escapeHTML(t)}
        </span>`
      );
    } else {
      outText.push(t);
    }

    if (dropdownCols[h.type]) {
      h.name = t;
      h.totalIdx = i;
      h.dropdownIdx = dropdownCols[h.type].length;
      dropdownCols[h.type].push(h);
    }

    if (h.type === 'top' || h.type === 'bottom') {
      sortItems.sortItems = i;
    } else if (h.type === 'order') {
      sortItems.order.push(i);
    }

    // Record count doesn't have table
    if (!connectionType && h.element && h.element.column && h.element.column.table()) {
      connectionType = h.element.column.table().dataset().dataconnection_type();
      const connMode = arc.utils.modeForConnectionType(connectionType);
      const connAggs = arc.report.supported_connection_aggregates[connMode];
      if (connAggs) {
        aggregateFunctions = makeListOfObj(Object.keys(connAggs), true);
      }
    }

    s = h.endIndex;
  });

  outText.push(text.slice(s, text.length));
  const o = outText.join('');
  $search.html(o);

  const sel = d3.select($search[0])
                .selectAll('.search-type');

  const marks =
      arc.marks()
         .addSelection(sel)
         .text_fn((d, i) => {
            const highlight = highlights[i];
            const { type } = highlight;

            const kvs = [
              {
                key: 'Type',
                value: type,
              },
            ];

            const { element } = highlight;

            if (highlight.type === 'expansion') {
              kvs.push({
                key: 'Expansion',
                value: highlight.tooltip,
              });
            } else {
              if (element && element.expr) {
                kvs.push({
                  key: 'Expression',
                  value: element.displayExpr || element.expr,
                });
              } else if (element && element.vizType) {
                const vizExt = arc.visual.getExtension(element.vizType);
                const vizName = vizExt ? vizExt.viz_tooltip() : element.vizType;
                const failedNote = element.matchingFound ?
                                   '' : `(Specified visual type not available for query)`;
                kvs.push({
                  key: 'Name',
                  value: `${vizName} ${failedNote}`,
                });
              }

              let tooltip;
              if (element && element.tooltip) {
                tooltip = element.tooltip;
              }

              if (!tooltip) {
                if (element.token && element.token.highlightNote) {
                  tooltip = element.token.highlightNote;
                }
              }

              if (tooltip) {
                kvs.push({
                  key: 'Notes',
                  value: tooltip,
                });
              }
            }

            return arc.misc.tooltipFromKVs(undefined, kvs);
          });

  if (redrawTextCb) {
    marks.search_menus([

       // Always include delete option
       {
         elementType: 'button',
         elementText: 'DELETE',
         elementClass: 'search-delete cm-linebelow',
         clickAction: 'DELETE',
       },

      // Aggregate dropdown
      {
        elementType: 'dropdownSelect',
        dropdownCols: aggregateFunctions,
        buttonText: 'Change Aggregate',
        buttonClass: 'search-measures-agg',
        dropdownClass: 'search-change-aggregates',
        clickAction: 'REPLACE',
      },
      {
        elementType: 'dropdownSelect',
        dropdownCols: dropdownCols.aggregate,
        buttonText: 'Switch Measures',
        buttonClass: 'search-measures',
        dropdownClass: 'search-switch-aggs',
        clickAction: 'SWITCH',
      },
      {
        elementType: 'input',
        elementName: 'search-agg-desc-value',
        elementClass: 'search-agg-order-desc',
        elementText: 'Top',
        inputType: 'number',
        clickAction: 'PREPEND',
      },
      {
        elementType: 'input',
        elementName: 'search-agg-asc-value',
        elementClass: 'search-agg-order-asc',
        elementText: 'Bottom',
        inputType: 'number',
        clickAction: 'PREPEND',
      },

      // Dimension dropdown
      {
        elementType: 'dropdownSelect',
        dropdownCols: makeListOfObj([
          'ascending',
          'descending',
        ]),
        buttonText: 'Apply Sort Order',
        buttonClass: 'search-order',
        dropdownClass: 'search-dim-orders',
        clickAction: 'PREPEND',
      },
      {
        elementType: 'dropdownSelect',
        dropdownCols: dropdownCols.dimension,
        buttonText: 'Switch Dimensions',
        buttonClass: 'search-dimensions',
        dropdownClass: 'search-switch-dims',
        clickAction: 'SWITCH',
      },

      // Filter dropdowns
      {
        elementType: 'input',
        elementName: 'search-filter-value',
        elementClass: 'search-filter',
        elementText: 'Modify Filter',
        clickAction: 'REPLACE',
      },
      {
        elementType: 'button',
        elementText: 'Exclude',
        elementClass: 'search-exclude',
        clickAction: 'PREPEND',
      },
      {
        elementType: 'dropdownSelect',
        dropdownCols: makeListOfObj(equalitySymbols),
        buttonText: 'Change Comparison',
        buttonClass: 'search-order-equality',
        dropdownClass: 'search-order-symbols',
        clickAction: 'REPLACE',
      },

      // Order dropdowns
      {
        elementType: 'input',
        elementName: 'search-limit-value',
        elementClass: 'search-limit',
        elementText: 'Limit',
        inputType: 'number',
        clickAction: 'REPLACE',
      },
      {
        elementType: 'button',
        elementText: 'Toggle Order',
        elementClass: 'search-order-toggle',
        clickAction: 'REPLACE',
      },

      // Visual type dropdown
      {
        elementType: 'dropdownSelect',
        dropdownCols: allVisuals,
        buttonText: 'Change Visual Type',
        buttonClass: 'search-visual-type',
        dropdownClass: 'visual-types',
        clickAction: 'REPLACE',
      },

      // Allow user to edit unknown values
      {
        elementType: 'input',
        elementText: 'Modify Unknown Value',
        elementName: 'arc-unknown-value',
        elementClass: 'arc-unknown',
        clickAction: 'REPLACE',
      },
    ])
    .cm_show_cb(function processAndShowCM(contextmenu, active_paramsl, elem, menu_elem) {
      const $menuItems = $(menu_elem).find('.contextmenu-foot > div');
      $menuItems.find('.dropdown-menu');
      const currHighlight = highlights[elem.dataset.search];
      const { type } = currHighlight;

      const hasCoupleHighlights = highlights.filter(h => !!h).length > 1;
      const deleteSelector = hasCoupleHighlights ? '.search-delete' : '';
      const dropdownSelector = getTypeData(type, 'dropdownSelector');
      const connector = (deleteSelector && dropdownSelector) ? ',' : '';

      $menuItems.addClass('arc-invisible');
      $menuItems.filter(`${deleteSelector}${connector}${dropdownSelector}`)
                .removeClass('arc-invisible');

      const processDropdown = getTypeData(type, 'processDropdown');
      if (processDropdown) {
        processDropdown($menuItems, currHighlight);
      }
    })
    .search_select(function updateTitleAndHighlights(menuItem, data, highlightElem) {
      // Remove arc-invisible from .redo-title-search
      const $header = $(highlightElem.closest('.search-result-header'));
      const idx = highlightElem.dataset.search;
      const currHighlight = highlights[idx];
      function getHeaderText() {
        return $header.find('h4')
                      .text().replace(/\s+/g, ' ')
                      .trim();
      }

      $header.find('.redo-title-search')
         .removeClass('arc-invisible');

      // Either delete, add, replace, or switch out text in title and redraw visual
      switch (menuItem.clickAction) {
        case 'DELETE':
          highlightElem.remove();
          break;
        case 'SWITCH':
          highlightElem.textContent = data.name;
          $(highlightElem)
            .closest('h4')
            .find('.search-type')
            .eq(data.totalIdx)
            .text(currHighlight.name);
          break;
        case 'PREPEND':
          const elemName = menuItem.elementName;
          let text = highlightElem.textContent;

          if (['search-agg-desc-value', 'search-agg-asc-value'].includes(elemName)) {
            text = ` ${menuItem.elementText} ${data.name} `;
          } else if (menuItem.elementClass === 'search-exclude') {
            text = ' not ';
          } else if (data && data.name) {
            text = data.name;
          }

          highlightElem.prepend(arc.misc.escapeHTML(text));

          break;
        case 'REPLACE':
          if (menuItem.dropdownClass === 'search-change-aggregates') {
            highlightElem.textContent = `${data.name} ${currHighlight.element.phrase}`;
          } else if (menuItem.elementClass === 'search-order-toggle' &&
            highlightElem.textContent.match(/\basc/i)) {
              highlightElem.textContent = 'descending';
          } else if (menuItem.elementClass === 'search-order-toggle' &&
            highlightElem.textContent.match(/\bdesc/i)) {
              highlightElem.textContent = 'ascending';
          } else if (menuItem.elementClass === 'search-order-toggle' &&
            highlightElem.textContent.match(/\btop\b/i)) {
              highlightElem.textContent = 'bottom';
          } else if (menuItem.elementClass === 'search-order-toggle' &&
            highlightElem.textContent.match(/\bbottom\b/i)) {
              highlightElem.textContent = 'top';
          } else if (menuItem.dropdownClass === 'search-order-symbols') {
            highlightElem.textContent =
              highlightElem.textContent.replace(new RegExp(`\\s+[>|<]=?\\s+`), ` ${data.name} `);
          } else if (menuItem.elementClass === 'search-filter') {
            const isQuotedString = data.name.match(/^\s*['"].+?['"]\s*$/);
            let escapedFilter = isQuotedString ? data.name.slice(1, -1) : data.name;
            const escapes = isQuotedString ? '\\' : '';
            escapedFilter = escapedFilter.replaceAll('"', `${escapes}"`);
            escapedFilter = escapedFilter.replaceAll("'", `${escapes}'${escapes}'`);
            highlightElem.textContent = isQuotedString ? `"${escapedFilter}"` : escapedFilter;
          } else if (data && highlightElem.textContent.includes(data.originalValue)) {
            highlightElem.textContent =
              highlightElem.textContent.replace(data.originalValue, data.name);
          } else if (data &&
            highlightElem.textContent.includes(highlightElem.dataset.originalValue)) {
              highlightElem.textContent =
                highlightElem.textContent.replace(highlightElem.dataset.originalValue, data.name);
          } else if (data) {
            highlightElem.textContent = data.name;
          }

          break;
        default:
          break;
      }

      redrawTextCb(getHeaderText());
    })
    .marker_name('searchMenu');
  }

  marks.activate();

  return marks;
};

arc.search.highlight = function() {
  const searchHighlight = function() { return 'arc.search.highlight'; };

  let searchCompiler;
  let container;
  let marks;
  let enable = true;

  searchHighlight.enable = function(_) {
    if (!arguments.length) return enable;
    enable = _;
    return searchHighlight;
  };

  searchHighlight.container = function(_) {
    if (!arguments.length) return container;
    container = _;
    return searchHighlight;
  };

  searchHighlight.searchCompiler = function(_) {
    if (!arguments.length) return searchCompiler;
    searchCompiler = _;
    return searchHighlight;
  };

  searchHighlight.showHighlights = function(text, dataset) {
    searchCompiler.dataset(dataset)
                  .initialize();

    // First, highlight anything that is known without a search
    // This should be very quick
    searchCompiler.processSearchString(text, {
      finalCb(config) {
        config.redrawTextCb = (newText) => {
          searchHighlight.displaySearchElements(newText, config);
        };

        config.showUnknowns = false;
        searchHighlight.displaySearchElements(text, config);
      },
    });

    // Then, highlight searches as well
    searchCompiler.processSearchString(text, {
      validateFilters: true,
      finalCb(config) {
        config.redrawTextCb = (newText) => {
          searchHighlight.displaySearchElements(newText, config);
        };

        searchHighlight.displaySearchElements(text, config);
      },
    });

    return searchHighlight;
  };

  searchHighlight.removeHighlights = function() {
    const $search = $(container).find('.pop-search-box');
    $search.find('span').removeClass('search-type');

    // if (marks) {
    //   marks.tooltips().disable();
    // }

    // remove any tooltips
    hideTooltip();
    return searchHighlight;
  };

  searchHighlight.displaySearchElements = function(text, config) {
    config = config || {};

    if (!enable) {
      return searchHighlight;
    }

    if (getSearchText() !== text) {
      return searchHighlight;
    }

    if (marks) {
      marks.destroy();
      marks.tooltips().disable();
    }

    const $search = $(container).find('.pop-search-box');

    marks = arc.search.stylizeSearchElement($search, text, config);
    return searchHighlight;
  };

  searchHighlight.destroy = function() {
    marks.destroy();
  };

  function getSearchText() {
    const $search = $(container).find('.pop-search-box');
    const v = $search.text();
    return v;
  }

  return searchHighlight;
};
}());
