window.arc = window.arc || {};

(function() {
'use strict';

arc.search = arc.search || {};

function validKeystroke(keycode) {
  const valid =
      keycode === 8 || // backspace
      (keycode > 47 && keycode < 58) || // number keys
      keycode === 32 || // spacebar
      (keycode > 64 && keycode < 91) || // letter keys
      (keycode > 95 && keycode < 112) || // numpad keys
      (keycode > 185 && keycode < 193) || // ;=,-./` (in order)
      (keycode > 218 && keycode < 223); // [\]' (in order)

   return valid;
}

function hideTooltip() {
  $('#tooltip').fadeOut();
}

arc.search.input = function() {
  const input = function() { return 'arc.search.input'; };

  let $elem;
  let autoSuggestContainer;
  let activeDataset;
  let datasets;
  let searchHighlight;
  let searchCb;
  let keystrokeCb;
  let autoHighlights = true;
  let deactivateCb;

  const autoSuggest = arc.search.autoSuggest();
  autoSuggest.input(input);

  input.close = function() {
    input.deactivate();
    autoSuggest.hide();
  };

  input.deactivate = function() {
    if (deactivateCb) {
      deactivateCb();
    }
  };

  input.deactivateCb = function(_) {
    if (!arguments.length) return deactivateCb;
    deactivateCb = _;
    return input;
  };

  input.autoHighlights = function(_) {
    if (!arguments.length) return autoHighlights;
    autoHighlights = _;
    return input;
  };

  input.$elem = function(_) {
    if (!arguments.length) return $elem;
    $elem = _;
    return input;
  };

  input.autoSuggestContainer = function(_) {
    if (!arguments.length) return autoSuggestContainer;
    autoSuggestContainer = _;
    autoSuggest.container(autoSuggestContainer);
    return input;
  };

  input.datasets = function(_) {
    if (!arguments.length) return datasets;
    datasets = _;
    autoSuggest.datasets(datasets);
    return input;
  };

  input.activeDataset = function(_) {
    if (!arguments.length) return activeDataset;
    activeDataset = _;
    autoSuggest.activeDataset(activeDataset);
    return input;
  };

  input.searchHighlight = function(_) {
    if (!arguments.length) return searchHighlight;
    searchHighlight = _;
    return input;
  };

  input.searchCb = function(_) {
    if (!arguments.length) return searchCb;
    searchCb = _;
    return input;
  };

  input.keystrokeCb = function(_) {
    if (!arguments.length) return keystrokeCb;
    keystrokeCb = _;
    return input;
  };

  input.showHighlights = function(text, dataset) {
    searchHighlight.showHighlights(text, dataset);
    return input;
  };

  input.text = function() {
    return $elem.text();
  };

  input.runSearch = function(text) {
    autoSuggest.hide();
    $elem.text(text);
    if (autoHighlights) {
      if (datasets.length === 1) {
        searchHighlight.showHighlights(text, datasets[0]);
      } else if (activeDataset) {
        searchHighlight.showHighlights(text, activeDataset);
      }
    }

    searchCb(text, activeDataset);
  };

  function setupDeactivationOnDocumentClick() {
    $(document)
        .off('click.search')
        .on('click.search', (e) => {
          // Check if click was triggered on or within this widget
          if ($(e.target).closest(autoSuggestContainer).length > 0) {
            return false;
          }

          $(document).off('click.search');
          input.deactivate();
        });
  }

  function activateDataset(datasetId) {
    datasets.some((d) => {
      if (d.dataset_id() === datasetId) {
        input.activeDataset(d);
      }

      return false;
    });
  }

  function deactivateDataset() {
    input.activeDataset(undefined);
  }

  function activeSuggestionToTerm(activeSuggestion, v) {
    activateDataset(activeSuggestion.datasetId);
    if (activeSuggestion.termType === arc.search.AUTOSUGGEST.REPLACE_ALL) {
      return activeSuggestion.term;
    }

    // Handle replacements after new spaces
    if (v.match(/\s$/)) {
      return `${v}${activeSuggestion.term} `;
    }

    let i = 0;

    if (v.length > 0) {
      i = v.lastIndexOf(' ');
      if (i < 0) {
        i = 0;
      }
    }

    // Handle multiword replacement
    const splitTerm = activeSuggestion.term.split(/\s+/);
    if (splitTerm.length > 1) {
      const regexpEachWord = splitTerm.map(el => `(\\b${el}\\b\\s*)?`);
      regexpEachWord.pop();
      const regexpPrevious = new RegExp(`\\s*${regexpEachWord.join('')}$`, 'i');
      i = v.substring(0, i).search(regexpPrevious);
    }

    return v.substring(0, i > 0 ? i + 1 : i) + activeSuggestion.term;
  }

  input.setup = function() {
    // Fetch favorites if it doesn't exist
    let waitToRedraw = !arc.globals.favorites && !arc.globals.search_history;
    const refetchSuggestions = () => {
      if (waitToRedraw) {
        waitToRedraw = false;
      } else {
        // To always show drop down of search suggestions on new viz,
        // send a 2nd parameter = false in the function below
        autoSuggest.enteredText($elem.text());
      }
    };

    if (!arc.globals.favorites) {
      arc.search.displayedQuery().favorite(true).fetch(refetchSuggestions);
    }

    // fetch search history if it doesn't exist
    if (!arc.globals.search_history) {
      arc.search.displayedQuery().fetch(refetchSuggestions);
    }

    setupDeactivationOnDocumentClick();

    autoSuggest
        .container(autoSuggestContainer)
        .datasets(datasets)
        .selectedCb((activeSuggestion) => {
          let v = $elem.text();
          v = activeSuggestionToTerm(activeSuggestion, v);
          activateDataset(activeSuggestion.datasetId);
          input.runSearch(v);
          arc.misc.moveCursorToEnd($elem);
        })
        .activate();

    $elem
        .off('keyup')
        .on('keyup', function(e) {
          const keycode = e.keyCode;

          if (validKeystroke(keycode)) {
            const v = $(this).text();
            autoSuggest.enteredText(v);

            if (v.length <= 1) {
              deactivateDataset();
            }

            if (keystrokeCb) {
              keystrokeCb(v);
            }
          }
        })
        .off('keydown')
        .on('keydown', function(e) {
          hideTooltip();

          const $this = $(this);
          let v = $this.text();

          const keycode = e.keyCode;

          if (keycode === 27) {
            // Escape key pressed - call the deactivation cb
            input.close();
            return false;
          }

          if (keycode === 38) {
            // up arrow
            autoSuggest.up();
            return false;
          }

          if (keycode === 40) {
            // down arrow
            autoSuggest.down();
            return false;
          }

          if (keycode === 9) {
            // tab places the autosuggest text in the input and then
            // stays on the input
            const activeSuggestion = autoSuggest.activeSuggestion();
            if (activeSuggestion) {
              $this.text(activeSuggestionToTerm(activeSuggestion, v));
              arc.misc.moveCursorToEnd($elem);
              e.preventDefault();
            }

            return;
          }

          if (keycode === 13) {
            try {
              // enter key - get the autosuggest text if it's active
              // and then run the query
              const activeSuggestion = autoSuggest.activeSuggestion();
              if (activeSuggestion && activeSuggestion === 'ShowAll') {
                e.preventDefault();
                return;
              }

              if (activeSuggestion) {
                v = activeSuggestionToTerm(activeSuggestion, v);
                $this.text(v);
              }

              if (v.trim().length === 0) {
                return false;
              }

              autoSuggest.hide();
              if (autoHighlights) {
                if (datasets.length === 1) {
                  searchHighlight.showHighlights(v, datasets[0]);
                } else if (activeDataset) {
                  searchHighlight.showHighlights(v, activeDataset);
                }
              }

              arc.misc.moveCursorToEnd($elem);
              searchCb(v);
            } catch (err) {
              console.log('error', err);
            }

            return false;
          }

          if (validKeystroke(keycode)) {
            searchHighlight.removeHighlights();
          }
        });

    // Setup audio search
    arc.misc.setupSpeechDetection(
      $(`${autoSuggestContainer} .pop-search-wrapper`),
      (finalText) => {
        finalText = (finalText || '').trim();
        const searchAtEnd = finalText.startsWith('search ');

        if (searchAtEnd) {
          input.runSearch(finalText.replace(/^search\s+/i, ''));
        } else {
          $elem.text(finalText);
          autoSuggest.enteredText(finalText);
        }

        arc.misc.moveCursorToEnd($elem);
      },
    );

    return input;
  };

  return input;
};
}());
