window.arc = window.arc || {};

(function() {
'use strict';

arc.search = arc.search || {};

function hideTooltip() {
  $('#tooltip').fadeOut();
}

function datasetSel() {
  const sel = function() { return 'datasetSel'; };

  let inputWrapper;
  const $popDatasetsWrapper = $('.pop-datasets-wrapper');
  const $datasetSearchView = $popDatasetsWrapper.find('.datasets-search-view');
  const $datasetSearchBox = $('.datasets-search-box');
  const $dropdown = $('.dataset-selector-dropdown');
  const $dropdownForm = $dropdown.find('form');

  sel.inputWrapper = function(_) {
    if (!arguments.length) return inputWrapper;
    inputWrapper = _;
    return sel;
  };

  function changeAndShowSelection(allSearchDatasets, datasetsHash) {
    const $unselected = $dropdownForm.find('div.dataset-sel i.fa-circle-thin');
    const $selected = $dropdownForm.find('div.dataset-sel i.fa-check-circle');

    if (($unselected.length === 0) || ($selected.length === 0)) {
      $datasetSearchBox.text(`All (${allSearchDatasets.length})`);
      inputWrapper.datasets(allSearchDatasets);
      return;
    }

     if ($selected.length === 1) {
       const $elem = $selected.closest('.form-check');
       if ($elem.attr('data-id') === '0') {
         $datasetSearchBox.text('All');
         inputWrapper.datasets(allSearchDatasets);
         return;
       }

       const dataset_name = $elem.attr('data-name');
       $datasetSearchBox.text(dataset_name);
       $datasetSearchBox.attr('title', dataset_name);
     }

     if ($selected.length > 1) {
       $datasetSearchBox.text(`${$selected.length} selected`);
     }

     const datasets = [];
     $selected.each(function addSelectedValue() {
       const $this = $(this);
       const datasetId = $this.closest('.form-check').attr('data-id');
       datasets.push(datasetsHash[Number(datasetId)]);
     });

     inputWrapper.datasets(datasets);
     arc.topnav.setCurrentDatasets(datasets);
  }

  function displaySelectedDatasets() {
    const datasets = inputWrapper.datasets();
    if (datasets.length === 1) {
      $datasetSearchBox.text(datasets[0].dataset_name());
      $datasetSearchBox.attr('title', datasets[0].dataset_name());
      return;
    }

    if (datasets.length === 0) {
      $datasetSearchBox.text('No datasets available');
      return;
    }

    $datasetSearchBox.text(`${datasets.length} selected`);
  }

  function datasetToSearchGroup(dataset) {
    return dataset && (dataset.searchSettings().group || '');
  }

  function sortByGroupAndName(datasets) {
    return datasets.sort(function sortDatasetName(a, b) {
      const aGroup = datasetToSearchGroup(a);
      const bGroup = datasetToSearchGroup(b);

      if (aGroup < bGroup) {
        if (aGroup === '') {
          return 1;
        }

        return -1;
      }

      if (aGroup > bGroup) {
        if (bGroup === '') {
          return -1;
        }

        return 1;
      }

      if (a.dataset_name() < b.dataset_name()) {
        return -1;
      }

      return 1;
    });
  }

  sel.setupDropdown = function() {
    displaySelectedDatasets();

    const datasets = inputWrapper.datasets();
    const selectedDatasets = new Set();
    datasets.forEach((d) => {
      selectedDatasets.add(d.dataset_id());
    });

    let allSearchDatasets = [];
    const datasetsHash = {};

    const allTemplate = `
        <div class="form-check all-sel">
          <i class="fa %SELECTED_ICON% selected"></i>
          <label class="form-check-label">All</label>
        </div>`;

    const groupTemplate = `
        <div class="form-check group-sel" data-group="%GROUP_NAME%">
          <i class="fa %SELECTED_ICON% selected"></i>
          <label class="form-check-label">%GROUP_NAME%</label>
        </div>`;

    const datasetTemplate = `
        <div class="form-check dataset-sel" data-group="%GROUP_NAME%"
             data-name="%DATASET_NAME%" data-id="%DATASET_ID%">
          <i class="fa %SELECTED_ICON% selected"></i>
          <i title="Timed out on last search" class="fa fa-exclamation-circle timeout arc-invisible"></i>
          <label class="form-check-label">%DATASET_NAME%</label>
        </div>`;

    arc.search.utils.fetchAllDatasets(
      function addSearchItems(fetchedDs) {
        allSearchDatasets = sortByGroupAndName(fetchedDs);

        let allSelected = true;
        allSearchDatasets.forEach(function processDatasets(d) {
          datasetsHash[d.dataset_id()] = d;
          if (!selectedDatasets.has(d.dataset_id())) {
            allSelected = false;
          }
        });

        if (allSelected) {
          $datasetSearchBox.text(`All (${allSearchDatasets.length})`);
        }

        $dropdownForm.empty();

        if (allSearchDatasets.length > 0) {
          $dropdownForm.append(arc.misc.safe_substitute(allTemplate,
              {
                SELECTED_ICON: allSelected ? 'fa-check-circle' : 'fa-circle-o',
              }));
        }

        let group = '';
        const groupsExist = (datasetToSearchGroup(allSearchDatasets[0]) !== '');

        allSearchDatasets.forEach(function addDatasetSearchItems(d) {
          const currGroup = datasetToSearchGroup(d) || 'Ungrouped';
          if (groupsExist && currGroup !== group) {
            group = currGroup;

            $dropdownForm.append(arc.misc.safe_substitute(groupTemplate,
                {
                  GROUP_NAME: group,
                  SELECTED_ICON: allSelected ? 'fa-check-circle' : 'fa-circle-o',
                }));
          }

          $dropdownForm.append(arc.misc.safe_substitute(datasetTemplate,
              {
                GROUP_NAME: group,
                DATASET_ID: d.dataset_id(),
                DATASET_NAME: d.dataset_name(),
                SELECTED_ICON: (allSelected || selectedDatasets.has(d.dataset_id())) ?
                    'fa-check-circle' : 'fa-circle-thin',
              }));
        });

        if (allSearchDatasets.length > 0) {
          $datasetSearchView.show();
        }

        $dropdownForm.find('div.all-sel')
              .off('click.dataset')
              .on('click.dataset', function toggleAllDatasetSelection() {
                const $this = $(this);

                const $input = $this.find('i.selected');
                $input.toggleClass('fa-check-circle');
                $input.toggleClass('fa-circle-o');

                const selected = $input.hasClass('fa-check-circle');
                if (selected) {
                  $dropdownForm.find('div.group-sel i.fa-circle-o')
                      .removeClass('fa-circle-o')
                      .addClass('fa-check-circle');

                  $dropdownForm.find('div.dataset-sel i.fa-circle-thin')
                      .removeClass('fa-circle-thin')
                      .addClass('fa-check-circle');
                } else {
                  $dropdownForm.find('div.group-sel i.fa-check-circle')
                      .removeClass('fa-check-circle')
                      .addClass('fa-circle-o');

                  $dropdownForm.find('div.dataset-sel i.fa-check-circle')
                      .removeClass('fa-check-circle')
                      .addClass('fa-circle-thin');
                }

                changeAndShowSelection(allSearchDatasets, datasetsHash);
              });

        $dropdownForm.find('div.group-sel')
              .off('click.dataset')
              .on('click.dataset', function toggleDatasetGroupSelection() {
                const $this = $(this);
                const groupAttr = $this.attr('data-group');

                const $input = $this.find('i.selected');
                $input.toggleClass('fa-check-circle');
                $input.toggleClass('fa-circle-o');

                const selected = $input.hasClass('fa-check-circle');

                if (selected) {
                  $dropdownForm.find(`div.dataset-sel[data-group="${groupAttr}"] i.fa-circle-thin`)
                      .removeClass('fa-circle-thin')
                      .addClass('fa-check-circle');
                } else {
                  $dropdownForm.find(`div.dataset-sel[data-group="${groupAttr}"] i.fa-check-circle`)
                      .removeClass('fa-check-circle')
                      .addClass('fa-circle-thin');
                }

                // Uncheck all
                $dropdownForm.find('div.all-sel i.selected')
                    .removeClass('fa-check-circle')
                    .addClass('fa-circle-o');

                changeAndShowSelection(allSearchDatasets, datasetsHash);
              });

        $dropdownForm.find('div.dataset-sel')
              .off('click.dataset')
              .on('click.dataset', function toggleDatasetSelection() {
                const $this = $(this);
                const groupDataAttr = $this.attr('data-group');

                const $input = $this.find('i.selected');
                $input.toggleClass('fa-check-circle');
                $input.toggleClass('fa-circle-thin');

                // Uncheck the group
                $dropdownForm.find(`div.group-sel[data-group="${groupDataAttr}"] i.fa-check-circle`)
                    .removeClass('fa-check-circle')
                    .addClass('fa-circle-o');

                // Uncheck all
                $dropdownForm.find('div.all-sel i.selected')
                    .removeClass('fa-check-circle')
                    .addClass('fa-circle-o');

                changeAndShowSelection(allSearchDatasets, datasetsHash);
              });
      }
    );

    $dropdown.addClass('arc-invisible');
    $datasetSearchView.hide();
    if (allSearchDatasets.length > 0) {
      $datasetSearchView.show();
    }

    $popDatasetsWrapper
        .off('click.search')
        .on('click.search', function openDatasetDropdown(evt) {
          $dropdown.toggleClass('arc-invisible');

          if (!$dropdown.hasClass('arc-invisible')) {
            const evtName = 'click.datasets_dropdown';
            $('html')
                .off(evtName)
                .on(evtName, () => {
                  $dropdown.addClass('arc-invisible');
                  $('html').off(evtName);
                });

            $dropdown
                .off(evtName)
                .on(evtName, (evt2) => {
                  evt2.stopPropagation();
                });

            evt.stopPropagation();
          }
        });
  };

  return sel;
}

arc.search.modal = function() {
  const searchModal = function() { return 'arc.search.modal'; };

  const $modalContentsHolder = $('#modal-contents-holder');
  const $popSearchExamples = $('#test-search-modal').find('.pop-search-examples');
  const $popSearchInput = $('#test-search-modal').find('.pop-search-box');
  const $searchMenuOptions = $('#search-menu-options');
  const $searchMenuOptionsLi = $searchMenuOptions.find('li');
  const searchModalResultsTemplate = $('#search-modal-results-template').html();
  const rowTemplate = $('#search-history-template').html();

  const inputWrapper = arc.search.inputWrapper()
                                 .container('#test-search-modal');

  let searchResult;
  searchModal.text = function(_) {
    if (!arguments.length) return inputWrapper.text();
    inputWrapper.text(_);
    return searchModal;
  };

  searchModal.datasets = function(_) {
    if (!arguments.length) return inputWrapper.datasets();
    inputWrapper.datasets(_);
    return searchModal;
  };

  function cleanupTestSearch() {
    $modalContentsHolder.empty();
    $modalContentsHolder.html(searchModalResultsTemplate);
  }

  function prereqs() {
    if (!arc.globals.reports) {
      arc.globals.reports =
          arc.reporttracker.reports()
             .build_mode('display');
    }

    if (!arc.globals.params) {
      arc.globals.params = arc.params.tracker();
    }
  }

  function triggerDropdown() {
    $popSearchInput.trigger('focus');

    if ($popSearchInput.text()) {
      return;
    }

    if ($popSearchExamples.find('li').length) {
      $popSearchExamples.show();
    }
  }

  function onSearch(text) {
    if (searchResult) {
      searchResult.destroy();
    }

    showSearch();
    const $a = $searchMenuOptions.find('li a.search-menu-results');
    activateMenu($a);

    const datasets = inputWrapper.datasets();

    const q = arc.search.displayedQuery()
                 .search_text(text);

    searchResult = arc.search.results()
                             .container_id('modal-contents-holder')
                             .datasets(datasets);
    searchResult.search(text);
    setupSearchRedoIcon();
  }

  function activateHistory() {
    const $links = $modalContentsHolder.find('.search-history .search-history-section div a');

    $links.off('click.history')
          .on('click.history', function searchSelectedHistory() {
            const $this = $(this);
            inputWrapper.runSearch($this.text());
          });
  }

  function showHistory() {
    const maxHistoryDisplay = 1000;
    $modalContentsHolder.html('<div class="search-history"></div>');
    const q = arc.search.displayedQuery();
    const $historyDiv = $modalContentsHolder.find('.search-history');

    const nowDate = moment();

    const msToDaysFactor = 1000 * 60 * 60 * 24;

    q.fetch(function drawHistoryItems(displayedQueries) {
      let last;

      let processed;
      const today = [];
      const last7 = [];
      const last30 = [];
      const others = [];

      displayedQueries.forEach(function addItemToTimeGroup(currentQuery) {
        const curr = currentQuery.search_text();
        if (curr === last) {
          // Only show unique searches
          return;
        }

        last = curr;

        const currDate = currentQuery.search_time();
        const dayDiff = (nowDate - currDate) / msToDaysFactor;

        if (dayDiff === 0) {
          processed += 1;
          today.push(currentQuery);
          return;
        }

        if (dayDiff <= 7) {
          processed += 1;
          last7.push(currentQuery);
          return;
        }

        if (dayDiff <= 30) {
          processed += 1;
          last30.push(currentQuery);
          return;
        }

        if (processed > maxHistoryDisplay) {
          return;
        }

        processed += 1;
        others.push(currentQuery);
      });

      [
        [today, 'Today'],
        [last7, 'Last 7 Days'],
        [last30, 'Last 30 Days'],
        [others, 'Older Searches'],
      ].forEach(function addHistoryGroups(d) {
        const searchList = d[0];
        const text = d[1];

        if (searchList.length === 0) {
          return;
        }

        const data = [
            `<div class="search-history-section">
              <div class="search-history-header">${text}</div>
            `,
        ];

        searchList.forEach((currQuery) => {
          const t = currQuery.search_text();
          data.push(arc.misc.safe_substitute(rowTemplate, {
            SEARCH_TEXT: t,
            HTML_SEARCH_DATE: arc.misc.friendlyDate(currQuery.search_time(), 'MMM DD, h:mm A'),
          }));
        });

        data.push('</div>');

        $historyDiv.append(data.join(''));
      });

      activateHistory();
    });
  }

  function setupSearchRedoIcon(showIcons) {
    if (showIcons) {
      const $searchIcon = $modalContentsHolder.find('.redo-title-search');
      $searchIcon.removeClass('arc-invisible');
    }


    $modalContentsHolder.offOn('click.enterSearchValue', '.redo-title-search', function getResultsForBookmark(e) {
      const $this = $(e.target);
      const $result = $this.closest('.search-result-header').find('.search-text');
      const newSearchText = $result.text().replace(/\s+/g, ' ').trim();
      inputWrapper.text(newSearchText);
      onSearch(newSearchText);
     });
  }

  function showBookmarks() {
    $modalContentsHolder.html(searchModalResultsTemplate);

    $modalContentsHolder.find('.bookmark-bulk-options-container').removeClass('arc-invisible');

    // Shows "Add to current dashboard" if on dashboard edit page and can manage dashboard
    // Also calculated in arc_search_results.js as HIDE_NO_DASHBOARD
    if (arc.misc.isEditDashboard()) {
      $modalContentsHolder.find('.add-to-current-dashboard').removeClass('arc-invisible');
    }

    if (!inputWrapper.datasets()
                     .some(ds => arc.misc.hasPerm('ds_appedit', ds.dataset_perms()))) {
      $modalContentsHolder.find('.bookmark-bulk-options > .add-to-dashboard')
                          .addClass('arc-invisible');
    }

    const q = arc.search.displayedQuery();

    q.favorite(true);
    q.fetch(function setupBookmarkIcon(displayedQueries) {
      if (searchResult) {
        searchResult.destroy();
      }

      searchResult = arc.search.results()
         .container_id('modal-contents-holder')
         .datasets(inputWrapper.datasets());
      searchResult.showSearches(displayedQueries);
      setupSearchRedoIcon(true);
    });
  }

  function showSearch() {
    $modalContentsHolder.html(searchModalResultsTemplate);
  }

  function activateMenu($a) {
    const $li = $a.closest('li');

    if ($li.hasClass('active')) {
      return;
    }

    $li.closest('ul')
       .find('li')
       .removeClass('active');

    $li.addClass('active');
  }

  function setupSearchMenu() {
    $searchMenuOptionsLi
        .off('click.menu')
        .on('click.menu', function setupModalTabs() {
          const $a = $(this).find('a');
          activateMenu($a);

          $modalContentsHolder.empty();
          hideTooltip();
          $popSearchExamples.hide();
          if ($a.hasClass('search-menu-history')) {
            $modalContentsHolder.off('click.enterSearchValue');
            showHistory();
            return;
          }

          if ($a.hasClass('search-menu-bookmarks')) {
            showBookmarks();
            return;
          }

          if ($a.hasClass('search-menu-results')) {
            setupSearchRedoIcon();
            const t = inputWrapper.input().text().trim();
            if (t.length > 0) {
              onSearch(t);
            } else {
              showSearch();
              triggerDropdown();
            }
          }
        });
  }

  function setupDom() {
    // Disable dragging of the bootstrap modal
    const $container = inputWrapper.$container();
    $container.find('.modal-content').draggable('disable');

    datasetSel()
       .inputWrapper(inputWrapper)
       .setupDropdown();
  }

  searchModal.activate = function() {
    const $container = inputWrapper.$container();
    arc.misc.handleAlertContainerActivationOnModal($container);

    if (arc.misc.modalShown($container)) {
      console.log('modal already open');
      return;
    }

    prereqs();

    inputWrapper.searchCb(onSearch)
                .setup();

    setupSearchMenu();
    setupDom();
    cleanupTestSearch();

    $container.modal('show')
              .offOn('shown.bs.modal', function toggleSearchDropdown() {
      inputWrapper.shown();
      triggerDropdown();
    });

    $container.offOn('hide.bs.modal', function selectFirstOption() {
      $container.find('.alert-container').hide();
      $searchMenuOptionsLi.removeClass('active');
      $searchMenuOptionsLi[0].className = 'active';
    });

    setupSearchRedoIcon();

    return searchModal;
  };

  return searchModal;
};
}());
