window.arc = window.arc || {};

(function arcSearchResults() {
'use strict';

arc.search = arc.search || {};

arc.search.results = function searchResults() {
  const results = function getFnName() { return 'arc.search.results'; };

  let datasets;
  let searchVisualLib;
  let container_id;
  let $container_id;
  let alertContainerId;
  let $searchResults;
  let $searchProgress;
  let $addToDashboard;
  let $dropdownToggle;
  let selectedValues = {};
  let displayedQueriesGlobal = [];
  let styledMarks = [];

  const searchTemplate = $('#viz-search-result-template').html();

  results.datasets = function datasetsGetSet(_) {
    if (!arguments.length) return datasets;
    datasets = _;
    return results;
  };

  results.searchVisualLib = function searchVisualLibGetSet(_) {
    if (!arguments.length) return searchVisualLib;
    searchVisualLib = _;
    return results;
  };

  results.container_id = function containerIdGetSet(_) {
    if (!arguments.length) return container_id;
    container_id = _;
    const idText = `#${container_id}`;
    $container_id = $(idText);
    $searchProgress = $container_id.find('.search-progress');
    $searchResults = $container_id.find('.search-results');
    $addToDashboard = $container_id.find('.add-to-dashboard');
    $dropdownToggle = $container_id.find('.dropdown-toggle');

    if (container_id === 'modal-contents-holder') {
      alertContainerId = '#test-search-modal';
    } else {
      alertContainerId = null;
    }

    return results;
  };

  function removeTimeouts() {
    $('#test-search-modal .dataset-selector-dropdown .dataset-sel i.timeout')
        .addClass('arc-invisible');
  }

  function addTimeouts(timeouts) {
    timeouts.forEach(function showDatasetResults(config) {
      const {
        dataset,
      } = config;

      $(`#test-search-modal .dataset-selector-dropdown ` +
          `.dataset-sel[data-id="${dataset.dataset_id()}"] i.timeout`)
          .removeClass('arc-invisible');
    });
  }

  results.search = function searchText(text) {
    removeTimeouts();
    const timeouts = [];

    const $searchTimeouts = $container_id.find(`.search-timeouts`);

    $searchTimeouts.addClass('arc-invisible');

    if (!text) {
      return;
    }

    const formattedText = text.trim();
    if (!formattedText) {
      return;
    }

    results.destroy();
    $searchResults.empty();

    $searchProgress.removeClass('arc-invisible');
    $searchProgress.text(`Searching through ${datasets.length} datasets...`);

    function showSearchResults(scoredConfigs) {
      results.destroy();

      displayedQueriesGlobal = scoredConfigs.map(function createDisplayQuery(c) {
        return arc.search.displayedQuery()
                         .search_text(formattedText)
                         .search_data(c)
                         .dataset_id(c.dataset.dataset_id())
                         .dataset_name(c.dataset.dataset_name())
                         .store();
      });

      displaySearches(displayedQueriesGlobal);

      stylizeSearches(displayedQueriesGlobal, scoredConfigs);
      handleExpansions(displayedQueriesGlobal, scoredConfigs);

      $searchResults.find('.indicator-caret')
                    .first()
                    .trigger('click');
    }

    const threshold = 2;

    arc.search.score({
      text: formattedText,
      datasets,
      validateFilters: false,
      cb: function showScoredResults(scoredConfigs) {
        let largestScore = 0;
        let higherRatedConfigs = scoredConfigs.filter(function getGoodScores(c) {
          largestScore = Math.max(largestScore, c.score);
          return (c.score > 0);
        });

        if (higherRatedConfigs.length > 1) {
          higherRatedConfigs = higherRatedConfigs.filter(
            function getHigherThanThreshold(c) {
              if (c.score < largestScore - threshold) {
                return false;
              }

              return true;
            }
          );
        }

        if (higherRatedConfigs.length === 1) {
          showSearchResults(higherRatedConfigs);
          return;
        }

        let total = higherRatedConfigs.length === 0 ?
          datasets.length :
          higherRatedConfigs.length;
        $searchProgress.text(`Searching through ${total} datasets...`);

        // If searching through more than 1 dataset, search with a timeout of 10 seconds
        const timeoutSeconds = total > 1 ? 10 : 0;
        arc.search.score({
          text: formattedText,
          datasets: higherRatedConfigs.length === 0 ?
            datasets :
            higherRatedConfigs.map(c => c.dataset),
          validateFilters: true,
          timeoutSeconds,
          timeoutCb(config) {
            timeouts.push(config);
          },

          eachCb() {
            total -= 1;
            $searchProgress.text(`Searching through ${total} datasets...`);
          },

          cb: function showTimeouts(currentScored) {
            const nonZeroScores = currentScored.filter(c => (c.score > 0));

            if (timeouts.length > 0) {
              addTimeouts(timeouts);
              const timeoutList = timeouts.map(config => config.dataset.dataset_name());
              const timeoutDatasets = new Set(timeoutList);

              $searchTimeouts.text(`${timeoutDatasets.size} datasets timed out on search. ` +
                  `See datasets selector to view.`);
              $searchTimeouts.removeClass('arc-invisible');
            }

            if (nonZeroScores.length === 0) {
              $searchProgress.text(`No results found. Looked through ${datasets.length} ` +
                  `dataset${datasets.length === 1 ? '' : 's'}`);
              return;
            }

            showSearchResults(nonZeroScores);
          },
        });
      },
    });
  };

  results.destroy = function destroyMarks() {
    // Undraw previous visuals
    if (displayedQueriesGlobal && displayedQueriesGlobal.length) {
      displayedQueriesGlobal.forEach(dq => {
        if (dq && dq.createVisual() &&
            dq.createVisual().getReport() &&
            dq.createVisual().getReport().viz()) {
           dq.createVisual().getReport().viz().undraw();
        }
      });
    }

    // Destroy previous marks
    styledMarks.forEach(mark => mark.destroy());
    styledMarks = [];
  };

  results.showSearches = function showSearches(displayedQueries) {
    displaySearches(displayedQueries);
    handleExpansions(displayedQueries);
  };

  function displaySearches(displayedQueries) {
    $searchProgress.addClass('arc-invisible');

    results.destroy();

    $searchResults.empty();

    displayedQueries.forEach(function drawVisualResults(c, idx) {
      const isFavorite = c.favorite();

      // Shows "Add to current dashboard" if on dashboard edit page and can manage dashboard
      // Also calculated in arc_search_modal.js in showBookmarks
      const hasCurrentDashboard = arc.misc.isEditDashboard();
      $searchResults.append(arc.misc.safe_substitute(searchTemplate, {
        VISUAL_ID: arc.misc.uniqueId(),
        SEARCH_IDX: idx,
        SEARCH_TEXT: c.search_text(),
        DATASET_NAME: c.dataset_name(),
        BOOKMARK_CLASS: isFavorite ? 'fa fa-bookmark' : 'fa fa-bookmark-o',
        BOOKMARK_TEXT: isFavorite ? 'Unbookmark Visual' : 'Bookmark Visual',
        HIDE_IN_RESULTS: isFavorite ? '' : 'arc-invisible',
        SEARCH_SAVED_DATE: (c.search_time() || moment()).format('MMM DD, YYYY'),
        HIDE_NO_DASHBOARD: hasCurrentDashboard ? '' : 'arc-invisible',
      }));
    });

    handleActionButtons(displayedQueries);
  }

  function handleActionButtons(displayedQueries) {
    function bulkDelete(successCb) {
      $.ajax({
        url: `${arc.frontend.globals.urls.searchesUrl}?bookmark_ids=${Object.keys(selectedValues)}`,
        type: 'DELETE',
        beforeSend(xhr) {
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      }).done(function refreshBookmark() {
        arc.search.displayedQuery().favorite(true).force_fetch(true).fetch();
        if (successCb) {
          successCb();
        }
      });
    }

    function toggleFavorite($this, displayedQuery) {
      let isFavorite;
      const isArray = Array.isArray(displayedQuery);
      if (!isArray) {
        displayedQuery = [displayedQuery];
      }

      displayedQuery.forEach(function toggleFavoriteInUi(displayQ) {
        isFavorite = displayQ.favorite();
        displayQ.favorite(!isFavorite);
      });

      if (!isArray) {
        displayedQuery[0].store(function storeSuccess(successData) {
          $this.toggleClass('fa-bookmark', !isFavorite);
          $this.toggleClass('fa-bookmark-o', isFavorite);
          displayedQuery[0].bookmark_id(successData.bookmark_id);
          $this.attr('title', isFavorite ? 'Bookmark Visual' : 'Unbookmark Visual');
        });
      } else {
        // Currently only can bulk unbookmark on bookmark tab
        bulkDelete(function bulkSuccess() {
          const selectedIcons = $this.find('.active.multi-select');
          unselectAll();
          selectedIcons.closest('.actions')
                       .find('.fa-bookmark.favorite')
                       .attr('title', 'Bookmark Visual')
                       .toggleClass('active fa-bookmark fa-bookmark-o');
        });
      }
    }

    function unselectAll() {
      selectedValues = {};
      toggleSelection($searchResults.find('.active.multi-select'));
    }

    function toggleSelection($this, displayedQuery) {
      const bookmarkId = displayedQuery && displayedQuery.bookmark_id();
      const needUnselecting = $this.hasClass('active');
      if (needUnselecting && bookmarkId) {
        delete selectedValues[bookmarkId];
      } else if (bookmarkId) {
        selectedValues[bookmarkId] = displayedQuery;
      }

      $addToDashboard.prop('disabled', !Object.keys(selectedValues).length);
      $dropdownToggle.prop('disabled', !Object.keys(selectedValues).length);
      $this.toggleClass('fa-circle-o', needUnselecting);
      $this.toggleClass('fa-check-circle', !needUnselecting);
      $this.toggleClass('active', !needUnselecting);
    }

    function getDisplayQueryForSaving(dq, ignoredValues) {
      const createVisual = dq.createVisual();
      if (!createVisual) {
        if (ignoredValues) {
          ignoredValues.push(dq.search_text());
        }

        return;
      }

      const report = createVisual.getReport();
      const data = {
        report_search_text: dq.search_text(),
        dataset_id: dq.dataset_id(),
        report_type: report.report_type(),
      };

      const reportData = report.viz()
                               .viz()
                               .viz_getSettings();
      data.report_data = JSON.stringify({
        report_data: reportData,
        report_type: data.report_type,
      });
      return data;
    }

    function getQueryArray(displayedQuery) {
      let displayQueryArray;
      if (displayedQuery) {
        displayQueryArray = [displayedQuery];
      } else {
        displayQueryArray = Object.values(selectedValues);
      }

      return displayQueryArray;
    }

    function addToDashboard({ id, sheetOrderIds, successCb, displayedQuery }) {
      const displayQueryArray = getQueryArray(displayedQuery);
      let searchText = '';
      if (displayQueryArray.length === 1) {
        searchText = displayQueryArray[0].search_text();
      }

      const new_reports = [];
      const errors = [];
      displayQueryArray.forEach((dq) => {
        const reportData = getDisplayQueryForSaving(dq, errors);
        if (reportData) {
          new_reports.push(reportData);
        }
      });

      if (errors.length) {
        const errorMessage =
            `Expand the following searches to add to dashboard: ${errors.join('; ')}`;
        arc.report.alertMessageSmallBase({
          alertHeader: 'Alert',
          alertMessage: errorMessage,
          type: 'alert-warning',
        });
      }

      if (!new_reports.length) {
        return;
      }

      const params = {
        search_text: searchText,
        dataset_id: displayQueryArray[0].dataset_id(),
        sheet_order: sheetOrderIds ? sheetOrderIds[0] : undefined,
        report_id: id,
        new_reports: JSON.stringify(new_reports),
      };

      $.ajax({
        url: arc.frontend.globals.urls.reportsAddSearchUrl,
        data: params,
        type: 'POST',
        beforeSend(xhr) {
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
      .done(function openNewTab(data) {
        const dashId = data.report_id;
        const reportUrl = arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.reportBuildUrl}/${dashId}`);
        const successMessage = `Dashboard <a href=${reportUrl} target="_blank">` +
                               `${dashId}</a> ${id ? 'updated' : 'created'}`;
        arc.report.alertMessageSmallBase({
          type: 'alert-success',
          useHTML: true,
          alertHeader: 'Success',
          alertMessage: successMessage,
          fadeOut: true,
        });

        if (successCb) {
          successCb(data);
        }
      })
      .fail(function alertFailSave(req) {
        const error = req.responseText ? `: ${req.responseText}` : '';
        arc.report.alertMessageSmall(`Error saving to dashboard${error}`,
                                     'alert-danger', alertContainerId);
      });

      return id;
    }

    function createSearchVisual({ displayedQuery, idx = 0 }) {
      const displayQueryArray = getQueryArray(displayedQuery);
      const errors = [];
      while (displayQueryArray[idx] &&
            !getDisplayQueryForSaving(displayQueryArray[idx], errors)) {
        idx += 1;
      }

      if (errors.length) {
        const errorMessage =
            `Expand the following searches to add to dashboard: ${errors.join('; ')}`;
        arc.report.alertMessageSmallBase({
          alertHeader: 'Alert',
          alertMessage: errorMessage,
          type: 'alert-warning',
        });
      }

      if (!displayQueryArray[idx]) {
        return;
      }

      $.ajax({
        url: arc.frontend.globals.urls.reportUrl,
        data: getDisplayQueryForSaving(displayQueryArray[idx]),
        type: 'POST',
        beforeSend(xhr) {
          xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
        },
      })
      .done(function openNewTab(data) {
        idx += 1;
        arc.globals.dashboard.drawInWidget({
          report_id: Number(data),
          force_redraw: true,
          force_reloadreport: true
        });
        if (idx >= displayQueryArray.length) {
          arc.report.pagenav.needs_save(true);

          // Use settimeout otherwise alert gets overwritten during checkShelfContent
          setTimeout(() => {
            arc.report.alertMessageSmall('Search visuals added to current dashboard',
                                         'alert-success', alertContainerId);
          }, 500);
        } else {
          createSearchVisual({ displayedQuery, idx });
        }
      })
      .fail(function alertFailSave(req) {
        idx += 1;
        const error = req.responseText ? `: ${req.responseText}` : '';
        arc.report.alertMessageSmall(`Error creating to visual${error}`,
                                     'alert-danger', alertContainerId);
      });
    }

    $container_id.find('.clear-bookmark-selection')
                 .offOn('click.unselectAll', unselectAll);

    $container_id.find('.action-icon')
                 .offOn('click.search', function triggerButtonAction(e) {
          const $this = $(e.currentTarget);
          const $result = $this.closest('.search-result');
          const idx = $result.attr('data-idx');
          const displayedQuery = displayedQueries[idx];
          if ($this.hasClass('favorite')) {
            toggleFavorite($this, displayedQuery);
          } else if ($this.hasClass('unbookmark-selected')) {
            toggleFavorite($searchResults, Object.values(selectedValues));
          } else if ($this.hasClass('multi-select')) {
            toggleSelection($this, displayedQuery);
          } else if ($this.hasClass('add-to-existing-dashboard')) {
            const pickerConfig = {
              displaySheets: true,
              customValidator: report => arc.misc.hasPerm('ds_appedit', report.perm),
            };
            arc.report.reportPicker(false, /* no click behavior */
                                    (config) => {
                                      $('.report-pick-modal').modal('hide');
                                      config.displayedQuery = displayedQuery;
                                      addToDashboard(config);
                                    },

                                    displayedQuery && displayedQuery.dataset_id(),
                                    pickerConfig);
          } else if ($this.hasClass('add-to-current-dashboard')) {
            createSearchVisual({ displayedQuery });
          } else if ($this.hasClass('add-to-new-dashboard')) {
            addToDashboard({ displayedQuery });
          }
        });
  }

  function addRedrawTextCb(displayQuery, scoredConfig) {
      scoredConfig.redrawTextCb = (newText) => {
        if (displayQuery && displayQuery.createVisual()) {
          displayQuery.search_text(newText);
          displayQuery.bookmark_id(undefined);
          const createVisual = displayQuery.createVisual();
          const newConfig = createVisual.getOrigConfig();

          if (newConfig.report && newConfig.report.viz()) {
            newConfig.report.viz().undraw();
          }

          if (createVisual.getReport() && createVisual.getReport().viz()) {
            createVisual.getReport().viz().undraw();
          }

          delete newConfig.finalScoreConfig;
          delete newConfig.report;
          createVisual.makeNewVisual(newText, newConfig);
        }
      };
  }

  function stylizeSearches(displayedQueries, scoredConfigs) {
    $searchResults.find('.search-text')
                  .each(function displaySearchText(idx) {
                    const displayedQuery = displayedQueries[idx];
                    const scoredConfig = scoredConfigs[idx];
                    addRedrawTextCb(displayedQuery, scoredConfig);

                    const marks = arc.search.stylizeSearchElement($(this),
                        displayedQuery.search_text(), scoredConfig);
                    styledMarks.push(marks);
                    displayedQuery.highlightMarks(marks);
                  });
  }

  function handleExpansions(displayedQueries, scoredConfigs) {
    $searchResults.find('.indicator-caret')
                  .offOn('click.search', function openCont(e) {
          const $this = $(e.target);
          const $result = $this.closest('.search-result');
          const $indicator = $result.find('.indicator-caret');
          const $explore = $result.find('.explore-container');
          const $hideIfNoPerm = $result.find('.hide-if-no-permissions');
          const idx = $result.attr('data-idx');
          const displayedQuery = displayedQueries[idx];
          const scoredConfig = scoredConfigs ? scoredConfigs[idx] : undefined;

          const $container = $result.find('.arc-viz-container');
          const $popSearchIcon = $result.find('.pop-search-explain');
          const $viewDataIcon = $result.find('.search-view-data');
          const $toggleContainers = $result.find('.hide-until-visual');
          const $visual = $result.find('.search-visual');
          const shelves_dom_id = $result.find('.search-visual-shelf').attr('id');
          const chart_div = $visual.attr('id');
          const isOpen = $container.hasClass('open');
          const tooltipWarning = isOpen ? ': Expand to enable' : '';

          $result.toggleClass('open-result', !isOpen);
          $container.toggleClass('open', !isOpen);
          $container.toggleClass('arc-invisible', isOpen);
          $toggleContainers.toggleClass('disabled', isOpen)
                           .attr('title', `Explore Visual Types${tooltipWarning}`);
          $hideIfNoPerm.toggleClass('disabled', isOpen)
                       .attr('title', `Add to Dashboard${tooltipWarning}`);

          if (!arc.globals.enable_explore_interfaces) {
              $('.explore-icon').hide();
          }

          if (!isOpen) {
            $popSearchIcon.toggleClass('disabled', isOpen)
                          .attr('title', `Explain${tooltipWarning}`);

            $viewDataIcon.toggleClass('disabled', isOpen)
                         .attr('title', `View Data${tooltipWarning}`);
          }

        $indicator.toggleClass('fa-caret-down fa-caret-right');
        if (isOpen) {
          $explore.off('click.showexplore');
          $result.css('height', '');
          $visual.css('height', '');
          $visual.empty();
          displayedQuery.createVisual(null);
          $(`#${shelves_dom_id}`).empty();
          return;
        }

        $visual.html('<div class="search-result-loading">Loading...</div>');

        /*
          * If we have already validated filters, then we pass the data to makeNewVisual
          *   so we don't run the same validation a second time.
          */
        let finalScoreConfig;
        if (scoredConfig && scoredConfig.validateFilters) {
          finalScoreConfig = scoredConfig;
        }

        const makeVisualConfig = {
          isSearchShowme: true,
          shelves_dom_id,
          chart_div,
          finalScoreConfig,
          searchVisualLib,
          $searchButtonContainer: $result,
          displayedQuery,
          $explore,
          $addToDashboard: $hideIfNoPerm,
          highlight_cb(highlightConfig) {
            addRedrawTextCb(displayedQuery, highlightConfig);
            const $s = $result.find('.search-text');
            const marks = arc.search.stylizeSearchElement($s,
                displayedQuery.search_text(), highlightConfig);
            displayedQuery.highlightMarks(marks);
            styledMarks.push(marks);
          },
        };

        if (scoredConfig) {
          makeVisualConfig.datasets = [scoredConfig.dataset];
          const newCreatedVisual = arc.visual.create()
              .makeNewVisual(displayedQuery.search_text(), makeVisualConfig);
          displayedQuery.createVisual(newCreatedVisual);
          return;
        }

        fetchDatasetAndCb(displayedQuery.dataset_id(), function drawFromDataset(dataset) {
          if (dataset === undefined) {
            $visual.html('<div class="search-result-loading">Could not load dataset</div>');
            return;
          }

          makeVisualConfig.datasets = [dataset];
          const createdVisual = arc.visual.create()
              .makeNewVisual(displayedQuery.search_text(), makeVisualConfig);
          displayedQuery.createVisual(createdVisual);
        });
    });
  }

  function fetchDatasetAndCb(dataset_id, cb) {
    const assets = [];
    assets.push({
      asset: `${arc.frontend.globals.urls.datasetUrl}/${dataset_id}?search=true`,
      postprocess: d3.json,
    });

    arc.staticfetcher.fetch(assets,
      function drawAfterDatasetsFetched(error, _datasets) {
        if (error) {
          arc.report.alertMessage('Failed to load datasets', error);
          return;
        }

        const d = _datasets[0];
        if (d === undefined) {
          cb(undefined);
          return;
        }

        const dataset = arc.datasets.Dataset()
                           .dataset_id(d.id)
                           .init_from_fetch(d, undefined);
        cb(dataset);
      });
  }

  return results;
};
}());
