window.arc = window.arc || {};

(function arcSearchSaved() {
'use strict';

arc.search = arc.search || {};

arc.search.displayedQuery = function createDisplayedQuery() {
  const displayedQuery = function getFnName() { return 'displayedQuery'; };

  let search_text;
  let bookmark_id;
  let username;
  let dataset_id; // for favorite items
  let dataset_ids; // for history items
  let dataset_name;
  let favorite;
  let search_time;
  let force_fetch;
  let createVisual;
  let highlightMarks;
  let search_data;

  displayedQuery.search_text = function searchTextGetSet(_) {
    if (!arguments.length) return search_text;
    search_text = _;
    return displayedQuery;
  };

  displayedQuery.getFavorite = function() {
    if (search_data &&
        arc.globals.favoritesHash && arc.globals.favoritesHash[search_text] &&
        arc.globals.favoritesHash[search_text][dataset_id]) {
      const searchData = arc.globals.favoritesHash[search_text][dataset_id];
      const isFavorite = searchData.some(function assessFavorite(favData) {
        const isSameVizId = favData.search_data.vizId === search_data.vizId;

        if (isSameVizId) {
          favorite = true;
          bookmark_id = favData.bookmark_id;
          displayedQuery.fromParams(favData.search_data);
        }

        return isSameVizId;
      });

      if (isFavorite) {
        return displayedQuery;
      }

      favorite = false;
      bookmark_id = undefined;
    }
  };

  displayedQuery.search_data = function searchDataGetSet(_) {
    if (!arguments.length) return search_data;
    search_data = _;
    return displayedQuery;
  };

  displayedQuery.force_fetch = function forceFetchGetSet(_) {
    if (!arguments.length) return force_fetch;
    force_fetch = _;
    return displayedQuery;
  };

  displayedQuery.bookmark_id = function bookmarkIdGetSet(_) {
    if (!arguments.length) return bookmark_id;
    bookmark_id = _;
    return displayedQuery;
  };

  displayedQuery.username = function usernameGetSet(_) {
    if (!arguments.length) return username;
    username = _;
    return displayedQuery;
  };

  displayedQuery.dataset_id = function datasetIdGetSet(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return displayedQuery;
  };

  displayedQuery.dataset_ids = function datasetIdGetSet(_) {
    if (!arguments.length) return dataset_ids;
    dataset_ids = _;
    return displayedQuery;
  };

  displayedQuery.dataset_name = function datasetNameGetSet(_) {
    if (!arguments.length) return dataset_name;
    dataset_name = _;
    return displayedQuery;
  };

  displayedQuery.favorite = function favoriteGetSet(_) {
    if (!arguments.length) return favorite;
    favorite = _;

    const hasFavoriteHash = search_data &&
                            arc.globals.favoritesHash;
    const datasetHash = hasFavoriteHash && arc.globals.favoritesHash[search_text] &&
                        arc.globals.favoritesHash[search_text][dataset_id];
    if (hasFavoriteHash && favorite && !datasetHash) {
      if (!arc.globals.favoritesHash[search_text]) {
        arc.globals.favoritesHash[search_text] = {};
      }

      if (!arc.globals.favoritesHash[search_text][dataset_id]) {
        arc.globals.favoritesHash[search_text][dataset_id] = [];
      }

      arc.globals.favoritesHash[search_text][dataset_id].push({
        bookmark_id,
        search_data,
      });
    } else if (hasFavoriteHash && !favorite && datasetHash) {
      arc.globals.favoritesHash[search_text][dataset_id] =
        datasetHash.filter(function removeFavorite(dq) {
          return dq.search_data.vizId !== search_data.vizId;
        });
    }

    return displayedQuery;
  };

  displayedQuery.search_time = function searchTimeGetSet(_) {
    if (!arguments.length) return search_time;
    search_time = _;
    return displayedQuery;
  };

  displayedQuery.highlightMarks = function highlightMarkGetSet(_) {
    if (!arguments.length) return highlightMarks;

    if (highlightMarks) {
      highlightMarks.destroy();
    }

    highlightMarks = _;
    return displayedQuery;
  };

  displayedQuery.createVisual = function createVisualGetSet(_) {
    if (!arguments.length) return createVisual;

    if (!_ && createVisual &&
        createVisual.getReport() &&
        createVisual.getReport().viz()) {
      createVisual.getReport().viz().undraw();
    }

    if (!_ && highlightMarks) {
      highlightMarks.destroy();
    }

    createVisual = _;
    return displayedQuery;
  };

  displayedQuery.fromParams = function setFromParams(d) {
    if (d.bookmark_id) {
      bookmark_id = d.bookmark_id;
    }

    if (d.search_text) {
      search_text = d.search_text;
    }

    if (d.search_data) {
      try {
        search_data = JSON.parse(d.search_data);
      } catch (e) {
        // Ignore any errors here
      }
    }

    if (d.dataset_id) {
      dataset_id = d.dataset_id;
    }

    if (d.dataset_ids) {
      dataset_ids = d.dataset_ids;
    }

    if (d.dataset_name) {
      dataset_name = d.dataset_name;
    }

    if (d.favorite) {
      favorite = d.favorite === 'true';
    }

    if (d.search_time) {
      search_time = moment(d.search_time);
    }

    return displayedQuery;
  };

  displayedQuery.toParams = function setToParams() {
    const params = {
      search_text,
    };

    if (search_data) {
      params.search_data = JSON.stringify({
        vizId: search_data.vizId,
        showmeVisualConfig: search_data.showmeVisualConfig,
      });
    }

    if (bookmark_id) {
      params.bookmark_id = bookmark_id;
    }

    if (dataset_id) {
      params.dataset_id = dataset_id;
    }

    if (dataset_ids) {
      params.dataset_ids = dataset_ids;
    }

    if (favorite) {
      params.favorite = true;
    }

    return params;
  };

  displayedQuery.store = function storeParams(successCb) {
    const params = displayedQuery.toParams();
    params.search_time = new Date();

    if (arc.globals.search_history) {
      arc.globals.search_history.unshift(params);
    }

    $.ajax({
      url: arc.frontend.globals.urls.searchesUrl,
      data: params,
      type: 'POST',
      beforeSend(xhr) {
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done((data) => {
      if (favorite) {
        processFavorite(params);
      } else {
        displayedQuery.favorite(false);
      }

      if (successCb) {
        successCb(data);
      }
    });

    return displayedQuery;
  };

  function parseData(returnedData, usingCached) {
    return returnedData.map((d) => {
      const newDisplayQuery = arc.search.displayedQuery()
                                        .fromParams(d);

      if (favorite && !usingCached) {
        processFavorite(d);
      }

      return newDisplayQuery;
    });
  }

  displayedQuery.fetch = function fetchHistory(cb) {
    // Don't fetch history if we already have copy on the frontend
    if (arc.globals.search_history && !favorite) {
      if (cb) {
        cb(parseData(arc.globals.search_history, true));
      }

      return displayedQuery;
    }

    const params = displayedQuery.toParams();
    $.ajax({
      url: arc.frontend.globals.urls.searchesUrl,
      data: params,
      type: 'GET',
    })
    .done((data) => {
      // Save favorite on the front end so it can be used by autosuggest
      if (!favorite) {
        arc.globals.search_history = data;
      } else {
        arc.globals.favorites = data;
        arc.globals.favoritesHash = {};
      }

      if (cb) {
        cb(parseData(data));
      } else if (favorite) {
        arc.globals.favorites.forEach(processFavorite);
      }
    })
    .fail((req) => {
      arc.report.alertMessage('Error loading saved searches', req.responseText);
    });
  };

  function processFavorite(fav) {
    if (!arc.globals.favoritesHash[fav.search_text]) {
      arc.globals.favoritesHash[fav.search_text] = {};
    }

    if (!arc.globals.favoritesHash[fav.search_text][fav.dataset_id]) {
      arc.globals.favoritesHash[fav.search_text][fav.dataset_id] = [];
    }

    if (typeof fav === 'string') {
      fav = JSON.parse(fav);
    }

    arc.globals.favoritesHash[fav.search_text][fav.dataset_id].push({
      bookmark_id: fav.bookmark_id,
      search_data: JSON.parse(fav.search_data),
    });
  }

  return displayedQuery;
};
}());
