window.arc = window.arc || {};

(function arcSearchSettings() {
'use strict';

arc.search = arc.search || {};

function linkWithIcon(icon, classname) {
  return `<a class="${classname}"><i class="fa ${icon}"></i></a>`;
}

arc.search.searchSettingsHelper = function() {
  let s = function() { return 'arc.search.searchSettingsHelper'; };

  let builder;
  let datasetAdmin;
  const columnTracker = {};
  let columnId = 1;
  let searchSettings = {};
  const searchCompiler = arc.search.searchCompiler();
  let suggestions;
  let wordExpansions;
  let originalSearchColumn = 0;
  let searchableColumns = 0;

  s.builder = function(_) {
    if (!arguments.length) return builder;
    builder = _;
    return s;
  };

  s.datasetAdmin = function datasetAdminGetSet(_) {
    if (!arguments.length) return datasetAdmin;
    datasetAdmin = _;
    return s;
  };

  s.setupSearch = function setupSearch() {
    $('#search-table tbody').empty();
    $('#search-words tbody').empty();
    $('#default-agg').empty();

    columnId = 1;
    searchableColumns = 0;

    const dataset = builder.dataset();
    const hasEditPerm = arc.misc.hasPerm('ds_appedit', dataset.dataset_perms());
    searchSettings = dataset.searchSettings();
    setupSettings(hasEditPerm);

    if (searchSettings.defaultAgg) {
      const col = dataset.findColumn(searchSettings.defaultAgg);
      if ((col === undefined) || (!col.isAggregate())) {
        searchSettings.defaultAgg = undefined;
      }
    }

    if (searchSettings.enabled) {
      searchCompiler.dataset(dataset)
                    .initialize();

      arc.topnav.initialSearchDatasets([dataset]);
    } else {
      arc.topnav.resetDatasets();
    }

    dataset.columnIterator((column) => {
      if (column.attr('hidden') || column.isComplexType()) {
        return;
      }

      columnTracker[columnId] = column;
      addSearchColumn(columnId, column, hasEditPerm);
      columnId += 1;
    });

    addDefaultAgg('');

    if (hasEditPerm) {
      setupSaveUndo();
      setupSearchColumnSaves();
    }

    setupSearchGroup(hasEditPerm);
    setupSuggestions(hasEditPerm);
    setupWordExpansions(hasEditPerm);
    setupDefaultAgg(hasEditPerm);
  };

  function rowToSuggestion(row) {
    return {
      s: row[0],
    };
  }

  function suggestionToRow(suggestion) {
    return [suggestion.s];
  }

  function getSuggestions() {
    return suggestions.getRows().map(rowToSuggestion);
  }

  function setupSuggestions(hasEditPerm) {
    const $table = $('#search-suggestions');

    searchSettings.suggestions = searchSettings.suggestions || [];

    function getRows() {
      return searchSettings.suggestions.map(suggestionToRow);
    }

    function changeCb() {
      $table.find('.try-search')
            .off('click')
            .on('click', function tableChangeHandler() {
              const initialText = $(this)
                                  .closest('tr')
                                  .find('td:first')
                                  .text()
                                  .trim();
              arc.topnav.runModal(undefined, [builder.dataset()], initialText);
            });
    }

    suggestions = arc.utils.modifiableRowsTable({
      $table,
      initialData: getRows(),
      needsSaveCb() {
        needsSave(true);
      },

      extraColumns: [
        linkWithIcon('fa-search', 'try-search'),
      ],

      changeCb,
    });

    suggestions.setup();

    if (!hasEditPerm) {
      suggestions.makeEditable(false);
    }
  }

  function expansionToRow(expansion) {
    return [expansion.key, expansion.value];
  }

  function rowToExpansion(row) {
    return {
      key: row[0],
      value: row[1],
    };
  }

  function getExpansions() {
    return wordExpansions.getRows().map(rowToExpansion);
  }

  function setupWordExpansions(hasEditPerm) {
    const $table = $('#search-words');

    searchSettings.expansions = searchSettings.expansions || [];

    function getRows() {
      return searchSettings.expansions.map(expansionToRow);
    }

    wordExpansions = arc.utils.modifiableRowsTable({
      $table,
      initialData: getRows(),
      needsSaveCb() {
        needsSave(true);
      },
    });

    wordExpansions.setup();

    if (!hasEditPerm) {
      wordExpansions.makeEditable(false);
    }
  }

  function setupSettings(hasEditPerm) {
    const $searchEnabled = $('#dataset-search').find('.search-enabled');
    $searchEnabled.prop('checked', searchSettings.enabled);

    if (hasEditPerm) {
      $searchEnabled.offOn('change.toggleSearch', function toggleSearch() {
        needsSave(true);
        searchSettings.enabled = $(this).prop('checked');
      });
    } else {
      $searchEnabled.prop('disabled', true);
    }
  }

  const $undoSearch = $('#undo-search');
  const $saveSearch = $('#save-search');
  function needsSave(on) {
    arc.report.pagenav.needs_save(on);
    $undoSearch.toggleClass('disabled', !on);
    $saveSearch.toggleClass('disabled', !on);
    $saveSearch.toggleClass('btn-primary', on);
  }

  function setupSaveUndo() {
    $undoSearch.offOn('click.undoSearchChanges', () => {
                       needsSave(false);
                       searchableColumns = originalSearchColumn;
                       datasetAdmin.builderReload(() => {
                         datasetAdmin.datasetRefreshed();
                         s.setupSearch();
                       });
                     });

    const $searchableConfirm = $('#save-searchable-modal');
    const $searchableNumber = $searchableConfirm.find('.arc-searchable-number');
    const $searchableConfirmButton = $searchableConfirm.find('#save-searchable-yes');
    function saveSearchSettings() {
      searchSettings.expansions = getExpansions();
      searchSettings.suggestions = getSuggestions();
      const dataset = builder.dataset();
      dataset.searchSettings(searchSettings);

      dataset.save(() => {
               needsSave(false);
               arc.search.utils.forgetAllDatasets();
               datasetAdmin.builderReload(() => {
                 datasetAdmin.datasetRefreshed();
               });
             });
    }

    function setupSaveConfirm() {
      $searchableNumber.text(searchableColumns);
      $searchableConfirm.modal('show');
    }

    $searchableConfirmButton.offOn('click.confirmSave', () => {
      saveSearchSettings();
      $searchableConfirm.modal('hide');
    });

    $saveSearch
        .offOn('click.saveSearchSettings', function trySaveSettings() {
          if (searchableColumns > 10) {
            setupSaveConfirm();
          } else {
            saveSearchSettings();
          }
        });
  }

  function setupSearchGroup(hasEditPerm) {
    const $searchGroup = $('#search-group');
    $searchGroup.val(searchSettings.group || '');
    if (hasEditPerm) {
      $searchGroup.offOn(
        'change.updateSearchGroup',
        function updateSearchGroup() {
          needsSave(true);
          const $this = $(this);
          searchSettings.group = $this.val();
        }
      );
    } else {
      $searchGroup.prop('disabled', true);
    }
  }

  function setupSearchColumnSaves() {
    $('#search-table tbody .terms')
        .offOn('input', function handleSearchTableInput() {
          needsSave(true);
          const $this = $(this);
          const dataColumnId = $this.closest('tr').attr('data-columnid');
          const column = columnTracker[dataColumnId];
          const v = $this.text();
          column.attr('terms', v);
        });

    $('#search-table tbody .use-for-search')
        .offOn('change', function handleSearchChange() {
          needsSave(true);
          const $this = $(this);
          const dataColumnId = $this.closest('tr').attr('data-columnid');
          const column = columnTracker[dataColumnId];
          const newVal = parseInt($this.val(), 10);

          if (!column.attr('searchable') && newVal) {
            searchableColumns += 1;
          } else if (column.attr('searchable') && !newVal) {
            searchableColumns -= 1;
          }

          column.attr('searchable', newVal);
        });
  }

  function setupDefaultAgg(hasEditPerm) {
    if (searchSettings.defaultAgg !== undefined) {
      setDefaultAgg(searchSettings.defaultAgg, hasEditPerm);
    } else {
      searchSettings.defaultAgg = getDefaultAgg();
    }

    if (hasEditPerm) {
      $('#default-agg').offOn('change.defaultAgg', () => {
        needsSave(true);
        searchSettings.defaultAgg = getDefaultAgg();
      });
    }
  }

  function getDefaultAgg() {
    return $('#default-agg').val();
  }

  function setDefaultAgg(agg, hasEditPerm) {
    const $a = $('#default-agg');
    $a.val(agg);

    if (!hasEditPerm) {
      $a.prop('disabled', true);
    }
  }

  function addDefaultAgg(agg) {
    $('#default-agg').append(arc.misc.safe_substitute('<option>%AGG%</option>',
        {
          AGG: agg,
        }));
  }

  function addSearchColumn(currColumnId, column, hasEditPerm) {
    const attr = column.attr();

    const nameUnquoted = column.display_name();
    const name = arc.datasets.quoteColname(nameUnquoted);
    const terms = column.terms();
    const hasAggregate = column.attr('exprhasagg') || column.attr('recordcount');

    let kind;
    if (column.isDimension() || column.isGeo()) {
      kind = 'Dimension';
    } else if (hasAggregate) {
      kind = 'Aggregate';
      if (column.attr('recordcount')) {
        addDefaultAgg(nameUnquoted);
      }
    } else {
      const defaultAggregate = column.defaultAggregate().toLowerCase();
      kind = `Aggregate (${defaultAggregate})`;

      addDefaultAgg(nameUnquoted);
    }

    let searchable = '';
    if (!hasEditPerm) {
      searchable += ' disabled';
    } else if (hasAggregate) {
      searchable += ' title="This column has an aggregate and cannot be searched" disabled';
    }

    if (attr.searchable) {
      originalSearchColumn += 1;
      searchableColumns += 1;

      // Convert over true/false searchable values to high/low
      if (typeof attr.searchable !== 'number') {
        attr.searchable = 3;
      }
    }

    const tr = arc.misc.safe_substitute($('#dataset-searchcol-template tbody').html(),
    {
      COL_COLUMNID: currColumnId,
      COL_NAME: name,
      COL_DATATYPE: attr.type,
      COL_KIND: kind,
      COL_TERMS: terms,
      HTML_COL_USE_FOR_SEARCH: searchable,
      HIGH_SELECTED: attr.searchable === 1 ? 'selected' : '',
      MEDIUM_SELECTED: attr.searchable === 2 ? 'selected' : '',
      LOW_SELECTED: attr.searchable === 3 ? 'selected' : '',
      HAS_EDIT_PERMISSION: hasEditPerm ? 'true' : 'false',
    });

    $('#search-table tbody').append(tr);
  }

  return s;
};
}());
