window.arc = window.arc || {};

(function() {
'use strict';

arc.tree = arc.tree || {};

arc.tree.node = function(_data) {
  const node = function() { return 'arc.tree.node'; };

  let data = _data;
  let prev;
  let next;
  let parent;
  let firstChild;

  // Replace this node with another one
  node.replace = function(replacementNode) {
    replacementNode.prev(prev);
    if (prev) {
      prev.next(replacementNode);
    }

    replacementNode.next(next);
    if (next) {
      next.prev(replacementNode);
    }

    replacementNode.parent(parent);
    if (parent && parent._firstChild() === node) {
      parent._firstChild(replacementNode);
    }

    prev = undefined;
    next = undefined;
    parent = undefined;
    firstChild = undefined;
  };

  // Remove this node from the tree
  node.remove = function() {
    if (prev) {
      prev.next(next);
    }

    if (next) {
      next.prev(prev);
    }

    if (parent && parent._firstChild() === node) {
      parent._firstChild(next);
    }

    prev = undefined;
    next = undefined;
    parent = undefined;
    firstChild = undefined;
  };

  // Note that this function is for internal use only
  node._firstChild = function(_) {
    if (!arguments.length) return firstChild;
    firstChild = _;
    return node;
  };

  node.data = function(_) {
    if (!arguments.length) return data;
    data = _;
    return node;
  };

  node.prev = function(_) {
    if (!arguments.length) return prev;
    prev = _;
    return node;
  };

  node.next = function(_) {
    if (!arguments.length) return next;
    next = _;
    return node;
  };

  node.parent = function(_) {
    if (!arguments.length) return parent;
    parent = _;
    return node;
  };

  node.isLeaf = function() {
    return (data && (firstChild === undefined));
  };

  node.children = function() {
    const cl = [];
    let c = firstChild;
    while (c !== undefined) {
      cl.push(c);
      c = c.next();
    }

    return cl;
  };

  node.lastChild = function() {
    const cl = node.children();
    if (cl.length > 0) {
      return cl[cl.length - 1];
    }

    return undefined;
  };

  node.addChild = function(child, insertPos) {
    if (insertPos === undefined) {
      insertPos = node.lastChild();
    }

    let n;
    if (insertPos === undefined) {
      firstChild = child;
    } else {
      n = insertPos.next();
      insertPos.next(child);
    }

    child.prev(insertPos);
    child.next(n);
    child.parent(node);
  };

  node.walkLeaves = function(cb) {
    /* First, we walk the tree and save away all the nodes
     * that we need to call cb on.
     * Then, we make the callbacks. That way, the cbs themselves
     * may delete nodes and not cause any issues with our
     * traversing the tree while they are modifying it.
     */
    const storedNodes = [];

    node.walk((n) => {
      if (n.isLeaf()) {
        storedNodes.push(n);
      }
    });

    storedNodes.some(n => cb(n));
  };

  node.walk = function(cb, skipThis) {
    let t;

    if (!skipThis) {
      t = cb(node);
      if (t) {
        return t;
      }
    }

    if (firstChild) {
      t = firstChild.walk(cb);
      if (t) {
        return t;
      }
    }

    if (next) {
      return next.walk(cb);
    }
  };

  node.nextLeaf = function() {
    let leaf;

    node.walk((n) => {
      if (n.isLeaf()) {
        leaf = n;
        return true;
      }
    }, true);

    return leaf;
  };

  node.prevLeaf = function() {
    let leaf;

    node.walkPrev((n) => {
      if (n.isLeaf()) {
        leaf = n;
        return true;
      }
    });

    return leaf;
  };

  node.walkPrevLeaves = function(cb) {
    /* First, we walk the tree and save away all the nodes
     * that we need to call cb on.
     * Then, we make the callbacks. That way, the cbs themselves
     * may delete nodes and not cause any issues with our
     * traversing the tree while they are modifying it.
     */
    const storedNodes = [];

    node.walkPrev((n) => {
      if (n.isLeaf()) {
        storedNodes.push(n);
      }
    });

    storedNodes.some(n => cb(n));
  };

  node.walkPrev = function(cb) {
    // walk left
    if (prev) {
      const t = prev.walkBackwards(cb);
      if (t) {
        return t;
      }
    }

    // walk up and left
    if (parent) {
      const p = parent.prev();
      if (p) {
        return p.walkBackwards(cb);
      }
    }
  };

  // walk down
  node.walkBackwards = function(cb) {
    if (node.isLeaf()) {
      const r = cb(node);
      if (r) {
        return r;
      }
    }

    if (!node.isLeaf()) {
      let t;

      node.children().reverse().some((c) => {
        t = c.walkBackwards(cb);
        return !!t;
      });

      if (t) {
        return t;
      }
    }

    return node.walkPrev(cb);
  };

  return node;
};
}());
