window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};
arc.report.globals = arc.report.globals || {};
arc.report.user_settings = arc.report.user_settings || {};

/*
 * For table & crosstab visuals, tooltips should be disabled by default.
 * For all other visuals, tooltips should be enabled by default.
 *
 * We implement this by having 2 'Display tooltips' settings below,
 *   one used for tables/crosstabs, and the other used for all other
 *   visual types.
 *
 * This is necessary because we want to maintain this difference when
 *   switching between table and non-table visuals.
 */

arc.report.user_settings.display_tooltips = function() {
  return arc.visual.setting()
            .id('show_tooltip')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Display tooltips')
            .value('true');
};

arc.report.user_settings.display_tooltips_table = function() {
  return arc.visual.setting()
            .id('display_tooltips_table')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Display tooltips')
            .value('false');
};

arc.report.user_settings.remember_selections = function() {
  return arc.visual.setting()
            .id('remember_selections')
            .tab('Display Settings')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Remember previous selections when searching')
            .info('If unchecked, searching in the filter will unselect anything that does not appear in the search results.')
            .value('true');
};

arc.report.user_settings.smooth_line = function() {
  return arc.visual.setting()
            .id('smooth_line')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Ignore missing dates')
            .info('Otherwise, missing days will be graphed at 0.')
            .value('true');
};

arc.report.user_settings.same_domain = function() {
  return arc.visual.setting()
            .id('same_domain')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Source & destination data come from the same base values.')
            .value('false');
};

arc.report.user_settings.sync_settings = function() {
  return arc.visual.setting()
            .id('sync_queries')
            .tab('Performance')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Synchronous queries. Useful in some cases with ' +
                'a large number of queries and very fast performance.')
            .value('false');
};

arc.report.user_settings.explicit_scope = function() {
  return arc.visual.setting()
            .id('explicit_scope')
            .tab('Parameters')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Use parameters with explicit scope')
            .info('The visual uses only filters and parameters ' +
                'specified on its Filter shelf. It overrides dashboard ' +
                'and dataset filters and parameters.')
            .value('false');
};

/*
 * This parameter was converted from a CHECKBOX. That is why the 'true' value is
 * associated with dataset scope, and the 'false' value is associated with
 * explicit scope.
 */
arc.report.user_settings.filter_dataset_scope = function() {
  const setting = arc.visual.setting()
            .id('filter_dataset_scope')
            .tab('Scope')
            .type(arc.visual.settingTypeEnum.RADIO)
            .inline(false)
            .group('Scope of parameters sent by this filter:')
            .add_name('GLOBAL')
            .add_label('All Datasets')
            .add_name('true')
            .add_label('Current Dataset');

  let infoBlock =
    `<table class='form-text' style='border:0px solid;'>
       <tr>
         <td>All Datasets:</td>
         <td>This filter applies to all visuals in this dashboard.</td>
       </tr>`;
  infoBlock +=
    `<tr class="current_dataset">
       <td>Current Dataset:</td>
       <td>This filter applies to only those visuals in this dashboard
           that use the same dataset.</td>
     </tr>`;

  infoBlock +=
    `<tr>
       <td>Explicit:</td>
       <td>Automatic filtering is turned off.</td>
     </tr>
   </table>`;

  setting.add_name('false')
         .add_label('Explicit')
         .infoblock(infoBlock)
         .value('GLOBAL');
  return setting;
};

arc.report.user_settings.filter_appgroup_scope = function() {
  return arc.visual.setting()
            .id('filter_appgroup_scope')
            .tab('Scope')
            .type(arc.visual.settingTypeEnum.RADIO)
            .group('Set Filter Visibility to: ')
            .add_name('false')
            .add_label('Sheet')
            .add_name('true')
            .add_label('App')
            .value('false');
};

arc.report.user_settings.permit_png_download = function() {
  const allowPNGDownloadDefault = arc.globals.allow_png_download_default ?
                                'true' : 'false';

  const pngSetting = arc.visual.setting()
                        .id('permit_png_download')
                        .tab('Downloads')
                        .type(arc.visual.settingTypeEnum.CHECKBOX)
                        .add_label('Enable "Download as Image/PDF"')
                        .hidden(!arc.globals.enable_thumbnails)
                        .value(allowPNGDownloadDefault);

  return pngSetting;
};

arc.report.user_settings.print_pdf_scale = function() {
  const scaleDefault = '0';

  const printPDFScaleSetting = arc.visual.setting()
                        .id('print_pdf_scale')
                        .tab('Downloads')
                        .type(arc.visual.settingTypeEnum.INTEGER)
                        .add_name('PRINT_PDF_SCALE')
                        .add_label('Print PDF Scale (0 = auto)')
                        .min(0)
                        .max(200)
                        .hidden(!arc.globals.chromium_pdf_available)
                        .value(scaleDefault);

  return printPDFScaleSetting;
};

arc.report.user_settings.pdf_size = function() {
  const pdfSizeMap = {
    letter: 'Letter',
    A4: 'A4',
  };

  const span = '<span class="arc-pdf-size-';
  const s = arc.visual.setting()
             .id('print_pdf_size')
             .tab('Downloads')
             .type(arc.visual.settingTypeEnum.DROPDOWN)
             .group('PDF Page Size')
             .hidden(!arc.globals.chromium_pdf_available)
             .value(pdfSizeMap.letter);

  Object.keys(pdfSizeMap).forEach((size) => {
    s.add_name(size)
     .add_label(`${span + size}">${
                 pdfSizeMap[size]}</span>`);
  });

  s.mapPDFSizeToDisplayName = function(size) {
    return pdfSizeMap[size];
  };

  s.valueToClass = function() {
    const v = s.value();
    return `arc-pdf-size-${v}`;
  };

  return s;
};

arc.report.user_settings.pdf_orientation = function() {
  const pdfOrientationMap = {
    portrait: 'Portrait',
    landscape: 'Landscape',
  };

  const span = '<span class="arc-pdf-orientation-';
  const s = arc.visual.setting()
             .id('print_pdf_orientation')
             .tab('Downloads')
             .type(arc.visual.settingTypeEnum.DROPDOWN)
             .group('PDF Page Orientation')
             .hidden(!arc.globals.chromium_pdf_available)
             .value(pdfOrientationMap.portrait);

  Object.keys(pdfOrientationMap).forEach((orientation) => {
    s.add_name(orientation)
     .add_label(`${span + orientation}">${
                 pdfOrientationMap[orientation]}</span>`);
  });

  s.mapPDFOrientationToDisplayName = function(orientation) {
    return pdfOrientationMap[orientation];
  };

  s.valueToClass = function() {
    const v = s.value();
    return `arc-pdf-orientation-${v}`;
  };

  return s;
};

arc.report.user_settings.pdf_enable_header_footer = function() {
  return arc.visual.setting()
             .id('print_pdf_enable_header_footer')
             .tab('Downloads')
             .type(arc.visual.settingTypeEnum.CHECKBOX)
             .add_name('ENABLE_PDF_HEADER_FOOTER')
             .add_label('Enable PDF Header and Footer')
             .hidden(!arc.globals.chromium_pdf_available)
             .value(true);
};

arc.report.user_settings.permit_email = function() {
  const emailAvailable = arc.globals.email_enabled && arc.globals.enable_thumbnails;

  const emailSetting = arc.visual.setting()
                        .id('permit_email_report')
                        .tab('Downloads')
                        .type(arc.visual.settingTypeEnum.CHECKBOX)
                        .add_label('Enable email')
                        .hidden(!emailAvailable)
                        .add_postpaint_fn(() => {
                          emailSetting.toggleEmailDropdown();
                        })
                        .addChangeCb('updateEmailDropdown', () => {
                          emailSetting.toggleEmailDropdown();
                        })
                        .value('true');

  emailSetting.toggleEmailDropdown = function() {
    if (!emailSetting.hidden()) {
      $('.email-visual').toggleClass('arc-invisible', !emailSetting.isChecked);
    } else if (!emailAvailable) {
      $('.email-visual').addClass('arc-invisible');
    }
  };

  return emailSetting;
};

arc.report.user_settings.permit_csv_download_visual = function() {
  const allowCSVDownloadDefault = arc.globals.allow_csv_download_default ?
                                'true' : 'false';
  return arc.visual.setting()
            .id('permit_csv_download_visual')
            .tab('Downloads')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Enable "Download as CSV/Excel"')
            .value(allowCSVDownloadDefault);
};

arc.report.user_settings.csv_download_locale = function() {
  const csvSetting = arc.visual.setting()
                        .id('default_language')
                        .tab('Downloads')
                        .type(arc.visual.settingTypeEnum.CUSTOM)
                        .add_name('DEFAULT_LANGUAGE')
                        .add_label('Set "Download as CSV" Locale')
                        .value('');

  // specialty function that returns back object even if value empty
  csvSetting.getValue = function () {
    return csvSetting.value() === '' ? { value: '', label: '' } :
                                       JSON.parse(csvSetting.value());
  };

  let csvSelect;

  // Default countries
  const majorAutocompleteOptions = [
    { value: 'zh-CN', label: 'China', major: true },
    { value: 'de-DE', label: 'Germany', major: true },
    { value: 'nb-NO', label: 'Norway', major: true },
    { value: 'sv-SE', label: 'Sweden', major: true },
    { value: 'en-GB', label: 'United Kingdom', major: true },
    { value: 'en-US', label: 'United States', major: true },
  ];

  // Additional ones
  const autocompleteOptions = [
    { value: 'sq-AL', label: 'Albania' },
    { value: 'ar-DZ', label: 'Algeria' },
    { value: 'es-AR', label: 'Argentina' },
    { value: 'hy-AM', label: 'Armenia' },
    { value: 'en-AU', label: 'Australia' },
    { value: 'de-AT', label: 'Austria' },
    { value: 'az-AZ', label: 'Azerbaijan' },
    { value: 'ar-BH', label: 'Bahrain' },
    { value: 'be-BY', label: 'Belarus' },
    { value: 'fr-BE', label: 'Belgium' },
    { value: 'en-BZ', label: 'Belize' },
    { value: 'es-BO', label: 'Bolivia' },
    { value: 'qu-BO', label: 'Bolivia' },
    { value: 'bs-BA', label: 'Bosnia and Herzegovina' },
    { value: 'en', label: 'Botswana' },
    { value: 'pt-BR', label: 'Brazil' },
    { value: 'en', label: 'British Virgin Islands' },
    { value: 'en', label: 'British West Indies' },
    { value: 'ms-BN', label: 'Brunei Darussalam' },
    { value: 'bg-BG', label: 'Bulgaria' },
    { value: 'en-CA', label: 'Canada (English)' },
    { value: 'fr-CA', label: 'Canada (French)' },
    { value: 'en-CB', label: 'Caribbean' },
    { value: 'es-CL', label: 'Chile' },
    { value: 'es-CO', label: 'Colombia' },
    { value: 'es-CR', label: 'Costa Rica' },
    { value: 'hr-HR', label: 'Croatia' },
    { value: 'cs-CZ', label: 'Czech Republic' },
    { value: 'da-DK', label: 'Denmark' },
    { value: 'es-DO', label: 'Dominican Republic' },
    { value: 'es-EC', label: 'Ecuador' },
    { value: 'ar-EG', label: 'Egypt' },
    { value: 'es-SV', label: 'El Salvador' },
    { value: 'et-EE', label: 'Estonia' },
    { value: 'fo-FO', label: 'Faroe Islands' },
    { value: 'fi-FI', label: 'Finland' },
    { value: 'fr-FR', label: 'France' },
    { value: 'ka-GE', label: 'Georgia' },
    { value: 'en', label: 'Ghana' },
    { value: 'el-GR', label: 'Greece' },
    { value: 'es-GT', label: 'Guatemala' },
    { value: 'es-HN', label: 'Honduras' },
    { value: 'zh-HK', label: 'Hong Kong' },
    { value: 'hu-HU', label: 'Hungary' },
    { value: 'is-IS', label: 'Iceland' },
    { value: 'hi-IN', label: 'India' },
    { value: 'id-ID', label: 'Indonesia' },
    { value: 'ar-IQ', label: 'Iraq' },
    { value: 'en-IE', label: 'Ireland' },
    { value: 'ur-PK', label: 'Islamic Republic of Pakistan' },
    { value: 'he-IL', label: 'Israel' },
    { value: 'it-IT', label: 'Italy' },
    { value: 'en-JM', label: 'Jamaica' },
    { value: 'ja-JP', label: 'Japan' },
    { value: 'ar-JO', label: 'Jordan' },
    { value: 'kk-KZ', label: 'Kazakhstan' },
    { value: 'sw-KE', label: 'Kenya' },
    { value: 'ko-KR', label: 'Korea' },
    { value: 'ar-KW', label: 'Kuwait' },
    { value: 'ky-KG', label: 'Kyrgyzstan' },
    { value: 'lv-LV', label: 'Latvia' },
    { value: 'ar-LB', label: 'Lebanon' },
    { value: 'ar-LY', label: 'Libya' },
    { value: 'lt-LT', label: 'Lithuania' },
    { value: 'pt', label: 'Macau (Portuguese)' },
    { value: 'zh-MO', label: 'Macau (Chinese)' },
    { value: 'mk-MK', label: 'Macedonia' },
    { value: 'ms-MY', label: 'Malaysia' },
    { value: 'mt-MT', label: 'Malta' },
    { value: 'es-MX', label: 'Mexico' },
    { value: 'mn-MN', label: 'Mongolia' },
    { value: 'ar-MA', label: 'Morocco' },
    { value: 'nl-NL', label: 'Netherlands' },
    { value: 'en-NZ', label: 'New Zealand' },
    { value: 'es-NI', label: 'Nicaragua' },
    { value: 'ar-OM', label: 'Oman' },
    { value: 'es-PA', label: 'Panama' },
    { value: 'es-PY', label: 'Paraguay' },
    { value: 'es-PE', label: 'Peru' },
    { value: 'tl-PH', label: 'Philippines' },
    { value: 'pl-PL', label: 'Poland' },
    { value: 'pt-PT', label: 'Portugal' },
    { value: 'es-PR', label: 'Puerto Rico' },
    { value: 'ar-QA', label: 'Qatar' },
    { value: 'en-PH', label: 'Republic of the Philippines' },
    { value: 'ro-RO', label: 'Romania' },
    { value: 'ru-RU', label: 'Russia' },
    { value: 'ar-SA', label: 'Saudi Arabia' },
    { value: 'sr-SP', label: 'Serbia and Montenegro' },
    { value: 'zh-SG', label: 'Singapore' },
    { value: 'sk-SK', label: 'Slovakia' },
    { value: 'sl-SI', label: 'Slovenia' },
    { value: 'af-ZA', label: 'South Africa' },
    { value: 'es-ES', label: 'Spain' },
    { value: 'ar-SY', label: 'Syria' },
    { value: 'zh-TW', label: 'Taiwan' },
    { value: 'th-TH', label: 'Thailand' },
    { value: 'en-TT', label: 'Trinidad and Tobago' },
    { value: 'ar-TN', label: 'Tunisia' },
    { value: 'tr-TR', label: 'Turkey' },
    { value: 'ar-AE', label: 'U.A.E.' },
    { value: 'uk-UA', label: 'Ukraine' },
    { value: 'es-UY', label: 'Uruguay' },
    { value: 'uz-UZ', label: 'Uzbekistan' },
    { value: 'es-VE', label: 'Venezuela' },
    { value: 'vi-VN', label: 'Viet Nam' },
    { value: 'ar-YE', label: 'Yemen' },
    { value: 'en-ZW', label: 'Zimbabwe' },
  ];

  function paintSetting(tab_d3_sel) {
    const divId = csvSetting.div_id();

    // Create new setting container
    const csvSettingDiv = tab_d3_sel.append('div')
                              .attr('id', divId)
                              .classed('single-select', true);

    if (csvSetting.hidden()) {
      csvSettingDiv.classed('arc-invisible', true);
    }

    // Add label
    csvSettingDiv.append('label')
                 .text('Set "Download as CSV" Locale');

    // Get previously saved value
    const savedItem = csvSetting.value();

    // Add select element
    csvSelect = csvSettingDiv.append('select')
                              .attr({
                                name: csvSetting.names()[0],
                                id: `${divId}-language`,
                              });

    // Add empty option
    csvSelect.append('option');

    // Add major options
    addOptions(csvSelect, majorAutocompleteOptions);

    // Add optgroup
    const optgroup = csvSelect.append('optgroup')
                              .attr('label', 'Other Locales');

    // Add additional options
    addOptions(optgroup, autocompleteOptions);

    // select correct value
    csvSelect.node().value = savedItem;

    // Initialize chosen
    if (arc.globals.is_edit || arc.globals.screen_appgroup) {
      $(csvSelect.node()).chosen({
        allow_single_deselect: true,
        placeholder_text_single: `Default language: ${arc.misc.getLanguage()}`,
      }).offOn('change.csv', (e) => {
        e.stopPropagation();
        e.preventDefault();
        csvSetting.value(e.currentTarget.value);
        csvSetting.makeChangeCbs();
      });
    }

    function addOptions(container, options) {
      container.selectAll('.nosel')
               .data(options)
               .enter()
                .append('option')
                .attr('value', (d) => JSON.stringify(d))
                .text((d) => d.label);
    }
  }

  function fetchSetting() {
    return csvSelect.node().value;
  }

  csvSetting.paint_fn(paintSetting);
  csvSetting.fetch_fn(fetchSetting);

  return csvSetting;
};

arc.report.user_settings.permit_csv_download_dashboard = function() {
  const allowCSVDownloadDefault = arc.globals.allow_csv_download_default ?
                                'true' : 'false';
  return arc.visual.setting()
            .id('permit_csv_download_dashboard')
            .tab('Downloads')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Enable "Download as CSV/Excel" for visuals')
            .value(allowCSVDownloadDefault);
};

arc.report.user_settings.csvQuotationMark = function() {
  return arc.visual.setting()
            .id('csv_download_quotation')
            .tab('Downloads')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Quote numeric strings in CSV downloads')
            .add_name('CSV_SINGLE_QUOTE_NUMERIC')
            .info('Check this box to use single quotes around numeric strings. ' +
                'Otherwise, opening in Excel will strip leading zeros from numeric strings.')
            .value('false');
};

arc.report.user_settings.menu_on_left = function() {
  return arc.visual.setting()
            .id('menu_on_left')
            .section('Style')
            .tab('Menu Bar')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Display menu on left')
            .value('false');
};

arc.report.user_settings.navigate_to_apps_page = function() {
  return arc.visual.setting()
            .id('navigate_to_apps_page')
            .tab('General')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Enable navigation to the Visuals page on click of the logo in the menu bar')
            .value('false');
};

arc.report.user_settings.app_url_name = function() {
  return arc.visual.setting()
            .id('app_url_name')
            .tab('General')
            .type(arc.visual.settingTypeEnum.STRING)
            .add_name('APP_URL_NAME')
            .add_label('App URL name')
            .value('');
};

arc.report.user_settings.trendline_linear = function() {
  return arc.visual.setting()
            .id('trendline_linear')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Display linear trendline')
            .value('false');
};

arc.report.user_settings.trendline_color = function() {
  return arc.visual.setting()
            .id('trendline_color')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Separate trendlines by color')
            .value('false');
};

arc.report.user_settings.trendline_tooltip_decimal = function() {
  return arc.visual.setting()
            .id('trendline_decimal')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.SLIDER)
            .add_name('TRENDLINE_DECIMAL')
            .add_label('Trendline Decimals')
            .info('Decimals included when displaying the trendline ' +
                'equation and r-square value.')
            .min(0)
            .max(10)
            .value('3');
};

arc.report.user_settings.x_axis_label = function() {
  return arc.visual.setting()
            .id('x_axis_label')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Include X Axis label')
            .value('true');
};

arc.report.user_settings.y_axis_label = function() {
  return arc.visual.setting()
            .id('y_axis_label')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Include Y Axis label')
            .value('true');
};

arc.report.user_settings.dim_tick_label_maxlen = function() {
  const s = arc.visual.setting();

  s.id('dim_tick_label_maxlen')
   .tab('Axes')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('DIM_TICK_TEXT_MAXLEN')
   .add_label('Max Tick Length')
   .min(0)
   .max(500)
   .value('20');
  return s;
};

arc.report.user_settings.dynamic_bar_width = function() {
  return arc.visual.setting()
            .id('dynamic_bar_width')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Dynamically size graph to window size')
            .info('Takes effect when dimension is on x axis.')
            .value('false');
};

arc.report.user_settings.tooltip_percentage = function() {
  return arc.visual.setting()
            .id('tooltip_percentage')
            .tab('Tooltip')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Include percentage of total in tooltip')
            .value('true');
};

arc.report.user_settings.tooltip_percentage_name = function() {
  return arc.visual.setting()
            .id('tooltip_percentage_name')
            .tab('Tooltip')
            .type(arc.visual.settingTypeEnum.STRING)
            .add_name('PERCENTAGE_NAME')
            .add_label('Label for percentage')
            .value('percentage');
};

arc.report.user_settings.tooltip_percentage_decimal = function() {
  const s = arc.visual.setting();

   s.id('tooltip_percentage_decimal')
    .tab('Tooltip')
    .type(arc.visual.settingTypeEnum.INTEGER)
    .add_name('PERCENTAGE_DECIMALS')
    .add_label('Decimals in %')
    .min(0)
    .max(10)
    .value('2');

  return s;
};

arc.report.user_settings.sparklines_min = function() {
  return arc.visual.setting()
            .id('sparklines_min')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Mark minimum value')
            .value('false');
};

arc.report.user_settings.sparklines_max = function() {
  return arc.visual.setting()
            .id('sparklines_max')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Mark maximum value')
            .value('false');
};

arc.report.user_settings.sparklines_first = function() {
  return arc.visual.setting()
            .id('sparklines_first')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Mark first value')
            .value('false');
};

arc.report.user_settings.sparklines_last = function() {
  return arc.visual.setting()
            .id('sparklines_last')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Mark last value')
            .value('false');
};

arc.report.user_settings.bar_mark_size = function() {
  const s = arc.visual.setting();

  s.id('bar_mark_size')
   .tab('Marks')
   .type(arc.visual.settingTypeEnum.SLIDER_RANGE)
   .add_name('BAR_MARK_SIZE_RANGE')
   .add_label('Bar size range')
   .info('Minimum size bar - maximum size bar in pixels.')
   .min(1)
   .max(200)
   .value('20-30');

  return s;
};

arc.report.user_settings.mark_size_range = function() {
  const s = arc.visual.setting();

  s.id('mark_size_range')
   .tab('Marks')
   .type(arc.visual.settingTypeEnum.SLIDER_RANGE)
   .add_name('MARK_SIZE_RANGE')
   .add_label('Mark size range')
   .validate_fn((val) => s.validate_num_range(val, 1, 200))
   .min(1)
   .max(100)
   .value('10-50');

  return s;
};

arc.report.user_settings.colors_enabled = function() {
  return arc.visual.setting()
            .id('colors_enabled')
            .tab('Color')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Enable colors on all aggregates')
            .value('false');
};

arc.report.user_settings.show_points = function() {
  return arc.visual.setting()
            .id('show_points')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Show points')
            .value('false');
};

arc.report.user_settings.scale_log10_percentage = function() {
  const s = arc.visual.setting()
            .id('scale_type')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.DROPDOWN)
            .group('Axis Scale')
            .add_name('LINEAR')
            .add_label('Linear')
            .add_name('LOG10')
            .add_label('log<sub>10</sub>')
            .add_name('PERCENTAGE')
            .add_label('Percentage of dimension')
            .add_name('PERCENTAGE_COLOR')
            .add_label('Percentage of color')
            .setDisableTooltipText('If a secondary axis is enabled, only linear scale is available')
            .value('LINEAR');

  return s;
};

arc.report.user_settings.scale_log10 = function() {
  return arc.visual.setting()
            .id('scale_type')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.RADIO)
            .group('Axis Scale')
            .add_name('LINEAR')
            .add_label('Linear')
            .add_name('LOG10')
            .add_label('log<sub>10</sub>')
            .value('LINEAR');
};

arc.report.user_settings.scale_percentage = function() {
  return arc.visual.setting()
            .id('scale_type')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.DROPDOWN)
            .group('Axis Scale')
            .add_name('LINEAR')
            .add_label('Linear')
            .add_name('PERCENTAGE')
            .add_label('Percentage')
            .value('LINEAR');
};

arc.report.user_settings.table_cell_padding = function() {
  const s = arc.visual.setting();

  s.id('table_cell_padding')
   .tab('Table Style')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_label('Table Cell Padding')
   .min(0)
   .max(20)
   .value('5')
   .valueBeforeRestore('8');

  return s;
};

arc.report.user_settings.axes_padding = function() {
  const s = arc.visual.setting();

  s.id('axes_padding')
   .tab('Axes')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_name('AXES_PADDING')
   .add_label('Axes Padding (% range)')
   .min(0)
   .max(100)
   .value('5');

  return s;
};

arc.report.user_settings.x_axis_padding = function() {
  const s = arc.visual.setting();

  s.id('x_axis_padding')
   .tab('Axes')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('X_AXIS_PADDING')
   .add_label('X Axis Padding (%)')
   .min(0)
   .max(100)
   .value('5');

  return s;
};

arc.report.user_settings.y_axis_padding = function() {
  const s = arc.visual.setting();

  s.id('y_axis_padding')
   .tab('Axes')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('Y_AXIS_PADDING')
   .add_label('Y Axis Padding (%)')
   .min(0)
   .max(100)
   .value('5');

  return s;
};

arc.report.user_settings.color_opacity = function() {
  const s = arc.visual.setting();

  function setPaletteOpacity(v) {
    const lib = s.viz_lib();
    if (lib) {
      let c = lib.user_settings().find_by_id('color_palette');
      if (c) {
        c.setOpacity(Number(v));
      }

      c = lib.user_settings().find_by_id('dim_to_color_mapping');
      if (c) {
        c.setOpacity(Number(v));
      }

      c = lib.user_settings().find_by_id('measure_ranges');
      if (c) {
        c.setOpacity(Number(v));
      }
    }
  }

  s.id('color_opacity')
   .tab('Color')
   .section('Color')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('COLOR_OPACITY')
   .add_label('Color Opacity')
   .min(0)
   .max(100)
   .step(5)
   .value('80')
   .valueCb(() => {
    setPaletteOpacity(s.value());
   })
   .add_postpaint_fn(() => {
     setPaletteOpacity(s.value());
     s.changeCb('palette', () => {
       setPaletteOpacity(s.fetch_value());
     });
   });

  return s;
};

arc.report.user_settings.legend_visibility = function() {
    const s = arc.visual.setting();

    s.id('legend_visibility')
        .tab('Marks')
        .type(arc.visual.settingTypeEnum.CHECKBOX)
        .add_label('Show Legend')
        .value('true');
    return s;
};

arc.report.user_settings.legend_style = function() {
  const setting = arc.visual.setting()
                   .id('legend_style')
                   .tab('Marks')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('Legend_Style')
                   .add_label('Legend Style:')
                   .value('Right');

  const legendIconData = [
    { label: 'None', style: '' },
    { label: 'Right', style: 'border-right: double 2px black' },
    { label: 'Bottom', style: 'border-bottom: double 2px black' },
    { label: 'Above', style: 'border-top: double 2px black' },
  ];

  const paint_setting = function(tab_d3_sel) {
    const d3_setting_div =
      tab_d3_sel.append('div')
                .attr('data-setting-id', setting.id())
                .attr('id', setting.div_id());

    d3_setting_div.append('label')
              .text(setting.labels()[0]);

    const currentVal = setting.value();

    d3_setting_div.selectAll('.nosel')
              .data(legendIconData)
              .enter()
              .append('div')
                .classed('legend-style-button-container', true)
                .append('div')
                  .classed('legend-style-button', true)
                  .classed('legend-style-selected', (d) => d.label === currentVal)
                  .append('div')
                    .attr('class', (d) => `legend-style-icon legend-style-${
                      d.label.toLowerCase()}`)
                    .attr('data-test-id', (d) => `legend-style-${d.label.toLowerCase()}`)
                    .attr('style', (d) => d.style);

    d3_setting_div.selectAll('.legend-style-button-container')
      .append('p')
      .text((d) => d.label);

    setting.valueCb((value) => {
      d3_setting_div.selectAll('.legend-style-button')
                    .classed('legend-style-selected', (d) => d.label === value);
    });

    d3_setting_div.selectAll('.legend-style-button')
      .on('click', function(d) {
        const value = d.label;
        if (value !== setting.value()) {
          setting.value(value);
          d3_setting_div.selectAll('.legend-style-button')
                        .classed('legend-style-selected', false);
          d3.select(this).classed('legend-style-selected', true);
          setting.viz_lib().page_needs_save();
          setting.makeChangeCbs();
        }
      });
  };

  const fetch_setting = function() {
    const s = d3.select(`#${setting.div_id()} .legend-style-selected`);
    let label;
    s.data().forEach((d) => {
      if (d) {
        label = d.label;
      }
    });

    return label;
  };

  setting.paint_fn(paint_setting);
  setting.fetch_fn(fetch_setting);

  setting.legendRequired = function() {
    return setting.value() !== 'None';
  };

  return setting;
};

arc.report.user_settings.legend_width_override = function() {
  const s = arc.visual.setting();

  s.id('legend_width_override')
   .tab('Marks')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_name('LEGEND_WIDTH_OVERRIDE')
   .add_label('Legend item width (px)')
   .min(0)
   .max(1024)
   .value('');

  return s;
};

arc.report.user_settings.reverse_legend_order = function() {
    const s = arc.visual.setting();

    s.id('reverse_legend_order')
        .tab('Marks')
        .type(arc.visual.settingTypeEnum.CHECKBOX)
        .add_label('Reverse legend order')
        .value('false');

    return s;
};

arc.report.user_settings.fixed_color_legend = function() {
  const s = arc.visual.setting();

  s.id('fixed_color_legend')
   .section('Color')
   .tab('Color')
   .type(arc.visual.settingTypeEnum.CHECKBOX)
   .add_label('Map the same colors to dimension values across visuals')
   .info('Color settings for a specific value are fixed for ' +
             'all visuals using the color palette.')
   .value('false');
    return s;
};

arc.report.user_settings.transition_delay = function() {
  const s = arc.visual.setting();

  s.id('transition_delay')
   .tab('Marks')
   .type(arc.visual.settingTypeEnum.INTEGER_NOSPINNER)
   .add_name('TRANSITION_DELAY')
   .add_label('Transition Delay (ms)')
   .min(10)
   .max(5000)
   .value('1000');

  return s;
};

arc.report.user_settings.horizontal_gridlines_enabled = function() {
  return arc.visual.setting()
  // To avoid resource consuming migration it is not renamed to horizontal_gridlines_enabled
            .id('gridlines_enabled')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Enable Horizontal Gridlines')
            .info('They are not drawn if secondary axis is set')
            .value('false');
};

arc.report.user_settings.vertical_gridlines_enabled = function() {
  return arc.visual.setting()
            .id('vertical_gridlines_enabled')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Enable Vertical Gridlines')
            .value('false');
};

arc.report.user_settings.autoplay_animation = function() {
  return arc.visual.setting()
            .id('autoplay')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Autoplay transition animation')
            .value('false');
};

arc.report.user_settings.scale_dim_independent = function() {
  return arc.visual.setting()
            .id('scale_dim_independent')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Independent dimension scale across trellis')
            .value('false');
};

arc.report.user_settings.scale_agg_independent = function() {
  return arc.visual.setting()
            .id('scale_agg_independent')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Independent aggregate scale across trellis')
            .value('false');
};

arc.report.user_settings.scale_agg_zero = function() {
  return arc.visual.setting()
            .id('scale_agg_zero')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Positive aggregate scales start at zero')
            .value('true');
};

arc.report.user_settings.scale_independent = function() {
  return arc.visual.setting()
            .id('scale_type_independent')
            .tab('Trellis')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Independent scales')
            .value('false');
};

arc.report.user_settings.scale_type_radial = function() {
  return arc.visual.setting()
            .id('scale_type_radial')
            .tab('Axes')
            .type(arc.visual.settingTypeEnum.RADIO)
            .group('Axis Scale')
            .add_name('ONEMEASURE')
            .add_label('Measure values compare to values in the same ' +
                'measure across all dimensions')
            .add_name('ALLMEASURES')
            .add_label('Measure values compare to values across all measures')
            .add_name('ONEDIMENSION')
            .add_label('Measure values compare to other measure values in ' +
                'the same dimension')
            .value('ONEMEASURE');
};

arc.report.user_settings.overlapping_marks = function() {
  return arc.visual.setting()
            .id('overlapping_marks')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Overlap marks')
            .value('false');
};

arc.report.user_settings.nulls_order = function() {
  return arc.visual.setting()
            .id('nulls_order')
            .tab('Data')
            .type(arc.visual.settingTypeEnum.RADIO)
            .group('Show data with NULL values')
            .info('Available for Arcadia/ Impala/ PostgreSQL connections.')
            .add_name('Default')
            .add_label('Default')
            .add_name('First')
            .add_label('First')
            .add_name('Last')
            .add_label('Last')
            .value('Default');
};

// Setting used in table and crosstab visual
arc.report.user_settings.settingFullWidth = function() {
  let $table_element = $();

  const setting = arc.visual.setting()
                   .id('table_full_width')
                   .section('Style')
                   .tab('Table Style')
                   .type(arc.visual.settingTypeEnum.CHECKBOX)
                   .add_label('Full table width')
                   .value('false');

  setting.$table_element = function(_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  setting.paint_table_style = function() {
    // TODO: Override this function
    return setting;
  };

  return setting;
};

arc.report.settings_tab_unique_id = 1;

arc.report.settingsModal = function() {
  let settings_modal_id;
  let settings_modal_title = 'Settings';
  let settings_modal_holder = 'settings-modal-holder';
  let viz_lib;
  let large;
  let user_settings;
  let existingValues = {};

  const settings = function() { };

  settings.settings_modal_id = function(_) {
    if (!arguments.length) return settings_modal_id;
    settings_modal_id = _;
    return settings;
  };

  settings.settings_modal_holder = function(_) {
    if (!arguments.length) return settings_modal_holder;
    settings_modal_holder = _;
    return settings;
  };

  settings.settings_modal_title = function(_) {
    if (!arguments.length) return settings_modal_title;
    settings_modal_title = _;
    return settings;
  };

  settings.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    settings_modal_id = `settings-modal-${viz_lib.unique_report_id()}`;
    return settings;
  };

  settings.user_settings = function(_) {
    if (!arguments.length) return user_settings;
    user_settings = _;
    return settings;
  };

  settings.large = function(_) {
    if (!arguments.length) return large;
    large = _;
    return settings;
  };

  settings.setExistingValue = function (settingId, value) {
    if (settingId in existingValues) {
      existingValues[settingId] = value;
    }
  };

  settings.drawUserSettings = function() {
    if (!user_settings) {
      return settings;
    }

    const tabs = [];
    const tabs_h = {};
    const tab_settings = {};
    user_settings.organizeByTabs(tabs, tabs_h, tab_settings);

    // Now, draw each tab
    tabs.forEach((tab) => {
      // See if all settings within this tab are hidden
      let allHidden = true;
      tab_settings[tab].forEach((setting) => {
        if (!setting.hidden()) {
          allHidden = false;
        }
      });

      if (allHidden) {
        // Paint hidden settings
        tab_settings[tab].forEach((user_setting) => {
          user_setting.viz_lib(viz_lib);
          user_setting.paintControl(d3.select(`#${settings_modal_id} .hidden-settings`));
        });

        return;
      }

      // Draw the panel header
      d3.select(`#${settings_modal_id}`)
        .select('.nav-tabs')
          .append('li')
          .classed('nav-item', true)
          .attr('id', `settings-user-${tabs_h[tab]}`)
            .append('a')
            .classed('nav-link', true)
            .attr('href', `#settings-user-content-${tabs_h[tab]}`)
            .attr('data-toggle', 'tab')
            .attr('data-target', `#settings-user-content-${tabs_h[tab]}`)
            .text(tab);

      // Now the panel
      const tab_div =
        d3.select(`#${settings_modal_id}`)
          .select('.tab-content')
            .append('div')
            .classed('tab-pane', true)
            .classed('viz-settings-tab-pane', true)
            .attr('id', `settings-user-content-${tabs_h[tab]}`)
            .attr('data-test-id', 'settings-user-content');

      // Finally, each setting
      tab_settings[tab].forEach((user_setting) => {
        user_setting.viz_lib(viz_lib);
        user_setting.paintControl(tab_div);
      });
    });

    return settings;
  };

  let showingEmpty = false;

  settings.hide = function() {
    const $m = $(`#${settings_modal_id}`);
    $m.modal('hide');
  };

  settings.showEmpty = function() {
    const $m = $(`#${settings_modal_id}`);
    if (arc.misc.modalShown($m)) {
      return;
    }

    showingEmpty = true;
    $m.modal('show');
    $m.off('hidden.bs.modal')
      .on('hidden.bs.modal', () => {
        setTimeout(() => {
          $(`#${settings_modal_id}`).remove();
        }, 0);
      });

    $m.find('.modal-body-loading')
      .removeClass('arc-invisible');

    return settings;
  };

  function setupShownCb() {
    const $m = $(`#${settings_modal_id}`);

    // When a particular tab shows, call its 'shown' function
    $m.find('.nav-tabs a[data-toggle="tab"]').off('shown.bs.tab')
                                             .on('shown.bs.tab',
        function() {
          const tab = $(this).text();

          user_settings.settings().forEach((user_setting) => {
            if (user_setting.tab() === tab) {
              user_setting.shown();
            }
          });
        });

    $m.off('shown.bs.modal')
      .on('shown.bs.modal', () => {
        if (user_settings) {
          user_settings.settings().forEach((user_setting) => {
            user_setting.shown();
          });
        }
      });
  }

  settings.show = function() {
    const $m = $(`#${settings_modal_id}`);
    if (arc.misc.modalShown($m)) {
      return;
    }

    setupShownCb();

    $m.find('.nav-tabs li:first a').tab('show');
    $m.modal('show');

    return settings;
  };

  settings.drawModal = function() {
    let $m = $(`#${settings_modal_id}`);
    if (showingEmpty) {
      $m.find('.modal-body-loading')
        .addClass('arc-invisible');
      return settings;
    }

    $m.remove();
    const h = arc.misc.safe_substitute(
        $('#settings-modal-template').html(),
        {
          SETTINGS_MODAL_TITLE: settings_modal_title,
          SETTINGS_MODAL_ID: settings_modal_id,
          SETTINGS_MODAL_DIALOG_CLASS: (large ? 'modal-lg' : ''),
        }
    );

    $(`#${settings_modal_holder}`).html(h);
    $m = $(`#${settings_modal_id}`);
    arc.misc.handleAlertContainerActivationOnModal($m);
    return settings;
  };

  function saveCb(fn) {
    const $settings_form = $(`#${settings_modal_id}`).find('form');
    $settings_form.on('submit.settings', fn);
    $settings_form.on('keyup.settings', '[contenteditable=true]', (e) => {
                    if (e.which === arc.misc.ENTER_KEY) {
                      e.preventDefault();
                      $settings_form.trigger('submit');
                    }
                  });

    return settings;
  }

  function cancelCb(fn) {
    const settings_modal = d3.select(`#${settings_modal_id}`);
    settings_modal.selectAll('.settings-modal-cancel')
                  .on('click.settings', fn);
    return settings;
  }

  settings.setup = function(cb) {
    settings.drawModal();
    const settings_modal = d3.select(`#${settings_modal_id}`);

    settings_modal.select('.settings-modal-colors')
                  .attr('id', 'settings-colors');

    existingValues = {};
    user_settings.settings().forEach((user_setting) => {
      existingValues[user_setting.id()] = user_setting.value();
    });

    cancelCb(
      () => {
        if (viz_lib) {
          viz_lib.restoreUserSettings(existingValues);

          if (viz_lib.redraw_on_settings_cancel()) {
            viz_lib.redraw();
          }
        }
      }
    );

    function afterVisualSave() {
      if (viz_lib) {
        viz_lib.page_needs_save();
      }

      arc.report.alertMessageAllHide();
      if (cb) {
        cb();
      }
    }

    saveCb(
      function() {
        if (viz_lib) {
          const newValues = {};
          user_settings.settings().forEach((user_setting) => {
            if (user_setting.process_value()) {
              newValues[user_setting.id()] = user_setting.value();
            }
          });

          viz_lib.saveVisualSettings(afterVisualSave, existingValues);
        } else {
          afterVisualSave();
        }

        $(this).closest('.modal').modal('hide');
      }
    );

    // Remove all tabs
    $(`#${settings_modal_id} .nav-tabs li`).remove();
    $(`#${settings_modal_id} .tab-content .tab-pane`).not(':first').remove();

    settings.drawUserSettings();
    setupShownCb();

    // Make the first tab active
    $(`#${settings_modal_id} .nav-tabs a:first`).tab('show');

    return settings;
  };

  return settings;
};
}());
