/*
 * This file contains settings for apps (formerly appgroups).
 */

window.arc = window.arc || {};

(function() {
'use strict';

arc.settings = arc.settings || {};
arc.settings.apps = arc.settings.apps || {};

arc.settings.apps.appShowTitle = function() {
  return arc.visual.setting()
            .id('app_show_title')
            .section('Style')
            .tab('Menu Bar')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('APP_HIDE_DROPSHADOW')
            .add_label('Always show the dashboard title')
            .info('This only affects menu tab with one dashboard')
            .value('false')
            .removeChangeCbs();
};

arc.settings.apps.appHideDropShadow = function() {
  return arc.visual.setting()
            .id('app_hide_dropshadow')
            .section('Style')
            .tab('App Background')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('APP_HIDE_DROPSHADOW')
            .add_label('Hide Dashboard drop shadow')
            .value('false');
};

arc.settings.apps.appMenuTextUppercase = function() {
  return arc.visual.setting()
            .id('app_menutext_uppercase')
            .section('Style')
            .tab('App Menu Text')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('APP_MENUTEXT_UPPERCASE')
            .add_label('Uppercase')
            .value('true');
};

arc.settings.apps.appLogoImage = function() {
  var s = arc.visual.setting();

  s.id('app_logo_image')
   .section('Style')
   .tab('Custom Logo')
   .type(arc.visual.settingTypeEnum.STRING)
   .add_name('APP_LOGO_IMAGE')
   .add_label('Logo Image URL')
   .placeholder('enter image url')
   .info('Upload image to static assets and enter URL here. An external ' +
         'image URL can also be used. Save App to apply.')
   .value('');

  return s;
};

arc.settings.apps.appLogoWidth = function() {
  return arc.visual.setting()
            .id('app_logo_width')
            .section('Style')
            .tab('Custom Logo')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('APP_LOGO_WIDTH')
            .add_label('Width')
            .suffix('px')
            .info('Overall logo image size cannot exceed 300px x 50px')
            .min(0)
            .step(5)
            .value(185);
};

arc.settings.apps.appLogoHeight = function() {
  return arc.visual.setting()
            .id('app_logo_height')
            .section('Style')
            .tab('Custom Logo')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('APP_LOGO_HEIGHT')
            .add_label('Height')
            .suffix('px')
            .info('Overall logo image size cannot exceed 300px x 50px')
            .min(0)
            .step(5)
            .value(45);
};

arc.settings.apps.appLogoMarginTop = function() {
  return arc.visual.setting()
            .id('app_logo_margintop')
            .section('Style')
            .tab('Custom Logo')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('APP_LOGO_MARGIN_TOP')
            .add_label('Top Margin')
            .suffix('px')
            .min(0)
            .step(5)
            .value(5);
};

arc.settings.apps.appLogoMarginLeft = function() {
  return arc.visual.setting()
            .id('app_logo_marginleft')
            .section('Style')
            .tab('Custom Logo')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('APP_LOGO_MARGIN_LEFT')
            .add_label('Left Margin')
            .suffix('px')
            .min(0)
            .step(5)
            .value(15);
};

}());
