window.arc = window.arc || {};

(function() {
'use strict';

arc.settings = arc.settings || {};
arc.settings.fonts = arc.settings.fonts || {};
arc.settings.dashboards = arc.settings.dashboards || {};

arc.settings.dashboards.dashboardHideTitle = function() {
  return arc.visual.setting()
            .id('dashboard_hide_title')
            .section('Style')
            .tab('Dashboard Title')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('DASHBOARD_HIDE_TITLE')
            .add_label('Hide title in view mode')
            .value('false');
};

arc.settings.dashboards.caseSensitiveParameters = function() {
  return arc.visual.setting()
            .id('dashboard_case_sensitive')
            .tab('Filter Settings')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('DASHBOARD_CASE_SENSITIVE')
            .add_label('Parameter names are case sensitive')
            .valueBeforeRestore('false')
            .value('true');
};

arc.settings.dashboards.displayViewData = function() {
  return arc.visual.setting()
            .id('dashboard_display_view_data')
            .tab('General')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('DASHBOARD_DISPLAY_VIEW_DATA')
            .add_label('Enable data detail in visuals')
            .valueBeforeRestore('false')
            .value('true');
};

arc.settings.dashboards.width = function() {
  var s = arc.visual.setting();

  s.id('dashboard_width')
   .section('Style')
   .tab('Dashboard Basics')
   .type(arc.visual.settingTypeEnum.STRING)
   .add_name('DASHBOARD_WIDTH')
   .add_label('Dashboard Width')
   .infoblock('Clear field for full width.')
   .validate_fn(function(val) {
     if (val.trim() === '') {
       return true;
     }

     return s.validate_num_range(val, 500, 10000);
   })
   .valueBeforeRestore('')
   .value(arc.globals_init().dashboard_width() + '');

  return s;
};

arc.settings.dashboards.visualsAfterFilters = function() {
  return arc.visual.setting()
            .id('visuals_after_filters')
            .section('Style')
            .tab('Filter Settings')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('VISUALS_AFTER_FILTERS')
            .add_label('Render visuals after loading dashboard filters')
            .value('true');
};

arc.settings.dashboards.enableAnimations = function() {
  return arc.visual.setting()
            .id('dashboard_enable_animations')
            .tab('General')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('DASHBOARD_ENABLE_ANIMATIONS')
            .add_label('Enable animations')
            .value('true');
};

arc.settings.dashboards.manualVisualRefresh = function() {
  return arc.visual.setting()
            .id('manual_visual_refresh')
            .tab('General')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('MANUAL_VISUAL_REFRESH')
            .add_label('Refresh visuals manually')
            .info('Adds a Refresh button to the dashboard.')
            .value('false');
};

arc.settings.dashboards.dashboardURLName = function() {
  return arc.visual.setting()
            .id('dashboard_url_name')
            .tab('General')
            .type(arc.visual.settingTypeEnum.STRING)
            .add_name('DASHBOARD_URL_NAME')
            .add_label('Dashboard URL name')
            .value('');
};

arc.settings.dashboards.singleVisualDashboard = function() {
  return arc.visual.setting()
            .id('single_visual_dashboard')
            .section('Style')
            .tab('Dashboard Basics')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('SINGLE_VISUAL_DASHBOARD')
            .add_label('Make fullscreen for single visual')
            .info('Removes background shading and padding. Useful for dashboards ' +
                   'with background image that covers the entire dashboard.')
            .value('false');
};

arc.settings.dashboards.bordersAroundVisuals = function() {
  return arc.visual.setting()
            .id('borders_around_visuals')
            .section('Style')
            .tab('Visual Borders')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('BORDERS_AROUND_VISUALS')
            .add_label('Show borders around visuals')
            .value('false')
            .valueBeforeRestore('true');
};

arc.settings.dashboards.visualBorderRadius = function() {
  return arc.visual.setting()
            .id('visual_border_radius')
            .section('Style')
            .tab('Visual Borders')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('VISUAL_BORDER_RADIUS')
            .add_label('Border Radius')
            .suffix('px')
            .min(0)
            .max(30)
            .step(1)
            .value('');
};

arc.settings.dashboards.bordersAroundFilters = function() {
  return arc.visual.setting()
            .id('borders_around_filters')
            .section('Style')
            .tab('Filters')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('BORDERS_AROUND_FILTERS')
            .add_label('Display borders around filters')
            .value('true');
};

arc.settings.dashboards.filtersOnLeft = function() {
  return arc.visual.setting()
            .id('filters_on_left')
            .section('Style')
            .tab('Filters')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('FILTERS_ON_LEFT')
            .add_label('Align filters on the left')
            .info('Not compatible with inline filter labels')
            .value('false');
};

arc.settings.dashboards.filterInlineLabels = function() {
  return arc.visual.setting()
            .id('filter_inline_labels')
            .section('Style')
            .tab('Filters')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('FILTERS_HORIZONTAL')
            .add_label('Display labels inline')
            .info('Not compatible with filters aligned on the left')
            .value('false');
};

arc.settings.dashboards.filterLabelLine = function() {
  return arc.visual.setting()
            .id('filter_label_line')
            .section('Style')
            .tab('Filters')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('FILTER_LABEL_LINE')
            .add_label('Display line separator')
            .info('Not compatible with inline filter labels')
            .value('true');
};

arc.settings.dashboards.dashboardTitleFontWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('dashboard_title_font_weight')
            .tab('Dashboard Title')
            .value('default');
};

arc.settings.dashboards.autoRefreshSeconds = function() {
  return arc.visual.setting()
            .id('auto_refresh_seconds')
            .tab('General')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('AUTO_REFRESH_SECONDS')
            .add_label('Auto-refresh period (sec)')
            .suffix('seconds')
            .min(0)
            .max(600)
            .step(5)
            .value('');
};

arc.settings.dashboards.horizontalMarginBetweenVisuals = function() {
  var min = 0,
      max = 100;

  return arc.visual.setting()
            .id('horizontal_margin_between_visuals')
            .tab('Dashboard Basics')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('HORIZONTAL_MARGIN_BETWEEN_VISUALS')
            .add_label('Horizontal Margins')
            .suffix('px')
            .min(min)
            .max(max)
            .value('10');
};

arc.settings.dashboards.verticalMarginBetweenVisuals = function() {
  var min = 0,
      max = 100;

  return arc.visual.setting()
            .id('vertical_margin_between_visuals')
            .tab('Dashboard Basics')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('VERTICAL_MARGIN_BETWEEN_VISUALS')
            .add_label('Vertical Margins')
            .suffix('px')
            .min(min)
            .max(max)
            .value('10');
};

arc.settings.dashboards.detailedDataMaxColumnLimit = function() {
  const min = -1;
  const defaultLimit = '50';

  return arc.visual.setting()
            .id('detailed_data_max_columns')
            .tab('General')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('DETAILED_DATA_MAX_COLUMN_LIMIT')
            .add_label('Detailed Data Columns')
            .min(min)
            .info('Sets the number of columns displayed in the visual Detailed Data table.')
            .value(defaultLimit);
};
}());
