/*
 * arc_settings_fonts.js should contain font settings that are common between
 * various objects, like common between visuals and dashboards.
 * However, a setting that is specific only to a dashboard
 * should be in arc_settings_dashboards.js
 * A font setting that is specific only to a dashboard is still in this file
 * and not in arc_settings_dashboard.js because it uses local functions like
 * fontValidate and fontSizeValidate
 * This file currently may not necessarily be following these rules but it
 * should, so please add new settings accordingly.
 */

window.arc = window.arc || {};

(function() {
'use strict';

arc.settings = arc.settings || {};
arc.settings.fonts = arc.settings.fonts || {};

arc.settings.fonts.titleFont = function() {
  const fontNamesMap = {
    'arc-font-roboto': 'Roboto',
    'arc-font-arial': 'Arial',
    'arc-font-arial-black': 'Arial Black',
    'arc-font-comic-sans-ms': 'Comic Sans MS',
    'arc-font-courier-new': 'Courier New',
    'arc-font-georgia': 'Georgia',
    'arc-font-helvetica': 'Helvetica',
    'arc-font-impact': 'Impact',
    'arc-font-lato': 'Lato',
    'arc-font-lucida-console': 'Lucida Console',
    'arc-font-lucida-sans-unicode': 'Lucida Sans Unicode',
    'arc-font-palatino-linotype': 'Palatino Linotype',
    'arc-font-tahoma': 'Tahoma',
    'arc-font-times-new-roman': 'Times New Roman',
    'arc-font-trebuchet-ms': 'Trebuchet MS',
    'arc-font-verdana': 'Verdana',
  };

  var span = '<span class="arc-font ';
  let s = arc.visual.setting()
            .id('title_font')
            .section('Style')
            .tab('Visual Title')
            .type(arc.visual.settingTypeEnum.DROPDOWN)
            .add_class('setting-font-family')
            .group('Font Family')
            .add_name('default')
            .add_label('default')
            .value('default');

  Object.keys(fontNamesMap).forEach(function(fontName) {
    s.add_name(fontName)
     .add_label(span + fontName + '">' + fontNamesMap[fontName] + '</span>');
  });

  s.mapFontNameToDisplayName = function(fontName) {
    if (fontName === 'default') {
      return 'default';
    }

    return fontNamesMap[fontName];
  };

  return s;
};

arc.settings.fonts.fontStyle = function() {
  const fontStyleMap = {
    normal: 'Normal',
    italic: 'Italic',
    oblique: 'Oblique',
  };

  let span = '<span class="arc-font-style-';
  let s = arc.visual.setting()
             .section('Style')
             .add_class('crosstab_total_style')
             .type(arc.visual.settingTypeEnum.DROPDOWN)
             .add_name('default')
             .add_label('default')
             .value('default');

  Object.keys(fontStyleMap).forEach(function(fontStyle) {
    s.add_name(fontStyle)
     .add_label(span + fontStyle + '">' +
        fontStyleMap[fontStyle] + '</span>');
  });

  s.valueToClass = function() {
    let v = s.value();
    if (v === 'default') {
      return '';
    }

    return 'arc-font-style-' + v;
  };

  return s;
};

arc.settings.fonts.fullTableWidth = function() {
  return arc.visual.setting()
            .id('full_table_width')
            .tab('Table Style')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('FULL_TABLE_WIDTH')
            .add_label('Full table width')
            .value('false');
};

arc.settings.fonts.aggregateRowStyle = function() {
  return arc.settings.fonts.fontStyle()
            .id('aggregate_row_style')
            .tab('Aggregate Row Style')
            .group('Font Style');
};

arc.settings.fonts.fontWeight = function() {
  const fontWeightMap = {
    lighter: 'Lighter',
    normal: 'Normal',
    bold: 'Bold',
  };

  var span = '<span class="arc-font-weight-';
  let s = arc.visual.setting()
             .section('Style')
             .id('dashboard_title_font_weight')
             .add_class('setting-font-weight')
             .tab('Dashboard Title')
             .type(arc.visual.settingTypeEnum.DROPDOWN)
             .group('Font Weight')
             .add_name('default')
             .add_label('default')
             .value('default');

  Object.keys(fontWeightMap).forEach(function(fontWeight) {
    s.add_name(fontWeight)
     .add_label(span + fontWeight + '">' +
                 fontWeightMap[fontWeight] + '</span>');
  });

  s.mapFontWeightToDisplayName = function(fontWeight) {
    if (fontWeight === 'default') {
      return 'default';
    }

    return fontWeightMap[fontWeight];
  };

  s.valueToClass = function() {
    let v = s.value();
    if (v === 'default') {
      return '';
    }

    return 'arc-font-weight-' + v;
  };

  return s;
};

arc.settings.fonts.aggregateRowWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('aggregate_row_weight')
            .tab('Aggregate Row Style')
            .group('Font Weight');
};

arc.settings.fonts.vizTitleFontWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('visual_title_font_weight')
            .tab('Visual Title')
            .value('default');
};

arc.settings.fonts.totalsRowFontWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('totals_row_font_weight')
            .tab('Table Style')
            .add_class('setting-long-title-dropdown')
            .group('Totals Row Font Weight');
};

arc.settings.fonts.totalsColumnFontWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('totals_column_font_weight')
            .tab('Table Style')
            .add_class('setting-long-title-dropdown')
            .group('Totals Column Font Weight');
};

arc.settings.fonts.appMenuTextFontWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('app_menutext_font_weight')
            .tab('App Menu Text')
            .value('default');
};

arc.settings.fonts.crosstabTitleFont = function() {
  return arc.settings.fonts.titleFont()
            .id('crosstab_title_font')
            .tab('Table Style')
            .group('Header Font');
};

arc.settings.fonts.crosstabTitleFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
             .id('crosstab_title_font_size')
             .tab('Table Style');

  s.labels()[0] = 'Header Font Size';
  return s;
};

arc.settings.fonts.crosstabTitleFontWeight = function() {
  return arc.settings.fonts.fontWeight()
            .id('crosstab_title_font_weight')
            .tab('Table Style')
            .group('Header Font Weight')
            .value('default');
};

arc.settings.fonts.subtitleFont = function() {
  return arc.settings.fonts.titleFont()
            .id('subtitle_font')
            .tab('Visual Subtitle')
            .group('Font Family');
};

arc.settings.fonts.tooltipFont = function() {
  return arc.settings.fonts.titleFont()
            .id('tooltip_font')
            .tab('Tooltip')
            .group('Font Family');
};

arc.settings.fonts.appMenuTextFont = function() {
  return arc.settings.fonts.titleFont()
            .id('app_menutext_font')
            .tab('App Menu Text');
};

arc.settings.fonts.dashboardTitleFont = function() {
  return arc.settings.fonts.titleFont()
            .id('dashboard_title_font')
            .tab('Dashboard Title')
            .group('Font Family');
};

arc.settings.fonts.dashboardSubtitleFont = function() {
  return arc.settings.fonts.subtitleFont()
            .tab('Dashboard Subtitle')
            .id('dashboard_subtitle_font')
            .group('Font Family');
};

arc.settings.fonts.visualFont = function() {
  return arc.settings.fonts.titleFont()
            .tab('Visual Basics')
            .id('visual_font')
            .group('Font Family');
};

arc.settings.fonts.filterFont = function() {
  return arc.settings.fonts.titleFont()
            .id('filter_font')
            .tab('Filters')
            .group('Font Family');
};

arc.settings.fonts.filterFontSize = function() {
  var s = arc.visual.setting();
  s.id('filter_font_size')
   .section('Style')
   .tab('Filters')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_name('FILTER_FONT_SIZE')
   .add_label('Font Size')
   .suffix('px')
   .min(0)
   .max(30)
   .value('');

  return s;
};

arc.settings.fonts.titleFontSize = function() {
  var s = arc.visual.setting();

  var min = 0,
      max = 100;

  s.id('title_font_size')
   .section('Style')
   .tab('Visual Title')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_name('TITLE_FONT_SIZE')
   .add_label('Font Size')
   .suffix('px')
   .min(min)
   .max(max)
   .value('');

  return s;
};

arc.settings.fonts.appMenuTextFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
             .tab('App Menu Text')
             .id('app_menutext_font_size');

  return s;
};

arc.settings.fonts.subtitleFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
             .tab('Visual Subtitle')
             .id('subtitle_font_size');

  return s;
};

arc.settings.fonts.tooltipFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
             .tab('Tooltip')
             .id('tooltip_font_size');

  return s;
};

arc.settings.fonts.dashboardTitleFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
             .section('Style')
             .tab('Dashboard Title')
             .id('dashboard_title_font_size');

  s.labels()[0] = 'Font Size';
  return s;
};

arc.settings.fonts.dashboardSubtitleFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
             .section('Style')
             .tab('Dashboard Subtitle')
             .id('dashboard_subtitle_font_size');

  s.labels()[0] = 'Font Size';
  return s;
};

arc.settings.fonts.titleFontAlignment = function() {
  return arc.visual.setting()
            .section('Style')
            .id('title_alignment')
            .tab('Visual Title')
            .type(arc.visual.settingTypeEnum.RADIO)
            .add_class('dash-viz-alignment')
            .group('Alignment')
            .add_name('Left')
            .add_label('')
            .add_name('Center')
            .add_label('')
            .add_name('Right')
            .add_label('')
            .value('Left');
};

arc.settings.fonts.subtitleFontAlignment = function() {
  return arc.settings.fonts.titleFontAlignment()
            .tab('Visual Subtitle')
            .id('subtitle_alignment')
            .group('Alignment');
};

arc.settings.fonts.dashboardTitleFontAlignment = function() {
  return arc.settings.fonts.titleFontAlignment()
            .id('dashboard_title_alignment')
            .tab('Dashboard Title');
};

arc.settings.fonts.dashboardSubtitleFontAlignment = function() {
  return arc.settings.fonts.titleFontAlignment()
            .id('dashboard_subtitle_alignment')
            .tab('Dashboard Subtitle');
};

arc.settings.fonts.overrideVisualFontPreferences = function() {
  return arc.visual.setting()
            .id('override_visual_font_settings')
            .section('Style')
            .tab('Dashboard Basics')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('OVERRIDE_VISUAL_FONT_SETTINGS')
            .add_label('Override visual style settings')
            .value('false');
};

arc.settings.fonts.visualFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
                            .section('Style')
                            .tab('Visual Basics')
                            .id('visual_font_size');
  s.labels()[0] = 'Font Size';

  return s;
};

arc.settings.fonts.axisLabelFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
                            .tab('Visual Axes and Labels')
                            .id('axis_label_font_size');
  s.labels()[0] = 'Axis Label Font Size';

  return s;
};

arc.settings.fonts.axisTicksFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
                            .tab('Visual Axes and Labels')
                            .id('axis_ticks_font_size');
  s.labels()[0] = 'Axis Ticks Font Size';

  return s;
};

arc.settings.fonts.labelFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
                            .tab('Visual Axes and Labels')
                            .id('label_font_size');
  s.labels()[0] = 'Label Font Size';

  return s;
};

arc.settings.fonts.trellisFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
                            .tab('Visual Axes and Labels')
                            .id('trellis_font_size');
  s.labels()[0] = 'Trellis Font Size';

  return s;
};

arc.settings.fonts.legendFontSize = function() {
  var s = arc.settings.fonts.titleFontSize()
                            .tab('Visual Axes and Labels')
                            .id('legend_font_size');
  s.labels()[0] = 'Legend Font Size';

  return s;
};

arc.settings.fonts.visualAxisLabelFontSize = function() {
  return arc.settings.fonts.axisLabelFontSize()
            .tab('Visual Axes and Labels');
};

arc.settings.fonts.visualAxisTicksFontSize = function() {
  return arc.settings.fonts.axisTicksFontSize()
            .tab('Visual Axes and Labels');
};

arc.settings.fonts.visualLabelFontSize = function() {
  return arc.settings.fonts.labelFontSize()
            .tab('Visual Axes and Labels');
};

arc.settings.fonts.visualTrellisFontSize = function() {
  return arc.settings.fonts.trellisFontSize()
            .tab('Visual Axes and Labels');
};

arc.settings.fonts.visualLegendFontSize = function() {
  return arc.settings.fonts.legendFontSize()
            .tab('Visual Axes and Labels');
};

}());
