window.arc = window.arc || {};

(function() {
'use strict';

arc.settings = arc.settings || {};
arc.settings.network = arc.settings.network || {};

function opacity() {
  var s = arc.visual.setting();

  s.id('src_node_opacity')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_class('spinner-alignright')
   .min(0)
   .max(100)
   .validate_fn(function(val) {
     return s.validate_num_range(val, 0, 100);
   })
   .value('80');

  return s;
}

arc.settings.network.srcNodeOpacity = function() {
  return opacity()
            .id('src_node_opacity')
            .tab('Nodes')
            .add_name('SRC_NODE_OPACITY')
            .add_label('Node Opacity');
};

arc.settings.network.linkOpacity = function(vizType) {
  let tabName = vizType === 'network' ? 'Links' : 'Marks';
  return opacity()
            .id('link_opacity')
            .tab(tabName)
            .add_name('LINK_OPACITY')
            .add_label('Link Opacity');
};

arc.settings.network.showNodeLabels = function(vizType) {
  const tabName = vizType === 'network' ? 'Nodes' : 'Marks';
  const defaultVal = vizType === 'network' ? 'false' : 'true';
  return arc.visual.setting()
            .id('show_node_labels')
            .tab(tabName)
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('SHOW_NODE_LABELS')
            .add_label('Show node labels')
            .value(defaultVal);
};

arc.settings.network.rememberNodePositions = function() {
  return arc.visual.setting()
            .id('remember_node_positions')
            .tab('Nodes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('REMEMBER_NODE_POSITIONS')
            .add_label('Remember the positions of fixed nodes')
            .value('true');
};

// This is used by network and dendrogram visual
  //vizType should be 'network' or 'dendrogram'
arc.settings.network.linkColor = function(vizType) {
  let description, aggName, tabName;
  if (vizType === 'network') {
    description = 'Default Link Color';
    aggName = 'link width';
    tabName = 'Links';
  } else {
    description = 'Color links based on';
    aggName = 'aggregate';
    tabName = 'Marks';
  }

  return arc.visual.setting()
            .id('link_color')
            .tab(tabName)
            .type(arc.visual.settingTypeEnum.DROPDOWN)
            .group(description)
            .add_name('source')
            .add_label('source')
            .add_name('target')
            .add_label('target')
            .add_name('aggregate')
            .add_label(aggName)
            .add_name('fixed')
            .add_label('fixed')
            .value('aggregate');
};

arc.settings.network.srcNodeMarkType = function() {
  return arc.visual.setting()
            .id('src_node_mark_type')
            .tab('Nodes')
            .type(arc.visual.settingTypeEnum.DROPDOWN)
            .group('Source Node Mark')
            .add_name('circle')
            .add_label('circle')
            .add_name('square')
            .add_label('square')
            .add_name('diamond')
            .add_label('diamond')
            .add_name('cross')
            .add_label('cross')
            .add_name('triangle-up')
            .add_label('triangle-up')
            .add_name('triangle-down')
            .add_label('triangle-down')
            .value('circle');
};

arc.settings.network.combinedNodeMarkType = function() {
  var s = arc.settings.network.srcNodeMarkType();

  s.id('combined_node_mark_type')
   .group('Combined Node Mark')
   .info('Nodes which are both source and target.')
   .value('circle');

  return s;
};

arc.settings.network.tgtNodeMarkType = function() {
  var s = arc.settings.network.srcNodeMarkType();

  s.id('tgt_node_mark_type')
   .group('Target Node Mark')
   .value('circle');

  return s;
};

arc.settings.network.linkWidthRange = function(vizType) {
  var s = arc.visual.setting();
  let tabName = vizType === 'network' ? 'Links' : 'Marks';

  s.id('link_width_range')
   .tab(tabName)
   .type(arc.visual.settingTypeEnum.INTEGER_RANGE)
   .add_name('LINK_WIDTH_RANGE')
   .add_label('Link Width Range')
   .validate_fn(function(val) {
     return s.validate_num_range(val, 1, 200);
   })
   .value('4-10');

  return s;
};

arc.settings.network.nodeSizeRange = function() {
  var s = arc.visual.setting();

  s.id('node_size_range')
   .tab('Nodes')
   .type(arc.visual.settingTypeEnum.INTEGER_RANGE)
   .add_name('NODE_SIZE_RANGE')
   .add_label('Node size range')
   .validate_fn(function(val) {
     return s.validate_num_range(val, 1, 200);
   })
   .value('10-25');

  return s;
};

arc.settings.network.linkStrength = function() {
  var s = arc.visual.setting();

  s.id('link_strength')
   .tab('Links')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('LINK_STRENGTH')
   .add_label('Link Strength')
   .min(0)
   .max(1)
   .step(0.1)
   .value('1');

  return s;
};

arc.settings.network.nodeFriction = function() {
  var s = arc.visual.setting();

  s.id('node_friction')
    .section('Color')
   .tab('Nodes')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('NODE_FRICTION')
   .add_label('Node Friction')
   .min(0)
   .max(1)
   .step(0.1)
   .value('0.3');

  return s;
};

arc.settings.network.linkDistance = function() {
  var s = arc.visual.setting();

  s.id('link_distance')
   .tab('Links')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_class('spinner-alignright')
   .add_name('LINK_DISTANCE')
   .add_label('Link Distance')
   .min(0)
   .max(500)
   .value('80');

  return s;
};

arc.settings.network.nodeCharge = function() {
  var s = arc.visual.setting();

  s.id('node_charge')
   .tab('Nodes')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('NODE_CHARGE')
   .add_label('Node Charge')
   .min(-3000)
   .max(0)
   .value('-1000');

  return s;
};

arc.settings.network.layoutGravity = function() {
  var s = arc.visual.setting();

  s.id('layout_gravity')
   .tab('Marks')
   .type(arc.visual.settingTypeEnum.SLIDER)
   .add_name('LAYOUT_GRAVITY')
   .add_label('Layout Gravity')
   .min(0)
   .max(1)
   .step(0.1)
   .value('0.1');

  return s;
};

arc.settings.network.layoutTheta = function() {
  var s = arc.visual.setting();

  s.id('layout_theta')
   .tab('Marks')
   .type(arc.visual.settingTypeEnum.REAL)
   .add_name('LAYOUT_THETA')
   .add_label('Layout Theta')
   .value('0.8');

  return s;
};

arc.settings.network.unifyNodes = function() {
  return arc.visual.setting()
            .id('same_domain')
            .tab('Nodes')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('UNIFY_NODES')
            .add_label('Source & target node data come from the same base values.')
            .value('false');
};

arc.settings.network.emptyColorSetting = function() {
  return arc.visual.setting()
            .id('empty_color')
            .tab('Color')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .hidden('true')
            .add_name('HIDDEN_COLOR')
            .info('Added to add the color tab')
            .value('false');
};

arc.settings.network.defaultNodeSize = function() {
  return arc.visual.setting()
            .id('default_node_size')
            .tab('Nodes')
            .type(arc.visual.settingTypeEnum.INTEGER)
            .add_name('DEFAULT_NODE_SIZE')
            .add_label('Default node size')
            .value('5');
};

// TODO: implement various options for default node size and color

arc.settings.network.addDirectionArrows = function() {
  return arc.visual.setting()
            .id('add_direction_arrow')
            .tab('Links')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('ADD_DIRECTION_ARROW')
            .add_label('Add direction arrows to links')
            .value('false');
};

arc.settings.network.directionArrowSize = function() {
  return arc.visual.setting()
            .id('direction_arrow_size')
            .tab('Links')
            .type(arc.visual.settingTypeEnum.SLIDER)
            .add_name('DIRECTION_ARROW_SIZE')
            .add_label('Direction Arrow Size')
            .step(0.2)
            .min(0)
            .max(10)
            .value('2');
};

arc.settings.network.addNodeLegend = function() {
  const addLegendSetting = arc.visual.setting();

  addLegendSetting.id('add_node_legend')
                  .tab('Nodes')
                  .type(arc.visual.settingTypeEnum.CHECKBOX)
                  .add_name('ADD_NODE_LEGEND')
                  .add_label('Add node legend')
                  .value('false');
  return addLegendSetting;
};

arc.settings.network.addLinkLegend = function() {
  const linkLegendSetting = arc.visual.setting();

  linkLegendSetting.id('add_link_legend')
                  .tab('Links')
                  .type(arc.visual.settingTypeEnum.CHECKBOX)
                  .add_name('ADD_LINK_LEGEND')
                  .add_label('Add link legend')
                  .value('false');
  return linkLegendSetting;
};

arc.settings.network.legendWidth = function() {
  return arc.visual.setting()
            .id('network_legend_width')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.SLIDER)
            .add_name('NETWORK_LEGEND_WIDTH')
            .add_label('Legend Width')
            .min(100)
            .max(250)
            .value('150');
};

arc.settings.network.nodeLegendTitle = function() {
  return arc.visual.setting()
            .id('node_legend_title')
            .tab('Nodes')
            .type(arc.visual.settingTypeEnum.STRING)
            .add_name('NODE_LEGEND_TITLE')
            .add_label('Legend Title')
            .value('Node colors');
};

arc.settings.network.linkLegendTitle = function() {
  return arc.visual.setting()
            .id('link_legend_title')
            .tab('Links')
            .type(arc.visual.settingTypeEnum.STRING)
            .add_name('LINK_LEGEND_TITLE')
            .add_label('Legend Title')
            .value('Link colors');
};

arc.settings.network.linkPointerColor = function() {
  return arc.visual.setting()
            .id('link_pointer_color')
            .section('Color')
            .tab('Link Colors')
            .type(arc.visual.settingTypeEnum.COLOR)
            .add_name('LINK_POINTER_COLOR')
            .add_label('Direction Arrow Color')
            .value('black');
};

}());
