window.arc = window.arc || {};

(function() {
'use strict';

arc.settings = arc.settings || {};
arc.settings.fonts = arc.settings.fonts || {};
arc.settings.colors = arc.settings.colors || {};

function fontSizeValidate(v) {
  if ((v === '') || isNaN(v)) {
    return undefined;
  }

  v = Number(v);
  if (v < 0) {
    return undefined;
  }

  return v;
}

function pickSetting(user_settings, setting_id, validateFn) {
  var dashboardValue;

  if (arc.globals && arc.globals.dashboard) {
    var dashboard_settings = arc.globals.dashboard.user_settings();

    var ds = dashboard_settings.find_by_id(setting_id);
    if (ds) {
      dashboardValue = validateFn(ds.value());

      // Check if the dashboard level font overrides the visual level font
      if (dashboardValue && arc.misc.dashboardOverride()) {
        return dashboardValue;
      }
    }
  }

  var visualValue;
  var s = user_settings.find_by_id(setting_id);
  if (s) {
    visualValue = validateFn(s.value());
  }

  if (!visualValue) {
    return dashboardValue;
  }

  return visualValue;
}

function fontValidate(v) {
  if (v === '') {
    return undefined;
  }

  if (v === 'default') {
    return undefined;
  }

  return v;
}

arc.settings.fonts.fontSetting = function(user_settings, visual_setting_id) {
  return pickSetting(user_settings, visual_setting_id, fontValidate);
};

arc.settings.fonts.fontSizeSetting = function(user_settings, visual_setting_id) {
  return pickSetting(user_settings, visual_setting_id, fontSizeValidate);
};

function alignmentValidate(v) {
  v = v.toLowerCase();
  if (['left', 'center', 'right'].indexOf(v) < 0) {
    return undefined;
  }

  return v;
}

arc.settings.fonts.fontAlignmentSetting = function(user_settings, visual_setting_id) {
  return pickSetting(user_settings, visual_setting_id, alignmentValidate);
};

function colorValidate(v) {
  if (v === '') {
    return undefined;
  }

  return v;
}

arc.settings.colors.colorSetting = function(user_settings, visual_setting_id) {
  return pickSetting(user_settings, visual_setting_id, colorValidate);
};

}());
