window.arc = window.arc || {};

(function() {
'use strict';

arc.settings = arc.settings || {};
arc.settings.table = arc.settings.table || {};

arc.settings.table.autoSizeColumn = function() {
  return arc.visual.setting()
            .id('auto_size_column')
            .tab('Table Style')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('AUTO_SIZE_COLUMN')
            .add_label('Auto Size Columns by header')
            .value('false');
};

arc.settings.table.settingCrosstabHeaderAlignment = function () {
  const setting = arc.visual
    .setting()
    .id('crosstab_header_alignment')
    .section('Style')
    .tab('Table Style')
    .type(arc.visual.settingTypeEnum.RADIO)
    .group('Crosstab Header Alignment')
    .add_name('default')
    .add_label('Default')
    .add_name('left')
    .add_label('')
    .add_name('center')
    .add_label('')
    .add_name('right')
    .add_label('')
    .value('left');

  let $table_element = $();

  setting.$table_element = function (_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  function paintTable(tab_d3_sel) {
    const div_id = setting.div_id();
    tab_d3_sel.append('h5').text(setting.group());

    tab_d3_sel
      .append('div')
      .attr('id', div_id)
      .attr('class', `form-group ${setting.id()}`)
      .selectAll('.nosel')
      .data(setting.names())
      .enter()
      .append('div')
      .attr('class', 'form-check form-check-inline header-alignment')
      .append('label')
      .html((d, i) => {
        let icon = '';
        if (d !== 'default') {
          icon = `<i class='fa fa-align-${d}'></i>`;
        }

        const inputbox =
          `<input type="radio" name=" ${div_id}"class="form-check-input"` +
          `" value="${d}"${d === setting.value() ? '" checked>' : '">'}${
            setting.labels().length === 0 ? d : setting.labels()[i]
          }`;
        return inputbox + icon;
      });
    return setting;
  }

  function changeEventHandler() {
    $('div.header-alignment.form-check-inline').on('change', (evt) => {
      const $control = $(evt.currentTarget);
      const new_value = $control.find('input:radio:checked').val();
      const $allConstituentTables =
        arc.report.user_settings.getConstituentTable($table_element);
      if (setting.value() !== new_value) {
        setting.value(new_value);
        setting.makeChangeCbs();
      }

      $allConstituentTables.removeClass(
        'header-align-left header-align-center header-align-right'
      );
      if (new_value !== 'default') {
        $allConstituentTables.addClass(`header-align-${new_value}`);
      }
    });
  }

  setting.paint_fn(paintTable);

  setting.add_postpaint_fn(changeEventHandler);

  return setting;
};

arc.settings.table.settingCrosstabContentAlignment = function () {
  const setting = arc.visual
    .setting()
    .id('crosstab_content_alignment')
    .section('Style')
    .tab('Table Style')
    .type(arc.visual.settingTypeEnum.RADIO)
    .group('Crosstab Content Alignment')
    .add_name('default')
    .add_label('Default')
    .add_name('left')
    .add_label('')
    .add_name('center')
    .add_label('')
    .add_name('right')
    .add_label('')
    .value('default');

  let $table_element = $();

  setting.$table_element = function (_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  function paintTable(tab_d3_sel) {
    const div_id = setting.div_id();
    tab_d3_sel.append('h5').text(setting.group());

    tab_d3_sel
      .append('div')
      .attr('id', div_id)
      .attr('class', `form-group ${setting.id()}`)
      .selectAll('.nosel')
      .data(setting.names())
      .enter()
      .append('div')
      .attr('class', 'form-check form-check-inline content-alignment')
      .append('label')
      .html((d, i) => {
        let icon = '';

        icon = `<i class='fa fa-align-${d}'></i>`;

        const inputbox =
          `<input type="radio" name=" ${div_id}"class="form-check-input"` +
          `" value="${d}"${d === setting.value() ? '" checked>' : '">'}${
            setting.labels().length === 0 ? d : setting.labels()[i]
          }`;
        return inputbox + icon;
      });
    return setting;
  }

  function changeEventHandler() {
    $('div.content-alignment.form-check-inline').on('change', (evt) => {
      const $control = $(evt.currentTarget);
      const new_value = $control.find('input:radio:checked').val();
      const $allConstituentTables =
        arc.report.user_settings.getConstituentTable($table_element);
      if (setting.value() !== new_value) {
        setting.value(new_value);
        setting.makeChangeCbs();
      }

      $allConstituentTables.removeClass(
        'content-align-left conten-align-center conten-align-right'
      );
    });
  }

  setting.paint_fn(paintTable);

  setting.add_postpaint_fn(changeEventHandler);

  return setting;
};

arc.settings.table.settingTableTotalsHeader = function() {
  let $table_element = $();

  const setting = arc.visual.setting()
                   .id('table_totals_header')
                   .tab('Data Summary')
                   .type(arc.visual.settingTypeEnum.CHECKBOX)
                   .add_name('TABLE_TOTALS_HEADER')
                   .add_label('Show totals header')
                   .info("Only applies if 'Show totals' is checked.")
                   .value('false');

  setting.$table_element = function(_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  function paintTable() {
    if (!$table_element) {
      return;
    }

    const $settingContainer = $(`#${setting.div_id()}`);
    const isTotalsHeaderChecked = $settingContainer.find('input').prop('checked');
    if (isTotalsHeaderChecked !== setting.isChecked) {
      setting.value(isTotalsHeaderChecked.toString());
      setting.makeChangeCbs();
    }
  }

  function postPaint() {
    const $s = $(`#${setting.div_id()}`);
    $s.find('input').off('change').on('change', paintTable);
    return setting;
  }

  setting.add_postpaint_fn(postPaint);
  return setting;
};

arc.settings.table.settingPageTableTotalsHeader = function() {
  let $table_element = $();

  const setting = arc.visual.setting()
                   .id('table_page_totals_header')
                   .tab('Data Summary')
                   .type(arc.visual.settingTypeEnum.CHECKBOX)
                   .add_name('TABLE_PAGE-TOTALS_HEADER')
                   .add_label('Show page totals header')
                   .info("Only applies if 'Show page totals' is checked.")
                   .value('false');

  setting.$table_element = function(_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  function paintTable() {
    if (!$table_element) {
      return;
    }

    const $settingContainer = $(`#${setting.div_id()}`);
    const isPageTotalsHeaderChecked = $settingContainer.find('input').prop('checked');
    if (isPageTotalsHeaderChecked !== setting.isChecked) {
      setting.value(isPageTotalsHeaderChecked.toString());
      setting.makeChangeCbs();
    }
  }

  function postPaint() {
    const $settingContainer = $(`#${setting.div_id()}`);
    $settingContainer.find('input').offOn('change', paintTable);
    return setting;
  }

  setting.add_postpaint_fn(postPaint);
  return setting;
};

arc.settings.table.settingTableTotals = function() {
  let $table_element = $();

  const setting = arc.visual.setting()
                   .id('table_totals')
                   .tab('Data Summary')
                   .type(arc.visual.settingTypeEnum.CHECKBOX)
                   .add_name('TABLE_TOTALS')
                   .add_label('Show totals')
                   .value('false');

  setting.$table_element = function(_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  function paintTable() {
    if (!$table_element) {
      return;
    }

    const $settingContainer = $(`#${setting.div_id()}`);
    const isCheckedTotals = $settingContainer.find('input').prop('checked');
    if (isCheckedTotals !== setting.isChecked) {
      setting.value(isCheckedTotals.toString());
      setting.makeChangeCbs();
    }

    setting.paint_table_style(isCheckedTotals);
  }

  setting.paint_table_style = function(isCheckedTotals) {
    if (isCheckedTotals === undefined) {
      isCheckedTotals = setting.isChecked;
    }

    const $allConstituentTables = arc.report.user_settings.getConstituentTable($table_element);
    if (isCheckedTotals) {
      $allConstituentTables.addClass('table-totals');
    } else {
      $allConstituentTables.removeClass('table-totals');
    }

    return setting;
  };

  function postPaint() {
    const $settingContainer = $(`#${setting.div_id()}`);
    $settingContainer.find('input').offOn('change', () => {
      paintTable();
    });
    return setting;
  }

  setting.add_postpaint_fn(postPaint);
  return setting;
};

arc.settings.table.settingPageTableTotals = function() {
  let $table_element = $();

  const setting = arc.visual.setting()
                   .id('table_page_totals')
                   .tab('Data Summary')
                   .type(arc.visual.settingTypeEnum.CHECKBOX)
                   .add_name('TABLE_PAGE_TOTALS')
                   .add_label('Show page totals')
                   .value('false');

  setting.$table_element = function(_) {
    if (!arguments.length) return $table_element;
    $table_element = _;
    return setting;
  };

  function paintTable() {
    if (!$table_element) {
      return;
    }

    const $settingContainer = $(`#${setting.div_id()}`);
    const isCheckedPageTotals = $settingContainer.find('input').prop('checked');
    if (isCheckedPageTotals !== setting.isChecked) {
      setting.value(isCheckedPageTotals.toString());
      setting.makeChangeCbs();
    }

    setting.paint_table_style(isCheckedPageTotals);
  }

  setting.paint_table_style = function(isCheckedPageTotals) {
    if (isCheckedPageTotals === undefined) {
      isCheckedPageTotals = setting.isChecked;
    }

    const $allConstituentTables = arc.report.user_settings.getConstituentTable($table_element);
    if (isCheckedPageTotals) {
      $allConstituentTables.addClass('table-page-totals');
    } else {
      $allConstituentTables.removeClass('table-page-totals');
    }

    return setting;
  };

  function postPaint() {
    const $settingContainer = $(`#${setting.div_id()}`);
    $settingContainer.find('input').off('change');
    $settingContainer.find('input').on('change', () => {
      paintTable();
    });
    return setting;
  }

  setting.add_postpaint_fn(postPaint);
  return setting;
};

arc.settings.table.tableEvenRowsBackgroundColor = function() {
  var s = arc.visual.setting();

  s.id('table_even_rows_background_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_EVEN_ROWS_BACKGROUND_COLOR')
   .add_label('Even Rows Background')
   .value('');

  return s;
};

arc.settings.table.tableOddRowsBackgroundColor = function() {
  var s = arc.visual.setting();

  s.id('table_odd_rows_background_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_ODD_ROWS_BACKGROUND_COLOR')
   .add_label('Odd Rows Background')
   .value('');

  return s;
};

arc.settings.table.tableEvenRowsTextColor = function() {
  var s = arc.visual.setting();

  s.id('table_even_rows_text_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_EVEN_ROWS_TEXT_COLOR')
   .add_label('Even Rows Text')
   .value('');

  return s;
};

arc.settings.table.tableOddRowsTextColor = function() {
  var s = arc.visual.setting();

  s.id('table_odd_rows_text_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_ODD_ROWS_TEXT_COLOR')
   .add_label('Odd Rows Text')
   .value('');

  return s;
};

arc.settings.table.tableHeaderBackgroundColor = function() {
  var s = arc.visual.setting();

  s.id('table_header_background_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_HEADER_BACKGROUND_COLOR')
   .add_label('Header Background')
   .value('');

  return s;
};

arc.settings.table.tableHeaderTextColor = function() {
  var s = arc.visual.setting();

  s.id('table_header_text_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_HEADER_TEXT_COLOR')
   .add_label('Header Text')
   .value('');

  return s;
};

arc.settings.table.tableFooterBackgroundColor = function() {
  var s = arc.visual.setting();

  s.id('table_footer_background_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_FOOTER_BACKGROUND_COLOR')
   .add_label('Totals Row Background')
   .value('');

  return s;
};

arc.settings.table.tableFooterTextColor = function() {
  var s = arc.visual.setting();

  s.id('table_footer_text_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_FOOTER_TEXT_COLOR')
   .add_label('Totals Row Text')
   .value('');

  return s;
};

arc.settings.table.tableBorderColor = function() {
  var s = arc.visual.setting();

  s.id('table_border_color')
   .section('Style')
   .tab('Table Colors')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('TABLE_BORDER_COLOR')
   .add_label('Border')
   .value('');

  return s;
};

arc.settings.table.tableBorderWidth = function() {
  var s = arc.visual.setting();
  s.id('table_border_width')
   .section('Style')
   .tab('Table Style')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_label('Border Width')
   .suffix('px')
   .min(0)
   .max(5)
   .value('1');

  return s;
};

arc.settings.table.wrapColumnContents = function() {
  return arc.visual.setting()
            .id('table_wrap_column_contents')
            .tab('Table Style')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('TABLE_WRAP_COLUMN_CONTENTS')
            .add_label('Wrap column contents')
            .value('false')
            .valueBeforeRestore('true');
};

arc.settings.table.displayHeaderColumnBorders = function() {
  return arc.visual.setting()
            .id('display_header_column_borders')
            .tab('Table Style')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('DISPLAY_HEADER_COLUMN_BORDERS')
            .add_label('Display header column borders')
            .value('true');
};

arc.settings.table.displayColumnHeaders = function() {
  return arc.visual.setting()
            .id('display_header')
            .tab('Table Style')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_label('Display header')
            .value('true');
};

arc.settings.table.enableHighlighting = function() {
  return arc.visual.setting()
            .id('enable_highlighting')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('ENABLE_HIGHLIGHTING')
            .add_label('Enable highlighting')
            .info("Only applies if 'Table data is in HTML' is enabled and there are no aggregates.")
            .value('false');
};

arc.settings.table.cellsAreHTMLSetting = function(enable_highlighting) {
  const setting = arc.visual.setting()
    .id('cells_are_html')
    .tab('Marks')
    .type(arc.visual.settingTypeEnum.CHECKBOX)
    .add_name('CELLS_ARE_HTML')
    .add_label('Table data is in HTML')
    .value('false');

    if (enable_highlighting) {
      setting.addDependents([enable_highlighting]);
    }

    return setting;
};
}());
