window.arc = window.arc || {};

(function() {
'use strict';

arc.showme = arc.showme || {};

function removeTabWand(tab) {
  if (Array.isArray(tab)) {
    tab.forEach(t => removeTabWand(t));
  }

  $(`#${tab} .panel-header`).find('.viz-showme-container').remove();
}

function visualTypes(vizDriver, setupClick) {
  if (arc.showme.noVisualExplore().includes(vizDriver.viz_type())) {
    if (setupClick) {
      $('.viz-showme-visuals').hide();
    }

    return;
  }

  function exploreVisualTypes() {
    let showme = arc.showme.visuals({
      vizDriver: vizDriver,
    });

    showme.setup();
  }

  if (setupClick) {
    $('.viz-showme-visuals').show()
        .offOn('click', function(event) {
          event.stopPropagation();
          $(this).closest('.recommend').tooltip('dispose');
          exploreVisualTypes();
        });
  }

  return {
    id: 'visual-types',
    name: 'Visual Types',
    fn: exploreVisualTypes,
  };
}

function colorSettings(vizDriver, setupClick) {

  // If this visual does not have a color palette, then don't permit
  // color exploration
  const colorSetting = vizDriver.user_settings().find_by_id('color_palette');
  if (colorSetting === undefined) {
    return;
  }

  function exploreColors() {
    const p = [];

    let s = vizDriver.user_settings()
                     .find_by_id('color_palette'),
        originalColorSetting = s.value(),
        parsedColorSetting = JSON.parse(s.value());

    let allColors = s.allColors();

    allColors.forEach(function(c) {
      parsedColorSetting.colors = c;
      p.push({
        use_dataset_color_settings: 'false',
        color_palette: JSON.stringify(parsedColorSetting.colors),
        // The order is quite important
        // dim_to_color_mapping should happen after color_palette change,
        // otherwise it will (re)seeds and setups the old colors!
        dim_to_color_mapping: JSON.stringify({ mappings: {}, isChecked: false }),
        measure_ranges: JSON.stringify({ ranges: [] }),
      });
    });

    let showme = arc.showme.settings({
      vizDriver: vizDriver,
      settings: p,
    });

    if (!showme) {
      arc.misc.infoModal({
        title: 'Explore Settings',
        contentsHtml: `
        <p>
          Currently unable to explore style as there are no fields to visualize
        </p>
        <p>
          Please either select some fields from the 'Data' tab
          or click on the 'Explore Visuals' option.
        </p>`,
      });

      return;
    }

    showme.setup();
  }

  if (setupClick) {
    if (arc.globals.enable_explore_interfaces) {
    const $panelHeader = $('#viz-color-tab .panel-header');
    const $showMeWand = $('<div class="viz-showme-container">' +
      '<a class="viz-showme viz-showme-colors" data-toggle="tooltip" title="Explore Colors">' +
      '<span class="explore-icon"></span>' +
      '</a></div>');
    $panelHeader.append($showMeWand);
    $showMeWand.off('click.exploreColors').on('click.exploreColors', exploreColors);
    }
  }

  return {
    id: 'colors',
    name: 'Colors',
    fn: exploreColors,
  };
}

function generalStyles(vizDriver, setupClick) {

  const noStyleExplore = ['html', 'extension', 'link', 'column-stats'];

  if (noStyleExplore.includes(vizDriver.viz_type())) {
    return;
  }

  function exploreStyles() {

    const tableStyles = [
      {
        tableTemplate: 'table-template-1',
        name: 'Table with borders',
      },
      {
        tableTemplate: 'table-template-2',
        name: 'Table with borders and stripes',
      },
      {
        tableTemplate: 'table-template-3',
        name: 'Table with stripes and column borders',
      },
      {
        tableTemplate: 'table-template-4',
        name: 'Table with stripes and no borders',
      },
      {
        tableTemplate: 'table-template-5',
        name: 'Striped Table with first column highlighted',
      },
      {
        tableTemplate: 'table-template-6',
        name: 'Striped and Compact Table',
      },
      {
        tableTemplate: 'table-template-7',
        name: 'Headline Table',
      },
    ];

    const fontSetting = arc.settings.fonts.titleFont();
    const fonts = fontSetting.names();
    const p = [];

    const tableVisuals = ['table', 'rawtable'];

    if (tableVisuals.includes(vizDriver.viz_type())) {
      tableStyles.forEach(function (style) {
        p.push({
          name: style.name,
          table_template: style.tableTemplate,
        });
      });
    }

    fonts.forEach(function(f) {
      p.push({
        name: fontSetting.mapFontNameToDisplayName(f) + ' Font',
        visual_font: f,
      });
    });

    let showme = arc.showme.settings({
                      vizDriver: vizDriver,
                      settings: p,
                    });

    if (!showme) {
      arc.misc.infoModal({
        title: 'Explore Settings',
        contentsHtml: `
        <p>
          Currently unable to explore style as there are no fields to visualize
        </p>
        <p>
          Please either select some fields from the 'Data' tab
          or click on the 'Explore Visuals' option.
        </p>`,
      });

      return;
    }

    showme.displayTitle(true);
    showme.setup();
  }

  if (setupClick) {
    if (arc.globals.enable_explore_interfaces) {
    const $panelHeader = $('#viz-basic-styling-tab .panel-header');
    const $showMeWand = $('<div class="viz-showme-container">' +
      '<a class="viz-showme viz-showme-styles" data-toggle="tooltip" title="Explore Styles">' +
      '<span class="explore-icon"></span>' +
      '</a></div>');
    $panelHeader.append($showMeWand);
    $showMeWand.off('click').on('click', exploreStyles);
    }
  }

  return {
    id: 'style',
    name: 'Styles/Fonts',
    fn: exploreStyles,
  };
}

  /**
   * The settingsSettings object has the following form:
   * key (visual name, or whatever the settings apply to): array of settings arrays which have
   * the following form
   * [setting name (what will be displayed), setting id (no spaces), setting array with a list of
   * key-value pairs
   * key being that individual setting id, and the value being the value which to set the setting.
   * an optional fourth value is a shelf requirement object: yes points to required shelves,
   * no points to shelves
   * which if present should lead to
   *
   * @type {{radial: [*], pie: [*], scatter: [*], linesAndAreas: [*], bars: [*], boxplot: [*],
   * network: [*], dendrogram: [*], gauge: [*]}}
   */
let allSettingsSettingsOptions = {
  radial: [
    ['Legend to right', 'nonOverlapping', { overlapping_marks: 'false', legend_style: 'Right' }],
    ['Overlapping', 'overlapping', { overlapping_marks: 'true', legend_style: 'Right' }],
    ['Legend above', 'legendAbove', { legend_style: 'Above' }],
    ['No legend', 'noLegend', { legend_style: 'None' }],
    ['Compare to values in same measure across all dimensions', 'oneMeasure',
      { scale_type_radial: 'ONEMEASURE' }],
    ['Compare to values across all measures', 'allMeasures', { scale_type_radial: 'ALLMEASURE' }],
    ['Compare to values to other measure values in same dimension', 'oneDimension',
      { scale_type_radial: 'ONEDIMENSION' }],
  ],
  pie: [
    ['Pie, legend to the right', 'pieLegendRight', { legend_style: 'Right', arc_width: '0' }],
    ['No legend', 'noLegend', { legend_style: 'None' }],
    ['Donut with arc width of 60', 'arcWidth60', { arc_width: '60' }],
    ['Pie, legend Below', 'pieLegendBelow', { legend_style: 'Bottom', arc_width: '0' }],
  ],
  scatter: [
    ['Small size gradient, no large dots (Mark Size Range)', 'smallSizeGradientNoLarge',
      { mark_size_range: '5-30' }],
    ['Small size gradient, no small dots (Mark Size Range)', 'smallSizeGradientNoSmall',
      { mark_size_range: '20-50' }],
    ['Large size gradient (Mark Size Range)', 'largeSizeGradient', { mark_size_range: '5-60' }],
    ['Legend on right', 'legendRight', { legend_style: 'Right' }],
    ['Legend on bottom', 'legendBottom', { legend_style: 'Bottom' }],
    ['Gridlines enabled', 'gridlinesEnabled', { gridlines_enabled: 'true' }],
    ['No gridlines', 'noGridlines', { gridlines_enabled: 'false' }],
    ['Reverse x scale', 'xScaleReversed', { reverse_scale_x: 'true' }],
    ['Reverse y scale', 'yScaleReversed', { reverse_scale_y: 'true' }],
    ['Show trendline', 'showTrendline', { trendline_linear: 'true' }],
    ['Separate trendlines by color', 'colorTrendlines',
      { trendline_linear: 'true', trendline_color: 'true' }]
  ],
  linesAndAreas: [
    ['Linear segments with axis', 'linearWithAxis', { line_interpolation: 'linear',
    show_x_axis: 'true', show_y_axis: 'true' }],
    ['Basis-curved segments', 'basisCurved', { line_interpolation: 'basis', show_x_axis: 'true' }],
    ['Bundle-curved segments', 'bundleCurved', {line_interpolation: 'bundle', show_y_axis: 'true'}],
    ['Legend on right', 'legendRight', {legend_style: 'Right'}],
    ['Legend on bottom', 'legendBottom', {legend_style: 'Bottom'}],
    ['Gridlines enabled', 'gridlinesEnabled', {gridlines_enabled: 'true'}],
    ['No gridlines', 'noGridlines', {gridlines_enabled: 'false'}],
    ['Percentage Scale', 'percentageScale', {scale_type: 'PERCENTAGE'}],
    ['Percentage by color scale', 'percentageByColor', {scale_type: 'PERCENTAGE_COLOR'}],
  ],
  bars: [
    ['Wide Bars, no legend', 'wideNoLegend', {bar_mark_size: '25-35'}],
    ['Skinny Bars', 'skinnyBars', {bar_mark_size: '5-10'}],
    ['Skinny Bars, dynamic sizing', 'skinnyDynamic', {bar_mark_size: '5-10',
      dynamic_bar_width: 'true', legend_style: 'None'}],
    ['No axis', 'noAxis', {show_x_axis: 'false', show_y_axis: 'true'}],
    ['Axis with no labels', 'noAxisLabels', {show_x_axis: 'true', show_y_axis: 'true',
      x_axis_label: 'false', y_axis_label: 'true', legend_style: 'None'}],
    ['Legend on right', 'legendRight', {legend_style: 'Right'}],
    ['Legend on bottom', 'legendBottom', {legend_style: 'Bottom'}],
    ['Gridlines enabled', 'gridlinesEnabled', {gridlines_enabled: 'true'}],
    ['No gridlines', 'noGridlines', { gridlines_enabled: 'false' }],
  ],
  boxplot: [
    ['Show box plot horizontal', 'showBoxPlotHorizontal', { horizontalBoxplots: true }],
    ['Thick line width', 'thickLineWidth', { lineWidth: '5' }],
    ['Show gridlines', 'showGridlines', { gridlines_enabled: 'true' }],
    ['No gridlines', 'noGridlines', { gridlines_enabled: 'false' }],
    ['Remove measure axis label', 'noMeasureAxisLabel', { y_axis_label: 'true' }],
    ['Remove dimension axis label', 'noDimensionAxisLabel', { x_axis_label: 'true' }],
    ['Remove dimension axis', 'removeDimensionAxis', { show_x_axis: 'false' }],
    ['Remove measure axis', 'removeMeausreAxis', { show_y_axis: 'false' }],
  ],
  network: [
    ['Fixed linkColor', 'fixedLinkColor', { link_color: 'fixed'}, { no: 'link_color_shelf' }],
    ['Low Opacity', 'lowOpacity', { src_node_opacity: '30', link_opacity: '30' }],
    ['Link color based on link width', 'linkColorIsWidth', { link_color: 'aggregate' },
      { no: 'link_color_shelf'},
    ],
    ['Link distance: 120', 'linkDistance120', { link_distance: '120' }],
    ['Link distance: 25', 'linkDistance25', { link_distance: '25' }],
    ['All nodes as circles, nodes unified', 'circleNodesUnifed',
      {
        src_node_mark_type: 'circle',
        tgt_node_mark_type: 'circle',
        combined_node_mark_type: 'circle',
        same_domain: 'true'
      },
    ],
    ['Source as circle, target as square, combined as diamond', 'srcCircleCmbDiamondTgtSquare',
      {
        src_node_mark_type: 'circle',
        tgt_node_mark_type: 'square',
        combined_node_mark_type: 'diamond',
        same_domain: 'true',
      },
    ],
    ['Source as cross, target as triangle', 'sourceCrossTargetTriangle',
      {
        src_node_mark_type: 'cross',
        tgt_node_mark_type: 'triangle-up',
        same_domain: 'false',
      },
    ],
    ['Spread graph', 'spreadGraph',
      {
        link_strength: '5',
        node_friction: '0.8',
        link_distance: '100',
        layout_gravity: '0.3',
        node_charge: '-3000',
      },
    ],
    ['Bunched graph', 'bunchedGraph',
      {
        link_strength: '2',
        node_friction: '0.2',
        link_distance: '30',
        layout_gravity: '0.8',
        node_charge: '-600',
      },
    ],
    ['Low layout gravity', 'lowLayoutGravity', { layout_gravity: 0.2 }],
    ['High layout gravity', 'highLayoutGravity', { layout_gravity: 0.9 }],
    ['Low node charge', 'lowNodeCharge', { node_charge: -600 }],
    ['High node charge', 'highNodeCharge', { node_charge: -3000 }],
    ['Large node size spread', 'nodeSizeSpread', { node_size_range: '5-50' },
      { yes: ['src_node_size', 'tgt_node_size'] },
    ],
    ['Large link width spread', 'linkWidthSpread', { link_width_range: '2-30' }],
  ],
  dendrogram: [
    ['Link colors based on source nodes', 'linksBasedSource', {link_colors: 'source'}],
    ['Fixed link colors', 'fixedColoredLinks', {link_colors: 'source'}],
    ['Link colors based on target nodes', 'linksBasedTarget', {link_colors: 'target'}],
    ['Remove node labels', 'noNodeLabels', {show_node_labels: 'true'}],
  ],
  gauge: [
    ['Just Qualitative Arc', 'justQualitative', {gauge_key_measure_mark: 'NEEDLE',
      gauge_arc_thickness: '0', gauge_qualitative_arc_thickness: '20',
      qualitative_as_percentages: 'true', lowest_qualitative_percentage: '60',
      medium_qualitative_percentage: '80', highest_qualitative_percentage: '100',
      include_outside_compare_line: 'true'}, {yes: ['compare_shelf']}],
    ['Show axis ticks', 'axisTicks', {show_gauge_axis_ticks: 'true'}],
    ['No axis ticks', 'noAxisTicks', {show_gauge_axis_ticks: 'false'}],
    ['Both arcs with needle', 'bothArcsNeedle', {gauge_key_measure_mark: 'NEEDLE',
      gauge_arc_thickness: '20', gauge_qualitative_arc_thickness: '14',
      include_outside_compare_line: 'false'}],
    ['Only inner arc with line', 'innerArcOnly', {gauge_key_measure_mark: 'LINE',
      gauge_qualitative_arc_thickness: '0', gauge_arc_thickness: '22'}],
    ['Both arcs no key measure line', 'bothArcsNoLine', {gauge_arc_thickness: '20',
      gauge_qualitative_arc_thickness: '14', gauge_key_measure_mark: 'NONE'}],
  ],
};

function addTabWandAndClickHandler(visualType, mainTabName, clickFunction) {
   if (!arc.globals.enable_explore_interfaces) {
    return;
   }

  let $s = $(`#viz-${mainTabName}-tab .panel-header`);
  const tooltipString = 'data-toggle="tooltip" title="Explore Settings"';
  if ($s.length > 0) {
    $s.append(`<div class="viz-showme-container viz-showme-${mainTabName.toLowerCase()}-container">
                  <a class="viz-showme viz-showme-${visualType}-marks" ${tooltipString}>
                    <span class="explore-icon"></span>
                  </a>
              </div>`);

    $s.find(`.viz-showme-${visualType}-marks`)
      .off('click')
      .on('click', clickFunction);
  }
}

function createSettingsWand(vizDriver, setupClick, settingList) {

  function exploreSettings() {
    let showme = arc.showme.settings({
      vizDriver: vizDriver,
      settings: settingList,
    });

    if (!showme) {
      arc.misc.infoModal({
        title: 'Explore Settings',
        contentsHtml: `
        <p>
          Currently unable to explore settings as there are no fields to visualize
        </p>
        <p>
          Please either select some fields from the 'Data' tab
          or click on the 'Explore Visuals' option.
        </p>`,
      });

      return;
    }

    showme.setup();
  }

  if (setupClick) {
    addTabWandAndClickHandler(vizDriver.viz_type(), 'settings', exploreSettings);
  }

  return {
    id: 'settings',
    name: 'Settings',
    fn: exploreSettings,
  };
}

arc.showme.setupSettings = function(config) {
  let {
    vizDriver,
    setupClick,
  } = config;

  if (!arc.globals.enable_explore_interfaces) {
    return [];
  }

  if (setupClick) {
    removeTabWand(['viz-settings-tab', 'viz-color-tab', 'viz-basic-styling-tab']);
  }

  const settingsList = [];
  const vizType = vizDriver.viz_type();

  // Anything not to do with settings Tab
  const functionsToRun = [visualTypes, colorSettings, generalStyles];

  for (let i = 0; i < functionsToRun.length; i++) {
    let setting = functionsToRun[i](vizDriver, setupClick);
    if (setting !== undefined) {
      settingsList.push(setting);
    }
  }

  let settingsSettingsList;

  //shelfRequireiments: { required: ['shelfid1, shelfid2], forbidden: [shelfid3, shelfid4] }
  function makeSettingObject(name, id, settingOptions, shelfRequirements) {
    return Object.assign({ name, id, shelfRequirements }, settingOptions);
  }

  function createSettingList(settingsArray) {
    return settingsArray.map(setting => makeSettingObject(...setting));
  }

  const hasSetting = ['pie', 'radial', 'scatter', 'network', 'boxplot', 'dendrogram', 'gauge'];

  if (['trellis-bars', 'combo', 'trellis-groupedbars'].includes(vizType)) {
    settingsSettingsList = createSettingList(allSettingsSettingsOptions.bars);
  } else if (['trellis-line', 'trellis-areas'].includes(vizType)) {
    settingsSettingsList = createSettingList(allSettingsSettingsOptions.linesAndAreas);
  } else if (hasSetting.includes(vizType)) {
    settingsSettingsList = createSettingList(allSettingsSettingsOptions[vizType]);
  }

  if (settingsSettingsList) {
    settingsList.push(createSettingsWand(vizDriver, setupClick, settingsSettingsList));
  }

  return settingsList;
};

arc.showme.helpers = function(config) {
  let helpers = function() { return 'arc.showme.helpers'; };

  let baseVizDriver = config.vizDriver,
      $container,
      visualWidth,
      visualHeight = 370,
      showmeDiv = 'showme-viz';

  helpers.vizDriver = function(_) {
    if (!arguments.length) return baseVizDriver;
    baseVizDriver = _;
    return helpers;
  };

  helpers.setContainer = function(_) {
    $container = _;

    let w = $container.width();
    let vMargin = 20;
    let vMinWidth = 400;
    let visualCount = Math.floor(w / (vMinWidth + vMargin));

    visualWidth = Math.floor(w / visualCount) - vMargin;

    return helpers;
  };

  helpers.getDataRequest = function() {
    let dataRequest = baseVizDriver.viz().viz_getDataRequest();
    if (!dataRequest) {
      return undefined;
    }

    dataRequest.rb().limit(200);
    return dataRequest;
  };

  helpers.removeVisuals = function() {
    baseVizDriver.removeDataTables(showmeDiv);
    $('#' + showmeDiv).empty();
  };

  helpers.setupModal = function(pageCount, drawPageCb, includeBackButton) {
    let $m = $('#showme-modal');
    arc.misc.handleAlertContainerActivationOnModal($m);

    if (arc.misc.modalShown($m)) {
      return;
    }

    helpers.removeVisuals();

    if (!includeBackButton) {
      $m.find('.modal-footer #explore-back-button').remove();
    }

    let currPage = 1;
    $m.find('.modal-body')
      .height(($(window).height() - 220) + 'px');

    $m.modal('show', {
      keyboard: true,
    });

    $m.off('shown.bs.modal')
      .on('shown.bs.modal', function() {
        drawPageCb('showme-viz', currPage);

        $m.find('.next')
          .off('click')
          .on('click', function() {
            currPage++;

            if (currPage > pageCount) {
              currPage--;
            } else {
              drawPageCb('showme-viz', currPage);
            }
          });

        $m.find('.previous')
          .off('click')
          .on('click', function() {
            currPage--;

            if (currPage < 1) {
              currPage++;
            } else {
              drawPageCb('showme-viz', currPage);
            }
          });
      });
  };

  helpers.addVisualToContainer = function($visualContainer, v, prepend, initUserSettings) {
    let {
      uniqueVisualId,
      displayTitle,
      vizDriver,
    } = v;

    v.containerId = 'showme-vizcontainer-' + uniqueVisualId;
    v.chartId = 'showme-viz-' + uniqueVisualId;
    if (displayTitle) {
      if (baseVizDriver.title().trim().length > 0) {
        v.titleId = v.chartId + '-title';
      }

      if (baseVizDriver.subtitle().trim().length > 0) {
        v.subtitleId = v.chartId + '-subtitle';
      }
    }

    v.shelves_dom_id = `showme-app-shelves-${uniqueVisualId}`;
    if (vizDriver) {
      // If shelves_dom_id changes, we should update some of the inner state as well with setup.
      vizDriver.shelves_dom_id(v.shelves_dom_id);
      vizDriver.setup();
      const thisUserSettings = v.viz.viz_lib().user_settings();
      Object.entries(initUserSettings).forEach(([key, value]) => {
        const setting = thisUserSettings.find_by_id(key);
        if (key !== 'table_template') {
          setting?.value(value);
        }

        if (key === 'color_palette') {
          setting?.saveIntoVisual();
        }
      });
    }

    let vizName = v.name || '';

    const spinnerTopMargin = (visualHeight / 2) - 20;
    const spinnerLeftMargin = (visualWidth / 2) - 10;
    const spinnerStyle = `margin-left:${spinnerLeftMargin}px;margin-top:${spinnerTopMargin}px;`;

    let vizHtml = `
      <div class="showme-visual-container arc-viz-container" id="${v.containerId}"
           style="min-width:${visualWidth}px;max-width:${visualWidth}px;">

        <div class="viz-permanent" style="display:none;" id="${v.shelves_dom_id}"></div>
        <div class="badge badge-info badge-pill showme-visual-title">${vizName}</div>
        <div class="showme-spinner" style="${spinnerStyle}" id="showme-spinner-${uniqueVisualId}">
          <i class="fa fa-spinner fa-spin"></i>
        </div>
      </div>
      `;

    if (prepend) {
      $visualContainer.prepend(vizHtml);
    } else {
      $visualContainer.append(vizHtml);
    }

    let d3Container = d3.selectAll('#' + v.containerId);
    arc.visual.createTitleSubtitleArea({
      d3Container: d3Container,
      titleId: v.titleId,
      subtitleId: v.subtitleId,
      chartId: v.chartId,
    });

    /*
     * when displaying title/subtitle, adjust padding for individual visuals
     */
    if (displayTitle) {
      d3Container.selectAll('.dash-viz-wrapper')
                 .style('margin-top', '0px')
                 .style('padding-top', '5px');
    }

    d3Container.selectAll('#' + v.chartId)
               .classed('showme-visual', true)
               .style('height', visualHeight + 'px');
  };

  return helpers;
};

}());
