window.arc = window.arc || {};

(function() {
'use strict';

arc.showme = arc.showme || {};

arc.showme.settings = function(config) {
  let s = function() { return 'arc.showme.settings'; };

  let {
    vizDriver,
    isFont,
  } = config;

  let possibleSettings = config.settings,
      displayTitle,
      viz = vizDriver.viz();

  if (!vizDriver.dataResult()) {
    return;
  }

  let helpers = arc.showme.helpers({
        vizDriver,
        isFont,
      }),
      originalChartId = vizDriver.chart_div(),
      uniqueVisualId = 1,
      maxVisualsAtATime = 300;

  s.setup = function() {
    helpers.setupModal(Math.ceil(possibleSettings.length / maxVisualsAtATime), drawPage);
  };

  s.displayTitle = function(_) {
    if (!arguments.length) return displayTitle;
    displayTitle = _;
    return s;
  };

  /**
   * @param {*} repaintControls - some of the settings doesn't reflect the value,
   * unless we call repaintControl. False by default to
   * avoid the setting panel's unnecessary modification - (opening of the explore modal).
   * True when we actually want to apply the chosen style
   * and the setting panel needs to reflect the change!
   */
  function changeSettings(
    user_settings,
    newValues,
    visualPicked = false,
    settingsToSkip = [],
    repaintControls = false
    ) {
    const forbiddenKeys = ['name', 'id', 'shelfRequirements', ...settingsToSkip];
    const idOfSettingsToRepaint = ['table_template', 'measure_ranges', 'dim_to_color_mapping'];
    $.each(newValues, function(k, v) {
      if (forbiddenKeys.includes(k)) {
        return;
      }

      const setting = user_settings.find_by_id(k);
      setting.value(v);
      if (k === 'color_palette') {
        setting.saveIntoVisual();
        if (visualPicked) {
          setting.highlightLastStoredPalette();
        }
      }

      if (repaintControls && idOfSettingsToRepaint.includes(k)) {
        setting.repaintControl();
      }
    });
  }

  let viz_div;

  function drawPage(_viz_div, pageId) {
    let startIdx = (pageId - 1) * maxVisualsAtATime,
        endIdx = startIdx + maxVisualsAtATime,
        viz_div = _viz_div,
        $viz = $('#' + viz_div);

    helpers.removeVisuals();

    $viz.empty();
    $viz.append(`
      <div class="showme-wrapper"></div>
        <div class="showme-container">
        </div>
      </div>
    `);

    let $container = $viz.find('.showme-container'),
        settingsData = possibleSettings.slice(startIdx, endIdx);

    let vizData = settingsData.map(function(settings) {
      return {
        settings: settings,
      };
    });

    helpers.setContainer($container);

    vizData.forEach(function(v) {
      v.uniqueVisualId = uniqueVisualId++;
      v.displayTitle = displayTitle;
      v.name = v.settings.name;
      helpers.addVisualToContainer($container, v);
    });

    let savedConfig = saveSettings();
    savedConfig.vizData = vizData;
    savedConfig.titleElem = vizDriver.title_elem();
    savedConfig.subtitleElem = vizDriver.subtitle_elem();
    setupSelectionCb(savedConfig);
    drawVisuals(savedConfig, vizData, function() {
      if (selectionComplete) {
        return;
      }

      restoreSavedConfig(savedConfig);
      // re-target the visual chart div to the original one
      vizDriver.localEnableAnimations(true);
      viz.viz_drawVisual(originalChartId);
    });

    return s;
  }

  let selectionComplete = false;

  function setupSelectionCb(savedConfig) {
    let $viz = $('#showme-viz'),
        $container = $viz.find('.showme-container'),
        user_settings = vizDriver.user_settings();

    let $visuals = $container.find('.showme-visual-container');
    $visuals.off('click.picked')
            .on('click.picked', function() {
              let vizId = $(this).find('.showme-visual').attr('id'),
                  vizData = savedConfig.vizData;

              vizData.some(function(v) {
                if (v.chartId !== vizId) {
                  return false;
                }

                selectionComplete = true;

                restoreSavedConfig(savedConfig);
                changeSettings(user_settings, v.settings, true, [], true);
                vizDriver.page_needs_save();

                vizDriver.localEnableAnimations(true);

                const report_info = viz.viz_getSettings();
                if (arc.globals.is_dashboard && !arc.globals.isInModal()) {
                  vizDriver.report().saved_data(JSON.stringify({
                    report_type: vizDriver.viz_type(),
                    report_data: report_info,
                  }));
                }

                viz.viz_drawVisual(originalChartId);

                let $m = $('#showme-modal');
                $m.modal('hide');
                helpers.removeVisuals();

                return true;
              });
            });
  }

  function saveSettings() {
    let user_settings = vizDriver.user_settings();

    // save settings
    let savedSettings = {};
    user_settings.settings().forEach(function(s) {
      savedSettings[s.id()] = s.value();
    });

    // Use only the top 200 rows
    let dataResult = vizDriver.dataResult();
    let originalRows = dataResult.rows();
    dataResult.rows(originalRows.slice(0, 200));
    viz.viz_setDataResult(dataResult);

    // disable marks
    vizDriver.markActivationEnabled(false);

    return {
      originalRows: originalRows,
      dataResult: dataResult,
      savedSettings: savedSettings,
    };
  }

  function drawVisuals(savedConfig, vizData, cb) {
    let user_settings = vizDriver.user_settings();
    let idx = 0;

    function drawSingleVisual() {
      if (vizData.length <= idx) {
        cb();
        return;
      }

      if (selectionComplete) {
        cb();
        return;
      }

      let v = vizData[idx];
      idx++;

      try {
        if ($('#' + v.chartId).length > 0) {
          const chartIdSplit = v.chartId.split('-');
          const spinnerId = chartIdSplit[chartIdSplit.length - 1];
          $(`#showme-spinner-${spinnerId}`).hide();
          restoreSettings(user_settings, savedConfig, ['use_dataset_color_settings']);
          changeSettings(user_settings, v.settings);
          if (displayTitle) {
            vizDriver.title_elem(v.titleId);
            vizDriver.subtitle_elem(v.subtitleId);
          }

          vizDriver.localEnableAnimations(false);
          viz.viz_drawVisual(v.chartId, false);
        }
      } catch (e) {
        console.log(e);
      }

      setTimeout(drawSingleVisual, 0);
    }

    $('#showme-modal .modal-footer .btn').off('click.reset').on('click.reset', () => {
      changeSettings(user_settings, savedConfig.savedSettings);
    });

    drawSingleVisual();
  }

  function restoreSettings(user_settings, savedConfig, settingsToSkip) {
    changeSettings(user_settings, savedConfig.savedSettings, false, settingsToSkip);
  }

  function restoreSavedConfig(savedConfig) {
    let user_settings = vizDriver.user_settings();

    // restore title and subtitle elem
    vizDriver.title_elem(savedConfig.titleElem);
    vizDriver.subtitle_elem(savedConfig.subtitleElem);

    // restore data
    savedConfig.dataResult.rows(savedConfig.originalRows);
    viz.viz_setDataResult(savedConfig.dataResult);

    // enable marks
    vizDriver.markActivationEnabled(true);
    // Instead of creating separate (invisible) settings/shelves for the exploratory visuals
    // we use the same connected settings.
    // Unfortunately use_dataset_color_settings effect's other settings,
    // so when we restore the setting, it will affect the already drawn exploratory visuals
    // Full restore will be handled on $('#showme-modal .modal-footer .btn').on("click.reset")!
    // TODO: refactor this section not use these settings at all!
    restoreSettings(user_settings, savedConfig, ['use_dataset_color_settings']);
  }

  return s;
};

}());
