window.arc = window.arc || {};

/**
 * This are the visual config for the visual explore wand inside the visual editor
 */

(function() {
'use strict';

arc.showme = arc.showme || {};

arc.showme.visualOptions = function() {
  let visualOptions = function () {
    return 'arc.showme.visualOptions';
  };

  /**
   *otherData keys: isGeo, isDate, isTrellis, suggestDate, secondaryaxis, ascending
   */
  function createShelf(shelfId, colType, required, mustBeNumeric, catchall, otherData) {
    let shelfData = { shelfId, required, mustBeNumeric, catchall };
    colType = colType || 'UNKNOWN';
    shelfData.displayName = `${shelfId[0].toUpperCase()}${shelfId.slice(1).replace('_', ' ')}`;
    shelfData.colproperty = arc.report.columnTypeEnum[colType.toUpperCase()];
    Object.assign(shelfData, otherData);

    return shelfData;
  }

  function constraints(dimMin, dimMax, aggMin, aggMax) {
    return {
      minimums: {
        dimensions: dimMin,
        aggregates: aggMin,
      },
      maximums: {
        dimensions: dimMax,
        aggregates: aggMax,
      },
    };
  }

  visualOptions.noVisualExplore = [
    'column-stats',
    'extension',
    'flow',
    'funnel',
    'html',
    'link',
    'rawtable',
    'search',
  ];

  visualOptions.options = {
    requireDate: [
      {
        name: 'Calendar Heatmap',
        vizType: 'calendar-heatmap',
        id: 'calenderHeatmap',
        vizCreate: arc.visual.plugins.chm,
        constraints: constraints(1, 1, 1, undefined),
        isDate: true,
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true, false, false),
          createShelf('aggregates_shelf', 'aggregate', true, true, false, { ascending: true }),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
    ],
    requireGeoBasic: [
      {
        name: 'Map',
        vizType: 'map',
        id: 'map',
        vizCreate: arc.visual.plugins.map,
        isGeo: true,
        shelves: [
          createShelf('geo_shelf', 'dimension', true, false, true, { isGeo: true }),
          createShelf('x_shelf', 'dimension', false, false, false),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, true, true),
        ],
      },
    ],
    requireGeoInteractive: [
      {
        name: 'Interactive Map',
        vizType: 'leaflet',
        id: 'leaflet',
        vizCreate: arc.visual.plugins.leaflet,
        isGeo: true,
        shelves: [
          createShelf('geo_shelf', 'dimension', true, false, true, { isGeo: true }),
          createShelf('aggregates_shelf', 'aggregate', false, true, false),
          createShelf('tooltip_shelf', 'dimension', false, false, true),
          createShelf('color_shelf', 'aggregate'),
        ],
      },
    ],
    requireDateAndCardinality: [
      {
        name: 'Trellised Calendar Heatmap',
        vizType: 'calendar-heatmap',
        id: 'trellisedHeatmap',
        vizCreate: arc.visual.plugins.chm,
        constraints: constraints(2, 3, 1, undefined),
        maxGenDim: 0,
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true, false, false, { isDate: true }),
          createShelf('y_shelf', 'dimension', true, false, false, { isTrellis: true }),
          createShelf('x_shelf', 'dimension', false, false, false, { isTrellis: true }),
          createShelf('aggregates_shelf', 'aggregate', true, true, false, { ascending: true }),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
    ],
    requireLowCardinality: [
      {
        name: 'Trellis bar',
        vizType: 'trellis-bars',
        id: 'trellis_bar',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },
        constraints: constraints(2, 3, 1, undefined),
        maxGenDim: 2,
        shelves: [
          createShelf('y_shelf', 'dimension', true, false, false, { isTrellis: true }),
          createShelf('x_shelf', 'dimension', true),
          createShelf('color_shelf', 'dimension', false),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Trellised sparklines',
        vizType: 'sparklines',
        id: 'sparklinesTrellised',
        vizCreate: arc.visual.plugins.sparklines,
        constraints: constraints(2, 2, 1, 1),
        maxGenDim: 1,
        shelves: [
          createShelf('y_shelf', 'dimension', true, false, false, { isTrellis: true }),
          createShelf('x_shelf', 'dimension', true, false),
          createShelf('y_shelf', 'aggregate', true, true),
        ]
      },
    ],
    requireMediumCardinality: [
      {
        name: 'Trellised bullet',
        vizType: 'bullet',
        id: 'bullet_with_trellis',
        vizCreate: arc.visual.plugins.bullet,
        constraints: constraints(1, 2, 1, undefined),
        maxGenDim: 0,
        shelves: [
          createShelf('y_shelf', 'dimension', true, false, false, { isTrellis: true }),
          createShelf('x_shelf', 'dimension', false, false, false, { isTrellis: true }),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('compare_shelf', 'aggregate', false, true),
          createShelf('projection_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Trellised gauge',
        vizType: 'gauge',
        id: 'gauge_with_trellis',
        vizCreate: arc.visual.plugins.gauge,
        constraints: constraints(1, 2, 1, undefined),
        maxGenDim: 0,
        shelves: [
          createShelf('y_shelf', 'dimension', true, false, false, { isTrellis: true }),
          createShelf('x_shelf', 'dimension', false, false, false, { isTrellis: true }),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('compare_shelf', 'aggregate', false, true),
          createShelf('projection_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
    ],
    generalOptions: [
      {
        name: 'Multi-Aggregate Scatter',
        vizType: 'scatter',
        id: 'scatter_two_dims_twothree_agg',
        vizCreate: arc.visual.plugins.scatter,
        constraints: constraints(2, 2, 2, 3),
        shelves: [
          createShelf('x_shelf', 'unknown', true, false),
          createShelf('y_shelf', 'unknown', true, false),
          createShelf('color_shelf', 'unknown', true),
          createShelf('tooltip_shelf', 'unknown', true),
          createShelf('size_shelf', 'aggregate', false, true),
        ],
        settings: {
          gridlines_enabled: 'true',
          mark_size_range: '5-50',
        },
      },
      {
        name: 'KPI',
        vizType: 'kpi',
        id: 'kpi',
        vizCreate: arc.visual.plugins.kpi,
        constraints: constraints(0, 0, 1, 1),
        shelves: [
          createShelf('aggregates_shelf', 'aggregate', true, false),
        ],
      },
      {
        name: 'Metrics',
        vizType: 'table',
        id: 'headline_table_multiple_aggs',
        vizCreate: arc.visual.plugins.table,
        constraints: constraints(0, 0, 1, undefined),
        shelves: [
          createShelf('aggregates_shelf', 'aggregate', true, false, true),
        ],
        settings: {
          table_full_width: 'false',
          table_template: 'table-template-7',
        },
      },
      {
        name: 'Gauge',
        vizType: 'gauge',
        vizCreate: arc.visual.plugins.gauge,
        id: 'gauge',
        constraints: constraints(0, 0, 1, undefined),
        shelves: [
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('compare_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('projection_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          show_gauge_axis_ticks: 'false',
        },
      },
      {
        name: 'Bullet',
        vizType: 'bullet',
        id: 'bullet',
        vizCreate: arc.visual.plugins.bullet,
        constraints: constraints(0, 0, 1, 8),
        shelves: [
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('compare_shelf', 'aggregate', false, true),
          createShelf('projection_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('qualitative_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Horizontal bars, Pct Scale',
        vizType: 'trellis-bars',
        id: 'horizontal_bars',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('y_shelf', 'dimension', true),
          createShelf('x_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
          scale_type: 'PERCENTAGE',
        },
      },
      {
        name: 'Horizontal bars',
        vizType: 'trellis-bars',
        id: 'horizontal_bars',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(1, 1, 1, undefined),
        shelves: [
          createShelf('y_shelf', 'dimension', true),
          createShelf('x_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Horizontal stacked bars',
        vizType: 'trellis-bars',
        id: 'horizontal_stacked_bar',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('color_shelf', 'dimension', true),
          createShelf('y_shelf', 'dimension', true),
          createShelf('x_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Horizontal stacked bars',
        vizType: 'trellis-bars',
        id: 'horizontal_stacked_bar',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('y_shelf', 'dimension', true),
          createShelf('color_shelf', 'dimension', true),
          createShelf('x_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Horizontal stacked bars, Pct Scale',
        vizType: 'trellis-bars',
        id: 'horizontal_stacked_bar',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('color_shelf', 'dimension', true),
          createShelf('y_shelf', 'dimension', true),
          createShelf('x_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
          scale_type: 'PERCENTAGE',
        },
      },
      {
        name: 'Horizontal stacked bars, Pct Scale',
        vizType: 'trellis-bars',
        id: 'horizontal_stacked_bar',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('y_shelf', 'dimension', true),
          createShelf('color_shelf', 'dimension', true),
          createShelf('x_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
          scale_type: 'PERCENTAGE',
        },
      },
      {
        name: 'Vertical bar',
        vizType: 'trellis-bars',
        id: 'vertical_bar',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },
        constraints: constraints(1, 2, 1, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'dimension', false, false),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Line measures',
        vizType: 'trellis-lines',
        id: 'lines-measures',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('line');
        },
        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false, false, { ascending: true }),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'aggregate', true, true, true),
        ],
      },
      {
        name: 'Headline table',
        vizType: 'table',
        id: 'headline_table_single_agg',
        vizCreate: arc.visual.plugins.table,
        constraints: constraints(0, 0, 1, 1),
        shelves: [
          createShelf('aggregates_shelf', 'aggregate', true, false, true),
        ],
        settings: {
          table_full_width: 'false',
          table_template: 'table-template-7',
        },
      },
      {
        name: 'Headline table',
        vizType: 'table',
        id: 'headline_table_single_dim',
        vizCreate: arc.visual.plugins.table,
        constraints: constraints(1, 1, 0, 0),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
        ],
        settings: {
          table_template: 'table-template-7',
        },
      },
      {
        name: 'Scatter',
        vizType: 'scatter',
        id: 'scatter_one_dim_one_agg',
        vizCreate: arc.visual.plugins.scatter,
        constraints: constraints(1, 2, 1, 3),
        shelves: [
          createShelf('x_shelf', 'unknown', true),
          createShelf('y_shelf', 'unknown', true),
          createShelf('color_shelf', 'unknown', false),
          createShelf('tooltip_shelf', 'dimension', false, false),
          createShelf('size_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'unknown', false, false),
        ],
        settings: {
          gridlines_enabled: 'true',
          mark_size_range: '5-50',
        },
      },
      {
        name: 'Dimension scatter',
        vizType: 'scatter',
        id: 'scatter_dims',
        vizCreate: arc.visual.plugins.scatter,
        constraints: constraints(2, undefined, 0, 2),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('y_shelf', 'dimension', true),
          createShelf('color_shelf', 'unknown', false, false),
          createShelf('size_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'unknown', false, false, true),
        ],
        settings: {
          gridlines_enabled: 'true',
          mark_size_range: '5-50',
        },
      },
      {
        name: 'Trellised measures',
        vizType: 'trellis-bars',
        id: 'trellised_measure_bars',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('bar');
        },

        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('y_shelf', 'dimension', true),
          createShelf('y_shelf', 'aggregate', true, true, true),
        ],
        settings: {
          scale_agg_independent: 'true',
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Grouped bars',
        vizType: 'trellis-groupedbars',
        id: 'grouped_bars',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('groupedbar');
        },

        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'aggregate', true, true, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Grouped bars w/dimensions',
        vizType: 'trellis-groupedbars',
        id: 'grouped_bars_dim',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('groupedbar');
        },

        constraints: constraints(2, undefined, 1, 3),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'dimension', true, false, true),
        ],
        settings: {
          bar_mark_size: '20-30',
        },
      },
      {
        name: 'Packed bubbles',
        vizType: 'packed-bubbles',
        id: 'packed_bubbles',
        vizCreate: arc.visual.plugins.pb,
        constraints: constraints(1, 1, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Line measures',
        vizType: 'trellis-lines',
        id: 'lines-measures',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('line');
        },
        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false, false, { ascending: true }),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'aggregate', true, true, true),
        ],
      },
      {
        name: 'Line dimensions',
        vizType: 'trellis-lines',
        id: 'line-dimensions',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('line');
        },
        constraints: constraints(1, undefined, 1, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false, false, { ascending: true }),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'dimension', false, false, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Line dimensions with trends',
        vizType: 'trellis-lines',
        id: 'line-dimensions-trends',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('line');
        },
        constraints: constraints(1, undefined, 1, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false, false, { ascending: true }),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'dimension', false, false, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          trendline_linear: 'true',
          trendline_color: 'true',
        },
      },
      {
        name: 'Scatter w/trendline',
        vizType: 'scatter',
        id: 'scatterWithTrendline',
        vizCreate: arc.visual.plugins.scatter,
        constraints: constraints(1, 2, 1, 2),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'unknown', false),
          createShelf('size_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'unknown', false, false),
        ],
        settings: {
          trendline_linear: 'true',
          gridlines_enabled: 'true',
          mark_size_range: '5-50',
        },
      },
      {
        name: 'Color line w/secondary axis',
        vizType: 'trellis-lines',
        id: 'color_with_secondary_axis',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('line');
        },

        constraints: constraints(1, 1, 2, 2),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false, false, { ascending: true }),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'aggregate', true, true, false, { secondaryaxis: true }),
        ],
      },
      {
        name: 'Combo chart, multiple lines',
        vizType: 'combo',
        id: 'combo_multiple_lines',
        vizCreate: arc.visual.plugins.combined,
        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('bar_measure', 'aggregate', true, true),
          createShelf('line_measure', 'aggregate', true, true, true),
        ],
      },
      {
        name: 'Combo chart, stacked bars',
        vizType: 'combo',
        id: 'combo_stacked_bars',
        vizCreate: arc.visual.plugins.combined,
        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true),
          createShelf('line_measure', 'aggregate', true, true),
          createShelf('bar_measure', 'aggregate', true, true, true),
        ],
      },
      {
        name: 'Combo chart w/secondary axis',
        vizType: 'combo',
        id: 'combo_with_secondary_axis',
        vizCreate: arc.visual.plugins.combined,
        constraints: constraints(1, 1, 2, 2),
        shelves: [
          createShelf('x_shelf'),
          createShelf('bar_measure'),
          createShelf('line_measure', 'aggregate', true, true, false, { secondaryaxis: true }),
        ],
      },
      {
        name: 'Radial',
        vizType: 'radial',
        id: 'radial',
        vizCreate: arc.visual.plugins.radial,
        constraints: constraints(1, 1, 2, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('aggregates_shelf', 'aggregate', true, true, true),
        ],
      },
      {
        name: 'Area',
        vizType: 'trellis-areas',
        id: 'area',
        vizCreate: function () {
          return arc.visual.plugins.bar().mark_type('area');
        },

        constraints: constraints(1, 2, 1, undefined),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false),
          createShelf('y_shelf', 'aggregate', true, true),
          createShelf('color_shelf', 'dimension', false, false),
          createShelf('color_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Pie',
        vizType: 'pie',
        id: 'pie',
        vizCreate: arc.visual.plugins.pie,
        constraints: constraints(1, undefined, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
          createShelf('dimensions_shelf', 'dimension', false, false, true),
        ],
        settings: {
          legend_style: 'None',
          arc_width: 0,
        },
      },
      {
        name: 'Chord',
        vizType: 'chord',
        id: 'chord',
        vizCreate: arc.visual.plugins.chord,
        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Treemap',
        vizType: 'treemap',
        id: 'treemap',
        vizCreate: arc.visual.plugins.treemap,
        constraints: constraints(2, 4, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('dimensions_shelf', 'dimension', false, false, true),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Network',
        vizType: 'network',
        id: 'network',
        vizCreate: arc.visual.plugins.network,
        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('aggregates_shelf', 'aggregate', true),
          createShelf('link_color_shelf', 'aggregate', false, false),
          createShelf('source_node_size_shelf', 'aggregate', false, true),
          createShelf('target_node_size_shelf', 'aggregate', false, true),
          createShelf('source_node_color_shelf', 'aggregate', false, false),
          createShelf('target_node_color_shelf', 'aggregate', false, false),
          createShelf('tooltip_shelf', false, false),
        ],
      },
      {
        name: 'Correlation heatmap',
        vizType: 'corelation',
        id: 'correlation_heatmap',
        vizCreate: arc.visual.plugins.correlation,
        constraints: constraints(2, 2, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('dimensions_shelf', 'dimension', true),
          createShelf('aggregates_shelf', 'aggregate', true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Dendrogram',
        vizType: 'dendrogram',
        id: 'dendrogram',
        vizCreate: arc.visual.plugins.dendrogram,
        constraints: constraints(2, undefined, 1, 1),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true, false, true),
          createShelf('aggregates_shelf', 'aggregate', true, true),
        ],
      },
      {
        name: 'Correlation flow',
        vizType: 'corelation-flow',
        id: 'correlation_flow',
        vizCreate: arc.visual.plugins.correlation_flow,
        constraints: constraints(2, undefined, 1, undefined),
        shelves: [
          createShelf('dimensions_shelf', 'dimension', true, false, true),
          createShelf('aggregates_shelf', 'aggregate', true, true),
          createShelf('tooltip_shelf', 'aggregate', false, false, true),
        ],
      },
      {
        name: 'Crosstab',
        vizType: 'crosstab',
        id: 'crosstab',
        vizCreate: arc.visual.plugins.crosstab,
        constraints: constraints(1, 4, 1, undefined),
        shelves: [
          createShelf('y_dimensions_shelf', 'dimension', false),
          createShelf('x_dimensions_shelf', 'dimension', false),
          createShelf('y_dimensions_shelf', 'dimension', false),
          createShelf('x_dimensions_shelf', 'dimension', false),
          createShelf('aggregates_shelf', 'aggregate', true, false, true),
        ],
      },
      {
        name: 'Striped table',
        vizType: 'table',
        id: 'striped_table',
        vizCreate: arc.visual.plugins.table,
        constraints: {},
        shelves: [
          createShelf('dimensions_shelf', 'dimension', false, false, true),
          createShelf('aggregates_shelf', 'aggregate', false, false, true),
        ],
        settings: {
          table_full_width: 'true',
          table_template: 'table-template-6',
        },
      },
      {
        name: 'Sparkline',
        vizType: 'sparklines',
        id: 'sparkline',
        vizCreate: arc.visual.plugins.sparklines,
        constraints: constraints(1, 1, 1, 1),
        shelves: [
          createShelf('x_shelf', 'dimension', true, false),
          createShelf('y_shelf', 'aggregate', true, true),
        ],
      },
      {
        name: 'Crosstab, horizontal',
        vizType: 'crosstab',
        id: 'crosstab-horizontal',
        vizCreate: arc.visual.plugins.crosstab,
        constraints: constraints(1, 4, 1, undefined),
        shelves: [
          createShelf('x_dimensions_shelf', 'dimension', false),
          createShelf('y_dimensions_shelf', 'dimension', false),
          createShelf('x_dimensions_shelf', 'dimension', false),
          createShelf('y_dimensions_shelf', 'dimension', false),
          createShelf('aggregates_shelf', 'aggregate', true, false, true),
        ],
      },
      {
        name: 'Multi-aggregate KPI',
        vizType: 'kpi',
        id: 'kpi',
        vizCreate: arc.visual.plugins.kpi,
        constraints: constraints(0, 0, 2, 3),
        shelves: [
          createShelf('aggregates_shelf', 'aggregate', true, false),
          createShelf('compare_shelf', 'aggregate', false, true),
          createShelf('tooltip_shelf', 'aggregate', false),
        ],
      },
    ],
  };

  // Add a filters_shelf to each option so we can re-use filters from existing visuals
  $.each(visualOptions.options, function(k, v) {
    v.forEach(function(c) {
      c.shelves.push(createShelf('filters_shelf', false, false, false, true));
    });
  });

  return visualOptions;
};

// TODO: Visuals to add in the future
      // {
      //   name: 'Histogram',
      //   vizType: 'histogram',
      //   id: 'histogram',
      //   vizCreate: arc.visual.plugins.histogram,
      //   constraints: constraints(0, 0, 1, 1),
      //   shelves: [
      //     createShelf('aggregates_shelf', 'dimension', true, true),
      //     createShelf('filters_shelf', false, false, false, true),
      //   ],
      // },
      // {
      //   name: 'Boxplot',
      //   vizType: 'boxplot',
      //   id: 'boxplot',
      //   vizCreate: arc.visual.plugins.boxplot,
      //   constraints: constraints(1, 1, 1, 1),
      //   shelves: [
      //     createShelf('aggregates_shelf', 'aggregate', true, true),
      //     createShelf('dimensions_shelf', 'dimension', true, false),
      //     createShelf('x_shelf', 'dimension', false, false),
      //     createShelf('y_shelf', 'dimension', false, false),
      //     createShelf('filters_shelf', false, false, false, true),
      //   ]
      // },
}());
