window.arc = window.arc || {};

/*
 * The code in this file is used for the 'Show Me' button in the single visual
 * builder.
 */

(function() {
'use strict';

arc.showme = arc.showme || {};

arc.showme.visuals = function(config) {
  let showme = function() { return 'arc.showme.visuals'; };

  const visualOptions = arc.showme.visualOptions().options;

  let {
    vizDriver,
    showmeVisualConfig,
    isSearchShowme,
    displayedQuery,
    pickedCb,
  } = config;

  const initUserSettings = vizDriver.viz().viz_getSettings().user_settings;

  let helpers = arc.showme.helpers({
        vizDriver: vizDriver,
      }),
      maxVisualsAtATime = 300,
      drawnVisuals = [],
      possibleVisuals,
      dateVisualsAdded = false,
      trellisedAdded = false;

  function tryApplyConstraints(visuals) {
    if (!showmeVisualConfig) {
      return visuals;
    }

    let filteredVisuals = [...visuals];

    // prioritize viz type
    if (showmeVisualConfig.vizId) {
      const visualsMatchingId = visuals
        .filter(({ visualData }) => visualData.id === showmeVisualConfig.vizId);

      if (visualsMatchingId.length > 0) {
        filteredVisuals = visualsMatchingId;
      }
    } else if (showmeVisualConfig.vizType) {
      const visualsMatchingTypes = visuals
        .filter(({ visualData }) => visualData.vizType === showmeVisualConfig.vizType);

      if (visualsMatchingTypes.length > 0) {
        if (showmeVisualConfig.vizTypeElement) {
          showmeVisualConfig.vizTypeElement.matchingFound = true;
        }

        filteredVisuals = visualsMatchingTypes;
      }
    }

    if (!showmeVisualConfig.settings || Object.keys(showmeVisualConfig.settings).length === 0) {
      return filteredVisuals;
    }

    const visualsMatchingAtleastOneSetting = visuals.filter(({ visualData: { settings } }) => {
      if (!settings) {
        return false;
      }

      return Object.entries(showmeVisualConfig.settings)
        .some(([key, value]) => settings[key] !== value);
    });

    if (visualsMatchingAtleastOneSetting.length > 0) {
      filteredVisuals = visualsMatchingAtleastOneSetting;
    }

    return filteredVisuals;
  }

  showme.pickVisual = function(config) {
    let {
      elementOrdering,
      showmeVisualConfig,
    } = config;

    showmeVisualConfig = showmeVisualConfig || {};
    let {
      vizType,
    } = showmeVisualConfig;

    let dr = helpers.getDataRequest();
    if (dr === undefined) {
      return;
    }

    let columns = dr.columns();
    possibleVisuals = findPossibleVisuals({
      elementOrdering,
      columns,
      vizType,
    });

    if (possibleVisuals.length === 0) {
      return;
    }

    possibleVisuals = tryApplyConstraints(possibleVisuals);
    let pickedVisual = possibleVisuals[0];
    return pickedVisual;
  };

  showme.setup = function(addBackButton) {
    let dr = helpers.getDataRequest();
    let initialVizType = vizDriver.viz_type();
    if (dr === undefined) {
      arc.misc.displayVisualType('table');
      vizDriver.report().switchViz('table');
      vizDriver = vizDriver.report().viz();
      dr = helpers.vizDriver(vizDriver).getDataRequest();
    }

    let columns = dr.columns();
    if (dr.isDefault()) {
      const $vizCreationModal = $('#showme-new-visual-modal');
      if (arc.misc.modalShown($vizCreationModal)) {
        return;
      }

      const $vizEmptyList = $('<div id="explore-empty">Select the fields you are' +
        ' interested in to view the available visuals appropriate to your selections</div>');
      const $aggShelf = $vizCreationModal.find('.agg-app-params');
      const $dimShelf = $vizCreationModal.find('.dim-app-params');
      const dataset = vizDriver.dataset();
      arc.misc.setupDimAggSearch('#showme-new-visual-modal', '');

      $vizCreationModal.modal('show');
      const $possibleVisualList = $('#possible-visual-list');
      $possibleVisualList.empty();
      $possibleVisualList.append($vizEmptyList);
      arc.report.datasetSetupCols({
        dataset,
        includeAggregateColumns: true,
        $dimShelf,
        $aggShelf,
        includeSegment: true,
      });

      let shelfTracker = ShelfTracker(this, dataset, $possibleVisualList, initialVizType);
      $('#new-visual-explore').attr('disabled', true);

      $dimShelf.find('.selection_element').off('click.selectShelf')
                                          .on('click.selectShelf', shelfTracker.selectShelf);
      $aggShelf.find('.selection_element').off('click.selectShelf')
                                          .on('click.selectShelf', shelfTracker.selectShelf);

      $('#new-visual-explore').off('click.exploreVisuals')
                              .on('click.exploreVisuals', shelfTracker.exploreVisuals);

      let $backButton = $('#explore-back-button');
      if ($backButton.length === 0) {
        const $modalFooter = $('#showme-modal .modal-footer');
        $backButton = $('<button id="explore-back-button" type="button" ' +
          'class="btn btn-secondary"><i class="fa fa-angle-double-left"></i> Back</button>');
        $modalFooter.prepend($backButton);
      }

      $backButton.off('click.showmeBack')
        .on('click.showmeBack', function(event) {
          event.stopPropagation();

          const $modal = $('#showme-new-visual-modal');
          if (arc.misc.modalShown($modal)) {
            return;
          }

          $('#showme-modal').modal('hide');
          $modal.modal('show');
          shelfTracker.updateVisuals();
          trellisedAdded = false;
          dateVisualsAdded = false;
        });

      $('#showme-new-visual-modal .explore-cancel-btn').off('click.cancel')
        .on('click.cancel', shelfTracker.cancel);

      return;
    } else {
      $('#showme-modal .showme-cancel').off('click.cancel');
    }

    possibleVisuals = findPossibleVisuals({
      columns,
    });

    const pageCount = Math.ceil(possibleVisuals.length / maxVisualsAtATime);
    helpers.setupModal(pageCount, drawPage, addBackButton);
  };

  function shelvesIterate(visualConfig, cb) {
    visualConfig.shelves.forEach(function (shelf) {
      let colIdxs = [];

      if (shelf.colIdxes) {
        colIdxs = shelf.colIdxes;
      }

      colIdxs.forEach(function(colIdx) {
        cb(shelf, colIdx);
      });
    });
  }

  /*
   * The first time we get data back from a request, we look at the actual
   * contents of the data, if there is a date column, graph those
   */
  function getDateVisuals($container, dataRequest, data) {
    if (dateVisualsAdded) {
      return undefined;
    }

    dateVisualsAdded = true;

    /*
     * If any column is date, use calendar heat map
     */
    const cols = dataRequest.columns();
    let dateCols = [];
    let dateConfigs = [];
    for (let i = 0; i < cols.length; i++) {
      let dateConfig = arc.misc.columnIsDate(data.rows, i);
      if (dateConfig) {
        dateCols.push(i);
        dateConfigs.push(dateConfig);
      }
    }

    let prepend = true;
    if (dateConfigs.length === 1) {
      const newPossibleVisualTypes = visualOptions.requireDate;
      prepend = !dateConfigs[0].is_datetime;
      const colInfo = getColumnInfo(cols);
      colInfo.dateCols = dateCols;
      const vizData = [];
      const dimCount = vizDriver.dim_cnt(),
            aggCount = vizDriver.agg_cnt();
      newPossibleVisualTypes.forEach(function (visualData) {

        if (checkConstraints(visualData, colInfo, dimCount, aggCount)) {
          visualData = assignColumns(visualData, colInfo);
          const viz = handleIfSupported(visualData);
          visualData.colInfo = colInfo;
          if (viz) {
            viz.visualData = visualData;
            vizData.push(viz);
          }
        }
      });

      drawVisuals($container, vizData, prepend);
      return dateCols;
    }

    return undefined;
  }

  function getTrellisedVisuals($container, dataRequest, data, dateCols) {
    if (trellisedAdded) return;

    trellisedAdded = true;
    const dimCount = vizDriver.dim_cnt(),
          aggCount = vizDriver.agg_cnt();
    if (dimCount < 1) return;

    const cols = dataRequest.columns();
    const colUniqueData = {};
    const colCardCount = [];
    for (let i = 0; i < dimCount; i++) {
      colUniqueData[i] = {};
      colCardCount.push(0);
    }

    data.rows.forEach(function (row) {
      for (let i = 0; i < dimCount; i++) {
        if (!colUniqueData[i][row[i]]) {
          colUniqueData[i][row[i]] = true;
          colCardCount[i]++;
        }
      }
    });

    const trellisLimitSmall = 17;
    const trellisLimitLarge = 40;
    const trellisSmallCols = [];
    const trellisLargeCols = [];
    let maxCard = colCardCount[0];
    let minCard = colCardCount[0];
    let totalLargeTrellis = 1;

    for (let i = 0; i < dimCount; i++) {
      if (colCardCount[i] > maxCard) maxCard = colCardCount[i];
      if (colCardCount[i] < minCard) minCard = colCardCount[i];
      if (colCardCount[i] < trellisLimitSmall) {
        trellisSmallCols.push(i);
      }

      if (colCardCount[i] < trellisLimitLarge) {
        trellisLargeCols.push(i);
        totalLargeTrellis = totalLargeTrellis * colCardCount[i];
      }
    }

    const colInfo = getColumnInfo(cols);
    const includeSmall = dimCount >= 2 && ((maxCard - minCard) > minCard);
    const vizData = [];
      let trellisedVisualTypes = [];
    const vizOptions = arc.showme.visualOptions().options;
    function addVisualOption(vizOption, specialDim, colInfo) {
      if (vizOption.maxGenDim >= dimCount - specialDim) {
        vizOption.colInfo = colInfo;
        trellisedVisualTypes.push(vizOption);
      }
    }

    if (includeSmall && trellisSmallCols.length > 0) {
      const colInfoSmall = Object.assign({}, colInfo);
      colInfoSmall.trellisCols = trellisSmallCols;
      if (dateCols) {
        colInfoSmall.dateCols = dateCols;
        vizOptions.requireDateAndCardinality.forEach(function (vizOption) {
          addVisualOption(vizOption, trellisSmallCols.length + 1, colInfoSmall);
        });
      }

      vizOptions.requireLowCardinality.forEach(function (vizOption) {
        addVisualOption(vizOption, trellisSmallCols.length, colInfoSmall);
      });
    }

    const numTrellis = trellisLargeCols.length;
    if (numTrellis > 0 && numTrellis <= 2 && numTrellis === dimCount &&
      totalLargeTrellis < trellisLimitLarge) {
      colInfo.trellisCols = trellisLargeCols;
      vizOptions.requireMediumCardinality.forEach(function (vizOption) {
        addVisualOption(vizOption, numTrellis, colInfo);
      });
    }

    trellisedVisualTypes.forEach(function (visualData) {
      if (checkConstraints(visualData, visualData.colInfo, dimCount, aggCount)) {
        visualData = assignColumns(visualData, visualData.colInfo);
        const viz = handleIfSupported(visualData);
        if (viz) {
          viz.visualData = visualData;
          vizData.push(viz);
        }
      }
    });

    drawVisuals($container, vizData);
  }

  function drawPage(viz_div, pageId) {
    let startIdx = (pageId - 1) * maxVisualsAtATime,
        endIdx = startIdx + maxVisualsAtATime,
        $viz = $('#' + viz_div);

    helpers.removeVisuals();

    $viz.empty();
    $viz.append(`
      <div class="showme-wrapper"></div>
        <div class="showme-container">
        </div>
      </div>
    `);

    let $container = $viz.find('.showme-container');
    helpers.setContainer($container);

    let vizData = possibleVisuals.slice(startIdx, endIdx);
    return drawVisuals($container, vizData);
  }

  showme.computeSettings = function(visualInfo) {
    const settings = visualInfo.viz.viz_getSettings();
    settings.report_title = vizDriver.title();
    settings.report_subtitle = vizDriver.subtitle();

    // filters_shelf is going to always be preserved.
    const activeShelves = { filters_shelf: true };
    const dataRequest = helpers.getDataRequest();
    const columns = dataRequest.columns();
    const savedShelves = [];
    const shelves = vizDriver.shelf_data();
    const prevVizType = vizDriver.viz_type();
    const currVizType = visualInfo.visualData.vizType;

    let info = {};
    vizDriver.report_save_shelves(shelves, info);
    for (let colIdx = 0; colIdx < columns.length; colIdx++) {
      let sourceShelf = columns[colIdx].colsource();

      if (info[sourceShelf][0]) {
        savedShelves.push(info[sourceShelf][0]);
      }

      info[sourceShelf] = info[sourceShelf].slice(1);
    }

    const filterShelves = info.filters_shelf.slice();

    info = {};
    info.filters_shelf = filterShelves;

    function shelfIterator(shelf, colIdx) {
      const sourceShelf = columns[colIdx].colsource();
      const targetShelf = shelf.shelfId;

      activeShelves[sourceShelf] = true;
      activeShelves[targetShelf] = true;

      if (info[targetShelf] === undefined) {
        info[targetShelf] = [];
      }

      const shelfToMove = savedShelves[colIdx];
      if (!shelfToMove) {
        return;
      }

      if (shelf.order) {
        shelfToMove.order = {
          ascending: shelf.order.ascending,
          priority: shelf.order.priority,
        };
      }

      if (shelf.coldisplayformat) {
        shelfToMove.display_format = shelf.coldisplayformat;
      }

      if (shelf.secondaryaxis) {
        shelfToMove.secondaryaxis = shelf.secondaryaxis;
      }

      info[targetShelf].push(shelfToMove);
    }

    shelvesIterate(visualInfo.visualData, shelfIterator);

    $.each(activeShelves, function(k) {
      settings[k] = info[k] || [];
    });

    visualInfo.visualData.settings = settings;
  };

  showme.drawSelectedVisual = function(visualInfo) {
    showme.computeSettings(visualInfo);
    const currVizType = visualInfo.visualData.vizType;

    const prevVizType = vizDriver.viz_type();

    let report = vizDriver.report();

    report.switchViz(visualInfo.visualData.vizType, visualInfo.visualData.settings);

    report.viz().needs_save(true);
    vizDriver.page_needs_save();

    if (isSearchShowme) {
      vizDriver = report.viz();
      helpers.vizDriver(vizDriver);
      const searchData = displayedQuery.search_data();
      searchData.showmeVisualConfig.vizId = visualInfo.visualData.id;
      displayedQuery.search_data(searchData);
    }

    arc.misc.displayVisualType(currVizType);

    if (prevVizType === visualInfo.visualData.vizType) {
      report.refresh();
    }

    if (arc.globals.is_dashboard && !arc.globals.isInModal() && !isSearchShowme) {
      vizDriver.report().saved_data(JSON.stringify({
        report_type: visualInfo.visualData.vizType,
        report_data: visualInfo.visualData.settings,
      }));
    }
  };

  function drawVisuals($container, vizData, prepend) {
    vizData.forEach(function(v) {
      let {
        viz: viz,
        vizDriver: vizDriver,
      } = v;
      let dataRequest = helpers.getDataRequest();
      if (v.afterDataRequest) {
        dataRequest = v.afterDataRequest(dataRequest);
      }

      v.dataRequest = dataRequest;
      v.columns = dataRequest.columns();

      v.uniqueVisualId = vizDriver.unique_report_id();
      v.name = v.visualData.name;
      helpers.addVisualToContainer($container, v, prepend, initUserSettings);
      drawnVisuals.push(v);
    });

    let $visuals = $container.find('.showme-visual-container');
    $visuals.off('click.picked')
            .on('click.picked', function() {
              let vizId = $(this).find('.showme-visual').attr('id');
              drawnVisuals.some(function(v) {
                if (v.chartId !== vizId) {
                  return false;
                }

                showme.drawSelectedVisual(v);
                let $m = $('#showme-modal');
                $m.modal('hide');
                helpers.removeVisuals();

                if (pickedCb) {
                  pickedCb(v);
                }
              });
            });

    /*
     * Find unique queries and only send those down to the execution engine.
     *
     * In order to find unique queries, get the dsreq objects we are sending down
     * and collect up the visuals that are sending down identical dsreqs.
     */
    let uniqueDsReqs = {};
    vizData.forEach(function(v) {
      let {
        dataRequest: dataRequest,
      } = v;

      let rb = dataRequest.rb();
      let dsreq = JSON.stringify(rb.dsreq());

      if (uniqueDsReqs[dsreq] === undefined) {
        uniqueDsReqs[dsreq] = [];
      }

      uniqueDsReqs[dsreq].push(v);
    });

    /*
     * For each unique dsreq, send down the query, and then use the results to
     * plot all the related visuals
     */
    $.each(uniqueDsReqs, function(key, value) {
      // Pick the first visual
      let v = value[0];

      let {
        vizDriver: vizDriver,
        dataRequest: dataRequest,
      } = v;

      let rb = dataRequest.rb();

      // Use the first visual to run the query
      vizDriver.runQuery(
        rb,
        (data) => {
          // Use the results for all the visuals that are using identical queries
          let idx = 0;

          const dateCols = getDateVisuals($container, dataRequest, data);
          getTrellisedVisuals($container, dataRequest, data, dateCols);

          function drawOne() {
            if (idx >= value.length) {
              return;
            }

            let vWithData = value[idx];
            idx += 1;
            vWithData.rows = data.rows.slice();
            const lengthError = 'No data';
            try {
              if (!data.rows.length) {
                throw lengthError;
              }

              drawVisual(vWithData);
            } catch (e) {
              hideSpinner(vWithData.chartId);
              const errorMsg = e === lengthError ? lengthError : 'No Preview Available';
              const $noPreviewDiv = $(`<div>${errorMsg}</div>`);
              $noPreviewDiv.css({ textAlign: 'center', marginTop: '150px' });
              $(`#${vWithData.chartId}`).append($noPreviewDiv);
              console.log(e);
            }

            setTimeout(drawOne, 0);
          }

          setTimeout(drawOne, 0);
        },
        () => {
          /* Nothing to do on fail - error reporting happens via another method see below! */
        },
        (_qstate, msg, error) => arc.report.alertMessage('Failed to draw explore visuals', `${msg}: ${error}`, { alertContainerSelector: '#showme-modal' })
      );
    });

    return showme;
  }

  function hideSpinner(chartId) {
    const chartIdSplit = chartId.split('-');
    const spinnerId = chartIdSplit[chartIdSplit.length - 1];
    $(`#showme-spinner-${spinnerId}`).hide();
  }

  function drawVisual(vizData) {
    let {
      vizDriver: vizDriver,
      viz: viz,
      chartId: chartId,
      rows: rows,
      columns: columns,
    } = vizData;

    let result = arc.data.result();
    result.rows(rows);
    result.columns(columns);
    result.finalize();

    arc.counters.increment('dataResultTotalRows', rows.length);

    // create the visual by setting the data and then drawing it
    viz.viz_setDataResult(result);

    vizDriver.sb_colnames(columns.map(function(c) {
      return c.colname();
    }));

    vizDriver.sb_rows(rows);
    hideSpinner(chartId);
    viz.viz_fetchCollateral(chartId, false, function() {
      if ($('#' + chartId).length > 0) {
        vizDriver.markActivationEnabled(false);
        vizDriver.localEnableAnimations(false);
        viz.viz_drawVisual(chartId);
      }
    });
  }

  function setOrder(dataRequest, idx, ascending, priority) {
    ascending = (ascending === undefined) ? false : ascending;
    priority = (priority === undefined) ? 1 : priority;

    let rb = dataRequest.rb();
    let dims = rb.dimensions();
    let aggs = rb.aggregates();

    if (idx < dims.length) {
      dims[idx].order = {
        asc: ascending,
        pri: priority,
      };
    } else {
      aggs[idx - dims.length].order = {
        asc: ascending,
        pri: priority,
      };
    }
  }

  function setColumns(vizDriver, dataRequest, columns) {
    if (columns) {
      dataRequest.columns(columns);

      let validColIndices = {};
      columns.forEach(function(c) {
        validColIndices[c.colindex()] = true;
      });

      let rb = dataRequest.rb();
      let dims = rb.dimensions();
      let aggs = rb.aggregates();
      aggs = aggs.filter(function(agg, i) {
        return (validColIndices[i + dims.length]);
      });

      dims = dims.filter(function(agg, i) {
        return (validColIndices[i]);
      });

      rb.dimensions(dims);
      rb.aggregates(aggs);
    } else {
      columns = dataRequest.columns();
    }

    let dimStartIdx,
        dimEndIdx,
        aggStartIdx,
        aggEndIdx;

    columns.forEach(function(c, idx) {
      c.colindex(idx);

      if (c.colproperty() === arc.report.columnTypeEnum.DIMENSION) {
        dimEndIdx = idx;
        if (dimStartIdx === undefined) {
          dimStartIdx = idx;
        }
      }

      if (c.colproperty() === arc.report.columnTypeEnum.AGGREGATE) {
        aggEndIdx = idx;
        if (aggStartIdx === undefined) {
          aggStartIdx = idx;
        }
      }
    });

    vizDriver.dim_start_idx(dimStartIdx)
             .dim_end_idx(dimEndIdx)
             .agg_start_idx(aggStartIdx)
             .agg_end_idx(aggEndIdx);
  }

  function dim(idx) {
    let s = vizDriver.dim_start_idx(),
        e = vizDriver.dim_end_idx();

    let d = idx + s;
    return (d <= e) ? d : undefined;
  }

  function agg(idx) {
    let s = vizDriver.agg_start_idx(),
        e = vizDriver.agg_end_idx();

    let d = idx + s;
    return (d <= e) ? d : undefined;
  }

  function handleIfSupported(vizConfig) {

    let viz = vizConfig.vizCreate();
    let thisVizDriver = arc.report.driver(viz)
                           .dataset(vizDriver.dataset())
                           .params(arc.params.tracker());
    thisVizDriver.viz_type(vizConfig.vizType);

    if (vizConfig.settings) {
      let settings = thisVizDriver.user_settings();

      $.each(vizConfig.settings, function(k, v) {
        let s = settings.find_by_id(k);
        if (s) {
          s.value(v);
        }
      });
    }

    function handleShelfCb(shelf, colIdx, vizDriver, dataRequest) {
      let columns = dataRequest.columns();

      if (shelf.secondaryaxis) {
        columns[colIdx].secondaryaxis(true);
      }

      columns[colIdx].colsource(shelf.shelfId);
      if (shelf.coldisplayformat) {
        columns[colIdx].setDisplayFormat(vizDriver.translatedDisplayFormat(shelf.coldisplayformat));
      }

      setColumns(vizDriver, dataRequest, columns);
      if (shelf.order) {
        setOrder(dataRequest, colIdx, shelf.order.ascending);
      }

      return columns;
    }

    function afterDataRequest(dataRequest) {
      shelvesIterate(vizConfig, function(shelf, colIdx) {
        handleShelfCb(shelf, colIdx, thisVizDriver, dataRequest);
      });

      const shelfIndexTracker = {};
      dataRequest.columns().forEach(function(col) {
        const colsource = col.colsource();
        shelfIndexTracker[colsource] = shelfIndexTracker[colsource] || 0;

        col.shelfIndex(shelfIndexTracker[colsource]);
        shelfIndexTracker[colsource] += 1;
      });

      return dataRequest;
    }

    return {
      viz: viz,
      vizDriver: thisVizDriver,
      afterDataRequest: afterDataRequest,
    };
  }

  function checkConstraints(visualData, colInfo, dimCount, aggCount) {
    if (visualData.isGeo) {
      if (colInfo.numGeo === 0) {
        return false;
      }
    }

    let dimMin = visualData.constraints &&
                  visualData.constraints.minimums &&
                  visualData.constraints.minimums.dimensions;
    if (dimMin !== undefined && dimMin > dimCount) {
      return false;
    }

    let dimMax = visualData.constraints &&
                  visualData.constraints.maximums &&
                  visualData.constraints.maximums.dimensions;
    if (dimMax !== undefined && dimMax < dimCount) {
      return false;
    }

    let aggMin = visualData.constraints &&
                  visualData.constraints.minimums &&
                  visualData.constraints.minimums.aggregates;
    if (aggMin !== undefined && aggMin > aggCount) {
      return false;
    }

    let aggMax = visualData.constraints &&
                  visualData.constraints.maximums &&
                  visualData.constraints.maximums.aggregates;
    if (aggMax !== undefined && aggMax < aggCount) {
      return false;
    }

    let requiredNumeric = 0;

    visualData.shelves.forEach(function(shelf) {
      if (shelf.required && shelf.mustBeNumeric) {
        requiredNumeric++;
      }
    });

    if (requiredNumeric > colInfo.numNumeric) {
      return false;
    }

    return true;
  }

  function getColumnInfo(cols) {
    const colData = {
      numNumeric: 0,
      numString: 0,
      numGeo: 0,
      numCols: cols.length,
      cols: cols,
      colObj: {},
    };

    cols.forEach(function(col) {
      if (col.colgeotype()) {
        const geotype = col.colgeotype();
        if (geotype !== 'county') {
          /*
           * We are checking to see if map visuals should be used. The presence
           * of county information should not lead to a map unless state is
           * also present. In this case, states will be tracked separately. So,
           * ignore the presence of counties here.
           */
          colData.numGeo++;
        }
      } else if (arc.report.numeric_types[col.coltype().toUpperCase()]) {
        colData.numNumeric++;
      } else if (col.coltype().toUpperCase() === 'STRING') {
        colData.numString++;
      }
    });

    return colData;
  }

  function assignColumns(visualData, colInfo, elementOrdering) {
    const colAssignments = {};
    let numAssigned = 0;
    const cols = colInfo.cols;
    let dateColIdx = 0;
    let trellisColIdx, trellisCols;
    if (colInfo.trellisCols) {
      trellisColIdx = 0;
      trellisCols = colInfo.trellisCols;
    }

    const orderedCols = [];

    if (elementOrdering) {
      const dims = [],
            aggs = [],
            others = [];
      cols.forEach(function(c) {
        if (c.colproperty() === arc.report.columnTypeEnum.DIMENSION) {
          dims.push(c);
          return;
        }

        if (c.colproperty() === arc.report.columnTypeEnum.AGGREGATE) {
          aggs.push(c);
          return;
        }

        others.push(c);
      });

      elementOrdering.forEach(function(type) {
        if (type === 'dimension') {
          orderedCols.push(dims.shift());
          return;
        }

        if (type === 'aggregate') {
          orderedCols.push(aggs.shift());
          return;
        }
      });

      others.forEach(function(c) {
        orderedCols.push(c);
      });
    } else {
      cols.forEach(function(c) {
        orderedCols.push(c);
      });
    }

    for (let shelfIdx = 0; shelfIdx < visualData.shelves.length; shelfIdx++) {
      let shelf = visualData.shelves[shelfIdx];
      shelf.colIdxes = [];
      if (shelf.isDate) {
        if (colInfo.dateCols) {
          shelf.colIdxes.push(colInfo.dateCols[dateColIdx]);
          colAssignments[colInfo.dateCols[dateColIdx]] = true;
          dateColIdx++;
          numAssigned++;
        }
      } else if (shelf.isTrellis && trellisCols && trellisCols.length > trellisColIdx) {
        shelf.colIdxes.push(colInfo.trellisCols[trellisColIdx]);
        colAssignments[colInfo.trellisCols[trellisColIdx]] = true;
        trellisColIdx++;
        numAssigned++;
      } else {
        for (var idx = 0; idx < orderedCols.length; idx++) {
          const col = orderedCols[idx];
          const colindex = col.colindex();
          if (colAssignments[colindex]) {
            continue;
          }

          let assigned = false;
          if (shelf.isGeo) {
            if (col.colgeotype()) {
              shelf.colIdxes.push(colindex);
              colAssignments[colindex] = true;
              assigned = true;
              numAssigned++;
            }
          } else if (col.colproperty() === shelf.colproperty ||
            shelf.colproperty === arc.report.columnTypeEnum.UNKNOWN) {
            const colIsNumeric = arc.report.coltypeIsNumber(col.coltype());
            if ((shelf.mustBeNumeric && colIsNumeric) || !shelf.mustBeNumeric) {
              shelf.colIdxes.push(colindex);
              colAssignments[colindex] = true;
              assigned = true;
              numAssigned++;
              if (trellisCols && trellisCols.indexOf(colindex) !== -1) {
                trellisCols.splice(trellisCols.indexOf(colindex), 1);
              }
            }
          }

          if (assigned && !shelf.catchall) {
            break;
          }
        }
      }

      if (numAssigned === orderedCols.length) {
        break;
      }
    }

    return visualData;
  }

  function createVizOption(visualData, colInfo, dimCount, aggCount, elementOrdering) {
    let viz;
    visualData.settings = visualData.settings || {};
    visualData.settings.nulls_order = visualData.settings.nulls_order || 'Last';
    if (checkConstraints(visualData, colInfo, dimCount, aggCount)) {
      visualData = assignColumns(visualData, colInfo, elementOrdering);
      viz = handleIfSupported(visualData);
      visualData.colInfo = colInfo;
      if (viz) {
        viz.visualData = visualData;
        return viz;
      }
    }
  }

  function findPossibleVisuals(config) {
    let {
      elementOrdering,
      columns,
      vizType,
    } = config;

    let possibleVisuals = [];
    let colInfo = getColumnInfo(columns);
    let possibleVisualTypes = [];
    if (vizType === 'calendar-heatmap') {
      visualOptions.requireDate.forEach((v) => {
        if (v.vizType === vizType) {
          possibleVisualTypes.push(v);
        }
      });
    }

    visualOptions.generalOptions.forEach((v) => {
      possibleVisualTypes.push(v);
    });

    const dimCount = vizDriver.dim_cnt(),
          aggCount = vizDriver.agg_cnt();

    if (colInfo.numGeo > 0 && colInfo.numGeo <= (dimCount + 1)) {
      const geoCols = columns.filter(col => col.colgeotype());
      const mapType = arc.geo.computeMapType(geoCols);

      if (['latlong', 'zipcode'].includes(mapType)) {
        visualOptions.requireGeoInteractive.forEach(function(m) {
          const geoViz = createVizOption(m, colInfo, dimCount, aggCount);

          if (geoViz) {
            possibleVisuals.push(geoViz);
          }
        });
      } else if (['country', 'county', 'dma', 'state'].includes(mapType)) {
        visualOptions.requireGeoBasic.forEach(function(m) {
          const geoViz = createVizOption(m, colInfo, dimCount, aggCount);

          if (geoViz) {
            possibleVisuals.push(geoViz);
          }
        });
      }
    }

    for (var i = 0; i < possibleVisualTypes.length; i++) {
      let visualData = possibleVisualTypes[i];
      let viz;

      viz = createVizOption(visualData, colInfo, dimCount, aggCount, elementOrdering);
      if (viz) {
        possibleVisuals.push(viz);
      }
    }

    return possibleVisuals;
  }

  function ShelfTracker(showmeInst, dataset, $possibleVisualList, initialVizType) {
    let shelfTracker = function() {return 'shelfTrackerInstance'; };

    const shelfElements = {
      dataset: dataset,
      aggregates: {},
      dates: {},
      geo: {},
      dimensions: {},
      lat: null,
      long: null,
    };

    const columnInfo = {
      numNumeric: 0,
      numGeo: 0,
      numDimensions: 0,
      numDates: 0,
    };

    let numSelected = 0;

    const $vizEmptyList = $('<div id="explore-empty">Select the fields you are' +
      ' interested in to view the available visuals appropriate to your selections</div>');

    function saveSingleShelf(type, shelfElementsObj, currShelves) {
      const shelfInfo = {};
      if (type === 'dimension') {
        shelfInfo.shelfProperty = arc.report.columnTypeEnum.DIMENSION;
        shelfInfo.shelfId = 'dimensions_shelf';
        shelfInfo.targetIsAgg = 'false';
      } else {
        shelfInfo.shelfProperty = arc.report.columnTypeEnum.AGGREGATE;
        shelfInfo.shelfId = 'aggregates_shelf';
        shelfInfo.targetIsAgg = 'true';
      }

      for (let colname in shelfElementsObj) {
        let shelfElem = shelfElementsObj[colname];
        let se = arc.report.shelfElementPostProcess({
          shelf_elem: shelfElem,
          shelf_type: arc.report.shelf_types.COLUMN,
          shelf_property: shelfInfo.shelfProperty,
          shelf_id: shelfInfo.shelfId,
          tgtshelf_is_aggregate: shelfInfo.targetIsAgg,
        });

        let shelfElementObj = {};
        se.save(shelfElementObj);
        currShelves.add(shelfInfo.shelfId, [shelfElementObj]);
      }
    }

    function getNumGeo() {
      let numGeo = 0;
      if (shelfElements.lat && shelfElements.long) numGeo++;
      numGeo += Object.keys(shelfElements.geo).length;

      return numGeo;
    }

    function saveShelfData() {
      const currShelves = vizDriver.shelves().emptyAll();

      saveSingleShelf('dimension', shelfElements.dimensions, currShelves);
      saveSingleShelf('aggregate', shelfElements.aggregates, currShelves);
    }

    function drawPreSelected(vizData) {
      saveShelfData();
      let dr;
      const dataRequest = helpers.getDataRequest();
      const columns = dataRequest.columns();
      const colInfo = getColumnInfo(columns);
      const visualData = assignColumns(vizData, colInfo);
      const viz = handleIfSupported(visualData);
      visualData.colInfo = colInfo;
      viz.visualData = visualData;
      if (viz.afterDataRequest) {
        dr = viz.afterDataRequest(dataRequest);
      }

      viz.dataRequest = dr;
      viz.columns = columns;
      viz.uniqueVisualId = vizDriver.unique_report_id();
      viz.name = viz.visualData.name;
      showme.drawSelectedVisual(viz);
      $possibleVisualList.empty();
      $('#showme-new-visual-modal').modal('hide');
    }

    shelfTracker.updateVisuals = function() {
      const numDims = columnInfo.numDimensions;
      const numAggs = numSelected - columnInfo.numDimensions;
      const numDates = columnInfo.numDates;
      columnInfo.numGeo = getNumGeo();

      let displayVisuals = visualOptions.generalOptions
        .filter(visual => checkConstraints(visual, columnInfo, numDims, numAggs));

      if (numDates === 1 && numDims === 1 && numAggs > 0) {
        visualOptions.requireDate.forEach(visual => displayVisuals.push(visual));
      }

      $possibleVisualList.empty();
      for (let i = 0; i < displayVisuals.length; i++) {
        let currViz = displayVisuals[i];
        let $vizOption = $(`<li class="explore-visual-option"></li>`);
        let $vizOptionTitle = $(`<div class="explore-option-title">${currViz.name}</div>`);
        let $vizImageContainer = $('<div class="explore-image-container"></div>');
        let $vizImage = $(`<img src="/arc/static/img/icon_${currViz.vizType}.png?v=6" />`);
        $vizImage.addClass('explore-visual-image');
        $vizImageContainer.append($vizImage);
        $vizOption.append($vizImageContainer);
        $vizOption.append($vizOptionTitle);

        $vizOption.on('click.selectVizFromList',
          drawPreSelected.bind(null, currViz));
        $possibleVisualList.append($vizOption);
      }

      if (displayVisuals.length === 0) {
        $('#new-visual-explore').attr('disabled', true);
        $possibleVisualList.append($vizEmptyList);
      }
    };

    shelfTracker.selectShelf = function selectShelf(event) {
      event.stopPropagation();
      const $this = $(this);
      const shelfData = arc.report.shelf.getDatum($this);
      const shelfElem = shelfData.clone();
      const colname = shelfElem.dataset_colname();
      const colnameLowerCase = colname.toLowerCase();
      const hasGeo = (shelfElem.dataset_geotype() && (shelfElem.dataset_geotype() !== 'none')) ||
          arc.geo.autocastGeoType(colname);

      const isLat = hasGeo === 'latitude';
      const isLong = hasGeo === 'longitude';
      if (hasGeo) shelfElem.dataset_isdim(true);

      if ($this.hasClass('selectedField')) {
        if (shelfElem.dataset_isdim()) {
          delete shelfElements.dimensions[colname];
          columnInfo.numDimensions--;
        } else {
          delete shelfElements.aggregates[colname];
        }

        if (arc.report.numeric_types[shelfElem.dataset_coltype().toUpperCase()]) {
          columnInfo.numNumeric--;
        }

        if (shelfElem.dataset_coltype().toUpperCase() === 'TIMESTAMP') {
          columnInfo.numDates--;
          delete shelfElements.dates[colname];
        } else if (isLat) {
          delete shelfElements.lat;
        } else if (isLong) {
          delete shelfElements.long;
        } else if (hasGeo) {
          delete shelfElements.geo[colname];
        }

        numSelected--;
        $this.removeClass('selectedField');
        $this.find('.field-wrapper').css('background-color', '#f0f0f0');
      } else {
        if (shelfElem.dataset_isdim()) {
          shelfElements.dimensions[colname] = shelfElem;
          columnInfo.numDimensions++;
        } else {
          shelfElements.aggregates[colname] = shelfElem;
        }

        if (arc.report.numeric_types[shelfElem.dataset_coltype().toUpperCase()]) {
          columnInfo.numNumeric++;
        }

        if (shelfElem.dataset_coltype().toUpperCase() === 'TIMESTAMP') {
          columnInfo.numDates++;
          shelfElements.dates[colname] = shelfElem;
        } else if (isLat) {
          shelfElements.lat = shelfElem;
        }else if (isLong) {
          shelfElements.long = shelfElem;
        } else if (hasGeo) {
          shelfElements.geo[colname] = shelfElem;
        }

        if (numSelected === 0) {
          $('#new-visual-explore').attr('disabled', false);
        }

        $this.addClass('selectedField');
        $this.find('.field-wrapper').css('background-color', '#E7F6FF');
        numSelected++;
      }

      shelfTracker.updateVisuals();
    };

    shelfTracker.cancel = function () {
      vizDriver.report().clearShelfHistory();
      vizDriver.shelves().emptyAll();

      arc.misc.displayVisualType(initialVizType);
      vizDriver.report().switchViz(initialVizType);
    };

    shelfTracker.exploreVisuals = function() {
      $possibleVisualList.empty();
      saveShelfData();
      $('#showme-new-visual-modal').modal('hide');
      showmeInst.setup(true);
      $('#showme-modal .showme-cancel').off('click.cancel')
        .on('click.cancel', shelfTracker.cancel);
    };

    return shelfTracker;
  }

  return showme;
};

arc.showme.noVisualExplore = function() {
  return arc.showme.visualOptions().noVisualExplore;
};

}());
