window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};

arc.report.bar = function () {
  //
  // Supported chart_type values are:
  //   bar
  //   line
  //   lines
  //   area
  //
  var dsreq,
    sb_sql,
    is_horizontal;

  var setting_scale = arc.report.user_settings.scale_log10();
  const horizontal_gridlines_enabled = arc.report.user_settings.horizontal_gridlines_enabled();
  const vertical_gridlines_enabled = arc.report.user_settings.vertical_gridlines_enabled();
  var legend_visibility = arc.report.user_settings.legend_visibility();
  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  setting_scale,
                                  horizontal_gridlines_enabled,
                                  vertical_gridlines_enabled,
                                  legend_visibility,
                                ]);

  var bar = arc.report.base()
    .chart_type("bar")
    .user_settings(user_settings);

  bar.is_horizontal = function (_) {
    if (!arguments.length) return is_horizontal;
    is_horizontal = _;
    return bar;
  };

  // Called once at startup time
  bar.setup = function () {
    bar.base_setup();
    return bar;
  };

  var saved_info = {};

  // Called to return an object that can later be passed into
  // bar.restore to restore state
  bar.save = function () {
    var bar_info = saved_info;
    bar_info = bar.report_save_shelves(bar.shelf_data(), bar_info);
    bar_info = bar.saveClickBehaviors(bar_info);
    return bar_info;
  };

  bar.restore = function (bar_info) {
    if (bar_info === undefined) {
      return bar;
    }
    saved_info = bar_info;
    bar.report_restore_shelves(bar.shelf_data(), bar_info);
    bar.restoreClickBehaviors(bar_info);
    return bar;
  };

  bar.generateSql = function () {
    var chart_type = bar.chart_type();

    var sb = bar.getSelectBuilder();
    var rb = bar.getRequestBuilder();
    var dimensions_count = bar.shelfElementCount("dimensions_shelf");
    if (dimensions_count === 0) {
      arc.report.alertMessage("Please specify at least one dimension and one aggregate");
      return bar;
    }

    var aggregate_count = bar.shelfElementCount("aggregates_shelf");
    if (aggregate_count === 0) {
      arc.report.alertMessage("Please specify at least one dimension and one aggregate");
      return bar;
    }
    bar.addDimensionsAndAggregates(sb, true,
      chart_type === "lines-agg" ? 1 : null, rb);
    bar.addFiltersToSb(sb);
    bar.addFiltersToRb(rb);
    sb = bar.finalizeSql(sb);

    sb_sql = sb.getSql();
    console.log(sb_sql);

    try {
      dsreq = JSON.stringify(rb.dsreq());
    }
    catch (err) {
      console.log(err);
    }
    return bar;
  };

  bar.refresh = function (force_redraw) {
    bar.generateSql();
    bar.loadData(sb_sql, force_redraw, dsreq);
    return bar;
  };

  bar.chart_is_lines = function () {
    var chart_type = bar.chart_type();
    return ((chart_type === "lines") || (chart_type === "lines-agg"));
  };

  bar.redraw = function () {
    bar.clear_errors();
    var chart_div = bar.chart_div();
    bar.undraw();

    if (sb_sql !== bar.sqlQuery()) {
      console.log(sb_sql);
      console.log(bar.sqlQuery());
      return bar.refresh();
    }

    var sb_colnames = bar.sb_colnames(),
      sb_rows = bar.sb_rows(),
      chart_type = bar.chart_type();

    if (!sb_rows) {
      return bar;
    }

    var date_formatter = d3.time.format("%Y-%m-%d");
    var dim_is_date = bar.dimension_axis_is_date();

    var dim_colindex = bar.dim_start_idx();

    if (!sb_rows) {
      bar.draw_error("No data available");
      return bar;
    }

    bar.unselectOnClick();

    var unique_dims = [],
      unique_dims_hash = {},
      unique_dims_total = {},
      dim_values = [],
      dim_values_hash = {},
      a_min = Infinity,// Initialize to Infinity, expecting real data to be < Infinity.
      a_max = 0,
      didx_to_d = {},
      dname_cnt = 0;

    // Apply local filters
    sb_rows = bar.filterLocally(sb_rows, dim_colindex);

    sb_rows.forEach(function (d) {
      if (dim_is_date) {
        if (!(d[dim_colindex] instanceof Date)) {
          d[dim_colindex] = date_formatter.parse(d[dim_colindex]);
        }
      }
      var agg_ci;
      if (chart_type === "lines-agg") {
        for (agg_ci = bar.agg_start_idx(); agg_ci <= bar.agg_end_idx();
             agg_ci++) {
          d[agg_ci] = (d[agg_ci] === 'NULL') ? 0 : Number(d[agg_ci]);
          a_min = Math.min(a_min, d[agg_ci]);
          a_max = Math.max(a_max, d[agg_ci]);
        }
      } else {
        agg_ci = bar.agg_start_idx();
        d[agg_ci] = (d[agg_ci] === 'NULL') ? 0 : Number(d[agg_ci]);
        a_min = Math.min(a_min, d[agg_ci]);
        a_max = Math.max(a_max, d[agg_ci]);
      }
      if (setting_scale.value() === 'LOG10' && a_min <= 0) {
        arc.report.alertMessage("Axis Scale is set to Log 10, but data contains " +
        "values <= 0 which is not valid for log scale. " +
        "Please change the Axis Scale in Settings for this data. ");
        return;
      }
      if (!(d[dim_colindex] in dim_values_hash)) {
        dim_values_hash[d[dim_colindex]] = 1;
        dim_values.push(d[dim_colindex]);
        didx_to_d[dname_cnt] = d;
        ++dname_cnt;
      }
      if (bar.dim_cnt() > 1) {
        agg_ci = bar.agg_start_idx();
        var dim = d.slice(bar.dim_start_idx() + 1, bar.dim_end_idx() + 1);
        if (dim in unique_dims_hash) {
          unique_dims_total[dim] += d[agg_ci];
        } else {
          unique_dims_hash[dim] = 1;
          unique_dims_total[dim] = d[agg_ci];
          unique_dims.push(dim);
        }
      }
    });
    if (bar.dim_cnt() === 1) {
      unique_dims = [[]];
    }
    if ((chart_type === "lines-agg") && (bar.agg_cnt() > 1)) {
      unique_dims = sb_colnames.slice(bar.agg_start_idx(), bar.agg_end_idx() + 1);
    }
    if (!bar.chart_is_lines()) {
      if (bar.shelves().getOrder().length === 0) {
        unique_dims.sort(function (a, b) {
          return unique_dims_total[b] - unique_dims_total[a];
        });
      }
    }

    var max_text_len = bar.dim_axis_max_textlen(dim_values);
    var xaxis_len = 0;
    var x_is_dim = !bar.idx_is_numeric([dim_values], 0);
    if (is_horizontal) {
      xaxis_len = bar.dim_axis_max_textlen(
        [sb_colnames[bar.agg_start_idx()].substr(
          0, bar.xaxis_label_maxlen())]);
    } else {
      xaxis_len = bar.dim_axis_max_textlen(
        [sb_colnames[bar.dim_start_idx()].substr(
          0, bar.xaxis_label_maxlen())]);
    }
    if (bar.chart_is_lines()) {
      xaxis_len = Math.max(xaxis_len,
          15 + bar.dim_axis_max_textlen(unique_dims,
          bar.xaxis_label_maxlen()));
    }

    var margin;
    if (is_horizontal) {
      margin = {
        top: 20, right: xaxis_len + 10,
        bottom: 40, left: max_text_len + 20
      };
    } else {
      margin = {
                top: 20, right: xaxis_len + 10,
                bottom: 20 + (dim_is_date || !x_is_dim ? 20 : max_text_len), left: 60};
               }
    if (bar.in_dashboard()) {
      margin.bottom -= 10;
      margin.bottom = Math.max(0, margin.bottom);
    }
    var width = $("#" + chart_div).width() - margin.left - margin.right - 30;
    var height = $("#" + chart_div).height() - margin.top - margin.bottom;
    var min_height = 250;
    if (!bar.chart_is_lines() && (unique_dims.length >= 2) &&
      (height > min_height * 2)) {
      height = height / 2;
      height -= 75;
    } else {
      height -= 75;
    }
    if (height < 150) {
      height = 150;
    }
    if (width < 150) {
      width = 150;
    }

    // SUSHIL TODO: When dim_is_date, we cannot use dim_values.length below,
    // we must instead calculate the difference between the first and last day
    var d_singlewidth_max = 30,
      d_minw = 12 * dim_values.length,
      d_maxw = d_singlewidth_max * dim_values.length;

    // Lines are areas will always fit in the provided area. Bars have a
    // minimum width
    if (chart_type === "bar") {
      if (dim_values.length > 800) {
        bar.draw_error("There are too many data points (> 800) to plot as a bar " +
        "chart. Try plotting this as a line chart instead, or use Top-K to " +
        "reduce the number of data points you are looking at");
        return;
      }
      if (is_horizontal) {
        height = Math.max(d_minw, height);
        height = Math.min(d_maxw, height);
      } else {
        width = Math.max(d_minw, width);
        width = Math.min(d_maxw, width);
      }
    }

    var a_maxrange, d_maxrange;
    if (is_horizontal) {
      a_maxrange = width;
      d_maxrange = height;
    } else {
      a_maxrange = height;
      d_maxrange = width;
    }

    var d_scale,
      dim_sz;
    if (dim_is_date) {
      var dl = bar.setup_date_scale(dim_values, d_maxrange, d_singlewidth_max);
      d_scale = dl[0];
      dim_sz = dl[1];
      dim_values = dl[2];
    }

    if (!d_scale) {
      d_scale =
        d3.scale.ordinal()
          .domain(dim_values)
          .rangeBands([0, d_maxrange]);
      dim_sz = d_scale.rangeBand();
    }

    var a_scale;
    var a_scalei;
    switch (setting_scale.value()) {
      case 'LOG10':
        a_scale = d3.scale.log()
          .domain([a_min, a_max])
          .nice()
          .clamp(true)
          .range([0, a_maxrange]);
        a_scalei = d3.scale.log()
          .domain([a_min, a_max])
          .nice()
          .clamp(true)
          .range([a_maxrange, 0]);
        break;
      default:  // Default to linear scale
        if (a_min > 0) {  // For linear scale charts we start the axis scale at 0.
          a_min = 0;
        }
        a_scale = d3.scale.linear()
          .domain([a_min, a_max])
          .range([0, a_maxrange]);
        a_scalei = d3.scale.linear()
          .domain([a_min, a_max])
          .range([a_maxrange, 0]);
    }

    var color = d3.scale.ordinal()
      .range(bar.colors())
      .domain(unique_dims);

    var unique_dim_elements = [];
    var agg_ci;
    unique_dims.forEach(function (unique_dim, idx) {
      if (chart_type === "lines-agg") {
        agg_ci = bar.agg_start_idx() + idx;
      } else {
        agg_ci = bar.agg_start_idx();
      }
      var sb_rows_dim = bar.sb_rows_for_unique_dim(sb_rows,
        unique_dim, dim_values, dim_colindex, agg_ci);
      unique_dim_elements.push(bar.unique_dim_element()
        .unique_dim(unique_dim)
        .sb_rows_dim(sb_rows_dim)
        .title(chart_type === "lines-agg" ? sb_colnames[bar.agg_start_idx() + idx] : null));
    });

    if (horizontal_gridlines_enabled.isChecked || vertical_gridlines_enabled.isChecked) {
      // If we are drawing gridlines, first delete all the current marks.
      // If we re-use marks, gridlines will be drawn on top of the marks.
      d3.select("#" + chart_div)
        .selectAll(".viz-marks")
        .remove();
      d3.select("#" + chart_div)
        .selectAll(".viz-marks2")
        .remove();
    }

    var dim_divs;

    if (bar.chart_is_lines()) {
      dim_divs = d3.select("#" + chart_div)
        .selectAll(".viz-bar-div")
        .data([unique_dim_elements]);
    } else {
      dim_divs = d3.select("#" + chart_div)
        .selectAll(".viz-bar-div")
        .data(unique_dim_elements);
    }

    // Delete extra divs
    dim_divs.exit()
      .remove();

    // Make new divs if necessary
    var new_divs = dim_divs.enter()
      .append("div")
      .classed("viz-bar-div", true);
    new_divs.append("div")
      .classed("viz-bar-title", true);
    new_divs.append("svg")
      .classed("viz-bar-svg", true)
      .append("g")
      .classed("viz-bar-g", true);

    // Rebind data to the 'g' elements to handle older g elements that
    // still have older data bound to them
    if (bar.chart_is_lines()) {
      d3.select("#" + chart_div)
        .selectAll(".viz-bar-g")
        .data([unique_dim_elements]);
    } else {
      d3.select("#" + chart_div)
        .selectAll(".viz-bar-g")
        .data(unique_dim_elements);
    }

    bar.render_divs(width, height, margin, dim_is_date);
    if (bar.chart_is_lines()) {
      d3.select("#" + chart_div)
        .selectAll(".viz-bar-g")
        .each(function (unique_dim_elements, idx) {
          if (chart_type === "lines-agg") {
            agg_ci = bar.agg_start_idx() + idx;
          } else {
            agg_ci = bar.agg_start_idx();
          }
          var svg = d3.select(this);

          // Lines only have one svg. All the individual lines
          // are drawn within that single svg
          bar.draw_svg(svg,
            unique_dim_elements, null, width, height,
            margin, d_scale, a_scale, a_scalei,
            dim_values,
            didx_to_d,
            dim_colindex, agg_ci,
            color, dim_sz, dim_is_date);
        });
    } else {
      bar.draw_titles();
      d3.select("#" + chart_div)
        .selectAll(".viz-bar-g")
        .each(function (unique_dim_element) {
          var svg = d3.select(this);
          bar.draw_svg(d3.select(this),
            unique_dim_elements, unique_dim_element.sb_rows_dim(), width, height,
            margin, d_scale, a_scale, a_scalei,
            dim_values,
            didx_to_d,
            dim_colindex, bar.agg_start_idx(),
            color, dim_sz, dim_is_date);
        });
    }

    if (bar.chart_is_lines()) {
      bar.lines_highlight();
    }
  };

  bar.draw_titles = function () {
    var sb_colnames = bar.sb_colnames(),
      chart_div = bar.chart_div();

    d3.select("#" + chart_div)
      .selectAll(".viz-bar-title")
      .html(function (unique_dim_element) {
        var ud = unique_dim_element.unique_dim();
        var hdr = [];
        ud.forEach(function (d, i) {
          hdr.push(sb_colnames[i + bar.dim_start_idx() + 1]);
          hdr.push(": ");
          hdr.push("<b style='margin: 0px 30px 0px 5px;'>" + d + "</b>");
        });
        return hdr.join("");
      });
  };

  bar.render_divs = function (width, height, margin, dim_is_date) {
    var chart_div = bar.chart_div();

    d3.select("#" + chart_div)
      .selectAll(".viz-bar-svg")
      .style("width", width + margin.left + margin.right +
      (dim_is_date ? 20 : 0) + "px")
      .style("height", height + margin.top + margin.bottom + "px");
    d3.select("#" + chart_div)
      .selectAll(".viz-bar-g")
      .attr("transform", "translate(" + margin.left + "," + margin.top + ")");
  };

  bar.unique_dim_element = function () {
    var sb_rows_dim,
      unique_dim,
      title,
      brush;

    var unique_dim_element = function () {
    };

    unique_dim_element.unique_dim = function (_) {
      if (!arguments.length) return unique_dim;
      unique_dim = _;
      return unique_dim_element;
    };

    unique_dim_element.sb_rows_dim = function (_) {
      if (!arguments.length) return sb_rows_dim;
      sb_rows_dim = _;
      return unique_dim_element;
    };

    unique_dim_element.title = function (_) {
      if (!arguments.length) return title;
      title = _;
      return unique_dim_element;
    };

    unique_dim_element.brush = function (_) {
      if (!arguments.length) return brush;
      brush = _;
      return unique_dim_element;
    };

    return unique_dim_element;
  };

  bar.sb_rows_for_unique_dim = function (sb_rows,
                                         unique_dim, dim_values, dim_colindex, agg_colindex) {

    var chart_type = bar.chart_type(),
      uniquedim_start = bar.dim_start_idx() + 1,
      uniquedim_end = bar.dim_end_idx() + 1;

    var sb_rows_dim = sb_rows.filter(function (d) {
      if (chart_type === "lines-agg") {
        return true;
      }
      return bar.arraysEqual(d.slice(uniquedim_start, uniquedim_end),
        unique_dim);
    });

    // We have filtered out rows for the combination of dimensions we
    // are going to plot. We may have a set of missing values here because
    // there was no data for particular dimensions - for these use-cases
    // we fill in an aggregate of zero so we can plot these correctly.

    // 1. Capture all the dim values that exist in the filtered dataset
    var sb_rows_by_dimvalue = {};
    sb_rows_dim.forEach(function (d) {
      sb_rows_by_dimvalue[d[dim_colindex]] = d;
    });

    // 2. Walk through all known dimensions, back-filling any that
    //    don't exist in the filtered dataset
    var sb_rows_all_dimvalues = [];
    dim_values.forEach(function (dim_value) {
      if (dim_value in sb_rows_by_dimvalue) {
        sb_rows_all_dimvalues.push(sb_rows_by_dimvalue[dim_value]);
      }
    });

    // Finally, use this new dataset to plot
    return sb_rows_all_dimvalues;
  };

  bar.lines_highlight = function () {
    var chart_div = bar.chart_div();

    // When multiple line charts are drawn, the code below highlights the
    // line chart that we mouseover by dimming all others
    d3.select("#" + chart_div)
      .selectAll(".dim_all")
      .on("mouseover.makeprimary", function (d) {
        d3.select("#" + chart_div)
          .selectAll(".dim_all")
          .transition()
          .duration(250)
          .style("opacity", "0.2");
        d3.select(this)
          .transition()
          .duration(250)
          .style("opacity", "1");
      })
      .on("mouseout.makeprimary", function (d) {
        d3.select("#" + chart_div)
          .selectAll(".dim_all")
          .transition()
          .duration(250)
          .style("opacity", "1");
      });
  };

  bar.draw_bar = function (svg, sb_rows_dim, is_horizontal,
                           d_scale, a_scale, dim_colindex, agg_colindex, dim_sz, height) {

    var chart_div = bar.chart_div();

    var marks =
      svg.selectAll(".viz-marks")
        .data(sb_rows_dim);

    // Update existing elements
    marks.style("fill", bar.colors()[0]);

    marks.exit()
      .remove();

    var marks_new =
      marks.enter()
        .append("rect")
        .style("fill", bar.colors()[0])
        .classed("viz-marks", true)
        .classed("dim_all", true);
    marks = svg.selectAll(".viz-marks");

    if (is_horizontal) {
      marks_new.attr("x", function (d) {
        if (d[agg_colindex] < 0) {
          return a_scale(d[agg_colindex]);
        }
        return a_scale(a_scale.range()[0]);
      })
        .attr("y", function (d) {
          return d_scale(d[dim_colindex]);
        })
        .attr("height", function (d) {
          return dim_sz;
        })
        .attr("width", 0);
      marks.transition()
        .ease("eastOutCirc")
        .delay(function (d, i) {
          return 100 + 10 * Math.min(i, 20);
        })
        .attr("x", function (d) {
          if (d[agg_colindex] < 0) {
            return a_scale(d[agg_colindex]);
          }
          return a_scale(a_scale.range()[0]);
        })
        .attr("y", function (d) {
          return d_scale(d[dim_colindex]);
        })
        .attr("height", function (d) {
          return dim_sz;
        })
        .attr("width", function (d) {
          if (d[agg_colindex] < 0) {
            return a_scale(a_scale.range()[0]) - a_scale(d[agg_colindex]);
          }
          return a_scale(d[agg_colindex]) - a_scale(a_scale.range()[0]);
        });
    } else {
      marks_new.attr("x", function (d, i) {
        return d_scale(d[dim_colindex]);
      })
        .attr("y", function (d) {
          return height;
        })
        .attr("width", dim_sz)
        .attr("height", 0);
      marks.transition()
        .ease("eastOutCirc")
        .delay(function (d, i) {
          return 100 + 10 * Math.min(i, 20);
        })
        .attr("x", function (d, i) {
          return d_scale(d[dim_colindex]);
        })
        .attr("y", function (d) {
          if (d[agg_colindex] < 0) {
            return height - a_scale(a_scale.range()[0]);
          }
          return height - a_scale(d[agg_colindex]);
        })
        .attr("width", dim_sz)
        .attr("height", function (d, i) {
          if (d[agg_colindex] < 0) {
            return a_scale(a_scale.range()[0]) - a_scale(d[agg_colindex]);
          }
          return a_scale(d[agg_colindex]) - a_scale(a_scale.range()[0]);
        });

      if (bar.editable()) {
        // Add in a brush to editable bars
        var brush =
          arc.report.brush()
            .svg(svg)
            .x_scale(d_scale)
            .y_scale(a_scale)
            .classname("viz-marks")
            .marks(marks)
            .y_scale_accessor(function (d) {
              return d[agg_colindex];
            })
            .activate();

        svg.datum()
          .brush(brush);
      }
    }

    return marks;
  };

  bar.draw_svg = function (svg,
                           unique_dim_elements,
                           sb_rows_dim, width, height,
                           margin, d_scale, a_scale, a_scalei,
                           dim_values,
                           didx_to_d,
                           dim_colindex, agg_colindex,
                           color, dim_sz, dim_is_date) {

    var chart_div = bar.chart_div(),
      chart_type = bar.chart_type(),
      marks,
      marks2;

    bar.bar_draw_axes(svg, dim_is_date, dim_values,
        is_horizontal, d_scale, a_scale, a_scalei, width, height,
        color, sb_rows_dim, margin);

    if (chart_type === "bar") {
      marks = bar.draw_bar(svg, sb_rows_dim, is_horizontal,
        d_scale, a_scale, dim_colindex, agg_colindex, dim_sz, height);
    } else {
      var m = bar.draw_linearea(svg,
        unique_dim_elements,
        sb_rows_dim, width, height,
        margin, d_scale, a_scale, a_scalei,
        dim_values,
        didx_to_d,
        dim_colindex, agg_colindex,
        color, dim_sz, dim_is_date);
      marks = m[0];
      marks2 = m[1];
    }

    bar.tooltips_and_contextmenu(svg, chart_type,
      dim_colindex, agg_colindex,
      is_horizontal, marks, marks2, dim_sz,
      didx_to_d, d_scale, dim_is_date);
    return svg;
  };

  bar.draw_linearea = function (svg,
                                unique_dim_elements,
                                sb_rows_dim, width, height,
                                margin, d_scale, a_scale, a_scalei,
                                dim_values,
                                didx_to_d,
                                dim_colindex, agg_colindex,
                                color, dim_sz, dim_is_date) {

    var chart_div = bar.chart_div(),
      chart_type = bar.chart_type(),
      sb_colnames = bar.sb_colnames(),
      marks,
      marks2;

    var line_color = function (d, i) {
      if (bar.chart_is_lines()) {
        return color(d.unique_dim());
      } else {
        return bar.colors()[0];
      }
    };

    var fill_color = function (d, i) {
      if (chart_type === "line") {
        return "none";
      }
      return line_color(d, i);
    };

    var opacity = function (d, i) {
      if (chart_type === "line") {
        return 1;
      }
      return 0.5;
    };

    var svg_a,
      svg_l,
      tr_svg_a,
      tr_svg_l,
      agg_start_idx = bar.agg_start_idx();

    var a_fn = function (d) {
      return a_scale(d[agg_start_idx]);
    };

    var d_fn = function (d) {
      if (chart_type === "line" || chart_type === "area" ||
        bar.chart_is_lines()) {
        return d_scale(d[dim_colindex]) + Math.floor(dim_sz / 2);
      } else {
        return d_scale(d[dim_colindex]);
      }
    };

    if (is_horizontal) {
      if (chart_type === "line" || chart_type === "area") {
        tr_svg_a = d3.svg.area()
          .x0(0)
          .x1(0)
          .y(d_fn);
        svg_a = d3.svg.area()
          .x0(0)
          .x1(a_fn)
          .y(d_fn);
        marks2 = svg.selectAll(".viz-marks2")
          .data([sb_rows_dim]);
        // Update existing elements
        marks2.style("fill", fill_color);

        marks2.exit()
          .remove();
        marks2.enter()
          .append("path")
          .classed("viz-marks2", true)
          .classed("dim_all", true)
          .style("opacity", opacity)
          .style("fill", fill_color)
          .attr("d", function (d) {
            return tr_svg_a(d);
          });
        marks2 = svg.selectAll(".viz-marks2");
        marks2.transition()
          .ease("easeOutCirc")
          .duration(300)
          .attr("d", function (d) {
            return svg_a(d);
          });
      }
      if (chart_type === "area" || bar.chart_is_lines()) {
        tr_svg_l = d3.svg.line()
          .x(0)
          .y(d_fn);
        svg_l = d3.svg.line()
          .x(a_fn)
          .y(d_fn);
        if (bar.chart_is_lines()) {
          marks = svg.selectAll(".viz-marks")
            .data(unique_dim_elements);
        } else {
          marks = svg.selectAll(".viz-marks")
            .data([sb_rows_dim]);
        }

        // Update existing elements
        marks.style("stroke", line_color);

        marks.exit()
          .remove();
        marks.enter()
          .append("path")
          .classed("viz-marks", true)
          .classed("dim_all", true)
          .style("fill", "none")
          .style("stroke", line_color)
          .style("stroke-width", "3px")
          .attr("d", function (d) {
            if (bar.chart_is_lines()) {
              return tr_svg_l(d.sb_rows_dim());
            }
            return tr_svg_l(d);
          });
        marks = svg.selectAll(".viz-marks");
        marks.transition()
          .ease("easeOutCirc")
          .duration(300)
          .attr("d", function (d, i) {
            if (bar.chart_is_lines()) {
              var svg_li = svg_l;
              if (chart_type === "lines-agg") {
                svg_li.x(function (d) {
                  return a_scale(d[agg_start_idx + i]);
                });
              }
              return svg_li(d.sb_rows_dim());
            }
            return svg_l(d);
          });
      }
    } else {
      var y_fn = function (d) {
        return height - a_fn(d);
      };
      if (chart_type === "line" || chart_type === "area") {
        tr_svg_a = d3.svg.area()
          .x(d_fn)
          .y0(height)
          .y1(height);
        svg_a = d3.svg.area()
          .x(d_fn)
          .y0(height)
          .y1(y_fn);
        marks2 = svg.selectAll(".viz-marks2")
          .data([sb_rows_dim]);

        // Update existing elements
        marks2.style("fill", fill_color);

        marks2.exit()
          .remove();
        marks2.enter()
          .append("path")
          .classed("dim_all", true)
          .classed("viz-marks2", true)
          .style("opacity", opacity)
          .style("fill", fill_color)
          .attr("d", function (d) {
            return tr_svg_a(d);
          });
        marks2 = svg.selectAll(".viz-marks2");
        marks2.transition()
          .ease("easeInCirc")
          .duration(300)
          .attr("d", function (d) {
            return svg_a(d);
          });
      }
      if (chart_type === "line" || chart_type === "area" || bar.chart_is_lines()) {
        tr_svg_l = d3.svg.line()
          .x(d_fn)
          .y(height);
        svg_l = d3.svg.line()
          .x(d_fn)
          .y(y_fn);
        if (bar.chart_is_lines()) {
          marks = svg.selectAll(".viz-marks")
            .data(unique_dim_elements);
        } else {
          marks = svg.selectAll(".viz-marks")
            .data([sb_rows_dim]);
        }

        // Update existing marks
        marks.style("stroke", line_color);
        marks.exit()
          .remove();
        marks.enter()
          .append("path")
          .classed("dim_all", true)
          .classed("viz-marks", true)
          .style("fill", "none")
          .style("stroke", line_color)
          .style("stroke-width", "3px")
          .attr("d", function (d) {
            if (bar.chart_is_lines()) {
              return tr_svg_l(d.sb_rows_dim());
            } else {
              return tr_svg_l(d);
            }
          });
        marks = svg.selectAll(".viz-marks");
        marks.transition()
          .ease("easeInCirc")
          .duration(300)
          .attr("d", function (d, i) {
            if (bar.chart_is_lines()) {
              var svg_li = svg_l;
              if (chart_type === "lines-agg") {
                svg_li.y(function (d) {
                  return height - a_scale(d[agg_start_idx + i]);
                });
              }
              return svg_li(d.sb_rows_dim());
            } else {
              return svg_l(d);
            }
          });
      }
    }
    return [marks, marks2];
  };

  bar.bar_draw_axes = function(svg, dim_is_date, dim_values, is_horizontal,
      d_scale, a_scale, a_scalei, width, height, color, sb_rows_dim, margin) {

    var tick_filter,
      sb_colnames = bar.sb_colnames();

    var x_axis_data = arc.visual.axis(),
      y_axis_data = arc.visual.axis();
    if (is_horizontal) {
      x_axis_data.scale(a_scale)
        .is_dim(false)
        .is_date(false)
        .colname(sb_colnames[bar.agg_start_idx()])
        .scale_type(setting_scale.value());
      y_axis_data.scale(d_scale)
        .is_dim(true)
        .is_date(dim_is_date)
        .colname(sb_colnames[bar.dim_start_idx()]);
    } else {
      x_axis_data.scale(d_scale)
        .is_dim(true)
        .is_date(dim_is_date)
        .colname(sb_colnames[bar.dim_start_idx()]);
      y_axis_data.scale(a_scalei)
        .is_dim(false)
        .is_date(false)
        .colname(sb_colnames[bar.agg_start_idx()])
        .scale_type(setting_scale.value());
    }

    if (!dim_is_date) {
      if (is_horizontal) {
        if (dim_values.length > height / 12) {
          tick_filter = Math.ceil(dim_values.length / (height / 12));
        }
      } else {
        if (dim_values.length > width / 12) {
          tick_filter = Math.ceil(dim_values.length / (width / 12));
        }
      }
    }

    // Remove any older axes
    svg.selectAll(".arc-viz-axis, .viz-bar-axis-grid")
      .remove();

    bar.draw_axes(svg, width, height, margin,
      {
        x_axis_data: x_axis_data,
        y_axis_data: y_axis_data,
        x_is_dim: x_axis_data.is_dim() && !x_axis_data.is_date(),
        y_is_dim: y_axis_data.is_dim() && !y_axis_data.is_date(),
        include_gridlines: (horizontal_gridlines_enabled.isChecked)
      });

    if (!dim_is_date) {
      svg.selectAll(".x .tick text")
        .style("text-anchor", "end")
        .attr("transform", function (d, i) {
          return "translate(-10, 0) rotate(" + bar.x_slant() + ",0,0)";
        });

      if (tick_filter) {
        svg.selectAll(".x .tick")
          .filter(function (d, i) {
            return ((i % tick_filter) !== 0);
          })
          .style("display", "none");
      }
    }

    var x_selector = ".x .tick text";
    var y_selector = ".y .tick text";

    if (is_horizontal) {
      x_selector = ".y .tick text";
      y_selector = ".x .tick text";
    }

    var x_tt =
      arc.tooltip()
        .dom_id("#tooltip")
        .text_fn(function (d, i) {
          return bar.base_text_fn(sb_rows_dim[i]);
        })
        .addSelection(svg.selectAll(x_selector))
        .activate();

    var y_tt =
      arc.tooltip()
        .dom_id("#tooltip")
        .text_fn(function (d) {
          return d;
        })
        .addSelection(d3.selectAll(y_selector))
        .activate();

    bar.contextmenu('legend')
      .addTooltip(is_horizontal ? y_tt : x_tt)
      .text_fn(function (d) {
        return d;
      })
      .params_fn(function (d, i) {
        return (bar.base_params_fn(sb_rows_dim[i]));
      })
      .activate();

    bar.draw_labels(svg, is_horizontal, width, height + margin.bottom - 10);
    if (bar.chart_is_lines() && legend_visibility.isChecked) {
      bar.draw_legend({
        svg: svg,
        width: width,
        height: height,
        color: color,
        sort_domain: true
      });
    }
    return bar;
  };

  bar.tooltips_and_contextmenu = function (svg, chart_type,
                                           dim_colindex, agg_colindex,
                                           is_horizontal, marks, marks2, dim_sz,
                                           didx_to_d, d_scale, dim_is_date) {

    var text_fn,
      params_fn,
      sb_colnames = bar.sb_colnames(),
      sb_coltypes = bar.sb_coltypes(),
      text_changes_on_move = false;

    if (chart_type === "bar") {
      text_fn = bar.base_text_fn;
      params_fn = bar.base_params_fn;
    } else {
      // For lines and area, we are drawing a single mark with the svg path
      // element that covers all values. So the tooltip/params have to change
      // as the mouse moves within that one element.
      text_changes_on_move = true;

      var get_didx = function () {
        if (is_horizontal) {
          var y = d3.mouse(svg.node())[1];
          return Math.floor(y / dim_sz);
        }
        var x = d3.mouse(svg.node())[0];
        return Math.floor(x / dim_sz);
      };

      text_fn = function (in_d) {
        var didx = get_didx(),
          d;

        if (bar.chart_is_lines()) {
          d = in_d.sb_rows_dim()[didx];
        } else {
          d = in_d[didx];
        }
        if (d.title) {
          return "<b>" + d.title() + "</b><br>" + bar.base_text_fn(d);
        }
        return bar.base_text_fn(d);
      };

      params_fn = function (in_d) {
        var didx = get_didx(),
          d;

        if (bar.chart_is_lines()) {
          d = in_d.sb_rows_dim()[didx];
        } else {
          d = in_d[didx];
        }
        return bar.base_params_fn(d);
      };
    }

    var tooltip =
      arc.tooltip()
        .dom_id("#tooltip")
        .text_changes_on_move(text_changes_on_move)
        .text_fn(text_fn)
        .addSelection(marks);

    if (marks2) {
      tooltip.addSelection(marks2);
    }
    tooltip.activate();

    var brush;
    if (chart_type === "bar") {
      brush = svg.datum().brush();
    }
    var cm =
      bar.contextmenu('primary')
        .addTooltip(tooltip)
        .text_fn(text_fn)
        .params_fn(params_fn)
        .brush(brush);
    if (brush) {
      brush.contextmenu(cm);
    }
    cm.activate();
  };

  return bar;
};

}());
