
window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};

/*
 * Helper class that encapsulates d3's brush element, and
 * co-ordinates work with an associated contextmenu object.
 */
arc.report.brush = function() {
  var svg,
      classname,
      d3_brush,
      x_scale,
      y_scale,
      y_scale_accessor,
      selections,
      contextmenu;

  var brush = function() { };

  brush.selections = function() {
    return selections;
  };

  brush.contextmenu = function(_) {
    if (!arguments.length) return contextmenu;
    contextmenu = _;
    return brush;
  };

  brush.y_scale_accessor = function(_) {
    if (!arguments.length) return y_scale_accessor;
    y_scale_accessor = _;
    return brush;
  };

  brush.svg = function(_) {
    if (!arguments.length) return svg;
    svg = _;
    return brush;
  };

  brush.marks = function(_) {
    if (!arguments.length) return marks;
    marks = _;
    return brush;
  };

  brush.classname = function(_) {
    if (!arguments.length) return classname;
    classname = _;
    return brush;
  };

  brush.x_scale = function(_) {
    if (!arguments.length) return x_scale;
    x_scale = _;
    return brush;
  };

  brush.y_scale = function(_) {
    if (!arguments.length) return y_scale;
    y_scale = _;
    return brush;
  };

  brush.svg = function(_) {
    if (!arguments.length) return svg;
    svg = _;
    return brush;
  };

  brush.dispatch_click = function(element) {
    if (element) {
      d3.select(element).each(function(d, i) {
        var f = d3.select(this)
                  .on("click.contextmenu");
        if (f) {
          f(d, i);
        }
      });
    } else {
      var f = svg.select(".brush .extent")
                 .on("click.contextmenu");
      if (f) {
        f();
      }
    }
  };

  var mouse_curr_elem;
  brush.dispatch_mouseout = function() {
    if (!mouse_curr_elem) {
      return;
    }
    d3.select(mouse_curr_elem).each(function(d, i) {
      d3.select(this).on("mouseout.tooltip")(d, i);
    });
    mouse_curr_elem = null;
    return brush;
  };

  brush.dispatch_mouseover = function(element) {
    if (mouse_curr_elem) {
      if (mouse_curr_elem !== element) {
        brush.dispatch_mouseout(mouse_curr_elem);
      }
    }
    if (element) {
      mouse_curr_elem = element;
      d3.select(mouse_curr_elem).each(function(d, i) {
        d3.select(this).on("mouseover.tooltip")(d, i);
      });
    }
    return brush;
  };

  brush.dispatch_mousemove = function(element) {
    if (mouse_curr_elem) {
      if (mouse_curr_elem !== element) {
        brush.dispatch_mouseover(element);
      }
    } else {
      if (element) {
        brush.dispatch_mouseover(element);
      }
    }
    if (element) {
      d3.select(mouse_curr_elem).each(function(d, i) {
        d3.select(this).on("mousemove.tooltip")(d, i);
      });
    }
    return brush;
  };

  brush.activate = function() {
    marks.classed("mark-selected", true)
         .classed("mark-unselected", false);

    d3_brush =
      d3.svg.brush()
        .x(x_scale)
        .y(y_scale);
    svg.selectAll(".brush")
       .remove();
    svg.append("g")
       .attr("class", "brush")
       .call(d3_brush);
    svg.on("mouseover.brush", function() {
         if (!contextmenu.tooltips_enabled()) {
           return;
         }
         brush.findEventSelection();
         brush.dispatch_mouseover(selections.size() > 0 ? selections.node(): null);
       })
       .on("mousemove.brush", function() {
         if (!contextmenu.tooltips_enabled()) {
           return;
         }
         brush.findEventSelection();
         brush.dispatch_mousemove(selections.size() > 0 ? selections.node() : null);
       })
       .on("mouseout.brush", function() {
         if (!contextmenu.tooltips_enabled()) {
           return;
         }
         brush.dispatch_mouseout();
       })
       .on("click.brush", function() {
         if (!contextmenu.tooltips_enabled()) {
           return;
         }
         brush.findEventSelection();
         brush.dispatch_click(selections.size() > 0 ? selections.node() : null);
       });
    d3_brush.on("brushstart", function() {
      brush.dispatch_mouseout();
      if (contextmenu) {
        contextmenu.hide();
      }
      contextmenu.tooltips_enabled(false);
    });
    d3_brush.on("brushend", function() {
      contextmenu.tooltips_enabled(true);
      brush.react();
      if (contextmenu) {
        contextmenu.show();
      }
    });
    d3_brush.on("brush", brush.react);
    return brush;
  };

  brush.findEventSelection = function() {
    selections = marks.filter(function(d, i) {
      var e = $(this);
      if (e.offset().left > d3.event.pageX) {
        return false;
      }
      if ((e.offset().left + parseInt(e.attr("width"))) < d3.event.pageX) {
        return false;
      }
      if (e.offset().top > d3.event.pageY) {
        return false;
      }
      if ((e.offset().top + parseInt(e.attr("height"))) < d3.event.pageY) {
        return false;
      }
      return true;
    });
    return selections;
  };

  brush.findSelection = function(extent) {
    var x1 = extent[0][0],
        x2 = extent[1][0],
        y1 = extent[0][1],
        y2 = extent[1][1];

    selections = marks.filter(function(d, i) {
      var mx1 = parseInt($(this).attr("x"));
      var mx2 = mx1 + parseInt($(this).attr("width"));
      if ((mx1 > x2) || (mx2 < x1)) {
        return false;
      }
      var my = y_scale.domain()[1] - y_scale_accessor(d, i);
      if (my > y2) {
        return false;
      }
      return true;
    });
    return selections;
  };

  brush.react = function(brushend) {
    var sel = brush.findSelection(d3_brush.extent());
    marks.classed("mark-selected", false)
         .classed("mark-unselected", true);
    sel.classed("mark-selected", true)
       .classed("mark-unselected", false);
    if (sel.size() === 0) {
      marks.classed("mark-selected", false)
           .classed("mark-unselected", false);
    }
    return brush;
  };

  return brush;
};

}());
