window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};
arc.visual = arc.visual || {};

arc.report.path = function() {
  var dialog_selection = null,
      sb = null,
      rb = null,
      mr = null,
      mr_sql,
      mr_output,
      fl_width,
      fl_height,
      report,
      woffset = 0,
      sql_output,
      flow_type,
      topk_spinner = null,
      topk_default = 20,
      enable_exit_events = false,
      user_settings,
      columnCounts,
      dataResult,
      goals_display = false;

  let linkColor = arc.settings.path.linkColor(),
      dropoffColor = arc.settings.path.dropoffColor(),
      highlightColor = arc.settings.path.highlightColor();

  const minHeightPerNodeSetting = arc.settings.flow.minHeightPerNode().value('50');
  const minWidthPerNodeSetting = arc.settings.flow.minWidthPerNode().value('150');

  var saved_info = {};

  var path = arc.report.base()
                       .needs_events(true);

  path.user_settings()
      .addSetting([
        arc.colorpalette.setting(),
        linkColor,
        dropoffColor,
        highlightColor,
        minHeightPerNodeSetting,
        minWidthPerNodeSetting,
      ])
      .removeSetting([
        'axis_label_font_size',
        'axis_ticks_font_size',
        'label_font_size',
        'label_color',
        'legend_font_size',
        'visual_show_detail_data',
      ]);

  // Dummy function added because path's base object is not same as driver (ARC-9501)
  path.report = function(_) {
    if (!arguments.length) return report;
    report = _;

    return path;
  };

  path.dataResult = function(_) {
    if (!arguments.length) return dataResult;
    dataResult = _;
    return path;
  };

  path.permitDetailData = function() {
    return false;
  };

  path.shelf_data([
    {
      name: "time_shelf",
      display_name: "Time",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "session_shelf",
      display_name: "Session",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "events_path_shelf",
      display_name: "Path",
      shelf_type: arc.report.shelf_types.EVENT_FLOW,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: "measure_shelf",
      display_name: "Measure",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: "filters_shelf",
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ]);

  var constraints = {
    minimums: {
      time_shelf: 1,
      session_shelf: 1,
      measure_shelf: 1,
      events_path_shelf: 1,
    },
    maximums: {
      measure_shelf: 1,
      time_shelf: 1
    }
  };

  var checkboxes = [];

  path.sqlOutput = function(_) {
    if (!arguments.length) return sql_output;
    sql_output = _;
    var sql = path.generateSql();
    if (sql === path.sqlQuery()) {
      // We have the output for our sql query - draw paths
      path.redraw();
    } else {
      path.undraw();
    }
    return path;
  };

  path.flow_type = function(_) {
    if (!arguments.length) return flow_type;
    flow_type = _;
    return path;
  };

  path.checkboxes = function(_) {
    if (!arguments.length) return checkboxes;
    checkboxes = _;
    return path;
  };

  path.saveEdit = function(new_text) {
    dialog_selection.text(new_text);
    return path;
  };

  var dropoff_behavior = [
    ['flow_cb_dropoff', 'Display dropoff node'],
    ['flow_cb_exitevent', 'Display exit events'],
  ];

  path.setupTopk = function(td) {
    // Add in the topk element
    const input =
      td.append('input')
        .attr('name', 'topk_spinner')
        .attr('type', 'number')
        .attr('placeholder', 'eg, 100')
        .classed('form-control form-control-sm topk_spinner', true);

    topk_spinner = $(input[0]);

    // For new reports, default to top 20 paths. Otherwise, set value to
    // saved topK value.

    topk_spinner.val(saved_info.topk || topk_default);

    return path;
  };

  path.setupDropoffBehavior = function(td) {
    if (!enable_exit_events) {
      return;
    }
    // Add in the multi-select element to pick dropoff mode
    td.append("select")
      .style("margin-left", "30px")
      .attr("id", "viz-flow-dropoff-select")
      .selectAll("#nosel")
      .data(dropoff_behavior)
      .enter()
        .append("option")
        .attr("value", function(d) {
          return d[0];
        })
        .text(function(d) {
          return d[1];
        });
    return path;
  };

  // Called once at startup time
  path.setup = function() {
    path.base_setup();

    var row,
        td;

    if ((flow_type === 'flow') || (flow_type === 'funnel')) {
      row = d3.select("#" + path.shelves_dom_id()).append("div")
             .classed("topk-wrapper", true);
      row.append("span")
         .classed("topk-label", true)
         .text("Top K:");

      td = row.append("span");
      path.setupTopk(td);
    }

    if (flow_type === 'funnel') {
      path.setupDropoffBehavior(td);
    }

    return path;
  };

  // Called to return an object that can later be passed into
  // path.restore to restore state
  path.save = function() {
    var flow_info = saved_info;
    flow_info = path.save_basics(flow_info);

    var topk = topk_spinner.val();
    if (topk !== null) {
      flow_info.topk = topk;
    }

    if (enable_exit_events) {
      var dropoff = $("#viz-flow-dropoff-select").val();
      dropoff_behavior.forEach(function(d) {
        if (dropoff === d[0]) {
          flow_info.dropoff_behavior = d[0];
        }
      });
    }
    return flow_info;
  };

  path.cleanup = function() {
    path.shelf_data().forEach(function(shelf_element) {
      var shelf_name = shelf_element[0];

      d3.select("#" + shelf_name)
        .remove();
    });
    d3.select("#" + path.shelves_dom_id())
      .selectAll("input")
      .remove();
    d3.select("#" + path.shelves_dom_id())
      .selectAll("label")
      .remove();
  };

  path.restore = function(flow_info, switchingVisualType) {
    saved_info = flow_info;
    path.restore_basics(flow_info, switchingVisualType);

    // Define messages to be included in shelves
    var timeMessage = [
                      path.createPopoverMessages('timePath'),
                      path.createPopoverMessages('acceptsOne')
                     ],
        sessionMessage = [
                      path.createPopoverMessages('sessionPath'),
                      path.createPopoverMessages('acceptsMultiple')
                    ],
        pathMessage = [
                        path.createPopoverMessages('eventPath'),
                        path.createPopoverMessages('goalPath'),
                        path.createPopoverMessages('acceptsMultiple')
                      ],
        measureMessages = [
                            path.createPopoverMessages('measurePath'),
                            path.createPopoverMessages('acceptsOne')
                          ];
    // Create those popovers
    path.addShelfDescription(constraints, {
                                            time_shelf: timeMessage,
                                            session_shelf: sessionMessage,
                                            events_path_shelf: pathMessage,
                                            measure_shelf: measureMessages
                                          });

    if (flow_info.topk) {
      topk_spinner.val(flow_info.topk);
    }

    // Default dropoff behavior
    $("#viz-flow-dropoff-select").val(dropoff_behavior[0][0]);

    if (enable_exit_events) {
      // Override with saved information
      dropoff_behavior.forEach(function(d) {
        if (flow_info.dropoff_behavior === d[0]) {
          $("#viz-flow-dropoff-select").val(d[0]);
        }
      });
    }
    return path;
  };

  path.generateSql = function() {
    sb = path.getSelectBuilder(true /* nolimit */);
    rb = path.getRequestBuilder(true);
    rb.requestType('MR');
    path.addFiltersToSb(sb);
    path.addFiltersToRb(rb);

    mr = arc.mrBuilder()
            .select_builder(sb);

    if (flow_type === 'funnel') {
      mr.is_funnel(true);

      if (enable_exit_events) {
        mr.show_dropoff($("#viz-flow-dropoff-select").val() === "flow_cb_dropoff")
          .show_exit_events($("#viz-flow-dropoff-select").val() === "flow_cb_exitevent");
      } else {
        mr.show_dropoff(true)
          .show_exit_events(false);
      }
    }

    path.shelfExpressionCb("time_shelf", true, function(expr) {
      mr.addOrderBy(expr);
      rb.time(expr);
    });

    path.shelfExpressionCb("session_shelf", true, function(expr) {
      mr.addPartitionBy(expr);
      rb.session(expr);
    });

    path.shelfElementCb("measure_shelf", function(element) {
      var expr = element.getExpression(null, true);
      var known_aggregates = {
        'sum': 1,
        'count': 1,
        'distinct': 1,
        'max': 1,
        'min': 1,
        'avg': 1,
        'ndv': 1,
        'partial_ndv': 1,
        'final_ndv': 1
      };
      var dl = expr.split(/[,()\s]+/);
      var mcols = [];
      dl.forEach(function(d) {
        if (d in known_aggregates) {
          return;
        }
        d = d.trim();
        if (d !== '') {
          mcols.push(d);
        }
      });
      // Look for count distincts first, and then other aggregates
      var r = expr.match(/^\s*count\s*\(\s*distinct\s*\((.+?)\s*\)/i);
      if (r) {
        var cd = ["count(distinct "];
        r[1].split(",").forEach(function(c) {
          cd.push("mr_tmp_1." + c.trim());
          cd.push(",");
        });
        cd[cd.length - 1] = ")";
        expr = cd.join("");
      } else {
        r = expr.match(/^(.+)\((.+)\)$/);
        if (r) {
          if ((r[1] in known_aggregates) && (r[2].search(/^\d+$/) < 0)) {
            expr = r[1] + "(mr_tmp_1." + r[2] + ")";
          }
        }
      }
      mcols.forEach(function(col) {
        mr.addMagnitudeMeasure(col);
      });
      mr.magnitude(expr)
        .magnitudeAlias('path_frequency');
      rb.addAggregate(expr);
    });

    var path_cnt = path.shelfElementCount("events_path_shelf");
    goals_display = false;

    columnCounts = [];
    var eventIds = [];
    path.shelfElementCb("events_path_shelf", function(shelf_element, idx) {
      columnCounts.push({
        colname: shelf_element.event_element().display(),
        min: shelf_element.event_min_count(),
        max: shelf_element.event_max_count(),
      });
      mr.addPath(shelf_element);
      if (idx === path_cnt - 1) {
        if (shelf_element.event_goal()) {
          goals_display = true;
        }
      }

      if (!shelf_element.event_element()) {
        return;
      }

      var id = shelf_element.event_element().id();
      var min = shelf_element.event_min_count() || 1;
      var max = shelf_element.event_max_count() || 1;
      eventIds.push({
        id:id,
        min:min,
        max:max,
        goal:goals_display}
      );
    });

    if (flow_type === 'funnel') {
      rb.events('funnel', eventIds);
    }
    else {
      rb.events('flow', eventIds);
    }

    var topk = topk_spinner.val();
    if (topk !== null) {
      mr.topk(topk);
      rb.limit(topk);
    }

    mr.addProjection('path');
    if (flow_type === 'funnel') {
      mr.addProjection('arc_path_priority');
    }

    var sql = mr.getSql();
    sql = path.substituteParams(sql);
    return sql;
  };

  path.refresh = function(force_redraw) {

    // NOTE: this counter is used in arcweb/apps/views_snapshot.py
    arc.counters.increment('viz_drawVisual_start');

    if (!path.check_shelf_contents(constraints)) {
      arc.report.alertMessage("Elements must be present in the 'Time', " +
          "'Session', 'Path', and 'Measure' shelves");
      return;
    }

    mr_sql = path.generateSql();
    path.sqlQuery(mr_sql);

    path.runSqlCmd(mr_sql, function(o) {
      path.setTitleText();
      sql_output = o;

      path.sb_colnames(o.colnames);
      path.sb_coltypes(o.coltypes);
      const result = arc.data.result()
                        .rows(o.rows);

      o.colnames.forEach(function(colname, colindex) {
        let col = arc.data.column()
          .colname(colname)
          .coltype(o.coltypes[colindex]);
        result.add_column(col);
      });

      result.finalize();
      path.dataResult(result);

      path.redraw();

      // NOTE: this counter is used in arcweb/apps/views_snapshot.py
      arc.counters.increment('viz_drawVisual_end');

    }, JSON.stringify(rb.dsreq()));
  };

  // Converts jsonselect data into nodes/links structure that sankey.js needs
  // Also add in information about full paths and frequencies, so that we can
  // display the corresponding path of a selected node/link
  path.jsontosankeydata = function(input, eliminatedups) {

    eliminatedups = typeof eliminatedups !== 'undefined' ? eliminatedups:false;

    // Internal variables
    var jo_int = {
      nodeidxs: {}, // [{'visits_1': 1}, {'spendtime' : 2}
      targets:[]
    };

    // Output format
    var jo_out = {
      "nodes": [], // [{"name":'visits_1', "index" :1, "display":'visits'}..
      "links": [],
      "pathrows": [], //[[1, 2, 4], [3, 4, 5]]
      "pathfrequencies": []
    };

    var pathcolidx,
        pathfreqcolidx;

    var current_nodes_index = 0;
    input.colnames.forEach(function(colname, colindex) {
      if (colname === "path") {
        pathcolidx = colindex;
        return;
      }
      if (colname === "path_frequency") {
        pathfreqcolidx = colindex;
        return;
      }
    });

    if ((pathcolidx === undefined) || (pathfreqcolidx === undefined)) {
      return undefined;
    }

    input.rows.forEach(function(row, pathrowidx) {

      var nodupfields, field_display, field, field_idx, lf, thisnodeidx;
      var linkexists, thislinkidx;
      var path = row[pathcolidx]; // path sequence
      var path_frequency = row[pathfreqcolidx]; // count per path

      var fields = path.split(",");

      // eliminate nodes that show up more than once in a path
      if (eliminatedups) {
        nodupfields = [];
        fields.forEach(function(f) {
          if (nodupfields.indexOf(f) === -1) {
            nodupfields.push(f);
          }
        });
        fields = [];
        nodupfields.forEach(function(n) {
          fields.push(n);
        });
      }

      if (goals_display) {
        // uniquely add last field of path to targets
        lf = fields[fields.length - 1];
        if (jo_int.targets.indexOf(lf) === -1) {
          jo_int.targets.push(lf);
        }
      }

      // process each node within each path sequence
      var last = false;
      var index = 1;
      for (var j = 0; j < fields.length; j++) {
        field = fields[j];

        // append _(sequentially_increasing_index) to each field to
        // differentiate from same field showing up multiple times in same path
        field_display = field;
        field_idx = null;
        if (!goals_display || (j < fields.length - 1)) {
          field_idx = index;
          field = field + '_' + index;
        }
        index += 1;

        // process field, add to the nodeidxs and to nodes
        if (jo_int.nodeidxs[field] === undefined) { //if node doesnt have an idx
          jo_int.nodeidxs[field] = current_nodes_index;
          current_nodes_index += 1;
          // add to full description of nodes (index, name, display)
          jo_out.nodes.push({"index": field_idx, "name": field, "display": field_display});
        }

        // Set up pathrows (full paths in terms of node idxs) and path frequency
        if (jo_out.pathrows[pathrowidx] === undefined) {
          jo_out.pathrows[pathrowidx] = [];
          jo_out.pathfrequencies[pathrowidx] = Number(path_frequency);
        }
        jo_out.pathrows[pathrowidx].push(jo_int.nodeidxs[field]);

        // Each node should know the paths that it shows up in
        thisnodeidx = jo_int.nodeidxs[field];
        if (jo_out.nodes[jo_int.nodeidxs[field]].paths === undefined) {
          jo_out.nodes[jo_int.nodeidxs[field]].paths = [];
        }
        jo_out.nodes[jo_int.nodeidxs[field]].paths.push(pathrowidx);

        // If a previous step exists in the path, add to link pairs
        // Update frequency if link pair already exists
        if (last) {
          linkexists = false;
          var link;
          for (var tli = 0; tli < jo_out.links.length; tli++) {
            link = jo_out.links[tli];
            if (link.source === jo_int.nodeidxs[last]) {
              if (link.target === jo_int.nodeidxs[field]) {
                linkexists = true;
                link.value = (Number(link.value)) + (Number(path_frequency));
                thislinkidx = tli;
              }
            }
          }
          if (!linkexists) {
            jo_out.links.push({"source": jo_int.nodeidxs[last],
                                "target" : jo_int.nodeidxs[field],
                                "value" : Number(path_frequency)});
            thislinkidx = jo_out.links.length - 1;
          }

          // Each link should know the paths that it shows up in
          if (jo_out.links[thislinkidx].paths === undefined) {
            jo_out.links[thislinkidx].paths = [];
          }
          jo_out.links[thislinkidx].paths.push(pathrowidx);

        }

        // remember last field traversed to help with link pair calculation
        last = field;
      }
    });

    /*
    $.each(jo_out, function(k, v) {
      console.log('key', k);
      $.each(v, function(ai, av) {
        console.log(av);
      });
    });
     */

    return (jo_out);
  };

  path.redraw = function() {
    var chart_div = this.chart_div();
    path.undraw();

    if (!sql_output) {
      return;
    }
    mr_output = path.jsontosankeydata(sql_output, /* eliminatedups */ false);

    if (!mr_output) {
      return;
    }

    path.draw_sankey({
      data: mr_output,
      funnel_prefix: mr.funnel_prefix(),
      flow_type: flow_type,
      goals_display: goals_display,
      columnCounts: columnCounts,
      linkColor: linkColor.value(),
      dropoffColor: dropoffColor.value(),
      highlightColor: highlightColor.value(),
      minHeightPerNode: Number(minHeightPerNodeSetting.value()),
      minWidthPerNode: Number(minWidthPerNodeSetting.value()),
    });

    return path;
  };

  return path;
};

}());
