window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};

arc.report.stackedbar = function() {
  var sb = null,
      rb = null,
      sb_sql,
      is_horizontal = false,
      formatPercent = d3.format(".0%"),
      formatPercentTwo = d3.format(".2%");

  var setting_scale = arc.report.user_settings.scale_percentage();
  var legend_visibility = arc.report.user_settings.legend_visibility();
  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  setting_scale,
                                  legend_visibility
                                ]);

  var bar = arc.report.base()
                      .colors(arc.report.highcard_colors())
                      .chart_type("bar")
                      .user_settings(user_settings);

  bar.is_percentage_supported = function() {
    return ((bar.chart_type() === "bar") || (bar.chart_type() === "area"));
  };

  bar.is_percentage_scale = function() {
    return (bar.is_percentage_supported() &&
            (setting_scale.value() === "PERCENTAGE"));
  };

  bar.is_horizontal = function(_) {
    if (!arguments.length) return is_horizontal;
    is_horizontal = _;
    return bar;
  };

  bar.sb_sql = function(_) {
    if (!arguments.length) return sb_sql;
    sb_sql = _;
    return bar;
  };

  // Called once at startup time
  bar.setup = function() {
    bar.base_setup();
    return bar;
  };

  var saved_info = {};

  // Called to return an object that can later be passed into
  // bar.restore to restore state
  bar.save = function() {
    var bar_info = saved_info;
    bar_info = bar.report_save_shelves(bar.shelf_data(), bar_info);
    bar_info = bar.saveClickBehaviors(bar_info);
    return bar_info;
  };

  bar.restore = function(bar_info) {
    if (bar_info === undefined) {
      return bar;
    }
    saved_info = bar_info;
    bar.report_restore_shelves(bar.shelf_data(), bar_info);
    bar.restoreClickBehaviors(bar_info);
    return bar;
  };

  bar.generateSql = function() {
    sb_sql = null;
    sb = bar.getSelectBuilder();
    rb = bar.getRequestBuilder();
    var dimensions_count = bar.shelfElementCount("dimensions_shelf");
    if (dimensions_count < 2) {
      arc.report.alertMessage("Please specify at least two dimensions and one aggregate");
      return bar;
    }
    var aggregate_count = bar.shelfElementCount("aggregates_shelf");
    if (aggregate_count === 0) {
      arc.report.alertMessage("Please specify at least two dimensions and one aggregate");
      return bar;
    }

    bar.addDimensionsAndAggregates(sb, true, null, rb);
    bar.addFiltersToSb(sb);
    bar.addFiltersToRb(rb);
    sb = bar.finalizeSql(sb);

    sb_sql = sb.getSql();
    console.log(sb_sql);
    return bar;
  };

  bar.refresh = function(force_redraw) {
    bar.generateSql();
    return bar.loadData(sb_sql, force_redraw, JSON.stringify(rb.dsreq()));
  };

  bar.redraw = function() {
    bar.clear_errors();
    var chart_div = this.chart_div();

    if (sb_sql !== bar.sqlQuery()) {
      console.log(sb_sql);
      console.log(bar.sqlQuery());
      return bar.refresh();
    }

    // Get output headers and rows
    var sb_colnames = bar.sb_colnames(),
        sb_coltypes = bar.sb_coltypes(),
        sb_rows = bar.sb_rows();

    if (!sb_rows) {
      return;
    }

    if (!sb_rows) {
      bar.undraw();
      d3.select("#" + chart_div)
        .append("svg")
          .append("div")
          .text("No data available.");
      return;
    }

    bar.undraw();

    bar.unselectOnClick();

    var date_formatter = d3.time.format("%Y-%m-%d");
    var dim_is_date = bar.dimension_axis_is_date();

    var unique_dims = [],
        unique_dims_hash = {},
        unique_dims_total = {},
        stack_values = [],
        stack_values_hash = {},
        dim_values = [],
        dim_values_hash = {},
        a_min = 0,
        a_max = 0;

    //
    // We expect
    //   col 1: dimension on the x axis
    //   col 2: dimension used for stacking
    //   col last: aggregate
    //   all other columns: individual charts are made for each unique
    //                      combination here
    //
    var dim_colindex = bar.dim_start_idx(),
        stack_colindex = bar.dim_start_idx() + 1,
        agg_colindex = bar.agg_start_idx(),
        dim_colname = sb_colnames[dim_colindex],
        stack_colname = sb_colnames[stack_colindex],
        agg_colname = sb_colnames[agg_colindex];

    var dname_to_idx = {};
    var didx_to_d = {};
    var dname_cnt = 0;
    var dim_totals = {};

    // Apply local filters
    sb_rows = bar.filterLocally(sb_rows, dim_colindex);
    if (bar.chart_type === "bar") {
      if (sb_rows.length > 800) {
        bar.draw_error("There are too many data points (> 800) to plot as a bar or area " +
            "chart. Try plotting this as a line chart instead, or use Top-K to " +
            "reduce the number of data points you are looking at");
        return;
      }
    }

    sb_rows.forEach(function(d) {
      if (dim_is_date) {
        if (!(d[dim_colindex] instanceof Date)) {
          d[dim_colindex] = date_formatter.parse(d[dim_colindex]);
        }
      }

      // Get all unique x axis dimensions in dname_to_idx and didx_to_d
      if (!(d[dim_colindex] in dname_to_idx)) {
        dname_to_idx[d[dim_colindex]] = dname_cnt;
        didx_to_d[dname_cnt] = d;
        ++dname_cnt;
      }

      // Convert the measure to a number
      d[agg_colindex] = Number(d[agg_colindex]);

      if (!(d[stack_colindex] in stack_values_hash)) {
        stack_values_hash[d[stack_colindex]] = 1;
        stack_values.push(d[stack_colindex]);
      }

      // Track the maximum value within a single dimension so
      // we know how to scale the y axis
      if (d[dim_colindex] in dim_values_hash) {
        dim_totals[d[dim_colindex]] += d[agg_colindex];
      } else {
        dim_values_hash[d[dim_colindex]] = 1;
        dim_values.push(d[dim_colindex]);
        dim_totals[d[dim_colindex]] = d[agg_colindex];
      }

      // Track the min and max measure values
      a_min = Math.min(a_min, d[agg_colindex]);
      a_max = Math.max(a_max, dim_totals[d[dim_colindex]]);

      // Track all unique values required to draw individual graphs
      if (bar.dim_cnt() > 2) {
        var dim = d.slice(bar.dim_start_idx() + 2, bar.dim_end_idx() + 1);
        if (dim in unique_dims_hash) {
          unique_dims_total[dim] += d[agg_colindex];
        } else {
          unique_dims_hash[dim] = 1;
          unique_dims_total[dim] = d[agg_colindex];
          unique_dims.push(dim);
        }
      }
    });
    if (bar.dim_cnt() === 2) {
      unique_dims = [[]];
    }
    unique_dims.sort(function(a, b) {
      return unique_dims_total[b] - unique_dims_total[a];
    });

    var max_text_len = bar.dim_axis_max_textlen(dim_values),
        x_is_dim = !bar.idx_is_numeric([dim_values], 0);
    var xaxis_len = 0;
    if (is_horizontal) {
      xaxis_len = bar.dim_axis_max_textlen(
          [sb_colnames[bar.agg_start_idx()]],
          bar.xaxis_label_maxlen());
    } else {
      xaxis_len = bar.dim_axis_max_textlen(
          [sb_colnames[bar.dim_start_idx()]],
          bar.xaxis_label_maxlen());
    }

    // Adjust for drawing out the legends for the stack values on the right
    // of the visual (along the X axis)
    var sv = stack_values.slice(0);
    // We may need to display a legend that says '35 not shown' if we cannot
    // display all legends. Ensure there is enough space for that text.
    // Note that this label does not come with a color, so we adjust for that
    // in the text below.
    sv.push(" not shown)");
    xaxis_len = Math.max(xaxis_len,
        15 + bar.dim_axis_max_textlen(sv,
        bar.xaxis_label_maxlen()));
    var chart_type = bar.chart_type();

    var margin;
    if (is_horizontal) {
      margin = {top: 20, right: xaxis_len + 10,
                bottom: 40, left: 20 + max_text_len};
    } else {
      margin = {top: 20, right: xaxis_len + 10,
                bottom: 20 + (dim_is_date || !x_is_dim ? 20 : max_text_len), left: 40};
    }
    if (bar.in_dashboard()) {
      margin.bottom -= 10;
      margin.bottom = Math.max(0, margin.bottom);
    }
    var width = $("#" + chart_div).width() - margin.left - margin.right;
    var height = $("#" + chart_div).height() - margin.top - margin.bottom;
    var min_height = 250;
    if ((unique_dims.length >= 3) && (height > min_height * 2)) {
      height = height / 2;
      height -= 75;
    } else {
      height -= 75;
    }
    if (height < 150) {
      height = 150;
    }

    var d_minw = 12 * dname_cnt,
        d_maxw;
    if (chart_type === "bar") {
      d_maxw = 30 * dname_cnt;
    } else {
      d_maxw = 100 * dname_cnt;
    }

    var a_maxrange, d_maxrange;
    if (is_horizontal) {
      height = Math.max(d_minw, height);
      height = Math.min(d_maxw, height);
      a_maxrange = width;
      d_maxrange = height;
    } else {
      width = Math.max(d_minw, width);
      width = Math.min(d_maxw, width);
      a_maxrange = height;
      d_maxrange = width;
    }

    var a_scale = d3.scale.linear();
    if (bar.is_percentage_scale()) {
      a_scale.domain([0, 1]);
    } else {
      a_scale.domain([a_min, a_max]);
    }
    if (is_horizontal) {
      a_scale.range([0, a_maxrange]);
    } else {
      a_scale.range([a_maxrange, 0]);
    }
    var d_scale,
        dim_sz;
    if (dim_is_date) {
      var dl = bar.setup_date_scale(dim_values, d_maxrange);
      d_scale = dl[0];
      dim_sz = dl[1];
      dim_values = dl[2];
    }
    if (!d_scale) {
      d_scale =
        d3.scale.ordinal()
          .domain(dim_values)
          .rangeRoundBands([0, d_maxrange]);
      dim_sz = d_scale.rangeBand();
    }

    unique_dims.forEach(function(unique_dim) {
      var sb_rows_dim = sb_rows.filter(function(d) {
        return bar.arraysEqual(
            d.slice(bar.dim_start_idx() + 2, bar.dim_end_idx() + 1),
            unique_dim);
      });

      // We have filtered out rows for the combination of dimensions we
      // are going to plot. We may have a set of missing values here because
      // there was no data for particular dimensions - for these use-cases
      // we fill in an aggregate of zero so we can plot these correctly.

      // 1. Capture all the dim values that exist in the filtered dataset
      var sb_rows_by_dimvalue = {};
      sb_rows_dim.forEach(function(d) {
        sb_rows_by_dimvalue[[d[dim_colindex], d[stack_colindex]]] = d;
      });

      // 2. Walk through all known dimensions, back-filling any that
      //    don't exist in the filtered dataset
      var sb_rows_all_dimvalues = [];
      dim_values.forEach(function(dim_value) {
        stack_values.forEach(function(stack_value) {
          var v = [dim_value, stack_value];
          if (v in sb_rows_by_dimvalue) {
            sb_rows_all_dimvalues.push(sb_rows_by_dimvalue[v]);
          }
        });
      });

      // Finally, use this new dataset to plot
      sb_rows_dim = sb_rows_all_dimvalues;

      bar.draw_svg(sb_colnames, sb_rows, sb_rows_dim, width, height,
          margin, d_scale, a_scale,
          unique_dim, dim_values, stack_values,
          dname_to_idx, didx_to_d,
          dim_colname, stack_colname, agg_colname,
          dim_colindex, stack_colindex, agg_colindex, dim_is_date, dim_sz);
    });
  };

  bar.draw_svg = function(sb_colnames, sb_rows, sb_rows_dim, width, height,
      margin, d_scale, a_scale,
      unique_dim, dim_values, stack_values,
      dname_to_idx, didx_to_d,
      dim_colname, stack_colname, agg_colname,
      dim_colindex, stack_colindex, agg_colindex, dim_is_date, dim_sz) {

    if (bar.is_percentage_scale()) {
      var percentage_colname = 'percentage';
      bar.compute_percentages(sb_rows_dim, percentage_colname,
          function(d) {
            return d[dim_colindex];
          },
          function(d) {
            return Number(d[agg_colindex]);
          });
      // Note that we are changing agg_colindex to refer to the newly computed
      // aggregate value
      agg_colindex = percentage_colname;
    }

    var a_fmt = d3.format(","),
        chart_div = this.chart_div(),
        hdr = [],
        sb_coltypes = bar.sb_coltypes();

    unique_dim.forEach(function(d, i) {
      hdr.push(sb_colnames[i + bar.dim_start_idx() + 2]);
      hdr.push(":");
      hdr.push("<b style='margin: 0px 30px 0px 5px;'>" + d + "</b>");
    });
    d3.select("#" + chart_div).append("div")
                              .classed("viz-bar-title", true)
                              .html(hdr.join(''));

    var dim_names_set = {};
    var dim_names = [];
    var agg_by_dimval = {};
    sb_rows_dim.forEach(function(d, i) {
      if (d[dim_colindex] in agg_by_dimval) {
        agg_by_dimval[d[dim_colindex]] += d[agg_colindex];
      } else {
        agg_by_dimval[d[dim_colindex]] = d[agg_colindex];
      }
      if (!(d[dim_colindex] in dim_names_set)) {
        dim_names_set[d[dim_colindex]] = 1;
        dim_names.push(d[dim_colindex]);
      }
    });

    var svg = d3.select("#" + chart_div)
                .append("svg")
                .classed("viz-bar-svg", true)
                .style("width", width + margin.left + margin.right + "px")
                .style("height", height + margin.top + margin.bottom + "px")
                  .append("g")
                  .classed("viz-bar-g", true)
                  .attr("transform",
                    "translate(" + margin.left + "," + margin.top + ")");

    var stack = d3.layout.stack()
        .values(function(d) { return d.values; });

    var color = d3.scale.ordinal()
                  .range(bar.colors());

    var stacks_hash = {};
    stack_values.forEach(function(stack_value) {
      stacks_hash[stack_value] = {};
    });

    sb_rows_dim.forEach(function(d) {
      stacks_hash[d[stack_colindex]][d[dim_colindex]] = d;
    });

    dim_values.forEach(function(dim_value) {
      stack_values.forEach(function(stack_value) {
        if (!(dim_value in stacks_hash[stack_value])) {
          stacks_hash[stack_value][dim_value] = sb_rows_dim[0].slice(0);
          stacks_hash[stack_value][dim_value][agg_colindex] = 0;
        }
      });
    });

    var stack_names = d3.keys(stacks_hash);

    // XXX: Hack - remove and add color.domain line in the else stmt
    if (stack_names.length === 3 &&
        stack_names.indexOf("fail") !== -1 &&
        stack_names.indexOf("pass") !== -1 &&
        stack_names.indexOf("soft_fail") !== -1) {

      stack_names = ["fail", "soft_fail", "pass"];
      color.domain(stack_names);
    } else if (stack_names.length === 2 &&
        stack_names.indexOf("fail") !== -1 &&
        stack_names.indexOf("soft_fail") !== -1) {

      stack_names = ["fail", "soft_fail"];
      color.domain(stack_names);
    } else {
      color.domain(stack_names.slice(0).sort());
    }

    var stack_data = stack(stack_names.map(function(name) {
      var s = {
        name: name,
        values: dim_names.map(function(d) {
          return {
            d: stacks_hash[name][d],
            x: d,
            y: stacks_hash[name][d][agg_colindex],
            stack_name: name};
        })
      };
      return s;
    }));

    var chart_type = bar.chart_type();
    if (chart_type === "bar") {
      bar.draw_bar(svg, stack_data,
          stack_colname, dim_colname, agg_colname,
          stack_colindex, dim_colindex, agg_colindex,
          color, d_scale, a_scale, dim_sz, height);
    }

    if (chart_type === "area") {
      bar.draw_area(svg, stack_data,
          stack_colname, dim_colname, agg_colname,
          stack_colindex, dim_colindex, agg_colindex,
          color, d_scale, a_scale,
          dname_to_idx, didx_to_d, stacks_hash, dim_sz, height);
    }

    bar.bar_draw_axes(svg, width, height, sb_rows_dim, d_scale, a_scale, dim_sz,
        dim_is_date, dim_values, color, margin);
  };

  bar.bar_draw_axes = function(svg, width, height, sb_rows_dim, d_scale, a_scale,
      dim_sz, dim_is_date, dim_values, color, margin) {

    var d_axis = d3.svg.axis()
                   .scale(d_scale);
    if (is_horizontal) {
      d_axis.orient("left");
    } else {
      d_axis.orient("bottom");
    }
    if (dim_is_date) {
      d_axis.tickFormat(d3.time.format("%Y-%m-%d"));
      if (dim_values.length < 20) {
        d_axis.ticks(2);
      } else if (dim_values.length < 50) {
        d_axis.ticks(5);
      }
    }
    // Do not create more ticks marks than dimension elements
    if (d_axis.ticks() > sb_rows_dim.length) {
      d_axis.ticks(sb_rows_dim.length);
    }
    var a_axis = d3.svg.axis()
                   .scale(a_scale);
    if (bar.is_percentage_scale()) {
      a_axis.tickFormat(formatPercent);
    } else {
      a_axis.tickFormat(bar.tickFormatDefaultNumeric);
    }

    var chartSVG;

    if (is_horizontal) {
      a_axis.orient("bottom");
      svg.append("g")
         .attr("class", "y arc-viz-axis")
         .call(d_axis);

      chartSVG = d3.select(svg.node().parentNode)
         .append("svg")
         .attr('height', height + margin.bottom + margin.top - 20)
         .append("g")
         .attr("class", "x arc-viz-axis")
         .attr("transform", "translate( " + margin.left + "," + (margin.top + height) + ")")
         .call(a_axis);
    } else {
      a_axis.orient("left");
      chartSVG = d3.select(svg.node().parentNode)
          .append("svg")
          .attr('height', height + margin.bottom + margin.top - 20)
          .append("g")
          .attr("class", "x arc-viz-axis")
          .attr("transform", "translate( " + margin.left + "," + (margin.top + height) + ")")
          .call(d_axis);
      svg.append("g")
          .attr("class", "y arc-viz-axis")
          .call(a_axis);
    }

    if (!is_horizontal && !dim_is_date && !bar.idx_is_numeric([dim_values], 0)) {
      chartSVG.selectAll(".x .tick text")
         .style("text-anchor", "end")
         .attr("transform", function (d, i) {
           return "translate(-10, 0) rotate(" + bar.x_slant() + ",0,0)";
         });
    }

    bar.draw_labels(svg, is_horizontal, width, height + margin.bottom - 10);
    if (legend_visibility.isChecked) {
      bar.draw_legend({
        svg: svg,
        width: width,
        height: height,
        color: color,
        sort_domain: true
      });
    }
    return bar;
  };

  bar.draw_area = function(svg, stack_data,
      stack_colname, dim_colname, agg_colname,
      stack_colindex, dim_colindex, agg_colindex,
      color, d_scale, a_scale,
      dname_to_idx, didx_to_d, stacks_hash, dim_sz, height) {

    var sb_coltypes = bar.sb_coltypes();
    var tr_area = d3.svg.area()
        .x(function(d) { return d_scale(d.x) + Math.floor(dim_sz/2); })
        .y0(height)
        .y1(height);

    var area = d3.svg.area()
        .x(function(d) { return d_scale(d.x) + Math.floor(dim_sz/2); })
        .y0(function(d) { return a_scale(d.y0); })
        .y1(function(d) { return a_scale(d.y0 + d.y); });

    var stack_elem = svg.selectAll(".stackedarea")
          .data(stack_data)
          .enter()
            .append("g")
            .attr("class", "stackedarea");

    var areas = stack_elem.append("path")
        .attr("class", "area")
        .style("fill", function(d) { return color(d.name); })
        .attr("d", function(d) { return tr_area(d.values); });

    areas.transition()
         .ease("easeOutCirc")
         .duration(300)
         .attr("d", function(d) { return area(d.values); });

    var get_didx = function() {
      var x = d3.mouse(svg.node())[0];
      return Math.floor(x/dim_sz);
    };

    var params_fn = function(d) {
      var didx = get_didx(),
          v = d.values[didx];
      return bar.base_params_fn(v.d);
    };

    var text_fn = function(d) {
      var didx = get_didx(),
          v = d.values[didx];
      return bar.base_text_fn(v.d);
    };

    var tooltip =
      arc.tooltip()
         .dom_id("#tooltip")
         .addSelection(areas)
         .text_fn(text_fn)
         .text_changes_on_move(true)
         .activate();

    bar.contextmenu('primary')
       .text_fn(text_fn)
       .params_fn(params_fn)
       .addTooltip(tooltip)
       .activate();
  };

  bar.draw_bar = function(svg, stack_data,
      stack_colname, dim_colname, agg_colname,
      stack_colindex, dim_colindex, agg_colindex,
      color, d_scale, a_scale, dim_sz, height) {

    var sb_coltypes = bar.sb_coltypes();

    stack_data.forEach(function(s) {
      var rect = svg.selectAll("#nosel")
          .data(s.values)
          .enter()
            .append("rect")
            .style("stroke", "white")
            .style("stroke-width", "1px")
            .style("fill", function(d) {
              return color(s.name);
            })
            .attr("x", function(d, i) {
              if (is_horizontal) {
                return 0;
              } else {
                return d_scale(d.x);
              }
            })
            .attr("y", function(d, i) {
              if (is_horizontal) {
                return d_scale(d.x);
              } else {
                return height;
              }
            })
            .attr("width", function(d, i) {
              if (is_horizontal) {
                return 0;
              } else {
                return dim_sz;
              }
            })
            .attr("height", function(d, i) {
              if (is_horizontal) {
                return dim_sz;
              } else {
                return 0;
              }
            });

      if (is_horizontal) {
        rect.transition()
            .ease("easeOutCirc")
            .delay(function(d, i) {
              return 100 + 10 * i;
            })
            .attr("x", function(d) {
              return a_scale(d.y0);
            })
            .attr("width", function(d) {
              return a_scale(d.y);
            });
      } else {
        rect.transition()
            .ease("easeOutCirc")
            .delay(function(d, i) {
              return 100 + 10 * i;
            })
            .attr("y", function(d) {
              return a_scale(d.y + d.y0);
            })
            .attr("height", function(d) {
              return a_scale(d.y0) - a_scale(d.y + d.y0);
            });
      }
    });

    var svg_rect = svg.selectAll("rect");
    var tt =
      arc.tooltip()
         .dom_id("#tooltip")
         .addSelection(svg_rect)
         .text_fn(function(d) {
           if (bar.is_percentage_scale()) {
             var tl = ["<br>"];
             bar.addTooltip(tl, 'Percentage', formatPercentTwo(d.d.percentage));
             return bar.base_text_fn(d.d) + tl.join("");
           }
           return bar.base_text_fn(d.d);
         })
         .activate();

    bar.contextmenu('primary')
       .addTooltip(tt)
       .text_fn(function(d) {
         if (bar.is_percentage_scale()) {
           var tl = ["<br>"];
           bar.addTooltip(tl, 'Percentage', formatPercentTwo(d.d.percentage));
           return bar.base_text_fn(d.d) + tl.join("");
         }
         return bar.base_text_fn(d.d);
       })
       .params_fn(function(d) {
         return bar.base_params_fn(d.d);
       })
       .activate();
  };

  return bar;
};

}());
