window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};

arc.report.stackedbar_agg = function() {
  var bar = arc.report.stackedbar()
                      .chart_type("bar-agg");

  bar.generateSql = function() {
    bar.sb_sql(null);
    var sb = bar.getSelectBuilder();
    var rb = bar.getRequestBuilder();
    var dimensions_count = bar.shelfElementCount("dimensions_shelf");
    if (dimensions_count < 1) {
      arc.report.alertMessage("Please specify at least one dimension and one aggregate");
      return bar;
    }
    var aggregate_count = bar.shelfElementCount("aggregates_shelf");
    if (aggregate_count === 0) {
      arc.report.alertMessage("Please specify at least one dimension and one aggregate");
      return bar;
    }

    bar.addDimensionsAndAggregates(sb, true, null, rb);
    bar.addFiltersToSb(sb);
    bar.addFiltersToRb(rb);
    sb = bar.finalizeSql(sb);

    bar.sb_sql(sb.getSql());
    return bar;
  };

  bar.redraw = function() {
    var chart_div = this.chart_div();

    if (bar.sb_sql() !== bar.sqlQuery()) {
      console.log(bar.sb_sql());
      console.log(bar.sqlQuery());
      return bar.refresh();
    }

    // Get output headers and rows
    var sb_colnames = bar.sb_colnames(),
        sb_coltypes = bar.sb_coltypes(),
        sb_rows = bar.sb_rows();

    if (!sb_rows) {
      return;
    }

    if (!sb_rows) {
      bar.undraw();
      d3.select("#" + chart_div)
        .append("svg")
          .append("div")
          .text("No data available.");
      return;
    }

    bar.undraw();
    bar.unselectOnClick();

    var date_formatter = d3.time.format("%Y-%m-%d");
    var dim_is_date = bar.dimension_axis_is_date();

    var unique_dims = [],
        unique_dims_hash = {},
        unique_dims_total = {},
        stack_values = [],
        dim_values = [],
        dim_values_hash = {};

    //
    // We expect
    //   first dimension: dimension on the x axis
    //   other dims: distinct bar graphs for unique combinations
    //   all aggregates: stacked colors
    //
    var dim_colindex = bar.dim_start_idx(),
        agg_startindex = bar.agg_start_idx(),
        agg_endindex = bar.agg_end_idx();

    var dname_to_idx = {},
        didx_to_d = {},
        dname_cnt = 0,
        agg_min = [],
        agg_max = [];

    stack_values = sb_colnames.slice(agg_startindex, agg_endindex + 1);

    // Apply local filters
    sb_rows = bar.filterLocally(sb_rows, dim_colindex);
    if (sb_rows.length > 800) {
      bar.draw_error("There are too many data points (> 800) to plot as a bar " +
          "chart. Try plotting this as a line chart instead, or use Top-K to " +
          "reduce the number of data points you are looking at");
      return;
    }

    sb_rows.forEach(function(d) {
      if (dim_is_date) {
        if (!(d[dim_colindex] instanceof Date)) {
          d[dim_colindex] = date_formatter.parse(d[dim_colindex]);
        }
      }

      // Get all unique x axis dimensions in dname_to_idx and didx_to_d
      if (!(d[dim_colindex] in dname_to_idx)) {
        dname_to_idx[d[dim_colindex]] = dname_cnt;
        didx_to_d[dname_cnt] = d;
        ++dname_cnt;
      }

      if (!(d[dim_colindex] in dim_values_hash)) {
        dim_values_hash[d[dim_colindex]] = 1;
        dim_values.push(d[dim_colindex]);
      }

      // Convert the measures into numbers
      for (var a = agg_startindex; a <= agg_endindex; ++a) {
        d[a] = Number(d[a]);
        if (agg_min[a - agg_startindex] === undefined) {
          agg_min[a - agg_startindex] = d[a];
          agg_max[a - agg_startindex] = d[a];
        } else {
          agg_min[a - agg_startindex] = Math.min(d[a], agg_min[a - agg_startindex]);
          agg_max[a - agg_startindex] = Math.max(d[a], agg_max[a - agg_startindex]);
        }
      }

      // Track all unique values required to draw individual graphs
      if (bar.dim_cnt() > 1) {
        var dim = d.slice(bar.dim_start_idx() + 2, bar.dim_end_idx() + 1);
        if (dim in unique_dims_hash) {
          unique_dims_total[dim] += d[agg_startindex];
        } else {
          unique_dims_hash[dim] = 1;
          unique_dims_total[dim] = d[agg_startindex];
          unique_dims.push(dim);
        }
      }
    });
    if (bar.dim_cnt() === 1) {
      unique_dims = [[]];
    }
    unique_dims.sort(function(a, b) {
      return unique_dims_total[b] - unique_dims_total[a];
    });

    var max_text_len = bar.dim_axis_max_textlen(dim_values),
        x_is_dim = !bar.idx_is_numeric([dim_values], 0);
    var xaxis_len = 0;
    if (bar.is_horizontal()) {
      xaxis_len = bar.dim_axis_max_textlen(
          ["aggregates"],
          bar.xaxis_label_maxlen());
    } else {
      xaxis_len = bar.dim_axis_max_textlen(
          [sb_colnames[bar.dim_start_idx()]],
          bar.xaxis_label_maxlen());
    }
    xaxis_len = Math.max(xaxis_len,
        15 + bar.dim_axis_max_textlen(stack_values,
        bar.xaxis_label_maxlen()));
    var chart_type = bar.chart_type();

    var margin;
    if (bar.is_horizontal()) {
      margin = {top: 20, right: xaxis_len + 10,
                bottom: 40, left: 20 + max_text_len};
    } else {
      margin = {top: 20, right: xaxis_len + 10,
                bottom: 20 + (dim_is_date || !x_is_dim ? 20 : max_text_len), left: 40};
    }
    if (bar.in_dashboard()) {
      margin.bottom -= 10;
      margin.bottom = Math.max(0, margin.bottom);
    }
    var width = $("#" + chart_div).width() - margin.left - margin.right;
    var height = $("#" + chart_div).height() - margin.top - margin.bottom;
    var min_height = 250;
    if ((unique_dims.length >= 3) && (height > min_height * 2)) {
      height = height / 2;
      height -= 75;
    } else {
      height -= 75;
    }
    if (height < 150) {
      height = 150;
    }

    var d_minw = 12 * dname_cnt,
        d_maxw = 30 * dname_cnt;

    var a_maxrange, d_maxrange;
    if (bar.is_horizontal()) {
      height = Math.max(d_minw, height);
      height = Math.min(d_maxw, height);
      a_maxrange = width;
      d_maxrange = height;
    } else {
      width = Math.max(d_minw, width);
      width = Math.min(d_maxw, width);
      a_maxrange = height;
      d_maxrange = width;
    }

    var a_min = Math.min.apply(Math, agg_min),
        a_max = Math.max.apply(Math, agg_max);

    a_min = Math.min(0, a_min);
    var a_scale = d3.scale.linear()
                    .domain([a_min, a_max]);
    if (bar.is_horizontal()) {
      a_scale.range([0, a_maxrange]);
    } else {
      a_scale.range([a_maxrange, 0]);
    }

    var d_scale,
        dim_sz;
    if (dim_is_date) {
      var dl = bar.setup_date_scale(dim_values, d_maxrange);
      d_scale = dl[0];
      dim_sz = dl[1];
      dim_values = dl[2];
    }
    if (!d_scale) {
      d_scale =
        d3.scale.ordinal()
          .domain(dim_values)
          .rangeRoundBands([0, d_maxrange]);
      dim_sz = d_scale.rangeBand();
    }

    unique_dims.forEach(function(unique_dim) {
      var sb_rows_dim = sb_rows.filter(function(d) {
        return bar.arraysEqual(
            d.slice(bar.dim_start_idx() + 1, bar.dim_end_idx() + 1),
            unique_dim);
      });

      bar.draw_svg(sb_colnames, sb_rows, sb_rows_dim, width, height,
          margin, d_scale, a_scale,
          unique_dim, dim_values, stack_values,
          dname_to_idx, didx_to_d,
          dim_colindex, dim_is_date, dim_sz);
    });
  };

  bar.draw_svg = function(sb_colnames, sb_rows, sb_rows_dim, width, height,
      margin, d_scale, a_scale,
      unique_dim, dim_values, stack_values,
      dname_to_idx, didx_to_d,
      dim_colindex, dim_is_date, dim_sz) {

    var a_fmt = d3.format(","),
        chart_div = this.chart_div(),
        hdr = [],
        sb_coltypes = bar.sb_coltypes();

    unique_dim.forEach(function(d, i) {
      hdr.push(sb_colnames[i + bar.dim_start_idx() + 2]);
      hdr.push(":");
      hdr.push("<b style='margin: 0px 30px 0px 5px;'>" + d + "</b>");
    });
    if (hdr.length) {
      d3.select("#" + chart_div).append("div")
                                .classed("viz-bar-title", true)
                                .html(hdr.join(''));
    }

    var agg_startindex = bar.agg_start_idx(),
        agg_endindex = bar.agg_end_idx(),
        dim_names_set = {},
        dim_names = [],
        agg_by_dimval = {};

    var svg = d3.select("#" + chart_div)
                .append("svg")
                .classed("viz-bar-svg", true)
                .style("width", width + margin.left + margin.right + "px")
                .style("height", height + margin.top + margin.bottom + "px")
                  .append("g")
                  .classed("viz-bar-g", true)
                  .attr("transform",
                    "translate(" + margin.left + "," + margin.top + ")");

    var color = d3.scale.ordinal()
                  .domain(d3.range(agg_startindex, agg_endindex+1))
                  .range(bar.colors())
                  .domain(sb_colnames.slice(agg_startindex, agg_endindex+1));

    var bar_data = [];
    sb_rows.forEach(function(d) {
      for (var a = agg_startindex; a <= agg_endindex; ++a) {
        bar_data.push({
          'data': d,
          'agg_idx': a,
          'val': d[a]
        });
      }
    });
    bar_data = bar_data.sort(function(a, b) {
      return b.data[b.agg_idx] - a.data[a.agg_idx];
    });
    var rect;
    if (bar.is_horizontal()) {
      rect = svg.selectAll(".nosel")
         .data(bar_data)
           .enter()
           .append("rect")
           .attr("x", function(d, i) {
             return 0;
           })
           .attr("y", function(d, i) {
             return d_scale(d.data[dim_colindex]);
           })
           .attr("width", function(d) {
             return a_scale(d.data[d.agg_idx]);
           })
           .attr("height", dim_sz)
           .style("fill", function(d) {
             return color(sb_colnames[d.agg_idx]);
           });
    } else {
      rect = svg.selectAll(".nosel")
         .data(bar_data)
           .enter()
           .append("rect")
           .attr("x", function(d, i) {
             return d_scale(d.data[dim_colindex]);
           })
           .attr("y", function(d) {
             return a_scale(d.data[d.agg_idx]);
           })
           .attr("width", dim_sz)
           .attr("height", function(d) {
             return height - a_scale(d.data[d.agg_idx]);
           })
           .style("fill", function(d) {
             return color(sb_colnames[d.agg_idx]);
           });
    }

    var text_fn = function(wrapped_d) {
      var d = wrapped_d.data;
      var textl = [];
      for (var dim_idx = bar.dim_start_idx(); dim_idx <= bar.dim_end_idx();
          dim_idx++) {
        if (dim_idx !== bar.dim_start_idx()) {
          textl.push("<br>");
        }
        bar.addTooltip(textl, sb_colnames[dim_idx], bar.d_string(d[dim_idx],
            sb_coltypes[dim_idx]), bar.formatter(dim_idx));
      }
      textl.push("<br>");
      bar.addTooltip(textl, sb_colnames[wrapped_d.agg_idx],
          d[wrapped_d.agg_idx],
          bar.formatter(wrapped_d.agg_idx));
      for (var agg_idx = bar.agg_start_idx(); agg_idx <= bar.agg_end_idx();
          agg_idx++) {
        if (agg_idx === wrapped_d.agg_idx) {
          continue;
        }
      }
      return textl.join("");
    };

    arc.tooltip()
       .dom_id("#tooltip")
       .text_fn(text_fn)
       .addSelection(rect)
       .activate();

    bar.bar_draw_axes(svg, width, height, sb_rows_dim, d_scale, a_scale, dim_sz,
        dim_is_date, dim_values, color, margin);
  };

  bar.bar_draw_axes = function(svg, width, height, sb_rows_dim, d_scale, a_scale,
      dim_sz, dim_is_date, dim_values, color, margin) {

    var d_axis = d3.svg.axis()
                   .scale(d_scale);
    if (bar.is_horizontal()) {
      d_axis.orient("left");
    } else {
      d_axis.orient("bottom");
    }
    if (dim_is_date) {
      d_axis.tickFormat(d3.time.format("%Y-%m-%d"));
      if (dim_values.length < 20) {
        d_axis.ticks(2);
      } else if (dim_values.length < 50) {
        d_axis.ticks(5);
      }
    }
    // Do not create more ticks marks than dimension elements
    if (d_axis.ticks() > sb_rows_dim.length) {
      d_axis.ticks(sb_rows_dim.length);
    }
    var a_axis = d3.svg.axis()
                   .scale(a_scale)
                   .tickFormat(bar.tickFormatDefaultNumeric),
        chartSVG;
    if (bar.is_horizontal()) {
      a_axis.orient("bottom");
      svg.append("g")
         .attr("class", "y arc-viz-axis")
         .call(d_axis);

      chartSVG = d3.select(svg.node().parentNode)
                   .append("svg")
                   .attr('height', height + margin.bottom + margin.top - 20)
                   .append("g")
                   .attr("class", "x arc-viz-axis")
                   .attr("transform", "translate( " + margin.left + "," + (margin.top + height) + ")")
                   .call(a_axis);
    } else {
      a_axis.orient("left");
      chartSVG = d3.select(svg.node().parentNode)
                   .append("svg")
                   .attr('height', height + margin.bottom + margin.top - 20)
                   .append("g")
                   .attr("class", "x arc-viz-axis")
                   .attr("transform", "translate( " + margin.left + "," + (margin.top + height) + ")")
                   .call(d_axis);

      svg.append("g")
          .attr("class", "y arc-viz-axis")
          .call(a_axis);
    }

    if (!bar.is_horizontal() && !dim_is_date && !bar.idx_is_numeric([dim_values], 0)) {
      chartSVG.selectAll(".x .tick text")
         .style("text-anchor","end")
         .attr("transform", function(d, i) {
           return "translate(-10, 0) rotate(" + bar.x_slant() + ",0,0)";
         });
    }

    bar.draw_labels(svg, bar.is_horizontal(), width, height + margin.bottom - 10);
    bar.draw_legend({
      svg: svg,
      width: width,
      height: height,
      color: color,
      sort_domain: true
    });

    return bar;
  };

  return bar;
};

}());
