window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

function findPlugin(actionName) {
  let actionExtension;

  arc.action.extensions.some(function(a) {
    if (a.enabled && !a.enabled()) {
      return;
    }

    if (a.action_name() === actionName) {
      actionExtension = a;
      return true;
    }
  });

  return actionExtension ? actionExtension.action() : undefined;
}

arc.visual.plugins.action = function() {
  let saved_settings = {},
      defaults = {
        actionType: 'button',
        actionURL: '/arc/static/img/actions/example_image.png',
        actionLabel: 'Click me!',
        buttonType: 'primary',
        buttonSize: 'normal',
        actionPlugin: 'openURL',
        actionSettings: {},
      },
      keys = Object.keys(defaults),
      data = $.extend({}, defaults),
      lib;

  // Fetch webhook-related static data
  arc.action.webhookFetch();

  const uppercase_button = arc.visual.setting()
                              .id('uppercase_button')
                              .tab('Marks')
                              .type(arc.visual.settingTypeEnum.CHECKBOX)
                              .add_name('UPPERCASE_BUTTON')
                              .add_label('Make button uppercase')
                              .value('true');

  const user_settings = arc.visual.defaultSettings({
                                    noAxes: true,
                                    noTrellis: true,
                                    noTooltip: true,
                                  })
                                  .addSetting(uppercase_button)
                                  .removeSetting([
                                    'visual_show_detail_data',
                                  ]);

  const action = arc.visual.api();

  action.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data([])
       .user_settings(user_settings);
    return action;
  };

  action.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.viz_type = settings.viz_type || {};
    settings.viz_type.action = settings.viz_type.action || {};

    keys.forEach(function(k) {
      settings.viz_type.action[k] = data[k];
    });

    saved_settings = settings;
    return settings;
  };

  action.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return action;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if (settings && settings.viz_type && settings.viz_type.action) {
      data = $.extend(data, settings.viz_type.action);
    }

    return action;
  };

  function repaintActionSettings() {
    if (!shelvesDiv) {
      return;
    }

    const $action = $(`#${shelvesDiv} .viz-actionbuilder .actionPlugin`);
    const actionName = $action.val();

    const $settings = $(`#${shelvesDiv} .viz-actionbuilder .action-settings`);
    if ($settings.attr('data-action') !== actionName) {
      $settings.attr('data-action', actionName);
      $settings.empty();

      const a = findPlugin(data.actionPlugin);
      if (a && a.settings) {
        const actionSpec = a.settings();

        actionSpec.forEach(function(s) {
          s.value = s.defaultValue;
        });

        let actionSettings = arc.css.settings.specificationToSettings(actionSpec);

        actionSettings.settings().forEach(function(s) {
          if (data.actionSettings[s.id()] === undefined) {
            data.actionSettings[s.id()] = s.value();
          } else {
            s.value(data.actionSettings[s.id()]);
          }

          s.paintControl(d3.selectAll($settings.toArray()));
          s.changeCb('action', function() {
            s.process_value();
            data.actionSettings[s.id()] = s.value();
            lib.page_needs_save();
          });
        });
      }
    }
  }

  function associate(shelvesDiv, name) {
    const s = `#${shelvesDiv} .viz-actionbuilder .${name}`;
    const v = data[name];

    $(s).val(v)
        .offOn('change.action', function() {
          data[name] = $(this).val();
          if (name === 'actionType') {
            drawTypeSpecificDesigner();
          }

          if (name === 'actionPlugin') {
            data.actionSettings = {};
            repaintActionSettings();
          }

          lib.redraw();
          lib.page_needs_save();
        });
  }

  function dataToClass() {
    let c = [];

    c.push('btn-' + data.buttonType);

    const sz = {
      normal: '',
      large: 'btn-lg',
      small: 'btn-sm',
    };

    c.push(sz[data.buttonSize]);

    return c.join(' ');
  }

  let shelvesDiv;

  action.viz_drawDesigner = function(_shelvesDiv) {
    shelvesDiv = _shelvesDiv;
    if (!shelvesDiv) {
      return;
    }

    const drawId = lib.getDrawId(shelvesDiv);

    const editMode = lib.build_mode();
    if (editMode) {
      /*
       * We have to fetch some assets from the backend before
       * we are able to create a JIRA action.
       *
       * Go do it now, and draw when it's done
       */
      arc.action.jiraFetch(function() {
        // See if another drawDesigner call has happened in between
        if (drawId !== lib.getDrawId(shelvesDiv)) {
          return;
        }

        drawDesigner();
      });

      return action;
    }

    return action;
  };

  function drawTypeSpecificDesigner() {
    if (!shelvesDiv) {
      return;
    }

    const $shelves = $('#' + shelvesDiv);
    if (data.actionType === 'button') {
      $shelves.find('.button-options')
              .removeClass('arc-invisible');
      $shelves.find('.image-options')
              .addClass('arc-invisible');
    } else {
      $shelves.find('.button-options')
              .addClass('arc-invisible');
      $shelves.find('.image-options')
              .removeClass('arc-invisible');
    }
  }

  function drawDesigner() {
    if (!shelvesDiv) {
      return;
    }

    $('#' + shelvesDiv).html($('#viz-actionbuilder').html());

    drawTypeSpecificDesigner();

    const $s = $(`#${shelvesDiv} .viz-actionbuilder .actionPlugin`);
    arc.action.extensions.forEach(function(e) {
      if (e.enabled && !e.enabled()) {
        return;
      }

      $s.append(arc.misc.safe_substitute(
          $('#simple-select-option select').html(),
          {
            VALUE: e.action_name(),
            LABEL: e.action_label(),
          }));
    });

    keys.forEach(function(k) {
      associate(shelvesDiv, k);
    });

    repaintActionSettings();

    return action;
  }

  action.viz_getDataRequest = function() {
    var ignore = arc.data.request()
                    .request_type('ignore');
    return ignore;
  };

  action.viz_setDataResult = function(result) {
    return action;
  };

  action.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();
    lib.removeMessageAboutMissingData($(`#${viz_div}`).closest('.arc-viz-container'));

    if (data.actionType === 'image') {
      drawImage(viz_div);
      return action;
    }

    drawButton(viz_div);
    return action;
  };

  action.viz_destroy = function () {
    const chartDiv = d3.select(`#${action.viz_lib().chart_div()}`);
    chartDiv.selectAll('a').remove();
  };

  function runAction() {
    const a = findPlugin(data.actionPlugin);
    if (!a) {
      arc.report.alertMessageSmall('This action is disabled on the system. ' +
          'Please contact your system administrator.',
          'alert-danger');
      return;
    }

    const p = {};
    $.each(data.actionSettings, function(k, v) {
      p[k] = lib.substituteParams(v);
    });

    a.viz(action)
     .run(p);
  }

  function drawImage(viz_div) {
    const $viz = $(`#${viz_div}`);
    $viz.html(
            arc.misc.safe_substitute(
                $('#viz-action-image').html(), {}));

    $viz.find('img.action-image')
        .attr('src', data.actionURL);

    const $a = $viz.find('a');
    $a.off('click.action')
      .on('click.action', runAction);
  }

  function drawButton(viz_div) {
    const editMode = lib.build_mode();
    const editInModal = lib.inModal() && lib.build_mode();

    const $viz = $(`#${viz_div}`);
    $viz.html(
            arc.misc.safe_substitute(
                $('#viz-action-button').html(),
                {
                  BUTTON_CLASS: dataToClass(),
                  LABEL: data.actionLabel,
                }));

    const $button = $viz.find('button');

    if (uppercase_button.value() === 'false') {
      $button.css('text-transform', 'none');
    }

    if (!editInModal) {
      $button.off('click.action')
             .on('click.action', runAction);
    }

    if (editInModal) {
      $viz.find('button span')
          .attr('contenteditable', 'true')
          .on('keyup', function() {
            data.actionLabel = $(this).text();
            drawDesigner();
          });
    }

    return action;
  }

  return action;
};

arc.visual.plugins.action_extension = function() {
  var ext = function() { return 'arc.visual.plugins.action_extension'; };

  ext.viz_name = function() {
    return 'action';
  };

  ext.viz_tooltip = function() {
    return 'action';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_action.png?v=1';
  };

  ext.viz = arc.visual.plugins.action;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.action_extension());

}());
