window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.testutil = arc.testutil || {};

arc.visual.defaultSettings = function(config) {
  config = config || {};
  const defaultSettings = arc.visual.settings();
  const { isFilter } = config;
  const includeTrellis = !config.noTrellis;
  const includeAxes = !config.noAxes;
  const includeTooltip = !config.noTooltip;

  if (arc.testutil && arc.testutil.derived_data) {
    return defaultSettings;
  }

  const c1 = arc.css.settings.containerCssClass(isFilter);
  const c2 = arc.css.settings.cssStyles();
  const c3 = arc.css.settings.inlineCss();
  const c4 = arc.report.user_settings.permit_csv_download_visual();
  const c5 = arc.report.user_settings.csv_download_locale();
  const c6 = arc.report.user_settings.permit_png_download();
  const c8 = arc.report.user_settings.permit_email();
  const quotationMarkSetting = arc.report.user_settings.csvQuotationMark();

  if (isFilter) {
    // Filters are always explicit scope
    defaultSettings.addSetting([c1, c2, c3]);
  } else {
    const c7 = arc.report.user_settings.explicit_scope();
    defaultSettings.addSetting([
        c1,
        c2,
        c3,
        c4,
        quotationMarkSetting,
        c5,
        c6,
        c7,
        c8,
        arc.settings.params.sendAggregateParameters(),
        arc.settings.params.visualDatasetScope(),
        arc.report.user_settings.nulls_order(),
    ]);

    if (includeTrellis) {
      defaultSettings.addSetting([
        arc.settings.trellis.borders(),
        arc.settings.trellis.minimumWidth(),
        arc.settings.trellis.minimumHeight(),
      ]);
    }
  }

  // Hide png/pdf download (also email) setting in dashboard -- inherit from dashboard itself
  if (arc.globals.is_dashboard) {
    c6.hidden(true);
    c8.hidden(true);
  }

  if (!arc.globals.enable_custom_styles) {
    c1.hidden(true);
    c2.hidden(true);
    c3.hidden(true);
  }

  if (!isFilter) {
    defaultSettings.addSetting([
      arc.settings.fonts.visualFont(),
      arc.settings.fonts.visualFontSize(),
      arc.settings.colors.visualBgColor(),

      arc.settings.fonts.titleFont(),
      arc.settings.fonts.titleFontSize(),
      arc.settings.fonts.vizTitleFontWeight(),
      arc.settings.colors.visualTitleFg(),
      arc.settings.colors.visualTitleBg(),
      arc.settings.fonts.titleFontAlignment(),

      arc.settings.fonts.subtitleFont(),
      arc.settings.fonts.subtitleFontSize(),
      arc.settings.colors.visualSubtitleFg(),
      arc.settings.colors.visualSubtitleBg(),
      arc.settings.fonts.subtitleFontAlignment(),

      includeAxes && arc.settings.fonts.visualAxisLabelFontSize(),
      includeAxes && arc.settings.fonts.visualAxisTicksFontSize(),
      includeAxes && arc.settings.fonts.visualLabelFontSize(),
      includeAxes && arc.settings.colors.labelColor(),
      includeTrellis && arc.settings.fonts.visualTrellisFontSize(),
      includeAxes && arc.settings.fonts.visualLegendFontSize(),
      arc.settings.visuals.visualHideTitle(),
      arc.settings.visuals.visualShowDetailData(),
      arc.settings.data.downloadRows(),
      arc.settings.data.maxrows(),
    ]);

    if (includeTooltip) {
      const displayTooltips = arc.report.user_settings.display_tooltips();
      const tooltipFont = arc.settings.fonts.tooltipFont();
      const tooltipSize = arc.settings.fonts.tooltipFontSize();

      defaultSettings.addSetting([
        displayTooltips,
        tooltipFont,
        tooltipSize,
      ]);
    }
  }

  if (!isFilter) {
    defaultSettings.addSetting([
      arc.settings.data.disableIncrementalQueries(),
    ]);
  }

  defaultSettings.addSetting([
    arc.settings.data.ksqlStreamPosition(),
  ]);

  return defaultSettings;
};

// Dashboard settings
arc.visual.dashboardSettings = function() {
  const defaultSettings = arc.visual.settings();

  const c1 = arc.css.settings.containerCssClass()
              .labels(['CSS classes for this dashboard'])
              .info('Separate multiple classes using a space.' +
                ' These classes add to the existing CSS classes of the ' +
                "dashboard's container.");

  const c2 = arc.css.settings.cssStyles();
  const c3 = arc.css.settings.inlineCss();

  if (!arc.globals.enable_custom_styles) {
    c1.hidden(true);
    c2.hidden(true);
    c3.hidden(true);
  }

  const tsz = arc.settings.fonts.titleFontSize();
  const ssz = arc.settings.fonts.subtitleFontSize();

  tsz.labels()[0] = 'Font Size';
  ssz.labels()[0] = 'Font Size';

  const bordersAroundVisuals = arc.settings.dashboards.bordersAroundVisuals();
  const visualBorderColor = arc.settings.colors.visualBorderColor();
  const visualBorderRadius = arc.settings.dashboards.visualBorderRadius();
  bordersAroundVisuals.addDependents([visualBorderColor, visualBorderRadius]);

  defaultSettings.addSetting([
    c1,
    c2,
    c3,
    arc.settings.dashboards.width(),
    arc.settings.fonts.overrideVisualFontPreferences(),
    arc.settings.colors.dashboardBgColor(),
    arc.settings.colors.dashboardFgColor(),
    arc.settings.colors.pageBgColor(),

    arc.settings.params.displayContextMenu(),
    arc.settings.dashboards.displayViewData(),
    arc.settings.dashboards.detailedDataMaxColumnLimit(),
    arc.settings.dashboards.manualVisualRefresh(),
    arc.settings.dashboards.autoRefreshSeconds(),
    arc.settings.dashboards.enableAnimations(),
    arc.settings.dashboards.horizontalMarginBetweenVisuals(),
    arc.settings.dashboards.verticalMarginBetweenVisuals(),
    arc.settings.dashboards.singleVisualDashboard(),

    arc.settings.params.displayFilters(),

    arc.settings.fonts.dashboardTitleFont(),
    arc.settings.fonts.dashboardTitleFontSize(),
    arc.settings.dashboards.dashboardTitleFontWeight(),
    arc.settings.colors.appTitleFg(),
    arc.settings.colors.appTitleBg(),
    arc.settings.fonts.dashboardTitleFontAlignment(),

    arc.settings.fonts.dashboardSubtitleFont(),
    arc.settings.fonts.dashboardSubtitleFontSize(),
    arc.settings.colors.dashboardSubtitleFgColor(),
    arc.settings.colors.dashboardSubtitleBgColor(),
    arc.settings.fonts.dashboardSubtitleFontAlignment(),

    arc.settings.dashboards.filtersOnLeft(),
    arc.settings.dashboards.filterInlineLabels(),
    arc.settings.dashboards.filterLabelLine(),
    arc.settings.dashboards.bordersAroundFilters(),
    arc.settings.fonts.filterFont(),
    arc.settings.fonts.filterFontSize(),
    arc.settings.colors.filterTitleColor(),
    arc.settings.colors.filterSelectionColor(),
    arc.settings.colors.filterBgColor(),
    arc.settings.colors.filterContainerBgColor(),

    arc.settings.fonts.visualFont()
                      .group('Font Family'),

    bordersAroundVisuals,
    visualBorderColor,
    visualBorderRadius,

    arc.settings.fonts.titleFont()
                      .tab('Visual Title')
                      .group('Font Family'),

    arc.settings.fonts.subtitleFont()
                      .tab('Visual Subtitle')
                      .group('Font Family'),

    arc.settings.fonts.visualFontSize(),
    arc.settings.fonts.axisLabelFontSize(),
    arc.settings.fonts.axisTicksFontSize(),
    arc.settings.fonts.labelFontSize(),
    arc.settings.colors.labelColor(),
    arc.settings.fonts.trellisFontSize(),
    arc.settings.fonts.legendFontSize(),

    arc.settings.colors.visualBgColor(),

    tsz,
    ssz,

    arc.settings.fonts.vizTitleFontWeight(),
    arc.settings.fonts.tooltipFont(),
    arc.settings.fonts.tooltipFontSize(),

    arc.settings.colors.visualTitleFg(),
    arc.settings.colors.visualTitleBg(),

    arc.settings.fonts.titleFontAlignment()
                      .tab('Visual Title')
                      .group('Alignment'),

    arc.settings.colors.visualSubtitleFg(),
    arc.settings.colors.visualSubtitleBg(),
    arc.settings.fonts.subtitleFontAlignment()
                      .tab('Visual Subtitle')
                      .group('Alignment'),

    arc.settings.dashboards.dashboardHideTitle(),
  ]);

  return defaultSettings;
};

// App settings (formerly appgroup)
arc.visual.appgroupSettings = function() {
  const defaultSettings = arc.visual.settings();

  const c1 = arc.css.settings.containerCssClass()
              .labels(['CSS classes for this app'])
              .info('When using multiple classes, separate them by spaces.' +
                ' These classes add to the existing CSS classes of the ' +
                "app's container.");

  const c2 = arc.css.settings.cssStyles();
  const c3 = arc.css.settings.inlineCss();

  defaultSettings.addSetting([c1, c2, c3]);

  if (!arc.globals.enable_custom_styles) {
    c1.hidden(true);
    c2.hidden(true);
    c3.hidden(true);
  }

  defaultSettings.addSetting([
      arc.report.user_settings.menu_on_left(),
      arc.report.user_settings.navigate_to_apps_page(),
      arc.settings.colors.appMenuBarBgColor(),
      arc.settings.colors.appMenuBarBottomBarColor(),
      arc.settings.colors.appMenuBarBarColorSelected(),
      arc.settings.fonts.appMenuTextFont(),
      arc.settings.fonts.appMenuTextFontSize(),
      arc.settings.fonts.appMenuTextFontWeight(),
      arc.settings.colors.appMenuTextColor(),
      arc.settings.colors.appMenuTextColorSelected(),
      arc.settings.apps.appMenuTextUppercase(),
      arc.settings.apps.appLogoImage(),
      arc.settings.apps.appLogoWidth(),
      arc.settings.apps.appLogoHeight(),
      arc.settings.apps.appLogoMarginTop(),
      arc.settings.apps.appLogoMarginLeft(),
      arc.settings.colors.appBgBgColor(),
      arc.settings.apps.appHideDropShadow(),
      arc.settings.apps.appShowTitle(),
      ]);

  if (arc.globals.arcviz_settings.ENABLE_URL_NAMES) {
    defaultSettings.addSetting(arc.report.user_settings.app_url_name());
  }

  return defaultSettings;
};

arc.visual.settings = function() {
  const settings = function() { return 'arc.visual.settings'; };

  let visualSettingsArray = [];
  let setting_by_id = {};
  const settings_by_tabs = {};

  settings.getSortedSettingsByDependency = function() {
    const tempVisualSettingsArray = [...visualSettingsArray];
    for (let i = 0; i < tempVisualSettingsArray.length; i++) {
      const setting = tempVisualSettingsArray[i];
      const dependents = setting.getDependents();
      if (dependents && dependents.length > 0) {
        const indexes = [];
        dependents.forEach((dependent) => {
          indexes.push(tempVisualSettingsArray.indexOf(dependent));
        });
        const indexofLastDependentInRow = Math.max(indexes);
        const lastDependent = tempVisualSettingsArray[indexofLastDependentInRow];
        if (lastDependent !== -1 && i < indexofLastDependentInRow) {
          tempVisualSettingsArray.splice(indexofLastDependentInRow, 1, setting);
          tempVisualSettingsArray.splice(i, 1, lastDependent);
        }
      }
    }

    return tempVisualSettingsArray;
  };

  settings.settings = function() {
    return visualSettingsArray;
  };

  settings.checkboxSet = function(setting_id) {
    const visualSetting = settings.find_by_id(setting_id);
    if (!visualSetting) {
      return false;
    }

    return visualSetting.isChecked;
  };

  settings.cleanupSettings = function() {
    visualSettingsArray.forEach((user_setting) => {
      user_setting.cleanupSetting();
    });
  };

  settings.loadFromValues = function(saved_settings) {
    visualSettingsArray.forEach((user_setting) => {
      const val = saved_settings[user_setting.id()];
      if (val !== undefined) {
        user_setting.value(val);
      }
    });
  };

  /*
   * Go through each setting, and fill in settings id/values into the
   * 'out' hash. Return 'true' if any of the values have been changed by the
   * user.
   */
  settings.fillValues = function(out) {
    let needs_save = false;
    visualSettingsArray.forEach((user_setting) => {
      const setting_needs_save = user_setting.process_value();
      if (setting_needs_save) {
        needs_save = true;
      }

      out[user_setting.id()] = user_setting.value();
    });
    return needs_save;
  };

  settings.organizeByTabs = function(tabs, tabs_h, tab_settings) {
    const distinctTabs = [];

    // First, find the distinct tabs that are requested, and give
    // each of them a unique id
    visualSettingsArray.forEach((user_setting) => {
      if (user_setting.tab() in tabs_h) {
        tab_settings[user_setting.tab()].push(user_setting);
        return;
      }

      tabs_h[user_setting.tab()] = arc.report.settings_tab_unique_id;
      arc.report.settings_tab_unique_id += 1;
      tab_settings[user_setting.tab()] = [user_setting];
      distinctTabs.push(user_setting.tab());
    });

    if (distinctTabs.indexOf('General') >= 0) {
      tabs.push('General');
    }

    if (distinctTabs.indexOf('Color') >= 0) {
      tabs.push('Color');
    }

    if (distinctTabs.indexOf('Table Style') >= 0) {
      tabs.push('Table Style');
    }

    if (distinctTabs.indexOf('Table Colors') >= 0) {
      tabs.push('Table Colors');
    }

    if (distinctTabs.indexOf('Values') >= 0) {
      tabs.push('Values');
    }

    distinctTabs.forEach((tab) => {
      if (tab === 'Color') {
        return;
      }

      if (tab === 'Table Style') {
        return;
      }

      if (tab === 'Table Colors') {
        return;
      }

      if (tab === 'Values') {
        return;
      }

      if (tab === 'General') {
        return;
      }

      if (tab === 'Custom Style') {
        return;
      }

      tabs.push(tab);
    });

    if (distinctTabs.indexOf('Custom Style') >= 0) {
      tabs.push('Custom Style');
    }

    return settings;
  };

  settings.addSetting = function(_) {
    if (Array.isArray(_)) {
      _.forEach(_addSetting);
      return settings;
    }

    _addSetting(_);
    return settings;

    function _addSetting(setting) {
      // Some cases, item in array is specifically undefined or false (eg: when includeAxes)
      if (!setting) {
        return;
      }

      const key = setting.id();
      if (typeof (setting_by_id[key]) === 'undefined') {
        setting_by_id[key] = visualSettingsArray.length;
        visualSettingsArray.push(setting);
      }
    }
  };

  // Used to remove setting from bar version of user_settings in histogram
  settings.removeSetting = function(setting) {
    if (!Array.isArray(setting)) {
      setting = [setting];
    }

    _removeSettings(setting);
    return settings;

    function _removeSettings(settingArray) {
      const toRemove = new Set(settingArray.map((o) => {
        if (o?.id) {
          return o.id();
        }

        return o;
      }));
      toRemove.forEach((settingId) => delete setting_by_id[settingId]);
      const newVisualSettingsArray = [];
      const newIndex = {};
      visualSettingsArray.forEach((visualSetting) => {
        const id = visualSetting.id();
        if (!toRemove.has(id)) {
          newIndex[id] = newVisualSettingsArray.length;
          newVisualSettingsArray.push(visualSetting);
        }
      });
      setting_by_id = newIndex;
      visualSettingsArray = newVisualSettingsArray;
    }
  };

  settings.find_by_id = function(id) {
    let found_setting;
    const setting_idx = setting_by_id[id];
    if (typeof (setting_idx) !== 'undefined') {
      return visualSettingsArray[setting_idx];
    }

    visualSettingsArray.some((setting) => {
      if (setting.id() === id) {
        found_setting = setting;
        return true;
      }

      return false;
    });
    return found_setting;
  };

  settings.filterBySection = function() {
    visualSettingsArray.forEach((setting) => {
        if (!settings_by_tabs[setting.section()]) {
          settings_by_tabs[setting.section()] = [];
        }

        settings_by_tabs[setting.section()].push(setting);
       });
  };

  settings.removeChangeCbs = function() {
    visualSettingsArray.forEach((s) => {
      s.removeChangeCbs();
    });

    return settings;
  };

  settings.addChangeCb = function(cbname, cb) {
    visualSettingsArray.forEach((s) => {
      s.addChangeCb(cbname, cb);
    });

    return settings;
  };

  settings.paint = function(settingsHolders, config) {
    arc.counters.increment('settingsPaint');

    if (config === undefined) {
      config = {
        tabsToOpen: new Set(['General', 'Dashboard Basics']),
      };
    }

    const {
      tabsToOpen,
    } = config;

    const d3SelH = {};

    $.each(settingsHolders, (section, $div) => {
      d3SelH[section] = ($div === null) ? null : d3.selectAll($div.toArray());
    });

    const tabSectionTemplate = $('#settings-tab-section-template').html();
    const tabSeparatorTemplate = $('#settings-tab-seperator-template').html();

    const tabs = [];
    const tabs_h = {};
    const tab_settings = {};
    settings.organizeByTabs(tabs, tabs_h, tab_settings);

    const firstTab = {};

    // Now, draw each tab
    tabs.forEach((tab) => {
      // See if all settings within this tab are hidden
      let allHidden = true;
      let d3_sel;
      let $div;
      let section;

      tab_settings[tab].forEach((setting) => {
        if (!setting.hidden()) {
          allHidden = false;
        }

        if (section === undefined) {
          section = setting.section();
          $div = settingsHolders[section];
          if ($div === undefined) {
            section = 'default';
            $div = settingsHolders[section];
            d3_sel = d3SelH[section];
          } else {
            d3_sel = d3SelH[section];
          }
        }
      });

      if ($div === null) {
        return;
      }

      let settings_paint_sel = d3_sel;

      // If not all hidden, display the tab
      if (firstTab[section] !== undefined && !allHidden) {
        $div.append(arc.misc.safe_substitute(tabSeparatorTemplate, {}));
      }

      firstTab[section] = true;

      // For visual goodness, take out some tab titles from display
      // CA: putting back in and will hide with css
      let tabTitle = tab;
      if (tabTitle === 'Color') {
        tabTitle = 'Colors';
      }

      $div.append(arc.misc.safe_substitute(tabSectionTemplate, {
        TAB_CLASS: (allHidden ? ' arc-invisible' : '') + ((tabsToOpen && tabsToOpen.has(tabTitle)) ? ' open' : ''),
        TAB_TITLE: tabTitle,
      }));

      settings_paint_sel = d3.selectAll(
          $div.find('.settings-tab-settings-holder:last').toArray()
);

      tab_settings[tab].forEach((setting) => {
        setting.paintControl(settings_paint_sel);
      });
    });

    hideUnusedTabs(tab_settings);
    return settings;
  };

  settings.handlePopovers = function($container) {
    $container
        .on('click', '.info-icon', (e) => {
          e.preventDefault();
        })
        .find('[data-toggle="popover"]')
        .popover({
          container: 'body',
          placement: 'top',
          trigger: 'hover'
        });
  };

  function getSidebarLi(optionName) {
    return $(`#${optionName}`);
  }

  function currentlySelectedSidebarTab() {
    return $('.user-control.active').attr('id');
  }

  function selectSidebarTab(optionName) {
    getSidebarLi(optionName).children().first().trigger('click');
  }

  function showSidebarTab(optionName) {
    getSidebarLi(optionName).removeClass('arc-invisible');
  }

  function hideSidebarTab(optionName) {
    // Switches selection to 'data' if removed tab is currently selected.
    // Case where 'data' is selected for removal is not handled
    if (currentlySelectedSidebarTab() === optionName) {
      selectSidebarTab('data');
    }

    getSidebarLi(optionName).addClass('arc-invisible');
  }

  function hideUnusedTabs(tabSettings) {
    function _get(obj, key, defaultValue) {
      return obj[key] || defaultValue;
    }

    const all_tabs = {};

    all_tabs.settings = _get(tabSettings, 'General', [])
                        .concat(_get(tabSettings, 'Parameters', []))
                        .concat(_get(tabSettings, 'Data', []))
                        .concat(_get(tabSettings, 'Marks', []))
                        .concat(_get(tabSettings, 'Axes', []));

    all_tabs.colors = _get(tabSettings, 'Color', []);

    all_tabs.style = _get(tabSettings, 'Visual Subtitle', [])
                        .concat(_get(tabSettings, 'Visual Basics', []))
                        .concat(_get(tabSettings, 'Visual Title', []));

    all_tabs.custom_style = arc.globals.enable_custom_styles ? [true] : [];

    Object.keys(all_tabs).forEach((tab) => {
      if (all_tabs[tab].length === 0) {
        hideSidebarTab(tab);
      } else {
        showSidebarTab(tab);
      }
    });
  }

  settings.resetDashboardStyleValues = () => {
    if (!settings_by_tabs.Style?.length) {
      settings.filterBySection();
    }

    settings_by_tabs.Style.forEach((setting) => {
      setting.resetValue();
    });
  };

  return settings;
};

arc.visual.settingTypeEnum = {
  CHECKBOX: 1,
  RADIO: 2,
  INTEGER: 3,
  STRING: 4,
  REAL: 5,
  INTEGER_RANGE: 6,
  CUSTOM: 7,
  PASSWORD: 8,
  DROPDOWN: 9,
  COLOR: 10,
  INTEGER_NOSPINNER: 11,
  SLIDER: 12,
  SLIDER_RANGE: 13,
  DISPLAY_ONLY: 14,
  DROPDOWN_CHOSEN: 15,
  EXPRESSION: 16,
};

arc.visual.setting_unique_id = 1;

arc.visual.setting = function() {
  const setting = function() { return 'arc.visual.setting'; };

  let tab;
  let section = 'default';
  let changeCbs = {};
  let id;
  let group;
  let inline = true;
  let type;
  let value;
  let defaultValue;
  let valueCb;
  let valuePreprocessCb;
  let validate_fn;
  let names = [];
  let labels = [];
  let info;
  let level;
  let infoblock;
  let classes = [];
  let placeholder = '';
  const unique_id = arc.visual.setting_unique_id;
  arc.visual.setting_unique_id += 1;
  let div_id = `settings-user-control-${unique_id}`;
  let paint_fn;
  let repaint_fn;
  const postpaint_fn = [];
  let dependentSettings = {};
  let fetch_fn;
  let shown_fn;
  let min;
  let max;
  let step;
  let is_textarea = false;
  let is_password = false;
  let is_flipper = false;
  let hidden = false;
  let disabled = false;
  let mutable = true;
  let valueBeforeRestore;
  let useAsDefault;
  let suffix = '';
  let disableTooltipText;
  const rangeTypes = [arc.visual.settingTypeEnum.INTEGER_RANGE,
    arc.visual.settingTypeEnum.SLIDER_RANGE];
  let viz_lib;
  let disable_fn;
  let enable_fn;

  setting.cleanupSetting = function() {
    // Cleanup any state. Overridden by subclasses
  };

  setting.get_range = function() {
    if (!rangeTypes.includes(type)) {
      return undefined;
    }

    const e = value.split('-');
    return [Number(e[0]), Number(e[1])];
  };

  setting.validate_num_range = function(val, minimum, maximum) {
    if (!rangeTypes.includes(type)) {
      // If the value is not a range, just validate that the value
      // is a number, and between minimum and maximum (inclusive)
      if (Number.isNaN(val)) {
        return false;
      }

      const n = Number(val);
      if ((n < minimum) || (n > maximum)) {
        return false;
      }

      return true;
    }

    // If the value is a range, check that the minimum range is >= minimum and the
    // maximum range is <= maximum
    const e = val.split('-');
    if (e.length !== 2) {
      return false;
    }

    if (Number.isNaN(e[0]) || Number.isNaN(e[1])) {
      return false;
    }

    if (Number(e[0]) < minimum) {
      return false;
    }

    if (Number(e[1]) > maximum) {
      return false;
    }

    if (minimum > maximum) {
      return false;
    }

    return true;
  };

  setting.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return setting;
  };

  setting.min = function(_) {
    if (!arguments.length) return min;
    min = _;
    return setting;
  };

  setting.max = function(_) {
    if (!arguments.length) return max;
    max = _;
    return setting;
  };

  setting.step = function(_) {
    if (!arguments.length) return step;
    step = _;
    return setting;
  };

  setting.valueBeforeRestore = function(_) {
    if (!arguments.length) return valueBeforeRestore;
    valueBeforeRestore = _;
    return setting;
  };

  setting.is_textarea = function(_) {
    if (!arguments.length) return is_textarea;
    is_textarea = _;
    return setting;
  };

  setting.is_password = function(_) {
    if (!arguments.length) return is_password;
    is_password = _;
    return setting;
  };

  setting.is_flipper = function(_) {
    if (!arguments.length) return is_flipper;
    is_flipper = _;
    return setting;
  };

  setting.validate_fn = function(_) {
    if (!arguments.length) return validate_fn;
    validate_fn = _;
    return setting;
  };

  setting.id = function(_) {
    if (!arguments.length) return id;
    id = _;
    return setting;
  };

  setting.hidden = function(_) {
    if (!arguments.length) {
      if (id === 'ksql_stream_position') {
        if (viz_lib && viz_lib.dataconnection_type() !== 'ksql') {
          return true;
        }
      }

      return hidden;
    }

    hidden = _;
    return setting;
  };

  setting.mutable = function(_) {
    if (!arguments.length) return mutable;
    mutable = _;
    return setting;
  };

  setting.valueCb = function(_) {
    if (!arguments.length) return valueCb;
    valueCb = _;
    return setting;
  };

  setting.valuePreprocessCb = function(_) {
    if (!arguments.length) return valuePreprocessCb;
    valuePreprocessCb = _;
    return setting;
  };

  setting.value = function(_) {
    // set the defaultValue as the init value
    if (defaultValue === undefined) {
      defaultValue = _;
    }

    if (!arguments.length) {
      if (is_password && value === '<placeholder>') {
        return undefined;
      }

      return value;
    }

    if (valuePreprocessCb) {
      _ = valuePreprocessCb(_);
    }

    // Validate before setting
    if (validate_fn && !validate_fn(_)) {
      console.log('Invalid setting', id, _);
      return setting;
    }

    if ((type === arc.visual.settingTypeEnum.INTEGER) ||
        (type === arc.visual.settingTypeEnum.INTEGER_NOSPINNER)) {
      if (_ !== '') {
        if (min && max) {
          if (!setting.validate_num_range(_, min, max)) {
            console.log('Invalid setting', id, _);
            return setting;
          }
        }
      }
    }

    value = _;
    setting.isChecked = value === 'true';

    // When a new value is being set, paint it in the visible setting
    paintNewValue();

    if (valueCb) {
      valueCb(value);
    }

    return setting;
  };

  setting.resetValue = function() {
    value = defaultValue;

    if (valuePreprocessCb) {
      value = valuePreprocessCb(value);
    }

    paintNewValue();
    if (valueCb) {
      valueCb(value);
    }

    setting.makeChangeCbs();

    if (type === arc.visual.settingTypeEnum.COLOR) {
      $(`#${div_id} .spectrum-colorpicker`).spectrum('set', value);
    }

    if (type === arc.visual.settingTypeEnum.CHECKBOX) {
      $(`#${div_id} input`).trigger('change');
    }

    return setting;
  };

  setting.unique_id = function() {
    return unique_id;
  };

  setting.div_id = function(_) {
    if (!arguments.length) return div_id;
    div_id = _;
    return setting;
  };

  function generateNewSettingUserControlId() {
    div_id = `settings-user-control-${arc.visual.setting_unique_id}`;
    arc.visual.setting_unique_id += 1;
  }

  setting.type = function(_) {
    if (!arguments.length) return type;
    type = _;
    setting.isChecked = value === 'true';
    return setting;
  };

  setting.section = function(_) {
    if (!arguments.length) return section;
    section = _;
    return setting;
  };

  setting.tab = function(_) {
    if (!arguments.length) return tab;
    tab = _;
    return setting;
  };

  setting.group = function(_) {
    if (!arguments.length) return group;
    group = _;
    return setting;
  };

  /**
   * When there's multiple options
   * and you want them to be next to each other,
   * set it to true. By default, it's true.
   * (Only radio inputs support this currently)
   */
  setting.inline = function (_) {
    if (!arguments.length) return inline;
    inline = _;
    return setting;
  };

  setting.add_name = function(_) {
    names.push(_);
    return setting;
  };

  setting.names = function(_) {
    if (!arguments.length) return names;
    names = _;
    return setting;
  };

  setting.add_label = function(_) {
    labels.push(_);
    return setting;
  };

  setting.labels = function(_) {
    if (!arguments.length) return labels;
    labels = _;
    return setting;
  };

  setting.info = function(_) {
    if (!arguments.length) return info;
    info = _;
    return setting;
  };

  setting.level = function(_) {
    if (!arguments.length) return level;
    level = _;
    return setting;
  };

  setting.infoblock = function(_) {
    if (!arguments.length) return infoblock;
    infoblock = _;
    return setting;
  };

  setting.add_class = function(_) {
    classes.push(_);
    return setting;
  };

  setting.classes = function(_) {
    if (!arguments.length) return classes;
    classes = _;
    return setting;
  };

  setting.placeholder = function(_) {
    if (!arguments.length) return placeholder;
    placeholder = _;
    return setting;
  };

  setting.paint_fn = function(_) {
    if (!arguments.length) return paint_fn;
    paint_fn = _;
    return setting;
  };

  setting.repaint_fn = function(_) {
    if (!arguments.length) return repaint_fn;
    repaint_fn = _;
    return setting;
  };

  setting.add_postpaint_fn = function(_) {
    postpaint_fn.push(_);
    return setting;
  };

  setting.fetch_fn = function(_) {
    if (!arguments.length) return fetch_fn;
    fetch_fn = _;
    return setting;
  };

  setting.shown_fn = function(_) {
    if (!arguments.length) return shown_fn;
    shown_fn = _;
    return setting;
  };

  setting.shown = function() {
    if (shown_fn) {
      shown_fn();
    }

    return setting;
  };

  setting.useAsDefault = function(_) {
    if (!arguments.length) return useAsDefault;
    useAsDefault = _;
    return setting;
  };

  setting.suffix = function(_) {
    if (!arguments.length) return suffix;
    suffix = _;
    return setting;
  };

  setting.hide = function() {
    setting.hidden(true);
    handleHidden();

    return setting;
  };

  setting.show = function() {
    setting.hidden(false);
    handleHidden();

    return setting;
  };

  setting.setDisableTooltipText = function(_) {
    if (!arguments.length) return disableTooltipText;
    disableTooltipText = _;
    return setting;
  };

  setting.disable_fn = function(_) {
    if (!arguments.length) return disable_fn;
    disable_fn = _;
    return setting;
  };

  setting.enable_fn = function(_) {
    if (!arguments.length) return enable_fn;
    enable_fn = _;
    return setting;
  };

  setting.disable = function() {
    disabled = true;

    if (disableTooltipText) {
      showDisableTooltipText();
    }

    // Don't show info tooltip when setting is disabled
    _getTooltipTarget().find('[data-toggle="popover"]').popover('disable');

    $(`#${setting.div_id()}`).addClass('disabled-setting');
    switch (type) {
      case arc.visual.settingTypeEnum.DROPDOWN:
        $(`#${setting.div_id()} .btn`).prop('disabled', true);
        break;

      case arc.visual.settingTypeEnum.DROPDOWN_CHOSEN:
        $(`#${setting.div_id()} select`).prop('disabled', true).trigger('chosen:updated');
        break;

        case arc.visual.settingTypeEnum.CHECKBOX:
          $(`#${setting.div_id()}`).find('input').prop('disabled', true);
          setting.isChecked = false; // disable functionality too not just visually
          break;

      case arc.visual.settingTypeEnum.SLIDER:
      case arc.visual.settingTypeEnum.STRING:
        $(`#${setting.div_id()}`).find('input,textarea').prop('disabled', true);
        break;

      case arc.visual.settingTypeEnum.INTEGER:
        $(`#${setting.div_id()}`).find('input').prop('disabled', true);
        $(`#${setting.div_id()}`).find('.plus').prop('disabled', true);
        $(`#${setting.div_id()}`).find('.minus').prop('disabled', true);
        break;

      case arc.visual.settingTypeEnum.RADIO:
        $(`#${setting.div_id()}`).find('input[type=radio]').prop('disabled', true);
        setting.isChecked = false; // disable functionality too not just visually
        break;

      case arc.visual.settingTypeEnum.SLIDER_RANGE:
        $(`#${setting.div_id()}`).find('.settings-range-slider-main').slider({ disable: true });
        break;

      default:
    }

   if (disable_fn) {
    disable_fn();
   }

    // Also disable dependent settings.
    const { settingsArray = [], showValues = [] } = dependentSettings;

    if (settingsArray.length) {
      settingsArray.forEach((item) => {
        if (showValues.includes('false') && !setting.isEnabled()) {
          item.enable();
        } else {
          item.disable();
        }
      });
    }
  };

  setting.enable = function() {
    disabled = false;

    if (disableTooltipText) {
      disposeTooltip();
    }

    // Enable any disabled info tooltip
    _getTooltipTarget().find('[data-toggle="popover"]').popover('enable');

    $(`#${setting.div_id()}`).removeClass('disabled-setting');
    switch (type) {
      case arc.visual.settingTypeEnum.DROPDOWN:
        $(`#${setting.div_id()} .btn`).prop('disabled', false);
        break;

      case arc.visual.settingTypeEnum.DROPDOWN_CHOSEN:
        $(`#${setting.div_id()} select`).prop('disabled', false).trigger('chosen:updated');
        break;

      case arc.visual.settingTypeEnum.CHECKBOX:
        $(`#${setting.div_id()}`).find('input').prop('disabled', false);
        setting.isChecked = setting.value() === 'true';
        break;

      case arc.visual.settingTypeEnum.SLIDER:
      case arc.visual.settingTypeEnum.STRING:
        $(`#${setting.div_id()}`).find('input,textarea').prop('disabled', false);
        break;

      case arc.visual.settingTypeEnum.INTEGER:
        $(`#${setting.div_id()}`).find('input').prop('disabled', false);
        $(`#${setting.div_id()}`).find('.plus').prop('disabled', false);
        $(`#${setting.div_id()}`).find('.minus').prop('disabled', false);
        break;

      case arc.visual.settingTypeEnum.RADIO:
        $(`#${setting.div_id()}`).find('input[type=radio]').prop('disabled', false);
        setting.isChecked = setting.value() === 'true';
        break;

      case arc.visual.settingTypeEnum.SLIDER_RANGE:
        $(`#${setting.div_id()}`).find('.settings-range-slider-main').slider({ disable: false });
        break;

      default:
    }

    if (enable_fn) {
      enable_fn();
     }

    // Also enable dependent settings
    const { settingsArray = [], showValues = [] } = dependentSettings;

    if (settingsArray.length) {
      settingsArray.forEach((item) => {
        if (showValues.includes(setting.value())) {
          if (setting.isEnabled()) {
            item.enable();
          } else {
            item.disable();
          }
        } else if (setting.isEnabled()) {
          item.disable();
        } else {
          item.enable();
        }
      });
    }
  };

  function _getTooltipTarget() {
    const $targetElement = $(`#${setting.div_id()} label`);

    // one label and has text -> tooltip on label
    // one label but no text -> tooltip on parent
    // multiple labels -> tooltip on parent
    if ($targetElement.length === 1 && $targetElement.text().length) {
      return $targetElement;
    }

    return $(`#${setting.div_id()}`);
  }

  function disposeTooltip() {
    _getTooltipTarget().tooltip('dispose');
  }

  function showDisableTooltipText() {
    _getTooltipTarget().tooltip({
      title: disableTooltipText,
      delay: { show: 300 }
    });
  }

  setting.setEnableDisable = function(enable = true) {
    if (enable) {
      setting.enable();
    } else {
      setting.disable();
    }
  };

  /**
   * This function is for adding a simple dependency on any given setting.
   * @param settingsArray: array of settings that are dependent on this setting
   * otherwise it will disable (and gray out)
   * @param hideValues:array of values you want trigger a hide/disable of dependent settings
   * @param showValues: array of values you want to trigger a show/enable of dependent settings
   * @returns {setting}
   *
   * Example: you have a checkbox setting addOptions that when checked you want a list of settings
   * that show various options to be accessible and when not checked to not be accessible
   * the following would suffice:
   * addOptions.addDependents([option1Setting, option2Setting, option3Setting])
   */

   setting.getDependents = function() {
    return dependentSettings.settingsArray;
  };

  setting.addDependents = function(settingsArray, hideValues = ['false'], showValues = ['true'], defaultDisableTooltipText = '') {
    if (!settingsArray) return;

    dependentSettings = {
      settingsArray,
      showValues
    };

    settingsArray.forEach((dependent) => {
      dependent.add_postpaint_fn(() => {
        const parentVal = setting.value();
        if (hideValues.includes(parentVal)) {
          dependent.disable();
        } else if (showValues.includes(parentVal) && setting.isEnabled()) {
          // If the parent is disabled, ignore if it's checked or not.
          dependent.enable();
        }
      });

      // Set a default tooltip text unless if it already has one.
      if (!dependent.setDisableTooltipText()) {
        if (defaultDisableTooltipText) {
          dependent.setDisableTooltipText(defaultDisableTooltipText);
        } else if (labels.length === 1) {
          dependent.setDisableTooltipText(`This is currently disabled due to the "${labels[0]}" setting`);
        }
      }
    });

    const oldProcessCb = setting.valuePreprocessCb();
    setting.valuePreprocessCb((newVal) => {
      if (oldProcessCb) {
        oldProcessCb(newVal);
      }

      if (hideValues.includes(newVal)) {
        settingsArray.forEach((s) => s.disable());
      } else if (showValues.includes(newVal)) {
        settingsArray.forEach((s) => {
          if (setting.isEnabled()) {
            s.enable();
          } else if (newVal === 'true') {
            s.disable();
          }
        });
      }

      return newVal;
    });

    return setting;
  };

  setting.paintSetting = function (tab_d3_sel, settingId) {
    drawSettingByType(tab_d3_sel);
    if (isEnabledBasedOnUserSettings(settingId)) {
      setting.enable();
    } else {
      setting.disable();
    }
  };

  function isEnabledBasedOnUserSettings(settingId) {
    const dashboardSettings = arc.globals.dashboard?.user_settings();
    return dashboardSettings && dashboardSettings.find_by_id(settingId).value() === 'true';
  }

  function paintColor(tab_d3_sel) {
    const v = setting.value();
    const name = setting.labels()[0];

    const container =
      tab_d3_sel.append('div')
                .attr('id', div_id)
                .attr('data-setting-id', id)
                .classed(setting.classes().join(' '), true)
                .html(arc.misc.safe_substitute($('#settings-color-picker').html(), {
                  NAME: name,
                }));

    if (hidden) {
      container.style('display', 'none');
    }

    /*
     * The color setting takes a long time to render on the screen.
     * This impacts edit-dashboard load time by about a second.
     * For tables, with the 9 table colors, this is about a second in delay again.
     * Of course, most users are not clicking to change colors very often.
     * So instead of always rendeting ahead of time, we render lazily.
     */

    let shown = false;

    // Attach to the expansion of all sections
    const $div = $(`#${div_id}`);
    const $allTabsShow = $div.closest('.tab-panel-content').find('.sections-expand-all');

    // Attach to the expansion of the containing section
    const $tabShow = $div.closest('.settings-tab-holder');

    const tabpanelId = $div.closest('.tab-pane').attr('id');

    // Attach to the expansion of the section (for Colors)
    const $clickTab = $(`a[href="\\#${tabpanelId}"]`);
    $clickTab.offOn(`click.${div_id}`, showOnce);

    if ($clickTab.closest('li[role="presentation"]').hasClass('active')) {
      showOnce();
    }

    $allTabsShow.off(`click.${div_id}`)
                .one(`click.${div_id}`, showOnce);

    if ($tabShow.hasClass('open')) {
      showOnce();
    } else {
      $tabShow.off(`click.${div_id}`)
              .one(`click.${div_id}`, showOnce);
    }

    if ($(`#${div_id}`).closest('.settings-modal').length > 0) {
      // If in a modal, draw immediately
      showOnce();
    }

    function showOnce() {
      if (shown) {
        return;
      }

      shown = true;

      $(`#${div_id} .spectrum-colorpicker`).spectrum({
          color: v,
          allowEmpty: true,
          showInput: true,
          showPalette: true,
          showAlpha: true,
          preferredFormat: 'hex',
          cancelText: 'Cancel',
          chooseText: 'OK',
          clickoutFiresChange: true,
          palette: [
            ['#0097a7', '#7dcad2', '#546e7a', '#b0bec5', '#ed8111', '#f6c088', '#e54954', '#f2a4a9'],
            ['#000', '#444', '#666', '#999', '#ccc', '#eee', '#f3f3f3', '#fff'],
            ['#f00', '#f90', '#ff0', '#0f0', '#0ff', '#00f', '#90f', '#f0f'],
            ['#f4cccc', '#fce5cd', '#fff2cc', '#d9ead3', '#d0e0e3', '#cfe2f3', '#d9d2e9', '#ead1dc'],
            ['#ea9999', '#f9cb9c', '#ffe599', '#b6d7a8', '#a2c4c9', '#9fc5e8', '#b4a7d6', '#d5a6bd'],
            ['#e06666', '#f6b26b', '#ffd966', '#93c47d', '#76a5af', '#6fa8dc', '#8e7cc3', '#c27ba0'],
            ['#c00', '#e69138', '#f1c232', '#6aa84f', '#45818e', '#3d85c6', '#674ea7', '#a64d79'],
            ['#900', '#b45f06', '#bf9000', '#38761d', '#134f5c', '#0b5394', '#351c75', '#741b47'],
            ['#600', '#783f04', '#7f6000', '#274e13', '#0c343d', '#073763', '#20124d', '#4c1130'],
          ],
          move(color) {
            if (color === null) {
              setting.value('');
            } else {
              setting.value(color.toRgbString());
            }

            setting.makeChangeCbs();
          },
          hide(color) {
            if (color === null) {
              setting.value('');
            } else {
              setting.value(color.toRgbString());
            }

            setting.makeChangeCbs();
          },
      });
    }
  }

  function paintIntegerSpinner(tab_d3_sel) {
    let v = setting.value();
    if (v === undefined) {
      v = '';
    }

    if (v !== '') {
      if (Number.isNaN(v)) {
        v = '';
      } else {
        v = Number(v);
      }
    }

    const container =
      tab_d3_sel.append('div')
                .attr('id', div_id)
                .attr('data-setting-id', id)
                .attr('data-test-id', `setting-id-${id}`)
                .classed(setting.classes().join(' '), true)
                .html(arc.misc.safe_substitute($('#settings-integer-spinner').html(), {
                  NAME: setting.labels()[0],
                  VALUE: v,
                  SUFFIX: suffix,
                }));

    if (hidden) {
      container.style('display', 'none');
    }

    const options = {
      start: v,
      valueChanged() {
        setting.process_value();
        setting.makeChangeCbs();
      },
    };

    if (min !== undefined) {
      options.minValue = min;
    }

    if (max !== undefined) {
      options.maxValue = max;
    }

    if (step !== undefined) {
      options.step = step;
    }

    const selector = $(`#${div_id} .wan-spinner`);
    selector.WanSpinner(options);
    selector.find('input')
            .attr('name', () => getPrimaryName())
            .attr('data-test-id', () => getPrimaryName());
  }

  function paintSlider(tab_d3_sel, repaint = false) {
    let setting_value = setting.value();
    if (setting_value === undefined) {
      setting_value = '';
    }

    if (min === undefined) min = 0;
    if (max === undefined) max = 100;
    if (step === undefined) step = 1;
    const displayValue = useAsDefault && setting_value === useAsDefault ? 'default' : setting_value;

    const template = arc.misc.safe_substitute($('#settings-slider-template').html(), {
      NAME: setting.labels()[0],
      VALUE: setting_value,
      DISPLAY: displayValue,
      MIN: min,
      MAX: max,
      STEP: step,
      SUFFIX: suffix,
    });

    let container;
    if (repaint) {
      // redraw
      // https://www.geeksforgeeks.org/d3-js-d3-select-function/
      container = $(`[data_setting_id=${id}]`);
      container.empty();
      container.html(template);
    } else {
      // create & draw
      container =
        tab_d3_sel.append('div')
          .attr('id', div_id)
          .attr('data_setting_id', id)
          .classed(setting.classes().join('s '), true)
          .html(template);
    }

    $(`#${div_id}`).find('input').on('input', (event) => {
      const newVal = $(event.currentTarget).val();
      if (useAsDefault && String(newVal) === String(useAsDefault)) {
        $(`#${div_id} .settings-slider-value`).html('default');
      } else {
        $(`#${div_id} .settings-slider-value`).html(newVal);
      }

      setting.value(newVal);
    });

    if (hidden) {
      container.style('display', 'none');
    }
  }

  function paintRangeSlider(tab_d3_sel) {
    let setting_value = setting.value();
    if (setting_value === undefined) {
      setting_value = '';
    }

    if (min === undefined) min = 0;
    if (max === undefined) max = 100;
    if (step === undefined) step = 1;

    const container =
      tab_d3_sel.append('div')
        .attr('id', div_id)
        .attr('data_setting_id', id)
        .classed(setting.classes().join('s '), true)
        .html(arc.misc.safe_substitute($('#settings-range-slider-template').html(), {
          NAME: setting.labels()[0],
          VALUE: setting_value,
          MIN: min,
          MAX: max,
          SUFFIX: suffix,
        }));

    $(`#${div_id}`).find('.settings-range-slider-main').slider({
      range: true,
      min,
      max,
      values: setting.get_range(),
      slide(event, ui) {
        $(`#${div_id} .settings-slider-value`).html(`${ui.values[0]} - ${ui.values[1]}`);
      },
      stop(event, ui) {
        setting.value(`${ui.values[0]}-${ui.values[1]}`);
        setting.makeChangeCbs();
      },
    });

    if (hidden) {
      container.style('display', 'none');
    }
  }

  function paintDisplayOnly(tab_d3_sel) {
    const container = tab_d3_sel.append('div')
                .attr('class', `form-group ${setting.id()}`)
                .attr('id', div_id)
                .attr('data-setting-id', id)
                .classed(setting.classes().join(' '), true)
                .html(`<label>${setting.labels()[0]}</label>` +
                  `<span class="display-setting-value">${setting.value()}</span>`);
    if (hidden) {
      container.style('display', 'none');
    }
  }

  setting._paint_textbox = function(tab_d3_sel) {
    if (is_textarea) {
      return setting._paint_textarea(tab_d3_sel);
    }

    const container =
      tab_d3_sel.append('div')
                .attr('class', `form-group ${setting.id()}`)
                .attr('id', div_id)
                .attr('data-setting-id', id)
                .classed(setting.classes().join(' '), true)
                .html($('#settings-textbox-template').html());
    if (hidden) {
      container.style('display', 'none');
    }

    const $s = $(`#${div_id}`);
    $s.find('label').text(setting.labels()[0]);
    const $input = $s.find('input');
    $input.val(setting.value())
          .attr('name', getPrimaryName())
          .attr('data-test-id', getPrimaryName())
          .attr('placeholder', setting.placeholder());
    if (mutable === false) {
      $input.prop('disabled', true);
      $input.css('cursor', 'default');
    }

    if (is_password) {
      $input.attr('type', 'password')
            .attr('autocomplete', 'off')
            .val('<placeholder>');
    }

    return setting;
  };

  setting._paint_textarea = function(tab_d3_sel) {
    const container =
      tab_d3_sel.append('div')
                .attr('class', `form-group ${setting.id()}`)
                .attr('data-setting-id', id)
                .attr('id', div_id)
                .classed(setting.classes().join(' '), true)
                .html($('#settings-textarea-template').html());
    if (hidden) {
      container.style('display', 'none');
    }

    const $s = $(`#${div_id}`);
    $s.find('label').text(setting.labels()[0]);
    $s.find('textarea').val(setting.value())
                       .attr('name', getPrimaryName());
    if (mutable === false) {
      $s.find('textarea').prop('disabled', true);
      $s.find('textarea').css('cursor', 'default');
    }

    return setting;
  };

  function getPrimaryName() {
    return (names.length === 0 ? id : names[0]);
  }

  setting._paint_checkbox = function (tab_d3_sel, repaint) {
    let container;
    const template = arc.misc.safe_substitute(
      $('#checkbox-template').html(),
      {
        ID: setting.id(),
        NAME: getPrimaryName(),
        CHECKED: value === 'true' ? 'checked' : '',
        DISABLED: mutable ? '' : 'disabled',
        LABEL: setting.labels()[0]
      }
    );

    if (repaint) {
      // redraw
      // https://www.geeksforgeeks.org/d3-js-d3-select-function/
      container = $(`[data-setting-id=${id}]`);
      container.empty();
      container.html(template);
    } else {
      container =
        tab_d3_sel.append('div')
          .attr('id', div_id)
          .attr('data-setting-id', id)
          .attr('data-test-id', `setting-id-${id}`)
          .attr('class', 'form-check my-2')
          .classed(setting.classes().join(' '), true)
          .style(hidden ? { display: 'none' } : undefined)
          .html(template);
    }

    const $input = $(`#${div_id} input`);

    $input.offOn('change', () => {
      setting.value($input.is(':checked') ? 'true' : 'false');
      setting.makeChangeCbs();
    });

    return setting;
  };

  setting._paint_switch = function(tab_d3_sel) {
    tab_d3_sel.append('div')
              .attr('data-setting-id', id)
              .attr('id', div_id)
              .attr('class', 'checkbox switch')
              .classed(setting.classes().join(' '), true)
              .style(function() {
                  if (hidden) {
                    return { display: 'none' };
                  }
              }())
                .append('label')
                .attr('class', `checkbox ${setting.id()}`)
                .html(() => {
                  const template = $('#switch-template').html();
                  return arc.misc.safe_substitute(
                      template,
                      {
                        NAME: setting.names()[0],
                        CHECKED: value === 'true' ? 'checked' : '',
                        DISABLED: mutable ? '' : 'disabled',
                        LABEL: setting.labels()[0]
                      }
                    );
                });

    const $sel = $(`#${div_id} input`);
    $sel.off('change')
        .on('change', () => {
          setting.value($sel.is(':checked') ? 'true' : 'false');
          setting.makeChangeCbs();
        });

    return setting;
  };

  setting.makeChangeCbs = function() {
    $.each(changeCbs, (k, v) => {
      if (v) {
        v(setting);
      }
    });

    return setting;
  };

  setting.removeChangeCbs = function() {
    changeCbs = {};
    return setting;
  };

  setting.addChangeCb = function(cbname, cb) {
    changeCbs[cbname] = cb;
    return setting;
  };

  setting.changeCb = setting.addChangeCb;

  function paintInfoblock() {
    if (hidden) {
      return setting;
    }

    if (!infoblock) {
      return setting;
    }

    $(`#${div_id}`).append(`<p class="form-text">${infoblock}</p>`);
    return setting;
  }

  function paintInfo() {
    if (hidden) {
      return setting;
    }

    if (!info) {
      return setting;
    }

    const $infoIcon = $(`<span class="info-icon" data-toggle="popover"></span>`);
    $infoIcon.data('content', info);

    $infoIcon.popover({
      container: 'body',
      placement: 'top',
      trigger: 'hover'
    });

    $(`#${div_id}`)
        .find('label')
        .first()
        .append($infoIcon);

    return setting;
  }

  function translateNameToLabel(name) {
    let label;
    const ns = setting.names();
    if (ns) {
      ns.some((n, i) => {
        if (n === name) {
          label = setting.labels()[i];
          return true;
        }

        return false;
      });
    }

    return label;
  }

  function translateLabelToName(label) {
    let name;
    const ls = setting.labels();
    if (ls) {
      ls.some((l, i) => {
        if (l === label) {
          name = setting.names()[i];
          return true;
        }

        return false;
      });
    }

    return name;
  }

  const primaryButtonSuffix = '<span class="caret" style="margin-left: 5px;"></span>';

  function dropdownHtml() {
    const v = setting.value();
    let display = translateNameToLabel(v);
    if (!display) {
      display = v;
    }

    return display + primaryButtonSuffix;
  }

  function paintDropdownChosen(tab_d3_sel) {
    const settingDiv = tab_d3_sel.append('div')
                                 .attr('id', div_id)
                                 .style('width', '200px')
                                 .style('margin-bottom', '10px');

    settingDiv.append('label')
              .text(group);

    const selectElem = settingDiv.attr('id', div_id)
                                 .append('select');
    const $select = $(selectElem.node());

    selectElem.selectAll('.nosel')
              .data(names)
              .enter()
                .append('option')
                .attr('value', (d) => d)
                .text((d, i) => labels[i]);

    selectElem.property('value', value);
    $select.chosen();

    $select.offOn('change.select', () => {
      setting.value($select.val());
      setting.makeChangeCbs();
    });
  }

  function paintDropdown(tab_d3_sel) {
    const div = tab_d3_sel.append('div')
                        .attr('id', div_id)
                        .attr('data-setting-id', id)
                        .attr('data-test-id', `setting-id-${id}`)
                        .classed(setting.classes().join(' '), true)
                          .append('div')
                          .classed('form-group dropdown-group clearfix', true);

    const $div = $(`#${div_id}`);
    if (hidden) {
      $div.css('display', 'none');
    }

    div.append('label')
       .text(setting.group());

    let conditionalClasses = '';
    if ($div.closest('#settings-modal-holder').length === 0) {
      conditionalClasses = 'pull-right';
    }

    const b = div.append('div')
               .classed(`btn-group btn-group-sm dropdown-btn-group ${conditionalClasses}`,
                        true);

    const primaryButton =
      b.append('button')
       .attr('type', 'button')
       .classed('btn btn-secondary btn-sm dropdown-toggle', true)
       .attr('data-toggle', 'dropdown')
       .attr('data-value', setting.value())
       .attr('data-test-id', 'dropdown-btn')
       .html(dropdownHtml());

    b.append('div')
     .classed('dropdown-menu', true)
     .selectAll('.nosel')
     .data(setting.labels())
     .enter()
      .append('a')
        .classed('dropdown-item', true)
        .html((d) => d)
        .attr('data-value', (d, i) => {
          const ns = setting.names();
          if (ns && ns.length > 0) {
            return ns[i];
          }

          return d;
        })
        .attr('data-test-id', (d, i) => {
          const ns = setting.names();
          if (ns && ns.length > 0) {
            return `value-${ns[i]}`;
          }

            return `value-${d}`;
          })
        .on('click.dropdown', (d, i) => {
          let v;
          const ns = setting.names();
          if (ns && ns.length > 0) {
            v = ns[i];
          } else {
            v = d;
          }

          primaryButton.html(d + primaryButtonSuffix)
                      .attr('data-value', v);

          setting.value(v);
          const $sel = $(`#${div_id} button`);
          $sel.trigger('change:on');
        });

    const $button = $(`#${div_id} button`);
    $button.offOn('change:on', () => {
      setting.makeChangeCbs();
    });
  }

  setting._paint_radio = function(tab_d3_sel) {
    const div_group = tab_d3_sel.append('div')
                          .attr('id', div_id)
                          .attr('data-setting-id', id)
                          .attr('class', `form-group ${setting.id()}`)
                          .attr('data-test-id', setting.id())
                          .classed(setting.classes().join(' '), true)
                          .classed('arc-invisible', () => hidden);

    div_group.append('label')
         .text(setting.group())
         .classed('m-0', true);

    div_group.selectAll('.nosel')
      .data(setting.names())
      .enter()
        .append('div')
        .attr('class', inline ? 'form-check form-check-inline' : 'form-check')
          .html((d, i) => `
            <input
              id="${setting.group()}-${d}"
              class="form-check-input"
              type="radio"
              name="${div_id}"
              data-test-id="radio-${d}"
              value="${d}"
              ${(d === setting.value()) ? ' checked' : ''}
            >
            <label for="${setting.group()}-${d}" class="form-check-label">
              ${(setting.labels().length === 0) ? d : setting.labels()[i]}
            </label>
          `);

    const $sel = $(`#${div_id} input`);
    $sel.offOn('change', () => {
      setting.value($(`#${div_id} input[name=${div_id}]:checked`).val());
      setting.makeChangeCbs();
    });

    return setting;
  };

  function paintNewValue() {
    const setting_type = setting.type();
    let $e;

    switch (setting_type) {
      case arc.visual.settingTypeEnum.COLOR:
        return;

      case arc.visual.settingTypeEnum.DROPDOWN:
        $e = $(`#${setting.div_id()} button`);
        $e.attr('data-value', value)
          .html(dropdownHtml());
        return;

      case arc.visual.settingTypeEnum.DROPDOWN_CHOSEN:
        $(`#${setting.div_id()} select`).val(value);
        return;

      case arc.visual.settingTypeEnum.CHECKBOX:
        $(`#${setting.div_id()} input`)
            .prop('checked', (value === 'true'));
        return;

      case arc.visual.settingTypeEnum.INTEGER:
        $(`#${setting.div_id()} .wan-spinner input`).val(value);
        return;

      case arc.visual.settingTypeEnum.PASSWORD:
      case arc.visual.settingTypeEnum.REAL:
      case arc.visual.settingTypeEnum.INTEGER_RANGE:
      case arc.visual.settingTypeEnum.STRING:
      case arc.visual.settingTypeEnum.INTEGER_NOSPINNER:
        if (is_textarea) {
          d3.select(`#${setting.div_id()}`)
            .select('textarea')
            .property('value', value);
        } else {
          d3.select(`#${setting.div_id()}`)
            .select('input')
            .property('value', value);
        }

        return;

      case arc.visual.settingTypeEnum.RADIO:
        $(`#${setting.div_id()} input`).prop('checked', false);
        $(`#${setting.div_id()} input[value="${value}"]`).prop('checked', true);
        return;

      case arc.visual.settingTypeEnum.SLIDER:
        if (useAsDefault && String(value) === String(useAsDefault)) {
          $(`#${setting.div_id()} .settings-slider-value`).html('default');
        } else {
          $(`#${setting.div_id()} .settings-slider-value`).html(value);
        }

        return;

      case arc.visual.settingTypeEnum.SLIDER_RANGE:
        $(`#${setting.div_id()} .settings-range-slider-main`).slider({ values: setting.get_range() });
        return;

      case arc.visual.settingTypeEnum.CUSTOM:
        return;

      case arc.visual.settingTypeEnum.DISPLAY_ONLY:
        $(`#${setting.div_id()}`).find('.display-setting-value').html(value);
        return;

      case arc.visual.settingTypeEnum.EXPRESSION:
        return;
      // no default
    }

    console.log('UNKNOWN setting type', setting_type);
  }

  setting.fetch_value = function() {
    if (fetch_fn) {
      return fetch_fn();
    }

    let new_value;
    const setting_type = setting.type();

    try {
      switch (setting_type) {
        case arc.visual.settingTypeEnum.DISPLAY_ONLY:
          new_value = setting.value();
          break;

        case arc.visual.settingTypeEnum.COLOR:
          new_value = setting.value() || '';

          break;

        case arc.visual.settingTypeEnum.DROPDOWN: {
          const t = $(`#${setting.div_id()} button`).attr('data-value');
          new_value = translateLabelToName(t);
          new_value = new_value || t;
          break;
        }

        case arc.visual.settingTypeEnum.DROPDOWN_CHOSEN:
          new_value = $(`#${setting.div_id()} select`).val();
          break;

        case arc.visual.settingTypeEnum.INTEGER: {
          const $e = $(`#${setting.div_id()} .wan-spinner input`);
          new_value = $e.val();
          break;
        }

        case arc.visual.settingTypeEnum.PASSWORD:
        case arc.visual.settingTypeEnum.REAL:
        case arc.visual.settingTypeEnum.SLIDER:
        case arc.visual.settingTypeEnum.INTEGER_RANGE:
        case arc.visual.settingTypeEnum.STRING:
        case arc.visual.settingTypeEnum.INTEGER_NOSPINNER:
          if (is_textarea) {
            new_value =
              d3.select(`#${setting.div_id()}`)
                .select('textarea')
                .property('value');
          } else {
            new_value =
              d3.select(`#${setting.div_id()}`)
                .select('input')
                .property('value');
          }

          break;

        case arc.visual.settingTypeEnum.CHECKBOX:
          new_value = $(`#${setting.div_id()} input`)
            .is(':checked') ? 'true' : 'false';
          break;

        case arc.visual.settingTypeEnum.RADIO:
          new_value =
            d3.select(`#${setting.div_id()}`)
              .select('input:checked')
              .attr('value');
          break;

        case arc.visual.settingTypeEnum.SLIDER_RANGE: {
          const valueRange = $(`#${div_id}`).find('.settings-range-slider-main').slider('values');
          new_value = `${valueRange[0]}-${valueRange[1]}`;
          break;
        }
        // no default
      }
    } catch (e) { console.log(e); }

    if (new_value !== undefined) {
      return new_value;
    }

    return undefined;
  };

  setting.process_value = function() {
    const new_value = setting.fetch_value();
    let needs_save = false;

    if (setting.value() !== new_value && typeof new_value !== 'undefined') {
      setting.value(new_value);
      needs_save = true;
    }

    return needs_save;
  };

  let lastPaintSel;

  setting.repaintControl = function() {
    if (lastPaintSel) {
      setting.paintControl(lastPaintSel, true);
    }
  };

  const noChangeHandling = new Set([
    arc.visual.settingTypeEnum.DROPDOWN,
    arc.visual.settingTypeEnum.DROPDOWN_CHOSEN,
    arc.visual.settingTypeEnum.COLOR,
    arc.visual.settingTypeEnum.CHECKBOX,
    arc.visual.settingTypeEnum.RADIO,
  ]);

  function handleChanges() {
    const settingType = setting.type();

    if (noChangeHandling.has(settingType)) {
      return;
    }

    const $sel = $(`#${div_id}`).find('input, textarea').not('.stop-event-propagation');

    $sel.off('change.setting');
    $sel.off('keyup.setting');

    switch (settingType) {
      case arc.visual.settingTypeEnum.STRING:
      case arc.visual.settingTypeEnum.INTEGER_NOSPINNER:
        $sel.on('change.setting', () => {
          setting.value($sel.val());
          setting.makeChangeCbs();
        });
        break;

      case arc.visual.settingTypeEnum.PASSWORD:
      case arc.visual.settingTypeEnum.INTEGER_RANGE:
      case arc.visual.settingTypeEnum.CUSTOM:
        if (id !== 'inline_css') {
          $sel.on('keyup.setting', () => {
            switch (id) {
              case 'upload_picker':
              case 'pattern_picker':
                setting.value(setting.fetch_fn()());
                break;
              default:
                setting.value($sel.val());
            }
            setting.makeChangeCbs();
          });
        }

        // Custom settings have custom values.
        // For example the colorpalette setting consists a reverse <input>, the color palettes,
        // <input> for fix colors for specified dimension values etc., stored and handled together.
        // They have the fetch_fn function to read from the UI the current state.
        $sel.on('change.setting', () => {
          const val = setting.id() === 'daterange_picker' ? $sel.val() : setting.fetch_fn()();
          setting.value(val);
          setting.makeChangeCbs();
        });
        break;

      default:
        $sel.on('change.setting', () => {
          setting.value($sel.val());
          setting.makeChangeCbs();
        });
    }
  }

  function handleHidden() {
    if (setting.hidden()) {
      $(`#${div_id}`).css('display', 'none');
    } else {
      $(`#${div_id}`).css('display', '');
    }
  }

  function handleDisabled() {
    if (disabled) {
      setting.disable();
    }
  }

  function drawSettingByType(tab_d3_sel, repaint = false) {
    const setting_type = setting.type();
    if (setting_type === arc.visual.settingTypeEnum.PASSWORD) {
      setting.is_password(true);
    }

    if (!repaint) {
      generateNewSettingUserControlId();
    }

    switch (setting_type) {
      case arc.visual.settingTypeEnum.INTEGER:
        paintIntegerSpinner(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.PASSWORD:
      case arc.visual.settingTypeEnum.REAL:
      case arc.visual.settingTypeEnum.INTEGER_RANGE:
      case arc.visual.settingTypeEnum.STRING:
      case arc.visual.settingTypeEnum.INTEGER_NOSPINNER:
        setting._paint_textbox(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.CHECKBOX:
        setting._paint_checkbox(tab_d3_sel, repaint);
        break;

      case arc.visual.settingTypeEnum.SWITCH:
        setting._paint_switch(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.RADIO:
        setting._paint_radio(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.DROPDOWN:
        paintDropdown(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.DROPDOWN_CHOSEN:
        paintDropdownChosen(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.COLOR:
        paintColor(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.SLIDER:
        paintSlider(tab_d3_sel, repaint);
        break;

      case arc.visual.settingTypeEnum.SLIDER_RANGE:
        paintRangeSlider(tab_d3_sel);
        break;

      case arc.visual.settingTypeEnum.DISPLAY_ONLY:
        paintDisplayOnly(tab_d3_sel);
        break;

      // no default
    }
  }

  setting.isEnabled = function () {
    return !disabled;
  };

  setting.paintControl = function(tab_d3_sel, repaintControl = false) {
    lastPaintSel = tab_d3_sel;

    function handlePostPaint() {
      if (postpaint_fn.length > 0) {
        postpaint_fn.forEach((f) => {
          f(tab_d3_sel);
        });
      }
    }

    function drawComplementarySettingDetails(paintInfoSection = false) {
      handlePostPaint();
      if (paintInfoSection) {
        paintInfo();
        paintInfoblock();
      }

      handleChanges();
      handleHidden();
      handleDisabled();
    }

    function drawSettingSectionByType(repaint = false) {
      drawSettingByType(tab_d3_sel, repaint);

      if (disabled && disableTooltipText) {
        showDisableTooltipText(this?.div_id?.());
      }

      drawComplementarySettingDetails(true);

      return setting;
    }

    if (repaintControl) {
      if (repaint_fn) {
        repaint_fn(lastPaintSel);
        drawComplementarySettingDetails();
        return setting;
      }

      if (!paint_fn) {
        drawSettingSectionByType(repaintControl);
        return setting;
      }

      if (paint_fn) {
        // Please if you use paint_fn, use repaint_fn as well!
      }
    }

    if (paint_fn) {
      generateNewSettingUserControlId();
      paint_fn(tab_d3_sel);
      drawComplementarySettingDetails();
      return setting;
    }

    if (disabled && disableTooltipText) {
      showDisableTooltipText();
    }

    drawSettingSectionByType();
    return setting;
  };

  return setting;
};

arc.visual.api = function() {
  const api = function() { return 'arc.visual.api'; };

  /*
   * Set the viz_lib connection to this visual. All external API
   * calls happen via this instance.
   */
  api.viz_lib = function() {
    throw new Error('viz_lib not implemented');
  };

  /*
   * Retrieve the parameters stored with this particular visual
   * There are two ways in which these parameters are set:
   * 1. The viz_setSettings() call below
   * 2. Interactions of the user with the builder widgets
   */
  api.viz_getSettings = function() {
    throw new Error('getSettings not implemented');
  };

  /*
   * Set the parameters stored with this particular visual
   */
  api.viz_setSettings = function() {
    throw new Error('setSettings not implemented');
  };

  /*
   * Draw the designer widgets associated with this visual. Use
   * the 'shelves_div' element passed in to display the elements.
   * Do not use anything outside of 'shelves_div'
   */
  api.viz_drawDesigner = function() {
    throw new Error('drawDesigner not implemented');
  };

  /*
   * Draw the display widgets associated with this visual. Use
   * the 'shelves_div' element passed in to display the elements.
   * Do not use anything outside of 'shelves_div'
   */
  api.viz_drawVisual = function() {
    throw new Error('drawVisual not implemented');
  };

  /*
   * Optional function, only overwrite this function if you need to
   * fetch visual specific resources. Gets called right before viz_drawVisual.
   */
  api.viz_fetchCollateral = function(viz_div, redraw, drawCb) {
    if (drawCb) {
      drawCb(viz_div, redraw);
    }
  };

  /*
   * Optional function, only overwrite this function if you need to explicitly
   * process dashboard refresh calls. Currently, the only visual that does this
   * is quickfilter daterange visuals.
   */
  api.viz_processRefresh = function() {
    return false;
  };

  /*
   * Filter controls have to respond explicitly to dashboard reset
   * events
   */
  api.viz_resetControl = function() {
    throw new Error('resetControl not implemented');
  };

  /*
   * Get the data request associated with this visual
   */
  api.viz_getDataRequest = function() {
    throw new Error('getDataRequest not implemented');
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  api.viz_setDataResult = function() {
    throw new Error('setDataRequest not implemented');
  };

  /*
   * In app mode, configure the visual. Typically throws up the settings modal
   * after some initial setup.
   */
  api.viz_configure = function() {
    throw new Error('configure not implemented');
  };

  /*
   * For filter widgets, this api entrypoint returns the name of the output
   * parameter emitted.
   */
  api.viz_getOutputParameter = function() {
    return undefined;
  };

  return api;
};
}());
