window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.user_settings = arc.visual.user_settings || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

// Used to run function multiple times on different variables given conditional
// variables is an array of objects with a "run" parameter, or just an array
function testAndRun(variables, cb) {
  const results = [];

  variables.forEach(function(v, i) {
    if (typeof(v.run) === "undefined" || v.run) {
      results.push(cb(v, i));
    } else {
      results.push(undefined);
    }
  });

  return results;
}

let LINE_WIDTH = 3;
let LINE_HIGHLIGHT_ARC = 4;
let LINE_HIDDEN_RADIUS = 18;
let LINE_HIDDEN_WIDTH = 40;

arc.visual.plugins.bar = function() {
  var sort_orders_asc = {},
      saved_settings = {},
      data_result,
    /*  makeDimDynamic tracks whether user :
      1) dimensions are on x axis and
      2) user specified "dynamic_bar_width" === true  */
      makeDimDynamic,
      lib,
      isHistogram,
      histogramConfig,
      cellSpecificHistConfig,
      mark_type = 'bar',
      previousDataTracker = arc.transition.dataTracker(),
      previousDataRenderer,
      formatPercentTwo = d3.format(".2%"),

      // arc_visual_histogram uses bars to do the actual drawing.
      //   markerLib is set to the library object associated with the
      //   histogram in this case so that the context-sensitive handling
      //   of marks is based on the histogram, not on the bar.
      markerLib,
      // Combined graphs have different y shelf
      y_shelf = 'y_shelf',
      x_shelf = 'x_shelf',
      secondary_y_shelf = 'color_shelf',
      y_scale, x_scale, y_scale_secondary,
      combined_visual = false,
      contextmenuDomId,
      allowLargeTrellisPerm = false,
      allowLargeTrellisTemp = false,
      last_x_is_dim,
      currentHelpers = [],
      maxDimWidth = arc.misc.isNumeric(arc.globals.app_settings.maximumCanvasWidth) ?
        Number(arc.globals.app_settings.maximumCanvasWidth) : 5000,
      transitionMaxRows = 500;
  let helper;

  var bar = arc.visual.api();

  const legendReverse = arc.report.user_settings.reverse_legend_order();
  const legendWidthOverRide = arc.report.user_settings.legend_width_override();
  const legendStyle = arc.report.user_settings.legend_style();
  legendStyle.addDependents(
    [legendReverse, legendWidthOverRide],
    ['None'],
    ['Right', 'Bottom', 'Above']
  );
  const trendByColor = arc.report.user_settings.trendline_color();
  const trendDecimals = arc.report.user_settings.trendline_tooltip_decimal();
  const trendLinear = arc.report.user_settings.trendline_linear();
  trendLinear.addDependents([trendByColor, trendDecimals]);
  const dimIsContinuous = arc.settings.line.continuousDimScale();
  const minDimNumeric = arc.settings.line.minDimAxis();
  const maxDimNumeric = arc.settings.line.maxDimAxis();
  dimIsContinuous.addDependents([minDimNumeric, maxDimNumeric]);

  const getCanHaveSecondaryAxis = (markType) => ['line', 'groupedbar'].includes(markType) || combined_visual;
  const hasSecondaryAxisInUse = (config) => config.y_col?.secondaryaxis() ||
    config.secondary_y_col?.secondaryaxis();
  let user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.report.user_settings.bar_mark_size()
                                                          .hidden(true),
                                  arc.settings.visuals.enableAnimations(),
                                  arc.settings.visuals.visualAutodetectTime(),
                                  legendStyle,
                                  legendWidthOverRide,
                                  legendReverse,
                                  trendLinear,
                                  trendByColor,
                                  trendDecimals,
                                  arc.report.user_settings.scale_log10_percentage(),
                                  arc.report.user_settings.horizontal_gridlines_enabled(),
                                  arc.report.user_settings.vertical_gridlines_enabled(),
                                  arc.report.user_settings.scale_dim_independent(),
                                  arc.report.user_settings.scale_agg_independent(),
                                  arc.report.user_settings.scale_agg_zero(),
                                  arc.settings.line.minAggAxis(),
                                  arc.settings.line.maxAggAxis(),
                                  arc.settings.line.secondaryMinAggAxis(),
                                  arc.settings.line.secondaryMaxAggAxis(),
                                  dimIsContinuous,
                                  minDimNumeric,
                                  maxDimNumeric,
                                  arc.report.user_settings.x_axis_label(),
                                  arc.report.user_settings.y_axis_label(),
                                  arc.report.user_settings.dim_tick_label_maxlen(),
                                  arc.settings.line.showSelectionRange(),
                                  arc.settings.line.showXAxis(),
                                  arc.settings.line.showYAxis(),
                                  arc.settings.bar.enableDrillInto(),
                                  arc.settings.bar.inlineMeasureDisplay(),
                                  arc.settings.bar.inlineSumMeasureDisplay(),
                                  arc.settings.bar.minCellHeight(),
                                  arc.settings.bar.minCellWidth(),
                                  arc.settings.bar.ignoreStackedNegativesWithMixedMeasures(),
                                ]);

  arc.addColorSettings(user_settings, {
    includeUseCustomColorSetting: true,
    includeDimensionMapSetting: true,
    includeOpacitySetting: true,
    includeFixLegendSetting: true,
  });

  bar.enableDrillInto = function() {
    if (combined_visual) {
      return false;
    }

    const setting = user_settings.find_by_id('enable_drill_into');
    return setting && setting.isChecked;
  };

  function ignoreStackedNegativesWithMixedMeasures() {
    const setting = user_settings.find_by_id('ignore_stacked_negatives_with_mixed_measures');
    return setting && setting.isChecked;
  }

  let maxWaterfallAgg;
  function waterfall() {
    const setting = user_settings.find_by_id('waterfall');
    return setting && setting.isChecked;
  }

  function enableMultipleTooltips() {
    const setting = user_settings.find_by_id('enable_multiple_tooltips');
    return setting && setting.isChecked;
  }

  function inlineMeasureDisplay() {
    const setting = user_settings.find_by_id('inline_measure_display');
    return setting && setting.isChecked;
  }

  function inlineSumMeasureDisplay() {
    const setting = user_settings.find_by_id('inline_sum_measure_display');
    return setting && setting.isChecked;
  }

  function showSelectionRange() {
    return user_settings.find_by_id('show_selection_range');
  }

  function lineInterpolation() {
    return user_settings.find_by_id('line_interpolation');
  }

  function minCellHeight() {
    const v = user_settings.find_by_id('min_cell_height').value();

    if (v && !isNaN(v)) {
      return Number(v);
    }

    return 0;
  }

  function minCellWidth() {
    const v = user_settings.find_by_id('min_cell_width').value();

    if (v && !isNaN(v)) {
      return Number(v);
    }

    return 0;
  }

  bar.curve = function() {
    let curveType = bar.interpolation();
    if (bar.mark_type() === 'area' && bar.interpolation() === 'bundle') {
      curveType = 'basis';
      lineInterpolation().value('basis');
    }

    return arc.settings.interpolationToCurve(curveType);
  };

  bar.interpolation = function() {
    var s = lineInterpolation();
    if (!s) {
      return 'linear';
    }

    var v = s.value();
    if ((v === undefined) || (v.length === 0)) {
      return 'linear';
    }

    return v;
  };

  function show_x_axis() {
    return user_settings.find_by_id('show_x_axis');
  }

  function show_y_axis() {
    return user_settings.find_by_id('show_y_axis');
  }

  bar.smooth_line = function() {
    return user_settings.find_by_id('smooth_line');
  };

  function dynamic_bar_width() {
    return user_settings.find_by_id('dynamic_bar_width');
  }

  function y_axis_label() {
    return user_settings.find_by_id('y_axis_label');
  }

  function x_axis_label() {
    return user_settings.find_by_id('x_axis_label');
  }

  function dim_tick_label_maxlen() {
    return user_settings.find_by_id('dim_tick_label_maxlen');
  }

  function trendline_linear() {
    return user_settings.find_by_id('trendline_linear');
  }

  function trendline_color() {
    return user_settings.find_by_id('trendline_color');
  }

  function trendline_tooltip_decimal() {
    return user_settings.find_by_id('trendline_decimal');
  }

  function color_opacity() {
    return user_settings.find_by_id('color_opacity');
  }

  function legend_style() {
    return user_settings.find_by_id('legend_style');
  }

  function legend_width_override() {
    return user_settings.find_by_id('legend_width_override');
  }

  function reverse_legend_order() {
    return user_settings.find_by_id('reverse_legend_order');
  }

  function show_points() {
    return user_settings.find_by_id('show_points');
  }

  function horizontal_gridlines_enabled() {
    return user_settings.find_by_id('gridlines_enabled')?.isChecked;
  }

  function vertical_gridlines_enabled() {
    return user_settings.find_by_id('vertical_gridlines_enabled')?.isChecked;
  }

  function setting_scale() {
    return user_settings.find_by_id('scale_type');
  }

  function scale_agg_zero() {
    return user_settings.find_by_id('scale_agg_zero');
  }

  function min_agg_axis() {
    return user_settings.find_by_id('min_agg_axis');
  }

  function max_agg_axis() {
    return user_settings.find_by_id('max_agg_axis');
  }

  function secondary_min_agg_axis() {
    return user_settings.find_by_id('secondary_min_agg_axis');
  }

  function secondary_max_agg_axis() {
    return user_settings.find_by_id('secondary_max_agg_axis');
  }

  function scale_dim_independent() {
    return user_settings.find_by_id('scale_dim_independent');
  }

  function scale_agg_independent() {
    return user_settings.find_by_id('scale_agg_independent');
  }

  function bar_mark_size() {
    let s = user_settings.find_by_id('bar_mark_size');
    if (!s.hidden()) {
      return s;
    }

    return undefined;
  }

  // Only in combined visuals
  function highlight_mark_type() {
    return user_settings.find_by_id('highlight_mark_type');
  }
  function highlight_mark_radius() {
    return user_settings.find_by_id('highlight_mark_radius');
  }

  var shelf_data = [
    {
      name: "x_shelf",
      display_name: "X Axis",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: "y_shelf",
      display_name: "Y Axis",
      display_html: arc.report.y_shelf_html(),
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: "color_shelf",
      display_name: "Colors",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: "tooltip_shelf",
      display_name: "Tooltips",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: "drill_shelf",
      display_name: "Drill",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
      drillShelf: true,
    },
    {
      name: "label_shelf",
      display_name: "Labels",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: "filters_shelf",
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  var constraints = {
    minimums: {
      "x_shelf": 1,
      "y_shelf": 1,
    },
    maximums: {
      "label_shelf": 1,
    },
  };

  bar.user_settings = function(_) {
    if (!arguments.length) return user_settings;
    user_settings = _;
    return bar;
  };

  bar.shelf_data = function(_) {
    if (!arguments.length) return shelf_data;
    shelf_data = _;
    return bar;
  };

  bar.y_shelf = function(_) {
    if (!arguments.length) return y_shelf;
    y_shelf = _;
    return bar;
  };

  bar.x_shelf = function(_) {
    if (!arguments.length) return x_shelf;
    x_shelf = _;
    return bar;
  };

  bar.x_scale = function(_) {
    if (!arguments.length) return x_scale;
    x_scale = _;
    return bar;
  };

  bar.y_scale = function(_) {
    if (!arguments.length) return y_scale;
    y_scale = _;
    return bar;
  };

  bar.y_scale_secondary = function(_) {
    if (!arguments.length) return y_scale_secondary;
    y_scale_secondary = _;
    return bar;
  };

  bar.combined_visual = function(_) {
    if (!arguments.length) return combined_visual;
    combined_visual = _;

    if (combined_visual) {
      user_settings.removeSetting([
        user_settings.find_by_id('legend_width_override'),
        user_settings.find_by_id('reverse_legend_order'),
        user_settings.find_by_id('legend_style'),
        user_settings.find_by_id('enable_line_labels'),
        user_settings.find_by_id('maximum_label_width'),
      ]);
      if (dynamic_bar_width()) {
        dynamic_bar_width().value("true");
      }
    }

    return bar;
  };

  bar.markerLib = function(_) {
    if (!arguments.length) return markerLib;
    markerLib = _;

    // if histogram, pass in proper report_id and in_dashboard
    lib.report_id(markerLib.report_id());
    lib.in_dashboard(markerLib.in_dashboard());
    lib.viz_type('histogram');
    return bar;
  };

  bar.mark_type = function(_) {
    if (!arguments.length) return mark_type;
    mark_type = _;

    if (mark_type !== 'bar') {
      user_settings.find_by_id('inline_sum_measure_display').hidden(true);
    }

    // Customize settings specific to bar vs line/area
    if ((mark_type === 'bar') || (mark_type === 'groupedbar')) {
      user_settings.find_by_id('bar_mark_size').hidden(false);
      user_settings.addSetting(arc.report.user_settings.dynamic_bar_width());
      user_settings.find_by_id('inline_measure_display')
                   .labels(['Label measure values for bars']);
    } else {
      user_settings.addSetting([
                     arc.report.user_settings.show_points(),
                     arc.report.user_settings.smooth_line(),
                   ]);
      user_settings.find_by_id('inline_measure_display')
                   .labels(['Label measure values next to points']);
      if (mark_type === 'area') {
        user_settings.addSetting([
                       arc.settings.area.lineInterpolation(),
                     ]);
      } else {
        user_settings.addSetting([
                       arc.settings.line.lineInterpolation(),
                     ]);
      }
    }

    if (mark_type === 'line') {
      user_settings.addSetting([
                     arc.settings.line.lineStyleFromColor(),
                     arc.settings.line.enableMultipleTooltips(),
                     arc.settings.line.enableLineLabels(),
                     arc.settings.line.maxLabelWidth()
                   ]);
    }

    if (mark_type === 'groupedbar') {
      user_settings.removeSetting([
        user_settings.find_by_id('vertical_gridlines_enabled')
      ]);
    }

    if (!getCanHaveSecondaryAxis(mark_type)) {
      user_settings.removeSetting([
        arc.settings.line.secondaryMinAggAxis(),
        arc.settings.line.secondaryMaxAggAxis(),
      ]);
    }

    return bar;
  };

  bar.makeDimDynamic = function(_) {
    if (!arguments.length) return makeDimDynamic;
    makeDimDynamic = _;
    return bar;
  };

  bar.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    previousDataTracker.lib(lib);
    contextmenuDomId = 'stacked-context-' + lib.unique_report_id();
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return bar;
  };

  bar.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  bar.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return bar;
    }

    saved_settings = settings;
    arc.misc.handleLegacyLegendSetting(saved_settings, legend_style());

    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return bar;
  };

  bar.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup()
       .setupSwitchXY(shelves_div);

      // Define messages to be included in shelves
      var xMessages = [
                        lib.createPopoverMessages(['trellisFirst', 'columns.'])
                       ],
          yMessages = [
                        lib.createPopoverMessages(['trellisFirst', 'rows.'])
                      ],
          colorMessages = [
                            lib.createPopoverMessages('colorStackMeasure')
                          ],
          drillShelfMessages = [
                                 lib.createPopoverMessages('acceptsMultiple'),
                                 lib.createPopoverMessages('drillShelf'),
                                 lib.createPopoverMessages('emptyDrillShelf'),
                               ],
          labelMessages = [lib.createPopoverMessages('axisLabel', 'acceptsOne')];
      // Messages have to be personalized for bar versus line versus grouped bars
      if (mark_type === 'groupedbar') {
        colorMessages.unshift(lib.createPopoverMessages('groupBar'));
      } else if (mark_type === 'line' || mark_type === 'area') {
        colorMessages.unshift(lib.createPopoverMessages('colorline'));
      } else {
        colorMessages.unshift(lib.createPopoverMessages('colorBar'));
      }

      if (mark_type === 'line' || mark_type === 'area') {
        xMessages.unshift(lib.createPopoverMessages('xAxis', 'xLineFlip'));
        yMessages.unshift(lib.createPopoverMessages('yAxis', 'yLineFlip'));
      } else {
        xMessages.unshift(lib.createPopoverMessages('xAxis', 'xFlip'));
        yMessages.unshift(lib.createPopoverMessages('yAxis', 'yFlip'));
      }

    // Create those popovers
    lib.addShelfDescription(constraints, {
                                            x_shelf: xMessages,
                                            y_shelf: yMessages,
                                            color_shelf: colorMessages,
                                            drill_shelf: drillShelfMessages,
                                            label_shelf: labelMessages,
                                          });

    return bar;
  };

  function largestShelfIndex(cols) {
    if (cols.length === 0) {
      return undefined;
    }

    let col;
    cols.forEach(function(c) {
      if (col === undefined) {
        col = c;
        return;
      }

      // Add in the === check for shelfIndex is set to undefined
      // This is defensive programming to mark the later column as
      // the largest shelf id if shelf ids are not specified
      if ((c.shelfIndex() >= col.shelfIndex()) ||
          (c.shelfIndex() === col.shelfIndex())) {
        col = c;
        return;
      }
    });

    return col;
  }

  bar.viz_getDataRequest = function(config={}) {
    let {
      previousDataRequest,
      previousDataResult,
      permitAutoTimeBucketing,
    } = config;

    /*
     * The behavior of this element is as follows:
     * Bars:
     *   x_shelf: <trellis_dimensions> <dimension>
     *   y_shelf: <trellis_dimensions> <aggregate>
     *   color_shelf: empty
     *
     * Horizontal Bars:
     *   x_shelf: <trellis_dimensions> <aggregate>
     *   y_shelf: <trellis_dimensions> <dimension>
     *   color_shelf: empty
     *
     * Bars colored by dimensions:
     *   x_shelf: <trellis_dimensions> <dimension>
     *   y_shelf: <trellis_dimensions> <aggregate>
     *   color_shelf: <dimension>
     *
     * Horizontal Bars colored by dimensions:
     *   x_shelf: <trellis_dimensions> <aggregate>
     *   y_shelf: <trellis_dimensions> <dimension>
     *   color_shelf: <dimension>
     *
     * Bars colored by aggregates:
     *   x_shelf: <trellis_dimensions> <dimension>
     *   y_shelf: <trellis_dimensions> <aggregate>
     *   color_shelf: <aggregate> [<aggregate> ...]
     *
     * Horizontal Bars colored by dimensions:
     *   x_shelf: <trellis_dimensions> <aggregate>
     *   y_shelf: <trellis_dimensions> <dimension>
     *   color_shelf: <aggregate> [<aggregate> ...]
     *
     * Note that the LAST shelf element on the X or Y axis has to be a dimension
     *   and the LAST shelf element on the other axis has to be an aggregate.
     *
     * If there are multiple aggregates on the X and Y axis, then they are all
     *   merged into a dimension that is a trellised measure name.
     * We then have to figure out whether this new trellised dimension goes on
     *   the X or Y axis. The way we decide this is:
     *   if all the measures are on the opposite axis to the dimension, then the
     *   trellis is on the opposite axis to the dimension.
     *   if any of the measures are on the same axis as the dimension, then the
     *   trellis is on the same axis as the dimension.
     *
     * Example 1:
     *   X axis (Bottom to Top): date
     *   Y axis (Bottom to Top): Record Count, Unique Vistors, Average Response Time
     *   This gives us vertical bars trellised on the Y axis
     *
     * Example 2:
     *   X axis (Bottom to Top): date, Average Response Time
     *   Y axis (Bottom to Top): Record Count, Unique Vistors
     *   This gives us vertical bars trellised on the X axis
     *
     * Example 3:
     *   X axis (Bottom to Top): date, Average Response Time, Unique Visitors
     *   Y axis (Bottom to Top): Record Count
     *   This gives us vertical bars trellised on the X axis
     *   (Note that this is identical to Example 2 above)
     *
     * Example 4:
     *   Y axis (Bottom to Top): date
     *   X axis (Bottom to Top): Record Count, Unique Vistors, Average Response Time
     *   This gives us horizontal bars trellised on the X axis
     *
     * Example 5:
     *   Y axis (Bottom to Top): date, Average Response Time
     *   X axis (Bottom to Top): Record Count, Unique Vistors
     *   This gives us horizontal bars trellised on the Y axis
     *
     * Example 6:
     *   Y axis (Bottom to Top): date, Average Response Time, Unique Visitors
     *   X axis (Bottom to Top): Record Count
     *   This gives us horizontal bars trellised on the Y axis
     *   (Note that this is identical to Example 2 above)
     */

    return lib.getCheckedDataRequest(constraints,
        {
          previousDataRequest,
          previousDataResult,
          permitAutoTimeBucketing,
          constraintsCb,
        });
  };

  function constraintsCb(req) {
    const columns = req.columns();

    let {
      x_is_dim,
      y_is_dim,
    } = columnsToXY(columns);

    if (x_is_dim === y_is_dim) {
      lib.alertMessage("Please specify one dimension and one " +
          "aggregate within the X and Y shelves");
      return undefined;
    }

    if (!getAggCols(columns, x_is_dim).length) {
      return undefined;
    }

    return req;
  }

  /*
   * Set the data this object should use to draw out the visual
   */
  bar.viz_setDataResult = function(result) {
    data_result = result;
    return bar;
  };

  function columnsToXY(columns) {
    let x_cols = lib.shelf_columns(columns, x_shelf),
        y_cols = lib.shelf_columns(columns, y_shelf),
        x_dims = lib.columnsOfType(x_cols, arc.report.columnTypeEnum.DIMENSION),
        y_dims = lib.columnsOfType(y_cols, arc.report.columnTypeEnum.DIMENSION),
        x_aggs = lib.columnsOfType(x_cols, arc.report.columnTypeEnum.AGGREGATE),
        y_aggs = lib.columnsOfType(y_cols, arc.report.columnTypeEnum.AGGREGATE),
        x_last,
        y_last,
        x_is_dim,
        y_is_dim;

    if (x_cols.length > 0) {
      x_last = largestShelfIndex(x_cols);
      x_is_dim = (x_last.colproperty() === arc.report.columnTypeEnum.DIMENSION);
    }

    if (y_cols.length > 0) {
      y_last = largestShelfIndex(y_cols);
      y_is_dim = (y_last.colproperty() === arc.report.columnTypeEnum.DIMENSION);
    }

    return {
      x_cols,
      y_cols,
      x_dims,
      y_dims,
      x_aggs,
      y_aggs,
      x_last,
      y_last,
      x_is_dim,
      y_is_dim,
    };
  }

  /*
   * In cases where we are plotting multiple measures in a trellis, we use this function
   * to convert data that looks like this:
   *    YEAR    STATE     SUM(POPULATION)      AVG(AGE)      AVG(YEARS_EDUCATION)
   *    1990       AZ                 355            32                        18
   *    1990       CA                 455            28                        17
   *
   * into data that looks like this:
   *    YEAR    STATE                MEASURE         VALUE
   *    1990       AZ        SUM(POPULATION)           355
   *    1990       AZ               AVG(AGE)            28
   *    1990       AZ   AVG(YEARS_EDUCATION)            18
   *    1990       CA        SUM(POPULATION)           455
   *    1990       CA               AVG(AGE)            28
   *    1990       CA   AVG(YEARS_EDUCATION)            17
   *
   * Then, we mark MEASURE as a dimension. This makes the trellising work
   *
   * Note that the LAST shelf element on the X or Y axis has to be a dimension
   *   and the LAST shelf element on the other axis has to be an aggregate.
   * If there are multiple aggregates on the X and Y axis, then they are all
   *   merged into a dimension that is a trellised measure name, and that
   *   new dimension is placed on the shelf with the dimension if that shelf
   *   also has measures, or on the other shelf otherwise.
   *
   */
  function wrangleMeasureTrellisData(config) {
    if (combined_visual) {
      return;
    }

    let columns = data_result.columns();

    const {
      x_cols,
      y_cols,
      x_dims,
      y_dims,
      x_aggs,
      y_aggs,
      x_last,
      y_last,
      x_is_dim,
      y_is_dim,
    } = columnsToXY(columns);

    if (x_is_dim === y_is_dim) {
      // This is an error condition - cannot proceed with any measure trellising
      return;
    }

    let xyAggCols = x_aggs.slice(0);
    y_aggs.forEach(function(c) {
      xyAggCols.push(c);
    });

    if (xyAggCols.length <= 1) {
      return;
    } else {
      config.measureTrellis = xyAggCols;
    }

    let otherDims = columns.filter(function(c) {
      if (c.colproperty() !== arc.report.columnTypeEnum.DIMENSION) {
        return;
      }

      let shelf = c.colsource();
      return ((shelf !== x_shelf) && (shelf !== y_shelf));
    });

    let otherAggs = columns.filter(function(c) {
      if (c.colproperty() !== arc.report.columnTypeEnum.AGGREGATE) {
        return;
      }

      let shelf = c.colsource();
      return ((shelf !== x_shelf) && (shelf !== y_shelf));
    });

    var aggColStart = xyAggCols[0].colindex(),
        aggColEnd = xyAggCols[xyAggCols.length - 1].colindex();

    let measureShelfSource;
    let measureDimIdx;

    if (x_is_dim) {
      if (x_aggs.length === 0) {
        measureShelfSource = y_shelf;
      } else {
        measureShelfSource = x_shelf;
      }
    } else {
      // y_is_dim === true
      if (y_aggs.length === 0) {
        measureShelfSource = x_shelf;
      } else {
        measureShelfSource = y_shelf;
      }
    }

    if (measureShelfSource === x_shelf) {
      if (x_is_dim) {
        measureDimIdx = x_dims.length - 1;
      } else {
        measureDimIdx = x_dims.length;
      }
    } else {
      if (y_is_dim) {
        measureDimIdx = x_dims.length + y_dims.length - 1;
      } else {
        measureDimIdx = x_dims.length + y_dims.length;
      }
    }

    let measureAggIdx = x_dims.length + y_dims.length + otherDims.length + 1;

    config.measureColumnExtra =
        arc.data.column()
           .colname('Measure')
           .coltype('STRING')
           .colproperty(arc.report.columnTypeEnum.DIMENSION)
           .colsource(measureShelfSource)
           .shelfIndex(-1)
           .colindex(aggColStart);

    let columnsMap = [];
    columns.forEach(function(c) {
      const sourceIdx = c.colindex();
      columnsMap[sourceIdx] = sourceIdx;
    });

    xyAggCols.forEach(function(c) {
      columnsMap[c.colindex()] = -1;
    });

    for (let i = measureDimIdx; i < measureAggIdx - 1; ++i) {
      columnsMap[i] = columnsMap[i] + 1;
    }

    columnsMap[measureAggIdx] = -1;

    config.measureColumnExtra.colindex(measureDimIdx);

    var newColumns = [];
    columnsMap.forEach(function(targetIdx, sourceIdx) {
      if (targetIdx === -1) {
        return;
      }

      newColumns[targetIdx] = columns[sourceIdx];
    });
    newColumns[measureDimIdx] = config.measureColumnExtra;
    newColumns[measureAggIdx] = xyAggCols[0].cloneDataColumn();
    newColumns[measureAggIdx].colindex(measureAggIdx)
                             .colname('Value')
                             // Ensure this is tracked as the last shelf element
                             .shelfIndex(x_dims.length + y_dims.length)
                             .colsource(x_is_dim ? y_shelf : x_shelf);

    var newRows = [];
    data_result.rows().forEach(function(row) {
      let newRow = [];

      columnsMap.forEach(function(targetIdx, sourceIdx) {
        if (targetIdx === -1) {
          return;
        }

        newRow[targetIdx] = row[sourceIdx];
      });

      xyAggCols.forEach(function(c) {
        var n = newRow.slice();
        n[measureDimIdx] = c.colname();
        n[measureAggIdx] = row[c.colindex()];
        newRows.push(n);
      });
    });

    columns.forEach(function(c) {
      const sourceIdx = c.colindex();
      if (sourceIdx !== -1) {
        c.colindex(columnsMap[c.colindex()]);
      }
    });

    data_result.columns(newColumns);
    data_result.rows(newRows);
  }

  function cleanupHelpers() {
    currentHelpers.forEach(function(helper) {
      helper.cleanup();
    });

    currentHelpers = [];
  }

  bar.viz_destroy = function() {
    cleanupHelpers();
  };

  bar.viz_drawVisual = function(viz_div, isWindowResize) {
    lib.chart_div(viz_div);
    lib.undraw();
    cleanupHelpers();

    previousDataTracker.enableOnRowCount(data_result.rows().length);

    if (data_result.rows().length === 0) {
      // If we are moving from vertical to horizontal bars,
      // cleanup previously tracked data for transtions. If
      // we do not do this, the transitions don't look natural.
      previousDataTracker.cleanup();
      return bar;
    }

    lib.unselectOnClick();

    var config = {};
    // Note that wrangleMeasureTrellisData may pivot the data, and may add a property
    // to config, so config should be initialized before this
    wrangleMeasureTrellisData(config);

    config.rows = data_result.rows();

    if (!bar.init_viz(config) ||
        !bar.setup_colors(config) ||
        !bar.setup_dims_aggs(config)) {
      previousDataTracker.cleanup();
      return bar;
    }

    if (config.x_is_dim !== last_x_is_dim) {
      previousDataTracker.cleanup();
    }

    last_x_is_dim = config.x_is_dim;

    var viz_settings = {
      legend_contextmenu_shown: false,
      show_cb: [],
      hide_cb: []
    };

    lib.filterDateNulls(config);

    let trellis = bar.setupTrellis(viz_div, config, viz_settings);
    if (trellis.error_state()) {
      previousDataTracker.cleanup();
      return bar;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      bar.viz_drawVisual(viz_div);
    });

    if (!config.color_enabled) {
      if (config.x_is_dim) {
        config.isWaterfall = waterfall();
      }
    }

    bar.organize_cells(config, config.rows);
    if (!scale_dim_independent().isChecked) {
      if (config.trellis_cell_groups) {
        const baseRowsTrellis = config.trellis_cell_groups[0] ?
                                config.trellis_cell_groups[0].data :
                                config.trellis_cell_groups[0];

        sortStackedBarsOnAggregate(config, config.rows, baseRowsTrellis);
      } else {
        sortStackedBarsOnAggregate(config, config.rows);
      }
    }

    bar.get_unique_dims(config, config.rows);
    bar.compute_dim_maxvalues(config, config.rows);

    if (legend_style()?.legendRequired) {
      // This happens now so we get the legend_width() set on the
      // config.trellis object. We need that to compute the width
      // permitted for visuals correctly
      config.trellis.reverse_legend_order(reverse_legend_order().value());
      config.trellis.legend_width_override(legend_width_override().value());
      config.trellis.draw_legend();
    }

    bar.compute_cell_size(config, viz_div);
    if (!scale_dim_independent().isChecked) {
      bar.compute_dim_scale(config, config.rows);
    }
    if (!scale_agg_independent().isChecked) {
      bar.compute_agg_scale(config, config.rows);
      bar.compute_percentages(config, config.rows);
      bar.compute_agg_domain(config, config.rows);
    }

    previousDataTracker.trellis(config.trellis);

    config.trellis.draw_visuals(function(cell_sel, xy_rows, xmeasure, ymeasure, idx) {
      if (!xy_rows.length) {
        return bar;
      }

      if (ignoreStackedNegativesWithMixedMeasures()) {
        if (config.is_stacked && config.color_enabled && !config.color_agg_cols) {
          xy_rows = filterOutMixedNegatives(config, xy_rows);
        }
      }

      if (config.isWaterfall) {
        xy_rows.push(totalRow(config, xy_rows));
      }

      computeMeasureTrellis(config, xy_rows);

      if (scale_dim_independent().isChecked) {
        sortStackedBarsOnAggregate(config, xy_rows);
        bar.get_unique_dims(config, xy_rows);
        bar.compute_dim_maxvalues(config, xy_rows);
        bar.compute_dim_scale(config, xy_rows);
      }

      if (scale_agg_independent().isChecked) {
        bar.compute_agg_scale(config, xy_rows);
        bar.compute_percentages(config, xy_rows);
        bar.compute_agg_domain(config, xy_rows);
      }

      bar.compute_axes(config);
      bar.draw_base_visual(config, cell_sel, xy_rows, viz_settings, idx);
    });

    config.trellis.tooltips('trellis');

    const isCanvasType = !!helper;
    if (isCanvasType) {
      helper.adjustTrellisHighlight();
    } else {
      config.trellis.addLegendHighlights();
    }

    lib.redrawLegendIfNeeded(config.trellis);
    return bar;
  };

  function filterOutMixedNegatives(config, rows) {
    let hasPositive,
        hasNegative;

    rows.some(function(row) {
      let a = config.agg_getter(row);
      if (a > 0) {
        hasPositive = true;
      } else if (a < 0) {
        hasNegative = true;
      }

      if (hasPositive && hasNegative) {
        return true;
      }

      return false;
    });

    let removalCnt = 0;
    if (hasPositive && hasNegative) {
      console.log('Aggregates have positive and negative values in a stacked bar. ' +
          'Removing negative values');
      rows = rows.filter(function(row) {
        let a = config.agg_getter(row);
        if (a < 0) {
          ++removalCnt;
          return false;
        }

        return true;
      });
    }

    return rows;
  }

  bar.setupTrellis = function (viz_div, config, viz_settings) {
    var y_shelf_reserved = 1;

    if (combined_visual) {
      y_shelf_reserved = lib.shelf_columns(config.columns, y_shelf).length;
    }

    let xIsMeasureTrellis, yIsMeasureTrellis;

    if (config.measureColumnExtra) {
      sort_orders_asc.Measure = 'none';
      if (config.x_is_dim) {
        yIsMeasureTrellis = true;
      }

      if (config.y_is_dim) {
        xIsMeasureTrellis = true;
      }
    }

    return arc.visual.trellis()
                     .x_shelf_name(x_shelf)
                     .y_shelf_name(y_shelf)
                     .x_shelf_reserved(1)
                     .y_shelf_reserved(y_shelf_reserved)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div(viz_div)
                     .viz_div_sel(d3.select("#" + viz_div))
                     .legend_column(config.color_column,
                         true /* color_is_dim */)
                     .legends(config.colors_domain)
                     .legendPositioning(legend_style()?.value?.())
                     .color(config.color)
                     .legend_show_cb(function(contextmenu, params) {

                      // Don't highlight corresponding color on graph if graph is
                      // bar or area graph. Otherwise, segments get pushed down
                       if (config.curr_mark_type === 'line') {
                         viz_settings.show_cb.forEach(function(cb) {
                           cb(contextmenu, params);
                         });
                         viz_settings.legend_contextmenu_shown = true;
                       }
                     })
                     .legend_hide_cb(function(contextmenu, params) {
                       viz_settings.legend_contextmenu_shown = false;
                       viz_settings.hide_cb.forEach(function(cb) {
                         cb();
                       });
                     })
                     .sortColorLegend(!config.color_agg_cols)
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       bar.allowLargeTrellisTemp(true);
                       bar.viz_drawVisual(viz_div);
                     })
                     .xIsMeasureTrellis(xIsMeasureTrellis)
                     .yIsMeasureTrellis(yIsMeasureTrellis)
                     .otherTrellisSettings([scale_dim_independent(), scale_agg_independent()])
                     .setup();
  };

  bar.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;

    if (isHistogram) {
      if (markerLib && markerLib.viz()) {
        markerLib.viz().allowLargeTrellisPerm(_);
      }
    }

    return bar;
  };

  bar.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;

    if (isHistogram) {
      if (markerLib && markerLib.viz()) {
        markerLib.viz().allowLargeTrellisTemp(_);
      }
    }

    return bar;
  };

  bar.isHistogram = function() {
    return isHistogram;
  };

  bar.histogramConfig = function(config) {
    isHistogram = config !== undefined;
    histogramConfig = config;
    return bar;
  };

  function getTrellisColumn(config, row) {
    var targetColindex = config.measureTrellis[1].colindex(),
        colname = row[targetColindex - 1],
        targetCol;

    config.measureTrellis.forEach(function(c) {
      if (c.colname() === colname) {
        targetCol = c.cloneDataColumn();
        return true;
      }
    });

    return targetCol;
  }

  function computeMeasureTrellis(config, rows) {
    if (config.measureTrellis === undefined) {
      return;
    }

    if (rows.length === 0) {
      return;
    }

    var targetCol = getTrellisColumn(config, rows[0]);
    if (targetCol) {
      config.columns[targetCol.colindex()] = targetCol;

      if (config.x_is_dim) {
        config.y_col = targetCol;
      } else {
        config.x_col = targetCol;
      }
    }
  }

  bar.compute_axes = function(config) {

    //Note dim_is_date is passed into the coldisplayformat function because for axis visuals with date axis
    //we want to not get back a default format if there has not been one explicitly set.
    var yDisplayFormat = combined_visual ? config.coldisplayformat :
                                           config.y_col.coldisplayformat(config.y_is_dim && config.dim_is_date);
    config.x_axis = arc.visual.axis()
                       .tickFormat(config.x_col.coldisplayformat(config.x_is_dim && config.dim_is_date))
                       .scale(config.x_scale)
                       .is_dim(config.x_is_dim)
                       .is_date(config.x_is_dim && config.dim_is_date)
                       .is_datetime(config.x_is_dim && config.dim_is_datetime)
                       .is_log10(
                           config.x_is_dim ? false:
                                             config.agg_is_log10)
                       .is_percentage(
                           config.x_is_dim ? false:
                                             config.agg_is_percentage)
                       .colname(config.x_col.colname())
                       .flip_ticks(
                           config.x_is_dim ? false :
                                             config.color_enabled && config.curr_mark_type !== 'groupedbar')
                       .coldesc(config.x_col.coldesc());
    if (isHistogram) {
      config.x_axis.ticks(histogramConfig.bucketCount);
    }

    config.y_axis = arc.visual.axis()
                       .tickFormat(yDisplayFormat)
                       .scale(config.y_scale)
                       .is_dim(config.y_is_dim)
                       .is_date(config.y_is_dim && config.dim_is_date)
                       .is_datetime(config.y_is_dim && config.dim_is_datetime)
                       .is_log10(
                           config.y_is_dim ? false:
                                             config.agg_is_log10)
                       .is_percentage(
                          config.y_is_dim ? false:
                                            config.agg_is_percentage)
                       .colname(config.y_col.colname())
                       .coldesc(config.y_col.coldesc());

    if ((mark_type === 'groupedbar' || mark_type === 'line') && !combined_visual) {
        config.agg_scale_secondary = config.y_scale_secondary;
      }

    if (config.has_secondary_agg_cols) {
      var secondaryDisplayFormat = combined_visual ?
                                   config.secondary_coldisplayformat :
                                   config.secondary_y_col.coldisplayformat();
      config.secondary_y_axis =
          arc.visual.axis()
                    .tickFormat(secondaryDisplayFormat)
                    .scale(config.agg_scale_secondary)
                    .is_dim(false)
                    .is_date(false)
                    .is_log10(config.agg_is_log10)
                    .is_percentage(config.agg_is_percentage)
                    .colname(config.secondary_y_col.colname());
    }

    return bar;
  };

  bar.compute_dim_scale = function(config, rows, xpos, ypos) {
    return bar._compute_scales(config, rows, "dim");
  };

  bar.compute_agg_scale = function(config, rows) {
    return bar._compute_scales(config, rows, "agg");
  };

  bar.agg_logscale = function() {
    return (setting_scale().value() === 'LOG10');
  };

  function agg_zero_scale() {
    if ((scale_agg_zero().isChecked) &&
        (setting_scale().value() !== 'LOG10')) {
      return true;
    }

    return false;
  }

  function getAggColIndices(config) {
    return config.color_secondary_agg_cols.map((col) => col.colindex());
  }

  // Calculate bar_padding
  // config needs to have width and max_slots properties
  bar._computeDynamicDimPadding = function (config) {
    const numberOfBars = (scale_dim_independent().isChecked && !makeDimDynamic) ?
      config.max_slots :
      config.max_slots + 1;
    const width_for_each_bar = config.width / numberOfBars;
    const max_bar_width = bar_mark_size() && bar_mark_size().get_range()[1];
    const bar_percent_of_width = max_bar_width / width_for_each_bar;
    const space_percent_of_width = 1 - bar_percent_of_width;

    return {
      space_percent_of_width,
      width_for_each_bar,
      max_bar_width,
    };
  };

  bar._compute_scales = function(config, rows, dim_or_agg) {
    function setHistogramConfig() {
      const row = rows[0];
      let startingObj = histogramConfig;

      if (scale_dim_independent().isChecked) {
        for (let i = 0; i < config.dim_start_idx; i++) {
          startingObj = startingObj[row[i]];
        }
      }

      cellSpecificHistConfig = startingObj;
      return cellSpecificHistConfig;
    }

    const isNumericContinuous = config.dimIsContinuous && config.dimIsNumeric;
    const padCalc = bar._computeDynamicDimPadding(config);
    if (((dim_or_agg === 'dim') && (config.x_is_dim)) ||
      ((dim_or_agg === 'agg') && (!config.x_is_dim))) {
      if (x_scale) {
        config.x_scale = x_scale;
      } else {
        // Set padding between bars based if makeDimDynamic is checked
        let bar_padding;
        if (config.x_is_dim && makeDimDynamic) {
          bar_padding = padCalc.space_percent_of_width;
          if (bar_padding < 0) {
            bar_padding = 0;
          }

          config.dynamic_bar_width = Math.min(padCalc.width_for_each_bar, padCalc.max_bar_width);
        }

        let canvasWidth = config.width;

        /**
         * If it's a line chart and the labels are visible at the right of each line,
         * I'd like to decrease the width of the canvas with the size of the longest
         * text in the list.
         */
        if (bar.mark_type() === 'line' && Array.isArray(config.colors_domain)) {
          const settings = lib.user_settings();
          const enableLineLabelsSetting = settings.find_by_id('enable_line_labels');
          if (enableLineLabelsSetting && enableLineLabelsSetting.isChecked) {
            const [longestLabelWidth] = arc.utils.findLongestText(config.colors_domain, {
              font: lib.getVisualFont(),
              fontSize: lib.getLabelFontSize(),
            }) || [0];
            if (longestLabelWidth) {
              const maximumLabelWidth = settings.find_by_id('maximum_label_width');
              canvasWidth -= Math.min(longestLabelWidth, maximumLabelWidth.value());
            }
          }
        }

        config.x_scale = lib.idx_to_scale_config({
          bar_padding,
          rows,
          idx: config.x_idx,
          range:
            (config.x_is_dim ||
             (!config.color_enabled && config.curr_mark_type === 'bar') ||
             config.curr_mark_type === 'groupedbar') ? [0, canvasWidth] : [canvasWidth, 0],
          is_date: config.x_is_dim ? config.dim_is_date : false,
          is_datetime: config.x_is_dim ? config.dim_is_datetime : false,
          date_format: config.date_format,
          is_dim: isHistogram ? false : config.x_is_dim,
          is_log10scale: config.agg_is_log10,
          sort_dim: config.trellis.is_active() && !config.has_sort_order,
          agg_scale_zero: isHistogram ? false : agg_zero_scale(),
          minScale: config.x_is_dim ? getDimScaleNumericMin : bar.getAggScaleMin,
          maxScale: config.x_is_dim ? getDimScaleNumericMax : bar.getAggScaleMax,
          isNumericContinuous,
          isWaterfall: config.isWaterfall,
          width_for_each_bar: config.x_is_dim && config.curr_mark_type === 'bar' &&
            Math.min(padCalc.width_for_each_bar, padCalc.max_bar_width),
          domain: !isHistogram ? undefined : setHistogramConfig().scaleDomain,
        });
      }
    }

    const yScaleConfig = {
      idx: config.y_idx,
      range: config.y_is_dim ? [0, config.height] : [config.height, 0],
      is_date: config.y_is_dim ? config.dim_is_date : false,
      is_datetime: config.y_is_dim ? config.dim_is_datetime : false,
      date_format: config.date_format,
      is_dim: config.y_is_dim,
      is_log10scale: config.agg_is_log10,
      sort_dim: config.trellis.is_active() && !config.has_sort_order,
      agg_scale_zero: agg_zero_scale(),
      minScale: config.y_is_dim ? getDimScaleNumericMin : bar.getAggScaleMin,
      maxScale: config.y_is_dim ? getDimScaleNumericMax : bar.getAggScaleMax,
      isSecondaryAxis: config.isSecondaryAxis,
      isWaterfall: config.isWaterfall,
      width_for_each_bar: config.curr_mark_type === 'bar' ? config.dim_sz : undefined,
      isNumericContinuous,
      rows,
    };

    if (((dim_or_agg === 'dim') && (config.y_is_dim)) ||
      ((dim_or_agg === 'agg') && (!config.y_is_dim))) {
      if (y_scale) {
        config.y_scale = y_scale;
      } else {
        config.y_scale = lib.idx_to_scale_config(yScaleConfig);
      }

      if (config.has_secondary_agg_cols && getCanHaveSecondaryAxis(mark_type)) {
        if (y_scale_secondary) {
          config.y_scale_secondary = y_scale_secondary;
        } else {
          config.y_scale_secondary = lib.idx_to_scale_config(
            {
              ...yScaleConfig,
              isSecondaryAxis: true,
              idx: getAggColIndices(config),
              range: config.secondary_y_is_dim ? [0, config.height] : [config.height, 0],
              is_date: config.secondary_y_is_dim ? config.dim_is_date : false,
              is_datetime: config.secondary_y_is_dim ? config.dim_is_datetime : false,
              is_dim: config.secondary_y_is_dim,
              minScale: config.secondary_y_is_dim ? getDimScaleNumericMin : bar.getAggScaleMin,
              maxScale: config.secondary_y_is_dim ? getDimScaleNumericMax : bar.getAggScaleMax,
            }
          );
        }
      }
    }

    if ((config.dim_is_datetime || isNumericContinuous) && config.curr_mark_type === 'bar' &&
      dim_or_agg === 'dim') {
      let minDistance = config.x_is_dim ?
            Math.min(padCalc.width_for_each_bar, padCalc.max_bar_width): config.dim_sz;
      let absoluteMin = bar_mark_size() && bar_mark_size().get_range()[0];
      let scale = config.x_is_dim ? config.x_scale : config.y_scale;
      let dimPositions = config.rows.map(row => scale(row[config.dim_start_idx]));
      dimPositions.sort((a, b) => a - b);
      for (let idx = 1; idx < dimPositions.length; idx++) {
        let currDistance = dimPositions[idx] - dimPositions[idx - 1];
        if (currDistance < minDistance) {
         if (currDistance < absoluteMin) {
           minDistance = absoluteMin;
         } else {
           minDistance = currDistance;
         }
        }
      }

      config.dim_sz = minDistance;
    }

    config.dim_scale = config.x_is_dim ? config.x_scale : config.y_scale;
    config.agg_scale = config.x_is_dim ? config.y_scale : config.x_scale;

    return bar;
  };

  function getYCol(config) {
    if (combined_visual) {
      return lib.first_column(config.columns, y_shelf);
    }

    let ycols = lib.shelf_columns(config.columns, y_shelf);
    return largestShelfIndex(ycols);
  }

  function getSecondaryYCol(config) {
    const y_cols = lib.shelf_columns(config.columns, y_shelf);
    const secondary_y_cols = lib.shelf_columns(config.columns, secondary_y_shelf);
    const all_y_cols = [...y_cols, ...secondary_y_cols];
    const all_y_cols_using_secondary_axis = all_y_cols.filter((col) => col.secondaryaxis());
    return largestShelfIndex(all_y_cols_using_secondary_axis);
  }

  function setAxisScaleSettings(isWithAllOptions) {
    if (isWithAllOptions) {
      setting_scale().enable();
      user_settings.find_by_id('secondary_min_agg_axis').disable();
      user_settings.find_by_id('secondary_max_agg_axis').disable();
    } else {
      setting_scale().value('LINEAR');
      setting_scale().disable();
      user_settings.find_by_id('secondary_min_agg_axis').enable();
      user_settings.find_by_id('secondary_max_agg_axis').enable();
    }
  }

  function handleSecondaryAxisCapableTypes(config) {
    config.secondary_y_col = getSecondaryYCol(config);
    if (hasSecondaryAxisInUse(config)) {
      const sec_y_colprop = config.secondary_y_col.colproperty();
      config.secondary_y_is_dim = (sec_y_colprop === arc.report.columnTypeEnum.DIMENSION);
      setAxisScaleSettings();
    } else if (!config.combined_visual) {
      setAxisScaleSettings(true);
    }
  }

  bar.init_viz = function(config) {
    maxWaterfallAgg = undefined;
    config.inlineMeasureDisplay = inlineMeasureDisplay();
    config.inlineSumMeasureDisplay = inlineSumMeasureDisplay();
    config.isWaterfall = false;
    config.curr_mark_type = mark_type;
    config.columns = data_result.columns();
    config.x_col = largestShelfIndex(lib.shelf_columns(config.columns, x_shelf));
    config.y_col = getYCol(config);

    if (getCanHaveSecondaryAxis(mark_type)) {
      handleSecondaryAxisCapableTypes(config);
    }

    // Return if there's nothing on y shelf (in combo visual) return false
    if (!config.y_col) {
      return false;
    }

    config.x_is_dim = (config.x_col.colproperty() === arc.report.columnTypeEnum.DIMENSION);
    config.y_is_dim = (config.y_col.colproperty() === arc.report.columnTypeEnum.DIMENSION);

    if ((config.x_is_dim && config.x_col.isNumeric()) ||
      (config.y_is_dim && config.y_col.isNumeric())) {
      config.dimIsNumeric = true;
      config.dimIsContinuous = dimIsContinuous.isChecked;
      dimIsContinuous.enable();
      if (dimIsContinuous.isChecked) {
        minDimNumeric.enable();
        maxDimNumeric.enable();
      }
    } else {
      config.dimIsNumeric = false;
      config.dimIsContinuous = false;
      dimIsContinuous.disable();
      minDimNumeric.disable();
      maxDimNumeric.disable();
    }

    if (config.x_is_dim === config.y_is_dim) {
      lib.alertMessage("Please specify one dimension and one " +
          "aggregate within the X and Y shelves");
      return undefined;
    }

    if (config.combined_visual) {
      config.filterShelf = y_shelf;
    }

    lib.filter_nulls_bar(config);

    // Store easily retrievable version of whether graph is dynamically sized
    var isDynamicallySized = !(mark_type === 'line' || mark_type === 'area') && config.x_is_dim;
    bar.makeDimDynamic(dynamic_bar_width() && dynamic_bar_width().value() === "true" &&
        isDynamicallySized);

    if (config.x_col.order() || config.y_col.order()) {
      config.presorted = true;
      if (config.x_col.order() && config.y_col.order()) {
        config.ordercol =
            config.x_col.order().priority() < config.y_col.order().priority() ?
              config.x_col : config.y_col;
      } else {
        if (config.x_col.order()) {
          config.ordercol = config.x_col;
        } else {
          config.ordercol = config.y_col;
        }
      }
    }
    return bar;
  };

  function getColorShelfCols(config) {
    if (combined_visual) {
      var cols = lib.shelf_columns(config.columns, y_shelf);
      cols = cols.slice(1, cols.length);
      return cols;
    }

    return lib.shelf_columns(config.columns, "color_shelf");
  }

  bar.setup_base_colors = function(config) {
    config.color = lib.dimensionalColorScale();
  };

  bar.isLineStyleFromColor = function() {
    const setting = user_settings.find_by_id('line_style_from_color');
    return setting && setting.isChecked;
  };

  bar.updateLineStyleScale = function(config) {
    if (bar.isLineStyleFromColor() && config.rows) {
      config.lineStyleFromColor = true;
      config.lineStyleScale = d3.scale.ordinal();
      const colors = arc.misc.arrayUnique(config.rows,
        config.last_color_getter).sort();
      config.lineStyleScale.domain(colors);
      config.lineStyleScale.range([
        [100, 0],
        [4, 4],
        [8, 8],
        [12, 12],
        [2, 2],
        [6, 6],
        [10, 10],
        [14, 14],
      ]);
    }
  };

  bar.setup_colors = function(config) {
    var dim_cnt = 0,
        agg_cnt = 0;

    if (enableMultipleTooltips()) {
      config.multipleTooltips = true;
    }

    config.color_enabled = false;

    var cols = getColorShelfCols(config);

    if (cols.length === 0) {
      // Nothing in 'color_shelf'
      // Preserve status of lib.colors() since it changes for combined_visual
      var currentColors = lib.colors();
      config.color = function() {
        return currentColors[0];
      };

      config.palette = lib.graded_colors();

      return bar;
    }

    cols.forEach(function(col) {
      if (col.colproperty() === arc.report.columnTypeEnum.AGGREGATE) {
        ++agg_cnt;
        return;
      }

      ++dim_cnt;
    });

    if ((dim_cnt > 0) && (agg_cnt > 0)) {
      lib.alertMessage('Color shelf should contain dimensions or aggregates, ' +
          'not both.');
      return undefined;
    }

    bar.setup_base_colors(config);

    config.color_enabled = true;

    if (dim_cnt > 0) {
      // Dimensions in 'color_shelf'
      config.color_column = cols[0];
      config.last_color_column = cols[cols.length - 1];
      config.last_color_getter = function(d) {
        return d[config.last_color_column.colindex()];
      };

      bar.updateLineStyleScale(config);

      config.color_is_dim = config.color_column.colproperty() === arc.report.columnTypeEnum.DIMENSION;

      {
        let color_start_idx = config.color_column.colindex(),
            color_end_idx = config.last_color_column.colindex();

        if (bar.isLineStyleFromColor()) {
          if (color_end_idx > color_start_idx) {
            --color_end_idx;
          }
        }

        config.color_getter = function(d) {
          let r = [];
          for (let i = color_start_idx; i <= color_end_idx; ++i) {
            r.push(d[i]);
          }

          return r.join(', ');
        };

        if (bar.isLineStyleFromColor()) {
          let e = config.last_color_column.colindex();
          config.color_group_getter = function(d) {
            let r = [];
            for (let i = color_start_idx; i <= e; ++i) {
              r.push(d[i]);
            }

            return r.join(', ');
          };

          config.distinct_lines = arc.misc.arrayUnique(config.rows,
              config.color_group_getter);
          config.distinct_line_colors = config.distinct_lines.map(function(c) {
            let i = c.lastIndexOf(', ');
            return c.substring(0, i);
          });
        }
      }

      config.colors_domain = arc.visual.barColorHelper.determineColorDomain(config);

      /* START OF EBAY SPECIFIC CODE */
      var stack_names = config.colors_domain;
      if (stack_names.length === 3 &&
          stack_names.indexOf("fail") !== -1 &&
          stack_names.indexOf("pass") !== -1 &&
          stack_names.indexOf("soft_fail") !== -1) {

        config.colors_domain = ["fail", "soft_fail", "pass"];
      } else if (stack_names.length === 2 &&
          stack_names.indexOf("fail") !== -1 &&
          stack_names.indexOf("soft_fail") !== -1) {

        config.colors_domain = ["fail", "soft_fail"];
      }
      /* END OF EBAY SPECIFIC CODE */

      config.color.domain(config.colors_domain);
      if (lib.has_sort_order(config)) {
        config.unsorted_colors_domain = config.colors_domain.slice(0);
      } else {
        config.unsorted_colors_domain = config.colors_domain;
      }
      config.colors_domain.sort();
      return bar;
    }

    /*
     * Now we know there are aggregates in the color shelf, and that there is
     * at least one aggregate on X and Y (that check is made before the call to
     * this function)
     */
    config.color_agg_cols = [];
    if (config.x_is_dim) {
      config.color_agg_cols.push(config.y_col);
    } else {
      config.color_agg_cols.push(config.x_col);
    }

    arc.misc.arrayAdd(config.color_agg_cols, cols);

    // Compute primary and secondary axes
    //   Note that we only permit secondary axes for line charts so there
    //   is a test for mark_type below. Note also that even if we don't need
    //   a secondary axis, config.aggs_primary_getter is used to track elements
    //   that get plotted on the primary axis.
    config.color_primary_agg_cols = [];
    config.color_secondary_agg_cols = [];

    config.color_agg_cols.forEach(function(col) {
      if (config.x_is_dim) {
        if ((mark_type === 'line' || mark_type === 'groupedbar') && col.secondaryaxis()) {
          config.color_secondary_agg_cols.push(col);
        } else {
          config.color_primary_agg_cols.push(col);
        }
      } else {
        config.color_primary_agg_cols.push(col);
      }
    });

    // Must have at least one aggregate for the primary axis.
    // If all aggregates are treated as being on the secondary axis
    if (config.color_primary_agg_cols.length === 0) {
      config.color_primary_agg_cols = config.color_secondary_agg_cols;
      config.color_secondary_agg_cols = [];
    }

    // Track whether we have a secondary axis.
    config.has_secondary_agg_cols = config.color_secondary_agg_cols.length > 0;

    config.colnames_to_idx = {};
    config.color_agg_cols.forEach(function(col) {
      config.colnames_to_idx[col.colname()] = col.colindex();
    });

    config.colors_domain = config.color_agg_cols.map(
      function(col) {
        return col.colname();
      });

    config.unsorted_colors_domain = config.colors_domain.slice();
    config.color.domain(config.colors_domain);
    config.aggs_getter = function(row) {
      return config.color_agg_cols.map(col => Number(row[col.colindex()]));
    };

    config.aggs_primary_getter = function(row) {
      return config.color_primary_agg_cols.map(col => Number(row[col.colindex()]));
    };

    config.aggs_secondary_getter = function(row) {
      return config.color_secondary_agg_cols.map(col => Number(row[col.colindex()]));
    };

    config.orig_aggs_getter = config.aggs_getter;
    return bar;
  };

  function sortStackedBarsOnAggregate(config, rowsToSort, baseRows) {
    if (!config.is_stacked) {
      return;
    }

    if (!baseRows) {
      baseRows = rowsToSort;
    }

    // We are going to stack bars or an area chart

    // Is the incoming data expected to be sorted? If not, there is nothing
    // to do
    if (!lib.shelves()) {
      return;
    }

    // Is there anything ordered?
    const orderCols = lib.shelves().getOrder();

    if (orderCols.length === 0) {
      return;
    }

    // Find the first order column
    const firstOrderCol = orderCols.find((col) => col.order().priority() === 1);

    // If the first order column is not the aggregate on the X/Y shelf,
    // then do not re-sort columns.
    // This way, if someone sorts on ascending hour, that stacked bar does
    // not change to sort on the aggregate.
    const shelf = firstOrderCol.shelf();
    if (shelf.display_name() === 'X Axis') {
      if (config.x_is_dim) {
        return;
      }
    } else if (shelf.display_name() === 'Y Axis') {
      if (config.y_is_dim) {
        return;
      }
    } else {
      return;
    }

    // If it is expected to be sorted by the aggregate, sort by the
    // combination of values that are stacked, not by the individual measures.
    // This requires a re-sort now.
    const asc = firstOrderCol.order().ascending();
    // Calculate the aggregates by dimension
    const dim_totals = {};
    baseRows.forEach((row) => {
      const dim = config.dim_getter(row);
      let a = dim_totals[dim] || 0;
      a += config.agg_getter(row);
      dim_totals[dim] = a;
    });

    // And then re-sort
    rowsToSort.sort((a, b) => {
      let a1 = dim_totals[config.dim_getter(a)];
      let a2 = dim_totals[config.dim_getter(b)];
      if (asc) {
        const tmp = a1;
        a1 = a2;
        a2 = tmp;
      }

      if (a1 > a2) {
        return -1;
      }

      if (a1 < a2) {
        return 1;
      }

      return 0;
    });
  }

  bar.setup_dims_aggs = function(config) {
    config.dim_start_idx = config.x_is_dim ? config.x_col.colindex() :
                                             config.y_col.colindex();
    config.dimCol = config.columns[config.dim_start_idx];
    config.dim_orig_start_idx = config.dim_start_idx;
    if (!config.color_enabled) {
      // A grouped bar without a color is a bar
      config.colors_domain = undefined;
      if (config.curr_mark_type === 'groupedbar') {
        config.curr_mark_type = 'bar';
      }
    }

    if (['bar', 'area'].indexOf(config.curr_mark_type) !== -1) {
      config.is_stacked = true;
    } else {
      config.is_stacked = false;
    }

    if (['line', 'area'].indexOf(config.curr_mark_type) !== -1) {
      config.mark_is_continuous = true;
    } else {
      config.mark_is_continuous = false;
    }

    var agg_cols = getAggCols(config.columns, config.x_is_dim);
    if (!agg_cols.length) {
      return undefined;
    }
    config.agg_start_idx = agg_cols[0].colindex();

    config.agg_is_percentage = false;
    config.agg_is_log10 = false;
    if (setting_scale().value() === 'LOG10') {
      config.agg_is_log10 = true;
    }
    if ((setting_scale().value() === 'PERCENTAGE') ||
        (setting_scale().value() === 'PERCENTAGE_COLOR')) {
      config.agg_is_percentage = true;
    }

    config.dim_getter = function(d) {
      return d[config.dim_start_idx];
    };
    config.agg_getter = function(d) {
      // Use parseFloat instead of Number(num) so null values return NaN instead of 0
      return parseFloat(d[config.agg_start_idx]);
    };
    config.orig_agg_getter = config.agg_getter;

    if (config.agg_is_log10) {
      var a_min;
      if (config.color_agg_cols) {
        // Need to get minimum for every aggregate getter if multiple aggs
        a_min = config.rows.reduce(getMin, undefined);
      } else {
        a_min = d3.min(config.rows, config.agg_getter);
      }
      if (a_min <= 0) {
        lib.alertMessage("Axis Scale is set to Log 10, but data " +
                        "contains values <= 0 which is not valid for " +
                        "log scale. Please change the Axis Scale " +
                        "in Settings for this app.");
        return undefined;
      }
    }

    return bar;

    function getMin(prev, row) {
      let minValue;
      for (let i = 0; i < config.color_agg_cols.length; i++) {
        const colidx = config.color_agg_cols[i].colindex(),
              currentVal = row[colidx];
        if (minValue) {
          minValue = Math.min(currentVal, minValue);
        } else {
          minValue = currentVal;
        }
      }
      if (prev) {
        return Math.min(prev, minValue);
      } else {
        return minValue;
      }
    }
  };

  bar.compute_cell_size = function(config, viz_div) {
    var d_minw,
        d_maxw,
        barSizeRange = bar_mark_size() && bar_mark_size().get_range(),
        extra_margin = {};
    if (barSizeRange && !makeDimDynamic &&
        (['bar', 'groupedbar'].indexOf(config.curr_mark_type) !== -1)) {
      d_minw = barSizeRange[0] * config.max_slots;

      if (config.dim_is_datetime) {
        d_maxw = 0;
      } else {
        d_maxw = barSizeRange[1] * config.max_slots;
      }

      const canvasPadding = 200;
      d_minw = Math.min(maxDimWidth - canvasPadding, d_minw);
      d_maxw = Math.min(maxDimWidth - canvasPadding, d_maxw);
    } else if (barSizeRange && makeDimDynamic) {
      d_minw = barSizeRange[0] * config.max_slots;
    } else {
      // ((config.curr_mark_type == 'line') ||
      //  (config.curr_mark_type == 'area'))
      d_minw = 0;
      d_maxw = 0;
    }

    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      includeLegend: config.color_enabled && legend_style()?.legendRequired,
      trellis: config.trellis,
      viz_div,
    });

    var x_is_date = config.dim_is_date && config.x_is_dim;

    var min_width = 0,
        max_width = 0;
    if (isHistogram) {
      var singleRange = bar_mark_size().get_range();
      min_width = histogramConfig.bucketCount * singleRange[0];
      max_width = histogramConfig.bucketCount * singleRange[1];
    } else if (config.x_is_dim) {
      min_width = d_minw;
      max_width = d_maxw;
    }

    if (min_width < minCellWidth()) {
      min_width = minCellWidth();
    }

    let min_height = config.y_is_dim ? d_minw : 0,
        max_height = config.y_is_dim ? d_maxw : 0;
    if (min_height < minCellHeight()) {
      min_height = minCellHeight();
    }

    lib.dim_tick_label_maxlen(Number(dim_tick_label_maxlen().value()));
    const svgConfig = {
      width: elem_w,
      height: elem_h,
      show_x_axis: show_x_axis().isChecked,
      show_y_axis: show_y_axis().isChecked,
      extra_margin: extra_margin,
      columns: config.columns,
      x_idx: config.x_col.colindex(),
      min_width: min_width,
      max_width: max_width,
      min_height: min_height,
      max_height: max_height,
      rows: config.rows,
      x_is_dim: config.x_is_dim,
      x_is_slanted: xIsSlanted(config),
      y_is_dim: config.y_is_dim,
      y_idx: combined_visual ? config.primary_y_col.colindex() : config.y_col.colindex(),
      y_all_indices: config.y_all_indices,
      y_second_all_indices: config.y_second_all_indices,
      x_axis_label_off: (x_axis_label().value() === 'false'),
      y_axis_label_off: (y_axis_label().value() === 'false'),
      x_formatter: isHistogram ? undefined : config.x_col.coldisplayformat(),
      y_formatter: combined_visual ? config.primary_y_col.coldisplayformat() :
                                     config.y_col.coldisplayformat(),
      secondary_idx: config.has_secondary_agg_cols ?
                     config.color_secondary_agg_cols[0].colindex() :
                     undefined,
      secondary_formatter: config.has_secondary_agg_cols ?
                           config.color_secondary_agg_cols[0].coldisplayformat() :
                           undefined,

      // Handle cases where aggregates are stacked
      add_y_values: ['bar', 'area'].includes(config.curr_mark_type),
    };

    if (config.color_primary_agg_cols && config.x_is_dim && !svgConfig.y_all_indices) {
      svgConfig.y_all_indices = config.color_primary_agg_cols.map(col => col.colindex());
    }

    if (config.color_secondary_agg_cols && config.x_is_dim && !svgConfig.y_second_all_indices) {
      svgConfig.y_second_all_indices = config.color_secondary_agg_cols.map(col => col.colindex());
    }

    const result = lib.compute_svg_params(svgConfig);
    config.margin = result.margin;
    config.width = result.width;
    config.height = result.height;
    bar._compute_dim_sz(config, config.max_slots);
    return bar;
  };

  bar._compute_dim_sz = function(config, total_slots, independent_dim_group) {
    if (config.width) {
      // the size of the bars are calculated
      // For lines, it is always width / total_slots and is not dependent on user specified bar_mark_size
      const total_dim_width = config.x_is_dim ? config.width : config.height;
      let dim_width = total_dim_width / total_slots;
      if (['line', 'area', 'highlight'].indexOf(config.curr_mark_type) === -1) {
        config.original_dim_width = dim_width;

        let configuredSize = bar_mark_size();
        if (configuredSize) {
          let configuredSizeRange = configuredSize.get_range();

          dim_width = Math.min(configuredSizeRange[1], dim_width);
          config.minBarSz = configuredSizeRange[0];
        }

        config.minBarSz = 1;

        let maxBarSize = maxDimWidth / total_slots;
        config.minBarSz = Math.min(config.minBarSz, maxBarSize);
        dim_width = Math.min(dim_width, maxBarSize);
      }

      config.dim_sz = dim_width;

      // Only need to do this for grouped bars
      // Regular grouped bars: 2x the bar width when bar graph not dynamically sized
      // Dynamically sized bars: width / number of groups
      // TODO: Calculate different spacing for each trellis cell
      // (each cell could have different # of bars or groups)
      if (config.curr_mark_type === 'groupedbar') {
        if (makeDimDynamic) {
          let highest_number_bars = 0, highest_number_groups = 0;
          const getHighestNumber = function (cell_group) {
            var current_number_bars = cell_group.cell_bar_max,
                current_number_groups = cell_group.x_groups.length;
            if (current_number_bars > highest_number_bars) {
              highest_number_bars = current_number_bars;
            }
            if (current_number_groups > highest_number_groups) {
              highest_number_groups = current_number_groups;
            }
          };

          if (independent_dim_group) {
            getHighestNumber(independent_dim_group);
          } else {
            config.trellis_cell_groups.forEach(getHighestNumber);
          }

          var width_of_bars = config.dim_sz * highest_number_bars,
              number_of_spaces = highest_number_groups + 1;
          config.group_spacing = (config.width - width_of_bars)/number_of_spaces;
        } else {
          config.group_spacing = config.dim_sz * 2;
        }
      }
    }

    return bar;
  };

  bar.getAggScaleMin = function(min, isSecondaryAxis) {
    if (bar.agg_logscale()) {
      return min;
    }

    /*
     * Note that in this implementation, a set minimum overrides
     * the agg_zero_scale setting
     */
    const m = isSecondaryAxis ? secondary_min_agg_axis().value() : min_agg_axis().value();

    if (m !== '') {
      if (!isNaN(m)) {
        return Number(m);
      }
    }

    if (agg_zero_scale()) {
      if (min > 0) {
        return 0;
      }
    }

    return min;
  };

  bar.getAggScaleMax = function(max, isSecondaryAxis) {
    if (maxWaterfallAgg) {
      max = Math.max(max, maxWaterfallAgg);
    }

    if (bar.agg_logscale()) {
      return max;
    }

    const m = isSecondaryAxis ? secondary_max_agg_axis().value() : max_agg_axis().value();
    if (m !== '') {
      if (!isNaN(m)) {
        return Number(m);
      }
    }

    if (agg_zero_scale()) {
      if (max < 0) {
        return 0;
      }
    }

    return max;
  };

  function getDimScaleNumericMax(max) {
    const setMax = maxDimNumeric.value();

    if (setMax !== '' && $.isNumeric(setMax)) {
      return Math.max(max, Number(setMax));
    }

    return max;
  }

  function getDimScaleNumericMin(min) {
    const setMin = minDimNumeric.value();

    if (setMin !== '' && $.isNumeric(setMin)) {
      return Math.min(min, Number(setMin));
    }

    return min;
  }

  bar.compute_agg_domain = function(config, rows) {
    var min,
        max,
        extent;
    if (config.color_enabled && !config.agg_is_percentage &&
        !config.is_stacked && config.color_agg_cols) {
      testAndRun(
        [
          { aggGetter: config.aggs_primary_getter, scale: config.agg_scale, run: true },
          { aggGetter: config.aggs_secondary_getter, run: config.has_secondary_agg_cols }
        ],
        function functionName(v, i) {
          if (i === 1 /*is secondary axis*/) {
            v.scale = config.agg_scale_secondary = config.agg_scale.copy();
          }
          min = d3.min(rows, function(row) {
            return d3.min(v.aggGetter(row));
          });
          max = d3.max(rows, function(row) {
            return d3.max(v.aggGetter(row));
          });

          min = bar.getAggScaleMin(min);
          max = bar.getAggScaleMax(max);

          v.scale.domain(lib.extendExtent([min, max]));
        }
      );
      return bar;
    }

    if (config.color_enabled && config.is_stacked) {
      if (config.color_agg_cols) {
        min = 0;
        max = 0;
        rows.forEach(function findOverallMinAndMax(row) {
          const rowVals = config.aggs_getter(row);
          let rowTotals = 0;
          rowVals.forEach(function findRowMinAndMax(val) {
            rowTotals += val;
            max = Math.max(max, rowTotals);
            min = Math.min(min, rowTotals);
          });
        });

        if (!config.agg_is_log10) {
          min = Math.min(min, 0);
        }

        min = bar.getAggScaleMin(min);
        max = bar.getAggScaleMax(max);

        config.agg_scale.domain(lib.extendExtent([min, max]));
        return bar;
      }

      // If we are building stacks, compute the max stack size we will be
      // drawing, and use that to set the domain on the aggregate axis
      const max_by_dim = {};
      const curr_negative_agg_by_dim = {};
      const curr_positive_agg_by_dim = {};
      const min_by_dim = {};
      let smallest_agg;

      if (rows.length > 0) {
        smallest_agg = config.agg_getter(rows[0]);
      }

      var xydim = function(d) {
        return [config.trellis.x_getter(d), config.trellis.y_getter(d),
                config.dim_getter(d)];
      };

      rows.forEach(function(row) {
        var c = xydim(row);
        if (!(c in max_by_dim)) {
          max_by_dim[c] = 0;
          min_by_dim[c] = 0;
          curr_negative_agg_by_dim[c] = 0;
          curr_positive_agg_by_dim[c] = 0;
        }

        var agg = config.agg_getter(row);
        if (agg < 0) {
          curr_negative_agg_by_dim[c] += agg;
        } else {
          curr_positive_agg_by_dim[c] += agg;
        }

        max_by_dim[c] = Math.max(max_by_dim[c], curr_positive_agg_by_dim[c]);
        min_by_dim[c] = Math.min(min_by_dim[c], curr_negative_agg_by_dim[c]);
        smallest_agg = Math.min(smallest_agg, agg);
      });

      var al = [];
      $.each(max_by_dim, function(key, value) {
        al.push(value);
        al.push(min_by_dim[key]);
      });

      extent = d3.extent(al);
      if (config.agg_is_log10) {
        extent[0] = smallest_agg;
      } else {
        extent[0] = Math.min(extent[0], 0);
      }

      extent[0] = bar.getAggScaleMin(extent[0]);
      extent[1] = bar.getAggScaleMax(extent[1]);

      config.agg_scale.domain(lib.extendExtent(extent));
      return bar;
    }

    if (config.agg_is_percentage) {
      testAndRun(
        [
          { aggGetter: config.aggs_getter, scale: config.agg_scale, run: true },
          { aggGetter: config.aggs_secondary_getter, run: config.has_secondary_agg_cols }
        ],
        function functionName(v, i) {
          if (i === 1 /*is secondary axis*/) {
            v.scale = config.agg_scale_secondary = config.agg_scale.copy();
          }
          // We have already computed the domain - do nothing here
          if (v.aggGetter) {
            max = d3.max(rows, function(row) {
              return d3.max(v.aggGetter(row));
            });

            min = d3.min(rows, function(row) {
              return d3.min(v.aggGetter(row));
            });

            min = bar.getAggScaleMin(min);
            max = bar.getAggScaleMax(max);

            v.scale.domain(lib.extendExtent([min, max]));
          } else {
            extent = d3.extent(rows, config.agg_getter);
            extent[0] = bar.getAggScaleMin(extent[0]);
            v.scale.domain(lib.extendExtent(extent));
          }
        }
      );

      return bar;
    }

    // Set the domain to the min and max of the rows
    extent = d3.extent(rows, config.agg_getter);
    // Handle the case where the domain values are not distinct
    if (extent[0] === extent[1]) {
      if (setting_scale().value() === 'LOG10') {
        if (extent[0] > 1) {
          extent[0] = 1;
        } else {
          extent[0] = extent[1] * 0.9;
        }
      } else {
        if (extent[0] > 0) {
          extent[0] = 0;
        } else if (extent[0] < 0) {
          extent[1] = 0;
        }
      }
    }

    if (bar.agg_logscale()) {
      // LOG10 scale - end at 1
      extent[0] = Math.min(extent[0], 1);
    } else {
      extent[0] = bar.getAggScaleMin(extent[0]);
      extent[1] = bar.getAggScaleMax(extent[1]);
    }

    config.agg_scale.domain(lib.extendExtent(extent));
    return bar;
  };

  bar.compute_percentages = function(config, rows) {
    if (!config.agg_is_percentage) {
      return bar;
    }

    config.agg_scale.domain([0, 1]);

    if (config.color_agg_cols) {
      // Handle the trellis per-cell case where 'aggs_getter' needs
      // to be used with the original settings once for each cell
      config.aggs_getter = config.orig_aggs_getter;

      // Multiple aggregates on each row
      var colnames = config.color_agg_cols.map(function(col) {
        return col.colname();
      });
      lib.compute_percentages_multiple(rows, config.aggs_getter, colnames,
        setting_scale().value() === 'PERCENTAGE_COLOR');

      config.colnames_to_idx = {};
      config.colname_is_secondary = {}; // when percentage scale and secondary axis
      config.color_agg_cols.forEach(function(col) {
        config.colnames_to_idx[col.colname()] = col.colname();
        if (config.has_secondary_agg_cols) {
          config.colname_is_secondary[col.colname()] = col.secondaryaxis();
        }
      });
      config.aggs_getter = function(row) {
        return config.color_agg_cols.map(function(col) {
          if (!config.colname_is_secondary[col.colname()]) {
            return row[col.colname()];
          }
        });
      };
      config.aggs_secondary_getter = function(row) {
        return config.color_agg_cols.map(function(col) {
          if (config.colname_is_secondary[col.colname()]) {
            return row[col.colname()];
          }
        });
      };

      return bar;
    }

    // Handle the trellis per-cell case where 'agg_getter' needs
    // to be used with the original settings once for each cell
    config.agg_getter = config.orig_agg_getter;

    config.percentage_colname = 'percentage';
    if (setting_scale().value() === 'PERCENTAGE') {
      var xydim = function(d) {
        return [config.trellis.x_getter(d), config.trellis.y_getter(d),
                config.dim_getter(d)];
      };
      lib.compute_percentages(rows, config.percentage_colname,
          xydim, config.agg_getter);
    } else {
      // (setting_scale.value() == 'PERCENTAGE_COLOR')
      var xycolor = function(d) {
        var xy = [config.trellis.x_getter(d), config.trellis.y_getter(d)];
        // Add color only if there is a color_getter
        if (config.color_getter) {
          xy.push(config.color_getter(d));
        }
        return xy;
      };
      lib.compute_percentages(rows, config.percentage_colname,
          xycolor, config.agg_getter);
    }
    config.agg_getter = function(d) {
      return d[config.percentage_colname];
    };

    return bar;
  };

  function xIsSlanted(config) {
    const x_is_date = config.dim_is_date && config.x_is_dim;

    return ((config.curr_mark_type === 'groupedbar') || (config.x_is_dim && !x_is_date));
  }

  bar.draw_base_visual = function(config, cell_sel, rows, viz_settings, idx) {
    config.rows = rows;
    if (rows.length === 0) {
      return bar;
    }

    let overlayContainer = cell_sel.selectAll('.arc-viz-overlay-container'),
        fullWidth = config.width + config.margin.left + config.margin.right,
        fullHeight = config.height + config.margin.top + config.margin.bottom;

    let cellConfig = {
      width: fullWidth,
      height: fullHeight,
      margin: config.margin,
      overlayContainer,
    };

    if (overlayContainer.size() === 0) {
      arc.canvas.createContainer(cell_sel, cellConfig);
      overlayContainer = cellConfig.overlayContainer;
    }

    arc.canvas.createAllCanvas(cellConfig);

    var svg = createSvgs(overlayContainer, config.width, config.height,
        config.margin);

    let gridlineSVG;
    if (horizontal_gridlines_enabled() || vertical_gridlines_enabled()) {
      gridlineSVG = cell_sel.select('.viz-gridline-svg');
    }

    cellConfig.svg = svg;
    cellConfig.viz_settings = viz_settings;

    var lastVizElements = svg.data()[0];

    var brush;
    if (showSelectionRange().isChecked) {
      config.x_is_date = config.dim_is_date && config.x_is_dim;

      if (config.x_is_date) {
        brush = arc.visual.brush()
                       .svg(svg);
      }
    }

    // This clippath prevents line from overlapping with x axis
    let clipPathId = arc.misc.uniqueId();
    const clippath_id = `scatter-cp-${clipPathId}`;
    svg.selectAll('defs')
       .data([1])
       .enter()
         .append('defs')
         .selectAll('clipPath')
         .data([1])
         .enter()
           .append('clipPath')
           .attr('id', clippath_id)
           .selectAll('rect')
           .data([1])
           .enter()
             .append('rect');

    svg.select(`#${clippath_id}`)
       .selectAll('rect')
       .attr('x', 0)
       .attr('y', 0)
       .attr('width', config.width)
       .attr('height', config.height);

    var x_label,
        y_label,
        secondary_agg_label;

    if (combined_visual || config.has_secondary_agg_cols) {
      x_label = [lib.Label(config.x_axis.colname(), config.x_axis.coldesc(), undefined)];

      if (combined_visual) {
        y_label = combined_visual.y_labels;
        secondary_agg_label = combined_visual.y_secondary_labels;
      } else if (config.has_secondary_agg_cols) {
        y_label = config.color_primary_agg_cols.map(function(col) {
          var n = col.colname();
          return lib.Label(n, col.coldesc() || n, config.color(n));
        });
        secondary_agg_label = config.color_secondary_agg_cols.map(function(col) {
          var n = col.colname();
          return lib.Label(n, col.coldesc() || n, config.color(n));
        });
      }
    }

    const x_is_date = config.dim_is_date && config.x_is_dim;
    const axesSVGs = lib.draw_axes(svg, config.width, config.height, config.margin,
        {
          x_axis_data: config.x_axis,
          y_axis_data: config.y_axis,
          secondary_y_axis_data: config.secondary_y_axis,
          gridlineSVG: gridlineSVG,
          drawVerticalGridLine: vertical_gridlines_enabled(),
          drawHorizontalGridLine: horizontal_gridlines_enabled(),
          disable_dim_labels: config.curr_mark_type === 'groupedbar',
          x_is_dim: config.x_is_dim && !config.x_axis.is_date(),
          y_is_dim: config.y_is_dim && !config.x_axis.is_date(),
          x_axis_label_off: (x_axis_label().value() === 'false'),
          y_axis_label_off: (y_axis_label().value() === 'false'),
          x_label: x_label,
          y_label: y_label,
          secondary_agg_label: secondary_agg_label,
          show_x_axis: show_x_axis().isChecked,
          show_y_axis: show_y_axis().isChecked,
          dynamic_bar_width: config.x_is_dim && config.x_axis.is_date() &&
            config.dynamic_bar_width,
          xAxisSVG: lastVizElements.xAxisSVG,
          yAxisSVG: lastVizElements.yAxisSVG,
          x_is_slanted: xIsSlanted(config),
        });

    function getHierarchyElement(shelf) {
      const elems = lib.getShelfElements(shelf);
      const elem = elems.slice(-1).pop();

      if (elem) {
        const dimhlistLength = (elem.dataset_dimhlist() && elem.dataset_dimhlist().length) || 0;
        if (dimhlistLength && (elem.dataset_dimhlevel() < dimhlistLength)) {
          //Should be looping till the extreme depth of the hierarchy
          return elem;
        }
      }
    }

    // Grab the last element from the x_shelf
    let xelem, yelem, hierarchy_shelf_element, axisIdx = -1;
    xelem = getHierarchyElement(x_shelf);
    if (xelem) {
      axisIdx = 0;
    } else {
      yelem = getHierarchyElement(y_shelf);
      if (yelem) {
        axisIdx = 1;
      }
    }

    hierarchy_shelf_element = xelem || yelem;

    let dataRenderer = config.curr_mark_type + "_" + config.color_enabled;
    if (previousDataRenderer !== dataRenderer) {
      previousDataTracker.cleanup();
      previousDataRenderer = dataRenderer;
    }

    if (config.color_enabled && config.isWaterfall && config.curr_mark_type === 'bar') {
      helper =
          arc.visual.barDrawHelper()
            .lib(lib)
            .bar(bar)
            .enableTransitions(previousDataTracker.enabled())
            .config(config)
            .cellConfig(cellConfig);

      currentHelpers.push(helper);
      helper.previousComputedData(previousDataTracker.getData(idx));
      helper.draw();
      previousDataTracker.setData(helper.computedData(), idx);
    } else if (config.color_enabled) {
      if (config.curr_mark_type === 'groupedbar') {
        bar.create_grouped_bars(svg, config, viz_settings);
      } else {
        let showPointsSetting = show_points();
        helper =
            arc.visual.lineAreaDrawHelper()
                .lib(lib)
                .bar(bar)
                .enableTransitions(previousDataTracker.enabled())
                .config(config)
                .cellConfig(cellConfig)
                .showPoints(showPointsSetting && showPointsSetting.isChecked);

        currentHelpers.push(helper);
        helper.previousComputedData(previousDataTracker.getData(idx));
        helper.draw();
        previousDataTracker.setData(helper.computedData(), idx);
      }
    } else {
      if (config.curr_mark_type === 'bar') {
        helper =
          arc.visual.barDrawHelper()
              .lib(lib)
              .bar(bar)
              .enableTransitions(previousDataTracker.enabled())
              .config(config)
              .cellConfig(cellConfig);

        currentHelpers.push(helper);
        helper.previousComputedData(previousDataTracker.getData(idx));
        helper.draw();
        previousDataTracker.setData(helper.computedData(), idx);
      }

      // Only used in combo visual
      if (config.curr_mark_type === 'highlight') {
        createHighlights(svg, config, viz_settings);
      }

      if ((config.curr_mark_type === 'line') ||
          (config.curr_mark_type === 'area')) {
        let showPointsSetting = show_points();
        helper =
          arc.visual.lineAreaDrawHelper()
              .lib(lib)
              .bar(bar)
              .enableTransitions(previousDataTracker.enabled())
              .config(config)
              .cellConfig(cellConfig)
              .showPoints(showPointsSetting && showPointsSetting.isChecked);

        currentHelpers.push(helper);
        helper.previousComputedData(previousDataTracker.getData(idx));
        helper.draw();
        previousDataTracker.setData(helper.computedData(), idx);
      }
    }

    if (hierarchy_shelf_element) {
      var dimh = hierarchy_shelf_element.dataset_dimhlist();
      var vals = hierarchy_shelf_element.dataset_dimhvals();
      var level = hierarchy_shelf_element.dataset_dimhlevel();
      var $chart =  $("#" + this.viz_lib().chart_div());
      var $haxis = $($chart.find(".arc-viz-axis-label")[axisIdx]);
      var axisName = $haxis.length > 0 ? $haxis[0].classList.item(0) : "";

      // If we're not at the bottom of the hierarchy, then set up click on the
      // dimension value to push this value into the list of active dimension
      // hierarchy values and do the redraw.
      if (dimh && vals.length < (dimh.length - 1)) {
        var $hierarchyTickText = $chart.find('.' + axisName + ' .tick text');
        $hierarchyTickText.attr('class', 'hierarchy-clickable');
        $hierarchyTickText.on('click', function() {
          var val = d3.select(this).datum();
          if (lib.viz_type().indexOf('groupedbars') >= 0) {
            val = val.groupname || val;
          }

          if (config.columns[0].coltype() === 'STRING') {
            val = "'" + val + "'";
          }

          vals.push(val);
          hierarchy_shelf_element.dataset_dimhvals(vals);
          hierarchy_shelf_element.dataset_dimhlevel(hierarchy_shelf_element.dataset_dimhlevel() + 1);
          lib.refresh();
        });
      }

      // If we're not at the top of the hierarchy, update the axis label to
      // include the immediately preceding to active dimension hierarchy value and also
      // set up a click on axis label to pop up the display one level (above).
      if (vals.length > 0) {
        var text = $haxis.text();
        $haxis.closest("svg").find("text." + axisName + ".arc-viz-axis-label").attr("class", "hierarchy-clickable");
        var i = vals.length - 1;
        text += " (" + dimh[i] + " = " + vals[i] + ") ";
        $haxis.text(text);
        // TODO: instead of enforcing drill down click on entire label, confine it to specific tspan
        $haxis.on("click", function () {
          vals.pop();
          lib.refresh();
          hierarchy_shelf_element.dataset_dimhlevel(hierarchy_shelf_element.dataset_dimhlevel() - 1);
        });
      } else {
        // TODO: not sure what this code is doing but most probably should be removed
        $haxis.on("click", function () {
          if (level < dimh.length - 1) {
            hierarchy_shelf_element.dataset_dimhlevel(level + 1);
            lib.refresh();
          }
        });
      }
    }

    var clippath = svg.append("svg")
                     .style("width", config.width + "px")
                     .style("height", config.height + "px")
                       .append("g")
                       .attr("clip-path", "url(#" + clippath_id + ")");

    if (trendline_linear().isChecked) {
      var xy_values = [],
          xIndex = config.x_col.colindex(),
          yIndex = config.y_col.colindex(),
          x, y;
      // If there are aggregates in the column shelf, need to create a new x_y_value for each color
      if (config.color_agg_cols && trendline_color().isChecked) {
        rows.forEach(function(d, i) {
          config.color_agg_cols.forEach(function (aggCol, i) {
            var colName = aggCol.colname(),
                color = config.color ? config.color(colName) : 'grey',
                aggIdx = aggCol.colindex(),
                aggScale;

            if (config.has_secondary_agg_cols && aggCol.secondaryaxis()) {
              aggScale = config.agg_scale_secondary;
            } else {
              aggScale = config.agg_scale;
            }

            if (config.dim_is_date) {
              x = config.x_is_dim ? config.x_scale(d.datecol) : aggScale(d[aggIdx]);
              y = config.y_is_dim ? config.y_scale(d.datecol) : aggScale(d[aggIdx]);
            } else {
              x = config.x_is_dim ? config.x_scale(d[xIndex]) : aggScale(d[aggIdx]);
              y = config.y_is_dim ? config.y_scale(d[yIndex]) : aggScale(d[aggIdx]);
            }

            xy_values.push([x, y, color]);
          });
        });
      } else {
        rows.forEach(function(d, i) {
          x = config.x_is_dim && config.dim_is_date ? config.x_scale(d.datecol) : config.x_scale(d[xIndex]);
          y = config.y_is_dim && config.dim_is_date ? config.y_scale(d.datecol) : config.y_scale(d[yIndex]);
          xy_values.push([x, y, config.color_getter ? config.color_getter(d) : 'grey']);
        });
      }

      lib.draw_trend_lines(clippath, config, xy_values, trendline_color().value(), trendline_tooltip_decimal().value());
    }

    if (brush) {
      brush.xScale(config.x_axis.scale())
           .yScale(config.y_axis.scale())
           .cb(function(e) {
             let col;
             if (config.x_is_dim) {
               col = config.x_col;
             } else {
               col = config.y_col;
             }

             const substitutedColname = lib.substituteParams(col.colname());
             const paramConfig = {
               baseField: col.colfiltername(),
               outputParameter: substitutedColname,
               value: [{
                 start: config.date_format(e[0]),
                 end: config.date_format(e[1]),
               }]
             };

             const paramObject = new arc.params.DateRangeParam(lib, paramConfig);

             lib.sendParameters([paramObject]);
           });

      brush.initialize();
      brush.draw();
    }

    lastVizElements.xAxisSVG = axesSVGs.xAxisSVG;
    lastVizElements.yAxisSVG = axesSVGs.yAxisSVG;

    return bar;
  };

  bar.compute_dim_maxvalues = function(config, rows) {
    var dim_numvalues;
    if (config.dim_is_date && !config.dim_is_datetime) {
      dim_numvalues = (config.dim_getter(rows[0]) -
          config.dim_getter(rows[rows.length - 1])) /
          (1000 * 60 * 60 * 24);
      dim_numvalues = Math.abs(dim_numvalues);
      dim_numvalues = Math.max(1, dim_numvalues);
      dim_numvalues++;

      // For dynamic dates axes with we add a day to the start and the end to
      // prevent overlap with the axes. Thus, add two additional bars.
      if (makeDimDynamic && config.x_is_dim) {
        dim_numvalues += 2;
      }
    } else {
      dim_numvalues = config.unique_dims.length;
    }

    // For grouped bars, we have already computed 'config.max_slots' in
    // bar.organize_cells
    if (config.curr_mark_type !== 'groupedbar') {
      config.max_slots = dim_numvalues;
      if (config.isWaterfall) {
        config.max_slots += 1;
      }

      bar._compute_dim_sz(config, config.max_slots);
    }
    return bar;
  };

  bar.create_grouped_bars = function(svg, config, viz_settings) {
    var x = config.trellis.x_getter(config.rows[0]),
        y = config.trellis.y_getter(config.rows[0]);

    config.trellis_cell_groups.some(function(cell_group) {
      if (arc.arraysIdentical(cell_group.group, [x, y])) {
        if (scale_dim_independent().isChecked) {
          bar._compute_dim_sz(config, cell_group.total_slots, cell_group);
        }

        bar._draw_cell_ticks(svg, config, cell_group.x_groups);
        bar._draw_cell_marks(svg, config, cell_group.x_groups);
        return true;
      }

      return false;
    });
  };

  bar.getLineXDimOffset = function(config, original_dim_width) {

    let offset = 0;
    const bar_padding = !config.dim_is_date &&
                        config.combined_visual &&
                        config.combined_visual.bar_padding &&
                        config.combined_visual.bar_padding_px;

    if (!config.dim_is_date || !config.combined_visual) {
      if (!config.dim_is_date) {
        if (original_dim_width) {
          offset += (original_dim_width / 2);
        } else {
          offset += (config.dim_sz / 2);
        }
      }

      if (bar_padding > 0) {
        offset -= bar_padding;
      }
    }

    if (config.combined_visual && !config.dim_is_date) {
      offset = config.barDimSz ? config.barDimSz / 2 : config.dim_sz / 2;
    }

    if (config.dimIsContinuous) {
      offset = 0;
    }

    return offset;
  };

  /*
   * Convert a transformation string, something like
   * "translate(10, 20),scale(5,6)"
   * into a structured hash like
     {
       transform: [10,20],
       scale: [5,6],
     }
   */
  function parseTransform(a) {
    const b = {};
    if (!a) {
      return b;
    }

    a = a.replace(/\s/g, '');
    for (let i in a = a.match(/(\w+\((\-?\d+\.?\d*e?\-?\d*,?)+\))+/g)) {
      var c = a[i].match(/[\w\.\-]+/g);
      b[c.shift()] = c;
    }

    return b;
  }

  /*
   * Convert a structured hash of the form returned by parseTransform
   * above into a transform string like: "translate(10, 20),scale(5,6)"
   */
  function stringifyTransform(b) {
    let s = [];

    Object.keys(b).forEach(function (k) {
      const v = b[k];
      s.push(`${k}(${v[0]},${v[1]})`);
    });

    return s.join(',');
  }

  function addTransform(x_is_dim, $e, offset) {
    let transform = parseTransform($e.attr('transform'));
    if (!transform) {
      return;
    }

    let translate = transform.translate;
    if (!translate) {
      return;
    }

    if (x_is_dim) {
      translate[0] = Number(translate[0]) + offset;
    } else {
      translate[1] = Number(translate[1]) + offset;
    }

    $e.attr('transform', stringifyTransform(transform));
  }

  bar.adjustAxes = function(x_is_dim, offset, cellConfig, axisRange) {
    if (!offset) {
      return;
    }

    let $ticks = $(cellConfig.svg.node())
        .closest('svg');

    if (x_is_dim) {
      $ticks = $ticks.find('g.x.arc-viz-axis g.tick');
    } else {
      $ticks = $ticks.find('g.y.arc-viz-axis g.tick');
    }

    $ticks.each(function() {
      addTransform(x_is_dim, $(this), offset);
    });

    {
      // Remove the last tick if it falls outside the axis
      let $lastTick = $ticks.last();
      let transform = parseTransform($lastTick.attr('transform'));
      if (!transform) {
        return;
      }

      let translate = transform.translate;
      if (!translate) {
        return;
      }

      let c;
      if (x_is_dim) {
        translate[0] = Number(translate[0]) + offset;
        c = translate[0];
      } else {
        translate[1] = Number(translate[1]) + offset;
        c = translate[1];
      }

      if (axisRange && (c > axisRange[1])) {
        $lastTick.remove();
      }
    }
  };

  bar.line_x_pos = function(d, config) {
    let p;
    if (config.x_is_dim) {
      p = config.x_scale(d[config.x_idx]) + bar.getLineXDimOffset(config);
    } else {
      p = config.x_scale(config.agg_getter(d));
    }

    return p;
  };

  bar.line_y_pos = function(d, config) {
    var p;
    if (config.y_is_dim) {
      p = config.y_scale(d[config.y_idx]);
      if (!config.dim_is_date) {
        p += (config.dim_sz / 2);
      }
    } else {
      var agg = config.agg_getter(d);
      if (config.combined_visual && config.y_col.secondaryaxis()) {
        p = config.agg_scale_secondary(agg);
      } else {
        p = config.y_scale(agg);
      }
    }

    return p;
  };

  bar.getBarXDimOffset = function(config) {
    // have slight offset to prevent bars overlapping with y axis
    let barOffset = -1;

    // For bars with dynamic date axes, move the bars over to the left by one half
    // the bar length
    // Always shift over for datetime in x axis when scale has more than one value
    // in its domain
    if ((makeDimDynamic && config.dim_is_date && !config.dim_is_datetime) ||
      (config.dim_is_datetime && config.x_is_dim && domainSize(config.x_scale) > 0)) {

      barOffset += config.dim_sz / 2;
    }

    if (!config.combined_visual && config.dimIsContinuous && config.x_is_dim) {
      barOffset += config.dim_sz / 2;
    }

    return barOffset;
  };

  function domainSize(scale) {
    let size = 0;
    if (scale) {
      let domain = scale.domain();
      size = domain[1] - domain[0];
    }

    return size;
  }

  bar._get_x = function(d, config, is_negative) {
    if (isHistogram) {
      return config.x_scale(d[config.x_idx]) -
          (bar._get_width(d, config, is_negative) / 2);
    }

    if (config.x_is_dim) {
      return config.x_scale(d[config.x_idx]) - bar.getBarXDimOffset(config);
    }

    var agg = config.agg_getter(d);
    return bar._get_agg_x(agg, config, is_negative);
  };

  bar._get_agg_x = function(agg, config, is_negative) {
    // have slight offset to prevent bars overlapping with y axis
    var x_offset = 0;
    if (is_negative) {
      if (agg > 0) {
        return config.x_scale(0) + x_offset;
      }
      return config.x_scale(agg) + x_offset;
    }
    return 0 + x_offset;
  };

  bar._get_y = function(d, config, is_negative) {
    if (config.x_is_dim) {
      var agg = config.agg_getter(d);
      let a = bar._get_agg_y(agg, config, is_negative);
      return a;
    }
    return config.y_scale(d[config.y_idx]);
  };

  bar._get_agg_y = function(agg, config, is_negative, index) {
    let scale = config.combined_visual && config.y_col.secondaryaxis() ?
                config.agg_scale_secondary : config.y_scale;

    if (mark_type === 'groupedbar' && config.color_agg_cols) {
      scale = config.color_agg_cols?.[index]?.secondaryaxis() ?
      config.y_scale_secondary : config.y_scale;
    }

    // have slight offset to prevent bars overlapping with y axis
    let y_offset = 0;

    if (config.isWaterfall) {
      y_offset = 0;
    }

    if (is_negative) {
      if (agg > 0) {
        return scale(agg) - y_offset;
      }

      var topY = d3.min(scale.range()),
          zeroOrTop = Math.max(scale(0), topY);
      return zeroOrTop - y_offset;
    }

    return scale(agg) - y_offset;
  };

  bar._get_width = function(d, config, is_negative) {
    if (isHistogram) {
      // 0 or 1 pixel. Sometimes it has some advantage to separate them visually.
      // Not more because because otherwise it would ruin the interval detection
      // (it would not start where it should within an interval if it is too big)
      if (!cellSpecificHistConfig.barWidth) {
        cellSpecificHistConfig.barWidth =
            (config.x_scale(cellSpecificHistConfig.bucketMax) -
            config.x_scale(cellSpecificHistConfig.bucketMin)) /
            histogramConfig.bucketCount;
      }

      return cellSpecificHistConfig.barWidth - cellSpecificHistConfig.spaceBetweenHistogramBars;
    }

    if (config.x_is_dim) {
      return config.dim_sz - 1;
    }

    var agg = config.agg_getter(d);
    return bar._get_agg_width(agg, config, is_negative);
  };

  bar._get_agg_width = function(agg, config, is_negative) {
    if (config.x_is_dim) {
      return config.dim_sz - 1;
    }
    if (is_negative) {
      var width = d3.max(config.agg_scale.range()),
          zeroOrRight = Math.min(config.agg_scale(0), width);
      return Math.abs(config.agg_scale(agg) - zeroOrRight);
    }
    return config.agg_scale(agg);
  };

  bar._get_height = function(d, config, is_negative) {
    if (!config.x_is_dim) {
      return config.dim_sz - 1;
    }
    var agg = config.agg_getter(d);
    return bar._get_agg_height(agg, config, is_negative);
  };

  bar._get_agg_height = function(agg, config, is_negative, index) {
    let height;
    let scale = config.combined_visual && config.y_col.secondaryaxis() ?
                config.agg_scale_secondary : config.y_scale;

    if (mark_type === 'groupedbar') {
      scale = config.color_agg_cols?.[index]?.secondaryaxis() ?
      config.y_scale_secondary : config.y_scale;
    }

    if (is_negative) {
      var topY = d3.min(scale.range()),
          zeroOrTop = Math.max(config.agg_scale(0), topY);
      height = Math.abs(scale(agg) - zeroOrTop);
    } else {
      height = config.height - scale(agg);
    }

    if (config.isWaterfall) {
      return height;
    }

    return height;
  };

  bar._create_tooltip = function (data, config) {
    var pre_kvs = [],
        primary_colindex;

    if (config.color_agg_cols) {
      if (data.color) {
          pre_kvs.push({ key: 'Measure', value: data.color });
      }

      if (!config.combined_visual) {
        primary_colindex = config.x_is_dim ? config.x_idx : config.y_idx;
      }
    } else if (config.combined_visual) {
      pre_kvs.push({ key: 'Measure', value: config.y_col.colname() });
    }

    if (config.agg_is_percentage) {
      pre_kvs.push({
       key: 'Percentage',
       value: formatPercentTwo(data.percentage),
       value_is_num: true
     });
    }

    var tooltipConfig = {primary_colindex: primary_colindex, pre_kvs: pre_kvs};
    return lib.text_from_columns(data, config.columns, tooltipConfig);
  };

  function adjustStroke(marks, config) {
    if (config.dim_sz < 3) {
      var strokeWidth = Math.max(0, (config.dim_sz - 1) / 2);
      marks.style('stroke-width', `${strokeWidth * strokeWidth}px`);
    }
  }

  function drawLabels(svg, config, labelColFunc, rows, labelIdentifier, groupFilter) {
    const outsideColor = '#222';
    const o = lib.colorOpacity();
    const labelColor = lib.getLabelColor();

    var getLabelText = function(d, i) {
      let labelCol = labelColFunc(d, i);

      if (!labelCol) {
        return '';
      }

      let coldisplayformat = labelCol.coldisplayformat();

      let text;
      if (config.colnames_to_idx && !config.label_col) {
        // config.colnames_to_idx is not defined for combo visual, so need this
        // check here. Also, don't enter this area if there is a value in the
        // label shelf.
        text = d[config.colnames_to_idx[labelCol.colname()]];
      } else {
        text = d[labelCol.colindex()];
      }

      if (text === '') {
        return '';
      }

      if (coldisplayformat) {
        return coldisplayformat(text);
      }

      return lib.default_formatter(text);
    };

    let curr_labels = svg.selectAll('.nosel');
    curr_labels = groupFilter ? curr_labels.filter(groupFilter) : curr_labels;
    curr_labels = curr_labels.data(rows, labelIdentifier);

    let enter_labels = curr_labels.enter()
                                  .append('text')
                                  .classed('arc-bar-label', true);

    if (config.x_is_dim) {
      let addPropsToXLabels = function(sel) {
        sel.attr('x', function(d) {
             return d.x + ((d.w || 0) / 2);
           })
           .attr('y', function(d) {
             // If the bar h is less than 20 pixels for grouped bar and bar, then
             // draw the label just on the outside of the bar.
             if (d.h && (d.h < 20)) {
               return d.y - 5;
             }

             return ((d.y + 15) < config.height) ? (d.y + 15) : (d.y - 5);
           })
           .attr('fill', function(d) {
             if (labelColor) {
               return labelColor;
             }

             if (d.h && (d.h < 20)) {
               return outsideColor;
             }

             if (!d.color) {
               return outsideColor;
             }

             return arc.colorpalette.colorForBrightness(d.color, o);
           })
           .style('text-anchor', function(d) {
             return 'middle';
           })
           .text(getLabelText);
      };

      addPropsToXLabels(enter_labels);
    } else {
      let addPropsToYLabels = function(sel) {
        sel.attr('x', function(d) {
             // If stacked, use d.x and do text-anchor center. For grouped bar and
             // bar, if the bar h is less than 20 pixels, place the text on the right
             // of the bar and use text-anchor start. Otherwise, place the text on
             // the left of the bar and use text-anchor end.
             if (d.w === undefined) {
               return d.x;
             } else if (d.x + d.w < 20) {
               return d.x + d.w + 5;
             }

             return d.x + d.w - 5;
           })
           .attr('y', function(d) {
             return d.y + ((d.h ? d.h : 0) / 2) + 3;
           })
           .attr('fill', function(d) {

             if (labelColor) {
               return labelColor;
             }

             if ((d.w === undefined) || (d.x + d.w < 20)) {
               return outsideColor;
             }

             if (!d.color) {
               return outsideColor;
             }

             return arc.colorpalette.colorForBrightness(d.color, o);
           })
           .style('text-anchor', function(d) {
             if (d.w === undefined) {
               return 'center';
             } else if (d.x + d.w < 20) {
               return 'start';
             }

             return 'end';
           })
           .text(getLabelText);
      };

      addPropsToYLabels(enter_labels);
    }
  }

  // Only for combined visuals
  function createHighlights(svg, config, viz_settings) {
    var isCircle = (highlight_mark_type().value() === "CIRCLE"),
        fill_opacity = isCircle ? 0 : lib.colorOpacity(),
        stroke_size = isCircle ? '3px' : '1px',
        isStar = (highlight_mark_type().value() === "STAR"),
        radius = Math.min(highlight_mark_radius().value(), config.dim_sz) / 3,
        marks;

    if (isStar) {
      marks = svg.selectAll(".nosel")
                  .data(config.rows)
                  .enter()
                    .append("polygon")
                    .attr("points", function (d) {
                      return calculateStarPoints(d, 5, radius, radius - 5, config);
                    });
    } else {
      marks =  svg.selectAll(".nosel")
                  .data(config.rows)
                  .enter()
                    .append("circle")
                    .classed("viz-circle", true)
                    .attr("cx", function (d) {
                      return bar.line_x_pos(d, config);
                    })
                    .attr("cy", function (d) {
                      return bar.line_y_pos(d, config);
                    })
                    .attr("r", function (d) {
                      return radius;
                    });
    }

    var c = config.color(config.y_col.colname());
    marks.style("fill", c)
         .style("fill-opacity", fill_opacity)
         .style("stroke", c)
         .style("stroke-opacity", lib.colorOpacity())
         .style("stroke-width", stroke_size)
         .classed("dim_all", true);

    arc.marks()
       .permitPivot(bar.enableDrillInto())
       .viz_lib(markerLib ? markerLib : lib)
       .marker_name('highlights')
       .addSelection(marks)
       .text_fn(function(d, i) {
         return bar._create_tooltip(d, config);
        })
       .params_fn(function(d, i) {
          return lib.params_from_columns(d, config.columns);
        })
       .highlight_on_select(true)
       .activate();

    return bar;

    //https://dillieodigital.wordpress.com/2013/01/16/quick-tip-how-to-draw-a-star-with-svg-and-javascript/
    function calculateStarPoints(d, arms, outerRadius, innerRadius) {
      var centerX = bar.line_x_pos(d, config),
          centerY = bar.line_y_pos(d, config),
          results = "",
          angle = Math.PI / arms;
      outerRadius = Math.max(outerRadius, 2);
      innerRadius = Math.max(innerRadius, 1);

      // Our first time we simply append the coordinates, subsequet times
      results = centerX + "," + (centerY + innerRadius);
      for (var i = 1; i < 2 * arms; i++){
          // Use outer or inner radius depending on what iteration we are in.
          var r = (i & 1) === 0 ? innerRadius : outerRadius;
          var currX = centerX + Math.sin(i * angle) * r;
          var currY = centerY + Math.cos(i * angle) * r;

          // we append a ", " to distinguish each coordinate pair.
         results += ", " + currX + "," + currY;
       }

       return results;
    }
  }

  function createSvgs(cell_sel, width, height, margin) {
    // If used in a combined visual, we may find a svg in the cell already.
    // Combined visuals cannot transition.
    var sel = cell_sel.selectAll('svg.viz-bar-svg .viz-bar-g');
    if (sel.size() > 0) {
      sel = sel.data([{}]);
      return sel;
    }

    // Otherwise, create one if necessary. Then transition it if necessary
    if (sel.size() < 1) {
      cell_sel.append('svg')
              .style('position', 'absolute')
              .style('left', '0px')
              .style('top', '0px')
              .classed('viz-bar-svg', true)
                .append('g')
                .classed('viz-bar-g', true)
                .data([{}]);

      if (horizontal_gridlines_enabled() || vertical_gridlines_enabled()) {
        cell_sel.insert('svg', ':first-child')
                .style('position', 'absolute')
                .style('left', '0px')
                .style('top', '0px')
                .classed('viz-gridline-svg', true);
      }
    }

    cell_sel.selectAll('.viz-bar-svg')
            .style('width', `${width + margin.left + margin.right}px`)
            .style('height', `${height + margin.top + margin.bottom}px`);
    cell_sel.selectAll('.viz-bar-g')
            .attr('transform', `translate(${margin.left},${margin.top})`);

    return cell_sel.selectAll('.viz-bar-g');
  }

  bar.get_unique_dims = function(config, rows) {
    var asc;

    config.has_sort_order = lib.has_sort_order(config);
    if ((config.presorted) &&
        (config.ordercol.colindex() !== config.dim_orig_start_idx)) {
      config.dim_is_date = false;
    } else {
      var ret = lib.idx_is_date(rows, config.dim_orig_start_idx);
      config.dim_is_date = ret.is_date;
      config.dim_is_datetime = ret.is_datetime;
      config.date_format = ret.dateFormat;

      if (config.dim_is_date) {
        lib.compute_dates(rows, config.date_format, 'datecol', function(row) {
          return row[config.dim_orig_start_idx];
        });

        // if date col does not have sort order on it then set asc = true
        asc = config.columns[config.dim_orig_start_idx].order();
        asc = typeof asc === 'undefined' ? true : asc.ascending();
        rows.sort(function(a, b) {
          if (asc) {
            return arc.misc.sortNumeric(
                a[config.dim_orig_start_idx],
                b[config.dim_orig_start_idx]);
          } else {
            return arc.misc.sortNumeric(
                b[config.dim_orig_start_idx],
                a[config.dim_orig_start_idx]);
          }
        });
        config.dim_start_idx = 'datecol';
      }
    }

    config.unique_dims = arc.misc.arrayUnique(rows, config.dim_getter);
    if (config.isWaterfall) {
      config.unique_dims.push(arc.misc.totalColumnName);
    }

    if (config.dim_is_date && !config.dim_is_datetime) {
      const firstDate = moment(config.unique_dims[0]);
      const lastDate = moment(config.unique_dims[config.unique_dims.length-1]);

      const dayDifference = lastDate.diff(firstDate, 'days');
      const length = Math.abs(dayDifference);
      const hasAscendingOrder = dayDifference >= 0;

      const dimensions = [];
      for(let i = 0; i <= length; i++) {
        const firstDateCopy = moment(firstDate);

        if(hasAscendingOrder) {
          firstDateCopy.add(i, 'days');
        } else {
          firstDateCopy.substract(i, 'days');
        }

        dimensions.push(firstDateCopy.toDate());
      }

      config.unique_dims = dimensions;
    } else if (config.trellis.is_active() && !config.has_sort_order) {
      var not_a_number = config.unique_dims.some(function(dim_value) {
        return isNaN(dim_value);
      });

      if (not_a_number) {
        config.unique_dims.sort();
      } else {
        config.unique_dims.sort(function(a, b) {
          a = Number(a);
          b = Number(b);
          if (a < b) {
            return -1;
          }
          if (a > b) {
            return 1;
          }
          return 0;
        });
      }
    }

    config.x_idx = config.x_is_dim ? config.dim_start_idx : config.agg_start_idx;
    config.y_idx = config.y_is_dim ? config.dim_start_idx : config.agg_start_idx;

    return bar;
  };

  function totalRow(config, rows) {
    let row = rows[0].slice();

    for (let i = config.dim_start_idx + 1; i <= config.dim_end_idx; ++i) {
      row[i] = '';
    }
    row[config.dim_start_idx] = arc.misc.totalColumnName;
    if (config.color_column) {
      row[config.color_column.colindex()] = arc.misc.totalColumnName;
    }

    for (let i = config.agg_start_idx + 1; i <= config.agg_end_idx; ++i) {
      row[i] = '';
    }

    row[config.agg_start_idx] = d3.sum(rows, function(d) {
      return Number(d[config.agg_start_idx]);
    });

    if (maxWaterfallAgg === undefined) {
      maxWaterfallAgg = row[config.agg_start_idx];
    } else {
      maxWaterfallAgg = Math.max(maxWaterfallAgg, row[config.agg_start_idx]);
    }

    return row;
  }

  bar.organize_cells = function(config, rows) {
    // Organization is only necessary for grouped bars
    if (['groupedbar', 'bar'].indexOf(config.curr_mark_type) === -1) {
      return bar;
    }

    // Hierarchize the data by trellis rows
    config.trellis_cell_groups = arc.misc.array_to_groups(rows,
        function(d) {
          return [config.trellis.x_getter(d), config.trellis.y_getter(d)];
        });

    if (config.x_is_dim && waterfall() && config.color_enabled && !config.color_agg_cols) {
      let stackedColors = false;
      config.trellis_cell_groups.some(function(cell_group) {
        let knownDims = {};
        cell_group.data.some(function(row) {
          let d = config.dim_getter(row);
          if (knownDims[d]) {
            stackedColors = true;
            return true;
          }

          knownDims[d] = true;
          return false;
        });

        if (stackedColors === true) {
          return true;
        }
      });

      if (stackedColors === false) {
        config.isWaterfall = true;
      }
    }

    // Now hierarchize groups within each cell
    config.trellis_cell_groups.forEach(function(cell_group) {
      if (config.isWaterfall) {
        cell_group.data.push(totalRow(config, cell_group.data));
      }

      // Compute the groups
      cell_group.x_groups = arc.misc.array_to_groups(cell_group.data,
          function(d) {
            if (config.curr_mark_type === 'groupedbar') {
              return config.dim_getter(d);
            }
            // For non-grouped bars, there is no grouping, but we still want to
            // compute total slots
            return undefined;
          });

      // Compute the start and end indices for each group
      var last_count = 0;
      var max_idx = 0;
      cell_group.x_groups.forEach(function(g, i) {
        g.group_idx_start = last_count;
        if (config.color_agg_cols) {
          last_count += config.color_agg_cols.length;
        } else {
          last_count += g.data.length;
        }
        g.group_idx_end = last_count - 1;
      });

      // Compute the maximum bar count for each cell
      cell_group.cell_bar_max = 1 +
          cell_group.x_groups[cell_group.x_groups.length-1].group_idx_end;

      cell_group.total_slots = cell_group.cell_bar_max +
          (cell_group.x_groups.length * 2);
    });

    config.max_slots = d3.max(config.trellis_cell_groups,
        function(cell_group) {
          return cell_group.total_slots;
        });

    bar._compute_dim_sz(config, config.max_slots);
    return bar;
  };

  bar._draw_cell_ticks = function(svg, config, x_groups) {
    var tick_positions = [];
    var groups = [];
    x_groups.forEach(function(g, i) {
      tick_positions.push({
        x: ((bar._group_x_pos(config, g, 0) +
            bar._group_x_pos(config, g,
                  config.color_agg_cols ? config.color_agg_cols.length :
                                          g.data.length)) / 2),
        groupname: g.group
      });
    });

    svg.selectAll('.gb')
       .remove();

    let ticks_g = svg.data()[0].lastTicks_g;
    if (!ticks_g) {
      ticks_g =
        svg.append('g')
           .attr('class', 'x arc-viz-axis');
    }

    if (config.x_is_dim) {
      ticks_g.attr('transform', `translate(0,${config.height})`);
    } else {
      ticks_g.attr('transform', 'translate(-8,0)');
    }

    let last_ticks = svg.data()[0].lastTicks;
    let last_tick_labels = svg.data()[0].lastTickLabels;
    if (!last_ticks) {
      last_ticks = ticks_g.selectAll('.nosel');
      last_tick_labels = ticks_g.selectAll('.nosel');
    }

    last_ticks = last_ticks.data(tick_positions, d => d.groupname);
    last_tick_labels = last_tick_labels.data(tick_positions, d => d.groupname);

    last_ticks
      .enter()
        .append('line')
        .attr('class', 'gb tick');

    last_tick_labels
      .enter()
        .append('g')
        .attr('class', 'tick')
        .append('text')
        .attr('class', 'gb text')
        .style('text-anchor', 'end')
        .text(function(d) {
          if (config.dim_is_date) {
            let jsDate = config.date_format.parse(d.groupname);
            return config.date_format(jsDate);
          }

          return d.groupname;
        });

    addPropsToTicks(last_ticks);
    addPropsToTickLabels(last_tick_labels);

    function addPropsToTicks(sel) {
      sel.attr('x1', function(d) {
           if (config.x_is_dim) {
             return d.x;
           }
           return 2;
         })
         .attr('y1', function(d) {
           if (config.x_is_dim) {
             return 0;
           }
           return d.x;
         })
         .attr('x2', function(d) {
           if (config.x_is_dim) {
             return d.x;
           }
           return 8;
         })
         .attr('y2', function(d) {
           if (config.x_is_dim) {
             return 6;
           }
           return d.x;
         })
         .style('stroke', 'gray')
         .style('stroke-width', '1px')
         .style('stroke-opacity', lib.colorOpacity());
    }

    function addPropsToTickLabels(sel) {
      sel
        .attr('transform', function(d) {
          if (config.x_is_dim) {
            return 'translate(' + (d.x+5) + ',14)';
          }
          return 'translate(0, ' + (d.x+4) + ')';
        })
        .selectAll('text')
          .text(function(d) {
            if (config.dim_is_date) {
              let jsDate = config.date_format.parse(d.groupname);
              return config.date_format(jsDate);
            }

            return config.dimCol.coldisplayformat()(d.groupname);
          });

      if (config.x_is_dim) {
        sel.selectAll('text').attr('transform', function(d) {
          return 'rotate(' + lib.x_slant() + ',0,0)';
        });
      }
    }

    svg.data()[0].lastTicks_g = ticks_g;
    svg.data()[0].lastTicks = last_ticks;
    svg.data()[0].lastTickLabels = last_tick_labels;

    return bar;
  };

  bar._group_x_pos = function(config, group, pos) {
    var numberSpaces = group.pos + 1,
        numberBars = group.group_idx_start + pos;
    return (numberSpaces * config.group_spacing) +
           (numberBars * config.dim_sz);
  };

  bar._groupedaggs_agg_getter = function(config, d, i) {
    if (config.color_agg_cols) {
      var column = config.color_agg_cols[i],
          // When aggregate is a percentage,
          // it is calculated on frontend and stored under the color column's name
          dIdx = config.agg_is_percentage ? column.colname(): column.colindex();
      return Number(d[dIdx]);
    }
    return config.agg_getter(d);
  };

  bar._draw_cell_marks = function(svg, config, x_groups) {
    var is_negative = bar.aggs_have_negative(config);

    config.label_col = bar.getLabelCol(config, lib);
    let labelCol = config.label_col;

    let curr_marks_g = svg.data()[0].lastMarksG;
    if (!curr_marks_g) {
      curr_marks_g = svg.selectAll('.nosel');
    }

    curr_marks_g = curr_marks_g.data(x_groups, d => d.group);
    curr_marks_g.enter().append('g').classed('arc-bar-group', true);

    const all_groups = {};
    curr_marks_g.each(function(x_group) {
      all_groups[x_group.group] = true;
      let data;
      if (config.color_agg_cols) {
        data = config.color_agg_cols.map(function(column) {
                        var data = $.extend({}, x_group.data[0]);
                        data.color = column.colname();
                        return data;
                      });
      } else {
        data = x_group.data;
      }

      let rect_update = d3.select(this).selectAll('rect').data(data);
      rect_update.enter().append('rect');

      addPropsToRectangles(rect_update, x_group);

      let labelColFunc;
      if (labelCol) {
        labelColFunc = function() {
          return labelCol;
        };
      } else if (inlineMeasureDisplay()) {
        if (config.color_agg_cols && config.color_agg_cols.length > 0) {
          labelColFunc = function(d, i) {
            return config.color_agg_cols[i];
          };
        } else {
          labelColFunc = function() {
            return config.columns[config.agg_start_idx];
          };
        }
      }

      const groupFilter = (d) => x_group.group === d[0];

      if (labelColFunc) {
        drawLabels(svg, config, labelColFunc, data, null, groupFilter);
      }
    });

    function addPropsToRectangles(sel, x_group) {
      sel
        .attr('x', function(d, i) {
          if (config.x_is_dim) {
            d.x = bar._group_x_pos(config, x_group, i);
            return d.x;
          }

          var agg = bar._groupedaggs_agg_getter(config, d, i);
          d.x = bar._get_agg_x(agg, config, is_negative);
          return d.x;
        })
        .attr('y', function(d, i) {
          if (config.x_is_dim) {
            var agg = bar._groupedaggs_agg_getter(config, d, i);
            d.y = bar._get_agg_y(agg, config, is_negative, i);
            return d.y;
          }

          d.y = bar._group_x_pos(config, x_group, i);
          return d.y;
        })
        .attr('width', function(d, i) {
          if (config.x_is_dim) {
            d.w = config.dim_sz - 1;
            return d.w;
          }

          var agg = bar._groupedaggs_agg_getter(config, d, i);
          d.w = bar._get_agg_width(agg, config, is_negative);
          return d.w;
        })
        .attr('height', function(d, i) {
          if (config.x_is_dim) {
            const agg = bar._groupedaggs_agg_getter(config, d, i);
            d.h = bar._get_agg_height(agg, config, is_negative, i);
            return d.h;
          }

          d.h = config.dim_sz - 1;
          return d.h;
        })
        .style('fill', function(d, i) {
          if (config.color_agg_cols) {
            return config.color(config.color_agg_cols[i].colname());
          }

          d.color = config.color(config.color_getter(d));
          return d.color;
        })
        .style('opacity', lib.colorOpacity());
    }

    var m = svg.selectAll("rect");
    arc.marks()
       .permitPivot(bar.enableDrillInto())
       .viz_lib(markerLib ? markerLib : lib)
       .marker_name('primary')
       .addSelection(m)
       .legendValueGetter((d) => {
         if (d) {
          return config.color_is_dim ? d[config.color_column.colindex()] : d.color;
         }

         return '';
       })
       .text_fn(function(d) {
         return bar._create_tooltip(d, config);
       })
       .params_fn(function(d, i, e, dimsOnly) {
          return lib.params_from_columns(d, config.columns, dimsOnly);
       })
       .highlight_on_select(true)
       .activate();

    svg.data()[0].lastMarksG = curr_marks_g;

    return bar;
  };

  // Returns true if there are any aggregates with negative values
  bar.aggs_have_negative = function(config, index) {
    var cols;
    if (config.color_enabled && config.color_agg_cols && !mark_type === 'groupedbar') {
      cols = config.color_agg_cols;
    } else if (config.combined_visual || mark_type === 'groupedbar') {
      if (config.y_col.secondaryaxis() || (config.color_agg_cols?.[index]?.secondaryaxis())) {
        return d3.min(config.agg_scale_secondary.domain()) < 0;
      } else {
        return d3.min(config.agg_scale.domain()) < 0;
      }
    } else {
      cols = [ config.x_is_dim ? config.y_col : config.x_col ];
    }

    return cols.some(function (c) {
      var colprop = config.col_properties[c.colindex()];
      return colprop.min < 0;
    });
  };

  bar.getLabelCol = function(config, lib) {
    let labelCols = lib.shelf_columns(config.columns, "label_shelf");
    if (labelCols.length > 0) {
      return labelCols[0];
    }

    return undefined;
  };

  // Used to run function multiple times on different variables given conditional
  // variables is an array of objects with a "run" parameter, or just an array
  bar.testAndRun = function(variables, cb) {
    var results = [];

    variables.forEach(function(v, i) {
      if (typeof (v.run) === "undefined" || v.run) {
        results.push(cb(v, i));
      } else {
        results.push(undefined);
      }
    });

    return results;
  };

  bar.barLabelTextFn = function(config) {
    let usesLabelShelf = false;
    let labelShelfDisplayFormatFn;
    config.label_col = bar.getLabelCol(config, lib);
    let labelCol = config.label_col;

    if (labelCol) {
      usesLabelShelf = true;
      labelShelfDisplayFormatFn = labelCol.coldisplayformat();
    }

    if ((labelCol === undefined) && config.inlineMeasureDisplay) {
      labelCol = config.x_is_dim ? config.y_col : config.x_col;
    }

    if (!labelCol) {
      return undefined;
    }

    let labelColIdx;
    if (config.colnames_to_idx && !config.label_col) {
      // config.colnames_to_idx is not defined for combo visual, so need this
      // check here. Also, don't enter this area if there is a value in the
      // label shelf.
      labelColIdx = config.colnames_to_idx[labelCol.colname()];
    } else {
      labelColIdx = labelCol.colindex();
    }

    let getLabelText = function(d, indexIncrement=0) {
      const columnIndex = labelColIdx + indexIncrement;
      const text = usesLabelShelf ? d[labelColIdx] : d[columnIndex];
      const coldisplayformat = usesLabelShelf ?
        labelShelfDisplayFormatFn :
        config.columns[columnIndex]?.coldisplayformat();

      if (coldisplayformat) {
        return coldisplayformat(text);
      }

      return lib.default_formatter(text);
    };

    return getLabelText;
  };

  return bar;

  // Helper functions

  // returns back agg_cols
  function getAggCols(columns, x_is_dim) {
    var agg_cols = lib.last_agg_columns(columns,
        x_is_dim ? y_shelf: x_shelf);
    return agg_cols;
  }
};

arc.visual.plugins.bars = function() {
  var bars = arc.visual.plugins.bar();
  bars.mark_type('bar');
  return bars;
};

arc.visual.plugins.bar_extension = function() {
  var ext = function() { return "arc.visual.plugins.bar_extension"; };

  ext.viz_name = function() {
    return "trellis-bars";
  };

  ext.viz_tooltip = function() {
    return "bars";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_stackedbar.png?v=1";
  };

  ext.viz = arc.visual.plugins.bars;

  return ext;
};

arc.visual.plugins.tline = function() {
  var line = arc.visual.plugins.bar();
  line.mark_type('line');
  return line;
};

arc.visual.plugins.tline_extension = function() {
  var ext = function() { return "arc.visual.plugins.tline_extension"; };

  ext.viz_name = function() {
    return "trellis-lines";
  };

  ext.viz_tooltip = function() {
    return "lines";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_lines.png?v=1";
  };

  ext.viz = arc.visual.plugins.tline;

  return ext;
};

arc.visual.plugins.area = function() {
  var line = arc.visual.plugins.bar();
  line.mark_type('area');
  return line;
};

arc.visual.plugins.area_extension = function() {
  var ext = function() { return "arc.visual.plugins.area_extension"; };

  ext.viz_name = function() {
    return "trellis-areas";
  };

  ext.viz_tooltip = function() {
    return "areas";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_stackedarea.png?v=1";
  };

  ext.viz = arc.visual.plugins.area;

  return ext;
};

arc.visual.plugins.groupedbar = function() {
  var line = arc.visual.plugins.bar();
  line.mark_type('groupedbar');
  return line;
};

arc.visual.plugins.groupedbar_extension = function() {
  var ext = function() { return "arc.visual.plugins.groupedbar_extension"; };

  ext.viz_name = function() {
    return "trellis-groupedbars";
  };

  ext.viz_tooltip = function() {
    return "grouped bars";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_groupedbars.png?v=1";
  };

  ext.viz = arc.visual.plugins.groupedbar;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.bar_extension());
arc.visual.extensions.push(arc.visual.plugins.tline_extension());
arc.visual.extensions.push(arc.visual.plugins.area_extension());
arc.visual.extensions.push(arc.visual.plugins.groupedbar_extension());

}());
