window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

arc.visual.barDrawHelper = function() {
  let b = function() { return 'arc.visual.barDrawHelper'; };

  let lib,
      config,
      cellConfig,
      bar,
      enableTransitions,
      previousComputedData,
      computedData,
      colorFetcher = arc.canvas.colorFetcher(),
      deleted;

  b.cleanup = function() {
    deleted = true;
    config = undefined;
    cellConfig = undefined;
    previousComputedData = undefined;
    computedData = undefined;
    colorFetcher = undefined;
  };

  b.enableTransitions = function(_) {
    if (!arguments.length) return enableTransitions;
    enableTransitions = _;
    return b;
  };

  b.previousComputedData = function(_) {
    if (!arguments.length) return previousComputedData;
    previousComputedData = _;
    return b;
  };

  b.computedData = function() {
    return computedData;
  };

  b.bar = function(_) {
    if (!arguments.length) return bar;
    bar = _;
    return b;
  };

  b.lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    return b;
  };

  b.config = function(_) {
    if (!arguments.length) return config;
    config = _;
    return b;
  };

  b.cellConfig = function(_) {
    if (!arguments.length) return cellConfig;
    cellConfig = _;
    return b;
  };

  function labelFontSize() {
    let fontSize = lib.getLabelFontSize() || 10;
    if (!config.x_is_dim) {
      if (fontSize > config.dim_sz - 2) {
        fontSize = config.dim_sz - 2;
      }
    }

    return fontSize;
  }

  function barComputeData() {
    const outsideColor = '#222',
          is_negative = bar.aggs_have_negative(config),
          fixedColor = config.color(),
          opacity = lib.colorOpacity();

    let yOffset = 0;

    let labelTextFn = bar.barLabelTextFn(config);

    let fontSize = labelFontSize();

    config.rows.forEach(function(d) {
      d.color = config.color_enabled ?
          config.color(config.color_getter(d)) : fixedColor;
      d.x = bar._get_x(d, config, is_negative);
      d.y = bar._get_y(d, config, is_negative);

      if (config.isWaterfall) {
        if (i !== config.rows.length - 1) {
          d.y -= yOffset;
          let agg = config.agg_getter(d);
          let h = bar._get_height(d, config, is_negative);
          yOffset += Math.abs(h);
        }
      }

      d.w = bar._get_width(d, config, is_negative);
      d.h = bar._get_height(d, config, is_negative);

      if (!labelTextFn) {
        d.blabel = null; // Need to reset the label
        return;
      }

      d.blabel = labelTextFn(d);
      const labelHeight = lib.axis_values_maxbounds([d.blabel], undefined, fontSize).height;
      const barLabelPadding = 2;

      if (config.x_is_dim) {
        d.lx = d.x + ((d.w || 0) / 2);

        // If label fits inside bar put it there, or else put it on top.
        if (d.h && (d.h > labelHeight)) {
          d.ly = d.y + labelHeight - barLabelPadding;
          d.lcolor = arc.colorpalette.colorForBrightness(d.color, opacity);
        } else {
          d.ly = d.y - barLabelPadding;
          d.lcolor = arc.colorpalette.colorForBrightness(
              lib.getBackgroundColor() || '#fff', opacity);
        }

        d.lanchor = 'center';
      } else {
        d.lbaseline = 'middle';
        let bounds = lib.axis_values_maxbounds([d.blabel], undefined, fontSize);

        // If stacked, use d.x and do text-anchor center. For grouped bar and
        // bar, if the bar h is less than 20 pixels, place the text on the right
        // of the bar and use text-anchor start. Otherwise, place the text on
        // the left of the bar and use text-anchor end.
        if (d.w === undefined) {
          d.lx = d.x;
          d.lcolor = outsideColor;
          d.lanchor = 'center';
        } else if (d.w < config.width / 2) {

          // For horizontal bars, draw to the right of the bar if the width of the bar
          // is smaller than half the width of the screen
          // Note that if we draw to the right, longer text gets clipped off
          // automatically
          d.lx = d.x + d.w + 5;
          d.lcolor = outsideColor;
          d.lanchor = 'start';
        } else {
          d.lx = d.x + d.w - 5;
          d.lcolor = arc.colorpalette.colorForBrightness(d.color, opacity);
          d.lanchor = 'end';

          // When we draw within the bar, longer text does not get clipped off automatically
          // Find the right label contents so that we can draw within the bar
          let lmaxw = d.lx - d.x - 5;
          let labelWidth = bounds.width;
          let l = d.blabel.length;
          while (labelWidth > lmaxw && l > 0) {
            --l;
            d.blabel = d.blabel.substring(0, l) + '...';
            bounds = lib.axis_values_maxbounds([d.blabel], undefined, fontSize);
            labelWidth = bounds.width;
          }
        }

        d.ly = d.y + (d.h / 2);
      }

      d.lcolor = lib.getLabelColor() || d.lcolor;
    });
  }

  function barDrawAllCanvas(inTransition) {
    // Draw the main canvas...
    barDrawCanvas(cellConfig.mainCanvasConfig, inTransition);

    if (inTransition) {
      return;
    }

    // ...the hidden canvas
    barDrawCanvas(cellConfig.hiddenCanvasConfig);

    // ...and the highlight canvas
    barDrawCanvas(cellConfig.highlightCanvasConfig);

    barSetupTooltips();
  }

  function barSetupTooltips() {
    let {
      hiddenCanvasConfig,
      highlightCanvasConfig,
      overlayContainer,
    } = cellConfig;

    function text_fn(d) {
      return bar._create_tooltip(d, config);
    }

    function params_fn(d, dimsOnly) {
      if (bar.isHistogram()) {
        return;
      }

      return lib.params_from_columns(d, config.columns, dimsOnly);
    }

    function highlight_fn(row) {
      if (highlightCanvasConfig.drawRows) {
        const rows = row === undefined ? [] : [row];
        highlightCanvasConfig.drawRows(rows);
      }
    }

    if (cellConfig.marks) {
      let tooltips = cellConfig.marks.tooltips();
      if (tooltips) {
        tooltips.disable();
      }

      cellConfig.marks.destroy();
      cellConfig.marks = undefined;
    }

    cellConfig.marks =
      arc.canvas.handleTooltips({
        mouseoverCanvas: overlayContainer,
        hiddenCanvas: hiddenCanvasConfig.canvas,
        hiddenColorMap: hiddenCanvasConfig.colorMap,
        mark_type: config.curr_mark_type,
        viz_lib: lib,
        text_fn,
        params_fn,
        highlight_fn,
        permitPivot: bar.enableDrillInto(),
      });
  }

  function barDrawCanvas(canvasConfig, inTransition) {
    let rows = computedData;

    let {
      canvas,
      hidden,
      highlight,
    } = canvasConfig;

    let drawFullWidth,
        drawFullHeight;

    if (hidden) {
      // For simple bars, we want to highlight the bar if the user moves
      // the mouse along the same axis. We do this by drawing long hidden
      // bars along the length of the data
      if (!config.combined_visual && !config.color_enabled) {
        if (config.x_is_dim) {
          drawFullHeight = true;
        } else {
          drawFullWidth = true;
        }
      }
    }

    let colorMap = {};
    let context = canvas.node().getContext('2d');

    if (!hidden && !highlight) {
      // Note that we do NOT set the globalAlpha for the hidden
      // canvas: otherwise, we end up with a different color map
      // from the one we expect when we read back colors.
      context.globalAlpha = lib.colorOpacity();
    }

    if (highlight) {
      context.lineWidth = 1;
      context.strokeStyle = arc.canvas.highlightStrokeStyle;
    }

    let font = lib.getVisualFont();
    let fontSize = labelFontSize();
    context.font = `${fontSize}px ${font}`;

    let fullWidth = config.width + config.margin.left + config.margin.right,
        fullHeight = config.height + config.margin.top + config.margin.bottom;

    if (highlight) {
      // We may be redrawing this on scroll
      // Always clear the current rectangle so there are no highlights left behind
      let ratio = arc.canvas.pixelRatio();
      context.setTransform(ratio, 0, 0, ratio, 0, 0);
      context.translate(config.margin.left, config.margin.top);
      context.clearRect(-config.margin.left, -config.margin.top, fullWidth, fullHeight);
    } else {
      drawRows(rows);
    }

    function drawRows(rows) {
      if (deleted) {
        return;
      }

      let ratio = arc.canvas.pixelRatio();
      context.setTransform(ratio, 0, 0, ratio, 0, 0);
      context.translate(config.margin.left, config.margin.top);
      context.clearRect(-config.margin.left, -config.margin.top, fullWidth, fullHeight);
      rows.forEach(function(row) {
        let color;
        if (hidden) {
          color = colorFetcher();
          colorMap[color] = row.row;
        } else {
          color = row.color;
        }

        context.fillStyle = color;
        context.beginPath();
        context.rect(
            drawFullWidth ? 0 : row.x,
            drawFullHeight ? 0 : row.y,
            drawFullWidth ? config.width : row.w,
            drawFullHeight ? config.height : row.h);
        context.fill();

        if (highlight) {
          context.stroke();
        } else {
          if ((config.x_is_dim && row.w < 2) ||
              (!config.x_is_dim && row.h < 2)) {
            context.strokeStyle = color;
            context.lineWidth = 1;
            context.stroke();
          }
        }

        // Transitioning measure labels is a bit tricky because of
        // interpolation between two numbers bringing in fractions.
        // Instead, we do not display labels in transition.
        if (!inTransition && row.blabel) {
          if (row.lbaseline) {
            context.textBaseline = row.lbaseline;
          }

          context.textAlign = row.lanchor;
          context.fillStyle = row.lcolor;
          context.fillText(row.blabel, row.lx, row.ly);
        }
      });
    }

    canvasConfig.drawRows = drawRows;
    canvasConfig.colorMap = colorMap;
  }

  // Previously the trellis highlight after any visual transition didn't work perfectly,
  // as it it was behind the real computed data(stored the previous value).
  // If computed data changes it has be adjusted... as well as the canvases
  b.adjustTrellisHighlight = () => {
    const legendConfig = {
    rows: b.computedData()?.layers ?? [],
    getLegendFromRow: (layer) => layer.group,
    highlightFn: (layers) => {
        const [layer] = layers;
        if (layer) {
          b.cellConfig()?.highlightCanvasConfig?.drawRows?.(layer.data, layer);
        } else {
          b.cellConfig()?.highlightCanvasConfig?.drawRows?.([]);
        }
      },
    };
    config?.trellis.addLegendHighlights(legendConfig);
  };

  b.draw = function() {
    let {
      svg,
      viz_settings,
    } = cellConfig;

    barComputeData();

    function rowToData(row) {
      let {
        x,
        y,
        w,
        h,
        blabel,
        lx,
        ly,
        lanchor,
        lbaseline,
        lcolor,
        color,
      } = row;

      let r = {
        x,
        y,
        w,
        h,
        blabel,
        lx,
        ly,
        lanchor,
        lbaseline,
        lcolor,
        color,
        row,
      };

      return r;
    }

    computedData = config.rows.map(rowToData);

    if (config.dim_is_date && !config.dim_is_datetime &&
      !config.dynamic_bar_width && !config.combined_visual) {
      bar.adjustAxes(config.x_is_dim,
          config.dim_sz / 2,
          cellConfig,
          config.dim_scale.range());
    }

    if (!enableTransitions) {
      barDrawAllCanvas();
      b.adjustTrellisHighlight();
      return b;
    }

    if (!previousComputedData) {
      // generate it
      previousComputedData = [];

      computedData.forEach(function(data) {
        let d = $.extend({}, data);

        if (config.x_is_dim) {
          d.h = 0;
          d.y = config.y_scale.range()[0];
        } else {
          d.w = 0;
        }

        previousComputedData.push(d);
      });
    }

    let transition = arc.transition.transition()
                        .visualId(lib.report_id())
                        .select(cellConfig.mainCanvasConfig.canvas.node())
                        .profileData([
                          [
                            'rows',
                            config.rows.length,
                          ],
                          [
                            'visual',
                            config.curr_mark_type,
                          ],
                        ]);

    let oldData = previousComputedData,
        newData = computedData;

    computedData = oldData;
    barDrawAllCanvas(/* inTransition */ true);
    computedData = newData;

    let oldDataReordered = [];

    {
      let groupToIdx = {};
      newData.forEach(function(d, i) {
        groupToIdx[config.dim_getter(d.row)] = i;
      });

      oldData.forEach(function(d) {
        let newIdx = groupToIdx[config.dim_getter(d.row)];
        if (newIdx !== undefined) {
          oldDataReordered[newIdx] = d;
        }
      });
    }

    let interpolation = d4.interpolate(oldDataReordered, newData);

    transition
      .drawFn(function(t) {
        if (deleted) {
          return;
        }

        let rowsData;

        let inTransition = true;

        if (t >= 1) {
          inTransition = false;
          rowsData = newData;
        } else {
          rowsData = interpolation(t);
        }

        computedData = rowsData;
        barDrawAllCanvas(inTransition);
        if (!inTransition) {
          b.adjustTrellisHighlight();
        }
      })
      .start();
  };

  return b;
};

}());
