window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

const LINE_WIDTH_HIDDEN = 6;
const LINE_WIDTH = 3;
const LINE_POINT_RADIUS = 4;

arc.visual.lineAreaDrawHelper = function() {
  const l = function() { return 'arc.visual.lineAreaDrawHelper'; };

  let lib;
  let config;
  let cellConfig;
  let bar;
  let colorFetcher = arc.canvas.colorFetcher();
  let enableTransitions;
  let previousComputedData;
  let computedData = {};
  let inRows;
  let showPoints;
  let dim_sz;
  let original_dim_width;
  let multipleTooltips;
  let uniqueDimsSorted;
  let barDimOffset = 0;
  let deleted;

  l.cleanup = function() {
    deleted = true;
    config = undefined;
    cellConfig = undefined;
    bar = undefined;
    colorFetcher = undefined;
    previousComputedData = undefined;
    computedData = undefined;
    inRows = undefined;
    showPoints = undefined;
  };

  l.bar = function(_) {
    if (!arguments.length) return bar;
    bar = _;
    return l;
  };

  l.showPoints = function(_) {
    if (!arguments.length) return showPoints;
    showPoints = _;
    return l;
  };

  l.lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    return l;
  };

  l.config = function(_) {
    if (!arguments.length) return config;
    config = _;
    inRows = config.rows;
    multipleTooltips = config.multipleTooltips;
    dim_sz = config.dim_sz;
    original_dim_width = config.original_dim_width;
    uniqueDimsSorted = config.unique_dims.slice().sort((a, b) => {
      if (a instanceof Date) {
        return a.valueOf() - b.valueOf();
      }

      return d3.ascending(a, b);
    });

    return l;
  };

  l.enableTransitions = function(_) {
    if (!arguments.length) return enableTransitions;
    enableTransitions = _;
    return l;
  };

  l.previousComputedData = function(_) {
    if (!arguments.length) return previousComputedData;
    previousComputedData = _;
    return l;
  };

  l.computedData = function() {
    return computedData;
  };

  l.cellConfig = function(_) {
    if (!arguments.length) return cellConfig;
    cellConfig = _;
    return l;
  };

  function labelFontSize() {
    let fontSize = lib.getLabelFontSize() || 10;
    if (!config.x_is_dim) {
      if (fontSize > config.dim_sz - 2) {
        fontSize = config.dim_sz - 2;
      }
    }

    return fontSize;
  }

  function computeData() {
    const o = lib.colorOpacity();

    let unique_colors = config.unsorted_colors_domain;
    let distinct_groups = unique_colors;
    const fontSize = labelFontSize();

    if (config.isWaterfall) {
      if (unique_colors[unique_colors.length - 1] !== arc.misc.totalColumnName) {
        unique_colors.push(arc.misc.totalColumnName);
      }
    }

    let color_group_getter = config.color_group_getter || config.color_getter;
    const rows_h = {};
    const dimHash = {};
    if (config.color_agg_cols) {
      // If we are coloring by aggregates, make a copy of the rows for
      // each color
      const rows_template = {};
      inRows.forEach((row) => {
        if (config.dim_is_date) {
          rows_template[config.date_format(config.dim_getter(row))] = row;
          rows_template[config.dim_getter(row)] = row;
        } else {
          rows_template[config.dim_getter(row)] = row;
        }
      });

      unique_colors.forEach((color_data) => {
        rows_h[color_data] = rows_template;
      });
    } else {
      // For drawing simple lines and areas, there are no distinct groups
      // Special handling so we can re-use the same code
      if (distinct_groups === undefined) {
        distinct_groups = ['fixed'];
        color_group_getter = function() {
          return 'fixed';
        };

        unique_colors = ['fixed'];
      }

      // If we are coloring by a dimension, split the rows out by color
      distinct_groups.forEach((color_data) => {
        rows_h[color_data] = {};
      });

      inRows.forEach((row) => {
        const currentRowGroup = rows_h[color_group_getter(row)];
        if (!currentRowGroup) {
          return;
        }

        if (config.dim_is_date) {
          currentRowGroup[config.date_format(config.dim_getter(row))] = row;
          currentRowGroup[config.dim_getter(row)] = row;
        } else {
          currentRowGroup[config.dim_getter(row)] = row;
        }
      });
    }

    // If a line or area graph, need to store offset for drawing trendline
    let lineOffset;

    if (!config.combined_visual && config.curr_mark_type === 'bar') {
      lineOffset = bar.getLineXDimOffset(config);
    }

    if (lineOffset === undefined) {
      lineOffset = bar.getLineXDimOffset(config, original_dim_width);
    }

    if (config.mark_is_continuous) {
      if (config.x_is_dim) {
        config.x_dim_offset = lineOffset;
      }
    }

    const dynamicBarOffset = bar.getBarXDimOffset(config);
    config.bar_offset = dynamicBarOffset;

    const labelTextFn = bar.barLabelTextFn(config);
    const includeInternalDimLabels = {};

    if (labelTextFn) {
      config.hasLabels = true;
      if (bar.mark_type() === 'bar' || bar.mark_type() === 'area') {
        config.unique_dims.forEach((dimName) => {
          distinct_groups.forEach((colorName) => {
            if (dimName in rows_h[colorName]) {
              dimHash[dimName] = dimHash[dimName] ? dimHash[dimName] + 1 : 1;
            }
          });
        });

        Object.keys(dimHash).forEach((dimName) => {
          includeInternalDimLabels[dimName] = dimHash[dimName] > 1;
        });
      }
    }

    const layers = [];

    if (config.dim_is_date && !config.mark_is_continuous) {
      barDimOffset = dim_sz / 2;
    }

    distinct_groups.forEach((color_data, i) => {
      // Don't bother processing this color if there's nothing in it
      if (!Object.keys(rows_h[color_data]).length) {
        return;
      }

      const layer = {};

      // If a dimension axis is missing values, use closest value
      let closestRow;
      let missingRowsFromBeginning = [];
      function processRowData(missingRowD) {
        if (closestRow) {
          missingRowD.row = closestRow;
        } else {
          missingRowsFromBeginning.push(missingRowD);
        }
      }

      layer.group = color_data;
      layer.color = config.color(unique_colors[i]);
      const labelColor = lib.getLabelColor() || arc.colorpalette.colorForBrightness(layer.color, o);
      layer.data = [];

      config.unique_dims.forEach((dim_data) => {
        let row;

        if (dim_data in rows_h[color_data]) {
          const origRow = rows_h[color_data][dim_data];
          row = origRow.slice(0);
          row.percentage = origRow.percentage;

          let yValue = config.color_agg_cols ?
            parseFloat(row[config.colnames_to_idx[color_data]]) :
            config.agg_getter(row);

          if (config.agg_is_percentage && config.color_agg_cols) {
            yValue = origRow[color_data];
            row.percentage = yValue;
          }

          if (arc.misc.check_if_value_null(yValue)) {
            if (config.is_stacked) {
              yValue = 0;
            } else {
              return;
            }
          }

          const d = {
            row,
            s: dim_data,
            x: config.dim_scale(dim_data) + lineOffset + barDimOffset,
            y: yValue,
          };

          /**
           * If there is a bar or area should only include the internal dim label if there is more
           * than one bar color.
           */
          if (labelTextFn &&
            (!['bar', 'area'].includes(bar.mark_type()) || includeInternalDimLabels[dim_data])) {
            const text = config.color_is_dim ? labelTextFn(row) : labelTextFn(row, i);
            const labelBounds = lib.axis_values_maxbounds([text], undefined, fontSize);

            d.row.lcolor = labelColor;
            d.row.l_height = labelBounds.height;
            d.row.l_width = labelBounds.width;

            d.row.ldata = {
              text,
              color: arc.colorpalette.colorForBrightness(layer.color, o),
              width: labelBounds.width,
              height: labelBounds.height,
            };
          }

          layer.data.push(d);
          closestRow = d.row;

          if (missingRowsFromBeginning && missingRowsFromBeginning.length) {
            missingRowsFromBeginning.forEach(processRowData);
            missingRowsFromBeginning = [];
          }
        } else {
          const areMissingDatesIgnored = bar.smooth_line() ? bar.smooth_line().isChecked : false;

          const layerKeys = Object.keys(rows_h);
          const isBarChart = config.curr_mark_type === 'bar';
          const isAreaChart = config.curr_mark_type === 'area';
          const isMultiLayeredAreaChart = isAreaChart && layerKeys.length > 1;
          const atleastOneLayerHasData = isMultiLayeredAreaChart &&
            layerKeys.some((layerKey) => rows_h[layerKey][dim_data]);

          const d = {
            s: dim_data,
            x: config.dim_scale(dim_data) + lineOffset + barDimOffset,
            y: 0,
          };

          if (config.dim_is_date && !areMissingDatesIgnored && !isBarChart) {
            d.row = [dim_data, 'Missing'];

            layer.data.push(d);
          } else if (
            (!config.dim_is_date && isAreaChart) ||
              atleastOneLayerHasData || isBarChart
            ) {
            processRowData(d);

            layer.data.push(d);
          }
        }
      });

      if (layer.data.length > 0) {
        if (config.has_secondary_agg_cols) {
          let col;

          /*
           * TODO:
           * We enter here sometimes with config.colnames_to_idx set and sometimes not.
           * This recent addition to a check for config.colnames_to_idx should noe be
           * necessary - all we are trying to do is to see if the column we are working
           * on here should use the secondary axis or not, and the indication for that
           * should be much clearer.
           */
          if (config.colnames_to_idx === undefined) {
            config.columns.some((c) => {
              if (c.colindex() === config.agg_start_idx) {
                col = c;
                return true;
              }

              return false;
            });
          } else if (config.combined_visual) {
            col = config.columns.filter((c) => c.colname() === color_data)[0];
          } else {
            const idx = config.colnames_to_idx[color_data];
            col = config.columns[idx];
          }

          if (col) {
            layer.secondaryaxis = col.secondaryaxis();
          } else if (config.colname_is_secondary) {
            layer.secondaryaxis = config.colname_is_secondary[color_data] || false;
          } else {
            layer.secondaryaxis = false;
          }
        }

        layers.push(layer);
      }
    });

    let area;

    if (config.is_stacked) {
      {
        // It is an existing bug: d3 stack can't handle positive and negative
        // values in the same time!
        // Solution is from here: https://github.com/d3/d3/issues/2265
        const outToHandleNegativeValues = function(dataSeriesCount) {
          let currentXOffsets = [];
          let current_xIndex = 0;
          return function(d, y0, y) {
            if (current_xIndex % dataSeriesCount === 0) {
                currentXOffsets = [0, 0];
            }

            if (y >= 0) {
                d.y0 = currentXOffsets[1];
                d.y = y;
                currentXOffsets[1] += y;
            } else {
                d.y0 = currentXOffsets[0] + y;
                d.y = -y;
                currentXOffsets[0] += y;
            }

            current_xIndex += 1;
          };
        };

        const s = d3.layout.stack()
                      .out(outToHandleNegativeValues(layers.length))
                      .values((d) => d.data);
        s(layers);
      }

      layers.forEach((layer) => {
        const final_values = [];
        layer.data.forEach((d) => {
          final_values.push({
            x: d.x,
            y: d.y,
            y0: d.y0,
            row: d.row,
            dim_data: d.s, // store dim_data for stacked bar highlighting
            s: d.s,
          });
        });

        layer.data = final_values;
      });

      layers.forEach((layer) => {
        layer.data.forEach((d) => {
          if (config.agg_is_log10 && (d.y0 === 0 || (d.y0 + d.y) === 0)) {
            d.dy0 = layer.secondaryaxis ?
              config.agg_scale_secondary.range()[0] :
              config.agg_scale.range()[0];
            d.dy1 = d.dy0;
          } else {
            const scaleFunction = layer.secondaryaxis ?
              config.agg_scale_secondary :
              config.agg_scale;
            d.dy0 = scaleFunction(d.y0);
            d.dy1 = scaleFunction(d.y0 + d.y);
          }
        });
      });

      area = d4.area()
               .curve(bar.curve())
               .x((d) => d.x)
               .y0((d) => d.dy0)
               .y1((d) => d.dy1);
    }

    if (['line', 'highlight'].indexOf(config.curr_mark_type) !== -1) {
      layers.forEach((layer) => {
        layer.data.forEach((row) => {
          if (config.has_secondary_agg_cols && layer.secondaryaxis) {
            row.y = config.agg_scale_secondary(row.y);
          } else {
            row.y = config.agg_scale(row.y);
          }
        });
      });

      area = d4.line()
               .curve(bar.curve())
               .x((d) => d.x)
               .y((d) => d.y);
    }

    layers.forEach((layer) => {
      layer.drawColor = layer.color;

      layer.data.forEach((d) => {
        d.drawColor = layer.drawColor;
      });
    });

    computedData = {
      area,
      layers,
    };
  }

  function preDrawAllCanvas() {
    preDrawCanvas(cellConfig.mainCanvasConfig);
    preDrawCanvas(cellConfig.hiddenCanvasConfig);
    preDrawCanvas(cellConfig.highlightCanvasConfig);
  }

  function drawAllCanvas(inTransition) {
    // Draw the main canvas...
    drawCanvas(cellConfig.mainCanvasConfig, inTransition);

    if (inTransition) {
      return;
    }

    // ...the hidden canvas
    drawCanvas(cellConfig.hiddenCanvasConfig, inTransition);

    // ...and the highlight canvas
    drawCanvas(cellConfig.highlightCanvasConfig, inTransition);

    setupTooltips();
  }

  function preDrawCanvas(canvasConfig) {
    const {
      canvas,
    } = canvasConfig;

    const context = canvas.node().getContext('2d');

    if (config.y_is_dim) {
      context.rotate(Math.PI / 2);
      context.translate(0, -config.width);
    }
  }

  function drawCanvas(canvasConfig, inTransition) {
    const {
      layers,
      area,
    } = computedData;

    const {
      canvas,
      hidden,
      highlight,
    } = canvasConfig;

    const fillShapes = bar.mark_type() === 'area' || bar.mark_type() === 'bar';
        const isBar = bar.mark_type() === 'bar';
        const isLine = bar.mark_type() === 'line';

    const enableLineLabelsSetting = lib.user_settings().find_by_id('enable_line_labels');
    const enableLineLabels = isLine && enableLineLabelsSetting &&
        enableLineLabelsSetting.isChecked;

    const colorMap = {};
        const context = canvas.node().getContext('2d');

    if (!hidden && !highlight) {
      // Note that we do NOT set the globalAlpha for the hidden
      // canvas: otherwise, we end up with a different color map
      // from the one we expect when we read back colors.
      context.globalAlpha = lib.colorOpacity();
    }

    if (highlight) {
      context.lineWidth = arc.canvas.highlightLineWidth;
      context.strokeStyle = arc.canvas.highlightStrokeStyle;
    } else {
      context.lineWidth = LINE_WIDTH;
    }

    const font = lib.getVisualFont();
    const fontSize = labelFontSize();
    context.font = `${fontSize}px ${font}`;

    const fullWidth = config.width + config.margin.left + config.margin.right;
        const fullHeight = config.height + config.margin.top + config.margin.bottom;

    if (!highlight) {
      drawRows();
    }

    function drawOnePoint(row) {
      context.beginPath();
      if (fillShapes) {
        context.arc(row.x, row.dy1, LINE_POINT_RADIUS, 0, 2 * Math.PI);
      } else {
        context.arc(row.x, row.y, LINE_POINT_RADIUS, 0, 2 * Math.PI);
      }

      context.stroke();
    }

    function placeLabels(ldata, row, config) {
      const dy = Math.abs(row.dy1 - row.dy0);
      context.fillStyle = lib.getLabelColor() || ldata.color;
      if (config.x_is_dim && dy > (ldata.height + 5) && dim_sz > ldata.width) {
        context.beginPath();
        context.textAlign = 'start';
        context.fillText(ldata.text, row.x - ldata.width / 2, row.dy1 + ldata.height);
      } else if (dy > (ldata.width + 5)) {
        context.translate(row.x, row.dy0);
        context.rotate(-Math.PI / 2);
        context.beginPath();
        context.textBaseline = 'middle';
        context.textAlign = 'end';
        context.fillText(ldata.text, dy - 5, 0);
        context.rotate(Math.PI / 2);
        context.translate(-row.x, -row.dy0);
      }
    }

    function highlightDrawRows(rows, rowLayer) {
      if (isBar) {
        context.lineWidth = 1;

        rows.forEach((row) => {
          context.fillStyle = row.drawColor;
          context.strokeStyle = 'white';

          context.beginPath();
          context.rect(row.x - dim_sz / 2, row.dy0, dim_sz, row.dy1 - row.dy0);
          context.fill();
          context.stroke();

          if (!inTransition && row.row && row.row.ldata) {
            const { ldata } = row.row;
            placeLabels(ldata, row, config);
          }
        });

        return;
      }

      if (rowLayer) {
        context.fillStyle = rowLayer.drawColor;
        if (fillShapes) {
          context.lineWidth = 1;
          context.beginPath();
          area(rowLayer.data);
          context.fill();
        } else {
          context.beginPath();
          area(rowLayer.data);
          context.strokeStyle = rowLayer.drawColor;
          context.stroke();
        }
      }

      rows.forEach((row) => {
        if (!rowLayer) {
          context.fillStyle = row.drawColor;
          context.strokeStyle = row.drawColor;
        }

        if (showPoints || !rowLayer) {
          if (fillShapes) {
            context.beginPath();
            context.arc(row.x, row.dy0, LINE_POINT_RADIUS, 0, 2 * Math.PI);
            context.fill();
            context.stroke();
            context.beginPath();
            context.arc(row.x, row.dy1, LINE_POINT_RADIUS, 0, 2 * Math.PI);
            context.fill();
            context.stroke();
          } else {
            drawOnePoint(row);
            context.fill();
          }
        }
      });
    }

    function hiddenDrawRows(rows, rowLayer) {
      if (multipleTooltips) {
        if (layers.length === 0) {
          return;
        }

        const layer = layers[0];
        layer.hiddenColor = colorFetcher();
        colorMap[layer.hiddenColor] = layer;

        context.fillStyle = layer.hiddenColor;
        context.beginPath();
        if (config.y_is_dim) {
          context.rect(-config.margin.top, -config.margin.left, fullHeight, fullWidth);
        } else {
          context.rect(-config.margin.left, -config.margin.top, fullWidth, fullHeight);
        }

        context.fill();

        return;
      }

      context.lineWidth = LINE_WIDTH_HIDDEN;

      layers.forEach((layer) => {
        layer.hiddenColor = colorFetcher();
        colorMap[layer.hiddenColor] = layer;

        context.strokeStyle = layer.hiddenColor;
        context.fillStyle = layer.hiddenColor;

        if (isBar) {
          layer.data.forEach((row) => {
            context.beginPath();
            context.rect(row.x - dim_sz / 2, row.dy0, dim_sz, row.dy1 - row.dy0);
            context.fill();
          });

          return;
        }

        context.beginPath();
        area(layer.data);
        context.stroke();

        if (fillShapes) {
          context.fill();
        }
      });

      if (isBar || fillShapes) {
        return;
      }

      layers.forEach((layer) => {
        context.strokeStyle = layer.hiddenColor;
        context.fillStyle = layer.hiddenColor;
        layer.data.forEach((row) => {
          drawOnePoint(row);
          context.fill();
        });
      });
    }

    function drawRows(rows, rowLayer) {
      if (!config) return;
      if (config.y_is_dim) {
        context.clearRect(-config.margin.top, 0, fullHeight, fullWidth);
      } else {
        context.clearRect(-config.margin.left, -config.margin.top, fullWidth, fullHeight);
      }

      area.context(context);

      if (highlight) {
        highlightDrawRows(rows, rowLayer);
        return;
      }

      if (hidden) {
        hiddenDrawRows(rows, rowLayer);
        return;
      }

      if (isBar) {
        context.lineWidth = 1;
        context.strokeStyle = 'white';
      }

      const isLineStyleOn = bar.isLineStyleFromColor();
      if (isLineStyleOn) {
        if (!config.lineStyleScale) {
          bar.updateLineStyleScale(config);
        }
      }

      layers.forEach((layer) => {
        context.fillStyle = layer.drawColor;
        if (dim_sz <= 2 || !isBar) {
          context.strokeStyle = layer.drawColor;
          if (isLineStyleOn) {
            const cl = layer.group.split(', ');
            const c = cl[cl.length - 1];
            context.setLineDash(config.lineStyleScale(c));
          }
        }

        if (isBar) {
          layer.data.forEach((row) => {
            const dy = row.dy1 - row.dy0;
            context.beginPath();
            context.rect(row.x - dim_sz / 2, row.dy0, dim_sz, dy);
            context.fill();
            context.stroke();

            if (!inTransition && row.row && row.row.ldata) {
              const { ldata } = row.row;
              placeLabels(ldata, row, config);
              context.fillStyle = layer.drawColor;
            }
          });

          return;
        }

        if (layer.data.length === 1) {
          layer.data.forEach((row) => drawOnePoint(row));

          return;
        }

        context.beginPath();
        area(layer.data.filter((d) => d !== undefined));

        if (fillShapes) {
          context.fill();
        } else {
          context.stroke();
        }
      });

      if (enableLineLabels) {
        const prev = [];
        const labelFont = lib.getVisualFont();
        const labelFontSize = lib.getLabelFontSize() || 10;
        const maximumLabelWidth = lib.user_settings().find_by_id('maximum_label_width').value();

        context.font = `${labelFontSize}px ${labelFont}`;

        layers.forEach((layer) => {
          if (layer.data && layer.data.length > 0) {
            const lastPoint = layer.data[layer.data.length - 1];
            context.fillStyle = layer.drawColor;
            context.textBaseline = 'middle';

            const tm = context.measureText(layer.group, lastPoint.x + 5, lastPoint.y);
            const w = tm.width;
            const h = tm.actualBoundingBoxAscent + tm.actualBoundingBoxDescent;
            const x = lastPoint.x + 5;
            const { y } = lastPoint;

            const overlap = prev.some((overl) => {
              if (x > (overl.x + overl.w) || (x + w) < overl.x ||
                  y > (overl.y + overl.h) || (y + h) < overl.y) {
                return false;
              }

              return true;
            });

            if (!overlap) {
              context.textAlign = 'left';
              context.textBaseline = 'middle';

              context.fillText(layer.group, x, y, maximumLabelWidth);
              prev.push({
                x,
                y,
                w,
                h,
              });
            }
          }
        });
      }

      if (!config.agg_is_percentage && !inTransition && config.is_stacked) {
        const textFn = barLabelTextFn();
        if (textFn) {
          const labelFont = lib.getVisualFont();
          const labelFontSize = lib.getLabelFontSize() || 10;
          const labelTextColor = lib.getLabelColor() ||
                                 arc.colorpalette.colorForBrightness(
                                     lib.getBackgroundColor() || '#fff');
          context.font = `${labelFontSize}px ${labelFont}`;
          context.fillStyle = labelTextColor;
          context.textAlign = config.y_is_dim ? 'start' : 'center';
          const lastLayer = layers[layers.length - 1];

          lastLayer.data.forEach((d) => {
            const y = d.y + d.y0;
            const xpos = d.x;
                const ypos = d.dy1 - 3;

            context.beginPath();
            if (config.y_is_dim) {
              context.translate(xpos, ypos);
              context.rotate(-Math.PI / 2);
              context.fillText(textFn(y), 0, 0);
              context.rotate(Math.PI / 2);
              context.translate(-xpos, -ypos);
            } else {
              context.fillText(textFn(y), xpos, ypos);
            }
          });
        }
      }

      // Draw points after the areas are drawn, so that the points are always on top
      // i.e. do not move the code below into the for loop above
      if (!isBar && showPoints) {
        layers.forEach((layer) => {
          context.fillStyle = layer.drawColor;
          context.strokeStyle = layer.drawColor;
          layer.data.forEach((row) => drawOnePoint(row));
        });
      }

      if (!isBar && config.hasLabels) {
        // offset distance of label from point
        const inlineLabelOffset = 6;
        const isArea = lib.viz_type() === 'trellis-areas';
        const includeRect = !config.combined_visual;
        const textBackgroundColor = lib.getBackgroundColor() || '#fff';
        const labelTextColor = lib.getLabelColor() ||
                          arc.colorpalette.colorForBrightness(textBackgroundColor);
        const backgroundPadding = 2;
        if (isArea) {
          if (layers.length === 1) return;
          context.shadowBlur = 5;
          context.shadowColor = textBackgroundColor;
          context.textAlign = 'start';
        }

        const lineValues = {};
        const prev = [];

        layers.forEach((layer) => {
          layer.data.forEach(function addLabels(d) {
            if (!d.row || !d.row.ldata) return;
            const labelWidth = d.row.ldata.width;
            const labelHeight = d.row.ldata.height;
            const x = d.x - (labelWidth / 2);
            let y;
            if (!lineValues[d.x]) lineValues[d.x] = [];

            if (isArea) {
              if (Math.abs(d.dy1 - d.dy0) < (labelHeight + 5)) return;
              y = d.dy1 + labelHeight;
            } else if (includeRect) {
              if (lineValues[d.x].some((val) => Math.abs(val - d.y) <= labelHeight)) return;
              y = d.y;
              lineValues[d.x].push(d.y);
            } else {
              y = d.y - inlineLabelOffset;
            }

            const tm = context.measureText(d.row.ldata.text, x, y);
            const w = tm.width;
            const h = tm.actualBoundingBoxAscent + tm.actualBoundingBoxDescent;

            const overlap = prev.some((overlay) => {
              if (x > (overlay.x + overlay.w) || (x + w) < overlay.x ||
                  y > (overlay.y + overlay.h) || (y + h) < overlay.y) {
                return false;
              }

              return true;
            });

            if (!overlap) {
              if (includeRect) {
                context.fillStyle = textBackgroundColor;
                context.fillRect(x - backgroundPadding,
                  y + backgroundPadding,
                  labelWidth + 2 * backgroundPadding,
                  -labelHeight);
              }

              context.fillStyle = labelTextColor;
              context.fillText(d.row.ldata.text, x, y);
              prev.push({
                x,
                y,
                w,
                h,
              });
            }
          });
        });
      }
    }

    canvasConfig.drawRows = drawRows;
    canvasConfig.colorMap = colorMap;
  }

  function barLabelTextFn() {
    if (!config.inlineSumMeasureDisplay) {
      return;
    }

    const labelCol = config.x_is_dim ? config.y_col : config.x_col;

    const coldisplayformat = labelCol.coldisplayformat();

    const getLabelText = function(text) {
      if (coldisplayformat) {
        return coldisplayformat(text);
      }

      return lib.default_formatter(text);
    };

    return getLabelText;
  }

  function setupTooltips() {
    const {
      margin,
    } = config;

    const {
      hiddenCanvasConfig,
      highlightCanvasConfig,
      overlayContainer,
    } = cellConfig;

    function closestRow(layer, compareTo, compareFn) {
      let prevRow;
          let prevDiff;
          let nextRow;
          let nextDiff;

      layer.data
           .filter((row) =>
             // Take out the fake data points we use for drawing
             // the same color as the same area
             // eslint-disable-next-line implicit-arrow-linebreak
              row.row !== undefined)
           .some((row) => {
             const compare = compareFn(row);

             if (compareTo <= compare) {
               nextRow = row;
               nextDiff = compare - compareTo;
               return true;
             }

             prevRow = row;
             prevDiff = compareTo - compare;

             return false;
           });

      if (prevRow) {
        if (!nextRow) {
          return prevRow;
        }

        return (prevDiff < nextDiff ? prevRow : nextRow);
      }

      return nextRow;
    }

    function layerToRow(layer) {
      const $c = $(overlayContainer.node());

      const canvasOffset = $c.offset();
      if (!canvasOffset) {
        return undefined;
      }

      if (!d3.event) {
        return undefined;
      }

      let compareTo;
      if (config.y_is_dim) {
        compareTo = d3.event.pageY - canvasOffset.top - config.margin.top;
      } else {
        compareTo = d3.event.pageX - canvasOffset.left - config.margin.left;
      }

      const selectedRow = closestRow(layer, compareTo, (row) => row.x);

      return selectedRow;
    }

    function getMultipleDimRows(dim) {
      const rows = [];

      computedData.layers.forEach(({ data }) => {
        data.forEach((row) => {
          if (row.s === dim) {
            rows.push(row);
          }
        });
      });

      return rows;
    }

    /*
     * Called in the context of a mouseover or click event to get the closest
     * dimension value to the mouse position.
     *
     * We use the ordinal dimension scale and compare it to either the X or Y
     * mouse position.
     */
    function getClosestDim() {
      const $c = $(overlayContainer.node());
      const canvasOffset = $c.offset();
      if (!canvasOffset) {
        return undefined;
      }

      const scale = config.dim_scale;

      let dimPagePos;
      if (config.y_is_dim) {
        dimPagePos = d3.event.pageY - canvasOffset.top - config.margin.top;
      } else {
        dimPagePos = d3.event.pageX - canvasOffset.left - config.margin.left;
      }

      if (config.dim_is_date) {
        const exactDateUnderMouse = scale.invert(dimPagePos);
              const bisectDate = d3.bisector((d, x) => d.valueOf() - x.valueOf()).left;
              const dimPos1 = bisectDate(uniqueDimsSorted, exactDateUnderMouse);
              const dimPos2 = dimPos1 - 1;

        let dim;
        if (dimPos1 === 0) {
          // dimPos1 is the left-most point
          return uniqueDimsSorted[dimPos1];
        }

        if (dimPos1 >= uniqueDimsSorted.length) {
          // dimPos1 is beyond the right-most point
          return uniqueDimsSorted[dimPos2];
        }

        const dimPagePos1 = scale(uniqueDimsSorted[dimPos1]);
              const dimPagePos2 = scale(uniqueDimsSorted[dimPos2]);

        if (Math.abs(dimPagePos1 - dimPagePos) < Math.abs(dimPagePos2 - dimPagePos)) {
          return uniqueDimsSorted[dimPos1];
        }

        return uniqueDimsSorted[dimPos2];
      }

      const dimPagePos1 = d3.bisect(scale.range(), dimPagePos);
      return scale.domain()[dimPagePos1 - 1];
    }

    function text_fn(layer) {
      if (!multipleTooltips || !config.color_getter) {
        return bar._create_tooltip(layerToRow(layer).row, config);
      }

      const closestDim = getClosestDim();
      const rowsData = getMultipleDimRows(closestDim);
      const kvs = [];

      rowsData.forEach((rowData, i) => {
        if (i === 0) {
          kvs.push(lib.text_from_column(rowData.row, config.dimCol));
        }

        kvs.push({
          key: lib.d_string(config.color_getter(rowData.row)),
          value: lib.d_string(config.agg_getter(rowData.row)),
          formatter: config.columns[config.agg_start_idx].coldisplayformat(),
        });
      });

      return lib.tooltip_from_kvs(kvs);
    }

    function params_fn(layer, dimsOnly) {
      if (!multipleTooltips || !config.color_getter) {
        return lib.params_from_columns(layerToRow(layer).row, config.columns, dimsOnly);
      }

      const closestDim = getClosestDim();
      const rowsData = getMultipleDimRows(closestDim);

      return lib.params_from_columns(
        rowsData[0].row,
        config.columns,
        true, // dimsOnly
        undefined,
        undefined,
        [config.last_color_column.colindex()]
      );
    }

    function highlight_fn(layer) {
      if (highlightCanvasConfig.drawRows) {
        if (layer === undefined) {
          highlightCanvasConfig.drawRows([]);
          return;
        }

        if (!multipleTooltips || !config.color_getter) {
          highlightCanvasConfig.drawRows([layerToRow(layer)]);
          return;
        }

        const closestDim = getClosestDim();
        const rowsData = getMultipleDimRows(closestDim);
        highlightCanvasConfig.drawRows(rowsData);
      }
    }

    if (cellConfig.marks) {
      cellConfig.marks.destroy();
      cellConfig.marks = undefined;
    }

    cellConfig.marks =
      arc.canvas.handleTooltips({
        mouseoverCanvas: overlayContainer,
        hiddenCanvas: hiddenCanvasConfig.canvas,
        hiddenColorMap: hiddenCanvasConfig.colorMap,
        mark_type: config.curr_mark_type,
        viz_lib: lib,
        text_fn,
        params_fn,
        highlight_fn,
        permitPivot: bar.enableDrillInto(),
      });
  }

  // Previously the trellis highlight after any visual transition didn't work perfectly,
  // as it it was behind the real computed data(stored the previous value).
  // If computed data changes it has be adjusted... as well as the canvases
  l.adjustTrellisHighlight = () => {
    const legendConfig = {
    rows: l.computedData()?.layers ?? [],
    getLegendFromRow: (layer) => layer.group,
    highlightFn: (layers) => {
        const [layer] = layers;
        if (layer) {
          l.cellConfig()?.highlightCanvasConfig?.drawRows?.(layer.data, layer);
        } else {
          l.cellConfig()?.highlightCanvasConfig?.drawRows?.([]);
        }
      },
    };
    config?.trellis.addLegendHighlights(legendConfig);
  };

  l.draw = function() {
    computeData();

    bar.adjustAxes(
      config.x_is_dim,
      barDimOffset,
      cellConfig,
      config.dim_scale.range()
    );
    preDrawAllCanvas();

    if (!enableTransitions) {
      drawAllCanvas();
      l.adjustTrellisHighlight();
      return l;
    }

    if (!previousComputedData) {
      // generate it
      previousComputedData = {};
      previousComputedData.layers = [];

      const lowRange = config.agg_scale.range()[0];
      computedData.layers.forEach((layer) => {
        const ly = $.extend({}, layer);
        ly.data = [];
        layer.data.forEach((data) => {
          const d = $.extend({}, data);
          if (d.dy1 !== undefined) {
            d.dy1 = d.dy0;
          } else {
            d.y = lowRange;
          }

          ly.data.push(d);
        });

        previousComputedData.layers.push(ly);
      });
    }

    const transition = arc.transition.transition()
                        .visualId(lib.report_id())
                        .select(cellConfig.mainCanvasConfig.canvas.node())
                        .profileData([
                          [
                            'rows',
                            inRows.length,
                          ],
                          [
                            'visual',
                            config.curr_mark_type,
                          ],
                        ]);

    let oldData = previousComputedData.layers;
        const newData = computedData.layers;

    computedData.layers = oldData;
    drawAllCanvas(/* inTransition */ true);

    oldData = reorder(oldData, newData);

    const interpolation = d4.interpolate(oldData, newData);

    transition
      .drawFn((t) => {
        if (deleted) {
          return;
        }

        let rowsData;

        let inTransition = true;

        if (t >= 1) {
          inTransition = false;
          rowsData = newData;
        } else {
          rowsData = interpolation(t);
        }

        computedData.layers = rowsData;
        drawAllCanvas(inTransition);
        if (!inTransition) {
          l.adjustTrellisHighlight();
        }
      })
      .start();
  };

  function reorderLayer(oldLayer, newLayer) {
    // Reorder to match the date dimension for when dimensions
    // are dates. This help give the effect of data moving to the
    // left/right instead of up/down
    if (!config.dim_is_date) {
      return oldLayer;
    }

    const groupToIdx = {};

    newLayer.data.forEach((d, i) => {
      groupToIdx[d.s] = i;
    });

    const r = [];
    oldLayer.data.forEach((d) => {
      const newIdx = groupToIdx[d.s];
      if (newIdx !== undefined) {
        r[newIdx] = d;
      }
    });

    // Fill in any remaining gaps from the new layer
    r.forEach((d, i) => {
      if (d === undefined) {
        r[i] = newLayer.data[i];
      }
    });

    oldLayer.data = r;
    return oldLayer;
  }

  function reorder(oldData, newData) {
    const oldDataReordered = [];

    {
      const groupToIdx = {};
          const groupToData = {};

      newData.forEach((layer, i) => {
        groupToIdx[layer.group] = i;
        groupToData[layer.group] = layer;
      });

      oldData.forEach((layer) => {
        const newIdx = groupToIdx[layer.group];
        if (newIdx !== undefined) {
          const newLayer = groupToData[layer.group];
          oldDataReordered[newIdx] = reorderLayer(layer, newLayer);
        }
      });
    }

    return oldDataReordered;
  }

  return l;
};
}());
