window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.boxplot = function() {
  var saved_settings = {},
      data_result,
      data_request,

      originalDSreq,
      originalDerivedData,

      // the aggregates added to each row
      addedAggregates = ['q0', 'q1', 'q2', 'q3', 'q4'],
      measureElement,

      lib,
      sort_orders_asc = {},
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  // Keeping same naming as in bar/scatter visual,
  // but x actually means "dimension" and y means "measure"
  const scale_x_independent = arc.report.user_settings.scale_dim_independent();
  scale_x_independent.labels(['Independent dimension scale']);
  const scale_y_independent = arc.report.user_settings.scale_agg_independent();
  scale_y_independent.labels(['Independent measure scale']);
  const x_axis_label = arc.report.user_settings.x_axis_label();
  x_axis_label.labels(['Include dimension axis label']);
  const y_axis_label = arc.report.user_settings.y_axis_label();
  y_axis_label.labels(['Include measure axis label']);
  const show_x_axis = arc.settings.line.showXAxis();
  show_x_axis.labels(['Show dimension axis']);
  const show_y_axis = arc.settings.line.showYAxis();
  show_y_axis.labels(['Show measure axis']);

  const scale_agg_zero = arc.report.user_settings.scale_agg_zero();

  const bar_mark_size = arc.report.user_settings.bar_mark_size();

  const horizontal_gridlines_enabled = arc.report.user_settings.horizontal_gridlines_enabled();
  const vertical_gridlines_enabled = arc.report.user_settings.vertical_gridlines_enabled();

  const horizontalBoxplots =
    arc.visual.setting()
              .id('horizontalBoxplots')
              .tab('Marks')
              .type(arc.visual.settingTypeEnum.CHECKBOX)
              .add_name('horizontalBoxplots')
              .add_label('Make box plots horizontal')
              .value('false');

  const lineWidth =
    arc.visual.setting()
              .id('lineWidth')
              .tab('Marks')
              .type(arc.visual.settingTypeEnum.SLIDER)
              .add_name('lineWidth')
              .add_label('Outline width')
              .min(0)
              .max(10)
              .step(0.5)
              .value(1);

  const numberDimensions =
    arc.visual.setting()
              .id('numberDimensions')
              .tab('Data')
              .type(arc.visual.settingTypeEnum.INTEGER_NOSPINNER)
              .add_name('numberDimensions')
              .add_label('The maximum number of distinct dimensions to visualize')
              .min(0)
              .max(10000000)
              .value('');

  const user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  scale_x_independent,
                                  scale_y_independent,
                                  scale_agg_zero,
                                  x_axis_label,
                                  y_axis_label,
                                  show_x_axis,
                                  show_y_axis,
                                  bar_mark_size,
                                  horizontal_gridlines_enabled,
                                  vertical_gridlines_enabled,
                                  horizontalBoxplots,
                                  lineWidth,
                                  numberDimensions,
                                  arc.settings.fonts.axisLabelFontSize(),
                                  arc.settings.fonts.axisTicksFontSize(),
                                  arc.settings.fonts.visualTrellisFontSize(),
                                ])
                                .removeSetting([
                                  'legend_font_size',
                                  'label_font_size',
                                  'label_color',
                                ]);

  const boxplot = arc.visual.api();

  const shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimension',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',

      // Use counter type to prevent from being added to dsreq
      shelf_type: arc.report.shelf_types.COUNTER,

      // Note that we don't want to apply a metric here, we
      // just want a numeric column. So represent this as
      // a dimension
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  const constraints = {
    minimums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
    },
    maximums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
    },
  };

  boxplot.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return boxplot;
  };

  boxplot.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  boxplot.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return boxplot;
    }

    lib.removeSimpleAggregations(settings);
    saved_settings = settings;

    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return boxplot;
  };

  boxplot.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    const xMessage = [
            lib.createPopoverMessages(['trellisOnly', 'columns.']),
            lib.createPopoverMessages('acceptsMultiple'),
          ],
          yMessage = [
            lib.createPopoverMessages(['trellisOnly', 'rows.']),
            lib.createPopoverMessages('acceptsMultiple'),
          ],
          dimensionMessages = [
            lib.createPopoverMessages('dimensionBox'),
            lib.createPopoverMessages('acceptsOne'),
          ],
          aggregateMessages = [
            lib.createPopoverMessages('aggregateBox'),
            lib.createPopoverMessages('aggregateBoxSupport'),
            lib.createPopoverMessages('acceptsOne'),
          ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      dimensions_shelf: dimensionMessages,
      aggregates_shelf: aggregateMessages,
    });

    return boxplot;
  };

  // Used to check if specific window functions are supported
  boxplot.supportsOptimizedBoxplot = function () {
    return arc.dataconnection.isHadoopOrSparkEngine(lib.dataconnection_type());
  };

  function cleanupDSreq(dsreq) {
    dsreq.derived_data = dsreq.derived_data.filter(dd => dd.label !== 'ntile');
  }

  // Run the few things necessary to commit a data request:
  //   Associate it with the lib and dd object,
  //   point dsreq within the dataRequest object directly to rb().dsreq()
  //   and call 'finalize' on it
  function commitDataRequest(dsreq) {
    const rb = data_request.rb();
    const dsreq_obj = dsreq || rb.dsreq();
    lib.dsreq(dsreq_obj);
    rb.dsreq(dsreq_obj);
    lib.derivedData().dsreq(dsreq_obj);
    data_request.dsreq(dsreq_obj);
  }

  boxplot.viz_getDataRequest = function() {
    // HELPER FUNCTIONS
    // Gets custom expression or colname
    function getMeasure() {
      const components = measureElement.getExpressionComponents(lib.substituteParams);
      return {
        expression: components.expr,

        // keep quotes around alias so no possible conflict with column names
        alias: components.alias,
        isMeasure: false,
        skipDD: true,
      };
    }

    function getCount(dimensions) {
      return {
        expression: `count(*) OVER (PARTITION by ${dimensions})`,
        alias: 'count',
      };
    }

    // Get needed details
    function getNtile(measure, dimensions) {
      return {
        expression: `ntile(4) OVER (PARTITION by ${dimensions} ORDER BY ${measure})`,
        alias: 'ntile',
        isMeasure: false,
      };
    }

    const ddPrefix = [];

    // Adds new derived data to dsreq
    // newDD is an array of objects with "expression" and "alias" properties
    function renameDDCols(elements) {
      return elements.map((el) => {
        // See if alias exists within dimension/measure
        // since dataset_deriveddata.check_colname can't do that properly outside edit modal
        const tables = lib.dataset().tables();
        const expr_alias = arc.misc.splitExprAndAlias(el.expr);
        let alias = expr_alias.alias || expr_alias.expr;
        while (checkAliasExists(alias, tables)) {
          alias = `_${alias}`;
        }

        // Now use rename_colname to rename and avoid clashing with other derived data
        el.expr = lib.derivedData().rename_colname(`${expr_alias.expr} as '${alias}'`);

        // Store dsreq alias so we know what column name to use to prevent needing joins
        // in outer min/max/dimensions select query
        ddPrefix.push(el.expr.match(/as '(.*)'/)[1]);
        return el;
      });
    }

    function checkAliasExists(alias, tables) {
      return tables.some(tabl => tabl.columns().some(hasAlias));

      function hasAlias(col) {
        return col.display_name() === alias;
      }
    }

    function addDerivedData(newDD, DSReq, measureType) {
      const derivedDataList = DSReq.derived_data;
      const DSReqForDD = JSON.parse(JSON.stringify(DSReq));

      // Building this DD dsreq, we remove the DD portion picked up from the base report
      DSReqForDD.derived_data = [];

      // Rename existing dimensions/aggregates so joins can be prevented
      DSReqForDD.dimensions = renameDDCols(DSReqForDD.dimensions);
      DSReqForDD.aggregates = renameDDCols(DSReqForDD.aggregates);

      newDD.forEach(addToDsreq);

      // Now push in this DD
      derivedDataList.push({
        dsreq: DSReqForDD,
        isMeasure: true,
        label: 'ntile',
        type: measureType,
      });

      // HELPER FUNCTION
      function addToDsreq(currentDD) {
        // Add to dsreq
        const expressionType = currentDD.isMeasure ? 'aggregates' : 'dimensions';
        const type = currentDD.isMeasure ? measureType : 'SIMPLE';
        DSReqForDD[expressionType].push({
          type,
          expr: `${currentDD.expression} as '${currentDD.alias}'`,
        });
      }
    }

    // MAIN FUNCTION
    data_request = lib.getCheckedDataRequest(constraints);

    if (!data_request) {
      return data_request;
    }

    // Set measureElement and only allow non-aggregates
    let dimensionIsAgg;
    let dimensionElement;
    let dimensionIdx;
    let nonAggregateTypes = new Set();
    lib.shelfExpressionsCb(true, function(shelfName, _expr, i, shelf_element) {
      if (shelfName === 'aggregates_shelf') {
        measureElement = shelf_element;
      }

      if (shelfName === 'dimensions_shelf') {
        dimensionElement = shelf_element;
        dimensionIdx = i;
      }

      if (shelf_element.expr_hasagg() || shelf_element.expressionIsAggregate()) {
        dimensionIsAgg = true;
        nonAggregateTypes.add(shelfName.replace('_shelf', ''));
      }
    });

    if (dimensionIsAgg) {
      lib.alertMessage(`Please specify non-aggregated ${[...nonAggregateTypes].join(', ')}`);
      return null;
    }

    originalDSreq = null;
    let currentDSReq;

    const rb = data_request.rb(),
          measureType = measureElement.dataset_coltype(),
          measureValue = getMeasure();

    if (boxplot.supportsOptimizedBoxplot()) {
      // Remove alias from dimensions
      const dimensions = rb.dimensions().map(dim => dim.expr.replace(/ as .*$/, '')),
            dimensionsString = dimensions.join(',');

      currentDSReq = rb.dsreq();

      // cleanup dsreq -- remove manually added ones
      cleanupDSreq(currentDSReq);

      originalDSreq = JSON.stringify(currentDSReq);

      // We create derived data for q1, median, q3
      const ntilCols = [
        measureValue,
        getCount(dimensionsString),
        getNtile(measureValue.expression, dimensionsString),
      ];

      addDerivedData(ntilCols, currentDSReq, measureType);

      // prevent joins
      const newDimensions = rb.dimensions().map((dim, i) => {
        dim.expr = dim.expr.replace(/^.+ +as /, `[${ddPrefix[i]}] as `);
        return dim;
      });

      rb.dimensions(newDimensions);

      // Add min, q1, median, q1, max to rb
      rb.addDimension('[ntile]');
      rb.addAggregate('avg([count])');
      rb.addAggregate(`min([${measureValue.alias}])`);
      rb.addAggregate(`max([${measureValue.alias}])`);

      // Don't need filters because already in derived data dsreq
      rb.replaceFilters([]);

      if (numberDimensions.value() !== '') {
        // Order by dimension so all quartiles appear
        const numberOfQuartiles = 4;
        const orderAsc = dimensionElement.order() ? dimensionElement.order().ascending() : true;
        newDimensions[dimensionIdx].order = { asc: orderAsc, pri: 1 };
        rb.limit(parseInt(numberDimensions.value()) * numberOfQuartiles);
      }
    } else {
      console.log('Box visual: Calculation on connection not supported, ' +
                  'box plot values will be calculated locally');
      rb.addDimension(measureValue.expression);
    }

    commitDataRequest(currentDSReq);

    return data_request;
  };

  // Clears manually added derived data
  boxplot.clearDerivedData = function () {
    // Reset dsreq if we added derived data
    if (boxplot.supportsOptimizedBoxplot() && originalDSreq) {
      // Delete unnecessary derived data
      const originalDSreqJSON = JSON.parse(originalDSreq);

      // const originalDerivedDataJSON = originalDSreqJSON.derived_data;
      // lib.derivedData().derivedDataList(originalDerivedDataJSON);
      // data_request.rb().deriveddataList(originalDerivedDataJSON);
      cleanupDSreq(originalDSreqJSON);

      // Reset dsreq
      commitDataRequest(originalDSreqJSON);
    }
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  boxplot.viz_setDataResult = function(result) {
    // Get q1, median, and q3
    // There are multiple ways to get the median. The way we're using is "Method 1"
    // as described here: https://en.wikipedia.org/wiki/Box_plot
    // Use the median to divide the ordered data set into two halves.
      // - If there are an odd number of data points in the original ordered data set,
      //   do not include the median (the central value in the ordered list) in either half.
      // - If there are an even number of data points in the original ordered data set,
      //   split this data set exactly in half.
      // - The lower quartile value is the median of the lower half of the data.
      //   The upper quartile value is the median of the upper half of the data.

    // MAIN FUNCTION
    const optimizedBoxplot = boxplot.supportsOptimizedBoxplot();

    // keeps track of data for grouping aggregates to dimensions
    const oldRows = result.rows();
    const indexToAggregate = {};
    const indexToDimensions = {};
    const dimToIndex = {};

    // For grabbing all aggregates
    const rb = data_request.rb(); // only needed for ntile calc
    const ntileIdx = rb.dimensions().length - 1;
    const aggIndex = result.columns().length;
    const aggGetter = optimizedBoxplot ? row => row.slice(aggIndex) : row => row[aggIndex];
    let currentIndex = 0;
    let numberFiltered = 0;

    // split up the values
    // indexToDimensions keeps track of dimensions we've already added
    // dimensionIdx keeps track of dimensions row
    // indexToAggregate keeps track of all aggregates for each dimension
    oldRows.forEach(function (row) {
      const aggregate = aggGetter(row);
      const dimensions = row.slice(0, aggIndex);
      const dimensionsString = dimensions.join('');
      const dimensionIdx = dimToIndex[dimensionsString];
      const currentNtile = row[ntileIdx];
      if (!optimizedBoxplot && arc.misc.check_if_value_null(aggregate)) {
        numberFiltered++;
        return;
      }

      if (dimensionIdx !== undefined) {
        if (optimizedBoxplot) {
          indexToAggregate[dimensionIdx][currentNtile] = aggregate;
        } else {
          indexToAggregate[dimensionIdx].push(aggregate);
        }
      } else {
        indexToDimensions[currentIndex] = dimensions;
        dimToIndex[dimensionsString] = currentIndex;
        if (optimizedBoxplot) {
          indexToAggregate[currentIndex] = {};
          indexToAggregate[currentIndex][currentNtile] = aggregate;
        } else {
          indexToAggregate[currentIndex] = [aggregate];
        }

        currentIndex += 1;
      }
    });

    if (numberFiltered) {
      const message = `Ignoring ${numberFiltered} rows with null values`;
      if (lib.displayFilterWarnings()) {
        lib.addMessage(message);
      } else {
        console.log(message);
      }
    }

    boxplot.clearDerivedData();

    // For locally calculating values
    function getBoxplotValueLocal(row, elementNumber) {
      const idx = elementNumber - 1;
      const floorIdx = Math.max(Math.floor(idx), 0),
            ceilIdx = Math.max(Math.ceil(idx), 0);

      const floorValue = row[floorIdx],
            ceilValue = row[ceilIdx] || floorValue;

      return (floorValue + ceilValue) / 2;
    }

    // For remotely returned ntile values
    function getMedianNtile(lowerCount, lowerMedian, upperCount, upperMedian) {
      lowerCount = parseFloat(lowerCount);
      lowerMedian = parseFloat(lowerMedian);
      upperCount = parseFloat(upperCount);
      upperMedian = parseFloat(upperMedian);
      if (lowerCount < upperCount) {
        return upperMedian;
      } else if (lowerCount > upperCount) {
        return lowerMedian;
      }

      return (upperMedian + lowerMedian) / 2;
    }

    // get the min, q1, median, q3, and max values and add to new rows
    const newRows = [];
    const maxValue = numberDimensions.value() !== '' ?
                     Math.min(parseInt(numberDimensions.value()), currentIndex) :
                     currentIndex;
    for (var i = 0; i < maxValue; i++) {
      const row = indexToDimensions[i];
      const aggregates = indexToAggregate[i];
      let min, q1, median, q3, max;

      if (optimizedBoxplot) {
        const countIdx = 1;
        const minIdx = 2;
        const maxIdx = 3;

        const q1Aggs = aggregates['1'];
        let q2Aggs = aggregates['2'];
        let q3Aggs = aggregates['3'];
        let q4Aggs = aggregates['4'];

        // Account for cases when 3 or less rows total (some quartiles not full)
        if (!q1Aggs) {
          // Something went wrong, this row has no data
          continue;
        } else if (!q2Aggs) {
          // Only has q1; assume no q3, q4
          q2Aggs = q1Aggs;
          q3Aggs = q1Aggs;
          q4Aggs = q1Aggs;
        } else if (!q3Aggs) {
          // Only has q2, q1; assume no q4
          q3Aggs = q2Aggs;
          q4Aggs = q2Aggs;
          q2Aggs = q1Aggs;
        } else if (!q4Aggs) {
          // Only has q3, q2, q1
          q4Aggs = q3Aggs;
        }

        const q1Count = q1Aggs[countIdx];
        const q2Count = q2Aggs[countIdx];
        const q3Count = q3Aggs[countIdx];
        const q4Count = q4Aggs[countIdx];

        const lowerQ1 = q1Aggs[maxIdx];
        const upperQ1 = q2Aggs[minIdx];
        q1 = getMedianNtile(q1Count, lowerQ1, q2Count, upperQ1);

        const lowerCount = q1Count + q2Count;
        const lowerMedian = q2Aggs[maxIdx];
        const upperCount = q3Count + q4Count;
        const upperMedian = q3Aggs[minIdx];
        median = getMedianNtile(lowerCount, lowerMedian, upperCount, upperMedian);

        const lowerQ3 = q3Aggs[maxIdx];
        const upperQ3 = q4Aggs[minIdx];
        q3 = getMedianNtile(q3Count, lowerQ3, q4Count, upperQ3);

        min = q1Aggs[minIdx];
        max = q4Aggs[maxIdx];
      } else {
        const numberAggregates = aggregates.length + 1;
        const q1Idx = numberAggregates / 4;
        const medianIdx = numberAggregates / 2;
        const q3Idx = numberAggregates * (3 / 4);

        // sort aggregates and get boxplot values
        aggregates.sort(arc.misc.sortNums);
        min = aggregates[0];
        q1 = getBoxplotValueLocal(aggregates, q1Idx);
        median = getBoxplotValueLocal(aggregates, medianIdx);
        q3 = getBoxplotValueLocal(aggregates, q3Idx);
        max = aggregates[numberAggregates - 2];
      }

      // create new rows with additional min, q1, median, and q3
      row.push(min, q1, median, q3, max);
      newRows.push(row);
    }

    result.rows(newRows);
    data_result = result;

    return boxplot;
  };

  boxplot.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();
    boxplot.clearDerivedData();

    // Setup config object
    const sb_rows = data_result.rows();
    if (!sb_rows || sb_rows.length === 0) {
      return boxplot;
    }

    const lastIdx = sb_rows[0].length - 1;
    const config = {
      rows: sb_rows, // null rows will be filtered out
      orig_rows: sb_rows,
      columns: data_result.columns(),

      // Support horizontal boxplots
      x_is_dim: !horizontalBoxplots.isChecked,
      dimensionPrefix: horizontalBoxplots.isChecked ? 'y' : 'x',
      measurePrefix: horizontalBoxplots.isChecked ? 'x' : 'y',
    };

    // Define indexes
    // Defining all the aggregate idxs and getters
    // q0_idx, q1_idx, q2_idx, q3_idx, q4_idx
    // q0_getter, q1_getter, q2_getter, q3_getter, q4_getter
    let currentSubtract = addedAggregates.length - 1;
    config.measure_idx = lastIdx - currentSubtract;
    const addedAggregatesGetters = [];
    addedAggregates.forEach((aggName) => {
      const aggIdx = lastIdx - currentSubtract;
      function aggGetter(row) {
        return row[aggIdx];
      }

      config[`${aggName}_idx`] = aggIdx;
      config[`${aggName}_getter`] = aggGetter;
      addedAggregatesGetters.push(aggGetter);
      currentSubtract -= 1;
    });

    const displayFormat = measureElement.display_format();
    config.measureColdisplayformat = displayFormat ?
                                     lib.translatedDisplayFormat(displayFormat) :
                                     undefined;

    // Define dimensions
    // dimension_idx
    // dimensions_getter
    config.columns.forEach((col, colidx) => {
      const colsource = col.colsource();
      const shelfType = colsource.replace('_shelf', '');
      config[`${shelfType}_idx`] = colidx;
      config[`${shelfType}_column`] = col;
      config[`${shelfType}_getter`] = function dimGetter(row) {
        const currentIdx = config[`${shelfType}_idx`];
        return row[currentIdx];
      };
    });

    // Add in y column so axis/label settings can be properly applied
    const aggregatesShelfIndex = 1;
    const measureColumn = lib.shelfElementToCol(measureElement,
                                                'aggregates_shelf',
                                                arc.report.columnTypeEnum.DIMENSION,
                                                aggregatesShelfIndex);
    config.columns.push(measureColumn);

    // Filter out nulls and return if no remaining rows
    lib.filter_nulls(config, addedAggregatesGetters);

    if (config.rows.length === 0) {
      return boxplot;
    }

    // Define datetime and sort if it is datetime
    const dateTime = lib.idx_is_date(config.rows, config.dimensions_idx);
    config.is_date = dateTime.is_date;
    config.is_datetime = dateTime.is_datetime;
    config.dateFormat = dateTime.dateFormat;

    if (config.is_date) {
      lib.compute_dates(config.rows, config.dateFormat, 'dim_date', config.dimensions_getter);
      config.dimensions_idx = 'dim_date';

      // Always sort
      if (!config.dimensions_column.order()) {
        config.sort_dim = true;
        config.rows.sort(function(a, b) {
          return arc.misc.sortNumeric(config.dimensions_getter(a), config.dimensions_getter(b));
        });
      }
    }

    // Setup colors including aggregate/dimension
    boxplot.setup_colors(config);

    lib.unselectOnClick();

    const viz_settings = {
      legend_contextmenu_shown: false,
      show_cb: [],
      hide_cb: [],
    };

    const trellis = arc.visual.trellis()
                              .x_shelf_name('x_shelf')
                              .y_shelf_name('y_shelf')
                              .x_shelf_reserved(0)
                              .y_shelf_reserved(0)
                              .rows(config.rows)
                              .sort_orders_asc(sort_orders_asc)
                              .columns(config.columns)
                              .viz_lib(lib)
                              .viz_div_sel(d3.select(`#${viz_div}`))
                              .color(config.color)
                              .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                              .redrawVisualCb(function() {
                                allowLargeTrellisTemp = true;
                                boxplot.viz_drawVisual(viz_div);
                              })
                              .setup();

    if (trellis.error_state()) {
      return boxplot;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      boxplot.viz_drawVisual(viz_div);
    });

    boxplot.compute_cell_size(config, viz_div);

    // Compute scales for dimensions or aggregates now if they're dependent
    if (!scale_x_independent.isChecked) {
      boxplot.compute_scales(config, config.rows, 'dimensions');
    }

    if (!scale_y_independent.isChecked) {
      boxplot.compute_scales(config, config.rows, 'measure');
    }

    // Now draw visuals within trellis
    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      const cell_div = cell_sel.attr('id');

      // Correctly set up axis scales
      if (scale_x_independent.isChecked) {
        config.rows = xy_rows;
        boxplot._compute_dim_sz(config);
        boxplot.compute_scales(config, xy_rows, 'dimensions');
      }

      if (scale_y_independent.isChecked) {
        boxplot.compute_scales(config, xy_rows, 'measure');
      }

      // Draw visual/axis/legend
      viz_drawVisualWork(config, cell_sel, viz_div, cell_div, xy_rows, viz_settings);
    });
  };

  boxplot.flattenRows = function (config, rows) {
    function addRow(dimensions, aggregate, finalRows, currentRow) {
      const newRow = dimensions.concat(aggregate);

      if (newRow[config.dimensions_idx] === undefined) {
        newRow[config.dimensions_idx] = currentRow[config.dimensions_idx];
      }

      finalRows.push(newRow);
    }

    return rows.reduce((finalRows, currentRow) => {
      const dimensions = currentRow.slice(0, config.measure_idx);
      for (let aggIdx = 0; aggIdx < addedAggregates.length; aggIdx++) {
        // add date if present
        const aggGetter = config[`${addedAggregates[aggIdx]}_getter`];
        const aggregate = aggGetter(currentRow);
        addRow(dimensions, aggregate, finalRows, currentRow);
      }

      return finalRows;
    }, []);
  };

  boxplot.compute_scales = function(config, xy_rows, type) {
    const is_dim = type === 'dimensions';
    const is_date = is_dim ? config.is_date : false;
    const idx = config[`${type}_idx`];

    let range;
    if (is_dim) {
      range = config.x_is_dim ? [0, config.width] : [config.height, 0];
    } else {
      range = config.x_is_dim ? [config.height, 0] : [0, config.width];
    }

    // Flatten the rows if not dimension since there's multiple aggs per row
    let rows;
    if (is_dim) {
      rows = xy_rows;
    } else {
      rows = boxplot.flattenRows(config, xy_rows);
    }

    config[`${type}_scale`] = lib.idx_to_scale_config({
      rows,
      idx,
      range,
      is_date,
      is_dim,
      date_format: config.date_format,
      sort_dim: config.sort_dim,
      agg_scale_zero: scale_agg_zero.isChecked,
    });

    return boxplot;
  };

  boxplot.compute_cell_size = function (config, viz_div) {
    const { elem_w, elem_h } = lib.computeTrellisElementParams({
      viz_div,
      trellis: config.trellis,
      includeLegend: false,
    });

    boxplot._compute_dim_sz(config);

    const $div = $(`#${viz_div}`);
    const paramConfig = {
      width: elem_w,
      height: elem_h,
      viz_div_elem: $div,
      columns: config.columns,
      x_is_slanted: config.x_is_dim && !config.is_date,
      rows: boxplot.flattenRows(config, config.rows),
    };

    const hasMin = config.x_is_dim ? 'width' : 'height';
    paramConfig[`min_${hasMin}`] = config.min_width;
    paramConfig[`${config.dimensionPrefix}_idx`] = config.dimensions_idx;
    paramConfig[`${config.dimensionPrefix}_formatter`] = config.dimensions_column
                                                               .coldisplayformat();
    paramConfig[`${config.dimensionPrefix}_axis_label_off`] = !x_axis_label.isChecked;
    paramConfig[`show_${config.dimensionPrefix}_axis`] = show_x_axis.isChecked;
    paramConfig[`${config.measurePrefix}_idx`] = config.measure_idx;
    paramConfig[`${config.measurePrefix}_formatter`] = config.measureColdisplayformat;
    paramConfig[`${config.measurePrefix}_axis_label_off`] = !y_axis_label.isChecked;
    paramConfig[`show_${config.measurePrefix}_axis`] = show_y_axis.isChecked;

    const r = lib.compute_svg_params(paramConfig);

    config.margin = r.margin;
    config.width = r.width;
    config.height = r.height;

    // Calculate dim_sz again now that we have the proper width
    const barRange = bar_mark_size.get_range();
    const distanceBetweenBars = 2;
    const totalDim = config.x_is_dim ? config.width : config.height;
    config.dim_sz = totalDim / config.max_slots;
    config.barWidth = Math.min(config.dim_sz - distanceBetweenBars, barRange[1]);

    return boxplot;
  };

  // Have to run this twice, once when calculating cell size, and again for each cell
  boxplot._compute_dim_sz = function(config) {
    let dim_numvalues;
    if (config.is_date) {
      dim_numvalues = (config.dimensions_getter(config.rows[0]) -
          config.dimensions_getter(config.rows[config.rows.length - 1])) /
          (1000 * 60 * 60 * 24);
      dim_numvalues = Math.max(1, Math.abs(dim_numvalues));
      dim_numvalues += 1;
    } else {
      // Define unique dims
      config.unique_dims = arc.misc.arrayUnique(config.rows, config.dimensions_getter);
      dim_numvalues = config.unique_dims.length;
    }

    config.max_slots = dim_numvalues;

    // the size of the bars are calculated
    const barRange = bar_mark_size.get_range();
    config.min_width = barRange[0] * config.max_slots;
    return boxplot;
  };

  // TODO: This function will become more useful once we add color shelf support (sz)
  boxplot.setup_colors = function(config) {
    config.color_is_dim = true;
    config.palette = lib.graded_colors();

    config.color = function() {
      return lib.colors()[0];
    };

    return boxplot;
  };

  boxplot.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return boxplot;
  };

  boxplot.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return boxplot;
  };

  boxplot.create_svgs = function (config, cell_sel) {
    var sel =
      cell_sel.append('svg')
              .classed('viz-boxplot-svg', true)
              .style('width', `${config.width + config.margin.left}px`)
              .style('height',
                     `${config.height + config.margin.top + config.margin.bottom}px`)
                .append('g')
                .classed('viz-boxplot-g', true)
                .attr('transform',
                      `translate(${config.margin.left},${config.margin.top})`);

    return sel;
  };

  function viz_drawVisualWork(config, cell_sel, viz_div, cell_div, xy_rows) {
    if (xy_rows.length === 0) {
      return;
    }

    // Draw axis
    const dimension_name = config.dimensions_column.colname();
    const measure_name = measureElement.col_alias() || measureElement.dataset_colname();
    const x_axis = arc.visual.axis()
                             .tickFormat(config.dimensions_column.coldisplayformat())
                             .scale(config.dimensions_scale)
                             .is_dim(true)
                             .is_date(config.is_date)
                             .colname(dimension_name)
                             .coldesc(config.dimensions_column.coldesc() || dimension_name);

    const y_axis = arc.visual.axis()
                             .tickFormat(config.measureColdisplayformat)
                             .scale(config.measure_scale)
                             .colname(measure_name)
                             .coldesc(measureElement.col_desc() || measure_name);

    const svg = boxplot.create_svgs(config, cell_sel);
    let gridlineSVG;
    if (horizontal_gridlines_enabled.isChecked || vertical_gridlines_enabled.isChecked) {
      gridlineSVG = svg;
    }

    const drawAxesParams = {
      gridlineSVG,
      x_is_dim: config.x_is_dim && !config.is_date,
      y_is_dim: !config.x_is_dim && !config.is_date,
      drawVerticalGridLine: vertical_gridlines_enabled.isChecked,
      drawHorizontalGridLine: horizontal_gridlines_enabled.isChecked,
    };

    drawAxesParams[`${config.dimensionPrefix}_axis_data`] = x_axis;
    drawAxesParams[`${config.measurePrefix}_axis_data`] = y_axis;
    drawAxesParams[`${config.dimensionPrefix}_axis_label_off`] = !x_axis_label.isChecked;
    drawAxesParams[`${config.measurePrefix}_axis_label_off`] = !y_axis_label.isChecked;
    drawAxesParams[`show_${config.dimensionPrefix}_axis`] = show_x_axis.isChecked;
    drawAxesParams[`show_${config.measurePrefix}_axis`] = show_y_axis.isChecked;

    lib.draw_axes(svg, config.width, config.height, config.margin, drawAxesParams);

    const unique_id = arc.misc.uniqueId();
    const clippath_id = `boxplot-cp-${unique_id}`;
    svg.append('defs')
         .append('clipPath')
         .attr('id', clippath_id)
           .append('rect')
           .attr('x', 0)
           .attr('y', -1)
           .attr('width', config.width + 1)
           .attr('height', config.height + 1);

    const clippath = svg.append('svg')
                        .attr('y', -1)
                        .style('width', `${config.width + 1}px`)
                        .style('height', `${config.height}px`)
                          .append('g')
                          .attr('clip-path', `url(#${clippath_id})`)
                          .attr('data-test-id', `boxplot-box-item`);

    // Dimensions needs to be offset by half the tick's width
    const numberTicks = config.max_slots;
    config.dim_offset = 0;
    if (!config.is_date) {
      if (config.x_is_dim) {
        config.dim_offset = (config.width / numberTicks) / 2;
      } else {
        config.dim_offset = (config.height / numberTicks) / 2;
      }
    }

    const halfWidth = config.barWidth / 2;

    function whiskersAt(f1, f2) {
      return clippath.selectAll('.nosel')
                     .data(xy_rows)
                     .enter()
                       .append('line')
                       .attr(`${config.dimensionPrefix}1`, function(d) {
                         const dim = config.dimensions_getter(d);
                         return config.dimensions_scale(dim) + config.dim_offset;
                       })
                       .attr(`${config.dimensionPrefix}2`, function(d) {
                         const dim = config.dimensions_getter(d);
                         return config.dimensions_scale(dim) + config.dim_offset;
                       })
                       .attr(`${config.measurePrefix}1`, f1)
                       .attr('data-test-id', 'whiskers')
                       .attr(`${config.measurePrefix}2`, f2)
                       .style({
                         fill: config.color(),
                         stroke: 'gray',
                         'fill-opacity': lib.colorOpacity,
                         'stroke-width': lineWidth.value(),
                       });
    }

    function lineAt(f) {
      return clippath.selectAll('.nosel')
                     .data(xy_rows)
                     .enter()
                       .append('line')
                       .attr(`${config.dimensionPrefix}1`, function(d) {
                         const dim = config.dimensions_getter(d);
                         return (config.dimensions_scale(dim) - halfWidth) + config.dim_offset;
                       })
                       .attr(`${config.dimensionPrefix}2`, function(d) {
                         const dim = config.dimensions_getter(d);
                         return (config.dimensions_scale(dim) + halfWidth) + config.dim_offset;
                       })
                       .attr(`${config.measurePrefix}1`, f)
                       .attr(`${config.measurePrefix}2`, f)
                       .style({
                         fill: config.color(),
                         stroke: 'gray',
                         'fill-opacity': lib.colorOpacity,
                         'stroke-width': lineWidth.value(),
                       });
    }

    const barHeight = config.x_is_dim ? 'height' : 'width';
    const barWidth = config.x_is_dim ? 'width' : 'height';
    const boxes =
      clippath.selectAll('.nosel')
        .data(xy_rows)
        .enter()
          .append('rect')
          .attr(config.dimensionPrefix, function(d) {
            const dim = config.dimensions_getter(d);
            return (config.dimensions_scale(dim) - halfWidth) + config.dim_offset;
          })
          .attr(barWidth, function() {
            return halfWidth * 2;
          })
          .attr(config.measurePrefix, function(d) {
            return Math.min(config.measure_scale(config.q1_getter(d)),
                            config.measure_scale(config.q3_getter(d)));
          })
          .attr(barHeight, function(d) {
            return Math.abs(config.measure_scale(config.q1_getter(d)) -
                            config.measure_scale(config.q3_getter(d)));
          })
          .style({
            fill: config.color(),
            stroke: 'gray',
            'fill-opacity': lib.colorOpacity,
            'stroke-width': lineWidth.value(),
          });

    // Offset all aggs by 1 so doesn't get clipped
    let aggOffset = 1;
    if (!config.x_is_dim) {
      aggOffset *= -1;
    }

    const q0 = lineAt(function(d) {
      return config.measure_scale(config.q0_getter(d)) + aggOffset;
    });

    const q2 = lineAt(function(d) {
      return config.measure_scale(config.q2_getter(d)) + aggOffset;
    });

    const q4 = lineAt(function(d) {
        return config.measure_scale(config.q4_getter(d)) + aggOffset;
    });

    const line1 = whiskersAt(d => config.measure_scale(config.q0_getter(d)) + aggOffset,
                             d => config.measure_scale(config.q1_getter(d)));

    const line2 = whiskersAt(d => config.measure_scale(config.q3_getter(d)),
                             d => config.measure_scale(config.q4_getter(d)) + aggOffset);

    arc.marks()
       .viz_lib(lib)
       .addSelection(boxes)
       .addSelection(q0)
       .addSelection(q2)
       .addSelection(q4)
       .addSelection(line1)
       .addSelection(line2)
       .text_fn(function(d) {
          const tooltipInfo = [];

          config.columns.forEach((col) => {
            const colsource = col.colsource();
            if (colsource !== 'aggregates_shelf') {
              tooltipInfo.push({
                key: col.colname(),
                value: config[colsource.replace('shelf', 'getter')](d),
                formatter: col.coldisplayformat(),
              });
            }
          });

          tooltipInfo.push(
            {
              key: 'Measure',
              value: measureElement.dataset_colname(),
            },
            {
              key: 'Minimum',
              value: config.q0_getter(d),
              formatter: config.measureColdisplayformat,
            },
            {
              key: 'First Quartile',
              value: config.q1_getter(d),
              formatter: config.measureColdisplayformat,
            },
            {
              key: 'Median',
              value: config.q2_getter(d),
              formatter: config.measureColdisplayformat,
            },
            {
              key: 'Third Quartile',
              value: config.q3_getter(d),
              formatter: config.measureColdisplayformat,
            },
            {
              key: 'Maximum',
              value: config.q4_getter(d),
              formatter: config.measureColdisplayformat,
            }
          );

          return lib.tooltip_from_kvs(tooltipInfo);
       })
       .activate();

    return boxplot;
  }

  return boxplot;
};

arc.visual.plugins.boxplot_extension = function() {
  var ext = function() { return 'arc.visual.plugins.boxplot_extension'; };

  ext.viz_name = function() {
    return 'boxplot';
  };

  ext.viz_tooltip = function() {
    return 'box plot';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_boxplot.png?v=1';
  };

  ext.viz = arc.visual.plugins.boxplot;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.boxplot_extension());
}());
