window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.bullet = function() {
  var saved_settings = {},
      data_result,
      lib,
      trellis,
      numIgnoredValues,
      sort_orders_asc = {},
      unifiedScale,
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  var bgColor = arc.settings.bullet.bulletBgColor(),
      fgColor = arc.settings.bullet.bulletFgColor(),
      projectionOpacity = arc.settings.bullet.projectionOpacity(),
      scaleTrellisIndependently = arc.settings.bullet.scaleTrellisIndependently(),
      reverseColors = arc.settings.bullet.reverseColors(),
      removeDefaultLabel = arc.settings.bullet.removeDefaultLabel(),
      vertical = arc.settings.bullet.orientVertically(),
      maxTickCount = arc.settings.bullet.approximateNumberOfTicks(),
      useQualitativePercentages = arc.settings.gauge.showQualitativeAsPercentages(),
      lowPercent = arc.settings.gauge.lowestQualitativePercentage(),
      midPercent = arc.settings.gauge.mediumQualitativePercentage(),
      highPercent = arc.settings.gauge.highestQualitativePercentage();
      useQualitativePercentages.addDependents([lowPercent, midPercent, highPercent]);

  var user_settings =
      arc.visual.defaultSettings({
           showColors: false,
         })
         .addSetting([
           bgColor,
           fgColor,
           projectionOpacity,
           reverseColors,
           scaleTrellisIndependently,
           vertical,
           removeDefaultLabel,
           maxTickCount,
           useQualitativePercentages,
           lowPercent,
           midPercent,
           highPercent,
         ])
         .removeSetting([
          'axis_label_font_size',
          'label_font_size',
          'label_color',
          'legend_font_size',
         ]);

  var bullet = arc.visual.api();

  var shelf_data = [
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'compare_shelf',
      display_name: 'Compare To',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'qualitative_shelf',
      display_name: 'Qualitative Ranges',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'projection_shelf',
      display_name: 'Projection',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'label_shelf',
      display_name: 'Label',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  var constraints = {
    minimums: {
      aggregates_shelf: 1,
    },
    maximums: {
      aggregates_shelf: 1,
      compare_shelf: 1,
      qualitative_shelf: 4,
      projection_shelf: 1,
      label_shelf: 2,
    },
  };

  bullet.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return bullet;
  };

  bullet.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  bullet.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return bullet;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return bullet;
  };

  bullet.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    let xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        measureMessage = [
          lib.createPopoverMessages('aggregateSingle'),
          lib.createPopoverMessages('featureMeasure'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        compareToMessage = [
          lib.createPopoverMessages('compareTo'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        qualitativeRangeMessage = [
          lib.createPopoverMessages('qualitativeRanges'),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        projectionMessage = [
          lib.createPopoverMessages('projection'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        labelMessage = [
          lib.createPopoverMessages('label'),
          lib.createPopoverMessages('acceptsAtMostTwo'),
        ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      aggregates_shelf: measureMessage,
      compare_shelf: compareToMessage,
      qualitative_shelf: qualitativeRangeMessage,
      projection_shelf: projectionMessage,
      label_shelf: labelMessage,
    });

    return bullet;
  };

  bullet.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  bullet.viz_setDataResult = function(result) {
    data_result = result;
    return bullet;
  };

  bullet.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return bullet;
  };

  bullet.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return bullet;
  };

  bullet.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    {
      let sb_rows = data_result.rows();
      if (!sb_rows || sb_rows.length === 0) {
        return bullet;
      }
    }

    let config = {
      rows: data_result.rows(),
      columns: data_result.columns(),
    };

    config.agg_col = lib.first_column(config.columns, 'aggregates_shelf');
    config.projection_col = lib.first_column(config.columns, 'projection_shelf');
    config.compare_col = lib.first_column(config.columns, 'compare_shelf');
    config.agg_idx = config.agg_col.colindex();
    config.qualitative_cols = lib.shelf_columns(config.columns, 'qualitative_shelf');
    config.label_cols = lib.shelf_columns(config.columns, 'label_shelf');

    config.color = function() {
      return base.colors()[0];
    };

    if (config.rows.length > 1 && !scaleTrellisIndependently.isChecked) {
      unifiedScale = {};
      let aggIndices = [];
      aggIndices.push(config.agg_idx);
      if (config.projection_col) {
        aggIndices.push(config.projection_col.colindex());
      }

      if (config.compare_col) {
        aggIndices.push(config.compare_col.colindex());
      }

      config.qualitative_cols.forEach(f => aggIndices.push(f.colindex()));
      let max = 0,
          min = 0;
      config.rows.forEach(function (row) {
        aggIndices.forEach(function getMaxMin(idx) {
            max = Math.max(max, row[idx]);
            min = Math.min(min, row[idx]);
        });
      });

      unifiedScale.min = min;
      unifiedScale.max = max;
    } else {
      unifiedScale = undefined;
    }

    // Filter rows with NULLs in the aggregate columns
    let agg_getter = function(row) {
      return row[config.agg_idx];
    };

    lib.filter_nulls(config, [agg_getter]);
    lib.unselectOnClick();

    const trellis = arc.visual.trellis()
                     .x_shelf_name('x_shelf')
                     .y_shelf_name('y_shelf')
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select(`#${viz_div}`))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       bullet.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return bullet;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      bullet.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');

    numIgnoredValues = 0;
    arc.report.alertMessageSmallHide();
    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      config.rows = xy_rows;
      let cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_sel, viz_div, cell_div);
    });

    if (arc.globals.is_edit && numIgnoredValues > 0) {
      const message = `Ignoring ${numIgnoredValues} invalid values.`;
      arc.report.alertMessageSmall(message, 'alert-warning');
    }

  };

  function _isNull(val) {
    const nullValues = [null, 'null', 'NULL', ''];
    return nullValues.includes(val);
  }

  function validateMeasure(measure) {
    if (measure !== undefined && (_isNull(measure) || isNaN(measure))) {
      numIgnoredValues++;
      return undefined;
    }

    return measure;
  }

  function textSize(containerDiv, divClass, labelText, textSize) {
    let container = d3.selectAll(`#${containerDiv}`)
                      .selectAll('.nosel')
                      .data([0])
                      .enter()
                        .append('div')
                        .classed('viz-bullet-container', true)
                        .style('opacity', 0);
    let label = container.append('div')
                         .classed('viz-bullet-labelcontainer', true);
    let width, height;
    label.append('div')
         .attr('class', divClass)
         .style('float', 'left')
         .style('font-size', `${textSize}px`)
         .text(labelText)
         .each(function () {
           width = $(this).width();
           height = $(this).height();
         });

    container.remove();

    return [width + 5, height + 5];
  }

  function viz_drawVisualWork(config, cell_sel, viz_div, cell_div) {
    if (config.rows.length === 0) {
      return;
    }

    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
    });

    let data = config.rows[0];
    let agg = data[config.agg_col.colindex()];
    let compare;

    if (config.compare_col) {
      compare = data[config.compare_col.colindex()];
    }

    let agg_projection;
    if (config.projection_col) {
      agg_projection = data[config.projection_col.colindex()];
    }

    compare = validateMeasure(compare);

    agg_projection = validateMeasure(agg_projection);

    let labelText = removeDefaultLabel.value() === 'true' ?  '' : config.agg_col.colname();
    let sublabelText = config.agg_col.coldesc() || '';

    if (config.label_cols.length > 0) {
      labelText = data[config.label_cols[0].colindex()];
      labelText = config.label_cols[0].formatForDisplay(labelText);

      if (config.label_cols.length > 1) {
        sublabelText = data[config.label_cols[1].colindex()];
        sublabelText = config.label_cols[1].formatForDisplay(sublabelText);
      } else {
        sublabelText = '';
      }
    }

    let qualitative_data;

    if (config.qualitative_cols.length === 0 && useQualitativePercentages.isChecked && compare) {
      qualitative_data = [
        (Number(lowPercent.value()) / 100) * compare,
        (Number(midPercent.value()) / 100) * compare,
        (Number(highPercent.value()) / 100) * compare,
      ];
    } else {
      qualitative_data = config.qualitative_cols.map(c => data[c.colindex()]);
    }

    let updatedQualitative = [];
    for (let i = 0; i < qualitative_data.length; i++) {
      if (validateMeasure(qualitative_data[i])) {
        updatedQualitative.push(qualitative_data[i]);
      }
    }

    qualitative_data = updatedQualitative;

    let allIntensities = [
      [0.10],
      [0.35, 0.10],
      [0.40, 0.25, 0.10],
      [0.50, 0.35, 0.20, 0.10],
      [0.50, 0.35, 0.20, 0.10, 0.03],
    ];

    let intensities = allIntensities[qualitative_data.length];

    if (reverseColors.isChecked) {
      intensities.reverse();
    }

    let labelContainerStyle = 'viz-bullet-label',
        sublabelContainerStyle = 'viz-bullet-sublabel';

    let domainMax, domainMin;

    if (unifiedScale) {
      domainMax = unifiedScale.max;
      domainMin = unifiedScale.min;
    } else {
      domainMin = Math.min(agg, 0);
      domainMax = Math.max(agg, 0);

      if (compare !== undefined) {
        domainMin = Math.min(domainMin, compare);
        domainMax = Math.max(domainMax, compare);
      }

      if (agg_projection !== undefined) {
        domainMin = Math.min(domainMin, agg_projection);
        domainMax = Math.max(domainMax, agg_projection);
      }

      if (qualitative_data.length > 0) {
        domainMin = Math.min(domainMin, Math.min(...qualitative_data));
        domainMax = Math.max(domainMax, Math.max(...qualitative_data));
      }
    }

    const fontRatio = 1.618;
    let isVertical = vertical.value() === 'true',
      hasLabel = removeDefaultLabel.value() !== 'true',
      labelFontSize = lib.getLabelFontSize() || 12,
      labelData = textSize(viz_div, 'viz-bullet-label', labelText, labelFontSize),
      sublabelData = textSize(viz_div, 'viz-bullet-sublabel', sublabelText, labelFontSize),
      labelLength = labelData[0],
      sublabelLength = sublabelData[0],
      tickFormat = config.agg_col.formatForDisplay,
      labelWidth = hasLabel ? Math.max(labelLength, sublabelLength) : 0,
      tickFontSize = lib.getAxisTicksFontSize() || 10,
      labelHeight = hasLabel ? labelData[1] + sublabelData[1] : 0,
      lowTickLength = tickFormat(domainMin).length * (tickFontSize / fontRatio),
      highTickLength = tickFormat(domainMax).length * (tickFontSize / fontRatio),
      maxTickLength = Math.max(lowTickLength, highTickLength),
      minMajorLength = 100,
      maxMajorLength = 400,
      bulletDiameter = 12,
      targetLine = 24,
      minorLength = 36,
      margin = {
        left: isVertical ? maxTickLength + 10 : Math.max(lowTickLength / 2, 10),
        right: isVertical ? 10 : Math.max(highTickLength / 2, 10),
        top: isVertical ? Math.max(tickFontSize / 2, 10) : 10,
        bottom: isVertical ? Math.max(tickFontSize / 2, 10) : tickFontSize + 10,
      },
      axisAdjustment = isVertical ? 0 : minorLength,
      majorLength,
      longDirection,
      shortDirection,
      longVariable,
      shortVariable,
      axisOrientation,
      totalMajorLength,
      majorMargin1 = isVertical ? margin.top : margin.left,
      majorMargin2 = isVertical ? margin.bottom : margin.right,
      minorMargin1 = isVertical ? margin.left : margin.top,
      minorMargin2 = isVertical ? margin.right : margin.bottom;

    if (isVertical) {
      if (labelWidth > (minorLength + margin.left + margin.right)) {
        minorMargin1 = Math.max((labelWidth - minorLength) / 2, minorMargin1);
        minorMargin2 = labelWidth - (minorLength + minorMargin1);
        margin.left = minorMargin1;
        margin.right = minorMargin2;
      }
    } else {
      if (labelHeight > (minorLength + margin.top + margin.bottom)) {
        minorMargin2 = Math.max((labelHeight - minorLength) / 2, minorMargin2);
        minorMargin1 = labelHeight - (minorLength + minorMargin2);
        margin.top = minorMargin1;
        margin.bottom = minorMargin2;
      }
    }

    let totalMinorLength = minorLength + minorMargin1 + minorMargin2;
    let sumMajorMargins = majorMargin1 + majorMargin2;

    if (isVertical) {
      majorLength = Math.max(maxMajorLength, minMajorLength);
      totalMajorLength = majorLength + sumMajorMargins + labelHeight;
      totalMinorLength = Math.max(totalMinorLength, labelWidth);
      longDirection = 'height';
      shortDirection = 'width';
      longVariable = 'y';
      shortVariable = 'x';
      axisOrientation = 'left';
    } else {
      longDirection = 'width';
      shortDirection = 'height';
      longVariable = 'x';
      shortVariable = 'y';
      axisOrientation = 'bottom';
      majorLength = Math.max(maxMajorLength, minMajorLength);
      totalMajorLength = labelWidth + majorLength + sumMajorMargins + 5;
      totalMinorLength = Math.max(totalMinorLength, labelHeight);
    }

    const theRange = [0, majorLength];
    const theDomain = [domainMin, domainMax];
    let tickCount;
    if (Number(maxTickCount.value()) === -1) {
      tickCount = isVertical ? Math.floor(majorLength / tickFontSize) :
        Math.floor(majorLength / maxTickLength);
    } else {
      tickCount = Number(maxTickCount.value());
    }

    if (isVertical) {
      theDomain.reverse();
    }

    let lengthScale = d3.scale.linear()
    .range(theRange)
    .domain(theDomain)
    .nice();

    let theAxis = d3.svg.axis()
      .scale(lengthScale)
      .orient(axisOrientation)
      .tickFormat(d => tickFormat(d));

    if (tickCount < 10 || Number(maxTickCount.value()) !== -1) {
      theAxis.ticks(tickCount);
    }

    let container =
      d3.selectAll('#' + cell_div)
        .selectAll('.nosel')
        .data([data])
        .enter()
          .append('div')
          .classed('viz-bullet-container', true)
          .style(longDirection, `${totalMajorLength}px`)
          .style(shortDirection, `${totalMinorLength}px`);

    let label =
      container.append('div')
               .classed('viz-bullet-labelcontainer', true);

    if (!isVertical) {
      label.style('float', 'left');
    }

    label.append('div')
         .classed(labelContainerStyle, true)
         .text(labelText)
         .style('font-size', `${labelFontSize}px`)
         .style('text-align', 'center');

    label.append('div')
         .classed(sublabelContainerStyle, true)
         .text(sublabelText);

    let svg =
      container.append('div')
               .classed('viz-bullet-value', true)
                 .append('svg')
                 .style(longDirection, (sumMajorMargins + majorLength) + 'px')
                 .style(shortDirection, (minorMargin1 + minorMargin2 + minorLength) + 'px');

    svg.append('g')
       .attr('class', longVariable + ' arc-viz-axis')
       .attr('transform', `translate(${margin.left}, ${margin.top + axisAdjustment})`)
       .call(theAxis);

    svg.selectAll('text').style('font-size', `${tickFontSize}px`);

    let g = svg.append('g')
               .attr('transform', `translate(${margin.left}, ${margin.top})`);

    if (isVertical) {
      domainMin = lengthScale.domain()[1];
      domainMax = lengthScale.domain()[0];
    } else {
      domainMin = lengthScale.domain()[0];
      domainMax = lengthScale.domain()[1];
    }

    if (domainMin < Math.min(...qualitative_data)) {
      qualitative_data.push(domainMin);
    }

    if (domainMax > Math.max(...qualitative_data)) {
      qualitative_data.push(domainMax);
    }

    qualitative_data.sort((a, b) => a - b);
    let bgData = [];

    for (let i = 0; i < qualitative_data.length - 1; i++) {
      bgData.push({
        start: isVertical ? qualitative_data[i + 1] : qualitative_data[i],
        end: isVertical ? qualitative_data[i] : qualitative_data[i + 1],
      });
    }

    let outerRect =
      g.selectAll('.nosel')
       .data(bgData)
       .enter()
         .append('rect')
         .attr(shortVariable, 0)
         .attr(shortDirection, minorLength)
         .attr(longVariable, function(d) {
           return lengthScale(d.start);
         })
         .attr(longDirection, function(d) {
           return lengthScale(d.end) - lengthScale(d.start);
         })
         .classed('viz-bullet-qualitative', true)
         .attr('fill', bgColor.value())
         .attr('fill-opacity', function(d, i) {
           return intensities[i];
         });

    let compareLine;

    if (compare !== undefined) {
      compareLine =
        g.selectAll('.nosel')
         .data([data])
         .enter()
           .append('line')
           .classed('viz-bullet-compareline', true)
           .attr('stroke', fgColor.value())
           .attr(`${longVariable}1`, lengthScale(compare))
           .attr(`${shortVariable}1`, (minorLength / 2) - (targetLine / 2))
           .attr(`${longVariable}2`, lengthScale(compare))
           .attr(`${shortVariable}2`, (minorLength / 2) + (targetLine / 2));
    }

    let projectionRect, projectionLength;
    let startAtZero = agg_projection > 0 && !isVertical || agg_projection < 0 && isVertical;
    if (agg_projection !== undefined) {
      projectionLength = Math.abs(lengthScale(agg_projection) - lengthScale(0));
      projectionRect =
        g.selectAll('.nosel')
         .data([data])
         .enter()
           .append('rect')
           .attr(longVariable, startAtZero ? lengthScale(0) : lengthScale(agg_projection))
           .attr(shortVariable, (minorLength / 2) - (bulletDiameter / 2))
           .attr(longDirection, projectionLength)
           .attr(shortDirection, bulletDiameter)
           .attr('fill', fgColor.value())
           .attr('fill-opacity', Number(projectionOpacity.value()) / 100)
           .classed('viz-bullet-projection', true);
    }

    startAtZero = agg > 0 && !isVertical || agg < 0 && isVertical;
    let innerRect =
      g.selectAll('.nosel')
       .data([data])
       .enter()
          .append('rect')
          .attr(longVariable, startAtZero ? lengthScale(0) : lengthScale(agg))
          .attr(shortVariable, (minorLength / 2) - (bulletDiameter / 2))
          .attr(longDirection, Math.abs(lengthScale(agg) - lengthScale(0)))
          .attr(shortDirection, bulletDiameter)
          .attr('fill', fgColor.value())
          .classed('viz-bullet-aggregate', true);

    let nonlabelShelves =
        config.columns.filter(function(c) {
          return c.colsource() !== 'label_shelf';
        });

    let marks =
      arc.marks()
         .viz_lib(lib)
         .marker_name('primary')
         .highlight_on_select(true)
         .addSelection(outerRect)
         .addSelection(innerRect)
         .addSelection(compareLine)
         .addSelection(projectionRect)
         .text_fn(function(d, i) {
            return lib.text_from_columns(data, nonlabelShelves);
          })
         .params_fn(function(d, i, e, dimsOnly) {
            // NOTE: data is intentionally passed. Do not pass d.
            return lib.params_from_columns(data, nonlabelShelves, dimsOnly);
          });

    if (projectionRect) {
      marks.addSelection(projectionRect);
    }

    marks.activate();

    return bullet;
  }

  return bullet;
};

arc.visual.plugins.bullet_extension = function() {
  let ext = function() { return 'arc.visual.plugins.bullet_extension'; };

  ext.viz_name = function() {
    return 'bullet';
  };

  ext.viz_tooltip = function() {
    return 'bullet';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_bullet.png?v=1';
  };

  ext.viz = arc.visual.plugins.bullet;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.bullet_extension());

}());
