window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

// Day of week with Monday as 0
var weekday_fn = d3.time.format('%w');
var weekday = function(d) {
  return (Number(weekday_fn(d)) + 6) % 7;
};

// Week number with Monday as the first day of the week
var week_fn = d3.time.format('%W');
var week = function(d) {
  return Number(week_fn(d));
};

var yearFn = d3.time.format('%Y');
var datekey = function(d) {
  return yearFn(d) + ' ' + week(d) + ' ' + weekday(d);
};

arc.visual.plugins.chm = function() {
  var sort_orders_asc = {},
      saved_settings = {},
      data_result,
      lib,
      highlighted_day,
      timeformat = d3.time.format('%Y-%m-%d'),
      parseDate = d3.time.format('%Y-%m-%d').parse,
      previousDataTracker = arc.transition.dataTracker(),
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  var scale_independent = arc.report.user_settings.scale_independent();
  scale_independent.labels(['Independent date scale']);
  var userSettings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.settings.visuals.enableAnimations(),
                                  scale_independent,
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  arc.colormap.measureRangesSetting(),
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'label_font_size',
                                  'label_color',
                                  'legend_font_size',
                                ]);

  var chm = arc.visual.api();

  var shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Date',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measures',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  var constraints = {
    minimums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
    },
    maximums: {
      dimensions_shelf: 1,
    },
  };

  chm.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    previousDataTracker.lib(lib);
    lib.shelf_data(shelf_data)
       .user_settings(userSettings)
       .chart_type_graded(true);
    return chm;
  };

  chm.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  chm.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return chm;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return chm;
  };

  chm.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

   // Define messages to be included in shelves
   var xMessage = [
                    lib.createPopoverMessages(['trellisOnly', 'columns.']),
                    lib.createPopoverMessages('acceptsMultiple'),
                   ],
       yMessage = [
                    lib.createPopoverMessages(['trellisOnly', 'rows.']),
                    lib.createPopoverMessages('acceptsMultiple'),
                  ],
       dateMessage = [
                    lib.createPopoverMessages('dateCalendar'),
                    lib.createPopoverMessages('acceptsOne'),
                   ],
       aggregateMessage = [
                    lib.createPopoverMessages(['aggregateAcross', 'dates.']),
                    lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException'),
                   ];

   // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      dimensions_shelf: dateMessage,
      aggregates_shelf: aggregateMessage,
    });

    return chm;
  };

  chm.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  chm.viz_setDataResult = function(result) {
    data_result = result;
    highlighted_day = undefined;
    return chm;
  };

  chm.viz_drawVisual = function(viz_div, isWindowResize) {
    lib.chart_div(viz_div);
    lib.undraw();

    var config = {};
    config.rows = data_result.rows();
    if (config.rows.length === 0) {
      return chm;
    }

    previousDataTracker.enableOnRowCount(config.rows.length);

    lib.unhighlight_cb(function() {
      if (highlighted_day !== undefined) {
        highlighted_day = undefined;
        chm.viz_drawVisual(viz_div);
      }
    });

    config.columns = data_result.columns();

    config.dim_col = lib.first_column(config.columns, 'dimensions_shelf');
    config.agg_col = lib.first_column(config.columns, 'aggregates_shelf');
    config.agg_colindex = config.agg_col.colindex();
    config.agg_getter = function(row) {
      return Number(row[config.agg_colindex]);
    };

    // Filter rows with NULLs in the aggregate columns
    lib.filter_nulls(config, [config.agg_col]);

    chm.setup_colors(config, config.rows);

    const trellis = arc.visual.trellis()
                     .x_shelf_name('x_shelf')
                     .y_shelf_name('y_shelf')
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select('#' + viz_div))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       chm.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return chm;
    }

    config.trellis = trellis;

    config.trellis.draw_skeleton(function() {
      chm.viz_drawVisual(viz_div);
    });

    config.trellis.draw_visuals(function(cell_sel, xy_rows,
        _unused_xmeasure, _unused_ymeasure, cellIdx) {
      if (scale_independent.isChecked) {
        chm.setup_colors(config, xy_rows);
      }

      chm.draw_base_visual(config, cell_sel, xy_rows, viz_div, cellIdx);
    });

    config.trellis.tooltips('trellis');
    return chm;
  };

  chm.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return chm;
  };

  chm.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return chm;
  };

  chm.setup_colors = function(config, rows) {
    const domain = arc.misc.getRange(rows, config.agg_getter);
    config.color = lib.aggScale(domain);
    return chm;
  };

  chm.draw_base_visual = function(config, cell_sel, xy_rows, viz_div, cellIdx) {
    var d_data = {},
        color_data = {},
        sb_tracked_rows = [],
        err_cnt = 0;

    xy_rows.forEach(function(d) {
      try {
        var dim = d[config.dim_col.colindex()];
        if (dim instanceof Date) {
          d.date = dim;
        } else {
          d.date = parseDate(dim.substring(0, 10));
        }

        let dk = datekey(d.date);
        let h = color_data[yearFn(d.date)];
        if (!h) {
          h = {};
          color_data[yearFn(d.date)] = h;
        }

        h[dk] = config.color(config.agg_getter(d));

        d_data[dk] = d;
        sb_tracked_rows.push(d);
      } catch (e) {
        ++err_cnt;
      }
    });

    if (err_cnt > 0) {
      var message = 'Ignoring ' + err_cnt + ' rows with errors';
      arc.report.alertMessageSmall(message, 'alert-warning');
    }

    xy_rows = sb_tracked_rows;

    var years = arc.misc.arrayUnique(xy_rows, function(d) {
      return Number(yearFn(d.date));
    }).sort();

    var margin = {
      top: 10,
      right: 35,
      bottom: 20,
      left: 20,
    };

    var elem_w = $('#' + viz_div).width() - config.trellis.x_screen_len();

    var cellsize = (elem_w - margin.left - margin.right) / 53;
    cellsize = Math.max(cellsize, 6);
    cellsize = Math.min(cellsize, 20);

    var width = cellsize * 53,
        height = cellsize * 7;

    let fullWidth = width + margin.left + margin.right,
        fullHeight = height + margin.top + margin.bottom;

    let canvas =
      cell_sel
        .selectAll('.nosel')
        .data(years)
        .enter()
          .append('canvas');

    canvas.each(function() {
      arc.canvas.adjustPixelRatio(d3.select(this), fullWidth, fullHeight);
    });

    var monthNames = [
        'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
        'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec',
    ];
    var dayNames = [
        'Mon', 'Tue', 'Wed', 'Thu',
        'Fri', 'Sat', 'Sun',
    ];

    var fontSize = Math.floor(cellsize) - 3;
    fontSize = Math.max(fontSize, 11);
    fontSize = Math.min(fontSize, 13);

    let visualStyleFontSize = lib.getAxisTicksFontSize();
    if (visualStyleFontSize) {
      if (Number(visualStyleFontSize) < cellsize) {
        fontSize = Number(visualStyleFontSize);
      } else {
        fontSize = cellsize;
      }
    }

    let font = lib.getVisualFont(),
        canvasFont = '300 ' + fontSize + 'px ' + font;

    canvas.each(function(year) {
      var c = this.getContext('2d');
      c.translate(margin.left, margin.top);

      // Label for years
      c.translate(-6, cellsize * 3.5);
      c.rotate(-Math.PI / 2);
      c.textAlign = 'center';
      c.font = canvasFont;
      c.fillText(year, 0, 0);

      // Reset transforms
      c.textAlign = 'start';
      c.rotate(Math.PI / 2);
      c.translate(6, -cellsize * 3.5);

      // Labels for months
      d3.range(0, 12).forEach(function(d) {
        var m = new Date(year, d, 1),
            w = week(m);
        c.fillText(monthNames[d], w * cellsize, cellsize * 7 + 15);
      });

      var offset = -4;

      if (cellsize < 10) {
        offset = 0;
      }

      // Labels for weekdays
      d3.range(0, 7).forEach(function(d, i) {
        if (cellsize < 10) {
          if ((d > 0) && (d < 6)) {
            return;
          }
        }

        if (cellsize < 13) {
          if ((d === 1) || (d === 3) || (d === 5)) {
            return;
          }
        }

        c.fillText(dayNames[d], cellsize * 53 + 10, cellsize * (d + 1) + offset);
      });
    });

    let previousComputedData = previousDataTracker.getData(cellIdx);
    previousDataTracker.setData(color_data, cellIdx);

    if (!previousComputedData && !previousDataTracker.enabled()) {
      drawDays();
    } else {
      if (!previousComputedData) {
        // generate
        previousComputedData = $.extend({}, color_data);
        for (let year in previousComputedData) {
          previousComputedData[year] = $.extend({}, previousComputedData[year]);
          let h = previousComputedData[year];
          for (let k in h) {
            h[k] = 'white';
          }
        }
      }

      let transition = arc.transition.transition()
                          .visualId(lib.report_id())
                          .select(`#${lib.chart_div()}`)
                          .transitionName(`cell-${cellIdx}`)
                          .profileData([
                            [
                              'rows',
                              xy_rows.length,
                            ],
                            [
                              'visual',
                              'chm',
                            ],
                          ]);

      let newData = color_data;
      color_data = previousComputedData;
      drawDays();
      color_data = newData;

      let interpolation = d4.interpolate(previousComputedData, newData);

      transition
        .drawFn(function(t) {
          if (t >= 1) {
            color_data = newData;

            // On the last draw, make sure we undraw all previous data cells
            for (let year in previousComputedData) {
              const previousYearData = previousComputedData[year];
              const currYearData = color_data[year];
              if (currYearData) {
                for (let dayKey in previousYearData) {
                  if (currYearData[dayKey] === undefined) {
                    currYearData[dayKey] = 'clear';
                  }
                }
              }
            }
          } else {
            color_data = interpolation(t);
          }

          drawDays();
        })
        .start();
    }

    function drawGridlines(year, c) {
      // Draw gridlines
      c.strokeStyle = 'lightgrey';
      c.lineWidth = 1;

      let startWeekday = weekday(new Date(year, 0, 1));
      let endWeekday = weekday(new Date(year, 11, 31));

      for (let weekday = 1; weekday < 7; ++weekday) {
        c.beginPath();
        c.moveTo(weekday >= startWeekday ? 0 : cellsize, weekday * cellsize);
        c.lineTo((weekday > endWeekday ? 52 : 53) * cellsize, weekday * cellsize);
        c.stroke();
      }

      for (let week = 0; week <= 52; ++week) {
        c.beginPath();
        c.moveTo(week * cellsize, week === 0 ? startWeekday * cellsize : 0);
        c.lineTo(week * cellsize, 7 * cellsize);
        c.stroke();
      }
    }

    function drawMonths(year, c) {
      // Draw month markers
      c.strokeStyle = 'black';
      d3.time.months(new Date(year, 0, 1), new Date(year + 1, 0, 1)).forEach(
        function(month) {
          var t1 = new Date(month.getFullYear(), month.getMonth() + 1, 0),
              d0 = weekday(month), w0 = week(month),
              d1 = weekday(t1), w1 = week(t1);
          c.beginPath();
          c.moveTo((w0 + 1) * cellsize, d0 * cellsize);
          c.lineTo(w0 * cellsize, d0 * cellsize);
          c.lineTo(w0 * cellsize, 7 * cellsize);
          c.lineTo(w1 * cellsize, 7 * cellsize);
          c.lineTo(w1 * cellsize, (d1 + 1) * cellsize);
          c.lineTo((w1 + 1) * cellsize, (d1 + 1) * cellsize);
          c.lineTo((w1 + 1) * cellsize, 0);
          c.lineTo((w0 + 1) * cellsize, 0);
          c.lineTo((w0 + 1) * cellsize, d0 * cellsize);
          c.stroke();
        });
    }

    function drawDays() {
      canvas.each(function(year) {
        let c = this.getContext('2d'),
            opacity = lib.colorOpacity(),
            color_data_year = color_data[year] || {};

        var hd;
        if (highlighted_day !== undefined) {
          hd = datekey(highlighted_day);
        }

        for (let tf in color_data_year) {
          let color = color_data_year[tf];

          // tf is a space-separted 'year week weekday' string
          let fields = tf.split(' '),
              x = (Number(fields[1])) * cellsize,
              y = (Number(fields[2])) * cellsize,
              width = cellsize,
              height = cellsize,
              yr = fields[0];

          c.globalAlpha = opacity;

          if (hd !== undefined) {
            if (hd !== tf) {
              if (opacity >= 0.5) {
                c.globalAlpha = opacity - 0.2;
              }
            }
          }

          if (color === 'clear') {
            c.clearRect(x, y, width, height);
          } else {
            c.beginPath();
            c.rect(x, y, width, height);
            c.fillStyle = color;
            c.fill();
          }
        }

        // Reset globalAlpha
        c.globalAlpha = 1;

        drawGridlines(year, c);
        drawMonths(year, c);
      });
    }

    function pos_to_row(d, i, elem) {
      var canvas_offset = $(elem).offset();
      if (!canvas_offset) {
        return undefined;
      }

      if (!d3.event) {
        return undefined;
      }

      var relx = d3.event.pageX - canvas_offset.left;
      var rely = d3.event.pageY - canvas_offset.top;

      var m = [relx, rely];
      m[0] -= margin.left;
      m[1] -= margin.top;

      var week = Math.floor(m[0] / cellsize),
          weekday = Math.floor(m[1] / cellsize),
          tf = d + ' ' + week + ' ' + weekday;

      if (tf in d_data) {
        return d_data[tf];
      }

      return undefined;
    }

    function highlight_cb(d, i, elem) {
      var row = pos_to_row(d, i, elem);
      if (row === undefined) {
        return [];
      }

      highlighted_day = row.date;
      chm.viz_drawVisual(viz_div);
    }

    function params_fn(d, i, elem) {
      var row = pos_to_row(d, i, elem);
      if (row === undefined) {
        return [];
      }

      return lib.params_from_columns(row, config.columns);
    }

    function text_fn(d, i, elem) {
      var row = pos_to_row(d, i, elem);
      if (row === undefined) {
        return '';
      }

      return lib.text_from_columns(row, config.columns);
    }

    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .addSelection(canvas)
       .text_changes_on_move(true)
       .text_fn(text_fn)
       .params_fn(params_fn)
       .highlight_cb(highlight_cb)
       .activate();

    return chm;
  };

  return chm;
};

arc.visual.plugins.chm_extension = function() {
  var ext = function() { return 'arc.visual.plugins.chm_extension'; };

  ext.viz_name = function() {
    return 'calendar-heatmap';
  };

  ext.viz_tooltip = function() {
    return 'calendar heatmap';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_calendar-heatmap.png?v=1';
  };

  ext.viz = arc.visual.plugins.chm;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.chm_extension());

}());
