import { paintProfileSetting, checkIfActiveProfileExists } from '../misc/arc_handle_ai_profile_change';

window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.chatbot = function() {
  let saved_settings = {};
  let data_result;
  let lib;
  let root;
  let downloadInfo = {};
  let embeddingProfileChangeSub;
  let completionProfileChangeSub;
  let embeddingProfiles;
  let completionProfiles;

  try {
    embeddingProfiles = JSON.parse(arc.frontend.globals.settings.embeddingProfiles);
    completionProfiles = JSON.parse(arc.frontend.globals.settings.completionProfiles);
  } catch (error) {
    const errorMessage =
      error instanceof Error ? error.message : String(error);

      throw new Error(errorMessage);
  }

  const setDefaultProfileValue = (profiles) => {
    if (!profiles || !Object.keys(profiles.profileSettings).length) {
      return '';
    }

    const { defaultProf } = profiles;

    return defaultProf ?
        defaultProf.value :
        Object.keys(profiles.profileSettings)[0];
  };

  const welcomeMessageSetting = arc.visual.setting()
    .id('welcome-message')
    .tab('Display')
    .type(arc.visual.settingTypeEnum.STRING)
    .add_name('WELCOME_MESSAGE')
    .add_label('Welcome message')
    .info('Customize the welcome message which is displayed as the first message from the AI visual.')
    .is_textarea(true)
    .value('Welcome to Cloudera Data Visualization AI. How can I help?');

  const chatAppearanceSetting = arc.visual.setting()
    .id('chat-appearance')
    .tab('Display')
    .type(arc.visual.settingTypeEnum.RADIO)
    .group('Chat Appearance')
    .add_name('plaintext')
    .add_label('Plaintext')
    .add_name('markdown')
    .add_label('Markdown')
    .value('markdown');

  const embeddingProfileSetting = arc.visual.setting()
    .id('embedding-profile')
    .tab('Embeddings')
    .type(arc.visual.settingTypeEnum.STRING)
    .add_name('EMBEDDING-PROFILE')
    .add_label('Selected Profile:')
    .value(setDefaultProfileValue(embeddingProfiles));

  embeddingProfileSetting.paint_fn((tab_d3_sel) => {
    if (embeddingProfileChangeSub) {
      embeddingProfileChangeSub.unsubscribe();
      embeddingProfileChangeSub = null;
    }

    paintProfileSetting(
      tab_d3_sel,
      embeddingProfileSetting,
      (sub) => {
        embeddingProfileChangeSub = sub;
      },
      lib,
      embeddingProfiles,
    );
  });

  const completionProfileSetting = arc.visual.setting()
    .id('completion-profile')
    .tab('Completion')
    .type(arc.visual.settingTypeEnum.STRING)
    .add_name('COMPLETION-PROFILE')
    .add_label('Selected Profile:')
    .value(setDefaultProfileValue(completionProfiles));

    completionProfileSetting.paint_fn((tab_d3_sel) => {
      if (completionProfileChangeSub) {
        completionProfileChangeSub.unsubscribe();
        completionProfileChangeSub = null;
      }

      paintProfileSetting(
        tab_d3_sel,
        completionProfileSetting,
        (sub) => {
          completionProfileChangeSub = sub;
        },
        lib,
        completionProfiles
      );
    });

    const embeddingMaxTokenSetting = arc.visual.setting()
    .id('embedding-max-tokens')
    .tab('Embeddings')
    .type(arc.visual.settingTypeEnum.INTEGER_NOSPINNER)
    .add_name('EMBEDDING_MAX_TOKENS')
    .add_label('Maximum tokens')
    .info('Provide zero or negative value to disable this setting.')
    .min(-1)
    .max(4000)
    .value('1000');

    const completionMaxTokenSetting = arc.visual.setting()
    .id('completion-max-tokens')
    .tab('Completion')
    .type(arc.visual.settingTypeEnum.INTEGER_NOSPINNER)
    .add_name('COMPLETION_MAX_TOKENS')
    .add_label('Maximum tokens')
    .info('Provide zero or negative value to disable this setting.')
    .min(-1)
    .max(10000)
    .value('4000');

    const completionEmptyContextPlaceholderSetting = arc.visual.setting()
    .id('completion-context-placeholder')
    .tab('Completion')
    .type(arc.visual.settingTypeEnum.STRING)
    .add_name('COMPLETION_CONTEXT_PLACEHOLDER')
    .add_label('Context placeholder text')
    .info('Placeholder text to use when context data is missing or empty. If set, this text will replace empty values to ensure meaningful input is provided to the model. If left blank, empty context values will be removed.')
    .value('');

    const contextOverflowPolicySetting = arc.visual.setting()
    .id('overflow-policy')
    .tab('Completion')
    .type(arc.visual.settingTypeEnum.RADIO)
    .info('Behavior for when the generated tokens length exceeds the context window size.')
    .group('Context Overflow Policy')
    .add_name('throw-error')
    .add_label('Throw an error')
    .add_name('remove-conversation-first')
    .add_label('Remove old conversation first')
    .add_name('truncate-context-first')
    .add_label('Truncate embedding context first')
    .value('throw-error');

  const contextPromptSetting = arc.visual.setting()
    .id('context-prompt')
    .tab('Completion')
    .type(arc.visual.settingTypeEnum.STRING)
    .add_name('CONTEXT_PROMPT')
    .add_label('Context prompt')
    .info('You can specify the first prompt message. Use <<context>> where you would like to add in the results from the vector database proximity search.')
    .is_textarea(true)
    .value('Use the below sources to answer the subsequent question. If the answer cannot be found in the provided sources, write "I could not find an answer".\n<<context>>');

  const questionPromptPrefixSetting = arc.visual.setting()
    .id('question-prompt-prefix')
    .tab('Completion')
    .type(arc.visual.settingTypeEnum.STRING)
    .add_name('QUESTION_PROMPT')
    .add_label('Question prompt')
    .info('You can specify the formatting for the user question. Use <<question>> where you would like to add in the user\'s question.')
    .is_textarea(true)
    .value('<<question>>');

  const parameterPassingForSourcesSetting = arc.visual.setting()
    .id('param-source')
    .tab('Parameters')
    .type(arc.visual.settingTypeEnum.RADIO)
    .info('Specify how many rows should be sent as parameters.')
    .group('Number of parameters to send')
    .add_name('none')
    .add_label('None')
    .add_name('first')
    .add_label('First')
    .add_name('all')
    .add_label('All')
    .value('all');

  const parameterSelectionPreferenceSetting = arc.visual.setting()
    .id('param-selection')
    .tab('Parameters')
    .type(arc.visual.settingTypeEnum.RADIO)
    .info('Select how parameters returned from the vector database should be passed on for further processing. Choose between sending all retrieved parameters or only those that were actually utilized by the Language Model (LLM) in its response.')
    .group('Parameter Specificity')
    .add_name('all')
    .add_label('All parameters')
    .add_name('used-by-llm')
    .add_label('Only used by LLM')
    .value('all');

  parameterPassingForSourcesSetting.addDependents([parameterSelectionPreferenceSetting], ['none'], ['first', 'all']);

  const saveConversationSetting = arc.visual.setting()
    .id('save-conversation')
    .tab('Parameters')
    .type(arc.visual.settingTypeEnum.CHECKBOX)
    .add_name('SAVE_CONVERSATION')
    .add_label('Save conversation in edit mode')
    .value('false');

  const userSettings = arc.visual.defaultSettings({ noAxes: true })
    .addSetting([
      welcomeMessageSetting,
      chatAppearanceSetting,
      embeddingMaxTokenSetting,
      completionProfileSetting,
      embeddingProfileSetting,
      completionMaxTokenSetting,
      contextOverflowPolicySetting,
      contextPromptSetting,
      completionEmptyContextPlaceholderSetting,
      questionPromptPrefixSetting,
      saveConversationSetting,
      parameterPassingForSourcesSetting,
      parameterSelectionPreferenceSetting,
    ])
    .removeSetting([
      'csv_download_quotation',
      'default_language',
      'nulls_order',
      'max_rowcount',
      'disable_incremental_queries',
      'trellis_borders',
      'trellis_cell_minimum_width',
      'trellis_cell_minimum_height',
      'visual_show_detail_data',
      'show_tooltip',
      'trellis_font_size',
      'tooltip_font',
      'tooltip_font_size',
    ]);

  arc.addColorSettings(userSettings, {
    excludeColorPaletteSetting: true,
  });

  const chatbot = arc.visual.api();

  const shelf_data = [
    {
      name: 'embedding_shelf',
      display_name: 'Embeddings',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'embedding_source_shelf',
      display_name: 'Embedding Context',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
      shelf_warning: true,
      isSensitive: true,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltip',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
  ];

  const constraints = {
    minimums: {
      embedding_shelf: 1,
      embedding_source_shelf: 1,
    },
  };

  chatbot.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(userSettings);
    return chatbot;
  };

  const setDownloadInfo = ({ conversation }) => {
    const createRows = () => {
      const rows = [];
      let lastQuestion = null;

      conversation.forEach((msg) => {
          if (msg.type === 'question') {
              lastQuestion = msg.text;
          } else if (msg.type === 'answer' && lastQuestion !== null) {
              rows.push([
                lastQuestion,
                msg.text,
                msg.tooltips.map((tooltip) => tooltip.source).join('\n'),
                (msg.embeddings ?? []).join(' '),
              ]);

              lastQuestion = null;
          }
      });

      return rows;
    };

    downloadInfo = {
      colNames: ['Question', 'Answer', 'Sources', 'Vector embeddings'],
      rows: createRows()
    };
  };

  chatbot.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);

    const sessionKey = arc.misc.sessionKeyGenerator.chatBot(lib.report_id());
    settings.conversation = settings.user_settings['save-conversation'] === 'true' ? JSON.parse(sessionStorage.getItem(sessionKey)) : null;

    lib.saveLimit(settings);
    saved_settings = settings;
    return settings;
  };

  chatbot.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return chatbot;
    }

    if (switchingVisualType) {
      settings.limit = 7;
    }

    saved_settings = settings;

    lib.restore_basics(settings, switchingVisualType);
    lib.restoreLimit(settings);

    return chatbot;
  };

  chatbot.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    const embeddingMessage = [
      lib.createPopoverMessages('embeddings'),
      lib.createPopoverMessages('acceptsMultiple')
    ];
    const sourceMessage = [
      lib.createPopoverMessages('embeddingResults'),
      lib.createPopoverMessages('acceptsMultiple')
    ];
    const tooltipMessage = [
      lib.createPopoverMessages('embeddingSource'),
      lib.createPopoverMessages('acceptsOne')
    ];

    const warningMessage = [
      lib.createPopoverMessages('embeddingWarning'),
    ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      embedding_shelf: embeddingMessage,
      embedding_source_shelf: sourceMessage,
      tooltip_shelf: tooltipMessage,
      embedding_source_shelf_warning: warningMessage,
    });

    lib.addLimit();

    return chatbot;
  };

  chatbot.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  chatbot.viz_setDataResult = function(result) {
    data_result = result;
    return chatbot;
  };

  chatbot.getDownloadInfo = function () {
    return downloadInfo;
  };

  chatbot.viz_drawVisual = function (viz_div) {
    lib.chart_div(viz_div);

    if (!root) {
      lib.undraw();
    }

    if (!arc.frontend.featureFlipper.getValueById('enable_ai_assistant')) {
      lib.alertMessage('Enable AI visual in site settings to see this visual');

      return;
    }

    if (!embeddingProfiles || !Object.keys(embeddingProfiles.profileSettings).length) {
      lib.alertMessage('Create at least one embeddings profile to use the AI visual'); // notificationMessage

      return;
    }

    if (!completionProfiles || !Object.keys(completionProfiles.profileSettings).length) {
      lib.alertMessage('Create at least one completion profile to use the AI visual'); // notificationMessage

      return;
    }

    checkIfActiveProfileExists(embeddingProfileSetting, embeddingProfiles, lib);
    checkIfActiveProfileExists(completionProfileSetting, completionProfiles, lib);

    const sb_rows = data_result.rows();
    if (!sb_rows || sb_rows.length === 0) {
      return chatbot;
    }

    const sendHttpReq = (proxyRequest, chunkCb) => new Promise((resolve, reject) => {
      const dsreq = { version: 1, type: 'HTTP', data: proxyRequest };
      const dataconn_id = arc.globals.http_dataconnection_id;
      const paramTransformer = (param) => ({ dsreq: param.dsreq,
        async: 'true',
        dataconnection_id: dataconn_id,
        parent_report_id: lib.dashboard()?.dashboard_id(),
        report_id: lib.report_id() });

      // there are cases when neither the success nor error callbacks are called
      // (e.g. the query is cancelled by the user)
      const errorReportingCb = (_, msg, error) => {
        const errorMsg = new Error(`${msg}: ${error}`);
        reject(errorMsg);
      };

      lib.runSqlCmd(
        'http proxy request',
        (data) => {
          if (data.response) {
            if (data.response.error) {
              reject(data.response.error); // Proxy error
            } else if (data.response.response?.body?.error) {
              reject(data.response.response?.body?.error.message); // upstream error
            } else {
              resolve(data.response);
            }
          } else {
            reject(new Error(`response not found in data ${data}`));
          }
        },
        JSON.stringify(dsreq),
        undefined,
        (error) => {
          reject(error);
        },
        paramTransformer,
        errorReportingCb,
        chunkCb
      );
    });

    const sendDsReq = (embeddings) => new Promise((resolve, reject) => {
        const dsreq = lib.getCheckedDataRequest(constraints);

        const embeddingsVectorName = dsreq.columns()[0].colname();

        const rb = dsreq.rb();
        const sql = dsreq.sql();

        rb.addFilter(`[${embeddingsVectorName}] KNN (${embeddings.join(',')})`);

        const dsregObj = rb.dsreq();

        // remove the first column, which is the embeddings. It can be large and it is not needed
        dsregObj.dimensions.splice(0, 1);

        lib.dsreq(dsregObj);
        dsreq.dsreq(dsregObj);
        dsreq.finalize();

        lib.runSqlCmd(
          sql,
          (data) => {
            resolve(data);
          },
          dsreq.dsreq(),
          undefined,
          (error) => {
            reject(error);
          }
        );
      });

      const getEmbeddingSourceInfo = (rows) => {
        const embeddingsSourceShelf = lib.shelves().getShelf('embedding_source_shelf');
        const embeddingsSourceShelfElements = embeddingsSourceShelf.elements();
        const embeddingSourceAliases = embeddingsSourceShelfElements.map((element) => {
            let { alias } = element.getExpressionComponents();
            alias = alias.slice(1, alias.length - 1);
            return alias;
        });

        return {
            filteredRows: rows.map((row) => row.slice(0, embeddingSourceAliases.length)),
            aliases: embeddingSourceAliases,
            contextPrompt: contextPromptSetting.value(),
            contextPlaceholder: completionEmptyContextPlaceholderSetting.value()
        };
    };

    const getTooltips = (rows) => {
      const tooltipsSourceShelf = lib.shelves().getShelf('tooltip_shelf');
      const tooltipsShelfElements = tooltipsSourceShelf.elements();
      const tooltipsAliases = tooltipsShelfElements.map((element) => {
        let { alias } = element.getExpressionComponents();
        alias = alias.slice(1, alias.length - 1);
        return alias;
      });

      // Remove duplicates
      const uniqueData = rows.reduce((acc, current) => {
        const currentAsString = JSON.stringify(current);

        if (!acc.temp.includes(currentAsString)) {
          acc.temp.push(currentAsString);
          acc.unique.push(current);
        }

        return acc;
      }, { temp: [], unique: [] }).unique;

      const tooltips = [];

      uniqueData.forEach((row) => {
        const tooltip = {};

        row
          .slice(row.length - tooltipsAliases.length)
          .forEach((column, columnIndex) => {
            const alias = tooltipsAliases[columnIndex];

            tooltip[alias] = column;
          });

          if (Object.keys(tooltip).length > 0) {
            tooltips.push(tooltip);
          }
      });

      return tooltips;
    };

    const sessionKey = arc.misc.sessionKeyGenerator.chatBot(lib.report_id());
    const conversation = sessionStorage.getItem(sessionKey);
    if (conversation) {
      setDownloadInfo({ conversation: JSON.parse(conversation) });
    }

    root = arc.frontend.renderReactComponent(
      root && !$(`#${viz_div}`).is(':empty') ? undefined : viz_div,
      arc.frontend.Chatbot,
      {
        sendDsReq,
        sendHttpReq,
        getEmbeddingSourceInfo,
        getTooltips,
        setDownloadInfo,
        alertMessage: lib.alertMessage,
        addParams: lib.sendParameters,
        needSave: lib.page_needs_save,
        reportId: lib.report_id(),
        datasetId: lib.dataset_id(),
        savedConversation: saved_settings.conversation,
        settings: {
          welcomeMessage: welcomeMessageSetting.value(),
          isMarkdown: chatAppearanceSetting.value() === 'markdown',
          embeddingProfile: embeddingProfileSetting.value(),
          completionProfile: completionProfileSetting.value(),
          embeddingMaxTokens: embeddingMaxTokenSetting.value(),
          completionMaxTokens: completionMaxTokenSetting.value(),
          overflowPolicy: contextOverflowPolicySetting.value(),
          paramSelection: parameterSelectionPreferenceSetting.value(),
          paramSource: parameterPassingForSourcesSetting.value(),
          saveConversation: saveConversationSetting.value() === 'true',
          questionPromptPrefix: questionPromptPrefixSetting.value(),
          fontSize: `${userSettings.find_by_id('visual_font_size').value() || 16}px`
        }
      },
      root
    );

    return chatbot;
  };

  chatbot.viz_destroy = function () {
    root?.unmount();

    root = undefined;

    return chatbot;
  };

  return chatbot;
};

arc.visual.plugins.chatbot_extension = function() {
  const ext = function() { return 'arc.visual.plugins.chatbot_extension'; };

  ext.viz_name = function() {
    return 'chatbot';
  };

  ext.viz_tooltip = function() {
    return 'chatbot';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_chatbot.png?v=1';
  };

  ext.viz = arc.visual.plugins.chatbot;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.chatbot_extension());
}());
