window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.report = arc.report || {};
arc.report.chord_id = 1;

arc.visual.plugins.chord = function(_mark_type) {
  var chord = arc.visual.api();

  var saved_settings = {},
      dataResult,
      lib,
      trellis,
      mark_type = _mark_type || 'chord',
      sort_orders_asc = {},
      fixed_nodes = [],
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  var fixed_color_legend = arc.report.user_settings.fixed_color_legend();
  var same_domain = arc.report.user_settings.same_domain();
  let colorPalette = arc.colorpalette.setting();
  let colorOpacity = arc.report.user_settings.color_opacity();
  var user_settings = arc.visual.defaultSettings();

  if (mark_type === 'chord') {
    user_settings.addSetting([
      arc.settings.chord.minimumSize(),
      same_domain,
      colorPalette,
      colorOpacity,
      fixed_color_legend,
      arc.colormap.dimToColorMappingSetting(),
      arc.colormap.measureRangesSetting(),
    ])
    .removeSetting([
      'axis_label_font_size',
      'axis_ticks_font_size',
      'label_font_size', // TODO: Need to implement
      'label_color',
      'legend_font_size',
    ]);
  }

  if (mark_type === 'treemap') {
    user_settings.addSetting([
      colorPalette,
      colorOpacity,
      fixed_color_legend,
      arc.colormap.dimToColorMappingSetting(),
    ]).removeSetting([
      'axis_label_font_size',
      'axis_ticks_font_size',
      'label_font_size', // TODO: Need to implement
      'label_color', // TODO: Need to implement
      'legend_font_size',
    ]);
  }

  if (mark_type === 'dendrogram') {
    user_settings.addSetting([
      arc.settings.network.linkOpacity(mark_type),
      arc.settings.network.linkWidthRange(mark_type),
      arc.settings.network.linkColor(mark_type),
      arc.settings.network.showNodeLabels(mark_type),
      same_domain,
      colorPalette,
      colorOpacity,
      fixed_color_legend,
      arc.colormap.dimToColorMappingSetting(),
    ])
    .removeSetting([
      'axis_label_font_size',
      'axis_ticks_font_size',
      'label_font_size', // TODO: Need to implement
      'label_color', // TODO: Need to implement
      'legend_font_size',
      'visual_show_detail_data',
    ]);
  }

  if (mark_type === 'network') {
    let nodeColors = colorPalette.tab('Node Colors');
    let nodeLegend = fixed_color_legend.tab('Node Colors');
    let nodeMapping = arc.colormap.dimToColorMappingSetting().tab('Node Colors');
    let linkColors = arc.colorpalette.setting().tab('Link Colors').id('link_color_palette');
    let linkLegendTitleSetting = arc.settings.network.linkLegendTitle();
    let nodeLegendTitleSetting = arc.settings.network.nodeLegendTitle();
    let combinedMarkType = arc.settings.network.combinedNodeMarkType();
    const enableDirectionArrows = arc.settings.network.addDirectionArrows();
    const directionArrowSize = arc.settings.network.directionArrowSize();
    enableDirectionArrows.addDependents([directionArrowSize]);
    const unifiedNodes = arc.settings.network.unifyNodes();
    unifiedNodes.addDependents([combinedMarkType]);
    const addNodeLegend = arc.settings.network.addNodeLegend();
    addNodeLegend.addDependents([nodeLegendTitleSetting]);
    const addLinkLegend = arc.settings.network.addLinkLegend();
    addLinkLegend.addDependents([linkLegendTitleSetting]);

    user_settings.addSetting([
      arc.settings.network.rememberNodePositions(),
      arc.settings.network.emptyColorSetting(),
      nodeColors,
      linkColors,
      nodeLegend,
      nodeMapping,
      unifiedNodes,
      arc.settings.network.linkColor(mark_type),
      arc.settings.network.linkOpacity(mark_type),
      arc.settings.network.srcNodeMarkType(),
      arc.settings.network.tgtNodeMarkType(),
      combinedMarkType,
      arc.settings.network.linkWidthRange(mark_type),
      arc.settings.network.nodeSizeRange(),
      arc.settings.network.linkDistance(),
      arc.settings.network.linkStrength(),
      arc.settings.network.nodeFriction(),
      arc.settings.network.nodeCharge(),
      arc.settings.network.srcNodeOpacity(),
      arc.settings.network.showNodeLabels(mark_type),
      arc.settings.network.layoutGravity(),
      arc.settings.network.layoutTheta(),
      enableDirectionArrows,
      directionArrowSize,
      addNodeLegend,
      addLinkLegend,
      arc.settings.network.legendWidth(),
      nodeLegendTitleSetting,
      linkLegendTitleSetting,
      arc.settings.network.linkPointerColor(),
    ])
    .removeSetting([
      'axis_label_font_size',
      'axis_ticks_font_size',
      'label_font_size', // TODO: Need to implement
      'label_color', // TODO: Need to implement
      'legend_font_size',
    ]);
  }

  let shelfLabels = {};
  if (mark_type === 'network') {
    shelfLabels.dimension = 'Nodes';
    shelfLabels.measure = 'Link Width';
  } else {
    shelfLabels.dimension = 'Dimensions';
    shelfLabels.measure = 'Measure';
  }

  function linkColor() {
    return user_settings.find_by_id('link_color').value();
  }

  function chordMinimumSize() {
    var s = user_settings.find_by_id('chord_minimum_size');
    if (!s || isNaN(s.value())) {
      return 200;
    }

    return Number(s.value());
  }

  chord.rememberNodePositions = function() {
    var s = user_settings.find_by_id('remember_node_positions');
    if (!s) {
      return false;
    }

    return s.isChecked;
  };

  chord.fixed_nodes = function() {
    return fixed_nodes;
  };

  chord.same_domain = function() {
    return same_domain.isChecked;
  };

  chord.opacity = function(settingId) {
    var o = user_settings.find_by_id(settingId)
                         .value();

    var v;
    if (o === '') {
      v = 80;
    }

    v = Number(o);
    if (v <= 0) {
      return 0;
    }

    if (v >= 100) {
      return 1;
    }

    return (v / 100);
  };

  chord.showNodeLabels = function() {
    return user_settings.find_by_id('show_node_labels').isChecked;
  };

  chord.linkOpacity = function() {
    return chord.opacity('link_opacity');
  };

  chord.linkWidthRange = function() {
    return user_settings.find_by_id('link_width_range')
                        .get_range();
  };

  var shelf_data = [
                                        {
                      name: 'dimensions_shelf',
                      display_name: shelfLabels.dimension,
                      shelf_type: arc.report.shelf_types.COLUMN,
                      column_type: arc.report.columnTypeEnum.DIMENSION,
                    },
                    {
                      name: 'aggregates_shelf',
                      display_name: shelfLabels.measure,
                      shelf_type: arc.report.shelf_types.COLUMN,
                      column_type: arc.report.columnTypeEnum.AGGREGATE,
                    },
                  ];
  if (mark_type === 'network') {
    const linkColorShelf = {
      name: 'link_color_shelf',
      display_name: 'Link Color',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    };
    const srcSizeShelf = {
      name: 'source_node_size_shelf',
      display_name: 'Source Node Size',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    };
    const tgtSizeShelf = {
      name: 'target_node_size_shelf',
      display_name: 'Target Node Size',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    };
    const srcColorShelf = {
      name: 'source_node_color_shelf',
      display_name: 'Source Node Color',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    };
    const tgtColorShelf = {
      name: 'target_node_color_shelf',
      display_name: 'Target Node Color',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    };
    shelf_data.push(linkColorShelf);
    shelf_data.push(srcSizeShelf);
    shelf_data.push(tgtSizeShelf);
    shelf_data.push(srcColorShelf);
    shelf_data.push(tgtColorShelf);
  }

  let tooltipShelf = {
                      name: 'tooltip_shelf',
                      display_name: 'Tooltips',
                      shelf_type: arc.report.shelf_types.COLUMN,
                      column_type: arc.report.columnTypeEnum.AGGREGATE,
                    };

  let xShelf = {
                  name: 'x_shelf',
                  display_name: 'X Trellis',
                  shelf_type: arc.report.shelf_types.COLUMN,
                  column_type: arc.report.columnTypeEnum.DIMENSION,
               };

  let yShelf = {
                  name: 'y_shelf',
                  display_name: 'Y Trellis',
                  shelf_type: arc.report.shelf_types.COLUMN,
                  column_type: arc.report.columnTypeEnum.DIMENSION,
                };

  let filterShelf = {
                      name: 'filters_shelf',
                      display_name: 'Filters',
                      display_html: arc.report.filters_shelf_html(),
                      shelf_type: arc.report.shelf_types.PREDICATE,
                      column_type: arc.report.columnTypeEnum.UNKNOWN,
                      onShelfListChange: arc.report.updateFilterShelfLabel,
                    };
  shelf_data.push(tooltipShelf);
  shelf_data.push(xShelf);
  shelf_data.push(yShelf);
  shelf_data.push(filterShelf);

  var constraints = {
    minimums: {
      dimensions_shelf: 2,
    },
    maximums: {
      dimensions_shelf: 2,
      source_node_size_shelf: 1,
      target_node_size_shelf: 1,
      source_node_color_shelf: 1,
      target_node_color_shelf: 1,
    },
  };

  if ((mark_type === 'dendrogram') ||
      (mark_type === 'treemap')) {
    constraints.maximums = {};
  }

  if (mark_type !== 'network') {
    constraints.minimums.aggregates_shelf = 1;
    constraints.maximums.aggregates_shelf = 1;
  }

  chord.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return chord;
  };

  chord.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;

    fixed_nodes = [];
    if (chord.rememberNodePositions()) {
      d3.selectAll('#' + lib.chart_div() + ' .viz-node.fixed')
        .each(function(d) {
          var $this = $(this);
          fixed_nodes.push({
            data: d.data,
            x: d.x,
            y: d.y,
            w: d.w,
            h: d.h,
          });
        });
    }

    settings.fixed_nodes = fixed_nodes;
    return settings;
  };

  chord.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return chord;
    }

    saved_settings = settings;
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    fixed_nodes = settings.fixed_nodes || [];
    lib.restore_basics(settings, switchingVisualType);
    return chord;
  };

  chord.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        dimensionMessages = [
          lib.createPopoverMessages('dimensionChordCorrelation'),
        ],
        measureMessages = [
          lib.createPopoverMessages('aggregateChordCorrelation'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        nodeSizeMessage = [
          lib.createPopoverMessages('nodeSize'),
        ],
        nodeColorMessage = [
          lib.createPopoverMessages('nodeColor'),
        ],
        linkSizeMessage = [
          lib.createPopoverMessages('linkSize'),
        ],
        linkColorMessage = [
          lib.createPopoverMessages('linkColor'),
        ];

    if (mark_type === 'treemap') {
      dimensionMessages = [
        lib.createPopoverMessages('treemapDim'),
      ];
      measureMessages = [
        lib.createPopoverMessages('treemapAg'),
        lib.createPopoverMessages('acceptsOne'),
      ];
    }

    // Create those popovers
     lib.addShelfDescription(constraints, {
       x_shelf: xMessage,
       y_shelf: yMessage,
       dimensions_shelf: dimensionMessages,
       aggregates_shelf: measureMessages,
       source_node_size_shelf: nodeSizeMessage,
       target_node_size_shelf: nodeSizeMessage,
       source_node_color_shelf: nodeColorMessage,
       target_node_color_shelf: nodeColorMessage,
       link_size_shelf: linkSizeMessage,
       link_color_shelf: linkColorMessage,
     });

    return chord;
  };

  chord.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  chord.viz_setDataResult = function(result) {
    dataResult = result;
    return chord;
  };

  chord.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    var sb_rows = dataResult.rows();
    if (!sb_rows) {
      return chord;
    }

    if (sb_rows.length === 0) {
      return chord;
    }

    // Filter out null values in aggregates

    if (dataResult.agg_start_idx() > 0) {
      let makeGetter = function (index) {
        return function(row) {
          return row[index];
        };
      };

      let getters = [];
      for (let idx = dataResult.agg_start_idx(); idx <= dataResult.agg_end_idx(); idx++) {
        getters.push(makeGetter(idx));
      }

      sb_rows = lib.filter_nulls(
        {
          rows: sb_rows,
        },
        getters);
    }

    lib.unselectOnClick();

    var config = {
      rows: sb_rows,
      columns: dataResult.columns(),
    };

    lib.setupColors(config);

    var cols = lib.shelf_columns(config.columns, 'dimensions_shelf');
    config.dim_startidx = cols[0].colindex();
    config.dim_endidx = cols[cols.length - 1].colindex();

    cols = lib.shelf_columns(config.columns, 'aggregates_shelf');
    if (cols.length === 0) {
      config.agg_startidx = -1;
    } else {
      config.agg_startidx = cols[0].colindex();
    }

    config.tooltipCols = lib.shelf_columns(config.columns, 'tooltip_shelf');

    trellis = arc.visual.trellis()
                        .x_shelf_name('x_shelf')
                        .y_shelf_name('y_shelf')
                        .x_shelf_reserved(0)
                        .y_shelf_reserved(0)
                        .rows(config.rows)
                        .sort_orders_asc(sort_orders_asc)
                        .columns(config.columns)
                        .viz_lib(lib)
                        .viz_div_sel(d3.select('#' + viz_div))
                        .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                        .redrawVisualCb(function() {
                          allowLargeTrellisTemp = true;
                          chord.viz_drawVisual(viz_div);
                        })
                        .setup();

    if (trellis.error_state()) {
      return chord;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      chord.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');

    computeSvgParams(config, viz_div);

    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      config.rows = xy_rows;
      var cell_div = cell_sel.attr('id');
      switch (mark_type) {
        case 'chord':
          viz_drawVisualChord(config, cell_div);
          break;

        case 'network':
          chord.viz_drawVisualNetwork(config, cell_div);
          break;

        case 'dendrogram':
          chord.viz_drawVisualBundle(config, cell_div);
          break;

        case 'treemap':
          chord.viz_drawVisualTreemap(config, cell_div);
          break;

        default:
          console.log('invalid visual type', mark_type);
          break;
      }
    });

    return chord;
  };

  chord.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return chord;
  };

  chord.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return chord;
  };

  function computeSvgParams(config, viz_div) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
    });

    var margin = {
      top: 5,
      right: 10,
      bottom: 40,
      left: 10,
    };
    var width = elem_w - margin.left - margin.right;
    var height = elem_h - margin.top - margin.bottom;
    let minSize = chordMinimumSize();

    width = Math.max(width, minSize);
    height = Math.max(height, minSize);
    if (mark_type === 'chord') {
      width = Math.min(width, height);
      height = width;
    }

    config.width = width;
    config.height = height;
  }

  chord.getLinkColorScale = function(config) {
    var nodeColorScale = config.nodeColorScale,
        sourceDimGetter = config.sourceDimGetter,
        targetDimGetter = config.targetDimGetter,
        aggGetter = config.aggGetter,
        aggExtent = config.aggExtent;

    var linkColorScale;
    var linkColorSelection = linkColor();

    switch (linkColorSelection) {
      case 'source':
        linkColorScale = function(d) {
          return nodeColorScale(sourceDimGetter(d));
        };

        return linkColorScale;

      case 'target':
        linkColorScale = function(d) {
          return nodeColorScale(targetDimGetter(d));
        };

        return linkColorScale;

      case 'aggregate':
        if (aggExtent) {
          const aggColorScale = lib.aggScale(aggExtent);

          linkColorScale = function(d) {
            return aggColorScale(aggGetter(d));
          };

          return linkColorScale;
        }

        linkColorScale = function(d) {
          return '#ddd';
        };

        return linkColorScale;

      case 'fixed':
        linkColorScale = function(d) {
          return '#ddd';
        };

        return linkColorScale;

      default:
        linkColorScale = function(d) {
          return '#ddd';
        };

        return linkColorScale;
    }

    // NOTREACHED
    return undefined;
  };

  chord.nodesAndLinks = function() {
    var f = function() {
      return 'nodesAndLinks';
    };

    var nodes = [],
        srcNodes = [],
        tgtNodes = [],
        nodesH = {},
        srcNodesH = {},
        tgtNodesH = {},
        links = [];

    f.nodes = function() {
      return nodes;
    };

    f.srcNodes = function() {
      return srcNodes;
    };

    f.tgtNodes = function() {
      return tgtNodes;
    };

    f.nodesH = function() {
      return nodesH;
    };

    f.srcNodesH = function() {
      return srcNodesH;
    };

    f.tgtNodesH = function() {
      return tgtNodesH;
    };

    f.links = function() {
      return links;
    };

    function getSourceNode(nodeIdentifier) {
      if (nodeIdentifier in srcNodesH) {
        return srcNodesH[nodeIdentifier];
      }

      if (!chord.same_domain()) {
        return undefined;
      }

      if (nodeIdentifier in tgtNodesH) {
        return tgtNodesH[nodeIdentifier];
      }

      return undefined;
    }

    function getTargetNode(nodeIdentifier) {
      if (nodeIdentifier in tgtNodesH) {
        return tgtNodesH[nodeIdentifier];
      }

      if (!chord.same_domain()) {
        return undefined;
      }

      if (nodeIdentifier in srcNodesH) {
        return srcNodesH[nodeIdentifier];
      }

      return undefined;
    }

    f.process = function(config) {
      config.rows.forEach(function(d) {
        var agg;
        if (config.agg_startidx >= 0 && $.isNumeric(d[config.agg_startidx])) {
          agg = Number(d[config.agg_startidx]);
        } else {
          agg = 1;
        }

        var sourceNodeIdentifier = [0, d[config.dim_startidx]],
            targetNodeIdentifier = [chord.same_domain() ? 0 : 1, d[config.dim_startidx + 1]],
            sourceNode,
            targetNode;

        sourceNode = getSourceNode(sourceNodeIdentifier);
        if (sourceNode === undefined) {
          sourceNode = {
            type: 'src',
            data: sourceNodeIdentifier,
            index: nodes.length - 1,
            size: agg,
            combinedNode: d.slice(0),
          };

          sourceNode.combinedNode[config.dim_startidx + 1] = undefined;

          srcNodes.push(sourceNode);
          nodes.push(sourceNode);
          nodesH[sourceNodeIdentifier] = sourceNode;
          srcNodesH[sourceNodeIdentifier] = sourceNode;
        } else {
          sourceNode.size += agg;
          if (config.agg_startidx) {
            sourceNode.combinedNode[config.agg_startidx] = sourceNode.size;
          }
        }

        targetNode = getTargetNode(targetNodeIdentifier);
        if (targetNode === undefined) {
          targetNode = {
            type: 'dst',
            data: targetNodeIdentifier,
            index: nodes.length - 1,
            size: agg,
            combinedNode: d.slice(0),
          };

          targetNode.combinedNode[config.dim_startidx] = undefined;

          tgtNodes.push(targetNode);
          nodes.push(targetNode);
          nodesH[targetNodeIdentifier] = targetNode;
          tgtNodesH[targetNodeIdentifier] = targetNode;
        } else {
          targetNode.size += agg;
          if (config.agg_startidx) {
            targetNode.combinedNode[config.agg_startidx] = targetNode.size;
          }
        }

        links.push({
          source: sourceNode,
          target: targetNode,
          data: d,
        });
      });

      return f;
    };

    return f;
  };

  function viz_drawVisualChord(config, viz_div) {
    if (config.rows.length === 0) {
      return;
    }

    // Allocate a unique chord id for use in group names below
    var chord_id = ++arc.report.chord_id;

    var sb_rows = config.rows,
        sb_colnames = config.columns.map(function(d) {
          return d.colname();
        }),

        sb_coltypes = config.columns.map(function(d) {
          return d.coltype();
        }),

        sb_coldisplayformat = config.columns.map(function (d) {
          return d.coldisplayformat();
        });

    var donutwidth = 24,
        outerRadius = config.width / 2 - 10,
        innerRadius = outerRadius - donutwidth;

    var labels_dict = {},
        labels_col0_dict = {},
        labels_col1_dict = {},
        label_idx_to_type = {},
        labels = [],
        label_idx = 0,
        aggs = {},
        agg_colindex = dataResult.agg_start_idx();

    sb_rows.forEach(function(d) {
      d[agg_colindex] = Number(d[agg_colindex]);
      if (!(d[config.dim_startidx] in labels_dict)) {
        labels_dict[d[config.dim_startidx]] = label_idx;
        labels.push(d[config.dim_startidx]);
        label_idx_to_type[label_idx] = 0;
        ++label_idx;
      }

      if (!(d[config.dim_startidx] in labels_col0_dict)) {
        labels_col0_dict[d[config.dim_startidx]] = labels_dict[d[config.dim_startidx]];
      }

      if (!(d[config.dim_endidx] in labels_dict)) {
        labels_dict[d[config.dim_endidx]] = label_idx;
        labels.push(d[config.dim_endidx]);
        label_idx_to_type[label_idx] = 1;
        ++label_idx;
      }

      if (!(d[config.dim_endidx] in labels_col1_dict)) {
        labels_col1_dict[d[config.dim_endidx]] = labels_dict[d[config.dim_endidx]];
      }

      aggs[[labels_dict[d[config.dim_startidx]], labels_dict[d[config.dim_endidx]]]] =
          d[agg_colindex];
    });

    var colors = lib.dimensionalColorScale();
    colors.domain(labels);

    var matrix = [];
    for (var i = 0; i < label_idx; i++) {
      var row = [];
      for (var j = 0; j < label_idx; j++) {
        var k = [i, j];
        if (k in aggs) {
          row.push(aggs[k]);
        } else {
          if (chord.same_domain()) {
            row.push(0);
          } else {
            k = [j, i];
            if (k in aggs) {
              row.push(aggs[k]);
            } else {
              row.push(0);
            }
          }
        }
      }

      matrix.push(row);
    }

    var svg_arc = d3.svg.arc()
        .innerRadius(innerRadius)
        .outerRadius(outerRadius);

    var layout = d3.layout.chord()
        .padding(0.04)
        .sortSubgroups(d3.descending)
        .sortChords(d3.ascending);

    var path = d3.svg.chord()
        .radius(innerRadius);

    var svg = d3.select('#' + viz_div).append('svg')
        .attr('width', config.width)
        .attr('height', config.height)
          .append('g')
          .attr('class', 'viz-chord-circle')
          .attr('transform', 'translate(' + config.width / 2 + ',' +
              config.height / 2 + ')');

    svg.append('circle')
       .attr('r', outerRadius);

    // Compute the chord layout.
    layout.matrix(matrix);

    // Add a group per neighborhood.
    var group = svg.selectAll('.viz-chord-group')
        .data(layout.groups)
        .enter()
          .append('g')
          .attr('class', 'viz-chord-group')
          .on('mouseover.group', function(d, i) {
            mouseover(d, i, this);
          })
          .on('mouseout.group', function(d, i) {
            mouseout(d, i, this);
          });

    var group_params_fn = function(d, i, e, dimsOnly) {
      let src_col_idx = chord.same_domain() ? 0 : label_idx_to_type[i];
      var params = [];
      params.push({
          key: sb_colnames[src_col_idx],
          value: labels[i],
          keytype: arc.report.columnTypeEnum.DIMENSION,
          valuetype: sb_coltypes[config.dim_startidx],
          expr: config.columns[src_col_idx].expr(),
      });

      if (!dimsOnly) {
        params.push({
          key: sb_colnames[agg_colindex],
          value: d.value,
          keytype: arc.report.columnTypeEnum.AGGREGATE,
          valuetype: sb_coltypes[agg_colindex],
          expr: config.columns[agg_colindex].expr(),
        });
      }

      return params;
    };

    function group_text_fn(d, i) {
      const dimIdx = chord.same_domain() ? 0 : label_idx_to_type[i];
      const dim = sb_colnames[dimIdx];
      const agg = sb_colnames[agg_colindex];
      const dimFormat = sb_coldisplayformat[dimIdx];
      const aggFormat = sb_coldisplayformat[agg_colindex];
      const dimValue = lib.formattedDisplay(dimFormat, labels[i]);
      const aggValue = lib.formattedDisplay(aggFormat, d.value);

      const pre_kvs = [
        { key: dim, value: dimValue, },
        { key: agg, value: aggValue, },
      ];

      return lib.text_from_columns([], [], { pre_kvs, });
    }

    // Tooltips
    var groups_tt =
        arc.tooltip()
           .dom_id('#tooltip')
           .addSelection(group)
           .text_fn(group_text_fn)
           .activate();

    var o = lib.colorOpacity();

    // Add the group arc.
    var groupPath = group.append('path')
        .attr('id', function(d, i) { return chord_id + 'group' + i; })
        .attr('d', svg_arc)
        .style('fill', function(d, i) {
          return colors(labels[i]);
        });

    if (o !== 1) {
      var groupOpacity = o;
      if (groupOpacity > 0.4) {
        groupOpacity = groupOpacity - 0.2;
      }

      groupPath.style('fill-opacity', groupOpacity);
    }

    // Add a text label.
    var groupText = group.append('text')
        .attr('x', 6)
        .attr('dy', 15);

    function label_text_fn(d, i) {
      const dimIdx = label_idx_to_type[i];
      return sb_coldisplayformat[dimIdx](labels[i]);
    }

    groupText.append('textPath')
        .attr('xlink:href', function(d, i) { return '#' + chord_id + 'group' + i; })
        .text(label_text_fn);

    // Remove the labels that don't fit.
    groupText.filter(function(d, i) {
               return ((groupPath[0][i].getTotalLength() / 2 - donutwidth) <
                  (this.getComputedTextLength() + 6));
             })
             .remove();

    // Add the chords.
    var chord_elems = svg.selectAll('.chord')
        .data(layout.chords)
        .enter().append('path')
          .attr('class', 'viz-chord-chordpath')
          .style('fill', function(d) {
            return colors(labels[d.source.index]);
          })
          .attr('d', path);

    if (o !== 1) {
      chord_elems.style('fill-opacity', o);
    }

    var params_fn = function(d) {
      var params = lib.getParams().getCopy();
      let src_col_idx = chord.same_domain() ? 0 : label_idx_to_type[d.source.index];
      let tgt_col_idx = chord.same_domain() ? 1 : label_idx_to_type[d.target.index];

      params.push({
        key: sb_colnames[src_col_idx],
        value: labels[d.source.index],
        keytype: arc.report.columnTypeEnum.DIMENSION,
        valuetype: sb_coltypes[src_col_idx],
        expr: config.columns[src_col_idx].expr(),
      });

      params.push({
        key: sb_colnames[tgt_col_idx],
        value: labels[d.target.index],
        keytype: arc.report.columnTypeEnum.DIMENSION,
        valuetype: sb_coltypes[tgt_col_idx],
        expr: config.columns[tgt_col_idx].expr(),
      });

      return params;
    };

    function dimensionsToRow(sourceDim, targetDim) {
      var outRow;

      sb_rows.some(function(row) {
        /*
         * Note that we are using == here, not ===, so that we can
         * do the appropriate string -> number conversions. Otherwise,
         * with integer inputs, the comparison does not work.
         */
        if ((row[config.dim_startidx] === sourceDim) &&
            (row[config.dim_endidx] === targetDim)) {
          outRow = row;
          return true;
        }

        return false;
      });

      return outRow;
    }

    function datumToData(d) {
      var sourceIdx = d.source.index,
          targetIdx = d.target.index,
          sourceLabelIdx = chord.same_domain() ? 0 : label_idx_to_type[sourceIdx],
          targetLabelIdx = chord.same_domain() ? 1 : label_idx_to_type[targetIdx],
          sourceColName = arc.misc.escapeHTML(sb_colnames[sourceLabelIdx]),
          targetColName = arc.misc.escapeHTML(sb_colnames[targetLabelIdx]),
          sourceLabel = labels[sourceIdx],
          targetLabel = labels[targetIdx],
          sourceFormat = sb_coldisplayformat[sourceLabelIdx],
          targetFormat = sb_coldisplayformat[targetLabelIdx],
          aggFormat = sb_coldisplayformat[agg_colindex],
          sourceAgg = lib.formattedDisplay(aggFormat, d.source.value),
          targetAgg = lib.formattedDisplay(aggFormat, d.target.value);

      sourceLabel = arc.misc.escapeHTML(lib.formattedDisplay(sourceFormat, sourceLabel));
      targetLabel = arc.misc.escapeHTML(lib.formattedDisplay(targetFormat, targetLabel));

      return {
        sourceLabel: sourceLabel,
        targetLabel: targetLabel,
        sourceAgg: sourceAgg,
        targetAgg: targetAgg,
      };
    }

    function text_fn(d) {
      var sel = datumToData(d);

      var row1 = dimensionsToRow(sel.sourceLabel, sel.targetLabel),
          row2 = dimensionsToRow(sel.targetLabel, sel.sourceLabel),
          row = row1 || row2;

      var cols = config.columns.filter(function(c) {
        var i = c.colindex();
        if ((i === config.dim_startidx) || (i === config.dim_endidx) || (i === agg_colindex)) {
          return false;
        }

        return true;
      });

      var kv1 = {
        key: sel.sourceLabel + ' → ' + sel.targetLabel,
        value: sel.sourceAgg,
      };

      var kv2 = {
        key: sel.targetLabel + ' → ' + sel.sourceLabel,
        value: sel.targetAgg,
      };

      var kvcombined = {
        key: sel.sourceLabel + ' ↔ ' + sel.targetLabel,
        value: sel.sourceAgg,
      };

      if (chord.same_domain()) {
        var t1 = '',
            t2 = '';

        if (row1) {
          t1 = lib.text_from_columns(row1, cols, {
            pre_kvs: [
              kv1,
            ],
          });
        }

        if (row2) {
          t2 = lib.text_from_columns(row2, cols, {
            pre_kvs: [
              kv2,
            ],
          });
        }

        return t1 + t2;
      }

      // Domains of the dimensions are different
      if (row) {
        return lib.text_from_columns(row, cols, {
          pre_kvs: [
            kvcombined,
          ],
        });
      }

      return lib.text_from_columns([], [], {
        pre_kvs: [
          kvcombined,
        ],
      });
    }

    lib.contextmenu('groups')
       .highlight_on_select(true)
       .addTooltip(groups_tt)
       .text_fn(group_text_fn)
       .params_fn(group_params_fn)
       .activate();

    arc.marks()
       .viz_lib(lib)
       .marker_name('chords')
       .highlight_on_select(true)
       .addSelection(chord_elems)
       .text_fn(text_fn)
       .params_fn(params_fn)
       .activate();

    function mouseout(d, i, elem) {
      arc.misc.removeClass($(elem), 'highlight-hover');
    }

    function mouseover(d, i, elem) {
      arc.misc.addClass($(elem), 'highlight-hover');

      chord_elems.classed('viz-chord-cfade', function(p) {
        return p.source.index !== i && p.target.index !== i;
      });
    }

    return chord;
  }

  chord.viz_destroy = function() {
    if (mark_type === 'network') {
      chord.viz_destroyNetwork();
    }
  };

  return chord;
};

arc.visual.plugins.chord_extension = function() {
  var ext = function() { return 'arc.visual.plugins.chord_extension'; };

  ext.viz_name = function() {
    return 'chord';
  };

  ext.viz_tooltip = function() {
    return 'chord';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_chord.png?v=1';
  };

  ext.viz = arc.visual.plugins.chord;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.chord_extension());

}());
