window.arc = window.arc || {};

(function() {
"use strict";

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.correlation_flow = function() {
  var saved_settings = {},
      data_result,
      lib;

  const minHeightPerNodeSetting = arc.settings.flow.minHeightPerNode();
  const minWidthPerNodeSetting = arc.settings.flow.minWidthPerNode();

  var fixed_color_legend = arc.report.user_settings.fixed_color_legend();
  var user_settings = arc.visual.defaultSettings({
                                  noTrellis: true,
                                })
                                .addSetting([
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  fixed_color_legend,
                                  minHeightPerNodeSetting,
                                  minWidthPerNodeSetting,
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'axis_ticks_font_size',
                                  'label_font_size', // TODO: Need to implement
                                  'label_color', // TODO: Need to implement
                                  'legend_font_size',
                                ]);

  var correlation_flow = arc.visual.api();

  var constraints = {
    minimums: {
      "dimensions_shelf": 2,
      "aggregates_shelf": 1,
    }
  };

  correlation_flow.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.user_settings(user_settings);
    return correlation_flow;
  };

  correlation_flow.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    saved_settings = settings;
    return settings;
  };

  correlation_flow.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return correlation_flow;
    }
    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    return correlation_flow;
  };

  correlation_flow.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();
    // Define messages to be included in shelves
    var dimensionMessages = [
                     lib.createPopoverMessages('dimensionChordCorrelation')
                    ],
        measureMessages = [
                     lib.createPopoverMessages('aggregateChordCorrelation'),
                     lib.createPopoverMessages('acceptsOne')
                    ];
    // Create those popovers
    lib.addShelfDescription(constraints, {
                                            dimensions_shelf: dimensionMessages,
                                            aggregates_shelf: measureMessages
                                          });

    return correlation_flow;
  };

  correlation_flow.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  correlation_flow.viz_setDataResult = function(result) {
    data_result = result;
    return correlation_flow;
  };

  correlation_flow.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    var rows = data_result.rows();
    var columns = data_result.columns();
    if (!rows) {
      return correlation_flow;
    }

    var data = correlation_flow.rows_to_data(rows, columns);

    lib.draw_sankey({
      data: data,
      funnel_prefix: 'FUNNEL',
      flow_type: 'flow',
      goals_display: false,
      showFixedColor: fixed_color_legend.isChecked,
      rows: rows,
      columns: columns,
      for_correlation_flow: true,
      minHeightPerNode: Number(minHeightPerNodeSetting.value()),
      minWidthPerNode: Number(minWidthPerNodeSetting.value()),
    });

    return correlation_flow;
  };

  correlation_flow.rows_to_data = function(rows, columns) {
    var dim_start_idx = data_result.dim_start_idx(),
        dim_end_idx = data_result.dim_end_idx();

    var data = {
          'nodes': [],
          'links': [],
          'pathrows': [],
          'pathfrequencies': []
        },
        nodes_idx = {},
        links_idx = {},
        row_idx;

    $.each(rows, function(pathrowidx, row) {
      for (row_idx = dim_start_idx;
           row_idx <= dim_end_idx;
           ++row_idx) {
        var pos = row_idx - dim_start_idx;
        var name = row[row_idx] + "_" + pos;
        if (!(name in nodes_idx)) {
          data.nodes.push({
            index: pos,
            name: name,
            display: row[row_idx],
            paths: [pathrowidx],
            formatter: columns[pos].coldisplayformat(),
          });
          nodes_idx[name] = data.nodes.length - 1;
        } else {
          data.nodes[nodes_idx[name]].paths.push(pathrowidx);
        }
        if (row_idx === dim_start_idx) {
          data.pathfrequencies.push(Number(row[data_result.agg_start_idx()]));
          data.pathrows.push([nodes_idx[row[row_idx] + "_" + (pos)]]);
        } else {
          data.pathrows[pathrowidx].push(nodes_idx[row[row_idx] + "_" + (pos)]);
          var src = nodes_idx[row[row_idx-1] + "_" + (pos-1)],
              tgt = nodes_idx[row[row_idx] + "_" + (pos)],
              links_key = [src, tgt],
              agg_start_idx = data_result.agg_start_idx();
          if (!(links_key in links_idx)) {
            data.links.push({
              source: src,
              target: tgt,
              formatter: columns[agg_start_idx].coldisplayformat(),
              value: Number(row[agg_start_idx]),
              paths: [pathrowidx]
            });
            links_idx[links_key] = data.links.length - 1;
          } else {
            data.links[links_idx[links_key]].value += Number(row[data_result.agg_start_idx()]);
            data.links[links_idx[links_key]].paths.push(pathrowidx);
          }
        }
      }
    });

    return data;
  };

  return correlation_flow;
};

arc.visual.plugins.correlation_flow_extension = function() {
  var ext = function() { return "arc.visual.plugins.correlation_flow_extension"; };

  ext.viz_name = function() {
    return "corelation-flow";
  };

  ext.viz_tooltip = function() {
    return "correlation flow";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_corelation-flow.png?v=1";
  };

  ext.viz = arc.visual.plugins.correlation_flow;

  return ext;
};

arc.visual.plugins['corelation-flow'] = arc.visual.plugins.correlation_flow;

arc.visual.extensions.push(arc.visual.plugins.correlation_flow_extension());

}());
