window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.correlation = function() {
  var saved_settings = {},
      data_result,
      lib,
      sort_orders_asc = {},
      trellisCellData = {},
      trellisCellParams = {},
      previousDataTracker = arc.transition.dataTracker(),
      colorFetcher = arc.canvas.colorFetcher(),
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  const trellisCellMargin = {
    top: 5,
    right: 5,
    bottom: 5,
    left: 5,
  };

  let minimumSize = arc.settings.correlation.minimumSize(),
      maxXLegendHeight = arc.settings.correlation.maxXLegendHeight(),
      maxYLegendWidth = arc.settings.correlation.maxYLegendWidth(),
      showXAxis = arc.settings.line.showXAxis(),
      showYAxis = arc.settings.line.showYAxis();

  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.settings.visuals.enableAnimations(),
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  arc.colormap.measureRangesSetting(),
                                  minimumSize,
                                  maxXLegendHeight,
                                  maxYLegendWidth,
                                  showXAxis,
                                  showYAxis,
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'label_font_size',
                                  'label_color',
                                  'legend_font_size',
                                ]);

  var correlation = arc.visual.api();

  var shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimensions',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measures',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  var constraints = {
    minimums: {
      dimensions_shelf: 2,
      aggregates_shelf: 1,
    },
    maximums: {
      dimensions_shelf: 2,
    },
  };

  correlation.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    previousDataTracker.lib(lib);
    lib.shelf_data(shelf_data)
       .user_settings(user_settings)
       .chart_type_graded(true);
    return correlation;
  };

  correlation.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  correlation.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return correlation;
    }

    saved_settings = settings;
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    lib.restore_basics(settings, switchingVisualType);
    return correlation;
  };

  correlation.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        dimensionMessages = [
          lib.createPopoverMessages('dimensionCorrelation'),
        ],
        measureMessages = [
          lib.createPopoverMessages(['aggregateAcross', 'columns and rows.']),
          lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException'),
        ];

     // Create those popovers
     lib.addShelfDescription(constraints, {
       x_shelf: xMessage,
       y_shelf: yMessage,
       dimensions_shelf: dimensionMessages,
       aggregates_shelf: measureMessages,
     });

    return correlation;
  };

  correlation.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  correlation.viz_setDataResult = function(result) {
    data_result = result;
    return correlation;
  };

  function axisTicksFontSize() {
    return lib.getAxisTicksFontSize() || 11;
  }

  function baseSetup(config, rows) {
    let {
      columns,
    } = config;

    const cols = lib.shelf_columns(columns, 'dimensions_shelf');
    config.dimYCol = cols[0];
    config.dimXCol = cols[1];
    config.dimYIdx = config.dimYCol.colindex();
    config.dimXIdx = config.dimXCol.colindex();

    const agg_idx = data_result.agg_start_idx();
    config.agg_getter = function(row) {
      return row[agg_idx];
    };

    rows.forEach(function(row) {
      row[agg_idx] = Number(row[agg_idx]);
    });

    let dimYLabels = arc.misc.arrayUnique(rows, function(row) {
      return row[config.dimYIdx];
    });

    let yFormatter = config.dimYCol.coldisplayformat();
    if (yFormatter === undefined) {
      yFormatter = function(v) {
        return v + '';
      };
    }

    dimYLabels = dimYLabels.map(function(v) {
      return yFormatter(v);
    });

    let dimXLabels = arc.misc.arrayUnique(rows, function(row) {
      return row[config.dimXIdx];
    });

    let xFormatter = config.dimXCol.coldisplayformat();
    if (xFormatter === undefined) {
      xFormatter = function(v) {
        return v + '';
      };
    }

    dimXLabels = dimXLabels.map(function(v) {
      return xFormatter(v);
    });

    if (showYAxis.isChecked) {
      config.maxYDimLen = Math.min(maxYLegendWidth.value() || 400,
          lib.dim_axis_max_textlen(dimYLabels, 1000, axisTicksFontSize()));
    } else {
      config.maxYDimLen = 0;
    }

    if (showXAxis.isChecked) {
      config.maxXDimLen = Math.min(maxXLegendHeight.value() || 400,
          lib.dim_axis_max_textlen(dimXLabels, 1000, axisTicksFontSize()));
    } else {
      config.maxXDimLen = 0;
    }

    config.cell_margin_left = config.maxYDimLen + 5;
    const colorDomain = arc.misc.getRange(rows, config.agg_getter);
    config.color = lib.aggScale(colorDomain);
  }

  correlation.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    let rows = data_result.rows();
    if (!rows || rows.length === 0) {
      return correlation;
    }

    previousDataTracker.enableOnRowCount(rows.length);

    lib.unselectOnClick();

    var config = {
      columns: data_result.columns(),
    };

    baseSetup(config, rows);

    let trellis = arc.visual.trellis()
                     .x_shelf_name('x_shelf')
                     .y_shelf_name('y_shelf')
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select('#' + viz_div))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       correlation.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return correlation;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      correlation.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');

    calculateTrellisCellParams(config, viz_div);

    config.trellis.trellisCellsCb(function(cell_sel) {
      const cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, trellisCellData[cell_div], cell_div);
    });

    return correlation;
  };

  correlation.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return correlation;
  };

  correlation.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return correlation;
  };

  function calculateTrellisCellParams(config, viz_div) {
    trellisCellData = {};
    trellisCellParams = {
      xElementCount: 0,
      yElementCount: 0,
    };

    config.trellis.draw_visuals(function(cell_sel, xy_rows,
        _unused_xmeasure, _unused_ymeasure, cellIdx) {
      var cell_div = cell_sel.attr('id');
      calculateLabelsAndData(config, xy_rows, cell_div, cellIdx);
    });

    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      viz_div,
      trellis: config.trellis,
      includeLegend: false,
    });

    let maxWidthX = Math.max((elem_w - trellisCellMargin.right - trellisCellMargin.left -
      config.maxYDimLen) / trellisCellParams.xElementCount, minimumSize.value());
    let maxWidthY = Math.max((elem_h - trellisCellMargin.top - trellisCellMargin.bottom -
      config.maxXDimLen) / trellisCellParams.yElementCount, minimumSize.value());
    trellisCellParams.rectDim = Math.min(maxWidthX, maxWidthY);

    Object.keys(trellisCellData).forEach(function(cell_div) {
      finalizeCalculations(config, cell_div);
    });
  }

  function calculateLabelsAndData(config, rows, cell_div, cellIdx) {
    let {
      dimXIdx,
      dimYIdx,
      agg_getter,
      maxXDimLen,
    } = config;

    let xlabels_dict = {},
        xlabels = [],
        xlabel_idx = 0,
        xlabels_idx_to_name = {},
        ylabels_dict = {},
        ylabels = [],
        ylabel_idx = 0,
        ylabels_idx_to_name = {},
        data = [];

    function process(d, l, k, v) {
      if (d[k] !== undefined) {
        return false;
      }

      d[k] = v;
      l.push(k);
      return true;
    }

    rows.forEach(function(d) {
      if (process(ylabels_dict, ylabels, d[dimYIdx], ylabel_idx)) {
        ylabels_idx_to_name[ylabel_idx] = d[dimYIdx];
        ++ylabel_idx;
      }

      if (process(xlabels_dict, xlabels, d[dimXIdx], xlabel_idx)) {
        xlabels_idx_to_name[xlabel_idx] = d[dimXIdx];
        ++xlabel_idx;
      }

      data.push({
        y: ylabels_dict[d[dimYIdx]],
        x: xlabels_dict[d[dimXIdx]],
        v: agg_getter(d),
        row: d,
      });
    });

    trellisCellData[cell_div] = {
      xlabels_dict,
      xlabels,
      xlabels_idx_to_name,
      ylabels_dict,
      ylabels,
      ylabels_idx_to_name,
      xdim_len: config.maxXDimLen,
      data,
      cellIdx,
    };

    trellisCellParams.xElementCount = Math.max(trellisCellParams.xElementCount,
        xlabel_idx);

    trellisCellParams.yElementCount = Math.max(trellisCellParams.yElementCount,
        ylabel_idx);
  }

  function finalizeCalculations(config, cell_div) {
    let {
      data,
      xdim_len,
      xlabels,
      ylabels,
      xlabels_idx_to_name,
      ylabels_idx_to_name,
    } = trellisCellData[cell_div];

    let {
      rectDim,
    } = trellisCellParams;

    let c = trellisCellData[cell_div];

    c.margin = {
      top: 5,
      left: config.cell_margin_left,
      right: 5,
      bottom: xdim_len + 5,
    };

    c.width = Math.max(rectDim * xlabels.length + c.margin.left + c.margin.right,
                 $('#' + cell_div).width());
    c.height = rectDim * ylabels.length + c.margin.top + c.margin.bottom;
    c.corWidth = c.width - c.margin.left - c.margin.right;
    c.corHeight = c.height - c.margin.top - c.margin.bottom;

    const x_scale =
        d3.scale.ordinal()
          .domain(xlabels)
          .rangeBands([0, c.corWidth]);

    const y_scale =
        d3.scale.ordinal()
          .domain(ylabels)
          .rangeBands([0, c.corHeight]);

    let computedData = data.map(function(d) {
      return {
        x: x_scale(xlabels_idx_to_name[d.x]),
        y: y_scale(ylabels_idx_to_name[d.y]),
        color: config.color(d.v),
        row: d.row,
      };
    });

    c.computedData = computedData;
    c.x_scale = x_scale;
    c.y_scale = y_scale;
    c.data = undefined;
  }

  function viz_drawVisualWork(config, cellConfig, cell_div) {
    setupVisual(config, cellConfig, cell_div);
    drawVisual(config, cellConfig, cell_div);
    return correlation;
  }

  function drawCanvasAxes(config, cellConfig, canvasConfig) {
    let {
      width,
      height,
      margin,
      x_scale,
      y_scale,
      corHeight,
    } = cellConfig;

    let {
      canvas,
      hidden,
      highlight,
      colorMap,
    } = canvasConfig;

    let {
      rectDim,
    } = trellisCellParams;

    if (highlight) {
      return;
    }

    const context = canvas.node().getContext('2d');
    context.fillStyle = '#666';

    let o = context.globalAlpha;
    let padding = 4;

    context.globalAlpha = 1;
    const yFormatter = config.dimYCol.coldisplayformat();
    const xFormatter = config.dimXCol.coldisplayformat();
    if (showYAxis.isChecked) {
      y_scale.domain().forEach(function(y) {
        if (hidden) {
          let color = colorFetcher();
          let d = [];
          d[config.dimYCol.colindex()] = y;
          colorMap[color] = {
            row: d,
          };

          context.fillStyle = color;
          context.fillRect(-margin.left, y_scale(y), margin.left, rectDim);
          return;
        }

        let y_pos = y_scale(y);
        y = yFormatter(y);

        let w = context.measureText(y).width;
        if (w < margin.left - padding) {
          context.fillText(y, -padding, y_pos);
        } else {
          while (w > 1) {
            y = y.substring(0, y.length / 2);
            w = context.measureText(y).width;
            if (w < margin.left - padding) {
              context.fillText(y, -padding, y_pos);
              break;
            }
          }
        }
      });
    }

    if (!hidden) {
      context.rotate(-Math.PI / 2);
    }

    if (showXAxis.isChecked) {
      x_scale.domain().forEach(function(x) {
        if (hidden) {
          let color = colorFetcher();
          let d = [];
          d[config.dimXCol.colindex()] = x;
          colorMap[color] = {
            row: d,
          };
          context.fillStyle = color;
          context.fillRect(x_scale(x), corHeight, rectDim, margin.bottom);
          return;
        }

        let x_pos = x_scale(x);
        x = xFormatter(x);

        let w = context.measureText(x).width;
        if (w < margin.bottom - padding) {
          context.fillText(x, -corHeight - padding, x_pos);
        } else {
          while (w > 1) {
            x = x.substring(0, x.length / 2);
            w = context.measureText(x).width;
            if (w < margin.bottom - padding) {
              context.fillText(x, -corHeight - padding, x_pos);
              break;
            }
          }
        }

        context.fillText(x, -corHeight - padding, x_pos);
      });
    }

    if (!hidden) {
      context.rotate(Math.PI / 2);
    }

    context.globalAlpha = o;

    canvasConfig.colorMap = colorMap;
    return;
  }

  function drawCanvas(config, cellConfig, canvasConfig) {
    let {
      width,
      height,
      corWidth,
      corHeight,
      margin,
      computedData,
    } = cellConfig;

    let {
      canvas,
      hidden,
      highlight,
    } = canvasConfig;

    let {
      rectDim,
    } = trellisCellParams;

    const colorMap = {},
          context = canvas.node().getContext('2d'),
          opacity = lib.colorOpacity();

    if (!hidden && !highlight) {
      context.globalAlpha = opacity;
      context.lineWidth = '2';
      context.strokeStyle = '#eee';
    }

    if (highlight) {
      context.lineWidth = 1;
      context.strokeStyle = arc.canvas.highlightStrokeStyle;
    }

    let font = lib.getVisualFont();
    let fontSize = axisTicksFontSize();
    context.font = `300 ${fontSize}px ${font}`;
    context.textBaseline = 'top';
    context.textAlign = 'right';

    if (!highlight) {
      drawRows(computedData);
    }

    // We separate out the drawRows function so that we can use the same
    // function for highlighting particular rows. We simply pass in an array
    // of rows to highlight, and draw those rows in the highlight canvas.
    // See usage of highlightDrawRows to see more.
    function drawRows(rows) {
      if (highlight) {
        context.clearRect(-1, -1, corWidth + 2, corHeight + 2);
      } else {
        context.clearRect(-margin.left, -margin.top,
            corWidth + margin.left + margin.right,
            corHeight + margin.top + margin.bottom);
      }

      rows.forEach(function(d, i) {
        let color;

        if (hidden) {
          color = colorFetcher();
          colorMap[color] = d;
        } else {
          color = d.color;
        }

        if (highlight) {
          context.beginPath();
          context.rect(d.x, d.y, rectDim, rectDim);
          context.stroke();
          return;
        }

        context.fillStyle = color;
        context.fillRect(d.x, d.y, rectDim, rectDim);

        if (!hidden && !highlight) {
          context.beginPath();
          context.rect(d.x, d.y, rectDim, rectDim);
          context.stroke();
        }
      });
    }

    canvasConfig.drawRows = drawRows;
    canvasConfig.colorMap = colorMap;
  }

  function setupVisual(config, cellConfig, cell_div) {
    arc.canvas.createContainer(d3.select('#' + cell_div), cellConfig);
    arc.canvas.createAllCanvas(cellConfig);
  }

  function drawVisual(config, cellConfig, cell_div) {
    let startTransitionGenerator = function(d) {
      return {
        x: d.x,
        y: d.y,
        color: '#fff',
        row: d.row,
      };
    };

    let xcol = config.dimXCol.colindex(),
        ycol = config.dimYCol.colindex();

    let dataToGroup = function(d) {
      let row = d.row;
      return [row[xcol], row[ycol]];
    };

    let columns = data_result.columns();

    function text_fn(d) {
      return lib.text_from_columns(d.row, columns);
    }

    function params_fn(d) {
      return lib.params_from_columns(d.row, columns);
    }

    let drawConfig = {
      drawCanvas,
      drawCanvasAxes,
      text_fn,
      params_fn,
      startTransitionGenerator,
      dataToGroup,
      previousDataTracker,
      lib,
    };

    arc.transition.drawTransition(config, cellConfig, drawConfig);
  }

  return correlation;
};

arc.visual.plugins.correlation_extension = function() {
  var ext = function() { return 'arc.visual.plugins.correlation_extension'; };

  ext.viz_name = function() {
    return 'corelation';
  };

  ext.viz_tooltip = function() {
    return 'correlation heatmap';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_correlation-heatmap.png?v=1';
  };

  ext.viz = arc.visual.plugins.correlation;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.correlation_extension());

}());
