window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

arc.visual.create = function() {
  let v = function() { return 'arc.visual.create'; };

  let queryText,
      _report,

      origConfig;

  v.getReport = function() {
    return _report;
  };

  v.getOrigConfig = function() {
    return origConfig;
  };

  v.makeNewVisual = function(origText, config) {
    origConfig = config;
    let {
      report,
      datasets,
      beforedraw_cb,

      highlight_cb,
      done_cb,

      // Add explain popover
      $searchButtonContainer,
      $explore,

      // Determines if we mark report as needing to be saved
      // searchVisualLib is needed for passing in parameters and information on
      //                 the dashboard container_id
      //                 Only present in results for search visual types
      // isSearchShowme is needed to determine if visual builder should get updated
      //                Is true for all visuals created from search
      searchVisualLib,
      isSearchShowme,

      // hide/show add to dashboard
      $addToDashboard,

      // optional computed score config. If this exists, we immediately
      // draw the visual
      finalScoreConfig,

      // input is optional. If specified, it is used for highlighting
      input,
      displayedQuery,

      // report, (optional report)
      // shelves_dom_id, (optional shelves_dom_id, used if report is not specified)
      // chart_div, (optional chart_div, used if report is not specified)
    } = config;

    const text = origText.trim();
    if (text.length === 0) {
      return;
    }

    queryText = text;

    v.finalDraw = function(scoreConfig) {
      highlight({
        elements: scoreConfig.elements,
        highlights: scoreConfig.highlights,
        showUnknowns: true,
      });

      if (scoreConfig.score === 0) {
        console.log('Search score is too low to execute');

        if (beforedraw_cb) {
          beforedraw_cb(scoreConfig);
        }

        return;
      }

      config.elements = scoreConfig.elements;
      config.dataset = scoreConfig.dataset;
      config.showmeVisualConfig = scoreConfig.showmeVisualConfig;
      config.$searchButtonContainer = $searchButtonContainer;
      config.$explore = $explore;
      config.searchVisualLib = searchVisualLib;
      config.isSearchShowme = isSearchShowme;
      config.displayedQuery = displayedQuery;

      const hasPerm = arc.misc.hasPerm('ds_appedit', config.dataset.dataset_perms());
      if ($addToDashboard) {
        const permMessage = 'Cannot Add to Dashboard: No manage dashboard permission for dataset';
        $addToDashboard.toggleClass('disabled', !hasPerm)
                       .attr('title', hasPerm ? 'Add to Dashboard' : permMessage);
      }

      displayVisualWork(config);
    };

    if (finalScoreConfig) {
      v.finalDraw(finalScoreConfig);
      return v;
    }

    function highlight(highlightConfig) {
      if (input) {
        highlightConfig.redrawTextCb = function(newText) {
          origConfig.report = v.getReport();
          v.makeNewVisual(newText, origConfig);
        };

        input.searchHighlight().displaySearchElements(origText, highlightConfig);
      }

      if (highlight_cb) {
        highlight_cb(highlightConfig);
      }
    }

    /*
     * First, score and highlight from the metadata without searching
     * through the data. We do this so we can pick the datasets to
     * run the queries on, and so we can do some early highlighting.
     */
    arc.search.score({
      text,
      datasets,
      validateFilters: false,
      cb: function(scoredConfigs) {
        const highestScoreConfig = scoredConfigs[0];

        /*
         * Do not highlight unknowns yet because we have not searched
         * through the data
         */
        highlight({
          elements: highestScoreConfig.elements,
          highlights: highestScoreConfig.highlights,
          showUnknowns: false,
        });

        /*
         * Now, pick the dataset to run the full search on.
         * If we have datasets that score > 0, then pick the
         * highest scoring datasets. Otherwise, run the query
         * on all of them.
         */
        let finalDatasets;
        if (highestScoreConfig.score > 0) {
          let highConfigs = scoredConfigs.filter(function(c) {
            return c.score === highestScoreConfig.score;
          });

          finalDatasets = highConfigs.map(function(c) {
            return c.dataset;
          });
        } else {
          finalDatasets = datasets;
        }

        /*
         * Next, score the datasets with all unknown values resolved
         * via the data.
         */
        arc.search.score({
          text,
          datasets: finalDatasets,
          validateFilters: true,
          cb: function(scoredConfigs) {
            const highestScoreConfig = scoredConfigs[0];

            // Store this search in search history
            arc.search.displayedQuery()
                             .search_text(text)
                             .search_data(highestScoreConfig)
                             .dataset_id(highestScoreConfig.dataset.dataset_id())
                             .dataset_name(highestScoreConfig.dataset.dataset_name())
                             .store();
            v.finalDraw(highestScoreConfig);
          },
        });
      },
    });

    return v;
  };

  // Converts a set of elements into arc.report.shelf_element objects
  // separated into dimensions, aggregates and filters.
  function elementsToShelfData(config) {
    let {
      dataset,
      elements,
    } = config;

    let shelfData = {
      dimensions: [],
      aggregates: [],
      filters: [],
    };

    elements.forEach(function(e) {
      if (e.column) {
        elementToShelfData(config, e, shelfData);
      }
    });

    return shelfData;
  }

  function elementToShelfData(config, element, shelfData) {
    let {
      dataset,
    } = config;

    let {
      // one of 'dimension', 'aggregate', 'filter' and 'unknown'
      type,

      // dataset column being referenced by this shelf element
      column,

      // In the case of a simple aggregation, this is the aggregation function
      aggregation,

      // This is the custom expression for the element
      expr,

      // contains 'ascending' and 'limit' members
      order,
    } = element;

    if (type === 'unknown') {
      return;
    }

    let rawColname = column.display_name();

    var se = arc.report.shelf.element()
                .dataset(dataset)
                .dataconnection_type(dataset.dataconnection_type());

    // Either filter element is column name or handle as custom expression
    se.dataset_colname(rawColname)
      .dataset_coltype(column.columnType())
      .expr_hasagg(column.attr('exprhasagg'));

    if (element.expr) {
      se.custom_expr(element.expr);
    } else {
      if (type === 'aggregate') {
        if (aggregation) {
          se.fn_name(aggregation);
        } else {
          se.custom_expr(expr);
        }
      }

      if (type === 'filter') {
        se.filter_column(rawColname);
        se.custom_expr(expr);
      }
    }

    if (order) {
      let o = arc.report.shelf.order()
                 .ascending(order.ascending);

      se.order(o)
        .order_limit(order.limit + '');

      if (element.order_limit_granular) {
        se.order_limit_granular(true);
      }
    }

    if (column.attr('displayformat')) {
      se.display_format(column.attr('displayformat'));
    }

    let d = {};
    se.save(d);

    switch (type) {
    case 'dimension':
      shelfData.dimensions.push(d);
      break;

    case 'aggregate':
      shelfData.aggregates.push(d);
      break;

    case 'filter':
      shelfData.filters.push(d);
      break;
    }
  }

  function newReport(config) {
    let {
      dataset,
      shelves_dom_id,
      chart_div,
    } = config;

    let viz = arc.visual.plugins.table();
    const dsSegment = arc.report.dataset_segments()
                                .dataset(dataset)
                                .parse_segments(dataset.segments());

    let report = arc.reporttracker.report()
                    .dataset(dataset)
                    .dataset_segments(dsSegment)
                    .shelves_dom_id(shelves_dom_id)
                    .chart_div(chart_div);
    let vizDriver = arc.report.driver(viz, false /* drawSettings */)
                       .report(report)
                       .dataset(dataset)
                       .shelves_dom_id(shelves_dom_id);

    report.viz(vizDriver);
    vizDriver.setup();

    return report;
  }

  function displayVisualWork(config) {
    let {
      dataset,
      elements,
      showmeVisualConfig,
      report,
      $searchButtonContainer,
      searchVisualLib,
      isSearchShowme,
      displayedQuery,
    } = config;

    if (config.beforedraw_cb) {
      config.beforedraw_cb(config);
    }

    let dimCount = 0,
        aggCount = 0;

    elements.forEach(function(e) {
      if (e.type === 'dimension') {
        ++dimCount;
        return;
      }

      if (e.type === 'aggregate') {
        ++aggCount;
        return;
      }
    });

    elements = elements.filter(function(e) {
      return ((e.type !== 'unknown') && (e.type !== 'control'));
    });

    if (elements.length === 0) {
      console.log('no elements');
      return;
    }

    //TODO: eliminate this call.
    if (report) {
      report.viz().undraw();
    }

    if (report === undefined) {
      report = newReport(config);
    }

    _report = report;

    let vizDriver = report.viz();
    let viz = vizDriver.viz();

    // If in dashboard search visual, need to properly set up report properties
    // in order to select correct elements for result visuals
    if (searchVisualLib) {
      report.in_dashboard(searchVisualLib.in_dashboard())
            .dashboard(searchVisualLib.dashboard());
      vizDriver.inModal(searchVisualLib.inModal())
               .dashboard(searchVisualLib.dashboard())
               .params(searchVisualLib.params());
    }

    let settings = viz.viz_getSettings();

    let shelfData = elementsToShelfData(config);

    settings.dimensions_shelf = shelfData.dimensions;
    settings.aggregates_shelf = shelfData.aggregates;
    settings.filters_shelf = shelfData.filters;
    settings.report_title = queryText;

    if (vizDriver.viz_type() !== 'table') {
      report.switchViz('table', settings, false);
      vizDriver = report.viz();
      viz = vizDriver.viz();
    } else {
      viz.viz_setSettings(settings);
    }

    vizDriver.setup();
    if (displayedQuery && displayedQuery.search_data().showmeVisualConfig.vizId &&
      !showmeVisualConfig.vizId) {
      showmeVisualConfig.vizId = displayedQuery.search_data().showmeVisualConfig.vizId;
    } else if (displayedQuery && displayedQuery.search_data().showmeVisualConfig.vizType &&
      !showmeVisualConfig.vizType) {
      showmeVisualConfig.vizType = displayedQuery.search_data().showmeVisualConfig.vizType;
    }

    const $favoriteIcon = $(`#${vizDriver.chart_div()}`)
                            .closest('.search-result')
                            .find('.favorite');
    let showme = arc.showme.visuals({
      isSearchShowme: isSearchShowme,
      vizDriver,
      showmeVisualConfig,
      displayedQuery,
      pickedCb: function clearDisplayQuery(pickedVisual) {
        if (!displayedQuery) {
          return;
        }

        const searchData = displayedQuery.search_data();

        if (searchData.vizId !== pickedVisual.visualData.id) {
          displayedQuery.bookmark_id(null);
          displayedQuery.favorite(null);
          searchData.vizId = pickedVisual.visualData.id;
        }

        const isFavorite = displayedQuery.getFavorite();
        $favoriteIcon.toggleClass('fa-bookmark', !!isFavorite);
        $favoriteIcon.toggleClass('fa-bookmark-o', !isFavorite);
      },
    });

    const elementOrdering = [];
    config.elements.forEach(function(e) {
      if ((e.type === 'dimension') || (e.type === 'aggregate')) {
        elementOrdering.push(e.type);
      }
    });

    let pickedVisual = showme.pickVisual({
      elementOrdering,
      showmeVisualConfig,
    });

    if (!pickedVisual) {
      return undefined;
    } else if (displayedQuery) {
      const searchData = displayedQuery.search_data();
      searchData.vizId = pickedVisual.visualData.id;
    }

    pickedVisual.vizDriver
                .dataset(dataset)
                .shelves_dom_id(report.shelves_dom_id());

    showme.computeSettings(pickedVisual);

    if (displayedQuery && displayedQuery.getFavorite()) {
      $favoriteIcon.addClass('fa-bookmark');
      $favoriteIcon.removeClass('fa-bookmark-o');
    }

    pickedVisual.viz.viz_lib(vizDriver);

    arc.misc.displayVisualType(pickedVisual.visualData.vizType);
    report.switchViz(pickedVisual.visualData.vizType,
        pickedVisual.visualData.settings, true);

    if (searchVisualLib) {
      report.viz().page_not_need_save(false);
    } else {
      report.viz().needs_save(true);
    }

    /*
     * In report.switchViz(), there is this bit of code:
     * if (sql_output) {
     *   viz.sqlQuery(sql_query);
     *   viz.sqlOutput(sql_output);
     *   viz.dsreq(dsreq);
     * }
     *
     * That code refreshes the report if sqlOutput is set.
     *
     * If sqlOutput is not set, though, refresh the visual
     *   explicitly below.
     */
    if (!report.viz().sqlOutput()) {
      report.viz().refresh(/* force_refraw */ true);
    }

    report.saved_data(JSON.stringify({
      report_type: pickedVisual.visualData.vizType,
      report_data: pickedVisual.visualData.settings,
    }));

    if (config.$explore && config.$explore.length) {
      config.$explore.removeClass('arc-invisible');
      config.$explore.offOn('click.showexplore', function showExplore(e) {
        e.stopPropagation();
        const showmeVisual = arc.showme.visuals({
          isSearchShowme: isSearchShowme,
          vizDriver: report.viz(),
          showmeVisualConfig,
          displayedQuery,
          pickedCb: function clearDisplayQuery(pickedVisual) {
            if (!displayedQuery) {
              return;
            }

            const searchData = displayedQuery.search_data();

            if (searchData.vizId !== pickedVisual.visualData.id) {
              displayedQuery.bookmark_id(null);
              displayedQuery.favorite(null);
              searchData.vizId = pickedVisual.visualData.id;
            }

            const isFavorite = displayedQuery.getFavorite();
            $favoriteIcon.toggleClass('fa-bookmark', !!isFavorite);
            $favoriteIcon.toggleClass('fa-bookmark-o', !isFavorite);
          },
        });

        showmeVisual.setup();
      });
    }

    if ($searchButtonContainer && $searchButtonContainer.length) {
      const $viewData = $searchButtonContainer.find('.search-view-data');
      $viewData.offOn('click.viewdata', function() {
        report.viz().showData();
      });

      const popoverTemplate = '<tr><td class="label">%NAME%:</td><td>%VALUE%</td></tr>';
      const $newPopover = $searchButtonContainer.find('.pop-search-explain');
      $newPopover.removeClass('arc-invisible');

      $newPopover.popover({
        // selector: '.pop-search-explain',
        trigger: 'manual',
        html: true,
        placement: 'left',
        delay: { show: '500' },
        title: 'Question breakdown',
        container: 'body',
        sanitize: false,
        content: function showSearchExplain() {
          // Some shelves are duplicated. Let's dedup them here
          const dedupedShelves = {};
          let currShelf = '';
          function iterateShelves(shelf) {
            currShelf = dedupedShelves[shelf.displayName];
            if (!currShelf) {
              currShelf = {};
              dedupedShelves[shelf.displayName] = currShelf;
            }

            pickedVisual.visualData.settings[shelf.shelfId].forEach(addShelfElement);
          }

          function addShelfElement(element) {
            currShelf[element.expression_for_trigger] = true;
          }

          pickedVisual.visualData.shelves.forEach(iterateShelves);

          let rows = arc.misc.safe_substitute(popoverTemplate, {
            NAME: 'Dataset',
            VALUE: `${config.dataset.dataset_name()}`,
          });

          rows += arc.misc.safe_substitute(popoverTemplate, {
            NAME: 'Connection',
            VALUE: `${config.dataset.dataconnection_name()}`,
          });

          if (_report.viz().sqlOutput() &&
              _report.viz().sqlOutput().info) {
            const { info } = _report.viz().sqlOutput();
            const queryPrefix = 'Query:';
            let sql;
            info.forEach(function(i) {
              if (i.startsWith(queryPrefix)) {
                sql = i.substring(queryPrefix.length);
              }
            });

            if (sql) {
              rows += arc.misc.safe_substitute(popoverTemplate, {
                NAME: 'SQL',
                VALUE: sql,
              });
            }
          }

          rows += Object.keys(dedupedShelves).map((shelfName) => {
            const shelfElements = Object.keys(dedupedShelves[shelfName]);
            if (!shelfElements.length) {
              return '';
            }

            return arc.misc.safe_substitute(popoverTemplate, {
              NAME: shelfName,
              VALUE: shelfElements.join(', '),
            });
          }).join('');

          return `<table class="popover-table search-explain-popover">${rows}</table>`;
        },
      }).on('mouseenter', function () {
        $newPopover.popover('show');
        $('.popover').one('mouseleave', function () {
            $newPopover.popover('hide');
        });
      }).on('mouseleave', function () {
        setTimeout(function () {
          if (!$('.popover:hover').length) {
            $newPopover.popover('hide');
          }
        }, 200);
      });
    }

    if (config.done_cb) {
      config.done_cb(vizDriver);
    }

    return v;
  }

  return v;
};

}());
