window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

arc.visual.plugins = arc.visual.plugins || {};

if (arc.frontend.featureFlipper.getValueById('enable_legacy_crosstab')) {
  return;
}

arc.visual.plugins.crosstab = function() {
  let sort_orders_asc = {};
  let saved_settings = {};
  let data_result;
  let trellis;
  let lib;

  let customWidths = [];

  const crosstabExpansionTracker = arc.expansionTracker();

  const colors_enabled_scale = arc.report.user_settings.colors_enabled();
  const showColTotals = arc.visual.setting()
            .id('show_crosstab_totals')
            .tab('Data Summary')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('SHOW_CROSSTAB_TOTALS')
            .add_label('Show column totals')
            .value('false');

  const combineRowMeasureTotals = arc.visual.setting()
            .id('combine_row_measure_totals')
            .tab('Data Summary')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('SEPARATE_ROW_MEASURES_totals')
            .add_label('Combine row totals')
            .info('Valid when there are multiple measures and at least one dimension in the ' +
                      ' Column shelf.')
            .value('false');

  const showRowTotals = arc.visual.setting()
            .id('show_row_totals')
            .tab('Data Summary')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .addDependents([combineRowMeasureTotals])
            .add_name('SHOW_ROW_TOTALS')
            .add_label('Show row totals')
            .value('false');

  const showAdjacentDuplicates = arc.visual.setting()
                                  .id('show_adjacent_duplicate_values')
                                  .tab('Marks')
                                  .type(arc.visual.settingTypeEnum.CHECKBOX)
                                  .add_name('SHOW_ADJACENT_DUPLICATE_VALUES')
                                  .add_label('Show adjacent duplicate values')
                                  .value('false')
                                  .valueBeforeRestore('false');

  const aggregateRowStyle = arc.settings.fonts.aggregateRowStyle();
  const aggregateRowWeight = arc.settings.fonts.aggregateRowWeight();

  const fullTableWidth = arc.settings.fonts.fullTableWidth();

  const autoSizeColumn = arc.settings.table.autoSizeColumn();

  const user_settings = arc.visual.defaultSettings({
                             noTrellis: true,
                             noAxes: true,
                         })
                         .addSetting([
                           arc.report.user_settings.display_tooltips_table(),
                           arc.colorpalette.setting(),
                           arc.report.user_settings.color_opacity(),
                           colors_enabled_scale,
                           showAdjacentDuplicates,
                           arc.settings.crosstab.totals(),
                           arc.settings.crosstab.crosstab_total_str(),
                           aggregateRowStyle,
                           aggregateRowWeight,
                           fullTableWidth,
                           autoSizeColumn,
                           arc.settings.table.tableHeaderBackgroundColor(),
                           arc.settings.table.tableHeaderTextColor(),
                           arc.settings.table.tableOddRowsBackgroundColor(),
                           arc.settings.table.tableEvenRowsBackgroundColor(),
                           arc.settings.table.tableOddRowsTextColor(),
                           arc.settings.table.tableEvenRowsTextColor(),
                           arc.settings.table.tableFooterBackgroundColor(),
                           arc.settings.table.tableFooterTextColor(),
                           arc.settings.table.tableBorderColor(),
                           arc.settings.fonts.crosstabTitleFont(),
                           arc.settings.fonts.crosstabTitleFontSize(),
                           arc.settings.fonts.crosstabTitleFontWeight(),
                           arc.report.user_settings.table_cell_padding(),
                           arc.settings.table.tableBorderWidth(),
                           showColTotals,
                           showRowTotals,
                           combineRowMeasureTotals,
                           arc.settings.fonts.totalsColumnFontWeight(),
                           arc.settings.fonts.totalsRowFontWeight(),
                           arc.settings.table.settingCrosstabHeaderAlignment(),
                           arc.settings.table.settingCrosstabContentAlignment(),
                         ]);

  user_settings.find_by_id('show_tooltip').hide();

  const crosstab = arc.visual.api();

  function aggregateRowClass() {
    const v = `${aggregateRowStyle.valueToClass()} ${
            aggregateRowWeight.valueToClass()}`;

    return v.trim();
  }

  const shelf_data = [
    {
      name: 'x_dimensions_shelf',
      display_name: 'Column',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_dimensions_shelf',
      display_name: 'Row',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measures',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  const constraints = {
    minimum_dims: 1,
    minimums: {
      aggregates_shelf: 1,
    }
  };

  crosstab.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    crosstabExpansionTracker.viz_lib(lib);
    lib.chart_type_graded(true)
       .shelf_data(shelf_data)
       .user_settings(user_settings);
    return crosstab;
  };

  crosstab.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    saved_settings.crosstabCustomWidths = customWidths;
    saved_settings = settings;

    return settings;
  };

  crosstab.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return crosstab;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('crosstabCustomWidths' in saved_settings) {
      customWidths = saved_settings.crosstabCustomWidths;
    }

    return crosstab;
  };

  crosstab.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup()
       .settings_color_disabled_show();

    // Define messages to be included in shelves
    const xDimensionMessages = [
      lib.createPopoverMessages(['aggregateCrosstab', 'columns.']),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const yDimensionMessages = [
      lib.createPopoverMessages(['aggregateCrosstab', 'rows.']),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const aggregateMessages = [
      lib.createPopoverMessages(['aggregateAcross', 'columns and rows.']),
      lib.createPopoverMessages('acceptsMultiple')
    ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_dimensions_shelf: xDimensionMessages,
      y_dimensions_shelf: yDimensionMessages,
      aggregates_shelf: aggregateMessages,
    });

    return crosstab;
  };

  crosstab.viz_getDataRequest = function() {
    const dataRequest = lib.getCheckedDataRequest(constraints);

    crosstabExpansionTracker.onGetDataRequest(dataRequest);

    return dataRequest;
  };

  let lastSpecifiedSorting = [];

  /*
   * Set the data this object should use to draw out the visual
   */
  crosstab.viz_setDataResult = function(result) {
    data_result = result;

    crosstabExpansionTracker.onSetDataResult(data_result);

    return crosstab;
  };

  crosstab.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    const rows = data_result.rows();

    if (!rows) {
      return crosstab;
    }

    lib.unselectOnClick();

    d3.select(`#${viz_div}`)
        .append('div')
        .attr('class', 'crosstab-table-container');
    crosstab.tabulate(rows, `#${viz_div} .crosstab-table-container`, viz_div);

    const $e = $(`#${viz_div}`);
    $e.off('click.crosstabexpand')
      .on('click.crosstabexpand', '.crosstab-expand', function(e) {
        e.preventDefault();
        e.stopPropagation();
        const rowIndex = $(e.target)
          .closest('.slick-cell')
          .data('parent-row-idx');
        crosstabExpansionTracker.expand(this, rowIndex);
      });

    $e.off('click.crosstabcollapse')
      .on('click.crosstabcollapse', '.crosstab-collapse', function(e) {
        e.preventDefault();
        e.stopPropagation();
        crosstabExpansionTracker.collapse(this);
      });

    return crosstab;
  };

  function computeColorsEnabled(config, rows) {
    const colors_enabled = [];
    const { agg_start } = config;
    const { agg_end } = config;

    let i;

    // Set 'colors_enabled' to indicate where aggregates should be colored.
    // First, set 'color_enabled' based on user settings.
    for (i = agg_start; i <= agg_end; ++i) {
      colors_enabled[i] = (colors_enabled_scale.isChecked);
      if (!colors_enabled[i]) {
        const shelfElement = config.aggShelves[i - agg_start];
        colors_enabled[i] = shelfElement && shelfElement.displayBackgroundColor();
      }
    }

    if (colors_enabled_scale.isChecked) {
      // Now, walk the rows, and set colors_enabled to false for any
      // aggregate that has non-numeric values
      rows.forEach((row) => {
        for (i = agg_start; i <= agg_end; ++i) {
          if (colors_enabled[i]) {
            if (Number.isNaN(row[i]) && row[i] !== '' && row[i] !== 'NULL' && row[i] !== 'null') {
              colors_enabled[i] = false;
            }
          }
        }
      });
    }

    return colors_enabled;
  }

  function theadStyleFetch() {
    const theadStyle = [];

    const headerTextColor = user_settings.find_by_id('table_header_text_color').value();
    if (headerTextColor) {
      theadStyle.push(`color:${headerTextColor};`);
    }

    {
      const font = lib.getCrosstabTitleFont();
      if (font) {
        theadStyle.push(`font-family:${font};`);
      }
    }

    {
      const fontSize = lib.getCrosstabTitleFontSize();
      if (fontSize) {
        theadStyle.push(`font-size:${fontSize}px;`);
      }
    }

    {
      const fontWeight = lib.getTotalsFontWeight('crosstab_title_font_weight');
      if (fontWeight) {
        theadStyle.push(`font-weight:${fontWeight};`);
      }
    }

    return theadStyle;
  }

  function tfootStyleFetch() {
    const tfootStyle = [];

    const footerBgColor = user_settings.find_by_id('table_footer_background_color').value();
    if (footerBgColor) {
      tfootStyle.push(`background:${footerBgColor};`);
    }

    const footerTextColor = user_settings.find_by_id('table_footer_text_color').value();
    if (footerTextColor) {
      tfootStyle.push(`color:${footerTextColor}`);
    }

    return tfootStyle;
  }

  function setVizTableStyles() {
    const chartId = lib.chart_div();

    const headerData = `#${chartId} .viz-tr-header, #${chartId} .viz-td-header`;
    const slickHeader = `#${chartId} .slick-header`;
    const slickHeaderColumns = `#${chartId} .slick-header .slick-header-column`;
    const slickFooter = `#${chartId} .slick-footerrow-column`;
    const slickTable = `#${chartId} .slick-viewport .grid-canvas`;
    const slickCells = `#${chartId} .slick-cell`;
    const slickTotalColumn = `#${chartId} .slick-viewport .grid-canvas .slick-row > .totals-row-cell`;
    // we have our own header row, and we hide the slickgrid built-in header
    const slickVizHeaderCells = `#${chartId} .slick-row.viz-tr-header .slick-cell`;
    const slickContentWithoutHeader = `#${chartId} .slick-row:not(.viz-tr-header) .slick-cell`;

    const allStyles = [];

    allStyles.push(`${slickHeaderColumns} {height:5px;}`);

    // hide the header row
    if (lib.inModal() || arc.globals.is_edit) {
      allStyles.push(`${slickHeaderColumns} ` +
          '{ background-color: #eee; border-right: solid 2px #888; }');
    } else {
      allStyles.push(`${slickHeader} {display:none;}`);
    }

    {
      const font = lib.getVisualFont();
      if (font) {
        allStyles.push(`${slickTable} {font-family: ${font};}`);
      }
    }

    {
      const fontSize = lib.getVisualFontSize();
      if (fontSize) {
        allStyles.push(`${slickTable} {font-size:${fontSize}px;}`);
      }
    }

    {
      const theadStyle = theadStyleFetch();
      if (theadStyle.length > 0) {
        allStyles.push(`${headerData} {`);
        theadStyle.forEach((s) => {
          allStyles.push(s);
        });

        allStyles.push('}');
      }
    }

    const headerBgColor = user_settings.find_by_id('table_header_background_color').value();
    if (headerBgColor) {
      allStyles.push(`${slickVizHeaderCells} {background-color:${headerBgColor}!important;}`);
    }

    const crosstabHeaderAlignment = user_settings.find_by_id('crosstab_header_alignment').value();
    if (crosstabHeaderAlignment) {
      allStyles.push(`${slickVizHeaderCells} {text-align:${crosstabHeaderAlignment}!important;}`);
    }

    {
      const tfootStyle = tfootStyleFetch();
      if (tfootStyle.length > 0) {
        allStyles.push(`${slickFooter} {`);
        tfootStyle.forEach((s) => {
          allStyles.push(s);
        });

        allStyles.push('}');
      }
    }

    {
      const padding = user_settings.find_by_id('table_cell_padding').value();
      if (padding !== undefined) {
        allStyles.push(`${slickCells} {padding: ${padding}px;}`);
      }
    }

    {
      const trEven = [];
      const bg = user_settings.find_by_id('table_even_rows_background_color').value();
      if (bg) {
        trEven.push(`background:${bg};`);
        trEven.push(`background-color:${bg};`);
      }

      const fg = user_settings.find_by_id('table_even_rows_text_color').value();
      if (fg) {
        trEven.push(`color:${fg};`);
      }

      if (trEven.length > 0) {
        allStyles.push(`${slickTable} .slick-row.odd {`);
        trEven.forEach((s) => {
          allStyles.push(s);
        });

        allStyles.push('}');
      }
    }

    {
      const trOdd = [];
      const bg = user_settings.find_by_id('table_odd_rows_background_color').value();
      if (bg) {
        trOdd.push(`background:${bg};`);
        trOdd.push(`background-color:${bg};`);
      } else {
        allStyles.push(
            `${slickTable} .slick-row.even {background-color:#f9f9f9;background:#f9f9f9}`
        );
      }

      const fg = user_settings.find_by_id('table_odd_rows_text_color').value();
      if (fg) {
        trOdd.push(`color:${fg};`);
      }

      if (trOdd.length > 0) {
        allStyles.push(`${slickTable} .slick-row.even {`);
        trOdd.forEach((s) => {
          allStyles.push(s);
        });

        allStyles.push('}');
      }
    }

    {
      const totalsRowFontWeight = lib.getTotalsFontWeight('totals_row_font_weight');

      if (totalsRowFontWeight) {
        allStyles.push(`${slickTotalColumn} {font-weight:${totalsRowFontWeight};}`);
      }
    }

    {
      const totalsColumnFontWeight = lib.getTotalsFontWeight('totals_column_font_weight');

      if (totalsColumnFontWeight) {
        allStyles.push(`${slickFooter} {font-weight:${totalsColumnFontWeight};}`);
      }
    }

    {
      const totalsColumnFont = lib.getCrosstabTitleFont();

      if (totalsColumnFont) {
        allStyles.push(`${slickTotalColumn} {font-family:${totalsColumnFont};}`);
      }
    }

    {
      const crosstabContentAlignment = user_settings.find_by_id('crosstab_content_alignment').value();
      if (crosstabContentAlignment !== 'default') {
        allStyles.push(`${slickContentWithoutHeader}, ${slickFooter} {text-align:${crosstabContentAlignment}!important;}`);
      }
    }

    {
      const borderColor =
          user_settings.find_by_id('table_border_color').value();
      const borderWidth =
          user_settings.find_by_id('table_border_width').value();

      allStyles.push(`${slickCells} {`);
      allStyles.push(`border-right-width:${borderWidth}px;`);
      allStyles.push(`border-bottom-width:${borderWidth}px;`);
      if (borderColor) {
        allStyles.push(`border-color:${borderColor};`);
      }

      allStyles.push('}');
    }

    const $viz = $(`#${chartId}`);
    let $style = $viz.find('style.vizstyle');
    if ($style.length === 0) {
      $viz.append(arc.misc.safe_substitute('<style class="vizstyle"></style>', {}));
      $style = $viz.find('style.vizstyle');
    }

    $style.html(`${allStyles.join('')}`);
  }

  function getAllShelves(config) {
    lib.shelfElementCb('aggregates_shelf', (shelfElement, i) => {
      config.aggShelves[i] = shelfElement;
    });

    lib.shelfElementCb('x_dimensions_shelf', (shelfElement, i) => {
        config.xDimShelves[i] = shelfElement;
    });

    lib.shelfElementCb('y_dimensions_shelf', (shelfElement, i) => {
        config.yDimShelves[i] = shelfElement;
    });
  }

  const setupInitAlignment = ({ agg_cnt: aggCount, yDimCols, dataRows }) => {
    const numberOfDimensions = yDimCols.length;

    Array.from({ length: aggCount }).forEach((_, aggIndex) => {
      dataRows.forEach((row) => {
        const { metadata: { columns, colData } } = row;

        for (
          let columnIndex = numberOfDimensions + aggIndex;
          columnIndex < columns.length;
          columnIndex += aggCount
        ) {
          const column = columns[columnIndex];

          if (column?.cellAttrs && colData[columnIndex] !== '') {
            if (column.cellAttrs.style) {
              column.cellAttrs.style += 'text-align:right;';
            } else {
              column.cellAttrs.style = 'text-align:right;';
            }
          }
        }
      });
    });
  };

  crosstab.tabulate = function(rows, container_div, viz_div) {
    setVizTableStyles();

    if (rows.length === 0) {
      return crosstab;
    }

    const table = arc.misc.trellisTable();
    const columns = data_result.columns();
    const agg_cols = lib.shelf_columns(columns, 'aggregates_shelf');
    const xDimCols = lib.shelf_columns(columns, 'x_dimensions_shelf');
    const yDimCols = lib.shelf_columns(columns, 'y_dimensions_shelf');
    const yDimStart = yDimCols.length && yDimCols[0].colindex();
    const agg_start = agg_cols[0].colindex();
    const agg_end = agg_cols[agg_cols.length - 1].colindex();
    const agg_cnt = agg_cols.length;

    const config = {
      agg_cols,
      agg_start,
      agg_end,
      agg_cnt,
      xDimCols,
      yDimCols,
      aggShelves: [],
      xDimShelves: [],
      yDimShelves: [],
    };

    getAllShelves(config);

    const colors_enabled = computeColorsEnabled(config, rows);

    // ARC-10628: Modify sort order based on order property
    const currentSorting =
        columns.filter((c) => c.order())
               .map((c) => c.colname());

    if (!arc.misc.isIdentical(currentSorting, lastSpecifiedSorting)) {
      // Sorting has changed in the shelves since the last time we ran this
      sort_orders_asc = {};

      columns.forEach((c) => {
        const columnName = c.colname();
        if (c.order()) {
          sort_orders_asc[columnName] = c.order().ascending();
        }
      });

      lastSpecifiedSorting = currentSorting;
    }

    /**
     * If expansion is enabled, it only makes sense if
     * the columns are sorted so here we sort all the columns (asc)
     * if they're not sorted already.
     */
    if (crosstabExpansionTracker.enabled()) {
      columns.forEach((c) => {
        const colName = c.colname();
        if (!(colName in sort_orders_asc)) {
          sort_orders_asc[colName] = true;
        }
      });
    }

    trellis = arc.visual.trellis()
                     .x_shelf_name('x_dimensions_shelf')
                     .y_shelf_name('y_dimensions_shelf')
                     .data_columns(agg_cols)
                     .rows(rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(columns)
                     .table(table)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select(`#${viz_div}`))
                     .ignoreLargeTrellis(true)
                     .showAdjacentDuplicates(showAdjacentDuplicates.value() === 'true')
                     .setup();

    // Collect up aggregate formatters
    const aggColTotals = [];
    const aggRowTotals = [];
    let separateRowTotals = false;
    let grandTotal = [0];
    if (agg_cnt > 1 && lib.shelf_columns(columns, 'x_dimensions_shelf').length > 0) {
      if (!combineRowMeasureTotals.isChecked) {
        separateRowTotals = true;
        grandTotal = Array.from({ length: agg_cnt }, () => 0);
      }
    }

    const agg_formatters = {};
    let agg_i;
    for (agg_i = agg_start; agg_i <= agg_end; ++agg_i) {
      if (!columns[agg_i].coldisplayformat(true)) {
        // override default number format for crosstab
        if (columns[agg_i].isNumeric()) {
          agg_formatters[agg_i] = d3.format(',');
        } else {
          agg_formatters[agg_i] = lib.default_formatter;
        }
      } else {
        agg_formatters[agg_i] = lib.formatter(agg_i);
      }
    }

    // Fetch aggregate values from a row
    const agg_getter = function(d) {
      return d.slice(agg_start, agg_end + 1);
    };

    // Given an array of aggregate values, return an array of formatted
    // aggregate values
    const agg_getter_display = function(al) {
      return al.map((a, i) => lib.formattedDisplay(agg_formatters[agg_start + i], a));
    };

    // Setup the color scales for each aggregate
    const xDimColors = getDimColorArrByIndex(0, config.xDimShelves.length, config.xDimShelves);
    const yDimColors = getDimColorArrByIndex(yDimStart, agg_start, config.yDimShelves);
    function getDimColorArrByIndex(startIdx, endIdx, dimShelves) {
      const dimColorArr = [];
      dimShelves.forEach((shelfElement, i) => {
        if (!shelfElement || !shelfElement.displayBackgroundColor()) {
          return;
        }

        const currColorObj = {};
        const bgColor = shelfElement.backgroundColor();
        const defaultColorDomain = arc.misc.arrayUnique(rows, (row) => row[i + startIdx]);

        if (bgColor) {
          currColorObj.colors = arc.colorpalette.getOrderedColorPalette(bgColor);

          currColorObj.dimensionMappings = bgColor.dimensionMappings;
          currColorObj.firstColor = bgColor.firstColor;
          currColorObj.fixedDimensionColors = bgColor.fixedDimensionColors;
          currColorObj.opacity = bgColor.opacity / 100;
          currColorObj.defaultColorScale = arc.colorpalette.dimScale(
              defaultColorDomain,
              currColorObj.colors,
              {
                fixedDimensionColors: false,
                dimensionMappings: {},
              }
          );
          dimColorArr[i] = currColorObj;
        }
      });

      return dimColorArr;
    }

    function setupDimColors(conf, xColors, yColors) {
      if (!conf.dataRows) {
        return;
      }

      conf.dataRows.forEach((dataRow, rowIdx) => {
        const measureHeaderRowIdx = conf.xDimCols.length === 0 ? 0 : conf.xDimCols.length + 1;
        if (rowIdx === 0 || rowIdx === measureHeaderRowIdx) {
          return;
        }

        const colStartIdx = (conf.yDimCols && conf.yDimCols.length) || 0;
        const metadataCols = dataRow.metadata.columns;
        const { colData } = dataRow.metadata;

        // Apply y_dimensions colors
        for (let i = 0; i < colStartIdx; i++) {
          if (yColors[i]) {
            const metadataCol = metadataCols[i];
            if (metadataCol && metadataCol.cellAttrs) {
              let bg;
              let opacity = 80;

              if (colData[i] !== '') {
                if (yColors[i].dimensionMappings && yColors[i].dimensionMappings[colData[i]]) {
                  bg = yColors[i].dimensionMappings[colData[i]];
                } else {
                  bg = yColors[i].defaultColorScale(colData[i]);
                }

                opacity = yColors[i].opacity;
                if (bg) {
                  const rgba = arc.colorpalette.colorToRGBA(bg, opacity);
                  const fg = arc.colorpalette.colorForBrightness(rgba, opacity);
                  if (metadataCol.cellAttrs.style) {
                    metadataCol.cellAttrs.style += `background-color:${rgba};color:${fg};`;
                  } else {
                    metadataCol.cellAttrs.style = `background-color:${rgba};color:${fg};`;
                  }
                }
              }
            }
          }
        }

        // Apply x_dimensions colors
        const rowXColor = xColors[rowIdx - 1];
        if (rowXColor) {
          for (let i = colStartIdx; i < metadataCols.length; i++) {
            const metadataCol = metadataCols[i];
            if (metadataCol && metadataCol.cellAttrs) {
              let bg;
              let opacity = 80;

              if (colData[i] !== '') {
                if (rowXColor.dimensionMappings && rowXColor.dimensionMappings[colData[i]]) {
                  bg = rowXColor.dimensionMappings[colData[i]];
                } else {
                  bg = rowXColor.defaultColorScale(colData[i]);
                }

                opacity = rowXColor.opacity;

                if (bg) {
                  const rgba = arc.colorpalette.colorToRGBA(bg, opacity);
                  const fg = arc.colorpalette.colorForBrightness(rgba, opacity);
                  if (metadataCol.cellAttrs.style) {
                    metadataCol.cellAttrs.style += `background-color:${rgba};color:${fg};`;
                  } else {
                    metadataCol.cellAttrs.style = `background-color:${rgba};color:${fg};`;
                  }
                }
              }
            }
          }
        }
      });
    }

    const colors = [];
    const opacity = [];
    const getAgg = (data) => Number(data[agg_i]);
    for (agg_i = agg_start; agg_i <= agg_end; ++agg_i) {
      if (colors_enabled[agg_i]) {
        const shelfElement = config.aggShelves[agg_i - agg_start];
        let bgColor;
        if (shelfElement && shelfElement.displayBackgroundColor()) {
          bgColor = shelfElement.backgroundColor();
        }

        let gradedColors;
        let colorSettings;

        if (bgColor) {
          gradedColors = bgColor.colors;
          colorSettings = bgColor.colorSettings;
          opacity[agg_i] = bgColor.opacity / 100;
        } else {
          gradedColors = lib.graded_colors();
          colorSettings = lib.colorSettings();
          opacity[agg_i] = lib.colorOpacity();
        }

        const colorDomain = d3.extent(rows, getAgg);
        const color = arc.colorpalette.aggScale(colorDomain, gradedColors, colorSettings);
        colors[agg_i] = color;
      }
    }

    const aggregateClass = aggregateRowClass();

    rows.forEach((d, d_i) => {
      const colIdx = trellis.x_pos_getter(d);
      const rowIdx = trellis.y_pos_getter(d);
      const aggs = agg_getter(d);
      const isExpandedSubTotalRow = d.slice(0, agg_start)
                                     .some((a) => String(a).includes('crosstab-collapse'));

      while (separateRowTotals && rowIdx >= aggRowTotals.length) {
        aggRowTotals.push([]);
      }

      const { isFullyExpanded } = d;

      agg_getter_display(aggs).forEach((val, i) => {
        if (!isFullyExpanded && aggregateClass) {
          val = `<span class="${aggregateClass}">${val}</span>`;
        }

        table.data().data(rowIdx, colIdx, i, val, {
          rowIdx: d_i,
          agg: aggs[i],
        });

        const aggColIdx = (colIdx * agg_cnt) + i;
        aggColTotals[aggColIdx] = aggColTotals[aggColIdx] || 0;
        if (separateRowTotals) {
          aggRowTotals[rowIdx][i] = aggRowTotals[rowIdx][i] || 0;
        } else {
          aggRowTotals[rowIdx] = aggRowTotals[rowIdx] || 0;
        }

        if (!Number.isNaN(aggs[i])) {
          const numericAgg = Number(aggs[i]);
          if (!isExpandedSubTotalRow) {
            aggColTotals[aggColIdx] += numericAgg;
          }

          if (separateRowTotals) {
            aggRowTotals[rowIdx][i] += numericAgg;
            if (!isExpandedSubTotalRow) {
              grandTotal[i] += numericAgg;
            }
          } else {
            aggRowTotals[rowIdx] += numericAgg;
            if (!isExpandedSubTotalRow) {
              grandTotal[0] += numericAgg;
            }
          }
        }
      });
    });

    const $container_div = $(container_div);

    const dataRows = table.getData(sort_orders_asc, trellis);
    config.dataRows = dataRows;
    let maxCols = 0;

    const slickRows = dataRows.map((dataRow, r_i) => {
      if (maxCols < dataRow.cols.length) {
        maxCols = dataRow.cols.length;
      }

      if (dataRow.attrs === undefined) {
        dataRow.attrs = {};
      }

      const newCols = [];
      const colData = [];
      dataRow.cols.forEach((c, c_i) => {
        newCols.push(c.data);

        const attrs = c.attrs || {};
        attrs['data-col'] = c_i;
        attrs['data-parent-row-idx'] = r_i - 1;
        const { colspan } = attrs;
        if (colspan) {
          colData.push({
            colspan: Number(colspan),
            cellAttrs: attrs,
          });

          for (let i = 1; i < colspan; ++i) {
            newCols.push('');
            colData.push(undefined);
          }
        } else {
          colData.push({
            cellAttrs: attrs,
          });
        }
      });

      dataRow.metadata = {
        cssClasses: dataRow.attrs && dataRow.attrs.class,
        rowattrs: dataRow.attrs,
        columns: colData,
        colData: newCols,
      };

      return newCols;
    });

    setupAggColors(config, colors_enabled, colors, opacity);
    setupDimColors(config, xDimColors, yDimColors);
    setupInitAlignment(config);
    const gridColumns = [];
    const widthRows = slickRows.slice(0, 20);
    const minWidth = 40;
    const maxWidth = 200;
    const padding = Number(user_settings.find_by_id('table_cell_padding').value());

    function colWidth(idx) {
      let width = customWidths[idx];
      if (width !== undefined) {
        return width;
      }

      const colData = widthRows.map((row) => row[idx]);
      const bounds = lib.textBounds(colData, {
        useSpan: true,
        textIsHtml: true,
      });

      width = bounds.width + padding * 2 + 2;
      width = Math.max(width, minWidth);
      width = Math.min(width, maxWidth);
      return width;
    }

    for (let idx = 0; idx < yDimCols.length; ++idx) {
      const width = colWidth(idx);
      gridColumns.push({
        // name: yDimCols[idx].colname(),
        name: '',
        field: idx,
        width,
      });
    }

    for (let idx = yDimCols.length; idx < maxCols; ++idx) {
      const width = colWidth(idx);
      gridColumns.push({
        // name: c.colname(),
        name: '',
        field: idx,
        width,
      });
    }

    if (showRowTotals.isChecked) {
      const aggRowTotalDisplays = aggRowTotals.map((a) => {
          if (separateRowTotals) {
            return a.map((val, i) => lib.formattedDisplay(agg_formatters[agg_start + i], val));
          }

          return lib.formattedDisplay(agg_formatters[agg_start], a);
        });

      const headerDataRow = (trellis.x_count() === 0) ?
          slickRows[0] : slickRows[1 + trellis.x_count()];
      if (trellis.x_count() === 0) {
        headerDataRow.push('Total');
        gridColumns.push({
          // name: 'Total',
          name: '',
          field: maxCols,
          width: customWidths[maxCols],
          cssClass: 'totals-row-cell',
        });
      } else if (separateRowTotals) {
        for (let i = 0; i < agg_cnt; i++) {
          headerDataRow.push(`Total ${agg_cols[i].colname()}`);
          gridColumns.push({
            // name: `Total ${agg_cols[i].colname()}`,
            name: '',
            field: maxCols + i,
            width: customWidths[maxCols + i],
            cssClass: 'totals-row-cell',
          });
        }
      } else {
        headerDataRow.push('Total');
        gridColumns.push({
          name: '',
          field: maxCols,
          width: customWidths[maxCols],
          cssClass: 'totals-row-cell',
        });
      }

      let startRowIdx = 0;
      if (trellis.x_count() === 0) {
        startRowIdx = 1;
      } else {
        startRowIdx = 2 + trellis.x_count();
      }

      for (let idx = startRowIdx; idx < slickRows.length; ++idx) {
        const dataRow = slickRows[idx];
        if (separateRowTotals) {
          const totals = aggRowTotalDisplays[idx - startRowIdx];
          for (let i = 0; i < totals.length; ++i) {
            dataRow.push(totals[i]);
          }
        } else {
          dataRow.push(aggRowTotalDisplays[idx - startRowIdx]);
        }
      }

      if (showColTotals.isChecked) {
        grandTotal.forEach((total) => aggColTotals.push(total));
      }
    }

    if (dataRows.length > 0) {
      dataRows[0].metadata.cssClasses = `${dataRows[0].metadata.cssClasses || ''} viz-tr-firstrow`;
    }

    const chartId = lib.chart_div();
    const h = $(`#${chartId}`).height();

    const bounds = lib.textBounds(['ABCgpy'], {
            useSpan: true,
          });
    const borderHeight = 1;

    const rowHeight = bounds.height + padding * 2 + borderHeight;
    const footerRowHeight = bounds.height + padding * 2 + borderHeight;
    const headerHeight = 10;
    const scrollbarHeight = 12;
    const topAndBottomBorderHeight = 2;

    const contentHeight = (slickRows.length + 1) *
      rowHeight +
      headerHeight +
      topAndBottomBorderHeight +
      scrollbarHeight;

    const height = contentHeight > h ? h : contentHeight;

    $container_div.addClass('viz-crosstab-table');

    // We hide the header row - see setVizTableStyles. So add in one more row here
    $container_div.height(`${height}px`);

    // enableTextSelectionOnCells works only in view mode
    // as on each cell click we are showing the context menu in edit mode
    // so the user able to copy the data from there
    const options = {
      showHeaderRow: false,
      enableColumnReorder: false,
      rowHeight,
      footerRowHeight,
      enableTextSelectionOnCells: true,
      defaultFormatter(row, cell, value) {
        return value;
      }
    };

    if (showColTotals.isChecked) {
      options.createFooterRow = true;
      options.showFooterRow = true;
    }

    slickRows.getItemMetadata = function(rowId) {
      const dataRow = dataRows[rowId];
      return dataRow.metadata;
    };

    const grid = new Slick.Grid(container_div, slickRows, gridColumns, options);
    grid.registerPlugin(new Slick.AutoTooltips({ enableForCells: true }));
    grid.render();

    function getColPropsFromUI(column, colIndex) {
      const colValArrayAsCSV = $(`#col ${colIndex}_colValueArray`).val();
      let colValueArray;
      if (colValArrayAsCSV) {
        colValueArray = colValArrayAsCSV.split(',');
      }

      const autoSizeOptions = {
          autosizeMode: true,
          forceFitColumns: true,
          colValueArray,
      };
      column.autoSize = autoSizeOptions;

      const width = parseInt($(`#col ${colIndex}_width`).val(), 10) || -1;
      if (width >= 0) {
        column.width = width;
      }
    }

    function syncGridHeaderWidthToGrid() {
      $('#grid-header-div').width($(grid.getContainerNode()).width());
    }

    function getAutosizeConfigFromUI() {
      const allCols = grid.getColumns();
      for (let i = 0; i < allCols.length; i++) {
        getColPropsFromUI(allCols[i], i + 1);
      }

      grid.setOptions({
        forceFitColumns: true,
        autosizeColsMode: true,
      });
    }

    if (fullTableWidth.isChecked) {
      getAutosizeConfigFromUI();
      grid.autosizeColumns();
      syncGridHeaderWidthToGrid();
    }

    const fallbackColumnWidth = 100;
    function extractHeaderWidth(columnIndex) {
      const headerCell = $(`.slick-cell[data-col="${columnIndex}"]`).first();
      const isTotalsCell = headerCell.hasClass('totals-row-cell');
      if (isTotalsCell) {
        return headerCell.getBoundingClientRect().width || fallbackColumnWidth;
      }

      const headerSpan = headerCell.find('.viz-col-sort').first()[0];

      return headerSpan?.getBoundingClientRect().width || fallbackColumnWidth;
    }

    function extractMaxDataWidth(columnIndex) {
      let maxColumnWidth = fallbackColumnWidth;

      // Create a single reusable temporary span element for measurement
      const tempSpan = $('<span>')
        .css({ visibility: 'hidden', whiteSpace: 'nowrap' })
        .appendTo('body');

      $(`#${chartId} .crosstab-table-container .slick-cell[data-col="${columnIndex}"]`).each(function () {
        const cellContent = $(this).text();

        tempSpan.text(cellContent);
        const cellWidth = tempSpan.get(0).getBoundingClientRect().width;

        if (cellWidth > maxColumnWidth) {
          maxColumnWidth = cellWidth;
        }
      });
      tempSpan.remove();

      return maxColumnWidth;
    }

    function calculateColumnWidth(columnIndex) {
      const headerWidth = extractHeaderWidth(columnIndex);
      const dataWidth = extractMaxDataWidth(columnIndex);

      const totalWidth = Math.max(headerWidth, dataWidth) + 40;

      return totalWidth;
    }

    function resizeColumnsToFitHeaders(gridLayout) {
      const allColumns = gridLayout.getColumns();

      allColumns.forEach((column, index) => {
        column.width = calculateColumnWidth(index);
      });

      // Update the grid with the new columns configuration
      gridLayout.setColumns(allColumns);
      gridLayout.resizeCanvas();
      syncGridHeaderWidthToGrid(); // Ensure header widths are synced
    }

    if (autoSizeColumn.isChecked) {
      resizeColumnsToFitHeaders(grid);
    }

    grid.onColumnsResized.subscribe(() => {
      const widths = grid.getColumns().map((c) => c.width);
      customWidths = widths;
      lib.page_needs_save();
    });

    trellis.sorting(d3.selectAll(container_div), () => {
      crosstab.viz_drawVisual(viz_div);
    });

    function posToRow(d, i, elem) {
      if (!elem) {
        return undefined;
      }

      const canvasOffset = $(elem).offset();
      if (!canvasOffset) {
        return undefined;
      }

      // eslint-disable-next-line no-undef
      const evt = d4.event || d3.event;
      if (!evt) {
        return undefined;
      }

      const $cellCol = $(evt.target).closest('.slick-cell');
      const row = $cellCol.attr('data-rowIdx');

      if (row !== undefined) {
        return rows[Number(row)];
      }

      return undefined;
    }

    arc.marks()
        .viz_lib(lib)
        .addSelection(d3.selectAll(container_div))
        .text_changes_on_move(true)
        .marker_name('nodes')
        .text_fn((d, i, elem) => {
          let row = posToRow(d, i, elem);
          if (row) {
            if (Array.isArray(row)) {
              row = row.map(
                (item) => {
                  if (typeof item === 'string') {
                    return item.startsWith('<span data-row-number=') ? '' : item;
                  }

                  return item;
                }
              );
            } else if (typeof row === 'string') {
              row = row.startsWith('<span data-row-number=') ? '' : row;
            }

            return lib.text_from_columns(row, columns);
          }
        })
        .params_fn((d, i, elem) => {
          const row = posToRow(d, i, elem);
          if (row) {
            return lib.params_from_columns(row, columns);
          }
        })
        .activate();

    let agg_total_displays;

    if (showColTotals.isChecked) {
      agg_total_displays = aggColTotals.map((a, i) => lib.formattedDisplay(
        agg_formatters[agg_start + (i % agg_cnt)], a
      ));

      if (trellis.y_count() > 0) {
        const $cell = $(grid.getFooterRowColumn(0));
        $cell.addClass('viz-td-header');
        $cell.html('Total');
      }

      const font = lib.getCrosstabTitleFont();

      agg_total_displays.forEach((val, idx) => {
        const $cell = $(grid.getFooterRowColumn(idx + trellis.y_count()));
        if (font) {
          $cell.css('font-family', `${font}`);
        }

        $cell.html(val);
      });
    }

    // download excel format
    this.slickRowsToTable = function slickRowsToTable() {
      const body = slickRows.reduce((acc, row) => {
        acc = acc.concat('<tr>', row.map((col) => `<td>${col}</td>`, '</tr>'));

        return acc;
      }, ['<table><tbody>']);

      if (showColTotals.isChecked) {
        body.push('<tr>');

        if (trellis.y_count() > 0) {
          body.push(`<td>Total</td>`);
        }

        agg_total_displays.forEach((val) => {
          body.push(Number.isNaN(val) ? `<td></td>` : `<td>${val}</td>`);
        });

        body.push('</tr>');
      }

      body.push('</tbody></table>');

      return body.join('');
    };

    // email attachment excel format
    this.exportCSVForCrosstab = function() {
      const cellDataArray = slickRows.map((row) => `${row.map((value) => `"${value.replace(/(<([^>]+)>)/ig, '')}"`).join(',')}\n`);

      if (showColTotals.isChecked) {
        if (trellis.y_count() > 0) {
          agg_total_displays.unshift('Total');
        }

        cellDataArray.push(`${agg_total_displays.map((value) => ((Number.isNaN(value)) ? '' : `"${value}"`)).join(',')}\n`);
      }

      return cellDataArray;
    };

    return crosstab;
  };

  function setupAggColors(config, colors_enabled, colors, opacity) {
    const {
      agg_start,
      agg_cnt,
      xDimCols,
      yDimCols,
      dataRows,
    } = config;

    let agg_i;
    for (agg_i = 0; agg_i < agg_cnt; ++agg_i) {
      if (colors_enabled[agg_start + agg_i]) {
        const colorScale = colors[agg_start + agg_i];
        const colorOpacity = opacity[agg_start + agg_i];

        // eslint-disable-next-line no-loop-func
        dataRows.forEach((dataRow, rowIdx) => {
          if (rowIdx < 1 + (xDimCols.length ? (xDimCols.length + 1) : 0)) {
            return;
          }

          const metadataCols = dataRow.metadata.columns;
          const { colData } = dataRow.metadata;
          for (let colIdx = yDimCols.length + agg_i;
              colIdx < metadataCols.length; colIdx += agg_cnt) {
            const metadataCol = metadataCols[colIdx];
            if (metadataCol && metadataCol.cellAttrs) {
              let fg = 'black';
              let bg = 'white';

              if (colData[colIdx] !== '') {
                bg = colorScale(Number(metadataCol.cellAttrs['data-agg']));

                const rgba = arc.colorpalette.colorToRGBA(bg, colorOpacity);
                fg = arc.colorpalette.colorForBrightness(rgba, colorOpacity);
                if (metadataCol.cellAttrs.style) {
                  metadataCol.cellAttrs.style += `background-color:${rgba};color:${fg};`;
                } else {
                  metadataCol.cellAttrs.style = `background-color:${rgba};color:${fg};`;
                }
              }
            }
          }
        });
      }
    }
  }

  return crosstab;
};

arc.visual.plugins.crosstab_extension = function() {
  const ext = function() { return 'arc.visual.plugins.crosstab_extension'; };

  ext.viz_name = function() {
    return 'crosstab';
  };

  ext.viz_tooltip = function() {
    return 'crosstab';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_crosstab.png?v=1';
  };

  ext.viz = arc.visual.plugins.crosstab;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.crosstab_extension());
}());
