window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

arc.visual.plugins = arc.visual.plugins || {};

if (!arc.frontend.featureFlipper.getValueById('enable_legacy_crosstab')) {
  return;
}

arc.visual.plugins.crosstab = function(doNotDrawSettings) {
  let sort_orders_asc = {};
  let saved_settings = {};
  let data_result;
  let trellis;
  let lib;

  const crosstabExpansionTracker = arc.expansionTracker();

  const colors_enabled_scale = arc.report.user_settings.colors_enabled()
                                .value('true');

  const showColTotals = arc.visual.setting()
            .id('show_crosstab_totals')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('SHOW_CROSSTAB_TOTALS')
            .add_label('Show column totals')
            .value('false');

  const combineRowMeasureTotals = arc.visual.setting()
            .id('combine_row_measure_totals')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('SEPARATE_ROW_MEASURES_totals')
            .add_label('Combine row totals')
            .info('Valid when there are multiple measures and at least one dimension in the ' +
                      ' Column shelf.')
            .value('false');

  const showRowTotals = arc.visual.setting()
            .id('show_row_totals')
            .tab('Marks')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .addDependents([combineRowMeasureTotals])
            .add_name('SHOW_ROW_TOTALS')
            .add_label('Show row totals')
            .value('false');

  const showAdjacentDuplicates = arc.visual.setting()
                                  .id('show_adjacent_duplicate_values')
                                  .tab('Marks')
                                  .type(arc.visual.settingTypeEnum.CHECKBOX)
                                  .add_name('SHOW_ADJACENT_DUPLICATE_VALUES')
                                  .add_label('Show adjacent duplicate values')
                                  .value('false')
                                  .valueBeforeRestore('false');

  const table_full_width = arc.report.user_settings.settingFullWidth();
  table_full_width.paint_table_style = function(isFullWidth) {
    if (isFullWidth === undefined) {
      // This code comes in play when no parameters are passed in to
      // paint_table_style. See usage below in this file.
      isFullWidth = table_full_width.isChecked;
    }

    if (isFullWidth !== table_full_width.isChecked) {
      table_full_width.value(isFullWidth);
      table_full_width.makeChangeCbs();
    }

    // figure out what is current visual
    // Remove all styles
    const $allConstituentTables = $(`#${lib.chart_div()} table.viz-crosstab-table`);
    $allConstituentTables.removeClass("table-width-auto resizable")
                         .removeClass("table-width-100");
    if (isFullWidth) {
      $allConstituentTables.addClass("table-width-100");
    } else {
      $allConstituentTables.addClass("table-width-auto resizable");
    }

    return table_full_width;
  };

  const aggregateRowStyle = arc.settings.fonts.aggregateRowStyle();
  const aggregateRowWeight = arc.settings.fonts.aggregateRowWeight();

  const user_settings = arc.visual.defaultSettings({
                             noTrellis: true,
                             noAxes: true,
                             doNotDrawSettings,
                         })
                         .addSetting([
                           arc.colorpalette.setting(),
                           colors_enabled_scale,
                           showAdjacentDuplicates,
                           showColTotals,
                           showRowTotals,
                           combineRowMeasureTotals,
                           table_full_width,
                           arc.settings.crosstab.totals(),
                           arc.settings.crosstab.crosstab_total_str(),
                           aggregateRowStyle,
                           aggregateRowWeight,
                           arc.settings.table.tableHeaderBackgroundColor(),
                           arc.settings.table.tableHeaderTextColor(),
                           arc.settings.fonts.crosstabTitleFont(),
                           arc.settings.fonts.crosstabTitleFontSize(),
                           arc.settings.fonts.crosstabTitleFontWeight(),
                           arc.settings.table.settingCrosstabHeaderAlignment(),
                           arc.settings.table.settingCrosstabContentAlignment(),
                         ]);

  const crosstab = arc.visual.api();

  function aggregateRowClass() {
    const v = `${aggregateRowStyle.valueToClass()} ${aggregateRowWeight.valueToClass()}`;

    return v.trim();
  }

  const shelf_data = [
    {
      name: 'x_dimensions_shelf',
      display_name: 'Column',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_dimensions_shelf',
      display_name: 'Row',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measures',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
    }
  ];

  const constraints = {
    minimum_dims: 1,
    minimums: {
      aggregates_shelf: 1,
    }
  };

  crosstab.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    crosstabExpansionTracker.viz_lib(lib);
    lib.chart_type_graded(true)
       .shelf_data(shelf_data)
       .user_settings(user_settings);
    return crosstab;
  };

  crosstab.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    saved_settings = settings;
    return settings;
  };

  crosstab.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return crosstab;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    return crosstab;
  };

  crosstab.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup()
       .settings_color_disabled_show();

    // Define messages to be included in shelves
    const xDimensionMessages = [
          lib.createPopoverMessages(['aggregateCrosstab', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ];
    const yDimensionMessages = [
          lib.createPopoverMessages(['aggregateCrosstab', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ];
    const aggregateMessages = [
          lib.createPopoverMessages(['aggregateAcross', 'columns and rows.']),
          lib.createPopoverMessages('acceptsMultiple')
        ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_dimensions_shelf: xDimensionMessages,
      y_dimensions_shelf: yDimensionMessages,
      aggregates_shelf: aggregateMessages,
    });

    return crosstab;
  };

  crosstab.viz_getDataRequest = function() {
    const dataRequest = lib.getCheckedDataRequest(constraints);

    crosstabExpansionTracker.onGetDataRequest(dataRequest);

    return dataRequest;
  };

  let lastSpecifiedSorting = [];

  /*
   * Set the data this object should use to draw out the visual
   */
  crosstab.viz_setDataResult = function(result) {
    data_result = result;

    crosstabExpansionTracker.onSetDataResult(data_result);

    return crosstab;
  };

  crosstab.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    const rows = data_result.rows();
    if (!rows) {
      return crosstab;
    }

    lib.unselectOnClick();

    d3.select(`#${viz_div}`)
        .append('div')
        .attr('class', 'crosstab-table-container');
    crosstab.tabulate(rows, `#${viz_div} .crosstab-table-container`, viz_div);
    table_full_width.paint_table_style();

    const $e = $(`#${viz_div}`);
    $e.off('click.crosstabexpand')
      .on('click.crosstabexpand', '.crosstab-expand', function(e) {
        e.preventDefault();
        e.stopPropagation();
        const rowIndex = $(e.target)
          .closest('tr')
          .data('parent-row-idx');
        crosstabExpansionTracker.expand(this, rowIndex);
      });

    $e.off('click.crosstabcollapse')
      .on('click.crosstabcollapse', '.crosstab-collapse', function(e) {
        e.preventDefault();
        e.stopPropagation();
        crosstabExpansionTracker.collapse(this);
      });

    return crosstab;
  };

  function computeColorsEnabled(config, rows) {
    const colors_enabled = [];
    const { agg_start } = config;
    const { agg_end } = config;

    let i;

    // Set 'colors_enabled' to indicate where aggregates should be colored.
    // First, set 'color_enabled' based on user settings.
    for (i = agg_start; i <= agg_end; ++i) {
      colors_enabled[i] = (colors_enabled_scale.isChecked);
    }

    if (colors_enabled_scale.isChecked) {
      // Now, walk the rows, and set colors_enabled to false for any
      // aggregate that has non-numeric values
      rows.forEach((row) => {
        for (i = agg_start; i <= agg_end; ++i) {
          if (colors_enabled[i]) {
            if (Number.isNaN(row[i]) && row[i] !== '' && row[i] !== 'NULL' && row[i] !== 'null') {
              colors_enabled[i] = false;
            }
          }
        }
      });
    }

    return colors_enabled;
  }

  function theadStyleFetch() {
    const theadStyle = [];

    const headerBgColor = user_settings.find_by_id('table_header_background_color').value();
    if (headerBgColor) {
      theadStyle.push(`background-color:${headerBgColor};`);
    }

    const headerTextColor = user_settings.find_by_id('table_header_text_color').value();
    if (headerTextColor) {
      theadStyle.push(`color:${headerTextColor};`);
    }

    {
      const font = lib.getCrosstabTitleFont();
      if (font) {
        theadStyle.push(`font-family:${font};`);
      }
    }

    {
      const fontSize = lib.getCrosstabTitleFontSizeLegacy();
      if (fontSize) {
        theadStyle.push(`font-size:${fontSize}px;`);
      }
    }

    {
      const fontWeight = lib.getTotalsFontWeight('crosstab_title_font_weight');
      if (fontWeight) {
        theadStyle.push(`font-weight:${fontWeight};`);
      } else {
        theadStyle.push('font-weight:300;');
      }
    }

    theadStyle.push('white-space:unset;');
    return theadStyle;
  }

  function setVizTableStyles() {
    const chartId = lib.chart_div();
    const allStyles = [];
    const legacyCrosstabHeaderCells = `#${chartId} table.viz-crosstab-table tbody tr.viz-tr-header td.viz-td-header`;
    const legacyCrosstabContentCells = `#${chartId} table.viz-crosstab-table tbody tr[data-row] td`;
    const legacyCrosstabFooter = `#${chartId} table.viz-crosstab-table tfoot tr td`;

    {
      const theadStyle = theadStyleFetch();
      if (theadStyle.length > 0) {
        allStyles.push(`#${chartId} tr.viz-tr-header, #${chartId} tr td.viz-td-header {`);
        theadStyle.forEach((s) => {
          allStyles.push(s);
        });

        allStyles.push('}');

        const crosstabHeaderAlignment = user_settings.find_by_id('crosstab_header_alignment').value();
        if (crosstabHeaderAlignment) {
          allStyles.push(`${legacyCrosstabHeaderCells} {text-align:${crosstabHeaderAlignment}!important;}`);
        }

        {
          const crosstabContentAlignment = user_settings.find_by_id('crosstab_content_alignment').value();
          if (crosstabContentAlignment !== 'default') {
            allStyles.push(`${legacyCrosstabContentCells}, ${legacyCrosstabFooter} {text-align:${crosstabContentAlignment}!important;}`);
          }
        }
      }
    }

    const $viz = $(`#${chartId}`);
    let $style = $viz.find('style.vizstyle');
    if ($style.length === 0) {
      $viz.append(arc.misc.safe_substitute('<style class="vizstyle"></style>', {}));
      $style = $viz.find('style.vizstyle');
    }

    $style.html(`${allStyles.join('')}`);
  }

  crosstab.tabulate = function(rows, container_div, viz_div) {
    setVizTableStyles();

    if (rows.length === 0) {
      return crosstab;
    }

    const table = arc.misc.trellisTable();
    const columns = data_result.columns();
    const agg_cols = lib.shelf_columns(columns, "aggregates_shelf");
    const tooltip_cols = lib.shelf_columns(columns, "tooltip_shelf");
    const agg_start = agg_cols[0].colindex();
    const agg_end = agg_cols[agg_cols.length - 1].colindex();
    const agg_cnt = agg_cols.length;

    const config = {
      agg_cols,
      agg_start,
      agg_end,
      agg_cnt,
    };

    const colors_enabled = computeColorsEnabled(config, rows);

    // ARC-10628: Modify sort order based on order property
    const currentSorting =
        columns.filter((c) => c.order())
               .map((c) => c.colname());

    if (!arc.misc.isIdentical(currentSorting, lastSpecifiedSorting)) {
      // Sorting has changed in the shelves since the last time we ran this
      sort_orders_asc = {};

      columns.forEach((c) => {
        const columnName = c.colname();
        if (c.order()) {
          sort_orders_asc[columnName] = c.order().ascending();
        }
      });

      lastSpecifiedSorting = currentSorting;
    }

     /**
     * If expansion is enabled, it only makes sense if
     * the columns are sorted so here we sort all the columns (asc)
     * if they're not sorted already.
     */
    if (crosstabExpansionTracker.enabled()) {
      columns.forEach((c) => {
        const colName = c.colname();
        if (!(colName in sort_orders_asc)) {
          sort_orders_asc[colName] = true;
        }
      });
    }

    trellis = arc.visual.trellis()
                     .x_shelf_name('x_dimensions_shelf')
                     .y_shelf_name('y_dimensions_shelf')
                     .data_columns(agg_cols)
                     .rows(rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(columns)
                     .table(table)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select(`#${viz_div}`))
                     .ignoreLargeTrellis(true)
                     .showAdjacentDuplicates(showAdjacentDuplicates.value() === 'true')
                     .setup();

    // Collect up aggregate formatters
    const aggColTotals = [];
    const aggRowTotals = [];
    let separateRowTotals = false;
    let grandTotal = [0];
    if (agg_cnt > 1 && lib.shelf_columns(columns, 'x_dimensions_shelf').length > 0) {
      if (!combineRowMeasureTotals.isChecked) {
        separateRowTotals = true;
        grandTotal = Array.from({ length: agg_cnt }, () => 0);
      }
    }

    const agg_formatters = {};
    let agg_i;
    for (agg_i = agg_start; agg_i <= agg_end; ++agg_i) {
      if (lib.formatter(agg_i)) {
        agg_formatters[agg_i] = lib.formatter(agg_i);
      } else {
        agg_formatters[agg_i] = lib.default_formatter;
      }
    }

    // Fetch aggregate values from a row
    const agg_getter = function(d) {
      return d.slice(agg_start, agg_end + 1);
    };

    // Given an array of aggregate values, return an array of formatted
    // aggregate values
    const agg_getter_display = function(al) {
      return al.map((a, i) => lib.formattedDisplay(agg_formatters[agg_start + i], a));
    };

    // Setup the color scales for each aggregate
    const colors = [];
    const gradedColors = lib.graded_colors();
    const colorSettings = lib.colorSettings();
    const getAgg = (data) => Number(data[agg_i]);
    for (agg_i = agg_start; agg_i <= agg_end; ++agg_i) {
      if (colors_enabled[agg_i]) {
        const colorDomain = d3.extent(rows, getAgg);
        const color = arc.colorpalette.aggScale(colorDomain, gradedColors, colorSettings);
        colors[agg_i] = color;
      }
    }

    const aggregateClass = aggregateRowClass();

    rows.forEach((d) => {
      const colIdx = trellis.x_pos_getter(d);
      const rowIdx = trellis.y_pos_getter(d);
      const aggs = agg_getter(d);
      const isExpandedSubTotalRow = d.slice(0, agg_start)
                                     .some((a) => String(a).includes('crosstab-collapse'));

      while (separateRowTotals && rowIdx >= aggRowTotals.length) {
        aggRowTotals.push([]);
      }

      const { isFullyExpanded } = d;

      agg_getter_display(aggs).forEach((val, i) => {
          if (!isFullyExpanded && aggregateClass) {
            val = `<span class="${aggregateClass}">${val}</span>`;
          }

          table.data().data(rowIdx, colIdx, i, val, {
            agg: aggs[i]
          });

          const aggColIdx = (colIdx * agg_cnt) + i;
          aggColTotals[aggColIdx] = aggColTotals[aggColIdx] || 0;
          if (separateRowTotals) {
            aggRowTotals[rowIdx][i] = aggRowTotals[rowIdx][i] || 0;
          } else {
            aggRowTotals[rowIdx] = aggRowTotals[rowIdx] || 0;
          }

          if (!Number.isNaN(aggs[i])) {
            if (!isExpandedSubTotalRow) {
              aggColTotals[aggColIdx] += Number(aggs[i]);
            }

            if (separateRowTotals) {
              aggRowTotals[rowIdx][i] += Number(aggs[i]);
              if (!isExpandedSubTotalRow) {
                grandTotal[i] += Number(aggs[i]);
              }
            } else {
              aggRowTotals[rowIdx] += Number(aggs[i]);
              if (!isExpandedSubTotalRow) {
                grandTotal[0] += Number(aggs[i]);
              }
            }
          }
        });
    });

    const div_sel = d3.select(container_div)
                    .append('div');
    const rendered_table = table.draw('table table-bordered viz-crosstab-table',
        sort_orders_asc, trellis);
    div_sel.html(rendered_table);

    aggColor(config, div_sel, colors_enabled, colors);
    trellis.sorting(div_sel, () => {
      crosstab.viz_drawVisual(viz_div);
    });

    function getRow(xData, yData) {
      let row;

      rows.some((d) => {
        const x = trellis.x_getter(d);
        if (x) {
          if (!arc.arraysIdentical(xData, x)) {
            return false;
          }
        }

        const y = trellis.y_getter(d);
        if (y) {
          if (!arc.arraysIdentical(yData, y)) {
            return false;
          }
        }

        row = d;
        return true;
      });

      return row;
    }

    function colToParam(row, col) {
      return {
        key: col.colname(),
        value: row[col.colindex()],
        keytype: col.colproperty(),
        valuetype: col.coltype(),
        formatter: col.coldisplayformat(),
        expr: col.expr(),
      };
    }

    trellis.tooltips('primary',
        (params, elem, x_data, y_data) => {
          const row = getRow(x_data, y_data);
          if (row) {
            agg_cols.forEach((col) => {
              params.push(colToParam(row, col));
            });

            tooltip_cols.forEach((col) => {
              params.push(colToParam(row, col));
            });
          }
        });

    const dataRows = table.getData(sort_orders_asc, trellis);
    config.dataRows = dataRows;
    let maxCols = 0;

    const slickRows = dataRows.map((dataRow, r_i) => {
      if (maxCols < dataRow.cols.length) {
        maxCols = dataRow.cols.length;
      }

      if (dataRow.attrs === undefined) {
        dataRow.attrs = {};
      }

      const newCols = [];
      const colData = [];
      dataRow.cols.forEach((c, c_i) => {
        newCols.push(c.data);

        const attrs = c.attrs || {};
        attrs['data-col'] = c_i;
        attrs['data-parent-row-idx'] = r_i - 1;
        const { colspan } = attrs;
        if (colspan) {
          colData.push({
            colspan: Number(colspan),
            cellAttrs: attrs,
          });

          for (let i = 1; i < colspan; ++i) {
            newCols.push('');
            colData.push(undefined);
          }
        } else {
          colData.push({
            cellAttrs: attrs,
          });
        }
      });

      dataRow.metadata = {
        cssClasses: dataRow.attrs && dataRow.attrs.class,
        rowattrs: dataRow.attrs,
        columns: colData,
        colData: newCols,
      };

      return newCols;
    });

    if (showRowTotals.isChecked) {
      const aggRowTotalDisplays = aggRowTotals.map((a) => {
          if (separateRowTotals) {
            return a.map((val, i) => lib.formattedDisplay(agg_formatters[agg_start + i], val));
          }

          return lib.formattedDisplay(agg_formatters[agg_start], a);
        });

      const $table = $(`#${lib.chart_div()} .viz-crosstab-table`);
      const $rowHeader = $table.find('.viz-tr-header');
      const rowSpan = 2 + trellis.x_count();
      const vAlign = 'style="vertical-align: middle"';
      const headerDataRow = (trellis.x_count() === 0) ?
          slickRows[0] : slickRows[1 + trellis.x_count()];

      if ($rowHeader.length === 1) {
        $rowHeader.first().append('<td class="viz-td-header viz-crosstab-measure">Total</td>');
      } else if (separateRowTotals) {
        for (let i = 0; i < agg_cnt; i++) {
          $rowHeader.first().append(`<td rowspan="${rowSpan}" ${vAlign}>Total ${agg_cols[i].colname()}</td>`);
        }
      } else {
        $rowHeader.first().append(`<td rowspan="${rowSpan}" ${vAlign}>Total</td>`);
      }

      const $tRows = $table.find('tr').not('.viz-tr-header');
      $tRows.each(function(idx) {
        if (separateRowTotals) {
          aggRowTotalDisplays[idx].forEach((val) => {
            $(this).append(`<td>${val}</td>`);
          });
        } else {
          $(this).append(`<td>${aggRowTotalDisplays[idx]}</td>`);
        }
      });

      let startRowIdx = 0;
      if (trellis.x_count() === 0) {
        headerDataRow.push('Total');
        startRowIdx = 1;
      } else {
        startRowIdx = 2 + trellis.x_count();
      }

      for (let idx = startRowIdx; idx < slickRows.length; ++idx) {
        const dataRow = slickRows[idx];
        if (separateRowTotals) {
          const totals = aggRowTotalDisplays[idx - startRowIdx];
          for (let i = 0; i < totals.length; ++i) {
            dataRow.push(totals[i]);
          }
        } else {
          dataRow.push(aggRowTotalDisplays[idx - startRowIdx]);
        }
      }

      if (showColTotals.isChecked) {
        grandTotal.forEach((total) => aggColTotals.push(total));
      }
    }

    let agg_total_displays;

    if (showColTotals.isChecked) {
      agg_total_displays = aggColTotals.map((a, i) => lib.formattedDisplay(
        agg_formatters[agg_start + (i % agg_cnt)], a
      ));

      const $table = $(`#${lib.chart_div()} .viz-crosstab-table`);
      $table.append($('<tfoot><tr></tr></tfoot>'));

      const $tfoot = $table.find('tfoot tr');
      if (trellis.y_count() > 0) {
        $tfoot.append(`<td class="viz-td-header" colspan="${trellis.y_count()}">Total</td>`);
      }

      agg_total_displays.forEach((t) => {
        $tfoot.append(`<td>${t}</td>`);
      });
    }

    this.slickRowsToTable = function slickRowsToTable() {
      const body = slickRows.reduce((acc, row) => {
        acc = acc.concat('<tr>', row.map((col) => `<td>${col}</td>`, '</tr>'));

        return acc;
      }, ['<table><tbody>']);

      if (showColTotals.isChecked) {
        body.push('<tr>');

        if (trellis.y_count() > 0) {
          body.push(`<td>Total</td>`);
        }

        agg_total_displays.forEach((val) => {
          body.push(Number.isNaN(val) ? `<td></td>` : `<td>${val}</td>`);
        });

        body.push('</tr>');
      }

      body.push('</tbody></table>');

      return body.join('');
    };

    this.exportCSVForCrosstab = function() {
      const cellDataArray = slickRows.map((row) => `${row.map((value) => `"${value.replace(/(<([^>]+)>)/ig, '')}"`).join(',')}\n`);

      if (showColTotals.isChecked) {
        if (trellis.y_count() > 0) {
          agg_total_displays.unshift('Total');
        }

        cellDataArray.push(`${agg_total_displays.map((value) => ((Number.isNaN(value)) ? '' : `"${value}"`)).join(',')}\n`);
      }

      return cellDataArray;
    };

    return crosstab;
  };

  // Change the background color on the aggregate columns to indicate
  // the magnitude visually
  function aggColor(config, div, colors_enabled, colors) {
    const { agg_start } = config;
    const { agg_end } = config;
    const { agg_cnt } = config;

    const agg_tds =
      div.selectAll('tr')
         .filter(function(d) {
           return !($(this).hasClass('viz-tr-header'));
         })
         .selectAll('td')
         .filter(function() {
           return !($(this).hasClass('viz-td-header'));
         });

    let agg_i;
    for (agg_i = 0; agg_i < agg_cnt; ++agg_i) {
      if (!colors_enabled[agg_start + agg_i]) {
        // eslint-disable-next-line no-continue
        continue;
      }

      // eslint-disable-next-line no-loop-func
      agg_tds.filter((d, i) => ((i % agg_cnt) === agg_i))
             // eslint-disable-next-line no-loop-func
             .style('background-color', function() {
               if ($(this).text() === '') {
                 return 'white';
               }

               return colors[agg_start + agg_i](Number(this.dataset.agg));
             })
             // eslint-disable-next-line no-loop-func
             .style('color', function() {
               if ($(this).text() === '') {
                 return 'black';
               }

               // Depending on the brightness of the background color, use
               // white or black text
               const hex = colors[agg_start + agg_i](Number(this.dataset.agg));
               return arc.colorpalette.colorForBrightness(hex);
             });
    }
  }

  return crosstab;
};

arc.visual.plugins.crosstab_extension = function() {
  const ext = function() { return 'arc.visual.plugins.crosstab_extension'; };

  ext.viz_name = function() {
    return 'crosstab';
  };

  ext.viz_tooltip = function() {
    return 'crosstab';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_crosstab.png?v=1';
  };

  ext.viz = arc.visual.plugins.crosstab;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.crosstab_extension());
}());
