window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};

const timeFormatDateTime = 'YYYY-MM-DD HH:mm:ss',
      timeFormatDate = 'YYYY-MM-DD';

arc.visual.plugins.daterange = function(inDataRangeType) {
  var saved_settings = {},
      filterSettings,
      data_result,
      min_timestamp,
      max_timestamp,
      lib,
      default_selections = {},
      defaultDateRangeType = inDataRangeType || 'fixed',
      timeformat = timeFormatDate,
      $quickFilterBtns = $(),

      // When the user makes changes in edit mode, we track those changes
      // in 'toSaveSelections'. If the user hits the 'Reset' icon in the
      // dashboard, we reset 'toSaveSelections'. If the user hits 'Save',
      // we copy 'toSaveSelections' into 'default_selections' and save
      // away the changes
      toSaveSelections,
      toSaveDateRangeType = defaultDateRangeType,

      // Track whether the same output parameters is being output in different forms
      hasConflictingParams,
      quickFilterSuffix = '.btn';

  // Time format for Solr and Elasticsearch
  const fullTextTimeFormat = 'YYYY-MM-DDTHH:mm:ss[Z]';

  // For solr, the query to load the min_timestamp and max_timestamp is slow.
  // This variable is set to true if we need to delay the loading of the min and
  // max values until after we've drawn the visual.
  let delayLoad = false;

  var daterange = arc.visual.api();

  var daterange_ranges =
    arc.visual.setting()
       .id('oneclick_dateranges')
       .add_class('setting-ranges-dropdown')
       .tab('Ranges')
       .type(arc.visual.settingTypeEnum.DROPDOWN)
       .group('Select a date range set')
       .add_name('default')
       .add_label('default')
       .value('default');

  daterange_ranges.paintTable = function(ranges) {
    const $sel = $(`#${daterange_ranges.div_id()}`);

    function paintRanges(ranges) {
      const $tbody = $sel.find('tbody');
      const row_template = $('#dateranges-table-row-template tbody').html();
      if (ranges) {
        Object.keys(ranges).forEach(function(k) {
          $tbody.append(arc.misc.safe_substitute(
            row_template,
            {
              VALUE: k,
            }));
        });
      } else {
        console.log('Invalid date range.');
      }
    }

    $sel.find('tbody').empty();
    let new_value = $sel.find('button').text();
    if (!new_value) {
      return;
    }

    $sel.find('.btn-group').switchClass('pull-right', 'pull-left', 1, 'ease');
    $sel.find('th').text(`${new_value} Range Set`);

    let drl = arc.dateranges.get();
    let painted = false;
    drl.forEach(function(dr) {
      if (dr.name === new_value) {
        painted = true;
        paintRanges(arc.misc.rangesFromText(dr.value));
      }
    });

    if ((!painted) && (new_value === 'default')) {
      paintRanges(arc.misc.defaultRanges(daterange_showtime.isChecked, toSaveDateRangeType));
    }
  };

  daterange_ranges.add_postpaint_fn(function() {
    const $sel = $(`#${daterange_ranges.div_id()}`);
    $sel.append($('#dateranges-table-template').html());

    daterange_ranges.paintTable();
  });

  daterange_ranges.addChangeCb('updateTable', daterange_ranges.paintTable);

  const daterange_showtime =
    arc.visual.setting()
       .id('show_time')
       .tab('Basic')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('SHOW_TIME')
       .add_label('Permit selection of time range')
       .value('false');

  const daterange_maxdate =
    arc.visual.setting()
      .id('max_date')
      .tab('Basic')
      .type(arc.visual.settingTypeEnum.CHECKBOX)
      .add_name('MAX_DATE')
      .setDisableTooltipText('This setting is unavailable when "Save date range selections in edit mode" is enabled.')
      .add_label('Default to the latest available timestamp')
      .value('false');

  const daterange_remembervalues =
       arc.visual.setting()
          .id('remember_values')
          .tab('Basic')
          .type(arc.visual.settingTypeEnum.CHECKBOX)
          .add_name('REMEMBER_VALUES')
          .setDisableTooltipText('This setting is unavailable when "Default to the latest available timestamp" is enabled.')
          .add_label('Save date range selections made in edit mode')
          .value('true')
          .valueBeforeRestore('true');

  if (toSaveDateRangeType === 'quick_filters') {
    // For quick filters, hour format should be used always
    daterange_showtime.value('true');
  }

daterange_maxdate.addChangeCb('disableRememberValues', ({ isChecked }) => {
  daterange_remembervalues.setEnableDisable(!isChecked);
});

daterange_remembervalues.addChangeCb('disableMaxdate', ({ isChecked }) => {
  daterange_maxdate.setEnableDisable(!isChecked);
});

  daterange_showtime.add_postpaint_fn(function() {
    const $checkbox = $(`#${daterange_showtime.div_id()} input`);
    $checkbox.off('change.timerange')
             .on('change.timerange', function() {
               daterange_ranges.paintTable();
               timeformat = daterange_showtime.isChecked ? timeFormatDateTime :
                  timeFormatDate;
             });
  });

  daterange_maxdate.add_postpaint_fn(() => {
    daterange_remembervalues.setEnableDisable(!daterange_maxdate.isChecked);
  });

  daterange_remembervalues.add_postpaint_fn(() => {
    daterange_maxdate.setEnableDisable(!daterange_remembervalues.isChecked);
  });

  const daterange_allownulls = arc.settings.range.allowNulls().tab('Basic');

  const daterangeDataset = arc.settings.filters.datasetDisplay()
                              .id('daterange_dataset')
                              .tab('Basic')
                              .add_name('DATERANGE_DATASET');

  const daterange_base_field =
    arc.visual.setting()
       .id('daterange_base_field')
       .tab('Basic')
       .type(arc.visual.settingTypeEnum.STRING)
       .add_name('PICKLIST_BASE_FIELD')
       .add_label('Base Field')
       .value('');

  const daterange_title =
    arc.visual.setting()
       .id('daterange_title')
       .tab('Basic')
       .type(arc.visual.settingTypeEnum.STRING)
       .add_name('PICKLIST_TITLE')
       .add_label('Title')
       .value('');

  const daterange_width =
    arc.visual.setting()
       .id('daterange_width')
       .tab('Style')
       .type(arc.visual.settingTypeEnum.STRING)
       .add_name('DATERANGE_WIDTH')
       .add_label('Width (in text mode)')
       .value('');

  const daterange_output_parameter =
    arc.visual.setting()
       .id('daterange_output_parameter')
       .tab('Basic')
       .type(arc.visual.settingTypeEnum.STRING)
       .add_name('PICKLIST_OUTPUT_PARAMETER')
       .add_label('Output Parameter')
       .value('');

  daterange.daterange_output_parameter = daterange_output_parameter;
  daterange.daterange_base_field = daterange_base_field;

  const filter_appgroup_scope = arc.report.user_settings.filter_appgroup_scope();
  const filter_dataset_scope = arc.report.user_settings.filter_dataset_scope();
  daterange.daterange_filter_appgroup_scope = filter_appgroup_scope;
  daterange.daterange_filter_dataset_scope = filter_dataset_scope;
  daterange.daterangeDatasetName = daterangeDataset;

  const timezoneSetting = arc.settings.filters.timezone();

  const user_settings =
    arc.visual.defaultSettings({ isFilter: true })
              .addSetting([
                daterangeDataset,
                daterange_showtime,
                daterange_allownulls,
                daterange_maxdate,
                daterange_remembervalues,
                timezoneSetting,
                daterange_base_field,
                daterange_title,
                daterange_output_parameter,
                daterange_width,
                daterange_ranges,
                filter_appgroup_scope,
                filter_dataset_scope,
              ]);

  daterange_showtime.addDependents([timezoneSetting]);

  daterange.setting = function(settingName, value) {
    if (arguments.length === 1) {
      return arc.params.setting(daterange, settingName, 'daterange');
    }

    return arc.params.setting(daterange, settingName, 'daterange', value);
  };

  daterange.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data([])
       .user_settings(user_settings)
       .title_visible(false);
    return daterange;
  };

  daterange.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    saved_settings = settings;

    settings.viz_type = settings.viz_type || {};
    settings.viz_type.daterange = settings.viz_type.daterange || {};
    if (toSaveSelections) {
      // We should be cloning instead of pointing to same object
      default_selections = $.extend(true, {}, toSaveSelections);
    }

    if (toSaveDateRangeType) {
      defaultDateRangeType = toSaveDateRangeType;
    }

    if (daterange_remembervalues.isChecked) {
      settings.viz_type.daterange.default_selections = default_selections;
    } else {
      settings.viz_type.daterange.default_selections = {};
    }

    settings.viz_type.daterange.dateRangeType = defaultDateRangeType;
    return settings;
  };

  function getOutputParameter() {
    var output_parameter = daterange_output_parameter.value();
    if (!output_parameter) {
      output_parameter = daterange_base_field.value();
    }

    return output_parameter;
  }

  daterange.viz_getOutputParameter = getOutputParameter;

  function paramNames() {
    let outputParameter = getOutputParameter(),
        startParameter = outputParameter + '.start',
        endParameter = outputParameter + '.end',
        dateConfigParameter = outputParameter + '.config',
        includeNullsParameter = outputParameter + '.include_nulls';

    return {
      outputParameter,
      startParameter,
      endParameter,
      dateConfigParameter,
      includeNullsParameter,
    };
  }

  /*
   * daterange visual types emit 'output.start' and 'output.end' parameters
   *
   * Other visuals on the page may emit 'output' - see ARC-7110
   *
   * In this scenario, we need to accept this parameter in the daterange visual
   * and adjust to it. The function here lets the dashboard know that we are
   * in this scenario and that the daterange visual should be updated.
   */
  daterange.viz_conflictingParams = function() {
    let {
      outputParameter,
      startParameter,
      endParameter,
      dateConfigParameter,
      includeNullsParameter,
    } = paramNames();

    let inParams = lib.getParams().getkvs();
    if (inParams[outputParameter]) {
      if (toSaveSelections[startParameter] !== toSaveSelections[endParameter]) {
        hasConflictingParams = true;
        return true;
      }

      if (toSaveSelections[startParameter] !== inParams[outputParameter]) {
        hasConflictingParams = true;
        return true;
      }

      return false;
    }

    return false;
  };

  // When we are instantiated, there may be input parameters in play that we
  // have to reflect.
  daterange.updateToSaveSelectionsFromParams = function(settings = saved_settings) {
    let {
      outputParameter,
      startParameter,
      endParameter,
      dateConfigParameter,
      includeNullsParameter,
    } = paramNames();

    function defaultsFromStore() {
      if (!settings.viz_type ||
          !settings.viz_type.daterange ||
          !settings.viz_type.daterange.default_selections) {
        return;
      }

      default_selections = settings.viz_type.daterange.default_selections;
      if (default_selections.relative_date) {
        arc.filter.helpers.fetchRelative(default_selections.relative_date, timeformat,
            timezoneSetting.value());
        default_selections.start_time = default_selections.relative_date.startDate;
        default_selections.end_time = default_selections.relative_date.endDate;
        defaultDateRangeType = 'relative';
      } else if (default_selections.quick_filters) {
        const computedTimes = getQuickFilterParams(default_selections.quick_filters.name);
        default_selections.start_time = computedTimes.startDate;
        default_selections.end_time = computedTimes.endDate;
        defaultDateRangeType = 'quick_filters';
      }

      if (default_selections.dateRangeType) {
        defaultDateRangeType = default_selections.dateRangeType;
      }

      default_selections.includeNulls = default_selections.includeNulls ?? true;
    }

    function updateToSaveSelectionsBasedOnConfigParameter(inParams, dateConfigParam) {
      if (includeNullsParameter in inParams) {
        toSaveSelections.includeNulls = inParams[includeNullsParameter] === 'true';
      }

      toSaveDateRangeType = 'fixed';
      if (inParams[dateConfigParam]) {
        try {
          const dateConfig = JSON.parse(inParams[dateConfigParam]);
          if (dateConfig[arc.misc.relativeFilterSuffix]) {
            toSaveSelections.relative_date = dateConfig[arc.misc.relativeFilterSuffix];

            arc.filter.helpers.fetchRelative(
              toSaveSelections.relative_date,
              dateConfig[arc.misc.relativeFilterSuffix].datetimeFormat,
              timezoneSetting.value()
            );
            toSaveSelections.start_time = toSaveSelections.relative_date.startDate;
            toSaveSelections.end_time = toSaveSelections.relative_date.endDate;
            toSaveDateRangeType = 'relative';
          } else if (dateConfig[quickFilterSuffix]) {
            toSaveSelections.quick_filters = dateConfig[quickFilterSuffix];
            toSaveDateRangeType = 'quick_filters';
          }
        } catch (e) {
          console.log('Invalid Parameter: ', inParams[dateConfigParam]);
        }
      }
    }

    var inParams = lib.getParams().getkvs();

    if ((startParameter in inParams) && (endParameter in inParams)) {
      toSaveSelections = {
        start_time: inParams[startParameter],
        end_time: inParams[endParameter],
      };

      updateToSaveSelectionsBasedOnConfigParameter(inParams, dateConfigParameter);
    } else if (outputParameter in inParams) {
      toSaveSelections = {
        start_time: inParams[outputParameter],
        end_time: inParams[outputParameter],
      };

      updateToSaveSelectionsBasedOnConfigParameter(inParams, dateConfigParameter);
      hasConflictingParams = true;
    } else {
      defaultsFromStore();
      toSaveSelections = $.extend(true, {}, default_selections);
      toSaveDateRangeType = defaultDateRangeType;
    }
  };

  daterange.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return daterange;
    }

    saved_settings = settings;

    // We want the default filter_dataset_scope for new filters to be
    // 'true', but we want the default filter_dataset_scope for existing
    // filters to be 'false'.
    //
    // In order to do this, we define filter_dataset_scope in
    // arc_report_settings with a default of 'true', but if we are being
    // asked to restore settings from the metastore, then we set it to false
    // before resroting.
    filter_dataset_scope.value('false');

    lib.restore_basics(settings, switchingVisualType);
    upgradeDaterange(settings);
    timeformat = daterange_showtime.isChecked ? timeFormatDateTime : timeFormatDate;

    daterange.updateToSaveSelectionsFromParams(settings);
    publishParameters(false);

    if (lib.dataset()) {
      daterange.daterangeDatasetName.value(lib.dataset().dataset_name());
    }

    toSaveDateRangeType = settings.viz_type.daterange.dateRangeType || toSaveDateRangeType;

    return daterange;
  };

  daterange.getDateRangeType = function() {
    return toSaveDateRangeType;
  };

  function publishParameters(do_not_store) {
    // Publish the default parameters to the rest of the system now rather than
    // only after the initial sql completes.
    if (toSaveSelections && toSaveSelections.start_time) {
      sendParameters(toSaveSelections.start_time,
                     toSaveSelections.end_time,
                     do_not_store,
                     toSaveSelections.relative_date,
                     toSaveSelections.quick_filters);
    }
  }

  // upgrade from previous dateRange versions to new one.
  function upgradeDaterange(settings) {
      if (!settings || !settings.viz_type || !settings.viz_type.daterange ||
          !settings.viz_type.daterange.default_selections) {
        return;
      }

      const tmpDateRange = settings.viz_type.daterange;
      if (tmpDateRange.dateRangeType &&
          tmpDateRange.dateRangeType !== tmpDateRange.default_selections.dateRangeType) {
        tmpDateRange.default_selections.dateRangeType = tmpDateRange.dateRangeType;
        delete tmpDateRange.dateRangeType;
      }

      if (!tmpDateRange.default_selections.dateRangeType) {
        if (tmpDateRange.default_selections.quick_filters) {
          tmpDateRange.default_selections.dateRangeType = 'quick_filters';
        } else if (tmpDateRange.default_selections.relative_date) {
          tmpDateRange.default_selections.dateRangeType = 'relative';
        }
      }
    }

  daterange.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();
    return daterange;
  };

  daterange.viz_getDataRequest = function() {
    const base_field = daterange_base_field.value();
    if (!base_field) {
      return arc.data.request()
                     .request_type('ignore');
    }

    // Since max/min queries take a long time for solr, don't create a request to
    // fetch that data. Instead, draw the visual first, and then perform
    // two queries to fetch the min and max of the data if the filter is a fixed
    // date visual, and then setup the calendar widget.
    if (lib.isSolr()) {
      return arc.data.request()
                     .request_type('ignore');
    }

    const data_request = arc.data.request();
    let sb = lib.getSelectBuilder();
    let rb = lib.getRequestBuilder();

    rb.addAggregate(`min([${base_field}]) as "min"`);
    rb.addAggregate(`max([${base_field}]) as "max"`);
    sb.addProjection(`min([${base_field}]) as "min"`);
    sb.addProjection(`max([${base_field}]) as "max"`);

    sb = lib.finalizeSql(sb);

    const col1 = arc.data.column()
                         .colsource('base_shelf')
                         .coltype('timestamp')
                         .colproperty(arc.report.columnTypeEnum.DIMENSION);
    const col2 = arc.data.column()
                         .colsource('base_shelf')
                         .coltype('timestamp')
                         .colproperty(arc.report.columnTypeEnum.DIMENSION);
    data_request.add_column(col1)
                .add_column(col2);

    data_request.sql(sb.getSql());

    data_request.dsreq(rb.dsreq());
    data_request.finalize();
    return data_request;
  };

  const updateCalendar = () => {
    let startDate = min_timestamp;
    let endDate = max_timestamp;

    if (daterange_maxdate.isChecked) {
      startDate = max_timestamp;
    } else {
      if (toSaveSelections.start_time) {
        startDate = toSaveSelections.start_time;
      }

      if (toSaveSelections.end_time) {
        endDate = toSaveSelections.end_time;
      }
    }

    const $viz = $(`#${lib.chart_div()}`);

    const dp = $viz.find('.filter-date-range').data('daterangepicker');
    if (dp) {
      if (startDate && endDate) {
        dp.element.attr('disabled', true);
      }

      /*
       * moment.utc is preferred over moment(x).utc() because the goal
       * is to not change to local timezone if time is in utc
       * The 3 possible cases of time specification can be handled like so:
       *  moment.utc("2018-01-31 15:19:53+02:00").format('YYYY-MM-DD HH:mm:ss')
       *  "2018-01-31 13:19:53"
       *  moment.utc("2018-01-31 15:19:53Z").format('YYYY-MM-DD HH:mm:ss')
       *  "2018-01-31 15:19:53"
       *  moment.utc("2018-01-31 15:19:53").format('YYYY-MM-DD HH:mm:ss')
       *  "2018-01-31 15:19:53"
       */
      if (startDate) {
        dp.setStartDate(moment.utc(startDate));
      }

      dp.element.attr('disabled', false);

      if (endDate) {
        dp.setEndDate(moment.utc(endDate));
      }
    }
  };

  // The input here is a timestamp in any format that moment.utc() supports
  // This includes:
  //   YYYY-MM-DD HH:MM:SS
  //   YYYY-MM-DD HH:MM:SS+00:00
  //   YYYY-MM-DDTHH:MM:SS.sssZ
  function timestringToStartofday(ts) {
    let m = moment.utc(ts);
    if (!m.isValid()) {
      return undefined;
    }

    return m.format('YYYY-MM-DD');
  }

  /*
   * Set the data this object should use to draw out the visual
   */
  daterange.viz_setDataResult = function(result) {
    if (result.rows() && result.rows().length) {
      data_result = result;

      min_timestamp = timestringToStartofday(data_result.rows()[0][0]);
      max_timestamp = timestringToStartofday(data_result.rows()[0][1]);
    }

    return daterange;
  };

  daterange.viz_processRefresh = function() {
    if (toSaveDateRangeType === 'quick_filters') {
      sendQuickFilterParams(/* name of the btn */ null, true);
      return true;
    }

    return false;
  };

  daterange.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw(true /* filter_widget */);

    const $viz = $(`#${viz_div}`);

    setupVisualElements($viz);

    let title = daterange_title.value();
    if (!title) {
      title = daterange_base_field.value();
    }

    $viz.find('.base_field').text(title);
    return daterange;
  };

  daterange.viz_resetControl = function(clearSelections) {
    // For dateranges, clearSelections should always be true.
    // Added condition to match with arc_visual_picklist code.
    if (clearSelections) {
      toSaveSelections = undefined;
      toSaveDateRangeType = undefined;
    }

    if (!data_result) {
      return daterange;
    }

    toSaveSelections = toSaveSelections || $.extend(true, {}, default_selections);
    toSaveDateRangeType = toSaveDateRangeType || defaultDateRangeType;

    return daterange;
  };

  function getRanges() {
    var range_name = daterange_ranges.value();
    if (!range_name) {
      return arc.misc.defaultRanges(daterange_showtime.isChecked, toSaveDateRangeType);
    }

    var r;
    var drl = arc.dateranges.get();
    if (drl) {
      drl.some(function(dr) {
        if (dr.name === range_name) {
          r = arc.misc.rangesFromText(dr.value);
          if (r) {
            return true;
          }
        }

        return false;
      });
    }

    if (!r) {
      return arc.misc.defaultRanges(daterange_showtime.isChecked, toSaveDateRangeType);
    }

    return r;
  }

  function setupVisualElements($viz) {
    if (lib.in_dashboard()) {
      $viz.append($('#daterange-template .selected-control').html());
      $viz.addClass('selected-control-daterange');
      let w = daterange_width.value();
      arc.misc.setWidth($viz, w);
    } else {
      $viz.append(
        '<div>' +
          '<ul class="daterange-container">' +
            $('#daterange-template').html() +
          '</ul>' +
        '</div>'
      );
    }

    toggleDropdownItems($viz);
    setupPicklistTypeHandlers($viz);

    switch (toSaveDateRangeType) {
      case 'fixed':
        drawFixed($viz);
        break;
      case 'relative':
        drawRelative($viz);
        break;
      case 'quick_filters':
        drawQuickFilters($viz);
        break;
      default:
        return;
    }

    if (!(lib.build_mode() && lib.in_dashboard())) {
      $viz.find('.widget-btns-wrapper').hide();
    }

    $viz.find('.widget-btns-wrapper a').tooltip({
      placement: 'top',
      container: 'body',
      delay: {
        show: 500,
        hide: 100,
      },
    });

    updateAppScopeVisibility();
    return daterange;
  }

  function updateAppScopeVisibility() {
    const $appScopeContainer = $(`#${lib.chart_div()} .filter-triangle`);
    const showAppScopeTriangle = arc.globals.is_edit &&
                                 arc.report.isAppScopeFilter(daterange.viz_lib());
    $appScopeContainer.toggleClass('arc-invisible', !showAppScopeTriangle);
  }

  daterange.filterSettings = function(_) {
    if (!arguments.length) return filterSettings;
    filterSettings = _;
    return daterange;
  };

  daterange.viz_configure = function() {
    // Get dateranges from site-specific settings
    daterange_ranges.names([])
                    .labels([]);

    let default_dr = 'default';
    let default_found;
    let drl = arc.dateranges.get();
    drl.forEach(function(dr) {
      if (dr.name === default_dr) {
        default_found = true;
      }
    });

    if (!default_found) {
      daterange_ranges.add_name('default')
                      .add_label('default');
    }

    drl.forEach(function(dr) {
      daterange_ranges.add_name(dr.name)
                      .add_label(dr.name);
    });

    lib.display_settings();
  };

  daterange.removeAllParams = function() {
    let paramConfig = {
      outputParameter: getOutputParameter(),
      isIgnore: true,
    };
    let paramObj = new arc.params.Param(lib, paramConfig);
    arc.params.sendParameters({
      paramObjects: [paramObj],
      lib,
    });
  };

  function sendParameters(start_time, end_time, do_not_store, dataForRelative,
      dataForQuick, do_not_refresh) {
    // Temp params object
    const params = {
      start_time,
      end_time,
      includeNulls: daterange_allownulls.isChecked &&
        toSaveSelections.includeNulls,
      dateRangeType: toSaveDateRangeType,
    };

    if (dataForRelative) {
      params.relative_date = dataForRelative;
    }

    if (dataForQuick) {
      params.quick_filters = dataForQuick;
    }

    if (!do_not_store) {
      if (!arc.misc.isIdentical(params, default_selections)) {
        // Selections NOT same as default, update toSaveSelections
        toSaveSelections = $.extend(true, {}, params);
        if (lib.build_mode()) {
          lib.page_needs_save();
        }
      }
    }

    if (lib.isSolr() || lib.isElasticsearch()) {
      params.start_time = arc.misc.dateToMoment(params.start_time).format(fullTextTimeFormat);
      params.end_time = arc.misc.dateToMoment(params.end_time).format(fullTextTimeFormat);

      if (params.quick_filters) {
        params.quick_filters.datetimeFormat = fullTextTimeFormat;
      }

      if (params.relative_date) {
        params.relative_date.start_time = arc.misc.dateToMoment(params.relative_date.start_time)
                                                  .format(fullTextTimeFormat);
        params.relative_date.end_time = arc.misc.dateToMoment(params.relative_date.end_time)
                                                .format(fullTextTimeFormat);
        params.relative_date.datetimeFormat = fullTextTimeFormat;
      }
    }

    let paramConfig = {
      outputParameter: getOutputParameter(),
      hideExtension: shouldEmitOnlyRootParamName(params),
      includeNulls: params.includeNulls,
      relativeDate: params.relative_date,
      quickFilters: params.quick_filters,
      value: [
        {
          start: params.start_time,
          end: params.end_time,
        },
      ],
    };

    let paramObj = new arc.params.DateRangeParam(lib, paramConfig);

    arc.params.sendParameters({
      paramObjects: [paramObj],
      do_not_refresh,
      lib,
    });

    return daterange;
  }

  function shouldEmitOnlyRootParamName(params) {
     /*
     * When there are conflicting params in play i.e. other visuals are emitting
     * 'param' to clash with our outputting 'param.start' and 'param.end', then
     * we emit 'param' in the case where .start is the same as .end.
     * 1. Why do we do this? So that the back controls work on a page. Otherwise,
     *    emitting 'param' leads to a second history element that emits 'param.start'
     *    and 'param.end'. Then, hitting back goes back to 'param', which then emits
     *    'param.start' and 'param.end' and we never can move back.
     * 2. Why not always do this i.e. why only do it when hasConflictingParams is
     *    set? If other visuals on the page are using params that look like
     *    param2 >= '<<param.start:>>', then it is very difficult to handle the
     *    case where daterange filters output 'param.start' in some cases and
     *    'param' in other cases. So, only make this happen in the scenario where
     *    it is necessary.
     */
    return (hasConflictingParams && params &&
      (params.start_time === params.end_time) &&
      (!params.quick_filters) &&
      (!params.relative_date));
  }

  function getCurrentSelections() {
    return $.extend(true, {}, toSaveSelections || default_selections || {});
  }

  function updateInputWithDateRange($input, currentSelections) {
    let st = '', et = '';
    if (currentSelections.start_time) {
      st = arc.misc.dateToMoment(currentSelections.start_time).format(timeformat);
    }

    if (currentSelections.end_time) {
      et = arc.misc.dateToMoment(currentSelections.end_time).format(timeformat);
    }

    $input.val(`${st} - ${et}`);
  }

  function drawRelative($viz) {
    let currentSelections = getCurrentSelections(),
        currentSelectionsRelative = currentSelections.relative_date;

    let $container = $viz.find('.date-range-input');

    arc.filter.helpers.drawRelative({
      isDropdown: true,
      $container,
      currentSelectionsRelative,
      timeformat,
      showtime: daterange_showtime.isChecked,
      container_div_id: lib.chart_div(),
      timezone: timezoneSetting.value(),
      applyCb: function(data) {
        toSaveSelections = {
          start_time: data.startDate,
          end_time: data.endDate,
          relative_date: $.extend(true, {}, data),
        };
        lib.page_needs_save();

        var sd, ed;
        if (data.startDate) {
          sd = arc.misc.dateToMoment(data.startDate).format(timeformat);
        }

        if (data.endDate) {
          ed = arc.misc.dateToMoment(data.endDate).format(timeformat);
        }

        // if no date is chosen, better to leave it blank then to default to today's date

        sendParameters(sd, ed, true, data);
      },
    });

    if (currentSelections.relative_date) {
      arc.filter.helpers.restoreUISelections($viz,
          currentSelections.relative_date, timeformat);
      updateInputWithDateRange($viz.find('.relative-date-range'), currentSelections);
      sendParameters(currentSelections.start_time,
                     currentSelections.end_time,
                     /* do_not_store */ false,
                     currentSelections.relative_date,
                     currentSelections.quick_filters);
    }
  }

  /*
  The default_selection is restored from the saved data. Sometimes it can show outdated information
  (date which in normal circumstances wouldn't be an option to select).
  This corrects it.
   */
  function corrigateDefaultSelectionWithFreshData(opts) {
    const { maxDate, minDate } = opts;
    if (default_selections.dateRangeType === 'fixed') {
      if (new Date(default_selections.start_time) < new Date(minDate)) {
        default_selections.start_time = minDate;
      }

      if (new Date(default_selections.end_time) > new Date(maxDate)) {
        default_selections.end_time = maxDate;
      }
    }
  }

  function drawFixed($viz) {
    const currentSelections = getCurrentSelections();
    let startDate = min_timestamp;
    let endDate = max_timestamp;

    $viz.find('.date-range-input').html(
      '<input type="text" class="form-control form-control-sm filter-date-range" ' +
      'placeholder="click to select range">'
    );

    if (daterange_maxdate.isChecked) {
      startDate = max_timestamp;
    } else {
      if (currentSelections.start_time) {
        startDate = currentSelections.start_time;
      }

      if (currentSelections.end_time) {
        endDate = currentSelections.end_time;
      }
    }

    if (startDate && endDate &&
        (!currentSelections.start_time || !currentSelections.end_time)) {
      // If we picked up some parameter from min/max of the date, then send
      // that information out to other visuals in this app
      // TODO: Add proper indication to user about how the new defaults have been identified
      // Currently, it is very confusing for user and appears to be a bug instead.
      // TODO: Also, provide a facility to disable this restriction of being bound to data limits
      currentSelections.start_time = startDate;
      currentSelections.end_time = endDate;
      console.log(
        'Since no default values for date filter were set but data is limited from ' +
        `${moment(startDate).format(timeformat)} to ${moment(endDate).format(timeformat)}, ` +
        'new defaults were set accordingly.');
    }

    if (currentSelections.start_time && currentSelections.end_time) {
      sendParameters(
        arc.misc.dateToMoment(currentSelections.start_time).format(timeformat),
        arc.misc.dateToMoment(currentSelections.end_time).format(timeformat),
        true);
    }

    updateInputWithDateRange($viz.find('.filter-date-range'), currentSelections);

    if (delayLoad) {
      const buildFixedDatePicker = function() {
        delayLoad = false;
        startDate = min_timestamp;
        endDate = max_timestamp;
        createDatePicker();
        toggleDropdownItems($viz);
      };

      Promise.all(buildSolrMinMaxPromises())
             .then(buildFixedDatePicker)
             .catch(buildFixedDatePicker);
    } else {
      createDatePicker();
    }

    function createDatePicker() {
      const opts = {
        timePickerIncrement: 5,
        applyClass: 'btn-primary',
        startDate,
        endDate,
        ranges: getRanges(),
        parentEl: $viz,
      };

      if (daterange_showtime.isChecked) {
        opts.timePicker = true;
        opts.format = timeFormatDateTime;
      } else {
        opts.format = timeFormatDate;
      }

      if (min_timestamp) {
        opts.minDate = min_timestamp;
      }

      if (max_timestamp) {
        opts.maxDate = max_timestamp;
      }

      if (lib.build_mode()) {
        // We arbitrarily set max/min date as 1 year from now if no max/min date comes from data
        // TODO: Either fix this, or make it configurable and inform user about what's going on.
        // TODO: If we chose to set it, set it for relative dates as well.
        // In Edit mode, user should be able to set any date.
        // In View mode, user should be restricted to min/max dates.
        // Its confusing to see, but somehow the way code executes, it works as expected.

        var todayDate = new Date();
        var calculatedMaxDate = new Date(opts.maxDate);
        var lastYearDate = new Date(todayDate);
        lastYearDate.setFullYear(lastYearDate.getFullYear() - 1);
        var calculatedMinDate = new Date(opts.minDate);
        var nextYearDate = new Date(todayDate);
        nextYearDate.setFullYear(nextYearDate.getFullYear() + 1);
        if (nextYearDate.getTime() >= calculatedMaxDate.getTime()) {
          opts.maxDate = nextYearDate;
        }

        if (lastYearDate.getTime() <= calculatedMinDate.getTime()) {
          opts.minDate = lastYearDate;
        }
      }

      corrigateDefaultSelectionWithFreshData(opts);
      $viz.find('.filter-date-range').daterangepicker(opts);
      updateCalendar();

      $viz.find('.filter-date-range')
          .off('change.raw')
          .on('change.raw', function changeFilterDateRange() {
            var d = arc.misc.daterangeToMinMax($(this).val(), timeformat);
            sendParameters(d.min, d.max);
          });

      $viz.find('.filter-date-range')
          .off('showCalendar.daterangepicker')
          .on('showCalendar.daterangepicker', updateCalendarWidth);

      $viz.find('.filter-date-range')
          .off('show.daterangepicker')
          .on('show.daterangepicker', updateCalendarWidth);
    }

    function updateCalendarWidth() {
      if (arguments[1].chosenLabel === 'Custom Range') {
        arguments[1].container.css('width', '665px');
      } else {
        arguments[1].container.css('width', 'auto');
      }
    }
  }

 function getMinMaxSolrRb(minMax) {
    const rb = lib.getRequestBuilder();
    const base_field = daterange_base_field.value();
    const order = (minMax === 'min') ? { asc: true } : { asc: false };

    rb.datasetId(lib.dataset().dataset_id());
    rb.requestType('SOLR');
    rb.dimensions([{ type: 'SIMPLE', expr: `[${base_field}]`, order: order }]);
    rb.limit(1);

    return rb;
  }

  function buildSolrMinMaxPromises() {
    return [
      new Promise(function(resolve, reject) {
        arc.data.runQueryParams({
          rb: getMinMaxSolrRb('max'),

          successCb: function(data) {
            if (data.rows && data.rows[0]) {
              max_timestamp = data.rows[0][0];
            } else {
              max_timestamp = '2099-12-31';
            }

            max_timestamp = timestringToStartofday(max_timestamp);

            resolve(data);
          },

          failCb: function(error) {
            reject(error);
          },

          lib,

          loader: lib && lib.loader(),
        });
      }),

      new Promise(function(resolve, reject) {
        arc.data.runQuery(getMinMaxSolrRb('min'),
          function minSuccess(data) {
            if (data.rows && data.rows[0]) {
              min_timestamp = data.rows[0][0];
            } else {
              min_timestamp = '1900-01-01';
            }

            min_timestamp = timestringToStartofday(min_timestamp);
            resolve(data);
          },

          function minFail(error) {
            reject(error);
          }, lib);
      }),
    ];
  }

  function drawQuickFilters($viz) {
    const ranges = getRanges();
    $viz.find('.date-range-input').html(arc.dateranges.getQuickFilterHTML(ranges));
    $quickFilterBtns = $viz.find('.btn-group');
    $quickFilterBtns.on('click', '.btn-secondary', function() {
      const name = $(this).val();
      activateQuickFilterButton(name);
      sendQuickFilterParams(name);
    });

    activateQuickFilterButton();
    sendQuickFilterParams(name);
  }

  function activateQuickFilterButton(name) {
    name = name || getBtnNameFromParams();
    $quickFilterBtns.find('.btn-secondary').removeClass('active');

    if (name) {
      $quickFilterBtns.find(`button[value="${name}"]`).addClass('active');
    }
  }

  function getBtnNameFromParams() {
    let currentSelections = getCurrentSelections();
    if (currentSelections.quick_filters) {
      return currentSelections.quick_filters.name;
    }

    return undefined;
  }

  function sendQuickFilterParams(name, do_not_refresh) {
    name = name || getBtnNameFromParams();

    if (!name) {
      return daterange;
    }

    const btnFound = $quickFilterBtns.find(`button[value="${name}"]`).length > 0;

    if (btnFound) {
      const data = getQuickFilterParams(name);
      const selection = {
        start_time: data.startDate,
        end_time: data.endDate,
        quick_filters: $.extend(true, {}, data),
      };

      $.extend(true, toSaveSelections, selection);

      lib.page_needs_save();

      sendParameters(
          arc.misc.dateToMoment(data.startDate, timezoneSetting.value()).format(timeformat),
          arc.misc.dateToMoment(data.endDate, timezoneSetting.value()).format(timeformat),
          true,
          null,
          data,
          do_not_refresh
      );
    }
  }

  function getQuickFilterParams(name) {
    const range = getRanges()[name];
    const data = { name };

    if (range) {
      data.startDate = range[0];
      data.endDate = range[1];
    }

    return data;
  }

  function setupPicklistTypeHandlers($viz) {
    $viz.find('.calendar-relative-picked')
        .off('click.openRelative')
        .on('click.openRelative', function pickRelative() {
          lib.page_needs_save();
          toSaveDateRangeType = 'relative';
          toggleDropdownItems($viz);
          drawRelative($viz);
          setTimeout(() => {
            $viz.find('.relative-date-range').trigger('click');
          }, 0);
        });

    $viz.find('.calendar-fixed-picked')
        .off('click.openFixed')
        .on('click.openFixed', function pickFixed() {
          lib.page_needs_save();
          toSaveDateRangeType = 'fixed';
          toggleDropdownItems($viz);
          drawFixed($viz);
          setTimeout(() => {
            $viz.find('.filter-date-range').trigger('click');
          }, 0);
        });

    $viz.find('.quick-filters-picked')
        .off('click.openQuickFilters')
        .on('click.openQuickFilters', function pickQuickFilters() {
          lib.page_needs_save();
          toSaveDateRangeType = 'quick_filters';
          toggleDropdownItems($viz);
          drawQuickFilters($viz);
        });

    $viz.find('.include-nulls-checked')
        .off('click.includeNulls')
        .on('click.includeNulls', function toggleIncludeNulls() {
          lib.page_needs_save();
          daterange.updateToSaveSelectionsFromParams();
          toSaveSelections.includeNulls = !toSaveSelections.includeNulls;
          toggleDropdownItems($viz);
          publishParameters(true);
        });
  }

  function toggleDropdownItems($viz) {
    setDelayLoad();

    $viz.find('.calendar-fixed-picked .fa')
        .toggleClass('icon-invisible', toSaveDateRangeType !== 'fixed');
    $viz.find('.calendar-relative-picked .fa')
        .toggleClass('icon-invisible', toSaveDateRangeType !== 'relative');
    $viz.find('.quick-filters-picked .fa')
        .toggleClass('icon-invisible', toSaveDateRangeType !== 'quick_filters');

    $viz.find('.dates-load')
        .toggleClass('arc-invisible', toSaveDateRangeType !== 'fixed' || !delayLoad);
    $viz.find('.calendar-picker-icon .dropdown-toggle')
        .toggleClass('disabled', toSaveDateRangeType === 'fixed' && delayLoad);

    $viz.find('.fixed-dates')
        .toggleClass('arc-invisible', delayLoad || toSaveDateRangeType !== 'fixed');
    $viz.find('.relative-dates')
        .toggleClass('arc-invisible', delayLoad || toSaveDateRangeType !== 'relative');
    $viz.find('.quick-filters')
        .toggleClass('arc-invisible', delayLoad || toSaveDateRangeType !== 'quick_filters');

    $viz.toggleClass('arc-daterange-type-fixed', toSaveDateRangeType === 'fixed');
    $viz.toggleClass('arc-daterange-type-relative', toSaveDateRangeType === 'relative');
    $viz.toggleClass('arc-daterange-type-quick_filters', toSaveDateRangeType === 'quick_filters');

    $viz.find('.include-nulls-divider').toggleClass('arc-invisible',
                                                    !daterange_allownulls.isChecked);
    $viz.find('.include-nulls-checked').toggleClass('arc-invisible',
                                                    !daterange_allownulls.isChecked);
    $viz.find('.include-nulls-checked .fa').toggleClass('icon-invisible', !toSaveSelections.includeNulls);
  }

  function setDelayLoad() {
    delayLoad = lib.isSolr() && toSaveDateRangeType === 'fixed' &&
      (!min_timestamp || !max_timestamp);
  }

  daterange.dateRangeType = function() {
    return toSaveDateRangeType;
  };

  daterange.base_field = function(_) {
    if (!arguments.length) return daterange.daterange_base_field.value();
    daterange.daterange_base_field.value(_);
    return daterange;
  };

  return daterange;
};

arc.visual.plugins.daterange_extension = function() {
  var ext = function() { return 'arc.visual.plugins.daterange_extension'; };

  ext.viz_name = function() {
    return 'daterange';
  };

  ext.viz_tooltip = function() {
    return 'daterange';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_daterange.png?v=1';
  };

  ext.viz = arc.visual.plugins.daterange;

  return ext;
};

arc.visual.extensions = arc.visual.extensions || [];
arc.visual.extensions.push(arc.visual.plugins.daterange_extension());

}());
