window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.dendrogram = function() {
  var dendrogram = arc.visual.plugins.chord('dendrogram');

  function findChild(curr, name) {
    var n;

    curr.children.some(function(c) {
      if (c.name === name) {
        n = c;
        return true;
      }

      return false;
    });

    return n;
  }

  dendrogram.viz_drawVisualBundle = function(config, viz_div) {
    const lib = dendrogram.viz_lib();

    if (config.rows.length === 0) {
      return;
    }

    var nl = dendrogram.nodesAndLinks().process(config);

    var root = {
      parent: null,
      children: [],
      depth: 0,
      name: '',
    };

    var depthCount = [];
    for (var i = config.dim_startidx; i <= config.dim_endidx; ++i) {
      depthCount.push(0);
    }

    let sourcesHash = {},
        targetsHash = {};

    config.rows.forEach(function(d) {
      let curr = root;
      for (let i = config.dim_startidx; i <= config.dim_endidx; ++i) {
        const depth = i - config.dim_startidx,
              name = d[i];
        let n = findChild(curr, name);
        if (n) {
          curr = n;
          n.agg += Number(d[config.agg_startidx]);
          n.tooltips = _assembleTooltips(d, n);

          continue;
        }

        n = {
          parent: curr,
          children: [],
          depth: depth + 1,
          name: name,
          agg: Number(d[config.agg_startidx]),
          tooltips: _assembleTooltips(d),
        };

        if (depth === 0) {
          sourcesHash[name] = n;
        }

        if (i === config.dim_endidx) {
          targetsHash[name] = n;
        }

        ++(depthCount[depth]);
        curr.children.push(n);
        curr = n;
      }
    });

    function _assembleTooltips(row, node) {
      let tooltipVals = [];
      if (node) {
        tooltipVals = node.tooltips.map((val, idx) => {
          if ($.isNumeric(val) && $.isNumeric(row[config.tooltipCols[idx].colindex()])) {
              val = Number(val) + Number(row[config.tooltipCols[idx].colindex()]);
            }

          return val;
        });
      } else {
        config.tooltipCols.forEach(function(tooltipCol) {
            tooltipVals.push(row[tooltipCol.colindex()]);
        });
      }

      return tooltipVals;
    }

    var leftTextMargin = 0,
        rightTextMargin = 0;
    let colFormats = config.columns.map(c => c.coldisplayformat());

    if (dendrogram.showNodeLabels()) {
      leftTextMargin = lib.dim_axis_max_textlen(
        Object.keys(sourcesHash).map(colFormats[config.dim_startidx]), 200,
        lib.getLabelFontSize());
      rightTextMargin = lib.dim_axis_max_textlen(
        Object.keys(targetsHash).map(colFormats[config.dim_endidx]), 200,
        lib.getLabelFontSize());
    }

    var nodeRadius = 5;
    var textWidth = 15;
    var height = Math.max(config.height, d3.max(depthCount) * textWidth);
    var margin = {
      top: nodeRadius,
      left: nodeRadius + leftTextMargin + 5,
      bottom: nodeRadius,
      right: nodeRadius + rightTextMargin + 5,
    };

    var c = d3.layout.cluster()
                     .separation(function separation(a, b) {
                       return a.parent === b.parent ? 20 : 30;
                     })
                     .size([
                             height - margin.top - margin.bottom,
                             config.width - margin.left - margin.right,
                           ]);

    var n = c.nodes(root);
    n = n.filter(function(d) {
      return d.parent !== null;
    });

    var p = d3.scale.linear()
              .domain(d3.extent(n, function(d) {
                return d.y;
              }))
              .range([
                       0,
                       config.width - margin.left - margin.right,
                     ]);

    n.forEach(function(d) {
      d.y = p(d.y);
    });

    var l = c.links(n);

    var svg = d3.select('#' + viz_div).append('svg')
                .attr('width', config.width)
                .attr('height', height)
                  .append('g')
                  .attr('transform', 'translate(' + margin.left + ',' + margin.top + ')')
                  .attr('class', 'viz-dendrogram');

    var nodeColorScale = arc.misc.dimensionColorScale();
    nodeColorScale.range(lib.colors());

    var lineLayout = d3.svg.diagonal()
                       .projection(function(d) {
                         return [d.y, d.x];
                       });

    var linkExtent = d3.extent(config.rows, function(d) {
      if (config.agg_startidx) {
        d.size = Number(d[config.agg_startidx]);
      } else {
        d.size = 2;
      }

      return d.size;
    });

    var linkScale = d3.scale.linear()
                     .domain(linkExtent)
                     .range(dendrogram.linkWidthRange());

    function aggGetter(d) {
      return d.target.agg;
    }

    var linkColorScale = dendrogram.getLinkColorScale({
      nodeColorScale: nodeColorScale,

      sourceDimGetter: function(d) {
        return d.source.name;
      },

      targetDimGetter: function(d) {
        return d.target.name;
      },

      aggGetter: aggGetter,

      aggExtent: d3.extent(l, function(d) {
                   return aggGetter(d);
                 }),
    });

    var o = dendrogram.linkOpacity();
    var linkMarks =
      svg.selectAll('.nosel')
         .data(l)
         .enter()
           .append('path')
           .attr('fill', 'none')
           .attr('stroke-width', function(d) {
             return linkScale(aggGetter(d));
           })
           .attr('stroke-opacity', o)
           .attr('stroke', function(d) {
             return linkColorScale(d);
           })
           .attr('d', function(d) {
             var p = lineLayout(d);
             return p;
           });

    function getLinkKVs(d) {
      const sourceColInd = d.source.depth + config.dim_startidx - 1;
      const targetColInd = d.target.depth + config.dim_startidx - 1;
      let nodeArray = [
        {
          key: config.columns[sourceColInd].colname(),
          value: colFormats[sourceColInd](d.source.name),
        },
        {
          key: config.columns[targetColInd].colname(),
          value: colFormats[targetColInd](d.target.name),
        },
        {
          key: config.columns[config.agg_startidx].colname(),
          value: colFormats[config.agg_startidx](d.target.agg),
        },
      ];

      d.target.tooltips.forEach(function(tooltipVal, i) {
        nodeArray.push({
          key: config.tooltipCols[i].colname(),
          value: colFormats[config.tooltipCols[i].colindex()](tooltipVal),
        });
      });

      return nodeArray;
    }

    arc.marks()
       .viz_lib(lib)
       .marker_name('links')
       .highlight_on_select(true)
       .addSelection(linkMarks)
       .enableTooltipHighlights(false)
       .text_fn(function(d) {
         return lib.text_from_columns([], [], {
           pre_kvs: getLinkKVs(d),
         });
       })
       .params_fn(function(d, i, e, dimsOnly) {
         let rowCol = getRowCol(d, true);
         return lib.params_from_columns(rowCol.row, rowCol.columns, dimsOnly);
       })
       .activate();

    o = lib.colorOpacity();
    var nodeMarks =
      svg.selectAll('.nosel')
         .data(n)
         .enter()
           .append('circle')
           .attr('cx', function(d) {
             return d.y;
           })
           .attr('cy', function(d) {
             return d.x;
           })
           .attr('r', function(d) {
             return nodeRadius;
           })
           .style('fill-opacity', o)
           .style('fill', function(d) {
             return nodeColorScale(d.name);
           });

    if (dendrogram.showNodeLabels()) {
      var lastDepth = config.dim_endidx - config.dim_startidx + 1;

      svg.selectAll('.nosel')
         .data(n)
         .enter()
           .append('text')
           .attr('x', function(d) {
             if (d.depth === lastDepth) {
               return d.y + nodeRadius + 2;
             }

             if (d.depth === 1) {
               return d.y - nodeRadius - 2;
             }

             return d.y - nodeRadius + 6;
           })
           .attr('y', function(d) {
             if ((d.depth === 1) || (d.depth === lastDepth)) {
               return d.x + 4;
             }

             return d.x - nodeRadius - 2;
           })
           .text(function(d) {
             return colFormats[d.depth + config.dim_startidx - 1](d.name);
           })
           .attr('text-anchor', function(d) {
             if (d.depth === lastDepth) {
               return 'start';
             }

             return 'end';
           });
    }

    function getNodeKVs(d) {
      const nodeColInd = d.depth + config.dim_startidx - 1;
      let nodeArray = [
        {
          key: config.columns[nodeColInd].colname(),
          value: colFormats[nodeColInd](d.name),
        },
        {
          key: config.columns[config.agg_startidx].colname(),
          value: colFormats[config.agg_startidx](d.agg),
        },
      ];

      d.tooltips.forEach(function(tooltipVal, i) {
        nodeArray.push({
          key: config.tooltipCols[i].colname(),
          value: colFormats[config.tooltipCols[i].colindex()](tooltipVal),
        });
      });

      return nodeArray;
    }

    function getRowCol(d, isLink) {
      let nodeKVs = [];
      if (!isLink) {
        nodeKVs = [
          {
            value: d.name,
            colIdx: d.depth + config.dim_startidx - 1,
          },
        ];
      } else {
        nodeKVs = [
          {
            value: d.source.name,
            colIdx: d.source.depth + config.dim_startidx - 1,
          },
          {
            value: d.target.name,
            colIdx: d.target.depth + config.dim_startidx - 1,
          },
          {
            value: d.target.agg,
            colIdx: config.agg_startidx,
          },
        ];
      }

      let nodeRow = [];
      config.rows.some(function matchSelectedMarkWithAllRows(row) {
        if (!nodeKVs.some(function findNonMatchingCell(kv) {
          let val1 = row[kv.colIdx];
          val1 = isNaN(parseFloat(val1)) ? val1 : parseFloat(val1);
          let val2 = kv.value;
          val2 = isNaN(parseFloat(val2)) ? val2 : parseFloat(val2);
          return val1 !== val2;
        })
        ) {
          nodeRow = row;
          return true;
        }
      });

      let cols = [];
      nodeKVs.forEach(function(kv) {
        cols.push(config.columns[kv.colIdx]);
      });

      return {
        row: nodeRow,
        columns: cols,
      };
    }

    arc.marks()
       .viz_lib(lib)
       .marker_name('nodes')
       .highlight_on_select(true)
       .addSelection(nodeMarks)
       .text_fn(function(d) {
         return lib.text_from_columns([], [], {
           pre_kvs: getNodeKVs(d),
         });
       })
       .params_fn(function(d, i, e, dimsOnly) {
         let rowCol = getRowCol(d);
         return lib.params_from_columns(rowCol.row, rowCol.columns, dimsOnly);
       })
       .activate();
  };

  return dendrogram;
};

arc.visual.plugins.dendrogram_extension = function() {
  var ext = function() { return 'arc.visual.plugins.dendrogram_extension'; };

  ext.viz_name = function() {
    return 'dendrogram';
  };

  ext.viz_tooltip = function() {
    return 'dendrogram';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_dendrogram.png?v=1';
  };

  ext.viz = arc.visual.plugins.dendrogram;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.dendrogram_extension());

}());
