window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.gauge = function() {
  var saved_settings = {},
      data_result,
      lib,
      trellis,
      sort_orders_asc = {},
      numInvalidVals,
      numInvalidMainAgg,
      allowLargeTrellisPerm = false,
      allowLargeTrellisTemp = false,
      unifiedScale;

  var fgColor = arc.settings.gauge.gaugeFgColor(),
      bgColor = arc.settings.gauge.gaugeBgColor(),
      pointerColor = arc.settings.gauge.gaugePointerColor(),
      bgOpacity = arc.settings.gauge.backgroundOpacity(),
      projectionOpacity = arc.settings.bullet.projectionOpacity(),
      gaugeOuterRadius = arc.settings.gauge.gaugeOuterRadius(),
      gaugeArcThickness = arc.settings.gauge.gaugeArcThickness(),
      gaugeQualitativeArcThickness = arc.settings.gauge.gaugeQualitativeArcThickness(),
      gaugeQualitativeArcGap = arc.settings.gauge.gaugeQualitativeArcGap(),
      keyMeasureMark = arc.settings.gauge.keyMeasureMark(),
      showGaugeAxisTicks = arc.settings.gauge.showGaugeAxisTicks(),
      labelFontSize = arc.settings.gauge.labelFontSize(),
      sublabelFontSize = arc.settings.gauge.sublabelFontSize(),
      labelVerticalOffset = arc.settings.gauge.labelVerticalOffset(),
      sublabelVerticalOffset = arc.settings.gauge.sublabelVerticalOffset(),
      lowQualPercent = arc.settings.gauge.lowestQualitativePercentage(),
      mediumQualPercent = arc.settings.gauge.mediumQualitativePercentage(),
      highQualPercent = arc.settings.gauge.highestQualitativePercentage(),
      showQualitativeAsPercentages = arc.settings.gauge.showQualitativeAsPercentages(),
      scaleTrellisIndependently = arc.settings.gauge.scaleTrellisIndependently(),
      alwaysShowCompareLine = arc.settings.gauge.alwaysShowCompareLine(),
      includeOutsideCompareLine = arc.settings.gauge.includeOutsideCompareLine();
  showQualitativeAsPercentages.addDependents([lowQualPercent, mediumQualPercent, highQualPercent]);

  var user_settings =
      arc.visual.defaultSettings()
                .addSetting([
                  arc.colorpalette.setting(),
                  fgColor,
                  pointerColor,
                  bgColor,
                  projectionOpacity,
                  bgOpacity,
                  keyMeasureMark,
                  alwaysShowCompareLine,
                  includeOutsideCompareLine,
                  showGaugeAxisTicks,
                  gaugeOuterRadius,
                  gaugeQualitativeArcThickness,
                  gaugeQualitativeArcGap,
                  gaugeArcThickness,
                  labelFontSize,
                  labelVerticalOffset,
                  sublabelFontSize,
                  sublabelVerticalOffset,
                  showQualitativeAsPercentages,
                  lowQualPercent,
                  mediumQualPercent,
                  highQualPercent,
                  scaleTrellisIndependently,
                ])
                .removeSetting([
                  'axis_label_font_size',
                  'label_font_size',
                  'label_color',
                  'legend_font_size',
                ]);

  var gauge = arc.visual.api();

  var shelf_data = [
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'compare_shelf',
      display_name: 'Compare To',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'qualitative_shelf',
      display_name: 'Qualitative Ranges',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'projection_shelf',
      display_name: 'Projection',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'label_shelf',
      display_name: 'Label',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  var constraints = {
    minimums: {
      aggregates_shelf: 1,
    },
    maximums: {
      aggregates_shelf: 1,
      compare_shelf: 1,
      projection_shelf: 1,
      label_shelf: 2,
    },
  };

  gauge.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return gauge;
  };

  gauge.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  gauge.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return gauge;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return gauge;
  };

  gauge.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    let xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        measureMessage = [
          lib.createPopoverMessages('aggregateSingle'),
          lib.createPopoverMessages('featureMeasure'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        compareToMessage = [
          lib.createPopoverMessages('compareTo'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        qualitativeRangeMessage = [
          lib.createPopoverMessages('qualitativeRangesGauge'),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        projectionMessage = [
          lib.createPopoverMessages('projection'),
          lib.createPopoverMessages('acceptsOne'),
        ],
        labelMessage = [
          lib.createPopoverMessages('label'),
          lib.createPopoverMessages('acceptsAtMostTwo'),
        ];


    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      aggregates_shelf: measureMessage,
      compare_shelf: compareToMessage,
      qualitative_shelf: qualitativeRangeMessage,
      projection_shelf: projectionMessage,
      label_shelf: labelMessage,
    });

    return gauge;
  };

  gauge.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  gauge.viz_setDataResult = function(result) {
    data_result = result;
    return gauge;
  };

  function computeUnifiedScale(rows, labelCols, tooltipCols) {
    let maxCol = rows[0].length;
    if (labelCols.length > 0) {
      maxCol = labelCols[0].colindex();
    } else if (tooltipCols.length > 0) {
      maxCol = tooltipCols[0].colindex();
    }

    let max = 0, min = 0;
    rows.forEach(function(row) {
      row.forEach(function(datum, idx) {
        if ($.isNumeric(datum) && idx < maxCol) {
          max = Math.max(max, datum);
          min = Math.min(min, datum);
        }
      });
    });
    let unifiedScale = {};
    unifiedScale.min = min;
    unifiedScale.max = max;
    return unifiedScale;
  }

  gauge.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    {
      let sb_rows = data_result.rows();
      if (!sb_rows || sb_rows.length === 0) {
        return gauge;
      }
    }

    let config = {
      rows: data_result.rows(),
      columns: data_result.columns(),
    };

    config.agg_col = lib.first_column(config.columns, 'aggregates_shelf');
    config.projection_col = lib.first_column(config.columns, 'projection_shelf');
    config.compare_col = lib.first_column(config.columns, 'compare_shelf');
    config.agg_idx = config.agg_col.colindex();
    config.qualitative_cols = lib.shelf_columns(config.columns, 'qualitative_shelf');
    config.label_cols = lib.shelf_columns(config.columns, 'label_shelf');
    config.tooltip_cols = lib.shelf_columns(config.columns, 'tooltip_shelf');

    // Filter rows with NULLs in the aggregate columns
    let agg_getter = function(row) {
      return row[config.agg_idx];
    };

    function makeUnifiedScale(numRows, scaleIndependently) {
      return numRows > 1 && scaleIndependently;
    }

    if (makeUnifiedScale(config.rows.length, !scaleTrellisIndependently.isChecked)) {
      unifiedScale = computeUnifiedScale(config.rows, config.label_cols, config.tooltip_cols);
    } else {
      unifiedScale = undefined;
    }

    lib.filter_nulls(config, [agg_getter]);
    lib.unselectOnClick();

    trellis = arc.visual.trellis()
                     .x_shelf_name('x_shelf')
                     .y_shelf_name('y_shelf')
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select(`#${viz_div}`))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       gauge.allowLargeTrellisTemp(true);
                       gauge.viz_drawVisual(viz_div);
                     })
                     .otherTrellisSettings([scaleTrellisIndependently])
                     .setup();

    if (trellis.error_state()) {
      return gauge;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      gauge.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');

    numInvalidMainAgg = 0;
    numInvalidVals = 0;
    arc.report.logAlertMessages('clear-small');
    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      config.rows = xy_rows;
      let cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_sel, viz_div, cell_div);
    });
    if (numInvalidVals > 0 && arc.globals.is_edit) {
      let message = 'Ignoring ' + numInvalidVals + ' invalid values.';
      arc.report.alertMessageSmall(message, 'alert-warning');
    }

    if (numInvalidMainAgg > 0 && arc.globals.is_edit) {
      let message = 'Unable to render ' + numInvalidMainAgg + ' graph(s) due to invalid measure.';
      arc.report.alertMessageSmall(message, 'alert-warning');
    }
  };

  function isValid(val) {
    if (val !== undefined && !$.isNumeric(val)) {
      numInvalidVals++;
      return false;
    }
    return true;
  }

  function computeMinMax(agg, compare, agg_projection, qualitative_data, unifiedScale) {
    let domainMin, domainMax;
    if (unifiedScale) {
      domainMin = unifiedScale.min;
      domainMax = unifiedScale.max;
    } else {
      domainMin = Math.min(0, agg);
      domainMax = Math.max(0, agg);

      if (compare !== undefined) {
        domainMax = Math.max(domainMax, compare);
        domainMin = Math.min(domainMin, compare);
      }

      if (agg_projection !== undefined) {
        domainMax = Math.max(domainMax, agg_projection);
        domainMin = Math.min(domainMin, agg_projection);
      }

      if (qualitative_data.length > 0) {
        domainMax = Math.max(domainMax, qualitative_data[qualitative_data.length -1]);
        domainMin = Math.min(domainMin, qualitative_data[0]);
      }
    }
    return [domainMin, domainMax];
  }

  function drawCompareLine(compare, domainMax) {
    return compare!== undefined && (compare < domainMax || alwaysShowCompareLine.isChecked);
  }

  function viz_drawVisualWork(config, cell_sel, viz_div, cell_div) {
    if (config.rows.length === 0) {
      return;
    }

    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
    });

    let data = config.rows[0];
    let agg = data[config.agg_col.colindex()];
    let compare;

    if (config.compare_col) {
      compare = data[config.compare_col.colindex()];
    }

    let agg_projection;
    if (config.projection_col) {
      agg_projection = data[config.projection_col.colindex()];
    }

    let labelText,
        sublabelText;

    if (config.label_cols.length > 0) {
      labelText = data[config.label_cols[0].colindex()];
      labelText = config.label_cols[0].formatForDisplay(labelText);

      if (config.label_cols.length > 1) {
        sublabelText = data[config.label_cols[1].colindex()];
        sublabelText = config.label_cols[1].formatForDisplay(sublabelText);
      } else {
        sublabelText = '';
      }
    }

    let qualitative_data = config.qualitative_cols.map(function(c) {
      return data[c.colindex()];
    });


    if (!$.isNumeric(agg)) {
      numInvalidMainAgg++;
      return;
    }

    if (!isValid(compare)) {
      compare = undefined;
    }

    if (!isValid(agg_projection)) {
      agg_projection = undefined;
    }

    let labelContainerStyle = 'viz-gauge-label',
        sublabelContainerStyle = 'viz-gauge-sublabel';

    let qualitativeArcThickness = Number(gaugeQualitativeArcThickness.value());
    let qualitativeArcGap = Number(gaugeQualitativeArcGap.value());

    let makePercentageQualitative = showQualitativeAsPercentages.isChecked;
    if (qualitative_data.length < 1 && makePercentageQualitative && compare !== undefined) {
      const compareNum = Number(compare);
      qualitative_data.push(compareNum * lowQualPercent.value() / 100);
      qualitative_data.push(compareNum * mediumQualPercent.value() / 100);
      qualitative_data.push(compareNum * highQualPercent.value() / 100);
      qualitative_data.sort((a, b) => a - b);
    } else if (qualitative_data.length === 0) {
      qualitativeArcThickness = 0;
      qualitativeArcGap = 0;
    } else {
      qualitative_data = qualitative_data.filter(isValid);
      qualitative_data.sort((a, b) => a - b);
    }

    let arcRange = computeMinMax(agg, compare, agg_projection, qualitative_data, unifiedScale);

    let arcScale = d3.scale.linear()
                           .range([-0.5 * Math.PI, 0.5 * Math.PI])
                           .domain(arcRange);

    let domainMin = arcRange[0];
    let domainMax = arcRange[1];
    let maxTickLabelLength = 0;
    let arcTickScale;
    let tickData;
    let tickFormatter;

    if (showGaugeAxisTicks.isChecked) {
      // Determine how much space should be added to the width to accommodate the axis tick marks
      // and labels

      arcTickScale = d3.scale.linear()
                           .domain([domainMin, domainMax])
                           .range([-Math.PI, 0]);
      tickData = arcScale.ticks(5)
                             .map(function(v) {
                               return {
                                 num: v,
                                 tickScale: arcTickScale(v),
                               };
                             });

      if (config.compare_col) {
        tickFormatter = config.compare_col.coldisplayformat();
      } else {
        tickFormatter = config.agg_col.coldisplayformat();
      }

      maxTickLabelLength = lib.dim_axis_max_textlen(tickData.map(v => tickFormatter(v.num)));
    }

    let outerRadius = gaugeOuterRadius.get_range(),
        minWidth = outerRadius[0],
        maxWidth = outerRadius[1],
        margin = {
          left: 30 + maxTickLabelLength,
          right: 30 + maxTickLabelLength,
          top: 20,
          bottom: 20,
        },
        w = Math.max(Math.min(elem_w - margin.left - margin.right, maxWidth), minWidth),
        radius = w / 2,
        totalWidth = w + margin.left + margin.right,
        h = radius,
        totalHeight = h + margin.top + margin.bottom,
        arcThickness = Number(gaugeArcThickness.value());

    // Everything goes within a div
    let container =
      d3.selectAll('#' + cell_div)
        .selectAll('.nosel')
        .data([data])
        .enter()
          .append('div')
          .classed('viz-gauge-container', true)
          .style('width', totalWidth + 'px')
          .style('height', totalHeight + 'px');

    let svg =
      container.append('svg')
               .style('width', (margin.left + margin.right + w) + 'px')
               .style('height', (margin.top + margin.bottom + h) + 'px');

    let arcg = svg.append('g')
                  .attr('transform', 'translate(' + (margin.left + radius) + ',' + (margin.top + radius) + ')');

    // Draw the background

    function drawLineMark(location, onQualitativeArc, lineType) {
      if (lineType === 'NEEDLE') {
        let arcPolarScale = d3.scale.linear()
                            .domain([domainMin, domainMax])
                            .range([-90, 90]);

        let cr = radius,
            theta = arcPolarScale(location);

        let pointerW = 4,
            pointerPath = [
          [0, 0],
          [-pointerW, -pointerW],
          [0, -radius],
          [pointerW, -pointerW],
          [0, 0],
        ];

        let pointerG = arcg.append('g');
        pointerG.append('path')
            .attr('d', function() {
              return d3.svg.line()(pointerPath);
            })
            .attr('fill', pointerColor.value())
            .attr('transform', 'rotate(' + arcPolarScale(agg) + ')');
      } else {
        let lineMarkExtension = 2;
        let lineMarkThickness = 2;
        let innerRadius = radius - qualitativeArcThickness - lineMarkExtension;
        let outerRadius = radius + lineMarkExtension;
        if (!onQualitativeArc) {
          innerRadius = innerRadius - qualitativeArcGap - arcThickness;
        }

        if (!onQualitativeArc) {
          outerRadius = outerRadius - qualitativeArcThickness - qualitativeArcGap;
        }
        arcg.append('path')
        .attr('fill', pointerColor.value())
        .attr('d',
          d3.svg.arc()
          .innerRadius(innerRadius)
          .outerRadius(outerRadius)
          .startAngle(arcScale(Number(location)) - (lineMarkThickness * Math.PI / 360))
          .endAngle(arcScale(Number(location)) + (lineMarkThickness * Math.PI / 360)));
      }

    }


    // Draw the metric, projection, and background arcs (if requested)
    if (arcThickness > 0) {

      //Draw background
      arcg.append('path')
          .attr('fill', bgColor.value())
          .attr('d',
              d3.svg.arc()
                .innerRadius(radius - qualitativeArcThickness - qualitativeArcGap - arcThickness)
                .outerRadius(radius - qualitativeArcThickness - qualitativeArcGap)
                .startAngle(arcScale(domainMin))
                .endAngle(arcScale(domainMax)));

      //feature Measure
      arcg.append('path')
          .attr('fill', fgColor.value())
          .attr('d',
              d3.svg.arc()
                .innerRadius(radius - qualitativeArcThickness - qualitativeArcGap - arcThickness)
                .outerRadius(radius - qualitativeArcThickness - qualitativeArcGap)
                .startAngle( arcScale(0) )
                .endAngle(arcScale(agg)));

      if (agg_projection !== undefined) {
        arcg.append('path')
            .attr('fill', fgColor.value())
            .attr('fill-opacity', Number(projectionOpacity.value() / 100))
            .attr('d',
                d3.svg.arc()
                  .innerRadius(radius - qualitativeArcThickness - qualitativeArcGap - arcThickness)
                  .outerRadius(radius - qualitativeArcThickness - qualitativeArcGap)
                  .startAngle(arcScale(agg))
                  .endAngle(arcScale(agg_projection)));
      }

      if (drawCompareLine(compare, domainMax)) {
        drawLineMark(compare);
      }

      if (keyMeasureMark.value() === 'LINE') {
        drawLineMark(agg, false);
      }
    }

    // Draw the metric, projection, and background arcs (if requested)
    if (qualitative_data.length > 0 && qualitativeArcThickness > 0) {
      if (domainMax > qualitative_data[qualitative_data.length - 1]) {
        qualitative_data.push(domainMax);
      }

      if (domainMin < qualitative_data[0]) {
        qualitative_data.unshift(domainMin);
      }

      let colors = lib.colors();
      qualitative_data.forEach(function(q, i) {
        if (i < qualitative_data.length - 1) {
          arcg.append('path')
          .attr('fill', colors[i % colors.length])
          .attr('d',
            d3.svg.arc()
              .innerRadius(radius - qualitativeArcThickness)
              .outerRadius(radius)
              .startAngle(arcScale(qualitative_data[i]))
              .endAngle(arcScale(qualitative_data[i + 1])));
        }
      });

      if (includeOutsideCompareLine.isChecked && compare !== undefined && compare < domainMax) {
        drawLineMark(compare, true);
      }

    }

    // Add in a gauge line (if requested)
    if (keyMeasureMark.value() === 'NEEDLE') {
      drawLineMark(agg, false, 'NEEDLE');
    }

    // Display requested labels
    let textg = svg.append('g')
                   .attr('transform', 'translate(' + (margin.left) + ',' + (margin.top) + ')');

    if (labelText) {
      let _labelFontSize = Number(labelFontSize.value());
      textg.append('text')
           .attr('x', radius)
           .attr('y', radius * 3 / 4 - _labelFontSize / 2 + (Number(labelVerticalOffset.value())))
           .attr('font-size', _labelFontSize)
           .attr('text-anchor', 'middle')
           .text(labelText);

      if (sublabelText) {
        let _sublabelFontSize = Number(sublabelFontSize.value());
        textg.append('text')
             .attr('x', radius)
             .attr('y', radius - _sublabelFontSize / 2 + (Number(sublabelVerticalOffset.value())))
             .attr('font-size', _sublabelFontSize)
             .attr('text-anchor', 'middle')
             .text(sublabelText);
      }
    }

    if (showGaugeAxisTicks.isChecked) {
      let arcAxis =
          arcg.append('g')
              .classed('arc-viz-axis', true);

      arcAxis.selectAll('.nosel')
          .data(tickData)
          .enter()
            .append('text')
            .attr('x', function(d) {
              return (radius + 4) * Math.cos(d.tickScale);
            })
            .attr('y', function(d) {
              return (radius + 4) * Math.sin(d.tickScale);
            })
            .attr('text-anchor', function(d) {
              if (d.tickScale > -(Math.PI / 2)) {
                return 'start';
              } else {
                return 'end';
              }
            })
            .text(function(d) {
              return lib.formattedDisplay(tickFormatter, d.num);
            });

      arcAxis.selectAll('.nosel')
          .data(tickData)
          .enter()
            .append('line')
            .attr('x1', function(d) {
              return (radius) * Math.cos(d.tickScale);
            })
            .attr('y1', function(d) {
              return (radius) * Math.sin(d.tickScale);
            })
            .attr('x2', function(d) {
              return (radius + 3) * Math.cos(d.tickScale);
            })
            .attr('y2', function(d) {
              return (radius + 3) * Math.sin(d.tickScale);
            })
            .style('stroke', 'black')
            .style('stroke-width', '1');
    }

    let nonlabelShelves =
        config.columns.filter(function(c) {
          return c.colsource() !== 'label_shelf';
        });

    let marks =
      arc.marks()
         .viz_lib(lib)
         .marker_name('primary')
         .highlight_on_select(true)
         .addSelection(arcg.selectAll('path,circle,line'))
         .addSelection(textg.selectAll('text'))
         .text_fn(function(d, i) {
            return lib.text_from_columns(data, nonlabelShelves);
          })
         .params_fn(function(d, i, e, dimsOnly) {
            // NOTE: data is intentionally passed. Do not pass d.
            return lib.params_from_columns(data, nonlabelShelves, dimsOnly);
          });

    marks.activate();

    return gauge;
  }

  gauge.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return gauge;
  };

  gauge.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return gauge;
  };

  return gauge;
};

arc.visual.plugins.gauge_extension = function() {
  let ext = function() { return 'arc.visual.plugins.gauge_extension'; };

  ext.viz_name = function() {
    return 'gauge';
  };

  ext.viz_tooltip = function() {
    return 'gauge';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_gauge.png?v=1';
  };

  ext.viz = arc.visual.plugins.gauge;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.gauge_extension());

}());
