window.arc = window.arc || {};

(function() {
"use strict";

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.groupedbars = function() {
  var saved_settings = {},
      data_result,
      formatPercent = d3.format(".0%"),
      formatPercentTwo = d3.format(".2%"),
      lib;

  const legendReverse = arc.report.user_settings.reverse_legend_order();
  const legendWidthOverRide = arc.report.user_settings.legend_width_override();
  const legendStyle = arc.report.user_settings.legend_style();
  const horizontal_gridlines_enabled = arc.report.user_settings.horizontal_gridlines_enabled();
  const vertical_gridlines_enabled = arc.report.user_settings._vertical_gridlines_enabled();
  const setting_scale = arc.report.user_settings.scale_percentage();

  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  legendStyle,
                                  legendWidthOverRide,
                                  legendReverse,
                                  setting_scale,
                                  horizontal_gridlines_enabled,
                                  vertical_gridlines_enabled,
                                  arc.report.user_settings.fixed_color_legend(),
                                  arc.colormap.dimToColorMappingSetting(),
                                ]);

  var groupedbars = arc.visual.api();

  groupedbars.is_percentage = function() {
    return (setting_scale.value() === "PERCENTAGE");
  };

  groupedbars.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.user_settings(user_settings);
    return groupedbars;
  };

  groupedbars.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    saved_settings = settings;
    return settings;
  };

  groupedbars.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return groupedbars;
    }
    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    return groupedbars;
  };

  groupedbars.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();
    return groupedbars;
  };

  groupedbars.viz_getDataRequest = function() {
    var constraints = {
      minimums: {
        "dimensions_shelf": 1,
        "aggregates_shelf": 1
      },
      maximums: {
        "dimensions_shelf": 2
      }
    };

    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  groupedbars.viz_setDataResult = function(result) {
    data_result = result;
    return groupedbars;
  };

  groupedbars.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    var config = {};
    config.rows = data_result.rows();
    if (config.rows.length === 0) {
      return groupedbars;
    }

    lib.unselectOnClick();

    config.x_idx = 0;
    config.color_idx = 1;
    config.y_idx = 2;
    config.columns = data_result.columns();
    config.y_column = config.columns[config.y_idx];
    config.dim_getter = function(d) {
      return d[config.x_idx];
    };
    config.agg_getter = function(d) {
      return Number(d[config.y_idx]);
    };
    config.color_getter = function(d) {
      return d[config.color_idx];
    };

    groupedbars.organize_cells(config);

    var unique_groups = arc.misc.arrayUnique(config.rows, function(d) {
      return config.color_getter(d);
    });

    config.color = lib.dimensionalColorScale();
    config.color.domain(unique_groups);

    var x_singlewidth_min = 12,
        x_singlewidth_max = 30,
        x_minw = x_singlewidth_min * config.total_slots,
        x_maxw = x_singlewidth_max * config.total_slots;

    var x_tick_textlen = lib.dim_axis_max_textlen(unique_groups,
        lib.xaxis_label_maxlen(), base.getAxisTicksFontSize());

    var r = lib.compute_svg_params({
      viz_div_elem: $("#" + viz_div),
      columns: config.columns,
      x_idx: config.x_idx,
      legend_idx: config.color_idx,
      min_width: x_minw,
      max_width: x_maxw,
      rows: config.rows,
      extra_margin: {bottom: x_tick_textlen}
    });

    config.margin = r.margin;
    config.width = r.width;
    config.height = r.height;

    config.dim_sz = Math.floor(config.width / config.total_slots);

    var y_colname = config.y_column.colname();
    if (groupedbars.is_percentage()) {
      var percentage_colname = 'percentage';
      lib.compute_percentages(config.rows, percentage_colname,
          config.dim_getter, config.agg_getter);
      config.agg_getter = function(d) {
        return d[percentage_colname];
      };
    }

    config.x_scale = lib.idx_to_scale_config({
      rows: config.rows,
      idx: config.x_idx,
      range: [0, config.width],
    });

    config.y_scale = lib.idx_to_scale_config({
      rows: config.rows,
      idx: config.y_idx,
      range: [config.height, 0],
    });

    var y_domain = config.y_scale.domain();
    if (y_domain[0] > 0) {
      y_domain[0] = 0;
      config.y_scale.domain(y_domain);
    }

    var x_axis = arc.visual.axis()
                           .scale(config.x_scale)
                           .is_dim(!lib.idx_is_numeric(config.rows, config.x_idx))
                           .colname(config.columns[config.x_idx].colname());
    var y_axis = arc.visual.axis()
                           .scale(config.y_scale)
                           .is_dim(false)
                           .colname(y_colname);
    if (groupedbars.is_percentage()) {
      y_axis.tickFormat(formatPercent);
    }

    config.svg = groupedbars.create_svgs(viz_div, config.width, config.height, config.margin);
    lib.draw_axes(config.svg, config.width, config.height, config.margin,
                  {
                    y_axis_data: y_axis,
                    include_gridlines: (horizontal_gridlines_enabled.isChecked),
                    x_is_dim: config.x_is_dim,
                    y_is_dim: config.y_is_dim
                  });

    groupedbars.draw_ticks(config);
    groupedbars.draw_marks(config);

    if (legendStyle.legendRequired()) {
      lib.draw_legend({
        svg: config.svg,
        width: config.width,
        height: config.height,
        color: config.color,
        sort_domain:true,
      });
    }

    return groupedbars;
  };

  groupedbars.organize_cells = function(config) {
    var x_getter = function() {
      return [];
    };
    var y_getter = function() {
      return [];
    };

    // Hierarchize the data by trellis rows
    config.trellis_cell_groups = arc.misc.array_to_groups(config.rows,
        function(d) {
          return [x_getter(d), y_getter(d)];
        });

    // Now hierarchize groups within each cell
    config.trellis_cell_groups.forEach(function(cell_group) {
      // Compute the groups
      cell_group.x_groups = arc.misc.array_to_groups(cell_group.data,
          config.dim_getter);

      // Compute the start and end indices for each group
      var last_count = 0;
      var max_idx = 0;
      cell_group.x_groups.forEach(function(g, i) {
        g.group_idx_start = last_count;
        last_count += g.data.length;
        g.group_idx_end = last_count - 1;
      });

      // Compute the maximum bar count for each cell
      cell_group.cell_bar_max = 1 +
          cell_group.x_groups[cell_group.x_groups.length-1].group_idx_end;

      cell_group.total_slots = cell_group.cell_bar_max +
          (cell_group.x_groups.length * 2);
    });
    config.total_slots = d3.max(config.trellis_cell_groups,
        function(cell_group) {
          return cell_group.total_slots;
        });
    return groupedbars;
  };

  groupedbars.draw_ticks = function(config) {
    config.trellis_cell_groups.forEach(function(cell_group) {
      groupedbars.draw_cell_ticks(config, cell_group.x_groups);
    });
    return groupedbars;
  };

  groupedbars.draw_cell_ticks = function(config, x_groups) {
    var tick_positions = [];
    var groups = [];
    x_groups.forEach(function(g, i) {
      tick_positions.push({
        x: config.dim_sz +
            config.dim_sz * ((g.group_idx_start + g.group_idx_end) / 2),
        groupname: g.group
      });
    });

    var ticks_g =
      config.svg.append("g")
                .attr("class", "x arc-viz-axis")
                .attr("transform",
                    "translate(" + 0 + "," + config.height + ")");

    ticks_g.selectAll(".nosel")
           .data(tick_positions)
           .enter()
             .append("line")
             .attr("x1", function(d) {
               return d.x;
             })
             .attr("y1", 0)
             .attr("x2", function(d) {
               return d.x;
             })
             .attr("y2", 6)
             .style("stroke", "gray")
             .style("stroke-width", 1);

    var ticks =
      ticks_g.selectAll(".nosel")
             .data(tick_positions)
             .enter()
               .append("g")
               .attr("transform", function(d) {
                   return "translate(" + (d.x+5) + ",14)";
               })
                 .append("text")
                 .attr("transform", function(d) {
                     return "rotate(" + lib.x_slant() + ",0,0)";
                 })
                 .style("text-anchor", "end")
                 .text(function(d) {
                   return d.groupname;
                 });

    return groupedbars;
  };

  groupedbars.draw_marks = function(config) {
    config.trellis_cell_groups.forEach(function(cell_group) {
      groupedbars.draw_cell_marks(config, cell_group.x_groups);
    });
    return groupedbars;
  };

  groupedbars.draw_cell_marks = function(config, x_groups) {
    x_groups.forEach(function(x_group) {
      config.svg.selectAll(".nosel")
         .data(x_group.data)
         .enter()
           .append("rect")
           .attr("x", function(d, i) {
             var x = config.dim_sz +
                (x_group.pos * 2 * config.dim_sz) +
                (x_group.group_idx_start + i) * config.dim_sz;
             return x;
           })
           .attr("y", function(d) {
             return config.y_scale(config.agg_getter(d));
           })
           .attr("width", config.dim_sz)
           .attr("height", function(d) {
             return config.height - config.y_scale(config.agg_getter(d));
           })
           .style("fill", function(d) {
             return config.color(config.color_getter(d));
           });
    });

    var m = config.svg.selectAll(".rect");
    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .addSelection(m)
       .text_fn(function(d, i) {
         var kvs = [];
         if (groupedbars.is_percentage()) {
           kvs.push([{
             key: 'Percentage',
             value: formatPercentTwo(d.percentage),
             value_is_num: true
           }]);
         }
         var tooltipConfig = {pre_kvs: kvs};
         return lib.text_from_columns(d, config.columns, tooltipConfig);
       })
       .params_fn(function(d, i, e, dimsOnly) {
          return lib.params_from_columns(d, config.columns, dimsOnly);
       })
       .activate();

    return groupedbars;
  };

  groupedbars.create_svgs = function(viz_div, width, height, margin) {
    d3.select("#" + viz_div)
        .append("svg")
        .classed("viz-bar-svg", true)
          .append("g")
          .classed("viz-bar-g", true);
    d3.select("#" + viz_div)
      .selectAll(".viz-bar-svg")
      .style("width", width + margin.left + margin.right + "px")
      .style("height", height + margin.top + margin.bottom + "px");
    d3.select("#" + viz_div)
      .selectAll(".viz-bar-g")
      .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    var sel = d3.select("#" + viz_div)
                .selectAll(".viz-bar-g");
    return sel;
  };

  return groupedbars;
};

arc.visual.plugins.groupedbars_extension = function() {
  var ext = function() { return "arc.visual.plugins.groupedbars_extension"; };

  ext.viz_name = function() {
    return "groupedbars";
  };

  ext.viz_tooltip = function() {
    return "grouped bars";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_groupedbars.png?v=1";
  };

  ext.viz = arc.visual.plugins.groupedbars;

  return ext;
};

// arc.visual.extensions.push(arc.visual.plugins.groupedbars_extension());

}());
