window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.histogram = function() {
  let saved_settings = {};
  let data_result;
  let lib;
  let sort_orders_asc = {};
  let allowLargeTrellisPerm = false;
  let allowLargeTrellisTemp = false;

  const bucketCount = arc.visual.setting();
  bucketCount.id('bucket_count')
             .tab('Basic')
             .type(arc.visual.settingTypeEnum.INTEGER)
             .add_name('BUCKET_COUNT')
             .add_label('Bucket count')
             .min(2)
             .max(100)
             .value('10');

const minOfIntervalSetting = arc.visual.setting();
minOfIntervalSetting.id('histogram_interval_min')
          .tab('Basic')
          .type(arc.visual.settingTypeEnum.INTEGER)
          .add_name('HISTOGRAM_INTERVAL_MIN')
          .info('The minimum value to include in the histogram')
          .add_label('Range - min');

const maxOfIntervalSetting = arc.visual.setting();
maxOfIntervalSetting.id('histogram_interval_max')
          .tab('Basic')
          .type(arc.visual.settingTypeEnum.INTEGER)
          .add_name('HISTOGRAM_INTERVAL_MAX')
          .info('The maximum value to include in the histogram')
          .add_label('Range - max');

  const normalized = arc.visual.setting();
  normalized.id('normalized')
            .tab('Basic')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('NORMALIZED')
            .add_label('Normalized histogram')
            .value('false');

  const cumulative = arc.visual.setting();
  cumulative.id('cumulative')
            .tab('Basic')
            .type(arc.visual.settingTypeEnum.CHECKBOX)
            .add_name('CUMULATIVE')
            .add_label('Cumulative histogram')
            .value('false');

  const scale_agg_independent = arc.report.user_settings.scale_agg_independent()
                                                        .labels(['Independent Y axis']);
  const scale_dim_independent = arc.report.user_settings.scale_dim_independent()
                                                        .labels(['Independent X axis']);
  const bar_mark_size = arc.report.user_settings.bar_mark_size();
  const space_between_histogram_bars = arc.visual.setting()
              .id('space_between_histogram_bars')
              .tab('Marks')
              .type(arc.visual.settingTypeEnum.CHECKBOX)
              .add_label('Separate columns')
              .value('false');
  const scale_agg_zero = arc.report.user_settings.scale_agg_zero();
  const show_y_axis = arc.settings.line.showYAxis();
  const show_x_axis = arc.settings.line.showXAxis();
  const dynamic_bar_width = arc.report.user_settings.dynamic_bar_width();
  const user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  show_y_axis,
                                  show_x_axis,
                                  arc.report.user_settings.x_axis_label(),
                                  arc.report.user_settings.y_axis_label(),
                                  dynamic_bar_width,
                                  arc.settings.bar.inlineMeasureDisplay(),
                                  bucketCount,
                                  minOfIntervalSetting,
                                  maxOfIntervalSetting,
                                  normalized,
                                  cumulative,
                                  scale_agg_independent,
                                  scale_dim_independent,
                                  bar_mark_size,
                                  space_between_histogram_bars,
                                  scale_agg_zero,
                                ])
                                .removeSetting([
                                  'label_font_size',
                                  'label_color',
                                  'legend_font_size',
                                ]);

  user_settings.removeSetting('visual_show_detail_data');
  const histogram = arc.visual.api();

  const shelf_data = [
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',
      shelf_type: arc.report.shelf_types.COLUMN,

      // Note that we don't want to apply a metric here, we
      // just want a numeric column. So represent this as
      // a dimension
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  const constraints = {
    minimums: {
      aggregates_shelf: 1,
    },
    maximums: {
      aggregates_shelf: 1,
    },
  };

  histogram.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return histogram;
  };

  histogram.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  histogram.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return histogram;
    }

    lib.removeSimpleAggregations(settings);

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return histogram;
  };

  histogram.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    const xMessage = [
      lib.createPopoverMessages(['trellisOnly', 'columns.']),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const yMessage = [
      lib.createPopoverMessages(['trellisOnly', 'rows.']),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const aggregateMessages = [
      lib.createPopoverMessages('aggregateHistogram'),
      lib.createPopoverMessages('acceptsOne'),
    ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      aggregates_shelf: aggregateMessages,
    });

    return histogram;
  };

  function minAgg(colname) {
    return `min(${colname}) as 'minVal'`;
  }

  function maxAgg(colname) {
    return `max(${colname}) as 'maxVal'`;
  }

  let histogramCol;

  const histogramDimensionIndex = 0;
  let trellisingDims;

  let masterHistogramConfig;

  // Run the few things necessary to commit a data request:
  //   Associate it with the lib object,
  //   point dsreq within the dataRequest object directly to rb().dsreq()
  //   and call 'finalize' on it
  function commitDataRequest(dataRequest) {
    const rb = dataRequest.rb();
    const dsreq_obj = rb.dsreq();

    lib.dsreq(dsreq_obj);
    dataRequest.dsreq(dsreq_obj);
    dataRequest.finalize();
  }

  function validHistogramCol(col, dim) {
    // Check for whether this is a valid histogram column
    const colName = col.colname();

    if (dim.type !== 'SIMPLE') {
      return `Invalid histogram measure "${colName}"`;
    }

    if (arc.report.containsAggregate(dim.expr, lib.dataconnection_type())) {
      return `Histogram measure "${dim.expr}" cannot contain an aggregate`;
    }

    return undefined;
  }

  histogram.viz_getDataRequest = function() {
    const dsreq = lib.getCheckedDataRequest(constraints);
    if (!dsreq) {
      return dsreq;
    }

    const rb = dsreq.rb();
    const cols = dsreq.columns();
    const dims = rb.dimensions();

    histogramCol = cols.filter((col) => col.colsource() === 'aggregates_shelf')[0];
    if (cols.length !== dims.length) {
      lib.alertMessage('Histogram metric cannot contain an aggregate');
      lib.forceReloadData();
      return arc.data.request()
                .request_type('ignore');
    }

    const err = validHistogramCol(histogramCol, dims[histogramCol.colindex()]);
    if (err) {
      lib.alertMessage(`Invalid histogram column: ${err}`);
      lib.forceReloadData();
      return arc.data.request()
                .request_type('ignore');
    }

    // First, we need to get the min and max metric
    // If the x-axes are independent, we need to issue a SQL command to get the
    // max/min aggregate for each dimension. Otherwise, remove all dimensions.
    trellisingDims = dims.slice(1, dims.length);
    if (scale_dim_independent.isChecked) {
      rb.dimensions(trellisingDims);
    } else {
      rb.dimensions([]);
    }

    const substitutedColname = lib.substituteParams(histogramCol.expr());

    // ...and add in the two aggregates
    rb.addAggregate(minAgg(substitutedColname));
    rb.addAggregate(maxAgg(substitutedColname));

    commitDataRequest(dsreq);

    // Create a structure for the new column
    dsreq.add_column(histogramCol.cloneDataColumn());

    masterHistogramConfig = {};

    return dsreq;
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  histogram.viz_setDataResult = function(result) {
    data_result = result;
    return histogram;
  };

  /*
  * When the min and max is equal, in the histogram only one bar should be displayed,
  * however the bucketSize would be zero because their equality.
  * To encounter this issue, in that scenario this method sets the min max
  * as the end of a symmetrical interval estimated by the exponential form.
  */
  const determineMinMaxBasedOnTheirEquality = (min, max) => {
    if (min !== max) {
      return { min, max };
    }

    const exponential = max.toExponential().split('e')[1];
    const regex = /\+|-/;
    const sign = exponential.match(regex)[0];
    const exponent = exponential.split(sign)[1];
    const halfInterval = parseFloat(`1e${sign}${exponent}`) / 2;
    return { min: min - halfInterval, max: max + halfInterval };
  };

  function getBucketConfig(min, max) {
    const config = {};
        const numBuckets = Number(bucketCount.value());

    const { min: calcMin, max: calMax } = determineMinMaxBasedOnTheirEquality(min, max);
    config.bucketSize = (calMax - calcMin) / numBuckets;
    config.bucketMin = calcMin;
    config.bucketMax = calMax;
    config.scaleDomain = [
      config.bucketMin,
      config.bucketMax,
    ];
    /*
    space_between_histogram_bars is not a normal bar setting, related to the histogram visual only,
    This visual is specially configured a bar visual,
    with custom features and settings and even workflow.
    Reach space_between_histogram_bars value via the config.
    */
    config.spaceBetweenHistogramBars = Number(space_between_histogram_bars.value() === 'true');
    return config;
  }

  function bucketToCentroid(bucketId, config) {
    return (bucketId * config.bucketSize) + config.bucketMin +
      (config.bucketSize / 2);
  }

  function constructDataRequest(row) {
    const minFromSetting =
      parseFloat(minOfIntervalSetting.value());
    const maxFromSetting =
      parseFloat(maxOfIntervalSetting.value());

    const userDefinedMinIsHigherThanMax =
      !Number.isNaN(minFromSetting) && !Number.isNaN(maxFromSetting) &&
      minFromSetting > maxFromSetting;

    let min = userDefinedMinIsHigherThanMax ?
      Number(row[histogramDimensionIndex]) :
      (minFromSetting || Number(row[histogramDimensionIndex]));
    let max = userDefinedMinIsHigherThanMax ?
      Number(row[histogramDimensionIndex + 1]) :
      maxFromSetting || Number(row[histogramDimensionIndex + 1]);

    const dsreq = lib.getCheckedDataRequest(constraints);
    const rb = dsreq.rb();
    const cols = dsreq.columns();
    // NOTE: Histogram visual is already a little different than other visuals.
    // It gets the result in 2 separate steps, first min-max after that the buckets.
    // During DatasetProfiling we know min-max value beforehand, I decided to avoid to re-request.
    // viz_getDataRequest (which gets the min-max) was the method which initialized histogramCol.
    // In case Data Profiling is necessary to (re)init histogramCol as well trellisingDims.
    histogramCol = cols.filter((col) => col.colsource() === 'aggregates_shelf')[0];
    const c = histogramCol.expr();

    // Note: positioned here because getCheckedDataRequest will eventually removes all the
    // notifications added previously.
    if (userDefinedMinIsHigherThanMax) {
        arc.report.alertMessageSmallBase({
          type: 'alert-warning',
          alertMessage: `The minimum interval setting is higher than the maximum setting,
          it will not be applied! Check your histogram settings to fix it.`,
          fadeOut: true,
        });

        lib.alertMessage(`The minimum interval setting is higher than the maximum setting,
          it was not applied! Check your histogram settings to fix it.`);
    } else {
      if (!Number.isNaN(minFromSetting)) {
        rb.filters(`(${c} >= ${min})`);
      }

      if (!Number.isNaN(maxFromSetting)) {
        rb.filters(`(${c} <= ${max})`);
      }
    }

    let dims = rb.dimensions();

    // Now, create columns in order that's expected by bar visual
    // With the "aggregate_shelf" element last
    const newCols = cols.slice(1);
    newCols.push(histogramCol);
    newCols.forEach((col, i) => col.colindex(i));
    dsreq.columns(newCols);
    const config = getBucketConfig(min, max);
    let startingObj = masterHistogramConfig;
    trellisingDims = dims.slice(1, dims.length);
    const numDims = trellisingDims.length;
    if (numDims && scale_dim_independent.isChecked) {
      trellisingDims.forEach((d, dimIndex) => {
        if (dimIndex === (numDims - 1)) {
          startingObj[row[dimIndex]] = config;
        } else {
          if (!startingObj[row[dimIndex]]) {
            startingObj[row[dimIndex]] = {};
          }

          startingObj = startingObj[row[dimIndex]];
        }

        rb.filters(`( ${cols[dimIndex + 1].expr()} )  = ${row[dimIndex]}`);
      });
    } else {
      masterHistogramConfig = config;
    }

    const substitutedColname = lib.substituteParams(histogramCol.expr());
    const dctype = lib.dataset().dataconnection_type();
    const dccast = arc.dataconnection.types[dctype]().cast_bigint;

    let expr_inner = `((${substitutedColname} - ${config.bucketMin}) / ${config.bucketSize})`;

    // TODO: This logic should be moved to a connection 'cast_expr_to_bigint' function
    if (dctype === 'bigquery') {
      expr_inner = `floor(${expr_inner})`;
    }

    const expr = `cast(${expr_inner} as ${dccast}) as 'Buckets'`;
    if (c !== histogramCol.expr()) {
      // Something changed between the last call for min/max and this
      // call
      console.log('ignoring, column changed');
      return undefined;
    }

    // The last dim needs to be converted into the histogram measure
    // Remove the last dimension...
    dims = dims.slice(1, dims.length);
    rb.dimensions(dims);
    rb.addDimension(expr);

    // And add in a sum(1) measure
    rb.addAggregate('sum(1) as \'Record Count\'');

    commitDataRequest(dsreq);

    // Create a structure for the new column
    const aggCol = histogramCol.cloneDataColumn()
                             .coltype('BIGINT')
                             .colname('Frequency')
                             .colsource('y_shelf')
                             .colproperty(arc.report.columnTypeEnum.AGGREGATE);
    dsreq.add_column(aggCol);

    let tooltipDisplayFormat;
    let axisDisplayFormat;
    if (histogramCol.coldisplayformat(true)) {
      tooltipDisplayFormat = histogramCol.coldisplayformat();
      axisDisplayFormat = histogramCol.coldisplayformat();
    } else {
      tooltipDisplayFormat = lib.translatedDisplayFormat('.4s');
      axisDisplayFormat = lib.translatedDisplayFormat('s');
    }

    const lastBucketMax = tooltipDisplayFormat(config.bucketMax);

    function displayFormat(v, isForTooltip) {
      // This call is made for axis labels, and for tooltips.
      // When displaying a tooltip, we want to display the range.
      // When displaying an axis labels, we want to display the actual value

      if (isForTooltip === true) {
        const minVal = tooltipDisplayFormat(v - config.bucketSize / 2);
        const maxVal = tooltipDisplayFormat(v + config.bucketSize / 2);
        return `${minVal} ≤ x ${lastBucketMax === maxVal ? '≤' : '<'} ${maxVal}`;
      }

      return axisDisplayFormat(v);
    }

    histogramCol.colsource('x_shelf').setDisplayFormat(displayFormat);

    if (normalized.isChecked) {
      aggCol.setDisplayFormat(lib.translatedDisplayFormat('.2f'));
    } else {
      aggCol.setDisplayFormat(lib.translatedDisplayFormat(','));
    }

    return dsreq;
  }

  function normalizeData(inputRows) {
    let rows = inputRows.slice(0);
    const numBuckets = Number(bucketCount.value());

    if (rows.length === 0) {
      return rows;
    }

    const l = rows[0].length;
    const bucketIdx = l - 2;
    const aggIdx = l - 1;

    function nullGetter(row) {
      return row[bucketIdx];
    }

    rows = lib.filter_nulls(
        {
          rows,
        },
        [
          nullGetter,
        ]
);

    rows.forEach((row) => {
      row[bucketIdx] = Number(row[bucketIdx]);
      row[aggIdx] = Number(row[aggIdx]);
    });

    // Sort from smallest to largest bucket
    function sortRows(unsortedRows) {
      unsortedRows.sort((a, b) => {
        if (a[bucketIdx] < b[bucketIdx]) {
          return -1;
        }

        if (a[bucketIdx] > b[bucketIdx]) {
          return 1;
        }

        return 0;
      });
    }

    sortRows(rows);

    function consolidateRows(cellRows) {
      /*
        Consolidate the last bucket if it is outside the range

        The use-case here is this:
        Metric values: 6, 6, 7, 8, 9, 10, 11, 12, 12, 12
        Bucket Count: 7
        Bucket Size: (12 - 6) / (7) = 0.85714285714
        Buckets:
                  6.0 - 6.85714285714
                  6.85714285714 - 7.71428571429
                  7.71428571429 - 8.57142857143
                  8.57142857143 - 9.42857142857
                  9.42857142857 - 10.2857142857
                  10.2857142857 - 11.1428571429
                  11.1428571429 - 12.0

        Query: select cast((metric - 6) / (1) as integer), sum(1)
               from t
               group by 1;

        Response:
               1, 2
               2, 1
               3, 1
               4, 1
               5, 1
               6, 1
               7, 0
               8, 3

        Note that we got data for a 8th bucket even though the number
        of buckets is 7. That is because the max value of the metric
        shows up in the last+1 bucket.

        In js below, we consolidate the last and second-last buckets.

       */
      if (cellRows.length <= 1) {
        return cellRows;
      }

      const lastRow = cellRows[cellRows.length - 1];
      const lastButOneRow = cellRows[cellRows.length - 2];

      if (lastRow[bucketIdx] === numBuckets) {
        if (lastButOneRow[bucketIdx] === numBuckets - 1) {
          lastButOneRow[aggIdx] += lastRow[aggIdx];
          cellRows = cellRows.slice(0, cellRows.length - 1);
        } else {
          lastRow[bucketIdx] = numBuckets - 1;
        }
      }

      return cellRows;
    }

    const columnCount = rows[0] ? rows[0].length : 0;
    const trellisCells = {};

    if (columnCount === 2) {
      // No trellising is in play
      trellisCells.base = rows.slice(0);
    } else {
      // We are trellising on X or Y. Separate out the rows for normalization
      // and cumulation
      rows.forEach((row) => {
        const trellisData = row.slice(0, columnCount - 2);
        if (trellisData in trellisCells) {
          trellisCells[trellisData].push(row);
        } else {
          trellisCells[trellisData] = [row];
        }
      });
    }

    const newRows = [];
    Object.keys(trellisCells).forEach((k) => {
      let cellRows = trellisCells[k];
      cellRows = consolidateRows(cellRows);
      trellisCells[k] = cellRows;

      if (cumulative.isChecked) {
        const bucketToRow = {};
        cellRows.forEach((row) => {
          bucketToRow[row[bucketIdx]] = row;
        });

        const missingRows = [];
        for (let i = 0; i < numBuckets; ++i) {
          if (!(i in bucketToRow)) {
            const templateRow = cellRows[0].slice(0, cellRows[0].length);
            templateRow[bucketIdx] = i;
            templateRow[aggIdx] = 0;
            missingRows.push(templateRow);
          }
        }

        missingRows.forEach((row) => {
          cellRows.push(row);
        });

        sortRows(cellRows);
      }

      let config = masterHistogramConfig;

      // Setup the X axis with bucket centroids
      cellRows.forEach((row) => {
        if (scale_dim_independent.isChecked && trellisingDims.length) {
          let startingObj = masterHistogramConfig;
          const numDims = trellisingDims.length;
          trellisingDims.forEach((d, dimIndex) => {
            if (dimIndex === (numDims - 1)) {
              config = startingObj[row[dimIndex]];
            } else {
              startingObj = startingObj[row[dimIndex]];
            }
          });
        }

        row[bucketIdx] = bucketToCentroid(row[bucketIdx], config);
      });

      let total;
      if (normalized.isChecked) {
        total = cellRows.reduce((previousValue, row) => previousValue + row[aggIdx], 0.0);
      }

      let cumulativeCount = 0;
      cellRows.forEach((row) => {
        if (normalized.isChecked) {
          row[aggIdx] /= total;
        }

        if (cumulative.isChecked) {
          cumulativeCount += row[aggIdx];
          row[aggIdx] = cumulativeCount;
        }

        newRows.push(row);
      });
    });

    return newRows;
  }

  function drawHistogram(rows, columns) {
    /*
     * TODO: Substitute a hand-constructed column name. Ideally, this should
     *   happen in base.shelvesToDataRequest, but that is too risky a change
     *   for the release this fix goes into.
     */
    columns.map((col) => {
      const colname = col.colname();
      return col.colname(lib.substituteParams(colname));
    });

    data_result = arc.data.result()
                         .columns(columns)
                         .rows(rows)
                         .finalize();
    const bar = arc.visual.plugins.bar()
                                  .mark_type('bar');

    // Remove csv and style settings from bar version of user_settings for histogram
    function removeAndAddSetting(settingId) {
      bar.user_settings().removeSetting(settingId);
      bar.user_settings().addSetting(user_settings.find_by_id(settingId));
    }

    [
      'permit_csv_download_visual',
      'permit_png_download',
      'permit_email_report',
      'container_css_class',
      'show_tooltip',
      'trellis_borders',
      'trellis_cell_minimum_width',
      'trellis_cell_minimum_height',
      'disable_incremental_queries',
      'send_aggregate_parameters',
      'visual_dataset_scope',
      'explicit_scope',
      'visual_font_size',
      'label_font_size',
      'axis_label_font_size',
      'axis_ticks_font_size',
      'trellis_font_size',
      'tooltip_font',
      'tooltip_font_size',
      'visual_font',
      'visual_title_font_weight',
      'inline_measure_display',
      'show_y_axis',
      'show_x_axis',
      'y_axis_label',
      'x_axis_label',
      'dynamic_bar_width',
    ].forEach(removeAndAddSetting);

    const driver = arc.report.driver(bar);
    bar.markerLib(lib);

    // Make dashboard click parameters work
    bar.viz_lib().dashboard_click_behavior(lib.dashboard_click_behavior());
    bar.viz_lib().click_behavior_change_cb(lib.click_behavior_change_cb());
    bar.viz_lib().dataset(lib.dataset());
    bar.viz_lib().shelves(lib.shelves());

    driver.user_settings().find_by_id('bar_mark_size')
          .value(bar_mark_size.value());
    driver.user_settings().find_by_id('show_y_axis')
          .value(show_y_axis.value());
    driver.user_settings().find_by_id('show_x_axis')
          .value(show_x_axis.value());

    driver.user_settings().find_by_id('scale_agg_zero')
          .value(scale_agg_zero.value());

    driver.user_settings().find_by_id('color_opacity')
          .value(lib.colorOpacity() * 100);

    driver.colors(lib.colors());

    driver.user_settings().find_by_id('scale_agg_independent')
          .value(scale_agg_independent.value());

    driver.user_settings().find_by_id('scale_dim_independent')
          .value(scale_dim_independent.value());

    masterHistogramConfig.bucketCount = Number(bucketCount.value());

    bar.allowLargeTrellisTemp(allowLargeTrellisTemp);
    bar.allowLargeTrellisPerm(allowLargeTrellisPerm);

    const barVizLib = bar.viz_lib();
    barVizLib.inModal(lib.inModal());
    barVizLib.dashboard(lib.dashboard());

    masterHistogramConfig.allowLargeTrellisPerm = histogram.allowLargeTrellisPerm;
    masterHistogramConfig.allowLargeTrellisTemp = histogram.allowLargeTrellisTemp;

    bar.histogramConfig(masterHistogramConfig)
       .viz_setDataResult(data_result)
       .viz_drawVisual(lib.chart_div());
  }

  histogram.viz_drawVisual_test = function(rows, columns, min, max) {
    // This function is for testing purposes only.
    // From the relating qunit test this way we can call directly the
    // drawHistogram function with the preseted masterHistogramConfig
    // without triggering the constructDataRequest where we would fail
    // without heavy mocking processes.
    masterHistogramConfig = getBucketConfig(min, max);
    drawHistogram(rows, columns);
  };

  histogram.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    let rowsAggregated = [];
    const maximumsAndMinimums = data_result.rows();
    const numberOfSqlRuns = scale_dim_independent.isChecked ?
      maximumsAndMinimums.length : 1;
    let totalSqlRuns = 0;


    maximumsAndMinimums.forEach((row) => {
      const dsreq = constructDataRequest(row);
      const sql = dsreq.sql();
      lib.runSqlCmd(sql, (data) => {
        totalSqlRuns += 1;
        rowsAggregated = rowsAggregated.concat(data.rows);

        if (totalSqlRuns === numberOfSqlRuns) {
          drawHistogram(normalizeData(rowsAggregated), dsreq.columns());
        }
      }, dsreq.dsreq());
    });

    return histogram;
  };

  histogram.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return histogram;
  };

  histogram.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return histogram;
  };

  return histogram;
};

arc.visual.plugins.histogram_extension = function() {
  const ext = function() { return 'arc.visual.plugins.histogram_extension'; };

  ext.viz_name = function() {
    return 'histogram';
  };

  ext.viz_tooltip = function() {
    return 'histogram';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_histogram.png?v=1';
  };

  ext.viz = arc.visual.plugins.histogram;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.histogram_extension());
}());
