// arc_visual_html corresponds to the Rich Text visual
window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.html = function() {
  let saved_settings = {};
  let lib;
  let contents = '';
  let $editor;
  let $applyButton;
  let contentToShow;

  const html = arc.visual.api();

  // Remove 'Visual Axes and Labels' and 'Tooltips' options
  // in Style section of visual editor
  const userSettings = arc.visual.defaultSettings({
                                   noTrellis: true,
                                   noAxes: true,
                                   noTooltip: true,
                                 })
                                 .removeSetting([
                                   'visual_show_detail_data',
                                 ]);

  html.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data([])
       .user_settings(userSettings)
       .refresh_visible(false);

    return html;
  };

  html.viz_getSettings = function() {
    const settings = lib.save_basics(saved_settings);

    settings.viz_type = settings.viz_type || {};
    settings.viz_type.html = settings.viz_type.html || {};
    settings.viz_type.html.contents = contents;
    saved_settings = settings;

    return settings;
  };

  html.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return html;
    }

    saved_settings = settings;

    lib.restore_basics(settings, switchingVisualType);

    const hasContents = settings.viz_type &&
      settings.viz_type.html &&
      settings.viz_type.html.contents;

    if (hasContents) {
      contents = settings.viz_type.html.contents;
    }

    return html;
  };

  html.viz_drawDesigner = function(_shelves_div) {
    const hasManageCustomStylesPerm = arc.globals.perms.sysperms.includes('sys_styles');
    lib.shelves_dom_id(_shelves_div)
       .base_setup();

    $('#app-shelves').find('.viz-html-info').remove();
    if (hasManageCustomStylesPerm) {
      $('#app-shelves').html(arc.misc.vizHTMLInfo());
    } else {
      $('#app-shelves').html(arc.misc.vizHTMLInfoNoPermission());
    }
    $('#app-shelves').find('.viz-html-view').offOn('click.fullscreen', () => {
      const report = lib.report();
      const widgetId = report.reportWidgetId();
      $(`#${widgetId} .viz-modal-link`).trigger('click');
    });

    return html;
  };

  html.viz_getDataRequest = function() {
    arc.counters.increment('dataapi_start');
    return arc.data.request()
                   .request_type('ignore');
  };

  // Helper function to check whether Summernote editor should be shown
  function _isEditorShown() {
    return arc.globals.is_edit && lib.inModal();
  }

  function _getCode() {
    return $editor ? $editor.summernote().data('summernote').code() : contents;
  }

  /*
   * Set the data this object should use to draw out the visual
   */
  html.viz_setDataResult = function(result) {
    arc.counters.increment('dataapi_end');
    return html;
  };

  var fileTypes = {
    'image/gif': 'Image (gif)',
    'image/png': 'Image (png)',
    'image/jpg': 'Image (jpg)',
    'image/jpeg':'Image (jpeg)',
    'image/svg+xml': 'Image (svg)',
  };

  html.viz_drawVisual = function (viz_div) {
    lib.chart_div(viz_div);
    contentToShow = _isEditorShown() && $editor ? _getCode() : contents;
    lib.undraw();

    const $viz = $(`#${viz_div}`);
    $viz.find('.viz-htmleditor').remove();
    const $editorContainer = $('<div class="viz-htmleditor">');
    const $potentialEditor = $('<div>').attr('class', 'viz-htmleditor-predecessor');
    $editorContainer.append($potentialEditor);
    $viz.append($editorContainer);

    if (_isEditorShown()) {
      $editor = $potentialEditor;
      setupRichTextEditorOptions(lib);
      $editor.summernote('enable');

      $applyButton = $editorContainer
        .closest('.super-modal')
        .find('#apply-button');

      $applyButton.offOn('click.applyChanges', () => {
        const newValue = _getCode();

        if (newValue !== contents) {
          contents = newValue;
          lib.page_needs_save();
        }
      });
    } else {
      $editor = null;
      contentToShow = null;
      $potentialEditor.empty().append(contents);
    }

    return html;
  };

  const customIndentation = arc.misc.customSummerNoteTabs();

  function setupRichTextEditorOptions(lib) {
    const options = {
      height: 480,                 // set editor height
      minHeight: null,             // set minimum height of editor
      maxHeight: null,             // set maximum height of editor
      focus: true,                 // set focus to editable area after initializing summernote
      lang: 'en-US',               // default: 'en-US'
      dialogsFade: true,           // Add fade effect on dialogs
      airMode: false,
      disableDragAndDrop: false,
      shortcuts: true,
      dialogsInBody: true,
      tabDisable: true,
      tabSize: 4,
      toolbar: [
        ['controls', ['undo', 'redo', 'clear'],],
        ['color', ['color'],],
        ['font', ['style', 'fontname', 'fontsize', 'height'],],
        ['style', ['bold', 'italic', 'underline'],],
        ['style2', ['strikethrough', 'superscript', 'subscript'],],
        ['para', ['ul', 'ol', 'paragraph'],],
        ...customIndentation.toolbar,
        ['insert', ['hr', 'link', 'table', 'picture', 'video'],],
        ['misc', ['codeview'],],
      ],
      buttons: customIndentation.buttons,
      popover: {
        image: [
          ['imagesize', ['imageSize100', 'imageSize50', 'imageSize25'],],
          ['remove', ['removeMedia'],]
        ],
        link: [
          ['link', ['linkDialogShow', 'unlink'],]
        ],
      },
      fontNames: ['Andale Mono', 'Arial', 'Arial Black', 'Book Antiqua', 'Comic Sans MS',
        'Courier New', 'Georgia', 'Helvetica', 'Impact', 'Lato', 'Roboto', 'Symbol', 'Tahoma',
        'Terminal', 'Times New Roman', 'Trebuchet MS', 'Verdana', 'Webdings', 'Wingings'],
      fontNamesIgnoreCheck: ['Andale Mono', 'Book Antiqua', 'Georgia', 'Lato', 'Roboto',
        'Symbol', 'Terminal', 'Times New Roman', 'Trebuchet MS', 'Webdings', 'Wingings'],
      fontSizes: ['6', '8', '10', '11', '12', '14', '16', '18', '20', '22', '24', '26',
        '28', '30', '32', '34', '36', '38', '40', '48', '64', '82', '150', ],
      callbacks: {
        onInit: function () {
          $editor.summernote('code', contentToShow);
        },
        onImageUpload: function (files) {
          if (files.length === 1) {
            var f = files[0];
            var mimetype = f.type;
            if (mimetype in fileTypes) {
              var fileData = new FileReader();
              fileData.readAsDataURL(f);
              fileData.onloadend = function () {
                var fileContents = fileData.result.replace(/^data:.*base64,/, '');

                var asset = arc.admin.StaticAsset()
                                        .assetName('Auto upload for visual ' + lib.report_id())
                                        .assetDescription('Visual ' + lib.report_id())
                                        .assetMimetype(mimetype)
                                        .assetContents(fileContents)
                                        .fileSize(f.size);

                asset.save(function() {
                  $editor.summernote('insertImage', asset.url());
                });
              };
            }
          }
        },
      },
    };

    $editor.summernote(options);
    $editor.summernote('code', contentToShow);
  }

  html.viz_destroy = () => {
    if ($applyButton) {
      $applyButton.off('click.applyChanges');
      $applyButton = undefined;
    }
  };

  return html;
};

arc.visual.plugins.html_extension = function() {
  var ext = function() { return 'arc.visual.plugins.html_extension'; };

  ext.viz_name = function() {
    return 'html';
  };

  ext.viz_tooltip = function() {
    return 'html';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_extension.png?v=1';
  };

  ext.viz = arc.visual.plugins.html;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.html_extension());

}());
