window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.user_settings = arc.visual.user_settings || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.kpi = function() {
  let saved_settings = {},
    data_result,
    lib,
    trellis,
    sort_orders_asc = {};

  const mainFontSize =  arc.settings.kpi.mainFontSize(),
        mainFontSizeAutosize =  arc.settings.kpi.mainFontSizeAutosize(),
        mainValueDisplay = arc.settings.kpi.mainValueDisplay(),
        mainColorAsArrowColor = arc.settings.kpi.useArrowColorForMainFont(),
        displayFirstMeasure = arc.settings.kpi.displayFirstMeasure(),
        compareFormat = arc.settings.kpi.displayCompareFormat(),
        compareArrowType = arc.settings.kpi.compareArrowType(),
        compareArrowLocation = arc.settings.kpi.compareArrowLocation(),
        labelLocation = arc.settings.kpi.labelLocation(),
        labelFontSize = arc.settings.kpi.labelFontSize(),
        upArrowColor = arc.settings.kpi.upArrowColor(),
        flatArrowColor = arc.settings.kpi.flatArrowColor(),
        downArrowColor = arc.settings.kpi.downArrowColor(),
        kpiFontColor = arc.settings.kpi.kpiFontColor(),
        flipComparison = arc.settings.kpi.switchCompareDirection(),
        kpiLabelColor = arc.settings.kpi.kpiLabelColor(),
        displayDecimals = arc.settings.kpi.numDecimals();
        const hideXTrellisLabel = arc.settings.kpi.hideXTrellisLabel();
        const hideYTrellisLabel = arc.settings.kpi.hideYTrellisLabel();

  let user_settings = arc.visual.defaultSettings({ noAxes: true })
                                .addSetting([
                                  mainFontSize,
                                  mainFontSizeAutosize,
                                  mainValueDisplay,
                                  mainColorAsArrowColor,
                                  displayFirstMeasure,
                                  compareFormat,
                                  compareArrowLocation,
                                  compareArrowType,
                                  flipComparison,
                                  labelLocation,
                                  labelFontSize,
                                  upArrowColor,
                                  flatArrowColor,
                                  downArrowColor,
                                  kpiFontColor,
                                  kpiLabelColor,
                                  displayDecimals,
                                  hideXTrellisLabel,
                                  hideYTrellisLabel,
                                ]);

  let kpi = arc.visual.api();

  function arrowIcon(style, dir) {
    return `<i class="fa fa-${style}-${dir}"></i>`;
  }

  const shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimensions',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'compare_shelf',
      display_name: 'Compare To',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'label_shelf',
      display_name: 'Label',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  const constraints = {
    minimums: {},
    maximums: {
      aggregates_shelf: 1,
      compare_shelf: 1,
      label_shelf: 1,
    },
  };

  kpi.user_settings = function(_) {
    if (!arguments.length) return user_settings;
    user_settings = _;
    return kpi;
  };

  kpi.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return kpi;
  };

  kpi.viz_getSettings = function () {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    saved_settings = settings;
    return settings;
  };

  kpi.viz_setSettings = function (settings, switchingVisualType) {
    if (settings === undefined) {
      return kpi;
    }

    saved_settings = settings;
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    lib.restore_basics(settings, switchingVisualType);
    return kpi;
  };

  kpi.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        aggregateMessages = [lib.createPopoverMessages('aggregateSingle', 'onlyOneNeeded')],
        compareToMessage = [lib.createPopoverMessages('aggregateSingle', 'onlyOneNeeded')],
        labelMessage = [lib.createPopoverMessages('label')],
        dimMessage = [lib.createPopoverMessages('kpiDimension', 'kpiDimensionNotes')];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      aggregates_shelf: aggregateMessages,
      compare_shelf: compareToMessage,
      label_shelf: labelMessage,
      dimensions_shelf: dimMessage,
    });

    return kpi;
  };

  kpi.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  kpi.viz_setDataResult = function(result) {
    data_result = result;
    return kpi;
  };

  kpi.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    const rows = data_result.rows();
    if (!rows || rows.length === 0) {
      return kpi;
    }

    const columns = data_result.columns();
    const config = {
      rows: rows,
      columns: columns,
      dim_col: lib.first_column(columns, 'dimensions_shelf'),
      agg_col: lib.first_column(columns, 'aggregates_shelf'),
      compare_col: lib.first_column(columns, 'compare_shelf'),
      label_col: lib.first_column(columns, 'label_shelf'),
      hasTrellis: lib.first_column(columns, 'x_shelf') || lib.first_column(columns, 'y_shelf'),
    };

    trellis = arc.visual.trellis()
                        .x_shelf_name('x_shelf')
                        .y_shelf_name('y_shelf')
                        .x_shelf_reserved(0)
                        .y_shelf_reserved(0)
                        .rows(config.rows)
                        .sort_orders_asc(sort_orders_asc)
                        .columns(config.columns)
                        .viz_lib(lib)
                        .viz_div_sel(d3.select(`#${viz_div}`))
                        .ignoreLargeTrellis(true)
                        .setup();
    if (trellis.error_state()) {
      return kpi;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      kpi.viz_drawVisual(viz_div);
    });

    computeSvgParams(config, viz_div);
    config.trellis.tooltips('trellis');

    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      config.rows = xy_rows;
      var cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_div);
    });

    return kpi;
  };

  function computeSvgParams(config, viz_div) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
    });

    let margin = { top: 5, right: 10, bottom: 40, left: 10 };
    let width = elem_w - margin.left - margin.right;
    let height = elem_h - margin.top - margin.bottom;
    let minWidth = 100;
    let minHeight = 50;
    width = Math.max(width, minWidth);
    height = Math.max(height, minHeight);
    config.width = width;
    config.height = height;
  }

  function createValueDisplay(val, compare, colIdx, format, isCompare, decimalCount) {
    let diff, denom, quot;
    if (flipComparison.isChecked) {
      diff = compare - val;
      denom = val;
      quot = compare / val;
    } else {
      diff = val - compare;
      denom = compare;
      quot = val / compare;
    }

    const mult = quot < 1 ? -1 / quot : quot;
    const _round = num => num.toFixed(decimalCount);
    const _format = lib.formatter(colIdx) || (x => x);

    switch (format) {
      case '':
        return '';
      case 'value':
        const returnValue = isCompare ? _round(compare) : _round(val);
        return _format(returnValue);
      case 'percent':
        const percent = diff / denom * 100;
        return `${percent > 0 ? '+' : ''}${_round(percent)}%`;
      case 'difference':
        return `${diff > 0 ? '+' : ''}${_format(_round(diff))}`;
      case 'multiple':
        return `${_format(_round(mult))}x`;
    }
  }

  function getMainFontSize(_) {
    const {
      contents,
      viz_div,
      compareIncluded,
    } = _;

    if (!mainFontSizeAutosize.isChecked) {
      return Number(mainFontSize.value());
    }

    const $viz = $(`#${viz_div}`);
    const $container = $viz.closest('.viz-chart, #chart-container, .search-visual');

    let w = $container.width();
    if (compareIncluded) {
      w = w * 0.75;
    }

    const config = {
      fontSize: Number(mainFontSize.value()),
      cssClasses: 'testsvg',
      useSpan: true,
      textIsHtml: true,
    };

    let checkedMax = config.fontSize,
        checkedMin = 10;

    let b = lib.textBounds([contents], config);
    while (true) {
      if (b.width > w) {
        checkedMax = Math.max(config.fontSize, checkedMin);
      } else {
        checkedMin = Math.min(config.fontSize, checkedMax);
      }

      if (checkedMin >= (checkedMax - 1)) {
        config.fontSize = checkedMin;
        break;
      }

      config.fontSize = parseInt((checkedMin + checkedMax) / 2);
      b = lib.textBounds([contents], config);
    }

    return config.fontSize;
  }

  function viz_drawVisualWork(config, viz_div) {
    if (config.rows.length === 0) {
      return;
    }

    if (!config.agg_col && !config.dim_col) {
      return;
    }

    let data = config.rows[0];

    let mainValue, mainDisplay, mainIsDim, mainIdx, compare, compareDisplay, label, labelDisplay;

    if (config.agg_col && displayFirstMeasure.isChecked) {
      mainIdx = config.agg_col.colindex();
    } else if (config.dim_col) {
      mainIdx = config.dim_col.colindex();
      mainIsDim = true;
    } else if (config.agg_col) {
      mainIdx = config.agg_col.colindex();
    }

    mainValue = data[mainIdx];
    mainDisplay = lib.formatter(mainIdx) ? lib.formatter(mainIdx)(mainValue) : mainValue;

    if (config.agg_col) {
      mainValue = Number(data[config.agg_col.colindex()]);
    }

    if (config.compare_col) {
      const compareIdx = config.compare_col.colindex();
      compare = Number(data[config.compare_col.colindex()]);
      let displayFormat = compareFormat.value();
      let decimalCount = displayDecimals.value();
      compareDisplay = createValueDisplay(mainValue, compare, compareIdx,
                                      displayFormat, true, decimalCount);
      if (!mainIsDim) {
        displayFormat = mainValueDisplay.value();
        mainDisplay = createValueDisplay(mainValue, compare, mainIdx,
                                      displayFormat, false, decimalCount);
      }
    }

    if (config.label_col) {
      const labelIdx = config.label_col.colindex();
      label = data[labelIdx];
      labelDisplay = lib.formatter(labelIdx) ? lib.formatter(labelIdx)(label) : label;
    }

    //Main div holds everything dataContainer holds main value and compareContainer
    //Compare container holds compare arrow and compare value
    let $mainDiv = $(`<div id="kpi-${viz_div}" class="kpi-container singlevalue-container"></div>`);
    let $dataContainer = $('<div class="outer-data-container"></div>');
    let $compareContainer = $('<div class="inner-data-container"></div>');
    $dataContainer.css({ display: 'flex', justifyContent: 'center' });
    $mainDiv.append($dataContainer);

    const contents = `<span class="kpi-main-value">${mainDisplay}</span>`;
    let $mainValueDiv = $(contents);
    const fontSize = getMainFontSize({
      contents,
      viz_div,
      compareIncluded: (compare || compare === 0),
    });

    $mainValueDiv.css({
                       fontSize: fontSize + 'px',
                       color: kpiFontColor.value(),
                       textAlign: 'center',
                      });
    $dataContainer.append($mainValueDiv);
    let $compareDiv, $labelDiv;

    if (compare || compare === 0) {
      const ratioOfCompareFont = 4,
      ratioOfArrow = 2,
      ratioOfCompareMargin = 7,
      arrowLocation = compareArrowLocation.value(),
      compareFontSize = Math.floor(fontSize / ratioOfCompareFont),
      arrowSize = Math.floor(fontSize / ratioOfArrow),
      arrowType = compareArrowType.value(),
      compareMargin = Math.floor(fontSize / ratioOfCompareMargin),
      compareDir = (arrowLocation === 'above' || arrowLocation === 'below') ? 'row' : 'column',
      dataDir = { right: 'row', left: 'row-reverse', below: 'column', above: 'column-reverse' };

      // directions are set based on what was chosen by the user for the location of the compare div
      // they use flex-direction css key which says which way to stack new divs in a flexbox
      //row and column follow their conventional meanings (stacked horizontally vs. vertically)
      $dataContainer.append($compareContainer);
      $dataContainer.css({ flexDirection: dataDir[arrowLocation] });
      $compareContainer.css({ display: 'flex',
                              flexDirection: compareDir,
                              justifyContent: compareDir === 'column' ? 'space-around' : 'center',
                            });

      if (arrowType !== 'none') {
        let arrowDirection, arrowColor, $arrowDiv;
        const flipped = flipComparison.isChecked;
        if ((compare > mainValue && !flipped) || (compare < mainValue && flipped)) {
          arrowDirection = 'down';
          arrowColor = downArrowColor.value();
        } else if ((compare < mainValue && !flipped) || (compare > mainValue && flipped)) {
          arrowDirection = 'up';
          arrowColor = upArrowColor.value();
        } else {
          arrowDirection = 'right';
          arrowColor = flatArrowColor.value();
        }

        if (mainColorAsArrowColor.value() === 'true') {
          $mainValueDiv.css({ color: arrowColor });
        }

        const arrow = arrowIcon(arrowType, arrowDirection);
        $arrowDiv = $(`<div><span class="compare-arrow">${arrow}</span></div>`);
        $arrowDiv.css({
          fontSize: arrowSize + 'px',
          color: arrowColor,
          textAlign: 'center',
        });
        $compareContainer.append($arrowDiv);
      }

      if (compareFormat.value() !== 'none') {
        $compareDiv = $(`<span class="kpi-compare">${compareDisplay}</span>`);
        const compareStyle = {
          fontSize: compareFontSize + 'px',
          color: kpiFontColor.value(),
          textAlign: 'center',
        };
        if (arrowLocation === 'above' || arrowLocation === 'below') {
          compareStyle.display = 'flex';
          compareStyle.flexDirection = 'column';
          compareStyle.justifyContent = 'center';
          compareStyle.marginLeft = compareMargin + 'px';
        }

        $compareDiv.css(compareStyle);
        $compareContainer.append($compareDiv);
      }

      if (arrowLocation === 'right' || arrowLocation === 'left') {
        $compareContainer.css({ margin: compareMargin + 'px' });
      } else {
        $compareContainer.css({ margin: -compareMargin + 'px' });
      }
    }

    if (label) {
      $labelDiv = $(`<span class="kpi-label">${labelDisplay}</span>`);
      let labelStyle = {
        fontSize: labelFontSize.value() + 'px',
        color: kpiLabelColor.value(),
        textAlign: 'center',
      };

      $labelDiv.css(labelStyle);
      if (labelLocation.value() === 'above') {
        $mainDiv.css({ flexDirection: 'column-reverse' });
      }

      $mainDiv.append($labelDiv);
    }

    if (config.hasTrellis) {
      $mainDiv.css({ minWidth: `${config.width}px` });
      $mainDiv.css({ minHeight: `${config.height}px` });
    }

    const $vizContainer = $('#' + viz_div);
    $vizContainer.closest('table').css({ margin: 'auto' });
    $vizContainer.append($mainDiv);

    let d3sel = d3.select(`#kpi-${viz_div}`);
    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .addSelection(d3sel)
       .text_fn(d => lib.base_text_fn(data))
       .params_fn(d => lib.base_params_fn(data))
       .activate();

    return kpi;
  }

  return kpi;
};

arc.visual.plugins.kpi_extension = function() {
  var ext = function () {
    return 'arc.visual.plugins.kpi_extension';
  };

  ext.viz_tooltip = function() {
    return 'kpi';
  };

  ext.viz_name = function() {
    return 'kpi';
  };

  ext.viz = arc.visual.plugins.kpi;
  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.kpi_extension());

})();
