window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.user_settings = arc.visual.user_settings || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.link = function() {
  let savedSettings = {};
  let lib;
  let shelvesDiv;
  let linkedPage = '';

  let userSettings = arc.visual.defaultSettings({
                                 noAxes: true,
                                 noTrellis: true,
                                 noTooltip: true,
                               })
                               .removeSetting([
                                 arc.colorpalette.setting(),
                                 'visual_show_detail_data',
                               ]);

  const link = arc.visual.api();

  const getLinkedPageInput = (shelvesContainer) => $(`#${shelvesContainer} .viz-linkbuilder > input`);

  link.user_settings = function(_) {
    if (!arguments.length) {
      return userSettings;
    }

    userSettings = _;
    return link;
  };

  link.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data([])
       .user_settings(userSettings);
    return link;
  };

  link.viz_getSettings = function() {
    const settings = lib.save_basics(savedSettings);
    settings.viz_type = settings.viz_type || {};
    settings.viz_type.link = settings.viz_type.link || {};

    linkedPage = getLinkedPageInput(shelvesDiv).val();
    settings.viz_type.link.link_name = linkedPage;

    savedSettings = settings;

    return settings;
  };

  link.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return link;
    }

    savedSettings = settings;

    lib.restore_basics(settings, switchingVisualType);

    if (
      settings.viz_type &&
      settings.viz_type.link &&
      settings.viz_type.link.link_name
    ) {
      linkedPage = settings.viz_type.link.link_name;

      getLinkedPageInput(shelvesDiv).val(linkedPage);
    }

    return link;
  };

  link.viz_drawDesigner = function(_shelves_div) {
    shelvesDiv = _shelves_div;
    lib.shelves_dom_id(shelvesDiv)
       .base_setup();

    const $template = $('#viz-linkbuilder');
    const $container = $(`#${shelvesDiv}`);

    $container.html($template.html());

    const $linkInput = getLinkedPageInput(shelvesDiv);

    $linkInput.val(linkedPage);

    const setLinkedPage = (page) => {
      linkedPage = page;
      lib.page_needs_save();
    };

    $linkInput.offOn('change', (event) => {
      setLinkedPage(event.target.value);
    });

    $linkInput.offOn('keypress', (event) => {
      const isEnter = event.which === 13;

      if (isEnter) {
        setLinkedPage(event.target.value);

        link.viz_drawVisual(lib.chart_div());
      }
    });

    return link;
  };

  link.viz_getDataRequest = function() {
    arc.counters.increment('dataapi_start');
    return arc.data.request()
                   .request_type('ignore');
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  link.viz_setDataResult = function() {
    arc.counters.increment('dataapi_end');
    return link;
  };

  link.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    if (!lib.in_dashboard() && lib.build_mode()) {
      linkedPage = getLinkedPageInput(shelvesDiv).val();
    }

    const display_link = lib.substituteParams(linkedPage, [linkedPage]);

    const viz_elem = $(`#${viz_div}`);
    const width = viz_elem.width();
    const height = viz_elem.height();

    const container = d3.select(`#${viz_div}`)
                        .append('div');

    const iframe = container.append('iframe')
                          .attr('frameBorder', '0')
                          .attr('width', width)
                          .attr('height', height)
                          .attr('src', display_link)
                          .style('visibility', 'hidden');
    // Prevents iframe from scrolling the whole page by hiding until loaded
    iframe.on('load', function () {
      $(this).hide().css('visibility', 'visible').fadeIn(1000, 'linear');
    });
    return link;
  };

  return link;
};

arc.visual.plugins.link_extension = function() {
  const ext = function() { return 'arc.visual.plugins.link_extension'; };

  ext.viz_name = function() {
    return 'link';
  };

  ext.viz_tooltip = function() {
    return 'link';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_link.png?v=1';
  };

  ext.viz = arc.visual.plugins.link;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.link_extension());
}());
