window.arc = window.arc || {};

(function () {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.list = function () {
  let data_result;
  let lib;

  const list = arc.visual.api();

  const shelf_data = [
    {
        name: 'dimensions_shelf',
        display_name: 'Dimensions',
        shelf_type: arc.report.shelf_types.COLUMN,
        column_type: arc.report.columnTypeEnum.DIMENSION,
      },
      {
        name: 'aggregates_shelf',
        display_name: 'Measures',
        shelf_type: arc.report.shelf_types.COLUMN,
        column_type: arc.report.columnTypeEnum.AGGREGATE,
      },
      {
        name: 'tooltip_shelf',
        display_name: 'Tooltips',
        shelf_type: arc.report.shelf_types.COLUMN,
        column_type: arc.report.columnTypeEnum.AGGREGATE,
      },
      {
        name: 'filters_shelf',
        display_name: 'Filters',
        display_html: arc.report.filters_shelf_html(),
        shelf_type: arc.report.shelf_types.PREDICATE,
        column_type: arc.report.columnTypeEnum.UNKNOWN,
        onShelfListChange: arc.report.updateFilterShelfLabel,
      },
  ];

  const listStyle = arc.settings.list.listStyle();
  const showColumnNames = arc.settings.list.showColumnNames();
  const marginBetweenItems = arc.settings.apps.marginBetweenItems();
  const cardWidth = arc.settings.apps.cardWidth();
  const enable_highlighting = arc.settings.table.enableHighlighting();
  const cells_are_html_setting = arc.settings.table.cellsAreHTMLSetting(enable_highlighting);

  const user_settings = arc.visual.defaultSettings({
    noTrellis: true,
    noAxes: true,
  }).addSetting([
    cells_are_html_setting,
    enable_highlighting,
  ]);

  user_settings.addSetting([
      showColumnNames,
      marginBetweenItems,
      listStyle,
      cardWidth,
  ]);

  const constraints = {
    minimums: {
      dimensions_shelf: 1,
    },
  };

  list.viz_lib = function (_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
      .user_settings(user_settings);
    return list;
  };

  list.viz_drawDesigner = function (shelves_div) {
    lib.shelves_dom_id(shelves_div)
      .base_setup();

    lib.addLimit();

    const dimensionMessages = [
      lib.createPopoverMessages('dimensionTable'),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const aggregateMessages = [
      lib.createPopoverMessages('aggregateTable'),
      lib.createPopoverMessages('acceptsMultiple'),
    ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      dimensions_shelf: dimensionMessages,
      aggregates_shelf: aggregateMessages,
    });

    return list;
  };

  list.viz_getDataRequest = function () {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  list.viz_setDataResult = function (result) {
    data_result = result;

    // Hide 'Enable highlighting' setting for non-solr connections
    if (lib.dataconnection_type() === 'solr') {
      enable_highlighting.show();
    } else {
      enable_highlighting.hide();
    }

    return list;
  };

  let saved_info = {};

  list.viz_getSettings = function () {
    let settings = saved_info;
    settings = lib.save_basics(settings);
    lib.saveLimit(settings);
    saved_info = settings;
    return settings;
  };

  list.viz_setSettings = function (settings, switchingVisualType) {
    if (settings === undefined) {
      return list;
    }

    if (switchingVisualType) {
      settings.limit = 100;
    }

    saved_info = settings;

    lib.restore_basics(settings, switchingVisualType);
    lib.restoreLimit(settings);

    return list;
  };

  function addListStyles(viz_div) {
    const $viz = $(`#${viz_div}`);
    const styleName = listStyle.value();

    const marginBetweenItemsPx = marginBetweenItems.value();
    const cardWidthPx = cardWidth.value();

    const allStyles = [];
    allStyles.push(`#${viz_div} .viz-list-row {margin-bottom: ${marginBetweenItemsPx}px;}`);

    const fontSize = lib.getVisualFontSize();
    if (fontSize) {
      allStyles.push(`#${viz_div} {font-size: ${fontSize}px;}`);
    }

    if (styleName === 'headline') {
      // The 'headline' style:
      //   emphasizes the first column,
      //   leaves the second column alone, and
      //   de-emphasizes all other columns

      $(`#${viz_div} .viz-list-wrapper`).addClass('viz-list-headline');

      if (fontSize) {
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col {font-size: ${fontSize * 0.6}px;}`);
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col.viz-list-col-0 {font-size: ${fontSize}px;}`);
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col.viz-list-col-1 {font-size: ${fontSize * 0.7}px;}`);
      }
    }

    if (styleName === 'headline-2') {
      // The 'headline-2' style:
      //    de-emphasizes the first column,
      //    emphasizes the second column,
      //    leaves the 3rd column alone, and
      //    de-emphasizes all other columns

      $(`#${viz_div} .viz-list-wrapper`).addClass('viz-list-headline-2');

      if (fontSize) {
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col {font-size: ${fontSize * 0.6}px;}`);
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col.viz-list-col-1 {font-size: ${fontSize}px;}`);
      }
    }

    if (styleName === 'card') {
      // The 'card' style:
      //   creates a floating box with a border,
      //   emphasizes the first column,
      //   makes the second column smaller and de-emphasizes by making a lighter color
      //   de-emphasizes all other columns by making smaller, a lighter color and lower font-weight
      $(`#${viz_div} .viz-list-wrapper`).addClass('viz-list-card');
      allStyles.push(`#${viz_div} .viz-list-row {width: ${cardWidthPx}px; ` +
          `margin-right: ${marginBetweenItemsPx}px;}`);

      if (fontSize) {
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col {font-size: ${fontSize * 0.67}px;}`);
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col.viz-list-col-0 {font-size: ${fontSize}px;}`);
        allStyles.push(`#${viz_div} .viz-list-wrapper .viz-list-col.viz-list-col-1 {font-size: ${fontSize * 0.78}px;}`);
      }
    }

    listStyle.addDependents(
      [cardWidth], ['unformatted', 'headline', 'headline-2'], ['card']
    );

    let $style = $viz.find('style.vizstyle');
    if ($style.length === 0) {
      $viz.append(arc.misc.safe_substitute('<style class="vizstyle"></style>', {}));
      $style = $viz.find('style.vizstyle');
    }

    $style.html(`${allStyles.join(' ')}`);
  }

  list.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);

    $(`#${viz_div} div`).remove();

    lib.undraw();

    const sb_rows = data_result.rows();
    if (!sb_rows || sb_rows.length === 0) {
      return list;
    }

    const cols = data_result.columns();
    let firstInvalidIdx = cols.length;
    cols.forEach((c) => {
      if (c.colsource() === 'tooltip_shelf') {
        firstInvalidIdx = c.colindex();
      }
    });

    const colnames = cols.map((c) => c.colname());

    const coldisplayformats = cols.map((c) => {
      // override default number format for list visual
      if (c.isNumeric() && !c.coldisplayformat(true)) {
        return d3.format(',');
      }

      return c.coldisplayformat();
    });

    $(`#${viz_div}`).append('<div class="viz-list-wrapper"></div>');
    addListStyles(viz_div);

    const d = [];
    sb_rows.forEach((row, idx) => {
      d.push(`<div class="viz-list-row viz-list-row-${idx}">`);
      row.some((c, i) => {
        if (i >= firstInvalidIdx) {
          return true;
        }

        let cell = `${c}`;
        if (coldisplayformats[i]) {
          cell = coldisplayformats[i](cell);
        }

        // Note that even if html cells are permitted, script tags are not allowed
        if (!cells_are_html_setting.isChecked || arc.misc.hasScriptTag(cell)) {
          cell = arc.misc.escapeHTML(cell);
        }

        if (showColumnNames.isChecked) {
          d.push(`<div class="viz-list-col viz-list-col-${i}"><span class="viz-list-colname">` +
              `${colnames[i]}</span><span class=viz-list-colsep>:</span> ` +
              `<span class="viz-list-coltext">${cell}</span></div>`);
        } else {
          d.push(`<div class="viz-list-col viz-list-col-${i}"><span class="viz-list-coltext">` +
              `${cell}</span></div>`);
        }

        return false;
      });

      d.push('</div>');
    });

    $(`#${viz_div} .viz-list-wrapper`).append(d.join(''));

    const cells = d3.selectAll(`#${viz_div}`)
                    .selectAll('.viz-list-row');

    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .highlight_on_select(false)
       .addSelection(cells)
       .text_changes_on_move(true)
       .text_fn((_, i) => lib.base_text_fn(sb_rows[i]))
       .params_fn((_, i) => lib.base_params_fn(sb_rows[i]))
       .activate();

    return list;
  };

  return list;
};

arc.visual.plugins.list_extension = function () {
  const ext = function () { return 'arc.visual.plugins.list_extension'; };

  ext.viz_name = function () {
    return 'list';
  };

  ext.viz_tooltip = function () {
    return 'list';
  };

  ext.viz_background = function () {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function () {
    return '0 0';
  };

  ext.viz_img_large_src = function () {
    return '../static/img/icon_table.png?v=1';
  };

  ext.viz = arc.visual.plugins.list;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.list_extension());
}());
