import svgPanZoom from 'svg-pan-zoom';
import { feature } from 'topojson';

window.arc = window.arc || {};

(function() {
"use strict";

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.map = function() {
  var sort_orders_asc = {},
      saved_settings = {},
      data_result,
      plotted_rows_count = 0,
      ignored_rows_count = 0,
      lib,
      trellis,
      config = {},
      supported_types = "country, state, county, dma, zipcode/postal code, or latitude/longitude",
      supported_types_quoted = '"country", "state", "county", "dma", "zipcode/postal code", "latitude" or "longitude"',
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  const mapViewOptions = [
    'Default',
    'Canada',
    'United States',
    'World',
  ];

  var map = arc.visual.api();

  map.map_bubbles = arc.visual.setting()
                       .id('map_bubbles')
                       .tab('Marks')
                       .type(arc.visual.settingTypeEnum.CHECKBOX)
                       .add_name('MAP_BUBBLES')
                       .add_label('Always display marks as bubbles')
                       .value('false');

  var zoom_enabled = arc.visual.setting()
                        .id("zoom_enabled")
                        .tab("Zoom")
                        .type(arc.visual.settingTypeEnum.CHECKBOX)
                        .add_name("ZOOM_ENABLED")
                        .add_label("Enable pan and zoom")
                        .value("false");

  var zoom_to_active = arc.visual.setting()
                         .id("zoom_to_active")
                         .tab("Zoom")
                         .type(arc.visual.settingTypeEnum.CHECKBOX)
                         .add_name("ZOOM_TO_ACTIVE")
                         .add_label("Zoom to active area")
                         .value("true");

  const map_view_setting = function() {
    const mapViewSetting = arc.visual.setting()
                          .id('map_view')
                          .tab('Map')
                          .type(arc.visual.settingTypeEnum.CUSTOM)
                          .add_name('MAP_VIEW')
                          .add_label('Select Map View')
                          .value('Default');

    let mapViewSelect;

    function paintSetting(tab_d3_sel) {
      const divId = mapViewSetting.div_id();

      const mapViewSettingDiv = tab_d3_sel.append('div')
                                .attr('id', divId)
                                .classed('single-select', true);

      mapViewSettingDiv.append('label')
                   .text(mapViewSetting.labels()[0]);

      mapViewSelect = mapViewSettingDiv
                                .append('select')
                                .attr({
                                  name: mapViewSetting.names()[0],
                                  id: divId,
                                });

      mapViewSelect.selectAll('.nosel')
                   .data(mapViewOptions)
                     .enter()
                     .append('option')
                     .attr('value', function (d) {
                       return d;
                     })
                     .text(function (d) {
                       return d;
                     });

      const savedItem = mapViewSetting.value();
      mapViewSelect.node().value = savedItem;

      if (arc.globals.is_edit) {
        $(mapViewSelect.node()).chosen({
        }).offOn('change.view', (e) => {
          e.stopPropagation();
          e.preventDefault();
          mapViewSetting.value(e.target.value);
          mapViewSetting.makeChangeCbs();
        });
      }
    }

    function fetchSetting() {
      return mapViewSelect.node().value;
    }

    mapViewSetting.paint_fn(paintSetting);
    mapViewSetting.fetch_fn(fetchSetting);

    return mapViewSetting;
  }();

  var scale_independent = arc.report.user_settings.scale_independent();
  var mark_size_range = arc.report.user_settings.mark_size_range();
  let legend_width_override = arc.report.user_settings.legend_width_override();
  var reverse_legend_order = arc.report.user_settings.reverse_legend_order();
  let legend_style = arc.report.user_settings.legend_style();
  legend_style.addDependents([legend_width_override, reverse_legend_order], ['None'],
    ['Right', 'Bottom', 'Above']);

  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  map_view_setting,
                                  scale_independent,
                                  legend_style,
                                  legend_width_override,
                                  reverse_legend_order,
                                  map.map_bubbles,
                                  mark_size_range,
                                  zoom_enabled,
                                  zoom_to_active,
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'axis_ticks_font_size',
                                  'label_font_size',
                                  'label_color',
                                ]);

  arc.addColorSettings(user_settings, {
    includeMeasureRangeMapSetting: true,
    includeOpacitySetting: true,
  });

  var shelf_data = [
    {
      name: "geo_shelf",
      display_name: "Geo",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
      geoShelf: true,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Color',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: 'size_shelf',
      display_name: 'Size',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: "drill_shelf",
      display_name: "Drill",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
      geoShelf: true,
      drillShelf: true,
    },
    {
      name: "tooltip_shelf",
      display_name: "Tooltips",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: "x_shelf",
      display_name: "X Trellis",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "y_shelf",
      display_name: "Y Trellis",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "filters_shelf",
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  var constraints = {
    minimums: {
      'geo_shelf': 1,
    },
    maximums: {
      'size_shelf': 1,
    }
  };

  map.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings)
       .chart_type_graded(true);
    return map;
  };

  map.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  map.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return map;
    }

    saved_settings = settings;
    arc.misc.handleLegacyLegendSetting(saved_settings, legend_style);

    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    let savedUserSettings = saved_settings.user_settings;
    if (savedUserSettings && savedUserSettings.map_view === 'Auto-Detect') {
      map_view_setting.value('Default');
    }

    return map;
  };

  map.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
            lib.createPopoverMessages(['trellisOnly', 'columns.']),
            lib.createPopoverMessages('acceptsMultiple')
        ],
        yMessage = [
            lib.createPopoverMessages(['trellisOnly', 'rows.']),
            lib.createPopoverMessages('acceptsMultiple')
        ],
        geoMessage = [
            lib.createPopoverMessages(['geoMap1', supported_types]) +
            lib.createPopoverMessages('geoMap2'),
            lib.createPopoverMessages('multipleGeoConfigs')
        ],
        drillMessage = [
            lib.createPopoverMessages('acceptsMultiple'),
            lib.createPopoverMessages('drillShelf'),
            lib.createPopoverMessages('drillGeoShelf'),
            lib.createPopoverMessages('emptyDrillShelf'),
        ],
        aggregateMessage = [
            lib.createPopoverMessages(['aggregateAcross', 'locations.']),
            lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException')
        ];
    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      geo_shelf: geoMessage,
      drill_shelf: drillMessage,
      aggregates_shelf: aggregateMessage
    });

    return map;
  };

  map.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  map.viz_setDataResult = function(result) {
    data_result = result;
    return map;
  };

  map.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return map;
  };

  map.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return map;
  };

  const fetchedCollateral = {
    'Default': {},
    'Canada': {},
    'United States': {},
    'World': {},
  };

  const mapRequirements = {
    Default:
      {
        zipcode: ['usa_map', 'state_fips', 'zipcode_latlong' ],
        dma: [ 'usa_map', 'dma_map_data'],
        county: [ 'usa_map', 'state_fips', 'county_fips'],
        state: [ 'usa_map', 'state_fips'],
        latlong: ['world_map'],
        country: ['world_map', 'country_codes'],
      },
    Canada:
      {
        state: [ 'canada_topojson_quantized'],
        zipcode: ['fsa_topojson_quantized'],
        latlong: [ 'canada_topojson_quantized'],
      },
    'United States':
      {
        zipcode: ['usa_map', 'state_fips', 'zipcode_latlong' ],
        dma: [ 'usa_map', 'dma_map_data'],
        county: [ 'usa_map', 'state_fips', 'county_fips'],
        state: [ 'usa_map', 'state_fips'],
        latlong: ['usa_map'],
      },
    World:
      {
        country: ['world_map', 'country_codes'],
        latlong: ['world_map']
      },
  };

  const map_base_url = arc.frontend.globals.urls.mapBaseUrl;
  const collateralInfo = {
    canada_topojson_quantized: {
      asset: map_base_url + '/Canada/State/canada_topojson_quantized.json',
      postprocess: d3.json
    },
    fsa_topojson_quantized: {
      asset: map_base_url + '/Canada/FSA/fsa_topojson_quantized.json',
      postprocess: d3.json
    },
    usa_map: {
      asset: map_base_url + '/United States/us-10m.json',
      postprocess: d3.json
    },
    dma_map_data: {
      asset: map_base_url + '/United States/dma.json',
      postprocess: d3.json
    },
    state_fips: {
      asset: map_base_url + '/United States/states.fips.txt',
      postprocess: d3.csv
    },
    county_fips: {
      asset: map_base_url + '/United States/counties.fips.txt',
      postprocess: d3.csv
    },
    world_map: {
      asset: map_base_url + '/World/world-50m.json',
      postprocess: d3.json
    },
    country_codes: {
      asset: map_base_url + '/World/country.codes.txt',
      postprocess: d3.csv
    },
    zipcode_latlong: {
      asset: map_base_url + '/United States/zipcodes.json',
      postprocess: d3.json
    },
  };

  var dma_map_data,
      dma_valid_ids = {},
      state_fips,
      county_fips,
      country_codes,
      names_to_fips = {},
      state_fips_to_names = {},
      state_fips_to_short_names = {},
      state_codes_names_map = {},
      county_fips_to_names = {},
      county_name_to_fips = {},
      country_codes_to_names = {},
      country_names_to_codes = {},
      valid_canadian_territories = {},
      valid_canadian_fsas = {},
      zipcode_latlong;

  function includeLegend(config) {
    /*
     * We draw a legend in one of two cases:
     * 1. There is exactly one map on screen i.e. trellis is not in play
     * 2. If there is more than one map on screen, they share the same scale.
     *    If the scales are not shared across maps, we end up with too many
     *    distinct scales drawn otherwise, and it is visually confusing
     *
     * if there is no color column I remove the legend because it isn't useful
     */

    if (!config.colorCol || legend_style.value() === 'None') {
      return false;
    }

    var x_trellis = lib.shelf_columns(config.columns, "x_shelf");
    var y_trellis = lib.shelf_columns(config.columns, "y_shelf");
    if ((x_trellis.length === 0) && (y_trellis.length === 0)) {
      return true;
    }

    if (scale_independent.isChecked) {
      return false;
    }

    return true;
  }

  map.viz_fetchCollateral = function(viz_div, isWindowResize, drawCb) {
    config = {};
    config.columns = data_result.columns();
    config.map_view = map_view_setting.value();

    map.compute_map_type(config);
    if (config.errors) {
      return map;
    }

    if (!map.collateral_is_available(config)) {
      if (config.errors) {
        return map;
      }

      map.fetch_collateral(config, function() {
        drawCb(viz_div, isWindowResize);
      });
    } else {
      drawCb(viz_div, isWindowResize);
    }

    return map;
  };

  map.viz_drawVisual = function(viz_div, isWindowResize) {
    lib.chart_div(viz_div);
    lib.undraw();

    config.rows = data_result.rows();
    if (config.rows.length === 0) {
      return map;
    }

    lib.unselectOnClick();

    config.colorCol = lib.first_column(config.columns, 'aggregates_shelf');
    config.sizeCol = lib.first_column(config.columns, 'size_shelf');
    const colsForFilter = [];
    if (config.colorCol === undefined) {
      config.colorMissing = true;
      config.colorColIndex = config.columns.length;
      config.colorColName = 'Count';
    } else {
      config.colorColIndex = config.colorCol.colindex();
      config.colorColName = config.colorCol.colname();
      colsForFilter.push(config.colorCol);
    }

    if (config.sizeCol === undefined) {
      config.sizeMissing = true;
      config.sizeColIndex = config.columns.length;
      config.sizeColName = 'Count';
    } else {
      config.sizeColIndex = config.sizeCol.colindex();
      config.sizeColName = config.sizeCol.colname();
      colsForFilter.push(config.sizeCol);
    }

    // Filter rows with NULLs in the aggregate columns
    lib.filter_nulls(config, colsForFilter);

    computeAggregates(config, config.rows);
    setupColors(config, config.rows);
    setupSizeScale(config, config.rows);

    trellis = arc.visual.trellis()
                     .x_shelf_name("x_shelf")
                     .y_shelf_name("y_shelf")
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .legendPositioning(legend_style.value())
                     .viz_lib(lib)
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       map.viz_drawVisual(viz_div);
                     })
                     .viz_div_sel(d3.select("#" + viz_div));

    if (includeLegend(config)) {
      // Display trellis colors for choropleths
      trellis.color(config.color, config.palette, config.colorSettings)
             .legend_column(config.colorCol, false /* color_is_dim */);
    }
    trellis.setup();

    if (trellis.error_state()) {
      return map;
    }

    config.trellis = trellis;

    config.trellis.draw_skeleton(function() {
      map.viz_drawVisual(viz_div);
    });
    config.trellis.tooltips('trellis');

    if (legend_style.legendRequired()) {
      config.trellis.reverse_legend_order(reverse_legend_order.value());
      config.trellis.legend_width_override(legend_width_override.value());
      config.trellis.draw_legend();
    }
    map.compute_cell_size(config, viz_div);
    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      if (scale_independent.isChecked) {
        setupColors(config, xy_rows);
        setupSizeScale(config, xy_rows);
      }
      map.draw_base_visual(config, cell_sel, xy_rows);
    });

    config.trellis.tooltips('trellis');
    lib.redrawLegendIfNeeded(config.trellis);
    return map;
  };

  map.compute_cell_size = function(config, viz_div) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      viz_div,
      trellis: config.trellis,
      includeLegend: legend_style.legendRequired(),
    });

    var r = lib.compute_svg_params({
      rows: config.rows,
      width: elem_w,
      height: elem_h,
      viz_div_elem: $("#" + viz_div),
      normal_margin: {top: 0, right: 0, bottom: 0, left: 0},
      show_x_axis: false,
      show_y_axis: false,
      x_axis_label_off: true,
    });

    config.margin = r.margin;
    config.width = r.width;
    config.height = r.height;
    config.svgwidth = r.width;
    config.svgheight = r.height;
    var factor = 1.8;
    if (config.map_type === 'country' || config.map_type === 'latlong') {
      factor = 2;
    }
    if ((config.width / config.height) > (factor)) {
      config.width = config.height * factor;
    } else {
      config.height = config.width / factor;
    }
    return map;
  };

  function computeAggregates(config, rows) {
    rows.forEach((row) => {
      if (config.colorMissing) {
        row[config.colorColIndex] = 1;
      } else if (!arc.misc.isNumeric(row[config.colorColIndex])) {
        row[config.colorColIndex] = 0;
      } else {
        row[config.colorColIndex] = Number(row[config.colorColIndex]);
      }

      if (config.sizeMissing) {
        row[config.sizeColIndex] = 1;
      } else if (!arc.misc.isNumeric(row[config.sizeColIndex])) {
        row[config.sizeColIndex] = 0;
      } else {
        row[config.colorColIndex] = Number(row[config.colorColIndex]);
      }
    });
  }

  function isBubbleMap(config) {
    // We draw bubbles instead of choropleths in 3 cases:
    //   latlong maps
    //   zipcode maps that aren't Canadian
    //   map bubbles are explicitly requested
    return (config.map_type === 'latlong' || (config.map_type === 'zipcode' &&
      config.map_view !== 'Canada') || map.map_bubbles.isChecked);
  }

  function setupColors(config, rows) {
    let colorDomain = d3.extent(rows, function getColorValue(d) {
      return d[config.colorColIndex];
    });
    if (colorDomain[0] === colorDomain[1]) {
      colorDomain = [colorDomain[0] - 1, colorDomain[0]];
    }

    config.palette = lib.graded_colors();
    config.colorSettings = lib.colorSettings();
    config.color = lib.aggScale(colorDomain);
    return map;
  }

  map.draw_base_visual = function(config, cell_sel, rows) {
    config.base_svg_id = "map-svg-" + lib.unique_report_id();
    const { margin, svgwidth, svgheight } = config;
    config.base_svg =
      cell_sel.append('svg')
              .attr('id', config.base_svg_id)
              .attr('width', svgwidth + margin.left + margin.right)
              .attr('height', svgheight + margin.top + margin.bottom);

    config.svg =
      config.base_svg.append("g")
            .attr("transform", "translate(" + config.margin.left + "," +
                config.margin.top + ")");
    var svg = config.svg;
    var curr_map,
        elements,
        state_map,
        state_elements;

    const collateral = fetchedCollateral[config.map_view];
    switch (config.map_view) {
      case 'Default':
        if (['state', 'county', 'dma', 'zipcode'].includes(config.map_type)) {
          config.projection = d3.geo.albersUsa();

          /*
           * For the albersUsa projection, a scale of 1000 treanslates to 830x480
           */
          const scaleH = svgheight * 1000 / 480;
          const scaleW = svgwidth * 1000 / 830;
          const scale = Math.min(scaleH, scaleW);

          config.projection.scale(scale);
          config.projection.translate([config.width/2, config.height/2]);

          if (['state', 'zipcode'].includes(config.map_type)) {
            curr_map = collateral.usa_map;
            elements = curr_map.objects.states;
          } else if (config.map_type === 'county') {
            curr_map = collateral.usa_map;
            elements = curr_map.objects.counties;
            state_map = collateral.usa_map;
            state_elements = state_map.objects.states;
          } else if (config.map_type === 'dma') {
            curr_map = collateral.dma_map_data;
            elements = curr_map.objects.nielsen_dma;
            state_map = collateral.usa_map;
            state_elements = state_map.objects.states;
          }
        } else {
          config.projection = d3.geo.mercator()
                      .scale(config.width / (2.5 * Math.PI))
                      .translate([config.width/2, config.height/1.5]);

          curr_map = collateral.world_map;
          elements = curr_map.objects.countries;
        }

        config.a_data_keyGetter = function(d) {
          return d.id;
        };

        break;
      case 'United States':
        config.projection = d3.geo.albersUsa();

        /*
         * For the albersUsa projection, a scale of 1000 treanslates to 830x480
         */
        const scaleH = svgheight * 1000 / 480;
        const scaleW = svgwidth * 1000 / 830;
        const scale = Math.min(scaleH, scaleW);

        config.projection.scale(scale);
        config.projection.translate([config.width/2, config.height/2]);


        if (['state', 'zipcode', 'latlong'].includes(config.map_type)) {
          curr_map = collateral.usa_map;
          elements = curr_map.objects.states;
          state_map = collateral.usa_map;
          state_elements = state_map.objects.states;
        } else if (config.map_type === 'county') {
          curr_map = collateral.usa_map;
          elements = curr_map.objects.counties;
          state_map = collateral.usa_map;
          state_elements = state_map.objects.states;
        } else if (config.map_type === 'dma') {
          curr_map = collateral.dma_map_data;
          elements = curr_map.objects.nielsen_dma;
          state_map = collateral.usa_map;
          state_elements = state_map.objects.states;
        }

        config.a_data_keyGetter = function(d) {
          return d.id;
        };

        break;
      case 'Canada':
        config.projection = d3.geo.azimuthalEqualArea()
            .rotate([100, -45])
            .center([5, 20])
            .scale(config.width)
            .translate([config.width / 2, config.height / 2]);

        switch (config.map_type) {
          case 'latlong':
          case 'state':
            curr_map = collateral.canada_topojson_quantized;
            elements = curr_map.objects.canada_geojson;
            config.a_data_keyGetter = function(d) {
              return d.properties.PRENAME;
            };

            break;
          case 'zipcode':
            curr_map = collateral.fsa_topojson_quantized;
            elements = collateral.fsa_topojson_quantized.objects.fsa;
            config.a_data_keyGetter = function(d) {
              return d.properties.CFSAUID;
            };

            break;
        }

        break;
      default:
        // Right now, when map view is "World"
        config.projection = d3.geo.mercator()
                      .scale(config.width / (2.5 * Math.PI))
                      .translate([config.width/2, config.height/1.5]);
        curr_map = collateral.world_map;
        elements = curr_map.objects.countries;
        config.a_data_keyGetter = d => d.id;
    }

    config.path = d3.geo.path().projection(config.projection);
    config.a_data = map.data_cleanup_and_return_indexed_aggs(config, rows);
    config.bounds = [[0, 0], [config.width, config.height]];

    drawMarks(config, curr_map, elements);
    drawLatLong(config, rows);
    drawZipCodeBubbles(config, rows);
    drawStateMarks(config, state_map, state_elements);
    zoomSvg(config);
    createTooltips(config);
    return map;
  };

  function setupSizeScale(config, rows) {
    if (isBubbleMap(config)) {
      config.sizeFn = (row) => Math.sqrt((row[config.sizeColIndex]) / Math.PI);

      var extent = d3.extent(rows, config.sizeFn);
      config.size_scale = d3.scale.linear()
                            .domain(extent)
                            .range(mark_size_range.get_range());
    }
  }

  function getValidZipCodeRows(config, rows) {
    var valid_rows = [];
    let zipcode_latlong = fetchedCollateral[config.map_view].zipcode_latlong;
    rows.forEach(function(row) {
      var zipcode = row[config.zipcode_colindex],
          zipcode_s = zipcode + "";
      var l = zipcode_latlong[zipcode_s];
      if (!l) {
        ignored_rows_count++;
        return;
      }
      var xy = config.projection(l);
      if (xy) {
        row.__arc_x = xy[0];
        row.__arc_y = xy[1];
        row.__arc_r = config.size_scale(config.sizeFn(row));
        valid_rows.push(row);
        plotted_rows_count++;
      } else {
        // console.log('ignoring', latitude, longitude);
        ignored_rows_count++;
      }
    });
    if (!valid_rows) {
      lib.notificationMessage("Could not plot any rows. The data expected " +
          "is latitude and longitude within the US. Please look at the " +
          "generated data in the table visual.");
    }
    return valid_rows;
  }

  function drawZipCodeBubbles(config, rows) {
    if (config.map_type !== 'zipcode' || config.map_view === 'Canada') {
      // For canadian maps, we have a shape data for the zipcodes, so no need
      // to try and plot bubbles from latitude and longitudes.
      return;
    }

    var valid_rows = getValidZipCodeRows(config, rows);
    drawBubbles(config, valid_rows);
  }

  function getValidLatLongRows(config, rows) {
    //Only allow latitudes between -85 and 85 for now. The d3.projection for the
    //world map projects latitudes between 85 and 90 incorrectly.
    function isValidLatLong(lat, long) {
      return arc.misc.isNumeric(lat) &&
        arc.misc.isNumeric(long) &&
        (Number(lat) >= -85 && Number(lat) <= 85) &&
        (Number(long) >= -180 && Number(long) <= 180);
    }

    const valid_rows = [];
    rows.forEach(function(row) {
      const latitude = row[config.latitude_colindex];
      const longitude = row[config.longitude_colindex];

      const xy = isValidLatLong(latitude, longitude) ?
        config.projection([longitude, latitude]) :
        null;

      if (xy) {
        row.__arc_x = xy[0];
        row.__arc_y = xy[1];
        row.__arc_r = config.size_scale(config.sizeFn(row));
        valid_rows.push(row);
        plotted_rows_count++;
      } else {
        ignored_rows_count++;
      }
    });

    if (ignored_rows_count > 0) {
      lib.notificationMessage("Ignored " + ignored_rows_count + " rows. To " +
        "have points be visible on the map, only latitudes between -85 and 85 " +
        "degrees and longitudes between -180 and 180 are accepted.");

      ignored_rows_count = 0;
    }

    return valid_rows;
  }

  function drawLatLong(config, rows) {
    if (config.map_type !== 'latlong') {
      return;
    }

    var valid_rows = getValidLatLongRows(config, rows);
    drawBubbles(config, valid_rows);
  }

  function drawBubbles(config, valid_rows) {
    // Sort rows so smaller aggregates are drawn last, and the circles can be
    // moused over
    valid_rows.sort((a, b) => b[config.sizeColIndex] - a[config.sizeColIndex]);

    const circles = config.svg.selectAll('.nosel')
                          .data(valid_rows)
                          .enter()
                          .append('circle')
                          .attr('cx', d => d.__arc_x)
                          .attr('cy', d => d.__arc_y)
                          .attr('r', d => d.__arc_r)
                          .style('opacity', lib.colorOpacity())
                          .style('fill', d => config.color(d[config.colorColIndex]));

    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .highlight_on_select(true)
       .addSelection(circles)
       .text_fn(function(d, i) {
          return lib.text_from_columns(d, config.columns);
        })
       .params_fn(function(d, i, e, dimsOnly) {
          return lib.params_from_columns(d, config.columns, dimsOnly);
        })
       .permitPivot(true)
       .activate();
  }

  function maxBounds(b1, b2) {
    var r = [[0, 0], [0, 0]];
    r[0][0] = Math.min(b1[0][0], b2[0][0]);
    r[0][1] = Math.min(b1[0][1], b2[0][1]);
    r[1][0] = Math.max(b1[1][0], b2[1][0]);
    r[1][1] = Math.max(b1[1][1], b2[1][1]);
    return r;
  }

  function computeBounds(config, features) {
    // No a_data implies no bounds
    if (!config.a_data) {
      return;
    }

    // Look for bounds within features that we are going to color
    config.bounds = undefined;
    features.forEach(function(d) {
      if (!(config.a_data_keyGetter(d) in config.a_data)) {
        // We are not tracking this feature in the data
        return;
      }

      var b = config.path.bounds(d);
      var hasInvalidBound = b.some(corner => corner.some(coor =>
        typeof coor !== 'number' || !isFinite(coor)));

      if (hasInvalidBound) {
        // Don't track features with invalid bounds
        return;
      }

      if (!config.bounds) {
        config.bounds = b;
      } else {
        config.bounds = maxBounds(b, config.bounds);
      }
    });
  }

  function drawMarks(config, curr_map, elements) {
    let a_data_keyGetter = config.a_data_keyGetter;
    const { features } = feature(curr_map, elements);

    // For state, county & dma maps, we support zooming in to the active area
    if (zoom_to_active.isChecked && (['country', 'county', 'dma', 'state'].includes(config.map_type) ||
      (config.map_type === 'zipcode' && config.map_view === 'Canada'))) {
      computeBounds(config, features);
    }

    var o = lib.colorOpacity();
    var circles_data = [];
    config.marks =
      config.svg
         .selectAll(".nosel")
         .data(features)
         .enter()
           .append("path")
           .classed("viz-map-path", true)
           .attr("d", config.path)
           .attr("stroke-width", '.5')
           .attr("stroke", function() {
             if (['county', 'dma'].includes(config.map_type)) {
               return "lightgray";
             } else {
               return "gray";
             }
           })
           .attr("fill", function(d) {
             if (map.map_bubbles.isChecked) {
               if (a_data_keyGetter(d) in config.a_data) {
                 circles_data.push(d);
               }
               return "white";
             }
             if (!config.a_data) {
               return "white";
             }
             if (!(a_data_keyGetter(d) in config.a_data)) {
               return "white";
             }
             return config.color(config.a_data[a_data_keyGetter(d)][config.colorColIndex]);
           });

    if (o) {
      config.marks.attr('fill-opacity', o);
    }

    if (map.map_bubbles.isChecked) {
      // Over-write config.marks
      config.marks = config.svg.selectAll('#nosel')
              .data(circles_data)
              .enter()
                .append('circle')
                .attr('transform', function(d) {
                  return "translate(" + config.path.centroid(d) + ")";
                })
                .attr("r", function(d) {
                  return Number(config.size_scale(config.sizeFn(config.a_data[a_data_keyGetter(d)])));
                })
                .style("opacity", lib.colorOpacity())
                .style("fill", function(d) {
                  return config.color(config.a_data[a_data_keyGetter(d)][config.colorColIndex]);
                });
    }
  }

  function drawStateMarks(config, curr_map, state_elements) {
    // Draw state marks in gray for these kinds of maps
    if (!state_elements || !(['county', 'dma', 'latlong'].includes(config.map_type))) {
      return;
    }

    const { features } = feature(curr_map, state_elements);
    var state_marks =
      config.svg.selectAll('#nosel')
            .data(features)
            .enter()
              .append('path')
              .classed('viz-map-state', true)
              .attr('d', config.path)
              .attr('stroke', 'gray')
              .attr('fill', 'none');
  }

  function zoomSvg(config) {
    let minZoom = 1;

    if (zoom_to_active.isChecked && config.bounds) {
      var w_new = config.bounds[1][0] - config.bounds[0][0],
          h_new = config.bounds[1][1] - config.bounds[0][1];
      if ((w_new < config.width/2) || (h_new < config.height/2)) {
        var b = config.bounds;
        var dx = b[1][0] - b[0][0],
            dy = b[1][1] - b[0][1],
            x = (b[0][0] + b[1][0]) / 2,
            y = (b[0][1] + b[1][1]) / 2,
            scale = 0.75 / Math.max(dx / config.width, dy / config.height),
            translate = [config.width / 2 - scale * x,
                         config.height / 2 - scale * y];

        minZoom = 1 / scale;
        config.svg.transition()
              .duration(750)
              .style("stroke-width", 1.5 / scale + "px")
              .attr("transform",
                  "translate(" + translate + ")scale(" + scale + ")");
      }
    }

    if (zoom_enabled.isChecked) {
      svgPanZoom(`#${config.base_svg_id}`, {
        contain: false,
        fit: false,
        center: false,
        minZoom,
      });
    }
  }

  function createTooltips(config) {
    if (config.map_type === 'latlong') {
      return;
    }

    const dataCopy = arc.misc.OBJ.sliceRecursive(config.a_data);
    function params_fn(d) {
      if (config.a_data_keyGetter(d) in dataCopy) {
        const row = dataCopy[config.a_data_keyGetter(d)];
        return lib.params_from_columns(row, config.columns);
      }
      return [];
    }

    function text_fn(d) {
      if (config.a_data_keyGetter(d) in dataCopy) {
        const row = dataCopy[config.a_data_keyGetter(d)];
        return lib.text_from_columns(row, config.columns);
      }
      return '';
    }

    arc.marks()
       .permitPivot(true)
       .viz_lib(lib)
       .marker_name('primary')
       .highlight_on_select(true)
       .addSelection(config.marks)
       .text_fn(text_fn)
       .params_fn(params_fn)
       .activate();

    return map;
  }

  map.data_cleanup_and_return_indexed_aggs = function(config, rows) {
    if (config.map_type === 'latlong' || (config.map_type === 'zipcode' &&
      config.map_view !== 'Canada')) {
        return {};
    }

    plotted_rows_count = 0;
    ignored_rows_count = 0;
    var a_data = {};

    let processRowFunc = processRowFunctions[config.map_type];

    rows.forEach(function(d) {
      d[config.colorColIndex] = Number(d[config.colorColIndex]);
      processRowFunc(d, config, a_data);
    });
    if (!plotted_rows_count) {
      lib.notificationMessage('Could not plot any rows. The data expected ' +
          'is country, state, and county names. Please look at the generated ' +
          'data in the table visual');
    }
    return a_data;
  };

  const processRowFunctions = {
    'county': processCountyRow,
    'zipcode': processZipcodeRow,
    'dma': processDmaRow,
    'state': processStateRow,
    'country': processCountryRow,
  };

  function processCountryRow(d, config, a_data) {
    var c = d[config.country_colindex].toLowerCase();
    if (c in country_names_to_codes) {
      plotted_rows_count++;
      a_data[country_names_to_codes[c]] = d;
    } else {
      ignored_rows_count++;
    }
  }

  function processDmaRow(d, config, a_data) {
    if (isNaN(d[config.dma_colindex])) {
      ignored_rows_count++;
      return;
    }
    d[config.dma_colindex] = Number(d[config.dma_colindex]);
    if (d[config.dma_colindex] in dma_valid_ids) {
      plotted_rows_count++;
      a_data[d[config.dma_colindex]] = d;
    } else {
      ignored_rows_count++;
    }
  }

  function processStateRow(d, config, a_data) {
    if (config.map_view === 'Canada') {
      if (d[config.state_colindex] in valid_canadian_territories) {
        plotted_rows_count++;
        a_data[d[config.state_colindex]] = d;
      } else {
        ignored_rows_count++;
      }
    } else {
      if (d[config.state_colindex] in names_to_fips) {
        plotted_rows_count++;
        a_data[names_to_fips[d[config.state_colindex]]] = d;
      } else {
        ignored_rows_count++;
      }
    }
  }

  function processZipcodeRow(d, config, a_data) {
    if (config.map_view === 'Canada') {
      if (d[config.zipcode_colindex] in valid_canadian_fsas) {
        plotted_rows_count++;
        a_data[d[config.zipcode_colindex]] = d;
      } else {
        ignored_rows_count++;
      }
    }
  }

  function processCountyRow(d, config, a_data) {
    // This code will strip the following suffixes off of cSearch in the order specified
    // in the array of truncations using.  To remove an additional suffix add
    // it to the array (regex syntax applies).
    const truncations = ['city and county', 'census area', 'county', 'parish', 'borough', 'area'];
    const truncate = (s, e) => s.replace(new RegExp(` ${e}$`), '').trim();
    let cSearch = String(d[config.county_colindex]).toLowerCase().replace(/^st\.\s/, 'st ');
    d.county_search = truncations.reduce((a, b) => truncate(a, b), cSearch);
    d.state_search = String(d[config.state_colindex]).toLowerCase();
    var k = [d.state_search, d.county_search];

    if (k in county_name_to_fips) {
      plotted_rows_count++;
      a_data[county_name_to_fips[k]] = d;
    } else {
      ignored_rows_count++;
    }
  }

  // Note than when used in drill-into scenarios, there may be state + ZIP code
  // in the Geo shelf, or state + dma. For the drill-into scenario to work correctly,
  // it is important that if the higher resolution dimension be used as the primary
  // geo field.
  map.compute_map_type = function(config) {
    config.country_colindex = -1;
    config.state_colindex = -1;
    config.county_colindex = -1;
    config.longitude_colindex = -1;
    config.latitude_colindex = -1;
    config.zipcode_colindex = -1;

    config.errors = false;

    const geo_columns = lib.shelf_columns(config.columns, 'geo_shelf');
    const geo_element_locations = arc.misc.geoCastLocations(geo_columns);

    // The ordering below is important - go from highest resolution to lowest resolution
    if ((geo_element_locations.latitude >= 0) && (geo_element_locations.longitude >= 0)) {
      config.longitude_colindex = geo_element_locations.longitude;
      config.latitude_colindex = geo_element_locations.latitude;
      config.map_type = 'latlong';
      return map;
    }

    if (geo_element_locations.zipcode >= 0) {
      config.zipcode_colindex = geo_element_locations.zipcode;
      config.map_type = 'zipcode';
      return map;
    }

    if (geo_element_locations.dma >= 0) {
      config.dma_colindex = geo_element_locations.dma;
      config.map_type = 'dma';
      return map;
    }

    if (geo_element_locations.county >= 0) {
      if (geo_element_locations.state === undefined) {
        lib.notificationMessage("Error: A field geo-typed to 'state' must be specified if a " +
            "field geo-typed to 'county' is specified");
        return map;
      }

      config.county_colindex = geo_element_locations.county;
      config.state_colindex = geo_element_locations.state;
      config.map_type = 'county';
      return map;
    }

    if (geo_element_locations.state >= 0) {
      config.state_colindex = geo_element_locations.state;
      config.map_type = 'state';
      return map;
    }

    if (geo_element_locations.country >= 0) {
      config.country_colindex = geo_element_locations.country;
      config.map_type = 'country';
      return map;
    }

    if ((geo_element_locations.latitude >= 0) || (geo_element_locations.longitude >= 0)) {
      lib.notificationMessage("Error: 'longitude' and 'latitude' should be specified together");
      return map;
    }

    lib.notificationMessage("Error: Could not detect any geo-typed fields.");
    config.errors = true;
    return map;
  };

  map.collateral_is_available = function(config) {
    const requirements = mapRequirements[config.map_view][config.map_type];

    if (!requirements) {
      lib.alertMessage(`Error: Unsupported map configuration. For ${config.map_view},` +
        ` only ${Object.keys(mapRequirements[config.map_view]).join(', ')} maps` +
        ` are allowed.`);
      config.errors = true;
      return false;
    }

    config.requirements = requirements;
    const collateral = fetchedCollateral[config.map_view];
    let hasAllReqs = true;
    requirements.forEach(requirement => {
      hasAllReqs = hasAllReqs && collateral[requirement];
    });

    return hasAllReqs;
  };

  map.fetch_collateral = function(config, cb) {
    let map_view = config.map_view;
    let requirementNames = config.requirements;
    let requiredAssets = requirementNames.map(req_name => collateralInfo[req_name]);
    arc.staticfetcher.fetch(requiredAssets, function(error, ...fetchedData) {
      if (error) {
        lib.alertMessage("Error fetching map metadata");
        return;
      }

      fetchedData.forEach((d, i) => {
        fetchedCollateral[map_view][requirementNames[i]] = d;
      });

      postprocessMappingData(config, requirementNames, cb);
    });

    return map;
  };

  function postprocessMappingData(config, requirementNames, cb) {
    requirementNames.forEach(asset => {
      let assetData = fetchedCollateral[config.map_view][asset];

      switch (asset) {
        case 'county_fips':
          assetData.forEach(function(d) {
            d.CFIPS = Number(d.CFIPS);
            d.SFIPS = Number(d.SFIPS);
            d.state = d.State.toLowerCase();
            d.county = d.County.toLowerCase();
            county_fips_to_names[d.SFIPS * 1000 + d.CFIPS] = d;
            county_name_to_fips[[d.state, d.county]] = d.SFIPS * 1000 + d.CFIPS;
            county_name_to_fips[
              [state_codes_names_map[d.state].toLowerCase(), d.county]
            ] = d.SFIPS * 1000 + d.CFIPS;
          });
          break;
        case 'state_fips':
          let toPlotCounty = (requirementNames.indexOf('county_fips') >= 0);
          assetData.forEach(function(d) {
            d.FIPS = Number(d.FIPS);
            state_fips_to_names[d.FIPS] = d.s;
            state_fips_to_short_names[d.FIPS] = d.sa;
            names_to_fips[d.s] = d.FIPS;
            names_to_fips[d.sa] = d.FIPS;
            if (toPlotCounty) {
              state_codes_names_map[d.s.toLowerCase()] = d.sa;
            }
          });
          break;
        case 'country_codes':
          assetData.forEach(function(d) {
            const dCode = Number(d.code);
            country_codes_to_names[dCode] = d.name.toLowerCase();
            country_names_to_codes[d.name.toLowerCase()] = dCode;
            country_names_to_codes[d.a2.toLowerCase()] = dCode;
            country_names_to_codes[d.a3.toLowerCase()] = dCode;
          });
          break;
        case 'dma_map_data':
          assetData.objects.nielsen_dma.geometries.forEach(function(d) {
            dma_valid_ids[d.id] = true;
          });
          break;
        case 'canada_topojson_quantized':
          assetData.objects.canada_geojson.geometries.forEach(d => {
            valid_canadian_territories[d.properties.PRENAME] = true;
          });
          break;
        case 'fsa_topojson_quantized':
          assetData.objects.fsa.geometries.forEach(d => {
            valid_canadian_fsas[d.properties.CFSAUID] = true;
          });
          break;
      }
    });

    cb();
    return map;
  }

  return map;
};

arc.visual.plugins.map_extension = function() {
  var ext = function() { return "arc.visual.plugins.map_extension"; };

  ext.viz_name = function() {
    return "map";
  };

  ext.viz_tooltip = function() {
    return "map";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_map.png?v=1";
  };

  ext.viz = arc.visual.plugins.map;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.map_extension());

}());
