window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.network = function() {
  var network = arc.visual.plugins.chord('network');
  var layouts = [],
      drags = [];

  function findSetting(settingId) {
    var lib = network.viz_lib();

    return lib.user_settings().find_by_id(settingId);
  }

  function srcNodeMarkType() {
    return findSetting('src_node_mark_type').value();
  }

  function srcNodeOpacity() {
    return network.opacity('src_node_opacity');
  }

  function combinedNodeMarkType() {
    return findSetting('combined_node_mark_type').value();
  }

  function tgtNodeMarkType() {
    return findSetting('tgt_node_mark_type').value();
  }

  function nodeSizeRange() {
    return findSetting('node_size_range')
                        .get_range().map(function(d) {
                          return d * d;
                        });
  }

  function linkDistance() {
    var v = findSetting('link_distance').value();

    if (v === '') {
      v = undefined;
    }

    return v;
  }

  function linkStrength() {
    return findSetting('link_strength').value();
  }

  function defaultLinkColor() {
    return findSetting('link_color').value();
  }

  function nodeFriction() {
    return findSetting('node_friction').value();
  }

  function nodeCharge() {
    return findSetting('node_charge').value();
  }

  function layoutGravity() {
    return findSetting('layout_gravity').value();
  }

  function layoutTheta() {
    return findSetting('layout_theta').value();
  }

  function includeDirectionArrows() {
    return findSetting('add_direction_arrow').value() === 'true';
  }

  function unifyNodes() {
    return findSetting('same_domain').value() === 'true';
  }

  function nodeColors(getSettings = false) {
    let colorData = JSON.parse(findSetting('color_palette').value());
    if (getSettings) {
      return colorData.colorSettings;
    }

    return colorData.isReversed ? colorData.colors : colorData.colors.reverse();
  }

  function linkColors(getSettings = false) {
    let colorData = JSON.parse(findSetting('link_color_palette').value());
    if (getSettings) {
      return colorData.colorSettings;
    }

    return colorData.isReversed ? colorData.colors : colorData.colors.reverse();
  }

  function directionArrowSize() {
    return findSetting('direction_arrow_size').value();
  }

  function addNodeLegend() {
    return findSetting('add_node_legend').value() === 'true';
  }

  function addLinkLegend() {
    return findSetting('add_link_legend').value() === 'true';
  }

  function networkLegendWidth() {
    return Number(findSetting('network_legend_width').value());
  }

  function nodeLegendTitle() {
    return findSetting('node_legend_title').value();
  }

  function linkLegendTitle() {
    return findSetting('link_legend_title').value();
  }

  function linkPointerColor() {
    return findSetting('link_pointer_color').value();
  }

  function showNodeLabels() {
    return findSetting('show_node_labels').value() === 'true';
  }

  network.nodesAndLinks = function() {
    var f = function() {
      return 'nodesAndLinks';
    };

    var nodes = [],
        nodeHash = {},
        links = [],
        srcNodeHash = {},
        tgtNodeHash = {},
        srcNodes = [],
        tgtNodes = [],
        nodeColorNumeric = 0,
        linkColorNumeric = 0;

    f.nodes = function() {
      return nodes;
    };

    f.srcNodes = function() {
      return srcNodes;
    };

    f.tgtNodes = function() {
      return tgtNodes;
    };

    f.nodeHash = function() {
      return nodeHash;
    };

    f.links = function() {
      return links;
    };

    f.srcNodeHash = function() {
      return srcNodeHash;
    };

    f.tgtNodeHash = function() {
      return tgtNodeHash;
    };

    function createNode(key, nodeInfo, ignore, linkIgnore, combine) {

      /*
      key: The name of the node, the value passed into the 'nodes' shelf
      nodeInfo: object with the following:
        type: 'src' or 'tgt'
        size: numeric value representing size of the node
        color: numeric or dimension value representing the color of the node
        data: array of info from the row of data
       ignore: array of indices of the data array that are not pertinent to this node
       linkIgnore: list of indices of the data array that are only pertinent to the link
       combine: true or false. If true that means nodes can be both source and target
                if false, nodes must be either source or target, not both
       */
      let node;
      if (nodeHash[key] && combine) {
        node = nodeHash[key];
        node.connections += 1;
        if (node.type !== nodeInfo.type) { // when there is a combined node
          node.type = 'both';
          node.ignore.forEach(function(idx) {
            node.tooltipData[idx] = nodeInfo.data[idx];
          });

          node.ignore = linkIgnore;
        }

      } else if (srcNodeHash[key] && nodeInfo.type === 'src') {
        node = srcNodeHash[key];
        node.connections += 1;
      } else if (tgtNodeHash[key] && nodeInfo.type === 'tgt') {
        node = tgtNodeHash[key];
        node.connections += 1;
      } else {
        node = {
          type: nodeInfo.type,
          size: nodeInfo.size,
          color: nodeInfo.color,
          data: nodeInfo.data,
          index: nodes.length,
          connections: 1,
          tooltipData: nodeInfo.data.slice(0),
          ignore: ignore.concat(linkIgnore),
          key: key,
        };

        if (combine) {
          nodeHash[key] = node;
        } else if (nodeInfo.type === 'src') {
          srcNodeHash[key] = node;
          srcNodes.push(node);
        } else {
          tgtNodeHash[key] = node;
          tgtNodes.push(node);
        }

        nodes.push(node);
      }

      return node;
    }

    f.process = function(config) {
      let srcSizeIndex, tgtSizeIndex, srcColorIndex, tgtColorIndex, linkWidthIndex, linkColorIndex;
      let srcIgnore = [config.dim_startidx + 1];
      let tgtIgnore = [config.dim_startidx];
      let linkIgnore = [config.agg_startidx];

      if (config.srcSizeCol) {
        srcSizeIndex = config.srcSizeCol.colindex();
        tgtIgnore.push(srcSizeIndex);
      }

      if (config.tgtSizeCol) {
        tgtSizeIndex = config.tgtSizeCol.colindex();
        srcIgnore.push(tgtSizeIndex);
      }

      if (config.srcColorCol) {
        srcColorIndex = config.srcColorCol.colindex();
        tgtIgnore.push(srcColorIndex);
      }

      if (config.tgtColorCol) {
        tgtColorIndex = config.tgtColorCol.colindex();
        srcIgnore.push(tgtColorIndex);
      }

      if (config.linkColorCol) {
        linkColorIndex = config.linkColorCol.colindex();
        linkIgnore.push(linkColorIndex);
      }

      if (config.linkWidthCol) {
        linkWidthIndex = config.linkWidthCol.colindex();
        linkIgnore.push(linkWidthIndex);
      }

      config.rows.forEach(function(d) {
        const defaultSize = 20,
        defaultColor = '#ddd';
        let srcNodeSize, tgtNodeSize, srcNodeColor, tgtNodeColor, linkWidth, linkColor, linkClrSrc;

        if ($.isNumeric(d[linkWidthIndex])) {
          linkWidth = Number(d[linkWidthIndex]);
        } else {
          linkWidth = 5;
        }

        srcNodeSize = $.isNumeric(d[srcSizeIndex]) ? d[srcSizeIndex] : defaultSize;
        tgtNodeSize = $.isNumeric(d[srcSizeIndex]) ? d[tgtSizeIndex] : defaultSize;
        srcNodeColor = d[srcColorIndex] || 'default';
        tgtNodeColor = d[tgtColorIndex] || 'default';

        if (linkColorIndex) {
          linkColor = d[linkColorIndex];
          linkClrSrc = 'shelf';
        } else {
          switch (defaultLinkColor()) {
            case 'source':
              linkColor = srcNodeColor;
              linkClrSrc = 'node';
              break;
            case 'target':
              linkColor = tgtNodeColor;
              linkClrSrc = 'node';
              break;
            case 'aggregate':
              linkColor = linkWidth;
              linkClrSrc = 'link';
              break;
            case 'fixed':
              linkColor = defaultColor;
              linkClrSrc = 'fixed';
          }
        }

        let sourceNode,
            targetNode;

        let srcNodeVal = d[config.dim_startidx];
        let tgtNodeVal = d[config.dim_startidx + 1];

        let srcNodeInfo = { color: srcNodeColor, size: srcNodeSize, type: 'src', data: d };
        let tgtNodeInfo = { color: tgtNodeColor, size: tgtNodeSize, type: 'tgt', data: d };

        // TODO: implment various methods of default node coloring and sizing

        sourceNode = createNode(srcNodeVal, srcNodeInfo, srcIgnore, linkIgnore, unifyNodes());
        targetNode = createNode(tgtNodeVal, tgtNodeInfo, tgtIgnore, linkIgnore, unifyNodes());

        links.push({
          source: sourceNode,
          target: targetNode,
          data: d,
          color: linkColor,
          size: linkWidth,
          colorSrc: linkClrSrc,
        });
      });

      return f;
    };

    return f;
  };

  network.viz_drawVisualNetwork = function(config, viz_div) {
    var lib = network.viz_lib();

    if (config.rows.length === 0) {
      return;
    }

    var nodes,
        nodeHash,
        srcNodeHash,
        tgtNodeHash,
        links;

    config.srcSizeCol = lib.first_column(config.columns, 'source_node_size_shelf');
    config.tgtSizeCol = lib.first_column(config.columns, 'target_node_size_shelf');
    config.srcColorCol = lib.first_column(config.columns, 'source_node_color_shelf');
    config.tgtColorCol = lib.first_column(config.columns, 'target_node_color_shelf');
    config.linkColorCol = lib.first_column(config.columns, 'link_color_shelf');
    config.linkWidthCol = lib.first_column(config.columns, 'aggregates_shelf');

    var nl = network.nodesAndLinks().process(config);
    nodes = nl.nodes();
    nodeHash = nl.nodeHash();
    links = nl.links();
    tgtNodeHash = nl.tgtNodeHash();
    srcNodeHash =  nl.srcNodeHash();

    var n1 = Math.sqrt(nodes.length),
        n2 = Math.ceil(Math.sqrt(n1));
    nodes.forEach(function(d, i) {
      d.x = config.width / n1 * (i / n2);
      d.y = config.height / n1 * (i % n2);
    });

    var force = d3.layout.force()
                  .nodes(nodes)
                  .links(links)
                  .size([config.width, config.height])
                  .linkStrength(linkStrength())
                  .friction(nodeFriction())
                  .linkDistance(linkDistance() || Math.min(config.width / 4, config.height / 4))
                  .charge(nodeCharge())
                  .gravity(layoutGravity())
                  .theta(layoutTheta())
                  .on('end', end)
                  .on('tick', tick);

    layouts.push(force);

    const hasLinkLegend = addLinkLegend() &&
      (config.linkColorCol || (defaultLinkColor() === 'aggregate' && config.linkWidthCol));

    if (hasLinkLegend || addNodeLegend()) {
      config.width -= (networkLegendWidth() - 30);
    }

    var svg = d3.select('#' + viz_div).append('svg')
                .attr('width', config.width)
                .attr('height', config.height)
                  .append('g')
                  .attr('class', 'viz-network');
    let allNodes = svg.selectAll('.nosel');
    var link = svg.selectAll('.link');

    var nodeExtent = d3.extent(nodes, function(d) {
      return Number(d.size);
    });

    var linkExtent = d3.extent(links, function(d) {
      return Number(d.size);
    });

    function getColorDomain(objArray) {
      const colorDomain = [];
      const colorHash = {};
      let isContinuous = false;
      let isNumeric = true;

      function createColorDomain(obj) {
        if (!$.isNumeric(obj.color)) {
          isNumeric = false;
        }

        if ($.isNumeric(obj.color) && Math.floor(Number(obj.color)) !== Number(obj.color)) {
          isContinuous = true;
        }

        if (colorHash[obj.color] === undefined) {
          colorHash[obj.color] = true;
          colorDomain.push(obj.color);
        }
      }

      objArray.forEach(createColorDomain);
      return { domain: colorDomain, isContinuous, isNumeric };
    }

    let linkScale = d3.scale.linear()
                      .domain(linkExtent)
                      .range(network.linkWidthRange());

    let nodeScale = d3.scale.linear()
                      .domain(nodeExtent)
                      .range(nodeSizeRange());

    const nodeColorInfo = getColorDomain(nodes);
    const nodeColorDomain = nodeColorInfo.isNumeric ? nodeColorInfo.domain.sort((a, b) => a - b) :
      nodeColorInfo.domain.sort();
    const nodeColorRange = nodeColors();
    let nodeColorScale;
    let nodeColorExtent;
    let nodeColorIsDim;

    if (config.srcColorCol && config.srcColorCol.isNumeric() &&
      config.tgtColorCol && config.tgtColorCol.isNumeric()) {
      nodeColorInfo.isContinuous = true;
    }

    if (nodeColorInfo.isContinuous) {
      nodeColorExtent = d3.extent(nodes, d => d === 'default' ? 0 : Number(d.color));
      nodeColorScale = arc.colorpalette.aggScale(nodeColorExtent, nodeColorRange, nodeColors(true));
      nodeColorIsDim = false;
    } else {
      nodeColorScale = lib.dimensionalColorScale();
      nodeColorScale.range(nodeColorRange);
      nodeColorScale.domain(nodeColorDomain);
      nodeColorIsDim = true;
    }

    function getNodeColor(color) {
      if (!nodeColorIsDim) {
        return nodeColorScale(Number(color));
      } else {
        return nodeColorScale(color);
      }
    }

    const linkColorInfo = getColorDomain(links);
    const linkColorDomain = linkColorInfo.isNumeric ? linkColorInfo.domain.sort((a, b) => a - b) :
      linkColorInfo.domain.sort();
    const linkColorRange = linkColors();
    let linkColorScale;
    let linkColorExtent;
    let linkColorIsDim;

    if ((config.linkColorCol && config.linkColorCol.isNumeric()) ||
      (!config.linkColorCol && defaultLinkColor() === 'aggregate')) {
      linkColorInfo.isContinuous = true;
    }

    if (linkColorInfo.isContinuous) {
      linkColorExtent = d3.extent(links, function(d) {
        if (d.color && !arc.misc.check_if_value_null(d.color)) {
          return Number(d.color);
        } else {
          return 0;
        }
      });

      linkColorScale = arc.colorpalette.aggScale(linkColorExtent, linkColorRange, linkColors(true));
      linkColorIsDim = false;
    } else {
      linkColorScale = arc.misc.dimensionColorScale();
      linkColorScale.range(linkColorRange);
      linkColorScale.domain(linkColorDomain);
      linkColorIsDim = true;
    }

    function getLinkColor(color) {
      if (linkColorIsDim) {
        return linkColorScale(color);
      } else {
        return linkColorScale(Number(color));
      }
    }

    let uniqueArrowsId = `directional-arrows-${viz_div}`;
    if (includeDirectionArrows()) {

      svg.append('svg:defs').selectAll('marker')
          .data([uniqueArrowsId])
        .enter().append('svg:marker')
          .attr('id', uniqueArrowsId)
          .attr('viewBox', '0 -5 8 10')
          .attr('refX', 15)
          .attr('refY', 0)
          .attr('markerWidth', directionArrowSize())
          .attr('markerHeight', directionArrowSize())
          .attr('orient', 'auto')
        .append('svg:path')
          .attr('d', 'M0,-5L10,0L0,5')
          .attr('fill', linkPointerColor())
          .attr('opacity', 0.8);
    }

    link = link.data(links)
               .enter()
                 .append('line')
                 .attr('class', 'link')
                 .style('stroke', function(d) {

                   if (d.colorSrc === 'fixed') {
                     return d.color;
                   } else if (d.colorSrc === 'node') {
                     return getNodeColor(d.color);
                   } else {
                     return getLinkColor(d.color);
                   }
                 })
                 .style('stroke-opacity', network.linkOpacity())
                 .style('stroke-width', d => linkScale(d.size));

    if (includeDirectionArrows()) {
      link.attr('marker-end', `url(#${uniqueArrowsId})`);
    }

    var nodeMarks = arc.marks();

    var drag = force.drag()
                    .on('dragstart', function(d) {
                      d.w = config.width;
                      d.h = config.height;
                      arc.misc.addClass($(this), 'fixed');
                      lib.page_needs_save();
                      d.fixed = true;
                    })
                    .on('dragend', function(d) {
                      d3.event.sourceEvent.stopPropagation();
                    });

    drags.push(drag);

    allNodes = allNodes.data(nodes)
                        .enter()
                          .append('g')
                          .call(drag);

    allNodes.append('path')
             .each(function(d) {
               if (d.type === 'tgt') {
                 nodeHash[d.data[config.dim_startidx + 1]] = this;
                 tgtNodeHash[d.data[config.dim_startidx + 1]] = this;
               } else {
                 nodeHash[d.data[config.dim_startidx]] = this;
                 srcNodeHash[d.data[config.dim_startidx]] = this;
               }
             })
             .attr('class', 'viz-node-src viz-node')
             .attr('d', d3.svg.symbol()
                          .type(function(d) {
                            if (d.type === 'tgt') {
                              return tgtNodeMarkType();
                            } else if (d.type === 'src') {
                              return srcNodeMarkType();
                            } else {
                              return combinedNodeMarkType();
                            }
                          })
                          .size(d => nodeScale(Number(d.size))))
             .style('fill-opacity', srcNodeOpacity())
             .style('fill', d => getNodeColor(d.color));

    if (showNodeLabels()) {
      try {
        allNodes.append('text')
                .attr('dx', 12)
                .attr('dy', '.35em')
                .text(d => d.key);
      } catch (error) {
        console.log('Network node labels failed');
        console.log(error);
      }
    }

    try {
      nodeMarks
      .viz_lib(lib)
      .marker_name('nodes')
      .highlight_on_select(true)
      .addSelection(allNodes)
      .text_fn(function(d) {
        return lib.base_text_fn(d.tooltipData, {
          ignore_colindices: d.ignore,
        });
      })
      .params_fn(function (d) {
        return lib.params_from_columns(d.tooltipData, config.columns, true, undefined, undefined,
          d.ignore);
      })
      .combineWithOr(true)
      .activate();

      var linkMarks = arc.marks()
      .viz_lib(lib)
      .marker_name('nodes')
      .highlight_on_select(true)
      .addSelection(link)
      .text_fn(d => lib.base_text_fn(d.data))
      .params_fn(d => lib.params_from_columns(d.data, config.columns))
      .activate();

    } catch (error) {
      console.log('network tooltips failed');
      console.log(error);
    }

    if (addNodeLegend() || hasLinkLegend) {
      try {
        let legendDivClass = 'arc-legend-container';
        if (lib.inModal()) {
          legendDivClass += '-modal';
        }

        if (arc.globals.is_edit) {
          legendDivClass += '-edit';
        }

        const legendWidth = networkLegendWidth();
        const $thisDiv = $(`#${viz_div}`);
        const $legendTD = $thisDiv.siblings();
        const $legendDiv = $('<div/>').addClass(legendDivClass)
                                      .css('width', `${legendWidth + 10}px`)
                                      .css('height', config.height + 'px');
        $legendTD.append($legendDiv);

        let addLegend = function(type, legendDivClassName, legendWidth, legendTitle) {
          if (!legendWidth) {
            legendWidth = 150;
          }

          let legendData;
          if (type === 'node') {
            legendData = {
              legends: nodeColorIsDim ? nodeColorDomain : nodeColorExtent,
              legendType: nodeColorIsDim ? 'discrete' : 'continuous',
              color: nodeColorScale,
              color_palette: nodeColors(),
              colorIsDim: nodeColorIsDim,
              legendID: 'node',
              colorSettings: nodeColors(true),
            };
          } else {
            legendData = {
              legends: linkColorIsDim ? linkColorDomain : linkColorExtent,
              legendType: linkColorIsDim ? 'discrete' : 'continuous',
              color: linkColorScale,
              color_palette: linkColors(),
              colorIsDim: linkColorIsDim,
              legendID: 'link',
              colorSettings: linkColors(true),
            };
          }

          const legendHeight = legendData.legends.length * 15 + 30;
          const legendSVG = d3.select(`.${legendDivClassName}`)
                            .append('svg')
                            .attr('width', legendWidth)
                            .attr('height', legendHeight).style('float', 'left');

          let legendConfig = {
            svg: legendSVG,
            legendType: legendData.legendType,
            legendPositioning: 'Right',
            height: legendHeight,
            width: 0,
            legends: legendData.legends,
            color: legendData.color,
            color_is_dim: legendData.colorIsDim,
            color_palette: legendData.color_palette,
            legend_width: legendWidth,
            legend_title: legendTitle,
            sort_domain: 'true',
            legendID: legendData.legendID,
            colorSettings: legendData.colorSettings,
          };

          lib.draw_legend(legendConfig);
        };

        if (addNodeLegend() && (config.srcColorCol || config.tgtColorCol)) {
          addLegend('node', legendDivClass, legendWidth, nodeLegendTitle());
        }

        if (hasLinkLegend) {
          addLegend('link', legendDivClass, legendWidth, linkLegendTitle());
        }
      }
      catch (error) {
        console.log('Network legend rendering failed');
        console.log(error);
      }

    }

    if (network.rememberNodePositions()) {
      network.fixed_nodes().forEach(function(d) {
        var n = nodeHash[d.key];
        if (n) {
          n.x = d.x * config.width / d.w;
          n.y = d.y * config.height / d.h;
          n.w = config.width;
          n.h = config.height;
          n.fixed = true;
          arc.misc.addClass($(n.elem), 'fixed');
        }
      });
    }

    force.start();

    function end() {
      setPositions();
    }

    function tick() {
      setPositions();
    }

    var padding = 5;
    function constraint(v, min, max) {
      return Math.min(Math.max(v, min + padding), max - padding);
    }

    function setPositions() {
      link.each(function(d) {
        d.source.x = constraint(d.source.x, 0, config.width);
        d.source.y = constraint(d.source.y, 0, config.height);
        d.target.x = constraint(d.target.x, 0, config.width);
        d.target.y = constraint(d.target.y, 0, config.height);
      });

      link.attr('x1', function(d) { return d.source.x; })
          .attr('y1', function(d) { return d.source.y; })
          .attr('x2', function(d) { return d.target.x; })
          .attr('y2', function(d) { return d.target.y; });

      allNodes.each(function(d) {
        d.x = constraint(d.x, 0, config.width);
        d.y = constraint(d.y, 0, config.height);
      });

      allNodes.attr('transform', function(d) {
         return 'translate(' + d.x + ',' + d.y + ')';
      });
    }
  };

  network.viz_destroyNetwork = function() {
    // Something else in d3 is leaking memory, unsure of what it is as of now.
    layouts.forEach(force => {
      force.on('end', undefined)
           .on('tick', undefined);
    });

    drags.forEach(drag => {
      drag.on('dragstart', undefined)
          .on('dragend', undefined);
    });

    layouts = [];
    drags = [];
  };

  return network;
};

arc.visual.plugins.network_extension = function() {
  var ext = function() { return 'arc.visual.plugins.network_extension'; };

  ext.viz_name = function() {
    return 'network';
  };

  ext.viz_tooltip = function() {
    return 'network';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_network.png?v=1';
  };

  ext.viz = arc.visual.plugins.network;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.network_extension());

}());
