window.arc = window.arc || {};

(function() {
"use strict";

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.pb = function() {
  var saved_settings = {},
      data_result,
      lib,
      trellis,
      sort_orders_asc = {},
      previousDataTracker = arc.transition.dataTracker(),
      colorFetcher = arc.canvas.colorFetcher(),
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

const legendWidthOverride = arc.report.user_settings.legend_width_override();
const reverseLegend = arc.report.user_settings.reverse_legend_order();
const legend_style = arc.report.user_settings.legend_style();
legend_style.addDependents(
  [legendWidthOverride, reverseLegend],
  ['None'],
  ['Right', 'Bottom', 'Above']
);

  const user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.settings.visuals.enableAnimations(),
                                  legend_style,
                                  reverseLegend,
                                  legendWidthOverride,
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'axis_ticks_font_size',
                                  'legend_font_size',
                                ]);

  arc.addColorSettings(user_settings, {
    includeUseCustomColorSetting: true,
    includeDimensionMapSetting: true,
    includeOpacitySetting: true,
    includeFixLegendSetting: true,
  });

  const pb = arc.visual.api();

  const shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimension',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: 'color_shelf',
      display_name: 'Colors',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  const constraints = {
    minimums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
    },
    maximums: {
      dimensions_shelf: 1,
      color_shelf: 1,
    }
  };

  pb.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    previousDataTracker.lib(lib);
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return pb;
  };

  pb.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  pb.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return pb;
    }

    saved_settings = settings;
    arc.misc.handleLegacyLegendSetting(saved_settings, legend_style);

    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    lib.restore_basics(settings, switchingVisualType);
    return pb;
  };

  pb.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    const xMessage = [
      lib.createPopoverMessages(['trellisOnly', 'columns.']),
      lib.createPopoverMessages('acceptsMultiple')
    ];
    const yMessage = [
      lib.createPopoverMessages(['trellisOnly', 'rows.']),
      lib.createPopoverMessages('acceptsMultiple')
    ];
    const dimensionMessages = [
      lib.createPopoverMessages('dimensionPacked'),
      lib.createPopoverMessages('acceptsOne')
    ];
    const aggregateMessages = [
      lib.createPopoverMessages('aggregatePacked'),
      lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException')
    ];
    const colorMessages = [
      lib.createPopoverMessages('packedColor'),
      lib.createPopoverMessages('acceptsOne'),
    ];
    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      dimensions_shelf: dimensionMessages,
      aggregates_shelf: aggregateMessages,
      color_shelf: colorMessages,
    });

    return pb;
  };

  pb.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  pb.viz_setDataResult = function(result) {
    data_result = result;
    return pb;
  };

  pb.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    var sb_rows = data_result.rows();
    if (!sb_rows || sb_rows.length === 0) {
      return pb;
    }

    // Filter out null values in aggregates
    var agg_getter = function(row) {
      return row[lib.agg_start_idx()];
    };

    sb_rows = lib.filter_nulls({rows: sb_rows}, [agg_getter]);
    if (sb_rows.length === 0) {
      return pb;
    }

    previousDataTracker.enableOnRowCount(sb_rows.length);

    lib.unselectOnClick();
    const config = {
      rows: sb_rows,
      columns: data_result.columns(),
      agg_start_idx: lib.agg_start_idx(),
    };

    lib.setupColors(config);

    var cols = lib.shelf_columns(config.columns, 'dimensions_shelf');
    config.dim_start_idx = cols[0].colindex();

    if (config.color_idx < 0) {
      config.color_idx = config.dim_start_idx;
    }

    trellis = arc.visual.trellis()
                     .x_shelf_name("x_shelf")
                     .y_shelf_name("y_shelf")
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select("#" + viz_div))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .legend_column(
                      (config.color_idx >= 0) ? config.color_column : undefined,
                        config.color_is_dim
                      )
                    .color(config.color, config.palette, config.colorSettings)
                    .legendPositioning(legend_style.value())
                     .redrawVisualCb(() => {
                       allowLargeTrellisTemp = true;
                       pb.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return pb;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(() => {
      pb.viz_drawVisual(viz_div);
    });

    if (legend_style.legendRequired()) {
      config.trellis.reverse_legend_order(reverseLegend.value());
      config.trellis.legend_width_override(legendWidthOverride.value());
      config.trellis.draw_legend();
    }

    config.trellis.tooltips('trellis');
    computeSvgParams(config, viz_div);

    config.trellis.draw_visuals(function(cell_sel, xy_rows,
        _unused_xmeasure, _unused_ymeasure, cellIdx) {
      config.rows = xy_rows;
      const cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_sel, cell_div, cellIdx);
    });

    return pb;
  };

  pb.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return pb;
  };

  pb.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return pb;
  };

  function computeSvgParams(config, viz_div) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
      includeLegend: legend_style.legendRequired(),
    });

    config.width = Math.max(elem_w, 100);
    config.height = Math.max(elem_h, 100);
  }

  function getCellConfig(config, cell_sel, cell_div, cellIdx) {
    // Special handling for circular pie charts, without axis marks
    // and legends
    var margin = {
                   top: 0,
                   right: 0,
                   left: 0,
                   bottom: 0,
                 },
        width = config.width,
        height = config.height;

    let cellConfig = {
      width,
      height,
      margin,
      cellIdx,
    };

    arc.canvas.createContainer(cell_sel, cellConfig);

    return cellConfig;
  }

  function computeData(config, cellConfig) {
    let {
      width,
      height,
      rows,
    } = config;

    const margin = {
      left: 2,
      right: 2,
      top: 2,
      bottom: 2,
    };

    const bubble = d4.pack()
                     .size([config.width - margin.left - margin.right,
                            config.height - margin.top - margin.bottom])
                     .padding(1.5);

    const tree = {};
    tree.name = "root";
    tree.children = config.rows.map(function(row, i) {
      return {
        name: i,
        value: row[config.agg_start_idx],
        dim: row[config.dim_start_idx] + '',
        color: config.color(row[config.color_idx]),
        row: row,
      };
    });

    const root = d4.hierarchy(tree)
                   .sum((d) => { return d.value || 0; })
                   .sort(function(a, b) { return b.value - a.value; });

    let data = bubble(root).children;

    let opacity = lib.colorOpacity();

    cellConfig.computedData = data.map(function(d) {
      return {
        x: d.x + margin.left,
        y: d.y + margin.top,
        r: d.r,
        dim: d.data.dim,
        color: d.data.color,
        textColor: arc.colorpalette.colorForBrightness(d.data.color, opacity),
        row: d.data.row,
      };
    });
  }

  function drawCanvas(config, cellConfig, canvasConfig) {
    let {
      width,
      height,
      computedData,
    } = cellConfig;

    let {
      canvas,
      hidden,
      highlight,
    } = canvasConfig;

    let colorMap = {},
        textHeight = 10 / 2;

    let context = canvas.node().getContext('2d');

    const opacity = lib.colorOpacity();

    if (!hidden && !highlight) {
      context.globalAlpha = opacity;
    }

    if (highlight) {
      context.lineWidth = arc.canvas.highlightLineWidth;
      context.strokeStyle = arc.canvas.highlightStrokeStyle;
    }

    let font = lib.getVisualFont();
    const fontSize = lib.getLabelFontSize() || 12;
    context.font = `${fontSize}px ${font}`;
    context.textBaseline = 'middle';
    context.textAlign = 'center';

    if (!highlight) {
      drawRows(computedData);
    }

    // We separate out the drawRows function so that we can use the same
    // function for highlighting particular rows. We simply pass in an array
    // of rows to highlight, and draw those rows in the highlight canvas.
    // See usage of highlightDrawRows to see more.
    function drawRows(rows) {
      context.clearRect(0, 0, width, height);

      rows.forEach(function(d, i) {
        let color;

        if (hidden) {
          color = colorFetcher();
          colorMap[color] = d;
        } else {
          color = d.color;
        }

        context.beginPath();
        context.arc(d.x, d.y, d.r, 0, 2 * Math.PI);
        context.fillStyle = color;
        context.fill();

        if (highlight) {
          context.stroke();
        }

        if (!hidden) {
          drawLabel(d);
        }
      });

      function drawLabel(d) {
        const formatter = config.columns[config.dim_start_idx].formatForDisplay;
        const text = formatter(d.dim);
        const lColor = lib.getLabelColor();
        context.fillStyle = lColor || d.textColor;

        const w = context.measureText(text).width;
        if ((w + 3) < d.r * 2) {
          context.fillText(text, d.x, d.y);
          return true;
        }

        return false;
      }
    }

    canvasConfig.drawRows = drawRows;
    canvasConfig.colorMap = colorMap;
  }

  function viz_drawVisualWork(config, cell_sel, cell_div, cellIdx) {
    if (config.rows.length === 0) {
      return;
    }

    let cellConfig = getCellConfig(config, cell_sel, cell_div, cellIdx);
    arc.canvas.createAllCanvas(cellConfig);
    computeData(config, cellConfig);

    let startTransitionGenerator = function(d) {
      return {
        x: d.x,
        y: d.y,
        r: 0,
        dim: d.dim,
        color: d.color,
      };
    };

    let dataToGroup = function(d) {
      return d.dim;
    };

    let columns = data_result.columns();

    function text_fn(d) {
      return lib.text_from_columns(d.row, columns);
    }

    function params_fn(d) {
      return lib.params_from_columns(d.row, columns);
    }

    let drawConfig = {
      drawCanvas,
      text_fn,
      params_fn,
      startTransitionGenerator,
      dataToGroup,
      previousDataTracker,
      lib,
    };
    arc.transition.drawTransition(config, cellConfig, drawConfig);

    return pb;
  }

  return pb;
};

arc.visual.plugins.pb_extension = function() {
  var ext = function() { return "arc.visual.plugins.pb_extension"; };

  ext.viz_name = function() {
    return "packed-bubbles";
  };

  ext.viz_tooltip = function() {
    return "packed bubbles";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_packed-bubbles.png?v=1";
  };

  ext.viz = arc.visual.plugins.pb;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.pb_extension());

}());
