window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};

/* Notes:
There are four kinds of picklists in the UI:
1. Simple textbox used to pass parameters out to other visuals
2. Single select radio button: emits outputparam
3. Single select drop down list: emits outputparam
4. Multi select check boxes: emits outputparam.data, outputparam.exclude

All of these are shown configured as 'Filters' or can be created through
'Create Custom Filter' as 'Parameter' viz

Filter widgets take values from a base field in the tables of the visual.
Parameters present a simple textbox, or take values as specified by the
user while configuring the widget.

While configuring 'Parameter' viz, the Output Parameter option is required,
but is optional for filter widgets.
If configured as a 'Parameter' viz, then it will also emit outputparam.alias
(set to the label for the value specified)
Note that parameters do not have a searchbox facility.

Target visuals can have the following formats in their filter shelf:
[outputparam]  (will work for both single/multi select)
foo <<outputparam.exclude:in> (<<outputparam.data:'some val'>>)
  (will work only for multi-select)

For optional dims/measures, set up shelf(s)in the target visual like so:
[<<dim1:''>>] or [<<dim1:>>] or <<dim1:''>> or <<dim1:>>
[<<mea1:''>>] or [<<mea1:>>] or <<mea1:''>> but not <<mea1:>>
and set up a Parameter viz that passes different values for dimension1 or
measure1. (Output parameter is dimension1/measure1 and values/labels are
names of different dims/measures). When the Parameter viz is reset or passes
no value, that will make the target visual drop the dimension1/measure1
(since its default value is set to '', and that is not a valid setting for
a dimension/measure shelf)

A url sample which will cause a target visual with [<<dim1:>>] to get
dropped will look like ../arc/apps/32?param.dim1= or just ../arc/apps/32

TODO for code cleanup:
1. The code is interspersed with if statements for checking whether its
a dropdown or a single select, so there should be wrapper function for that
(or part of a settings function).
2. Some of the if statements are purely because a different class
picklist-list vs picklist-dropdown has to be picked. It should be easier to
just add another class and always choose that so that the if statements aren't
needed
3. Label and alias are same thing. Should be renamed that way.
4. There are multiple places where we are extracting label from filters.
Instead we should now use data_labels.ua

*/
arc.visual.picklist_values_setting = function() {
  const setting = arc.visual.setting()
                   .id('picklist_values')
                   .tab('Values')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('PICKLIST_VALUES')
                   .add_label('Picklist values')
                   .value('[]');

  const fetch_setting = function() {
    const s = [];
    const $tableContainer = $(`#${setting.div_id()}`);
    $tableContainer.find('tbody tr').each(function() {
      const v = $(this).find('.setting-picklist-value').text();
      const t = $(this).find('.setting-picklist-title').text();
      if (v) {
        s.push([v, t]);
      }
    });

    return JSON.stringify(s);
  };

  const paint_setting = function(tab_d3_sel) {
    const container =
        tab_d3_sel.append('div')
                  .attr('id', setting.div_id());
    if (setting.hidden()) {
      container.attr('class', 'arc-invisible');
    }

    const $tablediv = $(`#${setting.div_id()}`);
    $tablediv.append($('#editable-data-table-template').html());

    const $tablebody = $tablediv.find('tbody');

    const templateIn = $('#editable-data-table-row-template tbody').html();
    const j = JSON.parse(setting.value());
    j.forEach((v) => {
      const value = v[0];
      const label = v[1];
      const h = arc.misc.safe_substitute(
          templateIn,
          {
            LABEL: label,
            VALUE: value,
          }
      );
      $tablebody.append(h);
    });

    $tablebody.append($('#editable-data-table-addrow-template tbody').html());

    let previousValue;

    // if not in settings modal.
    // in settings modal, there's an apply button.
    // in the right pane, the change should be applied immediatelly.
    const isTargetInSettingsModal = ($target) => $target.closest('.settings-modal').length === 1;

    function onRowInputBlur(event) {
      const $target = $(event.currentTarget);
      if (isTargetInSettingsModal($target) === false) {
        // eslint-disable-next-line no-use-before-define
        const value = fetch_setting();
        if (value !== previousValue) {
          setting.value(value);
          setting.makeChangeCbs();
          previousValue = value;
        }
      }
    }

    function setupClicks() {
      /**
       * Delete a row
       */
      const $deletes = $tablebody.find('.fa-trash-o');
      $deletes.offOn('click.deleteRow', function() {
        const $this = $(this);
        const isInModal = isTargetInSettingsModal($this);
        $this.closest('tr').remove();
        if (isInModal === false) {
          // eslint-disable-next-line no-use-before-define
          setting.value(fetch_setting());
          setting.makeChangeCbs();
        }
      });

      /**
       * Save changes after editing a row input
       */
      const $inputs = $tablebody.find('.setting-picklist-value, .setting-picklist-title');
      $inputs.offOn('blur.editRow', onRowInputBlur);

      /**
       * Add new row
       */
      const $add = $tablebody.find('.viz-picklist-value-add');
      $add.offOn('click.addrow', () => {
        $tablebody.find('.viz-picklist-value-add-row').remove();
        const newTableRow = arc.misc.safe_substitute(
            templateIn,
            {
              LABEL: '',
              VALUE: '',
            }
        );

        const $newTableRow = $(newTableRow);
        $tablebody.append($newTableRow);

        const $newInputs = $newTableRow.find('.setting-picklist-value, .setting-picklist-title');
        $newInputs.offOn('blur.editRow', onRowInputBlur);

        $newTableRow.find('.setting-picklist-value').first().focus();
        $tablebody.append($('#editable-data-table-addrow-template tbody').html());
        setupClicks();
      });
    }

    setupClicks();
    previousValue = fetch_setting();
  };

  setting.paint_fn(paint_setting);
  setting.fetch_fn(fetch_setting);

  return setting;
};

arc.visual.plugins.picklist = function() {
  let saved_settings = {};
  let lib;
  let default_selections = {}; // Last saved selection items to be visualized by default

  /*
    * toSaveSelections is the store of active selections in the visual.
    * Note that it is initialized from default_selections, so it is not storing
    * a diff, but the actual settings.
    *
    * If the user hits the 'Reset' icon in the dashboard, we reset
    * 'toSaveSelections' (in picklist.viz_resetControl).
    *
    * If the user hits 'Save', we copy 'toSaveSelections' into 'default_selections'
    * and save away the changes (in picklist.viz_getSettings)
    *
    * When the user makes changes in edit mode, we track those changes
    * in 'toSaveSelections'.
    */
  let toSaveSelections; // Currently selected items (on the whole list)
  let isListLongerThanLimit = false;
  let showSelectedOptionsOnly = false;
  let dropdown_active;
  let textbox_text;
  let backend_search_text;
  let frontend_search_text;
  let displayedFilters;
  let data_labels;
  let valueType;

  // Track whether the same output parameters is being output in different forms
  let hasConflictingParams;

  // Track whether this is a dataset segment - this is setup within arc_dashboard_filters.js
  let isDatasetSegment;
  let filterSettings;

  let expanded; // expanded ? picklist is in the visual area : the filter shelf

  // It is introduced to help filter on local storage while the request is running
  // It can store up to the max(picklist_searchcount, picklist_itemcount)!
  let storedFilters = [];

  const picklist = arc.visual.api();

  const picklist_remember_selections = arc.report.user_settings.remember_selections();

  picklist.picklistDataset = arc.settings.filters.datasetDisplay();
  picklist.picklist_base_field = arc.settings.filters.baseField();
  picklist.picklist_title = arc.settings.filters.title();
  picklist.picklist_title.addChangeCb('updateReportTitle', (e) => {
    lib.subtitle(e.value());
  });

  picklist.picklist_title.add_postpaint_fn(() => {
    if (expanded) {
      picklist.picklist_title.value(lib.subtitle());
    }
  });

  picklist.picklist_output_parameter = arc.settings.filters.outputParameter();
  picklist.picklist_filter_columns = arc.settings.filters.filterColumns();

  picklist.picklist_itemcount = arc.visual.setting()
          .id('picklist_itemcount')
          .tab('Data')
          .type(arc.visual.settingTypeEnum.INTEGER)
          .add_name('PICKLIST_ITEMCOUNT')
          .add_label('Maximum default number of values displayed')
          .min(0)
          .max(10000)
          .value('1000');

  picklist.picklist_searchcount = arc.visual.setting();

  picklist.picklist_searchcount
          .id('picklist_searchcount')
          .tab('Data')
          .type(arc.visual.settingTypeEnum.INTEGER)
          .add_name('PICKLIST_SEARCHCOUNT')
          .add_label('Maximum number of search results displayed')
          .min(1)
          .max(10000)
          .value('1000');

  picklist.picklist_width =
    arc.visual.setting()
       .id('picklist_width')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.STRING)
       .add_name('PICKLIST_WIDTH')
       .add_label('Width of this filter (in px)')
       .value('');

  picklist.picklist_single_select =
    arc.visual.setting()
       .id('picklist_single_select')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_SINGLE_SELECT')
       .add_label('Allow only one item to be selected at a time')
       .value('false');

  picklist.picklist_dropdown =
    arc.visual.setting()
       .id('picklist_dropdown')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_DROPDOWN')
       .add_label('Select values from a dropdown menu')
       .info('Values are displayed as radio buttons by default.')
       .value('false');

  picklist.picklist_display_no_selection =
    arc.visual.setting()
       .id('picklist_display_no_selection')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_DISPLAY_NO_SELECTION')
       .add_label("Include an option for 'All'")
       .value('true');

  picklist.picklist_remember_in_viewmode =
  arc.visual.setting()
    .id('picklist_remember_in_viewmode')
    .tab('Values')
    .type(arc.visual.settingTypeEnum.CHECKBOX)
    .setDisableTooltipText('Text input is not preserved. Please disable "Display a textbox parameter" setting to enable')
    .add_name('PICKLIST_REMEMBER_IN_VIEWMODE')
    .add_label('Save selections made in edit mode')
    .value('true');

    picklist.allow_nulls = arc.settings.range.allowNulls();

  picklist.picklist_multi_parm =
    arc.visual.setting()
       .id('picklist_multi_parm')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_MULTI_PARM')
       .add_label('Emit distinct parameters for each selected item')
       .value('false');

  picklist.picklist_textbox =
    arc.visual.setting()
       .id('picklist_textbox')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_TEXTBOX')
       .add_label('Display a textbox parameter')
       .info('When checked, the filter ignores all other display settings, and ' +
            "the values specified on the 'Values' tab.")
       .value('false');

  picklist.picklist_upload =
    arc.visual.setting()
       .id('picklist_upload')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_UPLOAD')
       .add_label('Allow the user to add values to the filter')
       .info('This setting is currently not supported with single selection.')
       .value('false');

  picklist.picklist_hide_filter_when_empty =
    arc.visual.setting()
       .id('picklist_hide_filter_when_empty')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_HIDE_FILTER_WHEN_EMPTY')
       .add_label('Hide filter if no input data')
       .info('If no viable selection data exists, this option hides the filter in View mode.')
       .value('false');

  picklist.picklist_apply_selections_together = arc.visual.setting()
       .id('picklist_apply_selections_together')
       .tab('Display Settings')
       .type(arc.visual.settingTypeEnum.CHECKBOX)
       .add_name('PICKLIST_APPLY_SELECTIONS_TOGETHER')
       .add_label('Apply all changes to a multi-select list at the same time')
       .info('When checked, all changes to this filter apply after ' +
           ' clicking outside the filter.')
       .value('false');

  picklist.improve_hive_distinct_request =
    arc.visual.setting()
        .id('improve_hive_distinct_request')
        .tab('Data')
        .type(arc.visual.settingTypeEnum.CHECKBOX)
        .add_label('Enhance Hive sorting')
        .info('Turn on in case you experience a sorting mismatch in the filter list. Keep in mind that it can reduce performance!')
        .value('false');

  picklist.picklist_options_sort_order =
    arc.visual.setting()
       .id('filter_options_sort_order')
       .tab('Data')
       .type(arc.visual.settingTypeEnum.RADIO)
       .group('Sort order of filter widget values')
       .add_name('ASCENDING')
       .add_label('Ascending')
       .add_name('DESCENDING')
       .add_label('Descending')
       .value('ASCENDING');

  picklist.picklist_filter_appgroup_scope = arc.report.user_settings.filter_appgroup_scope();
  picklist.picklist_filter_dataset_scope = arc.report.user_settings.filter_dataset_scope();

  picklist.picklist_values = arc.visual.picklist_values_setting();

  // Disabled tooltip texts
  const allowSingleSelectionTooltipText = 'This setting is currently disabled. Please enable "Allow only one item to be selected at a time" setting to enable';
  const disableTextboxTooltipText = 'This setting is currently disabled. Please disable "Display a textbox parameter" setting to enable';

  picklist.picklist_dropdown.addDependents([picklist.allow_nulls], ['true'], ['false'], undefined, true);

  picklist.picklist_single_select
          .addDependents([
              picklist.picklist_dropdown,
              picklist.picklist_display_no_selection
          ], ['false'], ['true'], allowSingleSelectionTooltipText);

  picklist.picklist_textbox
  .addDependents([
    picklist.picklist_remember_in_viewmode,
    picklist.picklist_single_select,
    picklist.picklist_multi_parm,
    picklist.picklist_upload,
    picklist.picklist_hide_filter_when_empty,
    picklist.picklist_apply_selections_together,
    picklist_remember_selections,
    picklist.allow_nulls
  ], ['true'], ['false'], disableTextboxTooltipText);

  const user_settings =
      arc.visual.defaultSettings({
           isFilter: true,
         })
         .addSetting([
             picklist.picklistDataset,
             picklist.picklist_textbox,
             picklist.picklist_width,
             picklist.picklist_upload,
             picklist.picklist_remember_in_viewmode,
             picklist.allow_nulls,
             picklist.picklist_base_field,
             picklist.picklist_title,
             picklist.picklist_output_parameter,
             picklist.picklist_filter_columns,
             picklist.picklist_itemcount,
             picklist.picklist_searchcount,
             picklist.picklist_single_select,
             picklist.picklist_dropdown,
             picklist.picklist_display_no_selection,
             picklist.picklist_multi_parm,
             picklist.picklist_values,
             picklist.picklist_hide_filter_when_empty,
             picklist.picklist_filter_appgroup_scope,
             picklist.picklist_filter_dataset_scope,
             picklist_remember_selections,
             picklist.picklist_apply_selections_together,
             picklist.picklist_options_sort_order,
             picklist.improve_hive_distinct_request,
         ]);

  picklist.updateWithUrlParameters = function(settingsAndValues, picklistConfig) {
    let hasOutput;
    let hasBaseField;

    Object.entries(settingsAndValues).forEach(([settingName, value]) => {
      if (settingName === 'output_parameter' && picklistConfig) {
        hasOutput = true;
      } else if (settingName === 'base_field' && picklistConfig) {
        hasBaseField = true;
      }

      picklist.setting(settingName, value, picklistConfig);
    });

    // Update params if output has been changed when creating new param
    // (Why would the user doe this? Who knows...)
    if (hasOutput || (!isCustomFilter() && hasBaseField)) {
      delete arc.globals.parameters.params()[picklistConfig.output];
      arc.globals.parameters.params()[picklist.getOutputParameterRaw()] =
          picklistConfig.picklistSettings;
    }

    return picklist;
  };

  picklist.filter_dataset_scope = function() {
    return picklist.picklist_filter_dataset_scope;
  };

  picklist.setting = function(settingName, value) {
    if (arguments.length === 1) {
      return arc.params.setting(picklist, settingName, 'picklist');
    }

    return arc.params.setting(picklist, settingName, 'picklist', value);
  };

  picklist.getOutputParameterRaw = function(oldValue, type) {
    let hasOldOutput; let hasOldBaseField;
        let outputParameter;
    if (arguments.length) {
      hasOldOutput = arguments.length && type === 'output_parameter';
      hasOldBaseField = arguments.length && type === 'base_field';
    }

    outputParameter = hasOldOutput ?
                      oldValue : picklist.picklist_output_parameter.value();
    if (!outputParameter) {
      if (!isCustomFilter()) {
        outputParameter = hasOldBaseField ? oldValue : picklist.picklist_base_field.value();
      }
    }

    return outputParameter;
  };

  picklist.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;

    lib.shelf_data([])
       .user_settings(user_settings);

    if (isCustomFilter()) {
      lib.visualSupportsViewData(false);
    }

    return picklist;
  };

  picklist.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    saved_settings = settings;

    settings.viz_type = settings.viz_type || {};
    settings.viz_type.picklist = settings.viz_type.picklist || {};

    const currentSelections = getCurrentSelections();
    if (currentSelections) {
      // There is something to save in toSaveSelections, save it away
      default_selections = arc.misc.extend({}, currentSelections);
      default_selections.coltype = currentSelections.coltype || getColtype();
    }

    settings.viz_type.picklist.default_selections = default_selections;
    settings.viz_type.picklist.expanded = (expanded === true);
    settings.viz_type.picklist.textbox_text = textbox_text;
    settings.viz_type.picklist.isDatasetSegment = isDatasetSegment;
    settings.viz_type.picklist.version = 3;

    if (settings.viz_type.picklist.expanded) {
      picklist.picklist_title.value(settings.report_subtitle);
    } else {
      lib.subtitle(picklist.picklist_title.value());
    }

    return settings;
  };

  /*
   * picklist multiselect visuals emit 'output.data' and 'output.exclude' parameters
   *
   * Other visuals on the page may emit 'output' - see ARC-7110
   *
   * In this scenario, we need to accept this parameter in the picklist visual
   * and adjust to it. The function here lets the dashboard know that we are
   * in this scenario and that the picklist visual should be updated.
   */
  picklist.viz_conflictingParams = function() {
    const o = getSubstitutedOutputParameter();
    if (!o) {
      return false;
    }

    if (!selectionIsSingleSelect()) {
      const params = lib.getParams();
      if (params.get(o, true)) {
        hasConflictingParams = true;
        return true;
      }
    }

    return false;
  };

  function getSubstitutedOutputParameter() {
    let output_parameter = picklist.picklist_output_parameter.value();

    if (output_parameter) {
      return lib.substituteParams(output_parameter);
    }

    output_parameter = getBaseField();

    return output_parameter;
  }

  function getOutputParameter() {
    const output_parameter = getSubstitutedOutputParameter();
    if (!output_parameter) {
      if (!isCustomFilter()) {
        return getBaseField();
      }
    }

    return output_parameter;
  }

  picklist.viz_getOutputParameter = getOutputParameter;

  function getBaseField() {
    let base_field = picklist.picklist_base_field.value();
    if (base_field) {
      base_field = lib.substituteParams(base_field);
    }

    return base_field;
  }

  function getTitle() {
    let title = expanded ? lib.subtitle() : picklist.picklist_title.value();
    if (title) {
      title = lib.substituteParams(title);
    }

    return title;
  }

  // When we are instantiated, there may be input parameters in play that we
  // have to reflect.
  function setDefaultSelections(settings) {
    const outputParameter = getOutputParameter(); // Single Select
    const aliasParameter = `${outputParameter}.alias`; // Multi Select or Custom Filter
    const dataParameter = `${outputParameter}.data`; // Multi Select
    const excludeParameter = `${outputParameter}.exclude`; // Any picklist
    const includeNullsParameter = `${outputParameter}.include_nulls`; // Multi + single select but not dropdown and textbox
    const inParams = lib.getParams().getkvs();

    // Put any parameters from the page into toSaveSelections
    picklist.updateToSaveSelectionsFromParams();
    default_selections = {};
    if (settings && settings.viz_type && settings.viz_type.picklist &&
        settings.viz_type.picklist.default_selections &&
        picklist.picklist_remember_in_viewmode.isChecked) {
      default_selections = settings.viz_type.picklist.default_selections;
    }

    // Start with default_selections, and then override them with toSaveSelections
    // that we just filled in by calling picklist.updateToSaveSelectionsFromParams()
    // above
    const currentSelections = getCurrentSelections();
    default_selections = arc.misc.extend(default_selections, currentSelections);

    if (picklist.picklist_textbox.isChecked) {
      // We are a textbox
      if (outputParameter in inParams) {
        // There is an input parameter with the same name as we are outputting
        // Take on the value from that parameter
        textbox_text = inParams[outputParameter];
      }

      // Exit - textboxes are simple
      return;
    }

    textbox_text = undefined;

    function defaultsFromStore() {
      if (!settings.viz_type ||
          !settings.viz_type.picklist ||
          !settings.viz_type.picklist.default_selections) {
        return;
      }

      if (picklist.picklist_remember_in_viewmode.isChecked) {
        default_selections = settings.viz_type.picklist.default_selections;
      }

      if (!outputParameter) {
        return;
      }

      default_selections.includeNulls = default_selections.includeNulls ?? true;

      setCurrentSelections({
        includeNulls: default_selections.includeNulls
      });
    }

    if (selectionIsMultiParm()) {
      // In multi-parameter output, there is no easy way to map the parameters
      // to the selections, so don't try.
      defaultsFromStore();
      return;
    }

    if (selectionIsSingleSelect()) {
      // Single-select parameters. If not input parameter matching our output
      // exists, fall back to the store
      if (!(includeNullsParameter in inParams) && !(outputParameter in inParams)) {
        defaultsFromStore();
        return;
      }

      const dataValue = inParams[outputParameter];
      let aliasValue = dataValue;

      if (inParams[aliasParameter]) {
        aliasValue = inParams[aliasParameter];
      }

      let coltype = 'STRING';
      if (arc.misc.isNumberType(dataValue) ||
          ((dataValue instanceof Array) && dataValue.every((d) => arc.misc.isNumberType(d)))
      ) {
        coltype = 'NUMERIC';
      }

      default_selections = {
        coltype,
        all_selected: false,
        data: dataValue ? [dataValue] : [],
        alias: aliasValue ? [aliasValue] : [],
        includeNulls: default_selections.includeNulls
      };

      return;
    }

    // Multi-select

    // Even if a filter is setup as a multi-select filter, we want to accept
    // parameters passed in as single=select parameters. See ARC-7645
    if (
      !(includeNullsParameter in inParams) &&
      !(dataParameter in inParams) &&
      !(outputParameter in inParams)) {
      defaultsFromStore();
      return;
    }

    let excludeValue = false;
    let dataArray;
    let aliasArray;

    if (dataParameter in inParams) {
      if (excludeParameter in inParams) {
        if (inParams[excludeParameter] === 'not in') {
          excludeValue = true;
        }
      }

      dataArray = arc.params.parse_list(inParams[dataParameter]);
      aliasArray = dataArray;

      if (aliasParameter in inParams) {
        aliasArray = arc.params.parse_list(inParams[aliasParameter]);
      }
    } else {
      // (outputParameter in inParams) === true
      dataArray = inParams[outputParameter] ? [inParams[outputParameter]] : [];
      aliasArray = dataArray;
      hasConflictingParams = true;
    }

    default_selections = {
      coltype: getColtype(),
      all_selected: false,
      data: dataArray,
      alias: aliasArray,
      exclude: excludeValue,
      includeNulls: default_selections.includeNulls
    };

    return picklist;
  }

  picklist.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return picklist;
    }

    saved_settings = settings;

    textbox_text = '';
    if (settings.viz_type &&
        settings.viz_type.picklist) {
      isDatasetSegment = settings.viz_type.picklist.isDatasetSegment;

      if (settings.viz_type.picklist.textbox_text) {
        textbox_text = settings.viz_type.picklist.textbox_text;
      }

      if (settings.viz_type.picklist.expanded) {
        picklist.expanded(settings.viz_type.picklist.expanded);
      } else {
        picklist.expanded(false);
      }

      if (!settings.viz_type.picklist.version) {
        settings.viz_type.picklist.version = 1;
      }
    } else {
      picklist.expanded(false);
    }

    // We want the default filter_dataset_scope for new filters to be
    // 'true', but we want the default filter_dataset_scope for existing
    // filters to be 'false'.
    //
    // In order to do this, we define filter_dataset_scope in
    // arc_report_settings with a default of 'true', but if we are being
    // asked to restore settings from the metastore, then we set it to false
    // before resroting.
    picklist.picklist_filter_dataset_scope.value('false');

    // Restore after setting up the 'expanded' field so title display
    // occurs correctly
    lib.restore_basics(settings, switchingVisualType);

    upgradePicklist(settings);
    setupData();
    setDefaultSelections(settings);
    if (selectionIsTextbox()) {
      textboxSendParam();
    } else if (default_selections) {
      // Publish the default selections to the rest of the system now.
      // We have to do this after 'restore_basics' so the base_field etc
      // is setup
      if (default_selections.data) {
        const $viz = $(`#${lib.chart_div()}`);
        valueType = default_selections.coltype;
        sendParametersParsed($viz, default_selections);
      }
    }

    if (picklist.picklist_base_field.value()) {
      picklist.picklist_values.hidden(true);
    } else if (isCustomFilter()) {
      let storedValue = picklist.picklist_filter_dataset_scope.value();
      if (storedValue === 'true') {
        // custom filters cannot have 'Current Dataset' selected.
        storedValue = 'false';
      }

      picklist.picklist_filter_dataset_scope.value(storedValue);
      picklist.picklist_filter_dataset_scope.add_class('custom_filter');
    }

    if (lib.dataset() && !isCustomFilter()) {
      const datasetName = lib.dataset().dataset_name();
      picklist.picklistDataset.value(datasetName);
    } else if (isCustomFilter()) {
      picklist.picklistDataset.hide();
    }

    if (isCustomFilter()) {
      picklist.allow_nulls.hide();
    }

    if (!['hive', 'hyve'].includes(lib.dataset().dataconnection_type())) {
      picklist.improve_hive_distinct_request.hide();
    }

    return picklist;
  };

  // upgrade from previous picklist versions to new one.
  function upgradePicklist(settings) {
    upgradePicklistTov2(settings);
    upgradePicklistTov3(settings);
  }

  /*
   * v3 was added in to fix an issue where the column name 'Segment'
   * was being used to identify a dataset segment, and this was causing
   * any actual column called 'Segment' in the dataset to be mis-identified
   * as a dataset segment.
   *
   * Now, we explicitly identify dataset segments all over the code, including
   * in picklists. So, for any version of saved picklist metadata prior to v3,
   * having the name 'Segment' implies this is a dataset segment. For v3 and
   * higher, all that matters is the saved 'isDatasetSegment' state.
   */
  function upgradePicklistTov3(settings) {
    if (!settings || !settings.viz_type || !settings.viz_type.picklist ||
        settings.viz_type.picklist.version !== 2) {
      return;
    }

    const base_field = getBaseField();
    if (base_field === arc.report.segment_name) {
      isDatasetSegment = true;
      settings.viz_type.picklist.version = 3;
    }
  }

  function upgradePicklistTov2(settings) {
    if (!settings || !settings.viz_type || !settings.viz_type.picklist ||
        !settings.viz_type.picklist.default_selections ||
        settings.viz_type.picklist.version !== 1) {
      return;
    }

    const tmpDefaultSelections = settings.viz_type.picklist.default_selections;

    // Filter widgets from older released stored numeric multi select as string
    // In version 2, expectation is:
    // values for numeric coltypes should not have quotes
    // values for string coltypes should be unquoted strings (unless value itself has quotes)
    if (tmpDefaultSelections.data === undefined) {
      tmpDefaultSelections.data = [];
    } else if (!Array.isArray(tmpDefaultSelections.data)) {
      tmpDefaultSelections.data = [tmpDefaultSelections.data];
    }

    let { coltype } = tmpDefaultSelections;
    if (!coltype || coltype.toUpperCase() === 'UNKNOWN') {
      coltype = 'STRING';
      if (tmpDefaultSelections.data.every(arc.misc.isNumberType)) {
        coltype = 'NUMERIC';
      }
    }

    if (coltype.toUpperCase() !== 'STRING') {
      if (selectionIsMultiSelect()) {
        tmpDefaultSelections.data = tmpDefaultSelections.data.map((value) => Number(value));
      }
    }

    tmpDefaultSelections.coltype = coltype;

    settings.viz_type.picklist.default_selections = tmpDefaultSelections;
    settings.viz_type.picklist.version = 2;
  }

  picklist.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();
    return picklist;
  };

  function picklistShowItemCount(scaleUp) {
    let count = parseInt(picklist.picklist_itemcount.value(), 10);
    if (!arc.misc.isNumeric(count)) {
      count = 1000;
    }

    if (scaleUp) {
      count += 1;
    }

    return count;
  }

  const createFilterRequestBuilder = (baseField, limit, coltype) => {
    let areFilterValuesAdded = false;
    let requestBuilder = lib.getRequestBuilder();
    const order = arc.data.order();
    order.ascending(picklist.picklist_options_sort_order.value() !== 'DESCENDING');
    const improveHive = picklist.improve_hive_distinct_request.value() === 'true';
    requestBuilder = arc.data_requests.distinctColumnValues(
      requestBuilder,
      lib.dataset(),
      baseField,
      limit,
      coltype,
      order,
      false,
      improveHive,
    );
    areFilterValuesAdded = addFilterValsToRequest(requestBuilder, baseField);
    if (areFilterValuesAdded) {
      requestBuilder.limit(picklist.picklist_searchcount.value());
    }

    return { requestBuilder, areFilterValuesAdded };
  };

  const runQuery = (requestBuilder, successCb, errorCb) => {
    const dataset = lib.dataset();
    const queryCmd = arc.data.queryCmd().dsreq(JSON.stringify(requestBuilder.dsreq()))
      .dataconnection_id(dataset.dataconnection_id())
      .dataset_id(dataset.dataset_id())
      .widget_id(lib.report_id())
      .skip_dsreq(lib.skip_dsreq());
    arc.data.runQuery(
      undefined,
      successCb,
      errorCb,
      lib,
      undefined,
      undefined,
      undefined,
      queryCmd,
      );
  };

  function picklistShowSearchItemCount(scaleUp) {
    let count = parseInt(picklist.picklist_searchcount.value(), 10);
    if (!arc.misc.isNumeric(count)) {
      count = 1000;
    }

    if (scaleUp) {
      count += 1;
    }

    return count;
  }

  picklist.viz_getDataRequest = function() {
    if (isCustomFilter() || picklist.picklist_textbox.isChecked) {
      return arc.data.request()
                .request_type('ignore');
    }

    const base_field = getBaseField();
    if (!base_field || picklist.isDatasetSegment()) {
      // We are going to return an 'ignore' request and get back an empty
      // result set. Make sure the library is setup to correctly execute
      // the next sql command.
      lib.forceReloadData();
      return arc.data.request()
                .request_type('ignore');
    }

    const data_request = arc.data.request();
    let sb = lib.getSelectBuilder();
    const dataset = lib.dataset();
    const colBaseField = dataset.get_column(base_field);
    const coltype = colBaseField ? colBaseField.attr('type') : '';
    const limit = Math.max(picklistShowItemCount(true), picklistShowSearchItemCount(true));

    const col = arc.data.column()
      .colname(base_field)
      .colsource('base_shelf')
      .coltype('string')
      .colproperty(arc.report.columnTypeEnum.DIMENSION);
    data_request.add_column(col);

    const { requestBuilder } = createFilterRequestBuilder(base_field, limit, coltype);
    if (requestBuilder.limit() === 0) {
      return arc.data.request()
                .request_type('ignore');
    }

    const filter_columns = picklist.picklist_filter_columns.value();
    if (filter_columns) {
      const filterNames = filter_columns.split(',');
      lib.handleSpecificFilters(filterNames, (_, expr) => {
        requestBuilder.addFilter(expr);
        sb.addFilter(expr);

      });
    }

    data_request.dsreq(requestBuilder.dsreq());

    // This part of the code actually do not effect the requestBuilder run,
    // but serves as request identifier to keep in track and do not rerun if not needed
    sb.distinct(picklist.improve_hive_distinct_request.value() === 'true');
    sb.addProjection(`[${base_field}]`);
    const sortOrder = picklist.picklist_options_sort_order.value() !== 'DESCENDING' ?
      'asc' :
      'desc';
    sb.addOrderBy(`1 ${sortOrder}`);
    sb.limit(limit);
    sb = lib.finalizeSql(sb);
    data_request.sql(sb.getSql());
    data_request.finalize();
    return data_request;
  };

  /*
   * If filter values need to be added to the backend request, do it.
   * Returns true if user-specified filter values are added, and false otherwise.
   *
   * This helps us pick between picklist_itemcount and picklist_searchcount for
   * the number of results to fetch.
   */
  function addFilterValsToRequest(rb, base_field) {
    if (lib.isSolr()) {
      lib.handleSolrParam((expr, isSearch) => {
        if (isSearch) {
          rb.addSolrQuery(expr);
        }
      });

      lib.datasetFiltersCb((expr) => {
        rb.filters(expr);
      });

      let userSpecifiedFiltersAdded = false;
      if (backend_search_text) {
        rb.filters(`[${base_field}] in ("${backend_search_text}*")`);
        userSpecifiedFiltersAdded = true;
      }

      return userSpecifiedFiltersAdded;
    }

      if (backend_search_text) {
        const conn = lib.dataset().dataconnection_type();
        const lower_fn = arc.dataconnection.types[conn]().lower;
        const cast_expr = arc.dataconnection.castAsString(base_field, conn);
        rb.filters(` ${lower_fn}(${cast_expr}) LIKE '%${backend_search_text.toLowerCase()}%'`);
        rb.phase({
          phase: 'search_text',
          search: backend_search_text.toLowerCase(),
        });

        return true;
      }

    return false;
  }

  const setStoredFilters = (rows) => {
    storedFilters = rows || [];
  };

  const setFilters = (rows, checkPicklistItemCountLimit = false) => {
    displayedFilters = rows;

    if (checkPicklistItemCountLimit && rows !== null && rows !== undefined) {
      const countLimit = picklistShowItemCount(false);
      isListLongerThanLimit = rows.length > countLimit;
      if (isListLongerThanLimit) {
        displayedFilters = rows.slice(0, countLimit);
      }
    }
  };

  const initializeFilterLists = (_filter) => {
    setStoredFilters(_filter);
    setFilters(_filter);

    // If all_selected is true, we need to update the current selection.
    if (getCurrentSelections()?.all_selected && storedFilters) {
      const data = storedFilters.map((filter) => filter[0]); // [200, 1], ["Brian", 1] etc..

      setCurrentSelections({
        data,
        alias: data
      });
    }
  };

  function setupData(data_result) {
    data_labels = [];
    if (picklist.isDatasetSegment()) {
      valueType = 'STRING';
      const data = lib.dataset_segments().segmentDataRows();
      initializeFilterLists(data.rows);
      return;
    }

    if (isCustomFilter()) {
      valueType = 'STRING';
      const dataRows = JSON.parse(picklist.picklist_values.value());
      initializeFilterLists(dataRows);

     if (!picklist.picklist_textbox.isChecked) {
        /**
         * Custom filter values might be changed since the last render.
         * Let's make sure unexisting options (not in displayedFilters) are not in the current
         * selection array (currentSelections).
         */
        const currentSelections = getCurrentSelections();
        const removeOnIndexArr = [];
        if (currentSelections && Array.isArray(currentSelections.data)) {
          currentSelections.data.forEach((option, i) => {
            const optionInRows = displayedFilters.find((row) => row[0] === option);
            if (!optionInRows) {
              removeOnIndexArr.push(i);
            }
          });
        }

        if (removeOnIndexArr.length > 0) {
          setCurrentSelections({
            data: currentSelections.data
              .filter((datum, i) => !removeOnIndexArr.includes(i)),
            alias: (currentSelections.alias ?? [])
              .filter((alias, i) => !removeOnIndexArr.includes(i)),
          });
        }
     }

      return;
    }

    if (data_result) {
      if (data_result.columns().length > 0) {
        valueType = data_result.columns()[0].coltype();
      } else {
        valueType = 'STRING';
      }

      if (!isBackendSearchOn()) {
        const dataResult = data_result.rows().filter((row) => row[0] !== null);
        initializeFilterLists(dataResult);
      }

      return;
    }

    initializeFilterLists(displayedFilters || []);
  }

  /*
   * Set the data this object should use to draw out the visual
   */
  picklist.viz_setDataResult = function(result) {
    setupData(result);
    return picklist;
  };

  function isCustomFilter() {
    const specifiedValues = JSON.parse(picklist.picklist_values.value());

    // Values being specified overrides the base field
    if (specifiedValues.length > 0) {
      return true;
    }

    // We are trying to determine if this is a parameter picker, but no
    // parameters have been specified yet. Get the 'base_field' value and
    // return true if no base field exists
    const base_field = picklist.picklist_base_field.value();
    if (base_field) {
      return false;
    }

    // No base field - the user hasn't gotten around to specifying values yet
    return true;
  }

  function hideFilter() {
    if (!expanded) {
      $(`#${lib.chart_div()}`).closest('.selected-control')
                              .addClass('arc-invisible');
    }
  }

  function showFilter() {
    if (!expanded) {
      $(`#${lib.chart_div()}`).closest('.selected-control')
                              .removeClass('arc-invisible');
    }
  }

  const handleIncludeNullHtml = () => {
    if (!isCustomFilter()) {
      const $viz = $(`#${lib.chart_div()}`);

      $viz.find('.picklist-include-nulls-wrapper').toggleClass('arc-invisible', !picklist.allow_nulls.isChecked);
      $viz.find('.picklist-include-nulls').prop('checked', getCurrentSelections()?.includeNulls ?? true);
    }
  };

  picklist.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    const $viz = $(`#${viz_div}`);

    const $saveCsv = $viz.closest('li.arc-viz-container')
                       .find('.widget-btns-wrapper .save-csv');
    $saveCsv.addClass('arc-invisible');

    // If we are redrawing in a dashboard, remember our dropdown state for
    // later restoration
    if ($viz.find('.picklist-list-wrapper').css('display') === 'block') {
      dropdown_active = true;
    } else {
      dropdown_active = false;
    }

    lib.undraw(!expanded);
    setupVisualElements($viz);

    // In non-expanded mode, display the title
    let title = getTitle();
    if (!title) {
      // No title specified.
      // If this picklist comes from a column, use the name of the column
      if (!isCustomFilter()) {
        const base_field = getBaseField();
        if (base_field) {
          title = base_field;
          picklist.picklist_title.value(title);
        }
      }
    }

    if (expanded) {
      lib.subtitle(title);
    } else {
      $viz.find('.base_field').text(title);
    }

    $viz.find('.picklist-list').empty();
    $viz.find('.picklist-dropdown').empty();
    setupEvents($viz);

    if (displayedFilters) {
      // Add not fetched but already selected (by default) items to displayedFilters
      handleData($viz, false);
      picklist.viz_resetControl(false, true);
    } else {
      showHide();
    }

    if (isCustomFilter()) {
      if (picklist.picklist_options_sort_order.labels().indexOf('Default') < 0) {
        picklist.picklist_options_sort_order
                .add_name('DEFAULT')
                .add_label('Default')
                .value('DEFAULT');
      }

      if (arc.visual.plugins.picklist.showSettings) {
        $viz.find('.widget-btns-wrapper .configure-app').trigger('click');
        arc.visual.plugins.picklist.showSettings = false;
      }
    }

    return picklist;
  };

  function getCurrentSelections() {
    return toSaveSelections;
  }

  function setCurrentSelections(params) {
    if (params != null) {
      toSaveSelections = Object.assign({}, (toSaveSelections || {}), params);
      Object.freeze(toSaveSelections);
    } else {
      toSaveSelections = null;
    }

    return toSaveSelections;
  }

  function getCurrentSelectionsInHash() {
    const hashOfCurrentSelections = {};
    const currentSelections = getCurrentSelections() || {};
    const selection = Object.keys(currentSelections).length > 0 ?
    currentSelections :
    default_selections;

    if (!selection.alias) {
      setCurrentSelections({
        alias: [],
      });
    }

    if (selection.data) {
      selection.data.forEach((d, i) => {
        hashOfCurrentSelections[d] = selection.alias[i] || d;
      });
    }

    return hashOfCurrentSelections;
  }

  // When is the function called:
  //  1. When user hits dashboard's param reset button
  //  2. When user searches something within displayedFilters.
  // Function is expected to not reset default_selections.
  // Function when called from dashboard should reset toSaveSelections
  picklist.viz_resetControl = function(clearSelections, sendParams) {
    if (displayedFilters || (selectionIsTextbox() && picklist.picklist_base_field.value())) {
      if (clearSelections) {
        // Remove any saved selections
        setCurrentSelections(null);
        picklist.resetSearch();
      }

      const $viz = $(`#${lib.chart_div()}`);
      visualizeSelections($viz);
      summarize($viz);
      if (sendParams) {
        sendParameters($viz,
          !selectionIsTextbox() && picklist.picklist_remember_in_viewmode.isChecked);
      }
    }

    return picklist;
  };

  picklist.setVisibility = function (showViz) {
    const $viz = $(`#${lib.chart_div()}`);
    if (showViz) {
      $viz.removeClass('arc-invisible');
    } else {
      $viz.addClass('arc-invisible');
    }
  };

  picklist.base_field = function(_) {
    if (!arguments.length) return picklist.picklist_base_field.value();

    picklist.picklist_base_field.value(_);
    if (!picklist.picklist_output_parameter.value()) {
      picklist.picklist_output_parameter.value(_);
    }

    return picklist;
  };

  picklist.expanded = function(_) {
    if (!arguments.length) return expanded;
    expanded = _;
    return picklist;
  };

  picklist.isDatasetSegment = function(_) {
    if (!arguments.length) return isDatasetSegment;
    isDatasetSegment = _;
    return picklist;
  };

  picklist.filterSettings = function(_) {
    if (!arguments.length) return filterSettings;
    filterSettings = _;
    return picklist;
  };

  picklist.selectionIsDropdown = function() {
    return (!picklist.picklist_textbox.isChecked &&
             picklist.picklist_single_select.isChecked &&
             picklist.picklist_dropdown.isChecked);
  };

  function setupVisualElementsExpanded($viz) {
    $viz.append($('#picklist-list-template').html());

    if (selectionIsTextbox()) {
      $viz.find('.pick-textbox')
          .removeClass('arc-invisible');

      $viz.find('.picklist-textbox')
          .val(textbox_text);
    }

    return picklist;
  }

  const applyLoadingStyleOnInput = () => {
    const $viz = $(`#${lib.chart_div()}`);
    // we have different inputs, different way to handle loading state
    if (picklist.selectionIsDropdown()) {
      $viz.find('.chosen-search').append('<i class="fa fa-spinner  ml-1" id="loading-incidicator" style="position: relative;right: 25px;background: white; padding: 2px;"><i>');
    } else {
      $viz.find('.search-spin').show();
      $viz.find('.fa-search').hide();
    }
  };

  const removeLoadingStyleOnInput = () => {
    const $viz = $(`#${lib.chart_div()}`);
    if (picklist.selectionIsDropdown()) {
      $viz.find('.chosen-search #loading-incidicator').remove();
    } else {
      $viz.find('.search-spin').hide();
      $viz.find('.fa-search').show();
    }
  };

  function populateSearchResults(error, data, cb) {
    const $viz = $(`#${lib.chart_div()}`);
    if (!error) {
      setFilters(data.rows);
      if (isBackendSearchOn()) {
        if (displayedFilters.length >= picklistShowSearchItemCount(true)) {
          displayedFilters.splice(picklistShowSearchItemCount(false));
        }
      }

      data_labels = [];
      lib.sb_rows(displayedFilters);
      handleData($viz);
      picklist.viz_resetControl();
    } else {
      // eslint-disable-next-line no-console
      console.log('Oops! Something went wrong while searching: ', error);
    }

    if (typeof cb === 'function') {
      cb(error, data);
    }
  }

  const doBackendSearch = (
    searchBoxValue,
    base_field,
    type,
    onAfterSearch /* add loading syle */,
    onBeforeSearch /* remove loading style */
  ) => {
    backend_search_text = searchBoxValue;
    const { requestBuilder } =
    createFilterRequestBuilder(base_field, picklistShowItemCount(true), type);

    const filter_columns = picklist.picklist_filter_columns.value();
    if (filter_columns) {
      const filterNames = filter_columns.split(',');
      lib.handleSpecificFilters(filterNames, (_, expr) => {
        requestBuilder.addFilter(expr);
      });
    }

    if (requestBuilder.limit() === 0) {
      populateSearchResults(undefined, {
        rows: [],
      }, onAfterSearch);
      return;
    }

    onBeforeSearch();
    runQuery(
      requestBuilder,
      function success(resp) {
        populateSearchResults(undefined, resp, onAfterSearch);
        removeLoadingStyleOnInput();
        backend_search_text = null;
      },
      function fail(error) {
        populateSearchResults(error, undefined, onAfterSearch);
        removeLoadingStyleOnInput();
        backend_search_text = null;
      }
    );
  };

  const getSearchBox = ($viz) => $viz.find(`${picklist.selectionIsDropdown() ? '.chosen-search-input' : '.picklist-searchbox'}`);
  const getSearchBoxValue = ($viz) => getSearchBox($viz).val();
  const setSearchBoxValue = (value, $viz) => $viz.find('.picklist-searchbox,.chosen-search-input').val(value);
  const searchInTheBackendIndicatorContainerId = 'search-on-the-backend-indicator-container';

  const setUpBackendSearch = ($viz) => {
    const loadMoreBtnId = 'load-more-filter-value';
    const loadingIndicatorId = 'loading-more-filter-value';
    const $container = picklist.selectionIsDropdown() ? $viz.find('.chosen-container .chosen-drop') : $viz.find('.picklist-list-wrapper.dropdown-menu');
    const initalText = 'Load more';
    const loadingText = 'Loading...';
    const moreValueIsAvailbleInTheDatabase = storedFilters.length >= picklistShowItemCount();
    if (getSearchBoxValue($viz) && moreValueIsAvailbleInTheDatabase) {
      const hasLoadMoreBtn = $container.find(`#${loadMoreBtnId}`).length > 0;
      const hasLoadingIndicator = $container.find(`#${loadingIndicatorId}`).length > 0;
      if (!hasLoadMoreBtn && !hasLoadingIndicator) {
        const $loadMoreBtn = $(`<a class="d-flex justify-content-center align-items-center p-1" id="${loadMoreBtnId}">${initalText}</a>${picklist.selectionIsDropdown() ? '' : '<span>⇧</span><span>⏎</span>'}`);
        const $loadingContent = $(`<div class="d-flex justify-content-center align-items-center p-1" id="${loadingIndicatorId}">${loadingText}</a>`);
        const searchOnTheBackend = () => {
            const base_field = getBaseField();
            // re-read  input value, to avoid using out of date input value
            // due (possible) further user typing
            doBackendSearch(getSearchBoxValue($viz), base_field, valueType, () => {
              $container.find(`#${searchInTheBackendIndicatorContainerId}`).remove();
              removeLoadingStyleOnInput();
            }, () => {
              $container.find(`#${searchInTheBackendIndicatorContainerId}`).html($loadingContent);
              applyLoadingStyleOnInput();
              $container.off('keyup.searchOnTheBackend');
            });
        };

        getSearchBox($viz).offOn('keyup.searchOnTheBackend', (e) => {
          if (e.shiftKey && e.keyCode === 13) {
            e.stopPropagation();
            searchOnTheBackend();
          }
        });

        const $indicatorContainer = $(`<div id="${searchInTheBackendIndicatorContainerId}"></div>`);
        $container.append($indicatorContainer);
        $container.find(`#${searchInTheBackendIndicatorContainerId}`).offOn('click.searchOnBackend', () => {
          searchOnTheBackend();
        });
        $container.find(`#${searchInTheBackendIndicatorContainerId}`).append(isBackendSearchOn() ? $loadingContent : $loadMoreBtn);
        }
      } else {
        $container.find(`#${searchInTheBackendIndicatorContainerId}`).remove();
      }
    };

  /**
     *
     * @param {Function} cb - callback function which gets called when the
     * search is done.
     */
  function search_fn(cb) {
    const $viz = $(`#${lib.chart_div()}`);
    const base_field = getBaseField();
    if (!base_field) {
      if (typeof cb === 'function') {
        cb();
      }

      return;
    }

    if (backend_search_text) {
      setSearchBoxValue(backend_search_text, $viz);
    }

    const searchBoxValue = getSearchBoxValue($viz);
    frontend_search_text = searchBoxValue;

    // Quickly search through the available list,
    // to show something the user until the backend query finishes

    const storedFiltersValuesOnly = storedFilters.map((f) => f[0]); // ['Brooklyn', 123]
    const enteredAndOrSelectedValues =
      getCurrentSelections()?.data?.filter((f) => !storedFiltersValuesOnly.includes(f)) ?? [];

    // They are  in different forms I unify them
    const numberInCaseNeededInCustomFilter = -1;
    const storedAndEnteredValues = [
      ...storedFilters,
      ...enteredAndOrSelectedValues.map((v) => [v, numberInCaseNeededInCustomFilter])
    ];
    const listToQuickSearch = showSelectedOptionsOnly ?
      getCurrentSelections()?.data?.map((v) => [v, numberInCaseNeededInCustomFilter]) :
      storedAndEnteredValues;
    const temporaryFiltersIncludesSearchText = searchBoxValue ?
      listToQuickSearch.filter(
        (value) => value[0]?.toString().toLowerCase().includes(searchBoxValue?.toLowerCase())
      ) : [...listToQuickSearch];

    populateSearchResults(undefined, {
      rows: temporaryFiltersIncludesSearchText,
    }, cb);

    setUpBackendSearch($viz);
  }

  const setupSearchEvent = ($searchInput) => {
    $searchInput.offOn('keyup.searchInput', arc.Stream.debounce(
      (e) => {
        // It should mostly fire only if the keycode is a real textual character:
        // not whitspace, not shift,  not tabs etc. These are listed in ingnoredKeyCodes,
        // ingnoredKeyCodes are characters which doesn't have textual meaning
        const ignoredKeyCodes =
          [9, 13, 16, 17, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 45, 47, 91, 92, 93, 94,
          95, 144, 145, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184];
        if (!ignoredKeyCodes.includes(e.keyCode)) {
          search_fn();
        }
      }, 300
     ));
  };

  // If second argument is not passed, function will toggle visibility.
  // Else it respects the second argument
  function switchPicklistVisibility($viz, isShow) {
    if (!$viz) {
      return;
    }

    const $filter_widget = $viz.find('.picklist-list-wrapper');
    const $filter_widget_open = $('.picklist-list-wrapper.dropdown-menu.open');
    const $range_filter_open = $('.selected-control-wrapper .range-filter-dropdown.show');

    if (isShow === undefined) {
      if ($range_filter_open.length > 0) {
        toggleRangeFilter($range_filter_open);
        showFilterWidget($filter_widget);
      } else if ($filter_widget_open.length === 0) {
        showFilterWidget($filter_widget);
      } else {
        toggleFilterWidget($filter_widget_open, $filter_widget);
      }
    } else if (isShow) {
      showFilterWidget($filter_widget);
    } else {
      hideFilterWidget($filter_widget);

      if (isApplyMultiSelectChangesTogether()) {
        sendParameters($viz, true, true);
      }
    }

    function toggleRangeFilter($rangeFilter) {
      $rangeFilter.toggleClass('show');
      $rangeFilter.find('.dropdown-menu-right').toggleClass('show');
    }

    function toggleFilterWidget($filterWidgetOpen, $filterWidget) {
      $filterWidget.toggleClass('open').toggle();
      $filterWidgetOpen.hide().removeClass('open');
      picklist.resetSearch();
    }

    function showFilterWidget($filterWidget) {
      $filterWidget.show().addClass('open');
    }

    function hideFilterWidget($filterWidget) {
      $filterWidget.hide('swing', picklist.resetSearch).removeClass('open');
    }
  }

  function uploadPicker($viz) {
    const $modal = $('#filter-values-modal');
    if (arc.misc.modalShown($modal)) {
      return;
    }

    const $modalBody = $modal.find('.modal-body');
    const currentSelections = getCurrentSelections();
    const vals = currentSelections.data.join('\n');
    $modalBody.empty();
    $modalBody.html($('#filter-upload-template').html());
    $modalBody.find('textarea').val(vals);
    $modal.find('.filter-upload-exclude').addClass('arc-invisible');
    $modal.modal('show');
    $modal.find('.filtervalues-add')
          .off('click')
          .on('click', () => {
            const textareaValues = $modal.find('textarea')
                               .val()
                               .split('\n')
                               .filter((s) => (s.trim().length > 0));
            const uniqueValues = Array.from(new Set(textareaValues)); // prevent duplicates

            const container = $viz.find(`.picklist-dropdown, .picklist-list`);

            uniqueValues.forEach((value) => {
              const $element = container.find(`input[data-value="${value}"]`);

              if ($element.length) {
                $element.prop('checked', true);
              } else { // new option
                container.append(renderOptions('checkbox', {
                  NAME: `picklist-${lib.unique_report_id()}`,
                  LABEL: value,
                  VALUE: value,
                  CHECKED: true,
                  STYLE: expanded ? `max-width:${$viz.width() - 30}px;` : '',
                }));
              }
            });

            const dataNum = [...uniqueValues].map((val) => {
              // number values need to be converted in order to not break the filter
              if (!Number.isNaN(val) && String(Number(val)) === val) {
                return Number(val);
              }

              return val;
             });

            setCurrentSelections({
              data: dataNum,
              alias: uniqueValues,
            });

            picklist.viz_resetControl(false, true);
            $modal.modal('hide');
          });
  }

  function setupVisualElements($viz) {
    if (expanded) {
      return setupVisualElementsExpanded($viz);
    }

    if (lib.in_dashboard()) {
      const w = picklist.picklist_width.value();
      arc.misc.setWidth($viz, w);
      if (w) {
        $viz.addClass('fixed-width-filter');
      } else {
        $viz.removeClass('fixed-width-filter');
      }

      $viz.append(
          $('#picklist-template .selected-control-picklist').html()
      );

      $viz.addClass('selected-control-picklist');
    } else {
      $viz.append(`<div><ul class="control-list">${$('#picklist-template').html()}</ul></div>`);
    }

    if (picklist.picklist_upload.isChecked) {
      $viz.find('.picklist-upload').removeClass('arc-invisible');
      $viz.find('.picklist-upload a').on('click', () => {
        uploadPicker($viz);
      });
    }

    handleIncludeNullHtml();

    if (selectionIsTextbox()) {
      $viz.find('.pick-textbox').removeClass('arc-invisible');
    } else if (!picklist.selectionIsDropdown()) {
      $viz.find('.selected-picklist-link').removeClass('arc-invisible');
    }

    if (!(lib.build_mode() && lib.in_dashboard())) {
      $viz.find('.widget-btns-wrapper').hide();
    }

    $viz.find('.widget-btns-wrapper a').tooltip({
      placement: 'top',
      container: 'body',
      delay: {
        show: 500,
        hide: 100,
      },
    });

    frontend_search_text = '';
    getSearchBox($viz).val(frontend_search_text);

    // open picklist dropdown when user clicks on link
    $viz.find('.selected-picklist-link').offOn('click.picklistDropdown', () => {
      switchPicklistVisibility($viz);
    });

    if (!picklist.selectionIsDropdown() && dropdown_active) {
      switchPicklistVisibility($viz, true);
    }

    // Close picklist list if user clicks anywhere else
    // This happens in two steps:
    // 1. On click anywhere in the body, except in modal, close the sub menu
    // 2. On click in the element, stop propagating the click onward
    const _togglePicklist = (evt) => {
      if ($(evt.target).closest('.modal').length > 0) {
        return;
      }
      // console.log('========== SeatupSearch 2 ', $viz.find('.picklist-searchbox'));
      switchPicklistVisibility($viz, false);
    };

    const id = lib.unique_report_id();
    $(document).off(`click.app${id}`).on(`click.app${id}`, _togglePicklist);

    $viz.find('.selected-control-wrapper')
        .off('click')
        .on('click', (e) => {
          e.stopPropagation();
        });

    if (picklist.selectionIsDropdown()) {
      const $picklist = $viz.find('.picklist-dropdown');
      const updateDropdownMenuZIndex = (value, event) => {
        $(event.target).closest('li').css('z-index', value);
      };

      $picklist.chosen().offOn('chosen:showing_dropdown', (event) => {
        updateDropdownMenuZIndex('100', event);
        search_fn();
      });

      $picklist.chosen().offOn('chosen:hiding_dropdown', (event) => {
        updateDropdownMenuZIndex('', event);
        // I added timeout here, because the jquery dropdown selection change runs after
        // the hiding_dropdown event, and resetSearch would reset the values
        // before the change could take its effect
        setTimeout(() => {
          picklist.resetSearch();
        }, 300);
      });
    }

    picklist.resetSearch();

    if (picklist.picklist_textbox.isChecked) {
      displayTextboxData($viz);
    }

    return picklist;
  }

  picklist.viz_configure = function() {
    // TODO: Change this to use isCustomFilter
    // and change isCustomFilter definition to use picklist_values.hidden
    // instead of base_field
    if (picklist.picklist_values.hidden()) {
      picklist.picklist_base_field.hidden(false);
    } else {
      picklist.picklist_base_field.hidden(true);
      picklist.picklist_filter_columns.hidden(true);
    }

    if (picklist.isDatasetSegment()) {
      picklist.picklist_base_field.hidden(true);
    }

    const appgroupScopeSetting = picklist.picklist_filter_appgroup_scope;
    const enableAppScopeSetting = !expanded;
    if (!enableAppScopeSetting) {
      appgroupScopeSetting.info(
        `${appgroupScopeSetting.info()} To enable this setting, move the filter from grid to filter area.`
      );
    } else {
      appgroupScopeSetting.info(arc.report.user_settings.filter_appgroup_scope().info());
    }

    lib.display_settings();

    // This is the best place to resolve conflicting dependencies.

    if (picklist.picklist_textbox.isChecked) {
      picklist.allow_nulls.disable();
    }

    if (!enableAppScopeSetting) {
      appgroupScopeSetting.disable();
    } else {
      appgroupScopeSetting.enable();
    }

    return picklist;
  };

  /*
    If a picklist does not have any option selected by default
  */
  function setToNoOptionSelected($viz) {
    let $first;
    setCurrentSelections({
      data: [],
      alias: [],
    });
    if (picklist.selectionIsDropdown()) {
      $first = $viz.find('.picklist-dropdown option:first').attr('selected', true);
    } else if (selectionIsSingleSelect()) {
      $first = $viz.find('.picklist-list input:first').attr('checked', true);
    } else {
      $viz.find('.picklist-list input:checked').attr('checked', false);
    }

    if ($first && $first.length > 0 && isMandatorySingleSelection()) {
      setCurrentSelections({
        data: [$first.attr('data-value')],
        alias: [$first.attr('data-alias')],
      });
    }
  }

  picklist.updateToSaveSelectionsFromParams = function() {
    function stringToParameterArray(paramString) {
      if (!paramString) {
        return;
      }

      const coltype = getColtype();
      const conntype = lib.dataconnection_type();
      let regex;
      if (arc.misc.isString(coltype, conntype)) {
        regex = /',\s*'/;
      } else {
        regex = /,\s*/;
      }

      return paramString.split(regex)
                        .map((singleParamValue) => arc.report.shelf.parse_filter_value(
                            singleParamValue, coltype, true, conntype
                          ));
    }

    const currentSelections = getCurrentSelections() || {};
    if (!currentSelections?.data || !currentSelections?.data?.length) {
      /*
       * Start with default selections.
       * Only update toSaveSelections from here if there is something
       * specified in the input parameters.
       *
       * Note that this could be called before setDefaultSelections, so don't
       * make any unnecessary changes in toSaveSelections.data unless there is
       * a parameter that impacts this picklist.
       */
      setCurrentSelections(default_selections);
    }

    const o = getSubstitutedOutputParameter();
    if (!o) {
      return;
    }

    let paramData;

    if (selectionIsMultiParm()) {
      paramData = lib.getParams().getkvs();

      const newData = [];
      const newAliases = [];
      displayedFilters.forEach((d) => {
        const alias = d[1];
        if (paramData[alias]) {
          newAliases.push(alias);
          newData.push(d[0]);
        }
      });

      if (newData) {
        setCurrentSelections({
          data: newData,
          alias: newAliases,
        });
      }

      updateToSaveSelectionsWhenIgnoreInParams();
      return;
    }

    const params = lib.getParams();
    const includeNullsData = params.get(`${o}.include_nulls`, true);

    if (includeNullsData) {
      setCurrentSelections({
        includeNulls: includeNullsData.value === 'true'
      });
    }

    if (!selectionIsSingleSelect()) {
      paramData = params.get(`${o}.data`, true);
      if (paramData) {
        paramData = paramData.value_list ? paramData.value_list.slice(0) :
                                           stringToParameterArray(paramData.value);
      } else {
        const singleData = params.get(o, true);
        if (singleData) {
          paramData = singleData.value_list ? singleData.value_list.slice(0) : [singleData.value];
          hasConflictingParams = true;
        }
      }

      if (paramData) {
        setCurrentSelections({
          data: paramData,
          exclude: (params.get(`${o}.exclude`) === 'not in'),
        });
        const aliasData = params.get(`${o}.alias`, true);
        if (aliasData) {
          setCurrentSelections({
            alias: aliasData.value_list ?
              aliasData.value_list.slice(0) :
              stringToParameterArray(aliasData.value),
          });
        } else {
          setCurrentSelections({
            alias: paramData,
          });
        }
      } else {
        // No parameters currently exist for this visual. Go back to default parameters
        // If includeNullsData exists but data doesn't,
        // we need to set data to [] instead of the default value.

        setCurrentSelections(includeNullsData ? {
          data: [],
          alias: []
        } : default_selections);
      }
    } else {
      paramData = params.get(o, true);
      if (paramData) {
        setCurrentSelections({
          // eslint-disable-next-line no-nested-ternary
          data: paramData.value_list ?
            paramData.value_list.slice(0) :
            (paramData.value ?
              [paramData.value] :
              []),
        });

        const paramAlias = params.get(`${o}.alias`, true);
        if (paramAlias) {
          setCurrentSelections({
            // eslint-disable-next-line no-nested-ternary
            alias: paramAlias.value_list ?
              paramAlias.value_list.slice(0) :
              (paramAlias.value ?
                [paramAlias.value] :
                []),
          });
        } else if (isCustomFilter() && displayedFilters) {
          setCurrentSelections({
            alias: displayedFilters
              .filter((d) => getCurrentSelections().data.includes(d[0]))
              .map((d) => d[1]),
          });
        } else {
          setCurrentSelections({
            alias: getCurrentSelections().data.slice(0),
          });
        }
      } else {
        // No parameters currently exist for this visual. Go back to default parameters
        // If includeNullsData exists but data doesn't,
        // we need to set data to [] instead of the default value.

        setCurrentSelections(includeNullsData ? {
          data: [],
          alias: [],
        } : default_selections);
      }
    }

    if (getCurrentSelections().data !== undefined && !selectionIsSingleSelect()) {
      if (isCustomFilter()) {
        const paramAlias = params.get(`${o}.alias`, true);
        if (!paramAlias) {
          updateToSaveSelectionsWhenIgnoreInParams();
          return;
        }

        if (paramAlias.value_list) {
          // Duplicating value_list and not pointing to same array as that of data
          setCurrentSelections({
            alias: paramAlias.value_list.slice(0),
          });
        } else {
          setCurrentSelections({
            alias: stringToParameterArray(paramAlias.value) || [],
          });
        }
      } else {
        // Non-segment and non-parameters picklists don't have aliases, update with data field
        const selections = getCurrentSelections();
        if (!selections.alias) {
          setCurrentSelections({
            alias: selections.data ? selections.data.slice(0) : [],
          });
        }
      }
    }

    updateToSaveSelectionsWhenIgnoreInParams();

    function updateToSaveSelectionsWhenIgnoreInParams() {
      const paramsByKey = lib.getParams().getkvs();
      if (paramsByKey[`${o}.ignore`]) {
        if (selectionIsMultiSelect()) {
          setCurrentSelections({
            exclude: false,
          });
        }

        if (!selectionIsMultiParm()) {
          setCurrentSelections({
            data: [],
            alias: [],
          });
        } else {
          const selections = getCurrentSelections();
          const tempData = selections.data.slice(0);
          const tempAlias = selections.alias.slice(0);
          setCurrentSelections({
            data: [],
            alias: [],
          });
          const currentSelectionData = [];
          const currentSelectionAlias = [];
          tempAlias.forEach((alias, index) => {
            if (alias !== o) {
              currentSelectionData.push(tempData[index]);
              currentSelectionAlias.push(alias);
            }

          });
          setCurrentSelections({
            data: currentSelectionData,
            alias: currentSelectionAlias,
          });
        }
      }
    }
  };

  function getColtype() {
    let coltype = 'UNKNOWN';
    const base_field = getBaseField();
    const dataset = lib && lib.dataset();

    if (base_field && dataset) {
      const col = dataset.get_column(base_field);
      if (col !== undefined) {
        coltype = col.attr('type');
      }
    } else {
      // isCustomFilter()
      coltype = 'STRING';
    }

    return coltype;
  }

  /*
   * Visualize current selections
   */
  function visualizeSelections($viz) {
    let selectionsToVisualize = getCurrentSelections();
    if (selectionsToVisualize !== null && selectionsToVisualize?.data === undefined) {
      const selection = { ...default_selections };
      const params = lib.getParams().getkvsList();

      if (params.length) {
        const currentPicklistParam = params.find((param) => param.key === getTitle());

        if (currentPicklistParam) {
          selection.alias.push(currentPicklistParam.value);
          selection.data.push(currentPicklistParam.value);
        }
      }

      setCurrentSelections(selection);
      selectionsToVisualize = getCurrentSelections();
    }

    if (selectionsToVisualize === null || !selectionsToVisualize?.data?.length) {
      selectionsToVisualize = setCurrentSelections({
        coltype: getColtype(),
        data: [],
        alias: [],
        exclude: false,
        all_selected: false,
      });
    } else if (isCustomFilter()) {
      // Filter out data/aliases that are not in displayedFilters
      // But only for custom filters -- aka has user-defined values
      const selAliases = selectionsToVisualize.alias || [];
      const filteredData = [];
      const filteredAlias = [];

      selectionsToVisualize.data.forEach((selVal, i) => {
        const currentAlias = selAliases[i];
        const dataAndAliasExist = displayedFilters.find(
          (dr) => dr[0] === selVal && (!dr[1] || dr[1] === currentAlias)
        );

        if (dataAndAliasExist || selectionIsTextbox()) {
          filteredData.push(selVal);
          filteredAlias.push(currentAlias);
        }
      });

      if (filteredData.length !== selectionsToVisualize.data.length) {
        selectionsToVisualize = setCurrentSelections({
          data: filteredData,
          alias: filteredAlias,
        });
      }
    }

    if (selectionIsSingleSelect() && getCurrentSelections().data.length > 1) {
      const selections = getCurrentSelections();
      setCurrentSelections({
        data: [selections.data[0]],
        alias: [selections.alias[0] || selections.data[0]],
      });
      // Currentselection is outdated without calling this
      selectionsToVisualize = getCurrentSelections();
    }

    // Make the selected data active in the dropdown
    if (picklist.selectionIsDropdown()) {
      const $picklist = $viz.find('.picklist-dropdown');
      const $search_input = $picklist.parent().find('.chosen-search-input');
      const serch_term = $search_input.val();
      if ($picklist.find('option').length) {
        if (selectionsToVisualize.data.length > 0) {
          // Even if there are multiple selections, we can only make one active
          const selected_value = selectionsToVisualize.data[0];
          $picklist.val(selected_value).trigger('chosen:updated');
        } else {
          $picklist.trigger('chosen:updated');
        }
      }

      // this has to be done, because 'chosen:updated' clears the searchbox;
      $search_input.val(serch_term);
    } else {
      const selections_hash = {};
      selectionsToVisualize.data.forEach((v) => {
        selections_hash[v] = true;
      });

      if (Object.keys(selections_hash).length > 0) {
        const foundKeys = {};
        const $e = $viz.find('.picklist-list input');
        $e.each(function () {
          const t = $(this).attr('data-value');
          if (t in selections_hash) {
            $(this).prop('checked', true);
            foundKeys[t] = true;
          } else {
            $(this).prop('checked', false);
          }
        });

        if (!picklist_remember_selections.isChecked) {
          const newData = [];
          const newAlias = [];
          selectionsToVisualize.data.forEach((v, i) => {
            if (v in foundKeys) {
              newData.push(v);
              newAlias.push(getCurrentSelections().alias[i]);
            }
          });

          setCurrentSelections({
            data: newData,
            alias: newAlias,
          });
        }
      }

      if ($viz.find('.picklist-list input:not(:checked)').length > 0 || (!showSelectedOptionsOnly && displayedFilters.length === 0)) {
        setCurrentSelections({
          all_selected: false,
        });
      } else {
        setCurrentSelections({
          all_selected: true,
        });
      }

      selectionsToVisualize = getCurrentSelections();

      $viz.find('.picklist-all').first().prop('checked',
        (selectionsToVisualize.all_selected && !selectionIsSingleSelect()));
    }

    $viz.find('.picklist-exclude').first().prop('checked',
      (selectionsToVisualize.exclude && !selectionIsSingleSelect()));

    if (selectionsToVisualize.data.length === 0) {
      setToNoOptionSelected($viz);
    }

    updateAppScopeVisibility();

    return picklist;
  }

  function updateAppScopeVisibility() {
    const $appScopeContainer = $(`#${lib.chart_div()} .filter-triangle`);
    const $moveIcon = $(`#${lib.chart_div()} .widget-btns-wrapper .move-app`);
    let showAppScopeTriangle = arc.globals.is_edit;
    let showMoveIcon = arc.globals.is_edit;
    if (arc.globals.is_edit) {
      showAppScopeTriangle = arc.report.isAppScopeFilter(picklist.viz_lib());
      showMoveIcon = !showAppScopeTriangle;
    }

    $appScopeContainer.toggleClass('arc-invisible', !showAppScopeTriangle);
    $moveIcon.toggleClass('arc-invisible', !showMoveIcon);
  }

  // If 'picklist_hide_filter_when_empty' is set, then hide the filter if the
  // base field is empty, or if there is no input data.
  function showHide(data_rows) {
    if (lib.build_mode()) {
      return;
    }

    if (!picklist.picklist_hide_filter_when_empty.isChecked) {
      return;
    }

    const base_field = getBaseField();
    let hideit = false;
    if (!base_field) {
      hideit = true;
    }

    if (!hideit) {
      if (data_rows && (data_rows.length === 0) && !isBackendSearchOn()) {
        hideit = true;
      }
    }

    if (hideit) {
      hideFilter();
    } else {
      showFilter();
    }
  }

  function combineRows(rows) {
    const outRows = [];
        const outRowsH = {};

    rows.forEach((row) => {
      const dim = row[0];
          const agg = row[1];

      const h = outRowsH[dim];
      if (h === undefined) {
        outRows.push(row);
        outRowsH[dim] = outRows.length - 1;
        return;
      }

      // eslint-disable-next-line operator-assignment
      outRows[h][1] = outRows[h][1] + agg;
    });

    return outRows;
  }

  function renderOptions(type, options) {
    let liElem;
    let labelElem;
    let inputElem;
    let optionElem;
    // eslint-disable-next-line default-case
    switch (type) {
      case 'dropdown':
        // <select>
        //   <option data-value="%VALUE%" data-alias="%LABEL%" value="%VALUE%">%LABEL%</option>
        // </select>
        optionElem = arc.misc.createFragmentElement({
          tagName: 'option',
          attributes: {
            'data-value': options.VALUE,
            'data-alias': options.LABEL,
            value: options.VALUE,
          },
          text: options.LABEL,
        });
        return optionElem;

      case 'radio': {
        // <ul>
        //   <li>
        //     <div class="form-check">
        //       <label for="%LABEL%-%VALUE%" title="%LABEL%">%LABEL%/label>
        //       <input
        //         id="%LABEL%-%VALUE%"
        //         data-value="%VALUE%"
        //         data-alias="%LABEL%"
        //         type="radio"
        //         name="%NAME%"
        //       >
        //     </div>
        //   </li>
        // </ul>

        liElem = arc.misc.createFragmentElement({
          tagName: 'li',
        });

        const divElem = arc.misc.createFragmentElement({
          tagName: 'div',
          classes: ['form-check']
        });

        inputElem = arc.misc.createFragmentElement({
          tagName: 'input',
          attributes: {
            'data-value': options.VALUE,
            'data-alias': options.LABEL,
            type: 'radio',
            name: options.NAME,
            value: options.VALUE,
            id: `${options.LABEL}-${options.VALUE}`
          },
          classes: ['form-check-input']
        });

        labelElem = arc.misc.createFragmentElement({
          tagName: 'label',
          attributes: {
            title: options.LABEL,
            for: `${options.LABEL}-${options.VALUE}`
          },
          text: options.LABEL,
          classes: ['form-check-label']
        });

        liElem.appendChild(divElem);

        divElem.appendChild(inputElem);
        divElem.appendChild(labelElem);

        return liElem;
      }

      case 'checkbox':
        return arc.misc.createSelectableOption(options);
    }
  }

  function applyFormat(value) {
    const col = lib.dataset().get_column(getBaseField());
    const colFormatter = col?.build_dsobject().displayformat;
    const formatter = colFormatter ?
      arc.utils.translateDisplayFormat(colFormatter, lib.dataset()) :
      undefined;
    return formatter ? formatter(value) : value;
  }

  function handleData($viz) {
    if (picklist.picklist_textbox.isChecked) {
      return picklist;
    }

    let dataRowsToShow = displayedFilters;
    if (lib.isKSQL()) {
      dataRowsToShow = combineRows(dataRowsToShow);
    }

    $viz.find('.picklist-list').empty();
    $viz.find('.picklist-dropdown').empty();
    let style = '';
    if (expanded) {
      style = `max-width:${$viz.width() - 30}px;`;
    }

    const hashOfCurrentSelections = getCurrentSelectionsInHash();
    let $container;
    const isDropdown = picklist.selectionIsDropdown();
    if (picklist.selectionIsDropdown()) {
      $viz.find('.picklist-list').addClass('arc-invisible');
      $container = $viz.find('.picklist-dropdown');
    } else {
      $viz.find('.picklist-dropdown').addClass('arc-invisible');
      $container = $viz.find('.picklist-list');
    }

    $container.removeClass('arc-invisible');
    let filterType;
    const singleSelect = selectionIsSingleSelect();
    if (singleSelect) {
      if (isDropdown) {
        filterType = 'dropdown';
      } else {
        filterType = 'radio';
      }
    } else {
      filterType = 'checkbox';
      $('.picklist-list-wrapper > .picklist-displayed').remove();
    }

    if (picklist.picklist_single_select.isChecked) {
      if (picklist.picklist_display_no_selection.isChecked) {
        $container.append(renderOptions(filterType, {
          NAME: `picklist-${lib.unique_report_id()}`,
          LABEL: '(All)',
          VALUE: 'Select',
          STYLE: style,
        }));
      }
    }

    if (isCustomFilter()) {
      // eslint-disable-next-line default-case
      switch (picklist.picklist_options_sort_order.value()) {
        case 'DEFAULT':
          break;
        case 'ASCENDING':
          dataRowsToShow.sort((a, b) => a[1].localeCompare(b[1]));
          break;
        case 'DESCENDING':
          dataRowsToShow.sort((a, b) => a[1].localeCompare(b[1])).reverse();
          break;
      }
    }

    let notDisplayedCurrentSelectionsCount = Object.keys(hashOfCurrentSelections).length || 0;
    let lastGroup;
    // eslint-disable-next-line array-callback-return
    dataRowsToShow.forEach((row, index) => {
      let val = row[0];

      if (!(isBackendSearchOn() || isFrontendSearchOn($viz)) &&
        (index + notDisplayedCurrentSelectionsCount >= picklistShowItemCount())) {
          isListLongerThanLimit = true;
          dataRowsToShow = dataRowsToShow.slice(0, index);

          return;
      }

      let label;

      let dctype;
      const dataset = lib.dataset();

      if (dataset) {
        dctype = dataset.dataconnection_type();
      }

      if ((dctype === 'teradata') || lib.isKSQL()) {
        label = applyFormat(val);
      } else if (picklist.isDatasetSegment()) {
        label = applyFormat(val);
        val = row[2];
      } else if (lib.isSolr() && !isCustomFilter()) {
        label = `${applyFormat(val)} (${row[1]})`;
      } else if (isCustomFilter()) {
        label = applyFormat(row[1] || val);
      } else {
        label = applyFormat(val);
      }

      data_labels.push(label);
      if (typeof hashOfCurrentSelections[val] !== 'undefined') {
        delete (hashOfCurrentSelections[val]);
        // eslint-disable-next-line no-plusplus
        notDisplayedCurrentSelectionsCount--;
      }

      if (picklist.isDatasetSegment()) {
        const group = row[1] ? row[1] : 'Ungrouped';
        if (group !== lastGroup) {
          lastGroup = group;
          $container.append(`<li class="picklist-segment-label">${group}</li>`);
        }
      }

      $container.append(renderOptions(filterType, {
        NAME: `picklist-${lib.unique_report_id()}`,
        LABEL: label,
        VALUE: val,
        STYLE: style,
      }));
    });

    if (!isFrontendSearchOn($viz)) {
      /*
       * When picklists are used as a cascaded filter, we expect values to
       * be cleared out when the parent value changes. That clearing out
       * happens in the if condition below.
       *
       * The else code path is taken when 'Show Selections' is clicked
       * in the picklist UI element, or when there is no cascading in
       * effect that impacts this picklist.
       */
      const filter_columns = picklist.picklist_filter_columns.value();
      if (!showSelectedOptionsOnly && filter_columns.trim() !== '') {
        const currentSelections = Object.keys(hashOfCurrentSelections);
        if (currentSelections.length > 0) {
          const currentStatus = false;
          currentSelections.forEach((currentValue) => {
            const currentAlias = hashOfCurrentSelections[currentValue];
            selectionChange(currentValue, currentAlias, currentStatus);
          });
        }
      } else {
        const currentSelections = Object.keys(hashOfCurrentSelections);
        currentSelections.sort((a, b) => {
          if ($.isNumeric(a) && $.isNumeric(b)) {
            return Number(a) - Number(b);
          }

          return String(a).localeCompare(String(b));
        });

        if (picklist.picklist_options_sort_order.value() === 'DESCENDING') {
          currentSelections.reverse();
        }

        currentSelections.forEach((k) => {
          $container.append(renderOptions(filterType, {
            NAME: `picklist-${lib.unique_report_id()}`,
            LABEL: hashOfCurrentSelections[k],
            VALUE: k,
            STYLE: style,
          }));
          dataRowsToShow.push([k]);
        });
      }
    }

    showHide(dataRowsToShow);

    updateCountSpan(
      isListLongerThanLimit && !showSelectedOptionsOnly,
      $container,
      dataRowsToShow.length
    );

    return picklist;
  }

  function updateCountSpan(receivedMoreValuesThanLimit, $container, rowCount = 0) {
    const $viz = $(`#${lib.chart_div()}`);
    const $countSpan = $viz.find('.picklist-displayed > span');

    // If more values are fetched then the limit or
    // (Number of values received + already selected values) > limit
    const moreValuesThanLimitPresent = receivedMoreValuesThanLimit ||
      rowCount > picklistShowItemCount(false);
    if (moreValuesThanLimitPresent) {
      $countSpan.addClass('alert-warning');
      $countSpan.parent().attr('title', 'There might be more items than displayed');
    } else {
      $countSpan.removeClass('alert-warning');
      $countSpan.parent().attr('title', null);
    }

    $viz.find('.picklist-displayed span.count').text(rowCount);

    if (selectionIsSingleSelect()) {
      const $countSpanClone = $countSpan.clone();
      $countSpanClone.prependTo($container);
      $countSpanClone.parent().attr('title', () => {
        if (moreValuesThanLimitPresent) {
          return 'There might be more items than displayed';
        }

        return null;
      });
    }
  }

  function displayTextboxData($viz) {
    const o = getSubstitutedOutputParameter();
    if (!o) {
      return;
    }

    const currentSelections = getCurrentSelections();
    // eslint-disable-next-line no-nested-ternary
    const displayTextValue = typeof currentSelections.data === 'undefined' ?
                              '' :
                              (
                                Array.isArray(currentSelections.data) ?
                                currentSelections.data[0] :
                                currentSelections.data
                              );

    const $searchbox = $viz.find('.picklist-textbox');
    $searchbox.val(displayTextValue);
    textbox_text = displayTextValue;
  }

  function picklistSendParamObjects(paramObjects) {
    arc.params.sendParameters({
      paramObjects,
      do_not_refresh: false,
      lib,
    });
  }

  function textboxSendParam($viz) {
    const paramObject = textboxToParam($viz);
    if (paramObject) {
      picklistSendParamObjects([paramObject]);
    }
  }

  /*
   * If $viz is defined, get the current value of the textbox field and place it
   * in textbox_text before sending out.
   *
   * If $viz is undefined, send out the current value of textbox_text.
   */
  function textboxToParam($viz) {
    if ($viz) {
      const s = $viz.find('.picklist-textbox').val();
      textbox_text = s;
    }

    const outputParameter = getOutputParameter();
    if (!outputParameter) {
      // eslint-disable-next-line no-console
      console.log('no output parameter name set for textbox');
      return;
    }

    const newParamObject = new arc.params.Param(lib, {
      outputParameter,
      colDataType: getColtype(),
      value: textbox_text ? [textbox_text] : [],
    });

    return newParamObject;
  }

  /**
   * Removes loading styles, resets search inputs and stored values.
   */
  picklist.resetSearch = function () {
    const shouldResetSearchResult = frontend_search_text !== '';
    const $viz = $(`#${lib.chart_div()}`);
    const $container = picklist.selectionIsDropdown() ?
      $viz.find('.chosen-container .chosen-drop') :
      $viz.find('.picklist-list-wrapper.dropdown-menu');
    $container.find(`#${searchInTheBackendIndicatorContainerId}`).remove();
    setSearchBoxValue('', $viz);
    frontend_search_text = '';
    if (shouldResetSearchResult) {
      search_fn();
    }
  };

  function setupEvents($viz) {
    const $searchInput = getSearchBox($viz);
    setupSearchEvent($searchInput, $viz);
    function textbox_search_fn() {
      const s = $viz.find('.picklist-textbox').val();
      if (textbox_text === s) {
        return;
      }

      lib.page_needs_save();
      textbox_text = s;
      setCurrentSelections({
        data: [textbox_text],
        alias: [textbox_text],
      });
      textboxSendParam($viz);
    }

    $viz.find('.picklist-textbox')
        .off('change')
        .on('change', textbox_search_fn);

    $viz.find('.picklist-textbox-search-button')
        .off('click')
        .on('click', textbox_search_fn);

    $viz.find('.picklist-all')
        .off('change')
        .on('change', function() {
          if (!displayedFilters) {
            return;
          }

          const allInputs = $viz.find('.picklist-list input');
          let $e;
          let dataValue;
          let dataAlias;
          let idx;
          const allSelected = $(this).prop('checked');
          setCurrentSelections({
            all_selected: allSelected,
          });
          if (!isBackendSearchOn() && !isFrontendSearchOn($viz) && !allSelected) {
            // Unset all items in the picklist
            setCurrentSelections({
              data: [],
              alias: [],
            });
            $.each(allInputs, (i, e) => {
              $(e).prop('checked', false);
            });
          } else {
            let isNumeric = false;
            if (!arc.misc.isString(getColtype(), lib.dataset().dataconnection_type())) {
              isNumeric = true;
            }

            $.each(allInputs, (i, e) => {
              $e = $(e);
              dataValue = $e.attr('data-value');
              dataAlias = $e.attr('data-alias');

              if (isNumeric) {
                dataValue = Number(dataValue);
              }

              const currentSelections = getCurrentSelections();
              $e.prop('checked', currentSelections.all_selected);
              idx = currentSelections.data.indexOf(dataValue);
              const currentSelectionsData = currentSelections.data ?
                currentSelections.data.slice() :
                [];
              const currentSelectionsAlias = currentSelections.alias ?
                currentSelections.alias.slice() :
                [];
              if (currentSelections.all_selected) {
                if (idx < 0) {
                  currentSelectionsData.push(dataValue);
                  currentSelectionsAlias.push(dataAlias);
                  setCurrentSelections({
                    data: currentSelectionsData,
                    alias: currentSelectionsAlias,
                  });
                }
              } else if (idx >= 0) {
                  currentSelectionsData.splice(idx, 1);
                  currentSelectionsAlias.splice(idx, 1);
                  setCurrentSelections({
                    data: currentSelectionsData,
                    alias: currentSelectionsAlias,
                  });
                }
            });
          }

          summarize($viz);
          sendParameters($viz, true);
        });

    $viz.find('.picklist-exclude')
        .off('change')
        .on('change', function () {
          setCurrentSelections({
            exclude: $(this).prop('checked') || false,
          });
          summarize($viz);
          sendParameters($viz, true);
        });

    $viz.find('.picklist-include-nulls')
    .off('change.includeNulls')
    .on('change.includeNulls', function toggleIncludeNulls() {
      setCurrentSelections({
        includeNulls: $(this).prop('checked') || false
      });

      sendParameters($viz, true);
    });

    $viz.off('change')
        .on('change', '.picklist-dropdown, .picklist-list input', function() {
          const $this = $(this);

          const currentValue = $this.attr('data-value');
          const currentAlias = $this.attr('data-alias');
          const currentStatus = $this.prop('checked');
          selectionChange(currentValue, currentAlias, currentStatus);
        });

    function onShowSelectedClicked(e) {
      const $target = $(e.target);
      const showSelectedWasClicked = $target.text() === 'Show Selected';
      $target.text(showSelectedWasClicked ? 'Show All' : 'Show Selected');
      showSelectedOptionsOnly = showSelectedWasClicked;

      if (!showSelectedOptionsOnly) {
        getSearchBox($viz).val('');
      }

      search_fn();
    }

    $viz.find('.show-selected-items').offOn('click.showSelected', onShowSelectedClicked);
  }

  function selectionChange(currentValue, currentAlias, currentStatus) {
    const $viz = $(`#${lib.chart_div()}`);
    if (!arc.misc.isString(getColtype(), lib.dataset().dataconnection_type())) {
      if ($.isNumeric(currentValue) && String(Number(currentValue)) === currentValue) {
        // Do not change to NaN for values like 'null', 'NULL', etc.
        // Do not change for huge values that cannot be same
        currentValue = Number(currentValue);
      }
    }

    if (!selectionIsSingleSelect()) {
      if ($viz.find('.picklist-list input:not(:checked)').length > 0) {
        setCurrentSelections({
          all_selected: false,
        });
      } else {
        setCurrentSelections({
          all_selected: true,
        });
      }

      $viz.find('.picklist-all').prop('checked', getCurrentSelections().all_selected);
    }

    if (picklist.selectionIsDropdown()) {
      const $selectedItem = $viz.find('.picklist-dropdown option:selected');
      currentValue = $selectedItem.attr('data-value');
      currentAlias = $selectedItem.attr('data-alias');
      currentStatus = isAnyOptionSelected($viz);
    }

    if (currentStatus) {
      if (selectionIsSingleSelect()) {
        if (isAnyOptionSelected($viz)) {
          setCurrentSelections({
            data: [currentValue],
            alias: [currentAlias],
          });
        } else {
          setCurrentSelections({
            data: [],
            alias: [],
          });
        }
      } else {
        const currentSelections = getCurrentSelections();
        const currentSelectionData = currentSelections.data ?
          currentSelections.data.slice() :
          [];
        const currentSelectionAlias = currentSelections.alias ?
          currentSelections.alias.slice() :
          [];
        currentSelectionData.push(currentValue);
        currentSelectionAlias.push(currentAlias);
        setCurrentSelections({
          data: currentSelectionData,
          alias: currentSelectionAlias,
        });
      }
    } else {
      const currentSelections = getCurrentSelections();
      const idx = currentSelections.data.indexOf(currentValue);
      const currentSelectionData = currentSelections.data ? currentSelections.data.slice() : [];
      const currentSelectionAlias = currentSelections.alias ? currentSelections.alias.slice() : [];
      currentSelectionData.splice(idx, 1);
      currentSelectionAlias.splice(idx, 1);
      setCurrentSelections({
        data: currentSelectionData,
        alias: currentSelectionAlias,
      });
    }

    if (selectionIsSingleSelect()) {
      switchPicklistVisibility($viz, false);
    }

    summarize($viz);
    sendParameters($viz, true);
  }

  function selectionIsMultiSelect() {
    return !selectionIsSingleSelect() &&
           !selectionIsMultiParm() &&
           !selectionIsTextbox();
  }

  function selectionIsSingleSelect() {
    return picklist.picklist_single_select.isChecked;
  }

  function selectionIsMultiParm() {
    return picklist.picklist_multi_parm.isChecked;
  }

  function selectionIsTextbox() {
    return picklist.picklist_textbox.isChecked;
  }

  function isMandatorySingleSelection() {
    // There is not an option called 'All'.
    return (selectionIsSingleSelect() && !picklist.picklist_display_no_selection.isChecked);
  }

  function isApplyMultiSelectChangesTogether() {
    return (!selectionIsSingleSelect() && picklist.picklist_apply_selections_together.isChecked);
  }

  function isAnyOptionSelected($viz) {
    // As we now search on the fly, the temporary displayed search result list
    //  might do not contains checked html elements.
    // In this scenario, the ssaved selection should count first.
    if (getCurrentSelections().data.length > 0) {
      return true;
    }

    if (!selectionIsSingleSelect()) {
      // Multi select
      return $viz.find('.picklist-list input:checked').length > 0;
    }

    let $checked; let $list;
    if (picklist.selectionIsDropdown()) {
      // Single select dropdown
      $checked = $viz.find('.picklist-dropdown option:selected');
      $list = $viz.find('.picklist-dropdown option');
    } else {
      // Single select radio
      $checked = $viz.find('.picklist-list input:checked');
      $list = $viz.find('.picklist-list input');
    }

    // There is not an entry called 'All'. Return true if anything
    // is checked
    if (isMandatorySingleSelection()) {
      return ($checked.length > 0);
    }

      // See if the item picked is 'All'
      const $firstItem = $list[0];
      return !($firstItem && $firstItem === $checked[0]);
  }

  function isBackendSearchOn() {
    return !!backend_search_text;
  }

  function isFrontendSearchOn($viz) {
    return !!getSearchBoxValue($viz);
  }

  /*
   * Display summary information in the picklist
   * - how many items are selected
   * - the actual items, if there are only 1 or 2 selections
   */
  function summarize($viz) {
    if (selectionIsTextbox()) {
      return picklist;
    }

    if (isCustomFilter()) {
      $viz.find('.pick-searchbox').hide();
      $viz.find('.line-separator').hide();
    } else {
      $viz.find('.pick-searchbox').show();
      $viz.find('.line-separator').show();
    }

    if (selectionIsSingleSelect()) {
      $viz.addClass('single-select').removeClass('multi-select');
      $viz.find('.picklist-exclude').prop('checked', false);
    } else {
      $viz.addClass('multi-select').removeClass('single-select');
    }

    handleIncludeNullHtml();

    setCurrentSelections({
      exclude: $viz.find('.picklist-exclude').prop('checked') || false,
    });
    const currentSelections = getCurrentSelections();

    arc.misc.updatePicklistSummary($viz, {
      selectedCount: currentSelections.data.length,
      excluded: currentSelections.exclude,
      individualSelections: isAnyOptionSelected($viz) || isBackendSearchOn(),
      selectionString: currentSelections.data.map((d) => applyFormat(d)).join(', '),
      isSegmentFilter: picklist.isDatasetSegment(),
    });

    return picklist;
  }

  function isDataExtension(params) {
    /*
     * When there are conflicting params in play i.e. other visuals are emitting
     * 'param' to clash with our outputting 'param.data' and 'param.exclude', then
     * we emit 'param' in the case where there is exactly one .data element and
     * .exclude is not set
     * 1. Why do we do this? So that the back controls work on a page. Otherwise,
     *    emitting 'param' leads to a second history element that emits 'param.data'
     *    and 'param.exclude'. Then, hitting back goes back to 'param', which then
     *    emits 'param.data' and 'param.exclude' and we never can move beyond that
     *    state.
     * 2. Why not always do this i.e. why only do it when hasConflictingParams is
     *    set? If other visuals on the page are using params that look like
     *    param2 in ('<<param.data:>>', then it is very difficult to handle the
     *    case where daterange filters output 'param.data' in some cases and
     *    'param' in other cases. So, only make this happen in the scenario where
     *    it is necessary.
     */
    return (selectionIsMultiSelect() &&
      (!(hasConflictingParams && !params.exclude && params.data.length === 1)));
  }

  // params => toSaveSelections or default_selections
  function sendParametersParsed($viz, params = {}) {
    let paramObjects = [];
    const outputParameterRoot = getOutputParameter();
    if (!params.data) {
      params = setCurrentSelections({
        data: [],
      });
    }

    if (!params.alias) {
      params = setCurrentSelections({
        alias: params.data.slice(0),
      });
    }

    const commonParamConfig = {
      colDataType: getColtype(),
      outputParameter: outputParameterRoot,
      isExcluded: !!params.exclude,
      hideExtension: !isDataExtension(params),
      valueType: selectionIsMultiSelect() ? arc.params.ParamEnums.ValueTypeEnum.MULTI :
                                            arc.params.ParamEnums.ValueTypeEnum.SINGLE,
      isDatasetSegment,
      includeNulls: picklist.allow_nulls.isChecked && params.includeNulls
    };

    let paramConfig;
    if (!outputParameterRoot) {
      // eslint-disable-next-line no-console
      console.log('Output parameter not specified!', lib.report_id());
      return picklist;
    }

    if (selectionIsTextbox()) {
      paramObjects.push(textboxToParam($viz));
    } else if (!selectionIsMultiParm()) {
      let newObject;
      paramConfig = arc.misc.extend({}, commonParamConfig);
      paramConfig.value = params.data.slice(0);
      if (isCustomFilter() || picklist.isDatasetSegment()) {
        paramConfig.alias = params.alias.slice(0);
        newObject = new arc.params.CustomParam(lib, paramConfig);
      } else {
        newObject = new arc.params.Param(lib, paramConfig);
      }

      paramObjects.push(newObject);
    } else if (selectionIsMultiParm()) {
      const hashOfAllSelectionAlias = {};
      paramConfig = arc.misc.extend({}, commonParamConfig);
      params.data.forEach((p, i) => {
        const alias = params.alias[i];
        paramConfig.outputParameter = alias;
        paramConfig.value = [params.data[i]];
        paramConfig.isIgnore = false;
        const newObject = new arc.params.Param(lib, paramConfig);
        paramObjects.push(newObject);
        hashOfAllSelectionAlias[alias] = true;
      });
    }

    // This might not be needed when .ignore param is removed
    if (arc.globals.dashboard) {
      /*
       * Are we sending out an ignore? If so, is it necessary?
       * This is important because sending out parameters when they are not necessary
       *   can mess up the history tracking. Try taking this code out, and then using a filter
       *   in a dashboard which is initially unselected. Then make a couple of selections.
       *   Then, go back all the way and notice how you can't go forward anymore.
       */

      let onlyIgnore = true;
        let paramCount = 0;
        let paramName;
      paramObjects.forEach((param) => {
        // eslint-disable-next-line no-plusplus
        ++paramCount;
        if (param.outputParameter.endsWith('.ignore') || param.isIgnore) {
          paramName = arc.params.splitParams(param.outputParameter)[0];
        } else {
          onlyIgnore = false;
        }
      });

      if ((paramCount === 1) && onlyIgnore) {
        let requiredParam = false;

        const globalParams = arc.globals.dashboard.current_params()[0];
        globalParams.some((p) => {
          const keyArr = arc.params.splitParams(p.key);
          if (keyArr[1] !== 'ignore') {
            const k = keyArr[0];
            if (paramName === k) {
              requiredParam = true;
              return true;
            }
          }

          return false;
        });

        if (!requiredParam) {
          return picklist;
        }
      }
    }

    // Escape non 'search' parameters for solr
    if (lib.isSolr()) {
      if (!selectionIsTextbox()) {
        paramObjects.forEach((param) => {
          if (param.outputParameter !== 'search' && typeof param.value === typeof '') {
            // eslint-disable-next-line no-useless-escape
            param.value = param.value.replace(/[\-\&|!\(\)\{\}\[\]\^\"~\*\?:]/g, (x) => `\\${x}`);
          }
        });
      }
    }

    if (!paramObjects.length) {
      paramConfig.isIgnore = true;
      const ignoreParam = new arc.params.Param(lib, paramConfig);
      paramObjects = [ignoreParam];
    }

    picklistSendParamObjects(paramObjects);
    return picklist;
  }

  function sendParameters($viz, needs_save, applyAllTogether) {
    const params = getCurrentSelections();

    // Set page needs save if current selections differ from defaults
    if (lib.build_mode()) {
      if (selectionIsTextbox() ||
          ((params.exclude === default_selections.exclude) &&
          params.includeNulls === default_selections.includeNulls &&
          arc.arraysIdentical(params.alias, default_selections.alias) &&
          arc.arraysIdentical(params.data, default_selections.data))) {
        // Selection same as default, or picklist is a textbox. Thus, don't
        // highlight the save button.
      } else if (needs_save) {
          lib.page_needs_save();
        }
    }

    if (!isApplyMultiSelectChangesTogether() || applyAllTogether) {
      sendParametersParsed($viz, params);
    }

    return picklist;
  }

  picklist.removeAllParams = function() {
    sendParametersParsed();
  };

  picklist.viz_destroy = function() {
    const $viz = $(`#${lib.chart_div()}`);

    // Remove handlers in setupVisualElements and uploadPicker
    $viz.find('.show-selected-items').off('click');
    $viz.find('.selected-picklist-link').off('click');
    $viz.find('.picklist-upload a').off('click');
    $viz.find('.selected-control-wrapper').off('click');
    if (picklist.picklist_upload.isChecked) {
      $viz.find('.configure-app').off('click');
      $('#filter-values-modal .filtervalues-add').off('click');
    }

    // Remove handlers set in setupEvents
    $viz.find('.picklist-searchbox').off('change');
    $viz.find('.picklist-search-button').off('click');
    $viz.find('.picklist-textbox').off('change');
    $viz.find('.picklist-textbox-search-button').off('click');
    $viz.find('.picklist-all').off('change');
    $viz.find('.picklist-include-nulls').off('change.includeNulls');
    $viz.find('.picklist-exclude').off('change');
    $viz.find('.picklist-dropdown, .picklist-list input').off('change');

    $(document).off(`click.app${lib.unique_report_id()}`);
  };

  return picklist;
};

arc.visual.plugins.picklist_extension = function() {
  const ext = function() { return 'arc.visual.plugins.picklist_extension'; };

  ext.viz_name = function() {
    return 'picklist';
  };

  ext.viz_tooltip = function() {
    return 'picklist';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_picklist.png?v=1';
  };

  ext.viz = arc.visual.plugins.picklist;

  return ext;
};

arc.visual.plugins.parameter = function() {
  const picklist = arc.visual.plugins.picklist();
  picklist.expanded(true);
  return picklist;
};

arc.visual.plugins.parameter_extension = function() {
  const ext = function() { return 'arc.visual.plugins.parameter_extension'; };

  ext.viz_name = function() {
    return 'parameter';
  };

  ext.viz_tooltip = function() {
    return 'parameter';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_table.png?v=1';
  };

  ext.viz = arc.visual.plugins.parameter;

  return ext;
};

arc.visual.extensions = arc.visual.extensions || [];
arc.visual.extensions.push(arc.visual.plugins.picklist_extension());
arc.visual.extensions.push(arc.visual.plugins.parameter_extension());
}());
