window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.pie = function() {
  let saved_settings = {},
      data_result,
      lib,
      trellis,
      sort_orders_asc = {},
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false,
      previousDataTracker = arc.transition.dataTracker(),
      colorFetcher = arc.canvas.colorFetcher();
  const makePie = arc.settings.pie.pieOrDonut();
  const useDefaultWidth = arc.settings.pie.defaultWidth();
  const arcWidth = arc.settings.pie.arcWidth();
  useDefaultWidth.addDependents([arcWidth], ['true'], ['false']);
  makePie.addDependents([useDefaultWidth, arcWidth], ['true'], ['false']);
  const showDimensionsOnMarks = arc.settings.pie.showDimensionsOnMarks();
  const showPercentagesOnMarks = arc.settings.pie.showPercentagesOnMarks();
  const markPercentageDecimal = arc.settings.pie.markPercentageDecimal();
  showPercentagesOnMarks.addDependents([markPercentageDecimal]);

  const borderColorSetting = arc.settings.pie.borderColorSetting();
  const borderWidthSetting = arc.settings.pie.borderWidthSetting();

  const tooltipPercentage = arc.report.user_settings.tooltip_percentage();
  const tooltipPercentageName = arc.report.user_settings.tooltip_percentage_name();
  const tooltipPercentageDecimal = arc.report.user_settings.tooltip_percentage_decimal();
  const legendWidthOverride = arc.report.user_settings.legend_width_override();
  const reverseLegend = arc.report.user_settings.reverse_legend_order();
  const legend_style = arc.report.user_settings.legend_style();
  legend_style.addDependents([legendWidthOverride, reverseLegend], ['None'],
    ['Right', 'Bottom', 'Above']);
  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.settings.visuals.enableAnimations(),
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  legend_style,
                                  legendWidthOverride,
                                  reverseLegend,
                                  tooltipPercentage,
                                  tooltipPercentageName,
                                  tooltipPercentageDecimal,
                                  arc.report.user_settings.fixed_color_legend(),
                                  arc.colormap.dimToColorMappingSetting(),
                                  makePie,
                                  useDefaultWidth,
                                  arcWidth,
                                  showDimensionsOnMarks,
                                  showPercentagesOnMarks,
                                  markPercentageDecimal,
                                  borderColorSetting,
                                  borderWidthSetting,
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'axis_ticks_font_size',
                                ]);

  const pie = arc.visual.api();
  let isSunburst;

  const shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimensions',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measures',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  const constraints = {
    minimums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
    },
  };

  pie.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    previousDataTracker.lib(lib);
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return pie;
  };

  pie.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  pie.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return pie;
    }

    saved_settings = settings;
    arc.misc.handleLegacyLegendSetting(saved_settings, legend_style);

    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return pie;
  };

  pie.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        dimensionMessages = [
          lib.createPopoverMessages('dimensionPie'),
          lib.createPopoverMessages('acceptsMultiple'),
        ],
        aggregateMessages = [
          lib.createPopoverMessages('aggregatePie'),
          lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException'),
        ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      dimensions_shelf: dimensionMessages,
      aggregates_shelf: aggregateMessages,
    });

    return pie;
  };

  pie.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  pie.viz_setDataResult = function(result) {
    data_result = result;
    return pie;
  };

  pie.viz_drawVisual = function(viz_div, isWindowResize) {
    lib.chart_div(viz_div);
    lib.undraw();

    const rowData = data_result.rows();
    if (!rowData || rowData.length === 0) {
      return pie;
    }

    // previousDataTracker.enableOnRowCount(sb_rows.length);
    previousDataTracker.enabled(false);

    const config = {
      rows: rowData,
      columns: data_result.columns(),
    };

    config.dimColumns = lib.shelf_columns(config.columns, 'dimensions_shelf');
    config.dim_col = lib.first_column(config.columns, 'dimensions_shelf');
    config.agg_col = lib.first_column(config.columns, 'aggregates_shelf');
    config.color_column = config.dimColumns[config.dimColumns.length - 1];
    config.color_idx = config.color_column.colindex();
    config.agg_idx = config.agg_col.colindex();
    config.colors_domain = arc.misc.arrayUnique(
        config.rows.map(function(row) {
          return row[config.color_idx];
        }));

    config.color = lib.dimensionalColorScale();
    config.color.domain(config.colors_domain);

    if (config.dimColumns.length > 1) {
      isSunburst = true;
    } else {
      isSunburst = false;
    }

    // Filter rows with NULLs in the aggregate columns
    var agg_getter = function(row) {
      return row[config.agg_idx];
    };

    lib.filter_nulls(config, [agg_getter]);
    lib.unselectOnClick();

    trellis = arc.visual.trellis()
                        .x_shelf_name('x_shelf')
                        .y_shelf_name('y_shelf')
                        .x_shelf_reserved(0)
                        .y_shelf_reserved(0)
                        .rows(config.rows)
                        .sort_orders_asc(sort_orders_asc)
                        .columns(config.columns)
                        .viz_lib(lib)
                        .viz_div_sel(d4.select(`#${viz_div}`))
                        .legend_column(config.color_column,
                             true /* color_is_dim */)
                        .legends(config.colors_domain)
                        .legendPositioning(legend_style.value())
                        .color(config.color)
                        .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                        .redrawVisualCb(function() {
                           allowLargeTrellisTemp = true;
                           pie.viz_drawVisual(viz_div);
                         })
                        .setup();

    if (trellis.error_state()) {
      return pie;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      pie.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');

    if (legend_style.legendRequired()) {
      config.trellis.reverse_legend_order(reverseLegend.value());
      config.trellis.legend_width_override(legendWidthOverride.value());
      config.trellis.draw_legend();
    }

    config.trellis.draw_visuals(function(cell_sel, xy_rows,
        _unused_xmeasure, _unused_ymeasure, cellIdx) {
      config.rows = xy_rows;
      var cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_sel, viz_div, cell_div, cellIdx);
    });

    lib.redrawLegendIfNeeded(config.trellis);
  };

  pie.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return pie;
  };

  pie.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return pie;
  };

  function computePercentages(config) {
    // Calculate total for creating percentages
    if (!tooltipPercentage.isChecked && !showPercentagesOnMarks.isChecked) {
      return;
    }

    let total = config.rows.reduce((a, b) => a + Math.abs(Number(b[config.agg_idx])), 0);
    config.rows.forEach(function(row) {
      const aggVal = Number(row[config.agg_idx]);
      const aggPercent = (aggVal / total) * 100;
      row.arcMarkPct = lib.trimDecimal(markPercentageDecimal.value(), aggPercent) + '%';
      row.arcTooltipPct = lib.trimDecimal(tooltipPercentageDecimal.value(), aggPercent) + '%';
    });
  }

  function getCellConfig(config, cell_sel, viz_div, cell_div, cellIdx) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      includeLegend: legend_style.legendRequired(),
      viz_div,
    });

    var r = lib.compute_svg_params({
      width: elem_w,
      height: elem_h,
      columns: data_result.columns(),
      showLegend: false,
      rows: config.rows,
      normal_margin: {
        top: 0,
        right: 0,
        left: 0,
        bottom: 0,
      },
    });

    // Special handling for circular sunburst charts, without axis marks
    // and legends
    var margin = {
                   top: 0,
                   right: 0,
                   left: 0,
                   bottom: 0,
                 },
        width = r.width + r.margin.left + r.margin.right,
        height = r.height + r.margin.top + r.margin.bottom,
        radius = Math.min(width, height) / 2;
    width = radius * 2;
    height = radius * 2;

    let cellConfig = {
      width: width,
      height: height,
      margin: margin,
      radius: radius,
      cellIdx,
    };

    arc.canvas.createContainer(cell_sel, cellConfig);

    return cellConfig;
  }

  /*
   * We are using canvas instead of svg to draw sunbursts.
   *
   * The way we do this is as follows:
   * 1. We have a mainCanvas area where the actual visual is drawn
   * 2. We have a hiddenCanvas area where an invisible layer maps
   *    unique colors to data points
   * 3. We have a highlightCanvas area where a visible layer is drawn
   *    on top of the mainCanvas, and where we can draw marks to
   *    highlight necessary data points.
   *
   * So:
   * We draw the mainCanvas
   * Place a mouseover event over the highlightCanvas (the topmost canvas)
   * Map the mouse position to the color in hiddenCanvas so we can resolve it
   *     to a data point
   * Use highlightCanvas to highlight that data point
   */
  function viz_drawVisualWork(config, cell_sel, viz_div, cell_div, cellIdx) {
    let {
      dimColumns,
    } = config;

    if (config.rows.length === 0) {
      return;
    }

    computePercentages(config);

    let cellConfig = getCellConfig(config, cell_sel, viz_div, cell_div, cellIdx);
    computeData(config, cellConfig);
    arc.canvas.createAllCanvas(cellConfig);
    preDrawAllCanvas(config, cellConfig);

    let startTransitionGenerator = function(d) {
      return {
        data: d.data,
        index: d.index,
        startAngle: d.startAngle,

        // Note that we are setting the endAngle to be the same as startAngle
        // so we can show a transition
        endAngle: d.startAngle,

        padAngle: d.padAngle,
        value: d.value,
      };
    };

    let dataToGroup = function(d) {
      return d.data[config.color_idx];
    };

    let columns = data_result.columns();

    function dataRowFromHierarchyRow(row) {
      const data = row.data;
      const cols = dimColumns.filter(function(c, i) {
        return i < row.depth;
      });

      // Find a row
      let d = data;
      while (d.key) {
        d = d.values[0];
      }

      const dimRow = [];
      cols.forEach(function(c) {
        dimRow[c.colindex()] = d[c.colindex()];
      });

      return {
        row: dimRow,
        columns: cols,
      };
    }

    function text_fn(row) {
      const data = row.data;

      if (data.key) {
        let dataRow = dataRowFromHierarchyRow(row);
        return lib.text_from_columns(dataRow.row, dataRow.columns);
      }

      let tooltipConfig = {};
      if (tooltipPercentage.isChecked) {
        tooltipConfig = {
          post_kvs: [
            {
              key: tooltipPercentageName.value(),
              value: data.arcTooltipPct,
            },
          ],
        };
      }

      return lib.text_from_columns(data, columns, tooltipConfig);
    }

    function params_fn(row) {
      const data = row.data;
      if (data.key) {
        let dataRow = dataRowFromHierarchyRow(row);
        return lib.params_from_columns(dataRow.row, dataRow.columns);
      }

      return lib.params_from_columns(data, columns);
    }

    let drawConfig = {
      drawCanvas,
      text_fn,
      params_fn,
      startTransitionGenerator,
      dataToGroup,
      previousDataTracker,
      lib,
    };

    arc.transition.drawTransition(config, cellConfig, drawConfig);

    const { computedData, highlightCanvasConfig } = cellConfig;
    const legendConfig = {
      rows: computedData,
      getLegendFromRow: (row) => row.data[config.color_idx],
      highlightFn: highlightCanvasConfig.drawRows,
    };

    config.trellis.addLegendHighlights(legendConfig);

    return pie;
  }

  function pointsAreInArc(pts, ptData, larc) {
    // Center of the arc is assumed to be 0,0
    // (pt.x, pt.y) are assumed to be relative to the center
    var r1 = larc.innerRadius()(ptData),
        r2 = larc.outerRadius()(ptData),
        theta1 = larc.startAngle()(ptData),
        theta2 = larc.endAngle()(ptData);

    let inArc = true;
    pts.forEach(function(pt) {
      if (!inArc) {
        return;
      }

      let dist = pt.x * pt.x + pt.y * pt.y,
          angle = Math.atan2(pt.x, -pt.y);

      angle = (angle < 0) ? (angle + Math.PI * 2) : angle;

      inArc = (r1 * r1 <= dist) && (dist <= r2 * r2) &&
          (theta1 <= angle) && (angle <= theta2);
    });

    return inArc;
  }

  function computeData(config, cellConfig) {
    let {
      radius,
    } = cellConfig;

    let {
      dimColumns,
      agg_idx,
      rows,
    } = config;

    var lpie = d4.pie()
                 .sort(null)
                 .value(function(d) {
                    if (!$.isNumeric(d[agg_idx])) {
                      console.log('Expected an aggregate to be a number');
                      return;
                    }

                    return Math.abs(Number(d[agg_idx]));
                 });

    const hierarchyBuilder = d4.nest();

    dimColumns.forEach(function(d, i) {
      if (i < dimColumns.length - 1) {
        hierarchyBuilder.key((row) => row[d.colindex()]);
      }
    });

    const hierarchyData = {
      key: 'root',
      values: hierarchyBuilder.entries(rows),
    };

    const partition = d4.partition()
                        .size([2 * Math.PI, radius - 1]);

    const root = d4.hierarchy(hierarchyData, (d) => d.values)
                 .sum((d) => (typeof (d.values) === 'object') ? 0 : Math.abs(Number(d[agg_idx])));

    partition(root);
    cellConfig.computedData = root.descendants().slice(1);
  }

  function preDrawAllCanvas(config, cellConfig) {
    preDrawCanvas(config, cellConfig, cellConfig.mainCanvasConfig);
    preDrawCanvas(config, cellConfig, cellConfig.hiddenCanvasConfig);
    preDrawCanvas(config, cellConfig, cellConfig.highlightCanvasConfig);
  }

  function preDrawCanvas(config, cellConfig, canvasConfig) {
    let {
      width,
      height,
    } = cellConfig;

    let {
      canvas,
    } = canvasConfig;

    let context = canvas.node().getContext('2d');

    context.translate(width / 2, height / 2);
  }

  function drawCanvas(config, cellConfig, canvasConfig) {
    let {
      width,
      height,
      margin,
      radius,
      computedData,
    } = cellConfig;

    let {
      canvas,
      hidden,
      highlight,
    } = canvasConfig;

    let colorMap = {},
        textHeight = 10 / 2;

    let context = canvas.node().getContext('2d');
    let arcThickness = Number(arcWidth.value());
    let outerRadius = radius > 10 ? radius : 0;
    let innerRadius;
    if (makePie.value() === 'true') {
      innerRadius = 0;
    } else if (useDefaultWidth.value() === 'false' && !isSunburst) {
      if (arcThickness === 0) {
        innerRadius = 0;
      } else {
        innerRadius = Math.max(0, outerRadius - arcThickness);
      }
    }

    var larc = d4.arc()
                 .startAngle(function (d) { return d.x0; })
                 .endAngle(function (d) { return d.x1; })
                 .innerRadius(function (d) {
                   if ((innerRadius || innerRadius === 0) && d.depth === 1) {
                     return innerRadius;
                   }

                   return d.y0;
                 })
                 .outerRadius(function (d) { return d.y1; })
                 .context(context);

    const opacity = lib.colorOpacity();

    let showDims = showDimensionsOnMarks.isChecked;
    let showPercent = showPercentagesOnMarks.isChecked && !isSunburst;
    let numLabels = (showDims && showPercent) ? 2 : ((showDims || showPercent) ? 1 : 0);
    if (!hidden && !highlight) {
      context.globalAlpha = opacity;
    }

    const borderWidth = Number(borderWidthSetting.value());

    if (highlight) {
      context.lineWidth = arc.canvas.highlightLineWidth;
      context.strokeStyle = arc.canvas.highlightStrokeStyle;
    } else {
      context.lineWidth = borderWidth;
      context.strokeStyle = borderColorSetting.value();
    }

    let font = lib.getVisualFont();
    let fontSize = lib.getLabelFontSize() || 10;
    context.font = `${fontSize}px ${font}`;
    context.textBaseline = 'middle';
    context.textAlign = 'center';

    if (!highlight) {
      drawRows(computedData);
    }

    // We separate out the drawRows function so that we can use the same
    // function for highlighting particular rows. We simply pass in an array
    // of rows to highlight, and draw those rows in the highlight canvas.
    // See usage of highlightDrawRows to see more.
    function drawRows(rows) {
      context.clearRect(-width / 2, -height / 2, width, height);

      rows.forEach(function(d, i) {
        const displayFunc = data_result.columns()[config.color_idx].coldisplayformat();
        let colorData = d.data.key ? d.data.key : d.data[config.color_idx];
        let color;

        if (hidden) {
          color = colorFetcher();
          colorMap[color] = d;
        } else {
          color = config.color(colorData);
        }

        // Draw the arc
        context.beginPath();
        larc(d);

        if (highlight) {
          context.stroke();
        } else {
          if (borderWidth > 0) {
            context.stroke();
          }
        }

        context.fillStyle = color;
        context.fill();

        if (hidden || numLabels === 0) {
          return;
        }

        // Draw the labels
        if (showDims) {
          if (!drawLabel(displayFunc(colorData), numLabels === 2 ? -7 : 0)) {
            return;
          }
        }

        if (showPercent) {
          drawLabel(d.data.arcMarkPct, numLabels === 2 ? 7 : 0);
        }

        function drawLabel(text, dy) {
          let center = larc.centroid(d);
          center[1] += dy;
          const libColor = lib.getLabelColor();
          const labelColor = libColor || arc.colorpalette.colorForBrightness(color, opacity);
          context.fillStyle = labelColor;

          const w = context.measureText(text).width;

          const points = [
            {
              x: center[0] - w / 2,
              y: center[1] - textHeight,
            },
            {
              x: center[0] - w / 2,
              y: center[1] + textHeight,
            },
            {
              x: center[0] + w / 2,
              y: center[1] - textHeight,
            },
            {
              x: center[0] + w / 2,
              y: center[1] + textHeight,
            },
          ];

          if (!pointsAreInArc(points, d, larc)) {
            return false;
          }

          context.fillText(text, center[0], center[1]);
          return true;
        }
      });
    }

    canvasConfig.drawRows = drawRows;
    canvasConfig.colorMap = colorMap;
  }

  return pie;
};

arc.visual.plugins.pie_extension = function() {
  var ext = function() { return 'arc.visual.plugins.pie_extension'; };

  ext.viz_name = function() {
    return 'pie';
  };

  ext.viz_tooltip = function() {
    return 'pie';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_pie.png?v=1';
  };

  ext.viz = arc.visual.plugins.pie;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.pie_extension());

}());
