window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.radial = function() {
  var saved_settings = {},
      data_result,
      lib,
      trellis,
      sort_orders_asc = {},
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;

  var scale_type_radial = arc.report.user_settings.scale_type_radial();
  let legend_width_override = arc.report.user_settings.legend_width_override();
  var reverse_legend_order = arc.report.user_settings.reverse_legend_order();
  let legend_style = arc.report.user_settings.legend_style();
  legend_style.addDependents([legend_width_override, reverse_legend_order], ['None'],
    ['Right', 'Bottom', 'Above']);
  var overlap_setting = arc.report.user_settings.overlapping_marks();
  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  legend_style,
                                  legend_width_override,
                                  reverse_legend_order,
                                  overlap_setting,
                                  arc.colorpalette.setting(),
                                  arc.report.user_settings.color_opacity(),
                                  scale_type_radial,
                                ])
                                .removeSetting([
                                  'axis_label_font_size',
                                  'axis_ticks_font_size',
                                  'label_font_size',
                                  'label_color',
                                ]);

  var radial = arc.visual.api();

  var shelf_data = [
    {
      name: "dimensions_shelf",
      display_name: "Dimensions",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "aggregates_shelf",
      display_name: "Measures",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: "tooltip_shelf",
      display_name: "Tooltips",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: "x_shelf",
      display_name: "X Trellis",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "y_shelf",
      display_name: "Y Trellis",
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: "filters_shelf",
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  var constraints = {
    minimums: {
      "dimensions_shelf": 1,
      "aggregates_shelf": 1,
    },
    maximums: {
      "dimensions_shelf": 1
    }
  };

  radial.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return radial;
  };

  radial.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  radial.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return radial;
    }

    saved_settings = settings;
    arc.misc.handleLegacyLegendSetting(saved_settings, legend_style);

    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    lib.restore_basics(settings, switchingVisualType);
    return radial;
  };

  radial.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple')
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple')
        ],
        dimensionMessages = [
          lib.createPopoverMessages('dimensionRadial'),
          lib.createPopoverMessages('acceptsOne')
        ],
        measureMessages = [
          lib.createPopoverMessages('aggregateRadial'),
          lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException')
        ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      dimensions_shelf: dimensionMessages,
      aggregates_shelf: measureMessages
    });

    return radial;
  };

  radial.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  radial.viz_setDataResult = function(result) {
    data_result = result;
    return radial;
  };

  radial.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    var sb_rows = data_result.rows();
    if (!sb_rows || sb_rows.length === 0) {
      return radial;
    }

    lib.unselectOnClick();

    var config = {
      rows: data_result.rows(),
      columns: data_result.columns()
    };

    var columns = data_result.columns(),
        agg_cols = lib.shelf_columns(columns, "aggregates_shelf");

    config.agg_start_idx = agg_cols[0].colindex();
    config.agg_end_idx = agg_cols[agg_cols.length - 1].colindex();
    config.agg_cnt = config.agg_end_idx - config.agg_start_idx + 1;
    config.agg_colnames = columns.slice(config.agg_start_idx,
        config.agg_end_idx + 1).map(
            function(d) {
              return d.colname();
            });

    config.color = d3.scale.ordinal()
                     .domain(config.agg_colnames)
                     .range(lib.colors());

    trellis = arc.visual.trellis()
                     .x_shelf_name("x_shelf")
                     .y_shelf_name("y_shelf")
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select("#" + viz_div))
                     .legends(config.agg_colnames)
                     .legendPositioning(legend_style.value())
                     .color(config.color)
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       radial.viz_drawVisual(viz_div);
                     })
                     .setup();
    if (trellis.error_state()) {
      return radial;
    }

    config.trellis = trellis;

    config.trellis.draw_skeleton(function() {
      radial.viz_drawVisual(viz_div);
    });
    config.trellis.tooltips('trellis');

    if (legend_style.legendRequired()) {
      config.trellis.reverse_legend_order(reverse_legend_order.value());
      config.trellis.legend_width_override(legend_width_override.value());
      config.trellis.draw_legend();
    }

    computeSvgParams(config, viz_div);

    var cols = lib.shelf_columns(config.columns, 'dimensions_shelf');
    var dim_colidx = cols[0].colindex();
    config.dim_getter = function(d) {
      return d.slice(dim_colidx, dim_colidx + 1);
    };
    config.dim_formatter = cols[dim_colidx].coldisplayformat();

    recalulateSvgParams(config);

    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      config.rows = xy_rows;
      var cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_div);
    });

    config.trellis.addLegendHighlights();
    lib.redrawLegendIfNeeded(config.trellis);
    return radial;
  };

  radial.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return radial;
  };

  radial.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return radial;
  };

  // Magic number for padding - this is the existing value being
  // promoted in scope so we can also use it in computeSvgParams
  var padding = 40,
      minSingleTrellisWH = 240;

  function recalulateSvgParams(config) {
    /*
     * Radials are unique in one way - there is a set of visuals to draw within
     * each trellis cell because there is a dimension parameter that specifies
     * distinct radial elements.
     *
     * Figure out the max number of radials to draw within each cell. Then,
     * calculate the dimension of each radial taking into account how many
     * radials need to be drawn within each cell.
     */

    var maxUniqueDims = 0;
    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      maxUniqueDims = Math.max(maxUniqueDims,
          arc.misc.arrayUnique(xy_rows, config.dim_getter).length);
    });

    var min_diam = minSingleTrellisWH - padding;
    config.diam = (Math.min(config.width, config.height) / maxUniqueDims) -
        padding;
    config.diam = Math.max(config.diam, min_diam);

    config.width = Math.max(config.width, config.diam + padding);
    config.height = config.diam + padding;

    if (config.width > config.height) {
      config.width = Math.floor(config.width / config.height) * config.height;
    } else if (config.height > config.width) {
      config.height = Math.floor(config.height / config.width) * config.width;
    }
  }

  function computeSvgParams(config, viz_div) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
    });
    var margin = {top: 0, right: 0, bottom: 0, left: 0};
    var width = elem_w - margin.left - margin.right;
    var height = elem_h - margin.top - margin.bottom;

    width = Math.max(width, 240);
    height = Math.max(height, 240);

    config.margin = margin;
    config.width = width;
    config.height = height;
  }

  function viz_drawVisualWork(config, viz_div) {
    if (config.rows.length === 0) {
      return;
    }

    var sb_rows = config.rows;

    var columns = data_result.columns();
    var agg_getter = function(d) {
      return d.slice(config.agg_start_idx, config.agg_end_idx+1);
    };

    var aggs_from_dim = {};
    var data_from_dim = {};
    sb_rows.forEach(function(d) {
      data_from_dim[config.dim_getter(d)] = d;
      aggs_from_dim[config.dim_getter(d)] = agg_getter(d);
    });
    var unique_dims = arc.misc.arrayUnique(sb_rows, config.dim_getter);

    var margin = config.margin,
        width = config.width,
        height = config.height,
        radius,
        rowcount,
        colcount;

    {
      radius = config.diam / 2;
      width = Math.max((config.diam + padding) * 1, width);
      colcount = Math.max(1, Math.floor(width / (config.diam + padding)));
      rowcount = Math.ceil(unique_dims.length / colcount);
      height = Math.max((config.diam + padding) * rowcount, height);
    }

    var agg_mins,
        agg_maxs,
        agg_scales = [];

    var insist_on_pos_number = function(d) {
      if (isNaN(d)) {
        return 0;
      }
      d = Number(d);
      if (d < 0) {
        return 0;
      }
      return d;
    };

    sb_rows.forEach(function(sb_row, row_idx) {
      var agg_d = agg_getter(sb_row);
      sb_row.agg_total = d3.sum(agg_d, insist_on_pos_number);
      if (agg_mins === undefined) {
        agg_mins = agg_d.slice(0).map(insist_on_pos_number);
        agg_maxs = agg_mins.slice(0);
      }
      agg_d.forEach(function(d, i) {
        agg_mins[i] = Math.min(agg_mins[i], insist_on_pos_number(agg_d[i]));
        agg_maxs[i] = Math.max(agg_maxs[i], insist_on_pos_number(agg_d[i]));
      });
    });

    agg_mins = agg_mins.map(function(d) {
      if (d < 0) {
        return 0;
      }
      return d;
    });
    agg_maxs = agg_maxs.map(function(d) {
      if (d < 0) {
        return 0;
      }
      return d;
    });

    if (scale_type_radial.value() === "ALLMEASURES") {
      var agg_min = d3.min(agg_mins),
          agg_max = d3.max(agg_maxs);

      var agg_extent = [
          Math.sqrt(agg_min/Math.PI),
          Math.sqrt(agg_max/Math.PI)
      ];
      if (agg_extent[0] > 0) {
        agg_extent[0] = 0;
      }
      agg_mins.forEach(function(d, i) {
        agg_scales.push(d3.scale.linear()
                          .domain(agg_extent)
                          .range([0, radius]));
      });
    } else if (scale_type_radial.value() === "ONEDIMENSION") {
    } else {
      // scale_type_radial.value() == "ONEMEASURE"
      agg_mins.forEach(function(d, i) {
        var extent = [
            Math.sqrt((agg_mins[i])/Math.PI),
            Math.sqrt((agg_maxs[i])/Math.PI)
        ];
        if (extent[0] > 0) {
          extent[0] = 0;
        }
        agg_scales.push(d3.scale.linear()
                          .domain(extent)
                          .range([0, radius]));
      });
    }

    var max_domain;
    if (scale_type_radial.value() === "ONEDIMENSION") {
      max_domain = d3.max(sb_rows, function(sb_row) {
        return d3.max(agg_getter(sb_row), function(d) {
          return Math.sqrt(d/Math.PI) / Math.sqrt(sb_row.agg_total/Math.PI);
        });
      });
    }

    var agg_angle = 2 * Math.PI / config.agg_cnt;
    var svg = radial.create_svgs(viz_div, width, height, margin);
    var apath = d3.svg.arc()
                  .innerRadius(0)
                  .outerRadius(function(d, i) {
                    if (d < 0) {
                      d = 0;
                    }
                    if (scale_type_radial.value() === "ONEDIMENSION") {
                      var data = data_from_dim[
                          d3.select(this.parentNode).datum()];
                      return radius * ((Math.sqrt(d/Math.PI) /
                          Math.sqrt(data.agg_total/Math.PI))) / max_domain;
                    }
                    return agg_scales[i](Math.sqrt(d/Math.PI));
                  })
                  .startAngle(function(d, i) {
                    if (overlap_setting.isChecked) {
                      return 0;
                    }
                    return agg_angle * i;
                  })
                  .endAngle(function(d, i) {
                    if (overlap_setting.isChecked) {
                      return 2 * Math.PI;
                    }
                    return agg_angle * (i + 1);
                  });

    var containers =
      svg.selectAll(".nosel")
         .data(unique_dims)
         .enter()
           .append("g")
           .attr("transform", function(d, i) {
             var r = Math.floor(i / colcount);
             var c = i % colcount;
             return "translate(" + (radius + ((config.diam + padding) * c)) + "," +
                 (radius + ((config.diam + padding) * r)) + ")";
           })
           .each(function(d) {
             d3.select(this)
               .selectAll(".nosel")
               .data(aggs_from_dim[d])
               .enter()
                 .append("path")
                 .attr("d", function(d, i) {
                   if (d === "NULL") {
                     d = 0;
                   }
                   var p = apath.call(this, Number(d), i);
                   return p;
                 })
                 .style("opacity", lib.colorOpacity())
                 .style("fill", function(d, i) {
                   return config.color(config.columns[i + config.agg_start_idx].colname());
                 });
           });

    containers.append("line")
              .attr("x1", -radius - 2)
              .attr("y1", radius + (padding/2))
              .attr("x2", radius + 2)
              .attr("y2", radius + (padding/2))
              .style("stroke", "gainsboro")
              .style("stroke-width", 1);

    function wrap(elem) {
      var d3elem = d3.select(elem),
          textLength = elem.getComputedTextLength(),
          text = d3elem.text();

      while ((textLength > config.diam) && (text.length > 0)) {
        text = text.slice(0, -1);
        d3elem.text(text + '...');
        textLength = elem.getComputedTextLength();
      }
    }

    containers.append('text')
              .style('text-anchor', 'middle')
              .attr('y', () => radius + 12)
              .text((d) => d.map((val) => config.dim_formatter(val)))
              .each(function() {
                wrap(this);
              });

    const marks = arc.marks()
      .viz_lib(lib)
      .marker_name('primary')
      .addSelection(containers.selectAll('path'))
      .text_fn((d, i, elem) => {
        const data = data_from_dim[d3.select(elem.parentNode).datum()];
        const c = columns[config.agg_start_idx + i];
        let cValue;
        if (c.coldisplayformat(true)) {
          cValue = lib.formattedDisplay(c.coldisplayformat(), data[config.agg_start_idx + i]);
        } else {
          cValue = data[config.agg_start_idx + i];
        }

        const kvs = [{
          key: c.colname(),
          value: cValue,
          value_is_num: true
        }];
        const tooltipConfig = { pre_kvs: kvs };
        return lib.text_from_columns(data, columns, tooltipConfig);
      })
      .params_fn((d, i, elem, dimsOnly) => lib
        .params_from_columns(data_from_dim[d3.select(elem.parentNode).datum()], columns, dimsOnly))
      .highlight_on_select(true)
      .activate();

    if (legend_style.legendRequired()) {
      marks.legendValueGetter((_, i) => config.agg_colnames[i]);
    }

    return radial;
  }

  radial.create_svgs = function(viz_div, width, height, margin) {
    d3.select("#" + viz_div)
        .append("svg")
        .classed("viz-radial-svg", true)
          .append("g")
          .classed("viz-radial-g", true);
    d3.select("#" + viz_div)
      .selectAll(".viz-radial-svg")
      .style("width", width + margin.left + margin.right + "px")
      .style("height", height + margin.top + margin.bottom + "px");
    d3.select("#" + viz_div)
      .selectAll(".viz-radial-g")
      .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    var sel = d3.select("#" + viz_div)
                .selectAll(".viz-radial-g");
    return sel;
  };

  return radial;
};

arc.visual.plugins.radial_extension = function() {
  var ext = function() { return "arc.visual.plugins.radial_extension"; };

  ext.viz_name = function() {
    return "radial";
  };

  ext.viz_tooltip = function() {
    return "radial";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_radial.png?v=1";
  };

  ext.viz = arc.visual.plugins.radial;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.radial_extension());

}());
