window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};

function sliderBarColor() {
  var s = arc.visual.setting();

  s.id('range_sliderbar_color')
   .section('Style')
   .tab('Style')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('RANGE_SLIDERBAR_COLOR')
   .add_label('Slider Bar Color')
   .value('#45BAC7');

  return s;
}

function innerBarColor() {
  var s = arc.visual.setting();

  s.id('range_innerbar_color')
   .section('Style')
   .tab('Style')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('RANGE_INNERBAR_COLOR')
   .add_label('Inner Bar Color')
   .value('#ccc');

  return s;
}

function labelBackgroundColor() {
  var s = arc.visual.setting();

  s.id('range_labelbg_color')
   .section('Style')
   .tab('Style')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('RANGE_LABELBG_COLOR')
   .add_label('Label Background Color')
   .value('#999');

  return s;
}

function labelForegroundColor() {
  var s = arc.visual.setting();

  s.id('range_labelfg_color')
   .section('Style')
   .tab('Style')
   .type(arc.visual.settingTypeEnum.COLOR)
   .add_name('RANGE_LABELFG_COLOR')
   .add_label('Label Text Color')
   .value('#FFF');

  return s;
}

function sliderBarOpacity() {
  var s = arc.visual.setting();

  s.id('range_sliderbar_opacity')
   .section('Style')
   .tab('Style')
   .type(arc.visual.settingTypeEnum.INTEGER)
   .add_name('RANGE_SLIDERBAR_OPACITY')
   .add_label('Slider Bar Opacity')
   .min(0)
   .max(100)
   .step(2)
   .value('100');

  return s;
}

arc.visual.plugins.range = function() {
  let saved_settings = {};
  let filterSettings;
  let lib;
  let rows;
  let defaultValues = {};
  let columnName;

  const rangeSetting = arc.filter.range();
  const range = arc.visual.api();
  const allowNulls = arc.settings.range.allowNulls();

  const baseFieldSetting = arc.settings.filters.baseField(),
        titleSetting = arc.settings.filters.title(),
        outputParameterSetting = arc.settings.filters.outputParameter();

  const filterAppgroupScope = arc.report.user_settings.filter_appgroup_scope(),
        filterScope = arc.report.user_settings.filter_dataset_scope(),
        filterColumns = arc.settings.filters.filterColumns();

  const saveSelections = arc.settings.range.saveSelections();

  const fixedMinimumSetting = arc.settings.range.fixedMinimum(),
        fixedMaximumSetting = arc.settings.range.fixedMaximum();

  var user_settings =
      arc.visual.defaultSettings({
           isFilter: true,
         })
         .addSetting([
           arc.settings.filters.datasetDisplay(),
           sliderBarColor(),
           sliderBarOpacity(),
           innerBarColor(),
           labelBackgroundColor(),
           labelForegroundColor(),
           arc.settings.range.significantDigits(),
           saveSelections,
           allowNulls,
           fixedMinimumSetting,
           fixedMaximumSetting,
           baseFieldSetting,
           titleSetting,
           outputParameterSetting,
           filterColumns,
           filterAppgroupScope,
           filterScope,
         ]);

  range.base_field = function(_) {
    if (!arguments.length) return columnName;
    columnName = _;
    baseFieldSetting.value(columnName);
    return range;
  };

  range.filterSettings = function(_) {
    if (!arguments.length) return filterSettings;
    filterSettings = _;
    return range;
  };

  function getColtype() {
    let coltype = 'UNKNOWN';
    const base_field = range.base_field();
    const dataset = lib && lib.dataset();

    if (base_field && dataset) {
      const col = dataset.get_column(base_field);
      if (col !== undefined) {
        coltype = col.attr('type');
      }
    } else {
      // isParameterViz()
      coltype = 'STRING';
    }

    return coltype;
  }

  range.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;

    rangeSetting.viz_lib(lib);
    lib.shelf_data([])
       .user_settings(user_settings);
    return range;
  };

  range.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    saved_settings = settings;

    settings.viz_type = settings.viz_type || {};
    settings.viz_type.range = {
      defaultValues,
    };

    return settings;
  };

  range.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return range;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);

    columnName = baseFieldSetting.value();
    if (saveSelections.isChecked) {
      if (settings.viz_type.range) {
        defaultValues = settings.viz_type.range.defaultValues;
      }
    }

    if (lib.dataset()) {
      const datasetName = lib.dataset().dataset_name();
      lib.user_settings().find_by_id('picklist_dataset').value(datasetName);
    }

    range.updateToSaveSelectionsFromParams();

    defaultValues.includeNulls = defaultValues.includeNulls ?? true;

    return range;
  };

  range.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();
    return range;
  };

  range.viz_configure = function() {
    lib.display_settings();
    return range;
  };

  range.removeAllParams = function() {
    let paramConfig = {
      outputParameter: getOutputParameter(),
      isIgnore: true,
    };
    let paramObj = new arc.params.RangeValueParam(lib, paramConfig);
    arc.params.sendParameters({
      paramObjects: [paramObj],
      lib,
    });
  };

  range.viz_getDataRequest = function() {
    if (!columnName) {
      return arc.data.request()
                     .request_type('ignore');
    }

    const dataset = lib && lib.dataset();
    const col = dataset.get_column(columnName);
    if (col.attr('exprhasagg')) {
      return arc.data.request()
                     .request_type('ignore');
    }

    if ((fixedMinimumSetting.value().trim() !== '') &&
        (fixedMaximumSetting.value().trim() !== '')) {
      return arc.data.request()
                     .request_type('ignore');
    }

    const data_request = arc.data.request();
    let sb = lib.getSelectBuilder();
    let rb = lib.getRequestBuilder();

    rb.addAggregate(`min([${columnName}])`);
    rb.addAggregate(`max([${columnName}])`);
    sb.addProjection(`min([${columnName}])`);
    sb.addProjection(`max([${columnName}])`);

    const cols = filterColumns.value();
    if (cols) {
      const filterNames = cols.split(',');
      lib.handleSpecificFilters(filterNames, function(filterName, expr) {
        rb.addFilter(expr);
        sb.addFilter(expr);
      });
    }

    sb = lib.finalizeSql(sb);

    const col1 = arc.data.column()
                         .colsource('base_shelf')
                         .coltype(col.coltype())
                         .colproperty(arc.report.columnTypeEnum.DIMENSION);
    const col2 = arc.data.column()
                         .colsource('base_shelf')
                         .coltype(col.coltype())
                         .colproperty(arc.report.columnTypeEnum.DIMENSION);
    data_request.add_column(col1)
                .add_column(col2);

    data_request.sql(sb.getSql());

    data_request.dsreq(rb.dsreq());
    data_request.finalize();
    return data_request;
  };

  function fixedMinValueSet() {
    return fixedMinimumSetting.value().trim() !== '';
  }

  function fixedMaxValueSet() {
    return fixedMaximumSetting.value().trim() !== '';
  }

  function getDataBoundary() {
    return { min: rows[0][0], max: rows[0][1] };
  }

  function enforceFixedMinMax() {
    const fixedMinimumValueSet = fixedMinValueSet();
    const fixedMaximumValueSet = fixedMaxValueSet();
    if (rows.length === 0) {
      if (fixedMinimumValueSet && fixedMaximumValueSet) {
        rows = [
          [
            Number(fixedMinimumSetting.value()),
            Number(fixedMaximumSetting.value()),
          ]
        ];
      }

      return;
    }

    if (fixedMinimumValueSet) {
      rows[0][0] = Number(fixedMinimumSetting.value());
    }

    if (fixedMaximumValueSet) {
      rows[0][1] = Number(fixedMaximumSetting.value());
    }
  }

  /*
  The defaultValues is restored from the saved data. Sometimes it can show outdated information
  (number which in normal circumstances wouldn't be an option to select).
  This corrects it.
  */

  function correctDefaultValues() {
    if ($.isEmptyObject(defaultValues)) {
      return;
    }

    const { min, max } = getDataBoundary();
    const notFixedAndMinimumOutOfRange = !fixedMinValueSet() && defaultValues.min < min;
    const notFixedAndMaximumOutOfRange = !fixedMaxValueSet() && defaultValues.max > max;

    if (notFixedAndMaximumOutOfRange && notFixedAndMinimumOutOfRange) {
      defaultValues = {};
      return;
    }

    if (notFixedAndMinimumOutOfRange) {
      defaultValues.min = min;
    }

    if (notFixedAndMaximumOutOfRange) {
      defaultValues.max = max;
    }
  }

  /*
   * Set the data this object should use to draw out the visual
   */
  range.viz_setDataResult = function(result) {
    rows = result.rows();
    correctDefaultValues();
    enforceFixedMinMax();
    return range;
  };

  function getTitle() {
    let t = titleSetting.value().trim();
    if (t.length > 0) {
      return t;
    }

    return baseFieldSetting.value();
  }

  function getOutputParameter() {
    let o = outputParameterSetting.value().trim();
    if (o.length > 0) {
      return o;
    }

    return baseFieldSetting.value();
  }

  range.viz_getOutputParameter = getOutputParameter;

  range.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    const $viz = $(`#${viz_div}`);

    enforceFixedMinMax();

    const col = lib.dataset().findColumn(columnName);
    const value = {};
    if (defaultValues && defaultValues.min !== undefined) {
      value.values = {
        min: defaultValues.min,
        max: defaultValues.max,
      };
    }

    const displayFormatter = col.build_dsobject().displayformat ?
      arc.utils.translateDisplayFormat(col.build_dsobject().displayformat, lib.dataset()) :
      undefined;

    rangeSetting.div_id(viz_div)
                .rangeDisplayType(arc.filter.rangeDisplayTypeEnum.minimal)
                .digits(Number(lib.user_settings().find_by_id('significant_digits').value()))
                .colname(getTitle())
                .coltype(col.coltype())
                .displayFormatter(displayFormatter)
                .value(value);

    rangeSetting.paintControl(d3.selectAll(`#${viz_div}`));

    $viz.find('.range-include-nulls-wrapper').toggleClass('arc-invisible', !allowNulls.isChecked);
    $viz.find('.range-include-nulls').prop('checked', defaultValues.includeNulls ?? true);

    if (rows && rows.length > 0) {
      rangeSetting.inputValues(rows);
      postParams();
    }

    rangeSetting.addChangeCb('visual.range', () => {
      if (saveSelections.isChecked) {
        lib.page_needs_save();
      }

      postParams();
    });

    function postParams() {
      const v = rangeSetting.fetch_value();

      if (saveSelections.isChecked) {
        defaultValues = $.extend(defaultValues, v.values);
        defaultValues.includeNulls = allowNulls.isChecked ? $viz.find('.range-include-nulls').prop('checked') : false;
      }

      const do_not_refresh = false;
      const paramConfig = {
        baseField: range.base_field(),
        colDataType: getColtype(),
        outputParameter: getOutputParameter(),
        includeNulls: defaultValues.includeNulls,
        keyType: arc.report.columnTypeEnum.MEASURE,
        value: [{
          start: v.values.min,
          end: v.values.max,
        }],
      };

      const paramObject = new arc.params.RangeValueParam(lib, paramConfig);
      arc.params.sendParameters({
        paramObjects: [paramObject],
        do_not_refresh,
        lib,
      });
    }

    return range;
  };

  // This function is called when user hits dashboard's param reset button
  range.viz_resetControl = function() {
    if (rangeSetting) {
      let v;

      if (saved_settings.viz_type.range && saved_settings.viz_type.range.defaultValues) {
        v = saved_settings.viz_type.range.defaultValues;
      }

      if (!v) {
        if (rows && rows.length > 0) {
          v = {
            min: rows[0][0],
            max: rows[0][1],
          };
        }
      }

      if (v) {
        rangeSetting.defaultValues(v);
      }
    }

    return range;
  };

  range.setVisibility = function (showViz) {
    var $viz = $('#' + lib.chart_div());
    if (showViz) {
      $viz.removeClass('arc-invisible');
    } else {
      $viz.addClass('arc-invisible');
    }
  };

  range.updateToSaveSelectionsFromParams = function() {
    const paramData = lib.getParams().getkvs();
    let changed = false;

    if (paramData[`${columnName}.start`]) {
      const min = Number(paramData[`${columnName}.start`]);
      if (min !== defaultValues.min) {
        defaultValues.min = min;
        changed = true;
      }
    }

    if (paramData[`${columnName}.end`]) {
      const max = Number(paramData[`${columnName}.end`]);
      if (max !== defaultValues.max) {
        defaultValues.max = max;
        changed = true;
      }
    }

    if (paramData[`${columnName}.include_nulls`]) {
      const includeNulls = paramData[`${columnName}.include_nulls`] === 'true';
      if (includeNulls !== defaultValues.includeNulls) {
        defaultValues.includeNulls = includeNulls;
        changed = true;
      }
    }

    if (!changed) {
      const newNumber = Number(paramData[`${columnName}`]);

      if (newNumber) {
        defaultValues.min = newNumber;
        defaultValues.max = newNumber;
      } else if (saveSelections.isChecked) {
        defaultValues = saved_settings.viz_type?.range?.defaultValues ?? {};
      }
    }

    rangeSetting.defaultValues(defaultValues);
  };

  return range;
};

arc.visual.plugins.range_extension = function() {
  var ext = function() { return 'arc.visual.plugins.range_extension'; };

  ext.viz_name = function() {
    return 'range';
  };

  ext.viz_tooltip = function() {
    return 'range';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_range.png?v=1';
  };

  ext.viz = arc.visual.plugins.range;

  return ext;
};

arc.visual.extensions = arc.visual.extensions || [];
arc.visual.extensions.push(arc.visual.plugins.range_extension());

}());
