window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};

const vizSqlInfo = `
  <div class="viz-sql-info">
    Click <a class="viz-sql-view" data-test-id="viz-sql-view">here</a> to open the visual in full-screen mode and
    view/edit the SQL statement being run
  </div>
`;

arc.visual.plugins.rawtable = function() {
  var sb = null,
      rb = null,
      sqlText,
      dsreq,
      data_result,
      modified = false,
      sqlEditor;
  let previousSqlText;
  let isEditModalCancelled;
  let $applyButton;
  let $cancelButtons;

  const rawtable = arc.visual.plugins.table();
  const tableDrawVisual = rawtable.viz_drawVisual;

  const userSettings = arc.visual.defaultSettings({
                                   noTrellis: true,
                                   noAxes: true,
                                   noTooltip: true,
                                 });

  const getSuperModal = ($editor) => $editor.closest('.super-modal');

  const getApplyButton = ($editor) => getSuperModal($editor).find('#apply-button');

  const getCancelButtons = ($editor) => getSuperModal($editor).find('.modal-cancel-btn');

  const setUpOnApply = (lib, $editor) => {
    $applyButton = getApplyButton($editor);

    $applyButton.offOn('click.applyChanges', () => {
      const newValue = sqlEditor.value();

      if (newValue !== previousSqlText) {
        sqlText = newValue;
        rawtable.viz_lib().page_needs_save();
      }

      lib.sqlQuery(sqlText);

      previousSqlText = null;
      isEditModalCancelled = false;
    });
  };

  const setUpOnCancel = ($editor) => {
    $cancelButtons = getCancelButtons($editor);
    $cancelButtons.offOn('click.cancelModal', () => {
      isEditModalCancelled = true;
    });
  };

  const setUpEditModal = () => {
    const lib = rawtable.viz_lib();

    const isEditorShown = arc.globals.is_edit && lib.inModal();
    if (!isEditorShown) {
      previousSqlText = sqlText;
      return;
    }

    const vizDiv = lib.chart_div();
    const $viz = $(`#${vizDiv}`);

    const $editorContainer = $(`<div class="viz-sqleditor edit-expression-holder"></div>`);
    const $editor = $(`<div id="viz-sqleditor-${vizDiv}" class="viz-sqleditor-predecessor"></div>`);
    $editorContainer.append($editor);

    $viz.find('.viz-sqleditor').remove();
    $viz.prepend($editorContainer);

    setUpOnApply(lib, $editor);
    setUpOnCancel($editor);

    $editor.append($('#expression-editor-template').html());

    const editorFieldId = `viz-expreditor-${vizDiv}`;

    $editor
      .find('.expression-field.expression-data')
      .attr('id', editorFieldId);

    sqlEditor = arc.misc.expressionEditor(
      lib.dataset(),
      null,
      editorFieldId,
      $editor,
      {
        includeTables: true,
        useRawNames: true
      }
    );

    const $validationBtn = $editor.find('.expression-validate');
    const $refreshBtn = $editor.find('.validate-and-save');

    arc.misc.addExpressionValidation(
      $validationBtn,
      $refreshBtn,
      sqlEditor,
      lib.dataset(),
      'query',
      lib,
      {
        submitCb: () => {
          arc.report.alertMessageAllHide();
          arc.css.CUSTOMCSS_MANAGER.applyCustomCss();

          sqlText = sqlEditor.value();
          lib.sqlQuery(sqlText);

          lib.refresh(true);
        }
      }
    );

    if (sqlText) {
      sqlEditor.value(sqlText);
    }

    // Ensures text fill the entire available space rather than one thin column
    sqlEditor.resize();
  };

  rawtable.sqlText = function() {
    return sqlText;
  };

  rawtable.viz_drawDesigner = function(shelves_div) {
    const lib = rawtable.viz_lib();

    if (lib.build_mode()) {
      if (lib.sqlQuery()) {
        sqlText = lib.sqlQuery();
      }
    }

    $('#app-shelves').find('.viz-sql-info').remove();
    $('#app-shelves').html(vizSqlInfo);
    $('#app-shelves').find('.viz-sql-view').offOn('click.fullscreen', () => {
      const report = lib.report();
      const widgetId = report.reportWidgetId();
      $(`#${widgetId} .viz-modal-link`).trigger('click');
    });
    $('#viz-refresh').hide();

    return rawtable;
  };

  rawtable.viz_getDataRequest = function() {
    const data_request = arc.data.request();
    const lib = rawtable.viz_lib();

    if (lib.isSolr()) {
      const requestBuilder = arc.requestBuilder();
      requestBuilder.addSolrQuery(lib.solrSearchData())
        .requestType('SOLR')
        .datasetId(lib.dataset().dataset_id());

      data_request.sql('').dsreq(requestBuilder.dsreq());
    } else {
      if (sqlText) {
        sqlText = arc.misc.addLimitToSQLQuery(sqlText, lib.maxRowCount());
      }

      data_request.sql(sqlText);
    }

    return data_request;
  };

  let saved_info = {};

  rawtable.viz_getSettings = function() {
    let settings = saved_info;
    const lib = rawtable.viz_lib();
    settings = lib.save_basics(settings);
    settings.sql_text = sqlText;
    settings.dsreq = dsreq;
    settings.customWidths = rawtable.customWidths();
    saved_info = settings;
    return settings;
  };

  rawtable.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return rawtable;
    }

    saved_info = settings;
    const lib = rawtable.viz_lib();
    lib.restore_basics(settings, switchingVisualType);
    if (lib.sqlQuery()) {
      sqlText = lib.sqlQuery();
      dsreq = lib.dsreq();
    } else {
      sqlText = settings.sql_text;
      dsreq = settings.dsreq;
    }

    if (lib.sqlQuery()) {
      const requestBuilder = arc.requestBuilder();
      requestBuilder.datasetId(lib.dataset_id());
      if (dsreq) {
        if (typeof dsreq === 'string') {
          requestBuilder.dsreq(JSON.parse(dsreq));
        } else {
          requestBuilder.dsreq(dsreq);
        }
      }

      if (typeof settings.sql_text === 'string' && lib.sqlQuery() !== settings.sql_text) {
        lib.sqlQuery(settings.sql_text);
        sqlText = settings.sql_text;
      }
    }

    if (sqlEditor) {
      sqlEditor.value(sqlText);
    }

    rawtable.customWidths(settings.customWidths);

    return rawtable;
  };

  rawtable.viz_drawVisual = function(viz_div) {
    tableDrawVisual(viz_div);
    setUpEditModal();

    return rawtable;
  };

  rawtable.drawOnDataFetchError = function(viz_div) {
    setUpEditModal();
    return rawtable;
  };

  rawtable.viz_destroy = () => {
    const lib = rawtable.viz_lib();
    const vizDiv = lib.chart_div();

    if ($applyButton) {
      $applyButton.off('click.applyChanges');
      $applyButton = undefined;
    }

    if ($cancelButtons) {
      $cancelButtons.off('click.cancelModal');
    }

    if (isEditModalCancelled) {
      sqlText = previousSqlText;
      previousSqlText = null;
      isEditModalCancelled = false;

      lib.sqlQuery(sqlText);
    }

    if (sqlEditor) {
      sqlEditor.destroy();
    }

    $(`#${vizDiv}`).find('.viz-sqleditor').remove();
  };

  return rawtable;
};

arc.visual.plugins.rawtable_extension = function() {
  const ext = function() { return 'arc.visual.plugins.rawtable_extension'; };

  ext.viz_name = function() {
    return 'rawtable';
  };

  ext.viz_tooltip = function() {
    return 'raw table';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_rawtable.png?v=1';
  };

  ext.viz = arc.visual.plugins.rawtable;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.rawtable_extension());
}());
