window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.scatter = function() {
  let sort_orders_asc = {},
      saved_settings = {},
      data_result,
      lib,
      trellis,
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false,
      previousDataTracker = arc.transition.dataTracker(),
      colorFetcher = arc.canvas.colorFetcher();

  var scale_x_independent = arc.report.user_settings.scale_dim_independent();
  scale_x_independent.labels(['Independent x scale']);
  var scale_y_independent = arc.report.user_settings.scale_agg_independent();
  scale_y_independent.labels(['Independent y scale']);
  var x_axis_label = arc.report.user_settings.x_axis_label();
  var y_axis_label = arc.report.user_settings.y_axis_label();

  // donotuse_axes_padding only exists for backward compatibility
  var donotuse_axes_padding = arc.report.user_settings.axes_padding();
  donotuse_axes_padding.hidden(true);

  var x_axis_padding = arc.report.user_settings.x_axis_padding();
  var y_axis_padding = arc.report.user_settings.y_axis_padding();
  var mark_size_range = arc.report.user_settings.mark_size_range();
  const horizontal_gridlines_enabled = arc.report.user_settings.horizontal_gridlines_enabled();
  const vertical_gridlines_enabled = arc.report.user_settings.vertical_gridlines_enabled();
  let legendWidthOverride = arc.report.user_settings.legend_width_override();
  var reverseLegend = arc.report.user_settings.reverse_legend_order();
  let legend_style = arc.report.user_settings.legend_style();
  legend_style.addDependents([legendWidthOverride, reverseLegend], ['None'],
    ['Right', 'Bottom', 'Above']);
  var transition_delay = arc.report.user_settings.transition_delay();
  var trendColor = arc.report.user_settings.trendline_color();
  var trendDecimal = arc.report.user_settings.trendline_tooltip_decimal();
  var trendlineLinear = arc.report.user_settings.trendline_linear();
  trendlineLinear.addDependents([trendColor, trendDecimal]);
  var x_setting_scale = arc.report.user_settings.scale_log10();
  let reverse_scale_x = arc.settings.scatter.reverse_scale_x();
  let reverse_scale_y = arc.settings.scatter.reverse_scale_y();
  x_setting_scale.group("X Axis Scale");
  x_setting_scale.id("scale_type_x");
  var y_setting_scale = arc.report.user_settings.scale_log10();
  y_setting_scale.group("Y Axis Scale");
  y_setting_scale.id("scale_type_y");
  var autoplay = arc.report.user_settings.autoplay_animation();
  const minXScale = arc.settings.scatter.minXAxis(),
    maxXScale = arc.settings.scatter.maxXAxis(),
    minYScale = arc.settings.scatter.minYAxis(),
    maxYScale = arc.settings.scatter.maxYAxis(),
    continuousY = arc.settings.scatter.makeYAxisContinuous(),
    continuousX = arc.settings.scatter.makeXAxisContinuous();

  x_setting_scale.addDependents([maxXScale, minXScale], ['LOG10'], ['LINEAR']);
  y_setting_scale.addDependents([minYScale, maxYScale], ['LOG10'], ['LINEAR']);

  const labelByColor = arc.settings.scatter.labelByColor();

  var user_settings = arc.visual.defaultSettings()
                                .addSetting([
                                  arc.settings.visuals.enableAnimations(),
                                  arc.settings.visuals.visualAutodetectTime(),
                                  labelByColor,
                                  mark_size_range,
                                  legend_style,
                                  legendWidthOverride,
                                  reverseLegend,
                                  transition_delay,
                                  autoplay,
                                  trendlineLinear,
                                  trendColor,
                                  trendDecimal,
                                  x_setting_scale,
                                  y_setting_scale,
                                  continuousX,
                                  continuousY,
                                  minXScale,
                                  maxXScale,
                                  minYScale,
                                  maxYScale,
                                  donotuse_axes_padding,
                                  x_axis_padding,
                                  y_axis_padding,
                                  horizontal_gridlines_enabled,
                                  vertical_gridlines_enabled,
                                  scale_x_independent,
                                  scale_y_independent,
                                  x_axis_label,
                                  y_axis_label,
                                  reverse_scale_x,
                                  reverse_scale_y,
                                ])
                                .removeSetting([
                                  'label_color',
                                ]);

  arc.addColorSettings(user_settings, {
    includeUseCustomColorSetting: true,
    includeDimensionMapSetting: true,
    includeMeasureRangeMapSetting: true,
    includeOpacitySetting: true,
    includeFixLegendSetting: true,
  });

  var scatter = arc.visual.api();

  var shelf_data = [
    {
      name: 'x_shelf',
      display_name: 'X Axis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: 'y_shelf',
      display_name: 'Y Axis',
      display_html: arc.report.y_shelf_html(),
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: 'color_shelf',
      display_name: 'Colors',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: 'size_shelf',
      display_name: 'Size',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE
    },
    {
      name: 'transition_shelf',
      display_name: 'Transition',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  var constraints = {
    minimums: {
      "x_shelf": 1,
      "y_shelf": 1,
    },
    maximums: {
      "color_shelf": 1,
      "size_shelf": 1,
    }
  };

  scatter.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    previousDataTracker.lib(lib);
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return scatter;
  };

  scatter.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  function restoreAxesPadding() {
    // Make axes_padding settings backward compatible
    var v = Number(donotuse_axes_padding.value());
    var defaultValue = 5;
    var defaultValueStr = defaultValue + '';

    if (v !== defaultValue) {
      donotuse_axes_padding.value(defaultValueStr);
      x_axis_padding.value(v + '');
      y_axis_padding.value(v + '');
    }
  }

  scatter.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return scatter;
    }

    saved_settings = settings;
    arc.misc.handleLegacyLegendSetting(saved_settings, legend_style);

    lib.restore_basics(settings, switchingVisualType);

    restoreAxesPadding();

    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return scatter;
  };

  scatter.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup()
       .setupSwitchXY(shelves_div);

     // Define messages to be included in shelves
     var xMessages = [
                       lib.createPopoverMessages('xAxis'),
                       lib.createPopoverMessages(['trellisFirst', 'columns.'])
                      ],
         yMessages = [
                       lib.createPopoverMessages('yAxis'),
                       lib.createPopoverMessages(['trellisFirst', 'rows.'])
                     ],
         colorMessages = [
                           lib.createPopoverMessages('colorScatter'),
                           lib.createPopoverMessages('acceptsOne')
                         ],
         sizeMessages = [
                           lib.createPopoverMessages('sizeScatter'),
                           lib.createPopoverMessages('acceptsOne')
                         ],
         transitionMessages = [
                                 lib.createPopoverMessages('transitionScatter'),
                                 lib.createPopoverMessages('acceptsOne')
                               ];
     // Create those popovers
     lib.addShelfDescription(constraints, {
                                             color_shelf: colorMessages,
                                             y_shelf: yMessages,
                                             x_shelf: xMessages,
                                             size_shelf: sizeMessages,
                                             transition_shelf: transitionMessages,
                                           });

    return scatter;
  };

  scatter.viz_getDataRequest = function(config={}) {
    let {
      previousDataRequest,
      previousDataResult,
      permitAutoTimeBucketing,
    } = config;

    return lib.getCheckedDataRequest(constraints,
        {
          previousDataRequest,
          previousDataResult,
          permitAutoTimeBucketing,
        });
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  scatter.viz_setDataResult = function(result) {
    data_result = result;
    return scatter;
  };

  var config;

  scatter.config = function() {
    return config;
  };

  scatter.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    config = {};
    config.rows = data_result.rows();
    if (config.rows.length === 0) {
      return scatter;
    }

    previousDataTracker.enableOnRowCount(config.rows.length);

    lib.unselectOnClick();

    initTransitions(config);
    setupColumns(config);
    setupTrendlines(config);
    lib.setupColors(config);

    config.orig_rows = config.rows;
    setupTransitions(config);
    if (config.transition.col) {
      drawTransitionControls(config, viz_div);
    }

    if (config.color_is_dim !== false) {
      user_settings.find_by_id('measure_ranges').hide();
    }

    // Saves x and y scale from settings
    if (!setupXY(config)) {
      return scatter;
    }

    var viz_settings = {
      legend_contextmenu_shown: false,
      show_cb: [],
      hide_cb: []
    };

    trellis = arc.visual.trellis()
                     .x_shelf_extracol(config.x_measure_values)
                     .y_shelf_extracol(config.y_measure_values)
                     .x_shelf_name("x_shelf")
                     .y_shelf_name("y_shelf")
                     .x_shelf_reserved(
                         config.x_agg_cols.length > 1 ?
                           config.x_agg_cols.length :
                           1)
                     .y_shelf_reserved(
                         config.y_agg_cols.length > 1 ?
                           config.y_agg_cols.length :
                           1)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d4.select("#" + viz_div))
                     .legend_column(
                         (config.color_idx >= 0) ? config.color_column : undefined,
                         config.color_is_dim
                     )
                     .legendPositioning(legend_style.value())
                     //TODO: .legend_size_column(config.size_idx >= 0 ? config.size_scale : undefined)
                     .color(config.color, config.palette, config.colorSettings)
                     .legend_show_cb(function(contextmenu, params) {
                       viz_settings.show_cb.forEach(function(cb) {
                         cb(contextmenu, params);
                       });
                       viz_settings.legend_contextmenu_shown = true;
                     })
                     .legend_hide_cb(function(contextmenu, params) {
                       viz_settings.legend_contextmenu_shown = false;
                       viz_settings.hide_cb.forEach(function(cb) {
                         cb();
                       });
                     })
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       scatter.viz_drawVisual(viz_div);
                     })
                     .otherTrellisSettings([scale_x_independent, scale_y_independent])
                     .setup();

    if (trellis.error_state()) {
      return scatter;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      scatter.viz_drawVisual(viz_div);
    });

    if (legend_style.legendRequired()) {
      config.trellis.reverse_legend_order(reverseLegend.value());
      config.trellis.legend_width_override(legendWidthOverride.value());
      config.trellis.draw_legend();
    }

    computeCellSize(config, viz_div);

    // Compute scales for dimensions or aggregates now if they're dependent
    if (!scale_x_independent.isChecked) {
      computeScales(config, config.orig_rows, 'x');
    }

    if (!scale_y_independent.isChecked) {
      computeScales(config, config.orig_rows, 'y');
    }

    let cellConfig;

    // Now draw visuals within trellis
    if (!config.transition.col) {
      config.trellis.draw_visuals(
        function(cell_sel, xy_rows, xmeasure, ymeasure, cellIdx) {
          // Correctly set up axis scales
          if (scale_x_independent.isChecked) {
            computeScales(config, xy_rows, 'x');
          }

          if (scale_y_independent.isChecked) {
            computeScales(config, xy_rows, 'y');
          }

          cellConfig = {
            cell_sel,
            rows: xy_rows,
            xmeasure,
            ymeasure,
            viz_settings,
            cellIdx,
          };

          // Draw visual/axis/legend
          drawBaseVisualCanvas(config, cellConfig);
        });
    } else {
      const cells = [];
      config.trellis.draw_visuals(
        (cell_sel, xy_rows, xmeasure, ymeasure, cellIdx) => {
          cells.push({
            cell_sel,
            rows: xy_rows,
            xmeasure,
            ymeasure,
            viz_settings,
            cellIdx,
          });
        }
      );

      config.transition.draw = function() {
        cells.forEach((cell) => {
          // Correctly set up Axis
          if (scale_x_independent.isChecked) {
            computeScales(config, cell.rows, 'x');
          }

          if (scale_y_independent.isChecked) {
            computeScales(config, cell.rows, 'y');
          }

          // Draw visual/axis/legend
          drawBaseVisualCanvas(config, cell);
          cellConfig = cell;
        });

        config.initialSketch = false;
      };

      config.transition.curr_idx = 0;
      config.initialSketch = true;
      config.transition.draw();
    }

    config.trellis.tooltips('trellis');

    const legendConfig = {
      rows: config.rows,
      getLegendFromRow: (row) => row.colorLabel,
      highlightFn: cellConfig.highlightCanvasConfig.drawRows,
    };

    config.trellis.addLegendHighlights(legendConfig);
    lib.redrawLegendIfNeeded(config.trellis);
    return scatter;
  };

  scatter.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return scatter;
  };

  scatter.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return scatter;
  };

  function initTransitions(config) {
    config.transition = {};
    if (isNaN(transition_delay.value())) {
      config.transition.delay = 1000;
    } else {
      config.transition.delay = Number(transition_delay.value());
      config.transition.delay = Math.max(50, config.transition.delay);
      config.transition.delay = Math.min(60000, config.transition.delay);
    }
  }

  function drawTransitionControls(config, viz_div) {
    var controls_div =
      d4.select("#" + viz_div)
        .append("div")
        .classed("viz-transition-controls", true)
        .classed("unselectable", true);

    config.transition.display =
      controls_div.append("span")
                  .text(config.transition.values[0]);
    config.transition.playpause =
      controls_div.append("span")
                    .append("i")
                    .attr("class", "fa fa-play");
    config.transition.first =
      controls_div.append("span")
                    .append("i")
                    .attr("class", "fa fa-fast-backward");
    config.transition.back =
      controls_div.append("span")
                    .append("i")
                    .attr("class", "fa fa-step-backward");
    config.transition.forward =
      controls_div.append("span")
                    .append("i")
                    .attr("class", "fa fa-step-forward");
    config.transition.last =
      controls_div.append("span")
                    .append("i")
                    .attr("class", "fa fa-fast-forward");

    function play_visibility() {
      if (config.transition.playing) {
        config.transition.playpause.attr("data-action", "pause");
        config.transition.playpause.attr("class", "fa fa-pause");
      } else {
        config.transition.playpause.attr("data-action", "play");
        config.transition.playpause.attr("class", "fa fa-play");
      }
    }

    // Setup initial visibility
    play_visibility();

    function set_playing(playing) {
      config.transition.playing = playing;
      play_visibility();
    }

    function at_end() {
      return (config.transition.curr_idx >= config.transition.values.length - 1);
    }

    function play_animation(playIcon) {
      var play_clicked = (playIcon.dataset.action === "play");
      if (play_clicked) {
        if (!config.transition.playing) {
          set_playing(true);
          if (!at_end()) {
            config.transition.curr_idx++;
            setTimeout(draw_next_transition, 0);
          }
        }
      } else {
        set_playing(false);
      }
    }

    function draw_next_transition() {
      if (!config.transition.playing || $('.fa-play').length === 1) {
        return;
      }

      config.transition.draw();
      if (at_end()) {
        set_playing(false);
        return;
      }

      setTimeout(draw_next_transition, config.transition.delay);
      ++config.transition.curr_idx;
    }

    config.transition.playpause.on("click.transition", function() {
      play_animation(this);
    });

    config.transition.forward.on("click.transition", function() {
      if (!at_end()) {
        config.transition.curr_idx++;
        if (!config.transition.playing) {
          config.transition.draw();
        }
      }
    });

    config.transition.back.on("click.transition", function() {
      if (config.transition.curr_idx > 0) {
        set_playing(false);
        config.transition.curr_idx--;
        if (!config.transition.playing) {
          config.transition.draw();
        }
      }
    });

    config.transition.first.on("click.transition", function() {
      if (config.transition.curr_idx > 0) {
        config.transition.curr_idx = 0;
        set_playing(false);
        config.transition.draw();
      }
    });

    config.transition.last.on("click.transition", function() {
      if (config.transition.curr_idx < config.transition.values.length - 1) {
        config.transition.curr_idx = config.transition.values.length - 1;
        set_playing(false);
        config.transition.draw();
      }
    });

    if (autoplay.value() === 'true') {
      config.transition.playpause._groups[0][0].dataset.action = 'play';
      play_animation(config.transition.playpause._groups[0][0]);
    }

    return scatter;
  }

  function computeCellSize(config, viz_div) {
    const { elem_w, elem_h } = lib.computeTrellisElementParams({
      viz_div,
      trellis: config.trellis,
      includeLegend: legend_style.legendRequired(),
      extraVerticalPadding: config.transition.col ? 30 : 0,
    });

    const r = lib.compute_svg_params({
      width: elem_w,
      height: elem_h,
      viz_div_elem: $(`#${viz_div}`),
      columns: config.columns,
      x_idx: config.x_orig_idx,
      rows: config.orig_rows,
      x_is_dim: config.x_is_dim,
      x_is_slanted: config.x_is_slanted,
      x_formatter: config.x_col.coldisplayformat(),
      y_idx: config.y_col.colindex(),
      x_axis_label_off: (x_axis_label.value() === 'false'),
      y_axis_label_off: (y_axis_label.value() === 'false'),
      y_formatter: config.y_col.coldisplayformat(),
    });

    config.margin = r.margin;
    config.width = r.width;
    config.height = r.height;
    return scatter;
  }

  function populateCellConfig(config, cellConfig) {
    let {
      cell_sel,
      rows,
      xmeasure,
      ymeasure,
      viz_settings,
      cellIdx,
    } = cellConfig;

    let transition_val,
        x_scale,
        y_scale,
        x_idx,
        y_idx,
        x_colname,
        y_colname;

    // Sort rows by size so smallest circles are on top if there is overlap
    if (config.size_idx !== -1) {
      rows.sort(function (a, b) {
        return b[config.size_idx] - a[config.size_idx];
      });
    }

    if (xmeasure) {
      config.x_agg_cols.forEach(function(col, i) {
        if (col.colname() === xmeasure) {
          x_scale = config.x_scales[i];
          x_idx = col.colindex();
          x_colname = config.columns[x_idx].colname();
        }
      });
    } else {
      x_scale = config.x_scales[0];
      x_idx = config.x_idx;
      x_colname = config.columns[config.x_orig_idx].colname();
    }

    if (ymeasure) {
      config.y_agg_cols.forEach(function(col, i) {
        if (col.colname() === ymeasure) {
          y_scale = config.y_scales[i];
          y_idx = col.colindex();
          y_colname = config.columns[y_idx].colname();
        }
      });
    } else {
      y_scale = config.y_scales[0];
      y_idx = config.y_idx;
      y_colname = config.columns[config.y_orig_idx].colname();
    }

    //note: config.x_is_date is passed into col display format because for dates we don't want
    // any default formatting if nothing has been explicitly set.
    var x_axis = arc.visual.axis()
                           .tickFormat(config.x_col.coldisplayformat(config.x_is_date))
                           .scale(x_scale)
                           .is_dim(config.x_is_dim && !config.xIsNumeric)
                           .is_date(config.x_is_date)
                           .is_log10(config.x_is_log10)
                           .colname(x_colname)
                           .coldesc(config.x_col.coldesc() || x_colname);
    var y_axis = arc.visual.axis()
                           .tickFormat(config.y_col.coldisplayformat(config.x_is_date))
                           .scale(y_scale)
                           .is_dim(config.y_is_dim && !config.yIsNumeric)
                           .is_date(config.y_is_date)
                           .is_log10(config.y_is_log10)
                           .colname(y_colname)
                           .coldesc(config.y_col.coldesc() || y_colname);

    cellConfig.width = config.width + config.margin.left + config.margin.right;
    cellConfig.height = config.height + config.margin.top + config.margin.bottom;
    cellConfig.x_values = [];
    cellConfig.y_values = [];
    cellConfig.color_values = [];

    config.x_dim_offset = 0;
    config.y_dim_offset = 0;
    if (config.x_is_dim && !config.x_is_date && !config.xIsNumeric) {
      config.x_dim_offset = x_scale.rangeBand() / 2;
    }

    if (config.y_is_dim && !config.y_is_date && !config.yIsNumeric) {
      config.y_dim_offset = y_scale.rangeBand() / 2;
    }

    cellConfig.x_axis_data = x_axis;
    cellConfig.y_axis_data = y_axis;
    cellConfig.include_gridlines =
      horizontal_gridlines_enabled.isChecked || vertical_gridlines_enabled.isChecked;
    cellConfig.x_is_dim = config.x_is_dim && !config.x_is_date;
    cellConfig.y_is_dim = config.y_is_dim && !config.y_is_date;
    cellConfig.x_axis_label_off = (x_axis_label.value() === 'false');
    cellConfig.y_axis_label_off = (y_axis_label.value() === 'false');
    cellConfig.x_scale = x_scale;
    cellConfig.y_scale = y_scale;
    cellConfig.x_idx = x_idx;
    cellConfig.y_idx = y_idx;
    cellConfig.rows = rows;
    cellConfig.transition_val = transition_val;
    cellConfig.populated = true;
  }

  function getDimGetterFunction(config) {
    var transitionColIndex;

    if (config.transition && config.transition.col) {
      transitionColIndex = config.transition.col.colindex();
    }

    let dims = config.columns.filter(function(col, i) {
      if (col.colindex() === transitionColIndex) {
        return false;
      }

      return (col.colproperty() === arc.report.columnTypeEnum.DIMENSION);
    });

    let dimIndices = dims.map(function(dim) {
      return dim.colindex();
    });

    function dims_getter(row) {
      return dimIndices.map(function(i) {
        return row[i];
      });
    }

    return dims_getter;
  }

  function drawBaseVisualCanvas(config, cellConfig) {
    let {
      cell_sel,
      rows,
      viz_settings,
      cellIdx,
    } = cellConfig;

    let transition_idx = config.transition.curr_idx;

    if (!cellConfig.populated) {
      populateCellConfig(config, cellConfig);
    }

    if (transition_idx !== undefined) {
      let transition_val = config.transition.values[transition_idx];
      config.transition.display.text(transition_val);
      cellConfig.transition_val = transition_val;
    }

    let previousComputedData = previousDataTracker.getData(cellIdx);
    computeData(config, cellConfig);

    let dims_getter = getDimGetterFunction(config);

    let newData = rows.map(function(row) {
      let d;

      if (row.newValues) {
        d = {
          cx: row.newValues.cx,
          cy: row.newValues.cy,
          radius: row.newValues.radius,
          color: row.newValues.color,
          colorLabel: row.newValues.colorLabel,
        };
      } else {
        d = {
          cx: row.cx,
          cy: row.cy,
          radius: row.radius,
          color: row.color,
          colorLabel: row.colorLabel,
        };
      }

      d.row = row;
      d.dim = dims_getter(row);
      return d;
    });

    previousDataTracker.setData(newData, cellIdx);

    let oldData,
        duration = 250;

    if ((transition_idx === undefined) || config.initialSketch) {
      // First time (and possibly the only time)
      cellConfig.margin = config.margin;
      arc.canvas.createContainer(cell_sel, cellConfig);

      arc.canvas.createAllCanvas(cellConfig);
      drawAxesTrendlines(config, cellConfig);

      if (!previousComputedData) {
        if (!previousDataTracker.enabled()) {
          arc.canvas.drawAllCanvas(config, cellConfig, drawCanvas);
          setupTooltips(config, cellConfig);
          return;
        }

        // generate it
        oldData = newData.map(function(row) {
          return {
            cx: row.cx,
            cy: row.cy,

            // Note that we are starting with a radius of 0
            // to show a transition
            radius: 0,
            color: row.color,
            colorLabel: row.colorLabel,
            dim: dims_getter(row.row),
          };
        });
      } else {
        oldData = previousComputedData;
      }

      let oldDataReordered = [];

      {
        let groupToIdx = {};
        newData.forEach(function(d, i) {
          groupToIdx[d.dim] = i;
        });

        oldData.forEach(function(d) {
          let newIdx = groupToIdx[d.dim];
          if (newIdx !== undefined) {
            oldDataReordered[newIdx] = d;
          }
        });
      }

      oldData = oldDataReordered;
    } else {
      oldData = rows.map(function(row) {
        return row.oldValues;
      });

      duration = config.transition.delay;
    }

    let transition = arc.transition.transition()
                        .visualId(lib.report_id())
                        .select(`#${lib.chart_div()}`)
                        .transitionName(`cell-${cellIdx}`)
                        .duration(duration)
                        .profileData([
                          [
                            'rows',
                            newData.length,
                          ],
                          [
                            'visual',
                            lib.viz_type(),
                          ],
                        ]);

    cellConfig.transition = transition;

    function setRowsData(rowsData) {
      rowsData.forEach(function(rowData, i) {
        rows[i].cx = rowData.cx;
        rows[i].cy = rowData.cy;
        rows[i].radius = rowData.radius;
        rows[i].color = rowData.color;
        rows[i].colorLabel = rowData.colorLabel;
      });
    }

    setRowsData(oldData);
    arc.canvas.drawAllCanvas(config, cellConfig, drawCanvas, /* inTransition */ true);

    let interpolation = d4.interpolate(oldData, newData);

    transition
      .drawFn(function(t) {
        let inTransition = true,
            rowsData;

        if (t >= 1) {
          inTransition = false;
          rowsData = newData;
        } else {
          rowsData = interpolation(t);
        }

        setRowsData(rowsData);
        arc.canvas.drawAllCanvas(config, cellConfig, drawCanvas, inTransition);
        if (!inTransition) {
          setupTooltips(config, cellConfig);
        }
      })
      .start();
  }

  function setupTooltips(config, cellConfig) {
    let {
      hiddenCanvasConfig,
      highlightCanvasConfig,
      overlayContainer,
    } = cellConfig;

    function text_fn(d) {
      if (config.transition.col) {
        d = d.currentRow;
      }

      return lib.text_from_columns(d, config.columns);
    }

    function params_fn(d, dimsOnly) {
      if (config.transition.col) {
        d = d.currentRow;
      }

      return lib.params_from_columns(d, config.columns, dimsOnly);
    }

    function highlight_fn(row) {
      if (highlightCanvasConfig.drawRows) {
        const rows = row ? [row] : [];
        highlightCanvasConfig.drawRows(rows);
      }
    }

    if (cellConfig.marks) {
      cellConfig.marks.destroy();
      cellConfig.marks = undefined;
    }

    cellConfig.marks =
      arc.canvas.handleTooltips({
        mouseoverCanvas: overlayContainer,
        hiddenCanvas: hiddenCanvasConfig.canvas,
        hiddenColorMap: hiddenCanvasConfig.colorMap,
        viz_lib: lib,
        text_fn,
        params_fn,
        highlight_fn,
      });
  }

  function computeData(config, cellConfig) {
    let {
      transition_val,
      x_values,
      y_values,
      color_values,
      x_scale,
      y_scale,
      x_idx,
      y_idx,
      rows,
    } = cellConfig;

    rows.forEach(function(row) {
      let d = row;
      if (transition_val) {
        d = d.groups[transition_val];
      }

      if (!config.transition.col || config.initialSketch) {
        if (d === undefined) {
          row.currentRow = row;
          row.cx = 0;
          row.cy = 0;
          row.radius = 0;
          row.color = 'none';
          return;
        }

        row.currentRow = d;
        row.cx = x_scale(d[x_idx]) + config.x_dim_offset;
        row.cy = y_scale(d[y_idx]) + config.y_dim_offset;
        row.radius = config.size_scale(
            Math.sqrt(d[config.size_idx]/Math.PI));

        let v = d[config.color_idx];
        if (!config.color_is_dim) {
          v = Number(v);
        }

        row.color = config.color(v);
        row.colorLabel = v;

        if (config.trendlinesEnabled) {
          x_values.push(x_scale(d[x_idx]));
          y_values.push(y_scale(d[y_idx]));
          color_values.push(v);
        }

        return;
      }

      // Here, we know we are in the midst of a transition. Store oldValues
      // and newValues per row, and then we will transition in the transition
      row.oldValues = {
        cx: row.cx,
        cy: row.cy,
        radius: row.radius,
        color: row.color,
        colorLabel: row.colobLabel,
      };

      if (d === undefined) {
        row.newValues = row.oldValues;
        return;
      }

      row.currentRow = d;
      let v = d[config.color_idx];
      if (!config.color_is_dim) {
        v = Number(v);
      }

      row.newValues = {
        cx: x_scale(d[x_idx]) + config.x_dim_offset,
        cy: y_scale(d[y_idx]) + config.y_dim_offset,
        radius: config.size_scale(
            Math.sqrt(d[config.size_idx]/Math.PI)),
        color: config.color(v),
        colorLabel: v,
      };

    });
  }

  function drawAxesTrendlines(config, cellConfig) {
    let {
      overlayContainer,
      width,
      height,
      x_axis_data,
      y_axis_data,
      include_gridlines,
      x_is_dim,
      y_is_dim,
      x_axis_label_off,
      y_axis_label_off,
      x_values,
      y_values,
      color_values,
    } = cellConfig;

    let {
      x_is_slanted,
    } = config;

    let axisSvg = arc.canvas.axisOverlay({
      overlayContainer,
      width,
      height,
      margin: config.margin,
    });

    let gridlineSVG;
    if (include_gridlines) {
      gridlineSVG = overlayContainer.insert('svg', ':first-child')
                              .style('position', 'absolute')
                              .style('left', '0px')
                              .style('top', '0px')
                              .classed('viz-gridline-svg', true);
    }

    lib.draw_axes(axisSvg, config.width, config.height, config.margin,
                  {
                    x_axis_data,
                    y_axis_data,
                    gridlineSVG,
                    drawVerticalGridLine: vertical_gridlines_enabled.isChecked,
                    drawHorizontalGridLine: horizontal_gridlines_enabled.isChecked,
                    x_is_dim,
                    y_is_dim,
                    x_axis_label_off,
                    y_axis_label_off,
                    x_is_slanted,
                  });

    if (config.trendlinesEnabled) {
      var xy_values = [];
      x_values.forEach(function(d, i) {
        xy_values.push([x_values[i], y_values[i], color_values[i]]);
      });

      lib.draw_trend_lines(axisSvg, config, xy_values,
          trendColor.value(), trendDecimal.value());
    }
  }

  function drawCanvas(config, cellConfig, canvasConfig) {
    let {
      width,
      height,
      rows,
    } = cellConfig;

    let {
      canvas,
      hidden,
      highlight,
    } = canvasConfig;

    let colorMap = {};
    let context = canvas.node().getContext('2d');

    let font = lib.getVisualFont();
    const fontSize = lib.getLabelFontSize() || 12;
    context.font = `${fontSize}px ${font}`;
    context.textBaseline = 'middle';
    context.textAlign = 'center';

    const opacity = lib.colorOpacity();

    if (!hidden && !highlight) {
      // Note that we do NOT set the globalAlpha for the hidden
      // canvas: otherwise, we end up with a different color map
      // from the one we expect when we read back colors.
      context.globalAlpha = opacity;
    }

    if (highlight) {
      context.lineWidth = arc.canvas.highlightLineWidth;
      context.strokeStyle = arc.canvas.highlightStrokeStyle;
    }

    if (!highlight) {
      drawRows(rows);
    }

    function drawLabel(row) {
      const text = row.colorLabel;
      const textColor = arc.colorpalette.colorForBrightness(row.color, opacity);
      context.fillStyle = textColor;

      const w = context.measureText(text).width;
      if ((w + 3) < row.radius * 2) {
        context.fillText(text, row.cx, row.cy);
        return true;
      }

      return false;
    }

    function drawRows(rows) {
      context.clearRect(-config.margin.left, -config.margin.top, width, height);

      rows.forEach(function(row) {
        context.beginPath();

        let color;
        if (hidden) {
          color = colorFetcher();
          colorMap[color] = row;
        } else {
          color = row.color;
        }

        context.arc(row.cx, row.cy, row.radius, 0, 2 * Math.PI);

        if (highlight) {
          context.stroke();
        }

        context.fillStyle = color;
        context.fill();

        context.closePath();

        if (!hidden && labelByColor.isChecked) {
          drawLabel(row);
        }
      });
    }

    // TODO: viz_settings.show_cb
    // TODO: viz_settings.hide_cb

    canvasConfig.drawRows = drawRows;
    canvasConfig.colorMap = colorMap;
  }

  function setupTransitions(config) {
    if (!config.transition.col) {
      return scatter;
    }

    let transColIdx = config.transition.col.colindex();
    const isNotTransXOrYDim = (col) => col.colproperty() === arc.report.columnTypeEnum.DIMENSION &&
      col.colindex() !== transColIdx && col.colindex() !== config.x_col.colindex() &&
      col.colindex() !== config.y_col.colindex();

    const transitionGetter = (row) => row[transColIdx];
    const hasNonXYTransDims = config.columns.some(isNotTransXOrYDim);

    let isSavedDimArray;
    if (!hasNonXYTransDims) {
      isSavedDimArray = config.columns.map(col =>
      col.colproperty() === arc.report.columnTypeEnum.DIMENSION && col.colindex() !== transColIdx);
    } else {
      isSavedDimArray = config.columns.map(isNotTransXOrYDim);
    }

    function dims_getter(row) {
      var i;
      var d = [];
      for (i = 0; i < config.columns.length; ++i) {
        if (isSavedDimArray[i]) {
          d.push(row[i]);
          break;
        }
      }

      return d;
    }

    config.transition.values =
        arc.misc.arrayUnique(config.rows, transitionGetter)
                .sort();

    // Group rows by transitions
    var out_rows = [];
    var out_rows_hash = {};
    config.rows.forEach(function(row) {
      var t = transitionGetter(row);
      var d = dims_getter(row);
      if (!(d in out_rows_hash)) {
        out_rows_hash[d] = row;
        out_rows.push(row);
        row.groups = {};
        row.groups[t] = row;
      } else {
        out_rows_hash[d].groups[t] = row;
      }
    });

    config.rows = out_rows;
    return scatter;
  }

  function setupTrendlines(config) {
    if (!config.transition.col && trendlineLinear.isChecked) {
      config.trendlinesEnabled = true;
    }
  }

  function setupColumns(config) {
    // First, figure out the columns that refer to scatter plot properties
    config.x_idx = -1;
    config.x_orig_idx = -1;
    config.y_idx = -1;
    config.y_orig_idx = -1;
    config.size_idx = -1;
    config.transition.col = undefined;
    config.columns = data_result.columns();

    config.columns.forEach(function(col, colindex) {
      if (col.colsource() === 'transition_shelf') {
        config.transition.col = col;
        return;
      }

      if (col.colsource() === 'x_shelf') {
        config.x_idx = colindex;
        config.x_orig_idx = colindex;
        config.x_col = col;
        return;
      }

      if (col.colsource() === 'y_shelf') {
        config.y_idx = colindex;
        config.y_orig_idx = colindex;
        config.y_col = col;
        return;
      }

      if (col.colsource() === 'size_shelf') {
        config.size_idx = colindex;
        return;
      }

      if (col.colsource() === 'color_shelf') {
        return;
      }

      if (col.colsource() === 'tooltip_shelf') {
        return;
      }

      console.log('Unknown shelf', col.colsource());
    });

    config.x_getter = function(row) {
      return row[config.x_idx];
    };

    config.y_getter = function(row) {
      return row[config.y_idx];
    };

    // Filter out nulls
    lib.filter_nulls(config, [config.x_getter, config.y_getter]);

    var ret = lib.idx_is_date(config.rows, config.x_idx);
    config.x_is_date = ret.is_date;
    config.x_is_datetime = ret.is_datetime;
    if (config.x_is_date) {
      config.date_format = ret.dateFormat;
    }

    ret = lib.idx_is_date(config.rows, config.y_idx);
    config.y_is_date = ret.is_date;
    config.y_is_datetime = ret.is_datetime;
    if (config.y_is_date) {
      config.date_format = ret.dateFormat;
    }

    if (config.x_is_date) {
      lib.compute_dates(config.rows, config.date_format, 'x_date',
          config.x_getter);
      config.x_idx = 'x_date';
      config.x_is_dim = true;
    } else {
      config.x_is_dim = config.x_col.colproperty() === arc.report.columnTypeEnum.DIMENSION;
      config.x_is_slanted = config.x_is_dim;
    }

    if (config.y_is_date) {
      lib.compute_dates(config.rows, config.date_format, 'y_date',
          config.y_getter);
      config.y_idx = 'y_date';
      config.y_is_dim = true;
    } else {
      config.y_is_dim = config.y_col.colproperty() === arc.report.columnTypeEnum.DIMENSION;
    }

    if (config.x_is_date && config.y_is_date) {
      let both_getter = function(row) {
        return [row[config.x_idx], row[config.y_idx]];
      };

      config.rows.sort(function(a, b) {
        return arc.misc.sortNumeric(both_getter(a), both_getter(b));
      });
    } else {
      if (config.x_is_date) {
        config.rows.sort(function(a, b) {
          return arc.misc.sortNumeric(config.x_getter(a), config.x_getter(b));
        });
      }
      if (config.y_is_date) {
        config.rows.sort(function(a, b) {
          return arc.misc.sortNumeric(config.y_getter(a), config.y_getter(b));
        });
      }
    }

    config.x_agg_cols = lib.last_agg_columns(config.columns, 'x_shelf');
    if (config.x_agg_cols.length > 1) {
      config.x_measure_values =
          arc.data.column()
                  .colname('Measure')
                  .colproperty(arc.report.columnTypeEnum.DIMENSION)
                  .coltype('string')
                  .colvalues(config.x_agg_cols.map(function(d) {
                    return d.colname();
                  }));
    }
    config.y_agg_cols = lib.last_agg_columns(config.columns, 'y_shelf');
    if (config.y_agg_cols.length > 1) {
      config.y_measure_values =
          arc.data.column()
                  .colname('Measure')
                  .colproperty(arc.report.columnTypeEnum.DIMENSION)
                  .coltype('string')
                  .colvalues(config.y_agg_cols.map(function(d) {
                    return d.colname();
                  }));
    }

    return scatter;
  }

  function computeScales(config, rows, x_or_y) {
    // Define all property names:
    // y_scales/x_scales, y_agg_cols/x_agg_cols, y_idx/x_idx
    var scales = x_or_y + '_scales',
        agg_cols = x_or_y + '_agg_cols',
        idx = x_or_y + '_idx';

    // Compute scale, but only for x or y depending on what is passed in
    config[scales] = [];
    if (config[agg_cols].length > 0) {
      config[agg_cols].forEach(function(col) {
        computeScale(rows, col.colindex(), config, x_or_y);
      });
    } else {
      computeScale(rows, config[idx], config, x_or_y);
    }

    // Create size scale
    if (config.size_idx < 0) {
      config.size_scale = function() {
        return mark_size_range.get_range()[0];
      };
    } else {
      var extent = d4.extent(config.orig_rows, function(row) {
        return Math.sqrt((Number(row[config.size_idx]))/Math.PI);
      });
      config.size_scale = d4.scaleLinear()
                            .domain(extent)
                            .range(mark_size_range.get_range());
    }

    return scatter;
  }

  function getEdgeVal(xOrY, maxOrMin) {
    const minX = minXScale.value();
    const maxX = maxXScale.value();
    const minY = minYScale.value();
    const maxY = maxYScale.value();
    const { yIsNumeric, xIsNumeric } = config;
    function getVal(val) {
      if (xOrY === 'x') {
        if (maxOrMin === 'max') {
          if ((arc.misc.isNumeric(maxX)) && xIsNumeric) {
            return Math.max(val, Number(maxX));
          }

          return val;
        }

        if ((arc.misc.isNumeric(minX)) && xIsNumeric) {
          return Math.min(val, Number(minX));
        }

        return val;
      }

      if (maxOrMin === 'max') {
        if ((arc.misc.isNumeric(maxY)) && yIsNumeric) {
          return Math.max(val, Number(maxY));
        }

        return val;
      }

      if ((arc.misc.isNumeric(minY)) && yIsNumeric) {
        return Math.min(val, Number(minY));
      }

      return val;
    }

    return getVal;
  }

  function computeScale(rows, row_idx, config, x_or_y) {
    // Define all property names:
    // y_scales/x_scales, y_idx/x_idx,
    // y_is_dim/x_is_dim, y_is_log10/x_is_log10, y_is_date/x_is_date
    // width/height
    var scales = x_or_y + '_scales',
        idx = x_or_y + '_idx',
        is_dim = x_or_y + '_is_dim',
        is_log10 = x_or_y + '_is_log10',
        is_date = x_or_y + '_is_date',
        is_datetime = x_or_y + '_is_datetime',
        dimension = x_or_y === 'x' ? 'width' : 'height',
        range = x_or_y === 'x' ? [0, config[dimension]] : [config[dimension], 0];

    var axisPadding;
    if (x_or_y === 'x') {
      axisPadding = Number(x_axis_padding.value());
    } else {
      axisPadding = Number(y_axis_padding.value());
    }

    var all_rows = rows;
    if (rows.length > 0) {
      all_rows = rows.slice();
      rows.forEach(function(row) {
        if (row.groups) {
          $.each(row.groups, function(k, v) {
            all_rows.push(v);
          });
        }
      });
    }

    let isNumeric;
    if (x_or_y === 'x') {
      isNumeric = !config[is_dim] || (!!config.x_col.isNumeric() && continuousX.isChecked);
    } else {
      isNumeric = !config[is_dim] || (!!config.y_col.isNumeric() && continuousY.isChecked);
    }

    if (x_or_y === 'x') {
      config.xIsNumeric = isNumeric;
      if (isNumeric && !config[is_log10]) {
        minXScale.enable();
        maxXScale.enable();
      } else {
        minXScale.disable();
        maxXScale.disable();
      }
    } else {
      config.yIsNumeric = isNumeric;
      if (isNumeric && !config[is_log10]) {
        minYScale.enable();
        maxYScale.enable();
      } else {
        minYScale.disable();
        maxYScale.disable();
      }
    }

    var scale = lib.idx_to_scale_config({
      rows: all_rows,
      idx: row_idx,
      range: range,
      is_date: config[is_date],
      is_datetime: config[is_datetime],
      is_dim: config[is_dim],
      is_log10scale: config[is_log10],
      minScale: getEdgeVal(x_or_y, 'min'),
      maxScale: getEdgeVal(x_or_y, 'max'),
      sort_dim: true,
      paddingPct: axisPadding,
      isNumericContinuous: isNumeric,
    });

    var d, m;
    if (config[is_log10]) {
      d = scale.domain();
      // Only add padding if needed
      var exp = Math.round(Math.abs((axisPadding / 100) * (Math.log10(d[1]) - Math.log10(d[0]))));
      var change = Math.pow(10, Math.max(exp, 0));
      d[0] /= change;
      d[1] *= change;
      scale.domain(d);
    } else if (!config[is_dim] || isNumeric) {
      d = scale.domain();
      m = Math.abs((axisPadding / 100) * (d[1] - d[0]));
      d[0] -= m;
      d[1] += m;
      scale.domain(d);
    }

    if ((reverse_scale_y.isChecked && x_or_y === 'y') ||
      (reverse_scale_x.isChecked && x_or_y === 'x')) {
      let range = scale.range().reverse();
      scale.range(range);
    }

    config[scales].push(scale);
  }

  function setupXY(config) {
    setupXorY(config, 'x', x_setting_scale);
    setupXorY(config, 'y', y_setting_scale);

    return scatter;
  }

  function setupXorY(config, x_or_y, setting_scale) {
    // Define all property names:
    // y_shelf/x_shelf, y_start_idx/x_start_idx,
    // y_is_log10/x_is_log10, y_getter/x_getter

    var shelf = x_or_y + "_shelf",
        start_idx = x_or_y + '_start_idx',
        is_log10 = x_or_y + '_is_log10',
        getter = x_or_y + '_getter';

    var col = lib.last_column(config.columns, shelf);
    if (!col) {
      return;
    }
    config[start_idx] = col.colindex();
    config[is_log10] = false;
    // Taking a guess at all the number column_types
    // (see arc.report.column_types and columnstats.fetch_detail)
    if (setting_scale.value() === 'LOG10' && arc.report.coltypeIsNumber(col.coltype())) {
      config[is_log10] = true;
    }

    if (config[is_log10]) {
      var a_min = d4.min(config.orig_rows, config[getter]);
      if (a_min <= 0) {
        lib.alertMessage("Axis Scale is set to Log 10, but data " +
                        "contains values <= 0 which is not valid for " +
                        "log scale. Please change the Axis Scale " +
                        "in Settings for this app.");
      }
    }
  }

  return scatter;
};

arc.visual.plugins.scatter_extension = function() {
  var ext = function() { return "arc.visual.plugins.scatter_extension"; };

  ext.viz_name = function() {
    return "scatter";
  };

  ext.viz_tooltip = function() {
    return "scatter";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_scatter.png?v=1";
  };

  ext.viz = arc.visual.plugins.scatter;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.scatter_extension());

}());
