window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.search = arc.search || {};
arc.search.utils = arc.search.utils || {};

arc.search.utils.forgetAllDatasets = function() {
  arc.staticfetcher.forget([
    {
      asset: `${arc.frontend.globals.urls.datasetUrl}?search=true`,
    },
  ]);
};

arc.search.utils.fetchAllDatasets = function(successCb, errorCb) {
  const assets = [
    {
      asset: `${arc.frontend.globals.urls.datasetUrl}?search=true`,
      postprocess: d3.json,
    },
  ];

  arc.staticfetcher.fetch(assets,
    function(error, datasetsData) {

      if (error) {
        errorCb(error);
        return;
      }

      let datasets = [];
      datasetsData.forEach(function(d) {
        const dataset = arc.datasets.Dataset()
                           .dataset_id(d.id)
                           .init_from_fetch(d);
        datasets.push(dataset);
      });

      successCb(datasets);
    });
};

arc.visual.plugins.search = function() {
  let lib,
      allSearchDatasets = [],
      searchText,
      datasets,
      inputWrapper,
      datasetSelType = 'this',
      datasetSelections;

  const userSettings = arc.visual.defaultSettings({ noTrellis: true });

  var search = arc.visual.api();

  search.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data([])
       .user_settings(userSettings);
    return search;
  };

  let saved_settings;

  search.searchText = function(_) {
    if (!arguments.length) return searchText;
    searchText = _;
    return search;
  };

  search.datasets = function(_) {
    if (!arguments.length) return datasetSelections;
    datasetSelType = 'selected';
    datasetSelections = _;
    return search;
  };

  search.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.viz_type = settings.viz_type || {};
    settings.viz_type.search = settings.viz_type.search || {};
    if (arc.globals.is_edit) {
      settings.viz_type.search.searchText = inputWrapper ? inputWrapper.input().text() : searchText;
      settings.viz_type.search.datasetSelType = datasetSelType;
      settings.viz_type.search.datasetSelections = datasetSelections;
    }

    if (searchText !== settings.viz_type.search.searchText) {
      searchText = settings.viz_type.search.searchText;
    }

    saved_settings = settings;
    return settings;
  };

  search.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return search;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if (settings && settings.viz_type && settings.viz_type.search) {
      searchText = settings.viz_type.search.searchText;
      datasetSelType = settings.viz_type.search.datasetSelType || 'this';
      datasetSelections = settings.viz_type.search.datasetSelections || [];
    }

    return search;
  };

  let shelvesDiv;

  search.viz_drawDesigner = function(_shelvesDiv) {
    shelvesDiv = _shelvesDiv;
    if (!shelvesDiv) {
      return search;
    }

    const editMode = lib.build_mode();
    if (editMode) {
      drawDesigner();
      return search;
    }

    return search;
  };

  function drawDesigner() {
    if (!shelvesDiv) {
      return;
    }

    $('#' + shelvesDiv).html($('#viz-searchbuilder').html());
  }

  search.viz_getDataRequest = function() {
    var ignore = arc.data.request()
                    .request_type('ignore');
    return ignore;
  };

  search.viz_setDataResult = function(result) {
    return search;
  };

  function searchCb(text, activeDataset) {
    let searchDatasets = activeDataset ? [activeDataset] : datasets;

    if (searchText !== text) {
      arc.search.displayedQuery()
                .search_text(text)
                .store();
    }

    searchText = text;
    arc.search.results()
       .searchVisualLib(lib)
       .container_id(lib.chart_div())
       .datasets(searchDatasets)
       .search(text);
  }

  search.viz_fetchCollateral = function(viz_div, isWindowResize, drawCb) {
    if (datasetSelType === 'all') {
      arc.search.utils.fetchAllDatasets(
          function(in_datasets) {
            datasets = in_datasets.slice(0);
            allSearchDatasets = in_datasets.slice(0);
            drawCb(viz_div, isWindowResize);
          },
          function(error) {
            arc.report.alertMessage('Failed to load datasets', error);
            drawCb(viz_div, isWindowResize);
            return;
          });

      return search;
    }

    if (((allSearchDatasets === undefined) || (allSearchDatasets.length === 0)) &&
        arc.globals.is_edit &&
        !lib.inModal()) {
      /*
       * Fill in allSearchDatasets so we can show the user all datasets to pick from
       * when he is setting up search
       */
      arc.search.utils.fetchAllDatasets(
          function(in_datasets) {
            allSearchDatasets = in_datasets.slice(0);
            if (allSearchDatasets.length === 0) {
              return;
            }

            /*
             * Now, go fetch the datasets necessary for this visual. Note that
             * we are calling this function again but with allSearchDatasets set to
             * something other than undefined, which means we will not be in this
             * codepath again.
             */
            search.viz_fetchCollateral(viz_div, isWindowResize, drawCb);
          },
          function(error) {
            arc.report.alertMessage('Failed to load datasets', error);
            return;
          });

      return search;
    }

    if (datasetSelType === 'selected') {
      const assets = [];
      if (!datasetSelections) {
        datasetSelections = [];
      }

      datasetSelections.forEach(function(sel) {
        assets.push({
          asset: `${arc.frontend.globals.urls.datasetUrl}/${sel}?search=true`,
          postprocess: d3.json,
        });
      });

      arc.staticfetcher.fetch(assets,
        function(error) {
          if (error) {
            arc.report.alertMessage('Failed to load datasets', error);
            drawCb(viz_div, isWindowResize);
            return;
          }

          datasets = [];
          for (let i = 1; i < arguments.length; ++i) {
            const d = arguments[i][0];
            var dataset = arc.datasets.Dataset()
                             .dataset_id(d.id)
                             .init_from_fetch(d, undefined);

            datasets.push(dataset);
          }

          drawCb(viz_div, isWindowResize);
        });

      return search;
    }

    if (datasetSelType === 'this') {
      datasets = [lib.dataset()];
      drawCb(viz_div, isWindowResize);
      return search;
    }

    return search;
  };

  function drawVisual(viz_div) {
    const $viz = $(`#${viz_div}`);
    lib.removeMessageAboutMissingData($viz.closest('.arc-viz-container'));

    const $container = $viz.closest('.arc-viz-container');
    if (arc.globals.is_edit && !lib.inModal()) {
      $viz.append($('#viz-search-edit-template').html());
      $viz.find('.search-dataset-type').val(datasetSelType);

      $viz.find('.search-dataset-type').chosen({
        disable_search_threshold: 10,
      });

      let $datasetSel = $viz.find('.search-dataset-selections');
      $datasetSel.empty();

      let template = $('#search-dataset-template select').html();

      allSearchDatasets.forEach(function(dataset) {
        $datasetSel.append(arc.misc.safe_substitute(template, {
          LABEL: dataset.dataset_name(),
          VALUE: dataset.dataset_id(),
        }));
      });

      if (datasetSelections) {
        datasetSelections.forEach(function(selection) {
          $datasetSel.find(`option[value="${selection}"]`).prop("selected", true);
        });
      }

      $datasetSel.chosen({
        disable_search_threshold: 10,
      });
    }

    $viz.append($('#viz-search-content-template').html());
  }

  function setupVisual(viz_div) {
    const $viz = $(`#${viz_div}`);

    if (datasetSelType === 'selected') {
      $viz.find('.search-datasets-holder')
          .css('display', 'block');
    }

    if (arc.globals.is_edit && !lib.inModal()) {
      $viz.find('.search-dataset-type').offOn('change.type', function(e) {
            let $currentTarget = $(e.currentTarget);
            lib.page_needs_save();

            datasetSelType = $currentTarget.val();
            search.viz_fetchCollateral(viz_div, false, () => {
              search.viz_drawVisual(viz_div);
            });
          });

      $viz.find('.search-dataset-selections').offOn('change.type', function() {
            datasetSelections = $(this).val();
            search.viz_fetchCollateral(viz_div, false, () => {
              search.viz_drawVisual(viz_div);
            });
            lib.page_needs_save();
          });

      $viz.find('.pop-search-box').offOn('input.triggerSave', function() {
            lib.page_needs_save();
          });

      $viz.offOn('click.triggerNeedSave', '.pop-search-list > li > a', function() {
            lib.page_needs_save();
          });
    }

    $viz.offOn('click.ignore', '.search-result', function(e) {
          e.stopPropagation();
        })
        .offOn('dblclick.ignore', '.search-result', function(e) {
          e.stopPropagation();
        });

    if (arc.globals.dashboard) {
      const gridster = arc.globals.dashboard.gridster();

      $viz.offOn('click.searchItem', '.pop-search-box', function() {
               gridster.disable().disable_resize();
               $(this).attr('contenteditable', 'true');
               $(this).focus();
             });
    }
  }

  search.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    if (!datasets) {
      datasets = [lib.dataset()];
    }

    const $viz = $(`#${viz_div}`);
    drawVisual(viz_div);
    setupVisual(viz_div);

    inputWrapper = arc.search.inputWrapper()
                             .autoHighlights(false)
                             .container(`#${viz_div}`)
                             .datasets(datasets);

    inputWrapper.searchCb(searchCb)
                .setup()
                .shown();

    if (searchText) {
      inputWrapper.runSearch(searchText);
    }

    $viz.offOn('click.enterSearchValue', '.redo-title-search', (e) => {
      const $this = $(e.target);
      const $result = $this.closest('.search-result-header').find('.search-text');
      const newSearchText = $result.text().replace(/\s+/g, ' ').trim();
      inputWrapper.text(newSearchText);

      if (newSearchText) {
        inputWrapper.runSearch(newSearchText);
      }
    });

    return search;
  };

  return search;
};

arc.visual.plugins.search_extension = function() {
  var ext = function() { return 'arc.visual.plugins.search_extension'; };

  ext.viz_name = function() {
    return 'search';
  };

  ext.viz_tooltip = function() {
    return 'search';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_search.png?v=1';
  };

  ext.viz = arc.visual.plugins.search;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.search_extension());

}());
