window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.sparklines = function() {
  var sort_orders_asc = {},
      saved_settings = {},
      data_result,
      lib,
      trellis,
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;
  const userSettings = arc.visual.defaultSettings({ noAxes: true });

  const independentAggScale = arc.report.user_settings.scale_agg_independent();
  independentAggScale.value('true')
                     .valueBeforeRestore('false');

  const tooltipMinSetting = arc.settings.sparklines.tooltipMin();
  const tooltipMaxSetting = arc.settings.sparklines.tooltipMax();
  const tooltipFirstSetting = arc.settings.sparklines.tooltipFirst();
  const tooltipLastSetting = arc.settings.sparklines.tooltipLast();
  const tooltipSetting = userSettings.find_by_id('show_tooltip');
  tooltipSetting.addDependents([
    tooltipMinSetting,
    tooltipMaxSetting,
    tooltipFirstSetting,
    tooltipLastSetting,
  ]);

   userSettings.addSetting([
     arc.colorpalette.setting(),
     arc.report.user_settings.color_opacity(),
     tooltipMinSetting,
     tooltipMaxSetting,
     tooltipFirstSetting,
     tooltipLastSetting,
     independentAggScale,
   ])
   .removeSetting([
     'visual_show_detail_data',
   ]);

  function tooltipMin() {
    return userSettings.find_by_id('tooltip_min').isChecked;
  }

  function tooltipMax() {
    return userSettings.find_by_id('tooltip_max').isChecked;
  }

  function tooltipFirst() {
    return userSettings.find_by_id('tooltip_first').isChecked;
  }

  function tooltipLast() {
    return userSettings.find_by_id('tooltip_last').isChecked;
  }

  var sparklines = arc.visual.api();

  var shelf_data = [
    {
      name: 'x_shelf',
      display_name: 'X Axis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Axis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  var constraints = {
    minimums: {
      x_shelf: 1,
      y_shelf: 1,
    },
  };

  sparklines.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(userSettings);
    return sparklines;
  };

  sparklines.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  sparklines.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return sparklines;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return sparklines;
  };

  sparklines.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup()
       .setupSwitchXY(shelves_div);

     // Define messages to be included in shelves
     var xMessages = [
                       lib.createPopoverMessages('xAxis', 'xSpark'),
                       lib.createPopoverMessages(['trellisFirst', 'columns.']),
                      ],
         yMessages = [
                       lib.createPopoverMessages('yAxis', 'ySpark'),
                       lib.createPopoverMessages(['trellisFirst', 'rows.']),
                     ];

   // Create those popovers
   lib.addShelfDescription(constraints, {
                                           x_shelf: xMessages,
                                           y_shelf: yMessages,
                                         });
    return sparklines;
  };

  sparklines.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints, {
      constraintsCb: function(req) {
        const columns = req.columns(),
              x_is_dim = (lib.last_column(columns, 'x_shelf').colproperty() ===
                  arc.report.columnTypeEnum.DIMENSION),
              y_is_dim = (lib.last_column(columns, 'y_shelf').colproperty() ===
                  arc.report.columnTypeEnum.DIMENSION);

        if (x_is_dim && y_is_dim) {
          lib.alertMessage('Please specify at least one measure');
          return undefined;
        }

        return req;
      },
    });
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  sparklines.viz_setDataResult = function(result) {
    data_result = result;
    return sparklines;
  };

  sparklines.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    var config = {};
    config.rows = data_result.rows();
    if (config.rows.length === 0) {
      return sparklines;
    }

    // Filter out null values in aggregates
    var agg_getter = function(row) {
      return row[data_result.agg_start_idx()];
    };

    lib.filter_nulls(config, [agg_getter]);

    lib.unselectOnClick();

    config.columns = data_result.columns();
    config.x_col = lib.last_column(config.columns, 'x_shelf');
    config.y_col = lib.last_column(config.columns, 'y_shelf');

    var x_is_dim = (config.x_col.colproperty() ===
                    arc.report.columnTypeEnum.DIMENSION),
        y_is_dim = (config.y_col.colproperty() ===
                    arc.report.columnTypeEnum.DIMENSION);

    if (!x_is_dim) {
      lib.alertMessage('The last X dimension must be a dimension');
      return sparklines;
    }

    if (y_is_dim) {
      lib.alertMessage('The last Y must be an aggregate');
      return sparklines;
    }

    // Filter rows with NULLs in the aggregate columns
    lib.filter_nulls(config, [config.y_col]);

    trellis = arc.visual.trellis()
                     .x_shelf_name('x_shelf')
                     .y_shelf_name('y_shelf')
                     .x_shelf_reserved(1)
                     .y_shelf_reserved(1)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select('#' + viz_div))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       sparklines.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return sparklines;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      sparklines.viz_drawVisual(viz_div);
    });

    config.margin = {
      left: 0,
      top: 0,
      right: 0,
      bottom: 0,
    };

    config.width = 200;
    config.height = 30;
    config.w_padding = 4;
    config.h_padding = 4;

    config.dim_getter = function(row) {
      return row[config.x_col.colindex()];
    };

    config.agg_getter = function(row) {
      return Number(row[config.y_col.colindex()]);
    };

    config.x_values = arc.misc.arrayUnique(config.rows, config.dim_getter);

    // Throw an error if the user tried to sort by y_value
    // Sorting by agg would make the dim different for each cell of the trellis
    config.trellisSortByY = config.y_col.order() && config.trellis.is_active();
    config.sortByY =  config.y_col.order() && !config.trellis.is_active();

    // Sort each set of rows by the sorted x_values array
    if (!config.x_col.order() && !config.sortByY) {
      config.x_values.sort(arc.misc.sortNums);
    }

    config.x_values_h = arc.misc.arrayToMap(config.x_values);
    config.x_scale = d3.scale.linear()
                       .domain([0, config.x_values.length - 1])
                       .range([
                                config.w_padding / 2,
                                config.width - (config.w_padding / 2),
                              ]);

    const y_extent = d3.extent(config.rows, config.agg_getter);
    config.y_scale = d3.scale.linear()
                       .domain(y_extent)
                       .range([
                                config.height - config.h_padding / 2,
                                config.h_padding / 2,
                              ]);

    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      drawBaseVisual(config, cell_sel, xy_rows);
    });

    config.trellis.tooltips('trellis');

    return sparklines;
  };

  sparklines.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return sparklines;
  };

  sparklines.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return sparklines;
  };

  function drawBaseVisual(config, cell_sel, rows) {
    var min,
        max,
        min_dim,
        max_dim;

    // Sort each set of rows by the sorted x_values array
    if (!config.x_col.order() && !config.sortByY) {
      rows.sort(function(a, b) {
        return arc.misc.sortNums(config.dim_getter(a), config.dim_getter(b));
      });
    }

    let cellConfig = {
      width: config.width,
      height: config.height,
      margin: config.margin,
      mainCanvasOnly: true,
    };

    arc.canvas.createContainer(cell_sel, cellConfig);
    arc.canvas.createAllCanvas(cellConfig);

    let canvas = cellConfig.mainCanvasConfig.canvas,
        context = canvas.node().getContext('2d');

    let color = lib.colors()[0];

    let y_scale = config.y_scale;

    if (independentAggScale.isChecked) {
      const y_extent = d3.extent(rows, config.agg_getter);
      y_scale = d3.scale.linear()
                  .domain(y_extent)
                  .range([
                      config.height - config.h_padding / 2,
                      config.h_padding / 2,
                  ]);
    }

    context.globalAlpha = lib.colorOpacity();
    context.lineWidth = 2;
    context.strokeStyle = color;
    context.beginPath();

    rows.forEach(function(row, i) {
      var a = config.agg_getter(row),
          dim = config.dim_getter(row),
          x_idx = config.x_values_h[dim],
          x_px = config.x_scale(x_idx),
          y_px = y_scale(a);
      if (i === 0) {
        min = a;
        max = a;
        min_dim = x_idx;
        max_dim = x_idx;
      }

      if (a < min) {
        min = a;
        min_dim = x_idx;
      }

      if (a > max) {
        max = a;
        max_dim = x_idx;
      }

      if (i === 0) {
        context.moveTo(x_px, y_px);
      } else {
        context.lineTo(x_px, y_px);
      }
    });

    context.stroke();

    let tooltip_cols = config.columns.filter(function(col) {
      if (col.colindex() === config.x_col.colindex()) {
        return false;
      }

      if (col.colindex() === config.y_col.colindex()) {
        return false;
      }

      return true;
    });

    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .addSelection(canvas)
       .text_fn(function() {
         let post_kvs = [],
             formatter = config.y_col.coldisplayformat() || lib.tickFormatDefaultNumeric;

         if (tooltipMin()) {
           post_kvs.push({
             key: 'Minimum',
             value: min,
             formatter,
           });
         }

         if (tooltipMax()) {
           post_kvs.push({
             key: 'Maximum',
             value: max,
             formatter,
           });
         }

         if (tooltipFirst()) {
           post_kvs.push({
             key: 'First Value',
             value: config.agg_getter(rows[0]),
             formatter,
           });
         }

         if (tooltipLast()) {
           post_kvs.push({
             key: 'Last Value',
             value: config.agg_getter(rows[rows.length - 1]),
             formatter,
           });
         }

         let tooltipConfig = {
           post_kvs,
         };

         return lib.text_from_columns(rows[0], tooltip_cols, tooltipConfig);
       })
       .params_fn(function(d, i, e, dimsOnly) {
          return lib.params_from_columns(rows[0], tooltip_cols, dimsOnly);
        })
       .activate();

    return sparklines;
  }

  return sparklines;
};

arc.visual.plugins.sparklines_extension = function() {
  var ext = function() { return 'arc.visual.plugins.sparklines_extension'; };

  ext.viz_name = function() {
    return 'sparklines';
  };

  ext.viz_tooltip = function() {
    return 'sparklines';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_sparklines.png?v=1';
  };

  ext.viz = arc.visual.plugins.sparklines;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.sparklines_extension());

}());
