window.arc = window.arc || {};

(function () {
  'use strict';

  arc.visual = arc.visual || {};
  arc.visual.user_settings = arc.visual.user_settings || {};
  arc.visual.plugins = arc.visual.plugins || {};
  arc.testutil = arc.testutil || {};
  arc.visual.extensions = arc.visual.extensions || [];

  arc.report.user_settings.getConstituentTable = function ($table_element) {
    let $allConstituentTables = $table_element
      .closest('.dataTables_wrapper')
      .find('table');
    if ($allConstituentTables.length === 0) {
      // Data Table is being drawn from scratch
      $allConstituentTables = $table_element;
    }

    return $allConstituentTables;
  };

  arc.visual.user_settings.settingTableHeaderAlignment = function () {
    const setting = arc.visual
      .setting()
      .id('table_header_alignment')
      .section('Style')
      .tab('Table Style')
      .type(arc.visual.settingTypeEnum.RADIO)
      .group('Table Header Alignment')
      .add_name('data_dependent')
      .add_label('<span>Align with content</span>')
      .add_name('left')
      .add_label('')
      .add_name('center')
      .add_label('')
      .add_name('right')
      .add_label('')
      .value('data_dependent');

    let $table_element = $();

    setting.$table_element = function (_) {
      if (!arguments.length) return $table_element;
      $table_element = _;
      return setting;
    };

    function paintTable(tab_d3_sel) {
      const div_id = setting.div_id();
      tab_d3_sel.append('h5').text(setting.group());

      tab_d3_sel
        .append('div')
        .attr('id', div_id)
        .attr('class', `form-group ${setting.id()}`)
        .selectAll('.nosel')
        .data(setting.names())
        .enter()
        .append('div')
        .attr('class', 'form-check form-check-inline header-alignment')
        .append('label')
        .html((d, i) => {
          let icon = '';
          if (d !== 'data_dependent') {
            icon = `<i class='fa fa-align-${d}'></i>`;
          } else {
            icon = '<i class="fa fa-columns" aria-hidden="true"></i>';
          }

          const inputbox =
            `<input type="radio" name=" ${div_id}"class="form-check-input"` +
            `" value="${d}"${d === setting.value() ? '" checked>' : '">'}${
              setting.labels().length === 0 ? d : setting.labels()[i]
            }`;
          return inputbox + icon;
        });
      return setting;
    }

    function changeEventHandler() {
      $('div.header-alignment.form-check-inline').on('change', (evt) => {
        const $control = $(evt.currentTarget);
        const new_value = $control.find('input:radio:checked').val();
        const $allConstituentTables =
          arc.report.user_settings.getConstituentTable($table_element);
        if (setting.value() !== new_value) {
          setting.value(new_value);
          setting.makeChangeCbs();
        }

        $allConstituentTables.removeClass(
          'header-align-left header-align-center header-align-right'
        );
        if (new_value !== 'data_dependent') {
          $allConstituentTables.addClass(`header-align-${new_value}`);
        }
      });
    }

    setting.paint_fn(paintTable);

    setting.add_postpaint_fn(changeEventHandler);

    return setting;
  };

  const template_styles = {
    'table-template-1': 'table-viz table-bordered',
    'table-template-2': 'table-viz table-striped table-bordered',
    'table-template-3': 'table-viz table-striped',
    'table-template-4': 'table-viz table-striped table-noborder',
    'table-template-5': 'table-viz table-striped table-firstcolumn',
    'table-template-6': 'table-viz table-striped table-sm',
    'table-template-7':
      'table-viz table-headline table-noborder header-align-center',
    'table-template-8': 'table-viz',
  };

  arc.visual.user_settings.settingTableTemplate = function () {
    let $table_element = $();
    let header_alignment = '';

    const setting = arc.visual
      .setting()
      .id('table_template')
      .section('Style')
      .tab('Table Style')
      .type(arc.visual.settingTypeEnum.CUSTOM)
      .add_name('TABLE_Template')
      .add_label('Table Template')
      .value('table-template-2');

    const template_classes = [
      'table-bordered',
      'table-striped',
      'table-noborder',
      'table-firstcolumn',
      'table-sm',
      'table-headline',
      'table-noborder',
      'header-align-center',
      'header-align-left',
      'header-align-right',
    ];

    setting.unittest_validate = function () {
      header_alignment = arc.visual.user_settings.settingTableHeaderAlignment();
      // Used by unit tests. Validate that 'template_classes' contain every class
      // defined in 'template_styles', and vice versa.
      const t = {};
      template_classes.forEach((c) => {
        t[c] = true;
      });
      const t2 = {
        'header-align-left': true,
        'header-align-right': true,
      };
      $.each(template_styles, (k, v) => {
        v.split(' ')
          .slice(1)
          .forEach((c) => {
            t2[c] = true;
            if (!(c in t)) {
              if (!(c === 'header-align-left' && k !== 'table-template-7')) {
                throw new Error(
                  `Found ${c} in template_styles, but not in template_classes`
                );
              }
            }
          });
      });
      $.each(t, (k) => {
        if (!(k in t2)) {
          throw new Error(
            `Found ${k} in template_classes, but not in template_styles`
          );
        }
      });
    };

    setting.$table_element = function (_) {
      if (!arguments.length) return $table_element;
      $table_element = _;
      return setting;
    };

    setting.header_alignment = function (_) {
      if (!arguments.length) return header_alignment;
      header_alignment = _;
      return setting;
    };

    setting.paint_table_style = function (tablestyle) {
      if (!tablestyle) {
        // This code is only expected to come in play when no parameters are passed in to
        // paint_table_style. See usage below in this file.
        // Because this is a 'string' setting, we are being a little more defensive
        // in the code than just checking for 'tablestyle === undefined'
        tablestyle = setting.value();
      }

      const $allConstituentTables =
        arc.report.user_settings.getConstituentTable($table_element);

      // Remove all styles
      $allConstituentTables.removeClass(template_classes.join(' '));

      // Add in picked styles
      const c = template_styles[tablestyle];
      const alignmentValue = header_alignment.value();
      if (c) {
        $allConstituentTables.addClass(c);
        if (tablestyle !== 'table-template-7') {
          if (alignmentValue !== 'data_dependent') {
            $allConstituentTables.addClass(`header-align-${alignmentValue}`);
          }
        }
      }

      return setting;
    };

    const fetch_setting = function () {
      const $s = $(`#${setting.div_id()}`);
      return $s.find('li.active').data('tablestyle');
    };

    const paint_setting = function (tab_d3_sel) {
      tab_d3_sel
        .append('div')
        .attr('id', setting.div_id())
        .style('margin-left', '-5px')
        .html($('#table-styles-template').html());

      const $s = $(`#${setting.div_id()}`);
      $s.find('.table-template-list li').removeClass('active');
      $s.find(`.${setting.value()}`)
        .closest('li')
        .addClass('active');

      $s.find('.table-template-list li').offOn(
        'click.modifyingTemplate',
        function () {
          $s.find('.table-template-list li').removeClass('active');
          $(this).addClass('active');
          setting.value(fetch_setting());
          setting.viz_lib().refresh(true);
        }
      );

      return setting;
    };

    setting.paint_fn(paint_setting);
    setting.fetch_fn(fetch_setting);

    return setting;
  };

  arc.visual.plugins.table = function () {
    let sb_sql;
    let data_result;
    let lib;
    let unique_tableid;
    let currentParams;
    let resizeTracker;
    let customWidths = [];
    let firstTooltipIdx;
    let tooltipIdxs = [];
    let preventOrderOverride;

    // Introduced to able to configure the table outside of the dashboard
    // where the table settings are not available
    let configuration = {
      showTooltip: true,
      showDetailedData: false,
      columnNamesAreHtml: false,
      fullWidth: false,
    };

    const pageSizeMax = 1000;
    const pageSizeDefault = 20;

    const table = arc.visual.api();

    const expansionTracker = arc.expansionTracker();

    table.setConfiguration = (newPartialConfiguration) => {
      configuration = { ...configuration, ...newPartialConfiguration };
    };

    const parameter_all_cols = arc.visual
      .setting()
      .id('parameters_send_all_cols')
      .tab('Parameters')
      .type(arc.visual.settingTypeEnum.CHECKBOX)
      .add_name('PARAMETERS_SEND_ALL_COLS')
      .add_label('Send all fields as parameters')
      .info('Unless selected, the visual only sends the clicked field.')
      .value('false');

    const show_contextmenu = arc.visual
      .setting()
      .id('show_contextmenu')
      .tab('Marks')
      .type(arc.visual.settingTypeEnum.CHECKBOX)
      .add_name('SHOW_CONTEXTMENU')
      .add_label('Show context menu')
      .value('true');

    const http_auto_link = arc.visual
      .setting()
      .id('http_auto_link')
      .tab('Data')
      .type(arc.visual.settingTypeEnum.CHECKBOX)
      .add_name('HTTP_AUTO_LINK')
      .add_label('Show values starting with http(s):// as links')
      .value('true');

    const nulls_display_value = arc.visual
      .setting()
      .id('nulls_display_value')
      .tab('Data')
      .type(arc.visual.settingTypeEnum.STRING)
      .add_name('NULL_DISPLAY_VALUE')
      .add_label('Display null value as: ')
      .value('');

    const maxColumns = arc.visual
      .setting()
      .id('max_columns')
      .section('Style')
      .tab('Table Style')
      .type(arc.visual.settingTypeEnum.INTEGER)
      .add_name('MAX_COLUMNS')
      .add_label('Max # Columns')
      .value('20');

    const enable_info = arc.visual
      .setting()
      .id('enable_info')
      .section('Style')
      .tab('Table Style')
      .type(arc.visual.settingTypeEnum.CHECKBOX)
      .add_name('SHOW_TABLE_INFO')
      .add_label('Show row information')
      .value('false')
      .valueCb(() => {
        table.setConfiguration({ enableInfo: enable_info.isChecked });
      });

    const keepWithinLimits = function (value, errorMessage = {}) {
      value = Number(value);
      if (value < 0 || value > pageSizeMax) {
        // Give user the most they can see on a page while still paginating if
        // they set the value too high or try to use the old negative one trick
        value = pageSizeMax;
        errorMessage.msg = 'Invalid value. Defaulted to maximum permitted';
      } else if (!arc.misc.isNumeric(value) || value === 0) {
        // Unclear what the user wants, but shouldn't let an invalid value stand
        value = pageSizeDefault;
        errorMessage.msg = 'Invalid value. Reset to default.';
      }

      value = Math.round(value);

      return value;
    };

    const pageSizeSetting = arc.visual
      .setting()
      .id('page_size')
      .section('Style')
      .tab('Table Style')
      .type(arc.visual.settingTypeEnum.INTEGER_NOSPINNER)
      .add_name('PAGE_SIZE')
      .add_label('Number of rows per page: ')
      .valuePreprocessCb(keepWithinLimits)
      .info(
        `Applies only if pagination is enabled. (Range: 1 - ${pageSizeMax})`
      )
      .value(pageSizeDefault);

    const updateToWithinLimits = function () {
      const errorMessage = {};
      const $input = $(`#${pageSizeSetting.div_id()} input`);
      $input.nextAll('p.text-danger').remove();
      const currentVal = $input.val();
      $input.val(keepWithinLimits(currentVal, errorMessage));
      const { msg } = errorMessage;
      if (msg) {
        const $msg = $(`<p class="text-danger">${msg}</p>`);
        $msg.opacity = 0;
        $input.after($msg);
        $msg.fadeIn(100).delay(3000).fadeOut(200);
      }
    };

    pageSizeSetting
      .addChangeCb('updateToWithinLimits', updateToWithinLimits)
      .valueCb(() => {
        table.setConfiguration({ pageSize: pageSizeSetting.value() });
      });

    const enablePaginationSetting = arc.visual
      .setting()
      .id('enable_pagination')
      .section('Style')
      .tab('Table Style')
      .type(arc.visual.settingTypeEnum.CHECKBOX)
      .add_name('ENABLE_PAGINATION')
      .add_label('Show pagination')
      .value('true')
      .valueCb(() => {
        table.setConfiguration({
          enablePagination: enablePaginationSetting.isChecked,
        });
      });

    function settingTableFullWidth() {
      const widthSetting = arc.report.user_settings.settingFullWidth();

      widthSetting.enabled = function () {
        return widthSetting.isChecked;
      };

      widthSetting.paint_table_style = function (isFullWidth) {
        if (isFullWidth === undefined) {
          // This code comes in play when no parameters are passed in to
          // paint_table_style. See usage below in this file.
          isFullWidth =
            widthSetting.isChecked || Boolean(configuration?.fullWidth);
        }

        if (widthSetting.isChecked !== isFullWidth) {
          widthSetting.value(isFullWidth);
          widthSetting.makeChangeCbs();
        }

        // Remove all styles
        const $allCTables = arc.report.user_settings.getConstituentTable(
          widthSetting.$table_element()
        );
        $allCTables.removeClass('table-width-auto resizable table-width-100');
        if (!isFullWidth) {
          $allCTables.addClass('table-width-auto resizable');
        } else {
          $allCTables.addClass('table-width-100');
        }

        return widthSetting;
      };

      return widthSetting;
    }

    const table_full_width = settingTableFullWidth();
    const table_template = arc.visual.user_settings.settingTableTemplate();
    const page_totals = arc.settings.table.settingPageTableTotals();
    const table_totals = arc.settings.table.settingTableTotals();
    const table_totals_header = arc.settings.table.settingTableTotalsHeader();
    const page_totals_header =
      arc.settings.table.settingPageTableTotalsHeader();
    const table_header_alignment =
      arc.visual.user_settings.settingTableHeaderAlignment();
    const enable_highlighting = arc.settings.table.enableHighlighting();
    const cells_are_html_setting = arc.settings.table
      .cellsAreHTMLSetting(enable_highlighting)
      .valueCb(() => {
        table.setConfiguration({ cellsHtml: cells_are_html_setting.isChecked });
      });

      const template_styles_copy = { ...template_styles };
      delete template_styles_copy['table-template-7'];
      delete template_styles_copy['table-template-8'];

      table_template.addDependents([
        table_header_alignment
    ], ['table-template-7', 'table-template-8'], Object.keys(template_styles_copy));

    // valueCb only called when user changes the values not when the setting is first created
    // Here we set the initial values
    table.setConfiguration({
      enablePagination: enablePaginationSetting.isChecked,
      enableInfo: enable_info.isChecked,
      pageSize: pageSizeSetting.value(),
      cellsHtml: cells_are_html_setting.isChecked,
    });

    const user_settings = arc.visual
      .defaultSettings({
        noTrellis: true,
        noAxes: true,
      })
      .addSetting([
        arc.report.user_settings.display_tooltips_table(),
        table_template,
        table_header_alignment,
        table_full_width,
        arc.settings.table.wrapColumnContents(),
        parameter_all_cols,
        show_contextmenu,
        cells_are_html_setting,
        enable_highlighting,
        nulls_display_value,
        http_auto_link,
        page_totals,
        page_totals_header,
        table_totals,
        table_totals_header,
        arc.settings.fonts.totalsColumnFontWeight(),
        maxColumns,
        enable_info,
        enablePaginationSetting,
        pageSizeSetting,
        arc.settings.table.tableHeaderBackgroundColor(),
        arc.settings.table.tableHeaderTextColor(),
        arc.settings.table.tableOddRowsBackgroundColor(),
        arc.settings.table.tableEvenRowsBackgroundColor(),
        arc.settings.table.tableOddRowsTextColor(),
        arc.settings.table.tableEvenRowsTextColor(),
        arc.settings.table.tableBorderColor(),
        arc.settings.table.tableFooterBackgroundColor(),
        arc.settings.table.tableFooterTextColor(),
        arc.settings.table.displayColumnHeaders(),
        arc.settings.table.displayHeaderColumnBorders(),
      ]);

    user_settings.find_by_id('show_tooltip').hide();

    table.viz_lib = function (_) {
      if (!arguments.length) return lib;
      lib = _;
      expansionTracker.viz_lib(lib);
      lib.user_settings(user_settings).redraw_on_settings_cancel(true);
      return table;
    };

    table.sb_sql = function (_) {
      if (!arguments.length) return sb_sql;
      sb_sql = _;
      return table;
    };

    table.viz_drawDesigner = function (shelves_div) {
      lib.shelves_dom_id(shelves_div).base_setup();

      // Define messages to be included in shelves
      const dimensionMessages = [
        lib.createPopoverMessages('dimensionTable'),
        lib.createPopoverMessages('acceptsMultiple'),
      ];
      const aggregateMessages = [
        lib.createPopoverMessages('aggregateTable'),
        lib.createPopoverMessages('acceptsMultiple'),
      ];
      // Create those popovers
      lib.addShelfDescription(
        {},
        {
          dimensions_shelf: dimensionMessages,
          aggregates_shelf: aggregateMessages,
        }
      );

      lib.addLimit();
      return table;
    };

    table.viz_getDataRequest = function () {
      const req = lib.shelvesToDataRequest(lib.shelf_data(), {
        build_default_viz: true,
        dflviz_all_tables: false,
        dflviz_column_limit: parseInt(maxColumns.value(), 10),
      });
      const columns = req.columns();

      // Don't allow empty measure/dimension with tooltips
      const firstTooltip = lib.first_column(columns, 'tooltip_shelf');
      if (firstTooltip && firstTooltip.colindex() === 0) {
        lib.alertMessage(
          'Please specify a dimension or measure.' +
            ' Otherwise remove columns from tooltip shelf'
        );
      }

      expansionTracker.onGetDataRequest(req);

      return req;
    };

    /*
     * Set the data this object should use to draw out the visual
     */
    table.viz_setDataResult = function (result) {
      data_result = result;
      expansionTracker.onSetDataResult(data_result);

      // Hide 'Enable highlighting' setting for non-solr connections
      if (lib.dataconnection_type() === 'solr') {
        enable_highlighting.show();
      } else {
        enable_highlighting.hide();
      }

      return table;
    };

    let saved_info = {};

    table.viz_getSettings = function () {
      let settings = saved_info;
      settings = lib.save_basics(settings);
      lib.saveLimit(settings);
      saved_info = settings;
      settings.customWidths = table.customWidths();
      return settings;
    };

    table.customWidths = function (_) {
      if (!arguments.length) return customWidths;
      customWidths = _ || [];
      return table;
    };

    table.viz_setSettings = function (settings, switchingVisualType) {
      if (settings === undefined) {
        return table;
      }

      if (switchingVisualType) {
        settings.limit = 100;
      }

      saved_info = settings;

      // Backward compatibility - when restoring for existing visuals, start
      // at -1 i.e. no limit
      maxColumns.value(-1);

      lib.restore_basics(settings, switchingVisualType);
      lib.restoreLimit(settings);

      table.customWidths(settings.customWidths);
      return table;
    };

    table.viz_fetchCollateral = function (viz_div, isWindowResize, drawCb) {
      // Don't draw table visual unless the fonts have been loaded. Otherwise,
      // the headers can be misaligned.
      function drawAfterLoad() {
        drawCb(viz_div, isWindowResize);
      }

      if (
        !arc.misc.isIEOptimisationEnabled() &&
        !arc.testutil.ignoreFontLoader
      ) {
        arc.misc.detectFontsLoaded(drawAfterLoad, drawAfterLoad);
      } else {
        drawAfterLoad();
      }

      return table;
    };

    table.viz_drawVisual = function (viz_div) {
      lib.chart_div(viz_div);

      // if not called on window resize
      lib.undraw();

      let sb_rows = data_result.rows();
      if (!sb_rows || sb_rows.length === 0) {
        return table;
      }

      // Get firstTooltipIdx for calculating totals and getting maximum columns to visualize
      // Store list of tooltipIdxs for adding tooltips upon hover
      const columns = data_result.columns();

      // TODO: update base.shelfElementCount to something similar that doesn't rely on html
      const tooltipCols = lib.shelf_columns(columns, 'tooltip_shelf');
      firstTooltipIdx =
        tooltipCols.length > 0 ? tooltipCols[0].colindex() : columns.length;
      tooltipIdxs = tooltipCols.map((col) => col.colindex());

      lib.unselectOnClick();

      const dim_colindex = data_result.dim_start_idx();
      if (dim_colindex >= 0) {
        sb_rows = lib.filterLocally(sb_rows, dim_colindex);
      }

      if (!configuration?.cellsHtml && expansionTracker.enabled()) {
        table.setConfiguration({ cellsHtml: true });
      }

      table.tabulate(sb_rows, `#${viz_div}`);

      return table;
    };

    function tableCellFormat(col) {
      // override default number format for tables
      if (col.isNumeric()) {
        return col.coldisplayformat(true) || d3.format(',');
      }

      return col.coldisplayformat();
    }

    let postPaintCb;
    table.postPaintCb = function (_) {
      if (!arguments.length) return postPaintCb;
      postPaintCb = _;
      return table;
    };

    function setWrapStyle($table) {
      const wrap = user_settings.find_by_id(
        'table_wrap_column_contents'
      ).isChecked;
      if (wrap) {
        return;
      }

      $table.addClass('nowrap');
    }

    function theadStyleFetch() {
      const theadStyle = [];

      const headerBgColor = user_settings
        .find_by_id('table_header_background_color')
        .value();
      if (headerBgColor) {
        theadStyle.push(`background-color:${headerBgColor};`);
      }

      const headerTextColor = user_settings
        .find_by_id('table_header_text_color')
        .value();
      if (headerTextColor) {
        theadStyle.push(`color:${headerTextColor}`);
      }

      return theadStyle;
    }

    function tfootStyleFetch() {
      const tfootStyle = [];

      const footerBgColor = user_settings
        .find_by_id('table_footer_background_color')
        .value();
      if (footerBgColor) {
        tfootStyle.push(`background-color:${footerBgColor};`);
      }

      const footerTextColor = user_settings
        .find_by_id('table_footer_text_color')
        .value();
      if (footerTextColor) {
        tfootStyle.push(`color:${footerTextColor}`);
      }

      return tfootStyle;
    }

    /*
     * This function adds in a <style.vizstyle> section within the table
     * visual so we can apply user-specified styling.
     */
    function setVizTableStyles() {
      const chartId = lib.chart_div();
      const allStyles = [];

      const headTable = `#${chartId} .dataTables_scrollHead table`;
      const bodyTable = `#${chartId} .dataTables_scrollBody table`;
      const footTable = `#${chartId} .dataTables_scrollFoot table`;
      const totalsTable = `#${chartId} .dataTables_scrollFoot table .table-total-row td span`;
      const totalsPageTable = `#${chartId} .dataTables_scrollFoot table .table-page-total-row td span`;
      // print styles for multi-page tables
      const printTable = `.arc-print-last-table #${chartId} .dataTables_scrollBody table`;

      const slickTableHeader = `#${chartId} .slick-header .slick-header-columns .slick-header-column`;
      const slickTable = `#${chartId} .slick-viewport .grid-canvas`;
      const slickCells = `#${chartId} .slick-header .slick-header-column, #${chartId} .slick-viewport .grid-canvas .slick-row .slick-cell`;
      // Note that slickHeaderCells needs more CSS selectivity than slickCells above
      const slickHeaderCells = `#${chartId} .slick-header .slick-header-columns .slick-header-column`;

      {
        const displayHeader =
          user_settings.find_by_id('display_header').isChecked;
        if (!displayHeader) {
          allStyles.push(`${headTable} th { display: none; }`);
        }
      }

      {
        // Default max column width. Users who want a larger value can
        // resize columns
        const maxColWidth = 400;
        const isWrapped = user_settings.find_by_id(
          'table_wrap_column_contents'
        ).isChecked;
        allStyles.push(`${headTable} th, ${bodyTable} td, ${footTable} td {`);

        if (chartId !== 'viz-queries-table') {
          allStyles.push(`max-width:${maxColWidth}px;`);
        }

        if (!isWrapped) {
          allStyles.push('overflow:hidden;text-overflow:ellipsis;');
        } else {
          allStyles.push('word-break: break-word;vertical-align: top;white-space: normal;');
        }

        allStyles.push('}');
      }

      {
        const font = lib.getVisualFont();
        if (font) {
          allStyles.push(
            `#${chartId} .table-responsive-md.ui-widget {font-family: ${font};}`
          );
        }
      }

      {
        const fontSize = lib.getVisualFontSize();
        if (fontSize) {
          allStyles.push(
            `${headTable}, ${bodyTable}, ${footTable}, #${chartId} .table-responsive-md.ui-widget {`
          );
          allStyles.push(`font-size:${fontSize}px;`);
          allStyles.push('}');
        }
      }

      {
        const theadStyle = theadStyleFetch();
        if (theadStyle.length > 0) {
          allStyles.push(
            `${headTable} thead th, ${bodyTable}.table-firstcolumn > tbody > tr > td:first-child, ${slickTableHeader}, ${printTable} thead tr th {`
          );
          theadStyle.forEach((s) => {
            allStyles.push(s);
          });

          allStyles.push('}');
        }
      }

      {
        const tfootStyle = tfootStyleFetch();
        if (tfootStyle.length > 0) {
          allStyles.push(`${footTable} tfoot td, ${printTable} tfoot tr td {`);
          tfootStyle.forEach((s) => {
            allStyles.push(s);
          });

          allStyles.push('}');
        }
      }

      {
        const totalsColumnFontWeight = lib.getTotalsFontWeight(
          'totals_column_font_weight'
        );

        if (totalsColumnFontWeight) {
          allStyles.push(
            `${totalsTable} {font-weight:${totalsColumnFontWeight};}`
          );
          allStyles.push(
            `${totalsPageTable} {font-weight:${totalsColumnFontWeight};}`
          );
        }
      }

      {
        const displayHeaderColumnBorders = user_settings.find_by_id(
          'display_header_column_borders'
        ).isChecked;

        if (!displayHeaderColumnBorders) {
          allStyles.push(
            `${headTable} thead th:not(:last-child), ${slickHeaderCells} {`
          );
          allStyles.push('border-right-width:0px;');
          allStyles.push('}');
          allStyles.push(`${slickHeaderCells} {`);
          allStyles.push('border-left-width:0px;');
          allStyles.push('}');
          allStyles.push(`${slickHeaderCells}:first-child {`);
          allStyles.push('border-left-width:1px;');
          allStyles.push('}');
          allStyles.push(`${slickHeaderCells}:last-child {`);
          allStyles.push('border-right-width:1px;');
          allStyles.push('}');
        }
      }

      const $viz = $(`#${chartId}`);

      {
        const borderColor = user_settings
          .find_by_id('table_border_color')
          .value();
        if (borderColor) {
          allStyles.push(`${headTable} thead th, ${slickCells} {`);
          allStyles.push(`border-color:${borderColor};`);
          allStyles.push('}');
          allStyles.push(`${bodyTable} tbody td {`);
          allStyles.push(`border-color:${borderColor};`);
          allStyles.push('}');
        }
      }

      {
        const trEven = [];
        const bg = user_settings
          .find_by_id('table_even_rows_background_color')
          .value();
        if (bg) {
          trEven.push(`background:${bg};`);
          trEven.push(`background-color:${bg};`);
        }

        const fg = user_settings
          .find_by_id('table_even_rows_text_color')
          .value();
        if (fg) {
          trEven.push(`color:${fg};`);
        }

        if (trEven.length > 0) {
          allStyles.push(
            `${bodyTable} tbody tr:nth-of-type(even), ${slickTable} .slick-row.odd {`
          );
          trEven.forEach((s) => {
            allStyles.push(s);
          });

          allStyles.push('}');
        }
      }

      {
        const trOdd = [];
        const bg = user_settings
          .find_by_id('table_odd_rows_background_color')
          .value();
        if (bg) {
          trOdd.push(`background:${bg};`);
          trOdd.push(`background-color:${bg};`);
        } else {
          allStyles.push(
            `${slickTable} .slick-row.even {background-color:#f9f9f9;background:#f9f9f9}`
          );
        }

        const fg = user_settings
          .find_by_id('table_odd_rows_text_color')
          .value();
        if (fg) {
          trOdd.push(`color:${fg};`);
        }

        if (trOdd.length > 0) {
          allStyles.push(
            `${bodyTable} tbody tr:nth-of-type(odd), ${slickTable} .slick-row.even {`
          );
          trOdd.forEach((s) => {
            allStyles.push(s);
          });

          allStyles.push('}');
        }
      }

      let $style = $viz.find('style.vizstyle');
      if ($style.length === 0) {
        $viz.append(
          arc.misc.safe_substitute('<style class="vizstyle"></style>', {})
        );
        $style = $viz.find('style.vizstyle');
      }

      $style.html(`${allStyles.join('')}`);
    }

    table.adjustColumnWidths = function () {
      const $dt = $(
        `#${lib.chart_div()} .dataTables_scrollBody table`
      ).data('data-table');
      if ($dt) {
        $dt.columns.adjust();
        resizeTracker.collectSizes();
      }
    };

    function computeDisplayOrder(columns) {
      if (!columns) {
        return {};
      }

      const orderedColumns = [];
      columns.forEach((col, idx) => {
        const insertBefore = col.coldisplaybefore();
        if (!insertBefore) {
          orderedColumns.push(col);
          return;
        }

        let inserted = false;
        for (let before = 0; before < idx; ++before) {
          const beforeCol = orderedColumns[before];
          if (beforeCol.colname() === insertBefore) {
            orderedColumns.splice(before, 0, col);
            inserted = true;
            break;
          }
        }

        if (!inserted) {
          orderedColumns.push(col);
        }
      });

      const orderMap = {};
      const orderMapReversed = {};
      orderedColumns.forEach((col, i) => {
        orderMap[i] = col.colindex();
        orderMapReversed[col.colindex()] = i;
      });

      return {
        orderMap,
        orderMapReversed,
        orderedColumns,
      };
    }

    function tabulateGrid(sb_rows, container_div) {
      setVizTableStyles();

      const $container_div = $(container_div);
      const chartId = lib.chart_div();
      const h = $(`#${chartId}`).height();

      $container_div.height(`${h}px`);

      const columns = [];
      data_result.columns().forEach((c, idx) => {
        columns.push({
          name: c.colname(),
          field: idx,
          width: customWidths[idx] ? customWidths[idx] : undefined,
        });
      });

      const options = {
        enableCellNavigation: true,
        enableColumnReorder: false,
        rowHeight: 40,
      };

      // eslint-disable-next-line no-undef
      const grid = new Slick.Grid(container_div, sb_rows, columns, options);
      grid.onColumnsResized.subscribe(() => {
        const widths = grid.getColumns().map((c) => c.width);
        customWidths = widths;
        lib.page_needs_save();
      });
    }

    let sortOrder = [];
    let currentPage = 0;

    table.tabulate = function (sb_rows, container_div) {
      if (sb_rows.length === 0) {
        return table;
      }

      // Initialize variables
      const {
        enablePagination,
        enableInfo,
        pageSize,
        showTooltip,
        showDetailedData,
        cellsHtml,
        columnNamesAreHtml,
        disableProfile,
      } = configuration;

      const template = table_template.value();
      if (template === 'table-template-8') {
        tabulateGrid(sb_rows, container_div);
        return table;
      }

      const start = new Date();

      const columns = data_result && data_result.columns();
      const columnsPriority = columns.map((col) => {
        if (col.order()) {
          return col.order().priority();
        }

        return Number.MAX_SAFE_INTEGER;
      });

      const { orderMap, orderMapReversed, orderedColumns } =
        computeDisplayOrder(columns);

      // When table.tabulate called outside of table visual, firstTooltipIdx doesn't exist
      const dimensionsMeasures = firstTooltipIdx || orderedColumns.length;
      const maxColValue = parseInt(maxColumns.value(), 10);
      const maxColNumber =
        maxColValue > 0 && !showDetailedData ?
          Math.min(dimensionsMeasures, maxColValue) :
          dimensionsMeasures;

      let expansionIndex = -1;
      if (expansionTracker.enabled()) {
        expansionIndex = expansionTracker.firstExpansionDimensionIndex();
      }

      sortOrder = columns.map((column, i) => {
        if (sortOrder[i] !== undefined) {
          if (
            expansionTracker.enabled() &&
            sortOrder[i] === arc.consts.SORT_OPTION_UNSORTED
          ) {
            return true;
          }

          if (preventOrderOverride) {
            return sortOrder[i];
          }
        }

        if (column.order()) {
          return column.order().ascending();
        }

        return expansionTracker.enabled() ?
          true :
          arc.consts.SORT_OPTION_UNSORTED;
      });

      preventOrderOverride = false;

      function drawSortIcon(sorted, colIndex) {
        const classes = [];

        switch (sorted) {
          case arc.consts.SORT_OPTION_UNSORTED: {
            classes.push(arc.consts.SORT_OPTION_UNSORTED);
            classes.push('fa-sort');
            break;
          }

          case true: {
            classes.push('asc');
            classes.push('fa-sort-amount-asc');
            break;
          }

          default: {
            classes.push('desc');
            classes.push('fa-sort-amount-desc');
            break;
          }
        }

        return `
          <a href="#" class="custom-sort">
            <span style="display: inline-block; width: 14px; text-align: center;" class="viz-col-sort">
              <i class="viz-sort-icon fa ${classes.join(
                ' '
              )}" data-col-idx=${colIndex}></i>
            </span>
          </a>`;
      }

      const is_headline_table = template === 'table-template-7';

      const dtcols = [];
      for (let i = 0; i < maxColNumber; i++) {
        const c = orderedColumns[i];
        let className = c.isNumeric() ? 'dt-right' : 'dt-left';
        if (is_headline_table) {
          className = undefined;
        }

        const type = c.isNumeric() ? 'num' : 'string';
        let title = columnNamesAreHtml ?
          c.colname() :
          arc.misc.escapeHTML(c.colname());
        if (
          expansionIndex >= 0 &&
          expansionIndex <= i &&
          c.colproperty() === arc.report.columnTypeEnum.DIMENSION
        ) {
          title +=
            `<span data-row-number="-1" data-column-index="${i}" class="crosstab-collapse"><i class="fa fa-minus-square-o"></i></span>`;
        }

        if (configuration.ordering || configuration.ordering === undefined) {
          title += drawSortIcon(sortOrder[i], i);
        }

        const data = {
          _: `col${i}.cell`,
          display: `col${i}.display`,
          sort: `col${i}.sort`,
        };
        const desc = c.coldesc() || title;
        dtcols.push({ className, title, data, type, desc });
      }

      const timestamp = Date.now();
      unique_tableid = `data-table${timestamp}`;
      const t = d3
        .selectAll(container_div)
        .append('table')
        .attr('id', unique_tableid)
        .attr(
          'class',
          `${
            'table table-responsive-md table-striped table-bordered table-sm ' +
            'header-align-'
          }${table_header_alignment.value()}`
        );
      const $table = $(`#${unique_tableid}`);

      // The tfoot operation seems required so that the 'tfoot' parameter to
      // 'footerCallback' function is valid.
      const colTypes = orderedColumns.map((c) => c.coltype);
      t.append('thead');
      t.append('tbody');
      t.append('tfoot')
        .append('tr')
        .data(colTypes)
        .enter()
        .append('td');

      // In non-paging mode, the 'end' variable seems to be determined by the 'scrollY'
      // parameter rather than what is actually visible.
      // DataTable.defaults.infoCallback variables used here
      // viz/arcweb/static/js/ext/datatables-1.11.5/datatables.js
      function getInfoData(oSettings, startIndex, endIndex, max, total) {
        let text = '';
        const isPagingEnabled = enablePagination;
        if (enableInfo) {
          if (isPagingEnabled) {
            text = `Showing ${startIndex} - ${endIndex} of ${total} rows`;
          } else {
            text = `Showing all ${total} rows`;
          }
        }

        return text;
      }

      const stopProp = '(function(e){e.stopPropagation();})(event)';

      function linkify(cell) {
        if (
          http_auto_link.isChecked &&
          String(cell).match(/^(http|https):\/\//)
        ) {
          const url = arc.misc.getBlankTargetUrl(cell);
          return `<a target="_blank" onClick="${stopProp}" href="${url}">${cell}</a>`;
        }

        return cell;
      }

      const _isNull = (val) => [undefined, null].includes(val);

      // TODO: refactor and remove call to formatted display
      function _formatFunc(col) {
        function safeString(input) {
          if (cellsHtml && !arc.misc.hasScriptTag(input)) {
            return input;
          }

          return arc.misc.escapeHTML(input);
        }

        function tryAsNumber(input) {
          const parsedAsNum = Number(input);
          if (arc.misc.isNumeric(parsedAsNum)) {
            return parsedAsNum;
          }

          return safeString(input);
        }

        const _parse = col.isNumeric() ? tryAsNumber : safeString;
        const colFormat = tableCellFormat(col);
        return (n) => linkify(lib.formattedDisplay(colFormat, _parse(n)));
      }

      // TODO(Travis): Seperate user settings for frontend and backend sorting
      // TODO: Make user setting option better analog of DB sort options
      // null sorting action defaults to last consistent with JS and Impala sorting
      const nullSortOrder = lib.nulls_order_with_default();

      const stringInf = new Array(100).fill(String.fromCharCode(-1)).join('');
      const nullSortVal = (col) => {
        if (col.isNumeric()) {
          return nullSortOrder === 'Last' ? Infinity : -Infinity;
        }

        return nullSortOrder === 'Last' ? stringInf : '';
      };

      const nullDisplay = nulls_display_value.value();
      const nullVals = orderedColumns.map(nullSortVal);
      const formats = orderedColumns.map(_formatFunc);
      const isFullWidth = table_full_width.enabled();
      const dtrows = [];

      sb_rows = arc.misc.arrayMultiSort(sb_rows, sortOrder, true, columnsPriority);

      for (let r = 0; r < sb_rows.length; r++) {
        const row = {};
        for (let c = 0; c < maxColNumber; c++) {
          const cell = sb_rows[r][orderMap[c]];
          const sort = _isNull(cell) ? nullVals[c] : cell;
          const display = _isNull(cell) ? nullDisplay : formats[c](cell);
          row[`col${c}`] = { sort, display, cell };
        }

        dtrows.push(row);
      }

      let fnCreatedRow;

      // If there are shelves with column background color,
      // apply color to row data when creating table rows.
      if (columns) {
        const shelfColumnColorArr = getAllColumnColorArrFromShelf();
        if (shelfColumnColorArr.length) {
          fnCreatedRow = (e, row) => {
            const $cols = $(e).find('>td');

            shelfColumnColorArr.forEach((bgColor) => {
              const { column, colorScale, opacity } = bgColor;

              try {
                const colIndex = orderMapReversed[column.colindex()];
                const $col = $cols.eq(colIndex);
                const colData = row[`col${colIndex}`];
                const agg = colData.sort;
                let bg = colorScale(agg);
                if (
                  bgColor.dimensionMappings &&
                  Object.keys(bgColor.dimensionMappings).length > 0 &&
                  bgColor.dimensionMappings[colData.cell]
                ) {
                  bg = bgColor.dimensionMappings[colData.cell];
                }

                if (!bg) {
                  return;
                }

                const rgba = arc.colorpalette.colorToRGBA(bg, opacity);
                $col
                  .css('background-color', rgba)
                  .css(
                    'color',
                    arc.colorpalette.colorForBrightness(rgba, opacity)
                  );
              } catch (error) {
                // Ignore any errors in setting colors
              }
            });
          };
        }
      }

      function getAllColumnColorArrFromShelf() {
        const totalColumnColorArr = [];
        const coloredAggShelves = [];
        const coloredDimShelves = [];

        lib.shelfElementCb('aggregates_shelf', (shelfElement, i) => {
          if (shelfElement.displayBackgroundColor()) {
            coloredAggShelves[i] = shelfElement;
          }
        });

        lib.shelfElementCb('dimensions_shelf', (shelfElement, i) => {
          if (shelfElement.displayBackgroundColor()) {
            coloredDimShelves[i] = shelfElement;
          }
        });

        const aggCols = lib
          .shelf_columns(columns, 'aggregates_shelf')
          .filter((col) => coloredAggShelves[col.shelfIndex()]);

        aggCols.forEach((col) => {
          if (!col) {
            return;
          }

          const shelfElement = coloredAggShelves[col.shelfIndex()];
          try {
            const getAgg = (data) => Number(data[col.colindex()]);
            const colorDomain = d3.extent(sb_rows, getAgg);

            let colors;
            let colorSettings;
            let opacity;

            const bgColor = shelfElement.backgroundColor();
            if (bgColor) {
              colors = arc.colorpalette.getOrderedColorPalette(bgColor);
              colorSettings = bgColor.colorSettings;
              opacity = bgColor.opacity / 100;
            } else {
              colors = lib.graded_colors();
              colorSettings = lib.colorSettings();
              opacity = lib.colorOpacity();
            }

            totalColumnColorArr.push({
              column: col,
              colorScale: arc.colorpalette.aggScale(
                colorDomain,
                colors,
                colorSettings
              ),
              opacity,
            });
          } catch (e) {
            // Ignore errors
          }
        });

        const dimCols = lib
          .shelf_columns(columns, 'dimensions_shelf')
          .filter((col) => coloredDimShelves[col.shelfIndex()]);

        dimCols.forEach((col) => {
          if (!col) {
            return;
          }

          const shelfElement = coloredDimShelves[col.shelfIndex()];
          if (!shelfElement) {
            return;
          }

          try {
            const defaultColorDomain = arc.misc.arrayUnique(
              sb_rows.map((d) => d[col.colindex()])
            );

            let colors;
            let fixedDimensionColors;
            let dimensionMappings;
            let opacity;

            const bgColor = shelfElement.backgroundColor();
            if (bgColor) {
              colors = arc.colorpalette.getOrderedColorPalette(bgColor);
              dimensionMappings = bgColor.dimensionMappings;
              fixedDimensionColors = bgColor.fixedDimensionColors;
              opacity = bgColor.opacity / 100;
            } else {
              colors = lib.colors();
              opacity = lib.colorOpacity();
              dimensionMappings = {};
              fixedDimensionColors = false;
            }

            const colorScale = arc.colorpalette.dimScale(
              defaultColorDomain,
              colors,
              {
                fixedDimensionColors: false,
                dimensionMappings: {},
              }
            );
            totalColumnColorArr.push({
              column: col,
              colorScale,
              dimensionMappings,
              fixedDimensionColors,
              opacity,
            });
          } catch (e) {
            // Ignore errors
          }
        });

        return totalColumnColorArr;
      }

      const dtSettings = {
        data: dtrows,
        columns: dtcols,
        deferRender: true,
        scrollX: true,
        info: enableInfo,
        ordering: false,
        $parent: $(`#${lib.chart_div()}`),
        infoCallback: getInfoData,
        showPageTotals: page_totals.isChecked,
        showTotals: table_totals.isChecked,
        adjustTableHeight: true,
        footerCallback: handleColumnTotals,
        fnCreatedRow,
        fnDrawCallback() {
          if (configuration.drawCallback) {
            configuration.drawCallback();
          }

          if (resizeTracker) {
            resizeTracker.setSizes();
            if (!isFullWidth) {
              table.adjustColumnWidths();
            }
          }

          if (expansionTracker.enabled()) {
            $(`#${lib.chart_div()} .crosstab-expand`).offOn(
              'click.dataTable',
              function (e) {
                e.preventDefault();
                e.stopPropagation();
                e.stopImmediatePropagation();
                const rowIdx = $(e.currentTarget)
                  .closest('tr')
                  .data('parent-row-idx');
                expansionTracker.expand(this, rowIdx);
              }
            );

            $(`#${lib.chart_div()} .crosstab-collapse`).offOn(
              'click.dataTable',
              function (e) {
                e.preventDefault();
                e.stopPropagation();
                e.stopImmediatePropagation();
                expansionTracker.collapse(this);
              }
            );
          }

          $(`#${lib.chart_div()}`).offOn(
            'click.dataTable',
            '.custom-sort',
            (e) => {
              e.stopPropagation();
              e.preventDefault();
              const $this = $(e.target);
              const colIdx = $this.data('col-idx');

              preventOrderOverride = true;

              if (expansionTracker.enabled()) {
                for (let i = 0; i < colIdx; i++) {
                  if (
                    sortOrder[i] === arc.consts.SORT_OPTION_UNSORTED ||
                    sortOrder[i] === undefined
                  ) {
                    sortOrder[i] = true;
                  }
                }

                if ($this.hasClass('asc')) {
                  sortOrder[colIdx] = false;
                } else {
                  sortOrder[colIdx] = true;
                }
              } else if ($this.hasClass('desc')) {
                sortOrder[colIdx] = arc.consts.SORT_OPTION_UNSORTED;
              } else if ($this.hasClass('asc')) {
                sortOrder[colIdx] = false;
              } else {
                sortOrder[colIdx] = true;
              }

              let $sel = $(`#${lib.chart_div()}`).find(
                '.dataTables_scrollBody'
              );
              const scrollTop = $sel.scrollTop();
              const scrollLeft = $sel.scrollLeft();
              table.viz_drawVisual(lib.chart_div());
              $sel = $(`#${lib.chart_div()}`).find('.dataTables_scrollBody');
              $sel.scrollTop(scrollTop);
              $sel.scrollLeft(scrollLeft);
            }
          );
        },
        rowCallback(row, data, displayNum, displayIndex) {
          $(row).attr('data-parent-row-idx', displayIndex);
        },
      };

      function totalDataCalculation(totals) {
        const format = (value, index) => (value === '' ? value : lib.formattedDisplay(formats[index], value));
        return totals.map(
          (total, index) => `<td class="dt-right">${format(total, index)}</td>`
        );
      }

      function totalHeaderBuilder(title, totals, totalData, orderCols) {
        const totalHeader = `${title}${orderCols.length > 2 ? 's' : ''}`;

        if (totals[0] === '') {
          totalData[0] = `<td class="dt-left">${totalHeader}</td>`;
        } else {
          totalData[0] = `<td class="dt-right">${totalHeader} ${totals[0]}</td>`;
        }

        return totalData;
      }

      function getTotalHTML(total, headerCheckbox, title, className) {
        const totals = total;

        if (totals.some((column) => column || column === 0)) {
          const totalData = totalDataCalculation(totals);

          if (headerCheckbox.isChecked) {
            totalHeaderBuilder(title, totals, totalData, orderedColumns);
          }

          return `<tr role="row" class="${className}">${totalData.join(
            ''
          )}</tr>`;
        }
      }

      function handleColumnTotals(tablefoot) {
        const $tfoot = $(tablefoot).parent();
        const api = this.api();
        const columnLength = api.columns().header().length;

        $tfoot.empty();

        try {
          const totals = [];

          if (page_totals.isChecked && enablePagination) {
            const pageTotals = [''];
            for (let i = columnLength > 1 ? 1 : 0; i < columnLength; i++) {
              let hasNumber = false;

              const dtPageTotal = api
                .column(i, { page: 'current' })
                .data()
                .reduce((sum, value) => {
                  const number = Number(value);

                  if (Number.isNaN(number)) {
                    return sum;
                  }

                  if (!hasNumber) {
                    hasNumber = true;
                  }

                  return sum + number;
                }, 0);
              pageTotals.push(hasNumber ? dtPageTotal : '');
            }

            const pageTotalHTML = getTotalHTML(
              pageTotals,
              page_totals_header,
              'Page total',
              'table-page-total-row'
            );
            totals.push(pageTotalHTML);
          }

          if (table_totals.isChecked) {
            const grandTotals = [''];

            for (let i = columnLength > 1 ? 1 : 0; i < columnLength; i++) {
              let hasNumber = false;

              const dtGrantTotal = api
                .column(i)
                .data()
                .reduce((sum, value) => {
                  const number = Number(value);

                  if (Number.isNaN(number)) {
                    return sum;
                  }

                  if (!hasNumber) {
                    hasNumber = true;
                  }

                  return sum + number;
                }, 0);
              grandTotals.push(hasNumber ? dtGrantTotal : '');
            }

            const tableTotalHTML = getTotalHTML(
              grandTotals,
              table_totals_header,
              'Grand total',
              'table-total-row'
            );
            totals.push(tableTotalHTML);
          }

          totals
            .filter((total) => Boolean(total))
            .forEach((total) => $tfoot.append(total));
        } catch (e) {
          table_totals.value('false');
          lib.alertMessage('Total cannot be computed');
        }
      }

      if (enablePagination) {
        dtSettings.paging = true;
        dtSettings.iDisplayLength = Number(pageSize);
        dtSettings.pagingType = 'full_numbers';
      } else {
        dtSettings.paging = true;
        dtSettings.iDisplayLength = -1;
        if (sb_rows.length > pageSizeMax) {
          dtSettings.scroller = {
            displayBuffer: 2,
          };
        }

        // Turn off deferred rendering when we are doing the funky
        // full table expansion for printing thing.
        const css_class = lib
          .user_settings()
          .find_by_id('container_css_class')
          .value();
        if (
          css_class &&
          css_class.includes('arc-print-last-table') &&
          arc.globals.gflags &&
          arc.globals.gflags.is_snapshot
        ) {
          dtSettings.deferRender = false;
          dtSettings.scroller = undefined;
          dtSettings.paging = false;
        }
      }

      const rowsProcessed = new Date();
      if (!disableProfile) {
        arc.perf.PAGEPERF.addProfile({
          id: lib.report_id(),
          type: 'Table rows processed',
          time: rowsProcessed - start,
          profile: [
            ['rows', dtrows.length],
            ['cols', dtrows.length > 0 ? dtrows[0].length : 0],
          ],
        });
      }

      setVizTableStyles();
      setWrapStyle($table);

      table_full_width.$table_element($table).paint_table_style();
      table_header_alignment.$table_element($table);
      table_template
        .$table_element($table)
        .header_alignment(table_header_alignment)
        .paint_table_style();
      table_totals.$table_element($table).paint_table_style(true);
      page_totals.$table_element($table).paint_table_style(true);

      const $visualDataTable =
        $table.data('data-table') || arc.misc.DataTable($table, dtSettings);

      $table.data('data-table', $visualDataTable);

      $($visualDataTable.table().container()).addClass(
        'd-flex flex-column justify-content-start align-items-stretch'
      );

      if (enablePagination) {
        const { pages } = $visualDataTable.page.info();

        if (pages - 1 < currentPage) {
          currentPage = 0;
        }

        $visualDataTable.page(currentPage).draw(false);
        $visualDataTable.off('page.dataTable').on('page.dataTable', () => {
          currentPage = $visualDataTable.page.info().page;
        });
      }

      // Sort of a hack...with our css, the datatables scroller mode results in the page
      // numbers being shown, so here we hide them.
      if (!enablePagination) {
        $(`#${unique_tableid}_wrapper`)
          .find('.dataTables_paginate')
          .closest('.col-sm-7')
          .addClass('arc-invisible');
      }

      function rowTextFn(d, i, elem, for_tooltip) {
        const $elem = $(elem);
        const $tdElem = $elem.closest('td');
        if ($tdElem.length === 0) {
          return;
        }

        const $rowElem = $tdElem.closest('tr');
        const rowindex = $visualDataTable.row($rowElem).index();
        let row = [...sb_rows[rowindex]];
        const colindex = $visualDataTable.column($tdElem).index();

        if (Array.isArray(row)) {
          row = row.map((cell) => (typeof cell === 'string' ? arc.misc.removeHTML(cell).trim() : cell));
        }

        if (parameter_all_cols.isChecked) {
          return lib.base_text_fn(row);
        }

        if (cellsHtml) {
          // unescaped html - do not display within tooltip or context menu
          if (arc.misc.escapeHTML(row[colindex]) !== row[colindex]) {
            return '';
          }
        }

        let tooltipArray = [orderMap[colindex]];
        if (for_tooltip) {
          // The code in this if statement makes it so that we do not
          // show a tooltip when the data being displayed is identical
          // to the tooltip. We only show the tooltip if some of the data
          // was truncated.
          //
          // To make this work, we have to process the display format on
          // the column first, and then run the comparison
          //
          const cols = orderedColumns;
          let coldisplayformat;
          cols.some((c) => {
            if (c.colindex() === orderMap[colindex]) {
              coldisplayformat = tableCellFormat(c);
              return true;
            }

            return false;
          });

          // Now that we have gotten the display format, compute the
          // data display value
          let v = row[orderMap[colindex]];
          if (coldisplayformat) {
            v = coldisplayformat(v);
          }

          /*
           * When we set max width for table cells, we set the following
           * properties on tds:
           *   width: 400px;
           *   max-width: 400px;
           *   overflow: none;
           *   text-overflow: ellipsis;
           *
           * In this situation, checking offsetWidth against scrollWidth
           *   help detect the times when the overflow is in effect and
           *   all the text is not displayed on screen.
           */
          if (Math.abs(elem.offsetWidth - elem.scrollWidth) <= 1) {
            /*
             * Now we know that text overflow is not in effect. Check if what we
             * are showing users is identical to what we will place in the tooltip.
             * If so, do not show the tooltip.
             */
            const text = $elem.text();
            if (v === text || !text) {
              if (firstTooltipIdx === row.length) {
                return '';
              }

              tooltipArray = [];
            }
          }
        }

        return lib.base_text_fn(row, {
          single_colindex: tooltipArray.concat(tooltipIdxs),
        });
      }

      if (showTooltip) {
        const cells = d3.selectAll(
          `#${lib.chart_div()} .dataTables_scrollBody tbody`
        );
        const marks = arc
          .marks()
          .viz_lib(lib)
          .marker_name(show_contextmenu.isChecked ? 'primary' : undefined)
          .highlight_on_select(true)
          .activate(false)
          .addSelection(cells)
          .text_fn((d, i, elem) => rowTextFn(d, i, elem, false))
          .params_fn((d, i, elem) => {
            const $elem = $(elem);
            const $tdElem = $elem.closest('td');
            if ($tdElem.length === 0) {
              return;
            }

            const $rowElem = $tdElem.closest('tr');
            const rowindex = $visualDataTable.row($rowElem).index();
            let row;

            if (parameter_all_cols.isChecked) {
              row = sb_rows[rowindex];
              return lib.params_from_columns(row, orderedColumns, false);
            }

            const colindex = $visualDataTable.column($tdElem).index();
            row = sb_rows[rowindex];
            const single_colindex = colindex;

            const newParams = lib.params_from_columns(
              row,
              orderedColumns,
              false,
              undefined,
              single_colindex
            );

            const newKeys = {};
            newParams.forEach((p) => {
              const k_arr = arc.params.splitParams(p.key);
              if (k_arr[1] === 'ignore') {
                return;
              }

              newKeys[k_arr[0]] = true;
            });

            if (currentParams) {
              currentParams.forEach((p) => {
                // Delete current params no longer in play
                const k_arr = arc.params.splitParams(p.key);
                if (k_arr[1] === 'ignore') {
                  return;
                }

                if (k_arr[0] in newKeys) {
                  return;
                }

                newParams.push({
                  key: `${k_arr[0]}.ignore`,
                  value: false,
                });
              });
            }

            currentParams = newParams;
            return newParams;
          })
          .activate();

        if (marks.tooltips()) {
          marks.tooltips().text_fn((d, i, elem) => rowTextFn(d, i, elem, true));
        }

        const theaders = d3.selectAll(
          `#${lib.chart_div()} .dataTables_scrollHead thead th`
        );
        arc
          .marks()
          .viz_lib(lib)
          .addSelection(theaders)
          .text_fn((d, i) => arc.misc.removeHTML(dtcols[i].desc))
          .activate();
      }

      if (postPaintCb) {
        postPaintCb();
      }

      resizeTracker = arc.visual.resizeTracker(lib, $table);

      resizeTracker
        .setupTracking()
        .isFullWidth(table_full_width.enabled())
        .customWidths(customWidths.slice(0))
        .changeCb(() => {
          customWidths = resizeTracker.customWidths();
        })
        .setSizes();

      return table;
    };

    table.viz_destroy = function () {
      $(`#${lib.chart_div()} .crosstab-expand`).off('.dataTable');
      $(`#${lib.chart_div()} .crosstab-collapse`).off('.dataTable');
      $(`#${lib.chart_div()}`).off('.dataTable');
    };

    return table;
  };

  arc.visual.plugins.table_extension = function () {
    const ext = function () {
      return 'arc.visual.plugins.table_extension';
    };

    ext.viz_name = function () {
      return 'table';
    };

    ext.viz_tooltip = function () {
      return 'table';
    };

    ext.viz_background = function () {
      return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
    };

    ext.viz_background_position = function () {
      return '0 0';
    };

    ext.viz_img_large_src = function () {
      return '../static/img/icon_table.png?v=1';
    };

    ext.viz = arc.visual.plugins.table;

    return ext;
  };

  arc.visual.extensions.push(arc.visual.plugins.table_extension());
}());
