window.arc = window.arc || {};

(function() {
  'use strict';

  arc.visual = arc.visual || {};

  arc.visual.resizeTracker = function(lib) {
    const f = function() { return 'resizeTracker'; };

    let activeElem;
    let $mouseDown;
    let mouseUpCheck;
    let dragStartWidth;
    let dragStartWidthRight;
    let dragStartX;
    let dragLeft;
    let widths;
    let customWidths = [];
    let changeCb;
    let isFullWidth;
    let $thead;
    let $th;

    const chartId = lib.chart_div();

    f.isFullWidth = function(_) {
      if (!arguments.length) return isFullWidth;
      isFullWidth = _;
      return f;
    };

    f.changeCb = function(_) {
      if (!arguments.length) return changeCb;
      changeCb = _;
      return f;
    };

    f.customWidths = function(_) {
      if (!arguments.length) return customWidths;
      customWidths = _ || [];

      for (let i = 0; i < widths.length; ++i) {
        if (customWidths[i]) {
          widths[i] = customWidths[i];
        }
      }

      return f;
    };

    f.setupTracking = function() {
      f.collectSizes();

      $thead = $(`#${chartId} .dataTables_scrollHead table thead`);
      $th = $thead.find(`th`);

      $th.off('mousedown.resize')
        .on('mousedown.resize', function(event) {
          mousedown(event, this);
        });

      $th.off('mouseover.resize')
        .on('mouseover.resize', function(event) {
          mouseover(event, this);
        });

      $th.off('mousemove.resize')
        .on('mousemove.resize', function(event) {
          mousemove(event, this);
        });

      $thead.off('mouseup.resize')
            .on('mouseup.resize', function(event) {
              mouseup(event, this);
            });

      $thead.off('mouseleave.resize')
            .on('mouseleave.resize', function(event) {
              mouseleave(event, this);
            });

      return f;
    };

    f.collectSizes = () => {
      const $head = $(`#${chartId} .dataTables_scrollHead table thead`);

      widths = [];
      $head.find('th')
          .each(function() {
            widths.push($(this).width());
          });
    };

    f.setSizes = function() {
      if (!customWidths || customWidths.length === 0 || !widths || widths.length === 0) {
        return f;
      }

      const $head = $(`#${chartId} .dataTables_scrollHead table`);
      const $body = $(`#${chartId} .dataTables_scrollBody table`);
      const $foot = $(`#${chartId} .dataTables_scrollFoot table`);

      function setSize($elem) {
        const idx = $elem.index();
        const w = widths[idx];
        const px = `${w}px`;

        if (isFullWidth && !customWidths[idx]) {
          return;
        }

        $elem.css('width', px);
        $elem.css('max-width', px);
        $elem.css('min-width', px);
      }

      $body.find('thead tr th, tbody tr td, tfoot tr td')
          .each(function() {
            setSize($(this));
          });

      if (!isFullWidth) {
        let total = 0;
        widths.forEach((w) => {
          total += w;
        });

        $head.css('width', `${total}px`);
        $body.css('width', `${total}px`);
        $foot.css('width', `${total}px`);
      }

      $head.find('thead th')
          .each(function() {
            setSize($(this));
          });

      $foot.find('tfoot tr td')
          .each(function() {
            setSize($(this));
          });

      if (isFullWidth) {
        f.collectSizes();
      }

      return f;
    };

    function activeResize(elem) {
      if (elem === activeElem) {
        return;
      }

      activeElem = elem;

      if (activeElem) {
        $th.css('cursor', 'col-resize');
      } else {
        $th.css('cursor', '');
      }
    }

    function mousedown(event, elem) {
      if (elem !== activeElem) {
        return;
      }

      $mouseDown = $(elem);
      mouseUpCheck = elem;

      const offset = $mouseDown.offset();
      const x = event.pageX - offset.left;
      const w = $mouseDown.outerWidth();

      dragStartX = event.pageX - $thead.offset().left;
      dragLeft = (x < (w / 2));

      if (dragLeft) {
        const idx = $mouseDown.index();
        $mouseDown = $th.eq(idx - 1);
        dragStartWidthRight = $th.eq(idx).width();
      }

      dragStartWidth = $mouseDown.width();
    }

    function mouseup(event, elem) {
      function disableClickOnce(e) {
        console.log('resize disabling sort');
        elem.removeEventListener('click', disableClickOnce, true /* useCapture */);
        e.stopPropagation();
      }

      if ($mouseDown) {
        if (mouseUpCheck === event.target) {
          /*
          * We are handling mouseup on the same event as the earlier mousedown.
          * A click event is going to be sent here, which will sort the column.
          * Add in a one-time event listener in the capture event cycle and disable
          *     the first click on this event so the sort does not happen.
          * Note that we are not using the options structure as the third parameter
          *     to addEventListener so we can support older browsers.
          */
          elem.addEventListener('click', disableClickOnce, true /* useCapture */);
        } else {
          console.log('no match', $mouseDown[0], event.target);
        }
      }

      $mouseDown = undefined;
    }

    function mouseleave() {
      $mouseDown = undefined;
      activeResize(undefined);
    }

    function mousemove(event, elem) {
      if ($mouseDown === undefined) {
        const $elem = $(elem);
        const offset = $elem.offset();
        const x = event.pageX - offset.left;
        const w = $elem.outerWidth();

        if ((x < 15) || ((w - x) < 15)) {
          const onLeft = (x < (w / 2));
          if (onLeft && $elem.index() === 0) {
            activeResize(undefined);
            return;
          }

          activeResize(elem);
        } else {
          activeResize(undefined);
        }

        return;
      }

      const x = event.pageX - $thead.offset().left;
      const diff = x - dragStartX;

      if (diff === 0) {
        // do nothing
        return;
      }

      // change size of currently dragged element
      const newWidth = Math.max(25, dragStartWidth + diff);
      widths[$mouseDown.index()] = newWidth;
      customWidths[$mouseDown.index()] = newWidth;

      if (dragLeft && diff < 0) {
        widths[$mouseDown.index() + 1] = dragStartWidthRight - diff;
        customWidths[$mouseDown.index() + 1] = dragStartWidthRight - diff;
      }

      lib.page_needs_save();
      f.setSizes();

      if (changeCb) {
        changeCb();
      }
    }

    function mouseover(event, elem) {
      return mousemove(event, elem);
    }

    return f;
  };
}());
