window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.removeTimeControl = function() {
  const $c = $('.timecontrol-container');
  $c.empty();

  $('.arc-app-container')
    .off('scroll.timecontrol');

  if (!arc.globals.dashboard) {
    return;
  }

  const filter_controls = arc.globals.dashboard.dashboard_filter_controls();
  if (!filter_controls) {
    return;
  }

  filter_controls.timeControl().remove();
};

function stickyRelocate() {
  var window_top = $('.arc-app-container').scrollTop();
  var app_top = $('.arc-app-container').offset().top;
  var app_width = $('.arc-app-container').width() + 20;
  var div_top = $('.timeline-anchor').offset().top;
  var app_left = $('.arc-app-container').offset().left;
  if (window_top > div_top) {
    $('.arc-app-container .timeline-container')
        .addClass('stick')
        .css('top', app_top)
        .css('width', app_width)
        .css('left', app_left);

    $('.arc-app-container .timeline-anchor')
        .height($('#sticky').outerHeight());
  } else {
    $('.arc-app-container .timeline-container')
        .removeClass('stick');

    $('.arc-app-container .timeline-anchor')
        .height(0);
  }
}

arc.visual.plugins.createTimeControl = function(config) {
  let {
    dataset,
    dashboard,
    settings,
  } = config;

  if (!dataset.validateTimeControl()) {
    const $c = $('.timecontrol-container');
    $c.addClass('arc-invisible');
  }

  const timeControl = arc.visual.plugins.timecontrol(),
        timeControlDriver = arc.report.driver(timeControl)
                               .enableFilterWarnings(false)
                               .report_id(arc.report.REPORTID_TIMECONTROL)
                               .viz_type('timecontrol');

  timeControlDriver.dataset(dataset)
                   .dashboard(dashboard);

  if (settings.user_settings) {
    timeControl.viz_setSettings(settings.user_settings);
  }

  timeControl.create();

  $('.arc-app-container')
      .off('scroll.timecontrol')
      .on('scroll.timecontrol', stickyRelocate);

  stickyRelocate();

  return timeControl;
};

arc.visual.plugins.timecontrol = function() {
  var sort_orders_asc = {},
      saved_settings = {},
      data_result,
      lib,
      savedStart,
      savedEnd,
      brush,
      config;

  const filterColumns = arc.settings.filters.filterColumns(),
        moveStart = arc.settings.timecontrol.moveStartTimeForward(),
        width = arc.settings.timecontrol.width(),
        minDate = arc.settings.timecontrol.minFixedDateTime(),
        maxDate = arc.settings.timecontrol.maxFixedDateTime(),
        bgColor = arc.settings.colors.timeControlBgColor();

  filterColumns.tab('General');

  var user_settings = arc.visual.defaultSettings({
                                  isFilter: true,
                                  noTrellis: true,
                                  noAxes: true,
                                  noTooltip: true,
                                })
                                .addSetting([
                                  filterColumns,
                                  moveStart,
                                  width,
                                  arc.report.user_settings.filter_dataset_scope(),
                                  minDate,
                                  maxDate,
                                  bgColor,
                                ]);

  var timecontrol = arc.visual.api();

  timecontrol.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;

    lib.shelf_data([])
       .user_settings(user_settings);
    return timecontrol;
  };

  timecontrol.viz_getSettings = function() {
    var settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    saved_settings = settings;
    return settings;
  };

  timecontrol.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return timecontrol;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);
    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    return timecontrol;
  };

  timecontrol.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    return timecontrol;
  };

  timecontrol.viz_getDataRequest = function() {
    let timeSettings,
        dim,
        agg,
        dimCol,
        aggCol;

    timeSettings = lib.dataset().timeSettings();
    dim = timeSettings.defaultDateTime;
    agg = timeSettings.defaultAgg;

    dimCol = lib.dataset().findColumn(dim);
    aggCol = lib.dataset().findColumn(agg);

    let sb = lib.getSelectBuilder();
    const rb = lib.getRequestBuilder();

    {
      let order = arc.report.shelf.order()
                     .priority(1)
                     .ascending(true);
      rb.addDimension('[' + dim + ']', order);
    }

    rb.addAggregate(aggCol.attr('expr') || 'sum([' + aggCol.display_name() + '])');

    const dataconnection_type = lib.dataset().dataconnection_type(),
          coltype = dimCol.coltype();

    if (minDate.value().trim() !== '') {
      const v = arc.misc.quoteValue(minDate.value().trim(), coltype, dataconnection_type);
      const f = `[${dim}] >= ${v}`;
      sb.addFilter(f);
      rb.addFilter(f);
    }

    if (maxDate.value().trim() !== '') {
      const v = arc.misc.quoteValue(maxDate.value().trim(), coltype, dataconnection_type);
      const f = `[${dim}] <= ${v}`;
      sb.addFilter(f);
      rb.addFilter(f);
    }

    const filterColumnsData = filterColumns.value().trim();
    if (filterColumnsData) {
      var filterNames = filterColumnsData.split(',');
      lib.handleSpecificFilters(filterNames, function(filterName, expr) {
        sb.addFilter(expr);
        rb.addFilter(expr);
      });
    }

    sb.addGroup(dim);
    sb.addProjection(agg);
    sb = lib.finalizeSql(sb);

    const col1 = arc.data.column()
                        .colname(dimCol.display_name())
                        .colsource('base_shelf')
                        .coltype('string')
                        .colproperty(arc.report.columnTypeEnum.DIMENSION);
    const col2 = arc.data.column()
                        .colname(aggCol.display_name())
                        .colsource('base_shelf')
                        .coltype('int')
                        .colproperty(arc.report.columnTypeEnum.AGGREGATE);

    const data_request = arc.data.request();
    data_request.add_column(col1);
    data_request.add_column(col2);
    data_request.sql(sb.getSql());
    data_request.dsreq(rb.dsreq());
    data_request.finalize();
    return data_request;
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  timecontrol.viz_setDataResult = function(result) {
    data_result = result;
    return timecontrol;
  };

  let currentPlayVersion = 1,
      storedEnableLoadbarsState;

  function startPlay(playConfig) {
    ++currentPlayVersion;
    if (storedEnableLoadbarsState === undefined) {
      storedEnableLoadbarsState = arc.loader.enableLoadbars;
      arc.loader.enableLoadbars = false;
    }

    playConfig.storedPlayVersion = currentPlayVersion;
    play(playConfig);
  }

  function playDone(playConfig) {
    let {
      playDoneCb,
    } = playConfig;

    playDoneCb();
    if (storedEnableLoadbarsState !== undefined) {
      arc.loader.enableLoadbars = storedEnableLoadbarsState;
      storedEnableLoadbarsState = undefined;
    }
  }

  function play(playConfig) {
    let {
      storedPlayVersion,
    } = playConfig;

    if (storedPlayVersion !== currentPlayVersion) {
      return;
    }

    arc.misc.afterCountersEqual({
      counter1: 'dataapi_start',
      counter2: 'dataapi_end',
      cb: function() {
        playNow(playConfig);
      },
    });
  }

  function sanitize(value, domain) {
    let mDomain = domain.map(function(d) {
      return moment.utc(d);
    });

    if (value > mDomain[1]) {
      value = mDomain[1];
    }

    if (value < mDomain[0]) {
      value = mDomain[0];
    }

    return value;
  }

  function playNow(playConfig) {
    let {
      storedPlayVersion,
    } = playConfig;

    const $c = $('.timecontrol-container'),
          $freq = $c.find('.timeline-rate.frequency'),
          $inc = $c.find('.timeline-rate.increment'),
          frequency = $freq.find('.selected').attr('data-value'),
          increment = $inc.find('.selected').attr('data-value');

    if (!increment) {
      return;
    }

    if (storedPlayVersion !== currentPlayVersion) {
      playDone(playConfig);
      return;
    }

    let sel = brush.selection();
    if (!sel) {
      firstSelect(config);
      sel = brush.selection();
    }

    if (!sel) {
      $c.find('.pauseplay').trigger('click');
      return;
    }

    let {
      xValues,
    } = sel;

    let xDomain = config.xScale.domain(),
        xRange = config.xScale.range();

    xValues = xValues.map(function(d) {
      return moment.utc(d);
    });

    xDomain = xDomain.map(function(d) {
      return moment.utc(d);
    });

    if (xValues[1] > xDomain[1]) {
      playDone(playConfig);
      return;
    }

    /*
     * Increment and frequency and both in the "<x> unit" format
     * Examples:
     *   1 seconds
     *   10 years
     *   3 months
     */
    const incrementD = increment.split(' ');

    let done = false;

    let prevStart = xValues[0].clone(),
        prevEnd = xValues[1].clone();

    let nextStart = moveStart.isChecked ?
            xValues[0].clone().add(incrementD[0], incrementD[1]) : xValues[0].clone(),
        nextEnd = xValues[1].clone().add(incrementD[0], incrementD[1]);

    /*
     * The increment could be negative, so we may be moving backwards
     */
    if (nextStart < xDomain[0]) {
      const extraMs = xDomain[0] - nextStart;
      nextStart = xDomain[0];
      nextEnd.add(extraMs, 'milliseconds');
      done = true;
    }

    if (nextEnd > xDomain[1]) {
      const extraMs = nextEnd - xDomain[1];
      nextEnd = xDomain[1];
      nextStart.subtract(extraMs, 'milliseconds');
      done = true;
    }

    nextEnd = sanitize(nextEnd, config.xScale.domain());
    nextStart = sanitize(nextStart, config.xScale.domain());

    let nextRange = [nextStart, nextEnd].map(function(d) {
      return config.xScale(d.toDate());
    });

    brush.select(nextRange);

    if (!done && prevStart.isSame(nextStart) && prevEnd.isSame(nextEnd)) {
      done = true;
    }

    const frequencyD = frequency.split(' ');

    let frequencyCount = frequencyD[0],
        frequencyUnit = frequencyD[1];

    if ((frequencyUnit === 'second') ||
        (frequencyUnit === 'seconds') ||
        (frequencyUnit === 'sec') ||
        (frequencyUnit === 'secs')) {
      if (done) {
        playDone(playConfig);
      } else {
        arc.misc.afterCountersEqual({
          counter1: 'dataapi_start',
          counter2: 'dataapi_end',
          cb: () => {
            setTimeout(function() {
              play(playConfig);
            }, frequencyCount * 1000);
          },
        });
      }
    } else {
      playDone(playConfig);
    }
  }

  timecontrol.pause = function() {
    // The next check against storedPlayVersion will stop any current plays
    ++currentPlayVersion;
  };

  function setWidth() {
    const format = lib && lib.dataset() && lib.dataset().defaultDateTimeFormat(),
          isDate = format === 'YYYY-MM-DD';

    const $c = $('.timecontrol-container');

    let w = $c.width() - 400;
    if (!isDate) {
      // Additional space for date+time display format
      w -= 120;
    }

    if (width.value() !== '') {
      w = Math.min(w, Number(width.value()));
    }

    $('#viz_timecontrol').css('width', w + 'px');
  }

  timecontrol.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    setWidth();
    paintTimeControl();

    config = {};
    config.rows = data_result.rows();
    if (config.rows.length === 0) {
      return timecontrol;
    }

    // Filter out null values in aggregates
    var agg_getter = function(row) {
      return row[1];
    };

    function dim_datechecker(row) {
      if (arc.misc.isValidDate(row[0])) {
        return row[0];
      }

      return undefined;
    }

    lib.filter_nulls(config, [agg_getter, dim_datechecker]);
    if (config.rows.length < 2) {
      return timecontrol;
    }

    lib.unselectOnClick();

    var x_is_dim = true,
        y_is_dim = false;

    config.margin = {
      left: 0,
      top: 0,
      right: 0,
      bottom: 0,
    };

    config.width = $('#' + viz_div).width();
    config.height = $('#' + viz_div).height();

    config.dim_getter = function(row) {
      return row[0];
    };

    config.agg_getter = function(row) {
      return Number(row[1]);
    };

    // Need at least 2 valid dates to keep going
    config.xValues = arc.misc.arrayUnique(config.rows, config.dim_getter);
    config.xExtent = d3.extent(config.xValues)
        .map(function(d) {
          return moment.utc(d).toDate();
        });

    config.xValuesH = arc.misc.arrayToMap(config.xValues);
    config.xScale = d4.scaleTime()
                      .domain(config.xExtent)
                      .range([
                               0,
                               config.width,
                             ]);

    config.yExtent = d3.extent(config.rows, config.agg_getter);
    config.yScale = d3.scale.linear()
                      .domain(config.yExtent)
                      .range([
                               config.height,
                               0,
                             ]);

    drawBaseVisual(config, d3.selectAll('#' + viz_div), config.rows);
    setupPlayPause();
    return timecontrol;
  };

  function setupPlayPause() {
    const $c = $(`.timecontrol-container .pauseplay`);
    $c.off('click')
      .on('click', function() {
        let $this = $(this);
        if ($this.hasClass('paused')) {
          $this.removeClass('paused');
          $this.find('.fa-play')
               .removeClass('fa-play')
               .addClass('fa-pause');
          startPlay({
            playDoneCb: function() {
              $this.addClass('paused');
              $this.find('.fa-pause')
                   .removeClass('fa-pause')
                   .addClass('fa-play');
            },
          });
        } else {
          $this.addClass('paused');
          $this.find('.fa-pause')
               .removeClass('fa-pause')
               .addClass('fa-play');
          timecontrol.pause();
        }
      });
  }

  function firstSelect(config) {
    const timeSettings = lib.dataset().timeSettings(),
          $c = $('.timecontrol-container'),
          $inc = $c.find('.timeline-rate.increment'),
          increment = $inc.find('.selected').attr('data-value');

    const pos = timeSettings.selectionPos || 'end',
          range = timeSettings.selectionRange || increment,
          incrementD = range.split(' '),
          format = lib.dataset().defaultDateTimeFormat(),
          isDate = format === 'YYYY-MM-DD';

    let startDate,
        endDate;

    const domain = config.xScale.domain();

    if (savedStart) {
      startDate = moment.utc(savedStart);
      endDate = moment.utc(savedEnd);

      if (domain[0] > savedStart) {
        startDate = moment.utc(domain[0]);
      } else {
        startDate = moment.utc(savedStart);
      }

      if (domain[1] < savedEnd) {
        endDate = moment.utc(domain[1]);
      } else {
        endDate = moment.utc(savedEnd);
      }
    } else if (pos === 'end') {
      startDate = moment.utc(domain[1]);
      endDate = moment.utc(domain[1]);

      startDate = startDate.subtract(incrementD[0], incrementD[1]);
    } else if (pos === 'start') {
      startDate = moment.utc(domain[0]);
      endDate = moment.utc(domain[0]);

      endDate = endDate.add(incrementD[0], incrementD[1]);
    } else {
      startDate = moment.utc(domain[0]);
      endDate = moment.utc(domain[1]);
    }

    startDate = sanitize(startDate, domain);
    endDate = sanitize(endDate, domain);

    let brushRange = [startDate, endDate].map(function(d) {
      return config.xScale(d.toDate());
    });

    brush.select(brushRange);
  }

  function drawBrush(config, cellConfig) {
    let svg = cellConfig.overlayContainer
        .append('svg')
        .style('position', 'absolute')
        .style('left', '0px')
        .style('top', '0px')
        .style('width', config.width + 'px')
        .style('height', config.height + 'px');

    let start,
        end;

    let disableParams = true;

    brush = arc.visual.timecontrolBrush()
       .svg(svg)
       .xScale(config.xScale)
       .yScale(config.yScale)
       .cb(function(e) {
         const dataset = lib.dataset();
         if (!dataset) {
           return;
         }

         const dimName = dataset.defaultDateTime();
         const format = dataset.defaultDateTimeFormat();
         const $c = $('.timecontrol-container');

         savedStart = e[0];
         savedEnd = e[1];

         start = moment.utc(e[0]).format(format);
         end = moment.utc(e[1]).format(format);

         const $timelineRange = $c.find('.timeline-range');
         $timelineRange.find('.starttime')
                       .text(start);

         $timelineRange.find('.endtime')
                       .text(end);

         $timelineRange.removeClass('arc-invisible');

         if (disableParams) {
           return;
         }

         const substitutedColname = lib.substituteParams(dimName);
         const datasetScopeSetting = lib.user_settings().find_by_id('filter_dataset_scope') ||
           lib.user_settings().find_by_id('visual_dataset_scope');
         const paramConfig = {
           source: {
             baseField: substitutedColname,
             colDataType: String(format).toLowerCase().indexOf('hh') < 0 ?
               arc.params.ParamEnums.DataTypeEnum.DATE :
               arc.params.ParamEnums.DataTypeEnum.DATETIME,
             connectionType: lib.dataconnection_type(),

             entityType: arc.params.ParamEnums.EntityTypeEnum.FILTER,
             entityId: lib.report_id(),
             datasetId: lib.dataset_id(),
           },
           outputParameter: substitutedColname,
           value: [
             {
               start,
               end,
             },
           ],
         };

         const paramObject = new arc.params.DateRangeParam(lib, paramConfig);
         lib.sendParameters([paramObject]);
       })
       .initialize()
       .draw();

    disableParams = false;
    firstSelect(config);

    let brushSel = svg.selectAll('rect.selection');

    arc.marks()
       .viz_lib(lib)
       .addSelection(brushSel)
       .text_changes_on_move(true)
       .text_fn(function() {
         let pre_kvs = [];
         pre_kvs.push({
           key: 'Start Time',
           value: start,
         });

         pre_kvs.push({
           key: 'End Time',
           value: end,
         });

         return lib.text_from_columns([], [], {
           pre_kvs,
         });
       })
       .activate();
  }

  function drawBaseVisual(config, cell_sel, rows) {
    let cellConfig = {
      width: config.width,
      height: config.height,
      margin: config.margin,
      mainCanvasOnly: true,
    };

    arc.canvas.createContainer(cell_sel, cellConfig);
    arc.canvas.createAllCanvas(cellConfig);

    let canvas = cellConfig.mainCanvasConfig.canvas,
        context = canvas.node().getContext('2d');

    let color = lib.colors()[0];

    context.globalAlpha = 0.6;
    context.lineWidth = 2;
    context.strokeStyle = color;
    context.fillStyle = color;
    context.beginPath();

    let yMax = config.yScale.range()[0];

    rows.forEach(function(row, i) {
      var a = config.agg_getter(row),
          dim = config.dim_getter(row),
          x_px = config.xScale(moment.utc(dim).toDate()),
          y_px = config.yScale(a);

      if (i === 0) {
        context.moveTo(x_px, yMax);
      }

      context.lineTo(x_px, y_px);

      if (i === rows.length - 1) {
        context.lineTo(x_px, yMax);
      }
    });

    context.fill();

    drawBrush(config, cellConfig);

    return timecontrol;
  }

  function paintTimeControl() {
    const timeSettings = lib.dataset().timeSettings();
    const agg = timeSettings.defaultAgg;

    const $c = $('.timecontrol-container');
    $c.empty();
    $c.removeClass('arc-invisible');

    $c.append($('#timecontrol-template').html());
    $c.find('.viz_timecontrol')
      .attr('id', 'viz_timecontrol');

    $c.find('.timeline-field')
      .text(agg);

    if (bgColor.value() !== '') {
      $c.find('.timeline-container')
        .css('background-color', bgColor.value());
    }

    const $freq = $c.find('.timeline-rate.frequency'),
          currFreq = timeSettings.frequencies[0];

    $freq.find('ul')
         .empty();
    $freq.find('.selected')
         .attr('data-value', currFreq[1])
         .text(currFreq[0]);

    const $inc = $c.find('.timeline-rate.increment'),
         currInc = timeSettings.increments[0];

    $inc.find('ul')
        .empty();
    $inc.find('.selected')
        .attr('data-value', currInc[1])
        .text(currInc[0]);

    timeSettings.frequencies.forEach(function(f) {
      $freq.find('ul')
           .append(arc.misc.safe_substitute('<li class="dropdown-item"><a class="link" data-value="%VALUE%">%LABEL%</a></li>',
               {
                 LABEL: f[0],
                 VALUE: f[1],
               }));
    });

    $freq.find('ul a')
         .off('click')
         .on('click', function() {
           const $this = $(this);
           $freq.find('.selected')
                .attr('data-value', $this.attr('data-value'))
                .text($this.text());
         });

    timeSettings.increments.forEach(function(f) {
      $inc.find('ul')
          .append(arc.misc.safe_substitute('<li class="dropdown-item"><a class="link" data-value="%VALUE%">%LABEL%</a></li>',
              {
                LABEL: f[0],
                VALUE: f[1],
              }));
    });

    $inc.find('ul a')
        .off('click')
        .on('click', function() {
          const $this = $(this);
          $inc.find('.selected')
              .attr('data-value', $this.attr('data-value'))
              .text($this.text());
        });

    setWidth();

    $c.find('.timecontrol-settings')
      .on('click.settings', function() {
        lib.display_settings();
      });

    $c.find('.fa-remove')
        .offOn('click.remove', function removeControl() {
          arc.visual.plugins.removeTimeControl();
          arc.report.pagenav.needs_save(true);
        });
  }

  timecontrol.create = function() {
    paintTimeControl();

    const dataRequest = timecontrol.viz_getDataRequest();

    lib.runSqlCmd(undefined, function(o) {
      afterData(o);
    }, dataRequest.dsreq());

    return true;
  };

  function afterData(o) {
    const timeSettings = lib.dataset().timeSettings(),
          dim = timeSettings.defaultDateTime,
          agg = timeSettings.defaultAgg;

    let dsCol = arc.data.column()
                   .colname(dim)
                   .coltype('STRING');

    let aggCol = arc.data.column()
                    .colname(agg)
                    .coltype('NUMERIC');

    let result = arc.data.result()
                         .rows(o.rows)
                         .add_column(dsCol)
                         .add_column(aggCol)
                         .finalize();

    timecontrol.viz_setDataResult(result);
    timecontrol.viz_drawVisual('viz_timecontrol');
  }

  return timecontrol;
};

arc.visual.plugins.timecontrol_extension = function() {
  var ext = function() { return 'arc.visual.plugins.timecontrol_extension'; };

  ext.viz_name = function() {
    return 'timecontrol';
  };

  ext.viz_tooltip = function() {
    return 'time control';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_timecontrol.png?v=1';
  };

  ext.viz = arc.visual.plugins.timecontrol;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.timecontrol_extension());

}());
