window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.timeline = function() {
  let saved_settings = {};
  let data_result;
  let lib;
  let trellis;
  let allowLargeTrellisTemp = false;
  let allowLargeTrellisPerm = false;

  const labelPositionSetting = arc.settings.timeline.labelPosition();
  const labelMarginSetting = arc.settings.timeline.labelMargin();
  const legendWidthOverride = arc.report.user_settings.legend_width_override();
  const reverseLegendOrder = arc.report.user_settings.reverse_legend_order();
  const legendStyle = arc.report.user_settings.legend_style();
  legendStyle.addDependents(
    [legendWidthOverride, reverseLegendOrder],
    ['None'],
    ['Right', 'Bottom', 'Above']
  );

  const user_settings = arc.visual.defaultSettings({ noTooltip: true })
                                  .addSetting([
                                    legendStyle,
                                    legendWidthOverride,
                                    reverseLegendOrder,
                                    labelPositionSetting,
                                    labelMarginSetting,
                                  ]);

  arc.addColorSettings(user_settings, {
    includeUseCustomColorSetting: true,
    includeDimensionMapSetting: true,
    includeOpacitySetting: true,
    includeFixLegendSetting: true,
  });

  const timeline = arc.visual.api();

  const shelfData = [
    {
      name: 'starttime_shelf',
      display_name: 'Start Time',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'endtime_shelf',
      display_name: 'End Time',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'ydimension_shelf',
      display_name: 'Y',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'color_shelf',
      display_name: 'Color',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'label_shelf',
      display_name: 'Labels',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  const constraints = {
    minimums: {
      starttime_shelf: 1,
      endtime_shelf: 1,
      ydimension_shelf: 1,
    },
    maximums: {
      starttime_shelf: 1,
      endtime_shelf: 1,
      label_shelf: 1,
    },
  };

  timeline.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelfData)
       .user_settings(user_settings);
    return timeline;
  };

  timeline.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  timeline.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return timeline;
    }

    saved_settings = settings;

    arc.misc.handleLegacyLegendSetting(saved_settings, legendStyle);

    lib.restore_basics(settings, switchingVisualType);

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return timeline;
  };

  timeline.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    const xMessage = [
      lib.createPopoverMessages(['trellisOnly', 'columns.']),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const yMessage = [
      lib.createPopoverMessages(['trellisOnly', 'rows.']),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const startTimeMessages = [
      lib.createPopoverMessages('startTime'),
      lib.createPopoverMessages('acceptsOne'),
    ];
    const endTimeMessages = [
      lib.createPopoverMessages('endTime'),
      lib.createPopoverMessages('acceptsOne'),
    ];
    const colorMessages = [
      lib.createPopoverMessages('acceptsOne', 'acceptsMultipleException'),
    ];
    const yDimensionMessages = [
      lib.createPopoverMessages('yDimension'),
      lib.createPopoverMessages('acceptsMultiple'),
    ];
    const labelMessages = [
      lib.createPopoverMessages('axisLabel', 'acceptsOne'),
    ];

    // Create those popovers
    lib.addShelfDescription(constraints, {
      x_shelf: xMessage,
      y_shelf: yMessage,
      starttime_shelf: startTimeMessages,
      endtime_shelf: endTimeMessages,
      color_shelf: colorMessages,
      ydimension_shelf: yDimensionMessages,
      label_shelf: labelMessages,
    });

    return timeline;
  };

  timeline.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  timeline.viz_setDataResult = function(result) {
    data_result = result;
    return timeline;
  };

  timeline.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return timeline;
  };

  timeline.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return timeline;
  };

  timeline.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    const sbRows = data_result.rows();
    if (!sbRows || sbRows.length === 0) {
      return timeline;
    }

    const config = {
      rows: sbRows,
      columns: data_result.columns(),
    };

    config.columns.forEach((col, colindex) => {
      if (col.colsource() === 'starttime_shelf') {
        config.starttime_idx = colindex;
        return;
      }

      if (col.colsource() === 'endtime_shelf') {
        config.endtime_idx = colindex;
        return;
      }

      if (col.colsource() === 'color_shelf') {
        config.color_idx = colindex;
        return;
      }

      if (col.colsource() === 'label_shelf') {
        config.label_idx = colindex;
      }
    });

    config.level_cols = lib.shelf_columns(config.columns, 'ydimension_shelf');

    config.startTime_getter = function(row) {
      return row[config.starttime_idx];
    };

    config.endTime_getter = function(row) {
      return row[config.endtime_idx];
    };

    config.level_getter = function(row) {
      const levelData = config.level_cols.map((c) => row[c.colindex()]);
      return levelData.join('.').replace(/(\.*)$/, '');
    };

    config.color_getter = function(row) {
      return row[config.color_idx];
    };

    lib.filter_nulls(config, [
                              config.startTime_getter,
                              config.endTime_getter,
                              config.level_getter,
                             ]);

    lib.unselectOnClick();

    lib.setupColors(config);

    const legends = Array.from(
      new Set(
        config.rows
          .map((row) => row[config.color_idx])
          .filter((value) => value !== undefined)
      )
    );

    trellis = arc.visual.trellis()
                     .x_shelf_name('x_shelf')
                     .y_shelf_name('y_shelf')
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select(`#${viz_div}`))
                     .legends(legends)
                     .legendPositioning(legendStyle.value())
                     .color(config.color)
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(() => {
                       allowLargeTrellisTemp = true;
                       timeline.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return timeline;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(() => {
      timeline.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');

    if (legendStyle.legendRequired()) {
      config.trellis.reverse_legend_order(reverseLegendOrder.value());
      config.trellis.legend_width_override(legendWidthOverride.value());
      config.trellis.draw_legend();
    }

    computeSvgParams(config, viz_div);

    config.trellis.draw_visuals((cell_sel, xy_rows) => {
      config.nofilterRows = xy_rows;
      config.rows = xy_rows;
      const cellDiv = cell_sel.attr('id');
      viz_drawVisualWork(config, cellDiv);
    });

    config.trellis.addLegendHighlights();
    lib.redrawLegendIfNeeded(config.trellis);

    return timeline;
  };

  function computeSvgParams(config, viz_div) {
    const { elem_w, elem_h } = lib.computeTrellisElementParams({
      viz_div,
      trellis: config.trellis,
      includeLegend: legendStyle.legendRequired(),
    });

    const r = lib.compute_svg_params({
      show_x_axis: true,
      x_is_dim: true,
      x_is_slanted: true,
      x_axis_label_off: false,
      x_formatter: config.columns[config.starttime_idx].coldisplayformat(true),
      x_idx: config.starttime_idx,
      width: elem_w,
      height: elem_h,
      viz_div_elem: $(`#${viz_div}`),
      columns: config.columns,
      rows: config.rows,
      y_idx: config.level_cols[0].colindex(),
      y_axis_label_off: false,
      y_formatter: config.level_cols[0].coldisplayformat(),
      show_y_axis: true,
    });

    config.margin = r.margin;
    if (config.level_cols.length > 1) {
      config.margin.left = config.margin.left > 60 ? config.margin.left : 60;
    }

    config.width = r.width;
    config.height = r.height;
  }

  function viz_drawVisualWork(config, viz_div) {
    const { margin } = config;

    const $viz = $(`#${viz_div}`);
    const w = config.width;
    let h = config.height;
    const { rows, columns } = config;

    const startColIdx = config.starttime_idx;
    const endColIdx = config.endtime_idx;

    let yScale;
    {
      const levelList = [];
      rows.forEach((r) => {
        levelList.push(config.level_getter(r));
      });

      levelList.push('');

      const yOffsetBetweenLevels = 20;
      const requiredHeight = (levelList.length + 1) * yOffsetBetweenLevels;

      h = Math.min(h, requiredHeight);
      yScale = d3.scale.ordinal()
                 .domain(levelList)
                 .rangePoints([10, h - 10]); // h - 10 to allow vertical bars
    }

    rows.forEach((r) => {
       r.arc_y = yScale(config.level_getter(r));
    });

    const minTime = d3.min(rows, (r) => r[startColIdx]);

    const maxTime = d3.max(rows, (r) => r[endColIdx]);

    const xScale = d3.time.scale()
                   .domain([new Date(minTime), new Date(maxTime)])
                   .range([1, w - 5]);

    $(`#${viz_div} .viz-timeline`).remove();

    const uniqueId = arc.misc.uniqueId();
    const svgId = `svg-timeline-${uniqueId}`;

    const svgContainer =
        d3.select(`#${viz_div}`)
          .append('svg')
          .attr('id', svgId)
          .attr('class', 'viz-timeline')
          .attr('width', w + margin.left + margin.right)
          .attr('height', h + margin.top + margin.bottom)
          .append('g')
          .attr('transform', `translate(${margin.left}, 0)`);

    const $showme = $viz.closest('#showme-modal');
    if ($showme.length === 0) {
      // Do not display a selection brush within the showme modal
      const brush = arc.visual.brush()
                       .svg(svgContainer);
      brush.initialize();

      brush.xScale(xScale)
           .yScale(yScale)
           .cb((bcb) => {
              const filteredRows = rows.filter((d) => {
              const endtime = new Date(config.endTime_getter(d));
              const starttime = new Date(config.startTime_getter(d));
              if (endtime < bcb[0]) {
                return false;
              }

              if (starttime > bcb[1]) {
                return false;
              }

              return true;
              });

              config.rows = filteredRows;
              $viz.closest('.arc-viz-type-timeline')
                              .find('.clear-app')
                              .show();
              $viz.closest('.arc-viz-type-timeline')
                              .find('.viz-params-clear')
                              .removeClass('arc-invisible');
              viz_drawVisualWork(config, viz_div);
           });

      brush.draw();
    }

    const $showapp = $viz.closest('.arc-viz-type-timeline')
                                   .find('.clear-app');
    const $showviz = $viz.closest('.arc-viz-type-timeline')
                                   .find('.viz-params-clear');
    $showapp.off('click.timelineclear')
            .on('click.timelineclear', resetBrush);
    $showviz.off('click.timelineclear')
            .on('click.timelineclear', resetBrush);

    function resetBrush() {
      config.rows = config.nofilterRows;
      $showapp.hide();
      $showviz.addClass('arc-invisible');
      viz_drawVisualWork(config, viz_div);
    }

    const yAxis = d3.svg.axis()
                    .scale(yScale)
                    .orient('left')
                    .tickFormat((tick, idx) => {
                      if (idx === yScale.domain().length - 1) {
                        return '';
                      }

                      return config.level_cols[0].coldisplayformat()(tick);
                    });
    svgContainer.append('g')
      .classed('arc-viz-axis', true)
      .call(yAxis);

    const xAxis = d3.svg.axis()
                        .scale(xScale)
                        .orient('bottom');

    const xAxisDisplayFormat = config.columns[config.starttime_idx].coldisplayformat(true);
    const xAxisConfig = arc.dates.computeTimeAxisConfig({
      domain: xAxis.scale().domain(),
      totalLen: w,
      fontSize: 12,
      customFormat: xAxisDisplayFormat,
      isDateTime: true,
    });

    if (xAxisConfig.tickFunction) {
      xAxis.ticks(xAxisConfig.tickFunction);
    }

    xAxis.tickFormat(xAxisDisplayFormat || d3.time.format(xAxisConfig.tickFormat));

    const xAxisSVG = svgContainer.append('g')
                                .attr('transform', `translate(0,${h - 10})`)
                                .classed('arc-viz-axis', true)
                                .call(xAxis);

    /*
    * Date axes sometimes draw a tick mark outside the max length of the axis.
    * Delete any tick outside the axis.
    */
    const maxRange = xAxis.scale().range()[1];
    xAxisSVG.selectAll('.tick')
      .each(function(d) {
        const v = xAxis.scale()(d);
        if (v > maxRange) {
          $(this).remove();
        }
      });

    xAxisSVG.selectAll('text')
      .attr('transform', `translate(-10,0) rotate(${lib.x_slant()})`)
      .style('text-anchor', 'end');

    const svg = svgContainer.append('g');

    const fill = lib.dimensionalColorScale();
    fill.range(lib.colors());

    function getColor(d) {
      return fill(config.color_getter(d));
    }

    function widthScale() {
        return '3';
    }

    const marks =
        svg.selectAll('.nosel')
          .data(rows)
          .enter()
          .append('line')
          .attr('stroke', getColor)
          .attr('stroke-opacity', lib.colorOpacity())
          .attr('stroke-width', widthScale)
          .attr('x1', (d) => xScale(new Date(d[startColIdx])))
          .attr('x2', (d) => xScale(new Date(d[endColIdx])))
          .attr('y1', (d) => d.arc_y)
          .attr('y2', (d) => d.arc_y);

    if (config.label_idx) {
      const fontSize = lib.getLabelFontSize() || 10;
      const fontColor = lib.getLabelColor() || 'grey';
      const labelPosition = labelPositionSetting.value();
      const labelMargin = Number(labelMarginSetting.value());

      const prev = [];
      svg.selectAll('.nosel')
        .data(rows)
        .enter()
        .append('text')
        .attr('text-anchor', labelPosition)
        .attr('x', (d) => {
          if (labelPosition === 'start') {
            return xScale(new Date(d[startColIdx]));
          }

          if (labelPosition === 'end') {
            return xScale(new Date(d[endColIdx]));
          }

          return ((xScale(new Date(d[startColIdx])) +
                    xScale(new Date(d[endColIdx]))) / 2);
        })
        .attr('y', (d) => d.arc_y + labelMargin)
        .style('font-size', `${fontSize}px`)
        .style('fill', fontColor)
        .text((d) => d[config.label_idx])
        .each(function() {
          const { width, height, x, y } = this.getBoundingClientRect();

          const overlap = prev.some((l) => {
            if (x > (l.x + l.w) || (x + width) < l.x ||
                y > (l.y + l.h) || (y + height) < l.y) {
              return false;
            }

            return true;
          });

          if (overlap) {
            $(this).hide();
          } else {
            prev.push({
              x,
              y,
              w: width,
              h: height,
            });
          }
        });
    }

    const barLength = 5;
    const startEndRows = [];
    rows.forEach((r) => {
      startEndRows.push(r);
      startEndRows.push(r);
    });

    const startEndMarks = svg.selectAll('.nosel')
      .data(startEndRows)
      .enter()
      .append('rect')
      .attr('fill', getColor)
      .attr('stroke', getColor)
      .attr('stroke-opacity', lib.colorOpacity)
      .attr('stroke-width', widthScale)
      .attr('x', (d, i) => {
        let x;
        if (i % 2 === 0) {
          x = xScale(new Date(d[startColIdx]));
        } else {
          x = xScale(new Date(d[endColIdx])) - 2;
        }

        return x;
      })
      .attr('y', (d) => d.arc_y - barLength)
      .attr('width', 2)
      .attr('height', barLength * 2);

    columns[config.endtime_idx].setDisplayFormat(xAxisDisplayFormat);
    arc.marks()
       .viz_lib(lib)
       .addSelection(marks)
       .addSelection(startEndMarks)
       .legendValueGetter((d) => d[config.color_idx])
       .text_fn((d) => lib.text_from_columns(d, columns))
       .params_fn((d) => lib.params_from_columns(d, columns))
       .highlight_on_select(true)
       .marker_name('primary')
       .activate();

    return timeline;
  }

  return timeline;
};

arc.visual.plugins.timeline_extension = function() {
  const ext = function() { return 'arc.visual.plugins.timeline_extension'; };

  ext.viz_name = function() {
    return 'timeline';
  };

  ext.viz_tooltip = function() {
    return 'timeline';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_timeline.png?v=1';
  };

  ext.viz = arc.visual.plugins.timeline;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.timeline_extension());
}());
