window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.treemap = function() {
  var treemap = arc.visual.plugins.chord('treemap');

  let rootStr = 'arc_root';

  function rowsToHierarchy(config) {
    let {
      rows,
      dim_startidx,
      dim_endidx,
      agg_startidx,
    } = config;

    let data = {
      name: rootStr,
      childrenHash: {},
      children: [],
    };

    rows.forEach(function(row) {
      let curr = data;

      for (let i = dim_startidx; i < dim_endidx; ++i) {
        let id = row[i];

        let node = curr.childrenHash[id];

        if (node === undefined) {
          node = {
            name: String(id),
            childrenHash: {},
            children: [],
          };

          curr.childrenHash[id] = node;
          curr.children.push(node);
        }

        curr = node;
      }

      let id = row[dim_endidx];
      curr.children.push({
        name: String(id),
        allData: row,
        size: row[agg_startidx],
      });
    });

    function sumBySize(d) {
      return d.size;
    }

    let root = d4.hierarchy(data)
        .eachBefore(function(d) {
          d.data.id = (d.parent ? d.parent.data.id + '.' : '') + d.data.name;
        })
        .sum(sumBySize)
        .sort(function(a, b) {
          return b.height - a.height || b.value - a.value;
        });

    return root;
  }

  treemap.viz_drawVisualTreemap = function(config, viz_div) {
    let {
      width,
      height,
      rows,
      columns,
      color,
      dim_startidx,
      dim_endidx,
      agg_startidx,
      trellis,
    } = config;

    if (rows.length === 0) {
      return treemap;
    }

    let trellisParams = trellis.getFixedParams(rows[0]);

    let lib = treemap.viz_lib();

    let svg = d4.select('#' + viz_div)
                .append('svg')
                .attr('width', width)
                .attr('height', height)
                .append('g');

    let root = rowsToHierarchy(config);

    let opacity = lib.colorOpacity();
    d4.treemap()
      .tile(d4.treemapResquarify)
      .size([width, height])
      .round(true)
      .paddingInner(1)
      (root);

    let cell = svg.selectAll('g')
        .data(root.leaves())
        .enter().append('g')
          .attr('transform', function(d) { return 'translate(' + d.x0 + ',' + d.y0 + ')'; });

    root.leaves().forEach(function(d) {
      d.color = color(d.parent.data.id.substring(rootStr.length + 1));
    });

    cell.append('rect')
        .attr('id', function(d) { return d.data.id; })
        .attr('width', function(d) { return d.x1 - d.x0; })
        .attr('height', function(d) { return d.y1 - d.y0; })
        .attr('fill', function(d) {
          return d.color;
        })
        .attr('fill-opacity', opacity);

    let texts = cell.append('text')
                    .style('fill', function(d) {
                      return arc.colorpalette.colorForBrightness(d.color, opacity);
                    });

    texts.selectAll('tspan')
         .data(function(d) {
           return d.data.name.split(/\s+/g);
         })
         .enter()
           .append('tspan')
           .attr('x', 4)
           .attr('y', function(d, i) { return 15 + i * 15; })
           .text(function(d) { return d; });

    texts.filter(function(d) {
            let bbox = this.getBBox();
            let width = bbox.width;
            let height = bbox.height;
            return (width > d.x1 - d.x0 - 4) || (height > d.y1 - d.y0);
          })
          .remove();

    function getKVs(d) {
      let params = trellisParams.slice(0);

      let dims = [];
      let i = dim_endidx;
      let curr = d;
      while (i >= dim_startidx) {
        if (curr.data.name === rootStr) {
          break;
        }

        dims = lib.param_from_column(columns[i], curr.data.name).concat(dims);
        curr = curr.parent;
        --i;
      }

      dims.forEach(function(d) {
        params.push(d);
      });

      for (let idx = agg_startidx; idx < columns.length && d.data.allData; idx++) {
        params = params.concat(lib.param_from_column(columns[idx], d.data.allData[idx]));
      }

      return params;
    }

    arc.marks()
       .viz_lib(lib)
       .marker_name('primary')
       .highlight_on_select(true)
       .addSelection(cell)
       .text_changes_on_move(true)
       .text_fn(function(d) {
         return lib.text_from_columns([], [], {
           pre_kvs: getKVs(d),
         });
       })
       .params_fn(function(d, i, e, dimsOnly) {
         return getKVs(d);
       })
       .activate();
  };

  return treemap;
};

arc.visual.plugins.treemap_extension = function() {
  var ext = function() { return 'arc.visual.plugins.treemap_extension'; };

  ext.viz_name = function() {
    return 'treemap';
  };

  ext.viz_tooltip = function() {
    return 'treemap';
  };

  ext.viz_background = function() {
    return 'url(../../static/img/chart-icons.png?v=6) top no-repeat';
  };

  ext.viz_background_position = function() {
    return '0 0';
  };

  ext.viz_img_large_src = function() {
    return '../static/img/icon_treemap.png?v=1';
  };

  ext.viz = arc.visual.plugins.treemap;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.treemap_extension());

}());
