import cloud from 'd3-cloud';

window.arc = window.arc || {};

(function() {
"use strict";

arc.visual = arc.visual || {};
arc.visual.plugins = arc.visual.plugins || {};
arc.visual.extensions = arc.visual.extensions || [];

arc.visual.plugins.wordcloud = function() {
  var saved_settings = {},
      data_result,
      lib,
      sort_orders_asc = {},
      allowLargeTrellisTemp = false,
      allowLargeTrellisPerm = false;


  var tokenizeInput = arc.settings.wordcloud.tokenizeInput();
  var user_settings = arc.visual.defaultSettings({ noAxes: true })
                                .addSetting([
                                  tokenizeInput,
                                ]);

  arc.addColorSettings(user_settings, {
    includeUseCustomColorSetting: true,
    includeDimensionMapSetting: true,
    includeOpacitySetting: true,
    includeFixLegendSetting: true,
  });

  var wordcloud = arc.visual.api();

  var shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimension',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measure',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'color_shelf',
      display_name: 'Colors',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'x_shelf',
      display_name: 'X Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'y_shelf',
      display_name: 'Y Trellis',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    },
  ];

  var constraints = {
    minimums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
    },
    maximums: {
      dimensions_shelf: 1,
      aggregates_shelf: 1,
      color_shelf: 1,
    }
  };

  wordcloud.viz_lib = function(_) {
    if (!arguments.length) return lib;
    lib = _;
    lib.shelf_data(shelf_data)
       .user_settings(user_settings);
    return wordcloud;
  };

  wordcloud.viz_getSettings = function() {
    let settings = saved_settings;
    settings = lib.save_basics(settings);
    settings.sort_orders_asc = sort_orders_asc;
    settings.allowLargeTrellisPerm = allowLargeTrellisPerm;
    saved_settings = settings;
    return settings;
  };

  wordcloud.viz_setSettings = function(settings, switchingVisualType) {
    if (settings === undefined) {
      return wordcloud;
    }

    saved_settings = settings;
    lib.restore_basics(settings, switchingVisualType);

    if ('sort_orders_asc' in saved_settings) {
      sort_orders_asc = saved_settings.sort_orders_asc;
    }

    if ('allowLargeTrellisPerm' in saved_settings) {
      allowLargeTrellisPerm = saved_settings.allowLargeTrellisPerm;
    }

    return wordcloud;
  };

  wordcloud.viz_drawDesigner = function(shelves_div) {
    lib.shelves_dom_id(shelves_div)
       .base_setup();

    // Define messages to be included in shelves
    var xMessage = [
          lib.createPopoverMessages(['trellisOnly', 'columns.']),
          lib.createPopoverMessages('acceptsMultiple')
        ],
        yMessage = [
          lib.createPopoverMessages(['trellisOnly', 'rows.']),
          lib.createPopoverMessages('acceptsMultiple')
        ],
        dimensionMessages = [
          lib.createPopoverMessages('dimensionWordCloud'),
          lib.createPopoverMessages('acceptsOne')
        ],
        aggregateMessages = [
          lib.createPopoverMessages('aggregateWordCloud'),
          lib.createPopoverMessages('acceptsOne')
        ],
        colorMessages = [
          lib.createPopoverMessages('wordColor'),
          lib.createPopoverMessages('acceptsOne'),
        ];

    // Create those popovers
    lib.addShelfDescription(constraints,{
      x_shelf: xMessage,
      y_shelf: yMessage,
      dimensions_shelf: dimensionMessages,
      aggregates_shelf: aggregateMessages,
      color_shelf: colorMessages,
    });

    return wordcloud;
  };

  wordcloud.viz_getDataRequest = function() {
    return lib.getCheckedDataRequest(constraints);
  };

  /*
   * Set the data this object should use to draw out the visual
   */
  wordcloud.viz_setDataResult = function(result) {
    data_result = result;
    return wordcloud;
  };

  wordcloud.allowLargeTrellisPerm = function(_) {
    if (!arguments.length) return allowLargeTrellisPerm;
    allowLargeTrellisPerm = _;
    return wordcloud;
  };

  wordcloud.allowLargeTrellisTemp = function(_) {
    if (!arguments.length) return allowLargeTrellisTemp;
    allowLargeTrellisTemp = _;
    return wordcloud;
  };

  wordcloud.viz_drawVisual = function(viz_div) {
    lib.chart_div(viz_div);
    lib.undraw();

    let sb_rows = data_result.rows();
    if (!sb_rows || sb_rows.length === 0) {
      return wordcloud;
    }

    let config = {
      rows: data_result.rows(),
      columns: data_result.columns(),
    };

    config.agg_col = lib.first_column(config.columns, 'aggregates_shelf');
    config.agg_idx = config.agg_col.colindex();

    config.color = function() {
      return base.colors()[0];
    };

    // Filter rows with NULLs in the aggregate columns
    let agg_getter = function(row) {
      return Number(row[config.agg_idx]);
    };

    config.agg_getter = agg_getter;

    config.tokenizeInput = tokenizeInput.isChecked;
    lib.filter_nulls(config, [agg_getter]);
    lib.unselectOnClick();

    var cols = lib.shelf_columns(config.columns, 'dimensions_shelf');
    config.dim_start_idx = cols[0].colindex();

    lib.setupColors(config);
    if (config.color_idx < 0) {
      config.color_idx = config.dim_start_idx;
    }

    var trellis = arc.visual.trellis()
                     .x_shelf_name("x_shelf")
                     .y_shelf_name("y_shelf")
                     .x_shelf_reserved(0)
                     .y_shelf_reserved(0)
                     .rows(config.rows)
                     .sort_orders_asc(sort_orders_asc)
                     .columns(config.columns)
                     .viz_lib(lib)
                     .viz_div_sel(d3.select("#" + viz_div))
                     .ignoreLargeTrellis(allowLargeTrellisPerm || allowLargeTrellisTemp)
                     .redrawVisualCb(function() {
                       allowLargeTrellisTemp = true;
                       wordcloud.viz_drawVisual(viz_div);
                     })
                     .setup();

    if (trellis.error_state()) {
      return wordcloud;
    }

    config.trellis = trellis;
    config.trellis.draw_skeleton(function() {
      wordcloud.viz_drawVisual(viz_div);
    });

    config.trellis.tooltips('trellis');
    computeSvgParams(config, viz_div);

    config.trellis.draw_visuals(function(cell_sel, xy_rows) {
      config.rows = xy_rows;
      var cell_div = cell_sel.attr('id');
      viz_drawVisualWork(config, cell_div);
    });

    return wordcloud;
  };

  function computeSvgParams(config, viz_div) {
    let { elem_w, elem_h } = lib.computeTrellisElementParams({
      trellis: config.trellis,
      viz_div,
    });

    config.width = elem_w;
    config.height = elem_h;

    config.diameter = Math.min(elem_w, elem_h);
    config.diameter -= 10;

    config.diameter = Math.max(config.diameter, 200);
  }

  function viz_drawVisualWork(config, viz_div) {
    const excludedWords = new Set([
      'FROM', 'BY', 'SET', 'HOW', 'MY', 'ME', 'WILL', 'THAN', 'HAD', 'ARE', 'THEY', 'THERE',
      'I', 'OR', 'OUR', 'AN', 'DO', 'IT\'S', 'GOT', 'UP', 'HIS', 'HER', 'WHICH', 'SHE', 'HE',
      'MADE', 'YOU', 'WE', 'WERE', 'SO', 'IT', 'WHERE', 'THAT', 'CAN', 'AS', 'WAS', 'AND', 'IF',
      'AT', 'WITH', 'IS', 'OF', 'TO', '', '&', 'ON', 'THE', 'IN', 'BE', 'FOR', 'A', 'GET', 'ALSO',
      'THIS', 'YOUR', 'ALL', 'VERY', 'WHO', 'HAS', 'JUST', 'ONCE', 'HAVE', 'NULL', 'UNDEFINED', 'NAN',
      'NOT', 'HIM', 'DID', 'BUT', 'THEIR', 'THEN'
    ]);

    var words = {};
    if (config.rows.length === 0) {
      return;
    }

    var sb_rows = config.rows;
    var agg_getter_local = config.agg_getter;
    const wordIdx = config.dim_start_idx;
    let textFormatter = config.columns[wordIdx].coldisplayformat();
    let colorCol = config.columns[config.color_idx];
    let colorIsNumeric = colorCol.isNumeric();

    sb_rows.forEach(function(row) {
      let descString = textFormatter(row[wordIdx]);
      descString = String(descString).toUpperCase().replace(/[^A-Z0-9]/g, ' ');
      var descArray;
      if (config.tokenizeInput) {
        descArray = descString.split(' ');
      } else {
        descArray = descString.split();
      }
      descArray.forEach(function(d) {
        if (!excludedWords.has(d)) {
          words[d] = words[d] ? words[d] : [0, 0];
          const [size, color] = words[d];
          words[d][0] = size + agg_getter_local(row);
          if (colorIsNumeric) {
            words[d][1] = color + row[config.color_idx];
          } else {
            words[d][1] = row[config.color_idx];
          }
          if (config.tokenizeInput) {
            words[d][2] = row.slice(0);
            words[d][2][config.dim_start_idx] = d;
            words[d][2][config.agg_idx] = words[d][0];
          } else {
            words[d][2] = row;
          }
        }
      });
    });

    const l = Object.keys(words)
                    .map(k => ({text: k, size: words[k][0], color: words[k][1], data: words[k][2]}))
                    .sort((a, b) => b.size - a.size)
                    .slice(0, 200);

    let maxwordlen = l[0].size;
    let maxNum = Math.max(config.diameter / (maxwordlen * 2), 40);
    let m = d3.max(l, function(c) {
      return c.size;
    });
    let scale = maxNum / m;
    l.forEach(function (c) {
      c.size = Math.floor(c.size * scale);
    });

    var $chart = $('#' + viz_div);

    const opacity = lib.colorOpacity();

    const layout = cloud()
        .size([config.diameter, config.diameter])
        .words(l)
        .padding(2)
        .random(() => 0.5)
        .font('Impact')
        .fontSize((d) => d.size)
        .spiral('archimedean')
        .timeInterval(1)
        .on('end', draw);

    layout.start();

    function draw(words, bounds) {

      var svgg = d3.select(`#${viz_div}`)
                   .append('svg')
                   .attr('width', layout.size()[0])
                   .attr('height', layout.size()[1]);

      var svgint = svgg.append('g');

      svgint.selectAll('text')
            .data(words)
            .enter()
            .append('text')
            .style('font-size', d => `${d.size}px`)
            .style('font-family', 'Impact')
            .style('fill', d => config.color(d.color))
            .style('opacity', opacity)
            .attr('text-anchor', 'middle')
            .attr('transform', d => `translate(${[d.x, d.y]})rotate(${d.rotate})`)
            .text(d => d.text);

      var bbox = svgg.node() ? svgg.node().getBBox() : undefined;
      let scalesvg = 1;
      if (bbox && bbox.width !== 0 && bbox.height !== 0) {
        let scalesvgx = layout.size()[0]/bbox.width;
        let scalesvgy = layout.size()[1]/bbox.height;
        scalesvg = Math.min(scalesvgx, scalesvgy);
      }

      if (bbox) {
        let xtranslate = bbox.width/-2 - bbox.x + layout.size()[0]/2;
        let ytranslate = bbox.height/-2 - bbox.y + layout.size()[1]/2;
        svgint.attr('transform', `translate(${xtranslate},${ytranslate})scale(${scalesvg})`);
      }

      const allWords = d3.selectAll(`#${viz_div} text`);

      arc.marks().viz_lib(lib)
                  .marker_name('words')
                  .highlight_on_select(true)
                  .addSelection(allWords)
                  .text_fn(d => lib.text_from_columns(d.data, config.columns))
                  .params_fn(d => lib.params_from_columns(d.data, config.columns))
                  .activate();
    }

    return wordcloud;
  }

  return wordcloud;
};

arc.visual.plugins.wordcloud_extension = function() {
  var ext = function() { return "arc.visual.plugins.wordcloud_extension"; };

  ext.viz_name = function() {
    return "wordcloud";
  };

  ext.viz_tooltip = function() {
    return "wordcloud";
  };

  ext.viz_background = function() {
    return "url(../../static/img/chart-icons.png?v=6) top no-repeat";
  };

  ext.viz_background_position = function() {
    return "0 0";
  };

  ext.viz_img_large_src = function() {
    return "../static/img/icon_wordcloud.png?v=1";
  };

  ext.viz = arc.visual.plugins.wordcloud;

  return ext;
};

arc.visual.extensions.push(arc.visual.plugins.wordcloud_extension());

}());
