window.arc = window.arc || {};

(function() {
'use strict';

arc.utils = arc.utils || {};

  const connectionTypeToMode = {
    arcengine: 'impala_arcengine',
    arcengine_v2: 'impala_arcengine',
    bigquery: 'bigquery',
    sparksql: 'hive',
    hive: 'hive',
    hyve: 'hive',
    impala: 'impala',
    impyla: 'impala',
    sqlite: 'mysql_sqlite',
    duckdb: 'mysql_sqlite',
    druid: 'mysql',
    mysql: 'mysql',
    mariadb: 'mysql',
    postgres: 'pgsql',
    redshift: 'pgsql',
    mongodb: 'sql', // not sure what to use
    odbc: 'sql',
    aster: 'pgsql',
    solr: 'solr',
    drill: 'mysql',
    oracle: 'mysql',
    ksql: 'ksql',
    vertica: 'pgsql',
    denodo: 'pgsql',
    elasticsearch: 'elasticsearch',
  };

  arc.utils.modeForConnectionType = function (connectionType) {
    if (connectionTypeToMode[connectionType]) {
      return connectionTypeToMode[connectionType];
    }

    return 'sql'; // default is "sql"
  };

// INITIALIZES ACE EDITOR
// Parameters:
//  editorId: The id for editable div
//  mode: The type of highlighting/syntax to use
//  config: Optional configs composed of:
//    autocompleteType: OPTIONAL string that is used to differentiate autocomplete on/off
//      setting from default
//    extraAutocompletes: an array of extra autocomplete objects -- needs a value and meta
//      property.  The string "column" will add all columns from the dataset
//    includeCalculated: boolean if calculated columns are to be included in extraAutocompletes
//    excludeRecordCount: boolean if record count is to excluded from column completers
//      Used in conjunction with extraAutocmpletes: 'column' and 'tablesAndColumns'
//    excludeCompleter: an array of completer names that should be excluded.
//      If there are no excludes, and extraAutocompletes is set to "column", local
//      autocompletes will be omitted
//    tables: an array of tables within a dataset
//      Used in conjunction with extraAutocompletes: "column".
//    useRawNames: boolean if the raw tablename.columnname should be used as autocomplete and
//      snippet in place of [column alias]  Used when extraAutocompletes is set to "column"
//      (default behavior is false)
//    snippetSidebars: an array of selectList items
//      if there's an array of selectList items, will reset items upon aceEditor.focusEditor()
//    autocompleteOnOffButton: selector or jquery object
//      if there's a selector, will display an on/off checkbox
//    hideAutocompleteOnOffButtonLabel: true or false (by default is false)
//    aceOptions: a hash of ace options to pass in upon initialization
arc.utils.autocompleteEditor = function(editorId, mode, config) {
  let identifierRegexps;
  const aceEditor = ace.edit(editorId);
  let resizeObserver;
  const aceLanguageTools = ace.require('ace/ext/language_tools');
  const { extraAutocompletes } = config;
  const isSQLQueryEditor = ['columns', 'tablesAndColumns'].includes(extraAutocompletes);
  const defaultCompleters = ['snippetCompleter', 'textCompleter', 'keyWordCompleter'];

  const aceConfig = {
    enableBasicAutocompletion: true, // Basic mode autocomplete
    enableSnippets: true, // Autocomplete snippets -- essential
    enableMultiselect: true, // Allow users to change multiple places at once
    wrap: true, // Wrap line
    enableLiveAutocompletion: true, // Autocomplete suggestions on type
    showGutter: isSQLQueryEditor, // Numbers in margins for sql editor
  };


  try {
    ace.require('ace/ext/settings_menu').init(aceEditor);
  } catch (err) {
    console.log('Could not fetch ace settings');
  }

  config = config || {};
  let fieldMappings = {};
  let tableMappings = {};
  let tableRawMappings = {};

  const autocompleteEditor = function() { return 'arc.utils.autocompleteEditor'; };

  autocompleteEditor.aceLanguageTools = function() {
    return aceLanguageTools;
  };

  autocompleteEditor.aceEditor = function() {
    return aceEditor;
  };

  autocompleteEditor.getSession = function() {
    return aceEditor.getSession();
  };

  autocompleteEditor.config = function() {
    return config;
  };

  autocompleteEditor.fieldMappings = function(_) {
    if (!arguments.length) return fieldMappings;
    fieldMappings = _;
    return autocompleteEditor;
  };

  autocompleteEditor.tableMappings = function(_) {
    if (!arguments.length) return tableMappings;
    tableMappings = _;
    return autocompleteEditor;
  };

  autocompleteEditor.tableRawMappings = function(_) {
    if (!arguments.length) return tableRawMappings;
    tableRawMappings = _;
    return autocompleteEditor;
  };

  autocompleteEditor.enableResize = function() {
    if (window.ResizeObserver) {
      $(aceEditor.container).addClass('resizable');
      if (resizeObserver) {
        // almost never should occur to recall this method again, but I leave it for safety
        resizeObserver.disconnect();
      } else {
        resizeObserver = new ResizeObserver(() => {
          aceEditor.resize();
        });
      }

      resizeObserver.observe(aceEditor.container);
    } else {
      // eslint-disable-next-line
      console.warn("SQL Editor can't be resized, this browser doesn't support ResizeObservable.");
    }
  };

  autocompleteEditor.destroy = function() {
    aceEditor.destroy();
    resizeObserver?.disconnect();
  };

  autocompleteEditor.tables = function(_) {
    if (!arguments.length) return config.tables;
    config.tables = _;
    autocompleteEditor.setAutocompletes();
    return autocompleteEditor;
  };

  autocompleteEditor.autocompleteOnOffButton = function(_) {
    if (!arguments.length) return !!config.autocompleteOnOffButton;
    if (config.autocompleteOnOffButton &&
        !$(config.autocompleteOnOffButton).find('input[type=checkbox]').length) {
      $(config.autocompleteOnOffButton).empty();
    }

    config.autocompleteOnOffButton = _;
    autocompleteEditor.configAutocompleteOnOffButton();
    return autocompleteEditor;
  };

  autocompleteEditor.extraAutocompletes = function(_) {
    if (!arguments.length) return config.extraAutocompletes;
    config.extraAutocompletes = _;
    autocompleteEditor.setAutocompletes();
    return autocompleteEditor;
  };

  autocompleteEditor.snippetSidebars = function(_) {
    if (!arguments.length) return config.snippetSidebars;
    config.snippetSidebars = _;
    return autocompleteEditor;
  };

  autocompleteEditor.excludeAutocomplete = function(_) {
    if (!arguments.length) return config.excludeAutocomplete;
    config.excludeAutocomplete = config.excludeAutocomplete || [];
    config.excludeAutocomplete = config.excludeAutocomplete.concat(_);
    autocompleteEditor.setAutocompletes();
    return autocompleteEditor;
  };

  // For non-data connections, the passed in "mode" is the actual mode
  // (eg: css and js is the "mode" and doesn't need to be converted)
  // For data connections, mode is converted to available mode type
  function getMode() {
    if (isSQLQueryEditor) {
      return arc.utils.modeForConnectionType(mode);
    }

    return mode;
  }

  autocompleteEditor.mode = function(_) {
    if (!arguments.length) {
      return getMode();
    }

    mode = _;
    aceEditor.session.setMode(`ace/mode/${getMode()}`);
    return autocompleteEditor;
  };

  // EDITOR-SPECIFIC Functions

  // Get an array of current completers + any extraAutocompletes
  autocompleteEditor.getCompleters = function() {
    let completers = [];
    if (config.extraAutocompletes) {
      completers = [
        {
          identifierRegexps,
          getCompletions(editor, session, pos, prefix, callback) {
            if (prefix.length === 0) {
              callback(null, []);
              return;
            }

            const fieldsToShow = config.isSensitiveShelf ?
              config.extraAutocompletes.filter((field) => !field.isSensitive) :
              config.extraAutocompletes;

            callback(null, fieldsToShow);
          },
        },
      ];
    }

    defaultCompleters.forEach(function (c) {
      if (!config.excludeAutocomplete ||
          config.excludeAutocomplete.indexOf(c) === -1) {
        if (aceLanguageTools && aceLanguageTools[c]) {
          completers.push(aceLanguageTools[c]);
        }
      }
    });

    return completers;
  };

  // Sets the value and moves cursor to the end of the text
  // or returns trimmed editor text
  autocompleteEditor.value = function(_) {
    if (!arguments.length) return aceEditor.getValue().trim();
    aceEditor.setValue(_, 1);
    return autocompleteEditor;
  };

  // Should be called whenever changing editor text --
  // Updates the editor text and focuses on editor
  autocompleteEditor.focusEditor = function(resetSidebars=true) {
    if (config.snippetSidebars && resetSidebars) {
      config.snippetSidebars.forEach(function (widget) {
        widget.clickableList('selectFirst');
      });
    }

    aceEditor.resize();
    aceEditor.focus();
  };

  autocompleteEditor.resize = function() {
   aceEditor.resize();
 };

  // Get extraAutocompletes for columns
  autocompleteEditor.getColumnAutocompletes = function(includeTables=false) {
    config.extraAutocompletes = [];
    if (!config.excludeRecordCount) {
      config.extraAutocompletes.push({
        value: '[Record Count]',
        snippetName: 'Record Count',
        expression: 'sum(1)',
        meta: 'field',
        type: 'Measures',
        coltype: 'BIGINT',
        completer: {
          insertMatch(editor, data) {
            if (editor.completer.completions.filterText) {
              const ranges = editor.selection.getAllRanges();
              ranges.forEach(function (r) {
                r.start.column -= editor.completer.completions.filterText.length;
                editor.session.remove(r);
              });
            }

            aceEditor.execCommand('insertstring', data.expression);
          },
        },
      });
    }

    if (!config.tables) {
      config.tables = arc.report.getPrimaryReport().dataset().tables();
    }

    config.tables.forEach(function addTableFieldsAutocompletes(table) {
      let tableName = table.tablename().split('.').map((val) => `\`${val}\``).join('.');
      let nameParts = table.tablename_noalias().split('.');
      const snippetName = nameParts[1];
      nameParts = nameParts.map((val) => `\`${val}\``);
      const unaliasedName = nameParts.join('.');
      let completion;
      if (config.useRawNames) {
        completion = unaliasedName;
      } else {
        if (table.table_alias()) {
          completion = `\`${table.table_alias()}\``;
        } else {
          completion = nameParts[1];
        }
      }

      if (includeTables && !tableMappings[completion]) {
        config.extraAutocompletes.push({
          value: completion,
          coltype: 'table',
          type: 'Tables',
          meta: 'field',
          snippetName,
        });
        tableMappings[completion] = tableName;
        tableRawMappings[completion] = unaliasedName;
      }

      table.columns().forEach(function addColumnAutocomplete(col) {
        // Omitting copied columns, as these may have associated SQL expressions, unless explicitly
        // included
        if (!col.attr('hidden') && (col.attr('name') || config.includeCalculated)) {
          const name = col.attr('name');
          const fullName = `${nameParts[1]}.\`${name}\``;

          // if looking at the raw name, there is the risk of duplicates when tables are self-joined
          if (name && config.useRawNames && fieldMappings[name]) return;
          const colname = col.attr('alias') || name;
          config.extraAutocompletes.push({
              value: config.useRawNames ? fullName :
                config.datasetSql ? colname : `[${colname}]`,
              coltype: col.attr('type'),
              type: col.attr('isdim') ? 'Dimensions' : 'Measures',
              meta: 'field',
              isSensitive: col.isSensitive(),
              snippetName: colname,
          });
          fieldMappings[colname] = `${tableName}.\`${name}\``;
        }
      });
    });

    // Add "\[\]" to ID_REGEX within ace's util function
    identifierRegexps = [/[\[\]a-zA-Z_0-9]/];
    if (!config.excludeAutocomplete) {
      config.excludeAutocomplete = ['textCompleter', 'keyWordCompleter'];
    }
  };

  // Sets the editor's autocompletes, including extraAutocompletes
  autocompleteEditor.setAutocompletes = function() {
    if (extraAutocompletes === 'columns') {
      autocompleteEditor.getColumnAutocompletes();
    } else if (extraAutocompletes === 'tablesAndColumns') {
      autocompleteEditor.getColumnAutocompletes(true);
    } else {
      identifierRegexps = null;
    }

    const completers = autocompleteEditor.getCompleters();
    if (aceLanguageTools) {
      aceLanguageTools.setCompleters(completers);
    }
  };

  // Turns autocomplete on/off
  autocompleteEditor.configAutocompleteOnOffButton = function() {
    if (autocompleteEditor.autocompleteOnOffButton()) {
      const autocompleteType = config.autocompleteType ? `_${config.autocompleteType}` : '';
      const autocompleteCookieName = arc.frontend.vizCookie.getArcCookieKey(
        `autocomplete_on${autocompleteType}`,
        true
      );
      const previousAutocompleteChoice = arc.frontend.vizCookie.getJson(autocompleteCookieName);
      const $autocompleteControl = $(config.autocompleteOnOffButton).addClass('form-check');
      const isPreviousAutocompleteChoiceSet = previousAutocompleteChoice !== undefined &&
        !(typeof previousAutocompleteChoice === 'object' && $.isEmptyObject(previousAutocompleteChoice));
      let checked = isPreviousAutocompleteChoiceSet ? previousAutocompleteChoice : ' checked';
      let buttonHtml = `<input type="checkbox" class="input-control form-check-input"${checked}>`;

      if (!config.hideAutocompleteOnOffButtonLabel) {
        // Need to make label non-draggable or else clicking on label
        // doesn't toggle checkbox in modal
        buttonHtml =
          `${buttonHtml}
          <label class="arc-no-dragging noselect form-check-label">
            Autocomplete on
          </label>`;
      }

      if (!$autocompleteControl.html()) {
        $autocompleteControl.html(buttonHtml);
      }

      const $input =
        $autocompleteControl.find('input').on('change.changeLiveAutocomplete',
          function (e, preventFocus) {
            var autocompleteOn = e.target.checked;
            checked = autocompleteOn ? 'checked' : '';
            arc.frontend.vizCookie.setJson(autocompleteCookieName, checked, 30);
            aceEditor.setOptions({
              enableLiveAutocompletion: autocompleteOn,
              enableBasicAutocompletion: autocompleteOn,
              enableSnippets: autocompleteOn,
            });
            if (!preventFocus) {
              aceEditor.resize();
              aceEditor.focus();
            }
        });

      $input.trigger('change', true);
    }
  };

  autocompleteEditor.exprEditorMapperFilters = function(expr) {
    expr = autocompleteEditor.exprEditorFieldMapperNoTablenames(expr);

    const tableRegex =
      new RegExp(Object.keys(tableRawMappings).map(tblName => ` ${tblName}`).join('|'), 'g');

    return expr.replace(tableRegex, tblName => ` ${tableRawMappings[tblName.trim()]}`);
  };

  autocompleteEditor.exprEditorFieldMapperNoTablenames = function(expr) {
    const columnRegex =
      new RegExp(Object.keys(fieldMappings).map(fieldName => `\\[${fieldName}\\]`).join('|'), 'g');

    let quote_start = '`',
        quote_end = '`';

    const tables = config.tables;
    if (tables && tables.length > 0) {
      const table = tables[0];
      const dc_type = table.dataset().dataconnection_type();
      const conn = arc.dataconnection.types[dc_type]();

      if (conn) {
        quote_start = conn.quote_start;
        quote_end = conn.quote_end;
      }
    }

    return expr.replace(columnRegex, fieldName => quote_start + fieldName.slice(1, -1) + quote_end);
  };

  // Add ace options
  aceConfig.mode = `ace/mode/${autocompleteEditor.mode()}`;
  if (config.aceOptions) {
    for (const o in config.aceOptions) {
      aceConfig[o] = config.aceOptions[o];
    }
  }

  aceEditor.setOptions(aceConfig);

  // Calls to $blockScrolling below are because of this console message:
  // Automatically scrolling cursor into view after selection change this
  // will be disabled in the next version set
  // editor.$blockScrolling = Infinity to disable this message
  aceEditor.$blockScrolling = Infinity;

  autocompleteEditor.setAutocompletes();
  autocompleteEditor.configAutocompleteOnOffButton();

  // If there's a form, submit on command + enter or command + s
  aceEditor.commands.addCommand({
    name: 'saveEditor',
    bindKey: {
      win: 'Ctrl-S|Ctrl-Return',
      mac: 'Command-S|Command-Return',
      sender: 'editor|cli',
    },
    exec(env, args, request) {
      console.log('saving...', env, args, request);

      let $c = $(env.container);
      if ($c.closest('#dashboard-visual-style-sidebar').length > 0) {
        // visual css
        $('#app-save-link').trigger('click');
        return;
      }

      if ($c.closest('#dashboard-styling-settings-sidebar').length > 0) {
        // dashboard css
        $('#app-save-link').trigger('click');
        return;
      }

      if ($c.closest('.super-modal').length > 0) {
        // visual modal css
        $('#save-link').trigger('click');
        return;
      }

      $(env.container).closest('form').trigger('submit');
    },
  });

  // add command to lazy-load keybinding_menu extension
  aceEditor.commands.addCommand({
      name: 'showKeyboardShortcuts',
      bindKey: { win: 'Ctrl-Alt-h', mac: 'Command-Alt-h' },
      exec(aEditor) {
        ace.config.loadModule('ace/ext/keybinding_menu', function(module) {
            module.init(aEditor);
            aEditor.showKeyboardShortcuts();
        });
      },
  });

  // add command to lazy-load keybinding_menu extension
  aceEditor.commands.addCommand({
    name: 'showSettingsMenu',
    bindKey: { win: 'Ctrl-Shift-p', mac: 'Command-Shift-p' },
    exec(aEditor) {
      aEditor.showSettingsMenu();
    },

    readOnly: true,
  });

  return autocompleteEditor;
};
}());
