window.arc = window.arc || {};

(function() {
'use strict';

arc.utils = arc.utils || {};

// RETURNS SNIPPETS FOR VARIOUS CONNECTIONS
// Used to generate sidebar of snippets in expression editor (in hash format)
// And to create snippet text for ace editor modes (in string mode)
// _format (Optional):
//    Returns snippets as string
//      or hash of function types pointing to array of functions.
//    Default type is string
// TODO: add analytic functions
// TODO: Add: sparksql, mysql, postgres, redshift, mariadb, mongodb modes
// SNIPPET FORMAT:
// Snippets can come in numerous formats:
// 1) Specific caption: Snippet titles can be defined by:
//    TITLE => Snippet
// 2) Regular function: User can tab through each parameter in the function
//    snippetTitle(tabParameter1, tabParameter2, ...)
// 3) Special function: The title will be the snippet up until the curly braces or brackets
//    snippet title {tab parameter1 [nested tab parameter 2]}
//    The [] will be tabbed (unless escaped with \) and typically indicate optional parameters
arc.utils.snippetsGenerator = function (mode, _format) {
  // For data connections.
  // The "shared" key provides expressions that will be included in all connections
  // Each data connection can have an array associated with a "shared" type which indicates a
  // hash that will also be included
  var noSqlDatabases = {
      solr: true,
      ksql: true,
      elasticsearch: true,
    },
    snippetsDatabase = {
        shared: {
          Aggregate: [
            'avg(expression)', 'count(expression)', 'max(expression)',
            'min(expression)', 'sum(expression)',
          ],
          Conditional: [
            'CASE {value} WHEN {option1} THEN {result1} [WHEN {option2} THEN {result2}] [ELSE {default}] END',
            'COALESCE(type v1, type v2, ...)', 'IFNULL(type, type if_not_null)',
            'NULLIF(expr1, expr2)', 'IF(cond, true_result, else_result)',
          ],
          Mathematical: [
            'abs(numeric_type)', 'round({double|decimal} {, [int|int_type d]})',
          ],
          Miscellaneous: [
            'cast({expr} as {type})',
          ],
          String: [
            'group_concat({string} {, [string separator]})',
            'instr(string str, string substr)', 'lower(string)', 'ltrim(string)',
            'substr({string}, {int start} {, [int len]})', 'upper(string)',
          ],
          Filter: [
            'AND', 'BETWEEN {lower number} AND {upper number}', 'EXISTS',
            'IN (included items)', 'IS NULL', 'IS NOT NULL',
            'LIKE {wildcard_expression}', 'NOT', 'NOT EXISTS', 'NOT IN (excluded items)',
            'OR',
          ],
          Cloudera: [
            'ML Model API => cviz_rest(\'\\{"url":"{url}","accessKey":"{accessKey}","colnames":\\[{columns}\\],"response_colname":"{response}"[, config: {config}]\\}\')',
          ],
          duplicate: [
            'CASE WHEN {option1} THEN {result1} [WHEN {option2} THEN {result2}] [ELSE {default}] END',
          ],
        },
        bigquery: {
          Aggregate: [
            'ANY_VALUE({expression})',
            'ARRAY_AGG({DISTINCT} {expression} [{IGNORE|RESPECT} NULLS])',
            'ARRAY_CONCAT_AGG({expression})',
            'AVG([DISTINCT] {expression})',
            'BIT_AND({expression})',
            'BIT_OR({expression})',
            'BIT_XOR([DISTINCT] {expression})',
            'COUNT([DISTINCT] {*|expression})',
            'COUNTIF({predicate})',
            'LOGICAL_AND({expression})',
            'LOGICAL_OR({expression})',
            'MAX({expression})',
            'MIN({expression})',
            'STRING_AGG([DISTINCT] {expression} [, {delimiter}]',
            'SUM([DISTINCT] {expression})',
            'CORR({expression}, {expression})',
            'COVAR_POP({expression}, {expression})',
            'COVAR_SAMP({expression}, {expression})',
            'STDDEV_POP([DISTINCT] {expression})',
            'STDDEV_SAMP([DISTINCT] {expression})',
            'VAR_POP([DISTINCT] {expression})',
            'VAR_SAMP([DISTINCT] {expression})',
            'APPROX_COUNT_DISTINCT({expression})',
            'APPROX_QUANTILES([DISTINCT] {expression}, {number})',
            'APPROX_TOP_COUNT({expression}, {number})',
            'APPROX_TOP_SUM({expression}, {weight}, {number})',
          ],
          Array: [
            'ARRAY({subquery})',
            'ARRAY_CONCAT({array_expression_1} [, {array_expression_n}])',
            'ARRAY_LENGTH({array_expression})',
            'ARRAY_TO_STRING({array_expression}, {delimiter})',
            'GENERATE_ARRAY({start_expression}, {end_expression} [, {step_expression}])',
            'GENERATE_DATE_ARRAY({start_date}, {end_date} [, INTERVAL {step_expression} [DAY|WEEK|MONTH|QUARTER|YEAR]])',
            'GENERATE_TIMESTAMP_ARRAY({start_timestamp}, {end_timestamp} [, INTERVAL {step_expression} [MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAY]])',
            'OFFSET({zero_based_offset})',
            'ORDINAL({one_based_offset})',
            'ARRAY_REVERSE({array_expression})',
            'SAFE_OFFSET({zero_based_offset})',
            'SAFE_ORDINAL({one_based_offset})',
          ],
          Bit: [
            'BIT_COUNT({expression})',
          ],
          'Date and Time': [
            'CURRENT_DATE()',
            'DATE({timestamp_expression})',
            'DATE_ADD({date_expression}, INTERVAL {step_expression} {DAY|WEEK|MONTH|QUARTER|YEAR})',
            'DATE_SUB({date_expression}, INTERVAL {step_expression} {DAY|WEEK|MONTH|QUARTER|YEAR})',
            'DATE_DIFF({date_expression_1}, {date_expression_2}, {DAY|WEEK|ISOWEEK|MONTH|QUARTER|YEAR|ISOYEAR})',
            'DATE_TRUNC({date_expression}, {DAY|WEEK|ISOWEEK|MONTH|QUARTER|YEAR|ISOYEAR})',
            'DATE_FROM_UNIX_DATE({expresion})',
            'FORMAT_DATE({format_string}, {date_expression})',
            'PARSE_DATE({format_string}, {date_string})',
            'UNIX_DATE({date_expression})',
            'CURRENT_DATETIME()',
            'DATETIME({date_expression}, {time_expression})',
            'DATETIME_ADD({datetime_expression}, INTERVAL {step_expression} {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAY|WEEK|MONTH|QUARTER|YEAR})',
            'DATETIME_SUB({datetime_expression}, INTERVAL {step_expression} {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAY|WEEK|MONTH|QUARTER|YEAR})',
            'DATETIME_DIFF({datetime_expression_1}, {datetime_expression_2}, {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAY|WEEK|MONTH|QUARTER|YEAR})',
            'DATETIME_TRUNC({datetime_expression}, {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAY|WEEK|MONTH|QUARTER|YEAR})',
            'FORMAT_DATE({format_string}, {datetime_expression})',
            'PARSE_DATE({format_string}, {datetime_string})',
            'CURRENT_TIME()',
            'TIME({datetime_expression})',
            'TIME_ADD({time_expression}, INTERVAL {step_expression} {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'TIME_SUB({time_expression}, INTERVAL {step_expression} {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'TIME_DIFF({time_expression_1}, {time_expression_2}, {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'TIME_TRUNC({time_expression_1}, {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'FORMAT_TIME({format_string}, {time_expression})',
            'PARSE_TIME({format_string}, {time_string})',
            'CURRENT_TIMESTAMP()',
            'EXTRACT({MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAYOFWEEK|DAY|DAYOFYEAR|WEEK|ISOWEEK|MONTH|QUARTER|YEAR|ISOYEAR|DATE|DATETIME|TIME} FROM {timestamp_expression})',
            'STRING({timestamp_expression})',
            'TIMESTAMP({expression})',
            'TIMESTAMP_ADD({timestamp_expression}, INTERVAL {step_expression} {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'TIMESTAMP_SUB({timestamp_expression}, INTERVAL {step_expression} {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'TIMESTAMP_DIFF({timestamp_expression_1}, {timestamp_expression_2}, {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR})',
            'TIMESTAMP_TRUNC({timestamp_expression}, {MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAY|WEEK|MONTH|QUARTER|YEAR|ISOYEAR})',
            'FORMAT_TIMESTAMP({format_string}, {timestamp_expression}])',
            'PARSE_TIMESTAMP({format_string}, {timestamp_expression}])',
            'TIMESTAMP_SECONDS({expression})',
            'TIMESTAMP_MILLIS({expression})',
            'TIMESTAMP_MICROS({expression})',
            'UNIX_SECONDS({timestamp_expression})',
            'UNIX_MILLIS({timestamp_expression})',
            'UNIX_MICROS({timestamp_expression})',
          ],
          Hash: [
            'FARM_FINGERPRINT({expression})',
            'MD5({expression})',
            'SHA1({expression})',
            'SHA256({expression})',
            'SHA512({expression})',
          ],
          HLL: [
            'HLL_COUNT.INIT({expression} [, {precision}])',
            'HLL_COUNT.MERGE({expression})',
            'HLL_COUNT.MERGE_PARTIAL({expression})',
            'HLL_COUNT.EXTRACT({expression})',
          ],
          JSON: [
            'JSON_EXTRACT({json_string_expr}, {json_path_string_literal})',
            'JSON_EXTRACT_SCALAR({json_string_expr}, {json_path_string_literal})',
            'TO_JSON_STRING({value} [, {true|false}])',
          ],
          Mathematical: [
            'ABS({x})', 'SIGN({x})', 'IS_INF({x})', 'IS_NAN({x})',
            'IEEE_DIVIDE({x}, {y})', 'RAND()', 'SORT({x})', 'POWER({x}, {y})',
            'EXP({x})', 'LN({x})', 'LOG({x}, [{y}])', 'LOG10({x})',
            'GREATEST({x1}, {x2} [, {xn}])', 'LEAST({x1}, {x2} [, {xn}])',
            'DIV({x}, {y})', 'SAFE_DIVIDE({x}, {y})', 'MOD({x}, {y})',
            'ROUND({x} [, {n}])', 'TRUNC({x} [, {n}])', 'CEILING({x})',
            'FLOOR({x})',
            'COS({x})', 'COSH({x})', 'ACOS({x})', 'ACOSH({x})',
            'SIN({x})', 'SINH({x})', 'ASIN({x})', 'ASINH({x})',
            'TAN({x})', 'TANH({x})', 'ATAN({x})', 'ATANH({x})', 'ATAN2({x})',
          ],
          Navigational: [
            'FIRST_VALUE({expression})',
            'LAST_VALUE({expression})',
            'NTH_VALUE({expression}, {n})',
            'LEAD({expression}, [, {offset} [, {default}]])',
            'LAG({expression}, [, {offset} [, {default}]])',
            'PERCENTILE_CONT({expression}, {percentile})',
            'PERCENTILE_DISC({expression}, {percentile})',
          ],
          Net: [
            'NET.IP_FROM_STRING({addr})', 'NET.SAFE_IP_FROM_STRING({addr})',
            'NET.IP_TO_STRING({addr_bin})', 'NET.IP_NET_MASK({num_output_bytes}, {prefix_length})',
            'NET.IP_TRUNC({addr_bin}, {prefix_length})',
            'NET.IPV4_FROM_INT64({integer_value})', 'NET.IPV4_TO_INT64({addr_bin})',
            'NET.HOST({url})', 'NET.PUBLIC_SUFFIX({url})', 'NET.REG_DOMAIN({url})',
          ],
          Miscellaneous: [
            'SESSION_USER()',
          ],
          String: [
            'BYTE_LENGTH({value})',
            'CHAR_LENGTH({value})',
            'CHARACTER_LENGTH({value})',
            'CODE_POINTS_TO_BYTES({ascii_values_array})',
            'CODE_POINTS_TO_STRING({ascii_values_array})',
            'CONCAT({value1} [, {value2}])',
            'ENDS_WITH({value1}, {value2})',
            'FORMAT({format_string}, {values})',
            'FROM_BASE32({string_expr})',
            'FROM_BASE64({string_expr})',
            'FROM_HEX({string})',
            'LENGTH({value})',
            'LPAD({original_value}, {return_length} [, {pattern}])',
            'LOWER({value})',
            'LTRIM({value1}, {value2})',
            'NORMALIZE({value})',
            'NORMALIZE_AND_CASEFOLD({value})',
            'REGEXP_CONTAINS({value}, {regex})',
            'REGEXP_EXTRACT({value}, {regex})',
            'REGEXP_EXTRACT_ALL({value}, {regex})',
            'REGEXP_REPLACE({value}, {regex}, {replacement})',
            'REPLACE({original_value}, {from_value}, {to_value})',
            'REPEAT({original_value}, {repetitions})',
            'REVERSE({value})',
            'RPAD({original_value}, {return_length} [, {pattern}])',
            'RTRIM({value1}, {value2})',
            'SAFE_CONVERT_BYTES_TO_STRING({value})',
            'SPLIT(value[, {delimiter}])',
            'STARTS_WITH({value1}, {value2})',
            'STRPOS({string}, {substring})',
            'SUBSTR({value}, {position} [, {length}])',
            'TO_BASE32({bytes_expr})',
            'TO_BASE64({bytes_expr})',
            'TO_CODE_POINTS({value})',
            'TO_HEX({bytes})',
            'TRIM({value1} [, {value2}])',
            'UPPER({value})',
          ],
        },
        hiveAndImpala: {
          Aggregate: [
            'variance(expression)', 'var_samp(expression)',
          ],
          Conditional: [
            'if(boolean condition, type if_true, type if_false_or_null)',
            'nvl(value, value_if_null)',
          ],
          'Date and Time': [
            'add_months(timestamp date, int|bigint months)',
            'current_timestamp()', 'day(string date)',
            'date_add(timestamp startdate, int|interval_expression days)',
            'datediff(string enddate, string startdate)',
            'date_sub(timestamp startdate, int|interval_expression days)',
            'from_unixtime(bigint unixtime {, [string format]})',
            'from_utc_timestamp(timestamp, string timezone)', 'month(string date)',
            'to_date(timestamp)', 'to_utc_timestamp(timestamp, string timezone)',
            'trunc(timestamp, string unit)',
            'unix_timestamp({[string datetime] {, [string|timestamp format]}})',
            'weekofyear(string date)', 'year(string date)',
          ],
          Mathematical: [
            'acos(double)', 'asin(double)', 'atan(double)',
            'bin(bigint)', 'ceiling(double|decimal)', 'cos(double)',
            'conv(bigint|string num, int from_base, int to_base)', 'degrees(double)',
            'e()', 'exp(double)', 'floor(double)',
            'greatest(bigint|double|decimal|string|timestamp)',
            'hex(bigint|string)', 'hour(string date)',
            'least(bigint|double|decimal|string|timestamp)', 'ln(double)',
            'log(double base, double)', 'log10(double)', 'log2(double)',
            'minute(string date)', 'negative(numeric_type)',
            'pi()', 'pmod(int|double, int|double)', 'positive(numeric_type)',
            'power(double, double p)', 'radians(double)',
            'rand([int seed])', 'second(string date)', 'sign(double)', 'sin(double)',
            'sqrt(double)', 'tan(double)',
            'unhex(string)',
          ],
          String: [
            'ascii(string str)', 'concat(string, string)',
            'concat_ws(string separator, string, string)',
            'find_in_set(string str, string strList)', 'initcap(string str)', 'length(string)',
            'locate({string substr}, {string str} {, [int pos]})',
            'lpad(string str, int len, string pad)',
            'parse_url({string urlString}, {string partToExtract} {, [string keyToExtract]})',
            'regexp_extract(string subject, string pattern, int index)',
            'regexp_replace(string initial, string pattern, string replacement)',
            'repeat(string str, int n)', 'reverse(string)',
            'rpad(string str, int len, string pad)',
            'rtrim(string)', 'space(int n)',
            'translate(string input, string from, string to)',
            'trim(string)',
          ],
          Filter: [
            'RLIKE => RLIKE \'{regexp}\'',
          ],
          duplicate: [
            'ceil(double|decimal)', 'dayofmonth(string date)', 'lcase(string)',
            'pow(double, double p)',
            'REGEXP {wildcard_expression}', 'substring({string}, {int start} {, [int len]})',
            'ucase(string)', 'var_pop({expression})',
          ],
        },
        hive: {
          shared: ['hiveAndImpala'],
          Aggregate: [
            'stddev_pop(col)', 'stddev_samp(col)', 'covar_pop(col1, col2)',
            'covar_samp(col1, col2)', 'corr(col1, col2)', 'percentile(BIGINT col, p|array)',
            'percentile_approx(DOUBLE col, p|array {, [B accuracy]})',
          ],
          Conditional: [
            'isnull(a)', 'isnotnull (a)',
          ],
          'Date and Time': [
            'current_date()', 'date_format(date|timestamp|string ts, string fmt)',
            'from_unixtime(int unixtime)',
            'last_day(string date)', 'months_between(date1, date2',
            'next_day(string start_date, string day_of_week)',
            'quarter(date|timestamp|string)',
          ],
          Mathematical: [
            'bround(double a {, [int decimal_places]})', 'cbrt(double a)',
            'factorial(int)', 'histogram_numeric(col, bin_size)', 'ntile(INTEGER bins)',
            'shiftleft(int|bigint)', 'shiftright(int|bigint)',
            'shiftrightunsigned(int|bigint)',
          ],
          String: [
            'base64(binary)', 'context_ngrams(array_of_arrays, array_strings|int, int K, int pf)',
            'decode(binary bin, string charset)', 'encode(string src, string charset)',
            'format_number(number, int decimal_places)', 'levenshtein(string A, string B)',
            'md5(string|binary)', 'printf(String format, Objs...)',
            'sentences(string str, string lang, string locale)',
            'str_to_map(text {, [delimiter1_for_key_value] {, [delimiter2_for_pairs]}})',
            'soundex(string A)', 'split(string str, string regexp)',
            'substring_index(string A, string delim, int count)', 'unbase64(string str)',
          ],
          Miscellaneous: [
            'aes_encrypt(input string|binary, key string|binary)',
            'aes_decrypt(input binary, key string|binary)',
            'array_contains(array, value)', 'binary(string|binary)',
            'collect_list(col)', 'collect_set(col)', 'crc32(string|binary)',
            'hash(args...)',
            'java_method(class, method {, [args...]})',
            'sort_array(array)',
            'get_json_object(json_string, string path)', 'map_keys(map)', 'map_values(map)',
            'size(map|array)',
            'sha1(string|binary)', 'sha2(string|binary, int)',
          ],
          duplicate: [
            'reflect(class, method {, [args...]})', 'sha(string|binary)',
          ],
        },
        impala_arcengine: {
          shared: ['impala'],
          Aggregate: [
            'gcdo(string)', 'boolean_and(boolean)',
            'boolean_or(boolean)', 'ndvp(expression)',
          ],
        },
        impala: {
          shared: ['hiveAndImpala'],
          Mathematical: [
            'fmod(double|float, double|float)',
            'fnv_hash(type)', 'is_inf(double)', 'is_nan(double)',
            'max_int(int)', 'max_tinyint(tinyint)', 'max_smallint(smallint)',
            'max_bigint(bigint)', 'min_int(int)', 'min_tinyint(tinyint)',
            'min_smallint(smallint)', 'min_bigint(bigint)',
            'precision(numeric_expression)',
            'quotient(int numerator, int denominator)',
            'scale(numeric_expression)',
          ],
          'Date and Time': [
            'adddate(timestamp startdate, int|bigint days)',
            'date_part(string, timestamp)',
            'dayname(string date)', 'dayofweek(string date)', 'dayofyear(timestamp date)',
            'days_add(timestamp startdate, int|bigint days)',
            'days_sub(timestamp startdate, int|bigint days)',
            'extract(timestamp, string unit)',
            'hours_add(timestamp date, int|bigint hours)',
            'hours_sub(timestamp date, int|bigint hours)',
            'microseconds_add(timestamp date, int|bigint microseconds)',
            'microseconds_sub(timestamp date, int|bigint microseconds)',
            'milliseconds_add(timestamp date, int|bigint milliseconds)',
            'milliseconds_sub(timestamp date, int|bigint milliseconds)',
            'minutes_add(timestamp date, int|bigint minutes)',
            'minutes_sub(timestamp date, int|bigint minutes)',
            'months_add(timestamp date, int|bigint months)',
            'months_sub(timestamp date, int|bigint months)',
            'nanoseconds_add(timestamp date, int|bigint nanoseconds)',
            'nanoseconds_sub(timestamp date, int|bigint nanoseconds)',
            'now()', 'seconds_add(timestamp date, int|bigint seconds)',
            'seconds_sub(timestamp date, int|bigint seconds)',
            'subdate(timestamp startdate, int|bigint days)',
            'weeks_add(timestamp date, int|bigint weeks)',
            'weeks_sub(timestamp date, int|bigint weeks)',
            'years_add(timestamp date, int|bigint years)',
            'years_sub(timestamp date, int|bigint years)',
          ],
          String: [
            'character_length(string)',
            'strleft(string, int num_chars)',
            'strright(string, int num_chars)',
          ],
          Conditional: [
            'decode({type expression}, {type search1}, {type result1} {, [type search2]} {, [type result2]} {, [type default]})',
            'nullifzero(numeric_expr)',
            'zeroifnull(numeric_expr)',
          ],
          Aggregate: [
            'appx_median(expression)', 'ndv(expression)', 'stddev(expression)',
            'variance_pop(expression)',
          ],
          Filter: [
          ],
          duplicate: [
            'extract({unit} FROM {timestamp})', 'char_length(string)',
            'isnull(type, type if_not_null)', 'variance_samp(expression)',
          ],
          Miscellaneous: [
            'current_database()', 'pid()', 'user()', 'version()',
          ],
        },
        mysql: {
          String: [
            'concat(string, string...)',
          ],
        },
        mysql_sqlite: {
          duplicate: [
            'likely({boolean} {, [float probability]})',
          ],
          Conditional: [
          ],
          'Date and Time': [
            'date(timestring, modifier...)', 'time(timestring, modifier...)',
            'datetime(timestring, modifier...)', 'julianday(timestring, modifier...)',
            'strftime(format, timestring, modifier...)',
          ],
          Mathematical: [
            'total(expression)', 'hex(blob)',
          ],
          Miscellaneous: [
            'glob(X, Y)',
            'likelihood({boolean} {, [float probability]})', 'quote(X)',
            'random()', 'randomblob(int bytes)', 'soundex(X)',
            'typeof(X)', 'unlikely(X)', 'zeroblob(N)',
          ],
          Filter: [
            'IS', 'IS NOT',
          ],
          String: [
            'char(ints)', 'length(string)',
            'like({string pattern}, {string subject} {, [escape character]})',
            'printf(string format, values...)', 'replace(string, string search_text, string replacement_text)',
            'rtrim({string} {, [string trim_char]})', 'trim({string} {, [string trim_char]})',
            'unicode(X)',
          ],
        },
        pgsql: [],
        sql: [],
        ksql: {
          Aggregate: [
            'COUNT(expression)', 'MAX(expression)', 'MIN(expression)', 'SUM(expression)',
            'TOPK(column_or_window, K)', 'TOPKDISTINCT(column_or_window, K)',
          ],
          Conditional: [],
          'Date and Time': [
            'STRINGTOTIMESTAMP({col}, \'{formatstring_like_yyyy-MM-dd HH:mm:ss.SSS}\')',
            'TIMESTAMPTOSTRING({bigint col}, \'{formatstring_like_yyyy-MM-dd HH:mm:ss.SSS}\')',
          ],
          Mathematical: [
            'ABS(col)',
            'CEIL(col)',
            'FLOOR(col)',
            'RANDOM()',
            'ROUND(col)',
          ],
          String: [
            'CONCAT(col, string str)',
            'LCASE(col)',
            'LEN(col)',
            'SUBSTRING(col, int startindex, int endindex)',
            'TRIM(col)',
            'UCASE(col)',
          ],
        },
        elasticsearch: {
          Aggregate: [
            'AVG(expression)', 'COUNT(expression)', 'MAX(expression)', 'MIN(expression)',
            'SUM(expression)', 'KURTOSIS(expression', 'PERCENTILE(expression, int percent)',
            'PERCENTILE_RANK(expression, int percent)', 'SKEWNESS(expression)',
          ],
          Conditional: [
            'coalesce({type expression}, {type expression2}, {type expression3}...)',
            'greatest(numeric_expr)',
            'ifnull(expression1, expression2)',
            'isnull(expression1, expression2)',
            'least(numeric_expr)',
            'nullif(expression1, expression2)',
            'nvl(expression1, expression2)',
          ],
          'Date and Time': [
            'CURRENT_DATE()',
            'CURDATE()',
            'CURRENT_TIMESTAMP()',
            'DAY({datetime_expression})',
            'DAY_OF_MONTH({datetime_expression})',
            'DAY_OF_WEEK({datetime_expression})',
            'DAY_OF_YEAR({datetime_expression})',
            'HOUR({datetime_expression})',
            'MINUTE({datetime_expression})',
            'MONTH({datetime_expression})',
            'MONTHNAME({datetime_expression})',
            'NOW()',
            'SECOND({datetime_expression})',
            'QUARTER({datetime_expression})',
            'TODAY()',
            'WEEK({datetime_expression})',
            'YEAR({datetime_expression})',
            'EXTRACT({MICROSECOND|MILLISECOND|SECOND|MINUTE|HOUR|DAYOFWEEK|DAY|DAYOFYEAR|WEEK|ISOWEEK|MONTH|QUARTER|YEAR|ISOYEAR|DATE|DATETIME|TIME} FROM {datetime_expression})',
          ],
          Mathematical: [
            'ABS(col)',
            'CEIL(col)',
            'FLOOR(col)',
            'RANDOM()',
            'ROUND(col)',
          ],
          String: [
            'ASCII(col)',
            'CHAR(int code)',
            'CHAR_LENGTH(col)',
            'CONCAT(col, string str)',
            'INSERT(col, int start, int length, string str)',
            'LCASE(col)',
            'LEFT(col, int count)',
            'LENGTH(col)',
            'LTRIM(col)',
            'OCTET_LENGTH(col)',
            'POSITION(col, string str2)',
            'REPEAT(col, int count)',
            'REPLACE(col, string pattern, string replacement)',
            'RIGHT(col, int count)',
            'RTRIM(col)',
            'SPACE(int count)',
            'SUBSTRING(col, int startindex, int length)',
            'UCASE(col)',
          ],
        },
        solr: {
          // from https://solr.apache.org/guide/8_8/function-queries.html
          // and https://solr.apache.org/guide/8_8/analytics-mapping-functions.html
          Aggregate: [
            'stddev(expression)', 'avg(expression)', 'count(expression)', 'max(expression)',
            'min(expression)', 'sum(expression)', 'distinct count => count(distinct(expression))',
            'count of missing values => missing(expression)', 'sumOfSquares(expression)',
          ],
          Conditional: [
            'exists(field|function)', 'def(field|function, defaultValue)', 'not(function|field)',
            'and(x, y, ...)', 'or(x, y, ...)', 'xor(x, y, ...)', 'gt(value1, value2)',
            'gte(value1, value2)', 'lt(value1, value2)', 'lte(value1, value2)', 'eq(value1, value2)',
            'if(test, value1, value2)', 'equal(value1, value2)',
          ],
          'Date and Time': [
            'ms({date}{, [subtrahend]})',
          ],
          Distance: [
            'dist(power, pointA, pointB)', 'sqedist(pointA, pointB)', 'hsin(radius, true|false, x1,y1,x2,y2)',
            'strdist(s1, s2, {jw|edit|ngram|FQN}{, [ngram size]})',
          ],
          Mathematical: [
            'sum math => sum(value1, value2, ...)', 'sub(minuend, subtrahend)', 'product(value1, value2, ...)',
            'div(dividend, divisor)', 'pow(base, power)', 'abs(number)', 'log(number)', 'sqrt(number)',
            'map(x, min, max, target {, [default]})', 'scale(x, minTarget, maxTarget)', 'query(subquery, default)',
            'linear(x, m, c)', 'recip(x, m, a, b)', 'max math => max(value1, value2, ...)',
            'min math => min(value1, value2, ...)', 'ceil(number)', 'floor(number)', 'neg(number|boolean)',
            'round(number)', 'add(value1, value2, ...)', 'mult(value1, value2, ...)',
          ],
          Miscellaneous: [
            'field(field)', 'top(value1, ...)', 'bottom(value1, ...)',
          ],
          Search: [
            'ord(field)', 'rordsearch(term)', 'docfreq(text, term)', 'termfreq(field, term)',
            'totaltermfreq(field, term)', 'sumtotaltermfreq(field)', 'idf(field, term)', 'tf(field, term)',
            'norm(field)', 'maxdoc()', 'numdocs()',
          ],
          String: [
            'concat(field|function|literal, ...)',
          ],
        },
      },
    snippets = {
      html: {
        Defaults: [
          'nbs => \n\t&nbsp;\n',
          'left => \n\t&#x2190;\n',
          'right => \n\t&#x2192;\n',
          'up => \n\t&#x2191;\n',
          'down => \n\t&#x2193;\n',
          'return => \n\t&#x21A9;\n',
          'backtab => \n\t&#x21E4;\n',
          'tab => \n\t&#x21E5;\n',
          'shift => \n\t&#x21E7;\n',
          'ctrl => \n\t&#x2303;\n',
          'enter => \n\t&#x2305;\n',
          'cmd => \n\t&#x2318;\n',
          'option => \n\t&#x2325;\n',
          'delete => \n\t&#x2326;\n',
          'backspace => \n\t&#x232B;\n',
          'esc => \n\t&#x238B;\n',
          'attr => \n\t{attribute}="{property}"\n',
          '. => \n\tclass="{}"\n',
          'alt => \n\talt="{}"\n',
          'charset => \n\tcharset="{utf-8}"\n',
          'data => \n\tdata-{}="{}"\n',
          'for => \n\tfor="{}"\n',
          'height => \n\theight="{}"\n',
          'href => \n\thref="{#}"\n',
          'lang => \n\tlang="{en}"\n',
          'media => \n\tmedia="{}"\n',
          'name => \n\tname="{}"\n',
          'rel => \n\trel="{}"\n',
          'scope => \n\tscope="{row}"\n',
          'src => \n\tsrc="{}"\n',
          'title => \n\ttitle="{}"\n',
          'type => \n\ttype="{}"\n',
          'value => \n\tvalue="{}"\n',
          'width => \n\twidth="{}"\n',
          'a => \n\t<a href="{#}">{}</a>\n',
          'a. => \n\t<a class="{}" href="{#}">{}</a>\n',
          'a# => \n\t<a id="{}" href="{#}">{}</a>\n',
          'a:ext => \n\t<a href="http://{example.com}">{}</a>\n',
          'abbr => \n\t<abbr title="{}">{}</abbr>\n',
          'address => \n\t<address>\n\t\t{}\n\t</address>\n',
          'area => \n\t<area shape="{rect}" coords="{}" href="{}" alt="{}" />\n',
          'area:c => \n\t<area shape="circle" coords="{}" href="{}" alt="{}" />\n',
          'area:d => \n\t<area shape="default" coords="{}" href="{}" alt="{}" />\n',
          'area:p => \n\t<area shape="poly" coords="{}" href="{}" alt="{}" />\n',
          'area:r => \n\t<area shape="rect" coords="{}" href="{}" alt="{}" />\n',
          'article => \n\t<article>\n\t\t{}\n\t</article>\n',
          'article. => \n\t<article class="{}">\n\t\t{}\n\t</article>\n',
          'article# => \n\t<article id="{}">\n\t\t{}\n\t</article>\n',
          'aside => \n\t<aside>\n\t\t{}\n\t</aside>\n',
          'aside. => \n\t<aside class="{}">\n\t\t{}\n\t</aside>\n',
          'aside# => \n\t<aside id="{}">\n\t\t{}\n\t</aside>\n',
          'audio => \n\t<audio src="{}">{}</audio>\n',
          'b => \n\t<b>{}</b>\n',
          'base => \n\t<base href="{}" target="{}" />\n',
          'blockquote => \n\t<blockquote>\n\t\t{}\n\t</blockquote>\n',
          'br => \n\t<br />\n',
          'button => \n\t<button type="{submit}">{}</button>\n',
          'button. => \n\t<button class="{button}" type="{submit}">{}</button>\n',
          'button# => \n\t<button id="{}" type="{submit}">{}</button>\n',
          'button:s => \n\t<button type="submit">{}</button>\n',
          'button:r => \n\t<button type="reset">{}</button>\n',
          'canvas => \n\t<canvas>\n\t\t{}\n\t</canvas>\n',
          'caption => \n\t<caption>{}</caption>\n',
          'cite => \n\t<cite>{}</cite>\n',
          'code => \n\t<code>{}</code>\n',
          'col => \n\t<col />\n',
          'colgroup => \n\t<colgroup>\n\t\t{}\n\t</colgroup>\n',
          'datalist => \n\t<datalist>\n\t\t{}\n\t</datalist>\n',
          'dd => \n\t<dd>{}</dd>\n',
          'dd. => \n\t<dd class="{}">{}</dd>\n',
          'dd# => \n\t<dd id="{}">{}</dd>\n',
          'del => \n\t<del>{}</del>\n',
          'details => \n\t<details>{}</details>\n',
          'dfn => \n\t<dfn>{}</dfn>\n',
          'dialog => \n\t<dialog>\n\t\t{}\n\t</dialog>\n',
          'div => \n\t<div>\n\t\t{}\n\t</div>\n',
          'div. => \n\t<div class="{}">\n\t\t{}\n\t</div>\n',
          'div# => \n\t<div id="{}">\n\t\t{}\n\t</div>\n',
          'dl => \n\t<dl>\n\t\t{}\n\t</dl>\n',
          'dl. => \n\t<dl class="{}">\n\t\t{}\n\t</dl>\n',
          'dl# => \n\t<dl id="{}">\n\t\t{}\n\t</dl>\n',
          'dt => \n\t<dt>{}</dt>\n',
          'dt. => \n\t<dt class="{}">{}</dt>\n',
          'dt# => \n\t<dt id="{}">{}</dt>\n',
          'em => \n\t<em>{}</em>\n',
          'embed => \n\t<embed src="{}" type="{}" />\n',
          'fieldset => \n\t<fieldset>\n\t\t{}\n\t</fieldset>\n',
          'fieldset. => \n\t<fieldset class="{}">\n\t\t{}\n\t</fieldset>\n',
          'fieldset# => \n\t<fieldset id="{}">\n\t\t{}\n\t</fieldset>\n',
          'figcaption => \n\t<figcaption>{}</figcaption>\n',
          'figure => \n\t<figure>{}</figure>\n',
          'footer => \n\t<footer>\n\t\t{}\n\t</footer>\n',
          'footer. => \n\t<footer class="{}">\n\t\t{}\n\t</footer>\n',
          'footer# => \n\t<footer id="{}">\n\t\t{}\n\t</footer>\n',
          'h1 => \n\t<h1>{}</h1>\n',
          'h1. => \n\t<h1 class="{}">{}</h1>\n',
          'h1# => \n\t<h1 id="{}">{}</h1>\n',
          'h2 => \n\t<h2>{}</h2>\n',
          'h2. => \n\t<h2 class="{}">{}</h2>\n',
          'h2# => \n\t<h2 id="{}">{}</h2>\n',
          'h3 => \n\t<h3>{}</h3>\n',
          'h3. => \n\t<h3 class="{}">{}</h3>\n',
          'h3# => \n\t<h3 id="{}">{}</h3>\n',
          'h4 => \n\t<h4>{}</h4>\n',
          'h4. => \n\t<h4 class="{}">{}</h4>\n',
          'h4# => \n\t<h4 id="{}">{}</h4>\n',
          'h5 => \n\t<h5>{}</h5>\n',
          'h5. => \n\t<h5 class="{}">{}</h5>\n',
          'h5# => \n\t<h5 id="{}">{}</h5>\n',
          'h6 => \n\t<h6>{}</h6>\n',
          'h6. => \n\t<h6 class="{}">{}</h6>\n',
          'h6# => \n\t<h6 id="{}">{}</h6>\n',
          'header => \n\t<header>\n\t\t{}\n\t</header>\n',
          'header. => \n\t<header class="{}">\n\t\t{}\n\t</header>\n',
          'header# => \n\t<header id="{}">\n\t\t{}\n\t</header>\n',
          'hgroup => \n\t<hgroup>\n\t\t{}\n\t</hgroup>\n',
          'hgroup. => \n\t<hgroup class="{}">\n\t\t{}\n\t</hgroup>\n',
          'hr => \n\t<hr />\n',
          'i => \n\t<i>{}</i>\n',
          'iframe => \n\t<iframe src="{}" frameborder="0"></iframe>\n',
          'iframe. => \n\t<iframe class="{}" src="{}" frameborder="0"></iframe>\n',
          'iframe# => \n\t<iframe id="{}" src="{}" frameborder="0"></iframe>\n',
          'img => \n\t<img src="{}" alt="{}" />\n',
          'img. => \n\t<img class="{}" src="{}" alt="{}" />\n',
          'img# => \n\t<img id="{}" src="{}" alt="{}" />\n',
          'ins => \n\t<ins>{}</ins>\n',
          'kbd => \n\t<kbd>{}</kbd>\n',
          'legend => \n\t<legend>{}</legend>\n',
          'li => \n\t<li>{}</li>\n',
          'li. => \n\t<li class="{}">{}</li>\n',
          'lia => \n\t<li><a href="{#}">{}</a></li>\n',
          'link => \n\t<link rel="{}" href="{}" title="{}" type="{}" />\n',
          'link:css => \n\t<link rel="stylesheet" href="{style.css}" type="text/css" media="{all}" />\n',
          'map => \n\t<map name="{}">\n\t\t{}\n\t</map>\n',
          'map. => \n\t<map class="{}" name="{}">\n\t\t{}\n\t</map>\n',
          'map# => \n\t<map name="{}" id="{}">\n\t\t{}\n\t</map>\n',
          'mark => \n\t<mark>{}</mark>\n',
          'meter => \n\t<meter>{}</meter>\n',
          'object => \n\t<object data="{}" type="{}">\n\t\t{}\n\t</object>\n',
          'ol => \n\t<ol>\n\t\t{}\n\t</ol>\n',
          'ol. => \n\t<ol class="{}">\n\t\t{}\n\t</ol>\n',
          'ol# => \n\t<ol id="{}">\n\t\t{}\n\t</ol>\n',
          'p => \n\t<p>{}</p>\n',
          'pre => \n\t<pre>\n\t\t{}\n\t</pre>\n',
          'progress => \n\t<progress>{}</progress>\n',
          'q => \n\t<q>{}</q>\n',
          's => \n\t<s>{}</s>\n',
          'samp => \n\t<samp>\n\t\t{}\n\t</samp>\n',
          'script => \n\t<script type="text/javascript" charset="utf-8">\n\t\t{}\n\t</script>\n',
          'section => \n\t<section>\n\t\t{}\n\t</section>\n',
          'section. => \n\t<section class="{}">\n\t\t{}\n\t</section>\n',
          'section# => \n\t<section id="{}">\n\t\t{}\n\t</section>\n',
          'small => \n\t<small>{}</small>\n',
          'source => \n\t<source src="{}" type="{}" media="{}" />\n',
          'span => \n\t<span>{}</span>\n',
          'strong => \n\t<strong>{}</strong>\n',
          'style => \n\t<style type="text/css" media="{all}">\n\t\t{}\n\t</style>\n',
          'sub => \n\t<sub>{}</sub>\n',
          'summary => \n\t<summary>\n\t\t{}\n\t</summary>\n',
          'sup => \n\t<sup>{}</sup>\n',
          'table => \n\t<table border="{0}">\n\t\t{}\n\t</table>\n',
          'table. => \n\t<table class="{}" border="{0}">\n\t\t{}\n\t</table>\n',
          'table# => \n\t<table id="{}" border="{0}">\n\t\t{}\n\t</table>\n',
          'tbody => \n\t<tbody>\n\t\t{}\n\t</tbody>\n',
          'td => \n\t<td>{}</td>\n',
          'td. => \n\t<td class="{}">{}</td>\n',
          'td# => \n\t<td id="{}">{}</td>\n',
          'tfoot => \n\t<tfoot>\n\t\t{}\n\t</tfoot>\n',
          'th => \n\t<th>{}</th>\n',
          'th. => \n\t<th class="{}">{}</th>\n',
          'th# => \n\t<th id="{}">{}</th>\n',
          'thead => \n\t<thead>\n\t\t{}\n\t</thead>\n',
          'time => \n\t<time datetime="{}" pubdate="{}">{}</time>\n',
          'tr => \n\t<tr>\n\t\t{}\n\t</tr>\n',
          'track => \n\t<track src="{}" srclang="{}" label="{}" default="{default}">{}</track>\n',
          'ul => \n\t<ul>\n\t\t{}\n\t</ul>\n',
          'ul. => \n\t<ul class="{}">\n\t\t{}\n\t</ul>\n',
          'ul# => \n\t<ul id="{}">\n\t\t{}\n\t</ul>\n',
          'var => \n\t<var>{}</var>\n',
        ],
      },
      javascript: {
        Functions: [
          'function => function {function_name}({arguments}) \\{\n\t\t{body}\n\t}',
          'if => if ({true}) \\{\n\t\t{true_statement}\n\t}',
          'if...else => if ({true}) \\{\n\t\t{true_statement}\n\t} else \\{\n\t\t{else_statement}\n\t}',
          '? {if_true} : {if_false}',
          'case/switch => {expression} \\{\n\t\tcase "{case1}":\n\t\t\t{code}\n\t\t\tbreak;\n\t\t{additional_cases}\n\t\t{default: \n\t\t\t[default_action]}\n\t}',
          'try => try \\{\n\t\t{code}\n\t} catch (e) \\{\n\t\t{error_action}\n\t}',
        ],
        Loops: [
          'while => while ({conditional}) \\{\n\t\t{code}\n\t}',
          'do...while => do \\{\n\t\t{code}\n\t} while ({conditional});',
          'for object => for (var {property} in {object}) \\{\n\t\t{code}\n\t}',
          'for+ => for (var {[i] = 0}; {[i] < [object.length]}; {i++}) \\{\n\t\t{code}\n\t}',
          'for- => for (var {[i] = [object.length]}; {[i] > 0}; {i--}) \\{\n\t\t{code}\n\t}',
        ],
        DOM: [
          '.getElementsByTagName => .getElementsByTagName("{tag}")',
          '.getElementsByClass => .getElementsByClass("{class}")',
          '.getElementById => .getElementById("{id}")',
        ],
        Misc: [
          'setTimeout(function() \\{\n\t\t{code}\n\t}, {time_in_ms});',
          'console.log({item});',
          'console.time({timer_name})',
          'console.timeEnd({timer_name})',
          'JSON.stringify({object});',
          'JSON.parse({json_string});',
        ],
        Arcadia: [
          'return (Arc) => return \\{\n\t\tversion: "{1}",\n\t\t{arcadia_api}\n\t};',
          'disableDraw (Arc) => disableDraw: {true}{{|}false},\n\t',
          'beforeDraw (Arc) => beforeDraw: function () \\{\n\t\t{body}\n\t},\n\t',
          'afterDraw (Arc) => afterDraw: function () \\{\n\t\t{body}\n\t},\n\t',
          'settings (Arc) => settings: function () \\{\n\t\treturn [array of setting objects];\n\t},\n\t',
          'supported (Arc) => supported: \\{\n\t\tvisualType: "[trellis-bars]",\n\t},\n\t',
          'chartId (Arc) => {"#" + }arcapi.chartId()',
          'data (Arc) => arcapi.dataResult()',
          'addStyles (Arc) => arcapi.addStyles({filepath_urls}, function ([arguments]) \\{\n\t\t[body]\n\t})',
          'addScripts (Arc) => arcapi.addScripts({filepath_urls}, function ([arguments]) \\{\n\t\t[body]\n\t})',
          'getAllSettings (Arc) => arcapi.settings()',
          'getSettings (Arc) => arcapi.getSettings(settingName)',
          'sendParameters (Arc) => arcapi.sendParameters(params_hash, config)',
          'beforeParamDistribution (Arc) => arcapi.beforeParamDistribution(config)',
          'getParameters (Arc) => arcapi.getParameters()',
        ],
      },
      css: {
        Default: [
          '!important => !important;',
          '@media => @media {print} \\{\n\t\t{styling}\n\t}',
          'background => background-color: {#FFF};',
          'background image => background-image: url({image_url});',
          'border => border: {1px} {solid} {#000};',
          'border radius => border-radius: {5px};',
          'display => display: {inline|block|inline-block|none};',
          'float => float: {right|left};',
          'font-family => font-family: {"Helvetica Neue",Helvetica,Arial,sans-serif};',
          'font-size => font-size: {14px};',
          'font-weight => font-weight: {normal|bold|500};',
          'color => color: {#333}',
        ],
        Arcadia: [
          '.arc-apps-container => .arc-app-container \\{\n\t\t{code}\n\t}',
          '.arc-visual-container => .arc-viz-container \\{\n\t\t{code}\n\t}',
          '.arc-visual-type-container => .arc-viz-type-{table} \\{\n\t\t{code}\n\t}',
          '.arc-dashboard-title-container => .arc-dashboard-title-container \\{\n\t\t{code}\n\t}',
          '.arc-dashboard-title => .arc-dashboard-title \\{\n\t\t{code}\n\t}',
          '.arc-dashboard-subtitle-container => .arc-dashboard-subtitle-container \\{\n\t\t{code}\n\t}',
          '.arc-dashboard-subtitle => .arc-dashboard-subtitle \\{\n\t\t{code}\n\t}',
          '.arc-visual-title-container => .arc-viz-title-container \\{\n\t\t{code}\n\t}',
          '.arc-visual-title => .arc-viz-title \\{\n\t\t{code}\n\t}',
          '.arc-visual-subtitle-container => .arc-viz-subtitle-container \\{\n\t\t{code}\n\t}',
          '.arc-visual-subtitle => .arc-viz-subtitle \\{\n\t\t{code}\n\t}',
          '.arc-visual-axis => .arc-viz-axis \\{\n\t\t{code}\n\t}',
          '.arc-visual-axis-label => .arc-viz-axis-label \\{\n\t\t{code}\n\t}',
          '.arc-visual-legend => .arc-viz-legend \\{\n\t\t{code}\n\t}',
        ],
      },
    };

  if (_format === 'hash') {
    return generateSnippetHash();
  } else {
    return generateSnippetString();
  }

  // HELPER FUNCTIONS
  function generateSnippetString() {
    var snippetText = '';
    if (snippetsDatabase[mode]) {
      getAllSnippets(parseString);
    } else {
      for (var type in snippets[mode]) {
        parseString(type, snippets[mode][type]);
      }
    }

    return snippetText;

    function parseString(type, functions) {
      var parsedText;
      functions.forEach(function (f) {
        parsedText = arc.utils.snippetsGenerator.parseText(f);
        snippetText += 'snippet ';
        snippetText += parsedText.caption + '\n';
        snippetText += '\t' + f + '\n';
      });
    }
  }

  function generateSnippetHash() {
    var result = {};
    if (snippetsDatabase[mode]) {
      getAllSnippets(parseHash);
    } else {
      for (var type in snippets[mode]) {
        parseHash(type, snippets[mode][type]);
      }
    }

    return result;

    function parseHash(type, functions) {
      result[type] = result[type] ? result[type].concat(functions) : functions;
    }
  }

  function getAllSnippets(cb) {
    var type,
        snippetTypes;
    if (!noSqlDatabases[mode]) {
      for (type in snippetsDatabase.shared) {
        cb(type, snippetsDatabase.shared[type]);
      }
    }

    for (type in snippetsDatabase[mode]) {
      snippetTypes = snippetsDatabase[mode][type];
      if (type === 'shared') {
        getAdditionalFunctions(snippetTypes, cb);
      } else {
        cb(type, snippetTypes);
      }
    }

    // Recursively get all .shared functions
    function getAdditionalFunctions(additional, cb) {
      if (additional) {
        var snippetTypes;
        additional.forEach(function (sharedName) {
          var addFuns = snippetsDatabase[sharedName];
          for (var type in addFuns) {
            snippetTypes = addFuns[type];
            if (type === 'shared') {
              getAdditionalFunctions(snippetTypes, cb);
            } else {
              cb(type, snippetTypes);
            }
          }
        });
      }
    }

  }
};

// Turns a function into both:
//    caption
//      can either be predefined by =>, is the function name, or is the string before the
//      first parameter
//    and snippet
//      Needed to turn functions within snippets and snippetsDatabase objects into snippet
//      format with {} and [] turned into ${#:name}
arc.utils.snippetsGenerator.parseText = function (text) {
  var userDefinedCaption = text.indexOf('=>') !== -1,
      caption;
  if (userDefinedCaption) {
    var functionParameter = text.split(/\s*=>\s*/);
    caption = functionParameter[0];
    text = functionParameter[1];
  }

  var parenthesisIdx = text.indexOf('('),
      functionKeyword = text.slice(0, parenthesisIdx),
      parameterWords = text.slice(parenthesisIdx + 1),
      snippet = '';
  caption = caption ? caption : functionKeyword;
  var snippetParameters = [];
  if (text.indexOf('(') === -1 || text.indexOf(')') === -1) {
    var parsedSnippet = parseSpecialSnippet(text);
    caption = userDefinedCaption ? caption : parsedSnippet.functionName;
    return { snippet: parsedSnippet.specialSnippetParameter, caption: caption };
  } else {
    // Only allow user to edit within {} if they're present
    var specialIdx = text.indexOf('{'),
        ending = '';
    if (specialIdx !== -1) {
      snippet = parseSpecialSnippet(text, specialIdx).specialSnippetParameter;
    } else {
      snippet += functionKeyword + '(';
      var endIdx = parameterWords.indexOf(')');
      ending = parameterWords.slice(endIdx);
      parameterWords = parameterWords.slice(0, endIdx);
      var parametersArray = parameterWords.split(/,\s+/g);
      if (!(parametersArray.length === 1 && parametersArray[0] === '')) {
        parametersArray.forEach(function (p, i) {
          snippetParameters.push(startParam(i + 1) + p + '}');
        });
      }

      snippet += snippetParameters.join(', ') + ending;
    }

    return { snippet: snippet, caption: caption };
  }

  // HELPER FUNCTIONS
  function startParam(paramNumber) {
    return '${' + (paramNumber) + ':';
  }

  function parseSpecialSnippet(functionString, specialIdx) {
    var noParams = true;
    for (var i = 0; i < functionString.length; i++) {
      if (functionString[i] === '[' || functionString[i] === '{') {
        if (functionString[i - 1] !== '\\') {
          specialIdx = i;
          noParams = false;
          break;
        } else {
          functionString = functionString.slice(0, i - 1) + functionString.slice(i);
          i--;
        }
      }
    }

    var specialSnippetParameter, numberWords = 0, currentLetter,
        functionName = functionString.slice(0, specialIdx);
    if (noParams) {
      functionName = functionString;
      specialSnippetParameter = functionString;
    } else {
      specialSnippetParameter = functionName + startParam(numberWords += 1);
      for (specialIdx += 1; specialIdx < functionString.length; specialIdx++) {
        currentLetter = functionString[specialIdx];
        const isNotEscaped = functionString[specialIdx - 1] !== '\\';
        if (currentLetter === '{' || currentLetter === '[') {
          if (isNotEscaped) {
            specialSnippetParameter += startParam(numberWords += 1);
          } else {
            specialSnippetParameter += currentLetter;
          }
        } else if (currentLetter === ']' && isNotEscaped) {
          specialSnippetParameter += '}';
        } else if (currentLetter !== '\\') {
          specialSnippetParameter += currentLetter;
        }
      }
    }

    return {
      functionName,
      specialSnippetParameter,
    };
  }
};

// Turns function into displayable format by getting rid of {}
//    Used in getting tooltip for sidebar widget and also within language_tools
arc.utils.snippetsGenerator.convertSnippetToDisplay = function (text) {
  var display = text;
  if (text.indexOf('=>') !== -1) {
    display = text.split(/\s*=>\s*/)[1];
  }

  return display.replace(/\{|\}/g, '');
};
}());
