window.arc = window.arc || {};

(function() {
'use strict';

arc.visual = arc.visual || {};

arc.visual.brush = function() {
  const brush = function() {
    return 'arc.visual.brush';
  };

  let svg,
      xScale,
      yScale,
      brushg,
      cb;

  brush.svg = function(_) {
    if (!arguments.length) return svg;
    svg = _;
    return brush;
  };

  brush.xScale = function(_) {
    if (!arguments.length) return xScale;
    xScale = _;
    return brush;
  };

  brush.yScale = function(_) {
    if (!arguments.length) return yScale;
    yScale = _;
    return brush;
  };

  brush.cb = function(_) {
    if (!arguments.length) return cb;
    cb = _;
    return brush;
  };

  brush.initialize = function() {
    if (brushg) {
      brushg.remove();
    }

    brushg = svg.append('g')
                .attr('class', 'brush');

    return brush;
  };

  brush.draw = function() {
    const b =
        d3.svg.brush()
          .x(xScale)
          .on('brushstart', brushstart)
          .on('brush', brushmove)
          .on('brushend', brushend);

    function brushstart() {
      d3.event.sourceEvent.stopPropagation();
    }

    function brushmove() {
      d3.event.sourceEvent.stopPropagation();
    }

    function brushend() {
      d3.event.sourceEvent.stopPropagation();

      var e = b.extent();
      cb(e);
    }

    brushg.call(b);

    const yRange = yScale.range();
    const h = Math.max(yRange[0], yRange[yRange.length - 1]);
    brushg.selectAll('rect')
          .attr('height', h);

    return brush;
  };

  return brush;
};

arc.visual.timecontrolBrush = function() {
  let brush = function() {
    return 'arc.visual.timecontrolBrush';
  };

  let svg,
      svgNode,
      xScale,
      yScale,
      brushg,
      cb;

  brush.svg = function(_) {
    if (!arguments.length) return svg;
    svg = _;
    svgNode = svg.node();
    return brush;
  };

  brush.xScale = function(_) {
    if (!arguments.length) return xScale;
    xScale = _;
    return brush;
  };

  brush.yScale = function(_) {
    if (!arguments.length) return yScale;
    yScale = _;
    return brush;
  };

  brush.cb = function(_) {
    if (!arguments.length) return cb;
    cb = _;
    return brush;
  };

  brush.initialize = function() {
    if (brushg) {
      brushg.remove();
    }

    brushg = d4.select(svgNode)
                 .append('g')
                 .attr('class', 'brush');

    return brush;
  };

  let d4_brush;

  brush.select = function(xRange) {
    brushg.call(d4_brush.move, xRange);
  };

  brush.selection = function() {
    let xSelection = d4.brushSelection(brushg.node());
    if (!xSelection) {
      return undefined;
    }

    let xValues = xSelection.map(function(d) {
      return xScale.invert(d);
    });

    return {
      xSelection,
      xValues,
    };
  };

  brush.draw = function() {
    let xRange = xScale.range(),
        yRange = yScale.range();

    d4_brush = d4.brushX()
                 .extent([
                   [xRange[0], Math.min(yRange[0], yRange[yRange.length - 1])],
                   [xRange[1], Math.max(yRange[0], yRange[yRange.length - 1])],
                 ])
                 .on('start', brushstart)
                 .on('brush', brushmove);

    function brushstart() {
      let e = d4.event.sourceEvent;
      if (e && e.stopPropagation) {
        e.stopPropagation();
      }
    }

    function brushmove() {
      let e = d4.event.sourceEvent;
      if (e && e.stopPropagation) {
        e.stopPropagation();
      }

      d4_brush.on('end', brushend);
    }

    function brushend(d) {
      let e = d4.event.sourceEvent;
      if (e && e.stopPropagation) {
        e.stopPropagation();
      }

      d4_brush.on('end', null);

      let sel = brush.selection();
      if (sel && sel.xValues) {
        cb(sel.xValues);
      }
    }

    brushg.call(d4_brush);

    return brush;
  };

  return brush;
};

}());
