window.arc = window.arc || {};

(function () {
  'use strict';

  arc.utils = arc.utils || {};

  let _canvas;

  /**
   * Draws a list of texts on a canvas and returns width the longest text
   * based on the given style parameters.
   * Returns with a tuple. The first item in the tuple is the width of the text (number)
   * and the second item is the text itself.
   *
   * Example:
   * const [width, text] = arc.utils.findLongestText(['ff', 'ffff', 'fff'], {
   *   font: 'Roboto',
   *   fontSize: 14
   * });
   * console.log(width, text); // 19.44140625, "ffff"
   */
  arc.utils.findLongestText = function findLongestText(texts = [], style = {}) {
    const font = style.font || 'sherif';
    const fontSize = style.fontSize || 10;
    const canvas = _canvas || document.createElement('canvas');
    if (!_canvas) {
      canvas.width = 500;
      canvas.height = 500;
      _canvas = canvas;
    }
    const ctx = canvas.getContext('2d');

    ctx.font = `${fontSize}px ${font}`;

    return texts.reduce((acc, current) => {
      ctx.clearRect(0, 0, canvas.width, canvas.height);
      const currentWidth = ctx.measureText(current).width;
      const [previousWidth] = acc || [0];
      if (currentWidth > previousWidth) {
        return [currentWidth, current];
      }
      return acc;
    }, null);
  };

}());
