window.arc = window.arc || {};

(function() {
'use strict';

const colormap_expanded_template =
    '<div class="colormap-expanded">' +
    '<a class="viz-customcolor-addvalue btn btn-secondary viz-btn-xs">' +
    '  <span class="fa fa-plus"></span>' +
    ' Add Dimension Values</a>' +
    '<table class="dim-color-mapping">' +
    '  <thead>' +
    '    <tr>' +
    '      <th></th>' +
    '      <th>Color</th>' +
    '      <th>Value</th>' +
    '    </tr>' +
    '  </thead>' +
    '  <tbody>' +
    '  </tbody>' +
    '</table>' +
    '</div>';

const settings_palette_container_head =
    '<div class="color-mapper-palette-container">' +
    '  <span class="settings-palette">';

const settings_palette_container_foot =
    '  </span>' +
    '</div>';

const row_template =
    '<tr>' +
    '  <td><a class="viz-customcolor-remove btn btn-secondary viz-btn-xs" data-value="%VALUE%">' +
    '<span class="fa fa-remove"></span></a>' +
    '  <td><div tabindex="0" class="single-color" data-value="%VALUE%" ' +
    'style="background-color:%COLOR%;"></div></td>' +
    '  <td class="single-color-value"><div title="%VALUE%" class="dim-value">%VALUE%</div></td>' +
    '</tr>';

const info_template =
    '<tr><td colspan="3" class="form-text" style="width:100%">' +
      'A dimension is required in the Colors shelf for dimension values to populate here.' +
    '</td></tr>';

const checkbox_template =
    '<div class="checkbox">' +
    '<label class="checkbox">' +
      '<input type="checkbox"> ' +
      '<span data-test-id="fix-color-for-specified-checkbox">' +
        'Fix colors for specified dimension values' +
      '</span>' +
    '</label>' +
    '</div>';

const colorTemplate = '<span class="settings-swatch" style="height: 10px; ' +
    'background-color: %COLOR%; opacity: %OPACITY%"></span>';

arc.colormap = arc.colormap || {};

arc.colormap.colorPicker = function(config) {
  const {
    containerDiv,
    getColors,
    getOpacity,
    changeCb,
  } = config;
  let { $colors } = config;

  const $sel = $(containerDiv);

  if ($colors === undefined) {
    $colors = $sel.find('.single-color');
  }

  $colors.popover({
    container: containerDiv,
    viewport: containerDiv,
    placement: 'top',
    content() {
      const colors = getColors();
      let popover_content = settings_palette_container_head;
      colors.forEach((color) => {
        popover_content += arc.misc.safe_substitute(colorTemplate, {
          COLOR: color,
          OPACITY: getOpacity() / 100,
        });
      });

      popover_content += settings_palette_container_foot;
      return popover_content;
    },

    trigger: 'click',
    html: true,
    sanitize: false,
  })
  .on('shown.bs.popover', function() {
    const $pop = $(this);

    const $popoverContent = $(containerDiv)
        .find('.color-mapper-palette-container')
        .closest('.popover');
    $popoverContent.css('z-index', 1061);

    $sel.find('.popover .color-mapper-palette-container .settings-swatch')
        .offOn('click', function() {
          const $this = $(this);

          const c = $this.css('background-color');
          $pop.css('background-color', c);
          if (changeCb) {
            changeCb($pop, $(this));
          }

          $sel.find('.popover').popover('hide');
        });
  });
};

arc.colormap.dimToColorMappingSetting = function(getColorsOverWrite) {
  const setting = arc.visual.setting()
                   .id('dim_to_color_mapping')
                   .section('Color')
                   .tab('Color')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('DIM_TO_COLOR_MAPPING')
                   .value('{"mappings":{},"isChecked":false}');

  let mappings = {};
  let isChecked = false;
  let colorScale;

  let opacity = 100;

  setting.setOpacity = function(v) {
    opacity = Number(v);
    drawOpacity();
  };

  function drawOpacity() {
    const $settingDiv = $(`#${setting.div_id()}`);
    $settingDiv
        .find('.single-color')
        .css('opacity', opacity / 100);

    $settingDiv
        .find('.settings-swatch')
        .css('opacity', opacity / 100);
  }

  setting.getMappings = function() {
    if (isChecked) {
      return mappings;
    }

    return {};
  };

  setting.colorScale = function(_) {
    if (!arguments.length) return colorScale || setting.viz_lib().colorScale();
    colorScale = _;
    return setting;
  };

  function seedColors() {
    const _colorScale = setting.colorScale();
    const knownInputs = _colorScale.inputs();

    Object.keys(knownInputs).forEach((m) => {
      if (mappings[m] === undefined) {
        mappings[m] = _colorScale(m);
      }
    });
  }

  function paint_fn(d3_sel, repaint = false) {
    const data = JSON.parse(setting.value());

    mappings = data.mappings || {};
    isChecked = data.isChecked || false;

    const div_id = setting.div_id();
    let $container = $(`#${div_id}`);

    if (repaint) {
      $container.empty();
    } else {
      // Color palette setting is used not just in the dashboard editor
      // but in the dataset page -> fields -> edit field modal.
      // There we do not repaint, but redraw all the time.
      // This is why we have to remove it.
      $container?.remove();
      d3_sel
        .append('div')
        .attr('id', div_id)
        .attr('data-setting-id', setting.id());
      $container = $(`#${div_id}`);
    }

    $container.append(checkbox_template);
    $container.append(colormap_expanded_template);
    const $tbody = $container.find('tbody');

    const $expanded = $container.find('.colormap-expanded');
    const $modal = $('#dim-values-modal');

    $container.find('.viz-customcolor-addvalue')
        .offOn('click', () => {
          if (arc.misc.modalShown($modal)) {
            return;
          }

          const $text = $modal.find('textarea');
          $text.val('');

          $modal.modal('show');
          $modal.find('.dimvalues-add')
                .offOn('click', () => {
                  $modal.modal('hide');
                  const t = $text.val();
                  t.split('\n').forEach((v) => {
                    mappings[v] = setting.colorScale().range()[0];
                  });

                  seedColors();
                  setupColors();
                  setting.viz_lib().page_needs_save();
                });
        });

    // check can be triggered by user mouse event on frontend
    // or by restoring settings
    function handleChecked(triggeredByUser) {
      if (isChecked) {
        $expanded.removeClass('arc-invisible');
        $container.find('table').removeClass('arc-invisible');
      } else {
        $container.find('table').addClass('arc-invisible');
        $expanded.addClass('arc-invisible');
        mappings = {};
        setting.colorScale().clearFixedColors();
      }

      seedColors();
      setupColors();
      if (triggeredByUser) {
      setting.process_value();
      setting.makeChangeCbs();
      }
    }

    const $input = $container.find('input');
    $input.on('change', function() {
      isChecked = this.checked;
      $container.find('.single-color').popover('hide');
      handleChecked(true);
    });

    if (isChecked) {
      $input.prop('checked', isChecked);
    }

    handleChecked();

    function setupColors() {
      $tbody.empty();

      const mappingKeys = Object.keys(mappings);
      let sortedMappings;
      if (arc.misc.idxIsNumeric(mappingKeys)) {
        sortedMappings = Object.keys(mappings).sort((a, b) => a - b);
      } else {
        sortedMappings = Object.keys(mappings).sort();
      }

      sortedMappings.forEach((k) => {
        $tbody.append(arc.misc.safe_substitute(row_template, {
          COLOR: mappings[k],
          VALUE: k,
        }));
      });

      if (sortedMappings.length === 0) {
        $tbody.append(info_template);
      }

      drawOpacity();
      setting.value(`{"mappings":${JSON.stringify(mappings)},"isChecked":${isChecked}}`);
      setting.makeChangeCbs();

      $tbody.find('.viz-customcolor-remove')
          .offOn('click', function() {
            const $this = $(this);
            const v = $this.attr('data-value');
            delete mappings[v];
            setupColors();
            setting?.viz_lib?.()?.page_needs_save();
          });

      function getColors() {
        let colors;
        if (setting.viz_lib()) {
          colors = setting.viz_lib().user_settings().find_by_id('color_palette').activeColors();
        }

        if (colors === undefined || colors.length === 0) {
          colors = setting.colorScale().range();
        }

        return colors;
      }

      function getOpacity() {
        return opacity;
      }

      function changeCb($pop, $color) {
        const v = $pop.attr('data-value');
        const c = $color.css('background-color');

        if (mappings[v] === c) {
          return;
        }

        mappings[v] = c;
        setting.process_value();
        setting.makeChangeCbs();
      }

      arc.colormap.colorPicker({
        containerDiv: `#${div_id}`,
        getColors: getColorsOverWrite || getColors,
        getOpacity,
        changeCb,
      });
    }
  }

  function fetch_fn() {
    const data = {};
    data.mappings = mappings;
    data.isChecked = isChecked;

    if (!data.isChecked) {
      data.mappings = {};
    }

    return JSON.stringify(data);
  }

  function valueCb(value) {
    const data = JSON.parse(value);
    mappings = data.mappings || {};
    isChecked = data.isChecked || false;
    if (setting.viz_lib()) {
      setting.viz_lib().dimensionalColorScale?.();
    }
  }

  setting.valueCb(valueCb)
         .paint_fn(paint_fn)
         .fetch_fn(fetch_fn)
         .repaint_fn((lastd3Selection) => paint_fn(lastd3Selection, true));

  setting.disable_fn(() => {
    $(`#${setting.div_id()}`).find('input').prop('disabled', true);
    $(`#${setting.div_id()} .btn`).addClass('disabled');
    $(`#${setting.div_id()} table`).css('pointer-events', 'none');
  });

  setting.enable_fn(() => {
    $(`#${setting.div_id()}`).find('input').prop('disabled', false);
    $(`#${setting.div_id()} .btn`).removeClass('disabled');
    $(`#${setting.div_id()} table`).css('pointer-events', '');
  });

  return setting;
};

const measureValueHtml = `
  <div class="measure-value">
    <span class="single-color" data-value="%VALUE%" data-placement="top"
        style="background-color:%COLOR%;">
    </span>

    <span class="range-text"></span>
    <div class="range-wrapper">
      <div class="range-sub-wrapper">
        <select class="form-control btn btn-secondary range-left-type">
          <option value="gte">\u2265</option>
          <option value="gt">&gt;</option>
        </select>
        <input type="text" class="range-left form-control" placeholder="lower">
      </div>

      <div class="range-wrapper">
        <select class="form-control btn btn-secondary range-right-type">
          <option value="lt">&lt;</option>
          <option value="lte">\u2264</option>
        </select>
        <input type="text" class="range-right form-control" placeholder="upper">
      </div>
    </div>

    <a class="btn btn-secondary viz-btn-xs remove-color-btn"><i class="fa fa-remove"></i></a>
  </div>
`;

const measureSettingHtml = `
  <div class="measure-ranges-header">Custom Colors for Measure Value Ranges
    <span class="info-icon" data-toggle="popover" data-trigger="hover" data-content="Numerical range and percentage range both can be applied. Define percentage (range) by adding % character at the end of the input (e.g. 53%). Please note that the Color shelf should contain a field with numerical values." data-original-title="" title=""></span>
  </div>
  <div class="measure-value-list">
  </div>
  <div class="add-range-btn-wrapper">
    <a class="btn btn-secondary viz-btn-xs add-more-btn" data-test-id="add-color-range">
      <i class="fa fa-plus"></i> Add Range
    </a>
  </div>
`;

arc.colormap.measureRangesSetting = function() {
  const setting = arc.visual.setting()
                   .id('measure_ranges')
                   .section('Color')
                   .tab('Color')
                   .type(arc.visual.settingTypeEnum.CUSTOM)
                   .add_name('MEASURE_RANGES')
                   .value(JSON.stringify({
                     ranges: [],
                   }));

  let ranges = [];
  let opacity = 100;
  let colors;

  setting.setColors = function(_) {
    colors = _;
    return setting;
  };

  setting.setOpacity = function(v) {
    opacity = Number(v);
    drawOpacity();
  };

  function drawOpacity() {
    $(`#${setting.div_id()}`)
        .find('.single-color')
        .css('opacity', opacity / 100);
  }

  function fetchValues() {
    const $sel = $(`#${setting.div_id()}`);

    const $values = $sel.find('.measure-value');
    const _ranges = [];
    $values.each(function() {
      const $this = $(this);
      const color = $this.find('.single-color')
                         .css('background-color');

      const rangeLeftType = $this.find('.range-left-type').val();
      const rangeLeft = $this.find('.range-left').val();
      const rangeRightType = $this.find('.range-right-type').val();
      const rangeRight = $this.find('.range-right').val();

      _ranges.push({
        color,
        rangeLeftType,
        rangeLeft,
        rangeRightType,
        rangeRight,
      });
    });

    return {
      ranges: _ranges,
    };
  }

  function paint_fn(d3_sel, repaint = false) {
    const data = JSON.parse(setting.value());
    ranges = data.ranges || [];

    const div_id = setting.div_id();
    let $container = $(`#${div_id}`);

    if (repaint) {
      $container.empty();
    } else {
      $container?.remove();
      d3_sel.append('div')
            .classed('measure-ranges-setting', true)
            .attr('data-setting-id', setting.id())
            .attr('id', div_id);

      $container = $(`#${div_id}`);
    }

    d3_sel.append('div')
          .classed('measure-ranges-setting', true)
          .attr('id', div_id);
    $container.append(measureSettingHtml);
    $container.on('click', '.info-icon', (e) => {
      e.preventDefault();
    })
    .find('[data-toggle="popover"]')
    .popover({
      container: 'body',
      placement: 'top',
      trigger: 'hover'
    });

    function getColors() {
      if (colors) {
        return colors;
      }

      const viz_lib = setting.viz_lib();
      if (viz_lib) {
        return viz_lib.getColorPaletteColors();
      }

      return ['#ccc'];
    }

    function getOpacity() {
      return opacity;
    }

    function changeCb() {
      handleChanges();
    }

    function setEventHandlingOnMeasureItems($measureRangeItem) {
      // Handle color or interval change
      $measureRangeItem.find('select, input').offOn('change.range', () => handleChanges());

      // Handle removal
      $measureRangeItem.find('.remove-color-btn').offOn('click.remove', function() {
        $(this).closest('div').remove();
        handleChanges();
      });
    }

    function addMeasureColorIntervalItem($values, range) {
      const color = range ? range.color : getColors()[0];
      // Append and color
      $values
      .append(arc.misc.safe_substitute(measureValueHtml, {
        VALUE: color,
        COLOR: color,
      }));

      // Set opacity
      const $newlyAddedItem = $values.find('.measure-value').last();
      $newlyAddedItem.find('.single-color')
                     .css('opacity', opacity / 100);

      // Set values
      if (range) {
        $newlyAddedItem.find('.range-left-type')
              .val(range.rangeLeftType);
        $newlyAddedItem.find('.range-right-type')
              .val(range.rangeRightType);
        $newlyAddedItem.find('.range-left')
              .val(range.rangeLeft);
        $newlyAddedItem.find('.range-right')
              .val(range.rangeRight);
      }

      setEventHandlingOnMeasureItems($newlyAddedItem);
    }

    const $values = $container.find('.measure-value-list');

    if (ranges) {
      ranges.forEach((range) => {
        addMeasureColorIntervalItem($values, range);
      });
    }

    arc.colormap.colorPicker({
      containerDiv: `#${div_id}`,
      getColors,
      getOpacity,
      changeCb,
    });

    $container.find('.add-more-btn')
      .offOn('click.add', () => {
        addMeasureColorIntervalItem($values);

        arc.colormap.colorPicker({
          containerDiv: `#${div_id}`,
          getColors,
          getOpacity,
          changeCb,
          $colors: $container.find('.measure-value-list .single-color').last(),
        });

        handleChanges();
      });
  }

  function handleChanges() {
    setting.process_value();
    setting.makeChangeCbs();
    if (setting.viz_lib()) {
      setting.viz_lib().page_needs_save();
    }
  }

  function fetch_fn() {
    const data = fetchValues();
    return JSON.stringify(data);
  }

  function valueCb(value) {
    const data = JSON.parse(value);
    ranges = data.ranges || [];
  }

  setting.valueCb(valueCb)
         .paint_fn(paint_fn)
         .fetch_fn(fetch_fn)
         .repaint_fn((d3Sel) => {
            paint_fn(d3Sel, true);
        });

  setting.disable_fn(() => {
    $(`#${setting.div_id()} .btn`).addClass('disabled');
    $(`#${setting.div_id()}`).find('input').prop('disabled', true);
    $(`#${setting.div_id()}`).find('.measure-value').css('pointer-events', 'none');
  });

  setting.enable_fn(() => {
    $(`#${setting.div_id()} .btn`).removeClass('disabled');
    $(`#${setting.div_id()}`).find('input').prop('disabled', false);
    $(`#${setting.div_id()}`).find('.measure-value').css('pointer-events', '');
  });
  return setting;
};
}());
