window.arc = window.arc || {};

(function() {
'use strict';

const searchOptions = {
  minMatchCharLength: 2,
  threshold: 0.4,
  keys: ['name', 'description', 'author', 'last_editor'],
  id: 'id'
};

// Regex for matching CSS class or id name
// const CSS_CLASS_ID_REGEX = /\.([a-zA-Z_-]{1}[\w+-_]+)|#([a-zA-Z_-]{1}[\w+-_]+)(?=.*{)/gm;
// const JS_FUNCTION_REGEX = /function ([a-zA-Z_\$]{1}[\w+\$_]+)\s?\(/gm;

arc.css = arc.css || {};

/**
 * Searches through the array of objects `assets`
 * for `term` in the keys specified in searchOptions.
 * Assets: name, description, (author/last editor),
 * or text in custom style (exact match in css or js)
 * @param {string} term
 * @param {Array} assets
 */
arc.css.searchStyles = function (term, assets) {
  const parsedAssets = Object.values(assets).slice();
  const fuse = new window.Fuse(parsedAssets, searchOptions);
  const searchResults = fuse.search(term).map((result) => result.item.id);
  parsedAssets.forEach((style) => {
    const re = new RegExp(term, 'i');
    if (style.css.search(re) > -1 || style.js.search(re) > -1) {
      const idStr = String(style.id);
      if (!searchResults.includes(idStr)) {
        searchResults.push(idStr);
      }
    }
  });
  return searchResults;
};
}());
