window.arc = window.arc || {};

(function() {
'use strict';

arc.utils = arc.utils || {};

const currencies = ['$', '€', '¥', '£', '₹', '¢'];

arc.utils.displayOptions = function(displayDiv) {
  const display_options = {};
  let displayTypesEl;
  let errorMsg;
  let unsavedValue;
  const defaultJS = `function myFunc(value) {
    // Your custom JS code goes here
    // Make sure to return an updated version of value
    return value;
  }`;

  display_options.displayTypesEl = function(_) {
    if (!arguments.length) {
      return displayTypesEl;
    }

    displayTypesEl = _;
    return displayTypesEl;
  };

  const $displayFormatDiv = displayDiv;
  const $optionsContainer = displayDiv.find('#display-format-options');
  const $displayTypes = display_options.displayTypesEl(
    displayDiv.find('#display-format-type select')
  );
  let autocompleteEditor;
  const readOnly = !arc.misc.sysPerm('sys_styles');
  if (readOnly) {
    $displayTypes.find('option[value="js"]').addClass('arc-invisible');
  }

  display_options.autocompleteEditor = function(_) {
    if (!arguments.length) return autocompleteEditor;
    autocompleteEditor = _;
    return display_options;
  };

  const optionSections = { // Sections to be hidden/shown based on typeOptions
    decimal: $optionsContainer.find('#display-format-decimal'),
    suffix: $optionsContainer.find('#display-format-suffix'),
    comma: $optionsContainer.find('#display-format-comma'),
    currencysymbol: $optionsContainer.find('#display-format-currencysymbol'),
    currencyformat: $optionsContainer.find('#display-format-currencyformat'),
    currencyoptions: $optionsContainer.find('#display-format-currencyoptions'),
    currencydoclink: $optionsContainer.find('#display-format-currencydoclink'),
    dateformat: $optionsContainer.find('#display-format-dateformat'),
    dateoptions: $optionsContainer.find('#display-format-dateoptions'),
    customize: $optionsContainer.find('#display-format-customize'),
    js: $optionsContainer.find('#display-format-js'),
  };
  const optionInput = { // All form inputs in the modal
    suffix: optionSections.suffix.find('input'),
    decimal: optionSections.decimal.find('input[type=number]'),
    decimalall: optionSections.decimal.find('input[type=checkbox]'),
    comma: optionSections.comma.find('input'),
    currencysymbol: optionSections.currencysymbol.find('select'),
    currencyformat: optionSections.currencyformat.find('input'),
    currencyoptions: optionSections.currencyoptions.find('select'),
    dateformat: optionSections.dateformat.find('input'),
    dateoptions: optionSections.dateoptions.find('select'),
    customize: optionSections.customize.find('input'),
    decimalLabel: optionSections.decimal.find('.display-format-dp'),
    sigfigLabel: optionSections.decimal.find('.display-format-sf'),
    example: displayDiv.find('#display-format-example'),
    exampleResult: displayDiv.find('#display-format-example-result'),
  };
  const typeOptions = { // Sections to be shown for each display type
    integer: ['comma'],
    number: ['decimal', 'comma'],
    percentage: ['decimal', 'comma'],
    scientific: ['decimal', 'suffix'],
    currency: ['currencysymbol', 'currencyformat', 'currencyoptions', 'currencydoclink'],
    date: ['dateformat', 'dateoptions'],
    custom: ['customize'],
    js: ['js'],
  };

  display_options.optionInput = optionInput;

  display_options.errorMsg = function() {
    return errorMsg;
  };

  // Returns back object with all the selected options and
  // full display format text under 'result' property
  display_options.parseDisplayFormat = function(includeDefaultJS = true) {
    const userSelection = { type: $displayTypes.val(), result: '' };
    if (userSelection.type === '') {
      return userSelection;
    }

    switch (userSelection.type) {
      case 'custom':
        userSelection.customize = optionInput.customize.val();
        userSelection.result = userSelection.customize;
        break;
      case 'js': {
        const customJS = autocompleteEditor && autocompleteEditor.value().trim();
        if (customJS === defaultJS) {
          userSelection.js = includeDefaultJS ? customJS : '';
        } else {
          userSelection.js = customJS || '';
        }

        userSelection.result = userSelection.js;
        break;
      }

      case 'date':
        userSelection.dateoptions = optionInput.dateoptions.val();
        userSelection.dateformat = optionInput.dateformat.val();
        userSelection.result = userSelection.dateformat;
        break;
      case 'currency':
        userSelection.currencysymbol = optionInput.currencysymbol.val();
        userSelection.currencyoptions = optionInput.currencyoptions.val();
        userSelection.currencyformat = optionInput.currencyformat.val();
        userSelection.result = userSelection.currencyformat;
        break;
      default:
        if (!optionSections.comma.hasClass('arc-invisible')) {
          userSelection.comma = optionInput.comma.prop('checked');
          if (userSelection.comma) {
            userSelection.result += ',';
          }
        }

        if (!optionSections.decimal.hasClass('arc-invisible')) {
          let ending = 'f';
          userSelection.decimal = optionInput.decimal.val();
          userSelection.decimalall = optionInput.decimalall.prop('checked');
          if (userSelection.type === 'scientific') {
            userSelection.suffix = optionInput.suffix.filter(':checked').val();
            ending = userSelection.suffix;
          }

          if (userSelection.type === 'percentage') {
            ending = '';
          }

          if (!userSelection.decimalall) {
            userSelection.result += (`.${userSelection.decimal}${ending}`);
          } else if (userSelection.type === 'scientific') {
            userSelection.result += ending;
          }
        }

        if (userSelection.type === 'percentage') {
          if (userSelection.decimalall) {
            userSelection.result += 'p'; // preserves all decimal places
          } else {
            userSelection.result += '%'; // rounds off at specified decimal places
          }
        }

        if (userSelection.type === 'integer') {
          userSelection.result += '.0f'; // rounds to nearest integer
        }
    }

    return userSelection;
  };

  display_options.triggerChange = function(e) {
      $(e.target).trigger('change');
  };

  display_options.destroyEditor = function() {
    if (autocompleteEditor) {
      autocompleteEditor.destroy();
      autocompleteEditor = null;
    }
  };

  display_options.setDefaultJS = function() {
    if (autocompleteEditor) {
      autocompleteEditor.value(defaultJS);
    }
  };

  display_options.resetHandlers = function(dataset, formatValue) {
    function handleOptionChange(e) {
      const type = e.target.value;
      const activeSections = typeOptions[type];
      const exampleNumber = 12345.6789123;
      const exampleDate = '2010-01-05T13:05:53.010';
      const currentExample = optionInput.example.val();
      if (!activeSections) {
        $optionsContainer.addClass('arc-invisible');
      } else {
        $optionsContainer.removeClass('arc-invisible');
        Object.entries(optionSections).forEach(([section, $section]) => {
          $section.toggleClass(
            'arc-invisible', !activeSections.includes(section)
          );
        });

        switch (type) {
        case 'custom': {
          const valueToTest = unsavedValue || formatValue;
          if (valueToTest && valueToTest.result && valueToTest.type !== 'js') {
            optionInput.customize.val(valueToTest.result);
          } else if (typeof valueToTest === 'string') {
            optionInput.customize.val(valueToTest);
          }

          break;
        }

        case 'js':
          if (!arc.globals.enable_custom_styles) {
            $displayTypes.val('');
          } else if (!autocompleteEditor) {
            autocompleteEditor = display_options.setUpEditor();
          } else if (autocompleteEditor.value().trim() === '') {
            autocompleteEditor.value(defaultJS);
          }

          break;
        case 'scientific':
          optionInput.suffix.trigger('change');
          break;
        default:
          optionInput.decimalLabel.removeClass('arc-invisible');
          optionInput.sigfigLabel.addClass('arc-invisible');
        }
      }

      if (type === 'date') {
        if (!currentExample || !arc.misc.isValidDate(currentExample)) {
          optionInput.example.val(exampleDate);
        }
      // eslint-disable-next-line no-restricted-globals
      } else if (!currentExample || isNaN(currentExample)) {
        optionInput.example.val(exampleNumber);
      }

      optionInput.dateoptions.trigger('change');
      optionInput.currencysymbol.trigger('change');

      if (readOnly && type !== 'js') {
        $displayTypes.find('option[value="js"]').addClass('arc-invisible');
      }
    }

    // To update form if user chooses different type
    $displayTypes.off('change.changeOptions')
                 .on('change.changeOptions', handleOptionChange);

    optionInput.customize.off('keyup.triggerChange')
                         .on('keyup.triggerChange', display_options.triggerChange);

    optionInput.decimal.off('keyup.triggerChange')
                       .on('keyup.triggerChange', display_options.triggerChange);

    optionInput.example.off('keyup.triggerChange')
                       .on('keyup.triggerChange', display_options.triggerChange);

    // Change decimal to sig fig depending which is chosen
    optionInput.suffix.off('change.updateDecimal')
                      .on('change.updateDecimal', function updateDecimal(e) {
                        if (e.target.value === 's' && e.target.checked) {
                          optionInput.decimalLabel.addClass('arc-invisible');
                          optionInput.sigfigLabel.removeClass('arc-invisible');
                        } else {
                          optionInput.decimalLabel.removeClass('arc-invisible');
                          optionInput.sigfigLabel.addClass('arc-invisible');
                        }
                      });

    // Fills currency customizer field with selected pre-populated currency format
    optionInput.currencyoptions.off('change.populateCurrencyFormat')
                              .on('change.populateCurrencyFormat', function populateCurrencyFormat(e) {
                                if (e.target.value !== 'custom') {
                                  optionInput.currencyformat.val(e.target.value);
                                }
                              });

    // Replaces currency symbol of pre-populated currency format options and
    // currency customizer field with selected currency symbol
    optionInput.currencysymbol.off('change.populateCurrencyOptionsFormat')
                          .on('change.populateCurrencyOptionsFormat', function populateCurrencyOptionsFormat(e) {
                            optionInput.currencyoptions.find('option').each(function() {
                              const option = $(this)[0];
                              let optionVal;
                              let optionText;
                              if (option.value !== 'custom') {
                                if (currencies.includes(option.value[0])) {
                                  optionVal = option.value.replace(option.value[0],
                                                                    e.target.value);
                                  optionText = option.text.replace(option.text[0],
                                                                    e.target.value);
                                } else {
                                  optionVal = `${e.target.value}${option.value}`;
                                  optionText = `${e.target.value}${option.text}`;
                                }

                                option.value = optionVal;
                                option.text = optionText;
                              }
                            });

                            if (optionInput.currencyoptions.val() !== 'custom') {
                              optionInput.currencyoptions.trigger('change');
                            } else {
                              let currencyFormatVal = optionInput.currencyformat.val();
                              if (currencies.includes(currencyFormatVal[0])) {
                                currencyFormatVal = currencyFormatVal.replace(currencyFormatVal[0],
                                                                              e.target.value);
                              } else {
                                currencyFormatVal = `${e.target.value}${currencyFormatVal}`;
                              }

                              optionInput.currencyformat.val(currencyFormatVal);
                            }
                          });

    // Marks pre-populated currency format option as 'custom' and selects appropriate currency
    // symbol option if user changes currency customizer field
    optionInput.currencyformat.off('keyup.markCustomCurrency')
                              .on('keyup.markCustomCurrency', function markCustomCurrency(e) {
                                optionInput.currencysymbol.val(e.target.value[0]);
                                if (optionInput.currencysymbol.val() === null) {
                                  optionInput.currencysymbol.val('');
                                }

                                optionInput.currencyoptions.val(e.target.value);
                                if (optionInput.currencyoptions.val() === null) {
                                  optionInput.currencyoptions.val('custom');
                                }

                                optionInput.currencysymbol.trigger('change');
                              });

    // Fills date customizer field with selected pre-populated date
    optionInput.dateoptions.off('change.populateDateFormat')
                           .on('change.populateDateFormat', function populateDateFormat(e) {
                             if (e.target.value !== 'custom') {
                               optionInput.dateformat.val(e.target.value);
                             }
                           });

    // Marks pre-populated date option as 'custom' if user changes date field
    optionInput.dateformat.off('keyup.markCustomDate')
                          .on('keyup.markCustomDate', function markCustomDate(e) {
                            optionInput.dateoptions.val(e.target.value);
                            if (optionInput.dateoptions.val() === null) {
                              optionInput.dateoptions.val('custom');
                            }

                            optionInput.dateoptions.trigger('change');
                          });

    // Deactivates ability for user to indicate decimals if they select 'all'
    optionInput.decimalall.off('change.toggleDecimal')
                          .on('change.toggleDecimal', function toggleDecimal() {
                            const newProp = !optionInput.decimal.prop('disabled');
                            optionInput.decimal.prop('disabled', newProp);
                          });

    function handleSampleUpdate() {
      display_options.updateSample(dataset);
    }

    // update unsavedValue and static example
    $displayFormatDiv.off('change.updateSample')
                     .on('change.updateSample', handleSampleUpdate);

    // update static example
    $displayFormatDiv.find('.update-example-btn').off('click.updateSample')
                     .on('click.updateSample', handleSampleUpdate);
  };

  display_options.updateSample = function(dataset) {
    const example = optionInput.example.val();
    unsavedValue = display_options.parseDisplayFormat();
    const displayFormatType = $displayTypes.val();
    let format;
    let result;
    errorMsg = undefined;

    if (['date', 'js'].includes(displayFormatType)) {
      try {
        format = arc.utils.translateDisplayFormat(unsavedValue, dataset);
      } catch (error) {
        errorMsg = error;
        result = error;
      }
    } else if (displayFormatType) {
      format = arc.utils.translateDisplayFormat(unsavedValue.result, dataset);
    }

    // When there's a display format, show example and error
    // Just display example if no display format defined
    if (!errorMsg && format) {
      try {
        result = format(example) ? format(example) : example;
        errorMsg = undefined;
      } catch (error) {
        errorMsg = error;
        result = error;
      }
    } else {
      result = example;
    }

    optionInput.exampleResult.html(result);
  };

  display_options.setUpEditor = function() {
    if (!arc.globals.enable_custom_styles) {
      return;
    }

    const $autocompleteOnOffButton = $('.autocomplete');
    const editorSettings = {
      autocompleteType: 'customstyle',
      autocompleteOnOffButton: $autocompleteOnOffButton,
      excludeAutocomplete: ['keyWordCompleter'],
      excludeCompleter: ['Arcadia'],
      aceOptions: {
        theme: 'ace/theme/chrome',
        showGutter: true,
        wrap: false,
      }
    };

    if (readOnly) {
      editorSettings.aceOptions.readOnly = true;
      editorSettings.aceOptions.highlightActiveLine = false;
    }

    autocompleteEditor = arc.utils.autocompleteEditor('style-js', 'javascript', editorSettings);
    autocompleteEditor.value(defaultJS);
    if (readOnly) {
      $displayFormatDiv.find('.ace_editor').addClass('read-only');
      $displayFormatDiv.find('.form-text')
        .text('This field is read only because you do not have permission to edit custom styles');
    }

    return autocompleteEditor;
  };

  display_options.populateValues = function(formatValue, from) {
    optionInput.decimal.val('2'); // reset default
    optionInput.customize.val('');
    if (formatValue && formatValue.type) {
      $displayTypes.val(formatValue.type);
      Object.keys(formatValue).forEach((field) => {
        if (['type', 'result', 'autoAdded'].indexOf(field) === -1) {
          switch (field) {
            case 'suffix':
              optionInput[field].val([formatValue[field]]);
              break;
            case 'js':
              if (!arc.globals.enable_custom_styles) {
                $displayTypes.val('');
              } else if (autocompleteEditor) {
                autocompleteEditor.value(formatValue[field]);
                $displayTypes.find('option[value="js"]').removeClass('arc-invisible');
              }

              break;
            case 'decimalall':
            case 'comma':
              optionInput[field].prop('checked', formatValue[field]);
              if (field === 'decimalall' && formatValue[field] === true) {
                optionInput.decimal.prop('disabled', true);
              }

              break;
            default:
              if (formatValue.type === 'currency') {
                optionInput.currencysymbol.trigger('change');
              }

              optionInput[field].val(formatValue[field]);
          }
        }
      });
    } else if (typeof formatValue === 'string') {
      // Allow backward compatibility to string display formats
      $displayTypes.val('custom');
      optionSections.customize.val(formatValue);
    } else {
      $displayTypes.val('');
    }

    const shelfCurrencyURL = 'https://docs.cloudera.com/data-visualization/latest/howto-customize-fields/topics/viz-display-format-currency.html';
    const shelfCustomizeURL = 'https://docs.cloudera.com/data-visualization/latest/howto-customize-fields/topics/viz-cust-display-format.html';
    const dsCurrencyURL = 'https://docs.cloudera.com/data-visualization/latest/work-with-data/topics/viz-change-currency.html';
    const dsCustomizeURL = 'https://docs.cloudera.com/data-visualization/latest/work-with-data/topics/viz-change-custom-field.html#viz-change-custom-field';
    if (from === 'fromShelves') {
      optionSections.currencydoclink.find('a').attr('href', shelfCurrencyURL);
      optionSections.customize.find('#display-format-examples a').attr('href', shelfCustomizeURL);
    } else if (from === 'fromDatasets') {
      optionSections.currencydoclink.find('a').attr('href', dsCurrencyURL);
      optionSections.customize.find('#display-format-examples a').attr('href', dsCustomizeURL);
    }

    $displayTypes.trigger('change');
  };

  return display_options;
};

// Wrapper to convert strings to dates before formatting if necessary
arc.utils.timeFormat = function(timeFormat, dctype) {
  let dc;

  if (dctype) {
    dc = arc.dataconnection.types[dctype]();
  }

  return function (time) {
    if (time === null) {
      return null;
    }

    // eslint-disable-next-line no-restricted-globals
    if (!isNaN(time)) {
      const v = Number(time);
      if ((v >= 0) && (v <= 8)) {
        if (timeFormat === 'ddd') {
          return dc.dowToShortName(v);
        }

        if (timeFormat === 'dddd') {
          return dc.dowToLongName(v);
        }
      }
    }

    let jsTime = time;
    if (typeof jsTime.getMonth !== 'function') {
      jsTime = new Date(time);
      if (!arc.misc.isValidDate(jsTime) && arc.misc.isValidDate(time)) {
        // For IE/PhantomJS, new Date('2018-03-24 21:11:20') shows up as Invalid Date
        jsTime = moment(time);
      }
    }

    if (arc.misc.isValidDate(jsTime)) {
      return moment(time).format(timeFormat);
    }

    throw new Error(`Invalid Date set for formatting: ${jsTime}, ${time}`);
  };
};

// Returns undefined if empty formatter
arc.utils.translateDisplayFormat = function(fmt, dataset) {
  const formatString = typeof fmt === 'string' ? fmt : fmt.result;
  if (fmt && fmt.type === 'date') {
    return arc.utils.timeFormat(formatString, dataset && dataset.dataconnection_type());
  }

  if (fmt && fmt.type === 'js') {
    if (fmt.js) {
      return new Function(`return (${fmt.js})`)(); // eslint-disable-line no-new-func
    }

    return function(val) { return val; };
  }

  if (formatString) {
    return arc.utils.displayFormat(formatString);
  }
};

arc.utils.displayFormat = function(displayFormat) {
  // Facilitate varied currency support.

  // 'S' is a display format we provide to our users for currencies
  // The differences between 'S' and 's' are:
  // 1. For numbers < 1, 'S' is replaced by 'f' for floating point
  // 2. For numbers in the gig range for 's', 'G' is replaced by 'B'
  //      (for billion) for 'S'

  // Identify if any currency is applied, if yes store it in cur_idx
  let cur_idx = -1;
  let modifiedDisplayFormat;
  let baseFormat;
  for (let i = 0; cur_idx < 0 && i < currencies.length; i++) {
    modifiedDisplayFormat = displayFormat.replace(currencies[i], '$');
    if (modifiedDisplayFormat !== displayFormat) {
      cur_idx = i;
      break;
    }
  }

  // If there was some currency involved, change display format to USD for now
  if (cur_idx >= 0) {
    displayFormat = modifiedDisplayFormat;
  }

  // If number format is our custom S then
  // Change G for giga to B for billion and also subzero formats.
  modifiedDisplayFormat = displayFormat.replace('S', 's');
  if (modifiedDisplayFormat !== displayFormat) {
    baseFormat = d3.format(modifiedDisplayFormat);
    const decimalFormat = d3.format(displayFormat.replace('S', 'f'));
    return function(x) {
      // eslint-disable-next-line no-restricted-globals
      if (isNaN(x)) {
        return undefined;
      }

      x = Number(x);
      if (x < 1 && x > -1) {
        return revertToOrigCurrency(decimalFormat(x));
      }

      return revertToOrigCurrency(baseFormat(x).replace('G', 'B'));
    };
  }

  // If some other currency was mentioned then
  // 1. format data as per d3 rules as if currency was USD
  // 2. replace $ -> original currency.
  if (cur_idx >= 0) {
    baseFormat = d3.format(displayFormat);
    return function (x) {
      return revertToOrigCurrency(baseFormat(x));
    };
  }

  function revertToOrigCurrency(value) {
    return cur_idx >= 0 ? value.replace('$', currencies[cur_idx]) : value;
  }

  return d3.format(displayFormat);
};
}());
