window.arc = window.arc || {};

(function() {
'use strict';

arc.dates = arc.dates || {};

arc.dates.formatSearch = function() {
  const formatSearch = function() {
    return 'arc.dates.formatSearch';
  };

  const dateFormatList = [
    'YYYY-MM',
    'YYYY-MM-DD',
    'YYYY-MM-DD HH',
    'YYYY-MM-DD HH:mm',
    'YYYY-MM-DD HH:mm:ss',
    'YYYY-MM-DDTHH:mm:ssZ',
    'YYYY-MM-DDTHH:mm:ss.SSSZ',
    'YYYY-MM-DD HH:mm:ss.SSSZ',
  ];

  const dateFormats = {
    'YYYY-MM': {
      re: /^\d\d\d\d-\d\d$/,
      fmt: d3.time.format('%Y-%m'),
      is_datetime: false,
      arcengineAutoDetect: true,
      fmtString: 'YYYY-MM',
    },
    'YYYY-MM-DD': {
      re: /^\d\d\d\d-\d\d-\d\d$/,
      fmt: d3.time.format('%Y-%m-%d'),
      is_datetime: false,
      arcengineAutoDetect: true,
      fmtString: 'YYYY-MM-DD',
    },
    'YYYY-MM-DD HH': {
      re: /^\d\d\d\d-\d\d-\d\d \d\d$/,
      fmt: d3.time.format('%Y-%m-%d %H'),
      is_datetime: true,
      arcengineAutoDetect: true,
      fmtString: 'YYYY-MM-DD HH',
    },
    'YYYY-MM-DD HH:mm': {
      re: /^\d\d\d\d-\d\d-\d\d \d\d:\d\d$/,
      fmt: d3.time.format('%Y-%m-%d %H:%M'),
      is_datetime: true,
      arcengineAutoDetect: true,
      fmtString: 'YYYY-MM-DD HH:mm',
    },
    'YYYY-MM-DD HH:mm:ss': {
      re: /^\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d$/,
      fmt: d3.time.format('%Y-%m-%d %H:%M:%S'),
      is_datetime: true,
      arcengineAutoDetect: true,
      fmtString: 'YYYY-MM-DD HH:mm:ss',
    },
    'YYYY-MM-DD HH:mm:ss.LLL': {
      re: /^\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d\.\d\d\d$/,
      fmt: d3.time.format('%Y-%m-%d %H:%M:%S.%L'),
      is_datetime: true,
      arcengineAutoDetect: true,
      fmtString: 'YYYY-MM-DD HH:mm:ss.LLL',
    },
    'YYYY-MM-DDTHH:mm:ssZ': {
      re: /^\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\dZ$/,
      fmt: d3.time.format('%Y-%m-%dT%H:%M:%SZ'),
      is_datetime: true,
      fmtString: 'YYYY-MM-DDTHH:mm:ss[Z]',
    },
    'YYYY-MM-DDTHH:mm:ss.SSSZ': {
      re: /^\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d\.\d\d\dZ$/,
      fmt: d3.time.format.iso,
      is_datetime: true,
      fmtString: 'YYYY-MM-DDTHH:mm:ss.SSS[Z]',
    },
  };

  formatSearch.identify = function(s) {
    let df;
    dateFormatList.some((dateFormatName) => {
      const dateFormat = dateFormats[dateFormatName];
      if (!dateFormat) {
        return false;
      }

      if (dateFormat.re.exec(s) !== null) {
        df = {
          fmt: dateFormat.fmt,
          is_datetime: dateFormat.is_datetime,
          fmtString: dateFormat.fmtString,
          arcengineAutoDetect: dateFormat.arcengineAutoDetect,
        };
        return true;
      }

      return false;
    });

    return df;
  };

  formatSearch.isUTCtime = function(ts) {
    const utcRe1 = dateFormats['YYYY-MM-DDTHH:mm:ssZ'].re;
    const utcRe2 = dateFormats['YYYY-MM-DDTHH:mm:ss.SSSZ'].re;
    return !!(utcRe1.exec(ts) || utcRe2.exec(ts));
  };

  return formatSearch;
};

const seconds = 1;
const minutes = 60;
const hours = 3600;
const days = 24 * hours;
const months = 30 * days;
const years = 365 * days;
const decades = 10 * years;
const centuries = 10 * decades;

const intervalOptions = {
  milli: {
    options: [1, 2, 5, 10, 20, 50, 100, 1000],
    format: '%H:%M:%S.%L',
    num: 24 * 60 * 60 * 1000,
    dist: 1 / 1000,
    name: 'milli',
  },
  second: {
    options: [1, 2, 5, 10, 15, 30, 60],
    format: '%Y-%m-%d %X',
    num: 24 * 60 * 60,
    dist: seconds,
    name: 'second',
  },
  minute: {
    options: [1, 2, 5, 10, 15, 30, 60],
    format: '%Y-%m-%d %H:%M',
    num: 24 * 60,
    dist: minutes,
    name: 'minute',
  },
  hour: {
    options: [1, 2, 3, 6, 12, 24],
    format: '%Y-%m-%d %H:%M',
    num: 24,
    dist: hours,
    name: 'hour',
  },
  day: {
    options: [1, 2, 3, 4, 5, 7, 10, 30],
    format: '%Y-%m-%d',
    num: 1,
    dist: days,
    name: 'day',
  },
  month: {
    options: [1, 2, 3, 4, 6, 12],
    format: '%Y-%m',
    num: 1 / 30,
    dist: months,
    name: 'month',
  },
  year: {
    options: [1, 2, 3, 4, 5, 10],
    format: '%Y',
    num: 1 / 365,
    dist: years,
    name: 'year',
  },
  decade: {
    options: [1, 2, 3, 4, 5, 10],
    format: '%Y',
    num: 1 / (365 * 10),
    dist: decades,
    name: 'decade',
  },
  century: {
    options: [],
    format: '%Y',
    num: 1 / (365 * 100),
    dist: centuries,
    name: 'century',
  },
};

function getInterval(numSeconds, isDateTime) {
  if (numSeconds > 2 * centuries) {
    return intervalOptions.century;
  }

  if (numSeconds > 2 * decades) {
    return intervalOptions.decade;
  }

  if (numSeconds > 2 * years) {
    return intervalOptions.year;
  }

  if (numSeconds > 2 * months) {
    return intervalOptions.month;
  }

  if (numSeconds > 2 * days || !isDateTime) {
    return intervalOptions.day;
  }

  if (numSeconds > 2 * hours) {
    return intervalOptions.hour;
  }

  if (numSeconds > 2 * minutes) {
    return intervalOptions.minute;
  }

  if (numSeconds > 1) {
    return intervalOptions.second;
  }

  return intervalOptions.milli;
}

  /**
   * This function takes the domain (i.e. date range) and length of a date axis
   * as well as custom format if one has been set. It then uses that information to create
   * a tick function that properly places ticks on a continuous date axis at reasonable intervals
   *
   * @param domain: domain of custom Date
   * @param totalLen: length of axis
   * @param customFormat format for tick values if one exists
   * @returns {
   * tickFunction: function passed into d3.ticks to place ticks
   * tickLength: length of each tick value
   * tickFormat: the format each tick value should take}
   */

  arc.dates.computeTimeAxisConfig = function ({
    domain,
    totalLen,
    fontSize,
    customFormat,
    isDateTime,
  }) {
  if (domain.length < 2) {
    return {};
  }

  /**
   *   custom function for generation of ticks every X intervals because using the d3 built-in
   *   interval
   *   functionality causes irregular spacing around starts and ends of a new time interval
   *   (i.e. entering a month when counting by days, or a day when counting by hours, etc.)
   *   resetting to zero each month.  https://github.com/d3/d3/issues/2240
   *   Possibly can be removed/updated upon upgrade to d3 v4
   */

  function tickEveryIntervals(step, intervalData) {
    const intervalStep = step;
    const intervalsSinceEpoch = function(date) {
      return Math.floor((date * intervalData.num) / 8.64e7);
    };

    function tickGenerator(t0, t1) {
      const startTime = new Date(t0);
      const endTime = new Date(t1);
      const ticks = [];
      let leadTime;
      let addTime;
      let intervalFactor = intervalData.name === 'century' ? 100 : 10;
      switch (intervalData.name) {
        case 'century':
        case 'decade': {
          const startYear = startTime.getUTCFullYear();
          const numYears = intervalStep * intervalFactor;
          leadTime = intervalFactor - (startYear % intervalFactor);
          addTime = leadTime > numYears / 4 ? leadTime : leadTime + numYears;
          startTime.setUTCFullYear(startYear + addTime);
          break;
        }

        case 'year':
          startTime.setUTCFullYear(startTime.getUTCFullYear() + intervalStep);
          intervalFactor = 1;
          break;
        case 'month':
          startTime.setUTCMonth(startTime.getUTCMonth() + intervalStep);
          break;
        case 'day':
          startTime.setUTCDate(startTime.getUTCDate() + intervalStep);
          break;
        case 'hour':
          startTime.setUTCHours(startTime.getUTCHours() + intervalStep);
          break;
        case 'minute': {
          const startMins = startTime.getUTCMinutes();
          leadTime = intervalStep - (startMins % intervalStep);
          addTime = leadTime > intervalStep / 4 ? leadTime : leadTime + intervalStep;
          startTime.setUTCMinutes(startTime.getUTCMinutes() + addTime);
          break;
        }

        case 'second': {
          const startSecs = startTime.getUTCSeconds();
          leadTime = intervalStep - (startSecs % intervalStep);
          addTime = leadTime > intervalStep / 4 ? leadTime : leadTime + intervalStep;
          startTime.setUTCSeconds(startTime.getUTCSeconds() + addTime);
          break;
        }

        case 'milli': {
          const startMillis = startTime.getUTCMilliseconds();
          leadTime = intervalStep - (startMillis % intervalStep);
          addTime = leadTime > intervalStep / 4 ? leadTime : leadTime + intervalStep;
          startTime.setUTCMilliseconds(startTime.getUTCMilliseconds() + addTime);
          break;
        }

        default:
          break;
      }

      const maxNumIterations = 1000;
      let numIterations = 0;
      while (intervalsSinceEpoch(startTime) <= intervalsSinceEpoch(endTime)) {
        switch (intervalData.name) {
          case 'century':
          case 'decade':
          case 'year': {
            const tickYear = startTime.getFullYear();
            ticks.push(new Date(new Date('January 1 00:00:00').setFullYear(tickYear)));
            startTime.setUTCFullYear(startTime.getUTCFullYear() + (intervalStep * intervalFactor));
            break;
          }

          case 'month':
            ticks.push(new Date(new Date(startTime.setDate(1)).setHours(0, 0, 0, 0)));
            startTime.setUTCMonth(startTime.getUTCMonth() + intervalStep);
            break;
          case 'day':
            ticks.push(new Date(startTime.setHours(0, 0, 0, 0)));
            startTime.setUTCDate(startTime.getUTCDate() + intervalStep);
            break;
          case 'hour':
            ticks.push(new Date(startTime.setMinutes(0, 0, 0)));
            startTime.setUTCHours(startTime.getUTCHours() + intervalStep);
            break;
          case 'minute':
            ticks.push(new Date(startTime.setSeconds(0, 0)));
            startTime.setUTCMinutes(startTime.getUTCMinutes() + intervalStep);
            break;
          case 'second':
            ticks.push(new Date(startTime.setMilliseconds(0)));
            startTime.setUTCSeconds(startTime.getUTCSeconds() + intervalStep);
            break;
          case 'milli':
            ticks.push(new Date(startTime));
            startTime.setUTCMilliseconds(startTime.getUTCMilliseconds() + intervalStep);
            break;
          default:
            break;
        }

        numIterations += 1;
        if (numIterations > maxNumIterations) {
          // eslint-disable-next-line no-console
          console.log('Creating date axis has failed');
          break;
        }
      }

      return ticks;
    }

    return tickGenerator;
  }

  const numSeconds = Math.abs(new Date(domain[1]) - new Date(domain[0])) / 1000;
  const interval = getInterval(numSeconds, isDateTime);
  let customLength;
  const exampleDate = new Date('December 29 1942');
  if (customFormat) {
    // This is a  less than ideal method of getting the length of a custom date format
    // December 27 2017 was chosen because it is in December (longest name for a month)
    // and on a Wednesday (longest name for a day)
    customLength = String(customFormat(exampleDate)).length * (fontSize / 2);
  }

  const intervalTextLength = d3.time.format(interval.format)(exampleDate).length * (fontSize / 2);

  const tickLength = customLength || intervalTextLength;
  const maxNumTicks = Math.floor(totalLen / tickLength);
  let bestStepOption;

  for (let i = 0; i < interval.options.length; i++) {
    if ((numSeconds / (interval.options[i] * interval.dist)) < maxNumTicks) {
      bestStepOption = interval.options[i];
      break;
    }
  }

  if (!bestStepOption) {
    bestStepOption = Math.ceil((numSeconds / interval.dist) / maxNumTicks);
  }

  if (!bestStepOption) {
    bestStepOption = 1;
  }

  const tickFunction = tickEveryIntervals(bestStepOption, interval);

  return {
    tickFunction,
    tickLength,
    tickFormat: interval.format,
  };
};
}());
