window.arc = window.arc || {};

(function() {
'use strict';

arc.geo = arc.geo || {};

arc.geo.geoTypes = [
  'none',
  'latitude',
  'longitude',
  'country',
  'state',
  'county',
  'dma',
  'zipcode',
  'custom',
];

arc.geo.autocastGeoLookup = {
  postal_code: 'zipcode',
  'postal code': 'zipcode',
  zip: 'zipcode',
  zip_code: 'zipcode',
  'zip code': 'zipcode',
  id: 'custom',
};

arc.geo.geoTypes.forEach(t => {
  arc.geo.autocastGeoLookup[t] = t;
});

arc.geo.geoTypesToDisplayNames = {
  none: 'None',
  latitude: 'Latitude',
  longitude: 'Longitude',
  country: 'Country',
  state: 'State',
  county: 'County',
  dma: 'DMA',
  zipcode: 'ZIP Code/Postal Code',
  custom: 'Shape ID',
};

arc.geo.geoTypeToDisplayName = function(t) {
  return arc.geo.geoTypesToDisplayNames[t];
};

arc.geo.validGeoTypes = function() {
  return arc.geo.geoTypes.slice(1);
};

arc.geo.autocastGeoType = function(colname) {
  return arc.geo.autocastGeoLookup[colname];
};

arc.geo.typeError = undefined;

arc.geo.computeMapType = function(geoColumns, allowCustom, geoTypes = {}) {
  if ((geoColumns && geoColumns.length < 1) || (!geoColumns && Object.keys(geoTypes).length < 1)) {
    return 'none';
  }

  if (geoColumns) {
    geoColumns.forEach(col => geoTypes[col.colgeotype()]  = true);
  }

  if (geoTypes.latitude ^ geoTypes.longitude) {
    arc.geo.typeError = 'Latitude and Longitude must be specified together';
    return 'error';
  }

  if (geoTypes.latitude && geoTypes.longitude) {
    return 'latlong';
  }

  if (geoTypes.county && geoTypes.state) {
    return 'county';
  } else if (geoTypes.county) {
    arc.geo.typeError = 'County and State must be specified together';
    return 'error';
  }

  if (geoTypes.zipcode) {
    return 'zipcode';
  }

  let assignedType;

  arc.geo.geoTypes.forEach(function(type) {
    if (geoTypes[type] && type !== 'none' && type !== 'custom') {
      assignedType = type;
    }
  });

  if (assignedType) {
    return assignedType;
  }

  // only mapbox allows for custom shape ids
  if (allowCustom) {
    return 'custom';
  } else {
    return 'none';
  }

  arc.geo.typeError = 'Unable to identify map type';
  return 'error';
};

}());
