window.arc = window.arc || {};

(function() {
'use strict';

arc.loader = arc.loader || {};
arc.loader.enableLoadbars = true;
arc.loader.visibleTimeout = 1;

arc.loader.showRefreshing = function() {
  $('#refresh-icon').hide();
  $('#refresh-spin').show();
};

arc.loader.refreshDone = function() {
  $('#refresh-icon').show();
  $('#refresh-spin').hide();
};

arc.loadbar = function() {
  var loaderVisible = false,
      visibleTimeout,
      error = false,
      dom_id = '#loadbar',
      progress = 0,
      total = 10,
      refresh_icon = false,
      startTime,
      enabled = true,
      cancelCb;

  var loader = function() { return 'arc.loadbar'; };

  loader.reset = function() {
    progress = 0;
    total = 10;
    $(dom_id).find('.loader-cancel').show();
    setupCancelHandler();
  };

  function setupCancelHandler() {
    $(dom_id).find('.loader-cancel')
             .off('click.loadcancel')
             .on('click.loadcancel', function() {
               if (cancelCb) {
                 cancelCb();
               }
             });
  }

  loader.cancelCb = function(_) {
    if (!arguments.length) return cancelCb;
    cancelCb = _;
    return loader;
  };

  loader.enabled = function(_) {
    if (!arguments.length) return enabled;
    enabled = _;
    return loader;
  };

  loader.error = function(_) {
    if (!arguments.length) return error;
    error = _;
    return loader;
  };

  loader.startTime = function(_) {
    if (!arguments.length) return startTime;
    startTime = _;
    return loader;
  };

  loader.updateProgress = function() {
    var w = (100 * progress / total);
    if (w < 1) {
      w = 1;
    }

    var $e = $(dom_id).find('.loader-detail .progress-bar');
    $e.attr('aria-valuenow', progress)
      .attr('aria-valuemax', total)
      .css('width', w + '%');
    return loader;
  };

  loader.progress = function(_) {
    if (!arguments.length) return progress;
    progress = _;
    if (refresh_icon) {
      var currTime = new Date();
      if ((currTime - startTime) > 0) {
        $(dom_id).find('.loader-detail').show();
        loader.updateProgress();
      }
    }

    return loader;
  };

  loader.total = function(_) {
    if (!arguments.length) return total;
    total = _;
    return loader;
  };

  function makeVisibleWork() {
    startTime = new Date();
    const $e = $(dom_id);
    if (refresh_icon) {
      $e.fadeIn('fast');
      $(dom_id).find('.loader-detail').show();
    } else {
      $e.css('background-color', '');
      $e.text('');
      $e.fadeIn('fast');
    }
  }

  function makeVisible() {
    if (visibleTimeout) {
      clearTimeout(visibleTimeout);
      visibleTimeout = undefined;
    }

    visibleTimeout = setTimeout(makeVisibleWork, arc.loader.visibleTimeout);
    loaderVisible = true;
  }

  function makeInvisible() {
    if (visibleTimeout) {
      clearTimeout(visibleTimeout);
      visibleTimeout = undefined;
    }

    var $e = $(dom_id);
    $e.stop();
    $e.hide();
    loaderVisible = false;
  }

  loader.showProgress = function() {
    if (error) {
      return;
    }

    if (progress === 0) {
      var $p = $(dom_id).find('.loader-detail .progress-bar');
      $p.css('transition', 'width 0s');
      $p.attr('aria-valuenow', progress)
        .attr('aria-valuemax', total)
        .css('width', '4%')
        .css('transition', '');
    }

    if (!loaderVisible && enabled && arc.loader.enableLoadbars) {
      arc.loader.showRefreshing();
      makeVisible();
    }
  };

  loader.showError = function(label) {
    var $e = $(dom_id);
    $e.text(label);
    $e.fadeIn('fast');
    $e.css('background-color', '#ff4444');
    error = true;

    var closewin =
      d3.select(dom_id)
        .append('i')
          .style('cursor', 'pointer')
          .style('margin-left', '10px')
          .classed('fa', true)
          .classed('fa-lg', true)
          .classed('fa-times-circle', true)
          .on('click', function() {
            makeInvisible();
            error = false;
          });
  };

  loader.hideProgress = function() {
    $('#loading-modal').modal('hide');
    arc.loader.refreshDone();

    if (error) {
      return;
    }

    if (loaderVisible) {
      makeInvisible();
    }
  };

  loader.hideAll = function() {
    error = false;
    makeInvisible();
  };

  loader.swapDomIds = function(new_id) {
    // Use this function when going in or out of super modal view and the
    // query is already in flight and the progress needs to be maintained
    var previouslyVisible = loaderVisible;
    makeInvisible();
    dom_id = new_id;
    setupCancelHandler();
    if (previouslyVisible) {
      loader.showProgress();
    }
  };

  loader.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return loader;
  };

  loader.refresh_icon = function(_) {
    if (!arguments.length) return refresh_icon;
    refresh_icon = _;
    return loader;
  };

  return loader;
};

}());
