window.arc = window.arc || {};

(function() {
'use strict';

arc.misc = arc.misc || {};

const contextmenuHdr = `
  <div class="contextmenu-hdr"></div>
`;

const contextmenuFoot = `
  <div class="contextmenu-foot"></div>
`;

const contextmenuButton = `
  <div class="contextmenu-button-label dropdown-item %CLASS%">
    %TEXT%
  </div>
`;

const contextmenuInput = `
  <div class="contextmenu-button-label dropdown-item form-inline %CLASS%">
    %TEXT%
    <input type="%TYPE%" class="form-control form-control-sm" name="%NAME%">
  </div>
`;

const contextmenuLink = `
  <a class="contextmenu-link dropdown-item %CLASS%">
    %TEXT%
  </a>
`;

const contextmenuDropdown = `
  <div id="%ID%" class="dropdown-on-hover %DROPDOWN_ON_HOVER_CLASS%" data-toggle="tooltip" data-original-title="" title="">
    <div style="position: relative;">
      <div class="contextmenu-button-label %CLASS%">
        %BUTTON_TEXT%
        <i class="fa fa-caret-down"></i>
      </div>

      <ul class="contextmenu-dropdown-menu dropdown-menu %DROPDOWN_CLASS%">
      </ul>
    </div>
  </div>
`;

const columnActionsTemplate = `
  <div class="dropdown-on-hover %DROPDOWN_ON_HOVER_CLASS%" data-toggle="tooltip" data-original-title="" title="" data-test-id="contextmenu-dropdown">
    <div>
      <div class="contextmenu-button-label %CLASS%">
        %BUTTON_TEXT%
        <i class="fa fa-caret-down"></i>
      </div>

      <ul class="dropdown-menu %DROPDOWN_CLASS%">
        %HTML_SEARCHBOX%
      </ul>
    </div>
  </div>
`;

const fadeFast = true;

function fadeIn($elem) {
  if (fadeFast) {
    if ($elem.css('display') !== 'block') {
      $elem.css('display', 'block');
    }
  } else {
    $elem.stop(true, true).fadeIn(200);
  }
}

function fadeOut($elem) {
  if (fadeFast) {
    if ($elem.css('display') !== 'none') {
      $elem.css('display', 'none');
    }
  } else {
    $elem.stop(true, true).fadeOut(200);
  }
}

arc.misc.filterSupportedElems = {
  path: true,
  rect: true,
  circle: true,
  tr: true,
  td: true,
};

function removeExistingHoverHighlights($g) {
  // Remove existing tooltip highlights
  const $filters = $g.find('.highlight-hover:not(.viz-managed)');
  if ($filters.length > 0) {
    arc.misc.removeClass($filters, 'highlight-hover');
  }
}

function removeExistingClickHighlights($g) {
  // Remove existing tooltip highlights
  const $filters = $g.find('.highlight-click');
  if ($filters.length > 0) {
    arc.misc.removeClass($filters, 'highlight-click');
  }
}

function getContainer(elem) {
  const nodeName = elem.nodeName.toLowerCase();
  if (arc.misc.filterSupportedElems[nodeName] === undefined) {
    return $();
  }

  if ((nodeName === 'td') || (nodeName === 'tr')) {
    return $(elem).closest('table');
  }

  return $(elem).closest('svg').find('> g');
}

function removeFilter(elem, forClick) {
  const $g = getContainer(elem);
  if ($g.length === 0) {
    return;
  }

  if (forClick) {
    removeExistingClickHighlights($g);
  } else {
    removeExistingHoverHighlights($g);
  }
}

function applyFilter(elem, forClick) {
  const $g = getContainer(elem);
  if ($g.length === 0) {
    return;
  }

  if (forClick) {
    removeExistingHoverHighlights($g);
    removeExistingClickHighlights($g);
  } else {
    removeExistingHoverHighlights($g);
  }

  const $elem = $(elem);
  if (forClick) {
    arc.misc.addClass($elem, 'highlight-click');
    return;
  }

  arc.misc.addClass($elem, 'highlight-hover');
}

function highlight(elem, forClick) {
  if (!arc.globals.enable_tooltip_highlights) {
    return;
  }

  if (elem === undefined) {
    return;
  }

  applyFilter(elem, forClick);
}

function unhighlight(elem, forClick) {
  if (!arc.globals.enable_tooltip_highlights) {
    return;
  }

  if (elem === undefined) {
    return;
  }

  removeFilter(elem, forClick);
}

arc.createContextMenuContainer = function(dom_id) {
  d3.select('body')
      .append('div')
      .attr('id', dom_id)
      .classed('contextmenu', true)
      .style('display', 'none');
};

// kvs is object with "key", "value", and optional "info" properties
arc.misc.tooltipFromKVs = function(config, kvs) {
  config = config || {};

  const {
    font,
    fontSize,
  } = config;

  if (kvs.length === 0) {
    return '';
  }

  let textl;

  {
    const classes = ['popover-table'];

    let attr = '';
    if (font !== undefined) {
      attr = ` data-fontclass="${font}" `;
      classes.push(font);
    }

    let s = '';
    if (fontSize !== undefined) {
      s = ` style="font-size:${fontSize}px" `;
    }

    textl = [`<table class="${classes.join(' ')}"${attr}${s}>`];
  }

  let display_value;
  kvs.forEach((kv) => {
    textl.push('<tr><td class="header-label">');
    textl.push(arc.misc.escapeHTML(kv.key));
    textl.push('</td>');

    if (kv.formatter) {
      textl.push('<td class="viz-tooltip-num">');
      display_value = arc.format.formattedDisplay(kv.formatter, kv.value, true);
    } else {
      display_value = arc.format.defaultFormatter(kv.value);
      // eslint-disable-next-line no-restricted-globals
      if ((kv.value === '') || isNaN(kv.value)) {
        if (kv.value_is_num) {
          textl.push('<td class="viz-tooltip-num">');
        } else {
          textl.push('<td class="viz-tooltip-string">');
        }
      } else {
        textl.push('<td class="viz-tooltip-num">');
      }
    }

    if (kv.ishtml) {
      textl.push(display_value);
    } else {
      textl.push(arc.misc.escapeHTML(display_value));
    }

    if (kv.info) {
      textl.push('<br/><small class="pull-right"><em>');
      textl.push(arc.misc.escapeHTML(kv.info));
      textl.push('</em></small>');
    }

    textl.push('</td></tr>');
  });

  textl.push('</table>');
  return textl.join('');
};

arc.marks = function() {
  let selections = [];
  let tooltips;
  let contextmenu;
  let marker_name;
  let mark_type;
  let viz_lib;
  let text_fn;
  let params_fn;
  let highlight_on_select = false;
  let text_changes_on_move = false;
  let cm_show_cb;
  let cm_hide_cb;
  let search_menus;
  let search_select;
  let tooltip_highlight_cb;
  let highlight_cb;
  let enableTooltipHighlights = true;
  let permitIncludeAndExclude = true;
  let permitPivot;
  let dom_id = '#tooltip';
  let contextmenu_dom_id;
  let activateTooltip = true;
  let highlightMarksCb;
  let combineWithOr = false; // used by detail data

  // Ensures context menu doesn't get closed accidentally when hover over
  // highlight paths
  let associatedMark;

  let legendValueGetter;

  const marks = function() { return 'arc.marks'; };

  marks.default_text_fn = function(d, i, elem) {
    return arc.misc.tooltipFromKVs(undefined, params_fn(d, i, elem));
  };

  text_fn = marks.default_text_fn;

  marks.highlightMarks = function(kvs) {
    if (highlightMarksCb) {
      highlightMarksCb(kvs);
    } else if (contextmenu) {
        contextmenu.highlightMarks(kvs);
      }

    return marks;
  };

  marks.tooltips = function() {
    return tooltips;
  };

  marks.contextmenu = function() {
    return contextmenu;
  };

  marks.highlightMarksCb = function(_) {
    if (!arguments.length) return highlightMarksCb;
    highlightMarksCb = _;
    return marks;
  };

  marks.activateTooltip = function(_) {
    if (!arguments.length) return activateTooltip;
    activateTooltip = _;
    return marks;
  };

  marks.enableTooltipHighlights = function(_) {
    if (!arguments.length) return enableTooltipHighlights;
    enableTooltipHighlights = _;
    return marks;
  };

  marks.permitIncludeAndExclude = function(_) {
    if (!arguments.length) return permitIncludeAndExclude;
    permitIncludeAndExclude = _;
    return marks;
  };

  marks.permitPivot = function(_) {
    if (!arguments.length) return permitPivot;
    permitPivot = _;
    return marks;
  };

  marks.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return marks;
  };

  marks.contextmenu_dom_id = function(_) {
    if (!arguments.length) return contextmenu_dom_id;
    contextmenu_dom_id = _;
    return marks;
  };

  marks.cm_show_cb = function(_) {
    if (!arguments.length) return cm_show_cb;
    cm_show_cb = _;
    return marks;
  };

  marks.cm_hide_cb = function(_) {
    if (!arguments.length) return cm_hide_cb;
    cm_hide_cb = _;
    return marks;
  };

  marks.highlight_on_select = function(_) {
    if (!arguments.length) return highlight_on_select;
    highlight_on_select = _;
    return marks;
  };

  marks.tooltip_highlight_cb = function(_) {
    if (!arguments.length) return tooltip_highlight_cb;
    tooltip_highlight_cb = _;
    return marks;
  };

  marks.highlight_cb = function(_) {
    if (!arguments.length) return highlight_cb;
    highlight_cb = _;
    return marks;
  };

  marks.text_changes_on_move = function(_) {
    if (!arguments.length) return text_changes_on_move;
    text_changes_on_move = _;
    return marks;
  };

  /*
   * Note that when we are drawing a combined bar/line/highlight visual,
   * we attach to the mouseover/click events of the overlay svg from each
   * visual. So we need a unique suffix for each visual. This is gotten
   * by using the mark_type that is set here as part of the event name suffix.
   */
  marks.mark_type = function(_) {
    if (!arguments.length) return mark_type;
    mark_type = _;
    return marks;
  };

  marks.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return marks;
  };

  marks.marker_name = function(_) {
    if (!arguments.length) return marker_name;
    marker_name = _;
    return marks;
  };

  marks.text_fn = function(_) {
    if (!arguments.length) return text_fn;
    text_fn = _;
    return marks;
  };

  // Just for search visual dropdown
  marks.search_menus = function(_) {
    if (!arguments.length) return search_menus;
    search_menus = _;
    return marks;
  };

  // Just for search visual dropdown
  marks.search_select = function(_) {
    if (!arguments.length) return search_select;
    search_select = _;
    return marks;
  };

  marks.params_fn = function(_) {
    if (!arguments.length) return params_fn;
    params_fn = _;
    return marks;
  };

  marks.legendValueGetter = function(_) {
    if (!arguments.length) return legendValueGetter;
    legendValueGetter = _;
    return marks;
  };

  marks.associatedMark = function(_) {
    if (!arguments.length) return associatedMark;
    associatedMark = _;
    return marks;
  };

  marks.combineWithOr = function(_) {
    if (!arguments.length) return combineWithOr;
    combineWithOr = _;
    return marks;
  };

  marks.selections = () => selections;

  marks.addSelection = function(_) {
    if (!arguments.length) return selections;
    if (_) {
      selections.push(_);
    }

    return marks;
  };

  marks.activate = function() {
    if (viz_lib) {
      viz_lib.addMark(marks);
    }

    // Mark activation has been disabled for this visual by the 'showme' code
    if (viz_lib && viz_lib.markActivationEnabled() === false) {
      return marks;
    }

    tooltips = arc.tooltip()
                  .mark_type(mark_type)
                  .dom_id(dom_id)
                  .enableTooltipHighlights(enableTooltipHighlights)
                  .text_changes_on_move(text_changes_on_move)
                  .highlight_cb(tooltip_highlight_cb)
                  .associatedMark(associatedMark)
                  .text_fn(text_fn);
    selections.forEach((sel) => {
      tooltips.addSelection(sel);
    });

    if (!viz_lib || (viz_lib && viz_lib.displayTooltips())) {
      if (activateTooltip) {
        tooltips.activate();
      }
    }

    if (marker_name) {
      if (search_menus) {
        contextmenu = arc.contextmenu()
                       .mark_type(mark_type)
                       .marker_name(marker_name)
                       .search_menus(search_menus)
                       .click_behavior_cb(search_select)
                       .show_cb(cm_show_cb)
                       .text_fn(text_fn)
                       .addTooltip(tooltips)
                       .activate();
      } else if (viz_lib) {
        contextmenu = viz_lib.contextmenu(marker_name)
                           .mark_type(mark_type)
                           .dom_id(contextmenu_dom_id)
                           .addTooltip(tooltips)
                           .highlight_on_select(highlight_on_select)
                           .highlight_cb(highlight_cb)
                           .text_fn(text_fn)
                           .params_fn(params_fn)
                           .show_cb(cm_show_cb)
                           .hide_cb(cm_hide_cb)
                           .dataset_id(viz_lib.dataset_id())
                           .permitIncludeAndExclude(permitIncludeAndExclude)
                           .permitPivot(permitPivot)
                           .combineWithOr(combineWithOr)
                           .activate();
      } else {
        contextmenu = arc.contextmenu()
                       .mark_type(mark_type)
                       .marker_name(marker_name)
                       .text_fn(text_fn)
                       .params_fn(params_fn)
                       .addTooltip(tooltips)
                       .activate();
      }
    }

    return marks;
  };

  marks.destroy = function() {
    if (tooltips) {
      tooltips.destroy();
    }

    if (contextmenu) {
      contextmenu.destroy();
    }

    cm_show_cb = undefined;
    cm_hide_cb = undefined;

    if (search_select) {
      search_select = undefined;
      search_select = undefined;
      selections.forEach((sel) => {
        sel.on('click.contextmenu', null);
      });
    }

    selections = [];
    text_fn = undefined;
    params_fn = undefined;
    cm_show_cb = undefined;

    tooltip_highlight_cb = undefined;
    highlight_cb = undefined;

    return marks;
  };

  return marks;
};

function dropDown() {
  const dropdown = function() {
    return 'dropDown';
  };

  let menus = [];
  let $container;
  let contextmenu;
  let permitPivot;
  let permitMultiSelect;
  let in_dashboard;
  let showContextMenu = true;
  let viz;

  dropdown.container = function(_) {
    $container = _;
    return dropdown;
  };

  dropdown.contextmenu = function(_) {
    contextmenu = _;
    return dropdown;
  };

  dropdown.showContextMenu = function(_) {
    showContextMenu = _;
    return dropdown;
  };

  dropdown.in_dashboard = function(_) {
    if (!arguments.length) return in_dashboard;
    in_dashboard = _;
    return dropdown;
  };

  dropdown.viz = function(_) {
    if (!arguments.length) return viz;
    viz = _;
    return dropdown;
  };

  dropdown.addMenu = function(config) {
    if (!config.elementClassSelector) {
      config.elementClassSelector = `.${config.elementClass}`;
    }

    menus.push(config);
    return dropdown;
  };

  function paintDropdownSelect({
      $containerCurr,
      selectDropdownCb,
      dropdownCols,
      buttonText,
      buttonClass,
      dropdownClass,
    }) {
    let htmlSearchbox;

    // If there are more than a few dimensions, show a searchbox
    if (dropdownCols.length > 5) {
      htmlSearchbox = $('#column-actions-searchbox-template').html();
    } else {
      htmlSearchbox = '';
    }

    const colHtml = arc.misc.safe_substitute(
        columnActionsTemplate,
        {
          BUTTON_TEXT: buttonText,
          CLASS: buttonClass,
          DROPDOWN_CLASS: dropdownClass,
          DROPDOWN_ON_HOVER_CLASS: `${buttonClass}-hover`,
          HTML_SEARCHBOX: htmlSearchbox,
        }
    );

    $containerCurr.append(colHtml);
    $containerCurr.find('.dropdown-on-hover').hover(
      function () {
        $(this).not('.disabled-dropdown')
          .find('.dropdown-menu')
          .stop(true, true)
          .delay(200)
          .fadeIn(100);
      },

      function () {
        $(this).not('.disabled-dropdown')
          .find('.dropdown-menu')
          .stop(true, true)
          .delay(200)
          .fadeOut(100);
        }
      );

    const searchDropdownClass = dropdownClass ? `.${dropdownClass}` : '';
    const $search = $containerCurr.find(`${searchDropdownClass} .searchbox-container input`);
    $search.on('click', (e) => {
      e.stopPropagation();
    });

    const lis =
      d3.selectAll($containerCurr.toArray())
        .select(`.${dropdownClass}`)
        .selectAll('.nosel')
        .data(dropdownCols)
        .enter()
          .append('li')
          .classed(dropdownClass.replace(/e?s$/, ''), true);

    lis.append('a')
       .text((d_attr) => d_attr.alias || d_attr.name)
       .attr('data-col', (d, i) => i)
       .classed('dropdown-item', true);

    lis.on('click', selectDropdownCb);

    function sel_cb() {
      return lis;
    }

    function search_selection(d_attr, term) {
      const name = d_attr.alias || d_attr.name;

      if (name && name.toLowerCase().indexOf(term) >= 0) {
        return true;
      }

      return false;
    }

    arc.report.searchboxSetup({
      searchboxId: $search[0],
      search_cb: search_selection,
      sel_cb,
    });
  }

  function paintPivot($containerCurr) {
    const ds = viz.dataset();
    const drillColumns = viz.getDrillColumns();

    const dropdownCols = [];

    if (!ds) {
      return;
    }

    ds.tables().forEach((table) => {
      table.columns().forEach((column) => {
        const attr = column.attr();
        let current_alias = '';
        if (drillColumns !== undefined) {
          // eslint-disable-next-line no-prototype-builtins
          if (!drillColumns.hasOwnProperty(attr.alias || attr.name)) {
            return;
          }

          current_alias = drillColumns[attr.alias || attr.name];
        }

        if (!attr.hidden && attr.isdim) {
          dropdownCols.push(
            $.extend({}, {
              name: attr.alias || attr.name,
              alias: current_alias || attr.alias,
            })
          );
        }
      });
    });

    if (dropdownCols.length === 0) {
      return;
    }

    function selectDropdownCb(d_attr) {
      let complexFilterDetected = false;
      const filters = contextmenu.params()[0].filter((param) => {
        const isDim = (param.keytype === arc.report.columnTypeEnum.DIMENSION);
        const hasFunc = (
          (param.custom_expr && param.custom_expr !== param.expr) ||
          param.fn_name ||
          param.fn_cast
        );
        if (isDim && hasFunc) {
          complexFilterDetected = true;
        }

        return isDim;
      }).slice(0);

      if (complexFilterDetected) {
        arc.report.alertMessageSmall('Drill-Into, Include, and Exclude not ' +
          'supported for dimensions with functions.', 'alert-info');
        return;
      }

      // clear shelf history
      viz.shelfTrackerInstance().clearHistory();

      filters.forEach((filter) => {
        filter.source_pivot = true;
      });

      const paramConfig = {
        keytype: arc.report.columnTypeEnum.DIMENSION,
        value: [
          JSON.stringify({
            vizId: viz.report_id(),
            columnAlias: d_attr.alias || d_attr.name,
            columnName: d_attr.name,
            filters,
          }),
        ],
      };
      const paramObject = new arc.params.PivotParam(viz, paramConfig);

      viz.sendParameters([paramObject], function () {
        if (in_dashboard && arc.globals.click_behavior_cb && !viz.inModal()) {
          // eslint-disable-next-line prefer-rest-params
          arc.globals.click_behavior_cb.apply(viz, arguments);
        } else {
          let report = arc.globals.reports.getReport(viz.report_id());
          if (!report) {
            report = arc.report.getPrimaryReport();
          }

          if (report && report.visual_params_history && report.visual_params_history()) {
            // eslint-disable-next-line prefer-rest-params
            report.visual_params_history().add(arguments[6], viz, false);
          }
        }
      });
    }

    paintDropdownSelect({
      $containerCurr,
      selectDropdownCb,
      dropdownCols,
      buttonText: 'Drill Into',
      buttonClass: 'cm-lineabove',
      dropdownClass: 'pivot-dims',
    });
  }

  dropdown.handleContext = function(paramsl) {
    // Get filter names in a hash
    const filterNames = {};

    // eslint-disable-next-line array-callback-return
    paramsl[0].map((p) => {
      if (p.key) {
        filterNames[p.key] = true;
      }
    });

    // Hide all the pivot dims that have the same name as one of the filters
    // If we are filtering on, say, 'manufacturer', it does not make sense to
    //   also pivot on 'manufacturer' at the same time.
    d3.selectAll($container.toArray())
      .selectAll('li.pivot-dim')
      .classed('arc-invisible', false)
      .filter((d_attr) => {
        const name = d_attr.alias || d_attr.name;
        return name && (name.toLowerCase() in filterNames);
      })
      .classed('arc-invisible', true);

    const $pivotDims = $container.find('li.pivot-dim');
    const numberOfPivotDim = $pivotDims.length;
    const numberOfInvisiblePivotDim = $pivotDims.filter('.arc-invisible').length;

    const $contextItem = $container.find('.dropdown-on-hover:has(.dropdown-menu.pivot-dims)');
    if (numberOfInvisiblePivotDim === numberOfPivotDim) {
      $contextItem
        .addClass('disabled-dropdown')
        .attr('title', 'All dimensions already filtered.')
        .tooltip();
    } else {
      $contextItem
        .removeClass('disabled-dropdown')
        .removeAttr('data-original-title');
    }
  };

  dropdown.permitPivot = function(_) {
    if (!arguments.length) return permitPivot;
    permitPivot = _;
    return dropdown;
  };

  dropdown.permitMultiSelect = function(_) {
    if (!arguments.length) return permitMultiSelect;
    permitMultiSelect = _;
    return dropdown;
  };

  dropdown.paintDropdown = function (paintPivotDropdown, additionalMenus) {
    $container.empty();
    $container.append(contextmenuHdr);
    const additionalDropdown = [];

    // Add manual way of adding menus, needed for search visual titles
    if (contextmenu && additionalMenus) {
      additionalMenus.forEach(function methodName(menu) {
        if (menu.elementType === 'dropdownSelect') {
          additionalDropdown.push(menu);
          menu.elementCb = function handleSelectionCb(data) {
            contextmenu.update_search_cb(menu, data);
          };
        } else {
          menu.elementCb = function handleSelectionCb() {
            contextmenu.update_search_cb(menu);
          };

          menu.elementClassSelector = `.${menu.elementClass.split(' ').join('.')}`;

          dropdown.addMenu(menu);
        }
      });
    }

    // If length is zero, there will be a weird empty footer hanging near the bottom of the menu
    // So omit footer if there is no item to put inside
    if ((!menus.length && !additionalDropdown.length) || !showContextMenu) {
      return;
    }

    $container.append(contextmenuFoot);

    const $menuFoot = $container.find('.contextmenu-foot');

    const paintMenu = (menu, $contextMenuFooter) => {
      if (menu.elementType === 'dropdown') {
        const {
          id,
          buttonText,
          buttonClass,
          dropdownClass,
          items,
        } = menu;

        const element = arc.misc.safe_substitute(
          contextmenuDropdown,
          {
            ID: id,
            BUTTON_TEXT: buttonText,
            CLASS: buttonClass,
            DROPDOWN_CLASS: dropdownClass,
            DROPDOWN_ON_HOVER_CLASS: `${buttonClass}-hover`,
          }
        );

        $contextMenuFooter.append(element);

        const $dropdownContainer = $contextMenuFooter.find(`#${id}`);

        $dropdownContainer.hover(
          function () {
            $(this).not('.disabled-dropdown')
              .find('.dropdown-menu')
              .stop(true, true)
              .delay(200)
              .fadeIn(100);
          },
          function () {
            $(this).not('.disabled-dropdown')
              .find('.dropdown-menu')
              .stop(true, true)
              .delay(200)
              .fadeOut(100);
          }
        );

        const $menuList = $dropdownContainer.find(`.${dropdownClass}`);

        items.forEach((item) => paintMenu(item, $menuList));
      } else {
        const isButton = menu.elementType === 'button';
        const isInput = menu.elementType === 'input';

        let template = contextmenuLink;
        if (isButton || isInput) {
          template = isButton ? contextmenuButton : contextmenuInput;
        }

        const element = arc.misc.safe_substitute(template, {
          CLASS: menu.elementClass,
          TEXT: menu.elementText,
          NAME: menu.elementName,
          TYPE: menu.inputType || 'text',
        });

        $contextMenuFooter.append(element);

        const $menuElement = $contextMenuFooter.find(menu.elementClassSelector);

        if (isInput) {
          $menuElement.offOn('keydown.cmEvent', (e) => {
            const isEnter = e.keyCode === 13;
            if (isEnter) {
              const newValue = {
                name: e.target.value,
                originalValue: e.target.dataset.originalvalue,
              };
              contextmenu.update_search_cb(menu, newValue);
            }
          });
          // Otherwise, jquery focusin events prevents focusing on input
          $menuElement.offOn('focusin', (e) => {
            e.stopPropagation();
          });
        } else {
          $menuElement.offOn('click.cmEvent', menu.elementCb);
        }
      }
    };

    menus.forEach((menu) => paintMenu(menu, $menuFoot));

    // ARC-11090: Don't draw in search visual for now
    if (permitPivot && paintPivotDropdown) {
      paintPivot($menuFoot);
    }

    additionalDropdown.forEach((menu) => {
      paintDropdownSelect({
        $containerCurr: $menuFoot,
        selectDropdownCb: menu.elementCb,
        dropdownCols: menu.dropdownCols,
        buttonText: menu.buttonText,
        buttonClass: menu.buttonClass,
        dropdownClass: menu.dropdownClass,
      });

      // Otherwise, jquery focusin events prevents focusing on input
      $menuFoot.find(`.${menu.dropdownClass} input`)
               .offOn('focusin', (e) => {
                 e.stopPropagation();
               });
    });
  };

  dropdown.destroy = function() {
    if ($container) {
      $container.find('.dropdown-on-hover').off();
      $container.find('.searchbox-container input').off();
      menus.forEach((menu) => {
        $container.find(`.contextmenu-foot ${menu.elementClassSelector}`)
                  .off('click');
      });

      $container = $();
    }

    menus = [];

    return dropdown;
  };

  return dropdown;
}

arc.contextmenu = function() {
  let dom_id;
  let mark_type;
  let tooltips = [];
  let selections = [];
  let in_dashboard = false;
  let marker_name = null;
  let dashboard_click_behavior;
  let click_behavior = null;
  let click_behavior_change = null;
  let text_fn = null;
  let params_fn = null;

  // In display mode, the callback to invoke when a contextmenu
  // element is clicked
  let click_behavior_cb = null;

  // For not overwriting highlighted stacked bar's context menu
  let associatedMark;

  let filter_cb = null;
  let save_segment_cb;
  let activeParamsl;
  let active_d;
  let active_i;
  let active_this;
  let highlight_on_select;
  let highlight_cb;
  let menu;
  let viz;
  let tooltips_enabled = true;
  let dataset_id;
  let show_cb;
  let hide_cb;
  let search_menus;
  let setup_cb;
  let permitIncludeAndExclude = true;
  let permitPivot;
  let dropdown;
  let getMarksCb;
  let clickEnabled = true;
  let combineWithOr = false;
  let mapLayerId; // for mapbox map
  let click_position; // to keep track of mouse position when clicking

  const EnumClickTypes = {
    None: 0,
    FilterOnClick: 1,
    LinkOnClick: 2,
    ShowMenuOptions: 3,
  };

  const contextmenu = function() { return 'arc.contextmenu'; };

  function matchMark(params, kvs) {
    if (!params || params.length !== kvs.length) {
      return false;
    }

    const kvh = {};
    params.forEach((p) => {
      kvh[p.key] = p.value;
    });

    let match = true;
    kvs.some((p) => {
      if (kvh[p.key] !== p.value) {
        match = false;
        return true;
      }

      return false;
    });

    return match;
  }

  contextmenu.params = function() {
    return activeParamsl;
  };

  contextmenu.highlightMarks = function(kvs) {
    if (highlight_cb || highlight_on_select) {
      selections.forEach((selection) => {
        selection.each(function(d, i) {
          const ps = params_fn(d, i, this, {
            notForCurrentParams: true,
          });

          if (matchMark(ps, kvs)) {
            if (highlight_cb) {
              highlight_cb(d, i, this);
            } else if (highlight_on_select) {
              applyFilter(this, true);
            }
          }
        });
      });
    }

    return contextmenu;
  };

  contextmenu.mark_type = function(_) {
    if (!arguments.length) return mark_type;
    mark_type = _;
    return contextmenu;
  };

  contextmenu.getMarksCb = function(_) {
    if (!arguments.length) return getMarksCb;
    getMarksCb = _;
    return contextmenu;
  };

  contextmenu.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return contextmenu;
  };

  contextmenu.associatedMark = function(_) {
    if (!arguments.length) return associatedMark;
    associatedMark = _;
    return contextmenu;
  };

  contextmenu.permitIncludeAndExclude = function(_) {
    if (!arguments.length) return permitIncludeAndExclude;
    permitIncludeAndExclude = _;
    return contextmenu;
  };

  contextmenu.permitPivot = function(_) {
    if (!arguments.length) return permitPivot;
    permitPivot = _;
    return contextmenu;
  };

  contextmenu.show_cb = function(_) {
    if (!arguments.length) return show_cb;
    show_cb = _;
    return contextmenu;
  };

  contextmenu.hide_cb = function(_) {
    if (!arguments.length) return hide_cb;
    hide_cb = _;
    return contextmenu;
  };

  contextmenu.search_menus = function(_) {
    if (!arguments.length) return search_menus;
    search_menus = _;
    return contextmenu;
  };

  contextmenu.viz = function(_) {
    if (!arguments.length) return viz;
    viz = _;
    return contextmenu;
  };

  contextmenu.highlight_on_select = function(_) {
    if (!arguments.length) return highlight_on_select;
    highlight_on_select = _;
    return contextmenu;
  };

  contextmenu.highlight_cb = function(_) {
    if (!arguments.length) return highlight_cb;
    highlight_cb = _;
    return contextmenu;
  };

  contextmenu.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return contextmenu;
  };

  contextmenu.save_segment_cb = function(_) {
    if (!arguments.length) return save_segment_cb;
    save_segment_cb = _;
    return contextmenu;
  };

  contextmenu.dashboard_click_behavior = function(_) {
    if (!arguments.length) return dashboard_click_behavior;
    dashboard_click_behavior = _;
    return contextmenu;
  };

  contextmenu.update_search_cb = function(menuItem, data) {
    click_behavior_cb(menuItem, data, active_this);
    contextmenu.hide();
  };

  contextmenu.click_behavior_cb = function(_) {
    if (!arguments.length) return click_behavior_cb;
    click_behavior_cb = _;
    return contextmenu;
  };

  contextmenu.filter_cb = function(_) {
    if (!arguments.length) return filter_cb;
    filter_cb = _;
    return contextmenu;
  };

  contextmenu.marker_name = function(_) {
    if (!arguments.length) return marker_name;
    marker_name = _;
    return contextmenu;
  };

  contextmenu.click_behavior = function(_) {
    if (!arguments.length) return click_behavior;
    click_behavior = _;
    return contextmenu;
  };

  contextmenu.click_behavior_change = function(_) {
    if (!arguments.length) return click_behavior_change;
    click_behavior_change = _;
    return contextmenu;
  };

  contextmenu.in_dashboard = function(_) {
    if (!arguments.length) return in_dashboard;
    in_dashboard = _;
    return contextmenu;
  };

  contextmenu.text_fn = function(_) {
    if (!arguments.length) return text_fn;
    text_fn = _;
    return contextmenu;
  };

  contextmenu.mapLayerId = function(_) {
    if (!arguments.length) return mapLayerId;
    mapLayerId = _;
    return contextmenu;
  };

  contextmenu.params_fn = function(_) {
    if (!arguments.length) return params_fn;
    params_fn = _;
    return contextmenu;
  };

  contextmenu.addTooltip = function(_) {
    if (!arguments.length) return tooltips;
    tooltips.push(_);

    // Add in selections from the added tooltip
    _.selections().forEach((selection) => {
      contextmenu.addSelection(selection);
    });

    return contextmenu;
  };

  contextmenu.setup_cb = function(_) {
    if (!arguments.length) return setup_cb;
    setup_cb = _;
    return contextmenu;
  };

  contextmenu.combineWithOr = function(_) {
    if (!arguments.length) return combineWithOr;
    combineWithOr = _;
    return contextmenu;
  };

  // Add in d3 selections that point at dom elements. These dom
  // elements need to have this contextmenu associated with them.
  contextmenu.addSelection = function(_) {
    if (!arguments.length) return selections;
    selections.push(_);
    return contextmenu;
  };

  contextmenu.tooltips_enabled_local = function(_) {
    if (!arguments.length) return tooltips_enabled;

    tooltips_enabled = _;
    if (tooltips_enabled) {
      tooltips.forEach((tt) => {
        tt.enable();
      });
    } else {
      tooltips.forEach((tt) => {
        tt.disable();
      });
    }

    clickEnabled = tooltips_enabled;

    return contextmenu;
  };

  contextmenu.tooltips_enabled = function(_) {
    if (!arguments.length) return tooltips_enabled;

    if (getMarksCb) {
      getMarksCb().forEach((marks) => {
        const cm = marks.contextmenu();
        if (cm) {
          cm.tooltips_enabled_local(_);
        }
      });
    }

    if (tooltips_enabled !== _) {
      contextmenu.tooltips_enabled_local(_);
    }

    return contextmenu;
  };

  contextmenu.hide = function() {
    fadeOut($(dom_id));
    d3.select(document)
      .on('click.contextmenu', null);
    contextmenu.tooltips_enabled(true);
    if (hide_cb) {
      hide_cb(contextmenu);
    }
  };

  function placeInFilters(inout) {
    let complexFilterDetected;

    function filterToDims(param) {
      const isDim = param.keytype === arc.report.columnTypeEnum.DIMENSION;
      const hasFunc = (param.custom_expr || param.fn_name || param.fn_cast);
      if (isDim && hasFunc) {
        complexFilterDetected = true;
      }

      return param.keytype !== arc.report.columnTypeEnum.AGGREGATE &&
             !param.computed && !hasFunc;
    }

    for (let i = 0; i < activeParamsl.length; i++) {
      const activeParams = activeParamsl[i];
      complexFilterDetected = false;
      const params = activeParams.filter(filterToDims);

      if (complexFilterDetected) {
        arc.report.alertMessageSmall('Drill-Into, Include, and Exclude not ' +
          'supported for dimensions with functions.', 'alert-info');
      } else if (inout) {
          const activeParamsValue = activeParams[i].value;
          const alertHeader = `${activeParamsValue} has been successfully added to the filters.`;
          const alertMessage = 'Filters can be found in the Build panel. <br>Please refresh the visual to see the changes.';

          displayInOutAlert(alertHeader, alertMessage);
      }

      if (filter_cb) {
        filter_cb(params, inout);
      }
    }

    contextmenu.hide();
  }

  function displayInOutAlert(header, message) {
    arc.report.alertMessageSmallBase({
      type: 'alert-info',
      alertHeader: header,
      alertMessage: message,
      fadeOut: true,
      fadeOutDelay: 8000,
      useHTML: true,
    });
  }

  function includeInFilters() {
    return placeInFilters(true);
  }

  function excludeInFilters() {
    return placeInFilters(false);
  }

  function filterAll() {
    contextmenu.filterAllParams(activeParamsl[0], active_d, active_i, active_this);
    contextmenu.hide();
  }

  function sendParameters() {
    contextmenu._activateClick(active_d, active_i, active_this,
        activeParamsl[0], click_position);
    contextmenu.hide();
  }

  function setLinkOnClick() {
    fadeOut($(dom_id));

    arc.report.reportPicker(click_behavior,
      (d, cb, passParams, openTarget) => {
        const cl = {};
        if (click_behavior) {
          cl.dashboard_behavior =
            click_behavior.dashboard_behavior;
        }

        if (d) {
          cl.report_id = d.id;
          cl.shouldPassParams = passParams;
          cl.openTarget = openTarget;
        } else {
          cl.report_id = 0;
        }

        click_behavior = cl;
        if (arc.globals.is_dashboard) {
          dashboard_click_behavior[marker_name] = cl;
        }

        click_behavior_change(cl);
        updateContextMenu();
        cb();
      }, dataset_id);
  }

  function updateContextMenu() {
    const $menu = $(dom_id);
    if (!$menu || !$menu.length) {
      contextmenu.hide();
      return;
    }

    const $navLink = $menu.find('.cm-nav-link');
    const $testLink = $menu.find('.cm-test-link');
    const $enableClick = $menu.find('.cm-enableclick');
    const $disableClick = $menu.find('.cm-disableclick');
    const $navigateLink = $menu.find('.cm-navigateclick');

    switch (getEnabledClickBehavior()) {
      case EnumClickTypes.FilterOnClick:
        $enableClick.addClass('selected');
        $navigateLink.removeClass('selected');
        $disableClick.removeClass('selected');

        $navLink.css('display', 'none');
        $testLink.css('display', 'block');

        break;

      case EnumClickTypes.LinkOnClick:
        // eslint-disable-next-line no-case-declarations
        let temp_report_id;
        if (arc.globals.is_dashboard) {
          temp_report_id = dashboard_click_behavior[marker_name].report_id;
        } else {
          temp_report_id = click_behavior && click_behavior.report_id;
        }

        $enableClick.removeClass('selected');
        $navigateLink.addClass('selected');
        $disableClick.removeClass('selected');

        $navLink.css('display', 'block');
        $testLink.css('display', 'none');

        $navLink.text(`Navigate to ID: ${temp_report_id}`);
        break;

      default:
        $enableClick.removeClass('selected');
        $navigateLink.removeClass('selected');
        $disableClick.addClass('selected');

        $navLink.css('display', 'none');
        $testLink.css('display', 'none');
    }
  }

  function setClickBehavior() {
    // If a link was already setup, remove it.
    if (click_behavior && click_behavior.report_id) {
      click_behavior.report_id = 0;
    }

    click_behavior_change(dashboard_click_behavior[marker_name]);
    updateContextMenu();
  }

  function disableClick() {
    const clickBehavior = getEnabledClickBehavior();

    if ((clickBehavior !== EnumClickTypes.FilterOnClick) &&
        (clickBehavior !== EnumClickTypes.LinkOnClick)) {
      return;
    }

    dashboard_click_behavior[marker_name] = false;
    setClickBehavior();
  }

  function enableClick() {
    if (getEnabledClickBehavior() === EnumClickTypes.FilterOnClick) {
      return;
    }

    dashboard_click_behavior[marker_name] = true;
    setClickBehavior();
  }

  function isInModal() {
    return ($('.super-modal').css('display') === 'block');
  }

  function addDetailedDataButton() {
    // Add check b/c dataset is not defined for visuals plotted on Activity Log page
    if (viz && viz.permitDetailData() && viz.dataset()) {
      const detailData = arc.detaildata.display()
                          .viz(viz)
                          .combineWithOr(combineWithOr)
                          .contextMenu(contextmenu);

      dropdown.addMenu({
        elementType: 'button',
        elementClass: 'cm-show-detailed-data',
        elementText: 'Show Detailed Data',
        elementCb: detailData.showDetail,
      });
    }
  }

  function createMenu() {
    // In case we are not on a dashboard the dashboard.loadTitleSubtitle fails
    // during Drill into
    let paintPivotDropdown = in_dashboard;
    const inModal = isInModal();

    let showContextMenu = true;
    if (viz) {
      if (viz.chart_div) {
        // In case we are on the search_modal or search visual,
        // the dashboard.loadTitleSubtitle fails -> Drill into not enabled
        paintPivotDropdown = paintPivotDropdown && !viz.chart_div().includes('search-visual');
      }

      if (viz.showContextMenu) {
        showContextMenu = viz.showContextMenu();
      }
    }

    // Create a dom element for the menu if we need to
    if (!dom_id) {
      dom_id = `contextmenu-${arc.contextmenu.id}`;
      // eslint-disable-next-line no-plusplus
      ++arc.contextmenu.id;
      arc.createContextMenuContainer(dom_id);
      dom_id = `#${dom_id}`;
    }

    // Create the detailed menu element. Add in the individual items,
    // and make them click-able
    menu = d3.select(dom_id)
             .datum({
               menu: contextmenu,
             });

    const $menu = $(dom_id);
    $menu.css('opacity', 0);
    $menu.css('z-index', '1200');
    dropdown = dropDown().viz(viz)
                         .in_dashboard(in_dashboard)
                         .contextmenu(contextmenu)
                         .permitPivot(permitPivot)
                         .showContextMenu(showContextMenu)
                         .container($menu);

    addDetailedDataButton();
    if (!arc.globals.is_edit) {
      // We are setup to send out filters on click without a menu dropdown
      if (in_dashboard && getEnabledClickBehavior() === EnumClickTypes.FilterOnClick) {
        return;
      }

      if (in_dashboard && !inModal) {
        dropdown.addMenu({
          elementType: 'button',
          elementClass: 'cm-filterall',
          elementText: 'Filter All',
          elementCb: filterAll,
        });
      }

      dropdown.paintDropdown(paintPivotDropdown, search_menus);
      return;
    }

    if (marker_name) {
      if (in_dashboard) {
        if (!inModal) {
          dropdown.addMenu({
            elementType: 'button',
            elementClass: 'cm-filterall',
            elementText: 'Filter All',
            elementCb: filterAll,
          });

          const isContextMenuEnabled = arc.globals.dashboard?.displayContextMenu?.();

          dropdown.addMenu({
            id: 'click-behaviors',
            elementType: 'dropdown',
            buttonText: 'Click Behaviour',
            buttonClass: 'cm-lineabove',
            dropdownClass: 'click-behaviours',
            items: [
              {
                elementType: 'button',
                elementClass: 'cm-disableclick',
                elementClassSelector: '.cm-disableclick',
                elementText: isContextMenuEnabled ? 'Show Context Menu on Click' : 'No Action on Click',
                elementCb: disableClick,
              },
              {
                elementType: 'button',
                elementClass: 'cm-enableclick',
                elementClassSelector: '.cm-enableclick',
                elementText: 'Enable Filter on Click',
                elementCb: enableClick,
              },
              {
                elementType: 'link',
                elementClass: 'cm-test-link',
                elementClassSelector: '.cm-test-link',
                elementText: 'Test Click Behavior',
                elementCb: sendParameters,
              },
            ]
          });
        }
      }
    }

    if ((in_dashboard && !inModal) || (!in_dashboard && inModal)) {
      dropdown.addMenu({
        elementType: 'button',
        elementClass: 'cm-navigateclick',
        elementText: 'Navigate to Dashboard...',
        elementCb: setLinkOnClick,
      });
      dropdown.addMenu({
        elementType: 'link',
        elementClass: 'cm-nav-link',
        elementText: 'Navigate to ID',
        elementCb: sendParameters,
      });
    }

    if (permitIncludeAndExclude && !inModal) {
      dropdown.addMenu({
        elementType: 'button',
        elementClass: 'cm-include',
        elementText: 'Include',
        elementCb: includeInFilters,
      });

      dropdown.addMenu({
        elementType: 'button',
        elementClass: 'cm-exclude',
        elementText: 'Exclude',
        elementCb: excludeInFilters,
      });
    }

    dropdown.paintDropdown(paintPivotDropdown, search_menus);
    updateContextMenu();
  }

  function getEnabledClickBehavior() {
    let clickType = '';
    if (!arc.globals.is_dashboard && isLinkNavigationEnabled()) {
      clickType = EnumClickTypes.LinkOnClick;
    } else if (!dashboard_click_behavior) {
      clickType = showMenuOptions();
    } else if (typeof dashboard_click_behavior[marker_name] === 'object' &&
                isLinkNavigationEnabled()) {
      clickType = EnumClickTypes.LinkOnClick;
    } else if (typeof dashboard_click_behavior[marker_name] === 'boolean' &&
                dashboard_click_behavior[marker_name]) {
      clickType = EnumClickTypes.FilterOnClick;
    } else {
      clickType = showMenuOptions();
    }

    function showMenuOptions() {
      if (
        (!arc.globals.is_dashboard && arc.globals.is_edit) ||
        (arc.globals.dashboard && arc.globals.dashboard.displayContextMenu &&
          arc.globals.dashboard.displayContextMenu())
      ) {
        return EnumClickTypes.ShowMenuOptions;
      }

      return EnumClickTypes.None;
    }

    function isLinkNavigationEnabled() {
      return (click_behavior && click_behavior.report_id !== 0);
    }

    return clickType;
  }

  contextmenu._setSelection = function(peer_selections, in_d, in_i, elem) {
    active_d = in_d;
    active_i = in_i;
    active_this = elem;

    const t = text_fn(in_d, in_i, elem);
    activeParamsl = search_menus ?
      [] :
      [params_fn(in_d, in_i, elem, viz ? !viz.sendAggregateParameters() : undefined)];
    menu.select('.contextmenu-hdr')
        .html(t);
    return contextmenu;
  };

  contextmenu._setXY = function(pageX, pageY) {
    // eslint-disable-next-line no-undef
    let ev = d4.event || d3.event;
    if (ev && ev.sourceEvent) {
      ev = ev.sourceEvent;
    }

    pageX = pageX || ev.pageX;
    pageY = pageY || ev.pageY;

    let xpos = pageX + 10;
    let ypos = pageY;
    if (xpos > $(window).width() - $(dom_id).width() - 35) {
      xpos = pageX - $(dom_id).width() - 20;
      if (xpos < 10) {
        xpos = 10;
      }
    }

    if (ypos > $(window).height() - $(dom_id).height() - 20) {
      ypos = pageY - $(dom_id).height() - 20;
      if (ypos < 10) {
        ypos = 10;
      }
    }

    menu.style('top', `${ypos}px`)
        .style('left', `${xpos}px`);
  };

  contextmenu.show = function(peer_selections, d, i, elem) {
    // Setup the menu's context
    contextmenu._setSelection(peer_selections, d, i, elem);

    if (!search_menus && !activeParamsl[0]) {
      return;
    }

    if (!search_menus && activeParamsl[0].length === 0) {
      return;
    }

    if (activeParamsl[0]) {
      activeParamsl[0] = activeParamsl[0].map(((item) => {
        if (typeof item.value === 'string') {
          item.value = item.value.startsWith('<span data-row-number=') ? '' : item.value;
        }

        return item;
      })).filter((item) => item.value !== '');
    }

    if (!search_menus && dropdown) {
      // Let the dropdown menu adjust to the context
      dropdown.handleContext(activeParamsl);
    }

    // Hide other contextmenus
    d3.selectAll('.contextmenu')
      .filter(function() {
        return $(this).css('display') === 'block';
      })
      // eslint-disable-next-line no-shadow
      .each((d) => {
        d.menu.hide();
      });

    if (show_cb) {
      show_cb(contextmenu, activeParamsl, elem, menu.node());
    }

    // Make the menu visible
    menu.style('opacity', 0)
        .style('display', 'block');

    if (mapLayerId) {
      contextmenu._setXY(d.originalEvent.pageX, d.originalEvent.pageY);
    } else {
      contextmenu._setXY();
    }

    menu.style('opacity', 1);

    contextmenu.tooltips_enabled(false);
    let first_click = true;

    // Setup a document click handler that will hide the menu when
    // there is a click anywhere else
    d3.select(document)
      .on('click.contextmenu', () => {
        if (first_click) {
          first_click = false;
          return;
        }

        const xpos = d3.event.pageX;
            const ypos = d3.event.pageY;
            const $menu_sel = $(menu.node());
            const menu_offset = $menu_sel.offset();

        let in_menu = false;
        if ((menu_offset.left <= xpos) &&
            (menu_offset.left + $menu_sel.outerWidth(true) >= xpos) &&
            (menu_offset.top <= ypos) &&
            (menu_offset.top + $menu_sel.outerHeight(true) >= ypos)) {
          in_menu = true;
        }

        if (!in_menu) {
          contextmenu.hide();
        }

        return true;
      });

    return true;
  };

  function areWeInSameAppAsTarget(target_id) {
    target_id = parseInt(target_id, 10);
    const dashboard_id = viz.dashboard() ? viz.dashboard().dashboard_id() : viz.report_id();
    if (!arc.globals.appgroup) {
      // We are not in an app
      return false;
    }

    const ms = arc.globals.appgroup.appgroup_membership();
    if (ms.some((m) => (m[0] === dashboard_id))) {
      // we are in a visual within app
      if (target_id) {
        if (ms.some((m) => (m[0] === target_id))) {
          // target is in same app as visual
          return true;
        }
      }

      // target is not in same app as visual
      return false;
    }

    // we are not in a visual within app
    return false;
  }

  contextmenu.filterAllParams = function(params, datum, index, elem) {
    if (arc.globals.click_behavior_cb) {
      if (params === undefined) {
        if (params_fn) {
          params = params_fn(datum, index, elem);
        }
      }

      viz.sendParameters(params, click_behavior_cb);
    }
  };

  // eslint-disable-next-line no-shadow
  contextmenu._activateClick = function(datum, index, elem, params, click_position) {
    if (getEnabledClickBehavior() === EnumClickTypes.LinkOnClick) {
      if (areWeInSameAppAsTarget(click_behavior.report_id)) {
        let paramsToSend;
        if (click_behavior.shouldPassParams) {
          paramsToSend = params || params_fn(datum, index, elem, !viz.sendAggregateParameters());
        } else {
          paramsToSend = null;
        }

        arc.globals.appgroup.navigateToApp(click_behavior.report_id, null, paramsToSend);
        return;
      }

      viz.click_behavior_cb_default(datum, index, click_behavior,
          params_fn, viz, elem, params);
      return;
    }

    contextmenu.filterAllParams(params, datum, index, elem);
    if (!arc.globals.enable_tooltip_highlights) {
      return;
    }

    if (highlight_cb) {
      highlight_cb(datum, index, elem, click_position);
    } else if (highlight_on_select) {
      applyFilter(elem, true);
    }
  };

  function setupClickDropdown() {
    let suffix = '.contextmenu';
    if (mark_type) {
      suffix += `_${mark_type}`;
    }

    if (mapLayerId) {
      selections.forEach((map) => {
        map.on('click', (e) => {
          if (!clickEnabled) {
            return;
          }

          click_position = [e.originalEvent.clientX, e.originalEvent.clientY];

          contextmenu.show(map, e, 0, e.target);
        });
      });

      return;
    }

    // Bind to the click items of the selections, popping up the menu
    // when this happens
    selections.forEach((sel) => {
      sel.on(`click${suffix}`, (d, i) => {
        if (!clickEnabled) {
          return;
        }

        // eslint-disable-next-line no-undef
        const evt = d4.event || d3.event;
        if (evt) {
          click_position = [evt.clientX, evt.clientY];
          contextmenu.show(sel, d, i, evt.target);
        }
      });
    });
  }

  function setupBehaviors() {
    // in dashboard view mode, or when link navigation is enabled
    if ((!arc.globals.is_edit && in_dashboard &&
            getEnabledClickBehavior() === EnumClickTypes.FilterOnClick) ||
        (!arc.globals.is_edit && getEnabledClickBehavior() === EnumClickTypes.LinkOnClick)
    ) {
      let suffix = '.contextmenu';
      if (mark_type) {
        suffix += `_${mark_type}`;
      }

      if (contextmenu.mapLayerId()) {
        selections.forEach((map) => {
          map.on('click', (e) => contextmenu._activateClick(e, 0, e.originalEvent.target));
        });
      } else {
        selections.forEach((sel) => {
          sel.style('cursor', 'pointer')
          .on(`click${suffix}`, (d, i) => {
            // eslint-disable-next-line no-undef
            const evt = d4.event || d3.event;
            contextmenu._activateClick(d, i, evt.target);
          })
          .on(`touchstart${suffix}`, (d, i) => {
            // eslint-disable-next-line no-undef
            const evt = d4.event || d3.event;
            contextmenu._activateClick(d, i, evt.target);
          });
        });
      }
    } else if (!(!arc.globals.is_edit && in_dashboard &&
                getEnabledClickBehavior() !== EnumClickTypes.ShowMenuOptions)) {
      // in view mode of dashboard when context menu should be shown
      // in edit mode for both dashboards and visuals
      setupClickDropdown();

      // else case (opposite of condition)
      // would be in view mode of dashboard when context menu should not be shown
    }

    if (setup_cb) {
      setup_cb();
    }
  }

  contextmenu.activate = function() {
    if (selections.length === 0) {
      return;
    }

    createMenu();
    setupBehaviors();
    return contextmenu;
  };

  contextmenu.destroy = function() {
    if (menu) {
      menu.remove();
    }

    if (dropdown) {
      dropdown.destroy();
    }

    click_behavior_cb = undefined;
    filter_cb = undefined;
    params_fn = undefined;
    highlight_cb = undefined;

    selections.forEach((sel) => {
      sel.on('touchstart.contextmenu', null)
         .on('click.contextmenu', null);
    });

    selections = [];
    tooltips = [];
  };

  return contextmenu;
};

arc.contextmenu.id = 1;

let hidePreviouslyFrozenTooltip;

arc.tooltip = function() {
  let dom_id = null;
  let mark_type;
  let tooltip_sel;
  let isFrozen = false;
  let selections = [];
  let highlight_cb;
  let associatedMark; // For not overwriting highlighted stacked bar's context menu
  let enableTooltipHighlights = false;
  let text_changes_on_move = false;
  let enabled = true;
  let current_text_fn;
  let showingTooltip;
  let mapLayerId; // Mapbox map has multiple layers, need to know what layer to add listeners to

  let text_fn = (txt) => `<span class="viz-tooltip-text">${txt}</span>`;

  let getXPos = function(baseX) {
    if (baseX === undefined) {
      // eslint-disable-next-line no-undef
      const evt = d4.event || d3.event;
      baseX = evt.pageX;
    }

    // eslint-disable-next-line no-shadow
    const $dom_id = $(dom_id);
    let xpos = baseX + 10;
    if (xpos > $(window).width() - $dom_id.width() - 20) {
      xpos = baseX - $dom_id.width() - 20;
      if (xpos < 10) {
        xpos = 10;
      }
    }

    return xpos;
  };

  let getYPos = function(baseY) {
    if (baseY === undefined) {
      // eslint-disable-next-line no-undef
      const evt = d4.event || d3.event;
      baseY = evt.pageY;
    }

    // eslint-disable-next-line no-shadow
    const $dom_id = $(dom_id);

    /*
     * If the tooltip is drawn in a position that overlaps with the mouse, then
     * we will immediately trigger mouseout on the underlying element, and that
     * will undraw the tooltip. As soon as the tooltip undraws, we get mouseover
     * on the underlying element and the tooltip appears. And so on.
     *
     * Fix the y position so it is always south of the mouse position. That way,
     * we will never get to fast-blinking tooltips.
     */
    let ypos = baseY;
    if (ypos > $(window).height() - $dom_id.height() - 20) {
      ypos = baseY - $dom_id.height() - 20;
      if (ypos < 10) {
        ypos = 10;
      }
    }

    return ypos;
  };

  const tooltip = function() { };

  function highlightsEnabled() {
    return enableTooltipHighlights && arc.globals.enable_tooltip_highlights;
  }

  tooltip.mark_type = function(_) {
    if (!arguments.length) return mark_type;
    mark_type = _;
    return tooltip;
  };

  tooltip.enableTooltipHighlights = function(_) {
    if (!arguments.length) return enableTooltipHighlights;
    enableTooltipHighlights = _;
    return tooltip;
  };

  tooltip.disable = function() {
    enabled = false;
    if (dom_id !== null) {
      if (showingTooltip && !isFrozen) {
        fadeOut($(dom_id));
        showingTooltip = false;
      }
    }

    if (associatedMark && associatedMark.tooltips()) {
      associatedMark.tooltips().disable();
    }

    return tooltip;
  };

  tooltip.enable = function() {
    enabled = true;

    if (associatedMark && associatedMark.tooltips()) {
      associatedMark.tooltips().enable();
    }

    return tooltip;
  };

  tooltip.associatedMark = function(_) {
    if (!arguments.length) return associatedMark;
    associatedMark = _;
    return tooltip;
  };

  tooltip.redraw = function() {
    if (current_text_fn) {
      const $domId = $(dom_id);
      const t = current_text_fn();
      if (t) {
        fadeIn($domId);
        $domId.empty().html(t);
        showingTooltip = true;
        let marginForCurrentParams = (t.length - 200) / 4;
        marginForCurrentParams = Math.min(marginForCurrentParams, 200);
        marginForCurrentParams = Math.max(marginForCurrentParams, 110);
        $domId.css('left', `-${marginForCurrentParams}px`);
      } else if (showingTooltip) {
          fadeOut($domId);
          showingTooltip = false;
          isFrozen = false;
        }
    }

    return tooltip;
  };

  function hideTooltip(_this) {
    if (arc.frontend.globals.isMobileView) {
      // TODO: See the TODO in the mouseout.tooltip handler in this file
      tooltip_sel.on('click.dismiss', null);
    }

    if (!enabled) {
      return;
    }

    if (highlightsEnabled()) {
      if (highlight_cb) {
        highlight_cb(undefined, undefined, _this);
      } else {
        unhighlight(_this);
      }
    }

    current_text_fn = undefined;
    isFrozen = false;
    if (showingTooltip) {
      fadeOut($(dom_id));
      showingTooltip = false;
    }
  }

  tooltip.setXY = function(xPos, yPos) {
    tooltip_sel.style({
      top: `${getYPos(yPos)}px`,
      left: `${getXPos(xPos)}px`,
      opacity: 1,
      'z-index': '1200', // necessary to go above $('.super-modal')
                         // z-index of 1000
    });

    return tooltip;
  };

  tooltip.activate = function() {
    if (selections.length === 0) {
      return tooltip;
    }

    if (dom_id === null) {
      dom_id = `tooltip-${arc.tooltip.id}`;
      // eslint-disable-next-line no-plusplus
      ++arc.tooltip.id;
      d3.select('body')
          .append('div')
          .attr('id', dom_id)
          .classed('viz-tooltip', true)
          .classed('tooltip', true)
          .style('display', 'none');

      tooltip.dom_id(`#${dom_id}`);
    }

    let suffix = '.tooltip';
    if (mark_type) {
      suffix += `_${mark_type}`;
    }

    const mouseoverFunction = function(d, i) {
      if (!enabled) {
        return true;
      }

      if (hidePreviouslyFrozenTooltip) {
        hidePreviouslyFrozenTooltip();
      }

      // eslint-disable-next-line no-undef
      const evt = d4.event || d3.event;

      if (highlightsEnabled()) {
        if (highlight_cb) {
          const click_position = [evt.clientX, evt.clientY];
          highlight_cb(d, i, this, click_position);
        } else {
          highlight(this);
        }
      }

      let text;
      if (evt) {
        text = text_fn(d, i, evt.target);
      } else {
        text = text_fn(d);
      }

      if (text) {
        tooltip_sel.html(text);
        if (arc.frontend.globals.isMobileView) {
          tooltip_sel.on('click.dismiss', null);
        }

        if (tooltip.mapLayerId()) {
          const pointX = d.originalEvent.pageX;
          const pointY = d.originalEvent.pageY;
          tooltip.setXY(pointX, pointY);
        } else {
          tooltip.setXY();
        }

        fadeIn($(dom_id));
        showingTooltip = true;

        if (arc.frontend.globals.isMobileView) {
          // TODO: See the TODO in the mouseout.tooltip handler in this file
          setTimeout(() => {
           tooltip_sel.on('click.dismiss', function() {
             hideTooltip(this);
           });
          }, 0);
        }
      }
    };

    const mouseoutFunction = function() {
      // TODO: Note that there is a problem if the tooltip position
      // overlaps the mark we draw the tooltip for. The tooltip display
      // immediately triggers a mouseout event on the mark, and the
      // tooltip disappears.
      // This is being fixed explicitly for mobile by disabling the
      // mouseout event for mobile and instead having the user click on
      // the tooltip itself to close it. This is clearly not ideal, a
      // better solution should be found that works for both cases.

      if (arc.frontend.globals.isMobileView) {
        // On mobile, ignore mouseout
        return;
      }

      if (showingTooltip && !isFrozen) {
        hideTooltip(this);
        showingTooltip = false;
      }
    };

    selections.forEach((sel) => {
      if (mapLayerId) {
        sel.on('mousemove', mapLayerId, mouseoverFunction);
        sel.on('mouseout', mapLayerId, mouseoutFunction);
      } else {
        sel.on(`mouseover${suffix}`, mouseoverFunction)
           .on(`mouseout${suffix}`, mouseoutFunction);
      }

      if (text_changes_on_move) {
        sel.on(`mousemove${suffix}`, function(d, i) {
          if (!enabled) {
            return true;
          }

          // eslint-disable-next-line no-undef
          const evt = d4.event || d3.event;
          evt.stopPropagation();

          if (highlightsEnabled()) {
            if (highlight_cb) {
              const click_position = [evt.clientX, evt.clientY];
              highlight_cb(d, i, this, click_position);
            } else {
              highlight(this);
            }
          }

          const t = text_fn(d, i, this);
          if (t) {
            tooltip_sel.html(t);
            tooltip.setXY();
            fadeIn($(dom_id));
            showingTooltip = true;
          } else if (showingTooltip) {
              fadeOut($(dom_id));
              showingTooltip = false;
            }
        });
      } else {
        sel.on('mousemove.notooltip', () => {
          // eslint-disable-next-line no-undef
          const evt = d4.event || d3.event;
          evt.stopPropagation();
        });
      }
    });

    return tooltip;
  };

  tooltip.destroy = function() {
    selections.forEach((sel) => {
      sel.on('mouseover.tooltip', null)
         .on('mouseout.tooltip', null)
         .on('mousemove.tooltip', null);
    });

    selections = [];
    highlight_cb = undefined;
  };

  tooltip.isFrozen = function() {
    return isFrozen;
  };

  tooltip.updateTooltipText = function (d, i) {
    // eslint-disable-next-line no-undef
    const evt = d4.event || d3.event;
    if (isFrozen) {
      const text = text_fn(d, i, evt.target);
      if (text) {
        tooltip_sel.html(text);
      }

      tooltip.setXY();
    }
  };

  tooltip.hide = ($target) => {
    if (showingTooltip) {
      fadeOut($(dom_id));
      showingTooltip = false;
      isFrozen = false;
      $target.toggleClass('freeze', isFrozen);
    }
  };

  tooltip.freeze = function() {
    if (tooltip_sel.html() === '') {
      return;
    }

    const event = d4.event || d3.event;
    const $eventTarget = $(event.target);

    if (hidePreviouslyFrozenTooltip) {
      hidePreviouslyFrozenTooltip();
    }

    hidePreviouslyFrozenTooltip = () => {
      $(tooltip.dom_id()).css('zIndex', 999);
      tooltip.hide($eventTarget);
      tooltip.activate();
    };

    if (!isFrozen) {
      selections.forEach((selection) => {
        selection
          .on('mouseover.tooltip', null)
          .on('mouseout.tooltip', null);
      });

      tooltip_sel.on('click', () => {
        // eslint-disable-next-line no-undef
        const evt = d4.event || d3.event;
        evt.stopPropagation();
      });

      fadeIn($(dom_id));
      showingTooltip = true;
      current_text_fn = text_fn;
      isFrozen = !isFrozen;
    }

    $eventTarget.toggleClass('freeze', isFrozen);

    $('body').offOn('click.tooltip_unfreeze', function() {
        $(this).off('click.tooltip_unfreeze');
        $(dom_id).css('zIndex', 999);

        if (showingTooltip) {
          $(this).off('click.tooltip_unfreeze');
          fadeOut($(dom_id));
          showingTooltip = false;
          isFrozen = false;
          $eventTarget.toggleClass('freeze', isFrozen);
        }

        tooltip.activate();
    });
  };

  tooltip.addSelection = function(_) {
    if (!arguments.length) return selections;
    selections.push(_);
    return tooltip;
  };

  tooltip.selections = function(_) {
    if (!arguments.length) return selections;
    selections = _;
    return tooltip;
  };

  tooltip.dom_id = function(_) {
    if (!arguments.length) return dom_id;

    dom_id = _;
    tooltip_sel = d3.select(dom_id);

    return tooltip;
  };

  tooltip.highlight_cb = function(_) {
    if (!arguments.length) return highlight_cb;
    highlight_cb = _;
    return tooltip;
  };

  tooltip.text_fn = function(_) {
    if (!arguments.length) return text_fn;
    text_fn = _;
    return tooltip;
  };

  tooltip.getXPos = function(_) {
    if (!arguments.length) return getXPos;
    getXPos = _;
    return tooltip;
  };

  tooltip.getYPos = function(_) {
    if (!arguments.length) return getYPos;
    getYPos = _;
    return tooltip;
  };

  tooltip.mapLayerId = function(_) {
    if (!arguments.length) return mapLayerId;
    mapLayerId = _;
    return tooltip;
  };

  tooltip.text_changes_on_move = function(_) {
    if (!arguments.length) return text_changes_on_move;
    text_changes_on_move = _;
    return tooltip;
  };

  return tooltip;
};

arc.tooltip.id = 1;

arc.add_tooltip = function(tooltip_dom_id, selection, f, text_changes_on_move) {
  const tooltip = d3.select(tooltip_dom_id);

  selection.on('mouseover.tooltip', (d, i) => {
             tooltip.html(f(d, i));
             fadeIn($(tooltip_dom_id));
           })
           .on('mousemove.tooltip', (d, i) => {
             const evt = d4.event || d3.event;
             let xpos = evt.pageX + 10;
             let ypos = evt.pageY - 10;
             if (xpos > $(window).width() - $(tooltip_dom_id).width() - 60) {
               xpos = evt.pageX - $(tooltip_dom_id).width() - 100;
             }

             if (ypos > $(window).height() - $(tooltip_dom_id).height() - 55) {
               ypos = evt.pageY - $(tooltip_dom_id).height() - 55;
             }

             tooltip.style('top', `${ypos}px`).style('left', `${xpos}px`);
             if (text_changes_on_move) {
               tooltip.html(f(d, i));
             }
           })
           .on('mouseout.tooltip', () => {
             fadeOut($(tooltip_dom_id));
           });
};

arc.add_tooltips = function(tooltip_dom_id, selections, f) {
  selections.forEach((s) => {
    arc.add_tooltip(tooltip_dom_id, s, f);
  });
};
}());
