window.arc = window.arc || {};

(function() {
'use strict';

const menuFrame = `
  <div class="contextmenu-body arc-menu-contents">
  </div>
`;

arc.menu_hideAll = function() {
  $('.arc-menu-contents').each(function() {
    $(this).parent().hide();
  });

  $(document).off('click.dismissmenu');
};

arc.menu = function() {
  const menu = function() { return 'arc.menu'; };

  let visualId,
      containerId,
      $container;

  menu.containerId = function(_) {
    if (!arguments.length) return containerId;
    containerId = _;
    $container = $(containerId);
    return menu;
  };

  menu.container = function() {
    return $container;
  };

  menu.build = function($copyFrom, report) {
    $container.empty();
    $container.append(menuFrame);
    const $contents = $container.find('.contextmenu-body');
    $contents.html($copyFrom[0].outerHTML);
    $contents.find('.more-options').show();

    const driver = report.viz(),
          dataset = driver && driver.dataset();

    const viz_type = driver && driver.viz_type();

    if (driver && driver.showEmptyTitleSubtitle()) {
      $contents.find('.showhide-empty-title span').text('Hide');
    } else {
      $contents.find('.showhide-empty-title span').text('Show');
    }

    $contents.find('.showhide-empty-title')
             .offOn('click.showhide', function() {
               menu.hide();

               const driver = report.viz();
               const newState = report.showEmptyTitleSubtitle() ? false : true;

               report.showEmptyTitleSubtitle(newState);
               if (driver) {
                 driver.showEmptyTitleSubtitle(newState);
                 driver.refresh(true);
               }
             });

    if (dataset) {
      const dc_type = dataset.dataconnection_type();
      $container.find('.export-table-dataset')
                .toggle(arc.dataconnection.supportsExportTable(dc_type));
    }

    let viewDataAllowed = true;

    const enableEditDatasetSql =
      arc.frontend.featureFlipper.getValueById('enable_cloudera_style') &&
      report?.dataset()?.dataset_detail() === 'Created from SQL' &&
      arc.misc.hasPerm('dc_explore', report?.dataset()?.dataset_perms());

    let csvAllowed,
        pngAllowed,
        emailAllowed;

    const dashboard = arc.globals.dashboard;
    if (dashboard) {
      csvAllowed = dashboard.csvDownloadPermitted();
      pngAllowed = dashboard.pngDownloadPermitted();
      emailAllowed = dashboard.emailPermitted();
    } else if (driver) {
      const settings = driver.user_settings(),
            csvSetting = settings.find_by_id('permit_csv_download_visual');
            pngSetting = settings.find_by_id('permit_png_download');
            emailSetting = settings.find_by_id('permit_email_report');

      csvAllowed = csvSetting && csvSetting.isChecked;
      pngAllowed = pngSetting && !pngSetting.hidden() && pngSetting.isChecked;
      emailAllowed = emailSetting && !emailSetting.hidden() && emailSetting.isChecked;
    }

    /*
     * For many visual types like link, extension etc., don't permit csv download
     */
    if (arc.report.noResultExcludeTypes.has(viz_type) && viz_type !== 'leaflet') {
      csvAllowed = false;
      viewDataAllowed = false;
    }

    if (viz_type === 'chatbot') {
      pngAllowed = false;
    }

    if (arc.misc.isFilterVisual(viz_type)) {
      $container.find('.more-options > li:not(.filter-action)').hide();
    } else {
      $container.find('.view-data-li')
                .toggleClass('arc-invisible', !viewDataAllowed);

      $container.find('.export-csv-link, .export-xlsx-link')
                .toggleClass('arc-invisible', !csvAllowed);

      $container.find('.png-download, .download-image-link, .pdf-download')
                .toggleClass('arc-invisible', !pngAllowed);

      $container.find('.image-dropdown-submenu')
                .toggleClass('arc-invisible', !csvAllowed && !pngAllowed);

      $container.find('.email-visual')
                .toggleClass('arc-invisible', !emailAllowed);

      $container.find("#edit-dataset-sql-li")
                .toggleClass("arc-invisible", !enableEditDatasetSql)
                .find("a")
        .offOn('click', () => {
          arc.dataset.editor(report).open();
          menu.hide();
                });
    }

    // Setup 'Save Table' button
    if (dataset) {
      var savetable = arc.report.savetable();

      savetable.vizDriver(report.viz());
      savetable.datasetId(dataset.dataset_id());
      savetable.dataconnectionId(dataset.dataconnection_id());
      savetable.dataconnectionType(dataset.dataconnection_type());

      try {
        var firstTablename = dataset.tables()[0].tablename();
        savetable.defaultDb(firstTablename.split('.')[0]);
      } catch (e) {
        console.log(e);
      }

      savetable.setup();

      $container.find('.export-table-dataset')
                .offOn('click.export', () => { menu.hide(); });
    }

    return menu;
  };

  menu.visualId = function(_) {
    if (!arguments.length) return visualId;
    visualId = _;
    return menu;
  };

  menu.isVisible = function() {
    return ($container.css('display') !== 'none');
  };

  menu.hide = function() {
    $container.hide();
    $(document).off('click.dismissmenu');
  };

  menu.show = function(_x, _y) {
    const o = $container.parent().offset();
    let x = _x - 170 - o.left;
    let y = _y + 10 - o.top;

    if (x < 20) {
      x = 20;
    }

    const w = $container.width();
    const $gridster = $('.gridster');
    const gridWidth = $gridster.width();
    const gridLeft = $gridster.offset().left;

    $container.css('top', y + 'px')
              .css('left', x + 'px')
              .show();

    /*
     * When a click is made outside the menu, we want to close the menu.
     * However, the menu is being opened up as a result of a click, and
     * we do not want to close the menu immediately as the event bubbles up.
     * So, add the event within a setTimeout call.
     */
    setTimeout(() => {
      $(document).offOn('click.dismissmenu', function(e) {
        if ($(e.target).closest(containerId).length > 0) {
          return;
        }

        menu.hide();
      });
    }, 0);

    return menu;
  };

  return menu;
};
}());
