window.arc = window.arc || {};

(function() {
"use strict";

arc.migration = arc.migration || {};

var data_sort_attr_map = {
  1: 'appgroups',
  2: 'dashboards',
  3: 'visuals',
  4: 'datasets',
  5: 'customcss',
  6: 'colorpalette',
  7: 'staticasset'
};

function parseError(error) {
  var title, error_message;
  if (typeof error === 'string' && error.split(":").length === 2) {
    title = error.split(":")[0];
    error_message = error.split(":")[1];
  } else {
    title = "Performing the operation failed: ";
    error_message = error;
  }
  return {
    "title": title,
    "errormessage": error_message
  };
}

function getTDs() {
  const tbody = $('#import_detail_table')
      .trigger('select');
  var tds = {};
  for (var key in data_sort_attr_map) {
    tds[key] = tbody.find('td[data-sort="' + key + '"]');
  }
  return tds;
}

function removePicklist(data) {
  // removing picklist and daterange visuals from visuals' list
  // to make sure they are not shown as visuals to be exported
  const all_visuals = data[data_sort_attr_map[3]];
  const new_viz = [];
  // eslint-disable-next-line no-restricted-syntax
  for (const i in all_visuals) {
    if (all_visuals[i].type !== 'picklist' &&
      all_visuals[i].type !== 'daterange') {
      new_viz.push(all_visuals[i]);
    }
  }

  data[data_sort_attr_map[3]] = new_viz;
  return data;
}

arc.migration.ajaxCall = function(url, type, cb) {
  $.ajax({
    url: url,
    type: type,
    beforeSend: function(xhr) {
      // Because of a jquery bug when handling PUT requests,
      // we have to explicitly set X-CSRFToken here
      xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
    },
  })
    .done(function(res) {
      if (cb) {
        cb(res);
      }
    })
    .fail(function(req, textstatus, errorthrown) {
      if (req.responseText !== undefined) {
        errorthrown = req.responseText;
      }
      var error = parseError(errorthrown);
      arc.report.alertMessage(error.title, error.errormessage);
    });
};

arc.migration.export = function() {
  var exportAppGroup = false;

  function paintObjects(data, tds, key) {
    d3.select('#import_detail_table')
      .select('tbody')
      .selectAll(`tr[export-tag="${data_sort_attr_map[key]}"][template="false"]`)
        .data(data[data_sort_attr_map[key]])
        .enter()
        .append('tr')
        .attr('export-tag', data_sort_attr_map[key])
        .attr('template', false)
        .each(function(d) {
          d3.select(this)
            .append(() => tds[key][0].cloneNode(true));
          d3.select(this).append('td')
            .text(d.name);
          d3.select(this).append('td')
            .text(d.id);
        });
  }

  function paintExportReport(data) {
    let key;
    const tds = getTDs();
    data = removePicklist(data);

    d3.select('#import_detail_table')
      .select("tbody")
      .selectAll('tr[template="false"]')
      .remove();

    for (key in data_sort_attr_map) {
      paintObjects(data, tds, key);
    }
  }

  arc.migration.export.setup = function setupExport() {
    const main_url = arc.frontend.globals.urls.exportUrl;
    let appIds;

    // enter export filename as user types into input
    $('#export-visual-filename').on('keyup', function() {
      const fn = $(this).val();
      $('#display-export-filename').text(fn);
    });

    arc.migration.export.setup.exportArtifacts = function exportArtifacts(evt, ids) {
      const $modal = $('#export-visuals-modal');
      if (arc.misc.modalShown($modal)) {
        return;
      }

      appIds = ids;
      const url = `${arc.frontend.globals.urls.exportUrl}?appgroups=[${ids.apps.join(',')}]&dashboards=[${ids.dashboard.join(',')}]&visuals=[${ids.visual.join(',')}]&filename=temp&dry_run=true`;

      // prevent showing the export modal
      evt.stopPropagation();
      $(evt.target).closest('.dropdown-menu').parent().find('.more-btn')
        .dropdown('toggle');
      arc.migration.ajaxCall(url, 'GET', (data) => {
        $('#export-visual-filename').val(`visuals_${new Date().getTime()}`);
        paintExportReport(data);
      });

      $modal.modal('show');
    };

    // setting up visual and dashboard export
    $('#viz-dash-export')
      .off('click')
      .on('click', (evt) => {
        exportAppGroup = false;
        arc.migration.export.setup.exportArtifacts(evt, arc.screen.apps.selectedAppIdsAndTypes());
      });

    // setting up confirm button for export
    $('#viz-dash-export-confirm')
      .off('click')
      .on('click', () => {
        let url;
        const filename = $('#export-visual-filename').val();
        if (exportAppGroup) {
          url = `${main_url}?dry_run=false&appgroups=[${arc.screen.apps.groupActiveId()}]&filename=${filename}`;
        } else {
          url = `${main_url}?dry_run=false&dashboards=[${appIds.dashboard.join(',')}]&` +
                `appgroups=[${appIds.apps.join(',')}]&` +
                `visuals=[${appIds.visual.join(',')}]&filename=${filename}`;
        }

        document.location.href = url;
      });
  };
};

arc.migration.import = function() {

  var link_map = {
    1: arc.frontend.globals.urls.editAppUrl,
    2: arc.frontend.globals.urls.viewDashboardUrl,
    3: arc.frontend.globals.urls.viewDashboardUrl,
    4: arc.frontend.globals.urls.datasetEditUrl,
    5: '/arc/apps/customstyles',
    6: '/arc/apps/customcolors',
    7: arc.frontend.globals.urls.staticassetsUrl
  };

  function paintObjects(data, tds, icon_tds, key) {
    if (data_sort_attr_map[key] in data) {
      d3.select('#import_detail_table')
        .select('tbody')
        .selectAll(`tr[import-tag="${data_sort_attr_map[key]}"][template="false"]`)
        .data(data[data_sort_attr_map[key]])
        .enter()
        .append('tr')
        .attr('export-tag', data_sort_attr_map[key])
        .attr('template', false)
        .each(function (d) {
          d3.select(this)
            .append(() => tds[key][0].cloneNode(true));
          d3.select(this).append('td')
            .text(d.name);
          d3.select(this)
            .append(() => {
              const $td = $('<td></td>');
              if (d.action.create) {
                $td.append(icon_tds.create.cloneNode(true));
              } else {
                let updated = false;
                // eslint-disable-next-line no-restricted-syntax
                for (const actionKey in d.action) {
                  if (d.action[actionKey]) {
                    updated = true;
                    $td.append(icon_tds[actionKey].cloneNode(true));
                  }
                }

                if (!updated) {
                  $td.append(icon_tds.no_update.cloneNode(true));
                }
              }

              return $td[0];
            });

          if (!isNaN(d.id) && d.id === parseInt(d.id, 10)) {
            d3.select(this).append('td')
              .append('a')
                .attr('href', arc.misc.getBlankTargetUrl(`${link_map[key]}/${d.id}`))
                .attr('target', '_blank')
                .text(d.id);
          } else {
              d3.select(this)
                .append('td')
                .attr('class', 'text-center')
                .text('--');
          }
      });
    }
  }

  arc.migration.import.paintImportReport = function(data) {
    var key;
    var tds = getTDs();
    var icons = [
      "create",
      "data_update",
      "no_update",
      "workspace_update",
      "thumbnail_update"
    ];

    if (data_sort_attr_map[3] in data) {
      data = removePicklist(data);
    }

    const tbody = $('#import_detail_table').trigger('select');
    var span_icons = {};
    for (key in icons) {
      span_icons[icons[key]] = tbody.find(`span[action-label='${icons[key]}']`)[0];
    }

    d3.select('#import_detail_table')
      .select("tbody")
      .selectAll('tr[template="false"]')
      .remove();

    for (key in data_sort_attr_map) {
        paintObjects(data, tds, span_icons, key);
      }

      var error = $('#error-on-post').data('error-on-post');
      if (error !== "None") {
        error = parseError(error);
        arc.report.alertMessage(error.title, error.errormessage);
      }
  };

  arc.migration.import.modalSetup = function() {
    $('#import-visual-form').on("submit", function () {
      $('#import-vis-dash-form-spin').show();
    });

    // on the import get the list of workspace that user has edit access and
    // paint the modal with the list
    var $modal = $('#import-visual-modal');
    $modal.on('show.bs.modal', () => {
      arc.migration.ajaxCall(
        arc.frontend.globals.urls.workspacesUrl,
        'GET',
        (data) => {
          data = JSON.parse(data);
          const editable_ws = [];
          // eslint-disable-next-line no-restricted-syntax
          for (const i in data) {
            if (data[i].perms.indexOf('ws_appedit') !== -1 ||
                data[i].perms.indexOf('ws_manage') !== -1) {
              const isReservedWorkspace = data[i].name === 'Public' || data[i].name === 'Private';
              if (isReservedWorkspace) {
                editable_ws.unshift(data[i]);
              } else {
                editable_ws.push(data[i]);
              }
            }
          }

          // make sure Private workspace is at top
          if (editable_ws[0].name !== 'Private') {
            editable_ws[0] = editable_ws.splice(1, 1, editable_ws[0])[0];
          }

          d3.select('#workspace-import-visual-artifacts')
            .selectAll('option[temp="true"]')
            .remove();

          d3.select('#workspace-import-visual-artifacts')
            .select('select')
            .selectAll()
            .data(editable_ws)
            .enter()
            .append(function (d) {
                var $n = $('<option></option>');
                if (d.name.toLowerCase() === 'private') {
                  $n.attr('selected', 'selected');
                }
                $n.text(d.name);
                if (d.id === null) {d.id = '';}
                $n.attr('value', d.id)
                  .attr('temp', true)
                  .attr('style', null);
                return $n[0];
              });
        });
    });
  };

  arc.migration.import.setup = function() {

    // setting up don't import button
    $('#viz-dash-import-cancel')
      .off('click')
      .on('click', function() {
        window.location = arc.frontend.globals.urls.dataUrl;
      });

    // setting up confirm button for import
    $('#viz-dash-import-confirm')
      .off('click')
      .on('click', function() {
          $("#viz-dash-import-confirm-spin").show();
        arc.migration.ajaxCall(window.location.href + "?dry_run=false",
          'PUT',
          function (data) {
            arc.migration.import.paintImportReport(data);
            var error = $('#error-on-post').data('error-on-post');
            if (error === "None") {
              d3.select('div#sucess-message-viz-dash-import')
                .attr('style', null);
              $("#viz-dash-import-cancel")
                .html(`<i class="fa fa-arrow-left"></i> Back To Data`);
              $('#viz-dash-import-confirm')
                .css('display', 'none');
            }
          $("#viz-dash-import-confirm-spin").hide();
        });
      });
  };
};

}());
