window.arc = window.arc || {};

(function() {
'use strict';

arc.model = arc.model || {};

/*
 * CustomStyles is used to fetch the list of cssid -> cssname pairs from the
 *   backend. If the information is fetched already, calls to 'fetch' do not
 *   re-fetch the information, but simply make any callback that is passed in.
 *
 * The expected usage pattern is:

     c = arc.model.CustomStyles();
     c.fetch(function() {
       // do something with c.styles()
       //    or with c.cssname(cssid)
     });

 *
 */

arc.model.CustomStyles = function() {
  var CustomStyles = function() { return 'arc.admin.CustomStyles'; };

  var styles = {};
  var fetched = false;

  const assets = [
    {
      asset: arc.frontend.globals.urls.customextensionUrl,
      postprocess: d3.json,
    },
  ];

  CustomStyles.invalidate = function() {
    fetched = false;
    arc.staticfetcher.forget(assets);
  };

  /**
   * The fetch function runs an API to fetch the custom styles
   * then stores in an object.
   * @param {Function} cb: takes a callback function which is passed an object
   *                       containing all custom styles fetched
   */
  CustomStyles.fetch = function(cb) {
    if (!fetched && arc.testutil.test_css) {
      fetched = true;
    }

    if (fetched) {
      if (cb) {
        cb(styles);
      }

      return;
    }

    arc.staticfetcher.fetch(assets,
      function(error, customStyles) {
        if (error) {
          arc.report.alertMessage('Could not load custom styles',
              error);
          return;
        }

        fetched = true;
        styles = {};
        customStyles.forEach(c => styles[c.id] = c);

        if (cb) {
          cb(styles);
        }
      });
  };

  CustomStyles.styles = function() {
    return styles;
  };

  CustomStyles.cssname = function(cssid) {
    return styles[cssid] && styles[cssid].name;
  };

  return CustomStyles;
};

}());
