window.arc = window.arc || {};

(function() {
'use strict';

arc.utils = arc.utils || {};

function linkWithIcon(icon, classname) {
  return '<a class="' + classname + '"><i class="fa ' + icon + '"></i></a>';
}

arc.utils.modifiableRowsTable = function(config) {
  let t = function() { return 'arc.utils.modifiableRowsTable'; };

  let {
    $table,
    initialData,
    extraColumns,
    needsSaveCb,
    changeCb,
    editableCellType = 'div.editablecell',
  } = config;

  const colCount = $table.find('thead th').length;
  const $tbody = $table.find('tbody');

  extraColumns = extraColumns || [];
  initialData = initialData || [];

  function rowToData($tr) {
    let row = [];

    const $td = $tr.find('td'),
          cnt = $td.length;

    const dataCols = cnt - 1 - extraColumns.length;

    $td.each(function(i) {
      if (i >= dataCols) {
        return;
      }

      row.push($(this).text().trim());
    });

    if (row[0] === '') {
      return undefined;
    }

    return row;
  }

  t.getRows = function() {
    let rows = [];

    const $tr = $tbody.find('tr'),
          cnt = $tr.length;

    $tr.each(function(i) {
      if (i === (cnt - 1)) {
        return;
      }

      let row = rowToData($(this));
      if (row !== undefined) {
        rows.push(row);
      }
    });

    return rows;
  };

  function setupContentEditables() {
    $table.find('td.editablecell')
          .off('keydown')
          .on('keydown', function(e) {
            if (e.keyCode === 13) {
              return false;
            }
          });
  }

  // Expected delete elements in all added rows
  const deleteSel = 'tr td:last-child a.delete-row';

  t.makeEditable = function(editable) {
    const $addRowLocation = $tbody.find('tr:last');

    if (editable) {
      setupDeletes();
      $table.find(deleteSel)
            .show();
      $addRowLocation.show();
      $tbody.find('.editablecell')
            .prop('contenteditable', true);
    } else {
      unsetupDeletes();
      $table.find(deleteSel)
            .hide();
      $addRowLocation.hide();
      $tbody.find('.editablecell')
            .prop('contenteditable', false);
    }
  };

  function unsetupDeletes() {
    $table.find(deleteSel)
        .off('click');
  }

  function setupDeletes() {
    $table.find(deleteSel)
        .off('click')
        .on('click', function() {
          $(this).closest('tr').remove();
          if (changeCb) {
            changeCb();
          }

          if (needsSaveCb) {
            needsSaveCb();
          }
        });
  }

  function dataToRow(data) {
    let newRow = [];
    newRow.push('<tr>');

    if (data === undefined) {
      for (let i = 0; i < colCount - 1 - extraColumns.length; ++i) {
        newRow.push('<td><div class="editablecell" contenteditable="true"></div></td>');
      }
    } else {
      data.forEach(function(d) {
        newRow.push('<td><div class="editablecell" contenteditable="true">' +
            arc.misc.escapeHTML(d) + '</div></td>');
      });
    }

    extraColumns.forEach(function(c) {
      newRow.push('<td>');

      // NOTE: this is not escaped with arc.misc.escapeHTML() so that we can add in
      // html. It is only available via code.
      newRow.push(c);
      newRow.push('</td>');
    });

    newRow.push('<td>');
    newRow.push(linkWithIcon('fa-trash-o', 'delete-row'));
    newRow.push('</td>');

    newRow.push('</tr>');
    return $(newRow.join(''));
  }

  t.setup = function() {
    // Initialize the table
    $tbody.empty();
    const lastRow = [];
    lastRow.push('<tr>');
    lastRow.push(`
      <td class="add-row" colspan=${colCount}>
        <a class="link">
          <i class="sym-plus"></i> Add New Item
        </a>
      </td>
    `);
    lastRow.push('</tr>');
    $tbody.append(lastRow.join(''));

    // Find the last row, so we can add data rows before it
    const $addRowLocation = $tbody.find('tr:last');

    initialData.forEach(function(d) {
      const $row = dataToRow(d);
      $row.insertBefore($addRowLocation);
    });

    setupContentEditables();

    $tbody.offOn('blur', editableCellType, function() {
          const currData = t.getRows();

          if (!arc.misc.isIdentical(currData, initialData)) {
            if (needsSaveCb) {
              needsSaveCb();
            }
          }
        });

    setupDeletes();
    if (changeCb) {
      changeCb();
    }

    $addRowLocation.find('a')
                    .offOn('click.addNewRow', function() {
                     const $row = dataToRow();
                     $row.insertBefore($addRowLocation);
                     $row.find('td:first div').focus();
                     setupContentEditables();
                     if (changeCb) {
                       changeCb();
                     }

                     setupDeletes();
                     if (needsSaveCb) {
                       needsSaveCb();
                     }
                   });
  };

  return t;
};

}());

