window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};
arc.report.shelf = arc.report.shelf || {};

function filterOutAggregates(el) {
  return (!el.expressionIsAggregate());
}

arc.report.analyticalBuilder = function (userConfig) {
  // Default analytical function text
  var required = "<span class='required'>* </span>",
      optional = "<span class='help-text'> (optional)</span>",
      analyticalFunctionHeaders = {
        aggregate: 'Select aggregate for this measure',
        partitionPercent: 'Select entities over which to calculate percentage',
        partitionAverage: 'Select entities over which to create moving averages',
        partitionRowNumber: 'Select entities over which to calculate row number',
        partitionRank: 'Select entities over which to calculate rank',
        partitionDenseRank: 'Select entities over which to calculate dense rank',
        partitionPrevious: 'Select entities over which to take previous value',
        partitionPreviousDifference:
            'Select entities over which to take difference from previous value',
        partitionPreviousDifferencePct:
            'Select entities over which to take % difference from previous value',
        partitionNext: 'Select entities over which to take next value',
        partitionNextDifference:
            'Select entities over which to take difference from next value',
        partitionNextDifferencePct:
            'Select entities over which to take % difference from next value',
        partitionTotal:   'Select entities over which to take running total',
        partitionHelp:    '(Selecting nothing returns percent of total)',
        sortOrder:        'Select the sort order',
        sortOrderHelp:    '(If you do not select a sort order, ' +
                          'the aggregate of the current column will be applied)',
        number:           'Rows to include in the average (relative to current row)',
      },
      pctFormat = {
        val: {
          type: 'percentage',
          result: '.2%',
          comma: false,
          decimal: '2',
          decimalall: false,
          autoAdded: true,
        },
        message: 'Display format has been updated so that column will display as a percentage.',
      },

      successCbMade,

      abConfig = {
        // Defined as part of initialize
        analyticalAlias: undefined,
        analyticalAliasUseColumn: true,
        show_el: [],
        custom_labels: {},
        format: undefined,

        // User-defined values
        shelves: undefined,
        d3_elem: undefined,
        connection: undefined,
        currentElValue: undefined,
        currentElName: undefined,
      },

      // Main object
      analyticalBuilder = {
        // OPEN MODAL FOR ANALYTICAL FUNCTIONS
        createMovingAverageModal: function(connectionType, successCb) {
          abConfig.show_el = ['analytical-partition', 'analytical-sort', 'analytical-number'];
          abConfig.analyticalAlias = 'moving average';
          abConfig.custom_labels = {
            'analytical-function': ': Moving Average',
            'analytical-aggregate': analyticalFunctionHeaders.aggregate,
            'analytical-partition': analyticalFunctionHeaders.partitionAverage + optional,
            'analytical-sort': required + analyticalFunctionHeaders.sortOrder,
            'analytical-sort-help': analyticalFunctionHeaders.sortOrderHelp,
            'analytical-number': required + analyticalFunctionHeaders.number,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results);

            /*
              https://www.periscope.io/
              blog/use-subqueries-and-window-functions-to-compute-running-averages.html
            */

            return 'avg(' + components.colFunction + ') over (' +
                    components['analytical-partition'] + ' ' +
                    components['analytical-sort'] + ' ' +
                    components['analytical-number'] +
                    ')';
          });
        },

        createRankModal: function(connectionType, successCb) {
          abConfig.rankFunction = 'RANK';
          abConfig.analyticalAlias = 'rank';
          abConfig.partitionStr = analyticalFunctionHeaders.partitionRank;
          abConfig.title = 'Rank';
          return analyticalBuilder.createRowNumberModal(connectionType, successCb);
        },

        createDenseRankModal: function(connectionType, successCb) {
          abConfig.rankFunction = 'DENSE_RANK';
          abConfig.analyticalAlias = 'dense rank';
          abConfig.partitionStr = analyticalFunctionHeaders.partitionDenseRank;
          abConfig.title = 'Dense Rank';
          return analyticalBuilder.createRowNumberModal(connectionType, successCb);
        },

        createRowNumberModal: function(connectionType, successCb) {
          abConfig.show_el = ['analytical-partition', 'analytical-sort'];
          abConfig.analyticalAliasUseColumn = false;
          abConfig.hideAnalyticalAggregate = true;
          abConfig.rankFunction = abConfig.rankFunction || 'ROW_NUMBER';
          abConfig.partitionStr = abConfig.partitionStr ||
              analyticalFunctionHeaders.partitionRowNumber;
          abConfig.title = abConfig.title || 'Row Number';
          abConfig.analyticalAlias = abConfig.analyticalAlias || 'row number';
          abConfig.custom_labels = {
            'analytical-function': `: ${abConfig.title}`,
            'analytical-partition': abConfig.partitionStr + optional,
            'analytical-sort': required + analyticalFunctionHeaders.sortOrder,
            'analytical-sort-help': analyticalFunctionHeaders.sortOrderHelp,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results);

            return `${abConfig.rankFunction}() over (` +
                    components['analytical-partition'] + ' ' +
                    components['analytical-sort'] + ')';
          });
        },

        createNextValueModal: function(connectionType, successCb) {
          abConfig.leadlagFunction = 'LEAD';
          abConfig.analyticalAlias = 'next value';
          abConfig.partitionStr = analyticalFunctionHeaders.partitionNext;
          abConfig.title = 'Next Value';
          return analyticalBuilder.createPreviousValueModal(connectionType, successCb);
        },

        createPreviousValueModal: function(connectionType, successCb) {
          abConfig.leadlagFunction = abConfig.leadlagFunction || 'LAG';
          abConfig.analyticalAlias = abConfig.analyticalAlias || 'previous value';
          abConfig.partitionStr = abConfig.partitionStr ||
              analyticalFunctionHeaders.partitionPrevious;
          abConfig.show_el = ['analytical-partition', 'analytical-sort', 'analytical-offset'];
          abConfig.title = abConfig.title || 'Previous Value';

          abConfig.custom_labels = {
            'analytical-function': `: ${abConfig.title}`,
            'analytical-aggregate': analyticalFunctionHeaders.aggregate,
            'analytical-partition': abConfig.partitionStr + optional,
            'analytical-sort': required + analyticalFunctionHeaders.sortOrder,
            'analytical-sort-help': analyticalFunctionHeaders.sortOrderHelp,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results);

            const offset = components['analytical-offset'] || 1;
            return `${abConfig.leadlagFunction}(${components.colFunction}, ${offset}) over (` +
                    components['analytical-partition'] + ' ' +
                    components['analytical-sort'] +
                    ')';
          });
        },

        createNextDifferenceModal: function(connectionType, successCb) {
          abConfig.leadlagFunction = 'LEAD';
          abConfig.analyticalAlias = 'diff from next';
          abConfig.partitionStr = analyticalFunctionHeaders.partitionNextDifference;
          abConfig.title = 'Difference from Next Value';
          return analyticalBuilder.createPreviousDifferenceModal(connectionType, successCb);
        },

        createPreviousDifferenceModal: function(connectionType, successCb) {
          abConfig.leadlagFunction = abConfig.leadlagFunction || 'LAG';
          abConfig.analyticalAlias = abConfig.analyticalAlias || 'diff from prev';
          abConfig.partitionStr = abConfig.partitionStr ||
              analyticalFunctionHeaders.partitionPreviousDifference;
          abConfig.title = abConfig.title || 'Difference from Previous Value';

          abConfig.show_el = ['analytical-partition', 'analytical-sort', 'analytical-offset'];
          abConfig.custom_labels = {
            'analytical-function': `: ${abConfig.title}`,
            'analytical-aggregate': analyticalFunctionHeaders.aggregate,
            'analytical-partition': abConfig.partitionStr + optional,
            'analytical-sort': required + analyticalFunctionHeaders.sortOrder,
            'analytical-sort-help': analyticalFunctionHeaders.sortOrderHelp,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results);

            const offset = components['analytical-offset'] || 1;
            return `${components.colFunction} - ` +
                `${abConfig.leadlagFunction}(${components.colFunction}, ${offset}) ` +
                `over (${components['analytical-partition']} ${components['analytical-sort']})`;
          });
        },

        createNextDifferencePctModal: function(connectionType, successCb) {
          abConfig.leadlagFunction = 'LEAD';
          abConfig.analyticalAlias = 'pct diff from next';
          abConfig.partitionStr = analyticalFunctionHeaders.partitionNextDifferencePct;
          abConfig.title = 'Percentage Difference from Next Value';
          return analyticalBuilder.createPreviousDifferencePctModal(connectionType, successCb);
        },

        createPreviousDifferencePctModal: function(connectionType, successCb) {
          abConfig.leadlagFunction = abConfig.leadlagFunction || 'LAG';
          abConfig.analyticalAlias = abConfig.analyticalAlias || 'pct diff from prev';
          abConfig.format = pctFormat;
          abConfig.partitionStr = abConfig.partitionStr ||
              analyticalFunctionHeaders.partitionPreviousDifferencePct;
          abConfig.title = abConfig.title || 'Percentage Difference from Previous Value';
          abConfig.show_el = ['analytical-partition', 'analytical-sort', 'analytical-offset'];
          abConfig.custom_labels = {
            'analytical-function': `: ${abConfig.title}`,
            'analytical-aggregate': analyticalFunctionHeaders.aggregate,
            'analytical-partition': abConfig.partitionStr + optional,
            'analytical-sort': required + analyticalFunctionHeaders.sortOrder,
            'analytical-sort-help': analyticalFunctionHeaders.sortOrderHelp,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results);

            const offset = components['analytical-offset'] || 1;
            const lag = `${abConfig.leadlagFunction}(${components.colFunction}, ${offset}) ` +
                `over (${components['analytical-partition']} ${components['analytical-sort']})`;
            return `(${components.colFunction} - ${lag}) / ${lag}`;
          });
        },

        createPercentTotalModal: function(connectionType, successCb) {
          abConfig.show_el = ['analytical-partition'];
          abConfig.analyticalAlias = 'percent';
          abConfig.format = pctFormat;
          abConfig.custom_labels = {
            'analytical-function': ': Percent of Group',
            'analytical-aggregate': analyticalFunctionHeaders.aggregate,
            'analytical-partition': analyticalFunctionHeaders.partitionPercent + optional,
            'analytical-partition-help': analyticalFunctionHeaders.partitionHelp,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results);
            return `${components.colFunction} / sum(${components.colFunction}) over (` +
                   `${components['analytical-partition']})`;
          });
        },

        createRunningTotalModal: function(connectionType, successCb) {
          abConfig.show_el = ['analytical-partition', 'analytical-sort'];
          abConfig.analyticalAlias = 'running total';
          abConfig.custom_labels = {
            'analytical-function': ': Running Total',
            'analytical-aggregate': analyticalFunctionHeaders.aggregate,
            'analytical-partition': analyticalFunctionHeaders.partitionTotal + optional,
            'analytical-sort': required + analyticalFunctionHeaders.sortOrder,
            'analytical-sort-help': analyticalFunctionHeaders.sortOrderHelp,
          };

          _modalBuilder.initialize(connectionType, successCb, function(results) {
            var components = getComponents(results),
                row_text = 'rows between unbounded preceding and current row';
            return 'sum(' + components.colFunction + ') over (' +
                    components['analytical-partition'] + ' ' +
                    components['analytical-sort'] + ' ' +
                    row_text +
                    ')';
          });
        },

        // Config getter/setter
        get: function () {
          if (!arguments.length) return abConfig;
          var result = {};
          for (var property in arguments) {
            result[property] = abConfig[property];
          }

          return result;
        },

        set: function (config) {
          for (var property in config) {
            abConfig[property] = config[property];
            if (property === 'currentElName') {
              abConfig.currentElValue = config.currentElName === 'Record Count' ?
                                        'sum(1)' : config.currentElName;
            }
          }

          return analyticalBuilder;
        },
      },

      // Used to create sections of various modals
      _modalBuilder = {
        // Populated partition section of analytical function modal
        // config: uses shelfItems, $modal
        create_analytical_partition: function(config) {
          var $partitionSection = config.$modal.find('#analytical-partition-options'),
              $partitionBase = config.$modal.find('#partition-base'),
              shelfItems = config.shelfItems.filter(filterOutAggregates);
          $partitionSection.empty();

          // If there are no shelf items, show warning
          if (shelfItems.length === 0) {
            var helpText = 'This option is only available with non-aggregates. ' +
                           'Please add a dimension or other non-aggregate to one of the shelves';
            config.$modal.find('#analytical-partition-help').text(helpText);
          } else {
            var $partitionItem;
            shelfItems.forEach(function(d, i) {
              $partitionItem = $($partitionBase.html()).appendTo($partitionSection);
              $partitionItem.find('input')
                            .attr('value', d.getExpressionForEdit(true));
              $partitionItem.find('.checkbox-label')
                            .text(d.getComputedDisplayName());
            });
          }
        },

        //Populated sort section of analytical function modal
        // config: uses shelves, shelfItems, and $modal
        create_analytical_sort: function(config) {
          var sortOrder = abConfig.shelves.getOrder(),
              $sortSection = config.$modal.find('.analytical-sort-options'),
              $sortBase = config.$modal.find('#sort-base'),
              $addSortButton = config.$modal.find('#add-sort-order'),
              dimShelfItems = config.shelfItems.filter(filterOutAggregates);

          $sortSection.empty();
          addSortOptions($sortBase, config.shelfItems);

          // Setup sort order if the same analytical function hadn't been built previously
          // The default sort order is sorting based on shelf settings + ascending sort
          //     on all other dimensions
          if (!config.formData) {
            let currentElem = abConfig.d3_elem.datum();

            let dupTracker = {};

            // First, sort by columns that are sorted in the shelves
            sortOrder.forEach(function(element) {
              let elementName = (currentElem === element) ?
                  '_CURRENT_COLUMN_' : element.getExpressionForEdit(true);

              if (dupTracker[elementName]) {
                return;
              }

              dupTracker[elementName] = true;

              let elementToAdd = {
                elementName,
                elementOrderAscending: element.order().ascending(),
              };
              createNewSortInput($sortSection, $sortBase, $addSortButton, elementToAdd);
            });

            // Then, sort by all other dimensions
            dimShelfItems.forEach(function(element) {
              let elementName = (currentElem === element) ?
                  '_CURRENT_COLUMN_' : element.getExpressionForEdit(true);

              if (dupTracker[elementName]) {
                return;
              }

              dupTracker[elementName] = true;

              let elementToAdd = {
                elementName,
                elementOrderAscending: true,
              };
              createNewSortInput($sortSection, $sortBase, $addSortButton, elementToAdd);
            });
          }

          // Turn on sort button again if turned off
          $addSortButton.show();

          // Turn off if there are already maximum number of sort initial_elements
          hideSortButton(config.shelfItems, $sortSection);

          // Add event listener to + sign so that new options are added
          $addSortButton.off('click.addEvent').on('click.addEvent', function (e) {
            e.preventDefault();
            createNewSortInput($sortSection, $sortBase, $addSortButton);
            hideSortButton(config.shelfItems, $sortSection);
          });

          // HELPER FUNCTIONS
          // Adds options to sort by dropdown
          function addSortOptions($sortBase, shelfItems) {
            var $controls = $sortBase.find('.form-control');
            $controls.empty();
            $controls.append('<option>None');
            var $option = $('<option>').appendTo($controls);

            // Add current column as an option
            $option.text(abConfig.currentElName)
                   .attr('value', '_CURRENT_COLUMN_');

            shelfItems.forEach(function (d) {
              $option = $('<option>').appendTo($controls);
              $option.text(d.getComputedDisplayName())
                     .attr('value', d.getExpressionForEdit(true));
            });
          }

          // Hides sort button once all options chosen
          function hideSortButton(shelfItems, $sortSection) {
            // Hide 'add' button if the sort items already exceeds number of items
            // on the shelf
            if ($sortSection.find('.form-control').length > shelfItems.length) {
              $addSortButton.hide();
            }
          }
        },

        create_analytical_offset: function(config) {
          // If the modal was previously built, reuse the data
          let precedingNum = 1;

          if (config.formData) {
            precedingNum = config.formData['analytical-offset-preceding-number'];
          }

          config.$modal.find('#previous-offset-row-number')
                       .val(precedingNum);
        },

        //Just in case analytical number section ever needs to be changed
        // config: uses number and $modal
        create_analytical_number: function(config) {
          // If the modal was previously built, reuse the data
          var precedingOption = '',
              precedingNum = 5,
              followingOption = 0,
              followingNum = 5;

          if (config.formData) {
            precedingOption = config.formData['analytical-number-preceding'];
            precedingNum = config.formData['analytical-number-preceding-number'];
            followingOption = config.formData['analytical-number-following'];
            followingNum = config.formData['analytical-number-following-number'];
          }

          config.$modal.find('.analytical-number .select-row').off('change.processNumber')
                       .on('change.processNumber', processNumber);
          config.$modal.find('#previous-row-select')
                       .val(precedingOption).trigger('change');
          config.$modal.find('#previous-row-number')
                       .val(precedingNum);
          config.$modal.find('#forward-row-select')
                       .val(followingOption).trigger('change');
          config.$modal.find('#forward-row-number')
                       .val(followingNum);

          function processNumber(e) {
            var $option = $(e.target),
                $nearestNum = $option.next('.number-input');
            if ($option.val()) {
              $nearestNum.fadeOut();
            } else {
              $nearestNum.fadeIn();
            }
          }
        },

        // Allows user to choose aggregate for analytical function if there isn't already one
        //Config: uses currentEl
        create_analytical_aggregate: function(config) {
          var currentAggregate = config.currentEl.fn_name(),
          aggregateSelect = config.$modal.find('.analytical-aggregate select');
          if (currentAggregate) {
            if (currentAggregate === 'countdistinct') {
              currentAggregate = 'count(distinct';
            }

            if (aggregateSelect.find("[value='" + currentAggregate + "']").length) {
              aggregateSelect.val(currentAggregate);
            }
          } else {
            aggregateSelect.val('sum');
          }
        },

        // MAIN: Creates modal with appropriate sections
        initialize: function(connectionType, successCb, parseSql) {
          var currentEl = abConfig.d3_elem.datum(),
              $modal = $('#analytical-modal'),
              allShelfItems = d3.selectAll('#app-shelves .selected-list-wrapper .selection_field')
                                .data(),
              currentElDeleted = false,
              currentElName = currentEl.getComputedDisplayName(),
              currentElExpression = currentEl.getExpressionForEdit(),

              // Filters out repeats, the current element, filter elements
              // And window functions (you can't nest window functions)
              shelfItems = allShelfItems.filter(filterOutUnwanted),
              results = {};

          if (arc.misc.modalShown($modal)) {
            return;
          }

          successCbMade = false;

          // First hide all elements and unselect/remove all values
          $modal.find('.analytical-section').hide();
          $modal.find(':input').not(':button, :submit')
               .val('')
               .removeAttr('checked')
               .removeAttr('selected');

          // Hide connection-specific information
          const $dbDependentEls = $modal.find('.conn-dependent');
          $dbDependentEls.each(function checkConn() {
            const $this = $(this);
            $this.toggleClass('arc-invisible', !$this.data('conn').includes(connectionType));
          });

          const $dbExcludedEls = $modal.find('.conn-exclude');
          $dbExcludedEls.each(function checkExcludedConn() {
            const $this = $(this);
            $this.toggleClass('arc-invisible', $this.data('conn-exclude').includes(connectionType));
          });

          // Give modal title if specified
          setLabel('analytical-function', $modal);

          // Get previous analytical function form data if available
          // This value will be null if another window function was previously chosen
          var formData = currentEl.analyticalFunctionData();

          // Add an aggregate option if element doesn't already have an aggregate
          if (!currentEl.expr_hasagg() && !abConfig.hideAnalyticalAggregate) {
            abConfig.show_el.push('analytical-aggregate');
          }

          // Show sections that are needed
          abConfig.show_el.forEach(function (el) {
            setLabel(el, $modal);
            var elSnakeCase = el.split('-').join('_'),
                functionName = 'create_' + elSnakeCase;

            // Run create_analytical_number, create_analytical_sort,
            // create_analytical_partition, create_analytical_aggregate
            _modalBuilder[functionName]({
              shelfItems,
              $modal: $modal,
              formData: formData,
              currentEl: currentEl,
            });
          });

          // fill in previous window function value_is_num
          if (formData) {
            var $sortSection = $modal.find('.analytical-sort-options'),
                $sortBase = $modal.find('#sort-base'),
                $addSortButton = $modal.find('#add-sort-order'),
                $partitionSection = $modal.find('#analytical-partition-options');
            for (var formOption in formData) {
              var previousValues = formData[formOption];
              if (formOption === 'analytical-sort') {
                previousValues.forEach(addSortValue);
              } else if (formOption === 'analytical-partition') {
                previousValues.forEach(addPartition);
              } else if (formOption === 'originalFmt') {
                continue;
              } else {
                previousValues.forEach(addInputSelect);
              }
            }
          }

          // Event listener for when form is submitted
          $modal.find('#create-function').off('click.submit').on('click.submit', function (e) {
            e.preventDefault();
            var $form = $modal.find('#analytical-function-form');

            // Remove empty select fields so they don't accidentally get recorded
            $form.find('select option:selected:not([value])').parent().remove();

            // Add desc sort order
            var descendingSort = $modal.find('div.desc option:selected');
            descendingSort.each(function (i, el) {
              var elValue = el.getAttribute('value');
              el.setAttribute('value', elValue + ' desc');
            });

            // collect data and pass onto success callback
            var data = $form.serializeArray();
            data.forEach(function (d) {
              if (d.value !== '') {
                if (results[d.name]) {
                  results[d.name].push(d.value);
                } else {
                  results[d.name] = [d.value];
                }
              }
            });

            currentEl.analyticalFunctionData(results);
            setExprAlias(results, parseSql);
            arc.report.shelf.settext(abConfig.d3_elem);
            const d = abConfig.d3_elem.datum();
            const currFmt = d.display_format();
            if (abConfig.format) {
              const newFmt = abConfig.format.val;
              if (!arc.misc.isIdentical(currFmt, newFmt)) {
                if (currFmt && !currFmt.autoAdded) {
                  results.originalFmt = currFmt;
                }

                d.display_format(newFmt);
                arc.report.alertMessageSmall(abConfig.format.message, 'alert-info');
              }
            } else if (currFmt && currFmt.autoAdded) {
              if (formData) {
                d.display_format(formData.originalFmt);
              } else {
                d.display_format(undefined);
              }
            }

            if (successCb) {
              successCb(true);
              successCbMade = true;
            }

            $modal.modal('hide');
          });

          // Open modal
          $modal.modal('show');

          $modal.offOn('shown.bs.modal', function adjustSeparators() {
            // hide the line separator on the last visible section
            const $visibleSections = $modal.find('.analytical-section').filter(':visible');
            $visibleSections.find('.line-separator').show();
            $visibleSections.last().find('.line-separator').hide();
          });

          $modal.off('hide.bs.modal')
                .on('hide.bs.modal', function() {
            if (!successCbMade) {
              if (successCb) {
                successCb(false);
                successCbMade = true;
              }
            }
          });

          // HELPER FUNCTIONS
          // Sorts element based on computed name
          function sortElement(el1, el2) {
            var el1Name = el1.getComputedDisplayName(),
                el2Name = el2.getComputedDisplayName();
            if (el1Name > el2Name) {
              return 1;
            } else if (el2Name > el1Name) {
              return -1;
            } else {
              return 0;
            }
          }

          // Removes window function, duplicate, filter, and current elements
          function filterOutUnwanted(el, i, a) {
            var currentName = el.getComputedDisplayName(),
                shelfName = el.shelf().shelf_name(),
                currentExpression = el.getExpressionForEdit(),
                isWindowFunction = arc.misc.isWindowFunction(currentExpression),
                isFilterColumn = el.filter_column(),
                isCurrentElName = currentName === currentElName &&
                                  currentExpression === currentElExpression &&
                                  !currentElDeleted;

            if (shelfName === 'drill_shelf') {
              // Drill shelf elements are not present in the base visual, and hence
              // cannot be used in windows functions
              return false;
            }

            if (isFilterColumn || isWindowFunction) {
              return false;
            }

            if (isCurrentElName) {
              currentElDeleted = true;
              return false;
            }

            if (i === 0) {
              return true;
            }

            for (let idx = 0; idx < i; ++idx) {
              const previousName = a[idx].getComputedDisplayName();
              if (currentName === previousName) {
                return false;
              }
            }

            return true;
          }

          // Used to set labels within analytical function modal
          function setLabel(el, $modal) {
            var title = abConfig.custom_labels[el] || '';
            var helpTitle = abConfig.custom_labels[el + '-help'] || '';
            $modal.find('#' + el + '-label').html(title);
            $modal.find('#' + el + '-help').html(helpTitle);
            $modal.find('.' + el).show();
          }

          // Adds previous sort, partition, input elements
          function addSortValue(sortVal) {
            var $sortEl = createNewSortInput($sortSection, $sortBase, $addSortButton);
            const match = sortVal.match(/(.+) desc$/);
            const fieldName = match ? match[1] : sortVal;
            $sortEl.find('select').val(fieldName);
            if (!$sortEl.find('select').val()) {
              $sortEl.find('.delete-sort').trigger('click');
            } else if (match) {
              $sortEl.find('.order-number i').trigger('click');
            }
          }

          function addPartition(partitionVal) {
            $partitionSection.find("input[value='" + partitionVal + "']").prop('checked', true);
          }

          function addInputSelect(val) {
            $modal.find(".analytical-section [name='" + formOption + "']").val(val);
          }

          // Success callback
          function setExprAlias(results, parseSql) {
            let col_alias = abConfig.analyticalAlias;
            if (abConfig.analyticalAliasUseColumn) {
              col_alias = col_alias + `(${abConfig.currentElName})`;
            }

            abConfig.d3_elem.datum()
                    .fn_type(arc.report.fn_types.AGGREGATE)
                    .custom_expr(parseSql(results))
                    .col_alias(col_alias);
          }
        },
      };

  // Define config
  analyticalBuilder.set(userConfig);

  return analyticalBuilder;

  // HELPER FUNCTIONS
  // Creates new analytical sort input option
  function createNewSortInput($sortSection, $sortBase, $addSortButton, elementToAdd) {
    elementToAdd = elementToAdd || {};
    let {
      elementName,
      elementOrderAscending,
    } = elementToAdd;

    var $element = $($sortBase.html()).appendTo($sortSection);

    // Set event listener to change asc/desc icon
    var orderIcon = $element.find('.order-number i');
    orderIcon.offOn('click.order', (e) => {
      e.preventDefault();
      orderIcon.toggleClass(arc.report.shelf.DESC_CLASSES + ' ' + arc.report.shelf.ASC_CLASSES);
      if (orderIcon.hasClass('desc')) {
        $element.addClass('desc');
      } else {
        $element.removeClass('desc');
      }
    });

    var priority = $sortSection.find('.form-group').length;
    $element.find('.priority').text(priority);

    // Set event listener to remove order
    $element.find('.delete-sort').offOn('click.deleteSort', (e) => {
      e.preventDefault();
      deleteElement();
    });

    // If this sort item is part of pre-defined sort order, indicate it
    if (elementName) {
      // TODO: Just toggle click
      if (!elementOrderAscending) {
        orderIcon.removeClass(arc.report.shelf.ASC_CLASSES)
                 .addClass(arc.report.shelf.DESC_CLASSES);
      }

      $element.find('.form-control')
              .val(elementName);

      // Delete if there is nothing selected
      if ($element.find('option:selected').length === 0) {
        deleteElement();
      }
    }

    return $element;

    function deleteElement() {
      $element.remove();
      $addSortButton.show();
      var sortItems = $sortSection.find('.form-group');
      for (var i = 0; i < sortItems.length; i++) {
        sortItems[i].getElementsByClassName('priority')[0].textContent = i + 1;
      }
    }
  }

  // Get SQL components
  // Return back order_by, partition_by, preceding and following
  function getComponents(userInput) {
    var result = {};
    var aggregate = userInput['analytical-aggregate'];
    result.colFunction = getAggAndColumn(aggregate);
    abConfig.show_el.forEach(function (f) {
      switch (f) {
        case 'analytical-sort':
          result[f] = userInput['analytical-sort'] ?
                      'order by ' + joinFields(userInput['analytical-sort']) :
                      'order by ' + result.colFunction;
          break;
        case 'analytical-partition':
          result[f] = userInput['analytical-partition'] ?
                      'partition by ' + joinFields(userInput['analytical-partition']) :
                      '';
          break;
        case 'analytical-number':
          var preceding = userInput['analytical-number-preceding'] ||
                          userInput['analytical-number-preceding-number'],
              following = userInput['analytical-number-following'] ||
                          userInput['analytical-number-following-number'];
          result[f] = parseBetweenStatement(preceding, following);
          break;
        case 'analytical-offset':
          result[f] = userInput['analytical-offset-preceding-number'];
          break;
      }
    });

    return result;

    // HELPER FUNCTIONS
    // Replaces _CURRENT_COLUMN_ with actual column name
    function joinFields(fields) {
      return fields.join(', ')
                   .replace(/_CURRENT_COLUMN_/g, result.colFunction);
    }

    // Combine columname and aggregate for analytical functions
    function getAggAndColumn(aggregate) {
      var colname = arc.dataconnection.types.brackets(abConfig.currentElValue);
      aggregate = aggregate ? aggregate[0] + '(' : '';
      var endParen1 = aggregate === '' ? '' : ')';

      // add extra parenthesis for count(distinct())
      var endParen2 = aggregate === 'count(distinct(' ? ')' : '';
      return aggregate + colname + endParen1 + endParen2;
    }

    // parse numbers selected in analytical functions builder
    function parseBetweenStatement(precedingNum, followingNum) {
      if (precedingNum[0] === '0') {
        precedingNum = 'current row';
      } else {
        precedingNum = precedingNum[0] + ' preceding';
      }

      if (followingNum[0] === '0') {
        followingNum = 'current row';
      } else {
        followingNum = followingNum[0] + ' following';
      }

      return 'rows between ' + precedingNum + ' and ' + followingNum;
    }
  }
};
}());
