window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};

arc.report.dataset_deriveddata = function() {
  var
    modal = $('#derived-data-modal'),
    derived_data = {}, // derived data object
    deriveddataList = [], // list with built derived data saved objects
    deriveddataHash = {}, // hash of derived data labels to objects
    deriveddataAggCounter = 0, // field counters
    deriveddataDimCounter = 0,
    enabled = arc.globals.use_derived_data;

  derived_data._element = {};
  derived_data._colname = '';
  derived_data._dsreq = {};
  derived_data._label = '';
  derived_data._isMeasure = '';
  derived_data._children = undefined;
  derived_data._parents = undefined;
  derived_data._type = undefined;

  const dataset_deriveddata = function() { };

  dataset_deriveddata.derivedDataList = function(_) {
    if (!arguments.length) return deriveddataList;
    deriveddataList = _;
    deriveddataHash = {};
    deriveddataAggCounter = 0;
    deriveddataDimCounter = 0;
    deriveddataList.forEach((data, idx) => {
      deriveddataHash[data.label] = idx;
      if (data.isMeasure) {
        dataset_deriveddata.incrementAggCounter();
      } else {
        dataset_deriveddata.incrementDimCounter();
      }
    });
    return dataset_deriveddata;
  };

  dataset_deriveddata.clear = function() {
    derived_data._element = {};
    derived_data._colname = '';
    derived_data._dsreq = {};
    derived_data._label = '';
    derived_data._isMeasure = '';
    derived_data._children = undefined;
    derived_data._parents = undefined;
    derived_data._type = undefined;
  };

  dataset_deriveddata.clearList = function() {
    dataset_deriveddata.deleteDerivedDataList(
      deriveddataList.map(function (d) {
        return d.label;
      }));
    dataset_deriveddata.clear();
    deriveddataList = [];
    deriveddataHash = {};
    deriveddataAggCounter = 0;
    deriveddataDimCounter = 0;
  };

  dataset_deriveddata.deriveddata = function(_) {
    if (!arguments.length) return derived_data;
    derived_data = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.enabled = function(_) {
    if (!arguments.length) return enabled;
    enabled = (_ === 'True') || (_ === true);
    return dataset_deriveddata;
  };

  dataset_deriveddata.saved_deriveddata = function() {
    var data = {};
    data.label = derived_data._label;
    data.dsreq = derived_data._dsreq;
    data.isMeasure = derived_data._isMeasure;
    data.children = derived_data._children;
    data.parents = derived_data._parents;
    data.type = derived_data._type;
    return data;
  };

  dataset_deriveddata.shelfElement = function(_) {
    if (!arguments.length) return derived_data._element;
    derived_data._element = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.isMeasure = function(_) {
    if (!arguments.length) return derived_data._isMeasure;
    derived_data._isMeasure = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.type = function(_) {
    if (!arguments.length) return derived_data._type;
    derived_data._type = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.colname = function(_) {
    if (!arguments.length) return derived_data._colname;
    derived_data._colname = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.dsreq = function(_) {
    if (!arguments.length) return derived_data._dsreq;
    derived_data._dsreq = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.label = function(_) {
    if (!arguments.length) return derived_data._label;
    derived_data._label = _;
    return dataset_deriveddata;
  };

  dataset_deriveddata.children = function(_) {
    if (!arguments.length) return derived_data._children;
    if (derived_data._children === undefined) {
      derived_data._children = [_];
    } else {
      derived_data._children.push(_);
    }
    return dataset_deriveddata;
  };

  function column_regex(expression) {
    // use regular expression below to match expressions withing brackets, []
    var regExp = /\[([^\]]+)]/g;
    return expression.match(regExp);
  }

  dataset_deriveddata.parents = function(deriveddata_list) {
    if (!arguments.length) return derived_data._parents;

    function find_parent(labels, parents, expression) {
      // pass a list of existing derived data labels, an expression and a parents list
      // if the expression in the brackets is equal to a label add that label to parent list
      var matches = column_regex(expression);
      if (matches) {
        for (var i = 0; i < matches.length; i++) {
          let str = matches[i];
          str = str.substring(1, str.length - 1);

          for (var j = 0; j < labels.length; j++) {
            if (str === labels[j]) {
              if (parents.indexOf(str) === -1) {
                parents.push(str);
              }
            }
          }
        }
      }
    }

    function parentList(labels, dsreq, parents) {
      // given a new -to be created- derived data label and a dsreq from it
      // given an existing list of derived data labels
      // find if dsreq expression references the existing derived data label
      // if so, add the label into the list of parents of the newly created derived data
      if (!labels || !labels.length) return;
      dsreq.dimensions.forEach(function(dim) {
        find_parent(labels, parents, dim.expr);
      });
      dsreq.aggregates.forEach(function(agg) {
        find_parent(labels, parents, agg.expr);
      });
      dsreq.filters.forEach(function(fil) {
        find_parent(labels, parents, fil);
      });
    }

    // given a derived data list look up for parents in that list
    const parents = [];
    const dsreq = dataset_deriveddata.dsreq();

    const labels = [];
    deriveddata_list.forEach(function(deriveddata) {
      if (deriveddata) {
       labels.push(deriveddata.label);
      }
    });

    parentList(labels, dsreq, parents);

    if (parents.length) {
      derived_data._parents = parents;
    }
    return dataset_deriveddata;
  };

  function addParent(parent_label) {
    if (parent_label !== undefined) {
      if (derived_data._parents) {
        derived_data._parents.push(parent_label);
      } else {
        derived_data._parents = [parent_label];
      }
    }
    return dataset_deriveddata;
  }

  function check_dd_expr(dd_expr, new_expr) {
    const expr_alias = arc.misc.splitExprAndAlias(dd_expr);
    const alias = expr_alias.alias || expr_alias.expr;
    return new_expr !== alias;
  }

  dataset_deriveddata.check_colname = function(col_name, error_modal) {
    if (!col_name) return false;

    let $error_field, $error_exists;
    if (error_modal) {
      const $error_modal = $(error_modal);
      $error_field = $error_modal.find('#deriveddata-field-error');
      $error_exists = $error_modal.find('#derived_data_alreadyexists');
    }

    let valid_colname = true;
    // go over list of dims
    $('#visual-data-panel .dim-app-params li span.selection_field').each(function (index, dim) {
      if (col_name === dim.title) {
        if (error_modal) {
          $error_field.text('Dimensions');
          $error_exists.show();
        }
        valid_colname = false;
        return false;
      }
    });

    // return false if colname is invalid, no need to go over aggregate names
    if (!valid_colname) {
      return valid_colname;
    }

    // go over list of aggs
    $('#visual-data-panel .agg-app-params li span.selection_field').each(function (index, agg) {
      if (col_name === agg.title || col_name === 'Record Count') {
        if (error_modal) {
          $error_field.text('Measures');
          $error_exists.show();
        }
        valid_colname = false;
        return false;
      }
    });

    // return false if colname is invalid, no need to go over DD names
    if (!valid_colname) {
      return valid_colname;
    }

    // go over other DD
    if (dataset_deriveddata.deriveddata_list()) {
      dataset_deriveddata.derivedDataList().every(function(dd) {
        if (dd) {
          dd.dsreq.dimensions.every(function(dim) {
            valid_colname = check_dd_expr(dim.expr, col_name);
            return valid_colname === true;
          });
          if (!valid_colname) return false;
          dd.dsreq.aggregates.every(function(agg) {
            valid_colname = check_dd_expr(agg.expr, col_name);
            return valid_colname === true;
          });
        }
        return valid_colname === true;
      });
    }

    return valid_colname;
  };

  dataset_deriveddata.check_dimension = function(dim) {
    if (dim.type === 'SEGMENT') return false;
    return true;
  };

  function updatefields(elem) {
    const ismeasure = dataset_deriveddata.isMeasure();
    if (ismeasure) {
      arc.report.add_derived_list_item($('#visual-data-panel .agg-app-params'),
          'derived-data-aggs-wrapper', 'derived-data-aggs-header', elem, ismeasure);
      $('#derived-data-aggs-header').show();
    } else {
      arc.report.add_derived_list_item($('#visual-data-panel .dim-app-params'),
          'derived-data-dims-wrapper', 'derived-data-dims-header', elem, ismeasure);
      $('#derived-data-dims-header').show();
    }
  }

  function addToField() {
    // get (cloned) shelf element from derived data
    var elem = dataset_deriveddata.shelfElement();

    if (!elem) return;

    // set element's name (no alias)
    elem.dataset_colname(dataset_deriveddata.colname());
    // empty alias for derived data
    // if you don't do this then the data-original-title gets added to the element
    elem.col_alias('');
    elem.fn_name('');

    updatefields(elem);
  }

  dataset_deriveddata.deriveddata_list = function(_) {
    if (!arguments.length) {
      return {
        deriveddata_hash: deriveddataHash,
        deriveddata_list: deriveddataList,
      };
    }
    // replace existing derived data
    if (deriveddataHash[_.label()] !== undefined) {
      deriveddataList[deriveddataHash[_.label()]] = undefined;
    }
    deriveddataHash[_.label()] = deriveddataList.length;
    deriveddataList.push(_.saved_deriveddata());
    if (_.saved_deriveddata().isMeasure) {
      dataset_deriveddata.incrementAggCounter();
    } else {
      dataset_deriveddata.incrementDimCounter();
    }
    return dataset_deriveddata;
  };

  dataset_deriveddata.getFromList = function(label) {
    let dd;

    deriveddataList.some(function(d) {
      if (d.label === label) {
        dd = d;
        return true;
      }
    });

    return dd;
  };

  function derivedDataLabels(label, label_list) {
    // given a derived data label and a list add label to list
    // and every child derived data label recursively

    var ddata = deriveddataList[deriveddataHash[label]];

    if (!ddata) return;

    label_list.push(label);

    const children_labels = ddata.children;

    if (children_labels && children_labels.length) {
      children_labels.forEach(function(child_label) {
        derivedDataLabels(child_label, label_list);
      });
    }
  }

  dataset_deriveddata.deleteDerivedData = function(label) {
    const delete_modal = $('#delete-deriveddata-modal');
    if (arc.misc.modalShown(delete_modal)) {
      return;
    }

    const ddata = deriveddataList[deriveddataHash[label]];

    if (!ddata) return;

    // if derived data has children must delete them as well
    if (ddata.children && ddata.children.length) {
      const label_list = [];
      derivedDataLabels(label, label_list);

      $('#deriveddata-children-delete').text(label_list.slice(1).join(', '));
      delete_modal.modal('show');
      $('#delete-deriveddata-yes')
        .off('click.derivedata')
        .on('click.derivedata', function() {
          delete_modal.modal('hide');
          dataset_deriveddata.deleteDerivedDataList(label_list);
      });
    } else {
      $('#deriveddata-children-delete').text('');
      dataset_deriveddata.deleteDerivedDataList([label]);
    }
  };

  dataset_deriveddata.deleteDerivedDataList = function(label_list) {
    for (var i = 0; i < label_list.length; i++) {
      const label = label_list[i];
      const ddata = deriveddataList[deriveddataHash[label]];
      if (ddata) {
        const parents = ddata.parents;
        // removed from builder list and hash table
        deriveddataList[deriveddataHash[label]] = undefined;
        delete deriveddataHash[label];
        // removed from fields
        dataset_deriveddata.removeDerivedDataFromField(label, ddata.isMeasure);
        // remove from list of children in parents
        dataset_deriveddata.removeChildLabel(label, parents);
      }
    }
  };

  dataset_deriveddata.removeDerivedDataFromField = function(label, isMeasure) {
    const $vizTab = $('#visual-data-panel');
    const fieldType = isMeasure ? 'aggs' : 'dims';
    const fullFieldType = isMeasure ? 'aggregates' : 'dimensions';
    const fields = $(`#derived-data-${fieldType}-wrapper`).find('span.selection_field');
    fields.each(function removeField(index, field) {
      if (field.title === label) {
        $(field.parentElement).remove();
        dataset_deriveddata.decrementFieldCounter(fieldType);
        const field_count = $vizTab.find(`.app-${fullFieldType}-count`);
        const count = parseInt(field_count.text());
        field_count.text(count - 1);

        if (dataset_deriveddata.fieldCounter(fieldType) === 0) {
          $(`#derived-data-${fieldType}-header`).hide();
        }
      }
    });

    return dataset_deriveddata;
  };

  dataset_deriveddata.removeChildLabel = function(label, parents) {
    // remove child label from list of children in its parents
    if (parents) {
      parents.forEach(function(parent_label) {
        var parent = deriveddataList[deriveddataHash[parent_label]];
        if (parent && parent.children) {
          const child_index = parent.children.indexOf(label);
          if (child_index > -1) {
            parent.children.splice(child_index, 1);
          }
        }
      });
    }
  };

  dataset_deriveddata.incrementAggCounter = function() {
    deriveddataAggCounter += 1;
    return dataset_deriveddata;
  };

  dataset_deriveddata.incrementDimCounter = function() {
    deriveddataDimCounter += 1;
    return dataset_deriveddata;
  };

  dataset_deriveddata.decrementFieldCounter = function(fieldType) {
    if (fieldType === 'aggs') {
      deriveddataAggCounter -= 1;
      return dataset_deriveddata;
    }

    deriveddataDimCounter -= 1;
    return dataset_deriveddata;
  };

  dataset_deriveddata.fieldCounter = function(fieldType) {
    if (fieldType === 'aggs') {
      return deriveddataAggCounter;
    }

    return deriveddataDimCounter;
  };

  dataset_deriveddata.show_hide = function(_) {
    if (_) {
      if (arc.misc.modalShown(modal)) {
        return;
      }

      modal.find('#dd-name-input').val('');
      modal.find('.alert').hide();
      const dataset = dataset_deriveddata.shelfElement().dataset();
      const savemode = (dataset && arc.misc.hasPerm('ds_manage', dataset.dataset_perms())) ?
                       'ds-expression-only' : 'viz-expression-only';
      modal.find(`input[value=${savemode}]`).prop("checked", true);
      modal.find('#dd-save-type').off().on("change", function () {
        modal.find('.alert').hide();
      });
      modal.find('#dd-name-input').off().on("update", function () {
        console.log("Byte me: " + $(this).val());
      });
      modal.modal('show');
    } else if (!_) {
      modal.modal('hide');
    }

    return dataset_deriveddata;
  };

  dataset_deriveddata.rename_colname = function (expression) {
    const expr_alias = arc.misc.splitExprAndAlias(expression);
    let alias = expr_alias.alias || expr_alias.expr;

    while (!dataset_deriveddata.check_colname(alias, null /* error_modal */)) {
      alias = `_${alias}`;
    }

    return `${expr_alias.expr} as '${alias}'`;
  };

  // Function to handle 'save shelf expression to dataset' option
  function dd_save_to_dataset(element, user_label) {

    const dataset = element.dataset();
    const expr = element.getExpressionComponents().expr;
    const col = dataset.get_column(user_label);

    if (col && col.isBaseCol()) {
      $('#derived_data_alreadyexists').hide();
      $('#dataset_basecol_alreadyexists').show();
      return;
    }

    // Validate new DD name is not the same as existing DD, Record Count or Segment
    let valid_name = true;
    let error_field_text;

    if (user_label === 'Segment' && dataset.segments().length) {
      valid_name = false;
      error_field_text = 'Dimensions';
    } else if (user_label === 'Record Count') {
      valid_name = false;
      error_field_text = 'Measures';
    } else if (dataset_deriveddata.deriveddata_list()) {
      dataset_deriveddata.derivedDataList().every((dd) => {
        if (dd) {
          dd.dsreq.dimensions.every((dim) => {
            valid_name = check_dd_expr(dim.expr, user_label);
            error_field_text = 'Dimensions';
            return valid_name === true;
          });

          if (!valid_name) {
            return valid_name;
          }

          dd.dsreq.aggregates.every((agg) => {
            valid_name = check_dd_expr(agg.expr, user_label);
            error_field_text = 'Measures';
            return valid_name === true;
          });
        }

        return valid_name;
      });
    }

    if (!valid_name) {
      $('#dataset_basecol_alreadyexists').hide();
      $('#deriveddata-field-error').text(error_field_text);
      $('#derived_data_alreadyexists').show();
      return;
    }

    // Helper function to save/update column to dataset
    const savecol = function(el, ds, column, type, hasagg, ispart) {
      if (column === undefined) {
        let srccol = ds.get_column(el.dataset_colname());
        // If no column associated with this element, then just get the first
        // column in the dataset.
        if (!srccol) {
          srccol = ds.tables()[0].base_columns()[0];
        }

        column = srccol.table().clone_column(srccol);
      }

      column.attr('expr', expr);
      column.attr('type', type);
      column.attr('alias', user_label);
      column.attr('isdim', !(arc.report.measure_types[type]));
      column.attr('exprhasagg', hasagg);
      column.attr('ispart', ispart);
      ds.save(() => {
        $('#expression-modal').modal('hide');
        arc.report.dataset_setup(ds, true);
        // Redrawing all DDs on empty sidepanel
        dataset_deriveddata.derivedDataList().forEach((ddItem) => {
          dataset_deriveddata.addDerivedDataField(ddItem, ds);
        });
        const report = arc.report.getPrimaryReport();
        report.viz().shelves().setup_dimagg_draggable();
        $('#derived-data-modal').modal('hide');
        $('.properties-panel-close').trigger('click');
        arc.report.alertMessageSmall('Dataset fields updated!', 'alert-success');
      });
    };

    // Validate the expression first.  Then either show error, ask to confirm overwrite
    // of existing column, or go directly to save column if name doesn't exist.
    function validate_deriveddata(params) {
      const { err, fieldLevelError, $errField, type, hasagg, ispart, info } = params;
      if (err) {
        $('#dataset_expr_error_text').text(err);
        $('#dataset_expr_error').show();
      } else if (col) {
        const m = $('#save-to-ds-confirm-modal');
        if (arc.misc.modalShown(m)) {
          return;
        }

        m.find('.modal-confirm-txt')
         .text(`Calculated field ${user_label} already exists in dataset`);
        m.find('#save-to-ds-anyway').hide();
        m.find('#save-to-ds-overwrite')
          .show()
          .off('click.overwrite')
          .on('click.overwrite', function () {
            savecol(element, dataset, col, type, hasagg, ispart);
            m.modal('toggle');
        });
        m.modal('show');
      } else {
        savecol(element, dataset, col, type, hasagg, ispart);
      }
    }

    arc.validate_expression(expr, dataset, validate_deriveddata, 'column', { alias: user_label });
  }

  dataset_deriveddata.setupDerivedData = function(existingAggregates = []) {
    // event handler for save button in modal
    const $form = $('#edit-deriveddata-form');
    $form.validator();
    $form
      .offOn('submit.create-deriveddata', function(e) {
        e.preventDefault();

        const user_label = $(e.currentTarget).find('#dd-name-input').val();

        const save_mode = $(e.currentTarget).find('input[name=dd-save-type]:checked').val();
        const expression_only = (save_mode === 'viz-expression-only');

        // get original shelf element passed from shelf to dataset_deriveddata
        const element = dataset_deriveddata.shelfElement();

        if (save_mode === 'ds-expression-only') {
          dd_save_to_dataset(element, user_label);
          return;
        }

        // user label becomes the deriveddata label and colname
        if (dataset_deriveddata.check_colname(user_label, this)) {
          const ddList = dataset_deriveddata.deriveddata_list();

          // controls whether derived data is a dimension or a measure field
          if (element.expr_hasagg() || element.expressionIsAggregate({ existingAggregates })) {
            dataset_deriveddata.isMeasure(true);
          } else {
            dataset_deriveddata.isMeasure(false);
          }

          // clone shelf element
          const cloned_element = element.clone();

          // alias original shelf element, need alias in created dsreq from shelves
          const old_alias = element.col_alias();
          if (old_alias === 'Record Count') {
            cloned_element.col_alias(user_label);
          }

          // derived data column is simply a column from a new table,
          // it does not contain an aggregate expression
          // or a custom expression even if it was defined as such
          cloned_element.custom_expr(undefined);
          cloned_element.expr_hasagg(false);

          dataset_deriveddata.shelfElement(cloned_element);

          element.col_alias(user_label);

          const viz = arc.report.getPrimaryViz();

          // from shelves to datarequest
          let dsreq;
          if (!expression_only) {
            const datarequest = viz.shelvesToDataRequest(undefined, {no_param_sub:true});
            dsreq = datarequest.dsreqNoDerivedData(true);
            // rename DD dims/aggs aliases here
            dsreq.dimensions.forEach(function(dim) {
              dim.expr = dataset_deriveddata.rename_colname(dim.expr);
            });

            dsreq.aggregates.forEach(function(agg) {
              agg.expr = dataset_deriveddata.rename_colname(agg.expr);
            });
          } else {
            const rb = arc.requestBuilder();

            // cloned_element.custom_expr(element.custom_expr());
            cloned_element.expr_hasagg(element.expressionIsAggregate({ existingAggregates }));
            dataset_deriveddata.shelfElement(cloned_element);
            if (element.expressionIsAggregate({ existingAggregates })) {
              rb.addAggregate(element.getExpression());
              if (element.getExpression().match(/over *\(.* *\[.*\] *.*\)/)) {
                arc.report.alertMessageSmall('A window function is saved as an expression only.\n' +
                  'Any referenced dimensions will be needed in the Dimensions shelf when this expression is used.',
                  'alert-success');
              }
            } else {
              rb.addDimension(element.getExpression());
            }

            rb.limit(undefined);
            rb.deriveddataList([]);
            dsreq = rb.dsreq();
          }

          let contains_segment = false;
          dsreq.dimensions.forEach(function(dim) {
            if (!dataset_deriveddata.check_dimension(dim)) {
              contains_segment = true;
            }
          });

          if (contains_segment) {
            $(this).find('#derived_data_segment_error').show();
            // reset the original shelf element alias
            element.col_alias(old_alias);
            return false;
          }

          // reset the original shelf element alias
          element.col_alias(old_alias);

          dataset_deriveddata.colname(user_label);
          dataset_deriveddata.label(user_label);
          dataset_deriveddata.type(element.dataset_coltype());
          dataset_deriveddata.dsreq(dsreq);

          // find and set parents of new derived data
          dataset_deriveddata.parents(ddList.deriveddata_list);

          // if parents are present, add derived data label to list of children in them
          if (dataset_deriveddata.parents()) {
            let parent_index;
            let parent;
            dataset_deriveddata.parents().forEach(function(parent_label) {
              parent_index = ddList.deriveddata_hash[parent_label];
              parent = ddList.deriveddata_list[parent_index];
              if (!parent.children) {
                parent.children = [dataset_deriveddata.label()];
              } else {
                parent.children.push(dataset_deriveddata.label());
              }
            });
          }

          // add derived element to Derived Data Fields
          addToField();

          // add derived data to builder list (used in data API dsreq)
          dataset_deriveddata.deriveddata_list(dataset_deriveddata);

          dataset_deriveddata.derived_popover_add();
          dataset_deriveddata.derived_popover_delete();

          dataset_deriveddata.clear();

          $(this).find('input[type=checkbox]').prop('checked', false);
          $(this).closest('#derived-data-modal').modal('hide');
        }
        return false;
      });

    // other event handlers below...
  };

  dataset_deriveddata.derived_popover_add = function() {
    var popover_label = $('#derived-popover-label');
    var popover_name = $('#derived-popover-name');
    var popover_dims = $('#derived-popover-dims');
    var popover_dim = $('#derived-popover-dim');
    var popover_aggs = $('#derived-popover-aggs');
    var popover_agg = $('#derived-popover-agg');
    var popover_fils = $('#derived-popover-fils');
    var popover_fil = $('#derived-popover-fil');
    var popover_limit = $('#derived-popover-limit');
    var popover_lim = $('#derived-popover-lim');

    $('.deriveddata-viewdefinition-popover:not([data-original-title])')
      .offOn('click.derivedDataPropagation', (e) => e.stopPropagation())
      .popover({
        animation: false,
        html: true,
        sanitize: false,
        trigger: 'click',
        placement: 'left',
        title: 'Derived Data Definition',
        content() {
          const label = $(this).parent()
                             .parent()
                             .parent()
                             .find('span.selection_field')
                              .text();
          const ddata = dataset_deriveddata.getFromList(label);
          const dims = ddata.dsreq.dimensions;
          const aggs = ddata.dsreq.aggregates;
          const fils = ddata.dsreq.filters;
          const havs = ddata.dsreq.having;
          const limit = ddata.dsreq.limit;

          // Empty the previous content
          $('#derived-view-popover').find('.data').empty();

          popover_name.append(`<span class="field-highlight">${label}</span>`);
          popover_label.show();
          popover_name.show();
          if (dims.length) {
            dims.forEach(function(dim, index) {
              var order_text = '';

              if (dim.order) {
                if (dim.order.asc) {
                  order_text = ' [ascending]';
                } else {
                  order_text = ' [descending]';
                }
              }

              let breakline = '';

              if (index) {
                breakline = '<br>';
              }

              const dim_text = dim.expr.replace(/</g, '&lt;').replace(/>/g, '&gt;');
              if (dim_text.indexOf(`as '${label}'`) > -1) {
                popover_dims.append(`${breakline}<span class="field-highlight">${dim_text + order_text}</span>`);
              } else {
                popover_dims.append(`${breakline}<span>${dim_text + order_text}</span>`);
              }
            });

            popover_dims.show();
            popover_dim.show();
          }

          if (aggs.length) {
            aggs.forEach(function(agg, index) {
              var order_text = '';
              if (agg.order) {
                if (agg.order.asc) {
                  order_text = ' [ascending]';
                } else {
                  order_text = ' [descending]';
                }
              }

              let breakline = '';

              if (index) {
                breakline = '<br>';
              }

              const agg_text = agg.expr.replace(/</g, '&lt;').replace(/>/g, '&gt;');

              if (agg_text.indexOf(`as '${label}'`) > -1) {
                popover_aggs.append(`${breakline}<span class="field-highlight">${agg_text + order_text}</span>`);
              } else {
                popover_aggs.append(`${breakline}<span>${agg_text + order_text}</span>`);
              }
            });

            popover_aggs.show();
            popover_agg.show();
          }

          let fil_text = '';
          if (fils.length) {
            const fil_array = [];
            fils.forEach(function(fil) {
              fil_array.push(fil);
            });
            fil_text = fil_array.join(', ');
          }

          if (havs.length) {
            const hav_array = [];
            havs.forEach(function(hav) {
              hav_array.push(hav);
            });

            fil_text += hav_array.join(', ');
          }
          if (fils.length || havs.length) {
            popover_fils.text(fil_text);
            popover_fils.show();
            popover_fil.show();
          }
          if (limit && limit !== '') {
            popover_limit.text(limit);
            popover_lim.show();
            popover_limit.show();
          }
          return $('#derived-view-popover').html();
        },

        container: 'body',
      });

    $('html').off('click').on('click', function (e) {
      // close popovers when user clicks anywhere other than the open popover
      if ($(e.target).data('toggle') !== 'popover' && $(e.target).parents('.popover.in').length === 0) {
            popover_name.text('');
            popover_label.hide();
            popover_name.hide();
            popover_dims.text('');
            popover_dim.hide();
            popover_dims.hide();
            popover_aggs.text('');
            popover_aggs.hide();
            popover_agg.hide();
            popover_fils.text('');
            popover_fils.hide();
            popover_fil.hide();
            popover_limit.text('');
            popover_lim.hide();
            popover_limit.hide();
            $('[data-toggle="popover"]').popover('hide');
        }
    });

    return dataset_deriveddata;
  };

  dataset_deriveddata.derived_popover_delete = function() {
    const $delView = $('.del-view-popover:not([data-original-title])');

    $delView.on('click', (e) => e.stopPropagation());

    const options = {
      animation: false,
      html: true,
      trigger: 'click',
      sanitize: false,
      content() {
        const label = $(this).parent()
                           .parent()
                           .parent()
                           .find('span.selection_field')
                           .text();

        $('[data-toggle="popover"]').popover('hide');
        dataset_deriveddata.deleteDerivedData(label);
        arc.report.getPrimaryViz().page_needs_save();
      },
    };

    $delView.popover(options);

    $('html').on('click', function (e) {
      // close popovers when user clicks anywhere other than the open popover
      if ($(e.target).data('toggle') !== 'popover' && $(e.target).parents('.popover.in').length === 0) {
        $('[data-toggle="popover"]').popover('hide');
      }
    });
  };

  dataset_deriveddata.createDerivedData = function(data, dataset, hidden) {
    if (!data) return;

    let show_message = false;

    // determine whether to remove an expression from a DD creation
    // by matching columns in an expression to dataset columns
    // if hidden column is found in matches return true (expression to be removed)
    // else return false
    function remove_expression(currentDataset, expression) {
      var matches = column_regex(expression);
      if (matches) {
        for (var i = 0; i < matches.length; i++) {
          let str = matches[i];
          str = str.substring(1, str.length - 1);
          const column = currentDataset.get_column(str);
          if (column && column.attr('hidden')) {
            return true;
          }
        }
      }
      return false;
    }

    // determine whether or not to create derived data
    // if derived data label refers to missing column return false
    // else return true
    function remove_deriveddata(label, expression) {
      var splits = expression.split(' as \'');
      return splits.length > 1 && splits[splits.length - 1].slice(0, -1) === label;
    }

    function check_dsreq(currentDataset, dsreq, label) {
      var i;
      for (i = 0; i < dsreq.dimensions.length; i++) {
        if (remove_expression(currentDataset, dsreq.dimensions[i].expr)) {
          if (remove_deriveddata(label, dsreq.dimensions[i].expr)) {
            return false;
          }

          dsreq.dimensions[i] = undefined;
          show_message = true;
        }
      }

      dsreq.dimensions = dsreq.dimensions.filter(function(e) { return e; });

      for (i = 0; i < dsreq.aggregates.length; i++) {
        if (remove_expression(currentDataset, dsreq.aggregates[i].expr)) {
          if (remove_deriveddata(label, dsreq.aggregates[i].expr)) {
            return false;
          }

          dsreq.aggregates[i] = undefined;
          show_message = true;
        }
      }
      dsreq.aggregates = dsreq.aggregates.filter(function(e) { return e; });

      for (i = 0; i < dsreq.filters.length; i++) {
        if (remove_expression(currentDataset, dsreq.filters[i])) {
          dsreq.filters[i] = undefined;
          show_message = true;
        }
      }

      dsreq.filters = dsreq.filters.filter(function(e) { return e; });

      return true;
    }

    if (dataset && !check_dsreq(dataset, data.dsreq, data.label)) {
      return dataset_deriveddata;
    }

    dataset_deriveddata.addDerivedDataField(data, dataset);

    dataset_deriveddata.deriveddata_list(dataset_deriveddata);

    dataset_deriveddata.clear();

    if (show_message) {
      hidden.push(`"${data.label}"`);
    }

    return dataset_deriveddata;
  };

  dataset_deriveddata.addDerivedDataField = function(data, dataset) {
    // Not adding when DD was deleted
    if (!data) {
      return dataset_deriveddata;
    }

    const element = arc.report.shelf.element().dataset_colname(data.label)
                                              .fn_name('')
                                              .dataset(dataset);

    dataset_deriveddata.label(data.label)
                       .colname(data.label)
                       .dsreq(data.dsreq)
                       .isMeasure(data.isMeasure)
                       .type(data.type);

    element.dataset_coltype(dataset_deriveddata.type());

    if (data.children) {
      data.children.forEach(function (child) {
        dataset_deriveddata.children(child);
      });
    }

    if (data.parents) {
      data.parents.forEach(function (parent) {
        addParent(parent);
      });
    }

    dataset_deriveddata.shelfElement(element);
    addToField();
    dataset_deriveddata.derived_popover_add();
    dataset_deriveddata.derived_popover_delete();

    return dataset_deriveddata;
  };

  return dataset_deriveddata;
};
}());
