window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};

arc.counters.data.renderTime = 0;
arc.counters.data.maxRenderTime = 0;

arc.counters.initialize('dataResultTotalRows');
arc.counters.initialize('viz_drawVisual');

arc.report.driver = function(_viz) {
  var sb = null,
      dsreq,
      sb_sql,
      shelf_contents = [],
      sort_orders_asc = {},
      viz,
      report,
      data_request,
      dataResult,
      refreshCount = 0,
      filter_widget;

  arc.counters.increment('arcReportDriver');

  var driver = arc.report.base();
  viz = _viz;
  driver.viz(viz);
  viz.viz_lib(driver);

  driver.report = function(_) {
    if (!arguments.length) return report;
    report = _;
    return driver;
  };

  function countRows(dataResult) {
    if (!dataResult) {
      return;
    }

    const rows = dataResult.rows();
    if (!rows) {
      return;
    }

    arc.counters.increment('dataResultTotalRows', rows.length);
  }

  driver.dataResult = function(_) {
    if (!arguments.length) return dataResult;
    dataResult = _;

    if (driver.aliasesAfterQuery()) {
      // Solr and BQ don't deal with aliases - do it here
      // TODO: This may be a better way to deal with aliases for all
      //       connections.
      if (data_request) {
        const resultColumns = dataResult.columns();
        const sb_colnames = driver.sb_colnames();

        data_request.columns().forEach(function(c, i) {
          const colname = driver.substituteParams(c.colname());
          if (resultColumns.length >= i + 1) {
            resultColumns[i].colname(colname);
          }

          if (sb_colnames.length >= i + 1) {
            sb_colnames[i] = colname;
          }
        });
      }
    }

    if (viz) {
      arc.counters.increment('viz_setDataResult');
      driver.apply_dataTransformation(dataResult);
      countRows(dataResult);
      viz.viz_setDataResult(dataResult);
      arc.perf.PAGEPERF.addProfile({
        id: driver.report_id(),
        type: 'Set data result',
        time: 0,
        profile: [
          [
            'rows',
            dataResult.rows().length,
          ],
        ],
      });
    }

    return driver;
  };

  var inlineExtensions = [];

  driver.setInlineJsAssets = function(assets) {
    inlineExtensions =
        assets.map(function(asset) {
          return arc.extension()
                    .js(asset)
                    .processJs(driver);
        });

    return driver;
  };

  driver.shelf_contents = function(_) {
    if (!arguments.length) return shelf_contents;
    shelf_contents = _;
    return driver;
  };

  driver.viz = function() {
    return viz;
  };

  driver.sb_sql = function(_) {
    if (!arguments.length) return sb_sql;
    sb_sql = _;
    return driver;
  };

  function setupInlineExtentions() {
    var ds = driver.dataset();
    var jsAssets = [];
    var incremental;

    if (ds && (ds.dataconnection_type() === 'ksql')) {
      jsAssets.push('return ' +
          arc.ext.streamingQueries.toString() + '()');
    }

    if (!driver.disableIncrementalQueries() && ds) {
      incremental = ds.enable_incremental_results();

      if (incremental) {
        jsAssets.push('return ' +
            arc.ext.incrementalQueries.toString() + '()');
      }
    }

    driver.setInlineJsAssets(jsAssets);
  }

  /*
   * Everytime we draw something within the shelves div, setup
   * a data-drawid element that indicates a new draw.
   *
   * See arc_visual_action.js for how this is used - some
   * draw calls are handled asynchronously, which means it is
   * important to know if the same draw is still active, or
   * a different draw has occurred after it.
   */
  function setDrawId(dom_id) {
    const uniqueId = `${arc.misc.uniqueId()}`;
    if (dom_id) {
      const $shelves = $(`#${dom_id}`);
      $shelves.attr('data-drawid', uniqueId);
    }
  }

  driver.getDrawId = function(dom_id) {
    return $(`#${dom_id}`).attr('data-drawid');
  };

  // Called once at startup time
  driver.setup = function() {
    arc.counters.increment('viztype_' + driver.viz_type());
    arc.counters.increment('viz_drawDesigner');

    const shelves_dom_id = driver.shelves_dom_id();
    const chart_dom_id = driver.chart_div();
    setDrawId(shelves_dom_id);
    viz.viz_drawDesigner(shelves_dom_id, chart_dom_id);

    // Setup inline extensions in .setup because if this is a new
    // visual on a dataset, .restore will never be called
    setupInlineExtentions();

    return driver;
  };

  // Called to return an object that can later be passed into
  // driver.restore to restore state
  driver.save = function() {
    arc.counters.increment('viz_getSettings');
    return viz.viz_getSettings();
  };

  driver.afterDetailedDataRequestCb = function(dataRequest) {
    // Implement the .afterDetailedDataRequest API
    extensions().forEach(function(extension) {
      var code = extension.code();
      if (code) {
        try {
          if (code.afterDetailedDataRequest) {
            arc.counters.increment('ext_afterDetailedDataRequest_start');
            dataRequest = code.afterDetailedDataRequest(dataRequest);
            arc.counters.increment('ext_afterDetailedDataRequest_end');
          }
        } catch (e) {
          arc.counters.increment('ext_afterDetailedDataRequest_error');
          extensionError(extension.extensionId(), 'afterDetailedDataRequest',
              e.toString() + e.stack);
        }
      }
    });

    return dataRequest;
  };

  function extensions() {
    if (!arc.globals.apply_custom_styles) {
      return [];
    }

    var cssSetting = driver.cssSetting();
    if (!cssSetting) {
      return [];
    }

    var es = cssSetting.extensions();
    return es.all().concat(inlineExtensions);
  }

  function extensionError(id, entryPoint, detail) {
    driver.alertMessage('Error rendering visual ' + driver.report_id() +
        '. Extension ' + id +
        '. Entrypoint ' + entryPoint,
        detail);
  }

  function drawEnabled() {
    var enableDraw = true;

    // Implement the .disableDraw API
    extensions().forEach(function(extension) {
      var code = extension.code();
      if (code) {
        try {
          if (code.disableDraw && (code.disableDraw() === true)) {
            enableDraw = false;
          }
        } catch (e) {
          extensionError(extension.extensionId(), 'disableDraw', e.toString() + e.stack);
        }
      }
    });

    return enableDraw;
  }

  driver.drawVisual = function(isWindowResize) {
    arc.misc.afterCountersEqual({
      counter1: 'transitionStart',
      counter2: 'transitionEnd',
      delay: 50,
      cb: function() {
        drawVisualAfterTransitions(isWindowResize);
      },

      finalTimeout: 500,
    });
  };

  function drawVisualAfterTransitions(isWindowResize) {
    // NOTE: this counter is used in arcweb/apps/views_snapshot.py
    arc.counters.increment('viz_drawVisual_start');
    var t1 = new Date();

    try {
      drawVisualWork(isWindowResize);
    } finally {
      // NOTE: this counter is used in arcweb/apps/views_snapshot.py
      arc.counters.increment('viz_drawVisual_end');

      var t2 = new Date();
      arc.perf.PAGEPERF.addProfile({
        id: driver.report_id(),
        type: 'Draw visual',
        time: t2 - t1,
      });
    }
  }

  function drawVisualWork(isWindowResize) {
    // Clear errors for a visual when drawing
    arc.errors.clearErrors(driver);

    if (!driver.active()) {
      return;
    }

    if ($('#' + driver.chart_div()).length === 0) {
      // In appgroups, we may navigate away from a dashboard while still
      // having some pending draw requests
      return;
    }

    var enableDraw = drawEnabled();

    driver.sizeChartDivHeight();

    // Implement the .beforeDraw API
    extensions().forEach(function(extension) {
      var code = extension.code();
      if (code) {
        try {
          if (code.beforeDraw) {
            arc.counters.increment('ext_beforeDraw_start');
            code.beforeDraw();
            arc.counters.increment('ext_beforeDraw_end');
          }
        } catch (e) {
          arc.counters.increment('ext_beforeDraw_error');
          extensionError(extension.extensionId(), 'beforeDraw', e.toString() + e.stack);
        }
      }
    });

    viz.viz_fetchCollateral(driver.chart_div(), isWindowResize, function(viz_div, redraw) {
      if (!driver.active()) {
        return;
      }

      if (enableDraw) {
        const d1 = new Date();
        arc.counters.increment('viz_drawVisual');
        viz.viz_drawVisual(viz_div, redraw);
        const d2 = new Date();
        const diff = d2 - d1;
        arc.counters.data.renderTime += diff;
        arc.counters.data.maxRenderTime = Math.max(diff,
            arc.counters.data.maxRenderTime);

        if (dataResult) {
          const rowCount = dataResult.rows() && dataResult.rows().length;
          driver.warnIfMaxRows(rowCount);
        }

        if (!isWindowResize) {
          driver.displayPersistentMessages();
        }

        driver.highlightMarks();
        driver.updateChartDivHeight();
      }

      // Implement the .afterDraw API
      extensions().forEach(function(extension) {
        var code = extension.code();
        if (code) {
          try {
            if (code.afterDraw) {
              arc.counters.increment('ext_afterDraw_start');
              code.afterDraw();
              arc.counters.increment('ext_afterDraw_end');
            }
          } catch (e) {
            arc.counters.increment('ext_afterDraw_error');
            extensionError(extension.extensionId(), 'afterDraw', e.toString() + e.stack);
          }
        }
      });
    });
  }

  driver.restore = function(table_info, switchingVisualType) {
    arc.counters.increment('viz_setSettings');
    viz.viz_setSettings(table_info, switchingVisualType);

    // new settings will impact the inline extensions we enable
    setupInlineExtentions();

    if (arc.globals.is_dashboard && !driver.refresh_visible()) {
      driver.drawVisual();
    }

    return driver;
  };

  function fetchCustomJsAndCallback(cb, from_cache) {
    if (!arc.globals.apply_custom_styles) {
      cb();
      return;
    }

    var e = extensions();
    if (e.length === 0) {
      // Bind any inline scripts, and then call refresh
      driver.bindScripts();
      cb();
      return;
    }

    var cssSetting = driver.cssSetting();
    if (cssSetting) {
      cssSetting.extensions().fetchExtensions(driver, cb, from_cache);
      return;
    }

    cb(false);
  }

  function fetchCustomJsAndDraw(force_redraw, resolve, reject) {
    fetchCustomJsAndCallback(function(jsChanged) {
      if (jsChanged) {
        force_redraw = true;
      }

      refreshAfterFetch(
          {
            force_redraw,
          },
          resolve, reject);

    }, /* from_cache */ arc.globals.is_edit ? false : true);
  }

  driver.bindScripts = function() {
    extensions().forEach(function(extension) {
      extension.processJs(driver);
    });
  };

  driver.refreshCount = function() {
    return refreshCount;
  };

  function decrementRefreshCounters() {
    arc.counters.increment('refresh_end');
    arc.counters.increment(`refresh_end_${driver.report_id()}`);
    if (arc.misc.isFilterVisual(driver.viz_type())) {
      arc.counters.increment('refresh_end_filter');
    }
  }

  function incrementRefreshCounters() {
    ++refreshCount;
    arc.counters.increment('refresh_start');
    arc.counters.increment(`refresh_start_${driver.report_id()}`);
    if (arc.misc.isFilterVisual(driver.viz_type())) {
      arc.counters.increment('refresh_start_filter');
    }
  }

  driver.refresh = function(force_redraw, _filter_widget, manualRefresh) {
    incrementRefreshCounters();

    // Clear errors for a visual when refreshing
    arc.errors.clearErrors(driver);
    new Promise((resolve, reject) => {
      if (!arc.misc.isFilterVisual(driver.viz_type()) && !manualRefresh) {
        const { dashboard } = arc.globals;
        if (dashboard && dashboard.manualVisualRefresh()) {
          dashboard.addManualRefresh({
            driver,
            force_redraw,
            filter_widget: _filter_widget,
          });

          resolve();
          return;
        }
      }

      filter_widget = _filter_widget;
      fetchCustomJsAndDraw(force_redraw, resolve, reject);
    })
    .then(decrementRefreshCounters)
    .catch(function(error) {
      if (error === 'Cancelled') {
        console.log(driver.getLogMessage('Error refreshing visual'), error);
      } else {
        driver.alertMessage('Error refreshing visual', error);
      }

      decrementRefreshCounters();
    });

    return driver;
  };

  function getDataRequest(config={}) {
    const {
      previousDataRequest,
      previousDataResult,
    } = config;

    arc.counters.increment('viz_getDataRequest');
    driver.resetParameterTracking();
    var data_request = viz.viz_getDataRequest({
      previousDataRequest,
      previousDataResult,
      permitAutoTimeBucketing: true,
    });

    if (
      arc.globals?.gflags?.is_snapshot &&
      arc.globals?.gflags?.limit &&
      data_request.request_type() !== 'ignore'
    ) {
      var dsreq = JSON.parse(data_request.dsreq());
      dsreq.limit = arc.globals.gflags.limit;
      data_request.dsreq(dsreq);
    }

    // To able to create visual on unsaved dataset
    // (DatasetProfiling a dimension field on sql query result),
    // we add the dataset_object to the request
    if (driver.dataset() && !driver.dataset().dataset_id()) {
      const dsreq_locale = JSON.parse(data_request.dsreq());
      dsreq_locale.dataset_object = driver.dataset().buildInfo();
      data_request.dsreq(dsreq_locale);
    }

    // Implement the .afterDataRequest API
    extensions().forEach(function(extension) {
      var code = extension.code();
      if (code) {
        try {
          if (code.afterDataRequest) {
            data_request = code.afterDataRequest(data_request);
          }
        } catch (e) {
          extensionError(extension.extensionId(), 'afterDataRequest', e.toString() + e.stack);
        }
      }
    });

    return data_request;
  }

  function refreshAfterFetch(config, resolve, reject) {
    let {
      force_redraw,
      previousDataRequest,
      previousDataResult,
    } = config;

    var c = driver.get_shelf_contents();
    var comments = c.map(function(d) {
      return d[3];
    }).join(', ');

    if (!previousDataRequest) {
      if (!arc.arraysIdentical(c, shelf_contents)) {
        if (!force_redraw) {
          force_redraw = true;
        }

        driver.shelf_contents(c);
      }

      // First, apply CSS classes and draw the title
      driver.initializeCssAndTitle(filter_widget);
    }

    data_request = getDataRequest({
      previousDataRequest,
      previousDataResult,
    });

    if (data_request) {
      if (data_request.request_type() === 'ignore') {
        driver.forceReloadData();
        driver.initDataColumns();
        driver.dataResult(arc.data.result());
        viz.viz_lib().setTitleText();

        if (force_redraw) {
          driver.drawVisual();
        }

        dsreq = data_request.dsreq();

        resolve();
        return driver;
      }

      if (data_request.isPrereqDataRequest()) {
        driver.fetchData({
          dsreq: data_request.dsreq(),

          successCb(results) {
            const previousResult = arc.data.result();
            previousResult.rows(results.rows);
            previousResult.columns(data_request.columns());
            previousResult.finalize();

            refreshAfterFetch({
              force_redraw,
              previousDataRequest: data_request,
              previousDataResult: previousResult,
            }, resolve, reject);
          },

          failCb: reject,
        });

        return driver;
      }

      sb_sql = data_request.sql();
      if (comments && sb_sql && sb_sql.indexOf('http') !== 0) {
        sb_sql = '/* ' + comments + ' */ ' + sb_sql;
      }

      dsreq = data_request.dsreq();
    }

    // Reload data if the new query is not the same as the old query
    driver.loadData(sb_sql, force_redraw, dsreq, resolve, reject);
  }

  function isTableVisual(vizType) {
    return (['table', 'rawtable'].indexOf(driver.viz_type()) >= 0);
  }

  driver.resultFromRows = function(rows, data_request_columns) {
    var sb_colnames = driver.sb_colnames(),
        sb_coltypes = driver.sb_coltypes();

    var result = arc.data.result()
                         .rows(rows);

    if (!data_request || !sb_colnames) {
      return undefined;
    }

    {
      if (data_request_columns === undefined) {
        data_request_columns = data_request.columns();
      }

      // Tables that do not have any information in shelves return all column
      // data
      if (isTableVisual(driver.viz_type()) &&
         (data_request_columns.length === 0 ||

          // If there is item on shelf, but visual hasn't been refreshed,
          // redraw full table
          sb_colnames.length !== data_request_columns.length)
         ) {

        var colsource = 'dimensions_shelf',
            colproperty = arc.report.columnTypeEnum.DIMENSION;

        // Get all the columns from the dataset
        let dscolslist = driver.dataset().tables().reduce((a, t) => a.concat(t.columns()), []);
        dscolslist = dscolslist.filter(function(col) {
          return !col.attr('hidden') && !col.attr('exprhasagg');
        });

        const dscols = {};
        dscolslist.forEach(col => dscols[col.attr().alias] = col);
        if (driver.isSolr()) {
          // For Solr, order in which queryResults columns are received is not same as
          // order in which dataset columns are received
          const sbColnamesMap = {};
          sb_colnames.forEach((colName, idx) => sbColnamesMap[colName] = idx);
          dscolslist.sort(function sortDSColNames(col1, col2) {
            const a = col1.display_name();
            const b = col2.display_name();
            if (typeof sbColnamesMap[a] !== 'undefined' &&
              typeof sbColnamesMap[b] !== 'undefined') {
              return sbColnamesMap[a] - sbColnamesMap[b];
            }

            if (typeof sbColnamesMap[a] !== 'undefined') {
              return -1;
            }

            if (typeof sbColnamesMap[b] !== 'undefined') {
              return 1;
            }

            return a.localeCompare(b);
          });
        }

        sb_colnames.forEach(function(colname, colindex) {
          const colCopy = arc.data.column()
            .colname(colname)
            .coltype(sb_coltypes[colindex])
            .colsource(colsource)
            .colproperty(colproperty)
            .order(undefined)
            .shelfIndex(colindex)
            .expr('[' + colname + ']');

          // Use column names from the dataset definition
          if (driver.aliasesAfterQuery()) {
            const column = dscolslist[colindex];
            const colAttr = column.attr();
            colCopy.colname(colAttr.alias);
            colCopy.colfiltername(colAttr.alias);
          }

          if (dscols.hasOwnProperty(colname)) {  // Found in dataset cols
            const column = dscols[colname];

            if (column.attr().hidden) {
              return;
            }

            const colAttr = column.attr();
            colCopy.colfiltername(colAttr.alias);
            if (colAttr.displayformat) {
              colCopy.setDisplayFormat(
                arc.utils.translateDisplayFormat(colAttr.displayformat, driver.dataset()));
            }
          } else {
           // Note that in this case, the filtername is the
           // same as colname
            colCopy.colfiltername(colname)
                   .setDisplayFormat(undefined);
          }

          result.add_column(colCopy);
        });
      } else {
        var newcol, itercol;

        sb_colnames.forEach(function(colname, colindex) {
          itercol = data_request_columns[colindex];
          if (itercol === undefined) {
            return;
          }

          newcol = arc.data.column()
                       .colname(colname)
                       .coldesc(itercol.coldesc())
                       .coltype(sb_coltypes[colindex])
                       .secondaryaxis(itercol.secondaryaxis())
                       .colfiltername(itercol.colfiltername())
                       .colsource(itercol.colsource())
                       .colproperty(itercol.colproperty())
                       .dimhlist(itercol.dimhlist())
                       .dimhlevel(itercol.dimhlevel())
                       .shelfIndex(itercol.shelfIndex())
                       .setDisplayFormat(itercol.coldisplayformat(true))
                       .colgeotype(itercol.colgeotype())
                       .order(itercol.order())
                       .expr(itercol.expr())
                       .custom_expr(itercol.custom_expr())
                       .dataTransformation(itercol.dataTransformation())
                       .coldisplaybefore(itercol.coldisplaybefore())
                       .fn_name(itercol.fn_name())
                       .fn_cast(itercol.fn_cast());

          result.add_column(newcol);
        });
      }
    }

    result.finalize();

    return result;
  };

  function setupExploreButtons() {
    if (!arc.globals.is_edit) {
      return;
    }

    if (!arc.globals.dashboard || arc.globals.isInModal()) {
      return;
    }

    if (!arc.globals.enable_explore_interfaces) {
      return;
    }

    const $explore = $(`#${driver.chart_div()}`)
                       .closest('.arc-viz-container')
                       .find('.explore-container');

    if ($explore.length === 0) {
      return;
    }

    $explore.off('click.explore')
            .on('click.explore', function() {
              $('#tooltip').fadeOut();
              arc.menu_hideAll();
            });

    const $dropdown = $explore.find('.explore-dropdown');
    $dropdown.empty();
    $dropdown.append('<li class="dropwdown-item explore-label">Explore...</li>');

    const exploreData = arc.showme.setupSettings({
      vizDriver: driver,
      setupClick: true,
    });

    const exploreOption = '<a class="explore dropdown-item" data-explore="%ID%">%NAME%</a>';

    exploreData.forEach((d) => {
      $dropdown.append(
        arc.misc.safe_substitute(
          exploreOption,
          {
            ID: d.id,
            NAME: d.name,
          }
        )
      );
    });

    $explore.removeClass('arc-invisible');
  }

  // always refresh
  driver.redraw = function(isWindowResize) {
    if (!driver.active()) {
      return driver;
    }

    setupExploreButtons();

    if (dsreq !== driver.dsreq()) {
      return driver.refresh();
    }

    var result = driver.resultFromRows(driver.sb_rows());
    if (result === undefined) {
      return driver;
    }

    result.finalize();

    if (result.rows().length > 0) {
      // Filter rows that have been excluded using the visual 'Exclude' button
      var dim_colindex = result.dim_start_idx();
      if (dim_colindex >= 0) {
        var sb_rows = result.rows();
        sb_rows = driver.filterLocally(sb_rows, dim_colindex);
        result.rows(sb_rows);
      }
    }

    // Note that we draw the visual even when there are no result rows.
    // Some visuals do not need result rows
    fetchCustomJsAndCallback(function() {
      if ($('#' + driver.chart_div()).length === 0) {
        // In appgroups, we may navigate away from a dashboard while
        // having some draw requests in the pipeline
        return;
      }

      driver.dataResult(result);
      driver.drawVisual(isWindowResize);
      driver.updateChartDivHeight();
    }, true);

    return driver;
  };

  return driver;
};

}());
