window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};
arc.report.events = arc.report.events || {};

// An 'DatasetEvent' object reflects the 'Event' model in the backend.
// 'DatasetEvent.deserialize' converts from the model object to this object.
// 'DatasetEvent.save' creates/updates this object
// 'DatasetEvent.delete' deletes this object
arc.report.events.DatasetEvent = function() {
  var id,
      event_name,
      dataset_id,
      filters,
      display;

  var DatasetEvent = function() { };

  DatasetEvent.clone = function() {
    var e = arc.report.events.DatasetEvent()
                             .id(id)
                             .event_name(`Copy of ${event_name}`)
                             .dataset_id(dataset_id)
                             .filters(filters.slice(0))
                             .display(display);
    return e;
  };

  DatasetEvent.filterExpr = function() {
    return "(" + filters.join(") AND (") + ")";
  };

  DatasetEvent.showModal = function(create_mode, nameHash, done_cb, dataset) {
    const $eventModal = $('#viz-event-create');
    if (arc.misc.modalShown($eventModal)) {
      return;
    }

    if (create_mode) {
      $("#viz-event-create-title").text("Add Event");
      $("#viz-event-save").text("Save");
    } else {
      $("#viz-event-create-title").text("Edit Event");
      $("#viz-event-save").text("Save");
    }

    $("#viz-event-create-name").val(event_name || "");
    $("#viz-event-create-display").val(display || "");
    const filterText =
      'Enter an expression that can serve as a WHERE clause. For example: event_type="login"';

    arc.report.addableExprList(
      'viz-event-filters',
      filters,
      $eventModal.find('#add-event-filter-btn'),
      dataset,
      {
        title: 'Filter Expression',
        helptext: filterText,
        exprType: 'filter',
        excludeRecordCountAutocomplete: true,
        includeTables: true,
      }
    );

    if (create_mode) {
      $eventModal.off('shown.bs.modal')
                 .on('shown.bs.modal', () => $('#viz-event-create-name').focus());
    }

    $eventModal.find('.alert').remove();
    $eventModal.modal('show');
    d3.select('#viz-event-save')
      .on('click.saveevent', null)
      .on('click.saveevent', function processEventData() {
        $eventModal.find('.alert').remove();
        const modal_filters = [];
        const errs = [];
        $('#viz-event-filters li input').each(function getFilterValue(idx, elem) {
          if (elem.value.trim() !== '') {
            modal_filters.push(elem.value);
          }
        });

        const eventName = $('#viz-event-create-name').val();
        const $modalBody = $eventModal.find('.modal-body');
        const eventWithName = nameHash[eventName];
        if (!eventName) {
          errs.push('Event must have a name.');
        } else if (eventWithName && eventWithName !== id) {
          errs.push(`Event named ${eventName} already exists.`);
        }

        const display = $('#viz-event-create-display').val();

        if (!display) {
          errs.push('Event display must be specified.');
        }

        if (!modal_filters.length) {
          errs.push('At least one filter must be specified.');
        }

        if (errs.length) {
          arc.misc.addErrors(errs, $modalBody);
          return;
        }

        DatasetEvent.event_name(eventName)
             .display($('#viz-event-create-display').val())
             .filters(modal_filters)
             .save(function () {
               done_cb(function () {
                 $eventModal.modal('hide');
                 if (create_mode) {
                   const $eventCounter = $('#dataset-events-count');
                   $eventCounter.text(Number($eventCounter.text()) + 1);
                 }
               });
             });
    });
  };

  DatasetEvent.id = function(_) {
    if (!arguments.length) return id;
    id = _;
    return DatasetEvent;
  };

  DatasetEvent.event_name = function(_) {
    if (!arguments.length) return event_name;
    event_name = _.trim();
    return DatasetEvent;
  };

  DatasetEvent.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return DatasetEvent;
  };

  DatasetEvent.filters = function(_) {
    if (!arguments.length) return filters;
    filters = _;
    return DatasetEvent;
  };

  DatasetEvent.display = function(_) {
    if (!arguments.length) return display;
    display = _.trim();
    return DatasetEvent;
  };

  DatasetEvent.isStringLiteral = function() {
    var d = display.trim();
    var first_char = d[0];
    var last_char = d[d.length - 1];
    if (((first_char === '"') && (last_char === '"')) ||
        ((first_char === "'") && (last_char === "'"))) {
      return true;
    }
    return false;
  };

  // Within the context of a M-R query, we need to export the symbol in one
  // of two ways. If the display is a column name, then the export happens as
  // A@column_name where 'A' is the symbol passed in, and 'column_name' is the
  // name of the column in the display variable.
  // If the display is a literal string, then we need to just return the
  // literal string, single-quoted.
  DatasetEvent.getComputedDisplay = function(symbol) {
    var d = display.trim();
    var first_char = d[0];
    var last_char = d[d.length - 1];
    if (((first_char === '"') && (last_char === '"')) ||
        ((first_char === "'") && (last_char === "'"))) {
      return "'" + d.slice(1, d.length - 1) + "'";
    }
    return symbol + "@" + d;
  };

  DatasetEvent.getComputedDisplayName = function() {
    return event_name;
  };

  DatasetEvent.delete = function(done_cb) {
    if (!done_cb) {
      done_cb = function() { };
    }

    const url = `${arc.frontend.globals.urls.eventUrl}/${id}`;
    $.ajax({
      url: url,
      type: 'DELETE',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .done(function updateEventCounter() {
      const $eventCounter = $('#dataset-events-count');
      $eventCounter.text(Number($eventCounter.text()) - 1);
    })
    .fail(function(req) {
      arc.report.alertMessage("Failed to delete event", req);
    })
    .always(function() {
      done_cb();
    });
  };

  DatasetEvent.save = function(done_cb) {
    if (!done_cb) {
      done_cb = function() { };
    }

    var event_data = JSON.stringify({
      'filters': filters,
      'display': display
    });

    var post_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
      'name': event_name.trim(),
      'data': event_data
    };

    if (id) {
      post_data.id = id;
    } else {
      // If creating a new event, set the dataset id
      if (!dataset_id) {
        arc.report.alertMessage("Event must have a dataset id");
        done_cb();
        return DatasetEvent;
      }
      post_data.dataset_id = dataset_id;
    }
    console.log(post_data);

    let url = arc.frontend.globals.urls.eventUrl;
    if (id) {
      url += "/" + id;
    }

    // save event
    $.ajax({
      url: url,
      data: post_data,

      // 'PUT' updates a event, 'POST' creates a new one
      type: id ? 'PUT' : 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .done(console.log)
    .fail(function(req) {
      arc.report.alertMessage("Failed to " +
          (id ? "update" : "create") + " event",
          req.responseText ? req.responseText : req);
    })
    .always(function() {
      done_cb();
    });

    return DatasetEvent;
  };

  DatasetEvent.deserialize = function(_) {
    var event_data = JSON.parse(_.data);
    DatasetEvent.id(_.id)
         .event_name(_.name)
         .dataset_id(_.dataset_id)
         .filters(event_data.filters)
         .display(event_data.display);
    return DatasetEvent;
  };

  return DatasetEvent;
};

arc.report.events.dataset_events = function() {
  var dom_id,
      dataset,
      events,
      nameHash,
      events_by_id;

  var dataset_events = function() { };

  dataset_events.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return dataset_events;
  };

  dataset_events.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return dataset_events;
  };

  dataset_events.events = function(_) {
    if (!arguments.length) return events;
    var in_events = _;
    events = [];
    events_by_id = {};
    nameHash = {};

    // console.log(in_events);
    in_events.forEach(function(e) {
      var ev = arc.report.events.DatasetEvent()
                                .deserialize(e);
      events.push(ev);
      events_by_id[ev.id()] = ev;
      nameHash[ev.event_name()] = ev.id();
    });
    return dataset_events;
  };

  dataset_events.getEventById = function(event_id) {
    return events_by_id[event_id];
  };

  dataset_events.getNameHash = function() {
    return nameHash;
  };

  // First time set up of the event create/edit modal and its callback functions
  dataset_events.setupDatasetDetail = function() {
    d3.select("#event-create")
      .on("click.createevent", function() {
        var ee = arc.report.events.DatasetEvent()
                                  .dataset_id(dataset.dataset_id());

        // first parameter is create_mode
        ee.showModal(true, dataset_events.getNameHash(),
          function(done_cb) {
            dataset_events.reloadDetail(done_cb);
          }, dataset);
        return false;
      });
    dataset_events.paintDetail(dataset);
    return dataset_events;
  };

  // Retrieve event information after a create/edit
  dataset_events.reloadDetail = function(done_cb) {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.eventUrl}?dataset_id=${dataset.dataset_id()}`)
      .await(function(error, events) {
        if (error) {
          arc.report.alertMessage("Failed to load events", error);
        } else {
          events.forEach(event => event.dataset = dataset);
          dataset_events.events(events)
                        .paintDetail(dataset);
        }
        done_cb();
      });

    return dataset_events;
  };

  // Draw the events subnav bar on the builder page, not needed
  // in display only mode (or in dashboards)
  dataset_events.paintBuilder = function() {
    var event_elements = [];
    events.forEach(function(d) {
      let se = arc.report.shelf.element();
      se.dataset(dataset)
        .event_element(d);
      event_elements.push(se);
    });

    d3.selectAll(".events-avail-list li")
      .remove();

    var dim_lis =
      d3.selectAll(".events-avail-list")
        .append("div")
        .classed("field-list-wrapper", true)
        .selectAll("#nosel")
        .data(event_elements)
        .enter()
          .append("li")
          .classed("selection_element", true)
          .append("span")
          .classed("field-wrapper", true);

    dim_lis.append("span")
           .classed("selection_field", true)
           .text(function(d) {
             return d.getComputedDisplayName();
           });

    dim_lis.append("span")
           .classed("dropdown-placeholder", true);

    $('.app-events-count').text(events.length);
    return dataset_events;
  };

  // Display retrieved events in the dataset detail screen
  dataset_events.paintDetail = function(dataset) {
    d3.select("#events-table")
      .select("tbody")
      .selectAll("tr")
      .remove();

    var num_commas = d3.format(",");
    const rows = d3.select("#events-table")
                   .select("tbody")
                   .selectAll("tr")
                   .data(events)
                   .enter()
                     .append("tr");

    rows.append("td")
        .text(function(d) {
          return d.event_name();
        });
    rows.append("td")
        .text(function(d) {
          return d.display();
        });
    rows.append("td")
        .append("div")
		.classed("data-filter-wrapper", true)
        .text(function(d) {
          return d.filters();
        });

    var event_action_content = '<div class="action-button-wrapper">' +
        '<a class="dataset-event-edit"><span class="fa fa-pencil"></span> ' +
        'Edit</a> <a class="dataset-event-clone"><span class="fa fa-copy"></span> ' +
        'Clone</a> <a  class="dataset-event-delete"><span class="fa fa-trash-o"></span> ' +
        'Delete</a></div>';

    var actions = rows.append("td");
    if (arc.misc.hasPerm('ds_manage', dataset.dataset_perms())) {
      actions.html(event_action_content);
    }

    d3.selectAll(".dataset-event-edit")
      .on("click.editevent", function() {
        const d = this.parentNode.parentNode.__data__;

        // first parameter is create_mode
        d.showModal(false, dataset_events.getNameHash(),
          function(done_cb) {
            dataset_events.reloadDetail(done_cb);
          }, dataset);
        return false;
      });

    d3.selectAll(".dataset-event-clone")
      .on("click.cloneevent", function() {
        const d = this.parentNode.parentNode.__data__;
        d.clone()
         .id(null)

         // first parameter is create_mode
         .showModal(true, dataset_events.getNameHash(),
          function(done_cb) {
            dataset_events.reloadDetail(done_cb);
          }, dataset);
        return false;
      });

    d3.selectAll(".dataset-event-delete")
      .on("click.deleteevent", function() {
        const $modal = $("#delete-event-modal");
        if (arc.misc.modalShown($modal)) {
          return;
        }

        // Show the delete confirmation modal,
        // and delete when confirmed
        const d = this.parentNode.parentNode.__data__;
        d3.select('#delete-event-yes')
          .on("click.deleteapp", function() {
            d.delete(function() {
              dataset_events.reloadDetail(function() {
                $("#delete-event-modal").modal('hide');
              });
            });
          });

        $modal.modal('show');
      });

    $("#events-table").trigger('update');

/** removing popover
    // popover for event info
    $("#events-table tbody tr").popover({
      container: 'body',  // makes popover be on top
      html : true,
      trigger: 'hover',
      placement: 'bottom',
      content: function() {
        var popover = true;
        d3.select(this).each(function(d) {
          if (!d) {
            popover = false;
            return;
          }
          d3.select("#event-popover-header").text(d.event_name() || "");
          d3.select("#event-popover-display").text(d.display() || "");
          if (d.filters()) {
            d3.select("#event-popover-filters").html(d.filters().join("<br>"));
          } else {
            d3.select("#event-popover-filters").text("");
          }
        });
        if (!popover) {
          return '';
        }
        return $('#event-detail-popover').html();
      }
    });
	**/
  };

  return dataset_events;
};
}());
