window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};
arc.report.filterassociations = arc.report.filterassociations || {};

// A 'FilterAssociation' object reflects the 'FilterAssociation' model in the
// backend.
// 'FilterAssociation.deserialize' converts from model object to this object
// 'FilterAssociation.save' creates/updates this object
// 'FilterAssociation.delete' deletes this object
arc.report.filterassociations.FilterAssociation = function() {

  var id,
      filterassociation_name,
      dataset_id,
      applied_segments,
      all_dataset_segments,
      users,
      groups;

  var FilterAssociation = function() {};

  FilterAssociation.id = function(_) {
    if (!arguments.length) return id;
    id = _;
    return FilterAssociation;
  };

  FilterAssociation.filterassociation_name = function(_) {
    if (!arguments.length) return filterassociation_name;
    filterassociation_name = _.trim();
    return FilterAssociation;
  };

  FilterAssociation.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return FilterAssociation;
  };

  FilterAssociation.applied_segments = function(_) {
    if (!arguments.length) return applied_segments;
    applied_segments = _;
    return FilterAssociation;
  };

  FilterAssociation.all_dataset_segments = function(_) {
    if (!arguments.length) return all_dataset_segments;
    all_dataset_segments = _;
    return FilterAssociation;
  };

  FilterAssociation.users = function(_) {
    if (!arguments.length) return users;
    users = _;
    return FilterAssociation;
  };

  FilterAssociation.groups = function(_) {
    if (!arguments.length) return groups;
    groups = _;
    return FilterAssociation;
  };

  FilterAssociation.deserialize = function(_) {
    var filterassociation_data = JSON.parse(_.data);
    FilterAssociation.id(_.id)
         .filterassociation_name(_.name)
         .dataset_id(_.dataset_id)
         .users(_.users)
         .groups(_.groups)
         .applied_segments(filterassociation_data.applied_segments);
    return FilterAssociation;
  };

  FilterAssociation.showModal = function(create_mode, userSelectionWidget,
      groupSelectionWidget, faNameHash, done_cb) {
    const $faModal = $('#create-filterassociation-modal');
    if (arc.misc.modalShown($faModal)) {
      return;
    }

    if (create_mode) {
      $("#filterassociation-create-title").text("Add Filter Association");
      $("#filterassociation-save").text("Create");
      userSelectionWidget.selectionField('setPreviousSelections', []);
      groupSelectionWidget.selectionField('setPreviousSelections', []);
    } else {
      $("#filterassociation-create-title").text("Edit Filter Association");
      $("#filterassociation-save").text("Update");
      userSelectionWidget.selectionField('setPreviousSelections', users);
      groupSelectionWidget.selectionField('setPreviousSelections', groups);
    }
    $("#filterassociation-create-name").val(filterassociation_name || "");

    all_dataset_segments.paint();

    if (!create_mode) {
      // here, appliedsegments already has selected data, we need to paint it
      // the following does that by first clearing up everything and next by
      // painting only selected ones
      all_dataset_segments.selected_segments(applied_segments);
    }

    $faModal.modal('show');

    $faModal.off('hide.bs.modal')
               .on('hide.bs.modal', function resetAlerts() {
                  $faModal.find('.alert').remove();
                });

    $("#fa-loading-spinner").hide();

    d3.select("#filterassociation-save").on("click.savefilterassociation", function() {
      const errs = [];
      $faModal.find('.alert').remove();

      users = userSelectionWidget.selectionField('selected');
      groups = groupSelectionWidget.selectionField('selected');

      const $modalBody = $faModal.find('.modal-body');
      const faName = $('#filterassociation-create-name').val();
      const faWithName = faNameHash[faName];

      if (!faName) {
        errs.push('Filter association must have a name.');
      } else if (faWithName && faWithName !== id) {
        errs.push(`Filter association named ${faName} already exists.`);
      }
      const $segmentChecks = $('#filterassociation-modal-datasets-segments-container li input');
      const checked = $.makeArray($segmentChecks).slice(1).filter(i => i.checked);
      if (checked.length !== 1) {
        errs.push('Filter association requires exactly one selected segment');
      }

      if (errs.length) {
        arc.misc.addErrors(errs, $modalBody);
        return;
      }

      $("#fa-loading-spinner").show();

      function hideModal() {
        done_cb(function() {
          $("#dataset-loading-spinner").hide();
          $faModal.modal('hide');
        });
      }

      FilterAssociation.filterassociation_name(faName)
           .all_dataset_segments(all_dataset_segments)
           .save(hideModal, create_mode ? 'POST' : 'PUT');
    });
  };

  FilterAssociation.save_backend = function(save_backend_cb, request_type) {
    var filterassociation_data = JSON.stringify({
      'applied_segments': applied_segments
    });

    var post_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
      "name": filterassociation_name,
      "data": filterassociation_data,
      "dataset_id": dataset_id,
      "users": JSON.stringify(users),
      "groups": JSON.stringify(groups),
    };

    if (id) {
      post_data.id = id;
    }

    let url = arc.frontend.globals.urls.filterassociationUrl;
    if (id) {
      url += "/" + id;
    }
    $.ajax({
      url: url,
      data: post_data,

      // 'PUT' updates a event, 'POST' creates a new one
      type: request_type ? request_type : (id ? 'PUT' : 'POST'),
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .done(function(req) {
      const $associationCounter = $('#dataset-filterassociations-count');
      const associationCounterNum = Number($associationCounter.text());
      if (request_type === 'DELETE') {
        $associationCounter.text(associationCounterNum - 1);
      } else if (request_type === 'POST') {
        $associationCounter.text(associationCounterNum + 1);
      }

      // TODO: Remove this console log... possible?
      console.log(req);
    })
    .fail(function(req) {
      arc.report.alertMessage("Failed to " +
          (request_type ? request_type : (id ? "update" : "create")) + " event",
          req.responseText ? req.responseText : req);
    })
    .always(function() {
      save_backend_cb();
    });
    return FilterAssociation;
  };

  FilterAssociation.delete = function(delete_done_cb) {
    FilterAssociation.save_backend(delete_done_cb, 'DELETE');
  };

  FilterAssociation.save = function(save_done_cb, request_type) {

    // This is not great but need to ensure selected_segments() picks up the
    // currently selected segments by setting the global is_edit flag.
    arc.globals.is_edit = true;

    var ss = all_dataset_segments.selected_segments();
    var sss = [];
    ss.forEach(function(s) {
      sss.push({
        'id':s.id,
        'name':s.data.name,
        'negate':s.negate
      });
    });

    FilterAssociation.applied_segments(sss);
    FilterAssociation.save_backend(save_done_cb, request_type);
    return FilterAssociation;
  };

  return FilterAssociation;
};

// Handling Filter Associations
// dataset_filterassociations is an object containing a collection of
// FilterAssociation objects
arc.report.filterassociations.dataset_filterassociations = function() {
  var dataset_id,
      dom_id,
      d3_dom_id,
      dataset_perms,
      fa_dataset_segments, // all segments of this dataset
      filterassociations,
      filterassociations_by_id,
      faNameHash,
      users,
      groups,
      userSelectionWidget,
      groupSelectionWidget;

  var dataset_filterassociations = function() {};

  dataset_filterassociations.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return dataset_filterassociations;
  };

  dataset_filterassociations.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return dataset_filterassociations;
  };

  dataset_filterassociations.dataset_perms = function(_) {
    if (!arguments.length) return dataset_perms;
    dataset_perms = _;
    return dataset_filterassociations;
  };

  dataset_filterassociations.fa_dataset_segments = function(_) {
    if (!arguments.length) return fa_dataset_segments;
    fa_dataset_segments = _;
    return dataset_filterassociations;
  };

  dataset_filterassociations.getNameHash = function() {
    return faNameHash;
  };

  // Parse retrieved input filter associations from backend into
  // Filter Association objects
  dataset_filterassociations.filterassociations = function(_) {
    if (!arguments.length) return filterassociations;
    var in_filterassociations = _;
    filterassociations = [];
    filterassociations_by_id = {};
    faNameHash = {};
    in_filterassociations.forEach(function(ifa) {
      var fa = arc.report.filterassociations.FilterAssociation()
                                .all_dataset_segments(fa_dataset_segments)
                                .deserialize(ifa);
      filterassociations.push(fa);
      filterassociations_by_id[fa.id()] = fa;
      faNameHash[fa.filterassociation_name()] = fa.id();
    });
    return dataset_filterassociations;
  };

  dataset_filterassociations.getFilterAssociationById = function(filterassociation_id) {
    return filterassociations_by_id[filterassociation_id];
  };

  // First time set up of the filterassociation create/edit modal and its
  // callback functions
  dataset_filterassociations.setupDatasetDetail = function() {
    //Fetch the full user and group list here. Put here because we need this info
    //on the filter associations admin page, but not everytime we build a report.
    //Thus, don't add the requests to the reportbuilder queue. This fetch and draw
    //of the filter association page is done last since it is not an often used feature.

    dataset_filterassociations.users = {};
    dataset_filterassociations.groups = {};
    d3_dom_id = d3.select(dom_id);

    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.usersDetailedApiUrl}?summary=true`)
      .defer(d3.json, arc.frontend.globals.urls.groupsUrl)
      .defer(d3.json, `${arc.frontend.globals.urls.filterassociationUrl}?dataset_id=${dataset_id}`)
      .await(function(error, users, groups, falist) {
        if (error) {
          arc.report.alertMessage("Failed to load user and/or group data",
              error.response ? error.response : error);
          return;
        }
        var userObj = dataset_filterassociations.users;
        userObj.userlookup = {};
        userObj.userlist = users.map(function(d) {
          userObj.userlookup[d.id] = d.username;
          return {id: d.id, name:d.username};
        });

        var groupObj = dataset_filterassociations.groups;
        groupObj.grouplookup = {};
        groupObj.grouplist = groups.map(function(d) {
          groupObj.grouplookup[d.id] = d.name;
          return {id: d.id, name:d.name};
        });

        dataset_filterassociations.filterassociations(falist);

        //now, paint and setup the page after registering users and groups
        d3_dom_id.select("#filterassociation-create")
          .on("click.createfilterassociation", function() {
            var fa = arc.report.filterassociations.FilterAssociation()
                                      .all_dataset_segments(fa_dataset_segments)
                                      .dataset_id(dataset_id);

            // first parameter is create_mode
            fa.showModal(true,
              userSelectionWidget,
              groupSelectionWidget,
              dataset_filterassociations.getNameHash(),
              function(done_cb) {
                dataset_filterassociations.reloadDetail(done_cb);
              });
            return false;
          });

          userSelectionWidget =
            $(".first-tab-selection").selectionField({
               selectionList: dataset_filterassociations.users.userlist,
            });

          groupSelectionWidget =
            $(".second-tab-selection").selectionField({
               selectionList: dataset_filterassociations.groups.grouplist,
            });

        dataset_filterassociations.paintDetail();

        $('#dataset-filterassociations-sidebar').removeClass('arc-invisible');

      });
    return dataset_filterassociations;
  };

  // Retrieve filterassociation information after a create/edit
  dataset_filterassociations.reloadDetail = function(done_cb) {
    d4.queue()
      .defer(d3.json, `${arc.frontend.globals.urls.filterassociationUrl}?dataset_id=${dataset_id}`)
      .await(function(error, filterassociations) {
        if (error) {
          arc.report.alertMessage("Failed to load filterassociations", error);
        } else {
          dataset_filterassociations.filterassociations(filterassociations)
                        .paintDetail();
        }
        if (done_cb) {
          done_cb();
        }
      });

    return dataset_filterassociations;
  };

  // Display retrieved filterassociations in the dataset detail screen
  dataset_filterassociations.paintDetail = function() {

    $('#dataset-filterassociations-count').text(filterassociations.length);

    d3_dom_id.select("#filterassociations-table")
      .select("tbody")
      .selectAll("tr")
      .remove();

    var rows = d3_dom_id.select("#filterassociations-table")
                        .select("tbody")
                        .selectAll("tr")
                        .data(filterassociations)
                        .enter()
                          .append("tr");

    rows.append("td")
        .text(function(d) {
          return d.filterassociation_name();
        });
    rows.append("td")
        .text(function(d) {
          var das = d.applied_segments();
          var strarr = [];
          das.forEach(function(y) {
              if (y.negate) {
                  strarr.push("NOT " + y.name);
              } else {
                  strarr.push(y.name);
              }
          });

          return (strarr.join(", "));
        });
    rows.append("td")
        .text(function(d) {
          return d.users().map(function(d) {
            return dataset_filterassociations.users.userlookup[d];
          }).join(', ');
        });

    rows.append("td")
        .text(function(d) {
          return d.groups().map(function(d) {
            return dataset_filterassociations.groups.grouplookup[d];
          }).join(', ');
        });

    var filterassociation_action_content = '<div class="action-button-wrapper">' +
      '<a class="dataset-filterassociation-edit"> ' +
      '<span class="fa fa-pencil"></span> Edit</a> <a  class="dataset-filterassociation-delete"> ' +
      '<span class="fa fa-trash-o"></span> Delete</a></div>';

    var actions = rows.append("td");
    if (arc.misc.hasPerm('ds_manage', dataset_perms)) {
      actions.html(filterassociation_action_content);
    }

    d3_dom_id.selectAll(".dataset-filterassociation-edit")
      .on("click.editfilterassociation", function() {
        const d = this.parentNode.parentNode.__data__;

        // first parameter of showModal is create_mode
        d.showModal(false,
          userSelectionWidget,
          groupSelectionWidget,
          dataset_filterassociations.getNameHash(),
          function(done_cb) {
            dataset_filterassociations.reloadDetail(done_cb);
          });
        return false;
      });

    d3_dom_id.selectAll(".dataset-filterassociation-delete")
      .on("click.deletefilterassociation", function() {
        const $modal = $("#delete-filterassociation-modal");

        if (arc.misc.modalShown($modal)) {
          return;
        }

        // Show the delete confirmation modal, and delete when confirmed
        const d = this.parentNode.parentNode.__data__;
        d3.select('#delete-filterassociation-yes')
          .on("click.deletefilterassociation", function() {
            d.delete(function() {
              dataset_filterassociations.reloadDetail(function() {
                $("#delete-filterassociation-modal").modal('hide');
              });
            });
          });

        $modal.modal('show');
      });

    $("#filterassociations-table").trigger('update');
  };

  return dataset_filterassociations;
};
}());
