window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};

arc.report.filtersaver = function() {
  let dataset_segments;

  const filtersaver = function() { };

  filtersaver.save = function(create_mode, segment) {
    let sd;
    const predicate = arc.grammar._predicate;

    if (segment) {
      sd = dataset_segments.segmentDataToModalData(segment);
    } else {
      sd = {};
      sd.name = '';
      sd.dataset_id = dataset_segments.dataset().dataset_id();
      sd.group = '';
      sd.filters = [];
      sd.entities = [];
    }

    d3.selectAll('.app-shelves #filters_shelf .selection_field')
      .each((d) => {
        sd.filters.push(d.getExpression(undefined, true));
      });

    const predicates = {};
    const filters = [];
    sd.filters.forEach((expr) => {
      try {
        const e = predicate.parse(expr);
        const pred = predicates[[e.filter_column(), e.filter_exclude()]];
        if (pred) {
          e.filter_data().forEach((d) => {
            pred.add_filter_data(d);
          });
        } else {
          predicates[[e.filter_column(), e.filter_exclude()]] = e;
          filters.push(e);
        }
      } catch (err) {
        filters.push(arc.report.shelf.element().custom_expr(expr));
      }
    });

    sd.filters = [];
    filters.forEach((d) => {
      sd.filters.push(d.getExpression(undefined, true));
    });

    // DJC -- here if the 'cancel' is hit, the in memory segment is messed up as
    //        it still contains the added element.
    dataset_segments.segmentModalAndRefresh(sd, create_mode);
  };

  filtersaver.populateSegments = function() {
    const $list = $('#filters-addtosegment-list');

    if (!dataset_segments?.segments().length) {
      $list.hide();
      return;
    }

    $list.show();

    const groupedSegments = dataset_segments.groupedSegments();

    const container = d3.select('#filters-addtosegment-list');
    container.selectAll('.existing-segments.dropdown-menu').remove();

    const existingSegmentsMenu = container
      .append('div')
      .classed('existing-segments dropdown-menu', true);

    groupedSegments.forEach((group) => {
      existingSegmentsMenu.append('p')
        .classed('filters-groupnames', true)
        .text(() => group[0].data.group);

      existingSegmentsMenu.selectAll('#nosel')
        .data(group)
        .enter()
          .append('a')
          .classed('dropdown-item filters-save-link', true)
          .attr('data-segment-id', ({ id }) => id)
          .text(({ data }) => data.name)
          .on('click.add', (d) => {
            filtersaver.save(false, d);
          });
    });
  };

  filtersaver.activate = function() {
    // Initialize the filters dropdown
    $('.viz-filters-save').tooltip();
    $('#filters-save-dropdown').off('click.dropdown').on('click.dropdown', () => {
      filtersaver.populateSegments();
      $('.viz-filters-save').tooltip('hide');
    });
    d3.select('#filters-create-segment')
      .on('click.create', () => {
        filtersaver.save(true);
      });
    return filtersaver;
  };

  filtersaver.dataset_segments = function(_) {
    if (!arguments.length) return dataset_segments;
    dataset_segments = _;
    return filtersaver;
  };

  return filtersaver;
  };

  arc.report.enableToSaveFilterAsSegment = function () {
    $('span').has('#filters-save-dropdown').removeClass('disabled-feature');
  };

  arc.report.disableSaveFilterAsSegment = function () {
    $('span').has('#filters-save-dropdown').addClass('disabled-feature');
  };

  arc.report.updateFilterShelfLabel = function (list) {
    const emptyList = list?.length === 0;
    if (emptyList) {
      arc.report.disableSaveFilterAsSegment();
    } else {
      arc.report.enableToSaveFilterAsSegment();
    }
  };
}());
