window.arc = window.arc || {};

(function() {
"use strict";

arc.report = arc.report || {};

arc.report.picker = function() {
  var d3_elem,
      shelf_element,
      dataset,
      dataset_segments;

  var picker = function() { };

  picker.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return picker;
  };

  picker.d3_elem = function(_) {
    if (!arguments.length) return d3_elem;
    d3_elem = _;
    return picker;
  };

  picker.shelf_element = function(_) {
    if (!arguments.length) return shelf_element;
    shelf_element = _;
    return picker;
  };

  var picker_coltype;

  picker.dataset_segments = function(_) {
    if (!arguments.length) return dataset_segments;
    dataset_segments = _;
    return picker;
  };

  picker.activate_segment_picker = function() {
    const $modal = $("#filter-pick-modal");
    if (arc.misc.modalShown($modal)) {
      return;
    }

    var colname = arc.report.segment_name;

    d3.select("#filter-pick-values")
      .select("tbody")
      .selectAll("tr")
      .remove();

    // TODO: Figure out if this is duplicate of same event listener below
    d3.select("#filter-pick-save")
      .on("click.savefilter", function() {
        arc.report.pagenav.needs_save(true);
        var excluded = $("#filter-pick-exclude").is(":checked");
        shelf_element.filter_column(colname)
                     .filter_exclude(0)
                     .filter_data([]);
        d3.select("#filter-pick-modal")
          .selectAll("td input:checked")
          .each(function(d) {
            shelf_element.add_filter_data({
              'id': d.id,
              'negate': d.negate
            });
          });

        d3_elem.call(arc.report.shelf.settext);
      });

    $("#filter-pick-colname").text('Segment Names');
    var grouped_segments = dataset_segments.groupedSegments();
    grouped_segments.forEach(function(group) {
      var gsel =
        d3.select("#filter-pick-values")
          .select("tbody")
            .append("tr")
              .append("td")
                .append("div");
      gsel.append("span")
          .style("font-weight", "bold")
          .text(function() {
            return group[0].data.group;
          });

        var segments_x2 = [];
        group.forEach(function(s) {
          segments_x2.push({
            'id': s.id,
            'data': s.data,
            'negate': false
          });
          segments_x2.push({
            'id': s.id,
            'data': s.data,
            'negate': true
          });
        });
      var divs =
        gsel.selectAll("#nosel")
            .data(segments_x2)
            .enter()
            .append("div")
            .classed("checkbox", true)
            .append("label");
      divs.append("input")
          .attr("type", "checkbox")
          .filter(function(d) {
            var found = false;
            shelf_element.filter_data().forEach(function(s) {
              if ((s.id === d.id) && (s.negate === d.negate)) {
                found = true;
              }
            });
            return found;
          })
          .property('checked', true);
      divs.append("span")
          .classed("filter-value-checkbox", true)
          .text(function(d) {
            if (d.negate) {
              return '(not) ' + d.data.name;
            } else {
              return d.data.name;
            }
          });
    });

    $("#filter-contents").show();
    $("#filter-loading-spinner").hide();
    $modal.modal('show');

    return picker;
  };

  picker.activate = function() {
    const $modal = $("#filter-pick-modal");
    if (arc.misc.modalShown($modal)) {
      return;
    }

    var colname = shelf_element.filter_column();
    if (!colname) {
      colname = shelf_element.dataset_colname();
    }

    if (!colname) {
      arc.report.alertMessage("Cannot find field name to filter on");
      return;
    }

    if (shelf_element.isDatasetSegment()) {
      return picker.activate_segment_picker();
    }

    $("#filter-pick-colname").text(colname);
    if (shelf_element.filter_exclude()) {
      $("#filter-pick-exclude").prop('checked', true);
    } else {
      $("#filter-pick-exclude").prop('checked', false);
    }

    d3.select("#filter-pick-values")
      .select("tbody")
      .selectAll("tr")
      .remove();
    d3.select("#filter-pick-save")
      .on("click.savefilter", function() {
        arc.report.pagenav.needs_save(true);
        var excluded = $("#filter-pick-exclude").is(":checked");
        shelf_element.filter_column(colname)
                     .filter_exclude(excluded  ? 1 : 0)
                     .filter_data([]);
        d3.select("#filter-pick-modal")
          .selectAll("td input:checked")
          .each(function(d) {
            var v = arc.report.shelf.parse_filter_value(d[0], picker_coltype);
            shelf_element.add_filter_data(v);
          });

        d3_elem.call(arc.report.shelf.settext);
      });

    $("#filter-loading-spinner").show();
    $modal.modal('show');
    $("#filter-contents").hide();

    // DJC -- more tablename issues -- probably want to find all selectBuilder() usages
    var s = arc.selectBuilder()
               .dataset(dataset)
               .addOrderBy('1')
               .distinct(true)
               .limit(1000);

    var expr = '[' + colname + ']';
    if (shelf_element.custom_expr()) {
      expr = shelf_element.getExpression(undefined, true);
    }

    var rb = arc.requestBuilder();
    rb.datasetId(dataset.dataset_id());
    rb.addAggregate(expr);
    rb.limit(1000);
    rb.distinctON();

    expr = s.addTablenamesFromExpression(expr);
    s.addProjection(expr);

    var cmd = s.getSql();

    arc.report.sql_runner.run(dataset.dataconnection_id(), cmd,
        function(errorMsg, data) {
      $("#filter-loading-spinner").hide();
      $("#filter-contents").show();
      if (errorMsg) {
        return;
      }
      picker_coltype = data.coltypes[0];
      var filterData = shelf_element.filter_data();
      var rows =
        d3.select("#filter-pick-values")
          .select("tbody")
          .selectAll("tr")
          .data(data.rows)
          .enter()
            .append("tr")
            .append("td");
      var labels =
        rows.append("label")
            .classed("picker-input-label", true);

      labels.append("input")
            .attr("type", "checkbox")
            .filter(function(d) {
              var singleQuotedName = d[0],
                  doubleQuotedName = d[0],
                  doubleQuotePresent;
              if (picker_coltype.toLowerCase() === "string") {
                singleQuotedName = "'" + d[0].replace(/'/g, "\\'") + "'";
                doubleQuotedName = '"' + d[0].replace(/"/g, '\\"') + '"';
                doubleQuotePresent = inArray(filterData, doubleQuotedName);
              }
              return inArray(filterData, singleQuotedName) ||
                     doubleQuotePresent;
            })
            .property('checked', true);
      labels.append("span")
          .classed("filter-value-checkbox", true)
          .text(function(d) {
            return d;
          });

      function inArray(arrayOfItems, item) {
        return arrayOfItems.indexOf(item) >= 0;
      }
    }, true, rb.dsreq());
  };

  return picker;
};

}());
