window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};

function updateSegmentsCount() {
  const $segmentCounter = $('#dataset-segments-count');
  $segmentCounter.text($('#segments-table tbody tr').length);
}

arc.report.segment_isnot_link = function(lnk, dontSave) {
  // check to see if associated checkbox is checked then alert to save page
  var cb_checked = $(lnk).parent().find("[type='checkbox']").is(':checked');
  if (cb_checked && !dontSave) {
    arc.report.pagenav.needs_save(true);
  }

  // toggle 'is' and 'not'
  const newText = lnk.getAttribute('segment-isnot-attr') === 'not' ? 'is' : 'not';
  $(lnk).text(newText).attr('segment-isnot-attr', newText).fadeIn();
};

arc.report.filters_array_to_string = function(filters, negate) {
  var filter_condition;
  if (filters.length === 0) {
    return '';
  }

  if (negate) {
    // Handle the selection of rows NOT in this segment
    filter_condition = ['NOT ('];
  } else {
    filter_condition = ['('];
  }

  filters.forEach(function(filter, idx) {
    if (idx > 0) {
      filter_condition.push('and');
    }

    filter_condition.push(filter);
  });

  filter_condition.push(')');
  return filter_condition.join(' ');
};

// Track a selected group of segments, and apply conditions to the
// provided selectbuilder in the 'applyGroupToSelectBuilder' function
arc.report.segment_group = function() {
  var groupdata;

  var segment_group = function() { };

  segment_group.groupdata = function(_) {
    if (!arguments.length) return groupdata;
    groupdata = _;
    return segment_group;
  };

  // DJC -- More tablename/dataset issue
  // Apply a single group of segments to a select statement
  // Note that individual selections within a group are applied
  // using the OR predicate i.e. if you have a segment group
  // called 'User Location' and it has two selected segments -
  // one for 'US' and another for 'Canada' - then selecting both
  // will lead to users matching either predicate being selected.
  segment_group.applyGroupToSelectBuilder = function(sb, prefix) {
    var group_filters = [];
    var has_entities = (groupdata[0].data.entities.length > 0);
    var negate_segment = groupdata[0].negate || false;
    var dbname = sb.dbname();
    var tablename = sb.tablename();
    var dataset = sb.dataset();

    prefix = prefix || '_arc_t';

    var with_sb,
        with_tablename,
        join_condition,
        ename;
    if (has_entities) {
      with_sb = arc.selectBuilder();
      with_sb.baseTable(dbname, tablename);
      with_sb.dataset(dataset);
      with_tablename = prefix + groupdata[0].id;
      join_condition = [];
      groupdata[0].data.entities.forEach(function(entity, idx) {
        ename = with_tablename + '_c' + idx;

        // Ensure the tables containing the entities are present in both the outer
        // select and in select that produces them.
        sb.addTablenamesFromExpression(entity);
        entity = with_sb.addTablenamesFromExpression(entity);
        with_sb.addProjection(entity + ' as ' + ename);
        with_sb.addGroupWithoutProjection(entity);
        if (idx > 0) {
          join_condition.push('and');
        }

        join_condition.push('(');
        join_condition.push(entity + ' = ' + ename);
        join_condition.push(')');
      });

      if (negate_segment) {
        groupdata[0].data.entities.forEach(function(entity, idx) {
          ename = with_tablename + '_c' + idx;
          sb.addFilter(ename + ' is NULL');
        });
      }
    }

    groupdata.forEach(function(data) {
      var entities = data.data.entities;
      var filters = data.data.filters;
      var filter_condition;
      var data_negate_segment = data.negate || false;
      if (negate_segment !== data_negate_segment) {
        console.log('Error: Cannot mix negated and normal segments in the ' +
            'same group! Ignoring segment ', data.data.name);
      }

      if (entities.length === 0) {
        if (has_entities) {
          console.log('Error: Cannot mix segments with entities and without ' +
              'entities in the same group! Ignoring segment ', data.data.name);
          return;
        }

        // No entities, this is a simple row filter segment
        if (filters.length > 0) {
          group_filters.push(arc.report.filters_array_to_string(
              filters, negate_segment));
        }

        return;
      }

      if (!has_entities) {
        console.log('Error: Cannot mix segments with entities and without ' +
            'entities in the same group! Ignoring segment ', data);
        return;
      }

      filter_condition = ['('];
      filters.forEach(function(filter, idx) {
        if (idx > 0) {
          filter_condition.push('and');
        }

        filter_condition.push(filter);
      });

      filter_condition.push(')');
      group_filters.push(filter_condition.join(' '));
    });

    if (has_entities) {
      if (group_filters.length > 0) {
        var withexpr = group_filters.join(' OR ');
        withexpr = with_sb.addTablenamesFromExpression(withexpr);
        with_sb.addFilter(withexpr);
      }

      sb.addJoin(' (' + with_sb.getSql() + ') ' + with_tablename, join_condition.join(' '),
          negate_segment ? 'left outer join' : 'join');
    } else {
      if (group_filters.length > 0) {
        var exprtext = group_filters.join(' OR ');
        exprtext = sb.addTablenamesFromExpression(exprtext);
        sb.addFilter(exprtext);
      }
    }

    return segment_group;
  };

  return segment_group;
};

// Unique id maintained within arc.report.dataset_segments()
// to track the current on-screen segments that the user can modify
let datasetSegmentOnscreenId = 1;

// In the app builder screen, we display a list of segments that can be
// picked by the user. The object below reflects that ui element. It has
// a function to display the element ('paint'), return selected segments
// ('selected_segments'), and apply segments to the dataset
// ('applyToSelectBuilder')
arc.report.dataset_segments = function() {
  var segments_dom_id = 'apps-segments-container',
      dataset,
      segments = [],
      selectedSegments = [],
      onscreenId;

  var dataset_segments = function() { };

  dataset_segments.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return dataset_segments;
  };

  dataset_segments.forced_segs = function(_) {
    if (!arguments.length) return forced_segs;
    forced_segs = _;
    return dataset_segments;
  };

  dataset_segments.segments_dom_id = function(_) {
    if (!arguments.length) return segments_dom_id;
    segments_dom_id = _;
    return dataset_segments;
  };

  dataset_segments.setupDatasetDetail = function(setup_filterassociation_segments) {
    d3.select('#segment-create')
      .on('click.createsegment', () => {
        const in_sd = {};
        dataset_segments.segmentModalAndRefresh(in_sd, true);
        return false;
      });

    dataset_segments.paintSegmentsDetail();
    dataset_segments.paint();
    return dataset_segments;
  };

  dataset_segments.segments_from_id = function(filter_data) {
    var ss = [];
    filter_data.forEach(function(f) {
      segments.forEach(function(s) {
        if (s.id !== f.id) {
          return;
        }

        ss.push({
          id: s.id,
          data: s.data,
          negate: f.negate,
        });
      });
    });

    return ss;
  };

  dataset_segments.selected_segments = function(_) {
    if (!arguments.length) {
      // If this set of segments is not on screen right now,
      // then return the saved data.
      if (!arc.globals.is_edit) {
        return selectedSegments;
      }

      if (onscreenId !== datasetSegmentOnscreenId) {
        return selectedSegments;
      }

      // Otherwise, scrape the data on the screen right now
      var s = [];
      d3.selectAll(`#${segments_dom_id} input:checked:not(.all)`)
        .each(function(d) {
          if ($(this).parent().children('a').attr('segment-isnot-attr') === 'is') {
            s.push({
              id: d.id,
              data: d.data,
              negate: false,
            });
          } else {
            s.push({
              id: d.id,
              data: d.data,
              negate: true,
            });
          }
        });

      return s;
    }

    // Set checked values
    selectedSegments = _;
    let sh = {};
    _.forEach(function(s) {
      sh[s.id] = s.negate;
    });

    d3.selectAll(`#${segments_dom_id} input:not(.all)`)
      .each(function(d) {
        if (d.id in sh) {
          $(this).parent().show();
          if (sh[d.id]) {
            d3.select(this)
              .property('checked', false);
            $(this).parent().children('a').text('not').attr('segment-isnot-attr', 'not');
            this.checked = true;
          } else {
            d3.select(this)
              .property('checked', true);
            $(this).parent().children('a').text('is').attr('segment-isnot-attr', 'is');
            this.checked = true;
          }
        } else {
          d3.select(this)
            .property('checked', false);
          $(this).parent().children('a').text('is').attr('segment-isnot-attr', 'is');
        }
      });

    return dataset_segments;
  };

  dataset_segments.applyToSelectBuilder = function(sb, selected_segments, prefix) {
    /*
     * This function is called to apply currently selected segments
     * to the select statement being built.
     * We do this in two steps
     * - first, we make groups of segment selections
     *   Within a group, the OR statement is used between predicates
     *   Across groups, the AND statement is used between predicates
     *   For example, lets say there is a segment group called 'Gender'
     *   with 'Male' and 'Female' segments, and there is a second segment
     *   group called 'Age Buckets' with segments '0-16', '16-25' and '26-35'.
     *   If a user clicks on 'Male' and '16-25', he gets males between 16-25
     *   If a user clicks on 'Female', '16-25' and '26-35', he gets females
     *   between 16 and 35. This works because we place an OR predicate between
     *   the 2 segments in the age bucket group, and an AND predicate across
     *   the 2 groups.
     * - then, we apply the segments, one group at a time
     */
    if (!selected_segments) {
      selected_segments = dataset_segments.selected_segments();
    }

    var individuals = [];
    var grouped_segments = {};
    var groups = [];
    selected_segments.forEach(function(s) {
      if (s.negate) {
        // All exclusions apply across an AND.
        // Place them in individual segments
        individuals.push(s);
        return;
      }

      var g = s.data.group || '';

      // Move segments applied by filter association into there own groups so they
      // are applied seperately.
      if (g && s.faseg) {
        g = '_arc_faseg_' + g;
      }

      if (!g) {
        // Ungrouped segments apply across an AND.
        // Place them in individual segments
        individuals.push(s);
        return;
      }

      if (g in grouped_segments) {
        grouped_segments[g].push(s);
      } else {
        grouped_segments[g] = [s];
        groups.push(g);
      }
    });

    individuals.forEach(function(individual_segment) {
      arc.report.segment_group()
         .groupdata([individual_segment])
         .applyGroupToSelectBuilder(sb, prefix);
    });

    groups.forEach(function(g) {
      arc.report.segment_group()
         .groupdata(grouped_segments[g])
         .applyGroupToSelectBuilder(sb, prefix);
    });

    return dataset_segments;
  };

  dataset_segments.parse_segments = function(raw_segments) {
    segments = raw_segments;
    segments.forEach(function(d) {
      if (typeof (d.data) === typeof ('')) {
        d.data = JSON.parse(d.data);
      }
    });

    return dataset_segments;
  };

  dataset_segments.segments = function(_) {
    if (!arguments.length) return segments || [];
    segments = _;
    return dataset_segments;
  };

  dataset_segments.segmentDataRows = function() {
    const data = {
      coltypes: ['STRING', 'STRING', 'INTEGER'],
      rows: [],
    };

    const ss = dataset_segments.segments().slice();

    // sort by group first, then name
    ss.sort(function(a, b) {
      if (a.data.group < b.data.group) {
        return -1;
      }

      if (a.data.group > b.data.group) {
        return 1;
      }

      if (a.data.name < b.data.name) {
        return -1;
      }

      if (a.data.name > b.data.name) {
        return 1;
      }

      return 0;
    });

    ss.forEach(function(segment) {
      data.rows.push([segment.data.name, segment.data.group, Number(segment.id)]);
    });

    return data;
  };

  dataset_segments.groupedSegments = function() {
    var sl = dataset_segments.segments().slice(0);
    sl.forEach(function(s) {
      if (s.data.group === '') {
        s.data.group = 'Not Grouped';
      }
    });

    sl.sort(function(a, b) {
      if (a.data.group < b.data.group) {
        return -1;
      }

      if (a.data.group > b.data.group) {
        return 1;
      }

      return 0;
    });

    var grouped_segments = [];
    sl.forEach(function(s) {
      if ((grouped_segments.length > 0) &&
          (grouped_segments[grouped_segments.length - 1][0].data.group === s.data.group)) {
        grouped_segments[grouped_segments.length - 1].push(s);
      } else {
        grouped_segments.push([s]);
      }
    });

    return grouped_segments;
  };

  dataset_segments.updateSegmentDimElem = function(viz_lib) {
    if (viz_lib && viz_lib.isSolr()) {
      return dataset_segments;
    }

    var $segment_element = $('.super-modal .dim-app-params .selection_element').first();
    if (segments && $segment_element.find('span[title="' +
      arc.report.segment_display_name + '"]').length > 0) {
      if (segments.length === 0) {
        $segment_element.draggable('option', 'disabled', true);
        $segment_element.addClass('ui-state-disabled');
      } else {
        $segment_element.draggable('option', 'disabled', false);
        $segment_element.removeClass('ui-state-disabled');
      }
    }

    return dataset_segments;
  };

  // Display segments in the app builder screen
  // This is also called for the app display/detail screen
  dataset_segments.paint = function() {
    if (!segments_dom_id || !$('#' + segments_dom_id).length) {
      return dataset_segments;
    }

    ++datasetSegmentOnscreenId;
    onscreenId = datasetSegmentOnscreenId;

    // In the app_edit.html screen, where segments_dom_id is set,
    // display available segments
    const sel = document.getElementById(segments_dom_id);
    const $sel = $(sel);
    const d3sel = d3.select(sel);
    const dontSave = $sel.closest('.segments-container').hasClass('no-save');
    $sel.find('ul').remove();
    $sel.find('div').remove();
    $sel.parent().find('#segments-count').text(segments.length);
    if (dataset_segments.segments().length === 0) {
      d3sel.append('div').text('No segments available');

      return dataset_segments;
    }

    const $selectAll =
      $('<div><ul><li><input type="checkbox" class="all"><span>Select All</span></li></ul></div>');
    $sel.prepend($selectAll);
    $selectAll.off('click.toggleAll').on('click.toggleAll', function() {
      const isChecked = $selectAll.find('input').prop('checked');
      $selectAll.closest('.segments-container').find('input').prop('checked', isChecked);
      if (!dontSave) {
        arc.report.pagenav.needs_save(true);
      }
    });

    var grouped_segments = dataset_segments.groupedSegments();
    grouped_segments.forEach(function(group) {
      d3sel.append('div')
         .text(function() {
           return group[0].data.group;
         });

      var ul = d3sel.append('ul');
      var labels = ul.selectAll('#nosel')
                     .data(group)
                       .enter()
                       .append('li');
      labels.append('input')
            .attr('type', 'checkbox')
            .on('click.segment_state_changed', function(d) {
              if (!dontSave) {
                arc.report.pagenav.needs_save(true);
              }
            });

      labels.append('a')
            .attr('class', 'segment-isnot')
            .text('is')
            .attr('segment-isnot-attr', 'is')
            .on('click.segment_state_changed', function(d) {
              return arc.report.segment_isnot_link(this, dontSave);
            });

      labels.append('span')
            .text(function(d) {
              return d.name;
            });
    });

    return dataset_segments;
  };

  dataset_segments.segmentDataToModalData = function(segment_data) {
    var in_sd = {};

    in_sd.id = segment_data.id;
    in_sd.group = segment_data.data.group;
    if (in_sd.group === 'Not Grouped') {
      in_sd.group = '';
    }

    in_sd.name = segment_data.name;
    in_sd.applyToNewVisuals = segment_data.data.applyToNewVisuals;
    in_sd.filters = segment_data.data.filters.slice();
    in_sd.entities = segment_data.data.entities.slice();

    return in_sd;
  };

  function segmentsDetailFetch(cb, error_cb) {
    const assets = [
        {
            asset: `${arc.frontend.globals.urls.segmentUrl}?dataset_id=${dataset.dataset_id()}`,
            postprocess: d3.json,
        },
    ];

    arc.staticfetcher.forget(assets);
    arc.staticfetcher.fetch(assets,
        function(error, segmentDetail) {
          if (error) {
            arc.report.alertMessage('Failed to fetch dataset segments',
                error);
            if (error_cb) {
              error_cb();
            }

            return;
          }

          cb(segmentDetail);
        });

    return dataset_segments;
  }

  dataset_segments.segmentModalAndRefresh = function(in_sd, create_mode) {
    var segment_modal = arc.report.segment_modal();
    const segmentNameHash = {};
    dataset_segments.segments().forEach(s => segmentNameHash[s.name] = s.id);

    function handleSegmentSave(segment_data, done_cb) {
      function segmentSaveCb() {
        segmentsDetailFetch(
          function(in_segments) {
            // Remember currently selected segments
            var ss = dataset_segments.selected_segments();
            dataset_segments.parse_segments(in_segments)
                            .paintSegmentsDetail()
                            .paint()
                            .updateSegmentDimElem()

                            // Set selections back
                            .selected_segments(ss);
            arc.report.alertMessageSmall('Segment data saved', 'alert-success');
            done_cb();
          },

          // Error callback
          done_cb);
      }

      dataset_segments.saveSegment(segment_data, segmentSaveCb);
    }

    segment_modal.edit_modal(in_sd, handleSegmentSave, create_mode, segmentNameHash, dataset);
  };

  dataset_segments.reloadSegments = function(done_cb) {
    segmentsDetailFetch(function(segments) {
      // Remember currently selected segments
      var ss = dataset_segments.selected_segments();
      dataset_segments.parse_segments(segments)

                     // Set selections back
                     .selected_segments(ss)
                     .paint();

      if (done_cb) {
        done_cb();
      }
    });

    return dataset_segments;
  };

  // Display segments in the dataset detail screen
  dataset_segments.paintSegmentsDetail = function() {
    if (d3.select('#segments-table').size() === 0) {
      return dataset_segments;
    }

    dataset_segments.segments().sort(function(a, b) {
      if (a.data.group < b.data.group) {
        return -1;
      }

      if (a.data.group > b.data.group) {
        return 1;
      }

      if (a.data.entities < b.data.entities) {
        return -1;
      }

      if (a.data.entities > b.data.entities) {
        return 1;
      }

      if (a.data.name < b.data.name) {
        return -1;
      }

      if (a.data.name > b.data.name) {
        return 1;
      }

      return 0;
    });

    d3.select('#segments-table')
      .select('tbody')
      .selectAll('tr')
      .remove();

    var num_commas = d3.format(',');

    var rows =
      d3.select('#segments-table')
        .select('tbody')
        .selectAll('tr')
        .data(dataset_segments.segments())
        .enter()
          .append('tr');

    updateSegmentsCount();

    rows.append('td')
        .text(function(d) {
          return d.data.group;
        });

    rows.append('td')
        .text(function(d) {
          return d.data.entities.join(', ');
        });

    rows.append('td')
        .text(function(d) {
          return d.data.name;
        });

    rows.append('td')
        .text(function(d) {
          return (d.data.applyToNewVisuals === true) ? 'Yes' : '';
        });

    rows.append('td')
        .append('div')
        .classed('data-filter-wrapper', true)
        .text(function(d) {
          return '(' + d.data.filters.join(') AND (') + ')';
        });

    var segment_action_content = '<div class="action-button-wrapper">' +
        '<a class="dataset-segment-edit">' +
        '<span class="fa fa-pencil"></span> Edit</a> <a class="dataset-segment-clone">' +
        '<span class="fa fa-copy"></span> Clone</a> <a class="dataset-segment-delete">' +
        '<span class="fa fa-trash-o"></span> Delete</a></div>';

    var actions = rows.append('td');
    if (arc.misc.hasPerm('ds_manage', dataset.dataset_perms())) {
      actions.html(segment_action_content);
    }

    d3.selectAll('.dataset-segment-edit')
      .on('click.editsegment', function() {
        var d = d3.select(d3.select(this).node().parentNode.parentNode).datum();
        var in_sd = dataset_segments.segmentDataToModalData(d);
        dataset_segments.segmentModalAndRefresh(in_sd, false);
        return false;
      });

    d3.selectAll('.dataset-segment-clone')
      .on('click.clonesegment', function() {
        var d = d3.select(d3.select(this).node().parentNode.parentNode).datum();
        var in_sd = dataset_segments.segmentDataToModalData(d);
        in_sd.id = null;
        in_sd.name = 'Copy of ' + d.name;
        dataset_segments.segmentModalAndRefresh(in_sd, true);
        return false;
      });

    d3.selectAll('.dataset-segment-delete')
      .on('click.deletesegment', function() {
        const $modal = $('#delete-segment-modal');
        if (arc.misc.modalShown($modal)) {
          return;
        }

        const dataSegment = d3.select(d3.select(this).node().parentNode.parentNode).datum();

        // Show the delete confirmation modal,
        // and delete when confirmed
        d3.select('#delete-segment-yes')
          .on('click.deleteapp', () => {
            let url = arc.frontend.globals.urls.segmentUrl;
              url += `/${dataSegment.id}?dataset_id=${dataset.dataset_id()}`;

              // save new segment
              $.ajax({
                url,
                type: 'DELETE',
                beforeSend(xhr) {
                  // Because of a jquery bug when handling DELETE requests,
                  // we have to explicitly set X-CSRFToken here
                  xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
                },
              })
              .fail((req) => {
                arc.report.alertMessage('Failed to delete segment', req);
              })
              .done(updateSegmentsCount)
              .always(() => {
                segmentsDetailFetch((s) => {
                  dataset_segments.parse_segments(s)
                                  .paintSegmentsDetail();
                  $('#delete-segment-modal').modal('hide');
                  arc.frontend.legacyBridge.events.next({ type: 'new-dataset-version-created' });
                },

                // Error function for 'segmentsDetailFetch'
                () => {
                  $('#delete-segment-modal').modal('hide');
                });
              });
          });

        $modal.modal('show');
      });

    $('#segments-table').trigger('update');

    return dataset_segments;
  };

  dataset_segments.saveSegment = function(segment_data, done_cb) {
    const dataset_id = dataset.dataset_id();
    const dataset_version_id = dataset.version_id();
    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      name: segment_data.name.trim(),
      data: JSON.stringify(segment_data),
      dataset_id,
      dataset_version_id,
    };
    let url = arc.frontend.globals.urls.segmentUrl;
    if (segment_data.id) {
      url += `/${segment_data.id}?dataset_id=${dataset_id}&dataset_version_id=${dataset_version_id}`;
    }

    // save new segment
    $.ajax({
      url,
      data: post_data,

      // 'PUT' updates a segment, 'POST' creates a new one
      type: segment_data.id ? 'PUT' : 'POST',
      beforeSend(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .fail((req) => {
      arc.report.alertMessage(`Failed to ${
          segment_data.id ? 'update' : 'create'} segment`,
          req.responseText ? req.responseText : req);
    })
    .always(() => {
      if (done_cb) {
        done_cb();
      }
    });

    return dataset_segments;
  };

  return dataset_segments;
};

// This object displays and manages the dialog box that handles segment
// creation and editing.
arc.report.segment_modal = function() {

  var segment_modal = function() { };

  segment_modal.clearErrors = function() {
    $('#viz-segment-create').find('.alert').remove();
  };

  segment_modal.edit_modal = function(segment_data, save_cb, create_mode, segmentNames, dataset) {
    const $segmentModal = $('#viz-segment-create');
    if (arc.misc.modalShown($segmentModal)) {
      return;
    }

    $segmentModal.off('hide.bs.modal')
                 .on('hide.bs.modal', segment_modal.clearErrors);
    var segment_id = segment_data.id;

    if (create_mode) {
      $('#viz-segment-create-title').text('Add Segment');
      $('#viz-segment-save').text('Save');
    } else {
      $('#viz-segment-create-title').text('Edit Segment');
      $('#viz-segment-save').text('Save');
    }

    $('#viz-segment-default').prop('checked', segment_data.applyToNewVisuals === true);
    $('#viz-segment-create-name').val(segment_data.name || '');
    $('#viz-segment-create-group').val(segment_data.group || '');
    const filterText =
      'Enter an expression that can serve as a WHERE clause. For example: event_type="login"';
    $segmentModal.modal('show');
    arc.report.addableExprList('viz-segment-filters', segment_data.filters,
      $segmentModal.find('#add-segment-filter-btn'), dataset, {
        title: 'Filter Expression',
        helptext: filterText,
        exprType: 'filter',
        excludeRecordCountAutocomplete: true,
        includeTables: true,
      });
    arc.report.addableExprList('viz-segment-entities', segment_data.entities,
      $segmentModal.find('#add-segment-entity-btn'), dataset, {
        title: 'Entity Expression',
        exprType: 'column',
        noAggs: true,
      });

    const $segment_name = $('#viz-segment-create-name');
    const $segment_group = $('#viz-segment-create-group');
    var is_create = false;
    if (!segment_data.name) {
      // Starting with an empty name - this is a create
      //
      // Once the modal is visible, focus on the segment name
      // If the user enters a segment name and then hits 'Enter',
      // focus on the Save button.
      // In this way, if we get the filters right, you are a few
      // keystrokes away from a new segment.
      is_create = true;
      $segmentModal.off('shown.bs.modal')
                   .on('shown.bs.modal', function() {
                     $segment_name.focus();
      });
    }

    $segment_name.on('keypress.segment', function(event) {
      // If enter or ctrl-enter is pressed...
      if (event.which === arc.misc.ENTER_KEY) {
        if (is_create) {
          $('#viz-segment-create-group').focus();
        } else {
          $('#viz-segment-create-name').blur();
        }
      }

      // Do not propagate newline onwards
      return event.which !== 13;
    });

    $segment_group.on('keypress.segment', function(event) {
      // If enter or ctrl-enter is pressed...
      if (event.which === arc.misc.ENTER_KEY) {
        if (is_create) {
          $('#viz-segment-save').focus();
        } else {
          $('#viz-segment-create-group').blur();
        }
      }

      // Do not propagate newline onwards
      return event.which !== 13;
    });

    d3.select('#viz-segment-save').on('click.savesegment', () => {
      segment_modal.clearErrors();
      const errs = [];
      const sd = {};
      sd.name = $('#viz-segment-create-name').val();
      const segmentWithName = segmentNames[sd.name];
      const $modalBody = $segmentModal.find('.modal-body');
      if (!sd.name) {
        errs.push('Segment must have a name.');
      } else if (segmentWithName && segmentWithName !== segment_data.id) {
        errs.push(`Segment named ${sd.name} already exists.`);
      }

      sd.group = $('#viz-segment-create-group').val();
      sd.filters = [];
      sd.entities = [];
      sd.applyToNewVisuals = ($('#viz-segment-default').prop('checked') === true);
      sd.id = segment_data.id;
      $('#viz-segment-filters li input').each((idx, elem) => {
        if (elem.value.trim() !== '') {
          sd.filters.push(elem.value);
        }
      });

      if (!sd.filters.length) {
        errs.push('Segment must have at least one filter specified.');
      }

      $('#viz-segment-entities li input').each((idx, elem) => {
        if (elem.value.trim() !== '') {
          sd.entities.push(elem.value);
        }
      });

      if (errs.length) {
        arc.misc.addErrors(errs, $modalBody);
        return;
      }

      save_cb(sd, () => {
        $('#viz-segment-create').modal('hide');
        arc.frontend.legacyBridge.events.next({ type: 'new-dataset-version-created' });
        if (!sd.id) {
          updateSegmentsCount();
        }
      });
    });
  };

  return segment_modal;
};
}());
