window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};
arc.report.shelf = arc.report.shelf || {};

// Save class names as variable so easy to change classes of sort icon
// (used in analytical functions)
arc.report.shelf.ASC_CLASSES = 'fa-sort-amount-asc asc';
arc.report.shelf.DESC_CLASSES = 'fa-sort-amount-desc desc';
arc.report.shelf.EXPR_TRUE = '(1 = 1)';

arc.report.shelf.autotimeGapFetch = null;

arc.report.shelf.dataTransformationsMap = {
  cumulativeSumAsc: 'cumulativeSumAsc',
  cumulativeSumDesc: 'cumulativeSumDesc',
  cumulativeDiffAsc: 'cumulativeDiffAsc',
  cumulativeDiffDesc: 'cumulativeDiffDesc',
  differenceDesc: 'differenceDesc',
  differenceAsc: 'differenceAsc',
};

const dataTransformationToActionIdMap = {
  cumulativeSumAsc: 'shelf-cal-cumulative-asc',
  cumulativeSumDesc: 'shelf-cal-cumulative-desc',
  cumulativeDiffAsc: 'shelf-cum-difference-asc',
  cumulativeDiffDesc: 'shelf-cum-difference-desc',
  differenceAsc: 'shelf-cal-difference-asc',
  differenceDesc: 'shelf-cal-difference-desc',
};

const $propertiesPanel = $('.properties-panel');

function fadeOutProperties() {
  $propertiesPanel.fadeOut();
}

arc.report.shelf.getCurrentParams = function() {
  if (arc.globals && arc.globals.dashboard && arc.globals.dashboard.current_params) {
    const current_param_history = arc.globals.dashboard.current_params();
    if (current_param_history && current_param_history[0]) {
      return current_param_history[0];
    }
  }

  if (arc.globals && arc.globals.params) {
    return arc.globals.params.getCopy();
  }

  return undefined;
};

// TODO: unify d3 vs jquery data -- not sure if this is needed here
function getDatum($sel) {
  return d3.selectAll($sel.toArray()).datum() || $sel.data('shelf_elem');
}

arc.report.shelf.getDatum = getDatum;

function pageNeedsSave() {
  const report = arc.report.getPrimaryReport();
  if (report && report.viz()) {
    report.viz().page_needs_save();
  } else {
    arc.report.pagenav.needs_save(true);
  }
}

function removeClick(e, isDropdown) {
  const report = arc.report.getPrimaryReport();
  report.clearShelfHistory();
  const shelves = report.viz().shelves();
  let $li;
  if (isDropdown) {
    $li = $('.selected-list > li.selected');
  } else {
    $li = $(e.currentTarget);
  }

  const idx = $li.index();
  pageNeedsSave();

  const $ul = $li.closest('ul.selected-list');
  const shelfId = $ul.attr('id');
  shelves.remove(shelfId, idx);
  if (shelfId === 'color_shelf') {
    handleColorSettingsOnColorShelfElementChange(shelves.getShelf(shelfId));
  }

  fadeOutProperties();
}

const tabularVizTypes = new Set(['crosstab', 'table']);
const colorShelfDomIds =
    new Set([
      'dimensions_shelf',
      'aggregates_shelf',
      'x_dimensions_shelf',
      'y_dimensions_shelf',
    ]);

const activeDropdowns = (function() {
  const dropdowns = function() { return 'activeDropdowns'; };

  let shelfElement;
  let d3_elem;
  let $sel;
  let bgColorSetting;

  dropdowns.shelfElement = function(_) {
    if (!arguments.length) return shelfElement;
    shelfElement = _;
    return dropdowns;
  };

  dropdowns.d3_elem = function(_) {
    if (!arguments.length) return d3_elem;
    d3_elem = _;
    return dropdowns;
  };

  dropdowns.$sel = function(_) {
    if (!arguments.length) return $sel;
    $sel = _;
    return dropdowns;
  };

  dropdowns.activate = function() {
    setupDropdowns();
  };

  dropdowns.selectActions = function() {
    // unselect previous selections
    unselectMenuItem($propertiesPanel.find('.category1 .selected'));

    // find the actions to select
    selectAction(castToActionId(shelfElement));

    selectAction(displayBackgroundColorToActionId(shelfElement));

    selectAction(analyticalFunctionToShelfId(shelfElement));
    selectAction(dataTransformationToActionId(shelfElement));
    selectAction(aggregateToActionId(shelfElement));
    selectAction(dateFunctionToActionId(shelfElement));
    selectAction(stringFunctionToActionId(shelfElement));
    selectAction(secondaryAxisToActionId(shelfElement));

    selectConditionalAggFromElement(shelfElement);

    selectAlias();
    selectExpansion(shelfElement);
  };

  return dropdowns;

  function selectAlias() {
    const $aliasInput = $propertiesPanel.find('.shelf-alias-text');
    if (shelfElement.col_alias()) {
      $aliasInput.val(shelfElement.col_alias());
      selectParentMenuItem($aliasInput);
    } else {
      $aliasInput.val('');
      unselectParentMenuItem($aliasInput);
    }
  }

  function selectExpansion(shelfElem) {
    if ((shelfElement.viz_lib().viz_type() !== 'crosstab') &&
        (shelfElement.viz_lib().viz_type() !== 'table')) {
      return;
    }

    const $enableExpansionInput = $propertiesPanel.find('[data-action="shelf-expansion-enable"]');
    if (shelfElem.enableExpansion()) {
      selectMenuItem($enableExpansionInput);
    } else {
      unselectMenuItem($enableExpansionInput);
    }
  }

  function setupConditionalAggs() {
    const dataset = shelfElement.dataset();
    if (dataset) {
      const dateTimeColumn = dataset.defaultDateTime();
      if (dateTimeColumn) {
        $propertiesPanel
            .find('.datetime-conditional')
            .show();

        return;
      }
    }

    $propertiesPanel
        .find('.datetime-conditional')
        .hide();
  }

  function setupColumnPositions() {
    const report = arc.report.getPrimaryReport();

    if (!report.viz()) {
      return;
    }

    if (report.viz().viz_type() !== 'table') {
      // nothing to do
      return;
    }

    if (shelfElement.shelf().display_name() !== 'Measures') {
      // nothing to do
      return;
    }

    const $positionFunctions = $('.builder-container li[data-category="position-functions"]');
    const $positions = $positionFunctions.find('.positions');

    $positionFunctions.removeClass('arc-invisible');

    const shelves = report.viz().shelves();
    const shelf = shelves.getShelf('dimensions_shelf');
    const elements = shelf.elements();

    $positions.append(arc.misc.safe_substitute('<option val="">None</option>', {}));

    const validValues = {};

    elements.forEach((element) => {
      const name = element.getComputedDisplayName(true);
      $positions.append(arc.misc.safe_substitute(
          '<option val="%VALUE%">%LABEL%</option>',
          {
            LABEL: name,
            VALUE: name,
          }
      ));

      validValues[name] = true;
    });

    $positions.off('change.sel');
    const savedPos = shelfElement.coldisplaybefore();
    if (savedPos && validValues[savedPos]) {
      $positions.val(savedPos);
      $positionFunctions.addClass('property-selected');
    }

    $positions.on('change.sel', function() {
                shelfElement.coldisplaybefore($(this).val());
                pageNeedsSave();
                if ($positions.val() === '') {
                  $positionFunctions.removeClass('property-selected');
                } else {
                  $positionFunctions.addClass('property-selected');
                }
              });
  }

  function colValuesForShelfElement() {
    const report = arc.report.getPrimaryReport();
    const viz_lib = report && report.viz();
    if (!report || !viz_lib) {
      return [];
    }

    const dataResult = viz_lib.dataResult();
    if (!dataResult) {
      return [];
    }

    const rows = dataResult.rows();
    const columns = dataResult.columns();

    const shelf = shelfElement.shelf();
    const elements = shelf.elements();
    let shelfIndex;
    elements.some((e, i) => {
      if (e === shelfElement) {
        shelfIndex = i;
        return true;
      }

      return false;
    });

    if (shelfIndex === undefined) {
      return [];
    }

    const shelfName = shelf.shelf_name();

    let shelfCol;
    columns.some((c) => {
      if ((c.colsource() === shelfName) && (c.shelfIndex() === shelfIndex)) {
        shelfCol = c;
        return true;
      }

      return false;
    });

    if (shelfCol) {
      const colValues = arc.misc.arrayUnique(rows, (row) => row[shelfCol.colindex()]);
      return colValues;
    }

    return [];
  }

  function setupColors() {
    const report = arc.report.getPrimaryReport();
    const viz_lib = report && report.viz();
    if (!report || !viz_lib) {
      return;
    }

    // Setup color in shelf settings for crosstab and table
    if (!tabularVizTypes.has(viz_lib.viz_type())) {
      return;
    }

    if (!colorShelfDomIds.has(shelfElement.shelf().dom_id())) {
      return;
    }

    const colValues = colValuesForShelfElement();

    bgColorSetting = arc.shelf.color()
                              .colValues(colValues);

    shelfElement.setupShelfElementBackgroundColor(true);

    bgColorSetting.viz_lib(viz_lib)
                  .$container($propertiesPanel.find('.bgcolor.settings-colorswatch-colors'))
                  .$toggler($propertiesPanel.find('[data-action=shelf-data-bgcolor]'))
                  .shelfElement(shelfElement)
                  .colorChangeCb((colors) => {
                    pageNeedsSave();

                    const bgColor = shelfElement.backgroundColor();
                    bgColor.colors = colors.colors;
                    bgColor.firstColor = colors.firstColor;
                    bgColor.colorSettings = colors.colorSettings;
                    bgColor.isReversed = colors.isReversed;
                    bgColor.opacity = colors.opacity;
                    bgColor.dimensionMappings = colors.dimensionMappings;
                    bgColor.fixedDimensionColors = colors.fixedDimensionColors;
                    shelfElement.backgroundColor(bgColor);
                    shelfElement.displayBackgroundColor(true);
                    selectAction(displayBackgroundColorToActionId(shelfElement));
                    if (viz_lib) {
                      viz_lib.redraw();
                    }
                  })
                  .draw();
  }

  // Hide irrelevant dropdown menus
  function setupDropdowns() {
    if (!arc.globals.is_edit) {
      // Nothing to do in view mode
      return;
    }

    const conn = arc.report.shelf.CONNECTION_TYPE;
    const dctype = conn.connection_type;

    setupConditionalAggs();

    setupColumnPositions();

    setupColors();

    if (shelfElement.shelf().drillShelf()) {
      dropdowns.selectActions();
      handleAlias();
      handleRemoveItem();
      return;
    }

    if (!shelfElement.shelf().geoShelf()) {
      // Only the geo shelf sees geo types
      $propertiesPanel.find('.geo-type').hide();
    }

    // For most shelves
    dropdowns.selectActions();
    handleDerivedData();
    handleConnection();
    handleOrdering();
    handleAlias();
    handleDesc();

    // For filters
    handleAggregatePredicate();
    handleSelectValues();

    // For all
    handleClearValues();
    handleRemoveItem();

    // For flow and funnel
    handleMinMaxRange();
    handleGoals();

    // For segments
    handleSegments();

    // For autotiming on solr connections
    handleAutotimeSolr();

    function handleSegments() {
      // When 'Pick values from a list' is clicked, open the modal
      $propertiesPanel.find('.values-picker-link').offOn('click.openSegmentModal', () => {
        setupSegmentModal(shelfElement, d3_elem);
      });

      if (shelfElement.isDatasetSegment() &&
          (shelfElement.shelf().shelf_name() === 'filters_shelf')) {
        $propertiesPanel.find('.expression-link').closest('li').hide();
        $propertiesPanel.find('.app-shelf-clear').closest('li').hide();
      }
    }

    function handleAutotimeSolr() {
      const hideAutotimeOptions = (dctype !== 'solr') ||
        shelfElement.dataset_coltype().toLowerCase() !== 'timestamp';

      $propertiesPanel.find('li[data-category="range-faceting-functions"]')
                      .toggleClass('arc-invisible', hideAutotimeOptions);

      if (hideAutotimeOptions) {
        return;
      }

      const $defaultGap = $propertiesPanel.find('.default-range-gap');
      const $defaultGapSpan = $defaultGap.find('.range-gap');
      const $inputGap = $propertiesPanel.find('.range-gap-override .range-gap');

      const autotime_override = shelfElement.autotime_override();
      if (autotime_override === 'AUTO') {
        selectMenuItem($defaultGapSpan);
      } else if (autotime_override) {
        $inputGap.val(autotime_override);
        selectMenuItem($inputGap);
      }

      $inputGap
        .off('change')
        .on('change', function() {
          const val = $(this).val();
          pageNeedsSave();
          shelfElement.autotime_override(val);
          if (val) {
            unselectMenuItem($defaultGapSpan);
            selectMenuItem($inputGap);
          } else {
            unselectMenuItem($inputGap);
            selectMenuItem($defaultGapSpan);
            shelfElement.autotime_override('AUTO');
          }
        });

      $defaultGap
        .off('click.autotimeToggle')
        .on('click.autotimeToggle', () => {
          pageNeedsSave();
          if ($defaultGapSpan.closest('li').hasClass('selected')) {
            shelfElement.autotime_override('');
          } else {
            shelfElement.autotime_override('AUTO');
          }
        });
    }

    function handleDerivedData() {
      const report = arc.report.getPrimaryReport();

      if (report.viz()) {
        const deriveddata = report.viz().derivedData();
        const dataset = report.viz().dataset();
        const existingAggs = [];
        if (deriveddata &&
          deriveddata.enabled() &&
          dataset && arc.dataconnection.supports_derived_data(dataset.dataconnection_type())
        ) {
          dataset.tables().forEach(function findAggCols(table) {
            table.measuresOnly().forEach(function checkIsAgg(col) {
              if (col.attr('exprhasagg')) {
                existingAggs.push(col.attr('alias'));
              }
            });
          });

          deriveddata.derivedDataList().forEach(function checkForDerivedAgg(elem) {
            if (elem && elem.dsreq && elem.dsreq.aggregates && elem.dsreq.aggregates.length) {
              existingAggs.push(elem.label);
            }
          });
          deriveddata.setupDerivedData(existingAggs);
          $propertiesPanel.find('a[data-action="shelf-deriveddata"]').show();
        } else {
          $propertiesPanel.find('a[data-action="shelf-deriveddata"]').hide();
        }
      }
    }

    function handleAggregatePredicate() {
      if (shelfElement.expressionIsAggregate() || shelfElement.expr_hasagg()) {
        $propertiesPanel.find('.minmax-picker-link').remove();
      }
    }

    function handleRemoveItem() {
      // Permits shelf elements to be removed
      $propertiesPanel.find('.app-shelf-remove').on('click.removeShelf', (e) => {
        removeClick(e, true);
      });
    }

    function handleSelectValues() {
      // When 'Pick values from a list' is clicked, open the modal
      $propertiesPanel.find('.minmax-picker-link').on('click', () => {
        setupFilterModal(shelfElement, d3_elem);
      });
    }

    function handleClearValues() {
      // Permits shelf elements to be removed
      $propertiesPanel.find('.app-shelf-clear').on('click', () => {
        shelfElement.clearCustomExpr();
        shelfElement.filter_input({});
        d3_elem.call(arc.report.shelf.settext);
      });
    }

    function handleConnection() {
      const isArcengineCompatible = arc.dataconnection.isArcEngineOrImpala(dctype);
      if (!isArcengineCompatible) {
        $propertiesPanel.find('[data-conn="arcengine"]')
          .addClass('arc-invisible', true);
      }

      $propertiesPanel
          .find('[data-conn-exclude]')
          .each(function() {
            const $this = $(this);
            const c = $this.attr('data-conn-exclude').split(' ');
            if (c.indexOf(dctype) >= 0) {
              $this.addClass('arc-invisible');
            }
          });
    }

    function handleGoals() {
      const $e = $propertiesPanel.find('.shelf-goal-set');
      if (shelfElement.event_goal()) {
        $e.closest('li').addClass('selected');
      }

      $propertiesPanel.find('.shelf-goal-set').offOn('click.shelfGoal', () => {
        if (shelfElement.event_goal()) {
          shelfElement.event_goal(false);
          $e.closest('li').removeClass('selected');
        } else {
          shelfElement.event_goal(true);
          $e.closest('li').addClass('selected');
        }
      });
    }

    function handleMinMaxRange() {
      const $minrange = $propertiesPanel.find('.shelf-range-min input');
          const $maxrange = $propertiesPanel.find('.shelf-range-max input');

      if (shelfElement.event_min_count()) {
        $minrange.val(shelfElement.event_min_count());
      }

      if (shelfElement.event_max_count()) {
        $maxrange.val(shelfElement.event_max_count());
      }

      $minrange.on('change', function() {
                 const val = $(this).val();
                 save_min_range_behavior(shelfElement, d3_elem, val);
               });

      $maxrange.on('change', function() {
                 const val = $(this).val();
                 save_max_range_behavior(shelfElement, d3_elem, val);
               });
    }

    function handleOrdering() {
      const $topKInput = $propertiesPanel.find('.shelf-order-top input');
      const $bottomKInput = $propertiesPanel.find('.shelf-order-bottom input');

      const $granular = $propertiesPanel.find(
          'a[data-action="shelf-order-granular"]'
      );

      const $ascDesc = $propertiesPanel.find('.define-order-asc-desc');
      $ascDesc.on('click.order', () => {
        unselectMenuItem($topKInput);
        unselectMenuItem($bottomKInput);
        remove_order_limit(shelfElement, d3_elem);
      });

      if (shelfElement.order_limit() && shelfElement.order()) {
        if (shelfElement.order().ascending()) {
          $bottomKInput.val(shelfElement.order_limit());
          selectMenuItem($bottomKInput);
        } else {
          $topKInput.val(shelfElement.order_limit());
          selectMenuItem($topKInput);
        }
      } else if (shelfElement.order()) {
        const asc = shelfElement.order().ascending();
        if (asc) {
          selectMenuItem($propertiesPanel.find('a[data-action="shelf-order-asc"]'));
        } else {
          selectMenuItem($propertiesPanel.find('a[data-action="shelf-order-desc"]'));
        }
      }

      if (shelfElement.order_limit() &&
          shelfElement.order() &&
          shelfElement.order_limit_granular()) {
        const $granular_li = $granular.closest('li');
        $granular_li.addClass('selected');
      }

      $granular.on('click.granular', () => {
        const $granular_li = $granular.closest('li');

        $granular_li.toggleClass('selected');

        if ($granular_li.hasClass('selected')) {
          $ascDesc.closest('li').removeClass('selected');
          shelfElement.order_limit_granular(true);
        } else {
          shelfElement.order_limit_granular(false);
        }
      });

      let hasChanged = false;

      function kInputChangeOrClick($kInput, $otherKInput, isAsc, e) {
        e.stopPropagation();
        e.preventDefault();
        const isClick = e.type === 'click';
        if ((isClick && e.target.nodeName === 'INPUT') || hasChanged) {
          return;
        }

        const isSelected = $kInput.closest('li').hasClass('selected');
        const val = $kInput.val();
        if (isSelected && (isClick || !val)) {
          unselectMenuItem($kInput);
          save_order_limit_behavior(shelfElement, d3_elem, val, isAsc, false);
        } else if (val) {
          save_order_limit_behavior(shelfElement, d3_elem, val, isAsc, true);
          selectMenuItem($kInput);
          unselectMenuItem($ascDesc);
          unselectMenuItem($otherKInput);
          selectParentMenuItem($kInput);
        }
      }

      $topKInput.closest('li').offOn('click.kInput', (e) => {
                  kInputChangeOrClick($topKInput, $bottomKInput, false, e);
                });

      $bottomKInput.closest('li').offOn('click.kInput', (e) => {
                     kInputChangeOrClick($bottomKInput, $topKInput, true, e);
                   });

      $topKInput.offOn('change', (e) => {
                  pageNeedsSave();
                  kInputChangeOrClick($topKInput, $bottomKInput, false, e);
                  hasChanged = true;

                  // timeout to insure that a change and click handler don't undo themselves.
                  setTimeout(() => {
                    hasChanged = false;
                  }, 250);
                });

      $bottomKInput.offOn('change', (e) => {
                  pageNeedsSave();
                  kInputChangeOrClick($bottomKInput, $topKInput, true, e);
                  hasChanged = true;

                  // timeout to insure that a change and click handler don't undo themselves.
                  setTimeout(() => {
                    hasChanged = false;
                  }, 250);
                });
    }

    function handleAliasChange($el) {
      const alias = $el.val().trim();
      if (alias === '') {
        if (shelfElement.col_alias()) {
          shelfElement.col_alias(undefined);
          unselectParentMenuItem($el);
          pageNeedsSave();
        }
      } else if (alias !== shelfElement.col_alias()) {
          shelfElement.col_alias(alias);
          pageNeedsSave();
          selectParentMenuItem($el);

          autocastGeoType(shelfElement);
        }

      d3_elem.call(arc.report.shelf.settext);
    }

    function handleAlias() {
      selectAlias();

      const $aliasInput = $propertiesPanel.find('.shelf-alias-text');
      $aliasInput.off('change')
                 .on('change', function() {
        handleAliasChange($(this));
      });

      $aliasInput.off('blur')
                 .on('blur', function() {
        handleAliasChange($(this));
      });
    }

    function handleDesc() {
      const $descInput = $propertiesPanel.find('.shelf-desc-text');
      if (shelfElement.col_desc()) {
        $descInput.val(shelfElement.col_desc());
        selectParentMenuItem($descInput);
      }

      $descInput.off('change')
                 .on('change', function() {
        const desc = $(this).val().trim();
        if (desc === '') {
          if (shelfElement.col_desc()) {
            shelfElement.col_desc(undefined);
            unselectParentMenuItem($descInput);
            pageNeedsSave();
          }
        } else if (desc !== shelfElement.col_desc()) {
            shelfElement.col_desc(desc);
            pageNeedsSave();
            selectParentMenuItem($descInput);
          }

        d3_elem.call(arc.report.shelf.settext);
      });
    }
  }
}());

arc.report.shelf.editModeInitialize = function() {
  const propertiesMenuTemplate = $('#properties-menu-template').html();
  const propertiesMenuDrillTemplate = $('#properties-menu-drillshelf-template').html();
  const predicatesMenuTemplate = $('#predicates-menu-template').html();
  const segmentsMenuTemplate = $('#segments-menu-template').html();
  const eventsFunnelMenuTemplate = $('#events-funnel-menu-template').html();
  const eventsFlowMenuTemplate = $('#events-flow-menu-template').html();

  // open properties panel when clicking on pill caret in shelf
  // Note: see also related handler in arc.report.shelf.setup
  $('#app-shelves').on('click', '.selected-list > li', function(e) {
    const $elem = $(this);
    const category = $elem.closest('li.has-submenu').attr('data-category');
    const shelf_elem = getDatum($elem.find('.selection_field'));
    const shelf = getDatum($elem.closest('ul.selected-list'));
    const shelf_type = shelf.type();

    if ($(e.target).attr('class') === 'field-remove') {
      e.stopPropagation();
      removeClick(e);
      return;
    }

    if ($elem.hasClass('selected')) {
      // If a selected window function is clicked on again, do nothing. Otherwise unselect it.
      // The window function handler pops up a modal and canceling the modal does not select
      // the property, so unselecting it on click is not correct.
      if (category !== 'window-functions') {
        $elem.removeClass('selected');
        $elem.find('.fa-caret-right').removeClass('column-selected');
        fadeOutProperties();
        $('#expand-viz-settings').hide();
      }
    } else {
      $('.selected-list li').removeClass('selected');
      $('.selected-list li').find('.fa-caret-right').removeClass('column-selected');
      $elem.addClass('selected');
      $elem.find('.fa-caret-right').addClass('column-selected');

      let template;
      const panel_header = $('.properties-panel .panel-header-title');

      if (shelf_type === arc.report.shelf_types.PREDICATE) {
        panel_header.text('Filter Properties');
        template = predicatesMenuTemplate;
      } else if (shelf_elem.isDatasetSegment()) {
        panel_header.text('Segment Properties');
        template = segmentsMenuTemplate;
      } else if (shelf_type === arc.report.shelf_types.EVENT_FUNNEL) {
        panel_header.text('Event Properties');
        template = eventsFunnelMenuTemplate;
      } else if (shelf_type === arc.report.shelf_types.EVENT_FLOW) {
        panel_header.text('Event Properties');
        template = eventsFlowMenuTemplate;
      } else {
        panel_header.text('Field Properties');
        if (shelf.drillShelf()) {
          template = propertiesMenuDrillTemplate;
        } else {
          template = propertiesMenuTemplate;
        }
      }

      $propertiesPanel.find('.tab-panel-content').html(template);

      const $sel = $elem.find('.selection_field');
      const d3_elem = d3.selectAll($sel.toArray());
      const d = d3_elem.datum();

      // set the display span below the alias field to the column epression
      const expr = d.getExpression(undefined, true);
      $('.alias-base-display').text(expr);

      activeDropdowns
          .d3_elem(d3_elem)
          .shelfElement(d)
          .$sel($sel)
          .activate();

      cleanupDropdowns(d.shelf(), d3.select('.properties-panel'), d);

      $propertiesPanel.find('.popover').remove();
      $propertiesPanel.fadeIn();
      $('#expand-viz-settings').hide();
    }

    e.stopPropagation();
  });

  // expand properties submenu
  $propertiesPanel.on('click',
      '.properties-menu > li.has-submenu > a', function() {
    $(this).parent().find('ul').slideToggle();
    $(this).parent().toggleClass('open');
  });

  // set the class of selected property to 'selected'
  // and removes other selections in group first (only one selection allowed)
  // css class adds a checkmark
  $propertiesPanel.on('click',
      '.properties-menu > li.has-submenu > ul > li > a', function() {
    const $this = $(this);
    if ($this.hasClass('manual-select')) {
      // some selections are handled manually
      return;
    }

    const $parent = $this.parent();
    const category = $this.closest('li.has-submenu').attr('data-category');

    if ($parent.hasClass('selected')) {
      // Do not un-select analytical functions, but un-select others
      if (category !== 'window-functions') {
        $parent.removeClass('selected');
        $this.closest('.properties-menu > li').removeClass('property-selected');
      }
    } else {
      $parent.parent().find('li').removeClass('selected');
      $parent.addClass('selected'); // geo problem
      // remove class 'selected' from parent properties group
      $this.closest('.properties-menu > li').addClass('property-selected');
    }
  });

  function hideProperties() {
    fadeOutProperties();
    $('.selected-list>li.selected').removeClass('selected');
    $('.selected-list>li .column-selected').removeClass('column-selected');
    $('#expand-viz-settings').show();
  }

  // close the properties panel if user clicks on the x
  $('.properties-panel-close').offOn('click.closePanel', () => {
    hideProperties();
  });

  // close the properties panel if user clicks outside of properties-list
  // or shelf element
  $(document).on('click.closeOutsideClick', (evt) => {
    const $target = $(evt.target);
    const isOutSideOfModal = $target.closest('.modal').length === 0;
    const isPropertiesPanelElement = $target.closest('.properties-panel-element').length;
    const isModalBackdrop = $target.hasClass('modal-backdrop');
    const isModalOpen = $target.hasClass('modal-open');
    const isDashVisualDataTab = $target.is('#dash-visual-data-tab');

    if (
      isOutSideOfModal &&
      !isPropertiesPanelElement &&
      !isModalBackdrop &&
      !isModalOpen &&
      !isDashVisualDataTab
    ) {
      hideProperties();
    }
  });

  $propertiesPanel.offOn('click.shelfStopPropagation', (e) => {
    e.stopPropagation();
  });
};

function geoCastToActionId(geo_cast) {
  if (!geo_cast) {
    return undefined;
  }

  return `shelf-cast-geo-${geo_cast}`;
}

function fnCastToActionId(fn_cast) {
  if (!fn_cast) {
    return undefined;
  }

  const conn = arc.report.shelf.CONNECTION_TYPE;

  switch (fn_cast) {
  case conn.cast_timestamp:
    return 'shelf-cast-timestamp';

  case conn.cast_string:
    return 'shelf-cast-string';

  case conn.cast_bigint:
    return 'shelf-cast-integer';

  case conn.cast_real:
    return 'shelf-cast-real';

  case conn.cast_boolean:
    return 'shelf-cast-boolean';

  default:
    return undefined;
  }
}

function actionIdToFnCast(actionId) {
  const conn = arc.report.shelf.CONNECTION_TYPE;

  switch (actionId) {
  case 'shelf-cast-timestamp':
    return conn.cast_timestamp;

  case 'shelf-cast-string':
    return conn.cast_string;

  case 'shelf-cast-integer':
    return conn.cast_bigint;

  case 'shelf-cast-real':
    return conn.cast_real;

  case 'shelf-cast-boolean':
    return conn.cast_boolean;

  default:
    return undefined;
  }
}

function secondaryAxisToActionId(d) {
  if (d.secondaryaxis()) {
    return 'shelf-axis-secondary';
  }

  return 'shelf-axis-primary';
}

function displayBackgroundColorToActionId(d) {
  if (d.displayBackgroundColor()) {
    return 'shelf-data-bgcolor';
  }

  return undefined;
}

function dataTransformationToActionId(d) {
  const m = d.dataTransformation();
  if (m === undefined) {
    return;
  }

  return dataTransformationToActionIdMap[m];
}

  function actionIdToDataTransformation(actionId) {
    return Object.keys(dataTransformationToActionIdMap)
      .find((key) => dataTransformationToActionIdMap[key] === actionId);
  }

function selectConditionalAggFromElement(d) {
  if (d.fn_type() !== arc.report.fn_types.AGGREGATE) {
    return;
  }

  const c = d.conditionalAgg();
  arc.report.shelf.selectConditionalAgg(c !== undefined);
}

function aggregateToActionId(d) {
  const conn = arc.report.shelf.CONNECTION_TYPE;

  if (conn && conn.expr_groupconcat) {
    if (d.custom_expr() === conn.expr_groupconcat(d.dataset_colname())) {
      return 'shelf-agg-groupconcat';
    }
  }

  if (d.fn_type() !== arc.report.fn_types.AGGREGATE) {
    return;
  }

  switch (d.fn_name()) {
  case conn.sum:
    return 'shelf-agg-sum';

  case conn.count:
    return 'shelf-agg-count';

  case conn.missing:
    return 'shelf-agg-missing';

  case conn.ndv:
    return 'shelf-agg-ndv';

  case 'countdistinct':
    return 'shelf-agg-countdistinct';

  case conn.min:
    return 'shelf-agg-min';

  case conn.max:
    return 'shelf-agg-max';

  case conn.avg:
    return 'shelf-agg-avg';

  case conn.sumofsquares:
    return 'shelf-agg-sumofsquares';

  case conn.appx_median:
    return 'shelf-agg-appx-median';

  case conn.stddev:
    return 'shelf-agg-stddev';

  case conn.stddev_pop:
    return 'shelf-agg-stddev-pop';

  case conn.variance:
    return 'shelf-agg-variance';

  case conn.variance_pop:
    return 'shelf-agg-variance-pop';

  case conn.aggregate:
    return 'shelf-agg-aggregate';

  case conn.group_concat:
    return 'shelf-agg-groupconcat';

  // no default
  }
}

function fnCastToColumnType(_typeString = '') {
  const typeString = _typeString.toLowerCase();

  const typeMap = {
    boolean: 'boolean',
    string: 'string',
    date: 'string',
    text: 'text',
    timestamp: 'timestamp',
    hierarchy: 'hierarchy',
    integer: 'integer',
    tinyint: 'integer',
    smallint: 'integer',
    int: 'integer',
    bigint: 'integer',
    real: 'real',
    float: 'real',
    double: 'real',
    array: 'array',
    map: 'map',
    struct: 'struct',
    binary: 'binary',
    vector: 'vector',
  };

  // TODO -- here the creation of the dataset should have done the mapping
  if (typeString.startsWith('decimal')) {
    return 'real';
  }

  const type = typeMap[typeString] || 'unknown';

  if (type === 'unknown') {
    // eslint-disable-next-line no-console
    console.log('Unknown type', typeString);
  }

  return type;
}

function dateFunctionToActionId(d) {
  return d.shelf_date_function();
}

function stringFunctionToActionId(d) {
  const conn = arc.report.shelf.CONNECTION_TYPE;

  if (d.fn_type() !== arc.report.fn_types.SCALAR) {
    return undefined;
  }

  const fn = d.fn_name();
  if (fn === conn.upper) {
    return 'shelf-fn-upper';
  }

  if (fn === conn.lower) {
    return 'shelf-fn-lower';
  }

  return undefined;
}

function analyticalFunctionToShelfId(d) {
  return d.analyticalFunctionSelection();
}

function castToActionId(d) {
  return geoCastToActionId(d.geo_cast()) || fnCastToActionId(d.fn_cast());
}

function removeOrderAndRepaint(d) {
  d.order(undefined);

  const shelves = d.shelf().shelves();
  if (shelves) {
    shelves.repaintOrder();
  }
}

function updateOrderAndRepaint(d, asc, priority) {
  d.order().priority(priority).ascending(asc);
  const shelves = d.shelf().shelves();
  if (shelves) {
    shelves.repaintOrder();
  }
}

function addOrderAndRepaint(d, asc, priority) {
  const order  = arc.report.shelf.order();
  order.ascending(asc);
  order.priority(priority ?? d.order?.()?.priority());

  d.order(order);

  const shelves = d.shelf().shelves();
  if (shelves) {
    shelves.repaintOrder();
  }
}

function dateConvert(d, unit) {
  const connection = arc.report.shelf.CONNECTION_TYPE;
  if (!connection) {
    return;
  }

  const c = arc.dataconnection.dateConvert({
    columnName: d.dataset_colname(),
    connection,
    unit,
  });

  if (c !== undefined) {
    d.clearCustomExpr()
     .custom_expr(c.expr)
     .col_alias(c.alias)
     .display_format(undefined);

    return;
  }

  let scalar;
  switch (unit) {
    case 'DATE':
      scalar = connection.date;
      break;

    case 'YEAR':
      scalar = connection.year;
      break;

    case 'MONTH':
      scalar = connection.month;
      break;

    case 'DAYOFMONTH':
      scalar = connection.dayofmonth;
      break;

    case 'DAYOFYEAR':
      scalar = connection.dayofyear;
      break;

    case 'DAYOFWEEK':
      scalar = connection.dayofweek;
      break;

    // no default
  }

  if (!scalar) {
    return;
  }

  d.fn_type(arc.report.fn_types.SCALAR)
   .fn_name(scalar)
   .col_alias('')
   .display_format(undefined);
}

function actionIdToDateFn(actionId, d) {
  const conn = arc.report.shelf.CONNECTION_TYPE;
  const column_name = d.dataset_colname();

  const shelves = d.shelf().shelves();

  d.shelf_date_function(actionId);

  switch (actionId) {
  case 'shelf-fn-autotime-auto':
    d.autotimeEnabled(true);
    addOrderAndRepaint(d, true);
    break;

  case 'shelf-fn-date':
    dateConvert(d, 'DATE');
    break;

  case 'shelf-fn-year':
    dateConvert(d, 'YEAR');
    break;

  case 'shelf-fn-quarter':
    dateConvert(d, 'QUARTER');
    break;

  case 'shelf-fn-month':
    dateConvert(d, 'MONTH');
    break;

  case 'shelf-fn-yearmonth':
    dateConvert(d, 'YEARMONTH');
    break;

  case 'shelf-fn-yearquarter':
    d.clearCustomExpr();
    d.custom_expr(conn.expr_yearquarter(column_name))
     .col_alias(`year and quarter of ${column_name}`);
    break;

  case 'shelf-fn-yearweek':
    d.clearCustomExpr();
    d.custom_expr(conn.expr_yearweek(column_name))
     .col_alias(`year and week of ${column_name}`);
    break;

  case 'shelf-fn-dom':
    dateConvert(d, 'DAYOFMONTH');
    break;

  case 'shelf-fn-doy':
    dateConvert(d, 'DAYOFYEAR');
    break;

  case 'shelf-fn-dow-num':
  case 'shelf-fn-dow-short':
  case 'shelf-fn-dow-long':
    dateConvert(d, 'DAYOFWEEK');
    if (actionId === 'shelf-fn-dow-num') {
      d.display_format(undefined);
    } else if (actionId === 'shelf-fn-dow-short') {
      d.display_format({
        type: 'date',
        result: 'ddd',
      });
    } else if (actionId === 'shelf-fn-dow-long') {
      d.display_format({
        type: 'date',
        result: 'dddd',
      });
    }

    addOrderAndRepaint(d, true);
    break;

  case 'shelf-fn-dow-num-mon':
  case 'shelf-fn-dow-short-mon':
  case 'shelf-fn-dow-long-mon':
    if (actionId === 'shelf-fn-dow-num-mon') {
      dateConvert(d, 'DAYOFWEEK_STARTMONDAY_ENDAT7');
    } else {
      dateConvert(d, 'DAYOFWEEK_STARTMONDAY_ENDAT6');
    }

    if (actionId === 'shelf-fn-dow-short-mon') {
      d.display_format({
        type: 'date',
        result: 'ddd',
      });
    } else if (actionId === 'shelf-fn-dow-long-mon') {
      d.display_format({
        type: 'date',
        result: 'dddd',
      });
    }

    addOrderAndRepaint(d, true);
    break;

    // no default
  }
}

arc.report.shelf.settext = function(sel) {
  sel.text((d) => d.getComputedDisplayName())
     .attr('title', (d) => {
        if (d.isDatasetSegment()) {
          return '';
        }

        if (d.col_alias()) {
          return d.getExpression(undefined, true);
        }

        return d.getComputedDisplayName();
     })
     .each(function(d) {
        const $element = $(this);
        const tooltipExists = ($element.data && $element.data('bs.tooltip'));

        // If this element is at its maximum width, then
        // display a tooltip
        if ($element.width() === 150 || ($element.text() !== $element.attr('title'))) {
          // If there is currently a tooltip here, all we have to do is update text
          if (tooltipExists) {
            $element.attr('data-original-title', $element.attr('title'));
          } else {
            // Otherwise, create the tooltip
            $element.tooltip({
              placement: 'top',
              container: 'body',
              delay: {
                show: 500,
                hide: 100,
              },
            });

            // On the actual mouse press, make the tooltip go away so that we don't
            // see it dragged along.
            $element.mousedown(() => {
              $element.tooltip('hide');
            });
          }
        } else if (tooltipExists) {
          $element.tooltip('dispose');
        }

        let coltype = d.dataset_coltype_class();
        if (d.fn_cast()) {
          coltype = `type-${fnCastToColumnType(d.fn_cast())}`;
        }

        const $li = $element.closest('ul.selected-list > li');
        $li.find('.dim-icon-type').attr('class', `dim-icon-type ${coltype}`);
        d.order_display($li);

        const $geo_icon = $li.find('.geo-icon');
        if (d.shelf() && d.shelf().geoShelf()) {
          if ($geo_icon) {
            if (d.geo_cast()) {
              $geo_icon.html("<i class='fa fa-globe'></i>");
              $geo_icon.addClass('geo-icon-style');
            } else {
              $geo_icon.html('');
              $geo_icon.removeClass('geo-icon-style');
            }
          }
        } else {
          $geo_icon.html('');
          $geo_icon.removeClass('geo-icon-style');
        }
      });
};

function remove_order_limit(d, d3_elem) {
  const report = arc.report.getPrimaryReport();

  if (d.order_limit() === undefined) {
    return;
  }

  d.order_limit(undefined);
  d.order_limit_granular(false);

  const shelves = report.viz().shelves();
  shelves.repaintOrder();
  d3_elem.call(arc.report.shelf.settext);
}

// Helper function used by the function below to save away custom
// ranges
function save_min_range_behavior(d, d3_elem, val) {
  if (d.event_min_count() === val) {
    return;
  }

  pageNeedsSave();
  d.event_min_count(val);
  d3_elem.call(arc.report.shelf.settext);
}

function save_max_range_behavior(d, d3_elem, val) {
  if (d.event_max_count() === val) {
    return;
  }

  pageNeedsSave();
  d.event_max_count(val);
  d3_elem.call(arc.report.shelf.settext);
}

function save_order_limit_behavior(d, d3_elem, val, asc, hasLimit) {
  const report = arc.report.getPrimaryReport();

  // Make 'asc' true/false
  asc = !!asc;

  const shelves = report.viz().shelves();
  if (d.order_limit() === val && d.order().ascending() === asc && hasLimit) {
    return;
  }

  if (!hasLimit) {
    removeOrderAndRepaint(d);
    return;
  }

  // Remove all other order-limits
  const order_elements = shelves.getOrder();
  order_elements.forEach((o) => {
    o.order_limit(undefined);
  });

  d.order_limit(val);
  const priority = 0;
  if (!d.order()) {
    // If element is not currently ordered, order by it so we can do TopK
    // We only permit one order at a time. Remove all current ordering
    addOrderAndRepaint(d, asc, priority);
  } else {
    updateOrderAndRepaint(d, asc, priority);
  }

  d3_elem.call(arc.report.shelf.settext);
}

function setupSegmentModal(d, d3_elem) {
  const report = arc.report.getPrimaryReport();
  const viz = report.viz();
  if (viz && (viz.isSolr() || viz.isKSQL() || viz.isElasticsearch())) {
    return;
  }

  arc.report.picker()
     .d3_elem(d3_elem)
     .shelf_element(d)
     .dataset(report.dataset())
     .dataset_segments(report.dataset_segments())
     .activate();
}

function setupFilterModal(d, d3_elem) {
  arc.filter.input()
     .d3_elem(d3_elem)
     .shelf_element(d)
     .lib(arc.report.getPrimaryReport())
     .activate();
}

function selectMenuItem($e) {
  if ($e.length === 0) {
    return;
  }

  $e.closest('li').addClass('selected');
  selectParentMenuItem($e);
}

function unselectMenuItem($e) {
  $e.closest('li').removeClass('selected');
  unselectParentMenuItem($e);
}

function selectParentMenuItem($e) {
  $e.closest('.properties-menu > li').addClass('property-selected');
}

function unselectParentMenuItem($e) {
  $e.closest('.properties-menu > li').removeClass('property-selected');
}

function selectAction(actionId) {
  if (!actionId) {
    return false;
  }

  const $a = $propertiesPanel.find(`a[data-action="${actionId}"]`);
  selectMenuItem($a);
  return true;
}

// Hide irrelevant dropdown menus
function cleanupDropdowns(shelf, sel, d) {
  // TODO: Talk with Sushil about hiding un-related items from the dropdown
  /**
  var element = d ? d : sel.select(".selection_field").datum();
  if (["string"].indexOf(element.getComputedType()) !== -1) {
    sel.selectAll(".dim-only")
               .classed('arc-invisible', false);
    sel.selectAll(".measures-only")
               .classed('arc-invisible', true);
  } else {
    if (["date", "boolean", "timestamp", "unknown"].indexOf(element.getComputedType()) !== -1) {
      sel.selectAll(".dim-only")
                 .classed('arc-invisible', false);
      sel.selectAll(".measures-only")
                 .classed('arc-invisible', false);
    } else {
      sel.selectAll(".measures-only")
                 .classed('arc-invisible', false);
      sel.selectAll(".dim-only")
                 .classed('arc-invisible', true);
    }
  }
    * */

  if (!arc.globals.is_edit) {
    // Nothing to do in view mode
    return;
  }

  // Hide window functions when using unsupported data connections
  if (arc.report.shelf.CONNECTION_TYPE &&
      arc.report.shelf.CONNECTION_TYPE.supportsWindowFunctions) {
    sel.selectAll('.window-functions').classed('arc-invisible', false);
  } else {
    sel.selectAll('.window-functions').classed('arc-invisible', true);
  }

  // If element already has aggregate, hide date/time and aggregate functions
  if (d) {
    changeShelfDropdownOptions(shelf, d, sel);
  } else {
    // d isn't available when shelves are being populated with saved elements
    // so iterate through all sel
    sel.each(function (s) {
      changeShelfDropdownOptions(shelf, s, d3.select(this));
    });
  }

  function changeShelfDropdownOptions(shelf_elem, d_elem, el) {
    const vizLib = d_elem.viz_lib();
    const vizType = vizLib.viz_type();

    if (!tabularVizTypes.has(vizType)) {
      el.selectAll('.viz-tabular').classed('arc-invisible', true);
    } else if (!colorShelfDomIds.has(shelf_elem.dom_id())) {
      el.selectAll('[data-category=data-bgcolor]').classed('arc-invisible', true);
    }

    if (shelf_elem.property() === arc.report.columnTypeEnum.DIMENSION) {
      el.selectAll('.agg-shelves').classed('arc-invisible', true);
    } else if (d_elem.expr_hasagg()) {
      el.selectAll('.agg-shelves').classed('arc-invisible', true);
    }

    const connType = arc.report.shelf.CONNECTION_TYPE.connection_type;
    $(el[0]).find('.agg-shelves li, .allagg-shelves li').each(function updateVisibility() {
      const $this = $(this);
      const permittedTypes = $this.data('conn') && $this.data('conn').split(' ');
      if (permittedTypes) {
        $this.toggleClass('arc-invisible', !permittedTypes.includes(connType));
      }
    });

    const hasagg = d_elem.expr_hasagg() || d_elem.expressionIsAggregate();
    el.selectAll('.shelf-axis').classed('arc-invisible', true);

    const shelfName = shelf_elem.display_name();
    if (!hasagg &&
        ((vizType === 'crosstab') || (vizType === 'table')) &&
        ((shelfName === 'Row') || (shelfName === 'Dimensions'))) {
      el.selectAll('.shelf-expansion').classed('arc-invisible', false);
    }

    // ARC-4766: Hide sort and limit in boxplots visuals that use window functions
    if (vizType === 'boxplot' && vizLib.viz().supportsOptimizedBoxplot()) {
      if (shelf_elem.dom_id() === 'aggregates_shelf') {
        sel.selectAll('.order-functions').classed('arc-invisible', true);
      } else {
        sel.selectAll('.shelf-order').classed('arc-invisible', true);
      }
    }

    // For aggregate shelf elements within trellis lines, permit picking
    // a secondary axis

    const aggregateShelves = ['Colors', 'Highlight Mark Measure', 'Bar Measure', 'Line Measure', 'Y Axis'];
    const isAggregateShelf = aggregateShelves.includes(shelfName);
    const secondayAxisSupportedOnVisual = ['combo', 'trellis-groupedbars', 'trellis-lines'].includes(vizType);
    if (isAggregateShelf && secondayAxisSupportedOnVisual) {
      let hideAxisSetting = false;
      switch (vizType) {
        case 'trellis-lines':
          hideAxisSetting = !hasagg;
          break;
        case 'combo':
          hideAxisSetting = (shelfName === 'Bar Measure' && shelf_elem.elements().length > 1);
          break;
        default:
          // nothing to do
      }

      el.selectAll('.shelf-axis').classed('arc-invisible', hideAxisSetting);
    }

    if (vizType === 'timeline') {
      if (shelfName === 'Start Time' || shelfName === 'End Time') {
        let info;
        if (shelfName === 'Start Time') {
          info = 'Display Format on the Start Time shelf will also be applied on the End Time shelf.';
        } else {
          info = 'Display Format can only be applied on the Start Time shelf,' +
                ' and it will also be applied on the End Time shelf.';

          $propertiesPanel.find('.display-format-link')
          .addClass('disabled-setting')
          .prop('disabled', true);
        }

        $propertiesPanel.find('.display-format-link')
          .append(`<span class="info-icon" data-toggle="popover" data-content="${info}"></span>`)
          .on('click', '.info-icon', (e) => {
            e.preventDefault();
          })
          .find('[data-toggle="popover"]')
          .popover({
            container: 'body',
            placement: 'top',
            trigger: 'hover'
          });
      }
    }

    $propertiesPanel.find('.order-functions-info')
      .offOn('click', '.info-icon', (e) => {
        e.preventDefault();
      })
      .find('[data-toggle="popover"]')
      .popover({
        container: 'body',
        placement: 'top',
        trigger: 'hover'
      });
  }
}

function set_connection_type(conn_type) {
  if (conn_type) {
    arc.report.shelf.CONNECTION_TYPE = arc.dataconnection.types[conn_type]();
  }
}

function autocastGeoType(shelf_elem) {
  const geo_types = arc.geo.validGeoTypes();

  const ds = shelf_elem.dataset();
  if (ds) {
    const col = ds.get_column(shelf_elem.dataset_colname());
    if (col) {
      const g = col.attr('geotype');
      if (geo_types.indexOf(g) > -1) {
        shelf_elem.geo_cast(g);
        return;
      }
    }
  }

  let col_name = shelf_elem.col_alias();
  if (col_name === undefined) {
    col_name = shelf_elem.dataset_colname();
  }

  if (col_name === undefined || shelf_elem.geo_cast()) {
    return;
  }

  col_name = col_name.toLowerCase();
  const autodetectGeo = arc.geo.autocastGeoType(col_name);

  if (autodetectGeo) {
    shelf_elem.geo_cast(autodetectGeo);
  }
}

arc.report.shelf.addNewParamToFilters = function(config = {}) {
  const {
    filterName,
    filterData,
    filterColtype,
    include,
    isSourcePivot,
    shelves,
  } = config;

  const shelf_elem = arc.report.shelf.element()
                          .dataset_colname(filterName)
                          .filter_column(filterName)
                          .filter_data(filterData)
                          .source_pivot(isSourcePivot)
                          .dataset_coltype(filterColtype);

  if (!include) {
    shelf_elem.filter_exclude(1);
  }

  const se = arc.report.shelfElementPostProcess({
    shelf_elem,
    shelf_type: arc.report.shelf_types.PREDICATE,
    shelf_property: arc.report.columnTypeEnum.UNKNOWN,
    shelf_id: 'filters_shelf',
  });

  const se_d = {};
  se.save(se_d);
  shelves.add('filters_shelf', [se_d], true);

  // Drill into feature add extra filters to apply visual changes.
  // In view mode this should not be saved.
  if (arc.globals.is_edit) {
    pageNeedsSave();
  }
};

function resetAllowLargeTrellis(viz_lib) {
  if (viz_lib && viz_lib.viz()) {
    const viz_api = viz_lib.viz();
    if (viz_api.allowLargeTrellisPerm) {
      viz_api.allowLargeTrellisPerm(false);
      viz_api.allowLargeTrellisTemp(false);
    }
  }
}

arc.report.shelf.selectConditionalAgg = function(on) {
  const $a = $propertiesPanel
               .find('a[data-action="shelf-agg-conditional"]')
               .closest('li');

  if (on) {
    $a.addClass('selected');
  } else {
    $a.removeClass('selected');
  }
};

function resetDateFunction(d) {
  if (d.shelf_date_function()) {
    d.fn_type(null)
     .fn_name(null)
     .display_format(undefined)
     .col_alias('');
  }

  d.shelf_date_function(undefined);
  d.autotimeEnabled(false);
}

arc.report.shelf.setup = function() {
  // When a shelf dropdown is clicked, modify the shelf element contents
  // handle Expansion
  // Note: see also related handler in arc.report.shelf.editModeInitialize
  $propertiesPanel.on('click', 'a[data-action]', function(event) {
    const conn = arc.report.shelf.CONNECTION_TYPE;
    const report = arc.report.getPrimaryReport();
    const viz = report.viz();
    const shelves = viz.shelves();

    $('.open .dropdown-toggle').dropdown('toggle');
    const $elem = $(this);
    const category = $elem.closest('li.has-submenu').attr('data-category');
    const selected = $elem.parent().hasClass('selected');
    const shelf_id = $elem.attr('data-action');
    const $li = $('.selected-list > li.selected');
    const $sel = $li.find('.selection_field');
    const d3_elem = d3.selectAll($sel.toArray());
    const d = d3_elem.datum();
    const column_name = d.dataset_colname();

    pageNeedsSave();

    if (shelf_id === 'shelf-agg-conditional') {
      arc.report.conditionalAggHandler()
         .element(d)
         .d3_elem(d3_elem)
         .setup();
      return;
    }

    if (shelf_id === 'shelf-agg-conditional-remove') {
      d.conditionalAgg(undefined);
      arc.report.shelf.selectConditionalAgg(false);
      return;
    }

    // Derived data
    if (shelf_id === 'shelf-deriveddata') {
      const deriveddata = arc.report.getPrimaryViz().derivedData();
      deriveddata.shelfElement(d);

      // bring up modal
      deriveddata.show_hide(true);
    }

    // Primary axis
    if (shelf_id === 'shelf-axis-primary') {
      d.secondaryaxis(false);
    }

    // Secondary axis
    if (shelf_id === 'shelf-axis-secondary') {
      d.secondaryaxis(true);
    }

    // Expansion in crosstab
    if (shelf_id === 'shelf-expansion-enable') {
      d.enableExpansion(!d.enableExpansion());
    }

    // Aggregations
    if (category === 'data-transformations') {
      if (!selected) {
        d.dataTransformation(undefined);
      } else {
        const actionId = actionIdToDataTransformation(shelf_id);
        if (actionId !== undefined) {
          d.dataTransformation(actionId);
        }
      }
    }

    if (category === 'data-bgcolor') {
      if (!selected) {
        d.displayBackgroundColor(false);
      } else if (shelf_id === 'shelf-data-bgcolor') {
        d.displayBackgroundColor(true);
      }

      $(event.target).trigger('colorUpdate');
      if (viz) {
        viz.redraw();
      }
    }

    if (category === 'aggregate-functions') {
      resetDateFunction(d);
      resetAllowLargeTrellis(viz);
      d.setShelfFnNameFromShelfId(shelf_id, conn, selected);
    }

    if (category === 'date-functions') {
      resetDateFunction(d);
      resetAllowLargeTrellis(viz);

      // Date functions
      if (!selected) {
        d.fn_type(null)
         .fn_name(null)
         .removeFunction();
        d.display_format(undefined);
      } else {
        d.fn_type(null)
         .fn_name(null)
         .removeFunction();
        actionIdToDateFn(shelf_id, d);
      }
    }

    if (category === 'string-functions') {
      resetDateFunction(d);
      resetAllowLargeTrellis(viz);

      // String functions
      if (!selected) {
        d.fn_type(null)
         .fn_name(null);
      } else if (shelf_id === 'shelf-fn-upper') {
        d.fn_type(arc.report.fn_types.SCALAR)
         .fn_name(conn.upper);
      } else if (shelf_id === 'shelf-fn-lower') {
        d.fn_type(arc.report.fn_types.SCALAR)
         .fn_name(conn.lower);
      }
    }

    if (category === 'window-functions') {
      resetAllowLargeTrellis(viz);

      // Advanced Analytical Functions
      if (!selected) {
        d.removeFunction();
      } else {
        // Variables for creating analytical function builder modal
        const iAnalyticalBuilder = arc.report.analyticalBuilder({
                                  shelves,
                                  d3_elem,
                                  connection: conn,
                                  currentElName: column_name,
                                });

        const shelfIdToFn = {
          'shelf-fn-moving-average': iAnalyticalBuilder.createMovingAverageModal,
          'shelf-fn-percent-total': iAnalyticalBuilder.createPercentTotalModal,
          'shelf-fn-running': iAnalyticalBuilder.createRunningTotalModal,
          'shelf-fn-previous-value': iAnalyticalBuilder.createPreviousValueModal,
          'shelf-fn-previous-difference': iAnalyticalBuilder.createPreviousDifferenceModal,
          'shelf-fn-previous-difference-pct': iAnalyticalBuilder.createPreviousDifferencePctModal,
          'shelf-fn-next-value': iAnalyticalBuilder.createNextValueModal,
          'shelf-fn-next-difference': iAnalyticalBuilder.createNextDifferenceModal,
          'shelf-fn-next-difference-pct': iAnalyticalBuilder.createNextDifferencePctModal,
          'shelf-fn-row-number': iAnalyticalBuilder.createRowNumberModal,
          'shelf-fn-rank': iAnalyticalBuilder.createRankModal,
          'shelf-fn-dense-rank': iAnalyticalBuilder.createDenseRankModal,
        };

        $propertiesPanel.find('.window-functions.category1 .selected')
                        .removeClass('selected');

        selectAction(shelf_id);
        const f = shelfIdToFn[shelf_id];
        if (f) {
          f(conn.connection_type, (success) => {
            if (success) {
              d.analyticalFunctionSelection(shelf_id);
            }

            activeDropdowns.selectActions();
          });
        }

        // Return early - after the modal is done, post-modal functions will run via
        // the callback above
        return;
      }
    }

    // Duplicate element, without the order
    if (shelf_id === 'app-shelf-clone') {
      const new_elem = d.clone().order(null);
      const shelf = new_elem.shelf();
      const $shelf_dom = $(`#${shelves.shelves_dom_id()}`).find(`#${shelf.dom_id()}`);
      const $clone_elem = $sel.closest('li').clone();
      $clone_elem.removeClass('selected');

      shelf.addElement(new_elem);
      $shelf_dom.append($clone_elem);

      const sel = d3.selectAll($clone_elem.toArray())
                    .select('.selection_field');

      new_elem.d3_elem(sel);
      sel.datum(() => new_elem);

      shelves.repaintOrder();

      // clear shelf history since shelf changed
      report.clearShelfHistory();
    }

    // Type casts
    if (category === 'cast-functions') {
      resetAllowLargeTrellis(viz);

      if (!selected) {
        d.fn_cast(null);
      } else {
        const c = actionIdToFnCast(shelf_id);
        if (c) {
          d.fn_cast(c);
        }
      }

      if (shelf_id.includes('shelf-cast-geo-')) {
        const geo_cast = shelf_id.substring('shelf-cast-geo-'.length);

        resetDateFunction(d);

        d.geo_cast(selected ? geo_cast : undefined);
      }
    }

    if (category === 'order-functions') {
      resetAllowLargeTrellis(viz);

      if (shelf_id === 'shelf-order-granular') {
        shelves.repaintOrder();
      } else if (!selected) { // Ordering
        removeOrderAndRepaint(d);
      } else if (shelf_id === 'shelf-order-desc') {
        addOrderAndRepaint(d, false);
      } else if (shelf_id === 'shelf-order-asc') {
        addOrderAndRepaint(d, true);
      }
    }

    cleanupDropdowns(d.shelf(), d3.select(d3_elem.node().parentNode), d);

    // Modify the display text for the element
    d3_elem.call(arc.report.shelf.settext);

    activeDropdowns.selectActions();

    return false;
  });

  // Helper function used by the function below to save away custom
  // expressions
  const save_expression = function(d3_elem, val) {
    const d = d3_elem.datum();
    if (d.getExpressionForEdit() === val) {
      $('#expression-modal').modal('hide');
      return;
    }

    d.clearCustomExpr();
    d.custom_expr(val);

    // Ensure we don't set alias or description for filter elements
    if (d.filter_column()) {
      d.col_alias(undefined);
      d.col_desc(undefined);

      // If the expression is exactly the filter column name with brackets
      // then clear out the custom expression so that we get the special
      // (1=1) behavior when no parameter is present matching the column name
      if (val === `[${d.filter_column()}]`) {
        d.clearCustomExpr();

        // Allow users to change custom expression when
        // filter items were created in arcviz versions before release_3.3.0
        d.filter_data([]);
      }
    }

    d3_elem.call(arc.report.shelf.settext);

    // Clear out other function selections in the menu
    d.fn_name(null)
     .fn_type(null)
     .fn_cast(null);

    // Part of resetDateFunction
    if (d.shelf_date_function()) {
      d.shelf_date_function(null);
      d.display_format(null)
       .col_alias('');
    }

    activeDropdowns.selectActions();
    pageNeedsSave();
    cleanupDropdowns(d.shelf(), d3.select(d3_elem.node().parentNode), d);
    $('#expression-modal').modal('hide');
  };

  // When 'Enter Expression' is clicked, open the modal
  $propertiesPanel.off('click.colexpr');
  $propertiesPanel.on('click.colexpr', '.expression-link', () => {
    const $modal = $('#expression-modal');
    const isModalAlreadyShown = arc.misc.modalShown($modal);
    if (isModalAlreadyShown) {
      return;
    }

    const $selectedFilter = $('.selected-list > li.selected .selection_field');
    const d3SelectedFilter = d3.selectAll($selectedFilter.toArray());
    const selectedFilter = d3SelectedFilter.datum();
    const report = arc.report.getPrimaryReport();
    const dataset = report.dataset();

    const $expressionForm = $('#edit-expression-form');
    const $validateExpressionButton = $expressionForm.find('.expression-validate');
    $validateExpressionButton.parent().find('.alert').remove();

    // initialize AceEditor with expression
    const expressionEditor = arc.misc.shelfExpressionEditor(
      dataset, selectedFilter.shelf().isSensitive()
    );
    expressionEditor.value(selectedFilter.getExpressionForEdit());

    $validateExpressionButton
      .offOn(
        'click',
        (e, alternativeSuccessCb) => {
          // Remove any existing alert text
          const $parent = $(e.target).parent();
          $parent.find('.alert').remove();
          $parent.find('[arcid=validate-spinner]').show();

          let expr = expressionEditor.value();

          // Do parameter substitution & split out the "as 'alias'" part if present
          const exprBeforeSubstitution = expr;
          expr = report.viz().substituteParams(expr);
          const esplit = arc.misc.splitExprAndAlias(expr);
          const splitExpr = esplit.expr;

          if (
            splitExpr !== exprBeforeSubstitution &&
            // When the expression is evaluated to be an empty string after
            // substitution, don't do full validation and instead immediately call
            // validation success callback
            report.viz().isShelfExpressionEmpty(splitExpr)
          ) {
            validateExpressionCallback({ type: 'STRING', hasagg: false, ispart: false });
          } else if (arc.misc.isWindowFunction(splitExpr)) {
            validateExpressionCallback({
              type: 'STRING',
              hasagg: false,
              ispart: false,
              info: 'Analytic functions cannot be properly validated. Please refresh visual to see any error messages.'
            });
          } else {
            arc.validate_expression(splitExpr, dataset, validateExpressionCallback, 'column');
          }

          function validateExpressionCallback(params) {
            const { err, info } = params;
            $parent.find('.alert').remove();
            $parent.find('[arcid=validate-spinner]').hide();
            const loc = $parent.find('.expression-validate');
            if (err) {
              loc.after(arc.clone_template('col-val-failed', { errtext: err }).html());
            } else if (info) {
              loc.after(arc.clone_template('col-val-info', { info }).html());
            } else if (alternativeSuccessCb) {
              alternativeSuccessCb();
            } else {
              loc.after(arc.clone_template('col-val-passed').html());
            }
          }
        }
      );

    $('#validate-and-save')
      .offOn('click.validateAndSave', () => {
        $validateExpressionButton.trigger('click', () => {
          $expressionForm.trigger('submit.save');
          arc.report.alertMessageSmall('Expression validated and saved', 'alert-success');
        });
     });

    $expressionForm
    .offOn('submit.save', (e) => {
        e.preventDefault();
        save_expression(d3SelectedFilter, expressionEditor.value());
        arc.globals.reports.getPrimaryReport()?.viz()?.saveAwayShelfInfo();
    });

    $modal.modal('show');
    $modal
      .offOn('hide.bs.modal', () => {
            // The ace editor will not repaint the screen
            // when the modal is hidden. This leads to
            // a shadow element showing the previous value
            // if the modal is transitioning in.
            //
            // Instead, on the way out, remove the value
            // from the editor
            expressionEditor.value('');
          });

    $modal
      .offOn('shown.bs.modal', () => { expressionEditor.focusEditor(); });
  });

  // Setup display format modal
  const $displayFormatModal = $('#display-format-modal');
  const displayOptions = arc.utils.displayOptions($displayFormatModal);
  $propertiesPanel.off('click.displayformat');
  $propertiesPanel.on('click.displayformat', '.display-format-link', () => {
    if (arc.misc.modalShown($displayFormatModal)) {
      return;
    }

    const $elem = $('.selected-list > li.selected .selection_field');
    const d3_elem = d3.selectAll($elem.toArray());
    const d = d3_elem.datum();

    const viz = arc.report.getPrimaryViz(); // For translatedDisplayFormat
    const previousResult = d.display_format(); // Used to see if format changed
    const unsavedValue = d.display_format(); // Used to populate customize field

    displayOptions.setUpEditor();
    displayOptions.resetHandlers(arc.report.getPrimaryReport().dataset(), previousResult);

    // For when user submits display format
    d3.select('#display-format-form').on('submit.expr', () => {
      let changed;
      const val = displayOptions.parseDisplayFormat();
      const d_elem = d3_elem.datum();
      const prevResult = previousResult && previousResult.type ?
                           previousResult.result : previousResult;

      d3.event.preventDefault();
      d3.event.stopPropagation();

      if (val.result !== prevResult) {
        pageNeedsSave();
        changed = true;
      }

      // remove old errors
      $displayFormatModal.find('.alert-dismissible').remove();

      displayOptions.updateSample();
      let err = displayOptions.errorMsg();
      if (val.type === 'js' && err) {
        err = `Error thrown by Custom JS Format Function: ${err}`;
        $('#display-format-js').after(arc.clone_template('col-val-failed',
                                      { errtext: err }).html());
      } else {
        d_elem.display_format(val);
        $displayFormatModal.modal('hide');
      }

      if (changed) {
        if (viz) {
          viz.refresh(true);
        }
      }
    });

    $displayFormatModal.off('hidden.bs.modal')
                       .on('hidden.bs.modal', function destroyEditor() {
                         $displayFormatModal.find('.alert-dismissible').remove();
                         displayOptions.destroyEditor();
                       });

    displayOptions.populateValues(unsavedValue, 'fromShelves');

    $displayFormatModal.modal('show');
  });
};

arc.report.shelf.order = function() {
  let ascending = false;
  let order_field;
  let priority;

  const o = function() { return 'arc.report.shelf.order'; };

  o.clone = function() {
    const shelf_order = arc.report.shelf.order()
               .ascending(ascending)
               .priority(priority)
               .order_field(order_field);
    return shelf_order;
  };

  o.save = function(d) {
    d.ascending = ascending;
    if (order_field) {
      d.order_field = order_field;
    }

    if (priority) {
      d.priority = priority;
    }

    return o;
  };

  o.restore = function(d) {
    if ('ascending' in d) {
      o.ascending(d.ascending);
    }

    if ('order_field' in d) {
      o.order_field(d.order_field);
    }

    if ('priority' in d) {
      o.priority(d.priority);
    }

    return o;
  };

  o.priority = function(_) {
    if (!arguments.length) return priority;
    priority = _;
    return o;
  };

  o.ascending = function(_) {
    if (!arguments.length) return ascending;
    ascending = _;
    return o;
  };

  o.order_field = function(_) {
    if (!arguments.length) return order_field;
    order_field = _;
    return o;
  };

  return o;
};

arc.report.shelf.shelves = function() {
  let data;
  let viz_lib;
  let shelves_dom_id;
  let dataset_events;
  let dataset;
  let dom_shelves;
  const dimagg_shelf_ids = [];
  let shelves_by_id;
  let $shelf_divs = $();
  let defaultInsertShelf;

  const shelves = function() { return 'arc.report.shelf.shelves'; };

  shelves.defaultInsertShelf = function() {
    if (!defaultInsertShelf) {
      if (dom_shelves && dom_shelves.length > 0) {
        return dom_shelves[0];
      }
    }

    return defaultInsertShelf;
  };

  shelves.dom_shelves = function() {
    return dom_shelves;
  };

  shelves.getShelf = function(shelf_name) {
    if (!dom_shelves) {
      return undefined;
    }

    let ret;
    dom_shelves.some((s) => {
      if (s.shelf_name() === shelf_name) {
        ret = s;
        return true;
      }

      return false;
    });

    return ret;
  };

  shelves.highlightDefaultShelf = function(shelf) {
    if (!shelf) {
      shelf = shelves.defaultInsertShelf();

      if (!shelf) {
        return;
      }
    }

    const $shelf = $(`#${shelves_dom_id}`)
                    .find(`#${shelf.dom_id()}`)
                    .closest('.selected-list-wrapper');

    if ($shelf.hasClass('default-shelf')) {
      return shelf;
    }

    $shelf_divs.removeClass('default-shelf');
    $shelf.addClass('default-shelf');
    defaultInsertShelf = shelf;
    return shelf;
  };

  shelves.repaintOrder = function() {
    const order_elements = shelves.getOrder();
    order_elements.sort((e1, e2) => {
      const a = e1.order();
      const b = e2.order();
      if (a.priority() === undefined) {
        if (b.priority() === undefined) {
          return 0;
        }

        return 1;
      }

      if (b.priority() === undefined) {
        if (a.priority() === undefined) {
          return 0;
        }

        return -1;
      }

      if (a.priority() === b.priority()) {
        return 0;
      }

      if (a.priority() < b.priority()) {
        return -1;
      }

      return 1;
    });

    order_elements.forEach((d, i) => {
      d.order().priority(i + 1);
    });

    d3.selectAll(`#${shelves_dom_id} .selection_field`)
      .call(arc.report.shelf.settext);
  };

  shelves.remove = function(shelf_dom_id, idx, unique_report_id) {
    const s = shelves_by_id[shelf_dom_id];
    const shelves_selector = typeof unique_report_id !== 'undefined' ?
                          `#app-shelves-${unique_report_id} ` : `#${shelves_dom_id} `;
    $(`${shelves_selector}#${shelf_dom_id} li:nth-child(${idx + 1})`).remove();
    s.remove(idx);
    return shelves;
  };

  /** This method is for emptying all non-filter shelves */
  shelves.emptyAll = function() {
    $(`#${shelves_dom_id}`).find('.selected-list').not(':last').empty();
    shelves.emptyContents();
    return shelves;
  };

  shelves.shelves_dom_id = function(_) {
    if (!arguments.length) return shelves_dom_id;
    shelves_dom_id = _;
    return shelves;
  };

  // Get and set the associated viz_lib for this set of shelves
  shelves.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return shelves;
  };

  shelves.dataset_events = function(_) {
    if (!arguments.length) return dataset_events;
    dataset_events = _;
    return shelves;
  };

  shelves.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return shelves;
  };

  shelves.shelves_dom_id = function(_) {
    if (!arguments.length) return shelves_dom_id;
    shelves_dom_id = _;
    return shelves;
  };

  shelves.data = function(_) {
    if (!arguments.length) return data;
    data = _;
    return shelves;
  };

  shelves.findByPersistentId = function(id) {
    let e;

    shelves.iterateElements((shelf, element) => {
      if (element.persistentId() === id) {
        e = element;
        return true;
      }
    });

    return e;
  };

  shelves.iterateElements = function(cb) {
    if (dom_shelves) {
      dom_shelves.some((shelf) => shelf.elements().some((element) => cb(shelf, element)));
    }

    return shelves;
  };

  shelves.getOrder = function() {
    const order_elements = [];

    // For analytic_builder_testing when there is no dom_shelve
    if (dom_shelves) {
      dom_shelves.forEach((shelf) => {
        shelf.getOrder().forEach((order_element) => {
          order_elements.push(order_element);
        });
      });
    }

    return order_elements;
  };

  function addSearch() {
    const shelf_html = ' ' +
      '<div class="input-group input-group-sm">' +
      '  <input type="text" class="form-control form-control-sm searchbox"' +
      '    placeholder="*:*"> ' +
      '  <div class="input-group-append">' +
      '    <span class="input-group-text searchbutton" role="button">' +
      '      <span class="fa fa-search"></span>' +
      '    </span>' +
      '  </div>' +
      '</div>';

    $(`#${shelves_dom_id}`).append(shelf_html);
  }

  function isSolr(dataconnection_type) {
    if (!arc.dataconnection) {
      return false;
    }

    if (!arc.dataconnection.connectionTypeEnum) {
      return false;
    }

    return (dataconnection_type === 'solr');
  }

  function isElasticsearch(dataconnection_type) {
    if (!arc.dataconnection) {
      return false;
    }

    if (!arc.dataconnection.connectionTypeEnum) {
      return false;
    }

    return (dataconnection_type === 'elasticsearch');
  }

  // Prevent adding a dataset element that include an aggregation onto
  // the dimensions shelf.
  // DJC -- perhaps we should display these elements somehow
  function _isValidMove(shelfType, shelfElement, isSensitiveShelf) {
    if (shelfType === arc.report.columnTypeEnum.DIMENSION) {
      if (shelfElement.expr_hasagg()) {
        arc.report.alertMessageSmall(
          'Cannot move an aggregate onto a dimension shelf',
          'alert-danger',
          true
        );

        return false;
      }

      if (shelfElement.fn_name() &&
          shelfElement.fn_type() === arc.report.fn_types.AGGREGATE) {
        shelfElement.fn_type(null)
                    .fn_name(null);
      }
    }

    const conn = arc.report.shelf.CONNECTION_TYPE;
    const dctype = conn.connection_type;

    if (shelfType === arc.report.columnTypeEnum.AGGREGATE) {
      if (dctype === 'solr' && ['string', 'boolean'].includes(shelfElement.dataset_coltype()
        .toLowerCase())) {
        arc.report.alertMessageSmall(
          'Aggregates on this field type is not currently supported',
          'alert-danger',
          true
        );

        return false;
      }
    }

    if (isSensitiveShelf && shelfElement.isSensitiveElement()) {
      arc.report.alertMessageSmall(
        'Cannot move a sensitive field onto a sensitive shelf',
        'alert-danger',
        true
      );

      return false;
    }

    return true;
  }

  shelves.setup = function() {
    dom_shelves = [];
    shelves_by_id = {};
    data.forEach((d) => {
      const s = Shelf().shelves(shelves)
                     .shelf_name(d.name)
                     .dom_id(d.name)
                     .display_html(d.display_html ? d.display_html : '')
                     .display_name(d.display_name)
                     .type(d.shelf_type)
                     .drillShelf(d.drillShelf)
                     .geoShelf(d.geoShelf)
                     .shelf_warning(d.shelf_warning)
                     .isSensitive(d.isSensitive)
                     .onShelfListChange(d.onShelfListChange)
                     .property(d.column_type);
      dom_shelves.push(s);
      shelves_by_id[d.name] = s;
    });

    $(`#${shelves_dom_id}`).find('div').remove();

    let dataconnection_type;
    const ds = viz_lib.dataset();
    if (ds) {
      dataconnection_type = ds.dataconnection_type();
      set_connection_type(dataconnection_type);
    }

    if (isSolr(dataconnection_type) ||
        isElasticsearch(dataconnection_type)) {
      addSearch();
    }

    const rows =
      d3.select(`#${shelves_dom_id}`)
        .selectAll('.nosel')
        .data(dom_shelves)
        .enter()
        .append('div')
        .attr('id', (d) => `wrp-${d.dom_id()}`);

    const shelfLabelDivs = rows.append('div')
                             .classed('shelf-label', true)
                             .html((d) => d.display_html());

    if (arc.globals.is_edit) {
      shelfLabelDivs.on('click', shelves.highlightDefaultShelf);
    }

    shelfLabelDivs.append('span')
                  .classed('shelf-description', true)
                  .attr('id', (d) => `${d.dom_id()}_popover`)
                  .html((d) => d.display_name());

    const selectedListWrapperDivs = rows.append('div')
                                        .append('div')
                                        .classed('selected-list-wrapper', true);
    if (arc.globals.is_edit) {
      selectedListWrapperDivs.on('click', shelves.highlightDefaultShelf);
    }

    selectedListWrapperDivs.append('ul')
                           .attr('id', (d) => d.dom_id())
                           .attr('data-test-id', (d) => `${d.dom_id()}_list`)
                           .classed('selected-list', true)
                           .classed('ui-sortable', true);

    let events_available = false;
    const shelfAddons = $('#dropdown-base').html();

    dom_shelves.forEach((d) => {
      const this_shelf = d;
      const shelf_type = d.type();
      const shelf_property = d.property();
      const geoShelf = d.geoShelf();
      const isSensitiveShelf = d.isSensitive();

      if (d.shelf_warning()) {
        d3.select(`#wrp-${d.dom_id()} .shelf-label`).append('span')
                                                    .classed('shelf-warning fa fa-warning ml-2', true)
                                                    .attr('id', (x) => `${x.dom_id()}_popover`);
        }

      function updateShelfElements() {
        const elements = [];

        d3.select(`#${shelves_dom_id}`)
          .selectAll(`#${d.dom_id()}`)
          .selectAll('.selection_field')
          .each(function(d_elem) {
            elements.push(d_elem);
            d_elem.d3_elem(d3.select(this));
          });

        this_shelf.elements(elements);
      }

      if (arc.globals.is_edit) {
        $(`#${shelves_dom_id}`).find(`#${d.dom_id()}`).sortable({
          // Make element slightly easier to sort to the first and last position
          tolerance: 'pointer',

          // allow elements to be dragged across shelves
          connectWith: $('.selected-list'),

          // Don't move the element if only the submenu or any of its items are dragged
          cancel: '.submenu',

          start(event, ui) {
            d.elementDragStart(ui.item);
          },

          stop(event, ui) {
            // If 'stop' is called before 'update' or 'remove', then
            // see if we should trigger a removal
            const item = d.elementDragStart();
            if (item) {
              const {
                originalPosition,
                position,
              } = ui;

              // The user has to drag something at least 70 pixels away
              if ((Math.abs(originalPosition.top - position.top) < 70) &&
                (Math.abs(originalPosition.left - position.left) < 70)) {
                return;
              }

              const shelf_elem = getDatum(item.find('.selection_field'));
              if (shelf_elem) {
                this_shelf.removeElement(shelf_elem);
                item.remove();
              }
            }

            updateShelfElements();
          },

          update() {
            pageNeedsSave();
            d.elementDragStart(undefined);
          },

          remove(event, ui) {
            d.elementDragStart(undefined);
            const shelf_elem = getDatum(ui.item.find('.selection_field'));

            if (shelf_elem) {
              this_shelf.removeElement(shelf_elem);
              updateShelfElements();
            }

            handleColorSettingsOnColorShelfElementChange(this_shelf);
          },

          // add data and dropdown when item is dropped
          receive(event, ui) {
            shelves.highlightDefaultShelf(this_shelf);

            // jquery does not copy over the data elements. We need to manually
            // copy over the data that used to be in the original element
            let shelf_elem = getDatum($(ui.item));

            // define shelf_elem for when element was dragged across shelves
            if (!shelf_elem) {
              shelf_elem = getDatum(ui.item.find('.selection_field'));
            }

            const shelf_id = $(this).attr('id');

            let new_elem =
              d3.select('.selected-list .selection_element')
                .classed({
                  'app-shelf-item': true,
                  selection_element: false
                })
                .attr('data-test-id', 'app-shelf-item');

            // Define new_elem if element was dragged across shelves
            if (!new_elem.node()) {
              ui.item.toggleClass('moving-item');
              new_elem = d3.select('.selected-list .moving-item');
              ui.item.toggleClass('moving-item');
            }

            // remove any partition marks
            new_elem.select('.partition-key')
              .remove();

            new_elem.select('.dropdown-placeholder')
              .html(() => shelfAddons);

            // Hierarchy elements that are dragged in need to be cleaned up a bit
            new_elem.each(function () {
              const $this = $(this);

              // Remove hierarchy elements
              $this.find('> ul')
                .remove();

              $this.find('> .field-wrapper > a')
                .remove();

              const $hierarchy = $this.find('.field-wrapper');
              if ($hierarchy.length > 0) {
                const h = $hierarchy.html();
                $this.html(h);
              }
            });

            const shelfType = d3.select(this).datum().property();
            if (!_isValidMove(shelfType, shelf_elem, isSensitiveShelf)) {
              const sender = $(ui.sender);
              if (sender.hasClass('ui-sortable')) {
                $(ui.sender).sortable('cancel');
              } else {
                new_elem.remove();
              }

              return;
            }

            const primaryReport = arc.globals.reports.getPrimaryReport();
            // The shelf has been changed, old history is no longer relevant
            if (primaryReport) {
              primaryReport.clearShelfHistory();
            }

            // Reset trellis warning for large trellises when shelf config changes
            resetAllowLargeTrellis(viz_lib);

            if (geoShelf) {
              const liItem = d3.select(new_elem.select('.selection_field').node().parentNode);

              if (liItem.selectAll('.geo-icon').size() < 1) {
                liItem
                  .insert('span', ':first-child')
                  .classed('geo-icon', true);
              }
            } else {
              new_elem.selectAll('.geo-icon').remove();
            }

            // if the colname of the shelf element in the geo shelf is a direct
            // match with a geotype, auto cast it.
            autocastGeoType(shelf_elem);

            const se = arc.report.shelfElementPostProcess({
              shelf_elem,
              shelf_type,
              shelf_property,
              shelf_id,
              tgtshelf_is_aggregate:
                ($(ui.helper).attr('data-srcshelf') === '#visual-data-panel .agg-app-params'),
            });

            // Element is being dragged off the shelf
            if (new_elem.size() === 0) {
              return;
            }

            const d3_elem =
              new_elem
                .select('.selection_field');

            se.d3_elem(d3_elem);

            d3_elem.datum(() => {
              // Note that the 'datum' function cannot be passed in
              // a function ('shelf_elem'). It presumes that a function is
              // to be invoked, not stored. So we instead create an anonymous
              // function that returns the function object we want to store
              // instead.

              this_shelf.addElement(se);
              cleanupDropdowns(this_shelf, new_elem, se);
              return se;
            })
              .call(arc.report.shelf.settext);

            if (se.isDatasetSegment()) {
              if (shelf_id !== 'filters_shelf') {
                setupSegmentModal(se, d3_elem);
              } else {
                setupFilterModal(se, d3_elem);
              }

              updateShelfElements();
              return;
            }

            shelf_elem.setupShelfElementBackgroundColor();
            switch (shelf_id) {
              case 'filters_shelf':
                {
                  // Select the dragged element and remove selection from the other elements.
                  const $selectedListItem = $(d3_elem.node().parentNode);
                  $selectedListItem.addClass('selected');
                  $selectedListItem.siblings('li').removeClass('selected');

                  /*
                  * If we dragged a new element in the filters shelf, bring up
                  * the selection modal
                  */
                  if (!se.expressionIsAggregate()) {
                    setupFilterModal(se, d3_elem);
                  }
                }

                break;
              case 'color_shelf':
                handleColorSettingsOnColorShelfElementChange(this_shelf);
                break;
              default:
                // nothing to do
            }

            updateShelfElements();
            if (primaryReport && primaryReport.viz()) {
              primaryReport.viz().saveAwayShelfInfo();
            }
          },
        }).droppable({ greedy: true });
      }

      if (d.type() === arc.report.shelf_types.EVENT_FLOW ||
          d.type() === arc.report.shelf_types.EVENT_FUNNEL) {
        events_available = true;
      } else {
        dimagg_shelf_ids.push(`#${d.dom_id()}`);
      }
    });

    if (dom_shelves && dom_shelves.length > 0) {
      shelves.setup_dimagg_draggable();
    }

    if (events_available && arc.globals.is_edit) {
      ['events-avail-list'].forEach((src) => {
        $(`.${src} .selection_element`).draggable({
          cursor: 'pointer',
          connectToSortable: '#events_path_shelf',
          helper: 'clone',

          // it is not necessary to append to the shelf list, but need to appendTo the body
          // to resolve issue of the field going behind the sidenav
          // appendTo: 'ul' + dimagg_shelf_ids[0], // fixes the problem with li
                                                   // in a fixed position parent
          appendTo: 'body',
          opacity: 0.5,
          zIndex: 1040,

          // highlight the shelf that is being dragged onto
          drag() {
            $('#events_path_shelf').parent().addClass('shelf-highlight');
          },

          start(event, ui) {
            // When dragging into shelves, the 'sortable' jquery ui helper
            // adds in a ui-sortable-helper class and sets the item width.
            // It looks like this class does not deal with fractions in the
            // width by rounding up - so just set the width at the start of
            // the drag to be 1 larger than the current width
            ui.helper.width($(ui.helper).width() + 1);
            ui.helper.attr('data-srcshelf', src);
            ui.helper.addClass('field-drag-helper');
          },

          // unhighlight shelves when drag completes
          stop(event, ui) {
            $('#events_path_shelf').parent().removeClass('shelf-highlight');
            ui.helper.removeClass('field-drag-helper');
            ui.helper.removeAttr('style');
          }

        });
      });
    }

    // enables 'save as segment' on Filters
    const $shelves = $(`#${shelves_dom_id}`);
    arc.misc.updateDivForPerms(shelves.dataset()?.dataset_perms(), 'dataset', $shelves);
    $shelf_divs = $shelves.find('.selected-list-wrapper');

    return shelves;
  };

  shelves.setup_dimagg_draggable = function() {
    ['#visual-data-panel .dim-app-params', '#visual-data-panel .agg-app-params']
        .forEach((src) => {
      /*
       * If the user clicks on a dimension or an aggregate, we add it to the last
       * referenced shelf. The click behavior below accomplishes that.
       */
      $(`${src} .selection_element`)
        .filter((index, elem) => !$(elem).attr('cdt-name'))
        .off('click.addshelf')
        .on('click.addshelf', _addShelf);

      function _addShelf(evt) {
        const report = arc.report.getPrimaryReport();
        if (!report || !report.viz()) {
          return;
        }

        const reportShelves = report.viz().shelves();
        const $current = $(evt.currentTarget);

        if ($current.draggable('option', 'disabled')) {
          // do not add if there are no segments
          return;
        }

        const $eventTarget = $(evt.target);
        if ($eventTarget.hasClass('dropdown-toggle') ||
            ($eventTarget.parent('.dropdown-menu, .dropdown-toggle').length > 0)) {
          // Within derived data shelves, if a dropdown is triggered or
          // a dropdown selection is picked, do not trigger an add
          return;
        }

        const shelf_elem = getDatum($current).clone();
        shelf_elem.setupShelfElementBackgroundColor();
        const shelf = reportShelves.defaultInsertShelf();
        if (!_isValidMove(shelf.property(), shelf_elem, shelf.isSensitive())) {
          return;
        }

        // The shelf has been changed, old history is no longer relevant
        if (report) {
          report.clearShelfHistory();
        }

        pageNeedsSave();

        const shelf_id = shelf.shelf_name();
        const se = arc.report.shelfElementPostProcess({
          shelf_elem,
          shelf_type: shelf.type(),
          shelf_property: shelf.property(),
          shelf_id: shelf.shelf_name(),
          tgtshelf_is_aggregate: $current.closest('.app-params').hasClass('agg-app-params'),
        });

        autocastGeoType(se);

        const se_d = {};
        se.save(se_d);
        reportShelves.add(shelf_id, [se_d], true);
        handleColorSettingsOnColorShelfElementChange(shelf);
      }

      $(`${src} .selection_element`).filter((index, elem) => $(elem).attr('cdt-name'))
                                     .off()
                                     .on('click', '.expand-icon', function (e) {
        const $this = $(this);
        $this.toggleClass('fa-angle-down fa-angle-right');
        $this.closest('li').find('> ul').slideToggle();
        e.preventDefault();
        e.stopPropagation();
      });

      $(`${src} .selection_element`).filter((index, elem) => !$(elem).attr('cdt-name'))
                                     .draggable({
        cursor: 'pointer',
        connectToSortable: dimagg_shelf_ids.join(', '),
        helper: 'clone',

        // it is not necessary to append to the shelf list, but need to appendTo the body
        // to resolve issue of the field going behind the sidenav
        // appendTo: 'ul' + dimagg_shelf_ids[0], // fixes the problem with li
                                                 // in a fixed position parent
        appendTo: 'body',
        opacity: 0.5,
        zIndex: 1040,

        // highlight the shelf that is being dragged onto
        drag() {
          $(dimagg_shelf_ids.join(', ')).parent().addClass('shelf-highlight');
        },

        // unhighlight shelves when drag completes
        stop(event, ui) {
          $(dimagg_shelf_ids.join(', ')).parent().removeClass('shelf-highlight');
          ui.helper.removeClass('field-drag-helper');
          ui.helper.removeAttr('style');
        },

        start(event, ui) {
          // When dragging into shelves, the 'sortable' jquery ui helper
          // adds in a ui-sortable-helper class and sets the item width.
          // It looks like this class does not deal with fractions in the
          // width by rounding up - so just set the width at the start of
          // the drag to be 1 larger than the current width.
          // And also, set hierarchy element to be in collapsed state.
          const $uiHelper = ui.helper;
          const roundUpWidth = 1;
          $uiHelper.width(`${$uiHelper.width() + roundUpWidth}px`);
          $uiHelper.attr('data-srcshelf', src);
          $uiHelper.addClass('field-drag-helper');

          // TODO: Simplify code below to be done through CSS
          // Cannot be done currently because $().slideToggle() sets inline style
          if ($uiHelper.hasClass('hierarchy')) {
            const $ul = $uiHelper.find('ul');
            if ($ul.length > 0) {
              if ($ul.css('display') === 'block') {
                $ul.hide();
              }
            }
          }
        }
      });
    });

    // Added checks for reporttracker and dataset_segments since the unit tests
    // shouldn't need to build these objects for their test shelves.
    if (arc.globals.reports) {
      const r = arc.globals.reports.getPrimaryReport();
      const dataset_segments = r.dataset_segments();
      if (dataset_segments) {
        dataset_segments.updateSegmentDimElem(r.viz());
      }
    }
  };

  shelves.createNewElements = function(shelf_id, shelf_contents) {
    if (!shelf_contents) {
      return undefined;
    }

    if (shelf_contents.length === 0) {
      return undefined;
    }

    const s = shelves_by_id[shelf_id];
    if (!s) {
      // eslint-disable-next-line no-console
      console.log(`No shelf with id ${shelf_id}`);
      return undefined;
    }

    const elements = [];

    shelf_contents.forEach((d) => {
      const e = arc.report.shelf.element()
                                .restore(d, dataset_events, dataset)
                                .shelf(s);

      const drillShelf = shelves_by_id.drill_shelf;
      if (d.source_pivot && drillShelf && shelf_id === 'geo_shelf') {
        e.geo_cast(getGeoTypeFromDrillShelf(d, drillShelf));
      }

      elements.push(e);
    });

    return elements;
  };

  function getGeoTypeFromDrillShelf(element_data, drillShelf) {
    let geo_cast;
    const de = drillShelf.elements();
    const drillElemMatches = de.filter((e) => e.dataset_colname() === element_data.dataset_colname);

    if (drillElemMatches.length) {
      geo_cast = drillElemMatches[0].geo_cast();
    }

    return geo_cast;
  }

  function makeDataConnectionSpecificChanges(elem, dctype) {
    elem.selectAll('[data-conn]')
        .classed('arc-invisible', function() {
          const c = $(this).attr('data-conn').split(' ');
          if (c.indexOf(dctype) >= 0) {
            return false;
          }

          return true;
        });

    elem.selectAll('[data-conn="arcengine"]')
        .classed('arc-invisible', !arc.dataconnection.isArcEngineCompatible(dctype));

    elem.selectAll('[data-conn-exclude]')
        .classed('arc-invisible', function() {
          const $this = $(this);
          const c = $this.attr('data-conn-exclude').split(' ');
          if (c.indexOf(dctype) >= 0) {
            return true;
          }

          return false;
        });
  }

  shelves.add = function(shelf_id, shelf_contents, saveIntoVisual) {
    const elements = shelves.createNewElements(shelf_id, shelf_contents);
    if (elements === undefined || shelves_dom_id === undefined) {
      return shelves;
    }

    const s = shelves_by_id[shelf_id];
    const shelf_type = s.type();
    s.addElements(elements);

    const shelfAddons = $('#dropdown-base').html();
    const lis =
      d3.select(`#${shelves_dom_id}`)
        .select(`#${shelf_id}`)
        .selectAll('.app-shelf-item')
        .data(s.elements())
        .enter()
          .append('li')
          .classed('dropdown', true)
          .classed('app-shelf-item', true)
          .attr('data-test-id', `${shelf_id}-app-shelf-item`)
          .each(function(d) {
            d.d3_elem(d3.select(this));
          });

    // Also add type span if measure shelf in boxplot or shelf type is filter
   if ([arc.report.shelf_types.COLUMN, arc.report.shelf_types.PREDICATE].includes(shelf_type) ||
        (shelves.viz_lib() && shelves.viz_lib().viz_type() === 'boxplot')) {
      if (s.geoShelf()) {
        lis.append('span')
           .classed('geo-icon', true);
      }

      lis.append('span')
         .attr('class', (d) => `dim-icon-type ${d.dataset_coltype_class()}`);
    }

    lis.append('span')
         .classed('selection_field', true)
         .call(arc.report.shelf.settext);

    lis.append('span')
      .classed('dropdown-placeholder', true)
      .html(() => shelfAddons);

    makeDataConnectionSpecificChanges(lis, viz_lib.dataconnection_type());

    // Hide irrelevant dropdown menus
    if (lis[0] && lis[0].length !== 0) {
      cleanupDropdowns(s, lis);
    }

    lis.each((d) => {
          autocastGeoType(d);
        });

    if (saveIntoVisual) {
      const report = arc.report.getPrimaryReport();
      if (report && report.viz()) {
        report.viz().saveAwayShelfInfo();
      }
    }

    return shelves;
  };

  shelves.set = function(shelf_id, shelf_contents) {
    const s = shelves_by_id[shelf_id];
    if (!s) {
      // eslint-disable-next-line no-console
      console.log(`No shelf with id ${shelf_id}`);
      return;
    }

    s.clearElements();
    d3.select(`#${shelves_dom_id}`)
      .select(`#${shelf_id}`)
      .selectAll('.app-shelf-item')
      .remove();

    if (!shelf_contents) {
      return;
    }

    return shelves.add(shelf_id, shelf_contents);
  };

  shelves.destroy = function() {
    if (arc.globals.is_edit) {
      // TODO: what do we do with .super-modal
      $('div.super-modal').droppable('destroy');
    }

    ['#visual-data-panel .dim-app-params',
     '#visual-data-panel .agg-app-params', '.events-avail-list'].forEach((src) => {
        $(`${src} .selection_element`).draggable('destroy');
    });

    dom_shelves.forEach((d) => {
      const $dom_id = $(`#${d.dom_id()}`);
      $dom_id.sortable('destroy');
      $dom_id.droppable('destroy');
    });

    $shelf_divs = $();
  };

  shelves.emptyContents = function() {
    dom_shelves.forEach((shelf) => {
      if (shelf.display_name() !== 'Filters') {
        shelf.clearElements();
      }
    });
  };

  return shelves;
};

  /**
   * Handles the visibility of the Custom Colors for Measure Ranges setting based on the value
   * of the field set in the Colors shelf.
   * A color range can only be set if the value is numerical.
   * The Custom Colors for Measure Ranges setting should:
   *  - be shown when the Color shelf contans measures
   *  - be hidden when the Color shelf contains dimensions
   *  - only appear for the visual types that supprot it
   */
  function handleColorRangesSettingVisibility(viz, col) {
    const isMeasure = col?.attr('isdim') === false;
    const measureRangesSetting = viz
      .user_settings()
      .find_by_id('measure_ranges');

    if (!measureRangesSetting) {
      return;
    }

    if (isMeasure) {
      measureRangesSetting.show();
    } else {
      measureRangesSetting.hide();
    }
  }

  function handleUseDatasetDefinedColorSetting(shelf, viz, col) {
    // Don't draw setting if it is crosstab or table visual
    if (tabularVizTypes.has(viz.viz_type())) {
      return;
    }

    const colors = col && col.attr('colors');
    const useDatasetDefinedColorStyleSetting = viz
      .user_settings()
      .find_by_id('use_dataset_color_settings');

    if (!useDatasetDefinedColorStyleSetting) {
      return;
    }

    const setDatasetDefinedColorSetting =
      shelf.elements().length === 1 && colors?.customColorsEnabled;
    if (useDatasetDefinedColorStyleSetting && setDatasetDefinedColorSetting) {
        useDatasetDefinedColorStyleSetting.value('true');
    } else {
      useDatasetDefinedColorStyleSetting.value('false');
    }
  }

  function handleColorSettingsOnColorShelfElementChange(shelf) {
    if (shelf.shelf_name() !== 'color_shelf') {
      return;
    }

    const report = arc.report.getPrimaryReport();
    if (!report) {
      return;
    }

    const viz = report.viz();
    if (!viz) {
      return;
    }

    const colorElement = shelf.elements()[0];
    const col = colorElement && colorElement.dataset().findColumn(colorElement.dataset_colname());

    handleColorRangesSettingVisibility(viz, col);
    handleUseDatasetDefinedColorSetting(shelf, viz, col);
  }

function Shelf() {
  let display_name;
  let display_html;
  let shelves;
  let shelf_name;
  let dom_id;
  let type;
  let property;
  let drillShelf;
  let geoShelf;
  let shelf_warning; // bool
  let isSensitive;
  let elements = [];
  let onShelfListChange;
  const shelf = function () { return 'Shelf'; };

  shelf.drillShelf = function(_) {
    if (!arguments.length) return drillShelf;
    drillShelf = _;
    return shelf;
  };

  shelf.geoShelf = function(_) {
    if (!arguments.length) return geoShelf;
    geoShelf = _;
    return shelf;
  };

  shelf.shelf_warning = function(_) {
    if (!arguments.length) return shelf_warning;
    shelf_warning = _;
    return shelf;
  };

  shelf.isSensitive = function(_) {
    if (!arguments.length) return isSensitive;
    isSensitive = _;
    return shelf;
  };

  shelf.shelf_name = function(_) {
    if (!arguments.length) return shelf_name;
    shelf_name = _;
    return shelf;
  };

  shelf.remove = function (idx) {
    elements.splice(idx, 1);
    const report = arc.report.getPrimaryReport();
    if (report) {
      report.clearShelfHistory();
    }

    shelf.onShelfListChange(elements);
    return shelf;
  };

  shelf.shelves = function(_) {
    if (!arguments.length) return shelves;
    shelves = _;
    return shelf;
  };

  // Fetch the associated viz_lib for this shelf
  shelf.viz_lib = function() {
    return shelves.viz_lib();
  };

  shelf.display_name = function(_) {
    if (!arguments.length) return display_name;
    display_name = _;
    return shelf;
  };

  shelf.display_html = function(_) {
    if (!arguments.length) return display_html;
    display_html = _;
    return shelf;
  };

  shelf.onShelfListChange = function (_) {
    if (!arguments.length) return onShelfListChange;

    if (typeof (_) === 'function') {
      onShelfListChange = _;
    }

    if (typeof (_) === 'object') {
      onShelfListChange?.(_);
    }

    return shelf;
  };

  shelf.dom_id = function(_) {
    if (!arguments.length) return dom_id;
    dom_id = _;
    return shelf;
  };

  shelf.type = function(_) {
    if (!arguments.length) return type;
    type = _;
    return shelf;
  };

  //
  // property should be one of:
  //   arc.report.columnTypeEnum.DIMENSION
  //   arc.report.columnTypeEnum.AGGREGATE
  //   arc.report.columnTypeEnum.UNKNOWN
  //
  shelf.property = function(_) {
    if (!arguments.length) return property;
    property = _;
    return shelf;
  };

  let elementDragStart;

  shelf.elementDragStart = function(_) {
    if (!arguments.length) return elementDragStart;
    elementDragStart = _;
    return shelf;
  };

  shelf.elements = function(_) {
    if (!arguments.length) return elements;
    elements = _;
    return shelf;
  };

  shelf.clearElements = function() {
    elements = [];
    shelf.onShelfListChange(elements);
  };

   // to avoid to many onShelfListChange when we are setting/resetting shelf
   shelf.addElements = function(newElements) {
    elements.forEach((element) => {
      element.shelf(shelf);
    });

    elements.push(...newElements);
    shelf.onShelfListChange(elements);
    return shelf;
  };

  shelf.addElement = function(element) {
    element.shelf(shelf);
    elements.push(element);
    shelf.onShelfListChange(elements);
    return shelf;
  };

  shelf.removeElement = function(element) {
    let to_remove;

    elements.some((d, i) => {
      if (d === element) {
        to_remove = i;
        return true;
      }

      return false;
    });

    if (to_remove !== undefined) {
      shelf.remove(to_remove);
    }

    return shelf;
  };

  shelf.getOrder = function() {
    const order_elements = [];

    elements.forEach((element) => {
      if (element.order()) {
        order_elements.push(element);
      }
    });

    return order_elements;
  };

  return shelf;
}

// Pass in dataset after creation, needed for element.expressionIsAggregate
arc.report.shelf.record_count_element = function(dataconnection_type) {
  const e = arc.report.shelf.element();
  const expr = (dataconnection_type === 'ksql' ||
              dataconnection_type === 'elasticsearch' ||
              dataconnection_type === 'cassandra') ? 'count(*)' : 'sum(1)';
  e.col_alias('Record Count')
   .expr_hasagg(true)
   .dataset_colname('Record Count')
   .dataset_coltype('BIGINT')
   .custom_expr(expr);

  return e;
};

arc.report.shelf.full_text_score_element = function(dataconnection_type) {
  if (dataconnection_type === 'elasticsearch') {
    const e = arc.report.shelf.element();
    const expr = 'score()';
    e.col_alias('Search Score')
     .expr_hasagg(false)
     .dataset_colname('Search Score')
     .dataset_coltype('DOUBLE')
     .custom_expr(expr);

    return e;
  }
};

arc.report.shelf.segment_element = function() {
  const e = arc.report.shelf.element();

  e.col_alias(arc.report.segment_display_name)
   .dataset_colname(arc.report.segment_name)
   .dataset_coltype('STRING')
   .isDatasetSegment(true);

  return e;
};

// Reverse parameter to undo formatting from arc.params.format_list for url parameters
// TODO: Temporary fix for ARC-6207, should undo with parameter refactor
//       and instead store the original filter_value in params so don't need to recalculate
// @param reverse - It should always be false. Keeping the argument for Arches.
// @param conn_type - As of Arches, is passed only for multi-select values and custom filter aliases
// @param forceQuoteURLParam - true only for multi select string params,
//                             because they need to be quoted when generating URL
arc.report.shelf.parse_filter_value = function(filter_value, coltype, reverse, conn_type,
                                               forceQuoteURLParam) {
  coltype = String(coltype).toLowerCase();
  if ((coltype === 'string') ||
      (coltype === 'timestamp')) {
    // Note that string columns that only contain numeric values are, at times,
    // converted to numeric values by some visuals. Convert these back to
    // strings.
    if (!conn_type) {
     conn_type = arc.report.shelf.CONNECTION_TYPE &&
       arc.report.shelf.CONNECTION_TYPE.connection_type;
    }

    switch (conn_type) {
      case 'postgres':
      case 'sqlite':
      case 'vertica':
      case 'snowflake':
        return reverse ? String(filter_value).replace(/^'|'$/g, '').replace(/''/g, "'") :
               `'${String(filter_value).replace(/'/g, "''")}'`;

      case 'url':

        // Used for Get URL Params, Email, etc.
        return forceQuoteURLParam ? `'${filter_value}'` : filter_value;
      default:

        // Used for 'oracle' and other data connections
        return reverse ? String(filter_value).replace(/^'|'$/g, '').replace(/\\'/g, "'") :
               `'${String(filter_value).replace(/'/g, "\\'")}'`;
    }
  } else if (arc.misc.isNumeric(filter_value)) {
    filter_value = Number(filter_value);
  }

  return filter_value;
};

arc.report.shelf.element = function() {
  /*
   * shelf_type: Type of shelf
   *     Domain: arc.report.shelf_types
   *
   * DJC -- Dataset tablename and uniqname are a function of the dataset and will need
   *        to be recomputed if the report is retargetted to a different dataset.
   *   -->  Might be better to recompute these items each time a report is loaded
   *        via some sort of 'setupReportForDataset()' function
   *
   * dataset_colname: Dataset column name this shelf was initially based on
   *
   * dataset_coltype: Column type of the original dataset column
   *     Domain: arc.report.column_types
   *
   * col_alias: Column alias
   *
   * custom_expr: Entered custom expression
   *
   * fn_cast: User-specified cast that changes the output type of this column
   *     Domain: arc.report.column_types
   *
   * fn_type: User-specified function type - aggregate or scalar
   *     Domain: arc.report.fn_types
   *
   * fn_name: Name of the function applied. Only set when fn_type is set
   *
   * geo_cast: A cast on the geo type for a shelf element. Shelf elements can be
   *           marked as latitude, longitude, country, state, county, DMA, or zip
   *
   * shelf_date_function: FOR DISPLAY ONLY, this is the clicked-on date shelf element
   */

  let shelf;
  let dataconnection_type;
  let dataset;

  // Refers persistently to a particular shelf element.
  // Use in conditional aggregation (and possible other places in the future)
  // to refer to an external shelf element reliably.
  let persistentId;

  // Column alias
  let col_alias;

  // Column table position
  let coldisplaybefore;

  // Column description
  let col_desc;

  // Events for flow/funnel analysis
  let event_element;
  let event_min_count;
  let event_max_count;
  let event_goal;

  // topk
  let order_limit;
  let order_limit_granular;

  // Predicate information for filters
  let filter_column;
  let filter_exclude;
  let filter_data = [];

  let filter_input = {};

  // Information for standard columns
  let dataset_colname;
  let dataset_coltype;
  let dataset_colcomment;
  let dataset_isdim;
  let dataset_hivecomment;
  let dataset_dimhlist;
  let dataset_dimhvals = [];
  let dataset_dimhlevel = 0;
  let dataset_geotype;
  let isSensitiveElement;
  let dflagg;
  let expr_hasagg;
  let custom_expr;
  let fn_cast;
  let fn_type;
  let fn_name;
  let order;
  let geo_cast;
  let display_format; // inherited, but can be overridden for a particular visual
  let source_pivot;
  let shelf_date_function;

  // Data used for autotime for date columns. The min and max are refetched
  // on builder draw or dataset switch. Possible values of autotime_override
  // are "AUTO", null, or some value that the user has entered in.
  let autotime_override = 'AUTO';

  // Control background colors & bars in tables & crosstabs
  let displayBackgroundColor;
  let backgroundColor;

  // Setup auto time bucketing
  let autotimeEnabled;

  // Control how elements are displayed to the user
  let dataTransformation;
  let analyticalFunctionData;
  let analyticalFunctionSelection;

  // d3_elem is the last drawn vusla element for this shelf element
  let d3_elem;

  let conditionalAgg;
  let enableExpansion;

  let isDatasetSegment = false;

  let secondaryaxis;

  const element = function() { return 'arc.report.shelf.element'; };

  element.d3_elem = function(_) {
    if (!arguments.length) return d3_elem;
    d3_elem = _;
    return element;
  };

  element.setShelfFnNameFromShelfId = function(shelf_id, conn, selected = true) {
    if (!selected) {
      element.fn_type(null)
             .fn_name(null);
    } else if (shelf_id === 'shelf-agg-sum') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.sum);
    } else if (shelf_id === 'shelf-agg-count') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.count);
    } else if (shelf_id === 'shelf-agg-missing') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.missing);
    } else if (shelf_id === 'shelf-agg-ndv') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.ndv);
    } else if (shelf_id === 'shelf-agg-countdistinct') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name('countdistinct');
    } else if (shelf_id === 'shelf-agg-min') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.min);
    } else if (shelf_id === 'shelf-agg-max') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.max);
    } else if (shelf_id === 'shelf-agg-avg') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.avg);
    } else if (shelf_id === 'shelf-agg-sumofsquares') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.sumofsquares);
    } else if (shelf_id === 'shelf-agg-appx-median') {
      if (conn.expr_appxmedian) {
        element.custom_expr(conn.expr_appxmedian(dataset_colname))
               .col_alias(`appx_median of ${dataset_colname}`);
      } else {
        element.fn_type(arc.report.fn_types.AGGREGATE)
               .fn_name(conn.appx_median);
      }
    } else if (shelf_id === 'shelf-agg-stddev') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.stddev);
    } else if (shelf_id === 'shelf-agg-stddev-pop') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.stddev_pop);
    } else if (shelf_id === 'shelf-agg-variance') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.variance);
    } else if (shelf_id === 'shelf-agg-variance-pop') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.variance_pop);
    } else if (shelf_id === 'shelf-agg-aggregate') {
      element.fn_type(arc.report.fn_types.AGGREGATE)
             .fn_name(conn.aggregate);
    } else if (shelf_id === 'shelf-agg-groupconcat') {
      if (conn && conn.expr_groupconcat) {
        element.clearCustomExpr();
        element.custom_expr(conn.expr_groupconcat(dataset_colname))
               .col_alias('groupconcat');
      } else {
        element.fn_type(arc.report.fn_types.AGGREGATE)
               .fn_name(conn.group_concat);
      }
    }
  };

  element.isSegment = function() {
    return isDatasetSegment;
  };

  element.clearCustomExpr = function() {
    element.custom_expr(undefined);
    element.analyticalFunctionData(undefined);
    element.analyticalFunctionSelection(undefined);
    return element;
  };

  // Note that because we may re-target a visual to a different dataset, the
  // dataconnection_type information should not be saved/restored, but should
  // be re-created when the visual is being generated
  element.dataconnection_type = function(_) {
    if (!arguments.length) return dataconnection_type;
    dataconnection_type = _;
    return element;
  };

  // For remembering previous entries in analytical functions builder
  // Records the data from the analytical functions and
  element.analyticalFunctionData = function(_) {
    if (!arguments.length) return analyticalFunctionData;
    analyticalFunctionData = _;
    if (analyticalFunctionData === undefined) {
      analyticalFunctionSelection = undefined;
    }

    return element;
  };

  // For remembering previous entries in analytical functions builder
  // Records the last shelf selected
  element.analyticalFunctionSelection = function(_) {
    if (!arguments.length) return analyticalFunctionSelection;
    analyticalFunctionSelection = _;
    return element;
  };

  element.secondaryaxis = function(_) {
    if (!arguments.length) return secondaryaxis;
    secondaryaxis = _;
    return element;
  };

  element.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return element;
  };

  // Fetch the associated viz_lib for this shelf element
  element.viz_lib = function() {
    return shelf.viz_lib();
  };

  element.clone = function() {
    const e = arc.report.shelf.element()
               .shelf(shelf)
               .dataconnection_type(dataconnection_type)
               .persistentId(persistentId)
               .col_alias(col_alias)
               .coldisplaybefore(coldisplaybefore)
               .col_desc(col_desc)
               .event_element(event_element)
               .event_min_count(event_min_count)
               .event_max_count(event_max_count)
               .dataset(dataset)
               .dataset_colname(dataset_colname)
               .dataset_coltype(dataset_coltype)
               .dataset_colcomment(dataset_colcomment)
               .dataset_isdim(dataset_isdim)
               .dataset_geotype(dataset_geotype)
               .isSensitiveElement(isSensitiveElement)
               .dataset_hivecomment(dataset_hivecomment)
               .dataset_dimhlist(dataset_dimhlist)
               .dflagg(dflagg)
               .expr_hasagg(expr_hasagg)
               .filter_data(filter_data)
               .filter_exclude(filter_exclude)
               .filter_input(filter_input)
               .fn_cast(fn_cast)
               .fn_type(fn_type)
               .fn_name(fn_name)
               .geo_cast(geo_cast)
               .shelf_date_function(shelf_date_function)
               .autotime_override(autotime_override)
               .custom_expr(custom_expr)
               .displayBackgroundColor(displayBackgroundColor)
               .backgroundColor(backgroundColor)
               .autotimeEnabled(autotimeEnabled)
               .dataTransformation(dataTransformation)
               .display_format(display_format)
               .analyticalFunctionData(analyticalFunctionData)
               .analyticalFunctionSelection(analyticalFunctionSelection)
               .conditionalAgg(conditionalAgg)
               .enableExpansion(enableExpansion)
               .isDatasetSegment(isDatasetSegment)
               .secondaryaxis(secondaryaxis)
               .source_pivot(source_pivot);

    if (order) {
      e.order(order.clone());
    }

    return e;
  };

  element.setupShelfElementBackgroundColor = function(keepPrevious) {
    if (keepPrevious && element.backgroundColor()) {
      return;
    }


    const col = element.dataset().findColumn(element.dataset_colname());
    if (col) {
      const colors = col.attr('colors');
      element.backgroundColor({
        customColorsEnabled: (colors && colors.customColorsEnabled) || false,
        colorSettings: (colors && colors.colorSettings) || {},
        colors: (colors && colors.colors) || arc.report.graded_colors(),
        isReversed: (colors && colors.isReversed) || false,
        opacity: (colors && colors.opacity) || 80,
        dimensionMappings: colors && colors.dimensionMappings,
        fixedDimensionColors: (colors &&
          colors.fixedDimensionColors) || false,
        firstColor: colors?.firstColor
      });
      return;
    }

    const viz = arc.report.getPrimaryViz();
    if (!viz) {
      return;
    }

    element.backgroundColor({
      colors: viz.colors(),
      opacity: viz.colorOpacity() * 100,
      customColorsEnabled: false,
      colorSettings: {},
      isReversed: false,
      dimensionMappings: false,
      fixedDimensionColors: false,
    });
  };

  element.getOrderExpression = function(ordinal) {
    let full_expr;
    if (ordinal) {
      full_expr = ordinal;
    } else {
      full_expr = element.getExpression(undefined, true);
    }

    if (order.ascending()) {
      full_expr += ' asc';
    } else {
      full_expr += ' desc';
    }

    return full_expr;
  };

  element._tryFilterSubstitution = function(substitute_params_cb, colname,
      coltype) {
    if (!coltype) {
      coltype = dataset_coltype || '';
    }

    if (substitute_params_cb) {
      const in_data = `<<${colname}>>`;
      const out_data = substitute_params_cb(in_data);

      // Workaround for ARC-1009
      if (in_data.toLowerCase() !== out_data.toLowerCase()) {
        // Use the dataset unique name in the actual expression
        if (dataset_colname &&
            arc.params.convertKey(dataset_colname) === arc.params.convertKey(colname)) {
          colname = `[${colname}]`;
        }

        return arc.misc.inClause(colname, coltype, out_data,
            dataconnection_type);
      }
    }

    // Now try other filter substitutions
    // First get current parameters
    const cur_params = arc.report.shelf.getCurrentParams();
    if (!cur_params) {
      return arc.report.shelf.EXPR_TRUE;
    }

    const expr = arc.misc.filterExpression(colname, coltype, cur_params,
        dataconnection_type);
    if (expr) {
      return expr;
    }

    // default to EXPR_TRUE if we did not return any other expressions
    return arc.report.shelf.EXPR_TRUE;
  };

  // TODO: This needs to go to the backend and be validated. We should not
  // be checking for whether an expression is an aggregate on the client.
  element.expressionIsAggregate = function(config = {}) {
    let dctype = dataset && dataset.dataconnection_type();
    if (!dctype) {
      dctype = 'arcengine';
    }

    const str = element.getExpression(config.substCb);
    let containsAggField;
    if (config.existingAggregates) {
      containsAggField = config.existingAggregates.some((field) => str.includes(`[${field}]`));
    }

    const r = containsAggField || arc.report.containsAggregate(str, dctype);
    return r;
  };

  element.getExpression = function(substitute_params_cb, ignore_alias, for_data_api) {
    const exprl = element.getExpressionComponents(substitute_params_cb, ignore_alias, for_data_api);

    if (ignore_alias || (exprl.alias === undefined)) {
      return exprl.expr;
    }

    return `${exprl.expr} as ${exprl.alias}`;
  };

  function applyCastFn(expr, conn, connType, castType) {
    // Timestamp casting is not necessary in sqlite
    if (connType === 'sqlite' && !['timestamp', 'TIMESTAMP'].includes(castType)) {
      const castFn = conn ? conn.cast_fn : 'cast';
      return `${castFn}(${expr} as ${castType})`;
    }

    return expr;
  }

  element.getExpressionComponents = function(substitute_params_cb, ignore_alias, for_data_api) {
    const elem_filter_input = element.filter_input();
    const relative_date_picker = elem_filter_input.relative_daterange_picker;
    const conn_type = arc.report.shelf.CONNECTION_TYPE &&
        arc.report.shelf.CONNECTION_TYPE.connection_type;
    const conn = arc.report.shelf.CONNECTION_TYPE;

    if (elem_filter_input.activeSetting === 'relative_daterange_picker' &&
      relative_date_picker.anchorType === 'current') {
        element.custom_expr(arc.filter.helpers.fetchRelativeExpression(
          relative_date_picker,
          dataset_colname,
          dataset_coltype,
          conn_type
        ));
    }

    let alias;

    if (isDatasetSegment && substitute_params_cb) {
      return {
        expr: arc.report.segment_name,
        alias: arc.report.segment_name,
      };
    }

    if (custom_expr) {
      // Here we want to handle parameter substitution now so that if the input
      // parameter contains ... as 'alias', we don't want to force
      // the " as 'expr text' " in that case.

      let e = custom_expr;

      const extra_argsl = [custom_expr];
      if (substitute_params_cb) {
        e = substitute_params_cb(e, extra_argsl);
      }

      const esplit = arc.misc.splitExprAndAlias(e);
      if (ignore_alias) {
        if (fn_cast) {
          return {
            expr: applyCastFn(esplit.expr, conn, conn_type, fn_cast),
            alias: esplit.alias
          };
        }

        return esplit;
      }

      if (for_data_api) {
        // Take out the 'fake' AGG aggregate
        esplit.expr = esplit.expr.replace(/^agg\((.*)\)$/i, '$1');
      }

      // Select the alias (column display name)
      let a = esplit.alias || col_alias || esplit.expr.replace(/[[\]]/g, '');

      // Look for a quote character that is not in the string
      let q = "'";
      if (a.indexOf(q) !== -1) {
        q = '"';
        if (a.indexOf(q) !== -1) {
          q = '`';
          if (a.indexOf(q) !== -1) {
            q = '';
          }
        }
      }

      if (substitute_params_cb) {
        a = substitute_params_cb(a, extra_argsl);
      }

      alias = q + a + q;
      return {
        expr: fn_cast ? applyCastFn(esplit.expr, conn, conn_type, fn_cast) : esplit.expr,
        alias,
      };
    }

    if (filter_column && !isDatasetSegment) {
      if (filter_data.length === 0) {
        return {
          expr: element._tryFilterSubstitution(substitute_params_cb, filter_column),
        };
      }

      return {
        expr: arc.misc.whereClauseFromValues(filter_column, filter_data,
            filter_exclude, true, conn_type),
      };
    }

    if (dataset_colname) {
      let c = dataset_colname;

      // DJC -- Need a better method for distinguishing how we display an element vs how we
      //        we refer to the element with a select statement.
      //  -->   Need to ensure we don't break custom expressions here.
      alias = col_alias || c;
      c = `[${c}]`;
      if (fn_cast) {
        c = applyCastFn(c, conn, conn_type, fn_cast);
      }

      if (fn_name) {
        if (fn_type === arc.report.fn_types.AGGREGATE) {
          if (conditionalAgg) {
            if (!col_alias) {
              col_alias = `${fn_name}(${dataset_colname})`;
            }

            const relativeDate = conditionalAgg.relative_date;
            relativeDate.baseFieldDate = undefined;
            if (relativeDate.anchorType === 'basefield') {
              const se = shelf.shelves().findByPersistentId(relativeDate.baseField);
              if (se) {
                const a = se.conditionalAgg();
                if (a) {
                  const d = a.relative_date;
                  if (d) {
                    relativeDate.baseFieldDate = d.startDate;
                  }
                }
              }

              if (relativeDate.baseFieldDate === undefined) {
                arc.report.alertMessageSmall(
                    `Expression invalid for ${element.col_alias()}`, 'alert-danger'
                );
              }
            }

            arc.filter.helpers.fetchRelative(relativeDate, relativeDate.datetimeFormat);
            c = conn.expr_if(c, element.conditionalClause());
          }
        }

        if (fn_name === 'countdistinct') {
          c = `count(distinct (${c}))`;
        } else {
          c = `${fn_name}(${c})`;
        }

        if (!col_alias) {
          alias = `${fn_name}(${dataset_colname})`;
        }
      }

      if (substitute_params_cb) {
        alias = substitute_params_cb(alias);
      }

      return {
        expr: c,
        alias: `'${alias}'`,
      };
    }

    return {
      expr: '--invalid--',
    };
  };

  element.getComputedDisplayName = function(ignore_alias, no_suffix) {
    if (isDatasetSegment) {
      if (shelf && shelf.shelf_name() === 'filters_shelf') {
        if (!custom_expr) {
          return arc.report.segment_display_name;
        }

        if (custom_expr.trim() === '') {
          return arc.report.segment_display_name;
        }

        const segments = JSON.parse(custom_expr);
        const len = segments.length;

        if (len === 0) {
          return arc.report.segment_display_name;
        }

        return `Segments (${segments.length})`;
      }

      return arc.report.segment_display_name;
    }

    if (event_element) {
      let e = event_element.event_name();
      if (no_suffix) {
        return e;
      }

      if (event_min_count !== undefined) {
        e += ` {${event_min_count}-`;
        if (event_max_count !== null) {
          e += event_max_count;
        }

        e += '}';
      } else if (event_max_count !== undefined) {
        e += `{1-${event_max_count}}`;
      }

      if (event_goal) {
        e += ' [Goal]';
      }

      return e;
    }

    if (filter_column && (filter_data.length === 0)) {
      if (!custom_expr || custom_expr === arc.report.shelf.EXPR_TRUE || custom_expr === `([${filter_column}] is not NULL)`) {
        return filter_column;
      }
    }

    let suffix = '';
    if (order && order_limit && order_limit_granular) {
      suffix = ` [granular limit ${order_limit}]`;
    } else if (order && order_limit) {
      suffix = ` [limit ${order_limit}]`;
    }

    let expr;
    if (!ignore_alias) {
      expr = col_alias;
    }

    if (!expr && !custom_expr && !fn_name) {
      expr = dataset_colname;
    }

    if (!expr || filter_column) {
      expr = element.getExpression(undefined, true);

      // Here is we wanted to make column names stand out somehow, we could
      // replace the []'s with some html.
      expr = expr.replace(/[[\]]/g, '');
    }

    return no_suffix ? expr : expr + suffix;
  };

  // ignore_alias is true for values of shelf elements for analytic function
  element.getExpressionForEdit = function(ignore_alias) {
    if (filter_column && (filter_data.length === 0)) {
      if (!custom_expr || custom_expr === arc.report.shelf.EXPR_TRUE) {
        return `[${filter_column}]`;
      }
    }

    const expr = element.getExpression(undefined, !col_alias || ignore_alias);
    return expr;
  };

  element.save = function(d) {
    if (source_pivot) {
      d.source_pivot = source_pivot;
    }

    if (persistentId) {
      d.persistentId = persistentId;
    }

    if (col_alias) {
      d.col_alias = col_alias;
    }

    if (coldisplaybefore) {
      d.coldisplaybefore = coldisplaybefore;
    }

    if (col_desc) {
      d.col_desc = col_desc;
    }

    if (event_element) {
      d.event_id = event_element.id();
    }

    if (event_goal) {
      d.event_goal = event_goal;
    }

    if (event_min_count) {
      d.event_min_count = event_min_count;
    }

    if (event_max_count) {
      d.event_max_count = event_max_count;
    }

    if (custom_expr) {
      d.custom_expr = custom_expr;
    }

    // Save away the editable expression for creating dsreq for triggers
    // otherwise, would have to convert arc.misc.whereClauseFromValues to python
    d.expression_for_trigger = element.getExpression(undefined, true);

    if (dataset_colname) {
      d.dataset_colname = dataset_colname;
    }

    if (dataset_dimhlist) {
      d.dataset_dimhlist = dataset_dimhlist;
    }

    if (dataset_coltype) {
      d.dataset_coltype = dataset_coltype;
    }

    if (dataset_isdim) {
      d.dataset_isdim = dataset_isdim;
    }

    if (dflagg) {
      d.dflagg = dflagg;
    }

    if (secondaryaxis) {
      d.secondaryaxis = secondaryaxis;
    }

    if (expr_hasagg) {
      d.expr_hasagg = expr_hasagg;
    }

    if (fn_cast) {
      d.fn_cast = fn_cast;
    }

    if (analyticalFunctionData) {
      d.analyticalFunctionData = analyticalFunctionData;
    }

    if (analyticalFunctionSelection) {
      d.analyticalFunctionSelection = analyticalFunctionSelection;
    }

    if (conditionalAgg) {
      d.conditionalAgg = conditionalAgg;
    }

    if (enableExpansion) {
      d.enableExpansion = enableExpansion;
    }

    /*
     * Note on isDatasetSegment Upgrades
     *
     * We added in this 'isDatasetSegment' property for shelf elements
     * at v6.1.1 so there are a lot of saved dashboards without this property.
     *
     * We handle the older dashboards in this manner: when we restore the
     * isDatasetSegment property from a saved state, if we see nothing saved
     * for isDatasetSegment (i.e. if the value is neither true nor false), then
     * we key off the column name (as was the practice in earlier code). This means
     * that when we save away isDatasetSegment here, we have to do it unconditionally
     * i.e. we save both true and false values, unlike what we do for most other
     * properties that don't have this upgrade factor.
     */
    d.isDatasetSegment = (isDatasetSegment === true);

    if (fn_type) {
      d.fn_type = fn_type;
    }

    if (fn_name) {
      d.fn_name = fn_name;
    }

    if (geo_cast) {
      d.geo_cast = geo_cast;
    }

    if (isSensitiveElement) {
      d.isSensitiveElement = isSensitiveElement;
    }

    if (shelf_date_function) {
      d.shelf_date_function = shelf_date_function;
    }

    if (autotime_override || autotime_override === '') {
      d.autotime_override = autotime_override;
    }

    if (displayBackgroundColor) {
      d.displayBackgroundColor = displayBackgroundColor;
    }

    if (backgroundColor) {
      d.backgroundColor = backgroundColor;
    }

    if (autotimeEnabled) {
      d.autotimeEnabled = autotimeEnabled;
    }

    if (dataTransformation) {
      d.dataTransformation = dataTransformation;
    }

    if (display_format) {
      d.display_format = display_format;
    }

    if (order) {
      d.order = {};
      order.save(d.order);
      if (order_limit) {
        d.order_limit = order_limit;
        if (order_limit_granular) {
          d.order_limit_granular = order_limit_granular;
        }
      }
    }

    if (filter_column) {
      d.filter_column = filter_column;
      d.filter_exclude = filter_exclude;
      d.filter_data = filter_data;
      d.filter_input = filter_input;
    }

    return element;
  };

  element.restore = function(d, dataset_events, ds) {
    if (ds) {
      element.dataset(ds);
    }

    if ('persistentId' in d) {
      element.persistentId(d.persistentId);
    } else {
      element.persistentId(arc.misc.getUniqueId());
    }

    if ('col_alias' in d) {
      // Don't overwrite alias that was set by custom expression
      if (!element.col_alias()) {
        element.col_alias(d.col_alias);
      }
    }

    if ('coldisplaybefore' in d) {
      element.coldisplaybefore(d.coldisplaybefore);
    }

    if ('col_desc' in d) {
      // Don't overwrite alias that was set by custom expression
      if (!element.col_desc()) {
        element.col_desc(d.col_desc);
      }
    }

    if ('event_id' in d) {
      element.event_element(dataset_events.getEventById(d.event_id));
    }

    if ('event_min_count' in d) {
      element.event_min_count(d.event_min_count);
    }

    if ('event_goal' in d) {
      element.event_goal(d.event_goal);
    }

    if ('event_max_count' in d) {
      element.event_max_count(d.event_max_count);
    }

    if ('dataset_colname' in d) {
      element.dataset_colname(d.dataset_colname);
    }

    if ('dataset_dimhlist' in d) {
      element.dataset_dimhlist(d.dataset_dimhlist);
    }

    if ('dataset_coltype' in d) {
      element.dataset_coltype(d.dataset_coltype);
    }

    if ('dataset_isdim' in d) {
      element.dataset_isdim(d.dataset_isdim);
    }

    if ('dflagg' in d) {
      element.dflagg(d.dflagg);
    }

    if ('expr_hasagg' in d) {
      element.expr_hasagg(d.expr_hasagg);
    }

    if ('secondaryaxis' in d) {
      element.secondaryaxis(d.secondaryaxis);
    }

    if ('fn_cast' in d) {
      element.fn_cast(d.fn_cast);
    }

    if ('conditionalAgg' in d) {
      element.conditionalAgg(d.conditionalAgg);
    }

    if ('enableExpansion' in d) {
      element.enableExpansion(d.enableExpansion);
    }

    // Please read the "Note on isDatasetSegment Upgrades" earlier in this file
    if ('isDatasetSegment' in d) {
      element.isDatasetSegment(d.isDatasetSegment === true);
    } else {
      element.isDatasetSegment(dataset_colname === arc.report.segment_name);
    }

    if ('fn_type' in d) {
      element.fn_type(d.fn_type);
    }

    if ('fn_name' in d) {
      element.fn_name(d.fn_name);
    }

    if ('analyticalFunctionData' in d) {
      element.analyticalFunctionData(d.analyticalFunctionData);
    }

    if ('analyticalFunctionSelection' in d) {
      element.analyticalFunctionSelection(d.analyticalFunctionSelection);
    }

    if ('geo_cast' in d) {
      element.geo_cast(d.geo_cast);
    }

    if ('isSensitiveElement' in d) {
      element.isSensitiveElement(d.isSensitiveElement);
    }

    if ('source_pivot' in d) {
      element.source_pivot(d.source_pivot);
    }

    if ('shelf_date_function' in d) {
      element.shelf_date_function(d.shelf_date_function);
    }

    if ('autotime_override' in d) {
      element.autotime_override(d.autotime_override);
    }

    if ('displayBackgroundColor' in d) {
      element.displayBackgroundColor(d.displayBackgroundColor);
    }

    if ('backgroundColor' in d) {
      element.backgroundColor(d.backgroundColor);
    }

    if ('autotimeEnabled' in d) {
      element.autotimeEnabled(d.autotimeEnabled);
    }

    if ('dataTransformation' in d) {
      element.dataTransformation(d.dataTransformation);
    }

    if ('display_format' in d) {
      element.display_format(d.display_format);
    }

    if ('order' in d) {
      element.order(arc.report.shelf.order().restore(d.order));
      if ('order_limit' in d) {
        element.order_limit(d.order_limit);
        if ('order_limit_granular' in d) {
          element.order_limit_granular(d.order_limit_granular);
        }
      }
    }

    if ('filter_column' in d) {
      element.filter_column(d.filter_column);
    }

    if ('filter_exclude' in d) {
      element.filter_exclude(d.filter_exclude);
    }

    if ('filter_data' in d) {
      element.filter_data(d.filter_data);
    }

    if ('filter_input' in d) {
      element.filter_input(d.filter_input);
    }

    if ('v' in d) {
      element.custom_expr(d.v);
    }

    if ('custom_expr' in d) {
      element.custom_expr(d.custom_expr);
    }

    return element;
  };

  element.col_alias = function(_) {
    if (!arguments.length) return col_alias;
    col_alias = arc.datasets.stripQuotes(_);
    return element;
  };

  element.coldisplaybefore = function(_) {
    if (!arguments.length) return coldisplaybefore;
    coldisplaybefore = _;
    return element;
  };

  element.col_desc = function(_) {
    if (!arguments.length) return col_desc;
    col_desc = _;
    return element;
  };

  element.order_limit = function(_) {
    if (!arguments.length) return order_limit;
    order_limit = _;
    return element;
  };

  element.order_limit_granular = function(_) {
    if (!arguments.length) return order_limit_granular;
    order_limit_granular = _;
    return element;
  };

  element.filter_column = function(_) {
    if (!arguments.length) return filter_column;
    filter_column = _;
    return element;
  };

  element.filter_exclude = function(_) {
    if (!arguments.length) return filter_exclude;
    filter_exclude = _;
    return element;
  };

  element.filter_data = function(_) {
    if (!arguments.length) return filter_data;
    filter_data = _;
    return element;
  };

  element.filter_input = function(_) {
    if (!arguments.length) return filter_input;
    filter_input = _;
    return element;
  };

  element.remove_filter_data = function(_) {
    const idx = filter_data.indexOf(_);
    if (idx < 0) {
      return false;
    }

    filter_data.splice(idx, 1);
    if (!filter_data.length) {
      const $allLis = $(`#${element.shelf().shelf_name()} li`);
      $.each($allLis, (i, li) => {
        if (d3.select(li).datum() === element) {
          $(li).remove();
        }
      });
      element.shelf().removeElement(element);
    }

    return true;
  };

  element.add_filter_data = function(_) {
    if (!Array.isArray(_)) {
      // TODO: Should never be called
      addFilterValue(_);
    } else {
      _.forEach(addFilterValue);
    }

    function addFilterValue(filter_value) {
      if (filter_data.indexOf(filter_value) < 0) {
        filter_data.push(filter_value);
      }
    }

    return element;
  };

  element.event_goal = function(_) {
    if (!arguments.length) return event_goal;
    event_goal = _;
    return element;
  };

  element.shelf = function(_) {
    if (!arguments.length) return shelf;
    shelf = _;
    return element;
  };

  element.event_element = function(_) {
    if (!arguments.length) return event_element;
    event_element = _;
    return element;
  };

  element.event_min_count = function(_) {
    if (!arguments.length) return event_min_count;
    event_min_count = _;
    return element;
  };

  element.event_max_count = function(_) {
    if (!arguments.length) return event_max_count;
    event_max_count = _;
    return element;
  };

  element.dataset_colname = function(_) {
    if (!arguments.length) return dataset_colname;
    dataset_colname = _;
    return element;
  };

  element.dataset_coltype = function(_) {
    if (!arguments.length) return dataset_coltype;
    dataset_coltype = _;
    return element;
  };

  element.dataset_colcomment = function(_) {
    if (!arguments.length) return dataset_colcomment;
    dataset_colcomment = _;
    return element;
  };

  element.dataset_isdim = function(_) {
    if (!arguments.length) return dataset_isdim;
    dataset_isdim = _;
    return element;
  };

  element.dataset_hivecomment = function(_) {
    if (!arguments.length) return dataset_hivecomment;
    dataset_hivecomment = _;
    return element;
  };

  element.dataset_dimhlist = function(_) {
    if (!arguments.length) return dataset_dimhlist;
    if (typeof (_) === 'string') {
      dataset_dimhlist = _.split(',');
    } else {
      dataset_dimhlist = _;
    }

    return element;
  };

  element.dataset_dimhvals = function(_) {
    if (!arguments.length) return dataset_dimhvals;
    dataset_dimhvals = _;
    return element;
  };

  element.dataset_dimhlevel = function(_) {
    if (!arguments.length) return dataset_dimhlevel;
    dataset_dimhlevel = _;
    return element;
  };

  element.dataset_geotype = function(_) {
    if (!arguments.length) return dataset_geotype;
    dataset_geotype = _;
    return element;
  };

  element.isSensitiveElement = function(_) {
    if (!arguments.length) return isSensitiveElement;
    isSensitiveElement = _;
    return element;
  };

  element.dflagg = function(_) {
    if (!arguments.length) return dflagg;
    dflagg = _;
    return element;
  };

  element.expr_hasagg = function(_) {
    if (!arguments.length) return expr_hasagg;
    expr_hasagg = _;
    return element;
  };

  element.getComputedType = function() {
    if (custom_expr) {
      // In general, we do not know the types of custom expressions
      const x = custom_expr.trim();
      const m = x.match(/^\s*(\S+)\s*\(/);
      if (m) {
        let f = m[1];
        if ((f === 'to_date') || (f === 'date')) {
          return 'date';
        }

        f = f.toLowerCase();
        if (f.indexOf(['sum']) >= 0) {
          return 'integer';
        }
      }

      if (dataset_coltype) {
        return dataset_coltype.toLowerCase();
      }

      return 'unknown';
    }

    if (fn_type === arc.report.fn_types.AGGREGATE) {
      // This is a simple aggregate - we know the type
      // based on the aggregation operator
      if (fn_name === 'group_concat') {
        return 'string';
      }

      return 'integer';
    }

    if (fn_name) {
      // This is a function. Walk through known functions
      if ((fn_name === 'to_date') ||
          (fn_name === 'date')) {
        return 'date';
      }

      if ((fn_name === 'upper') ||
          (fn_name === 'lower')) {
        return 'string';
      }

      if ((fn_name === 'year') ||
          (fn_name === 'month') ||
          (fn_name === 'dayofmonth') ||
          (fn_name === 'dayofyear')) {
        return 'integer';
      }

      return 'unknown';
    }

    // Now look at the base datatype and any cast that was performed
    let t;
    if (dataset_coltype) {
      t = dataset_coltype.toLowerCase();
    }

    // Override base column type with a function cast
    if (fn_cast) {
      t = fn_cast;
    }

    return fnCastToColumnType(t);
  };

  element.dataset_coltype_class = function() {
    // TODO: Port this functionality to Column.coltype_class
    const t = element.getComputedType();
    let returnValue = '';
    switch (t) {
      case 'string':
        returnValue = 'type-string';
        break;
      case 'numeric':
      case 'int':
      case 'integer':
      case 'bigint':
        returnValue = 'type-integer';
        break;
      case 'real':
      case 'double':
        returnValue = 'type-real';
        break;
      case 'boolean':
        returnValue = 'type-boolean';
        break;
      case 'timestamp':
      case 'date':
        returnValue = 'type-timestamp';
        break;
      case 'hierarchy':
        returnValue = 'type-hierarchy';
        break;
      case 'array':
        returnValue = 'type-array';
        break;
      case 'map':
        returnValue = 'type-map';
        break;
      case 'struct':
        returnValue = 'type-struct';
        break;
      case 'vector':
        returnValue = 'type-vector';
        break;
      case 'text':
        returnValue = 'type-text';
        break;
      default:
        returnValue = 'type-unknown';
    }

    return returnValue;
  };

  element.custom_expr = function(_) {
    if (!arguments.length) return custom_expr;

    if (!_) {
      custom_expr = _;
    } else {
      const esplit = arc.misc.splitExprAndAlias(_);
      if (!esplit.alias) {
        custom_expr = _;
      } else {
        custom_expr = esplit.expr;
        col_alias = esplit.alias;
      }
    }

    return element;
  };

  element.conditionalClause = function() {
    const dateTimeColumn = dataset.defaultDateTime();
    if (conditionalAgg.relative_date) {
      const r = conditionalAgg.relative_date;
      return arc.misc.whereClauseFromMinMax(dateTimeColumn,
          arc.dataconnection.types.quoteString(dataconnection_type, r.startDate),
          arc.dataconnection.types.quoteString(dataconnection_type, r.endDate),
          '>=', '<=');
    }
  };

  element.persistentId = function(_) {
    if (!arguments.length) return persistentId;
    persistentId = _;
    return element;
  };

  element.conditionalAgg = function(_) {
    if (!arguments.length) return conditionalAgg;
    conditionalAgg = _;
    return element;
  };

  element.enableExpansion = function(_) {
    if (!arguments.length) return enableExpansion;
    enableExpansion = _;
    return element;
  };

  element.isDatasetSegment = function(_) {
    if (!arguments.length) return isDatasetSegment;
    isDatasetSegment = _;
    return element;
  };

  element.fn_cast = function(_) {
    if (!arguments.length) return fn_cast;
    fn_cast = _;
    if (fn_cast) {
      fn_cast = fn_cast.toLowerCase();
    }

    return element;
  };

  element.fn_type = function(_) {
    if (!arguments.length) return fn_type;
    fn_type = _;
    return element;
  };

  element.fn_name = function(_) {
    if (!arguments.length) return fn_name;
    fn_name = _;
    if (fn_name) {
      element.clearCustomExpr();
    }

    return element;
  };

  element.geo_cast = function(_) {
    if (!arguments.length) return geo_cast;
    geo_cast = _;
    if (geo_cast) {
      geo_cast = geo_cast.toLowerCase();
    }

    return element;
  };

  element.source_pivot = function(_) {
    if (!arguments.length) return source_pivot;
    source_pivot = _;
    return element;
  };

  element.shelf_date_function = function(_) {
    if (!arguments.length) return shelf_date_function;
    shelf_date_function = _;
    return element;
  };

  element.autotime_override = function(_) {
    if (!arguments.length) return autotime_override;
    autotime_override = _;
    return element;
  };

  element.removeFunction = function() {
    element.col_alias(null);
    element.custom_expr(null);
    element.col_desc(null);
  };

  element.displayBackgroundColor = function(_) {
    if (!arguments.length) {
      return displayBackgroundColor;
    }

    displayBackgroundColor = _;
    return element;
  };

  element.backgroundColor = function(_) {
    if (!arguments.length) {
      return backgroundColor;
    }

    backgroundColor = _;
    return element;
  };

  element.autotimeEnabled = function(_) {
    if (!arguments.length) {
      return autotimeEnabled;
    }

    autotimeEnabled = _;
    return element;
  };

  element.dataTransformation = function(_) {
    if (!arguments.length) {
      return dataTransformation;
    }

    dataTransformation = _;
    return element;
  };

  element.display_format = function(_) {
    if (!arguments.length) {
      if (display_format && display_format.type !== '') {
        return display_format;
      }

      return undefined;
    }

    display_format = _;
    return element;
  };

  element.order = function(_) {
    if (!arguments.length) return order;

    order = _;
    if (!order) {
      order_limit = 0;
      order_limit_granular = false;
    }

    return element;
  };

  element.order_display = function($li) {
    const $order_span = $li.find('.ordering');
    const $order_i = $order_span.find('i');
    const el_order = element.order();
    $li.find('.priority')
       .hide();

    if (el_order) {
      if (el_order.ascending()) {
        $order_i.removeClass('fa-sort-amount-desc').addClass('fa-sort-amount-asc');
      } else {
        $order_i.removeClass('fa-sort-amount-asc').addClass('fa-sort-amount-desc');
      }

      if (el_order.priority()) {
        $li.find('.priority')
           .text(el_order.priority())
           .show();
      }

      // Display this order
      $order_span.show();
    } else {
      $order_i.removeClass('fa-sort-amount-desc').removeClass('fa-sort-amount-asc');

      // Hide the ordering indicator
      $order_span.hide();
    }
  };

  element.getColRows = function() {
    const deferObj = $.Deferred();

    const rb = arc.data_requests.distinctColumnValues(
        undefined, element.dataset(),
        element.dataset_colname(), 500, element.dataset_coltype()
    );

    function success(data) {
      deferObj.resolve(data);
    }

    function fail(errMsg) {
      // eslint-disable-next-line no-console
      console.log(`Error fetching values for ${element.dataset_colname()}: ${errMsg}`);
      deferObj.reject();
    }

    arc.data.runQuery(rb, success, fail);

    return deferObj.promise();
  };

  return element;
};
}());
