window.arc = window.arc || {};

(function() {
'use strict';

arc.report = arc.report || {};

function sortBasedOnUpdatedInfo(a, b) {
  if (a.updated > b.updated) {
    return -1;
  }

  if (a.updated < b.updated) {
    return 1;
  }

  return 0;
}

function sortBasedOnOrder(a, b) {
  return a.order - b.order;
}

// config:
//  displaySheets, which shows dashboard sheets
//    hides "Pass current parameters when navigating" and "Open Target Dashboard in" options
//  customValidator, filters reports displayed
arc.report.reportPicker = function(click_behavior, selection_cb, curr_dataset_id, config = {}) {
  const $modal = $('.report-pick-modal');
  if (arc.misc.modalShown($modal)) {
    return;
  }

  const $targetNavSelector = $('#target-nav-selector');

  // Remove current reports
  $targetNavSelector.empty();

  $modal.modal('show');

  let curr_report_id;
  let passParams = false;
  let openTarget = 'current_tab';
  if (click_behavior) {
    curr_report_id = Number(click_behavior.report_id);
    passParams = click_behavior.shouldPassParams || false;
    openTarget = click_behavior.openTarget || 'current_tab';
  }

  const showDetail = !config.displaySheets ? '' : '&detail=true';
  const assets = [
    {
      asset: arc.frontend.globals.urls.datasetUrl,
      postprocess: d3.json,
    },
    {
      asset: `${arc.frontend.globals.urls.reportUrl}?report_type=dashboard${showDetail}`,
      postprocess: d3.json,
    },
  ];

  const reportPickerTemplate = $('#report-picker-set-section-template').html();
  $targetNavSelector.append(
    arc.misc.safe_substitute(reportPickerTemplate, {
      SET_OR_MODIFY: curr_report_id ? 'modify' : 'set',
    })
  );

  // Get all saved reports, and enable adding report to grid on click
  arc.staticfetcher.fetch(assets, function getReportDataset(error, datasets, reports) {
    if (error) {
      console.log(error);
      return;
    }

    reports = reports.filter(report => {
        const customValidate = !config.customValidator || config.customValidator(report);
        return report.report_type === 'dashboard' && customValidate;
      })
      .sort(sortBasedOnUpdatedInfo);

    if (curr_dataset_id) {
      // Sort datasets so the current dataset is first, and datasets are
      // in alphabetical order after that
      datasets.sort(function (a, b) {
        if (a.id === b.id) {
          return 0;
        }

        if (a.id === curr_dataset_id) {
          return -1;
        }

        if (b.id === curr_dataset_id) {
          return 1;
        }

        if (a.name < b.name) {
          return -1;
        }

        if (a.name > b.name) {
          return 1;
        }

        return 0;
      });
    }

    if (!config.displaySheets) {
      // Being used to link DTR and not for import report purposes.
      $targetNavSelector.append(
        `<div class="checkbox">
           <label>
             <input type="checkbox" class="pass-params"/>
            Pass current parameters when navigating.
           </label>
         </div>`
      );

      $targetNavSelector.find('.pass-params').attr('checked', passParams);

      $targetNavSelector.append(
        `<div class="open-target form-group form-inline">
           <span> Open Target Dashboard in </span>
           <select class="form-control form-control-sm">
              <option val="current_tab">Current Tab</option>
              <option val="new_tab">New Tab</option>
              <option val="new_window">New Window</option>
            </select>
         </div>`
      );
      $targetNavSelector.find(`.open-target select option[val="${openTarget}"]`)
                        .attr('selected', true);

      if (curr_report_id) {
        reports.filter((r) => r.id === curr_report_id).forEach(function (r) {
          $targetNavSelector.append(`
            <div class="current-selection">
              <ul class="app-avail-list">
                <li class="app_selection_element" arc-report-id="${r.id}">
                  <div class="app-img-wrapper">
                    <img src="${r.img}">
                  </div>
                  <div class="app-title">${r.title}</div>
                </li>
              </ul>
            </div>
          `);
        });
      }
    }

    function setup_search() {
      $targetNavSelector.find('.attr-search-clear')
                .off('click.search-app-nav')
                .on('click.search-app-nav', hideDatasetHeaders);

      arc.report.searchboxSetup({
        searchboxId: '#searchbox',
        search_cb: search_selection,
        sel_cb,
      });

      toggleDatasetHeaders();

      function sel_cb() {
        toggleDatasetHeaders();

        return d3.selectAll(
          $targetNavSelector.find('div.card div.panel-collapse ul.app-avail-list li')
        );
      }

      function toggleDatasetHeaders() {
        if ($targetNavSelector.find('#searchbox').val() === '') {
          showDatasetHeaders();
        } else {
          hideDatasetHeaders();
        }
      }

      function toggleClassDatasetHeaders(show) {
        $targetNavSelector.toggleClass('searching', !show)
                          .find('.panel-collapse')
                          .toggleClass('in', !show);
      }

      function hideDatasetHeaders() {
        toggleClassDatasetHeaders(false);
      }

      function showDatasetHeaders() {
        toggleClassDatasetHeaders(true);
      }

      function search_selection(searchItems, term, searchedElement) {
        let term_found = false;

        if (!searchItems) {
          const report_id = Number(searchedElement.getAttribute('arc-report-id'));
          searchItems = reports.find(r => r.id === report_id);
        }

        $.each([
          'title',
          'created',
          'updated',
          'created_by',
          'dataset_name',
          'dataconn_name',
          'dataset_type',
          'report_tags',
          'type',
          'id',
        ], function searchProperty(pidx, property) {
          if (searchItems[property].toString().toLowerCase().includes(term)) {
            term_found = true;

            // stop looking in other properties
            return false;
          }
        });

        return term_found;
      }
    }

    setup_search();

    const reportsHash = {};
    reports.forEach(function (report) {
      if (!reportsHash[report.dataset_id]) {
        reportsHash[report.dataset_id] = [];
      }

      reportsHash[report.dataset_id].push(report);
    });

    const datasetPanelTemplate = $('#dataset-panel-template').html();
    const reportPickerLiTemplate = $('#report-picker-li-template').html();
    const sheetOptionsTemplate = $('#sheet-options-template').html();
    datasets.concat({
      id: '', // Add for the case of dashboards which belong to multiple datasets
      name: 'Multiple Datasets',
    }).forEach(function drawReportsInDataset(dataset_elem) {
      var dataset_reports = (reportsHash[dataset_elem.id] || []).slice(0);

      if (dataset_reports.length === 0) {
        return;
      }

      const $ds_div = $(arc.misc.safe_substitute(datasetPanelTemplate, {
        DATASET_TITLE: `${dataset_elem.name} `,
        DATASET_REPORTS_COUNT: dataset_reports.length,
        DATASET_ID: dataset_elem.id || 'm',
      }));
      $targetNavSelector.append($ds_div);

      const $ul = $ds_div.find('ul.app-avail-list');
      let lis = '';
      dataset_reports.forEach(function drawReportInModal(report) {
        let report_data;
        try {
          // There's no report.data when displaySheets is false
          if (report.data) {
            report_data = JSON.parse(report.data);
            report_data = report_data.report_data || report_data;
          }
        } catch (e) {
          console.log('Invalid report data. should not have occurred');
          report_data = {};
        }

        lis += arc.misc.safe_substitute(reportPickerLiTemplate, {
          REPORT_THUMBNAIL_URL: report.img,
          REPORT_TITLE: report.title,
          REPORT_ID: report.id,
          HTML_SHEET_DATA: populateSheetData(report_data),
        });
      });

      $ul.append(lis);

      setTimeout(function renderReportImages() {
        // Putting this code in setTimeout allows modal to populate faster
        $.each($targetNavSelector.find('[data-arc-src]'), function (i, img) {
          const $img = $(img);
          $img.attr('src', $img.attr('data-arc-src'));
          $img.removeAttr('data-arc-src');
        });
      }, 0);
    });

    function populateSheetData(report_data) {
      if (!config.displaySheets) {
        return '';
      }

      let sheetData;
      if (arc.misc.isDataInSheetedFormat(report_data)) {
        sheetData = report_data.dashboard_sheets
          .sort(sortBasedOnOrder)
          .map(function populateSheetOptionObject(sheet) {
            return {
              title: sheet.sheet_handle_title || (`Sheet ${sheet.order}`),
              order: sheet.order,
            };
          });
      } else {
        sheetData = [
          {
            title: report_data.report_title && report_data.report_title !== 'Untitled' ?
                      report_data.report_title :
                      'Sheet 1',
            order: 1,
          },
        ];
      }

      const htmlSheetOptions = sheetData.map(
          rep => `<option val=${rep.order}>${rep.title}</option>`
      );

      let htmlSheetData = '';
      htmlSheetData = arc.misc.safe_substitute(sheetOptionsTemplate, {
        SHEET_OPTIONS: htmlSheetOptions,
      });

      return htmlSheetData;
    }

    $targetNavSelector.offOn('click.selectSheet', 'li.app_selection_element', function (evt) {
      if ($(evt.target).is('select')) {
        return;
      }

      const $target = $(evt.currentTarget);
      const currPassParam = $targetNavSelector.find('input.pass-params').prop('checked');
      const $openTargetSelect = $targetNavSelector.find('.open-target select');
      const currOpenTarget = $openTargetSelect.find(`option:contains("${$openTargetSelect.val()}")`)
                                        .attr('val');

      selection_cb({
        id: $target.attr('arc-report-id'),
        sheetOrderIds: [$target.find('select option:selected').index() + 1],
      }, function selectionCallback() {
        $('.report-pick-modal').modal('hide');
        arc.report.pagenav.needs_save(true);
      }, currPassParam, currOpenTarget);
    });
  });
};
}());
