window.arc = window.arc || {};

(function() {
"use strict";

arc.testutil = arc.testutil || {};
arc.report = arc.report || {};

var permittedErrors = [
  'SHOW COLUMN STATS not applicable to a view',
  'SHOW TABLE STATS not applicable to a view',
  'COMPUTE STATS not supported for view',
];

arc.report.mapDashboardOrVisualInfo = function(dashboardOrVisual) {
  const updatedDate = moment(dashboardOrVisual.updated).fromNow();
  const createdDate = moment(dashboardOrVisual.created).fromNow();
  let editLinkIfPermission = `${arc.frontend.globals.urls.reportBuildUrl}/${dashboardOrVisual.id}`;

  if (!arc.misc.hasPerm('ds_appedit', dashboardOrVisual.perm)) {
    editLinkIfPermission = `${arc.frontend.globals.urls.viewDashboardUrl}/${dashboardOrVisual.id}`;
  }

  return {
    IMAGE: dashboardOrVisual.img,
    TITLE: dashboardOrVisual.title || 'Untitled',
    EDIT_LINK: editLinkIfPermission,
    ID: dashboardOrVisual.id,
    MODIFIED_AND_AUTHOR: dashboardOrVisual.updated_by ?
                         `${dashboardOrVisual.updated_by} (${updatedDate})` :
                         '',
    CREATED_AND_AUTHOR: `${dashboardOrVisual.created_by} (${createdDate})`,
    DATACONNECTION_NAME: dashboardOrVisual.dataconn_name,
    DATASET_SHORTNAME: dashboardOrVisual.dataset_name,
    HTML_DATASET_TAGS: dashboardOrVisual.HTML_DATASET_TAGS || '',
    TAGS_VISIBILITY: dashboardOrVisual.TAGS_VISIBILITY || '',
    WORKSPACE_NAME: dashboardOrVisual.workspace_name,
    HIDE_INFO: '',
  };
};

// { alert_report_id, noConsoleLog, alertContainerSelector, is_html } = optionalAlertConfig;
arc.report.contextualAlertMessage = function(lib, msg, detail, optionalAlertConfig) {
  const isPermittedError = permittedErrors.some((error) => typeof detail === 'string' && detail.includes(error));

  if (isPermittedError) {
    return;
  }

  if (lib) {
    lib.alertMessage(msg, detail, optionalAlertConfig?.is_html);
  } else {
    arc.report.alertMessage(msg, detail, optionalAlertConfig);
  }
};

arc.report.sql_runner = (function() {
  var sql_runner = function() { };

  sql_runner.runParams = function(config) {
    let {
      dataconnection_id,
      query,
      cb,
      cache,
      dsreq,
      lib,
      hideAlertsOnSuccess,
    } = config;

    let displayErrorMessage = true;

    if (config.displayErrorMessage !== undefined) {
      displayErrorMessage = config.displayErrorMessage;
    }

    const params = {
      'query': query,
      'dataconnection_id': dataconnection_id,
      'cache_read': cache,
      'dsreq': dsreq ? JSON.stringify(dsreq) : '',
    };

    arc.sqlrun.runSQLWithParams(
      params,
      function successCb(response) {
        if (hideAlertsOnSuccess) {
          arc.report.alertMessageAllHide();
        }

        cb(null, response.response);
      },
      function errorCb(errorMsg) {
        if (displayErrorMessage) {
          arc.report.alertMessageAllHide();
          arc.report.contextualAlertMessage(undefined, "Could not load data",
              errorMsg, true);
        }

        cb(errorMsg, null);
      });
  };

  sql_runner.run = function(dataconnection_id, query, cb, cache, dsreq, lib,
      displayErrorMessage=true, hideAlertsOnSuccess=true) {

    const params = {
      dataconnection_id,
      query,
      cb,
      cache,
      dsreq,
      lib,
      displayErrorMessage,
      hideAlertsOnSuccess,
    };

    return sql_runner.runParams(params);
  };

  return sql_runner;
}());

// Helper object that indicates to the user a save is required
// when he is leaving the page.
// Updated by arc_reportbuilder.js on report save
arc.report.pagenav = (function() {
  var needs_save = false;

  var pagenav = function() { };

  var unload_funcs = {};
  var unload_index = 0;

  pagenav.needs_save = function(_) {
    if (!arguments.length) return needs_save;

    needs_save = _;
    if (needs_save) {
      d3.selectAll("#app-save-link, #save-link, #save-attr-btn, #save-join-btn, #save-role-btn, #save-dsperm-btn")
        .classed("app-btn-highlight", true);
    } else {
      d3.selectAll("#app-save-link, #save-link, #save-attr-btn, #save-join-btn, #save-role-btn, #save-dsperm-btn")
        .classed("app-btn-highlight", false);
    }

    return pagenav;
  };

  pagenav.add_unload = function(func) {
    unload_index += 1;
    unload_funcs[unload_index] = func;
    return unload_index;
  };

  pagenav.del_unload = function(index) {
    delete unload_funcs[index];
  };

  pagenav.add_unload(function() {
    if (needs_save) {
      return "You have some unsaved changes. " +
        "Navigating away from this page before saving will abandon those " +
        "changes.";
    }
  });

  // We run all posted unload functions in order of when they were set up.
  // First result returned (if any) is taken as the overall result.
  window.onbeforeunload = function() {
    var res;
    var keys = Object.keys(unload_funcs).sort(function (a, b) {return (a - b);});
    keys.forEach(function (k) {
      var newres = unload_funcs[k]();
      res = res || newres;
    });
    return res;
  };

  return pagenav;
}());

arc.report.REPORTID_NEW = -2;
arc.report.REPORTID_TIMECONTROL = -3;

arc.report.shelf_types = {
  'COLUMN': 1,
  'PREDICATE': 2,
  'ORDER': 3,
  'COUNTER': 4,
  'EVENT_FLOW': 5,
  'EVENT_FUNNEL': 6
};

arc.report.supported_aggregates = {
  'sum': 1,
  'count': 2,
  'max': 3,
  'min': 4,
  'avg': 5,
  'ndv': 6,
  'partial_ndv': 7,
  'final_ndv': 8,
  'having': 9,

  'stddev': 10,
  'stddev_pop': 11,
  'variance': 12,
  'var_pop': 13,
  'variance_pop': 14,

  // Typically, Arcadia's UI code understands whether an expression is a dimension
  // or an aggregate by parsing some part of the expression. Sometimes, the
  // parsing does not work and Arcadia's UI code cannot detect an aggregate
  // automatically. In these cases, we have a workaround: the user can wrap
  // the expression in an agg(expr) function call, and the UI layer will then
  // treat it as an aggregate.
  'agg': 15,
};

arc.report.column_types = {
  'TINYINT': 1,
  'SMALLINT': 2,
  'INT': 3,
  'BIGINT': 4,
  'REAL': 5,
  'FLOAT': 6,
  'DOUBLE': 7,
  'STRING': 8,
  'TIMESTAMP': 9,
  'BOOLEAN': 10,

  // Needed if hierarchy is being set in dataset level
  'HIERARCHY': 11,

  'INTEGER': 12,
};

// List out measure types
// tinyint and smallint are typically ids, not measures
arc.report.measure_types = {
  'INT': true,
  'INTEGER': true,
  'BIGINT': true,
  'REAL': true,
  'FLOAT': true,
  'DOUBLE': true,
};

arc.report.fn_types = {
  'AGGREGATE': 1,
  'SCALAR': 2
};

arc.report.numeric_types = {
  'TINYINT': true,
  'SMALLINT': true,
  'INT': true,
  'INTEGER': true,
  'BIGINT': true,
  'REAL': true,
  'FLOAT': true,
  'DOUBLE': true,
};

arc.report.integer_types = {
  'TINYINT': true,
  'SMALLINT': true,
  'INT': true,
  'INTEGER': true,
  'BIGINT': true,
};

arc.report.real_types = {
  'REAL': true,
  'FLOAT': true,
  'DOUBLE': true,
};

arc.report.boolean_types = {
  'BOOLEAN': true,
  'BOOL': true,
};

arc.report.coltypeIsReal = function(coltype) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toUpperCase();

  return arc.report.real_types[coltype];
};

arc.report.coltypeIsInteger = function(coltype) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toUpperCase();

  return arc.report.integer_types[coltype];
};

arc.report.coltypeIsHierarchy = function(coltype) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toUpperCase();

  return (coltype === 'HIERARCHY');
};

arc.report.coltypeIsNumber = function(coltype) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toUpperCase();

  return arc.report.numeric_types[coltype];
};

arc.report.coltypeIsBoolean = function(coltype) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toUpperCase();

  return arc.report.boolean_types[coltype];
};

arc.report.coltypeIsReal = function(coltype) {
  if (!coltype) {
    return false;
  }

  coltype = coltype.toUpperCase();

  return arc.report.real_types[coltype];
};

arc.report.setSamplingPct = function(pct) {
  pct = pct.trim();
  $("#dataset-sample-display").text(pct);
  if (pct === "Off") {
    $("#dataset-sample-display").removeClass("btn-primary");
  } else {
    $("#dataset-sample-display").addClass("btn-primary");
  }
};

// Keep track of the error messages on the page, for snapshot code
// Add new errors by calling
//  arc.report.logAlertMessages('alert-danger', {
//    type: 'big',
//    message: 'some msg',
//    header: 'some header',
//    useHTML: false
//  })
// Get errors by calling arc.report.logAlertMessages('alert-danger')
// Clear errors by calling arc.report.logAlertMessages('clear-small-big')
var loggedErrors = {};
arc.report.logAlertMessages = function (type, message) {
  if (message) {
    if (!loggedErrors[type]) {
      loggedErrors[type] = [];
    }
    loggedErrors[type].push(message);
  } else if (type === undefined) {
      return loggedErrors;
  } else if (type === 'clear-small-big' || type === 'clear-small' || type === 'clear-big') {
    // Can't use array.includes because that's not available in phantomjs
    for (var errorType in loggedErrors) {
      loggedErrors[errorType] = loggedErrors[errorType].filter(removeIfFound);
    }
  } else {
    return loggedErrors[type] || [];
  }

  function removeIfFound (e) {
    return type.indexOf(e.type) === -1;
  }
};

/**
 * This can be applied when displaying a group of alert messages one after another.
 * returns:
 * addDelayedMessage: (message, variant, delay) => void Displays an alert after the specified delay.
 * addFinalMessage: (message, variant) => void Displays the final alert from the group and
 *         clears the timeouts, so that the other messages will not appear after it.
 *         There's no need to remove the previous notifications if they were displayed already,
 *         because they will be replaced by the last one.
 */
arc.report.createAlertMessageGroup = function() {
  const timeouts = [];

  return {
    addDelayedMessage: (message, variant, delay = 300) => {
      timeouts.push(setTimeout(() => arc.report.alertMessageSmall(message, variant), delay));
    },
    addFinalMessage: (message, variant) => {
      timeouts.forEach((t) => clearTimeout(t));
      arc.report.alertMessageSmall(message, variant);
    }
  };
};

// Show a timed alert message
// Type expects 'alert-danger', 'alert-success', 'alert-warning', or 'alert-info'
// alert-success will timeout
arc.report.alertMessageSmall = function(message, type, fadeOut = false) {
  let header = 'Alert';
  let stackMessages = false;
  if (type === 'alert-success') {
    fadeOut = true;
    header = 'Success';
  } else if (type === 'alert-danger') {
    header = 'Error';
  } else if (type === 'alert-warning') {
    stackMessages = true;
  }

  arc.report.alertMessageSmallBase({
    type,
    stackMessages,
    fadeOut,
    alertHeader: header,
    alertMessage: message,
  });
};

// Display alert message based on a number of parameters:
// id, type, alertMessage, alertHeader, useHtml, stackMessages, onClose, fadeOutDelay, fadeOut
arc.report.alertMessageSmallBase = function(config) {
  const {
    id,
    // This should never be undefined, but we don't have a way to enforce it
    // so we are handling the possiblity
    type = 'alert-info',
    alertMessage,
    alertHeader,
    useHTML,
    stackMessages,
    onClose,
    fadeOutDelay,
    fadeOut,
  } = config;

  // Display the message in the notification container
  arc.frontend.legacyBridge.events.next({
    type: 'add-notification',
    payload: {
      id,
      variant: type.replace('alert-', ''),
      title: alertHeader,
      message: alertMessage,
      delay: fadeOut ? fadeOutDelay : 0,
      useHtml: useHTML,
      onClose,
      stackMessages
    }
  });
};

// Hide small alert message manually
// If it has a given id, hide only that message, otherwise remove all of them
arc.report.alertMessageSmallHide = function(id) {
  arc.frontend.legacyBridge.events.next({
    type: 'remove-notification',
    payload: { id },
  });
};

// Hide alert message
arc.report.alertMessageHide = function($element) {
  arc.misc.resetAlerts($element);
  arc.report.logAlertMessages('clear-big');
};

// Hide all alert message
arc.report.alertMessageAllHide = function($element) {
  arc.report.alertMessageSmallHide();
  arc.report.alertMessageHide($element);
};

arc.report.activateMainAlertContainer = function() {
  arc.report.alertMessageHide();
  arc.misc.deactiveAlertContainer($('.super-modal'));
  arc.misc.activeAlertContainer($('.builder-container'));
  arc.misc.activeAlertContainer($('.main-app-view'));
};

arc.report.activateModalAlertContainer = function() {
  arc.misc.activeAlertContainer($('.super-modal'));
  arc.misc.deactiveAlertContainer($('.builder-container'));
  arc.misc.deactiveAlertContainer($('.main-app-view'));
};

const seachModalCssSelector = '#test-search-modal';
const $searchModal = $(seachModalCssSelector);
arc.report.alertMessage = function(alertHeader, alertDetail = '', optionalAlertConfig = {}) {
  /*
  alert_header: Header for the alert
  alert_detail: Detailed message for the alert

  Optional Alert Configuration Items:

  is_html: if the detail should be treated as HTML or not
  alert_report_id: If a report is calling this function, we should include this value.
    This value is used to determine if the error should be thrown in the supermodal
    or in the dashboard alert container.
  */

  let { is_html } = optionalAlertConfig;
  let { is_traceback_included } = optionalAlertConfig;
  const { alert_report_id, noConsoleLog, alertContainerSelector } = optionalAlertConfig;
  if (!is_html && alertDetail && typeof alertDetail === 'string') {
    if (alertDetail.substring(0, 20).trim().substring(0, 15) === '<!DOCTYPE html>') {
      is_html = true;
    } else if (alertDetail.indexOf('Traceback') !== -1) {
      is_traceback_included = true;
    }
  }

  if (!arc.testutil.ignoreAlerts && !noConsoleLog) {
    arc.misc.logtrace(`${alertHeader}: ${alertDetail}`);
  }

  arc.report.logAlertMessages('alert-danger', {
    type: 'big',
    header: alertHeader,
    message: alertDetail
  });

  const $dashboardOrAppBuilderPageErrorWrapper = $('.viz-error-page-wrapper');
  const $alert = getActiveAlertContainer(alert_report_id, alertContainerSelector);
  const isAlertInModal =
    $alert.closest('.modal:visible').length +
    $alert.closest('.super-modal:visible').length +
    $alert.closest('.sql-super-modal:visible').length;

  // If an error occurs on the dashbaord/app builder page, and alertContainerSelector not defined
  // by default display error under the orange warning icons.
  // Otherwise the currently active alert-containers (show-usage-info modal,
  // sql-editor-modal, create-new-dataset modal search-modals etc., )!
  const displayErrorWithinDashboardWarningPopover =
     !alertContainerSelector &&
     $dashboardOrAppBuilderPageErrorWrapper.length > 0 &&
    !isAlertInModal;

  if (displayErrorWithinDashboardWarningPopover) {
    arc.errors.displayPageLevelAlert(alertHeader, alertDetail);
    return;
  }

  const $alertsContainer = $alert.find('.alert-error-messages');
  const $closeButton = $alert.find('.close');
  const $alertNumber = $alert.find('.alert-number-errors');

  if ($alert.length === 0) {
    if (alertDetail) {
      console.log(alertHeader + alertDetail);
    } else {
      console.log(alertHeader);
    }

    return;
  }

  // Update number of errors in error message
  const currentNumberAlerts = parseInt($alertNumber.text()) + 1;
  $alertNumber.text(currentNumberAlerts);
  if (currentNumberAlerts === 1) {
    // Don't show number if it's the first alert
    $alertNumber.hide();
  } else if (currentNumberAlerts === 2) {
    // Show number and plural "s" if there is more than 1 error
    $alertNumber.show();
    $alert.find('.alert-plural').text('s');
  }

  // Set up X (close button) click behavior just once
  $closeButton.offOn('click.closeButton', function() {
    arc.report.alertMessageHide($alert.parent());
  });

  // Set up click behavior and text for new error message
  const $alertMessage = $($("#alert-entry").html()).appendTo($alertsContainer);
  const $moreLink = $alertMessage.find(".alert-more-link");
  const $lessLink = $alertMessage.find(".alert-less-link");
  const $alertDetail = $alertMessage.find(".alert-detail");
  const $alertHeaderText = $alertMessage.find(".alert-header-text");
  const $alertDetailText = $alertMessage.find(".alert-detail-text");

  if (alertDetail) {
    $moreLink.show();
    $lessLink.hide();
    $moreLink.on('click', function() {
        $moreLink.hide();
        $lessLink.show();
        $alertDetail.show();
    });

    $lessLink.on('click', function() {
        $moreLink.show();
        $lessLink.hide();
        $alertDetail.hide();
    });
  } else {
    $lessLink.hide();
    $moreLink.hide();
  }

  $alertDetail.hide();
  $alertHeaderText.text(alertHeader);
  if (is_html) {
    // Add details to each alert detail on page
    $alertDetailText.each(function addIframeDetail(i, el) {
      const newIframe = $('<iframe src="about:blank" style="width: 100%"></iframe>')[0];
      $(el).html(newIframe);
      const doc = newIframe.contentWindow ||
                  newIframe.contentDocument.document ||
                  newIframe.contentDocument;
      doc.document.open();
      doc.document.write(alertDetail);
      doc.document.close();
    });
  } else if (is_traceback_included) {
    const $pre = $('<pre>');
    const $errorCode = $('<code>').text(alertDetail);
    $pre.append($errorCode);
    $alertDetailText.append($pre);
  } else {
    $alertDetailText.text(alertDetail);
  }

  $alert.show();
};

const $singleVisualContainer = $('.builder-container .alert-container');
const $dashboardContainer = $('.main-app-view .alert-container');
const $searchModalAlert = $searchModal.find('.alert-container');
function getActiveAlertContainer(alert_report_id, alertContainerSelector) {
  // If in search modal, show alert in search modal
  if ($searchModal.hasClass('show')) {
    return $searchModalAlert;
  }

  const defaultAlertSelector = `.alert-container.active`;
  let $alert = $(`${alertContainerSelector ?? ''}`).find(defaultAlertSelector);
  if (!$alert.length) {
    // Unfortunately multiple alert-container can be present in the DOM.
    // At least those filtered out which is not visible at the time...
    $alert = $(`${defaultAlertSelector}`).filter(function() { return $(this).parent().is(':visible'); });
  }

  if (!alert_report_id) {
    return $alert;
  }

  if (arc.misc.isInSuperModal()) {
    const modalReportId = arc.report.getPrimaryViz().report_id();
    if (modalReportId !== alert_report_id) {
      if ($singleVisualContainer.length > 0) {
        return $singleVisualContainer;
      }

      if ($dashboardContainer.length > 0) {
        return $dashboardContainer;
      }
    }
  }

  return $alert;
}

arc.report.supported_connection_aggregates = {};

arc.report.addAndCheckAggregates = function (word, dataConnectionType, snippetItems) {
  const connectionMode = arc.utils.modeForConnectionType(dataConnectionType);

  if (!arc.report.supported_connection_aggregates[connectionMode]) {
    snippetItems = snippetItems || arc.utils.snippetsGenerator(connectionMode, 'hash');
    arc.report.supported_connection_aggregates[connectionMode] = {};
    snippetItems.Aggregate.forEach(function(a) {
      const parsedText = arc.utils.snippetsGenerator.parseText(a).caption.toLowerCase();
      arc.report.supported_connection_aggregates[connectionMode][parsedText] = true;
    });
  }

  if (word !== undefined) {
    const wordCleaned = word.trim().toLowerCase();
    const inDefaultAggregates = wordCleaned in arc.report.supported_aggregates;
    const connectionAggregates = arc.report.supported_connection_aggregates[connectionMode];
    const inConnectionAggregates = wordCleaned in connectionAggregates;

    return inDefaultAggregates || inConnectionAggregates;
  }
};

// TODO: This needs to go to the backend and be validated. We should not
// be checking for whether an expression is an aggregate on the client.
arc.report.containsAggregate = function(str, dataConnectionType) {
  // remove anything between two single/double quotes
  const expr = str.replace(/"([^"]+)"|'([^']+)'/g, '');

  // extract tokens separated buy '[', ']', '(', ')' and space, and keep separators except space
  const tokens = expr.split(/([([)\]\s])/g).filter((token) => /\S/.test(token));

  function parstackMatch(parstack, token) {
    if (parstack.length === 0) { return false; }

    if (token === ')' && parstack[parstack.length - 1] === '(') { return true; }

    if (token === ']' && parstack[parstack.length - 1] === '[') { return true; }

    return false;
  }

  function isOpening(token) {
    return ['(', '['].includes(token);
  }

  function isClosing(token) {
    return [')', ']'].includes(token);
  }

  // check valid agg name is also followed by ([column name]) or (1) and not in []
  const parstack = [];
  let isValidAggName = false; // agg name that's not in brackets

  for (const t of tokens) {
    if (isOpening(t)) {
      // opening bracket found, remembering it
      parstack.push(t);
    } else if (isClosing(t)) {
      // check if parstack top is matching
      if (parstackMatch(parstack, t)) {
        // the closing bracket matches the last opening one on the stack, removing it
        parstack.pop();
      } else {
        // syntax error, the closing bracket didn't match the top of the stack
        return isValidAggName; // there's no point going forward if we can't keep track of brackets
      }

    } else if (arc.report.addAndCheckAggregates(t, dataConnectionType) && !parstack.includes('[')) {
      // seems like we've found a valid aggregation function name
      isValidAggName = true;
    }
  }

  // (parstack.length === 0) would indicate no syntax errors, but we'll ignore that check, this
  // method is about aggregation functions
  return isValidAggName;
};

arc.report.isSupportedAggregate = function(data, dataConnectionType) {
  data = data.trim();
  var al = data.split(/[,(]/);
  if (al.length === 0) {
    return false;
  }
  while (al.length) {
    if (al[0] === '') {
      al.shift();
    } else {
      return arc.report.addAndCheckAggregates(al[0], dataConnectionType);
    }
  }
  return false;
};

arc.report.columnTypeEnum = {
  DIMENSION: 1,
  AGGREGATE: 2,
  UNKNOWN: 3,
};

arc.report.base_colors = function() {
  return ['#0097a7', '#7dcad2', '#546e7a', '#b0bec5', '#ed8111',
          '#f6c088', '#e54954', '#f2a4a9', '#ad73b1', '#d6b9d8', '#2375b8',
          '#91badb', '#3e8111', '#b0cb73',
         ];
};

arc.report.contrast_colors = function() {
  return ["#0097A7", "#ED8111", "#2375B8", "#E54954", "#42996F",
          "#AD73B1", "#B0CB73", "#EA8DCD"];
};

arc.report.graded_colors = function() {
  return [ '#E0F7FA', '#B2EBF2', '#80DEEA', '#4DD0E1', '#26C6DA',
    '#00BCD4', '#00ACC1', '#0097A7', '#00838F', '#006064'];
};

arc.report.highcard_colors = function() {
  return [
    '#fd8d3c', '#fdae6b', '#fdd0a2', '#31a354', '#74c476', '#a1d99b',
    '#c7e9c0', '#756bb1', '#9e9ac8',
    '#bcbddc', '#dadaeb', '#636363', '#969696', '#bdbdbd', '#d9d9d9',
    '#1f77b4', '#aec7e8', '#ff7f0e', '#ffbb78', '#2ca02c', '#98df8a',
    '#d62728', '#ff9896', '#9467bd',
    '#c5b0d5', '#8c564b', '#c49c94', '#e377c2', '#f7b6d2', '#7f7f7f',
    '#c7c7c7', '#bcbd22', '#dbdb8d',
    '#17becf', '#9edae5', '#393b79', '#5254a3', '#6b6ecf', '#9c9ede',
    '#637939', '#8ca252', '#b5cf6b',
    '#cedb9c', '#8c6d31', '#bd9e39', '#e7ba52', '#e7cb94', '#843c39',
    '#ad494a', '#d6616b', '#e7969c',
    '#7b4173', '#a55194', '#ce6dbd', '#de9ed6', '#3182bd', '#6baed6',
    '#9ecae1', '#c6dbef', '#e6550d',
  ];
};

arc.report_xhr_serializer_parallel = function() {
  var token = function() { };

  token.next_call = function(url, parms, blocking_txt, get_fn) {
    if (parms) {
      arc.report_xhr_post(url, parms, get_fn);
    } else {
      arc.report_xhr(url, blocking_txt, get_fn);
    }
  };

  return token;
};

arc.report_xhr_serializer = function() {
  var queued_calls = [],
      current_call = false;

  var token = function() { };

  token.next_call = function(url, parms, blocking_txt, get_fn) {
    if (url) {
      queued_calls.push({
        'url': url,
        'parms': parms,
        'blocking_txt': blocking_txt,
        'get_fn': get_fn
      });
    }
    if (current_call) {
      return;
    }
    if (queued_calls.length === 0) {
      return;
    }
    current_call = true;
    var o = queued_calls.shift();
    var func = function(error, response) {
      current_call = false;
      try {
        o.get_fn(error, response);
      } finally {
        token.next_call();
      }
    };

    if (o.parms) {
      arc.report_xhr_post(o.url, o.parms, func);
    } else {
      arc.report_xhr(o.url, o.blocking_txt, func);
    }
  };

  return token;
};

arc.report_xhr = function(url, blocking_txt, get_fn) {
  d3.xhr(url).get(function(error, response) {
    get_fn(error, response);
  });
};

// Issue xhr via POST method
// DJC -- Couldn't get d3.xhr to add the parms object to the request
//        so using jquery/ajax which has slightly different return format.
arc.report_xhr_post = function(url, parms, get_fn) {

  $.ajax({
    url: url,
    data: parms,
    type: 'POST',
    beforeSend: function(xhr) {
      // Because of a jquery bug when handling PUT requests,
      // we have to explicitly set X-CSRFToken here
      xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
    }
  })
  .done(function(resp) {
    get_fn(null, {'response':resp});
  })
  .fail(function(req, textStatus, errorThrown) {
    get_fn(req.responseText || errorThrown || textStatus, null);
  });
};

// Validate custom expression for a dataset.
// Result via callback with error text or expression type/agg/ispart info
/* Query backend whether a given SQL expression is valid for the particular dataset/connection
 * @param  expr       string - SQL expression for validation
 * @param  dataset    object - dataset on which the expression is to be validated
 * @param  cb         function - function to call with server response
 * @param  exprType   string - type of expression being validated (filter, column, query)
 * @param  config     object - with following possible keys
 *                      inclds (boolean) whether to include dataset object
 *                      dsObj (object) dataset object to override default one from dataset
 *                      skipcol (string) column to be skipped in building dsreq
 *                      alias (string) alias of expression to be validated (for column expressions)
 * @return
 */
arc.validate_expression = function(expr, dataset, cb, exprType, config = {}) {

  var rb = arc.requestBuilder();

  rb.requestType(`.validate-${exprType}`);
  rb.validate(expr);

  rb.datasetId(dataset.dataset_id());

  var dsreq = rb.dsreq();
  if (config.inclds) {
    dsreq.dataset_object = dataset.buildInfo(config.skipcol);
  }

  if (config.dsObj) {
    dsreq.dataset_object = config.dsObj;
  }

  dsreq.alias = config.alias;

  const params = {
    'dsreq': JSON.stringify(dsreq),
    'cache_read': false,
  };

  arc.sqlrun.runSQLWithParams(
    params,
    function successCb(response) {
      const res = response.response;
      const isvalid = res.is_valid;
      if (isvalid === "false" || isvalid === "unknown") {
        const error_msg = res.error_msg || "Invalid query. Please see activity logs.";
        return cb({
          err: error_msg,
        });
      }

      const type = res.type;
      const hasagg = res.isaggregate !== "false";
      const ispart = res.ispart !== "false";
      return cb({
        type,
        hasagg,
        ispart,
      });
    },
    function errorCb(errorMsg) {
      return cb({
        err: errorMsg,
      });
    });
};

arc.queryResults = function() {
  var columns,
      data,
      colnames,
      coltypes,
      hivecomment,
      partitions = [];

  var results = function() { };

  results.data = function(_) {
    if (!arguments.length) return data;
    data = _;
    return results._setup();
  };

  results.partitions = function() {
    return partitions;
  };

  results.columns = function() {
    return columns;
  };

  results.colnames = function() {
    return colnames;
  };

  results.coltypes = function() {
    return coltypes;
  };

  results._setup = function() {
    colnames = data.colnames;
    coltypes = data.coltypes;
    hivecomment = data.hive_comments || [];

    columns = [];
    colnames.forEach(function(colname, colindex) {
      var colInfo = [colname, coltypes[colindex]];
      var comment = hivecomment[colindex];
      if (comment) {
        colInfo.push(comment.trim());
      }
      columns.push(colInfo);
    });

    partitions = [];
    if (data.partitions) {
      data.partitions.forEach(function(p) {
        partitions.push(p);
      });
    }

    return results;
  };

  return results;
};

arc.report_cleanup_shelves = function(shelf_data) {
  shelf_data.forEach(function(shelf_element) {
    var shelf_name = shelf_element[0];

    d3.select("#" + shelf_name)
      .remove();
  });
};

function add_list_items($ul, fqname, columns, partitions, config) {
  config = config || {};

  let showAll = config.showAll,
      searchTerm = config.searchTerm,
      maxCols = 200;

  // Don't include tables with no column in this catagory
  if (columns.length === 0) {
    return;
  }

  var tablename = arc.datasets.drop_dbname(fqname);

  var wrapper =
    d3.selectAll($ul.toArray())
      .append("div")
      .classed("field-list-wrapper", true);

  var header =
    wrapper.append("div")
           .classed("field-list-header", true)
           .attr("title", tablename);

  setUpHeaderSlideToggle(header);

  header.html("<span class='fa fa-caret-down'></span> " + tablename);

  // Custom tooltip for table name portion
  // Also, would like a better way to set the tooltip text to fqname but this at least works.
  header.each(function() {
    $(this).tooltip({
      container: 'body',
      placement: "top",
      delay: {
        show: 1000,
        hide: 100
      }
    }).attr('data-original-title', fqname);
  });

  if (searchTerm) {
    searchTerm = searchTerm.toLowerCase();
    columns = columns.filter(function(d) {
      return ((d.getComputedDisplayName().toLowerCase().search(searchTerm)) >= 0);
    });
  }

  let showMore = false;
  if (!showAll) {
    if (columns.length > maxCols) {
      columns = columns.slice(0, maxCols);
      showMore = true;
    }
  }

  // Gotta get tooltips in there
  var nokids = [];
  var cdtkids = [];

  columns.forEach(c => {
    let dscol = c.dataset().get_column(c.dataset_colname());
    if (!dscol) {
      nokids.push(c);
    } else if (dscol.hasComplexParent()) {
      cdtkids.push(c);
    } else {
      nokids.push(c);
    }
  });

  var dim_lis =
    wrapper.selectAll(".nosel")
      .data(nokids)
      .enter()
        .append("li")
        .attr('cdt-name', function (d) {
          return arc.datasets.is_complex_type(d.dataset_coltype()) ? d.dataset_colname() : '';
        })
        .classed('cdt-col', function (d) {
          return arc.datasets.is_complex_type(d.dataset_coltype());
        })
        .classed("selection_element", true)
        .classed("hierarchy", function (d) {
          return (d.dataset_dimhlist() && d.dataset_dimhlist().length > 0);
        })
        .append("span")
        .classed("field-wrapper", true);

  // Can be filtered based on class hierarchy as well
  const $dimLisHierarchy = dim_lis.filter((d) => (d.dataset_dimhlist() || []).length > 0).map($)[0];

  const $link = $('<a></a>');
  const $icon = $('<i class="fa fa-angle-down expand-icon"></i>')
          .on('click', function(e) {
              e.stopPropagation();
              $(this).toggleClass('fa-angle-down fa-angle-right');
              $(this).closest('.selection_element').find('ul').slideToggle();
            });

  $link.append($icon);
  $dimLisHierarchy.append($link);

  dim_lis.append('span')
         .attr('class', (d) => `dim-icon-type ${d.dataset_coltype_class()}`);

  dim_lis.append('span')
         .classed('selection_field', true)
         .attr('data-test-id', 'selection-field-item')
         .attr('data-test-item', (d) => d.getComputedDisplayName())
         .call(arc.report.shelf.settext);

  dim_lis.append('span')
         .classed('dropdown-placeholder', true);

  // adding comment triangle to field
  dim_lis.filter((d) => (d.dataset_colcomment && d.dataset_colcomment()) ||
                      (d.dataset_hivecomment && d.dataset_hivecomment()))
          .append('span')
          .classed('comment-triangle', true)
          .attr('data-toggle', 'popover')
          .attr('data-trigger', 'hover')
          .attr('data-placement', 'top')
          .attr('data-content', (d) => d.dataset_colcomment() || d.dataset_hivecomment());

  // Appending constituent elements for the hierarchy
  var dim_sublis_hierarchy = wrapper.selectAll("li.hierarchy").append("ul").selectAll(".nosel")
    .data(function (d) {
      return d.dataset_dimhlist();
    })
    .enter()
    .append("li")
    .append("span")
    .classed("field-wrapper", true);

  dim_sublis_hierarchy
    .append("span")
    .each(function(d) {
      var hierarchy_col = d3.select($(this).closest("li.hierarchy")[0]).data()[0];
      var Dataset = hierarchy_col.dataset();
      var column = Dataset.get_column(d);
      var classToAdd = 'dim-icon-type ' + column.coltype_class();
      $(this).addClass(classToAdd);
    });

  dim_sublis_hierarchy
    .append("span")
    .attr("title", function (d) {
      return d;
    })
    .text(function (d) {
      return d;
    });

  // adding comment triangle to hierarchy child field
  dim_sublis_hierarchy.filter(d => (d.dataset_colcomment && d.dataset_colcomment()) ||
                                   (d.dataset_hivecomment && d.dataset_hivecomment()))
                      .append('span')
                      .classed('comment-triangle', true)
                      .attr('data-toggle', 'popover')
                      .attr('data-trigger', 'hover')
                      .attr('data-placement', 'top')
                      .attr('data-content', d => d.dataset_colcomment() || d.dataset_hivecomment());

  if (partitions) {
    var partitions_hash = {};
    partitions.forEach(function(p) {
      partitions_hash[p] = 1;
    });
    dim_lis.filter(function(d) {
             return d.dataset_colname() in partitions_hash;
           })
           .append('div')
             .classed('partition-key', true)
             .attr('data-toggle', 'tooltip')
             .attr('title', 'Partition key')
             .each(function() {
               $(this).tooltip({
                 container: 'body', // makes tooltip display on top of content
                 delay: {
                   show: 500,
                   hide: 100
                 }
               });
             });
  }

  const $dimLisSensitive = dim_lis.filter((d) => d.isSensitiveElement()).map($)[0];

  $.each($dimLisSensitive, (_, elem) => {
    const $newSensitiveWarningSpan = $('<svg class="pl-1" style="font-size: 13px; padding-bottom: 3px;" width="1.3em" height="1.3em" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M12 4.43C13.31 5.29 15.56 6.62 18 7.48V13C18 14.6 15.44 17.73 12 19.73C8.56 17.73 6 14.61 6 13V7.48C8.44 6.62 10.69 5.29 12 4.43ZM12 2C12 2 8 5 4 6V13C4 16 8 20 12 22C16 20 20 16 20 13V6C16 5 12 2 12 2ZM16.54 10.44L15.13 9.03L11.01 13.15L8.89 11.03L7.48 12.44L11.02 15.98L16.56 10.44H16.54Z" fill="#2e89c5"/></svg>');

    $newSensitiveWarningSpan.attr('data-toggle', 'tooltip')
                            .attr('title', 'Sensitive field')
                            .tooltip({
                                container: 'body',
                                delay: {
                                  show: 500,
                                  hide: 100
                                }
                              });

    $(elem).append($newSensitiveWarningSpan);
  });

  var base_template = $('#builder-field-base-template').html();
  var cdt_template = $('#builder-field-cdt-template').html();
  var $div = $(header[0]).closest('.field-list-wrapper');

  nokids.forEach(d => {
    let name = d.dataset_colname();
    let type = d.dataset_coltype();
    if (arc.datasets.is_complex_type(type)) {
      let $col = $div.find(`[cdt-name="${d.dataset_colname()}"]`);
      let $new = $(arc.misc.safe_substitute(cdt_template,
        {COL_name: name,
         HTML_coltype_class: d.dataset_coltype_class()}));
      $new.data('shelf_elem', d);
      $col.replaceWith($new);
      if (d.dataset_colcomment() || d.dataset_hivecomment()) {
        const commentCorner = $('<span class="comment-triangle"></span>');
        const comment = d.dataset_colcomment() ? d.dataset_colcomment() : d.dataset_hivecomment();
        commentCorner.attr('data-toggle', 'popover')
                     .attr('data-trigger', 'hover')
                     .attr('data-placement', 'top')
                     .attr('data-content', comment);
        $new.find('.field-wrapper').append(commentCorner);
      }
    }
  });

  cdtkids.forEach(d => {
    let name = d.dataset_colname();
    let dscol = d.dataset().get_column(name);
    let dspar = dscol.getComplexParent();
    let $par = $div.find(`[cdt-name="${dspar.display_name()}"]`);
    let templ = dscol.isComplexType() ? cdt_template : base_template;
    let $new = $(arc.misc.safe_substitute(templ,
        {COL_name: name,
         HTML_coltype_class: d.dataset_coltype_class()}));
    $new.data('shelf_elem', d);
    if (d.dataset_colcomment() || d.dataset_hivecomment()) {
      const commentCorner = $('<span class="comment-triangle"></span>');
      const comment = d.dataset_colcomment() ? d.dataset_colcomment() : d.dataset_hivecomment();
      commentCorner.attr('data-toggle', 'popover')
                   .attr('data-trigger', 'hover')
                   .attr('data-placement', 'top')
                   .attr('data-content', comment);
      $new.find('.field-wrapper').append(commentCorner);
    }
    $par.find('.cdt-children').append($new);
  });

  if (showMore) {
    wrapper.selectAll('.nosel')
      .data([0])
      .enter()
        .append('div')
          .append('a')
          .classed('cols-showmore', true)
          .text('Show more');
  }
}

function setUpHeaderSlideToggle(d3_header){
  d3_header.on('click', function() {
    var $header = $(this);
    var $caret = $header.children('.fa-caret-down, .fa-caret-right');

    if ($caret.hasClass("fa-caret-right")) {
      $header.siblings("li").removeClass("arc-invisible");
      $header.siblings("li:visible")
             .css({"display" : "none"})
             .slideDown();
    } else if ($caret.hasClass("fa-caret-down")) {
      var $itemsToHide = $header.siblings('li:visible');
      var postAnimateProcess = function(){
        $header.siblings("li").addClass("arc-invisible");
        $itemsToHide.show();
      };
      $itemsToHide.slideUp({"done": postAnimateProcess});

      // Jquery's slideUp doesn't trigger "done" if there are no items
      // Trigger it manually here
      if ($itemsToHide.length === 0) {
        postAnimateProcess();
      }
    }
    $caret.toggleClass('fa-caret-down, fa-caret-right');
  });
}

arc.report.add_derived_list_item = function($container, wrapper_id, header_id, element, ismeasure) {
  let wrapper = d3.select(`#${wrapper_id}`);

  if (wrapper.empty()) {
    wrapper = d3.selectAll($container.toArray())
      .insert('div', ':first-child')
      .classed('field-list-wrapper derived-list-wrapper', true)
      .attr('id', wrapper_id);

    const header = wrapper.append('div')
      .classed('field-list-header derived-data-header', true)
      .attr('id', header_id)
      .attr('style', 'display: none;');

    setUpHeaderSlideToggle(header);

    header.append('span')
      .attr('class', 'fa fa-caret-down');

    header.append('i')
      .attr('class', 'fa fa-flask');
    header.append('span').text(' Derived Data');
  }

  const lis = `
    <a
      href="#"
      class="dropdown-item deriveddata-viewdefinition-popover"
      data-toggle="popover"
    >
      View Definition
    </a>

    <div class="dropdown-divider"></div>

    <a href="#" class="dropdown-item del-view-popover" data-toggle="popover">
      <i class="fa fa-trash-o"></i>
      Delete
    </a>
  `;

  const dim_lis =
    wrapper.selectAll('#nosel')
      .data([element])
      .enter()
        .append('li')
        .classed('selection_element dropdown', true)
        .append('span')
        .classed('field-wrapper', true);

  dim_lis.append('span')
         .attr('class', (d) => `dim-icon-type ${d.dataset_coltype_class()}`);

  dim_lis.append('span')
         .classed('selection_field', true)
         .call(arc.report.shelf.settext);

  dim_lis.append('span')
         .classed('dropdown-placeholder', true);

  dim_lis.append('a')
         .attr({ href: '#' })
         .classed('dropdown-toggle caret-padding derived-caret', true)
         .attr({ 'data-toggle': 'dropdown' })
         .attr({ 'area-expanded': 'true' });

  dim_lis.append('div')
         .classed('dropdown-menu', true)
         .html(lis);

  const $badge = $('#visual-data-panel').find(`.app-${ismeasure ? 'aggregates' : 'dimensions'}-count`);
  $badge.text(parseInt($badge.text(), 10) + 1);

  arc.globals.reports
             .getPrimaryReport()
             .viz()
             .shelves()?.setup_dimagg_draggable();
};

const visualDimAggSearchbarSel = '#visual-data-panel .dim-agg-searchbox';

arc.report.dataset_setup = function(dataset, norefresh) {
  if (arc.globals.is_edit) {
    // Empty out previously set value from the searchbox
    $(visualDimAggSearchbarSel).val('');

    $('#app-dataset').text(dataset.dataset_name());
    $('#app-dataset').attr('data-original-title', dataset.dataset_name());

    arc.report.datasetSetupCols({
      dataset,
      includeAggregateColumns: true,
      $dimShelf: $('#visual-data-panel .dim-app-params'),
      $aggShelf: $('#visual-data-panel .agg-app-params'),
      config: {
        setupColumnSearch: true,
      },
      includeSegment: true,
    });
  }

  if (!norefresh) {
    arc.report.reportRefresh();
  }
};

arc.report.columnToShelfElement = function(dataset, column) {

  return arc.report.shelf.element()
            .dataset(dataset)
            .dataconnection_type(dataset ? dataset.dataconnection_type() : undefined)
            .expr_hasagg(column.attr('exprhasagg'))
            .dflagg(column.attr('dflagg'))
            .dataset_dimhlist(column.attr('dimh'))
            .dataset_colname(column.attr('alias') || column.attr('name'))
            .dataset_coltype(column.attr('cast') || column.attr('type'))
            .display_format(column.attr('displayformat'))
            .dataset_isdim(column.attr('isdim'))
            .dataset_geotype(column.attr('geotype'))
            .isSensitiveElement(column.attr('isSensitive'))
            .dataset_colcomment(column.attr('comment'))
            .dataset_hivecomment(column.attr('hivecomment'));
};

arc.report.shelfElementPostProcess = function(config) {
  var shelf_elem = config.shelf_elem,
      shelf_type = config.shelf_type,
      shelf_property = config.shelf_property,
      shelf_id = config.shelf_id,
      tgtshelf_is_aggregate = config.tgtshelf_is_aggregate;

  // Note that the 'datum' function cannot be passed in
  // a function ('shelf_elem'). It presumes that a function is
  // to be invoked, not stored. So we instead create an anonymous
  // function that returns the function object we want to store
  // instead.
  var se = shelf_elem.clone();

  if (shelf_type === arc.report.shelf_types.PREDICATE) {
    se.filter_column(se.dataset_colname());
  }

  if (config.discardFunctionName) {
    se.fn_name(undefined);
  }

  if (shelf_type === arc.report.shelf_types.COLUMN && shelf_id !== 'geo_shelf') {
    if ((shelf_property === arc.report.columnTypeEnum.AGGREGATE) ||
        ((shelf_property === arc.report.columnTypeEnum.UNKNOWN) &&
         tgtshelf_is_aggregate)) {

      var t = se.dataset_coltype().toLowerCase();

      if (!se.expressionIsAggregate()) {

        // The target shelf is an aggregate shelf, or the target
        // shelf is mixed but the source column is from the measures
        // shelf. Apply an aggregation function on the column.
        se.fn_type(arc.report.fn_types.AGGREGATE);

        // Default aggregation depending on data types
        // (Unless we already have a default agg specified or if
        //  the custom expression contains an agg already)
        if (se.expr_hasagg()) {
          se.fn_name(undefined);
        } else if (se.dflagg()) {
          se.fn_name(se.dflagg());
        } else {
          se.fn_name(arc.misc.defaultAgg(t));
        }
      }
    }
  }

  // DJC -- getExpression() ignores alias, cast, and agg if the
  //        custom_expr is set.  However, in the dataset attributes
  //        page, we want to allow a custom expression where each
  //        of these still applies.
  //        So for now, here just as the element is dropped onto a
  //        shelf, we modify the custom expression to apply these
  //        modifiers.
  se.custom_expr(shelf_elem.custom_expr());
  if (se.custom_expr()) {
    var c = se.custom_expr();
    if (se.fn_cast()) {
      c = `cast(${c} as ${se.fn_cast()})`;
    }

    if (!arc.misc.isWindowFunction(c)) {
      if (se.fn_name()) {
        if (se.fn_name() === 'countdistinct') {
          c = `count(distinct (${c}))`;
        } else {
          c = `${se.fn_name()}(${c})`;
        }
      }
    }

    se.custom_expr(c);
  }

  return se;
};

arc.report.reportRefresh = function() {
  var report = arc.report.getPrimaryReport();
  if (report.viz()) {
    report.viz().setup();
  }
};

arc.report.datasetSetupCols = function(_) {
  let {
    dataset,
    includeRecordCount = true,
    includeAggregateColumns,
    $dimShelf,
    $aggShelf,
    config,
    includeSegment,
    alwaysIncludeSegment,
  } = _;

  if (!includeAggregateColumns) {
    includeRecordCount = false;
  }

  config = config || {};
  config.callback = config.callback || function () {};

  const {
    setupColumnSearch,
  } = config;

  if (arc.dataconnection.isSamplingSupported(dataset.dataconnection_type())) {
    $('.dataset-sample-mode').show();
  } else {
    $('.dataset-sample-mode').hide();
  }

  // Emptying dims and aggs, clearing all events and setting up click handlers
  $dimShelf.empty().off();
  $aggShelf.empty().off();

  $dimShelf.on('click.showMoreDims', '.cols-showmore', function showMoreDimensions() {
      $(this).css('cursor', 'wait');
      config.showAllDims = true;
      showCols();
  });

  $aggShelf.on('click.showMoreAggs', '.cols-showmore', function showMoreAggregates() {
    $(this).css('cursor', 'wait');
    config.showAllMeasures = true;
    showCols();
  });

  $dimShelf.on('click.addDimFilter', '.selection_element', config.callback);

  $aggShelf.on('click.showAggFilter', '.selection_element', config.callback);

  var dimlist_len = 0;
  var measurelist_len = 0;

  const segmentsCount = dataset.segments().length;

  dataset.tables().forEach(function(table, idx) {
    var fqname = table.tablename();
    var dimensions = [];
    if (alwaysIncludeSegment || (includeAggregateColumns && includeSegment && segmentsCount > 0)) {
      if (idx === 0) {
        dimensions.push(arc.report.shelf.segment_element().dataset(dataset));
      }
    }

    if (includeAggregateColumns && dataset.dataconnection_type() === 'elasticsearch') {
      dimensions.push(arc.report.shelf.full_text_score_element(dataset.dataconnection_type()).dataset(dataset));
    }
    table.dimensionsAndHierarchies().forEach(function(d) {
      if (!d.attr('hidden')) {
        var dim = arc.report.columnToShelfElement(dataset, d);
        dimensions.push(dim);
      }
    });

    add_list_items($dimShelf, fqname, dimensions, table.partitions(), {
      searchTerm: config.searchTerm,
      showAll: config.showAllDims,
    });

    var measures = [];
    if (includeRecordCount) {
      if (idx === 0) {
        measures.push(arc.report.shelf.record_count_element(dataset.dataconnection_type()).dataset(dataset));
      }
    }
    table.measuresOnly().forEach(function(d) {
      if (!d.attr('hidden')) {
        if (!includeAggregateColumns && d.attr('exprhasagg')) {
          return;
        } else {
          var measure = arc.report.columnToShelfElement(dataset, d);
          measures.push(measure);
        }
      }
    });

    add_list_items($aggShelf, fqname, measures, undefined, {
      searchTerm: config.searchTerm,
      showAll: config.showAllMeasures,
    });

    dimlist_len += dimensions.length;
    measurelist_len += measures.length;

      // enable popovers
    $('[data-toggle="popover"]').popover({ container: 'body' });
  });

  function showCols() {
    arc.report.datasetSetupCols({
      dataset,
      includeAggregateColumns,
      $dimShelf,
      $aggShelf,
      config,
      includeSegment,
    });

    let report = arc.report.getPrimaryReport();
    let searchTerm = config.searchTerm || '';
    searchTerm = searchTerm.toLowerCase();

    if (report.viz()) {
      let derivedData = report.viz().derivedData();
      const derivedDataList = derivedData.derivedDataList();
      derivedDataList.forEach(function addIfInSearch(ddItem) {
        if (ddItem.label.toLowerCase().search(searchTerm) >= 0) {
          derivedData.addDerivedDataField(ddItem, dataset);
        }
      });

      let shelves = report.viz().shelves();

      if (shelves) {
        shelves.setup_dimagg_draggable();
      }
    }
  }

  $dimShelf.closest('.tab-panel-content').find('.app-dimensions-count').text(dimlist_len);
  $aggShelf.closest('.tab-panel-content').find('.app-aggregates-count').text(measurelist_len);

  if (setupColumnSearch) {
    const $searchbox = $(visualDimAggSearchbarSel);

    let c = {
      $searchbox,
      search_fn: function() {
        clearTimeout(c.maxTimeId);
        c.maxTimeId = undefined;

        var v = $searchbox.val();
        config.searchTerm = v;
        showCols();
      },
    };

    arc.report.triggerOnSearch(c);
  }
};

// Helper function for selecting text within an element
arc.report.select_text = function(element) {
  var doc = document,
      text = doc.getElementById(element),
      range, selection;
  if (doc.body.createTextRange) { // ms
      range = doc.body.createTextRange();
      range.moveToElementText(text);
      range.select();
  } else if (window.getSelection) { // all others
      selection = window.getSelection();
      range = doc.createRange();
      range.selectNodeContents(text);
      selection.removeAllRanges();
      selection.addRange(range);
  }
};


// TODO: make unit tests
// 'abc !def ghi'; => { exclude: ['def'], include: ['abc', 'ghi']}
// '\'one term\' "one term" "quote\'s" "double\\"quotes" \'single\\\'quotes\' \'mixed"quotes\' "mixed\'quotes"';
//  {include: ['\'one term', 'one term', 'quote\'s', 'double"quotes', 'single\'quotes', 'mixed"quotes', 'mixed\'quotes'], exclude: []}
// 'include !exclude !!exclude "!include" !"exclude" !"! exclude" "! include';
// {include: ['include', '!include', '! include'], exclude: ['exclude', '!exclude', 'exclude', '! exclude']}

const _tokenizeQuery = str => {
  const include = [];
  const exclude = [];

  let word = '';
  let quoteEnclosed = false;
  let quoteType = null;
  let negated = false;

  const _reset = () => {
    (negated ? exclude : include).push(word);
    word = '';
    negated = false;
    quoteEnclosed = false;
    quoteType = null;
  };

  for (let idx = 0; idx < str.length; idx++) {
    const [prev, curr, next] = [-1, 0, 1].map((n) => str[idx + n] || '');

    if (!word) {
      if (curr === ' ') {
        continue;
      }

      if (curr === '"' || curr === "'") {
        quoteEnclosed = true;
        quoteType = curr;
        continue;
      }

      if (curr === '!' && !quoteEnclosed && !negated) {
        negated = true;
        continue;
      }
    }

    if (word) {
      if (curr === '\\' && quoteEnclosed && next === quoteType) {
        continue;
      }

      if (curr === quoteType && prev !== '\\') {
        _reset();
        continue;
      }

      if (curr === ' ' && !quoteEnclosed) {
        _reset();
        continue;
      }
    }
    word += curr;

    if (!next) {
      _reset();
    }
  }

  return { exclude, include };
};


arc.report.buildSearch = ({ searchKeys, caseSensitive }) => query => {
  const { include, exclude } = _tokenizeQuery(query);

  if ([...include, ...exclude].length === 0) {
    return obj => ({ obj, priority: 1});
  }

  let flags = '';
  flags += caseSensitive ? '' : 'i';

  const excludeRegex = exclude.length ? new RegExp(exclude.join('|'), flags) : null;
  const includeRegex = RegExp(include.join('|'), flags);

  return obj => {
    let found = false;
    let priority = 1;
    for (var key of searchKeys) {
      const value = String(obj[key]);

      if (excludeRegex && excludeRegex.test(value)) {
        return null;
      }

      if (!found) {
        found = includeRegex.test(value);

        if (found && !excludeRegex) {
          return { obj, priority };
        }

        priority++;
      }
    }

    return found ? { obj, priority } : null;
  };
};

arc.report.searchboxSetup = function(searchboxConfig) {
  let { searchboxId,
        search_cb,
        sel_cb,
        initialValue, } = searchboxConfig;
  // Search functionality:
  //   Words are looked for individually (not next to each other)
  //   Quoted strings are looked for together
  //   ! used before a word is used to negate the search
  //     - i.e. !map will look for all reports not containing the word 'map'
  //

  var $searchbox = $(searchboxId);
  var search_fn = function() {
    var gridListSel = sel_cb();
    var s = $searchbox.val();
    var terms = s.match(/[^\s"]+|"[^"]+"/g);
    function searchForSelection(d, _this) {
      var terms_found = true;
      $.each(terms, function (tidx, term) {
        term = term.toLowerCase();
        term = term.replace(/^"+|"+$/g, '');
        var invert = false;
        if (term[0] === "!") {
          invert = true;
          term = term.slice(1);
        }

        if (term.length === 0) {
          return true;
        }

        let term_found = search_cb(d, term, _this);
        if (invert) {
          if (term_found === true) {
            terms_found = false;
            return true;
          }
        } else {
          if (term_found === false) {
            terms_found = false;
            return true;
          }
        }

        return false;
      });

      return terms_found;
    }

    // TODO: Change this filter function
    // We are using the d3 filter function improperly here.  It is designed to use on
    // data passed into d3 as an array, not on an array of nodes as it is here.  This
    // results in 'd' always being undefined and needing to use 'this' to run the search
    // on the nodes in the d3 selection
    function hackyD3filter(d) {
      return searchForSelection(d, this);
    }

    // fix for bootstrap accordion:
    $(gridListSel[0]).parents('.collapse.show').removeClass('show');

    if (terms) {
      gridListSel.style('display', 'none');
      const resultArray = gridListSel.filter(hackyD3filter);
      resultArray.style('display', null);

      // expand the ones containing results:
      $(resultArray[0]).parents('.collapse:not(.show)').addClass('show');
    } else {
      gridListSel.style('display', null);
    }
  };

  arc.report.triggerOnSearch({
    $searchbox,
    search_fn,
    initialValue,
    searchImmediately: true,
  });

  return search_fn;
};

arc.report.triggerOnSearch = (config) => {
  var $searchbox = config.$searchbox,
      search_fn = config.search_fn,
      initialValue = config.initialValue,
      searchImmediately = config.searchImmediately;

  const maxTime = 600;
  const minTime = 200;

  function debounceAndThrottleSearch() {
    if (!config.maxTimeId) {
      config.maxTimeId = setTimeout(search_fn, maxTime);
    }

    if (config.minTimeId) {
      clearTimeout(config.minTimeId);
    }

    config.minTimeId = setTimeout(search_fn, minTime);
  }

  $searchbox.off('input').on('input', debounceAndThrottleSearch);
  $searchbox.siblings('.attr-search-clear')
            .off('click')
            .on('click', function(e) {
              $searchbox.focus();
              $searchbox.val('');
              debounceAndThrottleSearch();
            });

  if (initialValue !== undefined) {
    $searchbox.val(initialValue);
  }

  if (searchImmediately && $searchbox.val()) {
    // Found a value in the search box
    search_fn();
  }

  // Remove focus from the searchbox on enter
  $searchbox.off('keypress').on('keypress', function(event) {
    if (event.which === arc.misc.ENTER_KEY) {
      $searchbox.blur();
    }

    // Do not propagate newline onwards
    return event.which !== 13;
  });
}

//
// Used in segment edit/create and event edit/create
// Handles UI for list of input fields that need expression editors
// Accepts following parameters
//         ulID                ID of unordered list element that holds inputs
//         initialElements     array of initial values that inputs should have
//         $addBtn             jQuery element that when clicked should add one new element to list
//         dataset             object - dataset for which expression is being generated
//         config              object which may include  following properties
//            title               title for SQL editor modal
//            helptext            text to go above SQL editor in expanded modal
//            noAggs              boolean indicating that aggregate expressions should error
//            exprType            type of SQL expression being entered (filter for filters,
//                                     column for entities)
//            tables              array of tables for which expression editors are to be generated
//                                  (otherwise will use all tables in dataset)
//            includeTables    boolean indicating whether or not table names should be included in
//                              autocompletes (default is false)
//            excludeRecordCountAutocomplete  boolean indicating whether or not record count measure
//                                  should be excluded
arc.report.addableExprList = function(ulID, initialElements, $addBtn, dataset, config) {
  const ulSel = `#${ulID}`;
  const $ulSel = $(ulSel);
  const placeholder = 'Click to update in SQL expression editor';

  const openSqlModal = function() {
    const $this = $(this);
    const value = $this.val();
    const $sqlModal = $('#sql-expr-edit');
    if (arc.misc.modalShown($sqlModal)) {
      return;
    }

    const divId = 'sql-filter-expr';
    $sqlModal.find('.modal-title').text(config.title);
    const helptext = config.helptext || '';
    $sqlModal.find('.form-text').text(helptext);
    $sqlModal.find('.expression-data').attr('id', divId);
    const editor = arc.misc.expressionEditor(dataset, null, divId, $sqlModal, {
      tables: config.tables,
      includeTables: config.includeTables,
      excludeRecordCountAutocomplete: config.excludeRecordCountAutocomplete,
      includeCalculated: true,
    });
    editor.value(value);
    $sqlModal.find('.alert').remove();
    $sqlModal.off('shown.bs.modal')
             .on('shown.bs.modal', () => editor.focusEditor());
    $sqlModal.modal('show');
    const $validateBtn = $sqlModal.find('.expression-validate');
    const $applyBtn = $('#sql-expr-apply');
    const validatorConfig = {
      submitCb: acceptValidExpression,
      noAggs: config.noAggs,
      srcElem: $this,
    };

    if (config.exprType === 'filter') {
      validatorConfig.exprModifierCb = editor.exprEditorMapperFilters;
    }

    arc.misc.addExpressionValidation(
      $validateBtn,
      $applyBtn,
      editor,
      dataset,
      config.exprType,
      null,
      validatorConfig
    );

    function acceptValidExpression() {
      $this.val(editor.value());
      $sqlModal.modal('hide');
    }

   $sqlModal.off('hide.bs.modal')
            .on('hide.bs.modal', function cleanUpModal() {
              editor.destroy();
              $sqlModal.find('.alert').remove();
            });
  };

  const handleFilterDelete = function() {
    $(this).closest('li').remove();
    updateDeleteVisibility();
  };

  // hide delete icons if there is only one filter input field
  function updateDeleteVisibility() {
    $ulSel.find('.fa-minus-circle').toggleClass('arc-invisible', $ulSel.find('li').length === 1);
  }

  d3.select(ulSel)
    .selectAll('li')
    .remove();

  if (!initialElements) {
    initialElements = [];
  }

  const lis =
    d3.select(ulSel)
      .selectAll('#nosel')
      .data(initialElements)
      .enter()
        .append('li');

  addInput(lis);

  $addBtn.off('click.addFilter')
         .on('click.addFilter', addNewFilter);

  // Takes d3 selector for lis that need input, creates input elements with appropriate listeners
  function addInput(lis) {
    lis.append('input')
       .attr('type', 'text')
       .attr('placeholder', placeholder)
       .attr('class', 'form-control modal-custom-expr')
       .attr('value', d => d)
       .on('focus.expandInput', openSqlModal);

    lis.append('span')
       .attr('class', 'fa fa-minus-circle')
       .on('click.removefilter', handleFilterDelete);
  }

  function addNewFilter() {
    const li =
      d3.select(ulSel)
        .append('li');

    addInput(li);
    updateDeleteVisibility();
  }

  addNewFilter();
};

arc.report.fetchDatasetApps = function({ datasetId, successCb, errorCb, includeUnlinked, includeAll }) {
  const _errorCb = errorCb || (err => arc.report.alertMessage("Could not load apps", err));
  let url = `${arc.frontend.globals.urls.reportUrl}?dataset_id=${datasetId}`;
  if (includeUnlinked || includeAll) {
    url += '&include_all=true';
  }

  $.ajax({
    url,
    method: 'GET',
    dataType: 'json',
  }).done(successCb)
    .fail(_errorCb);
};

arc.report.fetchDatasetPossibleDeletion = function({ datasetId, successCb, errorCb }) {
  const _errorCb = errorCb || ((err) => arc.report.alertMessage('Could not load apps', err));

  $.ajax({
    url: arc.frontend.globals.urls.possibleDeletedReportsUrl,
    data: { dataset_id: datasetId },
    type: 'GET',
  }).done(successCb)
    .fail(_errorCb);
};

arc.report.drawPermissionedApps = function (_apps) {
  const apps = _apps.filter(d => arc.misc.hasPerm('ds_appview', d.perm));
  if (apps.length === 0) {
    $("#viz-delete-apps-label").hide();
    $("#apps-table").hide();
  } else {
    drawApps(apps);
  }
};

arc.report.toTemplate = function(app) {
  // only show delete for linked viz if not in any dashboards
  const related = JSON.parse(app.shared_visual_dashboards || '[]');
  const deleteable = related.length === 0 || app.report_type === 'dashboard';
  const deletePerm = arc.misc.hasPerm('ds_appedit', app.perm) && deleteable;
  return {
    APP_URL: app.url,
    VISUAL_TYPE: app.type === 'dashboard' ? 'dashboard' : 'visual',
    APP_ID: app.id,
    APP_TYPE: app.report_type,
    APP_IMG: app.img || '',
    APP_TITLE: app.title || '',
    HTML_DATE_CREATED_FORMATTED: arc.misc.friendlyDate(app.created),
    HTML_DATE_UPDATED_FORMATTED: arc.misc.friendlyDateSince(app.updated),
    APP_UPDATED_BY: app.updated_by,
    APP_VIEWS_COUNT: app.views_count || '',
    EDIT_PERM: arc.misc.hasPerm('ds_appedit', app.perm) ? '' : ' arc-invisible',
    EDIT_URL: app.edit_url,
    WORKSPACE_NAME: app.workspace_name,
    WORKSPACE_URL: `${arc.frontend.globals.urls.appsUrl}?group=${app.workspace_name}&grouptype=workspaces`,
    DELETE_PERM: deletePerm ? '' : ' arc-invisible',
    CSS_COUNT: related.length,
    HTML_APPS: arc.misc.appsToLinks(related, 5),
  };
};

// Used within apps_apps.html and apps_dataset.html
// This helper function renders apps within the 'apps-table' table
function drawApps(apps) {
  const $table = $('#apps-table');
  const template = $('#apps-table-template').html();

  // disable sort on Thumbnails, Action columns
  const actionColumnIdx = $table.find("tr:first td").length - 1;
  const non_sortable_cols = [0, actionColumnIdx];

  const tableSettings = {
     paging: true,
     ordering: true,
     columnDefs: [{ targets: non_sortable_cols, orderable: false }],
     order: [[5, 'desc']],
   };

  const $appsDataTable = arc.misc.DataTable($table, tableSettings);
  $($appsDataTable.table().container())
    .addClass('d-flex flex-column justify-content-start align-items-stretch');
  let numberRelatedApps = 0;
  apps.forEach(function(app) {
    // Filter out apps that user doesn't have view permissions for
    // Or if is unlinked visual
    const isUnlinkedVisual = app.report_type !== 'dashboard' && !app.shared_visual_dashboards;
    const noPermission = !arc.misc.hasPerm('ds_appview', app.perm);
    if (isUnlinkedVisual || noPermission) {
      return;
    }

    const templateObj = arc.report.toTemplate(app);
    const tableRow = $(arc.misc.safe_substitute(template, templateObj));

    $appsDataTable.rows.add(tableRow);

    numberRelatedApps += 1;
  });

  $('#dataset-apps-count').text(numberRelatedApps);

  $appsDataTable.draw();
  $appsDataTable.columns.adjust();

  $('ul li a[href=\\#apps]').on('shown.bs.tab', $appsDataTable.columns.adjust);

  $table.find("tbody").on("click", "tr a.app-app-edit", function (e) {
    e.preventDefault();
    e.stopPropagation();
    window.open($(e.currentTarget).data('edit-url'), '_self');
  });

  $table.find('tbody').on('click', 'tr a.app-app-delete', function (e) {
    e.preventDefault();
    e.stopPropagation();

    const $tr = $(e.currentTarget).closest('tr');
    const objectType = $tr.find('.object-type').hasClass('visual') ? 'visual' : 'dashboard';

    const $modal = $(`#delete-${objectType}-modal`);
    if (arc.misc.modalShown($modal)) {
      return;
    }

    // Show the delete confirmation modal,
    // and delete when confirmed
    // This code below needs consolidation: there seem to be
    // multiple places where an app is getting deleted:
    // reporttracker.js, reportbuilder.js, groups.html

    $(`#delete-${objectType}-yes`).off('click.deleteapp').on('click.deleteapp', function () {
      arc.reporttracker.report()
         .report_id($tr.data("app-id"))
         .report_type($tr.data('app-type'))
         .delete(function (_) {
            $('[data-href="' + $tr.data("href") + '"]').remove();

            //  Update badge
            var $count = $("#dataset-apps-count"),
              oldCount = $count.text();
            $count.text(oldCount - 1);
         });
    });

    $modal.modal('show');
  });
  /*
   if (arcl.first_sort) {
     arcl.first_sort = false;
     if (arcl.initialsort == "popular") {
      //TODO: sortList: [[5,1]]
     } else {
       //TODO: sortList: [[3,1]]
     }
   } else {
      $("#apps-table").trigger('update');
   }
   */

  // Enable tooltips
  $('[data-toggle="tooltip"]').tooltip();
  // // Enable popover
  // $('[data-toggle="popover"]').popover();
}

}());
