window.arc = window.arc || {};

(function() {
"use strict";

arc.reporttracker = {};

arc.reporttracker.unique_report_id = 1;

arc.reporttracker.report = function() {
  var report_id,
      build_mode,
      dataset,
      dataset_id,
      lastUpdatedDate,
      lastUpdatedUser,
      report_type = 'table',
      report_description,
      shared_visual_dashboards,
      workspace,
      saved_data,
      loader,
      loader_dom_id,
      chart_div,
      title_elem = 'viz-title',
      subtitle_elem = 'viz-subtitle',
      params,
      click_behavior_cb,
      in_userlist,
      viz,
      redraw_cb,
      save_segment_cb,
      dataset_segments,
      dataset_events,
      dashboard,
      unique_report_id,
      in_dashboard,

      /*
       * X/Y position of this report in a dashboard.
       * This information is used to prioritize queries generated by
       *   this visual in relation to other visuals - we go top to bottom and then
       *   left to right on priority. See report.queryPriority() definition and
       *   usage.
       */
      dashboardX,
      dashboardY,

      shelves_dom_id,
      premodalConfig,
      modalConfig,
      shelfTrackerInstance = arc.report.shelftracker(),
      visual_params_history,
      permitParamHistory,
      showEmptyTitleSubtitle,
      oldReportSearchText,
      reportSearchText;

  var report = function() { return "arc.reporttracker.report"; };
  unique_report_id = arc.reporttracker.unique_report_id;
  arc.reporttracker.unique_report_id += 1;

  report.unique_report_id = function() {
    return unique_report_id;
  };

  report.clearShelfHistory = shelfTrackerInstance.clearHistory;

  report.showEmptyTitleSubtitle = function (_) {
    if (!arguments.length) return showEmptyTitleSubtitle;
    showEmptyTitleSubtitle = _;
    return report;
  };

  report.lastUpdatedDate = function (_) {
    if (!arguments.length) return lastUpdatedDate;
    lastUpdatedDate = _;
    return report;
  };

  report.lastUpdatedUser = function (_) {
    if (!arguments.length) return lastUpdatedUser;
    lastUpdatedUser = _;
    return report;
  };

  report.permitParamHistory = function (_) {
    if (!arguments.length) return permitParamHistory;
    permitParamHistory = _;
    return report;
  };

  report.isOldSearchText = function (title) {
    return title === oldReportSearchText;
  };

  report.reportSearchText = function (_) {
    if (!arguments.length) return reportSearchText;
    oldReportSearchText = reportSearchText;
    reportSearchText = _;
    return report;
  };

  report.visual_params_history = function (_) {
    if (!arguments.length) return visual_params_history;
    visual_params_history = _;
    return report;
  };

  report.setupParamsControl = function (_) {
    if (!visual_params_history) {
      let vph = arc.report.params_history();
      vph.dashboard(_ || report);
      visual_params_history = vph;
    }

    if (shouldParamControlsBeVisible()) {
      visual_params_history.setupControls(report.viz().inModal());
    } else {
      visual_params_history.hideControls();
    }

    return report;
  };

  function shouldParamControlsBeVisible() {
    let db = report.dashboard();
    return (
            report.permitParamHistory() &&
            (
              !arc.globals.is_dashboard ||
              (
                report.viz() &&
                report.viz().inModal() &&
                db &&
                db.displayContextMenu()
              )
            )
    );
  }

  report.params = function(_) {
    if (!arguments.length) return params;
    params = _;
    if (viz) {
      viz.params(params);
    }

    if (permitParamHistory) {
      visual_params_history.add(params);
    }
    return report;
  };

  report.current_params = function () {
    return visual_params_history.current_params();
  };

  report.redrawExistingWidgetsWithParams = function(datum, index,
                             in_click_behavior, params_fn, viz, elem,
                             in_params_data, dontUnpivot) {
    if (!permitParamHistory) {
      return report;
    }

    var params_data = in_params_data;
    if (params_data === undefined) {
      if (params_fn) {
        params_data = params_fn(datum, index, elem);
      }
    }

    visual_params_history.add(params_data, viz, dontUnpivot);
    // TODO: Chances of double page refresh here when params are added.
    // If below line is commented and params are not updated but Refresh is hit
    // then the visual won't refresh.
    report.redrawExistingWidgetsWithParamsData(
        in_params_data, null, null, report.viz().params().getCopy());
    return report;
  };

  report.highlightMarks = function () {
    if (viz) {
      viz.highlightMarks();
    }
  };

  report.redrawExistingWidgetsWithParamsData = function(new_params_data, viz,
    updateFromParamHistory, prev_params_data, forward) {
    if (arc.globals.send_message_params) {
      /*
       Send information about current parameters to any containing window.

       The containing window needs to contain code similar to this:

       <head></head>
       <body>
       <script>
       window.addEventListener("message", receiveMessage, false);

       function receiveMessage(event) {
       // This is where you receive the posted messages
       console.log(event.data);
       }
       </script>

       <iframe src="http://127.0.0.1:8000/arc/apps/app/106?embed=true&send_message_params=true">
       </iframe>
       </body>
       */
      if (window.parent !== window) {
        window.parent.postMessage({
          'type': 'parameters',
          'data': new_params_data
        }, "*");
      }
    }

    // Clear previous errors when parameter navigation causes visuals refresh
    arc.report.alertMessageAllHide();

    if (!viz) {
      viz = report.viz();
      if (!viz) {
        return;
      }
    }

    if (!arc.arraysIdentical(prev_params_data, new_params_data)) {
      viz.prevParams(arc.misc.paramsFromData(prev_params_data));
      viz.params(arc.misc.paramsFromData(new_params_data));
      viz.unselectAll();
      if (updateFromParamHistory && !forward) {
        viz.unpivotParamsExcept(new_params_data);
      } else {
        viz.pivotParams(new_params_data);
      }

      report.refresh();
    }

    // Intentionally returning dashboard
    return dashboard;
  };

  report.shelves_dom_id = function(_) {
    if (!arguments.length) return shelves_dom_id;
    shelves_dom_id = _;
    if (viz) {
      viz.shelves_dom_id(shelves_dom_id);
    }
    return report;
  };

  report.dataset_segments = function(_) {
    if (!arguments.length) {
      if (viz) {
        return viz.dataset_segments();
      }

      return dataset_segments;
    }

    dataset_segments = _;
    if (viz) {
      viz.dataset_segments(dataset_segments);
    }
    return report;
  };

  report.dataset_events = function(_) {
    if (!arguments.length) return dataset_events;
    dataset_events = _;
    if (viz) {
      viz.dataset_events(dataset_events);
    }
    return report;
  };

  report.click_behavior_cb = function(_) {
    if (!arguments.length) return click_behavior_cb;
    click_behavior_cb = _;
    if (viz) {
      if (click_behavior_cb) {
        viz.click_behavior_cb(click_behavior_cb);
      }
    }
    return report;
  };

  report.save_segment_cb = function(_) {
    if (!arguments.length) return save_segment_cb;
    save_segment_cb = _;
    if (viz) {
      viz.save_segment_cb(save_segment_cb);
    }
    return report;
  };

  report.in_dashboard = function(_) {
    if (!arguments.length) return in_dashboard;
    in_dashboard = _;
    return report;
  };

  report.dashboardXY = function(_) {
    if (!arguments.length) return [dashboardX, report.dashboardY];
    [dashboardX, dashboardY] = _;
    return report;
  };

  report.queryPriority = function() {
    if (dashboardX !== undefined) {
      return (dashboardX * 1000 + dashboardY);
    }

    return 0;
  };

  report.redraw_cb = function(_) {
    if (!arguments.length) return redraw_cb;
    redraw_cb = _;
    return report;
  };

  report.title_elem = function(_) {
    if (!arguments.length) return title_elem;
    title_elem = _;
    return report;
  };

  report.subtitle_elem = function(_) {
    if (!arguments.length) return subtitle_elem;
    subtitle_elem = _;
    return report;
  };

  report.loader = function(_) {
    if (!arguments.length) return loader;
    loader = _;
    return report;
  };

  report.in_userlist = function(_) {
    if (!arguments.length) return in_userlist;
    in_userlist = _;
    return report;
  };

  report.unmodal = function() {
    // Revert to the pre-modal configuration
    if (premodalConfig) {
      chart_div = premodalConfig.chart_div;
      shelves_dom_id = premodalConfig.shelves_dom_id;
      title_elem = premodalConfig.title_elem;
      subtitle_elem = premodalConfig.subtitle_elem;
      loader_dom_id = premodalConfig.loader_dom_id;

      if (viz) {
        viz.user_settings().cleanupSettings();
        viz.chart_div(chart_div)
           .shelves_dom_id(shelves_dom_id)
           .title_elem(title_elem)
           .subtitle_elem(subtitle_elem)
           .inModal(false);

        resetLargeTrellisTemp(viz);

        if (viz.loader()) {
          loader.swapDomIds(loader_dom_id);
        }
      }
    }

    premodalConfig = undefined;
    modalConfig = undefined;

    return report;
  };

  report.modalConfig = function(_) {
    if (!arguments.length) return modalConfig;

    /*
     * We are being given a configuration that should be
     * used as the visual gets rendered within a modal.
     *
     * When .unmodal() is called later, we need to revert back
     * to saved settings.
     */

    // Remember the current config
    premodalConfig = {
      chart_div: chart_div,
      shelves_dom_id: shelves_dom_id,
      title_elem: title_elem,
      subtitle_elem: subtitle_elem,
      loader_dom_id: loader.dom_id(),
    };

    // Use the new one
    modalConfig = _;
    chart_div = modalConfig.chart_div;
    shelves_dom_id = modalConfig.shelves_dom_id;
    title_elem = modalConfig.title_elem;
    subtitle_elem = modalConfig.subtitle_elem;
    loader_dom_id = modalConfig.loader_dom_id;

    let params_data = modalConfig.params_data || [];

    if (viz) {
      viz.chart_div(chart_div)
         .shelves_dom_id(shelves_dom_id)
         .title_elem(modalConfig.title_elem)
         .subtitle_elem(modalConfig.subtitle_elem)
         .inModal(true);

      resetLargeTrellisTemp(viz);

      if (viz.loader()) {
        loader.swapDomIds(loader_dom_id);
      }
    }

    report.setupParamsControl(report);
    if (arc.globals.is_dashboard) {
      report.visual_params_history().baseParamsl(params_data);
    } else {
      report.visual_params_history().setCheckPointState();
    }

    return report;
  };

  function resetLargeTrellisTemp(viz) {
    if (viz.viz() && viz.viz().allowLargeTrellisTemp) {
      viz.viz().allowLargeTrellisTemp(false);
    }
  }

  report.chart_div = function(_) {
    if (!arguments.length) {
      if (viz) {
        return viz.chart_div();
      }

      return chart_div;
    }

    chart_div = _;
    if (viz) {
      viz.chart_div(_);
    }

    return report;
  };

  report.report_description = function(_) {
    if (!arguments.length) return report_description;
    report_description = _;
    return report;
  };

  report.reportWidgetId = function() {
    return `uri-${report.unique_report_id()}-widget-${report.report_id()}`;
  };

  function reportAppId() {
    return `uri-${report.unique_report_id()}-widget-${report.report_id()}-a`;
  }

  report.updateSharedIconInDashboard = function() {
    const $dashboardSharedIcon = $(`#${reportAppId()}`).closest('.arc-viz-container')
                                                       .find('.linked-visual-icon.in-visual');
    if ($dashboardSharedIcon && $dashboardSharedIcon.length) {
      $dashboardSharedIcon.toggleClass('arc-invisible', !report.shared_visual_dashboards());
    }
  };

  report.shared_visual_dashboards = function(_) {
    if (!arguments.length) return shared_visual_dashboards;
    shared_visual_dashboards = _;
    return report;
  };

  report.workspace = function(_) {
    if (!arguments.length) return workspace;
    workspace = _;
    return report;
  };

  report.report_type = function(_) {
    if (!arguments.length) return report_type;
    report_type = _;
    return report;
  };

  report.saved_data = function(_) {
    if (!arguments.length) return saved_data;
    saved_data = _;
    return report;
  };

  report.report_id = function(_) {
    if (!arguments.length) return report_id;
    report_id = _;
    if (viz) {
      viz.report_id(report_id);
    }
    return report;
  };

  report.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    if (viz) {
      viz.dashboard(dashboard);
    }
    return report;
  };

  report.build_mode = function(_) {
    if (!arguments.length) return build_mode;
    build_mode = _;
    return report;
  };

  report.dataset_id = function(_) {
    if (!arguments.length) return dataset_id;
    dataset_id = _;
    return report;
  };

  report.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    if (viz) {
      viz.dataset(dataset);
    }
    return report;
  };

  report.viz = function(_) {
    if (!arguments.length) return viz;
    //TODO: It doesn't look like below code is ever executed
    viz = _;
    viz.chart_div(chart_div)
       .title_elem(title_elem)
       .subtitle_elem(subtitle_elem)
       .showEmptyTitleSubtitle(showEmptyTitleSubtitle)
       .in_dashboard(in_dashboard)
       .loader(loader)
       .params(arc.globals.params)
       .redraw_cb(redraw_cb)
       .save_segment_cb(save_segment_cb)
       .dataset_segments(dataset_segments)
       .dataset_events(dataset_events)
       .dashboard(dashboard)
       .report_id(report_id)
       .shelfTrackerInstance(shelfTrackerInstance)
       .inModal(premodalConfig !== undefined);

    if (click_behavior_cb) {
      // Set only if click_behavior_cb is not null, since otherwise
      // it will override the default viz callback
      viz.click_behavior_cb(click_behavior_cb);
    }
    return report;
  };

  report.redraw = function(isWindowResize) {
    if (viz) {
      viz.redraw(isWindowResize);
      if (viz.redraw_cb()) {
        viz.redraw_cb()(viz, isWindowResize);
      }
    }
    return report;
  };

  function afterFilters(cb) {
    arc.misc.afterCountersEqual({
      counter1: 'refresh_start_filter',
      counter2: 'refresh_end_filter',
      delay: 50,
      finalTimeout: 30000,
      cb,
    });
  }

  report.refresh = function(force_redraw, _filter_widget, manualRefresh) {
    if (!viz || !viz.dataset()) {
      return report;
    }

    /*
     * In dashboards, if the setting that indicates visuals should be drawn after filters
     * is set, then we honor that below.
     */

    let visualsAfterFiltersEnabled = false;
    if (arc.globals.dashboard) {
      visualsAfterFiltersEnabled = arc.globals.dashboard.visualsAfterFiltersEnabled();
    }

    if (manualRefresh || !visualsAfterFiltersEnabled || _filter_widget ||
        !viz.refreshCount || viz.refreshCount() > 0) {
      viz.refresh(force_redraw, _filter_widget, manualRefresh);
    } else {
      afterFilters(function() {
        viz.refresh(force_redraw, _filter_widget, manualRefresh);
      });
    }

    return report;
  };

  report.updateSharedVisualDashboards = function(successCb, failCb) {
    if (report.shared_visual_dashboards()) {
      var dedup = new Set();
      JSON.parse(report.shared_visual_dashboards()).forEach(v => dedup.add(v));
      report.shared_visual_dashboards(JSON.stringify(Array.from(dedup)));
    }

    const post_data = {
      csrfmiddlewaretoken: arc.globals.csrf_token,
      shared_visual_dashboards: report.shared_visual_dashboards(),
      shared_visual_only: 'true',
      dataset_id: report.dataset_id(),
      dashboard_id: arc.misc.dashboard_id(),
    };
    const url = `${arc.frontend.globals.urls.reportUrl}/${report.report_id()}`;

    $.ajax({
      url,
      data: post_data,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .done((result) => {
      // Need to update saved info if successfully unlinked/linked
      arc.staticfetcher.update([
        {
          asset: url,
          cb: function updateReport(data) {
            data[0].shared_visual_dashboards = report.shared_visual_dashboards();
          },
        },
      ]);

      if (successCb) {
        successCb(result);
      }
    })
    .fail((req) => {
      if (failCb) {
        failCb(req);
      }

      // Notify of failure
      arc.report.alertMessage("Failed to update linked visual " + report.report_id(),
          req.responseText ? req.responseText : req,
          { is_html: true, alert_report_id: report_id });
    });
  };

  report.delete = function(success_cb, failure_cb, sourceDashboardId='') {
    if (report_id <= 0) {
      if (success_cb) {
        success_cb();
      }
      return report;
    }

    // Make sure we remove this report from the static fetcher.
    // TODO: There are likely other places this is needed--like when a report is
    //       updated.  Or deleted via another code point.
    if (arc.staticfetcher) {
      arc.staticfetcher.forget([
        {
          asset: `${arc.frontend.globals.urls.reportUrl}/${report_id}`,
        },
      ]);
    }

    var post_data = {
      "csrfmiddlewaretoken": arc.globals.csrf_token,
    };

    const objectType = report.report_type() === 'dashboard' ? 'dashboard' : 'visual';
    let url = `${arc.frontend.globals.urls.reportUrl}/${report.report_id()}`;
    if (sourceDashboardId) {
      url += '?source_dashboard=' + sourceDashboardId;
    }

    $.ajax({
      url: url,
      data: post_data,
      type: 'DELETE',
      beforeSend: (xhr) => {
        // Because of a jquery bug when handling DELETE requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader("X-CSRFToken", arc.globals.csrf_token);
      },
    })
    .done(() => {
      if (success_cb) {
        success_cb(arc.globals.csrf_token);
      }
    })
    .fail((req) => {
      // Notify of failure
      arc.report.alertMessage(`Failed to delete ${objectType} ${report.report_id()}`,
          req.responseText ? req.responseText : req,
          { is_html: true, alert_report_id: report_id });
      if (failure_cb) {
        failure_cb(req);
      }
    })
    .always(function() {
      $(`#delete-${objectType}-modal`).modal('hide');
    });
    return report;
  };

  // Discard changes from edit visual modal upon closing modal or cloning visuals
  // This destroys modal-specific elements, resets report data, and undos any pivots
  report.discardChanges = function(refreshUponClose) {
    report.unmodal();

    var previousVisualType = arc.globals.singleVisualBuilder.visual_type();
    if (previousVisualType && (previousVisualType !== report.report_type())) {
      report.viz().shelfTrackerInstance().clearHistory();

      // Properly replace report_data/info with saved information
      if (report.saved_data()) {
        const currentReportData = report.viz().shelfTrackerInstance().info();
        const savedReportData = JSON.parse(report.saved_data()).report_data;
        Object.assign(currentReportData, savedReportData);
      }

      report.switchViz(previousVisualType);
    }

    let reportSetup = report.viz().setup(),
        report_vph = report.visual_params_history(),
        dashboard_ph = arc.globals.is_dashboard && dashboard.params_history();

    if (!report.saved_data()) {
      // TODO: Consider removing this --
      // we are now including saved_data even for new visuals
      reportSetup.restore({});
      if (dashboard_ph && report_vph) {
        dashboard_ph.add(report_vph.current_params()[0], report_vph.current_params()[1]);
      }
    } else {
      reportSetup.restore(JSON.parse(report.saved_data()).report_data);
      if (dashboard_ph) {
        if (arc.globals.is_edit) {
          report.viz().unpivotParamsExcept(dashboard_ph.current_params()[0]);
          report.viz().pivotParams(dashboard_ph.current_params()[0]);
          report_vph.clear();
        } else {
          if (!(report_vph &&
              arc.arraysIdentical(report_vph.current_params()[0], report_vph.baseParamsl()))) {

            // dashboard state should be a, b, b1, b2
            // TODO: Need to add all of states of report_history.
            //  Currently we are adding b1 but not b2.

            dashboard_ph.add(report_vph.current_params()[0], report_vph.current_params()[1]);
          }
        }
      } else {
        // single visual mode with no changes in visual while being in edit modal
        let cps = report_vph.getCheckPointState();
        report.viz().unpivotParamsExcept(cps[0]);
        report.viz().pivotParams(cps[0]);
        report_vph.setVisibility();
      }

      // if in edit mode
        // dashboard state should stick to a, b, c
      // if in view mode
        // if history items were added, then
          // dashboard state should be a, b, b1, b2
        // if history items were not added, then
          // dashboard state should stick to a, b, c
    }
  };

  report.switchViz = function(_report_type, settings, enableRefresh=true) {
    var info,
        sql_query,
        sql_output,
        dsreq,
        click_behavior;

    arc.report.alertMessageAllHide();
    if (viz) {
      viz.unpivotParamsExcept([]);
      if (settings) {
        info = settings;
      } else {
        info = viz.save();
      }
      sql_query = viz.sqlQuery();
      sql_output = viz.sqlOutput();
      dsreq = viz.dsreq();
      click_behavior = viz.dashboard_click_behavior();

      $(`#${viz.chart_div()} .viz-removeonswitch`).remove();
    }

    report.createViz(_report_type);

    if (viz) {
      viz.dashboard_click_behavior(click_behavior);
      if (arc.globals.is_dashboard) {
        viz.click_behavior_change_cb(
          function (marker_name, filtering_enabled) {
            var report_id = viz.report_id();
            if (arc.globals.dashboard) {
              arc.globals.dashboard.save_click_behavior(report_id, marker_name,
                filtering_enabled);
            }
          });
      }
    }

    if (info && viz) {
      if (arc.globals.is_edit) {
        viz.page_needs_save(true);
      }

      // The 'rawtable' visual needs the sql query
      if (sql_query) {
        viz.sqlQuery(sql_query);
        viz.dsreq(dsreq);
      }

      if (viz.viz_type() === 'rawtable') {
        // in rawtable restore, the display is set to info.sql_text. We are setting
        // that to the sql that get generated by django

        if (sql_output === undefined) {
          return;
        }

        var query = sql_output.info.filter(function (s)  {
          return s.indexOf('Query:') === 0;
        });

        if (query.length !== 1) {
          arc.report.alertMessage("Cannot get query string");
          return;
        }

        info.sql_text = query[0].substring('Query:'.length);
      }

      viz.restore(info, /* switchingVisualType */ !settings);

      if (sql_query && sql_output) {
        viz.sqlOutput(sql_output);
      }

      if (arc.globals.dashboard) {
        arc.globals.dashboard.activateReport(report, /* force_redraw */ true);
      }
    }
  };

  report.createViz = function(_report_type, isNew) {
    if (_report_type && (report_type !== _report_type)) {
      report_type = _report_type;
    }

    if (viz) {
      viz.cleanup();
      viz.active(false);
      viz = undefined;
    }

    if (report_type === 'singlevalue') {
      report_type = 'kpi';
    }

    if (report_type === 'sunburst') {
      report_type = 'pie';
    }

    let permitParamHistory = false;

    const vizMap = {
      flow: () => arc.report.path().flow_type('flow'),
      funnel: () => arc.report.funnel(),
      'column-stats': () => arc.report.columnstats(),
    };

    const oldBar = {
      bar: () => arc.report.bar().chart_type('bar'),
      'bar-horizontal': () => arc.report.bar().chart_type('bar').is_horizontal(true),
      line: () => arc.report.bar().chart_type('line'),
      lines: () => arc.report.bar().chart_type('lines'),
      'lines-agg': () => arc.report.bar().chart_type('lines-agg'),
      stackedbar: () => arc.report.stackedbar().chart_type('bar'),
      'stackedbar-agg': () => arc.report.stackedbar_agg(),
      'stackedbar-horizontal': () => arc.report.stackedbar().chart_type('bar').is_horizontal(true),
      'stackedbar-agg-horizontal': () => arc.report.stackedbar_agg().is_horizontal(true),
      stackedarea: () => arc.report.stackedbar().chart_type('area'),
      area: () => arc.report.bar().chart_type('area'),
    };

    if (arc.globals.enable_old_bar) {
      permitParamHistory = true;
      Object.assign(vizMap, oldBar);
    }

    viz = vizMap[report_type] && vizMap[report_type]();

    if (viz === undefined) {
      permitParamHistory = report_type.startsWith('trellis-') || report_type === 'timeline';
      var extension = arc.visual.getExtension(report_type);
      if (extension) {
        viz = arc.report.driver(extension.viz());
      } else {
        console.log(report_type, ": report type not found");
        throw (report_type + ": report type not found");
      }
    }

    if (viz) {
      report.populateViz(viz, report_type, isNew, permitParamHistory);
    }

    return report;
  };

  report.populateViz = function(viz, report_type, isNew, permitParamHistory) {
    if (!viz) {
      return;
    }

    let paramsToApply = !in_dashboard ? arc.globals.params :
      arc.misc.paramsFromData(dashboard.params_history().current_params()[0]);

    viz.chart_div(chart_div)
      .report(report)
      .title_elem(title_elem)
      .subtitle_elem(subtitle_elem)
      .showEmptyTitleSubtitle(showEmptyTitleSubtitle)
      .in_dashboard(in_dashboard)
      .loader(loader)
      .params(paramsToApply)
      .redraw_cb(redraw_cb)
      .save_segment_cb(save_segment_cb)
      .dataset_segments(dataset_segments)
      .dataset_events(dataset_events)
      .build_mode(build_mode)
      .viz_type(report_type)
      .dashboard(dashboard)
      .report_id(report_id)
      .dataset(dataset)
      .shelfTrackerInstance(shelfTrackerInstance)
      .inModal(premodalConfig !== undefined)
      .report(report);

    report.permitParamHistory(permitParamHistory);

    report.setupParamsControl(report.dashboard() || report);
    if (arc.globals.is_dashboard) {
      visual_params_history.baseParamsl(paramsToApply.getkvsList());
    } else {
      visual_params_history.setCheckPointState();
    }

    if (shelves_dom_id) {
      viz.shelves_dom_id(shelves_dom_id);
    }

    if (click_behavior_cb) {
      // Set only if click_behavior_cb is not null, since otherwise
      // it will override the default viz callback
      viz.click_behavior_cb(click_behavior_cb);
    }

    viz.setup();

    if (isNew || (report_id === arc.report.REPORTID_NEW)) {
      viz.title('');
      viz.subtitle('');

      viz.setupInitialFilters();
      viz.restoreLimit({
        limit: 100,
      });

      const newSettings = {
        report_type: viz.viz_type(),
        report_data: viz.viz().viz_getSettings(),
      };

      report.saved_data(JSON.stringify(newSettings));
    }
  };

  return report;
};

arc.reporttracker.reports = function() {
  var all_reports_on_sheets = {}, // hashed on key being sheet_id
      all_reports = {}, // hashed on key being report_id
      number_of_reports = {},
      currentSheetId,
      currentSheetReports, // hashed on key being report_id
      build_mode,
      loader,
      title_elem = 'viz-title',
      subtitle_elem = 'viz-subtitle',
      click_behavior_cb,
      redraw_cb,
      save_segment_cb,
      dataset_segments,
      dataset_events,
      params;

  var reports = function() { return "arc.reporttracker.reports"; };

  reports.switchSheets = function(sheetId) {
    currentSheetId = sheetId;
    if (all_reports_on_sheets[currentSheetId]) {
      currentSheetReports = all_reports_on_sheets[currentSheetId];
    } else {
      currentSheetReports = {};
      number_of_reports[currentSheetId] = 0;
      all_reports_on_sheets[currentSheetId] = currentSheetReports;
    }
  };

  // Set sheet and currentSheetId to 1 for tests and non-dashboard pages
  reports.switchSheets(1);

  reports.vizCallback = function(fn) {
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        var viz = currentSheetReports[id].viz();
        if (viz) {
          fn(viz);
        }
      }
    }
    return reports;
  };

  reports.params = function(_) {
    if (!arguments.length) {
      if (params) {
        return params;
      }

      return arc.globals.params;
    }

    params = _;

    // 'params' is being set. Send it down to all owned reports
    // ..whether or not they belong to the same dataset
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].params(params);
      }
    }

    return reports;
  };

  reports.dssw = function(_) {
    if (!arguments.length) return dssw;
    dssw = _;
    return reports;
  };

  reports.numberReports = function(sheet_id) {
    if (sheet_id) {
      return number_of_reports[sheet_id];
    }

    return number_of_reports[currentSheetId];
  };

  reports.redraw_cb = function(_) {
    if (!arguments.length) return redraw_cb;
    redraw_cb = _;

    // 'redraw_cb' is being set. Send it down to all owned reports
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].redraw_cb(redraw_cb);
      }
    }
    return reports;
  };

  reports.dataset_segments = function(_) {
    if (!arguments.length) return dataset_segments;
    dataset_segments = _;

    // 'dataset_segments' is being set. Send it down to all owned reports
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].dataset_segments(dataset_segments);
      }
    }
    return reports;
  };

  reports.dataset_events = function(_) {
    if (!arguments.length) return dataset_events;
    dataset_events = _;

    // 'dataset_events' is being set. Send it down to all owned reports
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].dataset_events(dataset_events);
      }
    }
    return reports;
  };

  reports.save_segment_cb = function(_) {
    if (!arguments.length) return save_segment_cb;
    save_segment_cb = _;

    // 'save_segment_cb' is being set. Send it down to all owned reports
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].save_segment_cb(save_segment_cb);
      }
    }
    return reports;
  };

  reports.getSharedUnsharedVisuals = function() {
    const sharedVisuals = [];
    const unsharedVisuals = [];
    for (var id in all_reports) {
      if (all_reports.hasOwnProperty(id) && id >= 0) {
        const report = all_reports[id];
        if (report.shared_visual_dashboards()) {
          sharedVisuals.push(parseInt(id));
        } else {
          unsharedVisuals.push(parseInt(id));
        }
      }
    }

    return {
      sharedVisuals,
      unsharedVisuals,
    };
  };

  reports.title_elem = function(_) {
    if (!arguments.length) return title_elem;
    title_elem = _;
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].title_elem(title_elem);
      }
    }
    return reports;
  };

  reports.subtitle_elem = function(_) {
    if (!arguments.length) return subtitle_elem;
    subtitle_elem = _;
    for (var id in currentSheetReports) {
      if (currentSheetReports.hasOwnProperty(id)) {
        currentSheetReports[id].subtitle_elem(subtitle_elem);
      }
    }
    return reports;
  };

  reports.click_behavior_cb = function(_) {
    if (!arguments.length) return click_behavior_cb;
    click_behavior_cb = _;
    return reports;
  };

  reports.loader = function(_) {
    if (!arguments.length) return loader;
    loader = _;
    return reports;
  };

  reports.getAllReports = function() {
    const distinctReports = { ...all_reports };
    delete distinctReports[-1];
    return Object.values(distinctReports);
  };

  reports.getPrimaryReport = function() {
    if (-1 in all_reports) {
      return all_reports[-1];
    }

    return reports.addReport(-1);
  };

  reports.setReport = function (report, report_id) {
    all_reports_on_sheets[report_id] = report;
    all_reports[report_id] = report;
  };

  // Just add an ID and build/debug mode info
  // If a report with the report_id exists, we are done
  // If an already create report has been passed in, use it
  // Otherwise create a new report with the specified report_id
  reports.addReport = function(report_id, dashboard, report) {
    let sheet_id = dashboard ? dashboard.sheet_id() : currentSheetId;

    let addReportSheet = all_reports_on_sheets[sheet_id];

    if (!addReportSheet) {
      addReportSheet = {};
      all_reports_on_sheets[dashboard.sheet_id()] = addReportSheet;
    }

    if (report_id in addReportSheet) {
      return addReportSheet[report_id];
    }

    if (report) {
      addReportSheet[report_id] = report;
      all_reports[report_id] = report;
      report.dashboard(dashboard);
    } else {
      all_reports[report_id] = arc.reporttracker.report()
                                 .loader(loader)
                                 .title_elem(title_elem)
                                 .subtitle_elem(subtitle_elem)
                                 .redraw_cb(redraw_cb)
                                 .save_segment_cb(save_segment_cb)
                                 .dataset_segments(dataset_segments)
                                 .dataset_events(dataset_events)
                                 .dashboard(dashboard)
                                 .setupParamsControl(dashboard)
                                 .build_mode(reports.isBuildMode());

      addReportSheet[report_id] = all_reports[report_id];

      if (report_id >= 0) {
        addReportSheet[report_id].report_id(report_id);
      }
    }

    if (report_id >= 0) {
      const updateSheetId = sheet_id || currentSheetId;
      if (number_of_reports[updateSheetId]) {
        number_of_reports[updateSheetId]++;
      } else {
        number_of_reports[updateSheetId] = 1;
      }
    }

    if (click_behavior_cb) {
      addReportSheet[report_id].click_behavior_cb(click_behavior_cb);
    }

    return addReportSheet[report_id];
  };

  reports.getReport = function(report_id) {
    if (!(report_id in all_reports)) {
      // console.log("report_id", report_id, "not available in all_reports_on_sheets");
      return (null);
    }

    return all_reports[report_id];
  };

  reports.getAllSheetReports = function (sheet_id) {
    return $.extend({}, all_reports_on_sheets[sheet_id]);
  };

  reports.removeReport = function (report_id, sheet_id) {
    const sheetId = sheet_id || currentSheetId;
    if (!(report_id in all_reports)) {
      // console.log("report_id", report_id, "not available in all_reports_on_sheets to delete");
      return (null);
    }

    if (arc.globals.parameters &&
        all_reports[report_id].report_type() === 'picklist') {
      arc.globals.parameters.deletePicklist(all_reports_on_sheets[sheetId][report_id]);
    }

    if (all_reports_on_sheets[sheetId][report_id]) {
      delete all_reports_on_sheets[sheetId][report_id];
    }

    const presentOnAtLeastOneSheet = Object.values(all_reports_on_sheets)
    .some((sheet) => (report_id in sheet));

    if (!presentOnAtLeastOneSheet) {
      delete all_reports[report_id];
    }

    if (report_id >= 0 && number_of_reports[sheetId]) {
      number_of_reports[sheetId]--;
    }
  };

  reports.build_mode = function(_) {
    if (!arguments.length) return build_mode;
    build_mode = _;
    return reports;
  };

  reports.isBuildMode = function() {
    return (build_mode === "build");
  };

  return reports;
};

}());
