window.arc = window.arc || {};

(function () {
  "use strict";

  arc.report = arc.report || {};

  arc.report.savetable = function () {

    var $modal = $("#create-newtable"),
        $spinner = $("#create-newtable-loading-spinner");

    let default_db,
        dataconnection_id,
        dataconnection_type,
        dataset_id,
        sqlEditor,
        vizDriver;

    var savetable = function() { };

    savetable.defaultDb = function(_) {
      if (!arguments.length) return default_db;
      default_db = _;
      return savetable;
    };

    savetable.vizDriver = function(_) {
      if (!arguments.length) return vizDriver;
      vizDriver = _;
      return savetable;
    };

    savetable.dataconnectionId = function(_) {
      if (!arguments.length) return dataconnection_id;
      dataconnection_id = _;
      return savetable;
    };

    savetable.dataconnectionType = function(_) {
      if (!arguments.length) return dataconnection_type;
      dataconnection_type = _;
      return savetable;
    };

    savetable.datasetId = function(_) {
      if (!arguments.length) return dataset_id;
      dataset_id = _;
      return savetable;
    };

    function queryCreatesKSQLTTable() {
      vizDriver = vizDriver || arc.report.getPrimaryViz();
      const datarequest = vizDriver.shelvesToDataRequest();
      return (datarequest.agg_cnt() > 0);
    }

    function modalDefaults() {
      $modal.find('input[type=text]').val("");
      $modal.find('input[name="newTableRadio"][value="materialized"]').prop('checked', true);
      $modal.find('input[name="tableFormatRadio"][value="parquet"]').prop('checked', true);
      $modal.find('#newtable-storedAsDiv').removeClass('arc-invisible');
      $modal.find('#newtable-databaseDiv').removeClass('arc-invisible');
      $modal.find('#newtable-dbname').val("");
      $modal.find('input[type=checkbox][value="newt_newds"]').prop('checked', false);
      $modal.find('#newt_ds').addClass('arc-invisible');
      $modal.find('#newt_ds_name').val("");
      $modal.find('#newt_ds_name').prop('required', false);
      $modal.find('#create-newtable-error').addClass('arc-invisible');
      $modal.find('#create-newtable-error-detail').text("");
      $modal.find('.view_name').removeClass('arc-invisible');
      $modal.find('.create-dataset').removeClass('arc-invisible');
      $modal.find('.object-name').text('Table');
      $modal.find('#newt_sql').addClass('arc-invisible');

      vizDriver = vizDriver || arc.report.getPrimaryViz();
      if (vizDriver.viz_type() === 'rawtable') {
        $modal.find('.form-text').addClass('arc-invisible');
      } else {
        $modal.find('.form-text').removeClass('arc-invisible');
      }

      let $options = $modal.find('#typeOfView');

      if (dataconnection_type === 'ksql') {
        $options.find('div[value="materialized"]').addClass('arc-invisible');
        $options.find('div[value="logical"]').addClass('arc-invisible');
        $options.find('div[value="virtual"]').addClass('arc-invisible');
        $('#newt_view_name').css('text-transform', 'uppercase');
        $('#newt_view_name').attr('placeholder', '');
        $('#newtable-storedAsDiv').addClass('arc-invisible');
        $('#newtable-databaseDiv').addClass('arc-invisible');

        return;
      }

      $('#newt_view_name').attr('placeholder', 'enter table name');
      $('#newt_view_name').css('text-transform', '');

      $options.find('div[value="ksql_stream"]').addClass('arc-invisible');
      $options.find('div[value="ksql_table"]').addClass('arc-invisible');

      // On non-arcengine or if no upload permissions, then only option is as
      // a virtual dataset (i.e. the result of a query).
      if (!arc.misc.hasPerm('dc_upload', vizDriver.dataset().dataset_perms()) ||
          !arc.dataconnection.isArcEngineCompatible(dataconnection_type)) {
        $options.find('div[value="materialized"]').addClass('arc-invisible');
        $options.find('div[value="logical"]').removeClass('arc-invisible');
        $modal.find('input[name="newTableRadio"][value="virtual"]').prop('checked', true);
        pickVirtual();
      } else {
        $options.find('div[value="materialized"]').removeClass('arc-invisible');
        $options.find('div[value="logical"]').removeClass('arc-invisible');
        $options.find('div[value="virtual"]').removeClass('arc-invisible');
      }

      if (['hyve', 'hive'].includes(dataconnection_type)) {
        pickLogicalView();
        $modal.find('input[name="newTableRadio"][value="materialized"]').prop('checked', false);
        $options.find('div[value="materialized"]').addClass('arc-invisible');
      }
    }

    function pickTable() {
      $('#newt_sql').addClass('arc-invisible');
      $modal.find('#newtable-storedAsDiv').removeClass('arc-invisible');
      $modal.find('#newtable-databaseDiv').removeClass('arc-invisible');
      $modal.find('.view_name').removeClass('arc-invisible');
      $modal.find('.create-dataset').removeClass('arc-invisible');
      $modal.find('.object-name').text('Table');
      $modal.find('#newt_view_name').attr('placeholder', 'enter table name');
    }

    function pickLogicalView() {
      $('#newt_sql').addClass('arc-invisible');
      $modal.find('#newtable-storedAsDiv').addClass('arc-invisible');
      $modal.find('#newtable-databaseDiv').removeClass('arc-invisible');
      $modal.find('.view_name').removeClass('arc-invisible');
      $modal.find('.create-dataset').removeClass('arc-invisible');
      $modal.find('.object-name').text('View');
      $modal.find('#newt_view_name').attr('placeholder', 'enter view name');
    }

    function setupSQLEditor() {
      if (sqlEditor) {
        sqlEditor.destroy();
      }

      $('#newt_sql').find('.rawsql-show-container')
                    .remove();
      $('#newt_sql').append($('#rawsql-show-template').html());

      const editorFieldId = `viz-expreditor-${arc.misc.uniqueId()}`;
      const $editor = $('#newt_sql');
      $editor.find('.expression-field').attr('id', editorFieldId);

      vizDriver = vizDriver || arc.report.getPrimaryViz();

      sqlEditor = arc.misc.expressionEditor(vizDriver.dataset(), null, editorFieldId, $editor);
      if (vizDriver.viz_type() === 'rawtable') {
        let q = vizDriver.viz().sqlText();
        q = arc.misc.removeLimitFromSQL(q);
        sqlEditor.getSession().setValue(q);
        return;
      }

      // If this is not a rawsql visual, we have to get the contents of the query
      // from the backend to display it
      sqlEditor.getSession().setValue('Loading...');
      vizDriver.getVisualSQL(function(r) {
        let q = vizDriver.queryFromResponse(r.response);
        q = arc.misc.removeLimitFromSQL(q);
        sqlEditor.getSession().setValue(q);
      }, function(errorMsg) {
        sqlEditor.getSession().setValue('Error: could not validate generated SQL command');
      });
    }

    function pickKSQLStream() {
      $('#newtable-databaseDiv').addClass('arc-invisible');
      $('#newtable-storedAsDiv').addClass('arc-invisible');
      $modal.find('.object-name').text('Stream');
    }

    function pickKSQLTable() {
      $('#newtable-databaseDiv').addClass('arc-invisible');
      $('#newtable-storedAsDiv').addClass('arc-invisible');
      $modal.find('.object-name').text('Table');
    }

    function pickVirtual() {
      $('#newt_sql').removeClass('arc-invisible');
      setupSQLEditor();

      $modal.find('#newtable-storedAsDiv').addClass('arc-invisible');
      $modal.find('#newtable-databaseDiv').addClass('arc-invisible');
      var $newds = $modal.find('input[value="newt_newds"]');
      if (!$newds.is(':checked')) {
        $newds.trigger('click');
      }

      $modal.find('.view_name').addClass('arc-invisible');
      $modal.find('.create-dataset').addClass('arc-invisible');
      $modal.find('#newt_ds_name').prop('required', true);
      $modal.find('#newt_view_name').prop('required', false);
    }

    savetable.setup = function () {
      if (!default_db || !dataset_id || !dataconnection_id || !dataconnection_type) {
        $('.export-table-dataset').hide();
        return savetable;
      }

      $('#table-definition').popover();

      $modal.off('show.bs.modal').on('show.bs.modal', function () {

        modalDefaults();

        if (dataconnection_type === 'ksql') {
          if (queryCreatesKSQLTTable()) {
            $modal.find('div[value="ksql_stream"]').addClass('arc-invisible');
            $modal.find('div[value="ksql_table"]').removeClass('arc-invisible');
            $modal.find('input[name="newTableRadio"][value="ksql_table"]').prop('checked', true);
          } else {
            $modal.find('div[value="ksql_table"]').addClass('arc-invisible');
            $modal.find('div[value="ksql_stream"]').removeClass('arc-invisible');
            $modal.find('input[name="newTableRadio"][value="ksql_stream"]').prop('checked', true);
          }
        }
      });

      $modal.off('shown.bs.modal').on('shown.bs.modal', function () {
        arc.apps_data.load_databases_cb(dataconnection_id, function (error, data) {
          if (error) {
            arc.report.alertMessage('Could not read a list of databases', error);
            return;
          }

          d3.select("#newtable-dbname")
            .selectAll("option")
            .data(data.rows)
            .enter()
            .append("option")
            .text(function (d) {
              return d[0];
            });

          function checkDefaultDb(dbs, default_db) {
            return dbs.some(function(db) {
              return default_db === db[0];
            });
          }

          if (!checkDefaultDb(data.rows, default_db)) {
            default_db = data.rows[0][0];
          }

          $modal.find("#newtable-dbname").val(default_db);
        });
      });

      $('#typeOfView').off('change').on('change', function () {
        if ($(this).find('[name="newTableRadio"][value="logical"]').prop('checked')) {
          pickLogicalView();
        } else if ($(this).find('[name="newTableRadio"][value="virtual"]').prop('checked')) {
          pickVirtual();
        } else if ($(this).find('[name="newTableRadio"][value="ksql_stream"]').prop('checked')) {
          pickKSQLStream();
        } else if ($(this).find('[name="newTableRadio"][value="ksql_table"]').prop('checked')) {
          pickKSQLTable();
        } else {
          pickTable();
        }
      });

      $('input[type=checkbox][value="newt_newds"]').off('change').on('change', function() {
        if ($(this).prop('checked')) {
          $modal.find('#newt_ds').removeClass('arc-invisible');
          $modal.find('#newt_ds_name').prop('required', true);
        } else {
          $modal.find('#newt_ds').addClass('arc-invisible');
          $modal.find('#newt_ds_name').prop('required', false);
        }
      });

      // event handler for save button in modal
      $('#create-newtable-form')
        .off("submit.create-newtable")
        .on("submit.create-newtable", function (e) {

          e.preventDefault();

          // Remove current errors
          $modal.find('#create-newtable-error').addClass('arc-invisible');

          var db_name, name, materialized, table_view, parquet, tableFormat,
            create_dataset, ds_name, query, virtual,
            datarequest, rb, dsreq, parms, post_data, eventType, msg;

          var $currentTarget = $(e.currentTarget);

          name = $currentTarget.find('#newt_view_name').val();

          materialized = $currentTarget.find(
              '[name="newTableRadio"][value="materialized"]').prop('checked');

          virtual = $currentTarget.find(
              '[name="newTableRadio"][value="virtual"]').prop('checked');

          if (materialized) {
            table_view = "Table ";
            parquet = $currentTarget.find(
                '[name="tableFormatRadio"][value="parquet"]').prop('checked');
            if (parquet) {
              tableFormat = "PARQUET";
            } else {
              tableFormat = "TEXTFILE";
            }
          } else {
            table_view = "View ";
          }

          db_name = $currentTarget.find('#newtable-dbname').val();

          create_dataset = $modal.find('input[type=checkbox][value="newt_newds"]').prop('checked');

          if (create_dataset) {
            ds_name = $currentTarget.find('#newt_ds_name').val();
          }

          $spinner.show();

          var donefunc = function() {
            $spinner.hide();
            $modal.modal("hide");
          };

          var errorfunc = function (msg) {
            $spinner.hide();
            $modal.find("#create-newtable-error-detail").text(msg);
            $modal.find("#create-newtable-error").removeClass('arc-invisible');
            arc.report.alertMessage(msg, {
              alertContainerSelector: '#reload-tables-modal',
            });
          };

          vizDriver = vizDriver || arc.report.getPrimaryViz();
          if (virtual) {
            let createFromSql =
                arc.datasets.createFromSql()
                   .dataconnection_id(dataconnection_id)
                   .dataconnection_type(dataconnection_type)
                   .tgt_dataset_name(ds_name)
                   .sql(sqlEditor.getSession().getValue())
                   .showSuccessMsg(true)
                   .successCb(donefunc)
                   .errorCb(errorfunc);

            createFromSql.create();
            return savetable;
          }

          parms = {
            'dataconnection_id': dataconnection_id,
            'cache_read': false,
          };

          if (vizDriver.viz_type() === 'rawtable') {
            query = vizDriver.viz().sqlText();
            query = query.replace(/\s+limit\s+\d+\s*$/i, '');
            rb = arc.requestBuilder();
            rb.query(query);
          } else {
            datarequest = vizDriver.shelvesToDataRequest();
            rb = datarequest.rb().cloneRequestBuilder();
            rb.query(undefined);
            rb.limit(undefined);
          }

          const materializedType = $('input[name=newTableRadio]:checked').val();
          rb.requestType('save_table');
          if (dataconnection_type === 'ksql') {
            rb.tableName(name.toUpperCase());
          } else {
            rb.tableName(db_name + '.' + name);
          }
          rb.tableFormat(tableFormat);
          rb.materializedType(materializedType);
          rb.materialized(materialized);
          rb.datasetId(dataset_id);

          if (!parms.query) {
            parms.dsreq = JSON.stringify(rb.dsreq());
          }

          parms.viz_params = JSON.stringify(vizDriver.params().getkvs());

          arc.sqlrun.runSQLWithParams(
            parms,
            function successCb(resp) {
              if (!ds_name) {
                arc.report.alertMessageSmall(table_view + name + " created", "alert-success");
                $spinner.hide();
                $modal.modal("hide");
              } else {

                var dataset = arc.datasets.Dataset()
                                 .dataconnection_id(dataconnection_id)
                                 .dataconnection_type(dataconnection_type)
                                 .dataset_name(ds_name)
                                 .dataset_type("singletable")
                                 .dataset_info("")
                                 .dataset_detail(db_name + "." + name);

                // Indicate table created while we're building the dataset
                arc.report.alertMessageSmall(table_view + name + " created", "alert-success");

                var savefunc = function() {
                  dataset.save(function() {
                    if (!dataset.dataset_id()) {
                      return;
                    }

                    const dsid = dataset.dataset_id();
                    const dsname = ds_name.trim();

                    const dslink = arc.misc.getBlankTargetUrl(`/arc/apps/dataset/${dsid}`);
                    const dblink = arc.misc.getBlankTargetUrl(`${arc.frontend.globals.urls.reportBuildUrl}/0?report_type=dashboard&dataset_id=${dsid}`);

                    msg = `${table_view} ${name} and ` +
                        `Dataset <a target="_blank" href="${dslink}">${dsname}</a> created\n` +
                        `<div><a target="_blank" href="${dblink}">Create new dashboard</a></div>`;

                    arc.report.alertMessageSmallBase({
                      type: "alert-success",
                      alertHeader: "Success",
                      alertMessage: msg,
                      fadeOut: false,
                      useHTML: true
                    });

                    });

                  donefunc();
                };

                dataset.fetch_and_add_table(dataset.dataset_detail(), savefunc, donefunc);
              }
            },
            function errorCb(errorMsg) {
              errorfunc(errorMsg);
            });
        });

      return savetable;
    };

    return savetable;
  };
}());
