window.arc = window.arc || {};

(function() {
'use strict';

arc.shelf = arc.shelf || {};

arc.shelf.color = function() {
  const c = function() { return 'arc.shelf.color'; };

  let $container,
      $toggler,
      shelfElement,
      colorChangeCb,
      bgColor,
      colValues,
      viz_lib;

  let colorPalettes;
  let colorOpacitySlider;
  let measureRanges;
  let dimToColorSetting;
  let fixedColors;

  function isAggregate() {
    if (shelfElement.expressionIsAggregate({ substCb: viz_lib.substituteParams }) ||
        shelfElement.expr_hasagg()) {
      return true;
    }

    return false;
  }

  c.colValues = function(_) {
    if (!arguments.length) return colValues;
    colValues = _;
    return c;
  };

  c.$container = function(_) {
    if (!arguments.length) return $container;
    $container = _;
    return c;
  };

  c.$toggler = function(_) {
    if (!arguments.length) return $toggler;
    $toggler = _;
    return c;
  };

  c.shelfElement = function(_) {
    if (!arguments.length) return shelfElement;
    shelfElement = _;
    return c;
  };

  c.colorChangeCb = function(_) {
    if (!arguments.length) return colorChangeCb;
    colorChangeCb = _;
    return c;
  };

  c.viz_lib = function(_) {
    if (!arguments.length) return viz_lib;
    viz_lib = _;
    return c;
  };

  c.draw = function() {
    /*
        Default bgColor
        config = {
          colorSettings: colors.colorSettings || {},
          colors: colors.colors || arc.report.graded_colors(),
          isReversed: colors.isReversed || false,
          opacity: colors.opacity || 80,
          dimensionMappings: colors.dimensionMappings,
          fixedDimensionColors: colors.fixedDimensionColors,
        }
    */

    bgColor = shelfElement.backgroundColor();
    setColorBar();
    setupPopover();
    bindListener();
    $toggler.trigger('colorUpdate');
    return c;
  };

  function setColorBar() {
    const tempColor = bgColor.colors.slice();
    $container.find('.colorpalette').remove();
    $container.prepend('<div class="colorpalette"></div>');
    const $b = $container.find('.colorpalette');

    const maxWidth = 150;
    let width = 10;
    if (tempColor.length * width > maxWidth) {
      width = maxWidth / tempColor.length;
    }

    const colorTemplate = '<div class="color" style="width:%WIDTH%px;' +
        'background-color:%COLOR%;opacity:%OPACITY%"></div>';
    if (bgColor.colors.isReversed) {
      tempColor.reverse();
    }

    tempColor.forEach(function(color) {
      $b.append(arc.misc.safe_substitute(
          colorTemplate, {
            COLOR: color,
            OPACITY: bgColor.opacity / 100,
            WIDTH: width,
          }));
    });
  }

  function setupPopover() {

    const colorPalettesId = `color-palettes-${arc.misc.uniqueId()}`,
          opacitySliderId = `opacity-slider-${arc.misc.uniqueId()}`,
          measureRangesId = `measure-ranges-${arc.misc.uniqueId()}`,
          fixDimToColorId = `fix-dimtocolor-${arc.misc.uniqueId()}`;

    $container.popover({
      container: 'body',
      viewport: 'body',
      boundary: 'viewport',
      trigger: 'click',
      placement: 'left',
      template:
        `<div class="popover column-color-popover properties-panel-element" role="tooltip">
           <div class="arrow"></div>
           <div class="popover-body"></div>
        </div>`,
      content: () => {
        colorPalettes = arc.colorpalette.setting().viz_lib(viz_lib);
        colorOpacitySlider = arc.report.user_settings.color_opacity().viz_lib(viz_lib);
        measureRanges = arc.colormap.measureRangesSetting().viz_lib(viz_lib);
        dimToColorSetting = arc
                                .colormap
                                .dimToColorMappingSetting(() => colorPalettes.activeColors())
                                .viz_lib(viz_lib);
        fixedColors = arc.report.user_settings.fixed_color_legend().viz_lib(viz_lib);
        _initPopover();
        return `
          <div class="color-popover">
            <div id="${colorPalettesId}">
              Select a color palette below
            </div>
            <div id="${opacitySliderId}">
            </div>
            <div id="${measureRangesId}">
            </div>
            <div id="${fixDimToColorId}">

          </div>
        `;
      },

      html: true,
    })
    .on('inserted.bs.popover', () => {
      colorPalettes.paint_fn()(d3.select(`#${colorPalettesId}`));
      colorPalettes.changeCb('shelf-color', function() {
        const colors = JSON.parse(colorPalettes.fetch_fn()());
        const o = colorOpacitySlider.fetch_value();
        bgColor.opacity = Number(o);
        measureRanges.setColors(colors.colors);
        bgColor.colors = colors.colors;
        bgColor.firstColor = colors.firstColor;
        bgColor.isReversed = colors.isReversed;
        bgColor.colorSettings = colors.colorSettings || {};
        bgColor.firstColor = colors.firstColor;
        bgColor.colorSettings.measureRanges = JSON.parse(measureRanges.value()).ranges;
        setColorBar();
        _setupDimColorRange();
        if (colorChangeCb) {
          colorChangeCb(bgColor);
        }
      });

      colorOpacitySlider.paintControl(d3.select(`#${opacitySliderId}`));

      if (shelfElement.dataset_isdim()) {
        fixedColors.value((bgColor.fixedDimensionColors === true) ? 'true' : 'false');
        fixedColors.paintControl(d3.select(`#${fixDimToColorId}`));
        dimToColorSetting.paintControl(d3.select(`#${fixDimToColorId}`));
        fixedColors.changeCb('fixed-color', function() {
          bgColor.fixedDimensionColors = JSON.parse(fixedColors.fetch_value());
          if (colorChangeCb) {
            colorChangeCb(bgColor);
          }
        });

        dimToColorSetting.changeCb('column-color', function() {
          const v = JSON.parse(dimToColorSetting.fetch_value());
          if (v.isChecked) {
            bgColor.dimensionMappings = v.mappings;
          } else {
            bgColor.dimensionMappings = {};
          }

          $(`#${fixDimToColorId}`).attr('data-mappings', JSON.stringify({
            isChecked: (Object.keys(bgColor.dimensionMappings).length > 0),
            mappings: bgColor.dimensionMappings,
          }));
          if (colorChangeCb) {
            colorChangeCb(bgColor);
          }
        });
      } else {
        measureRanges.setColors(bgColor.colors);
        measureRanges.setOpacity(Number(colorOpacitySlider.fetch_value()));
        measureRanges.paintControl(d3.select(`#${measureRangesId}`));
        measureRanges.changeCb('shelf-color', function() {
          bgColor.colorSettings.measureRanges = JSON.parse(measureRanges.value()).ranges;
          if (colorChangeCb) {
            colorChangeCb(bgColor);
          }
        });
      }

      //Handle visibility of popover
      //Use mousedown to make sure it does not fire unnecessary click event listener
      $(document.body).on('mousedown.colorPopover', (e) => {
        const $modal = $('#dim-values-modal');

        if (!arc.misc.modalShown($modal)) {
          const $currElem = $(e.target);
          const isInsideThePopover = $.contains($('.column-color-popover')[0], e.target);
          const isThePopover = $('.column-color-popover')[0] === e.target;
          if (isInsideThePopover || isThePopover) {
            if ($currElem.is('input') ||
               $currElem.is('select') ||
               $currElem.hasClass('single-color')) {
              return;
            }

            e.preventDefault();
            return false;
          }

          if ($.contains($container[0], e.target)) {
            if ($('.column-color-popover').is(':visible')) {
              return false;
            }

            $container.popover('show');
          } else {
            $container.popover('hide');
            e.stopImmediatePropagation();
            return false;
          }
        }
      });
    })
    .on('hide.bs.popover', () => {
      $(document.body).off('mousedown.colorPopover');
    });

    function _initPopover() {
      colorPalettes.value(JSON.stringify(bgColor));
      $('.column-color-popover .colorpalettes-setting').css('opacity', bgColor.opacity / 100);

      if (!isAggregate()) {
        _setupDimColorRange();
      } else {
        _setupMeasureColorRange();
      }

      colorOpacitySlider.valueCb(opacityValueCb);
      colorOpacitySlider.value(bgColor.opacity);
    }

    function _setupDimColorRange() {
      if (isAggregate()) {
        return;
      }

      const dimColors = bgColor.dimensionMappings || {};
      const dimToColorSettingValue = {
        isChecked: Object.keys(dimColors).length > 0,
        mappings: dimColors,
      };

      /* Refactor possibility:
        This colorpalette/reversed/firtcolor/dimension/etc
        settings and their callbacks implementations seems to be reducable
        (the visual's color setting and here, might use dependency array!)
        Would be nice to have a consistent implementation:)!
      */
      dimToColorSetting.value(JSON.stringify(dimToColorSettingValue));
      const tempColor = arc.colorpalette.getOrderedColorPalette(bgColor);

      const dimColorScale = arc.colorpalette.dimScale(colValues, tempColor, {
        // Future note: be careful later when the default VIZ-1143!
        fixedDimensionColors: bgColor.fixedDimensionColors || false,
        dimensionMappings: dimColors,
      });

      colValues.forEach(function(d) {
        dimColorScale.setColor(d, dimColorScale(d));
      });

      dimToColorSetting.colorScale(dimColorScale);
    }

    function opacityValueCb(value) {
      bgColor.opacity = Number(value);
      measureRanges.setOpacity(bgColor.opacity);
      dimToColorSetting.setOpacity(bgColor.opacity);
      colorPalettes.setOpacity(bgColor.opacity);
      setColorBar();
      _setupDimColorRange();
      _setupMeasureColorRange();

      if (colorChangeCb) {
        colorChangeCb(bgColor);
      }
    }

    function _setupMeasureColorRange() {
      measureRanges.setColors(bgColor.colors);
      measureRanges.setOpacity(bgColor.opacity);
      if (bgColor.colorSettings.measureRanges) {
        measureRanges.value(JSON.stringify({
          ranges: bgColor.colorSettings.measureRanges,
        }));
      } else {
        measureRanges.value(JSON.stringify({
          ranges: [],
        }));
      }
    }
  }

  const bindListener = () => {
    $toggler.offOn('colorUpdate', () => {
      if (!shelfElement.displayBackgroundColor()) {
        $container.addClass('arc-invisible');
        return;
      }

      $container.removeClass('arc-invisible');
    });
  };

  return c;
};
}());
