window.arc = window.arc || {};

(function() {
'use strict';

arc.showdata = arc.showdata || {};

/*
 * Note that the detaildata-modal is used in arc.showdata.display and in
 * arc.detaildata.display
 */

arc.showdata.display = function() {
  const display = function() { return 'arc.showdata.display'; };

  let viz;
  let result;

  display.viz = function(_) {
    if (!arguments.length) return viz;
    viz = _;
    return display;
  };

  display.result = function(_) {
    if (!arguments.length) return result;
    result = _;
    return display;
  };

  function cleanupTable() {
    viz.removeDataTables('datadetail-viz');
    $('#datadetail-viz').empty();

    const $dataDetailModal = $('#datadetail-modal');
    $dataDetailModal
        .off('shown.bs.modal');
  }

  display.show = function() {
    displayDetail();
    return display;
  };

  function setupModal() {
    cleanupTable();

    const $dataDetailModal = $('#datadetail-modal');
    const report_id = arc.report.getPrimaryReport().report_id();
    $dataDetailModal.attr('data-visual-id', report_id);
    if (arc.misc.modalShown($dataDetailModal)) {
      console.log('modal already open');
      return;
    }

    $dataDetailModal.find('.filters-section').hide();

    const $spinner = $dataDetailModal.find('#datadetail-spinner');
    $spinner.hide();

    const $alert = $dataDetailModal.find('#datadetail-alert');
    $alert.addClass('arc-invisible');
    $dataDetailModal.modal('show', {
      keyboard: true,
    });

    $dataDetailModal
        .off('hidden.bs.modal')
        .on('hidden.bs.modal', () => {
          cleanupTable();
          $('#datadetail-viz').removeAttr('data-microquery-version');
          // set the Data tab back to active
          $('#datadetail-modal').find('.data-section ul.nav.nav-tabs li.datadetail-header a').tab('show');
        });

    const $p = $dataDetailModal.find('.popover-table');
    $p.empty();
  }

  function displayDetail() {
    setupModal();
    setupVisual();

    const $dataDetailModal = $('#datadetail-modal');
    $dataDetailModal
        .off('shown.bs.modal')
        .on('shown.bs.modal', drawVisual);
  }

  let table;
  let driver;

  function setupVisual() {
    table = arc.visual.plugins.table();
    driver = arc.report.driver(table)
                .chart_div('datadetail-viz')
                .params(arc.params.tracker());

    let detailDataColMax;

    if (arc.globals.dashboard) {
      detailDataColMax = arc.globals.dashboard.user_settings()
        .find_by_id('detailed_data_max_columns').value();
    }

    const s = driver.user_settings();
    s.find_by_id('table_full_width')
     .value('true');

    s.find_by_id('max_columns')
     .value(detailDataColMax || '50');

    s.find_by_id('table_template')
     .value('table-template-6');

    // Set the download_rows setting from the source visual
    const src = viz.user_settings().find_by_id('download_rows');
    const tgt = s.find_by_id('download_rows');
    if (src && tgt) {
      tgt.value(src.value());
    }
  }

  function drawVisual() {
    const $dataDetailModal = $('#datadetail-modal');

    const minimumHeight = 300;
    const offsetHeight = 400;

    const $viz = $('#datadetail-viz');
    let h = ($(window).height() - $('#datadetail-viz').position().top - offsetHeight);

    h = Math.max(h, minimumHeight);
    $viz.height(`${h}px`);

    // create the visual by setting the data and then drawing it
    table.viz_setDataResult(result);

    driver.sb_colnames(result.columns().map((c) => c.colname()));
    driver.sb_coltypes(result.columns().map((c) => c.coltype()));
    driver.sb_rows(result.rows());

    table.setConfiguration({
      cellsHtml: false,
      showTooltip: false,
    });

    table.viz_drawVisual(`datadetail-viz`);
    table.adjustColumnWidths();

    if (display.viz().isEmptyTable()) {
      arc.misc.toggleColWarningByTotalCount(result, display.viz());
    } else {
      $dataDetailModal.find('.column-count-warning').addClass('arc-invisible');
    }

    $dataDetailModal.find('.detaildata-download')
        .off('click')
        .on('click', () => {
          viz.exportCSV({ forceFullDownload: display.viz().isEmptyTable() });
        });
  }

  return display;
};
}());
