window.arc = window.arc || {};

(function() {
'use strict';

arc.misc = arc.misc || {};

function attrsToArray(attrs) {
  const l = [];
  if (!attrs) {
    return l;
  }

  $.each(attrs, (k, v) => {
    l.push(`${k}="${v}"`);
  });

  return l;
}

function drawRowToTableRow(dataRow) {
  const d = [];

  {
    const l = attrsToArray(dataRow.attrs);

    if (l.length > 0) {
      d.push(`<tr ${l.join(' ')}>`);
    } else {
      d.push('<tr>');
    }
  }

  dataRow.cols.forEach((c) => {
    const l = attrsToArray(c.attrs);

    if (l.length > 0) {
      d.push(`<td ${l.join(' ')}>${c.data}</td>`);
    } else {
      d.push(`<td>${c.data}</td>`);
    }
  });

  d.push('</tr>');

  return d.join('');
}

function tableData() {
  const rows = [];
  const rows_attrs = [];
  let isHeader = false;
  let sort_orders_asc;

  const tabledata = function() { return 'arc.misc.tabledata'; };

  tabledata.isHeader = function(_) {
    if (!arguments.length) return isHeader;
    isHeader = _;
    return tabledata;
  };

  tabledata.sort_orders_asc = function(_) {
    if (!arguments.length) return sort_orders_asc;
    sort_orders_asc = _;
    return tabledata;
  };

  tabledata.rows = function() {
    return rows;
  };

  tabledata.rows_attrs = function() {
    return rows_attrs;
  };

  tabledata.max_cols = function() {
    let m = 0;
    rows.forEach((row) => {
      m = Math.max(row.length, m);
    });
    return m;
  };

  tabledata.remove_identical_adjacent_rows = function() {
    const to_remove = [];
    rows.forEach((row, row_idx) => {
      if (row_idx > 0) {
        row.forEach((col, col_idx) => {
          if (col_idx === row.length - 1) {
            // Don't remove identical adjacent rows for the final column
            return;
          }

          if (col && (rows[row_idx - 1][col_idx])) {
            if (arc.arraysIdentical(col, rows[row_idx - 1][col_idx])) {
              to_remove.push([row_idx, col_idx]);
            }
          }
        });
      }
    });
    to_remove.forEach((d) => {
      rows[d[0]][d[1]] = [''];
    });
    return tabledata;
  };

  let colspanIndexes = [];
  let newColspanIndexes = [];

  tabledata.draw_row = function(dataRow, row_idx, agg_cnt, fixed_max_cols, use_colspan,
                                use_colspan_for_identical, is_Header, css_class, trellis) {
    const colAttrs = {};

    let allMeasureColNames;
    if (is_Header) {
      allMeasureColNames = trellis.data_columns().map((c) => c.colname());
      colAttrs.class = `viz-td-header ${(css_class || '')}`;
    } else if (css_class) {
      colAttrs.class = css_class;
    }

    if (rows[row_idx]) {
      newColspanIndexes = [];
      const row = rows[row_idx];

      const row_attrs = rows_attrs[row_idx];
      let col_idx = 0;
      const next_cols = [];

      if (use_colspan) {
        let prev_col;
        for (col_idx = 0; col_idx < fixed_max_cols; ++col_idx) {
          if (col_idx in row) {
            if (prev_col !== undefined) {
              next_cols[prev_col] = col_idx;
              if (use_colspan_for_identical) {
                if (!(is_Header &&
                  allMeasureColNames.length === 1 && row_idx === rows.length - 1)) {
                  if (
                      !arc.arraysIdentical(row[prev_col], row[col_idx]) ||
                      (is_Header && allMeasureColNames.indexOf(row[col_idx][0]) >= 0) ||
                      (colspanIndexes.indexOf(col_idx) >= 0)
                     ) {
                    // Do not merge previous and current col header in same colspan
                    prev_col = col_idx;
                    newColspanIndexes.push(col_idx);
                  }
                }
              } else {
                prev_col = col_idx;
                newColspanIndexes.push(col_idx);
              }
            } else {
              next_cols[0] = col_idx;
              prev_col = col_idx;
              newColspanIndexes.push(col_idx);
            }
          }
        }

        colspanIndexes = newColspanIndexes;
        if (prev_col !== undefined) {
          next_cols[prev_col] = fixed_max_cols;
        }

        if (next_cols.length === 0) {
          // No data in this row, add in a td with an empty column span
          const _colAttrs = {
            class: colAttrs.class,
            'data-colidx': '0',
            colspan: `${fixed_max_cols}`,
          };

          dataRow.cols.push({
            attrs: _colAttrs,
            data: '',
          });
        }
      }

      let skip_until = 0;

      for (col_idx = 0; col_idx < fixed_max_cols; ++col_idx) {
        if (col_idx < skip_until) {
          // eslint-disable-next-line no-continue
          continue;
        }

        if (col_idx in row) {
          const col = row[col_idx];
          const col_attr = row_attrs[col_idx];
          let data_idx;
          for (data_idx = 0; data_idx < col.length; ++data_idx) {
            const data = col[data_idx];

            skip_until = drawCol(dataRow, colAttrs.class, data, col_attr[data_idx],
                col_idx, use_colspan, next_cols, is_Header, row_idx, trellis);
          }
        } else if (!use_colspan) {
            for (let agg_idx = 0; agg_idx < agg_cnt; ++agg_idx) {
              dataRow.cols.push({
                data: '',
              });
            }
          }
      }
    }
  };

  function drawCol(dataRow, dataClass, data, data_attrs, col_idx, use_colspan,
      next_cols, is_header, row_idx, trellis) {
    const colAttrs = {
      class: dataClass,
    };

    if (data_attrs) {
      const attrl = [];
      $.each(data_attrs, (prop_name, prop_value) => {
        colAttrs[`data-${prop_name}`] = `${prop_value}`;

        attrl.push('data-');
        attrl.push(prop_name);
        attrl.push('="');
        attrl.push(prop_value);
        attrl.push('"');
      });
    }

    let skip_until = col_idx;
    let isMeasureHeader = false;
    let measureColPos = -1;
    if (is_header) {
      trellis.data_columns().filter((col, i) => {
        if (col.colname() === data) {
          measureColPos = i;
          return true;
        }

        return false;
      });
      isMeasureHeader = measureColPos >= 0;
    }

    const config = {
      pos: measureColPos,
      colname: data,
      sort_orders_asc,
      headerType: 'measure',
      trellis,
      col_idx
    };

    if (!use_colspan) {
      if (isMeasureHeader) {
        dataRow.cols.push({
          attrs: colAttrs,
          data: arc.misc.getHeaderSorter(config, data),
        });
      } else {
        dataRow.cols.push({
          attrs: colAttrs,
          data,
        });
      }
    } else {
      if (next_cols[col_idx] === (col_idx + 1)) {
        if (isMeasureHeader) {
          dataRow.cols.push({
            attrs: colAttrs,
            data: arc.misc.getHeaderSorter(config, data),
          });
        } else {
          dataRow.cols.push({
            attrs: colAttrs,
            data,
          });
        }
      } else {
        const colspan = (next_cols[col_idx] - col_idx);

        const _colAttrs = $.extend({}, colAttrs);
        _colAttrs['data-colidx'] = `${col_idx}`;
        _colAttrs.colspan = `${colspan}`;

        if (isMeasureHeader) {
          dataRow.cols.push({
            attrs: _colAttrs,
            data: arc.misc.getHeaderSorter(config, data),
          });
        } else {
          dataRow.cols.push({
            attrs: _colAttrs,
            data,
          });
        }

        skip_until = next_cols[col_idx];
      }
    }
    return skip_until;
  }

  tabledata.data = function(row_idx, col_idx, data_idx, data, attrs) {
    if (!rows[row_idx]) {
      rows[row_idx] = [];
      rows_attrs[row_idx] = [];
    }

    if (!rows[row_idx][col_idx]) {
      rows[row_idx][col_idx] = [];
      rows_attrs[row_idx][col_idx] = [];
    }

    rows[row_idx][col_idx][data_idx] = data;
    rows_attrs[row_idx][col_idx][data_idx] = attrs;
    return tabledata;
  };

  tabledata.data_col_range = function(row_idx, col_start_idx, data_idx, data) {
    data.forEach((d, i) => {
      const col_idx = col_start_idx + i;
      tabledata.data(row_idx, col_idx, data_idx, d);
    });
    return tabledata;
  };

  tabledata.data_row_range = function(row_start_idx, col_idx, data_idx, data) {
    data.forEach((d, i) => {
      const row_idx = row_start_idx + i;
      tabledata.data(row_idx, col_idx, data_idx, d);
    });
    return tabledata;
  };

  return tabledata;
}

// Each table element here is an array of values
arc.misc.trellisTable = function() {
  const table = function() { return 'arc.report.table'; };

  const data = tableData();
      const x_headers = tableData();
      const y_headers = tableData();

  y_headers.isHeader(true);

  table.data = function() {
    return data;
  };

  table.x_headers = function() {
    return x_headers;
  };

  table.y_headers = function() {
    return y_headers;
  };

  table.draw = function(table_css_classes, sort_orders_asc, trellis) {
    const dataRows = table.getData(sort_orders_asc, trellis);

    const l = [];
    l.push(`<table class="${table_css_classes}">`);
    dataRows.forEach((dataRow) => {
      l.push(drawRowToTableRow(dataRow));
    });

    l.push('</table>');
    return l.join('');
  };

  table.getData = function(sort_orders_asc, trellis) {
    x_headers.sort_orders_asc(sort_orders_asc);
    y_headers.sort_orders_asc(sort_orders_asc);

    const dataRows = [];

    // Headers
    const x_headers_max_cols = x_headers.max_cols();
    const y_headers_max_cols = y_headers.max_cols();
    const data_max_cols = data.max_cols();
    const agg_cols = trellis.data_columns();
    const agg_cnt = (agg_cols && agg_cols.length) || 0;
    const isHeader = true;
    if (x_headers_max_cols > 0) {
      x_headers.rows().forEach((x_row, x_i) => {
        const dataRow = {
          attrs: {
            class: 'viz-tr-header',
          },
          cols: [],
        };

        dataRows.push(dataRow);

        if (y_headers_max_cols > 0) {
          if (x_i === x_headers.rows().length - 1) {
            // Last x header row is the first y header row
            y_headers.draw_row(dataRow, 0, 0, y_headers_max_cols,
                undefined, undefined, isHeader, undefined, trellis);
          } else {
            dataRow.cols.push({
              attrs: {
                class: 'viz-td-header',
                colspan: `${y_headers_max_cols}`,
              },
              data: '',
            });
          }
        }

        if (x_i === x_headers.rows().length - 1) {
          x_headers.draw_row(dataRow, x_i, agg_cnt, x_headers_max_cols,
              true, true, isHeader, 'viz-crosstab-measure', trellis);
        } else {
          x_headers.draw_row(dataRow, x_i, agg_cnt, x_headers_max_cols,
              true, true, isHeader, undefined, trellis);
        }
      });
    } else if (y_headers_max_cols > 0) {
      const dataRow = {
        attrs: {
          class: 'viz-tr-header',
        },
        cols: [],
      };

      dataRows.push(dataRow);

      y_headers.draw_row(dataRow, 0, 0, y_headers_max_cols, false,
          true, isHeader, undefined, trellis);
      dataRow.cols.push({
        attrs: {
          class: 'viz-tr-header',
        },
        data: '',
      });
    }

    // Data
    data.rows().forEach((d_row, d_i) => {
      const dataRow = {
        attrs: {
          'data-row': d_i,
        },
        cols: [],
      };

      dataRows.push(dataRow);

      if (y_headers_max_cols > 0) {
        const compareToPrevRow = true;
        y_headers.draw_row(dataRow, d_i + 1, agg_cnt, y_headers_max_cols,
            false, false, isHeader, undefined, trellis, compareToPrevRow);
      }

      data.draw_row(dataRow, d_i, agg_cnt, data_max_cols, undefined, undefined, undefined,
          undefined, trellis);
    });

    return dataRows;
  };

  return table;
};

}());
