// Arcadia Widgets

/*
Multiple selection box
TO USE
  1) Include in form:
     {% include "partial_multiselect.html"
     with title="TITLE OF FIELD" type="MODAL NAME(default is 'user')"
     instructions="(default is 'Select options from the left')"%}
  2) Call $(selector).selectionField({
        selectionList: [{id: id, name: name}, etc],
        modalName: default is "user" ("MODAL NAME" from 1),
        searchProperty: OPTIONAL (only if not displaying "name"),
        liTemplate: OPTIONAL,
        easyAdd: OPTIONAL {
          addCb: Called upon click of button: function() {
              this.element.find("button").trigger("click");
            },
          validator: OPTIONAL, by default checks for empty or duplicates based on search property
          placeholder: OPTIONAL -- appears in input box,
          buttonLabel: OPTIONAL - what appears to left of input box,
      }
      })
  3) Fill in previous selectionList with:
      $(selector).selectionField('setPreviousSelections', arrayOfSelectedObjects or arrayOfIds)
      This will automatically reset the form first
  4) Reset form by calling:
      $(selector).selectionField('refresh')
  5) Get back a list of selected values (ids) with:
      $(selector).selectionField('selected', OPTIONAL formDiv)
  6) Optional: Remove 1 extra item with:
      $(selector).selectionField(removeSelection, item)
      This will automatically redraw form option
  7) Optional: Add extra item(s) with:
      $(selector).selectionField(addSelection, item)
      This will automatically redraw form option
  8) Optional: Add extra item(s) to unselected list:
      $(selector).addSelectionToUnselected(addSelection, item)
      This will append item to the "unselected" side of the widget
  9) Optional: Change selectionList with:
      $(selector).selectionField(selectionList, arrayOfSelections)
      This will automatically redraw form option
*/

const labelDOMSort = (a, b) => {
  const aValue = a.getElementsByTagName('label')[0].textContent.trim().toLowerCase();
  const bValue = b.getElementsByTagName('label')[0].textContent.trim().toLowerCase();
  if (aValue > bValue) {
    return 1;
  }

  if (bValue > aValue) {
    return -1;
  }

  return 0;
};

$.widget('arc.selectionField', {
    // Default options
    options: {
      selectionList: [],
      modalName: 'user',
      searchProperty: 'name',
      easyAdd: undefined,
      liTemplate: `<li class="form-check">
                    <input type="checkbox" name='groups' class="form-check-input" value='%ID%' id='%UNIQUE_ID%'>
                    <label class="form-check-label" for="%UNIQUE_ID%"> %NAME% </label>
                  </li>`
    },

    labelDOMSort,

    // Public methods
    // need to pass in the form because cached jquery object
    // doesn't have updated selected items
    selected(formDiv) {
      let $form;
      if (!formDiv) {
        $form = $(_getSelector(this.element));
      } else {
        $form = $(formDiv);
      }

      const $selectedListItems = $form.find('.selection-list.selected li input');
      const groupIds = [];
      $selectedListItems.each(function() {
        groupIds.push(parseInt(this.value, 10));
      });
      return groupIds;

      // From http://stackoverflow.com/questions/2420970/how-can-i-get-selector-from-jquery-object
      function _getSelector(el) {
        let selector = $(el).parents()
                            .map(function() {
                              const sel = getIdClass(this);
                              return this.tagName + sel;
                            })
                            .get()
                            .reverse()
                            .concat([el.nodeName])
                            .join('>');

        selector += getIdClass(el);

        // Ignore BODY and HTML tag because it caused problems for me
        return selector.match(/^.*BODY.*?>(.*)$/)[1];

         function getIdClass(elem) {
          const id = $(elem).attr('id');
          let sel = '';
          if (id) {
            sel += `#${id}`;
          }

          const classNames = $(elem).attr('class');
          if (classNames) {
            sel += `.${$.trim(classNames).replace(/\s/gi, '.')}`;
          }

          return sel;
        }
      }
    },
    setPreviousSelections(selectedList) {
      this.refresh(_setPreviousSelections.bind(this));

      // Function to be run after the refresh within setPreviousSelections
      function _setPreviousSelections() {
        const selectedListIdx = {};
        selectedList.forEach((g) => {
          if (g.id) {
            selectedListIdx[g.id] = true;
          } else {
            selectedListIdx[g] = true;
          }
        });

        const $selected = this.$notSelectedList.find('li').filter(function() {
          return selectedListIdx[this.__data__.id];
        });
        if ($selected.length) {
          this.$selectedList.append($selected);
          this._hideInstructions();
        }

        if (!this.$notSelectedList.find('li').length) {
          this._hideNotSelectedFeatures();
        }
      }
    },
    selectionList(list) {
      if (!list) return this.options.selectionList;
      this.options.selectionList = list;
      this._redraw();
    },
    addSelection(items) {
      const selectionList = this;

      selectionList._addToArrayOrItem(items, addItem);
      function addItem(itm) {
        selectionList.options.selectionList.push(itm);
        selectionList._redraw();
      }
    },
    addSelectionToUnselected(items) {
      const selectionList = this;

      selectionList._addToArrayOrItem(items, addItem);
      function addItem(itm) {
        selectionList.options.selectionList.push(itm);
        const $row = $(selectionList._addLiFunction(itm))
                    .appendTo(selectionList.$notSelectedList);
        $row.each(function() {
          this.__data__ = itm;
        });
      }
    },
    _addToArrayOrItem(items, cb) {
      if (Array.isArray(items)) {
        items.forEach(cb);
      } else {
        cb(items);
      }
    },
    removeSelection(item) {
      const { searchProperty, selectionList } = this.options;

      const itemIndex = selectionList.findIndex(
        (selection) => selection.id === item.id &&
          selection[searchProperty] === item[searchProperty]
      );

      if (itemIndex !== -1) {
        selectionList.splice(itemIndex, 1);
      }

      this._redraw();
    },
    refresh(functionCb) {
      // clear all searchbox on the page
      $('[arcid=searchbox]').val('');
      if (this.easyAdd) {
        this.easyAdd.easyAdd('empty');
      }

      this._runIfOptionsPresent(_refresh.bind(this));

      // Only run this function if there are items within selectionList
      function _refresh() {
        // show and sort options again -- append every option in order
        const lis = this.element.find('li').show().sort(labelDOMSort);
        this.$notSelectedList.append(lis);

        // clear all checkmarks
        this.element.find(':checked').prop('checked', false);

        // Show "All" and searchbox in not selected box
        // hide "All" and show instructions in selected box
        this._showNotSelectedFeatures();
        this._showInstructions();
        if (functionCb) {
          functionCb();
        }
      }
    },
    _redraw() {
      this._removeLists();
      this._addSelectionList();
    },
    _create() {
      const selectionList = this;

      selectionList._defineChildrenElements();
      selectionList._runIfOptionsPresent(_createWithOptions);

      // Add easyAdd input if included as option
      if (selectionList.options.easyAdd) {
        if (!selectionList.options.easyAdd.validator) {
          selectionList.options.easyAdd.validator = [{
            error: 'Cannot be empty',
            validation(val) {
              if (val.length === 0) return false;
              return true;
            }
          },
          {
            error: 'Cannot be duplicate',
            validation(val) {
              return !selectionList.options.selectionList
                .some((l) => val === l[selectionList.options.searchProperty]);
            }
          }];
        }

        if (!selectionList.options.easyAdd.width) {
          selectionList.options.easyAdd.width = 180;
        }

        selectionList.easyAdd = selectionList.element.find('.arc-easy-add')
            .easyAdd(selectionList.options.easyAdd);
      }

      // Set up event listener on add and remove buttons
      selectionList.element.find(`.${selectionList.options.modalName}-selection-btn-wrapper`)
          .on('click.add_groups', null, { widget: this }, function handleAdd(e) {
            const button = e.target;
            if (button.classList.contains('btn')) {
              let $currentChecked = selectionList.element.find(':visible:checked')
                                                 .parent().filter('li');
              if (button.classList.contains('add-selection')) {
                $currentChecked = $currentChecked.filter(function () {
                  return $(this).parent().hasClass('not-selected');
                });
                const $alreadySelected = selectionList.$selectedList.children();
                const $updatedSelections = $alreadySelected
                  .add($currentChecked)
                  .sort(labelDOMSort);
                selectionList.$selectedList.append($updatedSelections);
              } else {
                $currentChecked = $currentChecked.filter(function () {
                  return $(this).parent().hasClass('selected');
                });
                const $alreadyNotSelected = selectionList.$notSelectedList.children();
                const $updatedNotSelected = $alreadyNotSelected
                  .add($currentChecked)
                  .sort(labelDOMSort);
                selectionList.$notSelectedList.append($updatedNotSelected);
              }

              $currentChecked.find(':checked').prop('checked', false);

             // Hide "all" option if there is nothing in the selected or not-selected list
             const $selectedItems = selectionList.$selectedList.find('li');
             const $notSelectedItems = selectionList.$notSelectedList.find('li');
             if (!$selectedItems.length) {
               selectionList._showInstructions();
             } else {
               selectionList._hideInstructions();
             }

             if (!$notSelectedItems.length) {
               selectionList._hideNotSelectedFeatures();
             } else {
               selectionList._showNotSelectedFeatures();
             }
           }
         });

      // Set up event listener on "All" checkbox
      selectionList.element.find('.check-all-wrapper input')
          .on('click.select_all', (e) => {
            const selectAllStatus = e.target.checked;
            $(e.target).parent().parent().parent()
                       .find("li:visible [type='checkbox']")
                       .prop('checked', selectAllStatus);
          });

      // Set up event listener on search-box
      const getUnselectedLis = function() {
        return d3.selectAll(selectionList.element)
                 .selectAll('.selection-list li');
      };

      const searchLis = function(d, term) {
        const sidx = d[selectionList.options.searchProperty].toLowerCase().search(term);
        if (sidx >= 0) return true;
        return false;
      };

      selectionList.element.find("[arcid='searchbox']").each(function() {
        arc.report.searchboxSetup({
          searchboxId: this,
          search_cb: searchLis,
          sel_cb: getUnselectedLis,
        });
      });

      // Only run this function if there are items within selectionList
      function _createWithOptions() {
        // Add selections to element
        selectionList._addSelectionList();

        // Hide the "all" button for selected box by default
        selectionList.$selectAllNotSelected.hide();
      }
    },

    // Private functions
    _defineChildrenElements() {
      this.$notSelectedList = this.element
                                       .find('.selection-list.not-selected');
      this.$selectedList = this.element
                                       .find('.selection-list.selected');
      this.$selectAllSelected = this.$selectedList.parent()
                                       .find('.check-all-wrapper');
      this.$selectAllNotSelected = this.$notSelectedList.parent()
                                       .find('.check-all-wrapper');
      this.$selectInstructions = this.$selectedList.parent()
                                       .find('.instructions');
    },
    _addSelectionList() {
      this.options.selectionList
          .sort((a, b) => {
             let aValue = a[this.options.searchProperty];

             let bValue = b[this.options.searchProperty];

             if ((typeof (aValue) === typeof ('')) &&
                 (typeof (bValue) === typeof (''))) {
               aValue = aValue.toLowerCase();
               bValue = bValue.toLowerCase();
             }

             if (aValue > bValue) return 1;
             if (bValue > aValue) return -1;
             return 0;
           })
           .forEach((item) => {
              const $row = $(this._addLiFunction(item))
                          .appendTo(this.$notSelectedList);
              $row.each(function() {
                this.__data__ = item;
              });
            });
    },
    _removeLists() {
      this.$selectedList.empty();
      this.$notSelectedList.empty();
    },
    _addLiFunction(selectionOption) {
      selectionOption = {
        ID: selectionOption.id,
        UNIQUE_ID: `input-label-id-${arc.misc.getUniqueId()}`,
        NAME: selectionOption[this.options.searchProperty],
      };
      return arc.misc.safe_substitute(this.options.liTemplate, selectionOption);
    },
    _hideNotSelectedFeatures() {
      this.$selectAllNotSelected.hide();
    },
    _showNotSelectedFeatures() {
      this.$selectAllNotSelected.show();
    },
    _showInstructions() {
      this.$selectAllSelected.hide();
      this.$selectInstructions.show();
    },
    _hideInstructions() {
      this.$selectAllSelected.show();
      this.$selectInstructions.hide();
    },
    _runIfOptionsPresent(functionToRun) {
      if (this.options.selectionList.length) {
        this.element.show();
        functionToRun();
      } else {
        this.element.hide();
      }
    }
});

/*
1) Include in form:
   {% include "partial_easyadd.html" %}
2) Call $(selector).easyAdd({
    width: integer width of input field,
    placeholder: OPTIONAL -- appears in input box,
    buttonLabel: OPTIONAL - what appears to left of input box,
    validator: OPTIONAL - an array of objects that validate successfully when returns true:
      [{
        error: "Doesn't validate",
        validation: function(val) {
          if (val > 0) {
            return true;
          } else {
            return false;
          }
        }
      }],
    addCb: Called upon click of button: function() {
      this.element.find("button").trigger("click");
  });
3) Clear searchbox
    $(selector).easyAdd('empty')
4) Get value
    $(selector).easyAdd('value')
5) Trigger create:
    $(selector).easyAdd('add', OPTIONAL value)
*/
$.widget('arc.easyAdd', {
    options: {
      placeholder: 'Add new item',
      buttonLabel: 'Add',
      validator: [],
      addCb: undefined,
      width: undefined,
    },

    // Public methods
    // need to pass in the form because cached version doesn't have updated selected items
    empty() {
      this.$input.val('');
      this._emptyErrors();
    },
    value() {
      return this.$input.val();
    },
    add(e, easyAdd) {
      easyAdd = easyAdd || this;
      const val = easyAdd.value();
      if (easyAdd.validateValue()) {
        if (easyAdd.options.addCb) {
          easyAdd.options.addCb(val, e, easyAdd);
        }

        easyAdd.empty();
      }
    },

    // clearErrorsOnly is true when passed in from keyup action
    // Only want to clear errors, not introduce new ones
    validateValue(clearErrorsOnly) {
      const easyAdd = this;
      const val = easyAdd.value();

      if (Array.isArray(easyAdd.options.validator)) {
        const errors = [];
        easyAdd.options.validator.forEach((v) => {
          if (!v.validation(val)) {
            errors.push(v.error);
          }
        });
        if (errors.length) {
          if (!clearErrorsOnly) {
            easyAdd.element.addClass('has-error');
            easyAdd.$errors.removeClass('arc-invisible');
            const ul = easyAdd.$errors.find('.list-unstyled');
            errors.forEach((error) => {
              ul.append(`<li>${error}</li>`);
            });
          }

          return false;
        }

        easyAdd._emptyErrors();
        return true;
      }
    },
    _emptyErrors() {
      this.element.removeClass('has-error');
      this.$errors.find('.list-unstyled').empty();
      this.$errors.addClass('arc-invisible');
    },
    _destroy() {
      this.$input.attr('placeholder', '');
      this.$button.attr('data-original-title', '');
      this.$button.empty();
      this.$button.off('click.easyAdd');
      this.$button.off('keyup.easyAdd');
      this.element.addClass('arc-invisible');
      this.$errors.find('.list-unstyled').empty();
      this.$errors.addClass('arc-invisible');
    },
    _create() {
      const _this = this;

      // Define important children elements
      _this._defineChildrenElements();

      // Set up desired title, helpertext, title
      if (_this.options.placeholder) {
        _this.$input.attr('placeholder', _this.options.placeholder);
      }

      if (_this.options.buttonLabel) {
        _this.$button.html(_this._plusIcon + _this.options.buttonLabel);
      }

      if (_this.options.width) {
        _this.$input.css('width', _this.options.width);
      }

      _this.element.removeClass('arc-invisible');

      // Set up event listener on "Add" button
      _this.$button.on('click.easyAdd', (e) => {
        _this.add(e, _this);
      });

      // Clear errors upon input change and prevent modal from closing
      _this.$input.on('keyup.easyAdd', (e) => {
        if (e.which === arc.misc.ENTER_KEY) {
          _this.add(e, _this);
        } else {
          _this.validateValue.apply(_this, [e]);
        }
      });
    },

    // Private method
    _defineChildrenElements() {
      this.$button = this.element.find('button');
      this.$input = this.element.find('input');
      this.$errors = this.element.find('.with-errors');
    },
    _plusIcon: '<i class="fa fa-plus"></i> '
});

/*
CHECKBOX Setup
1) Call widget on a form/table/group of checkboxes:
    $(selector).checkBoxes({
      objectType: default is "user",
      paginatedDataTables: OPTIONAL (default is false),
      allId: OPTIONAL (default is "all-[objectType]s-checkbox"),
      hasAllCheckbox: OPTIONAL (default is true, indicates that there is an all checkbox),
      $actionButtons: OPTIONAL (buttons that will become clickable upon selection)
    })
2) Get back value of selected elements by calling:
    $(selector).checkBoxes('selected') //all checkboxes on page
    $(selector).checkBoxes('selectedVisible')  //only visible checkboxes on page
    $(selector).checkBoxes('selectedVisibleEl') // visible jquery checkboxes
3) Clear checkboxes by calling:
    $(selector).checkBoxes('clear')
3) Update this.element by calling:
    $(selector).checkBoxes('reset', $el)
*/
$.widget('arc.checkBoxes', {
    // Default options
    options: {
      objectType: 'user',
      allId: undefined,
      hasAllCheckbox: true,
      paginatedDataTables: false,
      $actionButtons: undefined,
    },

    // Public methods
    selected() {
      const selectedValues = [];
      this.checkboxes.each(function() {
        if (this.checked) {
          selectedValues.push(this.value);
        }
      });
      return selectedValues;
    },
    selectedVisible() {
      const selectedValues = [];
      this.checkboxes.filter(':visible').each(function() {
        if (this.checked) {
          selectedValues.push(this.value);
        }
      });
      return selectedValues;
    },
    selectedVisibleEl() {
      return this.checkboxes.filter(':visible:checked');
    },
    clear() {
      if (this.options.hasAllCheckbox) {
        this.allCheckbox.prop('checked', false);
      }

      if (this.options.$actionButtons) {
        this.options.$actionButtons.prop('disabled', true);
      }

      this.checkboxes.prop('checked', false);
    },
    reset($el) {
      this.element = $el || this.element;

      // Define important children elements
      this._defineChildrenElements();
      if (this.options.$actionButtons) {
        this.options.$actionButtons.prop('disabled', true);
      }

      this._create();
    },
    _destroy() {
      if (this.options.hasAllCheckbox) {
        this.allCheckbox.off('click.select_all');
      }

      this.checkboxes.off('change.select_checkbox');
    },
    _create() {
      // Define important children elements
      this._defineChildrenElements();

      // Set up event listener on "All" checkbox
      const _this = this;
      if (this.options.hasAllCheckbox) {
        this.allCheckbox.off('click.select_all')
                        .on('click.select_all', (e) => {
          e.stopPropagation();
          if (_this.options.paginatedDataTables) {
            _this._defineChildrenElements();
          }

          const selectAllStatus = e.target.checked;
          _this.checkboxes.filter(':visible').prop('checked', selectAllStatus);
          if (_this.options.$actionButtons) {
            _this.options.$actionButtons.prop('disabled', !selectAllStatus);
          }
        });
      }

      function changeUponChecks() {
        if (_this.options.hasAllCheckbox) {
          const allBoxes = _this.checkboxes.filter(':visible');
          const selectedBoxes = allBoxes.filter(':checked');
          const allSelected = selectedBoxes.length === allBoxes.length;

          _this.allCheckbox.prop('checked', allSelected);
        }

        if (_this.options.$actionButtons) {
          const checkboxSelected = !_this.checkboxes.filter(':checked').length;
          _this.options.$actionButtons.prop('disabled', checkboxSelected);
        }
      }

      if (_this.options.paginatedDataTables) {
        this.element.off('click.check_all')
                    .on('click.check_all', this._checkboxSelector(), (e) => {
          e.stopPropagation();
            _this._defineChildrenElements();
          changeUponChecks();
        });
      } else {
        this.checkboxes.off('click.stopPropagation')
                       .on('click.stopPropagation', (e) => {
          e.stopPropagation();
          changeUponChecks();
        });
      }
    },

    // Private method
    _checkboxSelector() {
      return `input[type=checkbox]:not(${this._allId()})`;
    },
    _defineChildrenElements() {
      if (this.options.hasAllCheckbox) {
        this.allCheckbox = this.element.find(this._allId());
      }

      const checkboxSelector = this._checkboxSelector();
      this.checkboxes = this.element.find(checkboxSelector);
    },
    _allId() {
      if (this.options.allId) {
        return `#${this.options.allId}`;
      }

      return `#all-${this.options.objectType}s-checkbox`;
    }
});
