// Widget for creating hyperlink library
// To use:
//    1) Call function and pass in options:
//        $(selector).clickableList(options);
//    2) Best format (Where this item is put inside #snippet-sidebar):
//        <div class="modal-body edit-expression-body">
//         <div class="edit-expression-textbox"></div>
//         <div id="snippet-sidebar"></div>
//        </div>
//    3) Options:
//        items: Hash of items to list in library (eg: {type1: [typeItems], type2: [typeItems]})
//        skipTypes: Any types to skip (eg: ['duplicate']),
//        clickCb: What to do if item is clicked,
//        itemParser: function this returns an object with text, data, and optionally tooltip
//          eg: {text: what to show as <a>, data: __data__, tooltip: show as tooltip};
//        }
//        showAll: (OPTIONAL, by default "true", also accepts string).
//          If set to "true" or string, will create option to show items of all types at once
//          If it's a string, will append the string to "All" label
//      }
//    4) To select first item of list, Call
//        $(selector).clickableList("selectFirst")
//    4) To change the list of items, first destroy and then re-initialize app
//        $(selector).clickableList("destroy").clickableList(options)
$.widget('arc.clickableList', {
    // Default options
    options: {
      items: {},
      skipTypes: [],
      clickCb: function () {},
      itemParser: undefined,
      showAll: true,
    },

    // HTML for the library
    _containers: "<select class='form-control form-control-sm library-catalog'></select>" +
                 "<ul class='clickable-list-ul'></ul>",
    _destroy: function () {
      this.element.empty();
    },
    _create: function() {
      var listWidget = this,
          lis = [];

      if (Object.keys(listWidget.options.items).length) {
        listWidget.element.addClass('edit-expression-selectlists')
                          .removeClass("arc-invisible")
                          .html(listWidget._containers);
        listWidget._defineChildrenElements();
        var itemTypeHyphenated, list;
        if (listWidget.options.showAll) {
          var allText = listWidget.options.showAll === true ? "All" : "All " +
              listWidget.options.showAll;
          listWidget.select.append("<option value='arc-all-items'>" + allText);
        }

        // Add each category to the select field and
        // each item within this category to a new list
        for (var itemType in listWidget.options.items) {
          if (listWidget.options.skipTypes.indexOf(itemType) === -1) {
            itemTypeHyphenated = "arc" + itemType.toLowerCase().split(" ").join("-") + "-items";
            listWidget.select.append("<option value='" + itemTypeHyphenated + "'>" + itemType);
            listWidget.options.items[itemType].forEach(addLi);
          }
        }

        // Sort all the items because some people can't seem to handle sorting within groups :/
        lis.sort(function (a, b) {
          var hasArcField = a.find(".arc-field").length > 0,
              cssSelector = hasArcField ? ".arc-field" : "a",
              aValue = a.find(cssSelector).text().toLowerCase(),
              bValue = b.find(cssSelector).text().toLowerCase();
          if (aValue > bValue) return 1;
          else if (bValue > aValue) return -1;
          else return 0;
        }).forEach(function (item) {
          listWidget.list.append(item);
        });

        // Add event listener for inserting clicked snippets
        listWidget.list.on("click.insertSnippet", "li", function (e) {
          listWidget.options.clickCb(this, e);
        });

        // Add event listener for when user changes library category
        listWidget.lis = listWidget.list.find("li");
        listWidget.select.on("change.changeList", function (e) {
          var lisToShow;
          if (e.target.value === 'arc-all-items') {
            lisToShow = listWidget.lis;
          } else {
            listWidget.lis.addClass("arc-invisible");
            lisToShow = listWidget.lis.filter("." + e.target.value);
          }
          lisToShow.removeClass("arc-invisible");
        });

        // Select first option
        listWidget.select.find("option:first").prop("selected", true);
        listWidget.select.change();
      } else {
        listWidget.element.addClass("arc-invisible");
      }

      // HELPER FUNCTION
      // Appends each library item to the list
      function addLi(itm) {
        var parsedItm, text, data, tooltip;
        if (listWidget.options.itemParser) {
          parsedItm = listWidget.options.itemParser(itm);
          text = parsedItm.text;
          data = parsedItm.data;
          tooltip = parsedItm.tooltip;
        } else {
          text = itm; data = itm;
        }
        var $item = $("<li class='" + itemTypeHyphenated + "'><a>" + text + "</a></li>");
        $item[0].__data__ = data;

        // tooltip if there's one returned by the parser
        if (tooltip) {
          const $a = $item.find('span.arc-field');
          $a.attr('data-original-title', tooltip);
          $a.tooltip({delay: {
             show: 500,
             hide: 100
          }});
        }
        lis.push($item);
      }
    },

    // Public method
    selectFirst: function () {
      this.select.find("option:first").prop("selected", true);
      this.select.change();
    },

    // Private method
    _defineChildrenElements: function () {
      this.select = this.element.find('.library-catalog');
      this.list = this.element.find(".clickable-list-ul");
    }
});

