/*
 * Parser for a point in time relative to a TODAY or NOW anchor
 */

start
  = (named_range:(named_range) { return named_range; })+

named_range
  = label:label _ ',' _ start_time:time_expression _ ',' _ end_time:time_expression _ { return [label, start_time, end_time]; }

label
  = label:[^,]+ { return label.join(""); }

time_expression
  = anchor:anchor rules:((__ rule:rule) {return rule; })* _ {
  return anchor.substring(1) + (rules.length ? "." : "") + rules.join(".") + (anchor.substring(0, 1) === 'n' ? ".format('YYYY-MM-DD HH:mm:ss')": ".format('YYYY-MM-DD')");
  }

anchor
  = anchor:(ntoday / today / now / hyphenyyyymmddhhMMss / yyyymmdd / yyyymmddhhMMss ) { return anchor; }

/*
 * Using nmoment and tmoment to mark for today vs now. This "t" and "n" are
 * stripped from the moment expression in the time_expression rule
 */

ntoday
  = 'nTODAY' { return "nmoment().utc()"; }

today
  = 'TODAY' { return "tmoment().utc()"; }

now
  = 'NOW' { return "nmoment().utc()"; }

yyyymmdd
  = y:yyyy hyphen m:mm hyphen d:dd { return "tmoment('" + y + '-' + m + '-' + d + "')"; }

yyyymmddhhMMss
  = y:yyyy colon m:mm colon d:dd space h:hh colon M:MM colon s:ss  { return "nmoment('" + y + '-' + m + '-' + d + " " + h + ":" + M + ":" + s + "')"; }

hyphenyyyymmddhhMMss
  = y:yyyy hyphen m:mm hyphen d:dd space h:hh colon M:MM colon s:ss { return "nmoment('" + y + '-' + m + '-' + d + " " + h + ":" + M + ":" + s + "')"; }

yyyy
  = digits:([0-2] [0-9] [0-9] [0-9]) { return digits.join(""); }

mm
  = digits:([0-1] [0-9]) { return digits.join(""); }

dd
  = digits:([0-3] [0-9]) { return digits.join(""); }

hh
  = digits:([0-2] [0-9]) { return digits.join(""); }

MM
  = digits:([0-5] [0-9]) { return digits.join(""); }

ss
  = digits:([0-5] [0-9]) { return digits.join(""); }

rule
  = rule:(addition / subtraction / startof / endof) { return rule; }

addition
  = 'ADD' __ i:integer __ timeunit:timeunit { return "add(" + i + ", '" + timeunit + "')"; }

subtraction
  = 'SUBTRACT' __ i:integer __ timeunit:timeunit { return "subtract(" + i + ", '" + timeunit + "')"; }

startof
  = 'STARTOF' __ timeunit:timeunit { return "startOf('" + timeunit + "')"; }

endof
  = 'ENDOF' __ timeunit:timeunit {return "endOf('" + timeunit + "')"; }

timeunit
  = timeunit:(second / minute / hour / day / week / month / quarter / year) { return timeunit; }

second
  = ('SECONDS' / 'SECOND') { return 'second'; }

minute
  = ('MINUTES' / 'MINUTE') { return 'minute'; }

hour
  = ('HOURS' / 'HOUR') { return 'hour'; }

day
  = ('DAYS' / 'DAY') { return 'days'; }

week
  = ('WEEKS' / 'WEEK') { return 'week'; }

month
  = ('MONTHS' / 'MONTH') { return 'month'; }

quarter
  = ('QUARTERS' / 'QUARTER') { return 'quarter'; }

year
  = ('YEARS' / 'YEAR') { return 'year'; }

integer "integer"
  = digits:[0-9]+ { return parseInt(digits.join(""), 10); }

hyphen
  = '-' { return '-'; }

space
  = ' ' { return ' '; }

colon
  = ':' { return ':'; }

_
  = [ \t\r\n]* { return undefined; }

__
  = [ \t\r\n]+ { return undefined; }
