/*
 * Text elements in a visual can be parametrized. Parameters are specified in
 * the language below.
 *
 * Possible parameter representations that are being parsed below:
 *   <<paramname>>
 *   <<paramname:defaultvalue>>
 *   <script> js code </script>
 *
 * The parser output a two-element array. The first element is a hash of defaults,
 * and the second element is an array of parsed element. See usage of the parser
 * in arc_viztemplates.js
 */

{
  var outdata = [];
  var defaults = {};
}

start
  = singlesection*
  {
    return [
      defaults,
      outdata
    ];
  }

singlesection
  = str_without_lessthan / possiblevarsection

possiblevarsection
  = varsection / codesection / str_starting_with_lessthan

str_starting_with_lessthan
  = "<" s:str_without_lessthan_raw
  {
    outdata.push({
      "element": "str",
      "str": "<" + s
    });
  }

str_without_lessthan
  = s:str_without_lessthan_raw
  {
    outdata.push({
      "element": "str",
      "str": s
    });
  }

str_without_lessthan_raw
  = s:[^<]+
  {
    return s.join("");
  }

codesection
  = "<script>" code:coderaw "</script>"
  {
    outdata.push({
      "element": "code",
      "code": code
    });
  }

coderaw
  = s:codesnippet+
  {
    return s.join("");
  }

codesnippet
  = s:(codesnippet_without_lessthan / codesnippet_with_lessthan)
  {
    return s;
  }

codesnippet_without_lessthan
  = s:[^<]+
  {
    return s.join("");
  }

codesnippet_with_lessthan
  = s1:[^<]* s2:"<" s3:[^\/]
  {
    return s1.join("") + s2 + s3;
  }

varsection
  = "<<" varname:varname vardefaultwrapper:vardefaultwrapper? ">>"
  {
    var lcase = arguments[0];
    if (arguments[1] !== null) {
      defaults[lcase] = arguments[1];
    }
    outdata.push({
      "element": "variable",
      "varname": lcase
    });
  }

varname
  = v:[^:>]+
  {
    return v.join("");
  }

vardefaultwrapper
  = ":" vardefault:vardefault
  {
    return arguments[0]
  }

vardefault
  = v:[^>]*
  {
    return v.join("");
  }

