QUnit.module('admin_arc_admin_dataset', {});

QUnit.test(
  'column data retieval works without server info info stored',
  (assert) => {
    const dataset = arc.testutil.ds.create_dataset();
    const col1 = dataset.tables()[0].columns()[0];
    const col2 = dataset.tables()[0].columns()[1];
    const col3 = dataset.tables()[1].columns()[0];
    const col4 = dataset.tables()[1].columns()[1];

    let qp;
    arc.data.runQueryParams = function (_) {
      qp = _;
    };

    arc.admin.columnColorHandler(col1).draw();
    assert.equal(qp.rb.dimensions()[0].expr, "[Name A] as 'Name A'");

    arc.admin.columnColorHandler(col2).draw();
    assert.equal(qp.rb.dimensions()[0].expr, '[Name A] || "suffixed"');

    arc.admin.columnColorHandler(col3).draw();
    assert.equal(qp.rb.dimensions()[0].expr, "[Name B] as 'Name B'");

    arc.admin.columnColorHandler(col4).draw();
    assert.equal(qp.rb.dimensions()[0].expr, '[Name B] || "suffixed"');

    // after changing the column alias, the new alias is used

    col3.attr('alias', 'Name B2');
    arc.admin.columnColorHandler(col3).draw();
    assert.equal(qp.rb.dimensions()[0].expr, "[Name B2] as 'Name B2'");
  },
);

QUnit.test('column data retieval uses server info when available', (assert) => {
  const builder = arc.reportbuilder();

  const dataset = arc.testutil.ds.create_dataset();
  const col1 = dataset.tables()[0].columns()[0];
  const col2 = dataset.tables()[0].columns()[1];
  const col3 = dataset.tables()[1].columns()[0];
  const col4 = dataset.tables()[1].columns()[1];

  builder.get_dataset_server_info = function () {
    return {
      column_aliases_by_fqn: {
        'testdb.a.name': 'Old Name A',
        'testdb.b.name': 'Old Name B',
      },
    };
  };

  let qp;
  arc.data.runQueryParams = function (_) {
    qp = _;
  };

  // for simple columns old names are used:
  arc.admin.columnColorHandler(col1, builder).draw();
  assert.equal(qp.rb.dimensions()[0].expr, "[Old Name A] as 'Old Name A'");

  arc.admin.columnColorHandler(col2, builder).draw();
  assert.equal(qp.rb.dimensions()[0].expr, '[Name A] || "suffixed"');

  arc.admin.columnColorHandler(col3, builder).draw();
  assert.equal(qp.rb.dimensions()[0].expr, "[Old Name B] as 'Old Name B'");

  arc.admin.columnColorHandler(col4, builder).draw();
  assert.equal(qp.rb.dimensions()[0].expr, '[Name B] || "suffixed"');

  // after changing the column alias, does not affect the query:
  col3.attr('alias', 'Name B2');
  arc.admin.columnColorHandler(col3, builder).draw();
  assert.equal(qp.rb.dimensions()[0].expr, "[Old Name B] as 'Old Name B'");
});

QUnit.test(
  'falls back to column alias when server info misses column entry',
  (assert) => {
    const builder = arc.reportbuilder();

    const dataset = arc.testutil.ds.create_dataset();
    const col1 = dataset.tables()[0].columns()[0];

    builder.get_dataset_server_info = function () {
      return {
        column_aliases_by_fqn: {
          // unlikely, but better to test:
          // 'testdb.a.name': 'Old Name A',
        },
      };
    };

    let qp;
    arc.data.runQueryParams = function (_) {
      qp = _;
    };

    // for simple columns old names are used:
    arc.admin.columnColorHandler(col1, builder).draw();
    assert.equal(qp.rb.dimensions()[0].expr, "[Name A] as 'Name A'");
  },
);

QUnit.test(
  'falls back to column alias when server info misses alias map',
  (assert) => {
    const builder = arc.reportbuilder();

    const dataset = arc.testutil.ds.create_dataset();
    const col1 = dataset.tables()[0].columns()[0];

    builder.get_dataset_server_info = function () {
      return {};
    };

    let qp;
    arc.data.runQueryParams = function (_) {
      qp = _;
    };

    // for simple columns old names are used:
    arc.admin.columnColorHandler(col1, builder).draw();
    assert.equal(qp.rb.dimensions()[0].expr, "[Name A] as 'Name A'");
  },
);
