QUnit.module('admin_arc_admin_users', {
  beforeEach() {
    arc.frontend.globals.urls.rbacRolesUrl = 'ROLES_URL';
    arc.frontend.globals.urls.usersDetailedApiUrl = 'USERS_URL';
    arc.frontend.globals.urls.groupsUrl = 'GROUPS_URL';
    arc.frontend.globals.urls.usersBasicApiUrl = 'USERS_BASIC_API_URL';
    arc.frontend.globals.urls.groupsBasicUrl = 'GROUPS_BASIC_API_URL';
  },
});

function controllablePromise() {
  const result = {};
  const p = new Promise((resolve, reject) => {
    result.resolve = resolve;
    result.reject = reject;
  });
  result.promise = p;
  return result;
}

QUnit.test('New user button disabled while retrieving data', async (assert) => {
  const userRequest = controllablePromise();
  const groupsRequest = controllablePromise();
  const rolesRequest = controllablePromise();
  const usersBasicRequest = controllablePromise();
  const groupsBasicRequest = controllablePromise();

  const promiseMap = {
    [arc.frontend.globals.urls.rbacRolesUrl]: rolesRequest.promise,
    [arc.frontend.globals.urls.usersDetailedApiUrl]: userRequest.promise,
    [arc.frontend.globals.urls.groupsUrl]: groupsRequest.promise,
    [arc.frontend.globals.urls.usersBasicApiUrl]: usersBasicRequest.promise,
    [arc.frontend.globals.urls.groupsBasicUrl]: groupsBasicRequest.promise,
  };

  arc.frontend.vizHttp.get = (args) => promiseMap[args.url];

  // disabled already on page load
  assert.true($('[href="#new-user-modal"]').hasClass('disabled'));

  arc.useradmin.init();

  // disabled because data download has started
  assert.true($('[href="#new-user-modal"]').hasClass('disabled'));

  // simulate roles arrival:
  rolesRequest.resolve({ data: [] });
  await smallWait();

  // still disabled
  assert.true($('[href="#new-user-modal"]').hasClass('disabled'));

  // simulate user data arrival:
  userRequest.resolve({ data: { users: [] } });
  await smallWait();

  // still disabled
  assert.true($('[href="#new-user-modal"]').hasClass('disabled'));

  // simulate groups and roles data arrival:
  groupsRequest.resolve({ data: { groups: [] } });
  await smallWait();

  // button is enabled now:
  assert.false($('[href="#new-user-modal"]').hasClass('disabled'));
});

async function smallWait() {
  // have issues with waiting for all promises to settle
  // this is a workaround until we find a better way
  return new Promise((resolve) => {
    setTimeout(resolve, 100);
  });
}
