arc.globals.reports = arc.reporttracker.reports();
arc.globals.app_settings = { maximumCanvasWidth: 8000 };

QUnit.module('builder_arc_bar');

QUnit.test('bar_builder', (assert) => {
  const column = arc.datasets.Column();

  const table = arc.datasets.Table().tablename('a.b');
  column.table(table);

  const dataset = arc.datasets
    .Dataset()
    .dataset_id(1)
    .tables([table])
    .dataconnection_type('sqlite')
    .add_column(column);
  table.dataset(dataset);

  const bar = arc.visual.plugins.bar();
  const driver = arc.report
    .driver(bar)
    .dataset(dataset)
    .params(arc.params.tracker());

  bar.viz_drawDesigner('app-shelves');
  const tr_sel = d3.selectAll('#app-shelves > div').each(function (d, tr_i) {
    if (tr_i === 0) {
      assert.equal($(this).find('.shelf-label').text(), '* X Axis');
    }

    if (tr_i === 1) {
      assert.equal($(this).find('.shelf-label').text(), '* Y Axis');
    }

    if (tr_i === 2) {
      assert.equal($(this).find('.shelf-label').text(), 'Colors');
    }

    if (tr_i === 3) {
      assert.equal($(this).find('.shelf-label').text(), 'Tooltips');
    }
  });

  assert.equal(tr_sel.size(), 7, 'number of shelves');
  assert.equal(bar.viz_getDataRequest(), undefined);
  const e = arc.report.shelf
    .element()
    .col_alias('population')
    .dataset(dataset)
    .expr_hasagg(true)
    .dataset_colname('population')
    .dataset_coltype('BIGINT')
    .fn_type(arc.report.fn_types.AGGREGATE)
    .fn_name('sum');

  const e2 = arc.report.shelf
    .element()
    .col_alias('state')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('state')
    .dataset_coltype('STRING')
    .fn_type(arc.report.fn_types.DIMENSION);

  const e3 = arc.report.shelf
    .element()
    .col_alias('year')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('year')
    .dataset_coltype('BIGINT')
    .filter_column('year')
    .custom_expr('[year] in (1790)');

  const shelves = driver.shelves();
  let se_d = {};
  e2.save(se_d);
  shelves.add('x_shelf', [se_d]);

  let captured_msg;
  const bak = arc.report.alertMessageSmallBase;
  try {
    arc.report.alertMessageSmallBase = function (config) {
      captured_msg = config.alertMessage;
    };

    assert.equal(bar.viz_getDataRequest(), undefined);
    assert.equal(
      captured_msg,
      'Please specify at least 1 element in the Y Axis shelf',
    );
  } finally {
    arc.report.alertMessageSmallBase = bak;
  }

  se_d = {};
  e.save(se_d);
  shelves.add('y_shelf', [se_d]);

  se_d = {};
  e3.save(se_d);
  shelves.add('filters_shelf', [se_d]);

  const dataRequest = bar.viz_getDataRequest();
  assert.equal(
    dataRequest.sql(),
    "select state as 'state', sum(population) as 'population' from `a`.`b` where (year in (1790)) group by 1 limit 5000",
  );
  assert.equal(
    dataRequest.dsreq(),
    '{"version":1,"highlighting":false,"type":"SQL","limit":5000,"time":[],"session":[],"dimensions":[{"type":"SIMPLE","expr":"[state] as \'state\'"}],"aggregates":[{"expr":"sum([population]) as \'population\'"}],"partitions":[],"filters":["[year] in (1790)"],"outerFilters":[],"having":[],"global_segments":[],"events":[],"distinct":false,"derived_data":[],"solr_query":[],"autotime_override":"","ksqlStreamPosition":"","dataset_id":1}',
  );
  const cols = dataRequest.columns();
  assert.equal(cols.length, 2);
  assert.equal(cols[0].colsource(), 'x_shelf');
  assert.equal(cols[0].colproperty(), arc.report.columnTypeEnum.DIMENSION);
  assert.equal(cols[1].colsource(), 'y_shelf');
  assert.equal(cols[1].colproperty(), arc.report.columnTypeEnum.AGGREGATE);
});

QUnit.test('bar_tline_builder', (assert) => {
  const column = arc.datasets.Column();

  const table = arc.datasets.Table().tablename('a.b');
  column.table(table);

  const dataset = arc.datasets
    .Dataset()
    .dataset_id(1)
    .tables([table])
    .dataconnection_type('sqlite')
    .add_column(column);
  table.dataset(dataset);

  const bar = arc.visual.plugins.tline();
  const driver = arc.report
    .driver(bar)
    .dataset(dataset)
    .params(arc.params.tracker());

  bar.viz_drawDesigner('app-shelves');
  const tr_sel = d3.selectAll('#app-shelves > div').each(function (d, tr_i) {
    if (tr_i === 0) {
      assert.equal($(this).find('.shelf-label').text(), '* X Axis');
    }

    if (tr_i === 1) {
      assert.equal($(this).find('.shelf-label').text(), '* Y Axis');
    }

    if (tr_i === 2) {
      assert.equal($(this).find('.shelf-label').text(), 'Colors');
    }

    if (tr_i === 3) {
      assert.equal($(this).find('.shelf-label').text(), 'Tooltips');
    }
  });

  assert.equal(tr_sel.size(), 7, 'number of shelves');
  assert.equal(bar.viz_getDataRequest(), undefined);
  const e = arc.report.shelf
    .element()
    .col_alias('population')
    .dataset(dataset)
    .expr_hasagg(true)
    .dataset_colname('population')
    .dataset_coltype('BIGINT')
    .fn_type(arc.report.fn_types.AGGREGATE)
    .fn_name('sum');

  const e2 = arc.report.shelf
    .element()
    .col_alias('state')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('state')
    .dataset_coltype('STRING')
    .fn_type(arc.report.fn_types.DIMENSION);

  const e3 = arc.report.shelf
    .element()
    .col_alias('year')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('year')
    .dataset_coltype('BIGINT')
    .filter_column('year')
    .filter_data(['2000', '2001', '2003']);

  const shelves = driver.shelves();
  let se_d = {};
  e2.save(se_d);
  shelves.add('x_shelf', [se_d]);

  let captured_msg;
  const bak = arc.report.alertMessageSmallBase;
  try {
    arc.report.alertMessageSmallBase = function (config) {
      captured_msg = config.alertMessage;
    };

    assert.equal(bar.viz_getDataRequest(), undefined);
    assert.equal(
      captured_msg,
      'Please specify at least 1 element in the Y Axis shelf',
    );
  } finally {
    arc.report.alertMessageSmallBase = bak;
  }

  se_d = {};
  e.save(se_d);
  shelves.add('y_shelf', [se_d]);

  se_d = {};
  e3.save(se_d);
  shelves.add('filters_shelf', [se_d]);

  const dataRequest = bar.viz_getDataRequest();
  assert.equal(
    dataRequest.sql(),
    "select state as 'state', sum(population) as 'population' from `a`.`b` where (year in (2000, 2001, 2003)) group by 1 limit 5000",
  );
  assert.equal(
    dataRequest.dsreq(),
    '{"version":1,"highlighting":false,"type":"SQL","limit":5000,"time":[],"session":[],"dimensions":[{"type":"SIMPLE","expr":"[state] as \'state\'"}],"aggregates":[{"expr":"sum([population]) as \'population\'"}],"partitions":[],"filters":["[year] in (2000, 2001, 2003)"],"outerFilters":[],"having":[],"global_segments":[],"events":[],"distinct":false,"derived_data":[],"solr_query":[],"autotime_override":"","ksqlStreamPosition":"","dataset_id":1}',
  );
  const cols = dataRequest.columns();
  assert.equal(cols.length, 2);
  assert.equal(cols[0].colsource(), 'x_shelf');
  assert.equal(cols[0].colproperty(), arc.report.columnTypeEnum.DIMENSION);
  assert.equal(cols[1].colsource(), 'y_shelf');
  assert.equal(cols[1].colproperty(), arc.report.columnTypeEnum.AGGREGATE);
});

QUnit.test('bar_group_builder', (assert) => {
  const column = arc.datasets.Column();

  const table = arc.datasets.Table().tablename('a.b');
  column.table(table);

  const dataset = arc.datasets
    .Dataset()
    .dataset_id(1)
    .tables([table])
    .dataconnection_type('sqlite')
    .add_column(column);
  table.dataset(dataset);

  const bar = arc.visual.plugins.tline();
  const driver = arc.report
    .driver(bar)
    .dataset(dataset)
    .params(arc.params.tracker());
  bar.viz_drawDesigner('app-shelves');
  const tr_sel = d3.selectAll('#app-shelves > div').each(function (d, tr_i) {
    if (tr_i === 0) {
      assert.equal($(this).find('.shelf-label').text(), '* X Axis');
    }

    if (tr_i === 1) {
      assert.equal($(this).find('.shelf-label').text(), '* Y Axis');
    }

    if (tr_i === 2) {
      assert.equal($(this).find('.shelf-label').text(), 'Colors');
    }

    if (tr_i === 3) {
      assert.equal($(this).find('.shelf-label').text(), 'Tooltips');
    }
  });

  assert.equal(tr_sel.size(), 7, 'number of shelves');
  assert.equal(bar.viz_getDataRequest(), undefined);
  const e = arc.report.shelf
    .element()
    .col_alias('population')
    .dataset(dataset)
    .expr_hasagg(true)
    .dataset_colname('population')
    .dataset_coltype('BIGINT')
    .fn_type(arc.report.fn_types.AGGREGATE)
    .fn_name('sum');

  const e2 = arc.report.shelf
    .element()
    .col_alias('state')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('state')
    .dataset_coltype('STRING')
    .fn_type(arc.report.fn_types.DIMENSION);

  const e3 = arc.report.shelf
    .element()
    .col_alias('year')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('year')
    .dataset_coltype('BIGINT')
    .filter_column('year')
    .custom_expr('[year] in (1790)');

  const e4 = arc.report.shelf
    .element()
    .col_alias('year')
    .dataset(dataset)
    .expr_hasagg(false)
    .dataset_colname('year')
    .dataset_coltype('BIGINT')
    .fn_type(arc.report.fn_types.AGGREGATE);

  const shelves = driver.shelves();
  let se_d = {};
  e2.save(se_d);
  shelves.add('x_shelf', [se_d]);

  se_d = {};
  e4.save(se_d);
  shelves.add('x_shelf', [se_d]);

  let captured_msg;
  const bak = arc.report.alertMessageSmallBase;
  try {
    arc.report.alertMessageSmallBase = function (config) {
      captured_msg = config.alertMessage;
    };

    assert.equal(bar.viz_getDataRequest(), undefined);
    assert.equal(
      captured_msg,
      'Please specify at least 1 element in the Y Axis shelf',
    );
  } finally {
    arc.report.alertMessageSmallBase = bak;
  }

  se_d = {};
  e.save(se_d);
  shelves.add('y_shelf', [se_d]);

  se_d = {};
  e3.save(se_d);
  shelves.add('filters_shelf', [se_d]);

  const dataRequest = bar.viz_getDataRequest();
  assert.equal(
    dataRequest.sql(),
    "select state as 'state', year as 'year', sum(population) as 'population' from `a`.`b` where (year in (1790)) group by 1, 2 limit 5000",
  );
  assert.equal(
    dataRequest.dsreq(),
    '{"version":1,"highlighting":false,"type":"SQL","limit":5000,"time":[],"session":[],"dimensions":[{"type":"SIMPLE","expr":"[state] as \'state\'"},{"type":"SIMPLE","expr":"[year] as \'year\'"}],"aggregates":[{"expr":"sum([population]) as \'population\'"}],"partitions":[],"filters":["[year] in (1790)"],"outerFilters":[],"having":[],"global_segments":[],"events":[],"distinct":false,"derived_data":[],"solr_query":[],"autotime_override":"","ksqlStreamPosition":"","dataset_id":1}',
  );
  const cols = dataRequest.columns();
  assert.equal(cols.length, 3);
  assert.equal(cols[0].colsource(), 'x_shelf');
  assert.equal(cols[0].colproperty(), arc.report.columnTypeEnum.DIMENSION);
  assert.equal(cols[1].colsource(), 'x_shelf');
  assert.equal(cols[1].colproperty(), arc.report.columnTypeEnum.DIMENSION);
  assert.equal(cols[2].colsource(), 'y_shelf');
  assert.equal(cols[2].colproperty(), arc.report.columnTypeEnum.AGGREGATE);
});
