QUnit.module('builder_arc_dataconnections', {});

QUnit.test('base', (assert) => {
  assert.equal(Object.keys(arc.dataconnection.connectionTypeEnum).length, 30);
});

QUnit.test('brackets', (assert) => {
  assert.equal(arc.dataconnection.types.brackets('colname'), '[colname]');
  assert.equal(arc.dataconnection.types.brackets('[colname]'), '[colname]');
  assert.equal(arc.dataconnection.types.brackets('sum(1)'), 'sum(1)');
});

QUnit.test('enquote', (assert) => {
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', 'mssqlserver'),
    '[dbname].[tablename]',
  );

  const backquotes = ['vertica', 'snowflake', 'athena'];
  backquotes.forEach((connectionType) => {
    assert.equal(
      arc.dataconnection._enquote_tablename('dbname.tablename', connectionType),
      '"dbname"."tablename"',
    );
  });

  const doubleQuotes = [
    'postgres',
    'aster',
    'teradata',
    'redshift',
    'oracle',
    'denodo',
  ];

  doubleQuotes.forEach((connectionType) => {
    assert.equal(
      arc.dataconnection._enquote_tablename('dbname.tablename', connectionType),
      '"dbname"."tablename"',
    );
  });

  Object.keys(arc.dataconnection.connectionTypeEnum).forEach((dc_type) => {
    if (doubleQuotes.indexOf(dc_type) > -1) {
      return;
    }

    if (
      dc_type === 'mssqlserver' ||
      dc_type === 'vertica' ||
      dc_type === 'snowflake' ||
      dc_type === 'athena' ||
      dc_type === 'elasticsearch' ||
      dc_type === 'cassandra'
    ) {
      return;
    }

    if (dc_type === 'druid' || dc_type === 'duckdb') {
      assert.equal(
        arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
        '"dbname"."tablename"',
      );
    } else {
      assert.equal(
        arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
        '`dbname`.`tablename`',
      );
    }
  });
});

QUnit.test('derived_data', (assert) => {
  const enabled = [
    'hive',
    'hyve',
    'impala',
    'impyla',
    'arcengine',
    'arcengine_v2',
    'sqlite',
    'sparksql',
    'bigquery',
    'snowflake',
    'duckdb',
  ];

  Object.keys(arc.dataconnection.connectionTypeEnum).forEach((dc_type) => {
    assert.equal(
      arc.dataconnection.supports_derived_data(dc_type),
      enabled.indexOf(dc_type) > -1,
    );
  });
});

QUnit.test('data_import', (assert) => {
  const enabled = [
    'arcengine_v2',
    'duckdb',
    'hyve',
    'impyla',
    'snowflake',
    'sqlite',
    'mariadb',
    'mysql',
    'postgres',
  ];

  Object.keys(arc.dataconnection.connectionTypeEnum).forEach((dc_type) => {
    assert.equal(
      arc.dataconnection.supports_data_import(dc_type),
      enabled.indexOf(dc_type) > -1,
    );
  });
});

QUnit.test('sample', (assert) => {
  assert.equal(
    arc.dataconnection.sample_sql('dbname.tablename', 'mssqlserver', 10),
    '.rows dbname.tablename',
  );
  assert.equal(
    arc.dataconnection.sample_sql('tablename', 'ksql', 10),
    '.rows tablename',
  );
  assert.equal(
    arc.dataconnection.sample_sql('dbname.tablename', 'bigquery', 10),
    '.rows dbname.tablename',
  );
  assert.equal(
    arc.dataconnection.sample_sql('dbname.tablename', 'oracle', 10),
    'select * from "dbname"."tablename" where rownum <= 10',
  );
  assert.equal(
    arc.dataconnection.sample_sql('dbname.tablename', 'teradata', 10),
    'select * from "dbname"."tablename" sample 10',
  );
  assert.equal(
    arc.dataconnection.sample_sql('dbname.tablename', 'arcengine', 10),
    'select * from `dbname`.`tablename` limit 10',
  );
  assert.equal(
    arc.dataconnection.sample_sql('dbname.tablename', 'arcengine_v2', 10),
    'select * from `dbname`.`tablename` limit 10',
  );
});

QUnit.test('database_label', (assert) => {
  assert.equal(arc.dataconnection.database_label('postgres'), 'Schema');
  assert.equal(arc.dataconnection.database_label('redshift'), 'Schema');
  assert.equal(arc.dataconnection.database_label('oracle'), 'User');
  assert.equal(arc.dataconnection.database_label('arcengine'), 'Database');
  assert.equal(arc.dataconnection.database_label('arcengine_v2'), 'Database');
  assert.equal(arc.dataconnection.database_label('ksql'), 'Topic');
});

QUnit.test('table_label', (assert) => {
  assert.equal(arc.dataconnection.table_label('postgres'), 'Table');
  assert.equal(arc.dataconnection.table_label('redshift'), 'Table');
  assert.equal(arc.dataconnection.table_label('oracle'), 'Table');
  assert.equal(arc.dataconnection.table_label('arcengine'), 'Table');
  assert.equal(arc.dataconnection.table_label('arcengine_v2'), 'Table');
  assert.equal(arc.dataconnection.table_label('ksql'), 'Table or Stream');
});

QUnit.test('cache', (assert) => {
  assert.equal(arc.dataconnection.defaultCacheEnabled('arcengine'), 1);
  assert.equal(arc.dataconnection.defaultCacheEnabled('arcengine_v2'), 1);

  Object.keys(arc.dataconnection.connectionTypeEnum).forEach((dc_type) => {
    if (
      dc_type !== 'arcengine' &&
      dc_type !== 'arcengine_v2' &&
      dc_type !== 'athena' &&
      dc_type !== 'vertica'
    ) {
      assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 0);
    }
  });
});

QUnit.test('cache_retention', (assert) => {
  assert.equal(arc.dataconnection.defaultCacheRetention('arcengine'), 300);
  assert.equal(arc.dataconnection.defaultCacheRetention('arcengine_v2'), 300);
  assert.equal(arc.dataconnection.defaultCacheRetention('vertica'), 1800);
  assert.equal(arc.dataconnection.defaultCacheRetention('athena'), 1800);
  assert.equal(arc.dataconnection.defaultCacheRetention('vertica'), 1800);

  Object.keys(arc.dataconnection.connectionTypeEnum).forEach((dc_type) => {
    if (
      dc_type !== 'arcengine' &&
      dc_type !== 'arcengine_v2' &&
      dc_type !== 'athena' &&
      dc_type !== 'vertica'
    ) {
      assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 86400);
    }
  });
});

QUnit.test('bigquery', (assert) => {
  const dc_type = 'bigquery';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 0);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    '.rows tablename',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Database');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), true);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), true);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), false);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as string)',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 0);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 86400);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 0);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '`dbname`.`tablename`',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'string');
  assert.equal(dc.cast_bigint, 'int64');
  assert.equal(dc.cast_real, 'float64');
  assert.equal(dc.cast_boolean, 'bool');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'approx_count_distinct');
  assert.equal(dc.group_concat, 'string_agg');
  assert.equal(dc.appx_median, undefined);
  assert.equal(dc.stddev, 'stddev');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'variance');
  assert.equal(dc.variance_pop, 'var_pop');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, true);
  assert.equal(dc.supportsTopk, false);
  assert.equal(dc.supportsWindowFunctions, true);

  // Date conversions
  assert.equal(dc.to_date('colname'), 'extract(date from [colname])');
  assert.equal(dc.expr_year('colname'), 'extract(year from [colname])');
  assert.equal(dc.expr_month('colname'), 'extract(month from [colname])');
  assert.equal(dc.expr_dayofmonth('colname'), 'extract(day from [colname])');
  assert.equal(
    dc.expr_dayofyear('colname'),
    'extract(dayofyear from [colname])',
  );
  assert.equal(
    dc.expr_dayofweek('colname'),
    'extract(dayofweek from [colname])',
  );

  assert.equal(dc.date, undefined);
  assert.equal(dc.year, undefined);
  assert.equal(dc.month, undefined);
  assert.equal(dc.dayofyear, undefined);
  assert.equal(dc.dayofmonth, undefined);
  assert.equal(dc.dayofweek, undefined);
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    'if(extract(dayofweek from [colname]) = 1, 8, extract(dayofweek from [colname]))',
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "format_timestamp('%Y-M%m', [colname])",
  );

  assert.equal(dc.dowToShortName(1), 'Sun');
  assert.equal(dc.dowToLongName(1), 'Sunday');
  assert.equal(dc.dowToShortName(7), 'Sat');
  assert.equal(dc.dowToLongName(7), 'Saturday');
  assert.equal(dc.dowToShortName(8), 'Sun');
  assert.equal(dc.dowToLongName(8), 'Sunday');
  assert.equal(dc.dow_low, 1);
});

QUnit.test('teradata', (assert) => {
  const dc_type = 'teradata';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 1025);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    'select * from "tablename" sample 10',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Database');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), false);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), false);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), false);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as varchar(4096))',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 1025);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 86400);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 0);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '"dbname"."tablename"',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'varchar(4096)');
  assert.equal(dc.cast_bigint, 'bigint');
  assert.equal(dc.cast_real, 'real');
  assert.equal(dc.cast_boolean, 'boolean');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'ndv');
  assert.equal(dc.group_concat, 'group_concat');
  assert.equal(dc.appx_median, undefined);
  assert.equal(dc.stddev, 'stddev_samp');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'var_samp');
  assert.equal(dc.variance_pop, 'var_pop');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, false);
  assert.equal(dc.supportsTopk, true);
  assert.equal(dc.supportsWindowFunctions, false);

  // Date conversions
  assert.equal(dc.to_date('colname'), "to_char([colname], 'YYYY-MM-DD')");
  assert.equal(dc.expr_year('colname'), 'extract(year from [colname])');
  assert.equal(dc.expr_month('colname'), 'extract(month from [colname])');
  assert.equal(dc.expr_dayofmonth('colname'), 'extract(day from [colname])');
  assert.equal(dc.expr_dayofyear('colname'), "to_char([colname], 'DDD')");
  assert.equal(dc.expr_dayofweek('colname'), "to_char([colname], 'd') - 1");

  assert.equal(dc.date, undefined);
  assert.equal(dc.year, undefined);
  assert.equal(dc.month, undefined);
  assert.equal(dc.dayofyear, undefined);
  assert.equal(dc.dayofmonth, undefined);
  assert.equal(dc.dayofweek, undefined);
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    "COALESCE(NULLIF((to_char(cast([colname] as date), 'd') - 1), 0), 7)",
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "concat(to_char([colname], 'YYYY'), '-M', to_char([colname], 'MM'))",
  );

  assert.equal(dc.dowToShortName(1), 'Mon');
  assert.equal(dc.dowToLongName(1), 'Monday');
  assert.equal(dc.dowToShortName(7), 'Sun');
  assert.equal(dc.dowToLongName(7), 'Sunday');
  assert.equal(dc.dowToShortName(8), undefined);
  assert.equal(dc.dowToLongName(8), undefined);
  assert.equal(dc.dow_low, 0);
});

QUnit.test('arcengine', (assert) => {
  const dc_type = 'arcengine';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 31050);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    'select * from `tablename` limit 10',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Database');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), true);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), true);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), false);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as string)',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 31050);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 300);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 1);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '`dbname`.`tablename`',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'string');
  assert.equal(dc.cast_bigint, 'bigint');
  assert.equal(dc.cast_real, 'real');
  assert.equal(dc.cast_boolean, 'boolean');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'ndv');
  assert.equal(dc.group_concat, 'group_concat');
  assert.equal(dc.appx_median, 'appx_median');
  assert.equal(dc.stddev, 'stddev');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'variance');
  assert.equal(dc.variance_pop, 'variance_pop');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, true);
  assert.equal(dc.supportsTopk, false);
  assert.equal(dc.supportsWindowFunctions, true);

  // Date conversions
  assert.equal(
    dc.to_date('colname'),
    "from_timestamp([colname], 'yyyy-MM-dd')",
  );
  assert.equal(dc.expr_year('colname'), "from_timestamp([colname], 'yyyy')");
  assert.equal(dc.expr_month, undefined);
  assert.equal(dc.expr_dayofmonth, undefined);
  assert.equal(dc.expr_dayofyear, undefined);
  assert.equal(dc.expr_dayofweek, undefined);

  assert.equal(dc.date, 'to_date');
  assert.equal(dc.year, 'year');
  assert.equal(dc.month, 'month');
  assert.equal(dc.dayofyear, 'dayofyear');
  assert.equal(dc.dayofmonth, 'dayofmonth');
  assert.equal(dc.dayofweek, 'dayofweek');
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    'if(dayofweek([colname]) = 1, 8, dayofweek([colname]))',
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "concat(from_timestamp([colname], 'yyyy'), '-M', from_timestamp([colname], 'MM'))",
  );

  assert.equal(dc.dowToShortName(1), 'Sun');
  assert.equal(dc.dowToLongName(1), 'Sunday');
  assert.equal(dc.dowToShortName(7), 'Sat');
  assert.equal(dc.dowToLongName(7), 'Saturday');
  assert.equal(dc.dowToShortName(8), 'Sun');
  assert.equal(dc.dowToLongName(8), 'Sunday');
  assert.equal(dc.dow_low, 1);
});

QUnit.test('arcengine_v2', (assert) => {
  const dc_type = 'arcengine_v2';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 31050);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    'select * from `tablename` limit 10',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Database');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), true);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), true);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), true);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as string)',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 31050);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 300);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 1);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '`dbname`.`tablename`',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'string');
  assert.equal(dc.cast_bigint, 'bigint');
  assert.equal(dc.cast_real, 'real');
  assert.equal(dc.cast_boolean, 'boolean');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'ndv');
  assert.equal(dc.group_concat, 'group_concat');
  assert.equal(dc.appx_median, 'appx_median');
  assert.equal(dc.stddev, 'stddev');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'variance');
  assert.equal(dc.variance_pop, 'variance_pop');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, true);
  assert.equal(dc.supportsTopk, false);
  assert.equal(dc.supportsWindowFunctions, true);

  // Date conversions
  assert.equal(
    dc.to_date('colname'),
    "from_timestamp([colname], 'yyyy-MM-dd')",
  );
  assert.equal(dc.expr_year('colname'), "from_timestamp([colname], 'yyyy')");
  assert.equal(dc.expr_month, undefined);
  assert.equal(dc.expr_dayofmonth, undefined);
  assert.equal(dc.expr_dayofyear, undefined);
  assert.equal(dc.expr_dayofweek, undefined);

  assert.equal(dc.date, 'to_date');
  assert.equal(dc.year, 'year');
  assert.equal(dc.month, 'month');
  assert.equal(dc.dayofyear, 'dayofyear');
  assert.equal(dc.dayofmonth, 'dayofmonth');
  assert.equal(dc.dayofweek, 'dayofweek');
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    'if(dayofweek([colname]) = 1, 8, dayofweek([colname]))',
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "concat(from_timestamp([colname], 'yyyy'), '-M', from_timestamp([colname], 'MM'))",
  );

  assert.equal(dc.dowToShortName(1), 'Sun');
  assert.equal(dc.dowToLongName(1), 'Sunday');
  assert.equal(dc.dowToShortName(7), 'Sat');
  assert.equal(dc.dowToLongName(7), 'Saturday');
  assert.equal(dc.dowToShortName(8), 'Sun');
  assert.equal(dc.dowToLongName(8), 'Sunday');
  assert.equal(dc.dow_low, 1);
});

QUnit.test('impala', (assert) => {
  const dc_type = 'impala';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 21050);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    'select * from `tablename` limit 10',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Database');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), true);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), true);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), false);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as string)',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 21050);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 86400);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 0);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '`dbname`.`tablename`',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'string');
  assert.equal(dc.cast_bigint, 'bigint');
  assert.equal(dc.cast_real, 'real');
  assert.equal(dc.cast_boolean, 'boolean');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'ndv');
  assert.equal(dc.group_concat, 'group_concat');
  assert.equal(dc.appx_median, 'appx_median');
  assert.equal(dc.stddev, 'stddev');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'variance');
  assert.equal(dc.variance_pop, 'variance_pop');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, true);
  assert.equal(dc.supportsTopk, false);
  assert.equal(dc.supportsWindowFunctions, true);

  // Date conversions
  assert.equal(
    dc.to_date('colname'),
    "from_timestamp([colname], 'yyyy-MM-dd')",
  );
  assert.equal(dc.expr_year('colname'), "from_timestamp([colname], 'yyyy')");
  assert.equal(dc.expr_month, undefined);
  assert.equal(dc.expr_dayofmonth, undefined);
  assert.equal(dc.expr_dayofyear, undefined);
  assert.equal(dc.expr_dayofweek, undefined);

  assert.equal(dc.date, 'to_date');
  assert.equal(dc.year, 'year');
  assert.equal(dc.month, 'month');
  assert.equal(dc.dayofyear, 'dayofyear');
  assert.equal(dc.dayofmonth, 'dayofmonth');
  assert.equal(dc.dayofweek, 'dayofweek');
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    'if(dayofweek([colname]) = 1, 8, dayofweek([colname]))',
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "concat(from_timestamp([colname], 'yyyy'), '-M', from_timestamp([colname], 'MM'))",
  );

  assert.equal(dc.dowToShortName(1), 'Sun');
  assert.equal(dc.dowToLongName(1), 'Sunday');
  assert.equal(dc.dowToShortName(7), 'Sat');
  assert.equal(dc.dowToLongName(7), 'Saturday');
  assert.equal(dc.dowToShortName(8), 'Sun');
  assert.equal(dc.dowToLongName(8), 'Sunday');
  assert.equal(dc.dow_low, 1);
});

QUnit.test('impyla', (assert) => {
  const dc_type = 'impyla';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 21050);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    'select * from `tablename` limit 10',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Database');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), true);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), true);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), true);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as string)',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 21050);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 86400);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 0);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '`dbname`.`tablename`',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'string');
  assert.equal(dc.cast_bigint, 'bigint');
  assert.equal(dc.cast_real, 'real');
  assert.equal(dc.cast_boolean, 'boolean');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'ndv');
  assert.equal(dc.group_concat, 'group_concat');
  assert.equal(dc.appx_median, 'appx_median');
  assert.equal(dc.stddev, 'stddev');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'variance');
  assert.equal(dc.variance_pop, 'variance_pop');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, true);
  assert.equal(dc.supportsTopk, false);
  assert.equal(dc.supportsWindowFunctions, true);

  // Date conversions
  assert.equal(
    dc.to_date('colname'),
    "from_timestamp([colname], 'yyyy-MM-dd')",
  );
  assert.equal(dc.expr_year('colname'), "from_timestamp([colname], 'yyyy')");
  assert.equal(dc.expr_month, undefined);
  assert.equal(dc.expr_dayofmonth, undefined);
  assert.equal(dc.expr_dayofyear, undefined);
  assert.equal(dc.expr_dayofweek, undefined);

  assert.equal(dc.date, 'to_date');
  assert.equal(dc.year, 'year');
  assert.equal(dc.month, 'month');
  assert.equal(dc.dayofyear, 'dayofyear');
  assert.equal(dc.dayofmonth, 'dayofmonth');
  assert.equal(dc.dayofweek, 'dayofweek');
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    'if(dayofweek([colname]) = 1, 8, dayofweek([colname]))',
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "concat(from_timestamp([colname], 'yyyy'), '-M', from_timestamp([colname], 'MM'))",
  );

  assert.equal(dc.dowToShortName(1), 'Sun');
  assert.equal(dc.dowToLongName(1), 'Sunday');
  assert.equal(dc.dowToShortName(7), 'Sat');
  assert.equal(dc.dowToLongName(7), 'Saturday');
  assert.equal(dc.dowToShortName(8), 'Sun');
  assert.equal(dc.dowToLongName(8), 'Sunday');
  assert.equal(dc.dow_low, 1);
});

QUnit.test('snowflake', (assert) => {
  const dc_type = 'snowflake';
  const dc = arc.dataconnection.types[dc_type]();

  // Basic configuration
  assert.equal(dc.connection_type, dc_type);
  assert.equal(dc.default_port, 0);

  // Functions based on dc_type
  assert.equal(
    arc.dataconnection.sample_sql('tablename', dc_type, 10),
    'select * from "tablename" limit 10',
  );
  assert.equal(arc.dataconnection.database_label(dc_type), 'Schema');
  assert.equal(arc.dataconnection.supportsNdv(dc_type), true);
  assert.equal(arc.dataconnection.supports_derived_data(dc_type), true);
  assert.equal(arc.dataconnection.supports_data_import(dc_type), true);
  assert.equal(
    arc.dataconnection.castAsString('colname', dc_type),
    'cast([colname] as string)',
  );
  assert.equal(arc.dataconnection.defaultPort(dc_type), 0);
  assert.equal(arc.dataconnection.defaultCacheRetention(dc_type), 86400);
  assert.equal(arc.dataconnection.defaultCacheEnabled(dc_type), 0);
  assert.equal(
    arc.dataconnection._enquote_tablename('dbname.tablename', dc_type),
    '"dbname"."tablename"',
  );

  // Column types
  assert.equal(dc.cast_timestamp, 'timestamp');
  assert.equal(dc.cast_string, 'string');
  assert.equal(dc.cast_bigint, 'integer');
  assert.equal(dc.cast_real, 'float');
  assert.equal(dc.cast_boolean, 'boolean');
  assert.equal(dc.is_string('string'), true);
  assert.equal(dc.is_string('STRING'), true);

  // Aggregate functions
  assert.equal(dc.sum, 'sum');
  assert.equal(dc.count, 'count');
  assert.equal(dc.min, 'min');
  assert.equal(dc.max, 'max');
  assert.equal(dc.avg, 'avg');
  assert.equal(dc.ndv, 'hll');
  assert.equal(dc.group_concat, 'group_concat');
  assert.equal(dc.appx_median, undefined);
  assert.equal(dc.stddev, 'stddev');
  assert.equal(dc.stddev_pop, 'stddev_pop');
  assert.equal(dc.variance, 'variance');
  assert.equal(dc.variance_pop, 'variance_pop');
  assert.equal(dc.expr_appxmedian('col'), 'approx_percentile([col],0.5)');

  // String functions
  assert.equal(dc.upper, 'upper');
  assert.equal(dc.lower, 'lower');

  // Functional support
  assert.equal(dc.supportsLimit, true);
  assert.equal(dc.supportsTopk, false);
  assert.equal(dc.supportsWindowFunctions, false);

  // Date conversions
  assert.equal(dc.to_date, undefined);
  assert.equal(dc.expr_year, undefined);
  assert.equal(dc.expr_month, undefined);
  assert.equal(dc.expr_dayofmonth, undefined);
  assert.equal(dc.expr_dayofyear, undefined);
  assert.equal(dc.expr_dayofweek, undefined);

  assert.equal(dc.date, 'to_date');
  assert.equal(dc.year, 'year');
  assert.equal(dc.month, 'month');
  assert.equal(dc.dayofyear, 'dayofyear');
  assert.equal(dc.dayofmonth, 'dayofmonth');
  assert.equal(dc.dayofweek, 'dayofweek');
  assert.equal(
    dc.expr_dayofweek_mon('colname', {}),
    'iff(dayofweek([colname]) = 0, 7, dayofweek([colname]))',
  );

  assert.equal(
    dc.expr_yearmonth('colname'),
    "concat(concat(YEAR([colname])::string,'-M'),lpad(month([colname])::string,2,'0'))",
  );

  assert.equal(dc.dowToShortName(0), 'Sun');
  assert.equal(dc.dowToLongName(0), 'Sunday');
  assert.equal(dc.dowToShortName(6), 'Sat');
  assert.equal(dc.dowToLongName(6), 'Saturday');
  assert.equal(dc.dowToShortName(7), 'Sun');
  assert.equal(dc.dowToLongName(7), 'Sunday');
  assert.equal(dc.dow_low, 0);
});
