(function () {
  // utility for dataset creation
  function createDataset() {
    const column = arc.datasets.Column();

    const table = arc.datasets.Table().tablename('a.b');
    column.table(table);

    const dataset = arc.datasets
      .Dataset()
      .dataset_id(1)
      .tables([table])
      .dataconnection_type('sqlite')
      .add_column(column);
    table.dataset(dataset);
    return dataset;
  }

  // utility for creating and adding a shelf element to a visual shelf
  function addShelfElement(driver, shelf_name, shelf_data) {
    const shelfElement = arc.report.shelf
      .element()
      .custom_expr(shelf_data.custom_expr)
      .dataset(shelf_data.dataset)
      .expr_hasagg(false);

    const shelves = driver.shelves();
    const se_d = {};
    shelfElement.save(se_d);
    shelves.add(shelf_name, [se_d]);
  }

  QUnit.module('builder_arc_histogram', {
    beforeEach() {
      // mock arc.data.baseQuery:
      const baseQueryArgs = [];
      const baseQueryRunArgs = [];
      this.baseQueryArgs = baseQueryArgs;
      this.baseQueryRunArgs = baseQueryRunArgs;
      this.oldBaseQuery = arc.data.baseQuery;
      arc.data.baseQuery = (...args1) => {
        baseQueryArgs.push(args1);
        return {
          run(...args2) {
            baseQueryRunArgs.push(args2);
          },
        };
      };
    },

    afterEach() {
      // restore stub:
      arc.data.baseQuery = this.oldBaseQuery;
    },
  });

  QUnit.test.only(
    'handles shelf element alias and expression',
    function (assert) {
      arc.globals.reports = arc.reporttracker.reports();
      arc.globals.params = arc.params.tracker();

      const histogram = arc.visual.plugins.histogram();

      const dataset = createDataset();
      const driver = arc.report
        .driver(histogram)
        .dataset(dataset)
        .params(arc.params.tracker());

      // must be painted to have shelves
      histogram.viz_drawDesigner('app-shelves');

      const shelf_element = {
        custom_expr: "[calories] * 2 as 'Doubled Calories'",
        dataset,
      };

      addShelfElement(driver, 'aggregates_shelf', shelf_element);

      // a histogram issues two data requests: one for retrieving the range min and max values,
      // and one for retrieving the distribution in the range
      const dataRequest = histogram.viz_getDataRequest();
      assert.ok(dataRequest);
      const dsreq = JSON.parse(dataRequest.dsreq());
      assert.equal(dsreq.aggregates.length, 2);
      assert.equal(dsreq.aggregates[0].expr, "min([calories] * 2) as 'minVal'");
      assert.equal(dsreq.aggregates[1].expr, "max([calories] * 2) as 'maxVal'");

      // mimics the minVal and maxVal values above:
      histogram.viz_setDataResult(arc.data.result().rows([[100, 320]]));
      histogram.viz_drawVisual('chart-div');

      assert.equal(this.baseQueryRunArgs.length, 1);
      const dsreq2 = JSON.parse(this.baseQueryRunArgs[0][0].dsreq);

      assert.equal(dsreq2.dimensions.length, 1);
      assert.equal(dsreq2.aggregates.length, 1);
      assert.equal(
        dsreq2.dimensions[0].expr,
        "cast((([calories] * 2 - 100) / 22) as bigint) as 'Buckets'",
      );
      assert.equal(dsreq2.aggregates[0].expr, "sum(1) as 'Record Count'");

      const dataCallback = this.baseQueryArgs[0][1];
      // simulate successful data retrieval:
      const frequencyRows = [
        [0, 3],
        [1, 2],
        [2, 1],
        [3, 7],
        [4, 17],
        [5, 29],
        [6, 10],
        [7, 2],
        [8, 3],
        [9, 2],
        [10, 1],
      ];
      dataCallback({ rows: frequencyRows });
      assert.equal($('.x.arc-viz-axis-label').text(), 'Doubled Calories');
    },
  );
}());
