let dashboard;
let dashboard_filter_controls;

QUnit.module('dashboards_arc_dashboards', {
  async before() {
    this.testArea = $('<div id="test-area"></div>');
    $('body').append(this.testArea);

    arc.testutil = arc.testutil || {};

    arc.globals.reports = arc.reporttracker.reports();
    arc.globals.params = arc.params.tracker();

    this.old_disableWorkspaces = arc.testutil.disableWorkspaces;
    this.old_mockColorPaletteData = arc.testutil.mockColorPaletteData;
    this.old_mockDateRanges = arc.testutil.mockDateRanges;

    this.old_fetch = arc.staticfetcher.fetch;
    arc.staticfetcher.fetch = () => ({});
  },

  after() {
    arc.testutil.disableWorkspaces = this.old_disableWorkspaces;
    arc.testutil.mockColorPaletteData = this.old_mockColorPaletteData;
    arc.testutil.mockDateRanges = this.old_mockDateRanges;

    arc.staticfetcher.fetch = this.old_fetch;
  },

  beforeEach(assert) {
    arc.globals.apply_custom_styles = true;
    arc.globals.enable_custom_styles = true;
    arc.globals.is_edit = true;

    arc.testutil.disableWorkspaces = true;
    arc.testutil.mockColorPaletteData = [
      {
        id: 1,
        type: 100,
        name: 'big_color_palette',
        value:
          '{"colors":[{"color":"rgb(69, 213, 21)","value":"","position":0},{"color":"rgb(46, 233, 33)","value":"","position":50},{"color":"rgb(116, 92, 243)","value":"","position":100}]}',
        description: '',
      },
      {
        id: 6,
        type: 1,
        name: 'single_color_palette',
        value: '#541eab',
        description: '',
      },
      {
        id: 8,
        type: 1,
        name: 'different_format_1color_palette',
        value: '\r\nrgb(10,20,30)',
        description: '',
      },
      {
        id: 9,
        type: 1,
        name: 'red-yellow-green custom palette',
        value: '#ff0000\r\n#e6e63b\r\n#27c431',
        description: '',
      },
      {
        id: 12,
        type: 1,
        name: '2colors',
        value: '#612121\r\n#3b7380',
        description: '',
      },
      {
        id: 13,
        type: 1,
        name: 'testing ARC-2444',
        value: 'rgba(25,250,250,1)',
        description: '',
      },
    ];
    arc.testutil.mockDateRanges = [
      { type: null, name: 'test', value: "['abc', 'def', 'ghi']" },
    ];

    dashboard_filter_controls = arc.report
      .dashboard_filter_controls()
      .build_mode(true)
      .dashboard(dashboard)
      .dom_id('#controls-sidebar')
      .activate(true);

    const msdb = arc
      .ms_dashboard()
      .dashboard_filter_controls(dashboard_filter_controls)
      .onSheetActivation('displayappCb', (newlyActiveSheet) => {
        dashboard = newlyActiveSheet;
      });
    arc.globals.ms_dashboard = msdb;
    msdb.activate();

    // Wait time to let dashboard be defined
    const restartQUnit = assert.async();
    setTimeout(() => {
      restartQUnit();
    }, 1000);
  },

  afterEach() {
    $('#chart-div').empty();

    // Reset the appSettings variable for the css manager to an empty set
    arc.css.CUSTOMCSS_MANAGER.appSettings(arc.visual.settings());
    arc.globals.apply_custom_styles = false;
    arc.globals.enable_custom_styles = false;
    arc.globals.is_edit = false;
    delete arc.globals.dashboard;
    delete arc.globals.ms_dashboard;
    dashboard = null;
  },
});

QUnit.test('Basic', (assert) => {
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([[]]),
  );
  assert.equal(dashboard.params_history().dashboard(), dashboard);
});

const params1 = [
  {
    key: 'k1',
    value: 'v1',
  },
  {
    key: 'k2',
    value: 'v2',
  },
];

const params2 = [
  {
    key: 'k1',
    value: 'v1.2',
  },
  {
    key: 'k3',
    value: 'v3.2',
  },
];

const combined_params = [
  {
    key: 'k2',
    value: 'v2',
  },
  {
    key: 'k1',
    value: 'v1.2',
  },
  {
    key: 'k3',
    value: 'v3.2',
  },
];

QUnit.test('Parameters', (assert) => {
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([[]]),
  );

  dashboard.redrawExistingWidgetsWithParams(
    undefined,
    undefined,
    undefined,
    undefined,
    undefined,
    undefined,
    params1,
  );
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([params1]),
  );

  assert.equal(Object.keys(params2).length, 2);
  dashboard.redrawExistingWidgetsWithParams(
    undefined,
    undefined,
    undefined,
    undefined,
    undefined,
    undefined,
    params2,
  );
  assert.equal(Object.keys(params2).length, 2);
  assert.equal(Object.keys(dashboard.current_params()[0]).length, 3);
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([combined_params]),
  );
});

// Parameter history is disabled in edit mode since VIZ 7.1.7
QUnit.test.skip('Parameter history', (assert) => {
  const params_history = dashboard.params_history();
  params_history.back();
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([params1]),
  );

  params_history.back();
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([[]]),
  );

  params_history.forward();
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([params1]),
  );

  params_history.forward();
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([combined_params]),
  );

  params_history.clear();
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([[]]),
  );

  params_history.forward();
  assert.equal(
    JSON.stringify(dashboard.current_params()),
    JSON.stringify([[]]),
  );
});

QUnit.test('Container CSS', (assert) => {
  const settings = dashboard.user_settings();
  const containerCss = settings.find_by_id('container_css_class');
  assert.ok(!$('#main').hasClass('def'));
  $(`#${containerCss.div_id()} input`).val('def').trigger('change');
  assert.ok($('#main').hasClass('def'));
});

QUnit.test('Inline CSS', (assert) => {
  const dashboardCSS = '.gridster .arc-viz-container { margin: 10px 10px;}\n';
  const css = '#main {background-color: green;}';
  const settings = dashboard.user_settings();
  const cssInline = settings.find_by_id('inline_css');
  const filterBg = '.selected-control { background-color: #f8f8f8;}\n';
  cssInline.value(css);

  arc.testutil.testWithoutAce = true;

  cssInline.makeChangeCbs();

  arc.testutil.testWithoutAce = false;

  let read_css = $('#arc-customcss1').html();
  assert.equal(read_css, '');

  read_css = $('#arc-customcss2').html();
  assert.equal(
    read_css,
    filterBg + dashboardCSS + css,
    'Custom css is dashboard.getCss and user-specified value',
  );
});

QUnit.test('External CSS Stylesheets', (assert) => {
  const bar = arc.visual.plugins.bars();
  const driver = arc.report.driver(bar);

  const settings = dashboard.user_settings();

  const s = settings.find_by_id('css_styles');
  s.viz_lib(driver);
  arc.frontend.globals.urls.customCssRawUrl = '/cssurl';
  s.value(JSON.stringify([
    {
      cssid: 1,
      settings: []
    },
    {
      cssid: 2,
      settings: []
    }
  ]));
  s.paintTable();
  s.makeChangeCbs();

  const $css1 = $('head link[href="/cssurl/1"]');
  assert.equal($css1.length, 1);
  assert.equal($css1.attr('href'), `${arc.frontend.globals.urls.customCssRawUrl}/1`);

  const $css2 = $('head link[href="/cssurl/2"]');
  assert.equal($css2.length, 1);
  assert.equal($css2.attr('href'), `${arc.frontend.globals.urls.customCssRawUrl}/2`);
});

QUnit.test('Does not load visuals if side panel is closed', (assert) => {
  // hide the side panel:
  $('.dash-settings-panel').hide();

  arc.staticfetcher.fetch = arc.testutil.createMock();

  // trigger dataset selection:
  $('[data-target="#dash-viz-tab"]').trigger('show.bs.tab.load-data');

  // no invocation of static fetcher:
  assert.equal(arc.staticfetcher.fetch.args.length, 0);
});

QUnit.test('Loads visuals if side panel is open', (assert) => {
  // open the side panel:
  $('.dash-settings-panel').show();

  arc.staticfetcher.fetch = arc.testutil.createMock();
  arc.misc.hasPerm = arc.testutil.createMock();
  arc.misc.hasPerm.returnValue = true;

  const ds_id = 123;
  dashboard.currentDataSetSelection(ds_id);
  dashboard.dataset_hash({
    [ds_id]: {
      dataset_perms: arc.testutil.createMock(),
    },
  });
  arc.frontend.globals.urls.reportUrl = 'report';
  // trigger dataset selection:
  $('[data-target="#dash-viz-tab"]').trigger('show.bs.tab.load-data');

  // 1 invocation of static fetcher:
  assert.equal(arc.staticfetcher.fetch.args.length, 1);
  assert.equal(
    arc.staticfetcher.fetch.args[0][0][0].asset,
    'report?all=true&exclude_dashboards=true&dataset_id=123',
  );

  // loading indicator is shown
  assert.equal($('.dash-settings-panel #visuals-loading').is(':visible'), true);

  // error handling
  const callback1 = arc.staticfetcher.fetch.args[0][1];
  callback1({ response: 'oops' });

  // loading indicator is hidden
  assert.equal(
    $('.dash-settings-panel #visuals-loading').is(':visible'),
    false,
  );

  // trigger dataset selection again:
  $('[data-target="#dash-viz-tab"]').trigger('show.bs.tab.load-data');

  // loading indicator is shown
  assert.equal($('.dash-settings-panel #visuals-loading').is(':visible'), true);

  // simulate success:
  const callback2 = arc.staticfetcher.fetch.args[1][1];
  callback2(null, [
    { updated: '2023.11.13' },
    { updated: '2023.11.14' },
    { updated: '2023.11.15' },
  ]);

  // loading indicator is hidden
  assert.equal(
    $('.dash-settings-panel #visuals-loading').is(':visible'),
    false,
  );
});
