'use strict';

window.arc = window.arc || {};
arc.testutil = arc.testutil || {};
arc.testutil.ds = arc.testutil.ds || {};

arc.testutil.ds.create_dataset = create_dataset;
arc.testutil.ds.create_table = create_table;
arc.testutil.ds.patch_dataset_init_from_fetch = patch_dataset_init_from_fetch;
arc.testutil.ds.mock_arcl = mock_arcl;

arc.testutil.createMock = createMock;
arc.testutil.patch = patchWithMock;

function createMock() {
  const mock = function (...args) {
    mock.args.push(args);
    return mock.returnValue;
  };

  mock.args = [];
  mock.returnValue = null;
  return mock;
}

function patchWithMock(owner, attributeName) {
  const mock = createMock();
  mock.oldAtt = owner[attributeName];
  mock.restore = function () {
    owner[attributeName] = mock.oldAtt;
  };

  return mock;
}

function mock_arcl() {
  window.arcl = window.arcl || {};
  window.arcl.attributes_set_column_hidden =
    window.arcl.attributes_set_column_hidden || function () {};

  window.arcl.attributes_set_editmode =
    window.arcl.attributes_set_editmode || function () {};
}

function patch_dataset_init_from_fetch(dataset) {
  const oDataset = arc.datasets.Dataset;

  arc.datasets.Dataset = function () {
    const od = oDataset();
    od.init_from_fetch = function () {
      return dataset;
    };

    return od;
  };
}

function create_dataset() {
  const dataset = arc.datasets
    .Dataset()
    .dataset_name('testds')
    .dataset_type('multitable');

  const table_a = create_table(dataset, 'a');
  const table_b = create_table(dataset, 'b');

  table_a.columns([
    create_simple_colum(table_a, 'name', 'Name A'),
    create_computed_colum(table_a, '[Name A] || "suffixed"', 'Name A suffixed'),
  ]);

  table_b.columns([
    create_simple_colum(table_b, 'name', 'Name B'),
    create_computed_colum(table_b, '[Name B] || "suffixed"', 'Name B suffixed'),
  ]);

  dataset.tables([table_a, table_b]);
  return dataset;
}

function create_table(dataset, table_name) {
  const table = arc.datasets
    .Table()
    .dataset(dataset)
    .tablename(`testdb.${table_name}`);
  return table;
}

function create_simple_colum(table, column_name, column_alias) {
  const column = arc.datasets
    .Column()
    .table(table)
    .attr('isdim', true)
    .attr('name', column_name)
    .attr('alias', column_alias);
  return column;
}

function create_computed_colum(table, column_expr, column_alias) {
  const column = arc.datasets
    .Column()
    .table(table)
    .attr('isdim', true)
    .attr('expr', column_expr)
    .attr('alias', column_alias);
  return column;
}
