// eslint-disable-next-line no-use-before-define, no-var
QUnit.module('arc_dataconnection_types', {});

QUnit.test('Dataconnection extensions', () => {
    const dc1 = {
        name: 'dc1',
        conf: {
            base: 'base',
            overrides: {
                default_port: 123,
                timestampRangeExprTemplate: '%HTML_colname% between timestamp %HTML_quotedStart% and timestamp %HTML_quotedEnd%'
            },
            has_advanced_settings: true
        }
    };

    const dc2 = {
        name: 'dc2',
        conf: {
            base: 'postgres',
            has_concurrency_settings: true
        }
    };

    const dc3 = {
        name: 'dc3',
        conf: {
            base: 'postgres',
            overrides: {
                timestampRangeExprTemplate: '%HTML_colname% between TO_TIMESTAMP(%HTML_quotedStart%) and TO_TIMESTAMP(%HTML_quotedEnd%)'
            }
        }
    };

    arc.dataconnection.add_extension(JSON.stringify(dc1));
    arc.dataconnection.add_extension(JSON.stringify(dc2));
    arc.dataconnection.add_extension(JSON.stringify(dc3));

    QUnit.assert.deepEqual(arc.dataconnection.conns_with_advanced_settings(), ['dc1']);
    QUnit.assert.deepEqual(arc.dataconnection.conns_with_concurrency_settings(), ['dc2']);

    const pg_type = arc.dataconnection.types.postgres();
    const dc1_type = arc.dataconnection.types.dc1();
    const dc2_type = arc.dataconnection.types.dc2();
    const dc3_type = arc.dataconnection.types.dc3();

    QUnit.assert.equal(dc1_type.connection_type, 'dc1');
    QUnit.assert.equal(dc1_type.default_port, 123);
    QUnit.assert.equal(dc1_type.timestampRangeExpr('trino_ts', 'timestamp', '"2025"', '"2026"'), 'trino_ts between timestamp "2025" and timestamp "2026"');

    QUnit.assert.equal(dc2_type.default_port, 5432, 'postgres default port is inherited');

    QUnit.assert.equal(dc3_type.timestampRangeExpr('phoenix_ts', 'timestamp', '"2025"', '"2026"'), 'phoenix_ts between TO_TIMESTAMP("2025") and TO_TIMESTAMP("2026")');

    QUnit.assert.equal(pg_type.timestampRangeExpr('pg_ts', 'timestamp', '"2025"', '"2026"'), 'pg_ts between "2025" and "2026"');
});
