QUnit.module('framework_arc_data', {
  beforeEach() {
    arc.globals_init().username('joe');

    window.arcl = {};
    arc.apps_data.initialize_data();

    arcl.dataconnections = [];
  },
});

const testDataConnections = [
  {
    id: 12,
    name: 'twelve',
    cmlId: null,
  },
  {
    id: 13,
    name: 'thirteen',
    cmlId: null,
  },
  {
    id: 14,
    name: 'fourteen',
    cmlId: null,
  },
];

QUnit.test('cookies', (assert) => {
  arc.frontend.vizCookie.removeCookie('dbactive.dc.14.joe', {
    path: '/arc/qunit',
  });
  arc.frontend.vizCookie.removeCookie('joe_dataconnection_active', {
    path: '/arc/qunit',
  });
  // By default, if no connections exist, return 0
  assert.equal(arc.apps_data.active_dataconnection_id(), 0);
  arcl.dataconnections = testDataConnections;
  assert.equal(
    arc.apps_data.cookie_dc_default_database(12),
    'dbactive.dc.12.joe',
  );
  assert.equal(
    arc.apps_data.cookie_active_data_connection_id(),
    'joe_dataconnection_active',
  );

  assert.equal(arc.apps_data.active_dataconnection_id(), 0);

  arc.apps_data.set_active_dataconnection(arcl.dataconnections[0]);
  assert.equal(arc.apps_data.active_dataconnection_id(), 12);
  arc.apps_data.set_active_dataconnection(arcl.dataconnections[1]);
  assert.equal(arc.apps_data.active_dataconnection_id(), 13);
  arc.apps_data.set_active_dataconnection(arcl.dataconnections[2]);
  assert.equal(arc.apps_data.active_dataconnection_id(), 14);

  assert.equal(arc.apps_data.cookie_get_defaultdb(), undefined);

  assert.equal(arc.apps_data.cookie_set_defaultdb('prod'));
  assert.equal(arc.apps_data.cookie_get_defaultdb(), 'prod');

  assert.equal(arc.apps_data.cookie_set_defaultdb('dev'));
  assert.equal(arc.apps_data.cookie_get_defaultdb(), 'dev');
});

QUnit.test('dataconnections', (assert) => {
  arcl.dataconnections = testDataConnections;
  assert.equal($('#dataconnections-list li').length, 0);
  arc.apps_data.draw_dataconnections();

  const $lis = $('#dataconnections-list li');
  assert.equal($lis.length, 4);
  $lis.each((i, e) => {
    if (i === 0) {
      assert.equal($(e).text(), 'All Connections');
      return;
    }

    assert.equal($(e).text(), arcl.dataconnections[i - 1].name);
  });
});
