QUnit.module('framework_arc_dateranges', {});

QUnit.test('ranges', (assert) => {
  const dateranges = arc.grammar._dateranges;

  let ranges = [
    'Random, TODAY SUBTRACT 1 WEEK SUBTRACT 1 DAY STARTOF WEEK ADD 2 DAYS, TODAY SUBTRACT 1 WEEK ENDOF WEEK',
    'Last Week, TODAY SUBTRACT 1 WEEK STARTOF WEEK, TODAY SUBTRACT 1 WEEK ENDOF WEEK',
    'Last 2 Weeks, TODAY SUBTRACT 2 WEEK STARTOF WEEK, TODAY SUBTRACT 1 WEEK ENDOF WEEK',
    'New Day Anchor, 2014-02-13 SUBTRACT 1 WEEK STARTOF WEEK, TODAY SUBTRACT 1 WEEK ENDOF WEEK',
    'Now Anchor, NOW SUBTRACT 36 MINUTES, NOW',
    'New Time Anchor, 2014:02:13 08:30:30 SUBTRACT 8 HOURS, NOW',
  ].join('\n');

  let e = dateranges.parse(ranges);
  assert.equal(e.length, 6);
  assert.equal(e[0].length, 3);
  assert.equal(e[1].length, 3);
  assert.equal(e[2].length, 3);
  assert.equal(e[3].length, 3);
  assert.equal(e[4].length, 3);
  assert.equal(e[5].length, 3);

  assert.equal(e[0][0], 'Random');
  assert.equal(
    e[0][1],
    "moment().utc().subtract(1, 'week').subtract(1, 'days').startOf('week').add(2, 'days').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[0][2],
    "moment().utc().subtract(1, 'week').endOf('week').format('YYYY-MM-DD')",
  );

  assert.equal(e[1][0], 'Last Week');
  assert.equal(
    e[1][1],
    "moment().utc().subtract(1, 'week').startOf('week').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[1][2],
    "moment().utc().subtract(1, 'week').endOf('week').format('YYYY-MM-DD')",
  );

  assert.equal(e[2][0], 'Last 2 Weeks');
  assert.equal(
    e[2][1],
    "moment().utc().subtract(2, 'week').startOf('week').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[2][2],
    "moment().utc().subtract(1, 'week').endOf('week').format('YYYY-MM-DD')",
  );

  assert.equal(e[3][0], 'New Day Anchor');
  assert.equal(
    e[3][1],
    "moment('2014-02-13').subtract(1, 'week').startOf('week').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[3][2],
    "moment().utc().subtract(1, 'week').endOf('week').format('YYYY-MM-DD')",
  );

  assert.equal(e[4][0], 'Now Anchor');
  assert.equal(
    e[4][1],
    "moment().utc().subtract(36, 'minute').format('YYYY-MM-DD HH:mm:ss')",
  );
  assert.equal(e[4][2], "moment().utc().format('YYYY-MM-DD HH:mm:ss')");

  assert.equal(e[5][0], 'New Time Anchor');
  assert.equal(
    e[5][1],
    "moment('2014-02-13 08:30:30').subtract(8, 'hour').format('YYYY-MM-DD HH:mm:ss')",
  );
  assert.equal(e[5][2], "moment().utc().format('YYYY-MM-DD HH:mm:ss')");

  ranges = 'Today, TODAY, TODAY';
  e = dateranges.parse(ranges);
  assert.equal(e.length, 1);
  assert.equal(e[0].length, 3);
  assert.equal(e[0][0], 'Today');
  assert.equal(e[0][1], "moment().utc().format('YYYY-MM-DD')");
  assert.equal(e[0][2], "moment().utc().format('YYYY-MM-DD')");

  // WEEKS vs WEEK
  ranges =
    'Last Week, TODAY SUBTRACT 1 WEEKS STARTOF WEEKS, TODAY SUBTRACT 1 WEEKS ENDOF WEEKS';
  e = dateranges.parse(ranges);
  assert.equal(e.length, 1);
  assert.equal(e[0].length, 3);
  assert.equal(e[0][0], 'Last Week');
  assert.equal(
    e[0][1],
    "moment().utc().subtract(1, 'week').startOf('week').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[0][2],
    "moment().utc().subtract(1, 'week').endOf('week').format('YYYY-MM-DD')",
  );

  // MONTH
  ranges =
    'Last Month, TODAY SUBTRACT 1 MONTHS STARTOF MONTH, TODAY SUBTRACT 1 MONTHS ENDOF MONTH';
  e = dateranges.parse(ranges);
  assert.equal(e.length, 1);
  assert.equal(e[0].length, 3);
  assert.equal(e[0][0], 'Last Month');
  assert.equal(
    e[0][1],
    "moment().utc().subtract(1, 'month').startOf('month').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[0][2],
    "moment().utc().subtract(1, 'month').endOf('month').format('YYYY-MM-DD')",
  );

  ranges =
    'Last 2 Quarters, TODAY SUBTRACT 2 QUARTERS STARTOF QUARTER, TODAY SUBTRACT 1 QUARTER ENDOF QUARTER';
  e = dateranges.parse(ranges);
  assert.equal(e.length, 1);
  assert.equal(e[0].length, 3);
  assert.equal(e[0][0], 'Last 2 Quarters');
  assert.equal(
    e[0][1],
    "moment().utc().subtract(2, 'quarter').startOf('quarter').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[0][2],
    "moment().utc().subtract(1, 'quarter').endOf('quarter').format('YYYY-MM-DD')",
  );
  ranges =
    'Last 2 Quarters, TODAY SUBTRACT 2 QUARTERS STARTOF QUARTER, TODAY SUBTRACT 1 QUARTER ENDOF QUARTER';

  ranges =
    'Last 2 Years, TODAY SUBTRACT 2 YEARS STARTOF YEAR, TODAY SUBTRACT 1 YEAR ENDOF YEAR';
  e = dateranges.parse(ranges);
  assert.equal(e.length, 1);
  assert.equal(e[0].length, 3);
  assert.equal(e[0][0], 'Last 2 Years');
  assert.equal(
    e[0][1],
    "moment().utc().subtract(2, 'year').startOf('year').format('YYYY-MM-DD')",
  );
  assert.equal(
    e[0][2],
    "moment().utc().subtract(1, 'year').endOf('year').format('YYYY-MM-DD')",
  );
});
