// Unit Tests for Derived Data
QUnit.module('framework_arc_deriveddata', {
  beforeEach() {
    arcl = {};
    // Indicate modal is open
    arc.report.superModal = {};
    arc.report.superModal.isOpen = false;

    arc.globals_init().username('joe');

    arc.testutil = arc.testutil || {};
    arc.testutil.derived_data = true;

    arc.globals.use_derived_data = true;
    arc.globals.params = arc.params.tracker();

    arc.globals.is_edit = true;
    // Track all reports associated with this page
    arc.globals.reports = arc.reporttracker.reports().build_mode('build');

    const table = arc.datasets.Table().tablename('default.census_pop');

    const column1 = arc.datasets.Column().attr('name', 'state').table(table);

    const column2 = arc.datasets
      .Column()
      .attr('name', 'population')
      .table(table);

    this.dataset = arc.datasets
      .Dataset()
      .dataset_id(1)
      .dataconnection_type('arcengine')
      .dataset_name('census');

    this.dataset.add_table(table);
    this.dataset.add_column(column1);
    this.dataset.add_column(column2);

    this.report = arc.report.getPrimaryReport();
    this.report.dataset(this.dataset);

    arc.globals.singleVisualBuilder = arc.reportbuilder();

    arc.globals.singleVisualBuilder.setupAndDrawReport();

    const pie = arc.visual.plugins.pie();

    const driver = arc.report.driver(pie).dataset(this.dataset);

    driver.derivedData().setupDerivedData();

    driver.chart_div('chart-div');

    pie.viz_drawDesigner('app-shelves');

    this.shelves = pie.viz_lib().shelves();
    this.shelves.shelves_dom_id('app-shelves');

    this.report.viz(driver);

    arc.report.shelf.setup();
  },
});

function showDerivedData(state_dim) {
  // bring up modal
  const d = d3.selectAll($(state_dim).find('.selection_field')).datum();
  const deriveddata = arc.report.getPrimaryViz().derivedData();
  deriveddata.shelfElement(d);
  deriveddata.show_hide(true);

  // Indicate modal is open
  arc.report.superModal.isOpen = true;
}

QUnit.test('simple dimension and measure', function (assert) {
  const shelves = jQuery.extend({}, this.shelves);

  // add state column to dimension shelf
  shelves.add('dimensions_shelf', [
    { dataset_colname: 'state', dataset_coltype: 'STRING' },
  ]);

  // save state column (no other columns) as derived data
  const state_dim = $('#dimensions_shelf')[0];

  showDerivedData(state_dim);

  const form = $('#edit-deriveddata-form');
  form.find('.form-control').val('foo');
  form.find('input[value=viz-derived-data]').trigger('click');
  form.trigger('submit');
  // test whether derived data "foo" is in the Dimensions fields section
  const field_dims = $('#visual-data-panel .dim-app-params');
  const foo_element = $(field_dims).find('.selection_element')[0];
  assert.equal($(foo_element).find('.selection_field')[0].title, 'foo');
  function getDerivedData() {
    return arc.report.getPrimaryViz().derivedData();
  }

  // test whether derived data "foo" is in the derived data builder list
  const foo_derived = getDerivedData().deriveddata_list().deriveddata_list[0];

  assert.equal(foo_derived.label, 'foo');
  assert.equal(foo_derived.isMeasure, false);
  assert.equal(
    JSON.stringify(foo_derived.dsreq),
    '{"version":1,"highlighting":false,"type":"SQL","time":[],"session":[],' +
      '"dimensions":[{"type":"SIMPLE","expr":"[state] as \'foo\'"}],' +
      '"aggregates":[],"partitions":[],"filters":[],"outerFilters":[],"having":[],"global_segments":[],"events":[],' +
      '"distinct":false,"derived_data":[],"solr_query":[],"autotime_override":"","ksqlStreamPosition":"","dataset_id":1}',
  );

  // remove derived data "foo"
  $(foo_element).find('.del-view-popover').trigger('click');
  // test whether derived data foo is removed from both field elements and builder list
  assert.equal(
    $(field_dims).find('.selection_element .selection_field')[0].title,
    'state',
  );
  assert.equal(
    getDerivedData().deriveddata_list().deriveddata_list[0],
    undefined,
  );

  // add sum of population to aggregates shelf
  shelves.add('aggregates_shelf', [
    {
      dataset_colname: 'population',
      fn_name: 'sum',
      dataset_coltype: 'BIGINT',
    },
  ]);

  // save sum of population per state
  const pop_agg = $('#aggregates_shelf')[0];

  showDerivedData(pop_agg);

  form.find('.form-control').val('bar');
  form.find('input[value=viz-derived-data]').trigger('click');
  form.trigger('submit');

  // test whether derived data "bar" is in the Measures fields section
  const field_aggs = $('#visual-data-panel .agg-app-params');
  const bar_element = $(field_aggs).find('.selection_element')[0];
  assert.equal($(bar_element).find('.selection_field')[0].title, 'bar');

  // test whether derived data "bar" is in the derived data builder list
  const bar_derived = getDerivedData().deriveddata_list().deriveddata_list[1];

  assert.equal(bar_derived.label, 'bar');
  assert.equal(bar_derived.isMeasure, true);
  assert.equal(
    JSON.stringify(bar_derived.dsreq),
    '{"version":1,"highlighting":false,"type":"SQL","time":[],"session":[],' +
      '"dimensions":[{"type":"SIMPLE","expr":"[state] as \'_state\'"}],' +
      '"aggregates":[{"expr":"sum([population]) as \'bar\'"}],' +
      '"partitions":[],' +
      '"filters":[],"outerFilters":[],"having":[],"global_segments":[],"events":[],' +
      '"distinct":false,"derived_data":[],"solr_query":[],"autotime_override":"","ksqlStreamPosition":"","dataset_id":1}',
  );

  // remove derived data "bar"
  const delete_bar = $(bar_element).find('.del-view-popover')[0];
  $(delete_bar).trigger('click');

  // test whether derived data bar is removed from both field elements and builder list
  assert.equal(
    $(field_aggs).find('.selection_element .selection_field')[0].title,
    '',
  );
  assert.equal(
    getDerivedData().deriveddata_list().deriveddata_list[1],
    undefined,
  );
});
