QUnit.module('framework_arc_queue', {
  beforeEach() {
    window.arcl = {};
  },

  afterEach() {
    arc.testutil.queue = undefined;
  },
});

QUnit.test('multistage', (assert) => {
  const cb = arc.staticfetcher.multistageCallback();

  let called = 0;
  cb.total(5);

  cb.callback(() => {
    called += 1;
  });

  cb.decrement();
  cb.decrement();
  cb.decrement();
  cb.decrement();

  assert.equal(called, 0);
  cb.decrement();

  assert.equal(called, 1);
});

QUnit.test('cb', (assert) => {
  let _uniqueId = 1;

  let totalCalls = 0;
  // Setup an async function for testing
  arc.testutil.queue = function () {
    const q = function () {};

    let cb;
    let asset;
    const uniqueId = _uniqueId;
    _uniqueId += 1;

    q.defer = function (_processor, _asset) {
      totalCalls += 1;
      asset = _asset;
      return q;
    };

    q.await = function (_cb) {
      cb = _cb;
      setTimeout(() => {
        cb(undefined, ['data', asset, uniqueId]);
      }, 100);
      return q;
    };

    return q;
  };

  const asset1 = {
    asset: 'asset1',
    postprocess() {},
  };
  const asset2 = {
    asset: 'asset2',
    postprocess() {},
  };
  const asset3 = {
    asset: 'asset3',
    postprocess() {},
  };
  const asset4 = {
    asset: 'asset3',
    postprocess() {},
  };

  let restartQUnit = assert.async();
  let called = false;
  assert.equal(totalCalls, 0);
  arc.staticfetcher.fetch([asset1, asset2], (error, data1, data2) => {
    restartQUnit();
    assert.equal(error, undefined);
    assert.equal(JSON.stringify(['data', 'asset1', 1]), JSON.stringify(data1));
    assert.equal(JSON.stringify(['data', 'asset2', 2]), JSON.stringify(data2));
    assert.equal(called, false);
    // Confirm that async requests were made
    assert.equal(totalCalls, 2);

    called = true;

    // Confirm that the next request for assets will be synchronous
    let syncCall = false;
    arc.staticfetcher.fetch([asset1, asset2], (_, secondData1, secondData2) => {
      assert.equal(
        JSON.stringify(['data', 'asset1', 1]),
        JSON.stringify(secondData1),
      );
      assert.equal(
        JSON.stringify(['data', 'asset2', 2]),
        JSON.stringify(secondData2),
      );
      // Confirm that no additional async requests were made
      assert.equal(totalCalls, 2);
      syncCall = true;
    });
    assert.equal(syncCall, true);

    restartQUnit = assert.async();
    arc.staticfetcher.fetch(
      [asset1, asset3, asset2],
      (_, thirdData1, data3, thirdData2) => {
        restartQUnit();
        // Confirm that one additional async request was made
        assert.equal(totalCalls, 3);
        assert.equal(
          JSON.stringify(['data', 'asset1', 1]),
          JSON.stringify(thirdData1),
        );
        assert.equal(
          JSON.stringify(['data', 'asset3', 3]),
          JSON.stringify(data3),
        );
        assert.equal(
          JSON.stringify(['data', 'asset2', 2]),
          JSON.stringify(thirdData2),
        );

        // Test for multiple callbacks that each one is getting called
        restartQUnit = assert.async();
        let callback1called = false;
        function callback1() {
          callback1called = true;
        }

        function callback2() {
          restartQUnit();
          assert.equal(callback1called, true);
        }

        arc.staticfetcher.fetch([asset4], callback1);
        arc.staticfetcher.fetch([asset4], callback2);
      },
    );
  });
});
