(function () {
  arc.test = arc.test || {};
  arc.data = arc.data || {};

  function Dataset() {
    const d = function () {
      return 'Dataset';
    };

    d.searchSettings = function () {
      return {};
    };

    d.dataset_id = function () {
      return 1;
    };

    d.dataconnection_type = function () {
      return 'arcengine';
    };

    d.defaultAgg = function () {
      return 'population';
    };

    d.defaultDateTime = function () {
      return 'datestring';
    };

    d.defaultDateTimeFormat = function () {
      return 'YYYY-MM-DD';
    };

    d.findColumn = function (colname) {
      if (colname === 'population') {
        return arc.test.populationColumn;
      }

      if (colname === 'datestring') {
        return arc.test.datestringColumn;
      }

      return undefined;
    };

    return d;
  }

  const testDataset = Dataset();

  function table() {
    const t = function () {
      return 'table';
    };

    t.dataset = function () {
      return testDataset;
    };

    return t;
  }

  QUnit.module('framework_arc_searchfilters', {
    beforeEach() {
      this.runQuery = arc.data.runQuery;
      arc.data.runQuery = function (rb, success, fail) {
        const filters = rb.filters();

        if (filters.length !== 1) {
          fail();
          return;
        }

        if (filters[0] === "(lower([state]) like '%ca%')") {
          success({
            rows: ['CA'],
          });

          return;
        }

        if (filters[0] === "(lower([state]) like 'c%')") {
          success({
            rows: ['CA'],
          });

          return;
        }

        if (filters[0] === "(lower([state]) like '%a')") {
          success({
            rows: ['CA'],
          });

          return;
        }

        if (filters[0] === "(lower([state]) like '%wa%')") {
          success({
            rows: ['WA'],
          });

          return;
        }

        if (filters[0] === '(([year] = 2000))') {
          success({
            rows: ['2000'],
          });

          return;
        }

        if (filters[0] === '(([year] = 2010))') {
          success({
            rows: ['2010'],
          });

          return;
        }

        fail();
      };

      this.handler = arc.search.searchCompiler();

      const colTable = table();

      this.datestring = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'datestring')
        .attr('type', 'STRING')
        .attr('isdim', true)
        .attr('searchable', false);

      this.state = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'state')
        .attr('type', 'STRING')
        .attr('searchable', true)
        .attr('terms', 'state "big area"')
        .attr('isdim', true);

      this.year = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'year')
        .attr('type', 'INTEGER')
        .attr('isdim', true)
        .attr('searchable', true);

      this.population = arc.datasets
        .Column()
        .table(colTable)
        .attr('type', 'INTEGER')
        .attr('name', 'population')
        .attr('terms', 'population people "number of folks"');

      arc.test.populationColumn = this.population;
      arc.test.datestringColumn = this.datestring;

      const _this = this;

      const dataset = Dataset();
      dataset.columnIterator = function (cb) {
        cb(_this.datestring);
        cb(_this.state);
        cb(_this.year);
        cb(_this.population);
      };

      this.handler.dataset(dataset).initialize();
    },

    afterEach() {
      arc.data.runQuery = this.runQuery;
    },

    elementsToData(elements) {
      const data = {
        dimensions: [],
        aggregates: [],
        filters: [],
        unknowns: [],
      };

      elements.forEach((e) => {
        if (e.type === 'aggregate') {
          let { expr } = e;
          if (e.order && e.order.limit) {
            expr += `${e.order.ascending ? ' asc ' : ' desc '}limit ${
              e.order.limit
            }`;
          } else if (e.order && e.order.ascending) {
            expr += e.order.ascending ? ' asc' : ' desc';
          }

          data.aggregates.push(expr);
          return;
        }

        if (e.type === 'dimension') {
          data.dimensions.push(e.expr);
          return;
        }

        if (e.type === 'filter') {
          data.filters.push(e.expr);
          return;
        }

        if (e.type === 'unknown') {
          data.unknowns.push(e.phrase);
        }
      });

      return data;
    },

    highlightsToData(highlights) {
      const data = [];

      highlights.forEach((h) => {
        data.push({
          startIndex: h.startIndex,
          endIndex: h.endIndex,
          type: h.type,
          phrase: h.element ? h.element.phrase : undefined,
        });
      });

      return data;
    },

    verifyHighlights(assert, str, expectedResult) {
      let elements;

      this.handler.processSearchString(str, {
        finalCb(config) {
          elements = config.elements;
        },

        validateFilters: true,
      });

      const highlights = arc.testutil.elementsToHighlights(elements);
      const data = this.highlightsToData(highlights);
      assert.deepEqual(data, expectedResult);
    },

    verifySearchResults(assert, str, expectedResult) {
      let elements;

      this.handler.processSearchString(str, {
        finalCb(config) {
          elements = config.elements;
        },

        validateFilters: true,
      });

      const data = this.elementsToData(elements);
      assert.deepEqual(data, expectedResult);
    },

    verifyStateDimAndFilter(assert, phrase, filter) {
      return this.verifySearchResults(assert, phrase, {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: [],
        filters: [filter],
        unknowns: [],
      });
    },

    verifyFilter(assert, phrase, filter, agg) {
      const column = filter.match(/\(\[.*?\]\)/)[0];
      return this.verifySearchResults(assert, phrase, {
        aggregates: [`${agg + column} desc limit 5000`],
        dimensions: [],
        filters: [filter],
        unknowns: [],
      });
    },
  });

  QUnit.test('simple', function (assert) {
    let elements;

    this.handler.processSearchString('population is 62', {
      finalCb(config) {
        elements = config.elements;
      },
    });

    assert.equal(elements.length, 2);

    assert.equal(elements[0].expr, '(([population]) = 62)');
    assert.equal(elements[0].source, 'compare');
    assert.equal(elements[0].type, 'filter');
    assert.equal(elements[0].subtype, 'fixed');

    assert.equal(elements[1].expr, 'SUM([population])');
    assert.equal(elements[1].source, 'Track first aggregate');
    assert.equal(elements[1].type, 'aggregate');
  });

  QUnit.test('comparison errors', function (assert) {
    this.verifySearchResults(assert, 'state more than CA', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ["(lower([state]) like '%ca%')"],
      unknowns: ['more', 'than'],
    });
  });

  QUnit.test('comparisons', function (assert) {
    this.verifyStateDimAndFilter(
      assert,
      'state is CA',
      "(lower([state]) in ('ca'))",
    );

    this.verifyStateDimAndFilter(
      assert,
      'state is CA or AZ',
      "(lower([state]) in ('ca', 'az'))",
    );

    this.verifyStateDimAndFilter(
      assert,
      'state is CA, AZ',
      "(lower([state]) in ('ca', 'az'))",
    );

    this.verifyStateDimAndFilter(
      assert,
      'state is CA,AZ',
      "(lower([state]) in ('ca', 'az'))",
    );

    this.verifyStateDimAndFilter(
      assert,
      'state is CA and AZ',
      "(lower([state]) in ('ca', 'az'))",
    );

    this.verifyStateDimAndFilter(
      assert,
      'state is CA AZ',
      "(lower([state]) in ('ca', 'az'))",
    );

    this.verifyFilter(
      assert,
      'population more than 1000000',
      '(([population]) > 1000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 1k',
      '(([population]) > 1000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 5K',
      '(([population]) > 5000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 3m',
      '(([population]) > 3000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 4M',
      '(([population]) > 4000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 7b',
      '(([population]) > 7000000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 6B',
      '(([population]) > 6000000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 8T',
      '(([population]) > 8000000000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population more than 9t',
      '(([population]) > 9000000000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population is more than 300',
      '(([population]) > 300)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'populations are more than 42',
      '(([population]) > 42)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population not more than 9819',
      '(([population]) <= 9819)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population less than 62626',
      '(([population]) < 62626)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population not less than 1000000',
      '(([population]) >= 1000000)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population < 62626',
      '(([population]) < 62626)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population <= 62626',
      '(([population]) <= 62626)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population > 62626',
      '(([population]) > 62626)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population >= 62626',
      '(([population]) >= 62626)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population is equal to 62626',
      '(([population]) = 62626)',
      'SUM',
    );

    this.verifyFilter(
      assert,
      'population is 62626',
      '(([population]) = 62626)',
      'SUM',
    );
  });
}());
