arc.data = arc.data || {};

(function () {
  arc.test = arc.test || {};

  function Dataset() {
    const d = function () {
      return 'Dataset';
    };

    d.searchSettings = function () {
      return {
        expansions: [
          {
            key: 'USA',
            value: '',
          },
          {
            key: '80s',
            value: 'year is 1970, 1980',
          },
          {
            key: 'the 90s',
            value: 'year is 1980, 1990',
          },
        ],
      };
    };

    d.dataset_id = function () {
      return 1;
    };

    d.dataconnection_type = function () {
      return 'arcengine';
    };

    d.defaultAgg = function () {
      return 'population';
    };

    d.defaultDateTime = function () {
      return 'datestring';
    };

    d.defaultDateTimeFormat = function () {
      return 'YYYY-MM-DD';
    };

    d.findColumn = function (colname) {
      if (colname === 'population') {
        return arc.test.populationColumn;
      }

      if (colname === 'datestring') {
        return arc.test.datestringColumn;
      }

      if (colname === 'datestring2') {
        return arc.test.datestring2Column;
      }

      if (colname === 'datestring3') {
        return arc.test.datestring3Column;
      }

      return undefined;
    };

    return d;
  }

  function table() {
    const t = function () {
      return 'table';
    };

    t.dataset = function () {
      return Dataset();
    };

    return t;
  }

  QUnit.module('framework_arc_searchhandler', {
    beforeEach() {
      this.runQuery = arc.data.runQuery;
      arc.data.runQuery = function (rb, success, fail) {
        const filters = rb.filters();

        if (filters.length !== 1) {
          fail();
          return;
        }

        if (filters[0] === "(lower([state]) like '%ca%')") {
          success({
            rows: ['CA'],
          });

          return;
        }

        if (filters[0] === "(lower([state]) like 'c%')") {
          success({
            rows: ['CA'],
          });

          return;
        }

        if (filters[0] === "(lower([state]) like '%a')") {
          success({
            rows: ['CA'],
          });

          return;
        }

        if (filters[0] === "(lower([state]) like '%wa%')") {
          success({
            rows: ['WA'],
          });

          return;
        }

        if (filters[0] === '(([year] = 2000))') {
          success({
            rows: ['2000'],
          });

          return;
        }

        if (filters[0] === '(([year] = 2010))') {
          success({
            rows: ['2010'],
          });

          return;
        }

        fail();
      };

      this.handler = arc.search.searchCompiler();

      const colTable = table();
      this.colTable = colTable;

      this.datestring = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'datestring')
        .attr('type', 'STRING')
        .attr('isdim', true)
        .attr('searchable', false);

      this.datestring2 = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'datestring2')
        .attr('type', 'TIMESTAMP')
        .attr('isdim', true)
        .attr('terms', 'datestring2 closed')
        .attr('searchable', false);

      this.datestring3 = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'datestring3')
        .attr('type', 'TIMESTAMP')
        .attr('isdim', true)
        .attr('terms', 'datestring3')
        .attr('searchable', false);

      this.state = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'state')
        .attr('type', 'STRING')
        .attr('searchable', true)
        .attr('terms', 'state "big area"')
        .attr('isdim', true);

      this.county = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'county')
        .attr('type', 'STRING')
        .attr('searchable', true)
        .attr('terms', 'county "state county"')
        .attr('isdim', true);

      this.year = arc.datasets
        .Column()
        .table(colTable)
        .attr('name', 'year')
        .attr('type', 'INTEGER')
        .attr('isdim', true)
        .attr('searchable', true);

      this.population = arc.datasets
        .Column()
        .table(colTable)
        .attr('type', 'INTEGER')
        .attr('name', 'population')
        .attr('terms', 'population people "num folks"');

      this.boolean = arc.datasets
        .Column()
        .table(colTable)
        .attr('type', 'BOOLEAN')
        .attr('name', 'boolean')
        .attr('isdim', true);

      this.boolean2 = arc.datasets
        .Column()
        .table(colTable)
        .attr('type', 'BOOL')
        .attr('name', 'boolean2');

      arc.test.populationColumn = this.population;
      arc.test.datestringColumn = this.datestring;
      arc.test.datestring2Column = this.datestring2;
      arc.test.datestring3Column = this.datestring3;

      const _this = this;
      const dataset = Dataset();
      dataset.columnIterator = function (cb) {
        cb(_this.datestring);
        cb(_this.datestring2);
        cb(_this.datestring3);
        cb(_this.state);
        cb(_this.county);
        cb(_this.year);
        cb(_this.population);
        cb(_this.boolean);
        cb(_this.boolean2);
      };

      this.handler.dataset(dataset).initialize();
    },

    afterEach() {
      arc.data.runQuery = this.runQuery;
    },

    elementsToData(elements) {
      const data = {
        dimensions: [],
        aggregates: [],
        filters: [],
        unknowns: [],
      };

      elements.forEach((e) => {
        let expr;
        if (e.type === 'aggregate' || e.type === 'dimension') {
          expr = e.expr;
          if (e.order && e.order.limit) {
            expr += `${e.order.ascending ? ' asc ' : ' desc '}limit ${e.order.limit}`;
          } else if (e.order && e.order.ascending) {
            expr += e.order.ascending ? ' asc' : ' desc';
          }

          if (e.type === 'aggregate') {
            data.aggregates.push(expr);
          } else {
            data.dimensions.push(expr);
          }

          return;
        }

        if (e.type === 'filter') {
          data.filters.push(e.expr);
          return;
        }

        if (e.type === 'unknown') {
          data.unknowns.push(e.phrase);
        }
      });

      return data;
    },

    highlightsToData(highlights) {
      const data = [];

      highlights.forEach((h) => {
        data.push({
          startIndex: h.startIndex,
          endIndex: h.endIndex,
          type: h.type,
          phrase: h.element ? h.element.phrase : undefined,
        });
      });

      return data;
    },

    verifyHighlights(assert, str, expectedResult) {
      let elements;

      this.handler.processSearchString(str, {
        finalCb(config) {
          elements = config.elements;
        },

        validateFilters: true,
      });

      const highlights = arc.testutil.elementsToHighlights(elements);
      const data = this.highlightsToData(highlights);
      assert.deepEqual(data, expectedResult);
    },

    verifySearchResults(assert, str, expectedResult) {
      let config;

      this.handler.processSearchString(str, {
        finalCb(_config) {
          config = _config;
        },

        validateFilters: true,
      });

      const data = this.elementsToData(config.elements);
      if (config.showmeVisualConfig.vizType) {
        data.showmeVisualConfig = {
          vizType: config.showmeVisualConfig.vizType,
        };
      }

      assert.deepEqual(data, expectedResult);
    },
  });

  QUnit.test('base', function (assert) {
    const s = this.handler.searchableColumns();

    assert.equal(3, s.length);

    assert.equal(s[0].column.display_name(), 'state');
    assert.equal(s[1].column.display_name(), 'county');
    assert.equal(s[2].column.display_name(), 'year');
  });

  function stripItems(elements) {
    elements.forEach((e) => {
      if (e.source) {
        delete e.source;
      }

      if (e.token) {
        delete e.token;
      }

      if (e.additionalElements) {
        delete e.additionalElements;
      }

      if (e.unknownNote) {
        delete e.unknownNote;
      }
    });
  }

  QUnit.test('simple', function (assert) {
    let elements;

    this.handler.processSearchString('state year population', {
      finalCb(config) {
        elements = config.elements;
      },
    });

    assert.equal(3, elements.length);
    stripItems(elements);

    assert.deepEqual(elements[0], {
      column: this.state,
      expr: '[state]',
      name: '[state]',
      phrase: 'state',
      type: 'dimension',
    });

    assert.deepEqual(elements[1], {
      column: this.year,
      expr: '[year]',
      name: '[year]',
      phrase: 'year',
      type: 'dimension',
    });

    assert.deepEqual(elements[2], {
      column: this.population,
      expr: 'SUM([population])',
      order: {
        ascending: false,
        limit: 5000,
      },
      aggregation: 'SUM',
      name: '[population]',
      phrase: 'population',
      type: 'aggregate',
    });
  });

  QUnit.test('simple 2', function (assert) {
    let elements;

    this.handler.processSearchString('state year population filter:year', {
      finalCb(config) {
        elements = config.elements;
      },
    });

    assert.equal(4, elements.length);
    stripItems(elements);

    assert.deepEqual(elements[0], {
      column: this.state,
      expr: '[state]',
      name: '[state]',
      phrase: 'state',
      type: 'dimension',
    });

    assert.deepEqual(elements[1], {
      column: this.year,
      expr: '[year]',
      name: '[year]',
      phrase: 'year',
      type: 'dimension',
    });

    assert.deepEqual(elements[2], {
      column: this.population,
      expr: 'SUM([population])',
      order: {
        ascending: false,
        limit: 5000,
      },
      aggregation: 'SUM',
      name: '[population]',
      phrase: 'population',
      type: 'aggregate',
    });

    assert.deepEqual(elements[3], {
      column: this.year,
      name: 'year',
      phrase: 'filter:year',
      type: 'control',
    });
  });

  QUnit.test('onedimsearch', function (assert) {
    let elements;

    this.handler.processSearchString(
      'What are the Populations by State and Year for CA and WA',
      {
        finalCb(config) {
          elements = config.elements;
        },
        validateFilters: true,
      },
    );

    assert.equal(elements.length, 4);
    stripItems(elements);

    assert.deepEqual(elements[0], {
      column: this.population,
      expr: 'SUM([population])',
      order: {
        ascending: false,
        limit: 5000,
      },
      aggregation: 'SUM',
      name: '[population]',
      phrase: 'Populations',
      type: 'aggregate',
    });

    assert.deepEqual(elements[1], {
      column: this.state,
      expr: '[state]',
      name: '[state]',
      phrase: 'State',
      type: 'dimension',
    });

    assert.deepEqual(elements[2], {
      column: this.year,
      expr: '[year]',
      name: '[year]',
      phrase: 'Year',
      type: 'dimension',
    });

    assert.deepEqual(elements[3], {
      column: this.state,
      displayExpr: "(lower([state]) like '%ca%')",
      expr: "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
      name: '[state]',
      phrase: 'CA',
      type: 'filter',
      values: ['CA', 'WA'],
      tooltip: 'Searched through [state], [county], [year]',
    });
  });

  QUnit.test('searchable', function (assert) {
    let elements;

    this.handler.processSearchString(
      'what are the populations by year for CA and WA in 2010',
      {
        finalCb(config) {
          elements = config.elements;
        },
        validateFilters: true,
      },
    );

    assert.equal(elements.length, 4);
    stripItems(elements);

    assert.deepEqual(elements[0], {
      column: this.population,
      expr: 'SUM([population])',
      order: {
        ascending: false,
        limit: 5000,
      },
      aggregation: 'SUM',
      name: '[population]',
      phrase: 'populations',
      type: 'aggregate',
    });

    assert.deepEqual(elements[1], {
      column: this.year,
      expr: '[year]',
      name: '[year]',
      phrase: 'year',
      type: 'dimension',
    });

    assert.deepEqual(elements[2], {
      column: this.state,
      displayExpr: "(lower([state]) like '%ca%')",
      expr: "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
      name: '[state]',
      phrase: 'CA',
      type: 'filter',
      values: ['CA', 'WA'],
      tooltip: 'Searched through [state], [county], [year]',
    });

    assert.deepEqual(elements[3], {
      column: this.year,
      displayExpr: '(([year] = 2010))',
      expr: '(([year] = 2010))',
      name: '[year]',
      phrase: '2010',
      type: 'filter',
      values: ['2010'],
      tooltip: 'Searched through [state], [county], [year]',
    });
  });

  QUnit.test('search_and_validate', function (assert) {
    let elements;

    /*
     * Note that this only works because we are stub'ing out arc.data.runQuery above and
     * returning information for searches like year -> 2010
     */
    this.handler.processSearchString(
      'what are the populations by year for CA and WA in 2010 and 2000',
      {
        finalCb(config) {
          elements = config.elements;
        },
        validateFilters: true,
      },
    );

    assert.equal(elements.length, 4);
    stripItems(elements);

    assert.deepEqual(elements[0], {
      column: this.population,
      expr: 'SUM([population])',
      order: {
        ascending: false,
        limit: 5000,
      },
      aggregation: 'SUM',
      name: '[population]',
      phrase: 'populations',
      type: 'aggregate',
    });

    assert.deepEqual(elements[1], {
      column: this.year,
      expr: '[year]',
      name: '[year]',
      phrase: 'year',
      type: 'dimension',
    });

    assert.deepEqual(elements[2], {
      column: elements[2].column,
      displayExpr: "(lower([state]) like '%ca%')",
      expr: "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
      name: '[state]',
      phrase: 'CA',
      type: 'filter',
      values: ['CA', 'WA'],
      tooltip: 'Searched through [state], [county], [year]',
    });

    assert.deepEqual(elements[3], {
      column: elements[3].column,
      displayExpr: '(([year] = 2010))',
      expr: '((([year] = 2010)) OR (([year] = 2000)))',
      name: '[year]',
      phrase: '2010',
      type: 'filter',
      values: ['2010', '2000'],
      tooltip: 'Searched through [state], [county], [year]',
    });
  });

  QUnit.test('search_and_validate 2', function (assert) {
    let elements;

    /*
     * Note that this only works because we are stub'ing out arc.data.runQuery above and
     * returning information for searches like year -> 2010
     *
     * Because only one state is being searched for here (CA) vs two states in the test above,
     * [state] will not be added in as a dimension
     */
    this.handler.processSearchString(
      'what are the populations by year for CA in 2010 and 2000',
      {
        finalCb(config) {
          elements = config.elements;
        },
        validateFilters: true,
      },
    );

    assert.equal(elements.length, 4);
    stripItems(elements);

    assert.deepEqual(elements[0], {
      column: this.population,
      expr: 'SUM([population])',
      order: {
        ascending: false,
        limit: 5000,
      },
      aggregation: 'SUM',
      name: '[population]',
      phrase: 'populations',
      type: 'aggregate',
    });

    assert.deepEqual(elements[1], {
      column: this.year,
      expr: '[year]',
      name: '[year]',
      phrase: 'year',
      type: 'dimension',
    });

    assert.deepEqual(elements[2], {
      column: elements[2].column,
      displayExpr: "(lower([state]) like '%ca%')",
      expr: "(lower([state]) like '%ca%')",
      name: '[state]',
      phrase: 'CA',
      type: 'filter',
      values: ['CA'],
      tooltip: 'Searched through [state], [county], [year]',
    });

    assert.deepEqual(elements[3], {
      column: elements[3].column,
      displayExpr: '(([year] = 2010))',
      expr: '((([year] = 2010)) OR (([year] = 2000)))',
      name: '[year]',
      tooltip: 'Searched through [state], [county], [year]',
      phrase: '2010',
      type: 'filter',
      values: ['2010', '2000'],
    });
  });

  QUnit.test('negative filters', function (assert) {
    this.verifySearchResults(assert, 'top 20 states excluding nulls', {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ['([state] is not NULL)'],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'top 20 states where population is false',
      {
        aggregates: ['SUM([population]) desc limit 20'],
        filters: ["(lower([state]) in ('false', '0'))"],
        dimensions: ['[state]'],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'top 20 states where boolean is true', {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ["(lower(cast([boolean] as string)) in ('true', '1'))"],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 20 states where boolean2 is false', {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ["(lower(cast([boolean2] as string)) in ('false', '0'))"],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 20 states where boolean is null', {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ['([boolean] is NULL)'],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 20 states where population is null', {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ['([population] is NULL)'],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'bottom 20 states when datestring2 starts with 2019',
      {
        aggregates: ['SUM([population]) asc limit 20'],
        filters: ["((lower(cast([datestring2] as string)) like '2019%'))"],
        dimensions: ['[state]'],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'bottom 20 states when datestring2 ends with 20',
      {
        aggregates: ['SUM([population]) asc limit 20'],
        filters: ["((lower(cast([datestring2] as string)) like '%20'))"],
        dimensions: ['[state]'],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      "bottom 20 states when datestring2 is '2019-01-01'",
      {
        aggregates: ['SUM([population]) asc limit 20'],
        filters: ["(lower(cast([datestring2] as string)) in ('2019-01-01'))"],
        dimensions: ['[state]'],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, "top 20 states excluding ''", {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ["(lower([state]) not in (''))"],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(assert, "top 20 states excluding nulls and ''", {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: [
        "((lower([state]) not in ('') AND lower([state]) is not NULL))",
      ],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 20 states excluding CA and WA', {
      aggregates: ['SUM([population]) desc limit 20'],
      filters: ["(lower([state]) not in ('ca', 'wa'))"],
      dimensions: ['[state]'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '((([year] = 2010)) OR (([year] = 2000)))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people not CA WA 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "(not ((lower([state]) like '%ca%') OR (lower([state]) like '%wa%')))",
        '((([year] = 2010)) OR (([year] = 2000)))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA except 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '(not ((([year] = 2010)) OR (([year] = 2000))))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA negate 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '(not ((([year] = 2010)) OR (([year] = 2000))))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA all years but 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '(not ((([year] = 2010)) OR (([year] = 2000))))',
      ],
      dimensions: ['[year]'],
      unknowns: ['all'],
    });
  });

  QUnit.test('top forces distinct', function (assert) {
    // Remove default aggregate for this test
    this.handler.dataset().defaultAgg = function () {
      return undefined;
    };

    this.verifySearchResults(assert, 'top 5 states', {
      aggregates: [],
      dimensions: ['distinct [state] desc limit 5'],
      filters: [],
      unknowns: [],
    });
  });

  QUnit.test('visual types', function (assert) {
    this.verifySearchResults(assert, 'distinct states', {
      aggregates: [],
      dimensions: ['distinct [state]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'unique states', {
      aggregates: [],
      dimensions: ['distinct [state]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 3 unique states', {
      aggregates: [],
      dimensions: ['distinct [state] desc limit 3'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'bottom 5 unique states', {
      aggregates: [],
      dimensions: ['distinct [state] asc limit 5'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population histogram', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: [],
      filters: [],
      unknowns: [],
      showmeVisualConfig: {
        vizType: 'histogram',
      },
    });

    this.verifySearchResults(assert, 'population by year as boxplot', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[year]'],
      filters: [],
      unknowns: [],
      showmeVisualConfig: {
        vizType: 'boxplot',
      },
    });
  });

  QUnit.test('quick searches', function (assert) {
    this.verifySearchResults(assert, 'top 20 state county', {
      aggregates: ['SUM([population]) desc limit 20'],
      dimensions: ['[county]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 20 state counties', {
      aggregates: ['SUM([population]) desc limit 20'],
      dimensions: ['[county]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'top 20 states', {
      aggregates: ['SUM([population]) desc limit 20'],
      dimensions: ['[state]'],
      filters: [],
      unknowns: [],
    });

    this.verifyHighlights(assert, 'top 20 states', [
      {
        startIndex: 0,
        endIndex: 6,
        type: 'top 20',
        phrase: 'top 20',
      },
      {
        startIndex: 7,
        endIndex: 13,
        type: 'dimension',
        phrase: 'states',
      },
    ]);

    this.verifyHighlights(assert, 'top states', [
      {
        startIndex: 0,
        endIndex: 3,
        type: 'top 1',
        phrase: 'top 1',
      },
      {
        startIndex: 4,
        endIndex: 10,
        type: 'dimension',
        phrase: 'states',
      },
    ]);

    this.verifyHighlights(assert, 'top something states', [
      {
        startIndex: 0,
        endIndex: 3,
        type: 'top 1',
        phrase: 'top 1',
      },
      {
        endIndex: 13,
        phrase: 'something',
        startIndex: 4,
        type: 'unknown',
      },
      {
        startIndex: 14,
        endIndex: 20,
        type: 'dimension',
        phrase: 'states',
      },
    ]);

    this.verifySearchResults(assert, 'bottom states', {
      aggregates: ['SUM([population]) asc limit 1'],
      dimensions: ['[state]'],
      filters: [],
      unknowns: [],
    });

    this.verifyHighlights(assert, 'bottom 10 states', [
      {
        startIndex: 0,
        endIndex: 9,
        type: 'bottom 10',
        phrase: 'bottom 10',
      },
      {
        startIndex: 10,
        endIndex: 16,
        type: 'dimension',
        phrase: 'states',
      },
    ]);

    this.verifyHighlights(assert, 'bottom states', [
      {
        startIndex: 0,
        endIndex: 6,
        type: 'bottom 1',
        phrase: 'bottom 1',
      },
      {
        startIndex: 7,
        endIndex: 13,
        type: 'dimension',
        phrase: 'states',
      },
    ]);

    this.verifySearchResults(assert, 'top 20 years', {
      aggregates: ['SUM([population]) desc limit 20'],
      dimensions: ['[year]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      "show me state populations for states that don't start with C",
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ["(not (lower([state]) like 'c%'))"],
        unknowns: [],
      },
    );

    this.verifyHighlights(
      assert,
      "show me state populations for states that don't start with C",
      [
        {
          startIndex: 8,
          endIndex: 13,
          type: 'dimension',
          phrase: 'state',
        },
        {
          startIndex: 14,
          endIndex: 25,
          type: 'aggregate',
          phrase: 'populations',
        },
        {
          startIndex: 30,
          endIndex: 60,
          type: 'filter',
          phrase: 'C',
        },
      ],
    );

    this.verifySearchResults(
      assert,
      'states with population greater than 5000000',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        filters: ['(SUM([population]) > 5000000)'],
        dimensions: ['[state]'],
        unknowns: [],
      },
    );

    this.verifyHighlights(
      assert,
      'states with population greater than 5000000',
      [
        {
          endIndex: 6,
          type: 'dimension',
          phrase: 'states',
          startIndex: 0,
        },
        {
          endIndex: 43,
          type: 'filter',
          phrase: 5000000,
          startIndex: 12,
        },
      ],
    );

    this.verifySearchResults(
      assert,
      'count of states with population greater than 5000000',
      {
        aggregates: ['COUNT([state]) desc limit 5000'],
        filters: ['(([population]) > 5000000)'],
        dimensions: [],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'count of states with population greater than 5000000',
      {
        aggregates: ['COUNT([state]) desc limit 5000'],
        filters: ['(([population]) > 5000000)'],
        dimensions: [],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'count of states in CA', {
      aggregates: ['COUNT([state]) desc limit 5000'],
      filters: ["(lower([state]) like '%ca%')"],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population in may 2010', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "from_timestamp([datestring], 'yyyy-MM-dd') between '2010-05-01' and '2010-05-31'",
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'population between may 2010 and july 2011',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        filters: [
          "from_timestamp([datestring], 'yyyy-MM-dd') between '2010-05-01' and '2011-07-31'",
        ],
        dimensions: [],
        unknowns: [],
      },
    );

    let mStart = moment.utc().subtract(3, 'months').startOf('month');
    let mEnd = moment.utc().subtract(1, 'months').endOf('month');

    this.verifySearchResults(assert, 'population last 3 months', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    mStart = moment.utc().startOf('year');
    mEnd = moment.utc().endOf('year');

    this.verifySearchResults(assert, 'population this year', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population in this year', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population in the current year', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    mStart = moment.utc().subtract(1, 'year').startOf('year');
    mEnd = moment.utc().subtract(1, 'year').endOf('year');

    this.verifySearchResults(assert, 'population last year', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population closed in the last year', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring2], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'population where datestring > datestring2',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        filters: ['([datestring] > [datestring2])'],
        dimensions: [],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'population where datestring between datestring2 and datestring3',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        filters: [
          '([datestring] >= [datestring2]) and ([datestring] <= [datestring3])',
        ],
        dimensions: [],
        unknowns: [],
      },
    );

    mStart = moment.utc().subtract(1, 'month').startOf('month');
    mEnd = moment.utc().subtract(1, 'month').endOf('month');

    this.verifySearchResults(assert, 'population last month', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        `from_timestamp([datestring], 'yyyy-MM-dd') between '${mStart.format(
          'YYYY-MM-DD',
        )}' and '${mEnd.format('YYYY-MM-DD')}'`,
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population Jan 3, 2018', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: ["[datestring] = '2018-01-03'"],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population January 3 2018', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: ["[datestring] = '2018-01-03'"],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'population from January 3 2018 to Jan 6 2019',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        filters: [
          "from_timestamp([datestring], 'yyyy-MM-dd') between '2018-01-03' and '2019-01-06'",
        ],
        dimensions: [],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'population from 2016-02-03 to 2017-09-10',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        filters: [
          "from_timestamp([datestring], 'yyyy-MM-dd') between '2016-02-03' and '2017-09-10'",
        ],
        dimensions: [],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'year:1950 state population', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ['([year] in (1950))'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'year:1950 state:WA state population', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ['([year] in (1950))', "(lower([state]) in ('wa'))"],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'what is the population of CA in 2010', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: ["(lower([state]) like '%ca%')", '(([year] = 2010))'],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'what are the populations by year for CA in 2010 and 2000',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[year]'],
        filters: [
          "(lower([state]) like '%ca%')",
          '((([year] = 2010)) OR (([year] = 2000)))',
        ],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'people CA 2010', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: ["(lower([state]) like '%ca%')", '(([year] = 2010))'],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "(lower([state]) like '%ca%')",
        '((([year] = 2010)) OR (([year] = 2000)))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA 2010', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '(([year] = 2010))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA 2010 2000', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '((([year] = 2010)) OR (([year] = 2000)))',
      ],
      dimensions: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'people CA WA AZ 2010 2000 1900 random', {
      aggregates: ['SUM([population]) desc limit 5000'],
      filters: [
        "((lower([state]) like '%ca%') OR (lower([state]) like '%wa%'))",
        '((([year] = 2010)) OR (([year] = 2000)))',
      ],
      dimensions: [],
      unknowns: ['AZ', '1900', 'random'],
    });

    this.verifySearchResults(
      assert,
      'show me the population where the state is CA',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: [],
        filters: ["(lower([state]) in ('ca'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'show me the population where states are CA and FL',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: [],
        filters: ["(lower([state]) in ('ca', 'fl'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'show me the population where states are CA and FL and year is 1960',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: [],
        filters: ["(lower([state]) in ('ca', 'fl'))", '([year] = 1960)'],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'show me the population where states are in ' +
        'CA, FL and year is 1960 or 2000',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: [],
        filters: [
          "(lower([state]) in ('ca', 'fl'))",
          '([year] in (1960, 2000))',
        ],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'show me state populations for year:2010',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ['([year] in (2010))'],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'show me state populations for C*', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ["(lower([state]) like 'c%')"],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'show me state populations for states that start with C',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ["((lower([state]) like 'c%'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'show me the count of states in 2000', {
      aggregates: ['COUNT([state]) desc limit 5000'],
      dimensions: [],
      filters: ['(([year] = 2000))'],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'show me the average population for states that start with C',
      {
        aggregates: ['AVG([population]) desc limit 5000'],
        dimensions: [],
        filters: ["((lower([state]) like 'c%'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'show me state populations for states that begin with C or D',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ["((lower([state]) like 'c%' OR lower([state]) like 'd%'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'show me state populations for states that end with A',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ["((lower([state]) like '%a'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'monthly population', {
      aggregates: ['SUM([population])'],
      dimensions: [
        'month([datestring]) as month asc limit 5000',
        "from_timestamp([datestring], 'yyyy') as year asc limit 5000",
      ],
      filters: [],
      unknowns: [],
    });

    const thisMonthStart = moment.utc().startOf('month').format('YYYY-MM-DD');
    const thisMonthEnd = moment.utc().endOf('month').format('YYYY-MM-DD');
    const lastMonthStart = moment
      .utc()
      .subtract(1, 'months')
      .startOf('month')
      .format('YYYY-MM-DD');
    const lastMonthEnd = moment
      .utc()
      .subtract(1, 'months')
      .endOf('month')
      .format('YYYY-MM-DD');
    this.verifySearchResults(
      assert,
      'compare monthly population this month vs last month',
      {
        aggregates: ['SUM([population])'],
        dimensions: [
          'month([datestring]) as month asc limit 5000',
          "from_timestamp([datestring], 'yyyy') as year asc limit 5000",
        ],
        filters: [
          `(from_timestamp([datestring], 'yyyy-MM-dd') between '${thisMonthStart}' and '${
            thisMonthEnd
          }' OR from_timestamp([datestring], 'yyyy-MM-dd') between '${lastMonthStart}' and '${lastMonthEnd}')`,
        ],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'population by month', {
      aggregates: ['SUM([population])'],
      dimensions: [
        'month([datestring]) as month asc limit 5000',
        "from_timestamp([datestring], 'yyyy') as year asc limit 5000",
      ],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population by quarter', {
      aggregates: ['SUM([population])'],
      dimensions: [
        "concat('Q', cast(cast(((month([datestring]) + 2) / 3) as int) as string)) as quarter asc limit 5000",
        "from_timestamp([datestring], 'yyyy') as year asc limit 5000",
      ],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'population by year', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[year]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'yearly population', {
      aggregates: ['SUM([population])'],
      dimensions: [
        "from_timestamp([datestring], 'yyyy') as year asc limit 5000",
      ],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'big area', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'big areas', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: [],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'num folks in big areas in 2010', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ['(([year] = 2010))'],
      unknowns: [],
    });

    this.verifySearchResults(
      assert,
      'states with population > 1M and population < 2M',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: [
          '((SUM([population]) > 1000000) AND (SUM([population]) < 2000000))',
        ],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'state populations where state is not C*',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ["((lower([state]) not like 'c%'))"],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'state populations where state is not *ca* and state is not *wa*',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: [
          "(((lower([state]) not like '%ca%')) AND ((lower([state]) not like '%wa%')))",
        ],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'state populations where year is 1980 or 1990',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ['([year] in (1980, 1990))'],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'state populations where year is 1980, 1990 or 2000',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: ['([year] in (1980, 1990, 2000))'],
        unknowns: [],
      },
    );

    this.verifySearchResults(
      assert,
      'state populations where year is 1980, 1990 or 2000 and state is CA',
      {
        aggregates: ['SUM([population]) desc limit 5000'],
        dimensions: ['[state]'],
        filters: [
          '([year] in (1980, 1990, 2000))',
          "(lower([state]) in ('ca'))",
        ],
        unknowns: [],
      },
    );

    this.verifySearchResults(assert, 'USA state populations in the 80s', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ['([year] in (1970, 1980))'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'state populations in the 80s', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ['([year] in (1970, 1980))'],
      unknowns: [],
    });

    this.verifySearchResults(assert, 'state populations in the 90s', {
      aggregates: ['SUM([population]) desc limit 5000'],
      dimensions: ['[state]'],
      filters: ['([year] in (1980, 1990))'],
      unknowns: [],
    });
  });
}());
