QUnit.module('framework_arc_searchtokens', {});

QUnit.test('relative date tokens', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('this year');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'datetime');
  assert.equal(tokens[0].period_count, undefined);
  assert.equal(tokens[0].period, 'year');
  assert.equal(tokens[0].period_prevnext, 'curr');

  ({ tokens } = arc.grammar._searchtokens.parse('last 2 years'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'datetime');
  assert.equal(tokens[0].period_count, 2);
  assert.equal(tokens[0].period, 'year');
  assert.equal(tokens[0].period_prevnext, 'prev');

  ({ tokens } = arc.grammar._searchtokens.parse('next 6 months'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'datetime');
  assert.equal(tokens[0].period_count, 6);
  assert.equal(tokens[0].period, 'month');
  assert.equal(tokens[0].period_prevnext, 'next');

  ({ tokens } = arc.grammar._searchtokens.parse('last year'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'datetime');
  assert.equal(tokens[0].period_count, 1);
  assert.equal(tokens[0].period, 'year');
  assert.equal(tokens[0].period_prevnext, 'prev');
});

QUnit.test('tokens_strings', (assert) => {
  const { tokens } = arc.grammar._searchtokens.parse(
    'average top 20 "abc def" ghi',
    {
      startRule: 'strings',
    },
  );
  assert.equal(tokens.length, 5);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'average');
  assert.equal(tokens[1].type, 'str');
  assert.equal(tokens[1].phrase, 'top');
  assert.equal(tokens[2].type, 'str');
  assert.equal(tokens[2].phrase, '20');
  assert.equal(tokens[3].type, 'str');
  assert.equal(tokens[3].phrase, 'abc def');
  assert.equal(tokens[4].type, 'str');
  assert.equal(tokens[4].phrase, 'ghi');
});

QUnit.test('tokens_simple', (assert) => {
  const { tokens } = arc.grammar._searchtokens.parse('summary ');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].phrase, 'summary');
  assert.equal(tokens[0].type, 'str');
});

QUnit.test('tokens_agg', (assert) => {
  arc.grammar._searchtokens.parse('summary');

  ['average', 'avg', 'mean'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'AVG');
  });

  ['median'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'MEDIAN');
  });

  ['sum', 'total'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'SUM');
  });

  ['count'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'COUNT');
  });

  ['distinct count', 'count distinct', 'unique count', 'count unique'].forEach(
    (searchString) => {
      const { tokens } = arc.grammar._searchtokens.parse(searchString);
      assert.equal(tokens.length, 1);
      assert.equal(tokens[0].type, 'aggregation');
      assert.equal(tokens[0].aggregation, 'COUNTDISTINCT');
    },
  );

  ['min', 'minimum'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'MIN');
  });

  ['max', 'maximum'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'MAX');
  });

  [
    'stddev',
    'standard deviation',
    'std dev',
    'standard dev',
    'std deviation',
  ].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'STDDEV');
  });

  ['population standard deviation', 'population stddev'].forEach(
    (searchString) => {
      const { tokens } = arc.grammar._searchtokens.parse(searchString);
      assert.equal(tokens.length, 1);
      assert.equal(tokens[0].type, 'aggregation');
      assert.equal(tokens[0].aggregation, 'STDDEV_POP');
    },
  );

  ['variance'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'VARIANCE');
  });

  ['population variance'].forEach((searchString) => {
    const { tokens } = arc.grammar._searchtokens.parse(searchString);
    assert.equal(tokens.length, 1);
    assert.equal(tokens[0].type, 'aggregation');
    assert.equal(tokens[0].aggregation, 'VARIANCE_POP');
  });
});

QUnit.test('tokens_top', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('top 7');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'top');
  assert.equal(tokens[0].k, '7');

  ({ tokens } = arc.grammar._searchtokens.parse('highest 7'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'top');
  assert.equal(tokens[0].k, '7');

  ({ tokens } = arc.grammar._searchtokens.parse('top '));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'top');
  assert.equal(tokens[0].k, '1');

  ({ tokens } = arc.grammar._searchtokens.parse('bottom 17'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'bottom');
  assert.equal(tokens[0].k, '17');

  ({ tokens } = arc.grammar._searchtokens.parse('lowest 17'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'bottom');
  assert.equal(tokens[0].k, '17');

  ({ tokens } = arc.grammar._searchtokens.parse('bottom '));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'bottom');
  assert.equal(tokens[0].k, '1');
});

QUnit.test('tokens_whitespace', (assert) => {
  assert.equal(arc.grammar._searchtokens.parse(' \t\r\n\xA0').tokens.length, 0);
});

QUnit.test('tokens_str', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('"abc def"');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc def');

  ({ tokens } = arc.grammar._searchtokens.parse("'abc def'"));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc def');

  ({ tokens } = arc.grammar._searchtokens.parse('abc'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  ({ tokens } = arc.grammar._searchtokens.parse('abc def'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');
  assert.equal(tokens[1].type, 'str');
  assert.equal(tokens[1].phrase, 'def');
});

QUnit.test('compare_checks', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('abc >= 3.24');
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '>=');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('abc>=3.24'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '>=');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('abc> 3.24'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '>');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('abc<=3.24'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '<=');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('abc <3.24'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '<');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('abc=3.24'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '=');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('abc!=3.24'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'abc');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '!=');
  assert.equal(tokens[1].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('channel="nflhd"'));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'channel');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '=');
  assert.equal(tokens[1].term, 'nflhd');

  ({ tokens } = arc.grammar._searchtokens.parse("channel='nflhd'"));
  assert.equal(tokens.length, 2);

  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'channel');

  assert.equal(tokens[1].type, 'compare');
  assert.equal(tokens[1].subtype, '=');
  assert.equal(tokens[1].term, 'nflhd');
});

QUnit.test('tokens_compare', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('>= 3.24');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, '>=');
  assert.equal(tokens[0].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('>=3.24'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, '>=');
  assert.equal(tokens[0].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('is not lesser than 3.24'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, '>=');
  assert.equal(tokens[0].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('is not over 3.24'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, '<=');
  assert.equal(tokens[0].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('is greater than 3.24'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, '>');
  assert.equal(tokens[0].term, 3.24);

  ({ tokens } = arc.grammar._searchtokens.parse('is under 3.24'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, '<');
  assert.equal(tokens[0].term, 3.24);
});

QUnit.test('tokens_equality', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('is Mary');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'equality');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('equals Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'equality');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('equal Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'equality');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('is equal to Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'equality');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('is equals to Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'equality');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('match Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'equality');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('plot between 10 and 20'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'plot');
  assert.equal(tokens[1].type, 'compare');

  ({ tokens } = arc.grammar._searchtokens.parse('plot from 10 to 20'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'plot');
  assert.equal(tokens[1].type, 'compare');
});

QUnit.test('tokens_compare', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('starts with Mary');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'startswith');
  assert.equal(tokens[0].phrase, 'Mary');
  assert.equal(tokens[0].negate, undefined);

  ({ tokens } = arc.grammar._searchtokens.parse('not starts with Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'startswith');
  assert.equal(tokens[0].phrase, 'Mary');
  assert.equal(tokens[0].negate, true);

  ({ tokens } = arc.grammar._searchtokens.parse("don't start with Mary"));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'startswith');
  assert.equal(tokens[0].phrase, 'Mary');
  assert.equal(tokens[0].negate, true);

  ({ tokens } = arc.grammar._searchtokens.parse('start with Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'startswith');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('begins with Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'startswith');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('ends with Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'endswith');
  assert.equal(tokens[0].phrase, 'Mary');

  ({ tokens } = arc.grammar._searchtokens.parse('EnDs With Mary'));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'compare');
  assert.equal(tokens[0].subtype, 'endswith');
  assert.equal(tokens[0].phrase, 'Mary');
});

QUnit.test('tokens_connector', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('Or ');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'connector');
  assert.equal(tokens[0].subtype, 'or');
  assert.equal(tokens[0].phrase, 'or');

  ({ tokens } = arc.grammar._searchtokens.parse('And '));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'connector');
  assert.equal(tokens[0].subtype, 'and');

  ({ tokens } = arc.grammar._searchtokens.parse(','));
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'connector');
  assert.equal(tokens[0].subtype, ',');
});

QUnit.test('tokens_keyvalue', (assert) => {
  const { tokens } = arc.grammar._searchtokens.parse('abc:def');
  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].type, 'keyvalue');
  assert.equal(tokens[0].key, 'abc');
  assert.equal(tokens[0].value, 'def');
  assert.equal(tokens[0].phrase, 'abc:def');
});

QUnit.test('parser', (assert) => {
  let { tokens } = arc.grammar._searchtokens.parse('abc def');

  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].phrase, 'abc');
  assert.equal(tokens[0].endIndex, 3);
  assert.equal(tokens[1].phrase, 'def');
  assert.equal(tokens[1].startIndex, 4);
  assert.equal(tokens[1].endIndex, 7);

  ({ tokens } = arc.grammar._searchtokens.parse('abc '));

  assert.equal(tokens.length, 1);
  assert.equal(tokens[0].phrase, 'abc');
  assert.equal(tokens[0].startIndex, 0);
  assert.equal(tokens[0].endIndex, 4);

  ({ tokens } = arc.grammar._searchtokens.parse("abc 'def g' hij"));

  assert.equal(tokens.length, 3);
  assert.equal(tokens[0].phrase, 'abc');
  assert.equal(tokens[0].startIndex, 0);
  assert.equal(tokens[0].endIndex, 3);
  assert.equal(tokens[1].phrase, 'def g');
  assert.equal(tokens[1].endIndex, 11);
  assert.equal(tokens[2].phrase, 'hij');
  assert.equal(tokens[2].endIndex, 15);

  ({ tokens } = arc.grammar._searchtokens.parse('abc "def g" hij'));

  assert.equal(tokens.length, 3);
  assert.equal(tokens[0].phrase, 'abc');
  assert.equal(tokens[0].startIndex, 0);
  assert.equal(tokens[0].endIndex, 3);
  assert.equal(tokens[1].phrase, 'def g');
  assert.equal(tokens[1].endIndex, 11);
  assert.equal(tokens[2].phrase, 'hij');
  assert.equal(tokens[2].endIndex, 15);

  ({ tokens } = arc.grammar._searchtokens.parse('ab\\\'c "de\\"f g" hi\\"j'));

  assert.equal(tokens.length, 3);
  assert.equal(tokens[0].phrase, "ab\\'c");
  assert.equal(tokens[1].phrase, 'de"f g');
  assert.equal(tokens[2].phrase, 'hi\\"j');

  ({ tokens } = arc.grammar._searchtokens.parse('top shows'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'top');
  assert.equal(tokens[0].k, 1);
  assert.equal(tokens[0].phrase, 'top 1');
  assert.equal(tokens[0].endIndex, 3);
  assert.equal(tokens[1].phrase, 'shows');
  // TODO
  //  assert.equal(tokens[1].endIndex, 10);

  ({ tokens } = arc.grammar._searchtokens.parse('top 20 shows'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'top');
  assert.equal(tokens[0].k, 20);
  assert.equal(tokens[0].phrase, 'top 20');
  assert.equal(tokens[0].endIndex, 6);
  assert.equal(tokens[1].phrase, 'shows');
  assert.equal(tokens[1].endIndex, 12);

  ({ tokens } = arc.grammar._searchtokens.parse('top30 shows'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'top');
  assert.equal(tokens[0].k, 30);
  assert.equal(tokens[0].phrase, 'top 30');
  assert.equal(tokens[0].endIndex, 5);
  assert.equal(tokens[1].phrase, 'shows');
  assert.equal(tokens[1].endIndex, 11);

  ({ tokens } = arc.grammar._searchtokens.parse('bottom shows'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'bottom');
  assert.equal(tokens[0].k, 1);
  assert.equal(tokens[0].phrase, 'bottom 1');
  assert.equal(tokens[1].phrase, 'shows');

  ({ tokens } = arc.grammar._searchtokens.parse('bottom2 shows'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'bottom');
  assert.equal(tokens[0].k, 2);
  assert.equal(tokens[0].phrase, 'bottom 2');
  assert.equal(tokens[1].phrase, 'shows');

  ({ tokens } = arc.grammar._searchtokens.parse('bottom520 shows'));
  assert.equal(tokens.length, 2);
  assert.equal(tokens[0].type, 'bottom');
  assert.equal(tokens[0].k, 520);
  assert.equal(tokens[0].phrase, 'bottom 520');
  assert.equal(tokens[1].phrase, 'shows');

  ({ tokens } = arc.grammar._searchtokens.parse(
    'compare population this month vs last month',
  ));
  assert.equal(tokens.length, 5);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'compare');
  assert.equal(tokens[1].type, 'str');
  assert.equal(tokens[1].phrase, 'population');
  assert.equal(tokens[2].type, 'datetime');
  assert.equal(tokens[2].phrase, 'this month');
  assert.equal(tokens[3].type, 'connector');
  assert.equal(tokens[3].phrase, 'or');
  assert.equal(tokens[4].type, 'datetime');
  assert.equal(tokens[4].phrase, 'last month');

  ({ tokens } = arc.grammar._searchtokens.parse(
    'compare population this month versus last month',
  ));
  assert.equal(tokens.length, 5);
  assert.equal(tokens[0].type, 'str');
  assert.equal(tokens[0].phrase, 'compare');
  assert.equal(tokens[1].type, 'str');
  assert.equal(tokens[1].phrase, 'population');
  assert.equal(tokens[2].type, 'datetime');
  assert.equal(tokens[2].phrase, 'this month');
  assert.equal(tokens[3].type, 'connector');
  assert.equal(tokens[3].phrase, 'or');
  assert.equal(tokens[4].type, 'datetime');
  assert.equal(tokens[4].phrase, 'last month');
});
