window.arcl = window.arcl || {};

QUnit.module('framework_arc_shelves', {
  beforeEach() {
    window.getCurrentParams = arc.report.shelf.getCurrentParams;
    arc.report.shelf.CONNECTION_TYPE = arc.report.shelf.CONNECTION_TYPE || {};
    arc.report.shelf.CONNECTION_TYPE.connection_type = 'arcengine';
  },

  afterEach() {
    arc.report.shelf.getCurrentParams = window.getCurrentParams;
    delete arc.report.shelf.CONNECTION_TYPE.connection_type;
  },
});

QUnit.test('simple dimension', (assert) => {
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .dataset_coltype('STRING');

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');
  assert.equal(s.getExpression(), "[state] as 'state'");
});

QUnit.test('dimension parameter', (assert) => {
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .dataset_coltype('STRING')
    .custom_expr('<<dim1>> as <<dim1.alias>>');

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');

  const params = arc.params.tracker();
  params.add({
    key: 'dim1',
    value: 'state',
  });
  params.add({
    key: 'dim1.alias',
    value: 'State',
  });
  assert.equal(s.getExpression(params.substitute), "state as 'State'");
});

QUnit.test('picklist single filter for oracle - double quote', (assert) => {
  arc.report.shelf.CONNECTION_TYPE.connection_type = 'oracle';
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .filter_column('state')
    .dataset_coltype('STRING');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'state',
        value: 'CA',
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');
  assert.equal(s.getExpression(), "[state] in ('CA')");
});

QUnit.test('picklist single filter for oracle - single quote', (assert) => {
  arc.report.shelf.CONNECTION_TYPE.connection_type = 'oracle';
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .filter_column('state')
    .dataset_coltype('STRING');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'state',
        value: 'CA',
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');
  assert.equal(s.getExpression(), "[state] in ('CA')");
});

QUnit.test('picklist single filter for postgres', (assert) => {
  arc.report.shelf.CONNECTION_TYPE.connection_type = 'postgres';
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .filter_column('state')
    .dataset_coltype('STRING');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'state',
        value: 'CA',
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');

  assert.equal(s.getExpression(), "[state] in ('CA')");
});

QUnit.test('picklist single filter for arcengine', (assert) => {
  arc.report.shelf.CONNECTION_TYPE.connection_type = 'arcengine';
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .filter_column('state')
    .dataset_coltype('STRING');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'state',
        value: 'CA',
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');

  assert.equal(s.getExpression(), "[state] in ('CA')");
});

QUnit.test('picklist single filter for sqlite', (assert) => {
  arc.report.shelf.CONNECTION_TYPE.connection_type = 'sqlite';
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .filter_column('state')
    .dataset_coltype('STRING');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'state',
        value: 'CA',
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');

  assert.equal(s.getExpression(), "[state] in ('CA')");
});

QUnit.test('picklist filter', (assert) => {
  const s = arc.report.shelf
    .element()
    .dataset_colname('state')
    .filter_column('state')
    .dataset_coltype('STRING');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'state.exclude',
        value: 'IN',
      },
      {
        key: 'state.data',
        value: "'CA', 'AZ'",
        value_list: ['CA', 'AZ'],
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'state');
  assert.equal(s.dataset_coltype(), 'STRING');
  assert.equal(s.getExpression(), "[state] in ('CA', 'AZ')");
});

QUnit.test('daterange filter', (assert) => {
  const s = arc.report.shelf
    .element()
    .dataset_colname('datestring')
    .filter_column('datestring')
    .dataset_coltype('TIMESTAMP');

  assert.equal(s.dataset_colname(), 'datestring');
  assert.equal(s.dataset_coltype(), 'TIMESTAMP');
  assert.equal(s.getExpression(), '(1 = 1)');

  arc.report.shelf.getCurrentParams = function () {
    return [
      {
        key: 'datestring.start',
        value: '2013-01-02',
      },
      {
        key: 'datestring.end',
        value: '2013-03-04',
      },
    ];
  };

  assert.equal(s.dataset_colname(), 'datestring');
  assert.equal(s.dataset_coltype(), 'TIMESTAMP');
  assert.equal(
    s.getExpression(),
    "from_timestamp([datestring], 'yyyy-MM-dd') between '2013-01-02' and '2013-03-04'",
  );
});
