QUnit.module('framework_arc_tree');

QUnit.test('basic', (assert) => {
  const head = arc.tree.node().data('h');
  const c1 = arc.tree.node().data('c1');
  const c2 = arc.tree.node().data('c2');
  const c3 = arc.tree.node().data('c3');

  function withNone() {
    assert.equal(head.prev(), undefined);
    assert.equal(head.next(), undefined);
    assert.equal(head.parent(), undefined);
    assert.equal(head.data(), 'h');
    assert.equal(head.lastChild(), undefined);
    assert.deepEqual(head.children(), []);
  }

  function withC1() {
    assert.equal(head.prev(), undefined);
    assert.equal(head.next(), undefined);
    assert.equal(head.parent(), undefined);
    assert.equal(head.data(), 'h');
    assert.equal(head.lastChild(), c1);
    assert.deepEqual(head.children(), [c1]);

    assert.equal(c1.prev(), undefined);
    assert.equal(c1.next(), undefined);
    assert.equal(c1.parent(), head);
    assert.equal(c1.data(), 'c1');
    assert.equal(c1.lastChild(), undefined);
    assert.deepEqual(c1.children(), []);
  }

  function withC2() {
    assert.equal(head.prev(), undefined);
    assert.equal(head.next(), undefined);
    assert.equal(head.parent(), undefined);
    assert.equal(head.data(), 'h');
    assert.equal(head.lastChild(), c2);
    assert.deepEqual(head.children(), [c1, c2]);

    assert.equal(c1.prev(), undefined);
    assert.equal(c1.next(), c2);
    assert.equal(c1.parent(), head);
    assert.equal(c1.data(), 'c1');
    assert.equal(c1.lastChild(), undefined);
    assert.deepEqual(c1.children(), []);

    assert.equal(c2.prev(), c1);
    assert.equal(c2.next(), undefined);
    assert.equal(c2.parent(), head);
    assert.equal(c2.data(), 'c2');
    assert.equal(c2.lastChild(), undefined);
    assert.deepEqual(c2.children(), []);
  }

  function withC3() {
    assert.equal(head.prev(), undefined);
    assert.equal(head.next(), undefined);
    assert.equal(head.parent(), undefined);
    assert.equal(head.data(), 'h');
    assert.equal(head.lastChild(), c3);
    assert.deepEqual(head.children(), [c1, c2, c3]);

    assert.equal(c1.prev(), undefined);
    assert.equal(c1.next(), c2);
    assert.equal(c1.parent(), head);
    assert.equal(c1.data(), 'c1');
    assert.equal(c1.lastChild(), undefined);
    assert.deepEqual(c1.children(), []);

    assert.equal(c2.prev(), c1);
    assert.equal(c2.next(), c3);
    assert.equal(c2.parent(), head);
    assert.equal(c2.data(), 'c2');
    assert.equal(c2.lastChild(), undefined);
    assert.deepEqual(c2.children(), []);

    assert.equal(c3.prev(), c2);
    assert.equal(c3.next(), undefined);
    assert.equal(c3.parent(), head);
    assert.equal(c3.data(), 'c3');
    assert.equal(c3.lastChild(), undefined);
    assert.deepEqual(c3.children(), []);
  }

  function withC1C3() {
    assert.equal(head.prev(), undefined);
    assert.equal(head.next(), undefined);
    assert.equal(head.parent(), undefined);
    assert.equal(head.data(), 'h');
    assert.equal(head.lastChild(), c3);
    assert.deepEqual(head.children(), [c1, c3]);

    assert.equal(c1.prev(), undefined);
    assert.equal(c1.next(), c3);
    assert.equal(c1.parent(), head);
    assert.equal(c1.data(), 'c1');
    assert.equal(c1.lastChild(), undefined);
    assert.deepEqual(c1.children(), []);

    assert.equal(c2.prev(), undefined);
    assert.equal(c2.next(), undefined);
    assert.equal(c2.parent(), undefined);
    assert.equal(c2.data(), 'c2');
    assert.equal(c2.lastChild(), undefined);
    assert.deepEqual(c2.children(), []);

    assert.equal(c3.prev(), c1);
    assert.equal(c3.next(), undefined);
    assert.equal(c3.parent(), head);
    assert.equal(c3.data(), 'c3');
    assert.equal(c3.lastChild(), undefined);
    assert.deepEqual(c3.children(), []);
  }

  function withOnlyC3() {
    assert.equal(head.prev(), undefined);
    assert.equal(head.next(), undefined);
    assert.equal(head.parent(), undefined);
    assert.equal(head.data(), 'h');
    assert.equal(head.lastChild(), c3);
    assert.deepEqual(head.children(), [c3]);

    assert.equal(c3.prev(), undefined);
    assert.equal(c3.next(), undefined);
    assert.equal(c3.parent(), head);
    assert.equal(c3.data(), 'c3');
    assert.equal(c3.lastChild(), undefined);
    assert.deepEqual(c3.children(), []);
  }

  withNone();
  head.addChild(c1);
  withC1();
  head.addChild(c2);
  withC2();
  head.addChild(c3);
  withC3();
  c3.remove();
  withC2();
  c2.remove();
  withC1();
  c1.remove();

  withNone();
  head.addChild(c1);
  withC1();
  head.addChild(c2);
  withC2();
  head.addChild(c3);
  withC3();

  c2.remove();
  withC1C3();
  c1.remove();
  withOnlyC3();
  c3.remove();
  withNone();
});
