QUnit.module('misc_arc_perf');

QUnit.test('Can display query runtimes', (assert) => {
  const runtimeColumn = '#test-table-id tbody tr td:nth-child(6)';

  // extract runtime from queries profile:
  arc.perf.PAGEPERF.profiles().splice(0);
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 1,
    profile: [['query_runtime', 10]],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 2,
    profile: [['query_runtime', 20]],
  });

  // filter for the visual
  arc.perf.showQueries('test-table-id', 1);
  assert.equal($(runtimeColumn).length, 1);
  assert.equal($(runtimeColumn).eq(0).text(), '10');

  // unfiltered
  arc.perf.showQueries('test-table-id');
  assert.equal($(runtimeColumn).length, 2);

  // extract runtime from status profile:
  arc.perf.PAGEPERF.profiles().splice(0);
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.A1'],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 2,
    profile: [
      ['querytracker', '2'],
      ['query', 'select * from main.A2'],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Status',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.B1'],
      ['query_runtime', 100],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Status',
    id: 2,
    profile: [
      ['querytracker', '2'],
      ['query', 'select * from main.B2'],
      ['query_runtime', 200],
    ],
  });

  arc.perf.showQueries('test-table-id', 2);
  assert.equal($(runtimeColumn).length, 1);
  assert.equal($(runtimeColumn).eq(0).text(), '200');

  arc.perf.showQueries('test-table-id');
  assert.equal($(runtimeColumn).length, 2);

  // same id
  arc.perf.PAGEPERF.profiles().splice(0);
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.A1'],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 1,
    profile: [
      ['querytracker', '2'],
      ['query', 'select * from main.A2'],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Status',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.B1'],
      ['query_runtime', 100],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Status',
    id: 1,
    profile: [
      ['querytracker', '2'],
      ['query', 'select * from main.B2'],
      ['query_runtime', 200],
    ],
  });

  arc.perf.showQueries('test-table-id', 1);
  assert.equal($(runtimeColumn).length, 2);
  assert.equal($(runtimeColumn).eq(0).text(), '200');
  assert.equal($(runtimeColumn).eq(1).text(), '100');

  arc.perf.showQueries('test-table-id');
  assert.equal($(runtimeColumn).length, 2);
});

QUnit.test('Displays queries with no runtime information', (assert) => {
  const runtimeColumn = '#test-table-id tbody tr td:nth-child(6)';

  // query + status data
  arc.perf.PAGEPERF.profiles().splice(0);
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.A1'],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Status',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.B1'],
    ],
  });

  arc.perf.showQueries('test-table-id');
  assert.equal($(runtimeColumn).length, 1);
  assert.equal($(runtimeColumn).eq(0).text(), '');
});

QUnit.test('Tolerates multiple query trackers in status', (assert) => {
  const runtimeColumn = '#test-table-id tbody tr td:nth-child(6)';

  // query + status data
  arc.perf.PAGEPERF.profiles().splice(0);
  arc.perf.PAGEPERF.addProfile({
    type: 'Query',
    id: 1,
    profile: [
      ['querytracker', '1'],
      ['query', 'select * from main.A1'],
    ],
  });
  arc.perf.PAGEPERF.addProfile({
    type: 'Status',
    id: 1,
    profile: [
      ['querytracker', '2'], // sometimes there are multiple trackers in the status
      ['query', 'select * from main.B1'],
      ['querytracker', '1'],
      ['query_runtime', '100'],
    ],
  });

  arc.perf.showQueries('test-table-id');
  assert.equal($(runtimeColumn).length, 1);
  assert.equal($(runtimeColumn).eq(0).text(), '100');
});
