(function () {
  let jqueryAjax;
  let ajaxMock;

  function mockAjax() {
    jqueryAjax = $.ajax;

    ajaxMock = {
      fail(failCallback) {
        ajaxMock.failCallback = failCallback;
        return ajaxMock;
      },
      done(doneCallback) {
        ajaxMock.doneCallback = doneCallback;
        return ajaxMock;
      },
    };

    $.ajax = (params) => {
      ajaxMock.params = params;
      return ajaxMock;
    };
  }

  QUnit.module('pages_arc_extractmgr', {
    afterEach() {
      if (jqueryAjax) {
        $.ajax = jqueryAjax;
      }
    },
  });

  QUnit.test('extract delete', async (assert) => {
    doTestExtractDelete(assert);
  });

  function doTestExtractDelete(assert) {
    const dataset = arc.testutil.ds.create_dataset();
    dataset.dataconnection_type('impyla');
    arc.testutil.ds.patch_dataset_init_from_fetch(dataset);
    arc.testutil.ds.mock_arcl();
    arc.globals.urls = new Proxy(
      {},
      {
        get(target, prop) {
          return prop;
        },
      },
    );

    const mockDatasets = [
      {
        perms: [],
        created: '2023-03-05',
        updated: '2023-03-05',
        events: [],
        segments: [],
      },
    ];
    const mockExtracts = [
      {
        id: 123,
        extract_config: {
          source: {
            dimData: [],
            aggData: [],
          },
        },
      },
    ];

    let fetchCallCounter = 1;

    const fetchMockData = [
      [mockDatasets],
      [mockDatasets, mockExtracts, [], []],
    ];

    arc.staticfetcher.fetch = function (asset, callback) {
      const error = null; // we mock successful retrieval
      callback.call(null, ...[error].concat(fetchMockData[fetchCallCounter]));
      fetchCallCounter += 1;
    };

    arc.admin.dataset().dataset_id('108').initialize();

    // trigger deleting:
    $('[data-test-id="extract-del-btn"]').trigger('click');
    mockAjax();
    $('button:contains("Delete Extract")').trigger('click');

    ajaxMock.doneCallback([]);

    assert.true(
      ajaxMock.params.url.split('?')[1].includes('deleteTable=false'),
    );

    // now trigger target table deleting:
    $('[data-test-id="extract-del-btn"]').trigger('click');
    $('#deleteExtractTableCheckbox').trigger('click');

    $('button:contains("Delete Extract")').trigger('click');
    $('#delete-confirmation-text').val('DELETE');
    $('#delete-confirmation-text').trigger('change');
    $('#delete-confirmation-yes').trigger('click');

    ajaxMock.doneCallback([]);

    assert.true(ajaxMock.params.url.split('?')[1].includes('deleteTable=true'));
  }
}());
