window.arc = window.arc || {};

(function (assert) {
  'use strict';

  arc.misc = arc.misc || {};

  // Extends QUnit assertions. Checks whether the expected string is within the actual string.
  arc.misc.includes = function (actual, expected, message) {
    QUnit.pushResult(actual.includes(expected), actual, expected, message);
  };

  arc.misc.deep_array_within_tolerance = function (arr1, arr2, tolerance) {
    if (tolerance === undefined) {
      tolerance = 100;
    }

    if (arr1.length !== arr2.length) {
      assert.equal(
        true,
        false,
        `Not equal. arr1 length ${arr1.length}, arr2 length ${
          arr2.length
        }. arr1 ${JSON.stringify(arr1)}. arr2 ${JSON.stringify(arr2)}`,
      );
      return false;
    }

    arr1.forEach((a1, idx1) => {
      const a2 = arr2[idx1];
      if (a1.length !== a2.length) {
        assert.equal(
          true,
          false,
          `Not equal. idx ${idx1} a1 length ${a1.length}, a2 length ${a2.length}`,
        );
        return false;
      }

      a1.forEach((e1, idx2) => {
        const e2 = a2[idx2];
        return arc.misc.within_tolerance(e2, e1, tolerance);
      });
    });

    return true;
  };

  arc.misc.within_tolerance = function (
    expectedNumber,
    actualNumber,
    tolerance,
  ) {
    expectedNumber = parseInt(expectedNumber, 10);
    actualNumber = parseInt(actualNumber, 10);
    const result =
      actualNumber <= expectedNumber + tolerance &&
      actualNumber >= expectedNumber - tolerance;

    if (!result) {
      assert.equal(
        true,
        false,
        `Not within tolerance. Expected: ${expectedNumber} +/- ${tolerance}, Actual: ${actualNumber}`,
      );
    }

    return result;
  };

  arc.misc.modifySettingValue = function (user_settings, id, value) {
    user_settings.find_by_id(id).value(value);
  };

  arc.misc.roundFirstDigit = function (number) {
    const formatter = d3.format('.1r');
    return formatter(number);
  };

  function componentToHex(colorComponent) {
    const hex = colorComponent.toString(16);
    return hex.length === 1 ? `0${hex}` : hex;
  }

  arc.misc.colorToHex = function (colorValue) {
    if (colorValue.startsWith('rgb')) {
      const rgb = colorValue
        .slice(colorValue.indexOf('(') + 1, colorValue.indexOf(')'))
        .split(',');
      const red = componentToHex(parseInt(rgb[0], 10));
      const green = componentToHex(parseInt(rgb[1], 10));
      const blue = componentToHex(parseInt(rgb[2], 10));
      return '#'.concat(red, green, blue);
    }

    if (colorValue.startsWith('#')) {
      return colorValue;
    }

    return colorNameToHex(colorValue);
  };

  function colorNameToHex(colorName) {
    const colors = {
      aliceblue: '#f0f8ff',
      antiquewhite: '#faebd7',
      aqua: '#00ffff',
      aquamarine: '#7fffd4',
      azure: '#f0ffff',
      beige: '#f5f5dc',
      bisque: '#ffe4c4',
      black: '#000000',
      blanchedalmond: '#ffebcd',
      blue: '#0000ff',
      blueviolet: '#8a2be2',
      brown: '#a52a2a',
      burlywood: '#deb887',
      cadetblue: '#5f9ea0',
      chartreuse: '#7fff00',
      chocolate: '#d2691e',
      coral: '#ff7f50',
      cornflowerblue: '#6495ed',
      cornsilk: '#fff8dc',
      crimson: '#dc143c',
      cyan: '#00ffff',
      darkblue: '#00008b',
      darkcyan: '#008b8b',
      darkgoldenrod: '#b8860b',
      darkgray: '#a9a9a9',
      darkgrey: '#a9a9a9',
      darkgreen: '#006400',
      darkkhaki: '#bdb76b',
      darkmagenta: '#8b008b',
      darkolivegreen: '#556b2f',
      darkorange: '#ff8c00',
      darkorchid: '#9932cc',
      darkred: '#8b0000',
      darksalmon: '#e9967a',
      darkseagreen: '#8fbc8f',
      darkslateblue: '#483d8b',
      darkslategray: '#2f4f4f',
      darkslategrey: '#2f4f4f',
      darkturquoise: '#00ced1',
      darkviolet: '#9400d3',
      deeppink: '#ff1493',
      deepskyblue: '#00bfff',
      dimgray: '#696969',
      dimgrey: '#696969',
      dodgerblue: '#1e90ff',
      firebrick: '#b22222',
      floralwhite: '#fffaf0',
      forestgreen: '#228b22',
      fuchsia: '#ff00ff',
      gainsboro: '#dcdcdc',
      ghostwhite: '#f8f8ff',
      gold: '#ffd700',
      goldenrod: '#daa520',
      gray: '#808080',
      grey: '#808080',
      green: '#008000',
      greenyellow: '#adff2f',
      honeydew: '#f0fff0',
      hotpink: '#ff69b4',
      indianred: '#cd5c5c',
      indigo: '#4b0082',
      ivory: '#fffff0',
      khaki: '#f0e68c',
      lavender: '#e6e6fa',
      lavenderblush: '#fff0f5',
      lawngreen: '#7cfc00',
      lemonchiffon: '#fffacd',
      lightblue: '#add8e6',
      lightcoral: '#f08080',
      lightcyan: '#e0ffff',
      lightgoldenrodyellow: '#fafad2',
      lightgray: '#d3d3d3',
      lightgrey: '#d3d3d3',
      lightgreen: '#90ee90',
      lightpink: '#ffb6c1',
      lightsalmon: '#ffa07a',
      lightseagreen: '#20b2aa',
      lightskyblue: '#87cefa',
      lightslategray: '#778899',
      lightslategrey: '#778899',
      lightsteelblue: '#b0c4de',
      lightyellow: '#ffffe0',
      lime: '#00ff00',
      limegreen: '#32cd32',
      linen: '#faf0e6',
      magenta: '#ff00ff',
      maroon: '#800000',
      mediumaquamarine: '#66cdaa',
      mediumblue: '#0000cd',
      mediumorchid: '#ba55d3',
      mediumpurple: '#9370d8',
      mediumseagreen: '#3cb371',
      mediumslateblue: '#7b68ee',
      mediumspringgreen: '#00fa9a',
      mediumturquoise: '#48d1cc',
      mediumvioletred: '#c71585',
      midnightblue: '#191970',
      mintcream: '#f5fffa',
      mistyrose: '#ffe4e1',
      moccasin: '#ffe4b5',
      navajowhite: '#ffdead',
      navy: '#000080',
      oldlace: '#fdf5e6',
      olive: '#808000',
      olivedrab: '#6b8e23',
      orange: '#ffa500',
      orangered: '#ff4500',
      orchid: '#da70d6',
      palegoldenrod: '#eee8aa',
      palegreen: '#98fb98',
      paleturquoise: '#afeeee',
      palevioletred: '#d87093',
      papayawhip: '#ffefd5',
      peachpuff: '#ffdab9',
      peru: '#cd853f',
      pink: '#ffc0cb',
      plum: '#dda0dd',
      powderblue: '#b0e0e6',
      purple: '#800080',
      rebeccapurple: '663399',
      red: '#ff0000',
      rosybrown: '#bc8f8f',
      royalblue: '#4169e1',
      saddlebrown: '#8b4513',
      salmon: '#fa8072',
      sandybrown: '#f4a460',
      seagreen: '#2e8b57',
      seashell: '#fff5ee',
      sienna: '#a0522d',
      silver: '#c0c0c0',
      skyblue: '#87ceeb',
      slateblue: '#6a5acd',
      slategray: '#708090',
      slategrey: '#708090',
      snow: '#fffafa',
      springgreen: '#00ff7f',
      steelblue: '#4682b4',
      tan: '#d2b48c',
      teal: '#008080',
      thistle: '#d8bfd8',
      tomato: '#ff6347',
      turquoise: '#40e0d0',
      violet: '#ee82ee',
      wheat: '#f5deb3',
      white: '#ffffff',
      whitesmoke: '#f5f5f5',
      yellow: '#ffff00',
      yellowgreen: '#9acd32',
    };

    if (typeof colors[colorName.toLowerCase()] !== 'undefined') {
      return colors[colorName.toLowerCase()];
    }

    return false;
  }

  arc.canteen = arc.canteen || {};

  arc.canteen.countMethod = function (stack, methodName) {
    let cnt = 0;

    stack.forEach((c) => {
      if (c.method === methodName) {
        cnt += 1;
      }
    });

    return cnt;
  };

  arc.canteen.methodArgs = function (stack, methodName) {
    const args = [];

    stack.forEach((c) => {
      if (c.method === methodName) {
        args.push(c.arguments);
      }
    });

    return args;
  };

  arc.canteen.attrVals = function (stack, attrName) {
    const args = [];

    stack.forEach((c) => {
      if (c.attr === attrName) {
        args.push(c.val);
      }
    });

    return args;
  };

  arc.canteen.getCanvasData = function (sel) {
    const plots = [];

    sel.each(function () {
      const context = this.getContext('2d');
      const s = context.stack();
      plots.push(s);
    });

    return plots;
  };

  arc.misc.poll = function (timeout, predicate) {
    const startTime = Date.now();
    const endTime = startTime + timeout;

    return new Promise((resolve, reject) => {
      function check() {
        if (safe_check()) {
          resolve();
        } else if (Date.now() < endTime) {
          setTimeout(check, 100);
        } else {
          reject(new Error(`Polling timed out while waiting for ${predicate}`));
        }
      }

      check();
    });

    function safe_check() {
      try {
        return predicate();
      } catch (e) {
        // eslint-disable-next-line no-console
        console.warn('predicate invocation error', e);
      }
    }
  };
}(QUnit.assert));
