// TODO: Test when there's sort item
// Test sql creation
// Test when there's an alias
// Test when there's a custom expression
// Test when column is an aggregate attribute
// Test that filter elements and other window functions don't appear in modal
// Try with different connection
// Try with each element as base element
// Try with connection that doesn't support window functions

function mockShelf() {
  const s = function () {
    return 'mockShelf';
  };

  s.geoShelf = function () {
    return undefined;
  };

  s.shelf_name = function () {
    return 'some_shelf';
  };

  return s;
}

// arc.misc.modalShown will be mocked - I store the original value to avoid ruining other tests!
const storedModalShown = arc.misc.modalShown;

QUnit.module('utils_analytic_builder_testing', {
  beforeEach() {
    arc.misc.modalShown = () => undefined;
    const qunit = this;
    const dim = arc.report.shelf
      .element()
      .shelf(mockShelf())
      .col_alias('state')
      .expr_hasagg(false)
      .dataset_colname('state')
      .dataset_coltype('STRING')
      .fn_type(arc.report.fn_types.DIMENSION);
    const agg = arc.report.shelf
      .element()
      .shelf(mockShelf())
      .col_alias('population')
      .expr_hasagg(true)
      .dataset_colname('population')
      .dataset_coltype('BIGINT')
      .fn_type(arc.report.fn_types.AGGREGATE)
      .fn_name('sum');

    qunit.record = arc.report.shelf.record_count_element();
    qunit.record.shelf(mockShelf());
    qunit.shelves = arc.report.shelf.shelves();
    qunit.conn = arc.dataconnection.types.arcengine();
    qunit.colum_name = this.record.dataset_colname();

    d3.selectAll('#app-shelves')
      .selectAll('.selected-list-wrapper .selection_field')
      .data([dim, agg, this.record])
      .enter()
      .append('div')
      .classed('selected-list-wrapper', true)
      .append('div')
      .classed('selection_field', true)
      .html(function (d) {
        if (d.col_alias() === 'Record Count') {
          qunit.d3_elem = d3.select(this);
          qunit.compareResults = function (assert, expectedValue) {
            $('#create-function').click();
            assert.equal(d.getExpressionForEdit(), expectedValue);
          };
        }

        return d.getComputedDisplayName();
      });

    this.iAnalyticalBuilder = arc.report.analyticalBuilder({
      shelves: qunit.shelves,
      d3_elem: qunit.d3_elem,
      connection: qunit.conn,
      currentElName: qunit.colum_name,
    });
  },
  afterEach() {
    $('.selected-list-wrapper').remove();
    $('.analytical-sort-options').empty();
    arc.misc.modalShown = storedModalShown;
  },
});

QUnit.test('Correct components are defined', function (assert) {
  assert.equal(
    this.conn.supportsWindowFunctions,
    true,
    'The data connection supports window functions',
  );
  assert.equal(
    this.conn.connection_type,
    'arcengine',
    'The connection is arcengine',
  );
  assert.equal(
    this.d3_elem.datum(),
    this.record,
    'The element is the record element',
  );
  assert.equal(
    this.colum_name,
    'Record Count',
    'The element is called Record Count',
  );
});

QUnit.test('Previous Value', function (assert) {
  const done = assert.async();
  this.iAnalyticalBuilder.createPreviousValueModal();
  setTimeout(() => {
    // The firt modal will open after 300 miliseconds because of tranistion
    // Afterwads the modal is not closed but repurposed
    comparePartitionSection(assert, 'take previous value');
    compareSortSection(assert);
    compareOffsetSection(assert);
    compareNumberSection(assert, true);
    compareAggregateSection(assert);
    this.compareResults(
      assert,
      "LAG(sum(1), 1) over ( order by [state]) as 'previous value(Record Count)'",
    );
    done();
  }, 300);
});

QUnit.test('Difference from Previous', function (assert) {
  this.iAnalyticalBuilder.createPreviousDifferenceModal();
  comparePartitionSection(assert, 'take difference from previous value');
  compareSortSection(assert);
  compareOffsetSection(assert);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    "sum(1) - LAG(sum(1), 1) over ( order by [state]) as 'diff from prev(Record Count)'",
  );
});

QUnit.test('% Difference from Previous', function (assert) {
  this.iAnalyticalBuilder.createPreviousDifferencePctModal();
  comparePartitionSection(assert, 'take % difference from previous value');
  compareSortSection(assert);
  compareOffsetSection(assert);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    '(sum(1) - LAG(sum(1), 1) over ( ' +
      "order by [state])) / LAG(sum(1), 1) over ( order by [state]) as 'pct diff from prev(Record Count)'",
  );
});

QUnit.test('Next Value', function (assert) {
  this.iAnalyticalBuilder.createNextValueModal();
  comparePartitionSection(assert, 'take next value');
  compareSortSection(assert);
  compareOffsetSection(assert);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    "LEAD(sum(1), 1) over ( order by [state]) as 'next value(Record Count)'",
  );
});

QUnit.test('Difference from Next', function (assert) {
  this.iAnalyticalBuilder.createNextDifferenceModal();
  comparePartitionSection(assert, 'take difference from next value');
  compareSortSection(assert);
  compareOffsetSection(assert);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    "sum(1) - LEAD(sum(1), 1) over ( order by [state]) as 'diff from next(Record Count)'",
  );
});

QUnit.test('% Difference from Next', function (assert) {
  this.iAnalyticalBuilder.createNextDifferencePctModal();
  comparePartitionSection(assert, 'take % difference from next value');
  compareSortSection(assert);
  compareOffsetSection(assert);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    '(sum(1) - LEAD(sum(1), 1) over ( ' +
      "order by [state])) / LEAD(sum(1), 1) over ( order by [state]) as 'pct diff from next(Record Count)'",
  );
});

QUnit.test('Moving Average', function (assert) {
  this.iAnalyticalBuilder.createMovingAverageModal();
  comparePartitionSection(assert, 'create moving averages');
  compareSortSection(assert);
  compareNumberSection(assert);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    "avg(sum(1)) over ( order by [state] rows between 5 preceding and current row) as 'moving average(Record Count)'",
  );
});

QUnit.test('Percent of Total', function (assert) {
  this.iAnalyticalBuilder.createPercentTotalModal();
  comparePartitionSection(assert, 'calculate percentage');
  compareSortSection(assert, true);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    "sum(1) / sum(sum(1)) over () as 'percent(Record Count)'",
  );
});

QUnit.test('Running Total', function (assert) {
  this.iAnalyticalBuilder.createRunningTotalModal();
  comparePartitionSection(assert, 'take running total');
  compareSortSection(assert);
  compareNumberSection(assert, true);
  compareAggregateSection(assert);
  this.compareResults(
    assert,
    "sum(sum(1)) over ( order by [state] rows between unbounded preceding and current row) as 'running total(Record Count)'",
  );
});

// Tests for each modal section
function comparePartitionSection(assert, type) {
  const $partitonSection = $('.analytical-partition.analytical-section');
  assert.equal(
    $partitonSection.find('#analytical-partition-label').text(),
    `Select entities over which to ${type} (optional)`,
    'Partition title is as expected',
  );
  assert.equal(
    $partitonSection.find('input').val(),
    '[state]',
    'State is the only partition option (dim only)',
  );
  assert.equal(
    $partitonSection.find('#analytical-partition-options label').text().trim(),
    'state',
    'State is properly labeled',
  );
}

function compareSortSection(assert, isHidden) {
  const $sortSection = $('.analytical-sort.analytical-section');
  if (isHidden) {
    assert.equal($sortSection.is(':visible'), false, 'Sort section is hidden');
  } else {
    const $button = $sortSection.find('button');
    const expectedLabels = ['None', 'Record Count', 'state', 'population'];
    const expectedValues = [
      'None',
      '_CURRENT_COLUMN_',
      '[state]',
      'sum([population])',
    ];
    assert.equal($sortSection.is(':visible'), true, 'Sort section is visible');
    assert.equal(
      $sortSection.find('#analytical-sort-label').text(),
      '* Select the sort order',
      'Sort title is as expected',
    );
    assert.equal(
      $sortSection.find('.analytical-sort-options select').length,
      1,
      'Sort has one element',
    );
    $button.trigger('click');
    assert.equal($button.is(':visible'), true, 'User can add 2 sort item');
    $button.trigger('click');
    assert.equal($button.is(':visible'), false, "User can't add 3 sort item");
    assert.equal(
      $sortSection.find('.form-group').length,
      3,
      'There are 3 sort options',
    );
    $sortSection.find('.delete-sort').last().trigger('click');
    assert.equal(
      $button.is(':visible'),
      true,
      'Add sort button reappears after delete',
    );
    assert.equal(
      $sortSection.find('.form-group').length,
      2,
      'One sort option is deleted',
    );

    $sortSection.find('.form-group select option').each((i, e) => {
      assert.equal(
        e.value,
        expectedValues[i % expectedValues.length],
        `#${i} option has correct value`,
      );
      assert.equal(
        e.textContent,
        expectedLabels[i % expectedLabels.length],
        `#${i} option has correct label`,
      );
    });
    const $orderI = $sortSection.find('.order-number').last().find('i');
    assert.equal(
      $orderI.attr('class'),
      'fa fa-sort-amount-asc asc',
      'Clicking i triggers sort order change',
    );
    $orderI.trigger('click');
    assert.equal(
      $orderI.attr('class'),
      'fa fa-sort-amount-desc desc',
      'Clicking i triggers sort order change',
    );
  }
}

function compareOffsetSection(assert, isHidden) {
  const $offsetSection = $('.analytical-offset.analytical-section');
  if (isHidden) {
    assert.equal(
      $offsetSection.is(':visible'),
      false,
      'Offset section is hidden',
    );
  } else {
    assert.equal(
      $offsetSection.is(':visible'),
      true,
      'Number section is visible',
    );
    assert.equal($offsetSection.find('#previous-offset-row-number').val(), '1');
  }
}

function compareNumberSection(assert, isHidden) {
  const $numberSection = $('.analytical-number.analytical-section');
  if (isHidden) {
    assert.equal(
      $numberSection.is(':visible'),
      false,
      'Number section is hidden',
    );
  } else {
    const expectedValues = ['', '5', '0', '5'];
    // var expectedVisibility = [true, true, true, false];
    assert.equal(
      $numberSection.is(':visible'),
      true,
      'Number section is visible',
    );
    assert.equal(
      $numberSection.find('#analytical-number-label').text(),
      '* Rows to include in the average (relative to current row)',
      'Number title is correct',
    );
    $numberSection.find('select, input').each((i, e) => {
      assert.equal(
        e.value,
        expectedValues[i],
        `#${i} default value is correct`,
      );
      // assert.equal(e.hidden, expectedVisibility[i], "#" + i + " default visibility is correct");
    });
  }
}

function compareAggregateSection(assert) {
  const $aggSection = $('.analytical-aggregate.analytical-section');
  const expectedValues = [
    'sum',
    'count',
    'ndv',
    'count(distinct',
    'min',
    'max',
    'avg',
  ];
  assert.equal(
    $aggSection.is(':visible'),
    false,
    'Aggregate is hidden for Record Count',
  );
  $aggSection.find('option').each((i, e) => {
    assert.equal(e.value, expectedValues[i], '#" + i + " option is correct');
  });
}
