QUnit.module('visuals_arc_area', {
  beforeEach() {
    // main chart object
    const area = arc.visual.plugins.area();

    // create the lib object
    const driver = arc.report.driver(area);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Palo Alto', '2000', 'CA', 9000],
      ['San Mateo', '2000', 'CA', 0],
      ['Raleigh', '2000', 'NC', -2000],
      ['Chapel Hill', '2000', 'NC', '4356', 1000],
      ['Palo Alto', '1990', 'CA', 1123],
      ['San Mateo', '1990', 'CA', 12],
      ['Raleigh', '1990', 'NC', 24],
      ['Chapel Hill', '1990', 'NC', 100],
    ]);
    const col1 = arc.data
      .column()
      .colname('year')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col3 = arc.data
      .column()
      .colname('state')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(2);
    const col4 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(3);
    result.columns([col1, col2, col3, col4]);

    // create the visual by setting the data and then drawing it
    area.viz_setDataResult(result);
    area.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.labels = $('table>tbody>tr>td.viz-td-header');
    this.canvas = $('.viz-trellis-div>.arc-viz-overlay-container>canvas');
    this.canvasMain = this.canvas.filter('.maincanvas');
    this.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
    this.xAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
    );
    this.yAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
    );
    this.axisLabel = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .arc-viz-axis-label',
    );

    this.canvasSel = d3.selectAll(
      '.viz-trellis-div>.arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Area Plot and Legend Elements Present', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');

  assert.equal(
    this.svg.length,
    7,
    'There are 7 svg elements (2 trellis cells with graph + x-axis and 1 legend and clippath)',
  );
  assert.equal(this.labels.length, 3, 'There are 3 td label elements');
  assert.equal(
    this.canvas.length,
    6,
    'There are 2 g elements that contains the charts',
  );
  assert.equal(
    this.legendG.length,
    2,
    'There is one g element that contains the legend',
  );

  assert.equal(this.xAxis.length, 2, 'There are 2 x axis elements');
  assert.equal(this.yAxis.length, 2, 'There are 2 y axis elements');
  assert.equal(this.axisLabel.length, 4, 'There are 4 axis label element');
  assert.equal(
    xAxisTicks.length,
    8,
    'There are 8 marks total on the two x axis',
  );
  assert.equal(
    yAxisTicks.length,
    12,
    'There are 12 marks total on the two y axis',
  );

  assert.equal(this.canvasMain.length, 2, 'There are 2 main canvas elements');
  assert.equal(arc.canteen.countMethod(this.canvasData[0], 'beginPath'), 2);
  assert.equal(arc.canteen.countMethod(this.canvasData[1], 'beginPath'), 2);
});

QUnit.test('Area Plot Elements Correct', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');
  const xAxisLabels = [];
  const yAxisLabels = [];
  const axisLabel = [];
  const trellisLabels = [];
  const xAxisLabelsExpected = [
    'Chapel Hill',
    'Palo Alto',
    'Raleigh',
    'San Mateo',
    'Chapel Hill',
    'Palo Alto',
    'Raleigh',
    'San Mateo',
  ];
  const yAxisLabelsExpected = [
    '-2,000',
    '0',
    '2,000',
    '4,000',
    '6,000',
    '8,000',
    '-2,000',
    '0',
    '2,000',
    '4,000',
    '6,000',
    '8,000',
  ];
  const axisLabelExpected = ['city', 'population', 'city', 'population'];
  const trellisLabelsExpected = ['year', '2000', '1990'];
  const areaColorsExpected = ['#0097a7', '#7dcad2'];
  let i;

  for (i = 0; i < xAxisTicks.length; i++) {
    xAxisLabels.push(xAxisTicks[i].textContent);
  }

  for (i = 0; i < yAxisTicks.length; i++) {
    yAxisLabels.push(yAxisTicks[i].textContent);
  }

  for (i = 0; i < this.labels.length; i++) {
    trellisLabels.push(this.labels[i].textContent);
  }

  for (i = 0; i < this.axisLabel.length; i++) {
    axisLabel.push(this.axisLabel[i].textContent);
  }

  assert.deepEqual(
    xAxisLabels,
    xAxisLabelsExpected,
    'The x axis labels are correct',
  );
  assert.deepEqual(
    yAxisLabels,
    yAxisLabelsExpected,
    'The y axis labels are correct',
  );
  assert.deepEqual(axisLabel, axisLabelExpected, 'The axis labels are correct');
  assert.deepEqual(
    trellisLabels,
    trellisLabelsExpected,
    'The trellis labels are correct',
  );

  assert.equal(this.canvasData.length, 2);

  this.canvasData.forEach((d) => {
    assert.deepEqual(arc.canteen.attrVals(d, 'fillStyle'), areaColorsExpected);
    assert.deepEqual(arc.canteen.attrVals(d, 'fillStyle'), areaColorsExpected);

    assert.equal(arc.canteen.countMethod(d, 'moveTo'), 2);
    assert.equal(arc.canteen.countMethod(d, 'lineTo'), 14);
  });

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[0], 'moveTo'),
      [
        [18.75, 263.727],
        [18.75, 136.083],
      ],
    ),
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[0], 'lineTo'),
      [
        [56.25, 0],
        [93.75, 263.727],
        [131.25, 263.727],
        [131.25, 263.727],
        [93.75, 263.727],
        [56.25, 263.727],
        [18.75, 263.727],
        [56.25, 0],
        [93.75, 322.333],
        [131.25, 263.727],
        [131.25, 263.727],
        [93.75, 263.727],
        [56.25, 0],
        [18.75, 263.727],
      ],
    ),
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[1], 'moveTo'),
      [
        [18.75, 263.727],
        [18.75, 260.797],
      ],
    ),
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[1], 'lineTo'),
      [
        [56.25, 230.82],
        [93.75, 263.727],
        [131.25, 263.376],
        [131.25, 263.727],
        [93.75, 263.727],
        [56.25, 263.727],
        [18.75, 263.727],
        [56.25, 230.82],
        [93.75, 263.024],
        [131.25, 263.376],
        [131.25, 263.376],
        [93.75, 263.727],
        [56.25, 230.82],
        [18.75, 263.727],
      ],
    ),
  );
});

QUnit.test('Legend Elements Correct', function (assert) {
  const rectangles = this.legendG[0].getElementsByTagName('rect');
  const labels = this.legendG[0].getElementsByTagName('text');
  const rectangleColors = [];
  const labelTexts = [];
  const rectangleColorsExpected = ['#0097a7', '#7dcad2'];
  const labelTextsExpected = ['CA', 'NC'];
  let i;

  assert.equal(rectangles.length, 2, 'The legend has 2 rectangles');
  assert.equal(labels.length, 2, 'The legend has 2 labels');

  for (i = 0; i < rectangles.length; i++) {
    rectangleColors.push(rectangles[i].style.fill);
    labelTexts.push(labels[i].textContent);
  }

  for (i = 0; i < rectangleColors.length; i++) {
    rectangleColors[i] = arc.misc.colorToHex(rectangleColors[i]);
  }

  assert.deepEqual(
    rectangleColors,
    rectangleColorsExpected,
    'The legend has the correctly colored rectangles',
  );
  assert.deepEqual(
    labelTexts,
    labelTextsExpected,
    'The legend has the correct labels',
  );
});
