QUnit.module('visuals_arc_visual_bar_ignore_missing_dates', {
  afterEach: () => {
    $('#chart-div').empty();
  },
});

QUnit.test(
  'When ignore missing dates is set, the line should be smooth',
  (assert) => {
    createChartByTypeAndIgnoringMissingDatesOption('line', 'true');
    const canvasData = getCanvasData();

    canvasData.forEach((d) => {
      assert.equal(arc.canteen.countMethod(d, 'moveTo'), 1);
      assert.equal(arc.canteen.countMethod(d, 'lineTo'), 3);
    });
    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(canvasData[0], 'lineTo'),
        [
          [57.029, 174],
          [228.115, 174],
          [285.143, 0],
        ],
      ),
    );
  },
);

QUnit.test(
  'When ignore missing dates is not set, the line should drive into the x axis when no data is provided at the specific date(s)',
  (assert) => {
    createChartByTypeAndIgnoringMissingDatesOption('line', 'false');
    const canvasData = getCanvasData();

    canvasData.forEach((d) => {
      assert.equal(arc.canteen.countMethod(d, 'moveTo'), 1);
      assert.equal(arc.canteen.countMethod(d, 'lineTo'), 5);
    });
    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(canvasData[0], 'lineTo'),
        [
          [57.029, 174],
          // the next two coordinates' y (second) coordinate shall be the same in this case!
          // these are the points where the graph crosses the x axis
          // as the data is these dates are missing(2014-01-11, 2014-01-12)
          [114.057, 435],
          [171.086, 435],
          [228.115, 174],
          [285.143, 0],
        ],
      ),
    );
  },
);

QUnit.test(
  'When ignore missing dates is set, the area chart should be smooth',
  (assert) => {
    createChartByTypeAndIgnoringMissingDatesOption('area', 'true');
    const canvasData = getCanvasData();

    canvasData.forEach((d) => {
      assert.equal(arc.canteen.countMethod(d, 'moveTo'), 1);
      assert.equal(arc.canteen.countMethod(d, 'lineTo'), 7);
    });
    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(canvasData[0], 'lineTo'),
        [
          [57.029, 174],
          [228.115, 174],
          [285.143, 0],
          [285.143, 435],
          [228.115, 435],
          [57.029, 435],
          [0, 435],
        ],
      ),
    );
  },
);

QUnit.test(
  'When ignore missing dates is not set, the area chart should drive into the x axis when no data is provided at the specific date(s)',
  (assert) => {
    createChartByTypeAndIgnoringMissingDatesOption('area', 'false');
    const canvasData = getCanvasData();

    canvasData.forEach((d) => {
      assert.equal(arc.canteen.countMethod(d, 'moveTo'), 1);
      assert.equal(arc.canteen.countMethod(d, 'lineTo'), 11);
    });
    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(canvasData[0], 'lineTo'),
        [
          [57.029, 174],
          [114.057, 435],
          [171.086, 435],
          [228.115, 174],
          [285.143, 0],
          [285.143, 435],
          [228.115, 435],
          [171.086, 435],
          [114.057, 435],
          [57.029, 435],
          [0, 435],
        ],
      ),
    );
  },
);

QUnit.test(
  'When ignore missing dates is not set, the combo chart should drive into the x axis when no data is provided at the specific date(s)',
  (assert) => {
    createChartByTypeAndIgnoringMissingDatesOption('combo', 'false');
    const canvasData = getCanvasData();

    canvasData.forEach((d) => {
      assert.equal(arc.canteen.countMethod(d, 'moveTo'), 1);
      assert.equal(arc.canteen.countMethod(d, 'lineTo'), 5);
    });

    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(canvasData[0], 'lineTo'),
        [
          [114.335, 174],
          [171.502, 435],
          [228.67, 435],
          [285.837, 174],
          [343.004, 0],
        ],
      ),
    );
  },
);

QUnit.test(
  'When ignore missing dates is set, the line on the combo chart should be smooth',
  (assert) => {
    createChartByTypeAndIgnoringMissingDatesOption('combo', 'true');
    const canvasData = getCanvasData();
    canvasData.forEach((d) => {
      assert.equal(arc.canteen.countMethod(d, 'moveTo'), 1);
      assert.equal(arc.canteen.countMethod(d, 'lineTo'), 3);
    });
    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(canvasData[0], 'lineTo'),
        [
          [114.335, 174],
          [285.837, 174],
          [343.004, 0],
        ],
      ),
    );
  },
);

function createChartByTypeAndIgnoringMissingDatesOption(
  chartType,
  ignoreMissingDates,
) {
  // main chart object
  let chart;
  let yColumnSource;
  switch (chartType) {
    case 'line':
      chart = arc.visual.plugins.tline();
      yColumnSource = 'y_shelf';
      break;
    case 'area':
      chart = arc.visual.plugins.area();
      yColumnSource = 'y_shelf';
      break;
    case 'combo':
      chart = arc.visual.plugins.combined();
      yColumnSource = 'line_measure';
      break;
    default:
      throw new Error('Not Implemented');
  }

  // create the lib object
  const driver = arc.report.driver(chart);
  driver.chart_div('chart-div');

  // set wether the missing dates should be ignored or not
  const lineSmoothness = driver.user_settings().find_by_id('smooth_line');
  lineSmoothness.value(ignoreMissingDates);

  // provide the data
  const result = arc.data.result();
  result.rows([
    ['2014-01-09', 50, 1],
    ['2014-01-10', 30, 1],
    // 2014-01-11 and 2014-01-12 are the missing DATE
    ['2014-01-13', 30, 1],
    ['2014-01-14', 50, 1],
  ]);
  const col1 = arc.data
    .column()
    .colname('year')
    .coltype('TIMESTAMP')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('x_shelf')
    .colindex(0);
  const col2 = arc.data
    .column()
    .colname('color')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('color_shelf')
    .colindex(2);
  const col3 = arc.data
    .column()
    .colname('population')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource(yColumnSource)
    .colindex(1);
  result.columns([col1, col3, col2]);

  // create the visual by setting the data and then drawing it
  chart.viz_setDataResult(result);
  chart.viz_drawVisual('chart-div');
}

function getCanvasData() {
  const canvasSel = d3.selectAll(
    '.arc-viz-overlay-container>canvas.maincanvas',
  );
  const canvasData = arc.canteen.getCanvasData(canvasSel);
  return canvasData;
}
