QUnit.module('visuals_arc_bar_stacked', {
  beforeEach() {
    // main chart object
    const bar = arc.visual.plugins.bar();
    this.bar = bar;

    // create the lib object
    this.driver = arc.report.driver(bar).chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Palo Alto', '2000', 'CA', 9000],
      ['San Mateo', '2000', 'CA', 0],
      ['Raleigh', '2000', 'NC', -2000],
      ['Chapel Hill', '2000', 'NC', '4356', 1000],
      ['Palo Alto', '1990', 'CA', 1123],
      ['San Mateo', '1990', 'CA', 12],
      ['Raleigh', '1990', 'NC', 24],
      ['Chapel Hill', '1990', 'NC', 100],
    ]);
    const col1 = arc.data
      .column()
      .colname('year')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col3 = arc.data
      .column()
      .colname('state')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(2);
    const col4 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(3);
    result.columns([col1, col2, col3, col4]);

    // create the visual by setting the data and then drawing it
    bar.viz_setDataResult(result);
    bar.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.labels = $('table>tbody>tr>td.viz-td-header');
    this.canvas = $('.viz-trellis-div>.arc-viz-overlay-container>canvas');
    this.canvasMain = this.canvas.filter('.maincanvas');
    this.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
    this.xAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
    );
    this.yAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
    );
    this.axisLabel = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .arc-viz-axis-label',
    );

    this.canvasSel = d3.selectAll(
      '.viz-trellis-div>.arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Bar Plot and Legend Elements Present', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');

  assert.equal(
    this.svg.length,
    7,
    'There are 7 svg elements (2 trellis cells with graph + x-axis and 1 legend and clippath)',
  );
  assert.equal(this.labels.length, 3, 'There are 3 td label elements');
  assert.equal(this.canvas.length, 6, 'There are 6 canvas elements');
  assert.equal(this.canvasMain.length, 2, 'There are 2 main canvas elements');
  assert.equal(
    this.legendG.length,
    2,
    'There are two g elements that contains the legend',
  );

  assert.equal(this.xAxis.length, 2, 'There are 2 x axis elements');
  assert.equal(this.yAxis.length, 2, 'There are 2 y axis elements');
  assert.equal(this.axisLabel.length, 4, 'There are 4 axis label element');
  assert.equal(
    xAxisTicks.length,
    8,
    'There are 8 marks total on the two x axis',
  );
  assert.equal(
    yAxisTicks.length,
    12,
    'There are 12 marks total on the two y axis',
  );

  assert.equal(arc.canteen.countMethod(this.canvasData[0], 'beginPath'), 8);
  assert.equal(arc.canteen.countMethod(this.canvasData[1], 'beginPath'), 8);
});

QUnit.test('Bar Plot Elements Correct', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');
  const xAxisLabels = [];
  const yAxisLabels = [];
  const axisLabel = [];
  const trellisLabels = [];
  const xAxisLabelsExpected = [
    'Chapel Hill',
    'Palo Alto',
    'Raleigh',
    'San Mateo',
    'Chapel Hill',
    'Palo Alto',
    'Raleigh',
    'San Mateo',
  ];
  const yAxisLabelsExpected = [
    '-2,000',
    '0',
    '2,000',
    '4,000',
    '6,000',
    '8,000',
    '-2,000',
    '0',
    '2,000',
    '4,000',
    '6,000',
    '8,000',
  ];
  const axisLabelExpected = ['city', 'population', 'city', 'population'];
  const trellisLabelsExpected = ['year', '2000', '1990'];
  const barColorsExpected = ['#0097a7', '#7dcad2'];

  for (let i = 0; i < xAxisTicks.length; i++) {
    xAxisLabels.push(xAxisTicks[i].textContent);
  }

  for (let i = 0; i < yAxisTicks.length; i++) {
    yAxisLabels.push(yAxisTicks[i].textContent);
  }

  for (let i = 0; i < this.labels.length; i++) {
    trellisLabels.push(this.labels[i].textContent);
  }

  for (let i = 0; i < this.axisLabel.length; i++) {
    axisLabel.push(this.axisLabel[i].textContent);
  }

  assert.deepEqual(
    xAxisLabels,
    xAxisLabelsExpected,
    'The x axis labels are correct',
  );

  assert.deepEqual(
    yAxisLabels,
    yAxisLabelsExpected,
    'The y axis labels are correct',
  );
  assert.deepEqual(axisLabel, axisLabelExpected, 'The axis labels are correct');
  assert.deepEqual(
    trellisLabels,
    trellisLabelsExpected,
    'The trellis labels are correct',
  );

  assert.equal(this.canvasData.length, 2);

  this.canvasData.forEach((d) => {
    assert.deepEqual(arc.canteen.attrVals(d, 'strokeStyle'), ['white']);
    assert.deepEqual(arc.canteen.attrVals(d, 'fillStyle'), barColorsExpected);

    assert.equal(arc.canteen.countMethod(d, 'beginPath'), 8);
    assert.equal(arc.canteen.countMethod(d, 'rect'), 8);
    assert.equal(arc.canteen.countMethod(d, 'fill'), 8);
    assert.equal(arc.canteen.countMethod(d, 'stroke'), 8);
  });

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[0], 'rect'),
      [
        [0, 263.727, 37.5, 0],
        [37.5, 263.727, 37.5, -263.727],
        [75, 263.727, 37.5, 0],
        [112.5, 263.727, 37.5, 0],
        [0, 263.727, 37.5, -127.644],
        [37.5, 0, 37.5, 0],
        [75, 263.727, 37.5, -60],
        [112.5, 263.727, 37.5, 0],
      ],
    ),
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[1], 'rect'),
      [
        [0, 263.727, 37.5, 0],
        [37.5, 263.727, 37.5, -32.907],
        [75, 263.727, 37.5, 0],
        [112.5, 263.727, 37.5, -0.352],
        [0, 263.727, 37.5, -2.93],
        [37.5, 230.82, 37.5, 0],
        [75, 263.727, 37.5, -0.703],
        [112.5, 263.376, 37.5, 0],
      ],
    ),
  );
});

QUnit.test('Bar Plot Elements Correct Are In Correct Order', function (assert) {
  const data = [
    ['Africa', '1-Before WW2', 5],
    ['Africa', '2-After WW2', 15],
    ['Americas', '1-Before WW2', 20],
    ['Americas', '2-After WW2', 25],
    ['Americas', '3-Current', 30],
  ];

  [
    {
      ascending: true,
      expectedCoordinates: [
        [0, 395.04, 157.523, -26.336],
        [157.523, 395.04, 157.523, -105.344],
        [0, 368.704, 157.523, -79.008],
        [157.523, 289.696, 157.523, -131.68],
        [0, 289.696, 157.523, 0],
        [157.523, 158.016, 157.523, -158.016],
      ],
      expectedColorLegend: ['3-Current', '2-After WW2', '1-Before WW2'],
    },
    {
      ascending: false,
      expectedCoordinates: [
        [0, 395.04, 157.523, 0],
        [157.523, 395.04, 157.523, -158.016],
        [0, 395.04, 157.523, -79.008],
        [157.523, 237.024, 157.523, -131.68],
        [0, 316.032, 157.523, -26.336],
        [157.523, 105.344, 157.523, -105.344],
      ],
      expectedColorLegend: ['1-Before WW2', '2-After WW2', '3-Current'],
    },
  ].forEach((testParams) => {
    $('#chart-div').empty();
    // main chart object
    const bar = arc.visual.plugins.bar();
    this.bar = bar;

    // create the lib object
    this.driver = arc.report.driver(bar).chart_div('chart-div');

    // provide the data
    const result = arc.data.result();

    result.rows([...data]);
    const col1 = arc.data
      .column()
      .colname('un_region')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);

    const col2 = arc.data
      .column()
      .colname('interval')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(1)
      .order({
        priority: () => 2,
        ascending: () => testParams.ascending,
      });

    const col3 = arc.data
      .column()
      .colname('life_expectancy')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(2);

    result.columns([col1, col2, col3]);

    // create the visual by setting the data and then drawing it
    bar.viz_setDataResult(result);
    bar.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.xAxis = $(
      '.viz-trellis-outertable .arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
    );
    this.yAxis = $(
      '.viz-trellis-outertable .arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
    );
    this.colorLegend = $('.viz-base-legend-g.small-legend text')
      .toArray()
      .map((x) => $(x).text());
    this.canvasSel = d3.selectAll(
      '.viz-trellis-outertable .arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);

    assert.ok(
      arc.misc.deep_array_within_tolerance(
        arc.canteen.methodArgs(this.canvasData[0], 'rect'),
        testParams.expectedCoordinates,
        10,
      ),
    );
    const xAxisTicks = this.xAxis.find('.tick');
    const xAxisLabels = [];
    const colorLegend = [];
    const xAxisLabelsExpected = ['Africa', 'Americas'];

    for (let i = 0; i < xAxisTicks.length; i++) {
      xAxisLabels.push(xAxisTicks[i].textContent);
    }

    for (let i = 0; i < this.axisLabel.length; i++) {
      colorLegend.push(this.axisLabel[i].textContent);
    }

    assert.deepEqual(
      xAxisLabels,
      xAxisLabelsExpected,
      'The x axis labels are correct',
    );
    assert.deepEqual(colorLegend, colorLegend, 'The axis labels are correct');
  });
});

QUnit.test('Legend Elements Correct', function (assert) {
  const rectangles = this.legendG[0].getElementsByTagName('rect');
  const labels = this.legendG[0].getElementsByTagName('text');
  const rectangleColors = [];
  const labelTexts = [];
  const rectangleColorsExpected = ['#0097a7', '#7dcad2'];
  const labelTextsExpected = ['CA', 'NC'];

  assert.equal(rectangles.length, 2, 'The legend has 2 rectangles');
  assert.equal(labels.length, 2, 'The legend has 2 labels');

  for (let i = 0; i < rectangles.length; i++) {
    rectangleColors.push(arc.misc.colorToHex(rectangles[i].style.fill));
    labelTexts.push(labels[i].textContent);
  }

  assert.deepEqual(
    rectangleColors,
    rectangleColorsExpected,
    'The legend has the correctly colored rectangles',
  );
  assert.deepEqual(
    labelTexts,
    labelTextsExpected,
    'The legend has the correct labels',
  );
});

/* Here we add some setting tests that require the use of a visual */
QUnit.test('Reverse Legend Setting: Discrete', function (assert) {
  const reverse_legend_order = this.driver
    .user_settings()
    .find_by_id('reverse_legend_order');

  function testLegend() {
    const $legend = $('.viz-base-legend-g.small-legend');
    const rectangles = $legend.find('rect');
    const labels = $legend.find('text');
    const rectangleColors = [];
    const labelTexts = [];
    const rectangleColorsExpected = ['#0097a7', '#7dcad2'];
    const labelTextsExpected = ['CA', 'NC'];
    const reverseLegendOrder = reverse_legend_order.value() === 'true';

    for (let i = 0; i < rectangles.length; i++) {
      rectangleColors.push(arc.misc.colorToHex(rectangles[i].style.fill));
      labelTexts.push(labels[i].textContent);
    }

    if (reverseLegendOrder) {
      rectangleColorsExpected.reverse();
      labelTextsExpected.reverse();
    }

    assert.deepEqual(
      rectangleColors,
      rectangleColorsExpected,
      `Rectangles colors are correct ` +
        `when reverse legend setting is ${reverseLegendOrder}`,
    );
    assert.deepEqual(
      labelTexts,
      labelTextsExpected,
      `Legend labels are correct when ` +
        `reverse legend setting is ${reverseLegendOrder}`,
    );
  }

  const { bar } = this;
  reverse_legend_order.value('false');
  bar.viz_drawVisual('chart-div');

  testLegend(this);
  $('#chart-div').empty();

  reverse_legend_order.value('true');
  bar.viz_drawVisual('chart-div');
  testLegend(this);
});
