QUnit.module('visuals_arc_bar_trellis', {
  beforeEach() {
    // main chart object
    const bar = arc.visual.plugins.bar().mark_type('bar');
    this.bar = bar;

    // create the lib object
    this.driver = arc.report.driver(bar).chart_div('chart-div');
  },

  afterEach() {
    $('#chart-div').height(500).empty();
  },
});

function setSettingValue(context, id, value) {
  context.bar.viz_lib().user_settings().find_by_id(id).value(value);
}

function drawTrellisBar(context, bar) {
  // provide the data
  const result = arc.data.result();
  result.rows([
    ['ABW', 'Communication', 1],
    ['ABW', 'Web Services', 1],
    ['AFG', 'Web Services', 1],
    ['ARE', 'Communication', 5],
    ['ARE', 'Web Services', 7],
    ['AUS', 'Communication', 4],
    ['AUS', 'Web Services', 5],
  ]);
  const col1 = arc.data
    .column()
    .colname('country')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('x_shelf')
    .colindex(0);
  const col2 = arc.data
    .column()
    .colname('industry')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('y_shelf')
    .colindex(1);
  const col3 = arc.data
    .column()
    .colname('Record Count')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('y_shelf')
    .colindex(2);

  result.columns([col1, col2, col3]);

  // create the visual by setting the data and then drawing it
  bar.viz_setDataResult(result);
  $('#chart-div').height(477);
  bar.viz_drawVisual('chart-div');

  // store elements on the page to be tested
  context.svg = $('.viz-trellis-table svg');
  context.labels = $('table>tbody>tr>td.viz-td-header');
  context.canvas = $('.viz-trellis-div>.arc-viz-overlay-container>canvas');
  context.canvasMain = context.canvas.filter('.maincanvas');
  context.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
  context.xAxis = $(
    '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
  );
  context.yAxis = $(
    '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
  );
  context.axisLabel = $(
    '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .arc-viz-axis-label',
  );

  context.canvasSel = d3.selectAll(
    '.viz-trellis-div>.arc-viz-overlay-container>canvas.maincanvas',
  );
  context.canvasData = arc.canteen.getCanvasData(context.canvasSel);
}

QUnit.test('Bar Plot Present', function (assert) {
  drawTrellisBar(this, this.bar, this.driver);
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');

  assert.equal(
    this.svg.length,
    6,
    'There are 6 svg elements (2 trellis cells with graph + x-axis)',
  );
  assert.equal(this.labels.length, 3, 'There are 3 td label elements');
  assert.equal(this.canvas.length, 6, 'There are 6 canvas elements');
  assert.equal(this.canvasMain.length, 2, 'There are 2 main canvas elements');
  assert.equal(
    this.legendG.length,
    0,
    'There are zero g elements that contains the legend',
  );

  assert.equal(this.xAxis.length, 2, 'There are 2 x axis elements');
  assert.equal(this.yAxis.length, 2, 'There are 2 y axis elements');
  assert.equal(this.axisLabel.length, 4, 'There are 4 axis label element');
  assert.equal(
    xAxisTicks.length,
    8,
    'There are 8 marks total on the two x axis',
  );
  assert.equal(
    yAxisTicks.length,
    4,
    'There are 4 marks total on the two y axis',
  );

  assert.equal(arc.canteen.countMethod(this.canvasData[0], 'beginPath'), 3);
  assert.equal(arc.canteen.countMethod(this.canvasData[1], 'beginPath'), 4);
});

QUnit.test(
  'Bar Plot Elements are correct once the scale_agg_independent setting turned on',
  function (assert) {
    setSettingValue(this, 'scale_agg_independent', 'true');

    drawTrellisBar(this, this.bar, this.driver);

    const xAxisTicks = this.xAxis.find('.tick');
    const yAxisTicks = this.yAxis.find('.tick');
    const xAxisLabels = [];
    const yAxisLabels = [];
    const axisLabel = [];
    const trellisLabels = [];
    const xAxisLabelsExpected = [
      'ABW',
      'AFG',
      'ARE',
      'AUS',
      'ABW',
      'AFG',
      'ARE',
      'AUS',
    ];
    const yAxisLabelsExpected = ['0', '2', '4', '0', '5'];
    const axisLabelExpected = [
      'country',
      'Record Count',
      'country',
      'Record Count',
    ];
    const trellisLabelsExpected = ['industry', 'Communication', 'Web Services'];

    for (let i = 0; i < xAxisTicks.length; i++) {
      xAxisLabels.push(xAxisTicks[i].textContent);
    }

    for (let i = 0; i < yAxisTicks.length; i++) {
      yAxisLabels.push(yAxisTicks[i].textContent);
    }

    for (let i = 0; i < this.labels.length; i++) {
      trellisLabels.push(this.labels[i].textContent);
    }

    for (let i = 0; i < this.axisLabel.length; i++) {
      axisLabel.push(this.axisLabel[i].textContent);
    }

    assert.deepEqual(
      xAxisLabels,
      xAxisLabelsExpected,
      'The x axis labels are correct',
    );
    assert.deepEqual(
      yAxisLabels,
      yAxisLabelsExpected,
      'The y axis labels are correct',
    );
    assert.deepEqual(
      axisLabel,
      axisLabelExpected,
      'The axis labels are correct',
    );
    assert.deepEqual(
      trellisLabels,
      trellisLabelsExpected,
      'The trellis labels are correct',
    );

    assert.equal(this.canvasData.length, 2);
  },
);

QUnit.test(
  'Bar Plot Elements are correct once the scale_dim_independent setting turned on',
  function (assert) {
    setSettingValue(this, 'scale_dim_independent', 'true');

    drawTrellisBar(this, this.bar, this.driver);

    const xAxisTicks = this.xAxis.find('.tick');
    const yAxisTicks = this.yAxis.find('.tick');
    const xAxisLabels = [];
    const yAxisLabels = [];
    const axisLabel = [];
    const trellisLabels = [];
    const xAxisLabelsExpected = [
      'ABW',
      'ARE',
      'AUS',
      'ABW',
      'AFG',
      'ARE',
      'AUS',
    ];
    const yAxisLabelsExpected = ['0', '5', '0', '5'];
    const axisLabelExpected = [
      'country',
      'Record Count',
      'country',
      'Record Count',
    ];
    const trellisLabelsExpected = ['industry', 'Communication', 'Web Services'];

    for (let i = 0; i < xAxisTicks.length; i++) {
      xAxisLabels.push(xAxisTicks[i].textContent);
    }

    for (let i = 0; i < yAxisTicks.length; i++) {
      yAxisLabels.push(yAxisTicks[i].textContent);
    }

    for (let i = 0; i < this.labels.length; i++) {
      trellisLabels.push(this.labels[i].textContent);
    }

    for (let i = 0; i < this.axisLabel.length; i++) {
      axisLabel.push(this.axisLabel[i].textContent);
    }

    assert.deepEqual(
      xAxisLabels,
      xAxisLabelsExpected,
      'The x axis labels are correct',
    );
    assert.deepEqual(
      yAxisLabels,
      yAxisLabelsExpected,
      'The y axis labels are correct',
    );
    assert.deepEqual(
      axisLabel,
      axisLabelExpected,
      'The axis labels are correct',
    );
    assert.deepEqual(
      trellisLabels,
      trellisLabelsExpected,
      'The trellis labels are correct',
    );

    assert.equal(this.canvasData.length, 2);
  },
);

QUnit.test(
  'Bar Plot Elements Correct once the dynamic_bar_width turned on',
  function (assert) {
    drawTrellisBar(this, this.bar, this.driver);

    setSettingValue(this, 'dynamic_bar_width', 'false');
    const notDynamicXaxisWidth = this.xAxis[0].getBoundingClientRect().width;

    setSettingValue(this, 'dynamic_bar_width', 'true');

    drawTrellisBar(this, this.bar, this.driver);

    const dynamicXaxisWidth = this.xAxis[0].getBoundingClientRect().width;

    assert.notDeepEqual(
      notDynamicXaxisWidth,
      dynamicXaxisWidth,
      'Dynamic bar width is correct',
    );
  },
);
