QUnit.module('visuals_arc_bar_unstacked', {
  beforeEach() {
    // main chart object
    const bar = arc.visual.plugins.bar().mark_type('bar');
    this.bar = bar;

    // create the lib object
    const driver = arc.report.driver(bar);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Palo Alto', '1990', 1123],
      ['San Mateo', '1990', 12],
      ['Raleigh', '1990', 24],
      ['Chapel Hill', '1990', 100],
      ['Palo Alto', '2000', 9000],
      ['San Mateo', '2000', 0],
      ['Raleigh', '2000', -1000],
      ['Chapel Hill', '2000', '4356', 1000],
    ]);
    const col1 = arc.data
      .column()
      .colname('year')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col3 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(2);
    result.columns([col1, col2, col3]);

    // create the visual by setting the data and then drawing it
    bar.viz_setDataResult(result);

    this.drawBar();
  },

  afterEach() {
    $('#chart-div').empty();
  },

  drawBar() {
    this.bar.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.labels = $('table>tbody>tr>td.viz-td-header');
    this.canvas = $('.viz-trellis-div>.arc-viz-overlay-container>canvas');
    this.canvasMain = this.canvas.filter('.maincanvas');
    this.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
    this.xAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
    );
    this.yAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
    );
    this.axisLabel = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .arc-viz-axis-label',
    );

    this.canvasSel = d3.selectAll(
      '.viz-trellis-div>.arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },
});

QUnit.test('bar_mark_size setting', function (assert) {
  this.canvasData.forEach((d) => {
    assert.deepEqual(arc.canteen.attrVals(d, 'fillStyle'), [
      '#0097a7',
      '#0097a7',
      '#0097a7',
      '#0097a7',
    ]);

    assert.equal(arc.canteen.countMethod(d, 'beginPath'), 4);
    assert.equal(arc.canteen.countMethod(d, 'rect'), 4);
    assert.equal(arc.canteen.countMethod(d, 'fill'), 4);

    const rectArgs = arc.canteen.methodArgs(d, 'rect');
    rectArgs.forEach((a) => {
      assert.ok(arc.misc.within_tolerance(30, a[2], 3));
    });
  });

  arc.misc.modifySettingValue(
    this.bar.viz_lib().user_settings(),
    'bar_mark_size',
    '50-50',
  );

  this.drawBar();

  this.canvasData.forEach((d) => {
    assert.deepEqual(arc.canteen.attrVals(d, 'fillStyle'), [
      '#0097a7',
      '#0097a7',
      '#0097a7',
      '#0097a7',
    ]);

    assert.equal(arc.canteen.countMethod(d, 'beginPath'), 4);
    assert.equal(arc.canteen.countMethod(d, 'rect'), 4);
    assert.equal(arc.canteen.countMethod(d, 'fill'), 4);

    const rectArgs = arc.canteen.methodArgs(d, 'rect');
    rectArgs.forEach((a) => {
      assert.ok(arc.misc.within_tolerance(50, a[2], 3));
    });
  });
});
