QUnit.module('visuals_arc_calendar_heatmap', {
  beforeEach() {
    // main chart object
    const calendarHeatmap = arc.visual.plugins.chm();

    // create the lib object
    const driver = arc.report.driver(calendarHeatmap);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['2015-03-12', '376'],
      ['2015-03-11', '120'],
      ['2015-03-10', '5'],
      ['2015-03-09', '11230'],
    ]);
    const col1 = arc.data
      .column()
      .colname('Date')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col2 = arc.data
      .column()
      .colname('record_count')
      .coltype('BIGINT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(1);
    result.columns([col1, col2]);

    // create the visual by setting the data and then drawing it
    calendarHeatmap.viz_setDataResult(result);
    calendarHeatmap.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.viz_div = d3.select('#chart-div');
    this.canvas = $('canvas')[0];
    this.context = this.canvas.getContext('2d');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Calendar Heatmap Canvas Correctly Drawn', function (assert) {
  const stack = this.context.stack();
  const translates = [];
  const rotates = [];
  const fillTexts = [];
  const setTransforms = [];
  let beginPaths = 0;
  const rects = [];
  let fills = 0;
  let strokes = 0;
  let moveTos = 0;
  const lineTos = [];
  let attributes = 0;
  const leftovers = [];

  stack.forEach((element) => {
    switch (element.method) {
      case 'translate':
        translates.push(element.arguments);
        break;
      case 'rotate':
        rotates.push(element.arguments);
        break;
      case 'fillText':
        fillTexts.push(element.arguments);
        break;
      case 'setTransform':
        setTransforms.push(element.arguments);
        break;
      case 'beginPath':
        beginPaths += 1;
        break;
      case 'rect':
        rects.push(element.arguments);
        break;
      case 'fill':
        fills += 1;
        break;
      case 'stroke':
        strokes += 1;
        break;
      case 'moveTo':
        moveTos += 1;
        break;
      case 'lineTo':
        lineTos.push(element.arguments);
        break;
      default:
        if (element.attr) {
          attributes += 1;
        } else {
          leftovers.push(element);
        }
    }
  });

  const translatesExpected = [
    [20, 10],
    [-6, 29.387],
    [6, -29.387],
  ];
  const rotatesExpected = [[-1.571], [1.571]];
  const fillTextsExpected = [
    [2015, 0, 0],
    ['Jan', 0, 73.774],
    ['Feb', 33.585, 73.774],
    ['Mar', 67.17, 73.774],
    ['Apr', 109.151, 73.774],
    ['May', 142.736, 73.774],
    ['Jun', 184.717, 73.774],
    ['Jul', 218.302, 73.774],
    ['Aug', 251.887, 73.774],
    ['Sep', 293.868, 73.774],
    ['Oct', 327.453, 73.774],
    ['Nov', 361.038, 73.774],
    ['Dec', 403.019, 73.774],
    ['Mon', 455, 8.396],
    ['Sun', 455, 58.774],
  ];
  const setTransformsExpected = [[1, 0, 0, 1, 0, 0]];

  assert.deepEqual(
    translates,
    translatesExpected,
    'The 3 translate commands are correct',
  );
  assert.deepEqual(rotates, rotatesExpected, 'The 1 rotate command is correct');
  assert.deepEqual(
    fillTexts,
    fillTextsExpected,
    'The 20 fillText commands are correct',
  );
  assert.deepEqual(
    setTransforms,
    setTransformsExpected,
    'The 6 setTransform commands are correct',
  );
  assert.equal(beginPaths, 75, 'There are 75 beginPath commands');
  assert.equal(rects.length, 4, 'There are 4 rect commands');
  assert.equal(fills, 4, 'There are 4 fill commands');
  assert.equal(strokes, 71, 'There are 71 stroke commands');
  assert.equal(moveTos, 71, 'There are 71 moveTo commands');
  assert.equal(lineTos.length, 155, 'There are 155 lineTo commands');
  assert.equal(attributes, 15, 'There are 15 attribute commands');
  assert.deepEqual(
    leftovers,
    [],
    'There are no canvas commands unaccounted for',
  );
});
