QUnit.module('visuals_arc_chord', {
  beforeEach() {
    // main chart object
    const chord = arc.visual.plugins.chord();

    // create the lib object
    const driver = arc.report.driver(chord);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Butte', 'CA', '123'],
      ['Merced', 'CA', '1123'],
      ['Sacramento', 'CA', '5123'],
      ['Burke', 'NC', '248'],
      ['Hoke', 'NC', '284'],
      ['Pender', 'NC', '824'],
      ['Ward 1', 'DC', '3456'],
      ['Ward 4', 'DC', '2345'],
      ['Ward 8', 'DC', '23456'],
      ['Fairfield', 'CT', '23'],
      ['Hartford', 'CT', '13'],
      ['Middlesex', 'CT', '123'],
    ]);
    const col0 = arc.data
      .column()
      .colname('County')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col1 = arc.data
      .column()
      .colname('State')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(2);
    result.columns([col0, col1, col2]);
    result.finalize();

    // create the visual by setting the data and then drawing it
    chord.viz_setDataResult(result);
    chord.viz_drawVisual('chart-div');
    // Window needs to be scrolled to the top.
    window.scrollTo(0, 0);

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.mainG = $('.viz-chord-circle');
    this.gs = $(this.mainG[0]).find('g');
    this.chords = $('path.viz-chord-chordpath');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Chart Element Present and Correct', function (assert) {
  const chordsExpected = [
    {
      top: 46,
      bottom: 231,
      left: 207,
      right: 231,
      width: 24,
      height: 185,
      fill: '#b0cb73',
    },
    {
      top: 47,
      bottom: 231,
      left: 197,
      right: 231,
      width: 34,
      height: 184,
      fill: '#3e8111',
    },
    {
      top: 45,
      bottom: 231,
      left: 231,
      right: 326,
      width: 95,
      height: 186,
      fill: '#0097a7',
    },
    {
      top: 45,
      bottom: 231,
      left: 205,
      right: 231,
      width: 26,
      height: 186,
      fill: '#b0cb73',
    },
    {
      top: 151,
      bottom: 231,
      left: 231,
      right: 410,
      width: 179,
      height: 80,
      fill: '#ed8111',
    },
    {
      top: 172,
      bottom: 231,
      left: 231,
      right: 413,
      width: 182,
      height: 59,
      fill: '#f6c088',
    },
    {
      top: 161,
      bottom: 231,
      left: 231,
      right: 416,
      width: 185,
      height: 70,
      fill: '#f6c088',
    },
    {
      top: 63,
      bottom: 231,
      left: 231,
      right: 345,
      width: 114,
      height: 168,
      fill: '#7dcad2',
    },
    {
      top: 231,
      bottom: 373,
      left: 66,
      right: 231,
      width: 165,
      height: 142,
      fill: '#d6b9d8',
    },
    {
      top: 217,
      bottom: 400,
      left: 111,
      right: 418,
      width: 307,
      height: 183,
      fill: '#ad73b1',
    },
    {
      top: 45,
      bottom: 231,
      left: 231,
      right: 396,
      width: 165,
      height: 186,
      fill: '#7dcad2',
    },
    {
      top: 50,
      bottom: 437,
      left: 25,
      right: 412,
      width: 387,
      height: 387,
      fill: '#d6b9d8',
    },
  ];

  assert.equal(this.svg.length, 1, 'There in one svg element');
  assert.equal(
    this.mainG.length,
    1,
    'There is one main g element with class viz-chord-circle',
  );
  assert.equal(this.gs.length, 16, 'There are 16 chord g elements');
  assert.equal(this.chords.length, 12, 'There are 12 chordpath g elements');

  for (let i = 0; i < this.chords.length; i++) {
    const chord = this.chords[i];
    // const bounds = chord.getBoundingClientRect();
    // assert.ok(
    //   arc.misc.within_tolerance(chordsExpected[i].top, bounds.top, 1),
    //   `Chord #${i} top edge is correct`
    // );
    // assert.ok(
    //   arc.misc.within_tolerance(chordsExpected[i].bottom, bounds.bottom, 1),
    //   `Chord #${i} bottom edge is correct`
    // );
    // assert.ok(
    //   arc.misc.within_tolerance(chordsExpected[i].left, bounds.left, 1),
    //   `Chord #${i} left egge is correct`
    // );
    // assert.ok(
    //   arc.misc.within_tolerance(chordsExpected[i].right, bounds.right, 1),
    //   `Chord #${i} right edge is correct`
    // );
    // assert.ok(
    //   arc.misc.within_tolerance(chordsExpected[i].width, bounds.width, 1),
    //   `Chord #${i} width is correct`
    // );
    // assert.ok(
    //   arc.misc.within_tolerance(chordsExpected[i].height, bounds.height, 1),
    //   `Chord #${i} height is correct`
    // );
    assert.equal(
      chordsExpected[i].fill,
      arc.misc.colorToHex(chord.style.fill),
      `Chord #${i}fill is correct`,
    );
  }
});
