QUnit.module('visuals_arc_colorpalette');

const cp = arc.colorpalette;

QUnit.test('colorStuff', (assert) => {
  assert.equal(cp.rgbToHex(0, 0, 0), '#000000', 'converts black to hex');
  assert.equal(cp.rgbToHex(255, 255, 255), '#ffffff', 'converts white to hex');
  assert.equal(cp.rgbToHex(33, 35, 140), '#21238c', 'converts blue to hex');
  assert.equal(
    cp.beginsWithPoundSign(' #545 '),
    true,
    'correctly identifies hex with spaces',
  );
  const black = cp.hexToRGB('#000000');
  let vals = [0, 0, 0];
  assert.deepEqual(black, vals, 'converted black hex properly');

  const white = cp.hexToRGB('#ffffff');
  vals = [255, 255, 255];

  assert.deepEqual(white, vals, 'converted white hex properly');
  const blue = cp.hexToRGB(' #21238c ');
  vals = [33, 35, 140];
  assert.deepEqual(blue, vals, 'converted blue hex properly');

  let rgbVals = cp.rgbStringToRGB('rgb(235, 22, 0)');
  vals = [235, 22, 0];
  assert.deepEqual(rgbVals, vals, 'converted rgb string to rgb properly');

  rgbVals = cp.rgbStringToRGB(' rgb(235,22,  0)');
  assert.deepEqual(
    rgbVals,
    vals,
    'converted rgb string with irrelgular spacing to rgb properly',
  );
});

QUnit.test('brightness', (assert) => {
  assert.equal(cp.colorForBrightness('#000000', 0.2), '#222', 'low opacity');
  assert.equal(
    cp.colorForBrightness('#000000', 0.9),
    '#eee',
    'black return light',
  );
  assert.equal(
    cp.colorForBrightness(' rgb(120, 235, 250) ', 0.9),
    '#222',
    'light returns dark',
  );
  assert.equal(
    cp.colorForBrightness(' #ffffff ', 0.9),
    '#222',
    'white hex return dark',
  );
});
