QUnit.module('visuals_arc_combined', {
  beforeEach() {
    // main chart object
    const combined = arc.visual.plugins.combined();

    // create the lib object
    const driver = arc.report.driver(combined);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Palo Alto', 852, 456, 9000],
      ['San Mateo', 471, 352, 112],
      ['Raleigh', 357, 159, 24],
      ['Chapel Hill', 258, 369, 100],
    ]);
    const col0 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col1 = arc.data
      .column()
      .colname('Elementary')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('bar_measure')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('Middle')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('line_measure')
      .colindex(2);
    const col3 = arc.data
      .column()
      .colname('High')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('highlighter_measure')
      .colindex(3);
    result.columns([col0, col1, col2, col3]);

    // create the visual by setting the data and then drawing it
    combined.viz_setDataResult(result);
    combined.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('#chart-div svg');
    this.tspan = $('tspan');
    this.chartG = $('.viz-bar-g');
    this.xAxis = $('.viz-bar-svg>svg>.x.arc-viz-axis');
    this.yAxis = $('.viz-bar-svg>.viz-bar-g .y.arc-viz-axis');
    this.xAxisLName = $('.x.arc-viz-axis-label>tspan');
    this.yAxisLName = $('.y.arc-viz-axis-label>tspan');
    this.circles = $('circle');

    this.canvas = $('.arc-viz-overlay-container>canvas');
    this.canvasMain = this.canvas.filter('.maincanvas');

    this.canvasSel = d3.selectAll(
      '.arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Bar Plot and Legend Elements Present', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');

  assert.equal(
    this.svg.length,
    7,
    'There are 7 svg elements (main, clipurl, and empty svg)',
  );
  assert.equal(this.tspan.length, 4, 'There are 4 tspan elements');
  assert.equal(
    this.chartG.length,
    1,
    'There are 1 g elements that contains the charts',
  );

  assert.equal(this.xAxis.length, 1, 'There are 1 x axis elements');
  assert.equal(this.yAxis.length, 1, 'There are 1 y axis elements');
  assert.equal(this.xAxisLName.length, 1, 'There are 1 x axis name elements');
  assert.equal(this.yAxisLName.length, 3, 'There are 1 y axis name elements');
  assert.equal(
    xAxisTicks.length,
    4,
    'There are 4 marks total on the two x axis',
  );
  assert.equal(
    yAxisTicks.length,
    10,
    'There are 10 marks total on the two y axis',
  );

  assert.equal(this.canvas.length, 9);
  assert.equal(this.canvasMain.length, 3);

  // 4 rectangles
  assert.equal(arc.canteen.countMethod(this.canvasData[0], 'rect'), 4);
  assert.equal(arc.canteen.countMethod(this.canvasData[0], 'fill'), 4);
  assert.equal(arc.canteen.countMethod(this.canvasData[0], 'beginPath'), 4);

  // 1 move and 3 line segments for the 4 points
  assert.equal(arc.canteen.countMethod(this.canvasData[1], 'moveTo'), 1);
  assert.equal(arc.canteen.countMethod(this.canvasData[1], 'lineTo'), 3);

  assert.equal(
    this.circles.length,
    4,
    'There are 4 circles, for one of the 4 locations in the chart',
  );
});

QUnit.test('Line, Circle, Rect Plot Elements Correct', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');
  const xAxisLNames = [];
  const yAxisLNames = [];
  const xAxisLabels = [];
  const yAxisLabels = [];
  const xAxisLNamesExpected = ['city'];
  const yAxisLNamesExpected = ['Elementary, ', 'Middle, ', 'High'];
  const xAxisLabelsExpected = [
    'Palo Alto',
    'San Mateo',
    'Raleigh',
    'Chapel Hill',
  ];
  const yAxisLabelsExpected = [
    '0',
    '1,000',
    '2,000',
    '3,000',
    '4,000',
    '5,000',
    '6,000',
    '7,000',
    '8,000',
    '9,000',
  ];
  const circleColors = [];
  const lineColorsExpected = ['#ED8111'];
  const circleColorsExpected = ['#2375b8', '#2375b8', '#2375b8', '#2375b8'];
  const rectColorsExpected = ['#0097A7', '#0097A7', '#0097A7', '#0097A7'];

  for (let g = 0; g < this.xAxisLName.length; g++) {
    xAxisLNames.push(this.xAxisLName[g].textContent);
  }

  for (let h = 0; h < this.yAxisLName.length; h++) {
    yAxisLNames.push(this.yAxisLName[h].textContent);
  }

  for (let i = 0; i < xAxisTicks.length; i++) {
    xAxisLabels.push(xAxisTicks[i].textContent);
  }

  for (let j = 0; j < yAxisTicks.length; j++) {
    yAxisLabels.push(yAxisTicks[j].textContent);
  }

  assert.deepEqual(
    xAxisLNames,
    xAxisLNamesExpected,
    'The x axis names are correct',
  );
  assert.deepEqual(
    yAxisLNames,
    yAxisLNamesExpected,
    'The y axis names are correct',
  );
  assert.deepEqual(
    xAxisLabels,
    xAxisLabelsExpected,
    'The x axis labels are correct',
  );
  assert.deepEqual(
    yAxisLabels,
    yAxisLabelsExpected,
    'The y axis labels are correct',
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[1], 'moveTo'),
      [[78.261, 367.675]],
    ),
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[1], 'lineTo'),
      [
        [166.984, 372.15],
        [255.706, 380.456],
        [344.429, 371.419],
      ],
    ),
  );

  for (let l = 0; l < this.circles.length; l++) {
    const circle = this.circles[l];
    circleColors.push(arc.misc.colorToHex(circle.style.stroke));
  }

  assert.deepEqual(
    arc.canteen.attrVals(this.canvasData[0], 'fillStyle'),
    rectColorsExpected,
  );
  assert.deepEqual(
    arc.canteen.attrVals(this.canvasData[1], 'fillStyle'),
    lineColorsExpected,
  );
  assert.deepEqual(
    circleColors,
    circleColorsExpected,
    'The circles are the right color',
  );
});
