QUnit.module('visuals_arc_correlation_flow', {
  beforeEach() {
    // main chart object
    this.table = arc.visual.plugins.correlation_flow();

    // create the lib object
    const driver = arc.report.driver(this.table);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['India', 'East', '200'],
      ['Haiti', 'West', '400'],
      ['Russia', 'East', '600'],
      ['Russia', 'West', '100'],
    ]);
    const col0 = arc.data
      .column()
      .colname('country')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col1 = arc.data
      .column()
      .colname('hemisphere')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('population')
      .coltype('BIGINT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(2);
    result.columns([col0, col1, col2]);
    result.finalize();

    // create the visual by setting the data and then drawing it
    this.table.viz_setDataResult(result);
    this.table.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('#chart-div svg');
    this.nodes = $('g.node');
    this.labels = this.nodes.find('text');
    this.rects = this.nodes.find('.noderect');
    this.paths = $('.viz-path-svg>g>g>path.link');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Correlation Flow Plot Elements Present', function (assert) {
  assert.equal(this.svg.length, 1, 'There are 1 svg elements');
  assert.equal(this.nodes.length, 5, 'There are 5 node elements');
  assert.equal(this.labels.length, 5, 'There are 5 node label elements');
  assert.equal(this.rects.length, 5, 'There are 5 node rect elements');
  assert.equal(
    this.paths.length,
    4,
    'There are 4 paths from nodes on left to nodes on right',
  );
});

QUnit.test('Line Plot Elements Correct', function (assert) {
  const nodeLabels = [];
  const rectStrokeColors = [];
  const rectFillColors = [];
  const nodeLabelsExpected = ['India', 'East', 'Haiti', 'West', 'Russia'];
  const rectStrokeColorsExpected = [
    '#004951',
    '#3d6266',
    '#29353b',
    '#565d60',
    '#743f08',
  ];
  const rectFillColorsExpected = [
    '#0097a7',
    '#7dcad2',
    '#546e7a',
    '#b0bec5',
    '#ed8111',
  ];
  const flowPathLengthsExpected = [405, 405, 406, 415];

  for (let i = 0; i < this.labels.length; i++) {
    nodeLabels.push(this.labels[i].textContent);
  }

  for (let j = 0; j < this.rects.length; j++) {
    rectStrokeColors.push(arc.misc.colorToHex(this.rects[j].style.stroke));
    rectFillColors.push(arc.misc.colorToHex(this.rects[j].style.fill));
  }

  assert.deepEqual(
    nodeLabels,
    nodeLabelsExpected,
    'The node labels are correct',
  );
  assert.deepEqual(
    rectStrokeColors,
    rectStrokeColorsExpected,
    'The node rect stroke colors are correct',
  );
  assert.deepEqual(
    rectFillColors,
    rectFillColorsExpected,
    'The node rect fill colors are correct',
  );

  let expectedLength;
  let actualLength;
  // The difference between Mac/Linux total lengths
  const pathLengthTolerance = 1;
  for (let k = 0; k < this.paths.length; k++) {
    const path = this.paths[k];

    // Check position here because we need to compare actual x/y positions to
    // range of expected values
    expectedLength = flowPathLengthsExpected[k];
    actualLength = path.getTotalLength();
    assert.ok(
      arc.misc.within_tolerance(
        expectedLength,
        actualLength,
        pathLengthTolerance,
      ),
      `The ${k} line is the right total length`,
    );
  }
});
