QUnit.module('visuals_arc_correlation_heatmap', {
  beforeEach() {
    // main chart object
    const correlationHeatmap = arc.visual.plugins.correlation();

    // create the lib object
    const driver = arc.report.driver(correlationHeatmap);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Dance', 'Classical', 'Anna', 'Ballet', 1881],
      ['Dance', 'Contemporary', 'Galina', 'Ballet', 1910],
      ['Dance', 'Classical', 'Mikhail', 'Ballet', 1948],
      ['Dance', 'Contemporary', 'Gene', 'Ball', 1881],
      ['Dance', 'Classical', 'Fred', 'Ball', 1881],
      ['Dance', 'Contemporary', 'Ginger', 'Ball', 1881],
      ['Music', 'Classical', 'Woody', 'Big Band', 1913],
      ['Music', 'Classical', 'Glenn', 'Big Band', 1904],
      ['Music', 'Contemporary', 'Artie', 'Big Band', 1910],
      ['Music', 'Contemporary', 'Miles', 'Jazz', 1926],
      ['Music', 'Classical', 'Ella', 'Jazz', 1917],
      ['Music', 'Contemporary', 'Dizzy', 'Jazz', 1917],
    ]);
    const col0 = arc.data
      .column()
      .colname('art_form')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col1 = arc.data
      .column()
      .colname('era')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('y_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('artist')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(2);
    const col3 = arc.data
      .column()
      .colname('genre')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(3);
    const col4 = arc.data
      .column()
      .colname('birth')
      .coltype('BIGINT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(4);
    result.columns([col0, col1, col2, col3, col4]);
    result.finalize();

    // create the visual by setting the data and then drawing it
    correlationHeatmap.viz_setDataResult(result);
    correlationHeatmap.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.viz_div = d3.select('#chart-div');
    this.canvas = this.viz_div.selectAll('canvas');

    this.labels = $('table>tbody>tr>td.viz-td-header');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Correlation Heatmap Elements Present', function (assert) {
  assert.equal(this.canvas.size(), 12);
  assert.equal(this.labels.length, 7, 'There are 7 label elements');
});

QUnit.test('Correlation Heatmap Elements Correct', function (assert) {
  const plots = arc.canteen.getCanvasData(this.canvas);

  assert.equal(plots.length, 12);

  assert.deepEqual(arc.canteen.attrVals(plots[0], 'strokeStyle'), ['#eee']);
  assert.deepEqual(arc.canteen.attrVals(plots[0], 'fillStyle'), [
    '#E0F7FA',
    '#006064',
    '#E0F7FA',
    '#666',
  ]);
  assert.equal(arc.canteen.countMethod(plots[0], 'fillRect'), 3);
  assert.equal(arc.canteen.countMethod(plots[0], 'rect'), 3);
  assert.equal(arc.canteen.countMethod(plots[0], 'stroke'), 3);
  assert.equal(arc.canteen.countMethod(plots[0], 'beginPath'), 3);
  assert.equal(arc.canteen.countMethod(plots[0], 'fillText'), 7);
  let t = arc.canteen.methodArgs(plots[0], 'fillText').map((d) => d[0]);

  assert.deepEqual(
    t,
    ['Anna', 'Mikhail', 'Fred', 'Ballet', 'Ballet', 'Ball', 'Ball'],
    'invalid correlation heat map labels',
  );

  assert.deepEqual(arc.canteen.attrVals(plots[1], 'strokeStyle'), []);
  assert.deepEqual(arc.canteen.attrVals(plots[1], 'fillStyle'), [
    'rgb(1,0,0)',
    'rgb(3,0,0)',
    'rgb(5,0,0)',
    '#666',
    'rgb(7,0,0)',
    'rgb(9,0,0)',
    'rgb(11,0,0)',
    'rgb(13,0,0)',
    'rgb(15,0,0)',
  ]);

  assert.equal(arc.canteen.countMethod(plots[1], 'fillRect'), 8);
  assert.equal(arc.canteen.countMethod(plots[1], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[1], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[1], 'beginPath'), 0);
  assert.equal(arc.canteen.countMethod(plots[1], 'fillText'), 0);

  assert.equal(arc.canteen.countMethod(plots[2], 'fillRect'), 0);
  assert.equal(arc.canteen.countMethod(plots[2], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[2], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[2], 'beginPath'), 0);
  assert.equal(arc.canteen.countMethod(plots[2], 'fillText'), 0);

  assert.deepEqual(arc.canteen.attrVals(plots[3], 'strokeStyle'), ['#eee']);
  assert.deepEqual(arc.canteen.attrVals(plots[3], 'fillStyle'), [
    '#26C6DA',
    '#4DD0E1',
    '#00BCD4',
    '#666',
  ]);
  assert.equal(arc.canteen.countMethod(plots[3], 'fillRect'), 3);
  assert.equal(arc.canteen.countMethod(plots[3], 'rect'), 3);
  assert.equal(arc.canteen.countMethod(plots[3], 'stroke'), 3);
  assert.equal(arc.canteen.countMethod(plots[3], 'beginPath'), 3);
  assert.equal(arc.canteen.countMethod(plots[3], 'fillText'), 7);
  t = arc.canteen.methodArgs(plots[0], 'fillText').map((d) => d[0]);
  assert.deepEqual(
    t,
    ['Anna', 'Mikhail', 'Fred', 'Ballet', 'Ballet', 'Ball', 'Ball'],
    'invalid correlation heatmap axis labels',
  );

  assert.deepEqual(arc.canteen.attrVals(plots[4], 'strokeStyle'), []);
  assert.deepEqual(arc.canteen.attrVals(plots[4], 'fillStyle'), [
    'rgb(17,0,0)',
    'rgb(19,0,0)',
    'rgb(21,0,0)',
    '#666',
    'rgb(23,0,0)',
    'rgb(25,0,0)',
    'rgb(27,0,0)',
    'rgb(29,0,0)',
    'rgb(31,0,0)',
  ]);
  assert.equal(arc.canteen.countMethod(plots[4], 'fillRect'), 8);
  assert.equal(arc.canteen.countMethod(plots[4], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[4], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[4], 'beginPath'), 0);

  assert.equal(arc.canteen.countMethod(plots[5], 'fillRect'), 0);
  assert.equal(arc.canteen.countMethod(plots[5], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[5], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[5], 'beginPath'), 0);
  assert.equal(arc.canteen.countMethod(plots[5], 'fillText'), 0);

  assert.deepEqual(arc.canteen.attrVals(plots[6], 'strokeStyle'), ['#eee']);
  assert.deepEqual(arc.canteen.attrVals(plots[6], 'fillStyle'), [
    '#26C6DA',
    '#E0F7FA',
    '#E0F7FA',
    '#666',
  ]);
  assert.equal(arc.canteen.countMethod(plots[6], 'fillRect'), 3);
  assert.equal(arc.canteen.countMethod(plots[6], 'rect'), 3);
  assert.equal(arc.canteen.countMethod(plots[6], 'stroke'), 3);
  assert.equal(arc.canteen.countMethod(plots[6], 'beginPath'), 3);
  assert.equal(arc.canteen.countMethod(plots[6], 'fillText'), 7);
  t = arc.canteen.methodArgs(plots[6], 'fillText').map((d) => d[0]);
  assert.deepEqual(
    t,
    ['Galina', 'Gene', 'Ginger', 'Ballet', 'Ballet', 'Ball', 'Ball'],
    'invalid correlation heatmap labels',
  );

  assert.equal(arc.canteen.countMethod(plots[7], 'fillRect'), 8);
  assert.equal(arc.canteen.countMethod(plots[7], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[7], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[7], 'beginPath'), 0);
  assert.equal(arc.canteen.countMethod(plots[7], 'fillText'), 0);

  assert.equal(arc.canteen.countMethod(plots[8], 'fillRect'), 0);
  assert.equal(arc.canteen.countMethod(plots[8], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[8], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[8], 'beginPath'), 0);
  assert.equal(arc.canteen.countMethod(plots[8], 'fillText'), 0);

  assert.deepEqual(arc.canteen.attrVals(plots[9], 'strokeStyle'), ['#eee']);
  assert.deepEqual(arc.canteen.attrVals(plots[9], 'fillStyle'), [
    '#26C6DA',
    '#00ACC1',
    '#00BCD4',
    '#666',
  ]);
  assert.equal(arc.canteen.countMethod(plots[9], 'fillRect'), 3);
  assert.equal(arc.canteen.countMethod(plots[9], 'rect'), 3);
  assert.equal(arc.canteen.countMethod(plots[9], 'stroke'), 3);
  assert.equal(arc.canteen.countMethod(plots[9], 'beginPath'), 3);
  assert.equal(arc.canteen.countMethod(plots[9], 'fillText'), 7);
  t = arc.canteen.methodArgs(plots[9], 'fillText').map((d) => d[0]);
  assert.deepEqual(
    t,
    ['Artie', 'Miles', 'Dizzy', 'Big Band', 'Big Band', 'Jazz', 'Jazz'],
    'invalid correlation heatmap labels',
  );

  assert.deepEqual(arc.canteen.attrVals(plots[10], 'strokeStyle'), []);
  assert.deepEqual(arc.canteen.attrVals(plots[10], 'fillStyle'), [
    'rgb(49,0,0)',
    'rgb(51,0,0)',
    'rgb(53,0,0)',
    '#666',
    'rgb(55,0,0)',
    'rgb(57,0,0)',
    'rgb(59,0,0)',
    'rgb(61,0,0)',
    'rgb(63,0,0)',
  ]);
  assert.equal(arc.canteen.countMethod(plots[10], 'fillRect'), 8);
  assert.equal(arc.canteen.countMethod(plots[10], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[10], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[10], 'beginPath'), 0);

  assert.equal(arc.canteen.countMethod(plots[11], 'fillRect'), 0);
  assert.equal(arc.canteen.countMethod(plots[11], 'rect'), 0);
  assert.equal(arc.canteen.countMethod(plots[11], 'stroke'), 0);
  assert.equal(arc.canteen.countMethod(plots[11], 'beginPath'), 0);
  assert.equal(arc.canteen.countMethod(plots[11], 'fillText'), 0);

  const labels = [];
  this.labels.each(function () {
    labels.push($(this).text());
  });

  const labelsExpected = [
    '',
    'art_form',
    'era',
    'Dance',
    'Music',
    'Classical',
    'Contemporary',
  ];

  assert.deepEqual(labels, labelsExpected, 'The labels are correct');
});
