QUnit.module('visuals_arc_crosstab', {
  beforeEach() {
    const crosstab = arc.visual.plugins.crosstab();
    arc.report.driver(crosstab).report_id(3).chart_div('chart-div');
    const result = arc.data.result();
    result.rows([
      ['AZ', '1920', '34523094'],
      ['AZ', '1930', '123934'],
      ['CA', '1920', '34509'],
      ['CA', '1930', '23401923'],
    ]);
    const col1 = arc.data
      .column()
      .colname('state')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_dimensions_shelf')
      .colindex(0);
    const col2 = arc.data
      .column()
      .colname('year')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('y_dimensions_shelf')
      .colindex(1);
    const col3 = arc.data
      .column()
      .colname('sum(population)')
      .coltype('BIGINT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(2);
    result.columns([col1, col2, col3]);
    result.finalize();
    crosstab.viz_setDataResult(result);
    crosstab.viz_drawVisual('chart-div');

    this.vizDiv = $('#chart-div');
    this.vizDivChild = this.vizDiv.children();
    this.vizRows = this.vizDiv.find('tr');
    this.vizCells = this.vizDiv.find('td');
    this.vizTrHeader = this.vizDiv.find('.viz-tr-header');
    this.vizTdHeader = this.vizDiv.find('.viz-td-header');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

// Check to make sure there are the expected number of columns, rows, and cells
QUnit.test('Cross Table Elements Present', function (assert) {
  const numberTrHeader = this.vizTrHeader.length;
  const numberTdHeader = this.vizTdHeader.length;
  const numberRows = this.vizRows.length;
  const numberCells = this.vizCells.length - numberTdHeader;

  assert.equal(numberRows, 5, 'There are 5 rows');
  assert.equal(numberTrHeader, 3, 'There are 3 header rows');
  assert.equal(numberTdHeader, 9, 'There is a header column');
  assert.equal(numberCells, 4, 'There are 4 cells');
  assert.equal(
    this.vizTdHeader[1].getElementsByTagName('i')[0].className,
    'fa fa-sort-amount-asc viz-sort-icon show-fixed',
    'The state header contains icon',
  );
  assert.equal(
    this.vizTdHeader[5].getElementsByTagName('i')[0].className,
    'fa fa-sort-amount-asc viz-sort-icon show-fixed',
    'The year header contains icon',
  );
});

// Check each cell's contents
QUnit.test('Cross Table Text Correct', function (assert) {
  const cellContentsExpected = [
    '',
    '<span desc="state"><span class="viz-col-sort asc" data-pos="0" data-is-measure="false" data-click-info="{}"><i class="fa fa-sort-amount-asc viz-sort-icon show-fixed"></i>state</span></span>',
    '',
    'AZ',
    'CA',
    '<span desc="year"><span class="viz-col-sort asc" data-pos="1" data-is-measure="false" data-click-info="{}"><i class="fa fa-sort-amount-asc viz-sort-icon show-fixed"></i>year</span></span>',
    '<span class="viz-col-sort desc" data-pos="0" data-is-measure="true" data-click-info="{&quot;state&quot;:&quot;AZ&quot;,&quot;arc-measure-colname&quot;:&quot;sum(population)&quot;}"><i class="fa fa-sort-amount-desc viz-sort-icon"></i>sum(population)</span>',
    '1920',
    '34.5M',
    '34,509',
    '1930',
    '124k',
    '23.4M',
  ];

  for (let i = 0; i < cellContentsExpected.length; i++) {
    assert.equal(
      this.vizCells[i].innerHTML,
      cellContentsExpected[i],
      `Cell #${i} matches expected value`,
    );
  }
});

// Check each cell's colors
QUnit.test('Cross Table Colors Correct', function (assert) {
  const cellColorExpected = [
    '',
    '',
    '',
    '',
    '',
    '',
    '',
    '',
    'rgb(0, 96, 100)',
    'rgb(224, 247, 250)',
    '',
    'rgb(224, 247, 250)',
    'rgb(0, 172, 193)',
  ];

  for (let i = 0; i < cellColorExpected.length; i++) {
    assert.equal(
      this.vizCells[i].style.backgroundColor,
      cellColorExpected[i],
      `Cell #${i} matches expected color`,
    );
  }
});
