window.addEventListener('load', () => {
  const arcl = {};

  QUnit.module('visuals_arc_daterange', {
    beforeEach() {
      arcl.daterange = arc.visual.plugins.daterange();
      arcl.driver = arc.report
        .driver(arcl.daterange)
        .chart_div('chart-div')
        .shelves_dom_id('app-shelves');
      arc.globals.reports = arc.reporttracker.reports();
    },

    afterEach() {
      $('#chart-div').empty();
      $('#app-shelves').empty();
      $('#filters-save-as-segments-html').empty();
    },
  });

  QUnit.test('daterange parameters (Fixed Type)', (assert) => {
    /*
     * Verify that stored parameters, and incoming parameters, are used
     *   correctly by date range widgets.
     */
    let call_count = 0;
    let stored_params;
    arcl.driver.dashboard_click = function (in_params) {
      call_count += 1;
      stored_params = arc.params.convertToOldFormatHash(in_params);
    };

    const settings = {
      core: { shelves: [] },
      sample_pct: 'Off',
      user_settings: {
        container_css_class: '',
        css_styles: '[]',
        inline_css: '',
        max_rowcount: '',
        disable_incremental_queries: 'false',
        daterange_base_field: 'time',
        daterange_title: '',
        daterange_output_parameter: '',
        show_time: 'false',
        oneclick_dateranges: 'default',
        filter_appgroup_scope: 'false',
        filter_dataset_scope: 'true',
      },
      settings: {
        colorSettings: {},
        colors: [
          '#0097a7',
          '#7dcad2',
          '#546e7a',
          '#b0bec5',
          '#ed8111',
          '#f6c088',
          '#e54954',
          '#f2a4a9',
          '#ad73b1',
          '#d6b9d8',
          '#2375b8',
          '#91badb',
          '#3e8111',
          '#b0cb73',
        ],
        graded_colors: [
          '#E0F7FA',
          '#B2EBF2',
          '#80DEEA',
          '#4DD0E1',
          '#26C6DA',
          '#00BCD4',
          '#00ACC1',
          '#0097A7',
          '#00838F',
          '#006064',
        ],
        contrast_colors: [
          '#0097A7',
          '#ED8111',
          '#2375B8',
          '#E54954',
          '#42996F',
          '#AD73B1',
          '#B0CB73',
          '#EA8DCD',
        ],
        reverse_colors: false,
      },
      report_title: '',
      report_subtitle: '',
      report_derived_data: [],
      click_behaviors: {},
      viz_type: { daterange: { default_selections: {} } },
    };
    settings.user_settings.daterange_base_field = 'ds';

    arcl.daterange.viz_setSettings(settings);
    assert.equal(call_count, 0);

    settings.viz_type = {
      daterange: {
        default_selections: {
          start_time: '2012-01-01',
          end_time: '2012-01-07',
        },
      },
    };

    arcl.daterange.viz_setSettings(settings);
    assert.equal(call_count, 1);
    assert.equal(
      Object.keys(stored_params).length,
      3,
      'Fixed Date filter emits correct count of params',
    );
    assert.equal(stored_params['ds.start'], '2012-01-01');
    assert.equal(stored_params['ds.end'], '2012-01-07');
    assert.equal(stored_params['ds.include_nulls'], 'true');
    assert.equal(
      stored_params['ds.config'],
      undefined,
      'Filter emits out as fixed date',
    );

    const params = arc.params
      .tracker()
      .add({
        key: 'ds.start',
        value: '2013-02-02',
      })
      .add({
        key: 'ds.end',
        value: '2013-02-08',
      });
    arcl.driver.params(params);

    arcl.daterange.viz_setSettings(settings);
    assert.equal(call_count, 2);
    assert.equal(
      Object.keys(stored_params).length,
      3,
      'Fixed Date filter emits correct count of params',
    );
    assert.equal(stored_params['ds.start'], '2013-02-02');
    assert.equal(stored_params['ds.end'], '2013-02-08');
    assert.equal(stored_params['ds.include_nulls'], 'false');
    assert.equal(
      stored_params['ds.config'],
      undefined,
      'Filter emits out as fixed date',
    );
  });
});
