QUnit.module('visuals_arc_extension', {
  beforeEach() {
    // main chart object
    const extension = arc.visual.plugins.extension();

    // create the lib object
    arc.report.driver(extension);

    // provide the settings
    const settings = {
      core: { viz_type: 'extension', shelves: [] },
      dimensions_shelf: [],
      aggregates_shelf: [],
      filters_shelf: [],
      sample_pct: 'Off',
      selected_segments: [],
      user_settings: {
        color_palette:
          '{"colors":["#0097a7","#b0cb73"],"colorSettings":{},"isReversed":false}',
        container_css_class: '',
        css_styles: '[]',
        inline_css: '',
        permit_csv_download_visual: 'true',
        permit_png_download: 'true',
        explicit_scope: 'false',
        visual_dataset_scope: 'true',
        send_aggregate_parameters: 'false',
        visual_font: 'default',
        title_font: 'default',
        title_font_size: '',
        title_alignment: 'Left',
        subtitle_font: 'default',
        subtitle_font_size: '',
        subtitle_alignment: 'Left',
        visual_hide_title: 'false',
        max_rowcount: '',
        disable_incremental_queries: 'false',
        table_template: 'table-template-2',
        table_header_alignment: 'data_dependent',
        table_full_width: 'false',
        enable_info: 'false',
        parameters_send_all_cols: 'false',
        show_tooltip: 'true',
        show_contextmenu: 'true',
        enable_highlighting: 'false',
        cells_are_html: 'false',
        nulls_order: 'Default',
        nulls_display_value: 'null',
        http_auto_link: 'true',
        table_totals: 'false',
        table_totals_header: 'false',
        max_columns: '20',
        download_rows: '-1',
        enable_pagination: 'false',
        page_size: '-1',
      },
      settings: {
        colorSettings: {},
        colors: ['#0097a7', '#b0cb73'],
        graded_colors: ['#E0F7FA', '#006064'],
        contrast_colors: ['#0097A7', '#EA8DCD'],
        reverse_colors: false,
      },
      report_title: 'testing html extension',
      report_subtitle: '',
      report_derived_data: [],
      click_behaviors: {},
      columnWidths: [],
      limit: 100,
      viz_type: { html: { html_contents: '' } },
    };

    this.html_contents =
      '<style>h1 {color: maroon; margin-left: 40px;}</style>' +
      '<h1>Hello World!</h1>' +
      '<h2>Goodbye Moon!</h2>';

    settings.viz_type.html.html_contents = this.html_contents;

    // create the visual by setting the data and then drawing it
    extension.viz_setSettings(settings);
    extension.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.chartDiv = $('#chart-div');
    this.vizDiv = this.chartDiv.find('div');
    this.header = this.chartDiv.find('h1');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Extension HTML Correct', function (assert) {
  const innerHtml = this.vizDiv.html().trim();
  const header1Color = this.header.css('color');
  const header1MarginLeft = this.header.css('margin-left');

  assert.equal(innerHtml, this.html_contents, 'Extension HTML is correct');
  assert.equal(
    arc.misc.colorToHex(header1Color),
    arc.misc.colorToHex('maroon'),
    'H1 color is correct',
  );
  assert.equal(header1MarginLeft, '40px', 'H1 margin-left is correct');
  assert.ok(this.vizDiv[0].id.includes('viz-html-uid-'));
});
