QUnit.module('visuals_arc_groupedbars', {
  beforeEach() {
    // main chart object
    const bar = arc.visual.plugins.groupedbar();

    // create the lib object
    const driver = arc.report.driver(bar);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Palo Alto', '2000', 'CA', 9000],
      ['San Mateo', '2000', 'CA', 0],
      ['Raleigh', '2000', 'NC', -2000],
      ['Chapel Hill', '2000', 'NC', '4356', 1000],
      ['Palo Alto', '1990', 'CA', 1123],
      ['San Mateo', '1990', 'CA', 12],
      ['Raleigh', '1990', 'NC', 24],
      ['Chapel Hill', '1990', 'NC', 100],
    ]);
    const col1 = arc.data
      .column()
      .colname('year')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col3 = arc.data
      .column()
      .colname('state')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(2);
    const col4 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(3);
    result.columns([col1, col2, col3, col4]);

    // create the visual by setting the data and then drawing it
    bar.viz_setDataResult(result);
    $('#chart-div').height(500);
    bar.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.labels = $('table>tbody>tr>td.viz-td-header');
    this.canvas = $('.viz-trellis-div>.arc-viz-overlay-container>canvas');
    this.canvasMain = this.canvas.filter('.maincanvas');
    this.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
    this.xAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
    );
    this.xAxisTicks = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g>.x.arc-viz-axis>.tick',
    );
    this.yAxis = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
    );
    this.yAxisTicks = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis>.tick',
    );
    this.axisLabel = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .arc-viz-axis-label',
    );
    this.bars = $(
      '.viz-trellis-div>.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g>.arc-bar-group>rect',
    );

    this.canvasSel = d3.selectAll(
      '.viz-trellis-div>.arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Bar Plot and Legend Elements Present', function (assert) {
  assert.equal(
    this.svg.length,
    7,
    'There are 7 svg elements (2 trellis cells with graph + x-axis and 1 legend and clippath)',
  );
  assert.equal(this.labels.length, 3, 'There are 3 td label elements');
  assert.equal(this.canvas.length, 6);
  assert.equal(this.canvasMain.length, 2);
  assert.equal(
    this.legendG.length,
    2,
    'There are two g elements that contains the legend',
  );

  assert.equal(this.xAxis.length, 2, 'There are 2 x axis elements');
  assert.equal(this.yAxis.length, 2, 'There are 2 y axis elements');
  assert.equal(this.axisLabel.length, 4, 'There are 4 axis label element');
  assert.equal(
    this.xAxisTicks.length,
    16,
    'There are 8 marks total on the two x axis',
  );
  assert.equal(
    this.yAxisTicks.length,
    12,
    'There are 12 marks total on the two y axis',
  );

  assert.equal(
    this.bars.length,
    8,
    'There are 8 bars, one for each of the two colors in the two charts',
  );
});

QUnit.test('Bar Plot Elements Correct', function (assert) {
  const { xAxisTicks } = this;
  const { yAxisTicks } = this;
  const xAxisLabels = [];
  const yAxisLabels = [];
  const axisLabel = [];
  const trellisLabels = [];
  const xAxisLabelsExpected = [
    '',
    '',
    '',
    '',
    'Palo Alto',
    'San Mateo',
    'Raleigh',
    'Chapel Hill',
    '',
    '',
    '',
    '',
    'Palo Alto',
    'San Mateo',
    'Raleigh',
    'Chapel Hill',
  ];
  const yAxisLabelsExpected = [
    '-2,000',
    '0',
    '2,000',
    '4,000',
    '6,000',
    '8,000',
    '-2,000',
    '0',
    '2,000',
    '4,000',
    '6,000',
    '8,000',
  ];
  const axisLabelExpected = ['city', 'population', 'city', 'population'];
  const trellisLabelsExpected = ['year', '2000', '1990'];
  const barColors = [];
  const barPathLengthExpected = [235, 2, 58, 115, 23, 2, 2, 2];
  const barColorsExpected = [
    '#0097a7',
    '#0097a7',
    '#7dcad2',
    '#7dcad2',
    '#0097a7',
    '#0097a7',
    '#7dcad2',
    '#7dcad2',
  ];

  for (let i = 0; i < xAxisTicks.length; i++) {
    xAxisLabels.push(xAxisTicks[i].textContent);
  }

  for (let j = 0; j < yAxisTicks.length; j++) {
    yAxisLabels.push(yAxisTicks[j].textContent);
  }

  for (let h = 0; h < this.labels.length; h++) {
    trellisLabels.push(this.labels[h].textContent);
  }

  for (let g = 0; g < this.axisLabel.length; g++) {
    axisLabel.push(this.axisLabel[g].textContent);
  }

  assert.deepEqual(
    xAxisLabels,
    xAxisLabelsExpected,
    'The x axis labels are correct',
  );
  assert.deepEqual(
    yAxisLabels,
    yAxisLabelsExpected,
    'The y axis labels are correct',
  );
  assert.deepEqual(axisLabel, axisLabelExpected, 'The axis labels are correct');
  assert.deepEqual(
    trellisLabels,
    trellisLabelsExpected,
    'The trellis labels are correct',
  );

  let expectedLength;
  let actualLength;
  // The difference between Mac/Linux total lengths
  const barLengthTolerance = 20;
  for (let k = 0; k < this.bars.length; k++) {
    const bar = this.bars[k];
    barColors.push(arc.misc.colorToHex(bar.style.fill));

    // Check position here because we need to compare actual x/y positions to
    // range of expected values
    expectedLength = barPathLengthExpected[k];
    actualLength = bar.getAttribute('height');
    assert.ok(
      arc.misc.within_tolerance(
        expectedLength,
        actualLength,
        barLengthTolerance,
      ),
      `The ${k} bar is the right total length`,
    );
  }

  assert.deepEqual(
    barColors,
    barColorsExpected,
    'The bars are the right color',
  );
});

QUnit.test('Legend Elements Correct', function (assert) {
  const rectangles = this.legendG[0].getElementsByTagName('rect');
  const labels = this.legendG[0].getElementsByTagName('text');
  const rectangleColors = [];
  const labelTexts = [];
  const rectangleColorsExpected = ['#0097a7', '#7dcad2'];
  const labelTextsExpected = ['CA', 'NC'];

  assert.equal(rectangles.length, 2, 'The legend has 2 rectangles');
  assert.equal(labels.length, 2, 'The legend has 2 labels');

  for (let i = 0; i < rectangles.length; i++) {
    rectangleColors.push(rectangles[i].style.fill);
    labelTexts.push(labels[i].textContent);
  }

  for (let i = 0; i < rectangleColors.length; i++) {
    rectangleColors[i] = arc.misc.colorToHex(rectangleColors[i]);
  }

  assert.deepEqual(
    rectangleColors,
    rectangleColorsExpected,
    'The legend has the correctly colored rectangles',
  );
  assert.deepEqual(
    labelTexts,
    labelTextsExpected,
    'The legend has the correct labels',
  );
});
