QUnit.module('visuals_arc_histogram', {
  beforeEach() {
    const histogram = arc.visual.plugins.histogram();

    arc.report.driver(histogram).chart_div('chart-div');

    // Here we set the container's width to roughly the same value
    // that the original visual has
    // on an average MacPro (1792*1120) in a full window mode chrome browser
    $('#chart-div').height(371);

    // The data is based on the samples-cereal dataset.
    // Hence the histogram is a special type of bar visual,
    // where we group the raw values into buckets
    // these row values are pre-calculated from the raw data
    // of the dataset (calories) on the server
    // and this test directly draws the visual from this array
    // to workaround the sql query that is
    // originally sent to the server to calculate these values.
    const rows = [
      [55.45, 2],
      [66.35, 4],
      [88.15, 5],
      [99.05, 1],
      [109.95, 2],
      [120.85000000000001, 2],
      [131.75, 2],
      [142.64999999999998, 1],
      [153.55, 1],
    ];

    const min = 50;
    const max = 159;
    this.x_label = 'calories';
    this.y_label = 'Frequency';

    const col1 = arc.data
      .column()
      .colname(this.x_label)
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);

    const col2 = arc.data
      .column()
      .colname(this.y_label)
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(1);

    histogram.viz_drawVisual_test(rows, [col1, col2], min, max);

    this.svg = $('.viz-trellis-outertable svg');
    this.labels = $('table>tbody>tr td.viz-td-header');
    this.svgContainer = $('.arc-viz-type-histogram .arc-viz-overlay-container');
    this.canvas = $(
      '.arc-viz-type-histogram .arc-viz-overlay-container canvas',
    );
    this.canvasMain = this.canvas.filter('.maincanvas');
    this.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
    this.xAxis = $(
      '.arc-viz-type-histogram .arc-viz-overlay-container .viz-bar-svg .x.arc-viz-axis',
    );
    this.yAxis = $(
      '.arc-viz-type-histogram .arc-viz-overlay-container .viz-bar-svg .y.arc-viz-axis',
    );
    this.axisLabelX = $(
      '.arc-viz-type-histogram .arc-viz-overlay-container .viz-bar-svg .viz-bar-g .x.arc-viz-axis-label',
    );
    this.axisLabelY = $(
      '.arc-viz-type-histogram .arc-viz-overlay-container .viz-bar-svg .viz-bar-g .y.arc-viz-axis-label',
    );

    this.canvasSel = d3.selectAll(
      '.arc-viz-type-histogram .arc-viz-overlay-container canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },

  afterEach() {
    // Height is set back to 500 to not break the other visuals tests
    $('#chart-div').height(500);
    $('#chart-div').empty();
  },
});

QUnit.test('SVG container created', (assert) => {
  assert.equal(d3.select('svg').empty(), false, 'SVG container is created');
});

QUnit.test('Correct axis labels', function (assert) {
  assert.equal(
    this.axisLabelX.text(),
    this.x_label,
    `The label on x axis is ${this.x_label}`,
  );
  assert.equal(
    this.axisLabelY.text(),
    this.y_label,
    `The label on y axis is ${this.y_label}`,
  );
});

QUnit.test(' Elements are correct on histogram', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');
  const xAxisTickText = [];
  const yAxisTickText = [];

  const xAxisxAxisTickTextExpected = [
    '50',
    '60',
    '70',
    '80',
    '90',
    '100',
    '110',
    '120',
    '130',
    '140',
    '150',
  ];
  const yAxisxAxisTickTextExpected = ['0', '1', '2', '3', '4', '5'];
  const barColorsExpected = ['#0097a7'];

  for (let i = 0; i < xAxisTicks.length; i++) {
    xAxisTickText.push(xAxisTicks[i].textContent);
  }

  for (let i = 0; i < yAxisTicks.length; i++) {
    yAxisTickText.push(yAxisTicks[i].textContent);
  }

  assert.equal(xAxisTicks.length, 11, 'There are 11 ticks on the x axis');
  assert.equal(yAxisTicks.length, 6, 'There are 6 ticks on the y axis');
  assert.deepEqual(
    xAxisTickText,
    xAxisxAxisTickTextExpected,
    'The x axis tick texts are correct',
  );
  assert.deepEqual(
    yAxisTickText,
    yAxisxAxisTickTextExpected,
    'The y axis tick texts are correct',
  );
  assert.equal(this.canvasData.length, 1, 'There is one graph has been drawn');

  const barColors = arc.canteen.attrVals(this.canvasData[0], 'fillStyle');
  const uniqueColors = Array.from(new Set(barColors));

  assert.deepEqual(uniqueColors, barColorsExpected, 'Colors are correct');
  assert.equal(
    arc.canteen.countMethod(this.canvasData[0], 'beginPath'),
    9,
    'Bars has been drawn correctly 1/1',
  );
  assert.equal(
    arc.canteen.countMethod(this.canvasData[0], 'rect'),
    9,
    'Bars has been drawn correctly 1/1',
  );
  assert.equal(
    arc.canteen.countMethod(this.canvasData[0], 'fill'),
    9,
    'Bars has been drawn correctly 1/1',
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[0], 'rect'),
      [
        [0, 172.415, 29.3, 114.944],
        [29.3, 57.472, 29.3, 229.887],
        [87.9, 0, 29.3, 287.359],
        [117.2, 229.887, 29.3, 57.472],
        [146.5, 172.415, 29.3, 114.944],
        [175.8, 172.415, 29.3, 114.944],
        [205.1, 172.415, 29.3, 114.944],
        [234.4, 229.887, 29.3, 57.472],
        [263.7, 229.887, 29.3, 57.472],
      ],
    ),
    'Canvas data is precise.',
  );
});
