QUnit.module('visuals_arc_kpi', {
  before() {
    $('#chart-div').empty();
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Chart Element Present', function (assert) {
  // main chart object
  const kpi = arc.visual.plugins.kpi();

  // create the lib object
  const driver = arc.report.driver(kpi);
  driver.chart_div('chart-div');

  // provide the data
  const resultTrellis = arc.data.result();
  resultTrellis.rows([['James']]);
  const col0 = arc.data
    .column()
    .colname('salesmen')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('dimensions_shelf')
    .colindex(0);

  resultTrellis.columns([col0]).finalize();

  // create the visual by setting the data and then drawing it
  kpi.viz_setDataResult(resultTrellis);
  kpi.viz_drawVisual('chart-div');

  // store elements on the page to be tested
  this.viz_div = d3.select('#chart-div');
  this.trs = this.viz_div.selectAll('tr');
  this.tds = this.viz_div.selectAll('td');
  this.kpi_containers = this.viz_div.selectAll('div.kpi-container');

  assert.equal(this.trs.length, 1, 'There are 1 tr elements');
  assert.equal(this.tds.length, 1, 'There are 1 td elements');
  assert.equal(
    this.kpi_containers.length,
    1,
    'There are 1 kpi_containers elements',
  );
});

QUnit.test('KPI has compare arrow and compare value', function (assert) {
  // main chart object
  const kpi = arc.visual.plugins.kpi();

  // create the lib object
  const driver = arc.report.driver(kpi);
  driver.chart_div('chart-div');

  this.user_settings = driver.user_settings();
  this.user_settings.find_by_id('compare_display_format').value('difference');

  // provide the data
  const resultTrellis = arc.data.result();
  resultTrellis.rows([['68.5', '58.5']]);
  const col0 = arc.data
    .column()
    .colname('salesThisYear')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('aggregates_shelf')
    .colindex(0);

  const col1 = arc.data
    .column()
    .colname('salesLastYear')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('compare_shelf')
    .colindex(1);

  resultTrellis.columns([col0, col1]).finalize();

  // create the visual by setting the data and then drawing it
  kpi.viz_setDataResult(resultTrellis);
  kpi.viz_drawVisual('chart-div');

  // store elements on the page to be tested
  this.viz_div = d3.select('#chart-div');
  this.kpiContainers = this.viz_div.selectAll('div.kpi-container');
  this.arrowContainers = this.viz_div.selectAll(
    '.kpi-container .compare-arrow',
  );
  this.compareContainer = this.viz_div.selectAll('.kpi-container .kpi-compare');

  assert.equal(
    this.kpiContainers.length,
    1,
    'There are 1 kpi_containers elements',
  );
  assert.equal(this.arrowContainers.length, 1, 'There is an arrow container');
  assert.equal(this.compareContainer.length, 1, 'There is a compare container');

  const compareValue = $('.kpi-container .kpi-compare').html();
  assert.equal(compareValue, '+10.0', 'Compare value properly displays');
});

QUnit.test('Arrow type setting works', function (assert) {
  // main chart object
  const kpi = arc.visual.plugins.kpi();

  // create the lib object
  const driver = arc.report.driver(kpi);
  driver.chart_div('chart-div');

  this.user_settings = driver.user_settings();
  this.user_settings.find_by_id('compare_arrow_type').value('none');

  // provide the data
  const resultTrellis = arc.data.result();
  resultTrellis.rows([['68.5', '58.5']]);
  const col0 = arc.data
    .column()
    .colname('salesThisYear')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('aggregates_shelf')
    .colindex(0);

  const col1 = arc.data
    .column()
    .colname('salesLastYear')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('compare_shelf')
    .colindex(1);

  resultTrellis.columns([col0, col1]).finalize();

  // create the visual by setting the data and then drawing it
  kpi.viz_setDataResult(resultTrellis);
  kpi.viz_drawVisual('chart-div');

  // store elements on the page to be tested
  this.viz_div = d3.select('#chart-div');
  this.kpiContainers = this.viz_div.selectAll('div.kpi-container');
  this.compareContainer = this.viz_div.selectAll('.kpi-container .kpi-compare');

  const arrowContainer = $('.kpi-container .compare-arrow');

  assert.equal(
    this.kpiContainers.length,
    1,
    'There are 1 kpi_containers elements',
  );
  assert.equal(arrowContainer.length, 0, 'No arrow');
  assert.equal(this.compareContainer.length, 1, 'There is a compare container');
});

QUnit.test('Label shelf is functioning', function (assert) {
  // main chart object
  const kpi = arc.visual.plugins.kpi();

  // create the lib object
  const driver = arc.report.driver(kpi);
  driver.chart_div('chart-div');

  // provide the data
  const resultTrellis = arc.data.result();
  resultTrellis.rows([['68.5', '58.5', 'sales']]);
  const col0 = arc.data
    .column()
    .colname('salesThisYear')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('aggregates_shelf')
    .colindex(0);

  const col1 = arc.data
    .column()
    .colname('salesLastYear')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('compare_shelf')
    .colindex(1);

  const col2 = arc.data
    .column()
    .colname('label')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('label_shelf')
    .colindex(2);

  resultTrellis.columns([col0, col1, col2]).finalize();

  // create the visual by setting the data and then drawing it
  kpi.viz_setDataResult(resultTrellis);
  kpi.viz_drawVisual('chart-div');

  // store elements on the page to be tested
  this.viz_div = d3.select('#chart-div');
  this.kpiContainers = this.viz_div.selectAll('div.kpi-container');
  this.arrowContainers = this.viz_div.selectAll(
    '.kpi-container .compare-arrow',
  );
  this.compareContainer = this.viz_div.selectAll('.kpi-container .kpi-compare');

  assert.equal(
    this.kpiContainers.length,
    1,
    'There are 1 kpi_containers elements',
  );
  assert.equal(this.compareContainer.length, 1, 'There is a compare container');

  const labelText = $('.kpi-label').text();
  assert.equal(labelText, 'sales', 'label properly displays');
});

QUnit.test('Chart Element Present - Trellis', function (assert) {
  // main chart object
  const kpi = arc.visual.plugins.kpi();

  // create the lib object
  const driver = arc.report.driver(kpi);
  driver.chart_div('chart-div');

  // provide the data
  const resultTrellis = arc.data.result();
  const dataRows = [
    ['American Home Food Products', 100, 'Maypo', 4, 'A'],
    ['General Mills', 73, 'Crispy_Wheat_&_Raisins', 2, 'G'],
    ['General Mills', 99, 'Multi-Grain_Cheerios', 7, 'G'],
    ['Kellogs', 120, 'Fruitful_Bran', 3, 'K'],
  ];
  resultTrellis.rows(dataRows);
  const col0 = arc.data
    .column()
    .colname('manufacturer')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('x_shelf')
    .colindex(0);
  const col1 = arc.data
    .column()
    .colname('calories')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('y_shelf')
    .colindex(1);
  const col2 = arc.data
    .column()
    .colname('cereal_name')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colsource('dimensions_shelf')
    .colindex(2);
  const col3 = arc.data
    .column()
    .colname('protein_grams')
    .coltype('INT')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('aggregates_shelf')
    .colindex(3);
  const col4 = arc.data
    .column()
    .colname('manufacturer_code')
    .coltype('STRING')
    .colproperty(arc.report.columnTypeEnum.AGGREGATE)
    .colsource('tooltip_shelf')
    .colindex(4);
  resultTrellis.columns([col0, col1, col2, col3, col4]).finalize();

  const expectedColumnTitles = [col0.colname(), col1.colname()];
  const expectedManufacturers = [];

  for (let i = 0; i < dataRows.length; i++) {
    const nextRow = dataRows[i][0];
    // skip duplicates
    if (expectedManufacturers.indexOf(nextRow) < 0) {
      expectedManufacturers.push(nextRow);
    }
  }

  // create the visual by setting the data and then drawing it
  kpi.viz_setDataResult(resultTrellis);
  kpi.viz_drawVisual('chart-div');

  // store elements on the page to be tested
  this.viz_div = $('#chart-div');
  this.viz_table = this.viz_div.find('table.viz-trellis-table');
  this.headers = this.viz_table.find('tr.viz-tr-header');
  this.trs = this.viz_table.find('tr:not(.viz-tr-header)');
  this.tds = this.trs.find('td');
  this.kpiContainers = this.viz_table.find('div.kpi-container');
  this.column_titles = this.viz_table.find(
    'tr.viz-tr-header>td.viz-td-header>span',
  );
  this.sub_column_titles = [].slice
    .call(
      this.viz_table.find('tr.viz-tr-header>td.viz-td-header>span')[1]
        .parentNode.parentNode.children,
    )
    .slice(1);

  // test elements
  assert.equal(this.headers.length, 2, 'There are 2 tr header elements');
  assert.equal(this.trs.length, 4, 'There are 4 tr row elements');
  assert.equal(this.tds.length, 16, 'There are 16 td row elements');
  assert.equal(
    this.kpiContainers.length,
    4,
    'There are 4 kpi_containers elements',
  );

  for (let i = 0; i < this.column_titles.length; i++) {
    assert.equal(this.column_titles[i].textContent, expectedColumnTitles[i]);
  }

  for (let i = 0; i < this.sub_column_titles.length; i++) {
    assert.equal(
      this.sub_column_titles[i].textContent,
      expectedManufacturers[i],
    );
  }
});
