const arc_base_dual = arc.report.base;

QUnit.module('visuals_arc_line_dual_axis', {
  beforeEach() {
    // For this visual we need a bit bigger height to be able to draw the visual
    $('#chart-div').height(600);

    // main chart object
    const line = arc.visual.plugins.tline();

    arc.report.base = () => {
      const base = arc_base_dual();
      base.permitDetailData = () => true;
      return base;
    };

    // create the lib object
    const driver = arc.report.driver(line);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Palo Alto', 1123, 323],
      ['San Mateo', 12, 110],
      ['Raleigh', 240, 80],
      ['Chapel Hill', 100, 220],
    ]);
    const col0 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col1 = arc.data
      .column()
      .colname('quantity')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('amount')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('color_shelf')
      .secondaryaxis(true)
      .colindex(2);
    result.columns([col0, col1, col2]);

    // create the visual by setting the data and then drawing it
    line.viz_setDataResult(result);
    line.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.labels = $('table>tbody>tr>td.viz-td-header');
    this.canvas = $('.arc-viz-overlay-container>canvas');
    this.canvasMain = this.canvas.filter('.maincanvas');
    this.legendG = $('table>tbody>tr>td>div>svg>.viz-base-legend-g');
    this.xAxis = $(
      '.arc-viz-overlay-container>.viz-bar-svg>svg>.x.arc-viz-axis',
    );
    this.yAxis = $(
      '.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .y.arc-viz-axis',
    );
    this.axisLabel = $(
      '.arc-viz-overlay-container>.viz-bar-svg>.viz-bar-g .arc-viz-axis-label',
    );

    this.canvasSel = d3.selectAll(
      '.arc-viz-overlay-container>canvas.maincanvas',
    );
    this.canvasData = arc.canteen.getCanvasData(this.canvasSel);
  },

  afterEach() {
    arc.report.base = arc_base_dual;
    const $chartDiv = $('#chart-div');
    // Height is set back to 500 to not break the other visuals tests
    $chartDiv.height(500);
    $chartDiv.empty();
  },
});

QUnit.test('Line Plot and Legend Elements Present', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');

  assert.equal(
    this.svg.length,
    4,
    'There are 4 svg elements (chart, clip path, x axis, legend)',
  );
  assert.equal(this.labels.length, 0, 'There are 0 td label elements');
  assert.equal(this.canvas.length, 3);
  assert.equal(this.canvasMain.length, 1);
  assert.equal(
    this.legendG.length,
    2,
    'There are 2 g elements that contains the legend',
  );

  assert.equal(this.xAxis.length, 1, 'There are 1 x axis elements');
  assert.equal(this.yAxis.length, 2, 'There are 2 y axis elements');
  assert.equal(this.axisLabel.length, 3, 'There are 3 axis label element');
  assert.equal(xAxisTicks.length, 4, 'There are 4 marks total on the x axis');
  // assert.equal(yAxisTicks.length, 19, 'There are 19 marks total on the y axes');

  this.canvasData.forEach((d) => {
    assert.equal(arc.canteen.countMethod(d, 'moveTo'), 2);
    assert.equal(arc.canteen.countMethod(d, 'lineTo'), 6);
  });
});

QUnit.test('Line Plot Elements Correct', function (assert) {
  const xAxisTicks = this.xAxis.find('.tick');
  const yAxisTicks = this.yAxis.find('.tick');
  const xAxisLabels = [];
  const yAxisLabels = [];
  const axisLabel = [];
  const trellisLabels = [];
  const xAxisLabelsExpected = [
    'Palo Alto',
    'San Mateo',
    'Raleigh',
    'Chapel Hill',
  ];
  const yAxisLabelsExpected = [
    '0',
    '100',
    '200',
    '300',
    '400',
    '500',
    '600',
    '700',
    '800',
    '900',
    '1,000',
    '1,100',
    '0',
    '50',
    '100',
    '150',
    '200',
    '250',
    '300',
  ];

  const axisLabelExpected = ['city', 'quantity', 'amount'];
  const trellisLabelsExpected = [];
  const lineColorsExpected = ['#0097a7', '#7dcad2'];

  for (let i = 0; i < xAxisTicks.length; i++) {
    xAxisLabels.push(xAxisTicks[i].textContent);
  }

  for (let i = 0; i < yAxisTicks.length; i++) {
    yAxisLabels.push(yAxisTicks[i].textContent);
  }

  for (let i = 0; i < this.labels.length; i++) {
    trellisLabels.push(this.labels[i].textContent);
  }

  for (let i = 0; i < this.axisLabel.length; i++) {
    axisLabel.push(this.axisLabel[i].textContent);
  }

  assert.deepEqual(
    xAxisLabels,
    xAxisLabelsExpected,
    'The x axis labels are correct',
  );
  assert.deepEqual(axisLabel, axisLabelExpected, 'The axis labels are correct');
  assert.deepEqual(
    trellisLabels,
    trellisLabelsExpected,
    'The trellis labels are correct',
  );
  assert.deepEqual(
    yAxisLabels,
    yAxisLabelsExpected,
    'the y axis labels are correct',
  );

  this.canvasData.forEach((d) => {
    assert.deepEqual(arc.canteen.attrVals(d, 'fillStyle'), lineColorsExpected);
    assert.deepEqual(
      arc.canteen.attrVals(d, 'strokeStyle'),
      lineColorsExpected,
    );
  });

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[0], 'moveTo'),
      [
        [37.589, 0],
        [37.589, 0],
      ],
      60,
    ),
  );

  assert.ok(
    arc.misc.deep_array_within_tolerance(
      arc.canteen.methodArgs(this.canvasData[0], 'lineTo'),
      [
        [117.584, 485.736],
        [195.974, 386.053],
        [274.364, 447.262],
        [117.584, 323.775],
        [195.974, 369.377],
        [274.364, 156.567],
      ],
    ),
  );
});

QUnit.test('Legend Elements Correct', function (assert) {
  const rectangles = this.legendG[0].getElementsByTagName('rect');
  const labels = this.legendG[0].getElementsByTagName('text');
  const rectangleColors = [];
  const labelTexts = [];
  const rectangleColorsExpected = ['#0097a7', '#7dcad2'];
  const labelTextsExpected = ['quantity', 'amount'];

  assert.equal(rectangles.length, 2, 'The legend has 2 rectangles');
  assert.equal(labels.length, 2, 'The legend has 2 labels');

  for (let i = 0; i < rectangles.length; i++) {
    rectangleColors.push(rectangles[i].style.fill);
    labelTexts.push(labels[i].textContent);
  }

  for (let i = 0; i < rectangleColors.length; i++) {
    rectangleColors[i] = arc.misc.colorToHex(rectangleColors[i]);
  }

  assert.deepEqual(
    rectangleColors,
    rectangleColorsExpected,
    'The legend has the correctly colored rectangles',
  );
  assert.deepEqual(
    labelTexts,
    labelTextsExpected,
    'The legend has the correct labels',
  );
});
