QUnit.module('visuals_arc_misc', {
  afterEach: () => {
    sinon.restore();
  },
});

QUnit.test('isIdentical', (assert) => {
  assert.equal(arc.misc.isIdentical({}, {}), true, 'Empty Objects');
  assert.equal(arc.misc.isIdentical([], []), true, 'Empty Arrays');
  assert.equal(arc.misc.isIdentical({}, []), false, 'Object vs Array');
  assert.equal(arc.misc.isIdentical([], true), false, 'Array vs Bool');
  assert.equal(arc.misc.isIdentical(true, true), true, 'Bool vs Bool');
  assert.equal(
    arc.misc.isIdentical(null, undefined),
    false,
    'null vs undefined',
  );
  assert.equal(
    arc.misc.isIdentical([null], [null]),
    true,
    '2 Identical arrays with nulls',
  );
  assert.equal(
    arc.misc.isIdentical([null], []),
    false,
    '2 Different arrays with nulls',
  );
  assert.equal(
    arc.misc.isIdentical([1], ['1']),
    false,
    '2 arrays with different data types',
  );
  assert.equal(
    arc.misc.isIdentical(
      [1, 2, 3, 4, -125, 0, 3.4, 21],
      [1, 2, 3, 4, -125, 0, 3.4, 21],
    ),
    true,
    'Same Filled Arrays',
  );
  assert.equal(
    arc.misc.isIdentical(
      [1, 2, 3, 4, -125, 0, 3.4, 21, undefined],
      [1, 2, 3, 4, -125, 0, 3.4, 21],
    ),
    false,
    'Different Filled Arrays - 1st one has more elements',
  );
  assert.equal(
    arc.misc.isIdentical(
      [1, 2, 3, 4, -125, 0, 3.4, 21],
      [1, 2, 3, 4, -125, 0, 3.4, 21, undefined],
    ),
    false,
    'Different Filled Arrays - 2nd one has more elements',
  );
  assert.equal(
    arc.misc.isIdentical(
      ['a', 1, null, [2], [undefined], 3, 'orange'],
      ['a', 1, null, [2], [undefined], 3, 'orange'],
    ),
    true,
    'Same Filled Arrays',
  );
  assert.equal(
    arc.misc.isIdentical(
      { red: 1, green: '2', blue: false, orange: -5 },
      { red: 1, green: '2', blue: false, orange: -5 },
    ),
    true,
    'Same Filled Objects',
  );
  assert.equal(
    arc.misc.isIdentical(
      { red: 1, green: '2', blue: false, orange: { pink: 6 } },
      { red: 1, green: '2', blue: false, orange: { pink: 6 } },
    ),
    true,
    'Same Filled Objects',
  );
  assert.equal(
    arc.misc.isIdentical(
      {
        red: 1,
        green: '2',
        blue: false,
        orange: { pink: 6 },
        magenta: undefined,
      },
      { red: 1, green: '2', blue: false, orange: { pink: 6 } },
    ),
    false,
    'Different Filled Objects - 1st one has more elements',
  );
  assert.equal(
    arc.misc.isIdentical(
      { red: 1, green: '2', blue: false, orange: { pink: 6 } },
      {
        red: 1,
        green: '2',
        blue: false,
        orange: { pink: 6 },
        magenta: undefined,
      },
    ),
    false,
    'Different Filled Objects - 2nd one has more elements',
  );
  assert.equal(
    arc.misc.isIdentical(
      {
        red: 1,
        green: '2',
        blue: false,
        orange: {
          apple: 6,
          mango: 7,
          kiwi: 8,
          ping: { first: 1, second: 2, third: ['alpha', 2, 3, 4, 5, { b: 5 }] },
        },
      },
      {
        red: 1,
        green: '2',
        blue: false,
        orange: {
          apple: 6,
          mango: 7,
          kiwi: 8,
          ping: { first: 1, second: 2, third: ['alpha', 2, 3, 4, 5, { b: 5 }] },
        },
      },
    ),
    true,
    'Same complexly nested Objects',
  );
  assert.equal(
    arc.misc.isIdentical(
      {
        red: 1,
        green: '2',
        blue: false,
        orange: {
          apple: 6,
          mango: 7,
          kiwi: 8,
          ping: { first: 1, second: 2, third: ['alpha', 2, 3, 4, 5, { b: 5 }] },
        },
      },
      {
        red: 1,
        green: '2',
        blue: false,
        orange: {
          apple: 6,
          mango: 7,
          kiwi: 8,
          ping: { first: 1, second: 2, third: ['alpha', 2, 3, 4, 5, { b: 6 }] },
        },
      },
    ),
    false,
    'Different complexly nested Objects',
  );
});

QUnit.test('isNumberType', (assert) => {
  assert.equal(arc.misc.isNumberType('1800'), false, 'String as number');
  assert.equal(arc.misc.isNumberType(1800), true, 'Integer');
  assert.equal(arc.misc.isNumberType(1800.123), true, 'Real');
  assert.equal(arc.misc.isNumberType(0), true, 'Zero');
  // eslint-disable-next-line no-new-wrappers
  assert.equal(arc.misc.isNumberType(new Number(1800)), false, 'Number');
  // eslint-disable-next-line no-new-wrappers
  assert.equal(arc.misc.isNumberType(new Number('1800')), false, 'Number');
});

QUnit.test('extend', (assert) => {
  const baseObject = {};
  let addOnObject = [];
  let resultObject;
  resultObject = arc.misc.extend(baseObject, addOnObject);
  assert.equal(
    arc.misc.isIdentical(addOnObject, resultObject),
    true,
    'New object has identical props',
  );

  addOnObject = {};
  resultObject = arc.misc.extend(baseObject, addOnObject);
  assert.equal(
    arc.misc.isIdentical(addOnObject, resultObject),
    true,
    'New object has identical props',
  );

  addOnObject = {
    str: 'string',
    num: 5,
    arr: [1, 2, 3, 4, 5],
    obj: {
      subarr: [6, 7, 8, 9],
    },
    fun() {},
  };
  resultObject = arc.misc.extend(baseObject, addOnObject);
  assert.equal(
    arc.misc.isIdentical(addOnObject, resultObject),
    true,
    'New object has identical props',
  );
  resultObject.arr.push(6);
  assert.equal(
    arc.misc.isIdentical(addOnObject.arr, resultObject.arr),
    false,
    'New obj does not point to same array',
  );
  resultObject.obj.subarr.push(0);
  assert.equal(
    arc.misc.isIdentical(addOnObject.obj.subarr, resultObject.obj.subarr),
    false,
    'New obj does not point to same obj',
  );
  resultObject.fun.newProperty = 6;
  assert.equal(
    arc.misc.isIdentical(
      addOnObject.fun.newProperty,
      resultObject.fun.newProperty,
    ),
    true,
    'New obj points to same function',
  );

  addOnObject = null;
  resultObject = arc.misc.extend(baseObject, addOnObject);
  assert.equal(arc.misc.isIdentical(addOnObject, resultObject), true);
});

QUnit.test('join expression validation', (assert) => {
  const $exprValBtn = $('<button/>');
  const $exprSubmitBtn = $('<button/>');
  // Ace editor mock:
  const exprEditor = { value: () => '1=1' };
  // sample dataset with two tables:
  const dataset = arc.datasets
    .Dataset()
    .dataset_name('testds')
    .dataset_type('multitable');
  const exprType = 'join';
  const viz = null;
  const config = {};

  // daset configuration
  const tableA = arc.testutil.ds.create_table(dataset, 'a');
  const tableB = arc.testutil.ds.create_table(dataset, 'b');
  const tableC = arc.testutil.ds.create_table(dataset, 'c');
  const tableD = arc.testutil.ds.create_table(dataset, 'd');
  dataset.tables([tableA, tableB, tableC, tableD]);

  tableB.parent_tablename(tableA.tablename());
  tableB.join_pairs([['col1', 'col1']]);
  tableB.join_exprs(['1=1']);

  tableC.parent_tablename(tableB.tablename());
  tableC.join_pairs([['col2', 'col2']]);
  tableC.join_exprs(['2=2']);

  tableD.parent_tablename(tableC.tablename());
  tableD.join_pairs([['col3', 'col3']]);
  tableD.join_exprs(['3=3']);

  const validator = arc.misc.addExpressionValidation(
    $exprValBtn,
    $exprSubmitBtn,
    exprEditor,
    dataset,
    exprType,
    viz,
    config,
  );

  assert.ok(validator);

  const editedParentTable = tableB.clone();
  const editedChildTable = tableC.clone();

  editedChildTable.join_pairs([['col0', 'col0']]);
  editedChildTable.join_exprs(['0=0']);

  const tableInfo = {
    join_exprs: [],
    src_table: {
      dstab: editedParentTable,
    },
    dstab: editedChildTable,
  };

  sinon.stub(arc.admin, 'dataset').returns({
    readJoinFromForm: sinon.stub().returns(tableInfo),
  });
  const runSQLWithParams = sinon.stub(arc.sqlrun, 'runSQLWithParams');
  validator.validate({ target: $exprValBtn[0] });
  assert.true(runSQLWithParams.calledOnce);

  const dsreq = JSON.parse(runSQLWithParams.args[0][0].dsreq);
  const tables = JSON.parse(dsreq.dataset_object);

  assert.equal(tables.length, 4, 'all tables must be used');

  assert.equal(
    tables[2].parent_tablename,
    'testdb.b',
    'the edited join must be used',
  );
  assert.deepEqual(
    tables[2].join_pairs,
    [['col0', 'col0']],
    'the edited join pair must be used',
  );
  assert.deepEqual(
    tables[2].join_exprs,
    ['0=0'],
    'the edited join expression must be used',
  );

  // the unedited joins are not included in the validation
  assert.deepEqual(tables[1].join_pairs, [], 'no join pairs');
  assert.deepEqual(tables[3].join_pairs, [], 'no join pairs');
  assert.deepEqual(tables[1].join_exprs, [], 'no join expressions');
  assert.deepEqual(tables[3].join_exprs, [], 'no join expressions');

  // but hierarchy (parent table) is preserved:
  assert.equal(
    tables[1].parent_tablename,
    'testdb.a',
    'parent table is preserved',
  );
  assert.equal(
    tables[2].parent_tablename,
    'testdb.b',
    'parent table is preserved',
  );
  assert.equal(
    tables[3].parent_tablename,
    'testdb.c',
    'parent table is preserved',
  );

  // originals must remain unchanged:
  assert.equal(tableA.parent_tablename(), undefined);
  assert.equal(tableB.parent_tablename(), 'testdb.a');
  assert.equal(tableC.parent_tablename(), 'testdb.b');
  assert.equal(tableD.parent_tablename(), 'testdb.c');

  assert.deepEqual(tableB.join_pairs(), [['col1', 'col1']]);
  assert.deepEqual(tableC.join_pairs(), [['col2', 'col2']]);
  assert.deepEqual(tableD.join_pairs(), [['col3', 'col3']]);

  assert.deepEqual(tableB.join_exprs(), ['1=1']);
  assert.deepEqual(tableC.join_exprs(), ['2=2']);
  assert.deepEqual(tableD.join_exprs(), ['3=3']);
});

QUnit.test('determineDatabaseInQuery', (assert) => {
  const testData = [
    {
      query: 'select  * from         databaseNameLotsOfSpaceBefore.trips_detail inner join main.trips on main.trips.pickup_neighborhood=main.trips_detail.pickup_neighborhood',
      expectedResult: 'databaseNameLotsOfSpaceBefore',
    },
    {
      query: 'select * from `databaseNameWithinBackTick.trips_detail` inner join main.trips on main.trips.pickup_neighborhood=main.trips_detail.pickup_neighborhood',
      expectedResult: 'databaseNameWithinBackTick',
    },
    {
      query: 'select  * from  `database name with spaces.trips detail` inner join main.trips on main.trips.pickup_neighborhood=main.trips_detail.pickup_neighborhood',
      expectedResult: 'database name with spaces',
    },
    {
      query: `WITH temporaryTable (averageValue) AS (
      SELECT AVG(Salary)
      FROM      'database name with space.table')
          SELECT EmployeeID,Name, Salary
          FROM Employee, temporaryTable
          WHERE Employee.Salary > temporaryTable.averageValue;`,
          expectedResult: `database name with space`,
    }
  ];

  testData.forEach((test) => {
    assert.equal(arc.misc.determineDatabaseInQuery(test.query), test.expectedResult);
  });
});
