QUnit.module('visuals_arc_packedbubbles', {
  beforeEach() {
    // main chart object
    const packedbubbles = arc.visual.plugins.pb();

    // create the lib object
    const driver = arc.report.driver(packedbubbles);
    driver
      .chart_div('chart-div')
      // This should not be needed, remove this when visual gets updated
      .dim_start_idx(0)
      .agg_start_idx(1);

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['MA', '2000'],
      ['OR', '4000'],
      ['AL', '6000'],
      ['NC', '7000'],
      ['SC', '8000'],
      ['CA', '9000'],
    ]);

    const col1 = arc.data
      .column()
      .colname('State')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col2 = arc.data
      .column()
      .colname('sum(population)')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(1);
    result.columns([col1, col2]);
    result.finalize();

    // create the visual by setting the data and then drawing it
    packedbubbles.viz_setDataResult(result);
    packedbubbles.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.viz_div = d3.select('#chart-div');
    this.canvas = this.viz_div.selectAll('canvas');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Chart Elements Present', function (assert) {
  assert.equal(this.canvas.size(), 3, 'There are three canvas elements');
});

QUnit.test('Bubble properties', function (assert) {
  const plots = arc.canteen.getCanvasData(this.canvas);

  assert.equal(arc.canteen.countMethod(plots[0], 'arc'), 6);
  assert.equal(arc.canteen.countMethod(plots[1], 'arc'), 6);
  assert.equal(arc.canteen.countMethod(plots[2], 'arc'), 0);

  assert.equal(arc.canteen.countMethod(plots[0], 'beginPath'), 6);
  assert.equal(arc.canteen.countMethod(plots[1], 'beginPath'), 6);
  assert.equal(arc.canteen.countMethod(plots[2], 'beginPath'), 0);

  assert.equal(arc.canteen.countMethod(plots[0], 'fillText'), 6);
  assert.equal(arc.canteen.countMethod(plots[1], 'fillText'), 0);
  assert.equal(arc.canteen.countMethod(plots[2], 'fillText'), 0);

  const arcs = arc.canteen.methodArgs(plots[0], 'arc');
  const fillTexts = arc.canteen.methodArgs(plots[0], 'fillText');

  const x = 0;
  const y = 1;
  const r = 2;
  const tolerance = 20;

  assert.equal(arcs.length, fillTexts.length);
  arcs.forEach((arc, i) => {
    assert.equal(arcs[i][x], fillTexts[i][1]);
    assert.equal(arcs[i][y], fillTexts[i][2]);
  });

  assert.equal(fillTexts[0][0], 'CA');
  assert.ok(arc.misc.within_tolerance(206, arcs[0][x], tolerance));
  assert.ok(arc.misc.within_tolerance(232, arcs[0][y], tolerance));
  assert.ok(
    arc.misc.within_tolerance(92, arcs[0][r], tolerance),
    'First bubble right size',
  );

  assert.equal(fillTexts[1][0], 'SC');
  assert.ok(arc.misc.within_tolerance(387, arcs[1][x], tolerance));
  assert.ok(arc.misc.within_tolerance(232, arcs[1][y], tolerance));
  assert.ok(
    arc.misc.within_tolerance(87, arcs[1][r], tolerance),
    'Second bubble right size',
  );

  assert.equal(fillTexts[2][0], 'NC');
  assert.ok(arc.misc.within_tolerance(302, arcs[2][x], tolerance));
  assert.ok(arc.misc.within_tolerance(380, arcs[2][y], tolerance));
  assert.ok(
    arc.misc.within_tolerance(81, arcs[2][r], tolerance),
    'Third bubble right size',
  );

  assert.equal(fillTexts[3][0], 'AL');
  assert.ok(arc.misc.within_tolerance(302, arcs[3][x], tolerance));
  assert.ok(arc.misc.within_tolerance(92, arcs[3][y], tolerance));
  assert.ok(
    arc.misc.within_tolerance(75, arcs[3][r], tolerance),
    'Fourth bubble right size',
  );

  assert.equal(fillTexts[4][0], 'OR');
  assert.ok(arc.misc.within_tolerance(163, arcs[4][x], tolerance));
  assert.ok(arc.misc.within_tolerance(82, arcs[4][y], tolerance));
  assert.ok(
    arc.misc.within_tolerance(61, arcs[4][r], tolerance),
    'Fifth bubble right size',
  );

  assert.equal(fillTexts[5][0], 'MA');
  assert.ok(arc.misc.within_tolerance(175, arcs[5][x], tolerance));
  assert.ok(arc.misc.within_tolerance(367, arcs[5][y], tolerance));
  assert.ok(
    arc.misc.within_tolerance(43, arcs[5][r], tolerance),
    'Sixth bubble right size',
  );
});
