const arcl = {};

async function smallWait(timeToWait = 300) {
  // have issues with waiting for all promises to settle
  // this is a workaround until we find a better way
  return new Promise((resolve) => {
    setTimeout(resolve, timeToWait);
  });
}

QUnit.module('visuals_arc_picklist', {
  beforeEach() {
    arc.globals.filter_scope_hash = {};
    arcl.picklist = arc.visual.plugins.picklist().expanded(false);
    arcl.driver = arc.report
      .driver(arcl.picklist)
      .chart_div('chart-div')
      .shelves_dom_id('app-shelves');

    const table = arc.datasets.Table()
        .tablename('default.census_pop');
    table.init_from_add_sq(['state'], ['string']);

    const dataset = arc.datasets.Dataset().add_table(table);

    arcl.driver.dataset(dataset);
    arc.globals.reports = arc.reporttracker.reports();
    arcl.backupObjects = {
      click_behavior_cb: arc.globals.click_behavior_cb,
    };
  },

  afterEach() {
    $('#chart-div').empty();
    $('#app-shelves').empty();
    $('#subtitle-div').empty();
    $('#filters-save-as-segments-html').empty();
    arc.globals.click_behavior_cb = arcl.backupObjects.click_behavior_cb;
  },
});

function setupStatePicker() {
  const col = arc.data
    .column()
    .colsource('base_shelf')
    .coltype('string')
    .colproperty(arc.report.columnTypeEnum.DIMENSION)
    .colname('state');
  const data_result = arc.data.result();
  data_result.add_column(col);
  data_result.rows([['CA'], ['AZ'], ['NV']]);

  arcl.picklist.base_field('state');
  arcl.picklist.viz_setDataResult(data_result);
  arcl.picklist.viz_drawVisual('chart-div');
  arcl.testutil = arcl.testutil || {};
  arcl.testutil.mockdata = arcl.testutil.mockdata || {};
  arcl.testutil.mockdata.data_result = arcl.testutil.mockdata.data_result =
    data_result;
}

async function searchPicklist(value, dataRows) {
  if (typeof value === 'undefined') {
    arc.testutil.searchResponse.data.rows =
      arcl.testutil.mockdata.data_result.rows();
  } else if (dataRows) {
    arc.testutil.searchResponse.data.rows = dataRows;
  }

  const $searchBoxSelectorBox = $('#chart-div input.picklist-searchbox');

  if (value && !$searchBoxSelectorBox.is(':visible')) {
    $('#chart-div .selected-picklist-link').click();
  }

  $searchBoxSelectorBox.val(value || '');
  $searchBoxSelectorBox.trigger('keyup');
  // Because of the debounce on the input it is necessary to see its effect.
  await smallWait();
}

// If no title is passed, default Title is expected
function testForPicklistTitle(assert, expectedTitle, message) {
  const $chartDiv = $('#chart-div');
  message = message || '';
  expectedTitle = expectedTitle || '';

  assert.equal(
    arcl.picklist.picklist_title.value(),
    expectedTitle,
    `${message} object value`,
  );
  assert.equal(
    $chartDiv.find('.base_field').text().trim(),
    expectedTitle,
    `${message} HTML text`,
  );
}

// If no expectedLabel is passed, default Label is expected
function testForPicklistSelectedLabels(assert, expectedLabel, message) {
  const $chartDiv = $('#chart-div');
  message = message || '';
  if (!expectedLabel) {
    assert.equal(
      $chartDiv
        .find('.selected-picklist-link .selected-names-wrapper')
        .css('display'),
      'none',
      `Default Label Visibility is correct. ${message}`,
    );
    assert.equal(
      $chartDiv
        .find('.selected-picklist-link .selected-count-wrapper .selected-count')
        .text()
        .trim(),
      '',
      `Default count label is correct. ${message}`,
    );
    assert.equal(
      $chartDiv
        .find(
          '.selected-picklist-link .selected-count-wrapper .selected-count-suffix',
        )
        .text()
        .trim(),
      '(All)',
      `Default count label is correct. ${message}`,
    );
  } else {
    assert.equal(
      $chartDiv
        .find('.selected-picklist-link .selected-names-wrapper')
        .css('display'),
      'flex',
      `Label Visibility is correct. ${message}`,
    );
    assert.equal(
      $chartDiv
        .find('.selected-picklist-link .selected-names-wrapper .selected-names')
        .text()
        .trim(),
      expectedLabel,
      `Label is correct. ${message}`,
    );
  }
}

function getPicklistOptions(picklistType) {
  let $options = $();
  const $chartDiv = $('#chart-div');
  switch (picklistType) {
    case 'multi.checkbox':
      $options = $chartDiv.find('.picklist-list input');
      break;
    case 'single.radio':
      $options = $chartDiv.find('.picklist-list input');
      break;
    case 'single.dropdown':
      $options = $chartDiv.find('.picklist-dropdown option');
      break;
    case 'textbox':
      break;
    default:
      break;
  }

  return $options;
}

function testForPicklistDropdownOptions(assert, picklistType, expectedOptions) {
  const $options = getPicklistOptions(picklistType);
  let $opt;
  $.each($options, (idx, opt) => {
    $opt = $(opt);
    assert.equal($opt.data('value'), expectedOptions[idx]);
  });
  assert.equal(
    $options.length,
    expectedOptions.length,
    'Valid number of options displayed',
  );
  testForDisplayCountLabel(assert, picklistType, expectedOptions);
}

function testForDisplayCountLabel(assert, picklistType, expectedOptions) {
  let noneCount = 0;
  if (
    picklistType.split('.')[0] === 'single' &&
    expectedOptions[0] === 'Select'
  ) {
    noneCount = 1;
  }

  assert.equal(
    $('#chart-div .picklist-displayed .count').text().trim(),
    expectedOptions.length - noneCount,
    'Valid display count label',
  );
}

function testForPicklistCheckBoxStatusSelectAll(assert, expectedChecked) {
  const $selectAll = $('#chart-div .picklist-list-wrapper .picklist-all');
  expectedChecked = expectedChecked || false;
  assert.equal(
    $selectAll.css('display'),
    'inline-block',
    'Valid visibility for Select All',
  );
  assert.equal(
    $selectAll.prop('checked'),
    expectedChecked,
    'Valid status of Select All checkbox',
  );
}

function testForPicklistCheckBoxStatusExcludeSelected(assert, expectedChecked) {
  const $excludeAll = $('#chart-div .picklist-list-wrapper .picklist-exclude');
  expectedChecked = expectedChecked || false;
  assert.equal(
    $excludeAll.parent().text().trim(),
    'Exclude Selected',
    "Valid label for 'Exclude Selected Options'",
  );
  assert.equal(
    $excludeAll.css('display').indexOf('block') > 0,
    true,
    "Valid visibility for 'Exclude Selected Options'",
  );
  assert.equal(
    $excludeAll.prop('checked'),
    expectedChecked,
    "Valid status of 'Exclude Selected Options' checkbox",
  );
}

function testForPicklistCheckBoxStatusOptions(
  assert,
  picklistType,
  expectedOptions,
) {
  const $options = getPicklistOptions(picklistType);
  let $opt;
  let selectedOptionsCount = 0;
  $.each($options, (idx, opt) => {
    $opt = $(opt);
    assert.equal(
      $opt.prop('checked'),
      expectedOptions.indexOf($opt.data('value')) >= 0,
    );
    selectedOptionsCount = $opt.prop('checked') ?
      selectedOptionsCount + 1 :
      selectedOptionsCount;
  });
  assert.equal(
    selectedOptionsCount,
    expectedOptions.length,
    'Valid count of selected options',
  );
}

function getParamInArr(paramArr, paramName) {
  const filteredArr =
    paramArr &&
    paramArr.filter((param) => param.key && param.key === paramName);

  return filteredArr && filteredArr.length === 1 ? filteredArr[0] : filteredArr;
}

function defineClickCbs(callbackTrackerObj) {
  const originalDashboardClick = arcl.driver.dashboard_click;
  arcl.driver.dashboard_click = function (in_params) {
    // eslint-disable-next-line no-plusplus
    callbackTrackerObj.call_count++;
    callbackTrackerObj.stored_params = {};
    arc.params.convertToOldFormatArr(in_params).forEach((param) => {
      const paramExtension = arc.params.splitParams(param.key)[1];
      if (paramExtension === 'data') {
        callbackTrackerObj.stored_params[param.key] = param.value_list;
      } else {
        callbackTrackerObj.stored_params[param.key] = param.value;
      }
    });
    // eslint-disable-next-line prefer-rest-params
    originalDashboardClick.apply(this, arguments);
  };

  arc.globals.click_behavior_cb = function (a, b, c, d, base, f, in_params) {
    callbackTrackerObj.stored_params_arr = in_params;
  };
}

// eslint-disable-next-line no-undef
QUnit.test('Designer', (assert) => {
  arcl.picklist.viz_drawDesigner('app-shelves');
  assert.equal($('#app-shelves').html().trim(), '');
});

// eslint-disable-next-line no-undef

const testConfig = [
  ['teradata', [{ expr: "sum(1) as 'Record Count'" }]],
  ['solr', [{ expr: "sum(1) as 'Record Count'" }]],
  ['ksql', [{ expr: 'count(*)' }]],
  ['elasticsearch', [{ expr: "count(*) as 'Record Count'" }]],
  ['mysql', []],
  ['sqlite', []],
  ['impala', []],
  ['hive', [], false],
  ['hive', [{ expr: "sum(1) as 'Record Count'" }], true],
];

testConfig.forEach((test) => {
  QUnit.test('Data Request', (assert) => {
    const testConnection = test[0];
    const testAggregate = test[1];
    const testImproveHideSortSettingsValue = testConnection === 'hive' ? test[2] : false;
    arcl.driver.dataset().dataconnection_type(testConnection);
    const user_settings = arcl.picklist.viz_lib().user_settings();
    const settings = user_settings.settings();
    user_settings.find_by_id('improve_hive_distinct_request').value(`${testImproveHideSortSettingsValue}`);

    arcl.picklist.viz_drawDesigner('app-shelves');
    let data_request = arcl.picklist.viz_getDataRequest();
    assert.equal(data_request.request_type(), 'ignore');
    assert.equal(data_request.columns().length, 0);

    arcl.picklist.base_field('state');
    data_request = arcl.picklist.viz_getDataRequest();
    assert.equal(data_request.request_type(), 'sql');
    assert.deepEqual(
      JSON.parse(data_request.dsreq()),
      {
        version: 1,
        type: testConnection === 'solr' ? 'SOLR' : 'SQL',
        limit: 1001,
        time: [],
        session: [],
        dimensions: [
          {
            expr: "[state] as 'state'",
            order: {
              asc: true
            },
            type: 'SIMPLE'
          }
        ],
        aggregates: testAggregate,
        autotime_override: '',
        ksqlStreamPosition: '',
        partitions: [],
        filters: [],
        having: [],
        highlighting: false,
        global_segments: [],
        outerFilters: [],
        events: [],
        distinct: !testAggregate.length,
        derived_data: [],
        solr_query: []
      }
    );
    assert.equal(data_request.columns().length, 1);
    assert.equal(settings.length, 28);

    let itemcount_idx;
    let searchcount_idx;
    const ids = settings.map((s, i) => {
      if (s.id() === 'picklist_itemcount') {
        itemcount_idx = i;
      }

      if (s.id() === 'picklist_searchcount') {
        searchcount_idx = i;
      }

      return s.id();
    });

    assert.equal(arc.misc.isIdentical(ids,
      [
        'container_css_class',
        'css_styles',
        'inline_css',
        'ksql_stream_position',
        'picklist_dataset',
        'picklist_textbox',
        'picklist_width',
        'picklist_upload',
        'picklist_remember_in_viewmode',
        'allow_nulls',
        'picklist_base_field',
        'picklist_title',
        'picklist_output_parameter',
        'picklist_filter_columns',
        'picklist_itemcount',
        'picklist_searchcount',
        'picklist_single_select',
        'picklist_dropdown',
        'picklist_display_no_selection',
        'picklist_multi_parm',
        'picklist_values',
        'picklist_hide_filter_when_empty',
        'filter_appgroup_scope',
        'filter_dataset_scope',
        'remember_selections',
        'picklist_apply_selections_together',
        'filter_options_sort_order',
        'improve_hive_distinct_request',
      ]), true, 'Checking if all settings are present for picklists');

    const itemcount = settings[itemcount_idx];
    const searchcountSettings = settings[searchcount_idx];
    itemcount.value('20');
    searchcountSettings.value('32');
    data_request = arcl.picklist.viz_getDataRequest();
    assert.equal(data_request.request_type(), 'sql');
    assert.deepEqual(JSON.parse(data_request.dsreq()),
      {
        version: 1,
        type: testConnection === 'solr' ? 'SOLR' : 'SQL',
        limit: 33,
        time: [],
        session: [],
        dimensions: [
          {
            expr: "[state] as 'state'",
            order: {
              asc: true
            },
            type: 'SIMPLE'
          }
        ],
        aggregates: testAggregate,
        autotime_override: '',
        ksqlStreamPosition: '',
        partitions: [],
        filters: [],
        having: [],
        highlighting: false,
        global_segments: [],
        outerFilters: [],
        events: [],
        distinct: !testAggregate.length,
        derived_data: [],
        solr_query: [],
      });

    assert.equal(data_request.columns().length, 1);
    const col = data_request.columns()[0];
    assert.equal(col.colsource(), 'base_shelf');
    assert.equal(col.coltype(), 'string');
    assert.equal(col.colproperty(), arc.report.columnTypeEnum.DIMENSION);
    assert.equal(col.colname(), 'state');
  });
});

// eslint-disable-next-line no-undef
QUnit.test('Title display', (assert) => {
  setupStatePicker();

  assert.equal($('#chart-div .control-list .base_field').text(), 'state');

  assert.equal(arcl.driver.subtitle(), '');
  arcl.picklist.picklist_title.value('state title');
  arcl.driver.subtitle_elem('subtitle-div');
  arcl.picklist.viz_drawVisual('chart-div');
  assert.equal($('#chart-div .control-list .base_field').text(), 'state title');
  assert.equal(arcl.driver.subtitle(), '');
});

// eslint-disable-next-line no-undef
QUnit.test('Multi-select picklist parameters', (assert) => {
  /*
   * Verify that stored parameters, and incoming parameters, are used
   *   correctly by multi-select picklists
   */
  const statusTracker = {
    call_count: 0,
    stored_params: undefined,
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);
  let param;

  let settings = {
    user_settings: {
      picklist_base_field: 'state',
    },
  };

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 0);

  settings = {
    user_settings: {
      picklist_base_field: 'state',
    },
    viz_type: {
      picklist: {
        default_selections: {
          coltype: 'STRING',
          all_selected: false,
          data: ['CA', 'AZ'],
          alias: ['CA', 'AZ'],
          exclude: false,
        },
      },
    },
  };

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 1);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 3);
  assert.deepEqual(statusTracker.stored_params['state.data'], ['CA', 'AZ']);
  assert.equal(statusTracker.stored_params['state.exclude'], 'in');
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'true');
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, "'CA', 'AZ'");
  assert.deepEqual(param.value_list, ['CA', 'AZ']);
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, 'in');

  const params = arc.params
    .tracker()
    .add({
      key: 'state.data',
      value: "'NV', 'AZ'",
    })
    .add({
      key: 'state.exclude',
      value: 'not in',
    });
  arcl.driver.params(params);

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 2);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 3);

  assert.deepEqual(statusTracker.stored_params['state.data'], ['NV', 'AZ']);
  assert.equal(statusTracker.stored_params['state.exclude'], 'not in');
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'true');
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(param.value, "'NV', 'AZ'");
  assert.deepEqual(param.value_list, ['NV', 'AZ']);
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(param.value, 'not in');
});

// eslint-disable-next-line no-undef
QUnit.test(
  'Setup and Click behavior on Multi-select picklist when apply selections together is enabled',
  async (assert) => {
    const done = assert.async();
    /*
     * Verify that stored parameters, and incoming parameters, are used
     *   correctly by multi-select picklists and on click new changes are updated at right time.
     */
    const statusTracker = {
      call_count: 0,
      stored_params: {},
      stored_params_arr: undefined,
    };
    defineClickCbs(statusTracker);
    let param;

    assert.equal(statusTracker.call_count, 0);
    arcl.picklist.picklist_apply_selections_together.value('true');
    setupStatePicker();
    assert.equal(
      statusTracker.call_count,
      0,
      'Dashboard click should not occur on creation of picklist',
    );
    assert.equal(Object.keys(statusTracker.stored_params).length, 0);
    assert.equal(statusTracker.stored_params['state.data'], undefined);
    assert.equal(statusTracker.stored_params['state.exclude'], undefined);
    assert.equal(statusTracker.stored_params['state.include_nulls'], undefined);
    const inputs = $('#chart-div .picklist-list input');

    testForPicklistTitle(assert, 'state');
    testForPicklistSelectedLabels(assert);
    testForPicklistDropdownOptions(assert, 'multi.checkbox', [
      'CA',
      'AZ',
      'NV',
    ]);
    testForPicklistCheckBoxStatusSelectAll(assert, false);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', []);

    inputs.first().prop('checked', true);
    inputs.first().trigger('change');
    assert.equal(
      statusTracker.call_count,
      0,
      'Dashboard click should not occur on change of params till user clicks outside picklist',
    );
    assert.equal(
      !!statusTracker.stored_params,
      true,
      'stored_params is defined',
    );
    assert.equal(
      Object.keys(statusTracker.stored_params).length,
      0,
      'Valid count of params after change in multi-select (checkbox) picklist but before applying',
    );
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', ['CA']);

    $('body').trigger('click');
    testForPicklistSelectedLabels(assert, 'CA');
    assert.equal(
      statusTracker.call_count,
      1,
      'Dashboard click should occur when user clicks outside picklist',
    );
    assert.equal(
      !!statusTracker.stored_params,
      true,
      'stored_params is defined',
    );
    assert.equal(
      Object.keys(statusTracker.stored_params).length,
      3,
      'Valid count of params after change in multi-select (checkbox) picklist and after applying',
    );
    assert.deepEqual(statusTracker.stored_params['state.data'], ['CA']);
    assert.equal(statusTracker.stored_params['state.exclude'], 'in');
    param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
    assert.equal(param.value, "'CA'");
    assert.deepEqual(param.value_list, ['CA']);
    param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
    assert.equal(param.value, 'in');
    // The click on the body triggers a process which finished later than this tests,
    // ruining the others.
    // I added wait here even if it is ugly but eleminates this issue
    await smallWait(500);
    done();
  },
);

QUnit.test(
  'Multi-select picklist test for show selections behavior',
  (assert) => {
    const statusTracker = {
      call_count: 0,
      stored_params: {},
      stored_params_arr: undefined,
    };
    arc.testutil.isSearchOn = true;
    arc.testutil.searchResponse = {
      error: null,
      data: { rows: [['CA'], ['AZ'], ['NV']] },
    };
    defineClickCbs(statusTracker);

    setupStatePicker();
    const $showSelectedItems = $(
      '#chart-div .picklist-displayed .show-selected-items',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', [
      'CA',
      'AZ',
      'NV',
    ]);
    testForPicklistCheckBoxStatusSelectAll(assert, false);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', []);

    assert.equal(
      $showSelectedItems.text().trim(),
      'Show Selected',
      'Valid Show Selected Label',
    );
    assert.equal(
      $showSelectedItems.css('display'),
      'block',
      'Valid Show Selected Visibility',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', [
      'CA',
      'AZ',
      'NV',
    ]);
    testForPicklistCheckBoxStatusSelectAll(assert, false);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', []);
    $showSelectedItems.trigger('click');

    assert.equal(
      $showSelectedItems.text().trim(),
      'Show All',
      'Valid Show Selected Label',
    );
    assert.equal(
      $showSelectedItems.css('display'),
      'block',
      'Valid Show Selected Visibility',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', []);
    testForPicklistCheckBoxStatusSelectAll(assert, true);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', []);

    $showSelectedItems.trigger('click');

    assert.equal(
      $showSelectedItems.text().trim(),
      'Show Selected',
      'Valid Show Selected Label',
    );
    assert.equal(
      $showSelectedItems.css('display'),
      'block',
      'Valid Show Selected Visibility',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', [
      'CA',
      'AZ',
      'NV',
    ]);
    testForPicklistCheckBoxStatusSelectAll(assert, false);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', []);

    const $inputs = getPicklistOptions('multi.checkbox');
    $inputs.first().prop('checked', true);
    $inputs.first().trigger('change');

    assert.equal(
      $showSelectedItems.text().trim(),
      'Show Selected',
      'Valid Show Selected Label',
    );
    assert.equal(
      $showSelectedItems.css('display'),
      'block',
      'Valid Show Selected Visibility',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', [
      'CA',
      'AZ',
      'NV',
    ]);
    testForPicklistCheckBoxStatusSelectAll(assert, false);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', ['CA']);
    $showSelectedItems.trigger('click');

    assert.equal(
      $showSelectedItems.text().trim(),
      'Show All',
      'Valid Show Selected Label',
    );
    assert.equal(
      $showSelectedItems.css('display'),
      'block',
      'Valid Show Selected Visibility',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', ['CA']);
    testForPicklistCheckBoxStatusSelectAll(assert, true);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', ['CA']);

    $showSelectedItems.trigger('click');

    assert.equal(
      $showSelectedItems.text().trim(),
      'Show Selected',
      'Valid Show Selected Label',
    );
    assert.equal(
      $showSelectedItems.css('display'),
      'block',
      'Valid Show Selected Visibility',
    );
    testForPicklistDropdownOptions(assert, 'multi.checkbox', [
      'CA',
      'AZ',
      'NV',
    ]);
    testForPicklistCheckBoxStatusSelectAll(assert, false);
    testForPicklistCheckBoxStatusExcludeSelected(assert, false);
    testForPicklistCheckBoxStatusOptions(assert, 'multi.checkbox', ['CA']);
  },
);

QUnit.test('Single-select picklist parameters', (assert) => {
  /*
   * Verify that stored parameters, and incoming parameters, are used
   *   correctly by single-select picklists
   */
  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);

  let settings = {
    user_settings: {
      picklist_base_field: 'state',
      picklist_single_select: 'true',
    },
  };

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 0);

  settings = {
    user_settings: {
      picklist_base_field: 'state',
      picklist_single_select: 'true',
    },
    viz_type: {
      picklist: {
        default_selections: {
          coltype: 'STRING',
          all_selected: false,
          data: ['CA'],
          alias: ['CA'],
          exclude: false,
        },
      },
    },
  };

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 1);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'CA');
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'true');

  const params = arc.params.tracker().add({
    key: 'state',
    value: 'NV',
  });
  arcl.driver.params(params);

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 2);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'NV');
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'true');
});

QUnit.test('Textbox picklist parameters', (assert) => {
  /*
   * Verify that stored parameters, and incoming parameters, are used
   *   correctly by textboxes
   */
  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);

  // First, call viz_setSettings with only user settings (no default selections)
  let settings = {
    user_settings: {
      picklist_output_parameter: 'searchfield',
      picklist_textbox: 'true',
    },
  };

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 1, 'Dashboard click triggered for textbox even when no default selections provided');

  // Reset statusTracker for clarity.
  statusTracker.call_count = 0;
  statusTracker.stored_params = {};
  statusTracker.stored_params_arr = undefined;

  // Now provide default selections with non‐empty text.
  settings = {
    user_settings: {
      picklist_output_parameter: 'searchfield',
      picklist_textbox: 'true',
    },
    viz_type: {
      picklist: {
        default_selections: {
          coltype: 'STRING',
          all_selected: false,
          data: ['CA'],
          alias: ['CA'],
          exclude: false,
          includeNulls: false,
        },
      },
    },
  };

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 1, 'Dashboard click triggered for textbox when default selections provided');
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(statusTracker.stored_params['searchfield.ignore'], undefined);

  // Now simulate an incoming parameter change.
  const params = arc.params.tracker().add({
    key: 'searchfield',
    value: 'NV',
  });
  arcl.driver.params(params);

  arcl.picklist.viz_setSettings(settings);
  assert.equal(statusTracker.call_count, 2, 'Dashboard click count updated as expected');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2, 'Two parameters were sent for textbox picklist');
  assert.equal(statusTracker.stored_params.searchfield, 'NV', 'Textbox parameter value updated correctly');
  assert.equal(
    statusTracker.stored_params['searchfield.include_nulls'],
    'false',
    'Include nulls flag is sent correctly'
  );
  // Our new textboxToParam no longer sets isIgnore based on text content.
  assert.equal(statusTracker.stored_params['searchfield.ignore'], undefined, 'Textbox parameter is not marked as ignored');
});

QUnit.test('Expanded title display', (assert) => {
  arcl.picklist.expanded(true);
  setupStatePicker();
  assert.equal(arcl.driver.subtitle(), 'state');
  arcl.picklist.picklist_title.value('state title');
  arcl.driver.subtitle_elem('subtitle-div');
  arcl.picklist.viz_drawVisual('chart-div');
  assert.equal(arcl.driver.subtitle(), 'state');
  assert.equal($(`#${arcl.driver.subtitle_elem()}`).text(), 'state');
});

QUnit.test('Data Result', (assert) => {
  setupStatePicker();

  const lis = $('#chart-div .picklist-list li label');
  assert.equal(lis.length, 3);
  lis.each((idx, item) => {
    if (idx === 0) {
      assert.equal($(item).text(), 'CA');
    }

    if (idx === 1) {
      assert.equal($(item).text(), 'AZ');
    }

    if (idx === 2) {
      assert.equal($(item).text(), 'NV');
    }
  });
});

QUnit.test('Click Behavior', (assert) => {
  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);
  let param;

  setupStatePicker();
  assert.equal(statusTracker.call_count, 1);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    1,
    'Valid count of params for empty multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.ignore'],
    undefined,
    'Valid value of ignore param for empty multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.include_nulls'],
    'false',
    'Valid value of include_nulls param for empty multi-select (checkbox) picklist',
  );

  const inputs = $('#chart-div .picklist-list input');
  assert.equal(statusTracker.call_count, 1);
  inputs.first().trigger('change');
  assert.equal(statusTracker.call_count, 2);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    1,
    'Valid count of params with no change in multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.ignore'],
    undefined,
    'Valid params with no change in multi-select (checkbox) picklist',
  );

  inputs.first().prop('checked', true);
  inputs.first().trigger('change');
  assert.equal(statusTracker.call_count, 3);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    3,
    'Valid count of params after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    statusTracker.stored_params['state.data'],
    ['CA'],
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.exclude'],
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(
    param.value,
    "'CA'",
    'Valid param.value after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    param.value_list,
    ['CA'],
    'Valid param.value_list after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(
    param.value,
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );

  inputs.last().prop('checked', true);
  inputs.last().trigger('change');
  assert.equal(statusTracker.call_count, 4);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    3,
    'Valid count of params after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    statusTracker.stored_params['state.data'],
    ['CA', 'NV'],
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.exclude'],
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(
    param.value,
    "'CA', 'NV'",
    'Valid param.value after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    param.value_list,
    ['CA', 'NV'],
    'Valid param.value_list after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(
    param.value,
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
});

QUnit.test('Click, Search, Click Behavior', async (assert) => {
  const done = assert.async();
  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);
  let param;

  arc.testutil = arc.testutil || {};
  arc.testutil.baseField = arcl.picklist.base_field();

  const settings = {
    user_settings: {
      picklist_base_field: 'state',
    },
  };

  arcl.picklist.viz_setSettings(settings);

  arc.testutil.searchResponse = {
    error: null,
    data: { rows: [['AZ'], ['CA']] },
  };

  // Initial setup
  // Select CA (first item)
  setupStatePicker();
  assert.equal(statusTracker.call_count, 1);
  let inputs = $('#chart-div .picklist-list input');
  inputs.first().prop('checked', true);
  inputs.first().trigger('change');
  assert.equal(statusTracker.call_count, 2);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    3,
    'Valid count of params after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    statusTracker.stored_params['state.data'],
    ['CA'],
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.exclude'],
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, "'CA'");
  assert.deepEqual(param.value_list, ['CA']);
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, 'in');
  assert.equal(
    $('.selected-picklist-link .selected-names').text().trim(),
    'CA',
    'Valid value for name of selected params in picklist for 1 item',
  );

  // search for Z
  // Expect AZ in dropdown while CA is already selected
  await searchPicklist('Z', [['AZ']]);
  inputs = $('#chart-div .picklist-list input');
  assert.equal(inputs.length, 1);
  assert.equal(
    $(inputs[0]).attr('data-value'),
    'AZ',
    'Valid number of options when searched within picklist',
  );
  assert.equal(
    $('.selected-picklist-link .selected-names').text().trim(),
    'CA',
  );
  assert.equal($('ul.control-list input.picklist-all').prop('checked'), false);

  // Select AZ in searched results
  // alias should be CA, AZ
  // All should be selected while count of checked items in searched items should be 1.
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    0,
    'Valid count of checked inputs without selecting anything',
  );
  inputs.first().prop('checked', true);
  inputs.first().trigger('change');
  assert.equal(statusTracker.call_count, 3);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    3,
    'Valid count of params after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    statusTracker.stored_params['state.data'],
    ['CA', 'AZ'],
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.exclude'],
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, "'CA', 'AZ'");
  assert.deepEqual(param.value_list, ['CA', 'AZ']);
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, 'in');

  assert.equal(
    $('.selected-picklist-link .selected-names').text().trim(),
    'CA, AZ',
    'Valid value for name of selected params in picklist for 2 items',
  );
  assert.equal($('ul.control-list input.picklist-all').prop('checked'), true);
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    1,
    'Valid count of checked inputs after selecting 1 item in search results',
  );

  // Empty searchbox
  // Both CA, AX should be selected
  // alias should be CA, AZ
  // all should not be selected
  await searchPicklist();
  inputs = $('#chart-div .picklist-list input');
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(
    Object.keys(statusTracker.stored_params).length,
    3,
    'Valid count of params after change in multi-select (checkbox) picklist',
  );
  assert.deepEqual(
    statusTracker.stored_params['state.data'],
    ['CA', 'AZ'],
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  assert.equal(
    statusTracker.stored_params['state.exclude'],
    'in',
    'Valid value of params after change in multi-select (checkbox) picklist',
  );
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, "'CA', 'AZ'");
  assert.deepEqual(param.value_list, ['CA', 'AZ']);
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, 'in');
  assert.equal(
    $('.selected-picklist-link .selected-names').text().trim(),
    'CA, AZ',
    'Valid value for name of selected params in picklist for 2 items',
  );
  assert.equal($('ul.control-list input.picklist-all').prop('checked'), false);
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    2,
    'Valid count of checked inputs after selecting 2 items',
  );

  // Search for A
  // All should be selected
  // Both CA, AZ should be present and selected
  await searchPicklist('A', [['CA'], ['AZ']]);
  inputs = $('#chart-div .picklist-list input');
  assert.equal(inputs.length, 2);
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    2,
    'Valid count of checked inputs after searching',
  );
  assert.equal(
    $('ul.control-list input.picklist-all').prop('checked'),
    true,
    'All is selected when all searched items are selected',
  );
  assert.equal(inputs[0].getAttribute('data-alias'), 'CA');
  assert.equal(inputs[1].getAttribute('data-alias'), 'AZ');

  // Hit on All to unselect it
  // Both CA, AZ should be removed from stored list
  $('ul.control-list input.picklist-all').prop('checked', false);
  $('ul.control-list input.picklist-all').trigger('change');
  assert.equal($('ul.control-list input.picklist-all').prop('checked'), false);
  assert.equal(
    statusTracker.stored_params['state.data'],
    undefined,
    'Valid value of data params after un-setting All',
  );
  assert.equal(
    statusTracker.stored_params['state.exclude'],
    undefined,
    'Valid value of exclude params after un-setting All',
  );
  assert.equal(
    statusTracker.stored_params['state.ignore'],
    undefined,
    'Valid value of ignore params after un-setting All',
  );
  assert.equal(
    statusTracker.stored_params['state.include_nulls'],
    'false',
    'Valid value of include_nulls params after un-setting All',
  );
  // Empty searchbox
  // None of the items should be selected
  await searchPicklist();
  inputs = $('#chart-div .picklist-list input');
  assert.equal(inputs.length, 3);
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    0,
  );
  assert.equal(
    $('#chart-div .selected-picklist-link .selected-count-wrapper .selected-count-suffix')
      .text()
      .trim(),
    '(All)',
    `Default count label is correct.`,
  );
  // Test if setting All selects corresponding inputs
  // Search for N
  // Select All
  // NV should be selected
  // alias should be set to NV
  // Empty searchbox
  // Only NV should be selected
  // All should not be set
  await searchPicklist('N', [['NV']]);
  $('ul.control-list input.picklist-all').prop('checked', true);
  $('ul.control-list input.picklist-all').trigger('change');
  await searchPicklist();
  inputs = $('#chart-div .picklist-list input');
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    1,
  );
  assert.equal(
    $('#chart-div ul.control-list .selected-picklist-link .selected-names').text().trim(),
    'NV',
    'Valid value for name of selected params (alias) in picklist',
  );
  assert.equal($('ul.control-list input.picklist-all').prop('checked'), false);
  done();
});

QUnit.test('Multi-select Checkbox Select All Behavior', async (assert) => {
  const done = assert.async();
  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);
  let param;

  let inputs;
  arc.testutil = arc.testutil || {};
  arc.testutil.baseField = arcl.picklist.base_field();
  arc.testutil.isSearchOn = true;
  arc.testutil.searchResponse = {
    error: null,
    data: { rows: [['AZ'], ['CA']] },
  };

  // Initial setup
  // Select All items
  // Search for N
  // unselect all (searched results)
  // Reset search
  // Verify that if remaining items continue to stay selected
  assert.equal(statusTracker.call_count, 0);
  setupStatePicker();
  assert.equal(statusTracker.call_count, 1);
  $('ul.control-list input.picklist-all').prop('checked', true);
  $('ul.control-list input.picklist-all').trigger('change');
  assert.equal(statusTracker.call_count, 2);
  await searchPicklist('N', [['NV']]);
  $('ul.control-list input.picklist-all').prop('checked', false);
  $('ul.control-list input.picklist-all').trigger('change');
  assert.equal(statusTracker.call_count, 3);
  await searchPicklist();
  // eslint-disable-next-line prefer-const
  inputs = $('#chart-div .picklist-list input');
  assert.equal($('ul.control-list input.picklist-all').prop('checked'), false);
  assert.equal(
    inputs.filter((idx, input) => $(input).prop('checked')).length,
    2,
    'Count of selected params after unselecting all searched items',
  );
  assert.deepEqual(statusTracker.stored_params['state.data'], ['CA', 'AZ']);
  assert.equal(statusTracker.stored_params['state.exclude'], 'in');
  param = getParamInArr(statusTracker.stored_params_arr, 'state.data');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, "'CA', 'AZ'");
  assert.deepEqual(param.value_list, ['CA', 'AZ']);
  param = getParamInArr(statusTracker.stored_params_arr, 'state.exclude');
  assert.equal(!!param, true, 'Searched param exists');
  assert.equal(param.value, 'in');
  done();
});

QUnit.test('Single-Value Radio Click Behavior', (assert) => {
  arcl.picklist.picklist_single_select.value('true');

  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);

  setupStatePicker();
  assert.equal(statusTracker.call_count, 1);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 1);
  assert.equal(statusTracker.stored_params['state.ignore'], undefined);
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'false');

  const inputs = $('#chart-div .picklist-list input');
  assert.equal(statusTracker.call_count, 1);
  assert.equal(inputs.length, 4);
  inputs.first().trigger('change');
  assert.equal(statusTracker.call_count, 2);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 1);
  assert.equal(statusTracker.stored_params['state.ignore'], undefined);
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'false');

  inputs.first().prop('checked', true);
  inputs.first().trigger('change');
  assert.equal(statusTracker.call_count, 3);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 1);
  assert.equal(statusTracker.stored_params['state.ignore'], undefined);
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'false');
  $(inputs.get(1)).prop('checked', true);
  $(inputs.get(1)).trigger('change');
  assert.equal(statusTracker.call_count, 4);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'CA');

  inputs.last().prop('checked', true);
  inputs.last().trigger('change');
  assert.equal(statusTracker.call_count, 5);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'NV');
});

// Now, test without 'No Selection'
QUnit.test('Single-Value Radio Click Mandatory Behavior', (assert) => {
  arcl.picklist.picklist_single_select.value('true');
  arcl.picklist.picklist_display_no_selection.value('false');

  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);

  setupStatePicker();
  testForPicklistDropdownOptions(assert, 'single.radio', ['CA', 'AZ', 'NV']);

  const inputs = $('#chart-div .picklist-list input');
  assert.equal(inputs.length, 3);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'CA');
});

QUnit.test('Single-Value Dropdown Click Behavior', (assert) => {
  arcl.picklist.picklist_single_select.value('true');
  arcl.picklist.picklist_dropdown.value('true');

  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);

  setupStatePicker();
  testForPicklistDropdownOptions(assert, 'single.dropdown', [
    'Select',
    'CA',
    'AZ',
    'NV',
  ]);
  assert.equal(statusTracker.call_count, 1);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 1);
  assert.equal(statusTracker.stored_params['state.ignore'], undefined);
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'false');

  const select = $('#chart-div .picklist-dropdown');
  const options = $('#chart-div .picklist-dropdown option');
  assert.equal(statusTracker.call_count, 1);
  assert.equal(options.length, 4);
  select.trigger('change');
  assert.equal(statusTracker.call_count, 2);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 1);
  assert.equal(statusTracker.stored_params['state.ignore'], undefined);
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'false');
  assert.equal(options.first().prop('selected'), true);

  $(options.get(1)).prop('selected', true);
  select.trigger('change');
  assert.equal(statusTracker.call_count, 3);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'CA');

  options.last().prop('selected', true);
  select.trigger('change');
  assert.equal(statusTracker.call_count, 4);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'NV');
});

// Now, test without 'No Selection'
QUnit.test('Single-Value Dropdown Click Mandatory Behavior', (assert) => {
  arcl.picklist.picklist_single_select.value('true');
  arcl.picklist.picklist_dropdown.value('true');
  arcl.picklist.picklist_display_no_selection.value('false');

  const statusTracker = {
    call_count: 0,
    stored_params: {},
    stored_params_arr: undefined,
  };
  defineClickCbs(statusTracker);

  setupStatePicker();

  const options = $('#chart-div .picklist-dropdown option');
  assert.equal(options.length, 3);
  assert.equal(!!statusTracker.stored_params, true, 'stored_params is defined');
  assert.equal(Object.keys(statusTracker.stored_params).length, 2);
  assert.equal(statusTracker.stored_params.state, 'CA');
  assert.equal(statusTracker.stored_params['state.include_nulls'], 'false');
});
