QUnit.module('visuals_arc_pie', {
  beforeEach() {
    // main chart object
    const pie = arc.visual.plugins.pie();
    this.pie = pie;

    // create the lib object
    this.driver = arc.report.driver(pie).chart_div('chart-div');

    this.user_settings = this.driver.user_settings();
    this.user_settings.find_by_id('showDimensionsOnMarks').value('true');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['M1', '20', 'Cereal 1'],
      ['M2', '40', 'Cereal 2'],
    ]);

    const col1 = arc.data
      .column()
      .colname('Manufacturer')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col2 = arc.data
      .column()
      .colname('record_count')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(1);
    const col3 = arc.data
      .column()
      .colname('Cereal')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(2);
    result.columns([col1, col2, col3]).finalize();

    // create the visual by setting the data and then drawing it
    pie.viz_setDataResult(result);
    pie.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.viz_div = d3.select('#chart-div');
    this.svg = this.viz_div.selectAll('svg');
    this.gs = this.svg.selectAll('g');
    this.canvas = this.viz_div.selectAll('canvas');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Chart Element Present', function (assert) {
  assert.equal(this.svg.size(), 1, 'There is one svg elements');

  assert.equal(this.gs.size(), 2, 'The svg element contains 2 g elements');

  assert.equal(this.canvas.size(), 3, 'There are three canvas elements');
});

QUnit.test('Chart Content Correct', function (assert) {
  const plots = arc.canteen.getCanvasData(this.canvas);

  assert.equal(arc.canteen.countMethod(plots[0], 'arc'), 4);
  assert.equal(arc.canteen.countMethod(plots[1], 'arc'), 4);
  assert.equal(arc.canteen.countMethod(plots[2], 'arc'), 0);

  assert.equal(arc.canteen.countMethod(plots[0], 'beginPath'), 2);
  assert.equal(arc.canteen.countMethod(plots[1], 'beginPath'), 2);
  assert.equal(arc.canteen.countMethod(plots[2], 'beginPath'), 0);

  assert.equal(arc.canteen.countMethod(plots[0], 'fillText'), 2);
  assert.equal(arc.canteen.countMethod(plots[1], 'fillText'), 0);
  assert.equal(arc.canteen.countMethod(plots[2], 'fillText'), 0);

  const a = arc.canteen.methodArgs(plots[0], 'fillText');
  assert.equal(a[0][0], 'M1');
  assert.equal(a[1][0], 'M2');
});

QUnit.test('Legend Test', function (assert) {
  const legend = this.gs.filter(function () {
    return arc.misc.hasClass($(this), 'viz-base-legend-g');
  });

  assert.equal(legend.size(), 2, 'There are 2 legends');
  assert.equal(
    legend.selectAll('rect').size(),
    4,
    'There are 2 colored rectangles in the legend',
  );
  assert.equal(
    legend.selectAll('text').size(),
    4,
    'There is descriptive text in the legend',
  );
});

QUnit.test('Test Pie Chart Mark Labels', function (assert) {
  const { user_settings } = this;
  const showDimensionsOnMarks = user_settings.find_by_id(
    'showDimensionsOnMarks',
  );
  const showPercentagesOnMarks = user_settings.find_by_id(
    'showPercentagesOnMarks',
  );
  const markPercentageDecimal = user_settings.find_by_id(
    'markPercentageDecimal',
  );

  function redrawVisual(that) {
    that.pie.viz_drawVisual('chart-div');
    that.canvas = that.viz_div.selectAll('canvas');
  }

  showDimensionsOnMarks.value('false');
  showPercentagesOnMarks.value('false');
  redrawVisual(this);

  let plots = arc.canteen.getCanvasData(this.canvas);
  assert.equal(arc.canteen.countMethod(plots[0], 'fillText'), 0);

  showPercentagesOnMarks.value('true');
  markPercentageDecimal.value('1');
  redrawVisual(this);

  plots = arc.canteen.getCanvasData(this.canvas);
  let a = arc.canteen.methodArgs(plots[0], 'fillText');
  assert.equal(a.length, 2);

  assert.equal(a[0][0], '33.3%');
  assert.equal(a[1][0], '66.7%');

  showDimensionsOnMarks.value('true');
  redrawVisual(this);

  plots = arc.canteen.getCanvasData(this.canvas);
  a = arc.canteen.methodArgs(plots[0], 'fillText');
  assert.equal(a.length, 4);

  assert.equal(a[0][0], 'M1');
  assert.equal(a[1][0], '33.3%');
  assert.equal(a[2][0], 'M2');
  assert.equal(a[3][0], '66.7%');
});
