QUnit.module('visuals_arc_radial', {
  beforeEach() {
    // main chart object
    const radial = arc.visual.plugins.radial();

    // create the lib object
    const driver = arc.report.driver(radial);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['CA', '9123', '50', '123534', '123'],
      ['NC', '24', '12346', '543982', '23'],
      ['DC', '23456', '57', '234', '0'],
      ['CT', '123', '4', '61345', '123465'],
    ]);
    const col1 = arc.data
      .column()
      .colname('State')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col2 = arc.data
      .column()
      .colname('record_count')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(1);
    const col3 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(2);
    const col4 = arc.data
      .column()
      .colname('puppies')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(3);
    const col5 = arc.data
      .column()
      .colname('heros')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(4);
    result.columns([col1, col2, col3, col4, col5]);

    // create the visual by setting the data and then drawing it
    radial.viz_setDataResult(result);
    radial.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.svg = $('.viz-trellis-outertable svg');
    this.mainG = $('g.viz-radial-g');
    this.legendg = $('g.viz-base-legend-g');
    this.gs = $(this.mainG[0]).find('g');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Chart Element Present and Correct', function (assert) {
  const radials = this.gs.slice(0, 4);
  const textsInRadialsExpected = ['CA', 'NC', 'DC', 'CT'];
  const dInPathsLengthsAllExpected = [
    [222, 22, 170, 11],
    [11, 357, 357, 4],
    [357, 24, 7, 0],
    [25, 6, 119, 357],
  ];

  assert.equal(this.svg.length, 2, 'There are two svg element');
  assert.equal(
    this.mainG.length,
    1,
    'There is one main g element with class viz-radial-g',
  );
  assert.equal(radials.length, 4, 'There are 4 radial g elements');

  for (let i = 0; i < radials.length; i++) {
    const radial = radials[i];
    const numberOfPaths = radial.getElementsByTagName('path').length;
    const numberOfLines = radial.getElementsByTagName('line').length;
    const text = radial.getElementsByTagName('text')[0].textContent;
    const numberOfTexts = radial.getElementsByTagName('text').length;

    assert.equal(numberOfPaths, 4, `Radial #${i} has 4 paths`);
    assert.equal(numberOfLines, 1, `Radial #${i} has 1 lines`);
    assert.equal(numberOfTexts, 1, `Radial #${i} has 1 texts`);

    assert.equal(
      text,
      textsInRadialsExpected[i],
      `Radial #${i} are properly labeled`,
    );

    const radialPaths = radials[i].getElementsByTagName('path');
    const dInPathsLengths = [];

    for (let j = 0; j < radialPaths.length; j++) {
      const radialPathsInt = parseInt(radialPaths[j].getTotalLength(), 10);
      dInPathsLengths.push(radialPathsInt);
    }

    assert.deepEqual(
      dInPathsLengths,
      dInPathsLengthsAllExpected[i],
      `Radial #${i} has correct path lengths`,
    );
  }
});

QUnit.test('Legend Present and Correct', function (assert) {
  const legend = this.legendg;
  const rectangles = legend[0].getElementsByTagName('rect');
  const texts = legend[0].getElementsByTagName('text');

  assert.equal(legend.length, 2, 'There are 2 legend g elements');
  assert.equal(
    rectangles.length,
    4,
    'There are 4 colored rectangles in the legend',
  );
  assert.equal(
    texts.length,
    4,
    'There are 4 pieces of discriptive text in the legend',
  );

  const rect1Fill = rectangles[0].style.fill;
  const rect2Fill = rectangles[1].style.fill;
  const rect3Fill = rectangles[2].style.fill;
  const rect4Fill = rectangles[3].style.fill;
  const rectFillActual = [rect1Fill, rect2Fill, rect3Fill, rect4Fill];
  const rectFillExpected = ['#b0bec5', '#7dcad2', '#546e7a', '#0097a7'];

  for (let i = 0; i < rectFillActual.length; i++) {
    rectFillActual[i] = arc.misc.colorToHex(rectFillActual[i]);
  }

  assert.deepEqual(
    rectFillActual,
    rectFillExpected,
    'The rectangles in the legend have the correct color',
  );

  const rect1Text = texts[0].textContent;
  const rect2Text = texts[1].textContent;
  const rect3Text = texts[2].textContent;
  const rect4Text = texts[3].textContent;
  const rectTextActual = [rect1Text, rect2Text, rect3Text, rect4Text];
  const rectTextExpected = ['heros', 'population', 'puppies', 'record_count'];

  assert.deepEqual(
    rectTextActual,
    rectTextExpected,
    'The legend items are correctly labeled',
  );
});
