QUnit.module('visuals_arc_rawtable', {
  beforeEach() {
    // main chart object
    this.rawTable = arc.visual.plugins.rawtable();

    // create the lib object
    const driver = arc.report.driver(this.rawTable);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['India', '200'],
      ['China', '400'],
    ]);
    const col1 = arc.data
      .column()
      .colname('country')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('dimensions_shelf')
      .colindex(0);
    const col2 = arc.data
      .column()
      .colname('population')
      .coltype('BIGINT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('aggregates_shelf')
      .colindex(1);
    result.columns([col1, col2]);
    result.finalize();

    // create the visual by setting the data and then drawing it
    this.rawTable.viz_setDataResult(result);
    this.rawTable.viz_drawVisual('chart-div');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Table Element Internals', (assert) => {
  const t = arc.visual.user_settings.settingTableTemplate();
  t.unittest_validate();
  assert.expect(0);
});

QUnit.test('Simple Table', (assert) => {
  const tables = $('#chart-div').find('table');
  assert.equal(tables.length, 3);

  const tr = tables.find('tbody tr');
  assert.equal(tr.length, 2);

  const td = tr.find('td');
  assert.equal(td.length, 4);
  td.each(function (i) {
    const d = $(this).text();
    if (i === 0) {
      assert.equal(d, 'India');
    } else if (i === 1) {
      assert.equal(d, '200');
    } else if (i === 2) {
      assert.equal(d, 'China');
    } else {
      assert.equal(i, 3);
      assert.equal(d, '400');
    }
  });
});
