QUnit.module('visuals_arc_sparklines', {
  beforeEach() {
    // main chart object
    const sparkLine = arc.visual.plugins.sparklines();

    // create the lib object
    const driver = arc.report.driver(sparkLine);
    driver.chart_div('chart-div');

    // provide the data
    const result = arc.data.result();
    result.rows([
      ['Chapel Hill', 1990, 100],
      ['Palo Alto', 1990, 1123],
      ['Raleigh', 1990, 24],
      ['San Mateo', 1990, 12],
      ['Chapel Hill', 2000, 1000],
      ['Palo Alto', 2000, 9000],
      ['Raleigh', 2000, 1000],
      ['San Mateo', 2000, 0],
    ]);
    const col0 = arc.data
      .column()
      .colname('city')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(0);
    const col1 = arc.data
      .column()
      .colname('year')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('x_shelf')
      .colindex(1);
    const col2 = arc.data
      .column()
      .colname('population')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('y_shelf')
      .colindex(2);
    result.columns([col0, col1, col2]);
    result.finalize();

    // create the visual by setting the data and then drawing it
    sparkLine.viz_setDataResult(result);
    sparkLine.viz_drawVisual('chart-div');

    // store elements on the page to be tested
    this.labels = $('table>tbody>tr>td.viz-td-header');
    this.canvas = $('.viz-trellis-div canvas.maincanvas');
    this.context = this.canvas[0].getContext('2d');
  },

  afterEach() {
    $('#chart-div').empty();
  },
});

QUnit.test('Sparkline Canvases Correctly Drawn', function (assert) {
  const stack = this.context.stack();
  let beginPaths = 0;
  const moveTos = [];
  const lineTos = [];
  const strokeStyles = [];
  const globalAlphas = [];
  let strokes = 0;
  let attributes = 0;
  let translates = 0;
  let setTransforms = 0;
  const leftovers = [];

  stack.forEach((element) => {
    switch (element.method) {
      case 'beginPath':
        beginPaths += 1;
        break;
      case 'moveTo':
        moveTos.push(element.arguments);
        break;
      case 'lineTo':
        lineTos.push(element.arguments);
        break;
      case '"strokeStyle"':
        strokeStyles.push(element.arguments);
        break;
      case '"globalAlpha"':
        globalAlphas.push(element.arguments);
        break;
      case 'setTransform':
        setTransforms += 1;
        break;
      case 'translate':
        translates += 1;
        break;
      case 'stroke':
        strokes += 1;
        break;
      default:
        if (element.attr) {
          attributes += 1;
        } else {
          leftovers.push(element);
        }
    }
  });

  const moveTosExpected = [[2, 28]];
  const lineTosExpected = [[198, 2]];

  assert.equal(beginPaths, 1, 'There are 1 beginPath commands');
  assert.deepEqual(
    moveTos,
    moveTosExpected,
    'The 1 moveTos commands are correct',
  );
  assert.deepEqual(
    lineTos,
    lineTosExpected,
    'The 2 lineTos commands are correct',
  );
  assert.equal(strokeStyles.length, 0, 'There are 0 strokeStyle commands');
  assert.equal(globalAlphas.length, 0, 'There are 0 globalAlpha commands');
  assert.equal(setTransforms, 1, 'There is 1 setTransform commands');
  assert.equal(translates, 1, 'There is 1 translate commands');
  assert.equal(strokes, 1, 'There are 1 stroke commands');
  assert.equal(attributes, 3, 'There are 2 attribute commands');
  assert.deepEqual(
    leftovers,
    [],
    'There are no canvas commands unaccounted for',
  );
});

QUnit.test('Label and Canvas Elements Present', function (assert) {
  assert.equal(this.labels.length, 5, 'There are 3 td label elements');
  assert.equal(
    this.canvas.length,
    4,
    'There are 2 canvas elements that contains the charts',
  );
});

QUnit.test('line Plot Elements Correct', function (assert) {
  const trellisLabels = [];
  const trellisLabelsExpected = [
    'city',
    'Chapel Hill',
    'Palo Alto',
    'Raleigh',
    'San Mateo',
  ];

  for (let h = 0; h < this.labels.length; h++) {
    trellisLabels.push(this.labels[h].textContent);
  }

  assert.deepEqual(
    trellisLabels,
    trellisLabelsExpected,
    'The trellis labels are correct',
  );
});
