QUnit.module('visuals_arc_viztemplates');

QUnit.test('Basic template should work', (assert) => {
  const vt = arc.misc.viztemplates();
  const kvs = {
    key1: 'value1',
    key2: 'value2',
  };

  $.each(kvs, (k, v) => {
    vt.addKV(k, v);
  });

  vt.addTemplate('test');
  assert.equal(vt.transformInput('test1'), 'test1');

  vt.addTemplate('<<key3:orange>>');
  assert.equal(vt.transformInput('<<key3:orange>>'), 'orange');

  vt.addTemplate('<<key1:orange>>');
  assert.equal(vt.transformInput('<<key1:orange>>'), 'value1');

  vt.addTemplate('<<key2>>');
  assert.equal(vt.transformInput('<<key2>>'), 'value2');
});

QUnit.test(
  'Templates with case insensitive - with pure lowercase param set',
  (assert) => {
    arc.globals.parameters_case_sensitive = false;
    const vt = arc.misc.viztemplates();
    const kvs = {
      simplekey: 'value1',
      simpleKey: 'value2',
    };

    $.each(kvs, (k, v) => {
      vt.addKV(k, v);
    });

    vt.addTemplate('<<simplekey>>');
    vt.addTemplate('<<simpleKey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simpleKey>>');
    vt.addTemplate('<<simplekey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simplekey:value3>>');
    vt.addTemplate('<<simpleKey:value4>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<simpleKey:value4>>');
    vt.addTemplate('<<simplekey:value3>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<siMplekey:value5>>');
    vt.addTemplate('<<simple key:value6>>');

    assert.equal(vt.transformInput('<<siMplekey:value5>>'), 'value2');
    assert.equal(vt.transformInput('<<simple key:value6>>'), 'value6');
  },
);

QUnit.test(
  'Templates with case insensitive - without pure lowercase param set',
  (assert) => {
    arc.globals.parameters_case_sensitive = false;
    const vt = arc.misc.viztemplates();
    const kvs = {
      siMplekey: 'value1',
      simpleKey: 'value2',
    };

    $.each(kvs, (k, v) => {
      vt.addKV(k, v);
    });

    vt.addTemplate('<<simplekey>>');
    vt.addTemplate('<<simpleKey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simpleKey>>');
    vt.addTemplate('<<simplekey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simplekey:value3>>');
    vt.addTemplate('<<simpleKey:value4>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<simpleKey:value4>>');
    vt.addTemplate('<<simplekey:value3>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value2');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<siMplekey:value5>>');
    vt.addTemplate('<<simple key:value6>>');
    assert.equal(vt.transformInput('<<siMplekey:value5>>'), 'value2');
    assert.equal(vt.transformInput('<<simple key:value6>>'), 'value6');
  },
);

QUnit.test(
  'Templates with case sensitive - with pure lowercase param set',
  (assert) => {
    arc.globals.parameters_case_sensitive = true;
    const vt = arc.misc.viztemplates();
    const kvs = {
      simplekey: 'value1',
      simpleKey: 'value2',
    };

    $.each(kvs, (k, v) => {
      vt.addKV(k, v);
    });

    vt.addTemplate('<<simplekey>>');
    vt.addTemplate('<<simpleKey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), 'value1');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simpleKey>>');
    vt.addTemplate('<<simplekey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), 'value1');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simplekey:value3>>');
    vt.addTemplate('<<simpleKey:value4>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value1');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<simpleKey:value4>>');
    vt.addTemplate('<<simplekey:value3>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value1');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<siMplekey:value5>>');
    vt.addTemplate('<<simple key:value6>>');
    assert.equal(vt.transformInput('<<siMplekey:value5>>'), 'value5');
    assert.equal(vt.transformInput('<<simple key:value6>>'), 'value6');
  },
);

QUnit.test(
  'Templates with case sensitive - without pure lowercase param set',
  (assert) => {
    arc.globals.parameters_case_sensitive = true;
    const vt = arc.misc.viztemplates();
    const kvs = {
      siMplekey: 'value1',
      simpleKey: 'value2',
    };

    $.each(kvs, (k, v) => {
      vt.addKV(k, v);
    });

    vt.addTemplate('<<simplekey>>');
    vt.addTemplate('<<simpleKey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), '<<simplekey>>');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simpleKey>>');
    vt.addTemplate('<<simplekey>>');
    assert.equal(vt.transformInput('<<simplekey>>'), '<<simplekey>>');
    assert.equal(vt.transformInput('<<simpleKey>>'), 'value2');

    vt.addTemplate('<<simplekey:value3>>');
    vt.addTemplate('<<simpleKey:value4>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value3');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<simpleKey:value4>>');
    vt.addTemplate('<<simplekey:value3>>');
    assert.equal(vt.transformInput('<<simplekey:value3>>'), 'value3');
    assert.equal(vt.transformInput('<<simpleKey:value4>>'), 'value2');

    vt.addTemplate('<<siMplekey:value5>>');
    vt.addTemplate('<<simple key:value6>>');
    assert.equal(vt.transformInput('<<siMplekey:value5>>'), 'value1');
    assert.equal(vt.transformInput('<<simple key:value6>>'), 'value6');
  },
);

QUnit.test('Templates with multi-value param - case sensitive', (assert) => {
  arc.globals.parameters_case_sensitive = true;
  const vt = arc.misc.viztemplates();
  const kvs = {
    'Priority.data': "'Critical', 'Major', 'Blocker'",
    'Priority.exclude': 'in',
    'priority.data': "'Yosemite', 'Crater Lake', 'Banff', 'Arches'",
    'priority.exclude': 'not in',
  };

  $.each(kvs, (k, v) => {
    vt.addKV(k, v);
  });

  vt.addTemplate('<<Priority.data>>');
  assert.equal(
    vt.transformInput('<<Priority.data>>'),
    "'Critical', 'Major', 'Blocker'",
  );

  vt.addTemplate('[Priority]');
  assert.equal(vt.transformInput('[Priority]'), '[Priority]');

  vt.addTemplate('[Priority] in (<<Priority.data:>>)');
  assert.equal(
    vt.transformInput('[Priority] in (<<Priority.data:>>)'),
    "[Priority] in ('Critical', 'Major', 'Blocker')",
  );

  vt.addTemplate("[Priority] in (<<Priority:'Critical'>>)");
  assert.equal(
    vt.transformInput("[Priority] in (<<Priority:'Critical'>>)"),
    "[Priority] in ('Critical')",
  );

  vt.addTemplate('<<prioRity.data>>');
  assert.equal(vt.transformInput('<<prioRity.data>>'), '<<prioRity.data>>');

  vt.addTemplate('<<priority.data>>');
  assert.equal(
    vt.transformInput('<<priority.data>>'),
    "'Yosemite', 'Crater Lake', 'Banff', 'Arches'",
  );
});

QUnit.test('Templates with multi-value param - case insensitive', (assert) => {
  arc.globals.parameters_case_sensitive = false;
  const vt = arc.misc.viztemplates();
  const kvs = {
    'priority.data': "'Yosemite', 'Crater Lake', 'Banff', 'Arches'",
    'priority.exclude': 'not in',
    'Priority.data': "'Critical', 'Major', 'Blocker'",
    'Priority.exclude': 'in',
  };

  $.each(kvs, (k, v) => {
    vt.addKV(k, v);
  });

  vt.addTemplate('<<Priority.data>>');
  assert.equal(
    vt.transformInput('<<Priority.data>>'),
    "'Critical', 'Major', 'Blocker'",
  );

  vt.addTemplate('[Priority]');
  assert.equal(vt.transformInput('[Priority]'), '[Priority]');

  vt.addTemplate('[Priority] in (<<Priority.data:>>)');
  assert.equal(
    vt.transformInput('[Priority] in (<<Priority.data:>>)'),
    "[Priority] in ('Critical', 'Major', 'Blocker')",
  );

  /*
  TODO: Work with Sushil to know expectations for this test scenario
  vt.addTemplate("[Priority] in (<<Priority:'Critical'>>)");
  assert.equal(
    vt.transformInput("[Priority] in (<<Priority:'Critical'>>)"),
    "[Priority] in ('Critical', 'Major', 'Blocker')"
  );
  */

  vt.addTemplate('<<prioRity.data>>');
  assert.equal(
    vt.transformInput('<<prioRity.data>>'),
    "'Critical', 'Major', 'Blocker'",
  );

  vt.addTemplate('<<priority.data>>');
  assert.equal(
    vt.transformInput('<<priority.data>>'),
    "'Critical', 'Major', 'Blocker'",
  );
});

QUnit.module('visuals_arc_filter_expression');

QUnit.test(
  'Filter Expression substitution with multi-value param - case sensitive',
  (assert) => {
    arc.globals.parameters_case_sensitive = true;
    const coltype = 'STRING';
    const dataconnection_type = 'arcengine';
    const cur_params = [
      {
        key: 'Priority.data',
        value: "'Critical', 'Major', 'Blocker'",
        value_list: ['Critical', 'Major', 'Blocker'],
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
      {
        key: 'Priority.exclude',
        value: 'in',
        value_list: 'in',
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
    ];

    assert.equal(
      arc.misc.filterExpression(
        'Priority',
        coltype,
        cur_params,
        dataconnection_type,
      ),
      "[Priority] in ('Critical', 'Major', 'Blocker')",
    );

    assert.equal(
      arc.misc.filterExpression(
        'priority',
        coltype,
        cur_params,
        dataconnection_type,
      ),
      undefined,
    );
  },
);

QUnit.test(
  'Filter Expression substitution with multi-value param - case insensitive',
  (assert) => {
    arc.globals.parameters_case_sensitive = false;
    const coltype = 'STRING';
    const dataconnection_type = 'arcengine';
    const cur_params = [
      {
        key: 'Priority.data',
        value: "'Critical', 'Major', 'Blocker'",
        value_list: ['Critical', 'Major', 'Blocker'],
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
      {
        key: 'Priority.exclude',
        value: 'in',
        value_list: 'in',
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
    ];

    assert.equal(
      arc.misc.filterExpression(
        'Priority',
        coltype,
        cur_params,
        dataconnection_type,
      ),
      "[Priority] in ('Critical', 'Major', 'Blocker')",
    );
    /*
  TODO: Confirm with Sushil/Shaun if these 2 tests are valid expectations
  assert.equal(
    arc.misc.filterExpression('priority', coltype, cur_params, dataconnection_type),
    "[priority] in ('Critical', 'Major', 'Blocker')"
  );

  assert.equal(
    arc.misc.filterExpression('prioRity', coltype, cur_params, dataconnection_type),
    "[prioRity] in ('Critical', 'Major', 'Blocker')"
  );
  */
  },
);

QUnit.test(
  'Filter Expression substitution with single-value param - case sensitive',
  (assert) => {
    arc.globals.parameters_case_sensitive = true;
    const coltype = 'STRING';
    const dataconnection_type = 'arcengine';
    const cur_params = [
      {
        key: 'Priority',
        value: "'Critical'",
        value_list: ['Critical'],
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
      {
        key: 'Priority.exclude',
        value: 'in',
        value_list: 'in',
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
    ];

    assert.equal(
      arc.misc.filterExpression(
        'Priority',
        coltype,
        cur_params,
        dataconnection_type,
      ),
      "[Priority] in ('\\'Critical\\'')",
    );

    assert.equal(
      arc.misc.filterExpression(
        'priority',
        coltype,
        cur_params,
        dataconnection_type,
      ),
      undefined,
    );
  },
);

QUnit.test(
  'Filter Expression substitution with single-value param - case insensitive',
  (assert) => {
    arc.globals.parameters_case_sensitive = false;
    const coltype = 'STRING';
    const dataconnection_type = 'arcengine';
    const cur_params = [
      {
        key: 'Priority',
        value: "'Critical'",
        value_list: ['Critical'],
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
      {
        key: 'Priority.exclude',
        value: 'in',
        value_list: 'in',
        keytype: 1,
        valuetype: 'STRING',
        source_visual_id: 4447,
        source_dataset_id: 225,
      },
    ];

    assert.equal(
      arc.misc.filterExpression(
        'Priority',
        coltype,
        cur_params,
        dataconnection_type,
      ),
      "[Priority] in ('\\'Critical\\'')",
    );

    /*
  TODO: Confirm with Sushil/Shaun if these 2 tests are valid expectations
  assert.equal(
    arc.misc.filterExpression('priority', coltype, cur_params, dataconnection_type),
    "[priority] in ('Critical', 'Major', 'Blocker')"
  );

  assert.equal(
    arc.misc.filterExpression('prioRity', coltype, cur_params, dataconnection_type),
    "[prioRity] in ('Critical', 'Major', 'Blocker')"
  );
  */
  },
);
