(function () {
  QUnit.module('visuals.common.arc_reports', {
    beforeEach() {
      const accordionText = `
        <div class="accordion">
          <input id="searchbox" type="text">
          <a class="attr-search-clear">Clear</a>
        </div>`;

      $('body').append('<div id="reports-test-div"></div>');

      const accordion = $(accordionText);
      $('#reports-test-div').append(accordion);
      for (let i = 0; i < 3; i++) {
        accordion.append(card(i));
      }

      arc.report.searchboxSetup({
        searchboxId: '#reports-test-div #searchbox',
        search_cb: search_by_data,
        sel_cb,
      });
    },

    afterEach() {
      $('#reports-test-div').empty();
    },
  });

  function card(index) {
    return $(`
      <div class="card">
        <div class="card-header collapsed" data-toggle="collapse" data-target="#collapse${index}" role="button" aria-expanded="false">Header${index}</div>
        <div id="collapse${index}" class="collapse">
            <li some-data="item${index}">item${index}</li>
        </div>
      </div>`);
  }

  function search_by_data(_, term, searchedElement) {
    const data = searchedElement.getAttribute('some-data');
    return data.includes(term);
  }

  function sel_cb() {
    return d3.selectAll($('#reports-test-div').find('.card li'));
  }

  async function checkVisibleItemCount(expectedCount) {
    let results;
    await arc.misc.poll(1000, () => {
      results = $('#reports-test-div').find('[some-data]:visible');
      return results.length === expectedCount;
    });
    QUnit.assert.equal(results.length, expectedCount);
  }

  QUnit.test('search works', async () => {
    $('#reports-test-div #searchbox').val('item').trigger('input');
    await checkVisibleItemCount(3);

    $('#reports-test-div #searchbox').val('item1').trigger('input');
    await checkVisibleItemCount(1);

    // clearing the search field:
    $('#reports-test-div .attr-search-clear').click();
    await checkVisibleItemCount(0);

    // expanding the cards:
    $('#reports-test-div .collapse').collapse('show');
    await checkVisibleItemCount(3);

    await arc.misc.poll(
      1000,
      () => $('#reports-test-div').find('.collapsing').length === 0,
    );

    // collapsing the cards:
    $('#reports-test-div .collapse').collapse('hide');
    await checkVisibleItemCount(0);
  });

  QUnit.test('containsAggregate functioning', (assert) => {
    assert.equal(
      arc.report.containsAggregate(`[count of id] as 'cntId'`),
      false,
      'ignores aggregate names between [] -- in field names alias',
    );
    assert.equal(
      arc.report.containsAggregate(`[count] as 'countCol'`),
      false,
      'ignores aggregate names between [] -- in field names',
    );
    assert.equal(
      arc.report.containsAggregate(`"avg([id])" || [id] as 'sum(id)'`),
      false,
      'ignores aggregate names between single/double quotes',
    );
    assert.equal(
      arc.report.containsAggregate(`sum([year]) as "sum(year)"`),
      true,
      'detects aggregate that has parenthesis around a field in []',
    );
    assert.equal(
      arc.report.containsAggregate(`sum(1) as "quick count"`),
      true,
      `detects aggregate that has special argument "1"`,
    );
    assert.equal(
      arc.report.containsAggregate(`sum(*) as "quick count"`),
      true,
      `detects aggregate that has special argument "*"`,
    );
    assert.equal(
      arc.report.containsAggregate(
        `count(distinct ([year])) as 'countdistinct(year)'`,
      ),
      true,
      'detects aggregate with distinct clause',
    );
    assert.equal(
      arc.report.containsAggregate(
        `"Count: " || count(distinct ([count])) as 'cnt'`,
      ),
      true,
      `detects aggregate not in the beginning of expr`,
    );
    assert.equal(
      arc.report.containsAggregate(
        `max([orders]) + 1000 as 'max orders + 1000'`,
      ),
      true,
      `detects aggregate with + X`,
    );
    assert.equal(
      arc.report.containsAggregate(
        `sum([calories]) OVER (ORDER BY [manufacturer])`,
      ),
      true,
      `detects aggregate with window function`,
    );
    assert.equal(
      arc.report.containsAggregate(
        `(sum([Sales]) - LAG(sum([Sales]), 1) over ( order by concat(from_timestamp([Order Date], 'yyyy'),` +
          `'-W', if(month([Order Date]) = 1 AND weekofyear([Order Date]) > 50, '00', if(month([Order Date]) = 12 AND weekofyear([Order Date]) = 1,` +
          `'53', concat(if(weekofyear([Order Date]) < 10, '0', ''), cast(weekofyear([Order Date]) as STRING))))))) / LAG(sum([Sales]), 1) over` +
          `( order by concat(from_timestamp([Order Date], 'yyyy'), '-W', if(month([Order Date]) = 1 AND weekofyear([Order Date]) > 50, '00',` +
          `if(month([Order Date]) = 12 AND weekofyear([Order Date]) = 1, '53', concat(if(weekofyear([Order Date]) < 10, '0', ''), cast(weekofyear([Order Date])` +
          `as STRING)))))) as 'pct diff from prev(Sales)'`,
      ),
      true,
      `detects aggregate (sum here) in a complex analytical function expression`,
    );
  });
}());
