QUnit.module('visuals_common_arc_visual_bar_color_helper', {});

const color_start_idx = 1;
const color_end_idx = 2;
const color_getter = (d) => {
  const r = [];
  for (let i = color_start_idx; i <= color_end_idx; ++i) {
    r.push(d[i]);
  }

  return r.join(', ');
};

const col0 = arc.data
  .column()
  .colname('industry')
  .coltype('STRING')
  .colproperty(arc.report.columnTypeEnum.DIMENSION)
  .colsource('x_shelf')
  .colindex(0)
  .order({
    priority: () => 1,
    ascending: () => true,
  });
const col3 = arc.data
  .column()
  .colname('count')
  .coltype('STRING')
  .colproperty(arc.report.columnTypeEnum.DIMENSION)
  .colsource('y_shelf')
  .colindex(3);
const col4 = arc.data
  .column()
  .colname('threat-year')
  .coltype('INT')
  .colproperty(arc.report.columnTypeEnum.AGGREGATE)
  .colsource('label')
  .colindex(4);

/*
    Previously the color_domain was determined by relying
    on the incoming data order, assuming the colors'
    order was kept. However when, like in this case, some of the color
    category (botnet - 2014 ) were missing at the beginning (Advertising),
    and just appeared later (Banking), during the stacked bar visual drawing,
    it ended up visually in the wrong order. An extra reorder takes place,
    to ensure if data is ordered by the color, we display the result accordingly.
*/
const rowsWithMissingData = [
  // I uncommented some lines to visually see what is missing.
  // [ 'Advertising', 'botnet', 2014, 169, 'bo-14' ],
  ['Advertising', 'botnet', 2015, 419, 'bo-15'],
  ['Advertising', 'malicious_ip', 2014, 638, 'ma-14'],
  // [ 'Advertising', 'malicious_ip', 2015, 250, 'ma-15' ],
  ['Advertising', 'phishing', 2014, 15, 'ph-14'],
  ['Advertising', 'phishing', 2015, 150, 'ph-15'],
  ['Advertising', 'scanning', 2014, 334, 'sc-14'],
  ['Advertising', 'scanning', 2015, 457, 'sc-15'],
  ['Banking', 'botnet', 2014, 243, 'bo-14'],
  ['Banking', 'botnet', 2015, 142, 'bo-15'],
  ['Banking', 'malicious_ip', 2014, 696, 'ma-14'],
  ['Banking', 'malicious_ip', 2015, 444, 'ma-15'],
  ['Banking', 'phishing', 2014, 786, 'ph-14'],
  ['Banking', 'phishing', 2015, 569, 'ph-15'],
  ['Banking', 'scanning', 2014, 975, 'sc-14'],
  ['Banking', 'scanning', 2015, 804, 'sc-15'],
];

QUnit.test(
  'Color domain is correct, when first color shelf with higher priority is ascending, second color shelf is ascending',
  async () => {
    const col1 = arc.data
      .column()
      .colname('threat')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(color_start_idx)
      .order({
        priority: () => 2,
        ascending: () => true,
      });
    const col2 = arc.data
      .column()
      .colname('year')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('color_shelf')
      .colindex(color_end_idx)
      .order({
        priority: () => 3,
        ascending: () => true,
      });

    const config = {
      curr_mark_type: 'bar',
      columns: [col0, col1, col2, col3, col4],
      x_is_dim: true,
      y_is_dim: false,
      color_enabled: true,
      color_is_dim: true,
      color_column: col1,
      last_color_column: col2,
      color_getter,
      rows: rowsWithMissingData,
    };
    const result = arc.visual.barColorHelper.determineColorDomain(config);
    // thread =>, year =>, 'grouped' by thread
    const expectedResult = [
      'botnet, 2014',
      'botnet, 2015',
      'malicious_ip, 2014',
      'malicious_ip, 2015',
      'phishing, 2014',
      'phishing, 2015',
      'scanning, 2014',
      'scanning, 2015',
    ];
    QUnit.assert.deepEqual(
      result,
      expectedResult,
      'Color domain is ordered by thread (higher priority) ascending and by year ascending',
    );
  },
);

QUnit.test(
  'Color domain is correct, when first color shelf with higher priority is ascending, second color shelf is descending',
  async () => {
    const col1 = arc.data
      .column()
      .colname('threat')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(color_start_idx)
      .order({
        priority: () => 2,
        ascending: () => true,
      });
    const col2 = arc.data
      .column()
      .colname('year')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('color_shelf')
      .colindex(color_end_idx)
      .order({
        priority: () => 3,
        ascending: () => false,
      });

    const config = {
      curr_mark_type: 'bar',
      columns: [col0, col1, col2, col3, col4],
      x_is_dim: true,
      y_is_dim: false,
      color_enabled: true,
      color_is_dim: true,
      color_column: col1,
      last_color_column: col2,
      color_getter,
      rows: rowsWithMissingData,
    };
    const result = arc.visual.barColorHelper.determineColorDomain(config);
    // thread =>, year <=, 'grouped' by thread
    const expectedResult = [
      'botnet, 2015',
      'botnet, 2014',
      'malicious_ip, 2015',
      'malicious_ip, 2014',
      'phishing, 2015',
      'phishing, 2014',
      'scanning, 2015',
      'scanning, 2014',
    ];
    QUnit.assert.deepEqual(
      result,
      expectedResult,
      'Color domain is ordered by thread (higher priority) ascending and by year descending',
    );
  },
);

QUnit.test(
  'Color domain is correct, when first color shelf with higher priority is descending, second color shelf is ascending',
  async () => {
    const col1 = arc.data
      .column()
      .colname('threat')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(color_start_idx)
      .order({
        priority: () => 2,
        ascending: () => false,
      });
    const col2 = arc.data
      .column()
      .colname('year')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('color_shelf')
      .colindex(color_end_idx)
      .order({
        priority: () => 3,
        ascending: () => true,
      });
    const config = {
      curr_mark_type: 'bar',
      columns: [col0, col1, col2, col3, col4],
      x_is_dim: true,
      y_is_dim: false,
      color_enabled: true,
      color_is_dim: true,
      color_column: col1,
      last_color_column: col2,
      color_getter,
      rows: rowsWithMissingData,
    };
    const result = arc.visual.barColorHelper.determineColorDomain(config);
    // thread <=, year =>, 'grouped' by thread
    const expectedResult = [
      'scanning, 2014',
      'scanning, 2015',
      'phishing, 2014',
      'phishing, 2015',
      'malicious_ip, 2014',
      'malicious_ip, 2015',
      'botnet, 2014',
      'botnet, 2015',
    ];
    QUnit.assert.deepEqual(
      result,
      expectedResult,
      'Color domain is ordered by thread (higher priority) descending and by year descending',
    );
  },
);

QUnit.test(
  'Color domain is correct, when first color shelf with lower priority is descending, second color shelf is ascending',
  async () => {
    const col1 = arc.data
      .column()
      .colname('threat')
      .coltype('STRING')
      .colproperty(arc.report.columnTypeEnum.DIMENSION)
      .colsource('color_shelf')
      .colindex(color_start_idx)
      .order({
        priority: () => 3,
        ascending: () => false,
      });
    const col2 = arc.data
      .column()
      .colname('year')
      .coltype('INT')
      .colproperty(arc.report.columnTypeEnum.AGGREGATE)
      .colsource('color_shelf')
      .colindex(color_end_idx)
      .order({
        priority: () => 2,
        ascending: () => true,
      });

    const config = {
      curr_mark_type: 'bar',
      columns: [col0, col1, col2, col3, col4],
      x_is_dim: true,
      y_is_dim: false,
      color_enabled: true,
      color_is_dim: true,
      color_column: col1,
      last_color_column: col2,
      color_getter,
      rows: rowsWithMissingData,
    };
    const result = arc.visual.barColorHelper.determineColorDomain(config);
    // thread <=, year =>, 'grouped' by year
    const expectedResult = [
      'scanning, 2014',
      'phishing, 2014',
      'malicious_ip, 2014',
      'botnet, 2014',
      'scanning, 2015',
      'phishing, 2015',
      'malicious_ip, 2015',
      'botnet, 2015',
    ];
    QUnit.assert.deepEqual(
      result,
      expectedResult,
      'Color domain is ordered by thread (lower priority) descending and by year ascending',
    );
  },
);
