# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import django.test
import threading
import time

from django.conf import settings
from usage import events_upload
from arcwebbase import middleware_storage

from unittest import mock


class TestAppIdRetrieval(django.test.TestCase):

  def call_get_application_id(self):
    self.appId = events_upload.get_application_id()

  @mock.patch('usage.events_upload.get_project_info')
  def test_app_retrieval(self, project_info_mock: mock.Mock):
    self.appId = None
    t = threading.Thread(target=self.call_get_application_id)
    t.start()

    time.sleep(settings.HOSTNAME_POLL_INTERVAL + 1)
    self.assertIsNone(self.appId)

    middleware_storage.HOSTNAME = 'localhost'
    time.sleep(settings.HOSTNAME_POLL_INTERVAL + 1)
    # not accepted as it has no dots
    self.assertIsNone(self.appId)

    project_info_mock.return_value = None
    middleware_storage.HOSTNAME = 'my-app.localhost'
    time.sleep(settings.HOSTNAME_POLL_INTERVAL + 1)
    # not accepted because project info is not found
    self.assertIsNone(self.appId)

    project_info_mock.return_value = 123
    middleware_storage.HOSTNAME = 'my-app2.localhost'
    time.sleep(settings.HOSTNAME_POLL_INTERVAL + 1)
    # accepted because we have a dot and found the project info
    self.assertEquals(123, self.appId)

    # the thread should exit after returning the appId:
    t.join()
