# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from datetime import datetime
from zoneinfo import ZoneInfo
from django.utils import timezone
try:
  from collections import OrderedDict
except ImportError:
  from ordereddict import OrderedDict

from usage.models import LogEntry
from vizlib.querylog_helper import QueryState, ComponentType, ActionType


def to_usage_logentry_model(logEntryDict):

  def to(field_name, type, rename_to=None, enum_class=None):
    value = logEntryDict.get(field_name)

    if enum_class:
      value = enum_class.getStr(value)
    
    value = convert_to_type(value, type)

    if rename_to:
      field_name = rename_to
    
    return (field_name, value)

  logEntryDict = OrderedDict([
          to('ts', datetime),
          to('action', str, enum_class=ActionType),
          to('component_type', str, enum_class=ComponentType),
          to('component_id', int),
          to('action_id', str),
          to('query_state', str, enum_class=QueryState),
          to('username', str),
          to('ip', str),
          to('visual', int, 'visual_id'),
          to('app', int, 'app_id'),
          to('appgroup', int, 'appgroup_id'),
          to('dataset', int, 'dataset_id'),
          to('dataconnection', int, 'dataconnection_id'),
          to('dataconnection_fbid', str),
          to('dataconnection_type', str),
          to('sync', bool),
          to('cache', bool),
          to('cache_check', bool),
          to('runtime', float),
          to('query', str),
          to('rewrite', bool),
          to('error', bool),
          to('error_detail', str),
          to('process_id', int),
          to('visual_type', str),
          to('visual_name', str),
          to('dataset_name', str),
          to('dataconnection_name', str),
          to('appgroup_name', str),
          to('dashboard_name', str),
          to('user_id', int),
  ])

  return LogEntry(**logEntryDict)


def convert_to_type(value, type):
    # convert '' default values to None
    if value == '':
      return None
    
    if type in [int, float, bool]:
      try:
        return type(value)
      except:
        return None
    
    if type == datetime:
      return parse_date(value)
    
    return value


def parse_date(date):
    try:
        if date.tzinfo is None or date.tzinfo.utcoffset(date) is None:
          date = date.replace(tzinfo=ZoneInfo('UTC'))  # Make the naive datetime UTC-aware
        else:
            date = date.astimezone(ZoneInfo('UTC'))  # Convert timezone-aware datetime to UTC
        return date
    except:
        return timezone.now().astimezone(ZoneInfo('UTC'))
