#!/usr/bin/python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

"""
This script analyzes and suggests acceleration for apps.
"""

import os
import sys

THISDIR = os.path.dirname(os.path.realpath(__file__))
sys.path.append("%s/.." % THISDIR)

import reports.models
import datasets.models

import re
import sys
import json
import optparse
from . import app
from . import sqlselect

def parse():
  parser = optparse.OptionParser()
  parser.add_option('--app',
    action="append", dest="app",
    help="app to analyze", default=None)
  return parser.parse_args()

def main():
  options, args = parse()

  apps = app.Apps()
  app_ids = []
  for o_app in options.app:
    for app_id in o_app.split(","):
      apps.add(app_id)

  apps.analyze()

  def display_app(a):
    print(a.title())
    for coltype, col in a.cols():
      print('  ', col)

  apps.cb(display_app)

  datasets = apps.datasets()

  def display(ds):
    s = ds.select_statement()
    s2 = sqlselect.SelectStatement()
    s2.table('(%s) t' % s.sql())
    s2.addAggregate('sum(1)')
    print(s2.sql())

  datasets.cb(display)

if __name__ == "__main__":
  main()

