#!/usr/bin/python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

"""
This script analyzes and suggests acceleration for apps.
"""

import os
import sys

THISDIR = os.path.dirname(os.path.realpath(__file__))
sys.path.append("%s/.." % THISDIR)

import reports.models
import datasets.models

import re
import sys
import json
import optparse
from . import app
import select

def parse():
  parser = optparse.OptionParser()
  parser.add_option('--dataset',
    action="append", dest="dataset",
    help="dataset to analyze", default=None)
  parser.add_option('--app',
    action="append", dest="app",
    help="app to analyze", default=None)
  return parser.parse_args()

def main():
  options, args = parse()

  apps = app.Apps()
  app_ids = []
  if options.app:
    for o_app in options.app:
      for app_id in o_app.split(","):
        apps.add(app_id)

  dsids = set()
  if options.dataset:
    for o_dataset in options.dataset:
      for dataset_id in o_dataset.split(","):
        dsids.add(int(dataset_id))

  if len(dsids) > 0:
    rl = reports.models.Report.objects.filter(dataset__id__in=dsids)
    for r in rl:
      if r.report_type == 'dashboard':
        continue
      apps.add(r.id)

  apps.analyze()

  combined_cols = set()
  def display_app(a):
    print(a.app_id(), a.title())
    for coltype, col in a.cols():
      print('  ', app.coltype_to_str(coltype), col)
      combined_cols.add((coltype, col))

  apps.cb(display_app)

  print('\n')
  print('Combined Column Usage')
  print('Dimensions')
  for coltype, col in combined_cols:
    if coltype == app.App.COL_DIMENSION:
      print('  ', col)
  print('Aggregates')
  for coltype, col in combined_cols:
    if coltype != app.App.COL_DIMENSION:
      print('  ', col)

if __name__ == "__main__":
  main()

