#!/usr/bin/python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

"""
This script prints out app information for import on other systems
"""

import os
import sys

THISDIR = os.path.dirname(os.path.realpath(__file__))
sys.path.append("%s/.." % THISDIR)

import optparse
from . import app

def parse():
  parser = optparse.OptionParser()
  parser.add_option('--ds',
    action="append", dest="ds",
    help="comma separated dataset ids to export all apps from",
    default=[])
  parser.add_option('--app',
    action="append", dest="app",
    help="comma separated app ids to export",
    default=[])
  parser.add_option('--file',
    dest="file",
    help="filename to export information to",
    default=None)
  return parser.parse_args()

def main():
  options, args = parse()

  if not options.file:
    raise Exception("Must specify --file")

  apps = app.Apps()
  for o_app in options.app:
    for app_id in o_app.split(","):
      apps.add(app_id)
  for o_ds in options.ds:
    for ds_id in o_ds.split(","):
      apps.add_dataset_apps(ds_id)

  if len(apps) == 0:
    raise Exception('No apps specified. Specify --app or --ds')

  apps.analyze()
  with open(options.file, 'w') as f:
    f.write(apps.serialize())

  print('%s apps exported' % (len(apps)))

if __name__ == "__main__":
  try:
    main()
  except Exception as e:
    print(str(e))
    sys.exit(1)

