#!/usr/bin/python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

"""
This script imports app information exported via app export on other systems
"""

import os
import sys

THISDIR = os.path.dirname(os.path.realpath(__file__))
sys.path.append("%s/.." % THISDIR)

import optparse
from . import app

def parse():
  parser = optparse.OptionParser()
  parser.add_option('--file',
    dest="file",
    help="filename to import information from",
    default=None)
  parser.add_option('--dataset',
    action="append", dest="dataset",
    help="oldid:newid for dataset modifications. You can specify this option multiple times",
    default=None)
  return parser.parse_args()

def main():
  options, args = parse()

  if not options.file:
    raise Exception("Must specify --file")

  dsmaps = {}
  if options.dataset:
    for dataset_id in options.dataset:
      old_id, sep, new_id = dataset_id.partition(":")
      old_id = int(old_id)
      new_id = int(new_id)
      dsmaps[old_id] = new_id

  with open(options.file) as f:
    app_data = f.read()

  apps = app.Apps()
  apps.deserialize(app_data, dsmaps)

if __name__ == "__main__":
  try:
    main()
  except Exception as e:
    print(str(e))
    sys.exit(1)

