# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
Utilities related to CAII models.
"""

def err_resp_transformer(body: dict):
  """
  The error object's format returned by CAII model is different than for OpenAI or Azure,
  hence transforming it to comply with them
  """
  if body.get('object', None) == 'error':
    err_msg = body['message']
    body = {
      'error': {
        'message': err_msg
      }
    }

    # Users need to add "-query" suffix to the model name if the model requires input_type parameter
    # after checking NVIDIA documentation. If they forgot to add the suffix then request them to do it in
    # error message.
    if 'the model expects an input_type' in err_msg.lower():
      body['error']['message'] = f'Please add "-query" suffix to the model name of the embedding model on Site Settings. Error: {err_msg}'

  if body.get('detail', None):
    body = {
      'error': {
        'message': body['detail']
      }
    }

  return body
