# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import logging
from django.http import HttpResponse

from arcwebbase import settings

# The asyncstatus and asyncresults queries require authenticated users.
# The previously used @login_required decorator returned the login page as a string html (as a result of the redirect)
# in case of unauthenticated user (session timout can lead to it).
# On the frontend the response is expected to be either the result or an error, not a redirected html page.
# I created this decorated to address this issue.
LOGGER = logging.getLogger("arcweb")
def login_required_401(func):
    def wrapper_func(request, *args, **kwargs):
      if request.user.is_authenticated:
        return func(request, *args, **kwargs)
      else:
        LOGGER.info('User is not authenticated')
        return HttpResponse(f'You\'ve been signed out. Please go to the homepage ({request.build_absolute_uri(settings.LOGIN_URL)}) to sign in again and continue working.', 401)
    return wrapper_func