#!/usr/bin/python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import os
import sys
THISDIR = os.path.dirname(os.path.realpath(__file__))
sys.path.append("%s/.." % THISDIR)

import optparse

from datasets.models import DataSet

def dataset_create(dsname='', tablename=None, username='admin'):
  dataset_type = 'singletable'
  dataset_name = dsname
  dataset_info = '[]'
  dataset_detail = tablename
  d = DataSet(dataset_name=dataset_name, dataset_type=dataset_type,
      dataset_info=dataset_info, dataset_detail=dataset_detail)
  d.create_user = username
  d.update_user = username
  d.save()
  return d

def parse():
  parser = optparse.OptionParser()
  parser.add_option('--tablename', dest="tablename",
    help="tablename to use for dataset creation", default=None)
  parser.add_option('--dsname', dest="dsname",
    help="Dataset name to use for creation", default='')
  return parser.parse_args()

def main():
  options, args = parse()

  dsids = set()
  if not options.tablename:
    raise Exception("Table name must be specified")

  if not options.dsname:
    raise Exception("Dataset name must be specified")

  dataset = dataset_create(dsname=options.dsname, tablename=options.tablename)
  print(dataset.id)

if __name__ == "__main__":
  main()

