# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# DEFAULT FLIPPER PROPERTIES:
#
# id: Identifier for fetching/storing flipper value
# value: Default value for flipper
# label [DEFINED IN DEFAULT ONLY]: User-friendly description
# level [DEFINED IN DEFAULT ONLY]: FLIPPING_LEVELS
# info [OPTIONAL - Used in DB level only]: Extra information for tooltip
# dependents [OPTIONAL - Used in DB level only]: Dependent settings
from django.conf import settings
from apps.config import product_name
import os

is_runtime = os.getenv('CDV_MODE') == 'runtime'

DEFAULT_FLIPPERS = [
    {
        'id': 'site_enable_search',
        'label': 'Enable Search',
        'value': False,
        'level': 'DB',
        'dependents': ['site_enable_search_menu', 'site_enable_search_visual'],
    },
    {
        'id': 'site_enable_search_menu',
        'label': 'Enable Search in Menu Bar',
        'value': True,
        'level': 'DB',
    },
    {
        'id': 'site_enable_search_visual',
        'label': 'Enable Search Visual',
        'value': True,
        'level': 'DB',
    },
     {
        'id': 'enable_ai_assistant',
        'label': 'Enable AI visual',
        'value': False,
        'level': 'DB',
    },
    {
        'id': 'redact_ai_assistant_logs',
        'label': 'Redact AI visual logs',
        'value': True,
        'level': 'DB',
    },
    {
        'id': 'enable_direct_access',
        'label': 'Enable direct access',
        'value': getattr(settings, 'ENABLE_DIRECT_ACCESS', True),
        'level': 'SETTINGS'
    },
    {
        'id': 'site_enable_extracts',
        'label': 'Enable Data Extracts',
        'value': False,
        'level': 'DB',
    },
    {
        'id': 'site_enable_data_profiling',
        'label': 'Enable Data Profiling',
        'value': True,
        'level': 'DB',
    },
    {
        'id': 'site_enable_extract_rewrites',
        'label': 'Enable Query Redirection to Data Extracts (Tech Preview)',
        'value': False,
        'level': 'SETTINGS',
    },
    {
        'id': 'enable_speech_recognition',
        'label': 'Enable Speech Detection on site (Tech Preview)',
        'value': False,
        'level': 'DB',
    },
    {
        'id': 'enable_clone_info_badge',
        'label': 'Show Descendant Dashboard information on the Visuals page',
        'value': False,
        'level': 'DB',
    },
    {
        'id': 'enable_legacy_crosstab',
        'label': 'Enable legacy crosstab visual',
        'value': False,
        'level': 'DB',
    },
    {
        'id': 'enable_counting_dashboards_for_datasets',
        'label': 'Enable showing the count of associated dashboards of the datasets',
        'value': True,
        'level': 'DB',
        'info': 'Disable if Data page is loading slowly',
    },
    # Settings for CML explore workflow
    {
        'id': 'disable_visuals_for_workflows',
        'label': 'Hide visuals in explore workflow',
        'value': False,
        'level': 'SETTINGS',
    },
    {
        'id': 'enable_explore_style',
        'label': 'Enable style in the explore workflow',
        'value': False,
        'level': 'SETTINGS',
    },
    {
        'id': 'enable_cloudera_style',
        'label': 'Enable cloudera style (elevated SQL, colors, tracking queries, datasets, dataconnections on homepage, direct access)',
        'value': True,
        'level': 'SETTINGS',
    },
    {
        'id': 'hide_workspaces',
        'label': 'Hide workspaces in the explore workflow',
        'value': False,
        'level': 'SETTINGS',
    },
    {
        'id': 'enable_cml_sync',
        'label': 'Enable Cloudera AI connection sync in the explore workflow',
        'value': is_runtime,
        'level': 'SETTINGS',
    },
    {
        'id': 'enable_jobs',
        'label': 'Enable jobs',
        'value': True,
        'level': 'SETTINGS'
    },
    {
        'id': 'enable_user_create',
        'label': 'Enable Local User Creation',
        'value': getattr(settings, 'ENABLE_USER_CREATE', True),
        'level': 'DB',
    },
    {
        'id': 'enable_usage_analytics',
        'label': f'Enable {product_name} Usage Analytics (Tech Preview)',
        'value': False,
        'level': 'DB',
        'tab': 'Usage Analytics',
    },
    {
        'id': 'mute_notifications',
        'label': 'Mute Notifications',
        'value': False,
        'level': 'DB',
    },
    {
        'id': 'enable_cml_usage_events',
        'label': 'Enable Cloudera AI usage events tracking',
        'value': is_runtime,
        'level': 'SETTINGS',
    },
    {
        'id': 'enable_version_control',
        'label': 'Enable version control',
        'value': False,
        'level': 'DB',
        'dependents': ['enable_dataset_version_control'],
    },
    {
        'id': 'enable_dataset_version_control',
        'label': 'Enable dataset version control',
        'value': False,
        'level': 'DB',
    },
]
