#!/usr/bin/env python
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
  Force users to login again by deleting all sessions.

  Importing of the django models stuff requires the settings path in the environment,
  something like:

    export DJANGO_SETTINGS_MODULE='arcwebbase.settings'

"""

import os
import os.path
import sys

THISDIR = os.path.dirname(os.path.realpath(__file__))
sys.path.append("%s/.." % THISDIR)

import warnings
with warnings.catch_warnings():
  warnings.filterwarnings("ignore", category=DeprecationWarning)

def main():
  if not 'DJANGO_SETTINGS_MODULE' in os.environ:
    print("error: Please set the 'DJANGO_SETTINGS_MODULE' environment variable")
    exit(1)

  # If the settings file has a relative path for the db we need to find it
  # relative to the settings module not this file
  dsmname=os.environ['DJANGO_SETTINGS_MODULE']
  print("Using: DJANGO_SETTINGS_MODULE=%s" % dsmname)
  dsm = __import__(dsmname, fromlist=[''])
  if dsm.DATABASES['default']['ENGINE'].find("sqlite") >= 0:
    dbname = dsm.DATABASES['default']['NAME']
    if os.path.isabs(dbname):
      print("Using: db = %s" % dbname)
      from django.conf import settings
    else:
      print(dsm.__file__)
      newdbname = os.path.normpath(os.path.join(os.path.dirname(dsm.__file__),
                                                "..",
                                                dsm.DATABASES['default']['NAME']))
      from django.conf import settings
      settings.DATABASES['default']['NAME'] = newdbname
      from django.db import connection
      connection.close()
      print("Using: db = %s" % newdbname)

  from django.contrib.sessions.models import Session
  Session.objects.all().delete()

if __name__ == "__main__":
  main()

