# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from reports.models import Report, ReportUser, ReportImage
from jobs.models import JobTarget, JobSchedule
from searches.models import Search

def delete_objects_without_reports(objectType):
  print(f'Cleaning up {objectType}')

  if objectType == 'JobTarget':
    print('Cleaning up JobTarget')
    try:
      for sch in JobTarget.objects.all():
        schedule_id = sch.schedule_id
        if not JobSchedule.objects.filter(id=schedule_id).exists():
          print(f'Deleting {objectType} {sch.pk}');
          JobTarget.objects.filter(schedule_id=schedule_id).delete()
    except Exception as e:
      print(f'{e} Skipping foreign_key_cleanup because missing {objectType}')

    return

  if objectType == 'SearchDataset':
    print('Cleaning up SearchDataset')
    try:
      for dataset_search in Search.datasets.through.objects.all():
        search_id = dataset_search.search_id
        if not Search.objects.filter(id=search_id).exists():
          print(f'Deleting {objectType} {dataset_search.pk}')
          Search.datasets.through.objects.filter(
              search_id=search_id).delete()
    except Exception as e:
      print(f'{e} Skipping foreign_key_cleanup because missing {objectType}')

    return

  objects = ReportImage
  if objectType == 'ReportUser':
    print('Cleaning up ReportUser')
    objects = ReportUser
  else:
    print('Cleaning up ReportImage')

  try:
    for obj in objects.objects.all():
      report_id = obj.report_id
      if not Report.objects.filter(id=report_id).exists():
        print(f'Deleting {objectType} {obj.pk}');
        objects.objects.filter(report_id=report_id).delete()
  except Exception as e:
    print(f'{e} Skipping foreign_key_cleanup because missing {objectType}')

def main():
  try:
    delete_objects_without_reports('ReportImage')
    delete_objects_without_reports('ReportUser')
    delete_objects_without_reports('JobTarget')
    delete_objects_without_reports('SearchDataset')
  except Exception as e:
    print(f'{e} Skipping foreign key cleanup')
