# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.contrib.auth.models import User

from datasets.models import DataSet

def is_dataset_superuser(user: User):
 return user.is_superuser

def get_dataset_query_for_permission_check():
  """
  DataSet query containing only the necessary database fields for performing
  permission checks.
  """
  datasets = DataSet.objects.select_related('dataconnection')\
    .filter(is_active_version=True) \
    .only('create_user',
          'version_group_id',
          'dataconnection__dataconnection_type',
          'dataconnection__create_user')
  return datasets

def get_dataset_ids_accessible_by_user(user: User):
  return get_dataset_ids_with_permission(user, 'ds_appview')

def get_dataset_ids_managable_by_user(user: User):
  return get_dataset_ids_with_permission(user, 'ds_manage')

def get_dataset_ids_with_permission(user: User, permission: str):
  if is_dataset_superuser(user):
    return None

  accessible_dataset_ids: list[str] = []

  datasets = get_dataset_query_for_permission_check()

  for dataset in datasets:
    if permission in dataset.getPerms(user):
      accessible_dataset_ids.append(dataset.id)

  return accessible_dataset_ids
