# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from reports.models import AppGroup, AppGroupAppMembership, Report

def get_sample_dashboards_query(
    dataset_ids_accessible_by_user: list[str] = None,
    workspace_ids_accessible_by_user: list[str] = None
):
  samples_appgroup = AppGroup.objects.get(group_type='samples')
  samples_sub_appgroup = AppGroup.objects.filter(group_parent__id=samples_appgroup.id).only('id')

  samples_report_ids = list[str](
    AppGroupAppMembership.objects \
      .filter(appgroup_group__id=samples_appgroup.id) \
      .order_by('appgroup_order') \
      .values_list('appgroup_app', flat=True)
  )

  samples_sub_appgroup_ids = [ssa.id for ssa in samples_sub_appgroup]
  samples_report_ids.extend(
    list(
      AppGroupAppMembership.objects \
        .filter(appgroup_group__id__in=samples_sub_appgroup_ids) \
        .order_by('appgroup_order') \
        .values_list('appgroup_app', flat=True)
    )
  )

  query = Report.objects.filter(id__in=samples_report_ids)

  if dataset_ids_accessible_by_user is not None:
    query = query.filter(dataset_id__in=dataset_ids_accessible_by_user)

  if workspace_ids_accessible_by_user is not None:
    query = query.filter(workspace_id__in=workspace_ids_accessible_by_user)

  return query