# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.contrib.auth.models import User

from workspace.models import Workspace
from rbac.models import RBAC

def is_workspace_superuser(user: User):
 return user.is_superuser or 'sys_misc' in RBAC.get_user_obj_perms(user)

def get_workspace_ids_accessible_by_user(user: User):
  if is_workspace_superuser(user):
    return None

  accessible_workspace_ids: list[str] = []

  for workspace in Workspace.objects.only('acl').all():
    if 'ws_appview' in workspace.getPerms(user):
      accessible_workspace_ids.append(workspace.id)

  return accessible_workspace_ids