# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from util.init_utils import create_group
from rbac.roles_constants import DEFAULT_USER_GROUP, VISUAL_CONSUMER, SYSTEM_ADMIN, DATABASE_ADMIN
from util.ff_flipper import FeatureFlipper
from reports.views_settings import setting_update_single


def createGuestGroup(rbac_role=DATABASE_ADMIN, group_name=DEFAULT_USER_GROUP):
    is_successful_creation = create_group(rbac_role, group_name)
    if is_successful_creation:
        setting_update_single(
            'default_group_membership_for_new_users', group_name, False)


def create_cml_workflow_groups():
    from dataconn.cml_dependencies import VIZCML_RW, VIZCML_RO

    group_created = create_group(SYSTEM_ADMIN, VIZCML_RW)
    if group_created:
        create_group(VISUAL_CONSUMER, VIZCML_RO)


def check_cml_syncing_and_init_groups(is_initial_run):
    is_connection_sync_enabled = FeatureFlipper().get_flipper_value('enable_cml_sync')
    if is_connection_sync_enabled:
        from dataconn.cml_connection_importer import CmlConnectionImporter

        # It sets enable_cml_sync to False if the cml api is not found!
        CmlConnectionImporter().connect()

    # We need to check again in case the cml api does not exist
    should_create_cml_groups = FeatureFlipper().get_flipper_value('enable_cml_sync')
    if should_create_cml_groups:
        # Always try to create group to handle upgrade cases in CML for non-data tab
        create_cml_workflow_groups()

    # Only create the guest group if this is the first time we have been started up
    # against this viz metadata
    if is_initial_run and not should_create_cml_groups:
        createGuestGroup()
