# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import subprocess


def get_password_using_script(password_script: str):
  p = subprocess.Popen(["bash", "-c", password_script],
                       stdout=subprocess.PIPE, stderr=subprocess.PIPE)

  std_out, std_err = p.communicate()
  out_str = std_out.decode()
  err_str = std_err.decode()

  if p.returncode == 0:
    return out_str.strip()
  elif 'java.security.UnrecoverableKeyException: Invalid key length' in err_str:
    print(f'Ignoring {err_str}')
    return ''
  else:
    msg = f"""\
        password retrieval failed:
            returncode {p.returncode}
            stdout: {out_str}
            stderr: {err_str}
        """
    raise Exception(msg)
