# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from collections import defaultdict
from datetime import datetime, timedelta
import vizlib.querylog as querylog
import logging

LOGGER = logging.getLogger('arcweb')

class QueryLogLoader:
  def __init__(self):
    self.logs_by_hour = defaultdict(list)
    self.current_datetime = datetime.now()
    self.logs = querylog.LogReader()
    self.start_of_day = self.current_datetime.replace(hour=0, minute=0, second=0, microsecond=0)

  def load_logs(self, time_frame=None):
    try:
      for queryDict in self.logs.read_json_backwards():
        if queryDict is Ellipsis:
          break

        ts = datetime.strptime(queryDict.get('ts'), '%Y-%m-%d %H:%M:%S.%f')

        if time_frame == 'initial':
          if ts < self.start_of_day:
            break

          hour_key = ts.strftime('%Y-%m-%d-%H')
          self.logs_by_hour[hour_key].append(queryDict)
        else:
          last_hour_datetime = self.current_datetime - timedelta(hours=1)

          if ts < last_hour_datetime:
            break

          last_hour_key = last_hour_datetime.strftime('%Y-%m-%d-%H')
          self.logs_by_hour[last_hour_key].append(queryDict)

      return self.logs_by_hour
    except Exception as e:
        LOGGER.exception('Error loading data in QueryLogLoader')
        return []

if __name__ == "__main__":
    loader = QueryLogLoader()
    logs_by_hour = loader.load_logs(time_frame='initial')