#!/usr/bin/env python
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
'''
Usage: take_thumbnail.py [options]

Options:
  -h, --help           show this help message and exit
  --url=URL            host url
  --username=USERNAME  username
  --password=PASSWORD  password
  --app=APPID          app id

Example:
take_thumbnail.py --url=http://127.0.0.1:8000 --user=admin --password=secret --app=58
'''

#######################################################

import requests
import optparse
from pprint import pprint

def take_snapshot(prefix, username, password, report_id, takeall):
  s = requests.session()
  # Disable https certificate verification
  requests.packages.urllib3.disable_warnings()
  s.verify = False
  # Get login page with csrf cookie
  url = prefix + "/arc/apps/login"
  r=s.get(url)
  s.headers['referer'] = r.url
  # Login and get sessionid
  payload = {'csrfmiddlewaretoken': s.cookies["arccsrftoken"], 'username': username, 'password': password}
  s.post(url, data=payload)
  # Take snapshot
  url = prefix + "/arc/apps/groupsnapshot"
  if takeall:
    r = s.get(url, params={'all': "true"})
  else:
    r = s.get(url, params={'ids': "%s" % report_id})
  return r.text


def parseargs():
  parser = optparse.OptionParser(epilog="Example: take_thumbnail.py --url=http://127.0.0.1:8000 --user=admin --password=secret --app=58")
  parser.add_option('--url', dest='url', help='host url')
  parser.add_option('--username', dest='username', help='username')
  parser.add_option('--password', dest='password', help='password')
  parser.add_option('--app', dest='appid', help='app id', default=False)
  parser.add_option('--all', dest='all', action="store_true", default=False, help='all apps')
  options, args = parser.parse_args()
  # check for missing options
  if any([ v==None for v in list(options.__dict__.values()) ]):
    parser.error("Missing options")
  if not options.appid and not options.all:
    parser.error("Missing options")
  return options


def main():
  options = parseargs()
  # print options
  print(take_snapshot(options.url, options.username, options.password, options.appid, options.all))

if __name__ == "__main__":
  main()

