# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from . import perfcounters

from concurrent.futures import ThreadPoolExecutor

from django.test import TestCase

import time

# Tests to verify perfcounters functionality

class SingleThreadedTest(TestCase):
  def setUp(self):
    self.counters = perfcounters.APPROX_COUNTERS

  def test_single(self):
    c = self.counters.get('abc')
    self.assertEqual(c.value, 0)

    c += 1
    self.assertEqual(c.value, 1)

    c += 1
    self.assertEqual(c.value, 2)

    c -= 1
    self.assertEqual(c.value, 1)

class MultiThreadedTest(TestCase):
  def setUp(self):
    self.counters = perfcounters.APPROX_COUNTERS
    self.execPool = ThreadPoolExecutor(10)

  def do_updates(self):
    c = self.counters.get('def')
    for i in range(0, 10):
      c += 1

  def test_threaded(self):
    futures = []
    for i in range(0, 100):
      futures.append(self.execPool.submit(self.do_updates))

    for f in futures:
      f.result()

    c = self.counters.get('def')
    self.assertGreater(c.value, 800)

class MultiThreadedMultiVariableTest(TestCase):
  def setUp(self):
    self.counters = perfcounters.APPROX_COUNTERS
    self.execPool = ThreadPoolExecutor(50)

  def do_updates(self, counterIdx):
    c = self.counters.get('ghi_%s' % (counterIdx))
    for i in range(0, 10):
      c += 1

  def test_threaded(self):
    futures = []
    for i in range(0, 10000):
      futures.append(self.execPool.submit(self.do_updates, i % 100))

    for f in futures:
      f.result()

    for i in range(0, 100):
      c = self.counters.get('ghi_%s' % (i))
      self.assertGreater(c.value, 700)

class TimeSeriesTest(TestCase):
  def setUp(self):
    pass

  def test_single(self):
    t1 = time.time()
    c = perfcounters.TimeSeriesCounter(name="test1", timePeriodSeconds=0.01,
          timePeriodRetentionSeconds=2)

    cnt = 0

    while time.time() - t1 < 4:
      cnt += 1
      c += 1

    data = c.data()
    total = data['rolloverTotal']
    zeroes = 0
    for v in data['values']:
      total += v
      if v == 0:
        zeroes += 1

    self.assertEqual(cnt, total)
    # self.assertLess(zeroes, 2)

  def test_gap(self):
    c = perfcounters.TimeSeriesCounter(name="test2", timePeriodSeconds=0.01,
          timePeriodRetentionSeconds=10)
    c += 1
    time.sleep(2)
    c += 1

    data = c.data()
    total = data['rolloverTotal']
    zeroes = 0
    gap = 0
    for v in data['values']:
      total += v
      if v == 0:
        zeroes += 1
        if total == 1:
          gap += 1

    self.assertEqual(2, total)
    self.assertEqual(zeroes, 998)
    self.assertGreater(gap, 198)

