# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.test import TestCase
from unittest.mock import patch, Mock
import json

from util.remove_columns_parenthesis import main
from datasets.models import DataSet, DataConnection

class RemoveColumnsParenthesisTest(TestCase):
    @patch('util.remove_columns_parenthesis.argparse')
    def test_when_run_without_arguments_should_not_remove(self, argparse):
        # Given
        self.__create_dataset_with_one_column('A', 'a', 'a (aa)')
        # When
        main()
        # Then
        alias = self.__get_column_alias(DataSet.objects.all()[0])
        self.assertEqual(alias, 'a (aa)')

    @patch('util.remove_columns_parenthesis.argparse')
    def test_when_run_given_datasets_ids_should_remove(self, argparse):
        # Given
        self.__create_dataset_with_one_column('A', 'a', 'a (aa)')
        self.__create_dataset_with_one_column('B', 'b', 'b (bb)')
        self.__create_dataset_with_one_column('C', 'c', 'c (cc)')

        argparse.ArgumentParser.return_value = self.__create_argparse([1, 2])
        # When
        main()
        # Then
        alias_1 = self.__get_column_alias(DataSet.objects.all()[0])
        alias_2 = self.__get_column_alias(DataSet.objects.all()[1])
        alias_3 = self.__get_column_alias(DataSet.objects.all()[2])

        self.assertEqual(alias_1, 'a - aa')
        self.assertEqual(alias_2, 'b - bb')
        self.assertEqual(alias_3, 'c (cc)')

    @patch('util.remove_columns_parenthesis.argparse')
    def test_when_run_given_zero_id_should_remove_all(self, argparse):
        # Given
        self.__create_dataset_with_one_column('A', 'a', 'a (aa)')
        self.__create_dataset_with_one_column('B', 'b', 'b (bb)')
        self.__create_dataset_with_one_column('C', 'c', 'c (cc)')

        argparse.ArgumentParser.return_value = self.__create_argparse([0])
        # When
        main()
        # Then
        alias_1 = self.__get_column_alias(DataSet.objects.all()[0])
        alias_2 = self.__get_column_alias(DataSet.objects.all()[1])
        alias_3 = self.__get_column_alias(DataSet.objects.all()[2])

        self.assertEqual(alias_1, 'a - aa')
        self.assertEqual(alias_2, 'b - bb')
        self.assertEqual(alias_3, 'c - cc')

    def __create_dataset_with_one_column(self, dataset_name, column_name, column_alias):
        dc = DataConnection(dataconnection_type='Sql')
        dc.save()

        ds = DataSet(dataset_name=dataset_name, dataset_type='type', dataset_detail='detail', dataconnection=dc)
        ds.dataset_info = json.dumps([{'columns': [{'name': column_name, 'alias': column_alias}]}])
        ds.save()

    def __create_argparse(self, datasets_to_process):
        parseargs = Mock()
        parseargs.remove = True
        parseargs.datasets_to_process = datasets_to_process

        parser = Mock()
        parser.parse_args.return_value = parseargs

        return parser
        
    def __get_column_alias(self, dataset):
        return json.loads(dataset.dataset_info)[0]['columns'][0]['alias']