# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from django.test import TestCase

from .streaming_json_str import StreamingJsonStr
import json
from dataconn.rows_data import RowsData

class SimpleTest(TestCase):
  def verify(self, o):
    s = StreamingJsonStr(o)
    o_j1 = ""
    cnt = 0
    for x in s:
      o_j1 += x
      cnt += 1

    o_j2 = json.dumps(o, separators=(",", ":"))
    self.assertEqual(cnt, 1)
    self.assertEqual(o_j1, o_j2)

  def test_str(self):
    self.verify("abc")
    self.verify([1,2,3])
    self.verify({'a': 1, 'b': 2, 'c': 3})
    self.verify({1: 'a', 2: 'b', 3: 'c'})
    self.verify({'a': 1, 2: 'b', 'c': 3})

  def test_rows(self):
    rows = [
      ["data1", 32],
      ["data2", 45]
    ]

    ret = {
      "response": {
        "colnames": ["col1", "col2"],
        "coltypes": ["STRING", "BIGINT"],
        "rows": rows
      }
    }

    self.verify(ret)

    rowsData = RowsData(10240)
    for r in rows:
      rowsData.addRow(r)

    ret = {
      "response": {
        "colnames": ["col1", "col2"],
        "coltypes": ["STRING", "BIGINT"],
        "rows": rowsData.fetchAndClose()
      }
    }

    s = StreamingJsonStr(ret)
    o_j1 = ""
    cnt = 0
    for x in s:
      cnt += 1
      o_j1 += x

    # Reflect the fact that 'rows' are handled specially by StreamingJsonStr
    ret["response"]["rows"] = rows
    o_j2 = json.dumps(ret, separators=(",", ":"))
    self.assertEqual(cnt, 1)
    self.assertEqual(o_j1, o_j2)

    # If we have a 10K rows, we should expect 3 chunks (see the
    #   comparison of cnt to 3 below)
    rows = [["a" * 10240]]

    ret = {
      "response": {
        "colnames": ["col1", "col2"],
        "coltypes": ["STRING", "BIGINT"],
        "rows": json.dumps(rows)
      }
    }

    s = StreamingJsonStr(ret)
    o_j1 = ""
    cnt = 0
    for x in s:
      cnt += 1
      o_j1 += x

    # Reflect the fact that 'rows' are handled specially by StreamingJsonStr
    ret["response"]["rows"] = rows
    o_j2 = json.dumps(ret, separators=(",", ":"))

    # We should have gotten the data in 3 sections
    # Everything before rows, rows, and everything after rows
    self.assertEqual(cnt, 3)

    self.assertEqual(o_j1, o_j2)

    # Identical results if we 10x the rows data
    rows = [["a" * 102400]]

    ret = {
      "response": {
        "colnames": ["col1", "col2"],
        "coltypes": ["STRING", "BIGINT"],
        "rows": json.dumps(rows)
      }
    }

    s = StreamingJsonStr(ret)
    o_j1 = ""
    cnt = 0
    for x in s:
      cnt += 1
      o_j1 += x

    # Reflect the fact that 'rows' are handled specially by StreamingJsonStr
    ret["response"]["rows"] = rows
    o_j2 = json.dumps(ret, separators=(",", ":"))

    # We should have gotten the data in 3 sections
    # Everything before rows, rows, and everything after rows
    self.assertEqual(cnt, 3)

    self.assertEqual(o_j1, o_j2)

