# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import io
import unittest

from util import json_utils

snowman = 'A\u2603B'


class TestJsonUtils(unittest.TestCase):

  def setUp(self):
    super().setUp()
    self.original_setting = json_utils.ensure_ascii

  def tearDown(self):
    json_utils.ensure_ascii = self.original_setting
    super().tearDown()

  def test_dumps_ascii_to_stream(self):
    # json_utils.ensure_ascii = True # this is the default
    buffer = io.StringIO()
    json_utils.json_dump(snowman, buffer)
    self.assertEquals(buffer.getvalue(), '"A\\u2603B"')

  def test_dumps_ascii_to_string(self):
    # json_utils.ensure_ascii = True # this is the default
    dump_string = json_utils.json_dumps(snowman)
    self.assertEquals(dump_string, '"A\\u2603B"')

  def test_dumps_unicode_to_stream(self):
    json_utils.ensure_ascii = False
    buffer = io.StringIO()
    json_utils.json_dump(snowman, buffer)
    self.assertEquals(buffer.getvalue(), f'"{snowman}"')

  def test_dumps_unicode_to_string(self):
    json_utils.ensure_ascii = False
    dump_string = json_utils.json_dumps(snowman)
    self.assertEquals(dump_string, f'"{snowman}"')
