# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from util.password_script import get_password_using_script

import unittest


class TestPasswordScript(unittest.TestCase):

  def test_success(self):
    password = get_password_using_script('echo abc')
    self.assertEquals(password, 'abc')

  def test_error(self):
    failing_script = 'python -c "import sys; sys.stderr.write(\'TEST ERROR\');sys.exit(13)"'

    with self.assertRaises(Exception) as exception_context:
      get_password_using_script(failing_script)

    self.assertTrue('password retrieval failed' in str(
      exception_context.exception))

  def test_error_workaround(self):
    jdk_error = 'java.security.UnrecoverableKeyException: Invalid key length'
    script = f'python -c "import sys; sys.stderr.write(\'{jdk_error}\');sys.exit(13)"'

    password = get_password_using_script(script)
    self.assertEquals(password, '')
