# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""Profile stuff for query progress tracking/reporting """
import time
from datetime import datetime

class ArcProf(object):

  def __init__(self, name, querytracker=None, username=None):
    self.stamp = time.time()
    self.firststamp = self.stamp
    self.entries = []
    self.timestamp('start')
    self.record('name', name)
    if querytracker:
      self._querytracker = querytracker
    else:
      self._querytracker = f"00-{self.stamp}"
    self._username = username
    self.record('querytracker', self._querytracker)

  def querytracker(self):
    return self._querytracker

  def username(self):
    return self._username

  def mark(self, text):
    now = time.time()
    self.record(text, "%.6f" % (now - self.stamp))
    self.stamp = now

  def markFromStart(self, text):
    now = time.time()
    self.record(text, "%.6f" % (now - self.firststamp))

  def timestamp(self, text):
    ts = datetime.fromtimestamp(time.time()).strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
    self.record(text, ts)

  def record(self, text, info):
    self.entries.append([text, info])

  def reset(self):
    self.entries = self.entries[:1]

  def results(self):
    return self.entries
