# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import re
import json

import reports
from dataconn.data_api_cmds import NORM_CMDS

def cviz_is_rest_expr(expr):
  is_dim, e = cviz_rest_expr(expr)
  return e is not None

def cviz_rest_expr(expr):
  m = re.search(r'\b[max|sum|count|avg|concat|distinct]\(\(cviz_rest\((.*)?\)\)\)\s*$', expr, re.IGNORECASE)
  if m:
    return False, m.group(1)

  m = re.search(r'\bcviz_rest\((.*)?\)\s*$', expr, re.IGNORECASE)
  if m:
    return True, m.group(1)


  return None, None

def cviz_modify_dsreq(dsreq_obj):
  changes = 0
  if dsreq_obj.get('type') == NORM_CMDS['getrows']:
    if dsreq_obj.get('dataset_object'):
      dataset_object = json.loads(dsreq_obj.get('dataset_object'))
      for table in dataset_object:
        columns = table.get('columns')
        for column in columns:
          expr = column.get('expr')
          if expr is not None:
            is_dim, e = cviz_rest_expr(expr)
            if e:
              column['expr'] = '"Remote call"'
              changes += 1

  if changes > 0:
    dsreq_obj['dataset_object'] = json.dumps(dataset_object)

def cviz_validate_column(dsreq_obj, expr):
  """validate a cviz_ column"""
  is_dim, e = cviz_rest_expr(expr)
  if e:
    if e[0] == "'" and e[len(e) - 1] == "'":
      e = e[1:len(e) - 1]
    elif e[0] == '"' and e[len(e) - 1] == '"':
      e = e[1:len(e) - 1]
    else:
      raise Exception('Invalid string parameter ' + e)

    print(e)
    o = json.loads(e)
    err = validURL(o.get('url'))
    if err is not None:
      raise Exception(err)

    data = {
      "type": "STRING",
      "is_valid": "true",
      "ispart": "false",
      "isaggregate": "false",
    }

    return {
      "data": data
    }

  return {
    "error": "Unsupported cviz expression"
  }

def validURL(url):
  if not url:
    return "No URL specified"

  remotedata_url_whitelist = reports.views_settings.setting_get_single("remotedata_url_whitelist")
  if not remotedata_url_whitelist:
    return "URL specified is not in the remote data settings whitelist"

  valid_urls = remotedata_url_whitelist.split("\n")
  if not url in valid_urls:
    return "URL specified is not in the remote data settings whitelist"

  return None

