# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
# SQL extensions for django access

from django.db.models import Aggregate, CharField
from django.db.models.expressions import Func

from django.db import connection
from django.conf import settings

def GroupConcatEnabled():
  return connection.vendor in ['sqlite', 'mysql'] and \
         getattr(settings, 'USE_GROUP_CONCAT', True)

class GroupConcat(Aggregate):
  function = 'GROUP_CONCAT'
  template = "%(function)s(%(expressions)s)"

  def as_mysql(self, compiler, connection, **extra):
    return super(GroupConcat, self).as_sql(
      compiler, connection,
      distinct='DISTINCT' if extra.pop('distinct', False) else '',
      separator="SEPARATOR '%s'" % extra.pop('separator', ','),
      template="%(function)s(%(distinct)s %(expressions)s %(separator)s)",
      **extra
    )

  def as_sqlite(self, compiler, connection, **extra):
    return super(GroupConcat, self).as_sql(
      compiler, connection,
      separator="'%s'" % extra.pop('separator', ','),
      template="%(function)s(%(expressions)s, %(separator)s)",
      **extra
    )

  def as_postgresql(self, compiler, connection, **extra):
    return super(GroupConcat, self).as_sql(
      compiler, connection,
      separator="'%s'" % extra.pop('separator', ','),
      template="STRING_AGG(CAST (%(expressions)s AS VARCHAR(255)), %(separator)s)",
      **extra
    )
