# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
class DsreqCreator:
  def __init__(self):
    self._dsreq = {
      'version': 1,
      'type': 'SQL',
      'limit': 5000,
      'filters': [],
      'dataset_id': 0
    }

  def setDistinct(self):
    self._dsreq['distinct'] = True

  def getdsreq(self):
    return self._dsreq

  def datasetId(self, _):
    self._dsreq['dataset_id'] = _

    return self

  def addFilter(self, _):
    # Valid filters are strings that may contain columns
    # Example: "[manufacturer] in ('Kelloggs')"
    self._dsreq['filters'].append(_)

    return self

  def orderBy(self, ascending=True):
    return {
      'asc': (ascending == True)
    }

  def addDimension(self, _, orderby=None):
    if not self._dsreq.get('dimensions'):
      self._dsreq['dimensions'] = []

    # Valid dimension expressions are strings that may contain columns
    # Example: "[cereal_name] as 'cereal_name'"

    d = {
      'type': 'SIMPLE',
      'expr': _
    }

    if orderby:
      d['order'] = orderby

    self._dsreq['dimensions'].append(d)
    return self

  def addAggregate(self, _, orderby=None):
    if not self._dsreq.get('aggregates'):
      self._dsreq['aggregates'] = []

    # Valid aggregate expressions are strings that may contain columns
    # Example: "avg([calories]) as 'avg(calories)'"
    d = {
      'expr': _
    }

    if orderby:
      d['order'] = orderby

    self._dsreq['aggregates'].append(d)
    return self

  def addGlobalSegment(self, segment_id: int, negate = False):
    if not self._dsreq.get('global_segments'):
      self._dsreq['global_segments'] = [
        {
          'type': 'SEGMENT',
          'segments': []
        }
      ]

    segments = self._dsreq['global_segments'][0]['segments']

    segments.append({
      'seg_id': segment_id,
      'negate': negate
    })

    return self

  def setLimit(self, _):
    # A valid limit is an unsigned integer
    self._dsreq['limit'] = _

    return self

