# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json

from datasets.models import FilterAssociation
from .dataset_request import DatasetRequest

from django.conf import settings

def find_fa(fal=None, user=None, dataset_id=None):
  association = None
  if fal:
    for fa in fal:
      if FilterAssociation.user_has_filter_perms_on_dataset(fa.id, None, dataset_id, user_obj=user):
        association = fa
        break
    for fa in fal:
      if FilterAssociation.user_not_group_has_filter_perms_on_dataset(fa.id, None, dataset_id, user_obj=user):
        association = fa
        break
  return association

class UserFilter(object):
  def __init__(self, user_id, dataset_id):
    self.association = None
    self.params = None
    # association filters
    # Avoid circular import
    from sqlrun.jsonselect_cache import jsonselect_get_falist, jsonselect_get_user, jsonselect_cached_function
    fal = jsonselect_get_falist(dataset_id)
    if fal:
      user = jsonselect_get_user(user_id)
      self.association = jsonselect_cached_function(('find_fa', user_id, dataset_id), find_fa, fal=fal, user=user, dataset_id=dataset_id)
      profkey = getattr(settings, 'PROFILE_FA_PARAMS_KEY', None)
      if profkey:
        self.params = user.userprofile.getkey(profkey)

  def applyFA(self, sb_builder, qcntl):
    if self.association:
      rule = json.loads(self.association.filterassociation_data)
      # Change rule to be consumable by global segment:
      #   rename 'applied_segment' key and add type item
      #   rename dictionary key "id" HACK! TODO fix me
      rule['segments'] = rule.pop('applied_segments')
      rule['type'] = 'SEGMENT'
      for d in rule['segments']:
        d['seg_id'] = d.pop('id')

      fa_seg = DatasetRequest.FilterAssociationSegment(rule, params=self.params)

      # Get the expression and extract the columns named in the expression
      columns = set()
      groups = fa_seg.segments.groups
      keys = list(groups.keys())
      if len(keys) != 1:
        raise Exception('Invalid FA %s (ID %s) : multiple segment groups selected' % \
                        (self.association.filterassociation_name, self.association.id))
      group = keys[0]
      segs = groups[group]
      if len(segs) != 1:
        raise Exception('Invalid FA %s (ID %s) : must have exactly on segmet selected' % \
                        (self.association.filterassociation_name, self.association.id))

      fa_expr = segs[0].getexpr()
      fa_expr = sb_builder.dataset.process_expr(fa_expr, columns)
      sb_builder.addFilterColumns(columns)
      if qcntl and qcntl.fa_engine:
        qcntl.fa_expr = fa_expr
        qcntl.fa_tables = set(col.table.tablename for col in columns)
      else:
        fa_seg.segments.addToSb(sb_builder)
