# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from dataconn.data_api_cmds import NORM_CMDS, SPECIAL_CMDS
from dataconn.connections_types import DATACONN_HIVE, DATACONN_HYVE, DATACONN_MONGODB, \
    DATACONN_SPARKSQL, DATACONN_DRILL, DATACONN_ORACLE, DATACONN_CASSANDRA
from .select_builder import BaseSelectBuilder, HiveSelectBuilder, OracleSelectBuilder, CassandraSelectBuilder


class StatsBuilder(object):
  def __init__(self, dataset, request, query=None, conn_type=None,
                                                   properties=None, prefix=None):
    self.dataset = dataset
    self.prefix = prefix
    self.query = None
    if request.request_type in [NORM_CMDS['getrows'], SPECIAL_CMDS['VALIDATE']]:
      if conn_type is not None and conn_type in [DATACONN_HIVE, DATACONN_HYVE, DATACONN_MONGODB, \
          DATACONN_SPARKSQL, DATACONN_DRILL]:
        sb = HiveSelectBuilder(dataset, request)
      elif conn_type == DATACONN_ORACLE:
        sb = OracleSelectBuilder(dataset, request)
      elif conn_type == DATACONN_CASSANDRA:
        sb = CassandraSelectBuilder(dataset, request)
      else:
        sb = BaseSelectBuilder(dataset, request)

      if request.request_type == NORM_CMDS['getrows']:
        if request.table_name is not None:
          if conn_type != DATACONN_ORACLE:
            self.query = "SELECT * from " + str(sb.dataset.quote_tablename(request.table_name)) + \
                         "\nLIMIT " + str(request.limit)
          else:
            self.query = "SELECT * from " + str(sb.dataset.quote_tablename(request.table_name)) + \
                         "\nWHERE ROWNUM <= " + str(request.limit)
      if not self.query:
        self.query = sb.build(request)
    elif request.request_type == NORM_CMDS['COLUMN_STATS']:
      self.query = query
    elif request.request_type == SPECIAL_CMDS['DATA_IMPORT']:
      if request.table_name is not None:
        if request.dbname != '' and request.dbname is not None:
          formatted_tablename = properties.quote + request.dbname +\
                                properties.quote_end + "." + request.table_name
        else:
          formatted_tablename = request.table_name
        self.query = "SELECT * from " + str(formatted_tablename) + "\nlimit " +\
                      str(request.limit)
      else:
        self.query = None
    else:
      self.query = request.query

  def build(self, dsreq=None):
    if self.prefix:
      return "%s %s" % (self.prefix, self.query)
    else:
      return self.query

  # StatsBuilder doesn't support filters. Only used for "Show data"
  def addFilterColumns(self, columns):
    pass

  # StatsBuilder doesn't support filters. Only used for "Show data"
  def addFilter(self, expr, addToColumns=True):
    pass
