# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from . import test_base_dataconn

MYSQL_TYPES = set(["mysql", "mariadb"])
PG_TYPES = set(["postgres", "redshift"])


class TestDataEnrichment(test_base_dataconn.Test_Base_Data_Connection):
  def __init__(self, *args, **kwargs):
    self.dataconnection_type = 'arcengine'
    super(TestDataEnrichment, self).__init__(*args, **kwargs)

  def test_me(self):
    self.derivedDatacase1()
    self.derivedDatacase2()
    self.derivedDatacase3()
    self.derivedDatacase4()
    self.derivedDatacase5()
    self.derivedDatacase6()
    self.derivedDatacase7()
    self.derivedDatacase8()
    self.derivedDatacase9()

  ### derived data definitions ###
  def derivedstate(self):
    return {"label": "derivedstate",
            "dsreq": {"version": 1,
                      "type": "SQL",
                      "time": [],
                      "session": [],
                      "dimensions": [{"type": "SIMPLE", "expr": "upper([state]) as 'ustate'"},
                                     {"type": "SIMPLE", "expr": "lower([state]) as 'lstate'"}],
                      "aggregates": [],
                      "filters": [],
                      "having": [],
                      "global_segments": [{"type": "SEGMENT", "segments": []}],
                      "events": [], "distinct": False,
                      "dataset_id": self.censusdata_id,
                      "limit": 100}}

  def derivedstateAgg1(self):
    return {"label": "derivedstateAgg1",
            "dsreq": {"version": 1, "type": "SQL", "time": [], "session": [],
                      "dimensions": [{"type": "SIMPLE", "expr": "[state] as '_state'"}],
                      "aggregates": [{"expr": "sum([population]) as 'pop_sum'", "order": {"asc": False, "pri": 4}}],
                      "filters": [], "having": [], "global_segments": [{"type": "SEGMENT", "segments": []}],
                      "events": [], "distinct": False, "dataset_id": self.censusdata_id, "limit": 100}}

  def derivedstateAgg2(self):
    return {"label": "derivedstateAgg2",
            "dsreq": {"version": 1, "type": "SQL", "time": [], "session": [],
                      "dimensions": [{"type": "SIMPLE", "expr": "[year] as '_year'"}],
                      "aggregates": [{"expr": "sum([population]) as 'pop_sum2'"}], "filters": [], "having": [],
                      "global_segments": [{"type": "SEGMENT", "segments": []}], "events": [], "distinct": False,
                      "dataset_id": self.censusdata_id, "limit": 100}}

  def derivedFact2Dim(self):
    return {"label": "derivedYear",
            "dsreq": {"version": 1, "type": "SQL", "time": [], "session": [],
                      "dimensions": [{"type": "SIMPLE", "expr": "[name] as '_name'"}],
                      "aggregates": [{"expr": "max([yr]) as 'max_yr'"}], "filters": [], "having": [],
                      "global_segments": [{"type": "SEGMENT", "segments": []}], "events": [], "distinct": False,
                      "dataset_id": self.jvfactdata_id, "limit": 100}}
  #############################

  ### unit tests ###
  def derivedDatacase1(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [{"type": "SIMPLE",
                              "expr": "[ustate] as 'ustate'",
                              "limit": "false"}],
              "aggregates": [],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.censusdata_id,
              "derived_data": [self.derivedstate(), self.derivedstateAgg1(), self.derivedstateAgg2()]}

    expected_output = """
      WITH T1 AS
      ( SELECT upper((TA_0.`state`)) as `ustate`, lower((TA_0.`state`)) as `lstate`
        FROM `default`.`census_pop` TA_0
        LIMIT 100
      ) SELECT T1.`ustate` as `ustate`
        FROM `T1` T1
        LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)

  def derivedDatacase2(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [{"type": "SIMPLE",
                              "expr": "[ustate] as 'ustate'",
                              "limit": "false"},
                             {"type": "SIMPLE",
                              "expr": "[lstate] as 'lstate'",
                              "limit": "false"}],
              "aggregates": [],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.censusdata_id,
              "derived_data": [self.derivedstate(), self.derivedstateAgg1(), self.derivedstateAgg2()]}

    expected_output = """
    WITH T1 AS
    ( SELECT upper((TA_0.`state`)) as `ustate`, lower((TA_0.`state`)) as `lstate`
      FROM `default`.`census_pop` TA_0
      LIMIT 100
    ) SELECT T1.`ustate` as `ustate`, T1.`lstate` as `lstate`
      FROM `T1` T1
      LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)

  def derivedDatacase3(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [{"type": "SIMPLE",
                              "expr": "concat([ustate],[lstate]) as 'ulstate'",
                              "limit": "false"}],
              "aggregates": [],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.censusdata_id,
              "derived_data": [self.derivedstate(), self.derivedstateAgg1(), self.derivedstateAgg2()]}

    expected_output = """
      WITH T1 AS
      ( SELECT upper((TA_0.`state`)) as `ustate`, lower((TA_0.`state`)) as `lstate`
        FROM `default`.`census_pop` TA_0
        LIMIT 100
      ) SELECT concat((T1.`ustate`),(T1.`lstate`)) as `ulstate`
        FROM `T1` T1
        LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)

  def derivedDatacase4(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [],
              "aggregates": [{"type": "SIMPLE",
                              "expr": "max([ustate]) as 'mstate'",
                              "limit": "false"}],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.censusdata_id,
              "derived_data": [self.derivedstate(), self.derivedstateAgg1(), self.derivedstateAgg2()]}

    expected_output = """
    WITH T1 AS
    ( SELECT upper((TA_0.`state`)) as `ustate`, lower((TA_0.`state`)) as `lstate`
      FROM `default`.`census_pop` TA_0
      LIMIT 100
    ) SELECT max((T1.`ustate`)) as `mstate`
      FROM `T1` T1
      LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)

  def derivedDatacase5(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [],
              "aggregates": [{"type": "SIMPLE",
                              "expr": "avg([pop_sum]) as 'mpop'",
                              "limit": "false"}],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.censusdata_id,
              "derived_data": [self.derivedstate(), self.derivedstateAgg1(), self.derivedstateAgg2()]}

    expected_output = """
    WITH T2 AS
    ( SELECT TA_0.`state` as `_state`, sum((TA_0.`population`)) as `pop_sum`
      FROM `default`.`census_pop` TA_0
      GROUP BY 1
      ORDER BY 2 DESC
      LIMIT 100
    ) SELECT avg((T2.`pop_sum`)) as `mpop`
      FROM `T2` T2
      LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)

  def derivedDatacase6(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [{"type": "SIMPLE", "expr": "[state] as 'state'"},
                             {"type": "SIMPLE", "expr": "[year] as 'year'", "order": {"asc": False, "pri": 1}}],
              "aggregates": [{"expr": "sum([population]) as 'population'"},
                             {"type": "SIMPLE",
                              "expr": "sum([pop_sum2]) as 'total population'"},
                             {"type": "SIMPLE",
                              "expr": "sum([population]) / sum([pop_sum2]) as 'fraction'",
                              "limit": "false"}],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.censusdata_id,
              "derived_data": [self.derivedstate(), self.derivedstateAgg1(), self.derivedstateAgg2()]}

    expected_output = """
    WITH T3 AS
    ( SELECT TA_0.`year` as `_year`, sum((TA_0.`population`)) as `pop_sum2`
      FROM `default`.`census_pop` TA_0
      GROUP BY 1
      LIMIT 100
    ) SELECT TA_0.`state` as `state`, TA_0.`year` as `year`,
      sum((TA_0.`population`)) as `population`,
      sum((T3.`pop_sum2`)) as `total population`,
      sum((TA_0.`population`)) / sum((T3.`pop_sum2`)) as `fraction`
      FROM `default`.`census_pop` TA_0 INNER JOIN `T3` T3 ON ( TA_0.`year` = T3.`_year` )
      GROUP BY 1, 2
      ORDER BY 2 DESC
      LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)

  def derivedDatacase7(self):
    dsjson = {"version": 1,
              "type": 'SQL',
              "dimensions": [{"type": "SIMPLE", "expr": "[sex] as 'sex'"}],
              "aggregates": [{"expr": "sum([day]) as 'tot_day'"},
                             {"expr": "sum([max_yr]) as 'tot_max_yr'",
                              "type": "SIMPLE"}],
              "filters": [],
              "global_segments": [{"type": "SEGMENT", "segments": []}],
              "limit": 100,
              "dataset_id": self.jvfactdata_id,
              "derived_data": [self.derivedFact2Dim()]}

    expected_output = """
    WITH T1 AS
    ( SELECT TA_0.`name` as `_name`, max((TA_2.`yr`)) as `max_yr`
      FROM `default`.`jvfact` TA_0 LEFT OUTER JOIN `default`.`jvdimp1` TA_2 ON ( TA_0.`name` = TA_2.`name` )
      GROUP BY 1
      LIMIT 100
    ) SELECT TA_1.`sex` as `sex`, sum((TA_0.`day`)) as `tot_day`, sum((T1.`max_yr`)) as `tot_max_yr`
      FROM `default`.`jvfact` TA_0 LEFT OUTER JOIN `default`.`jvdim` TA_1 ON ( TA_0.`name` = TA_1.`name` )  INNER JOIN `T1` T1 ON ( TA_0.`name` = T1.`_name` )
      GROUP BY 1
      LIMIT 100
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    # print sql
    self.assertStrings(sql, expected_output)


  # Test that when each DD only pulls columns from previous DDs, then no 
  # join is generated.
  def derivedDatacase8(self):
    dsjson = {
        'type': 'SQL',
        'version': 1,
        'dimensions': [{'expr': "[__a] as '__a'", 'type': 'SIMPLE'}],
        'aggregates': [{'expr': "sum([DD2_sumvalue]) as 'sum(DD2_sumvalue)'"}],
        'dataset_id': self.grouptest_id,
        'derived_data': [{'children': ['DD2_sumvalue'],
                           'dsreq': {'aggregates': [{'expr': "sum([value]) as 'DD1_sumvalue'"},
                                                    {'expr': "avg([value]) as 'dd1_avgvalue'"}],
                                      'dimensions': [{'expr': "[a] as '_a'", 'type': 'SIMPLE'}],
                                      'version': 1},
                           'isMeasure': True,
                           'label': 'DD1_sumvalue',
                           'type': 'INT'},
                         {'dsreq': {'aggregates': [{'expr': "sum([DD1_sumvalue]) as 'DD2_sumvalue'"}],
                                      'dimensions': [{'expr': "[_a] as '__a'", 'type': 'SIMPLE'}],
                                      'version': 1},
                           'isMeasure': True,
                           'label': 'DD2_sumvalue',
                           'parents': ['DD1_sumvalue'],
                           'type': 'INT'}],
        'limit': '110'}
    expected_output = """
      WITH T1 AS
      ( SELECT TA_0.`a` as `_a`, sum((TA_0.`value`)) as `DD1_sumvalue`, avg((TA_0.`value`)) as `dd1_avgvalue`
        FROM `default`.`grouptest` TA_0
        GROUP BY 1
      ),
      T2 AS
      ( SELECT T1.`_a` as `__a`, sum((T1.`DD1_sumvalue`)) as `DD2_sumvalue`
        FROM `T1` T1
        GROUP BY 1
      ) SELECT T2.`__a` as `__a`, sum((T2.`DD2_sumvalue`)) as `sum(DD2_sumvalue)`
        FROM `T2` T2
        GROUP BY 1
        LIMIT 110
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    self.assertStrings(sql, expected_output)

  # Test that a expression only DD does not generate a sub-select
  def derivedDatacase9(self):
    dsjson = {
        'type': 'SQL',
        'version': 1,
        'dimensions': [{'expr': "[a] as 'a'", 'type': 'SIMPLE'}],
        'aggregates': [{'expr': "[DDSumValue] as 'DDSumValue'"}],
        'dataset_id': self.grouptest_id,
        'derived_data': [
            {'dsreq': {'aggregates': [{'expr': "sum([value]) as 'DDSumValue'"}]},
             'isMeasure': True,
             'label': 'DDSumValue',
             'type': 'INT'}],
        'limit': '120'}
    expected_output = """
      SELECT TA_0.`a` as `a`, sum((TA_0.`value`)) as `DDSumValue`
        FROM `default`.`grouptest` TA_0
        GROUP BY 1
        LIMIT 120
    """

    self.request.user = self.admin
    sql = self.generateSql(dsjson, request=self.request)
    self.assertStrings(sql, expected_output)

