#!/usr/bin/python
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import unittest
import os
import sys
import time

THISDIR = os.path.dirname(os.path.realpath(__file__))

sys.path.append("%s/.." % THISDIR)

import handles

class TestHandles(unittest.TestCase):
  def setUp(self):
    self.handles = handles.Handles()

  def test_1(self):
    self.assertEqual(self.handles.get_data('abc'), None)
    self.handles.set_data('abc', 12)
    self.assertEqual(self.handles.get_data('abc'), 12)
    for i in range(0, 1023):
      self.handles.set_data(i, i+10)
    self.assertEqual(self.handles.cleanup_count(), 0)
    self.handles.set_data(1024, 2048)
    self.assertEqual(self.handles.cleanup_count(), 1)

    self.assertEqual(self.handles.handles_count(), 1025)
    self.handles.set_handle_reclaim_time(0)
    self.handles.set_data('xyz', 13)
    self.assertEqual(self.handles.handles_count(), 0)

    self.handles.set_handle_reclaim_time(10000)
    self.assertEqual(self.handles.get_data('abc'), None)
    self.handles.set_data('abc', 12)
    self.assertEqual(self.handles.get_data('abc'), 12)
    for i in range(0, 1023):
      self.handles.set_data(i, i+10)
    self.assertEqual(self.handles.cleanup_count(), 2)
    self.handles.set_data(1024, 2048)
    self.assertEqual(self.handles.cleanup_count(), 3)

    self.assertEqual(self.handles.handles_count(), 1025)
    self.handles.set_handle_reclaim_time(0)
    self.handles.set_data('xyz', 13)
    self.assertEqual(self.handles.handles_count(), 0)

  def tearDown(self):
    pass

if __name__ == '__main__':
  unittest.main()

