# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import unittest
from vizlib import misc
from unittest import mock


class TestMiscRequestSourceIP(unittest.TestCase):
  def setUp(self):
    self.mock_request = mock.MagicMock()
    self.mock_request.META = {}
    return super().setUp()

  def test_tolerates_missing_headers(self):

    ip = misc.get_request_source_ip(self.mock_request)
    # since this is mainly for audit logging, we return an empty string when the IP can not be found:
    self.assertEquals(ip, '')

  def test_uses_remote_address(self):
    self.mock_request.META['REMOTE_ADDR'] = 'AAA'

    ip = misc.get_request_source_ip(self.mock_request)
    self.assertEquals(ip, 'AAA')

  def test_prefers_forwarded_for(self):
    self.mock_request.META['REMOTE_ADDR'] = 'AAA'
    self.mock_request.META['HTTP_X_FORWARDED_FOR'] = 'BBB'

    ip = misc.get_request_source_ip(self.mock_request)
    self.assertEquals(ip, 'BBB')

  def test_returns_first_address(self):
    self.mock_request.META['REMOTE_ADDR'] = 'AAA'
    self.mock_request.META['HTTP_X_FORWARDED_FOR'] = 'BBB,CCC'

    ip = misc.get_request_source_ip(self.mock_request)
    self.assertEquals(ip, 'BBB')
