# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json

import django.db
import django.test
from django.test.client import Client
from unittest import mock

from rbac.models import Role
from rbac.permcache import RBAC
from vizlib.tests.util import run_with_db_query_check, run_with_db_query_check2
from .test_report_operations import make_user, create_client
from . import test_report_operations

# test scenario 1
# First, create a public dashboard with two visuals, each using different datasets.
# Convert the first visual into a linked visual.
# Create a private dashboard and clone the linked visual from the previously created dashboard.
# Create a third private dashboard where you will maintain the cloned linked visual from the first dashboard.

# test scenario 2
# First, create a private dashboard with two visuals, each using different datasets.
# Convert the first visual into a linked visual.
# Create a public dashboard and clone the linked visual from the previously created dashboard.
# Create a third public dashboard where you will maintain the cloned linked visual from the first dashboard.

dataset_ids = {
  'ds1_id': '',
  'ds2_id': '',
  'ds3_id': '',
}

dashboard_ids = {
  'dashboard1': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard2': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard3': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard4': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard5': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard6': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard7': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  'dashboard8': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  # test scenario 1
  # dashboard contains linked visual - PUBLIC
  'dashboard9': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  # clone the linked visual from dashboard9 - PRIVATE
  'dashboard10': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  # maintain the linked visual from dashboard9 - PRIVATE
  'dashboard11': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  # test scenario 2
  # dashboard contains linked visual - PRIVATE
  'dashboard12': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  # clone the linked visual from dashboard12 - PUBLIC
  'dashboard13': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
  # maintain the linked visual from dashboard12 - PUBLIC
  'dashboard14': {'id': '', 'associated_dataset_id': '', 'associated_dataset': '', 'visual_ids': [], 'workspace': ''},
}

visuals_and_dashboard1 = [
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard1', 'dashboard_name': 'dashboard1'},
    {'visual_dataset': 'ds2_id', 'visual_type': 'table', 'visual_name': 'ds2_id dashboard1', 'dashboard_name': 'dashboard1'}
]

visuals_and_dashboard2 = [
    {'visual_dataset': 'ds2_id', 'visual_type': 'table', 'visual_name': 'ds2_id dashboard2', 'dashboard_name': 'dashboard2'},
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard2', 'dashboard_name': 'dashboard2'}
]

visuals_and_dashboard3 = [
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard3', 'dashboard_name': 'dashboard3'}
]

visuals_and_dashboard4 = [
    {'visual_dataset': 'ds3_id', 'visual_type': 'table', 'visual_name': 'ds3_id dashboard4', 'dashboard_name': 'dashboard4'},
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard4', 'dashboard_name': 'dashboard4'}
]

visuals_and_dashboard5 = [
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard5', 'dashboard_name': 'dashboard5'},
    {'visual_dataset': 'ds3_id', 'visual_type': 'table', 'visual_name': 'ds3_id dashboard5', 'dashboard_name': 'dashboard5'}
]

visuals_and_dashboard6 = [
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard6', 'dashboard_name': 'dashboard6'},
]

visuals_and_dashboard7 = [
    {'visual_dataset': 'ds3_id', 'visual_type': 'table', 'visual_name': 'ds3_id dashboard7', 'dashboard_name': 'dashboard7'},
    {'visual_dataset': 'ds2_id', 'visual_type': 'table', 'visual_name': 'ds2_id dashboard7', 'dashboard_name': 'dashboard7'}
]

visuals_and_dashboard8 = [
    {'visual_dataset': 'ds2_id', 'visual_type': 'table', 'visual_name': 'ds2_id dashboard8', 'dashboard_name': 'dashboard8'},
    {'visual_dataset': 'ds3_id', 'visual_type': 'table', 'visual_name': 'ds3_id dashboard8', 'dashboard_name': 'dashboard8'},
    {'visual_dataset': 'ds1_id', 'visual_type': 'table', 'visual_name': 'ds1_id dashboard8', 'dashboard_name': 'dashboard8'}
]
# test scenario 1
# dashboard with two visuals, each using different datasets - PUBLIC
visuals_and_dashboard9 = [
    # will convert it to linked visual
    {'visual_dataset': 'ds4_id', 'visual_type': 'table', 'visual_name': 'ds4_id dashboard9', 'dashboard_name': 'dashboard9'},
    {'visual_dataset': 'ds5_id', 'visual_type': 'table', 'visual_name': 'ds5_id dashboard9', 'dashboard_name': 'dashboard9'}
]
# clone linked visual into dashboard10 from dashboard9 - PRIVATE
visuals_and_dashboard10 = [
    {'visual_dataset': 'ds5_id', 'visual_type': 'table', 'visual_name': 'ds5_id dashboard10', 'dashboard_name': 'dashboard10'}
]

# maintain linked visual into dashboard11 from dashboard9 - PRIVATE
visuals_and_dashboard11 = [
    {'visual_dataset': 'ds5_id', 'visual_type': 'table', 'visual_name': 'ds5_id dashboard11', 'dashboard_name': 'dashboard11'}
]

# test scenario 2
# dashboard with two visuals, each using different datasets - PRIVATE
visuals_and_dashboard12 = [
    # will convert it to linked visual
    {'visual_dataset': 'ds6_id', 'visual_type': 'table', 'visual_name': 'ds6_id dashboard12', 'dashboard_name': 'dashboard12'},
    {'visual_dataset': 'ds7_id', 'visual_type': 'table', 'visual_name': 'ds7_id dashboard12', 'dashboard_name': 'dashboard12'}
]
# clone linked visual into dashboard10 from dashboard12 - PUBLIC
visuals_and_dashboard13 = [
    {'visual_dataset': 'ds7_id', 'visual_type': 'table', 'visual_name': 'ds7_id dashboard13', 'dashboard_name': 'dashboard13'}
]

# maintain linked visual into dashboard12 from dashboard9 - PUBLIC
visuals_and_dashboard14 = [
    {'visual_dataset': 'ds7_id', 'visual_type': 'table', 'visual_name': 'ds7_id dashboard14', 'dashboard_name': 'dashboard14'}
]


def full_access_only_one_dataset_role(dc_id: str, ds_manage_id: str):
  role = Role()
  role.name = 'mixed_ds_perms'
  role.privileges = json.dumps(
    [
      # no system permissions:
      {'ptype': 'system', 'perms': []},
      # full rights for the other
      {
        'ptype': 'dataset',
        'dcid': str(dc_id),
        'dslist': [str(ds_manage_id)],
        'perms': ['ds_manage', 'ds_appview', 'ds_appedit'],
      },
    ]
  )
  role.save()
  return role

def setup_user_role(user, dc_id, ds_id):
  mixed_perms_role = full_access_only_one_dataset_role(
    dc_id=dc_id, ds_manage_id=ds_id
  )
  mixed_perms_role.add_users([user.username])
  mixed_perms_role.save()

def create_dashboard_and_visuals(admin_client, dataset_id, visuals_info, util, workspace, dashboard_name,
                                 dataset_name, dashboard_ids):
  visuals = []
  for visual_info in visuals_info:
    visual = util.create_visual(
      admin_client,
      dataset_ids[visual_info['visual_dataset']],
      visual_info['visual_type'],
    )
    visuals.append(visual)

  dashboard = util.create_dashboard(
    admin_client, dataset_id=dataset_id, visuals=visuals, report_name=visuals_info[0]['dashboard_name']
  )
  if workspace == 'Public':
    util.move_to_public_workspace(admin_client, dashboard_id=dashboard)

  dashboard_ids[dashboard_name]['id'] = dashboard
  dashboard_ids[dashboard_name]['associated_dataset_id'] = dataset_id
  dashboard_ids[dashboard_name]['associated_dataset'] = dataset_name
  dashboard_ids[dashboard_name]['visual_ids'] = visuals
  dashboard_ids[dashboard_name]['workspace'] = workspace

  return dashboard


def find_dashboard_info_by_id(dashboard_ids, target_id):
  for name, info in dashboard_ids.items():
    if info['id'] == target_id:
      return {'name': name, 'associated_dataset_id': info['associated_dataset_id'],
              'associated_dataset': info['associated_dataset']}
  return None


class TestReportCountBase(django.test.TestCase):
  def common_setup(self):
    util = test_report_operations.TestDashboardOperations()
    self.util = util
    # crete admin
    admin = make_user('admin', is_superuser=True)
    admin_client = create_client(admin)
    self.admin_client = admin_client
    # create user1
    user1 = make_user('user1', is_superuser=False)
    user_client1 = create_client(user1)
    self.user1 = user1
    self.user_client1 = user_client1
    # create user2
    user2 = make_user('user2', is_superuser=False)
    user_client2 = create_client(user2)
    self.user2 = user2
    self.user_client2 = user_client2
    # create user3 for linked visual test - for test scenario 1
    user3 = make_user('user3', is_superuser=False)
    user_client3 = create_client(user3)
    self.user3 = user3
    self.user_client3 = user_client3
    # create user4 for linked visual test for test scenario 2
    user4 = make_user('user4', is_superuser=False)
    user_client4 = create_client(user4)
    self.user4 = user4
    self.user_client4 = user_client4
    # create sqlite connection and dataset
    dc_id1 = self.create_dataconnection(admin_client, 'sqlite')
    self.dc_id1 = dc_id1
    ds1_id = util.create_dataset(admin_client, dc_id1, 'ds1')
    self.ds1_id = ds1_id
    dataset_ids['ds1_id'] = ds1_id
    # create impala connection and dataset
    dc_id2 = self.create_dataconnection(admin_client, 'impala')
    ds2_id = util.create_dataset(admin_client, dc_id2, 'ds2')
    self.ds2_id = ds2_id
    dataset_ids['ds2_id'] = ds2_id
    # create datasets for checking linked visuals - test scenario 1
    ds4_id = util.create_dataset(admin_client, dc_id1, 'ds4')
    self.ds4_id = ds4_id
    dataset_ids['ds4_id'] = ds4_id
    ds5_id = util.create_dataset(admin_client, dc_id1, 'ds5')
    self.ds5_id = ds5_id
    dataset_ids['ds5_id'] = ds5_id
    # create datasets for checking linked visuals - test scenario 2
    ds6_id = util.create_dataset(admin_client, dc_id1, 'ds6')
    self.ds6_id = ds6_id
    dataset_ids['ds6_id'] = ds6_id
    ds7_id = util.create_dataset(admin_client, dc_id1, 'ds7')
    self.ds7_id = ds7_id
    dataset_ids['ds7_id'] = ds7_id
    # role : user1 - full access to ds1_id
    setup_user_role(user1, dc_id1, ds1_id)
    # role : user2 - full access to ds2_id
    setup_user_role(user2, dc_id2, ds2_id)
    # role : user3 - full access to ds4_id - test scenario 1
    setup_user_role(user3, dc_id1, ds4_id)
    # role : user4 - full access to ds4_id - test scenario 2
    setup_user_role(user4, dc_id1, ds6_id)

  def create_dataconnection(self, admin_client, connection_type):
    impala_payload = {
      'dataconnection_type': 'impyla',
      'dataconnection_info': '{}',
      'dataconnection_name': 'Impala Judit',
    }
    sqlite_payload = {
      'dataconnection_type': 'sqlite',
      'dataconnection_info': '{}',
      'dataconnection_name': 'samples',
    }
    if connection_type == 'impala':
      dc_payload = impala_payload
    else:
      dc_payload = sqlite_payload

    return self.util.create_dataconnection_with_payload(admin_client, dc_payload)

  def setUp(self):
    self.common_setup()


class TestReportCount(TestReportCountBase):
  def setUp(self):
    super().common_setup()

  def test_report_count_on_data_page(self):
    user_client = self.user_client1

    self.dashboard1 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds1_id,
      visuals_and_dashboard1,
      self.util,
      'Public',
      'dashboard1',
      'ds1_id',
      dashboard_ids
    )

    self.dashboard2 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds2_id,
      visuals_and_dashboard2,
      self.util,
      'Public',
      'dashboard2',
      'ds2_id',
      dashboard_ids
    )

    _, first_db_calls_count = run_with_db_query_check(
      lambda: self._check_report_count_on_data_page(user_client, 1))

    self.dashboard3 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds1_id,
      visuals_and_dashboard3,
      self.util,
      'Public',
      'dashboard3',
      'ds1_id',
      dashboard_ids
    )
    # check whether the new db shows up
    self._check_report_count_on_data_page(user_client, 2)
    _, second_db_calls_count = run_with_db_query_check(
      lambda: self._check_report_count_on_data_page(user_client, 2))

    # the number of DB calls must remain the same even if the dashboard count increases:
    self.assertEqual(first_db_calls_count, second_db_calls_count)

    # admin can access all 3 dashboards
    self._check_report_count_on_data_page(self.admin_client, 3)

  def test_report_count_on_dataset_page(self):
    self.dashboard1 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds1_id,
      visuals_and_dashboard1,
      self.util,
      'Public',
      'dashboard1',
      'ds1_id',
      dashboard_ids
    )

    self.dashboard2 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds2_id,
      visuals_and_dashboard2,
      self.util,
      'Public',
      'dashboard2',
      'ds2_id',
      dashboard_ids
    )

    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds1_id, 1)
    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds2_id, 0)
    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds1_id, 0)
    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds2_id, 1)

    util = self.util
    admin_client = self.admin_client
    dc_id1 = self.dc_id1
    ds1_id = self.ds1_id

    ds3_id = util.create_dataset(admin_client, dc_id1, 'ds3')
    self.ds3_id = ds3_id
    dataset_ids['ds3_id'] = ds3_id

    self.dashboard4 = create_dashboard_and_visuals(
      admin_client,
      ds3_id,
      visuals_and_dashboard4,
      util,
      'Public',
      'dashboard4',
      'ds3_id',
      dashboard_ids
    )

    setup_user_role(self.user2, dc_id1, ds3_id)

    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds1_id, 1)
    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds2_id, 0)
    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds3_id, 0)

    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds1_id, 0)
    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds2_id, 1)
    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds3_id, 1)

    self.dashboard5 = create_dashboard_and_visuals(
      admin_client,
      ds1_id,
      visuals_and_dashboard5,
      util,
      'Public',
      'dashboard5',
      'ds1_id',
      dashboard_ids
    )

    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds1_id, 2)
    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds2_id, 0)
    self._check_report_list_count_on_dataset_page(self.user_client1, self.ds3_id, 0)

    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds1_id, 0)
    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds2_id, 1)
    self._check_report_list_count_on_dataset_page(self.user_client2, self.ds3_id, 1)

  def test_report_count_in_delete_modal(self):
    util = self.util
    admin_client = self.admin_client
    dc_id1 = self.dc_id1
    ds1_id = self.ds1_id

    self.dashboard1 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds1_id,
      visuals_and_dashboard1,
      self.util,
      'Public',
      'dashboard1',
      'ds1_id',
      dashboard_ids
    )

    self.dashboard2 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds2_id,
      visuals_and_dashboard2,
      self.util,
      'Public',
      'dashboard2',
      'ds2_id',
      dashboard_ids
    )

    self.dashboard3 = create_dashboard_and_visuals(
      admin_client,
      ds1_id,
      visuals_and_dashboard3,
      util,
      'Public',
      'dashboard3',
      'ds1_id',
      dashboard_ids
    )

    ds3_id = util.create_dataset(admin_client, dc_id1, 'ds3')
    self.ds3_id = ds3_id
    dataset_ids['ds3_id'] = ds3_id

    self.dashboard4 = create_dashboard_and_visuals(
      admin_client,
      ds3_id,
      visuals_and_dashboard4,
      util,
      'Public',
      'dashboard4',
      'ds3_id',
      dashboard_ids
    )

    # user1
    # A total of 4 dashboards were created up to that point.
    # There are 3 visuals in dashboards with mixed datasets.
    # 1 dashboard is built solely on ds1_id.
    # None of the dashboards are restricted; all are public.
    self._check_report_count_in_delete_modal(
      self.user_client1,
      self.ds1_id,
      visuals=3,
      dashboards=1,
      restricted_visuals=0,
      restricted_dashboards=0,
      debug=False
    )

    # user2
    # Out of the 4 dashboards, 3 have visuals that use mixed datasets.
    # 2 of these datasets is ds2_id.
    self._check_report_count_in_delete_modal(
      self.user_client2,
      self.ds2_id,
      visuals=2,
      dashboards=0,
      restricted_visuals=0,
      restricted_dashboards=0,
      debug=False
    )

    # create Private dashboard
    self.dashboard6 = create_dashboard_and_visuals(
      admin_client,
      ds1_id,
      visuals_and_dashboard6,
      util,
      'Private',
      'dashboard6',
      'ds1_id',
      dashboard_ids
    )

    self.dashboard7 = create_dashboard_and_visuals(
      admin_client,
      ds3_id,
      visuals_and_dashboard7,
      util,
      'Private',
      'dashboard7',
      'ds3_id',
      dashboard_ids
    )

    # user1
    # A total of 6 dashboards were created up to that point.
    # There are 3 visuals in dashboards with mixed datasets.
    # 1 dashboard is built solely on ds1_id.
    # And 1 private dashboard
    self._check_report_count_in_delete_modal(
      self.user_client1,
      self.ds1_id,
      visuals=3,
      dashboards=2,
      restricted_visuals=0,
      restricted_dashboards=1,
      debug=False
    )

    # user2
    # There is 2 dashboards using ds2_id with other dataset
    # And 1 visual is Private
    self._check_report_count_in_delete_modal(
      self.user_client2,
      self.ds2_id,
      visuals=3,
      dashboards=0,
      restricted_visuals=1,
      restricted_dashboards=0,
      debug=False
    )

    # linked visual - test scenario 1
    # create dashboard9 with 2 visuals
    # 1 visual will be converted to linked in line 555
    self.dashboard9 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds4_id,
      visuals_and_dashboard9,
      util,
      'Public',
      'dashboard9',
      'ds4_id',
      dashboard_ids
    )

    # user 3
    # 1 dashboard using mixed dataset
    self._check_report_count_in_delete_modal(
      self.user_client3,
      self.ds4_id,
      visuals=1,
      dashboards=0,
      restricted_visuals=0,
      restricted_dashboards=0,
      debug=False
    )

    # update the visual to linked visual in dashboard9
    self.dashboard9 = self.update_visual_to_linked_visual(self.admin_client, self.ds4_id, dashboard_ids, 'dashboard9')

    # nothing changed
    # still 1 dashboard using mixed dataset
    self._check_report_count_in_delete_modal(
      self.user_client3,
      self.ds4_id,
      visuals=1,
      dashboards=0,
      restricted_visuals=0,
      restricted_dashboards=0,
      debug=False
    )

    # create dashboard10 with dashboard9's linked visual + 1 different ds visual
    self.dashboard10 = self.create_dashboard_and_visuals_with_clone_visual(
      self.admin_client,
      self.ds4_id,
      visuals_and_dashboard10,
      util,
      'Private',
      'dashboard10',
      'ds4_id',
      dashboard_ids,
      'dashboard9'
    )

    # As we cloned dashboard9 linked visual to dashboard10
    # and dashboard10 is Private
    # 1 visual is Public
    # other one is Private
    self._check_report_count_in_delete_modal(
      self.user_client3,
      self.ds4_id,
      visuals=2,
      dashboards=0,
      restricted_visuals=1,
      restricted_dashboards=0,
      debug=False
    )

    self.dashboard11 = self.create_dashboard_and_visuals_with_maintain_visual(
      self.admin_client,
      self.ds4_id,
      visuals_and_dashboard11,
      util,
      'Private',
      'dashboard11',
      'ds4_id',
      dashboard_ids,
      self.dashboard9
    )

    # we use the dashboard9's linked visual in dashboard11
    # as maintain linked visual which is Private
    # so 1 visual the cloned linked visual
    # other visual is the maintain linked visual with ne id
    # both Private
    self._check_report_count_in_delete_modal(
      self.user_client3,
      self.ds4_id,
      visuals=2,
      dashboards=0,
      restricted_visuals=2,
      restricted_dashboards=0,
      debug=False
    )

    # linked visual - test scenario 2
    # create dashboard12 which will contains a linked visual
    self.dashboard12 = create_dashboard_and_visuals(
      self.admin_client,
      self.ds6_id,
      visuals_and_dashboard12,
      util,
      'Private',
      'dashboard12',
      'ds6_id',
      dashboard_ids
    )

    # user 4
    # dashboard12 is Private
    # so 1 restricted visual displayed
    self._check_report_count_in_delete_modal(
      self.user_client4,
      self.ds6_id,
      visuals=1,
      dashboards=0,
      restricted_visuals=1,
      restricted_dashboards=0,
      debug=False
    )

    # update the visual to linked visual in dashboard12
    self.dashboard12 = self.update_visual_to_linked_visual(self.admin_client, self.ds6_id, dashboard_ids, 'dashboard12')

    # after cloning the linked visual
    # nothing changed with the number
    # dashboard12 is still Private
    # so 1 restricted visual displayed
    self._check_report_count_in_delete_modal(
      self.user_client4,
      self.ds6_id,
      visuals=1,
      dashboards=0,
      restricted_visuals=1,
      restricted_dashboards=0,
      debug=False
    )

    # create dashboard13 with dashboard12's linked visual + 1 different ds visual
    self.dashboard13 = self.create_dashboard_and_visuals_with_clone_visual(
      self.admin_client,
      self.ds6_id,
      visuals_and_dashboard13,
      util,
      'Public',
      'dashboard13',
      'ds6_id',
      dashboard_ids,
      'dashboard12'
    )

    # we cloned the linked visual
    # to a Private dashboard
    # in that case the dashboard is Private
    # 1 restricted visual displayed
    self._check_report_count_in_delete_modal(
      self.user_client4,
      self.ds6_id,
      visuals=2,
      dashboards=0,
      restricted_visuals=1,
      restricted_dashboards=0,
      debug=False
    )

    self.dashboard14 = self.create_dashboard_and_visuals_with_maintain_visual(
      self.admin_client,
      self.ds6_id,
      visuals_and_dashboard14,
      util,
      'Public',
      'dashboard14',
      'ds6_id',
      dashboard_ids,
      self.dashboard12
    )

    # dashboard14 is using the dashboard12's linked visual
    # as maintain visual which stays Private inheriting from dashboard12
    # so 2 visuals
    # 1 private and 1 public
    self._check_report_count_in_delete_modal(
      self.user_client4,
      self.ds6_id,
      visuals=2,
      dashboards=0,
      restricted_visuals=1,
      restricted_dashboards=0,
      debug=False
    )

  def clone_linked_visual(self, client: Client, dataset_id, cloned_visual_id, workspace_id):
    payload = {
      'add_clone_of': 'true',
      'dataset_id': dataset_id,
      'report_id': cloned_visual_id,
      'sheet_id': 1,
      'workspace_id': workspace_id
    }

    response = client.post(f'/reports/clone', payload)
    self.assertEquals(response.status_code, 200)

    cloned_visual_info = response.json()[0]

    return cloned_visual_info

  def create_dashboard_and_visuals_with_maintain_visual(self, client: Client, dataset_id, visuals_info, util, workspace, dashboard_name, dataset_name, dashboard_ids, linked_visual_dashboard_id):
    linked_visual_id = linked_visual_dashboard_id
    # grab the linked visual id and create a dashboard with it
    visuals = [linked_visual_id]
    for visual_info in visuals_info:
      visual = util.create_visual(
        client,
        dataset_ids[visual_info['visual_dataset']],
        visual_info['visual_type'],
      )
      visuals.append(visual)

    dashboard = util.create_dashboard(
      client, dataset_id=dataset_id, visuals=visuals, report_name=visuals_info[0]['dashboard_name']
    )

    dashboard_ids[dashboard_name]['id'] = dashboard
    dashboard_ids[dashboard_name]['associated_dataset_id'] = dataset_id
    dashboard_ids[dashboard_name]['associated_dataset'] = dataset_name
    dashboard_ids[dashboard_name]['visual_ids'] = visuals
    dashboard_ids[dashboard_name]['workspace'] = workspace

    return dashboard

  def create_dashboard_and_visuals_with_clone_visual(self, client: Client, dataset_id, visuals_info, util, workspace, dashboard_name, dataset_name, dashboard_ids, dashboard):
    linked_visual_id = dashboard_ids[dashboard]['visual_ids'][0]
    workspace_id = 1 if workspace == 'Public' else 2
    # grab the linked_visual_id, clone it -> new id will be returned
    # create a dashboard with the new cloned linked visual id
    cloned_visual_info = self.clone_linked_visual(client, dataset_id, linked_visual_id, workspace_id)

    visuals = [cloned_visual_info['id']]

    for visual_info in visuals_info:
      visual = util.create_visual(
        client,
        dataset_ids[visual_info['visual_dataset']],
        visual_info['visual_type'],
      )
      visuals.append(visual)

    dashboard = util.create_dashboard(
      client, dataset_id=dataset_id, visuals=visuals, report_name=visuals_info[0]['dashboard_name']
    )
    if workspace == 'Public':
      util.move_to_public_workspace(client, dashboard_id=dashboard)

    dashboard_ids[dashboard_name]['id'] = dashboard
    dashboard_ids[dashboard_name]['associated_dataset_id'] = dataset_id
    dashboard_ids[dashboard_name]['associated_dataset'] = dataset_name
    dashboard_ids[dashboard_name]['visual_ids'] = visuals
    dashboard_ids[dashboard_name]['workspace'] = workspace

    return dashboard

  def update_visual_to_linked_visual(self, client: Client, dataset_id, dashboards, dashboard):
    dashboard_id = dashboards[dashboard]['id']
    visual_id = dashboards[dashboard]['visual_ids'][0]
    payload = {
      'shared_visual_dashboards': json.dumps([dashboard_id]),
      'shared_visual_only': 'true',
      'dataset_id': str(dataset_id),
      'dashboard_id': dashboard_id,
    }

    response = client.post(f'/reports/report/{visual_id}', payload)

    self.assertEquals(response.status_code, 200)
    updated_visual_id = int(response.content.decode('utf-8'))

    return updated_visual_id

  def _check_report_count_in_delete_modal(
    self,
    client: Client,
    ds_id,
    visuals: int,
    dashboards: int,
    restricted_visuals: int,
    restricted_dashboards: int,
    debug: bool,
  ):

    visuals_act, dashboards_act, restricted_visuals_act, restricted_dashboards_act = run_with_db_query_check2(
            lambda: self.get_report_count_list_in_delete_modal(client, ds_id, debug))

    self.assertEquals(visuals_act, visuals)
    self.assertEquals(dashboards_act, dashboards)
    self.assertEquals(restricted_visuals_act, restricted_visuals)
    self.assertEquals(restricted_dashboards_act, restricted_dashboards)

  def get_report_count_list_in_delete_modal(self, client: Client, ds_id, debug):
    visuals = 0
    dashboards = 0
    no_perms_visuals_expected_count = 0
    no_perms_dashboards_expected_count = 0
    payload = {
      'dataset_id': ds_id
    }
    response = client.get('/datasets/possible_deleted_reports', payload)
    self.assertEquals(response.status_code, 200)
    report_list_in_delete_modal = response.json()

    for report in report_list_in_delete_modal:
      if debug:
        print('report: ', report)

      # dataset id appear in the report
      # user has permission to the dataset
      if 'dataset_id' in report and report['dataset_id']:
        if report['type'] == 'dashboard':
          dashboards += 1
        else:
          visuals += 1
      # user has no permission to the dataset
      else:
        if report['type'] == 'dashboard':
          dashboards += 1
          no_perms_dashboards_expected_count += 1
        else:
          visuals += 1
          no_perms_visuals_expected_count += 1
    # visuals and dashboards are a summary variable
    # no_perms_visuals_expected_count and no_perms_dashboards_expected_count
    # represent individual counts for dashboards and visuals without permissions
    return visuals, dashboards, no_perms_visuals_expected_count, no_perms_dashboards_expected_count

  def _check_report_list_count_on_dataset_page(self, client: Client, ds_id, expected_count: int):

    actual_report_count = run_with_db_query_check2(
            lambda: self.get_report_count_list_on_dataset_page(client, ds_id))

    self.assertEquals(actual_report_count, expected_count)

  def _check_report_count_on_data_page(self, client: Client, expected_count: int):

    actual_report_count = run_with_db_query_check2(
            lambda: self.get_report_count_on_data_page(client))

    self.assertEquals(actual_report_count, expected_count)

  def get_report_count_list_on_dataset_page(self, client: Client, ds_id):
    payload = {
      'dataset_id': ds_id
    }
    response = client.get('/reports/report', payload)
    self.assertEquals(response.status_code, 200)
    report_list = response.json()
    return len(report_list)

  def get_report_count_on_data_page(self, client: Client):
    payload = {
      'get_associate_dashboard': 'true'
    }
    response = client.get('/datasets/dataset', payload)
    report_list = response.json()

    self.assertEquals(response.status_code, 200)

    count = 0
    for r in response.json():
      count += r['report_count']
    return count

class TestReportDatasetAfterVisualDeletion(TestReportCountBase):
  def setUp(self):
    super().common_setup()

  def test_report_dataset_on_visual_deletion(self):
    util = self.util
    admin_client = self.admin_client
    dc_id1 = self.dc_id1
    ds2_id = self.ds2_id

    ds3_id = util.create_dataset(admin_client, dc_id1, 'ds3')
    self.ds3_id = ds3_id
    dataset_ids['ds3_id'] = ds3_id

    self.dashboard8 = create_dashboard_and_visuals(
      admin_client,
      ds2_id,
      visuals_and_dashboard8,
      util,
      'Public',
      'dashboard8',
      'ds2_id',
      dashboard_ids
    )

    self._check_report_dataset_id(self.admin_client, 2)

    self.delete_visual_from_dashboard(self.admin_client, self.dashboard8, 1)

    # sending the new dataset_id manually in the payload
    # as we are updating the dashboard dataset_id in the frontend
    self.save_dashboard(self.admin_client, ds3_id, self.dashboard8, 'dashboard8', [1, 2, 3])

    self._check_report_dataset_id(self.admin_client, 7)

  def save_dashboard(self, client: Client, dataset_id, dashboard_id, dashboard_name, visuals):
    payload = {
      'dashboard_name': dashboard_name,
      'unshared_visuals': json.dumps(visuals),
      'dashboard_id': dashboard_id,
      'dashboard_dataset_id': dataset_id,
      'dashboard_data': json.dumps({}),
    }
    response = client.post('/reports/savedashboard', payload)
    self.assertEquals(response.status_code, 200)

  def delete_visual_from_dashboard(self, client: Client, dashboard_id, visual_id):

    response = client.delete(f'/reports/report/{visual_id}?source_dashboard={dashboard_id}')
    self.assertEquals(response.status_code, 200)

  def _check_report_dataset_id(self, client: Client, expected_dataset_id: int):

    current_dashboard_dataset_id = run_with_db_query_check2(
            lambda: self.get_report_dataset_id(client, dashboard_ids['dashboard8']['id']))

    self.assertEquals(current_dashboard_dataset_id, expected_dataset_id)

  def get_report_dataset_id(self, admin_client, dashboard_id):
    admin_client = admin_client
    current_dashboard_detail_list = self.get_report_details(admin_client, dashboard_id)

    # Assuming that current_dashboard_detail_list is a list with a single dictionary
    if current_dashboard_detail_list:
      current_dashboard_detail = current_dashboard_detail_list[0]
      return current_dashboard_detail.get('dataset_id')
    else:
      # Handle the case where the list is empty
      return None

  def get_report_details(self, client: Client, dashboard_id):
    payload = {
      'assets': 'true'
    }
    response = client.get(f'/reports/report/{dashboard_id}', payload)
    self.assertEquals(response.status_code, 200)
    report_data = response.json()
    return report_data

