# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from django.core.exceptions import ValidationError
from dataconn.dataconn import _build_dataconn
from .test_base_dataconn import Test_Base_Data_Connection


class Test_SqlInjection_arcengine(Test_Base_Data_Connection):
  def __init__(self, *args, **kwargs):
    self.dataconnection_type = 'arcengine'
    super(Test_SqlInjection_arcengine, self).__init__(*args, **kwargs)

  def testUnquote(self):
    unquote = _build_dataconn(self.dc.id).unquote
    self.assertEqual(unquote("abc"), "abc")
    self.assertEqual(unquote("abc 'from' def"), "abc   def")

    self.assertEqual(unquote("""abc 'from " ' def"""), "abc   def")
    self.assertEqual(unquote("""abc "from ' " def"""), "abc   def")

    self.assertEqual(unquote("abc 'from \\' def' def"), "abc   def")
    self.assertEqual(unquote("abc \" from def\" def"), "abc   def")

  def testDimensions(self):
    validate_dsreq = _build_dataconn(self.dc.id).validate_dsreq
    validate_dsreq({
    })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'dimensions': [{
            'expr': 'from',
          }
        ],
      })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'dimensions': [{
            'expr': 'a from b',
          }
        ],
      })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'dimensions': [{
            'expr': 'a from',
          }
        ],
      })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'dimensions': [{
            'expr': 'from b',
          }
        ],
      })

    validate_dsreq({
      'dimensions': [{
          'expr': 'frommage',
        }
      ],
    })

    validate_dsreq({
      'dimensions': [{
          'expr': 'a as "from"',
        }
      ],
    })

    validate_dsreq({
      'dimensions': [{
          'expr': "a as 'from'",
        }
      ],
    })

    validate_dsreq({
      'dimensions': [{
          'expr': 'a as " \\" from"',
        }
      ],
    })

  def testAggregates(self):
    validate_dsreq = _build_dataconn(self.dc.id).validate_dsreq
    validate_dsreq({
    })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'aggregates': [{
            'expr': 'from',
          }
        ],
      })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'aggregates': [{
            'expr': 'a from b',
          }
        ],
      })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'aggregates': [{
            'expr': 'a from',
          }
        ],
      })

    with self.assertRaises(ValidationError):
      validate_dsreq({
        'aggregates': [{
            'expr': 'from b',
          }
        ],
      })

    validate_dsreq({
      'aggregates': [{
          'expr': 'frommage',
        }
      ],
    })

class Test_SqlInjection_postgres(Test_Base_Data_Connection):
  def __init__(self, *args, **kwargs):
    self.dataconnection_type = 'postgres'
    super(Test_SqlInjection_postgres, self).__init__(*args, **kwargs)

  def testUnquote(self):
    unquote = _build_dataconn(self.dc.id).unquote
    self.assertEqual(unquote("/*comment*/from"), " from")
    self.assertEqual(unquote("/*comment*/from /*comment2*/"), " from  ")
    self.assertEqual(unquote("/*comment*/'from' /*comment2*/"), "    ")