# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import django.test
import tempfile
import xlrd

from datetime import datetime
from dateutil import parser
from sqlrun import views
from unittest import mock

def mock_case_restorer(build_request, colnames):
  return colnames

def save_response_to_file(response):
  # filename = '/tmp/test-output.xlsx'
  filename = tempfile.NamedTemporaryFile(dir='/tmp').name
  with open(filename, 'wb') as f:
    for chunk in response.streaming_content:
      if chunk:
        f.write(chunk)
  return filename

class TestXlsxCreation(django.test.TestCase):

  def check_xlsx_content(self, filename, expected_content):
    book = xlrd.open_workbook(filename)
    sheet = book.sheet_by_index(0)

    for rindex, expected_row in enumerate(expected_content):
      xlsx_row = sheet.row(rindex)
      for cindex, expected_value in enumerate(expected_row):

        actual_value = xlsx_row[cindex].value
        if isinstance(expected_value, datetime):
          actual_value = xlrd.xldate.xldate_as_datetime(actual_value, book.datemode)

        self.assertEqual(expected_value, actual_value)

  @mock.patch('sqlrun.views.restore_column_name_casing')
  def test_null_in_dates(self, mockRestorer: mock.Mock):
    mockRestorer.side_effect=mock_case_restorer

    column_names = ['unformatted_time', 'formatted_time']
    date1 = '2023-04-29 11:02:03'
    date2 = '2023-04-29 21:02:03'
    format_settings = [
      None,
      {
        "type": "date",
        "result": "YYYY-MM-DD",
        "dateoptions": "YYYY-MM-DD",
        "dateformat": "YYYY-MM-DD",
        "colName": "formatted_time"
      }
    ]

    data = {
      'coltypes': ['TIMESTAMP','TIMESTAMP'],
      'colnames': column_names,
      'rows': [
        [None, date1],
        [date2, None]
      ]
    }

    response = views.getXlsxResponse(
      cached_data=data,
      format_settings=format_settings
    )

    filename = save_response_to_file(response)

    expected_content = [
      column_names,
      ['', parser.parse(date1)],
      [ parser.parse(date2), '']
    ]
    self.check_xlsx_content(filename, expected_content)