# -*- coding: utf-8 -*-
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from __future__ import unicode_literals
from django.db import migrations, models
from django.conf import settings
from workspace.models import WSACL_Member, WSACL_Level
from settings import EVERYONE_GROUP

import django.db.models.deletion
import workspace.models
import arcwebbase.validator
import json


def create_public_workspace(apps, schema_editor):
    acl = json.dumps([[WSACL_Member.GROUP, WSACL_Level.EDIT, EVERYONE_GROUP]])
    desc = 'System wide public workspace'
    workspace = apps.get_model('workspace', 'Workspace')
    workspace.objects.create(id=1, name='Public', description=desc,
                             editable=False, acl=acl)

class Migration(migrations.Migration):

    atomic = False

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Workspace',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('name', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('description', models.CharField(max_length=1000, blank=True, null=True, validators=[arcwebbase.validator.validateStringNoHtml])),
                ('editable', models.BooleanField(default=True)),
                ('acl', models.TextField(max_length=32768, blank=True, null=True, default='[]', validators=[workspace.models.validate_acl_json])),
                ('private_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.RunPython(create_public_workspace)
    ]
