from django.conf import settings
from django.db import migrations
from django.db.models import Count

def remove_duplicate_private_user_workspaces(apps, schema_editor):
    Workspaces = apps.get_model('workspace', 'workspace')

    duplicates = Workspaces.objects \
        .values('private_user') \
        .annotate(count=Count('private_user')) \
        .filter(count__gt=1)

    for workspace in duplicates:
        workspaces = Workspaces.objects.filter(private_user=workspace.get('private_user'))
        latest_workspace = workspaces.order_by('create_time').first()
        workspaces.exclude(id=latest_workspace.id).delete()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('workspace', '0003_alter_workspace_id'),
    ]

    operations = [
        migrations.RunPython(remove_duplicate_private_user_workspaces, migrations.RunPython.noop)
    ]
