# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

from tests.migrationtests_base import MigrationsTestBase
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db import IntegrityError, transaction

import django.db
import json

connection: BaseDatabaseWrapper = django.db.connection

class TestMigrations(MigrationsTestBase):

  def test_0004(self):
    self.run_migration('workspace', '0003')

    if self.connection and self.connection.vendor == 'postgresql':
      # Postgres is so strict it won't allow the duplication by defualt
      return

    User = self.project_state.apps.get_model('auth', 'User')
    user = User.objects.create()
    Workspace = self.project_state.apps.get_model('workspace', 'workspace')

    workspace = {
      'name': 'Private',
      'editable': False,
      'private_user_id': user.id,
      'description': 'description',
      'acl':json.dumps([[1, 2, 'vizapps_admin']])
    }

    Workspace.objects.create(**workspace)
    Workspace.objects.create(**workspace)
    Workspace.objects.create(**workspace)

    number_of_private_workspaces_for_vizapps_admin = Workspace.objects.filter(private_user_id=user.id).count()

    self.assertEqual(number_of_private_workspaces_for_vizapps_admin, 3)

    self.run_migration('workspace', '0004')

    number_of_private_workspaces_for_vizapps_admin = Workspace.objects.filter(private_user_id=user.id).count()

    self.assertEqual(number_of_private_workspaces_for_vizapps_admin, 1)

  def test_0005(self):
    self.run_migration('workspace', '0005')

    User = self.project_state.apps.get_model('auth', 'User')
    Workspace = self.project_state.apps.get_model('workspace', 'workspace')

    user = User.objects.create()

    workspace = {
      'name': 'Private',
      'editable': False,
      'private_user_id': user.id,
      'description': 'description',
      'acl':json.dumps([[1, 2, 'vizapps_admin']])
    }

    # https://stackoverflow.com/questions/21458387/transactionmanagementerror-you-cant-execute-queries-until-the-end-of-the-atom
    try:
      with transaction.atomic():
        Workspace.objects.create(**workspace)
        Workspace.objects.create(**workspace)
      self.fail('Duplicate workspace allowed')
    except IntegrityError:
      pass