# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import django.test
import json

from vizlib.tests import test_report_operations as util_module

class TestWorkspaceViews(django.test.TestCase):
  def get_workspaces_json(self, client: django.test.client.Client) -> list[dict]:
    response = client.get('/workspace/workspace')
    self.assertEquals(response.status_code, 200)

    # this endoint does not set the content type to application/json,
    # but to 'text/html'
    return json.loads(response.content)

  def get_workspace_json(self, client: django.test.client.Client, workspace_id: int) -> dict:
    response = client.get(f'/workspace/workspace/{workspace_id}')
    self.assertEquals(response.status_code, 200)

    # this endoint does not set the content type to application/json,
    # but to 'text/html'
    return json.loads(response.content)[0]

  def test_get(self):
    admin = util_module.make_user('admin1', is_superuser=True)
    admin_client = util_module.create_client(admin)

    workspaces = self.get_workspaces_json(admin_client)

    # there are two workspaces: public and admin's private
    self.assertEquals(len(workspaces), 2)

    # retrive workspaces by id
    for ws in workspaces:
      ws_id = ws['id']
      if ws_id is not None:
        # the privte workspace has id=None
        ws_detail = self.get_workspace_json(admin_client, ws_id)
        print(ws_detail['name'])
        self.assertEquals(ws_detail['name'], 'Public')