#!/bin/bash

#
# This file is responsible for starting Dataviz processes (webserver and reverse proxy) in CDP-Base
#

set -x
set -e

export VIZ_PORT="${DATAVIZ_PORT:-8100}"
export VIZ_WEBSERVER_PORT=${DATAVIZ_WEBSERVER_PORT}

export CDV_MODE=${CDV_MODE:-cdpbase}
export DJANGO_SETTINGS_MODULE=${DJANGO_SETTINGS_MODULE:-settings_local}
export BASEDIR="${BASEDIR:-${HOME}/.arc}"

export PYTHONHOME="${DATAVIZ_CODE_BASE}/python"
export VENV_PATH="${DATAVIZ_CODE_BASE}/venv"
export LD_LIBRARY_PATH="${DATAVIZ_CODE_BASE}/python/lib:${DATAVIZ_CODE_BASE}/sqlite/lib:${LD_LIBRARY_PATH}"
export PYTHONPATH="${DATAVIZ_CODE_BASE}/venv/lib/python3.9/site-packages/arcweb"

# needed for Caddy static resource config
export ARCWEB_PATH="${DATAVIZ_CODE_BASE}/venv/lib/python3.9/site-packages/arcweb"

# this needs to be passed by CM
export CHROMIUM_PATH=${CHROMIUM_PATH:-/usr/lib64/chromium-browser/headless_shell}

if [[ "${DATAVIZ_PROC}" == "WEBSERVER" ]]; then
  export USE_MULTIPROC=false
  export ACTIVATE_VENV=no
  exec "${DATAVIZ_CODE_BASE}/tools/arcviz/startviz.sh"
fi

if [[ "${DATAVIZ_PROC}" == "REVERSE_PROXY" ]]; then
  export ROOT_STATIC_FILE_PATH=${ARCWEB_PATH}
  export VIZ_MULTIPROC_CONF=${BASEDIR}/multiprocess

  VENV_BASE_PATH=$VENV_PATH ${DATAVIZ_CODE_BASE}/tools/arcviz/gen_multiproc_conf.sh

  exec ${DATAVIZ_CODE_BASE}/tools/arcviz/caddy -conf $VIZ_MULTIPROC_CONF/Caddyfile -log ${LOG_DIR:-${BASEDIR}}/caddy.log
fi