from string import Template
import argparse

TEMPLATE = """\
{
  "schema_version": 1,
  "name": "DATAVIZ",
  "version": "${version}",
  "setActiveSymlink": true,

  "replaces": "DATAVIZ",
  "provides": [
    "dataviz"
  ],
  "components": [
    {
      "name": "dataviz",
      "version": "${version}",
      "pkg_version": "n/a",
      "pkg_release": "n/a"
    }
  ],

  "scripts": {
    "defines": "dataviz_env.sh"
  },

  "users": {
    "dataviz":{
      "longname": "Dataviz User",
      "home": "/home/dataviz",
      "shell": "/bin/bash",
      "extra_groups": [ "dataviz" ]
    }
  },
  "groups": ["dataviz"],
  "packages": []
}
"""


def main():
  parser = argparse.ArgumentParser(
      prog='parcel_json',
      description='Generate parcel JSON')
  parser.add_argument('--output-file')
  parser.add_argument('--var', action='append')

  args = parser.parse_args()

  output_file = args.output_file
  var_list = args.var

  substitutions = dict[str, str]()
  if var_list:
    for var in var_list:
      parts = var.split('=', maxsplit=1)
      substitutions[parts[0]] = parts[1]

  t = Template(TEMPLATE)
  result = t.substitute(**substitutions)

  if output_file:
    with open(output_file, mode="tw") as f:
      f.write(result)
  else:
    print(result)


if __name__ == '__main__':
  main()
