# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
"""
Django settings local development.
Put this file in arcviz/arcweb/arcwebbase
and
export DJANGO_SETTINGS_MODULE=arcweb.settings_direct
"""

from .settings import *
import os

DEBUG = False
if "ARCDEBUG" in os.environ:
  DEBUG = True
TEMPLATE_DEBUG = DEBUG
RUN_FROM_SOURCE_DEBUG = True
ALLOWED_HOSTS = ['*']

from appdirs import user_data_dir
appname = "Arcadia Instant"
appauthor = "Arcadia Data"

TEMPLATES[0]['OPTIONS']['libraries']={'arctags': 'apps.templatetags.arctags'}
#Use single user mode for now. Should not show up for other users
#Eg: user_data_dir is '/Users/trentm/Library/Application Support/SuperApp'
#Eg: site data dir is '/Library/Application Support/SuperApp'
userdatadir = user_data_dir(appname, appauthor)
ARCVIZ_LOGFILE = os.path.join(userdatadir, 'arcviz.log')
from os.path import abspath, join
db_path = abspath(join(userdatadir, "addirect.db"))
MEDIA_ROOT = abspath(join(userdatadir, "media/"))
CACHES['default']['LOCATION'] =  abspath(join(userdatadir, "cache"))
CACHES['default']['SHARDS'] = 1
DATABASES = {
  'default': {
    # Add 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
    'ENGINE': 'django.db.backends.sqlite3',
    # Or path to database file if using sqlite3.
    'NAME': db_path,
    # The following settings are not used with sqlite3:
    'USER': '',
    'PASSWORD': '',
    # Empty for localhost through domain sockets or '127.0.0.1' for
    # localhost through TCP.
    'HOST': '',
    # Set to empty string for default.
    'PORT': '',
    }
}

phantom_path = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..'))
phantom_path_js = phantom_path + "/lib/phantomjs-2.1.1/bin/phantomjs"

if os.path.isfile(phantom_path_js):
    PHANTOMJS_PATH = phantom_path_js.replace('Arcadia Instant','Arcadia\ Instant')
