# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from settings import *

import warnings
import django.core.cache

warnings.simplefilter("ignore", django.core.cache.CacheKeyWarning)

DATABASES['default']['NAME'] = "/home/ubuntu/larry_webserver_data/arcviz.sqlite3"

CACHES = {
  'default': {
    # To disable caching
    # 'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
    'LOCATION': '/home/ubuntu/django_cache',
    'TIMEOUT': 3600,
    'OPTIONS': {
      'MAX_ENTRIES': 10000
    }
  }
}

FORCE_SCRIPT_NAME = "/arc"

LOGGING = {
  'version': 1,
  'disable_existing_loggers': False,
  'formatters': {
    'verbose': {
      'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
    },
    'simple': {
      'format': '%(levelname)s %(message)s'
    },
    'arcfmt': {
      'format': '%(asctime)s %(levelname)s %(process)d %(message)s'
    },
    },
  'filters': {
    'require_debug_false': {
      '()': 'django.utils.log.RequireDebugFalse'
    }
  },
  'handlers': {
    #        'mail_admins': {
    #            'level': 'ERROR',
    #            'filters': ['require_debug_false'],
    #            'class': 'django.utils.log.AdminEmailHandler'
    #        },
    'console':{
      'level': 'DEBUG',
      'class': 'logging.StreamHandler',
      'formatter': 'arcfmt'
    },
    'file':{
      'level': 'DEBUG',
      'class': 'logging.FileHandler',
      'filename': '/tmp/arcweb.log',
      'formatter': 'arcfmt'
    },
    'rotatingfile':{
      'level': 'DEBUG',
      'class': 'logging.handlers.RotatingFileHandler',
      'filename': os.path.join("/tmp/arcviz.log"),
      'maxBytes': 1000000,
      'backupCount': 5,
      'formatter': 'arcfmt'
    },
    },
  'loggers': {
    #        'django.request': {
    #            'handlers': ['mail_admins'],
    #            'level': 'ERROR',
    #            'propagate': True,
    #        },
    'arcweb': {
      'handlers': ['rotatingfile'],
      'level': 'DEBUG',
      'propagate': True,
      }
  }
}
