# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from arcwebbase.settings import *
import os

RUN_FROM_SOURCE_DEBUG = True
DEBUG = True
AUDIT_LOGFILE = f'{BASEDIR}/querylog'
# LOG_ORM_QUERIES = True

#### DEFAULT METASTORE CONFIGURATION FOR SQLITE IN DEV TESTING ####
metadata_store = os.getenv('METADATA_STORE')
# If the environment variable is set, let's check it for 'mysql', 'postgresql', or 'oracle' values.
#   If a valid external database backend isn't provided, continue with using SQLite database.
if metadata_store not in ('mysql', 'postgresql', 'oracle'):
    DATABASES['default']['NAME'] = f'{BASEDIR}/db.local'

DATABASES['usage']['NAME'] = f'{BASEDIR}/usage.db'

ALLOWED_HOSTS = ['*']

LOGGING['loggers']['arcweb']['handlers'] = ['console']

ENABLE_DATACONN_ARCENGINE = True

ENABLE_VISUAL_ACTION = True
ENABLE_WEBHOOK = True

IS_CYPRESS = os.getenv('IS_CYPRESS', '').lower() == 'true'
ENABLE_DEBUG_TOOLBAR = os.getenv('ENABLE_DEBUG_TOOLBAR', '').lower() == 'true'

if IS_CYPRESS:
    INCLUDE_CANTEEN = True
    ADMIN_API_DEMO_LIST = ['*']
    ADMIN_API_URL_LIST  = ['*']
    ENABLE_API_KEYS = True

if ENABLE_DEBUG_TOOLBAR:
    # Django Debug Toolbar specific settings
    INTERNAL_IPS = [
        # ...
        "127.0.0.1",
        # ...
    ]
    target_middleware = 'django.middleware.common.CommonMiddleware'

    if target_middleware in MIDDLEWARE:
        target_index = MIDDLEWARE.index(target_middleware)
        MIDDLEWARE.insert(target_index + 1, 'debug_toolbar.middleware.DebugToolbarMiddleware')

    INSTALLED_APPS = list(INSTALLED_APPS)
    INSTALLED_APPS.append('debug_toolbar')
    INSTALLED_APPS = tuple(INSTALLED_APPS)

    DEBUG_TOOLBAR_CONFIG = {
        'SHOW_COLLAPSED': True,
    }

ENABLE_QUNIT = os.getenv('ENABLE_QUNIT', '').lower() == 'true'
if ENABLE_QUNIT:
    INSTALLED_APPS += tuple(['qunit'])

DATASET_AUTOCREATE_DATACONNECTION = "samples"

ENABLE_ADVANCED_SETTINGS = True

# To change the value of existent feature flippers, use the following syntax:
# FEATURE_FLIPPERS.append({'id': 'example_flipper',  'value': False})

WORKQ_DELAY_SECONDS = {
#  'sql_19': 5
}
