import { useMutation } from '@tanstack/react-query';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

interface CreateAliasParams {
  dashboardId: number;
  parameters: string;
  slug: string;
  type: 'dashboard' | (string & {});
  isCreatedInEmail?: boolean;
}

export const createAlias = async ({
  dashboardId,
  parameters,
  slug,
  type,
  isCreatedInEmail = true,
}: CreateAliasParams) => {
  await vizHttp.post({
    url: arc.frontend.globals.urls.urlShortenerUrl,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: false,
    data: {
      dashboardId,
      isCreatedInEmail,
      parameters,
      slug,
      type,
    },
  });
};

export const useCreateAliasMutation = (params: CreateAliasParams) =>
  useMutation({ mutationFn: () => createAlias(params) });
