import { useMutation } from '@tanstack/react-query';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

interface EditAliasParams {
  updateType: 'email' | 'managing' | (string & {});
  slug: string;
  newAlias?: string;
  urlData?: string;
}

export const editAlias = async ({
  slug,
  updateType,
  newAlias,
  urlData,
}: EditAliasParams) => {
  const queryParams = new URLSearchParams();

  if (slug) {
    queryParams.append('slug', slug);
  }

  if (updateType) {
    queryParams.append('updateType', updateType);
  }

  if (updateType === 'managing') {
    if (newAlias) {
      queryParams.append('newAlias', newAlias);
    }
    if (urlData) {
      queryParams.append('url_data', urlData);
    }
  }

  await vizHttp.patch({
    url: `${
      arc.frontend.globals.urls.urlShortenerUrl
    }?${queryParams.toString()}`,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: false,
  });
};

export const useEditAlias = (params: EditAliasParams) =>
  useMutation({ mutationFn: () => editAlias(params) });
