import { useQuery, UseQueryOptions } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

type BackendAlias = {
  is_created_in_email: boolean;
  parameters: Record<string, string>;
  parametersLength: boolean;
  slug: string;
  times_followed: number;
  last_viewed: string;
  dashboardId: number;
  alias_count: number;
  create_time: string;
  update_time: string;
  creator_name: string;
  visual_type: string;
  dashboard_name: string;
};

export const getAliasesByDashboardId = async ({
  dashboardId,
}: {
  dashboardId?: number;
}) => {
  const { data } = await vizHttp.get<BackendAlias[]>({
    url: `${arc.frontend.globals.urls.urlShortenerUrl}${
      dashboardId ? `?dashboardId=${dashboardId}` : ''
    }`,
    isOldEndpoint: false,
    params: {},
  });

  return data.map((item) => ({
    slug: item.slug,
    parameters: item.parameters,
  }));
};

type TQueryFnData = Awaited<ReturnType<typeof getAliasesByDashboardId>>;

export const useGetAliasesQuery = <TError = unknown, TData = TQueryFnData>(
  dashboardId: number,
  options?: UseQueryOptions<TQueryFnData, TError, TData>
) => {
  const query = useQuery({
    queryKey: [QueryKeyBase.Aliases],
    staleTime: 300000, // 5 minutes
    queryFn: () => getAliasesByDashboardId({ dashboardId }),
    ...options,
  });

  return query;
};
