import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const aliasesQueryToken = 'aliases';

type Alias = {
  slug: string;
  parameters: {
    sheet?: string;
    initialapp?: string;
  };
  times_followed: number;
  last_viewed: string;
  dashboardId: number;
  alias_count: number;
  create_time: string;
  update_time: string;
  creator_name: string;
  visual_type: string;
  dashboard_name: string;
  is_created_in_email: boolean;
};

export type transformAlias = Alias & {
  path: string;
  fullUrl: string;
  isApp: boolean;
  isDashboard: boolean;
  isDisabled: boolean;
};

type Aliases = {
  aliases: Alias[];
};

export const getAllAliases = async (): Promise<transformAlias[]> => {
  const { data } = await vizHttp.get<Aliases>({
    url: arc.frontend.globals.urls.urlShortenerUrl,
    isOldEndpoint: false,
  });

  const aliases = data?.aliases ?? [];

  const transformedAliases = aliases.map((alias) => {
    const path = `${window.location.origin}${arc.frontend.globals.urls.urlShortenerUrl}`;
    const url = `${path}${alias.slug}`;

    return {
      ...alias,
      path: path,
      fullUrl: url,
      isApp: alias.visual_type === 'apps',
      isDashboard: alias.visual_type === 'dashboard',
      isDisabled: alias.alias_count === 0,
    };
  });

  return transformedAliases;
};
