import { Dataset } from 'arc/types/dataset.type';
import { getGlobalArc } from 'utils/get-global-arc';
const arc = getGlobalArc();
export const createDatasetFromTable = (
  connectionId: number,
  connectionType: string,
  title: string,
  selectedDatabase: string,
  selectedTable: string
): Promise<Dataset> => {
  return new Promise((resolve, reject) => {
    const datasetToSave = arc.datasets
      .Dataset()
      .dataconnection_id(connectionId)
      .dataconnection_type(connectionType)
      .dataset_name(title)
      .dataset_type('singletable')
      .dataset_info('')
      .dataset_detail(`${selectedDatabase}.${selectedTable}`);

    const errorCb = (error?: Error | { response: string }) => reject(error);
    const doneCb = (errorMsg: string | Error) => {
      if (errorMsg) {
        console.error(errorMsg);
      } else {
        resolve(datasetToSave);
      }
    };

    // Legacy
    datasetToSave.fetch_and_add_table(
      datasetToSave.dataset_detail(),
      () => {
        datasetToSave.save(doneCb);
      },
      errorCb
    );
  });
};

export const createDatasetFromSql = (
  connectionId: number,
  connectionType: string,
  title: string,
  sql: string
): Promise<Dataset> => {
  let create = arc.datasets.createFromSql();
  return new Promise((resolve, reject) => {
    create
      .dataconnection_id(connectionId)
      .dataconnection_type(connectionType)
      .tgt_dataset_name(title)
      .sql(sql)
      .successCb((dataset: Dataset) => {
        resolve(dataset);
      })
      .errorCb((error) => {
        reject(error);
      });
    create.create();
  });
};
