import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';
import {
  DataConnectionInfo,
  DataConnectionType,
} from 'types/data-connection.type';

const arc = getGlobalArc();

export const validateOrMaybecreateOrUpdateConnectionQueryToken =
  'validate-or-maybe-create-dataconnection';

type Response = {
  status: string;
  id: number;
  name: string;
};

export const createOrUpdateConnection = async ({
  dataconnectionData,
  shouldValidate,
}: {
  dataconnectionData: {
    id?: number;
    name: string;

    type: DataConnectionType;
    typeName: string;

    info: DataConnectionInfo;

    cml?: {
      id: number | null;
      isAvailable: boolean;
    };

    password?: string;
  };
  shouldValidate: string;
}) => {
  const url = dataconnectionData.id
    ? `${arc.frontend.globals.urls.dataconnectionUrl}/${dataconnectionData.id}`
    : arc.frontend.globals.urls.dataconnectionUrl;

  const method = dataconnectionData.id ? 'put' : 'post';

  const payload: {
    csrfmiddlewaretoken: string;
    dataconnection_type: string;
    dataconnection_name: string;
    dataconnection_info: string;
    do_validate: string;
    dataconnection_password?: string;
  } = {
    csrfmiddlewaretoken: arc.frontend.globals.authentication.csrfToken,
    dataconnection_type: dataconnectionData.type,
    dataconnection_name: dataconnectionData.name,
    dataconnection_info: JSON.stringify(dataconnectionData.info),
    do_validate: shouldValidate,
  };

  // Add dataconnection_password only if it's non-empty
  // hive & hyve doesn't like empty string!
  if (dataconnectionData.password) {
    payload.dataconnection_password = dataconnectionData.password;
  }

  const { data } = await vizHttp[method]<Response>({
    isOldEndpoint: true,
    url,
    data: payload,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
  });

  return data;
};
