import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

type CreateReportResponse = {
  report_id: number;
  new_visual_ids: number[];
};

type CreateReportParams = {
  datasetId: number;
  workspaceId?: string;
  dashboardOnly?: boolean;
  visualType?: 'table' | 'extension' | (string & {});
  title?: string;
  createLimit?: number;
  isThreshold?: boolean;
};

export const createReport = async ({
  datasetId,
  ...params
}: CreateReportParams) => {
  const response = await vizHttp.post<CreateReportResponse>({
    url: `${arc.frontend.globals.urls.reportCreateUrl}/${datasetId}`,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: true,
    data: {
      workspace_id: params.workspaceId,
      dashboard_only: params.dashboardOnly,
      visual_type: params.visualType ?? 'table',
      report_title: params.title,
      create_limit: params.createLimit,
      is_trigger: params.isThreshold,
    },
  });

  return {
    dashboardId: response.data.report_id,
    visualIds: response.data.new_visual_ids,
  };
};
