import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';
import { Extract } from 'types/extract.type';

const arc = getGlobalArc();

type BackendTableField = {
  alias: string;
  expr: string;
  type: string;
};

type BackendExtract = {
  extract_type: string;
  id: number;
  srcDatasetId: number;
  srcDatasetName: string;
  tgtDataconnectionId: number;
  tgtDataconnectionName: string;
  tgtDbname: string;
  tgtTablename: string;
  extract_config: {
    username: string;
    rewrite_properties: {
      rewrite_status: string;
    };
    notification_email?: {
      email_fail: 'true' | 'false';
      email_success: 'true' | 'false';
      email_subject: string;
      to_addresses: string[];
    };
    source: {
      aggData: BackendTableField[];
      dimData: BackendTableField[];
    };
  };
};

const parseExtract = (backendExtract: BackendExtract): Extract => {
  return {
    id: backendExtract.id,
    type: backendExtract.extract_type,
    srcDatasetId: backendExtract.srcDatasetId,
    srcDatasetName: backendExtract.srcDatasetName,
    tgtDataConnectionId: backendExtract.tgtDataconnectionId,
    tgtDataConnectionName: backendExtract.tgtDataconnectionName,
    tgtDbName: backendExtract.tgtDbname,
    tgtTableName: backendExtract.tgtTablename,
    username: backendExtract.extract_config.username,
    rewriteStatus:
      backendExtract.extract_config.rewrite_properties.rewrite_status,
    emailFail:
      backendExtract.extract_config.notification_email?.email_fail === 'true',
    emailSuccess:
      backendExtract.extract_config.notification_email?.email_success ===
      'true',
    emailSubject:
      backendExtract.extract_config.notification_email?.email_subject,
    emailTo: backendExtract.extract_config.notification_email?.to_addresses,
    dimensions: backendExtract.extract_config.source.dimData,
    measures: backendExtract.extract_config.source.aggData,
  };
};

export const getExtractQueryToken = 'getExtract';

export const getExtract = async (id: number): Promise<Extract | undefined> => {
  const { data } = await vizHttp.get<BackendExtract[]>({
    url: `${arc.frontend.globals.urls.extractUrl}/${id}`,
    isOldEndpoint: false,
  });

  return data[0] ? parseExtract(data[0]) : undefined;
};
