import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';

import type { Dataset, SummaryDataset } from 'types/dataset.type';

type BackendDataset = {
  created: string;
  created_by: string;
  dc_id: number;
  dc_name: string;
  dc_type: string;
  dc_type_display: string;
  description: string;
  detail: string;
  detail_display: string;
  extractSource: boolean;
  extractTarget: boolean;
  id: number;
  info: string;
  lvcolumns: string;
  lvname: string;
  name: string;
  perms: string[];
  settings: string;
  type: string;
  updated: string;
  updatedLV: boolean;
  updated_by: string;
};

type BackendSummaryDataset = {
  id: number;
  name: string;
  dc_id: number;
  dc_name: string;
  dc_type: string;
  created_by: string;
  type: string;
};

const arc = getGlobalArc();

export const datasetsQueryToken = 'datasets';

const transformBackendDatasetToDataset = (
  dataset: BackendDataset
): Dataset => ({
  id: dataset.id,
  info: dataset.info,
  description: dataset.description,
  detail: dataset.detail,
  displayDetail: dataset.detail_display,
  name: dataset.name,
  perms: dataset.perms,
  settings: dataset.settings,
  type: dataset.type,
  dc_id: dataset.dc_id,
  dc_name: dataset.dc_name,
  created: {
    date: dataset.created,
    by: dataset.created_by,
  },
  updated: {
    date: dataset.updated,
    by: dataset.updated_by,
    lb: dataset.updatedLV,
  },
  dataConnection: {
    id: dataset.id,
    name: dataset.name,
    type: dataset.type,
    displayType: dataset.dc_type_display,
  },
  extract: {
    isSource: dataset.extractSource,
    isTarget: dataset.extractTarget,
  },
  lv: {
    columns: dataset.lvcolumns,
    name: dataset.name,
  },
});

const transformBackendDatasetToSummaryDataset = (
  dataset: BackendSummaryDataset
): SummaryDataset => ({
  id: dataset.id,
  name: dataset.name,
  dc_id: dataset.dc_id,
  dc_name: dataset.dc_name,
  type: dataset.type,
  created: {
    by: dataset.created_by,
  },
});

export const getAllDatasets = async ({
  isSummary = false,
}: {
  isSummary?: boolean;
}): Promise<SummaryDataset[] | Dataset[]> => {
  const params: Record<string, string> = {};

  if (isSummary) {
    params.summary = 'true';
  }

  const { data } = await vizHttp.get<
    BackendDataset[] | BackendSummaryDataset[]
  >({
    url: arc.frontend.globals.urls.datasetUrl,
    isOldEndpoint: true,
    params,
  });

  if (isSummary) {
    const summaryResult = (data as BackendSummaryDataset[]).map(
      transformBackendDatasetToSummaryDataset
    );

    return summaryResult;
  } else {
    const allResult = (data as BackendDataset[]).map(
      transformBackendDatasetToDataset
    );

    return allResult;
  }
};
