import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';
import type { SqlQuery } from 'types/sql-query.type';

type BackendSqlQuery = {
  id: number;

  dataconnectionId: number;
  dataconnectionName: string;
  dataconnectionType: string;

  favorite: boolean;

  query: string;
  queryTime: number;

  tableNames: string[];
};

const arc = getGlobalArc();

const transformBackendSqlQueryToSqlQuery = ({
  id,
  dataconnectionId,
  dataconnectionName,
  dataconnectionType,
  favorite,
  query,
  tableNames,
}: BackendSqlQuery): SqlQuery => ({
  id,
  dataconnection: {
    id: dataconnectionId,
    name: dataconnectionName,
    type: dataconnectionType,
  },
  isFavorite: favorite,
  query,
  tableNames,
});

export const getAllSqlQueries = async (
  { limit, shouldShortByFavorite } = {
    limit: 30,
    shouldShortByFavorite: false,
  }
): Promise<SqlQuery[]> => {
  try {
    const { data } = await vizHttp.get<BackendSqlQuery[]>({
      url: arc.frontend.globals.urls.savedSqlUrl,
      params: {
        get_type: 'all',
        limit: String(limit),
        should_sort: String(shouldShortByFavorite),
      },
      isOldEndpoint: true,
    });

    return data.map(transformBackendSqlQueryToSqlQuery);
  } catch (error) {
    console.error(error);

    return [];
  }
};
