import { CompletionSuccessResponse, ErrorResponse } from 'types/api.type';
import { OverflowPolicy } from 'types/chatbot-message.type';

type Response = CompletionSuccessResponse | ErrorResponse;

const handleResponse = (response: Response) => {
  if ('error' in response) {
    throw new Error(response.error);
  } else {
    return response.body.choices[0].message.content;
  }
};

export const getCompletion = async (
  sendHttpReq: (
    proxyReq: any,
    chunkCb: (chunk: string) => void
  ) => Promise<{
    request: {
      async: boolean;
      aiQuery: boolean;
      modelType: 'completion';
      profile: string;
      body: object;
    };
    response: Response;
  }>,
  chunkCb: (chunk: string) => void,
  sourceInfo: {
    filteredRows: string[][];
    aliases: string[];
    contextPrompt: string;
  },
  previousMessages: { role: string; content: string }[],
  question: string,
  settings: {
    completionProfile: string;
    paramSelection: string;
    questionPromptPrefix: string;
    completionMaxTokens: string;
    overflowPolicy: OverflowPolicy;
  }
) => {
  const reqBody = {
    upstreamReqBody: {
      input: [
        // Previous conversation
        ...previousMessages,
        // User question
        {
          role: 'user',
          content: settings.questionPromptPrefix
            ? settings.questionPromptPrefix.replace('<<question>>', question)
            : question,
        },
      ],
    },
    config: {
      completionMaxTokens: settings.completionMaxTokens,
      overflowPolicy: settings.overflowPolicy,
      paramSelection: settings.paramSelection,
      sourceInfo,
    },
  };

  const payload = {
    async: true,
    aiQuery: true,
    modelType: 'completion',
    profile: settings.completionProfile,
    body: reqBody,
  };

  const proxyResponse = await sendHttpReq(payload, chunkCb);

  return handleResponse(proxyResponse.response);
};
