import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

const asyncSqlRunnerWithParams = function (config: {
  dataconnection_id: number | string;
  dsreq: any;
  hideAlertsOnSuccess?: boolean;
}) {
  return new Promise((resolve, reject) => {
    const configWithCallback = {
      ...config,
      cache: false,
      // Error handled by the caller of this function, not the arc.report.sql_runner.runParam's automatic error handling callBack
      displayErrorMessage: false,
      cb: (errorMsg: string, response: unknown) => {
        if (errorMsg) {
          reject(errorMsg);
        } else {
          resolve(response);
        }
      },
    };

    return arc.report.sql_runner.runParams(configWithCallback);
  });
};

export const getDatatables = async (
  databaseName?: string,
  connectionId?: number | string
): Promise<string[]> => {
  const dataconnection_id =
    typeof connectionId == 'string' ? parseInt(connectionId) : connectionId;
  if (!dataconnection_id || dataconnection_id < 0) {
    throw new Error(`${`Connection id is not valid:${dataconnection_id}!`}`);
  }

  const requestBuilder = arc.requestBuilder();
  requestBuilder.requestType('.tables');
  requestBuilder.dbname(databaseName ?? '');
  const tableList = await asyncSqlRunnerWithParams({
    dataconnection_id,
    dsreq: requestBuilder.dsreq(),
    hideAlertsOnSuccess: true,
  });

  if (
    tableList &&
    typeof tableList === 'object' &&
    'rows' in tableList &&
    Array.isArray(tableList.rows)
  ) {
    return tableList.rows.flat();
  }

  throw new Error('Table list is invalid');
};

export const getDatabases = async (
  connectionId?: number | string
): Promise<string[]> => {
  const requestBuilder = arc.requestBuilder();
  requestBuilder.requestType('.databases');

  const dataconnection_id =
    typeof connectionId == 'string' ? parseInt(connectionId) : connectionId;
  if (!dataconnection_id) {
    throw new Error(`Connection id is invalid: ${dataconnection_id} `);
  }

  const databaseList = await asyncSqlRunnerWithParams({
    dataconnection_id,
    dsreq: requestBuilder.dsreq(),
    hideAlertsOnSuccess: true,
  });

  if (
    databaseList &&
    typeof databaseList === 'object' &&
    'rows' in databaseList &&
    Array.isArray(databaseList.rows)
  ) {
    // Impala returns two columns (first is the database name, second is a comment column)
    // Other connections return a single column
    return databaseList.rows.map((row) => (Array.isArray(row) ? row[0] : row));
  }

  throw new Error('Database list is invalid');
};
