import { getEmbeddings } from './get-embeddings';

afterEach(() => {
  jest.clearAllMocks();
});

describe('getEmbeddings', () => {
  const question = 'What is the capital of France?';
  const embeddingProfile = 'openai';
  const embeddingMaxTokens = '512';

  it('should call sendHttpReq with correct payload for open_ai engine', async () => {
    const sendHttpReq = jest.fn().mockResolvedValue({
      response: {
        body: {
          data: [
            {
              embedding: [0.1, 0.2, 0.3],
            },
          ],
        },
      },
    });

    await getEmbeddings(
      sendHttpReq,
      question,
      embeddingProfile,
      embeddingMaxTokens
    );

    expect(sendHttpReq).toHaveBeenCalledWith({
      async: true,
      aiQuery: true,
      modelType: 'embeddings',
      profile: 'openai',
      body: {
        upstreamReqBody: {
          input: question,
        },
        config: {
          embeddingMaxTokens,
        },
      },
    });
  });

  it('should return embedding array on successful API call', async () => {
    const expectedEmbedding = [0.1, 0.2, 0.3];
    const sendHttpReq = jest.fn().mockResolvedValue({
      response: {
        body: {
          data: [
            {
              embedding: expectedEmbedding,
            },
          ],
        },
      },
    });

    const embedding = await getEmbeddings(
      sendHttpReq,
      question,
      embeddingProfile,
      embeddingMaxTokens
    );

    expect(embedding).toEqual(expectedEmbedding);
  });

  it('should throw an error if API call fails', async () => {
    const sendHttpReq = jest
      .fn()
      .mockRejectedValue(new Error('API call failed'));

    await expect(
      getEmbeddings(sendHttpReq, question, embeddingProfile, embeddingMaxTokens)
    ).rejects.toThrow('API call failed');
  });
});
