import { EmbeddingSuccessResponse, ErrorResponse } from 'types/api.type';

type Response = EmbeddingSuccessResponse | ErrorResponse;

const handleResponse = (response: Response) => {
  if ('error' in response) {
    throw new Error(response.error);
  } else {
    return response.body.data[0].embedding;
  }
};

export const getEmbeddings = async (
  sendHttpReq: (proxyReq: any) => Promise<{
    request: {
      async: boolean;
      aiQuery: boolean;
      modelType: 'embeddings';
      profile: string;
      body: object;
    };
    response: Response;
  }>,
  question: string,
  embeddingProfile: string,
  embeddingMaxTokens: string
): Promise<number[]> => {
  const reqBody = {
    upstreamReqBody: {
      input: question,
    },
    config: {
      embeddingMaxTokens,
    },
  };

  const payload = {
    async: true,
    aiQuery: true,
    modelType: 'embeddings',
    profile: embeddingProfile,
    body: reqBody,
  };

  const proxyResponse = await sendHttpReq(payload);

  return handleResponse(proxyResponse.response);
};
