import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';
import { BackendReport } from './get-reports';

const arc = getGlobalArc();

export type PossibleDeletedReport = Omit<
  BackendReport,
  'shared_visual_dashboards'
> & {
  shared_visual_dashboards: number[] | undefined;
};

export const possibleDeletedReportsQueryToken = 'possible-deleted-reports';

export const getPossibleDeletedReports = async ({
  id,
}: {
  id: number;
}): Promise<PossibleDeletedReport[]> => {
  const { data } = await vizHttp.get<BackendReport[]>({
    url: arc.frontend.globals.urls.getPossibleDeletedReportsUrl,
    params: {
      dataset_id: id.toString(),
    },
    isOldEndpoint: true,
  });

  const reportsWithParsedSharedVisualDashboards = data.map((report) => {
    let parsedSharedVisualDashboards: number[] | undefined;

    if (typeof report.shared_visual_dashboards === 'string') {
      parsedSharedVisualDashboards = JSON.parse(
        report.shared_visual_dashboards
      );
    }

    return {
      ...report,
      shared_visual_dashboards: parsedSharedVisualDashboards,
    };
  });

  return reportsWithParsedSharedVisualDashboards;
};
