import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export type QueryLog = {
  action: number;
  action_id: string;
  action_name: string;
  app: number;
  appgroup: string;
  appgroup_name: string;
  cache: boolean;
  cache_check: boolean;
  component_id: number;
  component_type: number;
  component_type_name: string;
  dashboard_name: string;
  dataconnection: number;
  dataconnection_fbid: string;
  dataconnection_name: string;
  dataconnection_type: string;
  dataset: number;
  dataset_name: string;
  error: boolean;
  error_detail: string;
  ip: string;
  process_id: number;
  query: string;
  query_state: number;
  query_state_name: string;
  rewrite: string;
  rewrite_target: string;
  runtime: string;
  sync: boolean;
  ts: string;
  user_id: number;
  username: string;
  visual: number | string;
  visual_name: string;
  visual_type: string;
};

export const getQueryLogsQueryToken = 'get-query-logs';

export const getQueryLogs = async (
  querycount: string,
  dashboardId?: number | null,
  search?: string,
  isEditsonly?: boolean,
  isVisualsOnly?: boolean
) => {
  let url = `${arc.frontend.globals.urls.getQueryLogsUrl}/${querycount}`;

  const queryParams: string[] = [];
  if (search) {
    queryParams.push(`search=${encodeURIComponent(search)}`);
  }
  if (isEditsonly) {
    queryParams.push(`edits_only=true`);
  }
  if (isVisualsOnly) {
    queryParams.push(`visuals_only=true`);
  }
  if (dashboardId) {
    queryParams.push(`app_id=${dashboardId}`);
  }

  if (queryParams.length > 0) {
    url += `?${queryParams.join('&')}`;
  }

  const { data } = await vizHttp.get<QueryLog[]>({
    url: url,
    isOldEndpoint: true,
  });
  return data;
};
