import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const reportsQueryToken = 'reports';

export type BackendReport = {
  id: number;
  name: string;
  description: string;
  type: string;
  created: string;
  report_tags: string;
  created_by: string;
  updated: string;
  updated_by: string;
  views_count: number;
  descendants_count: string | number;
  is_clone: boolean;
  in_userlist?: boolean;
  mode: string | null;
  dataset_id: number;
  dataset_name: string;
  dataconn_name: string;
  dataset_detail: string;
  dataset_type: string;
  visuals: number[];
  pickers: number[];
  report_type: string;
  edit_url: string;
  shared_visual_dashboards?: string;
  title: string;
  dashboard_id?: string;
  url: string;
  img: string;
  baseImg: string;
  workspace_id: number;
  workspace_name: string;
  workspace_perm: string[];
  dataset_perm: string[];
  perm: string[];
  // available when detailed report is requested
  data?: string;
};

export const getReports = async ({
  reportType,
  isDetailed,
  shouldGetCloneInfo,
  workspaceId,
  reportIds,
}: {
  reportType?: 'dashboard';
  isDetailed?: boolean;
  shouldGetCloneInfo?: boolean;
  workspaceId?: number;
  reportIds?: (string | number)[];
}): Promise<BackendReport[]> => {
  const params: { [key: string]: string } = {};

  if (reportType) {
    params.report_type = reportType;
  }

  if (isDetailed) {
    params.detail = 'true';
  }

  if (shouldGetCloneInfo) {
    params.getcloneinfo = 'true';
  }

  if (workspaceId) {
    params.workspace_id = workspaceId.toString();
  }

  if (reportIds) {
    params.report_ids = JSON.stringify(reportIds);
  }

  const { data } = await vizHttp.get<BackendReport[]>({
    url: arc.frontend.globals.urls.reportUrl,
    isOldEndpoint: true,
    params,
  });

  return data;
};
