import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';
import { useMutationWithNotification } from 'hooks/use-mutation-with-notification';
import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';

const arc = getGlobalArc();

export const cancelJobRun = async ({
  jobId,
  logId,
}: {
  jobId: number;
  logId: number;
}) => {
  const { data } = await vizHttp.post({
    url: `${arc.frontend.globals.urls.jobCancelUrl}/${jobId}/${logId}`,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: false,
  });

  return data;
};

export const useCancelJobRunMutation = () => {
  const queryClient = useQueryClient();
  const mutation = useMutationWithNotification(
    {
      mutationFn: cancelJobRun,
      onSuccess: (_, { jobId }) => {
        queryClient.invalidateQueries([QueryKeyBase.JobLogs, jobId]);
      },
    },
    {
      successMessage: 'Cancelled run',
      errorMessage: 'Failed to cancel run',
    }
  );

  return mutation;
};
