import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';
import {
  JobThresholdTriggerCondition,
  JobThresholdTriggerType,
} from 'types/job.type';

const arc = getGlobalArc();

type EditJobParams = {
  name?: string;
  scheduleGroupId?: number;
  scheduleGroup?: {
    name: string;
    interval: string;
  } | null;
  emailTarget: {
    subject: string;
    message?: string;
    to: string[];
    cc?: string[];
    errorTo?: string[];
    replyTo?: string;
    templateId?: number;
    urlParams?: string;
  };
  thresholdTarget?: {
    triggerId: number;
    triggerType: JobThresholdTriggerType;
    triggerConditions: JobThresholdTriggerCondition[];
    minimumTimeInterval: number;
  } | null;
};

export const editJob = async (id: number, params: EditJobParams) => {
  await vizHttp.post({
    url: `${arc.frontend.globals.urls.jobEditUrl}/${id}`,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: true,
    data: {
      emailFormData: JSON.stringify({
        job_name: params.name,
        schedule_group: params.scheduleGroupId,
        schedule_group_name: params.scheduleGroup?.name,
        run_interval: params.scheduleGroup?.interval,

        // Email target params
        email_subject: params.emailTarget.subject,
        message: params.emailTarget.message,
        email_template: params.emailTarget.templateId,
        to_addresses: params.emailTarget.to,
        cc_addresses: params.emailTarget.cc,
        error_to_addresses: params.emailTarget.errorTo,
        reply_to_address: params.emailTarget.replyTo,
        url_params: params.emailTarget.urlParams,

        // Threshold target params
        trigger_id: params.thresholdTarget?.triggerId,
        trigger_type: params.thresholdTarget?.triggerType,
        trigger_conditions: params.thresholdTarget?.triggerConditions.map(
          (condition, index) => ({
            [index]: {
              'alert-equality': condition.equality,
              'alert-trigger-value': condition.value,
            },
          })
        ),
        minimum_time_interval: params.thresholdTarget?.minimumTimeInterval,
      }),
    },
  });
};
