import { getGlobalArc } from 'utils/get-global-arc';
import { vizHttp } from 'services/viz-http';
import { EmailTemplate } from 'types/email-template.type';

export const getEmailTemplatesQueryToken = 'getEmailTemplates';

const arc = getGlobalArc();

type BackendEmailTemplate = {
  id: number;
  template_name: string;
  template_description: string;
  template_default_message: string;
};

const parseEmailTemplates = (
  backendEmailTemplates: BackendEmailTemplate[]
): EmailTemplate[] => {
  return backendEmailTemplates.map((template) => ({
    id: template.id,
    name: template.template_name,
    description: template.template_description,
    defaultMessage: template.template_default_message,
  }));
};

export const getEmailTemplates = async (): Promise<EmailTemplate[]> => {
  const { data } = await vizHttp.get<{
    email_templates: BackendEmailTemplate[];
  }>({
    url: arc.frontend.globals.urls.emailTemplatesUrl,
    isOldEndpoint: false,
  });

  return parseEmailTemplates(data.email_templates);
};
