import { useQuery } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { vizHttp } from 'services/viz-http';
import { Job } from 'types/job.type';
import { getGlobalArc } from 'utils/get-global-arc';
import { BackendJob, parseJob } from './parse-job';

const arc = getGlobalArc();

export const getJob = async (id: number): Promise<Job> => {
  const { data } = await vizHttp.get<BackendJob>({
    url: arc.frontend.globals.urls.jobGetJobUrl.replace(
      'schedule_id',
      String(id)
    ),
    isOldEndpoint: false,
  });

  return parseJob(data);
};

export const useGetJobQuery = (id: number) => {
  const query = useQuery({
    queryKey: [QueryKeyBase.Jobs],
    staleTime: 300000, // 5 minutes
    queryFn: () => getJob(id),
  });

  return query;
};
