import { useQuery } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { vizHttp } from 'services/viz-http';
import { JobWithLog } from 'types/job.type';
import { getGlobalArc } from 'utils/get-global-arc';
import { BackendLog, parseJobLog } from './logs/parse-log';
import { BackendJob, parseJob } from './parse-job';

const arc = getGlobalArc();

type BackendJobWithLog = {
  job: BackendJob;
  log: BackendLog;
};

type BackendRuns = {
  data: BackendJobWithLog[];
};

const parseJobWithLogs = (jobsWithLogs: BackendJobWithLog[]): JobWithLog[] => {
  return jobsWithLogs
    .map(({ log: backendLog, job: backendJob }) => {
      try {
        const log = parseJobLog(backendLog);
        const job = parseJob(backendJob);
        return { job, log };
      } catch {
        return undefined;
      }
    })
    .filter((run): run is JobWithLog => !!run);
};

export const getLatestRuns = async (): Promise<JobWithLog[]> => {
  const { data } = await vizHttp.get<BackendRuns>({
    url: arc.frontend.globals.urls.jobsLatestLogsApiUrl,
    isOldEndpoint: false,
  });

  return parseJobWithLogs(data.data);
};

export const useGetLatestRunsQuery = () => {
  const query = useQuery({
    queryKey: [QueryKeyBase.Jobs, 'latest'],
    staleTime: 300000, // 5 minutes
    queryFn: () => getLatestRuns(),
  });

  return query;
};
