import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { useMutationWithNotification } from 'hooks/use-mutation-with-notification';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const deleteLogs = async (logIds: number[]): Promise<void> => {
  await vizHttp.delete({
    url: arc.frontend.globals.urls.jobLogsApiUrl,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: false,
    data: logIds.join(','),
  });
};

export const useDeleteLogsMutation = (jobId?: number) => {
  const queryClient = useQueryClient();
  const mutation = useMutationWithNotification(
    {
      mutationFn: deleteLogs,
      onSuccess: () => {
        if (jobId) {
          queryClient.invalidateQueries([QueryKeyBase.JobLogs, jobId]);
        } else {
          queryClient.invalidateQueries([QueryKeyBase.JobLogs]);
        }
      },
    },
    {
      successMessage: 'Deleted log(s)',
      errorMessage: 'Failed to delete log(s)',
    }
  );

  return mutation;
};
