import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { useMutationWithNotification } from 'hooks/use-mutation-with-notification';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const pauseOrResumeJobs = async (ids: number[], resumeOnly = false) => {
  await vizHttp.post<void>({
    url: arc.frontend.globals.urls.jobPauseOrResumeUrl,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: true,
    data: {
      schedules: JSON.stringify(ids.map((id) => String(id))),
      ...(resumeOnly && { resume_only: 'true' }),
    },
  });
};

export const usePauseOrResumeJobsMutation = () => {
  const queryClient = useQueryClient();
  const mutation = useMutationWithNotification(
    {
      mutationFn: pauseOrResumeJobs,
      onSuccess: () => queryClient.invalidateQueries([QueryKeyBase.Jobs]),
    },
    {
      successMessage: 'Job(s) successfully paused/resumed',
      errorMessage: 'Failed to pause/resume job(s)',
    }
  );

  return mutation;
};

export const useResumeJobsMutation = () => {
  const queryClient = useQueryClient();
  const mutation = useMutationWithNotification(
    {
      mutationFn: (ids: number[]) => pauseOrResumeJobs(ids, true),
      onSuccess: () => queryClient.invalidateQueries([QueryKeyBase.Jobs]),
    },
    {
      successMessage: 'Job(s) successfully resumed',
      errorMessage: 'Failed to resume job(s)',
    }
  );

  return mutation;
};
