import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { useMutationWithNotification } from 'hooks/use-mutation-with-notification';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const rerunJobs = async (ids: number[]) => {
  const { data } = await vizHttp.post({
    url: arc.frontend.globals.urls.jobRerunUrl,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: true,
    data: {
      schedules: JSON.stringify(ids.map((id) => String(id))),
    },
  });
  return data;
};

export const useRerunJobsMutation = () => {
  const queryClient = useQueryClient();

  const mutation = useMutationWithNotification(
    {
      mutationFn: rerunJobs,
      onSuccess: (_, jobIds) => {
        queryClient.invalidateQueries([QueryKeyBase.Jobs]);
        jobIds.forEach((jobId) =>
          queryClient.invalidateQueries([QueryKeyBase.JobLogs, jobId])
        );
      },
    },
    {
      successMessage: 'Successfully started to run job(s)',
      errorMessage: 'Failed to run job(s)',
    }
  );

  return mutation;
};
