import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { useMutationWithNotification } from 'hooks/use-mutation-with-notification';
import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const deleteScheduleGroups = async (ids: number[]) => {
  await vizHttp.delete({
    url: arc.frontend.globals.urls.jobScheduleGroupsUrl,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: false,
    data: ids.join(','),
  });
};

export const useDeleteScheduleGroupsMutation = () => {
  const queryClient = useQueryClient();
  const mutation = useMutationWithNotification(
    {
      mutationFn: deleteScheduleGroups,
      onSuccess: () =>
        queryClient.invalidateQueries([QueryKeyBase.ScheduleGroups]),
    },
    {
      successMessage: 'Deleted schedule interval(s)',
      errorMessage: 'Failed to delete schedule interval(s)',
    }
  );

  return mutation;
};
