import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { useMutationWithNotification } from 'hooks/use-mutation-with-notification';
import { vizHttp } from 'services/viz-http';
import { ScheduleGroup } from 'types/schedule-group.type';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

type ScheduleGroupParams = {
  id: number;
  interval: string;
  name: string;
};

export const editScheduleGroup = async ({
  id,
  interval,
  name,
}: ScheduleGroupParams) => {
  await vizHttp.post<ScheduleGroup>({
    url: `${arc.frontend.globals.urls.jobScheduleGroupsUrl}/${id}`,
    headers: {
      'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
    },
    isOldEndpoint: true,
    data: {
      formEntries: JSON.stringify({
        schedule_name: name,
        run_interval: interval,
      }),
    },
  });
};

export const useEditScheduleGroupMutation = () => {
  const queryClient = useQueryClient();
  const mutation = useMutationWithNotification(
    {
      mutationFn: editScheduleGroup,
      onSuccess: () =>
        queryClient.invalidateQueries([QueryKeyBase.ScheduleGroups]),
    },
    {
      successMessage: 'Interval updated',
    }
  );

  return mutation;
};
