import { UseQueryOptions, useQuery } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { vizHttp } from 'services/viz-http';
import { ScheduleGroup } from 'types/schedule-group.type';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

type BackendScheduleGroup = {
  id: number;
  schedule_name: string;
  run_interval: string;
  next_run_time: string;
  jobschedule_set: {
    model: string;
    pk: number;
    fields: {
      job_name: string;
    };
  }[];
};

const parseScheduleGroups = (
  backendScheduleGroups: BackendScheduleGroup[]
): ScheduleGroup[] => {
  return backendScheduleGroups.map((scheduleGroup) => ({
    id: scheduleGroup.id,
    name: scheduleGroup.schedule_name,
    runInterval: scheduleGroup.run_interval,
    nextRunTime: scheduleGroup.next_run_time,
    dependentJobs:
      scheduleGroup.jobschedule_set.map((job) => ({
        id: job.pk,
        name: job.fields.job_name,
      })) || [],
  }));
};

export const getScheduleGroups = async (): Promise<ScheduleGroup[]> => {
  const { data } = await vizHttp.get<{
    schedule_groups: BackendScheduleGroup[];
  }>({
    url: arc.frontend.globals.urls.jobScheduleGroupsUrl,
    isOldEndpoint: false,
  });

  return parseScheduleGroups(data.schedule_groups);
};

type TQueryFnData = Awaited<ReturnType<typeof getScheduleGroups>>;

export const useGetScheduleGroupsQuery = <
  TError = unknown,
  TData = TQueryFnData
>(
  options?: UseQueryOptions<TQueryFnData, TError, TData>
) => {
  const query = useQuery({
    queryKey: [QueryKeyBase.ScheduleGroups],
    staleTime: 600000,
    queryFn: () => getScheduleGroups(),
    ...options,
  });

  return query;
};
