import { addLimitToSQLQuery } from './misc';

describe('addLimitToSQLQuery', () => {
  test('it should leave it uppercased', () => {
    const query = 'SELECT * FROM foo WHERE test_filter = "Test"';
    const actual = addLimitToSQLQuery(query, 100);
    const expected = 'SELECT * FROM foo WHERE test_filter = "Test" LIMIT 100';

    expect(actual).toBe(expected);
  });

  test('it should add the limit if it is not set already', () => {
    const query = 'SELECT * FROM foo WHERE test_filter = "Test"';
    const actual = addLimitToSQLQuery(query, 100);
    const expected = 'SELECT * FROM foo WHERE test_filter = "Test" LIMIT 100';

    expect(actual).toBe(expected);
  });

  test('it should not change the limit value if it is already set', () => {
    const query = 'SELECT * FROM foo LIMIT 200';
    const actual = addLimitToSQLQuery(query, 50);
    const expected = 'SELECT * FROM foo LIMIT 200';

    expect(actual).toBe(expected);
  });

  test('it should throw an error if query is invalid', () => {
    const query = '';
    const expectedError = new Error(
      `The provided query (${query}) is invalid. Please, provide a valid SELECT query.`
    );
    expect(() => {
      addLimitToSQLQuery(query, 100);
    }).toThrowError(expectedError);
  });

  test('it should leave it lowercased', () => {
    const query = 'select * from foo';
    const actual = addLimitToSQLQuery(query, 100);
    const expected = 'select * from foo limit 100';

    expect(actual).toBe(expected);
  });

  test('it should add limit as uppercase', () => {
    const query = 'SELECT * FROM foo';
    const actual = addLimitToSQLQuery(query, 100);
    const expected = 'SELECT * FROM foo LIMIT 100';

    expect(actual).toBe(expected);
  });

  test('it should not change the limit and leave it lowercased', () => {
    const query = 'select * from foo limit 200';
    const actual = addLimitToSQLQuery(query, 50);
    const expected = 'select * from foo limit 200';

    expect(actual).toBe(expected);
  });

  test('it should leave the query as is if not a select query', () => {
    const query = 'insert into whatever not select query';
    const actual = addLimitToSQLQuery(query, 50);
    const expected = 'insert into whatever not select query';

    expect(actual).toBe(expected);
  });

  test('it should work with multiline queries and multiplace spaces', () => {
    const query = 'select\n*\nfrom foo   limit  200';
    const actual = addLimitToSQLQuery(query, 50);
    const expected = 'select * from foo limit 200';

    expect(actual).toBe(expected);
  });

  test('it should trim the query', () => {
    const query = '   select * from foo limit 200  ';
    const actual = addLimitToSQLQuery(query, 50);
    const expected = 'select * from foo limit 200';

    expect(actual).toBe(expected);
  });
});
