/* eslint-disable camelcase */
import Cookies from 'js-cookie';

import type { FeatureFlipperService } from 'services/feature-flipper';

import type { Setting } from './setting.type';
import type { Column, Dataset } from './dataset.type';
import type { EditorWrapper } from './editor-wrapper.type';
import type { Report } from './report.type';
import type { Dataconnection } from './dataconnection.type';
import type { FilterControls } from './dashboard-filter-controls.type';
import type { Globals } from 'services/globals';
import { query } from './query.type';
import { CreateDatasetFromSQL } from './create-dataset-request-from-sql';
import { BaseQuery } from './baseQuery.type';
import type { LegacyBridge } from '../../services/legacy-bridge/legacy-bridge';

type OptionalAlertConfig = {
  alert_report_id?: string | number;
  is_html?: boolean;
  noConsoleLog?: boolean;
  alertContainerSelector?: string;
};

export interface SelectBuilder {
  limit: (limit: number) => void;
  addProjection: (projection: string) => void;
  addTablenamesFromExpression: (text: string) => string;
  addGroupWithoutProjection: (groupBy: string | number) => void;
  addGroup: (groupBy: string | number) => void;
  addOrderBy: (orderBy: string) => void;
  getSql: () => string;
  dataset(dataset: Dataset): string;
}
export interface Arc {
  globals: {
    arcviz_settings: {
      SESSION_COOKIE_SECURE: boolean;
      SESSION_COOKIE_SAMESITE: Cookies.CookieAttributes['sameSite'];
      ENABLE_DIRECT_ACCESS: boolean;
      HELP_SHOW_GETSTARTED: boolean;
      HELP_SHOW_WHATSNEW: boolean;
      HELP_SHOW_DOCUMENTATION: boolean;
      HELP_MENU_LINKS: (
        | { type: 'link'; label: string; link: string }
        | { type: 'divider' }
      )[];
      CDV_MODE: string;
      ENABLE_DATACONN_IMPALA: boolean;
      ENABLE_DATACONN_HIVE: boolean;
    };
    app_settings: {
      unnamedVersionDeletionInterval: string;
    };
    csrf_token: string;
    can_user_explore_any_dataconnections: boolean;
    embed_url_prefix: string;
    username: string;
    dashboard: {
      dashboard_filter_controls(): FilterControls;
      data_connections_hash(): Record<string, Dataconnection>;
      paintVisualSidebarDatasetsOfConn(dataconnectionId: string | number): void;
      dashboard_id(): string;
      getActiveReport(): Report | undefined;
      fetchDataset(
        datasetInfo: { dataset_id: string | number },
        succesCallBack: (
          config: unknown,
          datasetData: unknown,
          columnInfo: unknown
        ) => void,
        errorCallBack: () => void
      ): Dataset;
      dataset_hash(): Record<string | number, Dataset>;
      patchDatasetRefOfReport(
        reportId: string,
        datasetInfo: { dataset_id: number | string }
      ): Promise<string>;
      patchDatasetRefOfReports(datasetInfo: {
        dataset_id: number | string;
        report_ids: string[];
      }): Promise<string[]>;
      reports(): {
        getAllReports(): Report[];
      };
      getVisuals(): Report[];
      data_connections(): Dataconnection[];
    };

    perms: { sysperms: string[] };

    productName: string;
  };
  report: {
    alertMessageSmall(
      message: string,
      type: 'alert-success' | 'alert-danger'
    ): void;
    alertMessage(
      message: string,
      detail?: string,
      optionalAlertConfig?: OptionalAlertConfig
    ): void;
    getPrimaryReport(): Report;
    sql_runner: {
      runParams: (config: {
        dataconnection_id: string | number;
        query?: string;
        cb?: (errorMsg: string, response: any) => void;
        cache?: boolean;
        dsreq: any;
        displayErrorMessage?: boolean;
        hideAlertsOnSuccess?: boolean;
        lib?: any;
      }) => any;
    };
  };

  visual: {
    setting(): Setting;
    settingTypeEnum: {
      CHECKBOX: 1;
      RADIO: 2;
      INTEGER: 3;
      STRING: 4;
      REAL: 5;
      INTEGER_RANGE: 6;
      CUSTOM: 7;
      PASSWORD: 8;
      DROPDOWN: 9;
      COLOR: 10;
      INTEGER_NOSPINNER: 11;
      SLIDER: 12;
      SLIDER_RANGE: 13;
      DISPLAY_ONLY: 14;
      DROPDOWN_CHOSEN: 15;
      EXPRESSION: 16;
    };
  };

  filter: {
    expression(): Setting;
  };

  aliasesadmin: {
    updateAliasRow(data: Object): void;
  };

  misc: {
    openInNewWindow(url: string): void;
    plotTable(
      rows: any,
      columnNames: string[],
      $container: JQuery<HTMLElement>,
      isSimpleTable?: boolean
    ): { adjustColumnWidths: () => void };
    drawTable(
      containerDivId: string,
      rows: Array<Array<any>>,
      colnames: Array<string>,
      setting?: Array<{ key: string; value: string }>,
      tableConfiguration?: {
        cellsHtml?: boolean;
        showTooltip?: boolean;
        disableProfile?: boolean;
        enablePagination?: boolean;
        pageSize?: number;
        enableInfo?: boolean;
      }
    ): void;
    expressionEditor(
      dataset: Dataset | undefined,
      editor: EditorWrapper | undefined,
      containerId: string,
      $editDiv: JQuery<HTMLElement>,
      config?: {
        includeTables?: boolean;
        excludeRecordCountAutocomplete?: boolean;
        datasetSql?: boolean;
        tables?: Record<string, unknown>[];
        includeCalculated?: boolean;
        useRowNames?: boolean;
        dataconnection_type?: unknown;
      },
      resizable?: boolean
    ): EditorWrapper;
    getBlankTargetUrl(url: string): string;
    createFromSql(): CreateDatasetFromSQL;
    splitExprAndAlias(expression: string): { expr: string; alias?: string };
    hasPerm(perm: string, permList: string[]): boolean;
    isWindowFunction(expression: string): boolean;
    uniqueId(): string;
    determineDatabaseInQuery: (query?: string) => string;
    notify(message: string): void;
    simpleUserConfirmation(config: {
      confirmTitle: string;
      confirmQus: string;
      confirmText: string;
    }): {
      askUser(callback: () => void): void;
    };
    exChangeElementsVisibility({
      elementToShow,
      elementToHide,
    }: {
      elementToShow: JQuery<HTMLElement>;
      elementToHide: JQuery<HTMLElement>;
    }): void;
    sessionKeyGenerator: {
      chatBot: (reportId: number) => string;
    };
  };

  dataprofileHelper: {
    plotTopFrequentValueVisual(
      dataset: Dataset,
      column: Column,
      chartDivId: string
    ): void;
    plotProfileHistogram(
      dataset: Dataset,
      column: Column,
      chartDivId: string,
      minMax: number[],
      onError: (msg: string, detail: string) => void
    ): void;
  };

  clone_template(
    id: string,
    substitutes?: Record<string, unknown>,
    attributes?: Record<string, unknown>
  ): JQuery<HTMLElement>;

  validate_expression(
    expression: string,
    dataset: Dataset,
    callback: (params: { [key: string]: unknown }) => void,
    type: string,
    config?: {
      inclds?: boolean;
      skipcol?: unknown;
      dsObj?: unknown;
      alias?: unknown;
    }
  ): unknown;
  dataconnection: {
    DataConnection(): Dataconnection;
    approximateOrExactCount: (conneectionType: string, expr: string) => string;
    types: { brackets: (text: string) => string };
  };
  datasets: {
    Dataset(): Dataset;
    getAssociatedDashboards(
      datasetId: string | number
    ): Promise<{ dashboardIds: string[] }>;
    createFromSql(): CreateDatasetFromSQL;
    drop_dbname(databaseAndTableName: string): string;
  };

  // Not this can be called on pages where we include the old part stuff - to have the requestBuilder initialized...
  // For example do not call on homepage:(
  requestBuilder(): {
    requestType(
      type:
        | '.tables'
        | '.databases'
        | '.getrows'
        | '.invalidate_metadata'
        | 'compute_stats'
        | 'SOLR'
        | '.drop_aviews'
        | 'createaview'
        | 'MR'
        | 'save_table'
        | '.validate-filter'
        | '.validate-column'
        | 'validate-query'
        | 'refresh_aviews'
        | 'data_import'
    ): unknown;
    dsreq: () => {
      dataset_object: string;
      add_column: (column: Column) => void;
      sql: (sql: string) => void;
      finalize: () => void;
    };
    dbname: (databaseName: string) => void;
    datasetId: (id: number | string) => void;
    limit: (limit: number) => void;
    dimensions: (
      dimensions: {
        type: string;
        expr: string;
        order?: { asc: boolean; pri: number };
      }[]
    ) => void;
  };

  selectBuilder(): SelectBuilder;

  data: {
    queryCmd(): query;
    runQuery(
      rb: any,
      successCb: (response: any) => any,
      failCb: (response: any) => any,
      lib: any,
      errorReportingCb: (qstate: any, msg: any, error: any) => any,
      timeoutSeconds: number | undefined,
      timeoutCb: any,
      queryCmd: query,
      loader?: any
    ): BaseQuery;
  };

  frontend: {
    featureFlipper: FeatureFlipperService;
    globals: Globals;
    legacyBridge: LegacyBridge;
  };

  sqlrun: {
    httpAsyncCall(params: {
      url: string;
      paramsToSend: string;
      doneCb: (result: any) => void;
      failCb: (error: any) => void;
      contentType?: string;
    }): void;
  };

  topnav: { setup(): void };
  apps_data: {
    fetch_and_draw: (callback: () => string) => void;
    set_active_dataconnection: (data: { id: number; name: string }) => void;
  };
}
