/* eslint-disable camelcase */
export interface Table {
  table_select(sql: string): Table;
  table_select(): string;
  tablename(): string;
  base_columns(): Column[];
  dimensionsAndHierarchies(): Column[];
  measuresOnly(): Column[];
}

export type ColumnTypeClass =
  | 'type-string'
  | 'type-integer'
  | 'type-real'
  | 'type-boolean'
  | 'type-timestamp'
  | 'type-hierarchy'
  | 'type-array'
  | 'type-map'
  | 'type-struct'
  | 'type-vector'
  | 'type-text'
  | 'type-unknown';

export type ColumnAttribute = {
  name: string;
  type?:
    | 'BIGINT'
    | 'STRING'
    | 'BOOLEAN'
    | 'INT'
    | 'DOUBLE'
    | 'TIMESTAMP'
    | 'BINARY'
    | 'VECTOR'
    | 'TEXT';
  alias: string;
  isdim: boolean;
};
export interface Column {
  table(): Table;
  expr(): string;
  expr(expr: string): void;
  coltype():
    | 'UNKNOWN'
    | 'STRING'
    | 'INT'
    | 'BIGINT'
    | 'REAL'
    | 'DOUBLE'
    | 'BOOLEAN'
    | 'TIMESTAMP'
    | 'BINARY'
    | 'VECTOR'
    | 'TEXT';
  columnType(): string;
  attr(): ColumnAttribute;
  build_dsobject(): BuildDsObject;
  cloneDataColumn(): Column;
  isBaseCol(): string | undefined;
  isHierarchy(): boolean;
  isComplexType(): boolean;
  display_name(): string;
  coltype_class(): ColumnTypeClass;
}

export interface BuildDsObject {
  alias: string;
  cast?: string;
  isdim: Boolean;
  name: string;
  type: string; // string, number, date
  expr: string;
  basecol?: string;
}

export interface Dataset {
  colnames_from_expr(expr: string): {
    colnames: string[];
    columns: Column;
    sqltext: string;
  };
  dataconnection_id(): number | string;
  dataconnection_id(id: string | number): Dataset;
  dataconnection_type(type: string): Dataset;
  dataconnection_type(): string;
  dataset_id(): number | string;
  dataset_id(id: string | number): Dataset;
  dataset_info(): string;
  dataset_info(info: string): Dataset;
  dataset_name(): string;
  dataset_name(name: string): Dataset;
  dataset_detail(): string;
  dataset_detail(detail: string): Dataset;
  dataset_type(type: string): Dataset;
  version_name(version_name: string): Dataset;
  init_from_fetch(datasetData: unknown, columnInfo: unknown): Dataset;
  cloneDatasetOnServer(): Promise<string>;
  tables(): Table[];
  fetch_and_add_table(
    tableName: string,
    cb: () => void,
    error_cb: (error?: Error | { response: string }) => void
  ): Dataset;
  table_select(sql: string): Dataset;
  table_select(): string;
  update_tables(callback: (_: unknown, errorMessage: string) => void): void;
  save(
    doneCb: (error: string | Error, dataset: Dataset) => void,
    errorCb?: (error: string | Error) => void
  ): void;
  buildInfo(buildInfo: any): void;
  buildInfo(): any;
}
