import React, { useMemo } from 'react';
import { ActivityLogStatus } from '../types/activity-logs.type';
import CancelIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-stopped.svg';
import CompletedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-success.svg';
import RunningIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-in-progress.svg';
import QueuedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-waiting.svg';
import SuspendedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-suspended.svg';
import classNames from 'classnames';

type ActivityLogsStatusIndicatorProps = {
  status?: ActivityLogStatus;
  isAnimated?: boolean;
};

export const ActivityLogsStatusIndicator = ({
  status,
  isAnimated = false,
}: ActivityLogsStatusIndicatorProps): JSX.Element => {
  const { Icon, label, className } = useMemo(() => {
    switch (status) {
      case ActivityLogStatus.Cancelled:
        return { Icon: CancelIcon, label: 'Cancelled' };
      case ActivityLogStatus.Done:
        return { Icon: CompletedIcon, label: 'Done' };
      case ActivityLogStatus.Queued:
        return { Icon: QueuedIcon, label: 'Queued' };
      case ActivityLogStatus.Running:
        return {
          Icon: RunningIcon,
          label: 'Running',
          className: isAnimated ? 'fa-spin' : '',
        };
      case ActivityLogStatus.Abandoned:
        return { Icon: SuspendedIcon, label: 'Abandoned' };
      default:
        return { Icon: null, label: null };
    }
  }, [status, isAnimated]);

  return (
    <div className="d-flex align-items-center">
      {Icon ? (
        <>
          <Icon
            fontSize="20"
            className={classNames('flex-shrink-0', className)}
          />
          <span className="ml-1">{label}</span>
        </>
      ) : null}
    </div>
  );
};
