import React, { useState } from 'react';
import { Popover, OverlayTrigger } from 'react-bootstrap';
import ErrorIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-error.svg';
import classes from '../ActivityLogs.module.scss';
import { VizButton } from 'components/viz-button/VizButton';
import { copyToClipboard } from 'arc/misc/misc';

type ErrorPopoverProps = {
  rowId: string;
  errorDetail: string;
  visiblePopoverId: string;
  onToggle: (id: string) => void;
};
const ErrorPopover = ({
  rowId,
  errorDetail,
  visiblePopoverId,
  onToggle,
}: ErrorPopoverProps) => {
  const [copySuccess, setCopySuccess] = useState(false);

  const handleCopy = () => {
    copyToClipboard(errorDetail).then(() => {
      setCopySuccess(true);
      setTimeout(() => setCopySuccess(false), 2000);
    });
  };

  const handleClose = () => {
    onToggle('');
  };

  const popover = (
    <Popover id={`popover-${rowId}-error`}>
      <Popover.Title>
        Error detail{' '}
        <div className={classes.closeButton}>
          <i
            data-test-id="tooltip-close-button"
            className="fa fa-times-circle"
            onClick={handleClose}
          />
        </div>
      </Popover.Title>
      <Popover.Content>
        <div style={{ maxHeight: '200px', overflowY: 'auto' }}>
          <p>{errorDetail}</p>
        </div>
        <div className="d-flex justify-content-end">
          <VizButton
            size="small"
            color="secondary"
            className={copySuccess ? classes.borderGreen : ''}
            onClick={handleCopy}
          >
            {copySuccess ? 'COPIED' : 'COPY'}
          </VizButton>
        </div>
      </Popover.Content>
    </Popover>
  );

  return (
    <div>
      <OverlayTrigger
        trigger="click"
        placement="top"
        overlay={popover}
        show={visiblePopoverId === `${rowId}-error`}
        onToggle={() =>
          onToggle(
            visiblePopoverId === `${rowId}-error` ? '' : `${rowId}-error`
          )
        }
      >
        <div
          className="d-flex align-items-center"
          style={{ cursor: 'pointer' }}
        >
          <ErrorIcon fontSize="20" />
          <span className="ml-1">Error</span>{' '}
        </div>
      </OverlayTrigger>
    </div>
  );
};

export default ErrorPopover;
