import { VizButton } from 'components/viz-button/VizButton';
import React, { useMemo, useState } from 'react';
import { Popover, OverlayTrigger } from 'react-bootstrap';
import classes from '../ActivityLogs.module.scss';
import { copyToClipboard, formattingQuery } from 'arc/misc/misc';

type QueryPopoverProps = {
  rowId: string;
  visiblePopoverId: string;
  onToggle: (id: string) => void;
  queryText: string;
};

const QueryPopover = ({
  rowId,
  visiblePopoverId,
  onToggle,
  queryText,
}: QueryPopoverProps) => {
  const [copySuccess, setCopySuccess] = useState(false);

  const handleCopy = () => {
    copyToClipboard(queryText).then(() => {
      setCopySuccess(true);
      setTimeout(() => setCopySuccess(false), 2000);
    });
  };

  const handleClose = () => {
    onToggle('');
  };

  const formattedQueryText = useMemo(
    () => formattingQuery(queryText, 'mysql'),
    [queryText]
  );

  const popover = (
    <Popover id={`popover-${rowId}-query`}>
      <Popover.Title as="h3">
        SQL Query{' '}
        <div className={classes.closeButton}>
          <i
            data-test-id="tooltip-close-button"
            className="fa fa-times-circle"
            onClick={handleClose}
          />
        </div>
      </Popover.Title>
      <Popover.Content>
        <div className={classes.popOverContainer}>
          <pre>{formattedQueryText}</pre>
        </div>
        <div className="d-flex justify-content-end">
          <VizButton
            size="small"
            color="secondary"
            className={copySuccess ? classes.borderGreen : ''}
            onClick={handleCopy}
          >
            {copySuccess ? 'COPIED' : 'COPY'}
          </VizButton>
        </div>
      </Popover.Content>
    </Popover>
  );

  return (
    <div>
      <OverlayTrigger
        trigger="click"
        placement="top"
        overlay={popover}
        show={visiblePopoverId === `${rowId}-query`}
        onToggle={() =>
          onToggle(
            visiblePopoverId === `${rowId}-query` ? '' : `${rowId}-query`
          )
        }
      >
        <VizButton
          className={classes.noWrap}
          color="secondary"
          size="small"
          onClick={() =>
            onToggle(
              visiblePopoverId === `${rowId}-query` ? '' : `${rowId}-query`
            )
          }
        >
          {visiblePopoverId === `${rowId}-query` ? 'Hide SQL' : 'Show SQL'}
        </VizButton>
      </OverlayTrigger>
    </div>
  );
};

export default QueryPopover;
