export enum ActivityLogStatus {
  Running = 1,
  Done = 2,
  Queued = 3,
  Cancelled = 4,
  Abandoned = 5,
}

export enum ActivityLogActions {
  Create = 1,
  View = 2,
  Update = 3,
  Delete = 4,
}

export enum ActivityLogComponentTypes {
  Dashboard = 1,
  Dataset = 2,
  'Data Connection' = 3,
  User = 4,
  App = 5,
  'Custom Style' = 6,
  'Custom Color' = 7,
  Snapshots = 8,
  Thumbnail = 9,
  'Static Asset' = 10,
  Role = 11,
  'Custom Email' = 12,
  'Custom Date Range' = 13,
  Workspace = 14,
  Search = 15,
  'Direct Query' = 16,
  Segment = 17,
  'Filter Association' = 18,
  Job = 19,
  Extract = 20,
}

type AggValue<T extends [string | number, number, ...Array<string | number>]> =
  Record<string, T>;

type AggAction = AggValue<[string, number, string]>;
type AggVisual = AggValue<[number, number, number, string]>;
type AggApp = AggValue<[number, number, string, number, string]>;
type AggDataset = AggValue<[number, number, number, string]>;
type AggCacheStat = AggValue<[number | string, number, string]>;

export type Statistics = {
  aggAction: AggAction;
  aggUsername: Record<string, number>;
  aggIp: Record<string, number>;
  aggConnection: Record<string, number>;
  aggVisual: AggVisual;
  aggApp: AggApp;
  aggDataset: AggDataset;
  aggRuntime: Record<string, number>;
  aggCache: {
    Yes: number;
    No: number;
  };
  aggCacheStat: AggCacheStat;
  firsttime: string | undefined;
  lasttime: string | undefined;
};
