import {
  ClouderaDialog,
  ClouderaDialogContent,
} from 'components/cloudera-dialog/ClouderaDialog';
import React, { useCallback, useState } from 'react';
import { useLegacyBridge } from 'services/legacy-bridge';
import { AnalyticalJobModal } from './AnalyticalJobModal';

type ViewRefreshPayload = {
  dataConnectionId: number;
  viewName: string;
};

export const AnalyticalJobModalWrapper = (): JSX.Element => {
  const [analyticalView, setAnalyticalView] = useState<ViewRefreshPayload>();
  const [isOpen, setIsOpen] = useState(false);

  const openAnalyticalJobModal = useCallback(
    ({ payload }: { payload?: ViewRefreshPayload }) => {
      setAnalyticalView(payload);
      setIsOpen(true);
    },
    []
  );

  useLegacyBridge('open-analytical-job-modal', openAnalyticalJobModal);

  if (!analyticalView) {
    return <></>;
  }

  return (
    <ClouderaDialog isOpen={isOpen} onChange={setIsOpen}>
      <ClouderaDialogContent size="lg" scrollable>
        <AnalyticalJobModal
          dataConnectionId={analyticalView.dataConnectionId}
          viewName={analyticalView.viewName}
        />
      </ClouderaDialogContent>
    </ClouderaDialog>
  );
};
