import React from 'react';
import Modal from 'react-bootstrap/Modal';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useChangePasswordForm } from 'hooks/use-change-password-form';
import { VizButton } from 'components/viz-button/VizButton';
import classes from './ChangePasswordModal.module.scss';

type ChangePasswordModalProps = {
  isOpen: boolean;
  close: () => void;
};

export const ChangePasswordModal = ({
  isOpen,
  close,
}: ChangePasswordModalProps) => {
  const { formFields, initialValues, onSubmit, validationSchema } =
    useChangePasswordForm({
      onSuccess: () => {
        close();
      },
    });

  const {
    register,
    handleSubmit,
    formState: { errors, isDirty, isValid },
    reset,
  } = useForm({
    defaultValues: initialValues,
    mode: 'onTouched',
    resolver: yupResolver(validationSchema),
  });

  const onHide = () => {
    reset();
    close();
  };

  return (
    <Modal show={isOpen} onHide={onHide}>
      <Modal.Header closeButton>
        <Modal.Title>Change My Password</Modal.Title>
      </Modal.Header>

      <form onSubmit={handleSubmit(onSubmit)}>
        <Modal.Body className={classes.modalBody}>
          {formFields(register, errors)}

          <p className={classes.helperText}>
            If you have forgotten your password, contact your Data Visualization
            admin to have it reset.
          </p>
        </Modal.Body>

        <Modal.Footer>
          <VizButton color="secondary" size="medium" onClick={onHide}>
            Cancel
          </VizButton>

          <VizButton
            color="primary"
            size="medium"
            disabled={!isDirty || !isValid}
            type="submit"
          >
            Save
          </VizButton>
        </Modal.Footer>
      </form>
    </Modal>
  );
};
