import React, {
  ButtonHTMLAttributes,
  PropsWithChildren,
  forwardRef,
} from 'react';
import classes from './ClouderaButton.module.scss';
import classNames from 'classnames';

type ClouderaButtonProps = PropsWithChildren<
  ButtonHTMLAttributes<HTMLButtonElement> & {
    // This is a hack for Dropdown.Toggle Typescript error
    size?: 'sm' | 'small' | 'default' | 'large';
    color?: 'primary' | 'secondary' | 'default' | 'danger';
    testId?: string;
    icon?: JSX.Element;
    iconPosition?: 'left' | 'right';
    variant?: 'button' | 'link' | 'pill' | 'ghost';
  }
>;

export const ClouderaButton = forwardRef<
  HTMLButtonElement,
  ClouderaButtonProps
>(
  (
    {
      size = 'default',
      color = 'default',
      children,
      testId,
      icon,
      iconPosition = 'left',
      className,
      variant = 'button',
      ...buttonProps
    },
    ref
  ): JSX.Element => {
    return (
      <button
        ref={ref}
        className={classNames(classes.button, {
          [classes.primary]: color === 'primary',
          [classes.secondary]: color === 'secondary',
          [classes.default]: color === 'default',
          [classes.danger]: color === 'danger',
          [classes.link]: variant === 'link',
          [classes.pill]: variant === 'pill',
          [classes.ghost]: variant === 'ghost',
          [classes.small]: size === 'small' || size === 'sm',
          [classes.large]: size === 'large',
        })}
        {...buttonProps}
        data-test-id={testId}
      >
        {icon && iconPosition === 'left' && (
          <div className={classes.iconContainer}>{icon}</div>
        )}

        {children && (
          <div
            className={classNames({
              'ml-1': icon && iconPosition === 'left',
              'mr-1': icon && iconPosition === 'right',
            })}
          >
            <span className={classes.children}>{children}</span>
          </div>
        )}

        {icon && iconPosition === 'right' && (
          <div className={classes.iconContainer}>{icon}</div>
        )}
      </button>
    );
  }
);
