import '@testing-library/jest-dom';
import { configure, fireEvent, render, screen } from '@testing-library/react';
import * as React from 'react';
import { ClouderaDateFilter } from './ClouderaDateFilter';

configure({ testIdAttribute: 'data-test-id' });

describe('ClouderaDateFilter component', () => {
  it('renders without error', () => {
    const mockOnChange = jest.fn();

    render(
      <ClouderaDateFilter
        id="startDate"
        label="Start Date"
        value={''}
        onChange={mockOnChange}
      />
    );
  });

  it('lets you select a date and sets it as a filter', async () => {
    const mockOnChange = jest.fn();

    render(
      <ClouderaDateFilter
        id="startDate"
        label="Start Date"
        value={''}
        onChange={mockOnChange}
      />
    );

    const dateFilter = screen.getByLabelText('Filter Start Date');
    fireEvent.change(dateFilter, {
      target: { value: '2024-06-05' },
    });
    expect(mockOnChange).toHaveBeenCalledWith('2024-06-05');
  });

  it('displays the existing filter on the button', () => {
    const mockOnChange = jest.fn();

    render(
      <ClouderaDateFilter
        id="startDate"
        label="Start Date"
        value={'2024-06-05'}
        onChange={mockOnChange}
      />
    );

    expect(screen.getByTestId('date-filter-startDate')).toHaveTextContent(
      '2024-06-05'
    );
  });
});
