import DropdownIcon from '@cloudera-internal/cuix-core/icons/svg/ic-dropdown.svg';
import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import moment from 'moment';
import React, { useRef } from 'react';
import { Form } from 'react-bootstrap';
import classes from './ClouderaDateFilter.module.scss';

interface ClouderaDateFilterProps
  extends Omit<
    React.ComponentProps<typeof ClouderaButton>,
    'value' | 'onChange'
  > {
  value: string | undefined;
  onChange: (value?: string) => void;
  id: string;
  label: string;
  includeTime?: boolean;
}

/**
 * A popover style date filter
 * @param id unique identifier of the filter element
 * @param value currently selected date
 * @param onChange callback invoked when the currently selected options change
 * @param label the text to be displayed on the filter button
 * @param includeTime changes the input type from date to date-time
 */
export const ClouderaDateFilter = ({
  id,
  value,
  onChange,
  label,
  includeTime = false,
  ...props
}: ClouderaDateFilterProps): JSX.Element => {
  const datePickerRef = useRef<HTMLInputElement>(null);

  return (
    <ClouderaButton
      color="secondary"
      variant="pill"
      size="sm"
      icon={<DropdownIcon />}
      iconPosition="right"
      onClick={() => {
        datePickerRef.current?.focus();
        datePickerRef.current?.showPicker();
      }}
      testId={`date-filter-${id}`}
      {...props}
    >
      <Form.Control
        ref={datePickerRef}
        className={classes.datePicker}
        type={includeTime ? 'datetime-local' : 'date'}
        value={value}
        onChange={(e) => onChange(e.target.value)}
        aria-label={`Filter ${label}`}
      />
      {label}
      {!!value &&
        `: ${moment
          .utc(value)
          .local()
          .format(includeTime ? 'YYYY-MM-DD HH:mm' : 'YYYY-MM-DD')}`}
    </ClouderaButton>
  );
};
