import { Column as TanstackColumn } from '@tanstack/react-table';
import { ClouderaDateFilter } from 'components/cloudera-filter/ClouderaDateFilter';
import React, { useCallback } from 'react';

interface ClouderaTableDateFilterProps<TData, TValue>
  extends Omit<
    React.ComponentProps<typeof ClouderaDateFilter>,
    'id' | 'onChange' | 'value'
  > {
  column: TanstackColumn<TData, TValue>;
  label: string;
  includeTime?: boolean;
}

/**
 * A wrapper around ClouderaDateFilter, to be used alongside the ClouderaTable component
 * @param column a Tanstack Table column definition for the data that should be filtered
 * @param label the text to be displayed on the filter button
 * @param includeTime changes the input type from date to date-time
 */
export const ClouderaTableDateFilter = <
  TData extends unknown,
  TValue extends unknown
>({
  column,
  ...props
}: ClouderaTableDateFilterProps<TData, TValue>): JSX.Element => {
  const value = (column?.getFilterValue() as string | undefined) ?? '';

  const handleChange = useCallback(
    (value: string | undefined) => column.setFilterValue(value),
    [column]
  );

  return (
    <ClouderaDateFilter
      value={value}
      onChange={handleChange}
      id={column.id}
      {...props}
    />
  );
};
