import { Column as TanstackColumn } from '@tanstack/react-table';
import { ClouderaListFilter } from 'components/cloudera-filter/ClouderaListFilter';
import React, { useCallback } from 'react';

interface ClouderaTableListFilterProps<TData, TValue>
  extends Omit<
    React.ComponentProps<typeof ClouderaListFilter>,
    'id' | 'onChange' | 'value'
  > {
  column: TanstackColumn<TData, TValue>;
  options: { value: TValue; label: React.ReactNode; stringLabel?: string }[];
}

// Stable reference to an empty array: https://github.com/TanStack/table/issues/4566
const emptyArray: unknown[] = [];

/**
 * A ClouderaTable compatible wrapper around the ClouderaListFilter
 * @param column a Tanstack Table column definition for the data that should be filtered
 * @param options a list of filter options
 * @param label the text to be displayed on the filter button
 * @param confirmChange all the selected filters will be applied at once, when the Apply button is pressed, @default true
 */
export const ClouderaTableListFilter = <
  TData extends unknown,
  TValue extends unknown
>({
  column,
  ...props
}: ClouderaTableListFilterProps<TData, TValue>): JSX.Element => {
  // ensure type safety of the filter value
  const filterArray = (
    Array.isArray(column.getFilterValue())
      ? column.getFilterValue()
      : emptyArray
  ) as TValue[];

  const handleChange = useCallback(
    (value: TValue[]) => {
      column.setFilterValue(value);
    },
    [column]
  );

  return (
    <ClouderaListFilter
      value={filterArray}
      onChange={handleChange}
      id={column.id}
      {...props}
    />
  );
};
