import { useToggle } from 'hooks/use-toggle';
import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import React from 'react';
import { Accordion, AccordionCollapse, AccordionToggle } from 'react-bootstrap';

type CollapsibleListProps = {
  id: string;
  threshold: number;
  children: React.ReactNode;
  expandText?: string;
  collapseText?: string;
};

export const CollapsibleList = ({
  id,
  threshold,
  children,
  expandText = 'View More',
  collapseText = 'View Less',
}: CollapsibleListProps) => {
  const [isOpen, toggle] = useToggle();

  const childList = Array.isArray(children) ? children : [children];

  return (
    <>
      {childList.slice(0, threshold)}
      {childList.length > threshold && (
        <Accordion activeKey={isOpen ? id : ''}>
          <AccordionCollapse eventKey={id}>
            <>{childList.slice(threshold)}</>
          </AccordionCollapse>
          <AccordionToggle
            eventKey={id}
            as={ClouderaButton}
            variant="link"
            onClick={toggle}
          >
            {isOpen ? collapseText : expandText}
          </AccordionToggle>
        </Accordion>
      )}
    </>
  );
};
