import React from 'react';
import {
  render,
  fireEvent,
  queryByAttribute,
  waitFor,
  act,
} from '@testing-library/react';
import { CopyToClipboard } from './CopyToClipboard';
import { copyToClipboard } from 'arc/misc/misc';
import '@testing-library/jest-dom';

jest.mock('arc/misc/misc', () => ({
  copyToClipboard: jest.fn().mockResolvedValue(undefined),
}));

describe('CopyToClipboard Component', () => {
  beforeEach(() => {
    jest.useFakeTimers();
    jest.clearAllMocks();
  });

  afterEach(() => {
    jest.runOnlyPendingTimers();
    jest.useRealTimers();
  });

  test('copy icon functionality - copies text and prevents duplicate copies', async () => {
    const textToCopy = 'Sample text';
    const { container } = render(<CopyToClipboard text={textToCopy} />);
    const copyIcon = queryByAttribute('data-test-id', container, 'copy-icon');

    if (!copyIcon) {
      throw new Error('Copy icon not found in the document');
    }

    fireEvent.click(copyIcon);

    await waitFor(() => {
      expect(copyIcon).toHaveClass('fa-check');
    });

    expect(copyToClipboard).toHaveBeenCalledWith(textToCopy);

    fireEvent.click(copyIcon);

    expect(copyToClipboard).toHaveBeenCalledTimes(1);

    act(() => {
      jest.advanceTimersByTime(2000);
    });

    await waitFor(() => {
      expect(copyIcon).toHaveClass('fa-clipboard');
    });
  });
});
