import React from 'react';
import Form from 'react-bootstrap/Form';
import { Control, Controller } from 'react-hook-form';
import Select from 'react-select';
import {
  getDatabaseListPlaceHolder,
  getTableListPlaceHolder,
} from 'utils/list-placeholder';

type FromTableFormSectionProps = {
  databaseList?: string[];
  tableList?: string[];
  control: Control<any>;
  isTableListLoading: boolean;
  isDatabaseListLoading: boolean;
  databaseLoadError?: Error | unknown;
  tableLoadError?: Error | unknown;
  defaultTable?: string;
  defaultDatabase?: string;
};
const FromTableFormSection = ({
  databaseList,
  tableList,
  control,
  isTableListLoading,
  isDatabaseListLoading,
  databaseLoadError,
  tableLoadError,
  defaultTable,
  defaultDatabase,
}: FromTableFormSectionProps): JSX.Element => {
  // Propagate the inital values first and let the full list (tableList/databaseList) loaded meanwhile!
  const placeHolderTableList =
    isTableListLoading && defaultTable ? [defaultTable] : [];
  const placeHolderDatabaseList =
    isDatabaseListLoading && defaultDatabase ? [defaultDatabase] : [];

  const tableOptions = (tableList ?? placeHolderTableList).map((table) => {
    return { value: table, label: table };
  });

  const databaseOptions = (databaseList ?? placeHolderDatabaseList).map(
    (database) => {
      return { value: database, label: database || '<<DEFAULT>>' };
    }
  );
  return (
    <>
      <Controller
        control={control}
        render={({ field: { onChange, value } }) => {
          return (
            <Form.Group data-test-id="selected-database-selector">
              <Form.Label htmlFor="selectedDatabase">
                Select Database
              </Form.Label>
              <Select
                options={databaseOptions}
                placeholder={getDatabaseListPlaceHolder(
                  isDatabaseListLoading,
                  databaseLoadError
                )}
                // without explicitly set it to null, it shows the previous result,
                // even if the value in not in the list and it is undefined...
                value={databaseOptions.find((x) => x.value === value) ?? null}
                id="selectedDatabase"
                onChange={(obj) => {
                  onChange(obj?.value);
                }}
              />
            </Form.Group>
          );
        }}
        name="selectedDatabase"
      />
      <Controller
        control={control}
        rules={{
          required: true,
        }}
        defaultValue={undefined}
        render={({ field: { onChange, value } }) => {
          return (
            <Form.Group data-test-id="selected-table-selector">
              <Form.Label htmlFor="selectedTable">Select Table</Form.Label>
              <Select
                options={tableOptions}
                placeholder={getTableListPlaceHolder(
                  isTableListLoading,
                  isDatabaseListLoading,
                  true,
                  tableLoadError,
                  databaseList,
                  tableList,
                  value
                )}
                // without explicitly set it to null, it shows the previous result,
                // even if not in the list and the result is undefined
                value={tableOptions.find((x) => x.value === value) ?? null}
                id="selectedTable"
                data-test-id={'table-selector'}
                onChange={(obj) => {
                  onChange(obj?.value);
                }}
              />
            </Form.Group>
          );
        }}
        name="selectedTable"
      />
    </>
  );
};

export { FromTableFormSection };
