import '@testing-library/jest-dom';
import {
  configure,
  render,
  screen,
  fireEvent,
  waitFor,
} from '@testing-library/react';
import * as React from 'react';
import { Cron } from './Cron';

configure({ testIdAttribute: 'data-test-id' });

const hourlyCron = '0 * * * *';

describe('Cron component', () => {
  it('renders without error', () => {
    render(<Cron value={hourlyCron} onChange={() => {}} />);
  });

  it('displays a list of inputs corresponding to the cron value', async () => {
    render(<Cron value={hourlyCron} onChange={() => {}} />);

    expect(screen.getByTestId('dropdown-input-period')).toHaveTextContent(
      'hour'
    );
    expect(screen.getByTestId('dropdown-input-minutes')).toHaveTextContent(
      '00'
    );
  });

  describe('when selecting the year period', () => {
    it('displays all the possible inputs', async () => {
      render(<Cron value={hourlyCron} onChange={() => {}} />);

      fireEvent.click(screen.getByTestId('dropdown-input-period'));
      await waitFor(() => {
        expect(screen.getByTestId('popover-year')).toBeInTheDocument();
      });
      fireEvent.click(screen.getByTestId('popover-year'));

      expect(screen.getByTestId('dropdown-input-period')).toHaveTextContent(
        'year'
      );
      expect(screen.getByTestId('dropdown-input-months')).toHaveTextContent(
        'every month'
      );
      expect(screen.getByTestId('dropdown-input-monthDays')).toHaveTextContent(
        'every day of the month'
      );
      expect(screen.getByTestId('dropdown-input-weekDays')).toHaveTextContent(
        'every day of the week'
      );
      expect(screen.getByTestId('dropdown-input-hours')).toHaveTextContent(
        'hour'
      );
      expect(screen.getByTestId('dropdown-input-minutes')).toHaveTextContent(
        '00'
      );
    });
  });

  it('allows the user to select multiple minutes', async () => {
    const mockOnChange = jest.fn();

    render(<Cron value={hourlyCron} onChange={mockOnChange} />);

    fireEvent.click(screen.getByTestId('dropdown-input-minutes'));

    await waitFor(() => {
      expect(screen.getByTestId('popover-0')).toBeInTheDocument();
    });

    fireEvent.click(screen.getByTestId('popover-15'));
    expect(mockOnChange).toHaveBeenCalledWith('0,15 * * * *');
  });

  it('allows the user to select multiple days', async () => {
    const mockOnChange = jest.fn();
    render(<Cron value={hourlyCron} onChange={mockOnChange} />);

    fireEvent.click(screen.getByTestId('dropdown-input-period'));
    await waitFor(() => {
      expect(screen.getByTestId('popover-week')).toBeInTheDocument();
    });
    fireEvent.click(screen.getByTestId('popover-week'));

    fireEvent.click(screen.getByTestId('dropdown-input-weekDays'));
    await waitFor(() => {
      expect(screen.getByTestId('popover-1')).toBeInTheDocument();
    });
    fireEvent.click(screen.getByTestId('popover-1'));

    expect(mockOnChange).toHaveBeenCalledWith('0 * * * 1');
  });
});
