import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import { ToggleDropdownInput } from 'components/toggle-dropdown-input/ToggleDropdownInput';
import React, { useMemo } from 'react';
import { CronUnitType, UNITS, UNIT_TYPE_LABELS } from './constants';
import { partToString } from './converter';

interface CronPartInputProps
  extends Omit<
    React.ComponentProps<typeof ClouderaButton>,
    'value' | 'onChange'
  > {
  unitType: CronUnitType;
  value: number[];
  onChange: (value: number[], unit: CronUnitType) => void;
}

/**
 * A wrapper around ClouderaToggleDropdownInput to handle cron units specifically
 * @param unitType the type of the cron unit (year, month, week, etc.)
 * @param value a list of the currently selected options
 * @param onChange callback invoked when the selection changes
 */
export const CronPartInput = ({
  unitType,
  value,
  onChange,
}: CronPartInputProps): JSX.Element => {
  const unit = useMemo(
    () => UNITS.find((item) => item.type === unitType)!,
    [unitType]
  );

  return (
    <ToggleDropdownInput
      id={unitType}
      label={
        !!value.length
          ? partToString(value, unit, true)
          : `every ${UNIT_TYPE_LABELS[unitType]}`
      }
      options={unit.options}
      value={value}
      onChange={(value) => onChange(value, unitType)}
      isMulti
      columnCount={unit.columnCount}
    />
  );
};
