const WEEKDAYS = [
  'Sunday',
  'Monday',
  'Tuesday',
  'Wednesday',
  'Thursday',
  'Friday',
  'Saturday',
];

const MONTHS = [
  'January',
  'February',
  'March',
  'April',
  'May',
  'June',
  'July',
  'August',
  'September',
  'October',
  'November',
  'December',
];

export enum CronPeriod {
  Minute = 'minute',
  Hour = 'hour',
  Day = 'day',
  Week = 'week',
  Month = 'month',
  Year = 'year',
}

export enum CronUnitType {
  Minutes = 'minutes',
  Hours = 'hours',
  MonthDays = 'monthDays',
  Months = 'months',
  WeekDays = 'weekDays',
}

export interface Unit {
  type: CronUnitType;
  min: number;
  max: number;
  total: number;
  options: { value: number; label: string }[];
  columnCount?: number;
}

export const periodOptions = Object.values(CronPeriod).map((period) => ({
  value: period,
  label: period,
}));

export const UNIT_TYPE_LABELS: Record<CronUnitType, string> = {
  [CronUnitType.Minutes]: 'minute',
  [CronUnitType.Hours]: 'hour',
  [CronUnitType.MonthDays]: 'day of the month',
  [CronUnitType.Months]: 'month',
  [CronUnitType.WeekDays]: 'day of the week',
};

export const UNITS: Unit[] = [
  {
    type: CronUnitType.Minutes,
    min: 0,
    max: 59,
    total: 60,
    options: Array.from({ length: 60 }, (_, i) => ({
      value: i,
      label: String(i),
    })),
    columnCount: 4,
  },
  {
    type: CronUnitType.Hours,
    min: 0,
    max: 23,
    total: 24,
    options: Array.from({ length: 24 }).map((_, i) => ({
      value: i,
      label: String(i),
    })),
    columnCount: 4,
  },
  {
    type: CronUnitType.MonthDays,
    min: 1,
    max: 31,
    total: 31,
    options: Array.from({ length: 31 }, (_, i) => ({
      value: i + 1,
      label: String(i + 1),
    })),
    columnCount: 4,
  },
  {
    type: CronUnitType.Months,
    min: 1,
    max: 12,
    total: 12,
    options: Array.from({ length: 12 }, (_, i) => ({
      value: i + 1,
      label: MONTHS[i],
    })),
    columnCount: 3,
  },
  {
    type: CronUnitType.WeekDays,
    min: 0,
    max: 6,
    total: 7,
    options: Array.from({ length: 7 }, (_, i) => ({
      value: i,
      label: WEEKDAYS[i],
    })),
  },
];
