import React, { useState } from 'react';

import { useLegacyBridge } from 'services/legacy-bridge';

import { DataConnectionModal } from 'components/data-connection-modal/DataConnectionModal';
import { ConnectionType } from 'components/data-connection-modal/utils/dataconnection-schema';

type DataConnectionModalWrapperProps = {
  allConnectionTypes: { [Key in ConnectionType]: string };
};

export const DataConnectionModalWrapper = ({
  allConnectionTypes,
}: DataConnectionModalWrapperProps): JSX.Element => {
  const [isDataConnectionModalOpen, setIsDataConnectionModalOpen] =
    useState(false);
  const [dataconnectionId, setDataconnectionId] = useState<number | null>(null);
  const modalType = dataconnectionId ? 'existing' : 'new';
  const [associatedDatasetLength, setAssociatedDatasetLength] = useState(0);

  useLegacyBridge(
    'open-data-connection-modal',
    ({
      payload,
    }: {
      payload?: {
        dataconnectionId: number;
        associatedDatasetLength: number;
      };
    }) => {
      if (!payload) return;
      const { dataconnectionId, associatedDatasetLength } = payload;

      setDataconnectionId(dataconnectionId);
      setAssociatedDatasetLength(associatedDatasetLength);
      setIsDataConnectionModalOpen(true);
    }
  );

  return (
    <DataConnectionModal
      isShow={isDataConnectionModalOpen}
      dataconnectionId={dataconnectionId}
      associatedDatasetLength={associatedDatasetLength}
      allConnectionTypes={allConnectionTypes}
      modalType={modalType}
      onClose={() => {
        setIsDataConnectionModalOpen(false);
      }}
    />
  );
};
