import React from 'react';
import { Col, Form, Row, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import { FieldPath, useFormContext } from 'react-hook-form';
import { DataConnectionSchemaType } from '../utils/dataconnection-schema';
import { get } from 'lodash-es';
import classes from '../DataConnectionModal.module.scss';

type DataconnectionCheckboxProps = {
  config: {
    type: string;
    label: string;
    placeholder?: string;
    info?: string;
    options?: { id: string; value: string; label: string }[];
    checkbox_label?: string;
  };
  fieldId: FieldPath<DataConnectionSchemaType>;
  impersonation?: boolean;
  trustedImpersonation?: boolean;
};

export const DataconnectionCheckbox = ({
  config,
  fieldId,
  impersonation,
  trustedImpersonation,
}: DataconnectionCheckboxProps) => {
  const {
    register,
    formState: { errors },
  } = useFormContext<DataConnectionSchemaType>();

  const isUserDependentCacheDisabled =
    fieldId.endsWith('user_dependent_cache') &&
    !(impersonation || trustedImpersonation);

  const error = get(errors, fieldId);

  return (
    <Form.Group as={Row} className="mb-3 align-items-center">
      <Form.Label htmlFor={fieldId} column sm="3" className="text-right">
        {config.label}{' '}
        {config.info && (
          <VizTooltip
            placement="top"
            overlay={
              <Popover id={`${fieldId}-tooltip`}>
                <span className="d-flex p-3">{config.info}</span>
              </Popover>
            }
          >
            <span
              className={classNames(classes.infoIcon, 'fa fa-info-circle')}
            />
          </VizTooltip>
        )}
      </Form.Label>
      <Col sm={9}>
        <Form.Check
          type="checkbox"
          label={config.checkbox_label}
          data-test-id={fieldId}
          id={fieldId}
          {...register(fieldId)}
          isInvalid={!!error}
          disabled={
            isUserDependentCacheDisabled ||
            (impersonation && fieldId.endsWith('trusted_impersonation')) ||
            (trustedImpersonation &&
              fieldId.endsWith('impersonation') &&
              !fieldId.endsWith('trusted_impersonation'))
          }
        />

        {error && typeof error.message === 'string' && (
          <Form.Control.Feedback type="invalid">
            {error.message}
          </Form.Control.Feedback>
        )}
      </Col>
    </Form.Group>
  );
};
