import React from 'react';
import { ConnectionType } from '../utils/dataconnection-schema';
import { DataconnectionInput } from './DataconnectionInput';
import { DataconnectionRadio } from './DataconnectionRadio';
import { DataconnectionCheckbox } from './DataconnectionCheckbox';
import { DataConnectionConfig } from '../constants/dataconnection-constants';

type DataconnectionDataTabProps = {
  dataFields: DataConnectionConfig[keyof DataConnectionConfig]['data'];
  connectionType: ConnectionType;
};

export const DataconnectionDataTab = ({
  dataFields,
  connectionType,
}: DataconnectionDataTabProps): JSX.Element | null => {
  return (
    <>
      {Object.keys(dataFields).map((key) => {
        const fieldId = `${connectionType}.data.${
          key as keyof typeof dataFields
        }` as const;

        const { config } = dataFields[key];

        switch (config.type) {
          case 'input':
            return (
              <DataconnectionInput
                key={key}
                config={config}
                fieldId={fieldId}
              />
            );

          case 'radio':
            return (
              <DataconnectionRadio
                key={key}
                config={config}
                fieldId={fieldId}
              />
            );
          case 'checkbox':
            return (
              <DataconnectionCheckbox
                key={key}
                config={config}
                fieldId={fieldId}
              />
            );
          default:
            return null;
        }
      })}
    </>
  );
};
