import React from 'react';
import { Col, Form, Row, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import { FieldPath, useFormContext } from 'react-hook-form';
import classes from '../DataConnectionModal.module.scss';
import Select from 'react-select';
import { DataConnectionSchemaType } from '../utils/dataconnection-schema';
import { get } from 'lodash-es';

type DataconnectionDropdownProps = {
  config: {
    type: string;
    label: string;
    placeholder?: string;
    info?: string;
    options?: { id: string; value: string; label: string }[];
    checkbox_label?: string;
  };
  fieldId: FieldPath<DataConnectionSchemaType>;
  options: {
    value: string;
    label: string;
  }[];
  placeholder: string;
};

export const DataconnectionDropdown = ({
  config,
  fieldId,
  options,
  placeholder,
}: DataconnectionDropdownProps) => {
  const {
    setValue,
    formState: { errors },
  } = useFormContext<DataConnectionSchemaType>();

  const error = get(errors, fieldId);

  return (
    <Form.Group as={Row} className="mb-3 align-items-center">
      <Form.Label
        htmlFor={fieldId}
        column
        sm="3"
        className={classNames('text-right', classes.modalLabel)}
      >
        {config.label}{' '}
        {config.info && (
          <VizTooltip
            placement="top"
            overlay={
              <Popover id={`${fieldId}-tooltip`}>
                <span className="d-flex p-3">{config.info}</span>
              </Popover>
            }
          >
            <span
              className={classNames(classes.infoIcon, 'fa fa-info-circle')}
            />
          </VizTooltip>
        )}
      </Form.Label>
      <Col sm={9}>
        <div>
          <Select
            id={fieldId}
            options={options}
            isSearchable
            placeholder={placeholder}
            value={options.find((option) => option?.label === config.label)}
            onChange={(option) => {
              setValue(fieldId, option?.value ?? '');
            }}
          />
        </div>

        {error && typeof error.message === 'string' && (
          <Form.Control.Feedback type="invalid">
            {error.message}
          </Form.Control.Feedback>
        )}
      </Col>
    </Form.Group>
  );
};
