import React from 'react';
import { ConnectionType } from '../utils/dataconnection-schema';
import type { DataConnectionConfig } from '../constants/dataconnection-constants';
import { checkHasArcengineHistoryETL } from '../utils/dataconnection-type-guards';
import { DataconnectionInput } from './DataconnectionInput';
import { DataconnectionCheckbox } from './DataconnectionCheckbox';
import { DataconnectionRadio } from './DataconnectionRadio';
import { DataconnectionDropdown } from './DataconnectionDropdown';

type DataconnectionHistoryTabProps = {
  historyFields: DataConnectionConfig[keyof DataConnectionConfig]['history'];
  connectionType: ConnectionType;
};

export const DataconnectionHistoryTab = ({
  historyFields,
  connectionType,
}: DataconnectionHistoryTabProps): JSX.Element | null => {
  if (!historyFields) return null;

  return (
    <>
      {Object.keys(historyFields).map((key) => {
        if (!checkHasArcengineHistoryETL(connectionType)) return null;
        const fieldId = `${connectionType}.history.${
          key as keyof typeof historyFields
        }` as const;

        const { config } = historyFields[key];

        switch (config.type) {
          case 'input':
            return (
              <DataconnectionInput
                key={key}
                fieldId={fieldId}
                config={config}
              />
            );
          case 'checkbox':
            return (
              <DataconnectionCheckbox
                key={key}
                config={config}
                fieldId={fieldId}
              />
            );
          case 'radio':
            return (
              config.options && (
                <DataconnectionRadio
                  key={key}
                  config={config}
                  fieldId={fieldId}
                />
              )
            );
          case 'dropdown':
            return (
              config.options && (
                <DataconnectionDropdown
                  key={key}
                  config={config}
                  fieldId={fieldId}
                  options={config.options}
                  placeholder={'Select a schedule'}
                />
              )
            );
          default:
            return null;
        }
      })}
    </>
  );
};
