import React from 'react';
import { Col, Form, FormControl, Row, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import { FieldPath, useFormContext } from 'react-hook-form';
import { DataConnectionSchemaType } from '../utils/dataconnection-schema';
import { get } from 'lodash-es';
import classes from '../DataConnectionModal.module.scss';

type DataconnectionInputProps = {
  config: {
    type: string;
    label: string;
    placeholder?: string;
    info?: string;
    options?: { id: string; value: string; label: string }[];
    checkbox_label?: string;
  };
  fieldId: FieldPath<DataConnectionSchemaType>;
};

export const DataconnectionInput = ({
  config,
  fieldId,
}: DataconnectionInputProps) => {
  const {
    register,
    formState: { errors },
  } = useFormContext<DataConnectionSchemaType>();

  const error = get(errors, fieldId);

  return (
    <Form.Group as={Row} className="mb-3 align-items-center">
      <Form.Label htmlFor={fieldId} column sm="3" className="text-right">
        {config.label}{' '}
        {config.info && (
          <VizTooltip
            placement="top"
            overlay={
              <Popover id={`${fieldId}-tooltip`}>
                <span className="d-flex p-3">{config.info}</span>
              </Popover>
            }
          >
            <span
              className={classNames(classes.infoIcon, 'fa fa-info-circle')}
            />
          </VizTooltip>
        )}
      </Form.Label>
      <Col sm={9}>
        <FormControl
          id={fieldId}
          data-test-id={fieldId}
          type={config.type}
          placeholder={config.placeholder}
          {...register(fieldId)}
          isInvalid={!!error}
        />

        {error && typeof error.message === 'string' && (
          <Form.Control.Feedback type="invalid">
            {error.message}
          </Form.Control.Feedback>
        )}
      </Col>
    </Form.Group>
  );
};
