import React from 'react';
import { Form, FormControl, Table } from 'react-bootstrap';
import Delete from '@cloudera-internal/cuix-core/icons/svg/ic-delete.svg';
import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import { useFormContext, useFieldArray } from 'react-hook-form';
import {
  ConnectionType,
  DataConnectionSchemaType,
} from '../utils/dataconnection-schema';
import { checkHasParameters } from '../utils/dataconnection-type-guards';

type DataconnectionParametersTabProps = {
  connectionType: ConnectionType;
};

export const DataconnectionParametersTab = ({
  connectionType,
}: DataconnectionParametersTabProps): JSX.Element => {
  const {
    register,
    watch,
    formState: { errors },
  } = useFormContext<DataConnectionSchemaType>();

  const { fields, append, remove } = useFieldArray<DataConnectionSchemaType>({
    name: `${connectionType}.parameters`,
  });

  const watchedParameters = watch(`${connectionType}.parameters`);

  const isCurrentRowFilled =
    (watchedParameters?.length ?? 0) === 0 ||
    (watchedParameters?.[watchedParameters.length - 1]?.name?.trim() !== '' &&
      watchedParameters?.[watchedParameters.length - 1]?.value?.trim() !== '');

  return (
    <div>
      <Table striped bordered hover>
        <thead>
          <tr>
            <th>Parameter Name</th>
            <th>Parameter Value</th>
            <th></th>
          </tr>
        </thead>
        <tbody>
          {fields?.map((field, index) => {
            if (!checkHasParameters(connectionType)) return null;
            return (
              <tr key={field.id}>
                <td>
                  <FormControl
                    id={`parameters.${index}-name-id`}
                    {...register(`${connectionType}.parameters.${index}.name`)}
                    isInvalid={
                      !!errors[connectionType]?.parameters?.[index]?.name
                    }
                  />
                  <Form.Control.Feedback type="invalid">
                    {errors[connectionType]?.parameters?.[index]?.name?.message}
                  </Form.Control.Feedback>
                </td>
                <td>
                  <FormControl
                    id={`parameters.${index}-value-id`}
                    {...register(`${connectionType}.parameters.${index}.value`)}
                    isInvalid={
                      !!errors[connectionType]?.parameters?.[index]?.value
                    }
                  />
                  <Form.Control.Feedback type="invalid">
                    {
                      errors[connectionType]?.parameters?.[index]?.value
                        ?.message
                    }
                  </Form.Control.Feedback>
                </td>
                <td>
                  <Delete onClick={() => remove(index)} />
                </td>
              </tr>
            );
          })}
          <tr>
            <td>
              <span title="Please fill the parameter row before you add a new one">
                <ClouderaButton
                  variant="link"
                  onClick={() => append({ name: '', value: '' })}
                  disabled={!isCurrentRowFilled}
                >
                  Add new row
                </ClouderaButton>
              </span>
            </td>
            <td></td>
            <td></td>
          </tr>
        </tbody>
      </Table>
    </div>
  );
};
