import React from 'react';
import { useFormContext, useFieldArray } from 'react-hook-form';
import { Table, FormControl, Form } from 'react-bootstrap';
import Delete from '@cloudera-internal/cuix-core/icons/svg/ic-delete.svg';
import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import {
  ConnectionType,
  DataConnectionSchemaType,
} from '../utils/dataconnection-schema';
import { checkHasArcengineHistoryETL } from '../utils/dataconnection-type-guards';

type DataconnectionStorageLocationsTabProps = {
  connectionType: ConnectionType;
};

export const DataconnectionStorageLocationsTab = ({
  connectionType,
}: DataconnectionStorageLocationsTabProps) => {
  const {
    register,
    watch,
    formState: { errors },
  } = useFormContext<DataConnectionSchemaType>();

  const { fields, append, remove } = useFieldArray<DataConnectionSchemaType>({
    name: `${connectionType}.storageLocations`,
  });

  const watchedstorageLocations = watch(`${connectionType}.storageLocations`);

  const isCurrentRowFilled =
    (watchedstorageLocations?.length ?? 0) === 0 ||
    (watchedstorageLocations?.[
      watchedstorageLocations.length - 1
    ]?.name?.trim() !== '' &&
      watchedstorageLocations?.[
        watchedstorageLocations.length - 1
      ]?.value?.trim() !== '');

  return (
    <div>
      <Table striped bordered hover>
        <thead>
          <tr>
            <th>Location Name</th>
            <th>Location String</th>
            <th></th>
          </tr>
        </thead>
        <tbody>
          {fields.map((field, index) => {
            if (!checkHasArcengineHistoryETL(connectionType)) return null;

            return (
              <tr key={field.id}>
                <td>
                  <FormControl
                    id={`storageLocations.${index}-name-id`}
                    {...register(
                      `${connectionType}.storageLocations.${index}.name`
                    )}
                    isInvalid={
                      !!errors[connectionType]?.storageLocations?.[index]?.name
                    }
                  />
                  <Form.Control.Feedback type="invalid">
                    {
                      errors[connectionType]?.storageLocations?.[index]?.name
                        ?.message
                    }
                  </Form.Control.Feedback>
                </td>
                <td>
                  <FormControl
                    id={`storageLocations.${index}-value-id`}
                    {...register(
                      `${connectionType}.storageLocations.${index}.value`
                    )}
                    isInvalid={
                      !!errors[connectionType]?.storageLocations?.[index]?.value
                    }
                  />
                  <Form.Control.Feedback type="invalid">
                    {
                      errors[connectionType]?.storageLocations?.[index]?.value
                        ?.message
                    }
                  </Form.Control.Feedback>
                </td>
                <td>
                  <Delete onClick={() => remove(index)} />
                </td>
              </tr>
            );
          })}
          <tr>
            <td>
              <span title="Please fill the storage location row before you add a new one">
                <ClouderaButton
                  variant="link"
                  onClick={() => append({ name: '', value: '' })}
                  disabled={!isCurrentRowFilled}
                >
                  Add new row
                </ClouderaButton>
              </span>
            </td>
            <td></td>
            <td></td>
          </tr>
        </tbody>
      </Table>
    </div>
  );
};
