const DATACONN_SETTINGS_LABEL = {
  basic: {
    hostname: 'Hostname or IP address',
    port: 'Port #',
    database: 'Database',
    warehouse: 'Warehouse',
    solrType: 'Type',
    array_join_char: 'Array Join Char',
    username: 'Username',
    password: 'Password',
    role: 'Role',
    account_name: 'Account Name',
    file_name: 'File name',
    dsn: 'DSN',
    service_account_key: 'Service Account Key',
    billing_project: 'Billing Project',
    data_project: 'Data Project',
    region_name: 'Region Name',
    s3_staging_directory: 'S3 Staging Directory',
    access_key_id: 'Access Key ID',
    secret_access_key: 'Secret Access Key',
    authentication: 'Authentication',
  },
  advanced: {
    connection_mode: 'Connection mode',
    http_path: 'HTTP Path',
    access_token: 'Access Token',
    socket_type: 'Socket type',
    hostname_mismatch: '',
    authentication_mode: 'Authentication mode',
    kerberos_service_name: 'Kerberos service name',
    query_timeout: 'Query Timeout',
    session_timeout: 'Session Timeout',
    socket_timeout: 'Socket timeout',
    queue_depth: 'Queue Depth',
    impersonation: 'Impersonation',
    trusted_impersonation: 'Trusted Impersonation',
    user_dependent_cache: 'User-dependent cache',
    application_name: 'Application name',
    fallback: 'Fallback',
  },
  data: {
    concurrency: 'Concurrency',
    concurrency_per_person: 'Concurrency Per Person',
    query_timeout: 'Query Timeout (Minutes)',
    query_loading_warning: 'Query Still Loading Warning (Seconds)',
    row_upload_limit: 'Row Upload Limit',
    result_cache: 'Result Cache',
    cache_retention_timeout: 'Cache Retention Timeout (Seconds)',
  },
  history: {
    history_etl: 'History ETL',
    target_table: 'Target Table',
    job_name: 'Job Name',
    schedule: 'Schedule',
  },
};

export const DATACONN_SETTINGS = {
  basic: {
    hostname: {
      label: DATACONN_SETTINGS_LABEL['basic']['hostname'],
      type: 'input',
      placeholder: 'Enter IP address of the server where your data resides',
      info: '(example: prod_db.yourcompany.com or 10.0.1.20)',
    },
    port: {
      label: DATACONN_SETTINGS_LABEL['basic']['port'],
      type: 'input',
    },
    warehouse: {
      label: DATACONN_SETTINGS_LABEL['basic']['warehouse'],
      type: 'input',
    },
    solrType: {
      label: DATACONN_SETTINGS_LABEL['basic']['solrType'],
      type: 'radio',

      options: [
        {
          id: 'solrTypeCloud',
          label: 'Cloud',
          value: 'cloud',
        },
        {
          id: 'solrTypeStandAlone',
          label: 'Standalone',
          value: 'standalone',
        },
      ],
    },
    array_join_char: {
      label: DATACONN_SETTINGS_LABEL['basic']['array_join_char'],
      type: 'input',
    },
    database: {
      label: DATACONN_SETTINGS_LABEL['basic']['database'],
      type: 'input',
    },
    username: {
      label: DATACONN_SETTINGS_LABEL['basic']['username'],
      type: 'input',
    },
    password: {
      label: DATACONN_SETTINGS_LABEL['basic']['password'],
      type: 'password',
    },
    role: {
      label: DATACONN_SETTINGS_LABEL['basic']['role'],
      type: 'input',
    },
    account_name: {
      label: DATACONN_SETTINGS_LABEL['basic']['account_name'],
      type: 'input',
      placeholder: 'Enter your Snowflake account name here',
      info: '(example: uc00141.us-east-1)',
    },
    file_name: {
      label: DATACONN_SETTINGS_LABEL['basic']['file_name'],
      type: 'input',
    },
    dsn: {
      label: DATACONN_SETTINGS_LABEL['basic']['dsn'],
      type: 'input',
    },
    service_account_key: {
      label: DATACONN_SETTINGS_LABEL['basic']['service_account_key'],
      type: 'textarea',
      placeholder:
        'Paste in the contents of your Service Account Key File here',
    },
    billing_project: {
      label: DATACONN_SETTINGS_LABEL['basic']['billing_project'],
      type: 'input',
    },
    data_project: {
      label: DATACONN_SETTINGS_LABEL['basic']['data_project'],
      type: 'input',
    },
    region_name: {
      label: DATACONN_SETTINGS_LABEL['basic']['region_name'],
      type: 'input',
    },
    s3_staging_directory: {
      label: DATACONN_SETTINGS_LABEL['basic']['s3_staging_directory'],
      type: 'input',
    },
    access_key_id: {
      label: DATACONN_SETTINGS_LABEL['basic']['access_key_id'],
      type: 'input',
    },
    secret_access_key: {
      label: DATACONN_SETTINGS_LABEL['basic']['secret_access_key'],
      type: 'input',
    },
    authentication: {
      label: DATACONN_SETTINGS_LABEL['basic']['authentication'],
      type: 'radio',

      options: [
        {
          id: 'conAuthNoAuthentication',
          label: 'No Authentication',
          value: 'No Authentication',
        },
        {
          id: 'conAuthBasicAuthentication',
          label: 'Basic Authentication',
          value: 'Basic Authentication',
        },
      ],
    },
  },

  advanced: {
    connection_mode: {
      label: DATACONN_SETTINGS_LABEL['advanced']['connection_mode'],
      type: 'radio',

      options: [
        {
          id: 'binaryConMode',
          label: 'Binary',
          value: 'binary',
        },
        {
          id: 'httpConMode',
          label: 'HTTP',
          value: 'http',
        },
      ],
    },
    http_path: {
      label: DATACONN_SETTINGS_LABEL['advanced']['http_path'],
      type: 'input',
      placeholder: 'SQL path (default cliservice)',
    },
    access_token: {
      label: DATACONN_SETTINGS_LABEL['advanced']['access_token'],
      type: 'input',
      placeholder: 'Access token (optional)',
    },
    socket_type: {
      label: DATACONN_SETTINGS_LABEL['advanced']['socket_type'],
      type: 'radio',

      options: [
        {
          id: 'normalSockType',
          label: 'Normal',
          value: 'normal',
        },
        {
          id: 'sslSockType',
          label: 'SSL',
          value: 'ssl',
        },
        {
          id: 'sslSockTypeWCertificate',
          label: 'SSL with certificate',
          value: 'sslcert',
        },
      ],
    },
    query_timeout: {
      label: DATACONN_SETTINGS_LABEL['advanced']['query_timeout'],
      type: 'input',
    },
    session_timeout: {
      label: DATACONN_SETTINGS_LABEL['advanced']['session_timeout'],
      type: 'input',
    },
    queue_depth: {
      label: DATACONN_SETTINGS_LABEL['advanced']['queue_depth'],
      type: 'input',
    },
    hostname_mismatch: {
      label: DATACONN_SETTINGS_LABEL['advanced']['hostname_mismatch'],
      type: 'checkbox',
      checkbox_label: 'Allow Common Name - Host Name Mismatch',
    },
    authentication_mode: {
      label: DATACONN_SETTINGS_LABEL['advanced']['authentication_mode'],
      type: 'radio',

      options: [
        {
          id: 'nosaslConAuthType',
          label: 'NoSasl',
          value: 'nosasl',
        },
        {
          id: 'plainConAuthType',
          label: 'Plain',
          value: 'plain',
        },
        {
          id: 'ldapConAuthType',
          label: 'LDAP',
          value: 'ldap',
        },
        {
          id: 'kerberosConAuthType',
          label: 'Kerberos',
          value: 'kerberos',
        },
      ],
    },
    kerberos_service_name: {
      label: DATACONN_SETTINGS_LABEL['advanced']['kerberos_service_name'],
      type: 'input',

      placeholder: 'Kerberos service name',
    },
    socket_timeout: {
      label: DATACONN_SETTINGS_LABEL['advanced']['socket_timeout'],
      type: 'input',
    },
    impersonation: {
      label: DATACONN_SETTINGS_LABEL['advanced']['impersonation'],
      type: 'checkbox',
      checkbox_label: 'Enabled',
      info: 'If configured correctly on the connection backend, impersonation uses the impala.doas.user/hive.server2.proxy.user connection property to run queries as the user who is logged in to Cloudera Data Visualization even as the connection is made as a different (trusted) user. It cannot be enabled at the same time as "Trusted Impersonation"',
    },
    trusted_impersonation: {
      label: DATACONN_SETTINGS_LABEL['advanced']['trusted_impersonation'],
      type: 'checkbox',
      checkbox_label: 'Enabled',
      info: 'Trusted Impersonation works in secure connection contexts (like in Cloudera Data Warehouse) to create data connections and run queries as the user who is logged in to DataViz. It cannot be enabled at the same time as "Impersonation"',
    },
    user_dependent_cache: {
      label: DATACONN_SETTINGS_LABEL['advanced']['user_dependent_cache'],
      type: 'checkbox',
    },
    application_name: {
      label: DATACONN_SETTINGS_LABEL['advanced']['application_name'],
      type: 'input',
    },
    fallback: {
      label: DATACONN_SETTINGS_LABEL['advanced']['fallback'],
      type: 'dropdown',
    },
  },

  parameters: [],

  data: {
    concurrency: {
      label: DATACONN_SETTINGS_LABEL['data']['concurrency'],
      type: 'input',
      info: 'Enter the maximum number of requests that will be sent to this data connection concurrently.',
    },
    concurrency_per_person: {
      label: DATACONN_SETTINGS_LABEL['data']['concurrency_per_person'],
      type: 'input',
      info: 'Enter the maximum number of requests that a single user can send to this data connection concurrently.',
    },
    query_timeout: {
      label: DATACONN_SETTINGS_LABEL['data']['query_timeout'],
      type: 'input',
      info: 'Queries are permitted to run for a maximum amount of time before they are canceled. This time can be configured per connection. Enter that maximum time in minutes here.',
    },
    query_loading_warning: {
      label: DATACONN_SETTINGS_LABEL['data']['query_loading_warning'],
      type: 'input',
    },
    row_upload_limit: {
      label: DATACONN_SETTINGS_LABEL['data']['row_upload_limit'],
      type: 'input',
      info: 'Enter the maximum number of rows that will be inserted into rows in this connection via the data import and data extract workflows. A value of 0 means that data cannot be inserted into this connection.',
    },
    result_cache: {
      label: DATACONN_SETTINGS_LABEL['data']['result_cache'],
      type: 'checkbox',
      checkbox_label: 'Enabled',
    },
    cache_retention_timeout: {
      label: DATACONN_SETTINGS_LABEL['data']['cache_retention_timeout'],
      type: 'input',
    },
  },

  history: {
    history_etl: {
      label: DATACONN_SETTINGS_LABEL['history']['history_etl'],
      type: 'checkbox',
    },
    target_table: {
      label: DATACONN_SETTINGS_LABEL['history']['target_table'],
      type: 'input',
    },
    job_name: {
      label: DATACONN_SETTINGS_LABEL['history']['job_name'],
      type: 'input',
    },
    schedule: {
      label: DATACONN_SETTINGS_LABEL['history']['schedule'],
      type: 'dropdown',
      options: [
        {
          id: 'weekly',
          label: 'Weekly',
          value: 'Weekly',
        },
        {
          id: 'daily',
          label: 'Daily',
          value: 'Daily',
        },
        {
          id: 'hourly',
          label: 'Hourly',
          value: 'Hourly',
        },
      ],
    },
  },

  storageLocations: [],
};

export type DataConnectionConfig = typeof DATACONN_CONFIG;
// specific configuration per dataconnection
export const DATACONN_CONFIG = {
  'impyla:cdw-impyla': {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      application_name: {
        config: DATACONN_SETTINGS.advanced.application_name,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  impyla: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      application_name: {
        config: DATACONN_SETTINGS.advanced.application_name,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  impala: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      application_name: {
        config: DATACONN_SETTINGS.advanced.application_name,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  'hyve:cdw-hyve': {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      access_token: {
        config: DATACONN_SETTINGS.advanced.access_token,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      application_name: {
        config: DATACONN_SETTINGS.advanced.application_name,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  hyve: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      access_token: {
        config: DATACONN_SETTINGS.advanced.access_token,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      application_name: {
        config: DATACONN_SETTINGS.advanced.application_name,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  hive: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      access_token: {
        config: DATACONN_SETTINGS.advanced.access_token,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      application_name: {
        config: DATACONN_SETTINGS.advanced.application_name,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  athena: {
    basic: {
      region_name: {
        config: DATACONN_SETTINGS.basic.region_name,
      },
      s3_staging_directory: {
        config: DATACONN_SETTINGS.basic.s3_staging_directory,
      },
      access_key_id: {
        config: DATACONN_SETTINGS.basic.access_key_id,
      },
      secret_access_key: {
        config: DATACONN_SETTINGS.basic.secret_access_key,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  cassandra: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  arcengine: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      query_timeout: {
        config: DATACONN_SETTINGS.advanced.query_timeout,
      },
      session_timeout: {
        config: DATACONN_SETTINGS.advanced.session_timeout,
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      queue_depth: {
        config: DATACONN_SETTINGS.advanced.queue_depth,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      fallback: {
        config: DATACONN_SETTINGS.advanced.fallback,
      },
    },
    parameters: [],
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: {
      history_etl: {
        config: DATACONN_SETTINGS.history.history_etl,
      },
      target_table: {
        config: DATACONN_SETTINGS.history.target_table,
      },
      job_name: {
        config: DATACONN_SETTINGS.history.job_name,
      },
      schedule: {
        config: DATACONN_SETTINGS.history.schedule,
      },
    },
    storageLocations: [] as { name: string; value: string }[],
  },
  arcengine_v2: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
      fallback: {
        config: DATACONN_SETTINGS.advanced.fallback,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: {
      history_etl: {
        config: DATACONN_SETTINGS.history.history_etl,
      },
      target_table: {
        config: DATACONN_SETTINGS.history.target_table,
      },
      job_name: {
        config: DATACONN_SETTINGS.history.job_name,
      },
      schedule: {
        config: DATACONN_SETTINGS.history.schedule,
      },
    },
    storageLocations: [] as { name: string; value: string }[],
  },
  aster: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      dsn: {
        config: DATACONN_SETTINGS.basic.dsn,
      },
      authentication: {
        config: DATACONN_SETTINGS.basic.authentication,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  ksql: {
    basic: {
      hostname: {
        config: {
          label: DATACONN_SETTINGS_LABEL['basic']['hostname'],
          type: 'input',
          placeholder: 'KSQL REST API (http[s]://192.168.1.1)',
        },
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  denodo: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  duckdb: {
    basic: {
      file_name: {
        config: DATACONN_SETTINGS.basic.file_name,
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  druid: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  drill: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      dsn: {
        config: DATACONN_SETTINGS.basic.dsn,
      },
      authentication: {
        config: DATACONN_SETTINGS.basic.authentication,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  elasticsearch: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  bigquery: {
    basic: {
      service_account_key: {
        config: DATACONN_SETTINGS.basic.service_account_key,
      },
      billing_project: {
        config: DATACONN_SETTINGS.basic.billing_project,
      },
      data_project: {
        config: DATACONN_SETTINGS.basic.billing_project,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  mariadb: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  mssqlserver: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  mysql: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  oracle: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  postgres: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  snowflake: {
    basic: {
      account_name: {
        config: DATACONN_SETTINGS.basic.account_name,
      },
      warehouse: {
        config: DATACONN_SETTINGS.basic.warehouse,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
      role: {
        config: DATACONN_SETTINGS.basic.role,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
  },
  solr: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      solrType: {
        config: DATACONN_SETTINGS.basic.solrType,
      },
      array_join_char: {
        config: DATACONN_SETTINGS.basic.array_join_char,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: { config: DATACONN_SETTINGS.advanced.socket_timeout },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  sparksql: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      access_token: {
        config: DATACONN_SETTINGS.advanced.access_token,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
    },
    parameters: [],
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  'postgres:ssb': {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  sqlite: {
    basic: {
      file_name: {
        config: DATACONN_SETTINGS.basic.file_name,
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  redshift: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
    },
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  teradata: {
    basic: {
      dsn: {
        config: DATACONN_SETTINGS.basic.dsn,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  vertica: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      database: {
        config: DATACONN_SETTINGS.basic.database,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    data: {
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    advanced: undefined,
    history: undefined,
    storageLocations: undefined,
    parameters: undefined,
  },
  trino: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
    },
    parameters: undefined,
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
  phoenix: {
    basic: {
      hostname: {
        config: DATACONN_SETTINGS.basic.hostname,
      },
      port: {
        config: DATACONN_SETTINGS.basic.port,
      },
      username: {
        config: DATACONN_SETTINGS.basic.username,
      },
      password: {
        config: DATACONN_SETTINGS.basic.password,
      },
    },
    advanced: {
      connection_mode: {
        config: DATACONN_SETTINGS.advanced.connection_mode,
      },
      http_path: {
        config: DATACONN_SETTINGS.advanced.http_path,
      },
      socket_type: {
        config: DATACONN_SETTINGS.advanced.socket_type,
      },
      hostname_mismatch: {
        config: DATACONN_SETTINGS.advanced.hostname_mismatch,
        show: 'sslSockTypeWCertificate',
      },
      authentication_mode: {
        config: DATACONN_SETTINGS.advanced.authentication_mode,
      },
      kerberos_service_name: {
        config: DATACONN_SETTINGS.advanced.kerberos_service_name,
        show: 'kerberosConAuthType',
      },
      socket_timeout: {
        config: DATACONN_SETTINGS.advanced.socket_timeout,
      },
      impersonation: {
        config: DATACONN_SETTINGS.advanced.impersonation,
      },
      trusted_impersonation: {
        config: DATACONN_SETTINGS.advanced.trusted_impersonation,
      },
      user_dependent_cache: {
        config: DATACONN_SETTINGS.advanced.user_dependent_cache,
      },
    },
    parameters: undefined,
    data: {
      concurrency: {
        config: DATACONN_SETTINGS.data.concurrency,
      },
      concurrency_per_person: {
        config: DATACONN_SETTINGS.data.concurrency_per_person,
      },
      query_timeout: {
        config: DATACONN_SETTINGS.data.query_timeout,
      },
      query_loading_warning: {
        config: DATACONN_SETTINGS.data.query_loading_warning,
      },
      row_upload_limit: {
        config: DATACONN_SETTINGS.data.row_upload_limit,
      },
      result_cache: {
        config: DATACONN_SETTINGS.data.result_cache,
      },
      cache_retention_timeout: {
        config: DATACONN_SETTINGS.data.cache_retention_timeout,
      },
    },
    history: undefined,
    storageLocations: undefined,
  },
};
