import { ConnectionType } from './dataconnection-schema';

export const checkHasAdvancedTab = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine'
  | 'arcengine_v2'
  | 'denodo'
  | 'druid'
  | 'postgres'
  | 'solr'
  | 'sparksql'
  | 'postgres:ssb'
  | 'redshift'
  | 'trino'
  | 'phoenix'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine',
    'arcengine_v2',
    'denodo',
    'druid',
    'postgres',
    'solr',
    'sparksql',
    'postgres:ssb',
    'redshift',
    'trino',
    'phoenix',
  ].includes(type);

export const checkHasParametersTab = (
  type: ConnectionType
): type is Exclude<
  ConnectionType,
  | 'athena'
  | 'aster'
  | 'cassandra'
  | 'ksql'
  | 'denodo'
  | 'duckdb'
  | 'druid'
  | 'drill'
  | 'elasticsearch'
  | 'bigquery'
  | 'mariadb'
  | 'mssqlserver'
  | 'mysql'
  | 'oracle'
  | 'postgres'
  | 'postgres:ssb'
  | 'sqlite'
  | 'redshift'
  | 'teradata'
  | 'vertica'
> =>
  ![
    'athena',
    'aster',
    'cassandra',
    'ksql',
    'denodo',
    'duckdb',
    'druid',
    'drill',
    'elasticsearch',
    'bigquery',
    'mariadb',
    'mssqlserver',
    'mysql',
    'oracle',
    'postgres',
    'postgres:ssb',
    'sqlite',
    'redshift',
    'teradata',
    'vertica',
  ].includes(type);

export const checkHasSocketType = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine'
  | 'arcengine_v2'
  | 'denodo'
  | 'druid'
  | 'solr'
  | 'postgres'
  | 'postgres:ssb'
  | 'sparksql'
  | 'redshift'
  | 'phoenix'
  | 'trino'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine',
    'arcengine_v2',
    'denodo',
    'druid',
    'solr',
    'postgres',
    'postgres:ssb',
    'sparksql',
    'redshift',
    'phoenix',
    'trino',
  ].includes(type);

export const checkHasConnectionMode = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'sparksql'
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'phoenix'
> =>
  [
    'sparksql',
    'impyla:cdw-impyla',
    'impyla',
    'hyve:cdw-hyve',
    'hyve',
    'phoenix',
  ].includes(type);

export const checkHasAuthenticationMode = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine'
  | 'arcengine_v2'
  | 'solr'
  | 'sparksql'
  | 'phoenix'
  | 'trino'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine',
    'arcengine_v2',
    'solr',
    'sparksql',
    'phoenix',
    'trino',
  ].includes(type);

export const checkHasFallBack = (
  type: ConnectionType
): type is Extract<ConnectionType, 'arcengine' | 'arcengine_v2'> =>
  ['arcengine', 'arcengine_v2'].includes(type);

export const checkHasAuthentication = (
  type: ConnectionType
): type is Extract<ConnectionType, 'aster' | 'drill'> =>
  ['aster', 'drill'].includes(type);

export const checkHasSolrTypeAndJoinChar = (
  type: ConnectionType
): type is Extract<ConnectionType, 'solr'> => ['solr'].includes(type);

export const checkHasArcengineHistoryETL = (
  type: ConnectionType
): type is Extract<ConnectionType, 'arcengine' | 'arcengine_v2'> =>
  ['arcengine', 'arcengine_v2'].includes(type);

export const checkHasStorageLoctions = (
  type: ConnectionType
): type is Extract<ConnectionType, 'arcengine' | 'arcengine_v2'> =>
  ['arcengine', 'arcengine_v2'].includes(type);

export const checkHasParameters = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine'
  | 'arcengine_v2'
  | 'snowflake'
  | 'solr'
  | 'sparksql'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine',
    'arcengine_v2',
    'snowflake',
    'solr',
    'sparksql',
  ].includes(type);

export const checkHasPassword = (
  type: ConnectionType
): type is Exclude<
  ConnectionType,
  'sqlite' | 'athena' | 'duckdb' | 'bigquery'
> => !['sqlite', 'athena', 'duckdb', 'bigquery'].includes(type);

export const checkHasHostnameAndPort = (
  type: ConnectionType
): type is Exclude<
  ConnectionType,
  'sqlite' | 'athena' | 'duckdb' | 'bigquery' | 'snowflake' | 'teradata'
> =>
  !['sqlite', 'athena', 'duckdb', 'bigquery', 'snowflake', 'teradata'].includes(
    type
  );

export const checkHasWarehouse = (
  type: ConnectionType
): type is Extract<ConnectionType, 'snowflake'> => ['snowflake'].includes(type);

export const checkHasDatabase = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'cassandra'
  | 'aster'
  | 'denodo'
  | 'druid'
  | 'drill'
  | 'mssqlserver'
  | 'mariadb'
  | 'oracle'
  | 'mysql'
  | 'postgres'
  | 'postgres:ssb'
  | 'snowflake'
  | 'redshift'
  | 'vertica'
> =>
  [
    'cassandra',
    'aster',
    'denodo',
    'druid',
    'drill',
    'mssqlserver',
    'mariadb',
    'oracle',
    'mysql',
    'postgres',
    'postgres:ssb',
    'snowflake',
    'redshift',
    'vertica',
  ].includes(type);

export const checkHasFileName = (
  type: ConnectionType
): type is Extract<ConnectionType, 'sqlite' | 'duckdb'> =>
  ['sqlite', 'duckdb'].includes(type);

export const checkHasHTTPPath = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'sparksql'
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'phoenix'
> =>
  [
    'sparksql',
    'impyla:cdw-impyla',
    'impyla',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'phoenix',
  ].includes(type);

export const checkHasAccessToken = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  'sparksql' | 'hyve:cdw-hyve' | 'hyve' | 'hive'
> => ['sparksql', 'hyve:cdw-hyve', 'hyve', 'hive'].includes(type);

export const checkHasSessionTimeOut = (
  type: ConnectionType
): type is Extract<ConnectionType, 'arcengine'> => ['arcengine'].includes(type);

export const checkHasSocketTimeout = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine'
  | 'arcengine_v2'
  | 'solr'
  | 'phoenix'
  | 'trino'
  | 'sparksql'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine',
    'arcengine_v2',
    'solr',
    'phoenix',
    'trino',
    'sparksql',
  ].includes(type);

export const checkHasAppName = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  'impyla:cdw-impyla' | 'impyla' | 'impala' | 'hyve:cdw-hyve' | 'hyve' | 'hive'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
  ].includes(type);

export const checkHasConcurrencyAndConcurrencyUser = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine_v2'
  | 'denodo'
  | 'duckdb'
  | 'druid'
  | 'postgres'
  | 'postgres:ssb'
  | 'sqlite'
  | 'redshift'
  | 'phoenix'
  | 'trino'
  | 'mysql'
  | 'mariadb'
  | 'solr'
  | 'snowflake'
  | 'sparksql'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine_v2',
    'denodo',
    'duckdb',
    'druid',
    'postgres',
    'postgres:ssb',
    'sqlite',
    'redshift',
    'phoenix',
    'trino',
    'mysql',
    'mariadb',
    'solr',
    'snowflake',
    'sparksql',
  ].includes(type);

export const checkHasQueryTimeout = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine_v2'
  | 'denodo'
  | 'duckdb'
  | 'druid'
  | 'postgres'
  | 'postgres:ssb'
  | 'sqlite'
  | 'redshift'
  | 'phoenix'
  | 'trino'
  | 'mysql'
  | 'mariadb'
  | 'solr'
  | 'snowflake'
  | 'sparksql'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine_v2',
    'denodo',
    'duckdb',
    'druid',
    'postgres',
    'postgres:ssb',
    'sqlite',
    'redshift',
    'phoenix',
    'trino',
    'mysql',
    'mariadb',
    'solr',
    'snowflake',
    'sparksql',
  ].includes(type);

export const checkHasRowUploadLimit = (
  type: ConnectionType
): type is Extract<
  ConnectionType,
  | 'impyla:cdw-impyla'
  | 'impyla'
  | 'impala'
  | 'hyve:cdw-hyve'
  | 'hyve'
  | 'hive'
  | 'arcengine_v2'
  | 'duckdb'
  | 'mariadb'
  | 'mysql'
  | 'postgres'
  | 'snowflake'
  | 'postgres:ssb'
  | 'sqlite'
  | 'phoenix'
  | 'trino'
  | 'solr'
  | 'sparksql'
  | 'druid'
> =>
  [
    'impyla:cdw-impyla',
    'impyla',
    'impala',
    'hyve:cdw-hyve',
    'hyve',
    'hive',
    'arcengine_v2',
    'duckdb',
    'mariadb',
    'mysql',
    'postgres',
    'snowflake',
    'postgres:ssb',
    'sqlite',
    'phoenix',
    'trino',
    'solr',
    'sparksql',
    'druid',
  ].includes(type);

export const checkHasDSN = (
  type: ConnectionType
): type is Extract<ConnectionType, 'aster' | 'drill' | 'teradata'> =>
  ['aster', 'drill', 'teradata'].includes(type);
