import { Warehouse } from '../types/dataconnection-types';
import {
  ConnectionType,
  DataConnectionSchemaType,
  dataConnectionDefaultValues,
} from './dataconnection-schema';

export const getDefaultValues = ({
  connectionType,
  connectionName,
  selectedWarehouse,
  isCdwPrivate,
}: {
  connectionType: { value: ConnectionType; label: string };
  connectionName: string;
  selectedWarehouse: Warehouse | undefined;
  isCdwPrivate: boolean;
}): DataConnectionSchemaType => {
  if (!selectedWarehouse || !connectionType) return dataConnectionDefaultValues;

  let impersonation: boolean;
  let impersonationTrusted: boolean;

  if (isCdwPrivate && connectionType.value === 'impyla:cdw-impyla') {
    impersonation = true;
    impersonationTrusted = false;
  } else {
    impersonationTrusted = true;
    impersonation = false;
  }

  return {
    ...dataConnectionDefaultValues,
    connectionType: {
      value: connectionType.value,
      label: connectionType.label,
      disabled: false,
    },
    connectionName: connectionName,
    [connectionType.value]: {
      basic: {
        hostname: selectedWarehouse.hostName,
        port: selectedWarehouse.port,
        username: selectedWarehouse.username,
      },
      advanced: {
        connection_mode: selectedWarehouse.connectionMode,
        http_path: selectedWarehouse.httpPath,
        socket_type: selectedWarehouse.socketType,
        authentication_mode: selectedWarehouse.authenticationMode,
        socket_timeout: 600,
        impersonation: impersonation,
        trusted_impersonation: impersonationTrusted,
      },
    },
  };
};
