import React, { useState } from 'react';

import { useLegacyBridge } from 'services/legacy-bridge';

import { DatasetDeleteModal } from '../dataset-delete-modal/DatasetDeleteModal';
import { DatasetActionsDropdownProps } from 'components/dataset-actions-dropdown/DatasetActionsDropdown';

export const DatasetDeleteModalWrapper = (): JSX.Element => {
  const [dataset, setDataset] = useState<DatasetActionsDropdownProps | null>(
    null
  );
  const [isDeleteModalOpen, setIsDeleteModalOpen] = useState<boolean>(false);

  useLegacyBridge(
    'dataset-delete-modal',
    ({
      payload,
    }: {
      payload?: { id: number; name: string; hasEditPerm: boolean };
    }) => {
      if (!payload) return;

      setDataset(payload);
      setIsDeleteModalOpen(true);
    }
  );

  if (dataset === null) return <></>;

  return (
    <DatasetDeleteModal
      dataset={dataset}
      isShow={isDeleteModalOpen}
      onClose={() => {
        setIsDeleteModalOpen(false);
      }}
    />
  );
};
