import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import React, { ReactElement, useState } from 'react';
import { FormControl, FormGroup, FormLabel, Modal } from 'react-bootstrap';
import classes from './DeleteDialog.module.scss';
import Delete from '@cloudera-internal/cuix-core/icons/svg/ic-delete.svg';
import Close from '@cloudera-internal/cuix-core/icons/svg/ic-close.svg';

type DeleteDialogProps = {
  title: string;
  isOpen: boolean;
  close: () => void;
  onConfirm: () => Promise<void>;
  isLoading: boolean;
  info?: ReactElement;
};

export const DeleteDialog = ({
  title,
  isOpen,
  close,
  onConfirm,
  isLoading,
  info,
}: DeleteDialogProps): JSX.Element => {
  const [inputValue, setInputValue] = useState('');

  const isValid = inputValue.toLowerCase() === 'delete';

  const onClose = () => {
    setInputValue('');
    close();
  };

  return (
    <Modal show={isOpen} onHide={close}>
      <Modal.Header className="align-items-center border-bottom-0">
        <h3
          className="text-lg font-light text-dark m-0"
          style={{
            height: 28,
            lineHeight: 1.4,
          }}
        >
          {title}
        </h3>

        <button
          className="outline-none bg-transparent border-0 p-2 text-dark-gray text-lg"
          onClick={onClose}
        >
          <Close />
        </button>
      </Modal.Header>

      <Modal.Body>
        {info}

        <FormGroup controlId="confirmation-input">
          <FormLabel className={classes.label}>
            To confirm, type the word DELETE in the field below and then click
            the Delete button.
          </FormLabel>

          <FormControl
            value={inputValue}
            onChange={(event) => {
              setInputValue(event.target.value);
            }}
            data-test-id="delete-confirmation-text"
          />
        </FormGroup>
      </Modal.Body>

      <Modal.Footer>
        <ClouderaButton onClick={onClose}>Cancel</ClouderaButton>

        <ClouderaButton
          color="danger"
          icon={<Delete />}
          disabled={!isValid || isLoading}
          onClick={() => {
            if (isValid) {
              onConfirm().then(() => {
                setInputValue('');
              });
            }
          }}
          testId="delete-confirmation-yes"
        >
          Delete
        </ClouderaButton>
      </Modal.Footer>
    </Modal>
  );
};
