import {
  ClouderaDialog,
  ClouderaDialogContent,
} from 'components/cloudera-dialog/ClouderaDialog';
import React, { useCallback, useState } from 'react';
import { useLegacyBridge } from 'services/legacy-bridge';
import { EmailJobModal } from './EmailJobModal';
import { JobReport } from 'types/job.type';
import { useEmailNowNotification } from './hooks/use-email-now-notification';

export const EmailJobModalWrapper = (): JSX.Element => {
  const [report, setReport] = useState<JobReport>();
  const [isOpen, setIsOpen] = useState(false);

  const { startCheckingJobStatus } = useEmailNowNotification();

  const openEmailJobModal = useCallback(
    ({ payload }: { payload?: JobReport }) => {
      if (!payload) return;
      setReport(payload);
      setIsOpen(true);
    },
    []
  );

  useLegacyBridge('open-email-job-modal', openEmailJobModal);

  if (!report) {
    return <></>;
  }
  return (
    <ClouderaDialog isOpen={isOpen} onChange={setIsOpen}>
      <ClouderaDialogContent size="lg" scrollable>
        <EmailJobModal
          report={report}
          onEmailSent={startCheckingJobStatus}
          isCreate
        />
      </ClouderaDialogContent>
    </ClouderaDialog>
  );
};
