import { useGetAliasesQuery } from 'apis/aliases/get-aliases-by-dashboard-id';
import React from 'react';
import { Form } from 'react-bootstrap';
import { Controller, UseFormReturn } from 'react-hook-form';
import CreatableSelect from 'react-select/creatable';
import { EmailJobFormSchema } from '../email-job-form-schema';

type EmailJobAliasSectionProps = {
  form: UseFormReturn<EmailJobFormSchema>;
  dashboardId?: number;
};

export const EmailJobAliasSection = ({
  dashboardId,
  form,
}: EmailJobAliasSectionProps): JSX.Element => {
  const { control, formState, register } = form;
  const isCreate = form.watch('isCreate');
  const useAlias = form.watch('emailTarget.useUrlAlias');

  const { data: aliasOptions = [] } = useGetAliasesQuery(dashboardId!, {
    enabled: !!dashboardId,
    select: (data) =>
      data.map((item) => ({ label: item.slug, value: item.slug })),
  });

  return (
    <Form.Group>
      <Form.Label>Use Alias</Form.Label>
      <Form.Check
        disabled={!isCreate}
        id="use-alias"
        label="Use URL alias"
        {...register('emailTarget.useUrlAlias')}
      />
      {useAlias && isCreate && dashboardId && (
        <div className="ml-4">
          <Form.Group>
            <Controller
              name="emailTarget.alias"
              control={control}
              shouldUnregister
              render={({
                field: { onChange, value, onBlur, name },
                fieldState: { invalid },
              }) => {
                const alias = aliasOptions.find(
                  (option) => option.value === value
                );

                return (
                  <>
                    <CreatableSelect
                      options={aliasOptions}
                      value={alias}
                      onChange={(event) => onChange(event?.value)}
                      onBlur={onBlur}
                      name={name}
                      placeholder="Alias"
                      inputId="alias"
                      id="alias"
                      classNamePrefix="react-select"
                      className={invalid ? 'is-invalid' : ''}
                    />
                    <Form.Control.Feedback type="invalid">
                      {formState.errors.emailTarget?.alias?.message}
                    </Form.Control.Feedback>
                  </>
                );
              }}
            />
          </Form.Group>
        </div>
      )}
    </Form.Group>
  );
};
