import styles from './FieldDetail.module.scss';
import React from 'react';
import classNames from 'classnames';
import ErrorIcon from '@cloudera-internal/cuix-core/icons/svg/ic-error.svg';
import { VizTooltip } from 'components/VizTooltip';
import { Tooltip } from 'react-bootstrap';
import { cdlRed500 } from '@cloudera-internal/cuix-core/variables';

export const FieldDetail = ({
  label,
  value,
  percentage,
  styleClass,
}: {
  label: string;
  value: number | string | Error;
  percentage?: number;
  styleClass?: {
    containerClass?: string;
    labelClass?: string;
    valueClass?: string;
  };
}): JSX.Element => {
  const isError = !!value && typeof value === 'object';
  return (
    <div
      className={classNames(styles.itemContainer, styleClass?.containerClass)}
    >
      <span className={classNames(styles.label, styleClass?.labelClass)}>
        {label}:{' '}
      </span>
      {!isError && (
        <span className={classNames(styles.value, styleClass?.valueClass)}>
          {value} {percentage !== undefined && ` | ${percentage.toFixed(3)}%`}
        </span>
      )}
      {isError && (
        <VizTooltip
          placement="top"
          overlay={<Tooltip id={label}>{value.message}</Tooltip>}
        >
          <div>
            <ErrorIcon color={cdlRed500} />
          </div>
        </VizTooltip>
      )}
    </div>
  );
};
