import React from 'react';
import Dropdown from 'react-bootstrap/Dropdown';

import { getGlobalArc } from 'utils/get-global-arc';

import { useWhatSNewModalStore } from 'stores/use-what-s-new-modal-store';

import { GlobalSearch } from '../global-search/GlobalSearch';
import { getVisibleHelpMenuLink } from './get-help-menu-links';
import { VersionDetailsItem } from '../verson-details-item/VersionDetailsItem';
import { VizNavDropdown } from '../viz-nav-dropdown/VizNavDropdown';

const arc = getGlobalArc();

const isExploreStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_explore_style'
);

const visibleLinks = getVisibleHelpMenuLink();

export const HelpMenu = (): JSX.Element => {
  const openWhatsNewModal = useWhatSNewModalStore(({ open }) => open);

  return (
    <VizNavDropdown
      toggle={{
        id: 'help-menu',
        testId: 'navbar-help-dropdown-toggle',
        content: <i className="fa fa-question-circle" />,
      }}
      menu={{
        testId: 'navbar-help-dropdown-menu',
        content: (
          <>
            <GlobalSearch />

            {!isExploreStyleEnabled && (
              <>
                {arc.frontend.globals.settings.shouldDisplayWhatSNew && (
                  <Dropdown.Item
                    data-test-id="whats-new-in-version"
                    onClick={openWhatsNewModal}
                  >
                    What's New in{' '}
                    {arc.frontend.globals.settings.whatSNewVersion}
                  </Dropdown.Item>
                )}

                {visibleLinks.map((item, index) => {
                  if (item.type === 'divider') {
                    return <Dropdown.Divider key={`divider-${index}`} />;
                  }

                  return (
                    <Dropdown.Item
                      key={item.href}
                      as="a"
                      href={item.href}
                      target={item.target}
                    >
                      {item.label}
                    </Dropdown.Item>
                  );
                })}
              </>
            )}

            {arc.frontend.globals.settings.shouldDisplayVersion && (
              <>
                {!isExploreStyleEnabled && <Dropdown.Divider />}

                <VersionDetailsItem />
              </>
            )}
          </>
        ),
      }}
    />
  );
};
