import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';

type Link = {
  type: 'link';
  href: string;
  target?: '_blank';
  label: JSX.Element | string;
};

type Divider = {
  type: 'divider';
};

type Item = (Link | Divider) & {
  isVisible: boolean;
};

const arc = getGlobalArc();

export const getVisibleHelpMenuLink = () => {
  const links: Item[] = [
    {
      type: 'link',
      href: arc.frontend.globals.documentation.url,
      target: '_blank',
      label: arc.frontend.globals.documentation.label,
      isVisible: arc.frontend.globals.settings.shouldDisplayDocumentation,
    },
    ...arc.frontend.globals.settings.customLinks.map((item): Item => {
      if (item.type === 'divider') {
        return { type: 'divider', isVisible: true };
      }

      return {
        type: 'link',
        href: item.link,
        target: '_blank',
        label: item.label,
        isVisible: true,
      };
    }),
    {
      type: 'link',
      href: arc.frontend.globals.settings.supportWebPageUrl,
      target: '_blank',
      label: arc.frontend.globals.settings.supportWebPageLabel,
      isVisible:
        arc.frontend.globals.settings.shouldDisplaySupportWebPage &&
        arc.frontend.globals.settings.supportOption === 'web_page',
    },
    {
      type: 'divider',
      isVisible:
        arc.frontend.globals.settings.shouldDisplaySupportEmail &&
        arc.frontend.globals.settings.supportOption === 'email' &&
        (arc.frontend.globals.settings.shouldDisplayWhatSNew ||
          arc.frontend.globals.settings.shouldDisplayDocumentation ||
          !!arc.frontend.globals.settings.customLinks),
    },
    {
      type: 'link',
      href: `mailto:${arc.frontend.globals.settings.supportEmail}?subject=Support for version ${arc.frontend.globals.product.fullVersion}`,
      label: (
        <>
          <i className="fa fa-envelope" />
          <span>{arc.frontend.globals.settings.supportEmail}</span>
        </>
      ),
      isVisible:
        arc.frontend.globals.settings.shouldDisplaySupportEmail &&
        arc.frontend.globals.settings.supportOption === 'email',
    },
  ];

  return links.filter(({ isVisible }) => isVisible);
};
