import React from 'react';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { render, screen } from '@testing-library/react';
import { NavigationBar } from './NavigationBar';
import '@testing-library/jest-dom';
import $ from 'jquery';

jest.mock('@cloudera-internal/cuix-core/variables', () => ({
  cdlRed500: '#fff',
  cdlBlue500: '#fff',
  cdlGreen500: '#fff',
  cdlOrange500: '#fff',
}));

jest.mock('utils/get-global-arc', () => {
  const arcMock: any = {
    frontend: {
      globals: {
        urls: {},
        settings: {
          customLinks: [],
        },
        authentication: {
          passwordValidation: {},
        },
        permissions: { system: [] },
        documentation: {},
        product: {},
      },
      featureFlipper: {
        getValueById: jest.fn(),
      },
    },
  };

  return {
    getGlobalArc: () => arcMock,
  };
});

describe('JobPage component', () => {
  beforeEach(() => {
    (global as any).$ = $;
  });

  it('should render without errors', async () => {
    render(
      <QueryClientProvider client={new QueryClient()}>
        <NavigationBar
          shouldShowModal={false}
          isAlreadySeen={false}
          activeTabs={undefined}
        />
      </QueryClientProvider>
    );

    const homePageLink = await screen.findByText('Home');
    expect(homePageLink).toBeInTheDocument();
  });

  it('should not render when ?embed=true', async () => {
    Object.defineProperty(window, 'location', {
      value: {
        search: '?embed=true',
      },
      writable: true,
    });

    render(
      <QueryClientProvider client={new QueryClient()}>
        <NavigationBar
          shouldShowModal={false}
          isAlreadySeen={false}
          activeTabs={undefined}
        />
      </QueryClientProvider>
    );

    const homePageLink = screen.queryByText('Home');
    expect(homePageLink).not.toBeInTheDocument();
  });
});
