import React, { ChangeEvent, forwardRef } from 'react';
import Form from 'react-bootstrap/Form';
import Col from 'react-bootstrap/Col';
import { InputGroup } from 'react-bootstrap';
import classNames from 'classnames';

import { useToggle } from 'hooks/use-toggle';
import classes from './PasswordField.module.scss';
import { FieldError } from 'react-hook-form/dist/types';

type PasswordFieldProps = {
  name: string;
  label: string;
  hasToggleVisibility?: boolean;
  placeholder?: string;
  autoFocus?: boolean;
  onChange: (e: ChangeEvent<HTMLInputElement>) => void;
  onBlur?: (e: ChangeEvent<HTMLInputElement>) => void;
  error: FieldError | undefined;
};

export const PasswordField = forwardRef<HTMLInputElement, PasswordFieldProps>(
  (
    {
      name,
      label,
      hasToggleVisibility,
      placeholder,
      autoFocus,
      onChange,
      onBlur,
      error,
    }: PasswordFieldProps,
    forwardedRef
  ): JSX.Element => {
    const [isVisible, toggleVisibility] = useToggle();

    const hasError = !!error;

    return (
      <Form.Group as={Col} className={'p-0'}>
        <Form.Label htmlFor={name} className={classes.passwordFormLabel}>
          {label}
        </Form.Label>
        <InputGroup
          className={classNames(classes.passwordFormGroup, 'mb-0 w-100')}
        >
          <Form.Control
            id={name}
            name={name}
            type={isVisible ? 'text' : 'password'}
            placeholder={placeholder}
            autoFocus={autoFocus}
            isInvalid={hasError}
            onChange={onChange}
            onBlur={onBlur}
            ref={forwardedRef}
          />
          {hasToggleVisibility && (
            <InputGroup.Append
              onClick={toggleVisibility}
              className={classes.clickable}
            >
              <InputGroup.Text id="show-password">
                <i className={`fa ${isVisible ? 'fa-eye' : 'fa-eye-slash'}`} />
              </InputGroup.Text>
            </InputGroup.Append>
          )}
          {hasError ? (
            <Form.Control.Feedback type="invalid">
              {error.message}
            </Form.Control.Feedback>
          ) : (
            <div className={classes.feedbackHeightSubstitute} />
          )}
        </InputGroup>
      </Form.Group>
    );
  }
);
