import React from 'react';

import classes from './Release.module.scss';
import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItem } from '../release-item/ReleaseItem';
import { Accordion } from '../accordion/Accordion';

type ReleaseProps = {
  release: WhatSNewRelease;
  openAccordionIds: string[];
  toggle: (id: string) => void;
};

export const Release = ({
  release,
  openAccordionIds,
  toggle,
}: ReleaseProps): JSX.Element => {
  const isOpen = openAccordionIds.includes(release.version);

  return (
    <Accordion
      isOpen={isOpen}
      eventKey={release.version}
      title={
        <>
          Version {release.version}{' '}
          <span className={classes.releaseDate}>( {release.releaseDate} )</span>
        </>
      }
      toggle={() => toggle(release.version)}
    >
      <>
        {release.items.map((item) => {
          const id = `${release.version}-${item.title}`;

          return (
            <ReleaseItem
              key={id}
              id={id}
              releaseItem={item}
              openAccordionIds={openAccordionIds}
              toggle={toggle}
            />
          );
        })}
      </>
    </Accordion>
  );
};
