import React, { useEffect, useState } from 'react';

import classes from './ReportTileImage.module.scss';

type ReportTileImageProps = {
  type: string;
  baseImg: string;
  img?: string;
  size: 'sm' | 'md';
};

export const ReportTileImage = ({
  baseImg,
  img,
  type,
  size,
}: ReportTileImageProps): JSX.Element => {
  const [shouldShowFallbackImage, setShouldShowFallbackImage] = useState(
    baseImg === img
  );

  useEffect(() => {
    setShouldShowFallbackImage(baseImg === img);
  }, [baseImg, img]);

  if (shouldShowFallbackImage || type === 'app') {
    return (
      <img
        className={size === 'md' ? classes.fallbackImage : 'h-100'}
        src={baseImg}
        alt="chart thumbnail"
        loading="lazy"
      />
    );
  } else {
    return (
      <img
        className="w-100 h-100"
        src={img}
        onError={() => setShouldShowFallbackImage(true)}
        alt="Report snapshot"
        loading="lazy"
      />
    );
  }
};
