import classNames from 'classnames';
import React, { FormEvent, useState } from 'react';
import Form from 'react-bootstrap/Form';

import classes from './SearchBar.module.scss';

type SearchBarProps = {
  defaultValue?: string;
  onChange?: (searchTerm: string) => void;
};

// Event listeners implementing the search logic
// are added on by the specific pages that use this search bar.
// You can find them, by searching for the input tag's id.
export const SearchBar = ({
  defaultValue,
  onChange,
}: SearchBarProps): JSX.Element => {
  const [value, setValue] = useState(defaultValue ?? '');

  return (
    <Form className={classes.form} role="search">
      <Form.Group
        className={classes.formGroup}
        onSubmit={(event: FormEvent) => {
          if (onChange) {
            event.preventDefault();
          }
        }}
      >
        <Form.Control
          className={classes.formControl}
          id="searchbox"
          type="text"
          size="sm"
          placeholder="find titles, viz types, datasets, authors..."
          data-test-id="searchbox"
          onChange={(event) => {
            if (onChange) {
              const newValue = event.target.value;
              onChange(newValue);
              setValue(newValue);
            }
          }}
          defaultValue={defaultValue}
          {...(onChange ? { value } : {})}
        />

        <i
          className={classNames(
            classes.searchIcon,
            'fa fa-search attr-search-icon'
          )}
        />

        <div
          className={classNames(classes.clearIcon, 'attr-search-clear')}
          onClick={() => {
            if (onChange) {
              setValue('');
              onChange('');
            }
          }}
        >
          <i className="fa fa-times-circle" />
        </div>
      </Form.Group>
    </Form>
  );
};
