import { getGlobalArc } from 'utils/get-global-arc';

export type SettingsMenuItem = {
  id: string;
  label: string;
  iconClass: string;
  href: string;
  isVisible: boolean;
  testId: string;
};

const arc = getGlobalArc();

const isExploreStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_explore_style'
);

const areJobsEnabled = arc.frontend.featureFlipper.getValueById('enable_jobs');
const isSetHomePageVisible =
  arc.frontend.globals.authentication.isSuperUser ||
  !arc.frontend.globals.settings.disableHomePageOverride;

const dropdownItems: SettingsMenuItem[] = [
  {
    id: 'navbar-settings-menu-set-home-page',
    label: 'Set Homepage',
    iconClass: 'fa-home',
    href: arc.frontend.globals.urls.setHomePageUrl,
    isVisible: !isExploreStyleEnabled && isSetHomePageVisible,
    testId: 'navbar-settings-menu-set-home-page',
  },
  {
    id: 'navbar-settings-menu-job-status',
    label: 'Job Status',
    iconClass: 'fa-hourglass-2',
    href: arc.frontend.globals.urls.jobOverviewUrl,
    isVisible: !!areJobsEnabled,
    testId: 'navbar-settings-menu-job-status',
  },
];

const siteAdministrationItems: SettingsMenuItem[] = [
  {
    id: 'navbar-settings-menu-activity-log',
    label: 'Logs and Usage',
    iconClass: 'fa-tasks',
    href: arc.frontend.globals.urls.activityUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_viewlogs'),
    testId: 'navbar-settings-menu-activity-log',
  },
  {
    id: 'navbar-settings-menu-users-and-groups',
    label: 'Users and Groups',
    iconClass: 'fa-user',
    href: arc.frontend.globals.urls.usersUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_editperm'),
    testId: 'navbar-settings-menu-users-and-groups',
  },
  {
    id: 'navbar-settings-menu-manage-roles',
    label: 'Manage Roles',
    iconClass: 'fa-users',
    href: arc.frontend.globals.urls.rolesUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_editperm'),
    testId: 'navbar-settings-menu-manage-roles',
  },
  {
    id: 'navbar-settings-menu-manage-url-aliases',
    label: 'Manage URL Aliases',
    iconClass: 'fa sym-dashboard',
    href: arc.frontend.globals.urls.aliasesUrl,
    isVisible: true,
    testId: 'navbar-settings-menu-manage-url-aliases',
  },
  {
    id: 'navbar-settings-menu-manage-api-keys',
    label: 'Manage API Keys',
    iconClass: 'fa-key',
    href: arc.frontend.globals.urls.appsApiKeysUrl!,
    isVisible:
      arc.frontend.globals.permissions.system.includes('sys_editperm') &&
      (arc.frontend.globals.settings.isDataApiEnabled ||
        arc.frontend.globals.settings.isApiKeysEnabled),
    testId: 'navbar-settings-menu-manage-api-keys',
  },
  {
    id: 'navbar-settings-menu-email-templates',
    label: 'Email Templates',
    iconClass: 'fa-envelope',
    href: arc.frontend.globals.urls.customEmailUrl,
    isVisible:
      arc.frontend.globals.permissions.system.includes('sys_schedules') &&
      arc.frontend.globals.settings.hasCustomEmailTemplate,
    testId: 'navbar-settings-menu-email-templates',
  },
  {
    id: 'navbar-settings-menu-custom-styles',
    label: 'Custom Styles',
    iconClass: 'fa-leaf',
    href: arc.frontend.globals.urls.customExtensionEditUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'navbar-settings-menu-custom-styles',
  },
  {
    id: 'navbar-settings-menu-custom-colors',
    label: 'Custom Colors',
    iconClass: 'fa-paint-brush',
    href: arc.frontend.globals.urls.customColorsUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'navbar-settings-menu-custom-colors',
  },
  {
    id: 'navbar-settings-menu-custom-dates',
    label: 'Custom Dates',
    iconClass: 'fa-calendar',
    href: arc.frontend.globals.urls.customDates,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'navbar-settings-menu-custom-dates',
  },
  {
    id: 'navbar-settings-menu-static-assets',
    label: 'Static Assets',
    iconClass: 'fa-file-image-o',
    href: arc.frontend.globals.urls.staticAssetsUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'navbar-settings-menu-static-assets',
  },
  {
    id: 'navbar-settings-menu-site-setting',
    label: arc.frontend.globals.siteSettingName,
    iconClass: 'fa-cogs',
    href: arc.frontend.globals.urls.adminSettingsUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_settings'),
    testId: 'navbar-settings-menu-site-setting',
  },
];

export const getVisibleSettingsMenuDropdownItems = () => ({
  visibleItems: dropdownItems.filter(({ isVisible }) => isVisible),
  visibleSiteAdministrationItems: siteAdministrationItems.filter(
    ({ isVisible }) => isVisible
  ),
});
