import React from 'react';
import { getGlobalArc } from '../../utils/get-global-arc';
import classes from './SettingsNavigation.module.scss';
import SettingsNavigationItem from '../settings-navigation-item/SettingsNavigationItem';

const arc = getGlobalArc();
const hasSystemEditPerm =
  arc.frontend.globals.permissions.system.includes('sys_editperm');
const hasSystemManageJobsPerm =
  arc.frontend.globals.permissions.system.includes('sys_schedules');
const hasSystemCustomStylePerm =
  arc.frontend.globals.permissions.system.includes('sys_styles');
const hasCustomEmailTemplate =
  arc.frontend.globals.settings.hasCustomEmailTemplate;
const isApiKeysEnabled = arc.frontend.globals.settings.isApiKeysEnabled;
const isDataApiEnabled = arc.frontend.globals.settings.isDataApiEnabled;

const colorCombinations = {
  white: '#fff',
  light: '#f4f5f6',
};

type SettingsNavigationProps = {
  activeTab: string;
  color: 'white' | 'light';
};

export const SettingsNavigation = ({
  activeTab,
  color,
}: SettingsNavigationProps): React.ReactElement => {
  const backgroundColor: string = colorCombinations[color];

  return (
    <div className={classes.settingsNav} style={{ backgroundColor }}>
      <ul className={classes.settingsTabs} role="tablist">
        {hasSystemEditPerm && (
          <>
            <SettingsNavigationItem
              href={arc.frontend.globals.urls.usersUrl}
              id="settings-users-groups"
              label="Users and Groups"
              ariaControls="settings-users-groups"
              isActive={false}
            />
            <SettingsNavigationItem
              href={arc.frontend.globals.urls.rolesUrl}
              id="settings-roles"
              label="Manage Roles"
              ariaControls="settings-roles"
              isActive={activeTab === 'settings-roles'}
            />
          </>
        )}

        <SettingsNavigationItem
          href={arc.frontend.globals.urls.aliasesUrl}
          id="settings-aliases"
          label="Manage URL Aliases"
          ariaControls="settings-aliases"
          isActive={activeTab === 'settings-aliases'}
        />

        {hasSystemEditPerm && (isDataApiEnabled || isApiKeysEnabled) && (
          <SettingsNavigationItem
            href={arc.frontend.globals.urls.appsApiKeysUrl!}
            id="settings-apikeys"
            label="Manage API Keys"
            ariaControls="settings-apikeys"
            isActive={false}
          />
        )}

        {hasSystemManageJobsPerm && hasCustomEmailTemplate && (
          <SettingsNavigationItem
            href={arc.frontend.globals.urls.customEmailUrl}
            id="settings-custom-email-templates"
            label="Email Templates"
            ariaControls="settings-custom-email-templates"
            isActive={false}
          />
        )}

        {hasSystemCustomStylePerm && (
          <>
            <SettingsNavigationItem
              href={arc.frontend.globals.urls.customExtensionEditUrl}
              id="settings-custom-styles"
              label="Custom Styles"
              ariaControls="settings-custom-styles"
              isActive={false}
            />
            <SettingsNavigationItem
              href={arc.frontend.globals.urls.customColorsUrl}
              id="settings-custom-colors"
              label="Custom Colors"
              ariaControls="settings-custom-colors"
              isActive={false}
            />
            <SettingsNavigationItem
              href={arc.frontend.globals.urls.customDates}
              id="settings-custom-dates"
              label="Custom Dates"
              ariaControls="settings-custom-dates"
              isActive={false}
            />
            <SettingsNavigationItem
              href={arc.frontend.globals.urls.staticAssetsUrl}
              id="settings-static-assets"
              label="Static Assets"
              ariaControls="settings-static-assets"
              isActive={false}
            />
          </>
        )}
      </ul>
    </div>
  );
};
