import React from 'react';
import Nav from 'react-bootstrap/Nav';
import { getGlobalArc } from 'utils/get-global-arc';
import { VizNavLink } from '../viz-nav-link/VizNavLink';

// THIS COMPONENT IS NOT USED CURRENTLY, BUT IT WILL BE USEFUL LATER

const arc = getGlobalArc();

type SettingsMenuItem = {
  id: string;
  label: string;
  href: string;
  isVisible: boolean;
  testId: string;
};

const siteAdministrationItems: SettingsMenuItem[] = [
  {
    id: 'activity-log',
    label: 'Logs and Usage',
    href: arc.frontend.globals.urls.activityUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_viewlogs'),
    testId: `test-menu-activity-log`,
  },
  {
    id: 'users-groups',
    label: 'Users and Groups',
    href: arc.frontend.globals.urls.usersUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_editperm'),
    testId: 'test-menu-users-and-groups',
  },
  {
    id: 'roles',
    label: 'Manage Roles',
    href: arc.frontend.globals.urls.rolesUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_editperm'),
    testId: 'test-menu-manage-roles',
  },
  {
    id: 'apikeys',
    label: 'Manage API Keys',
    href: arc.frontend.globals.urls.appsApiKeysUrl!,
    isVisible:
      arc.frontend.globals.permissions.system.includes('sys_editperm') &&
      (arc.frontend.globals.settings.isDataApiEnabled ||
        arc.frontend.globals.settings.isApiKeysEnabled),
    testId: 'test-menu-manage-api-keys',
  },
  {
    id: 'custom-email-templates',
    label: 'Email Templates',
    href: arc.frontend.globals.urls.customEmailUrl,
    isVisible:
      arc.frontend.globals.permissions.system.includes('sys_schedules') &&
      arc.frontend.globals.settings.hasCustomEmailTemplate,
    testId: 'test-menu-email-templates',
  },
  {
    id: 'custom-styles',
    label: 'Custom Styles',
    href: arc.frontend.globals.urls.customExtensionEditUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'test-menu-custom-styles',
  },
  {
    id: 'custom-colors',
    label: 'Custom Colors',
    href: arc.frontend.globals.urls.customColorsUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'test-menu-custom-colors',
  },
  {
    id: 'custom-dates',
    label: 'Custom Dates',
    href: arc.frontend.globals.urls.customDates,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'test-menu-custom-dates',
  },
  {
    id: 'static-assets',
    label: 'Static Assets',
    href: arc.frontend.globals.urls.staticAssetsUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_styles'),
    testId: 'test-menu-static-assets',
  },
  {
    id: 'site-settings',
    label: arc.frontend.globals.siteSettingName,
    href: arc.frontend.globals.urls.adminSettingsUrl,
    isVisible: arc.frontend.globals.permissions.system.includes('sys_settings'),
    testId: 'test-menu-site-settings',
  },
];

const SiteAdministrationMenu = () => {
  return (
    <>
      <Nav>
        {siteAdministrationItems
          .filter(({ isVisible }) => isVisible)
          .map(({ id, testId, label, href }) => (
            <Nav.Link
              id={`settings-${id}`}
              as={VizNavLink}
              key={id}
              eventKey={id}
              data-test-id={testId}
              href={href}
            >
              {label}
            </Nav.Link>
          ))}
      </Nav>
    </>
  );
};

export { SiteAdministrationMenu };
