import React from 'react';
import styles from './TableList.module.scss';
import { ToggleContainer } from './ToggleContainer';
import { Alert, Form } from 'react-bootstrap';
import { Controller, useFormContext } from 'react-hook-form';
import Select from 'react-select';

export interface ListProps {
  databaseList?: string[];
  tableList?: string[];
  onTableNameClick: (database: string, table: string) => void;
  open: boolean;
  databaseListPlaceHolder: string;
  tableListSelectorPlaceholder: string;
}

const TableList = ({
  onTableNameClick,
  open = true,
  databaseList,
  tableList,
  databaseListPlaceHolder,
  tableListSelectorPlaceholder,
}: ListProps): JSX.Element => {
  const { getValues, control } = useFormContext();
  const databaseSelectorId = 'selectedDatabase';
  const options = (databaseList ?? []).map((database) => {
    return { value: database, label: database };
  });

  return (
    <ToggleContainer open={open}>
      <Controller
        control={control}
        rules={{
          required: false,
        }}
        render={({ field: { onChange, value } }) => {
          return (
            <Form.Group>
              <Form.Label htmlFor={databaseSelectorId}>Database</Form.Label>
              <Select
                options={options}
                placeholder={databaseListPlaceHolder}
                defaultValue={options.find((x) => x.value === value)}
                id={databaseSelectorId}
                onChange={(e) => {
                  onChange(e?.value);
                }}
              />
            </Form.Group>
          );
        }}
        name={databaseSelectorId}
      />

      <br></br>
      <Form.Label>Tables</Form.Label>

      {Boolean(!tableList && tableListSelectorPlaceholder.length) && (
        <Alert variant="secondary">{tableListSelectorPlaceholder}</Alert>
      )}
      {Boolean(tableList?.length) && (
        <ul
          className={`${styles.removeListStyle} ${styles.autoYOverflow} pl-1 pr-1 mt-2 mb-2`}
          onClick={(e: React.MouseEvent<HTMLUListElement>) =>
            onTableNameClick(
              getValues(databaseSelectorId),
              (e.target as any).textContent
            )
          }
        >
          {tableList?.map((tableName) => (
            <li
              className={`${styles.tableNameItem}`}
              data-toggle="tooltip"
              title={tableName}
              key={tableName}
            >
              <i className="fa fa-table pr-1"></i>
              {tableName}
            </li>
          ))}
        </ul>
      )}
    </ToggleContainer>
  );
};

export { TableList };
