import { BaseQuery } from 'arc/types/baseQuery.type';
import { AceEditor } from 'arc/types/editor-wrapper.type';
import { getGlobalArc } from 'utils/get-global-arc';

const removeAllSemicolonAndSpaceFromTheEndOfAText = (text: string) => {
  return text.replace(/[\s;]+$/, '');
};

export const getSelectedTextFromTheExpressionEditor = (
  editor: AceEditor | undefined
) => {
  let command = '';
  if (!editor) {
    return command;
  }
  const space = ' ';

  const selection = editor.getSession().getSelection();
  if (selection.rangeCount) {
    // If somebody selected just a small part of the code to run extract it from the editor
    // Note: ranges seems to stored in a reverse order
    selection.ranges.reverse().forEach((range) => {
      command = `${command}${space}${editor
        .getSession()
        .getTextRange(range)}${space}`;
    });
  } else {
    // Full Editor Value
    command = editor.getSession().getValue();
  }

  command = removeAllSemicolonAndSpaceFromTheEndOfAText(command);
  return command;
};

type QueryWarnings = {
  inf: Array<string>; // ?
  nan: Array<string>; // ?
};
export type SqlQueryResponse<T> = {
  colnames: Array<string>;
  coltypes: Array<string>;
  info: Array<string>;
  query_warnings: QueryWarnings;
  rowcount: 7;
  rows: Array<Array<T>>;
};

export const runSqlQuery = <T>(
  query: string,
  connectionId: number,
  onSuccess: (response: SqlQueryResponse<T>) => void,
  onError: (error: Error | string) => void
): BaseQuery => {
  const queryCmd = getGlobalArc()
    .data.queryCmd()
    .query(query)
    .dataconnection_id(connectionId)
    .cache_read(false);
  return getGlobalArc().data.runQuery(
    undefined,
    onSuccess,
    () => {},
    undefined,
    (_a, _b, error) => {
      onError(error);
    },
    undefined,
    undefined,
    queryCmd,
    undefined
  );
};
