import '@testing-library/jest-dom';
import { fireEvent, render, screen } from '@testing-library/react';
import * as React from 'react';

import { TokenInput } from './TokenInput';

afterEach(() => {
  jest.resetAllMocks();
});

describe('TokenInput component', () => {
  it('renders without error', () => {
    const mockOnChange = jest.fn();

    render(
      <TokenInput
        id="token-input"
        name="token-input"
        value={['first', 'second']}
        onChange={mockOnChange}
      />
    );
  });

  it('registers a new value when pressing enter', () => {
    const mockOnChange = jest.fn();

    render(
      <TokenInput
        id="token-input"
        name="token-input"
        placeholder="Token Input"
        value={['first', 'second']}
        onChange={mockOnChange}
      />
    );

    const input = screen.getByRole('combobox', { name: 'Token Input' });

    fireEvent.change(input, { target: { value: 'third' } });
    fireEvent.keyDown(input, { keyCode: 13 });

    expect(mockOnChange).toHaveBeenCalledWith(['first', 'second', 'third']);
  });

  it('registers a new value when leaving the input', () => {
    const mockOnChange = jest.fn();

    render(
      <TokenInput
        id="token-input"
        name="token-input"
        placeholder="Token Input"
        value={['first', 'second']}
        onChange={mockOnChange}
      />
    );

    const input = screen.getByRole('combobox', { name: 'Token Input' });

    fireEvent.change(input, { target: { value: 'third' } });
    fireEvent.blur(input);

    expect(mockOnChange).toHaveBeenCalledWith(['first', 'second', 'third']);
  });
});
