import React, { forwardRef, useCallback } from 'react';
import CreatableSelect from 'react-select/creatable';
import { SelectInstance } from 'react-select';
import { FilterOptionOption } from 'react-select/dist/declarations/src/filters';

type TokenInputProps = {
  onChange: (value: string[]) => void;
  onBlur?: (e: React.FocusEvent<HTMLInputElement>) => void;
  value: string[];
  isInvalid?: boolean;
  disabled?: boolean;
  placeholder?: string;
  name: string;
  id: string;
  options?: { label: string; value: string }[];
};

const EmptyComponent = () => null;

export const TokenInput = forwardRef<
  SelectInstance<{ label: string; value: string }, true>,
  TokenInputProps
>(
  (
    {
      onChange,
      value,
      onBlur: onBlurFromProps,
      name,
      disabled,
      placeholder = '',
      isInvalid = false,
      id,
      options,
    },
    ref
  ): JSX.Element => {
    const onBlur = useCallback(
      (e: React.FocusEvent<HTMLInputElement>) => {
        if (e.target.value) {
          onChange([...value, e.target.value]);
        }
        onBlurFromProps?.(e);
      },
      [onBlurFromProps, onChange, value]
    );

    const filterOption = useCallback(
      (
        option: FilterOptionOption<{
          label: string;
          value: string;
        }>,
        value: string
      ) => !!value && option.value.includes(value),
      []
    );

    return (
      <CreatableSelect
        isMulti
        components={{ IndicatorsContainer: EmptyComponent }}
        value={value.map((item) => ({
          label: item,
          value: item,
        }))}
        onChange={(e) => onChange(e.map((item) => item.value))}
        onBlur={onBlur}
        name={name}
        options={options}
        filterOption={filterOption}
        noOptionsMessage={EmptyComponent}
        isDisabled={disabled}
        aria-label={placeholder}
        placeholder={placeholder}
        ref={ref}
        inputId={id}
        classNamePrefix="react-select"
        className={isInvalid ? 'is-invalid' : ''}
      />
    );
  }
);
