import React from 'react';
import Dropdown from 'react-bootstrap/Dropdown';

import { getGlobalArc } from 'utils/get-global-arc';
import { LastLogin } from 'components/last-login/LastLogin';

import classes from './UserMenu.module.scss';
import { UserIcon } from '../user-icon/UserIcon';
import { useToggle } from 'hooks/use-toggle';
import { ChangePasswordModal } from '../change-password-modal/ChangePasswordModal';
import { VizNavDropdown } from '../viz-nav-dropdown/VizNavDropdown';

const arc = getGlobalArc();

const { username, firstName, lastName } = arc.frontend.globals.authentication;

const displayName =
  firstName || lastName ? `${firstName} ${lastName}` : username;

export const UserMenu = (): JSX.Element => {
  const [
    isChangePasswordModalOpen,
    toggleChangePasswordModal,
    closeChangePasswordModal,
  ] = useToggle();

  return (
    <>
      <VizNavDropdown
        toggle={{
          id: 'user-menu',
          testId: 'navbar-user-dropdown-toggle',
          content: (
            <>
              <UserIcon />

              {displayName}
            </>
          ),
        }}
        menu={{
          testId: 'navbar-user-dropdown-menu',
          content: (
            <>
              {arc.frontend.globals.authentication
                .isAllowedToChangePassword && (
                <Dropdown.Item
                  data-test-id="navbar-user-change-password"
                  onClick={toggleChangePasswordModal}
                >
                  Change Password
                </Dropdown.Item>
              )}
              <Dropdown.Item
                as="a"
                href={`${
                  arc.frontend.globals.urls.logoutUrl
                }${encodeURIComponent(window.location.search)}`}
                data-test-id="logout"
              >
                Logout
              </Dropdown.Item>
              <div className={classes.lastLoginContainer}>
                <LastLogin />
              </div>
            </>
          ),
        }}
      />

      <ChangePasswordModal
        isOpen={isChangePasswordModalOpen}
        close={closeChangePasswordModal}
      />
    </>
  );
};
