import React, { forwardRef, useEffect, useRef, useState } from 'react';
import Tooltip from 'react-bootstrap/Tooltip';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizTooltip } from 'components/VizTooltip';

import classes from './VersionDetailsItem.module.scss';
import { copyToClipboard } from 'arc/misc/misc';

const arc = getGlobalArc();

type DynamicTooltipProps = {
  id: string;
  title: string;
  popper: { scheduleUpdate: () => void };
  show: boolean;
};

const DynamicTooltip = forwardRef<HTMLDivElement, DynamicTooltipProps>(
  ({ popper, title, show, ...props }, ref) => {
    useEffect(() => {
      popper.scheduleUpdate();
    }, [title, popper]);

    return (
      <Tooltip ref={ref} {...props}>
        {title}
      </Tooltip>
    );
  }
);

export const VersionDetailsItem = (): JSX.Element => {
  const versionDetail = `${arc.frontend.globals.product.name} ${arc.frontend.globals.product.fullVersion}`;
  const [tooltipTitle, setTooltipTitle] = useState(versionDetail);
  const ref = useRef(null);

  return (
    <VizTooltip
      overlay={(props) => (
        <DynamicTooltip {...props} id="version-details" title={tooltipTitle} />
      )}
      placement="bottom"
    >
      <div
        className={classes.text}
        ref={ref}
        onClick={(_) => {
          const name = `${arc.frontend.globals.product.name}${arc.frontend.globals.product.fullVersion}`;

          copyToClipboard(name).then(() => setTooltipTitle('copied'));
        }}
        onMouseLeave={() =>
          setTimeout(() => setTooltipTitle(versionDetail), 1000)
        }
      >
        {arc.frontend.globals.product.name}{' '}
        {arc.frontend.globals.product.trimmedVersion}
      </div>
    </VizTooltip>
  );
};
