import classNames from 'classnames';
import React, { PropsWithChildren } from 'react';

import classes from './VizButton.module.scss';

type VizButtonProps = {
  color: 'primary' | 'secondary' | 'danger';
  size: 'small' | 'medium';
  onClick?: () => void;
  className?: string;
  testId?: string;
  disabled?: boolean;
  type?: 'submit' | 'reset' | 'button';
};

export const VizButton = ({
  className,
  color,
  size,
  onClick,
  testId,
  children,
  disabled,
  type = 'button',
}: PropsWithChildren<VizButtonProps>): JSX.Element => (
  <button
    className={classNames(
      classes.button,
      classes[size],
      classes[color],
      className
    )}
    onClick={onClick}
    data-test-id={testId}
    disabled={disabled}
    type={type}
  >
    {children}
  </button>
);
