import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizLink } from 'components/viz-link/VizLink';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';

const version = '6.3.1';

const arc = getGlobalArc();

const items: WhatSNewRelease['items'] = [
  {
    title: ' Allow drilling into data discover from the context menu',
    content: (
      <>
        <ReleaseItemText>
          Drill-into enables you to discover the more granular information for a
          particular data point by examining other dimensions of your data.
        </ReleaseItemText>

        <ReleaseItemText>
          <VizLink
            href="https://docs.cloudera.com/r/viz-tpp74aa.html"
            target="_blank"
          >
            Read more
            <i className="fa fa-external-link ml-1" />
          </VizLink>
        </ReleaseItemText>

        <ReleaseItemImage
          srcInfo={{
            version,
            feature: 1,
            index: 1,
          }}
          alt="Drilling into data discovery"
        />
      </>
    ),
  },
  {
    title: 'Better filter to segments workflow',
    content: (
      <>
        <ReleaseItemText>
          Besides creating segments in the dataset detail view, CDP Data
          Visualization also lets you create segments from a filter definition
          of a visual.
        </ReleaseItemText>

        <ReleaseItemText>
          <VizLink
            href="https://docs.cloudera.com/r/viz-kpp789a.html"
            target="_blank"
          >
            Read more
            <i className="fa fa-external-link ml-1" />
          </VizLink>
        </ReleaseItemText>

        <ReleaseItemImage
          srcInfo={{
            version,
            feature: 2,
            index: 1,
          }}
          alt="Filter to segments"
        />
      </>
    ),
  },
];

if (arc.frontend.globals.permissions.system.includes('sys_styles')) {
  items.push({
    title: 'Improved HTML extension visual editor',
    content: (
      <>
        <ReleaseItemText>
          Now creating and editing HTML extension visuals is easier with syntax
          highlighting, autocompletion, and side-by-side preview
        </ReleaseItemText>

        <ReleaseItemText>
          <VizLink
            href="https://docs.cloudera.com/r/viz-kpp456a.html"
            target="_blank"
          >
            Read more
            <i className="fa fa-external-link ml-1" />
          </VizLink>
        </ReleaseItemText>

        <ReleaseItemImage
          srcInfo={{
            version,
            feature: 3,
            index: 1,
          }}
          alt="HTML extension visual editor"
        />
      </>
    ),
  });
}

export const r631: WhatSNewRelease = {
  version,
  releaseDate: 'August 24, 2021',
  items,
};
