import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const version = '6.3.2';

export const r632: WhatSNewRelease = {
  version,
  releaseDate: 'September 30, 2021',
  items: [
    {
      title:
        'Additional styling for data summarization in Table and Cross Tabulation visuals',
      content: (
        <>
          <ReleaseItemText>
            A new style setting allows the user to change the font weight of row
            and column summaries.
          </ReleaseItemText>

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-tpp74cc.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Font weight styling example for Cross Tabuliation visual"
          />

          <ReleaseItemText>
            Data summary settings have also been moved into their own subsection
            for better discoverability.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Data summary subsection example"
          />
        </>
      ),
    },
    {
      title: 'Multi-conditional threshold settings for emails',
      content: (
        <>
          <ReleaseItemText>
            It is now possible to chain multiple conditions for threshold based
            emails.
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/r/viz-tpp74bb.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Multi-conditional threshold based email setup example"
          />
        </>
      ),
    },
  ],
};
