import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const version = '6.3.3';

export const r633: WhatSNewRelease = {
  version,
  releaseDate: 'November 8, 2021',
  items: [
    {
      title: 'Find a specific job log with ease',
      content: (
        <>
          <ReleaseItemText>
            Logs can now be filtered by job type and status.
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/r/viz-def749a.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Filter jobs by type and/or status"
          />
        </>
      ),
    },
    {
      title: 'Timeline visuals can now contain a legend',
      content: (
        <>
          <ReleaseItemText>
            It is now possible to enable legends for the timeline visual type by
            adding a dimension to the color shelf.
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc749a.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Timeline visual example"
          />
        </>
      ),
    },
    {
      title: 'Editing a dataset field is now just one click away',
      content: (
        <>
          <ReleaseItemText>
            Users will now find the edit button on the dataset field tile, next
            to the dropdown icon.
          </ReleaseItemText>

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-tpp74dd.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Edit dataset fields form"
          />
        </>
      ),
    },
  ],
};
