import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const version = '6.3.6';

export const r636: WhatSNewRelease = {
  version,
  releaseDate: 'January 31, 2022',
  items: [
    {
      title:
        'Configure independent aggregate scales for visuals with primary and secondary axes',
      content: (
        <>
          <ReleaseItemText>
            The line, combined bar/line, and grouped bar visuals now support
            primary and secondary axes with independent minimum and maximum
            aggregate scales.
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/data-visualization/latest/howto-customize-visuals/topics/viz-change-aggregate-scale-min-max.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Combined bar/line chart with primary and secondary axes."
          />
        </>
      ),
    },
    {
      title: 'Email job improvements',
      content: (
        <>
          <ReleaseItemText>
            We’ve added two improvements to make sending schedule- and
            threshold-based jobs easier.
          </ReleaseItemText>

          <ReleaseItemText>
            Threshold-based email jobs can now be configured from applications.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Configure threshold-base email job from application."
          />

          <ReleaseItemText>
            And the page parameters of a previously-scheduled job can now be
            modified without re-creating the job.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 2,
            }}
            alt="Edit parameters for."
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/data-visualization/latest/howto-manage-jobs/topics/viz-edit-job.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
  ],
};
