import React from 'react';

import { VizLink } from 'components/viz-link/VizLink';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';

const version = '6.3.7';

export const r637: WhatSNewRelease = {
  version,
  releaseDate: 'March 3, 2022',
  items: [
    {
      title: 'Quicker filter search results with frontend filtering',
      content: (
        <>
          <ReleaseItemText>
            Searching in filters will now immediately show a list of matching
            filter options
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
              extension: 'gif',
            }}
            alt="Quick frontend filtering"
          />
        </>
      ),
    },
    {
      title: 'Improved Field Statistics Visual',
      content: (
        <>
          <ReleaseItemText>
            The field statistics visual is improved for hive connections
          </ReleaseItemText>

          <ReleaseItemText>
            Read more about Field Stats visual{' '}
            <VizLink
              href="https://docs.cloudera.com/data-visualization/latest/howto-visuals/topics/viz-visual-field-stat.html"
              target="_blank"
            >
              here
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Field stats visual of hurricane dataset"
          />
        </>
      ),
    },
    {
      title: 'Improved Derived Data',
      content: (
        <>
          <ReleaseItemText>
            Deleting derived data is now working, helping users create complex
            sub-queries easily through drag and drop
          </ReleaseItemText>

          <ReleaseItemText>
            Read more about Derived Data{' '}
            <VizLink
              href="https://docs.cloudera.com/data-visualization/latest/advanced-analytics-concepts/topics/viz-derived-data.html"
              target="_blank"
            >
              here
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Creating derived data sub-query"
          />
        </>
      ),
    },
  ],
};
