import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizLink } from 'components/viz-link/VizLink';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';

const version = '6.4.0';

const arc = getGlobalArc();

const items: WhatSNewRelease['items'] = [];

if (arc.frontend.featureFlipper.getValueById('enable_direct_access')) {
  items.push({
    title: 'Improved Direct Access',
    content: (
      <>
        <ReleaseItemText>
          Direct access to your data through SQL is now available on additional
          connection types, and to make using the query editor easier, you can
          send a query with cmd + enter keyboard shortcut.
        </ReleaseItemText>

        <ReleaseItemText>
          Read more about Direct Access{' '}
          <VizLink
            href="https://docs.cloudera.com/data-visualization/latest/connect-to-data/topics/viz-use-direct-access.html"
            target="_blank"
          >
            here
          </VizLink>
        </ReleaseItemText>

        <ReleaseItemImage
          srcInfo={{
            version,
            feature: 1,
            index: 1,
          }}
          alt="Direct access button"
        />
      </>
    ),
  });
}

if (arc.frontend.globals.permissions.system.includes('sys_styles')) {
  items.push({
    title: 'Improved Static Assets Uploading',
    content: (
      <>
        <ReleaseItemText>
          It is now easier to upload files including images, fonts, and
          JavaScript packages for use in custom styles, emails, or rich text
          visuals.
        </ReleaseItemText>

        <ReleaseItemText>
          Read more about uploading static assets{' '}
          <VizLink
            href="https://docs.cloudera.com/data-visualization/latest/howto-manage-static-assets/topics/viz-static-assets.html"
            target="_blank"
          >
            here
          </VizLink>
        </ReleaseItemText>

        <ReleaseItemImage
          srcInfo={{
            version,
            feature: 2,
            index: 1,
          }}
          alt="Uploading static asset"
        />
      </>
    ),
  });
}

items.push({
  title: 'Adding on-dashboard filter widgets',
  content: (
    <>
      <ReleaseItemText>
        Users can now move a variety of 'picklist' filter types to the dashboard
        canvas.
      </ReleaseItemText>

      <ReleaseItemText>
        Read more about filter widgets on the dashboard{' '}
        <VizLink
          href="https://docs.cloudera.com/data-visualization/latest/filter-widgets-with-dashboard/topics/viz-move-filter-widgets-dashboard.html"
          target="_blank"
        >
          here
        </VizLink>
      </ReleaseItemText>

      <ReleaseItemImage
        srcInfo={{
          version,
          feature: 3,
          index: 1,
        }}
        alt="Moving filter to dashboard"
      />
    </>
  ),
});

export const r640: WhatSNewRelease = {
  version,
  releaseDate: 'May 2, 2022',
  items,
};
