import React from 'react';

import { VizLink } from 'components/viz-link/VizLink';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';

const version = '7.0.0';

export const r700: WhatSNewRelease = {
  version,
  releaseDate: 'May 16, 2022',
  items: [
    {
      title: 'New design for querying data',
      content: (
        <>
          <ReleaseItemText>
            We’ve designed a new homepage layout so users can quickly add and
            access recent queries, data connections, and datasets alongside
            their dashboards and applications.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Home page style improvements"
          />

          <ReleaseItemText>
            You can access your data from the new SQL tab
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="New SQL tab"
          />

          <ReleaseItemText>
            You can now create a new dataset or dashboard from a query with just
            a single click.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 3,
            }}
            alt="Create dataset from Direct Access"
          />

          <ReleaseItemText>
            And when you’re in Edit mode of a dashboard, editing a dataset based
            on a SQL query is even easier with the new “Edit Dataset SQL” option
            from the in-visual options menu.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 4,
            }}
            alt="Edit a dataset based on an SQL query with the new Edit Dataset SQL option"
          />

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 5,
            }}
            alt="New Edit Dataset SQL modal"
          />
        </>
      ),
    },
    {
      title: 'Major updates to our Bootstrap and jQuery libraries.',
      content: (
        <>
          <ReleaseItemText>
            Bootstrap was upgraded from 3.3.7 to 4.6.0 and jQuery was upgraded
            from 2.2.4 to 3.6.0.
          </ReleaseItemText>

          <ReleaseItemText>
            This change has a potential impact to customers who have created
            Custom Styles in earlier versions of CDV, since they would
            potentially contain code that references legacy Bootstrap classes or
            jQuery methods in their CSS and/or JS respectively.
          </ReleaseItemText>

          <ReleaseItemText>
            For more information, please visit the{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-aaa1vz3.html"
              target="_blank"
            >
              migration guide
            </VizLink>
          </ReleaseItemText>
        </>
      ),
    },
  ],
};
