import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const version = '7.0.1';

export const r701: WhatSNewRelease = {
  version,
  releaseDate: 'June 29, 2022',
  items: [
    {
      title: 'Improved dataset switching',
      content: (
        <>
          <ReleaseItemText>
            We've added dataset switching to filters, so you can quickly change
            the filter across datasets and connections.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Switch filter's dataset"
          />

          <ReleaseItemText>
            You can now also switch datasets for all filters and visuals in a
            sheet
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Switch datasets for all visuals in a sheet"
          />

          {/* TODO: ADD ONCE DOCS IN PLACE
      <div class="list-item-desc">
        For more information on switching datasets, please visit
        <a
          href="https://docs.cloudera.com/data-visualization/latest/cdv-7.0-custom-style-migration-guide.pdf"
          target="_blank"
        >
          here
        </a>
      </div> */}
        </>
      ),
    },
    {
      title: 'Improved query queuing and analysis',
      content: (
        <>
          <ReleaseItemText>
            There is a now a per-user concurrency setting for data connections
            that can limit the number of queries a single user can occupy.
          </ReleaseItemText>

          <ReleaseItemText>
            This prevents one user from blocking access for that connection.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Concurrency Per User setting"
          />

          <ReleaseItemText>
            We've also added a "Current Load" panel to the "Profile Data" modal
            that allows users to view and manage ongoing queries.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 2,
            }}
            alt="Current Load tab of Profile Data Modal"
          />

          {/* TODO: ADD ONCE DOCS IN PLACE
        <div class="list-item-desc">
          For more information on managing concurrency, please visit
          <a
            href="https://docs.cloudera.com/data-visualization/latest/cdv-7.0-custom-style-migration-guide.pdf"
            target="_blank"
          >
            here
          </a>
        </div> */}
        </>
      ),
    },
    {
      title: 'Improved dashboard layout experience',
      content: (
        <>
          <ReleaseItemText>
            The dashboard layout interface now includes names and images of
            relevant visuals for easier visual identification
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/r/viz-ttx1vz3.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Home page style improvements"
          />

          {/* TODO: ADD ONCE DOCS IN PLACE
      <div class="list-item-desc">
        For more information in the layout interface, please visit
        <a
          href="https://docs.cloudera.com/data-visualization/latest/cdv-7.0-custom-style-migration-guide.pdf"
          target="_blank"
        >
          here
        </a>
      </div> */}
        </>
      ),
    },
  ],
};
