import React from 'react';

import { VizLink } from 'components/viz-link/VizLink';
import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.0.2';

export const r702: WhatSNewRelease = {
  version,
  releaseDate: 'August 9, 2022',
  items: [
    {
      title: 'An improved filtering experience',
      content: (
        <>
          <ReleaseItemText>
            We've improved the display of user-added values in filters. More
            information on using this feature{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ttx1vz6.html"
              target="_blank"
            >
              here
            </VizLink>
            .
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Enabling 'Allow the user to add values to the filter'"
          />

          <ReleaseItemText>
            You can also now include or exclude null values in date filters. See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ttx1vz7.html"
              target="_blank"
            >
              here
            </VizLink>{' '}
            for details on how to enable this feature.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 4,
            }}
            alt="Null selector in date filter"
          />

          <ReleaseItemText>
            Relative date filters now also work in emailed dashboards and when
            passed through the url. You can read more about relative date
            filters{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ttx1vz8.html"
              target="_blank"
            >
              here
            </VizLink>
            .
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Quickly search application from the navbar',
      content: (
        <>
          <ReleaseItemText>
            Users can now search for dashboards, applications, datasets, or
            documentation from anywhere in the application.
          </ReleaseItemText>

          <ReleaseItemText>
            Try it out by typing:
            <ul>
              <li>Find dataset taxicab</li>
              <li>Find dashboard taxi</li>
              <li>Find app sample</li>
              <li>Enabling ldap</li>
            </ul>
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/r/viz-ttx1vz5.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Global search from navbar"
          />
        </>
      ),
    },
    {
      title: 'Support for disabling manual user creation',
      content: (
        <>
          <ReleaseItemText>
            There is a now a site setting that allows administrators to disable
            manual user creation within the application.
          </ReleaseItemText>

          <ReleaseItemText>
            This helps prevent username collisions between SSO users and
            manually created users.
          </ReleaseItemText>

          <ReleaseItemText>
            <VizLink
              href="https://docs.cloudera.com/r/viz-ttx1vz4.html"
              target="_blank"
            >
              Read more
              <i className="fa fa-external-link ml-1" />
            </VizLink>
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Site setting 'Enable Local User Creation'"
          />
        </>
      ),
    },
  ],
};
