import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';

import { VizLink } from 'components/viz-link/VizLink';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.0.4';

export const r704: WhatSNewRelease = {
  version,
  releaseDate: 'October 28, 2022',
  items: [
    {
      title: 'Improved Cross Tabulation',
      content: (
        <>
          <ReleaseItemText>
            We've improved the "Expansion" feature for cross tabulations to more
            intuitively aggregate across row dimensions.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="'Enable Expansion' for cross tabulation"
          />

          <ReleaseItemText>
            Cross Tabulation headers are also more readable, with tooltips for
            titles that are too long to display.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Tooltip for long cross tabulation headers"
          />
        </>
      ),
    },
    {
      title:
        '"Save selections made in edit mode" settings available for all filters',
      content: (
        <>
          <ReleaseItemText>
            Users can now selectively enable an option to save text, numeric, or
            date range selections made in edit mode.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Filter setting for saving selection in edit mode"
          />

          <ReleaseItemText>
            When unselected, filter values selected by the dashboard editor will
            not be the default in edit mode.
          </ReleaseItemText>

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ppp1vz9.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'New "Select all" for filter selections',
      content: (
        <>
          <ReleaseItemText>
            When selecting values for a visual filter, there is now a "Select
            All" for bulk selection of the shown filter values.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="'Select all' in filter 'Select values' modal"
          />
        </>
      ),
    },
  ],
};
