import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.1.0';

export const r710: WhatSNewRelease = {
  version,
  releaseDate: 'February 3, 2023',
  items: [
    {
      title: 'Data Extracts are now available for accelerated visualization',
      content: (
        <>
          <ReleaseItemText>
            {' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ezx2vz1.html"
              target="_blank"
            >
              Data Extracts
            </VizLink>{' '}
            are a tool for managing the analytical capabilities, performance,
            concurrency and security of data access across data connections.
          </ReleaseItemText>

          <ReleaseItemText>
            The setting can be enabled via the site settings by a user with
            "Manage Site Settings" permission.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Enable Data Extracts"
          />

          <ReleaseItemText>
            Once enabled, data extracts can be defined and edited in the
            dataset.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Create Data Extracts"
          />

          <ReleaseItemText>
            An overview of data extracts can be seen and managed on the "Data"
            page.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 3,
            }}
            alt="Create Data Extracts"
          />

          <ReleaseItemText>
            For more information on using data extracts in dashboards, view{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ezx2vz2.html"
              target="_blank"
            >
              here
            </VizLink>
            .
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Accelerated data import and visualization',
      content: (
        <>
          <ReleaseItemText>
            Users can now create datasets or add data to a data connection
            directly from the homepage.
          </ReleaseItemText>

          <VizLink
            href="https://docs.cloudera.com/r/viz-ezx1vz1.html"
            target="_blank"
          >
            Read more
            <i className="fa fa-external-link ml-1" />
          </VizLink>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Add data from the homepage"
          />
        </>
      ),
    },
    {
      title: 'New notification center for quickly viewing application errors',
      content: (
        <>
          <ReleaseItemText>
            For admin users or normal users with the "View activity logs"
            privilege, webserver errors (such issues with applying advanced site
            settings) will be available in the new notification center.
          </ReleaseItemText>

          <ReleaseItemText>
            All users will be able to see browser-level errors that might be
            affecting CDV.
          </ReleaseItemText>

          <VizLink
            href="https://docs.cloudera.com/r/viz-ezx3vz3.html"
            target="_blank"
          >
            Read more
            <i className="fa fa-external-link ml-1" />
          </VizLink>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Notification Center"
          />
        </>
      ),
    },
  ],
};
