import React from 'react';
import { getGlobalArc } from 'utils/get-global-arc';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const arc = getGlobalArc();

const hasManageSettingsPermission =
  arc.frontend.globals.permissions.system.includes('sys_settings');

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.1.1';

const items = [
  {
    title: "We're continuing to improve your experience of using CDV",
    content: (
      <ReleaseItemText>
        This release of Cloudera Data Visualization contains performance,
        usability and security enhancements. For more information and a full
        list of the changes we've made,&#32;
        <VizLink
          href="https://docs.cloudera.com/r/viz-ezx1vz0.html"
          target="_blank"
        >
          view the complete release notes.
        </VizLink>
      </ReleaseItemText>
    ),
  },
];

if (hasManageSettingsPermission) {
  items.push({
    title: 'New Site Settings for job log retention policies',
    content: (
      <>
        <ReleaseItemText>
          Admins can now configure the length of time that job logs are retained
          on the system. The maximum historic job log entires, maximum retention
          days and interval for log deletion checks can be managed from Site
          Settings &gt; Job logs.
        </ReleaseItemText>

        <VizLink
          href="https://docs.cloudera.com/r/viz-ezx3vz4.html"
          target="_blank"
        >
          Read more
          <i className="fa fa-external-link ml-1" />
        </VizLink>

        <ReleaseItemImage
          srcInfo={{
            version,
            feature: 1,
            index: 1,
          }}
          alt="Job log retention Site Setting"
        />
      </>
    ),
  });
}

export const r711: WhatSNewRelease = {
  version,
  releaseDate: 'March 10, 2023',
  items,
};
