import React from 'react';
import { getGlobalArc } from 'utils/get-global-arc';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const arc = getGlobalArc();

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.1.2';

export const r712: WhatSNewRelease = {
  version,
  releaseDate: 'April 28, 2023',
  items: [
    {
      title:
        'Create short url links to specific views of a dashboard or application',
      content: (
        <>
          <ReleaseItemText>
            Short URLs of a specific sheet, dashboard, and set of parameters can
            be created when getting the URL for a dashboard or application.
            {arc.frontend.globals.permissions.system.includes('sys_settings')
              ? 'You can "Enable URL shortener" on the "Site Settings" page'
              : ''}
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Short URL in 'Get URL' modal"
          />

          <ReleaseItemText>
            These short URLs can be easily shared with collaborators.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Using short URLs in emails"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-tpp749a.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Improved null handling in filters',
      content: (
        <>
          <ReleaseItemText>
            Null values can be excluded for all filter types. This ability can
            be disabled under the "Basic" filter settings using the "Permit
            selection of NULL values" setting.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Filtering nulls"
          />

          <ReleaseItemText>
            For more information on null filters, view{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-ezx3vz5.html"
              target="_blank"
            >
              here
            </VizLink>
            .
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Quicker visualization upon data import',
      content: (
        <>
          <ReleaseItemText>
            When importing new tables into a data connection, there are now
            options for immediately creating a dataset or dashboard from the
            imported table.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Data import options"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-tpp777b.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Simplifying table management for data extracts',
      content: (
        <>
          <ReleaseItemText>
            When deleting an extract, the option to drop the extract's target
            table from the database is now available.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 4,
              index: 1,
            }}
            alt="Deleting data extract"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-tpp888c.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
  ],
};
