import React from 'react';
import { getGlobalArc } from 'utils/get-global-arc';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

const arc = getGlobalArc();

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.1.4';

export const r714: WhatSNewRelease = {
  version,
  releaseDate: 'July 27, 2023',
  items: [
    ...(arc.frontend.featureFlipper.getValueById('enable_cml_sync') &&
    arc.frontend.globals.settings.cdvSubMode !== 'MLExplore'
      ? [
          {
            title:
              'CML connections and user permissions now sync automatically for new CDV applications running in CML',
            content: (
              <>
                <ReleaseItemText>
                  Users can now sync their data connections, similar to{' '}
                  <VizLink
                    href="https://docs.cloudera.com/r/viz-ddd749d.html"
                    target="_blank"
                  >
                    CML Data tab deployments{' '}
                  </VizLink>
                  of CDV.
                </ReleaseItemText>

                <ReleaseItemImage
                  srcInfo={{
                    version,
                    feature: 1,
                    index: 1,
                  }}
                  alt="Sync data connections"
                />

                <ReleaseItemText>
                  Additionally, CML project owners, admin, contributor, and site
                  admin will automatically be assigned the system admin role in
                  CDV. All other project collaborators will be visual consumers.
                </ReleaseItemText>

                <ReleaseItemImage
                  srcInfo={{
                    version,
                    feature: 1,
                    index: 2,
                  }}
                  alt="System admin and visual consumer groups"
                />
              </>
            ),
          },
        ]
      : []),
    {
      title: 'Site Settings have received a redesign',
      content: (
        <>
          <ReleaseItemText>
            We've redesigned Site Administration {'>'} Site Settings to make
            finding, using and editing sitewide application settings easier.
            Users with the "Manage Site Settings" permission will see the new
            category view as well as a search functionality to make finding a
            specific control faster.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Redesigned site settings page"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc246y.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Added ability to change job owners',
      content: (
        <>
          <ReleaseItemText>
            Existing job owners and system admins can now change a job's owner
            on the "Job Status" page.
          </ReleaseItemText>

          <ReleaseItemText>
            Jobs from deleted users will have unassigned owners, and will not be
            able to run until the owner is changed.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Changing job owner"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc246x.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'Easily accessible filter information',
      content: (
        <>
          <ReleaseItemText>
            Filters now have an info popover that can provide useful information
            like the dataset and data connection the filter was created from.
            The filter id is also included for easier performance and query
            debugging.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 4,
              index: 1,
            }}
            alt="Filter information"
          />
        </>
      ),
    },
  ],
};
