import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.1.7';

export const r717: WhatSNewRelease = {
  version,
  releaseDate: 'October 27, 2023',
  items: [
    {
      title: 'Redesigned notification center',
      content: (
        <>
          <ReleaseItemText>
            Notifications are now automatically marked as read and can be pinned
            to prevent them from being cleared.
          </ReleaseItemText>
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="Pin notifications to preserve them"
          />

          <ReleaseItemText>
            Duplicate notifications are stacked to reduce clutter.
          </ReleaseItemText>
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Duplicate messages stack"
          />
          <ReleaseItemText>
            It is also possible to mute notifications application-wide via a new
            Site Setting available to administrators.
          </ReleaseItemText>
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 3,
            }}
            alt="Mute notifications in site settings"
          />
        </>
      ),
    },
    {
      title: 'Improved static asset upload',
      content: (
        <>
          <ReleaseItemText>
            Static asset upload now works for files up to 100 MB by default.
          </ReleaseItemText>

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc468b.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information on managing static assets.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Upload static asset"
          />
        </>
      ),
    },
    {
      title: 'Improved editor for SQL datasets',
      content: (
        <>
          <ReleaseItemText>
            Creating or editing a dataset from SQL is now easier with new
            autocompletion and syntax highlighting in the dataset modal.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="New dataset from sql"
          />
        </>
      ),
    },
  ],
};
