import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.2.0';

export const r720: WhatSNewRelease = {
  version,
  releaseDate: 'January 29, 2024',
  items: [
    {
      title: 'Artificial Intelligence (AI) visual type (Tech Preview)',
      content: (
        <>
          <ReleaseItemText>
            An AI visual introduces a natural language interface for interacting
            with dashboards and datasets.
          </ReleaseItemText>

          <ReleaseItemText>
            The new visual type is available for SQLite and Solr data
            connections that contain vectorized column information the user
            would like to use for creating a Retrieval-Augmented Generation
            (RAG) chatbot.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="AI visual builder"
          />

          <ReleaseItemText>
            Data upload is also now available for Solr data connections, with
            support for vectorized fields in Solr 9.0+.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 2,
            }}
            alt="Solr upload"
          />
        </>
      ),
    },
    {
      title:
        'Visuals interface redesigned for improved performance and workspace management',
      content: (
        <>
          <ReleaseItemText>
            The Visuals page has been optimized for speedier loading and greater
            usability.
          </ReleaseItemText>

          <ReleaseItemText>
            Administrators or users with "
            <VizLink
              href="https://docs.cloudera.com/r/viz-asp0aa0.html"
              target="_blank"
            >
              Additional system privilege
            </VizLink>
            " can now easily move dashboards and apps to any workspace,
            including private ones.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 2,
            }}
            alt="Moving workspaces"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc468d.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information on workspaces.
          </ReleaseItemText>
        </>
      ),
    },
    {
      title: 'More flexible data extract creation and editing',
      content: (
        <>
          <ReleaseItemText>
            Users can more granularly add aggregation to fields when creating
            data extracts.
          </ReleaseItemText>

          <ReleaseItemText>
            They can now also specify the same field with different aggregations
            multiple times in the same data extract.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Improved way of defining extracts"
          />

          <ReleaseItemText>
            See{' '}
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc468c.html"
              target="_blank"
            >
              docs{' '}
            </VizLink>
            for more information on data extracts.
          </ReleaseItemText>
        </>
      ),
    },
  ],
};
