import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemImage } from '../release-item-image/ReleaseItemImage';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '7.2.1';

export const r721: WhatSNewRelease = {
  version,
  releaseDate: 'January 29, 2024',
  items: [
    {
      title: 'Improvements to the redesigned Visuals interface ',
      content: (
        <>
          <ReleaseItemText>
            Shared workspaces and Private workspaces are now sorted and these
            lists can now be collapsed for easier navigation.
          </ReleaseItemText>

          <ReleaseItemText>
            Selected reports are now remembered when a user switch changes
            between workspaces.
          </ReleaseItemText>

          <ReleaseItemText>
            The positions of the Public and Samples workspaces in the workspace
            list now reflect user permissions.
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
            }}
            alt="New Visuals Page"
          />
        </>
      ),
    },
    {
      title: 'New "Sensitive" option in the dataset field editor',
      content: (
        <>
          <ReleaseItemText>
            Dataset fields can now be marked as “Sensitive” from the Dataset
            &gt; Field Editor. Sensitive fields cannot be moved to visual
            shelves, such as the “Embedding Context” shelf of the Artificial
            Intelligence (AI) Assistant visual (Technical Preview).
          </ReleaseItemText>

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Sensitive dataset fields"
          />

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 2,
            }}
            alt="Field editor sensitive checkbox"
          />

          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 3,
            }}
            alt="Sensitive fields visual builder"
          />
        </>
      ),
    },
    {
      title:
        'Snapshot browser session lifetime values have been consolidated to align with manually configured snapshot timeouts',
      content: (
        <>
          <ReleaseItemText>
            Additionally, user authentication errors encountered during snapshot
            creation are now properly surfaced.
          </ReleaseItemText>

          <ReleaseItemText>
            For more information and a full list of the application-wide
            performance improvements we made,&#32;
            <VizLink
              href="https://docs.cloudera.com/r/viz-ezx1vz0.html"
              target="_blank"
            >
              view the complete release notes.
            </VizLink>
          </ReleaseItemText>
        </>
      ),
    },
  ],
};
