import React from 'react';

import type { WhatSNewRelease } from 'types/what-s-new-release.type';
import { ReleaseItemText } from '../release-item-text/ReleaseItemText';
import { VizLink } from 'components/viz-link/VizLink';
import { ReleaseItemImage } from 'components/release-item-image/ReleaseItemImage';

// UPDATE WHATSNEWVER in arcweb/apps/views.py with latest
const version = '8.0.0';

export const r800: WhatSNewRelease = {
  version,
  releaseDate: 'Jan 31, 2025',
  items: [
    {
      title: 'AI Visual now Generally Available',
      content: (
        <>
          <ReleaseItemText>
            Ask questions of your data with voice or text and get query results,
            filtering, and additional context right in the dashboard. The AI
            visual has been improved with streaming capability for faster
            responses, targeted parameter passing, and built-in support for more
            AI model sources. Give your dashboards an AI-powered jumpstart. For
            more information, see&#32;
            <VizLink
              href="https://docs.cloudera.com/r/viz-abc468e.html"
              target="_blank"
            >
              docs.
            </VizLink>
          </ReleaseItemText>
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 1,
              index: 1,
              extension: 'gif',
            }}
            alt="AI Streaming response"
          />
        </>
      ),
    },
    {
      title: 'Trino and Phoenix Data Connections Tech Preview',
      content: (
        <>
          <ReleaseItemText>
            Two new data connections have been added to Cloudera Data
            Visualization. Upload data, query, and visualize on Trino and
            Phoenix data connections. See&#32;
            <VizLink
              href="https://docs.cloudera.com/r/viz-eee987a.html"
              target="_blank"
            >
              docs
            </VizLink>
            &#32;for the full list of supported data connections.
          </ReleaseItemText>
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 2,
              index: 1,
            }}
            alt="Phoenix and Trino dataconnection"
          />
        </>
      ),
    },
    {
      title: 'Filter improvements',
      content: (
        <>
          <ReleaseItemText>
            Filters have been improved with faster search, optimized queries and
            a clearer interface. For more information on configuring data
            options for dashboard filters, see&#32;
            <VizLink
              href="https://docs.cloudera.com/r/viz-eae012a.html"
              target="_blank"
            >
              docs.
            </VizLink>
            &#32;Display formats set for a field in the dataset will now be
            applied when displaying values. See&#32;
            <VizLink
              href="https://docs.cloudera.com/r/viz-eae345a.html"
              target="_blank"
            >
              docs
            </VizLink>
            &#32;for more information on applying display formats within a
            dataset.
          </ReleaseItemText>
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 1,
            }}
            alt="Filter display format in Data page"
          />
          <ReleaseItemImage
            srcInfo={{
              version,
              feature: 3,
              index: 2,
            }}
            alt="Filter display format in dashboard"
          />
        </>
      ),
    },
  ],
};
