import {
  UseMutationOptions,
  UseMutationResult,
  useMutation,
} from '@tanstack/react-query';
import { useNotificationStore } from 'stores/use-notification-store';

export const useMutationWithNotification = <
  TData = unknown,
  TError = unknown,
  TVariables = void,
  TContext = unknown
>(
  mutationOptions: UseMutationOptions<TData, TError, TVariables, TContext>,
  notificationOptions: {
    successMessage?: string;
    errorMessage?: string;
  }
): UseMutationResult<TData, TError, TVariables, TContext> => {
  const { addNotification } = useNotificationStore(({ addNotification }) => ({
    addNotification,
  }));

  const mutation = useMutation({
    ...mutationOptions,
    onSuccess: (...params) => {
      mutationOptions.onSuccess?.(...params);
      if (notificationOptions.successMessage) {
        addNotification(
          {
            variant: 'success',
            message: notificationOptions.successMessage,
          },
          { delay: 3000 }
        );
      }
    },
    onError: (...params) => {
      mutationOptions.onError?.(...params);
      if (notificationOptions.errorMessage) {
        addNotification(
          {
            variant: 'danger',
            message: notificationOptions.errorMessage,
          },
          { delay: 3000 }
        );
      }
    },
  });

  return mutation;
};
