import React from 'react';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';

import { useChangePasswordForm } from 'hooks/use-change-password-form';
import { Notifications } from 'components/notifications/Notifications';
import { UnauthorizedLayout } from 'components/unathorized-layout/UnauthorizedLayout';
import { UnauthorizedCard } from 'components/unauthorized-card/UnauthorizedCard';
import { UnauthorizedCardTitle } from 'components/unauthorized-card-title/UnauthorizedCardTitle';
import { UnauthorizedSubmitButton } from 'components/unauthorized-submit-button/UnauthorizedSubmitButton';

import classes from './ChangePasswordPage.module.scss';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

const { username, firstName, lastName } = arc.frontend.globals.authentication;
const displayName =
  firstName || lastName ? `${firstName} ${lastName}` : username;

export const ChangePasswordPage = () => {
  const { formFields, initialValues, onSubmit, validationSchema } =
    useChangePasswordForm({
      onSuccess: () => {
        window.location.reload();
      },
    });

  const {
    register,
    handleSubmit,
    formState: { errors, isDirty, isValid },
  } = useForm({
    defaultValues: initialValues,
    mode: 'onTouched',
    resolver: yupResolver(validationSchema),
  });

  return (
    <UnauthorizedLayout>
      <UnauthorizedCard className={classes.card}>
        <div className={classes.nameWrapper}>{displayName}</div>

        <UnauthorizedCardTitle className={classes.cardTitle}>
          Change Password
        </UnauthorizedCardTitle>

        <div className={classes.helpTextContainer}>
          <p>
            Your password either has been recently changed by an admin or you
            are still using the default username and password.
          </p>

          <p>For security reasons, please change your password.</p>
        </div>

        <form onSubmit={handleSubmit(onSubmit)}>
          {formFields(register, errors)}

          <UnauthorizedSubmitButton
            disabled={!isDirty || !isValid}
            type="submit"
            id="change-password-submit-button"
          >
            CHANGE PASSWORD
          </UnauthorizedSubmitButton>
        </form>
      </UnauthorizedCard>

      <Notifications />
    </UnauthorizedLayout>
  );
};
