import React from 'react';

import DisableIcon from '@cloudera-internal/cuix-core/icons/svg/ic-disable.svg';
import ErrorIcon from '@cloudera-internal/cuix-core/icons/svg/ic-error.svg';
import BarChartIcon from 'assets/icons/gradient-bar-chart.svg';
import ClouderaIcon from 'assets/logos/cloudera-logo-orange.svg';

import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import { getGlobalArc } from 'utils/get-global-arc';
import { NavigationBar } from 'components/navigation-bar/NavigationBar';
import classes from './ErrorPage.module.scss';

export enum ErrorPageType {
  NotFound = 'NotFound',
  Forbidden = 'Forbidden',
  Generic = 'Generic',
}

type ErrorDetails = {
  status: string;
  title: string;
  description: string;
  icon: React.FC<React.SVGProps<SVGSVGElement>>;
};

const errors = {
  [ErrorPageType.NotFound]: {
    status: '404',
    title: 'Page Not Found',
    description: 'Sorry, the page you requested was not found.',
    icon: ErrorIcon,
  },
  [ErrorPageType.Forbidden]: {
    status: '403',
    title: 'Access Forbidden',
    description:
      'Sorry, you don’t have permission to access the requested page. Please check with your system administrator to make sure you have appropriate access permissions.',
    icon: DisableIcon,
  },
  [ErrorPageType.Generic]: {
    status: '',
    title: 'Generic Error',
    description:
      'Sorry, the requested page could not be loaded. Please try again later.',
    icon: ErrorIcon,
  },
} satisfies Record<ErrorPageType, ErrorDetails>;

type ErrorPageProps = {
  type: ErrorPageType;
  showNavBar?: boolean;
};

const arc = getGlobalArc();

export const ErrorPage = ({
  type = ErrorPageType.NotFound,
  showNavBar = false,
}: ErrorPageProps): JSX.Element => {
  const Icon = errors[type].icon;

  return (
    <div className={classes.wrapper}>
      {showNavBar ? (
        <NavigationBar
          isAlreadySeen={false}
          shouldShowModal={false}
          activeTabs={{ main: null }}
        />
      ) : (
        <ClouderaIcon className={classes.clouderaIcon} />
      )}
      <BarChartIcon className={classes.bgIcon} />
      <div className={classes.container}>
        <Icon className={classes.icon} />
        <div className={classes.content}>
          <h1 className={classes.status}>{errors[type].status}</h1>
          <h2 className={classes.title}>{errors[type].title}</h2>
          <p className={classes.message}>{errors[type].description}</p>
          <ClouderaButton
            color="primary"
            onClick={() => {
              window.location.href = arc.frontend.globals.urls.homeUrl;
            }}
          >
            Return to Home
          </ClouderaButton>
        </div>
      </div>
    </div>
  );
};
