import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';
import type { DataConnection } from 'types/data-connection.type';
import { getDataConnectionExtendedTypeName } from 'utils/get-data-connection-extended-type-name';

import { PasswordIsNotSetIcon } from '../password-is-not-set-icon/PasswordIsNotSetIcon';
import { ProjectSyncedIcon } from '../project-synced-icon/ProjectSyncedIcon';
import { StatusInactiveIcon } from '../status-inactive-icon/StatusInactiveIcon';
import classes from './BottomSection.module.scss';
import classNames from 'classnames';

type BottomSectionProps = {
  id: number;
  cml: DataConnection['cml'];
  isPasswordSet: boolean;
  typeName: string;
  extendedName: DataConnection['info']['extendedName'];
};

const arc = getGlobalArc();

const isCmlSycEnabled =
  arc.frontend.featureFlipper.getValueById('enable_cml_sync');

export const BottomSection = ({
  id,
  cml,
  isPasswordSet,
  typeName,
  extendedName,
}: BottomSectionProps) => {
  let icon: JSX.Element | null = null;

  if (isCmlSycEnabled && cml?.id !== null) {
    if (!isPasswordSet) {
      icon = <PasswordIsNotSetIcon dataConnectionId={id} />;
    } else {
      icon = cml?.isAvailable ? (
        <ProjectSyncedIcon dataConnectionId={id} />
      ) : (
        <StatusInactiveIcon dataConnectionId={id} />
      );
    }
  }

  return (
    <div className={classes.bottomSection}>
      <div className={classes.connectionType}>
        <div className={classes.connectionTypeLabel}>TYPE</div>

        <div
          className={classNames(
            classes.connectionTypeExtendedName,
            'text-truncate'
          )}
        >
          {getDataConnectionExtendedTypeName({
            extendedName,
            typeName,
          })}
        </div>
      </div>

      {icon}
    </div>
  );
};
