import React, { useEffect, useState } from 'react';

import { homePageApi } from 'pages/home/api';

import type { HomePageCreatedRecently } from '../../types/home-page-created-recently.type';
import classes from './CreatedRecently.module.scss';

export const CreatedRecently = (): JSX.Element => {
  const [createdRecently, setCreatedRecently] =
    useState<HomePageCreatedRecently>({
      apps: 0,
      dashboards: 0,
      datasets: 0,
    });

  useEffect(() => {
    homePageApi.getNumberOfRecentlyCreatedResources().then(setCreatedRecently);
  }, []);

  const items = [
    {
      label: 'Dashboards were created',
      value: createdRecently.dashboards,
    },
    {
      label: 'Apps were created',
      value: createdRecently.apps,
    },
    {
      label: 'Datasets were created',
      value: createdRecently.datasets,
    },
  ];

  return (
    <div className={classes.container}>
      <h5 className="mt-0">Over the last 7 days...</h5>

      {items.map(({ label, value }) => (
        <span key={label}>
          <span className={classes.numberPill}>{value}</span> {label}
        </span>
      ))}
    </div>
  );
};
