import { getGlobalArc } from 'utils/get-global-arc';
import type {
  DataConnectionExtendedName,
  DataConnectionType,
} from 'types/data-connection.type';

const arc = getGlobalArc();

// icons (which are availble) are from https://design.cloudera.com/9f23dd142/p/929116-logos, rest is default
const logoFileNamesByDataConnectionType: Record<
  DataConnectionType,
  string | null
> = {
  arcengine_v2: 'arcadia.png',
  arcengine: 'arcadia.png',
  aster: null,
  athena: 'athena.png',
  bigquery: 'bigquery.png',
  cassandra: 'cassandra.png',
  denodo: 'denodo.png',
  drill: null,
  druid: 'druid.svg',
  duckdb: 'duckdb.png',
  elasticsearch: 'elasticsearch.png',
  hive: 'hive.svg',
  hyve: 'hive.svg',
  impala: 'impala.svg',
  impyla: 'impala.svg',
  ksql: 'kafka.svg',
  mariadb: 'mariadb.svg',
  mongodb: 'mongodb.png',
  mssqlserver: 'mssql.svg',
  mysql: 'mysql.svg',
  odbc: null,
  oracle: null,
  postgres: 'postgres.png',
  redshift: 'redshift.png',
  snowflake: 'snowflake.png',
  solr: 'solr.svg',
  sparksql: 'spark.svg',
  sqlite: 'sqlite.png',
  teradata: null,
  vertica: null,
  trino: null,
  phoenix: null,
};

export const getDataConnectionLogoFilePath = ({
  type,
  extendedName,
}: {
  type: DataConnectionType;
  extendedName?: DataConnectionExtendedName;
}) => {
  let fileName = logoFileNamesByDataConnectionType[type];

  if (!fileName) {
    return null;
  }

  const warehouseNames: Record<DataConnectionExtendedName, true> = {
    'cdw-hyve': true,
    'cdw-impyla': true,
    ssb: true,
  };

  if (extendedName && warehouseNames[extendedName]) {
    fileName = 'warehouses.svg';
  }

  return `${arc.frontend.globals.urls.staticUrl}/img/data-connection-logos/${fileName}`;
};
