import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizWrapperLink } from 'components/viz-wrapper-link/VizWrapperLink';
import type { DataConnection } from 'types/data-connection.type';

import classes from './DataConnectionTile.module.scss';
import { BottomSection } from '../bottom-section/BottomSection';
import { TopSection } from '../top-section/TopSection';
import { Icons } from '../icons/Icons';
import { getBlankTargetUrl } from 'arc/misc/misc';

export type DataConnectionTileProps = DataConnection;

const arc = getGlobalArc();

export const DataConnectionTile = ({
  id,
  type,
  typeName,
  info,
  name,
  cml,
  isPasswordSet,
  supportsImportData,
}: DataConnectionTileProps): JSX.Element => {
  const dataconnectionDetailUrl = getBlankTargetUrl(
    `${arc.frontend.globals.urls.dataUrl}?dataconnection_id=${id}&dataconnection_name=${name}`
  );

  return (
    <div className={classes.container}>
      <VizWrapperLink
        className="d-flex flex-column h-100"
        target="_blank"
        href={dataconnectionDetailUrl}
      >
        <TopSection
          id={id}
          name={name}
          extendedName={info.extendedName}
          type={type}
        />

        <BottomSection
          cml={cml}
          extendedName={info.extendedName}
          id={id}
          isPasswordSet={isPasswordSet}
          typeName={typeName}
        />
      </VizWrapperLink>

      <Icons
        id={id}
        name={name}
        connectionMaxRowUpload={info.PARAMS?.UPLOAD_MAXROWS}
        supportsImportData={supportsImportData}
        connectionType={type}
      />
    </div>
  );
};
