import classNames from 'classnames';
import React from 'react';
import Tooltip from 'react-bootstrap/Tooltip';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizTooltip } from 'components/VizTooltip';
import { VizWrapperLink } from 'components/viz-wrapper-link/VizWrapperLink';

import type { HomepageDataset } from '../../types/home-page-dataset.type';
import classes from './DatasetTile.module.scss';

export type DatasetTileProps = HomepageDataset;

const arc = getGlobalArc();

export const DatasetTile = ({
  id,
  name,
  tables,
  numberOfFields,
}: DatasetTileProps): JSX.Element => {
  const datasetUrl = `${arc.frontend.globals.urls.datasetEditUrl}/${id}`;
  const newDashboardUrl = `${arc.frontend.globals.urls.reportBuilderUrl}/0?report_type=dashboard&dataset_id=${id}`;

  return (
    <div className={classes.container}>
      <VizWrapperLink className={classes.tile} href={datasetUrl}>
        <div className="flex-grow-1">
          <div className={classNames(classes.datasetName, 'text-truncate')}>
            {name}
          </div>

          <div className={classNames(classes.tables, 'text-truncate')}>
            {tables.join(', ')}
          </div>
        </div>

        <div className="d-flex flex-column align-items-center">
          <div className={classes.numberOfColumns}>{numberOfFields}</div>

          <div className={classes.columns}>columns</div>
        </div>
      </VizWrapperLink>

      <VizTooltip
        placement="top"
        overlay={<Tooltip id={id.toString()}>New dashboard</Tooltip>}
      >
        <VizWrapperLink className={classes.newDashboard} href={newDashboardUrl}>
          <i className={classNames(classes.plusIcon, 'fa fa-plus')} />
          <i
            className={classNames(classes.dashboardIcon, 'fa sym-dashboard')}
          />
        </VizWrapperLink>
      </VizTooltip>
    </div>
  );
};
