import React, { Dispatch, SetStateAction } from 'react';
import Collapse from 'react-bootstrap/Collapse';

import { getGlobalArc } from 'utils/get-global-arc';

import { GetStartedState } from '../../types/get-started-state.type';
import classes from './GetStarted.module.scss';
import { getGetStartedItems } from './get-get-started-items';
import { GetStartedAccordionItem } from '../get-started-accordion-item/GetStartedAccordionItem';
import classNames from 'classnames';

type GetStartedProps = {
  getStartedState: GetStartedState;
  setGetStartedState: Dispatch<SetStateAction<GetStartedState>>;

  productName: string;
  canCreateDashboard: boolean;
  canCreateApp: boolean;
};

const arc = getGlobalArc();

const isExploreStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_explore_style'
);

export const GetStarted = ({
  getStartedState: { isOpen, isVisible },
  setGetStartedState,
  productName,
  canCreateDashboard,
  canCreateApp,
}: GetStartedProps): JSX.Element | null => {
  if (!isVisible) {
    return null;
  }

  const items = getGetStartedItems({
    productName,
    canCreateApp,
    canCreateDashboard,
  });

  const toggleIsOpen = () => {
    setGetStartedState((state) => ({ ...state, isOpen: !state.isOpen }));
  };

  const toggleIsVisible = () => {
    setGetStartedState((state) => ({ ...state, isVisible: !state.isVisible }));
  };

  return (
    <div className={classes.container}>
      <div
        className={classNames(classes.header, 'px-2 py-1')}
        onClick={toggleIsOpen}
      >
        <i
          className={classNames(
            classes.arrowIcon,
            isOpen ? classes.expanded : '',
            'fa fa-caret-right'
          )}
        />

        <div className={classes.title}>Get Started</div>

        {!isExploreStyleEnabled && (
          <i className="fa fa-1 fa-times" onClick={toggleIsVisible} />
        )}
      </div>

      {/* This extra divs are needed for smooth animations in version 1.6.5 -> https://react-bootstrap-v4.netlify.app/utilities/transitions/#collapse */}
      <div>
        <Collapse in={!!isOpen}>
          <div>
            {items.map((item, index) => (
              <GetStartedAccordionItem
                key={item.title}
                {...item}
                index={index}
              />
            ))}
          </div>
        </Collapse>
      </div>
    </div>
  );
};
