import classNames from 'classnames';
import React from 'react';
import { Tooltip } from 'react-bootstrap';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizTooltip } from 'components/VizTooltip';
import { VizWrapperLink } from 'components/viz-wrapper-link/VizWrapperLink';
import { useImportDataModalStore } from 'stores/use-import-data-modal-store';

import classes from './Icons.module.scss';
import { getBlankTargetUrl } from 'arc/misc/misc';

type IconsProps = {
  id: number;
  name: string;
  supportsImportData: boolean;
  connectionMaxRowUpload: number | undefined;
  connectionType: string | undefined;
};

const arc = getGlobalArc();
const isExploreStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_explore_style'
);
export const Icons = ({
  id,
  name,
  supportsImportData,
  connectionMaxRowUpload,
  connectionType,
}: IconsProps): JSX.Element => {
  const openImportDataModal = useImportDataModalStore(({ open }) => open);
  const sqlPageUrl = `${arc.frontend.globals.urls.sqlUrl}?dataconnection_id=${id}&dataconnection_name=${name}#sql-tab`;
  const datasetModalUrl = `${arc.frontend.globals.urls.dataUrl}?&dataconnection_id=${id}&dataconnection_name=${name}#dataset-modal`;
  const redirectUrl = getBlankTargetUrl(
    isExploreStyleEnabled ? sqlPageUrl : datasetModalUrl
  );

  return (
    <>
      <VizWrapperLink
        href={redirectUrl}
        target="_blank"
        className={classNames(classes.addDatasetIcon, classes.iconWrapper)}
      >
        <VizTooltip
          placement="top"
          overlay={<Tooltip id={id.toString()}>Add Dataset</Tooltip>}
        >
          <div>
            <i className={classNames(classes.plusIcon, 'fa fa-plus')} />
            <i className={classNames('fa fa-cubes')} />
          </div>
        </VizTooltip>
      </VizWrapperLink>

      {supportsImportData && (
        <div
          onClick={() => {
            openImportDataModal({
              connectionId: id,
              connectionMaxRowUpload,
              connectionType,
            });
          }}
          className={classNames(classes.addDataIcon, classes.iconWrapper)}
        >
          <VizTooltip
            placement="top"
            overlay={<Tooltip id={id.toString()}>Add Data</Tooltip>}
          >
            <div>
              <i className={classNames(classes.plusIcon, 'fa fa-plus')} />
              <i className="fa fa-align-justify" />
            </div>
          </VizTooltip>
        </div>
      )}
    </>
  );
};
