import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';
import { VizLink } from 'components/viz-link/VizLink';

const arc = getGlobalArc();

const isExploreStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_explore_style'
);

export const getLearnItems = ({
  forceOpenGetStartedAccordion,
  openWhatsNewModal,
}: {
  forceOpenGetStartedAccordion: () => void;
  openWhatsNewModal: () => void;
}) => {
  const learnItems: { id: string; element: JSX.Element }[] = [];

  if (
    arc.frontend.globals.settings.shouldDisplayGetStarted &&
    !isExploreStyleEnabled
  ) {
    learnItems.push({
      id: 'get-started',
      element: (
        <VizLink onClick={forceOpenGetStartedAccordion}>Get Started</VizLink>
      ),
    });
  }

  if (arc.frontend.globals.settings.shouldDisplayWhatSNew) {
    learnItems.push({
      id: 'what-s-new',
      element: (
        <>
          <VizLink onClick={openWhatsNewModal}>
            What's New in {arc.frontend.globals.settings.whatSNewVersion}
          </VizLink>
        </>
      ),
    });
  }

  if (arc.frontend.globals.settings.shouldDisplayDocumentation) {
    learnItems.push({
      id: 'documentation',
      element: (
        <VizLink href={arc.frontend.globals.documentation.url} target="_blank">
          {arc.frontend.globals.documentation.label}
        </VizLink>
      ),
    });
  }

  arc.frontend.globals.settings.customLinks.forEach((item) => {
    if (item.type === 'link') {
      learnItems.push({
        id: `${item.label}-${item.link}`,
        element: (
          <VizLink href={item.link} target="_blank">
            {item.label}
          </VizLink>
        ),
      });
    }
  });

  return learnItems;
};
