import classNames from 'classnames';
import { VizWrapperLink } from 'components/viz-wrapper-link/VizWrapperLink';
import { homePageApi } from 'pages/home/api';
import React, { useEffect, useState } from 'react';
import * as d3 from 'd3';

import type { HomePageStatistics } from '../../types/home-page-statistics.type';
import { LoadingStripesPlaceholder } from '../loading-stripes-placeholder/LoadingStripesPlaceholder';
import { getOverviews } from './get-overviews';
import classes from './Overviews.module.scss';

export const Overviews = (): JSX.Element => {
  const [statistics, setStatistics] = useState<HomePageStatistics>({
    apps: 0,
    dashboards: 0,
    dataConnections: 0,
    datasets: 0,
    queries: 0,
    views: 0,
  });
  const [isLoading, setIsLoading] = useState(false);

  useEffect(() => {
    setIsLoading(true);
    homePageApi.getStatistics().then((statistics) => {
      setStatistics(statistics);
      setIsLoading(false);
    });
  }, []);

  const overviews = getOverviews(statistics);

  return (
    <div className={classes.overviews}>
      {overviews.map(
        ({ icon, href, label, backgroundColor, number, order, testId }) => (
          <VizWrapperLink
            className={classNames(
              classes.overview,
              href ? classes.clickable : ''
            )}
            key={label}
            href={href}
            data-test-id={testId}
            style={{ order }}
          >
            <div className={classes.icon} style={{ backgroundColor }}>
              {icon}
            </div>

            <div className={classes.statistics}>
              {isLoading && (
                <LoadingStripesPlaceholder
                  height="100%"
                  width="100%"
                  numberOfStipes={2}
                />
              )}
              {!isLoading && (
                <>
                  <div className={classes.statisticsNumber}>
                    {d3.format(number > 999 ? ',.3s' : ',.0f')(number)}
                  </div>

                  <div className={classes.statisticsLabel}>{label}</div>
                </>
              )}
            </div>
          </VizWrapperLink>
        )
      )}
    </div>
  );
};
