import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';

import type { HomePageStatistics } from '../../types/home-page-statistics.type';

type Overview = {
  icon: JSX.Element;
  href: string | undefined;
  label: string;
  order: number;
  backgroundColor: string;
  number: number;
  testId: string;
};

const arc = getGlobalArc();

const isExploreStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_explore_style'
);

const isClouderaStyleEnabled = arc.frontend.featureFlipper.getValueById(
  'enable_cloudera_style'
);

export const getOverviews = (
  overviewStatistics: HomePageStatistics
): Overview[] => {
  const overviews: Overview[] = [
    {
      href: 'data',
      icon: <i className="fa fa-cubes" />,
      label: 'Datasets',
      number: overviewStatistics.datasets,
      backgroundColor: '#F4B371',
      order: isExploreStyleEnabled ? 4 : isClouderaStyleEnabled ? 3 : 3,
      testId: 'overview-datasets',
    },
    {
      href: 'apps?displaymode=dashboards',
      icon: <i className="sym-dashboard" />,
      label: 'Dashboards',
      number: overviewStatistics.dashboards,
      backgroundColor: '#4CB6C1',
      order: isExploreStyleEnabled ? 2 : isClouderaStyleEnabled ? 4 : 1,
      testId: 'overview-dashboards',
    },
    {
      href: 'apps?displaymode=apps',
      icon: <i className="sym-app" />,
      label: 'Apps',
      number: overviewStatistics.apps,
      backgroundColor: '#AD73B1',
      order: isExploreStyleEnabled ? 3 : isClouderaStyleEnabled ? 5 : 2,
      testId: 'overview-apps',
    },
  ];

  if (arc.frontend.globals.canUserExploreAnyDataconnections) {
    overviews.push({
      href: 'datasql?active_data_menu=menu-history',
      label: 'Queries',
      backgroundColor: '#B1CB72',
      icon: (
        <>
          <span
            style={{
              fontFamily:
                'Impact, Haettenschweiler, "Franklin Gothic Bold", "Arial Black", "sans-serif"',
            }}
          >
            {'{'}
          </span>
          <span
            style={{
              transform: 'translateY(-7px)',
              fontWeight: 'bold',
            }}
          >
            ...
          </span>
          <span
            style={{
              fontFamily:
                'Impact, Haettenschweiler, "Franklin Gothic Bold", "Arial Black", "sans-serif"',
            }}
          >
            {'}'}
          </span>
        </>
      ),
      number: overviewStatistics.queries,
      order: isClouderaStyleEnabled || isExploreStyleEnabled ? 1 : 4,
      testId: 'overview-queries',
    });
  }

  if (isClouderaStyleEnabled) {
    overviews.push({
      backgroundColor: '#9675D1',
      href: 'data',
      icon: <i className="fa fa-link" />,
      label: 'Connections',
      number: overviewStatistics.dataConnections,
      order: isExploreStyleEnabled ? 5 : isClouderaStyleEnabled ? 2 : 6,
      testId: 'overview-connections',
    });
  }

  if (!isExploreStyleEnabled) {
    overviews.push({
      href: arc.frontend.globals.permissions.system.includes('sys_viewlogs')
        ? '/arc/apps/activity#tab-users-and-apps'
        : undefined,
      icon: <i className="fa fa-eye" />,
      label: 'Total Views',
      number: overviewStatistics.views,
      backgroundColor: '#7C8B92',
      order: isClouderaStyleEnabled || isExploreStyleEnabled ? 6 : 5,
      testId: 'overview-total-views',
    });
  }

  return overviews;
};
