import classNames from 'classnames';
import React from 'react';

import { DataConnectionTileIcon } from '../data-connection-tile-icon/DataConnectionTileIcon';
import classes from './PasswordIsNotSetIcon.module.scss';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

type PasswordIsNotSetIconProps = {
  dataConnectionId: number;
};

export const PasswordIsNotSetIcon = ({
  dataConnectionId,
}: PasswordIsNotSetIconProps): JSX.Element => (
  <DataConnectionTileIcon
    id={dataConnectionId.toString()}
    icon={
      <i
        className={classNames(classes.orangeIcon, 'fa fa-exclamation-circle')}
      />
    }
    tooltip={{
      content: (
        <ol>
          <li>
            In the control panel, add a workload password on your profile page
          </li>

          <li>
            In your Cloudera AI User Settings add the password to
            "WORKLOAD_PASSWORD" under the "Environment Variables" tab
          </li>

          <li>Restart the "{arc.frontend.globals.product.name}" application</li>
        </ol>
      ),
      title: 'Setting Workload password steps',
    }}
  />
);
