import React from 'react';

import { getGlobalArc } from 'utils/get-global-arc';
import { SqlQuery } from 'types/sql-query.type';
import { VizWrapperLink } from 'components/viz-wrapper-link/VizWrapperLink';

import classes from './SqlQueryTile.module.scss';
import { SqlSyntaxHighlighter } from '../sql-syntax-highlighter/SqlSyntaxHighlighter';

export type SqlQueryTileProps = SqlQuery;

const arc = getGlobalArc();

export const SqlQueryTile = ({
  id,
  dataconnection,
  isFavorite,
  query,
  tableNames,
}: SqlQueryTileProps): JSX.Element => {
  const sqlQueryUrl = `${arc.frontend.globals.urls.sqlUrl}?dataconnection_id=${dataconnection.id}&dataconnection_name=${dataconnection.name}&query_id=${id}&favorite=${isFavorite}#sql-tab`;

  return (
    <VizWrapperLink className={classes.tile} href={sqlQueryUrl}>
      <div className={classes.sqlQuery}>
        <SqlSyntaxHighlighter query={query} />
      </div>

      <div className="text-truncate">
        <i className="fa fa-link" /> {dataconnection.name}
      </div>

      {tableNames.length > 0 && (
        <div className="text-truncate">
          <i className="fa fa-table" /> {tableNames.join(', ')}
        </div>
      )}
    </VizWrapperLink>
  );
};
