import classNames from 'classnames';
import React from 'react';
import { Tooltip } from 'react-bootstrap';

import { VizTooltip } from 'components/VizTooltip';
import type { DataConnection } from 'types/data-connection.type';

import classes from './TopSection.module.scss';
import { DataConnectionIcon } from '../data-connection-icon/DataConnectionIcon';

export const TopSection = ({
  id,
  name,
  type,
  extendedName,
}: {
  id: number;
  name: string;
  type: DataConnection['type'];
  extendedName: DataConnection['info']['extendedName'];
}) => {
  return (
    <div className="d-flex m-1 pt-3 align-items-center flex-grow-1">
      <div className={classes.logoContainer}>
        <DataConnectionIcon
          type={type}
          extendedName={extendedName}
        ></DataConnectionIcon>
      </div>

      <VizTooltip
        placement="bottom"
        overlay={<Tooltip id={id.toString()}>{name}</Tooltip>}
      >
        <div className={classNames(classes.connectionName, 'text-truncate')}>
          {name}
        </div>
      </VizTooltip>
    </div>
  );
};
