import RefreshIcon from '@cloudera-internal/cuix-core/icons/svg/ic-refresh.svg';
import { useQueryClient } from '@tanstack/react-query';
import { QueryKeyBase } from 'apis/query-keys';
import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import { NavigationBar } from 'components/navigation-bar/NavigationBar';
import React, { useCallback, useState } from 'react';
import { Container, Tab, Tabs } from 'react-bootstrap';
import classes from './JobOverviewPage.module.scss';
import { LatestRunsTable } from './components/latest-runs-table/LatestRunsTable';
import { ScheduleGroupsTable } from './components/schedule-groups-table/ScheduleGroupsTable';
import { ScheduledJobsTable } from './components/scheduled-jobs-table/ScheduledJobsTable';

type JobOverviewPageProps = {
  shouldShowModal: boolean;
  isAlreadySeen: boolean;
};

export const JobOverviewPage = ({
  shouldShowModal,
  isAlreadySeen,
}: JobOverviewPageProps): JSX.Element => {
  const queryClient = useQueryClient();
  const [tab, setTab] = useState(window.location.hash?.slice(1) || 'runs');

  const selectTab = useCallback((tab: string | null) => {
    if (!tab) {
      return;
    }

    setTab(tab);
    window.location.hash = tab;
  }, []);

  const handleRefresh = useCallback(() => {
    queryClient.invalidateQueries([QueryKeyBase.Jobs]);
    queryClient.invalidateQueries([QueryKeyBase.ScheduleGroups]);
  }, [queryClient]);

  return (
    <Container fluid className={classes.container}>
      <NavigationBar
        isAlreadySeen={isAlreadySeen}
        shouldShowModal={shouldShowModal}
        activeTabs={{
          main: 'settings',
          sub: 'navbar-settings-menu-job-status',
        }}
      />

      <div className="m-3">
        <div
          data-test-id="jobsHeader"
          className="d-flex justify-content-between mb-3"
        >
          <h2>Jobs</h2>
          <ClouderaButton
            icon={<RefreshIcon />}
            color="primary"
            onClick={handleRefresh}
            testId="refreshJobs"
          >
            Refresh
          </ClouderaButton>
        </div>

        <Tabs className={classes.jobTabs} activeKey={tab} onSelect={selectTab}>
          <Tab
            eventKey="runs"
            title="Runs (Latest)"
            data-test-id="runsTab"
            id="runsTab"
          >
            <LatestRunsTable />
          </Tab>
          <Tab
            eventKey="scheduledJobs"
            title="Scheduled Jobs"
            data-test-id="scheduledJobsTab"
            id="scheduledJobsTab"
          >
            <ScheduledJobsTable />
          </Tab>
          <Tab
            eventKey="intervals"
            title="Intervals"
            data-test-id="intervalsTab"
            id="intervalsTab"
          >
            <ScheduleGroupsTable />
          </Tab>
        </Tabs>
      </div>
    </Container>
  );
};
