import { UNITS } from 'components/cron/constants';
import { parsePartString } from 'components/cron/converter';
import * as yup from 'yup';

export const intervalSchema = yup.object({
  name: yup.string().required().label('Name'),
  interval: yup
    .string()
    .test(
      'is-not-too-frequent',
      () => 'Do not schedule triggers less than 10 minutes apart',
      (value) => {
        if (!value) {
          return true;
        }

        const minutes = parsePartString(value.split(' ')[0], UNITS[0]);
        const isTooFrequent = minutes.some(
          (value, i, array) => i !== 0 && array[i - 1] + 10 > value
        );

        return !isTooFrequent;
      }
    )
    .required()
    .label('Interval'),
});

export type IntervalSchema = yup.InferType<typeof intervalSchema>;
