import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import '@testing-library/jest-dom';
import {
  configure,
  fireEvent,
  render,
  screen,
  waitFor,
  within,
} from '@testing-library/react';
import * as deleteScheduleGroups from 'apis/jobs/schedule-groups/delete-schedule-group';
import * as getScheduleGroups from 'apis/jobs/schedule-groups/get-schedule-groups';
import { mockScheduleGroups } from 'pages/job-page/utils/mocks';
import * as React from 'react';
import { ScheduleGroupsTable } from './ScheduleGroupsTable';

configure({ testIdAttribute: 'data-test-id' });

window.arc.frontend = {
  globals: { urls: { previewCustomEmailUrl: 'customUrl' } },
} as any;

jest.mock('arc/misc/misc', () => ({
  getBlankTargetUrl: (url: string) => url,
}));
jest.mock('components/navigation-bar/NavigationBar', () => ({
  NavigationBar: () => null,
}));
jest.mock('@cloudera-internal/cuix-core/variables', () => ({
  cdlRed500: '#fff',
  cdlBlue500: '#fff',
  cdlGreen500: '#fff',
  cdlOrange500: '#fff',
}));

beforeAll(() => {
  jest
    .spyOn(deleteScheduleGroups, 'deleteScheduleGroups')
    .mockImplementation(jest.fn());
});

beforeEach(() => {
  jest
    .spyOn(getScheduleGroups, 'getScheduleGroups')
    .mockResolvedValue(mockScheduleGroups);
});

afterEach(() => {
  jest.resetAllMocks();
});

describe('ScheduleGroupsTable component', () => {
  it('renders without error', () => {
    render(
      <QueryClientProvider client={new QueryClient()}>
        <ScheduleGroupsTable />
      </QueryClientProvider>
    );
  });

  describe('displays the runs in a table', () => {
    it('should display a confirmation modal for the specified job', () => {
      render(
        <QueryClientProvider client={new QueryClient()}>
          <ScheduleGroupsTable />
        </QueryClientProvider>
      );

      expect(screen.getByTestId('scheduleGroupsTable')).toBeInTheDocument();
    });
  });

  describe('when clicking on the Delete button', () => {
    it('should display a confirmation modal for the specified jobs', async () => {
      render(
        <QueryClientProvider client={new QueryClient()}>
          <ScheduleGroupsTable />
        </QueryClientProvider>
      );

      fireEvent.click(
        (await screen.findAllByRole('checkbox', { name: 'Select Interval' }))[1]
      );

      const deleteButton = screen.getByTestId('bulkDeleteScheduleGroups');
      expect(deleteButton).not.toBeDisabled();
      fireEvent.click(deleteButton);

      expect(screen.getByText('Delete Intervals')).toBeInTheDocument();
      const bulkDeleteModal = screen.getByTestId('bulkModal');
      expect(within(bulkDeleteModal).getByText('Daily')).toBeInTheDocument();

      fireEvent.click(screen.getByTestId('deleteConfirm'));
      await waitFor(() => {
        expect(deleteScheduleGroups.deleteScheduleGroups).toHaveBeenCalledWith([
          2,
        ]);
      });
    });
  });

  describe('when clicking on the Create New button', () => {
    it('should display the interval modal', async () => {
      render(
        <QueryClientProvider client={new QueryClient()}>
          <ScheduleGroupsTable />
        </QueryClientProvider>
      );

      fireEvent.click(screen.getByTestId('createInterval'));
      expect(screen.getByText('Create Interval')).toBeInTheDocument();
    });
  });
});
