import moment from 'moment';
import React from 'react';
import { Col, Container, Row } from 'react-bootstrap';
import { JobLog } from 'types/job-log.type';
import classes from '../../JobPage.module.scss';
import { StatusIndicator } from '../status-indicator/StatusIndicator';

type LogDetailsGridProps = {
  log: JobLog;
};

export const LogDetailsGrid = ({ log }: LogDetailsGridProps): JSX.Element => {
  return (
    <Container fluid className={classes.detailContainer}>
      <Row xs={1} sm={2} md={3}>
        <Col>
          <label>STATUS</label>
          <StatusIndicator status={log.status} isAnimated />
        </Col>
        <Col>
          <label>LOG ID</label>
          <p aria-label="Log Id">{log.id}</p>
        </Col>
        <Col>
          <label>JOB ID</label>
          <p aria-label="Job Id">{log.jobId}</p>
        </Col>
      </Row>
      {!!log.errorMessage && (
        <Row>
          <Col>
            <label>ERROR</label>
            <p aria-label="Error">{log.errorMessage}</p>
          </Col>
        </Row>
      )}
      <Row xs={1} sm={2} md={3}>
        <Col>
          <label>OWNER</label>
          <p aria-label="Owner username">{log.ownerUsername || 'No owner'}</p>
        </Col>
        {!!log.editorUsername && (
          <Col>
            <label>TRIGGERED BY</label>
            <p aria-label="Triggered By">{log.editorUsername}</p>
          </Col>
        )}
        <Col>
          <label>TRIGGERED</label>
          <p data-test-id="triggerMode">
            {log.triggeredManually ? 'Manually' : 'Automatic schedule'}
          </p>
        </Col>
      </Row>
      <Row xs={1} sm={2} md={3}>
        <Col>
          <label>START TIME</label>
          <p>{moment.utc(log.startTime).format('YYYY-MM-DD HH:mm')}</p>
        </Col>
        <Col>
          <label>END TIME</label>
          <p>{moment.utc(log.endTime).format('YYYY-MM-DD HH:mm')}</p>
        </Col>
        <Col>
          <label>TOTAL RUN TIME</label>
          <p>
            {moment
              .duration(moment(log.endTime).diff(log.startTime))
              .humanize()}
          </p>
        </Col>
      </Row>
    </Container>
  );
};
