import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import {
  ClouderaDialogBody,
  ClouderaDialogContent,
  ClouderaDialogFooter,
  ClouderaDialogHeader,
  ClouderaDialogTitle,
  useClouderaDialogContext,
} from 'components/cloudera-dialog/ClouderaDialog';
import { Notification } from 'components/notification/Notification';
import React from 'react';
import { Tab, Tabs } from 'react-bootstrap';
import { JobLog } from 'types/job-log.type';
import { Job, JobStatus, JobType } from 'types/job.type';
import classes from 'components/email-job-modal/EmailJobModal.module.scss';
import { EmailDetailsGrid } from '../email-details-grid/EmailDetailsGrid';
import { ExtractDetailsGrid } from '../extract-details-grid/ExtractDetailsGrid';
import { LogDetailsGrid } from '../log-details-grid.tsx/LogDetailsGrid';
import { RunDetailsTasksTable } from '../run-details-tasks-table/RunDetailsTasksTable';
import { ScheduleDetailsGrid } from '../schedule-details-grid/ScheduleDetailsGrid';

type RunDetailsModalProps = {
  log?: JobLog;
  job?: Job;
};

export const RunDetailsModal = ({
  log,
  job,
}: RunDetailsModalProps): JSX.Element => {
  const { reset } = useClouderaDialogContext();

  const dataCopyTarget =
    job?.type === JobType.DataExtract
      ? job?.dataCopyTarget
      : log?.dataCopyTarget;

  const emailTarget =
    job?.type === JobType.Email ? job?.emailTarget : log?.emailTarget;

  const snapshotTarget =
    job?.type === JobType.Email ? job?.snapshotTarget : log?.snapshotTarget;

  return (
    <ClouderaDialogContent size="lg" scrollable testId="jobDetailsModal">
      <ClouderaDialogHeader closeButton>
        <ClouderaDialogTitle>{job ? 'Job' : 'Run'} Details</ClouderaDialogTitle>
      </ClouderaDialogHeader>

      <ClouderaDialogBody>
        {!!job && <ScheduleDetailsGrid job={job} />}
        {!!log && <LogDetailsGrid log={log} />}
        {!!log &&
          !emailTarget &&
          !dataCopyTarget &&
          ![JobStatus.Running, JobStatus.Queued].includes(log.status) && (
            <Notification
              message="Some information is missing because the run was incomplete."
              variant="warning"
            />
          )}
        {!log && !job && <p>The details for this asset are not available</p>}
        <Tabs
          defaultActiveKey={
            !!log?.taskList.length ? 'tasks' : emailTarget ? 'email' : 'extract'
          }
          className={classes.editModalTabs}
        >
          {!!log?.taskList.length && (
            <Tab eventKey="tasks" title="Tasks" data-test-id="taskDetailsTab">
              <RunDetailsTasksTable tasks={log?.taskList} />
            </Tab>
          )}
          {!!emailTarget && (
            <Tab
              eventKey="email"
              title="Email Details"
              data-test-id="emailDetailsTab"
            >
              <EmailDetailsGrid
                emailTarget={emailTarget}
                snapshotTarget={snapshotTarget}
              />
            </Tab>
          )}
          {!!dataCopyTarget && (
            <Tab
              eventKey="extract"
              title="Extract Details"
              data-test-id="extractDetailsTab"
            >
              <ExtractDetailsGrid dataCopyTarget={dataCopyTarget} />
            </Tab>
          )}
        </Tabs>
      </ClouderaDialogBody>

      <ClouderaDialogFooter>
        <ClouderaButton
          color="primary"
          onClick={reset}
          testId="closeRunDetails"
        >
          Close
        </ClouderaButton>
      </ClouderaDialogFooter>
    </ClouderaDialogContent>
  );
};
