import React from 'react';
import { Col, Container, Row } from 'react-bootstrap';
import RunningIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-running.svg';
import StatusSuspendedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-suspended.svg';
import { Job } from 'types/job.type';
import classes from '../../JobPage.module.scss';
import { useGetScheduleGroupsQuery } from 'apis/jobs/schedule-groups/get-schedule-groups';

type ScheduleDetailsGridProps = {
  job: Job;
};

export const ScheduleDetailsGrid = ({
  job,
}: ScheduleDetailsGridProps): JSX.Element => {
  const { data: scheduleGroup } = useGetScheduleGroupsQuery({
    enabled: job.scheduleGroup && !job.scheduleGroup.name,
    select: (scheduleGroups) =>
      scheduleGroups.find(
        (scheduleGroup) => scheduleGroup.id === job.scheduleGroup?.id
      ),
  });

  return (
    <Container fluid className={classes.detailContainer}>
      <Row xs={1} sm={2} md={3}>
        {!!job.scheduleGroup && (
          <Col className="d-flex align-items-center">
            {job.isSchedulingPaused ? (
              <StatusSuspendedIcon fontSize={24} />
            ) : (
              <RunningIcon fontSize={24} />
            )}
            <div className="ml-2">
              <label>SCHEDULE STATUS</label>
              <p aria-label="Schedule Status">
                {job.isSchedulingPaused ? 'Paused' : 'Active'}
              </p>
            </div>
          </Col>
        )}

        <Col>
          <label>NAME</label>
          <p aria-label="Name">{job.name}</p>
        </Col>
        <Col>
          <label>ID</label>
          <p aria-label="Id">{job.id}</p>
        </Col>
        <Col>
          <label>INTERVAL</label>
          <p aria-label="Interval">
            {job.scheduleGroup ? 'Scheduled' : 'Adhoc'}
          </p>
        </Col>
        {!!job.scheduleGroup && (
          <Col>
            <label>SCHEDULE</label>
            <p aria-label="Schedule">
              {job.scheduleGroup.name || scheduleGroup?.name}
            </p>
          </Col>
        )}
        <Col>
          <label>TYPE</label>
          <p aria-label="Type">{job.type}</p>
        </Col>
        <Col>
          <label>OWNER</label>
          <p aria-label="Owner">{job.owner || 'No owner'}</p>
        </Col>
        <Col>
          <label>THRESHOLD</label>
          <p aria-label="Threshold">{job.thresholdTarget ? 'Yes' : 'No'}</p>
        </Col>
      </Row>
    </Container>
  );
};
