import React, { useMemo } from 'react';
import CancelIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-stopped.svg';
import ErrorIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-error.svg';
import CompletedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-success.svg';
import RunningIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-in-progress.svg';
import QueuedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-waiting.svg';
import UnknownIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-unknown.svg';
import SuspendedIcon from '@cloudera-internal/cuix-core/icons/svg/ic-status-suspended.svg';
import { JobStatus, SchedulingStatus } from 'types/job.type';
import classNames from 'classnames';

type StatusProps = {
  status?: JobStatus | SchedulingStatus;
  isAnimated?: boolean;
};

export const StatusIndicator = ({
  status,
  isAnimated = false,
}: StatusProps): JSX.Element => {
  const { Icon, label, className } = useMemo(() => {
    switch (status) {
      case JobStatus.Cancelled:
        return { Icon: CancelIcon, label: 'Cancelled' };
      case JobStatus.Error:
        return { Icon: ErrorIcon, label: 'Error' };
      case JobStatus.Finished:
        return { Icon: CompletedIcon, label: 'Finished' };
      case JobStatus.Queued:
        return { Icon: QueuedIcon, label: 'Queued' };
      case JobStatus.Running:
        return {
          Icon: RunningIcon,
          label: 'Running',
          className: isAnimated ? 'fa-spin' : '',
        };
      case SchedulingStatus.Paused:
        return { Icon: SuspendedIcon, label: 'Scheduling Paused' };
      default:
        return { Icon: UnknownIcon, label: 'Pending' };
    }
  }, [status, isAnimated]);

  return (
    <div className="d-flex align-items-center">
      <Icon fontSize="20" className={classNames('flex-shrink-0', className)} />
      <span className="ml-1">{label}</span>
    </div>
  );
};
