import React from 'react';
import { Col, Container, Row } from 'react-bootstrap';
import { JobThresholdTriggerType, ThresholdTarget } from 'types/job.type';
import classes from '../../JobPage.module.scss';
import { getMinimumTimeIntervalLabel } from 'pages/job-page/utils/get-minimum-time-interval';

type ThresholdDetailsGridProps = {
  thresholdTarget: ThresholdTarget;
};

export const ThresholdDetailsGrid = ({
  thresholdTarget,
}: ThresholdDetailsGridProps): JSX.Element => {
  return (
    <Container fluid className={classes.detailContainer}>
      <Row xs={1} sm={2} md={3}>
        <Col>
          <label>TRIGGER VISUAL ID</label>
          <p aria-label="Trigger Visual ID">{thresholdTarget.triggerId}</p>
        </Col>
        <Col>
          <label>TRIGGER TYPE</label>
          <p aria-label="Trigger Type">
            {thresholdTarget.triggerType ===
            JobThresholdTriggerType.ThresholdMet
              ? 'Threshold met'
              : 'Value changed since last assessment'}
          </p>
        </Col>
        <Col>
          <label>TRIGGER CONDITIONS</label>

          {thresholdTarget.triggerConditions.map((c, i) => (
            <p
              key={i}
              aria-label="Trigger Condition"
            >{`${c.equality} ${c.value}`}</p>
          ))}
        </Col>
        {!!thresholdTarget.minimumTimeInterval && (
          <Col>
            <label>SEND EMAIL LIMIT</label>
            <p aria-label="Email Limit">
              {getMinimumTimeIntervalLabel(thresholdTarget.minimumTimeInterval)}
            </p>
          </Col>
        )}
      </Row>
    </Container>
  );
};
