import React from 'react';

import { useToggle } from 'hooks/use-toggle';
import { ChromeBrowserSuggestion } from 'components/chrome-browser-suggestion/ChromeBrowserSuggestion';
import { UnauthorizedCard } from 'components/unauthorized-card/UnauthorizedCard';
import { UnauthorizedCardTitle } from 'components/unauthorized-card-title/UnauthorizedCardTitle';
import { UnauthorizedLayout } from 'components/unathorized-layout/UnauthorizedLayout';

import classes from './LoginPage.module.scss';
import { ForgotPasswordModal } from './components/forgot-password-modal/ForgotPasswordModal';
import { LoginForm } from './components/login-form/LoginForm';

type LoginPageProps = {
  next: string;
  errorMessage: string;
  failureLimit: number;
  failureLimitRation: number;
  loginInstructions: string;
  querystring: string;
  showHelpText: boolean;
};

export const LoginPage = ({
  next,
  errorMessage,
  failureLimit,
  failureLimitRation,
  loginInstructions,
  querystring,
  showHelpText,
}: LoginPageProps): JSX.Element => {
  const [
    isForgotPasswordModalOpen,
    toggleForgotPasswordModal,
    closeForgotPasswordModal,
  ] = useToggle();

  return (
    <UnauthorizedLayout>
      <div className="d-flex flex-column my-0 mx-auto">
        <UnauthorizedCard className={classes.card}>
          <UnauthorizedCardTitle className={classes.cardTitle}>
            Login
          </UnauthorizedCardTitle>

          <LoginForm
            next={next}
            errorMessage={errorMessage}
            failureLimit={failureLimit}
            failureLimitRation={failureLimitRation}
            loginInstructions={loginInstructions}
            querystring={querystring}
            toggleForgotPasswordModal={toggleForgotPasswordModal}
          />
        </UnauthorizedCard>

        <ChromeBrowserSuggestion />

        {showHelpText && (
          <p>
            It looks like this is your first time logging in. The default
            administrative username is 'vizapps_admin' and the default password
            is 'vizapps_admin'. <br />
            Please change the password as soon as you register.
          </p>
        )}
      </div>

      <ForgotPasswordModal
        isOpen={isForgotPasswordModalOpen}
        close={closeForgotPasswordModal}
      />
    </UnauthorizedLayout>
  );
};
