import { ClouderaButton } from 'components/cloudera-button/ClouderaButton';
import {
  ClouderaDialogBody,
  ClouderaDialogContent,
  ClouderaDialogFooter,
  ClouderaDialogHeader,
  useClouderaDialogContext,
} from 'components/cloudera-dialog/ClouderaDialog';
import React from 'react';
import { Modal } from 'react-bootstrap';
import Delete from '@cloudera-internal/cuix-core/icons/svg/ic-delete.svg';
import { useMutation, useQueryClient } from '@tanstack/react-query';
import { useNotificationStore } from 'stores/use-notification-store';
import { deleteAlias } from '../../../apis/aliases/delete-alias';
import { appsQueryToken } from 'apis/get-apps';
import { reportsQueryToken } from 'apis/get-reports';
import { aliasesQueryToken } from '../../../apis/aliases/get-all-aliases';

export type DeleteAliasModalProps = {
  slug: string;
};

export const DeleteAliasModal = ({ slug }: DeleteAliasModalProps) => {
  const { reset: resetDialog } = useClouderaDialogContext();
  const { addNotification } = useNotificationStore(({ addNotification }) => ({
    addNotification,
  }));
  const { reset } = useClouderaDialogContext();

  const queryClient = useQueryClient();

  const handleDeleteAlias = useMutation({
    mutationFn: () => deleteAlias({ slug: slug }),
    onSuccess: () => {
      addNotification(
        {
          variant: 'success',
          message: `The ${slug} alias has been deleted`,
        },
        { delay: 2000 }
      );
      queryClient.invalidateQueries([aliasesQueryToken]);
      queryClient.invalidateQueries([reportsQueryToken]);
      queryClient.invalidateQueries([appsQueryToken]);
      resetDialog();
    },
    onError: () => {
      addNotification(
        {
          variant: 'danger',
          message: 'Failed to delete alias',
        },
        { delay: 3000 }
      );
    },
  });

  return (
    <ClouderaDialogContent>
      <ClouderaDialogHeader closeButton>
        <Modal.Title>Delete URL Alias</Modal.Title>
      </ClouderaDialogHeader>
      <ClouderaDialogBody>
        Are you sure you want to delete the selected alias?{' '}
      </ClouderaDialogBody>
      <ClouderaDialogFooter>
        <ClouderaButton onClick={reset}>Cancel</ClouderaButton>
        <ClouderaButton
          color="danger"
          icon={<Delete />}
          disabled={!slug}
          onClick={() => {
            handleDeleteAlias.mutate();
          }}
          testId="delete-confirmation-yes"
        >
          Delete
        </ClouderaButton>
      </ClouderaDialogFooter>
    </ClouderaDialogContent>
  );
};
