import React, { Fragment } from 'react';
import OverlayTrigger from 'react-bootstrap/OverlayTrigger';
import Popover from 'react-bootstrap/Popover';
import classNames from 'classnames';

import classes from '../../visuals-page/components/report-tile-popover/ReportTilePopover.module.scss';

type ParameterPopoverProps = {
  popOverTitle: string;
  parameterEntries: { key: string; value: string | number | JSX.Element }[];
  children: JSX.Element;
};

export const ParameterPopover = ({
  popOverTitle,
  parameterEntries,
  children,
}: ParameterPopoverProps): JSX.Element => {
  return (
    <OverlayTrigger
      trigger={['hover', 'focus']}
      placement="bottom"
      overlay={(props) => (
        <Popover id="parameter-popover" {...props}>
          <Popover.Title className={classes.title}>
            {popOverTitle}
          </Popover.Title>
          <Popover.Content className={classes.content}>
            {parameterEntries.map(({ key, value }) => (
              <Fragment key={key}>
                <div className={classes.key}>{key}: </div>
                <div className={classNames(classes.key, classes.value)}>
                  {value}
                </div>
              </Fragment>
            ))}
          </Popover.Content>
        </Popover>
      )}
      delay={{ show: 500, hide: 200 }}
    >
      {children}
    </OverlayTrigger>
  );
};
