import { useMutation, useQueryClient } from '@tanstack/react-query';
import { editAlias } from 'apis/aliases/edit-alias';
import { aliasesQueryToken } from 'apis/aliases/get-all-aliases';
import { appsQueryToken } from 'apis/get-apps';
import { reportsQueryToken } from 'apis/get-reports';

import { useNotificationStore } from 'stores/use-notification-store';
import { UrlAliasRow } from '../types/manage-aliases.type';
import { useClouderaDialogContext } from 'components/cloudera-dialog/ClouderaDialog';

export const useEditAliasMutation = () => {
  const { reset: resetDialog } = useClouderaDialogContext();
  const { addNotification } = useNotificationStore(({ addNotification }) => ({
    addNotification,
  }));
  const queryClient = useQueryClient();

  return useMutation({
    mutationFn: ({
      originAlias,
      updateType,
      newAlias,
      url_data,
    }: UrlAliasRow) =>
      editAlias({
        slug: originAlias,
        updateType: updateType,
        newAlias: newAlias,
        urlData: url_data,
      }),
    onSuccess: () => {
      queryClient.invalidateQueries([aliasesQueryToken]);
      queryClient.invalidateQueries([reportsQueryToken]);
      queryClient.invalidateQueries([appsQueryToken]);
      resetDialog();
      addNotification(
        {
          variant: 'success',
          message: 'Alias updated successfully!',
        },
        { delay: 2000 }
      );
    },
    onError: () => {
      addNotification(
        {
          variant: 'danger',
          message: 'Failed to update alias.',
        },
        { delay: 3000 }
      );
    },
  });
};
