import { useQuery } from '@tanstack/react-query';
import { appsQueryToken, getApps } from 'apis/get-apps';
import { useNotificationStore } from 'stores/use-notification-store';

export function useAppOptions() {
  const { addNotification } = useNotificationStore(({ addNotification }) => ({
    addNotification,
  }));
  const { data: appOptions } = useQuery({
    queryKey: [appsQueryToken],
    queryFn: () => getApps(),
    onError: () => {
      addNotification(
        {
          id: 'roles-danger',
          variant: 'danger',
          message: 'Error fetching apps.',
        },
        { delay: 3000 }
      );
    },
    initialData: [],
    select: (apps) => {
      const appOptions = apps
        .filter(
          ({ group_type }) =>
            group_type === 'default' || group_type === 'sample-app'
        )
        .map(({ group_id, name, subgroups }) => ({
          id: group_id,
          name,
          subgroups: subgroups?.map(({ group_id, name, apps }) => ({
            id: group_id,
            name,
            apps: apps?.flat(),
          })),
        }));

      return appOptions;
    },
  });

  return { appOptions };
}
