import { useQuery } from '@tanstack/react-query';
import { getReports, reportsQueryToken } from 'apis/get-reports';
import { useNotificationStore } from 'stores/use-notification-store';

function transformDashboardData(data: string): { id: number; name: string }[] {
  const parsedData: {
    dashboard_sheets: { sheet_id: number; sheet_handle_title: string }[];
  } = JSON.parse(data);
  return parsedData.dashboard_sheets.map(
    (sheet: { sheet_id: number; sheet_handle_title: string }) => ({
      id: sheet.sheet_id,
      name: sheet.sheet_handle_title,
    })
  );
}

export function useDashboardOptions() {
  const { addNotification } = useNotificationStore(({ addNotification }) => ({
    addNotification,
  }));
  const { data: dashboardOptions } = useQuery({
    queryKey: [reportsQueryToken],
    queryFn: () => getReports({ reportType: 'dashboard', isDetailed: true }),
    onError: () => {
      addNotification(
        {
          id: 'roles-danger',
          variant: 'danger',
          message: 'Error fetching dashboards.',
        },
        { delay: 3000 }
      );
    },
    initialData: [],
    select: (dashboards) => {
      return dashboards.map(({ id, name, data }) => ({
        id,
        name,
        subgroups: transformDashboardData(data ?? '{}'),
      }));
    },
  });

  return { dashboardOptions };
}
