import * as yup from 'yup';

export const subGroupSchema = yup.object({
  id: yup.number().required(),
  name: yup.string().required(),
  apps: yup.array().of(yup.number()).optional(),
});

export const mainSelectSchema = yup.object({
  id: yup.number().required(),
  name: yup.string().required(),
  subgroups: yup.array().of(subGroupSchema).optional().nullable(),
});

export const setUrlValidationSchema = yup.object({
  type: yup
    .mixed<'dashboard' | 'apps'>()
    .oneOf(['dashboard', 'apps'])
    .required(),
  newAlias: yup
    .string()
    .matches(
      /^[a-zA-Z0-9_-]*$/,
      'Alias can contain A-Z, a-z, 0-9, _ and - only!'
    )
    .required('Please fill the url alias field'),
  mainSelect: mainSelectSchema
    .required('Main selection cannot be empty')
    .nullable(),
  subSelect: subGroupSchema
    .required('Sub selection cannot be empty')
    .nullable(),
  keepParameters: yup.boolean().required(),
  parameters: yup.object().required(),
});
