import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();
export const rolesQueryToken = 'roles';

export type Privs = {
  ptype: string;
  perms: string[];
  dclist?: string[];
  dslist?: string[];
  dcid?: string;
  rolelist?: string[];
};

export type Role = {
  id: number;
  name: string;
  desc: string;
  privs: Privs[];
  users: string[];
  groups: string[];
  sentry_groups: string[];
  perms: string[];
  require_all_groups: boolean;
};

export type PaginatedRoles = {
  roles: Role[];
  total: number;
};

type PaginatedProps = {
  page: number;
  pageSize: number;
  sort?: string;
  order?: 'asc' | 'desc';
  searchTerm?: string;
};

export function getAllRoles(): Promise<Role[]>;
export function getAllRoles(props: PaginatedProps): Promise<PaginatedRoles>;
export async function getAllRoles(
  props?: PaginatedProps
): Promise<PaginatedRoles | Role[]> {
  const params: Record<string, string> = {};

  const mapProps = {
    page: 'page',
    pageSize: 'page_size',
    sort: 'sort',
    order: 'order',
    searchTerm: 'search',
  };

  Object.entries(mapProps).forEach(([key, value]) => {
    if (props && props[key as keyof PaginatedProps] !== undefined) {
      params[value] = props[key as keyof PaginatedProps]!.toString();
    }
  });

  const { data } = await vizHttp.get<PaginatedRoles | Role[]>({
    url: arc.frontend.globals.urls.rbacRolesUrl,
    isOldEndpoint: true,
    params,
  });

  // Check if the data is an array (non-paginated response)
  if (Array.isArray(data)) {
    return data;
  }

  // Otherwise, return it as PaginatedRoles
  return data;
}
