import React, { useState } from 'react';
import { Button, Form, Modal } from 'react-bootstrap';
import Select from 'react-select';

type AddConnectionPrivilegeModalProps = {
  isOpen: boolean;
  close: () => void;
  onSave: (dataconnections: { id: number; name: string }[]) => void;
  dataconnectionOptions: { id: number; name: string }[];
  areDataconnectionsLoading: boolean;
};

export const AddConnetionPrivilegeModal = ({
  isOpen,
  dataconnectionOptions,
  close,
  onSave,
  areDataconnectionsLoading,
}: AddConnectionPrivilegeModalProps): JSX.Element => {
  const [selectedDataconnections, setSelectDataconnections] = useState<
    {
      id: number;
      name: string;
    }[]
  >([]);

  return (
    <Modal
      show={isOpen}
      onHide={close}
      data-test-id="add-connections-privilege-modal"
    >
      <Modal.Header closeButton>
        <Modal.Title>Add Connection Privilege</Modal.Title>
      </Modal.Header>

      <Form>
        <Modal.Body>
          <div className="d-flex gap-5">
            <Form.Group className="w-100">
              <Form.Label>Connections</Form.Label>
              <div data-test-id="dataconnections" className="d-flex gap-2">
                <Select
                  className="w-100"
                  inputId="dataconnections"
                  classNamePrefix="form-select"
                  isMulti
                  isSearchable
                  isLoading={areDataconnectionsLoading}
                  options={dataconnectionOptions}
                  getOptionLabel={({ name, id }) => `${name} (${id})`}
                  getOptionValue={({ name }) => `${name}`}
                  placeholder="Please select a Connection"
                  value={selectedDataconnections}
                  onChange={(newValue) =>
                    setSelectDataconnections([...newValue])
                  }
                  required
                />
              </div>
            </Form.Group>
          </div>
        </Modal.Body>
      </Form>

      <Modal.Footer>
        <Button variant="secondary" onClick={close}>
          Cancel
        </Button>

        <Button
          variant="primary"
          onClick={() => {
            onSave(selectedDataconnections);
            setSelectDataconnections([]);
            close();
          }}
        >
          Add
        </Button>
      </Modal.Footer>
    </Modal>
  );
};
