import React, { useMemo, useState } from 'react';
import { Button, Form, Modal } from 'react-bootstrap';
import Select from 'react-select';

type AddDatasetPrivilegeModalProps = {
  isOpen: boolean;
  close: () => void;
  onSave: (
    datasets: {
      id: number;
      name: string;
      dc_id: number;
      dc_name: string;
    }[],
    dataConnection: { id: number; name: string }
  ) => void;
  dataconnectionOptions: { id: number; name: string }[];
  datasetOptions: {
    id: number;
    name: string;
    dc_id: number;
    dc_name: string;
  }[];
  areDatasetsLoading: boolean;
  areDataconnectionsLoading: boolean;
};

export const AddDatasetPrivilegeModal = ({
  isOpen,
  close,
  dataconnectionOptions,
  datasetOptions,
  onSave,
  areDatasetsLoading,
  areDataconnectionsLoading,
}: AddDatasetPrivilegeModalProps): JSX.Element => {
  const [selectedDataconnection, setSelectDataconnection] = useState<{
    id: number;
    name: string;
  } | null>(null);

  const datasetOptionsFilteredByDataconnection = useMemo(
    () =>
      datasetOptions.filter(({ dc_id, id }) => {
        const isAllDataset = id === -1;

        return dc_id === selectedDataconnection?.id || isAllDataset;
      }),
    [datasetOptions, selectedDataconnection?.id]
  );

  const [selectedDatasets, setSelectedDatasets] = useState<
    typeof datasetOptions
  >([]);

  return (
    <Modal
      show={isOpen}
      onHide={close}
      data-test-id="add-datasets-privilege-modal"
    >
      <Modal.Header closeButton>
        <Modal.Title>Add Dataset Privilege</Modal.Title>
      </Modal.Header>

      <Form>
        <Modal.Body>
          <div className="d-flex gap-5">
            <Form.Group className="w-100">
              <Form.Label>Connections</Form.Label>
              <div
                data-test-id="dataconnection-for-dataset"
                className="d-flex gap-2"
              >
                <Select
                  className="w-100"
                  inputId="dataconnectionForDataset"
                  classNamePrefix="form-select"
                  isSearchable
                  isLoading={areDataconnectionsLoading}
                  options={dataconnectionOptions}
                  getOptionLabel={({ name, id }) => `${name} (${id})`}
                  getOptionValue={({ name }) => `${name}`}
                  placeholder="Please select a Connection"
                  value={selectedDataconnection}
                  onChange={(newDataconnection) => {
                    setSelectDataconnection(newDataconnection);
                    setSelectedDatasets([]);
                  }}
                  required
                />
              </div>
            </Form.Group>
          </div>

          <div className="d-flex gap-5">
            <Form.Group className="w-100">
              <Form.Label>Datasets</Form.Label>
              <div
                data-test-id="filtered-datasets-by-dataconn"
                className="d-flex gap-2"
              >
                <Select
                  className="w-100"
                  inputId="filteredDatasetsByDataconn"
                  classNamePrefix="form-select"
                  isMulti
                  isSearchable
                  isLoading={areDatasetsLoading}
                  options={datasetOptionsFilteredByDataconnection}
                  getOptionLabel={({ name, id }) => `${name} (${id})`}
                  getOptionValue={({ name }) => `${name}`}
                  placeholder="Please select a Dataset"
                  value={selectedDatasets}
                  onChange={(newValue) => setSelectedDatasets([...newValue])}
                  required
                />
              </div>
            </Form.Group>
          </div>
        </Modal.Body>
      </Form>

      <Modal.Footer>
        <Button variant="secondary" onClick={close}>
          Cancel
        </Button>
        <Button
          variant="primary"
          onClick={() => {
            if (selectedDataconnection === null) return;

            onSave(selectedDatasets, selectedDataconnection);

            close();

            setSelectDataconnection(null);
            setSelectedDatasets([]);
          }}
        >
          Add
        </Button>
      </Modal.Footer>
    </Modal>
  );
};
