import React, { useState } from 'react';
import { Button, Form, Modal } from 'react-bootstrap';
import Select from 'react-select';

type AddRolesPrivilegeModalProps = {
  isOpen: boolean;
  close: () => void;
  onSave: (roles: { id: number; name: string }[]) => void;
  roleOptions: { id: number; name: string }[];
  areRolesLoading: boolean;
};

export const AddRolePrivilegeModal = ({
  isOpen,
  close,
  roleOptions,
  onSave,
  areRolesLoading,
}: AddRolesPrivilegeModalProps) => {
  const [selectedRole, setselectedRole] = useState<
    { id: number; name: string }[]
  >([]);

  const rolesOptions = roleOptions.map(({ id, name }) => ({
    id,
    name,
  }));

  return (
    <Modal
      show={isOpen}
      onHide={close}
      data-test-id="add-roles-privilege-modal"
    >
      <Modal.Header closeButton>
        <Modal.Title>Add Role Privilege </Modal.Title>
      </Modal.Header>
      <Form>
        <Modal.Body>
          <div className="d-flex gap-5">
            <Form.Group className="w-100">
              <Form.Label>Roles</Form.Label>
              <div data-test-id="roles" className="d-flex gap-2">
                <Select
                  className="w-100"
                  inputId="roles"
                  classNamePrefix="form-select"
                  isMulti
                  isSearchable
                  isLoading={areRolesLoading}
                  options={[...rolesOptions]}
                  getOptionLabel={({ name, id }) => `${name} (${id})`}
                  getOptionValue={({ name }) => `${name}`}
                  placeholder="Please select a Role"
                  value={selectedRole}
                  onChange={(newValue) => setselectedRole([...newValue])}
                  required
                />
              </div>
            </Form.Group>
          </div>
        </Modal.Body>
      </Form>
      <Modal.Footer>
        <Button variant="secondary" onClick={close}>
          Cancel
        </Button>

        <Button
          variant="primary"
          onClick={() => {
            onSave(selectedRole);
            setselectedRole([]);
            close();
          }}
        >
          Add
        </Button>
      </Modal.Footer>
    </Modal>
  );
};
