import React, { useMemo } from 'react';
import { Col, Row } from 'react-bootstrap';
import classes from '../RoleDetails.module.scss';
import Groups from '@cloudera-internal/cuix-core/icons/svg/ic-groups.svg';
import {
  createColumnHelper,
  getCoreRowModel,
  getPaginationRowModel,
  getSortedRowModel,
} from '@tanstack/react-table';
import { ClouderaTable } from 'components/cloudera-table';
import { useTable } from 'hooks/use-table';

type SentryGroup = { name: string };

type SentryGroupsProps = {
  sentryGroups: SentryGroup[];
};

const columnHelper = createColumnHelper<SentryGroup>();

export const SentryGroupsTable = ({
  sentryGroups,
}: SentryGroupsProps): React.ReactElement => {
  const columns = useMemo(
    () => [
      columnHelper.accessor('name', {
        header: () => 'Group name',
        cell: (info) => info.getValue(),
      }),
    ],
    []
  );

  const table = useTable({
    data: sentryGroups,
    columns,
    getCoreRowModel: getCoreRowModel(),
    getSortedRowModel: getSortedRowModel(),
    getPaginationRowModel: getPaginationRowModel(),
    initialState: {
      pagination: {
        pageIndex: 0,
        pageSize: 10,
      },
    },
  });

  return (
    <Col sm md={4} className={classes.groupsColumn}>
      <div style={{ minHeight: '145px' }}>
        <Row className="d-flex justify-content-between align-items-center pt-2 pb-0 pl-3 pr-3">
          <div className="d-flex align-items-center">
            <Groups className={classes.groupIcon} />
            <h4> Sentry Groups</h4>
          </div>
        </Row>
        <Row>
          <div className={classes.descriptionText}>
            Membership for roles imported from Sentry can not be changed here
            and must be managed through the Sentry system
          </div>
        </Row>
      </div>
      <Row className="d-flex justify-content-between align-items-center pt-2 pb-2 pl-3 pr-3">
        <ClouderaTable
          isLoading={!sentryGroups}
          table={table}
          data-test-id="roleSentryGroupsTable"
        />
      </Row>
    </Col>
  );
};
