export const PERMISSION_LABEL = {
  sys_editperm: 'Manage roles and users',
  sys_settings: 'Manage settings',
  sys_styles: 'Manage custom styles',
  sys_viewlogs: 'View activity logs',
  sys_schedules: 'Manage jobs, email templates',
  sys_editconn: 'Manage data connections',
  sys_workspace: 'Create workspaces',
  sys_viewroles: 'View roles and users',
  sys_misc: 'Additional system privilege',

  rgds_manage: 'Grant manage dataset',
  rgds_appedit: 'Grant manage dashboards',
  rgds_appview: 'Grant view dashboards',

  dc_aviews: 'Manage AVs/Extracts',
  dc_upload: 'Import data',
  dc_explore: 'Create datasets, explore tables',

  ds_manage: 'Manage dataset',
  ds_appedit: 'Manage dashboards',
  ds_appview: 'View dashboards',

  ws_manage: 'Manage workspace',
  ws_appedit: 'Manage workspace dashboards',
  ws_appview: 'View workspace dashboards',
};

export const PERMISSION_GROUPS = {
  system: {
    tab: 'System',
    privileges: [
      {
        id: 'sys_workspace' as const,
        label: PERMISSION_LABEL['sys_workspace'],
        description:
          'Allows users to create new workspaces that may be shared among users and user groups.',
        type: 'checkbox',
      },
      {
        id: 'sys_viewroles' as const,
        label: PERMISSION_LABEL['sys_viewroles'],
        description: 'Enables users to view users, user groups, and roles.',
        type: 'checkbox',
      },
      {
        id: 'sys_editperm' as const,
        label: PERMISSION_LABEL['sys_editperm'],
        description: 'Enables users to create users, user groups, and roles.',
        type: 'checkbox',
      },
      {
        id: 'sys_settings' as const,
        label: PERMISSION_LABEL['sys_settings'],
        description: 'Permits users to mange global site settings.',
        type: 'checkbox',
      },
      {
        id: 'sys_styles' as const,
        label: PERMISSION_LABEL['sys_styles'],
        description:
          'Authorizes users to create new styles for dashboards and visuals.',
        type: 'checkbox',
      },
      {
        id: 'sys_schedules' as const,
        label: PERMISSION_LABEL['sys_schedules'],
        description:
          'Enables users to manage scheduled jobs and create templates for email messages.',
        type: 'checkbox',
      },
      {
        id: 'sys_viewlogs' as const,
        label: PERMISSION_LABEL['sys_viewlogs'],
        description:
          'Allows users to view usage statistics and monitor the performance of Cloudera Data Visualization.',
        type: 'checkbox',
      },
      {
        id: 'sys_editconn' as const,
        label: PERMISSION_LABEL['sys_editconn'],
        description:
          'Grants users the ability to create and manage connections to various data sources.',
        type: 'checkbox',
      },
      {
        id: 'sys_misc' as const,
        label: PERMISSION_LABEL['sys_misc'],
        description:
          "Users can set a default homepage, manage dashboards in other users' private workspaces, perform administrative restart/stop operations, and request tickets from the Cloudera Data Visualization Server using Trusted Auth Get Ticket.",
        type: 'checkbox',
      },
    ],
  },
  roles: {
    tab: 'Roles',
    privileges: [
      {
        id: 'rgds_manage' as const,
        label: PERMISSION_LABEL['rgds_manage'],
        description:
          'Enables users to grant Manage dataset privileges to specific roles, provided the user has the Manage dataset permission for that dataset.',
        type: 'checkbox',
      },
      {
        id: 'rgds_appedit' as const,
        label: PERMISSION_LABEL['rgds_appedit'],
        description:
          'Enables users to grant Manage dashboard privileges to specific roles, provided the user has the Manage dataset permission for that dataset.',
        type: 'checkbox',
      },
      {
        id: 'rgds_appview' as const,
        label: PERMISSION_LABEL['rgds_appview'],
        description:
          'Enables users to grant View dashboard privileges to specific roles, provided the user has the Manage dataset permission for that dataset.',
        type: 'checkbox',
      },
    ],
  },
  connections: {
    tab: 'Connections',
    privileges: [
      {
        id: 'dc_aviews' as const,
        label: PERMISSION_LABEL['dc_aviews'],
        description:
          'Enables users to create and manage analytical views and data extracts.',
        type: 'checkbox',
      },
      {
        id: 'dc_upload' as const,
        label: PERMISSION_LABEL['dc_upload'],
        description:
          'Allows users to import supplemental data into an existing connection.',
        type: 'checkbox',
      },
      {
        id: 'dc_explore' as const,
        label: PERMISSION_LABEL['dc_explore'],
        description:
          'Allows users to create new datasets from existing tables, view sample data, and explore statistical reports on the data tables.',
        type: 'checkbox',
      },
    ],
  },
  datasets: {
    tab: 'Datasets',
    privileges: [
      {
        id: 'ds_manage' as const,
        label: PERMISSION_LABEL['ds_manage'],
        description:
          'Allows users to change the properties of datasets, create datasets over joined tables, modify the fields of the dataset, and more.',
        type: 'checkbox',
      },
      {
        id: 'ds_appedit' as const,
        label: PERMISSION_LABEL['ds_appedit'],
        description:
          'Enables users to create and modify visuals and dashboards.',
        type: 'checkbox',
      },
      {
        id: 'ds_appview' as const,
        label: PERMISSION_LABEL['ds_appview'],
        description:
          'Used to limit users to view-only privileges for visuals and dashboards, while denying edit privileges.',
        type: 'checkbox',
      },
    ],
  },
} as const;

export const PERMISSION_DEPENDENCIES = {
  sys_editperm: ['sys_viewroles'],

  ds_manage: ['ds_appedit', 'ds_appview'],
  ds_appedit: ['ds_appview'],

  dc_upload: ['dc_explore'],
  dc_aviews: ['dc_explore'],

  rgds_manage: ['rgds_appedit', 'rgds_appview'],
  rgds_appedit: ['rgds_appview'],
};
