import { PERMISSION_GROUPS } from '../constants/permission-constants';

type DatasetPerm = {
  privileges: Record<
    (typeof PERMISSION_GROUPS.datasets.privileges)[number]['id'],
    boolean
  >;
  dataset_ids: string[];
  connection_id: string;
};

// Utility function to parse permissions
export function systemParsePermission(systemPermissions: {
  perms: string[];
  ptype: string;
}) {
  return PERMISSION_GROUPS.system.privileges
    .map((privilege) => privilege.id)
    .reduce((obj, id) => {
      obj[id] = systemPermissions.perms.includes(id);
      return obj;
    }, {} as Record<(typeof PERMISSION_GROUPS.system.privileges)[number]['id'], boolean>);
}

export function connectionsParsePermissions(
  connectionsPermissions: {
    ptype: string;
    perms: string[];
    dclist: string[];
  }[]
) {
  return connectionsPermissions.map((connection) => {
    const connectionIds = connection.dclist;

    const connectionValues = {
      privileges: PERMISSION_GROUPS.connections.privileges
        .map((privilege) => privilege.id)
        .reduce((obj, id) => {
          obj[id] = connection.perms.includes(id);
          return obj;
        }, {} as Record<(typeof PERMISSION_GROUPS.connections.privileges)[number]['id'], boolean>),
      connection_ids: connectionIds,
    };

    return connectionValues;
  });
}

export function datasetsParsePermissions(
  datasetPermissions: {
    ptype: string;
    perms: string[];
    dslist: string[];
    dcid: string;
  }[]
) {
  return datasetPermissions
    .map((dataset): DatasetPerm | null => {
      const datasetValues = {
        privileges: PERMISSION_GROUPS.datasets.privileges
          .map((privilege) => privilege.id)
          .reduce((obj, id) => {
            obj[id] = dataset.perms.includes(id);
            return obj;
          }, {} as Record<(typeof PERMISSION_GROUPS.datasets.privileges)[number]['id'], boolean>),
        dataset_ids: dataset.dslist,
        connection_id: dataset.dcid,
      };

      return datasetValues;
    })
    .filter((dataset): dataset is DatasetPerm => Boolean(dataset));
}

export function rolesParsePermissions(
  rolePermissions: {
    perms: string[];
    ptype: string;
    rolelist: string[];
  }[]
) {
  return rolePermissions.map((role) => ({
    role_ids: role.rolelist,
    privileges: PERMISSION_GROUPS.roles.privileges
      .map((privilege) => privilege.id)
      .reduce((obj, id) => {
        obj[id] = true;
        return obj;
      }, {} as Record<(typeof PERMISSION_GROUPS.roles.privileges)[number]['id'], boolean>),
  }));
}
