import type { RoleDetailsFormData } from '../RoleDetails';
import type { Role } from '../types/role-details.type';

export function transformFormData(
  preFormattedData: RoleDetailsFormData,
  id?: number
) {
  let privs: Role['privs'] = [];

  // Iterate over system properties
  const systemPrivs: string[] = [];
  for (const [id, value] of Object.entries(preFormattedData.system)) {
    if (value) {
      systemPrivs.push(id);
    }
  }
  if (systemPrivs.length > 0) {
    privs.push({
      ptype: 'system',
      perms: systemPrivs,
    });
  }

  // Iterate over roles
  for (const role of preFormattedData.roles) {
    let rolePrivs: string[] = [];
    for (const [id, value] of Object.entries(role.privileges)) {
      if (value) {
        rolePrivs.push(id);
      }
    }
    if (rolePrivs.length > 0) {
      privs.push({
        ptype: 'role',
        perms: rolePrivs,
        rolelist: role.role_ids,
      });
    }
  }

  // Iterate over connections
  for (const connection of preFormattedData.connections) {
    let connPrivs: string[] = [];
    for (const [id, value] of Object.entries(connection.privileges)) {
      if (value) {
        connPrivs.push(id);
      }
    }
    if (connPrivs.length > 0) {
      privs.push({
        ptype: 'dataconn',
        perms: connPrivs,
        dclist: connection.connection_ids,
      });
    }
  }

  // Iterate over datasets
  for (const dataset of preFormattedData.datasets) {
    let datasetPrivs: string[] = [];

    if (dataset.connection_id && dataset.connection_id.length > 0) {
      for (const [id, value] of Object.entries(dataset.privileges)) {
        if (value) {
          datasetPrivs.push(id);
        }
      }
      if (datasetPrivs.length > 0) {
        privs.push({
          ptype: 'dataset',
          perms: datasetPrivs,
          dcid: dataset.connection_id,
          dslist: dataset.dataset_ids,
        });
      }
    }
  }

  return [
    {
      name: preFormattedData.name,
      desc: preFormattedData.description,
      privs: privs,
      users: preFormattedData.users,
      groups: preFormattedData.groups,
      sentry_groups: [],
      id,
      require_all_groups: preFormattedData.require_all_groups,
    },
  ];
}
