import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();
const siteSettingsQueryToken = 'siteSettings';

export type SiteSetting = {
  setting_key: string;
  setting_value: string;
};

const getSiteSettings = async () => {
  try {
    const { data } = await vizHttp.get<SiteSetting[]>({
      url: arc.frontend.globals.urls.siteSettingsUrl,
      isOldEndpoint: false,
    });

    const settingsMap: { [key: string]: unknown } = {};

    for (const setting of data) {
      let value: string | boolean = setting.setting_value;

      if (['true', 'True'].includes(setting.setting_value)) {
        value = true;
      } else if (['false', 'False'].includes(setting.setting_value)) {
        value = false;
      }

      settingsMap[setting.setting_key] = value;
    }

    return settingsMap;
  } catch (error) {
    console.error(error);

    return [];
  }
};

export { getSiteSettings, siteSettingsQueryToken };
