import React from 'react';
import { Form, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import classes from '../SiteSettingsPage.module.scss';
import {
  ConfirmObj,
  SettingValue,
  SiteSetting,
  State,
} from '../types/site-settings.type';

type SettingCheckboxProps = {
  setting: SiteSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue, confirmObj?: ConfirmObj) => void;
};

const SettingCheckbox = ({
  setting,
  formState,
  onChange,
}: SettingCheckboxProps) => {
  const {
    id,
    label,
    popoverText,
    tooltipPlacement = 'top',
    disabled,
    confirm,
    warningText,
  } = setting;
  const isDisabled = disabled?.(formState);
  const confirmObj = confirm?.(formState);

  return (
    <>
      <div className={classNames('d-flex align-items-center', classes.gap5)}>
        <Form.Check
          data-test-id={`test-${id}`}
          id={id}
          type="checkbox"
          checked={Boolean(formState[id])}
          disabled={isDisabled}
          label={label}
          onChange={(e) => {
            onChange(id, e.target.checked, confirmObj);
          }}
        />
        {popoverText && (
          <VizTooltip
            placement={tooltipPlacement}
            overlay={
              <Popover id={id}>
                {<span className="d-flex p-2">{popoverText}</span>}
              </Popover>
            }
          >
            <span className="info-icon" />
          </VizTooltip>
        )}
      </div>
      {warningText?.visible(formState) && (
        <span className={`${classes.warningMsg} mt-2`}>{warningText.msg}</span>
      )}
    </>
  );
};

export { SettingCheckbox };
