import React from 'react';
import Select from 'react-select';
import classNames from 'classnames';
import classes from '../SiteSettingsPage.module.scss';
import { Form } from 'react-bootstrap';
import { SettingValue, SiteSetting, State } from '../types/site-settings.type';

type SettingDropdownProps = {
  setting: SiteSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue) => void;
};

const SettingDropdown = ({
  setting,
  formState,
  onChange,
}: SettingDropdownProps) => {
  const { id, label, options } = setting;

  return (
    <div className="d-flex flex-column">
      <Form.Label className="mb-1 font-weight-bold">{label}</Form.Label>
      <div className="d-flex">
        <Select
          name="engineSelect"
          inputId="engineSelect"
          classNamePrefix="form-select"
          className={classNames(classes.select, 'select-container')}
          options={options}
          getOptionLabel={({ label }) => label}
          getOptionValue={({ id }) => id}
          value={options?.find((option) => option.id === formState[id])}
          onChange={(selectedOption) => {
            onChange(id, selectedOption!.id);
          }}
        />
      </div>
    </div>
  );
};

export { SettingDropdown };
