import React from 'react';
import { Form, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import classes from '../SiteSettingsPage.module.scss';
import { SettingValue, SiteSetting, State } from '../types/site-settings.type';

type SettingInputProps = {
  setting: SiteSetting;
  formState: State['form'];
  shouldValidate: boolean;
  additionalElement: JSX.Element | null;
  onChange: (id: string, value: SettingValue) => void;
};

const SettingInput = ({
  setting,
  formState,
  shouldValidate,
  onChange,
  additionalElement,
}: SettingInputProps) => {
  const {
    id,
    label,
    popoverText,
    placeholder,
    warningText,
    required,
    disabled,
  } = setting;
  const isDisabled = disabled?.(formState);

  return (
    <div className="d-flex flex-column">
      <div
        className={classNames('d-flex align-items-center mb-2', classes.gap5)}
      >
        <Form.Label htmlFor={id} className="mb-0 pr-0 font-weight-bold">
          {label}
        </Form.Label>
        {popoverText && (
          <VizTooltip
            placement="top"
            overlay={
              <Popover id={id}>
                {<span className="d-flex p-2">{popoverText}</span>}
              </Popover>
            }
          >
            <span className="info-icon" />
          </VizTooltip>
        )}
      </div>
      <Form.Control
        data-test-id={`test-${id}`}
        id={id}
        type="text"
        disabled={isDisabled}
        value={formState[id] as string}
        placeholder={placeholder}
        onChange={(e) => onChange(id, e.target.value)}
        isInvalid={required && shouldValidate && !formState[id]}
      />
      <Form.Control.Feedback type="invalid">
        This field is required.
      </Form.Control.Feedback>

      {warningText?.visible(formState) && (
        <span className={classes.warningMsg}>{warningText.msg}</span>
      )}
      {additionalElement}
    </div>
  );
};

export { SettingInput };
