import React from 'react';
import { Form, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import classes from '../SiteSettingsPage.module.scss';
import { SettingValue, SiteSetting, State } from '../types/site-settings.type';

type SettingNumericProps = {
  setting: SiteSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue) => void;
};

const SettingNumeric = ({
  setting,
  formState,
  onChange,
}: SettingNumericProps) => {
  const { id, label, popoverText, min, step } = setting;

  return (
    <div className="d-flex flex-column">
      <div className={classNames('d-flex align-items-center', classes.gap15)}>
        <div className={classNames('d-flex align-items-center', classes.gap5)}>
          <Form.Label htmlFor={id} className="mb-0 font-weight-bold">
            {label}
          </Form.Label>
          {popoverText && (
            <VizTooltip
              placement="top"
              overlay={
                <Popover id={id}>
                  {<span className="d-flex p-2">{popoverText}</span>}
                </Popover>
              }
            >
              <span className="info-icon" />
            </VizTooltip>
          )}
        </div>
        <Form.Control
          data-test-id={`test-${id}`}
          id={id}
          type="number"
          min={min ?? 1}
          step={step ?? 1}
          value={formState[id] as string}
          onChange={(e) => onChange(id, parseFloat(e.target.value).toString())}
        />
      </div>
    </div>
  );
};

export { SettingNumeric };
