import React from 'react';
import { Form, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import classes from '../SiteSettingsPage.module.scss';
import { SettingValue, SiteSetting, State } from '../types/site-settings.type';

type SettingPasswordProps = {
  setting: SiteSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue) => void;
};

const SettingPassword = ({
  setting,
  formState,
  onChange,
}: SettingPasswordProps) => {
  const { id, label, popoverText, warningText, disabled } = setting;
  const isDisabled = disabled?.(formState);

  return (
    <div className="d-flex flex-column">
      <div
        className={classNames('d-flex align-items-center mb-2', classes.gap5)}
      >
        <Form.Label htmlFor={id} className="mb-0 font-weight-bold">
          {label}
        </Form.Label>
        {popoverText && (
          <VizTooltip
            placement="top"
            overlay={
              <Popover id={id}>
                {<span className="d-flex p-2">{popoverText}</span>}
              </Popover>
            }
          >
            <span className="info-icon" />
          </VizTooltip>
        )}
      </div>
      <Form.Control
        type="password"
        data-test-id={`test-${id}`}
        autoComplete="off"
        disabled={isDisabled}
        id={id}
        value={formState[id] as string}
        onChange={(e) => onChange(id, e.target.value)}
      />
      {warningText?.visible(formState) && (
        <span className={classes.warningMsg}>{warningText.msg}</span>
      )}
    </div>
  );
};

export { SettingPassword };
