import React from 'react';
import { Form, Popover } from 'react-bootstrap';
import classNames from 'classnames';
import { VizTooltip } from 'components/VizTooltip';
import classes from '../SiteSettingsPage.module.scss';
import { SettingValue, SiteSetting, State } from '../types/site-settings.type';

type SettingRangeProps = {
  setting: SiteSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue) => void;
};

const SettingRange = ({ setting, formState, onChange }: SettingRangeProps) => {
  const {
    id,
    label,
    popoverText,
    tooltipPlacement = 'top',
    min = 0,
    max = 100,
    step = 1,
    disabled,
  } = setting;
  const isDisabled = disabled?.(formState);

  return (
    <div className={classNames('d-flex flex-column', classes.gap5)}>
      <Form.Group className="mb-0">
        <div
          className={classNames('d-flex align-items-center mb-2', classes.gap5)}
        >
          <Form.Label className="mb-0">{label}</Form.Label>
          {popoverText && (
            <VizTooltip
              placement={tooltipPlacement}
              overlay={
                <Popover id={id}>
                  {<span className="d-flex p-2">{popoverText}</span>}
                </Popover>
              }
            >
              <span className="info-icon" />
            </VizTooltip>
          )}
        </div>
        <Form.Control
          type="range"
          min={min}
          max={max}
          step={step}
          disabled={isDisabled}
          data-test-id={`test-${id}`}
          id={id}
          value={formState[id] as string}
          onChange={(e) => onChange(id, e.target.value)}
        />
      </Form.Group>
      <div className="d-flex justify-content-between mx-1">
        <span>{min}</span>
        <span>{min + max / 2}</span>
        <span>{max}</span>
      </div>
    </div>
  );
};

export { SettingRange };
