import classNames from 'classnames';
import React from 'react';
import { Form, Popover } from 'react-bootstrap';
import classes from '../SiteSettingsPage.module.scss';
import { VizTooltip } from 'components/VizTooltip';
import { SettingValue, SiteSetting, State } from '../types/site-settings.type';

type SettingTextareaProps = {
  setting: SiteSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue) => void;
};

const SettingTextarea = ({
  setting,
  formState,
  onChange,
}: SettingTextareaProps) => {
  const { id, label, popoverText, placeholder, disabled } = setting;
  const isDisabled = disabled?.(formState);

  return (
    <div className={classNames('d-flex flex-column', classes.textareaWidth)}>
      <div
        className={classNames('d-flex align-items-center mb-2', classes.gap5)}
      >
        <Form.Label htmlFor={id} className="mb-0 pr-0 font-weight-bold">
          {label}
        </Form.Label>
        {popoverText && (
          <VizTooltip
            placement="top"
            overlay={
              <Popover id={id}>
                {<span className="d-flex p-2">{popoverText}</span>}
              </Popover>
            }
          >
            <span data-test-id="info-icon-span" className="info-icon" />
          </VizTooltip>
        )}
      </div>
      <Form.Control
        as="textarea"
        rows={6}
        disabled={isDisabled}
        data-test-id={`test-${id}`}
        id={id}
        value={formState[id] as string}
        placeholder={placeholder}
        onChange={(e) => onChange(id, e.target.value)}
      />
    </div>
  );
};

export { SettingTextarea };
