import React, { ReactElement } from 'react';
import classes from '../SiteSettingsPage.module.scss';
import classNames from 'classnames';
import { SideNavItem } from './SideNavItem';
import { SettingTab } from '../types/site-settings.type';
import { DEFAULT_SETTING_GROUPS } from '../constants/setting-constants';

type SideNavProps = {
  items: typeof DEFAULT_SETTING_GROUPS;
  activeTab: string;
  onClick: (tab: string, settings: SettingTab['settings'] | null) => void;
};

const SideNav = ({ items, activeTab, onClick }: SideNavProps) => {
  const renderNavItems = (navItems): ReactElement[] => {
    return navItems.map((group) => {
      if (!group.children) {
        return (
          <div
            className={classNames(
              { [classes.active]: activeTab === group.tab },
              classes.navTab
            )}
            key={group.tab}
            onClick={(e) => {
              e.stopPropagation();
              onClick(group.tab, group.settings);
            }}
          >
            <span className={classNames('d-block', classes.navLabel)}>
              {group.tab}
            </span>
          </div>
        );
      }

      return (
        <SideNavItem
          key={group.tab}
          label={group.tab}
          activeTab={activeTab}
          items={group.children}
          renderItems={renderNavItems}
        />
      );
    });
  };

  return <div className={classes.sideNav}>{renderNavItems(items)}</div>;
};

export { SideNav };
