import React, { ReactElement, useEffect, useState } from 'react';
import classes from '../SiteSettingsPage.module.scss';
import classNames from 'classnames';
import { DEFAULT_SETTING_GROUPS } from '../constants/setting-constants';

type SideNavProps = {
  label: string;
  items: typeof DEFAULT_SETTING_GROUPS;
  activeTab: string;
  renderItems: (items: typeof DEFAULT_SETTING_GROUPS) => ReactElement[];
};

const SideNavItem = ({
  label,
  items,
  activeTab,
  renderItems,
}: SideNavProps) => {
  const [menuCollapsed, setMenuCollapsed] = useState(false);

  useEffect(() => {
    if (items.some((item) => item.tab === activeTab)) {
      setMenuCollapsed(false);
    }
  }, [items, activeTab]);

  return (
    <div key={label} onClick={() => setMenuCollapsed(!menuCollapsed)}>
      <div
        className={classNames(
          'd-flex align-items-center position-relative',
          classes.navTab
        )}
      >
        <i
          className={classNames(
            `fa fa-angle-${menuCollapsed ? 'right' : 'down'} fa-lg`,
            classes.navAngle
          )}
        />
        <span className={classNames('d-block', classes.navLabel)}>{label}</span>
      </div>
      <div
        className={classNames(
          { [classes.hide]: menuCollapsed },
          classes.nestedContainer
        )}
      >
        {renderItems(items)}
      </div>
    </div>
  );
};

export { SideNavItem };
