import React from 'react';
import {
  ConfirmObj,
  SearchedSetting,
  SettingValue,
  State,
} from '../types/site-settings.type';
import { Settings } from './Settings';
import classes from '../SiteSettingsPage.module.scss';
import classNames from 'classnames';

type SiteSettingSearchResultsProps = {
  searchedSettings: SearchedSetting;
  formState: State['form'];
  onChange: (id: string, value: SettingValue, confirmObj?: ConfirmObj) => void;
  changeToViewMode: (id: string, tab: string) => void;
};

const SiteSettingSearchResults = ({
  searchedSettings,
  formState,
  onChange,
  changeToViewMode,
}: SiteSettingSearchResultsProps) => {
  const settingsMap = Object.entries(searchedSettings);

  return (
    <div className={classes.searchPage}>
      <h4 className="mb-5">Search Results</h4>
      {!!settingsMap.length ? (
        settingsMap.map(([tab, settings], index) => (
          <div className="mb-2" key={tab}>
            <h5 className="mb-4">{tab}</h5>
            <Settings
              settings={settings}
              formState={formState}
              onChange={onChange}
              changeToViewMode={(data) => changeToViewMode(data, tab)}
              searchView={true}
            />
            {index !== settingsMap.length - 1 && <hr className="m-0" />}
          </div>
        ))
      ) : (
        <div
          className={classNames(
            'py-5 d-flex justify-content-center',
            classes.noResult
          )}
        >
          <h4>No results found matching your search.</h4>
        </div>
      )}
    </div>
  );
};

export { SiteSettingSearchResults };
