import React from 'react';
import classes from '../SiteSettingsPage.module.scss';
import classNames from 'classnames';
import { Button } from 'react-bootstrap';
import { SiteSettingsSearchBar } from './SiteSettingsSearchBar';

type SiteSettingsHeaderProps = {
  doSearch: (term: string) => void;
  isUndoVisible: boolean;
  isLoading: boolean;
  searchTerm: string;
  clear: () => void;
};

const SiteSettingsHeader = ({
  doSearch,
  isUndoVisible,
  isLoading,
  searchTerm,
  clear,
}: SiteSettingsHeaderProps) => {
  return (
    <div
      className={classNames(classes.header, 'd-flex justify-content-between')}
    >
      <div className={classNames('d-flex align-items-center', classes.gap30)}>
        <h3>Site Settings</h3>
        <div
          className={classNames('d-flex', {
            [classes.dirtyState]: isUndoVisible,
          })}
        >
          <Button
            variant="light"
            type="submit"
            disabled={isLoading}
            className={classNames(
              classes.saveBtn,
              'btn btn-secondary btn-sm save-btn'
            )}
            data-test-id="site_settings_save_btn"
          >
            <span
              className={`fa ${
                isLoading ? 'fa-spinner fa-spin' : 'fa-download'
              } `}
            ></span>{' '}
            Save
          </Button>
          <Button
            variant="link"
            type="reset"
            className={classNames(classes.reset, classes.linkBtn)}
            data-test-id="site_settings_undo_btn"
          >
            <span className="fa fa-undo" /> Undo
          </Button>
        </div>
      </div>
      <SiteSettingsSearchBar
        doSearch={doSearch}
        searchTerm={searchTerm}
        clear={clear}
      />
    </div>
  );
};

export { SiteSettingsHeader };
