import {
  ConfirmObj,
  SettingOption,
  SettingType,
  SettingValue,
  State,
} from './site-settings.type';

export type AiSettingsProps = {
  tab: AiTab;
  formState: State['form'];
  highlightedId?: string;
  onChange: (id: string, value: SettingValue, confirmObj?: ConfirmObj) => void;
};

export type ProfilesConfig = {
  defaultProf: ProfileOption | null;
  profileSettings: ProfileSettings;
};

export type ProfileSetting = {
  label: string;
  values: State['form'];
};

export type ProfileSettings = {
  [key: string]: ProfileSetting;
};

export type ClonedProfileSetting = ProfileSetting & {
  source: string;
};

export type ProfileOption = {
  value: string;
  label: string;
};

export type ProfileModal = {
  prof: ProfileOption;
  settingValues: State['form'];
};

export type AiTab = 'embeddings' | 'completion';
export type AiEngine =
  | 'open_ai'
  | 'caii'
  | 'azure_ai'
  | 'llama'
  | 'bedrock'
  | 'other';
export type AiCmlAuthMode = 'jwt' | 'api_key';

export type AiProfileCols = {
  profile: string;
  label: string;
  engine: AiEngine;
};

export type AiSetting = {
  id: string;
  label: string;
  type: SettingType;
  required?: boolean;
  options?: SettingOption[];
  popoverText?: string | React.JSX.Element;
  hidden?: (formState?: State['form']) => boolean;
  warningText?: {
    visible: (formState: State['form']) => boolean;
    msg: string;
  };
  placeholder?: string;
  min?: number;
  max?: number;
  step?: number;
  inline?: boolean;
};

export type EngineSpecificSettings = {
  [key in AiEngine]: {
    embeddings: AiSetting[];
    completion: AiSetting[];
  };
};

export type AlertMessage = { text: string; type: 'success' | 'danger' };
export type ProfileTestResult = {
  success: boolean;
  message: string;
};
