import { Placement } from 'react-bootstrap/esm/Overlay';

export type SettingValue = string | boolean;

export type ConfirmObj = {
  title: string;
  text: JSX.Element;
  dispatch: { type: string; payload: { [x: string]: SettingValue } };
};

export type SettingType =
  | 'checkbox'
  | 'input'
  | 'numeric'
  | 'radio'
  | 'range'
  | 'textarea'
  | 'dropdown';

export type SettingOption = {
  id: string;
  label: string;
};

export type SiteSetting = {
  id: string;
  label: string;
  type: SettingType;
  required?: boolean;
  options?: SettingOption[];
  popoverText?: string | React.JSX.Element;
  infoText?: string;
  inline?: boolean;
  disabled?: (formState: State['form']) => boolean;
  hidden?: (formState?: State['form']) => boolean;
  confirm?: (formState: State['form']) => ConfirmObj | undefined;
  warningText?: {
    visible: (formState: State['form']) => boolean;
    msg: string;
  };
  tooltipPlacement?: Placement;
  disabledText?: string;
  placeholder?: string;
  value?: SettingValue;
  min?: number;
  max?: number;
  step?: number;
};

export type SettingTab = {
  tab: string;
  settings: SiteSetting[];
};

export type DirtySetting = {
  id: string;
  label: string;
  value: SettingValue;
};

export type State = {
  form: {
    [x: string]: SettingValue;
  };
  defaultAfterApi: {
    [x: string]: SettingValue;
  } | null;
  dirtyFields: {
    [x: string]: SettingValue;
  } | null;
};

export type SearchedSetting = {
  [x: string]: SiteSetting[];
};

export type SubmitData = {
  setting_key: string;
  setting_value: SettingValue;
  setting_read_admin_only: number;
}[];

export type SubmitFlipper = {
  id: string;
  value: string;
}[];

export type SettingGroup = {
  tab: string;
  settings?: SiteSetting[];
  children?: SettingGroup[];
};
