import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export const getExportInfoQueryToken = 'get-export-info';

export type ExportInfo = {
  staticasset: { id: number; name: string }[];
  appgroupmembership: { id: number; name: string }[];
  segments: { id: number; name: string }[];
  reportimage: { id: number; name: string }[];
  visuals: { id: number; type: string; name: string }[];
  appgroups: { id: number; name: string }[];
  customcss: { id: number; name: string }[];
  colorpalette: { id: number; name: string }[];
  events: { id: number; name: string }[];
  datasets: { id: number; name: string }[];
  dateranges: { id: number; name: string }[];
  dashboards: { id: number; name: string }[];
  // version: {
  //   'Arcviz Version': string;
  //   'Raw Timestamp of last commit': string;
  //   'Timestamp of last commit': string;
  //   'SHA of last commit': string;
  //   Description: string;
  // };
};

export const exportVisualArtifacts = ({
  visualIds,
  dashboardIds,
  appIds,
  filename,
  isDryRun,
}: {
  visualIds?: number[];
  dashboardIds?: number[];
  appIds?: number[];
  filename: string;
  isDryRun: boolean;
}) => {
  const params: { [key: string]: string } = {
    filename,
    dry_run: isDryRun ? 'true' : 'false',
  };

  if (visualIds) {
    params.visuals = `[${visualIds.join(',')}]`;
  }

  if (dashboardIds) {
    params.dashboards = `[${dashboardIds.join(',')}]`;
  }

  if (appIds) {
    params.appgroups = `[${appIds.join(',')}]`;
  }

  return vizHttp
    .get<ExportInfo>({
      url: arc.frontend.globals.urls.exportUrl,
      isOldEndpoint: true,
      params,
    })
    .then(({ data }) => data);
};
