import { vizHttp } from 'services/viz-http';
import { getGlobalArc } from 'utils/get-global-arc';

const arc = getGlobalArc();

export type MoveDashboardInfo = {
  id: number;
  type: 'dashboard';
  dashboardName: string;
  sharedVisualIds: string[];
};

export type MoveReportsInfos = (
  | { id: number; type: 'app' }
  | {
      id: number;
      type: 'visual';
    }
  | MoveDashboardInfo
)[];

export const moveReportsToWorkspace = async ({
  moveReportsInfos,
  targetWorkspaceId,
  dashboardIdToVisualIdsToCloneMap,
}: {
  moveReportsInfos: MoveReportsInfos;
  targetWorkspaceId: number;
  dashboardIdToVisualIdsToCloneMap?: Map<number, Set<string>>;
}) => {
  return vizHttp
    .post<
      {
        report_id: number;
        reports_failed: string[];
        reports_to_clone: { old_id: number; new_id: number }[];
        reports_to_move: { [id: number]: true };
        error?: string;
      }[]
    >({
      url: arc.frontend.globals.urls.moveReportsToWorkspaceUrl,
      data: {
        dashboards_info: JSON.stringify(
          moveReportsInfos.map((report) => {
            if (report.type === 'app') {
              return {
                report_id: report.id,
                type: 'apps',
              };
            }

            if (report.type === 'visual') {
              return {
                report_id: report.id,
                type: 'visual',
              };
            }

            const dashboardInfo: {
              report_id: number;
              type: 'dashboard';
              dashboardName: string;
              sharedVisuals: string[];
              reports_to_clone?: number[];
            } = {
              report_id: report.id,
              type: 'dashboard',
              dashboardName: report.dashboardName,
              sharedVisuals: report.sharedVisualIds,
            };

            if (dashboardIdToVisualIdsToCloneMap?.has(report.id)) {
              dashboardInfo.reports_to_clone = [
                ...dashboardIdToVisualIdsToCloneMap.get(report.id)!,
              ].map(Number);
            }

            return dashboardInfo;
          })
        ),
        current_workspace_id: targetWorkspaceId.toString(),
      },
      headers: {
        'X-CSRFTOKEN': arc.frontend.globals.authentication.csrfToken,
      },
      isOldEndpoint: true,
    })
    .then(({ data }) => data);
};
