import Search from '@cloudera-internal/cuix-core/icons/svg/ic-search.svg';
import { useDebounce } from 'hooks/use-debounce';
import React, { ChangeEvent, useCallback } from 'react';
import classes from './SearchBar.module.scss';

type SearchBarProps = {
  id: string;
  placeholder: string;
  searchTerm: string;
  setSearchTerm: (searchTerm: string) => void;
  testId?: string;
  debounce?: number;
};

export const SearchBar = ({
  id,
  searchTerm,
  setSearchTerm,
  placeholder,
  testId,
  debounce = 300,
}: SearchBarProps): JSX.Element => {
  const [value, setValue] = React.useState(searchTerm);
  const debouncedSearch = useDebounce(setSearchTerm, debounce);

  const handleSearch = useCallback(
    (e: ChangeEvent<HTMLInputElement>) => {
      setValue(e.target.value);
      debouncedSearch(e.target.value);
    },
    [debouncedSearch]
  );

  return (
    <div className={classes.container}>
      <label htmlFor={id} className={classes.label}>
        <span className={classes.searchIcon}>
          <Search />
        </span>
      </label>

      <input
        id={id}
        value={value}
        onChange={handleSearch}
        placeholder={placeholder}
        className={classes.input}
        data-test-id={testId}
      />
    </div>
  );
};
