import { renderHook, waitFor } from '@testing-library/react';
import { BackendApp, getApps } from 'apis/get-apps';
import { useApps } from './use-apps';
import React from 'react';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';

jest.mock('apis/get-apps');

const queryClient = new QueryClient();

const wrapper = ({ children }) => (
  <QueryClientProvider client={queryClient}>{children}</QueryClientProvider>
);

window.arc.frontend = {
  globals: {
    urls: {
      editAppUrl: 'edit-app-url',
    },
  },
} as any;

it('should not include apps in the sample dashboards ids that have the same ids as the sample dashboards', async () => {
  const MOCK_APPS: BackendApp[] = [
    {
      group_id: 1,
      group_type: 'samples',
      name: 'Sample Dashboards',
      group_options:
        '{"user_settings":{"menu_on_left":"false","navigate_to_apps_page":"false","container_css_class":"","css_styles":"[]","inline_css":""}}',
      create_user: 'vizapps_admin',
      update_user: 'vizapps_admin',
      created: '2021-05-03T16:08:28.332005+00:00',
      updated: '2021-05-03T16:08:28.332095+00:00',
      workspace_name: 'Public',
      workspace_id: 1,
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      baseImg: '/arc/static/img/icon_app.png?v=6',
      perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      num_apps: 2,
      apps: [
        [12, 0.5],
        [8, 1],
      ],
      rw: true,
      in_userlist: 'false',
      subgroups: [
        {
          group_id: 2,
          group_type: 'default',
          name: 'Restaurant Inspections',
          subgroup_order: null,
          group_options: '',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:28.350773+00:00',
          updated: '2021-05-03T16:08:28.350862+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 2,
          apps: [
            [23, 0.5],
            [17, 1],
          ],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 1,
        },
        {
          group_id: 3,
          group_type: 'default',
          name: 'Global Trends in CyberAttacks',
          subgroup_order: null,
          group_options: '',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:28.367033+00:00',
          updated: '2021-05-03T16:08:28.367124+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 2,
          apps: [
            [35, 0.5],
            [27, 1],
          ],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 1,
        },
        {
          group_id: 4,
          group_type: 'default',
          name: 'Census Analysis',
          subgroup_order: null,
          group_options: '',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:28.385463+00:00',
          updated: '2021-05-03T16:08:28.385600+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 2,
          apps: [
            [49, 0.5],
            [43, 1],
          ],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 1,
        },
        {
          group_id: 5,
          group_type: 'default',
          name: 'Others',
          subgroup_order: null,
          group_options: '',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:28.403621+00:00',
          updated: '2021-05-03T16:08:28.403711+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 5,
          apps: [
            [78, 0.0625],
            [73, 0.125],
            [66, 0.25],
            [57, 0.5],
            [52, 1],
          ],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 1,
        },
      ],
    },
    {
      group_id: 6,
      group_type: 'sample-app',
      name: 'Sample App',
      group_options:
        '{"icon_class": "", "user_settings": {"menu_on_left": "false", "container_css_class": "", "css_styles": "[]", "inline_css": "#main .arc-dashboard-title-container{\\n    background: #45505a url(https://s3-media4.fl.yelpcdn.com/assets/srv0/yelp_styleguide/c146b0884f6a/assets/img/structural/header_stars.png) no-repeat left;\\n    border-bottom: solid 3px #d32323;\\n    font-size: 36px;\\n    padding-bottom: 16px;\\n    color: #fff8f8;\\n}\\n#main .viz-params-control .viz-params-btn i{\\n    color:#fff8f8 !important;\\n}\\n.arc-app-container .arc-viz-title-container {\\n    background-color: #f38181;\\n    color: #fff8f8;\\n}\\nbody{\\n     background-color: #f5f5f5;\\n     background-color: #eee;\\n}\\n.builder-container, .builder-container-app{\\n    background-color: #eee;\\n}\\n/*body{\\n    background-color: #f5f5f5;\\n    font-family: \'Helvetica Neue\', Helvetica, Arial, sans-serif;\\n}\\n.navbar{\\n    background: #d32323 url(https://s3-media4.fl.yelpcdn.com/assets/srv0/yelp_styleguide/c146b0884f6a/assets/img/structural/header_stars.png) no-repeat center;\\n    border-bottom: solid 3px #f5f5f5;\\n}\\n.navbar-default .navbar-nav > .active > a, \\n.navbar-default .navbar-nav > .active > a:hover, \\n.navbar-default .navbar-nav > .active > a:focus {\\n    background-color: #bd1f1f;\\n    border-bottom: solid 3px rgb(206, 112, 112);\\n    font-weight: 500;\\n    text-transform: none;\\n    font-size: 14px;\\n    letter-spacing: normal;\\n    font-family: \'Helvetica Neue\', Helvetica, Arial, sans-serif;\\n}\\n.arc-dashboard-title, .arc-dashboard-subtitle, .arc-viz-title, .arc-viz-subtitle{\\n    font-family: \'Helvetica Neue\', Helvetica, Arial, sans-serif;\\n    font-size: 30px;\\n    font-weight: bold;\\n    text-shadow: 0 1px rgba(255,255,255,0.7);\\n}\\n.arc-app-container .arc-viz-title-container{\\n    font-weight: 400;\\n}\\n.builder-container, .builder-container-app{\\n    background-color: #f5f5f5;\\n    width: 1200px;\\n    margin: 0 auto;\\n}*/", "navigate_to_apps_page": "false"}}',
      create_user: 'vizapps_admin',
      update_user: 'vizapps_admin',
      created: '2021-05-03T16:08:34.440727+00:00',
      updated: '2024-12-11T01:13:56.154910+00:00',
      workspace_name: 'Public',
      workspace_id: 1,
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      baseImg: '/arc/static/img/icon_app.png?v=6',
      perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      num_apps: 0,
      apps: [],
      rw: true,
      in_userlist: 'true',
      subgroups: [
        {
          group_id: 7,
          group_type: 'default',
          name: 'Food Store Inspections - State View',
          subgroup_order: 1,
          group_options: '{}',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:34.460155+00:00',
          updated: '2021-05-03T16:08:34.460246+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 1,
          apps: [[93, 1]],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 6,
        },
        {
          group_id: 8,
          group_type: 'default',
          name: 'Deficiency Details',
          subgroup_order: 2,
          group_options: '{}',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:34.482950+00:00',
          updated: '2021-05-03T16:08:34.483089+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 1,
          apps: [[94, 1]],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 6,
        },
        {
          group_id: 9,
          group_type: 'default',
          name: 'Store Details',
          subgroup_order: 3,
          group_options: '{}',
          create_user: 'vizapps_admin',
          update_user: 'vizapps_admin',
          created: '2021-05-03T16:08:34.501527+00:00',
          updated: '2021-05-03T16:08:34.501621+00:00',
          workspace_name: 'Public',
          workspace_id: 1,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 1,
          apps: [[92, 1]],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 6,
        },
      ],
    },
    {
      group_id: 11,
      group_type: 'starred',
      name: 'Starred',
      group_options: '',
      create_user: 'vizapps_admin',
      update_user: 'vizapps_admin',
      created: '2021-05-03T16:12:25.419647+00:00',
      updated: '2021-05-03T16:12:25.419746+00:00',
      workspace_name: 'Public',
      workspace_id: 1,
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      baseImg: '/arc/static/img/icon_app.png?v=6',
      perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      num_apps: 3,
      apps: [
        [35, 0.25],
        [8, 0.5],
        [1663, 1],
      ],
      rw: true,
      in_userlist: 'false',
      subgroups: undefined,
    },
    {
      group_id: 78,
      group_type: 'default',
      name: 'test impala app',
      group_options:
        '{"user_settings":{"container_css_class":"","css_styles":"[]","inline_css":"","menu_on_left":"false","navigate_to_apps_page":"false","app_menubar_bg_color":"","app_menubar_bottombar_color":"","app_menubar_barcolor_selected":"","app_menutext_font":"default","app_menutext_font_size":"","app_menutext_font_weight":"default","app_menutext_color":"","app_menutext_color_selected":"","app_menutext_uppercase":"true","app_logo_image":"","app_logo_width":"185","app_logo_height":"45","app_logo_margintop":"5","app_logo_marginleft":"15","app_bg_bg_color":"","app_hide_dropshadow":"false","app_show_title":"false"}}',
      create_user: 'roli_test',
      update_user: 'roli_test',
      created: '2022-09-28T13:27:20.827770+00:00',
      updated: '2022-09-28T13:27:20.827943+00:00',
      workspace_name: 'Private',
      workspace_id: 26,
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      baseImg: '/arc/static/img/icon_app.png?v=6',
      perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      num_apps: 0,
      apps: [],
      rw: true,
      in_userlist: 'false',
      subgroups: [
        {
          group_id: 79,
          group_type: 'default',
          name: 'test impala 6.3',
          subgroup_order: 1,
          group_options: '{}',
          create_user: 'roli_test',
          update_user: 'roli_test',
          created: '2022-09-28T13:27:20.865214+00:00',
          updated: '2022-09-28T13:27:20.865333+00:00',
          workspace_name: 'Private',
          workspace_id: 26,
          workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          baseImg: '/arc/static/img/icon_app.png?v=6',
          perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
          num_apps: 1,
          apps: [[1707, 1]],
          rw: true,
          in_userlist: 'false',
          group_parent_id: 78,
        },
      ],
    },
  ];

  jest.mocked(getApps).mockResolvedValueOnce(MOCK_APPS);

  const { result } = renderHook(useApps, {
    wrapper,
  });

  await waitFor(() => expect(getApps).toHaveBeenCalled());

  const { apps, dashboards } = result.current.samplesReportIds;

  expect([...apps].sort().join(',')).toEqual('6');
  expect([...dashboards].sort().join(',')).toEqual(
    '12,17,23,27,35,43,49,52,57,66,73,78,8'
  );
});
