import React from 'react';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { renderHook, waitFor } from '@testing-library/react';
import { useReports } from './use-reports';
import { BackendReport, getReports } from 'apis/get-reports';

jest.mock('utils/get-global-arc', () => {
  const arcMock: any = {
    frontend: {
      globals: {
        urls: {
          reportUrl: 'report-url',
        },
      },
      featureFlipper: {
        getValueById: jest.fn(),
      },
    },
  };

  return {
    getGlobalArc: () => arcMock,
  };
});

jest.mock('apis/get-reports');

const queryClient = new QueryClient();

const wrapper = ({ children }) => (
  <QueryClientProvider client={queryClient}>{children}</QueryClientProvider>
);

it('should not include apps in samples that have the same ids as the sample dashboards', async () => {
  const EXPECTED_SAMPLE_APP_IDS = [6];
  const EXPECTED_SAMPLE_DASHBOARD_IDS = [
    78, 73, 66, 57, 52, 49, 43, 35, 27, 23, 17, 12, 8,
  ];

  const MOCK_REPORTS: BackendReport[] = [
    {
      id: 8,
      name: 'Taxi rides application',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:28.689132+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:09.954404+00:00',
      updated_by: 'juno',
      views_count: 147,
      descendants_count: 1,
      is_clone: false,
      mode: '',
      in_userlist: true,
      dataset_id: 1,
      dataset_name: 'NYC Taxicab Rides',
      dataconn_name: 'samples',
      dataset_detail: 'main.trips',
      dataset_type: 'multipletables',
      visuals: [1, 2, 3, 4, 5, 6],
      pickers: [7],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/8',
      shared_visual_dashboards: '[]',
      title: 'Taxi rides application',
      url: '/arc/apps/app/8',
      img: '/arc/apps/snapshot/8?v=2023.04.19.23.00.25',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 3,
      workspace_name: 'juno private workspace',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 12,
      name: 'Iris species w/ images',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:29.057257+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:09.958011+00:00',
      updated_by: 'juno',
      views_count: 24,
      descendants_count: 0,
      is_clone: false,
      mode: null,
      dataset_id: 3,
      dataset_name: 'Iris',
      dataconn_name: 'samples',
      dataset_detail: 'main.iris',
      dataset_type: 'singletable',
      visuals: [9, 10, 11],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/12',
      shared_visual_dashboards: '[]',
      title: 'Iris species w/ images',
      url: '/arc/apps/app/12',
      img: '/arc/apps/snapshot/12?v=2023.04.19.22.56.52',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 3,
      workspace_name: 'juno private workspace',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 17,
      name: 'Consumer View',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:29.390690+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:09.961575+00:00',
      updated_by: 'taddwood',
      views_count: 19,
      descendants_count: 0,
      is_clone: false,
      mode: null,
      dataset_id: 4,
      dataset_name: 'Restaurant Inspection SF',
      dataconn_name: 'samples',
      dataset_detail: 'main.restaurant_scores_lives_standard',
      dataset_type: 'singletable',
      visuals: [16, 13, 14, 15],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/17',
      shared_visual_dashboards: '[]',
      title: 'Consumer View',
      url: '/arc/apps/app/17',
      img: '/arc/apps/snapshot/17?v=2021.09.27.21.15.56',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 23,
      name: 'Inspector View',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:29.711945+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:09.967557+00:00',
      updated_by: 'hossameldin.abdin',
      views_count: 133,
      descendants_count: 0,
      is_clone: false,
      mode: '',
      dataset_id: 5,
      dataset_name: 'Restaurant Inspection SF Search',
      dataconn_name: 'samples',
      dataset_detail: 'main.restaurant_scores_lives_standard',
      dataset_type: 'singletable',
      visuals: [18, 19, 20, 21, 22],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/23',
      shared_visual_dashboards: '[]',
      title: 'Inspector View',
      url: '/arc/apps/app/23',
      img: '/arc/apps/snapshot/23?v=2021.09.16.14.17.19',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 27,
      name: 'Time & Industry Threat View',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:30.079183+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:09.989928+00:00',
      updated_by: 'szhou',
      views_count: 138,
      descendants_count: 2,
      is_clone: false,
      mode: null,
      dataset_id: 6,
      dataset_name: 'Global Information Security Threats',
      dataconn_name: 'samples',
      dataset_detail: 'main.infoseq_1559',
      dataset_type: 'singletable',
      visuals: [24, 25, 26, 4430],
      pickers: [3609, 3610, 1403, 3613, 3614],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/27',
      shared_visual_dashboards: '[]',
      title: 'Time & Industry Threat View',
      url: '/arc/apps/app/27',
      img: '/arc/apps/snapshot/27?v=2024.02.20.17.34.53',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 35,
      name: 'Global Threats',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:30.574649+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:09.999816+00:00',
      updated_by: 'vizapps_admin',
      views_count: 193,
      descendants_count: 6,
      is_clone: false,
      mode: null,
      in_userlist: true,
      dataset_id: 6,
      dataset_name: 'Global Information Security Threats',
      dataconn_name: 'samples',
      dataset_detail: 'main.infoseq_1559',
      dataset_type: 'singletable',
      visuals: [32, 33, 34, 28, 29, 30, 31],
      pickers: [2222, 2223],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/35',
      shared_visual_dashboards: '[]',
      title: 'Global Threats',
      url: '/arc/apps/app/35',
      img: '/arc/apps/snapshot/35?v=2024.01.30.08.43.08',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 43,
      name: 'Census Dashboard',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:31.128837+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:10.002774+00:00',
      updated_by: 'vizapps_admin',
      views_count: 2337,
      descendants_count: 1,
      is_clone: false,
      mode: null,
      dataset_id: 7,
      dataset_name: 'US State Populations Over Time',
      dataconn_name: 'samples',
      dataset_detail: 'main.census_pop',
      dataset_type: 'singletable',
      visuals: [36, 37, 38, 39, 40, 41, 42],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/43',
      shared_visual_dashboards: '[]',
      title: 'Census Dashboard',
      url: '/arc/apps/app/43',
      img: '/arc/apps/snapshot/43?v=2021.05.03.16.08.31',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 49,
      name: 'US State Population Trends',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:31.690292+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:10.005321+00:00',
      updated_by: 'vizapps_admin',
      views_count: 73,
      descendants_count: 0,
      is_clone: false,
      mode: null,
      dataset_id: 7,
      dataset_name: 'US State Populations Over Time',
      dataconn_name: 'samples',
      dataset_detail: 'main.census_pop',
      dataset_type: 'singletable',
      visuals: [5348, 44, 45, 46, 47, 48],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/49',
      shared_visual_dashboards: '[]',
      title: 'US State Population Trends',
      url: '/arc/apps/app/49',
      img: '/arc/apps/snapshot/49?v=2024.06.24.13.42.37',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 52,
      name: 'Animated world population - GDP vs life expectancy',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:31.973747+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:10.007316+00:00',
      updated_by: 'vizapps_admin',
      views_count: 9,
      descendants_count: 0,
      is_clone: false,
      mode: null,
      dataset_id: 9,
      dataset_name: 'World Life Expectancy',
      dataconn_name: 'samples',
      dataset_detail: 'main.world_life_expectancy',
      dataset_type: 'singletable',
      visuals: [50, 51],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/52',
      shared_visual_dashboards: '[]',
      title: 'Animated world population - GDP vs life expectancy',
      url: '/arc/apps/app/52',
      img: '/arc/apps/snapshot/52?v=2021.05.03.16.08.31',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 57,
      name: '<<country>> Population & GDP Trends',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:32.275563+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:10.009326+00:00',
      updated_by: 'juno',
      views_count: 25,
      descendants_count: 1,
      is_clone: false,
      mode: null,
      dataset_id: 9,
      dataset_name: 'World Life Expectancy',
      dataconn_name: 'samples',
      dataset_detail: 'main.world_life_expectancy',
      dataset_type: 'singletable',
      visuals: [56, 53, 54, 55],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/57',
      shared_visual_dashboards: '[]',
      title: '<<country>> Population & GDP Trends',
      url: '/arc/apps/app/57',
      img: '/arc/apps/snapshot/57?v=2021.06.22.18.13.36',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 66,
      name: 'Life Expectancy Dashboard',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:32.735991+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:10.010930+00:00',
      updated_by: 'vizapps_admin',
      views_count: 27,
      descendants_count: 0,
      is_clone: false,
      mode: null,
      dataset_id: 9,
      dataset_name: 'World Life Expectancy',
      dataconn_name: 'samples',
      dataset_detail: 'main.world_life_expectancy',
      dataset_type: 'singletable',
      visuals: [64, 65, 58, 59, 60, 61, 62, 63],
      pickers: [],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/66',
      shared_visual_dashboards: '[]',
      title: 'Life Expectancy Dashboard',
      url: '/arc/apps/app/66',
      img: '/arc/apps/snapshot/66?v=2021.05.03.16.08.32',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 73,
      name: 'Earthquakes Around the World',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:33.185249+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-10-23T16:13:10.016156+00:00',
      updated_by: 'vizapps_admin',
      views_count: 26,
      descendants_count: 2,
      is_clone: false,
      mode: '',
      dataset_id: 10,
      dataset_name: 'Earthquake Data January 2019',
      dataconn_name: 'samples',
      dataset_detail: 'main.earthquake_data2019',
      dataset_type: 'singletable',
      visuals: [68, 69, 70, 71, 72],
      pickers: [67],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/73',
      shared_visual_dashboards: '[]',
      title: 'Earthquakes Around the World',
      url: '/arc/apps/app/73',
      img: '/arc/apps/snapshot/73?v=2021.05.03.16.08.33',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
    {
      id: 78,
      name: 'Cereal Comparisons',
      description: 'placeholder desc',
      type: 'dashboard',
      created: '2021-05-03T16:08:33.501814+00:00',
      report_tags: 'applications',
      created_by: 'vizapps_admin',
      updated: '2024-11-14T18:29:43.222346+00:00',
      updated_by: 'juno',
      views_count: 138,
      descendants_count: 0,
      is_clone: false,
      mode: null,
      dataset_id: 11,
      dataset_name: 'Cereals',
      dataconn_name: 'samples',
      dataset_detail: 'main.cereals',
      dataset_type: 'multipletables',
      visuals: [5569, 5570, 5571, 74, 75, 76, 77, 2648, 2653],
      pickers: [2654, 2655],
      report_type: 'dashboard',
      edit_url: '/arc/apps/builder/78',
      shared_visual_dashboards: '[]',
      title: 'Cereal Comparisons',
      url: '/arc/apps/app/78',
      img: '/arc/apps/snapshot/78?v=2024.11.14.18.29.55',
      baseImg: '/arc/static/img/icon_dashboard.png?v=6',
      workspace_id: 1,
      workspace_name: 'Public',
      workspace_perm: ['ws_manage', 'ws_appedit', 'ws_appview'],
      dataset_perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
      perm: [
        'dc_explore',
        'dc_upload',
        'dc_aviews',
        'ds_appview',
        'ds_manage',
        'ds_appedit',
      ],
    },
  ];

  jest.mocked(getReports).mockResolvedValue(MOCK_REPORTS);

  const SAMPLES_GROUP = {
    id: -4,
    name: 'Samples',
    desc: 'Get started by looking through our samples and see how they use a combination of visuals and filters to provide business insights in sample datasets.',
  };

  const { result } = renderHook(useReports, {
    wrapper,
    initialProps: {
      appsAsCombinedReports: [
        {
          id: 6,
          baseImg: '/arc/static/img/icon_app.png?v=6',
          editUrl: '/arc/apps/appedit/6',
          img: '/arc/static/img/icon_app.png?v=6',
          permissions: ['ws_manage', 'ws_appedit', 'ws_appview'],
          title: 'Sample App',
          type: 'app',
          updated: {
            date: '2024-12-11T01:13:56.154910+00:00',
            by: 'vizapps_admin',
          },
          isClone: false,
          numberOfDescendants: 0,
          url: '/arc/apps/appgroup/6',
          workspace: {
            id: 1,
            name: 'Public',
          },
          rw: true,
          created: {
            date: '2021-05-03T16:08:34.440727+00:00',
            by: 'vizapps_admin',
          },
        },
        {
          id: 78,
          baseImg: '/arc/static/img/icon_app.png?v=6',
          editUrl: '/arc/apps/appedit/78',
          img: '/arc/static/img/icon_dashboard.png?v=6',
          permissions: ['ws_manage', 'ws_appedit', 'ws_appview'],
          title: 'test impala app',
          type: 'app',
          updated: {
            date: '2022-09-28T13:27:20.827943+00:00',
            by: 'roli_test',
          },
          isClone: false,
          numberOfDescendants: 0,
          url: '/arc/apps/appgroup/78',
          workspace: {
            id: 26,
            name: 'Private',
          },
          rw: true,
          created: {
            date: '2022-09-28T13:27:20.827770+00:00',
            by: 'roli_test',
          },
        },
      ],
      favoriteReportIds: new Set<number>(),
      samplesReportIds: {
        apps: new Set(EXPECTED_SAMPLE_APP_IDS),
        dashboards: new Set(EXPECTED_SAMPLE_DASHBOARD_IDS),
      },
      selectedGroup: SAMPLES_GROUP,
      staticSideBarItems: {
        all: {
          id: -1,
          name: 'All',
          desc: '',
        },
        favorites: {
          id: -2,
          name: 'My Favorites',
          desc: 'Mark your frequently used dashboards as favorites. These bookmarks make it easy to find your most-used content in a subsequent work session.',
        },
        samples: SAMPLES_GROUP,
      },
    },
  });

  await waitFor(() => expect(getReports).toHaveBeenCalledTimes(1));
  await waitFor(() => expect(result.current.areReportsLoading).toBe(false));

  const sampleIds = result.current.groupIdToReportTypeToCombinedReports
    .get('samples')
    ?.get('all')
    ?.map(({ id }) => id);
  expect(sampleIds?.sort().join(',')).toEqual(
    [...EXPECTED_SAMPLE_DASHBOARD_IDS, ...EXPECTED_SAMPLE_APP_IDS]
      .sort()
      .join(',')
  );

  const sampleAppIds = result.current.groupIdToReportTypeToCombinedReports
    .get('samples')
    ?.get('apps')
    ?.map(({ id }) => id);
  expect(sampleAppIds?.sort().join(',')).toEqual(
    EXPECTED_SAMPLE_APP_IDS.sort().join(',')
  );

  const sampleDashboardIds = result.current.groupIdToReportTypeToCombinedReports
    .get('samples')
    ?.get('dashboards')
    ?.map(({ id }) => id);
  expect(sampleDashboardIds?.sort().join(',')).toEqual(
    EXPECTED_SAMPLE_DASHBOARD_IDS.sort().join(',')
  );
});
