import { useCallback, useState } from 'react';
import { vizCookie } from 'services/viz-cookie';
import { ReportFilterType } from 'types/report-filter-type.type';

const params = new URLSearchParams(window.location.search);

const cookieAndUrlName = 'displaymode' as const;

export const useSelectedReportType = () => {
  const [selectedReportType, setSelectedReportType] =
    useState<ReportFilterType>(() => {
      const reportTypeFromSearchParams = params.get(cookieAndUrlName);

      if (
        reportTypeFromSearchParams &&
        ['all', 'dashboards', 'apps'].includes(reportTypeFromSearchParams)
      ) {
        return reportTypeFromSearchParams as ReportFilterType;
      }

      const selectedTypeFromCookies =
        vizCookie.getVisualTabSettingByKey(cookieAndUrlName);

      return selectedTypeFromCookies ?? 'all';
    });

  const selectedTypeSetter = useCallback((type: ReportFilterType) => {
    setSelectedReportType(type);

    vizCookie.setVisualTabSettingByKey(cookieAndUrlName, type);

    if (window.location.search !== '') {
      const { pathname, hash } = window.location;

      const url = `${pathname}?${cookieAndUrlName}=${type}${hash}`;

      window.top?.postMessage(
        {
          urlUpdate: url,
        },
        '*'
      );

      if (window.history.pushState) {
        window.history.pushState(null, '', url);
      } else {
        window.open(url, '_self');
      }
    }
  }, []);

  return {
    selectedReportType,
    setSelectedReportType: selectedTypeSetter,
  };
};
