import { useCallback, useState } from 'react';
import { vizCookie } from 'services/viz-cookie';

export type ReportViewType = 'grid' | 'list';

const listHash = '#arc-list';
const gridHash = '#arc-grid';
const cookieName = 'displaystyle' as const;

export const useSelectedViewMode = () => {
  const [selectedViewMode, setSelectedViewMode] = useState<ReportViewType>(
    () => {
      if (window.location.hash === listHash) {
        return 'list';
      }

      if (window.location.hash === gridHash) {
        return 'grid';
      }

      const cookieIsDisplayMode =
        vizCookie.getVisualTabSettingByKey(cookieName);

      if (cookieIsDisplayMode === undefined) {
        return 'grid';
      }

      return cookieIsDisplayMode ? 'grid' : 'list';
    }
  );

  const selectedViewSetter = useCallback((view: ReportViewType) => {
    setSelectedViewMode(view);

    vizCookie.setVisualTabSettingByKey(cookieName, view === 'grid');

    window.location.replace(view === 'grid' ? gridHash : listHash);
  }, []);

  return {
    selectedViewMode,
    setSelectedViewMode: selectedViewSetter,
  };
};
