import { cloneDeep } from 'lodash-es';

import { exposeToGlobal } from 'utils/expose-to-global';

type Flipper = {
  id: string;
  label: string;
  info: string;
  value: boolean;
  level: 'DB' | 'SETTINGS' | 'URL';
};

export class FeatureFlipper {
  static #instance: FeatureFlipper;

  public static get(): FeatureFlipper {
    if (!this.#instance) {
      this.#instance = new FeatureFlipper();
    }

    return this.#instance;
  }

  #flippers: Flipper[] = [];

  public initialize(flippersAsJson: string): void {
    this.#flippers = JSON.parse(flippersAsJson) as Flipper[];
  }

  public getValueById(id: string): boolean | undefined {
    const selected = this.#flippers.find((flipper) => flipper.id === id);

    if (selected === undefined) {
      console.error(`Feature flipper is not found. id: "${id}"`);
    }

    return selected?.value;
  }

  public getAll(): Flipper[] {
    return cloneDeep(this.#flippers);
  }
}

export type FeatureFlipperService = FeatureFlipper;

exposeToGlobal({ featureFlipper: FeatureFlipper.get() }, 'frontend');
