import type {
  AiEngine,
  AiCmlAuthMode,
} from 'pages/site-settings-page/types/ai-settings.type';
import type { PasswordValidation } from 'types/password-validation.type';

// This has the same purpose as arc_globals.js, I separated out what was needed for the React pages
// This way we can load only the new code for those pages.
// The issue with arc_globals.js was that it is depending on other files and without a module system it is too complicated to figure out
// the complete list of dependencies it has, and even if we have the list we still would need to load other files that we are not really using.

export type Globals = {
  urls: {
    staticUrl: string;
    loginUrl: string;
    reportUrl: string;
    reportCreateUrl: string;
    appgroupUrl: string;
    homepageStatisticsUrl: string;
    homepageNumberOfRecentlyCreatedResourcesUrl: string;
    homepageRecentlyCreatedDashboardsByUserUrl: string;
    homepageLastViewedReportsByUserUrl: string;
    homepageMostPopularReportsUrl: string;
    homepageSamplesUrl: string;
    homepageFavoriteDashboardsUrl: string;
    homepageDatasetsUrl: string;
    reportBuilderUrl: string;
    dataImportUrl: string;
    dataUrl: string;
    sqlUrl: string;
    datasetEditUrl: string;
    appGroupEditUrl: string;
    dataConnectionUrl: string;
    savedSqlUrl: string;
    datasetUrl: string;
    homeUrl: string;
    appsUrl: string;
    setHomePageUrl: string;
    assignDefaultAppUrl: string;
    disableHomepageOverrideUrl: string;
    jobOverviewUrl: string;
    jobViewRefreshUrl: string;
    activityUrl: string;
    usersUrl: string;
    rolesUrl: string;
    aliasesUrl: string;
    apiKeysUrl: undefined | string;
    appsApiKeysUrl: undefined | string;
    customEmailUrl: string;
    previewCustomEmailUrl: string;
    customExtensionEditUrl: string;
    customColorsUrl: string;
    customDates: string;
    staticAssetsUrl: string;
    adminSettingsUrl: string;
    userDataUrl: string;
    usersDetailedApiUrl: string;
    logoutUrl: string;
    notificationsUrl: string;
    siteSettingsUrl: string;
    emailTestUrl: string;
    deleteUsageDatabaseUrl: string;
    saveFrontendEventsTrackerData: string;
    workspacesUrl: string;
    reportUsageUrl: string;
    viewAppUrl: string;
    editAppUrl: string;
    viewDashboardUrl: string;
    groupsUrl: string;
    exportUrl: string;
    accelerateUrl: string;
    moveReportsToWorkspaceUrl: string;
    jobGetJobUrl: string;
    jobGetStatusUrl: string;
    jobEditUrl: string;
    jobEditOwnerUrl: string;
    jobsApiUrl: string;
    jobScheduleGroupsUrl: string;
    jobCancelUrl: string;
    jobRerunUrl: string;
    jobPauseOrResumeUrl: string;
    jobGetLogsUrl: string;
    jobsLatestLogsApiUrl: string;
    jobLogsApiUrl: string;
    jobDetailPageUrl: string;
    emailTemplatesUrl: string;
    emailUrl: string;
    extractUrl: string;
    getReportCountPerGroup: string;
    getReportCreationPermissions: string;
    roleDetailsUrl: string;
    rbacRolesUrl: string;
    getPossibleDeletedReportsUrl: string;
    getQueryLogsUrl: string;
    urlShortenerUrl: string;
    validateAiProfile: string;
    usersBasicApiUrl: string;
    groupsBasicUrl: string;
    statsDataUrl: string;
    cssUsageUrl: string;
    colorPaletteUsageUrl: string;
    csvDownloadUrl: string;
    xlsxdownloadUrl: string;
    sqlRunUrl: string;
    sqlParallelAsyncUrl: string;
    sqlParallelUrl: string;
    asynCancelQtUrl: string;
    viewRecUrl: string;
    rawSqlViewRecUrl: string;
    rawSqlQueriesUrl: string;
    reportsAccelUrl: string;
    queryAccelUrl: string;
    longRunningTaskUrl: string;
    snapshotUrl: string;
    snapshotFetchUrl: string;
    snapshotReadyUrl: string;
    viewRefreshUrl: string;
    emailSnapshotUrl: string;
    scriptScheduleUrl: string;
    scheduleGroupUrl: string;
    dataCopyUrl: string;
    dataCopyLogsUrl: string;
    dataCopyJobsUrl: string;
    cancelJobUrl: string;
    editJobUrl: string;
    editJobOwnerUrl: string;
    templateUrl: string;
    reportCloneUrl: string;
    reportsAddSearchUrl: string;
    reportBuildUrl: string;
    cloneWidgetsUrl: string;
    dashboardSaveUrl: string;
    reportMyappUrl: string;
    reportCloneInfoUrl: string;
    userAppGroupUrl: string;
    dataconnectionUrl: string;
    syncCmlConnectionsUrl: string;
    cloneDataconnectionUrl: string;
    cdwWarehouseUrl: string;
    datasetsUserPermsUrl: string;
    datasetsGroupPermsUrl: string;
    colorpaletteUrl: string;
    daterangesUrl: string;
    staticassetsDataUrl: string;
    staticassetsRawUrl: string;
    staticassetsUrl: string;
    customextensionEditUrl: string;
    customextensionUrl: string;
    rerunJobUrl: string;
    customCssRawUrl: string;
    customJsRawUrl: string;
    searchesUrl: string;
    possibleDeletedReportsUrl: string;
    datasetColumnsUrl: string;
    eventUrl: string;
    segmentUrl: string;
    filterassociationUrl: string;
    querylogsUrl: string;
    activeusersUrl: string;
    arcvizSettingsUrl: string;
    serverLogsUrl: string;
    bundledLogsUrl: string;
    appsDatasqlUrl: string;
    bulkUsersGroupsUrl: string;
    aviewListUrl: string;
    importUsageDashboardUrl: string;
    actionsJiraCreateUrl: string;
    actionsJiraDefaultsUrl: string;
    actionsJiraPropertiesUrl: string;
    actionsWebhookWhitelistUrl: string;
    actionsWebhookInvokeUrl: string;
    stopWorkUrl: string;
    remotecallsUrl: string;
    swaggerUrl: string;
  };

  authentication: {
    csrfToken: string;
    lastLoginDate: string;
    username: string;
    firstName: string;
    lastName: string;
    isSuperUser: boolean;
    isDefaultAuthenticationMode: boolean;
    authenticationMode: string;
    isAllowedToChangePassword: boolean;
    passwordValidation: PasswordValidation;
  };

  product: {
    name: string;
    fullVersion: string;
    trimmedVersion: string;
  };

  permissions: {
    system: string[];
  };

  settings: {
    dataConnectionConcurrencyDefault: number;
    dataConnectionConcurrencyMax: number;
    dataConnectionConcurrencyUserDefault: number;
    dataConnectionConcurrencyUserMax: number;
    dataConnectionQueryTimeoutDefault: number;
    dataConnectionQueryTimeoutMax: number;
    uploadMaxrowsDefault: number;
    uploadMaxrowsMax: number;
    shouldDisplayGetStarted: boolean;
    shouldDisplayWhatSNew: boolean;
    shouldDisplayDocumentation: boolean;
    shouldDisplaySupportWebPage: boolean;
    shouldDisplaySupportEmail: boolean;
    shouldDisplayVersion: boolean;
    isArcengineDataconnectionEnable: boolean;
    isLegacyImpalaEnabled: boolean;
    isLegacyHiveEnabled: boolean;

    whatSNewVersion: string;
    thumbnailPilSupported: boolean;
    emailEncryptionSupported: boolean;
    disableHomePageOverride: boolean;

    customLinks: (
      | { type: 'link'; label: string; link: string }
      | { type: 'divider' }
    )[];

    // app settings
    dashboardWidth: string;
    dashboardLoadingMessageInViewMode: boolean;
    emailEnabled: boolean;
    emailHostUser: string;
    emailPrefix: string;
    supportOption: string;

    supportWebPageUrl: string;
    supportWebPageLabel: string;

    defaultEmbeddingsEngineSettings: string;
    defaultCompletionEngineSettings: string;
    embeddingsBedrockCustomTransformers: string;
    completionBedrockCustomTransformers: string;
    aiEngine: AiEngine;
    aiCmlAuthMode: AiCmlAuthMode;
    embeddingProfiles: string;
    completionProfiles: string;
    completionExtraArg: string;
    completionTemperature: string;
    completionStream: boolean;
    embeddingsModel: string;
    completionModel: string;
    embeddingsBedrockModel: string;
    completionBedrockModel: string;
    aiTermAgreed: boolean;
    aiTermAgreedDate: string;
    embeddingInputParam: string;
    completionInputParam: string;
    embeddingResponseParam: string;
    completionResponseParam: string;
    defaultGroupMembershipForNewUsers: string;
    siteMaxTrellisDisplayed: string;
    visualMaxDataResults: string;
    visualMaxRows: string;
    maximumCanvasWidth: string;
    downloadMaxRows: string;
    siteThumbnailsIp: string;
    googleApiId: string;
    googleClientId: string;
    mapBoxAccessToken: string;
    localMapBoxServer: string;
    loginInstructions: string;
    emailUsername: string;
    emailHostPassword: string;
    emailHost: string;
    emailPort: string;
    emailProtocol: string;
    bulkMaxEntries: string;
    bulkMaxInterval: string;
    bulkMaxRetention: string;
    webhookWhitelist: string;
    remoteDataUrlWhitelist: string;
    advancedSettings: string;
    jiraServer: string;
    jiraUser: string;
    jiraPassword: string;
    jiraNewIssueDefaults: string;
    usageAnalyticsExpirationDays: string;
    aiApiKey: string;
    aiAuthKey: string;
    aiBedrockCredentials: string;
    aiServiceUrl: string;

    supportEmail: string;

    isSessionCookieSecure: boolean;
    sessionCookieSameSite: Cookies.CookieAttributes['sameSite'];

    cdvMode: 'cdw' | 'runtime' | 'cdsw' | 'cml';
    cdvSubMode: 'C1C' | 'MLExplore' | null;
    isCdvModeCml: boolean;

    isDataApiEnabled: boolean;
    isApiKeysEnabled: boolean;

    hasCustomEmailTemplate: boolean;
    embedUrlPrefix: string;

    enableVisualAction: boolean;
    enableJiraSettings: boolean;
    enableWebHook: boolean;
    enableAdvanceSettings: boolean;
    eventsTrackerFrontendDataToUsageDB: number;
  };

  documentation: {
    baseUrl: string;
    releaseNotesUrl: string;
    url: string;
    label: string;
  };

  canUserExploreAnyDataconnections: boolean;
  incompatibleBrowserWarningDismissed: boolean;
  siteSettingName: string;
  emailAddresses: string;

  isMobileView: boolean;
  cdwEnvironment: string;
};
