import { BehaviorSubject, Observable, Subject, Subscriber } from 'rxjs';
import type { VIZEvent } from './viz-event.type';

const events = new Subject<VIZEvent>();
const navigationBarEvents = new BehaviorSubject<{
  type: 'set-active-navigation-tab';
  payload: {
    activeTab: string | null;
    activeDropdownItem?: string;
  };
}>({
  type: 'set-active-navigation-tab',
  payload: {
    activeTab: null,
  },
});
const arcLoadedEvent = new BehaviorSubject(false);

export const ofType =
  (type: string) =>
  (source: Observable<VIZEvent>): Observable<VIZEvent> => {
    const subscribe = (subscriber: Subscriber<VIZEvent>) =>
      source.subscribe({
        next(value) {
          if (value.type === type) {
            subscriber.next(value);
          }
        },
        error(error) {
          subscriber.error(error);
        },
        complete() {
          subscriber.complete();
        },
      });

    return new Observable(subscribe);
  };

export const legacyBridge = {
  events,
  navigationBarEvents,
  arcLoadedEvent,
  ofType,
};

export type LegacyBridge = typeof legacyBridge;
