import { ReplaySubject } from 'rxjs';

const logs$ = new ReplaySubject<{
  msg: string;
  stackTrace: string;
}>();

const initialize = () => {
  const oldLog = console.error;

  console.error = function (...args) {
    const [msg] = args;
    const stackTrace = Error().stack || '';

    logs$.next({ msg, stackTrace });

    oldLog.apply(console, args);
  };
};

export const logger = {
  initialize,
  logs$,
};
