import axios, { AxiosRequestConfig } from 'axios';

import { exposeToGlobal } from 'utils/expose-to-global';

// Currently we are using axios to handle HTTP requests.
// To decouple our code base from a specific library we have viz-http module as a wrapper around axios.

// You can use it in the frontend folder by just simply importing vizHttp.
// It is also exposed to the global object (arc.frontend.vizHttp) if you want to use it in the arcweb/static/js folder.

// It has a very basic interface with only 4 methods (get, post, put and delete).
// Each method requires a config object which implements the HTTPRequestConfig interface and returns a promise containing an object with a data property.

// The HTTPRequstConfig and the return type of these methods can be extended as new requirements emerge.

export interface HTTPRequestConfig {
  url?: string;
  baseURL?: string;
  headers?: Record<string, string>;
  params?: Record<string, string>;
  data?: unknown;
  // $.ajax converts the request body into a query string
  // old endpoints are expecting a query string in the body and not a JSON string
  isOldEndpoint: boolean;
}

type RequestFunction = <ResponseData>(
  config: HTTPRequestConfig
) => Promise<{ data: ResponseData }>;

const client = axios.create({
  baseURL: '',
});

const createRequestFunction =
  (method: string): RequestFunction =>
  (config) => {
    const { isOldEndpoint, data, ...options } = config;

    return client({
      ...options,
      data: data && isOldEndpoint ? $.param(data) : data,
      method,
    } as AxiosRequestConfig).then((response) => ({ data: response.data }));
  };

export const vizHttp = {
  get: createRequestFunction('GET'),
  post: createRequestFunction('POST'),
  put: createRequestFunction('PUT'),
  delete: createRequestFunction('DELETE'),
  patch: createRequestFunction('PATCH'),
};

exposeToGlobal({ vizHttp }, 'frontend');
