import { create, StateCreator } from 'zustand';
import { devtools } from 'zustand/middleware';

type ModalState = {
  open: (payload: {
    connectionId: number;
    connectionMaxRowUpload: number | undefined;
    connectionType: string | undefined;
  }) => void;
  close: () => void;
  connectionId: number | undefined;
  connectionMaxRowUpload: number | undefined;
  connectionType: string | undefined;
};

const stateCreator: StateCreator<
  ModalState,
  [['zustand/devtools', never]],
  [],
  ModalState
> = (set) => ({
  connectionId: undefined,
  connectionMaxRowUpload: undefined,
  connectionType: undefined,

  open: ({ connectionId, connectionMaxRowUpload, connectionType }) => {
    return set(
      () => ({
        connectionId,
        connectionMaxRowUpload,
        connectionType,
      }),
      false,
      'open'
    );
  },

  close: () => {
    return set(
      () => ({
        connectionId: undefined,
        connectionMaxRowUpload: undefined,
        connectionType: undefined,
      }),
      false,
      'close'
    );
  },
});

export const useImportDataModalStore = create(
  devtools(stateCreator, { name: 'import-data-modal' })
);
