/* This is a map for arcweb/dataconn/connections_types.py.
If something changes there should be changed here as well and vica versa */

export type BaseDataConnectionType =
  | 'arcengine'
  | 'arcengine_v2'
  | 'aster'
  | 'athena'
  | 'bigquery'
  | 'cassandra'
  | 'denodo'
  | 'drill'
  | 'druid'
  | 'duckdb'
  | 'elasticsearch'
  | 'hyve'
  | 'impyla'
  | 'ksql'
  | 'mariadb'
  | 'mssqlserver'
  | 'mysql'
  | 'oracle'
  | 'postgres'
  | 'redshift'
  | 'snowflake'
  | 'solr'
  | 'sparksql'
  | 'sqlite'
  | 'teradata'
  | 'vertica'
  | 'hive'
  | 'impala'
  | 'trino'
  | 'phoenix';

export type DataConnectionType = BaseDataConnectionType | 'mongodb' | 'odbc';

export type DataConnectionExtendedName = 'cdw-hyve' | 'cdw-impyla' | 'ssb';

export type DataConnectionInfo = {
  extendedName?: DataConnectionExtendedName;
  PARAMS?: {
    CONNECTIONNAME_EXTENDED?: string;
    HOST?: string;
    PORT?: string;
    WAREHOUSE?: string;
    SOLRTYPE?: string;
    SOLR_JOINCHAR?: string;
    DATABASE?: string;
    USERNAME?: string;
    ROLE?: string;
    ACCOUNT?: string;
    FILE?: string;
    AUTHENTICATION?: string;
    DSN?: string;

    MODE?: string;
    HS2_HTTP_SQLPATH?: string;
    HS2_HTTP_ACCESS_TOKEN?: string;
    SOCK?: string;
    SESSION_TIMEOUT?: string | number;
    QUEUE_DEPTH?: string | number;
    SSLCERT_NAME_MISMATCH?: boolean;
    AUTH?: string;
    KERBSERV?: string;
    SOCKET_TIMEOUT?: string | number;
    IMPERSONATION?: boolean;
    IMPERSONATION_TRUSTED?: boolean;
    USER_DEPENDENT_CACHE?: boolean;
    APP_NAME?: string;
    FALLBACK?: string;

    SETTINGS?: Record<string, string>;

    CONCURRENCY?: string | number;
    CONCURRENCY_USER?: string | number;
    QUERY_TIMEOUT?: string | number;
    QUERY_LOADING_WARNING_SECONDS?: string | number;
    UPLOAD_MAXROWS?: number;
    CACHE?: {
      ENABLED?: number;
      RETENTION?: string | number;
    };

    ARCENGINE_HISTORY_ETL?: {
      ENABLED?: number;
      TARGET_TABLE?: string;
      JOB_NAME?: string;
      SCHEDULE?: string;
    };

    LOCATIONS?: Record<string, string>;
  };
};

export interface ConnectionBase {
  // Smaller subset of the legacy Dataconnection's info
  id: number;
  type: DataConnectionType;
}
export interface DataConnection extends ConnectionBase {
  name: string;
  typeName: string;

  info: DataConnectionInfo;

  cml?: {
    id: number | null;
    isAvailable: boolean;
  };

  isPasswordSet: boolean;
  supportsImportData: boolean;
}
