export const frontendDownloadCSV = (
  csvData: (string | boolean | number)[][],
  fileName: string
) => {
  const escapeValue = (value: string | boolean | number | null | undefined) => {
    if (value === null || value === undefined) return '';
    const stringValue = String(value);
    if (
      stringValue.includes(',') ||
      stringValue.includes('"') ||
      stringValue.includes('\n')
    ) {
      return `"${stringValue.replace(/"/g, '""')}"`;
    }
    return stringValue;
  };

  const csvString = csvData
    .map((row) => row.map(escapeValue).join(','))
    .join('\n');

  const blob = new Blob([csvString], { type: 'text/csv' });

  const link = document.createElement('a');
  link.href = URL.createObjectURL(blob);
  link.download = `${fileName}.csv`;

  link.click();

  URL.revokeObjectURL(link.href);
};
