import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { ReactQueryDevtools } from '@tanstack/react-query-devtools';
import React from 'react';
import { Root, createRoot } from 'react-dom/client';

const REACT_QUERY_CONFIG_OPTIONS = {
  defaultOptions: {
    queries: {
      retry: false,
      refetchOnWindowFocus: false,
    },
  },
};

const queryClient = new QueryClient(REACT_QUERY_CONFIG_OPTIONS);

export const renderReactComponent = <Props extends Record<string, unknown>>(
  containerId: string | undefined,
  ReactComponent: (props: Props) => JSX.Element,
  props: Props,
  existingRoot?: Root
): Root => {
  let root = existingRoot;

  if (containerId) {
    const container = document.getElementById(containerId);

    if (container) {
      root = createRoot(container);
    } else {
      throw new Error(`Container with ID: ${containerId} does not exist.`);
    }
  }

  root!.render(
    <QueryClientProvider client={queryClient}>
      <ReactComponent {...props} />
      <ReactQueryDevtools initialIsOpen={false} position="bottom-right" />
    </QueryClientProvider>
  );

  return root!;
};
