import { datasetEditor } from 'arc/dashboard/dataset-editor';
import {
  addLimitToSQLQuery,
  moveCursorToEnd,
  setCommandEnterShortcutOnExpressionEditor,
  openInNewWindow,
  howToSetWorkloadPasswordText,
  isBrowserInternetExplorer,
  copyToClipboard,
  escapeHTML,
  formattingQuery,
  getBlankTargetUrl,
  resetAlerts,
  activeAlertContainer,
  deactiveAlertContainer,
  handleAlertContainerActivationOnModal,
  directAccessTypes,
  determineDatabaseInQuery,
} from 'arc/misc/misc';
import {
  setupSpeechDetection,
  setupNavbarSearch,
} from 'arc/misc/speech-detection';
import { expression } from 'arc/filters/expression';
import { passwordValidators } from 'arc/utils/password-validators';
import { sessionKeyGenerator } from 'arc/utils/session-key-generator';

import { DatasetDeleteModalWrapper } from 'components/dataset-delete-modal-wrapper/DatasetDeleteModalWrapper';
import { DatasetActionsDropdown } from 'components/dataset-actions-dropdown/DatasetActionsDropdown';
import { DataConnectionModalWrapper } from 'components/data-connection-modal-wrapper/DataConnectionModalWrapper';
import { ImportDataModal } from 'components/import-data-modal/ImportDataModal';
import { IncompatibleBrowserAlertModal } from 'components/incompatible-browser-alert-modal/IncompatibleBrowserAlertModal';
import TemporaryPasswordField from 'components/temporary-password-field/TemporaryPasswordField';
import ChangeAiProfileModal from 'components/change-ai-profile-modal/ChangeAiProfileModal';
import { Chatbot } from 'components/chatbot/Chatbot';
import VersionControlPage from 'components/version-control-page/VersionControlPage';
import { ActivityLogs } from 'components/activity-logs/ActivityLogs';
import { PerfUsageTable } from 'components/perf-usage-table/PerfUsageTable';

import { submitUserDefinedHomePage } from 'pages/set-home-page/api/submit-user-defined-homepage';

import 'services/legacy-bridge';
import 'services/viz-cookie';
import 'services/viz-http';
import 'services/usage';

import { exposeToGlobal } from 'utils/expose-to-global';
import { getDataConnectionExtendedTypeName } from 'utils/get-data-connection-extended-type-name';
import { CreateDatasetFromTableModal } from 'components/create-dataset-from-table-modal/CreateDatasetFromTableModal';
import { CreateDatasetModal } from 'components/create-dataset-modal/CreateDatasetModal';
import { EditDatasetSqlModal } from 'components/edit-dataset-sql-modal/EditDatasetSqlModal';
import { Notifications } from 'components/notifications/Notifications';
import { DatasetProfileContainer } from 'components/data-profile-container/DataProfileContainer';
import { EmailJobModalWrapper } from 'components/email-job-modal/EmailJobModalWrapper';
import { AnalyticalJobModalWrapper } from 'components/analytical-job-modal/AnalyticalJobModalWrapper';

exposeToGlobal(
  {
    moveCursorToEnd,
    addLimitToSQLQuery,
    setCommandEnterShortcutOnExpressionEditor,
    openInNewWindow,
    setupSpeechDetection,
    setupNavbarSearch,
    howToSetWorkloadPasswordText,
    isBrowserInternetExplorer,
    copyToClipboard,
    escapeHTML,
    formattingQuery,
    getBlankTargetUrl,
    activeAlertContainer,
    deactiveAlertContainer,
    handleAlertContainerActivationOnModal,
    resetAlerts,
    sessionKeyGenerator,
    directAccessTypes,
    determineDatabaseInQuery,
  },
  'misc'
);

exposeToGlobal(
  {
    expression,
  },
  'filter'
);

exposeToGlobal(
  {
    editor: datasetEditor,
  },

  'dataset'
);

exposeToGlobal(
  {
    passwordValidators,
  },
  'authentication'
);

exposeToGlobal(
  {
    DatasetDeleteModalWrapper,
    DatasetActionsDropdown,
    DataConnectionModalWrapper,
    ImportDataModal,
    IncompatibleBrowserAlertModal,
    TemporaryPasswordField,
    ChangeAiProfileModal,
    submitUserDefinedHomePage,
    getDataConnectionExtendedTypeName,
    Chatbot,
    CreateDatasetFromTableModal,
    CreateDatasetModal,
    EditDatasetSqlModal,
    Notifications,
    VersionControlPage,
    ActivityLogs,
    PerfUsageTable,
    DatasetProfileContainer,
    EmailJobModalWrapper,
    AnalyticalJobModalWrapper,
  },
  'frontend'
);
