#!/usr/bin/python -tt
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import os
from util.init_cml import check_cml_syncing_and_init_groups
from apps.config import default_support_email

def initializeViz():
  import uuid
  from django.contrib.auth.models import User
  from reports.views_settings import setting_get_single, \
      setting_update_single, register, is_registered
  from instant.instant import createSamples
  from jobs.views_encryption import encrypt_secret

  # If this is not a first-time startup, exit early. No initialization required.
  if is_registered():
    return

  VIZ_PORT = os.environ.get("VIZ_PORT", "38888")
  CDV_MODE = os.environ.get("CDV_MODE")
  if CDV_MODE in ["cml", "cdsw", "runtime"]:
    setting_update_single('enable_remote_user_auth', 'true', True)

  setting_update_single('site_enable_custom_styles', 'true', True)
  thumbnails_protocol = 'https' if os.getenv('SSL_ENABLED', 'false') == 'true' else 'http'
  setting_update_single('site_thumbnails_ip', f'{thumbnails_protocol}://127.0.0.1:{VIZ_PORT}', True)
  setting_update_single('site_thumbnails_enable', 'true', False)
  register("auto", "auto", "auto", "auto", "auto", str(uuid.uuid4()))

  ldapconfigfile = os.environ.get("LDAP_CONFIG_FILENAME", "/etc/viz/conf/ldap.ini")
  if os.path.isfile(ldapconfigfile):
    # DWX environment - tweak some features
    setting_update_single('site_enable_download_from_url', 'false', False)

  u = User.objects.filter(username='vizapps_admin')
  if not u or len(u) == 0:
    # Is there an admin user already? (Older installations will have this)
    u = User.objects.filter(username='admin')
    if not u or len(u) == 0:
      # No admin or vizapps_admin user - create vizapps_admin
      User.objects.create_superuser('vizapps_admin', default_support_email, 'vizapps_admin')

  setting_update_single('viz_initialized', 'true', True)

  # Note that we are creating samples after we set viz_initialized to true
  # This means we tolerate failures in creating samples, and some instances may
  # come up without samples
  createSamples()

  return True

is_initial_run = initializeViz()

check_cml_syncing_and_init_groups(is_initial_run)
