#!/usr/bin/env bash

set -x
set -e

DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"

VIZ_PORT=${VIZ_PORT:-${CDSW_APP_PORT}}
VIZ_HOST=${VIZ_HOST:-0.0.0.0}

# SETUP CDSW/CML EXPLORE
if [[ "${CDV_SUB_MODE}" == "MLExplore" ]]; then
  export BASEDIR='/home/cdsw/.explore'
fi

export BASEDIR=${BASEDIR:-"/var/lib/arcadia"}
PYTHONPATH=${PYTHONPATH}:${BASEDIR}

mkdir -p ${BASEDIR}
cd ${BASEDIR}

cp ${DIR}/settings_base.py ${BASEDIR}/settings_local.py
echo "CACHE_PER_PROCESS_KEY=`date +%s`" >> ${BASEDIR}/settings_local.py

if [[ "${CLEANUP_METADATA}" == "true" ]]; then
  # Clean up foreign key violations from Django 2.2. upgrade
  ${DIR}/arcviz util foreign_key_cleanup
fi

echo "Migrating"
${DIR}/arcviz migrate
echo "Migrating Usage Database"
${DIR}/arcviz migrate --database=usage

# Read custom settings (set via Site Settings/Advanced Settings) and
# place them in settings_local.py unless a NO_CUSTOM_SETTINGS environment
# variable is set, it is also disabled in the CDW environment, to avoid
# potential security issues

if [[ -z "${NO_CUSTOM_SETTINGS}" && "${CDV_MODE}" != "cdw" ]]; then
  ${DIR}/arcviz shell < ${DIR}/readsettings.py
fi

echo "Init VIZ"
${DIR}/arcviz shell < ${DIR}/initviz.py
echo "Init complete"

echo "Init before checking multiproc settings"

# For any form factor in MULIPROC_CDV_MODE_SUPPORT, proceed with multiproc setup
MULIPROC_CDV_MODE_SUPPORT="cdsw,cml,runtime,cdw"
function cdv_mode_supported() {
    LIST=$1
    VALUE=$2
    [[ "$LIST" =~ (,|^)$VALUE(,|$) ]]
}
if cdv_mode_supported "$MULIPROC_CDV_MODE_SUPPORT" "${CDV_MODE}" && [[ "${USE_MULTIPROC}" != "false" ]]; then

  if [[ "${CDV_MODE}" == "cdw" ]]; then
    # For cdw deployments, assume an arcviz process limit of 3.
    #  However, in the future we should check if the environment variable is set and allow the custom
    #  MULTIPROC_WORKER_NUM setting to take effect.
    if [[ -z "${MULTIPROC_WORKER_NUM}" ]]; then
      export MULTIPROC_WORKER_NUM=1
    fi
    # CDW form factor uses VIRTUAL_ENV to declare the virtual environment root location
    VENV_BASE_PATH=${VIRTUAL_ENV}
    # Get base path to arcweb library
    ARCWEB_PATH="$(python -c 'import arcweb as _; print(_.__path__[0])')"
  else
    # CDSW/CML form factors use VENV_PATH to declare the virtual environment root location
    VENV_BASE_PATH=${VENV_PATH}
    # Use PYTHONPATH to get base path to arcweb library
    ARCWEB_PATH=${ARCWEB_PATH}
  fi

   # If multiproc number not configured, assume an arcviz process limit of 1.
  if [[ -z "${MULTIPROC_WORKER_NUM}" ]]; then
    export MULTIPROC_WORKER_NUM=1
  fi

  # Set root static file path to base of arcweb library
  export ROOT_STATIC_FILE_PATH=$ARCWEB_PATH

  echo "Operating in multi viz process mode."
  echo "Generate multiproc config"
  VENV_BASE_PATH=$VENV_BASE_PATH ${DIR}/gen_multiproc_conf.sh
  echo "Starting supervisord"
  VIZ_MULTIPROC_CONF=${BASEDIR}/multiprocess
  exec $VENV_BASE_PATH/bin/supervisord -c $VIZ_MULTIPROC_CONF/supervisord.conf -e info -l ${BASEDIR}/supervisord.log
else
  HTTP_THREADS=${HTTP_THREADS:-10}
  SSL_OPTS=""
  if [[ "${SSL_ENABLED}" == "true" ]]; then
    SSL_OPTS="--ssl_private_key=${SSL_PRIVATE_KEY} --ssl_private_key_pass=${SSL_PRIVATE_KEY_PASSWORD} --ssl_certificate=${SSL_SERVER_CERTIFICATE} --ssl_certificate_chain=${SSL_SERVER_CERTIFICATE}"
  fi
  echo "Operating in default single viz process mode."
  exec ${DIR}/arcviz runcpserver --host=${VIZ_HOST} --port=${VIZ_WEBSERVER_PORT:-${VIZ_PORT}} --minthreads=$HTTP_THREADS --maxthreads=$HTTP_THREADS $SSL_OPTS
fi



