# this script expects:
#   - PKG_VERSION environment variable to be set
#   - the VIZ git repo mounted at /viz
#   - GBN content at /viz/packaging/build/parcel
# will produce:
#   - parcel manifest in /viz/packaging/build/parcel/parcels
#   - build.json in /viz/packaging/build/parcel
#   - create and populate a GBN (upload everything from /viz/packaging/build)

set -x
set -e

# important inside RHEL UBI
unset ENV BASH_ENV PROMPT_COMMAND

# fix "for detected dubious ownership in repository at '/viz'"
git config --global --add safe.directory /viz

# explicitly activate the Python env:
source /opt/app-root/bin/activate

pip install -i https://nexus-private.infra.cloudera.com/nexus/repository/pypi-all/simple/ cdh-cauldron

# Obtain the creds to get s3creds.
wget http://github.mtv.cloudera.com/QE/deploy/raw/master/cdep/data/id_rsa -O /tmp/id_rsa_systest
chmod 600 /tmp/id_rsa_systest

# Get the S3 credentials. Needed for pushing final artifacts to the cloud.
ssh -o StrictHostKeyChecking=no -i /tmp/id_rsa_systest s3@cloudcat-s3.infra.cloudera.com build  > /tmp/.s3-auth-file

# Retrieve a GBN from the api
GBN=$(curl http://gbn.infra.cloudera.com/)
if [ -z "$GBN" ];then
   echo "Could not get GBN for the build"
   exit 1
fi

echo "GBN=$GBN"

cd /viz/packaging/build/parcel

# Active virtualenv with cauldron installed is assumed
parcelmanifest parcels/

# Used by buildjson. From the code it looks like it is used as a timeframe on when to purge the buckets.
EXPIRE_DAYS=365
EXPIRATION=$(date --date "+${EXPIRE_DAYS} days" +%Y%m%d-%H%M%S)
BUILD_JSON_EXIPIRATION="--expiry  ${EXPIRATION}"

PRODUCT_NAME=DATAVIZ
# strip the part under the underscore
VERSION=`echo ${PKG_VERSION} | sed -nr 's/^([^_]+).*$/\1/p'`
BUILD_URL=`hostname -f`
user=${USER:-vizbuild}

buildjson -o build.json -p ${PRODUCT_NAME}  --version ${VERSION}  --gbn  $GBN \
  -os redhat8 \
  --build-environment $BUILD_URL  ${BUILD_JSON_EXIPIRATION}  --user ${user} \
  add_parcels --product-parcels ${PRODUCT_NAME} parcels \
  add_source --repo .

upload s3 --auth-file /tmp/.s3-auth-file --create-html-listing $GBN --base build/${GBN} .:.

curl -v http://builddb.infra.cloudera.com/save?gbn=$GBN