# - WHEEL_VERSION, PKG_VERSION must be defined
# - must be executed in a Git work tree
set -x
set -e

REPO=$(git rev-parse --show-toplevel)

if [[ "$GBN" == "" ]]; then
  # no GBN:
  PARCEL_VERSION="$PKG_VERSION"
else
  # remove eventual git hash from version (the part after the underscore)
  PARCEL_VERSION=`echo ${PKG_VERSION} | sed -nr 's/^([^_]+).*$/\1/p'`
  # include the GBN in the version
  PARCEL_VERSION="$PARCEL_VERSION.p1.$GBN"
fi

docker run \
  --name redhat8_parcel_builder \
  --entrypoint=/bin/bash \
  -e WHEEL_VERSION=${WHEEL_VERSION} \
  -e PARCEL_VERSION=${PARCEL_VERSION} \
  -v ${REPO}:/viz docker-sandbox.infra.cloudera.com/ckoncz/redhat8/python:3.9.19 \
  /viz/packaging/build/build_parcel_redhat8.sh

# parcel should show up under $REPO/packaging/build/parcel/parcels

# build CSD and Knox service descriptor:

CSD_VERSION=${CSD_VERSION:-${PARCEL_VERSION}}
KNOX_SERVICE_DEFINITION_VERSION=${KNOX_SERVICE_DEFINITION_VERSION:-${PARCEL_VERSION}}

mkdir -p $REPO/packaging/build/parcel/csd
mkdir -p cdh/csd/images
cp arcweb/static/img/favicon.png cdh/csd/images/icon.png
jar -vcf $REPO/packaging/build/parcel/csd/DATAVIZ-${CSD_VERSION}.jar  -C cdh/csd/ .

mkdir -p $REPO/packaging/build/parcel/knox
tar -vczf $REPO/packaging/build/parcel/knox/DATAVIZ-KNOX-${KNOX_SERVICE_DEFINITION_VERSION}.tgz  -C cdh/knox/ dataviz

echo "list generated content"
find $REPO/packaging/build/parcel

echo "GBN=$GBN"

if [[ "$GBN" == "" ]]; then
  echo "GBN is unset, will generate one and will publish artifacts there."
  docker run \
    --name parcel_uploader \
    --entrypoint=/bin/bash \
    -e PKG_VERSION=${PKG_VERSION} \
    -v ${REPO}:/viz docker-sandbox.infra.cloudera.com/ckoncz/redhat8/python:3.9.19 \
    /viz/packaging/build/build_gbn.sh
else
  echo "Existing GBN=$GBN, not attempting artifact publishing."
  echo "Existing GBN=$GBN, generating parcel manifest."
  docker run \
    --name parcel_manifest_generator \
    --entrypoint=/bin/bash \
    -v ${REPO}:/viz docker-sandbox.infra.cloudera.com/ckoncz/redhat8/python:3.9.19 \
    /viz/packaging/build/build_parcels_manifest.sh
fi
