# this script expects:
#   - WHEEL_VERSION, PARCEL_VERSION environment variables to be set
#   - PARCEL_TYPE variable set if other than el8
#   - the VIZ git repo mounted at /viz
#   - the built VIZ wheel available as /viz/packaging/assets/arcweb-${WHEEL_VERSION}-py3-none-any.whl
# will produce:
#   - a parcel in /viz/packaging/build/parcel/parcels
set -x
set -e

PARCEL_TYPE=${PARCEL_TYPE:-el8}
PARCEL_NAME=DATAVIZ-${PARCEL_VERSION}
PARCEL_FILE=${PARCEL_NAME}-${PARCEL_TYPE}.parcel
PARCEL_ROOT=/tmp/build/${PARCEL_NAME}


# important inside RHEL UBI
unset ENV BASH_ENV PROMPT_COMMAND

# we plan to use our on Python build:
export LD_LIBRARY_PATH=/tmp/opt/python/lib

mkdir -p ${PARCEL_ROOT}/dataviz
cd ${PARCEL_ROOT}/dataviz

# 1. create and populate virtual environment
/tmp/opt/python/bin/python3 -m venv --copies venv
source venv/bin/activate

# need to pin setuptools, otherwise we get errors with third party package install
pip install setuptools==70.0.0
pip install -r /viz/packaging/assets/requirements.txt
pip install /viz/packaging/assets/arcweb-${WHEEL_VERSION}-py3-none-any.whl
cp /viz/packaging/assets/TSocket.py venv/lib/python3.9/site-packages/thrift/transport/TSocket.py

# 2. add prebuilt SQLite and Python:
cp -a /tmp/opt/sqlite .
cp -a /tmp/opt/python .

# 3. add startup folder as "tools/arcviz"
mkdir tools
cp -a /viz/packaging/assets/startup tools/arcviz

# 4. add parcel metadata and dataviz/bin
cp -a /viz/cdh/parcel/DATAVIZ/* ${PARCEL_ROOT}

# 5. generate parcel json:
python $PARCEL_ROOT/meta/parcel_json.py --var version=${PARCEL_VERSION} --output-file ${PARCEL_ROOT}/meta/parcel.json

# 6. build archive
cd ${PARCEL_ROOT}/..

tar czf ${PARCEL_FILE} ${PARCEL_NAME}

mkdir -p /viz/packaging/build/parcel/parcels
mv ${PARCEL_FILE} /viz/packaging/build/parcel/parcels