#!/usr/bin/env python
# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.

import sys
import json
import os

def main():
  fname = sys.argv[1]
  if not fname:
    raise Exception("no filename provided")

  # Some of this information needs to be duplicated and used in packaging/assets/Dockerfile_runtimedataviz
  PKG_VERSION = os.environ.get('PKG_VERSION')
  LAST_COMMIT_ID = os.environ.get('LAST_COMMIT_ID')
  GBN = os.environ.get('GBN')
  MAINTENANCE_VERSION = 1
  short_pkg_version = PKG_VERSION.split('-')[0]
  with open(fname, "w") as f:
    f.write(json.dumps({
      "assembly_metadata_version": 1,
      "runtimes": [
        {
          "image_identifier": f"docker.repository.cloudera.com/cloudera/cdv/runtimedataviz:{PKG_VERSION}",
          "runtime_metadata_version": 2,
          "editor": "PBJ Workbench",
          "kernel": "Cloudera Data Visualization",
          "description": "Visualization, natural language search and app building application provided by Cloudera",
          "edition": "Data Visualization",
          "full_version": f"{short_pkg_version}.{MAINTENANCE_VERSION}",
          "short_version": short_pkg_version,
          "maintenance_version": MAINTENANCE_VERSION,
          "git_hash": LAST_COMMIT_ID,
          "gbn": int(GBN) if GBN else GBN
        },
      ],
    }, indent=4))
    f.write('\n')

  os.system(f'cat {fname}')

if __name__ == "__main__":
  main()

