set -x
set -e
# optional environment variable
# DJANGO_TESTS_MIGRATIONS can be the python test modules to run. Like 'datasets.migrationtests'

cd arcweb
# METADATA_STORE=sqlite:
DJANGO_SETTINGS_MODULE=arcwebbase.settings_tests coverage run --branch --append manage.py test --noinput --pattern='migrationtests*.py' $DJANGO_TESTS_MIGRATIONS
# check if docker exists

if docker --version; then
  DOCKER_EXISTS=1
else
  echo "Docker is not avilable. Exiting"
  exit 1
fi

docker stop pytest-mysql pytest-postgres || true
docker rm pytest-mysql pytest-postgres || true

# add the following environment setting to persist database contents:
# -v ~/pytest-mysql-data:/var/lib/mysql
docker run --name pytest-mysql \
  -d \
  -e MYSQL_ROOT_PASSWORD=cloudera \
  -e MYSQL_DATABASE=pytest_db \
  -e MYSQL_USER=pytest_user \
  -e MYSQL_PASSWORD=cloudera \
  -p 23306:3306 \
  docker-private.infra.cloudera.com/cloudera_thirdparty/mysql:8.0.36

# add the following environment setting to persist database contents:
# -v ~/pytest-postgres-data:/var/lib/postgresql/data
docker run --name pytest-postgres \
  -d \
  -e PGDATA=/var/lib/postgresql/data/pgdata \
  -e POSTGRES_DB=pytest_db \
  -e POSTGRES_USER=pytest_user \
  -e POSTGRES_PASSWORD=cloudera \
  -p 25432:5432 \
  docker-private.infra.cloudera.com/cloudera_thirdparty/postgres:15-alpine

# need to wait a bit until the containers start up
sleep 5

METADATA_STORE=postgresql METASTORE_DB_NAME=pytest_db METASTORE_DB_USER=pytest_user METASTORE_DB_PASSWORD=cloudera METASTORE_DB_HOST=localhost METASTORE_DB_PORT=25432 \
  DJANGO_SETTINGS_MODULE=arcwebbase.settings_tests coverage run --branch --append manage.py test --noinput --pattern='migrationtests*.py' $DJANGO_TESTS_MIGRATIONS

METADATA_STORE=mysql METASTORE_DB_NAME=pytest_db METASTORE_DB_USER=pytest_user METASTORE_DB_PASSWORD=cloudera METASTORE_DB_HOST=localhost METASTORE_DB_PORT=23306 \
  DJANGO_SETTINGS_MODULE=arcwebbase.settings_tests coverage run --branch --append manage.py test --noinput --pattern='migrationtests*.py' $DJANGO_TESTS_MIGRATIONS

# create coverage report
coverage html

docker stop pytest-mysql pytest-postgres
docker rm pytest-mysql pytest-postgres
