# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
from shell import run_cmd_pipe, touch_targets
import os
import sys

DEFAULT_ARGS = [
  '--renew-anon-volumes',
  '--force-recreate',
  '--remove-orphans',
  '--abort-on-container-exit',
]

def run_compose(compose_path, env_vars):
  def compose_builder(target, source, env):
    docker_compose(compose_path, env_vars)
    touch_targets(target)
  return compose_builder

def docker_compose(compose_path, env={}, compose_args=[], append_args=True):
  env = {k: str(v) for k, v in env.items()}
  os.environ.update(env)
  args = ' '.join(compose_args + DEFAULT_ARGS if append_args else compose_args)
  command = f'docker-compose -f - up {args}'
  print(f'Running docker compose: cat {compose_path} | {command}')
  sys.stdout.write(
    run_cmd_pipe(f'cat {compose_path}', command)
  )
