# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import os
import getpass

# We generate the following scons targets below:
#
# docker_push_cdwdataviz_registry
# docker_push_cdwdataviz_dev
#
# docker_push_cmldataviz_registry
# docker_push_cmldataviz_dev
#
# docker_push_cdswdataviz_registry
# docker_push_cdswdataviz_dev
#
# docker_push_runtimedataviz_registry
# docker_push_runtimedataviz_dev
#
# Note that to run these pushes, you need to be logged in to the appropriate docker repos
# or you need to run something like:
# DOCKER_CONFIG_DIR=/path/to/docker/config/dir scons docker_push_cdswdataviz_prod
#
# Also see packaging/cdwdataviz/README for a dev workflow for CDW
#

username = os.getenv('CLOUDERA_USERNAME', getpass.getuser())

docker_repository_paths = {
  'registry': 'docker-private.infra.cloudera.com/cloudera',
  'dev': 'docker-sandbox.infra.cloudera.com/' + username
}

def generate_docker_push_targets(env, PKG_VERSION):
  for image_name in [ 'cmldataviz', 'cdswdataviz', 'runtimedataviz', 'cdwdataviz' ]:
    for docker_repo in ['registry', 'dev']:
      docker_host = docker_repository_paths[docker_repo]
      target_image = f'{docker_host}/cdv/{image_name}'

      docker_config_dir = os.environ.get("DOCKER_CONFIG_DIR", '')

      alias_name = f'docker_push_{image_name}_{docker_repo}'

      env.AlwaysBuild(
        env.Alias(
          alias_name,
          [image_name],
          [
            f'docker tag cdv/{image_name}:{PKG_VERSION} {target_image}:{PKG_VERSION}',
            f'docker {docker_config_dir} push {target_image}:{PKG_VERSION}',
          ]
        )
      )
