# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#  
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#  
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import json
import os
from shell import touch_targets
from docker_compose import docker_compose

DEFAULT_VIZTEST = '../viztest'

def run_cypress_compose(compose_path, env_vars):
  def cypress_compose_builder(target, source, env):
    env_vars['VIZ_DIR'] = os.environ.get('VIZ_DIR') or env.Dir('#').abspath
    env_vars['VIZTEST_DIR'] = os.environ.get('VIZTEST_DIR') or DEFAULT_VIZTEST
    env_vars['CYPRESS_VERSION'] = get_cypress_version()
    docker_compose(compose_path, env_vars, ['--exit-code-from viztest'])
    touch_targets(target)
  return cypress_compose_builder

def get_cypress_version():
  VIZTEST_DIR = os.environ.get('VIZTEST_DIR') or DEFAULT_VIZTEST
  IS_IN_DOCKER = os.environ.get('IS_DOCKER_IN_DOCKER') == 'TRUE'
  # When running with `./dev/dockerrun` the `VIZTEST_DIR` variable references
  # the original volume path to make sure the other docker containers
  # attach the same volume, so in that case we can't use that path.
  PACKAGE_JSON_PATH = (DEFAULT_VIZTEST if IS_IN_DOCKER else VIZTEST_DIR) + \
    '/cypress/package.json'
  with open(PACKAGE_JSON_PATH, 'r') as p:
    return json.loads(p.read())['dependencies']['cypress'][1:]


