# Copyright (c) 2013 - 2025 Cloudera, Inc.  All Rights Reserved.
#
# Unless otherwise agreed to in writing by Cloudera, use of this file is exclusively governed by the Cloudera Standard License (the “License”), a copy of which may be found at https://www.cloudera.com/legal/commercial-terms-and-conditions.html.
#
# This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. Refer to the License for the specific permissions and limitations governing your use of the file.
import re
from shell import run_cmd

VERSION_RE = re.compile('(release_|v|v\.)(\d+\.\d+\.\d+(\.\d+)?)')

def version_tags():
  commit = run_cmd('git log --abbrev-commit --pretty=oneline -1').split()[0]
  branch = run_cmd('git rev-parse --abbrev-ref HEAD').strip()

  def release_version(name):
    m = VERSION_RE.search(name)
    if m:
      return m.groups()[1]
    return None

  version = release_version(branch)
  if version is None:
    existing_remotes = run_cmd('git remote').split()
    if 'author' not in existing_remotes:
      # Add the 'author' remote only if it doesn't exist (for canary checks)
      run_cmd('git remote add author https://github.infra.cloudera.com/Sense/viz')

    # Fetch the tags from the 'author' remote
    run_cmd('git fetch author --tags')

    # Only get tags that are in the current branch
    tags = run_cmd('git tag --merged | sort -r').split()

    versions = [release_version(t) for t in tags]
    versions = [v for v in versions if v is not None]

    versions.sort(reverse=True)
    if len(versions) > 0:
      version = versions[0]

  if version is None:
    raise Exception('Cannot find a version to build')

  # Get the SHA indicative of the last commit and put it in the version file
  # This version file may be read through a URL to figure out which version of
  # the package are running on a site
  GITVERSION = run_cmd('git log --date=raw --pretty=format:"%ad" -n 1')[:-6]
  GITVERSION = GITVERSION.strip()
  LAST_COMMIT_ID = run_cmd('git log -n 1 --pretty="%H"').strip()
  LAST_TS_HUMAN_READABLE = run_cmd('git log -n 1 --pretty="%ad"').strip()
  GITTAGF = f'v{version}-{run_cmd("git describe --always").strip()}'

  GITTAG = version

  PKG_VERSION = f'{version}_{commit}'

  v = {
    'WHEEL_VERSION': f'{version}+{commit.lstrip("0")}',
    'PKG_VERSION': PKG_VERSION,
    'GITVERSION': GITVERSION,
    'LAST_COMMIT_ID': LAST_COMMIT_ID,
    'LAST_TS_HUMAN_READABLE': LAST_TS_HUMAN_READABLE,
    'GITTAGF': GITTAGF,
    'GITTAG': GITTAG,
  }

  return v

