/* eslint-disable no-continue */
window.arc = window.arc || {};

(function() {
'use strict';

const MINUTE = 60,
      HOUR = 3600,
      DATE = 24 * HOUR,
      YEARWEEK = 7 * DATE,
      YEARMONTH = 30 * DATE,
      YEARQUARTER = 3 * YEARMONTH,
      YEAR = 365 * DATE;

const unitNameToSeconds = {
  SECOND: 1,
  MINUTE: MINUTE,
  HOUR: HOUR,
  DATE: DATE,
  YEARWEEK: YEARWEEK,
  YEARMONTH: YEARMONTH,
  YEARQUARTER: YEARQUARTER,
  YEAR: YEAR,
};

arc.report = arc.report || {};

arc.report.segment_name = 'arc_segment';
arc.report.segment_display_name = 'Segment';

arc.report.filters_shelf_html = function() {
  return $('#filters-save-as-segments-html').html();
};

arc.report.y_shelf_html = function() {
  return '<div class="shelves-pivot-wrapper">' +
           '<span class="shelves-pivot" data-toggle="tooltip" title="switch X and Y"></span>' +
         '<div>';
};

function isFilterOrTimeControl(visual_type) {
  return (arc.misc.isFilterVisual(visual_type) || visual_type === 'timecontrol');
}

arc.report.isAppScopeFilter = function(vizLib) {
  if (!vizLib || !vizLib.viz() || !isFilterOrTimeControl(vizLib.viz_type())) {
    return false;
  }

  const s = vizLib.user_settings().find_by_id('filter_appgroup_scope');
  if (!s) {
    return false;
  }

  return (s.value() === 'true');
};

arc.report.in_dashboard_grid = function (chart_div) {
  return $('#' + chart_div).closest('.gs-w').length > 0;
};

arc.report.base_unique_report_id = 1;

arc.queryStateLogging = false;

// consider adding the new filter to arc.misc.isFilterVisual too if it's filter
arc.report.noResultExcludeTypes = new Set([
  'link',
  'extension',
  'html',
  'column-stats',
  'leaflet',
  'picklist',
  'daterange',
  'range',
  'timecontrol',
  'search',
  'action',
]);

arc.report.base = function() {
  var base = function() { return 'arc.report.base'; };

  var derivedData = arc.report.dataset_deriveddata(),
      loader,
      enableLoader = true,
      dataset,

      inModal,
      in_dashboard,
      dashboard_click_behavior,
      shelfTrackerInstance,

      active = true,

      trellis,
      sql_query,
      dsreq,
      title_elem,
      subtitle_elem,
      chart_div,
      limit_spinner,
      limit_value,
      limit_default = 100,
      json_data,
      sb_colnames,
      sb_coltypes,
      sb_columns,
      sb_rows,
      dialog_selection = null,
      chart_type,
      params = arc.params.tracker(),
      prevParams = arc.params.tracker(),
      shelves,
      click_behaviors = {},
      click_behavior_cb = null,
      click_behavior_change_cb,
      redraw_cb,
      save_segment_cb,
      dataset_segments,
      dataset_events,
      dashboard,
      unique_report_id,
      report_id,
      last_run_sql,
      last_good_sql,
      last_good_dsreq,
      last_good_sample,
      last_good_results,
      last_good_tablesig,
      title = '',
      subtitle = '',
      afterTitleCb,
      build_mode,
      shelves_dom_id = 'app-shelves',
      needs_events = false,
      sample_pct = 'Off',
      filtersaver,
      marks = [],
      markActivationEnabled = true,
      contextmenus = [],
      currentAppliedParams = {},
      editAlertMessages = [],
      viewAlertMessages = [],
      localEnableAnimations = true,

      // contrast colors used when viz_lib.chart_type_graded() === 0 (combined visual)
      // graded colors used when viz_lib.chart_type_graded() is true (chm, crosstab, etc)
      // colors used by default (line, area, bar, etc)
      colorSettings = {},
      colors = arc.report.base_colors(),
      nodeColors = arc.report.base_colors(),
      linkColors = arc.report.base_colors(),
      graded_colors = arc.report.graded_colors(),
      contrast_colors = arc.report.contrast_colors(),
      reverse_colors = false,
      chart_type_graded = false,
      colorScale = arc.misc.dimensionColorScale(),
      nodeColorScale = arc.misc.dimensionColorScale(),
      linkColorScale = arc.misc.dimensionColorScale(),
      xaxis_label_text = null,
      yaxis_label_text = null,
      xaxis_label_maxlen = 20,
      yaxis_label_maxlen = 20,
      dim_tick_label_maxlen = 20,
      main_axis_label_maxlen = 100,
      highlight_color = '#4169e1',
      settings_modal,
      user_settings = arc.visual.defaultSettings({ noTrellis: true }),

      x_slant = -40,
      viz,
      viz_type,
      unhighlight_cb,

      enable_auto_page_save_handling = true,

      needs_save = false,

      title_visible = true,
      refresh_visible = true,

      redraw_on_settings_cancel = false,

      enableFilterWarnings = true,

      baseQuery,
      fullyExpandedDsreq, // for downloading csv of expansion-enabled crosstab
      activeQueryManager = arc.data.activeQueryManager(),

      visualSupportsViewData = true,

      // In edit mode, if visual title and subtitle are empty, this variable controls
      // whether we still show the title and subtitle
      showEmptyTitleSubtitle,

      // Track display formatters used for shelf elements
      sqlIndexToFormatter = {},
      solr_query;

  // Threshold of number of values at which to switch to server download
  const DOWNLOAD_BROWSER_THRESHOLD = 50000;
  const legendItemBottomMargin = 5;

  const $superModal = $('.super-modal');

  unique_report_id = arc.report.base_unique_report_id++;

  arc.report.base_async_sql_serializer = arc.report.base_async_sql_serializer ||
      arc.report_xhr_serializer_parallel();

  base.showEmptyTitleSubtitle = function(_) {
    if (!arguments.length) return showEmptyTitleSubtitle;
    showEmptyTitleSubtitle = _;
    return base;
  };

  base.active = function(_) {
    if (!arguments.length) return active;
    active = _;
    return base;
  };

  base.visualSupportsViewData = function(_) {
    if (!arguments.length) return visualSupportsViewData;
    visualSupportsViewData = _;
    return base;
  };

  base.enableFilterWarnings = function(_) {
    if (!arguments.length) return enableFilterWarnings;
    enableFilterWarnings = _;
    return base;
  };

  base.trellis = function(_) {
    if (!arguments.length) return trellis;

    // Cleanup any older trellis
    if (trellis) {
      trellis.destroy();
    }

    trellis = _;
    return base;
  };

  base.colorScale = function(_) {
    if (!arguments.length) return colorScale;
    colorScale = _;
    return base;
  };

  base.nodeColorScale = function(_) {
    if (!arguments.length) return nodeColorScale;
    nodeColorScale = _;
    return base;
  };

  base.linkColorScale = function(_) {
    if (!arguments.length) return linkColorScale;
    linkColorScale = _;
    return base;
  };

  base.clearAppliedParams = function () {
    currentAppliedParams = {};
  };

  base.shelfTrackerInstance = function(_) {
    if (!arguments.length) {
      if (!shelfTrackerInstance) {
        shelfTrackerInstance = arc.report.shelftracker();
      }
      return shelfTrackerInstance;
    }

    shelfTrackerInstance = _;
    return base;
  };

  base.enableViewData = function() {
    if (!visualSupportsViewData) {
      return false;
    }

    // View-data is enabled within search results
    if ($(`#${base.chart_div()}`).closest('.search-result').length > 0) {
      return true;
    }

    if (!dashboard) {
      return false;
    }

    const s = dashboard.user_settings().find_by_id('dashboard_display_view_data');
    if (s) {
      return s.isChecked;
    }

    return false;
  };

  base.isEmptyTable = function() {
    if (viz_type === 'table') {
      let shelfElements = base.shelfElementCounts();
      if (shelfElements === 0) {
        return true;
      }
    }

    return false;
  };

  base.showContextMenu = function() {
    return !autotimeColumnExists();
  };

  // Don't pass dsreq when doing server side request generation and
  // user has only app view on the dataset.
  base.skip_dsreq = function() {
    let skip_dsreq = false;
    if (!arc.globals.arcviz_settings.inclds) {
      if (base.report() && base.report().dataset()) {
        const perms = base.report().dataset().dataset_perms();
        skip_dsreq = (perms.length === 1 && perms[0] === 'ds_appview');
      }
    }
    return skip_dsreq;
  };

  base.enableAnimations = function() {
    if (!localEnableAnimations) {
      return false;
    }

    let s = user_settings.find_by_id('enable_animations');
    if (s) {
      return s.isChecked;
    }

    return false;
  };

  function minimumTrellisWidth(setting_name) {
    if (!setting_name) {
      setting_name = 'trellis_cell_minimum_width';
    }

    let minV = 50;

    let v = minV;

    var s = user_settings.find_by_id(setting_name);
    if (s) {
      v = s.value();
      if (isNaN(v)) {
        v = minV;
      } else {
        v = Math.max(Number(v), minV);
      }
    }

    return v;
  }

  function minimumTrellisHeight() {
    return minimumTrellisWidth('trellis_cell_minimum_height');
  }

  function autodetectDates() {
    var s = user_settings.find_by_id('visual_autodetect_time');
    if (!s) {
      return false;
    }

    return s.isChecked;
  }

  base.permitDetailData = function() {
    const showVisualDataDetail = user_settings.find_by_id('visual_show_detail_data');
    const showDashboardDataDetail = dashboard?.user_settings().find_by_id('dashboard_display_view_data');
    if (!showVisualDataDetail || !showDashboardDataDetail) {
      return false;
    }

    return showVisualDataDetail.isChecked && showDashboardDataDetail.isChecked;
  };

  base.displayTooltips = function() {
    var s = user_settings.find_by_id('display_tooltips_table');
    if (!s) {
      s = user_settings.find_by_id('show_tooltip');
    }

    if (!s) {
      return true;
    }

    return s.isChecked;
  };

  base.trellisBorders = function() {
    var s = user_settings.find_by_id('trellis_borders');
    if (!s) {
      return true;
    }

    return s.isChecked;
  };

  base.queryFromResponse = function(r) {
    // Get query from response info unless we are using raw table viz (i.e. the
    // query was directly entered into the sql ui box)
    var info = r.info.filter(function (s) {
      return s.indexOf('Query:') === 0;
    });

    if (info.length !== 1) {
      throw new Error("Query string not found in response");
    }

    return info[0].substring('Query:'.length);
  };

  base.getVisualSQL = function(successCb, errorCb) {
    let rb;

    let params = {
      dataconnection_id: dataset.dataconnection_id(),
      cache_read: true,
    };

    let datarequest = base.shelvesToDataRequest();

    rb = datarequest.rb().cloneRequestBuilder();
    rb.query(undefined);
    rb.limit(1);

    params.dsreq = JSON.stringify(rb.dsreq());
    params.viz_params = JSON.stringify(base.params().getkvs());
    return arc.sqlrun.runSQLWithParams(params, successCb, errorCb);
  };

  base.apply_dataTransformation = function(data_result) {
    let columns = data_result.columns(),
        rows = data_result.rows(),
        groupByCols = [],
        upModifiers = [],
        downModifiers = [];

    /*
     * Handle the data transformations that deal with cumulative and diff aggregate
     * displays.
     *
     * The intent here is to support things like an aggregate line chart on a particular
     * metric over time, without requiring the use of a window function. Window functions
     * are not supported in many backends, and constructing a window function is more
     * complicated than it needs to be.
     *
     * One interesting complication in this implementation is figuring out the right
     * 'window' in which we do the calculation. As an example, to plot states on the X
     * axis, population on the Y axis, and year on the Color axis, if we do a cumulative
     * aggregate across all rows, we get nonsensical answers. The intent is to do the
     * cumulative aggregation for each Year i.e. for each unique dimension value in the
     * Color shelf. The heuristic used below approximates for that by grouping rows by
     * dimension values that are NOT in the X, Y or Dimensions shelf.
     */

    function groupGetter(row) {
      return groupByCols.map((c) => row[c.colindex()]);
    }

    columns.forEach(function(c) {
      let m = c.dataTransformation();
      if ((c.colproperty() === arc.report.columnTypeEnum.DIMENSION) &&
          (['x_shelf', 'y_shelf', 'dimensions_shelf'].indexOf(c.colsource()) < 0)) {
        groupByCols.push(c);
      }

      if ((m === arc.report.shelf.dataTransformationsMap.differenceDesc) ||
          (m === arc.report.shelf.dataTransformationsMap.differenceAsc)) {
        let f = function(row, groupsContext) {
          groupsContext.groups = groupsContext.groups || {};
          const g = groupGetter(row);
          if (groupsContext.groups[g] === undefined) {
            groupsContext.groups[g] = {};
          }

          const context = groupsContext.groups[g];

          if (context.prevValue === undefined) {
            context.prevValue = Number(row[c.colindex()]);
            row[c.colindex()] = 0;
            return;
          }

          let v = Number(row[c.colindex()]),
              v2 = v - context.prevValue;

          context.prevValue = v;
          row[c.colindex()] = v2;
        };

        if (m === arc.report.shelf.dataTransformationsMap.differenceAsc) {
          downModifiers.push(f);
        } else {
          upModifiers.push(f);
        }

        return;
      }

      if ((m === arc.report.shelf.dataTransformationsMap.cumulativeSumAsc) ||
          (m === arc.report.shelf.dataTransformationsMap.cumulativeSumDesc)) {
        let f = function(row, groupsContext) {
          groupsContext.groups = groupsContext.groups || {};
          const g = groupGetter(row);
          if (groupsContext.groups[g] === undefined) {
            groupsContext.groups[g] = {};
          }

          const context = groupsContext.groups[g];

          if (context.value === undefined) {
            context.value = Number(row[c.colindex()]);
            return;
          }

          let v = Number(row[c.colindex()]);
          context.value += v;
          row[c.colindex()] = context.value;
        };

        if (m === arc.report.shelf.dataTransformationsMap.cumulativeSumAsc) {
          downModifiers.push(f);
        } else {
          upModifiers.push(f);
        }

        return;
      }

      if (m === arc.report.shelf.dataTransformationsMap.cumulativeDiffAsc ||
        m === arc.report.shelf.dataTransformationsMap.cumulativeDiffDesc) {
        const cumulativeDiff = function (row, groupsContext) {
          groupsContext.groups = groupsContext.groups || {};
          const g = groupGetter(row);
          if (groupsContext.groups[g] === undefined) {
            groupsContext.groups[g] = {};
          }

          const context = groupsContext.groups[g];

          if (context.prevValue === undefined || context.cumulatedDiff === undefined) {
            context.prevValue = Number(row[c.colindex()]);
            context.cumulatedDiff = 0;
            row[c.colindex()] = 0;
            return;
          }

          const actualValue = Number(row[c.colindex()]);
          context.cumulatedDiff += (actualValue - context.prevValue);
          context.prevValue = actualValue;
          row[c.colindex()] = context.cumulatedDiff;
        };

        if (m === arc.report.shelf.dataTransformationsMap.cumulativeDiffAsc) {
          downModifiers.push(cumulativeDiff);
        } else {
          upModifiers.push(cumulativeDiff);
        }
      }
    });

    if ((downModifiers.length > 0) ||
        (upModifiers.length > 0)) {
      rows = rows.map(function(row) {
        return row.slice();
      });
      data_result.rows(rows);
    }

    downModifiers.forEach(function(f) {
      let groupsContext = {};

      rows.forEach(function(row, i) {
        f(row, groupsContext);
      });
    });

    upModifiers.forEach(function(f) {
      let groupsContext = {};

      let i = rows.length - 1;
      while (i >= 0) {
        let row = rows[i];

        f(row, groupsContext);
        --i;
      }
    });
  };

  function handlePicklistFilterColumns(cb) {
    if (!dashboard) {
      // Only relevant in dashboards with multiple visual elements in place
      return;
    }

    var s = user_settings.find_by_id('picklist_filter_columns');
    if (!s) {
      return;
    }

    var filter_columns = s.value();

    var cols = [];

    if (filter_columns) {
      var collist = filter_columns.split(',');
      collist.forEach(function(cname) {
        // Do substitution here so that we can pick up a parameterized column name
        cname = base.substituteParams(cname.trim());
        cols.push(cname);
      });
    }

    if (cols.length === 0) {
      return;
    }

    var kvs = base.getParams().getkvsList();
    handleDynamicFilters(cols, kvs, cb);
  }

  base.disableIncrementalQueries = function() {
    var s = user_settings.find_by_id('disable_incremental_queries');
    if (!s) {
      return false;
    }

    return s.isChecked;
  };

  base.sendAggregateParameters = function sendAggregateParameters() {
    var s = user_settings.find_by_id('send_aggregate_parameters');
    if (!s) {
      return false;
    }

    return s.isChecked;
  };

  base.getBaseField = function() {
    let baseFieldSetting = user_settings.find_by_id('picklist_base_field') ||
                           user_settings.find_by_id('daterange_base_field');
    if (baseFieldSetting) {
      return base.substituteParams(baseFieldSetting.value());
    }
  };

  base.isAppScope = function() {
    let s = user_settings.find_by_id('filter_appgroup_scope');
    return (s && s.isChecked);
  };

  base.isIndependentDimScale = function isIndependentDimScale() {
    const s = user_settings.find_by_id('scale_dim_independent');
    if (!s) {
      return false;
    }

    return s.isChecked;
  };

  base.scopetype = function() {
    var s = user_settings.find_by_id('filter_dataset_scope');
    if (!s) {
      s = user_settings.find_by_id('visual_dataset_scope');
    }

    if (!s) {
      return arc.params.ParamEnums.ScopeTypeEnum.EXPLICIT;
    }

    const v = s.value();
    if (v === 'true') {
      return arc.params.ParamEnums.ScopeTypeEnum.DATASET;
    }

    if (v === 'GLOBAL') {
      return arc.params.ParamEnums.ScopeTypeEnum.GLOBAL;
    }

    return arc.params.ParamEnums.ScopeTypeEnum.EXPLICIT;
  };

  function displayFilters() {
    if (dashboard) {
      if (!isExplicitScope()) {
        // Temporary condition added for 4.1.1.0 and needs to be fixed properly later.
        return dashboard.displayFilters(base);
      }
    }

    return false;
  }

  var parametersSubstituted = [];
  base.resetParameterTracking = function() {
    parametersSubstituted = [];
  };

  function deduplicate(params) {
    var positions = {};
    params.forEach(function(p, i) {
      positions[p.key] = i;
    });

    var positionIds = {};
    $.each(positions, function(k, v) {
      positionIds[v] = true;
    });

    var ps = [];
    params.forEach(function(p, i) {
      if (i in positionIds) {
        ps.push(p);
      }
    });

    return ps;
  }

  function undrawParams() {
    var $c = $('#' + chart_div).closest('.arc-viz-container');
    var $s = $c.find('.arc-viz-filter');
    $s.empty();
  }

  function drawParams() {
    parametersSubstituted = deduplicate(parametersSubstituted);

    var $c = $('#' + chart_div).closest('.arc-viz-container');
    var $s = $c.find('.arc-viz-filter');
    $s.empty();

    if (parametersSubstituted.length === 0) {
      return;
    }

    var currentParams = base.getParams();

    // Get set of parameters with the .exclude, .start, etc. stripped out
    let baseKeys = {};
    Object.keys(currentParams.getkvs())
          .forEach(k => baseKeys[arc.params.splitParams(k)[0]] = 1);

    parametersSubstituted = parametersSubstituted.filter(
        p => (currentParams.get(arc.params.splitParams(p.key)) !== undefined) ||
             (p.key in baseKeys));

    if (parametersSubstituted.length === 0) {
      return;
    }

    var pillHtml = $('#filter-pill-template').html();
    var pills = [];
    parametersSubstituted.forEach(function(p) {
      pills.push(arc.misc.safe_substitute(pillHtml, {
        TEXT: p.key,
        TOOLTIP: encodeURIComponent(p.value),
      }));
    });

    var pillContainerHtml = $('#filter-pill-container-template').html();
    var s = arc.misc.safe_substitute(pillContainerHtml, {
      HTML_FILTER_PILLS: pills.join(''),
    });

    $s.html(s);

    var sels = d3.selectAll($c.toArray())
                 .selectAll('.filter-pill');
    var tooltip =
        arc.tooltip()
           .dom_id('#tooltip')
           .addSelection(sels)
           .text_fn(function(d, i, elem) {
             var $elem = $(elem);
             var k = $elem.text();
             var v = decodeURIComponent($elem.attr('data-tooltip-contents'));
             return base.tooltip_from_kvs([
               {
                 key: k,
                 value: v,
               },
             ]);
           })
           .activate();
  }

  base.warnIfMaxRows = function(rowCount) {
    if (!arc.globals.is_edit) return;

    const maxRows = base.maxRowCount();
    if (maxRowsExplicitlySpecified()) {
      if (rowCount === maxRows) {
        console.log(`Visual ${base.report_id()}: ` +
          `Rows fetched for this visual have been limited to ${rowCount} rows.`);
      }

      return;
    }

    if (rowCount !== maxRows) {
      return;
    }

    // Show row limit alerts only in edit mode.
    const message = `Rows fetched for this visual have been limited to ${rowCount} rows automatically`;
    const details = 'To stop this warning from showing, set visual limits ' +
    'explicitly in the setting labeled "Maximum number of rows to fetch" ' +
    '(global limit might still be applied.)';
    base.alertMessage(message, details);

    return base;
  };

  function maxRowsExplicitlySpecified() {

    if (arc.globals.gflags && $.isNumeric(arc.globals.gflags.limit)) {
      return true;
    }

    const maxRows = user_settings.find_by_id('max_rowcount');
    if (maxRows !== undefined) {
      let maxRowsValue =  maxRows.value();
      if (maxRowsValue !== undefined) {
        maxRowsValue = maxRowsValue.trim();

        if (maxRowsValue === '' || $.isNumeric(maxRowsValue) === false) {
          return false;
        }
      }
    }

    return true;
  }

  base.maxRowCount = function() {
    const defaultMaxRows = arc.settings.data.maxrows_default;
    const globalMaxRows = arc.globals.visual_max_rows;
    let userMaxRows = user_settings.find_by_id('max_rowcount');

    if (userMaxRows) {
      userMaxRows = (userMaxRows.value() || '').trim();

      if (arc.misc.isNumeric(userMaxRows)) {
        userMaxRows = Number(userMaxRows);
      }
    }

    let limit = Number(defaultMaxRows);

    if (userMaxRows) {
      limit = userMaxRows;
    }

    if (globalMaxRows) {
      limit = Math.min(limit, globalMaxRows);
    }

    return limit;
  };

  base.setupInitialFilters = function() {
    // re for finding column reference in expression
    var colRef = /\[([^\[\]]*)\]/;
    dataset_segments.segments().forEach(function(segment) {
      if (segment.data.applyToNewVisuals) {
        segment.data.filters.forEach(function(filterExpr) {
          var filterCol = '';
          var m = colRef.exec(filterExpr);
          if (m) {
            filterCol = m[1];
          }

          var f = {
            filter_column: filterCol,
            custom_expr: filterExpr,
          };
          shelves.add('filters_shelf', [f], true);
          base.page_needs_save();
        });
      }
    });
  };

  base.getColor = function(row, config) {
    var v = row[config.color_idx];
    if (!config.color_is_dim) {
      v = Number(v);
    }

    return config.color(v);
  };

  base.setupColors = function(config) {
    // Expected inputs are below
    var rows = config.rows,
        columns = config.columns;

    // Expected additional columns set in config are:
    //     config.color_idx
    //     config.color_is_dim
    //     config.palette
    //     config.color
    //     config.color_column
    //     config.colors_domain
    //     config.colorSettings

    config.color_idx = -1;
    columns.forEach(function(col, colindex) {
      if (col.colsource() === 'color_shelf') {
        config.color_idx = colindex;
        return;
      }
    });

    config.color_is_dim = true;

    if (base.chart_type_graded() === 0) {
      config.palette = base.contrast_colors();
    } else if (base.chart_type_graded()) {
      config.palette = base.graded_colors();
    } else {
      config.palette = base.colors();
    }

    var color_extent;

    if (config.color_idx < 0) {
      config.color = function() {
        return base.colors()[0];
      };
    } else {
      config.color_column = columns[config.color_idx];
      if (config.color_column.isNumeric()) {

        var allNumbers = true;
        color_extent = d3.extent(rows, function(row) {
          if (isNaN(row[config.color_idx])) {
            allNumbers = false;
            return 0;
          }

          return Number(row[config.color_idx]);
        });

        if (allNumbers) {
          config.color_is_dim = false;
        }
      }
    }

    if (config.color_is_dim) {
      config.colors_domain =
          arc.misc.arrayUnique(rows, function(row) {
                    return row[config.color_idx];
                  })
                  .sort();
      config.color = base.dimensionalColorScale();
      config.color.domain(config.colors_domain);
    } else {
      if (base.colors_could_be_graded_or_contrast()) {
        config.palette = base.graded_colors();
      }

      colors = config.palette;
      config.colorSettings = base.colorSettings();
      config.color = arc.colorpalette.aggScale(color_extent, config.palette,
          config.colorSettings);
    }

    return base;
  };

  base.aggScale = function(domain) {
    return arc.colorpalette.aggScale(domain, base.graded_colors(), base.colorSettings());
  };

  function fixedDimensionColors() {
    const s = user_settings.find_by_id('fixed_color_legend');
    if (!s) {
      return false;
    }

    return s.isChecked;
  }

  base.dimensionalColorScale = function() {
    var fixedMappings = {};

    var s = user_settings.find_by_id('dim_to_color_mapping');
    if (s) {
      fixedMappings = s.getMappings();
    }

    var c = colorScale.init(fixedDimensionColors(), fixedMappings);
    c.range(base.colors());

    return c;
  };

  base.tickFormatDefaultNumeric = arc.misc.tickFormatDefaultNumeric;
  base.d_string = arc.misc.d_string;

  base.derivedData = function() {
    derivedData.enabled(true);
    return derivedData;
  };

  base.set_viz_state = function(k, v) {
    arc.viz_saved_data = arc.viz_saved_data || {};
    arc.viz_saved_data[report_id] = arc.viz_saved_data[report_id] || {};
    arc.viz_saved_data[report_id][k] = v;
    return base;
  };

  base.get_viz_state = function(k) {
    arc.viz_saved_data = arc.viz_saved_data || {};
    arc.viz_saved_data[report_id] = arc.viz_saved_data[report_id] || {};
    return arc.viz_saved_data[report_id][k];
  };

  base.delete_viz_state = function(k) {
    if (arc.viz_saved_data &&
        arc.viz_saved_data[report_id]) {
      delete(arc.viz_saved_data[report_id][k]);
    }
    return base;
  };

  function destroyMarks() {
    marks.forEach(mark => mark.destroy());
    marks = [];

    return base;
  }

  base.localEnableAnimations = function(_) {
    if (!arguments.length) return localEnableAnimations;
    localEnableAnimations = _;
    return marks;
  };

  base.markActivationEnabled = function(_) {
    if (!arguments.length) return markActivationEnabled;
    markActivationEnabled = _;
    return marks;
  };

  base.marks = function() {
    return marks;
  };

  base.addMark = function(m) {
    marks.push(m);
    return base;
  };

  base.highlightMarks = function() {
    if (dashboard) {
      var ps = base.getParams();
      var kvs = ps.getkvsList(unique_report_id);

      var $c = $('#' + chart_div);
      $c.find('.remove-on-highlight-clear').remove();

      var $filters = $c.find('.highlight-click');
      if ($filters.length > 0) {
        arc.misc.removeClass($filters, 'highlight-click');
      }

      if (kvs.length > 0) {
        marks.forEach(function(m) {
          m.highlightMarks(kvs);
        });
      }
    }
  };

  base.colorOpacity = function() {
    var s = user_settings.find_by_id('color_opacity');
    if (!s) {
      return 1;
    }

    let opacity = Number(s.value());
    if (opacity <= 0) {
      return 0;
    }

    if (!opacity || isNaN(opacity) || opacity >= 100) {
      return 1;
    }

    return (opacity / 100);
  };

  function isExplicitScope() {
    if (viz_type === 'picklist') {
      return true;
    }

    var s = user_settings.find_by_id('explicit_scope');
    if (!s) {
      return false;
    }

    return s.isChecked;
  }

  base.defaultLanguage = function() {
    return user_settings.find_by_id('default_language');
  };

  function downloadRows() {
    return user_settings.find_by_id('download_rows');
  }

  base.visualQuotationMark = function() {
    return user_settings.find_by_id('csv_download_quotation');
  };

  base.cssSetting = function() {
    return user_settings.find_by_id('css_styles');
  };

  base.cssStyles = function() {
    var s = base.cssSetting();
    if (!s) {
      return [];
    }
    return s.getCssIds();
  };

  base.cssData = function() {
    var s = base.cssSetting();
    if (!s) {
      return [];
    }
    return s.getCssData();
  };

  base.inlineCss = function() {
    var s = user_settings.find_by_id('inline_css');
    if (!s) {
      return undefined;
    }
    return s.value();
  };

  function getColumns(filterName) {
    let d = arc.grammar._colnames.parse(filterName);
    return d.cols;
  }

  /**
   *
   * @param paramObjects      - an array of objects containing the new parameters you want to add.
   * @param do_not_refresh    - If true, this function just updates the parameter history and
   *                            does not redraw all the visuals.
   */
  base.dashboard_click = function(paramObjects, do_not_refresh) {
    if (!arc.globals.click_behavior_cb) {
      return base;
    }

    if (!(paramObjects instanceof Array)) {
      // Deprecate receiving of old format params
      console.log('Params received in old format. These will be deprecated soon.');
      paramObjects = arc.params.convertOldParamHashToNewParamObjects(paramObjects, base);
    }

    arc.perf.PAGEPERF.addProfile({
      id: report_id,
      type: 'Parameters',
      time: 0,
      profile: paramObjects.map(function (p) {
        return [p.outputParameter, JSON.stringify(p)];
      }),
    });

    let oldFormatParams = arc.params.convertToOldFormatArr(paramObjects);

    arc.globals.click_behavior_cb(undefined, undefined, undefined,
      undefined, base, undefined, oldFormatParams, do_not_refresh);

    return base;
  };

  base.inModal = function(_) {
    if (!arguments.length) return inModal;
    inModal = _;
    return base;
  };

  base.enable_auto_page_save_handling = function(_) {
    if (!arguments.length) return enable_auto_page_save_handling;
    enable_auto_page_save_handling = _;
    return base;
  };

  base.redraw_on_settings_cancel = function(_) {
    if (!arguments.length) return redraw_on_settings_cancel;
    redraw_on_settings_cancel = _;
    return base;
  };

  base.needs_save = function(_) {
    if (!arguments.length) return needs_save;
    needs_save = _;
    return base;
  };

  function toggleNeedSave(needsSave) {
    base.needs_save(needsSave);

    if (chart_div) {
      // Search results do not mark the page as saved
      if ($(`#${chart_div}`).closest('.search-result').length > 0) {
        return base;
      }
    }

    if (enable_auto_page_save_handling && arc.globals.is_edit) {
      arc.report.pagenav.needs_save(needsSave);
    }
  }

  base.page_not_need_save = function() {
    toggleNeedSave(false);
    return base;
  };

  base.page_needs_save = function() {
    toggleNeedSave(true);
    return base;
  };

  base.notificationMessage = function(msg) {
    arc.report.alertMessageSmallBase({
      type: 'alert-warning',
      alertMessage: msg,
      fadeOut: true,
    });

    return base;
  };

  base.getErrorPrefix = function() {
    if (viz_type === 'timecontrol') {
      return 'Time Control';
    }

    if (arc.misc.isFilterVisual(viz_type)) {
      return 'Filter';
    }

    return 'Visual';
  };

  base.getLogMessage = function(msg) {
    return `${base.getErrorPrefix()} ${report_id > 0 ? report_id : ''} - ${msg}`;
  };

  base.alertMessage = function(msg, detail, alertMessageConfig = {}) {
    msg = base.getLogMessage(msg);
    arc.frontend.usageTracker.incrementErrorCount(true);
    const $pageErrors = $('.viz-error-page-wrapper');
    if ($pageErrors.length > 0) {
      arc.errors.reportError(base, msg, detail, alertMessageConfig);
    } else {
      arc.report.alertMessage(msg, detail, alertMessageConfig);
    }

    return base;
  };

  base.displayPersistentMessages = function() {
    const msgs = arc.globals.is_edit ? base.editAlertMessages() : base.viewAlertMessages();
    $(`#${base.chart_div()} .viz-alert`).remove();
    if (msgs.length > 0) {
      const messageString = msgs.join(" ");
      arc.report.alertMessageSmallBase({
        type : 'alert-warning',
        alertMessage : messageString,
        fadeOut : false,
      });
    }
    return base;
  };

  base.formatter = function(colindex) {
    return sqlIndexToFormatter[colindex];
  };

  base.formattedDisplay = arc.format.formattedDisplay;

  base.safeFormatting = arc.format.safeFormatting;

  base.get_shelf_contents = function(ignore_alias) {
    var c = [];
    base.shelfExpressionsCb(ignore_alias, function(shelf_name, expr, idx,
        shelf_element) {
      c.push([shelf_name, expr, idx, shelf_element.enableExpansion()]);
    });

    if (limit_value) {
      c.push(['limit', limit_value, 0]);
    }

    if (base.isSolr() || base.isElasticsearch()) {
      c.push(['search', base.solrSearchData()]);
    }

    var d = base.shelvesToDataRequest().dsreq();
    c.push([d]);

    return c;
  };

  base.in_dashboard = function(_) {
    if (!arguments.length) return in_dashboard;
    in_dashboard = _;
    return base;
  };

  base.dashboard_click_behavior = function(_) {
    if (!arguments.length) return dashboard_click_behavior;
    dashboard_click_behavior = _;
    return base;
  };

  base.unhighlight_cb = function(_) {
    if (!arguments.length) return unhighlight_cb;
    unhighlight_cb = _;
    return base;
  };

  base.viz = function(_) {
    if (!arguments.length) return viz;
    viz = _;
    return base;
  };

  base.colors_could_be_graded_or_contrast = function() {
    return (viz_type === 'scatter');
  };

  base.viz_type = function(_) {
    if (!arguments.length) return viz_type;
    viz_type = _;
    return base;
  };

  base.x_slant = function(_) {
    if (!arguments.length) return x_slant;
    x_slant = _;
    return base;
  };

  base.loadColorIntoSetting = function() {
    var c = user_settings.find_by_id('color_palette');
    if (c) {
      c.loadColor();
    }

    if (viz_type === 'network') {
      let colorSetting = user_settings.find_by_id('link_color_palette');
      if (colorSetting) {
        colorSetting.loadColor();
      }
    }

    return base;
  };

  base.getColorPaletteColors = function() {
    let c = user_settings.find_by_id('color_palette');
    if (!c) {
      c = user_settings.find_by_id('link_color_palette');
    }

    if (!c) {
      return undefined;
    }

    const data = JSON.parse(c.value()),
          colors = data.colors || [];

    return colors;
  };

  base.saveVisualSettings = function(needsSaveCb, existingValues) {
    var needsSaveScoped = false;
    user_settings.settings().forEach(function(user_setting) {
      var setting_needs_save = user_setting.process_value();
      if (!needsSaveScoped && setting_needs_save) {
        needsSaveScoped = true;
      }

      if (existingValues) {
        if (user_setting.value() !== existingValues[user_setting.id()]) {
          needsSaveScoped = true;
        }
      }
    });

    // Save away color palette in lib
    var c = user_settings.find_by_id('color_palette');
    if (c) {
      c.saveIntoVisual();
    }

    if (viz_type === 'network') {
      let colorSetting = user_settings.find_by_id('link_color_palette');
      if (colorSetting) {
        colorSetting.saveIntoVisual();
      }
    }

    if (needsSaveScoped) {
      if (needsSaveCb) {
        needsSaveCb();
      }
    }

    return base;
  };

  base.user_settings = function(_) {
    if (!arguments.length) return user_settings;
    user_settings = _;
    user_settings.settings().forEach(function(user_setting) {
      user_setting.viz_lib(base);
    });

    return base;
  };

  base.settings_changed = function() {
    return base;
  };

  base.click_behavior_change_cb = function(_) {
    if (!arguments.length) return click_behavior_change_cb;
    click_behavior_change_cb = _;
    return base;
  };

  base.chart_type_graded = function(_) {
    if (!arguments.length) return chart_type_graded;
    chart_type_graded = _;
    return base;
  };

  base.xaxis_label_maxlen = function(_) {
    if (!arguments.length) return xaxis_label_maxlen;
    xaxis_label_maxlen = _;
    return base;
  };

  base.yaxis_label_maxlen = function(_) {
    if (!arguments.length) return yaxis_label_maxlen;
    yaxis_label_maxlen = _;
    return base;
  };

  base.dim_tick_label_maxlen = function(_) {
    if (!arguments.length) return dim_tick_label_maxlen;
    dim_tick_label_maxlen = _;
    return base;
  };

  base.main_axis_label_maxlen = function(_) {
    if (!arguments.length) return main_axis_label_maxlen;
    main_axis_label_maxlen = _;
    return base;
  };

  base.xaxis_label_text = function(_) {
    if (!arguments.length) return xaxis_label_text;
    xaxis_label_text = _;
    return base;
  };

  base.yaxis_label_text = function(_) {
    if (!arguments.length) return yaxis_label_text;
    yaxis_label_text = _;
    return base;
  };

  base.colorSettings = function(_) {
    if (!arguments.length) {
      const s = $.extend({}, colorSettings);

      const measureRanges = user_settings.find_by_id('measure_ranges');
      if (measureRanges) {
        const v = JSON.parse(measureRanges.value());
        s.measureRanges = v.ranges;
      }

      return s;
    }

    colorSettings = _;
    return base;
  };

  base.colors = function(_) {
    if (!arguments.length) return colors;
    colors = _;
    return base;
  };

  base.linkColors = function(_) {
    if (!arguments.length) return linkColors;
    linkColors = _;
    return linkColors;
  };

  base.contrast_colors = function(_) {
    if (!arguments.length) return contrast_colors;
    contrast_colors = _;
    return base;
  };

  base.graded_colors = function(_) {
    if (!arguments.length) return graded_colors;
    graded_colors = _;
    return base;
  };

  base.reverse_colors = function(_) {
    if (!arguments.length) return reverse_colors;
    reverse_colors = _;
    return base;
  };

  base.build_mode = function(_) {
    if (!arguments.length) return build_mode;
    build_mode = _;
    return base;
  };

  base.shelves_dom_id = function(_) {
    if (!arguments.length) return shelves_dom_id;
    shelves_dom_id = _;
    return base;
  };

  function hideTitle() {
    var s = user_settings.find_by_id('visual_hide_title');
    if (s === undefined) {
      return false;
    }

    return s.isChecked;
  }

  function fontSettingToFont(fontSetting) {
    let font;

    if (fontSetting) {
      font = $('.' + fontSetting).css('font-family');
    }

    return font;
  }

  base.getVisualFont = function() {
    let fontSetting = visualFont();

    return fontSettingToFont(fontSetting) || 'Roboto';
  };

  base.getVisualFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'visual_font_size');
  };

  base.getCrosstabTitleFont = function() {
    return fontSettingToFont(
        arc.settings.fonts.fontSetting(user_settings, 'crosstab_title_font')) ||
        base.getVisualFont();
  };

  base.getCrosstabTitleFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'crosstab_title_font_size') ||
        base.getVisualFontSize();
  };

  base.getCrosstabTitleFontSizeLegacy = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'crosstab_title_font_size') || '11';
  };

  base.getTotalsFontWeight = function(id) {
    const s = user_settings.find_by_id(`${id}`);
    if (!s) {
      return undefined;
    }

    if (s.value() === 'default') {
      return undefined;
    }

    return s.value();
  };

  base.getLabelFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'label_font_size') ||
        base.getVisualFontSize();
  };

  base.getAxisLabelFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'axis_label_font_size') ||
        base.getVisualFontSize();
  };

  base.getAxisTicksFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'axis_ticks_font_size') ||
        base.getVisualFontSize();
  };

  base.getLegendFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'legend_font_size') ||
        base.getVisualFontSize();
  };

  base.getTrellisFontSize = function() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'trellis_font_size') ||
        base.getVisualFontSize();
  };


  base.getBackgroundColor = function() {
    let v = arc.settings.colors.colorSetting(user_settings, 'visual_bg_color');
    if (v !== undefined) {
      return v;
    }

    if (dashboard) {
      const dashboardSettings = dashboard.user_settings();
      v = dashboardSettings.find_by_id('dashboard_bg_color').value();
    }

    return v;
  };

  base.getLabelColor = function() {
    const colorVal = dashboard && arc.misc.dashboardOverride() ?
                    dashboard.user_settings().find_by_id('label_color').value() :
                    user_settings.find_by_id('label_color').value();
    return colorVal ? arc.colorpalette.rgbStringToHex(colorVal) : '';
  };

  function tooltipFont() {
    return arc.settings.fonts.fontSetting(user_settings, 'tooltip_font');
  }

  function tooltipFontSize() {
    return arc.settings.fonts.fontSizeSetting(user_settings, 'tooltip_font_size');
  }

  function visualFont() {
    return arc.settings.fonts.fontSetting(user_settings, 'visual_font');
  }

  function titleFont() {
    return arc.settings.fonts.fontSetting(user_settings, 'title_font');
  }

  function subtitleFont() {
    return arc.settings.fonts.fontSetting(user_settings, 'subtitle_font');
  }

  function filterFont() {
    return arc.settings.fonts.fontSetting(user_settings, 'filter_font');
  }

  function textAlignment(type) {
    return arc.settings.fonts.fontAlignmentSetting(user_settings, type);
  }

  function setSubtitleStyle() {
    let $s = $(`#${subtitle_elem}`);
    let $visual = $(`#${chart_div}`);

    if ((subtitle === '') && !showEmptyTitleSubtitle) {
      checkAndSetCss($s.closest('.arc-viz-subtitle-container'), 'display', 'none');
      if (arc.globals.is_edit && (viz_type === 'crosstab' || viz_type === 'table')) {
        $visual.parent().css('marginTop', '20px');
      }
    } else {
      checkAndSetCss($s.closest('.arc-viz-subtitle-container'), 'display', '');
      $visual.parent().css('marginTop', '0px');
    }

    let c = $s.attr('data-fontclass');
    let f = subtitleFont();
    if (c !== f) {
      if (c) {
        $s.removeClass(c);
        $s.attr('data-fontclass', null);
      }

      if (f) {
        $s.attr('data-fontclass', f);
        $s.addClass(f);
      }
    }

    let sz = arc.settings.fonts.fontSizeSetting(user_settings, 'subtitle_font_size');
    if (sz === undefined) {
      $s.css('font-size', '');
    } else {
      $s.css('font-size', `${sz}px`);
    }

    let alignment = textAlignment('subtitle_alignment');
    if (alignment === undefined) {
      alignment = null;
    }

    $s.closest('.arc-viz-subtitle-container').css('text-align', alignment);
  }

  function checkAndSetCss($e, k, v) {
    let _v = $e.css(k);

    if (_v === v) {
      return;
    }

    $e.css(k, v);
  }

  function applyBgColor() {
    const viz_div = base.chart_div();
    let bgColor = base.getBackgroundColor() || '';

    const $vizDiv = $(`#${viz_div}`);
    if ($vizDiv.length === 0) {
      return;
    }

    let $vizContainer;

    if ($vizDiv.hasClass('.arc-viz-container')) {
      $vizContainer = $vizDiv;
    } else {
      $vizContainer = $vizDiv.closest('.arc-viz-container');
    }

    $vizContainer.css('background-color', bgColor);
  }

  function setTitleAndVisualStyle() {
    applyBgColor();

    const $s = $('#' + title_elem);
    if (((arc.globals.is_edit && base.inModal()) || (!arc.globals.is_edit)) &&
        hideTitle()) {
      checkAndSetCss($s.closest('.arc-viz-title-container'), 'display', 'none');
    } else {
      checkAndSetCss($s.closest('.arc-viz-title-container'), 'display', '');
    }

    if ((title === '') && !showEmptyTitleSubtitle) {
      checkAndSetCss($s.closest('.arc-viz-title-container'), 'display', 'none');
    }

    // If visual is untitled don't display background color
    // Add class with top padding so action buttons are not covered by visual
    const $vizWrapper = $s.closest('.arc-viz-container').find('.dash-viz-wrapper');
    if ((base.inModal()) && ($s.html() === '')) {
      $vizWrapper.addClass('viz-no-title');
      checkAndSetCss($s.closest('.arc-viz-title-container'), 'display', 'none');
    } else {
      $vizWrapper.removeClass('viz-no-title');
    }

    var f = titleFont();
    arc.misc.setFont($s, 'data-fontclass', f);

    var sz = arc.settings.fonts.fontSizeSetting(user_settings, 'title_font_size');
    if (sz === undefined) {
      checkAndSetCss($s, 'font-size', '');
    } else {
      checkAndSetCss($s, 'font-size', sz + 'px');
    }

    var alignment = textAlignment('title_alignment');
    if (alignment === undefined) {
      alignment = '';
    }

    checkAndSetCss($s.closest('.arc-viz-title-container'), 'text-align', alignment);

    let fontweight = arc.settings.fonts.fontSetting(user_settings,
                     'visual_title_font_weight');
    if (fontweight === 'default' || fontweight === undefined) {
      fontweight = '';
    }

    checkAndSetCss($s.closest('.arc-viz-title-container'), 'font-weight',
                     fontweight);

    var $container = $s.closest('.arc-viz-container');
    f = visualFont();
    arc.misc.setFont($container, 'data-fontclass', f);

    setFontSize($container);

    const titleFgColor = arc.settings.colors.colorSetting(user_settings, 'visual_title_fg') || '';
    $s.css('color', titleFgColor);

    const titleBgColor = arc.settings.colors.colorSetting(user_settings, 'visual_title_bg') || '';
    $s.closest('.arc-viz-title-container').css('background-color', titleBgColor);
  }

  base.setFontStyles = function() {
    setTitleAndVisualStyle();
    setSubtitleStyle();
    return base;
  };

  function displaySubtitle(s) {
    var $s = $('#' + subtitle_elem);
    setSubtitleStyle();

    if (base.inModal() && (s === '')) {
      checkAndSetCss($s.closest('.arc-viz-subtitle-container'), 'display', 'none');
    }

    $s.text(s);

    const subtitleFgColor = arc.settings.colors.colorSetting(user_settings, 'visual_subtitle_fg') || '';
    $s.css('color', subtitleFgColor);

    const subtitleBgColor = arc.settings.colors.colorSetting(user_settings, 'visual_subtitle_bg') || '';
    $s.closest('.arc-viz-subtitle-container').css('background-color', subtitleBgColor);

    return base;
  }

  function displayTitle(title) {
    if (!title_visible) {
      title = '';
    }

    var $s = $('#' + title_elem);
    $s.text(title);
    setTitleAndVisualStyle();
    return base;
  }

  base.title = function(_) {
    if (!arguments.length) return title || '';
    title = _;
    // When modifying the title, display it
    displayTitle(title);
    return base;
  };

  base.subtitle = function(_) {
    if (!arguments.length) return subtitle;
    subtitle = _;
    // When modifying the subtitle, display it
    displaySubtitle(subtitle);
    return base;
  };

  base.needs_events = function(_) {
    if (!arguments.length) return needs_events;
    needs_events = _;
    return base;
  };

  base.shelves = function(_) {
    if (!arguments.length) return shelves;
    shelves = _;
    return base;
  };

  // Returns back whether there's a pre-defined sort order
  // Used for calculating scale of trellised bar/line/area/scatter plots
  base.has_sort_order = function (config) {

    if (config.measureTrellis) {
      return config.columns.some(col => col.order()) || config.measureTrellis.some(col => col.order());
    } else {
      return config.columns.some(col => col.order());
    }
  };

  base.settings_color_disabled_show = function() {
    $('#chart_color_disabled_div').show();
    return base;
  };

  base.ctas_cmd = function(tgt_tablename, sql_cmd) {
    var c = 'CREATE TABLE ' + tgt_tablename + '\nSTORED AS PARQUET\nAS\n' +
        sql_cmd;
    if (c.substr(c.length - 12, c.length) === 'limit 100000') {
      return c.substr(0, c.length - 12);
    }
    return c;
  };

  base.cv_cmd = function(tgt_tablename, sql_cmd) {
    var c = 'CREATE VIEW ' + tgt_tablename + '\nAS\n' +
        sql_cmd;
    if (c.substr(c.length - 12, c.length) === 'limit 100000') {
      return c.substr(0, c.length - 12);
    }
    return c;
  };

  base.cav_cmd = function(tgt_tablename, ds_partitions, sql_cmd) {
    var partition_str = '';
    var c = 'CREATE ANALYTICAL VIEW ' + tgt_tablename + partition_str +
        '\nSTORED AS PARQUET\nAS\n' + sql_cmd;
    if (c.substr(c.length - 12, c.length) === 'limit 100000') {
      c = c.substr(0, c.length - 12);
    }
    c = c + ';\n';
    c = c + 'REFRESH ANALYTICAL VIEWS;\n';
    return c;
  };

  function settingsEnabled() {
    if (arc.globals.is_edit && !in_dashboard) {
      return true;
    }

    if (arc.globals.is_edit && isFilterOrTimeControl(viz_type)) {
      return true;
    }

    return false;
  }

  function isFiltersShelf() {
    var $c = $('#' + chart_div);
    return $c.hasClass('selected-control');
  }

  function hideFilterSettings() {
    user_settings.settings().forEach(function(s) {
      if ((s.tab() === 'Fonts') || (s.tab() === 'Alignment')) {
        s.hidden(true);
      }
    });
  }

  function hideColorSettings() {
    user_settings.settings().forEach(function(s) {
      if (s.tab() === 'Color') {
        s.hidden(true);
      }
    });
  }

  base.updateSettingsModal = function (settingId, value) {
    settings_modal?.setExistingValue(settingId, value);
  };

  base.display_settings = function() {
    if (settingsEnabled()) {
      if (isFiltersShelf()) {
        hideFilterSettings();
      }

      if (viz_type === 'table' || viz_type === 'rawtable') {
        hideColorSettings();
      }

      settings_modal = arc.report.settingsModal();

      settings_modal.viz_lib(base)
                    .user_settings(user_settings)
                    .large(true)
                    .setup(function() {
                      base.page_needs_save();
                      // When the settings modal is saved, always
                      // fully refresh the filter
                      base.refresh(true);
                    });

      settings_modal.show();
    }

    return base;
  };

  base.title_visible = function(_) {
    if (!arguments.length) return title_visible;
    title_visible = _;
    titleVisibleActions();
    return base;
  };

  base.refresh_visible = function(_) {
    if (!arguments.length) return refresh_visible;
    refresh_visible = _;
    return base;
  };

  function titleVisibleActions() {
    if (title_visible) {
      $('#viz-title-edit').show();
      $('#viz-subtitle-edit').show();
      $('#' + base.title_elem()).show();
      $('#' + base.subtitle_elem()).show();
    } else {
      $('#viz-title-edit').hide();
      $('#viz-subtitle-edit').hide();
      $('#' + base.title_elem()).hide();
      $('#' + base.subtitle_elem()).hide();
    }
  }

  // Base setup function
  base.base_setup = function() {
    const conn_type = base.dataconnection_type();
    if (arc.dataconnection.isArcEngine(conn_type)) {
      $('#chart-flow').show();
      $('#chart-funnel').show();
    } else {
      $('#chart-flow').hide();
      $('#chart-funnel').hide();
    }

    if (arc.dataconnection.isVectorDB(conn_type)) {
      $('#chart-chatbot').show();
    } else {
      $('#chart-chatbot').hide();
    }

    if (refresh_visible) {
      $('#viz-refresh').show();
    } else {
      $('#viz-refresh').hide();
    }

    titleVisibleActions();

    $('#chart_color_disabled_div').hide();
    $('.app-table').show();

    d3.select('#' + shelves_dom_id)
      .selectAll('tr')
      .remove();
    d3.select('#' + shelves_dom_id)
      .selectAll('div')
      .remove();

    d3.select('.add-to-ds-link')
      .on('click.export', function() {

        // DJC -- Should clone the select builder, drop order and limit, then
        //        get the sql from that.
        //        join cols would be any simple columns in the dimensions shelf
        //        need to ensure no error
        //        what does 'parent' mean? -- perhaps get tables listed in sql
        //        and use the farthest from root?
        var sql = last_run_sql || last_good_sql || sql_query;
        var join_pairs = [];
        base.shelfElementCb('dimensions_shelf', function(shelf_element, i) {
          if (!shelf_element.custom_expr()) {
            var expr = shelf_element.getExpression(base.substituteParams,
                true);
            join_pairs.push([expr,expr]);
          }
        });

        // TODO: Update this comment to keep console logs or remove console logs below
        console.log('Col names:', sb_colnames);
        console.log('Col types:', sb_coltypes);
        console.log('Join pairs: ', join_pairs);
        console.log('Sql: ', sql);
        dataset.add_sq_table(sql, dataset.tables()[0].tablename_noalias(),
                             'sqtab', join_pairs, sb_colnames, sb_coltypes);
        arc.report.dataset_setup(dataset);
        arc.globals.singleVisualBuilder.createReport();
      });

    shelves = arc.report.shelf.shelves();

    shelves.viz_lib(base)
           .shelves_dom_id(shelves_dom_id)
           .data(base.shelf_data())
           .dataset_events(dataset_events)
           .dataset(dataset)
           .setup();

    paintShelves(shelves, lastKnownShelves);

    function doSearch() {
      base.refresh(true);
    }

    if (base.dataconnection_type() &&
      (arc.dataconnection.connectionTypeEnum[base.dataconnection_type()] === arc.dataconnection.connectionTypeEnum.solr ||
       arc.dataconnection.connectionTypeEnum[base.dataconnection_type()] === arc.dataconnection.connectionTypeEnum.elasticsearch)) {
      $(`#${shelves_dom_id} .searchbox`).offOn('change', doSearch);
      $(`#${shelves_dom_id} .searchbutton`).offOn('click', doSearch);
    }

    // Hide Filters for Solr
    // May want to add this functionality directly in shelves constructor
    // if (base.dataconnection_type() && arc.dataconnection.connectionTypeEnum[base.dataconnection_type()] === arc.dataconnection.connectionTypeEnum.solr) {
    //   $('#filters_shelf_popover').closest('tr').hide();
    // }

    if (base.needs_events()) {
      $('.app-events-content').show();
    } else {
      $('.app-events-content').hide();
    }

    filtersaver = arc.report.filtersaver()
                            .dataset_segments(dataset_segments)
                            .activate();

    d3.select('.settings-link')
      .on('click.settings', function() {
        base.display_settings();
      });

    return base;
  };

  // Switch X shelf elements to point at Y, and Y shelf elements
  // to point at X
  function switchShelfElementRefs() {
    const xShelf = shelves.getShelf('x_shelf');
    const yShelf = shelves.getShelf('y_shelf');

    const xElements = xShelf.elements();
    const yElements = yShelf.elements();

    xElements.forEach(function(e) {
      e.shelf(yShelf);
    });

    yElements.forEach(function(e) {
      e.shelf(xShelf);
    });

    xShelf.elements(yElements);
    yShelf.elements(xElements);
  }

  // Setup shelves-pivot icon to switch x and y shelves
  base.setupSwitchXY = function (shelves_div) {
    $('#' + shelves_div + ' [data-toggle="tooltip"]').tooltip();

    d3.select('#' + shelves_div + ' .shelves-pivot')
      .on('click.pivot', function() {
        switchShelfElementRefs();

        var x_data = [],
            y_data = [];
        var x_shelf = d3.select('#' + shelves_div)
                        .select('#x_shelf');
        var y_shelf = d3.select('#' + shelves_div)
                        .select('#y_shelf');
        var x_html = x_shelf.html();
        var y_html = y_shelf.html();
        if (x_html !== y_html) {
          x_shelf.selectAll('.selection_field')
                 .each(function(d) {
                   x_data.push(d);
                 });
          y_shelf.selectAll('.selection_field')
                 .each(function(d) {
                   y_data.push(d);
                 });
          x_shelf.html(y_html);
          x_shelf.selectAll('.selection_field')
                 .data(y_data);
          y_shelf.html(x_html);
          y_shelf.selectAll('.selection_field')
                 .data(x_data);
          base.refresh(true);
          base.page_needs_save();
        }
      });
  };

  // for downloading csv of expansion-enabled crosstab
  base.fullyExpandedDsreq = function (_) {
    if (!arguments.length) return fullyExpandedDsreq;
    fullyExpandedDsreq = _;
    return base;
  };

  function getFormattingSettings() {
    const shelves = shelf_data.reduce((acc, shelf) => (
      acc.concat(base.getShelfElements(shelf.name))
    ), []);

    /**
     * Even though there's no any shelf elements,
     * in case of table and rawtable visuals, there's a chance that we can
     * get a default display format applied on the data column directly
     */
    if (shelves.length === 0 && ['rawtable', 'table'].includes(base.viz_type())) {
      const tables = base.dataset().tables();
      return tables
        .reduce((acc, table) => acc.concat(table.columns().map(col => col)), [])
        .map(col => col.attr('displayformat') || null);
    }

    return shelves.map((shelf, i) => {
      const displayFormat = shelf.display_format();
      if (!displayFormat) {
        return;
      }

      return Object.assign({}, displayFormat, {
        colName: sb_colnames[i]
      });
    });
  }

  base.getCsvDelimiter = function () {
    const languageOverride = base.defaultLanguage().getValue().value;
    return arc.misc.getCsvDelimiter(arc.misc.getLanguage(languageOverride));
  };

  base.exportCSV = function ({ query, numberCSVRows, forceFullDownload }) {
    const csvName = arc.misc.getTimestampFilename((base.title() || 'Untitled'), 'csv');

    if (base.viz_type() === 'chatbot') {
      const { colNames, rows } = base.viz().getDownloadInfo();

      if (colNames && rows) {
        arc.misc.downloadCsv(csvName, colNames, rows);
      } else {
        arc.report.alertMessage('No data to download');
      }

      return;
    }

    if (!query) {
      query = baseQuery;
    }

    const downloadRowsSetting = downloadRows();
    const quotationSetting = base.visualQuotationMark().value();
    const languageOverride = base.defaultLanguage().getValue().value;
    const languageCode = arc.misc.getLanguage(languageOverride);
    const languageCodePython = languageCode.replace(/\-/g, '_');
    const supportedBrowser = typeof MouseEvent === 'function';
    const formattingSettings = getFormattingSettings();
    const shouldDownload = false;

    // if we are returning CSV data for phantomjs job, just return back csv array
    // However, limit it to minimum between email-specified limit or
    // user-specified download limit from visual
    if (numberCSVRows !== undefined) {
      let localLimit = NaN;
      if (downloadRowsSetting && $.isNumeric(downloadRowsSetting.value())) {
        localLimit = Number(downloadRowsSetting.value());
      }

      if (!Number.isNaN(localLimit)) {
        numberCSVRows = Math.min(localLimit, numberCSVRows);
      }

      if (!forceFullDownload) {
        return arc.misc.downloadCsv(
          csvName,
          sb_colnames,
          sb_rows,
          sb_coltypes,
          languageOverride,
          numberCSVRows,
          quotationSetting,
          base.dataResult() ? base.dataResult().columns() : undefined,
          shouldDownload
        );
      }
    }

    if ((downloadRowsSetting || forceFullDownload) && supportedBrowser) {
      let v = downloadRowsSetting.value();
      let csvQueryLimit = 0;

      if ($.isNumeric(v)) {
        csvQueryLimit = Number(v);
      }

      if (csvQueryLimit || forceFullDownload) {
        query.exportFileViaSnapshot({
          filename: csvName,
          languageCode: languageCodePython,
          dsreqOverride: fullyExpandedDsreq,
          queryLimit: csvQueryLimit,
          quotationSetting,
          formattingSettings,
          forceFullDownload,
          fileFormat: 'csv',
        });
        return base;
      }
    }

    let numValues;
    if (sb_rows && sb_colnames) {
      numValues = sb_rows.length * sb_colnames.length;
    }

    if (!numValues && !query) {
      arc.report.alertMessage('No data to download');
      return base;
    }

    // For large enough downloads, insist on using remote downloads. Using a
    // local blob takes up a lot of memory and may crash some browsers
    // Also always export via django for crosstabs with expansion enabled
    // and for flow and funnel visuals

    if (!numValues || numValues > DOWNLOAD_BROWSER_THRESHOLD || fullyExpandedDsreq) {
      if (supportedBrowser) {
        query.exportFileViaSnapshot({
          filename: csvName,
          languageCode: languageCodePython,
          dsreqOverride: fullyExpandedDsreq,
          queryLimit: undefined,
          quotationSetting,
          formattingSettings,
          fileFormat: 'csv',
        });
      } else {
        arc.report.alertMessage(
          'Failed to download.',
          'Dataset is too large. Please try a different browser or use a smaller dataset.'
        );
      }

      return base;
    }

    try {
      arc.misc.downloadCsv(
        csvName,
        sb_colnames,
        sb_rows,
        sb_coltypes,
        languageOverride,
        undefined,
        quotationSetting,
        base.dataResult() ? base.dataResult().columns() : undefined
      );
    } catch (e) {
      console.log(e);
      if (query) {
        query.exportFileViaSnapshot({
          filename: csvName,
          languageCode: languageCodePython,
          dsreqOverride: fullyExpandedDsreq,
          queryLimit: undefined,
          quotationSetting,
          formattingSettings,
          fileFormat: 'csv'
        });
      }
    }

    return base;
  };

  base.exportXlsx = function(query) {
    if (!query) {
      query = baseQuery;
    }

    const downloadRowsNum = downloadRows().value();
    const supportedBrowser = typeof MouseEvent === 'function';
    const numValues = base.sb_rows().length * base.sb_columns().length;
    const xlsxFileName = arc.misc.getTimestampFilename(base.title() || 'Untitled', 'xlsx');
    const formattingSettings = getFormattingSettings();
    if ($.isNumeric(downloadRowsNum)) {
      let limit = Number(downloadRowsNum);
      if (limit >= 0) {
        query.exportFileViaSnapshot({
          filename: xlsxFileName,
          formattingSettings,
          queryLimit: limit,
          dsreqOverwrite: fullyExpandedDsreq,
          fileFormat: 'xlsx',
        });
      }

      return;
    }

    initiateXLSXDownload();

    return base;

    function initiateXLSXDownload() {
      if ((numValues < DOWNLOAD_BROWSER_THRESHOLD && supportedBrowser) && !fullyExpandedDsreq) {
        arc.misc.downloadXlsx(base);
      } else {
        query.exportFileViaSnapshot({
          filename: xlsxFileName,
          formattingSettings,
          queryLimit: undefined,
          dsreqOverwrite: fullyExpandedDsreq,
          fileFormat: 'xlsx',
        });
      }
    }
  };

  // Initializa the default setup function to be the base setup function
  // This is called once when the report is instantiated
  base.setup = base.base_setup;

  base.getDrillColumns = function() {
    let drillColumns = {};
    let isGeoShelf = false;
    let foundShelves = base.shelf_data().some(function(shelf) {
      if (shelf.drillShelf === true) {
        if (shelf.geoShelf === true) {
          isGeoShelf = true;
        }

        shelfElementSimpleCb(shelf.name, function(shelf_element, i) {
          drillColumns[shelf_element.dataset_colname()] =
            shelf_element.col_alias() || shelf_element.dataset_colname();
        });

        return true;
      }
    });

    if (isGeoShelf) {
      return drillColumns;
    }

    return Object.keys(drillColumns).length === 0 ? undefined : drillColumns;
  };

  base.connectDataConnectionToShelves = function() {
    var dataconnection_type;
    if (dataset) {
      dataconnection_type = dataset.dataconnection_type();
    }

    if (!dataconnection_type) {
      return;
    }

    base.shelf_data().forEach(function(shelf_element) {
      var shelf_name = shelf_element.name;
      shelfElementSimpleCb(shelf_name, function(shelf_element, i) {
        if (shelf_element) {
          shelf_element.dataconnection_type(dataconnection_type);
        }
      });
    });
  };

  base.shelfNames = function() {
    return base.shelf_data().map(function(s) {
      return s.name;
    });
  };

  base.shelfElementCounts = function(shelf_names) {
    if (shelf_names === undefined) {
      shelf_names = base.shelf_data().map(function(shelf_element) {
        return shelf_element.name;
      });
    }

    return shelf_names.reduce(function(previousValue, shelf_name) {
      return previousValue + base.shelfElementCount(shelf_name);
    },
    0);
  };

  // Optional dims can be specified as:
  // [<<dim1:>>], [<<dim1:''>>], <<dim1:''>>, <<dim1:>>
  // Optional measures can be specified as:
  // [<<mea1:>>], [<<mea1:''>>], <<mea1:''>> but not as <<mea1:>>
  // Note: Can't allow <<mea1:>>, because () might be a valid entry like pi()
  // TODO: To allow dim:"", then add expr === '""' || expr === '[""]'
  base.isShelfExpressionEmpty  = function(expr) {
    var retval;
    var emptystrings = ["[]", "['']", "('')"];
    if (expr === '' || expr === "''") {
      // For cases <<dim1:>> or <<dim1:''>>
      return true;
    }
    retval = emptystrings.some(function(emptystr) {
      // Need to check indexOf and not equality for cases like:
      // sum([<<mea1:>>]) or sum([<<mea1:''>>] or sum(<<mea1:''>>)
      return (expr.indexOf(emptystr) >= 0);
    });
    return retval;
  };

  base.shelfElementCount = function(shelf_name) {
    let cnt = 0;

    shelfElementSimpleCb(shelf_name, function(shelf_element) {
      var e = shelf_element.getExpression(base.substituteParams, true);
      if (!base.isShelfExpressionEmpty(e)) {
        ++cnt;
      }
    });

    return cnt;
  };

  function shelfElementSimpleCb(shelf_name, cb) {
    if (!shelves) {
      return;
    }
    const shelf = shelves.getShelf(shelf_name);
    if (!shelf) {
      return;
    }

    shelf.elements().forEach(function(shelf_element, i) {
      cb(shelf_element, i);
    });
  }

  // Duplicate of shelfElementCb but without cb mechanism
  // include_empty_elements is true when cloning, deleting, or in filter shelf
  base.getShelfElements = function(shelf_name, include_empty_elements, no_param_sub) {
    const found = [];

    shelfElementSimpleCb(shelf_name, function(shelf_element) {
      const par_sub_cb = !no_param_sub && base.substituteParams;

      const e = shelf_element.getExpression(par_sub_cb, true);
      if (include_empty_elements || (!base.isShelfExpressionEmpty(e))) {
        found.push(shelf_element);
      }
    });

    return found;
  };

  // This function should not be needed.
  // Changes in column should directly go to corresponding shelfElement
  function getLastShelfElementMatching(column) {
    if (!column || !column.isHierarchy()) {
      // Confining the usage of the function to hierarchy functions
      return;
    }

    const allShelfElements = base.getShelfElements(column.colsource());
    const hierarchyShelfElements = allShelfElements.filter(
      el => el.dataset_colname() === column.colname());

    return hierarchyShelfElements[hierarchyShelfElements.length - 1];
  }

  // @deprecated in favor of base.getShelfElement
  // include_empty_elements is true when cloning, deleting, or in filter shelf
  base.shelfElementCb = function(shelf_name, cb, include_empty_elements, no_param_sub) {
    shelfElementSimpleCb(shelf_name, function(shelf_element, i) {
      // Drop the shelf elements that have a default of '' to
      // handle optional dimension/measure through URL parameter passing
      var par_sub_cb;
      if (!no_param_sub) {
        par_sub_cb = base.substituteParams;
      }

      if (!shelf_element) {
        return base;
      }

      var e = shelf_element.getExpression(par_sub_cb , true);
      if (include_empty_elements || (!base.isShelfExpressionEmpty(e))) {
        cb(shelf_element, i);
      }
    }, no_param_sub);

    return base;
  };

  base.shelfExpressionCb = function(shelf_name, ignore_alias, cb) {
    base.shelfElementCb(shelf_name, function(shelf_element, i) {
      cb(shelf_element.getExpression(base.substituteParams, ignore_alias), i,
          shelf_element);
    });
    return base;
  };

  base.shelf_name_to_display_name = function(shelf_name) {
    var ret = shelf_name;
    base.shelf_data().some(function(shelf_element) {
      if (shelf_element.name === shelf_name) {
        ret = shelf_element.display_name;
        return true;
      }
      return false;
    });
    return ret;
  };

  base.shelfExpressionsCb = function(ignore_alias, cb) {
    base.shelf_data().forEach(function(shelf_element) {
      var shelf_name = shelf_element.name;
      base.shelfExpressionCb(shelf_name, ignore_alias, function(expr, i,
          shelf_element) {
        cb(shelf_name, expr, i, shelf_element);
      });
    });
    return base;
  };

  base.trimDecimal = function(offset, number) {
    var decimalOffset = Math.pow(10, offset);

    return Math.round(number * decimalOffset) / decimalOffset;
  };

  base.draw_trend_lines = function(svg, config, xy_values, trendline_color,
      trendline_decimal) {

    var trend_data = [];

    var t;
    // Account for offset when dimensions are used in scatter plots
    var x_dim_offset = config.x_dim_offset ? config.x_dim_offset : 0,
        y_dim_offset = config.y_dim_offset ? config.y_dim_offset : 0;

    if (trendline_color === 'true') {
      var xy_split = {};
      xy_values.forEach(function(xy_value) {
        var c = xy_value[2];
        if (!(c in xy_split)) {
          xy_split[c] = [];
        }
        xy_split[c].push(xy_value);
      });
      $.each(xy_split, function(c, v) {
        t = base.trend_data(v, config.width, x_dim_offset);
        var color;
        if (config.color_is_dim) {
          color = config.color(c);
        } else {
          color = config.palette ? config.palette[c] : c;
        }
        if (t) {
          t.color = color;
          trend_data.push(t);
        }
      });
    } else {
      t = base.trend_data(xy_values, config.width, x_dim_offset);
      if (t) {
        t.color = 'grey';
        trend_data.push(t);
      }
    }

    if (trend_data.length > 0) {

      var trendline = svg.selectAll('.trendline')
                           .data(trend_data);

       trendline.enter().append('line')
          .style('stroke', function(d) {
            return d.color;
          })
          .attr('class', 'viz-trendline')
          .attr('x1', function(d) { return d.x0 + x_dim_offset; })
          .attr('y1', function(d) { return d.y0 + y_dim_offset; })
          .attr('x2', function(d) { return d.xn + x_dim_offset; })
          .attr('y2', function(d) { return d.yn + y_dim_offset; });

        var trendlineHover = trendline.enter()
          .append('line')
          .style('stroke', function(d) {
            return d.color;
          })
          .attr('class', 'viz-trendline-hover')
          .attr('x1', function(d) { return d.x0 + x_dim_offset; })
          .attr('y1', function(d) { return d.y0 + y_dim_offset; })
          .attr('x2', function(d) { return d.xn + x_dim_offset; })
          .attr('y2', function(d) { return d.yn + y_dim_offset; });

        var tooltip = arc.tooltip()
                         .addSelection(trendlineHover)
                         .text_fn(function (d) {
                           var kvs = [];
                           var xName = config.x_col.colname(),
                               yName = config.y_col.colname(),
                               xscale = config.x_scales ?
                                        config.x_scales[0] : config.x_scale,
                               yscale = config.y_scales ?
                                        config.y_scales[0]: config.y_scale,
                               x1, x2, y1, y2, m, b, equation, equationRoot, caveat;
                           if (xscale.invert && yscale.invert) {
                             x1 = xscale.invert(d.x1);
                             x2 = xscale.invert(d.x2);
                             y1 = yscale.invert(d.y1);
                             y2 = yscale.invert(d.y2);
                           } else {
                             x1 = xscale.invert ? xscale.invert(d.x1) : 1;
                             x2 = xscale.invert ? xscale.invert(d.x2) : xscale.domain().length;
                             y1 = yscale.invert ? yscale.invert(d.y1) : 1;
                             y2 = yscale.invert ? yscale.invert(d.y2) : yscale.domain().length;
                             var dimName = xscale.invert ? yName : xName ;
                             caveat = '(every ' + dimName +
                                      ' mark is counted as 1 unit)';
                           }
                           m = base.trimDecimal(trendline_decimal, (y2 - y1) / (x2 - x1));
                           b = base.trimDecimal(trendline_decimal, y1 - m * x1);
                           equationRoot = yName + ' = ' + m +
                                          ' * ' + xName;
                           equation = b > 0 ?
                                      equationRoot + ' + ' + b :
                                      equationRoot + ' - ' + Math.abs(b);
                           kvs.push({key: 'equation', value: equation, info: caveat});
                          //  Add r-squared, p values, and t values if they exist
                           var leastSquaresCoeff = d.leastSquaresCoeff;
                           if (leastSquaresCoeff.errorMessage) {
                             kvs.push({
                               key: 'No statistical measures',
                               value: leastSquaresCoeff.errorMessage
                             });
                           } else {
                             var decimalOffset = trendline_decimal,
                                 rSquared = base.trimDecimal(decimalOffset, leastSquaresCoeff.rSquared),
                                 tValue = base.trimDecimal(decimalOffset, leastSquaresCoeff.tValue),
                                 pValue2 = base.trimDecimal(decimalOffset, leastSquaresCoeff.pValue),
                                 pValue1 = base.trimDecimal(decimalOffset, leastSquaresCoeff.pValue / 2),
                                 noFormat = function(val){ return val; },
                                 zeros = d3.format('.' + decimalOffset + 'f')(0),
                                 additionalPeriod = !zeros.includes('.') ? '.' : '',
                                 zeroText = `<${zeros}${additionalPeriod}5`;
                             kvs.push(
                              {
                                key: 'R-squared',
                                value: rSquared === 0 ? zeroText : rSquared,
                                formatter: noFormat,
                              },
                              {
                                key: 'p-value (two-tailed)',
                                value: pValue2 === 0 ? zeroText : pValue2,
                                formatter: noFormat,
                              },
                              {
                                key: 'p-value (one-tailed)',
                                value: pValue1 === 0 ? zeroText : pValue1,
                                formatter: noFormat,
                              },
                              {
                                key: 't-value',
                                value: tValue === 0 ? zeroText : tValue,
                                formatter: noFormat,
                              }
                             );
                           }
                           return base.tooltip_from_kvs(kvs);
                         })
                         .activate();
    } else {
      base.notificationMessage('Trendlines cannot be computed when there is only one value on the graph');
    }
  };

  base.trend_data = function(xy_values, width, x_dim_offset) {
    xy_values.sort(function(a, b) {
      if (a[0] < b[0]) {
        return -1;
      } else if (a[0] > b[0]) {
        return 1;
      } else {
        return 0;
      }
    });

    var xs_values = xy_values.map(function(d) { return d[0]; });
    var ys_values = xy_values.map(function(d) { return d[1]; });

    var xSeries = d3.range(1, xs_values.length + 1);
    var ySeries = ys_values;
    var leastSquaresCoeff = base.least_squares(xSeries, ySeries);

    if (leastSquaresCoeff) {
      // apply the results of the least squares regression
      var x1 = xy_values[0][0];
      var y1 = leastSquaresCoeff.slope + leastSquaresCoeff.intercept;
      var x2 = xy_values[xy_values.length-1][0];
      var y2 = leastSquaresCoeff.slope * xy_values.length + leastSquaresCoeff.intercept;
      // Get the first and last point of regression line within the graph
      var x0 = -x_dim_offset;
      var y0 = y2 - (((y2 - y1) / (x2 - x1)) * (x2 - x0));
      var xn = width - x_dim_offset;
      var yn = y2 - (((y2 - y1) / (x2 - x1)) * (x2 - xn));
      return {
        // Use for drawing the line
        x0: x0, y0: y0, xn: xn, yn: yn,
        // Use for calculating equation
        x1: x1, y1: y1, x2: x2, y2: y2,
        leastSquaresCoeff: leastSquaresCoeff,
      };
    } else {
      return undefined;
    }
  };

  // returns slope, intercept and r-square of the line
  // ssYY, ssXX, and ssXY were calculated using simplifications found on:
  // http://bl.ocks.org/benvandyke/8459843 -- tested to provide same slope and r-squared as full formula
  base.least_squares = function(xseries, yseries) {
    var reduceSumFunc = function(prev, cur) { return prev + cur; };

    var xBar = Number(xseries.reduce(reduceSumFunc)) / xseries.length;
    var yBar = Number(yseries.reduce(reduceSumFunc)) / yseries.length;

    var ssXX = xseries.map(function(d) { return Math.pow(d - xBar, 2); })
      .reduce(reduceSumFunc);

    var ssYY = yseries.map(function(d) { return Math.pow(d - yBar, 2); })
      .reduce(reduceSumFunc);

    var ssXY = xseries.map(function(d, i) { return (d - xBar) * (yseries[i] - yBar); })
      .reduce(reduceSumFunc);

    var slope = ssXY / ssXX;
    var intercept = yBar - (xBar * slope);
    var rSquared = Math.pow(ssXY, 2) / (ssXX * ssYY);
    var tValue, pValue, errorMessage,
        degreesOfFreedom = xseries.length - 2;
    // Don't calculate p value or t value if there's no degrees of freedom
    // It would be meaningless
    if (degreesOfFreedom === 0) {
      errorMessage = '2 values are not enough to affirm or reject this regression model';
    } else {
      tValue = Math.pow(rSquared, 1/2) * Math.pow(( degreesOfFreedom /(1 - rSquared)), 1/2);
      pValue = base.pValueCalulator(tValue, degreesOfFreedom);
    }

    if (isNaN(slope) || isNaN(intercept)) {
      return undefined;
    } else {
      return {slope: slope,
              intercept: intercept,
              rSquared: rSquared,
              pValue: pValue,
              tValue: tValue,
              errorMessage: errorMessage};
    }
  };

  // returns p value
  // code from Norm (Standard Normal Density) function on
  // https://home.ubalt.edu/ntsbarsh/business-stat/otherapplets/pvalues.htm
  base.pValueCalulator = function(tValue, degreesOfFreedom) {
    var Pi = Math.PI,
        PiD2 = Pi/2,
        floorDoF = Math.floor(degreesOfFreedom),
        ceilDoF = Math.ceil(degreesOfFreedom);
    var tFloor, tCeil;
    if (floorDoF !== degreesOfFreedom) {
      tFloor = Math.log( pValueCalulator(tValue, floorDoF));
      tCeil = Math.log( pValueCalulator(tValue, ceilDoF));
      return Math.exp( (ceilDoF - degreesOfFreedom) * tFloor +
             (degreesOfFreedom - floorDoF) * tCeil );
    }

    tValue = Math.abs(tValue);
    var w = tValue/Math.sqrt(degreesOfFreedom);
    var th=Math.atan(w);
    if(degreesOfFreedom === 1) {
      return 1 - th / PiD2 ;
    }
    var sth = Math.sin(th);
    var cth = Math.cos(th);

    if ( (degreesOfFreedom%2) === 1) {
      return 1 - (th + sth * cth *
             base.statCom(cth * cth, 2, degreesOfFreedom - 3, -1)) / PiD2;
    } else {
      return 1 - sth * base.statCom(cth * cth, 1, degreesOfFreedom - 3, -1);
    }
  };

  // No idea what this does....
  base.statCom = function(q, i, j, b) {
    var zz = 1;
    var z = zz;
    var k = i;
    while(k <= j) {
      zz = zz * q * k / (k - b);
      z = z + zz;
      k = k + 2;
    }
    return z;
  };

  // For getting shelf_name for addShelfDescription
  base.shelfNameFromId = function(shelfLabel) {
    var endIdx = shelfLabel.id.length - 8;
    return shelfLabel.id.substr(0, endIdx);
  };

  // For double-checking for correct shelf_data in addShelfDescription
  base.findShelf = function(shelfName) {
    for (var i = 0; i < base.shelf_data().length; i++) {
      if (base.shelf_data()[i].name === shelfName) {
        return base.shelf_data()[i];
      }
    }
  };

  // Parses popover html from array of paragraphs
  base.parsePopoverParagraphs = function(messages) {
    var html = '';
    messages.forEach(function (message) {
      html += '<p>' + message + '</p>';
    });
    return html;
  };

  // Defines all popover sentences except the tooltip and filter
  // Accepts a series of parameters (arrays or strings)
  //   Arrays: Name of stockMessages is the first element,
  //           second element will be appended to stock message
  //   Strings: Names of stockMessages
  base.createPopoverMessages = function () {
    var popoverMessage = '',
        stockMessages = {
        // SINGLE USE
            // Used in bar graphs
            colorBar: 'To stack bars by category, add a <b>dimension</b> to this shelf.',
            colorline: 'To stack lines by category, add a <b>dimension</b> to this shelf.',
            groupBar: 'To group bars by category, add a <b>dimension</b> to this shelf.',
            // Use in calendar heatmaps
            dateCalendar: 'Add a <b>date dimension</b> (eg: a date string in the format 03-05-2014) to show associated aggregates across time.',
            // Used in correlation heatmaps charts
            dimensionCorrelation: 'Add 2 <b>dimensions</b> from the data menu. The first dimension will be used to break the aggregation by row and the second by column.',
            // Use in crosstab
            // Need to specify 'rows.' or 'columns.' in argument
            aggregateCrosstab: 'Add <b>dimensions</b> from the data menu to break the aggregation by ',
            // Use in map
            geoMap1: 'Add a <b>',
            geoMap2: ' dimension</b> field to this shelf to designate location. If needed, click on a field in this shelf and use "Change Type" to indicate the Geo Type of that field.',
            // For packed bubbles only
            dimensionPacked: 'Add a <b>dimension</b> from the data menu to determine the number and name of bubbles to plot.',
            aggregatePacked: 'Add a <b>measure</b> from the data menu to determine the size of bubbles to plot.',
            // For word cloud
            dimensionWordCloud: 'Add a <b>dimension</b> from the data menu to determine the words to plot.',
            aggregateWordCloud: 'Add a <b>measure</b> from the data menu to determine the size of words to plot.',
            // For flow and funnel visuals only
            timePath: 'Add a <b>time measure</b> (like an event timestamp) to this shelf.',
            sessionPath: 'Add <b>unique identifers</b> to this shelf (eg: user id).',
            eventPath: 'Add in a pattern of events to be identified. The events need to first be created for the current dataset.',
            goalPath: 'Be sure to set an event as the goal in the dropdown menu.',
            measurePath: 'Add a <b>measure</b> to this shelf to specify the aggregate you want to apply at each stage of the path.',
            // Used in histograms
            aggregateHistogram: 'Add a <b>measure</b> from the data menu to generate a histogram for that measure.',
            // Used in combined line/bar graphs
            combinedXAxis: 'Add a <b>dimension</b> from the data menu to create lines and bars for values in this dimension.',
            barOnly: 'Add a <b>measure</b> from the data menu to create bars based on aggregates of this measure.',
            measuresShouldShareTheSameAxis: 'Multiple bar measures will share the same <b>primary</b> axis.',
            lineOnly: 'Add a <b>measure</b> from the data menu to create lines based on aggregates of this measure.',
            highlightOnly: 'Add a <b>measure</b> from the data menu to highlight aggregates of this measure.',
            // Used in pie charts
            dimensionPie: 'Add one or more <b>dimensions</b> from the data menu to create pie chart segments for each value in that dimension.' +
              ' If more than one <b>dimension</b> is added a sunburst will be plotted.',
            aggregatePie: 'Add a <b>measure</b> from the data menu to aggregate that measure for the size of each pie segment.',
            // For scatter plots only
            colorScatter: 'To change the color of the bubbles, add a <b>dimension</b> to this shelf.',
            sizeScatter: 'Add a <b>measure</b> to this shelf to change the size of the circles.',
            transitionScatter: 'Add a <b>dimension or measure</b> from the data menu to animate using the sequence defined by that dimensions/measure (eg: year).',
            // For box plots only
            dimensionBox: 'Add a <b>dimension</b> to this shelf to change the grouping of each box plot.',
            aggregateBoxSupport: 'Box plot calculations are optimized for Arcadia, Impala, Hive, and Spark SQL connections.',
            aggregateBox: 'Add a <b>measure</b> to this shelf to be plotted.',
            // Used in radial charts
            dimensionRadial: 'Add <b>dimension</b> from the data menu to create radials for each value in that dimension.',
            aggregateRadial: "Add <b>measure</b> from the data menu to create segments for each measure. Each segment's size will be determined by the aggregate of that measure.",
            // For single value only
            dimensionSingle: 'Add a <b>dimension</b> from the data menu to show a specific dimension value. A filter is recommended.',
            aggregateSingle: 'Add a <b>measure</b> from the data menu to show an aggregate.',
            onlyOneNeeded: 'Only the first dimension or measure will be shown.',
            // Used in table
            dimensionTable: 'Add <b>dimensions</b> from the data menu to add dimension columns.',
            aggregateTable: 'Add <b>measures</b> from the data menu to add aggregate columns.',
            // Used in treemap
            treemapDim: 'Add 2 <b>dimensions</b> from the data menu. The first field will define the <b>branches</b> (large rectangular areas) and the second field will define the <b>leaf nodes</b> (smaller rectangular areas).',
            treemapAg: "Add a <b>measure</b> field from the data menu to determine the leaf node's area.",
            // Used in network
            nodeSize: 'Add at most one <b>measure</b> from the data menu to determine the relative size of the nodes.',
            nodeColor: 'Add at most one <b>dimension or measure</b> from the data menu to color nodes by that value',
            linkSize: 'Add at most one <b>measure</b> from the data menu to determine the thickness of the links',
            linkColor: 'Add at most one <b>dimension or measure</b> from the data menu to color links',
            //for sparklines
            xSpark: 'Add one or more <b>dimensions</b> from the data menu.',
            ySpark: 'Add a <b>measure</b> from the data menu. Also add a <b>dimension</b> only if trellis is desired.',
            //Color shelves for world cloud and packed bubbles
            packedColor: 'Add a <b>measure</b> field to visually differentiate bubbles by color gradient based on the field value.',
            wordColor: 'Add a <b>measure</b> field to visually differentiate words by color gradient based on the field value.',
            //KPI
            kpiDimension: 'Add a <b>dimension</b> field from the data menu to display a specific dimension value.',
            kpiDimensionNotes: 'The first dimension value will display or the measure associated with the first dimension if a measure is provided.',
            // Chatbot
            embeddings: 'Add vector fields that contain embeddings. The first field that contains results will be returned.',
            embeddingResults: 'Add fields that should be sent as part of the chat prompt.',
            embeddingSource: 'Source information that will be included in the displayed results',
            embeddingWarning: 'Please be aware that any text and/or data you input into this AI visual, along with any associated metadata, is processed by the external third-party AI service provider.  You represent and warrant that you are authorized to use this AI visual, that you use it at your own risk, and that you will not disclose any sensitive, confidential, or personal information to it unless you have appropriate authorization and/or legal authority to do so.  You acknowledge and agree that Cloudera is not responsible for any data and information you share with and have processed by the third-party AI service provider.',

        // MULTIPLE USE
            // Used in bar graphs
            colorStackMeasure: "To stack multiple <b>measures</b>, add <b>measures</b> to this shelf.",
            // Generic info at the end -- let's people know if multiple fields are accepted
            acceptsMultiple: 'This shelf accepts multiple fields.',
            acceptsOne: 'This shelf is limited to one field.',
            // General info to let people know if multiple fields will just be stuffed into tooltip
            acceptsMultipleException: 'If more than 1 field is added, the additional fields will be displayed in the tooltip only.',
            acceptsAtMostTwo: 'This shelf can accept up to 2 fields.',
            drillShelf: 'Add <b>dimensions</b> to this shelf to limit the fields available for drilling-into to a smaller subset.',
            drillGeoShelf: 'For maps <b>dimensions</b> of only geo type are permitted.',
            emptyDrillShelf: 'If no dimension is added to shelf and setting to show Drill Into context menu is set, then all dimensions from dataset will be considered.',
            multipleGeoConfigs: 'If multiple valid map configurations are specified, the configuration with the highest resolution will be drawn.',
            // For x and y in bar and scatter plots
            xAxis: 'This is conceptually the x-axis.',
            yAxis: 'This is conceptually the y-axis.',
            // For bar graphs and sparkline
            xFlip: 'Use a <b>measure</b> for horizontal bars or a <b>dimension</b> for vertical bars.',
            yFlip: 'Use a <b>dimension</b> for horizontal bars or a <b>measure</b> for vertical bars.',
            xLineFlip: 'Use a <b>dimension</b> for a horizontal line or a <b>measure</b> for a vertical line.',
            yLineFlip: 'Use a <b>measure</b> for a horizontal line or a <b>dimension</b> for a vertical line.',
            // Used in chord, correlation flow charts
            dimensionChordCorrelation: 'Add 2 <b>dimensions</b> from the data menu to define the start and end for each connection in the plot.',
            aggregateChordCorrelation: 'Add a <b>measure</b> from the data menu. The size of each connection is determined by the aggregate of that measure.',
            // Use in calendar heatmaps, map, crosstab and correlation heatmaps
            // Need to specify "dates.", "locations.", "rows.", "columns.", or "rows and columns." in argument
            aggregateAcross: 'Add <b>measures</b> from the data menu to specify the values to be aggregated across ',
            // For trellis/axis (bar, scatter, sparkline)
            // Need to specify "rows." or "columns." in argument
            trellisFirst: 'If there are multiple fields, the <b>last</b> field will be the axis while the preceding fields will trellis the graphs in ',
            // For trellis only (calendar heatmap and map)
            // Need to specify "rows." or "columns." in argument
            trellisOnly: 'Add <b>dimensions</b> to this shelf to trellis the graphs in ',
            // Used in gauge, bullet, and kpi
            featureMeasure:'This is the feature measure, such as current value.',
            compareTo: 'Add a <b>measure</b> from the data menu to show a comparison aggregate, the target value.',
            qualitativeRanges: 'Add one or more <b>measures</b> to specify poor [darkest], satisfactory, and good  ranges. This sets the background colors.',
            qualitativeRangesGauge: 'Add one or more <b>measures</b> to specify poor, satisfactory, and good  ranges. This sets the background colors.',
            projection: 'Add a <b>measure</b> from the data menu to show the projected value based on current trends.',
            label: 'Add one or more <b>measures</b> or <b>dimensions</b> from the data menu to add a label to each graph (this will override the default, the measure).',
            startTime: 'Add a <b>time dimension</b> from the data menu to specify a starting point',
            endTime: 'Add a <b>time dimension</b> from the data menu to specify an ending point',
            yDimension: 'Add one or more <b>dimensions</b> that together uniquely indentify each element on the Y axis',
            axisLabel: 'Add a <b>measure</b> from the data menu to specify labels for every bar/point.'
          };

    for (var i = 0; i < arguments.length; i++) {
      var messageName = arguments[i];
      if (typeof(messageName) === 'object') {
        // If the argument is an array
        // first element: is the name of the message from "stockMessages"
        // second element: some text to be appended to the message (eg: personalize bars versus lines)
        popoverMessage += stockMessages[messageName[0]] + ' ' + messageName[1];
      } else {
        popoverMessage += stockMessages[messageName] + ' ';
      }
    }

    return popoverMessage;
  };

  // Adds popovers
  base.addPopovers = function (constraints, popoverMessages) {
    var defaultMessages = {
          // Generic shelves
          tooltip_shelf: ['Add <b>measure</b> fields from the data menu to add additional information to the graph\'s tooltip.',
                          base.createPopoverMessages('acceptsMultiple')],
          filters_shelf: ['Add <b>dimension or measure</b> fields from the data menu to set exclude or include criteria.',
                          base.createPopoverMessages('acceptsMultiple')],
        };

    $('.shelf-description')
      .popover({
        container: 'body', // makes popover stay on top
        html: true,
        sanitize: false,
        trigger: 'hover',
        placement: 'top',
        delay: {
          show: '500'
        },
        title: function () {
          var shelfName = base.shelfNameFromId(this),
              shelf = base.findShelf(shelfName),
              title = shelf.display_name + ' Shelf',
              minimum = constraints.minimums ? constraints.minimums[shelfName] : undefined,
              maximum = constraints.maximums ? constraints.maximums[shelfName] : undefined;

          if (minimum && maximum && minimum === maximum) {
            title += '<span class="popover-title-constraint pull-right"> (' + minimum + ' required)</span>';
          } else if (minimum && maximum) {
            title += '<span class="popover-title-constraint pull-right"> (' + minimum + '+ required, up to ' + maximum + ')</span>';
          } else if (minimum) {
            title += '<span class="popover-title-constraint pull-right"> (' + minimum + '+ required)</span>';
          } else if (maximum) {
            title += '<span class="popover-title-constraint pull-right"> (at most ' + maximum + ')</span>';
          }

          return title;
        },
        content: function () {
          var shelfName = base.shelfNameFromId(this),
              popoverMessage = popoverMessages ? popoverMessages[shelfName] : undefined;

          if (popoverMessage) {
            return base.parsePopoverParagraphs(popoverMessage);
          } else {
            var m = defaultMessages[shelfName];

            if (m) {
              return base.parsePopoverParagraphs(m);
            } else {
              return '';
            }
          }
        }
      });

      $('.shelf-warning')
      .popover({
        container: 'body',
        html: true,
        sanitize: false,
        trigger: 'hover',
        placement: 'top',
        delay: {
          show: '500'
        },
        title: () => 'Warning',
        // eslint-disable-next-line object-shorthand
        content: function () {
          const shelfName = base.shelfNameFromId(this);
          const popoverMessage = popoverMessages ? popoverMessages[`${shelfName}_warning`] : undefined;

          if (popoverMessage) {
            return base.parsePopoverParagraphs(popoverMessage);
          }
        }
      });
  };

  // Adds stars to shelves with minimum constraints
  base.starRequiredShelves = function (minimumConstraints) {
    for (var shelfName in minimumConstraints) {
      var $sel = $('#' + shelfName + '_popover');
      var $req = $sel.find('.required');
      if ($req.length === 0) {
        $sel.prepend('<span class="required">* </span>');
      }
    }
  };

  // Adds popovers and "required" stars
  base.addShelfDescription = function (constraints, popoverMessages) {
    base.addPopovers(constraints, popoverMessages);
    if (constraints.minimums) {
      base.starRequiredShelves(constraints.minimums);
    }
  };

  /*
   * This function is called when we update shelf information and want it reflected
   * within the visual across a re-paint.
   *
   * The current implementation of this function simply calls save_basics() and updates
   * 'lastKnownShelves' in that manner.
   */
  base.saveAwayShelfInfo = function() {
    base.save_basics({});
  };

  base.save_basics = function(settings) {
    arc.counters.increment('visual_save_basics');
    settings = base.report_save_shelves(base.shelf_data(), settings);
    settings = base.report_save_deriveddata(settings);
    settings = base.saveClickBehaviors(settings);

    trackLastKnownShelves(settings);

    // When we are called to save the visual, directly update the title and
    // subtitle here
    title = settings.report_title;
    subtitle = settings.report_subtitle;

    return settings;
  };

  var lastKnownShelves = {};

  function trackLastKnownShelves(settings) {
    lastKnownShelves = {};
    base.shelf_data().forEach(function(shelf_element) {
      var shelf_id = shelf_element.name;
      lastKnownShelves[shelf_id] = settings[shelf_id];
    });
  }

  base.restore_basics = function(settings, switchingVisualType) {
    trackLastKnownShelves(settings);

    base.clearAppliedParams();
    base.restoreClickBehaviors(settings);
    base.restore_deriveddata(settings);
    base.report_restore_shelves(base.shelf_data(), settings, switchingVisualType);
    return base;
  };

  base.restore_deriveddata = function(settings) {
    const derivedDataList = settings.report_derived_data;
    if (derivedDataList && derivedDataList instanceof Array) {
      derivedData.derivedDataList(derivedDataList);
    }
    return base;
  };

  base.report_save_deriveddata = function(info) {
    var savedList = [];
    if (derivedData) {
      const list = derivedData.derivedDataList();
      list.forEach(function(data) {
        if (data !== undefined) {
          savedList.push(data);
        }
      });
    }
    info.report_derived_data = savedList;
    return info;
  };

  // Called to return an object that can later be passed into the
  // arc.report_restore_shelves to restore state
  base.report_save_shelves = function(shelf_data, info) {
    info.core = info.core || {};
    info.core.viz_type = viz_type;

    base.shelfTrackerInstance().info(info)
                               .viz_lib(base)
                               .shelf_data(base.shelf_data())
                               .save();

    shelf_data.forEach(function(shelf_element) {
      var shelf_name = shelf_element.name;
      var shelf_info = [];
      base.shelfElementCb(shelf_name, function(shelf_element) {
        var s = {};
        shelf_element.save(s);
        shelf_info.push(s);
      }, true);

      info[shelf_name] = shelf_info;
    });

    // Add info to history now that info has been updated
    base.shelfTrackerInstance().addHistory(info.core.viz_type, info);

    // This can't be called unless supermodal is open
    // (except perhaps for filter widgets in dashboard)
    // However, ass base.inModal() just in case
    if (arc.globals.is_edit && base.inModal()) {
      sample_pct = $(base.dataset_elem()).text().trim();
    }

    info.sample_pct = sample_pct;

    if (info.user_settings === undefined) {
      info.user_settings = {};
    }

    user_settings.settings().forEach(function(user_setting) {
      info.user_settings[user_setting.id()] = user_setting.value();
    });

    info.settings = {};
    info.settings.colorSettings = colorSettings;
    if (colors) {
      info.settings.colors = colors;
    }

    if (graded_colors) {
      info.settings.graded_colors = graded_colors;
    }

    if (contrast_colors) {
      info.settings.contrast_colors = contrast_colors;
    }
    info.settings.reverse_colors = reverse_colors;

    if (arc.globals.is_edit) {
      info.report_title = $('#' + title_elem).text();
      info.report_subtitle = $('#' + subtitle_elem).text();
    } else {
      info.report_title = title;
      info.report_subtitle = subtitle;
    }

    if (solr_query) {
      info.solr_query = solr_query;
    }
    return info;
  };

  base.restoreUserSettings = function(inValues) {
    if (inValues === undefined) {
      return;
    }

    // Here is it very important to have the correct order. Those settings which
    // influnces other settings' state should be updated later than its dependent
    // to have the correct desired state of the settings!
    user_settings.getSortedSettingsByDependency().forEach(
      (user_setting) => {
        user_setting.viz_lib(base);
        const val = inValues[user_setting.id()];
        if (val !== undefined) {
          user_setting.value(val);
        }
      }
    );
  };

  /*
   * In canyonlands and earlier, segments selection for filtering was done via a distinct tab
   * in the visual edit modal.
   * Now, segment selections for filtering is done via a shelf element in the Filters shelf
   * The code here upgrades earlier versions.
   */
  function upgradeSegmentsIntoFiltersShelf(report_info) {
    if (report_info.selected_segments && report_info.selected_segments.length) {
      dataset_segments.selected_segments(report_info.selected_segments);
      const filterShelfElement = arc.report.shelf.segment_element();
      filterShelfElement.dataset(dataset)
                        .dataconnection_type(dataset.dataconnection_type())
                        .custom_expr(JSON.stringify(report_info.selected_segments.map(function(s) {
                          return [Number(s.id), !s.negate];
                        })));

      const d = {};
      filterShelfElement.save(d);
      d.filter_column = arc.report.segment_name;
      d.filter_input = {
        value_picker: {
          expr: filterShelfElement.custom_expr(),
        },
      };

      report_info.filters_shelf.push(d);
      report_info.selected_segments = undefined;
    }
  }

  base.setSamplingPct = function (_) {
    if (arguments.length) {
      // Mark visual as needing to be saved since sampling has changed
      base.needs_save(true);
      sample_pct = _;
    }

    arc.report.setSamplingPct(sample_pct);
  };

  base.report_restore_shelves = function(shelf_data, report_info, switchingVisualType) {
    upgradeSegmentsIntoFiltersShelf(report_info);

    base.shelfTrackerInstance().info(report_info)
                               .viz_lib(base)
                               .shelf_data(base.shelf_data())
                               .restore(switchingVisualType);

    trackLastKnownShelves(report_info);

    if (!switchingVisualType) {
      user_settings.settings().forEach(function(user_setting) {
        var v = user_setting.valueBeforeRestore();
        if (v !== undefined) {
          user_setting.value(v);
        }
      });
    }

    if (arc.globals.is_edit) {
      // Reapply permissions to layout on redraw
      // This can happen when user is editing the visual by changing the type, for example
      const datasetPermissions = base.dataset().dataset_perms();
      arc.misc.updateDivForPerms(datasetPermissions, 'dataset', $superModal);
    }

      if (report_info?.settings?.colorSettings) {
        colorSettings = report_info.settings.colorSettings;
      }

      if (report_info?.settings?.graded_colors) {
        graded_colors = report_info.settings.graded_colors;
      }

      if (report_info?.settings?.contrast_colors) {
        contrast_colors = report_info.settings.contrast_colors;
      }

      if (report_info?.user_settings?.color_palette) {
        const colorPaletteValue = JSON.parse(report_info.user_settings.color_palette);
        colors = arc.colorpalette.getOrderedColorPalette(colorPaletteValue);
      }

      if (report_info?.user_settings?.link_color_palette) {
        const linkColorsSetting = JSON.parse(report_info.user_settings.color_palette);
        linkColors = arc.colorpalette.getOrderedColorPalette(linkColorsSetting);
      }

      reverse_colors = !!report_info?.settings?.reverse_colors;
      if (report_info?.settings?.scale_type) {
        // For backward compatibility, we set the value of this
        // specific setting from an older setting
        var s = user_settings.find_by_id('scale_type');
        if (s) {
          s.value(report_info.settings.scale_type);
        }
        delete report_info.settings.scale_type;
      }

      if (report_info?.settings?.settings_colors_enabled !== undefined) {
        // For backward compatibility, we set the value of this
        // specific setting from an older setting
        var sc = user_settings.find_by_id('colors_enabled');
        if (sc) {
          if (report_info.settings.settings_colors_enabled) {
            sc.value('true');
          } else {
            sc.value('false');
          }
        }

        delete report_info.settings.colors_enabled;
      }

    sample_pct = 'Off';
    if (report_info.sample_pct) {
      sample_pct = report_info.sample_pct;
    }

    solr_query = report_info.solr_query;

    paintShelves(shelves, report_info);

    title = report_info.report_title;
    subtitle = report_info.report_subtitle;

    let report = arc.globals.reports.getReport(base.report_id()) ||
                 arc.report.getPrimaryReport(),
        visual_params_history;
    if (arc.globals.is_dashboard &&
      base.report_id() === report.report_id()) {
      visual_params_history = report.visual_params_history();
      if (visual_params_history) {
        let params_data = visual_params_history.current_params()[0] || [];
        if (report.viz()) {
          report.viz().pivotParams(params_data);
        }
      }
    } else if (!arc.globals.is_dashboard) {
      visual_params_history = report.visual_params_history();
      if (!visual_params_history) {
        visual_params_history = arc.report.params_history();
        visual_params_history.dashboard(report);
        report.visual_params_history(visual_params_history);
      }
      visual_params_history.setupControls(base.inModal());
      visual_params_history.baseParamsl(arc.globals.params.getCopy());
      base.pivotParams(visual_params_history.current_params()[0]);
    }

    base.connectDataConnectionToShelves();
    base.restoreUserSettings(report_info.user_settings);
    return base;
  };

  function paintShelves(shelves, shelfElementData) {
    if (!shelves) {
      return;
    }

    base.shelf_data().forEach((shelf_element) => {
      const shelf_id = shelf_element.name;
      let shelf_info = shelfElementData[shelf_id];

      if (shelf_info) {
        if (shelf_element.isSensitive && base.dataset()) {
          const { columns } = JSON.parse(base.dataset().dataset_info())[0];
          const sensitiveColumns = {};
          columns.forEach((col) => {
            if (col.isSensitive) {
              sensitiveColumns[col.alias] = true;
            }
          });

          shelf_info = shelf_info.filter(
            (item) => !sensitiveColumns[item.dataset_colname]
          );
        }

        shelves.set(shelf_id, shelf_info);
      }
    });

    if (solr_query) {
      $('#' + shelves_dom_id + ' .searchbox').val(solr_query);
    }

    shelves.repaintOrder();

    shelves.highlightDefaultShelf();
  }

  base.getClickBehavior = function(marker_name) {
    if (marker_name in click_behaviors) {
      return click_behaviors[marker_name];
    }
    return null;
  };

  base.click_behavior_cb = function(_) {
    if (!arguments.length) {
      if (arc.globals.click_behavior_cb) {
        return arc.globals.click_behavior_cb;
      } else {
        return base.click_behavior_cb_default;
      }
    }

    click_behavior_cb = _;
    return base;
  };

  base.click_behavior_cb_default = function(datum, index, click_behavior,
      params_fn, viz, elem, passedParams) {

    let url = `${arc.frontend.globals.urls.viewDashboardUrl}/${click_behavior.report_id}`;
    if (params_fn && click_behavior.shouldPassParams) {
      let currParams = passedParams || params_fn(datum, index, elem, !base.sendAggregateParameters());
      let allScopes = [];
      currParams = currParams.map(function(param) {
        let keyValue = `param.${encodeURIComponent(param.key)}=${encodeURIComponent(param.value)}`;
        if (param.scope) {
          if (!param.scope.filterId && param.source_visual_id) {
            param.scope.filterId = param.source_visual_id;
          }

          allScopes.push(param.scope);
        }

        return keyValue;
      });

      if (allScopes.length) {
        let allScopesValue = 'scope=' + JSON.stringify(allScopes);

        currParams.push(allScopesValue);
      }

      url = arc.misc.dedupURLParams(currParams, url);
    }

    const openTarget = click_behavior.openTarget || 'current_tab';
    const targetAttr = (openTarget === 'new_tab' || arc.globals.is_edit) ? '_blank' : '_self';

    if (openTarget !== 'new_window') {
      arc.misc.openInNewWindow(url, targetAttr);
    } else {
      arc.misc.openInNewWindow(url, openTarget);
    }
  };

  base.setClickBehavior = function(marker_name, click_behavior) {
    if (click_behavior) {
      click_behaviors[marker_name] = click_behavior;
      return base;
    }
    delete click_behavior[marker_name];
    return base;
  };

  base.saveClickBehaviors = function(bar_info) {
    bar_info.click_behaviors = click_behaviors;
    return bar_info;
  };

  base.restoreClickBehaviors = function(bar_info) {
    if (bar_info.click_behaviors) {
      click_behaviors = bar_info.click_behaviors;
    } else {
      click_behaviors = {};
    }
    return base;
  };

  base.report_id = function(_) {
    if (!arguments.length) return report_id;
    report_id = _;
    return base;
  };

  base.unique_report_id = function() {
    return unique_report_id;
  };

  base.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    return base;
  };

  base.dataset_events = function(_) {
    if (!arguments.length) return dataset_events;
    dataset_events = _;
    return base;
  };

  base.dataset_segments = function(_) {
    if (!arguments.length) return dataset_segments;
    dataset_segments = _;
    if (filtersaver) {
      filtersaver.dataset_segments(dataset_segments);
    }
    return base;
  };

  base.save_segment_cb = function(_) {
    if (!arguments.length) return save_segment_cb;
    save_segment_cb = _;
    return base;
  };

  base.redraw_cb = function(_) {
    if (!arguments.length) return redraw_cb;
    redraw_cb = _;
    return base;
  };

  base.dataconnection_type = function() {
    if (!dataset) {
      return undefined;
    }
    return dataset.dataconnection_type();
  };

  base.dataset = function(_) {
    if (!arguments.length) return dataset;
    dataset = _;
    return base;
  };

  base.dataset_id = function() {
    // When unit testing visualizations, they may not be associated with
    // datasets. Handle un-set datasets for testing use-cases
    if (dataset) {
      return dataset.dataset_id();
    } else {
      return -1;
    }
  };

  base.editAlertMessages = function(_) {
    if (!arguments.length) return editAlertMessages;
    editAlertMessages = _;
    return editAlertMessages;
  };

  base.viewAlertMessages = function(_) {
    if (!arguments.length) return viewAlertMessages;
    viewAlertMessages = _;
    return viewAlertMessages;
  };

  base.addMessage = function(msg, showOnView = false) {
    if (!editAlertMessages.includes(msg)) {
      editAlertMessages.push(msg);
    }
    if (showOnView && !viewAlertMessages.includes(msg)) {
      viewAlertMessages.push(msg);
    }

    return base;
  };

  base.resetMessages = function() {
    editAlertMessages = [];
    viewAlertMessages = [];
  };

  base.getParams = function() {
    if (dashboard) {
      return dashboard.getDashboardParamsTracker();
    }

    return params;
  };

  base.params = function(_) {
    if (!arguments.length) return params;
    params = _;
    return base;
  };

  base.prevParams = function(_) {
    if (!arguments.length) return prevParams;
    prevParams = _;
    return base;
  };

  base.chart_type = function(_) {
    if (!arguments.length) return chart_type;
    chart_type = _;
    viz_type = _;
    return base;
  };

  base.saveEdit = function(new_text) {
    dialog_selection.text(new_text);
    return base;
  };

  base.saveClickBehavior = function(marker_name, click_behavior_object) {
    click_behaviors.marker_name = click_behavior_object;
    return base;
  };

  var shelf_data = [
    {
      name: 'dimensions_shelf',
      display_name: 'Dimensions',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.DIMENSION
    },
    {
      name: 'aggregates_shelf',
      display_name: 'Measures',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'tooltip_shelf',
      display_name: 'Tooltips',
      shelf_type: arc.report.shelf_types.COLUMN,
      column_type: arc.report.columnTypeEnum.AGGREGATE,
    },
    {
      name: 'filters_shelf',
      display_name: 'Filters',
      display_html: arc.report.filters_shelf_html(),
      shelf_type: arc.report.shelf_types.PREDICATE,
      column_type: arc.report.columnTypeEnum.UNKNOWN,
      onShelfListChange: arc.report.updateFilterShelfLabel,
    }
  ];

  base.shelf_data = function(_) {
    if (!arguments.length) return shelf_data;
    shelf_data = _;
    return base;
  };

  base.loader = function(_) {
    if (!arguments.length) return loader;
    loader = _;
    if (!enableLoader) {
      loader.enable(false);
    }
    return base;
  };

  base.enableLoader = function(_) {
    if (!arguments.length) return enableLoader;
    enableLoader = _;
    if (loader) {
      loader.enabled(enableLoader);
    }
    return base;
  };

  base.title_elem = function(_) {
    if (!arguments.length) return title_elem;
    title_elem = _;

    var $s = $('#' + title_elem);
    $s.off('keyup')
      .on('keyup', function() {
        var t = $s.text();
        if (title !== t) {
          base.page_needs_save();
          title = t;
        }
      });

    return base;
  };

  base.subtitle_elem = function(_) {
    if (!arguments.length) return subtitle_elem;
    subtitle_elem = _;

    var $s = $('#' + subtitle_elem);
    $s.off('keyup')
      .on('keyup', function() {
        var t = $s.text();
        if (subtitle !== t) {
          base.page_needs_save();
          subtitle = t;
        }
      });

    return base;
  };

  base.chart_div = function(_) {
    if (!arguments.length) return chart_div;
    chart_div = _;
    arc.errors.setupVisualErrorPopover(base, chart_div);
    return base;
  };

  base.aggregateValue = function(data) {
    var re = /^\s*having\s*\((.*)\)\s*$/i;
    var l = re.exec(data);
    if (l === null) {
      return data;
    }
    return l[1];
  };

  base.shelvesContainSegmentName = function() {
    var has_arc_segment = false;
    var ret_shelf_elem;
    base.shelf_data().forEach(function(shelf) {
      if (!has_arc_segment) {
        base.shelfElementCb(shelf.name, function(shelf_elem) {
          if (!has_arc_segment) {
            if (shelf_elem.isDatasetSegment()) {
              ret_shelf_elem = shelf_elem;
              has_arc_segment = true;
            }
          }
        });
      }
    });

    return ret_shelf_elem;
  };

  base.finalizeSql = function(in_sb) {
    var shelf_elem = base.shelvesContainSegmentName();
    if (!shelf_elem) {
      return in_sb;
    }

    var selected_segments;
    if (shelf_elem.filter_data().length === 0) {
      selected_segments = selectedSegmentsFromFiltersShelf();
    } else {
      selected_segments = dataset_segments.segments_from_id(
          shelf_elem.filter_data());
    }

    var arc_segment_indices = [];
    in_sb.projections().forEach(function(p, i) {
      if (p.startsWith(arc.report.segment_name)) {
        arc_segment_indices.push(i);
      }
    });

    if (arc_segment_indices.length === 0) {
      return in_sb;
    }

    if (selected_segments.length === 0) {
      arc_segment_indices.forEach(function(arc_segment_index) {
        in_sb.setProjectionAtIndex('NULL', arc_segment_index);
      });

      return in_sb;
    }

    // When segments are based on a joined dimension table, getting segment
    // and record count gives weird results as we don't join the base table
    // so here we force the base table to be included
    if (in_sb.dataset()) {
      var tn = in_sb.dataset().tables()[0].tablename();
      if (tn && (in_sb.tablenames().indexOf(tn) === -1)) {
        in_sb.tablenames().push(tn);
      }
    }

    var ub = arc.unionBuilder();
    selected_segments.forEach(function(segment) {
      var sb = in_sb.clone();
      arc_segment_indices.forEach(function(arc_segment_index) {
        if (segment.negate) {
          sb.setProjectionAtIndex(
              `'not ${segment.data.name}' as '${arc.report.segment_display_name}' `,
              arc_segment_index);
        } else {
          sb.setProjectionAtIndex(
              `'${segment.data.name}' as '${arc.report.segment_display_name}' `,
              arc_segment_index);
        }
      });

      dataset_segments.applyToSelectBuilder(sb, [segment], '_arc_u');
      ub.addSb(sb);
    });

    if ((in_sb.limit() === null) &&
        (in_sb.orderBys().length === 0)) {
      return ub;
    }

    return ub;

    /* -- Still not working (need alias fix)
    // We should be applying the order and limit clauses after a union.
    // DJC -- note that this requires each projection to have '... as `alias`'
    //        which is currently the case but we need to fix this once
    //        projections become objects with {expression, alias} fields.

    var outer_sel = arc.selectBuilder();
    outer_sel.tablename('(' + ub.getSql() + ') o');
    outer_sel.orderBys(in_sb.orderBys());
    outer_sel.limit(in_sb.limit());
    var plist = in_sb.projections();
    plist.forEach(function(p, idx) {
      if (arc_segment_indices.indexOf(idx) !== -1) {
        outer_sel.addProjection(arc.report.segment_display_name);
      } else {
        var m = p.match(/^(.*)\s+as (`.+`)$/);
        if (m) {
          outer_sel.addProjection(m[2]);
        } else {
          outer_sel.addProjection(p);
        }
      }
    });

    return outer_sel;
    */
  };

  base.getSelectBuilder = function(no_limit) {
    var maxRows = base.maxRowCount();
    var sb = arc.selectBuilder();

    if (!no_limit) {
      // We cannot handle a very large number of rows on the client
      // Limit the max rows we get back from the system
      sb.limit(maxRows);
    }

    sb.dataset(dataset);
    if (dataset_segments) {
      dataset_segments.applyToSelectBuilder(sb);
    }
    return sb;
  };

  base.getFilteredRequest = function() {
    var rb = arc.requestBuilder()
                .report(base);

    if (dataset) {
      rb.datasetId(dataset.dataset_id());
    }

    base.addFiltersToRb(rb);
    return rb;
  };

  base.getRequestBuilder = function(no_limit) {
    var rb = arc.requestBuilder()
                .report(base);

    var maxRows = base.maxRowCount();

    if (dataset) {
      rb.datasetId(dataset.dataset_id());
    }

    if (!no_limit) {
      rb.limit(maxRows);
    }

    rb.nulls_order(base.nulls_order());

    addFilterSegmentsToRb(rb);

    return rb;
  };

  base.addLimit = function(container) {
    if (!container) {
      container = '#' + shelves_dom_id;
    }

    // Paint a visual representation of the limit shelf
    // only if the shelf needs to be visible i.e. when we are
    // drawing the visual within a edit modal. We do this
    // by looking for a 'app-limit-container' peer element,
    // which only exists in the edit modal...
    var $appLimitContainer =
        $(container).parent()
                    .find('> .app-limit-container');

    // ...and then only drawing the limit shelf if such a peer exists
    if ($appLimitContainer.length > 0) {
      $appLimitContainer.empty()
                        .append($('#app-limit-template').html())
                        .removeClass('arc-invisible');

      limit_spinner = $appLimitContainer.find('input');
    }

    // For new reports, default to a small number of rows
    if (report_id === arc.report.REPORTID_NEW) {
      setLimit(limit_default);
    } else if (limit_value !== undefined) {
      setLimit(limit_value);
    }

    return base;
  };

  function setLimit(limit) {
    if (limit_spinner) {
      limit_spinner.off('change');
    }

    limit_value = limit;
    if (limit_spinner) {
      limit_spinner.val(limit);
      limit_spinner.on('change', function() {
        limit_value = limit_spinner.val();
        base.page_needs_save();
      });
    }
  }

  base.saveLimit = function(info) {
    base.saveLimitBase(info);
    return info;
  };

  base.restoreLimit = function(info) {
    if (info.limit !== undefined) {
      setLimit(info.limit);
    }

    return base;
  };

  base.setSbLimit = function(sb) {
    base.saveLimitBase(sb, true /*substituteParams*/);
    return base;
  };

  base.setRbLimit = function(rb) {
    base.saveLimitBase(rb, true /*substituteParams*/);
    return base;
  };

  base.saveLimitBase = function(functionOrObject, substituteParams) {
    var newLimit,
        limit = limit_value;

    if (limit === undefined) {
      newLimit = undefined;
    } else if (isNaN(limit)) {
      if (limit.indexOf('<<') !== -1 && limit.indexOf('>>') !== -1 ) {
        if (substituteParams) {
          newLimit = base.substituteParams(limit);
          if (isNaN(newLimit)) {
            newLimit = undefined;
          }
        } else {
          newLimit = limit;
        }
      }
    } else {
      newLimit = limit;
    }

    if (!newLimit) {
      newLimit = base.maxRowCount();
    } else {
      const maxRows = base.maxRowCount();
      newLimit = Math.min(Number(newLimit), maxRows);
    }

    if (typeof functionOrObject === 'function') {
      functionOrObject.limit(newLimit);
    } else {
      functionOrObject.limit = newLimit;
    }
  };

  base.sb_colnames = function(_) {
    if (!arguments.length) return sb_colnames;
    sb_colnames = _;
    return base;
  };

  base.sb_coltypes = function(_) {
    if (!arguments.length) return sb_coltypes;
    sb_coltypes = _;
    return base;
  };

  base.sb_columns = function(_) {
    if (!arguments.length) return sb_columns;
    sb_columns = _;
    return base;
  };

  base.sb_rows = function(_) {
    if (!arguments.length) return sb_rows;
    sb_rows = _;
    return base;
  };

  base.sqlQuery = function(_) {
    if (!arguments.length) return sql_query;
    sql_query = _;
    return base;
  };

  base.dsreq = function(_) {
    if (!arguments.length) return dsreq;
    dsreq = _;
    return base;
  };

  base.generateSql = function() {
    return base;
  };

  base.substituteParams = function(sql_cmd, extra_argsl, leave_code_alone) {
    var argsl = [$('#' + title_elem).text(), $('#' + subtitle_elem).text(),
        title, subtitle];
    if (extra_argsl !== undefined) {
      extra_argsl.forEach(function(arg) {
        argsl.push(arg);
      });
    }

    return base.getParams().substitute(sql_cmd, argsl, leave_code_alone,
        parametersSubstituted);
  };

  base.translatedDisplayFormat = function(fmt) {
    if (typeof fmt === 'string') {
      fmt = base.substituteParams(fmt);
    } else {
      fmt.result = base.substituteParams(fmt.result);
    }

    return arc.utils.translateDisplayFormat(fmt, dataset);
  };

  base.setTitleText = function() {
    var title_text, subtitle_text;

    if (arc.globals.is_edit) {
      title_text = title;
      subtitle_text = subtitle;
    } else {
      var p = base.getParams();
      title_text = p.substitute(title, [title, subtitle]);
      subtitle_text = p.substitute(subtitle, [title, subtitle]);
    }

    displayTitle(title_text);
    displaySubtitle(subtitle_text);
  };

  base.updateCsvSettings = function() {
    if (isFilterOrTimeControl(viz_type)) {
      if (chart_div) {
        $('#' + chart_div.slice(0,-2)).find('.fa-arrow-circle-o-down').addClass('arc-invisible');
      }

      return;
    }
  };

  base.dataset_elem = function() {
    return '#dataset-sample-display';
  };

  base.hideLoader = function() {
    if (loader) {
      loader.hideProgress();
    } else {
      arc.loader.refreshDone();
    }

    return base;
  };

  function failCb() {
    base.notificationMessage('Could not load detailed data');
  }

  base.getQueryCmd = function() {
    let queryCmd = arc.data.queryCmd()
                      .sync(false);

    if (sample_pct !== 'Off') {
      queryCmd.sample_pct(sample_pct);
    }


    // Here in view mode (i.e. not edit) and when we have a dashboard, we deliver
    // the parameters in the updated format -- as a list of objects rather than a
    // simple key/value dictionary.
    //
    // This is to support the server-side data request construction/validation feature
    // as the parameter objects contain scope information.
    //
    let viz_params = {};
    if (!arc.globals.is_edit && arc.globals.ms_dashboard) {
      viz_params = arc.globals.ms_dashboard.params_history().current_params()[0];
    } else {
      viz_params = base.getParams().getkvs();
    }
    queryCmd.vizParams(JSON.stringify(viz_params));

    if (report_id) {
      queryCmd.widget_id(report_id);
      queryCmd.widget_name(title);
    }

    if (dataset) {
      queryCmd.dataset_id(dataset.dataset_id());
      queryCmd.dataset_name(dataset.dataset_name());

      if (dataset.dataconnection_id()) {
        queryCmd.dataconnection_id(dataset.dataconnection_id());
        queryCmd.dataconnection_name(dataset.dataconnection_name());
      }
    }

    if (dashboard) {
      queryCmd.app_id(dashboard.dashboard_id());
      queryCmd.app_name(dashboard.dashboard_name());
    }

    if (arc.globals.appgroup) {
      queryCmd.appgroup_id(arc.globals.appgroup.appgroup_id());
      queryCmd.appgroup_name(arc.globals.appgroup.appgroup_name());
    }

    return queryCmd;
  };

  base.runQuery = function(rb, successCb, failureCb, errorReportingCb) {
    let queryCmd = base.getQueryCmd();
    let dsreq = JSON.stringify(rb.dsreq());
    queryCmd.dsreq(dsreq);
    let queryParams = queryCmd.getParams();
    const queryFailedCb = failureCb ? failureCb : failCb;

    let queryPriority = 0;
    if (base.report()) {
      queryPriority = base.report().queryPriority();
    }

    queryParams.queryPriority = queryPriority;

    // Use a baseQuery object to actually run the query
    const qstate = activeQueryManager.queryStateAllocate(undefined, undefined, errorReportingCb);
    // name shadowing
    const baseQueryLocal = arc.data.baseQuery(qstate, successCb, queryFailedCb, undefined);
    baseQueryLocal.run(queryParams);

    return baseQueryLocal;
  };

  base.fetchData = function(config) {
    let {
      cmd,
      successCb,
      failCb,
      dsreq,
      qstate,
    } = config;

    return base.runSqlCmd(cmd, successCb, dsreq, qstate, failCb);
  };

  base.runSqlCmd = function(cmd, successCb, _dsreq, inQstate, _failCb, paramsTransformer,
    errorReportingCb, chunkCb) {
    let querySpecified = true;

    /*
    NOTE: this is not the usual workflow.
    The already existing selectBuilder or requestBuilder, and the way we parse expressions,
    fields in a sql command works with a dataset (backend parsing).
    Usually we identify the dataset and the related query by its id.
    To able to do the same with unsaved queryies (as dataset),
    I add the dataset_object to the request,
    During DataProfiling for example, where we don't want to save a query
    as a dataset on the SQL page,
    but would like to profile it, we need this option.
     */
    if (!dataset.dataset_id()) {
      const parsed_dsreq = JSON.parse(_dsreq);
      parsed_dsreq.dataset_object = dataset.buildInfo();
      _dsreq = JSON.stringify(parsed_dsreq);
    }
    if (!cmd) {
      querySpecified = false;
      cmd = JSON.stringify(_dsreq);
    }

    last_run_sql = cmd;

    // In build mode, read the sampling pct from the ui element
    if (arc.globals.is_edit && base.inModal()) {
      sample_pct = $(base.dataset_elem()).text().trim();
    }

    // Allocate a query state tracker
    const qstate = activeQueryManager.queryStateAllocate(base, cmd, errorReportingCb);

    if (inQstate) {
      qstate.qhandle(inQstate.qhandle());
    }

    base.dsreq(_dsreq);

    let queryCmd = base.getQueryCmd();

    queryCmd.dsreq(_dsreq);
    if (querySpecified) {
      queryCmd.query(cmd);
    }

    queryCmd.skip_dsreq(base.skip_dsreq());

    var params = queryCmd.getParams();

    if (loader) {
      loader.cancelCb(function() {
              qstate.cancel();
            });
      loader.reset();
      loader.showProgress();
    } else {
      arc.loader.showRefreshing();
    }

    let queryPriority = 0;
    if (base.report()) {
      queryPriority = base.report().queryPriority();
    }

    if (paramsTransformer) {
      params = paramsTransformer(params);
    }

    params.queryPriority = queryPriority;

    // Use a baseQuery object to actually run the query
    baseQuery = arc.data.baseQuery(qstate, successCb, _failCb, base, chunkCb);
    baseQuery.run(params);

    return base;
  };

  base.unselectAll = function() {
    d3.select('#' + base.chart_div())
      .selectAll('.viz-partially-visible')
      .classed('viz-partially-visible', false)
      .classed('viz-fully-visible', true);
    if (unhighlight_cb) {
      unhighlight_cb();
    }
  };

  base.unselectOnClick = function() {
    d3.select('#' + base.chart_div())
      .on('click.outside', function() {
        // There was a click outside the selections - unselect
        // any selections
        base.unselectAll();
      });
  };

  const realTimeFunctions = [
                                'now',
                                'current_timestamp',
                                'timeofday',
                                'unix_timestamp',
                                'current_database',
                                'effective_user',
                                'pid',
                                'user',
                                'uuid',
                                'version',
                                'rand',
                              ];
  const invalidSQLCachePattern = RegExp(realTimeFunctions.map(f => `${f} ?\\( *\\)` ).join('|'),'i');

  base.loadData = function(sql_cmd, force_redraw, dsreq, resolve, reject) {
    if (!resolve) {
      resolve = () => {};
    }

    if (!reject) {
      reject = () => {};
    }

    if (!sql_cmd) {
      // Initialize counters even if there's no sql since we're technically done
      // with dataapi call and done with draw
      arc.counters.increment('dataapi_start');
      arc.counters.increment('dataapi_end');
      arc.counters.increment('viz_drawVisual_start');
      arc.counters.increment('viz_drawVisual_end');
      resolve();
      return base;
    }

    base.dsreq(dsreq);
    // In build mode, read the sampling pct from the ui element
    if (arc.globals.is_edit && base.inModal()) {
      sample_pct = $(base.dataset_elem()).text().trim();
    }

    // We are being asked to load data.
    // The complication is that we may have a set of parameters
    // in the generated sql that need to be substituted before
    // being sent off for execution.
    // Default parameters may be stored in the title, or in the sql
    // command. Call a helper function with both parameters
    //
    var real_sql_cmd = base.substituteParams(sql_cmd);

    // See if we have an active query
    var qstate = activeQueryManager.activeQuery();
    if (qstate) {

      // Nothing to do if the active query matches this one
      if (qstate.query() === real_sql_cmd) {
        resolve();
        return base;
      }

      // Otherwise set the query state to cancel and flag no more active query
      qstate.cancel();
      activeQueryManager.markInactive(qstate.seqnum());
    }

    // Build table signature to pick up 'table from SQL' statement that can be
    // run through the parameter substitution -- this allows us to detect when
    // the final SQL can change due to a parameter change which affects the
    // parameterized 'table from SQL' type of dataset.
    let tablesig = '';
    if (base.dataset()) {
      tablesig = base.dataset().tables().map(t => t.tablename_withalias()).join('--');
      tablesig = base.substituteParams(tablesig);
    }
    const isSameTablesig = tablesig === last_good_tablesig;

    // If last completed result matches this query, we already have the data
    const isSameDsreq = dsreq && last_good_dsreq &&
                        arc.misc.isIdentical(dsreq, JSON.parse(last_good_dsreq));

    // Raw sql viz doesn't generate dsreq, so must compare raw sql
    const isSameSQL = base.viz_type() === 'rawtable' && real_sql_cmd === last_good_sql;
    const sqlComparisonValid = !String(real_sql_cmd).match(invalidSQLCachePattern);
    const sameSamplePct = last_good_sample === sample_pct;

    if ((isSameDsreq || isSameSQL) && sqlComparisonValid && sameSamplePct && isSameTablesig) {
      // DJC -- Note that here we will miss out on seeing the result of table updates
      // Nothing has changed in the sql command since the last time we fetched results.
      // Don't bother going back to the server and redrawing.
      if (force_redraw) {
        base.setTitleText();
        base.redraw(force_redraw);
      }

      resolve();
      return base;
    }

    function handleCompleteResults(o, qstate) {
      base.resetMessages();

      // on return of results save json string of dsreq us as comparisson to see if same dsreq sent twice
      last_good_dsreq = JSON.stringify(dsreq);
      last_good_sql = real_sql_cmd;
      last_good_sample = sample_pct;
      last_good_tablesig = tablesig;
      last_good_results = o;

      base.sqlQuery(sql_cmd);
      base.sqlOutput(o);

      if (o.rows.length === base.maxRowCount() && !maxRowsExplicitlySpecified() &&
          isFilterOrTimeControl(viz_type)) {
        const message = `Limiting display to ${o.rows.length} rows`;
        base.alertMessage(message);
      }

      resolve();
    }

    base.runSqlCmd(real_sql_cmd, function(o, qstate) {
      const dsreq_obj = dsreq ? JSON.parse(dsreq) : undefined;

      arc.cviz.remoteCaller()
         .dataset(dataset)
         .viz_lib(base)
         .results(o)
         .fetchRemoteData(dsreq_obj, function() {
           handleCompleteResults(o, qstate);
         });
    }, dsreq, undefined, function(err) {
      reject(err);

      let failedDraw = base.viz() && base.viz().drawOnDataFetchError;
      if (failedDraw) {
        failedDraw(base.chart_div());
      }
    });

    return base;
  };

  base.forceReloadData = function() {
    last_good_dsreq = undefined;
    last_good_sql = undefined;
    return base;
  };

  base.initDataColumns = function() {
    base.sb_colnames([])
        .sb_coltypes([])
        .sb_columns([])
        .sb_rows([]);
  };

  base.populateFromSqlOutput = function(_) {
    base.clearLocalFilters();
    json_data = _;
    var qr = arc.queryResults()
                .data(json_data);
    base.sb_colnames(qr.colnames())
        .sb_coltypes(qr.coltypes())
        .sb_columns(qr.columns())
        .sb_rows(json_data.rows);
    return base;
  };

  base.sqlOutput = function(_) {
    if (!arguments.length) return json_data;

    base.populateFromSqlOutput(_);
    base.redraw();
    if (redraw_cb) {
      redraw_cb(base);
    }
    return base;
  };

  base.cleanupDashboardAndVisualDesignerTab = () => {
    arc.report_cleanup_shelves(base.shelf_data());
    d3.select('#' + shelves_dom_id).selectAll('div').remove();
    // Clean up the limit shelf
    var $appLimitContainer =
        $('#' + shelves_dom_id).siblings('.app-limit-container');
    $appLimitContainer.empty();
  };

  base.cleanup = function() {
    base.cleanupDashboardAndVisualDesignerTab();
    base.undraw();
    return base;
  };

  base.arraysEqual = function(a, b) {
    if (a === b) return true;
    if (a === null || b === null) return false;
    if (a.length !== b.length) return false;

    for (var i = 0; i < a.length; ++i) {
      if (a[i] !== b[i]) return false;
    }
    return true;
  };

  base.sendParameters = function(paramObjects, click_cb) {
    if (!paramObjects || !paramObjects.length) {
      return;
    }

    if (!click_cb && arc.globals.click_behavior_cb) {
      click_cb = arc.globals.click_behavior_cb;
    }

    if (!click_cb) {
      return;
    }

    if (!(paramObjects[0] instanceof arc.params.Param)) {
      paramObjects = arc.params.convertOldParamArrToNewParamObjects(paramObjects, base);
    }

    var agg = base.sendAggregateParameters();

    let outParamObjects = agg ? paramObjects :
      paramObjects.filter(
        paramObject => paramObject.meta.keyType === arc.report.columnTypeEnum.DIMENSION);

    click_cb(undefined, undefined, undefined, undefined,
        base, undefined, arc.params.convertToOldFormatArr(outParamObjects));
  };

  /*
   * See ARC-11838
   *
   * When an appgroup scope filter is setup within an app, and some other visual
   * in the app is emitting a param with the same name, then the appgroup scope
   * setting is not in place, and this leads to unexpected behavior. The function
   * below checks to see if any parameter being emitted is also emitted by an
   * appgroup-scope filter. If so, it sets up scoping appropriately.
   */
  function updateAppScopeParams(outParamKeysSet) {
    if (!arc.globals.ms_dashboard) {
      return;
    }

    const globalFilters = arc.globals.ms_dashboard.globalFilters();
    globalFilters.filters().forEach(function(filterId) {
      const f = globalFilters.getFilterReport(filterId);
      if (!f) {
        return;
      }

      const vizLib = f.viz();
      if (!vizLib) {
        return;
      }

      const viz = vizLib.viz();
      if (!viz) {
        return;
      }

      if (!viz.viz_getOutputParameter) {
        return;
      }

      const outputParameter = viz.viz_getOutputParameter();
      if (!outParamKeysSet.has(outputParameter)) {
        return;
      }

      updateAppScope({
        outputParameter,
        isDatasetScope: visualDatasetScope(),
        isAppScope: true,
      });
    });
  }

  var pivots = [];
  function initializePivots() {
    let container = dashboard;
    if (!container) {
      container = arc.report.getPrimaryReport();
      if (!container.permitParamHistory()) {
        return;
      }
    }

    pivots = [];

    const kvs = container.current_params()[0];
    kvs?.forEach((kv) => {
      var k = kv.key;
      if (!k.endsWith('.pivot')) {
        return;
      }

      const v = JSON.parse(kv.value);
      if (v.vizId !== report_id || !v.filters || !v.filters.length) {
        return;
      }

      v.pivotShelfName = pivotShelfName(v.filters);
      pivots.push(v);
    });
  }

  base.handleSpecificFilters = function(filterNames, cb) {
    if ((filterNames.length === 0) || (!dataset)) {
      return;
    }

    filterNames.forEach(function(filterName) {
      // Do substitution here so that we can pick up a parameterized column name
      filterName = base.substituteParams(filterName.trim());
      let rawColname = arc.dataconnection.types.stripBrackets(filterName);
      let col = dataset.get_column(rawColname);
      let e = arc.report.shelf.element()
                 .dataset(dataset)
                 .dataconnection_type(dataset.dataconnection_type());

      // Either filter element is column name or handle as custom expression
      if (col !== undefined) {
        e.dataset_colname(rawColname)
         .dataset_coltype(col.attr('type'))
         .filter_column(rawColname);
      } else {
        e.custom_expr(filterName);
      }

      var expr = e.getExpression(base.substituteParams, true);
      if ((filterName === expr) || (rawColname === expr)) {
        // No substitutions with parameters
        return;
      }

      cb(filterName, expr);
    });
  };

  function handleDynamicFilters(filterNames, params, cb) {
    if (!dataset) {
      return;
    }

    // Add page-scope parameters that apply to the dataset that this
    // visual belongs to
    const filterSet = new Set(filterNames);
    params.forEach(function(p) {
      if (!paramAppliesToThisVisual(p)) {
        return;
      }

      const keyParts = arc.params.splitParams(p.key);

      const keyName = keyParts[0];
      if (filterSet.has(keyName)) {
        return;
      }

      if (dataset.findColumn(keyName) === undefined) {
        return;
      }

      filterNames.push(keyName);
      filterSet.add(keyName);
    });

    filterNames.forEach(function(filterName) {

      // Try and get the column type from the dataset
      // If we can't for some reason, use a string type
      var coltype = findColumnType(filterName);
      var expr = arc.misc.filterExpression(filterName, coltype, params,
          dataset.dataconnection_type(), base.report_id());
      if (expr) {
        parametersSubstituted.push({
          key: filterName,
          value: expr,
          type: 'expr',
        });

        cb(expr, filterName, params);
      }
    });
  }

  base.handleSolrParam = function(cb) {
    // SOLR magic. If there is a parameter starting with 'search' associated with this dataset,
    // use it as a filter
    if ((!base.isSolr() && !base.isElasticsearch()) || !dashboard) {
      return base;
    }

    if (isExplicitScope() && viz_type !== 'picklist') {
      // Even though picklists have explicit scope, we want our solr picklists
      // to respond to the solr search param.
      return base;
    }

    let params = base.getParams();
    let kvs = params.getkvsList();

    kvs.forEach(function(kv) {
      // Pickup search parameters but not search.ignore
      if ((kv.key.substring(0, 6) === 'search') &&
          (kv.key.substring(kv.key.length - 7, kv.key.length) !== '.ignore') &&
          (kv.source_visual_id !== report_id) &&
          (kv.source_dataset_id === base.dataset_id())) {
        cb(kv.value, /* isSearch */ true);
      }
    });

    return base;
  };

  function getParams() {
    if (dashboard) {
      return base.getParams().getkvsList();
    }

    return arc.globals.reports.getPrimaryReport().current_params()[0];
  }

  base.datasetFiltersCb = function(cb) {
    if (isExplicitScope()) {
      return;
    }

    let kvs = getParams();

    var filterNames;
    if (dashboard) {
      filterNames = [];
      const datasetId = base.dataset_id();
      filterNames = arc.globals.ms_dashboard.params_history()
        .current_params()[0]
        ?.filter((param) => {
          if (!param.scope) {
            return false;
          }

          // NOTE: If we do not have scopeType, that implies dashboard was saved before 5.1.0
          // If datasetId is present, that implies it is at least a DATASET scope param
          // If datasetId is not present, that implies it is an EXPLICIT scope param.
          // GLOBAL scope was introduced in 5.1.0 along with scopeType.
          return paramAppliesToThisVisual(param);
        })
        .map((param) => param.scope.filterName) ?? [];
      const dataset = base.dataset();
      if (!dataset) {
        // Verify that we have a dataset...
        return base;
      }
      filterNames = Array.from(new Set(filterNames))
        .filter(function filterInValidDatasetColnames(opName) {
          if (!!dataset.get_column(opName)) {
            // The filter specified is a column in the dataset
            return true;
          }

          // Is the filter made up entirely of columns from the dataset?
          let cols = getColumns(opName);
          if (cols.length === 0) {
            // no columns found
            return false;
          }

          // Is some column in the expression not from this dataset?
          const missingCol = cols.some(colname => !dataset.get_column(colname));
          return !missingCol;
        });

      kvs = base.getParams().getkvsList();
      handleDynamicFilters(filterNames, kvs, cb);
    } else if (arc.globals.filter_scope && arc.globals.filter_scope.length) {
      // We went to single value visual,
      // most likely from dashboard that had filter all selected
      filterNames = arc.globals.filter_scope.map((ds) => ds.filterName);
      handleDynamicFilters(filterNames, kvs, cb);
    }
  };

  base.addFiltersToSb = function(sb, tempFilter) {
    base.shelfElementCb('filters_shelf', function(elem) {
      var expr = elem.getExpression(base.substituteParams, true);
      if (expr === arc.report.shelf.EXPR_TRUE) {
        return;
      }

      expr = sb.addTablenamesFromExpression(expr);
      // DJC -- The element expression should indicate if it's an aggregate
      if (arc.report.isSupportedAggregate(expr, base.dataconnection_type()) ||
          elem.expr_hasagg()) {
        sb.addHavingFilter(base.aggregateValue(expr));
      } else {
        sb.addFilter(expr);
      }
    }, true);

    function addFilter(expr) {
      if (arc.report.isSupportedAggregate(expr, base.dataconnection_type())) {
        sb.addHavingFilter(expr);
      } else {
        sb.addFilter(expr);
      }
    }

    handlePicklistFilterColumns(addFilter);
    base.datasetFiltersCb(addFilter);
    base.handleSolrParam(addFilter);

    if (!dashboard) {
      if (base.isSolr()) {
        sb.addFilter(base.solrSearchData());
      }

      if (base.isElasticsearch()) {
        const queryString = base.solrSearchData();
        if (queryString !== '*:*') {
          sb.addFilter("query( '"+ queryString + "')");
        }
      }
    }

    if (tempFilter) {
      addFilter(tempFilter);
    }

    return base;
  };

  function secondsToAutotime(tdSeconds) {
    const bucketGranularities = [
      {
        unit: 'SECOND',
        unitCount: 1,
      },
      {
        unit: 'MINUTE',
        unitCount: 1,
      },
      {
        unit: 'HOUR',
        unitCount: 1,
      },
      {
        unit: 'DATE',
        unitCount: 1,
      },
      {
        unit: 'YEARWEEK',
        unitCount: 1,
      },
      {
        unit: 'YEARMONTH',
        unitCount: 1,
      },
      {
        unit: 'YEARQUARTER',
        unitCount: 1,
      },
      {
        unit: 'YEAR',
        unitCount: 1,
      },
    ];

    let outBucket = bucketGranularities[bucketGranularities.length - 1];
    bucketGranularities.some(function(bucket) {
      const bucketCount = tdSeconds / (unitNameToSeconds[bucket.unit] * bucket.unitCount);
      if (bucketCount < 100) {
        outBucket = bucket;
        return true;
      }
    });

    return outBucket;
  }

  function processAutodateFilter(filterName, previousDataResult) {
    let start,
        end;

    base.datasetFiltersCb(function(expr, _filterName, params) {
      if (_filterName !== filterName) {
        return;
      }

      params.forEach(function(param) {
        if (param.key === `${_filterName}.start`) {
          start = param.value;
          return;
        }

        if (param.key === `${_filterName}.end`) {
          end = param.value;
          return;
        }
      });
    });

    if (!start || !end) {
      if (previousDataResult) {
        const rows = previousDataResult.rows();
        if (rows && rows.length === 1) {
          if (!start) {
            start = rows[0][0];
          }

          if (!end) {
            end = rows[0][1];
          }
        }
      }
    }

    if (!start || !end) {
      return undefined;
    }

    const tdSeconds = (moment.utc(end) - moment.utc(start)) / 1000;
    if (isNaN(tdSeconds)) {
      return undefined;
    }

    if (tdSeconds <= 0) {
      return undefined;
    }

    const c = secondsToAutotime(tdSeconds);
    return {
      start,
      end,
      unit: c.unit,
      unitCount: c.unitCount,
    };
  }

  function augmentWithData(segments) {
    if (segments.length > 0) {
      const allSegments = dataset_segments.segments();
      segments.forEach(function(s) {
        allSegments.forEach(function(s2) {
          if (s2.id === s.id) {
            s.data = s2.data;
          }
        });
      });
    }
  }

  function paramAppliesToThisVisual(param) {
    if (!param.scope) {
      return false;
    }

    const scopeType = param.scope.scopeType;

    if (scopeType === arc.params.ParamEnums.ScopeTypeEnum.GLOBAL) {
      return true;
    }

    if (scopeType === arc.params.ParamEnums.ScopeTypeEnum.DATASET) {
      if (param.source_dataset_id === base.dataset_id()) {
        return true;
      }
    }

    return false;
  }

  function selectedSegmentsFromParams() {
    const params = base.getParams();

    let segmentData = params.getSegmentData(base.dataset_id());
    if (!segmentData) {
      return undefined;
    }

    const [key, ext] = arc.params.splitParams(segmentData.key);
    const segmentExclude = params.get(`${key}.exclude`, true);

    let negate = false;
    if (segmentExclude && segmentExclude.value === 'not in') {
      negate = true;
    }

    const segments = segmentData.value_list.map(function(id) {
      return {
        id,
        negate,
      };
    });

    augmentWithData(segments);
    return segments;
  }

  function selectedSegmentsFromFiltersShelf() {
    const segments = [];

    base.shelfElementCb('filters_shelf', function(elem) {
      if (!elem.isDatasetSegment()) {
        return;
      }

      const expr = elem.custom_expr();
      if ((expr === undefined) || (expr.trim() === '')) {
        return;
      }

      JSON.parse(expr).map(function(s) {
                        return {
                          id: Number(s[0]),
                          negate: !s[1],
                        };
                      })
                      .forEach(function(s) {
                        segments.push(s);
                      });
    });

    augmentWithData(segments);
    return segments;
  }

  function addParamSegmentFiltersToRb(rb) {
    if (isExplicitScope()) {
      return;
    }

    const segments = selectedSegmentsFromParams();
    if (segments && segments.length > 0) {
      rb.globalSegments(segments);
    }
  }

  function addFilterSegmentsToRb(rb) {
    const segments = selectedSegmentsFromFiltersShelf();
    if (segments && segments.length > 0) {
      rb.globalSegments(segments);
    }
  }

  // incremental filter currently only used by map visual for incremental queries in choropleth
  base.addFiltersToRb = function(rb, no_param_sub, incrementalFilter) {
    var par_sub_cb;
    if (!no_param_sub) {
      par_sub_cb = base.substituteParams;
    }

    base.shelfElementCb('filters_shelf', function(elem) {
      if (elem.isDatasetSegment()) {
        return;
      }

      var expr = elem.getExpression(par_sub_cb, true);

      if (expr === arc.report.shelf.EXPR_TRUE) {
        return;
      }

      expr = rb.expression(expr);
      if (arc.report.isSupportedAggregate(expr, base.dataconnection_type()) ||
          elem.expr_hasagg()) {
        rb.having(base.aggregateValue(expr));
      } else {
        rb.filters(expr);
      }
    }, true, par_sub_cb);

    function isAggregate(expr) {
      if (arc.report.isSupportedAggregate(expr, base.dataconnection_type())) {
        return true;
      }

      // Is the filter made up entirely of columns from the dataset?
      let cols = getColumns(expr);
      if (cols.length === 0) {
        // no columns found
        return false;
      }

      // Is some column in the expression not from this dataset?
      const missingCol = cols.some(colname => !dataset.get_column(colname));
      if (missingCol) {
        return false;
      }

      const hasAgg = cols.some(colname => dataset.get_column(colname).attr('exprhasagg'));
      return hasAgg;
    }

    function addFilter(expr) {
      if (isAggregate(expr)) {
        rb.having(expr);
      } else {
        rb.filters(expr);
      }
    }

    handlePicklistFilterColumns(addFilter);
    base.datasetFiltersCb(addFilter);
    base.handleSolrParam(function(expr, isSearch) {
      if (isSearch) {
        if (base.isSolr()) {
          rb.addSolrQuery(expr);
        }

        if (base.isElasticsearch()) {
          if (expr !== '*:*') {
            rb.filters("query('"+ expr + " ')");
          }
        }
      }
    });

    if (base.isSolr()) {
      rb.addSolrQuery(base.solrSearchData());
    }

    if (base.isElasticsearch()) {
      const queryString = base.solrSearchData();
      if (queryString !== '*:*') {
        rb.filters("query(' "+ queryString + " ')");
      }
    }

    if (incrementalFilter) {
      addFilter(incrementalFilter);
    }

    return base;
  };

  var dim_start_idx,
      dim_end_idx,
      agg_start_idx,
      agg_end_idx;

  base.dim_cnt = function() {
    if (dim_start_idx < 0) {
      return 0;
    }

    return 1 + dim_end_idx - dim_start_idx;
  };

  base.dim_start_idx = function(_) {
    if (!arguments.length) return dim_start_idx;
    dim_start_idx = _;
    return base;
  };

  base.dim_end_idx = function(_) {
    if (!arguments.length) return dim_end_idx;
    dim_end_idx = _;
    return base;
  };

  base.agg_cnt = function() {
    if (agg_start_idx < 0) {
      return 0;
    }

    return 1 + agg_end_idx - agg_start_idx;
  };

  base.agg_start_idx = function(_) {
    if (!arguments.length) return agg_start_idx;
    agg_start_idx = _;
    return base;
  };

  base.agg_end_idx = function(_) {
    if (!arguments.length) return agg_end_idx;
    agg_end_idx = _;
    return base;
  };

  base.dimensions_shelf_count = function() {
    return base.shelf_type_count(arc.report.shelf_types.COLUMN,
        arc.report.columnTypeEnum.DIMENSION);
  };

  base.aggregates_shelf_count = function() {
    return base.shelf_type_count(arc.report.shelf_types.COLUMN,
        arc.report.columnTypeEnum.AGGREGATE);
  };

  base.shelf_type_count = function(in_shelf_type, in_shelf_coltype) {
    var cnt = 0;
    base.shelf_type_cb(in_shelf_type, in_shelf_coltype, function(shelf_name) {
      cnt += base.shelfElementCount(shelf_name);
    });
    return cnt;
  };

  base.shelf_type_cb = function(in_shelf_type, in_shelf_coltype, cb) {
    base.shelf_data().forEach(function(shelf_element) {
      var shelf_name = shelf_element.name;
      var shelf_type = shelf_element.shelf_type;
      var shelf_coltype = shelf_element.column_type;
      if ((shelf_type === in_shelf_type) &&
          (shelf_coltype === in_shelf_coltype)) {
        cb(shelf_name);
      }
    });

    return base;
  };

  function autotimeColumnExists() {
    const no_param_sub = false;
    let exists = false;

    base.shelf_data().some(function(shelf_data_elem) {
      shelfToDimAggCol(shelf_data_elem, no_param_sub, function(col, shelf_elem) {
        if (exists) {
          return;
        }

        if (shelf_elem.autotimeEnabled()) {
          exists = true;
        }
      });
    });

    return exists;
  }

  function getAutotimeDataRequest(config) {
    let {
      previousDataRequest,
      permitAutoTimeBucketing,
    } = config;

    if (!permitAutoTimeBucketing) {
      return undefined;
    }

    if (previousDataRequest) {
      return undefined;
    }

    const no_param_sub = false;
    let dataRequest;

    base.shelf_data().some(function(shelf_data_elem) {
      shelfToDimAggCol(shelf_data_elem, no_param_sub, function(col, shelf_elem) {
        if (dataRequest) {
          return;
        }

        if (!shelf_elem.autotimeEnabled()) {
          return;
        }

        const params = getParams(),
              ds_colname = shelf_elem.dataset_colname();

        const keys = new Set(params.map(function(p) {
          return p.key;
        }));

        if (keys.has(`${ds_colname}.start`) &&
            keys.has(`${ds_colname}.end`)) {
          /*
           * .start and .end parameters already exist
           * Most likely, there is a date filter on the page. We do not
           * need to run min/max queriws to find the right autobucketing
           * ranges.
           */
          return;
        }

        const rb = base.getRequestBuilder();
        base.addFiltersToRb(rb, no_param_sub);
        const colname = col.colname();
        rb.addAggregate(`min([${colname}]) as "min"`);
        rb.addAggregate(`max([${colname}]) as "max"`);

        const dsreq = rb.dsreq();

        dataRequest = arc.data.request()
                              .rb(rb)
                              .add_column(col.cloneDataColumn())
                              .add_column(col.cloneDataColumn())
                              .dsreq(dsreq)
                              .finalize()
                              .isPrereqDataRequest(true);
      });

      if (dataRequest) {
        return true;
      }
    });

    return dataRequest;
  }

  // Basic viz_getDataRequest
  base.getCheckedDataRequest = function(constraints, config={}) {
    let {
      previousDataRequest,
      previousDataResult,
      constraintsCb,
      permitAutoTimeBucketing,
    } = config;

    // Return back null so data_request isn't undefined anymore
    if (!base.check_shelf_contents(constraints)) {
      return null;
    }

    let req = getAutotimeDataRequest(config);
    if (req) {
      return req;
    }

    req = base.shelvesToDataRequest(base.shelf_data(), config);
    if (!req) {
      return req;
    }

    if (!constraintsCb) {
      return req;
    }

    return constraintsCb(req);
  };

  base.isSolr = function() {
    return (base.dataconnection_type() === 'solr');
  };

  base.isElasticsearch = function() {
    return (base.dataconnection_type() === 'elasticsearch');
  };

  base.isKSQL = function() {
    return (base.dataconnection_type() === 'ksql');
  };

  const validStreamPositions = new Set(['earliest', 'latest']);

  base.ksqlStreamPosition = function() {
    if (!base.isKSQL()) {
      return undefined;
    }

    const s = user_settings.find_by_id('ksql_stream_position');
    if (s) {
      const pos = s.value();
      if (validStreamPositions.has(pos)) {
        return pos;
      }
    }

    if (!dataset) {
      return undefined;
    }

    return dataset.ksql_stream_position();
  };

  base.aliasesAfterQuery = function() {
    if (arc.globals.arcviz_settings.USE_VIZ_COLNAME_ALIASES && base.viz_type() !== 'rawtable') {
      return true;
    }

    let dctype = base.dataconnection_type();

    return ((dctype === 'solr') || (dctype === 'bigquery'));
  };

  base.solrSearchData = function() {
    solr_query = $('#' + shelves_dom_id + ' .searchbox').val();

    if (solr_query) {
      return base.substituteParams(solr_query);
    } else {
      solr_query = '*:*';
      return solr_query;
    }
  };

  base.shelfElementToCol = function(shelf_element, shelf_name, shelf_property, shelfIndex) {
    var col =
        arc.data.column()
                .colsource(shelf_name)
                .colproperty(shelf_property)
                .colfiltername(shelf_element.getComputedDisplayName(true, true))
                .coltype(shelf_element.dataset_coltype())
                .colname(shelf_element.getComputedDisplayName(false, true))
                .colgeotype(shelf_element.geo_cast())
                .dimhlist(shelf_element.dataset_dimhlist())
                .dimhlevel(shelf_element.dataset_dimhlevel())
                .shelfIndex(shelfIndex)
                .expr(shelf_element.getExpression(base.substituteParams, true))
                .custom_expr(shelf_element.custom_expr())
                .dataTransformation(shelf_element.dataTransformation())
                .coldisplaybefore(viz_type === 'table' ? shelf_element.coldisplaybefore() : undefined)
                .fn_name(shelf_element.fn_name())
                .fn_cast(shelf_element.fn_cast())
                .coldesc(shelf_element.col_desc());

    col.secondaryaxis(!!shelf_element.secondaryaxis());

    if (shelf_element.display_format()) {
      col.setDisplayFormat(base.translatedDisplayFormat(
          shelf_element.display_format()));
    }

    if (shelf_element.col_desc()) {
      col.coldesc(base.substituteParams(shelf_element.col_desc()));
    }

    if (shelf_element.order()) {
      col.order(arc.data.order()
                        .priority(shelf_element.order().priority())
                        .ascending(shelf_element.order().ascending())
                        .limit(shelf_element.order_limit())
                        .granular(shelf_element.order_limit_granular()));
    }

    return col;
  };

  function ignoreShelfForDimAgg(shelf_data_elem) {
    const {
      shelf_type,
      drillShelf,
    } = shelf_data_elem;

    if (shelf_data_elem.drillShelf === true) {
      return true;
    }

    if (shelf_type !== arc.report.shelf_types.COLUMN) {
      return true;
    }

    return false;
  }

  function shelfToDimAggCol(shelf_data_elem, no_param_sub, cb) {
    const shelf_name = shelf_data_elem.name,
          shelf_property = shelf_data_elem.column_type;

    if (ignoreShelfForDimAgg(shelf_data_elem)) {
      return;
    }

    function handleUnknownCol(shelf_element, shelfIndex) {
      let col,
          isAgg;
      // SG first method returns false for derived columns with an aggregate
      // second method returns true for derived columns with an aggregate
      if (shelf_element.expressionIsAggregate({ substCb: base.substituteParams }) ||
          shelf_element.expr_hasagg()) {
        isAgg = true;
      }

      col = base.shelfElementToCol(shelf_element, shelf_name,
          isAgg ? arc.report.columnTypeEnum.AGGREGATE : arc.report.columnTypeEnum.DIMENSION,
          shelfIndex);
      cb(col, shelf_element);
    }

    if (shelf_property === arc.report.columnTypeEnum.UNKNOWN) {
      base.shelfElementCb(shelf_name, handleUnknownCol, undefined, no_param_sub);
      return;
    }

    function handleCol(shelf_element, shelfIndex) {
      var col = base.shelfElementToCol(shelf_element, shelf_name,
          shelf_property, shelfIndex);
      cb(col, shelf_element);
    }

    base.shelfElementCb(shelf_name, handleCol, undefined, no_param_sub);
  }

  base.shelvesToDataRequest = function(shelvesData, config={}) {
    const {
      previousDataResult,
      no_param_sub,
      incrementalFilter,
    } = config;

    initializePivots();

    // when no_param_sub is set don't call parameter substitution callback
    // this is used when defining DD with parameters

    base.connectDataConnectionToShelves();
    var data_request = arc.data.request(),
        sb = base.getSelectBuilder(),
        rb = base.getRequestBuilder(),
        dims_cols = [],
        aggs_cols = [],
        dims = [],
        aggs = [];

    if (base.isSolr()) {
      rb.requestType('SOLR');
      if (viz_type === 'table' || viz_type === 'list') {
        let highlighting = user_settings.find_by_id('enable_highlighting');
        if (highlighting) {
          rb.highlighting(highlighting.isChecked);
        }
      }
    }

    if (base.ksqlStreamPosition()) {
      rb.ksqlStreamPosition(base.ksqlStreamPosition());
    }

    data_request.rb(rb);

    if (!shelvesData) {
      shelvesData = base.shelf_data();
    }

    shelvesData.forEach(function(shelf_data_elem) {
      shelfToDimAggCol(shelf_data_elem, no_param_sub, function(col, shelf_elem) {
        if (col.colproperty() === arc.report.columnTypeEnum.DIMENSION) {
          dims.push(shelf_elem);
          dims_cols.push(col);
        } else {
          aggs.push(shelf_elem);
          aggs_cols.push(col);
        }
      });
    });

    if (dims.length + aggs.length === 0 && config.build_default_viz && base.report().dataset()) {
      const dataset = base.report().dataset();
      let tables = dataset.tables();
      if (!config.dflviz_all_tables) {
        tables = [tables[0]];
      }

      tables.forEach(function(tab) {
        const columns = tab.columns();
        for (let i = 0; i < columns.length; i++) {
          const col = columns[i];
          if (config.dflviz_column_limit > 0 && dims.length >= config.dflviz_column_limit) {
            break;
          }

          // Ignore aggregate expressions - these cannot be setup as dimensions
          // and will cause errors
          if (col.attr('exprhasagg')) {
            continue;
          }

          // Can use the same column test as for the detailed data display
          if (col.showInDetailData()) {
            var elem = arc.report.shelf.element()
                                       .dataset_colname(col.attr('alias') || col.attr('name'))
                                       .display_format(col.attr('displayformat'))
                                       .dataset_coltype(col.attr('type'));
            dims.push(elem);
            dims_cols.push(base.shelfElementToCol(elem, 'dimensions_shelf',
                           arc.report.columnTypeEnum.DIMENSION, 0));
          }
        }
      });

      data_request.isDefault(true);
    }

    dims_cols.forEach(function(c) {
      data_request.add_column(c);
    });
    aggs_cols.forEach(function(c) {
      data_request.add_column(c);
    });

    base.setSbLimit(sb);
    base.setRbLimit(rb);
    base.addFiltersToSb(sb, incrementalFilter);
    base.addFiltersToRb(rb, no_param_sub, incrementalFilter);
    addCols({
      sb,
      dims,
      aggs,
      rb,
      no_param_sub,
      previousDataResult,
    });

    sb = base.finalizeSql(sb);
    data_request.sql(sb.getSql());
    let dsreq_obj = rb.dsreq();
    const derivedDataList = derivedData.derivedDataList();
    if ((!dsreq_obj.derived_data || !dsreq_obj.derived_data.length) && derivedDataList) {
      rb.deriveddataList(derivedDataList);
      dsreq_obj = rb.dsreq();
    }
    let newObject;
    // When no_param_sub is undefined we need to substitute DD parameters
    // We do this by creating a new dsreq object (deep copy) containing a new list of DD
    // with substituted parameters
    if (!no_param_sub) {
      newObject = jQuery.extend(true, {}, dsreq_obj);
      var ddlist = [];
      newObject.derived_data.forEach(function(ddata) {
        if (ddata) {
          var dd = jQuery.extend(true, {}, ddata);
          dd.dsreq = {};
          var dd_dsreq = jQuery.extend(true, {}, ddata.dsreq);
          var dim, agg, fil, hav;
          var dd_dims = [];
          var dd_aggs = [];
          var dd_fils = [];
          var dd_havs = [];
          ddata.dsreq.dimensions.forEach(function(d) {
            dim = jQuery.extend(true, {}, d);
            dim.expr = base.substituteParams(dim.expr);
            dd_dims.push(dim);
          });
          ddata.dsreq.aggregates.forEach(function(a) {
            agg = jQuery.extend(true, {}, a);
            agg.expr = base.substituteParams(agg.expr);
            dd_aggs.push(agg);
          });
          ddata.dsreq.filters.forEach(function(f) {
            fil = (' ' + f).slice(1); // new string from f
            fil = base.substituteParams(fil);
            dd_fils.push(fil);
          });
          ddata.dsreq.having.forEach(function(h) {
            hav = (' ' + h).slice(1); // new string from h
            hav = base.substituteParams(hav);
            dd_havs.push(hav);
          });
          dd_dsreq.dimensions = dd_dims;
          dd_dsreq.aggregates = dd_aggs;
          dd_dsreq.filters = dd_fils;
          dd_dsreq.having = dd_havs;
          dd.dsreq = dd_dsreq;
          ddlist.push(dd);
        }
      });
      newObject.derived_data = ddlist;
    }
    base.dsreq(dsreq_obj);
    if (newObject) {
      data_request.dsreq(newObject);
    } else {
      data_request.dsreq(dsreq_obj);
    }

    data_request.finalize();
    return data_request;
  };

  function addCols(config) {
    let {
      sb,
      dims,
      aggs,
      rb,
      no_param_sub,
      previousDataResult,
    } = config;

    const all_aggregates = true;

    let orders = [],
        ordinal = 0,
        expr,
        substitutedLimit,
        projections_len = sb.projections().length,
        dimExpressions = [],
        autotimeConfig;

    if (dataset && dataset.dataconnection_type) {
      const t = arc.dataconnection.types[dataset.dataconnection_type()];
      if (t) {
        const connection = t();
        if (connection) {
          dims.some(function(shelf_element, dimIdx) {
            if (!shelf_element.autotimeEnabled()) {
              return;
            }

            const c = processAutodateFilter(shelf_element.dataset_colname(),
                previousDataResult);
            if (c === undefined) {
              return true;
            }

            autotimeConfig = c;
            c.connection = connection;
            c.shelf_element = shelf_element;
            c.dimIdx = dimIdx;

            return true;
          });
        }
      }
    }

    dim_start_idx = projections_len;
    dim_end_idx = -1;
    var par_sub_cb;
    if (!no_param_sub) {
      par_sub_cb = base.substituteParams;
    }

    dims.forEach(function(shelf_element, i) {
      // Remember display formats used to display the data
      if (shelf_element.display_format()) {
        sqlIndexToFormatter[dim_start_idx + i] =
            base.translatedDisplayFormat(shelf_element.display_format());
      } else {
        delete sqlIndexToFormatter[dim_start_idx + i];
        shelf_element.display_format(null);
      }

      ordinal += 1;
      expr = undefined;

      if (autotimeConfig && autotimeConfig.dimIdx === i) {
        const t = arc.dataconnection.dateConvert({
          columnName: shelf_element.dataset_colname(),
          connection: autotimeConfig.connection,
          unit: autotimeConfig.unit,
          unitCount: autotimeConfig.unitCount,
        });

        if (t) {
          expr = `${t.expr} as "${t.alias}"`;
          dimExpressions.push(t.expr);
        }
      }

      if (!expr) {
        expr = shelf_element.getExpression(par_sub_cb);
        dimExpressions.push(shelf_element.getExpression(par_sub_cb, true));
      }

      if (shelf_element.isDatasetSegment()) {
        let segments;

        segments = dataset_segments.segments_from_id(shelf_element.filter_data());
        if (!segments || segments.length === 0) {
          segments = selectedSegmentsFromParams();
        }

        if (!segments || segments.length === 0) {
          segments = selectedSegmentsFromFiltersShelf();
        }

        rb.segments(segments);
      } else {
        var dimh = shelf_element.dataset_dimhlist();
        var vals = shelf_element.dataset_dimhvals();
        var ctlevel = 0; // shelf_element.dataset_dimhactive();

        if (dimh) {
            // If we're doing normal drilldown, then for each dimension value specified,
            // we push an expression into the filters list for that value and use the
            // next column in the hierarchy as the current dimension.
            for (var j = 0; j < vals.length; j++) {
              if (typeof vals[j] !== 'undefined') {
                rb.filters('[' + dimh[j] + '] = ' + vals[j]);
              }
            }

            const currDim = dimh[vals.length];
            if (currDim) {
              expr = `[${currDim}] as '${currDim}'`;
            }
        }

        rb.addDimension(expr, shelf_element.order());
        if (base.isSolr() && shelf_element.autotime_override()) {
          rb.autotime_override(shelf_element.autotime_override());
        }
      }

      expr = sb.addTablenamesFromExpression(expr);
      sb.addProjection(expr);
      if (aggs.length > 0) {
        sb.addGroupWithoutProjection(ordinal);
      }
      if (shelf_element.order()) {
        orders.push([shelf_element.order().priority(),
                     shelf_element.getOrderExpression(ordinal)]);
        if (shelf_element.order_limit()) {
          substitutedLimit = base.substituteParams(shelf_element.order_limit());
          sb.limit(substitutedLimit);
          rb.limit(substitutedLimit);
        }
      }
    });

    if (ordinal === 0) {
      dim_start_idx = -1;
    } else {
      dim_end_idx = dim_start_idx + ordinal - 1;
    }

    let rankAggregate,
        rankLimit;

    projections_len = sb.projections().length;
    agg_start_idx = -1;
    agg_end_idx = -1;
    aggs.forEach(function(shelf_element, i) {
      if (i === 0) {
        agg_start_idx = projections_len;
      }

      if (shelf_element.display_format()) {
        sqlIndexToFormatter[agg_start_idx + i] =
            base.translatedDisplayFormat(shelf_element.display_format());
      } else {
        delete sqlIndexToFormatter[agg_start_idx + i];
        shelf_element.display_format(null);
      }

      agg_end_idx = agg_start_idx + i;
      expr = shelf_element.getExpression(par_sub_cb,
          /* ignore_alias */ undefined,
          /* for_data_api */ true);

      if (all_aggregates || (i === 0)) {
        rb.addAggregate(expr, shelf_element.order());
        expr = sb.addTablenamesFromExpression(expr);
        sb.addProjection(expr);
        ordinal += 1;
        let order = shelf_element.order();

        if (order) {
          orders.push([order.priority(),
                       shelf_element.getOrderExpression(ordinal)]);
          if (shelf_element.order_limit()) {
            substitutedLimit = base.substituteParams(shelf_element.order_limit());
            if (shelf_element.order_limit_granular()) {
              rankAggregate = shelf_element.getExpression(par_sub_cb, true) +
                  (order.ascending() ? " asc": " desc");
              rankLimit = substitutedLimit;
            } else {
              sb.limit(substitutedLimit);
              rb.limit(substitutedLimit);
            }
          }
        }
      }
    });

    if (rankAggregate) {
      let aggExpr = "row_number() over (partition by " +
          dimExpressions.slice(0, dimExpressions.length - 1).join(",") +
          " order by " + rankAggregate + ") as '_arc_reserved_rank'";
      rb.addAggregate(aggExpr);
      rb.addOuterFilter("_arc_reserved_rank <= " + rankLimit);
    }

    // Sort all the ordering fields by priority
    orders.sort();
    // And then apply them to the select statement
    orders.forEach(function(o) {
      sb.addOrderBy(o[1]);
    });

    addParamSegmentFiltersToRb(rb);

    return base;
  }

  base.dimension_axis_is_date = function() {
    var dim_is_date = false;
    base.shelfElementCb('dimensions_shelf', function(shelf_element, i) {
      if (i === 0) {
        if (shelf_element.getComputedType() === 'date') {
          if (shelf_element.order()) {
            dim_is_date = true;
          }
        }
      }
    });
    return dim_is_date;
  };

  base.text_len = function(text) {
    var ll = base.textLengths([text], {
      container_div: chart_div,
      cssClasses: 'arc-viz-axis testsvg',
    });

    return ll[0];
  };

  base.textLengths = function(textl, config) {
    let { container_div,
          cssClasses,
          textClassesToApply,
          textGetter,
        } = config;

    var test_svg;

    // Remember the current value of 'overflow' and reapply it
    // when we are done
    var chart_div_sel = d3.selectAll('#' + container_div);
    var overflow_value = chart_div_sel.style('overflow');

    try {
      chart_div_sel.style('overflow', 'hidden');

      test_svg =
        chart_div_sel
          .append('svg');

      if (cssClasses) {
        test_svg.attr('class', cssClasses);
      }

      var test_text =
        test_svg.append('text')
                .style('opacity', '0');
      if (textClassesToApply) {
        test_text.attr('class', textClassesToApply);
      }

      var lengths = [];
      textl.forEach(function(textObj, i) {
        let text = textGetter ? textGetter(textObj, i) : textObj;
        test_text.text(text);
        lengths.push(test_text.node().getComputedTextLength());
      });

      return lengths;
    } finally {
      if (test_svg) {
        test_svg.remove();
      }

      chart_div_sel.style('overflow', overflow_value);
    }

    /* NOTREACHED */
  };

  base.dim_axis_max_textlen = function(values, max_display_len, fontSize) {
    let b = base.axis_values_maxbounds(values, max_display_len, fontSize);
    return b.width;
  };

  base.axisValuesFitToMax = function(values, max_display_len) {
    if (!max_display_len) {
      max_display_len = base.xaxis_label_maxlen();
    }

    return values.map(function(v) {
      if (typeof (v) === typeof ('')) {
        if (v.length > max_display_len) {
          v = v.substr(0, max_display_len) + '...';
          return v;
        }
      }

      return v;
    });
  };

  // Return the maximum width and height of values displayed on screen
  // as axis values
  base.axis_values_maxbounds = function(values, maxDisplayLen, fontSize) {
    const config = {
      maxDisplayLen,
      fontSize,
      cssClasses: 'arc-viz-axis testsvg',
    };

    return base.textBounds(values, config);
  };

  base.textBounds = function(values, config) {
    if (config === undefined) {
      config = {};
    }

    let {
      useSpan,
      textIsHtml,
      maxDisplayLen,
      fontSize,
      cssClasses,
    } = config;

    if (!maxDisplayLen) {
      maxDisplayLen = base.xaxis_label_maxlen();
    }

    // Figure out how much space we need to draw the dimension labels
    // We do this by actually drawing out the labels and computing the length
    // of each one, picking the max length we saw
    var toDelete,
        toMeasure;

    if (!useSpan) {
      toDelete = d3.selectAll('#' + chart_div)
                   .append('svg')
                   .attr('class', cssClasses);

      toMeasure = toDelete.append('text')
                          .style('opacity', '0');
    } else {
      toDelete = d3.selectAll('#' + chart_div)
                   .append('div');

      toMeasure = toDelete
                   .append('span')
                   .style('opacity', '0')
                   .attr('class', cssClasses);
    }

    fontSize = fontSize || base.getVisualFontSize();
    if (fontSize) {
      toMeasure.style('font-size', fontSize + 'px');
    }

    let maxH = 0,
        maxW = 0,
        date_formatter = d3.time.format('%Y-%m-%d');
    values.forEach(function(value) {
      var v = value;
      if (!v) {
        // Ignore null values
        return;
      }

      if (v instanceof Date) {
        v = date_formatter(v);
      } else if (!textIsHtml && maxDisplayLen && (v.length > maxDisplayLen)) {
        v = v.substr(0, maxDisplayLen) + '...';
      }

      if (textIsHtml) {
        toMeasure.html(v);
      } else {
        toMeasure.text(v);
      }

      let n = toMeasure.node();
      let b = useSpan ? n.getBoundingClientRect() : n.getBBox();

      maxH = Math.max(b.height + 1, maxH);
      maxW = Math.max(b.width + 1, maxW);
    });

    toDelete.remove();
    d3.selectAll('.testsvg').remove();

    return {
      width: maxW,
      height: maxH,
    };
  };

  function dateOf(d) {
    return moment.utc(d).toDate();
  }

  function nextday(d) {
    return moment(d).hours(0).seconds(0).minutes(0).add(1, 'days').toDate();
  }

  function prevday(d) {
    return moment(d).hours(0).seconds(0).minutes(0).subtract(1, 'days').toDate();
  }

  base.setup_date_scale = function(dim_values, d_maxrange, d_singlewidth_max) {
    var d_scale,
        dim_sz;
    base.shelfElementCb('dimensions_shelf', function(shelf_element, i) {
      if (i !== 0) {
        return;
      }
      var asc = (dim_values[dim_values.length-1] > dim_values[0]),
          end_date;
      if (asc) {
        end_date = nextday(dim_values[dim_values.length-1]);
      } else {
        end_date = prevday(dim_values[dim_values.length-1]);
      }

      d_scale =
        d3.time.scale()
          .domain([dim_values[0], end_date])
          .range([0, d_maxrange]);

      if (dim_values.length === 1) {
        dim_sz = d_singlewidth_max;
      } else {
        dim_sz = d_maxrange / ((end_date - dim_values[0]) / (60 * 60 * 24 * 1000));
        if (dim_sz < 0) {
          dim_sz = -dim_sz;
        }
      }

      var new_dim_values = [];
      var curr_date = dim_values[0];
      if (asc) {
        while (curr_date <= end_date) {
          new_dim_values.push(curr_date);
          curr_date = nextday(curr_date);
        }
      } else {
        while (curr_date >= end_date) {
          new_dim_values.push(curr_date);
          curr_date = prevday(curr_date);
        }
      }
      dim_values = new_dim_values;
    });
    return [d_scale, dim_sz, dim_values];
  };

  base.contextmenu = function(marker_name) {
    const menu =
       arc.contextmenu()
          .viz(base)
          .save_segment_cb(base.save_segment_cb())
          .click_behavior_cb(base.click_behavior_cb())
          .in_dashboard(base.in_dashboard())
          .filter_cb(base.addToFilters)
          .marker_name(marker_name)
          .highlight_on_select(true)
          .dataset_id(base.dataset_id())
          .dashboard_click_behavior(dashboard_click_behavior)
          .getMarksCb(base.marks)
          .click_behavior(
            base.dashboard() ?
              base.dashboard().get_click_behavior(base.report_id())[marker_name]
              : base.getClickBehavior(marker_name))
          .click_behavior_change(function(click_behavior) {
            if (click_behavior_change_cb) {
              click_behavior_change_cb(marker_name, click_behavior);
            } else {
              base.setClickBehavior(marker_name, click_behavior);
            }
          });

    contextmenus.push(menu);
    return menu;
  };

  function destroyContextMenus() {
    contextmenus.forEach(menu => menu.destroy());
    contextmenus = [];
    return base;
  }

  var local_filters = [];
  base.local_filters = function(_) {
    if (!arguments.length) return local_filters;
    local_filters = _;
    return base;
  };

  base.clearLocalFilters = function() {
    local_filters = [];
    return base;
  };

  base.addToLocalFilters = function(_) {
    local_filters.push(_);
    return base;
  };

  base.filterLocally = function(rows, dim_colindex) {
    if (local_filters.length === 0) {
      return rows;
    }
    var local_hash = {};
    local_filters.forEach(function(e) {
      local_hash[e] = 1;
    });
    return rows.filter(
      function(row) {
        return (!(base.d_string(row[dim_colindex]) in local_hash));
      });
  };

  // remove -> whether we want to add filter or remove filter
  // include -> whether filter is exclude type or include type
  function findAndUpdateExisting(filtername, filter_data, include, remove, isSourcePivot) {
    // isSourcePivot ? search in pivot filter items only
    // isSourcePivot is false, search in all
    let filter = findExistingFilter(filtername, include, isSourcePivot);
    let updated;

    if (filter) {
      if (isSourcePivot) {
        filter.shelf_element.source_pivot(isSourcePivot);
      }

      if (remove) {
        updated = filter.shelf_element.remove_filter_data(filter_data);
      } else {
        updated = filter.shelf_element.add_filter_data(filter_data);
      }

      if (updated) {
        filter.shelf_element.d3_elem().selectAll('.selection_field').call(arc.report.shelf.settext);
        return true;
      }
    }

    // See if we can find the opposite filter
    filter = findExistingFilter(filtername, !include, isSourcePivot);
    if (filter) {
      if (isSourcePivot) {
        filter.shelf_element.source_pivot(isSourcePivot);
      }

      if (remove) {
        updated = filter.shelf_element.add_filter_data(filter_data);
      } else {
        updated = filter.shelf_element.remove_filter_data(filter_data);
      }

      if (updated) {
        filter.shelf_element.d3_elem().selectAll('.selection_field').call(arc.report.shelf.settext);
        return true;
      }
    }

    return false;
  }

  function findExistingFilter(filtername, include, isSourcePivot) {
    var predicate = arc.grammar._predicate;
    let filterShelf;
    base.shelfElementCb('filters_shelf', function(shelf_element) {
      if (filterShelf) {
        return;
      }

      if (isSourcePivot !== shelf_element.source_pivot()) {
        return;
      }

      if (shelf_element.filter_column() && !shelf_element.custom_expr()) {
        if (filtername === shelf_element.filter_column()) {
          if (include ^ shelf_element.filter_exclude()) {

            filterShelf = {
              shelf_element: shelf_element,
            };

            return;
          }
        }

        return;
      }

      var expr = shelf_element.getExpression(base.substituteParams, true);
      try {
        var e = predicate.parse(expr);
        if (e.filter_column() && (filtername === e.filter_column())) {

          //should return if both are true or neither.
          if (!(include ^ e.filter_exclude())){
            return;
          }

          shelf_element.custom_expr(null);
          shelf_element.filter_column(e.filter_column());
          shelf_element.filter_exclude(e.filter_exclude());
          shelf_element.filter_data(e.filter_data());

          filterShelf = {
            shelf_element: shelf_element,
          };

          return;
        }
      } catch(err) {
        console.log(err);
        return;
      }
    });

    return filterShelf;
  }

  function pivotShelfName(params) {
    if (params.length === 0) {
      return undefined;
    }

    // If we can't find a dimension, use the first parameter
    let lastDimension = 0;
    let shelfList = {};

    params.forEach(function(param, i) {
      if (param.keytype === arc.report.columnTypeEnum.DIMENSION) {
        lastDimension = i;
        shelfList[param.source_shelf] = true;
      }
    });

    // Array of unique shelves in given params
    shelfList = Object.keys(shelfList);
    if (shelfList.length > 0) {
      const x_shelf_index = shelfList.indexOf('x_shelf');
      const y_shelf_index = shelfList.indexOf('y_shelf');
      if (x_shelf_index >= 0) {
        return shelfList[x_shelf_index];
      } else if (y_shelf_index >= 0) {
        return shelfList[y_shelf_index];
      }
    }

    return params[lastDimension].source_shelf;
  }

  base.addToFilters = function(params, include, isSourcePivot) {
    if (params.length === 0) {
      return base;
    }

    if (base.inModal()) {
      base.page_needs_save();
      base.shelfTrackerInstance().clearHistory();
    }

    let added = 0;
    params.forEach(function(param) {
      if (param.keytype === arc.report.columnTypeEnum.DIMENSION) {
        addParamToFilters(param, include, isSourcePivot);
        ++added;
      }
    });

    if (added === 0) {
      addParamToFilters(params[0], include, isSourcePivot);
    }

    if (!include) {
      base.redraw();
    }

    return base;
  };

  function addParamToFilters(param, include, isSourcePivot) {
    // Use the primary dimension to auto-add to filters
    const filterName = param.filtername || param.key;
    const filterColtype = param.valuetype || '';

    let filterData = typeof param.value !== 'undefined' ?
        [arc.report.shelf.parse_filter_value(param.value, filterColtype)] :
        param.filter_data.map(function (fd) {
          arc.report.shelf.parse_filter_value(fd.value, filterColtype);
        });

    if (!include) {
      // If excluding, we make the visuals reflect this immediately
      // by adding this value to local filters
      base.addToLocalFilters(param.value);
    }

    if (findAndUpdateExisting(filterName, filterData, include, false, isSourcePivot)) {
      return;
    }

    arc.report.shelf.addNewParamToFilters({
      filterName,
      filterData,
      filterColtype,
      include,
      isSourcePivot,
      shelves: base.shelves(),
      viz_lib: base,
    });
  }

  function findColumnType(columnName) {
    let col = dataset.get_column(columnName);

    if (!col) {
      const cols = getColumns(columnName);
      if (cols.length > 0) {
        col = dataset.get_column(cols[0]);
      }
    }

    if (col) {
      return col.coltype();
    }

    return 'string';
  }

  function changePrimaryDimension(columnName, filters, columnAlias) {
    var shelfName = pivotShelfName(filters);
    var columnType = findColumnType(columnName) || '';
    if (!shelfName) {
      return;
    }

    base.shelf_data().some(function(s) {
      if (s.name === shelfName) {
        shelves.add(
            s.name,
            [{
              dataset_colname: columnName,
              dataset_coltype: columnType,
              col_alias: columnAlias,
              source_pivot: true,
            }],
            true);

        return true;
      }

      return false;
    });
  }

  function pivotFilters(pivotInfo) {
    let { columns, filters } = pivotInfo;
    base.addToFilters(filters, true, true);
  }

  function pivot(pivotInfo) {
    let { columns, filters } = pivotInfo;
    base.addToFilters(filters, true, true);
    columns.forEach(function (column) {
      changePrimaryDimension(column.name, filters, column.alias);
    });
  }

  base.pivotParams = function (params, refreshNow) {
    let aph = arc.report.params_history();
    let newParams = {},
        newParamsKeys;
    params = aph.util.filterRelevantParams(params, report_id);
    params.forEach(function (p) {newParams[p.key] = p;});
    newParamsKeys = Object.keys(newParams);
    newParamsKeys = aph.util.sort(newParamsKeys);
    newParamsKeys = newParamsKeys.filter(function (npKey) {
      return (!currentAppliedParams[npKey]);
    });

    if (newParamsKeys.length) {
      newParamsKeys.forEach(function (np) {
        if (aph.util.isParamPivotType(newParams[np])) {
          pivot(aph.util.getInfoFromParam(newParams[np]));
          currentAppliedParams[np] = newParams[np];
        }
      });

      if (refreshNow) {
        base.refresh(/* force_redraw */ false);
      }
    }
  };

  function unpivot(pivotInfo) {
    let { columns, filters } = pivotInfo;
    filters.forEach(function (param) {
      let filtername = param.filtername || param.key,
          filter_data;
      param.valuetype = param.valuetype || '';
      if (typeof param.value !== 'undefined') {
        filter_data = arc.report.shelf.parse_filter_value(
        param.value, param.valuetype);
      } else {
        filter_data = param.filter_data.map(function (d) {
          arc.report.shelf.parse_filter_value(d, param.valuetype);
        });
      }

      if (!findAndUpdateExisting(filtername, filter_data, true, true, true)) {
        //@assert this case should never occur
        console.log('filter could not be removed: ', param);
      }
    });

    // TODO: Ideally each column should have a destination shelf info.
    var shelfName = pivotShelfName(filters);
    if (!shelfName) {
      return;
    }

    columns.forEach(function (col) {
      let idx;
      shelves.getShelf(shelfName).elements().slice(0).reverse().some(function (e, i) {
        if (e.dataset_colname() === col.name) {
          idx = i;
          return true;
        }
      });
      if (typeof idx !== 'undefined') {
        idx = shelves.getShelf(shelfName).elements().length - 1 - idx;
        let r = arc.globals.reports.getReport(base.report_id());
        r = r || arc.report.getPrimaryReport();
        if (arc.globals.is_dashboard && !base.inModal() && r) {
          shelves.remove(shelfName, idx, r.unique_report_id());
        } else {
          shelves.remove(shelfName, idx);
        }
      }
    });
  }

  base.unpivotParamsExcept = function (params) {
    let currAppliedParamsKeys = Object.keys(currentAppliedParams);
    let newParams = {};
    let newParamsKeys;
    let aph = arc.report.params_history();

    params = aph.util.filterRelevantParams(params, base.report_id());
    params.forEach(function (p) {newParams[p.key] = p;});
    newParamsKeys = Object.keys(newParams);
    let currAppliedParamsKeysToUnpivot = currAppliedParamsKeys.filter(function (cpKey) {
      return  (newParamsKeys.indexOf(cpKey) < 0);
    });
    let currAppliedParamsKeysToKeep = currAppliedParamsKeys.filter(function (cpKey) {
      return  (newParamsKeys.indexOf(cpKey) >= 0);
    });

    currAppliedParamsKeysToUnpivot = aph.util.sort(currAppliedParamsKeysToUnpivot).reverse();
    currAppliedParamsKeysToKeep = aph.util.sort(currAppliedParamsKeysToKeep);
    currAppliedParamsKeysToUnpivot.forEach(function (cpKey) {
      let pivotInfo = aph.util.getInfoFromParam(currentAppliedParams[cpKey]);
      if (pivotInfo) {
        unpivot(pivotInfo);
      }

      delete currentAppliedParams[cpKey];
    });

    currAppliedParamsKeysToKeep.forEach(function (cpKey) {
      let pivotInfo = aph.util.getInfoFromParam(currentAppliedParams[cpKey]);
      if (pivotInfo) {
        pivotFilters(pivotInfo);
      }
    });
  };

  base.addTooltip = function(textl, key, value, coltype, formatter) {
    textl.push(arc.misc.escapeHTML(key));
    var formattedValue = '';
    textl.push(': <b>');
    if (formatter) {
      formattedValue = base.formattedDisplay(formatter, value);
    } else {
      if (isNaN(value)) {
        formattedValue = base.d_string(value);
      } else {
        formattedValue = value;
      }
    }
    textl.push(arc.misc.escapeHTML(formattedValue));
    textl.push('</b>');
    return base;
  };

  base.param_from_column = function(col, value, row) {
    if (value === undefined) {
      value = row[col.colindex()];
    }

    let paramConfig = {
      baseField: col.colfiltername(),
      colDataType: col.coltype(),
      entityType: arc.params.ParamEnums.EntityTypeEnum.VISUAL,
      entityId: base.report_id(),
      datasetId: base.dataset_id(),
      keyType: col.colproperty(),
      valueType: arc.params.ParamEnums.ValueTypeEnum.SINGLE,
      outputParameter: arc.params.convertKey(col.colname()),
      value: [value],
    };

    let keyName = col.colname();
    if (col.isHierarchy()) {
      keyName = col.dimhlist()[col.dimhlevel()];
    }

    paramConfig.outputParameter = arc.params.convertKey(keyName);

    paramConfig.pivot = {
      sourceShelf: col.colsource(),
      customExpr: col.custom_expr(),
      expr: col.expr(),
      fnName: col.fn_name(),
      fnCast: col.fn_cast(),
    };

    let paramObject = new arc.params.ColParam(base, paramConfig);
    return arc.params.convertToOldFormatArr([paramObject]);
  };

  base.params_from_columns = function(row, columns, dims_only,
      primary_colindex, single_colindex, ignore_col_indices) {
    var params = [];
    columns.forEach(function(col) {
      if (ignore_col_indices !== undefined) {
        let ignore = ignore_col_indices.some(function(ignore_col_index) {
          if (col.colindex() === ignore_col_index) {
            return true;
          }

          return false;
        });

        if (ignore) {
          return;
        }
      }

      if (dims_only &&
           (col.colproperty() !== arc.report.columnTypeEnum.DIMENSION || col.colname() === 'Measure')
      ) {
        return;
      }

      if (typeof(single_colindex) !== 'undefined' &&
          col.colindex() !== single_colindex) {
        return;
      }

      let v = row[col.colindex()];
      if (v === undefined) {
        return;
      }

      let p = base.param_from_column(col, undefined, row);

      if (primary_colindex !== undefined) {
        if (primary_colindex === col.colindex()) {
          params = p.concat(params);
          return;
        }
      }

      params = params.concat(p);
    });

    return params.slice(0);
  };

  base.toggleTrellisSettings = function (isTrellis, moreTrellisSettings = []) {
    let trellisSettings = [
      user_settings.find_by_id('trellis_borders'),
      user_settings.find_by_id('trellis_cell_minimum_width'),
      user_settings.find_by_id('trellis_cell_minimum_height'),
    ];

    trellisSettings = trellisSettings.filter(setting => setting);
    if (isTrellis) {
      trellisSettings.forEach(setting => setting.enable());
      moreTrellisSettings.forEach(setting => setting.enable());
    } else {
      trellisSettings.forEach(setting => setting.disable());
      moreTrellisSettings.forEach(setting => setting.disable());
    }

    return isTrellis;
  };

  /**
   *
   * @desc  based on visual item being selected, generate appropriate params
   * @param d - data item being selected
   * @param config = {
   *
   *    hideUndefinedValues - boolean to keep/ignore params with undefined values
   *
    * }
   * @return params - generated from d based on being dim/agg and configs
   *
   */
  base.base_params_fn = function base_params_fn(d, config) {
    let params = base.getParams().getCopy();
    config = config || {};

    function addColToParams(colindex, keytype) {
      let param = {
        key: sb_colnames[colindex],
        value: base.d_string(d[colindex]),
        keytype: keytype,
        valuetype: sb_coltypes[colindex],
        source_visual_id: report_id,
        source_dataset_id: base.dataset_id(),
        source_shelf: sb_columns[colindex] && sb_columns[colindex].colsource ?
          sb_columns[colindex].colsource() :
          undefined,
      };

      if (config.hideUndefinedValues && typeof param.value === 'undefined') {

        // if current param value is undefined but it already existed in history then remove it
        let idx = -1;
        params.find(function (p, i) {
          if (p.key === param.key) {
            idx = i;
            return true;
          }
        });

        if (idx >= 0) {
          param.key = `${param.key}.ignore`;
          param.value = true;
          params.push(param);
        }
      } else {
        params.push(param);
      }
    }

    if (config.single_colindex !== undefined) {
      var keytype = arc.report.columnTypeEnum.DIMENSION;
      if ((base.agg_start_idx() >= 0) &&
          (config.single_colindex >= base.agg_start_idx())) {
        keytype = arc.report.columnTypeEnum.AGGREGATE;
      }

      addColToParams(config.single_colindex, keytype);
      return params;
    }

    if ((dim_start_idx === -1) && (agg_start_idx === -1)) {
      // This is for the table display. In the table display, without any
      // dimensions and measures specified, we run a select *. Parametrize
      // all values as dimensions in that case.
      d.forEach(function(col, idx) {
        addColToParams(idx, arc.report.columnTypeEnum.DIMENSION);
      });
      return params;
    }

    if (dim_start_idx >= 0) {
      for (var dim_idx = base.dim_start_idx(); dim_idx <= base.dim_end_idx();
          dim_idx++) {
        if (config && config.ignore_colindex === dim_idx) {
          return;
        }

        addColToParams(dim_idx, arc.report.columnTypeEnum.DIMENSION);
      }
    }

    if (agg_start_idx >= 0) {
      for (var agg_idx = base.agg_start_idx(); agg_idx <= base.agg_end_idx();
          agg_idx++) {
        if (config && config.ignore_colindex === agg_idx) {
          return;
        }

        addColToParams(agg_idx, arc.report.columnTypeEnum.AGGREGATE);
      }
    }
    return params;
  };

  base.default_formatter = arc.misc.default_formatter;

  // kvs is object with "key", "value", and optional "info" properties
  base.tooltip_from_kvs = function(kvs) {
  if (kvs.length === 0) {
    return '';
  }

  let textl;

  {
    let font = tooltipFont(),
      fontSize = tooltipFontSize();

    let classes = ['popover-table'];

    let attr = '';
    if (font !== undefined) {
      attr = ' data-fontclass="' + font + '" ';
      classes.push(font);
    }

    let s = '';
    if (fontSize !== undefined) {
      s = ' style="font-size:' + fontSize + 'px" ';
    }

    textl = [`<table data-test-id="visual-datapoint-tooltip" class="${classes.join(' ')}" ${attr}${s}>`];
  }

  let display_value;
  kvs.forEach((kv) => {
    textl.push('<tr><td class="header-label">');
    textl.push(arc.misc.escapeHTML(kv.key));
    textl.push('</td>');
    if (kv.formatter) {
      textl.push('<td class="viz-tooltip-num">');
      display_value = arc.misc.formattedDisplay(kv.formatter, kv.value, true);
    } else {
      display_value = arc.misc.default_formatter(kv.value);
      if ((kv.value === '') || isNaN(kv.value)) {
        if (kv.value_is_num) {
          textl.push('<td class="viz-tooltip-num">');
        } else {
          textl.push('<td class="viz-tooltip-string">');
        }
      } else {
        textl.push('<td class="viz-tooltip-num">');
      }
    }
    if (kv.ishtml) {
      textl.push(display_value);
    } else {
      textl.push(arc.misc.escapeHTML(display_value));
    }
    if (kv.info) {
      textl.push('<br/><small class="pull-right"><em>');
      textl.push(arc.misc.escapeHTML(kv.info));
      textl.push('</em></small>');
    }
    textl.push('</td></tr>');
  });
  textl.push('</table>');
  return textl.join('');
};

  base.text_from_column = arc.misc.text_from_column;

  base.text_from_columns = function(row, columns, config) {
    // The config element determins the key/value pairs that will be added
    // at the beginning (pre_kvs) and end (post_kvs) of the tooltip,
    // as well as if there is a primary_colindex
    var kvs = [];
    if (config && config.pre_kvs) {
      config.pre_kvs.forEach(function(kv) {
        kvs.push(kv);
      });
    }

    columns.forEach(function(col) {
      if (row[col.colindex()] === undefined) {
        return;
      }

      var kv = base.text_from_column(row, col);

      // override default number format on tooltip for crosstab
      if (viz_type === 'crosstab' && col.isNumeric() && !col.coldisplayformat(true)) {
        kv.formatter = d3.format(',');
      }

      if (config && config.primary_colindex !== undefined) {
        if (config.primary_colindex === col.colindex()) {
          kvs.unshift(kv);
          return;
        }
      }

      kvs.push(kv);
    });

    if (config && config.post_kvs) {
      config.post_kvs.forEach(function(kv) {
        kvs.push(kv);
      });
    }

    //remove duplicate key value pairs from tooltip text
    var map = Object.create(null);
    kvs = kvs.filter(function(kv){
      if(map[kv.key] === undefined){
        map[kv.key] = Object.create(null);
        map[kv.key][kv.value]=1;
        return true;
      } else if(map[kv.key][kv.value]){
        return false;
      }
      map[kv.key][kv.value]=1;
      return true;
    });

    return base.tooltip_from_kvs(kvs);
  };

  base.base_text_fn = function(d, config) {
    const kvs = [];
    const keyValueTracker = Object.create(null); // prevent duplicates
    function colToKv(colindex) {
      const key = sb_colnames && sb_colnames[colindex];
      const value = d[colindex];
      let formatter = base.formatter(colindex);

      // override default number format on tooltip for table, list and rawtable visuals
      if (['table', 'list', 'rawtable'].includes(viz_type) &&
            arc.report.coltypeIsNumber(sb_coltypes[colindex]) && !formatter) {
        formatter = d3.format(',');
      }

      if (!formatter) {
        formatter = base.d_string;
      }

      // Don't add duplicate to tooltip
      if (keyValueTracker[key] && keyValueTracker[key][value]) {
        return false;
      } else if (!keyValueTracker[key]) {
        keyValueTracker[key] = Object.create(null);
      }

      keyValueTracker[key][value] = 1;

      return {
        key,
        value,
        formatter,
      };
    }

    function addToKVS(colindex) {
      const colKv = colToKv(colindex);
      if (colKv) {
        kvs.push(colKv);
      }
    }

    if (config && (config.single_colindex !== undefined)) {
      if (Array.isArray(config.single_colindex)) {
        config.single_colindex.forEach(addToKVS);
      } else {
        addToKVS(config.single_colindex);
      }

      return base.tooltip_from_kvs(kvs);
    }

    // Special handling for tables without any dimensions or measures
    // dragged into the shelves.
    if (viz_type === 'table') {
      if ((dim_start_idx < 0) && (agg_start_idx < 0)) {
        sb_colnames.forEach(function(v, i) {
          addToKVS(i);
        });
      }
    }

    function pushToKVS(sIdx, fIdx) {
      for (var colIdx = sIdx; colIdx <= fIdx; colIdx++) {
        if (!(config && config.ignore_colindices && config.ignore_colindices.includes(colIdx))){
          kvs.push(colToKv(colIdx));
        }
      }
    }

    if (dim_start_idx >= 0) {
      pushToKVS(base.dim_start_idx(), base.dim_end_idx());
    }

    if (agg_start_idx >= 0) {
      pushToKVS(base.agg_start_idx(), base.agg_end_idx());
    }

    return base.tooltip_from_kvs(kvs);
  };

  base.clear_errors = function() {
    d3.select('#' + base.chart_div())
      .selectAll('.viz-error')
      .remove();
    return base;
  };

  function applyCustomClasses(filter_widget) {
    // Find the container
    var $sel;
    if (!base.in_dashboard()) {
      // If is search result, then get chart div in normal way
      $sel = $(`#${chart_div}`).closest('.arc-viz-container');
      if ($sel.length === 0) {
        $sel = $('.arc-viz-container, .main-app-view');
      }
    } else if (filter_widget) {
      $sel = $(`#${chart_div}`);
    } else {
      $sel = $(`#${chart_div}`).closest('.arc-viz-container');
    }

    // Add in a classname for the visual type
    if (viz_type) {
      var classToAdd = 'arc-viz-type-' + viz_type;
      var subclassToAdd = '';
      if (viz_type === 'picklist' && base.viz().selectionIsDropdown()) {
        subclassToAdd = 'arc-viz-type-'  + viz_type + '-dropdown';
      }
      var currClasses = [];

      // Delete previous classnames for visual types
      var ac = $sel.attr('class');
      if (ac) {
        currClasses = ac.trim().split(/\s+/);
        var toRemove = currClasses.filter(function(c) {
          return ((c !== classToAdd || c !== subclassToAdd) &&
                  (c.substr(0, 13) === 'arc-viz-type-'));
        });
        $sel.removeClass(toRemove.join(' '));
      }
      $sel.addClass(classToAdd);
      $sel.addClass(subclassToAdd);
      if (!filter_widget || !$sel.closest('li').hasClass('gs-w')) {
        //for filter widgets in grid, don't add arc-viz-container class. This is
        //not the best way to do it, ideally we shouldn't strip the info from the
        //DOM. But this is the safest way to do it at this point.
        $sel.addClass('arc-viz-container');
      }
    }

    if (!arc.globals.apply_custom_styles) {
      // Do not apply any styles if global settings disable them
      return;
    }

    let s = user_settings.find_by_id('container_css_class');
    if (!s) {
      return;
    }

    return arc.css.CUSTOMCSS_MANAGER.applyContainerClasses($sel, s.value());
  }

  base.removeDataTables = function(chart_div) {
    // Destroy datatable
    var $allTables = $('#' + chart_div).find('table');

    $allTables.each(function() {
      var $t = $(this);
      var dataTable = $t.data('data-table');
      if (dataTable) {
        dataTable.destroy(true);
      }
    });
  };

  // This function undraws all the chart visual elements from a previous
  // rendering of the chart. It also applies any container CSS class that
  // needs to be applied, sets up the title and subtitle elements, and
  // decides whether to hide or show the csv download button
  base.undraw = function(filter_widget, isTransition) {
    destroyMarks();
    destroyContextMenus();

    if (!isTransition) {
      base.destroy();
    }

    setupTitleAndButtons(filter_widget);
    return base;
  };

  base.destroy = function() {
    if (viz && viz.viz_destroy) {
      // Note that some visual types implement a viz_destroy interface (maps, network)
      // to cleanup
      viz.viz_destroy();
    }

    if (trellis) {
      trellis.destroy();
      trellis = undefined;
    }

    const chart_div = '#' + base.chart_div();
    const chart_div_container = d3.select(chart_div);

    // Destroy datatable
    base.removeDataTables(base.chart_div());

    chart_div_container
      .selectAll('table')
      .remove();
    chart_div_container
      .selectAll('svg')
      .remove();
    chart_div_container
      .selectAll('canvas')
      .remove();
    $(`${chart_div}>div:not(.viz-permanent)`)
      .remove();
    chart_div_container
      .on('click.outside', undefined);

    $(chart_div_container).find('div.contextmenu').hide();
  };

  function setupTitleAndButtons(filter_widget) {
    base.initializeCssAndTitle(filter_widget);

    // TTL: Should this be moved into setTitleText?
    if (afterTitleCb) {
      afterTitleCb();
    }

    initializeDashboardButtons();
    base.updateCsvSettings();

    // In Explore mode, when we are drawing visuals without
    // setting the title element, we need to set the overall
    // font on the container
    const $container = $(`#${chart_div}`).closest('.arc-viz-container');
    {
      let f = filter_widget ? filterFont() : visualFont();
      arc.misc.setFont($container, 'data-fontclass', f);
      setFontSize($container);
    }
  }

  function initializeDashboardButtons() {
    const $sel = $(`#${base.chart_div()}`).closest('.arc-viz-container');

    if ($sel.length === 0) {
      return;
    }
    if (arc.showme.noVisualExplore().includes(viz_type)) {
      $sel.find('.explore-container').addClass('arc-invisible');
    }

    var btns = d3.select($sel[0]).selectAll('.widget-btns-wrapper a');
    if (btns.size() === 0) {
      return;
    }

    arc.tooltip()
       .dom_id('#tooltip')
       .addSelection(btns)
       .text_fn(function(d, i, elem) {
         var k = $(elem).attr('data-tooltip-contents');
         return k;
       })
       .activate();
  }

  base.initializeCssAndTitle = function(filter_widget) {
    applyCustomClasses(filter_widget);
    base.setTitleText();
    if (displayFilters()) {
      drawParams();
    } else if (dashboard) {
      undrawParams();
    }

    return base;
  };

  base.afterTitleCb = function(_) {
    if (!arguments.length) return afterTitleCb;
    afterTitleCb = _;
    return base;
  };

  base.draw_error = function(msg) {
    base.undraw();
    d3.select('#' + base.chart_div())
      .append('div')
      .classed('viz-error', true)
      .text(msg);
    return base;
  };

  base.redrawLegendIfNeeded = function(trellis) {
    const $legendNode = trellis.getLegendContainer();
    const $visualNode = trellis.getVisualContainer();
    const legendPos = trellis.legendPositioning();
    if ($legendNode && $visualNode &&
        !arc.misc.isIEOptimisationEnabled() &&
        (legendPos === 'Above' || legendPos === 'Bottom')) {
      const $legendContainer = $legendNode.find('.arc-legend-container');
      const legendCellWidth = $legendContainer.width();
      const visualCellWidth = $visualNode.children().first().width();
      if (legendCellWidth < visualCellWidth) {
        $legendContainer.remove();
        trellis.draw_legend();
      }
    }
  };

  base.draw_legend = function(config) {
    /*
     * 'config' contains:
     *   'legendType': A string stating if it is a 'discrete', 'continuous', or
     *                  'area' legend.
     *   'legendPositioning': A string stating if the legend is positioned at
     *                   the 'Top', 'Right', or 'Bottom' of the trellis.
     *   'svg': The svg element to draw the legend in
     *   'width': The width of the svg element. This code assumes that the
     *            legend is drawn at the right edge of the svg element. It
     *            positions at the specified width as the starting point.
     *   'height': The height of the svg element
     *   'legendContainerWidth': The width of the container for the legend,
     *            used in calculations for the overall legend width
     *   'color': The color scale in use. This is where we get the colors to
     *            plot
     *   'legends': If specified, this is used as the list of legends to be
     *              drawn. If not specified, the domain of the color scale is
     *              used.
     *   'color_is_dim': The field on color shelf is a dimension, not an aggregate
     *   'color_palette': (optional) Defines the palette whose index is identified by color.
     *   'area_scale': (required for area legend) Instead of colors, this d3.scale maps legend_values
     *                  to the circle areas to be plotted on the legend.
     *   'opacity': If specified, this is applied as the opacity style to all
     *              colors that are drawn.
     *   'legend_column': The arc.data.column element for the legend. If
     *                    specified, this is used to show tooltips and context
     *                    menus for the legend elements.
     *   'legend_width': width for legend column display, given in pixels. Used to calculate
     *                    the character limit.
     *   'legend_title': String value for the title of the legend.
     *   'reverse_legend_order': By default, this value is false or undefined. If true, then
     *                  the order of the legend items is opposite of the default order.
     *                  Legends are ascending by default (unless the color order is specified
     *                  to be otherwise).
     *   'sort_domain': Sorts the domain if setting is true. By default, the legends are in
     *                  ascending order.
     *   'isTransition': Performs a transition on the legend instead of removing and redrawing
     *                  it fully.
     *    legendID: if you have more than one legend on a single chart, this is to uniquely identify
     *              said legend. It can be a number or a string.
     */

    let { legendType,
          legendPositioning,
          svg,
          width,
          height,
          legendContainerWidth,
          color,
          legends,
          color_is_dim,
          color_palette,
          area_scale,
          opacity,
          legend_column,
          legend_title,
          legend_width,
          legend_single_element_height,
          reverse_legend_order,
          sort_domain,
          isTransition,
          legendID,
          unformatted_legends,
          colorSettings,
        } = config;

    colorSettings = colorSettings || base.colorSettings();
    let colorPaletteType = colorSettings && colorSettings.type;
    legend_single_element_height = legend_single_element_height || 10;
    let main_g, small_g;
    if (isTransition) {
      main_g = svg.select('.large-legend').classed('arc-invisible', true);
      small_g = svg.select('.small-legend').classed('arc-invisible', false);
    } else {
      svg.selectAll('.viz-base-legend-g').remove();
      main_g =
        svg.append('g')
              .attr('class', 'viz-base-legend-g arc-invisible large-legend')
              .attr('transform',`translate(${width},0)`);
      small_g =
        svg.append('g')
              .attr('class', 'viz-base-legend-g small-legend')
              .attr('transform',`translate(${width},0)`);
    }

    color_palette = color_palette ? color_palette.concat() : undefined;

    if (!legends) {
      legends = color.domain();
    }
    let original_length = legends.length;

    if (sort_domain) {
      legends.sort(arc.misc.sortNums);
    }

    let color_shelf_element;

    color_shelf_element = base.getShelfElements('color_shelf')[0];
    let color_order = color_shelf_element && color_shelf_element.order ?
        color_shelf_element.order() : null;
    let current_asc;
    if (legendType === 'continuous' && unformatted_legends) {
      const currValue = Number(unformatted_legends[0]);
      const prevValue = Number(unformatted_legends[legends.length - 1]);
      current_asc = currValue < prevValue;
    } else {
      current_asc = legends[0] < legends[legends.length - 1];
    }

    // Continuous legends are descending for now.
    // For discrete legends, if color_order is not defined, we want ascending
    // order of the legends to be displayed
    let desired_asc;
    switch (legendType) {
      case 'discrete':
        desired_asc = (color_order ? color_order.ascending && color_order.ascending() :
                                     (!!sort_domain));
        break;
      case 'continuous':
        desired_asc = false;
        break;
      case 'area':
        desired_asc = true;
        break;
      default:
    }

    //reverse legend if appropriate
    const legendReverseRequired = desired_asc ^ current_asc;
    if (reverse_legend_order ^ legendReverseRequired) {
      legends.reverse();
      if (unformatted_legends) {
        unformatted_legends.reverse();
      }

      if (color_palette) {
        color_palette.reverse();
      }
    }

    let titleHeight = 0;
    if (legend_title) {
      titleHeight = 10;
      legend_title = truncateLabel(legend_title, legend_width);
    }

    switch (legendType) {
      case 'discrete':
        let heightPerLegend = legend_single_element_height + legendItemBottomMargin;
        let $svg = $(svg[0]);
        let svgHeight = $svg.closest('tr').height();

        let smallCount = 5;
        if (legendPositioning === 'Right' && svgHeight > 50) {
          smallCount = Math.max(smallCount, Math.floor((svgHeight / heightPerLegend) - 3));
        }

        let heightSmall, itemsPerRow;
        if (height) {
          let max_cnt = (height - titleHeight) / heightPerLegend - 1;
          legends = legends.slice(0, max_cnt);
        } else {
          if (legendPositioning === 'Right') {
            height = (legends.length + 2) * heightPerLegend + titleHeight;
            heightSmall = (smallCount + 2) * heightPerLegend + titleHeight;
          } else if (legendPositioning === 'Bottom' || legendPositioning === 'Above') {
            itemsPerRow = Math.floor(legendContainerWidth / legend_width) || 1;
            const numRows = Math.ceil(legends.length / itemsPerRow);
            height = numRows * heightPerLegend + titleHeight;

            const maxRowsToShow = 5;
            if (maxRowsToShow >= numRows) {
              heightSmall = height;
              smallCount = original_length;
            } else {
              heightSmall = (maxRowsToShow + 2) * heightPerLegend + titleHeight;
              smallCount = maxRowsToShow * itemsPerRow;
            }
          }
        }


        const isHierarchyElement = legend_column && legend_column.isHierarchy();

        if (isHierarchyElement) {
          const $legendContainerWrapper =
            $(main_g.node()).closest('.arc-legend-container').parent();
          const hierarchyDrillDownElement = legend_column.nextDrillDownColumnName();
          const hierarchyDrillUpElement = legend_column.prevDrillUpColumnName();

          const $hierarchyOptions = $(`<div class="hierarchy-clickable-controls">
            <span class="fa fa-arrow-up drill-up${hierarchyDrillUpElement ? ' active' : ''}"
              title="Drill Up${hierarchyDrillUpElement ?
                ' to ' + hierarchyDrillUpElement : ' disabled'}">
            </span>
            <span class="fa fa-arrow-down drill-down${hierarchyDrillDownElement ? ' active' : ''}"
              title="Drill Down${hierarchyDrillDownElement ?
                ' to ' + hierarchyDrillDownElement : ' disabled'}">
            </span>
            <span class="hierarchy-state">${legend_column.currentDrillDownColumnName()}</span>
          </div>`);

          $legendContainerWrapper.prepend($hierarchyOptions);
          const $drillUp = $hierarchyOptions.find('.fa-arrow-up');
          const $drillDown = $hierarchyOptions.find('.fa-arrow-down');

          $drillUp.off('click.drillUp')
                  .on('click.drillUp', function drillUpClick() {
            let shelfElement = getLastShelfElementMatching(legend_column);
            if (!shelfElement) {
              return;
            }

            var vals = shelfElement.dataset_dimhvals();
            shelfElement.dataset_dimhvals(vals.splice(0, vals.length - 1));
            shelfElement.dataset_dimhlevel(Math.max(shelfElement.dataset_dimhlevel() - 1, 0));
            legend_column.dimhlevel(Math.max(legend_column.dimhlevel() - 1, 0));
            base.refresh();
          });

          $drillDown.off('click.drillDown')
                    .on('click.drillDown', function drillDownClick() {
            let shelfElement = getLastShelfElementMatching(legend_column);
            if (!shelfElement || !legend_column.nextDrillDownColumnName()) {
              return;
            }

            var vals = shelfElement.dataset_dimhvals();
            vals.push(undefined);
            shelfElement.dataset_dimhvals(vals);
            const dimhlistLength = (shelfElement.dataset_dimhlist() || []).length;
            shelfElement.dataset_dimhlevel(Math.min(shelfElement.dataset_dimhlevel() + 1, dimhlistLength));
            legend_column.dimhlevel(Math.min(legend_column.dimhlevel() + 1, dimhlistLength));
            base.refresh();
          });
        }

        // Called to show large legend (to be displayed when more is clicked)
        draw_discrete_legend({
          g: main_g,
          original_length,
          base,
          legends,
          config,
          legendContainerWidth,
          legend_title,
          titleHeight,
          legend_width,
          legend_single_element_height,
          legendPositioning,
          itemsPerRow,
          isTransition,
          unformatted_legends,
        });

        svg.attr('data-height-large', height + 'px');

        legends = legends.slice(0, smallCount);
        if (unformatted_legends) {
          unformatted_legends = unformatted_legends.slice(0, smallCount);
        }

        // Called to show small legend (to be displayed on visual's first rendering)
        draw_discrete_legend({
          base,
          legends,
          legend_title,
          titleHeight,
          legend_width,
          legend_single_element_height,
          legendPositioning,
          legendContainerWidth,
          itemsPerRow,
          heightSmall,
          config,
          isTransition,
          g: small_g,
          original_length: Math.min(original_length, smallCount),
          hasMore: (original_length > smallCount),
          unformatted_legends,
        });

        svg.attr('data-height-small', heightSmall + 'px');
        $svg.css('height', heightSmall + 'px');

        svg.selectAll('.arc-see-all-legends').on('click', function() {
          let s = svg;
          let h = s.attr('data-height-large');

          s.selectAll('g.small-legend')
           .classed('arc-invisible', true);

          s.selectAll('g.large-legend')
           .classed('arc-invisible', false);

          s.style('height', arc.misc.parseD3Style(h));
        });
        break;
      case 'continuous':
        //@assert(config.color_is_dim == false)
        draw_continuous_legend({
            g: main_g,
            config,
            original_length,
            base,
            legend_width,
            legend_title,
            legendPositioning,
            color_palette,
            /* currently only used in maps */
            legends_to_show: unformatted_legends ?
              d3.extent(unformatted_legends, d => Number(d)) : legends.slice(0),
            legendID,
            isReversed: reverse_legend_order,
        });

        svg.style('height', config.height + 'px');
        main_g.classed('arc-invisible', false);
        break;
      case 'area':
        drawCircleLegend({
          g: main_g,
          legends,
          svg,
          area_scale,
          legend_column,
          legend_title,
          titleHeight,
          legend_width,
        });
        main_g.classed('arc-invisible', false);
        break;
      default:
    }
  };

  function truncateLabel(label, pixelLength) {
    // label is the string that you want to truncate
    // pixelLength is the positive pixel length that you want the string to fit into

    if (pixelLength < 0) {
      return label;
    }

    label = label + '';

    let test_svg =
      d3.selectAll('#' + chart_div)
        .append('svg')
        .classed('arc-viz-axis', true)
        .classed('testsvg', true);
    let test_text =
      test_svg.append('text')
              .style('opacity', '0');

    test_text.text(label);
    let l = test_text.node().getComputedTextLength();

    if (pixelLength >= l) {
      d3.selectAll('.testsvg').remove();
      return label;
    }

    for (let i = 0, len = label.length; i < len; i++) {
      let labelWithEllipses = label + '...';
      test_text.text(labelWithEllipses);
      let l = test_text.node().getComputedTextLength();

      if (l <= pixelLength) {
        d3.selectAll('.testsvg').remove();
        return labelWithEllipses;
      } else {

        // for very very large strings, converge faster
        if (((l / pixelLength) > 2) && (label.length > 10)) {
          label = label.slice(0, label.length / 2);
        } else {
          label = label.slice(0, -1);
        }
      }
    }

    d3.selectAll('.testsvg').remove();
    return '';
  }

  function drawLegendTitle(g, x, y, legend_title, isTransition) {
    let legendTitle;
    if (isTransition) {
      legendTitle = g.select('.arc-legend-title').transition();
    } else {
      legendTitle =
        g.append('text')
         .classed('arc-legend-title', true)
         .attr('text-anchor', 'middle');
    }

    legendTitle
      .attr('x', x + 'px')
      .attr('y', y + 'px')
      .text(legend_title);
  }

  function draw_discrete_legend(params) {
    let { g,
          config,
          original_length,
          base,
          hasMore,
          legends,
          legend_title,
          titleHeight,
          legend_width,
          legend_single_element_height,
          legendPositioning,
          legendContainerWidth,
          itemsPerRow,
          heightSmall,
          isTransition,
          unformatted_legends,
        } = params;

    let config_legends_dup = unformatted_legends ? unformatted_legends.slice() : legends.slice();
    let legend_column = config.legend_column;
    let color_shelf_element;
    color_shelf_element = base.getShelfElements('color_shelf')[0];

    const topMargin = 4, itemHeight = legend_single_element_height,
          labelLeftMargin = 2, legendColorRectWidth = itemHeight;
    const legendItemHeight = legendItemBottomMargin + itemHeight;
    const legend_extra_horizontal_margin = 40;

    let topMarginTotal = topMargin;
    let leftMargin = 10;

    if (legend_title) {
      // If theres a title, it looks better if we double the top margin
      topMarginTotal = topMarginTotal * 2 + titleHeight;
      drawLegendTitle(g, legend_width / 2, titleHeight, legend_title, isTransition);
    }

    let xPosRect, yPosRect;
    let xPosLabel, yPosLabel;
    if (legendPositioning === 'Above' || legendPositioning === 'Bottom') {
      let legend_width_rounded = Math.floor(legendContainerWidth / itemsPerRow);
      xPosRect = function(d, i) {
        return `${leftMargin + (i % itemsPerRow) * legend_width_rounded}px`;
      };
      xPosLabel = function(d, i) {
        return `${leftMargin + legendColorRectWidth + labelLeftMargin +
          (i % itemsPerRow) * legend_width_rounded}px`;
      };
      yPosRect = function (d, i) {
        return `${topMarginTotal + Math.floor(i / itemsPerRow) * legendItemHeight}px`;
      };
      yPosLabel = function (d, i) {
        return `${topMarginTotal + Math.floor(i / itemsPerRow) * legendItemHeight + (legendColorRectWidth / 2)}px`;
      };
    } else if (legendPositioning === 'Right') {
      xPosRect = function() {
        return `${leftMargin}px`;
      };
      xPosLabel = function() {
        return `${leftMargin + legendColorRectWidth + labelLeftMargin}px`;
      };
      yPosRect = function(d, i) {
        return `${(i * legendItemHeight) + topMarginTotal}px`;
      };
      yPosLabel = function(d, i) {
        // Align label with center of rect
        return `${(i * legendItemHeight) + topMarginTotal + (legendColorRectWidth / 2)}px`;
      };
    }

    let legend_rects = isTransition ? g.selectAll('.arc-legend-rect') : g.selectAll('.nosel');
    legend_rects = legend_rects.data(config_legends_dup, d => d);
    legend_rects.enter()
                .append('rect')
                .classed('arc-legend-rect', true);

    base.createTransitions(isTransition, legend_rects);
    addPropsToLegendRects(legend_rects);

    function addPropsToLegendRects(sel) {
      sel.attr('width', legendColorRectWidth + 'px')
         .attr('height', itemHeight + 'px')
         .attr('x', xPosRect)
         .attr('y', yPosRect)
         .style('fill', function(d) {
           return config.color(d);
         });

      let opacity = base.colorOpacity();
      if (opacity !== 1) {
        sel.style('opacity', opacity);
      }
    }

    let maxLabelLength = Math.max(0, legend_width - legend_extra_horizontal_margin);

    let legend_labels = isTransition ? g.selectAll('.arc-legend-label') : g.selectAll('.nosel');
    legend_labels = legend_labels.data(config_legends_dup, d => d);
    const legendLabelFontSize = base.getLegendFontSize() || 10;
    legend_labels.enter()
                 .append('text')
                 .attr('alignment-baseline', 'middle')
                 .style('font-size', legendLabelFontSize + 'px')
                 .classed('arc-legend-label', true);

    base.createTransitions(isTransition, legend_labels);
    addPropsToLegendLabels(legend_labels);

    function addPropsToLegendLabels(sel) {
      sel.attr('x', xPosLabel)
         .attr('y', yPosLabel)
         .text(function(d) {
           let colDisplayFunc;
           if (color_shelf_element) {
             if (color_shelf_element.display_format()) {
               colDisplayFunc = base.translatedDisplayFormat(color_shelf_element.display_format());
             }
           } else if (legend_column) {
             colDisplayFunc = legend_column.coldisplayformat();
           }

           d = base.formattedDisplay(colDisplayFunc, d);
           return truncateLabel(d, maxLabelLength);
         });
    }

    let legend_not_shown_text = isTransition ? g.selectAll('.arc-not-shown-text') : g.selectAll('.nosel');
    let data_text = config_legends_dup.length < original_length ? [1] : [];
    legend_not_shown_text = legend_not_shown_text.data(data_text, d => d);
    legend_not_shown_text.enter()
                         .append('text')
                         .classed('arc-not-shown-text', true);

    base.createTransitions(isTransition, legend_not_shown_text);
    addPropsToNotShownText(legend_not_shown_text);

    function addPropsToNotShownText(sel) {
      sel.attr('x', '2px')
         .attr('y', (((1 + config_legends_dup.length) * legendItemHeight) + topMarginTotal) + 'px')
         .text('(' + (original_length - config_legends_dup.length) + ' not shown)');
    }

    let legend_more_text = isTransition ? g.selectAll('.arc-see-all-legends') : g.selectAll('.nosel');
    legend_more_text = legend_more_text.data(hasMore ? [1] : [], d => d);
    legend_more_text.enter()
                    .append('text')
                    .attr('cursor', 'pointer')
                    .attr('x', leftMargin + 'px')
                    .classed('arc-see-all-legends', true)
                    .text('More');

    const legendHeightPadding = 10;
    if (hasMore) {
      config.legend_height = heightSmall + legendHeightPadding;
    } else {
      config.legend_height = topMarginTotal + legendHeightPadding +
        Math.max(1, Math.ceil(legends.length / itemsPerRow)) * legendItemHeight;
    }

    base.createTransitions(isTransition, legend_more_text);
    addPropsToMoreText(legend_more_text);

    function addPropsToMoreText(sel) {
      if (legendPositioning === 'Right') {
        // We place the "Click to see more" link (1 legend item height + 5 pixels)
        // below the last legend item
        sel.attr('y', ((config_legends_dup.length * legendItemHeight) + 15 +
            topMarginTotal + 5) + 'px');
      } else if (legendPositioning === 'Above' || legendPositioning === 'Bottom') {
        sel.attr('y', heightSmall);
        $(g.node()).closest('.arc-legend-container')
                   .height(`${config.legend_height}px`);
      }
    }

    var tooltips =
      arc.marks()
         .viz_lib(base)
         .addSelection(legend_rects)
         .addSelection(legend_labels)
         .text_fn(function(d, i) {
            if (config.legend_column) {
              return base.tooltip_from_kvs([{
                key: config.legend_column.colname(),
                value: d,
                formatter: config.legend_column.coldisplayformat()
              }]);
            }
            return d;
          })
          // Canvas and SVG type visulas differ in highlighting.
          // For SVG types an SVG filter is applied.
          // For canvas types the color is changed.
          // The default behavior for marks is to apply the SVG filter when highlighted.
          // This resulted in different colors for the legend and the rows in canvas type visuals.
          // Here the default behavior is overwritten.
          // Instead of changing the color of the legend rect, only a border is applied.
         .tooltip_highlight_cb((d, i, elem) => {
            const $elem = $(elem);
            if ($elem.prop('tagName') === 'rect') {
              const isMouseLeaveEvent = d === undefined && i === undefined;

              if (isMouseLeaveEvent) {
                $elem.removeClass('highlight-stroke-hover');
              } else {
                $elem.addClass('highlight-stroke-hover');
              }
            }
         });

    if (config.legend_column) {
      tooltips.marker_name('legend')
              .cm_hide_cb(config.cm_hide_cb)
              .cm_show_cb(config.cm_show_cb)
              .params_fn(function(d, i) {
                let params = base.getParams().getCopy();
                params.push({
                    key: config.legend_column.colname(),
                    value: d,
                    filtername: config.legend_column.colfiltername(),
                    keytype: config.legend_column.colproperty(),
                    valuetype: config.legend_column.coltype(),
                    source_visual_id: report_id,
                    source_dataset_id: base.dataset_id(),
                });
                return params;
              });
    }

    tooltips.activate();
  }

  function draw_continuous_legend(params) {
    let { g,
          config,
          base,
          original_length,
          // Should always be complete set of legends
          legends_to_show,
          color_palette,
          legend_width,
          legend_title,
          legendPositioning,
          legendID,
          isReversed = false,
        } = params;

    const legend_extra_horizontal_margin = 40;
    let palette_length = color_palette.length;
    let colorSettings = config.colorSettings;

    //This is added to ensure that color palettes with more than one measure do not get flipped
    if (colorSettings && colorSettings.type === 100) {
      const colorData = JSON.parse(colorSettings.value);
      if (colorData.colors.filter(col => col.value !== '').length > 1) {
        let firstColor = colorData.colors[0].color;
        firstColor = arc.colorpalette.rgbStringToRGB(firstColor);
        firstColor = arc.colorpalette.rgbToHex(...firstColor);
        if (color_palette[0] === firstColor && !isReversed) {
          color_palette.reverse();
        } else if (color_palette[0] !== firstColor && isReversed) {
          color_palette.reverse();
        }
      }
    }

    let delta = Math.ceil(100 / palette_length);
    let legend_size = delta * palette_length;

    let color_shelf_element;
    base.shelfElementCb('color_shelf', function (c) {
      color_shelf_element = c;
    });
    let legend_column = config.legend_column;

    let topMargin = 4;
    const titleHeight = 10;

    if (legend_title) {
      // If theres a title, it looks better if we double the top margin
      topMargin = topMargin * 2 + titleHeight;
      drawLegendTitle(g, legend_width / 2, titleHeight, legend_title, false);
    }

    legends_to_show.sort(function(a, b) {
      if (isReversed) {
        return Number(a) - Number(b);
      } else {
        return Number(b) - Number(a);
      }
    });

    let value_of_first_legend = Number(legends_to_show[0]),
        value_of_last_legend = Number(legends_to_show[legends_to_show.length - 1]),
        delta_value = value_of_last_legend - value_of_first_legend;

    if (isNaN(value_of_first_legend) || isNaN(value_of_last_legend)) {
      value_of_first_legend = legends_to_show[0];
      value_of_last_legend = legends_to_show[legends_to_show.length - 1];
      delta_value = legends_to_show.length;
    }

    let uniqueGradientId = `arc-gradient-${arc.misc.uniqueId()}`;

    let gradientConfig;
    const gradientThickness = 10;
    const paddingFromTop = 10 + topMargin;
    const paddingFromLeft = 10;
    const labelThickness = 11;
    if (legendPositioning === 'Right') {
      gradientConfig = {
        gradientWidth: `${gradientThickness}px`,
        gradientHeight: `${legend_size}px`,
        y1: 0,
        y2: legend_size,
        x1: 0,
        x2: 0,
        textAnchor: 'start',
        legendLabelX(d, i) {
          const labelLeftMargin = 2;
          return `${paddingFromLeft + gradientThickness + labelLeftMargin}px`;
        },
        legendLabelY(d, i) {
          // Move all labels to start of it's gradient color, then put it 5.5 px below
          return `${delta * d[1] + paddingFromTop + labelThickness / 2}px`;
        }
      };
      config.height = legend_size + paddingFromTop + labelThickness;
    } else if (legendPositioning === 'Above' || legendPositioning === 'Bottom') {
      const labelTopMargin = 2;
      const labelYPosition = paddingFromTop + gradientThickness + labelTopMargin + labelThickness;
      gradientConfig = {
        gradientWidth: `${legend_size}px`,
        gradientHeight: `${gradientThickness}px`,
        y1: 0,
        y2: 0,
        x1: 0,
        x2: legend_size,
        textAnchor: 'middle',
        legendLabelX(d, i) {
          return `${delta * d[1] + paddingFromLeft}px`;
        },
        legendLabelY(d, i) {
          const labelTopMargin = 2;
          return `${labelYPosition}px`;
        }
      };
      config.height = labelYPosition;
      const legendHeightPadding = 10;
      config.legend_height = labelYPosition + legendHeightPadding;
    }

    let gradient = g
      .append('defs')
      .append('linearGradient')
      .attr('y1', gradientConfig.y1)
      .attr('y2', gradientConfig.y2)
      .attr('x1', gradientConfig.x1)
      .attr('x2', gradientConfig.x2)
      .attr('id', uniqueGradientId)
      .attr('gradientUnits', 'userSpaceOnUse');

    let offset_array = [];
    for (let palette_pos = 0; palette_pos < palette_length; palette_pos++) {
      let offset = palette_pos / ( palette_length - 1 );
      offset_array.push([offset, palette_pos]);
      gradient
        .append('stop')
        .attr('offset', offset)
        .attr('stop-color', color_palette[palette_pos]);
    }

    let legend_rectangle =
      g.selectAll('.nosel')
       .data([1])
       .enter()
       .append('rect')
       .attr('width', gradientConfig.gradientWidth)
       .attr('height', gradientConfig.gradientHeight)
       .attr('x', `${paddingFromLeft}px`)
       .attr('y', `${paddingFromTop}px`)
       .style('fill', function() {
         return `url('#${uniqueGradientId}')`;
       });

    let o = base.colorOpacity();
    if (o !== 1) {
      legend_rectangle.style('opacity', function() {
        return o;
      });
    }

    // For gradient labels, only display the first, last and middle value
    let text_offset_array = offset_array.filter((e, i, a) => {
      return ((i === 0) || (i === a.length - 1) || (i === Math.floor(a.length / 2)));
    });

    if (colorSettings && colorSettings.value && colorSettings.type !== 1) {
      const colorObj = config.color;
      let m = colorObj.midColors().length,
          e = colorObj.endColors().length,
          ratio = e / (m + e);

      text_offset_array = colorObj.values().map((d, i) => {
        if (i === 0) {
          return [0, 0, d];
        }

        if (i === 1) {
          return [0, colorObj.dimColors().length * ratio, d];
        }

        if (i === 2) {
          return [0, colorObj.dimColors().length, d];
        }
      });
    }

    // We want to anchor the text to the end so that numbers align for
    // easy comparison. Find a location that the text ends at, offset from
    // the end of the legend div
    let maxLabelLength = Math.max(0, legend_width - legend_extra_horizontal_margin);
    let legend_labels =
      g.selectAll('.nosel')
       .data(text_offset_array)
       .enter()
       .append('text')
       .attr('text-anchor', gradientConfig.textAnchor)
       .attr('x', gradientConfig.legendLabelX)
       .attr('y', gradientConfig.legendLabelY)
       .text(function(d) {
          let t;

          if (d.length === 3) {
            t = d[2];
          } else {
            t = Math.round(d[0] * delta_value + value_of_first_legend);
          }
          let colDisplayFunc;
          if (color_shelf_element) {
            if (color_shelf_element.display_format()) {
              colDisplayFunc = base.translatedDisplayFormat(color_shelf_element.display_format());
            }
          } else if (legend_column) {
            colDisplayFunc = legend_column.coldisplayformat();
          }

          t = base.formattedDisplay(colDisplayFunc, t);
          return truncateLabel(t, maxLabelLength);
        });
  }

  function drawCircleLegend(params) {
    let { g,
          area_scale,
          legends,
          svg,
          legend_column,
          legend_title,
          titleHeight,
          legend_width,
        } = params;

    if (legend_title) {
      drawLegendTitle(g, legend_width / 2, titleHeight, legend_title);
    }

    let aggToRadiusLength = function(agg) {
      return Math.sqrt(area_scale(Number(agg)) / (Math.PI));
    };

    const verticalSpacingBetweenCircles = 5;
    const horizontalSpacing = 5;
    let circleConfigObjects = [], labelConfigObjects = [];
    let yPosition = titleHeight;
    let xPosition = 0;
    let labelFormatter = (legend_column ? legend_column.coldisplayformat() : null) ||
      area_scale.tickFormat(3);
    legends.forEach((label, i) => {
      let r = aggToRadiusLength(label);
      yPosition = yPosition + verticalSpacingBetweenCircles + r;

      if (i === legends.length - 1) {
        xPosition = horizontalSpacing + r;
      }

      let circle_obj = {
        r,
        y: yPosition,
        x: function() {
          return xPosition;
        }
      };

      let label_obj = {
        label: labelFormatter(label),
        y: yPosition,
        x: function() {
          return xPosition * 2;
        },
      };

      circleConfigObjects.unshift(circle_obj);
      labelConfigObjects.unshift(label_obj);
      yPosition = yPosition + r;
    });

    let legendItems =
      g.selectAll('.nosel')
        .data(circleConfigObjects)
        .enter()
          .append('circle')
          .attr('fill', '#AAAAAA')
          .attr('stroke', 'black')
          .attr('stroke-width', '2')
          .attr('r', function(d) {
            return d.r + 'px';
          })
          .attr('cy', function(d, i) {
            return d.y + 'px';
          })
          .attr('cx', function(d) {
            return d.x() + 'px';
          });

    let maxLabelLength = 2 * xPosition;
    let labels =
      g.selectAll('.nosel')
        .data(labelConfigObjects)
        .enter()
          .append('text')
          .attr('alignment-baseline', 'middle')
          .text(function(d) {
            return truncateLabel(d.label, maxLabelLength);
          })
          .attr('y', function(d, i) {
            return d.y + 'px';
          })
          .attr('x', function(d) {
            return d.x() + 'px';
          });

    svg.style({'width': (legend_width) + 'px',
             'height': (yPosition + verticalSpacingBetweenCircles) + 'px',
           });
  }

  base.draw_labels = function(svg, is_horizontal, width, height) {
    var sb_colnames = base.sb_colnames();
    var labels;
    if (is_horizontal) {
      labels = [
        sb_colnames[base.agg_start_idx()],
        sb_colnames[base.dim_start_idx()]
      ];
    } else {
      labels = [
        sb_colnames[base.dim_start_idx()],
        sb_colnames[base.agg_start_idx()]
      ];
    }
    return draw_xy_labels({
      svg: svg,
      width: width,
      height: height,
      labels: labels
    });
  };

  //TODO: whereever draw_xy_labels is being called, change labels object in config
  function draw_xy_labels(config) {
    let {
      svg,
      width,
      margin,
      height,
      labels,
      colored_labels,
    } = config;

    svg.selectAll('.arc-viz-axis-label').remove();
    var label_elems =
      svg.selectAll('.arc-viz-axis-label')
         .data(labels)
         .enter()
           .append('text')
           .attr('text-anchor', function(d, i) {
             if (i === 0) {
               return 'middle';
             }

             if (i === 1) {
               return 'start';
             }

             return 'end';
           })
           .attr('class', function(d, i) {
             if (i === 0) {
               return 'x arc-viz-axis-label';
             }
             return 'y arc-viz-axis-label';
           })
           .attr('x', function(d, i) {
             if (i === 0) {

               //d for date
               var label = d.name || 'd';
               if (label.length > base.main_axis_label_maxlen()) {
                 label = label.substr(0, base.main_axis_label_maxlen()) + '...';
               }

               return width / 2;
             } else if (i === 1) {
               return -5;
             } else {
               return width;
             }
           })
           .attr('y', function(d, i) {
             if (i === 0) {
               return height;
             }

             return -10;
           });

    var tooltipElems;
    if (colored_labels) {
      tooltipElems =
          label_elems.selectAll('tspan')
                     .data(function(d, i) {
                       d.forEach(function (l, idx) {
                         if (idx < d.length-1) {
                           l.name = l.name += ', ';
                         }
                       });
                       return d;
                     })
                     .enter()
                       .append('tspan')
                       .text(function(d) {
                         return d.name;
                       });

      tooltipElems.filter(function(l) {
                    return l.color !== undefined;
                  })
                  .attr('fill', function(l) {
                    return l.color;
                  });
    } else {
      tooltipElems = label_elems;
      label_elems.text(function(l) {
        var d = l.name;
        if (d.length > base.main_axis_label_maxlen()) {
          return d.substr(0, base.main_axis_label_maxlen()) + '...';
        } else {
          return d;
        }
      });
    }

    arc.tooltip()
       .dom_id('#tooltip')
       .addSelection(tooltipElems)
       .text_fn(function(l) {
        return arc.misc.escapeHTML(l.tooltip);
       })
       .activate();
  }

  base.removeMessageAboutMissingData = function($container) {
    $container.find('.viz-message-wrapper').remove();
    return base;
  };

  base.handleNoResultDiv = function($chartDiv) {
    // if there's no results, append "no results" icon
    if (!sb_rows.length) {
      if ($chartDiv.find('.viz-message-wrapper').length === 0 &&
        !arc.report.noResultExcludeTypes.has(base.viz_type())) {
        $chartDiv.append('<div class="viz-message-wrapper">No results found.</div>');
      }
    }
  };

  base.draw_sankey = function(config) {
    if (config === undefined) {
      config = {};
    }

    let {
      data,
      funnel_prefix,
      flow_type,
      goals_display,
      rows,
      columns,
      showFixedColor,
      for_correlation_flow,
      chart_div,
      columnCounts,
      linkColor,
      dropoffColor,
      highlightColor,
      minHeightPerNode,
      minWidthPerNode,
    } = config;

    minHeightPerNode = minHeightPerNode || 50;
    minWidthPerNode = minWidthPerNode || 150;

    if (!chart_div) {
      chart_div = base.chart_div();
    }

    linkColor = linkColor || '#a1a1a1';
    dropoffColor = dropoffColor || '#a1a1a1';
    highlightColor = highlightColor || '#4169e1';

    var color = base.dimensionalColorScale(showFixedColor);

    if (!data || !data.links || (data.links.length === 0)) {
      let $chart_div = $(`#${chart_div}`);
      if ($chart_div.find('.viz-message-wrapper').length === 0 &&
        !arc.report.noResultExcludeTypes.has(base.viz_type())) {
        $chart_div.append('<div class="viz-message-wrapper">No results found.</div>');
      }
      return;
    }

    var max_vertical_nodes = 0;

    var targets = {};
    var sources = {};
    data.links.forEach(function(d) {
      if (!(d.target in targets)) {
        targets[d.target] = 1;
      }
      if (!(d.source in sources)) {
        sources[d.source] = 1;
      }
    });
    for (var s in sources) {
      if (sources.hasOwnProperty(s)) {
        delete targets[s];
      }
    }
    for (var t in targets) {
      if (targets.hasOwnProperty(t)) {
        max_vertical_nodes++;
      }
    }

    var max_vertical_nodes_by_index = {};
    var max_horizontal_nodes = 0;
    data.nodes.forEach(function(d) {
      if (max_horizontal_nodes <= d.index) {
        max_horizontal_nodes = d.index + 1;
      }
      if (d.index in max_vertical_nodes_by_index) {
        max_vertical_nodes_by_index[d.index]++;
        if (max_vertical_nodes < max_vertical_nodes_by_index[d.index]) {
          max_vertical_nodes = max_vertical_nodes_by_index[d.index];
        }
      } else {
        max_vertical_nodes_by_index[d.index] = 1;
      }
    });

    var min_height = max_vertical_nodes * minHeightPerNode;
    var min_width = max_horizontal_nodes * minWidthPerNode;
    var fl_width, fl_height, $fl_elem;

    $fl_elem = $('#' + chart_div);
    fl_width = $fl_elem.width() - 25;
    fl_height = $fl_elem.height() - 30;
    fl_height -= 50;
    if (fl_height < 150) {
      fl_height = 150;
    }

    if (fl_height < min_height) {
      fl_height = min_height;
    }
    if (fl_width < min_width) {
      fl_width = min_width;
    }

    var computePrevValue = false;
    if ('prev_value' in data.links[0]) {
      computePrevValue = true;
    }

    var units = '';

    var margin = {top: 30, right: 20, bottom: 10, left: 10},
        width = fl_width - margin.left - margin.right,
        height = fl_height - margin.top - margin.bottom;

    var formatNumber = d3.format(',.0f'),
        format = function(d, formatter) {
          if (formatter) {
            return base.formattedDisplay(formatter, d) + ' ' + units;
          }
          return formatNumber(d) + ' ' + units;
        },
        formatPct = d3.format('.2p'),
        formatHPctSrc = function(d) {
          return formatPct(d.highfreq / d.source.value);
        },
        formatHPctTgt = function(d) {
          return formatPct(d.highfreq / d.target.value);
        },
        formatHPctLink = function(d) {
          return formatPct(d.highfreq / d.value);
        },

        formatPrev = function(d) {
          if (computePrevValue) {
            return d.prev_value;
          }
          return '';
        },
        formatDiff = function(d) {
          if (computePrevValue) {
            if (d.value > d.prev_value) {
              if (d.prev_value === 0) {
                return '\u25B2 \u221E';
              }
              return `\u25B2 ${d3.round((100 * (d.value - d.prev_value) / d.prev_value), 1)}%`;
            }
            if (d.value < d.prev_value) {
              if (d.prev_value === 0) {
                return '\u25BC \u221E';
              }
              return `\u25BC ${d3.round((100 * (d.value - d.prev_value) / d.prev_value), 1)}%`;
            }
            return 'Same';
          }
          return '';
        },
        formatDiff_br = function(d) {
          return `<br>${formatDiff(d)}`;
        };

    var bodywhole = d3.select('body')
       .on('click.body', function (d) {
          var element = $(document.elementFromPoint(d3.event.x, d3.event.y));
          if (!(arc.misc.hasClass(element, 'hlink') ||
                arc.misc.hasClass(element, 'link') ||
                arc.misc.hasClass(element, 'noderect') ||
                arc.misc.hasClass(element, 'node'))) {
            unhighlightLinks();
          }
        });
    d3.select('#' + chart_div).selectAll('svg').remove();

    var svg_base = d3.select('#' + chart_div).append('svg')
        .classed('viz-path-svg', true)
        .style('width', width + margin.left + margin.right + 'px')
        .style('height', height + margin.top + margin.bottom + 'px');

    var svg = svg_base.append('g')
                      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    var nodeWidth = 20;
    var nodePadding = 10;
    if (computePrevValue) {
      nodeWidth = 50;
    }

    var is_funnel = (flow_type === 'funnel');
    var movesinks = goals_display;

    var sankey = d3.sankey()
                   .width(width)
                   .nodeWidth(nodeWidth)
                   .nodePadding(nodePadding)
                   .size([width, height])
                   .movesinksright(movesinks);

    var lpath = sankey.link();

    sankey
        .computePrevValue(computePrevValue)
        .nodes(data.nodes)
        .links(data.links)
        .layout(32);

    var total_units = 0;
    data.nodes.forEach(function(d) {
      if (!(d.targetLinks) || (d.targetLinks.length === 0)) {
        total_units += Number(d.value);
      }
    });
    var formatPctTotalNum = function(d) {
      return formatPct(d.value/total_units);
    };
    var formatHPctTotal = function(d) {
      return formatPct(d.highfreq/total_units);
    };

    function funnel_element(d) {
      if (d.substring(0, funnel_prefix.length) === funnel_prefix) {
        return true;
      }
      return false;
    }

    function strip_funnel(d, formatter) {
      var formattedValue;
      if (is_funnel &&
          d.substring(0, funnel_prefix.length) === funnel_prefix) {
        formattedValue = d.substring(funnel_prefix.length);
      } else if (formatter) {
        // TODO: format for funnel cases as well
        formattedValue = base.formattedDisplay(formatter, d);
      } else {
        formattedValue = d;
      }

      return formattedValue;
    }

    var hlink; // set if any highlighted links are added, then used by dragmove
    var link = svg.append('g').selectAll('.link')
        .data(data.links)
        .enter()
          .append('path')
          .attr('class', 'link')
          .attr('d', lpath)
          .style('stroke', function(d) {
            if (!is_funnel) {
              d.link_color = linkColor;
              return linkColor;
            }
            if (funnel_element(d.source.name) &&
                funnel_element(d.target.name)) {
              d.link_color = highlightColor;
              return highlightColor;
            }
            d.link_color = linkColor;
            return linkColor;
          })
          .style('stroke-width', function(d) { return Math.max(1, d.dy); })
          .on('click', highlightAssociatedLinks)
          .sort(function(a, b) { return b.dy - a.dy; });

    if (is_funnel) {
      var funnel_links = data.links.filter(function(d) {
        if (funnel_element(d.source.name) &&
            funnel_element(d.target.name)) {
          return true;
        }
        return false;
      });
      svg_base.append('g')
              .selectAll('.funnel')
              .data(data.nodes.filter(function (node) {
                return node.display === 'Drop Off';
              }))
              .enter()
                .append('text')
                .attr('x', function(d) {
                  var link = d.targetLinks[0];
                  return margin.left + (link.source.x + link.target.x) / 2;
                })
                .attr('y', function(d) {
                  return margin.top - 10;
                })
                .html(function(d) {
                  return '&darr; ' +
                      formatPctTotalNum(d);
                });
    }

    function dimensionsToRow(sourceDim, targetDim) {
      var outRow;

      var dim_startidx = 0,
          dim_endidx = 1;

      if (!rows) {
        return undefined;
      }

      rows.some(function(row) {
        if ((row[dim_startidx] === sourceDim) &&
            (row[dim_endidx] === targetDim)) {
          outRow = row;
          return true;
        }

        return false;
      });

      return outRow;
    }

    function link_tooltip(d) {
      var kvs = [
        {
          key: strip_funnel(d.source.display, d.source.formatter) +
               ' → ' +
               strip_funnel(d.target.display, d.target.formatter),
          value: format(d.value, d.formatter),
        },
        {
          key: 'Percent of Total',
          value: formatPctTotalNum(d),
        },
        {
          key: 'Percent of Source',
          value: formatPct(d.value/d.source.value),
        },
        {
          key: 'Percent of Target',
          value: formatPct(d.value/d.target.value),
        },
      ];

      if (computePrevValue) {
        kvs.push({
          key: 'Previous value',
          value: d.prev_value,
        });

        kvs.push({
          key: formatDiff(d),
          value: '',
        });
      }

      var row = dimensionsToRow(d.source.display, d.target.display);
      if (row) {
        return base.text_from_columns(row, columns, {
          pre_kvs: kvs,
        });
      }

      return base.text_from_columns([], [], {
        pre_kvs: kvs,
      });
    }

    function link_params(d) {
      var kvs = [];
      if (for_correlation_flow) {
        var row = dimensionsToRow(d.source.display, d.target.display);
        if (row) {
          return base.params_from_columns(row, columns);
        }
      } else {
        kvs = [
           {
            key: colname(d.source.index) + '.src',
            value: strip_funnel(d.source.display, d.source.formatter),
            keytype: arc.report.columnTypeEnum.DIMENSION,
            valuetype: 'STRING',
            source_visual_id: report_id,
            source_dataset_id: base.dataset_id(),
          },
          {
            key: colname(d.target.index) + '.tgt',
            value: strip_funnel(d.target.display, d.target.formatter),
            keytype: arc.report.columnTypeEnum.DIMENSION,
            valuetype: 'STRING',
            source_visual_id: report_id,
            source_dataset_id: base.dataset_id(),
          },
          ];
      }
      return kvs;
    }

    if (for_correlation_flow) {
      arc.marks()
         .viz_lib(base)
         .marker_name('primary')
         .addSelection(link)
         .text_fn(link_tooltip)
         .params_fn(link_params)
         .permitIncludeAndExclude(false)
         .enableTooltipHighlights(false)
         .activate();
    } else {
      arc.marks()
         .viz_lib(base)
         .marker_name('link')
         .addSelection(link)
         .text_changes_on_move(true)
         .text_fn(link_tooltip)
         .params_fn(link_params)
         .permitIncludeAndExclude(false)
         .enableTooltipHighlights(false)
         .activate();
    }

    var node = svg.append('g').selectAll('.node')
        .data(data.nodes)
        .enter()
          .append('g')
          .attr('class', 'node')
          .attr('transform', function(d) {
            return `translate(${d.x}, ${d.y})`;
          })
          .call(d3.behavior.drag()
                           .origin(function(d) { return d; })
                           .on('drag', dragmove))
          .on('click', highlightAssociatedLinks);

    var node_rect =
      node.append('rect')
          .attr('class', 'noderect')
          .attr('height', function(d) { return d.dy; })
          .attr('width', sankey.nodeWidth())
          .style('fill', function fill(d) {
            if (is_funnel) {
              if (d.name.substring(0, funnel_prefix.length) === funnel_prefix) {
                d.color = color(d.name.replace(/_\d+/, ''));
                return d.color;
              }
              d.color = dropoffColor;
              return d.color;
            }
            d.color = color(d.name.replace(/_\d+/, ''));
            return d.color;
          })
          .style('stroke', function(d) { return d3.rgb(d.color).darker(2); });

    let opacity = base.colorOpacity();
    if (opacity !== 1) {
      node_rect.style('fill-opacity', opacity);
      node_rect.style('stroke-opacity', opacity);
    }

    function colname(i) {
      if (i === null) {
        return columnCounts[columnCounts.length - 1].colname;
      }

      if (i < columnCounts.length) {
        return columnCounts[i].colname;
      }

      let colname = 'unknown';
      columnCounts.some(function(c) {
        if (c.max) {
          colname = c.colname;
          return true;
        }

        return false;
      });

      return colname;
    }

    function rect_params(d) {
      var kvs = [
        {
          key: colname(d.index),
          value: strip_funnel(d.display, d.formatter),
          keytype: arc.report.columnTypeEnum.DIMENSION,
          valuetype: 'STRING',
          source_visual_id: base.report_id(),
          source_dataset_id: base.dataset_id(),
        },
      ];

      return kvs;
    }

    function rect_tooltip(d) {
      var kvs = [
        {
          key: strip_funnel(d.display, d.formatter),
          value: format(d.value, d.formatter),
        },
        {
          key: 'Percent of Total',
          value: formatPctTotalNum(d),
        },
      ];

      if (computePrevValue) {
        kvs.push({
          key: 'Previous value',
          value: d.prev_value,
        });

        kvs.push({
          key: formatDiff(d),
          value: '',
        });
      }

      if (for_correlation_flow) {
        kvs.push({
          key: columns[d.index].colname(),
          value: d.formatter(d.display),
        });
      }

      return base.text_from_columns([], [], {
        pre_kvs: kvs,
      });
    }

    if (for_correlation_flow) {
      arc.marks()
         .viz_lib(base)
         .addSelection(node_rect)
         .text_fn(rect_tooltip)
         .activate();
    } else {
      arc.marks()
         .viz_lib(base)
         .addSelection(node_rect)
         .text_changes_on_move(true)
         .text_fn(rect_tooltip)
         .params_fn(rect_params)
         .marker_name('node')
         .activate();
    }

    if (computePrevValue) {
      var textbg =
        node.append('rect')
            .attr('y', function(d) { return Math.max(0, (d.dy / 2) - 10); })
            .attr('width', sankey.nodeWidth())
            .attr('height', 15)
            .style('fill', 'lightgray');

      var updown_text =
        node.append('text')
            .attr('class', 'nodetext')
            .attr('dx', 2)
            .attr('dy', function(d) { return d.dy / 2; })
            .attr('fill', function(d) {
              if (d.value > d.prev_value) {
                return 'green';
              }
              if (d.value < d.prev_value) {
                return 'red';
              }
              return 'blue';
            })
            .text(function(d) {
              return formatDiff(d);
            });

      arc.marks()
         .viz_lib(base)
         .addSelection(textbg)
         .addSelection(updown_text)
         .text_changes_on_move(true)
         .text_fn(rect_tooltip)
         .activate();
    }

    node.append('text')
        .attr('x', -6)
        .attr('y', function(d) { return d.dy / 2; })
        .attr('dy', '.35em')
        .attr('text-anchor', 'end')
        .attr('transform', null)
        .text(function(d) {
          return strip_funnel(d.display, d.formatter);
        })
          .filter(function(d) { return d.x < width / 2; })
          .attr('x', 6 + sankey.nodeWidth())
          .attr('text-anchor', 'start');

    function dragmove(d) {
      d3.select(this).attr('transform', `translate( ${d.x}, ${(d.y = Math.max(0, Math.min(height - d.dy, d3.event.y)))})`);
      sankey.relayout();
      link.attr('d', lpath);
      if (hlink !== undefined) {
        hlink.attr('d', lpath);
      }
    }

    function unhighlightLinks() {
      svg.selectAll('.hlink').remove();
      hlink = undefined;
    }

    // Highlight all the links in the paths associated with the selected link
    // or node, by adding new highlighted links on top of the original ones
    function highlightAssociatedLinks(node, i) {

      // Don't do anything if dragged
      if (d3.event.defaultPrevented) {
        return;
      }

      unhighlightLinks();
      var associatedlinks = calculateAssociatedLinks(i, data, this);
      hlink = svg.append('g').selectAll('.hlink')
          .data(associatedlinks)
          .enter()
            .append('path')
            .attr('class', 'hlink')
            .attr('d', lpath)
            .style('stroke', function(d) {
              d.link_color = highlight_color;
              return highlight_color;
            })
            .style('fill', 'none')
            .style('stroke-width', function(d) { return Math.max(1, d.newdy); })
            .style('opacity', 0.5)

            // Allow highlighting upon clicking of already highlighted path
            .on('click', function(d, i) {
              unhighlightLinks(); // remove hlink paths (including one that was clicked)
              var clickEvent = new Event('click');

              // Trigger click event for underlying link path --
              // need to do this because highlightAssociatedLinks requires the correct 'i'
              // which we can't get from the hlink path
              document.elementFromPoint(d3.event.x, d3.event.y).dispatchEvent(clickEvent);
            })
            .sort(function(a, b) { return b.dy - a.dy; });

      function hlink_tooltip(d) {
        var kvs = [
          {
            key: strip_funnel(d.source.display, d.source.formatter) +
                "(pos:" + d.source.index + ")" + " → " +
                 strip_funnel(d.target.display, d.target.formatter),
            value: format(d.highfreq, d.formatter),
          },
          {
            key: 'Percent of Total',
            value: formatHPctTotal(d),
          },
          {
            key: 'Percent of Source',
            value: formatHPctSrc(d),
          },
          {
            key: 'Percent of Target',
            value: formatHPctTgt(d),
          },
          {
            key: "Percent of Link",
            value: formatHPctLink(d),
          }];

          if (computePrevValue) {
            kvs.push({
              key: 'Previous value',
              value: formatPrev(d),
            });

            kvs.push({
              key: formatDiff_br(d),
              value: '',
            });
          }
          return base.text_from_columns([], [], {
            pre_kvs: kvs,
          });

      }

      if (for_correlation_flow) {
        arc.marks()
           .viz_lib(base)
           .addSelection(hlink)
           .text_fn(link_tooltip)
           .activate();
      } else {
        arc.marks()
           .viz_lib(base)
           .text_changes_on_move(true)
           .addSelection(hlink)
           .text_fn(hlink_tooltip)
           .params_fn(link_params)
           .marker_name('hlink')
           .activate();
      }

      // Find all the links in the paths associated with the selected link/node
      function calculateAssociatedLinks(thiselmidx, inputdata, thisel) {

        var newlinks = [];
        var linksornodes;
        var thiselementclass = d3.select(thisel).attr('class');

        var pathidx, linkpath;
        inputdata.links.forEach(function(link, linkidx) {
          var highfreq = 0;
          for (pathidx = 0;
               pathidx < inputdata.links[linkidx].paths.length; pathidx++) {
            linkpath = inputdata.links[linkidx].paths[pathidx];

            if (thiselementclass === 'node') {
              linksornodes = inputdata.nodes[thiselmidx].paths;
            } else {
              linksornodes = inputdata.links[thiselmidx].paths;
            }
            if (linksornodes.indexOf(linkpath) !== -1) {
              highfreq += Number(inputdata.pathfrequencies[linkpath]);
            }
          }
          if (highfreq > 0) {
            link.highfreq = highfreq;
            link.newdy = (highfreq/link.value) * link.dy;
            newlinks.push(link);
          }
        });
        return (newlinks);
      }
    }
  };

  base.check_shelf_contents = function(constraints) {
    /*
     * 'constraints' is a hash containing:
     *   'minimum_dims': The minimum number of dimensions required for this visual
     *   'minimum_aggs': The minimum number of aggregates required for this visual
     *   'minimums':     The minimum number of columns in each shelf
     *   'maximums':     The maximum number of columns in each shelf
     */

    // If no columns are filled in, fail silently
    const _column = arc.report.shelf_types.COLUMN;
    let shelf_names = base.shelf_data().filter( d => d.shelf_type === _column ).map( d => d.name );

    let total_count = base.shelfElementCounts(shelf_names);
    if (total_count === 0) {
      // No elements are filled in. Do not alert, just fail.
      return false;
    }

    // Clear old error messages
    arc.report.alertMessageAllHide();

    const minDim = constraints.minimum_dims || 0;
    const minAgg = constraints.minimum_aggs || 0;
    const dimensionCount = base.dimensions_shelf_count();
    const aggregateCount = base.aggregates_shelf_count();

    if (dimensionCount < minDim || aggregateCount < minAgg) {
      return _shelfError1();
    }

    for (var shelfName in constraints.minimums || {}) {
      const minimum = constraints.minimums[shelfName];
      if (base.shelfElementCount(shelfName) < minimum) {
        return _shelfError2('least',  minimum, shelfName);
      }
    }

    for (shelfName in constraints.maximums || {}) {
      const maximum = constraints.maximums[shelfName];
      if (base.shelfElementCount(shelfName) > maximum) {
        return _shelfError2('most',  maximum, shelfName);
      }
    }

    return true;

    function _shelfError1() {
      const sa = minAgg > 1 ? 's' : '';
      const sd = minDim > 1 ? 's' : '';
      base.notificationMessage(`Please specify at least ${minDim} dimension${sd} and ${minAgg} aggregate${sa}`);
      return false;
    }

    function _shelfError2(mostleast, limit, name) {
      const s = limit > 1 ? 's' : '';
      const _shelfName = base.shelf_name_to_display_name(name);
      base.notificationMessage(`Please specify at ${mostleast} ${limit} element${s} in the ${_shelfName} shelf`);
      return false;
    }
  };

  base.Label = function(name, tooltip, color) {
    return {
      name: name || '',
      tooltip: tooltip || '',
      color: color,
    };
  };

  // _tickformat is custom display formatter
  // Need to use axis_formatting to create axis formatter in this case,
  // otherwise, dimensions not properly formatted
  base.axis_formatting = function(in_config) {
    let {
      axis_data,
      axis,
      len,
      optimalNumTicks,
      config,
      _tickformat,
    } = in_config;

    const is_datetime = axis_data.is_datetime();
    const is_date = axis_data.is_date();
    const domain = axis_data.scale().domain();

    let tickformat;
    if (axis_data.is_dim()) {
      if (config.disable_dim_labels) {
        axis.tickFormat(function() {
          return '';
        });

        axis.tickSize(0);
      } else if (is_date) {
        const tickFontSize = base.getAxisTicksFontSize() || 12;
        const dtConfig = arc.dates.computeTimeAxisConfig({
          domain,
          totalLen: len,
          fontSize: tickFontSize,
          customFormat: _tickformat,
          isDateTime: is_datetime,
        });

        if (dtConfig.tickFunction) {
          axis.ticks(dtConfig.tickFunction);
        }

        if (_tickformat) {
          axis.tickFormat(_tickformat);
        } else {
          axis.tickFormat(d3.time.format(dtConfig.tickFormat));
        }

      } else {
        const cnt = axis.scale().domain().length;
        tickformat = _tickformat;
        if (tickformat) {
          axis.tickFormat(tickformat);
        }

        if (cnt > optimalNumTicks) {
          var ignore = Math.ceil(cnt / optimalNumTicks);
          axis.tickFormat(function(d, i) {
            if ((i % ignore) === 0) {
              if (tickformat) {
                return tickformat(d);
              }

              return d;
            }

            $(this).closest('.tick').find('line').remove();
            return '';
          });
        }
      }
    } else {
      if (axis_data.is_percentage()) {
        tickformat = _tickformat ? _tickformat : d3.format('.0%');
        axis.tickFormat(tickformat);
      } else if (axis_data.is_log10()) {
        // The first parameter for log scale tickFormat is count - we
        // set it to 1 to trigger the minimum number of ticks, which
        // is 1 per power of 10
        axis.tickFormat(axis.scale().tickFormat(
            1, base.tickFormatDefaultNumeric));
      } else {
        tickformat = _tickformat ? _tickformat : base.tickFormatDefaultNumeric;
        axis.tickFormat(tickformat);
        const minAxisLength = 300;
        if (len < minAxisLength) {
          axis.ticks(Math.max(2, optimalNumTicks));
        }
      }
    }
  };

  base.draw_axes = function(svg, width, height, margin, config) {
    // config: x_axis_data, y_axis_data,
    //         x_is_dim, y_is_dim,
    //         xAxisSVG, yAxisSVG, gridlinesSVG,
    //         disable_dim_labels,
    //         isTransition
    var x_axis,
        y_axis,
        secondary_y_axis;

    let {
          isTransition,
          xAxisSVG,
          yAxisSVG,
          gridlineSVG,
          drawVerticalGridLine,
          drawHorizontalGridLine,
        } = config;

    var show_x_axis = (config.show_x_axis === undefined) || config.show_x_axis;
    var show_y_axis = (config.show_y_axis === undefined) || config.show_y_axis;

    if (config.x_axis_data) {
      if (config.x_axis_data.flip_ticks()) {
        var scale_copy = config.x_axis_data.scale().copy();
        var r = scale_copy.range();
        scale_copy.range([r[1], r[0]]);
        x_axis = d3.svg.axis()
                   .scale(scale_copy)
                   .orient('bottom');
      } else {
        x_axis = d3.svg.axis()
                   .scale(config.x_axis_data.scale())
                   .orient('bottom');
      }

      if (config.x_axis_data.ticks()) {
        x_axis.ticks(config.x_axis_data.ticks());
      }
    }
    if (config.y_axis_data) {
      y_axis = d3.svg.axis()
                 .scale(config.y_axis_data.scale())
                 .orient('left');
    }
    if (config.secondary_y_axis_data) {
      secondary_y_axis = d3.svg.axis()
                           .scale(config.secondary_y_axis_data.scale())
                           .orient('right');
    }

    [
      [config.x_axis_data, x_axis],
      [config.y_axis_data, y_axis],
      [config.secondary_y_axis_data, secondary_y_axis],
    ].forEach(function(axis_elem, i) {
      var axis_data = axis_elem[0];
      if (!axis_data) {
        return;
      }
      var axis = axis_elem[1];
      var len;
      if (i === 0) {
        len = width;
      } else {
        len = height;
      }

      const axisValues = axis.scale().ticks ? axis.scale().ticks() : axis.scale().domain();
      if (axis_data.tickFormat()) {
        try {
          const displayedAxisValues = axisValues.map(axis_data.tickFormat());
          const axisIsNaN = displayedAxisValues.some(v => {
            return (arc.format.formattingIsNaN(v));
          });

          if (axisIsNaN) {
            throw 'Axis display format making values NaN or is otherwise invalid';
          }

          // TODO: Make the axis formatter dynamic so there aren't repeated labels
          // Or just give the axis a different formatter (don't want to do that for now due to complexity)
          const optimalNumTicks = calculateOptimalNumTicks(displayedAxisValues);
          const xOffset = (i === 0) ? Math.floor(optimalNumTicks / 3) : 0;
          base.axis_formatting({
            axis_data,
            axis,
            optimalNumTicks,
            len,
            config,
            _tickformat: axis_data.tickFormat()
          });

          // Change ticks if the default is more than calculated maximum
          if (axis.ticks() > optimalNumTicks - xOffset) {
            // Calculate number of ticks based on width of labels if x_axis,
            // Calculate number of ticks using default width if y_axis
            axis.ticks(Math.max(2, optimalNumTicks - xOffset));
          }
        } catch (e) {
          console.log(e);
          const optimalNumTicks = calculateOptimalNumTicks(axisValues);
          base.axis_formatting({
            axis_data,
            axis,
            optimalNumTicks,
            len,
            config,
          });
        }
      } else {
        const optimalNumTicks = calculateOptimalNumTicks(axisValues);
        base.axis_formatting({
          axis_data,
          axis,
          optimalNumTicks,
          len,
          config,
        });
      }

      function calculateOptimalNumTicks(values) {
        const optimalTickLengthMargin = 2;
        let optimalTickLength;

        let bounds = base.axis_values_maxbounds(values, undefined, base.getAxisTicksFontSize());
        if (i === 0) {
          if (config.x_is_slanted) {
            optimalTickLength = bounds.height;
          } else {
            optimalTickLength = bounds.width;
          }
        } else {
          // i != 0, so we know this is the y axis
          if (config.x_is_dim) {
            optimalTickLength = 40;
          } else {
            optimalTickLength = bounds.height;
          }
        }

        optimalTickLength += optimalTickLengthMargin;

        return Math.floor(len / optimalTickLength);
      }

      // Remove duplicates,
      // especially for dates on a long graph and numbers with formatting
      if (axis.scale().ticks) {
        const uniqueAxisValues = arc.misc.arrayUnique(axisValues, (tick) => {
          const tickFormatting = axis.tickFormat();
          return tickFormatting(tick);
        });

        // Determine if the original domain is in descending order
        const domain = axis.scale().domain();
        const isDescending = domain[0] > domain[1];

        // Always use an ascending order domain for tick generation
        const ascendingDomain = isDescending ? [domain[1], domain[0]] : domain;

        const tickCount = axis.ticks()[0];
        let ticks;

        if (typeof tickCount === 'function') {
          ticks = tickCount(...ascendingDomain);
        } else {
          ticks = axis.scale().ticks(tickCount);
        }

        if (isDescending) {
          ticks = ticks.reverse();
        }

        if (ticks.length <= 1 || uniqueAxisValues.length < tickCount) {
          const newTickNumber = (axis_data.is_date() || !axis_data.is_dim()) ?
                                Math.max(1, uniqueAxisValues.length - 1) :
                                uniqueAxisValues.length;
          axis.ticks(newTickNumber);
        } else {
          axis.tickValues(ticks);
        }
      }
    });

    // If settings set to hide x axis label, remove that size from x axis height
    var x_label_margin = 0;
    if (!config.x_axis_label_off) {
      x_label_margin = 20;
    }

    if (x_axis && show_x_axis) {
      let x_labels;

      if (!xAxisSVG) {
        xAxisSVG = d3.select(svg.node().parentNode)
                     .append('svg')
                     .append('g')
                     .attr('class', 'x arc-viz-axis');
      }

      if (isTransition) {
        xAxisSVG
          .transition()
          .attr('transform', `translate(${margin.left},${margin.top + height})`)
          .call(x_axis);
      } else {
        xAxisSVG
          .attr('transform', `translate(${margin.left},${margin.top + height})`)
          .call(x_axis);
      }

      if (config.x_axis_data.is_date()) {
        /*
         * Date axes sometimes draw a tick mark outside the max length of the axis.
         * Delete any tick outside the axis.
         */
        const maxRange = x_axis.scale().range()[1];
        xAxisSVG.selectAll('.tick')
                .each(function(d) {
                  const v = x_axis.scale()(d);
                  if (v > maxRange) {
                    $(this).remove();
                  }
                });
      }

      if (config.x_is_dim) {
        //  Rotate x axis if horizontal chart
        //  ie: dimension is on the x axis and measure is in the y axis
        x_labels = xAxisSVG.selectAll('.x .tick text')
           .style('text-anchor','end')
           .attr('transform', function(d) {
               return `translate(-10, 0) rotate( ${base.x_slant()},0,0)`;
             });

        arc.tooltip()
           .dom_id('#tooltip')
           .addSelection(x_labels)
           .activate();

        if (x_labels[0]) {
          x_labels[0].forEach(function (label) {
            if (label.textContent.length > base.dim_tick_label_maxlen()) {
              label.innerHTML = label.textContent.substr(0, base.dim_tick_label_maxlen() - 3) + '...';
            }
          });
        }
      } else if (!config.x_axis_data.is_date()) {
        // Trim trailing zeros if axis has numbers
        x_labels = xAxisSVG.selectAll('.x .tick text')
                           .text(function (_number) {
                             var numberString = $(this).text();
                             return base.trimTrailingZeros(numberString);
                           });
      }
    }

    var y_labels;
    if (y_axis && show_y_axis) {

      if (!yAxisSVG) {
        yAxisSVG =
          svg.append('g')
             .attr('class', 'y arc-viz-axis');
      }

      if (isTransition) {
        yAxisSVG.transition().call(y_axis);
      } else {
        yAxisSVG.call(y_axis);
      }

      if (config.y_axis_data.is_date()) {
        /*
         * Date axes sometimes draw a tick mark outside the max length of the axis.
         * Delete any tick outside the axis.
         */
        const maxRange = y_axis.scale().range()[1];
        yAxisSVG.selectAll('.tick')
                .each(function(d) {
                  const v = y_axis.scale()(d);
                  if (v > maxRange) {
                    $(this).remove();
                  }
                });
      }

      if (config.y_is_dim) {
        // Add tooltips to y axis if vertical chart
        // ie: dimension is on the y axis and meausre is on the x axis
        y_labels = svg.selectAll('.y .tick text');

        arc.tooltip()
           .dom_id('#tooltip')
           .addSelection(y_labels)
           .activate();

        if (y_labels[0]) {
          y_labels[0].forEach(function (label) {
            if (label.textContent.length > base.dim_tick_label_maxlen()) {
              label.innerHTML = label.textContent.substr(0, base.dim_tick_label_maxlen() - 3) + '...';
            }
          });
        }
      } else if (!config.y_axis_data.is_date()) {
        // Trim trailing zeros if axis has numbers
        y_labels = svg.selectAll('.y .tick text')
                      .text(function (_number) {
                         var numberString = $(this).text();
                         return base.trimTrailingZeros(numberString);
                       });
      }
    }

    if (secondary_y_axis && show_y_axis) {
      let xScale = config.x_axis_data.scale();
      let secYAxisPos = xScale.rangeExtent ? xScale.rangeExtent()[1] : xScale.range()[1];
      svg.append('g')
         .attr('class', 'y arc-viz-axis arc-viz-secondary-axis')
         .attr('transform', `translate(${secYAxisPos}, 0)`)
         .call(secondary_y_axis);

      if (config.y_is_dim) {
        // Add tooltips to y axis if vertical chart
        // ie: dimension is on the y axis and meausre is on the x axis
        y_labels = svg.selectAll('.y .tick text');

        arc.tooltip()
           .dom_id('#tooltip')
           .addSelection(y_labels)
           .activate();

        y_labels[0].forEach(function (label) {
          if (label.textContent.length > base.dim_tick_label_maxlen()) {
            label.innerHTML = label.textContent.substr(0, base.dim_tick_label_maxlen() - 3) + '...';
          }
        });
      } else if (!config.y_axis_data.is_date()) {
        // Trim trailing zeros if axis has numbers
        y_labels = svg.selectAll('.y .tick text')
                      .text(function (_number) {
                         var numberString = $(this).text();
                         return base.trimTrailingZeros(numberString);
                       });
      }
    }

    if (gridlineSVG) {
      const drawVerticalGridLines =
      drawVerticalGridLine &&
      config.x_axis_data &&
      config.x_axis_data.is_dim();
      if (drawVerticalGridLines) {
        gridlineSVG
          .attr('width', width)
          .attr('height', height)
          .attr('transform', `translate(${margin.left}, ${margin.top})`)
          .append('g')
            .attr('class', 'x viz-bar-axis-grid')
            .attr('transform', `translate(0, ${height})`)
            .call(d3.svg.axis()
               .scale(config.x_axis_data.scale())
               .orient('bottom')
               .tickSize(-height, 0, 0)
               .tickFormat('')
            );
      }

      const drawHorizontalGridLines =
        drawHorizontalGridLine &&
        config.y_axis_data &&
        !config.y_axis_data.is_dim() &&
        !config.secondary_y_axis_data;
      if (drawHorizontalGridLines) {
        gridlineSVG
          .attr('width', width)
          .attr('height', height)
          .attr('transform', `translate(${margin.left},${margin.top})`)
          .append('g')
            .attr('class', 'y viz-bar-axis-grid')
            .call(d3.svg.axis()
                    .scale(config.y_axis_data.scale())
                    .orient('left')
                    .tickSize(-width, 0, 0)
                    .tickFormat('')
            );
      }
    }

    if (config.x_axis_data && config.x_axis_data.colname() && !config.x_axis_label_off ||
        config.y_axis_data && config.y_axis_data.colname() && !config.y_axis_label_off) {

        var label_data = [];
        var desc_data = [];
        var labels = [];

        if (config.secondary_agg_label) {
          if (config.x_axis_label_off) {
            labels.push([base.Label(undefined, undefined, undefined)]);
          } else {
            labels.push(config.x_label);
          }
          if (!config.y_axis_label_off) {
            labels.push(config.y_label);
            labels.push(config.secondary_agg_label);
          }
        } else {
          labels = [];
          labels.push(base.Label(
            (config.x_axis_data && !config.x_axis_label_off ?
                config.x_axis_data.colname() : ''),
            (config.x_axis_data.coldesc() || config.x_axis_data.colname() || '')
          ));
          labels.push(base.Label(
            (config.y_axis_data && !config.y_axis_label_off ?
                config.y_axis_data.colname() : ''),
            (config.y_axis_data.coldesc() || config.y_axis_data.colname() || '')
          ));

          if (config.secondary_y_axis_data && !config.y_axis_label_off) {
            labels.push(base.Label(
              config.secondary_y_axis_data.colname(),
              config.secondary_y_axis_data.coldesc() || config.secondary_y_axis_data.colname()
            ));
          }
        }

        draw_xy_labels({
          svg: svg,
          width: width,
          margin,
          height: height + margin.bottom - 10,
          labels: labels,
          colored_labels: config.secondary_agg_label
        });
    }

    setFontSize($(svg.node().parentNode));

    return {
      xAxisSVG,
      yAxisSVG,
    };
  };

  function setFontSize($container) {
    const axisLabelFontSize = base.getAxisLabelFontSize();
    if (axisLabelFontSize) {
      $container.find('.arc-viz-axis-label')
                .css('font-size', axisLabelFontSize + 'px');
    }

    const axisTicksFontSize = base.getAxisTicksFontSize();
    if (axisTicksFontSize) {
      $container.find('.arc-viz-axis text')
                .css('font-size', axisTicksFontSize + 'px');
    }

    const legendFontSize = base.getLegendFontSize();
    if (legendFontSize) {
      $container.find('.arc-viz-legend text')
                .css('font-size', legendFontSize + 'px');
    }
  }

  // Trim trailing zeros -- fixing d3's bugs
  base.trimTrailingZeros = function (numberString) {
    // create 3 groupings
    // 1st element is the full number (matches the full regex)
    // 2nd element matches the first grouping (up until 3+ zeros)
    // 3rd element is 3+ zeros and ending number
    var matches = /^(.*?\..*?)(0{3}0*[1-9])$/g.exec(numberString);
    if (matches) {
      // Take 2nd element of regex match and strips off ending period (eg if it was 1.000001)
      return matches[1].replace(/\.$/, '');
    } else {
      return numberString;
    }
  };

  // Return width and height of elements within a trellis
  base.computeTrellisElementParams = function(config) {
    let { trellis,
          viz_div,
          includeLegend,
          extraVerticalPadding, } = config;

    viz_div = `#${viz_div}`;
    extraVerticalPadding = extraVerticalPadding || 0;

    let legendLength = 0;
    let legendHeight = 0;
    if (includeLegend) {
      const legendPositioning = trellis.legendPositioning();
      if (legendPositioning === 'Right') {
        legendLength = trellis.legend_width();
      } else if (legendPositioning === 'Above' || legendPositioning === 'Bottom') {
        legendHeight = trellis.legend_height();
      }
    }

    const verticalPadding = 5;
    const horizontalPadding = 5;
    let verticalScrollbarWidth = arc.misc.estimateScrollbarWidth('vertical', viz_div);
    let horizontalScrollbarWidth = arc.misc.estimateScrollbarWidth('horizontal', viz_div);
    let $viz = $(viz_div);
    let elem_w = $viz.width() - legendLength - trellis.x_screen_len() -
      verticalScrollbarWidth - horizontalPadding;
    let elem_h = $viz.height() - horizontalScrollbarWidth - verticalPadding -
      legendHeight - extraVerticalPadding;

    let xTrellisLength = trellis.x_trellis().length;
    let yTrellisLength = trellis.y_trellis().length;

    // If there are X trellis elements, the extra rows for headers need space
    // on the Y axis
    if ((xTrellisLength > 0) || (yTrellisLength > 0)) {
      let $firstDataRow = $viz.find('.viz-trellis-table tr').not('.viz-tr-header').first();
      let headerSpacing = $firstDataRow.offset().top - $viz.find('.viz-trellis-table').offset().top;

      elem_h -= headerSpacing;

      // If there are Y values, the height of each element is sized proportionally
      if (yTrellisLength > 0) {
        elem_h = elem_h / trellis.unique_y_values().length;
      }

      // If there are X values, the width of each element is sized proportionally
      const extraWidthPadding = 20;
      if (xTrellisLength > 0) {
        elem_w = elem_w / trellis.unique_x_values().length - extraWidthPadding;
      }

      let $tableData = $firstDataRow.find('td:nth-child(1)');
      let cellTDPadding = $tableData.innerHeight() - $tableData.height();
      const extraBottomPadding = 5;
      elem_h = elem_h - cellTDPadding - extraBottomPadding;
      elem_w = elem_w - cellTDPadding;

    }

    elem_w = Math.max(elem_w, minimumTrellisWidth());
    elem_h = Math.max(elem_h, minimumTrellisHeight());

    return { elem_w, elem_h };
  };

  base.compute_svg_params = function(config) {
    // Defaults
    var normal_margin = config.normal_margin    || {top: 20, right: 20, bottom: 20, left: 20};
    var extra_margin = config.extra_margin      || {};
    extra_margin.top = extra_margin.top         || 0;
    extra_margin.right = extra_margin.right     || 0;
    extra_margin.bottom = extra_margin.bottom   || 0;
    extra_margin.left = extra_margin.left       || 0;
    var min_width = config.min_width            || 150;
    var min_height = config.min_height          || 75;

    var x_legend_margin = 20;
    var x_label_margin = 0;
    var legend_length = 0;
    var left_x_label_margin = 0;
    var right_x_label_margin = 0;
    var y_label_margin = 0;
    var showLegend = config.showLegend;

    var defaultLabelWidth = 25;

    var show_x_axis = (config.show_x_axis === undefined) || config.show_x_axis;
    var show_y_axis = (config.show_y_axis === undefined) || config.show_y_axis;

    let axisLabelFontSize = base.getAxisLabelFontSize();

    if (!show_x_axis) {
      normal_margin.bottom = 0;
    }

    var x_labels = [], y_labels = [], secondary_y_labels = [], legend_labels = [],
        x_label, legend_label,
        has_x_labels = show_x_axis &&
            (((config.x_idx !== undefined) && (config.x_idx !== -1)) ||
            config.x_formatter),
        has_y_labels = show_y_axis &&
            ((!config.x_is_dim && (config.y_idx !== undefined) && (config.y_idx !== -1)) ||
            config.y_formatter),
        has_secondary_y_labels = config.secondary_idx &&
                                 config.secondary_formatter,
        has_legend = showLegend &&
                     (config.legend_idx !== undefined) &&
                     (config.legend_idx !== -1);
    if (config.rows) {
      const labelVars = {
        primary: {
          hasLabels: has_y_labels,
          yIndices: config.y_all_indices,
          formatterName: 'y_formatter',
        },
        secondary: {
          hasLabels: has_secondary_y_labels,
          yIndices: config.y_second_all_indices,
          formatterName: 'secondary_formatter',
        },
      };

      config.rows.forEach(function addRowLabels(row) {
        if (has_x_labels) {
          x_label = row[config.x_idx];
          if (!arc.misc.check_if_value_null(x_label)) {
            x_labels.push(base.safeFormatting({
              value: x_label,
              formatterName: 'x_formatter',
              config: config
            }));
          }
        }

        // function to add axis labels for a particular y-axis to list
        function getYLabels(whichAxis) {
          let labels = [];
          let y_label_values = [];
          const axisVars = labelVars[whichAxis];

          if (axisVars.hasLabels) {
            if (axisVars.yIndices) {
              if (config.add_y_values) {
                y_label_values.push(
                  String(axisVars.yIndices.reduce((total, y_idx) => total + (Number(row[y_idx])), 0))
                );
              } else {
                axisVars.yIndices.forEach((y_idx) => y_label_values.push(row[y_idx]));
              }
            } else {
              y_label_values.push(row[config.y_idx]);
            }

            y_label_values.forEach(function addToLabelList(y_val) {
              if (config.y_is_dim || !arc.misc.check_if_value_null(y_val)) {
                labels.push(arc.format.safeFormatting({
                  value: y_val,
                  formatterName: axisVars.formatterName,
                  config: config,
                }));
              }
            });

            return labels;
          }
        }

        let ls = getYLabels('primary');
        if (ls) {
          y_labels.push(...ls);
        }

        ls = getYLabels('secondary');
        if (ls) {
          ls.forEach((l) => secondary_y_labels.push(l));
        }

        if (has_legend) {
          legend_label = row[config.legend_idx];
          legend_labels.push(legend_label);
        }
      });
    }

    // Additional bottom  margin for slanted x axis (ie dimension on x axis)
    if (has_x_labels) {
      x_labels = arc.misc.arrayUnique(x_labels);
      var max_x_len = config.x_is_dim ? base.dim_tick_label_maxlen() : base.xaxis_label_maxlen();
      // Calculations to slant x axis text
      if (config.x_is_slanted) {
        x_label_margin = 20;
        x_legend_margin = Math.sin(base.x_slant() * -1 * Math.PI / 180 ) *
                          base.dim_axis_max_textlen(x_labels, max_x_len,
                              base.getAxisTicksFontSize());
        left_x_label_margin = Math.cos(base.x_slant() * -1 * Math.PI / 180 ) *
                         base.dim_axis_max_textlen(x_labels.slice(0, 5), max_x_len,
                             base.getAxisTicksFontSize());
      } else {
        var num_x_labels = x_labels.length;
        right_x_label_margin = base.dim_axis_max_textlen(
            x_labels.slice(num_x_labels - 5, num_x_labels),
            max_x_len,
            base.getAxisTicksFontSize()) / 2;
      }
    }

    // Remove bottom or top margin if labels are hidden in settings
    let xLabelHeight = 10;

    if (config.x_axis_label_off) {
      x_label_margin -= xLabelHeight;
    } else {
      if (axisLabelFontSize) {
        if (config.columns && config.x_idx !== undefined) {
          let colname = config.columns[config.x_idx].colname();
          let b = base.axis_values_maxbounds([colname], undefined, axisLabelFontSize);
          x_label_margin = b.height;
        }
      }
    }

    let y_label_height = 0;
    if (config.y_axis_label_off) {
      normal_margin.top -= 10;
    } else {
      if (axisLabelFontSize) {
        if (config.columns && config.y_idx !== undefined) {
          let colname = config.columns[config.y_idx].colname();
          let b = base.axis_values_maxbounds([colname], undefined, axisLabelFontSize);
          y_label_height = b.height;
        }
      }
    }

    // Additional left margin for vertical graphs (ie dimension on y axis) or strangly formatted aggregates
    if (has_y_labels) {
      y_labels = arc.misc.arrayUnique(y_labels);

      var max_y_len = config.x_is_dim ? base.yaxis_label_maxlen() : base.dim_tick_label_maxlen();
      y_label_margin = base.dim_axis_max_textlen(y_labels, max_y_len, base.getAxisTicksFontSize());
    } else if (!show_y_axis) {
      y_label_margin = 0;
    } else {
      y_label_margin = defaultLabelWidth;
    }

    if (config.secondary_idx) {
      if (config.secondary_formatter) {
        secondary_y_labels = arc.misc.arrayUnique(secondary_y_labels);
        extra_margin.right += base.dim_axis_max_textlen(
                                secondary_y_labels,
                                base.yaxis_label_maxlen(),
                                base.getAxisTicksFontSize());
      } else {
        extra_margin.right += defaultLabelWidth;
      }
    }

    // Additional right margin for legend?
    if (has_legend) {
      var legends = arc.misc.arrayUnique(legend_labels);
      legend_length = base.dim_axis_max_textlen(legends, base.xaxis_label_maxlen(), base.getLegendFontSize());
    }

    var margin = {top:    normal_margin.top + extra_margin.top + y_label_height,
                  right:  normal_margin.right + legend_length + extra_margin.right + right_x_label_margin,
                  bottom: normal_margin.bottom + extra_margin.bottom +
                          x_legend_margin + x_label_margin,
                  left:   Math.max(normal_margin.left + extra_margin.left +
                          y_label_margin, left_x_label_margin)},
        width = config.width || $(config.viz_div_elem).width(),
        height = config.height || $(config.viz_div_elem).height();

    margin.top = Math.max(margin.top, 0);

    width = width - margin.left - margin.right;
    height = height - margin.top - margin.bottom;
    width = Math.max(width, min_width);
    height = Math.max(height, min_height);
    if (config.max_width) {
      width = Math.min(width, config.max_width);
    }

    if (config.max_height) {
      height = Math.min(height, config.max_height);
    }

    let r = {
      margin: margin,
      width: width,
      height: height
    };

    return r;
  };

  base.sizeChartDivHeight = function() {
    // Provide initial guess for chart_div based on viz type.
    setChartHeight();

    base.afterTitleCb(setChartHeight);

    function setChartHeight() {
      if (arc.misc.isIEOptimisationEnabled()) {
        return;
      }

      var $container,
          $chart_div = $(`#${chart_div}`),
          h;

      if (viz_type === 'timecontrol') {
        return;
      }

      if ($chart_div.hasClass('showme-visual')) {
        return;
      }

      if ($chart_div.hasClass('search-visual')) {
        base.handleNoResultDiv($chart_div);
        return;
      }

      if ($chart_div.length === 0) {
        return;
      }

      {
        let w = $chart_div.width();
        let $p = $chart_div.closest('.arc-viz-container').find('.progress-container');
        if ($p.length > 0) {
          $p.css('right', (w/2) + 'px');
        }
      }

      const hasNoData = sb_rows && sb_rows.length === 0 &&
        !arc.report.noResultExcludeTypes.has(base.viz_type());
      if ($chart_div.hasClass('inmodal')) {
        // offset() does not account for the padding of the body for hidden elements
        $container = $chart_div.closest('.main-app-modal');
        if ($container.length === 0) {
          $container = $('#main');
        }

        if ($container.length === 0) {
          return;
        }

        let $titleContainer = $container.find('.arc-viz-title-container');
        let titleHeight = $titleContainer.innerHeight();
        let negativeTopMargin = $titleContainer.css('margin-top').split('px')[0];
        if (!isNaN(negativeTopMargin)) {
          negativeTopMargin = Number(negativeTopMargin);
        }

        let subtitleHeight = $container.find('.arc-viz-subtitle-container').innerHeight();
        let padding = ($container.innerHeight() - $container.height());
        let scrollBarWidth = arc.misc.estimateScrollbarWidth('horizontal',
          `#${chart_div}`);
        h = $container.innerHeight() - titleHeight - subtitleHeight - padding -
          negativeTopMargin - scrollBarWidth;

        // Setup a minimum
        h = Math.max(h, 50);
        $chart_div.innerHeight(h);
        if (hasNoData) {
          if ($chart_div.find('.viz-message-wrapper').length === 0) {
            $chart_div.append('<div class="viz-message-wrapper">No results found.</div>');
          }
        } else {
          $chart_div.find('.viz-message-wrapper').remove();
        }

        return;
      }

      if (in_dashboard) {
        // If filter visual is being shown outside grid view then don't run
        // computation below
        if (!arc.report.in_dashboard_grid(chart_div)) {
          return;
        }

        // CA: Added .dash-viz-wrapper div inside gridser li and moved overflow to the new div
        // instead of the gridser element.  This is to solve the action button and resize handle
        // scrolling problem.
        var grid_e = $chart_div.closest('.gs-w');
        var grid_inner_e = $chart_div.closest('.dash-viz-wrapper');
        var grid_widget_e = $chart_div.closest('.arc-viz-container');

        // if no result, display no result
        if (hasNoData) {
          if (grid_inner_e.find('.viz-message-wrapper').length === 0) {
            grid_inner_e.append('<div class="viz-message-wrapper">No results found.</div>');
          }
        } else {
          grid_inner_e.find('.viz-message-wrapper').remove();
        }

        var grid_titles_e = grid_inner_e.offset().top - grid_e.offset().top;

        h = grid_e.innerHeight() - grid_titles_e -
                parseInt((grid_inner_e.css('padding-top') || '0').replace('px', '')) -
                parseInt((grid_inner_e.css('padding-bottom') || '0').replace('px', '')) -
                parseInt((grid_widget_e.css('margin-top') || '0').replace('px', ''));

        // Add about 2px of wiggle room. This is to prevent the scroll bar appearing
        // in dashboards when the gridster elements are placed if the fonts in the
        // titles are big
        h -= 2;

        if (h > 0) {
          $chart_div.innerHeight(h);
        }

        return;
      }

      $container = $('#main');
      if ($container.length === 0) {
        h = $chart_div.innerHeight();
      } else {

        // GOAL: To coincide bottom of #main and #chart-div
        // innerHeight includes top and bottom paddings
        // Top padding is removed as part of offset difference between #main
        // and #chart-div
        // Bottom padding needs to be explicitly removed
        h = $container.innerHeight() -
            ($chart_div.offset().top - $container.offset().top) -
            parseInt(($container.css('padding-bottom') || '0').split('px')[0]);
      }

      // Setup a minimum
      h = Math.max(h, 50);
      $chart_div.innerHeight(h);
    }
  };

  base.updateChartDivHeight = function() {
    // Update chart_div height that was set in sizeChartDivHeight if there
    // are horizontal scrollbars
    if (!chart_div) {
      return;
    }

    if (viz_type === 'timecontrol') {
      return;
    }

    const $chart_div = $('#' + chart_div);
    if ($chart_div.hasClass('showme-visual') || $chart_div.hasClass('search-visual')) {
      return;
    }

    let chart_el = document.getElementById(chart_div),
        parent_el = chart_el.parentElement,
        horizontal_scrollbar_width = parent_el.offsetHeight - parent_el.clientHeight;

    if (horizontal_scrollbar_width > 0) {
      chart_el.style.height = `${chart_el.clientHeight - horizontal_scrollbar_width}px`;
    }
  };

  base.idx_is_date = function(rows, idx) {
    if (!autodetectDates() || rows.length === 1) {
      return false;
    }

    // Do not make this a date axis unless there are at least 2 values
    let v;
    let twoValues = rows.some(function(r) {
      if (v === undefined) {
        v = r[idx];
        return false;
      }

      if (v !== r[idx]) {
        return true;
      }

      return false;
    });

    if (!twoValues) {
      return false;
    }

    var returnConfig = {
          is_date: false,
          is_datetime: false,
        };

    var c = arc.misc.columnIsDate(rows, idx);
    if (c) {
      returnConfig = {
        dateFormat: c.fmt,
        is_date: true,
        is_datetime: c.is_datetime,
      };
    }

    return returnConfig;
  };

  base.idx_is_numeric = function(rows, idx) {
    return arc.misc.idxIsNumeric(rows, idx);
  };

  var measureValues = [];
  base.measureValues = function(_) {
    if (!arguments.length) return measureValues;
    measureValues = _;
    return base;
  };

  base.extendExtent = function(extent) {
    measureValues.forEach(function(v) {
      if (extent[0] > v) {
        extent[0] = v;
        return;
      }

      if (extent[1] < v) {
        extent[1] = v;
        return;
      }
    });

    return extent;
  };

  /**
   * config is a dictionary with the following values:
   * rows: rows of data
   * idx(int): the index of column being used for the scale
   * range: range of the scale
   * is_date(bool): should the dimension be treated as a date for a continuous date axis
   * is_dim(bool): is the scale a dimension scale
   * is_log10scale(bool.): should the metric scale be put on a log10 scale
   * sort_dim(bool.): specify whether dimension values should be sorted
   * agg_scale_zero(bool.): should positive aggregate scales start at 0
   * bar_padding: additional padding for bar visuals
   * domain: if not provided, the domain is computed
   * isNumericContinuous: If it is desired to put the axis on a continuous numeric scale
   *  regardless of whether it is a dimension or not.
   *
   */

  base.idx_to_scale_config = function(config) {
    const { rows,
      idx,
      range,
      is_date,
      is_datetime,
      is_dim,
      is_log10scale,
      sort_dim,
      agg_scale_zero,
      minScale,
      maxScale,
      isSecondaryAxis,
      bar_padding,
      domain,
      isWaterfall,
      paddingPct,
      isNumericContinuous,
      width_for_each_bar } = config;

    var scale, extent;

    if (is_date) {
      extent = d3.extent(rows, function(r) {
                                  return r[idx];
                               });

      var asc = (rows[rows.length-1][idx] > rows[0][idx]),
          end_date, start_date;

      // For plotting dates, adjust the date plotted on the right to the right by
      // one day. Adjust the date plotted on the left only if there is bar padding
      // to make sure that the bar doesn't intersect with the y-axis.

      // For plotting datetimes, we don't know how granular the data is, and so
      // we don't want to adjust by the magnitude of the day. We will adjust the
      // time scale based upon the spacing of the bar.
      if (!asc) {
        extent.reverse();
      }

      if (is_datetime) {
        if (extent[0] instanceof Date) {
          start_date = extent[0];
          end_date = extent[1];
        } else {
          start_date = config.date_format.parse(extent[0]);
          end_date = config.date_format.parse(extent[1]);
        }
      } else {
        if (asc) {
          start_date = bar_padding !== undefined ? prevday(extent[0]) : new Date(extent[0]);
          end_date = nextday(extent[1]);
        } else {
          start_date = bar_padding !== undefined ? nextday(extent[0]) : new Date(extent[0]);
          end_date = prevday(extent[1]);
        }
      }

      scale = d3.time.scale()
                     .domain([start_date, end_date])
                     .range(range);

      if (is_datetime && width_for_each_bar) {
        // For datetime, we have not adjusted the scales based on the size of the
        // bars. If we do not do this, we will end up with the first and last bar
        // having their midpoint over the axes lines. Fix this by adjusting for
        // bar width below

        // Compute the date scale domain difference for a range difference half the
        // size of the bar
        var x = scale(start_date);
        var d = scale.invert(x + (width_for_each_bar / 2) + 2);
        var diffMs = Math.abs(d - start_date);

        // Add 1/2 bar width to the start date, 1 bar width to the end date to
        // fully encompass the last bar
        if (asc) {
          start_date = new Date(start_date - diffMs);
          end_date = new Date(Number(end_date) + diffMs * 2);
        } else {
          start_date = new Date(Number(start_date) + diffMs);
          end_date = new Date(end_date - diffMs * 2);
        }

        scale.domain([start_date, end_date]);
      }

      return scale;
    }

    if (is_dim && !isNumericContinuous) {
      var dim_values = rows.map(
                          function(row) {
                            return row[idx];
                          });
      dim_values = arc.misc.arrayUnique(dim_values);
      if (sort_dim) {
        dim_values.sort(arc.misc.sortNums);
      }

      if (isWaterfall) {
        dim_values.push(arc.misc.totalColumnName);
      }

      scale = d3.scale.ordinal()
                      .domain(dim_values);

      // bar_padding is coming from arc_visual_bar.js
      // we have bar_padding if the Dynamically size
      // graph settings is turned on
      if (config.bar_padding) {
          scale.rangeBands(range, config.bar_padding);
      } else if (base.isIndependentDimScale()) {
        // Independent dimensions scale across trellis is on
        const sum_width_of_bars = dim_values.length * width_for_each_bar;
        scale.rangeBands([range[0], sum_width_of_bars]);
      } else {
        // the graph is NOT trellis
        // or no above settings turned on
        scale.rangeBands(range);
      }

      return scale;
    }

    extent = d3.extent(rows, (row) => {
      let index;
      if (Array.isArray(idx)) {
        index = idx.reduce((p, v) => (Number(row[p]) > Number(row[v]) ? p : v));
      } else {
        index = idx;
      }

      return Number(row[index]);
    });

    extent = base.extendExtent(extent);

    if (extent[0] === extent[1]) {
      // Handle the case where the domain values are not distinct
      if (is_log10scale) {
        if (extent[0] > 1) {
          extent[0] = 1;
        } else {
          extent[0] = extent[1] * 0.9;
        }
      } else {
        if (extent[0] > 0) {
          extent[0] = 0;
        } else if (extent[0] < 0) {
          extent[1] = 0;
        }
      }
    }

    if (is_log10scale) {
      scale = d3.scale.log()
        .domain(extent)
        .nice()
        .clamp(true)
        .range(range);
    } else {
      if (agg_scale_zero && !is_dim) {
        if (extent[0] > 0) {
          extent[0] = 0;
        } else if (extent[1] < 0) {
          extent[1] = 0;
        }
      }

      if (domain) {
        extent = domain;
      }

      if (minScale) {
        extent[0] = minScale(extent[0], isSecondaryAxis);
      }

      if (maxScale) {
        extent[1] = maxScale(extent[1], isSecondaryAxis);
      }

      scale = d3.scale.linear()
                      .domain(extent)
                      .range(range);
    }

    return scale;
  };

  base.compute_dates = function(rows, dateFormat, date_colname, val_getter) {
    rows.forEach(function(d, i) {
      var v = val_getter(d);
      d[date_colname] = dateFormat.parse(v);
    });

    return base;
  };

  base.compute_percentages_multiple = function(rows, aggs_getter, colnames, by_color) {

    if (by_color) {
      var color_totals = {};
      colnames.forEach(function(colname) {
        color_totals[colname] = 0;
      });

      rows.forEach(function(d) {
        var aggs = aggs_getter(d);
        colnames.forEach(function(colname, i) {
          color_totals[colname] += aggs[i];
        });
      });

      rows.forEach(function(d) {
        var aggs = aggs_getter(d);
        for (var i = 0; i <= aggs.length; ++i) {
          d[colnames[i]] = aggs[i] / color_totals[colnames[i]];
        }
      });

      return base;
    }

    rows.forEach(function(d) {
      var aggs = aggs_getter(d);
      var t = aggs.reduce(function(pv, cv) {
        return pv + cv;
      }, 0);

      for (var i = 0; i <= aggs.length; i++) {
        d[colnames[i]] = aggs[i] / t;
      }
    });
    return base;
  };

  base.compute_percentages = function(rows, percentage_colname, dim_getter,
      agg_getter) {
    var totals_by_dim = {};
    // First, calculate the total for each dimension
    rows.forEach(function(d) {
      var dim = dim_getter(d),
          agg = agg_getter(d);

      if (!(dim in totals_by_dim)) {
        totals_by_dim[dim] = 0;
      }

      totals_by_dim[dim] += agg;
    });

    // Then, compute the percentage per dimension
    rows.forEach(function(d) {
      var dim = dim_getter(d),
          agg = agg_getter(d);

      if (agg === 0) {
        d[percentage_colname] = 0;
      } else {
        d[percentage_colname] = agg / totals_by_dim[dim];
      }
    });
    return base;
  };

  // Create an exit transition, and set the update/enter selection
  // to use a transition.
  base.createTransitions = function(isTransition, sel) {
    if (!isTransition) {
      return base;
    }

    const exit_sel = sel.exit().transition().remove();
    sel = sel.transition();
    return sel;
  };

  base.columnsOfType = function(columns, type) {
    return columns.filter(function(column) {
      return (column.colproperty() === type);
    });
  };

  base.shelf_columns = function(columns, shelf_name) {
    return columns.filter(function(column) {
      return (column.colsource() === shelf_name);
    });
  };

  base.first_column = function(columns, shelf_name) {
    for (var i = 0; i < columns.length; ++i) {
      if (columns[i].colsource() === shelf_name) {
        return columns[i];
      }
    }

    return undefined;
  };

  base.last_column = function(columns, shelf_name) {
    // Walk columns backwards, returning the last column for the shelf
    for (var i = columns.length - 1; i >= 0; --i) {
      if (columns[i].colsource() === shelf_name) {
        return columns[i];
      }
    }

    return undefined;
  };

  base.last_agg_columns = function(columns, shelf_name) {
    var agg_cols = [];

    // Walk columns backwards, returning the set of aggregate columns
    // at the end of the shelf
    for (var i = columns.length - 1; i >= 0; --i) {
      if (columns[i].colsource() !== shelf_name) {
        continue;
      }
      if (columns[i].colproperty() === arc.report.columnTypeEnum.DIMENSION) {
        return agg_cols;
      }
      agg_cols.unshift(columns[i]);
    }
    return agg_cols;
  };

  /*
   * config is a hash containing the following keys:
   *   rows: data rows
   *   columns: data columns
   *
   * When this function returns, it contains an array 'config.col_properties'
   * that contains a hash for each aggregate column in 'columns'
   * Each hash contains:
   *   'col': the aggregate column being tracked
   *   'nulls_present': whether there are nulls in the data
   *   'numeric': whether all the values in the data (apart from the nulls)
   *              are numbers
   *   'min': the minimum value
   *   'max': the maximum value
   *   'min_num': the minimum numeric value
   *   'max_num': the maximum numeric value
   *
   * This function sets the resulting rows in config.filtered_rows. Filters out
   *   rows with non-numeric aggregates. If
   *   filter_nulls is specified, then the rows being returned do not
   *   include aggregates values that are nulls. Otherwise, we return
   *   return rows filtered down to only numeric aggregates in config.filtered_rows
   */
  base.compute_agg_properties = function(config, filter_nulls) {
    config.filtered_rows = [];

    var col_aggs = config.columns.filter(function(col) {
      return (col.colproperty() === arc.report.columnTypeEnum.AGGREGATE) &&
             (col.colsource() !== 'tooltip_shelf') &&
             (col.colsource() !== 'label_shelf');
    });

    var col_properties = [];
    col_aggs.forEach(function(col) {
      col_properties[col.colindex()] =
        {
          col: col,
          nulls_present: 0,
          numeric: true,
          min: undefined,
          max: undefined,
          min_num: undefined,
          max_num: undefined
        };
    });
    config.col_properties = col_properties;

    config.rows.forEach(function(row) {
      var is_null = false;
      var aggsAreNumeric = true;
      col_aggs.forEach(function(col, idx) {
        const colprops = col_properties[col.colindex()];
        const agg = row[col.colindex()];
        const filterShelf = config.filterShelf ?
                            config.filterShelf === col.colsource() : true;
        if (filterShelf && arc.misc.check_if_value_null(agg)) {
          ++colprops.nulls_present;
          is_null = true;
          // If this is a null, no further analysis is necessary
          return;
        }

        var aggIsNaN = isNaN(agg);
        if (aggIsNaN) {
          aggsAreNumeric = false;
        }

        if (colprops.min === undefined) {
          colprops.min = agg;
          colprops.max = agg;
        } else {
          colprops.min = Math.min(colprops.min, agg);
          colprops.max = Math.max(colprops.max, agg);
        }

        if (colprops.numeric) {
          if (aggIsNaN) {
            colprops.numeric = false;
          } else {
            var agg_num = Number(agg);
            if (colprops.min_num === undefined) {
              colprops.min_num = agg_num;
              colprops.max_num = agg_num;
            } else {
              colprops.min_num = Math.min(colprops.min_num, agg_num);
              colprops.max_num = Math.max(colprops.max_num, agg_num);
            }
          }
        }
      });

      //Filter out rows with nulls or where the aggregate(s) are not numeric
      if (aggsAreNumeric) {
        if (filter_nulls) {
          if(!is_null){
            config.filtered_rows.push(row);
          }
        } else {
          config.filtered_rows.push(row);
        }
      }
    });

    return base;
  };

  base.filterDateNulls = function(config) {
    config.has_sort_order = base.has_sort_order(config);
    if ((config.presorted) &&
        (config.ordercol.colindex() !== config.dim_orig_start_idx)) {
      config.dim_is_date = false;
    } else {
      config.dim_is_date = base.idx_is_date(config.rows, config.dim_orig_start_idx).is_date;
      if (config.dim_is_date) {
        config.rows = base.filter_nulls(config, [function(row) {
          return row[config.dim_orig_start_idx];
        }]);
      }
    }
  };

  base.filter_nulls_bar = function(config) {
    var c = {
      rows: config.rows,
      columns: config.columns,
      filterShelf: config.filterShelf,
    };
    base.compute_agg_properties(c, true);
    config.col_properties = c.col_properties;
    if (config.rows.length > c.filtered_rows.length) {
      const skippedRows = config.rows.length - c.filtered_rows.length;
      const message = `Ignoring ${skippedRows > 1 ? skippedRows + " rows": skippedRows + " row"}
                      with NULL, NaN, infinity measures, and/or non-numeric aggregates.`;
      if (base.displayFilterWarnings()) {
        base.alertMessage(message, 'Note that this message will not show in dashboard view mode');
      } else {
        console.log(message);
      }
    }
    config.rows = c.filtered_rows;
    return base;
  };

  base.filter_nulls = function (config, getters) {
    config.null_row_count = 0;
    var rows = config.rows.filter(function(d) {
      for (var i = 0; i < getters.length; i++) {
        if (arc.misc.check_if_value_null(getters[i](d))) {
          config.null_row_count++;
          return false;
        }
      }
      return true;
    });

    const nullCount = config.null_row_count;
    if (nullCount > 0) {
      const message = `Ignoring ${nullCount}` +
                    ' rows with NULL, non-numeric (NaN), and/or infinity measures.';
      if (base.displayFilterWarnings()) {
        base.alertMessage(message, 'Note that this message will not show in dashboard view mode');
      } else {
        console.log(message);
      }
    }
    config.rows = rows;
    return rows;
  };

  base.nulls_order = function() {
    if (!user_settings) {
      return undefined;
    }

    var s = user_settings.find_by_id('nulls_order');
    if (!s) {
      return undefined;
    }

    let v = s.value();
    if (v === 'Default') {
      return undefined;
    }

    return s.value();
  };

  base.nulls_order_with_default = function() {
    const defaultSortOrder = 'Last';

    let v = base.nulls_order();
    if (!v) {
      return defaultSortOrder;
    }

    return v;
  };

  base.showData = function() {
    if (!base.enableViewData()) {
      return;
    }

    if (!base.dataResult) {
      return;
    }

    const result = base.dataResult();
    if (!result) {
      return;
    }

    arc.showdata.display()
       .viz(base)
       .result(result)
       .show();
  };

  /*
   * In recommended-visual codepaths, we may come into this visual with an
   * aggregation in the aggregates shelf that still has an agg applied. Remove
   * that here.
   */
  base.removeSimpleAggregations = function(settings) {
    if (!settings.aggregates_shelf) {
      return;
    }

    settings.aggregates_shelf.forEach(function(agg, idx) {
      let expr = agg.custom_expr || agg.expression_for_trigger;
      let bracketIndex = expr.indexOf('(');
      if (bracketIndex <= 0) {
        return;
      }

      let simpleAgg = expr.substring(0, bracketIndex);
      if (arc.report.supported_aggregates[simpleAgg.toLowerCase()]) {
        if (expr[expr.length - 1] === ')') {
          agg.custom_expr = expr.substring(bracketIndex + 1, expr.length - 1);
        } else {
          agg.custom_expr = expr.substring(bracketIndex);
        }
      }
    });

    return base;
  };

  base.displayFilterWarnings = function() {
    // Only show null, NaN, infinity filter warnings if we are in dashboard
    // edit mode or we are in the superModal in single visual mode.
    return enableFilterWarnings && arc.globals.is_edit && (arc.globals.is_dashboard || base.inModal());
  };

  return base;
};

}());
