window.arc = window.arc || {};

(function() {
'use strict';

arc.snapshot = arc.snapshot || {};

arc.snapshot = function() {
  var snapshot = function() { return 'arc.snapshot'; };

  var is_dashboard,
      dashboard;

  snapshot.is_dashboard = function(_) {
    if (!arguments.length) return is_dashboard;
    is_dashboard = _;
    return snapshot;
  };

  snapshot.dashboard = function(_) {
    if (!arguments.length) return dashboard;
    dashboard = _;
    return snapshot;
  };

  function getPageParams(doNotIncludeParameter, format) {
    var p;
    if (doNotIncludeParameter) {
      p = [];
    } else if (dashboard && !arc.misc.isInSuperModal()) {
      var h = dashboard.params_history();
      p = h.current_params()[0];
    } else {
      p = arc.globals.reports.getPrimaryReport().current_params()[0];
    }

    var l = ['embed=true'];

    arc.misc.getURLParams(l, p, dashboard, format);

    return l.join('&');
  }

  function getAppDimensions(params) {
    let $main = $('div#main.main-app');
    const { isDashboard, inModal, vizId } = params;
    if (isDashboard && !inModal && !vizId) {
      $main = $('div#main.main-app');
    } else if (isDashboard && !inModal) {
      if (vizId) {
        $main = $(`li[data-arc-reportid=${vizId}] div.arc-viz-container`);
      }

      if (!$main.length) {
        $main = $('div#main.main-app');
      }
    } else if (isDashboard && inModal) {
      $main = $('div.super-modal .main-app-modal.arc-viz-container');
    } else if (!isDashboard && inModal) {
      $main = $('div#main.main-app-modal');
    } else if (!isDashboard && !inModal) {
      $main = $('div#main.main-app-view');
    }

    if (!$main.length) {

      // can't think of scenario that this should occur
      $main = $('div#main').first();
    }

    let width = $main.outerWidth(true),
        mainHeight = $main.outerHeight(true);
    if (inModal || !isDashboard) {
      width = $main[0].scrollWidth || width;
      mainHeight = $main[0].scrollHeight || mainHeight;
    }

    let height = 0;
    $.each($main.find('> div'), function (i, div) {
      const $div = $(div);
      if ($div.hasClass('arc-viz-title-container')) {
        if (!$div.find('.arc-viz-title').text().trim()) {
          return;
        }
      }

      if ($div.hasClass('arc-viz-subtitle-container')) {
        if (!$div.find('.arc-viz-subtitle').text().trim()) {
          return;
        }
      }

      if (isDashboard && $div.hasClass('control-container')) {
        if (!$div.find('ul li').length) {
          return;
        }
      }

      height += $div.outerHeight(true);
    });

    height = Math.max(mainHeight, height);

    // if -1, we take the width/height values from settings in views_snapshot.py
    return {
      width: width || -1,
      height: height || -1,
    };
  }

  snapshot.getSnapshotParams = function(doNotIncludeParameter, app_id, format) {
    var pngParams = {};

    pngParams.saveType = 'screenshot';
    pngParams.paramString = getPageParams(doNotIncludeParameter, format);

    // snapshot.is_dashboard means on dashboard whether clicked on visual's or dashboard's icon
    // getAppDimensions is looking for whether user is in dashboard or not
    const params = {
      isDashboard: arc.globals.is_dashboard,
      inModal: arc.misc.isInSuperModal(),
      vizId: app_id,
    };

    const dbDim = getAppDimensions(params);
    pngParams.dbWidth = dbDim.width;
    pngParams.dbHeight = dbDim.height;
    pngParams.downloadFormat = format;

    return pngParams;
  };

  /**
   * Submits snapshot for download and starts download when ready.
   * Check that appId and title are populated, retrieving their values if needed.
   * @param {string} format: type of image to download (PNG or PDF)
   * @param {Object} [config] - configuration for downloading image
   * @param {boolean} config.isVisual - option to explicitly download visual, if false
   *                          can sometimes download as visual if `dashboard` is undefined.
   * @param {number} config.appId - dashboard id if dashboard or visual id for visual
   * @param {string} config.title - user submitted title of dashboard or visual
   */
  snapshot.downloadImage = function(format, config = {}) {
    let {
      appId,
      isVisual,
      title,
    } = config;

    if (!format) {
      format = 'png';
    }

    if (isVisual || !dashboard) {
      if (!$.isNumeric(appId)) {
        appId = arc.globals.reports.getPrimaryReport().report_id();
      }

      const vizHtmlTitle = $('#viz-title').length > 0 ?
        $('#viz-title').html().trim() :
        null;
      title = title || vizHtmlTitle || `Image For Visual ${appId}`;
    } else {
      if (!$.isNumeric(appId)) {
        appId = dashboard.dashboard_id();
      }

      title = title || dashboard.dashboard_title() || `Image For Dashboard ${appId}`;
    }

    if (appId <= 0) {
      arc.report.alertMessageSmallBase({
        type: 'alert-warning',
        alertHeader: 'Image generation is disabled',
        alertMessage: 'Images can only be downloaded for new dashboards after changes ' +
            'are saved. Click on the \'Save\' button first.',
        fadeOut: false,
      });
      return;
    }

    const pngUrl = `${arc.frontend.globals.urls.snapshotUrl}/${appId}`;
    const pngParams = snapshot.getSnapshotParams(false, appId, format);
    const pngFileName = arc.misc.getTimestampFilename(title);

    $.ajax({
      url: pngUrl,
      data: pngParams,
      type: 'POST',
      beforeSend: function(xhr) {
        // Because of a jquery bug when handling PUT requests,
        // we have to explicitly set X-CSRFToken here
        xhr.setRequestHeader('X-CSRFToken', arc.globals.csrf_token);
      },
    })
    .done(function(resp) {
      arc.report.alertMessageSmallBase({
        type: 'alert-success',
        alertHeader: 'Generating the requested file',
        alertMessage: 'The file will be available in a few seconds...',
        fadeOut: false,
      });

      var snapshotId = resp.snapshotId;

      function downloadSnapshot() {
        var snapshotLink = document.createElement('a');
        snapshotLink.download = pngFileName;
        snapshotLink.href = `${arc.frontend.globals.urls.snapshotFetchUrl}/${snapshotId}?title=${pngFileName}`;
        if (typeof MouseEvent === 'function') {
          snapshotLink.dispatchEvent(new MouseEvent('click'));
        } else {
          $(snapshotLink)[0].click();
        }
        arc.report.alertMessageSmallHide();
      }

      snapshot.downloadSnapshotIfReady(snapshotId, appId, downloadSnapshot, 0);
    })
    .fail(function(req) {
      arc.report.alertMessageSmallHide();
      arc.report.alertMessage('Failed to download image',
          req.responseText ? req.responseText : '', { alert_report_id: appId });
    });
  };

  snapshot.downloadSnapshotIfReady = (snapshotId, app_id, cb, callCount) => {
    const url = `${arc.frontend.globals.urls.snapshotReadyUrl}/${snapshotId}`;
    $.ajax({
      url,
      type: 'GET',
    })
    .done(function(resp) {
      callCount += 1;
      if (resp === 'ready') {
        cb();
        return;
      }

      // 3 minutes total timeout
      if (callCount > 180) {
        arc.report.alertMessageSmallBase({
          type: 'alert-warning',
          alertHeader: 'Generating the requested file failed',
          alertMessage: 'Could not generate the file within 3 minutes',
          fadeOut: false,
        });

        return;
      }

      // check every second
      setTimeout(snapshot.downloadSnapshotIfReady.bind(
        this,
        snapshotId,
        app_id,
        cb,
        callCount
      ), 1000);
    })
    .fail((req) => {
      arc.report.alertMessageSmallHide();

      arc.report.alertMessageSmallBase({
        type: 'alert-warning',
        alertHeader: 'Generating the requested file failed',
        alertMessage: req.responseText ? req.responseText : '',
        fadeOut: false,
      });
    });
  };

  return snapshot;
};
}());
