QUnit.module('arc_showme_api');

const CONSTANTS = {
  expectedGenericReports: [
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'scatter',
      report_data: {
        core: {
          viz_type: 'scatter',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'size_shelf', shelf_type: 1, column_type: 2 },
            { name: 'transition_shelf', shelf_type: 1, column_type: 1 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 3 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        size_shelf: [],
        transition_shelf: [],
        tooltip_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          label_by_color: 'true',
          mark_size_range: '5-50',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          transition_delay: '1000',
          autoplay: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type_x: 'LINEAR',
          scale_type_y: 'LINEAR',
          make_x_continuous: 'true',
          make_y_continuous: 'true',
          min_x_axis: '',
          max_x_axis: '',
          min_y_axis: '',
          max_y_axis: '',
          axes_padding: '5',
          x_axis_padding: '5',
          y_axis_padding: '5',
          gridlines_enabled: 'true',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          x_axis_label: 'true',
          y_axis_label: 'true',
          reverse_scale_x: 'false',
          reverse_scale_y: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          measure_ranges: '{"ranges":[]}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'kpi',
      report_data: {
        core: {
          viz_type: 'kpi',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'compare_shelf', shelf_type: 1, column_type: 2 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        compare_shelf: [],
        label_shelf: [],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          main_measure_font_size: '36',
          main_measure_font_size_autosize: 'true',
          main_value_display: 'value',
          main_as_arrow_color: 'false',
          display_first_measure: 'true',
          compare_display_format: 'value',
          kpi_arrow_location: 'right',
          compare_arrow_type: 'arrow-circle',
          switch_compare_direction: 'false',
          main_label_location: 'below',
          kpi_label_font_size: '14',
          kpi_up_color: '#5cb85c',
          kpi_flat_color: '#e7b416',
          kpi_down_color: '#d9534f',
          kpi_font_color: '#000',
          kpi_label_color: '#aaa',
          kpi_decimals: '1',
          hide_x_trellis: 'false',
          hide_y_trellis: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
      },
    },
    {
      report_type: 'table',
      report_data: {
        core: {
          viz_type: 'table',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          display_tooltips_table: 'false',
          table_template: 'table-template-7',
          table_header_alignment: 'data_dependent',
          table_full_width: 'false',
          table_wrap_column_contents: 'false',
          parameters_send_all_cols: 'false',
          show_contextmenu: 'true',
          cells_are_html: 'false',
          enable_highlighting: 'false',
          nulls_display_value: '',
          http_auto_link: 'true',
          table_page_totals: 'false',
          table_page_totals_header: 'false',
          table_totals: 'false',
          table_totals_header: 'false',
          totals_column_font_weight: 'default',
          max_columns: '20',
          enable_info: 'false',
          enable_pagination: 'true',
          page_size: 20,
          table_header_background_color: '',
          table_header_text_color: '',
          table_odd_rows_background_color: '',
          table_even_rows_background_color: '',
          table_odd_rows_text_color: '',
          table_even_rows_text_color: '',
          table_border_color: '',
          table_footer_background_color: '',
          table_footer_text_color: '',
          display_header: 'true',
          display_header_column_borders: 'true',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        limit: 5000,
        customWidths: [],
      },
    },
    {
      report_type: 'gauge',
      report_data: {
        core: {
          viz_type: 'gauge',
          shelves: [
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'compare_shelf', shelf_type: 1, column_type: 2 },
            { name: 'qualitative_shelf', shelf_type: 1, column_type: 2 },
            { name: 'projection_shelf', shelf_type: 1, column_type: 2 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        compare_shelf: [],
        qualitative_shelf: [],
        projection_shelf: [],
        label_shelf: [],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_ticks_font_size: '',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          gauge_fg_color: '#0097a7',
          gauge_pointer_color: '#707070',
          gauge_bg_color: '#cecece',
          projection_opacity: '40',
          gauge_background_opacity: '20',
          gauge_key_measure_mark: 'NEEDLE',
          always_show_compare_line: 'false',
          include_outside_compare_line: 'false',
          show_gauge_axis_ticks: 'false',
          gauge_outer_width: '200-250',
          gauge_qualitative_arc_thickness: '5',
          gauge_qualitative_arc_gap: '5',
          gauge_arc_thickness: '20',
          gauge_label_font_size: '30',
          label_vertical_offset: '0',
          gauge_sublabel_font_size: '18',
          sublabel_vertical_offset: '0',
          qualitative_as_percentages: 'false',
          lowest_qualitative_percentage: '60',
          medium_qualitative_percentage: '80',
          highest_qualitative_percentage: '100',
          scale_trellis_independently: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
      },
    },
    {
      report_type: 'bullet',
      report_data: {
        core: {
          viz_type: 'bullet',
          shelves: [
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'compare_shelf', shelf_type: 1, column_type: 2 },
            { name: 'qualitative_shelf', shelf_type: 1, column_type: 2 },
            { name: 'projection_shelf', shelf_type: 1, column_type: 2 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        compare_shelf: [],
        qualitative_shelf: [],
        projection_shelf: [],
        label_shelf: [],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_ticks_font_size: '',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bullet_bg_color: 'black',
          bullet_fg_color: 'black',
          projection_opacity: '40',
          reverse_bullet_colors: 'false',
          scale_trellis_independently: 'false',
          bullet_vertical: 'false',
          remove_default_label: 'false',
          approximate_tick_count: -1,
          qualitative_as_percentages: 'false',
          lowest_qualitative_percentage: '60',
          medium_qualitative_percentage: '80',
          highest_qualitative_percentage: '100',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'PERCENTAGE',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-lines',
      report_data: {
        core: {
          viz_type: 'trellis-lines',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'table',
      report_data: {
        core: {
          viz_type: 'table',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          display_tooltips_table: 'false',
          table_template: 'table-template-7',
          table_header_alignment: 'data_dependent',
          table_full_width: 'false',
          table_wrap_column_contents: 'false',
          parameters_send_all_cols: 'false',
          show_contextmenu: 'true',
          cells_are_html: 'false',
          enable_highlighting: 'false',
          nulls_display_value: '',
          http_auto_link: 'true',
          table_page_totals: 'false',
          table_page_totals_header: 'false',
          table_totals: 'false',
          table_totals_header: 'false',
          totals_column_font_weight: 'default',
          max_columns: '20',
          enable_info: 'false',
          enable_pagination: 'true',
          page_size: 20,
          table_header_background_color: '',
          table_header_text_color: '',
          table_odd_rows_background_color: '',
          table_even_rows_background_color: '',
          table_odd_rows_text_color: '',
          table_even_rows_text_color: '',
          table_border_color: '',
          table_footer_background_color: '',
          table_footer_text_color: '',
          display_header: 'true',
          display_header_column_borders: 'true',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        limit: 5000,
        customWidths: [],
      },
    },
    {
      report_type: 'table',
      report_data: {
        core: {
          viz_type: 'table',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          display_tooltips_table: 'false',
          table_template: 'table-template-7',
          table_header_alignment: 'data_dependent',
          table_full_width: 'false',
          table_wrap_column_contents: 'false',
          parameters_send_all_cols: 'false',
          show_contextmenu: 'true',
          cells_are_html: 'false',
          enable_highlighting: 'false',
          nulls_display_value: '',
          http_auto_link: 'true',
          table_page_totals: 'false',
          table_page_totals_header: 'false',
          table_totals: 'false',
          table_totals_header: 'false',
          totals_column_font_weight: 'default',
          max_columns: '20',
          enable_info: 'false',
          enable_pagination: 'true',
          page_size: 20,
          table_header_background_color: '',
          table_header_text_color: '',
          table_odd_rows_background_color: '',
          table_even_rows_background_color: '',
          table_odd_rows_text_color: '',
          table_even_rows_text_color: '',
          table_border_color: '',
          table_footer_background_color: '',
          table_footer_text_color: '',
          display_header: 'true',
          display_header_column_borders: 'true',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        limit: 5000,
        customWidths: [],
      },
    },
    {
      report_type: 'scatter',
      report_data: {
        core: {
          viz_type: 'scatter',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'size_shelf', shelf_type: 1, column_type: 2 },
            { name: 'transition_shelf', shelf_type: 1, column_type: 1 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 3 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        size_shelf: [],
        transition_shelf: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          label_by_color: 'true',
          mark_size_range: '5-50',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          transition_delay: '1000',
          autoplay: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type_x: 'LINEAR',
          scale_type_y: 'LINEAR',
          make_x_continuous: 'true',
          make_y_continuous: 'true',
          min_x_axis: '',
          max_x_axis: '',
          min_y_axis: '',
          max_y_axis: '',
          axes_padding: '5',
          x_axis_padding: '5',
          y_axis_padding: '5',
          gridlines_enabled: 'true',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          x_axis_label: 'true',
          y_axis_label: 'true',
          reverse_scale_x: 'false',
          reverse_scale_y: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          measure_ranges: '{"ranges":[]}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'scatter',
      report_data: {
        core: {
          viz_type: 'scatter',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'size_shelf', shelf_type: 1, column_type: 2 },
            { name: 'transition_shelf', shelf_type: 1, column_type: 1 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 3 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        size_shelf: [],
        transition_shelf: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          label_by_color: 'true',
          mark_size_range: '5-50',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          transition_delay: '1000',
          autoplay: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type_x: 'LINEAR',
          scale_type_y: 'LINEAR',
          make_x_continuous: 'true',
          make_y_continuous: 'true',
          min_x_axis: '',
          max_x_axis: '',
          min_y_axis: '',
          max_y_axis: '',
          axes_padding: '5',
          x_axis_padding: '5',
          y_axis_padding: '5',
          gridlines_enabled: 'true',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          x_axis_label: 'true',
          y_axis_label: 'true',
          reverse_scale_x: 'false',
          reverse_scale_y: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          measure_ranges: '{"ranges":[]}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
      },
    },
    {
      report_type: 'trellis-bars',
      report_data: {
        core: {
          viz_type: 'trellis-bars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'PERCENTAGE',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-groupedbars',
      report_data: {
        core: {
          viz_type: 'trellis-groupedbars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-groupedbars',
      report_data: {
        core: {
          viz_type: 'trellis-groupedbars',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'packed-bubbles',
      report_data: {
        core: {
          viz_type: 'packed-bubbles',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          legend_style: 'Right',
          reverse_legend_order: 'false',
          legend_width_override: '',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
      },
    },
    {
      report_type: 'trellis-lines',
      report_data: {
        core: {
          viz_type: 'trellis-lines',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-lines',
      report_data: {
        core: {
          viz_type: 'trellis-lines',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-lines',
      report_data: {
        core: {
          viz_type: 'trellis-lines',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'scatter',
      report_data: {
        core: {
          viz_type: 'scatter',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'size_shelf', shelf_type: 1, column_type: 2 },
            { name: 'transition_shelf', shelf_type: 1, column_type: 1 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 3 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        size_shelf: [],
        transition_shelf: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          label_by_color: 'true',
          mark_size_range: '5-50',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          transition_delay: '1000',
          autoplay: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type_x: 'LINEAR',
          scale_type_y: 'LINEAR',
          make_x_continuous: 'true',
          make_y_continuous: 'true',
          min_x_axis: '',
          max_x_axis: '',
          min_y_axis: '',
          max_y_axis: '',
          axes_padding: '5',
          x_axis_padding: '5',
          y_axis_padding: '5',
          gridlines_enabled: 'true',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          x_axis_label: 'true',
          y_axis_label: 'true',
          reverse_scale_x: 'false',
          reverse_scale_y: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          measure_ranges: '{"ranges":[]}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'trellis-lines',
      report_data: {
        core: {
          viz_type: 'trellis-lines',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'combo',
      report_data: {
        core: {
          viz_type: 'combo',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'bar_measure', shelf_type: 1, column_type: 2 },
            { name: 'line_measure', shelf_type: 1, column_type: 2 },
            { name: 'highlighter_measure', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        bar_measure: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        line_measure: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        highlighter_measure: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          highlight_mark_type: 'CIRCLE',
          use_area: 'false',
          highlight_mark_radius: '20',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'combo',
      report_data: {
        core: {
          viz_type: 'combo',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'bar_measure', shelf_type: 1, column_type: 2 },
            { name: 'line_measure', shelf_type: 1, column_type: 2 },
            { name: 'highlighter_measure', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        bar_measure: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        line_measure: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        highlighter_measure: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          highlight_mark_type: 'CIRCLE',
          use_area: 'false',
          highlight_mark_radius: '20',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'combo',
      report_data: {
        core: {
          viz_type: 'combo',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'bar_measure', shelf_type: 1, column_type: 2 },
            { name: 'line_measure', shelf_type: 1, column_type: 2 },
            { name: 'highlighter_measure', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        bar_measure: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        line_measure: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        highlighter_measure: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          secondary_min_agg_axis: '',
          secondary_max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          highlight_mark_type: 'CIRCLE',
          use_area: 'false',
          highlight_mark_radius: '20',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
          line_style_from_color: 'false',
          enable_multiple_tooltips: 'false',
          enable_line_labels: 'false',
          maximum_label_width: 200,
          dynamic_bar_width: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'radial',
      report_data: {
        core: {
          viz_type: 'radial',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          overlapping_marks: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          scale_type_radial: 'ONEMEASURE',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
      },
    },
    {
      report_type: 'trellis-areas',
      report_data: {
        core: {
          viz_type: 'trellis-areas',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'color_shelf', shelf_type: 1, column_type: 3 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'drill_shelf', shelf_type: 1, column_type: 1 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        color_shelf: [],
        tooltip_shelf: [],
        drill_shelf: [],
        label_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_label_font_size: '',
          axis_ticks_font_size: '',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          bar_mark_size: '20-30',
          enable_animations: 'true',
          visual_autodetect_time: 'true',
          legend_style: 'Right',
          legend_width_override: '',
          reverse_legend_order: 'false',
          trendline_linear: 'false',
          trendline_color: 'false',
          trendline_decimal: '3',
          scale_type: 'LINEAR',
          gridlines_enabled: 'false',
          vertical_gridlines_enabled: 'false',
          scale_dim_independent: 'false',
          scale_agg_independent: 'false',
          scale_agg_zero: 'true',
          min_agg_axis: '',
          max_agg_axis: '',
          dim_is_continuous: 'false',
          min_dim_axis: '',
          max_dim_axis: '',
          x_axis_label: 'true',
          y_axis_label: 'true',
          dim_tick_label_maxlen: '20',
          show_selection_range: 'false',
          show_x_axis: 'true',
          show_y_axis: 'true',
          enable_drill_into: 'true',
          inline_measure_display: 'false',
          inline_sum_measure_display: 'false',
          min_cell_height: '',
          min_cell_width: '',
          ignore_stacked_negatives_with_mixed_measures: 'false',
          use_dataset_color_settings: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          show_points: 'false',
          smooth_line: 'true',
          line_interpolation: 'linear',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'pie',
      report_data: {
        core: {
          viz_type: 'pie',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          label_font_size: '',
          label_color: '',
          trellis_font_size: '',
          legend_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          legend_style: 'None',
          legend_width_override: '',
          reverse_legend_order: 'false',
          tooltip_percentage: 'true',
          tooltip_percentage_name: 'percentage',
          tooltip_percentage_decimal: '2',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          pieOrDonut: 'false',
          defaultWidth: 'true',
          arc_width: 0,
          showDimensionsOnMarks: 'true',
          showPercentagesOnMarks: 'false',
          markPercentageDecimal: '1',
          mark_border_color: '#FFF',
          mark_border_width: '1',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
      },
    },
    {
      report_type: 'chord',
      report_data: {
        core: {
          viz_type: 'chord',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          chord_minimum_size: '200',
          same_domain: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          measure_ranges: '{"ranges":[]}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        fixed_nodes: [],
      },
    },
    {
      report_type: 'treemap',
      report_data: {
        core: {
          viz_type: 'treemap',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        fixed_nodes: [],
      },
    },
    {
      report_type: 'network',
      report_data: {
        core: {
          viz_type: 'network',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'link_color_shelf', shelf_type: 1, column_type: 2 },
            { name: 'source_node_size_shelf', shelf_type: 1, column_type: 2 },
            { name: 'target_node_size_shelf', shelf_type: 1, column_type: 2 },
            { name: 'source_node_color_shelf', shelf_type: 1, column_type: 2 },
            { name: 'target_node_color_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        link_color_shelf: [],
        source_node_size_shelf: [],
        target_node_size_shelf: [],
        source_node_color_shelf: [],
        target_node_color_shelf: [],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          remember_node_positions: 'true',
          empty_color: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          link_color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
          same_domain: 'false',
          link_color: 'aggregate',
          link_opacity: '80',
          src_node_mark_type: 'circle',
          tgt_node_mark_type: 'circle',
          combined_node_mark_type: 'circle',
          link_width_range: '4-10',
          node_size_range: '10-25',
          link_distance: '80',
          link_strength: '1',
          node_friction: '0.3',
          node_charge: '-1000',
          src_node_opacity: '80',
          show_node_labels: 'false',
          layout_gravity: '0.1',
          layout_theta: '0.8',
          add_direction_arrow: 'false',
          direction_arrow_size: '2',
          add_node_legend: 'false',
          add_link_legend: 'false',
          network_legend_width: '150',
          node_legend_title: 'Node colors',
          link_legend_title: 'Link colors',
          link_pointer_color: 'black',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        fixed_nodes: [],
      },
    },
    {
      report_type: 'corelation',
      report_data: {
        core: {
          viz_type: 'corelation',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          axis_ticks_font_size: '',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          enable_animations: 'true',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          measure_ranges: '{"ranges":[]}',
          correlation_cell_minimum_size: '12',
          correlation_max_x_height: '400',
          correlation_max_y_width: '400',
          show_x_axis: 'true',
          show_y_axis: 'true',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
      },
    },
    {
      report_type: 'dendrogram',
      report_data: {
        core: {
          viz_type: 'dendrogram',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          link_opacity: '80',
          link_width_range: '4-10',
          link_color: 'aggregate',
          show_node_labels: 'true',
          same_domain: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          dim_to_color_mapping: '{"mappings":{},"isChecked":false}',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        fixed_nodes: [],
      },
    },
    {
      report_type: 'corelation-flow',
      report_data: {
        core: {
          viz_type: 'corelation-flow',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
          {
            col_alias: 'sodium_mg',
            custom_expr: '[sodium_mg]',
            expression_for_trigger: '[sodium_mg]',
            dataset_colname: 'sodium_mg',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          fixed_color_legend: 'false',
          flow_min_height_per_node: 50,
          flow_min_width_per_node: 150,
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
      },
    },
    {
      report_type: 'crosstab',
      report_data: {
        core: {
          viz_type: 'crosstab',
          shelves: [
            { name: 'x_dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_dimensions_shelf: [],
        y_dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          display_tooltips_table: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          colors_enabled: 'false',
          show_adjacent_duplicate_values: 'false',
          totals_above_expansions: 'true',
          crosstab_total_str: '',
          aggregate_row_style: 'default',
          aggregate_row_weight: 'default',
          full_table_width: 'false',
          auto_size_column: 'false',
          table_header_background_color: '',
          table_header_text_color: '',
          table_odd_rows_background_color: '',
          table_even_rows_background_color: '',
          table_odd_rows_text_color: '',
          table_even_rows_text_color: '',
          table_footer_background_color: '',
          table_footer_text_color: '',
          table_border_color: '',
          crosstab_title_font: 'default',
          crosstab_title_font_size: '',
          crosstab_title_font_weight: 'default',
          table_cell_padding: '5',
          table_border_width: '1',
          show_crosstab_totals: 'false',
          show_row_totals: 'false',
          combine_row_measure_totals: 'false',
          totals_column_font_weight: 'default',
          totals_row_font_weight: 'default',
          crosstab_header_alignment: 'left',
          crosstab_content_alignment: 'default',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        crosstabCustomWidths: [],
        dimensions_shelf: [],
      },
    },
    {
      report_type: 'table',
      report_data: {
        core: {
          viz_type: 'table',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [],
        aggregates_shelf: [],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          display_tooltips_table: 'false',
          table_template: 'table-template-6',
          table_header_alignment: 'data_dependent',
          table_full_width: 'true',
          table_wrap_column_contents: 'false',
          parameters_send_all_cols: 'false',
          show_contextmenu: 'true',
          cells_are_html: 'false',
          enable_highlighting: 'false',
          nulls_display_value: '',
          http_auto_link: 'true',
          table_page_totals: 'false',
          table_page_totals_header: 'false',
          table_totals: 'false',
          table_totals_header: 'false',
          totals_column_font_weight: 'default',
          max_columns: '20',
          enable_info: 'false',
          enable_pagination: 'true',
          page_size: 20,
          table_header_background_color: '',
          table_header_text_color: '',
          table_odd_rows_background_color: '',
          table_even_rows_background_color: '',
          table_odd_rows_text_color: '',
          table_even_rows_text_color: '',
          table_border_color: '',
          table_footer_background_color: '',
          table_footer_text_color: '',
          display_header: 'true',
          display_header_column_borders: 'true',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        limit: 5000,
        customWidths: [],
      },
    },
    {
      report_type: 'sparklines',
      report_data: {
        core: {
          viz_type: 'sparklines',
          shelves: [
            { name: 'x_shelf', shelf_type: 1, column_type: 3 },
            { name: 'y_shelf', shelf_type: 1, column_type: 3 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        y_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          tooltip_min: 'true',
          tooltip_max: 'true',
          tooltip_first: 'true',
          tooltip_last: 'true',
          scale_agg_independent: 'true',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        allowLargeTrellisPerm: false,
        dimensions_shelf: [],
        aggregates_shelf: [],
      },
    },
    {
      report_type: 'crosstab',
      report_data: {
        core: {
          viz_type: 'crosstab',
          shelves: [
            { name: 'x_dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        x_dimensions_shelf: [],
        y_dimensions_shelf: [
          {
            col_alias: 'manufacturer_code',
            custom_expr: '[manufacturer_code]',
            expression_for_trigger: '[manufacturer_code]',
            dataset_colname: 'manufacturer_code',
            dataset_coltype: 'STRING',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        tooltip_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          display_tooltips_table: 'false',
          color_palette:
            '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
          color_opacity: '80',
          colors_enabled: 'false',
          show_adjacent_duplicate_values: 'false',
          totals_above_expansions: 'true',
          crosstab_total_str: '',
          aggregate_row_style: 'default',
          aggregate_row_weight: 'default',
          full_table_width: 'false',
          auto_size_column: 'false',
          table_header_background_color: '',
          table_header_text_color: '',
          table_odd_rows_background_color: '',
          table_even_rows_background_color: '',
          table_odd_rows_text_color: '',
          table_even_rows_text_color: '',
          table_footer_background_color: '',
          table_footer_text_color: '',
          table_border_color: '',
          crosstab_title_font: 'default',
          crosstab_title_font_size: '',
          crosstab_title_font_weight: 'default',
          table_cell_padding: '5',
          table_border_width: '1',
          show_crosstab_totals: 'false',
          show_row_totals: 'false',
          combine_row_measure_totals: 'false',
          totals_column_font_weight: 'default',
          totals_row_font_weight: 'default',
          crosstab_header_alignment: 'left',
          crosstab_content_alignment: 'default',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
        crosstabCustomWidths: [],
        dimensions_shelf: [],
      },
    },
    {
      report_type: 'kpi',
      report_data: {
        core: {
          viz_type: 'kpi',
          shelves: [
            { name: 'dimensions_shelf', shelf_type: 1, column_type: 1 },
            { name: 'aggregates_shelf', shelf_type: 1, column_type: 2 },
            { name: 'compare_shelf', shelf_type: 1, column_type: 2 },
            { name: 'label_shelf', shelf_type: 1, column_type: 2 },
            { name: 'tooltip_shelf', shelf_type: 1, column_type: 2 },
            { name: 'x_shelf', shelf_type: 1, column_type: 1 },
            { name: 'y_shelf', shelf_type: 1, column_type: 1 },
            { name: 'filters_shelf', shelf_type: 2, column_type: 3 },
          ],
        },
        dimensions_shelf: [],
        aggregates_shelf: [
          {
            col_alias: 'sum(calories)',
            custom_expr: 'sum([calories])',
            expression_for_trigger: 'sum([calories])',
            dataset_colname: 'calories',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        compare_shelf: [
          {
            col_alias: 'sum(fat_grams)',
            custom_expr: 'sum([fat_grams])',
            expression_for_trigger: 'sum([fat_grams])',
            dataset_colname: 'fat_grams',
            dataset_coltype: 'INT',
            isDatasetSegment: false,
            autotime_override: 'AUTO',
          },
        ],
        label_shelf: [],
        tooltip_shelf: [],
        x_shelf: [],
        y_shelf: [],
        filters_shelf: [],
        sample_pct: 'Off',
        user_settings: {
          container_css_class: '',
          css_styles: '[]',
          inline_css: '',
          permit_csv_download_visual: 'false',
          csv_download_quotation: 'false',
          default_language: '',
          permit_png_download: 'false',
          explicit_scope: 'false',
          permit_email_report: 'true',
          send_aggregate_parameters: 'false',
          visual_dataset_scope: 'GLOBAL',
          nulls_order: 'Last',
          trellis_borders: 'true',
          trellis_cell_minimum_width: '100',
          trellis_cell_minimum_height: '100',
          visual_font: 'default',
          visual_font_size: '',
          visual_bg_color: '',
          title_font: 'default',
          title_font_size: '',
          visual_title_font_weight: 'default',
          visual_title_fg: '',
          visual_title_bg: '',
          title_alignment: 'Left',
          subtitle_font: 'default',
          subtitle_font_size: '',
          visual_subtitle_fg: '',
          visual_subtitle_bg: '',
          subtitle_alignment: 'Left',
          trellis_font_size: '',
          visual_hide_title: 'false',
          visual_show_detail_data: 'true',
          download_rows: '',
          max_rowcount: '',
          show_tooltip: 'true',
          tooltip_font: 'default',
          tooltip_font_size: '',
          disable_incremental_queries: 'false',
          ksql_stream_position: 'default',
          main_measure_font_size: '36',
          main_measure_font_size_autosize: 'true',
          main_value_display: 'value',
          main_as_arrow_color: 'false',
          display_first_measure: 'true',
          compare_display_format: 'value',
          kpi_arrow_location: 'right',
          compare_arrow_type: 'arrow-circle',
          switch_compare_direction: 'false',
          main_label_location: 'below',
          kpi_label_font_size: '14',
          kpi_up_color: '#5cb85c',
          kpi_flat_color: '#e7b416',
          kpi_down_color: '#d9534f',
          kpi_font_color: '#000',
          kpi_label_color: '#aaa',
          kpi_decimals: '1',
          hide_x_trellis: 'false',
          hide_y_trellis: 'false',
        },
        settings: {
          colorSettings: {},
          colors: [
            '#0097a7',
            '#7dcad2',
            '#546e7a',
            '#b0bec5',
            '#ed8111',
            '#f6c088',
            '#e54954',
            '#f2a4a9',
            '#ad73b1',
            '#d6b9d8',
            '#2375b8',
            '#91badb',
            '#3e8111',
            '#b0cb73',
          ],
          graded_colors: [
            '#E0F7FA',
            '#B2EBF2',
            '#80DEEA',
            '#4DD0E1',
            '#26C6DA',
            '#00BCD4',
            '#00ACC1',
            '#0097A7',
            '#00838F',
            '#006064',
          ],
          contrast_colors: [
            '#0097A7',
            '#ED8111',
            '#2375B8',
            '#E54954',
            '#42996F',
            '#AD73B1',
            '#B0CB73',
            '#EA8DCD',
          ],
          reverse_colors: false,
        },
        report_title: '',
        report_subtitle: '',
        report_derived_data: [],
        click_behaviors: {},
        sort_orders_asc: {},
      },
    },
  ],
  expectedLeaflet: {
    report_type: 'leaflet',
    report_data: {
      core: {
        viz_type: 'leaflet',
        shelves: [
          {
            name: 'geo_shelf',
            shelf_type: 1,
            column_type: 3,
          },
          {
            name: 'aggregates_shelf',
            shelf_type: 1,
            column_type: 2,
          },
          {
            name: 'color_shelf',
            shelf_type: 1,
            column_type: 3,
          },
          {
            name: 'tooltip_shelf',
            shelf_type: 1,
            column_type: 3,
          },
          {
            name: 'filters_shelf',
            shelf_type: 2,
            column_type: 3,
          },
        ],
      },
      geo_shelf: [
        {
          col_alias: 'latitude',
          custom_expr: '[latitude]',
          expression_for_trigger: '[latitude]',
          dataset_colname: 'latitude',
          dataset_coltype: 'FLOAT',
          isDatasetSegment: false,
          geo_cast: 'latitude',
          autotime_override: 'AUTO',
        },
        {
          col_alias: 'longitude',
          custom_expr: '[longitude]',
          expression_for_trigger: '[longitude]',
          dataset_colname: 'longitude',
          dataset_coltype: 'FLOAT',
          isDatasetSegment: false,
          geo_cast: 'longitude',
          autotime_override: 'AUTO',
        },
      ],
      aggregates_shelf: [],
      color_shelf: [],
      tooltip_shelf: [
        {
          col_alias: 'magtype',
          custom_expr: '[magtype]',
          expression_for_trigger: '[magtype]',
          dataset_colname: 'magtype',
          dataset_coltype: 'STRING',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
        {
          col_alias: 'status',
          custom_expr: '[status]',
          expression_for_trigger: '[status]',
          dataset_colname: 'status',
          dataset_coltype: 'STRING',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
      ],
      filters_shelf: [],
      sample_pct: 'Off',
      user_settings: {
        container_css_class: '',
        css_styles: '[]',
        inline_css: '',
        permit_csv_download_visual: 'false',
        csv_download_quotation: 'false',
        default_language: '',
        permit_png_download: 'false',
        explicit_scope: 'false',
        permit_email_report: 'true',
        send_aggregate_parameters: 'false',
        visual_dataset_scope: 'GLOBAL',
        nulls_order: 'Last',
        visual_font: 'default',
        visual_font_size: '',
        visual_bg_color: '',
        title_font: 'default',
        title_font_size: '',
        visual_title_font_weight: 'default',
        visual_title_fg: '',
        visual_title_bg: '',
        title_alignment: 'Left',
        subtitle_font: 'default',
        subtitle_font_size: '',
        visual_subtitle_fg: '',
        visual_subtitle_bg: '',
        subtitle_alignment: 'Left',
        legend_font_size: '',
        visual_hide_title: 'false',
        visual_show_detail_data: 'true',
        download_rows: '',
        max_rowcount: '',
        show_tooltip: 'true',
        tooltip_font: 'default',
        tooltip_font_size: '',
        disable_incremental_queries: 'false',
        ksql_stream_position: 'default',
        color_palette:
          '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
        measure_ranges: '{"ranges":[]}',
        show_map_data_div: 'false',
        'tile-style': 'mapboxgl',
        local_map_key: '',
        visual_atlas_server: '',
        map_style: 'default',
        mapboxStyle: '',
        choropleth_layer_id: '',
        choropleth_opacity: 0.6,
        choropleth_legend: 'true',
        store_map_load_location: 'true',
        start_center: '-98.5794797, 39.8283459',
        start_zom: '3',
        heatmap_enabled: 'true',
        intensity_mag: '1000',
        heatmap_radius: '20',
        heatmap_visible_zoom_levels: '0-22',
        heatmap_legend: 'false',
        cluster_enabled: 'true',
        cluster_color: '#ff9900',
        cluster_radius: '80',
        circle_enabled: 'false',
        circle_legend_area: 'false',
        circle_legend_color: 'false',
        circle_radius_range: '5-30',
        circle_visible_zoom: '0-22',
        marker_enabled: 'false',
        marker_permanent_labels: 'false',
        marker_type: 'default',
        route_enabled: 'false',
        start_marker_type: 'circle',
        middle_marker_type: 'none',
        end_marker_type: 'default',
        custom_icon: '',
        custom_icon_size_adjustment: '50',
        hold_location_manual_move: 'false',
      },
      settings: {
        colorSettings: {},
        colors: [
          '#0097a7',
          '#7dcad2',
          '#546e7a',
          '#b0bec5',
          '#ed8111',
          '#f6c088',
          '#e54954',
          '#f2a4a9',
          '#ad73b1',
          '#d6b9d8',
          '#2375b8',
          '#91badb',
          '#3e8111',
          '#b0cb73',
        ],
        graded_colors: [
          '#E0F7FA',
          '#B2EBF2',
          '#80DEEA',
          '#4DD0E1',
          '#26C6DA',
          '#00BCD4',
          '#00ACC1',
          '#0097A7',
          '#00838F',
          '#006064',
        ],
        contrast_colors: [
          '#0097A7',
          '#ED8111',
          '#2375B8',
          '#E54954',
          '#42996F',
          '#AD73B1',
          '#B0CB73',
          '#EA8DCD',
        ],
        reverse_colors: false,
      },
      report_title: '',
      report_subtitle: '',
      report_derived_data: [],
      click_behaviors: {},
      dimensions_shelf: [],
    },
  },
  expectedMap: {
    report_type: 'map',
    report_data: {
      core: {
        viz_type: 'map',
        shelves: [
          {
            name: 'geo_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'aggregates_shelf',
            shelf_type: 1,
            column_type: 2,
          },
          {
            name: 'size_shelf',
            shelf_type: 1,
            column_type: 2,
          },
          {
            name: 'drill_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'tooltip_shelf',
            shelf_type: 1,
            column_type: 2,
          },
          {
            name: 'x_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'y_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'filters_shelf',
            shelf_type: 2,
            column_type: 3,
          },
        ],
      },
      geo_shelf: [
        {
          col_alias: 'country',
          custom_expr: '[country]',
          expression_for_trigger: '[country]',
          dataset_colname: 'country',
          dataset_coltype: 'STRING',
          isDatasetSegment: false,
          geo_cast: 'country',
          autotime_override: 'AUTO',
        },
      ],
      aggregates_shelf: [
        {
          col_alias: 'count',
          custom_expr: '[count]',
          expression_for_trigger: '[count]',
          dataset_colname: 'count',
          dataset_coltype: 'BIGINT',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
      ],
      size_shelf: [],
      drill_shelf: [],
      tooltip_shelf: [
        {
          col_alias: 'year',
          custom_expr: '[year]',
          expression_for_trigger: '[year]',
          dataset_colname: 'year',
          dataset_coltype: 'BIGINT',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
      ],
      x_shelf: [],
      y_shelf: [],
      filters_shelf: [],
      sample_pct: 'Off',
      user_settings: {
        container_css_class: '',
        css_styles: '[]',
        inline_css: '',
        permit_csv_download_visual: 'false',
        csv_download_quotation: 'false',
        default_language: '',
        permit_png_download: 'false',
        explicit_scope: 'false',
        permit_email_report: 'true',
        send_aggregate_parameters: 'false',
        visual_dataset_scope: 'GLOBAL',
        nulls_order: 'Last',
        trellis_borders: 'true',
        trellis_cell_minimum_width: '100',
        trellis_cell_minimum_height: '100',
        visual_font: 'default',
        visual_font_size: '',
        visual_bg_color: '',
        title_font: 'default',
        title_font_size: '',
        visual_title_font_weight: 'default',
        visual_title_fg: '',
        visual_title_bg: '',
        title_alignment: 'Left',
        subtitle_font: 'default',
        subtitle_font_size: '',
        visual_subtitle_fg: '',
        visual_subtitle_bg: '',
        subtitle_alignment: 'Left',
        trellis_font_size: '',
        legend_font_size: '',
        visual_hide_title: 'false',
        visual_show_detail_data: 'true',
        download_rows: '',
        max_rowcount: '',
        show_tooltip: 'true',
        tooltip_font: 'default',
        tooltip_font_size: '',
        disable_incremental_queries: 'false',
        ksql_stream_position: 'default',
        map_view: 'Default',
        scale_type_independent: 'false',
        legend_style: 'Right',
        legend_width_override: '',
        reverse_legend_order: 'false',
        map_bubbles: 'false',
        mark_size_range: '10-50',
        zoom_enabled: 'false',
        zoom_to_active: 'true',
        color_palette:
          '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
        color_opacity: '80',
        measure_ranges: '{"ranges":[]}',
      },
      settings: {
        colorSettings: {},
        colors: [
          '#0097a7',
          '#7dcad2',
          '#546e7a',
          '#b0bec5',
          '#ed8111',
          '#f6c088',
          '#e54954',
          '#f2a4a9',
          '#ad73b1',
          '#d6b9d8',
          '#2375b8',
          '#91badb',
          '#3e8111',
          '#b0cb73',
        ],
        graded_colors: [
          '#E0F7FA',
          '#B2EBF2',
          '#80DEEA',
          '#4DD0E1',
          '#26C6DA',
          '#00BCD4',
          '#00ACC1',
          '#0097A7',
          '#00838F',
          '#006064',
        ],
        contrast_colors: [
          '#0097A7',
          '#ED8111',
          '#2375B8',
          '#E54954',
          '#42996F',
          '#AD73B1',
          '#B0CB73',
          '#EA8DCD',
        ],
        reverse_colors: false,
      },
      report_title: '',
      report_subtitle: '',
      report_derived_data: [],
      click_behaviors: {},
      sort_orders_asc: {},
      allowLargeTrellisPerm: false,
      dimensions_shelf: [],
    },
  },
  expetedHeatmap: {
    report_type: 'calendar-heatmap',
    report_data: {
      core: {
        viz_type: 'calendar-heatmap',
        shelves: [
          {
            name: 'dimensions_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'aggregates_shelf',
            shelf_type: 1,
            column_type: 2,
          },
          {
            name: 'tooltip_shelf',
            shelf_type: 1,
            column_type: 2,
          },
          {
            name: 'x_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'y_shelf',
            shelf_type: 1,
            column_type: 1,
          },
          {
            name: 'filters_shelf',
            shelf_type: 2,
            column_type: 3,
          },
        ],
      },
      dimensions_shelf: [
        {
          col_alias: 'date',
          custom_expr: '[date]',
          expression_for_trigger: '[date]',
          dataset_colname: 'date',
          dataset_coltype: 'TIMESTAMP',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
      ],
      aggregates_shelf: [
        {
          col_alias: 'count',
          custom_expr: '[count]',
          expression_for_trigger: '[count]',
          dataset_colname: 'count',
          dataset_coltype: 'BIGINT',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
      ],
      tooltip_shelf: [
        {
          col_alias: 'year',
          custom_expr: '[year]',
          expression_for_trigger: '[year]',
          dataset_colname: 'year',
          dataset_coltype: 'BIGINT',
          isDatasetSegment: false,
          autotime_override: 'AUTO',
        },
      ],
      x_shelf: [],
      y_shelf: [],
      filters_shelf: [],
      sample_pct: 'Off',
      user_settings: {
        container_css_class: '',
        css_styles: '[]',
        inline_css: '',
        permit_csv_download_visual: 'false',
        csv_download_quotation: 'false',
        default_language: '',
        permit_png_download: 'false',
        explicit_scope: 'false',
        permit_email_report: 'true',
        send_aggregate_parameters: 'false',
        visual_dataset_scope: 'GLOBAL',
        nulls_order: 'Last',
        trellis_borders: 'true',
        trellis_cell_minimum_width: '100',
        trellis_cell_minimum_height: '100',
        visual_font: 'default',
        visual_font_size: '',
        visual_bg_color: '',
        title_font: 'default',
        title_font_size: '',
        visual_title_font_weight: 'default',
        visual_title_fg: '',
        visual_title_bg: '',
        title_alignment: 'Left',
        subtitle_font: 'default',
        subtitle_font_size: '',
        visual_subtitle_fg: '',
        visual_subtitle_bg: '',
        subtitle_alignment: 'Left',
        axis_ticks_font_size: '',
        trellis_font_size: '',
        visual_hide_title: 'false',
        visual_show_detail_data: 'true',
        download_rows: '',
        max_rowcount: '',
        show_tooltip: 'true',
        tooltip_font: 'default',
        tooltip_font_size: '',
        disable_incremental_queries: 'false',
        ksql_stream_position: 'default',
        enable_animations: 'true',
        scale_type_independent: 'false',
        color_palette:
          '{"colors":["#0097a7","#7dcad2","#546e7a","#b0bec5","#ed8111","#f6c088","#e54954","#f2a4a9","#ad73b1","#d6b9d8","#2375b8","#91badb","#3e8111","#b0cb73"],"colorSettings":{},"isReversed":false,"firstColor":"white"}',
        color_opacity: '80',
        measure_ranges: '{"ranges":[]}',
      },
      settings: {
        colorSettings: {},
        colors: [
          '#0097a7',
          '#7dcad2',
          '#546e7a',
          '#b0bec5',
          '#ed8111',
          '#f6c088',
          '#e54954',
          '#f2a4a9',
          '#ad73b1',
          '#d6b9d8',
          '#2375b8',
          '#91badb',
          '#3e8111',
          '#b0cb73',
        ],
        graded_colors: [
          '#E0F7FA',
          '#B2EBF2',
          '#80DEEA',
          '#4DD0E1',
          '#26C6DA',
          '#00BCD4',
          '#00ACC1',
          '#0097A7',
          '#00838F',
          '#006064',
        ],
        contrast_colors: [
          '#0097A7',
          '#ED8111',
          '#2375B8',
          '#E54954',
          '#42996F',
          '#AD73B1',
          '#B0CB73',
          '#EA8DCD',
        ],
        reverse_colors: false,
      },
      report_title: '',
      report_subtitle: '',
      report_derived_data: [],
      click_behaviors: {},
      sort_orders_asc: {},
      allowLargeTrellisPerm: false,
    },
  },
  datasets: {
    cereals: {
      id: 11,
      dc_id: 1,
      dc_type: 'sqlite',
      dc_name: 'samples',
      dc_type_display: 'SQLite',
      name: 'Cereals',
      type: 'singletable',
      info: JSON.stringify([
        {
          tablename: 'main.cereals',
          columns: [
            {
              alias: 'cereal_name',
              type: 'STRING',
              name: 'cereal_name',
              isdim: true,
            },
            {
              alias: 'manufacturer_code',
              type: 'STRING',
              name: 'manufacturer_code',
              isdim: true,
            },
            {
              alias: 'cold_or_hot',
              type: 'STRING',
              name: 'cold_or_hot',
              isdim: true,
            },
            { alias: 'calories', type: 'INT', name: 'calories', isdim: false },
            {
              alias: 'protein_grams',
              type: 'INT',
              name: 'protein_grams',
              isdim: false,
            },
            {
              alias: 'fat_grams',
              type: 'INT',
              name: 'fat_grams',
              isdim: false,
            },
            {
              alias: 'sodium_mg',
              type: 'INT',
              name: 'sodium_mg',
              isdim: false,
            },
            {
              alias: 'dietary_fiber_grams',
              type: 'INT',
              name: 'dietary_fiber_grams',
              isdim: false,
            },
            {
              alias: 'complex_carbohydrates_grams',
              type: 'INT',
              name: 'complex_carbohydrates_grams',
              isdim: false,
            },
            {
              alias: 'sugars_grams',
              type: 'INT',
              name: 'sugars_grams',
              isdim: false,
            },
            {
              alias: 'display_shelf',
              type: 'INT',
              name: 'display_shelf',
              isdim: false,
            },
            {
              alias: 'potassium_mg',
              type: 'INT',
              name: 'potassium_mg',
              isdim: false,
            },
            {
              alias: 'vitamins_and_minerals',
              type: 'INT',
              name: 'vitamins_and_minerals',
              isdim: false,
            },
            {
              alias: 'weight_ounces',
              type: 'INT',
              name: 'weight_ounces',
              isdim: false,
            },
            {
              alias: 'cups_per_serving',
              type: 'DOUBLE',
              name: 'cups_per_serving',
              isdim: false,
            },
            {
              alias: 'manufacturer_code_15',
              type: 'STRING',
              name: 'manufacturer_code_15',
              isdim: true,
            },
            {
              alias: 'manufacturer',
              type: 'STRING',
              name: 'manufacturer',
              isdim: true,
            },
          ],
        },
      ]),
      detail: 'main.cereals',
      detail_display: 'main.cereals',
      description: '',
      settings: '{}',
      lvname: '',
      lvcolumns: '',
      created: '2024-07-29T08:49:32.053+00:00',
      created_by: 'vizapps_admin',
      updated: '2024-10-28T12:25:18.416+00:00',
      updated_by: 'vizapps_admin',
      perms: [
        'ds_appedit',
        'dc_explore',
        'dc_aviews',
        'dc_upload',
        'ds_appview',
        'ds_manage',
      ],
      updatedLV: false,
      extractSource: true,
      extractTarget: false,
      version_name: '2024-07-29 08:49:32.053253+00:00',
      version_id: 11,
      version_group_id: 11,
      segments: [],
      events: [],
      filterassociations: [],
    },
    earthquakes: {
      id: 10,
      dc_id: 1,
      dc_type: 'sqlite',
      dc_name: 'samples',
      dc_type_display: 'SQLite',
      name: 'Earthquake Data January 2019',
      type: 'singletable',
      info: JSON.stringify([
        {
          tablename: 'main.earthquake_data2019',
          columns: [
            { name: 'time', type: 'TIMESTAMP', isdim: true, alias: 'time' },
            {
              name: 'latitude',
              type: 'DOUBLE',
              isdim: false,
              alias: 'latitude',
              useroverrodeisdim: true,
              cast: 'DOUBLE',
            },
            {
              name: 'longitude',
              type: 'DOUBLE',
              isdim: false,
              alias: 'longitude',
              useroverrodeisdim: true,
              cast: 'DOUBLE',
            },
            {
              name: 'depth',
              type: 'DOUBLE',
              isdim: false,
              alias: 'depth',
              expr: '',
              hidden: false,
              detaildata: true,
              ispartition: false,
              useroverrodeisdim: false,
              dflagg: 'avg',
              geotype: 'none',
              displayformat: { type: '', result: '' },
              comment: '',
            },
            {
              name: 'mag',
              type: 'DOUBLE',
              isdim: false,
              alias: 'mag',
              expr: '',
              hidden: false,
              detaildata: true,
              ispartition: false,
              useroverrodeisdim: false,
              dflagg: 'avg',
              geotype: 'none',
              displayformat: { type: '', result: '' },
              comment: '',
            },
            { name: 'magtype', type: 'STRING', isdim: true, alias: 'magtype' },
            { name: 'nst', type: 'BIGINT', isdim: false, alias: 'nst' },
            { name: 'gap', type: 'DOUBLE', isdim: false, alias: 'gap' },
            { name: 'dmin', type: 'DOUBLE', isdim: false, alias: 'dmin' },
            { name: 'rms', type: 'DOUBLE', isdim: false, alias: 'rms' },
            { name: 'net', type: 'STRING', isdim: true, alias: 'net' },
            { name: 'id', type: 'STRING', isdim: true, alias: 'id' },
            {
              name: 'updated',
              type: 'TIMESTAMP',
              isdim: true,
              alias: 'updated',
            },
            { name: 'place', type: 'STRING', isdim: true, alias: 'place' },
            {
              name: '',
              type: 'STRING',
              isdim: true,
              alias: 'Region',
              searchable: false,
              expr: "substr([place], instr([place], ',') + 1, length([place]))",
              basecol: 'place',
              useroverrodeisdim: false,
              hidden: false,
              detaildata: true,
              ispartition: false,
              dflagg: 'max',
              geotype: 'none',
              displayformat: { type: '', result: '' },
              comment: '',
              exprhasagg: false,
              ispart: false,
              terms: 'Region region area',
            },
            { name: 'type', type: 'STRING', isdim: true, alias: 'type' },
            {
              name: 'horizontalerror',
              type: 'DOUBLE',
              isdim: false,
              alias: 'horizontalerror',
            },
            {
              name: 'deptherror',
              type: 'DOUBLE',
              isdim: false,
              alias: 'deptherror',
            },
            {
              name: 'magerror',
              type: 'DOUBLE',
              isdim: false,
              alias: 'magerror',
            },
            { name: 'magnst', type: 'BIGINT', isdim: false, alias: 'magnst' },
            { name: 'status', type: 'STRING', isdim: true, alias: 'status' },
            {
              name: 'locationsource',
              type: 'STRING',
              isdim: true,
              alias: 'locationsource',
            },
            {
              name: 'magsource',
              type: 'STRING',
              isdim: true,
              alias: 'magsource',
            },
          ],
        },
      ]),
      detail: 'main.earthquake_data2019',
      detail_display: 'main.earthquake_data2019',
      description: '',
      settings:
        '{"SEARCH":{"enabled":true,"expansions":[{"key":"magnitude","value":"mag"}],"suggestions":[],"defaultAgg":"Record Count","recordCount":{"terms":"count rows"}},"TIME":{"defaultDateTime":"time","defaultDateTimeFormat":"YYYY-MM-DD","defaultTimeInterval":"","frequencies":[],"increments":[]}}',
      lvname: '',
      lvcolumns: '',
      created: '2024-07-29T08:49:31.880+00:00',
      created_by: 'vizapps_admin',
      updated: '2024-10-30T08:53:36.509+00:00',
      updated_by: 'vizapps_admin',
      perms: [
        'dc_aviews',
        'dc_upload',
        'ds_appedit',
        'ds_appview',
        'dc_explore',
        'ds_manage',
      ],
      updatedLV: false,
      extractSource: false,
      extractTarget: false,
      version_name: '2024-07-29 08:49:31.880818+00:00',
      version_id: 10,
      version_group_id: 10,
      segments: [],
      events: [],
      filterassociations: [],
    },
    infosec: {
      id: 5,
      dc_id: 1,
      dc_type: 'sqlite',
      dc_name: 'samples',
      dc_type_display: 'SQLite',
      name: 'Global Information Security Threats',
      type: 'singletable',
      info: '[{"tablename":"main.infoseq_1559","columns":[{"name":"id","type":"BIGINT","isdim":false,"alias":"id"},{"name":"date","type":"TIMESTAMP","isdim":true,"alias":"date"},{"name":"year","type":"BIGINT","isdim":false,"alias":"year"},{"name":"country","type":"STRING","isdim":true,"alias":"country"},{"name":"source_country","type":"STRING","isdim":true,"alias":"source_country"},{"name":"threat","type":"STRING","isdim":true,"alias":"threat"},{"name":"count","type":"BIGINT","isdim":false,"alias":"count"},{"name":"industry","type":"STRING","isdim":true,"alias":"industry"}]}]',
      detail: 'main.infoseq_1559',
      detail_display: 'main.infoseq_1559',
      description: '',
      settings:
        '{"SEARCH":{"enabled":false,"expansions":[],"suggestions":[],"defaultAgg":"Record Count","recordCount":{"terms":"count rows"}}}',
      lvname: '',
      lvcolumns: '',
      created: '2024-11-13T19:59:35.964+00:00',
      created_by: 'vizapps_admin',
      updated: '2024-11-13T19:59:36.273+00:00',
      updated_by: 'vizapps_admin',
      perms: [
        'dc_aviews',
        'dc_explore',
        'dc_upload',
        'ds_appedit',
        'ds_manage',
        'ds_appview',
      ],
      updatedLV: false,
      extractSource: false,
      extractTarget: false,
      version_name: '2024-11-13 19:59:35.964449+00:00',
      version_id: 5,
      version_group_id: 5,
      segments: [],
      events: [],
      filterassociations: [],
    },
  },
  aggregates: [
    {
      custom_expr: "sum([calories]) as 'sum(calories)'",
      dataset_colname: 'calories',
      dataset_coltype: 'INT',
    },
    {
      custom_expr: "sum([fat_grams]) as 'sum(fat_grams)'",
      dataset_colname: 'fat_grams',
      dataset_coltype: 'INT',
    },
  ],
  dimensions: [
    {
      custom_expr: "[manufacturer_code] as 'manufacturer_code'",
      dataset_colname: 'manufacturer_code',
      dataset_coltype: 'STRING',
    },
    {
      custom_expr: "[sodium_mg] as 'sodium_mg'",
      dataset_colname: 'sodium_mg',
      dataset_coltype: 'INT',
    },
  ],
};

const removePersistentIds = (result) => {
  const shelves = [
    'x_shelf',
    'y_shelf',
    'color_shelf',
    'tooltip_shelf',
    'aggregates_shelf',
    'dimensions_shelf',
    'bar_measure',
    'line_measure',
    'y_dimensions_shelf',
    'compare_shelf',
    'geo_shelf',
  ];

  shelves.forEach((shelf) => {
    result.report_data[shelf]?.forEach((obj) => {
      delete obj.persistentId;
    });
  });
};

QUnit.test('Returns expected JSON for each type', (assert) => {
  const flattenedOptions = Object.values(arc.showme.visualOptions().options)
    .flat()
    .filter(({ isGeo, isDate }) => !isGeo && !isDate);

  flattenedOptions.forEach(({ vizType, constraints }, optionIndex) => {
    const shelfData = {
      dimensions: [],
      aggregates: [],
      filters: [],
      type: vizType,
    };

    if (constraints?.minimums) {
      if (constraints.minimums.dimensions) {
        for (let index = 0; index < constraints.minimums.dimensions; index++) {
          const dimension = CONSTANTS.dimensions[index];
          shelfData.dimensions.push(dimension);
        }
      }

      if (constraints.minimums.aggregates) {
        for (let index = 0; index < constraints.minimums.aggregates; index++) {
          const aggregate = CONSTANTS.aggregates[index];
          shelfData.aggregates.push(aggregate);
        }
      }
    }

    const result = arc.showme.api(shelfData, CONSTANTS.datasets.cereals);

    removePersistentIds(result);

    assert.equal(
      JSON.stringify(result),
      JSON.stringify(CONSTANTS.expectedGenericReports[optionIndex])
    );
  });

  const resultLeaflet = arc.showme.api(
    {
      aggregates: [
        {
          custom_expr: "[latitude] as 'latitude'",
          dataset_colname: 'latitude',
          dataset_coltype: 'FLOAT',
        },
        {
          custom_expr: "[longitude] as 'longitude'",
          dataset_colname: 'longitude',
          dataset_coltype: 'FLOAT',
        },
      ],
      dimensions: [
        {
          custom_expr: "[magtype] as 'magtype'",
          dataset_colname: 'magtype',
          dataset_coltype: 'STRING',
        },
        {
          custom_expr: "[status] as 'status'",
          dataset_colname: 'status',
          dataset_coltype: 'STRING',
        },
      ],
      filters: [],
      type: 'leaflet',
    },
    CONSTANTS.datasets.earthquakes
  );

  removePersistentIds(resultLeaflet);

  assert.equal(
    JSON.stringify(resultLeaflet),
    JSON.stringify(CONSTANTS.expectedLeaflet)
  );

  const resultMap = arc.showme.api(
    {
      aggregates: [
        {
          custom_expr: "[count] as 'count'",
          dataset_colname: 'count',
          dataset_coltype: 'BIGINT',
        },
        {
          custom_expr: "[year] as 'year'",
          dataset_colname: 'year',
          dataset_coltype: 'BIGINT',
        },
      ],
      dimensions: [
        {
          custom_expr: "[country] as 'country'",
          dataset_colname: 'country',
          dataset_coltype: 'STRING',
        },
      ],
      filters: [],
      type: 'map',
    },
    CONSTANTS.datasets.infosec
  );

  removePersistentIds(resultMap);

  assert.equal(
    JSON.stringify(resultMap),
    JSON.stringify(CONSTANTS.expectedMap)
  );

  const resultCalendarHeatMap = arc.showme.api(
    {
      aggregates: [
        {
          custom_expr: "[count] as 'count'",
          dataset_colname: 'count',
          dataset_coltype: 'BIGINT',
        },
        {
          custom_expr: "[year] as 'year'",
          dataset_colname: 'year',
          dataset_coltype: 'BIGINT',
        },
      ],
      dimensions: [
        {
          custom_expr: "[date] as 'date'",
          dataset_colname: 'date',
          dataset_coltype: 'TIMESTAMP',
        },
      ],
      filters: [],
      type: 'calendar-heatmap',
    },
    CONSTANTS.datasets.infosec
  );

  removePersistentIds(resultCalendarHeatMap);

  assert.equal(
    JSON.stringify(resultCalendarHeatMap),
    JSON.stringify(CONSTANTS.expetedHeatmap)
  );
});
