QUnit.module('visuals_arc_params');

QUnit.test('ParamEnums.getKeyForValue', (assert) => {
  assert.equal(arc.params.ParamEnums.getKeyForValue(), undefined);
  assert.equal(arc.params.ParamEnums.getKeyForValue(null, {}), undefined);
  assert.equal(arc.params.ParamEnums.getKeyForValue('5', { 5: '5' }), 5);
  assert.equal(arc.params.ParamEnums.getKeyForValue('a', { a: 'a' }), 'a');
  assert.equal(
    arc.params.ParamEnums.getKeyForValue('a', { a: 'b' }),
    undefined,
  );
  assert.equal(arc.params.ParamEnums.getKeyForValue('b', { a: 'b' }), 'a');
  assert.equal(
    arc.params.ParamEnums.getKeyForValue('B', { a: 'b' }),
    undefined,
  );
  assert.equal(
    arc.params.ParamEnums.getKeyForValue('b', { a: 'B' }),
    undefined,
  );
  assert.equal(arc.params.ParamEnums.getKeyForValue('B', { a: 'B' }), 'a');
  assert.equal(arc.params.ParamEnums.getKeyForValue('B', { a: 'B' }), 'a');
  assert.equal(
    arc.params.ParamEnums.getKeyForValue(
      'quick_filters',
      arc.params.ParamEnums.DateEnums.DateRangeTypeEnum,
    ),
    'QUICK',
  );
});

QUnit.test('Param - basic instantiation', (assert) => {
  const paramObject = new arc.params.Param();
  assert.equal(paramObject.outputParameter, '', 'outputParameter');
  assert.deepEqual(paramObject.value, [], 'value');
  assert.equal(
    paramObject.scope.scopeType,
    arc.params.ParamEnums.ScopeTypeEnum.EXPLICIT,
    'scope.scopeType',
  );
  assert.equal(paramObject.scope.isAppScope, false, 'scope.isAppScope');
  assert.equal(paramObject.hideExtension, false, 'hideExtension');
  assert.equal(paramObject.isIgnore, false, 'isIgnore');
  assert.equal(paramObject.isExcluded, false, 'isExcluded');
  assert.deepEqual(
    paramObject.meta,
    {
      keyType: arc.report.columnTypeEnum.DIMENSION,
      paramType: arc.params.ParamEnums.ParamTypeEnum.PARAM,
      valueType: arc.params.ParamEnums.ValueTypeEnum.SINGLE,
    },
    'meta',
  );
  assert.deepEqual(
    paramObject.source,
    {
      colDataType: arc.params.ParamEnums.DataTypeEnum.STRING,
      baseField: undefined,
      connectionType: undefined,
      entityType: arc.params.ParamEnums.EntityTypeEnum.FILTER,
      sheetId: undefined,
      dashboardId: undefined,
      appId: undefined,
      datasetId: undefined,
      entityId: undefined,
    },
    'source',
  );
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    {
      '.include_nulls': ['false'],
    },
    'toOldParamFormat',
  );
});

QUnit.test(
  'Param - instantiation without lib but different paramConfigs',
  (assert) => {
    const baseObject = new arc.params.Param();
    let paramObject;
    let sourceObject;
    let expectedSourceObject;
    paramObject = new arc.params.Param(null, {
      outputParameter: 'abcd',
    });
    assert.equal(
      paramObject.outputParameter,
      'abcd',
      'output parameter is set properly',
    );

    paramObject = new arc.params.Param(null, {
      baseField: 'abcd',
    });
    sourceObject = paramObject.source;
    assert.equal(sourceObject.baseField, 'abcd', 'base Field is set properly');
    expectedSourceObject = baseObject.source;
    expectedSourceObject.baseField = 'abcd';
    assert.deepEqual(
      sourceObject,
      expectedSourceObject,
      'sourceObject is appropriate',
    );
    assert.equal(
      paramObject.outputParameter,
      '',
      'output parameter is set properly',
    );

    paramObject = new arc.params.Param(null, {
      baseField: 'abcd',
      outputParameter: 'efgh',
    });
    sourceObject = paramObject.source;
    expectedSourceObject = baseObject.source;
    expectedSourceObject.baseField = 'abcd';
    assert.equal(sourceObject.baseField, 'abcd', 'base Field is set properly');
    assert.deepEqual(
      sourceObject,
      expectedSourceObject,
      'sourceObject is appropriate',
    );
    assert.equal(
      paramObject.outputParameter,
      'efgh',
      'output parameter is set properly',
    );
  },
);

QUnit.test('CustomParam - basic instantiation', (assert) => {
  const baseObject = new arc.params.Param();
  const paramObject = new arc.params.CustomParam();
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter',
  );
  assert.deepEqual(paramObject.value, baseObject.value, 'value');
  assert.equal(
    paramObject.scope.scopeType,
    baseObject.scope.scopeType,
    'scope.scopeType',
  );
  assert.equal(
    paramObject.scope.isAppScope,
    baseObject.scope.isAppScope,
    'scope.isAppScope',
  );
  assert.equal(
    paramObject.hideExtension,
    baseObject.hideExtension,
    'hideExtension',
  );
  assert.equal(paramObject.isIgnore, baseObject.isIgnore, 'isIgnore');
  assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
  assert.deepEqual(paramObject.meta, baseObject.meta, 'meta');
  assert.deepEqual(paramObject.source, baseObject.source, 'source');
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    baseObject.toOldParamFormat(),
    'toOldParamFormat',
  );
});

QUnit.test('RangeValueParam - basic instantiation', (assert) => {
  const baseObject = new arc.params.Param();
  const paramObject = new arc.params.RangeValueParam();
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter',
  );
  assert.deepEqual(
    paramObject.value,
    [
      {
        start: 0,
        end: 0,
      },
    ],
    'value',
  );
  assert.equal(
    paramObject.scope.scopeType,
    baseObject.scope.scopeType,
    'scope.scopeType',
  );
  assert.equal(
    paramObject.scope.isAppScope,
    baseObject.scope.isAppScope,
    'scope.isAppScope',
  );
  assert.equal(
    paramObject.hideExtension,
    baseObject.hideExtension,
    'hideExtension',
  );
  assert.equal(paramObject.isIgnore, baseObject.isIgnore, 'isIgnore');
  assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
  const metaObj = baseObject.meta;
  metaObj.valueType = arc.params.ParamEnums.ValueTypeEnum.RANGE;
  assert.deepEqual(paramObject.meta, metaObj, 'meta');
  assert.deepEqual(
    paramObject.source,
    {
      appId: undefined,
      baseField: undefined,
      colDataType: 'NUMERIC',
      connectionType: undefined,
      dashboardId: undefined,
      datasetId: undefined,
      entityId: undefined,
      entityType: 'filter',
      sheetId: undefined,
    },
    'source',
  );
  const expectedObject = {};
  expectedObject[`${paramObject.outputParameter}.start`] = [0];
  expectedObject[`${paramObject.outputParameter}.end`] = [0];
  expectedObject[`${paramObject.outputParameter}.include_nulls`] = ['false'];
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    expectedObject,
    'toOldParamFormat',
  );
});

QUnit.test('DateRangeParam - basic instantiation', (assert) => {
  const baseObject = new arc.params.RangeValueParam();
  const paramObject = new arc.params.DateRangeParam();
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter',
  );
  assert.deepEqual(
    paramObject.value,
    [
      {
        start: null,
        end: null,
      },
    ],
    'value',
  );
  assert.equal(
    paramObject.scope.scopeType,
    baseObject.scope.scopeType,
    'scope.scopeType',
  );
  assert.equal(
    paramObject.scope.isAppScope,
    baseObject.scope.isAppScope,
    'scope.isAppScope',
  );
  assert.equal(
    paramObject.hideExtension,
    baseObject.hideExtension,
    'hideExtension',
  );
  assert.equal(paramObject.isIgnore, baseObject.isIgnore, 'isIgnore');
  assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
  const metaObj = baseObject.meta;
  metaObj.dateRangeType =
    arc.params.ParamEnums.DateEnums.DateRangeTypeEnum.FIXED;
  assert.deepEqual(paramObject.meta, metaObj, 'meta');
  assert.deepEqual(
    paramObject.source,
    {
      appId: undefined,
      baseField: undefined,
      colDataType: 'TIMESTAMP',
      connectionType: undefined,
      dashboardId: undefined,
      datasetId: undefined,
      entityId: undefined,
      entityType: 'filter',
      sheetId: undefined,
    },
    'source',
  );
  const expectedObject = {};
  expectedObject[`${paramObject.outputParameter}.start`] = [null];
  expectedObject[`${paramObject.outputParameter}.end`] = [null];
  expectedObject[`${paramObject.outputParameter}.include_nulls`] = ['false'];
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    expectedObject,
    'toOldParamFormat',
  );
});

QUnit.test('ColParam - basic instantiation', (assert) => {
  const baseObject = new arc.params.Param();
  const paramObject = new arc.params.ColParam();
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter',
  );
  assert.deepEqual(paramObject.value, baseObject.value, 'value');
  assert.equal(
    paramObject.scope.scopeType,
    baseObject.scope.scopeType,
    'scope.scopeType',
  );
  assert.equal(
    paramObject.scope.isAppScope,
    baseObject.scope.isAppScope,
    'scope.isAppScope',
  );
  assert.equal(
    paramObject.hideExtension,
    baseObject.hideExtension,
    'hideExtension',
  );
  assert.equal(paramObject.isIgnore, baseObject.isIgnore, 'isIgnore');
  assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
  assert.deepEqual(paramObject.pivot, {}, 'pivot');
  const metaObject = baseObject.meta;
  metaObject.paramType = arc.params.ParamEnums.ParamTypeEnum.PIVOT;
  assert.deepEqual(paramObject.meta, metaObject, 'meta');
  assert.deepEqual(paramObject.source, baseObject.source, 'source');
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    baseObject.toOldParamFormat(),
    'toOldParamFormat',
  );
});

QUnit.test('PivotParam - basic instantiation', (assert) => {
  const baseObject = new arc.params.ColParam();
  const paramObject = new arc.params.PivotParam();
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter',
  );
  assert.deepEqual(paramObject.value, baseObject.value, 'value');
  assert.equal(
    paramObject.scope.scopeType,
    baseObject.scope.scopeType,
    'scope.scopeType',
  );
  assert.equal(
    paramObject.scope.isAppScope,
    baseObject.scope.isAppScope,
    'scope.isAppScope',
  );
  assert.equal(
    paramObject.hideExtension,
    baseObject.hideExtension,
    'hideExtension',
  );
  assert.equal(paramObject.isIgnore, baseObject.isIgnore, 'isIgnore');
  assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
  assert.deepEqual(paramObject.pivot, baseObject.pivot, 'pivot');
  assert.deepEqual(paramObject.meta, baseObject.meta, 'meta');
  assert.deepEqual(paramObject.source, baseObject.source, 'source');
  const expectedOldFormatObject = baseObject.toOldParamFormat();
  expectedOldFormatObject.key = `${paramObject.outputParameter}.pivot`;
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    expectedOldFormatObject,
    'toOldParamFormat',
  );
});

QUnit.test('Param - instantiation from a visual with lib', (assert) => {
  const baseObject = new arc.params.Param();
  const lib = arc.report.base();
  const paramObject = new arc.params.Param(lib);
  const expectedSourceObject = baseObject.source;
  expectedSourceObject.entityType = arc.params.ParamEnums.EntityTypeEnum.VISUAL;
  assert.deepEqual(
    paramObject.source,
    expectedSourceObject,
    'sourceObject is appropriate',
  );
  assert.equal(
    paramObject.outputParameter,
    '',
    'output parameter is set properly',
  );
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter',
  );
  assert.deepEqual(paramObject.value, baseObject.value, 'value');
  assert.equal(
    paramObject.scope.scopeType,
    arc.params.ParamEnums.ScopeTypeEnum.GLOBAL,
    'scope.scopeType',
  );
  assert.equal(
    paramObject.scope.isAppScope,
    baseObject.scope.isAppScope,
    'scope.isAppScope',
  );
  assert.equal(
    paramObject.hideExtension,
    baseObject.hideExtension,
    'hideExtension',
  );
  assert.equal(paramObject.isIgnore, baseObject.isIgnore, 'isIgnore');
  assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
  assert.deepEqual(paramObject.meta, baseObject.meta, 'meta');
  assert.deepEqual(
    paramObject.toOldParamFormat(),
    baseObject.toOldParamFormat(),
    'toOldParamFormat',
  );
});

QUnit.test(
  'Param - conversion to and from OldFormat of basic instantiation',
  (assert) => {
    const paramObjects = [new arc.params.Param()];
    const oldFormatArr = arc.params.convertToOldFormatArr(paramObjects);
    assert.deepEqual(oldFormatArr, [
      {
        isDatasetSegment: undefined,
        key: '.include_nulls',
        keytype: 1,
        source_dataset_id: undefined,
        source_visual_id: undefined,
        value: 'false',
        value_list: ['false'],
        valuetype: 'STRING',
      },
    ]);

    const convertedParamObjects =
      arc.params.convertOldParamArrToNewParamObjects(oldFormatArr);
    assert.equal(paramObjects.length, convertedParamObjects.length);
    paramObjects.forEach((baseObject, i) => {
      const paramObject = convertedParamObjects[i];
      assert.equal(
        paramObject.constructor,
        baseObject.constructor,
        'constructor',
      );
      assert.equal(
        paramObject.outputParameter,
        baseObject.outputParameter,
        'outputParameter',
      );
      assert.equal(paramObject.isIgnore, false, 'isIgnore');
      assert.deepEqual(paramObject.value, [], 'value');
      assert.equal(
        paramObject.scope.scopeType,
        baseObject.scope.scopeType,
        'scope.scopeType',
      );
      assert.equal(
        paramObject.scope.isAppScope,
        baseObject.scope.isAppScope,
        'scope.isAppScope',
      );
      assert.equal(
        paramObject.hideExtension,
        baseObject.hideExtension,
        'hideExtension',
      );
      assert.equal(paramObject.isExcluded, baseObject.isExcluded, 'isExcluded');
      assert.deepEqual(paramObject.meta, baseObject.meta, 'meta');
      assert.deepEqual(paramObject.source, baseObject.source, 'source');
      assert.deepEqual(
        paramObject.toOldParamFormat(),
        baseObject.toOldParamFormat(),
        'toOldParamFormat',
      );
    });
  },
);

QUnit.test('Param - stringify and parse', (assert) => {
  const baseObject = new arc.params.Param(null, {
    outputParameter: 'testName',
    baseField: 'someBaseField',
    value: [1, 2],
  });

  const stringifiedObjects = arc.params.stringifyParamObjects([baseObject]);
  const expectedStringifiedObjects = JSON.stringify([
    {
      _scope: { scopeType: 'explicit' },
      _visibility: { entityType: 'sheet' },
      _source: {
        column: { baseField: 'someBaseField', dataType: 'STRING' },
        entity: { entityType: 'filter' },
      },
      _meta: { valueType: 'single', keyType: 1, paramType: 'param' },
      _state: { isIgnore: false, hideExtension: false },
      _value: [1, 2],
      _outputParameter: 'testName',
      _isExcluded: false,
      _includeNulls: false,
      _className: 'Param',
    },
  ]);
  assert.deepEqual(
    stringifiedObjects,
    expectedStringifiedObjects,
    'stringified objects match',
  );
  const paramObject = arc.params.parseParamObjects(stringifiedObjects)[0];
  delete baseObject.name;
  assert.deepEqual(paramObject, baseObject, 'objects match');
  assert.deepEqual(paramObject.source, baseObject.source, 'source match');
  assert.deepEqual(paramObject.meta, baseObject.meta, 'meta match');
  assert.deepEqual(paramObject.value, baseObject.value, 'value match');
  assert.equal(
    paramObject.outputParameter,
    baseObject.outputParameter,
    'outputParameter match',
  );
  assert.equal(
    paramObject.constructor.name,
    baseObject.constructor.name,
    'constructor.name match',
  );
});

QUnit.test(
  'Convert old format to new paramObject and reverse - 1',
  (assert) => {
    const oldFormatParams = {
      entity: 'account_number',
      'entity.alias': 'Account Number',
      'entity.include_nulls': 'true',
      entity_first_seen: 'account_number_first_seen',
      'entity_first_seen.include_nulls': 'false',
    };
    const paramObjects =
      arc.params.convertOldParamHashToNewParamObjects(oldFormatParams);
    assert.equal(paramObjects.length, 2);
    assert.equal(paramObjects[0].constructor, arc.params.CustomParam);
    assert.equal(paramObjects[1].constructor, arc.params.Param);
    const convertedOldFormatParams =
      arc.params.convertToOldFormatHash(paramObjects);
    assert.deepEqual(convertedOldFormatParams, oldFormatParams);
  },
);

QUnit.test(
  'Convert old format to new paramObject and reverse - 2',
  (assert) => {
    const oldFormatParams = {
      'entity.alias': 'Account Number',
      'entity.include_nulls': 'false',
      entity: 'account_number',
      entity_first_seen: 'account_number_first_seen',
      'entity_first_seen.include_nulls': 'true',
    };
    const paramObjects =
      arc.params.convertOldParamHashToNewParamObjects(oldFormatParams);
    assert.equal(paramObjects.length, 2);
    assert.equal(paramObjects[0].constructor, arc.params.CustomParam);
    assert.equal(paramObjects[1].constructor, arc.params.Param);
    const convertedOldFormatParams =
      arc.params.convertToOldFormatHash(paramObjects);
    assert.deepEqual(convertedOldFormatParams, oldFormatParams);
  },
);

QUnit.test(
  'Convert old format to new paramObject and reverse - 3',
  (assert) => {
    const oldFormatParams = {
      'entity.data': "'account_number'",
      'entity.exclude': 'in',
      'entity.include_nulls': 'true',
      'entity.alias': "'Account Number'",
      entity_first_seen: 'account_number_first_seen',
      'entity_first_seen.include_nulls': 'true',
    };
    const paramObjects =
      arc.params.convertOldParamHashToNewParamObjects(oldFormatParams);
    assert.equal(paramObjects.length, 2);
    assert.equal(paramObjects[0].constructor, arc.params.CustomParam);
    assert.equal(paramObjects[1].constructor, arc.params.Param);
    const convertedOldFormatParams =
      arc.params.convertToOldFormatHash(paramObjects);
    assert.deepEqual(convertedOldFormatParams, oldFormatParams);
  },
);

QUnit.test(
  'Convert old format to new paramObject and reverse - 4',
  (assert) => {
    const oldFormatParams = {
      'entity.alias': "'Account Number'",
      'entity.data': "'account_number'",
      'entity.exclude': 'in',
      'entity.include_nulls': 'false',
      entity_first_seen: 'account_number_first_seen',
      'entity_first_seen.include_nulls': 'false',
    };
    const paramObjects =
      arc.params.convertOldParamHashToNewParamObjects(oldFormatParams);
    assert.equal(paramObjects.length, 2);
    assert.equal(paramObjects[0].constructor, arc.params.CustomParam);
    assert.equal(paramObjects[1].constructor, arc.params.Param);
    const convertedOldFormatParams =
      arc.params.convertToOldFormatHash(paramObjects);
    assert.deepEqual(convertedOldFormatParams, oldFormatParams);
  },
);
