import { EmailTemplate } from 'types/email-template.type';
import {
  DataCopyTarget,
  EmailJob,
  EmailTarget,
  Job,
  JobStatus,
  JobType,
  JobThresholdTriggerType,
  PdfUploadFormatType,
  SnapshotTarget,
  ThresholdTarget,
} from 'types/job.type';
import { ScheduleGroup } from 'types/schedule-group.type';

export const mockEmailTemplateOptions: EmailTemplate[] = [
  {
    id: 1,
    name: 'Template',
    description: '',
    defaultMessage: '',
  },
];
export const mockScheduleGroups: ScheduleGroup[] = [
  {
    id: 1,
    name: 'Hourly',
    runInterval: '0 * * * *',
    nextRunTime: '',
    dependentJobs: [{ id: 1, name: 'dependent' }],
  },
  {
    id: 2,
    name: 'Daily',
    runInterval: '0 0 * * *',
    nextRunTime: '',
    dependentJobs: [],
  },
];

const mockEmailTarget: EmailTarget = {
  id: 1,
  visualURL: 'url',
  useUrlAlias: false,
  includeEmbeddedImage: true,
  includePng: false,
  includePdf: true,
  subject: 'Cloudera Data Visualization',
  message: 'Default message',
  emailTemplateId: 1,
  urlParams: 'embed=true',
  toAddresses: ['test@test.com', 'test2@test.com'],
  ccAddresses: [],
  errorToAddresses: [],
  replyToAddress: '',
};

const mockSnapshotTarget: SnapshotTarget = {
  id: 1,
  includePageParams: false,
  includeCSV: false,
  includeXLSX: false,
  csvDownloadRows: 5000,
  dbWidth: 500,
  dbHeight: 500,
  subject: 'Cloudera Data Visualization',
  errorToAddresses: [],
  ccAddresses: [''],
  paramString: 'embed=true&sheet=1',
  uploadFormat: PdfUploadFormatType.OneSheetPerPage,
};

const mockDataCopyTarget: DataCopyTarget = {
  id: 1,
  username: 'user',
  scrDatasetURL: 'http://src.com',
  jobsUrl: '/jobsUrl',
};

const mockThresholdTarget: ThresholdTarget = {
  triggerId: 1,
  triggerType: JobThresholdTriggerType.ThresholdMet,
  triggerConditions: [
    { equality: '>', value: 5 },
    { equality: '<', value: 2 },
  ],
  minimumTimeInterval: 604800,
};

export const mockEmailJob: EmailJob = {
  id: 1,
  name: 'Mock Job',
  owner: 'Owner',
  type: JobType.Email,
  isSchedulingPaused: false,
  lastStatus: JobStatus.Finished,
  scheduleGroup: {
    id: 1,
    name: 'Hourly',
  },
  emailTarget: mockEmailTarget,
  snapshotTarget: mockSnapshotTarget,
  thresholdTarget: mockThresholdTarget,
};

export const mockExtractJob: Job = {
  id: 1,
  extractId: 1,
  name: 'Mock Job',
  owner: 'Owner',
  type: JobType.DataExtract,
  lastStatus: JobStatus.Finished,
  isSchedulingPaused: false,
  dataCopyTarget: mockDataCopyTarget,
};

export const mockCompletedLog = {
  id: 1,
  status: JobStatus.Finished,
  jobId: 73,
  ownerUsername: 'vizapps_admin',
  editorUsername: 'vizapps_admin',
  startTime: '2024-04-02T14:11:27.886Z',
  endTime: '2024-04-02T14:11:57.927Z',
  totalRunTime: 1,
  taskList: [{ message: 'Get data' }],
  triggeredManually: true,
  triggeredByUsername: 'vizapps_admin',
};

export const mockEmailLog = {
  id: 1,
  status: JobStatus.Finished,
  jobId: 73,
  ownerUsername: 'vizapps_admin',
  editorUsername: 'vizapps_admin',
  startTime: '2024-04-02T14:11:27.886Z',
  endTime: '2024-04-02T14:11:57.927Z',
  totalRunTime: 1,
  taskList: [{ message: 'Get data' }],
  triggeredManually: true,
  triggeredByUsername: 'vizapps_admin',
  emailTarget: mockEmailTarget,
};

export const mockExtractLog = {
  id: 1,
  status: JobStatus.Finished,
  jobId: 73,
  ownerUsername: 'vizapps_admin',
  editorUsername: 'vizapps_admin',
  startTime: '2024-04-02T14:11:27.886Z',
  endTime: '2024-04-02T14:11:57.927Z',
  totalRunTime: 1,
  taskList: [
    { message: 'Get data', date: '2024-04-02T14:11:57.927Z' },
    { message: 'Start', date: '2024-04-02T14:11:27.927Z' },
  ],
  triggeredManually: true,
  triggeredByUsername: 'vizapps_admin',
  dataCopyTarget: mockDataCopyTarget,
};

export const mockRunningLog = {
  id: 2,
  status: JobStatus.Running,
  jobId: 73,
  ownerUsername: 'vizapps_admin',
  editorUsername: 'vizapps_admin',
  startTime: '2024-04-02T14:11:27.886Z',
  endTime: '2024-04-02T14:11:57.927Z',
  totalRunTime: 1,
  taskList: [{ message: 'Get data' }],
  triggeredManually: true,
  triggeredByUsername: 'vizapps_admin',
};

export const mockExtract = {
  id: 1,
  type: '',
  srcDatasetId: 1,
  srcDatasetName: 'sourceDataset',
  tgtDataConnectionId: 1,
  tgtDataConnectionName: 'targetDataConnection',
  tgtDbName: 'targetDb',
  tgtTableName: 'targetTable',
  username: 'targetUsername',
  rewriteStatus: 'rewrite',
  dimensions: [],
  measures: [],
  emailSuccess: true,
  emailFail: true,
  emailTo: ['test@test.com'],
  emailSubject: 'Subject',
};
