import { JobLog } from './job-log.type';

export enum JobType {
  Email = 'Email',
  DataExtract = 'Data Extract',
  Refresh = 'Refresh',
  Other = 'Other',
}

export enum JobStatus {
  Finished = 'C',
  Error = 'F',
  Running = 'R',
  Cancelled = 'X',
  Queued = 'Q',
  Unknown = 'U',
}

export enum SchedulingStatus {
  Paused = 'P',
  Active = 'A',
}

export enum JobThresholdTriggerType {
  ValueChanged = 'D',
  ThresholdMet = 'T',
}

export enum PdfUploadFormatType {
  CurrentSheet = 'pdf-current',
  OneVisualPerPage = 'pdf-one-viz',
  OneSheetPerPage = 'pdf-all-sheets',
}

export type SnapshotTarget = {
  id: number;
  includePageParams: boolean;
  includeCSV: boolean;
  includeXLSX: boolean;
  csvDownloadRows: number;
  dbWidth: number;
  dbHeight: number;
  subject: string;
  errorToAddresses: string[];
  ccAddresses: string[];
  paramString: string;
  uploadFormat?: PdfUploadFormatType;
};

export type EmailTarget = {
  id: number;
  visualURL: string;
  useUrlAlias: boolean;
  includeEmbeddedImage: boolean;
  includePng: boolean;
  includePdf: boolean;
  subject: string;
  message: string;
  urlParams: string;
  toAddresses: string[];
  ccAddresses: string[];
  errorToAddresses: string[];
  replyToAddress: string;
  emailTemplateId?: number;
};

export type ThresholdTarget = {
  triggerId: number;
  triggerType: JobThresholdTriggerType;
  triggerConditions: JobThresholdTriggerCondition[];
  minimumTimeInterval?: number;
};

export type RefreshTarget = {
  dataConnection: {
    id: number;
    name: string;
  };
  view: string;
  minimumTimeInterval?: number;
};

export type DataCopyTarget = {
  id: number;
  username: string;
  scrDatasetURL?: string;
  jobsUrl: string;
};

export type JobThresholdTriggerCondition = {
  equality: string;
  value: number;
};

type BaseJob = {
  id: number;
  name: string;
  isSchedulingPaused: boolean;
  lastStatus: JobStatus;
  owner?: string;
  lastFinishedTime?: string;
  thresholdTarget?: ThresholdTarget;
  scheduleGroup?: {
    id: number;
    name?: string;
  };
};

export type EmailJob = BaseJob & {
  type: JobType.Email;
  emailTarget: EmailTarget;
  snapshotTarget?: SnapshotTarget;
};

export type ExtractJob = BaseJob & {
  type: JobType.DataExtract;
  extractId: number;
  dataCopyTarget: DataCopyTarget;
};

export type RefreshJob = BaseJob & {
  type: JobType.Refresh;
  refreshTarget: RefreshTarget;
};

export type OtherJob = BaseJob & {
  type: JobType.Other;
};

export type Job = EmailJob | ExtractJob | RefreshJob | OtherJob;

export type JobWithLog = { log: JobLog; job: Job };

export interface JobReport {
  id: number; // The id of the main report: can be a visual id or a dashboard id
  type: 'sheet' | 'visual';
  title: string;
  dbWidth: number;
  dbHeight: number;
  paramString: string;
  url: string;
  dashboardId: number;
  datasetId?: number;
  disableXlsxAndCsv?: boolean;
}
